--- This file has been generated by scripts/build_db.py. DO NOT EDIT !

INSERT INTO "vertical_datum" VALUES('EPSG','1027','EGM2008 geoid',NULL,'2008-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13079','vertical_datum','EPSG','1027','EPSG','1262','EPSG','1027');
INSERT INTO "vertical_datum" VALUES('EPSG','1028','Fao 1979',NULL,'1979-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13080','vertical_datum','EPSG','1028','EPSG','3625','EPSG','1180');
INSERT INTO "vertical_datum" VALUES('EPSG','1030','N2000',NULL,'2000-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13082','vertical_datum','EPSG','1030','EPSG','3333','EPSG','1180');
INSERT INTO "vertical_datum" VALUES('EPSG','1039','New Zealand Vertical Datum 2009',NULL,'2009-09-14',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13091','vertical_datum','EPSG','1039','EPSG','1175','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','1040','Dunedin-Bluff 1960',NULL,'1960-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13092','vertical_datum','EPSG','1040','EPSG','3806','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','1049','Korean Vertical Datum 1964',NULL,'1963-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13101','vertical_datum','EPSG','1049','EPSG','3266','EPSG','1180');
INSERT INTO "vertical_datum" VALUES('EPSG','1050','Trieste',NULL,'1875-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13102','vertical_datum','EPSG','1050','EPSG','2370','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','1051','Genoa 1942',NULL,'1942-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13103','vertical_datum','EPSG','1051','EPSG','3736','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','1054','Sri Lanka Vertical Datum',NULL,'1932-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13106','vertical_datum','EPSG','1054','EPSG','3310','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','1059','Faroe Islands Vertical Reference 2009',NULL,'2009-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13111','vertical_datum','EPSG','1059','EPSG','3248','EPSG','1142');
INSERT INTO "vertical_datum" VALUES('EPSG','1079','Fehmarnbelt Vertical Reference 2010',NULL,'2010-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13131','vertical_datum','EPSG','1079','EPSG','3890','EPSG','1139');
INSERT INTO "vertical_datum" VALUES('EPSG','1080','Lowest Astronomical Tide',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13132','vertical_datum','EPSG','1080','EPSG','1262','EPSG','1198');
INSERT INTO "vertical_datum" VALUES('EPSG','1082','Highest Astronomical Tide',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13134','vertical_datum','EPSG','1082','EPSG','1262','EPSG','1198');
INSERT INTO "vertical_datum" VALUES('EPSG','1083','Lower Low Water Large Tide',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13135','vertical_datum','EPSG','1083','EPSG','1262','EPSG','1198');
INSERT INTO "vertical_datum" VALUES('EPSG','1084','Higher High Water Large Tide',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13136','vertical_datum','EPSG','1084','EPSG','1262','EPSG','1198');
INSERT INTO "vertical_datum" VALUES('EPSG','1085','Indian Spring Low Water',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13137','vertical_datum','EPSG','1085','EPSG','1262','EPSG','1198');
INSERT INTO "vertical_datum" VALUES('EPSG','1086','Mean Lower Low Water Spring Tides',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13138','vertical_datum','EPSG','1086','EPSG','1262','EPSG','1198');
INSERT INTO "vertical_datum" VALUES('EPSG','1087','Mean Low Water Spring Tides',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13139','vertical_datum','EPSG','1087','EPSG','1262','EPSG','1198');
INSERT INTO "vertical_datum" VALUES('EPSG','1088','Mean High Water Spring Tides',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13140','vertical_datum','EPSG','1088','EPSG','1262','EPSG','1198');
INSERT INTO "vertical_datum" VALUES('EPSG','1089','Mean Lower Low Water',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13141','vertical_datum','EPSG','1089','EPSG','1262','EPSG','1198');
INSERT INTO "vertical_datum" VALUES('EPSG','1090','Mean Higher High Water',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13142','vertical_datum','EPSG','1090','EPSG','1262','EPSG','1198');
INSERT INTO "vertical_datum" VALUES('EPSG','1091','Mean Low Water',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13143','vertical_datum','EPSG','1091','EPSG','1262','EPSG','1198');
INSERT INTO "vertical_datum" VALUES('EPSG','1092','Mean High Water',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13144','vertical_datum','EPSG','1092','EPSG','1262','EPSG','1198');
INSERT INTO "vertical_datum" VALUES('EPSG','1093','Low Water',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13145','vertical_datum','EPSG','1093','EPSG','1262','EPSG','1198');
INSERT INTO "vertical_datum" VALUES('EPSG','1094','High Water',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13146','vertical_datum','EPSG','1094','EPSG','1262','EPSG','1198');
INSERT INTO "vertical_datum" VALUES('EPSG','1096','Norway Normal Null 2000',NULL,'2000-01-01',2000.0,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13148','vertical_datum','EPSG','1096','EPSG','1352','EPSG','1180');
INSERT INTO "vertical_datum" VALUES('EPSG','1097','Grand Cayman Vertical Datum 1954',NULL,'1954-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13149','vertical_datum','EPSG','1097','EPSG','3185','EPSG','1180');
INSERT INTO "vertical_datum" VALUES('EPSG','1098','Little Cayman Vertical Datum 1961',NULL,'1961-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13150','vertical_datum','EPSG','1098','EPSG','4121','EPSG','1180');
INSERT INTO "vertical_datum" VALUES('EPSG','1099','Cayman Brac Vertical Datum 1961',NULL,'1961-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13151','vertical_datum','EPSG','1099','EPSG','3207','EPSG','1180');
INSERT INTO "vertical_datum" VALUES('EPSG','1101','Cais da Pontinha',NULL,'1913-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13153','vertical_datum','EPSG','1101','EPSG','4125','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','1102','Cais da Vila',NULL,'1936-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13154','vertical_datum','EPSG','1102','EPSG','3680','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','1103','Cais das Velas',NULL,'1937-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13155','vertical_datum','EPSG','1103','EPSG','2875','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','1104','Horta',NULL,'1935-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13156','vertical_datum','EPSG','1104','EPSG','2873','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','1105','Cais da Madalena',NULL,'1937-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13157','vertical_datum','EPSG','1105','EPSG','2874','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','1106','Santa Cruz da Graciosa',NULL,'1938-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13158','vertical_datum','EPSG','1106','EPSG','3681','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','1107','Cais da Figueirinha',NULL,'1951-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13159','vertical_datum','EPSG','1107','EPSG','2872','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','1108','Santa Cruz das Flores',NULL,'1965-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13160','vertical_datum','EPSG','1108','EPSG','1344','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','1109','Cais da Vila do Porto',NULL,'1965-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13161','vertical_datum','EPSG','1109','EPSG','4126','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','1110','Ponta Delgada',NULL,'1991-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13162','vertical_datum','EPSG','1110','EPSG','2871','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','1119','Northern Marianas Vertical Datum of 2003',NULL,'2009-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13171','vertical_datum','EPSG','1119','EPSG','4171','EPSG','1180');
INSERT INTO "vertical_datum" VALUES('EPSG','1121','Tutuila Vertical Datum of 1962',NULL,'1962-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13173','vertical_datum','EPSG','1121','EPSG','2288','EPSG','1180');
INSERT INTO "vertical_datum" VALUES('EPSG','1122','Guam Vertical Datum of 1963',NULL,'1963-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13174','vertical_datum','EPSG','1122','EPSG','3255','EPSG','1180');
INSERT INTO "vertical_datum" VALUES('EPSG','1123','Puerto Rico Vertical Datum of 2002',NULL,'2012-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13175','vertical_datum','EPSG','1123','EPSG','3294','EPSG','1180');
INSERT INTO "vertical_datum" VALUES('EPSG','1124','Virgin Islands Vertical Datum of 2009',NULL,'2011-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13176','vertical_datum','EPSG','1124','EPSG','3330','EPSG','1180');
INSERT INTO "vertical_datum" VALUES('EPSG','1125','American Samoa Vertical Datum of 2002',NULL,'2009-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13177','vertical_datum','EPSG','1125','EPSG','2288','EPSG','1180');
INSERT INTO "vertical_datum" VALUES('EPSG','1126','Guam Vertical Datum of 2004',NULL,'2009-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13178','vertical_datum','EPSG','1126','EPSG','3255','EPSG','1180');
INSERT INTO "vertical_datum" VALUES('EPSG','1127','Canadian Geodetic Vertical Datum of 2013 (CGG2013)',NULL,'2013-11-28',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','18090','vertical_datum','EPSG','1127','EPSG','1061','EPSG','1180');
INSERT INTO "vertical_datum" VALUES('EPSG','1129','Japanese Standard Levelling Datum 1972',NULL,'1972-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13181','vertical_datum','EPSG','1129','EPSG','4168','EPSG','1180');
INSERT INTO "vertical_datum" VALUES('EPSG','1130','Japanese Geodetic Datum 2000 (vertical)',NULL,'2002-04-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13182','vertical_datum','EPSG','1130','EPSG','3263','EPSG','1180');
INSERT INTO "vertical_datum" VALUES('EPSG','1131','Japanese Geodetic Datum 2011 (vertical)',NULL,'2011-10-21',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13183','vertical_datum','EPSG','1131','EPSG','3263','EPSG','1180');
INSERT INTO "vertical_datum" VALUES('EPSG','1140','Singapore Height Datum',NULL,'2009-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13192','vertical_datum','EPSG','1140','EPSG','1210','EPSG','1144');
INSERT INTO "vertical_datum" VALUES('EPSG','1146','Ras Ghumays',NULL,'1979-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13198','vertical_datum','EPSG','1146','EPSG','4225','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','1148','Famagusta 1960',NULL,'1960-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13200','vertical_datum','EPSG','1148','EPSG','3236','EPSG','1142');
INSERT INTO "vertical_datum" VALUES('EPSG','1149','PNG08',NULL,'2011-10-14',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13201','vertical_datum','EPSG','1149','EPSG','4384','EPSG','1027');
INSERT INTO "vertical_datum" VALUES('EPSG','1150','Kumul 34',NULL,'2005-06-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13202','vertical_datum','EPSG','1150','EPSG','4013','EPSG','1029');
INSERT INTO "vertical_datum" VALUES('EPSG','1151','Kiunga',NULL,'1990-10-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13203','vertical_datum','EPSG','1151','EPSG','4383','EPSG','1029');
INSERT INTO "vertical_datum" VALUES('EPSG','1161','Deutsches Haupthoehennetz 1912',NULL,'1912-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13213','vertical_datum','EPSG','1161','EPSG','3339','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','1162','Latvian Height System 2000',NULL,'2005-07-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13214','vertical_datum','EPSG','1162','EPSG','3268','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','1164','Ordnance Datum Newlyn (Offshore)',NULL,'2016-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13216','vertical_datum','EPSG','1164','EPSG','4391','EPSG','1027');
INSERT INTO "vertical_datum" VALUES('EPSG','1169','New Zealand Vertical Datum 2016',NULL,'2016-06-27',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13221','vertical_datum','EPSG','1169','EPSG','1175','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','1170','Deutsches Haupthoehennetz 2016',NULL,'2016-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13222','vertical_datum','EPSG','1170','EPSG','1103','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','1171','Port Moresby 1996',NULL,'1996-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13223','vertical_datum','EPSG','1171','EPSG','4425','EPSG','1029');
INSERT INTO "vertical_datum" VALUES('EPSG','1172','Port Moresby 2008',NULL,'2008-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13224','vertical_datum','EPSG','1172','EPSG','4425','EPSG','1029');
INSERT INTO "vertical_datum" VALUES('EPSG','1175','Jamestown 1971',NULL,'1971-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13227','vertical_datum','EPSG','1175','EPSG','3183','EPSG','1153');
INSERT INTO "vertical_datum" VALUES('EPSG','1176','St. Helena Tritan Vertical Datum 2011',NULL,'2011-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13228','vertical_datum','EPSG','1176','EPSG','3183','EPSG','1029');
INSERT INTO "vertical_datum" VALUES('EPSG','1177','St. Helena Vertical Datum 2015',NULL,'2015-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13229','vertical_datum','EPSG','1177','EPSG','3183','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','1190','Landshaedarkerfi Islands 2004',NULL,'2004-08-07',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13242','vertical_datum','EPSG','1190','EPSG','4662','EPSG','1180');
INSERT INTO "vertical_datum" VALUES('EPSG','1199','Greenland Vertical Reference 2000',NULL,'2000-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13251','vertical_datum','EPSG','1199','EPSG','4461','EPSG','1153');
INSERT INTO "vertical_datum" VALUES('EPSG','1200','Greenland Vertical Reference 2016',NULL,'2016-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13252','vertical_datum','EPSG','1200','EPSG','4454','EPSG','1153');
INSERT INTO "vertical_datum" VALUES('EPSG','1202','Baltic 1957',NULL,'1957-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13254','vertical_datum','EPSG','1202','EPSG','1306','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','1205','EPSG example wellbore vertical datum',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13257','vertical_datum','EPSG','1205','EPSG','4393','EPSG','1226');
INSERT INTO "vertical_datum" VALUES('EPSG','1210','Macao Height Datum',NULL,'1980-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13262','vertical_datum','EPSG','1210','EPSG','1147','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','1213','Helsinki 1943',NULL,'1943-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13265','vertical_datum','EPSG','1213','EPSG','4522','EPSG','1180');
INSERT INTO "vertical_datum" VALUES('EPSG','1215','Slovenian Vertical System 2010',NULL,'2010-10-10',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13267','vertical_datum','EPSG','1215','EPSG','3307','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','1216','Serbian Vertical Reference System 2012',NULL,'2012-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13268','vertical_datum','EPSG','1216','EPSG','4543','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','1219','MOMRA Vertical Geodetic Control',NULL,'1969-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13271','vertical_datum','EPSG','1219','EPSG','3303','EPSG','1180');
INSERT INTO "vertical_datum" VALUES('EPSG','1224','Taiwan Vertical Datum 2001',NULL,'2001-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13276','vertical_datum','EPSG','1224','EPSG','3982','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','1226','Datum Altimetrico de Costa Rica 1952',NULL,'1962-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13278','vertical_datum','EPSG','1226','EPSG','3232','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','1250','IGN 2008 LD',NULL,'2008-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13302','vertical_datum','EPSG','1250','EPSG','2893','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','1255','Nivellement General de Nouvelle Caledonie 2008',NULL,'2008-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13979','vertical_datum','EPSG','1255','EPSG','3430','EPSG','1026');
INSERT INTO "vertical_datum" VALUES('EPSG','1256','Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 2010',NULL,'2015-12-05',NULL,NULL,NULL,2010.0,0);
INSERT INTO "usage" VALUES('EPSG','13972','vertical_datum','EPSG','1256','EPSG','1061','EPSG','1180');
INSERT INTO "vertical_datum" VALUES('EPSG','1260','Sistema de Referencia Vertical Nacional 2016',NULL,'2016-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13893','vertical_datum','EPSG','1260','EPSG','4573','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','1261','European Vertical Reference Frame 2000 Austria',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13894','vertical_datum','EPSG','1261','EPSG','1037','EPSG','1027');
INSERT INTO "vertical_datum" VALUES('EPSG','1262','South Africa Land Levelling Datum',NULL,'2010-05-11',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13895','vertical_datum','EPSG','1262','EPSG','3309','EPSG','1181');
INSERT INTO "vertical_datum" VALUES('EPSG','1265','HS2 Vertical Reference Frame',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','14027','vertical_datum','EPSG','1265','EPSG','4582','EPSG','1260');
INSERT INTO "vertical_datum" VALUES('EPSG','1267','Wiener Null',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13981','vertical_datum','EPSG','1267','EPSG','4585','EPSG','1248');
INSERT INTO "vertical_datum" VALUES('EPSG','1269','Kingdom of Saudi Arabia Vertical Reference Frame Jeddah 2014',NULL,'2014-10-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13897','vertical_datum','EPSG','1269','EPSG','3303','EPSG','1181');
INSERT INTO "vertical_datum" VALUES('EPSG','1270','Mean Sea Level Netherlands',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','14119','vertical_datum','EPSG','1270','EPSG','4742','EPSG','1265');
INSERT INTO "vertical_datum" VALUES('EPSG','1274','European Vertical Reference Frame 2019',NULL,'2020-09-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','14738','vertical_datum','EPSG','1274','EPSG','4608','EPSG','1261');
INSERT INTO "vertical_datum" VALUES('EPSG','1275','Mallorca',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','14014','vertical_datum','EPSG','1275','EPSG','4602','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','1276','Menorca',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','14015','vertical_datum','EPSG','1276','EPSG','4603','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','1277','Ibiza',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','14016','vertical_datum','EPSG','1277','EPSG','4604','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','1278','Lanzarote',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','14017','vertical_datum','EPSG','1278','EPSG','4591','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','1279','Fuerteventura',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','14018','vertical_datum','EPSG','1279','EPSG','4592','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','1280','Gran Canaria',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','14019','vertical_datum','EPSG','1280','EPSG','4593','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','1281','Tenerife',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','14020','vertical_datum','EPSG','1281','EPSG','4594','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','1282','La Gomera',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','14021','vertical_datum','EPSG','1282','EPSG','4595','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','1283','La Palma',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','14022','vertical_datum','EPSG','1283','EPSG','4596','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','1284','El Hierro',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','14023','vertical_datum','EPSG','1284','EPSG','4597','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','1285','Ceuta 2',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','14024','vertical_datum','EPSG','1285','EPSG','4590','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','1287','European Vertical Reference Frame 2019 mean tide',NULL,'2020-09-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','14739','vertical_datum','EPSG','1287','EPSG','4608','EPSG','1262');
INSERT INTO "vertical_datum" VALUES('EPSG','1290','Lowest Astronomical Tide Netherlands',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','14120','vertical_datum','EPSG','1290','EPSG','4742','EPSG','1198');
INSERT INTO "vertical_datum" VALUES('EPSG','1292','Australian Vertical Working Surface',NULL,'2020-07-14',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','14134','vertical_datum','EPSG','1292','EPSG','4177','EPSG','1264');
INSERT INTO "vertical_datum" VALUES('EPSG','1294','Indonesian Geoid 2020 version 1',NULL,'2020-06-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','14149','vertical_datum','EPSG','1294','EPSG','1122','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','1296','Baltic 1986',NULL,'1986-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','14975','vertical_datum','EPSG','1296','EPSG','3293','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','1297','European Vertical Reference Frame 2007 Poland',NULL,'2019-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15033','vertical_datum','EPSG','1297','EPSG','3293','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','1298','Estonian Height System 2000',NULL,'2005-07-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','14743','vertical_datum','EPSG','1298','EPSG','3246','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','1299','Lithuanian Height System 2007',NULL,'2005-07-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','14758','vertical_datum','EPSG','1299','EPSG','3272','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','1300','Bulgarian Height System 2005',NULL,'2005-07-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','14768','vertical_datum','EPSG','1300','EPSG','3224','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','1301','Norwegian Chart Datum',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','14864','vertical_datum','EPSG','1301','EPSG','4615','EPSG','1198');
INSERT INTO "vertical_datum" VALUES('EPSG','1302','Local Tidal Datum at Pago Pago 2020',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','14795','vertical_datum','EPSG','1302','EPSG','2288','EPSG','1026');
INSERT INTO "vertical_datum" VALUES('EPSG','1303','National Vertical Datum 1992',NULL,'1994-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','14852','vertical_datum','EPSG','1303','EPSG','3217','EPSG','1181');
INSERT INTO "vertical_datum" VALUES('EPSG','1306','Catania 1965',NULL,'1965-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15330','vertical_datum','EPSG','1306','EPSG','2340','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','1307','Cagliari 1956',NULL,'1956-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15331','vertical_datum','EPSG','1307','EPSG','2339','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','1316','GNTRANS',NULL,'2003-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','17277','vertical_datum','EPSG','1316','EPSG','3339','EPSG','1141');
INSERT INTO "vertical_datum" VALUES('EPSG','1318','GNTRANS2016',NULL,'2016-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','17278','vertical_datum','EPSG','1318','EPSG','3339','EPSG','1141');
INSERT INTO "vertical_datum" VALUES('EPSG','1323','Svalbard vertical datum 2006',NULL,'2006-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','17963','vertical_datum','EPSG','1323','EPSG','4058','EPSG','1180');
INSERT INTO "vertical_datum" VALUES('EPSG','1325','Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 2002',NULL,'2019-02-01',NULL,NULL,NULL,2002.0,0);
INSERT INTO "usage" VALUES('EPSG','18301','vertical_datum','EPSG','1325','EPSG','1061','EPSG','1180');
INSERT INTO "vertical_datum" VALUES('EPSG','1326','Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 1997',NULL,'2019-02-01',NULL,NULL,NULL,1997.0,0);
INSERT INTO "usage" VALUES('EPSG','18302','vertical_datum','EPSG','1326','EPSG','1061','EPSG','1180');
INSERT INTO "vertical_datum" VALUES('EPSG','1328','Indonesian Geoid 2020 version 2',NULL,'2022-06-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','18439','vertical_datum','EPSG','1328','EPSG','1122','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','1330','Mean Sea Level UK & Ireland VORF08',NULL,'2008-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','18593','vertical_datum','EPSG','1330','EPSG','4668','EPSG','1198');
INSERT INTO "vertical_datum" VALUES('EPSG','1331','Chart Datum UK & Ireland VORF08',NULL,'2008-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','18581','vertical_datum','EPSG','1331','EPSG','4668','EPSG','1198');
INSERT INTO "vertical_datum" VALUES('EPSG','1354','Nivellement General de l''Algerie 2022',NULL,'2022-11-30',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','19359','vertical_datum','EPSG','1354','EPSG','3213','EPSG','1026');
INSERT INTO "vertical_datum" VALUES('EPSG','1361','Chart Datum Portugal ',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','20092','vertical_datum','EPSG','1361','EPSG','4738','EPSG','1198');
INSERT INTO "vertical_datum" VALUES('EPSG','1362','Formentera',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','19908','vertical_datum','EPSG','1362','EPSG','4739','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','1363','Alboran',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','19909','vertical_datum','EPSG','1363','EPSG','4741','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','1364','Melilla',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','19910','vertical_datum','EPSG','1364','EPSG','4740','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','1368','Dansk Vertikal Reference 1990 (2002)',NULL,'2002-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','20431','vertical_datum','EPSG','1368','EPSG','3237','EPSG','1142');
INSERT INTO "vertical_datum" VALUES('EPSG','1369','Dansk Vertikal Reference 1990 (2013)',NULL,'2013-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','20432','vertical_datum','EPSG','1369','EPSG','3237','EPSG','1142');
INSERT INTO "vertical_datum" VALUES('EPSG','1370','Dansk Vertikal Reference 1990 (2023)',NULL,'2023-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','20433','vertical_datum','EPSG','1370','EPSG','3237','EPSG','1142');
INSERT INTO "vertical_datum" VALUES('EPSG','1372','Denmark Mean Sea Level (2022)',NULL,'2022-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','21004','vertical_datum','EPSG','1372','EPSG','4756','EPSG','1198');
INSERT INTO "vertical_datum" VALUES('EPSG','1373','Denmark Lowest Astronomical Tide (2022)',NULL,'2022-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','21005','vertical_datum','EPSG','1373','EPSG','4756','EPSG','1198');
INSERT INTO "vertical_datum" VALUES('EPSG','1374','Denmark Mean Sea Level (2023)',NULL,'2024-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','21267','vertical_datum','EPSG','1374','EPSG','4756','EPSG','1198');
INSERT INTO "vertical_datum" VALUES('EPSG','1375','Denmark Lowest Astronomic Tide (2023)',NULL,'2024-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','21041','vertical_datum','EPSG','1375','EPSG','4756','EPSG','1198');
INSERT INTO "vertical_datum" VALUES('EPSG','1378','Greenland Local Mean Sea Level (2022)',NULL,'2022-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','20999','vertical_datum','EPSG','1378','EPSG','3119','EPSG','1142');
INSERT INTO "vertical_datum" VALUES('EPSG','1381','Saba Vertical Datum',NULL,'2023-04-05',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','21862','vertical_datum','EPSG','1381','EPSG','4757','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','1384','Canadian Geodetic Vertical Datum of 1928 (Height Transformation version 2.0)',NULL,'2001-08-15',NULL,NULL,NULL,1997.0,0);
INSERT INTO "usage" VALUES('EPSG','21632','vertical_datum','EPSG','1384','EPSG','4778','EPSG','1132');
INSERT INTO "vertical_datum" VALUES('EPSG','1387','Greenland Mean Sea Level (2023)',NULL,'2024-06-18',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','21471','vertical_datum','EPSG','1387','EPSG','4776','EPSG','1198');
INSERT INTO "vertical_datum" VALUES('EPSG','1388','Greenland Lowest Astronomic Tide (2023)',NULL,'2024-06-04',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','21470','vertical_datum','EPSG','1388','EPSG','4776','EPSG','1198');
INSERT INTO "vertical_datum" VALUES('EPSG','1390','Baltic Sea Chart Datum 2000',NULL,'2023-11-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','21819','vertical_datum','EPSG','1390','EPSG','4779','EPSG','1291');
INSERT INTO "vertical_datum" VALUES('EPSG','1395','Sint Eustatius Vertical Datum',NULL,'2023-04-05',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','22281','vertical_datum','EPSG','1395','EPSG','4788','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','1398','Bonaire Vertical Datum',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','22174','vertical_datum','EPSG','1398','EPSG','3822','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','1415','Asse vertical datum 2025',NULL,'2025-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','23488','vertical_datum','EPSG','1415','EPSG','4805','EPSG','1144');
INSERT INTO "vertical_datum" VALUES('EPSG','1416','Derived California Orthometric Heights of 1988 epoch 2025',NULL,'2025-06-15',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','23577','vertical_datum','EPSG','1416','EPSG','1375','EPSG','1180');
INSERT INTO "vertical_datum" VALUES('EPSG','1424','Svalbard vertical datum 2024',NULL,'2025-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','24132','vertical_datum','EPSG','1424','EPSG','4831','EPSG','1180');
INSERT INTO "vertical_datum" VALUES('EPSG','1443','IGN 2023 Mayotte',NULL,'2025-03-14',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','24610','vertical_datum','EPSG','1443','EPSG','3340','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','5100','Mean Sea Level',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13307','vertical_datum','EPSG','5100','EPSG','1262','EPSG','1199');
INSERT INTO "vertical_datum" VALUES('EPSG','5101','Ordnance Datum Newlyn',NULL,'1956-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13308','vertical_datum','EPSG','5101','EPSG','2792','EPSG','1180');
INSERT INTO "vertical_datum" VALUES('EPSG','5102','National Geodetic Vertical Datum 1929',NULL,'1929-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13309','vertical_datum','EPSG','5102','EPSG','1323','EPSG','1180');
INSERT INTO "vertical_datum" VALUES('EPSG','5103','North American Vertical Datum 1988',NULL,'1993-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13310','vertical_datum','EPSG','5103','EPSG','4161','EPSG','1180');
INSERT INTO "vertical_datum" VALUES('EPSG','5104','Yellow Sea 1956',NULL,'1956-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13311','vertical_datum','EPSG','5104','EPSG','3228','EPSG','1180');
INSERT INTO "vertical_datum" VALUES('EPSG','5105','Baltic 1977',NULL,'1977-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13312','vertical_datum','EPSG','5105','EPSG','2423','EPSG','1180');
INSERT INTO "vertical_datum" VALUES('EPSG','5106','Caspian Sea',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13313','vertical_datum','EPSG','5106','EPSG','1291','EPSG','1198');
INSERT INTO "vertical_datum" VALUES('EPSG','5107','Nivellement general de la France',NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','13314','vertical_datum','EPSG','5107','EPSG','1326','EPSG','1180');
INSERT INTO "vertical_datum" VALUES('EPSG','5109','Normaal Amsterdams Peil',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13315','vertical_datum','EPSG','5109','EPSG','1172','EPSG','1180');
INSERT INTO "vertical_datum" VALUES('EPSG','5110','Ostend',NULL,'1981-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13316','vertical_datum','EPSG','5110','EPSG','1347','EPSG','1180');
INSERT INTO "vertical_datum" VALUES('EPSG','5111','Australian Height Datum',NULL,'1971-05-05',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13317','vertical_datum','EPSG','5111','EPSG','4493','EPSG','1263');
INSERT INTO "vertical_datum" VALUES('EPSG','5112','Australian Height Datum (Tasmania)',NULL,'1972-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13318','vertical_datum','EPSG','5112','EPSG','2947','EPSG','1180');
INSERT INTO "vertical_datum" VALUES('EPSG','5113','Instantaneous Water Level',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13319','vertical_datum','EPSG','5113','EPSG','1262','EPSG','1200');
INSERT INTO "vertical_datum" VALUES('EPSG','5114','Canadian Geodetic Vertical Datum of 1928',NULL,'1935-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13320','vertical_datum','EPSG','5114','EPSG','1289','EPSG','1180');
INSERT INTO "vertical_datum" VALUES('EPSG','5115','Piraeus Harbour 1986',NULL,'1986-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13321','vertical_datum','EPSG','5115','EPSG','3254','EPSG','1180');
INSERT INTO "vertical_datum" VALUES('EPSG','5116','Helsinki 1960',NULL,'1960-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13322','vertical_datum','EPSG','5116','EPSG','3333','EPSG','1180');
INSERT INTO "vertical_datum" VALUES('EPSG','5117','Rikets hojdsystem 1970',NULL,'1970-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13323','vertical_datum','EPSG','5117','EPSG','3313','EPSG','1180');
INSERT INTO "vertical_datum" VALUES('EPSG','5118','Nivellement General de la France - Lallemand',NULL,'1897-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13324','vertical_datum','EPSG','5118','EPSG','1326','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','5119','Nivellement General de la France - IGN69',NULL,'1969-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13325','vertical_datum','EPSG','5119','EPSG','1326','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','5120','Nivellement General de la France - IGN78',NULL,'1978-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13326','vertical_datum','EPSG','5120','EPSG','1327','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','5121','Maputo',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13327','vertical_datum','EPSG','5121','EPSG','3281','EPSG','1153');
INSERT INTO "vertical_datum" VALUES('EPSG','5122','Japanese Standard Levelling Datum 1969',NULL,'1969-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13328','vertical_datum','EPSG','5122','EPSG','4166','EPSG','1180');
INSERT INTO "vertical_datum" VALUES('EPSG','5123','PDO Height Datum 1993',NULL,'1993-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13329','vertical_datum','EPSG','5123','EPSG','3288','EPSG','1216');
INSERT INTO "vertical_datum" VALUES('EPSG','5124','Fahud Height Datum',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13330','vertical_datum','EPSG','5124','EPSG','4009','EPSG','1216');
INSERT INTO "vertical_datum" VALUES('EPSG','5125','Ha Tien 1960',NULL,'1960-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13331','vertical_datum','EPSG','5125','EPSG','1302','EPSG','1180');
INSERT INTO "vertical_datum" VALUES('EPSG','5126','Hon Dau 1992',NULL,'1992-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13332','vertical_datum','EPSG','5126','EPSG','4015','EPSG','1180');
INSERT INTO "vertical_datum" VALUES('EPSG','5127','Landesnivellement 1902',NULL,'1902-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13333','vertical_datum','EPSG','5127','EPSG','1286','EPSG','1091');
INSERT INTO "vertical_datum" VALUES('EPSG','5128','Landeshohennetz 1995',NULL,'1995-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13334','vertical_datum','EPSG','5128','EPSG','1286','EPSG','1027');
INSERT INTO "vertical_datum" VALUES('EPSG','5129','European Vertical Reference Frame 2000',NULL,'2000-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13335','vertical_datum','EPSG','5129','EPSG','1299','EPSG','1027');
INSERT INTO "vertical_datum" VALUES('EPSG','5130','Malin Head',NULL,'1970-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13336','vertical_datum','EPSG','5130','EPSG','1305','EPSG','1153');
INSERT INTO "vertical_datum" VALUES('EPSG','5131','Belfast Lough',NULL,'1957-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13337','vertical_datum','EPSG','5131','EPSG','2530','EPSG','1209');
INSERT INTO "vertical_datum" VALUES('EPSG','5132','Dansk Normal Nul',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13338','vertical_datum','EPSG','5132','EPSG','3237','EPSG','1142');
INSERT INTO "vertical_datum" VALUES('EPSG','5133','AIOC 1995',NULL,'1995-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13339','vertical_datum','EPSG','5133','EPSG','2592','EPSG','1136');
INSERT INTO "vertical_datum" VALUES('EPSG','5134','Black Sea',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13340','vertical_datum','EPSG','5134','EPSG','1102','EPSG','1201');
INSERT INTO "vertical_datum" VALUES('EPSG','5135','Hong Kong Principal Datum',NULL,'1980-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13341','vertical_datum','EPSG','5135','EPSG','3334','EPSG','1184');
INSERT INTO "vertical_datum" VALUES('EPSG','5136','Hong Kong Chart Datum',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13342','vertical_datum','EPSG','5136','EPSG','3335','EPSG','1198');
INSERT INTO "vertical_datum" VALUES('EPSG','5137','Yellow Sea 1985',NULL,'1985-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13343','vertical_datum','EPSG','5137','EPSG','3228','EPSG','1180');
INSERT INTO "vertical_datum" VALUES('EPSG','5138','Ordnance Datum Newlyn (Orkney Isles)',NULL,'1956-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13344','vertical_datum','EPSG','5138','EPSG','2793','EPSG','1180');
INSERT INTO "vertical_datum" VALUES('EPSG','5139','Fair Isle',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13345','vertical_datum','EPSG','5139','EPSG','2794','EPSG','1180');
INSERT INTO "vertical_datum" VALUES('EPSG','5140','Lerwick',NULL,'1979-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13346','vertical_datum','EPSG','5140','EPSG','2795','EPSG','1180');
INSERT INTO "vertical_datum" VALUES('EPSG','5141','Foula',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13347','vertical_datum','EPSG','5141','EPSG','2796','EPSG','1180');
INSERT INTO "vertical_datum" VALUES('EPSG','5142','Sule Skerry',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13348','vertical_datum','EPSG','5142','EPSG','2797','EPSG','1180');
INSERT INTO "vertical_datum" VALUES('EPSG','5143','North Rona',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13349','vertical_datum','EPSG','5143','EPSG','2798','EPSG','1180');
INSERT INTO "vertical_datum" VALUES('EPSG','5144','Stornoway',NULL,'1977-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13350','vertical_datum','EPSG','5144','EPSG','2799','EPSG','1180');
INSERT INTO "vertical_datum" VALUES('EPSG','5145','St. Kilda',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13351','vertical_datum','EPSG','5145','EPSG','2800','EPSG','1180');
INSERT INTO "vertical_datum" VALUES('EPSG','5146','Flannan Isles',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13352','vertical_datum','EPSG','5146','EPSG','2801','EPSG','1180');
INSERT INTO "vertical_datum" VALUES('EPSG','5147','St. Marys',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13353','vertical_datum','EPSG','5147','EPSG','2802','EPSG','1180');
INSERT INTO "vertical_datum" VALUES('EPSG','5148','Douglas',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13354','vertical_datum','EPSG','5148','EPSG','2803','EPSG','1180');
INSERT INTO "vertical_datum" VALUES('EPSG','5149','Fao',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13355','vertical_datum','EPSG','5149','EPSG','3390','EPSG','1180');
INSERT INTO "vertical_datum" VALUES('EPSG','5150','Bandar Abbas',NULL,'2001-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13356','vertical_datum','EPSG','5150','EPSG','3336','EPSG','1180');
INSERT INTO "vertical_datum" VALUES('EPSG','5151','Nivellement General de Nouvelle Caledonie',NULL,'1969-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13357','vertical_datum','EPSG','5151','EPSG','2822','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','5152','Poolbeg',NULL,'1837-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13358','vertical_datum','EPSG','5152','EPSG','1305','EPSG','1153');
INSERT INTO "vertical_datum" VALUES('EPSG','5153','Nivellement General Guyanais 1977',NULL,'1977-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13359','vertical_datum','EPSG','5153','EPSG','3146','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','5154','Martinique 1987',NULL,'1987-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13360','vertical_datum','EPSG','5154','EPSG','3276','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','5155','Guadeloupe 1988',NULL,'1988-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13361','vertical_datum','EPSG','5155','EPSG','2892','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','5156','Reunion 1989',NULL,'1989-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13362','vertical_datum','EPSG','5156','EPSG','3337','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','5157','Auckland 1946',NULL,'1945-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13363','vertical_datum','EPSG','5157','EPSG','3764','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','5158','Bluff 1955',NULL,'1955-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13364','vertical_datum','EPSG','5158','EPSG','3801','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','5159','Dunedin 1958',NULL,'1958-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13365','vertical_datum','EPSG','5159','EPSG','3803','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','5160','Gisborne 1926',NULL,'1926-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13366','vertical_datum','EPSG','5160','EPSG','3771','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','5161','Lyttelton 1937',NULL,'1937-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13367','vertical_datum','EPSG','5161','EPSG','3804','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','5162','Moturiki 1953',NULL,'1953-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13368','vertical_datum','EPSG','5162','EPSG','3768','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','5163','Napier 1962',NULL,'1962-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13369','vertical_datum','EPSG','5163','EPSG','3772','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','5164','Nelson 1955',NULL,'1955-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13370','vertical_datum','EPSG','5164','EPSG','3802','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','5165','One Tree Point 1964',NULL,'1964-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13371','vertical_datum','EPSG','5165','EPSG','3762','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','5166','Tararu 1952',NULL,'1952-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13372','vertical_datum','EPSG','5166','EPSG','3818','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','5167','Taranaki 1970',NULL,'1970-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13373','vertical_datum','EPSG','5167','EPSG','3769','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','5168','Wellington 1953',NULL,'1953-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13374','vertical_datum','EPSG','5168','EPSG','3773','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','5169','Waitangi (Chatham Island) 1959',NULL,'1959-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13375','vertical_datum','EPSG','5169','EPSG','3894','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','5170','Stewart Island 1977',NULL,'1977-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13376','vertical_datum','EPSG','5170','EPSG','3338','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','5171','EGM96 geoid',NULL,'1996-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13377','vertical_datum','EPSG','5171','EPSG','1262','EPSG','1027');
INSERT INTO "vertical_datum" VALUES('EPSG','5172','Nivellement General du Luxembourg 1995',NULL,'1995-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13378','vertical_datum','EPSG','5172','EPSG','1146','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','5173','Antalya',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13379','vertical_datum','EPSG','5173','EPSG','3322','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','5174','Norway Normal Null 1954',NULL,'1954-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13380','vertical_datum','EPSG','5174','EPSG','1352','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','5175','Durres',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13381','vertical_datum','EPSG','5175','EPSG','3212','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','5176','Gebrauchshohen ADRIA',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13382','vertical_datum','EPSG','5176','EPSG','1037','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','5177','Slovenian Vertical System 2000',NULL,'1999-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13383','vertical_datum','EPSG','5177','EPSG','3307','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','5178','Cascais',NULL,'1938-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13384','vertical_datum','EPSG','5178','EPSG','1294','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','5179','Constanta',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13385','vertical_datum','EPSG','5179','EPSG','3295','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','5180','Alicante',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13386','vertical_datum','EPSG','5180','EPSG','4188','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','5181','Deutsches Haupthoehennetz 1992',NULL,'1992-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13387','vertical_datum','EPSG','5181','EPSG','3339','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','5182','Deutsches Haupthoehennetz 1985',NULL,'1985-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13388','vertical_datum','EPSG','5182','EPSG','2326','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','5183','Staatlichen Nivellementnetzes 1976',NULL,'1976-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13389','vertical_datum','EPSG','5183','EPSG','1343','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','5184','Baltic 1982',NULL,'1982-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13390','vertical_datum','EPSG','5184','EPSG','3224','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','5185','Baltic 1980',NULL,'1980-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13391','vertical_datum','EPSG','5185','EPSG','1119','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','5186','Kuwait PWD',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13392','vertical_datum','EPSG','5186','EPSG','3267','EPSG','1248');
INSERT INTO "vertical_datum" VALUES('EPSG','5187','KOC Well Datum',NULL,'1937-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13393','vertical_datum','EPSG','5187','EPSG','3267','EPSG','1205');
INSERT INTO "vertical_datum" VALUES('EPSG','5188','KOC Construction Datum',NULL,'1952-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13394','vertical_datum','EPSG','5188','EPSG','3267','EPSG','1206');
INSERT INTO "vertical_datum" VALUES('EPSG','5189','Nivellement General de la Corse 1948',NULL,'1948-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13395','vertical_datum','EPSG','5189','EPSG','1327','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','5190','Danger 1950',NULL,'1950-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13396','vertical_datum','EPSG','5190','EPSG','3299','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','5191','Mayotte 1950',NULL,'1950-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13397','vertical_datum','EPSG','5191','EPSG','3340','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','5192','Martinique 1955',NULL,'1955-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13398','vertical_datum','EPSG','5192','EPSG','3276','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','5193','Guadeloupe 1951',NULL,'1951-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13399','vertical_datum','EPSG','5193','EPSG','2892','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','5194','Lagos 1955',NULL,'1955-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13400','vertical_datum','EPSG','5194','EPSG','3287','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','5195','Nivellement General de Polynesie Francaise',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13401','vertical_datum','EPSG','5195','EPSG','3134','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','5196','IGN 1966',NULL,'1966-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13402','vertical_datum','EPSG','5196','EPSG','3124','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','5197','Moorea SAU 1981',NULL,'1981-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13403','vertical_datum','EPSG','5197','EPSG','3125','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','5198','Raiatea SAU 2001',NULL,'2001-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13404','vertical_datum','EPSG','5198','EPSG','3136','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','5199','Maupiti SAU 2001',NULL,'2001-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13405','vertical_datum','EPSG','5199','EPSG','3126','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','5200','Huahine SAU 2001',NULL,'2001-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13406','vertical_datum','EPSG','5200','EPSG','3135','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','5201','Tahaa SAU 2001',NULL,'2001-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13407','vertical_datum','EPSG','5201','EPSG','3138','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','5202','Bora Bora SAU 2001',NULL,'2001-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13408','vertical_datum','EPSG','5202','EPSG','3137','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','5203','EGM84 geoid',NULL,'1987-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13409','vertical_datum','EPSG','5203','EPSG','1262','EPSG','1027');
INSERT INTO "vertical_datum" VALUES('EPSG','5204','International Great Lakes Datum 1955',NULL,'1955-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13410','vertical_datum','EPSG','5204','EPSG','3468','EPSG','1202');
INSERT INTO "vertical_datum" VALUES('EPSG','5205','International Great Lakes Datum 1985',NULL,'1985-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13411','vertical_datum','EPSG','5205','EPSG','3468','EPSG','1202');
INSERT INTO "vertical_datum" VALUES('EPSG','5206','Dansk Vertikal Reference 1990 (2000)',NULL,'2000-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13412','vertical_datum','EPSG','5206','EPSG','3237','EPSG','1142');
INSERT INTO "vertical_datum" VALUES('EPSG','5207','Croatian Vertical Reference Datum 1971',NULL,'1971-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13413','vertical_datum','EPSG','5207','EPSG','3234','EPSG','1027');
INSERT INTO "vertical_datum" VALUES('EPSG','5208','Rikets hojdsystem 2000',NULL,'2000-01-01',2000.0,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13414','vertical_datum','EPSG','5208','EPSG','3313','EPSG','1180');
INSERT INTO "vertical_datum" VALUES('EPSG','5209','Rikets hojdsystem 1900',NULL,'1900-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13415','vertical_datum','EPSG','5209','EPSG','3313','EPSG','1142');
INSERT INTO "vertical_datum" VALUES('EPSG','5210','IGN 1988 LS',NULL,'1988-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13416','vertical_datum','EPSG','5210','EPSG','2895','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','5211','IGN 1988 MG',NULL,'1988-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13417','vertical_datum','EPSG','5211','EPSG','2894','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','5212','IGN 1992 LD',NULL,'1992-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13418','vertical_datum','EPSG','5212','EPSG','2893','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','5213','IGN 1988 SB',NULL,'1988-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13419','vertical_datum','EPSG','5213','EPSG','2891','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','5214','IGN 1988 SM',NULL,'1988-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13420','vertical_datum','EPSG','5214','EPSG','2890','EPSG','1178');
INSERT INTO "vertical_datum" VALUES('EPSG','5215','European Vertical Reference Frame 2007',NULL,'2008-01-01',NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','14655','vertical_datum','EPSG','5215','EPSG','3594','EPSG','1027');
INSERT INTO "vertical_datum" VALUES('EPSG','1288','British Isles height ensemble',NULL,NULL,NULL,0.4,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','14086','vertical_datum','EPSG','1288','EPSG','4606','EPSG','1026');
INSERT INTO "vertical_datum" VALUES('EPSG','1371','Dansk Vertikal Reference 1990 ensemble',NULL,NULL,NULL,0.05,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','20365','vertical_datum','EPSG','1371','EPSG','3237','EPSG','1142');
INSERT INTO "vertical_datum" VALUES('EPSG','1376','Denmark Mean Sea Level ',NULL,NULL,NULL,0.5,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','21052','vertical_datum','EPSG','1376','EPSG','4756','EPSG','1198');
INSERT INTO "vertical_datum" VALUES('EPSG','1377','Denmark Lowest Astronomical Tide ',NULL,NULL,NULL,0.5,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','21051','vertical_datum','EPSG','1377','EPSG','4756','EPSG','1198');
