--- This file has been generated by scripts/build_wgs84_realizations_concatenated_operations.py. DO NOT EDIT !

-- Concatenated accuracy is sum of accuracies

INSERT INTO concatenated_operation VALUES('PROJ','WGS_84_TRANSIT_TO_WGS_84_G873','WGS 84 (Transit) to WGS 84 (G873)','Transformation based on concatenation of transformations between WGS 84 realizations','EPSG','7815','EPSG','7658',0.74,NULL,0);
INSERT INTO concatenated_operation_step VALUES('PROJ','WGS_84_TRANSIT_TO_WGS_84_G873',1,'EPSG','9960','forward'); -- WGS 84 (Transit) to WGS 84 (G730) (EPSG:9960), 0.7 m
INSERT INTO concatenated_operation_step VALUES('PROJ','WGS_84_TRANSIT_TO_WGS_84_G873',2,'EPSG','9961','forward'); -- WGS 84 (G730) to WGS 84 (G873) (EPSG:9961), 0.04 m
INSERT INTO usage VALUES('PROJ','WGS_84_TRANSIT_TO_WGS_84_G873_USAGE','concatenated_operation','PROJ','WGS_84_TRANSIT_TO_WGS_84_G873',
    'EPSG','1262', -- extent: World
    'EPSG','1027' -- scope: Geodesy
);

INSERT INTO concatenated_operation VALUES('PROJ','WGS_84_TRANSIT_TO_WGS_84_G1150','WGS 84 (Transit) to WGS 84 (G1150)','Transformation based on concatenation of transformations between WGS 84 realizations','EPSG','7815','EPSG','7660',0.77,NULL,0);
INSERT INTO concatenated_operation_step VALUES('PROJ','WGS_84_TRANSIT_TO_WGS_84_G1150',1,'EPSG','9960','forward'); -- WGS 84 (Transit) to WGS 84 (G730) (EPSG:9960), 0.7 m
INSERT INTO concatenated_operation_step VALUES('PROJ','WGS_84_TRANSIT_TO_WGS_84_G1150',2,'EPSG','9961','forward'); -- WGS 84 (G730) to WGS 84 (G873) (EPSG:9961), 0.04 m
INSERT INTO concatenated_operation_step VALUES('PROJ','WGS_84_TRANSIT_TO_WGS_84_G1150',3,'EPSG','9962','forward'); -- WGS 84 (G873) to WGS 84 (G1150) (EPSG:9962), 0.03 m
INSERT INTO usage VALUES('PROJ','WGS_84_TRANSIT_TO_WGS_84_G1150_USAGE','concatenated_operation','PROJ','WGS_84_TRANSIT_TO_WGS_84_G1150',
    'EPSG','1262', -- extent: World
    'EPSG','1027' -- scope: Geodesy
);

INSERT INTO concatenated_operation VALUES('PROJ','WGS_84_TRANSIT_TO_WGS_84_G1674','WGS 84 (Transit) to WGS 84 (G1674)','Transformation based on concatenation of transformations between WGS 84 realizations','EPSG','7815','EPSG','7662',0.79,NULL,0);
INSERT INTO concatenated_operation_step VALUES('PROJ','WGS_84_TRANSIT_TO_WGS_84_G1674',1,'EPSG','9960','forward'); -- WGS 84 (Transit) to WGS 84 (G730) (EPSG:9960), 0.7 m
INSERT INTO concatenated_operation_step VALUES('PROJ','WGS_84_TRANSIT_TO_WGS_84_G1674',2,'EPSG','9961','forward'); -- WGS 84 (G730) to WGS 84 (G873) (EPSG:9961), 0.04 m
INSERT INTO concatenated_operation_step VALUES('PROJ','WGS_84_TRANSIT_TO_WGS_84_G1674',3,'EPSG','9962','forward'); -- WGS 84 (G873) to WGS 84 (G1150) (EPSG:9962), 0.03 m
INSERT INTO concatenated_operation_step VALUES('PROJ','WGS_84_TRANSIT_TO_WGS_84_G1674',4,'EPSG','9963','forward'); -- WGS 84 (G1150) to WGS 84 (G1674) (EPSG:9963), 0.02 m
INSERT INTO usage VALUES('PROJ','WGS_84_TRANSIT_TO_WGS_84_G1674_USAGE','concatenated_operation','PROJ','WGS_84_TRANSIT_TO_WGS_84_G1674',
    'EPSG','1262', -- extent: World
    'EPSG','1027' -- scope: Geodesy
);

INSERT INTO concatenated_operation VALUES('PROJ','WGS_84_TRANSIT_TO_WGS_84_G1762','WGS 84 (Transit) to WGS 84 (G1762)','Transformation based on concatenation of transformations between WGS 84 realizations','EPSG','7815','EPSG','7664',0.79,NULL,0);
INSERT INTO concatenated_operation_step VALUES('PROJ','WGS_84_TRANSIT_TO_WGS_84_G1762',1,'EPSG','9960','forward'); -- WGS 84 (Transit) to WGS 84 (G730) (EPSG:9960), 0.7 m
INSERT INTO concatenated_operation_step VALUES('PROJ','WGS_84_TRANSIT_TO_WGS_84_G1762',2,'EPSG','9961','forward'); -- WGS 84 (G730) to WGS 84 (G873) (EPSG:9961), 0.04 m
INSERT INTO concatenated_operation_step VALUES('PROJ','WGS_84_TRANSIT_TO_WGS_84_G1762',3,'EPSG','9962','forward'); -- WGS 84 (G873) to WGS 84 (G1150) (EPSG:9962), 0.03 m
INSERT INTO concatenated_operation_step VALUES('PROJ','WGS_84_TRANSIT_TO_WGS_84_G1762',4,'EPSG','7668','forward'); -- WGS 84 (G1150) to WGS 84 (G1762) (EPSG:7668), 0.02 m
INSERT INTO usage VALUES('PROJ','WGS_84_TRANSIT_TO_WGS_84_G1762_USAGE','concatenated_operation','PROJ','WGS_84_TRANSIT_TO_WGS_84_G1762',
    'EPSG','1262', -- extent: World
    'EPSG','1027' -- scope: Geodesy
);

INSERT INTO concatenated_operation VALUES('PROJ','WGS_84_TRANSIT_TO_WGS_84_G2139','WGS 84 (Transit) to WGS 84 (G2139)','Transformation based on concatenation of transformations between WGS 84 realizations','EPSG','7815','EPSG','9753',0.8,NULL,0);
INSERT INTO concatenated_operation_step VALUES('PROJ','WGS_84_TRANSIT_TO_WGS_84_G2139',1,'EPSG','9960','forward'); -- WGS 84 (Transit) to WGS 84 (G730) (EPSG:9960), 0.7 m
INSERT INTO concatenated_operation_step VALUES('PROJ','WGS_84_TRANSIT_TO_WGS_84_G2139',2,'EPSG','9961','forward'); -- WGS 84 (G730) to WGS 84 (G873) (EPSG:9961), 0.04 m
INSERT INTO concatenated_operation_step VALUES('PROJ','WGS_84_TRANSIT_TO_WGS_84_G2139',3,'EPSG','9962','forward'); -- WGS 84 (G873) to WGS 84 (G1150) (EPSG:9962), 0.03 m
INSERT INTO concatenated_operation_step VALUES('PROJ','WGS_84_TRANSIT_TO_WGS_84_G2139',4,'EPSG','7668','forward'); -- WGS 84 (G1150) to WGS 84 (G1762) (EPSG:7668), 0.02 m
INSERT INTO concatenated_operation_step VALUES('PROJ','WGS_84_TRANSIT_TO_WGS_84_G2139',5,'EPSG','9756','forward'); -- WGS 84 (G1762) to WGS 84 (G2139) (EPSG:9756), 0.01 m
INSERT INTO usage VALUES('PROJ','WGS_84_TRANSIT_TO_WGS_84_G2139_USAGE','concatenated_operation','PROJ','WGS_84_TRANSIT_TO_WGS_84_G2139',
    'EPSG','1262', -- extent: World
    'EPSG','1027' -- scope: Geodesy
);

INSERT INTO concatenated_operation VALUES('PROJ','WGS_84_TRANSIT_TO_WGS_84_G2296','WGS 84 (Transit) to WGS 84 (G2296)','Transformation based on concatenation of transformations between WGS 84 realizations','EPSG','7815','EPSG','10604',0.81,NULL,0);
INSERT INTO concatenated_operation_step VALUES('PROJ','WGS_84_TRANSIT_TO_WGS_84_G2296',1,'EPSG','9960','forward'); -- WGS 84 (Transit) to WGS 84 (G730) (EPSG:9960), 0.7 m
INSERT INTO concatenated_operation_step VALUES('PROJ','WGS_84_TRANSIT_TO_WGS_84_G2296',2,'EPSG','9961','forward'); -- WGS 84 (G730) to WGS 84 (G873) (EPSG:9961), 0.04 m
INSERT INTO concatenated_operation_step VALUES('PROJ','WGS_84_TRANSIT_TO_WGS_84_G2296',3,'EPSG','9962','forward'); -- WGS 84 (G873) to WGS 84 (G1150) (EPSG:9962), 0.03 m
INSERT INTO concatenated_operation_step VALUES('PROJ','WGS_84_TRANSIT_TO_WGS_84_G2296',4,'EPSG','7668','forward'); -- WGS 84 (G1150) to WGS 84 (G1762) (EPSG:7668), 0.02 m
INSERT INTO concatenated_operation_step VALUES('PROJ','WGS_84_TRANSIT_TO_WGS_84_G2296',5,'EPSG','9756','forward'); -- WGS 84 (G1762) to WGS 84 (G2139) (EPSG:9756), 0.01 m
INSERT INTO concatenated_operation_step VALUES('PROJ','WGS_84_TRANSIT_TO_WGS_84_G2296',6,'EPSG','10607','forward'); -- WGS 84 (G2139) to WGS 84 (G2296) (EPSG:10607), 0.01 m
INSERT INTO usage VALUES('PROJ','WGS_84_TRANSIT_TO_WGS_84_G2296_USAGE','concatenated_operation','PROJ','WGS_84_TRANSIT_TO_WGS_84_G2296',
    'EPSG','1262', -- extent: World
    'EPSG','1027' -- scope: Geodesy
);

INSERT INTO concatenated_operation VALUES('PROJ','WGS_84_G730_TO_WGS_84_G1150','WGS 84 (G730) to WGS 84 (G1150)','Transformation based on concatenation of transformations between WGS 84 realizations','EPSG','7656','EPSG','7660',0.07,NULL,0);
INSERT INTO concatenated_operation_step VALUES('PROJ','WGS_84_G730_TO_WGS_84_G1150',1,'EPSG','9961','forward'); -- WGS 84 (G730) to WGS 84 (G873) (EPSG:9961), 0.04 m
INSERT INTO concatenated_operation_step VALUES('PROJ','WGS_84_G730_TO_WGS_84_G1150',2,'EPSG','9962','forward'); -- WGS 84 (G873) to WGS 84 (G1150) (EPSG:9962), 0.03 m
INSERT INTO usage VALUES('PROJ','WGS_84_G730_TO_WGS_84_G1150_USAGE','concatenated_operation','PROJ','WGS_84_G730_TO_WGS_84_G1150',
    'EPSG','1262', -- extent: World
    'EPSG','1027' -- scope: Geodesy
);

INSERT INTO concatenated_operation VALUES('PROJ','WGS_84_G730_TO_WGS_84_G1674','WGS 84 (G730) to WGS 84 (G1674)','Transformation based on concatenation of transformations between WGS 84 realizations','EPSG','7656','EPSG','7662',0.09,NULL,0);
INSERT INTO concatenated_operation_step VALUES('PROJ','WGS_84_G730_TO_WGS_84_G1674',1,'EPSG','9961','forward'); -- WGS 84 (G730) to WGS 84 (G873) (EPSG:9961), 0.04 m
INSERT INTO concatenated_operation_step VALUES('PROJ','WGS_84_G730_TO_WGS_84_G1674',2,'EPSG','9962','forward'); -- WGS 84 (G873) to WGS 84 (G1150) (EPSG:9962), 0.03 m
INSERT INTO concatenated_operation_step VALUES('PROJ','WGS_84_G730_TO_WGS_84_G1674',3,'EPSG','9963','forward'); -- WGS 84 (G1150) to WGS 84 (G1674) (EPSG:9963), 0.02 m
INSERT INTO usage VALUES('PROJ','WGS_84_G730_TO_WGS_84_G1674_USAGE','concatenated_operation','PROJ','WGS_84_G730_TO_WGS_84_G1674',
    'EPSG','1262', -- extent: World
    'EPSG','1027' -- scope: Geodesy
);

INSERT INTO concatenated_operation VALUES('PROJ','WGS_84_G730_TO_WGS_84_G1762','WGS 84 (G730) to WGS 84 (G1762)','Transformation based on concatenation of transformations between WGS 84 realizations','EPSG','7656','EPSG','7664',0.09,NULL,0);
INSERT INTO concatenated_operation_step VALUES('PROJ','WGS_84_G730_TO_WGS_84_G1762',1,'EPSG','9961','forward'); -- WGS 84 (G730) to WGS 84 (G873) (EPSG:9961), 0.04 m
INSERT INTO concatenated_operation_step VALUES('PROJ','WGS_84_G730_TO_WGS_84_G1762',2,'EPSG','9962','forward'); -- WGS 84 (G873) to WGS 84 (G1150) (EPSG:9962), 0.03 m
INSERT INTO concatenated_operation_step VALUES('PROJ','WGS_84_G730_TO_WGS_84_G1762',3,'EPSG','7668','forward'); -- WGS 84 (G1150) to WGS 84 (G1762) (EPSG:7668), 0.02 m
INSERT INTO usage VALUES('PROJ','WGS_84_G730_TO_WGS_84_G1762_USAGE','concatenated_operation','PROJ','WGS_84_G730_TO_WGS_84_G1762',
    'EPSG','1262', -- extent: World
    'EPSG','1027' -- scope: Geodesy
);

INSERT INTO concatenated_operation VALUES('PROJ','WGS_84_G730_TO_WGS_84_G2139','WGS 84 (G730) to WGS 84 (G2139)','Transformation based on concatenation of transformations between WGS 84 realizations','EPSG','7656','EPSG','9753',0.1,NULL,0);
INSERT INTO concatenated_operation_step VALUES('PROJ','WGS_84_G730_TO_WGS_84_G2139',1,'EPSG','9961','forward'); -- WGS 84 (G730) to WGS 84 (G873) (EPSG:9961), 0.04 m
INSERT INTO concatenated_operation_step VALUES('PROJ','WGS_84_G730_TO_WGS_84_G2139',2,'EPSG','9962','forward'); -- WGS 84 (G873) to WGS 84 (G1150) (EPSG:9962), 0.03 m
INSERT INTO concatenated_operation_step VALUES('PROJ','WGS_84_G730_TO_WGS_84_G2139',3,'EPSG','7668','forward'); -- WGS 84 (G1150) to WGS 84 (G1762) (EPSG:7668), 0.02 m
INSERT INTO concatenated_operation_step VALUES('PROJ','WGS_84_G730_TO_WGS_84_G2139',4,'EPSG','9756','forward'); -- WGS 84 (G1762) to WGS 84 (G2139) (EPSG:9756), 0.01 m
INSERT INTO usage VALUES('PROJ','WGS_84_G730_TO_WGS_84_G2139_USAGE','concatenated_operation','PROJ','WGS_84_G730_TO_WGS_84_G2139',
    'EPSG','1262', -- extent: World
    'EPSG','1027' -- scope: Geodesy
);

INSERT INTO concatenated_operation VALUES('PROJ','WGS_84_G730_TO_WGS_84_G2296','WGS 84 (G730) to WGS 84 (G2296)','Transformation based on concatenation of transformations between WGS 84 realizations','EPSG','7656','EPSG','10604',0.11,NULL,0);
INSERT INTO concatenated_operation_step VALUES('PROJ','WGS_84_G730_TO_WGS_84_G2296',1,'EPSG','9961','forward'); -- WGS 84 (G730) to WGS 84 (G873) (EPSG:9961), 0.04 m
INSERT INTO concatenated_operation_step VALUES('PROJ','WGS_84_G730_TO_WGS_84_G2296',2,'EPSG','9962','forward'); -- WGS 84 (G873) to WGS 84 (G1150) (EPSG:9962), 0.03 m
INSERT INTO concatenated_operation_step VALUES('PROJ','WGS_84_G730_TO_WGS_84_G2296',3,'EPSG','7668','forward'); -- WGS 84 (G1150) to WGS 84 (G1762) (EPSG:7668), 0.02 m
INSERT INTO concatenated_operation_step VALUES('PROJ','WGS_84_G730_TO_WGS_84_G2296',4,'EPSG','9756','forward'); -- WGS 84 (G1762) to WGS 84 (G2139) (EPSG:9756), 0.01 m
INSERT INTO concatenated_operation_step VALUES('PROJ','WGS_84_G730_TO_WGS_84_G2296',5,'EPSG','10607','forward'); -- WGS 84 (G2139) to WGS 84 (G2296) (EPSG:10607), 0.01 m
INSERT INTO usage VALUES('PROJ','WGS_84_G730_TO_WGS_84_G2296_USAGE','concatenated_operation','PROJ','WGS_84_G730_TO_WGS_84_G2296',
    'EPSG','1262', -- extent: World
    'EPSG','1027' -- scope: Geodesy
);

INSERT INTO concatenated_operation VALUES('PROJ','WGS_84_G873_TO_WGS_84_G1674','WGS 84 (G873) to WGS 84 (G1674)','Transformation based on concatenation of transformations between WGS 84 realizations','EPSG','7658','EPSG','7662',0.05,NULL,0);
INSERT INTO concatenated_operation_step VALUES('PROJ','WGS_84_G873_TO_WGS_84_G1674',1,'EPSG','9962','forward'); -- WGS 84 (G873) to WGS 84 (G1150) (EPSG:9962), 0.03 m
INSERT INTO concatenated_operation_step VALUES('PROJ','WGS_84_G873_TO_WGS_84_G1674',2,'EPSG','9963','forward'); -- WGS 84 (G1150) to WGS 84 (G1674) (EPSG:9963), 0.02 m
INSERT INTO usage VALUES('PROJ','WGS_84_G873_TO_WGS_84_G1674_USAGE','concatenated_operation','PROJ','WGS_84_G873_TO_WGS_84_G1674',
    'EPSG','1262', -- extent: World
    'EPSG','1027' -- scope: Geodesy
);

INSERT INTO concatenated_operation VALUES('PROJ','WGS_84_G873_TO_WGS_84_G1762','WGS 84 (G873) to WGS 84 (G1762)','Transformation based on concatenation of transformations between WGS 84 realizations','EPSG','7658','EPSG','7664',0.05,NULL,0);
INSERT INTO concatenated_operation_step VALUES('PROJ','WGS_84_G873_TO_WGS_84_G1762',1,'EPSG','9962','forward'); -- WGS 84 (G873) to WGS 84 (G1150) (EPSG:9962), 0.03 m
INSERT INTO concatenated_operation_step VALUES('PROJ','WGS_84_G873_TO_WGS_84_G1762',2,'EPSG','7668','forward'); -- WGS 84 (G1150) to WGS 84 (G1762) (EPSG:7668), 0.02 m
INSERT INTO usage VALUES('PROJ','WGS_84_G873_TO_WGS_84_G1762_USAGE','concatenated_operation','PROJ','WGS_84_G873_TO_WGS_84_G1762',
    'EPSG','1262', -- extent: World
    'EPSG','1027' -- scope: Geodesy
);

INSERT INTO concatenated_operation VALUES('PROJ','WGS_84_G873_TO_WGS_84_G2139','WGS 84 (G873) to WGS 84 (G2139)','Transformation based on concatenation of transformations between WGS 84 realizations','EPSG','7658','EPSG','9753',0.06,NULL,0);
INSERT INTO concatenated_operation_step VALUES('PROJ','WGS_84_G873_TO_WGS_84_G2139',1,'EPSG','9962','forward'); -- WGS 84 (G873) to WGS 84 (G1150) (EPSG:9962), 0.03 m
INSERT INTO concatenated_operation_step VALUES('PROJ','WGS_84_G873_TO_WGS_84_G2139',2,'EPSG','7668','forward'); -- WGS 84 (G1150) to WGS 84 (G1762) (EPSG:7668), 0.02 m
INSERT INTO concatenated_operation_step VALUES('PROJ','WGS_84_G873_TO_WGS_84_G2139',3,'EPSG','9756','forward'); -- WGS 84 (G1762) to WGS 84 (G2139) (EPSG:9756), 0.01 m
INSERT INTO usage VALUES('PROJ','WGS_84_G873_TO_WGS_84_G2139_USAGE','concatenated_operation','PROJ','WGS_84_G873_TO_WGS_84_G2139',
    'EPSG','1262', -- extent: World
    'EPSG','1027' -- scope: Geodesy
);

INSERT INTO concatenated_operation VALUES('PROJ','WGS_84_G873_TO_WGS_84_G2296','WGS 84 (G873) to WGS 84 (G2296)','Transformation based on concatenation of transformations between WGS 84 realizations','EPSG','7658','EPSG','10604',0.07,NULL,0);
INSERT INTO concatenated_operation_step VALUES('PROJ','WGS_84_G873_TO_WGS_84_G2296',1,'EPSG','9962','forward'); -- WGS 84 (G873) to WGS 84 (G1150) (EPSG:9962), 0.03 m
INSERT INTO concatenated_operation_step VALUES('PROJ','WGS_84_G873_TO_WGS_84_G2296',2,'EPSG','7668','forward'); -- WGS 84 (G1150) to WGS 84 (G1762) (EPSG:7668), 0.02 m
INSERT INTO concatenated_operation_step VALUES('PROJ','WGS_84_G873_TO_WGS_84_G2296',3,'EPSG','9756','forward'); -- WGS 84 (G1762) to WGS 84 (G2139) (EPSG:9756), 0.01 m
INSERT INTO concatenated_operation_step VALUES('PROJ','WGS_84_G873_TO_WGS_84_G2296',4,'EPSG','10607','forward'); -- WGS 84 (G2139) to WGS 84 (G2296) (EPSG:10607), 0.01 m
INSERT INTO usage VALUES('PROJ','WGS_84_G873_TO_WGS_84_G2296_USAGE','concatenated_operation','PROJ','WGS_84_G873_TO_WGS_84_G2296',
    'EPSG','1262', -- extent: World
    'EPSG','1027' -- scope: Geodesy
);

INSERT INTO concatenated_operation VALUES('PROJ','WGS_84_G1150_TO_WGS_84_G2139','WGS 84 (G1150) to WGS 84 (G2139)','Transformation based on concatenation of transformations between WGS 84 realizations','EPSG','7660','EPSG','9753',0.03,NULL,0);
INSERT INTO concatenated_operation_step VALUES('PROJ','WGS_84_G1150_TO_WGS_84_G2139',1,'EPSG','7668','forward'); -- WGS 84 (G1150) to WGS 84 (G1762) (EPSG:7668), 0.02 m
INSERT INTO concatenated_operation_step VALUES('PROJ','WGS_84_G1150_TO_WGS_84_G2139',2,'EPSG','9756','forward'); -- WGS 84 (G1762) to WGS 84 (G2139) (EPSG:9756), 0.01 m
INSERT INTO usage VALUES('PROJ','WGS_84_G1150_TO_WGS_84_G2139_USAGE','concatenated_operation','PROJ','WGS_84_G1150_TO_WGS_84_G2139',
    'EPSG','1262', -- extent: World
    'EPSG','1027' -- scope: Geodesy
);

INSERT INTO concatenated_operation VALUES('PROJ','WGS_84_G1150_TO_WGS_84_G2296','WGS 84 (G1150) to WGS 84 (G2296)','Transformation based on concatenation of transformations between WGS 84 realizations','EPSG','7660','EPSG','10604',0.04,NULL,0);
INSERT INTO concatenated_operation_step VALUES('PROJ','WGS_84_G1150_TO_WGS_84_G2296',1,'EPSG','7668','forward'); -- WGS 84 (G1150) to WGS 84 (G1762) (EPSG:7668), 0.02 m
INSERT INTO concatenated_operation_step VALUES('PROJ','WGS_84_G1150_TO_WGS_84_G2296',2,'EPSG','9756','forward'); -- WGS 84 (G1762) to WGS 84 (G2139) (EPSG:9756), 0.01 m
INSERT INTO concatenated_operation_step VALUES('PROJ','WGS_84_G1150_TO_WGS_84_G2296',3,'EPSG','10607','forward'); -- WGS 84 (G2139) to WGS 84 (G2296) (EPSG:10607), 0.01 m
INSERT INTO usage VALUES('PROJ','WGS_84_G1150_TO_WGS_84_G2296_USAGE','concatenated_operation','PROJ','WGS_84_G1150_TO_WGS_84_G2296',
    'EPSG','1262', -- extent: World
    'EPSG','1027' -- scope: Geodesy
);

INSERT INTO concatenated_operation VALUES('PROJ','WGS_84_G1674_TO_WGS_84_G2139','WGS 84 (G1674) to WGS 84 (G2139)','Transformation based on concatenation of transformations between WGS 84 realizations','EPSG','7662','EPSG','9753',0.02,NULL,0);
INSERT INTO concatenated_operation_step VALUES('PROJ','WGS_84_G1674_TO_WGS_84_G2139',1,'EPSG','7667','forward'); -- WGS 84 (G1674) to WGS 84 (G1762) (EPSG:7667), 0.01 m
INSERT INTO concatenated_operation_step VALUES('PROJ','WGS_84_G1674_TO_WGS_84_G2139',2,'EPSG','9756','forward'); -- WGS 84 (G1762) to WGS 84 (G2139) (EPSG:9756), 0.01 m
INSERT INTO usage VALUES('PROJ','WGS_84_G1674_TO_WGS_84_G2139_USAGE','concatenated_operation','PROJ','WGS_84_G1674_TO_WGS_84_G2139',
    'EPSG','1262', -- extent: World
    'EPSG','1027' -- scope: Geodesy
);

INSERT INTO concatenated_operation VALUES('PROJ','WGS_84_G1674_TO_WGS_84_G2296','WGS 84 (G1674) to WGS 84 (G2296)','Transformation based on concatenation of transformations between WGS 84 realizations','EPSG','7662','EPSG','10604',0.03,NULL,0);
INSERT INTO concatenated_operation_step VALUES('PROJ','WGS_84_G1674_TO_WGS_84_G2296',1,'EPSG','7667','forward'); -- WGS 84 (G1674) to WGS 84 (G1762) (EPSG:7667), 0.01 m
INSERT INTO concatenated_operation_step VALUES('PROJ','WGS_84_G1674_TO_WGS_84_G2296',2,'EPSG','9756','forward'); -- WGS 84 (G1762) to WGS 84 (G2139) (EPSG:9756), 0.01 m
INSERT INTO concatenated_operation_step VALUES('PROJ','WGS_84_G1674_TO_WGS_84_G2296',3,'EPSG','10607','forward'); -- WGS 84 (G2139) to WGS 84 (G2296) (EPSG:10607), 0.01 m
INSERT INTO usage VALUES('PROJ','WGS_84_G1674_TO_WGS_84_G2296_USAGE','concatenated_operation','PROJ','WGS_84_G1674_TO_WGS_84_G2296',
    'EPSG','1262', -- extent: World
    'EPSG','1027' -- scope: Geodesy
);

INSERT INTO concatenated_operation VALUES('PROJ','WGS_84_G1762_TO_WGS_84_G2296','WGS 84 (G1762) to WGS 84 (G2296)','Transformation based on concatenation of transformations between WGS 84 realizations','EPSG','7664','EPSG','10604',0.02,NULL,0);
INSERT INTO concatenated_operation_step VALUES('PROJ','WGS_84_G1762_TO_WGS_84_G2296',1,'EPSG','9756','forward'); -- WGS 84 (G1762) to WGS 84 (G2139) (EPSG:9756), 0.01 m
INSERT INTO concatenated_operation_step VALUES('PROJ','WGS_84_G1762_TO_WGS_84_G2296',2,'EPSG','10607','forward'); -- WGS 84 (G2139) to WGS 84 (G2296) (EPSG:10607), 0.01 m
INSERT INTO usage VALUES('PROJ','WGS_84_G1762_TO_WGS_84_G2296_USAGE','concatenated_operation','PROJ','WGS_84_G1762_TO_WGS_84_G2296',
    'EPSG','1262', -- extent: World
    'EPSG','1027' -- scope: Geodesy
);

