--- This file has been generated by scripts/build_db.py. DO NOT EDIT !

-- Non-imported: (1026, 'transformation', 'Madrid 1870 (Madrid) to ED50 (1)', 9617, 'Madrid to ED50 polynomial', 4903, 4230, 10.0, 0)
-- Non-imported: (1027, 'transformation', 'Madrid 1870 (Madrid) to ED50 (2)', 9617, 'Madrid to ED50 polynomial', 4903, 4230, 5.0, 0)
-- Non-imported: (1028, 'transformation', 'Madrid 1870 (Madrid) to ED50 (3)', 9617, 'Madrid to ED50 polynomial', 4903, 4230, 5.0, 0)
-- Non-imported: (1041, 'transformation', 'TM75 to ETRS89 (1)', 9648, 'General polynomial of degree 6', 4300, 4258, 0.4, 0)
-- Non-imported: (1042, 'transformation', 'TM75 to WGS 84 (1)', 9648, 'General polynomial of degree 6', 4300, 4326, 1.0, 0)
-- Non-imported: (1044, 'transformation', 'Amersfoort / RD New to ED50 / UTM zone 31N (1)', 9653, 'Complex polynomial of degree 4', 28992, 23031, 1.0, 0)
-- Non-imported: (1046, 'transformation', 'Amersfoort / RD New to ED50 / UTM zone 31N (2)', 9653, 'Complex polynomial of degree 4', 28992, 23031, 1.0, 0)
-- Non-imported: (1048, 'transformation', 'BD72 / Lambert 72 to ED50 / UTM zone 31N (1)', 9652, 'Complex polynomial of degree 3', 31300, 23031, 1.0, 0)
-- Non-imported: (1050, 'transformation', 'Amersfoort / RD New to ED50 / TM 5 NE (1)', 9653, 'Complex polynomial of degree 4', 28992, 23095, 1.0, 0)
-- Non-imported: (1052, 'transformation', 'ED50 to WGS 84 (35)', 9654, 'Reversible polynomial of degree 13', 4230, 4326, 5.0, 0)
-- Non-imported: (1450, 'transformation', 'ED50 to WGS 84 (20)', 9620, 'Norway Offshore Interpolation', 4230, 4326, 0.5, 0)
-- Non-imported: (1589, 'transformation', 'ED50 to ETRS89 (3)', 9620, 'Norway Offshore Interpolation', 4230, 4258, 2.0, 0)
-- Non-imported: (1590, 'transformation', 'ED50 to WGS 84 (22)', 9620, 'Norway Offshore Interpolation', 4230, 4326, 0.5, 0)
-- Non-imported: (1603, 'transformation', 'NAD27 to ATS77 (1)', 9634, 'Maritime Provinces polynomial interpolation', 4267, 4122, 1.5, 0)
-- Non-imported: (1604, 'transformation', 'NAD27 to ATS77 (2)', 9634, 'Maritime Provinces polynomial interpolation', 4267, 4122, 1.5, 0)
-- Non-imported: (1605, 'transformation', 'NAD27 to ATS77 (3)', 9634, 'Maritime Provinces polynomial interpolation', 4267, 4122, 1.5, 0)
-- Non-imported: (1947, 'transformation', 'ATS77 to NAD27 (1)', 9634, 'Maritime Provinces polynomial interpolation', 4122, 4267, 1.5, 0)
-- Non-imported: (1948, 'transformation', 'ATS77 to NAD27 (2)', 9634, 'Maritime Provinces polynomial interpolation', 4122, 4267, 1.5, 0)
-- Non-imported: (1949, 'transformation', 'ATS77 to NAD27 (3)', 9634, 'Maritime Provinces polynomial interpolation', 4122, 4267, 1.5, 0)
-- Non-imported: (4649, 'transformation', 'DHDN / 3-degree Gauss-Kruger zone 3 to ETRS89 / zone TM32 (1)', 1036, 'Cartesian Grid Offsets from Form Function', 31467, 4647, 0.05, 0)
-- Non-imported: (4650, 'transformation', 'DHDN / 3-degree Gauss-Kruger zone 4 to ETRS89 / zone TM32 (1)', 1036, 'Cartesian Grid Offsets from Form Function', 31468, 4647, 0.05, 0)
-- Non-imported: (5073, 'transformation', 'DHDN / 3-degree Gauss-Kruger zone 4 to ETRS89 / UTM zone 32N (1)', 1040, 'GNTRANS', 31468, 25832, 0.02, 0)
-- Non-imported: (5074, 'transformation', 'DHDN / 3-degree Gauss-Kruger zone 3 to ETRS89 / UTM zone 32N (1)', 1040, 'GNTRANS', 31467, 25832, 0.02, 0)
-- Non-imported: (5075, 'transformation', 'DHDN / 3-degree Gauss-Kruger zone 2 to ETRS89 / UTM zone 32N (1)', 1040, 'GNTRANS', 31466, 25832, 0.02, 0)
-- Non-imported: (5686, 'transformation', 'DB_REF / 3GK zone 2 (E-N) to ETRS89 / UTM zone 32N (1)', 1040, 'GNTRANS', 5682, 25832, 0.05, 0)
-- Non-imported: (5687, 'transformation', 'DB_REF / 3GK zone 3 (E-N) to ETRS89 / UTM zone 32N (1)', 1040, 'GNTRANS', 5683, 25832, 0.05, 0)
-- Non-imported: (5688, 'transformation', 'DB_REF / 3GK zone 4 (E-N) to ETRS89 / UTM zone 32N (1)', 1040, 'GNTRANS', 5684, 25832, 0.1, 0)
-- Non-imported: (5689, 'transformation', 'DB_REF / 3GK zone 4 (E-N) to ETRS89 / UTM zone 33N (1)', 1040, 'GNTRANS', 5684, 25833, 0.1, 0)
-- Non-imported: (5690, 'transformation', 'DB_REF / 3GK zone 5 (E-N) to ETRS89 / UTM zone 33N (1)', 1040, 'GNTRANS', 5685, 25833, 0.1, 0)
-- Non-imported: (5691, 'transformation', 'DB_REF / 3GK zone 2 (E-N) to DHDN / 3GK zone 2 (E-N) (1)', 1040, 'GNTRANS', 5682, 5676, 0.05, 0)
-- Non-imported: (5692, 'transformation', 'DB_REF / 3GK zone 3 (E-N) to DHDN / 3GK zone 3 (E-N) (1)', 1040, 'GNTRANS', 5683, 5676, 0.05, 0)
-- Non-imported: (5693, 'transformation', 'DB_REF / 3GK zone 4 (E-N) to DHDN / 3GK zone 4 (E-N) (1)', 1040, 'GNTRANS', 5684, 5678, 0.05, 0)
-- Non-imported: (5694, 'transformation', 'DB_REF / 3GK zone 5 (E-N) to DHDN / 3GK zone 5 (E-N) (1)', 1040, 'GNTRANS', 5685, 5679, 0.05, 0)
-- Non-imported: (5695, 'transformation', 'DB_REF / 3GK zone 3 (E-N) to Pulkovo 1942(83) / 3GK zone 3 (E-N) (1)', 1040, 'GNTRANS', 5683, 5673, 0.1, 0)
-- Non-imported: (5696, 'transformation', 'DB_REF / 3GK zone 4 (E-N) to Pulkovo 1942(83) / 3GK zone 4 (E-N) (1)', 1040, 'GNTRANS', 5684, 5674, 0.1, 0)
-- Non-imported: (5697, 'transformation', 'DB_REF / 3GK zone 5 (E-N) to Pulkovo 1942(83) / 3GK zone 5 (E-N) (1)', 1040, 'GNTRANS', 5685, 5675, 0.1, 0)
-- Non-imported: (6303, 'transformation', 'ED50 / UTM zone 31N to Amersfoort / RD New (1)', 9653, 'Complex polynomial of degree 4', 23031, 28992, 1.0, 0)
-- Non-imported: (6304, 'transformation', 'ED50 / UTM zone 31N to Amersfoort / RD New (2)', 9653, 'Complex polynomial of degree 4', 23031, 28992, 1.0, 0)
-- Non-imported: (6305, 'transformation', 'ED50 / UTM zone 31N to BD72 / Lambert 72 (1)', 9652, 'Complex polynomial of degree 3', 23031, 31300, 1.0, 0)
-- Non-imported: (6306, 'transformation', 'ED50 / TM 5 NE to Amersfoort / RD New (1)', 9653, 'Complex polynomial of degree 4', 23095, 28992, 1.0, 0)
-- Non-imported: (7913, 'transformation', 'ETRS89 to OSGB36 / British National Grid (1)', 9633, 'Ordnance Survey National Transformation', 4258, 27700, 0.2, 0)
-- Non-imported: (7952, 'transformation', 'ETRS89 to OSGB36 / British National Grid (2)', 9633, 'Ordnance Survey National Transformation', 4258, 27700, 0.0, 0)
-- Non-imported: (7953, 'transformation', 'ETRS89 to OSGB36 / British National Grid (3)', 9633, 'Ordnance Survey National Transformation', 4258, 27700, 0.0, 0)
-- Non-imported: (8241, 'concatenated operation', 'Madrid 1870 (Madrid) to WGS 84 (1)', None, None, 4903, 4326, 14.0, 0)
-- Non-imported: (8653, 'concatenated operation', 'ED50 to WGS 84 (14)', None, None, 4230, 4326, 1.5, 0)
-- Non-imported: (8654, 'concatenated operation', 'ED50 to ETRS89 (2)', None, None, 4230, 4258, 2.0, 0)
-- Non-imported: (9710, 'transformation', 'EVRF2019 height to EVRF2019 mean-tide height (1)', 1107, 'zero-tide height to mean-tide height (EVRF2019)', 9389, 9390, 0.0, 0)
-- Non-imported: (10929, 'point motion operation', 'SOPAC deformation model for California v1', 1148, 'Point motion (geocentric domain) using NEU displacement grid (grd)', 10951, 10951, 0.01, 0)
-- Non-imported: (15753, 'transformation', 'ED50 to ED87 (1)', 9651, 'Reversible polynomial of degree 4', 4230, 4231, 1.0, 0)
-- Non-imported: (15950, 'transformation', 'Viti Levu 1912 / Viti Levu Grid to Fiji 1986 / Fiji Map Grid (1)', 9645, 'General polynomial of degree 2', 3140, 3460, 1.0, 0)
-- Non-imported: (15951, 'transformation', 'Vanua Levu 1915 / Vanua Levu Grid to Fiji 1986 / Fiji Map Grid (1)', 9645, 'General polynomial of degree 2', 3139, 3460, 1.0, 0)
