--- This file has been generated by scripts/build_db.py. DO NOT EDIT !

INSERT INTO "engineering_crs" VALUES('EPSG','5800','Astra Minas Grid',NULL,'EPSG','6507','EPSG','9300',0);
INSERT INTO "usage" VALUES('EPSG','4243','engineering_crs','EPSG','5800','EPSG','1265','EPSG','1136');
INSERT INTO "engineering_crs" VALUES('EPSG','5801','Barcelona Grid B1',NULL,'EPSG','4500','EPSG','9301',0);
INSERT INTO "usage" VALUES('EPSG','4244','engineering_crs','EPSG','5801','EPSG','1266','EPSG','1136');
INSERT INTO "engineering_crs" VALUES('EPSG','5802','Barcelona Grid B2',NULL,'EPSG','4500','EPSG','9301',0);
INSERT INTO "usage" VALUES('EPSG','4245','engineering_crs','EPSG','5802','EPSG','1266','EPSG','1136');
INSERT INTO "engineering_crs" VALUES('EPSG','5803','Maturin Grid',NULL,'EPSG','4500','EPSG','9302',0);
INSERT INTO "usage" VALUES('EPSG','4246','engineering_crs','EPSG','5803','EPSG','1320','EPSG','1136');
INSERT INTO "engineering_crs" VALUES('EPSG','5808','Maracaibo Cross Grid M4',NULL,'EPSG','4500','EPSG','9303',0);
INSERT INTO "usage" VALUES('EPSG','4251','engineering_crs','EPSG','5808','EPSG','1319','EPSG','1136');
INSERT INTO "engineering_crs" VALUES('EPSG','5809','Maracaibo Cross Grid M5',NULL,'EPSG','4500','EPSG','9303',0);
INSERT INTO "usage" VALUES('EPSG','4252','engineering_crs','EPSG','5809','EPSG','1319','EPSG','1136');
INSERT INTO "engineering_crs" VALUES('EPSG','5810','La Rosa Grid',NULL,'EPSG','4500','EPSG','9304',0);
INSERT INTO "usage" VALUES('EPSG','4253','engineering_crs','EPSG','5810','EPSG','1311','EPSG','1136');
INSERT INTO "engineering_crs" VALUES('EPSG','5811','Mene Grande',NULL,'EPSG','4500','EPSG','9305',0);
INSERT INTO "usage" VALUES('EPSG','4254','engineering_crs','EPSG','5811','EPSG','1270','EPSG','1136');
INSERT INTO "engineering_crs" VALUES('EPSG','5812','El Cubo',NULL,'EPSG','4500','EPSG','9306',0);
INSERT INTO "usage" VALUES('EPSG','4255','engineering_crs','EPSG','5812','EPSG','1269','EPSG','1136');
INSERT INTO "engineering_crs" VALUES('EPSG','5813','Dabajuro',NULL,'EPSG','4500','EPSG','9307',0);
INSERT INTO "usage" VALUES('EPSG','4256','engineering_crs','EPSG','5813','EPSG','1268','EPSG','1136');
INSERT INTO "engineering_crs" VALUES('EPSG','5814','Tucupita',NULL,'EPSG','4500','EPSG','9308',0);
INSERT INTO "usage" VALUES('EPSG','4257','engineering_crs','EPSG','5814','EPSG','1370','EPSG','1136');
INSERT INTO "engineering_crs" VALUES('EPSG','5815','Santa Maria de Ipire',NULL,'EPSG','4500','EPSG','9310',0);
INSERT INTO "usage" VALUES('EPSG','4258','engineering_crs','EPSG','5815','EPSG','1371','EPSG','1136');
INSERT INTO "engineering_crs" VALUES('EPSG','5816','Barinas west base',NULL,'EPSG','4500','EPSG','9311',0);
INSERT INTO "usage" VALUES('EPSG','4259','engineering_crs','EPSG','5816','EPSG','1267','EPSG','1136');
INSERT INTO "engineering_crs" VALUES('EPSG','5817','Tombak LNG plant',NULL,'EPSG','6510','EPSG','9314',0);
INSERT INTO "usage" VALUES('EPSG','4260','engineering_crs','EPSG','5817','EPSG','2362','EPSG','1029');
INSERT INTO "engineering_crs" VALUES('EPSG','6715','Christmas Island Grid 1985',NULL,'EPSG','4400','EPSG','1134',0);
INSERT INTO "usage" VALUES('EPSG','4925','engineering_crs','EPSG','6715','EPSG','4169','EPSG','1056');
