--- This file has been generated by scripts/build_db.py. DO NOT EDIT !

INSERT INTO "concatenated_operation_step" VALUES('EPSG','3896',1,'EPSG','3895',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','3896',2,'EPSG','1618',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','3966',1,'EPSG','3913',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','3966',2,'EPSG','3962',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','4435',1,'EPSG','1461',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','4435',2,'EPSG','1495',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','4837',1,'EPSG','1672',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','4837',2,'EPSG','1311',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','5190',1,'EPSG','5134',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','5190',2,'EPSG','5189',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','5192',1,'EPSG','5134',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','5192',2,'EPSG','5191',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','5230',1,'EPSG','1884',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','5230',2,'EPSG','4836',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','5240',1,'EPSG','5238',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','5240',2,'EPSG','5227',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','5242',1,'EPSG','1884',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','5242',2,'EPSG','5239',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','5838',1,'EPSG','1756',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','5838',2,'EPSG','1988',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','6714',1,'EPSG','6712',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','6714',2,'EPSG','6713',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','6739',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','6739',2,'EPSG','1496',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','6874',1,'EPSG','1265',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','6874',2,'EPSG','6873',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','7811',1,'EPSG','1763',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','7811',2,'EPSG','15958',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','7965',1,'EPSG','7813',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','7965',2,'EPSG','7964',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','7967',1,'EPSG','7813',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','7967',2,'EPSG','7966',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','7973',1,'EPSG','7813',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','7973',2,'EPSG','7969',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','7974',1,'EPSG','7813',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','7974',2,'EPSG','7970',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','7975',1,'EPSG','7813',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','7975',2,'EPSG','7971',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','7983',1,'EPSG','7812',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','7983',2,'EPSG','7977',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','7986',1,'EPSG','7980',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','7986',2,'EPSG','7812',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','7987',1,'EPSG','7980',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','7987',2,'EPSG','7812',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','7987',3,'EPSG','7813',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8047',1,'EPSG','1147',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8047',2,'EPSG','1146',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8094',1,'EPSG','1763',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8094',2,'EPSG','1193',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8174',1,'EPSG','1755',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8174',2,'EPSG','1125',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8175',1,'EPSG','1262',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8175',2,'EPSG','1169',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8176',1,'EPSG','1265',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8176',2,'EPSG','1227',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8178',1,'EPSG','1759',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8178',2,'EPSG','8452',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8183',1,'EPSG','1273',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8183',2,'EPSG','1149',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8186',1,'EPSG','1763',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8186',2,'EPSG','1276',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8188',1,'EPSG','1763',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8188',2,'EPSG','1277',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8190',1,'EPSG','1278',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8190',2,'EPSG','1150',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8192',1,'EPSG','1279',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8192',2,'EPSG','1150',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8194',1,'EPSG','1280',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8194',2,'EPSG','1150',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8195',1,'EPSG','1437',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8195',2,'EPSG','1149',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8199',1,'EPSG','1274',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8199',2,'EPSG','1283',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8211',1,'EPSG','1266',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8211',2,'EPSG','1294',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8215',1,'EPSG','1297',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8215',2,'EPSG','1302',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8217',1,'EPSG','1298',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8217',2,'EPSG','1302',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8219',1,'EPSG','1299',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8219',2,'EPSG','1302',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8221',1,'EPSG','1300',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8221',2,'EPSG','1302',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8223',1,'EPSG','1301',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8223',2,'EPSG','1302',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8234',1,'EPSG','1309',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8234',2,'EPSG','1149',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8236',1,'EPSG','1310',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8236',2,'EPSG','1149',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8243',1,'EPSG','1312',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8243',2,'EPSG','1188',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8245',1,'EPSG','1313',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8245',2,'EPSG','1188',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8263',1,'EPSG','1757',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8263',2,'EPSG','1306',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8363',1,'EPSG','8361',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8363',2,'EPSG','8362',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8386',1,'EPSG','1454',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8386',2,'EPSG','1188',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8388',1,'EPSG','1455',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8388',2,'EPSG','1188',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8390',1,'EPSG','1456',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8390',2,'EPSG','1188',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8392',1,'EPSG','1457',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8392',2,'EPSG','1188',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8394',1,'EPSG','1451',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8394',2,'EPSG','1188',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8396',1,'EPSG','1458',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8396',2,'EPSG','1150',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8398',1,'EPSG','1459',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8398',2,'EPSG','1150',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8400',1,'EPSG','1460',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8400',2,'EPSG','1150',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8402',1,'EPSG','1461',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8402',2,'EPSG','1188',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8404',1,'EPSG','1462',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8404',2,'EPSG','1188',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8406',1,'EPSG','1463',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8406',2,'EPSG','1188',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8408',1,'EPSG','1464',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8408',2,'EPSG','1150',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8418',1,'EPSG','1472',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8418',2,'EPSG','1473',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8419',1,'EPSG','1599',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8419',2,'EPSG','1473',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8420',1,'EPSG','1600',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8420',2,'EPSG','1473',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8421',1,'EPSG','1601',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8421',2,'EPSG','1473',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8422',1,'EPSG','1602',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8422',2,'EPSG','1473',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8442',1,'EPSG','8365',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8442',2,'EPSG','8364',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8443',1,'EPSG','8364',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8443',2,'EPSG','8367',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8453',1,'EPSG','1506',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8453',2,'EPSG','1150',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8454',1,'EPSG','1507',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8454',2,'EPSG','1150',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8457',1,'EPSG','1509',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8457',2,'EPSG','1511',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8460',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8460',2,'EPSG','1474',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8461',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8461',2,'EPSG','1475',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8462',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8462',2,'EPSG','1476',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8463',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8463',2,'EPSG','1477',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8464',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8464',2,'EPSG','1478',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8465',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8465',2,'EPSG','1479',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8466',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8466',2,'EPSG','1480',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8467',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8467',2,'EPSG','1481',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8468',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8468',2,'EPSG','1482',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8469',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8469',2,'EPSG','1483',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8470',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8470',2,'EPSG','1484',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8471',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8471',2,'EPSG','1485',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8472',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8472',2,'EPSG','1486',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8473',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8473',2,'EPSG','1487',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8474',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8474',2,'EPSG','1488',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8475',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8475',2,'EPSG','1489',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8476',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8476',2,'EPSG','1490',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8477',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8477',2,'EPSG','1491',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8478',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8478',2,'EPSG','1492',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8479',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8479',2,'EPSG','1493',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8480',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8480',2,'EPSG','1494',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8481',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8481',2,'EPSG','1495',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8482',1,'EPSG','1747',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8482',2,'EPSG','1496',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8483',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8483',2,'EPSG','1497',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8484',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8484',2,'EPSG','1498',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8485',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8485',2,'EPSG','1499',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8486',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8486',2,'EPSG','1500',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8487',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8487',2,'EPSG','1501',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8488',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8488',2,'EPSG','1502',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8489',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8489',2,'EPSG','1503',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8496',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8496',2,'EPSG','1515',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8497',1,'EPSG','1243',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8497',2,'EPSG','1515',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8508',1,'EPSG','1454',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8508',2,'EPSG','1520',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8509',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8509',2,'EPSG','1521',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8510',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8510',2,'EPSG','1522',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8511',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8511',2,'EPSG','1523',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8512',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8512',2,'EPSG','1524',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8513',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8513',2,'EPSG','1525',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8514',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8514',2,'EPSG','1526',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8517',1,'EPSG','1528',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8517',2,'EPSG','1527',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8530',1,'EPSG','1539',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8530',2,'EPSG','1540',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8532',1,'EPSG','1541',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8532',2,'EPSG','1240',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8537',1,'EPSG','1545',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8537',2,'EPSG','1237',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8553',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8553',2,'EPSG','1553',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8554',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8554',2,'EPSG','1554',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8560',1,'EPSG','1559',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8560',2,'EPSG','1150',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8562',1,'EPSG','1560',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8562',2,'EPSG','1240',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8563',1,'EPSG','1568',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8563',2,'EPSG','1565',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8564',1,'EPSG','1576',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8564',2,'EPSG','1473',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8565',1,'EPSG','1574',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8565',2,'EPSG','1188',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8566',1,'EPSG','1572',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8566',2,'EPSG','1188',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8568',1,'EPSG','1584',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8568',2,'EPSG','1240',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8569',1,'EPSG','1588',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8569',2,'EPSG','1149',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8571',1,'EPSG','1570',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8571',2,'EPSG','1240',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8572',1,'EPSG','1571',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8572',2,'EPSG','1149',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8573',1,'EPSG','1591',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8573',2,'EPSG','1149',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8574',1,'EPSG','1578',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8574',2,'EPSG','1580',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8575',1,'EPSG','1579',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8575',2,'EPSG','1580',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8576',1,'EPSG','1594',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8576',2,'EPSG','1150',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8577',1,'EPSG','1595',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8577',2,'EPSG','1150',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8578',1,'EPSG','1596',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8578',2,'EPSG','1150',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8579',1,'EPSG','1593',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8579',2,'EPSG','1150',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8580',1,'EPSG','1611',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8580',2,'EPSG','1149',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8581',1,'EPSG','1616',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8581',2,'EPSG','1237',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8582',1,'EPSG','1454',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8582',2,'EPSG','1741',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8583',1,'EPSG','1461',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8583',2,'EPSG','1731',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8584',1,'EPSG','1313',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8584',2,'EPSG','1752',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8585',1,'EPSG','1313',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8585',2,'EPSG','1702',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8586',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8586',2,'EPSG','1704',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8587',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8587',2,'EPSG','1705',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8588',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8588',2,'EPSG','1706',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8589',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8589',2,'EPSG','1707',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8590',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8590',2,'EPSG','1717',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8591',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8591',2,'EPSG','1728',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8592',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8592',2,'EPSG','1708',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8593',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8593',2,'EPSG','1739',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8594',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8594',2,'EPSG','1750',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8595',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8595',2,'EPSG','1712',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8596',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8596',2,'EPSG','1714',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8597',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8597',2,'EPSG','1713',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8598',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8598',2,'EPSG','1748',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8599',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8599',2,'EPSG','1742',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8600',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8600',2,'EPSG','1709',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8601',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8601',2,'EPSG','1743',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8602',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8602',2,'EPSG','1718',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8603',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8603',2,'EPSG','1719',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8604',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8604',2,'EPSG','1721',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8605',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8605',2,'EPSG','1720',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8606',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8606',2,'EPSG','1725',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8607',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8607',2,'EPSG','1722',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8608',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8608',2,'EPSG','1710',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8609',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8609',2,'EPSG','1723',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8610',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8610',2,'EPSG','1711',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8611',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8611',2,'EPSG','1715',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8612',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8612',2,'EPSG','1716',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8613',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8613',2,'EPSG','1724',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8614',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8614',2,'EPSG','1744',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8615',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8615',2,'EPSG','1749',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8616',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8616',2,'EPSG','1726',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8617',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8617',2,'EPSG','1727',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8618',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8618',2,'EPSG','1729',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8619',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8619',2,'EPSG','1745',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8620',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8620',2,'EPSG','1730',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8621',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8621',2,'EPSG','1737',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8622',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8622',2,'EPSG','1732',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8623',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8623',2,'EPSG','1733',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8624',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8624',2,'EPSG','1734',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8625',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8625',2,'EPSG','1735',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8626',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8626',2,'EPSG','1746',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8627',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8627',2,'EPSG','1736',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8628',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8628',2,'EPSG','1747',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8629',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8629',2,'EPSG','1738',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8630',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8630',2,'EPSG','1740',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8631',1,'EPSG','1805',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8631',2,'EPSG','1240',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8632',1,'EPSG','1806',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8632',2,'EPSG','1240',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8633',1,'EPSG','1828',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8633',2,'EPSG','1238',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8634',1,'EPSG','1839',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8634',2,'EPSG','1240',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8635',1,'EPSG','1313',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8635',2,'EPSG','1849',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8636',1,'EPSG','1881',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8636',2,'EPSG','1130',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8637',1,'EPSG','1756',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8637',2,'EPSG','1944',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8638',1,'EPSG','1260',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8638',2,'EPSG','1837',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8639',1,'EPSG','1762',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8639',2,'EPSG','1654',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8640',1,'EPSG','1882',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8640',2,'EPSG','1253',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8641',1,'EPSG','1883',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8641',2,'EPSG','1897',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8642',1,'EPSG','1884',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8642',2,'EPSG','1623',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8643',1,'EPSG','1891',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8643',2,'EPSG','1272',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8644',1,'EPSG','1761',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8644',2,'EPSG','1891',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8644',3,'EPSG','1272',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8645',1,'EPSG','1757',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8645',2,'EPSG','1618',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8646',1,'EPSG','1902',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8646',2,'EPSG','1240',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8647',1,'EPSG','1313',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8647',2,'EPSG','1950',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8647',3,'EPSG','1946',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8648',1,'EPSG','1991',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8648',2,'EPSG','1986',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8649',1,'EPSG','1991',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8649',2,'EPSG','1990',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8650',1,'EPSG','1071',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8650',2,'EPSG','1073',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8651',1,'EPSG','1063',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8651',2,'EPSG','1065',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8652',1,'EPSG','1064',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8652',2,'EPSG','1065',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8655',1,'EPSG','1902',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8655',2,'EPSG','1240',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8656',1,'EPSG','15790',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8656',2,'EPSG','1240',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8657',1,'EPSG','15792',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8657',2,'EPSG','1240',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8659',1,'EPSG','15896',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','8659',2,'EPSG','1158',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','9091',1,'EPSG','9079',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','9091',2,'EPSG','9084',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','9092',1,'EPSG','9080',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','9092',2,'EPSG','9084',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','9093',1,'EPSG','9081',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','9093',2,'EPSG','9084',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','9094',1,'EPSG','9082',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','9094',2,'EPSG','9084',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','9095',1,'EPSG','9082',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','9095',2,'EPSG','9085',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','9096',1,'EPSG','9082',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','9096',2,'EPSG','9086',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','9097',1,'EPSG','9082',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','9097',2,'EPSG','9087',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','9098',1,'EPSG','9082',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','9098',2,'EPSG','9088',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','9099',1,'EPSG','9082',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','9099',2,'EPSG','9089',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','9100',1,'EPSG','9083',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','9100',2,'EPSG','9088',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','9101',1,'EPSG','9083',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','9101',2,'EPSG','9089',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','9102',1,'EPSG','9083',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','9102',2,'EPSG','9090',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','9103',1,'EPSG','1241',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','9103',2,'EPSG','8971',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','9103',3,'EPSG','7807',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','9103',4,'EPSG','7790',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','9104',1,'EPSG','8555',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','9104',2,'EPSG','8556',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','9104',3,'EPSG','8861',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','9104',4,'EPSG','8862',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','9104',5,'EPSG','8559',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','9104',6,'EPSG','7807',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','9104',7,'EPSG','7790',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','9336',1,'EPSG','1313',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','9336',2,'EPSG','9244',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','9337',1,'EPSG','1763',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','9337',2,'EPSG','9327',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','9499',1,'EPSG','9276',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','9499',2,'EPSG','9275',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','9683',1,'EPSG','8049',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','9683',2,'EPSG','8447',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','9685',1,'EPSG','9459',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','9685',2,'EPSG','8447',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','9687',1,'EPSG','8447',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','9687',2,'EPSG','8448',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','9731',1,'EPSG','9729',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','9731',2,'EPSG','9726',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','9750',1,'EPSG','9727',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','9750',2,'EPSG','9726',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10146',1,'EPSG','9629',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10146',2,'EPSG','10145',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10392',1,'EPSG','7812',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10392',2,'EPSG','10380',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10393',1,'EPSG','7812',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10393',2,'EPSG','10381',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10394',1,'EPSG','7812',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10394',2,'EPSG','10382',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10395',1,'EPSG','7812',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10395',2,'EPSG','10383',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10396',1,'EPSG','7812',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10396',2,'EPSG','10384',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10397',1,'EPSG','7812',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10397',2,'EPSG','10385',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10398',1,'EPSG','7812',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10398',2,'EPSG','10386',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10399',1,'EPSG','7812',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10399',2,'EPSG','10387',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10400',1,'EPSG','7812',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10400',2,'EPSG','10388',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10409',1,'EPSG','7812',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10409',2,'EPSG','10389',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10410',1,'EPSG','7812',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10410',2,'EPSG','10390',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10411',1,'EPSG','7812',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10411',2,'EPSG','10391',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10495',1,'EPSG','10490',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10495',2,'EPSG','10492',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10496',1,'EPSG','10492',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10496',2,'EPSG','10494',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10616',1,'EPSG','9629',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10616',2,'EPSG','10145',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10675',1,'EPSG','10646',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10675',2,'EPSG','10657',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10754',1,'EPSG','10750',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10754',2,'EPSG','10752',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10755',1,'EPSG','10646',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10755',2,'EPSG','10658',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10756',1,'EPSG','10750',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10756',2,'EPSG','10753',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10778',1,'EPSG','18193',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10778',2,'EPSG','10703',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10778',3,'EPSG','16065',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10815',1,'EPSG','8880',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10815',2,'EPSG','10809',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10816',1,'EPSG','8880',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10816',2,'EPSG','10809',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10816',3,'EPSG','10810',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10817',1,'EPSG','8880',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10817',2,'EPSG','10809',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10817',3,'EPSG','10811',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10818',1,'EPSG','8880',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10818',2,'EPSG','10809',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10818',3,'EPSG','10812',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10824',1,'EPSG','8880',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10824',2,'EPSG','10809',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10824',3,'EPSG','10813',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10825',1,'EPSG','8880',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10825',2,'EPSG','10809',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10825',3,'EPSG','10814',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10868',1,'EPSG','10587',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10868',2,'EPSG','10809',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10868',3,'EPSG','10814',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10869',1,'EPSG','10587',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10869',2,'EPSG','10809',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10869',3,'EPSG','10810',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10870',1,'EPSG','10587',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10870',2,'EPSG','10809',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10870',3,'EPSG','10811',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10871',1,'EPSG','10587',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10871',2,'EPSG','10809',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10871',3,'EPSG','10812',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10872',1,'EPSG','10587',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10872',2,'EPSG','10809',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10872',3,'EPSG','10813',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10894',1,'EPSG','8880',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10894',2,'EPSG','10809',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10894',3,'EPSG','10893',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10895',1,'EPSG','10587',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10895',2,'EPSG','10809',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','10895',3,'EPSG','10893',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','11005',1,'EPSG','10107',NULL);
INSERT INTO "concatenated_operation_step" VALUES('EPSG','11005',2,'EPSG','11004',NULL);
