--- This file has been generated by scripts/build_db.py. DO NOT EDIT !

INSERT INTO "engineering_datum" VALUES('EPSG','1134','Christmas Island Datum 1985','1985-01-01',NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13186','engineering_datum','EPSG','1134','EPSG','4169','EPSG','1056');
INSERT INTO "engineering_datum" VALUES('EPSG','9300','Astra Minas',NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13850','engineering_datum','EPSG','9300','EPSG','1265','EPSG','1216');
INSERT INTO "engineering_datum" VALUES('EPSG','9301','Barcelona',NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13851','engineering_datum','EPSG','9301','EPSG','1266','EPSG','1216');
INSERT INTO "engineering_datum" VALUES('EPSG','9302','Maturin',NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13852','engineering_datum','EPSG','9302','EPSG','1320','EPSG','1216');
INSERT INTO "engineering_datum" VALUES('EPSG','9303','Maracaibo Cross',NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13853','engineering_datum','EPSG','9303','EPSG','1319','EPSG','1216');
INSERT INTO "engineering_datum" VALUES('EPSG','9304','La Rosa',NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13854','engineering_datum','EPSG','9304','EPSG','1311','EPSG','1216');
INSERT INTO "engineering_datum" VALUES('EPSG','9305','Mene Grande',NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13855','engineering_datum','EPSG','9305','EPSG','1270','EPSG','1216');
INSERT INTO "engineering_datum" VALUES('EPSG','9306','El Cubo',NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13856','engineering_datum','EPSG','9306','EPSG','1269','EPSG','1216');
INSERT INTO "engineering_datum" VALUES('EPSG','9307','Dabajuro',NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13857','engineering_datum','EPSG','9307','EPSG','1268','EPSG','1216');
INSERT INTO "engineering_datum" VALUES('EPSG','9308','Tucupita',NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13858','engineering_datum','EPSG','9308','EPSG','1370','EPSG','1216');
INSERT INTO "engineering_datum" VALUES('EPSG','9309','El Mene',NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13859','engineering_datum','EPSG','9309','EPSG','1270','EPSG','1216');
INSERT INTO "engineering_datum" VALUES('EPSG','9310','Santa Maria de Ipire',NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13860','engineering_datum','EPSG','9310','EPSG','1371','EPSG','1216');
INSERT INTO "engineering_datum" VALUES('EPSG','9311','Barinas west base',NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13861','engineering_datum','EPSG','9311','EPSG','1267','EPSG','1216');
INSERT INTO "engineering_datum" VALUES('EPSG','9314','Tombak LNG plant',NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13864','engineering_datum','EPSG','9314','EPSG','2362','EPSG','1029');
