--- This file has been generated by scripts/build_nadcon5_concatenated_operations.py. DO NOT EDIT !

-- Concatenated accuracy is sqrt(sum of squared accuracies) (confirmed by NGS to be a valid way)

INSERT INTO concatenated_operation VALUES('PROJ','NAD27_TO_NAD83_HARN_CONUS','NAD27 to NAD83(HARN) (NADCON5, CONUS)','Transformation based on concatenation of NADCON5 transformations','EPSG','4267','EPSG','4152',0.16,NULL,0);
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD27_TO_NAD83_HARN_CONUS',1,'EPSG','8555','forward'); -- NAD27 to NAD83 (EPSG:8555), 0.15 m
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD27_TO_NAD83_HARN_CONUS',2,'EPSG','8556','forward'); -- NAD83 to NAD83(HARN) (EPSG:8556), 0.05 m
INSERT INTO usage VALUES('PROJ','NAD27_TO_NAD83_HARN_CONUS_USAGE','concatenated_operation','PROJ','NAD27_TO_NAD83_HARN_CONUS',
    'EPSG','4516', -- extent: CONUS
    'EPSG','1027' -- scope: Geodesy
);

INSERT INTO concatenated_operation VALUES('PROJ','NAD27_TO_NAD83_FBN_CONUS','NAD27 to NAD83(FBN) (NADCON5, CONUS)','Transformation based on concatenation of NADCON5 transformations','EPSG','4267','EPSG','8860',0.17,NULL,0);
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD27_TO_NAD83_FBN_CONUS',1,'EPSG','8555','forward'); -- NAD27 to NAD83 (EPSG:8555), 0.15 m
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD27_TO_NAD83_FBN_CONUS',2,'EPSG','8556','forward'); -- NAD83 to NAD83(HARN) (EPSG:8556), 0.05 m
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD27_TO_NAD83_FBN_CONUS',3,'EPSG','8861','forward'); -- NAD83(HARN) to NAD83(FBN) (EPSG:8861), 0.05 m
INSERT INTO usage VALUES('PROJ','NAD27_TO_NAD83_FBN_CONUS_USAGE','concatenated_operation','PROJ','NAD27_TO_NAD83_FBN_CONUS',
    'EPSG','4516', -- extent: CONUS
    'EPSG','1027' -- scope: Geodesy
);

INSERT INTO concatenated_operation VALUES('PROJ','NAD27_TO_NAD83_NSRS2007_CONUS','NAD27 to NAD83(NSRS2007) (NADCON5, CONUS)','Transformation based on concatenation of NADCON5 transformations','EPSG','4267','EPSG','4759',0.17,NULL,0);
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD27_TO_NAD83_NSRS2007_CONUS',1,'EPSG','8555','forward'); -- NAD27 to NAD83 (EPSG:8555), 0.15 m
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD27_TO_NAD83_NSRS2007_CONUS',2,'EPSG','8556','forward'); -- NAD83 to NAD83(HARN) (EPSG:8556), 0.05 m
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD27_TO_NAD83_NSRS2007_CONUS',3,'EPSG','8861','forward'); -- NAD83(HARN) to NAD83(FBN) (EPSG:8861), 0.05 m
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD27_TO_NAD83_NSRS2007_CONUS',4,'EPSG','8862','forward'); -- NAD83(FBN) to NAD83(NSRS2007) (EPSG:8862), 0.05 m
INSERT INTO usage VALUES('PROJ','NAD27_TO_NAD83_NSRS2007_CONUS_USAGE','concatenated_operation','PROJ','NAD27_TO_NAD83_NSRS2007_CONUS',
    'EPSG','4516', -- extent: CONUS
    'EPSG','1027' -- scope: Geodesy
);

INSERT INTO concatenated_operation VALUES('PROJ','NAD27_TO_NAD83_2011_CONUS','NAD27 to NAD83(2011) (NADCON5, CONUS)','Transformation based on concatenation of NADCON5 transformations','EPSG','4267','EPSG','6318',0.18,NULL,0);
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD27_TO_NAD83_2011_CONUS',1,'EPSG','8555','forward'); -- NAD27 to NAD83 (EPSG:8555), 0.15 m
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD27_TO_NAD83_2011_CONUS',2,'EPSG','8556','forward'); -- NAD83 to NAD83(HARN) (EPSG:8556), 0.05 m
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD27_TO_NAD83_2011_CONUS',3,'EPSG','8861','forward'); -- NAD83(HARN) to NAD83(FBN) (EPSG:8861), 0.05 m
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD27_TO_NAD83_2011_CONUS',4,'EPSG','8862','forward'); -- NAD83(FBN) to NAD83(NSRS2007) (EPSG:8862), 0.05 m
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD27_TO_NAD83_2011_CONUS',5,'EPSG','8559','forward'); -- NAD83(NSRS2007) to NAD83(2011) (EPSG:8559), 0.05 m
INSERT INTO usage VALUES('PROJ','NAD27_TO_NAD83_2011_CONUS_USAGE','concatenated_operation','PROJ','NAD27_TO_NAD83_2011_CONUS',
    'EPSG','4516', -- extent: CONUS
    'EPSG','1027' -- scope: Geodesy
);

INSERT INTO concatenated_operation VALUES('PROJ','NAD83_TO_NAD83_FBN_CONUS','NAD83 to NAD83(FBN) (NADCON5, CONUS)','Transformation based on concatenation of NADCON5 transformations','EPSG','4269','EPSG','8860',0.07,NULL,0);
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD83_TO_NAD83_FBN_CONUS',1,'EPSG','8556','forward'); -- NAD83 to NAD83(HARN) (EPSG:8556), 0.05 m
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD83_TO_NAD83_FBN_CONUS',2,'EPSG','8861','forward'); -- NAD83(HARN) to NAD83(FBN) (EPSG:8861), 0.05 m
INSERT INTO usage VALUES('PROJ','NAD83_TO_NAD83_FBN_CONUS_USAGE','concatenated_operation','PROJ','NAD83_TO_NAD83_FBN_CONUS',
    'EPSG','4516', -- extent: CONUS
    'EPSG','1027' -- scope: Geodesy
);

INSERT INTO concatenated_operation VALUES('PROJ','NAD83_TO_NAD83_NSRS2007_CONUS','NAD83 to NAD83(NSRS2007) (NADCON5, CONUS)','Transformation based on concatenation of NADCON5 transformations','EPSG','4269','EPSG','4759',0.09,NULL,0);
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD83_TO_NAD83_NSRS2007_CONUS',1,'EPSG','8556','forward'); -- NAD83 to NAD83(HARN) (EPSG:8556), 0.05 m
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD83_TO_NAD83_NSRS2007_CONUS',2,'EPSG','8861','forward'); -- NAD83(HARN) to NAD83(FBN) (EPSG:8861), 0.05 m
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD83_TO_NAD83_NSRS2007_CONUS',3,'EPSG','8862','forward'); -- NAD83(FBN) to NAD83(NSRS2007) (EPSG:8862), 0.05 m
INSERT INTO usage VALUES('PROJ','NAD83_TO_NAD83_NSRS2007_CONUS_USAGE','concatenated_operation','PROJ','NAD83_TO_NAD83_NSRS2007_CONUS',
    'EPSG','4516', -- extent: CONUS
    'EPSG','1027' -- scope: Geodesy
);

INSERT INTO concatenated_operation VALUES('PROJ','NAD83_TO_NAD83_2011_CONUS','NAD83 to NAD83(2011) (NADCON5, CONUS)','Transformation based on concatenation of NADCON5 transformations','EPSG','4269','EPSG','6318',0.1,NULL,0);
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD83_TO_NAD83_2011_CONUS',1,'EPSG','8556','forward'); -- NAD83 to NAD83(HARN) (EPSG:8556), 0.05 m
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD83_TO_NAD83_2011_CONUS',2,'EPSG','8861','forward'); -- NAD83(HARN) to NAD83(FBN) (EPSG:8861), 0.05 m
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD83_TO_NAD83_2011_CONUS',3,'EPSG','8862','forward'); -- NAD83(FBN) to NAD83(NSRS2007) (EPSG:8862), 0.05 m
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD83_TO_NAD83_2011_CONUS',4,'EPSG','8559','forward'); -- NAD83(NSRS2007) to NAD83(2011) (EPSG:8559), 0.05 m
INSERT INTO usage VALUES('PROJ','NAD83_TO_NAD83_2011_CONUS_USAGE','concatenated_operation','PROJ','NAD83_TO_NAD83_2011_CONUS',
    'EPSG','4516', -- extent: CONUS
    'EPSG','1027' -- scope: Geodesy
);

INSERT INTO concatenated_operation VALUES('PROJ','NAD83_HARN_TO_NAD83_NSRS2007_CONUS','NAD83(HARN) to NAD83(NSRS2007) (NADCON5, CONUS)','Transformation based on concatenation of NADCON5 transformations','EPSG','4152','EPSG','4759',0.07,NULL,0);
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD83_HARN_TO_NAD83_NSRS2007_CONUS',1,'EPSG','8861','forward'); -- NAD83(HARN) to NAD83(FBN) (EPSG:8861), 0.05 m
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD83_HARN_TO_NAD83_NSRS2007_CONUS',2,'EPSG','8862','forward'); -- NAD83(FBN) to NAD83(NSRS2007) (EPSG:8862), 0.05 m
INSERT INTO usage VALUES('PROJ','NAD83_HARN_TO_NAD83_NSRS2007_CONUS_USAGE','concatenated_operation','PROJ','NAD83_HARN_TO_NAD83_NSRS2007_CONUS',
    'EPSG','4516', -- extent: CONUS
    'EPSG','1027' -- scope: Geodesy
);

INSERT INTO concatenated_operation VALUES('PROJ','NAD83_HARN_TO_NAD83_2011_CONUS','NAD83(HARN) to NAD83(2011) (NADCON5, CONUS)','Transformation based on concatenation of NADCON5 transformations','EPSG','4152','EPSG','6318',0.09,NULL,0);
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD83_HARN_TO_NAD83_2011_CONUS',1,'EPSG','8861','forward'); -- NAD83(HARN) to NAD83(FBN) (EPSG:8861), 0.05 m
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD83_HARN_TO_NAD83_2011_CONUS',2,'EPSG','8862','forward'); -- NAD83(FBN) to NAD83(NSRS2007) (EPSG:8862), 0.05 m
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD83_HARN_TO_NAD83_2011_CONUS',3,'EPSG','8559','forward'); -- NAD83(NSRS2007) to NAD83(2011) (EPSG:8559), 0.05 m
INSERT INTO usage VALUES('PROJ','NAD83_HARN_TO_NAD83_2011_CONUS_USAGE','concatenated_operation','PROJ','NAD83_HARN_TO_NAD83_2011_CONUS',
    'EPSG','4516', -- extent: CONUS
    'EPSG','1027' -- scope: Geodesy
);

INSERT INTO concatenated_operation VALUES('PROJ','NAD83_FBN_TO_NAD83_2011_CONUS','NAD83(FBN) to NAD83(2011) (NADCON5, CONUS)','Transformation based on concatenation of NADCON5 transformations','EPSG','8860','EPSG','6318',0.07,NULL,0);
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD83_FBN_TO_NAD83_2011_CONUS',1,'EPSG','8862','forward'); -- NAD83(FBN) to NAD83(NSRS2007) (EPSG:8862), 0.05 m
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD83_FBN_TO_NAD83_2011_CONUS',2,'EPSG','8559','forward'); -- NAD83(NSRS2007) to NAD83(2011) (EPSG:8559), 0.05 m
INSERT INTO usage VALUES('PROJ','NAD83_FBN_TO_NAD83_2011_CONUS_USAGE','concatenated_operation','PROJ','NAD83_FBN_TO_NAD83_2011_CONUS',
    'EPSG','4516', -- extent: CONUS
    'EPSG','1027' -- scope: Geodesy
);

INSERT INTO concatenated_operation VALUES('PROJ','NAD27_TO_NAD83_HARN_ALASKA','NAD27 to NAD83(HARN) (NADCON5, Alaska)','Transformation based on concatenation of NADCON5 transformations','EPSG','4267','EPSG','4152',0.52,NULL,0);
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD27_TO_NAD83_HARN_ALASKA',1,'EPSG','8549','forward'); -- NAD27 to NAD83 (EPSG:8549), 0.5 m
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD27_TO_NAD83_HARN_ALASKA',2,'EPSG','8550','forward'); -- NAD83 to NAD83(HARN) (EPSG:8550), 0.15 m
INSERT INTO usage VALUES('PROJ','NAD27_TO_NAD83_HARN_ALASKA_USAGE','concatenated_operation','PROJ','NAD27_TO_NAD83_HARN_ALASKA',
    'EPSG','1330', -- extent: Alaska
    'EPSG','1027' -- scope: Geodesy
);

INSERT INTO concatenated_operation VALUES('PROJ','NAD27_TO_NAD83_NSRS2007_ALASKA','NAD27 to NAD83(NSRS2007) (NADCON5, Alaska)','Transformation based on concatenation of NADCON5 transformations','EPSG','4267','EPSG','4759',0.52,NULL,0);
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD27_TO_NAD83_NSRS2007_ALASKA',1,'EPSG','8549','forward'); -- NAD27 to NAD83 (EPSG:8549), 0.5 m
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD27_TO_NAD83_NSRS2007_ALASKA',2,'EPSG','8550','forward'); -- NAD83 to NAD83(HARN) (EPSG:8550), 0.15 m
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD27_TO_NAD83_NSRS2007_ALASKA',3,'EPSG','8551','forward'); -- NAD83(HARN) to NAD83(NSRS2007) (EPSG:8551), 0.05 m
INSERT INTO usage VALUES('PROJ','NAD27_TO_NAD83_NSRS2007_ALASKA_USAGE','concatenated_operation','PROJ','NAD27_TO_NAD83_NSRS2007_ALASKA',
    'EPSG','1330', -- extent: Alaska
    'EPSG','1027' -- scope: Geodesy
);

INSERT INTO concatenated_operation VALUES('PROJ','NAD27_TO_NAD83_2011_ALASKA','NAD27 to NAD83(2011) (NADCON5, Alaska)','Transformation based on concatenation of NADCON5 transformations','EPSG','4267','EPSG','6318',0.53,NULL,0);
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD27_TO_NAD83_2011_ALASKA',1,'EPSG','8549','forward'); -- NAD27 to NAD83 (EPSG:8549), 0.5 m
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD27_TO_NAD83_2011_ALASKA',2,'EPSG','8550','forward'); -- NAD83 to NAD83(HARN) (EPSG:8550), 0.15 m
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD27_TO_NAD83_2011_ALASKA',3,'EPSG','8551','forward'); -- NAD83(HARN) to NAD83(NSRS2007) (EPSG:8551), 0.05 m
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD27_TO_NAD83_2011_ALASKA',4,'EPSG','8552','forward'); -- NAD83(NSRS2007) to NAD83(2011) (EPSG:8552), 0.05 m
INSERT INTO usage VALUES('PROJ','NAD27_TO_NAD83_2011_ALASKA_USAGE','concatenated_operation','PROJ','NAD27_TO_NAD83_2011_ALASKA',
    'EPSG','1330', -- extent: Alaska
    'EPSG','1027' -- scope: Geodesy
);

INSERT INTO concatenated_operation VALUES('PROJ','NAD83_TO_NAD83_NSRS2007_ALASKA','NAD83 to NAD83(NSRS2007) (NADCON5, Alaska)','Transformation based on concatenation of NADCON5 transformations','EPSG','4269','EPSG','4759',0.16,NULL,0);
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD83_TO_NAD83_NSRS2007_ALASKA',1,'EPSG','8550','forward'); -- NAD83 to NAD83(HARN) (EPSG:8550), 0.15 m
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD83_TO_NAD83_NSRS2007_ALASKA',2,'EPSG','8551','forward'); -- NAD83(HARN) to NAD83(NSRS2007) (EPSG:8551), 0.05 m
INSERT INTO usage VALUES('PROJ','NAD83_TO_NAD83_NSRS2007_ALASKA_USAGE','concatenated_operation','PROJ','NAD83_TO_NAD83_NSRS2007_ALASKA',
    'EPSG','1330', -- extent: Alaska
    'EPSG','1027' -- scope: Geodesy
);

INSERT INTO concatenated_operation VALUES('PROJ','NAD83_TO_NAD83_2011_ALASKA','NAD83 to NAD83(2011) (NADCON5, Alaska)','Transformation based on concatenation of NADCON5 transformations','EPSG','4269','EPSG','6318',0.17,NULL,0);
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD83_TO_NAD83_2011_ALASKA',1,'EPSG','8550','forward'); -- NAD83 to NAD83(HARN) (EPSG:8550), 0.15 m
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD83_TO_NAD83_2011_ALASKA',2,'EPSG','8551','forward'); -- NAD83(HARN) to NAD83(NSRS2007) (EPSG:8551), 0.05 m
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD83_TO_NAD83_2011_ALASKA',3,'EPSG','8552','forward'); -- NAD83(NSRS2007) to NAD83(2011) (EPSG:8552), 0.05 m
INSERT INTO usage VALUES('PROJ','NAD83_TO_NAD83_2011_ALASKA_USAGE','concatenated_operation','PROJ','NAD83_TO_NAD83_2011_ALASKA',
    'EPSG','1330', -- extent: Alaska
    'EPSG','1027' -- scope: Geodesy
);

INSERT INTO concatenated_operation VALUES('PROJ','NAD83_HARN_TO_NAD83_2011_ALASKA','NAD83(HARN) to NAD83(2011) (NADCON5, Alaska)','Transformation based on concatenation of NADCON5 transformations','EPSG','4152','EPSG','6318',0.07,NULL,0);
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD83_HARN_TO_NAD83_2011_ALASKA',1,'EPSG','8551','forward'); -- NAD83(HARN) to NAD83(NSRS2007) (EPSG:8551), 0.05 m
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD83_HARN_TO_NAD83_2011_ALASKA',2,'EPSG','8552','forward'); -- NAD83(NSRS2007) to NAD83(2011) (EPSG:8552), 0.05 m
INSERT INTO usage VALUES('PROJ','NAD83_HARN_TO_NAD83_2011_ALASKA_USAGE','concatenated_operation','PROJ','NAD83_HARN_TO_NAD83_2011_ALASKA',
    'EPSG','1330', -- extent: Alaska
    'EPSG','1027' -- scope: Geodesy
);

INSERT INTO concatenated_operation VALUES('PROJ','OLD_HAWAIIAN_TO_NAD83_HARN_HAWAII','Old Hawaiian to NAD83(HARN) (NADCON5, Hawaii)','Transformation based on concatenation of NADCON5 transformations','EPSG','4135','EPSG','4152',0.21,NULL,0);
INSERT INTO concatenated_operation_step VALUES('PROJ','OLD_HAWAIIAN_TO_NAD83_HARN_HAWAII',1,'EPSG','8561','forward'); -- Old Hawaiian to NAD83 (EPSG:8561), 0.2 m
INSERT INTO concatenated_operation_step VALUES('PROJ','OLD_HAWAIIAN_TO_NAD83_HARN_HAWAII',2,'EPSG','8660','forward'); -- NAD83 to NAD83(HARN) (EPSG:8660), 0.05 m
INSERT INTO usage VALUES('PROJ','OLD_HAWAIIAN_TO_NAD83_HARN_HAWAII_USAGE','concatenated_operation','PROJ','OLD_HAWAIIAN_TO_NAD83_HARN_HAWAII',
    'EPSG','1334', -- extent: Hawaii
    'EPSG','1027' -- scope: Geodesy
);

INSERT INTO concatenated_operation VALUES('PROJ','OLD_HAWAIIAN_TO_NAD83_PA11_HAWAII','Old Hawaiian to NAD83(PA11) (NADCON5, Hawaii)','Transformation based on concatenation of NADCON5 transformations','EPSG','4135','EPSG','6322',0.21,NULL,0);
INSERT INTO concatenated_operation_step VALUES('PROJ','OLD_HAWAIIAN_TO_NAD83_PA11_HAWAII',1,'EPSG','8561','forward'); -- Old Hawaiian to NAD83 (EPSG:8561), 0.2 m
INSERT INTO concatenated_operation_step VALUES('PROJ','OLD_HAWAIIAN_TO_NAD83_PA11_HAWAII',2,'EPSG','8660','forward'); -- NAD83 to NAD83(HARN) (EPSG:8660), 0.05 m
INSERT INTO concatenated_operation_step VALUES('PROJ','OLD_HAWAIIAN_TO_NAD83_PA11_HAWAII',3,'EPSG','8661','forward'); -- NAD83(HARN) to NAD83(PA11) (EPSG:8661), 0.05 m
INSERT INTO usage VALUES('PROJ','OLD_HAWAIIAN_TO_NAD83_PA11_HAWAII_USAGE','concatenated_operation','PROJ','OLD_HAWAIIAN_TO_NAD83_PA11_HAWAII',
    'EPSG','1334', -- extent: Hawaii
    'EPSG','1027' -- scope: Geodesy
);

INSERT INTO concatenated_operation VALUES('PROJ','NAD83_TO_NAD83_PA11_HAWAII','NAD83 to NAD83(PA11) (NADCON5, Hawaii)','Transformation based on concatenation of NADCON5 transformations','EPSG','4269','EPSG','6322',0.07,NULL,0);
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD83_TO_NAD83_PA11_HAWAII',1,'EPSG','8660','forward'); -- NAD83 to NAD83(HARN) (EPSG:8660), 0.05 m
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD83_TO_NAD83_PA11_HAWAII',2,'EPSG','8661','forward'); -- NAD83(HARN) to NAD83(PA11) (EPSG:8661), 0.05 m
INSERT INTO usage VALUES('PROJ','NAD83_TO_NAD83_PA11_HAWAII_USAGE','concatenated_operation','PROJ','NAD83_TO_NAD83_PA11_HAWAII',
    'EPSG','1334', -- extent: Hawaii
    'EPSG','1027' -- scope: Geodesy
);

INSERT INTO concatenated_operation VALUES('PROJ','PUERTO_RICO_TO_NAD83_HARN_PRVI','Puerto Rico to NAD83(HARN) (NADCON5, PRVI)','Transformation based on concatenation of NADCON5 transformations','EPSG','4139','EPSG','4152',0.21,NULL,0);
INSERT INTO concatenated_operation_step VALUES('PROJ','PUERTO_RICO_TO_NAD83_HARN_PRVI',1,'EPSG','8668','forward'); -- Puerto Rico to NAD83 (EPSG:8668), 0.15 m
INSERT INTO concatenated_operation_step VALUES('PROJ','PUERTO_RICO_TO_NAD83_HARN_PRVI',2,'EPSG','8669','forward'); -- NAD83 to NAD83(HARN) (EPSG:8669), 0.15 m
INSERT INTO usage VALUES('PROJ','PUERTO_RICO_TO_NAD83_HARN_PRVI_USAGE','concatenated_operation','PROJ','PUERTO_RICO_TO_NAD83_HARN_PRVI',
    'EPSG','3634', -- extent: PRVI
    'EPSG','1027' -- scope: Geodesy
);

INSERT INTO concatenated_operation VALUES('PROJ','PUERTO_RICO_TO_NAD83_HARN_CORRECTED_PRVI','Puerto Rico to NAD83(HARN Corrected) (NADCON5, PRVI)','Transformation based on concatenation of NADCON5 transformations','EPSG','4139','EPSG','8545',0.22,NULL,0);
INSERT INTO concatenated_operation_step VALUES('PROJ','PUERTO_RICO_TO_NAD83_HARN_CORRECTED_PRVI',1,'EPSG','8668','forward'); -- Puerto Rico to NAD83 (EPSG:8668), 0.15 m
INSERT INTO concatenated_operation_step VALUES('PROJ','PUERTO_RICO_TO_NAD83_HARN_CORRECTED_PRVI',2,'EPSG','8669','forward'); -- NAD83 to NAD83(HARN) (EPSG:8669), 0.15 m
INSERT INTO concatenated_operation_step VALUES('PROJ','PUERTO_RICO_TO_NAD83_HARN_CORRECTED_PRVI',3,'EPSG','9181','forward'); -- NAD83(HARN) to NAD83(HARN Corrected) (EPSG:9181), 0.05 m
INSERT INTO usage VALUES('PROJ','PUERTO_RICO_TO_NAD83_HARN_CORRECTED_PRVI_USAGE','concatenated_operation','PROJ','PUERTO_RICO_TO_NAD83_HARN_CORRECTED_PRVI',
    'EPSG','3634', -- extent: PRVI
    'EPSG','1027' -- scope: Geodesy
);

INSERT INTO concatenated_operation VALUES('PROJ','PUERTO_RICO_TO_NAD83_FBN_PRVI','Puerto Rico to NAD83(FBN) (NADCON5, PRVI)','Transformation based on concatenation of NADCON5 transformations','EPSG','4139','EPSG','8860',0.22,NULL,0);
INSERT INTO concatenated_operation_step VALUES('PROJ','PUERTO_RICO_TO_NAD83_FBN_PRVI',1,'EPSG','8668','forward'); -- Puerto Rico to NAD83 (EPSG:8668), 0.15 m
INSERT INTO concatenated_operation_step VALUES('PROJ','PUERTO_RICO_TO_NAD83_FBN_PRVI',2,'EPSG','8669','forward'); -- NAD83 to NAD83(HARN) (EPSG:8669), 0.15 m
INSERT INTO concatenated_operation_step VALUES('PROJ','PUERTO_RICO_TO_NAD83_FBN_PRVI',3,'EPSG','9181','forward'); -- NAD83(HARN) to NAD83(HARN Corrected) (EPSG:9181), 0.05 m
INSERT INTO concatenated_operation_step VALUES('PROJ','PUERTO_RICO_TO_NAD83_FBN_PRVI',4,'EPSG','8867','forward'); -- NAD83(HARN Corrected) to NAD83(FBN) (EPSG:8867), 0.05 m
INSERT INTO usage VALUES('PROJ','PUERTO_RICO_TO_NAD83_FBN_PRVI_USAGE','concatenated_operation','PROJ','PUERTO_RICO_TO_NAD83_FBN_PRVI',
    'EPSG','3634', -- extent: PRVI
    'EPSG','1027' -- scope: Geodesy
);

INSERT INTO concatenated_operation VALUES('PROJ','PUERTO_RICO_TO_NAD83_NSRS2007_PRVI','Puerto Rico to NAD83(NSRS2007) (NADCON5, PRVI)','Transformation based on concatenation of NADCON5 transformations','EPSG','4139','EPSG','4759',0.23,NULL,0);
INSERT INTO concatenated_operation_step VALUES('PROJ','PUERTO_RICO_TO_NAD83_NSRS2007_PRVI',1,'EPSG','8668','forward'); -- Puerto Rico to NAD83 (EPSG:8668), 0.15 m
INSERT INTO concatenated_operation_step VALUES('PROJ','PUERTO_RICO_TO_NAD83_NSRS2007_PRVI',2,'EPSG','8669','forward'); -- NAD83 to NAD83(HARN) (EPSG:8669), 0.15 m
INSERT INTO concatenated_operation_step VALUES('PROJ','PUERTO_RICO_TO_NAD83_NSRS2007_PRVI',3,'EPSG','9181','forward'); -- NAD83(HARN) to NAD83(HARN Corrected) (EPSG:9181), 0.05 m
INSERT INTO concatenated_operation_step VALUES('PROJ','PUERTO_RICO_TO_NAD83_NSRS2007_PRVI',4,'EPSG','8867','forward'); -- NAD83(HARN Corrected) to NAD83(FBN) (EPSG:8867), 0.05 m
INSERT INTO concatenated_operation_step VALUES('PROJ','PUERTO_RICO_TO_NAD83_NSRS2007_PRVI',5,'EPSG','8868','forward'); -- NAD83(FBN) to NAD83(NSRS2007) (EPSG:8868), 0.05 m
INSERT INTO usage VALUES('PROJ','PUERTO_RICO_TO_NAD83_NSRS2007_PRVI_USAGE','concatenated_operation','PROJ','PUERTO_RICO_TO_NAD83_NSRS2007_PRVI',
    'EPSG','3634', -- extent: PRVI
    'EPSG','1027' -- scope: Geodesy
);

INSERT INTO concatenated_operation VALUES('PROJ','PUERTO_RICO_TO_NAD83_2011_PRVI','Puerto Rico to NAD83(2011) (NADCON5, PRVI)','Transformation based on concatenation of NADCON5 transformations','EPSG','4139','EPSG','6318',0.23,NULL,0);
INSERT INTO concatenated_operation_step VALUES('PROJ','PUERTO_RICO_TO_NAD83_2011_PRVI',1,'EPSG','8668','forward'); -- Puerto Rico to NAD83 (EPSG:8668), 0.15 m
INSERT INTO concatenated_operation_step VALUES('PROJ','PUERTO_RICO_TO_NAD83_2011_PRVI',2,'EPSG','8669','forward'); -- NAD83 to NAD83(HARN) (EPSG:8669), 0.15 m
INSERT INTO concatenated_operation_step VALUES('PROJ','PUERTO_RICO_TO_NAD83_2011_PRVI',3,'EPSG','9181','forward'); -- NAD83(HARN) to NAD83(HARN Corrected) (EPSG:9181), 0.05 m
INSERT INTO concatenated_operation_step VALUES('PROJ','PUERTO_RICO_TO_NAD83_2011_PRVI',4,'EPSG','8867','forward'); -- NAD83(HARN Corrected) to NAD83(FBN) (EPSG:8867), 0.05 m
INSERT INTO concatenated_operation_step VALUES('PROJ','PUERTO_RICO_TO_NAD83_2011_PRVI',5,'EPSG','8868','forward'); -- NAD83(FBN) to NAD83(NSRS2007) (EPSG:8868), 0.05 m
INSERT INTO concatenated_operation_step VALUES('PROJ','PUERTO_RICO_TO_NAD83_2011_PRVI',6,'EPSG','8673','forward'); -- NAD83(NSRS2007) to NAD83(2011) (EPSG:8673), 0.05 m
INSERT INTO usage VALUES('PROJ','PUERTO_RICO_TO_NAD83_2011_PRVI_USAGE','concatenated_operation','PROJ','PUERTO_RICO_TO_NAD83_2011_PRVI',
    'EPSG','3634', -- extent: PRVI
    'EPSG','1027' -- scope: Geodesy
);

INSERT INTO concatenated_operation VALUES('PROJ','NAD83_TO_NAD83_HARN_CORRECTED_PRVI','NAD83 to NAD83(HARN Corrected) (NADCON5, PRVI)','Transformation based on concatenation of NADCON5 transformations','EPSG','4269','EPSG','8545',0.16,NULL,0);
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD83_TO_NAD83_HARN_CORRECTED_PRVI',1,'EPSG','8669','forward'); -- NAD83 to NAD83(HARN) (EPSG:8669), 0.15 m
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD83_TO_NAD83_HARN_CORRECTED_PRVI',2,'EPSG','9181','forward'); -- NAD83(HARN) to NAD83(HARN Corrected) (EPSG:9181), 0.05 m
INSERT INTO usage VALUES('PROJ','NAD83_TO_NAD83_HARN_CORRECTED_PRVI_USAGE','concatenated_operation','PROJ','NAD83_TO_NAD83_HARN_CORRECTED_PRVI',
    'EPSG','3634', -- extent: PRVI
    'EPSG','1027' -- scope: Geodesy
);

INSERT INTO concatenated_operation VALUES('PROJ','NAD83_TO_NAD83_FBN_PRVI','NAD83 to NAD83(FBN) (NADCON5, PRVI)','Transformation based on concatenation of NADCON5 transformations','EPSG','4269','EPSG','8860',0.17,NULL,0);
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD83_TO_NAD83_FBN_PRVI',1,'EPSG','8669','forward'); -- NAD83 to NAD83(HARN) (EPSG:8669), 0.15 m
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD83_TO_NAD83_FBN_PRVI',2,'EPSG','9181','forward'); -- NAD83(HARN) to NAD83(HARN Corrected) (EPSG:9181), 0.05 m
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD83_TO_NAD83_FBN_PRVI',3,'EPSG','8867','forward'); -- NAD83(HARN Corrected) to NAD83(FBN) (EPSG:8867), 0.05 m
INSERT INTO usage VALUES('PROJ','NAD83_TO_NAD83_FBN_PRVI_USAGE','concatenated_operation','PROJ','NAD83_TO_NAD83_FBN_PRVI',
    'EPSG','3634', -- extent: PRVI
    'EPSG','1027' -- scope: Geodesy
);

INSERT INTO concatenated_operation VALUES('PROJ','NAD83_TO_NAD83_NSRS2007_PRVI','NAD83 to NAD83(NSRS2007) (NADCON5, PRVI)','Transformation based on concatenation of NADCON5 transformations','EPSG','4269','EPSG','4759',0.17,NULL,0);
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD83_TO_NAD83_NSRS2007_PRVI',1,'EPSG','8669','forward'); -- NAD83 to NAD83(HARN) (EPSG:8669), 0.15 m
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD83_TO_NAD83_NSRS2007_PRVI',2,'EPSG','9181','forward'); -- NAD83(HARN) to NAD83(HARN Corrected) (EPSG:9181), 0.05 m
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD83_TO_NAD83_NSRS2007_PRVI',3,'EPSG','8867','forward'); -- NAD83(HARN Corrected) to NAD83(FBN) (EPSG:8867), 0.05 m
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD83_TO_NAD83_NSRS2007_PRVI',4,'EPSG','8868','forward'); -- NAD83(FBN) to NAD83(NSRS2007) (EPSG:8868), 0.05 m
INSERT INTO usage VALUES('PROJ','NAD83_TO_NAD83_NSRS2007_PRVI_USAGE','concatenated_operation','PROJ','NAD83_TO_NAD83_NSRS2007_PRVI',
    'EPSG','3634', -- extent: PRVI
    'EPSG','1027' -- scope: Geodesy
);

INSERT INTO concatenated_operation VALUES('PROJ','NAD83_TO_NAD83_2011_PRVI','NAD83 to NAD83(2011) (NADCON5, PRVI)','Transformation based on concatenation of NADCON5 transformations','EPSG','4269','EPSG','6318',0.18,NULL,0);
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD83_TO_NAD83_2011_PRVI',1,'EPSG','8669','forward'); -- NAD83 to NAD83(HARN) (EPSG:8669), 0.15 m
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD83_TO_NAD83_2011_PRVI',2,'EPSG','9181','forward'); -- NAD83(HARN) to NAD83(HARN Corrected) (EPSG:9181), 0.05 m
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD83_TO_NAD83_2011_PRVI',3,'EPSG','8867','forward'); -- NAD83(HARN Corrected) to NAD83(FBN) (EPSG:8867), 0.05 m
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD83_TO_NAD83_2011_PRVI',4,'EPSG','8868','forward'); -- NAD83(FBN) to NAD83(NSRS2007) (EPSG:8868), 0.05 m
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD83_TO_NAD83_2011_PRVI',5,'EPSG','8673','forward'); -- NAD83(NSRS2007) to NAD83(2011) (EPSG:8673), 0.05 m
INSERT INTO usage VALUES('PROJ','NAD83_TO_NAD83_2011_PRVI_USAGE','concatenated_operation','PROJ','NAD83_TO_NAD83_2011_PRVI',
    'EPSG','3634', -- extent: PRVI
    'EPSG','1027' -- scope: Geodesy
);

INSERT INTO concatenated_operation VALUES('PROJ','NAD83_HARN_TO_NAD83_FBN_PRVI','NAD83(HARN) to NAD83(FBN) (NADCON5, PRVI)','Transformation based on concatenation of NADCON5 transformations','EPSG','4152','EPSG','8860',0.07,NULL,0);
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD83_HARN_TO_NAD83_FBN_PRVI',1,'EPSG','9181','forward'); -- NAD83(HARN) to NAD83(HARN Corrected) (EPSG:9181), 0.05 m
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD83_HARN_TO_NAD83_FBN_PRVI',2,'EPSG','8867','forward'); -- NAD83(HARN Corrected) to NAD83(FBN) (EPSG:8867), 0.05 m
INSERT INTO usage VALUES('PROJ','NAD83_HARN_TO_NAD83_FBN_PRVI_USAGE','concatenated_operation','PROJ','NAD83_HARN_TO_NAD83_FBN_PRVI',
    'EPSG','3634', -- extent: PRVI
    'EPSG','1027' -- scope: Geodesy
);

INSERT INTO concatenated_operation VALUES('PROJ','NAD83_HARN_TO_NAD83_NSRS2007_PRVI','NAD83(HARN) to NAD83(NSRS2007) (NADCON5, PRVI)','Transformation based on concatenation of NADCON5 transformations','EPSG','4152','EPSG','4759',0.09,NULL,0);
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD83_HARN_TO_NAD83_NSRS2007_PRVI',1,'EPSG','9181','forward'); -- NAD83(HARN) to NAD83(HARN Corrected) (EPSG:9181), 0.05 m
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD83_HARN_TO_NAD83_NSRS2007_PRVI',2,'EPSG','8867','forward'); -- NAD83(HARN Corrected) to NAD83(FBN) (EPSG:8867), 0.05 m
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD83_HARN_TO_NAD83_NSRS2007_PRVI',3,'EPSG','8868','forward'); -- NAD83(FBN) to NAD83(NSRS2007) (EPSG:8868), 0.05 m
INSERT INTO usage VALUES('PROJ','NAD83_HARN_TO_NAD83_NSRS2007_PRVI_USAGE','concatenated_operation','PROJ','NAD83_HARN_TO_NAD83_NSRS2007_PRVI',
    'EPSG','3634', -- extent: PRVI
    'EPSG','1027' -- scope: Geodesy
);

INSERT INTO concatenated_operation VALUES('PROJ','NAD83_HARN_TO_NAD83_2011_PRVI','NAD83(HARN) to NAD83(2011) (NADCON5, PRVI)','Transformation based on concatenation of NADCON5 transformations','EPSG','4152','EPSG','6318',0.1,NULL,0);
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD83_HARN_TO_NAD83_2011_PRVI',1,'EPSG','9181','forward'); -- NAD83(HARN) to NAD83(HARN Corrected) (EPSG:9181), 0.05 m
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD83_HARN_TO_NAD83_2011_PRVI',2,'EPSG','8867','forward'); -- NAD83(HARN Corrected) to NAD83(FBN) (EPSG:8867), 0.05 m
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD83_HARN_TO_NAD83_2011_PRVI',3,'EPSG','8868','forward'); -- NAD83(FBN) to NAD83(NSRS2007) (EPSG:8868), 0.05 m
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD83_HARN_TO_NAD83_2011_PRVI',4,'EPSG','8673','forward'); -- NAD83(NSRS2007) to NAD83(2011) (EPSG:8673), 0.05 m
INSERT INTO usage VALUES('PROJ','NAD83_HARN_TO_NAD83_2011_PRVI_USAGE','concatenated_operation','PROJ','NAD83_HARN_TO_NAD83_2011_PRVI',
    'EPSG','3634', -- extent: PRVI
    'EPSG','1027' -- scope: Geodesy
);

INSERT INTO concatenated_operation VALUES('PROJ','NAD83_HARN_CORRECTED_TO_NAD83_NSRS2007_PRVI','NAD83(HARN Corrected) to NAD83(NSRS2007) (NADCON5, PRVI)','Transformation based on concatenation of NADCON5 transformations','EPSG','8545','EPSG','4759',0.07,NULL,0);
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD83_HARN_CORRECTED_TO_NAD83_NSRS2007_PRVI',1,'EPSG','8867','forward'); -- NAD83(HARN Corrected) to NAD83(FBN) (EPSG:8867), 0.05 m
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD83_HARN_CORRECTED_TO_NAD83_NSRS2007_PRVI',2,'EPSG','8868','forward'); -- NAD83(FBN) to NAD83(NSRS2007) (EPSG:8868), 0.05 m
INSERT INTO usage VALUES('PROJ','NAD83_HARN_CORRECTED_TO_NAD83_NSRS2007_PRVI_USAGE','concatenated_operation','PROJ','NAD83_HARN_CORRECTED_TO_NAD83_NSRS2007_PRVI',
    'EPSG','3634', -- extent: PRVI
    'EPSG','1027' -- scope: Geodesy
);

INSERT INTO concatenated_operation VALUES('PROJ','NAD83_HARN_CORRECTED_TO_NAD83_2011_PRVI','NAD83(HARN Corrected) to NAD83(2011) (NADCON5, PRVI)','Transformation based on concatenation of NADCON5 transformations','EPSG','8545','EPSG','6318',0.09,NULL,0);
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD83_HARN_CORRECTED_TO_NAD83_2011_PRVI',1,'EPSG','8867','forward'); -- NAD83(HARN Corrected) to NAD83(FBN) (EPSG:8867), 0.05 m
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD83_HARN_CORRECTED_TO_NAD83_2011_PRVI',2,'EPSG','8868','forward'); -- NAD83(FBN) to NAD83(NSRS2007) (EPSG:8868), 0.05 m
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD83_HARN_CORRECTED_TO_NAD83_2011_PRVI',3,'EPSG','8673','forward'); -- NAD83(NSRS2007) to NAD83(2011) (EPSG:8673), 0.05 m
INSERT INTO usage VALUES('PROJ','NAD83_HARN_CORRECTED_TO_NAD83_2011_PRVI_USAGE','concatenated_operation','PROJ','NAD83_HARN_CORRECTED_TO_NAD83_2011_PRVI',
    'EPSG','3634', -- extent: PRVI
    'EPSG','1027' -- scope: Geodesy
);

INSERT INTO concatenated_operation VALUES('PROJ','NAD83_FBN_TO_NAD83_2011_PRVI','NAD83(FBN) to NAD83(2011) (NADCON5, PRVI)','Transformation based on concatenation of NADCON5 transformations','EPSG','8860','EPSG','6318',0.07,NULL,0);
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD83_FBN_TO_NAD83_2011_PRVI',1,'EPSG','8868','forward'); -- NAD83(FBN) to NAD83(NSRS2007) (EPSG:8868), 0.05 m
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD83_FBN_TO_NAD83_2011_PRVI',2,'EPSG','8673','forward'); -- NAD83(NSRS2007) to NAD83(2011) (EPSG:8673), 0.05 m
INSERT INTO usage VALUES('PROJ','NAD83_FBN_TO_NAD83_2011_PRVI_USAGE','concatenated_operation','PROJ','NAD83_FBN_TO_NAD83_2011_PRVI',
    'EPSG','3634', -- extent: PRVI
    'EPSG','1027' -- scope: Geodesy
);

INSERT INTO concatenated_operation VALUES('PROJ','AMERICAN_SAMOA_1962_TO_NAD83_FBN_AS','American Samoa 1962 to NAD83(FBN) (NADCON5, AS)','Transformation based on concatenation of NADCON5 transformations','EPSG','4169','EPSG','8860',5.0,NULL,0);
INSERT INTO concatenated_operation_step VALUES('PROJ','AMERICAN_SAMOA_1962_TO_NAD83_FBN_AS',1,'EPSG','8662','forward'); -- American Samoa 1962 to NAD83(HARN) (EPSG:8662), 5.0 m
INSERT INTO concatenated_operation_step VALUES('PROJ','AMERICAN_SAMOA_1962_TO_NAD83_FBN_AS',2,'EPSG','8863','forward'); -- NAD83(HARN) to NAD83(FBN) (EPSG:8863), 0.05 m
INSERT INTO usage VALUES('PROJ','AMERICAN_SAMOA_1962_TO_NAD83_FBN_AS_USAGE','concatenated_operation','PROJ','AMERICAN_SAMOA_1962_TO_NAD83_FBN_AS',
    'EPSG','3110', -- extent: AS
    'EPSG','1027' -- scope: Geodesy
);

INSERT INTO concatenated_operation VALUES('PROJ','AMERICAN_SAMOA_1962_TO_NAD83_PA11_AS','American Samoa 1962 to NAD83(PA11) (NADCON5, AS)','Transformation based on concatenation of NADCON5 transformations','EPSG','4169','EPSG','6322',5.0,NULL,0);
INSERT INTO concatenated_operation_step VALUES('PROJ','AMERICAN_SAMOA_1962_TO_NAD83_PA11_AS',1,'EPSG','8662','forward'); -- American Samoa 1962 to NAD83(HARN) (EPSG:8662), 5.0 m
INSERT INTO concatenated_operation_step VALUES('PROJ','AMERICAN_SAMOA_1962_TO_NAD83_PA11_AS',2,'EPSG','8863','forward'); -- NAD83(HARN) to NAD83(FBN) (EPSG:8863), 0.05 m
INSERT INTO concatenated_operation_step VALUES('PROJ','AMERICAN_SAMOA_1962_TO_NAD83_PA11_AS',3,'EPSG','8864','forward'); -- NAD83(FBN) to NAD83(PA11) (EPSG:8864), 0.05 m
INSERT INTO usage VALUES('PROJ','AMERICAN_SAMOA_1962_TO_NAD83_PA11_AS_USAGE','concatenated_operation','PROJ','AMERICAN_SAMOA_1962_TO_NAD83_PA11_AS',
    'EPSG','3110', -- extent: AS
    'EPSG','1027' -- scope: Geodesy
);

INSERT INTO concatenated_operation VALUES('PROJ','NAD83_HARN_TO_NAD83_PA11_AS','NAD83(HARN) to NAD83(PA11) (NADCON5, AS)','Transformation based on concatenation of NADCON5 transformations','EPSG','4152','EPSG','6322',0.07,NULL,0);
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD83_HARN_TO_NAD83_PA11_AS',1,'EPSG','8863','forward'); -- NAD83(HARN) to NAD83(FBN) (EPSG:8863), 0.05 m
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD83_HARN_TO_NAD83_PA11_AS',2,'EPSG','8864','forward'); -- NAD83(FBN) to NAD83(PA11) (EPSG:8864), 0.05 m
INSERT INTO usage VALUES('PROJ','NAD83_HARN_TO_NAD83_PA11_AS_USAGE','concatenated_operation','PROJ','NAD83_HARN_TO_NAD83_PA11_AS',
    'EPSG','3110', -- extent: AS
    'EPSG','1027' -- scope: Geodesy
);

INSERT INTO concatenated_operation VALUES('PROJ','GUAM_1963_TO_NAD83_FBN_GUAM','Guam 1963 to NAD83(FBN) (NADCON5, GUAM)','Transformation based on concatenation of NADCON5 transformations','EPSG','4675','EPSG','8860',5.0,NULL,0);
INSERT INTO concatenated_operation_step VALUES('PROJ','GUAM_1963_TO_NAD83_FBN_GUAM',1,'EPSG','8665','forward'); -- Guam 1963 to NAD83(HARN) (EPSG:8665), 5.0 m
INSERT INTO concatenated_operation_step VALUES('PROJ','GUAM_1963_TO_NAD83_FBN_GUAM',2,'EPSG','8865','forward'); -- NAD83(HARN) to NAD83(FBN) (EPSG:8865), 0.05 m
INSERT INTO usage VALUES('PROJ','GUAM_1963_TO_NAD83_FBN_GUAM_USAGE','concatenated_operation','PROJ','GUAM_1963_TO_NAD83_FBN_GUAM',
    'EPSG','4525', -- extent: GUAM
    'EPSG','1027' -- scope: Geodesy
);

INSERT INTO concatenated_operation VALUES('PROJ','GUAM_1963_TO_NAD83_MA11_GUAM','Guam 1963 to NAD83(MA11) (NADCON5, GUAM)','Transformation based on concatenation of NADCON5 transformations','EPSG','4675','EPSG','6325',5.0,NULL,0);
INSERT INTO concatenated_operation_step VALUES('PROJ','GUAM_1963_TO_NAD83_MA11_GUAM',1,'EPSG','8665','forward'); -- Guam 1963 to NAD83(HARN) (EPSG:8665), 5.0 m
INSERT INTO concatenated_operation_step VALUES('PROJ','GUAM_1963_TO_NAD83_MA11_GUAM',2,'EPSG','8865','forward'); -- NAD83(HARN) to NAD83(FBN) (EPSG:8865), 0.05 m
INSERT INTO concatenated_operation_step VALUES('PROJ','GUAM_1963_TO_NAD83_MA11_GUAM',3,'EPSG','8866','forward'); -- NAD83(FBN) to NAD83(MA11) (EPSG:8866), 0.05 m
INSERT INTO usage VALUES('PROJ','GUAM_1963_TO_NAD83_MA11_GUAM_USAGE','concatenated_operation','PROJ','GUAM_1963_TO_NAD83_MA11_GUAM',
    'EPSG','4525', -- extent: GUAM
    'EPSG','1027' -- scope: Geodesy
);

INSERT INTO concatenated_operation VALUES('PROJ','NAD83_HARN_TO_NAD83_MA11_GUAM','NAD83(HARN) to NAD83(MA11) (NADCON5, GUAM)','Transformation based on concatenation of NADCON5 transformations','EPSG','4152','EPSG','6325',0.07,NULL,0);
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD83_HARN_TO_NAD83_MA11_GUAM',1,'EPSG','8865','forward'); -- NAD83(HARN) to NAD83(FBN) (EPSG:8865), 0.05 m
INSERT INTO concatenated_operation_step VALUES('PROJ','NAD83_HARN_TO_NAD83_MA11_GUAM',2,'EPSG','8866','forward'); -- NAD83(FBN) to NAD83(MA11) (EPSG:8866), 0.05 m
INSERT INTO usage VALUES('PROJ','NAD83_HARN_TO_NAD83_MA11_GUAM_USAGE','concatenated_operation','PROJ','NAD83_HARN_TO_NAD83_MA11_GUAM',
    'EPSG','4525', -- extent: GUAM
    'EPSG','1027' -- scope: Geodesy
);

