--- This file has been generated by scripts/build_db.py. DO NOT EDIT !

INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5104','Huang Hai 1956','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6125','Samboja P2 exc T9','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6160','Quini-Huao','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6174','Sierra Leone Peninsular 1924','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6211','Genuk','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6218','Bogota','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6222','South Africa','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6227','Levant','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6269','NAD83(1986)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6272','GD49','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6308','Rikets koordinatsystem 1938','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5100','MSL','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5101','ODN','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5102','NGVD29','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5103','NAVD88','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5104','Yellow Sea','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5105','Baltic','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5106','Caspian','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5107','NGF','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5109','NAP','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5111','AHD','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5112','AHD (Tasmania)','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5114','CVD28','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5115','Piraeus86','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5116','N60','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5117','RH70','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5118','NGF - Lallemand','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5119','NGF-IGN69','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5120','NGF-IGN78','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5122','JSLD69','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5123','PHD93','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5127','LN02','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5128','LHN95','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5129','EVRF2000','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6121','GGRS87','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6122','ATS77','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6123','KKJ','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6124','RT90','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6126','LKS94 (ETRS89)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6130','Moznet','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6132','FD58','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6133','EST92','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6134','PSD93','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6140','NAD83(CSRS)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6151','CHTRS95','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6152','NAD83(HARN)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6154','ED50(ED77)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6156','S-JTSK','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6159','ELD79','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6163','YNGN96','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6170','SIRGAS 1995','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6171','RGF93 v1','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6172','POSGAR','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6202','AGD66','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6203','AGD84','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6204','Ain el Abd','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6215','BD50','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6230','ED50','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6231','ED87','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6237','HD72','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6238','ID74','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6242','JAD69','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6246','KOC','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6248','PSAD56','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6258','ETRS89','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6267','NAD27','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6269','NAD83','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6272','NZGD49','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6275','NTF','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6278','OSGB70','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6279','OS(SN)80','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6280','Padang','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6283','GDA94','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6291','SAD69','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6297','Tananarive','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6303','TC(1948)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6308','RT38','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6312','MGI','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6313','BD72','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6314','DHDN','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6318','NGN','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6319','KUDAMS','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6322','WGS 72','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6324','WGS 72BE','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6326','WGS 84','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6608','NAD27(76)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6609','CGQ77','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6901','ATF (Paris)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6902','NDG (Paris)','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1161','DHHN12','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1161','Deutsches Haupthöhennetz 1912','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1162','LAS-2000','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1159','GSK-2011','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1159','GRS-2011','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6181','LUREF','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6180','EST97','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6179','42/58','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6178','42/83','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1031','HR1901','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6818','Systém Jednotné trigonometrické sítě katastrální (Ferro)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6176','AAD98','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6167','NZGD2000','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5119','Nivellement general de la France','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5119','NGF','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5120','IGN78','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5118','NGF','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5120','Nivellement general de la France','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5118','Nivellement general de la France','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6143','Côte d''Ivoire (Ivory Coast)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6142','Côte d''Ivoire (Ivory Coast)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6189','SIRGAS-REGVEN','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6189','REGVEN','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6171','Réseau Géodésique Français 1993','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6172','Posiciones Geodésicas Argentinas','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6190','POSGAR 98','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6190','Posiciones Geodésicas Argentinas 1998','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6182','Observatario Flores','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6182','Azores Occidental 1939','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6183','Graciosa','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6183','Azores Central 1948','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6184','Sao Bras','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6184','Azores Oriental 1940','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6156','Systém Jednotné trigonometrické sítě katastrální','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6123','Kartastokoordinaattijärjestelmä (1966)','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5133','AIOC95','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6199','New Egyptian','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6229','Old Egyptian','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6612','JGD2000','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5137','Huang Hai 1985','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5135','HKPD','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6611','HK80','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5135','Ordnance Datum','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5136','Admiralty Chart Datum','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1164','ODN (Offshore)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6820','Segara (Jakarta)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6613','Segara','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6613','Samboja','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6613','P2 Exc','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6613','P2 Exc-T9','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6614','QND95','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6620','12th Parallel traverse','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5131','Belfast','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5101','Newlyn','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6637','Perroud 1950','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6634','MHNC72','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6627','RGR92','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6624','RGFG95','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6623','CSG67','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6648','ITRF89','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6649','ITRF90','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6650','ITRF91','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6651','ITRF92','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6652','ITRF93','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6653','ITRF94','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6654','ITRF96','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6655','ITRF97','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6656','ITRF2000','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6623','Guyane Francaise','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5151','NGNC69','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6638','St. Pierre et Miquelon 1950','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6647','ITRF88','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6645','RGNC91','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6640','RRAF91','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6659','ISN93','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6300','TM75','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6300','1975 Mapping Adjustment','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5153','NGG1977','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5156','IGN89','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5155','IGN 1988','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5154','IGN87','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6661','LKS-92','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6615','Madeira SE Base','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6665','Graciosa','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6665','Azores Central 1995','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6665','Base SW','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6183','Base SW','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6664','Sao Bras','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6664','Azores Oriental 1995','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6615','Base SE','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6663','Base SE','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6663','Madeira SE Base 1995','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6615','Porto Santo','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6667','IKBD-92','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6668','ED79','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6131','Indian (DMA Reduced)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1192','NAD83(CSRS)v1','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1192','NAD83(CSRS96)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1195','NAD83(CSRS)v4','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6265','Rome 1940','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6806','Rome 1940 (Rome)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6673','CI1979','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5172','NG-L','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5174','NN1954','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5173','TNVCN99','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6173','ETRS89','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6281','Old Israeli Datum','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6281','OID','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1146','Abu Dhabi Vertical Datum','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6674','SIRGAS 2000','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6678','Lao 1997','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5182','DHHN85','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5183','SNN76','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5172','NG95','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5176','GHA','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5177','NVN99','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5178','RNGAP','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5180','REDNAP','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5181','DHHN92','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5185','EOMA 1980','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5186','PWD','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5187','WD','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5188','CD','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6152','NAD83 (High Precision Geodetic Network)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6152','NAD83(HPGN)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6625','Fort Desaix','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6622','Sainte Anne','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6626','Piton des Neiges','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5189','NGC','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5192','IGN 1955','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5191','IGN 1950','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5114','Canadian Vertical Datum of 1928','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5114','CGVD28','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6683','PRS92','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6683','Modified Luzon Datum','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6686','MAGNA-SIRGAS','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1133','NAD83(CORS96)','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5105','Baltic Sea','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6687','RGPF','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6688','MHEFO 55','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6628','IGN 1952','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6692','MOP 1983','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6628','Tahiti','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6629','Tahaa','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5195','NGPF','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6190','POSGAR','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6701','IGCB 1955','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6701','Bas Congo 1955','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6182','Observatorio Meteorologico 1939','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6182','Observatorio 1966','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6616','Selvagem Grande 1938','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6616','Marco Astro','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6703','Mhast 1951','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6705','Mhast','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6704','Mhast','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6259','Mhast','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6699','Le Pouce (Mauritius 94)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6699','Le Pouce (Mauritius PN 94)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6725','Johnston Atoll 1961','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6737','Korea 2000','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6740','PZ-90','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6739','HK63(67)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6738','HK63','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6690','IGN79 Tahiti','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6639','Uvea SHOM 1978','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6706','S-650 TL','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6741','FD54','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6742','GDM2000','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6745','RD/83','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6746','PD/83','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6747','GR96','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6720','FGD 1986','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6720','Fiji 1986','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6752','Viti Levu 1916','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6748','Vanua Levu 1917','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6749','RGNC91-93','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6749','RGNC','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6298','Timbalai 1968','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6298','Borneo Triangulation of 1968','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6298','BT68','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6245','Malaysia Revised Triangulation 1968','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6245','MRT68','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6714','Bellevue (IGN)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6730','Santo (DOS)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6753','FD54a','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6754','LGD2006','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6755','DGN95','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6755','Indonesian Geodetic Datum 1995','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6755','IGD95','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6697','IGC 1962 6th Parallel South','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6756','VN-2000','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6896','ITRF2005','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6647','IERS Terrestrial Reference Frame 1988','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6648','IERS Terrestrial Reference Frame 1989','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6649','IERS Terrestrial Reference Frame 1990','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6650','IERS Terrestrial Reference Frame 1991','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6651','IERS Terrestrial Reference Frame 1992','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6652','IERS Terrestrial Reference Frame 1993','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6653','IERS Terrestrial Reference Frame 1994','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5204','IGLD 1955','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5205','IGLD 1985','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1271','MML07-IRF','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5206','DVR90(2000)','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1140','SHD','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6758','JAD2001','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6760','WGS 66','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1034','SREF98','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5207','HVRS71','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6762','BDA2000','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5208','RH2000','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5209','RH00','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6764','RSRGD2000','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5157','Auckland','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5158','Bluff','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5169','Chatham Island 1959','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5159','Dunedin','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5160','Gisborne','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5161','Lyttelton','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5162','Moturiki','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5163','Napier','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5164','Nelson','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5165','One Tree Point','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5170','Stewart Island','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5167','Taranaki','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5166','Tararu','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5168','Wellington','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6765','D96','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6765','Slovenia 1996','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1024','HD1909','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1025','TWD67','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1026','TWD97','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6236','Hu Tzu Shan','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6316','Dealul Piscului 1933','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1107','Cais da Figueirinha - Angra do Heroísmo','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5202','Bora Bora 2001','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5124','Fahud HD','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5132','DNN','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5138','ODN Orkney','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6743','Karbala 1979 (Polservice)','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5149','British Vertical Datum','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1029','IGRS','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6191','ALB86','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6127','Tete 1960','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6201','Blue Nile 1958','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1112','CGRS93','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1113','RGTAAF07','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1113','Reseau Geodesique des TAAF 2007','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1147','ONGD14','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1031','D48','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6805','MGI (Ferro)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1033','RGRDC 2005','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5207','HVRD71','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6728','PN84','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1027','EGM2008','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1152','WGS 84 (G730)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1153','WGS 84 (G873)','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5203','EGM84','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1154','WGS 84 (G1150)','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5171','EGM96','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1155','WGS 84 (G1674)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1273','AbInvA96_2020-IRF','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6903','Madrid','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1156','WGS 84 (G1762)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1157','PZ-90.02','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1158','PZ-90.11','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1275','REDNAP','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1276','REDNAP','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1277','REDNAP','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1160','Kyrg-06','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1286','PN68','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1278','REDNAP','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1279','REDNAP','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1165','ITRF2014','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6818','Systém Jednotnej trigonometrickej siete katastrálnej (Ferro)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1167','BGS2005','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1166','WGS 84 (Transit)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6710','ASTRO DOS 71/4','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1168','GDA2020','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1169','NZVD2016','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6710','St. Helena 1971','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1170','DHHN2016','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1170','Deutsches Haupthöhennetz 2016','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1171','POM96','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1172','POM08','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1174','SHGD2015','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1173','WGS 84 Tritan St. Helena','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1177','SHVD2015','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1180','ETRF91','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1178','ETRF89','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1280','REDNAP','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1179','ETRF90','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1181','ETRF92','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1182','ETRF93','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1183','ETRF94','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1184','ETRF96','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1185','ETRF97','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1186','ETRF2000','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1176','MSL Tritan','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1175','MSL 1971','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1176','St. Helena Tritan 2011','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1281','REDNAP','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1193','NAD83(CSRS)v2','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1187','ISN2016','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1190','ISH2004','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1190','Landshæðarkerfi Islands 2004','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6156','Systém Jednotnej trigonometrickej siete katastrálnej','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1196','NAD83(CSRS)v5','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1194','NAD83(CSRS)v3','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1197','NAD83(CSRS)v6','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1198','NAD83(CSRS)v7','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1193','NAD83(CSRS98)','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1199','GVR2000','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1200','GVR2016','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6152','Guam Geodetic Network 1993','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1201','S-JTSK [JTSK03]','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1201','Systém Jednotnej trigonometrickej siete katastrálnej [JTSK03]','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5111','AHD71','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1208','Macao 2008','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1204','ETRF2005','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1206','ETRF2014','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5111','AHD-TAS83','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6152','NAD83','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1211','NAD83(FBN)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1212','NAD83(HARN Corrected)','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1213','N43','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1214','STRS00','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1214','SRB_ETRS89','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5177','National Vertical Network 1999','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1215','SVS2010','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1036','RGM04','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1038','RGSPM06','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1216','SRB_VRS12','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6220','Camacupa','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1219','MVGC','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1218','MGD-2000','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1218','MTRF-2000','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1220','Rede Permanentes de Estaciones GNSS de Angola','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1220','RSAO13','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1220','REPANGOL','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1205','Zero Depth Point (ZDP)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1223','RGWF96','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1224','TWVD 2001','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1225','CR14','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1226','DACR52','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1230','SIRGAS-CON DGF02P01','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1227','SIRGAS-CON DGF00P01','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1227','SIRGAS Multi-Year Solution 2000','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1230','SIRGAS Multi-Year Solution 2002','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1239','SIRGAS-CON SIR13P01','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1239','SIRGAS Multi-Year Solution 2013','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1228','SIRGAS-CON DGF01P01','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1228','SIRGAS Multi-Year Solution 2001','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1231','SIRGAS-CON DGF04P01','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1231','SIRGAS Multi-Year Solution 2004','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1229','SIRGAS-CON DGF01P02','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1229','SIRGAS Multi-Year Solution 2001 extended','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1232','SIRGAS-CON DGF05P01','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1232','SIRGAS Multi-Year Solution 2005','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1240','SIRGAS-CON SIR14P01','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1240','SIRGAS Multi-Year Solution 2014','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1233','SIRGAS-CON DGF06P01','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1233','SIRGAS Multi-Year Solution 2006','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1234','SIRGAS-CON DGF07P01','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1234','SIRGAS Multi-Year Solution 2007','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1241','SIRGAS-CON SIR15P01','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1241','SIRGAS Multi-Year Solution 2015','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1235','SIRGAS-CON DGF08P01','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1039','NZVD2009','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1235','SIRGAS Multi-Year Solution 2008','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1236','SIRGAS-CON SIR09P01','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1236','SIRGAS Multi-Year Solution 2009','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1242','SIRGAS-CON SIR17P01','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1242','SIRGAS Multi-Year Solution 2017','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1237','SIRGAS-CON SIR10P01','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1237','SIRGAS Multi-Year Solution 2010','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1238','SIRGAS-CON SIR11P01','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1238','SIRGAS Multi-Year Solution 2011','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1035','REGCAN95','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1282','REDNAP','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6761','HTRS96','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1047','RRAF91','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1251','KOSOVAREF12','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6269','NAD83(Original)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1221','NAD83(MARP00)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1249','NAD83(PACP00)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1064','SIRGAS-Chile 2002','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1251','KOSOVAREF01','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1251','KOSOVAREF01 (2012)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1252','SIRGAS-Chile 2013','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1256','CGVD2013a(2010)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1258','MMN','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1259','MMS','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1260','SRVN16','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6160','Quiñi-Huao','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1261','EVRF2000 Austria','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1262','SA LLD','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1264','HS2-IRF','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1263','ONGD17','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1269','KSA-VRF14','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1265','HS2-VRF','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5207','Croatian Vertical Reference System 1971','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1255','NGNC08','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1268','KSA-GRF17','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1266','TPEN11-IRF','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1283','REDNAP','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1284','REDNAP','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1285','REDNAP','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1243','SIRGAS-Chile 2010','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1289','GBK19-IRF','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1291','ATRF2014','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1270','MSL NL','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1290','LAT NL','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1292','AVWS','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1080','Lowest Astronomic Tide','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1082','Highest Astronomic Tide','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1293','SRGI2013','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1293','Indonesian Geospatial Reference System 2013','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1293','Jaring Kontrol Geodesi (JKG)','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1294','INAGeoid2020 v1','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1042','Red Geodesica Nacional 1992','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1046','Morro do Papagaio','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1046','Island of Principe datum','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1044','Fortaleza','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1044','Island of Sao Tome datum','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1041','PTRA08','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6301','Tokyo 1918','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1048','Tokyo 1898','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1054','SLVD','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1053','SLD99','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1057','TUREF','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1058','DRUKREF 03','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1056','GDBD2009','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6674','Sistema de Referencia Geocentrico para America del Sur 2000','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6818','S-JTSK (Ferro)','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1059','FVR09','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1052','S-JTSK/05','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1052','Systém Jednotné trigonometrické sítě katastrální/05','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1055','S-JTSK/05 (Ferro)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1055','Systém Jednotné trigonometrické sítě katastrální/05 (Ferro)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1060','ISN2004','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1061','ITRF2008','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6152','NAD83 (High Accuracy Regional Network)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1069','Red Geodésica Básica Nacional de El Salvador','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1067','Sistema Geodésico Nacional','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6309','ROU-USAMS','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1065','CR05','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1063','MARGEN','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1066','MACARIO SOLIS','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1073','RGAF09','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1062','POSGAR 2007','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1062','POSGAR','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1062','Posiciones Geodésicas Argentinas 2007','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1072','Balboa','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1072','Panamá-Colón 1911','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6225','Corrego Alegre','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1076','PNG94','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6674','SIRGAS2000','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5182','Deutsches Haupthöhennetz 1985','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5181','Deutsches Haupthöhennetz 1992','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1078','FEH10','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1079','FCSVR10','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1081','DB_REF','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5113','Sea Level','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1084','HHWLT','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1082','HAT','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1085','ISLW','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1085','Indian Tidal Plane','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1083','LLWLT','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1080','LAT','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1090','MHHW','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1092','MHW','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1088','MHWS','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1088','Spring High Water','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1089','MLLW','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1086','MLLWS','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1091','MLW','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1087','MLWS','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1087','Spring Low Water','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1093','Low tide','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1094','High Tide','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1095','TGD2005','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1098','LCVD61','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1099','CBVD61','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1097','GCVD54','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6723','GCGD59','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6726','SIGD61','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6726','Little Cayman Geodetic Datum 1961','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1100','CIGD11','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1096','NN2000','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6723','Grand Cayman 1959','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5110','Oostende','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1116','NAD83(2011)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1117','NAD83(PA11)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1118','NAD83(MA11)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1042','Mexican Datum of 1993','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1120','Red Geodesica Nacional 2008','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1119','NMVD03','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1122','GUVD63','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1126','GUVD04','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1125','ASVD02','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1121','Tutuila62','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1124','VIVD09','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1123','PRVD02','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1128','JGD2011','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1131','JGD2011 (vertical)','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1130','JGD2000 (vertical)','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1129','JSLD72','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1132','RDN2008','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1288','BI','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6258','European Terrestrial Reference System 1989','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6326','World Geodetic System 1984','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6670','IGM95','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1295','LTF2004','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5215','EVRF2007','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1274','EVRF2019','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1287','EVRF2019mean','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1298','EH2000','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1299','LAS07','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1300','BGS2005','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1302','Pago Pago 2020','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1303','NVD 1992','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1301','CD Norway','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1304','REDGEOMIN','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1297','EVRF2007-PL','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1305','ETRF2000-PL','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6277','OSGB36','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6277','OSGB 1936','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1051','Genoa','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1308','EOS21-IRF','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6282','Pointe Noire','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6215','Belge 1950','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6313','Belge 1972','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6809','BD50 (Brussels)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1156','World Geodetic System 1984 (G1762'')','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1310','ECML14_NB-IRF','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6151','Swiss Terrestrial Reference Frame 1995 (CHTRF95)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1311','EWR2-IRF','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6171','Reseau Geodesique Francais 1993','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1312','RGF93 v2','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1312','Réseau Géodésique Français 1993','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1313','RGF93 v2b','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1313','Réseau Géodésique Français 1993','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1309','WGS 84 (G2139)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1193','NAD83(CSRS) 1997','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1194','NAD83(CSRS) 1997','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1195','NAD83(CSRS) 2002','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1196','NAD83(CSRS) 2005','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1197','NAD83(CSRS) 2010','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1198','NAD83(CSRS) 2010','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1314','MRH21-IRF','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1315','MOLDOR11-IRF','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5191','SHOM 1953','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6181','Luxembourg 1930','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6181','NTL','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6181','Nouvelle Triangulation du Luxembourg','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5172','Nivellement General du Luxembourg','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1316','GNTRANS2003','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1319','EBBWV14-IRF','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1317','HULLEE13-IRF','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1320','SCM22-IRF','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1321','FNL22-IRF','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1322','ITRF2020','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1323','SVD2006','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1127','CGVD2013(CGG2013)','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1325','CGVD2013(CGG2013a) epoch 2002','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1325','CGVD2013a(2002)','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1326','CGVD2013(CGG2013a) epoch 1997','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1326','CGVD2013a(1997)','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1256','Canadian Geodetic Vertical Datum of 2013 (CGG2013a)','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1256','CGVD2013(CGG2013a) epoch 2010','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1324','MWC18-IRF','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1328','INAGEOID2020 v2','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1327','SIRGAS-Chile 2021','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1253','SIRGAS-Chile 2016','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1329','MAGNA-SIRGAS 2018','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6694','POSGAR 94','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6694','POSGAR','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6694','Posiciones Geodésicas Argentinas 1994','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1331','CD UK & Ireland VORF08','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1330','MSL UK & Ireland VORF08','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1082','Approximate Highest Astronomical Tide','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1094','Approximate High Water','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1090','Approximate Mean Higher High Water','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1092','Approximate Mean High Water','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1088','Approximate Mean High Water Spring Tides','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1083','Approximate Lower Low Water Large Tide','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1093','Approximate Low Water','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1085','Approximate Indian Spring Low Water','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1084','Approximate Higher High Water Large Tide','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1089','Approximate Mean Lower Low Water','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1086','Approximate Mean Lower Low Water Spring Tides','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1091','Approximate Mean Low Water','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1087','Approximate Mean Low Water Spring Tides','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1080','Approximate Lowest Astronomical Tide','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5100','Approximate Mean Sea Level','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1336','CNH22-IRF','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1338','CWS13-IRF','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1340','GWPBS22-IRF','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1341','GWWAB22-IRF','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1342','GWWWA22-IRF','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1343','MALS09-IRF','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1345','SYC20-IRF','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1339','DIBA15-IRF','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1344','OxWo08-IRF','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1334','DoPw22-IRF','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1335','ShAb07-IRF','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1351','SMITB20-IRF','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1353','ETRS89/DREF91/2016','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1354','NGA 2022','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1355','RGSH2020','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6661','Latvia 1992','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1356','LKS-2020','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1357','RGNC15','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1346','S45B-IRF','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1347','GS-IRF','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1348','GSB-IRF','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1349','KK-IRF','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1350','Ostenfeld-IRF','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1332','S34J-IRF','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1337','S34S-IRF','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1360','NSIDC Authalic Sphere','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1362','REDNAP','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1363','REDNAP','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1364','REDNAP','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6737','KGD2002','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1049','Incheon','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1049','KVD1964','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1101','Cais da Pontinha - Funchal','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1102','Cais da Vila - Porto Santo','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1361','ZH Portugal','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1361','Zero Hidrográfico Portugal','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1365','NAD83(CSRS)v8','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1365','NAD83(CSRS) 2010','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1366','COV23-IRF','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1367','BBT2000','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1367','Brenner basistunnel','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1367','Galleria di base del Brennero','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','5206','Dansk Vertikal Reference 1990','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1371','DVR90 ensemble','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1368','DVR90(2002)','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1369','DVR90(2013)','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1370','DVR90(2023)','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1378','GLLMSL(2022)','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1372','DKMSL(2022)','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1373','DKLAT(2022)','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1375','DKLAT(2023)','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1377','DKLAT','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1376','DKMSL','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1382','ETRF2020','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1383','WGS 84 (G2296)','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1374','DKMSL(2023)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1386','WC05-IRF','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1385','ECML14-IRF','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1388','GLLAT(2023)','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1387','GLMSL(2023)','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1384','CGVD28(HTv2.0)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1081','DB_REF2003','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1081','DB_REF2016','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1390','BSCD2000','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1380','BES2020','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1389','RGM23','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1389','Repère de référence Géodésique de Mayotte 2023','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1381','Saba','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1398','Bonaire','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1392','UZGD2024','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1394','BES2020','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1395','Sint Eustatius','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1399','ITRF2020-u2023','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1402','LibRef21','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1401','UGRF','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1401','Uganda 2010','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1405','EWR3-IRF','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1404','GGD','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1404','Georgia Geodetic Datum 2011','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1406','WSPG-IRF','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','6194','Qornoq 1927','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1417','QazTRF-23','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1417','Qazaqstan Terrestrial Reference Frame 2023','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1421','GR96','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1419','GR96(2021)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1413','Asse 2025','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1415','Asse 2025','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1418','CSRN2025 (ITRF2020)','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1416','COH88 2025 (NAVD88)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1414','CSRN2025 (NAD83 2011)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1420','GR96(1996)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1422','NATRF2022','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1407','EUREF89','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1403','NKG_ETRF14','EPSG');
INSERT INTO "alias_name" VALUES('vertical_datum','EPSG','1424','SVD2024','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1225','CR-SIRGAS','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_datum','EPSG','1428','CR14','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21100','Genuk / NEIEZ','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2140','NAD83(CSRS98) / SCoPQ zone 3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2141','NAD83(CSRS98) / SCoPQ zone 4','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2142','NAD83(CSRS98) / SCoPQ zone 5','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2143','NAD83(CSRS98) / SCoPQ zone 6','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2144','NAD83(CSRS98) / SCoPQ zone 7','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2145','NAD83(CSRS98) / SCoPQ zone 8','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2146','NAD83(CSRS98) / SCoPQ zone 9','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2147','NAD83(CSRS98) / SCoPQ zone 10','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2159','Sierra Leone 1924 / Peninsular Grid','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2291','NAD83 / PEI Stereo','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3200','Final Datum 1958 / Iraq zone','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4132','Final Datum 1958 (Iran)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4140','NAD83(CSRS)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4172','National Geodetic System [Argentina]','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4211','Genuk','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4218','Bogota','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4227','Levant','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4272','GD49','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4813','Genuk (Jakarta)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21100','Genuk (Jakarta) / NEIEZ','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21148','Genuk / UTM zone 48S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21150','Genuk / UTM zone 50S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22700','Levant / Levant Zone','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22770','Levant / Syria Lambert','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22780','Levant / Levant Stereo','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','25838','ETRF89 / UTM zone 38N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','27258','GD49 / UTM zone 58','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','27259','GD49 / UTM zone 59','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','27260','GD49 / UTM zone 60','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','27291','GD49 / North Island Grid','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','27292','GD49 / South Island Grid','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','30791','Nord Sahara 1959 / Lambert Nord Voirol Unifie 1960','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','30792','Nord Sahara 1959 / Lambert Sud Voirol Unifie 1960','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31170','Zanderij / Surinam Old','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31171','Zanderij / Surinam TM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31300','Belge Lambert 72','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2000','Anguilla 1957 / BWI Grid','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2001','Antigua 1943 / BWI Grid','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2002','Dominica 1945 / BWI Grid','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2003','Grenada 1953 / BWI Grid','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2004','Montserrat 58 / BWI Grid','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2005','St Kitts 1955 / BWI Grid','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2006','St Lucia 1955 / BWI Grid','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2007','St Vincent 45 / BWI Grid','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2008','CGQ77 / SCoPQ zone 2','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2009','CGQ77 / SCoPQ zone 3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2010','CGQ77 / SCoPQ zone 4','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2011','CGQ77 / SCoPQ zone 5','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2012','CGQ77 / SCoPQ zone 6','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2013','CGQ77 / SCoPQ zone 7','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2014','CGQ77 / SCoPQ zone 8','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2015','CGQ77 / SCoPQ zone 9','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2016','CGQ77 / SCoPQ zone 10','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2036','NAD83(CSRS) / NB Stereo','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2037','NAD83(CSRS) / UTM 19N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2038','NAD83(CSRS) / UTM 20N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2040','Locodjo 65 / UTM 30N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2041','Abidjan 87 / UTM 30N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2042','Locodjo 65 / UTM 29N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2043','Abidjan 87 / UTM 29N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2044','Hanoi 72 / Gauss zone 18','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2045','Hanoi 72 / Gauss zone 19','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2046','New S African CS zone 15','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2047','New S African CS zone 17','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2048','New S African CS zone 19','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2049','New S African CS zone 21','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2050','New S African CS zone 23','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2051','New S African CS zone 25','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2052','New S African CS zone 27','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2053','New S African CS zone 29','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2054','New S African CS zone 31','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2055','New S African CS zone 33','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2056','LV95','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2066','Mount Dillon / Tobago','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2067','Naparima 1955 / UTM 20N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2081','Chos Malal 1914 / Argentina zone 2','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2083','Hito XVIII 1963 / Argentina zone 2','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2084','Hito XVIII / UTM 19S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2089','Yemen NGN96 / UTM 38N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2090','Yemen NGN96 / UTM 39N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2091','S Yemen / Gauss zone 8','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2092','S Yemen / Gauss zone 9','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2097','Korean 1985 / Central','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2099','Qatar Plane CS','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2136','Accra / Gold Coast Grid','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2137','Accra / Ghana TM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2157','IRENET95 / ITM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2200','ATS77 / NB Stereographic','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2290','ATS77 / PEI Stereo','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2291','NAD83(CSRS) / PEI Stereo','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2294','ATS77 / MTM NS zone 4','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2295','ATS77 / MTM NS zone 5','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2393','KKJ / Finland zone 3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2600','LKS94','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3561','Old Hawaiian / SP zone 1','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3562','Old Hawaiian / SP zone 2','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3563','Old Hawaiian / SP zone 3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3564','Old Hawaiian / SP zone 4','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3565','Old Hawaiian / SP zone 5','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3991','Puerto Rico SPCS 27','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4134','PDO Survey Datum 1993','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4215','Belge 1950','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4268','NAD Michigan','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4313','Belge 1972','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4609','CGQ77','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4809','Belge 1950 (Brussels)','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5701','Newlyn height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5702','National Geodetic Vertical Datum of 1929 height (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5703','North American Vertical Datum of 1988 height (m)','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5709','Normaal Amsterdams Peil height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5712','Australian Height Datum (Tasmania) height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5713','Canadian Geodetic Vertical Datum of 1928 height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5714','Mean Sea Level height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5715','Mean Sea Level depth','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5723','Japan Levelling Datum height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5724','PDO Height Datum 1993 height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5728','Landesnivellement 1902 height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5729','Landeshohennetz 1995 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','7401','NTF / France II + Lalle','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','7402','NTF / France II + IGN69','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','7403','NTF / France III + IGN69','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','7405','GB Nat Grid + ODN ht','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','7407','NAD27 / TX_N + NGVD29 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20004','S-95 zone 4','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20005','S-95 zone 5','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20006','S-95 zone 6','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20007','S-95 zone 7','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20008','S-95 zone 8','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20009','S-95 zone 9','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20010','S-95 zone 10','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20011','S-95 zone 11','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20012','S-95 zone 12','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20013','S-95 zone 13','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20014','S-95 zone 14','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20015','S-95 zone 15','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20016','S-95 zone 16','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20017','S-95 zone 17','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20018','S-95 zone 18','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20019','S-95 zone 19','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20020','S-95 zone 20','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20021','S-95 zone 21','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20022','S-95 zone 22','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20023','S-95 zone 23','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20024','S-95 zone 24','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20025','S-95 zone 25','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20026','S-95 zone 26','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20027','S-95 zone 27','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20028','S-95 zone 28','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20029','S-95 zone 29','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20030','S-95 zone 30','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20031','S-95 zone 31','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20032','S-95 zone 32','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20064','Pulkovo 1995 / Gauss 4N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20065','Pulkovo 1995 / Gauss 5N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20066','Pulkovo 1995 / Gauss 6N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20067','Pulkovo 1995 / Gauss 7N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20068','Pulkovo 1995 / Gauss 8N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20069','Pulkovo 1995 / Gauss 9N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20070','Pulkovo 1995 / Gauss 10N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20071','Pulkovo 1995 / Gauss 11N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20072','Pulkovo 1995 / Gauss 12N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20073','Pulkovo 1995 / Gauss 13N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20074','Pulkovo 1995 / Gauss 14N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20075','Pulkovo 1995 / Gauss 15N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20076','Pulkovo 1995 / Gauss 16N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20077','Pulkovo 1995 / Gauss 17N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20078','Pulkovo 1995 / Gauss 18N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20079','Pulkovo 1995 / Gauss 19N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20080','Pulkovo 1995 / Gauss 20N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20081','Pulkovo 1995 / Gauss 21N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20082','Pulkovo 1995 / Gauss 22N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20083','Pulkovo 1995 / Gauss 23N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20084','Pulkovo 1995 / Gauss 24N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20085','Pulkovo 1995 / Gauss 25N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20086','Pulkovo 1995 / Gauss 26N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20087','Pulkovo 1995 / Gauss 27N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20088','Pulkovo 1995 / Gauss 28N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20089','Pulkovo 1995 / Gauss 29N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20090','Pulkovo 1995 / Gauss 30N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20091','Pulkovo 1995 / Gauss 31N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20092','Pulkovo 1995 / Gauss 32N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20437','Ain el Abd / UTM 37N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20438','Ain el Abd / UTM 38N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20439','Ain el Abd / UTM 39N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20790','Lisbon / Portuguese Nat','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21100','Batavia / NEIEZ','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21291','Barbados 1938 / BWI Grid','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21292','Barbados National Grid','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21413','Beijing / GK zone 13','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21414','Beijing / GK zone 14','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21415','Beijing / GK zone 15','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21416','Beijing / GK zone 16','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21417','Beijing / GK zone 17','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21418','Beijing / GK zone 18','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21419','Beijing / GK zone 19','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21420','Beijing / GK zone 20','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21421','Beijing / GK zone 21','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21422','Beijing / GK zone 22','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21423','Beijing / GK zone 23','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21473','Beijing / Gauss 13N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21474','Beijing / Gauss 14N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21475','Beijing / Gauss 15N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21476','Beijing / Gauss 16N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21477','Beijing / Gauss 17N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21478','Beijing / Gauss 18N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21479','Beijing / Gauss 19N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21480','Beijing / Gauss 20N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21481','Beijing / Gauss 21N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21482','Beijing / Gauss 22N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21483','Beijing / Gauss 23N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21500','Belge Lambert 50','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21780','LV03C','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21781','LV03','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21891','Bogota / Colombia 3W','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21892','Bogota / Colombia Bogota','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21893','Bogota / Colombia 3E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21894','Bogota / Colombia 6E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22191','C Inchauspe / Argentina 1','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22192','C Inchauspe / Argentina 2','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22193','C Inchauspe / Argentina 3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22194','C Inchauspe / Argentina 4','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22195','C Inchauspe / Argentina 5','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22196','C Inchauspe / Argentina 6','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22197','C Inchauspe / Argentina 7','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22275','South African CS zone 15','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22277','South African CS zone 17','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22279','South African CS zone 19','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22281','South African CS zone 21','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22283','South African CS zone 23','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22285','South African CS zone 25','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22287','South African CS zone 27','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22289','South African CS zone 29','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22291','South African CS zone 31','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22293','South African CS zone 33','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22300','Tunisia Mining Grid','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22523','Corrego Alegre / UTM 23S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22524','Corrego Alegre / UTM 24S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22994','Egypt 1907 / Ext. Purple','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','23946','Indian 1954 / UTM 46N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','23947','Indian 1954 / UTM 47N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','23948','Indian 1954 / UTM 48N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','24047','Indian 1975 / UTM 47N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','24048','Indian 1975 / UTM 48N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','24100','Jamaica 1875 / Old Grid','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','24200','JAD69 / Jamaica Grid','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','24305','Kalianpur 37 / UTM 45N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','24306','Kalianpur 37 / UTM 46N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','24311','Kalianpur 62 / UTM 41N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','24312','Kalianpur 62 / UTM 42N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','24313','Kalianpur 62 / UTM 43N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','24342','Kalianpur 75 / UTM 42N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','24343','Kalianpur 75 / UTM 43N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','24344','Kalianpur 75 / UTM 44N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','24345','Kalianpur 75 / UTM 45N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','24346','Kalianpur 75 / UTM 46N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','24347','Kalianpur 75 / UTM 47N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','24370','Kalianpur / India 0','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','24371','Kalianpur / India I','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','24372','Kalianpur / India IIa','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','24373','Kalianpur / India IIIa','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','24374','Kalianpur / India IVa','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','24375','Kalianpur 37 / India IIb','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','24376','Kalianpur 62 / India I','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','24377','Kalianpur 62 / India IIa','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','24378','Kalianpur 75 / India I','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','24379','Kalianpur 75 / India IIa','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','24380','Kalianpur 75 / India IIb','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','24381','Kalianpur 75 / India IIIa','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','24382','Kalianpur / India IIb','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','24383','Kalianpur 75 / India IVa','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','24892','PSAD56 / Peru central','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','25000','Leigon / Ghana Grid','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','25391','Luzon / Philippines I','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','25392','Luzon / Philippines II','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','25393','Luzon / Philippines III','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','25394','Luzon / Philippines IV','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','25395','Luzon / Philippines V','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','25700','Makassar / NEIEZ','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','25932','Malongo 1987 / UTM 32S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26391','Minna / Nigeria West','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26393','Minna / Nigeria East','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26591','Monte Mario / Italy 1','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26592','Monte Mario / Italy 2','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26632','M''poraloko / UTM 32N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26692','M''poraloko / UTM 32S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26741','NAD27 / California I','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26742','NAD27 / California II','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26743','NAD27 / California III','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26744','NAD27 / California IV','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26745','NAD27 / California V','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26746','NAD27 / California VI','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26747','NAD27 / California VII','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26786','NAD27 / Massachusetts','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26787','NAD27 / Massachusetts Is','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26792','NAD27 / Minnesota Cent.','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26801','NAD27 / Michigan East','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26802','NAD27 / Michigan Old Cen','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26803','NAD27 / Michigan West','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26811','NAD27 / Michigan North','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26812','NAD27 / Michigan Central','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26813','NAD27 / Michigan South','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26941','NAD83 / California 1','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26942','NAD83 / California 2','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26943','NAD83 / California 3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26944','NAD83 / California 4','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26945','NAD83 / California 5','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26946','NAD83 / California 6','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26986','NAD83 / Massachusetts','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26987','NAD83 / Massachusetts Is','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26992','NAD83 / Minnesota Cent.','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','27038','Nahrwan 1967 / UTM 38N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','27039','Nahrwan 1967 / UTM 39N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','27040','Nahrwan 1967 / UTM 40N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','27120','Naparima 1972 / UTM 20N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','27200','NZGD49 / NZ Map Grid','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','27391','NGO 1948 / I','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','27392','NGO 1948 / II','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','27393','NGO 1948 / III','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','27394','NGO 1948 / IV','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','27395','NGO 1948 / V','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','27396','NGO 1948 / VI','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','27397','NGO 1948 / VII','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','27398','NGO 1948 / VIII','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','27500','ATF / Nord de Guerre','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','27581','NTF / France I','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','27582','NTF / France II','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','27583','NTF / France III','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','27584','NTF / France IV','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','27591','NTF / Nord France','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','27592','NTF / Centre France','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','27593','NTF / Sud France','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','27594','NTF / Corse','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','27700','British National Grid','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28191','Palestine Grid','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28192','Palestine Belt','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28193','Israeli CS Grid','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28232','Point Noire / UTM 32S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28404','S-42 zone 4','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28405','S-42 zone 5','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28406','S-42 zone 6','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28407','S-42 zone 7','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28408','S-42 zone 8','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28409','S-42 zone 9','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28410','S-42 zone 10','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28411','S-42 zone 11','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28412','S-42 zone 12','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28413','S-42 zone 13','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28414','S-42 zone 14','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28415','S-42 zone 15','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28416','S-42 zone 16','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28417','S-42 zone 17','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28418','S-42 zone 18','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28419','S-42 zone 19','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28420','S-42 zone 20','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28421','S-42 zone 21','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28422','S-42 zone 22','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28423','S-42 zone 23','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28424','S-42 zone 24','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28425','S-42 zone 25','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28426','S-42 zone 26','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28427','S-42 zone 27','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28428','S-42 zone 28','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28429','S-42 zone 29','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28430','S-42 zone 30','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28431','S-42 zone 31','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28432','S-42 zone 32','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28462','Pulkovo / Gauss 2N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28463','Pulkovo / Gauss 3N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28464','Pulkovo / Gauss 4N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28465','Pulkovo / Gauss 5N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28466','Pulkovo / Gauss 6N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28467','Pulkovo / Gauss 7N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28468','Pulkovo / Gauss 8N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28469','Pulkovo / Gauss 9N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28470','Pulkovo / Gauss 10N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28471','Pulkovo / Gauss 11N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28472','Pulkovo / Gauss 12N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28473','Pulkovo / Gauss 13N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28474','Pulkovo / Gauss 14N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28475','Pulkovo / Gauss 15N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28476','Pulkovo / Gauss 16N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28477','Pulkovo / Gauss 17N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28478','Pulkovo / Gauss 18N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28479','Pulkovo / Gauss 19N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28480','Pulkovo / Gauss 20N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28481','Pulkovo / Gauss 21N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28482','Pulkovo / Gauss 22N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28483','Pulkovo / Gauss 23N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28484','Pulkovo / Gauss 24N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28485','Pulkovo / Gauss 25N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28486','Pulkovo / Gauss 26N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28487','Pulkovo / Gauss 27N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28488','Pulkovo / Gauss 28N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28489','Pulkovo / Gauss 29N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28490','Pulkovo / Gauss 30N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28491','Pulkovo / Gauss 31N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28492','Pulkovo / Gauss 32N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28600','Qatar National Grid','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','29220','Sapper Hill / UTM 20S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','29221','Sapper Hill / UTM 21S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','29333','Schwarzeck / UTM 33S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','29371','SW African CS zone 11','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','29373','SW African CS zone 13','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','29375','SW African CS zone 15','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','29377','SW African CS zone 17','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','29379','SW African CS zone 19','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','29381','SW African CS zone 21','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','29383','SW African CS zone 23','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','29385','SW African CS zone 25','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','29700','Tananarive  / Laborde','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','29738','Tananarive / UTM 38S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','29739','Tananarive / UTM 39S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','29849','Timbalai 1948 / UTM 49N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','29850','Timbalai 1948 / UTM 50N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','29871','Timbalai / Borneo (ch)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','29872','Timbalai / Borneo (ftSe)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','29900','TM65 / Irish Nat Grid','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','30161','Tokyo / Japan zone I','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','30162','Tokyo / Japan zone II','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','30163','Tokyo / Japan zone III','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','30164','Tokyo / Japan zone IV','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','30165','Tokyo / Japan zone V','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','30166','Tokyo / Japan zone VI','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','30167','Tokyo / Japan zone VII','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','30168','Tokyo / Japan zone VIII','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','30169','Tokyo / Japan zone IX','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','30170','Tokyo / Japan zone X','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','30171','Tokyo / Japan zone XI','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','30172','Tokyo / Japan zone XII','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','30173','Tokyo / Japan zone XIII','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','30174','Tokyo / Japan zone XIV','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','30175','Tokyo / Japan zone XV','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','30176','Tokyo / Japan zone XVI','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','30177','Tokyo / Japan zone XVII','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','30178','Tokyo / Japan zone XVIII','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','30200','Trinidad 1903 / Cassini','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','30491','Voirol75 / N Algeria old','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','30492','Voirol75 / S Algeria old','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','30729','Nord Sahara / UTM 29N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','30730','Nord Sahara / UTM 30N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','30731','Nord Sahara / UTM 31N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','30732','Nord Sahara / UTM 32N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','30791','Nord Sahara / N Algerie','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','30792','Nord Sahara / S Algerie','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31265','MGI / Gauss zone 5','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31266','MGI / Gauss zone 6','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31267','MGI / Gauss zone 7','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31268','MGI / Gauss zone 8','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31291','MGI / Austria West','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31292','MGI / Austria Central','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31293','MGI / Austria East','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31370','BD72 / Lambert 72','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31461','DHDN / Gauss zone 1','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31462','DHDN / Gauss zone 2','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31463','DHDN / Gauss zone 3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31464','DHDN / Gauss zone 4','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31465','DHDN / Gauss zone 5','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31600','Stereo 33','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31700','Stereo 70','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32013','NAD27 / New Mexico Cent.','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32018','NAD27 / New York Long Is','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32020','NAD27 / North Dakota N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32021','NAD27 / North Dakota S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32028','NAD27 / Pennsylvania N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32029','NAD27 / Pennsylvania S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32031','NAD27 / South Carolina N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32033','NAD27 / South Carolina S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32034','NAD27 / South Dakota N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32035','NAD27 / South Dakota S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32038','NAD27 / Texas North Cen.','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32040','NAD27 / Texas South Cen.','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32050','NAD27 / West Virginia N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32051','NAD27 / West Virginia S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32053','NAD27 / Wisconsin Cen.','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32056','NAD27 / Wyoming E. Cen.','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32057','NAD27 / Wyoming W. Cen.','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32113','NAD83 / New Mexico Cent.','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32118','NAD83 / New York Long Is','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32120','NAD83 / North Dakota N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32121','NAD83 / North Dakota S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32128','NAD83 / Pennsylvania N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32129','NAD83 / Pennsylvania S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32134','NAD83 / South Dakota N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32135','NAD83 / South Dakota S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32138','NAD83 / Texas North Cen.','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32140','NAD83 / Texas South Cen.','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32150','NAD83 / West Virginia N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32151','NAD83 / West Virginia S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32153','NAD83 / Wisconsin Cen.','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32156','NAD83 / Wyoming E. Cen.','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32157','NAD83 / Wyoming W. Cen.','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5702','NGVD29 height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','6359','NGVD29 depth','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4143','Côte d''Ivoire','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4143','Port Bouet','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4142','Port Bouet','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4142','Côte d''Ivoire','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2164','Cote d''Ivoire / TM 5 NW','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2164','Port Bouet / TM 5 NW','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2165','Cote d''Ivoire / TM 5 NW','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2165','Port Bouet / TM 5 NW','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9367','ETRS89 / TPEN11 SnakeGrid','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','9368','ETRS89 / TPEN11 SnakeGrid + Newlyn height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','6696','JGD2000','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','6697','JGD2011','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2393','KKJ / Basic Coordinate System zone 3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2391','KKJ / Basic Coordinate System zone 1','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2392','KKJ / Basic Coordinate System zone 2','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2394','KKJ / Basic Coordinate System zone 4','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31467','DHDN / Gauss-Kruger zone 3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31468','DHDN / Gauss-Kruger zone 4','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31469','DHDN / Gauss-Kruger zone 5','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6787','NAD83(2011) / OCRS_BKE (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6786','NAD83(2011) / OCRS_BKE (m)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4178','42/83','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31466','DHDN / Gauss-Kruger zone 2','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2166','Pulkovo 1942(83) / 3-degree Gauss-Kruger zone 3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2167','Pulkovo 1942(83) / 3-degree Gauss-Kruger zone 4','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4181','NTL (1930)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6791','NAD83(2011) / OCRS_BKF (ft)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8233','NAD83(CSRS98)','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','8881','Wiener Null','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','3906','HR1901','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6795','NAD83(2011) / OCRS_BRP (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6794','NAD83(2011) / OCRS_BRP (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6803','NAD83(2011) / OCRS_CGP (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31275','HDKS zone 5','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31276','HDKS zone 6','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31275','D48 zone 5','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2169','Luxembourg 1930 / Gauss','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3396','DHDN / 3GK zone 3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3397','DHDN / 3GK zone 4','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3398','DHDN / Gauss-Kruger zone 4','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3399','DHDN / Gauss-Kruger zone 5','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31282','MGI (Ferro) / Austria C','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31283','MGI (Ferro) / Austria E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31281','MGI (Ferro) / Austria W','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4176','AAD98','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2193','NZGD2000 / NZTM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6802','NAD83(2011) / OCRS_CGP (m)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4300','1975 Mapping Adjustment','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2196','System 2000 Jylland zoner','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2197','System 2000 Sjaelland zoner','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2198','System 2000 Bornholm zoner','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6807','NAD83(2011) / OCRS_CRE (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6810','NAD83(2011) / OCRS_CRW (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6811','NAD83(2011) / OCRS_CRW (ft)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4190','National Geodetic System [Argentina]','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6806','NAD83(2011) / OCRS_CRE (m)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4183','Graciosa','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4184','Sao Braz','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','27561','NTF (Paris) / Nord France','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','27562','NTF (Paris) / Cen France','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','27563','NTF (Paris) / Sud France','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','27564','NTF (Paris) / Corse','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','27571','NTF (Paris) / France I','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','27572','NTF (Paris) / France II','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','27573','NTF (Paris) / France III','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','27574','NTF (Paris) / France IV','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','27572','NTF (Paris) / Lambert zone II etendu','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6815','NAD83(2011) / OCRS_CGC (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2168','Pulkovo 1942(83) / 3-degree Gauss-Kruger zone 5','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2206','ED50 / Turkey zone 9','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2207','ED50 / Turkey zone 10','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2208','ED50 / Turkey zone 11','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2209','ED50 / Turkey zone 12','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2210','ED50 / Turkey zone 13','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2211','ED50 / Turkey zone 14','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2212','ED50 / Turkey zone 15','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6814','NAD83(2011) / OCRS_CGC (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2189','Graciosa / UTM zone 26N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2190','Sao Braz / UTM zone 26N','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4207','Lisbon 1937','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4803','Lisbon 1937 (Lisbon)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20791','Lisbon 1937 (Lisbon) / Portuguese Grid','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20790','Lisbon 1937 (Lisbon)/Portuguese National Grid','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6819','NAD83(2011) / OCRS_DFM (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6818','NAD83(2011) / OCRS_DFM (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2040','Port Bouet / UTM zone 30N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2040','Côte d''Ivoire / UTM zone 30N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2041','Port Bouet / UTM zone 30N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2041','Côte d''Ivoire / UTM zone 30N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2042','Port Bouet / UTM zone 29N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2042','Côte d''Ivoire / UTM zone 29N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2043','Port Bouet / UTM zone 29N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2043','Côte d''Ivoire / UTM zone 29N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6823','NAD83(2011) / OCRS_EUG (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6822','NAD83(2011) / OCRS_EUG (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6827','NAD83(2011) / OCRS_GPA (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2297','Qornoq 1927 / Greenland zone 1 west','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2307','Qornoq 1927 / Greenland zone 8 east','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32113','NAD83 / New Mexico Central (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26949','NAD83 / Arizona Central (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26929','NAD83 / Alabama East (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26930','NAD83 / Alabama West (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26931','NAD83 / Alaska zone 1 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26940','NAD83 / Alaska zone 10 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26932','NAD83 / Alaska zone 2 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26933','NAD83 / Alaska zone 3 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26934','NAD83 / Alaska zone 4 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26935','NAD83 / Alaska zone 5 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26936','NAD83 / Alaska zone 6 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26937','NAD83 / Alaska zone 7 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26938','NAD83 / Alaska zone 8 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26939','NAD83 / Alaska zone 9 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26948','NAD83 / Arizona East (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26950','NAD83 / Arizona West (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26951','NAD83 / Arkansas North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26952','NAD83 / Arkansas South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26941','NAD83 / California zone 1 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26942','NAD83 / California zone 2 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26943','NAD83 / California zone 3 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26944','NAD83 / California zone 4 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26945','NAD83 / California zone 5 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26946','NAD83 / California zone 6 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26954','NAD83 / Colorado Central (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26953','NAD83 / Colorado North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26955','NAD83 / Colorado South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26956','NAD83 / Connecticut (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26957','NAD83 / Delaware (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26958','NAD83 / Florida East (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26960','NAD83 / Florida North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26959','NAD83 / Florida West (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26966','NAD83 / Georgia East (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26967','NAD83 / Georgia West (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26961','NAD83 / Hawaii zone 1 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26962','NAD83 / Hawaii zone 2 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26963','NAD83 / Hawaii zone 3 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26964','NAD83 / Hawaii zone 4 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26965','NAD83 / Hawaii zone 5 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26969','NAD83 / Idaho Central (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26968','NAD83 / Idaho East (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26970','NAD83 / Idaho West (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26971','NAD83 / Illinois East (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26972','NAD83 / Illinois West (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26973','NAD83 / Indiana East (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26974','NAD83 / Indiana West (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26975','NAD83 / Iowa North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26976','NAD83 / Iowa South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26977','NAD83 / Kansas North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26978','NAD83 / Kansas South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2205','NAD83 / Kentucky North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26980','NAD83 / Kentucky South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26981','NAD83 / Louisiana North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26982','NAD83 / Louisiana South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26983','NAD83 / Maine East (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26984','NAD83 / Maine West (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26985','NAD83 / Maryland (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26987','NAD83 / Massachusetts Island (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26986','NAD83 / Massachusetts Mainland (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26989','NAD83 / Michigan Central (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26988','NAD83 / Michigan North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26990','NAD83 / Michigan South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26992','NAD83 / Minnesota Central (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26991','NAD83 / Minnesota North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26993','NAD83 / Minnesota South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26994','NAD83 / Mississippi East (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26995','NAD83 / Mississippi West (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26997','NAD83 / Missouri Central (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26996','NAD83 / Missouri East (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26998','NAD83 / Missouri West (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32100','NAD83 / Montana (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32104','NAD83 / Nebraska (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32108','NAD83 / Nevada Central (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32107','NAD83 / Nevada East (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32109','NAD83 / Nevada West (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32110','NAD83 / New Hampshire (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32111','NAD83 / New Jersey (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32112','NAD83 / New Mexico East (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32114','NAD83 / New Mexico West (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32116','NAD83 / New York Central (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32115','NAD83 / New York East (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32118','NAD83 / New York Long Island (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32117','NAD83 / New York West (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32119','NAD83 / North Carolina (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32120','NAD83 / North Dakota North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32121','NAD83 / North Dakota South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32122','NAD83 / Ohio North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32123','NAD83 / Ohio South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32124','NAD83 / Oklahoma North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32125','NAD83 / Oklahoma South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32126','NAD83 / Oregon North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32127','NAD83 / Oregon South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32128','NAD83 / Pennsylvania North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32129','NAD83 / Pennsylvania South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32130','NAD83 / Rhode Island (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32133','NAD83 / South Carolina (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32134','NAD83 / South Dakota North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32135','NAD83 / South Dakota South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32136','NAD83 / Tennessee (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32139','NAD83 / Texas Central (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32137','NAD83 / Texas North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32138','NAD83 / Texas North Central (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32141','NAD83 / Texas South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32140','NAD83 / Texas South Central (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32143','NAD83 / Utah Central (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32142','NAD83 / Utah North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32144','NAD83 / Utah South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32145','NAD83 / Vermont (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32146','NAD83 / Virginia North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32147','NAD83 / Virginia South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32148','NAD83 / Washington North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32149','NAD83 / Washington South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32150','NAD83 / West Virginia North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32151','NAD83 / West Virginia South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32153','NAD83 / Wisconsin Central (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32152','NAD83 / Wisconsin North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32154','NAD83 / Wisconsin South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32155','NAD83 / Wyoming East (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32156','NAD83 / Wyoming East Central (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32158','NAD83 / Wyoming West (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32157','NAD83 / Wyoming West Central (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32161','NAD83 / Puerto Rico & Virgin Is. (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6826','NAD83(2011) / OCRS_GPA (m)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4282','Congo 1960 Pointe Noire','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28232','Congo 1960 Pointe Noire / UTM zone 32S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21149','Genuk / UTM zone 49S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2308','Genuk / TM 109 SE','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5705','Kronstadt 1977 height','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4199','New Egyptian','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4229','Old Egyptian','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32064','NAD27 / UTM zone 14N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32065','NAD27 / UTM zone 15N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32066','NAD27 / UTM zone 16N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32067','NAD27 / UTM zone 17N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6831','NAD83(2011) / OCRS_GWS (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2319','ED50 / 3-degree Gauss-Kruger CM 27E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2320','ED50 / 3-degree Gauss-Kruger CM 30E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2321','ED50 / 3-degree Gauss-Kruger CM 33E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2322','ED50 / 3-degree Gauss-Kruger CM 36E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2323','ED50 / 3-degree Gauss-Kruger CM 39E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2324','ED50 / 3-degree Gauss-Kruger CM 42E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2325','ED50 / 3-degree Gauss-Kruger CM 45E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6830','NAD83(2011) / OCRS_GWS (m)','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5736','Huang Hai 1956 height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5738','Hong Kong Principal Datum height','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2343','Xian 1980 / 6-degree Gauss-Kruger CM 105E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2344','Xian 1980 / 6-degree Gauss-Kruger CM 111E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2338','Xian 1980 / 6-degree Gauss-Kruger CM 75E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2342','Xian 1980 / 6-degree Gauss-Kruger CM 99E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2348','Xian 1980 / 6-degree Gauss-Kruger CM 135E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2339','Xian 1980 / 6-degree Gauss-Kruger CM 81E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2340','Xian 1980 / 6-degree Gauss-Kruger CM 87E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2341','Xian 1980 / 6-degree Gauss-Kruger CM 93E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2345','Xian 1980 / 6-degree Gauss-Kruger CM 117E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2346','Xian 1980 / 6-degree Gauss-Kruger CM 123E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2347','Xian 1980 / 6-degree Gauss-Kruger CM 129E','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5737','Huang Hai 1985 height','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2326','HK 1980 Grid System','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4611','HK1980','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2327','Xian 1980 / 6-degree Gauss-Kruger zone 13','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2328','Xian 1980 / 6-degree Gauss-Kruger zone 14','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2329','Xian 1980 / 6-degree Gauss-Kruger zone 15','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2330','Xian 1980 / 6-degree Gauss-Kruger zone 16','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2331','Xian 1980 / 6-degree Gauss-Kruger zone 17','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2332','Xian 1980 / 6-degree Gauss-Kruger zone 18','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2333','Xian 1980 / 6-degree Gauss-Kruger zone 19','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2334','Xian 1980 / 6-degree Gauss-Kruger zone 20','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2335','Xian 1980 / 6-degree Gauss-Kruger zone 21','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2336','Xian 1980 / 6-degree Gauss-Kruger zone 22','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2337','Xian 1980 / 6-degree Gauss-Kruger zone 23','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21413','Beijing 1954 / 6-degree Gauss-Kruger zone 13','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21414','Beijing 1954 / 6-degree Gauss-Kruger zone 14','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21415','Beijing 1954 / 6-degree Gauss-Kruger zone 15','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21416','Beijing 1954 / 6-degree Gauss-Kruger zone 16','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21417','Beijing 1954 / 6-degree Gauss-Kruger zone 17','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21418','Beijing 1954 / 6-degree Gauss-Kruger zone 18','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21419','Beijing 1954 / 6-degree Gauss-Kruger zone 19','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21420','Beijing 1954 / 6-degree Gauss-Kruger zone 20','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21421','Beijing 1954 / 6-degree Gauss-Kruger zone 21','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21422','Beijing 1954 / 6-degree Gauss-Kruger zone 22','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21423','Beijing 1954 / 6-degree Gauss-Kruger zone 23','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21453','Beijing 1954 / 6-degree Gauss-Kruger CM 75E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21454','Beijing 1954 / 6-degree Gauss-Kruger CM 81E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21455','Beijing 1954 / 6-degree Gauss-Kruger CM 87E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21456','Beijing 1954 / 6-degree Gauss-Kruger CM 93E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21457','Beijing 1954 / 6-degree Gauss-Kruger CM 99E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21458','Beijing 1954 / 6-degree Gauss-Kruger CM 105E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21459','Beijing 1954 / 6-degree Gauss-Kruger CM 111E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21460','Beijing 1954 / 6-degree Gauss-Kruger CM 117E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21461','Beijing 1954 / 6-degree Gauss-Kruger CM 123E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21462','Beijing 1954 / 6-degree Gauss-Kruger CM 129E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21463','Beijing 1954 / 6-degree Gauss-Kruger CM 135E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2044','Hanoi 1972 / 6-degree Gauss-Kruger zone 18','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2045','Hanoi 1972 / 6-degree Gauss-Kruger zone 19','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2462','Albanian 1987 / 6-degree Gauss-Kruger zone 4','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2397','Pulkovo 1942(83) / Gauss-Kruger zone 3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2398','Pulkovo 1942(83) / Gauss-Kruger zone 4','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2399','Pulkovo 1942(83) / Gauss-Kruger zone 5','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2395','S Yemen / G-K zone 8','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2396','S Yemen / G-K zone 9','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2396','South Yemen / 6-degree Gauss-Kruger zone 9','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2395','South Yemen / 6-degree Gauss-Kruger zone 8','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22191','Campo Inchauspe / Gauss-Kruger zone 1','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22192','Campo Inchauspe / Gauss-Kruger zone 2','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22193','Campo Inchauspe / Gauss-Kruger zone 3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22194','Campo Inchauspe / Gauss-Kruger zone 4','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22195','Campo Inchauspe / Gauss-Kruger zone 5','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22196','Campo Inchauspe / Gauss-Kruger zone 6','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22197','Campo Inchauspe / Gauss-Kruger zone 7','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2081','Chos Malal 1914 / Gauss-Kruger zone 2','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2082','Pampa del Castillo / Gauss-Kruger zone 2','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2083','Hito XVIII 1963 / Gauss-Kruger zone 2','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20004','Pulkovo 1995 / 6-degree Gauss-Kruger zone 4','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20005','Pulkovo 1995 / 6-degree Gauss-Kruger zone 5','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20006','Pulkovo 1995 / 6-degree Gauss-Kruger zone 6','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20007','Pulkovo 1995 / 6-degree Gauss-Kruger zone 7','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20008','Pulkovo 1995 / 6-degree Gauss-Kruger zone 8','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20009','Pulkovo 1995 / 6-degree Gauss-Kruger zone 9','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20010','Pulkovo 1995 / 6-degree Gauss-Kruger zone 10','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20011','Pulkovo 1995 / 6-degree Gauss-Kruger zone 11','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20012','Pulkovo 1995 / 6-degree Gauss-Kruger zone 12','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20013','Pulkovo 1995 / 6-degree Gauss-Kruger zone 13','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20014','Pulkovo 1995 / 6-degree Gauss-Kruger zone 14','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20015','Pulkovo 1995 / 6-degree Gauss-Kruger zone 15','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20016','Pulkovo 1995 / 6-degree Gauss-Kruger zone 16','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20017','Pulkovo 1995 / 6-degree Gauss-Kruger zone 17','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20018','Pulkovo 1995 / 6-degree Gauss-Kruger zone 18','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20019','Pulkovo 1995 / 6-degree Gauss-Kruger zone 19','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20020','Pulkovo 1995 / 6-degree Gauss-Kruger zone 20','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20021','Pulkovo 1995 / 6-degree Gauss-Kruger zone 21','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20022','Pulkovo 1995 / 6-degree Gauss-Kruger zone 22','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20023','Pulkovo 1995 / 6-degree Gauss-Kruger zone 23','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20024','Pulkovo 1995 / 6-degree Gauss-Kruger zone 24','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20025','Pulkovo 1995 / 6-degree Gauss-Kruger zone 25','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20026','Pulkovo 1995 / 6-degree Gauss-Kruger zone 26','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20027','Pulkovo 1995 / 6-degree Gauss-Kruger zone 27','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20028','Pulkovo 1995 / 6-degree Gauss-Kruger zone 28','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20029','Pulkovo 1995 / 6-degree Gauss-Kruger zone 29','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20030','Pulkovo 1995 / 6-degree Gauss-Kruger zone 30','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20031','Pulkovo 1995 / 6-degree Gauss-Kruger zone 31','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20032','Pulkovo 1995 / 6-degree Gauss-Kruger zone 32','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28402','Pulkovo 1942 / 6-degree Gauss-Kruger zone 2','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28403','Pulkovo 1942 / 6-degree Gauss-Kruger zone 3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28404','Pulkovo 1942 / 6-degree Gauss-Kruger zone 4','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28405','Pulkovo 1942 / 6-degree Gauss-Kruger zone 5','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28406','Pulkovo 1942 / 6-degree Gauss-Kruger zone 6','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28407','Pulkovo 1942 / 6-degree Gauss-Kruger zone 7','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28408','Pulkovo 1942 / 6-degree Gauss-Kruger zone 8','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28409','Pulkovo 1942 / 6-degree Gauss-Kruger zone 9','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28410','Pulkovo 1942 / 6-degree Gauss-Kruger zone 10','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28411','Pulkovo 1942 / 6-degree Gauss-Kruger zone 11','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28412','Pulkovo 1942 / 6-degree Gauss-Kruger zone 12','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28413','Pulkovo 1942 / 6-degree Gauss-Kruger zone 13','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28414','Pulkovo 1942 / 6-degree Gauss-Kruger zone 14','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28415','Pulkovo 1942 / 6-degree Gauss-Kruger zone 15','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28416','Pulkovo 1942 / 6-degree Gauss-Kruger zone 16','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28417','Pulkovo 1942 / 6-degree Gauss-Kruger zone 17','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28418','Pulkovo 1942 / 6-degree Gauss-Kruger zone 18','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28419','Pulkovo 1942 / 6-degree Gauss-Kruger zone 19','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28420','Pulkovo 1942 / 6-degree Gauss-Kruger zone 20','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28421','Pulkovo 1942 / 6-degree Gauss-Kruger zone 21','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28422','Pulkovo 1942 / 6-degree Gauss-Kruger zone 22','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28423','Pulkovo 1942 / 6-degree Gauss-Kruger zone 23','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28424','Pulkovo 1942 / 6-degree Gauss-Kruger zone 24','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28425','Pulkovo 1942 / 6-degree Gauss-Kruger zone 25','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28426','Pulkovo 1942 / 6-degree Gauss-Kruger zone 26','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28427','Pulkovo 1942 / 6-degree Gauss-Kruger zone 27','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28428','Pulkovo 1942 / 6-degree Gauss-Kruger zone 28','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28429','Pulkovo 1942 / 6-degree Gauss-Kruger zone 29','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28430','Pulkovo 1942 / 6-degree Gauss-Kruger zone 30','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28431','Pulkovo 1942 / 6-degree Gauss-Kruger zone 31','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28432','Pulkovo 1942 / 6-degree Gauss-Kruger zone 32','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2463','Pulkovo 1995 / 6-degree Gauss-Kruger CM 21E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2464','Pulkovo 1995 / 6-degree Gauss-Kruger CM 27E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2465','Pulkovo 1995 / 6-degree Gauss-Kruger CM 33E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2466','Pulkovo 1995 / 6-degree Gauss-Kruger CM 39E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2467','Pulkovo 1995 / 6-degree Gauss-Kruger CM 45E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2468','Pulkovo 1995 / 6-degree Gauss-Kruger CM 51E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2469','Pulkovo 1995 / 6-degree Gauss-Kruger CM 57E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2470','Pulkovo 1995 / 6-degree Gauss-Kruger CM 63E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2471','Pulkovo 1995 / 6-degree Gauss-Kruger CM 69E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2472','Pulkovo 1995 / 6-degree Gauss-Kruger CM 75E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2473','Pulkovo 1995 / 6-degree Gauss-Kruger CM 81E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2474','Pulkovo 1995 / 6-degree Gauss-Kruger CM 87E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2475','Pulkovo 1995 / 6-degree Gauss-Kruger CM 93E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2476','Pulkovo 1995 / 6-degree Gauss-Kruger CM 99E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2477','Pulkovo 1995 / 6-degree Gauss-Kruger CM 105E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2478','Pulkovo 1995 / 6-degree Gauss-Kruger CM 111E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2479','Pulkovo 1995 / 6-degree Gauss-Kruger CM 117E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2480','Pulkovo 1995 / 6-degree Gauss-Kruger CM 123E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2481','Pulkovo 1995 / 6-degree Gauss-Kruger CM 129E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2482','Pulkovo 1995 / 6-degree Gauss-Kruger CM 135E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2483','Pulkovo 1995 / 6-degree Gauss-Kruger CM 141E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2484','Pulkovo 1995 / 6-degree Gauss-Kruger CM 147E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2485','Pulkovo 1995 / 6-degree Gauss-Kruger CM 153E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2486','Pulkovo 1995 / 6-degree Gauss-Kruger CM 159E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2487','Pulkovo 1995 / 6-degree Gauss-Kruger CM 165E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2488','Pulkovo 1995 / 6-degree Gauss-Kruger CM 171E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2489','Pulkovo 1995 / 6-degree Gauss-Kruger CM 177E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2490','Pulkovo 1995 / 6-degree Gauss-Kruger CM 177W','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2491','Pulkovo 1995 / 6-degree Gauss-Kruger CM 171W','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2492','Pulkovo 1942 / 6-degree Gauss-Kruger CM 9E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2493','Pulkovo 1942 / 6-degree Gauss-Kruger CM 15E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2494','Pulkovo 1942 / 6-degree Gauss-Kruger CM 21E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2495','Pulkovo 1942 / 6-degree Gauss-Kruger CM 27E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2496','Pulkovo 1942 / 6-degree Gauss-Kruger CM 33E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2497','Pulkovo 1942 / 6-degree Gauss-Kruger CM 39E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2498','Pulkovo 1942 / 6-degree Gauss-Kruger CM 45E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2499','Pulkovo 1942 / 6-degree Gauss-Kruger CM 51E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2500','Pulkovo 1942 / 6-degree Gauss-Kruger CM 57E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2501','Pulkovo 1942 / 6-degree Gauss-Kruger CM 63E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2502','Pulkovo 1942 / 6-degree Gauss-Kruger CM 69E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2503','Pulkovo 1942 / 6-degree Gauss-Kruger CM 75E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2504','Pulkovo 1942 / 6-degree Gauss-Kruger CM 81E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2505','Pulkovo 1942 / 6-degree Gauss-Kruger CM 87E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2506','Pulkovo 1942 / 6-degree Gauss-Kruger CM 93E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2507','Pulkovo 1942 / 6-degree Gauss-Kruger CM 99E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2508','Pulkovo 1942 / 6-degree Gauss-Kruger CM 105E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2509','Pulkovo 1942 / 6-degree Gauss-Kruger CM 111E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2510','Pulkovo 1942 / 6-degree Gauss-Kruger CM 117E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2511','Pulkovo 1942 / 6-degree Gauss-Kruger CM 123E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2512','Pulkovo 1942 / 6-degree Gauss-Kruger CM 129E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2513','Pulkovo 1942 / 6-degree Gauss-Kruger CM 135E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2514','Pulkovo 1942 / 6-degree Gauss-Kruger CM 141E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2515','Pulkovo 1942 / 6-degree Gauss-Kruger CM 147E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2516','Pulkovo 1942 / 6-degree Gauss-Kruger CM 153E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2517','Pulkovo 1942 / 6-degree Gauss-Kruger CM 159E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2518','Pulkovo 1942 / 6-degree Gauss-Kruger CM 165E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2519','Pulkovo 1942 / 6-degree Gauss-Kruger CM 171E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2520','Pulkovo 1942 / 6-degree Gauss-Kruger CM 177E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2521','Pulkovo 1942 / 6-degree Gauss-Kruger CM 177W','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2522','Pulkovo 1942 / 6-degree Gauss-Kruger CM 171W','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6792','CORS96 / OCRS_BRP (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6801','CORS96 / OCRS_CGP (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6800','CORS96 / OCRS_CGP (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6805','CORS96 / OCRS_CRE (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6809','CORS96 / OCRS_CRW (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6804','CORS96 / OCRS_CRE (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6808','CORS96 / OCRS_CRW (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6813','CORS96 / OCRS_CGC (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6812','CORS96 / OCRS_CGC (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6817','CORS96 / OCRS_DFM (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6816','CORS96 / OCRS_DFM (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6821','CORS96 / OCRS_EUG (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6820','CORS96 / OCRS_EUG (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6825','CORS96 / OCRS_GPA (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6824','CORS96 / OCRS_GPA (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6829','CORS96 / OCRS_GWS (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6828','CORS96 / OCRS_GWS (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6832','CORS96 / OCRS_LDG (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6833','CORS96 / OCRS_LDG (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6835','NAD83(2011) / OCRS_LDG (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6837','CORS96 / OCRS_ONT (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6838','NAD83(2011) / OCRS_ONT (m)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4327','DGN-95 (geographic 3D)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4328','DGN-95 (geocentric)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6834','NAD83(2011) / OCRS_LDG (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6839','NAD83(2011) / OCRS_ONT (ft)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4613','Samboja','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4613','P2 Exc-T9','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4613','P2 Exc','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6840','CORS96 / OCRS_ORC (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2933','Samboja / UTM zone 50S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2933','P2 Exc-T9 / UTM zone 50S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2952','NAD83(CSRS) / SCoPQ zone 10','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2945','NAD83(CSRS) / SCoPQ zone 3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2946','NAD83(CSRS) / SCoPQ zone 4','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2947','NAD83(CSRS) / SCoPQ zone 5','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2948','NAD83(CSRS) / SCoPQ zone 6','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2949','NAD83(CSRS) / SCoPQ zone 7','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2950','NAD83(CSRS) / SCoPQ zone 8','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2951','NAD83(CSRS) / SCoPQ zone 9','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22991','Egypt 1907 / Green Belt','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26191','Merchich / Zone 1 Nord Maroc','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26192','Merchich / Zone 2 Sud Maroc','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26194','Merchich / Zone 3 Sahara Nord','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26195','Merchich / Zone 4 Sahara Sud','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4620','12th Parallel traverse','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5732','Belfast Lough height','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6836','CORS96 / OCRS_ONT (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6841','CORS96 / OCRS_ORC (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31281','Gebrauchsnetz M28','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31282','Gebrauchsnetz M31','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31283','Gebrauchsnetz M34','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31288','Bundesmeldenetz M28','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31289','Bundesmeldenetz M31','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31290','Bundesmeldenetz M34','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6842','NAD83(2011) / OCRS_ORC (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6843','NAD83(2011) / OCRS_ORC (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6844','CORS96 / OCRS_PDT (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6845','CORS96 / OCRS_PDT (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6846','NAD83(2011) / OCRS_PDT (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6848','CORS96 / OCRS_PLG (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6849','CORS96 / OCRS_PLG (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6850','NAD83(2011) / OCRS_PLG (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6851','NAD83(2011) / OCRS_PLG (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6847','NAD83(2011) / OCRS_PDT (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6852','CORS96 / OCRS_PDX (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6853','CORS96 / OCRS_PDX (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6854','NAD83(2011) / OCRS_PDX (m)','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','7707','Newlyn (Offshore) height','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6855','NAD83(2011) / OCRS_PDX (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6856','CORS96 / OCRS_SLE (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6857','CORS96 / OCRS_SLE (ft)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4346','ETRF89 (geocentric)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4346','EUREF89 (geocentric)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4638','St Pierre Miquelon 1950','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2984','RGNC 1991 / Lambert NC','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6858','NAD83(2011) / OCRS_SLE (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2985','Petrels 1972 / Terre Adelie Stereo','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2986','Perroud 1950 / Terre Adelie Stereo','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4634','MHNC72','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2982','MHNC72 / UTM zone 58S','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4339','AAD98 (3D)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4340','AAD98 (geocentric)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4345','ETRF89 (3D)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4345','EUREF89 (3D)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4356','LKS94 (geocentric)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4126','LKS94','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4355','LKS94 (3D)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4365','National Geodetic System [Argentina] (3D)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4366','National Geodetic System [Argentina] (geocentric)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4820','Samboja (Jakarta)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6859','NAD83(2011) / OCRS_SLE (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6870','KRGJSH-2010','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6862','NAD83(2011) / OCRS_SAN (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6862','NAD83(2011) / Oregon Sweet Home-Sisters zone (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6863','NAD83(2011) / OCRS_SAN (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3019','RT90 7.5 gon V 0:-15','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6798','NAD83(2011) / OCRS_BBU (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3020','RT90 5 gon V 0:-15','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3021','RT90 2.5 gon V 0:-15','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3022','RT90 0 gon 0:-15','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3023','RT90 2.5 gon O 0:-15','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3024','RT90 5 gon O 0:-15','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6790','NAD83(2011) / OCRS_BKF (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6799','NAD83(2011) / OCRS_BBU (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6797','NAD83(CORS96) / Oregon Bend-Vale zone (ft)','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5758','IGN89 height','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6797','CORS96 / OCRS_BBU (ft)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4183','Base SW','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4665','Base SW','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4665','Graciosa','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2189','Base SW / UTM zone 26N','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4664','Sao Braz','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3062','Sao Braz / UTM zone 26N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3063','Base SW / UTM zone 26N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3063','Graciosa / UTM zone 26N','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4615','Porto Santo 1936','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4663','Base SE','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2942','Base SE / UTM zone 28N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3061','Base SE / UTM zone 28N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3028','RT38 0 gon 0:-15','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3029','RT38 2.5 gon O 0:-15','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3027','RT38 2.5 gon V 0:-15','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3030','RT38 5 gon O 0:-15','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3026','RT38 5 gon V 0:-15','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3025','RT38 2.5 gon V 0:-15','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7793','RDN2008 / TM34 (E-N)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4615','Base SE','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','25884','LKS-92 / TM Baltic93','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','25884','LKS94 / TM Baltic93','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','25884','EST97 / TM Baltic93','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3300','EST92','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3301','EST97','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3059','LKS92 / Latvia TM','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4131','Indian (DMA Reduced)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3176','Indian (DMA Reduced) / TM 106 NE','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3148','Indian (DMA Reduced) / UTM zone 48N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3149','Indian (DMA Reduced) / UTM zone 49N','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4930','AAD98','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6863','NAD83(2011) / Oregon Sweet Home-Sisters zone (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6796','CORS96 / OCRS_BBU (m)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4950','LKS94 (ETRS89)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4931','AAD98','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8235','NAD83(CSRS98)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4951','LKS94 (ETRS89)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6861','NAD83(CORS96) / Oregon Sweet Home-Sisters zone (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6860','CORS96 / OCRS_SAN (m)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4265','Rome 1940','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4806','Rome 1940 (Rome)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3004','Rome 1940 / Italy zone 2','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3003','Rome 1940 / Italy zone 1','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3066','ED50 / Jordan Transverse Mercator','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2157','ETRS89 / ITM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6860','NAD83(CORS96) / Oregon Sweet Home-Sisters zone (m)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4173','ETRS89','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4269','NAD83(1986)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4170','SIRGAS','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4974','SIRGAS','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32000','SIRGAS / UTM zone 25S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31999','SIRGAS / UTM zone 24S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31998','SIRGAS / UTM zone 23S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31997','SIRGAS / UTM zone 22S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31996','SIRGAS / UTM zone 21S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31995','SIRGAS / UTM zone 20S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31994','SIRGAS / UTM zone 19S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31993','SIRGAS / UTM zone 18S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31992','SIRGAS / UTM zone 17S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31991','SIRGAS / UTM zone 22N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31990','SIRGAS / UTM zone 21N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31989','SIRGAS / UTM zone 20N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31988','SIRGAS / UTM zone 19N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31987','SIRGAS / UTM zone 18N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31986','SIRGAS / UTM zone 17N','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4975','SIRGAS','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6861','CORS96 / OCRS_SAN (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3069','NAD27 / WTM 27','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5723','Japanese Standard Levelling Datum height','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7005','Nahrwan 1934 / UTM 37N','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5787','Baltic 1980 height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5782','REDNAP height','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3072','Maine Coordinate System of 2000 East Zone','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3073','Maine Coordinate System of 2000 Central Zone','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3074','Maine Coordinate System of 2000 West Zone','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2802','Maine Coordinate System of 1983 East Zone','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2803','Maine Coordinate System of 1983 West Zone','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26783','Maine Coordinate System of 1927 East Zone','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26784','Maine Coordinate System of 1927 West Zone','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26984','Maine Coordinate System of 1983 West Zone','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26983','Maine Coordinate System of 1983 East Zone','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3078','Michigan GeoRef','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3081','NAD83 / TSMS','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3083','NAD83 / TX Albers','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3083','Texas Centric Mapping System / Albers Equal Area','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3082','Texas Centric Mapping System / Lambert Conformal','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3082','TCMS/LC','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3091','NAD83(HPGN) / Kentucky Single Zone (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3089','NAD83 / KY1Z (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3087','NAD83(HPGN) / Florida GDL Albers','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3085','NAD83(HPGN) / Texas Centric Albers Equal Area','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3084','NAD83(HPGN) / Texas Centric Lambert Conformal','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3079','NAD83(HPGN) / Michigan Oblique Mercator','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3077','NAD83(HPGN) / Maine CS2000 West','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3076','NAD83(HPGN) / Maine CS2000 Central','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3075','NAD83(HPGN) / Maine CS2000 East','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3071','NAD83(HPGN) / Wisconsin Transverse Mercator','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3070','NAD83 / WTM 83','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3088','KY1Z','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3090','KY1Z','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3090','NAD83(HPGN) / Kentucky Single Zone','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4152','NAD83(HPGN)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4956','NAD83(HPGN)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4957','NAD83(HPGN)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2195','NAD83(HPGN) / UTM zone 2S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2759','NAD83(HPGN) / Alabama East','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2760','NAD83(HPGN) / Alabama West','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2761','NAD83(HPGN) / Arizona East','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2762','NAD83(HPGN) / Arizona Central','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2763','NAD83(HPGN) / Arizona West','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2764','NAD83(HPGN) / Arkansas North','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2765','NAD83(HPGN) / Arkansas South','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2766','NAD83(HPGN) / California zone 1','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2767','NAD83(HPGN) / California zone 2','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2768','NAD83(HPGN) / California zone 3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2769','NAD83(HPGN) / California zone 4','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2770','NAD83(HPGN) / California zone 5','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2771','NAD83(HPGN) / California zone 6','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2772','NAD83(HPGN) / Colorado North','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2773','NAD83(HPGN) / Colorado Central','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2774','NAD83(HPGN) / Colorado South','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2775','NAD83(HPGN) / Connecticut','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2776','NAD83(HPGN) / Delaware','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2777','NAD83(HPGN) / Florida East','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2778','NAD83(HPGN) / Florida West','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2779','NAD83(HPGN) / Florida North','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2780','NAD83(HPGN) / Georgia East','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2781','NAD83(HPGN) / Georgia West','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2782','NAD83(HPGN) / Hawaii zone 1','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2783','NAD83(HPGN) / Hawaii zone 2','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2784','NAD83(HPGN) / Hawaii zone 3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2785','NAD83(HPGN) / Hawaii zone 4','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2786','NAD83(HPGN) / Hawaii zone 5','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2787','NAD83(HPGN) / Idaho East','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2788','NAD83(HPGN) / Idaho Central','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2789','NAD83(HPGN) / Idaho West','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2790','NAD83(HPGN) / Illinois East','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2791','NAD83(HPGN) / Illinois West','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2792','NAD83(HPGN) / Indiana East','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2793','NAD83(HPGN) / Indiana West','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2794','NAD83(HPGN) / Iowa North','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2795','NAD83(HPGN) / Iowa South','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2796','NAD83(HPGN) / Kansas North','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2797','NAD83(HPGN) / Kansas South','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2798','NAD83(HPGN) / Kentucky North','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2799','NAD83(HPGN) / Kentucky South','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2800','NAD83(HPGN) / Louisiana North','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2801','NAD83(HPGN) / Louisiana South','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2802','NAD83(HPGN) / Maine East','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2803','NAD83(HPGN) / Maine West','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2804','NAD83(HPGN) / Maryland','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2805','NAD83(HPGN) / Massachusetts Mainland','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2806','NAD83(HPGN) / Massachusetts Island','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2807','NAD83(HPGN) / Michigan North','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2808','NAD83(HPGN) / Michigan Central','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2809','NAD83(HPGN) / Michigan South','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2810','NAD83(HPGN) / Minnesota North','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2811','NAD83(HPGN) / Minnesota Central','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2812','NAD83(HPGN) / Minnesota South','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2813','NAD83(HPGN) / Mississippi East','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2814','NAD83(HPGN) / Mississippi West','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2815','NAD83(HPGN) / Missouri East','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2816','NAD83(HPGN) / Missouri Central','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2817','NAD83(HPGN) / Missouri West','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2818','NAD83(HPGN) / Montana','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2819','NAD83(HPGN) / Nebraska','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2820','NAD83(HPGN) / Nevada East','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2821','NAD83(HPGN) / Nevada Central','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2822','NAD83(HPGN) / Nevada West','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2823','NAD83(HPGN) / New Hampshire','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2824','NAD83(HPGN) / New Jersey','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2825','NAD83(HPGN) / New Mexico East','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2826','NAD83(HPGN) / New Mexico Central','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2827','NAD83(HPGN) / New Mexico West','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2828','NAD83(HPGN) / New York East','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2829','NAD83(HPGN) / New York Central','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2830','NAD83(HPGN) / New York West','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2831','NAD83(HPGN) / New York Long Island','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2832','NAD83(HPGN) / North Dakota North','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2833','NAD83(HPGN) / North Dakota South','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2834','NAD83(HPGN) / Ohio North','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2835','NAD83(HPGN) / Ohio South','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2836','NAD83(HPGN) / Oklahoma North','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2837','NAD83(HPGN) / Oklahoma South','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2838','NAD83(HPGN) / Oregon North','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2839','NAD83(HPGN) / Oregon South','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2840','NAD83(HPGN) / Rhode Island','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2841','NAD83(HPGN) / South Dakota North','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2842','NAD83(HPGN) / South Dakota South','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2843','NAD83(HPGN) / Tennessee','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2844','NAD83(HPGN) / Texas North','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2845','NAD83(HPGN) / Texas North Central','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2846','NAD83(HPGN) / Texas Central','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2847','NAD83(HPGN) / Texas South Central','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2848','NAD83(HPGN) / Texas South','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2849','NAD83(HPGN) / Utah North','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2850','NAD83(HPGN) / Utah Central','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2851','NAD83(HPGN) / Utah South','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2852','NAD83(HPGN) / Vermont','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2853','NAD83(HPGN) / Virginia North','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2854','NAD83(HPGN) / Virginia South','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2855','NAD83(HPGN) / Washington North','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2856','NAD83(HPGN) / Washington South','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2857','NAD83(HPGN) / West Virginia North','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2858','NAD83(HPGN) / West Virginia South','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2859','NAD83(HPGN) / Wisconsin North','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2860','NAD83(HPGN) / Wisconsin Central','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2861','NAD83(HPGN) / Wisconsin South','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2862','NAD83(HPGN) / Wyoming East','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2863','NAD83(HPGN) / Wyoming East Central','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2864','NAD83(HPGN) / Wyoming West Central','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2865','NAD83(HPGN) / Wyoming West','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2866','NAD83(HPGN) / Puerto Rico and Virgin Is.','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2867','NAD83(HPGN) / Arizona East (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2868','NAD83(HPGN) / Arizona Central (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2869','NAD83(HPGN) / Arizona West (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2870','NAD83(HPGN) / California zone 1 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2871','NAD83(HPGN) / California zone 2 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2872','NAD83(HPGN) / California zone 3 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2873','NAD83(HPGN) / California zone 4 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2874','NAD83(HPGN) / California zone 5 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2875','NAD83(HPGN) / California zone 6 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2876','NAD83(HPGN) / Colorado North (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2877','NAD83(HPGN) / Colorado Central (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2878','NAD83(HPGN) / Colorado South (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2879','NAD83(HPGN) / Connecticut (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2880','NAD83(HPGN) / Delaware (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2881','NAD83(HPGN) / Florida East (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2882','NAD83(HPGN) / Florida West (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2883','NAD83(HPGN) / Florida North (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2884','NAD83(HPGN) / Georgia East (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2885','NAD83(HPGN) / Georgia West (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2886','NAD83(HPGN) / Idaho East (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2887','NAD83(HPGN) / Idaho Central (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2888','NAD83(HPGN) / Idaho West (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2891','NAD83(HPGN) / Kentucky North (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2892','NAD83(HPGN) / Kentucky South (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2893','NAD83(HPGN) / Maryland (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2894','NAD83(HPGN) / Massachusetts Mainland (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2895','NAD83(HPGN) / Massachusetts Island (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2896','NAD83(HPGN) / Michigan North (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2897','NAD83(HPGN) / Michigan Central (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2898','NAD83(HPGN) / Michigan South (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2899','NAD83(HPGN) / Mississippi East (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2900','NAD83(HPGN) / Mississippi West (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2901','NAD83(HPGN) / Montana (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2902','NAD83(HPGN) / New Mexico East (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2903','NAD83(HPGN) / New Mexico Central (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2904','NAD83(HPGN) / New Mexico West (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2905','NAD83(HPGN) / New York East (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2906','NAD83(HPGN) / New York Central (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2907','NAD83(HPGN) / New York West (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2908','NAD83(HPGN) / New York Long Island (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2909','NAD83(HPGN) / North Dakota North (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2910','NAD83(HPGN) / North Dakota South (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2911','NAD83(HPGN) / Oklahoma North (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2912','NAD83(HPGN) / Oklahoma South (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2913','NAD83(HPGN) / Oregon North (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2914','NAD83(HPGN) / Oregon South (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2915','NAD83(HPGN) / Tennessee (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2916','NAD83(HPGN) / Texas North (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2917','NAD83(HPGN) / Texas North Central (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2918','NAD83(HPGN) / Texas Central (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2919','NAD83(HPGN) / Texas South Central (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2920','NAD83(HPGN) / Texas South (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2921','NAD83(HPGN) / Utah North (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2922','NAD83(HPGN) / Utah Central (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2923','NAD83(HPGN) / Utah South (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2924','NAD83(HPGN) / Virginia North (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2925','NAD83(HPGN) / Virginia South (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2926','NAD83(HPGN) / Washington North (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2927','NAD83(HPGN) / Washington South (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2928','NAD83(HPGN) / Wisconsin North (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2929','NAD83(HPGN) / Wisconsin Central (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2930','NAD83(HPGN) / Wisconsin South (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2967','NAD83(HPGN) / Indiana East (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2968','NAD83(HPGN) / Indiana West (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2993','NAD83(HPGN) / LCC (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2994','NAD83(HPGN) / Oregon GIC Lambert (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3088','NAD83 / Kentucky Single Zone (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6879','NAD83(2011) / Wisconsin Central (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3089','KY1Z','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3091','NAD83(HARN) / KY1Z (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4622','Sainte Anne','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4625','Fort Desaix','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4626','Piton des Neiges','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2970','Sainte Anne / UTM zn 20N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2973','Fort Desaix / UTM zn 20N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2990','Piton des Neiges / TM Reunion','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5757','IGN 1988 height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5756','IGN 1987','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5758','IGN 1989 height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5793','IGN 1950 height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5794','IGN 1955 height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5795','IGN 1951 height','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3106','Gulshan 303 / Bangladesh UTM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3108','ETRS89 / New Guernsey Grid','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3109','ETRS89 / JTM','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5713','CVD28 height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5713','Canadian Vertical Datum of 1928 height','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6879','NAD83(2011) / WI C (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3110','Vicgrid','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4994','New Luzon','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4683','New Luzon','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4995','New Luzon','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3125','New Luzon / Philippines zone 5','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3124','New Luzon / Philippines zone 4','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3123','New Luzon / Philippines zone 3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3122','New Luzon / Philippines zone 2','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3121','New Luzon / Philippines zone 1','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21897','Bogota / Colombia Bogota','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21899','Bogota / Colombia 6E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21898','Bogota / Colombia 3E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21896','Bogota / Colombia 3W','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6880','NAD83(2011) / NE (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3034','ETRS - LCC','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3035','ETRS - LAEA','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3038','ETRS - TM26','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3039','ETRS - TM27','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3040','ETRS - TM28','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3041','ETRS - TM29','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3042','ETRS - TM30','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3043','ETRS - TM31','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3044','ETRS - TM32','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3045','ETRS - TM33','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3046','ETRS - TM34','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3047','ETRS - TM35','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3048','ETRS - TM36','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3049','ETRS - TM37','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3050','ETRS - TM38','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3051','ETRS - TM39','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4936','ETRS89 / (X, Y, Z)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4937','ETRS89-LatLonh','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5730','EVRF_AMST / NH','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6884','NAD83(CORS96) / Oregon North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2462','AL_ALB87 / TM_6','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5777','AL_DUR / NOH','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5778','AT_TRIE / NOH','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31281','AT_MGI / AT_TM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31282','AT_MGI / AT_TM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31283','AT_MGI / AT_TM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31370','(BE_BD72 / LAMB72 - see alias remarks)','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5786','BG_KRON / NH','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31275','HR_HDKS / HR_TM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31276','HR_HDKS / HR_TM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2065','CZ_S-JTSK / KROVAK','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2065','SK_S-JTSK / KROVAK','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','23032','(DK_ED50 / UTM - see alias remarks)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','23033','(DK_ED50 / UTM - see alias remarks)','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5733','DK_DK10 / OH','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3301','EE_L-EST97 / EST_LAMB','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5705','EE_KRON / NH','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5705','Baltic height','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2393','FI_KKJ / FI_TM','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5717','FI_HELS / OH','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2192','(FR_ED50 / EUROLAMB - see alias remarks)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','27562','(FR_NTF / FR_LAMB - see alias remarks)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','27564','(FR_NTF / FR_LAMB - see alias remarks)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','27561','(FR_NTF / FR_LAMB - see alias remarks)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','27563','(FR_NTF / FR_LAMB - see alias remarks)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','27572','(FR_NTF / FR_LAMB - see alias remarks)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2154','(FR_RGF93 / LAMB93 - see alias remarks)','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5720','FR_MARS / NH','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31466','DE_DHDN / GK_3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31467','DE_DHDN / GK_3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31468','DE_DHDN / GK_3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31469','DE_DHDN / GK_3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3398','DE_RD/83 / GK_3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3399','DE_RD/83 / GK_3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3396','DE_PD/83 / GK_3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3397','DE_PD/83 / GK_3','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5785','DE_KRON / NH','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5783','DE_AMST / NH','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5784','DE_AMST / NOH','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2399','DE_42/83 / GK_3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2398','DE_42/83 / GK_3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2397','DE_42/83 / GK_3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3045','DE_ETRS89 / UTM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','23030','(GI_ED50 / UTM - see alias remarks)','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5701','GB_NEWL / OH','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2100','(GR_GGRS87 / GR_TM - see alias remarks)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','27700','(GB_OSGB36 / NATIONALGRID - see alias remarks)','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5787','HU_KRON / NH','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5731','IE_MALH / OH','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','29902','(IE_IRELAND65 / IRELAND75_IRISHGRID - see alias remarks)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','29902','(NI_IRELAND65 / IRELAND75_IRISHGRID - see alias remarks)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','23033','(IT_ED50 / UTM - see alias remarks)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','23032','(IT_ED50 / UTM - see alias remarks)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3003','(IT_ROMA40 / EAST_WEST - see alias remarks)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3004','(IT_ROMA40 / EAST_WEST - see alias remarks)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3059','LV_LKS-92 / LV_TM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2600','LT_LKS94 / LT_TM','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5774','LU_AMST / OH','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2169','LU_LUREF / LU_TM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28992','(NL_RD / DUTCH_ST - see alias remarks)','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5709','NL_AMST / OH','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3044','NO_ETRS89 / UTM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3045','NO_ETRS89 / UTM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3047','NO_ETRS89 / UTM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','27391','NO_NGO1948 / NO_TM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','27392','NO_NGO1948 / NO_TM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','27393','NO_NGO1948 / NO_TM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','27394','NO_NGO1948 / NO_TM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','27395','NO_NGO1948 / NO_TM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','27396','NO_NGO1948 / NO_TM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','27397','NO_NGO1948 / NO_TM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','27398','NO_NGO1948 / NO_TM','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5776','NO_TREG / NOH','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2180','PL_EUREF89 / 1992','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2179','PL_EUREF89 / 2000','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2178','PL_EUREF89 / 2000','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2177','PL_EUREF89 / 2000','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2176','PL_EUREF89 / 2000','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2171','PL_42/58 / 1965','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2172','PL_42/58 / 1965','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2173','PL_42/58 / 1965','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2174','PL_42/58 / 1965','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2175','PL_42/58 / 1965','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5780','PT_CASC / OH','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2942','(PT_MAD / UTM - see alias remarks)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','27492','PT_D73 / TM_D73','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2190','(PT_AZO_ORIE / UTM - see alias remarks)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6884','CORS96 / OR N (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2189','(PT_AZO_CENT / UTM - see alias remarks)','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5779','SI_TRIE / NOH','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2170','SI_D48 / SI_TM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2170','D48 / Slovenia Grid','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','23028','(ES_ED50 / UTM - see alias remarks)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','23029','(ES_ED50 / UTM - see alias remarks)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','23030','(ES_ED50 / UTM - see alias remarks)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','23031','(ES_ED50 / UTM - see alias remarks)','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5718','SE_AMST / NH','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3021','SE_RT90 / SE_TM','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5728','CH_MARS / UNCOR','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5729','CH_MARS / OH','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2056','(CH_CH1903+ / CH_PROJECTION+ - see alias remarks)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21781','(CH_CH1903 / CH_PROJECTION - see alias remarks)','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5775','TR_ANT / OH','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2319','TR_ED50 / TR_TM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2320','TR_ED50 / TR_TM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2321','TR_ED50 / TR_TM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2322','TR_ED50 / TR_TM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2323','TR_ED50 / TR_TM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2324','TR_ED50 / TR_TM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2325','TR_ED50 / TR_TM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','23035','(TR_ED50 / UTM - see alias remarks)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','23036','(TR_ED50 / UTM - see alias remarks)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','23037','(TR_ED50 / UTM - see alias remarks)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','23038','(TR_ED50 / UTM - see alias remarks)','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5782','ES_ALIC / OH','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4692','MOP 1983','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4629','Tahaa','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4628','Tahiti','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3306','MOP 1983 / UTM zone 5S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2977','Tahaa / UTM zone 5S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2976','Tahiti / UTM zone 6S','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4688','MHEFO 55','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3303','MHEFO 55 / UTM zone 7S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3309','NAD27 / California (Teale) Albers','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3310','NAD83 / California (Teale) Albers','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3311','NAD83(HARN) / California (Teale) Albers','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4943','ETRS89','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4942','ETRS89','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4697','IGC 1962','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6886','NAD83(CORS96) / Oregon South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6886','CORS96 / OR S (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3120','PL_42/58 / 1965','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3328','System GUGiK-80','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3329','System 1942/15 (3)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3330','System 1942/18 (3)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3331','System 1942/21 (3)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3332','System 1942/24 (3)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3333','System 1942/15 (6)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3334','System 1942/21 (6)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3335','System 1942/27 (6)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2172','System 1965 zone II','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2173','System 1965 zone III','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2174','System 1965 zone IV','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2175','System 1965 zone V','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2176','PL-2000/15','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2177','PL-2000/18','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2178','PL-2000/21','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2179','PL-2000/24','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2180','PL-1992','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3120','System 1965 zone I','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3346','LKS94','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3346','LT_LKS94 / LT_TM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3346','Lietuvos Koordinaciu Sistema 1994','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4701','Bas Congo 1955','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3339','Bas Congo 1955 / Congo TM zone 12','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3340','Bas Congo 1955 / Congo TM zone 14','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3341','Bas Congo 1955 / Congo TM zone 16','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3342','Bas Congo 1955 / UTM zone 33S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3347','NAD83 / STC Lambert','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3348','NAD83(CSRS) / STC LCC','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3348','NAD83 / STC Lambert','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6885','NAD83(CORS96) / OR N (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6887','CORS96 / OR S (ft)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4616','Marco Astro','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4616','Selvagem Grande 1938','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2943','Selvagem Grande 1938 / UTM zone 28N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6799','NAD83(2011) / Oregon Bend-Vale zone (ft)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4259','Mhast','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','25932','Mhast / UTM zone 32S','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4705','Mhast','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4704','Mhast','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3354','Mhast / UTM zone 32S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3353','Mhast / UTM zone 32S','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4699','Le Pouce (Mauritius 94)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4699','Le Pouce (Mauritius PN 94)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3337','Le Pouce (Mauritius 94) / Mauritius Grid','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3337','Le Pouce (Mauritius PN 94) / Mauritius Grid','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4700','Mauritanian Mining Cadastre 1999','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3367','Mauritanian Mining Cadastre 1999 / UTM zone 28N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3368','Mauritanian Mining Cadastre 1999 / UTM zone 29N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3369','Mauritanian Mining Cadastre 1999 / UTM zone 30N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6798','NAD83(2011) / Oregon Bend-Vale zone (m)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4725','Johnston Atoll 1961','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2987','St. Pierre et Miquelon 1950 / UTM zone 21N','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4738','HK63','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4739','HK63(67)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','7415','RDNAP','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4690','IGN79 Tahiti','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4639','Uvea SHOM 1978','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3355','S-650 TL / Red Belt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4706','S-650 TL','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2004','Montserrat 58 / British West Indies Grid','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4604','Montserrat 58','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3386','KKJ / Basic Coordinate System zone 0','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3387','KKJ / Basic Coordinate System zone 5','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4669','LKS94 (ETRS89)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32164','NAD83 / UTM zone 14N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32165','NAD83 / UTM zone 15N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32166','NAD83 / UTM zone 16N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32167','NAD83 / UTM zone 17N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32664','WGS 84 / UTM zone 14N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32665','WGS 84 / UTM zone 15N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32666','WGS 84 / UTM zone 16N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32667','WGS 84 / UTM zone 17N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4720','FGD 1986','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3143','FGD 1986 / Fiji Map Grid','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3144','fk54','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3144','Faroe Cadastre 1954','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3145','fke','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3173','Faroe Cadastre 1989','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4752','Viti Levu 1916','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3140','Viti Levu 1916 / Viti Levu Grid','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4748','Vanua Levu 1917','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3139','Vanua Levu 1917 / Vanua Levu Grid','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3400','NAD83 / 10TM AEP Forest','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3401','NAD83 / 10TM AEP Resource','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3403','NAD83 / 10TM AEP Resource','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3402','NAD83 / 10TM AEP Forest','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3854','ST74 0 gon 65:-1','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3163','Lambert NC91','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4298','Timbalai 1968','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4298','BT68','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','29849','BT68 / UTM zone 49N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','29850','BT68 / UTM zone 50N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','29849','Timbalai 1968 / UTM zone 49N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','29850','Timbalai 1968 / UTM zone 50N','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4245','MRT68','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','24500','MRT68 / Singapore Grid','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','24547','MRT68 / UTM zone 47N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','24548','MRT68 / UTM zone 48N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3169','RGNC / UTM zone 57S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3170','RGNC / UTM zone 58S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3171','RGNC / UTM zone 59S','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4714','Bellevue (IGN)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4730','Santo (DOS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3143','Fiji 1986 / FMG','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3153','NAD83 / BC Albers','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3173','fk89','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4753','FD54a','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3311','NAD83(HPGN) / California (Teale) Albers','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3358','NAD83(HPGN) / North Carolina','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3359','NAD83(HPGN) / North Carolina (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3360','NAD83(HPGN) / South Carolina','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3361','NAD83(HPGN) / South Carolina (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3362','NAD83(HPGN) / Pennsylvania North','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3363','NAD83(HPGN) / Pennsylvania North (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3364','NAD83(HPGN) / Pennsylvania South','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3365','NAD83(HPGN) / Pennsylvania South (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3075','Maine Coordinate System of 2000 East Zone','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3076','Maine Coordinate System of 2000 Central Zone','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3077','Maine Coordinate System of 2000 West Zone','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3079','Michigan GeoRef','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3085','TCMS/AEA','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3085','Texas Centric Mapping System / Albers Equal Area','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3084','Texas Centric Mapping System / Lambert Conformal','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3084','TCMS/ LC','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4755','IGD95','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4898','IGD95','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4897','IGD95','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','23866','IGD95 / UTM zone 46N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','23867','IGD95 / UTM zone 47N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','23868','IGD95 / UTM zone 48N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','23869','IGD95 / UTM zone 49N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','23870','IGD95 / UTM zone 50N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','23871','IGD95 / UTM zone 51N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','23872','IGD95 / UTM zone 52N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','23877','IGD95 / UTM zone 47S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','23878','IGD95 / UTM zone 48S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','23879','IGD95 / UTM zone 49S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','23880','IGD95 / UTM zone 50S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','23881','IGD95 / UTM zone 51S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','23882','IGD95 / UTM zone 52S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','23883','IGD95 / UTM zone 53S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','23884','IGD95 / UTM zone 54S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3404','NAD83(HPGN) / North Carolina (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3414','SVY21 plane coordinate system','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3174','NAD83 / GLGIS Albers (basin)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3175','NAD83 / GLGIS Albers (basin+SLS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9373','ETRS89 / MML07 SnakeGrid','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3415','WGS 72BE / SCS Lambert','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3425','NAD83(HPGN) / Iowa North (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3426','NAD83(HPGN) / Iowa South (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3427','NAD83(HPGN) / Kansas North (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3428','NAD83(HPGN) / Kansas South (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3429','NAD83(HPGN) / Nevada East (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3430','NAD83(HPGN) / Nevada Central (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3431','NAD83(HPGN) / Nevada West (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3432','NAD83(HPGN) / New Jersey (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3441','NAD83(HPGN) / Arkansas North (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3442','NAD83(HPGN) / Arkansas South (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3443','NAD83(HPGN) / Illinois East (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3444','NAD83(HPGN) / Illinois West (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3445','NAD83(HPGN) / New Hampshire (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3446','NAD83(HPGN) / Rhode Island (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3447','ETRS89 / Lambert 2005','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4746','DHDN','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3397','PD/83 / Gauss-Kruger zone 4','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4745','DHDN','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3398','RD/83 / Gauss-Kruger zone 4','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3399','RD/83 / Gauss-Kruger zone 5','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','29702','Tananarive / Laborde app','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3448','WGS 84 / Jamaica Metric Grid 2001','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32199','NAD83 / Louisiana Offshore (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3072','NAD83 / Maine CS2000 East (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3074','NAD83 / Maine CS2000 West (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2759','NAD83(HARN) / Alabama East (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2760','NAD83(HARN) / Alabama West (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2762','NAD83(HARN) / Arizona Central (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2761','NAD83(HARN) / Arizona East (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2763','NAD83(HARN) / Arizona West (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2764','NAD83(HARN) / Arkansas North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2765','NAD83(HARN) / Arkansas South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2766','NAD83(HARN) / California zone 1 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2767','NAD83(HARN) / California zone 2 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2768','NAD83(HARN) / California zone 3 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2769','NAD83(HARN) / California zone 4 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2770','NAD83(HARN) / California zone 5 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2771','NAD83(HARN) / California zone 6 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2773','NAD83(HARN) / Colorado Central (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2772','NAD83(HARN) / Colorado North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2774','NAD83(HARN) / Colorado South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2775','NAD83(HARN) / Connecticut (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2776','NAD83(HARN) / Delaware (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2777','NAD83(HARN) / Florida East (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2779','NAD83(HARN) / Florida North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2778','NAD83(HARN) / Florida West (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2780','NAD83(HARN) / Georgia East (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2781','NAD83(HARN) / Georgia West (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2782','NAD83(HARN) / Hawaii zone 1 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2783','NAD83(HARN) / Hawaii zone 2 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2784','NAD83(HARN) / Hawaii zone 3 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2785','NAD83(HARN) / Hawaii zone 4 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2786','NAD83(HARN) / Hawaii zone 5 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2788','NAD83(HARN) / Idaho Central (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2787','NAD83(HARN) / Idaho East (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2789','NAD83(HARN) / Idaho West (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2790','NAD83(HARN) / IL E (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2791','NAD83(HARN) / Illinois West (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2792','NAD83(HARN) / Indiana East (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2793','NAD83(HARN) / Indiana West (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2794','NAD83(HARN) / Iowa North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2795','NAD83(HARN) / Iowa South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2796','NAD83(HARN) / Kansas North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2797','NAD83(HARN) / Kansas South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2798','NAD83(HARN) / Kentucky North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2799','NAD83(HARN) / Kentucky South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3090','NAD83(HARN) / Kentucky Single Zone (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2800','NAD83(HARN) / Louisiana North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3456','NAD83(HPGN) / Louisiana North (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2801','NAD83(HARN) / Louisiana South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3457','NAD83(HPGN) / Louisiana South (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3075','NAD83(HARN) / Maine CS2000 East (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3077','NAD83(HARN) / Maine CS2000 West (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6867','CORS96 / OR LCC (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6868','CORS96 / OR GIC Lam (ft)','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','8358','Bpv depth','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6785','CORS96 / OCRS_BKE (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6789','CORS96 / OCRS_BKF (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6784','CORS96 / OCRS_BKE (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6788','CORS96 / OCRS_BKF (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6793','CORS96 / OCRS_BRP (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6915','SE Island / UTM zone 40N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2802','NAD83(HARN) / Maine East (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2803','NAD83(HARN) / Maine West (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2804','NAD83(HARN) / Maryland (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2806','NAD83(HARN) / Massachusetts Island (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2805','NAD83(HARN) / Massachusetts Mainland (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2808','NAD83(HARN) / Michigan Central (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2807','NAD83(HARN) / Michigan North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2809','NAD83(HARN) / Michigan South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2811','NAD83(HARN) / Minnesota Central (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2810','NAD83(HARN) / Minnesota North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2812','NAD83(HARN) / Minnesota South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2813','NAD83(HARN) / Mississippi East (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2814','NAD83(HARN) / Mississippi West (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2816','NAD83(HARN) / Missouri Central (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2815','NAD83(HARN) / Missouri East (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2817','NAD83(HARN) / Missouri West (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2818','NAD83(HARN) / Montana (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2819','NAD83(HARN) / Nebraska (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2821','NAD83(HARN) / Nevada Central (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2820','NAD83(HARN) / Nevada East (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2822','NAD83(HARN) / Nevada West (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2823','NAD83(HARN) / New Hampshire (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2824','NAD83(HARN) / New Jersey (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2826','NAD83(HARN) / New Mexico Central (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2825','NAD83(HARN) / New Mexico East (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2827','NAD83(HARN) / New Mexico West (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2829','NAD83(HARN) / New York Central (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2828','NAD83(HARN) / New York East (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2831','NAD83(HARN) / New York Long Island (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2830','NAD83(HARN) / New York West (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3358','NAD83(HARN) / North Carolina (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2832','NAD83(HARN) / North Dakota North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2833','NAD83(HARN) / North Dakota South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2834','NAD83(HARN) / Ohio North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2835','NAD83(HARN) / Ohio South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2836','NAD83(HARN) / Oklahoma North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2837','NAD83(HARN) / Oklahoma South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2838','NAD83(HARN) / Oregon North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2839','NAD83(HARN) / Oregon South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3362','NAD83(HARN) / Pennsylvania North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3364','NAD83(HARN) / Pennsylvania South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2840','NAD83(HARN) / Rhode Island (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3360','NAD83(HARN) / South Carolina (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2841','NAD83(HARN) / South Dakota North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3458','NAD83(HPGN) / South Dakota North (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2842','NAD83(HARN) / South Dakota South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3459','NAD83(HPGN) / South Dakota South (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2843','NAD83(HARN) / Tennessee (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2846','NAD83(HARN) / Texas Central (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2844','NAD83(HARN) / Texas North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2845','NAD83(HARN) / Texas North Central (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2848','NAD83(HARN) / Texas South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2847','NAD83(HARN) / Texas South Central (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2850','NAD83(HARN) / Utah Central (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2849','NAD83(HARN) / Utah North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2851','NAD83(HARN) / Utah South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2852','NAD83(HARN) / Vermont (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2853','NAD83(HARN) / Virginia North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2854','NAD83(HARN) / Virginia South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2855','NAD83(HARN) / Washington North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2856','NAD83(HARN) / Washington South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2857','NAD83(HARN) / West Virginia North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2858','NAD83(HARN) / West Virginia South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2860','NAD83(HARN) / Wisconsin Central (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2859','NAD83(HARN) / Wisconsin North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2861','NAD83(HARN) / Wisconsin South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2862','NAD83(HARN) / Wyoming East (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2863','NAD83(HARN) / Wyoming East Central (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2865','NAD83(HARN) / Wyoming West (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2864','NAD83(HARN) / Wyoming West Central (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3568','NAD83(HPGN) / Utah North (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3569','NAD83(HPGN) / Utah Central (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3570','NAD83(HPGN) / Utah South (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3448','JAD2001 / Jamaica Metric Grid 2001','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','9374','ETRS89 / MML07 SnakeGrid + Newlyn height','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3038','ETRF89 / TM26','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3039','ETRF89 / TM27','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3050','ETRF89 / TM38','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3051','ETRF89 / TM39','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6922','NAD83 / Kansas LCC (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6924','NAD83(2011) / Kansas LCC (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3396','PD/83 / Gauss-Kruger zone 3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3463','Maine Coordinate System of 2000 Central Zone','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3463','NAD83 / Maine CS2000 Central (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3464','Maine Coordinate System of 2000 Central Zone','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3464','NAD83(HARN) / Maine CS2000 Central (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3464','NAD83(HPGN) / Maine CS2000 Central','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22275','South African Coordinate System zone 15','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22277','South African Coordinate System zone 17','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22279','South African Coordinate System zone 19','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22281','South African Coordinate System zone 21','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22283','South African Coordinate System zone 23','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22285','South African Coordinate System zone 25','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22287','South African Coordinate System zone 27','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22289','South African Coordinate System zone 29','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22291','South African Coordinate System zone 31','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22293','South African Coordinate System zone 33','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','29371','South West African Coord. System zone 11','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','29373','South West African Coord. System zone 13','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','29375','South West African Coord. System zone 15','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','29377','South West African Coord. System zone 17','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','29379','South West African Coord. System zone 19','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','29381','South West African Coord. System zone 21','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','29383','South West African Coord. System zone 23','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','29385','South West African Coord. System zone 25','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3465','NAD83(NSRS2007) / Alabama East (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3466','NAD83(NSRS2007) / Alabama West (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3468','NAD83(NSRS2007) / Alaska zone 1 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3477','NAD83(NSRS2007) / Alaska zone 10 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3469','NAD83(NSRS2007) / Alaska zone 2 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3470','NAD83(NSRS2007) / Alaska zone 3 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3471','NAD83(NSRS2007) / Alaska zone 4 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3472','NAD83(NSRS2007) / Alaska zone 5 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3473','NAD83(NSRS2007) / Alaska zone 6 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3474','NAD83(NSRS2007) / Alaska zone 7 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3475','NAD83(NSRS2007) / Alaska zone 8 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3476','NAD83(NSRS2007) / Alaska zone 9 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3478','NAD83(NSRS2007) / Arizona Central (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3480','NAD83(NSRS2007) / Arizona East (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3482','NAD83(NSRS2007) / Arizona West (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3484','NAD83(NSRS2007) / Arkansas North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3486','NAD83(NSRS2007) / Arkansas South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3489','NAD83(NSRS2007) / California zone 1 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3491','NAD83(NSRS2007) / California zone 2 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3493','NAD83(NSRS2007) / California zone 3 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3495','NAD83(NSRS2007) / California zone 4 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3497','NAD83(NSRS2007) / California zone 5 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3499','NAD83(NSRS2007) / California zone 6 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3501','NAD83(NSRS2007) / Colorado Central (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3503','NAD83(NSRS2007) / Colorado North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3505','NAD83(NSRS2007) / Colorado South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3507','NAD83(NSRS2007) / Connecticut (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3509','NAD83(NSRS2007) / Delaware (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3511','NAD83(NSRS2007) / Florida East (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3514','NAD83(NSRS2007) / Florida North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3516','NAD83(NSRS2007) / Florida West (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3518','NAD83(NSRS2007) / Georgia East (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3520','NAD83(NSRS2007) / Georgia West (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3522','NAD83(NSRS2007) / Idaho Central (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3524','NAD83(NSRS2007) / Idaho East (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3526','NAD83(NSRS2007) / Idaho West (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3528','NAD83(NSRS2007) / Illinois East (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3530','NAD83(NSRS2007) / Illinois West (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3532','NAD83(NSRS2007) / Indiana East (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3534','NAD83(NSRS2007) / Indiana West (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3536','NAD83(NSRS2007) / Iowa North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3538','NAD83(NSRS2007) / Iowa South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3540','NAD83(NSRS2007) / Kansas North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3542','NAD83(NSRS2007) / Kansas South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3544','NAD83(NSRS2007) / Kentucky North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3546','NAD83(NSRS2007) / Kentucky Single Zone (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3548','NAD83(NSRS2007) / Kentucky South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3550','NAD83(NSRS2007) / Louisiana North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3552','NAD83(NSRS2007) / Louisiana South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3554','NAD83(NSRS2007) / Maine CS2000 Central (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3555','NAD83(NSRS2007) / Maine CS2000 East (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3556','NAD83(NSRS2007) / Maine CS2000 West (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3557','NAD83(NSRS2007) / Maine CS83 East (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3558','NAD83(NSRS2007) / Maine West (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3559','NAD83(NSRS2007) / Maryland (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3583','NAD83(NSRS2007) / Massachusetts Island (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3585','NAD83(NSRS2007) / Massachusetts Mainland (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3587','NAD83(NSRS2007) / Michigan Central (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3589','NAD83(NSRS2007) / Michigan North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3592','NAD83(NSRS2007) / Michigan South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3594','NAD83(NSRS2007) / Minnesota Central (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3595','NAD83(NSRS2007) / Minnesota North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3596','NAD83(NSRS2007) / Minnesota South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3597','NAD83(NSRS2007) / Mississippi East (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3599','NAD83(NSRS2007) / Mississippi West (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3601','NAD83(NSRS2007) / Missouri Central (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3602','NAD83(NSRS2007) / Missouri East (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3603','NAD83(NSRS2007) / Missouri West (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3604','NAD83(NSRS2007) / Montana (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3606','NAD83(NSRS2007) / Nebraska (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3607','NAD83(NSRS2007) / Nevada Central (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3609','NAD83(NSRS2007) / Nevada East (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3611','NAD83(NSRS2007) / Nevada West (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3613','NAD83(NSRS2007) / New Hampshire (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3615','NAD83(NSRS2007) / New Jersey (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3617','NAD83(NSRS2007) / New Mexico Central (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3619','NAD83(NSRS2007) / New Mexico East (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3621','NAD83(NSRS2007) / New Mexico West (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3623','NAD83(NSRS2007) / New York Central (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3625','NAD83(NSRS2007) / New York East (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3627','NAD83(NSRS2007) / New York Long Island (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3629','NAD83(NSRS2007) / New York West (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3631','NAD83(NSRS2007) / North Carolina (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3633','NAD83(NSRS2007) / North Dakota North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3635','NAD83(NSRS2007) / North Dakota South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3637','NAD83(NSRS2007) / Ohio North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3638','NAD83(NSRS2007) / Ohio South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3639','NAD83(NSRS2007) / Oklahoma North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3641','NAD83(NSRS2007) / Oklahoma South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3643','NAD83(2007) / Oregon LCC (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3645','NAD83(NSRS2007) / Oregon North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3647','NAD83(NSRS2007) / Oregon South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3649','NAD83(NSRS2007) / Pennsylvania North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3651','NAD83(NSRS2007) / Pennsylvania South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3653','NAD83(NSRS2007) / Rhode Island (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3655','NAD83(NSRS2007) / South Carolina (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3657','NAD83(NSRS2007) / South Dakota North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3659','NAD83(NSRS2007) / South Dakota South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3661','NAD83(NSRS2007) / Tennessee (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3663','NAD83(NSRS2007) / Texas Central (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3667','NAD83(NSRS2007) / Texas North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3669','NAD83(NSRS2007) / Texas North Central (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3671','NAD83(NSRS2007) / Texas South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3673','NAD83(NSRS2007) / Texas South Central (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3675','NAD83(NSRS2007) / Utah Central (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3678','NAD83(NSRS2007) / Utah North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3681','NAD83(NSRS2007) / Utah South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3684','NAD83(NSRS2007) / Vermont (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3685','NAD83(NSRS2007) / Virginia North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3687','NAD83(NSRS2007) / Virginia South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3689','NAD83(NSRS2007) / Washington North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3691','NAD83(NSRS2007) / Washington South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3695','NAD83(NSRS2007) / Wisconsin Central (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3697','NAD83(NSRS2007) / Wisconsin North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3699','NAD83(NSRS2007) / Wisconsin South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3702','NAD83(NSRS2007) / Wyoming East (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3703','NAD83(NSRS2007) / Wyoming East Central (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3705','NAD83(NSRS2007) / Wyoming West (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3704','NAD83(NSRS2007) / Wyoming West Central (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3693','NAD83(NSRS2007) / West Virginia North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3694','NAD83(NSRS2007) / West Virginia South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3727','Piton des Neiges / TM Reunion','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3740','NAD83(HPGN) / UTM zone 10N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3741','NAD83(HPGN) / UTM zone 11N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3742','NAD83(HPGN) / UTM zone 12N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3743','NAD83(HPGN) / UTM zone 13N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3744','NAD83(HPGN) / UTM zone 14N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3745','NAD83(HPGN) / UTM zone 15N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3746','NAD83(HPGN) / UTM zone 16N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3747','NAD83(HPGN) / UTM zone 17N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3748','NAD83(HPGN) / UTM zone 18N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3749','NAD83(HPGN) / UTM zone 19N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3750','NAD83(HPGN) / UTM zone 4N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3751','NAD83(HPGN) / UTM zone 5N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3760','NAD83(HPGN) / Hawaii zone 3 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3546','KY1Z','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3547','NAD83(NSRS) / KY1Z (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3547','KY1Z','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3447','BE_ETRS89 / LB05','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3765','HTRS96 / TM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3766','HTRS96 / LCC','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3770','BNG2000','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3769','Bermuda National Grid','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26825','NAD83(HPGN) / Maine CS83 East (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26826','NAD83(HPGN) / Maine CS83 West (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26830','NAD83(HPGN) / Minnesota North (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26831','NAD83(HPGN) / Minnesota Central (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26832','NAD83(HPGN) / Minnesota South (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26833','NAD83(HPGN) / Nebraska (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26834','NAD83(HPGN) / West Virginia North (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26835','NAD83(HPGN) / West Virginia South (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26826','NAD83(HARN) / Maine CS83 West (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26825','NAD83(HARN) / Maine CS83 East (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26815','NAD83 / Maine CS83 West (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26814','NAD83 / Maine CS83 East (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3554','Maine Coordinate System of 2000 Central Zone','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3555','Maine Coordinate System of 2000 East Zone','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3556','Maine Coordinate System of 2000 West Zone','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3557','Maine Coordinate System of 1983 East Zone','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3558','Maine Coordinate System of 1983 West Zone','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26837','NAD83(NSRS2007) / Maine CS83 West (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26836','NAD83(NSRS2007) / Maine CS83 East (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5612','Baltic depth','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8518','NAD83(2011) / KS RCS zone 1 Goodland','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5612','Kronstadt 1977 depth','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4274','D73','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4803','DLx','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3771','NAD27 / Alberta 3TM ref merid 111','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3772','NAD27 / Alberta 3TM ref merid 114','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3773','NAD27 / Alberta 3TM ref merid 117','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3774','NAD27 / Alberta 3TM ref merid 120','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3775','NAD83 / Alberta 3TM ref merid 111','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3776','NAD83 / Alberta 3TM ref merid 114','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3777','NAD83 / Alberta 3TM ref merid 117','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3778','NAD83 / Alberta 3TM ref merid 120','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3779','NAD83(CSRS) / Alberta 3TM ref merid 111','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3780','NAD83(CSRS) / Alberta 3TM ref merid 114','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3781','NAD83(CSRS) / Alberta 3TM ref merid 117','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3782','NAD83(CSRS) / Alberta 3TM ref merid 120','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3787','D48 / GK','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3788','NZGD2000 / AKTM2000','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3789','NZGD2000 / CATM2000','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3790','NZGD2000 / AITM2000','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3791','NZGD2000 / RITM2000','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3793','NZGD2000 / CITM2000','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4673','CI1979','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4672','CI1971','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2193','NZGD2000 / New Zealand Transverse Mercator','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2105','NZGD2000 / Mount Eden Circuit 2000','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2106','NZGD2000 / Bay of Plenty Circuit 2000','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2107','NZGD2000 / Poverty Bay Circuit 2000','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2108','NZGD2000 / Hawkes Bay Circuit 2000','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2109','NZGD2000 / Taranaki Circuit 2000','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2110','NZGD2000 / Turhirangi Circuit 2000','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2111','NZGD2000 / Wanganui Circuit 2000','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2112','NZGD2000 / Wairarapa Circuit 2000','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2113','NZGD2000 / Wellington Circuit 2000','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2114','NZGD2000 / Collingwood Circuit 2000','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2115','NZGD2000 / Nelson Circuit 2000','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2116','NZGD2000 / Karamea Circuit 2000','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2117','NZGD2000 / Buller Circuit 2000','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2118','NZGD2000 / Grey Circuit 2000','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2119','NZGD2000 / Amuri Circuit 2000','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2120','NZGD2000 / Marlborough Circuit 2000','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2121','NZGD2000 / Hokitika Circuit 2000','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2122','NZGD2000 / Okarito Circuit 2000','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2123','NZGD2000 / Jacksons Bay Circuit 2000','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2124','NZGD2000 / Mount Pleasant Circuit 2000','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2125','NZGD2000 / Gawler Circuit 2000','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2126','NZGD2000 / Timaru Circuit 2000','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2127','NZGD2000 / Lindis Peak Circuit 2000','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2128','NZGD2000 / Mount Nicholas Circuit 2000','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2129','NZGD2000 / Mount York Circuit 2000','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2130','NZGD2000 / Observation Point Circuit 2000','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2131','NZGD2000 / North Taieri Circuit 2000','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2132','NZGD2000 / Bluff Circuit 2000','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2105','NZGD2000 / EDENTM2000','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2106','NZGD2000 / PLENTM2000','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2107','NZGD2000 / POVETM2000','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2108','NZGD2000 / HAWKTM2000','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2109','NZGD2000 / TARATM2000','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2110','NZGD2000 / TUHITM2000','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2111','NZGD2000 / WANGTM2000','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2112','NZGD2000 / WAIRTM2000','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2113','NZGD2000 / WELLTM2000','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2114','NZGD2000 / COLLTM2000','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2115','NZGD2000 / NELSTM2000','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2116','NZGD2000 / KARATM2000','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2117','NZGD2000 / BULLTM2000','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2118','NZGD2000 / GREYTM2000','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2119','NZGD2000 / AMURTM2000','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2120','NZGD2000 / MARLTM2000','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2121','NZGD2000 / HOKITM2000','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2122','NZGD2000 / OKARTM2000','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2123','NZGD2000 / JACKTM2000','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2124','NZGD2000 / PLEATM2000','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2125','NZGD2000 / GAWLTM2000','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2126','NZGD2000 / TIMATM2000','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2127','NZGD2000 / LINDTM2000','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2128','NZGD2000 / NICHTM2000','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2129','NZGD2000 / YORKTM2000','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2130','NZGD2000 / OBSETM2000','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2131','NZGD2000 / TAIETM2000','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2132','NZGD2000 / BLUFTM2000','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2193','NZGD2000 / NZTM2000','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5759','Auckland height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5760','Bluff height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5761','Dunedin height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5762','Gisborne height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5763','Lyttelton height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5764','Moturiki height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5765','Napier height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5766','Nelson height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5767','One Tree Point height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5768','Tararu height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5769','Taranaki height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5770','Wellington height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5771','Chatham Island height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5772','Stewart Island height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5739','Hong Kong Chart Datum depth','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5788','PWD height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5740','Newlyn (Orkney Isles) height','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4883','D96','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4882','D96','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4765','D96','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3794','D96/TM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26847','NAD83 / Maine CS83 East (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26848','NAD83 / Maine CS83 West (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26855','NAD83(HPGN) / Maine CS83 East (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26855','NAD83(HARN) / Maine CS83 East (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26856','NAD83(HARN) / Maine CS83 West (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26856','NAD83(HPGN) / Maine CS83 West (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26863','NAD83(NSRS2007) / Maine CS83 East (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26864','NAD83(NSRS2007) / Maine CS83 West (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26857','NAD83(HPGN) / Minnesota North (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26858','NAD83(HPGN) / Minnesota Central (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26859','NAD83(HPGN) / Minnesota South (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26860','NAD83(HPGN) / Nebraska (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26861','NAD83(HPGN) / West Virginia North (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26862','NAD83(HPGN) / West Virginia South (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3800','NAD27 / Alberta 3TM ref merid 120','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3801','NAD83 / Alberta 3TM ref merid 120','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3802','NAD83(CSRS) / Alberta 3TM ref merid 120','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9378','727','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3812','ETRS89 / Lambert 2008','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3814','NAD83 / MSTM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3815','NAD83(HARN) / MSTM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3815','NAD83(HPGN) / Mississippi TM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3816','NAD83(NSRS2007) / MSTM','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9378','IGb14 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3812','ETRS89 / LB08','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3447','ETRS89 / LB05','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31370','BD72 / LB72','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4236','Hu Tzu Shan','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4179','Pulkovo 1942(56)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4178','Uniform Astro-Geodetic Network (UAGN) 1983','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4179','Pulkovo 1942(57)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4178','S-42','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4179','Uniform Astro-Geodetic Network (UAGN) 1956','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4179','42/58','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4179','S-42','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4284','S-42','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4200','S-95','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3333','S-42 zone 3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3334','S-42 zone 4','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3334','S-42 zone 34','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3335','S-42 zone 5','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3335','S-42 zone 35','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3335','Pulkovo 1942(58) / 6-degree Gauss-Kruger zone 5','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3334','Pulkovo 1942(58) / 6-degree Gauss-Kruger zone 4','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3333','Pulkovo 1942(58) / 6-degree Gauss-Kruger zone 3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3834','Pulkovo 1942(83) / 6-degree Gauss-Kruger zone 2','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3834','S-42 zone 2','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3833','Pulkovo 1942(58) / 6-degree Gauss-Kruger zone 2','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3833','S-42 zone 2','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3835','Pulkovo 1942(83) / 6-degree Gauss-Kruger zone 3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3835','S-42 zone 3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3836','Pulkovo 1942(83) / 6-degree Gauss-Kruger zone 4','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3836','S-42 zone 4','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3844','Stereo 70','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3844','S-42 / Stereographic 1970','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3844','Dealul Piscului 1970/ Stereo 70','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4316','Dealul Piscului 1933','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31600','Dealul Piscului 1933/ Stereo 33','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31600','Stereo 30','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3848','SWEREF99 / RT90 0 gon 0:-15 emulation','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3849','SWEREF99 / RT90 2.5 gon O 0:-15 emulation','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3847','SWEREF99 / RT90 2.5 gon V 0:-15 emulation','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3850','SWEREF99 / RT90 5 gon O 0:-15 emulation','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3846','SWEREF99 / RT90 5 gon V 0:-15 emulation','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3845','SWEREF99 / RT90 7.5 gon V 0:-15 emulation','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21782','LV03C-G','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21781','LV03M','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2031','NAD27(CGQ77) / UTM 17N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2032','NAD27(CGQ77) / UTM 18N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2033','NAD27(CGQ77) / UTM 19N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2034','NAD27(CGQ77) / UTM 20N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2035','NAD27(CGQ77) / UTM 21N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2057','Rassadiran /Nakhl e Taqi','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2058','ED50(ED77) / UTM 38N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2059','ED50(ED77) / UTM 39N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2060','ED50(ED77) / UTM 40N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2061','ED50(ED77) / UTM 41N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2063','Dabola 1981 / UTM 28N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2064','Dabola 1981 / UTM 29N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2104','Lake / La Rosa Grid','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2136','Accra / Ghana Nat. Grid','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2138','CGQ77 / Quebec Lambert','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2159','Sierra Leone 24 / Colony','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2160','Sierra Leone 24 / WarOff','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2161','Sierra Leone 68 /UTM 28N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2162','Sierra Leone 68 /UTM 29N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2163','US National Atlas EA','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2189','Azores Cen. 48 / UTM 26N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2190','Sao Braz 1940 / UTM 26N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2192','ED50 / France EuroLamb','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2195','NAD83(HARN) / UTM 2S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2215','Manoca 1962 / UTM 32N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2216','Qornoq 1927 / UTM 22N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2217','Qornoq 1927 / UTM 23N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2218','Scoresbysund / GRL 5 E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2221','Scoresbysund / GRL 6 E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2222','NAD83 / Arizona E (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2223','NAD83 / Arizona Cen (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2224','NAD83 / Arizona W (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2225','NAD83 / CA zone 1 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2226','NAD83 / CA zone 2 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2227','NAD83 / CA zone 3 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2228','NAD83 / CA zone 4 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2229','NAD83 / CA zone 5 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2230','NAD83 / CA zone 6 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2231','NAD83 / CO North (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2232','NAD83 / CO Cen (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2233','NAD83 / CO South (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2234','NAD83 / CT (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2236','NAD83 / FL East (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2237','NAD83 / FL West (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2238','NAD83 / FL North (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2239','NAD83 / Georgia E (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2240','NAD83 / Georgia W (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2241','NAD83 / Idaho E (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2242','NAD83 / Idaho Cen (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2243','NAD83 / Idaho W (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2246','NAD83 / KY North (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2247','NAD83 / KY South (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2249','NAD83 / MA Mainld (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2250','NAD83 / MA Island (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2251','NAD83 / Michigan N (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2252','NAD83 / Michigan C (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2253','NAD83 / Michigan S (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2254','NAD83 / MS East (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2255','NAD83 / MS West (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2257','NAD83 / New Mex E (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2258','NAD83 / New Mex C (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2259','NAD83 / New Mex W (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2260','NAD83 / NY East (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2261','NAD83 / NY C (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2262','NAD83 / NY W (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2263','NAD83 / NY Island (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2264','NAD83 / NCarolina (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2265','NAD83 / N Dakota N (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2266','NAD83 / N Dakota S (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2267','NAD83 / OK North (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2268','NAD83 / OK South (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2269','NAD83 / Oregon N (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2270','NAD83 / Oregon S (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2271','NAD83 / PA North (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2272','NAD83 / PA South (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2273','NAD83 / S Carolina (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2275','NAD83 / Texas N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2276','NAD83 / Texas NC (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2277','NAD83 / Texas C (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2278','NAD83 / Texas SC (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2279','NAD83 / Texas S (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2281','NAD83 / Utah Cen. (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2283','NAD83 / VA North (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2284','NAD83 / VA South (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2285','NAD83 / WA North (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2286','NAD83 / WA South (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2287','NAD83 / WI North (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2288','NAD83 / WI Cen. (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2289','NAD83 / WI South (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2296','Ammassalik 58 / GRL 7 E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2298','Qornoq 1927 / GRL 2 east','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2299','Qornoq 1927 / GRL 2 west','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2301','Qoornoq 1927 / GRL 3 west','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2303','Qoornoq 1927 / GRL 4 west','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2304','Qoornoq 1927 / GRL 5 west','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2305','Qoornoq 1927 / GRL 6 west','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2306','Qoornoq 1927 / GRL 7 west','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2307','Qoornoq 1927 / GRL 8 east','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2314','Trinidad 03 Grid (ftCla)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2315','C Inchauspe / UTM 19S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2316','C Inchauspe / UTM 20S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2318','Ain el Abd / Aramco Lamb','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2327','Xian 1980 / G-K zone 13','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2328','Xian 1980 / G-K zone 14','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2329','Xian 1980 / G-K zone 15','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2330','Xian 1980 / G-K zone 16','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2331','Xian 1980 / G-K zone 17','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2332','Xian 1980 / G-K zone 18','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2334','Xian 1980 / G-K zone 20','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2335','Xian 1980 / G-K zone 21','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2336','Xian 1980 / G-K zone 22','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2337','Xian 1980 / G-K zone 23','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2338','Xian 1980 / G-K CM 75E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2339','Xian 1980 / G-K CM 81E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2340','Xian 1980 / G-K CM 87E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2341','Xian 1980 / G-K CM 93E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2342','Xian 1980 / G-K CM 99E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2343','Xian 1980 / G-K CM 105E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2344','Xian 1980 / G-K CM 111E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2345','Xian 1980 / G-K CM 117E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2346','Xian 1980 / G-K CM 123E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2347','Xian 1980 / G-K CM 129E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2348','Xian 1980 / G-K CM 135E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2349','Xian 1980 / 3GK zone 25','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2350','Xian 1980 / 3GK zone 26','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2351','Xian 1980 / 3GK zone 27','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2352','Xian 1980 / 3GK zone 28','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2353','Xian 1980 / 3GK zone 29','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2354','Xian 1980 / 3GK zone 30','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2355','Xian 1980 / 3GK zone 31','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2356','Xian 1980 / 3GK zone 32','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2357','Xian 1980 / 3GK zone 33','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2358','Xian 1980 / 3GK zone 34','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2359','Xian 1980 / 3GK zone 35','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2360','Xian 1980 / 3GK zone 36','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2361','Xian 1980 / 3GK zone 37','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2362','Xian 1980 / 3GK zone 38','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2363','Xian 1980 / 3GK zone 39','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2364','Xian 1980 / 3GK zone 40','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2365','Xian 1980 / 3GK zone 41','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2366','Xian 1980 / 3GK zone 42','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2367','Xian 1980 / 3GK zone 43','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2368','Xian 1980 / 3GK zone 44','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2369','Xian 1980 / 3GK zone 45','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2370','Xian 1980 / 3GK CM 75E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2371','Xian 1980 / 3GK CM 78E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2372','Xian 1980 / 3GK CM 81E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2373','Xian 1980 / 3GK CM 84E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2374','Xian 1980 / 3GK CM 87E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2375','Xian 1980 / 3GK CM 90E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2376','Xian 1980 / 3GK CM 93E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2377','Xian 1980 / 3GK CM 96E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2378','Xian 1980 / 3GK CM 99E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2379','Xian 1980 / 3GK CM 102E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2380','Xian 1980 / 3GK CM 105E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2381','Xian 1980 / 3GK CM 108E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2382','Xian 1980 / 3GK CM 111E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2383','Xian 1980 / 3GK CM 114E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2384','Xian 1980 / 3GK CM 117E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2385','Xian 1980 / 3GK CM 120E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2386','Xian 1980 / 3GK CM 123E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2387','Xian 1980 / 3GK CM 126E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2388','Xian 1980 / 3GK CM 129E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2389','Xian 1980 / 3GK CM 132E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2390','Xian 1980 / 3GK CM 135E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2397','Pulkovo 42(83) / 3GK zn3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2398','Pulkovo 42(83) / 3GK zn4','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2399','Pulkovo 42(83) / 3GK zn5','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2401','Beijing 1954 / 3GK zn 25','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2402','Beijing 1954 / 3GK zn 26','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2403','Beijing 1954 / 3GK zn 27','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2404','Beijing 1954 / 3GK zn 28','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2405','Beijing 1954 / 3GK zn 29','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2406','Beijing 1954 / 3GK zn 30','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2407','Beijing 1954 / 3GK zn 31','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2408','Beijing 1954 / 3GK zn 32','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2409','Beijing 1954 / 3GK zn 33','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2410','Beijing 1954 / 3GK zn 34','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2411','Beijing 1954 / 3GK zn 35','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2412','Beijing 1954 / 3GK zn 36','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2413','Beijing 1954 / 3GK zn 37','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2414','Beijing 1954 / 3GK zn 38','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2415','Beijing 1954 / 3GK zn 39','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2416','Beijing 1954 / 3GK zn 40','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2417','Beijing 1954 / 3GK zn 41','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2418','Beijing 1954 / 3GK zn 42','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2419','Beijing 1954 / 3GK zn 43','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2420','Beijing 1954 / 3GK zn 44','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2421','Beijing 1954 / 3GK zn 45','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2422','Beijing 1954 / 3GK 75E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2423','Beijing 1954 / 3GK 78E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2424','Beijing 1954 / 3GK 81E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2425','Beijing 1954 / 3GK 84E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2426','Beijing 1954 / 3GK 87E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2427','Beijing 1954 / 3GK 90E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2428','Beijing 1954 / 3GK 93E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2429','Beijing 1954 / 3GK 96E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2430','Beijing 1954 / 3GK 99E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2431','Beijing 1954 / 3GK 102E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2432','Beijing 1954 / 3GK 105E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2433','Beijing 1954 / 3GK 108E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2434','Beijing 1954 / 3GK 111E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2435','Beijing 1954 / 3GK 114E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2436','Beijing 1954 / 3GK 117E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2437','Beijing 1954 / 3GK 120E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2438','Beijing 1954 / 3GK 123E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2439','Beijing 1954 / 3GK 126E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2440','Beijing 1954 / 3GK 129E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2441','Beijing 1954 / 3GK 132E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2442','Beijing 1954 / 3GK 135E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2443','JGD2000 / Japan zone 1','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2444','JGD2000 / Japan zone 2','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2445','JGD2000 / Japan zone 3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2446','JGD2000 / Japan zone 4','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2447','JGD2000 / Japan zone 5','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2448','JGD2000 / Japan zone 6','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2449','JGD2000 / Japan zone 7','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2450','JGD2000 / Japan zone 8','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2451','JGD2000 / Japan zone 9','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2452','JGD2000 / Japan zone 10','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2453','JGD2000 / Japan zone 11','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2454','JGD2000 / Japan zone 12','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2455','JGD2000 / Japan zone 13','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2456','JGD2000 / Japan zone 14','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2457','JGD2000 / Japan zone 15','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2458','JGD2000 / Japan zone 16','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2459','JGD2000 / Japan zone 17','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2460','JGD2000 / Japan zone 18','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2461','JGD2000 / Japan zone 19','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2462','Albanian 1987 / GK zn 4','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2463','Pulkovo 1995 / 6GK 21E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2464','Pulkovo 1995 / 6GK 27E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2465','Pulkovo 1995 / 6GK 33E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2466','Pulkovo 1995 / 6GK 39E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2467','Pulkovo 1995 / 6GK 45E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2468','Pulkovo 1995 / 6GK 51E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2469','Pulkovo 1995 / 6GK 57E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2470','Pulkovo 1995 / 6GK 63E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2471','Pulkovo 1995 / 6GK 69E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2472','Pulkovo 1995 / 6GK 75E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2473','Pulkovo 1995 / 6GK 81E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2474','Pulkovo 1995 / 6GK 87E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2475','Pulkovo 1995 / 6GK 93E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2476','Pulkovo 1995 / 6GK 99E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2477','Pulkovo 1995 / 6GK 105E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2478','Pulkovo 1995 / 6GK 111E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2479','Pulkovo 1995 / 6GK 117E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2480','Pulkovo 1995 / 6GK 123E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2481','Pulkovo 1995 / 6GK 129E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2482','Pulkovo 1995 / 6GK 135E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2483','Pulkovo 1995 / 6GK 141E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2484','Pulkovo 1995 / 6GK 147E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2485','Pulkovo 1995 / 6GK 153E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2486','Pulkovo 1995 / 6GK 159E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2487','Pulkovo 1995 / 6GK 165E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2488','Pulkovo 1995 / 6GK 171E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2489','Pulkovo 1995 / 6GK 177E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2490','Pulkovo 1995 / 6GK 177W','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2491','Pulkovo 1995 / 6GK 171W','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2494','Pulkovo 1942 / 6GK 21E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2495','Pulkovo 1942 / 6GK 27E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2496','Pulkovo 1942 / 6GK 33E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2497','Pulkovo 1942 / 6GK 39E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2498','Pulkovo 1942 / 6GK 45E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2499','Pulkovo 1942 / 6GK 51E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2500','Pulkovo 1942 / 6GK 57E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2501','Pulkovo 1942 / 6GK 63E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2502','Pulkovo 1942 / 6GK 69E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2503','Pulkovo 1942 / 6GK 75E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2504','Pulkovo 1942 / 6GK 81E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2505','Pulkovo 1942 / 6GK 87E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2506','Pulkovo 1942 / 6GK 93E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2507','Pulkovo 1942 / 6GK 99E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2508','Pulkovo 1942 / 6GK 105E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2509','Pulkovo 1942 / 6GK 111E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2510','Pulkovo 1942 / 6GK 117E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2511','Pulkovo 1942 / 6GK 123E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2512','Pulkovo 1942 / 6GK 129E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2513','Pulkovo 1942 / 6GK 135E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2514','Pulkovo 1942 / 6GK 141E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2515','Pulkovo 1942 / 6GK 147E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2516','Pulkovo 1942 / 6GK 153E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2517','Pulkovo 1942 / 6GK 159E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2518','Pulkovo 1942 / 6GK 165E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2519','Pulkovo 1942 / 6GK 171E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2520','Pulkovo 1942 / 6GK 177E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2521','Pulkovo 1942 / 6GK 177W','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2522','Pulkovo 1942 / 6GK 171W','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2523','Pulkovo 1942 / 3GK zn 7','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2524','Pulkovo 1942 / 3GK zn 8','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2525','Pulkovo 1942 / 3GK zn 9','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2526','Pulkovo 1942 / 3GK zn 10','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2527','Pulkovo 1942 / 3GK zn 11','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2528','Pulkovo 1942 / 3GK zn 12','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2529','Pulkovo 1942 / 3GK zn 13','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2530','Pulkovo 1942 / 3GK zn 14','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2531','Pulkovo 1942 / 3GK zn 15','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2532','Pulkovo 1942 / 3GK zn 16','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2533','Pulkovo 1942 / 3GK zn 17','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2534','Pulkovo 1942 / 3GK zn 18','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2535','Pulkovo 1942 / 3GK zn 19','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2536','Pulkovo 1942 / 3GK zn 20','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2537','Pulkovo 1942 / 3GK zn 21','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2538','Pulkovo 1942 / 3GK zn 22','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2539','Pulkovo 1942 / 3GK zn 23','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2540','Pulkovo 1942 / 3GK zn 24','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2541','Pulkovo 1942 / 3GK zn 25','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2542','Pulkovo 1942 / 3GK zn 26','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2543','Pulkovo 1942 / 3GK zn 27','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2544','Pulkovo 1942 / 3GK zn 28','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2545','Pulkovo 1942 / 3GK zn 29','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2546','Pulkovo 1942 / 3GK zn 30','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2547','Pulkovo 1942 / 3GK zn 31','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2548','Pulkovo 1942 / 3GK zn 32','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2549','Pulkovo 1942 / 3GK zn 33','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2551','Pulkovo 1942 / 3GK zn 34','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2552','Pulkovo 1942 / 3GK zn 35','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2553','Pulkovo 1942 / 3GK zn 36','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2554','Pulkovo 1942 / 3GK zn 37','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2555','Pulkovo 1942 / 3GK zn 38','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2556','Pulkovo 1942 / 3GK zn 39','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2557','Pulkovo 1942 / 3GK zn 40','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2558','Pulkovo 1942 / 3GK zn 41','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2559','Pulkovo 1942 / 3GK zn 42','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2560','Pulkovo 1942 / 3GK zn 43','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2561','Pulkovo 1942 / 3GK zn 44','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2562','Pulkovo 1942 / 3GK zn 45','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2563','Pulkovo 1942 / 3GK zn 46','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2564','Pulkovo 1942 / 3GK zn 47','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2565','Pulkovo 1942 / 3GK zn 48','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2566','Pulkovo 1942 / 3GK zn 49','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2567','Pulkovo 1942 / 3GK zn 50','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2568','Pulkovo 1942 / 3GK zn 51','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2569','Pulkovo 1942 / 3GK zn 52','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2570','Pulkovo 1942 / 3GK zn 53','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2571','Pulkovo 1942 / 3GK zn 54','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2572','Pulkovo 1942 / 3GK zn 55','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2573','Pulkovo 1942 / 3GK zn 56','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2574','Pulkovo 1942 / 3GK zn 57','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2575','Pulkovo 1942 / 3GK zn 58','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2576','Pulkovo 1942 / 3GK zn 59','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3389','Pulkovo 1942 / 3GK zn 60','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2578','Pulkovo 1942 / 3GK zn 61','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2579','Pulkovo 1942 / 3GK zn 62','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2580','Pulkovo 1942 / 3GK zn 63','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2581','Pulkovo 1942 / 3GK zn 64','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2582','Pulkovo 1942 / 3GK 21E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2583','Pulkovo 1942 / 3GK 24E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2584','Pulkovo 1942 / 3GK 27E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2585','Pulkovo 1942 / 3GK 30E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2586','Pulkovo 1942 / 3GK 33E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2587','Pulkovo 1942 / 3GK 36E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2588','Pulkovo 1942 / 3GK 39E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2589','Pulkovo 1942 / 3GK 42E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2590','Pulkovo 1942 / 3GK 45E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2591','Pulkovo 1942 / 3GK 48E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2592','Pulkovo 1942 / 3GK 51E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2593','Pulkovo 1942 / 3GK 54E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2594','Pulkovo 1942 / 3GK 57E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2595','Pulkovo 1942 / 3GK 60E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2596','Pulkovo 1942 / 3GK 63E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2597','Pulkovo 1942 / 3GK 66E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2598','Pulkovo 1942 / 3GK 69E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2599','Pulkovo 1942 / 3GK 72E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2601','Pulkovo 1942 / 3GK 75E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2602','Pulkovo 1942 / 3GK 78E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2603','Pulkovo 1942 / 3GK 81E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2604','Pulkovo 1942 / 3GK 84E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2605','Pulkovo 1942 / 3GK 87E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2606','Pulkovo 1942 / 3GK 90E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2607','Pulkovo 1942 / 3GK 93E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2608','Pulkovo 1942 / 3GK 96E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2609','Pulkovo 1942 / 3GK 99E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2610','Pulkovo 1942 / 3GK 102E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2611','Pulkovo 1942 / 3GK 105E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2612','Pulkovo 1942 / 3GK 108E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2613','Pulkovo 1942 / 3GK 111E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2614','Pulkovo 1942 / 3GK 114E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2615','Pulkovo 1942 / 3GK 117E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2616','Pulkovo 1942 / 3GK 120E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2617','Pulkovo 1942 / 3GK 123E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2618','Pulkovo 1942 / 3GK 126E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2619','Pulkovo 1942 / 3GK 129E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2620','Pulkovo 1942 / 3GK 132E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2621','Pulkovo 1942 / 3GK 135E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2622','Pulkovo 1942 / 3GK 138E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2623','Pulkovo 1942 / 3GK 141E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2624','Pulkovo 1942 / 3GK 144E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2625','Pulkovo 1942 / 3GK 147E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2626','Pulkovo 1942 / 3GK 150E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2627','Pulkovo 1942 / 3GK 153E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2628','Pulkovo 1942 / 3GK 156E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2629','Pulkovo 1942 / 3GK 159E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2630','Pulkovo 1942 / 3GK 162E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2631','Pulkovo 1942 / 3GK 165E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2632','Pulkovo 1942 / 3GK 168E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2633','Pulkovo 1942 / 3GK 171E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2634','Pulkovo 1942 / 3GK 174E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2635','Pulkovo 1942 / 3GK 177E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2636','Pulkovo 1942 / 3GK 180E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2637','Pulkovo 1942 / 3GK 177W','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2638','Pulkovo 1942 / 3GK 174W','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2639','Pulkovo 1942 / 3GK 171W','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2640','Pulkovo 1942 / 3GK 168W','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2641','Pulkovo 1995 / 3GK zn 7','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2642','Pulkovo 1995 / 3GK zn 8','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2643','Pulkovo 1995 / 3GK zn 9','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2644','Pulkovo 1995 / 3GK zn 10','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2645','Pulkovo 1995 / 3GK zn 11','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2646','Pulkovo 1995 / 3GK zn 12','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2647','Pulkovo 1995 / 3GK zn 13','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2648','Pulkovo 1995 / 3GK zn 14','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2649','Pulkovo 1995 / 3GK zn 15','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2650','Pulkovo 1995 / 3GK zn 16','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2651','Pulkovo 1995 / 3GK zn 17','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2652','Pulkovo 1995 / 3GK zn 18','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2653','Pulkovo 1995 / 3GK zn 19','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2654','Pulkovo 1995 / 3GK zn 20','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2655','Pulkovo 1995 / 3GK zn 21','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2656','Pulkovo 1995 / 3GK zn 22','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2657','Pulkovo 1995 / 3GK zn 23','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2658','Pulkovo 1995 / 3GK zn 24','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2659','Pulkovo 1995 / 3GK zn 25','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2660','Pulkovo 1995 / 3GK zn 26','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2661','Pulkovo 1995 / 3GK zn 27','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2662','Pulkovo 1995 / 3GK zn 28','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2663','Pulkovo 1995 / 3GK zn 29','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2664','Pulkovo 1995 / 3GK zn 30','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2665','Pulkovo 1995 / 3GK zn 31','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2666','Pulkovo 1995 / 3GK zn 32','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2667','Pulkovo 1995 / 3GK zn 33','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2668','Pulkovo 1995 / 3GK zn 34','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2669','Pulkovo 1995 / 3GK zn 35','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2670','Pulkovo 1995 / 3GK zn 36','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2671','Pulkovo 1995 / 3GK zn 37','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2672','Pulkovo 1995 / 3GK zn 38','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2673','Pulkovo 1995 / 3GK zn 39','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2674','Pulkovo 1995 / 3GK zn 40','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2675','Pulkovo 1995 / 3GK zn 41','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2676','Pulkovo 1995 / 3GK zn 42','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2677','Pulkovo 1995 / 3GK zn 43','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2678','Pulkovo 1995 / 3GK zn 44','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2679','Pulkovo 1995 / 3GK zn 45','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2680','Pulkovo 1995 / 3GK zn 46','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2681','Pulkovo 1995 / 3GK zn 47','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2682','Pulkovo 1995 / 3GK zn 48','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2683','Pulkovo 1995 / 3GK zn 49','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2684','Pulkovo 1995 / 3GK zn 50','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2685','Pulkovo 1995 / 3GK zn 51','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2686','Pulkovo 1995 / 3GK zn 52','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2687','Pulkovo 1995 / 3GK zn 53','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2688','Pulkovo 1995 / 3GK zn 54','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2689','Pulkovo 1995 / 3GK zn 55','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2690','Pulkovo 1995 / 3GK zn 56','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2691','Pulkovo 1995 / 3GK zn 57','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2692','Pulkovo 1995 / 3GK zn 58','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2693','Pulkovo 1995 / 3GK zn 59','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3390','Pulkovo 1995 / 3GK zn 60','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2695','Pulkovo 1995 / 3GK zn 61','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2696','Pulkovo 1995 / 3GK zn 62','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2697','Pulkovo 1995 / 3GK zn 63','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2698','Pulkovo 1995 / 3GK zn 64','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2699','Pulkovo 1995 / 3GK 21E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2700','Pulkovo 1995 / 3GK 24E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2701','Pulkovo 1995 / 3GK 27E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2702','Pulkovo 1995 / 3GK 30E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2703','Pulkovo 1995 / 3GK 33E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2704','Pulkovo 1995 / 3GK 36E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2705','Pulkovo 1995 / 3GK 39E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2706','Pulkovo 1995 / 3GK 42E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2707','Pulkovo 1995 / 3GK 45E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2708','Pulkovo 1995 / 3GK 48E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2709','Pulkovo 1995 / 3GK 51E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2710','Pulkovo 1995 / 3GK 54E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2711','Pulkovo 1995 / 3GK 57E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2712','Pulkovo 1995 / 3GK 60E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2713','Pulkovo 1995 / 3GK 63E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2714','Pulkovo 1995 / 3GK 66E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2715','Pulkovo 1995 / 3GK 69E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2716','Pulkovo 1995 / 3GK 72E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2717','Pulkovo 1995 / 3GK 75E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2718','Pulkovo 1995 / 3GK 78E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2719','Pulkovo 1995 / 3GK 81E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2720','Pulkovo 1995 / 3GK 84E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2721','Pulkovo 1995 / 3GK 87E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2722','Pulkovo 1995 / 3GK 90E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2723','Pulkovo 1995 / 3GK 93E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2724','Pulkovo 1995 / 3GK 96E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2725','Pulkovo 1995 / 3GK 99E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2726','Pulkovo 1995 / 3GK 102E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2727','Pulkovo 1995 / 3GK 105E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2728','Pulkovo 1995 / 3GK 108E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2729','Pulkovo 1995 / 3GK 111E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2730','Pulkovo 1995 / 3GK 114E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2731','Pulkovo 1995 / 3GK 117E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2732','Pulkovo 1995 / 3GK 120E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2733','Pulkovo 1995 / 3GK 123E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2734','Pulkovo 1995 / 3GK 126E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2735','Pulkovo 1995 / 3GK 129E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2738','Pulkovo 1995 / 3GK 132E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2739','Pulkovo 1995 / 3GK 135E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2740','Pulkovo 1995 / 3GK 138E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2741','Pulkovo 1995 / 3GK 141E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2742','Pulkovo 1995 / 3GK 144E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2743','Pulkovo 1995 / 3GK 147E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2744','Pulkovo 1995 / 3GK 150E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2745','Pulkovo 1995 / 3GK 153E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2746','Pulkovo 1995 / 3GK 156E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2747','Pulkovo 1995 / 3GK 159E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2748','Pulkovo 1995 / 3GK 162E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2749','Pulkovo 1995 / 3GK 165E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2750','Pulkovo 1995 / 3GK 168E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2751','Pulkovo 1995 / 3GK 171E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2752','Pulkovo 1995 / 3GK 174E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2753','Pulkovo 1995 / 3GK 177E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2754','Pulkovo 1995 / 3GK 180E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2755','Pulkovo 1995 / 3GK 177W','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2756','Pulkovo 1995 / 3GK 174W','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2757','Pulkovo 1995 / 3GK 171W','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2758','Pulkovo 1995 / 3GK 168W','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2759','NAD83(HARN) / AL E (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2760','NAD83(HARN) / AL W (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2761','NAD83(HARN) / AZ E (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2762','NAD83(HARN) / AZ C (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2763','NAD83(HARN) / AZ W (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2764','NAD83(HARN) / AR N (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2765','NAD83(HARN) / AR S (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2766','NAD83(HARN) / CA 1 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2767','NAD83(HARN) / CA 2 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2768','NAD83(HARN) / CA 3 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2769','NAD83(HARN) / CA 4 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2770','NAD83(HARN) / CA 5 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2771','NAD83(HARN) / CA 6 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2772','NAD83(HARN) / CO N (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2773','NAD83(HARN) / CO C (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2774','NAD83(HARN) / CO S (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2775','NAD83(HARN) / CT (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2777','NAD83(HARN) / FL E (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2778','NAD83(HARN) / FL W (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2779','NAD83(HARN) / FL N (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2780','NAD83(HARN) / GA E (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2781','NAD83(HARN) / GA W (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2782','NAD83(HARN) / HI 1 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2783','NAD83(HARN) / HI 2 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2784','NAD83(HARN) / HI 3 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2785','NAD83(HARN) / HI 4 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2786','NAD83(HARN) / HI 5 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2788','NAD83(HARN) / ID C (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2791','NAD83(HARN) / IL W (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2792','NAD83(HARN) / IN E (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2793','NAD83(HARN) / IN W (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2796','NAD83(HARN) / KS N (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2797','NAD83(HARN) / KS S (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2798','NAD83(HARN) / KY N (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2799','NAD83(HARN) / KY S (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2800','NAD83(HARN) / LA N (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2801','NAD83(HARN) / LA S (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2805','NAD83(HARN) / MA md (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2806','NAD83(HARN) / MA Is (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2807','NAD83(HARN) / MI N (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2808','NAD83(HARN) / MI C (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2809','NAD83(HARN) / MI S (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2810','NAD83(HARN) / MN N (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2811','NAD83(HARN) / MN C (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2812','NAD83(HARN) / MN S (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2813','NAD83(HARN) / MS E (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2814','NAD83(HARN) / MS W (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2815','NAD83(HARN) / MO E (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2816','NAD83(HARN) / MO C (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2817','NAD83(HARN) / MO W (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2820','NAD83(HARN) / NV E (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2821','NAD83(HARN) / NV C (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2822','NAD83(HARN) / NV W (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2823','NAD83(HARN) / NH (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2825','NAD83(HARN) / NM E (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2826','NAD83(HARN) / NM C (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2827','NAD83(HARN) / NM W (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2828','NAD83(HARN) / NY E (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2829','NAD83(HARN) / NY C (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2830','NAD83(HARN) / NY W (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2831','NAD83(HARN) / NY LI (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2832','NAD83(HARN) / ND N (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2833','NAD83(HARN) / ND S (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2836','NAD83(HARN) / OK N (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2837','NAD83(HARN) / OK S (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2838','NAD83(HARN) / OR N (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2839','NAD83(HARN) / OR S (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2840','NAD83(HARN) / RI (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2841','NAD83(HARN) / SD N (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2842','NAD83(HARN) / SD S (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2844','NAD83(HARN) / TX N (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2845','NAD83(HARN) / TX NC (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2846','NAD83(HARN) / TX C (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2847','NAD83(HARN) / TX SC (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2848','NAD83(HARN) / TX S (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2850','NAD83(HARN) / UT C (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2853','NAD83(HARN) / VA N (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2854','NAD83(HARN) / VA S (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2855','NAD83(HARN) / WA N (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2856','NAD83(HARN) / WA S (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2857','NAD83(HARN) / WV N (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2858','NAD83(HARN) / WV S (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2859','NAD83(HARN) / WI N (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2860','NAD83(HARN) / WI C (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2861','NAD83(HARN) / WI S (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2862','NAD83(HARN) / WY E (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2863','NAD83(HARN) / WY EC (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2864','NAD83(HARN) / WY WC (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2865','NAD83(HARN) / WY W (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2866','NAD83(HARN) / PR and VI','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2867','NAD83(HARN) / AZ E (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2868','NAD83(HARN) / AZ C (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2869','NAD83(HARN) / AZ W (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2870','NAD83(HARN) / CA 1 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2871','NAD83(HARN) / CA 2 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2872','NAD83(HARN) / CA 3 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2873','NAD83(HARN) / CA 4 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2874','NAD83(HARN) / CA 5 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2875','NAD83(HARN) / CA 6 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2876','NAD83(HARN) / CO N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2877','NAD83(HARN) / CO C (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2878','NAD83(HARN) / CO S (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2879','NAD83(HARN) / CT (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2880','NAD83(HARN) / DE (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2881','NAD83(HARN) / FL E (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2882','NAD83(HARN) / FL W (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2883','NAD83(HARN) / FL N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2884','NAD83(HARN) / GA E (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2885','NAD83(HARN) / GA W (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2886','NAD83(HARN) / ID E (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2887','NAD83(HARN) / ID C (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2888','NAD83(HARN) / ID W (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2967','NAD83(HARN) / IN E (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2968','NAD83(HARN) / IN W (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2891','NAD83(HARN) / KY N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2892','NAD83(HARN) / KY S (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2893','NAD83(HARN) / MD (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2894','NAD83(HARN) / MA md (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2895','NAD83(HARN) / MA Is (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2896','NAD83(HARN) / MI N (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2897','NAD83(HARN) / MI C (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2898','NAD83(HARN) / MI S (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2899','NAD83(HARN) / MS E (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2900','NAD83(HARN) / MS W (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2901','NAD83(HARN) / MT (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2902','NAD83(HARN) / NM E (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2903','NAD83(HARN) / NM C (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2904','NAD83(HARN) / NM W (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2905','NAD83(HARN) / NY E (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2906','NAD83(HARN) / NY C (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2907','NAD83(HARN) / NY W (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2908','NAD83(HARN) / NY LI (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2909','NAD83(HARN) / ND N (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2910','NAD83(HARN) / ND S (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2911','NAD83(HARN) / OK N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2912','NAD83(HARN) / OK S (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2913','NAD83(HARN) / OR N (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2914','NAD83(HARN) / OR S (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2915','NAD83(HARN) / TN (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2916','NAD83(HARN) / TX N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2917','NAD83(HARN) / TX NC (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2918','NAD83(HARN) / TX C (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2919','NAD83(HARN) / TX SC (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2920','NAD83(HARN) / TX S (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2921','NAD83(HARN) / UT N (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2922','NAD83(HARN) / UT C (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2923','NAD83(HARN) / UT S (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2924','NAD83(HARN) / VA N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2925','NAD83(HARN) / VA S (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2926','NAD83(HARN) / WA N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2927','NAD83(HARN) / WA S (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2928','NAD83(HARN) / WI N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2929','NAD83(HARN) / WI C (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2930','NAD83(HARN) / WI S (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2932','QND95 / Qatar Nat Grid','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2935','Pulkovo 1942 / CS63 A1','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2936','Pulkovo 1942 / CS63 A2','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2937','Pulkovo 1942 / CS63 A3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2938','Pulkovo 1942 / CS63 A4','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2939','Pulkovo 1942 / CS63 K2','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2940','Pulkovo 1942 / CS63 K3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2941','Pulkovo 1942 / CS63 K4','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2942','Porto Santo / UTM 28N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2943','Selvagem Gr. / UTM 28N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2944','NAD83(CSRS) / SCoPQ 2','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2952','NAD83(CSRS) / MTM zn 10','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2953','NAD83(CSRS) / NB Stereo','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2954','NAD83(CSRS) / PEI Stereo','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2955','NAD83(CSRS) / UTM 11N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2956','NAD83(CSRS) / UTM 12N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2957','NAD83(CSRS) / UTM 13N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2958','NAD83(CSRS) / UTM 17N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2959','NAD83(CSRS) / UTM 18N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2960','NAD83(CSRS) / UTM 19N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2961','NAD83(CSRS) / UTM 20N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2962','NAD83(CSRS) / UTM 21N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2963','Lisbon 1890 / Bonne','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2965','NAD83 / Indiana E (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2966','NAD83 / Indiana W (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2969','Fort Marigot / UTM 20N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2978','IGN72 Nuku Hiva / UTM 7S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2980','Combani 1950 / UTM 38S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2981','IGN56 Lifou / UTM zn 58S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2985','Petrels 1972 / Adelie St','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2986','Perroud 1950 / Adelie St','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2987','St. Pierre Miq / UTM 21N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2992','NAD83 / OR GIC Lam (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2994','NAD83(HARN) / OR GIC Lam (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2995','IGN53 Mare / UTM zn 58S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2996','ST84 des Pins / UTM 58S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2997','ST71 Belep / UTM zn 58S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2998','NEA74 Noumea / UTM 58S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2999','Grand Comoros / UTM 38S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3031','WGS 84 / Antarctic PS','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3032','WGS 84 / Aus Antarc PS','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3033','WGS 84 / Aus Antarc LCC','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3052','Reykjavik 1900 / Lambert','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3053','Hjorsey 1955 / Lambert','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3054','Hjorsey 1955 / UTM 26N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3055','Hjorsey 1955 / UTM 27N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3056','Hjorsey 1955 / UTM 28N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3058','Helle 1954 / Jan Mayen','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3060','IGN72 Gr Terre / UTM 58S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3071','NAD83(HARN) / WTM 83','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3072','NAD83 / Maine CS2000 E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3463','NAD83 / Maine CS2000 C','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3074','NAD83 / Maine CS2000 W','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3075','NAD83(HARN) / ME 2000 E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3464','NAD83(HARN) / ME 2000 C','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3077','NAD83(HARN) / ME 2000 W','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3086','NAD83 / FL GDL Albers','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3087','NAD83(HARN) / FL GDL AEA','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3102','American Samoa 62 / LCC','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3114','MAGNA-SIRGAS / Col FW','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3115','MAGNA-SIRGAS / Col W','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3116','MAGNA-SIRGAS / Col Bog','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3117','MAGNA-SIRGAS / Col EC','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3118','MAGNA-SIRGAS / Col E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3121','PRS92 / Philippines 1','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3122','PRS92 / Philippines 2','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3123','PRS92 / Philippines 3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3124','PRS92 / Philippines 4','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3125','PRS92 / Philippines 5','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3139','Vanua Levu 1915 / Cass','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3140','Viti Levu 1912 / Cassini','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3146','Pulkovo 1942 / 3GK zn 6','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3147','Pulkovo 1942 / 3GK 18E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3148','Indian 1960 / UTM 48N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3149','Indian 1960 / UTM 49N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3150','Pulkovo 1995 / 3GK zn 6','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3151','Pulkovo 1995 / 3GK 18E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3154','NAD83(CSRS) / UTM 7N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3155','NAD83(CSRS) / UTM 8N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3156','NAD83(CSRS) / UTM 9N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3157','NAD83(CSRS) / UTM 10N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3158','NAD83(CSRS) / UTM 14N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3159','NAD83(CSRS) / UTM 15N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3160','NAD83(CSRS) / UTM 16N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3161','NAD83 / Ontario MNR LCC','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3162','NAD83(CSRS) / ON MNR LCC','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3164','ST87 Ouvea / UTM 58S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3165','NEA74 Noumea / LCC','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3166','NEA74 Noumea / LCC2','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3167','Kertau (RSO) / RSO (ch)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3168','Kertau (RSO) / RSO (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3172','IGN53 Mare / UTM 59S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3174','NAD83 / GL Albers','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3175','NAD83 / GL+SLS Albers','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3997','WGS 84 / DLTM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3995','WGS 84 / Arctic PS','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3996','WGS 84 / IBCAO PS','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','7410','PSD93 + PHD93','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5725','Fahud Height Datum height','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3190','LGD2006 / Libya TM 5','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3191','LGD2006 / Libya TM 6','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3192','LGD2006 / Libya TM 7','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3193','LGD2006 / Libya TM 8','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3194','LGD2006 / Libya TM 9','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3195','LGD2006 / Libya TM 10','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3196','LGD2006 / Libya TM 11','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3197','LGD2006 / Libya TM 12','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3198','LGD2006 / Libya TM 13','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3204','WGS 84 / SCAR SP19-20','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3205','WGS 84 / SCAR SP21-22','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6933','EASE-Grid 2.0 Global','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6931','EASE-Grid 2.0 North','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6932','EASE-Grid 2.0 South','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','6184','Cais da Figueirinha - Angra do Heroísmo height','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3206','WGS 84 / SCAR SP23-24','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3207','WGS 84 / SCAR SQ01-02','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3208','WGS 84 / SCAR SQ19-20','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3209','WGS 84 / SCAR SQ21-22','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3210','WGS 84 / SCAR SQ37-38','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3211','WGS 84 / SCAR SQ39-40','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3212','WGS 84 / SCAR SQ41-42','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3213','WGS 84 / SCAR SQ43-44','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3214','WGS 84 / SCAR SQ45-46','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3215','WGS 84 / SCAR SQ47-48','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3216','WGS 84 / SCAR SQ49-50','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3217','WGS 84 / SCAR SQ51-52','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3218','WGS 84 / SCAR SQ53-54','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3219','WGS 84 / SCAR SQ55-56','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3220','WGS 84 / SCAR SQ57-58','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3221','WGS 84 / SCAR SR13-14','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3222','WGS 84 / SCAR SR15-16','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3223','WGS 84 / SCAR SR17-18','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3224','WGS 84 / SCAR SR19-20','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3225','WGS 84 / SCAR SR27-28','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3226','WGS 84 / SCAR SR29-30','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3227','WGS 84 / SCAR SR31-32','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3228','WGS 84 / SCAR SR33-34','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3229','WGS 84 / SCAR SR35-36','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3230','WGS 84 / SCAR SR37-38','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3231','WGS 84 / SCAR SR39-40','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3232','WGS 84 / SCAR SR41-42','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3233','WGS 84 / SCAR SR43-44','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3234','WGS 84 / SCAR SR45-46','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3235','WGS 84 / SCAR SR47-48','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3236','WGS 84 / SCAR SR49-50','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3237','WGS 84 / SCAR SR51-52','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3238','WGS 84 / SCAR SR53-54','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3239','WGS 84 / SCAR SR55-56','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3240','WGS 84 / SCAR SR57-58','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3241','WGS 84 / SCAR SR59-60','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3242','WGS 84 / SCAR SS04-06','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3243','WGS 84 / SCAR SS07-09','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3244','WGS 84 / SCAR SS10-12','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3245','WGS 84 / SCAR SS13-15','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3246','WGS 84 / SCAR SS16-18','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3247','WGS 84 / SCAR SS19-21','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3248','WGS 84 / SCAR SS25-27','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3249','WGS 84 / SCAR SS28-30','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3250','WGS 84 / SCAR SS31-33','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3251','WGS 84 / SCAR SS34-36','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3252','WGS 84 / SCAR SS37-39','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3253','WGS 84 / SCAR SS40-42','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3254','WGS 84 / SCAR SS43-45','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3255','WGS 84 / SCAR SS46-48','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3256','WGS 84 / SCAR SS49-51','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3257','WGS 84 / SCAR SS52-54','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3258','WGS 84 / SCAR SS55-57','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3259','WGS 84 / SCAR SS58-60','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3260','WGS 84 / SCAR ST01-04','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3261','WGS 84 / SCAR ST05-08','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3262','WGS 84 / SCAR ST09-12','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3263','WGS 84 / SCAR ST13-16','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3264','WGS 84 / SCAR ST17-20','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3265','WGS 84 / SCAR ST21-24','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3266','WGS 84 / SCAR ST25-28','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3267','WGS 84 / SCAR ST29-32','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3268','WGS 84 / SCAR ST33-36','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3269','WGS 84 / SCAR ST37-40','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3270','WGS 84 / SCAR ST41-44','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3271','WGS 84 / SCAR ST45-48','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3272','WGS 84 / SCAR ST49-52','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3273','WGS 84 / SCAR ST53-56','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3274','WGS 84 / SCAR ST57-60','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3275','WGS 84 / SCAR SU01-05','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3276','WGS 84 / SCAR SU06-10','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3277','WGS 84 / SCAR SU11-15','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3278','WGS 84 / SCAR SU16-20','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3279','WGS 84 / SCAR SU21-25','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3280','WGS 84 / SCAR SU26-30','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3281','WGS 84 / SCAR SU31-35','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3282','WGS 84 / SCAR SU36-40','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3283','WGS 84 / SCAR SU41-45','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3284','WGS 84 / SCAR SU46-50','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3285','WGS 84 / SCAR SU51-55','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3286','WGS 84 / SCAR SU56-60','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3287','WGS 84 / SCAR SV01-10','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3288','WGS 84 / SCAR SV11-20','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3289','WGS 84 / SCAR SV21-30','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3290','WGS 84 / SCAR SV31-40','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3291','WGS 84 / SCAR SV41-50','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3292','WGS 84 / SCAR SV51-60','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3293','WGS 84 / SCAR SW01-60','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3294','WGS 84 / Transantarctic','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3302','IGN63 Hiva Oa / UTM 7S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3303','Fatu Iva 72 / UTM 7S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3307','Nakhl-e Ghanem / UTM 39N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3309','NAD27 / CA Albers','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3310','NAD83 / CA Albers','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3311','NAD83(HARN) / CA Albers','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3314','Katanga 1955 / Lambert','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3315','Katanga 1955 / TM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3316','Kasai 1953 / Congo TM 22','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3317','Kasai 1953 / Congo TM 24','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3318','IGC 1962 / Congo TM 12','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3319','IGC 1962 / Congo TM 14','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3320','IGC 1962 / Congo TM 16','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3321','IGC 1962 / Congo TM 18','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3322','IGC 1962 / Congo TM 20','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3323','IGC 1962 / Congo TM 22','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3324','IGC 1962 / Congo TM 24','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3325','IGC 1962 / Congo TM 26','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3326','IGC 1962 / Congo TM 28','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3327','IGC 1962 / Congo TM 30','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3329','Pulkovo 42(58) / 3GK zn5','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3840','Pulkovo 42(58) / 3GKzn10','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3837','Pulkovo 42(58) / 3GK zn3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3838','Pulkovo 42(58) / 3GK zn4','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3330','Pulkovo 42(58) / 3GK zn6','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3331','Pulkovo 42(58) / 3GK zn7','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3332','Pulkovo 42(58) / 3GK zn8','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3839','Pulkovo 42(58) / 3GK zn9','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3841','Pulkovo 42(83) / 3GK zn6','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3833','Pulkovo 42(58) / GK zn 2','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3333','Pulkovo 42(58) / GK zn 3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3334','Pulkovo 42(58) / GK zn 4','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3335','Pulkovo 42(58) / GK zn 5','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3842','Pulkovo 42(83) / 3GK zn7','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3843','Pulkovo 42(83) / 3GK zn8','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3834','Pulkovo 42(83) / GK zn 2','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3835','Pulkovo 42(83) / GK zn 3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3836','Pulkovo 42(83) / GK zn 4','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3779','NAD83(CSRS) / AB 3TM 111','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3780','NAD83(CSRS) / AB 3TM 114','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3781','NAD83(CSRS) / AB 3TM 117','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3802','NAD83(CSRS) / AB 3TM 120','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26891','NAD83(CSRS) / MTM zn 11','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26892','NAD83(CSRS) / MTM zn 12','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26893','NAD83(CSRS) / MTM zn 13','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26894','NAD83(CSRS) / MTM zn 14','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26895','NAD83(CSRS) / MTM zn 15','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26896','NAD83(CSRS) / MTM zn 16','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26897','NAD83(CSRS) / MTM zn 17','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3799','NAD83(CSRS) / MTQ LCC','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3581','NAD83(CSRS) / NWT LCC','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3761','NAD83(CSRS) / UTM 22N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3579','NAD83(CSRS) / YT Albers','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3441','NAD83(HARN) / AR N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3442','NAD83(HARN) / AR S (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3760','NAD83(HARN) / HI 3 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3443','NAD83(HARN) / IL E (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3444','NAD83(HARN) / IL W (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3425','NAD83(HARN) / IA N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3426','NAD83(HARN) / IA S (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3427','NAD83(HARN) / KS N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3428','NAD83(HARN) / KS S (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3456','NAD83(HARN) / LA N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3457','NAD83(HARN) / LA S (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26855','NAD83(HARN) / ME E (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26856','NAD83(HARN) / ME W (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3079','NAD83(HARN) / MI Georef','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26858','NAD83(HARN) / MN C (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26857','NAD83(HARN) / MN N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26859','NAD83(HARN) / MN S (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26860','NAD83(HARN) / NE (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3430','NAD83(HARN) / NV C (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3429','NAD83(HARN) / NV E (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3431','NAD83(HARN) / NV W (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3445','NAD83(HARN) / NH (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3432','NAD83(HARN) / NJ (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3358','NAD83(HARN) / NC (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3404','NAD83(HARN) / NC (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3753','NAD83(HARN) / OH N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3754','NAD83(HARN) / OH S (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3362','NAD83(HARN) / PA N (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3363','NAD83(HARN) / PA N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3364','NAD83(HARN) / PA S (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3365','NAD83(HARN) / PA S (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3446','NAD83(HARN) / RI (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3360','NAD83(HARN) / SC (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3361','NAD83(HARN) / SC (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3458','NAD83(HARN) / SD N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3459','NAD83(HARN) / SD S (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3569','NAD83(HARN) / UT C (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3568','NAD83(HARN) / UT N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3570','NAD83(HARN) / UT S (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3740','NAD83(HARN) / UTM 10N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3741','NAD83(HARN) / UTM 11N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3742','NAD83(HARN) / UTM 12N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3743','NAD83(HARN) / UTM 13N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3744','NAD83(HARN) / UTM 14N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3745','NAD83(HARN) / UTM 15N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3746','NAD83(HARN) / UTM 16N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3747','NAD83(HARN) / UTM 17N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3748','NAD83(HARN) / UTM 18N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3749','NAD83(HARN) / UTM 19N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3750','NAD83(HARN) / UTM 4N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3751','NAD83(HARN) / UTM 5N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26861','NAD83(HARN) / WV N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26862','NAD83(HARN) / WV S (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3755','NAD83(HARN) / WY E (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3756','NAD83(HARN) / WY EC (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3758','NAD83(HARN) / WY W (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3757','NAD83(HARN) / WY WC (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3757','NAD83(HPGN) / Wyoming West Central (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3758','NAD83(HPGN) / Wyoming West (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3756','NAD83(HPGN) / Wyoming East Central (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3755','NAD83(HPGN) / Wyoming East (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3754','NAD83(HPGN) / Ohio South (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3753','NAD83(HPGN) / Ohio North (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3336','Kerguelen 62 / UTM 42S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3337','Le Pouce 34 / Mauritius','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3339','IGCB 1955 / Congo TM 12','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3340','IGCB 1955 / Congo TM 14','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3341','IGCB 1955 / Congo TM 16','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3343','Mauritania 99 / UTM 28N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3344','Mauritania 99 / UTM 29N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3345','Mauritania 99 / UTM 30N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3350','Pulkovo 1942 / CS63 C0','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3351','Pulkovo 1942 / CS63 C1','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3352','Pulkovo 1942 / CS63 C2','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3354','Mhast offshore / UTM 32S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3353','Mhast onshore / UTM 32S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3356','G Cayman 1959 / UTM 17N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3357','L Cayman 1961 / UTM 17N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3367','IGN Astro 1960 / UTM 28N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3368','IGN Astro 1960 / UTM 29N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3369','IGN Astro 1960 / UTM 30N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3376','GDM2000 / E Malaysia RSO','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3378','GDM2000 / Melaka Grid','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3381','GDM2000 / Terengganu','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3383','GDM2000 / Kedah Grid','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3388','Pulkovo 1942 / Caspian','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4974','278','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3396','PD/83 / 3GK zone 3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3397','PD/83 / 3GK zone 4','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3398','RD/83 / 3GK zone 4','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3399','RD/83 / 3GK zone 5','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3403','NAD83(CSRS) / AB 10TM R','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3402','NAD83(CSRS) / AB 10TM F','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3401','NAD83 / AB 10TM Resource','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3400','NAD83 / AB 10TM Forest','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3407','Hong Kong 1963 Grid','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3411','NSIDC Sea Ice PS North','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3412','NSIDC Sea Ice PS South','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3413','WGS 84 / NSIDC PS North','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3417','NAD83 / Iowa N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3418','NAD83 / Iowa S (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3419','NAD83 / Kansas N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3420','NAD83 / Kansas S (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3421','NAD83 / Nevada E (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3422','NAD83 / Nevada C (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3423','NAD83 / Nevada W (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3433','NAD83 / Arkansas N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3434','NAD83 / Arkansas S (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3435','NAD83 / Illinois E (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3436','NAD83 / Illinois W (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3437','NAD83 / NH (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3438','NAD83 / RI (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3448','JAD2001 / JA Metric Grid','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3451','NAD83 / Louisiana N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3452','NAD83 / Louisiana S (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3453','NAD83 / LA Offshore (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3454','NAD83 / S Dakota N ftUS','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3455','NAD83 / S Dakota S (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3460','Fiji 1986 / Map Grid','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3461','Dabola 1981 / UTM 28N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3462','Dabola 1981 / UTM 29N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3566','NAD83 / Utah Cen (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3571','WGS 84 / LAEA Bering Sea','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3572','WGS 84 / LAEA Alaska','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3573','WGS 84 / LAEA Canada','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3574','WGS 84 / LAEA Atlantic','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3575','WGS 84 / LAEA Europe','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3576','WGS 84 / LAEA Russia','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3577','GDA94 / Aus Albers','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3706','NAD83(NSRS) / UTM 59N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3707','NAD83(NSRS) / UTM 60N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3708','NAD83(NSRS) / UTM 1N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3709','NAD83(NSRS) / UTM 2N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3710','NAD83(NSRS) / UTM 3N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3711','NAD83(NSRS) / UTM 4N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3712','NAD83(NSRS) / UTM 5N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3713','NAD83(NSRS) / UTM 6N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3714','NAD83(NSRS) / UTM 7N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3715','NAD83(NSRS) / UTM 8N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3716','NAD83(NSRS) / UTM 9N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3717','NAD83(NSRS) / UTM 10N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3718','NAD83(NSRS) / UTM 11N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3719','NAD83(NSRS) / UTM 12N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3720','NAD83(NSRS) / UTM 13N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3721','NAD83(NSRS) / UTM 14N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3722','NAD83(NSRS) / UTM 15N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3723','NAD83(NSRS) / UTM 16N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3724','NAD83(NSRS) / UTM 17N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3725','NAD83(NSRS) / UTM 18N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3726','NAD83(NSRS) / UTM 19N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3734','NAD83 / Ohio N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3735','NAD83 / Ohio S (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3736','NAD83 / Wyoming E (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3737','NAD83 / Wyoming EC (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3738','NAD83 / Wyoming WC (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3739','NAD83 / Wyoming W (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3759','NAD83 / Hawaii 3 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3762','WGS 84 / S Georgia LCC','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3764','NZGD2000 / CI Circuit','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3771','NAD27 / Alberta 3TM 111','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3772','NAD27 / Alberta 3TM 114','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3773','NAD27 / Alberta 3TM 117','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3800','NAD27 / Alberta 3TM 112','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3775','NAD83 / Alberta 3TM 111','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3776','NAD83 / Alberta 3TM 114','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3777','NAD83 / Alberta 3TM 117','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3801','NAD83 / Alberta 3TM 120','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3783','Pitcairn 2006 / TM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3784','Pitcairn 1967 / UTM 9S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3785','Popular Vis CRS / Merc','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3786','World Equidistant Cyl','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3829','Hu Tzu Shan / UTM 51N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3845','SWEREF99 / RT90 7.5 gon V','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3846','SWEREF99 / RT90 5 gon V','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3847','SWEREF99 / RT90 2.5 gon V','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3848','SWEREF99 / RT90 0 gon','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3849','SWEREF99 / RT90 2.5 gon O','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3850','SWEREF99 / RT90 5 gon O','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3920','Puerto Rico / UTM 20N','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5607','Bora Bora 2001 height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5733','Dansk Normal Nul height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5736','YS56 height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5737','YS85 height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5753','Nivellement General de Nouvelle Caledonie','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5755','Nivellement General Guyanais 1977','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20436','Ain el Abd / UTM 36N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20440','Ain el Abd / UTM 40N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20791','DLx','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21453','Beijing 1954 / GK CM 75E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21454','Beijing 1954 / GK CM 81E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21455','Beijing 1954 / GK CM 87E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21456','Beijing 1954 / GK CM 93E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21457','Beijing 1954 / GK CM 99E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21458','Beijing 1954 / GK CM 105E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21459','Beijing 1954 / GK CM 111E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21460','Beijing 1954 / GK CM 117E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21461','Beijing 1954 / GK CM 123E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21462','Beijing 1954 / GK CM 129E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21463','Beijing 1954 / GK CM 135E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21818','Bogota 1975 / UTM 18N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22521','Corrego Alegre / UTM 21S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22522','Corrego Alegre / UTM 22S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22525','Corrego Alegre / UTM 25S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','23830','DGN95 / Indonesia 46.2','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','23831','DGN95 / Indonesia 47.1','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','23832','DGN95 / Indonesia 47.2','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','23833','DGN95 / Indonesia 48.1','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','23834','DGN95 / Indonesia 48.2','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','23835','DGN95 / Indonesia 49.1','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','23836','DGN95 / Indonesia 49.2','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','23837','DGN95 / Indonesia 50.1','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','23838','DGN95 / Indonesia 50.2','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','23839','DGN95 / Indonesia 51.1','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','23840','DGN95 / Indonesia 51.2','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','23841','DGN95 / Indonesia 52.1','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','23842','DGN95 / Indonesia 52.2','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','23843','DGN95 / Indonesia 53.1','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','23844','DGN95 / Indonesia 53.2','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','23845','DGN95 / Indonesia 54.1','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26847','NAD83 / Maine E (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26848','NAD83 / Maine W (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26849','NAD83 / Minnesota N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26850','NAD83 / Minnesota C (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26851','NAD83 / Minnesota S (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26853','NAD83 / WV North (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26854','NAD83 / WV South (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','27037','Nahrwan 1967 / UTM 37N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','27205','NZGD49 / Mount Eden','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','27206','NZGD49 / Bay of Plenty','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','27207','NZGD49 / Poverty Bay','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','27208','NZGD49 / Hawkes Bay','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','27209','NZGD49 / Taranaki','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','27210','NZGD49 / Tuhirangi','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','27211','NZGD49 / Wanganui','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','27212','NZGD49 / Wairarapa','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','27213','NZGD49 / Wellington','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','27214','NZGD49 / Collingwood','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','27220','NZGD49 / Marlborough','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','27221','NZGD49 / Hokitika','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','27223','NZGD49 / Jacksons Bay','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','27224','NZGD49 / Mount Pleasant','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','27227','NZGD49 / Lindis Peak','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','27228','NZGD49 / Mount Nicholas','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','27229','NZGD49 / Mount York','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','27230','NZGD49 / Observation Pt','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','27231','NZGD49 / North Taieri','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','27493','Datum 73 / Modified Grid','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28404','Pulkovo 1942 / GK zone 4','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28405','Pulkovo 1942 / GK zone 5','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','29901','OSNI 52 / Irish Nat Grid','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','30179','Tokyo / Japan zone XIX','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','30493','Voirol79 / N Algeria old','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','30494','Voirol79 / S Algeria old','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31251','MGI (Ferro) / Aut GK W','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31252','MGI (Ferro) / Aut GK C','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31253','MGI (Ferro) / Aut GK E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31466','DHDN / 3GK zone 2','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31467','DHDN / 3GK zone 3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31468','DHDN / 3GK zone 4','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31469','DHDN / 3GK zone 5','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31528','Conakry 1905 / UTM 28N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31529','Conakry 1905 / UTM 29N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31965','SIRGAS 2000 / UTM 11N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31966','SIRGAS 2000 / UTM 12N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31967','SIRGAS 2000 / UTM 13N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31968','SIRGAS 2000 / UTM 14N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31969','SIRGAS 2000 / UTM 15N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31970','SIRGAS 2000 / UTM 16N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31971','SIRGAS 2000 / UTM 17N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31972','SIRGAS 2000 / UTM 18N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31973','SIRGAS 2000 / UTM 19N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31974','SIRGAS 2000 / UTM 20N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31975','SIRGAS 2000 / UTM 21N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31976','SIRGAS 2000 / UTM 22N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31977','SIRGAS 2000 / UTM 17S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31978','SIRGAS 2000 / UTM 18S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31979','SIRGAS 2000 / UTM 19S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31980','SIRGAS 2000 / UTM 20S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31981','SIRGAS 2000 / UTM 21S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31982','SIRGAS 2000 / UTM 22S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31983','SIRGAS 2000 / UTM 23S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31984','SIRGAS 2000 / UTM 24S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31985','SIRGAS 2000 / UTM 25S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32099','NAD27 / Louisiana Off','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32199','NAD83 / Louisiana Off m','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32600','WGS 84 / UTM system N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32700','WGS 84 / UTM system S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32663','WGS 84 / Equidistant Cyl','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3465','NAD83(NSRS) / AL E (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3466','NAD83(NSRS) / AL W (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3484','NAD83(NSRS) / AR N (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3485','NAD83(NSRS) / AR N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3486','NAD83(NSRS) / AR S (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3487','NAD83(NSRS) / AR S (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3479','NAD83(NSRS) / AZ C (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3478','NAD83(NSRS) / AZ C (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3481','NAD83(NSRS) / AZ E (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3480','NAD83(NSRS) / AZ E (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3483','NAD83(NSRS) / AZ W (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3482','NAD83(NSRS) / AZ W (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3489','NAD83(NSRS) / CA 1 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3490','NAD83(NSRS) / CA 1 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3491','NAD83(NSRS) / CA 2 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3492','NAD83(NSRS) / CA 2 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3493','NAD83(NSRS) / CA 3 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3494','NAD83(NSRS) / CA 3 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3495','NAD83(NSRS) / CA 4 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3496','NAD83(NSRS) / CA 4 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3497','NAD83(NSRS) / CA 5 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3498','NAD83(NSRS) / CA 5 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3499','NAD83(NSRS) / CA 6 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3500','NAD83(NSRS) / CA 6 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3488','NAD83(NSRS) / CA Albers','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3501','NAD83(NSRS) / CO C (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3502','NAD83(NSRS) / CO C (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3503','NAD83(NSRS) / CO N (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3504','NAD83(NSRS) / CO N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3505','NAD83(NSRS) / CO S (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3506','NAD83(NSRS) / CO S (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3508','NAD83(NSRS) / CT (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3507','NAD83(NSRS) / CT (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3510','NAD83(NSRS) / DE (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3511','NAD83(NSRS) / FL E (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3512','NAD83(NSRS) / FL E (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3513','NAD83(NSRS) / FL GDL AEA','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3514','NAD83(NSRS) / FL N (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3515','NAD83(NSRS) / FL N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3516','NAD83(NSRS) / FL W (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3517','NAD83(NSRS) / FL W (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3518','NAD83(NSRS) / GA E (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3519','NAD83(NSRS) / GA E (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3520','NAD83(NSRS) / GA W (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3521','NAD83(NSRS) / GA W (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3537','NAD83(NSRS) / IA N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3539','NAD83(NSRS) / IA S (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3522','NAD83(NSRS) / ID C (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3523','NAD83(NSRS) / ID C (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3525','NAD83(NSRS) / ID E (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3527','NAD83(NSRS) / ID W (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3528','NAD83(NSRS) / IL E (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3529','NAD83(NSRS) / IL E (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3530','NAD83(NSRS) / IL W (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3531','NAD83(NSRS) / IL W (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3532','NAD83(NSRS) / IN E (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3533','NAD83(NSRS) / IN E (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3534','NAD83(NSRS) / IN W (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3535','NAD83(NSRS) / IN W (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3540','NAD83(NSRS) / KS N (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3541','NAD83(NSRS) / KS N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3542','NAD83(NSRS) / KS S (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3543','NAD83(NSRS) / KS S (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3544','NAD83(NSRS) / KY N (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3545','NAD83(NSRS) / KY N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3548','NAD83(NSRS) / KY S (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3549','NAD83(NSRS) / KY S (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3546','NAD83(NSRS) / KY1Z (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3550','NAD83(NSRS) / LA N (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3551','NAD83(NSRS) / LA N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3552','NAD83(NSRS) / LA S (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3553','NAD83(NSRS) / LA S (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3583','NAD83(NSRS) / MA Is (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3584','NAD83(NSRS) / MA Is (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3585','NAD83(NSRS) / MA md (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3586','NAD83(NSRS) / MA md (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3582','NAD83(NSRS) / MD (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3554','NAD83(NSRS) / ME 2000 C','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3555','NAD83(NSRS) / ME 2000 E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3556','NAD83(NSRS) / ME 2000 W','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3557','NAD83(NSRS) / ME83 E (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26863','NAD83(NSRS) / ME E (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3558','NAD83(NSRS) / ME83 W (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26864','NAD83(NSRS) / ME W (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3588','NAD83(NSRS) / MI C (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3587','NAD83(NSRS) / MI C (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3591','NAD83(NSRS) / MI Georef','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3590','NAD83(NSRS) / MI N (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3589','NAD83(NSRS) / MI N (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3593','NAD83(NSRS) / MI S (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3592','NAD83(NSRS) / MI S (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3594','NAD83(NSRS) / MN C (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26866','NAD83(NSRS) / MN C (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3595','NAD83(NSRS) / MN N (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26865','NAD83(NSRS) / MN N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3596','NAD83(NSRS) / MN S (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26867','NAD83(NSRS) / MN S (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3601','NAD83(NSRS) / MO C (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3602','NAD83(NSRS) / MO E (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3603','NAD83(NSRS) / MO W (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3597','NAD83(NSRS) / MS E (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3598','NAD83(NSRS) / MS E (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3599','NAD83(NSRS) / MS W (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3600','NAD83(NSRS) / MS W (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3816','NAD83(NSRS) / MSTM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3605','NAD83(NSRS) / MT (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3632','NAD83(NSRS) / NC (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3631','NAD83(NSRS) / NC (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3634','NAD83(NSRS) / ND N (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3633','NAD83(NSRS) / ND N (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3636','NAD83(NSRS) / ND S (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3635','NAD83(NSRS) / ND S (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26868','NAD83(NSRS) / NE (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3606','NAD83(NSRS) / NE (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3614','NAD83(NSRS) / NH (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3613','NAD83(NSRS) / NH (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3616','NAD83(NSRS) / NJ (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3617','NAD83(NSRS) / NM C (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3618','NAD83(NSRS) / NM C (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3619','NAD83(NSRS) / NM E (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3620','NAD83(NSRS) / NM E (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3621','NAD83(NSRS) / NM W (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3622','NAD83(NSRS) / NM W (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3607','NAD83(NSRS) / NV C (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3608','NAD83(NSRS) / NV C (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3609','NAD83(NSRS) / NV E (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3610','NAD83(NSRS) / NV E (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3611','NAD83(NSRS) / NV W (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3612','NAD83(NSRS) / NV W (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3623','NAD83(NSRS) / NY C (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3624','NAD83(NSRS) / NY C (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3625','NAD83(NSRS) / NY E (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3626','NAD83(NSRS) / NY E (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3627','NAD83(NSRS) / NY LI (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3628','NAD83(NSRS) / NY LI (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3629','NAD83(NSRS) / NY W (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3630','NAD83(NSRS) / NY W (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3728','NAD83(NSRS) / OH N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3729','NAD83(NSRS) / OH S (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3639','NAD83(NSRS) / OK N (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3640','NAD83(NSRS) / OK N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3641','NAD83(NSRS) / OK S (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3642','NAD83(NSRS) / OK S (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3646','NAD83(NSRS) / OR N (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3645','NAD83(NSRS) / OR N (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3648','NAD83(NSRS) / OR S (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3647','NAD83(NSRS) / OR S (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3644','NAD83(2007) / OR GIC Lam (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3649','NAD83(NSRS) / PA N (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3650','NAD83(NSRS) / PA N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3651','NAD83(NSRS) / PA S (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3652','NAD83(NSRS) / PA S (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3654','NAD83(NSRS) / RI (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3653','NAD83(NSRS) / RI (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3656','NAD83(NSRS) / SC (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3655','NAD83(NSRS) / SC (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3657','NAD83(NSRS) / SD N (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3658','NAD83(NSRS) / SD N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3659','NAD83(NSRS) / SD S (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3660','NAD83(NSRS) / SD S (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3662','NAD83(NSRS) / TN (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3663','NAD83(NSRS) / TX C (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3664','NAD83(NSRS) / TX C ftUS','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3667','NAD83(NSRS) / TX N (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3668','NAD83(NSRS) / TX N ftUS','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3669','NAD83(NSRS) / TX NC (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3670','NAD83(NSRS) / TX NC ftUS','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3671','NAD83(NSRS) / TX S (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3672','NAD83(NSRS) / TX S ftUS','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3673','NAD83(NSRS) / TX SC (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3674','NAD83(NSRS) / TX SC ftUS','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3676','NAD83(NSRS) / UT C (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3675','NAD83(NSRS) / UT C (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3677','NAD83(NSRS) / UT C (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3679','NAD83(NSRS) / UT N (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3680','NAD83(NSRS) / UT N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3682','NAD83(NSRS) / UT S (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3683','NAD83(NSRS) / UT S (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3685','NAD83(NSRS) / VA N (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3686','NAD83(NSRS) / VA N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3687','NAD83(NSRS) / VA S (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3688','NAD83(NSRS) / VA S (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3689','NAD83(NSRS) / WA N (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3690','NAD83(NSRS) / WA N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3691','NAD83(NSRS) / WA S (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3692','NAD83(NSRS) / WA S (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3695','NAD83(NSRS) / WI C (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3696','NAD83(NSRS) / WI C (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3697','NAD83(NSRS) / WI N (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3698','NAD83(NSRS) / WI N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3699','NAD83(NSRS) / WI S (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3700','NAD83(NSRS) / WI S (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3701','NAD83(NSRS) / WTM 83','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3693','NAD83(NSRS) / WV N (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26869','NAD83(NSRS) / WV N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3694','NAD83(NSRS) / WV S (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','26870','NAD83(NSRS) / WV S (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3702','NAD83(NSRS) / WY E (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3730','NAD83(NSRS) / WY E (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3703','NAD83(NSRS) / WY EC (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3731','NAD83(NSRS) / WY EC (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3705','NAD83(NSRS) / WY W (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3733','NAD83(NSRS) / WY W (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3704','NAD83(NSRS) / WY WC (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3732','NAD83(NSRS) / WY WC (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3467','NAD83(NSRS) / AK Alb','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3468','NAD83(NSRS) / AK 1 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3469','NAD83(NSRS) / AK 2 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3470','NAD83(NSRS) / AK 3 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3471','NAD83(NSRS) / AK 4 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3472','NAD83(NSRS) / AK 5 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3473','NAD83(NSRS) / AK 6 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3474','NAD83(NSRS) / AK 7 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3475','NAD83(NSRS) / AK 8 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3476','NAD83(NSRS) / AK 9 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3477','NAD83(NSRS) / AK 10 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3509','NAD83(NSRS) / DE (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3524','NAD83(NSRS) / ID E (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3526','NAD83(NSRS) / ID W (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3536','NAD83(NSRS) / IA N (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3538','NAD83(NSRS) / IA S (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3559','NAD83(NSRS) / MD (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3604','NAD83(NSRS) / MT (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3615','NAD83(NSRS) / NJ (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3661','NAD83(NSRS) / TN (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3665','NAD83(NSRS) / TX Albers','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3666','NAD83(NSRS) / TX LC','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3678','NAD83(NSRS) / UT N (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3681','NAD83(NSRS) / UT S (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3684','NAD83(NSRS) / VT (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2790','NAD83(HARN) / Illinois East (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3084','NAD83(HARN) / TX LC','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3083','TCMS/AEA','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3082','NAD83 / TX LC','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3085','NAD83(HARN) / TX Albers','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3091','KY1Z','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3090','NAD83(HARN) / KY1Z','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3088','NAD83 / KY1Z (m)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9379','726','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3857','WGS 84 / Popular Visualisation Pseudo-Mercator','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9379','IGb14 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','3886','National Elevation Network height','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3893','ED50 / Iraq Nat. Grid','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4743','Karbala 1979 (Polservice)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3391','Karbala 1979 (Polservice) / UTM zone 37N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3392','Karbala 1979 (Polservice) / UTM zone 38N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3393','Karbala 1979 (Polservice) / UTM zone 39N','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4191','ALB86','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2462','ALB86 / GK zn 4','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4129','Observatario Campos Rodrigues 1907','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6984','IG05','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6962','ETRS89 / Albania LCC','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6991','IG05/12','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6870','ETRS89 / Albania TM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2039','Israel / Israeli TM Grid','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6796','NAD83(CORS96) / Oregon Bend-Vale zone (m)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4141','Israel','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2039','Israeli New Grid','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6997','NAD83(2011) / CCSF-CS13 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5843','Abu Dhabi height','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6996','NAD83(2011) / San Francisco CS13 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6996','NAD83(2011) / CCSF-CS13','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7006','Nahrwan 1934 / UTM 38N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7007','Nahrwan 1934 / UTM 39N','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4201','Blue Nile 1958','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20135','Blue Nile 1958 / UTM zone 35N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20136','Blue Nile 1958 / UTM zone 36N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20137','Blue Nile 1958 / UTM zone 37N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20138','Blue Nile 1958 / UTM zone 38N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6307','NAD83(CORS96) / PR and VI','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6312','CGRS93 / Local Transverse Mercator (LTM)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4171','RGF93 v1 (lat-lon)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4965','RGF93 v1 (lat-lon)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7057','NAD83(2011) / IaRCS zone 1 Spencer','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7058','NAD83(2011) / IaRCS zone 2 Mason City','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7059','NAD83(2011) / IaRCS zone 3 Elkader','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','5489','RGAF09 (lat-lon)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7060','NAD83(2011) / IaRCS zone 4 Sioux City-Iowa Falls','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4624','RGFG95 (lat-lon)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7061','NAD83(2011) / IaRCS zone 5 Waterloo','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4627','RGR92 (lat-lon)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7062','NAD83(2011) / IaRCS zone 6 Council Bluffs','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4470','RGM04 (lat-lon)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7063','NAD83(2011) / IaRCS zone 7 Carroll-Atlantic','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4463','RGSPM06 (lat-lon)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7064','NAD83(2011) / IaRCS zone 8 Ames-Des Moines','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4466','RGSPM06 (lat-lon)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7065','NAD83(2011) / IaRCS zone 9 Newton','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4469','RGM04 (lat-lon)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7066','NAD83(2011) / IaRCS zone 10 Cedar Rapids','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4971','RGR92 (lat-lon)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7067','NAD83(2011) / IaRCS zone 11 Dubuque-Davenport','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4967','RGFG95 (lat-lon)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7068','NAD83(2011) / IaRCS zone 12 Red Oak-Ottumwa','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','5488','RGAF09 (lat-lon)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7069','NAD83(2011) / IaRCS zone 13 Fairfield','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7073','RGTAAF07 (lat-lon)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7070','NAD83(2011) / IaRCS zone 14 Burlington','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7072','RGTAAF07 (lat-lon)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7109','NAD83(2011) / St Mary (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7114','NAD83(2011) / Ft Peck Sx (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7110','NAD83(2011) / Blackfeet (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7125','NAD83(2011) / Crow (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7111','NAD83(2011) / Milk R (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7115','NAD83(2011) / Crow (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7112','NAD83(2011) / Ft Belknap (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7278','InGCS 2011 11 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7113','NAD83(2011) / Ft Peck As (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7116','NAD83(2011) / Bobcat (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7126','NAD83(2011) / Bobcat (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7117','NAD83(2011) / Billings (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7118','NAD83(2011) / Wind R (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7127','NAD83(2011) / Billings (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7119','NAD83(2011) / St Mary (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7120','NAD83(2011) / Blackfeet (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7128','NAD83(2011) / Wind R (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7121','NAD83(2011) / Milk R (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7122','NAD83(2011) / Ft Belknap (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7123','NAD83(2011) / Ft Peck As (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7124','NAD83(2011) / Ft Peck Sx (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7279','InGCS 2011 12 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7267','InGCS 2011 06-32 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7142','Palestine Grid modified','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7267','NAD83(2011) / InGCS Boone (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7267','NAD83(2011) / InGCS Hendricks (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7259','InGCS 2011 02 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7323','InGCS 2011 39 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7131','NAD83(2011) / San Francisco CS13 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7131','NAD83(2011) / CCSF-CS13','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7132','NAD83(2011) / CCSF-CS13 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3973','WGS 84 / NSIDC EA North','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3974','WGS 84 / NSIDC EA South','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3975','WGS 84 / NSIDC EA Global','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3976','WGS 84 / NSIDC PS South','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3979','NAD83(CSRS) / Canada LCC','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3986','Katanga 1955 / TM zone A','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3987','Katanga 1955 / TM zone B','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7280','InGCS 2011 12 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7349','InGCS 2011 65 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7268','InGCS 2011 06-32 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7268','NAD83(2011) / InGCS Boone (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7268','NAD83(2011) / InGCS Hendricks (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7297','InGCS 2011 23-86 (m)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','3906','D48','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7257','InGCS 2011 01 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7294','InGCS 2011 20-43-85 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7294','NAD83(2011) / InGCS Elkhart (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7281','InGCS 2011 13-47-59 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7260','InGCS 2011 02 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7269','InGCS 2011 07 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7281','NAD83(2011) / InGCS Crawford (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7270','InGCS 2011 07 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7258','InGCS 2011 01 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7281','NAD83(2011) / InGCS Lawrence (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7281','NAD83(2011) / InGCS Orange (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7294','NAD83(2011) / InGCS Kosciusko (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7294','NAD83(2011) / InGCS Wabash (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7261','InGCS 2011 03 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7271','InGCS 2011 08 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7262','InGCS 2011 03 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7295','InGCS 2011 21-24-81 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7263','InGCS 2011 04 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7272','InGCS 2011 08 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7264','InGCS 2011 04 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7295','NAD83(2011) / InGCS Fayette (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7273','InGCS 2011 09 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7282','InGCS 2011 13-47-59 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7274','InGCS 2011 09 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7265','InGCS 2011 05-18 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7265','NAD83(2011) / InGCS Blackford (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7265','NAD83(2011) / InGCS Delaware (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7282','NAD83(2011) / InGCS Crawford (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7282','NAD83(2011) / InGCS Lawrence (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7282','NAD83(2011) / InGCS Orange (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7295','NAD83(2011) / InGCS Franklin (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7275','InGCS 2011 10-22-72 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7266','InGCS 2011 05-18 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7266','NAD83(2011) / InGCS Blackford (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7266','NAD83(2011) / InGCS Delaware (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7275','NAD83(2011) / InGCS Clark (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7275','NAD83(2011) / InGCS Floyd (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7275','NAD83(2011) / InGCS Scott (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7295','NAD83(2011) / InGCS Union (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7297','NAD83(2011) / InGCS Fountain (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3043','ETRS89 / UTM zone 31','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7283','InGCS 2011 14-28 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3043','DE_ETRS89 / UTM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7283','NAD83(2011) / InGCS Daviess (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3044','DE_ETRS89 / UTM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3044','ETRS89 / UTM zone 32','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7276','InGCS 2011 10-22-72 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7276','NAD83(2011) / InGCS Clark (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3045','ETRS89 / UTM zone 3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7276','NAD83(2011) / InGCS Floyd (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7276','NAD83(2011) / InGCS Scott (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7283','NAD83(2011) / InGCS Greene (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7277','InGCS 2011 11 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7297','NAD83(2011) / InGCS Warren (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7309','InGCS 2011 31-88 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7309','NAD83(2011) / InGCS Harrison (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7284','InGCS 2011 14-28 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3045','DE_ETRS89 / UTM_BB','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3045','ETRS89 / UTM zone 33','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7284','NAD83(2011) / InGCS Daviess (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7284','NAD83(2011) / InGCS Greene (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7296','InGCS 2011 21-24-81 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7296','NAD83(2011) / InGCS Fayette (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7296','NAD83(2011) / InGCS Franklin (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7296','NAD83(2011) / InGCS Union (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7285','InGCS 2011 15-58-78 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7285','NAD83(2011) / InGCS Dearborn (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7285','NAD83(2011) / InGCS Ohio (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7285','NAD83(2011) / InGCS Switzerland (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7309','NAD83(2011) / InGCS Washington (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7298','InGCS 2011 23-86 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7298','NAD83(2011) / InGCS Fountain (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7298','NAD83(2011) / InGCS Warren (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7286','InGCS 2011 15-58-78 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7286','NAD83(2011) / InGCS Dearborn (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7286','NAD83(2011) / InGCS Ohio (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7286','NAD83(2011) / InGCS Switzerland (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7315','InGCS 2011 35-92 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7315','NAD83(2011) / InGCS Huntington (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7315','NAD83(2011) / InGCS Whitley (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7287','InGCS 2011 16-70 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7287','NAD83(2011) / InGCS Decatur (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7287','NAD83(2011) / InGCS Rush (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7310','InGCS 2011 31-88 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7299','InGCS 2011 25-50-71 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7299','NAD83(2011) / InGCS Fulton (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7288','InGCS 2011 16-70 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7288','NAD83(2011) / InGCS Decatur (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7288','NAD83(2011) / InGCS Rush (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7299','NAD83(2011) / InGCS Marshall (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7289','InGCS 2011 17 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7299','NAD83(2011) / InGCS St. Joseph (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7290','InGCS 2011 17 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7310','NAD83(2011) / InGCS Harrison (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7310','NAD83(2011) / InGCS Washington (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7324','InGCS 2011 39 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7291','InGCS 2011 19-51 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7291','NAD83(2011) / InGCS Dubois (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7291','NAD83(2011) / InGCS Martin (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7311','InGCS 2011 33 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7300','InGCS 2011 25-50-71 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7300','NAD83(2011) / InGCS Fulton (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7292','InGCS 2011 19-51 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7292','NAD83(2011) / InGCS Dubois (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7292','NAD83(2011) / InGCS Martin (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7300','NAD83(2011) / InGCS Marshall (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7300','NAD83(2011) / InGCS St. Joseph (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7359','InGCS 2011 76 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7301','InGCS 2011 26 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7293','InGCS 2011 20-43-85 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7293','NAD83(2011) / InGCS Elkhart (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7293','NAD83(2011) / InGCS Kosciusko (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7293','NAD83(2011) / InGCS Wabash (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7312','InGCS 2011 33 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7302','InGCS 2011 26 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7325','InGCS 2011 40 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7303','InGCS 2011 27 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3970','NAD83(NSRS) / VA LCC','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7316','InGCS 2011 35-92 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3969','NAD83(HARN) / VA LCC','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3969','NAD83(HPGN) / Virginia Lambert','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7304','InGCS 2011 27 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7316','NAD83(2011) / InGCS Huntington (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7313','InGCS 2011 34-52 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7313','NAD83(2011) / InGCS Howard (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7305','InGCS 2011 29-80 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7305','NAD83(2011) / InGCS Hamilton (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7305','NAD83(2011) / InGCS Tipton (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7313','NAD83(2011) / InGCS Miami (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7316','NAD83(2011) / InGCS Whitley (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7333','InGCS 2011 45-56 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7306','InGCS 2011 29-80 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7306','NAD83(2011) / InGCS Hamilton (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7306','NAD83(2011) / InGCS Tipton (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7317','InGCS 2011 36 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7314','InGCS 2011 34-52 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7314','NAD83(2011) / InGCS Howard (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7307','InGCS 2011 30-48 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7307','NAD83(2011) / InGCS Hancock (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7307','NAD83(2011) / InGCS Madison (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7314','NAD83(2011) / InGCS Miami (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7326','InGCS 2011 40 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7318','InGCS 2011 36 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7308','InGCS 2011 30-48 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7308','NAD83(2011) / InGCS Hancock (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7308','NAD83(2011) / InGCS Madison (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7333','NAD83(2011) / InGCS Lake (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7333','NAD83(2011) / InGCS Newton (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7339','InGCS 2011 54-67 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7319','InGCS 2011 37-64 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7319','NAD83(2011) / InGCS Jasper (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7319','NAD83(2011) / InGCS Porter (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7327','InGCS 2011 41-49 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7327','NAD83(2011) / InGCS Johnson (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7327','NAD83(2011) / InGCS Marion (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7320','InGCS 2011 37-64 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7320','NAD83(2011) / InGCS Jasper (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7320','NAD83(2011) / InGCS Porter (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7339','NAD83(2011) / InGCS Montgomery (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7321','InGCS 2011 38 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7339','NAD83(2011) / InGCS Putnam (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7322','InGCS 2011 38 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7334','InGCS 2011 45-56 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7328','InGCS 2011 41-49 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7328','NAD83(2011) / InGCS Johnson (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7328','NAD83(2011) / InGCS Marion (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7334','NAD83(2011) / InGCS Lake (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7329','InGCS 2011 42 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7334','NAD83(2011) / InGCS Newton (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7330','InGCS 2011 42 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7343','InGCS 2011 61-83 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7343','NAD83(2011) / InGCS Parke (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7343','NAD83(2011) / InGCS Vermillion (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7331','InGCS 2011 44-57 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7331','NAD83(2011) / InGCS LaGrange (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7331','NAD83(2011) / InGCS Noble (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7340','InGCS 2011 54-67 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7335','InGCS 2011 46-66-75 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7335','NAD83(2011) / InGCS LaPorte (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7332','InGCS 2011 44-57 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7332','NAD83(2011) / InGCS LaGrange (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7332','NAD83(2011) / InGCS Noble (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7335','NAD83(2011) / InGCS Pulaski (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7335','NAD83(2011) / InGCS Starke (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7340','NAD83(2011) / InGCS Montgomery (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4049','RGRDC 2005 / Congo TM 14','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7340','NAD83(2011) / InGCS Putnam (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4048','RGRDC 2005 / Congo TM 12','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7350','InGCS 2011 65 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7341','InGCS 2011 60 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7336','InGCS 2011 46-66-75 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7336','NAD83(2011) / InGCS LaPorte (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7336','NAD83(2011) / InGCS Pulaski (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7336','NAD83(2011) / InGCS Starke (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7342','InGCS 2011 60 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7360','InGCS 2011 76 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7337','InGCS 2011 53-55 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7337','NAD83(2011) / InGCS Monroe (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7337','NAD83(2011) / InGCS Morgan (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7344','InGCS 2011 61-83 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7344','NAD83(2011) / InGCS Parke (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7344','NAD83(2011) / InGCS Vermillion (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7338','InGCS 2011 53-55 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7338','NAD83(2011) / InGCS Monroe (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7338','NAD83(2011) / InGCS Morgan (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6312','CGRS93 / Cyprus LTM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7345','InGCS 2011 62 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7351','InGCS 2011 68-89 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7346','InGCS 2011 62 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7351','NAD83(2011) / InGCS Randolph (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7351','NAD83(2011) / InGCS Wayne (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7361','InGCS 2011 77 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7347','InGCS 2011 63-87 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7347','NAD83(2011) / InGCS Pike (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7347','NAD83(2011) / InGCS Warrick (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7362','InGCS 2011 77 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7352','InGCS 2011 68-89 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7348','InGCS 2011 63-87 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7348','NAD83(2011) / InGCS Pike (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7348','NAD83(2011) / InGCS Warrick (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7352','NAD83(2011) / InGCS Randolph (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7352','NAD83(2011) / InGCS Wayne (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7353','InGCS 2011 69 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7354','InGCS 2011 69 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7355','InGCS 2011 73 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7363','InGCS 2011 79-91 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7356','InGCS 2011 73 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7363','NAD83(2011) / InGCS Tippecanoe (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7357','InGCS 2011 74 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7363','NAD83(2011) / InGCS White (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7358','InGCS 2011 74 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7364','InGCS 2011 79-91 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7364','NAD83(2011) / InGCS Tippecanoe (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7364','NAD83(2011) / InGCS White (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4037','WGS 84 / UTM zone 35N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7365','InGCS 2011 82 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4038','WGS 84 / UTM zone 36N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7366','InGCS 2011 82 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7367','InGCS 2011 84 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7368','InGCS 2011 84 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7369','InGCS 2011 90 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7370','InGCS 2011 90 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7535','NAD83(2011) / WISCRS Outagamie (m)','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','7446','Cyprus vertical system','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7542','WISCRS Door (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7560','WISCRS Marinette (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7531','WISCRS Bayfield (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7535','NAD83(2011) / WISCRS Winnebago (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7528','WISCRS Adams-Juneau (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7528','NAD83(2011) / WISCRS Adams (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7528','NAD83(2011) / WISCRS Juneau (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7532','WISCRS Brown (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7546','WISCRS Florence (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7533','WISCRS Buffalo (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7536','WISCRS Chippewa (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7529','WISCRS Ashland (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7534','WISCRS Burnett (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7563','WISCRS Oconto (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7537','WISCRS Clark (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7547','WISCRS Forest (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7535','WISCRS Out-Cal-Win-FdL (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7530','WISCRS Barron (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7535','NAD83(2011) / WISCRS Calumet (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7535','NAD83(2011) / WISCRS Fond du Lac (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7561','WISCRS Menominee (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7548','WISCRS Grant (m)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4728','Pico de las Nieves 1984','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7538','WISCRS Columbia (m)','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5610','HR_HRVD71 / NOH','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7580','WISCRS Walworth (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7562','WISCRS Monroe (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7564','WISCRS Oneida (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7539','WISCRS Crawford (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7549','WISCRS Green-Lafayette (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7540','WISCRS Dane (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7549','NAD83(2011) / WISCRS Green (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7549','NAD83(2011) / WISCRS Lafayette (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7589','WISCRS Barron (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7541','WISCRS Dodge-Jefferson (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7541','NAD83(2011) / WISCRS Dodge (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7541','NAD83(2011) / WISCRS Jefferson (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7565','WISCRS Pepin-Pierce (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7565','NAD83(2011) / WISCRS Pepin (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7550','WISCRS Green Lake-Marquette (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7543','WISCRS Douglas (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7550','NAD83(2011) / WISCRS Green Lake (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7544','WISCRS Dunn (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7550','NAD83(2011) / WISCRS Marquette (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7545','WISCRS Eau Claire (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7565','NAD83(2011) / WISCRS Pierce (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7551','WISCRS Iowa (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7581','WISCRS Washburn (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7552','WISCRS Iron (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7566','WISCRS Polk (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7553','WISCRS Jackson (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7603','WISCRS Dunn (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7567','WISCRS Portage (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7582','WISCRS Washington (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7568','WISCRS Price (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7590','WISCRS Bayfield (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7554','WISCRS Oz-Milw-Rac-Ken (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7554','NAD83(2011) / WISCRS Kenosha(m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7554','NAD83(2011) / WISCRS Milwaukee (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7554','NAD83(2011) / WISCRS Ozaukee (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7554','NAD83(2011) / WISCRS Racine (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7569','WISCRS Richland (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7583','WISCRS Waukesha (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7570','WISCRS Rock (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7619','WISCRS Marinette (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7571','WISCRS Rusk (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7555','WISCRS Kew-Manit-Sheb (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7555','NAD83(2011) / WISCRS Kewaunee (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7555','NAD83(2011) / WISCRS Manitowoc (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7555','NAD83(2011) / WISCRS Sheboygan (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7584','WISCRS Waupaca (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7572','WISCRS Sauk (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7556','WISCRS La Crosse (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7591','WISCRS Brown (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7557','WISCRS Langlade (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7573','WISCRS Sawyer (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7558','WISCRS Lincoln (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7585','WISCRS Waushara (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7559','WISCRS Marathon (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7574','WISCRS Shawano (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7604','WISCRS Eau Claire (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7575','WISCRS St. Croix (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7586','WISCRS Wood (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7576','WISCRS Taylor (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7592','WISCRS Buffalo (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7577','WISCRS Trempealeau (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7632','WISCRS Sawyer (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7578','WISCRS Vernon (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7593','WISCRS Burnett (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7579','WISCRS Vilas (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7587','WISCRS Adams-Juneau (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7587','NAD83(2011) / WISCRS Adams (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7587','NAD83(2011) / WISCRS Juneau (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7605','WISCRS Florence (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7588','WISCRS Ashland (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7620','WISCRS Menominee (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7606','WISCRS Forest (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7594','WISCRS Out-Cal-Win-FdL (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7594','NAD83(2011) / WISCRS Calumet (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7594','NAD83(2011) / WISCRS Fond du Lac (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7594','NAD83(2011) / WISCRS Outagamie (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7594','NAD83(2011) / WISCRS Winnebago (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7595','WISCRS Chippewa (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7607','WISCRS Grant (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7596','WISCRS Clark (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7621','WISCRS Monroe (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7633','WISCRS Shawano (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7622','WISCRS Oconto (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7597','WISCRS Columbia (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3152','ST74 0 gon 65:-1','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7608','WISCRS Green-Lafayette (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7608','NAD83(2011) / WISCRS Green (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7608','NAD83(2011) / WISCRS Lafayette (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7598','WISCRS Crawford (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7599','WISCRS Dane (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7623','WISCRS Oneida (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7634','WISCRS St. Croix (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7609','WISCRS GreenLake-Marqt (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7600','WISCRS Dodge-Jefferson (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7600','NAD83(2011) / WISCRS Dodge (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7600','NAD83(2011) / WISCRS Jefferson (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7609','NAD83(2011) / WISCRS Green Lake (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7601','WISCRS Door (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7609','NAD83(2011) / WISCRS Marquette (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7602','WISCRS Douglas (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7624','WISCRS Pepin-Pierce (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7610','WISCRS Iowa (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7624','NAD83(2011) / WISCRS Pepin (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7611','WISCRS Iron (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7624','NAD83(2011) / WISCRS Pierce (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7612','WISCRS Jackson (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7625','WISCRS Polk (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7635','WISCRS Taylor (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7626','WISCRS Portage (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7613','WISCRS Oz-Milw-Rac-Ken (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7613','NAD83(2011) / WISCRS Kenosha (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7613','NAD83(2011) / WISCRS Milwaukee (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7613','NAD83(2011) / WISCRS Ozaukee (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7613','NAD83(2011) / WISCRS Racine (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7627','WISCRS Price (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7636','WISCRS Trempealeau (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7628','WISCRS Richland (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7629','WISCRS Rock (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7614','WISCRS Kew-Manit-Sheb (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7614','NAD83(2011) / WISCRS Kewaunee (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7614','NAD83(2011) / WISCRS Manitowoc (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7614','NAD83(2011) / WISCRS Sheboygan (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7637','WISCRS Vernon (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7630','WISCRS Rusk (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7615','WISCRS La Crosse (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7616','WISCRS Langlade (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7631','WISCRS Sauk (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7617','WISCRS Lincoln (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7638','WISCRS Vilas (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7618','WISCRS Marathon (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7639','WISCRS Walworth (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9377','MAGNA-SIRGAS / CTM12','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9387','ETRS89 / AbInvA96_2020 SnakeGrid','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','9388','ETRS89 / AbInvA96_2020 SnakeGrid + Newlyn height','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7640','WISCRS Washburn (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7641','WISCRS Washington (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7642','WISCRS Waukesha (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7643','WISCRS Waupaca (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7644','WISCRS Waushara (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7645','WISCRS Wood (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6316','Macedonia State Coordinate System zone 7','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3857','Web Mercator','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9391','BGS2005 / UTM zone 35 (E-N)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5634','REGCAN95 - LCC','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','9400','REDNAP height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','9392','REDNAP height','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7692','Kyrg06-68','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7693','Kyrg06-71','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7694','Kyrg06-74','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7695','Kyrg06-77','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7696','Kyrg06-80','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','7700','LAS-2000 height','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5635','REGCAN95 - LAEA','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','9401','REDNAP height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5754','Poolbeg height','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7791','RDN2008 / UTM zone 32N (E-N)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7792','RDN2008 / TM33 (E-N)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7792','RDN2008 / UTM zone 33N (E-N)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7791','RDN2008 / TM32 (E-N)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7793','RDN2008 / UTM zone 34N (E-N)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7800','CS2005 zone 35','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7794','RDN2008 / Fuso Italia (E-N)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7795','RDN2008 / Fuso 12 (E-N)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7799','CS2005 zone 34','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7799','Coordinate System 2005 zone 34','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7815','WGS 84 (Original)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7800','Coordinate System 2005 zone 35','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7803','BGS2005 / UTM zone 34N (E-N)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7816','WGS 84 (Original)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7804','BGS2005 / UTM zone 35 (E-N)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7825','Pulkovo 1942 / CS63 X1','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7831','Pulkovo 1942 / CS63 X7','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7826','Pulkovo 1942 / CS63 X2','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7827','Pulkovo 1942 / CS63 X3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7828','Pulkovo 1942 / CS63 X4','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7829','Pulkovo 1942 / CS63 X5','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7830','Pulkovo 1942 / CS63 X6','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7845','GDA2020 / Geoscience Australia Lambert','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4710','ASTRO DOS 71/4','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4710','St. Helena 1971','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','7837','DE_AMST_2016 / NH','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7879','WGS 84 Tritan St. Helena','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7878','St. Helena 1971 / UTM zone 30S','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7880','WGS 84 Tritan St. Helena','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7881','WGS 84 Tritan St. Helena','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7882','WGS 84 Tritan St. Helena / SHLG(Tritan)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7899','GDA2020 / Vicgrid2020','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3111','GDA94 / Vicgrid94','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7908','388','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7909','355','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7909','ITRF2000 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7910','209','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7911','277','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7911','ITRF2008 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','5332','242','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','5332','ITRF2008 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4896','269','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4919','419','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4919','ITRF2000 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4918','349','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7887','SHGD2015 / UTM zone 30S','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','9393','REDNAP height','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7887','St. Helena Map Grid 2015','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','7888','MSL 1971 height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','7889','MSL Tritan height','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7877','St. Helena 1971 / SHLG71','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','7954','St. Helena 1971 / UTM zone 30S + Jamestown 1971 height','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22196','C Inchauspe /Argentina 6','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22197','C Inchauspe /Argentina 7','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22195','C Inchauspe /Argentina 5','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22193','C Inchauspe /Argentina 3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22194','C Inchauspe /Argentina 4','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22192','C Inchauspe /Argentina 2','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22191','C Inchauspe /Argentina 1','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2315','Campo Inchauspe /UTM 19S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2316','Campo Inchauspe /UTM 20S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','7992','Malongo 1987 / UTM 33S','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','7976','Hong Kong Principal Datum depth','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8042','S-SC','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8042','Stable Cadastre','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8043','Stable cadastre','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8043','S-SC','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','8050','Mean Sea Level height (ft)','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','8051','Mean Sea Level depth (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8065','NAD83(2011) / PCCS east zone(ft)','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','8052','Mean Sea Level height (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8066','NAD83(2011) / PCCS central zone(ft)','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','8053','Mean Sea Level depth (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8067','NAD83(2011) / PCCS west zone(ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8068','NAD83(2011) / PCCS Mt. Lemmon zone(ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8090','WISCRS Florence (m)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4974','SIRGAS95 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8091','WISCRS Florence (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8096','WISCRS Wood (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7789','425','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8095','WISCRS Wood (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8109','WISCRS Vilas (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8097','WISCRS Waushara (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8105','WISCRS Washburn (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8098','WISCRS Waushara (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8113','WISCRS Trempealeau (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8099','WISCRS Waupaca (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8106','WISCRS Washburn (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8100','WISCRS Waupaca (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8110','WISCRS Vilas (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8101','WISCRS Waukesha (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8107','WISCRS Walworth (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8102','WISCRS Waukesha (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8131','WISCRS Price (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8103','WISCRS Washington (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8108','WISCRS Walworth (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8104','WISCRS Washington (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8111','WISCRS Vernon (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8114','WISCRS Trempealeau (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8112','WISCRS Vernon (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8154','WISCRS Langlade (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8115','WISCRS Taylor (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8132','WISCRS Price (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8116','WISCRS Taylor (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8161','WISCRS Jackson (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8117','WISCRS St. Croix (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8133','WISCRS Portage (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8118','WISCRS St. Croix (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8155','WISCRS La Crosse (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8119','WISCRS Shawano (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8134','WISCRS Portage (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8120','WISCRS Shawano (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8165','WISCRS Iowa (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8121','WISCRS Sawyer (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8135','WISCRS Polk (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8122','WISCRS Sawyer (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8156','WISCRS La Crosse (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8123','WISCRS Sauk (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8136','WISCRS Polk (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8124','WISCRS Sauk (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8162','WISCRS Jackson (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8125','WISCRS Rusk (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8092','WISCRS Eau Claire (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8126','WISCRS Rusk (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8163','WISCRS Iron (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8127','WISCRS Rock (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8137','WISCRS Pepin-Pierce (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8128','WISCRS Rock (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8137','NAD83(HARN) / WISCRS Pepin (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8129','WISCRS Richland (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8137','NAD83(HARN) / WISCRS Pierce (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8130','WISCRS Richland (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8166','WISCRS Iowa (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8157','WISCRS Kew-Manit-Sheb (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8157','NAD83(HARN) / WISCRS Kewaunee (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8138','WISCRS Pepin-Pierce (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8138','NAD83(HARN) / WISCRS Pepin (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8138','NAD83(HARN) / WISCRS Pierce (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8157','NAD83(HARN) / WISCRS Manitowoc (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8139','WISCRS Oneida (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8157','NAD83(HARN) / WISCRS Sheboygan (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8140','WISCRS Oneida (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8164','WISCRS Iron (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8141','WISCRS Oconto (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8170','WISCRS Green-Lafayette (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8142','WISCRS Oconto (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8093','WISCRS Eau Claire (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8143','WISCRS Monroe (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8170','NAD83(HARN) / WISCRS Green (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8144','WISCRS Monroe (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8158','WISCRS Kew-Manit-Sheb (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8145','WISCRS Menominee (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8158','NAD83(HARN) / WISCRS Kewaunee (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8146','WISCRS Menominee (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8158','NAD83(HARN) / WISCRS Manitowoc (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8147','WISCRS Marinette (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8158','NAD83(HARN) / WISCRS Sheboygan (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8148','WISCRS Marinette (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8167','WISCRS Green Lake-Marquette (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8149','WISCRS Marathon (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8167','NAD83(HARN) / WISCRS Green Lake (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8150','WISCRS Marathon (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8167','NAD83(HARN) / WISCRS Marquette (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8151','WISCRS Lincoln (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8179','WISCRS Dunn (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8152','WISCRS Lincoln (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8170','NAD83(HARN) / WISCRS Lafayette (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8153','WISCRS Langlade (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8159','WISCRS Oz-Milw-Rac-Ken (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8159','NAD83(HARN) / WISCRS Kenosha(m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8159','NAD83(HARN) / WISCRS Milwaukee (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8159','NAD83(HARN) / WISCRS Ozaukee (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8159','NAD83(HARN) / WISCRS Racine (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8180','WISCRS Dunn (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8168','WISCRS GreenLake-Marqt (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8168','NAD83(HARN) / WISCRS Green Lake (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8168','NAD83(HARN) / WISCRS Marquette (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','8228','North American Vertical Datum of 1988 height (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8160','WISCRS Oz-Milw-Rac-Ken (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8160','NAD83(HARN) / WISCRS Kenosha (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8160','NAD83(HARN) / WISCRS Milwaukee (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8160','NAD83(HARN) / WISCRS Ozaukee (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8160','NAD83(HARN) / WISCRS Racine (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8181','WISCRS Douglas (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8200','WISCRS Columbia (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8169','WISCRS Green-Lafayette (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8169','NAD83(HARN) / WISCRS Green (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8169','NAD83(HARN) / WISCRS Lafayette (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8182','WISCRS Douglas (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8203','WISCRS Chippewa (m)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4152','Guam Geodetic Network 1993','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8171','WISCRS Grant (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8184','WISCRS Door (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8172','WISCRS Grant (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8173','WISCRS Forest (m)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4152','GGN93','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8185','WISCRS Door (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8177','WISCRS Forest (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8204','WISCRS Chippewa (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8237','NAD83(CSRS98)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8187','WISCRS Dodge-Jefferson (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8187','NAD83(HARN) / WISCRS Dodge (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8187','NAD83(HARN) / WISCRS Jefferson (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8189','WISCRS Dodge-Jefferson (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8189','NAD83(HARN) / WISCRS Dodge (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8189','NAD83(HARN) / WISCRS Jefferson (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8205','WISCRS Out-Cal-Win-FdL (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8191','WISCRS Dane (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8205','NAD83(HARN) / WISCRS Calumet (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8193','WISCRS Dane (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8205','NAD83(HARN) / WISCRS Fond du Lac (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8196','WISCRS Crawford (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8205','NAD83(HARN) / WISCRS Outagamie (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8197','WISCRS Crawford (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8205','NAD83(HARN) / WISCRS Winnebago (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8198','WISCRS Columbia (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8313','NAD83(2011) / OCRS_CBU (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8201','WISCRS Clark (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8202','WISCRS Clark (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8206','WISCRS Out-Cal-Win-FdL (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8206','NAD83(HARN) / WISCRS Calumet (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8206','NAD83(HARN) / WISCRS Fond du Lac (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8206','NAD83(HARN) / WISCRS Outagamie (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8206','NAD83(HARN) / WISCRS Winnebago (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8311','NAD83(2011) / OCRS_BHA (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8207','WISCRS Burnett (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8323','NAD83(2011) / OCRS_MDI (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8208','WISCRS Burnett (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8314','NAD83(2011) / OCRS_CBU (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8209','WISCRS Buffalo (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8312','NAD83(2011) / OCRS_BHA (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8210','WISCRS Buffalo (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8333','NAD83(2011) / OCRS_PRU (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8212','WISCRS Brown (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8315','NAD83(2011) / OCRS_CRN (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8213','WISCRS Brown (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8324','NAD83(2011) / OCRS_MDI (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8214','WISCRS Bayfield (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8316','NAD83(2011) / OCRS_CRN (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8216','WISCRS Bayfield (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8337','NAD83(2011) / OCRS_RLA (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8218','WISCRS Barron (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8317','NAD83(2011) / OCRS_DPR (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8220','WISCRS Barron (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8325','NAD83(2011) / OCRS_MTC (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8222','WISCRS Ashland (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8318','NAD83(2011) / OCRS_DPR (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8224','WISCRS Ashland (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8334','NAD83(2011) / OCRS_PRU (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8319','NAD83(2011) / OCRS_DBU (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8326','NAD83(2011) / OCRS_MTC (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8225','WISCRS Adams-Juneau (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8225','NAD83(HARN) / WISCRS Adams (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8225','NAD83(HARN) / WISCRS Juneau (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8320','NAD83(2011) / OCRS_DBU (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8339','NAD83(2011) / OCRS_SKP (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8321','NAD83(2011) / OCRS_HLF (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8226','WISCRS Adams-Juneau (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8226','NAD83(HARN) / WISCRS Adams (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8226','NAD83(HARN) / WISCRS Juneau (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8327','NAD83(2011) / OCRS_NCE (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8322','NAD83(2011) / OCRS_HLF (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8335','NAD83(2011) / OCRS_PCB (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8328','NAD83(2011) / OCRS_NCE (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8338','NAD83(2011) / OCRS_RLA (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8329','NAD83(2011) / OCRS_OCH (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8336','NAD83(2011) / OCRS_PCB (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8330','NAD83(2011) / OCRS_OCH (ft)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4957','Guam Geodetic Network 1993','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4957','GGN93','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8370','ETRS89 / Lambert 2008 + Oostende height','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8331','NAD83(2011) / OCRS_OWY (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8340','NAD83(2011) / OCRS_SKP (ft)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4956','Guam Geodetic Network 1993','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4956','GGN93','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8332','NAD83(2011) / OCRS_OWY (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8353','S-JTSK [JTSK03] / Krovak EN','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8341','NAD83(2011) / OCRS_UFX (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8342','NAD83(2011) / OCRS_UFX (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8343','NAD83(2011) / OCRS_WAL (m)','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','6130','GCVD54 height','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8344','NAD83(2011) / OCRS_WAL (ft)','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','6131','LCVD61 height','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8345','NAD83(2011) / OCRS_WHI (m)','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','6132','CBVD61 height','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8346','NAD83(2011) / OCRS_WHI (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8347','NAD83(2011) / OCRS_WPA (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8348','NAD83(2011) / OCRS_WPA (ft)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8360','ETRS89 [ETRF2000] + Bpv','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4417','Pulkovo 42(83) / 3GK zn7','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','8357','Bpv','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4434','Pulkovo 42(83) / 3GK zn8','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8351','S-JTSK (JTSK03)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8352','S-JTSK (JTSK03) / Krovak','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','8357','SK_KRON / NH','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','8378','TVD','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','8357','CZ_KRON / NH','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8384','NCRS Las Vegas (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8353','S-JTSK (JTSK03) / Krovak East North','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4152','NAD83','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5513','S-JTSK [JTSK] / Krovak','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8385','NCRS Las Vegas high (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8383','NCRS Las Vegas (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8387','NCRS Las Vegas high (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8433','Macao 1920 Grid System','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5514','S-JTSK [JTSK] / Krovak East North','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4414','GGN93 / Guam Map Grid','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4414','Guam Geodetic Network 1993 / Guam Map Grid','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4414','NAD83 / Guam Map Grid','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4414','NAD83(HPGN) / Guam Map Grid','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8538','NAD83(2011) / KS RCS zone 18 Arkansas City','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8519','NAD83(2011) / KS RCS zone 2 Colby','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8520','NAD83(2011) / KS RCS zone 3 Oberlin','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8539','NAD83(2011) / KS RCS zone 19 Coffeyville','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8521','NAD83(2011) / KS RCS zone 4 Hays','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8441','Tananarive / Laborde Grid (Greenwich)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8522','NAD83(2011) / KS RCS zone 5 Great Bend','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8540','NAD83(2011) / KS RCS zone 20 Pittsburg','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8523','NAD83(2011) / KS RCS zone 6 Beloit','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','7407','NAD27 / Texas North + NGVD29 height','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8524','NAD83(2011) / KS RCS zone 7 Salina','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8700','NAD83 / AZ_E + NAVD88 ht (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8525','NAD83(2011) / KS RCS zone 8 Manhattan','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8701','NAD83 / AZ_C + NAVD88 ht (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8526','NAD83(2011) / KS RCS zone 9 Emporia','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8702','NAD83 / AZ_W+ NAVD88 ht (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8527','NAD83(2011) / KS RCS zone 10 Atchison','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8528','NAD83(2011) / KS RCS zone 11 Kansas City','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8703','NAD83 / MI_N + NAVD88 ht (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8529','NAD83(2011) / KS RCS zone 12 Ulysses','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8531','NAD83(2011) / KS RCS zone 13 Garden City','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8533','NAD83(2011) / KS RCS zone 14 Dodge City','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8704','NAD83 / MI_C + NAVD88 ht (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8534','NAD83(2011) / KS RCS zone 15 Larned','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8705','NAD83 / MI_S+ NAVD88 ht (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8535','NAD83(2011) / KS RCS zone 16 Pratt','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8706','NAD83 / MT + NAVD88 ht (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8536','NAD83(2011) / KS RCS zone 17 Wichita','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8707','NAD83 / ND_N + NAVD88 ht (ft)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8708','NAD83 / ND_S + NAVD88 ht (ft)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8449','NAD83(2002 PRVI)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8709','NAD83 / OR_N + NAVD88 ht (ft)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8710','NAD83 / OR_S + NAVD88 ht (ft)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8711','NAD83 / SC + NAVD88 ht (ft)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8712','NAD83 / AR_N + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8713','NAD83 / AR_S + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8714','NAD83 / CA_1 + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8715','NAD83 / CA_2 + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8716','NAD83 / CA_3 + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8717','NAD83 / CA_4 + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8718','NAD83 / CA_5 + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8719','NAD83 / CA_6 + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8720','NAD83 / CO_N + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8721','NAD83 / CO_C + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8722','NAD83 / CO_S + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8723','NAD83 / CT + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8724','NAD83 / DE + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8725','NAD83 / FL_N + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8726','NAD83 / FL_E + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8727','NAD83 / FL_W + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8728','NAD83 / GA_E + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8729','NAD83 / GA_W + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8730','NAD83 / ID_E + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8731','NAD83 / ID_C + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8732','NAD83 / ID_W + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8733','NAD83 / IL_E + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8734','NAD83 / IL_W + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8735','NAD83 / IN_E + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8736','NAD83 / IN_W + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8737','NAD83 / IA_N + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8738','NAD83 / IA_S + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8739','NAD83 / KS_N + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8740','NAD83 / KS_S + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8741','NAD83 / KY_N + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8742','NAD83 / KY_S + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8743','NAD83 / LA_N + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8744','NAD83 / LA_S + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8745','NAD83 / ME_E + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8746','NAD83 / ME_W + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4437','NAD83(NSRS) / PR and VI','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8747','NAD83 / MD + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8748','NAD83 / MA_M + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8749','NAD83 / MA_I + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8750','NAD83 / MN_N + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8751','NAD83 / MN_C + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8752','NAD83 / MN_S + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8753','NAD83 / MS_E + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8754','NAD83 / MS_W + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8755','NAD83 / NE + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8756','NAD83 / NV_E + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8757','NAD83 / NV_C + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8758','NAD83 / NV_W + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8759','NAD83 / NH + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8760','NAD83 / NJ + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8761','NAD83 / NM_E + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8762','NAD83 / NM_C + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8763','NAD83 / NM_W + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8764','NAD83 / NY_E + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8765','NAD83 / NY_C + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8766','NAD83 / NY_W + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8767','NAD83 / NY_LI + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8768','NAD83 / NC + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8769','NAD83 / OH_N + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8770','NAD83 / OH_S + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8771','NAD83 / OK_N + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4455','NAD27 / Pennsylvania S','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8772','NAD83 / OK_S + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4456','NAD27 / New York Long Is','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8773','NAD83 / PA_N + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4457','NAD83 / S Dakota N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8774','NAD83 / PA_S + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8775','NAD83 / RI + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8776','NAD83 / SD_N + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8777','NAD83 / SD_S + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8778','NAD83 / TN + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8779','NAD83 / TX_N + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8780','NAD83 / TX_NC + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8781','NAD83 / TX_C + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8782','NAD83 / TX_SC + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8783','NAD83 / TX_S + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8784','NAD83 / UT_N + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8785','NAD83 / UT_C + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3112','GDA94 / GA LCC','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3112','GDA94 / Geoscience LCC','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8786','NAD83 / UT_S + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8787','NAD83 / VT + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8788','NAD83 / VA_N + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8789','NAD83 / VA_S + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8790','NAD83 / WA_N + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8791','NAD83 / WA_S + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8792','NAD83 / WV_N + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8793','NAD83 / WV_S + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8794','NAD83 / WI_N + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8795','NAD83 / WI_C + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8796','NAD83 / WI_S + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8797','NAD83 / WY_E + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8798','NAD83 / WY_EC + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8799','NAD83 / WY_WC + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8800','NAD83 / WY_W + NAVD88 ht (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8801','NAD83 / AL_E + NAVD88 ht (m)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8802','NAD83 / AL_W + NAVD88 ht (m)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8803','NAD83 / AK_1 + NAVD88 ht (m)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8804','NAD83 / AK_2 + NAVD88 ht (m)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8805','NAD83 / AK_3 + NAVD88 ht (m)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8806','NAD83 / AK_4 + NAVD88 ht (m)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8807','NAD83 / AK_5 + NAVD88 ht (m)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8808','NAD83 / AK_6 + NAVD88 ht (m)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8809','NAD83 / AK_7 + NAVD88 ht (m)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8810','NAD83 / AK_8 + NAVD88 ht (m)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8811','NAD83 / AK_9 + NAVD88 ht (m)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8812','NAD83 / AK_10 + NAVD88 ht (m)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8813','NAD83 / MO_E + NAVD88 ht (m)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8814','NAD83 / MO_C + NAVD88 ht (m)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8815','NAD83 / MO_W + NAVD88 ht (m)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','7406','NAD27 + NGVD29 height','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6316','B&H GK zone 7','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8677','HDKS zone 5','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','8675','N43','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8677','D48 zone 5','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8677','HR_HDKS / HR_TM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8678','HDKS zone 6','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8678','HR_HDKS / HR_TM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8677','B&H GK zone 5','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8686','SI_D48 / SI_TM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8686','D48 / Slovenia Grid','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8687','D96/UTM','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5779','NVN99 height','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8678','B&H GK zone 6','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8544','NAD83(1997 PRVI)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8545','NAD83(1997 PRVI)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4474','Cadastre 1997 / UTM 38S','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8541','NAD83(2002 AS)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8541','NAD83(2002 GU)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8542','NAD83(2002 AS)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8542','NAD83(2002 GU)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8542','NAD83(2002 PRVI)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8449','NAD83(2002 AS)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8449','NAD83(2002 GU)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8541','NAD83(2002 PRVI)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8543','NAD83(1997 PRVI)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4956','NAD83(1992 AK)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4956','NAD83(1993 HI)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4956','NAD83(1993 AS)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4956','NAD83(1993 GU)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4956','NAD83(1993 PRVI)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4957','NAD83(1992 AK)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4957','NAD83(1993 HI)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4957','NAD83(1993 AS)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4957','NAD83(1993 GU)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4957','NAD83(1993 PRVI)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4152','NAD83(1992 AK)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4152','NAD83(1993 HI)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4152','NAD83(1993 AS)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4152','NAD83(1993 GU)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4152','NAD83(1993 PRVI)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8683','STRS00','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8684','STRS00','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8685','STRS00','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8682','STRS00 / UTM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22032','Camacupa / UTM zone 32S','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4220','Camacupa','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22091','Camacupa / TM 11.30 SE','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8694','Camacupa','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8698','REPANGOL','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22092','Camacupa / TM 12 SE','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22033','Camacupa / UTM zone 33S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8677','MGI 1901 / Gauss–Kruger zone 5','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8678','MGI 1901 / Gauss–Kruger zone 6','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8679','MGI 1901 / Gauss–Kruger zone 8','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6316','MGI 1901 / Gauss–Kruger zone 7','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8826','NAD83 / IDTM','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8860','NAD83(2002 PRVI)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8860','NAD83(2002 AS)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8860','NAD83(2002 GU)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8697','REPANGOL','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','8897','TVD','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8905','CR14','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8899','RGWF96 (lat-lon)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8900','RGWF96 (lat-lon)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8906','CR14','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8908','CR14 / CRTM05','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7912','226','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8909','CR14 / UTM 16N','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','5358','SIRGAS-Chile realización 1','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8910','CR14 / UTM 17N','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8912','CR14 / CRTM05 + DACR52 height','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','5358','SIRGAS-Chile epoch 2002.00','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8988','ITRF88 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8988','337','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7789','ITRF2014 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7912','ITRF2014 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4896','ITRF2005 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7908','ITRF97 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4918','ITRF97 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4917','275','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4917','ITRF96 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7907','233','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7907','ITRF96 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4916','352','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4916','ITRF94 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7906','264','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7906','ITRF94 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4915','263','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4915','ITRF93 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7905','227','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7905','ITRF93 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4914','261','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4914','ITRF92 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7904','286','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7904','ITRF92 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8989','ITRF89 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8989','401','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4913','258','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4913','ITRF91 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7903','241','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7903','ITRF91 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4912','301','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4912','ITRF90 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7902','282','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7902','ITRF90 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4911','387','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4911','ITRF89 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7901','361','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7901','ITRF89 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4910','439','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4910','ITRF88 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7900','400','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7900','ITRF88 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8227','370','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8227','IGS14 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','6934','417','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','6934','IGS08 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7910','ITRF2005 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','4458','Dunedin-Bluff height','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4462','WGS 84 / ACRESLC','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9001','367','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4479','CGCS2000','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4480','CGCS2000','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9001','IGS97 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8990','ITRF90 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8990','254','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8997','ITRF2000 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8997','300','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8991','ITRF91 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8991','259','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9002','274','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9002','IGS97 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8992','ITRF92 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8992','326','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8998','ITRF2005 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8998','403','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8993','ITRF93 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4495','CGCS2000 / 6-degree Gauss-Kruger zone 17','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4495','CGCS2000 / G-K zone 17','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8993','450','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9003','440','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9003','IGS97 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8994','ITRF94 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4492','CGCS2000 / 6-degree Gauss-Kruger zone 14','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4492','CGCS2000 / G-K zone 14','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8994','434','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4493','CGCS2000 / 6-degree Gauss-Kruger zone 15','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4493','CGCS2000 / G-K zone 15','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8999','ITRF2008 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8999','392','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8995','ITRF96 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8995','389','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4496','CGCS2000 / 6-degree Gauss-Kruger zone 18','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4496','CGCS2000 / G-K zone 18','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8996','ITRF97 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8996','268','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9000','ITRF2014 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9000','333','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9004','289','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9004','IGS00 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9007','309','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9007','IGb00 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4498','CGCS2000 / 6-degree Gauss-Kruger zone 20','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4498','CGCS2000 / G-K zone 20','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9005','459','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9005','IGS00 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9015','382','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4499','CGCS2000 / 6-degree Gauss-Kruger zone 21','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4499','CGCS2000 / G-K zone 21','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9015','IGb08 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9006','364','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9006','IGS00 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4500','CGCS2000 / 6-degree Gauss-Kruger zone 22','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4500','CGCS2000 / G-K zone 22','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9008','420','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9008','IGb00 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4501','CGCS2000 / 6-degree Gauss-Kruger zone 23','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4501','CGCS2000 / G-K zone 23','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9009','336','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9009','IGb00 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4502','CGCS2000 / 6-degree Gauss-Kruger CM 75E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4502','CGCS2000 / G-K CM 75E','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9016','402','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9016','IGb08 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9010','310','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4503','CGCS2000 / 6-degree Gauss-Kruger CM 81E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4503','CGCS2000 / G-K CM 81E','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9010','IGS05 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4504','CGCS2000 / 6-degree Gauss-Kruger CM 87E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4504','CGCS2000 / G-K CM 87E','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9011','453','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9011','IGS05 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9017','297','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4505','CGCS2000 / 6-degree Gauss-Kruger CM 93E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4505','CGCS2000 / G-K CM 93E','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9017','IGb08 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9012','441','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9012','IGS05 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4506','CGCS2000 / 6-degree Gauss-Kruger CM 99E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4506','CGCS2000 / G-K CM 99E','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9013','205','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4507','CGCS2000 / 6-degree Gauss-Kruger CM 105E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4507','CGCS2000 / G-K CM 105E','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9013','IGS08 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9018','299','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9018','IGS14 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4508','CGCS2000 / 6-degree Gauss-Kruger CM 111E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4508','CGCS2000 / G-K CM 111E','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9014','424','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9014','IGS08 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3035','ETRS89 / LAEA Europe','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4509','CGCS2000 / 6-degree Gauss-Kruger CM 117E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4509','CGCS2000 / G-K CM 117E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3034','ETRS89 / LCC Europe','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9019','406','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9019','IGS14 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4510','CGCS2000 / 6-degree Gauss-Kruger CM 123E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4510','CGCS2000 / G-K CM 123E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4512','CGCS2000 / 6-degree Gauss-Kruger CM 135E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4512','CGCS2000 / G-K CM 135E','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4490','CGCS2000','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3907','HDKS zone 5','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3908','HDKS zone 6','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3907','D48 zone 5','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3907','HR_HDKS / HR_TM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3908','HR_HDKS / HR_TM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3911','SI_D48 / SI_TM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3911','D48 / Slovenia Grid','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3912','D48/GK','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3985','Katanga 1955 / Lambert','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4050','RGRDC 2005 / Congo TM 16','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4051','RGRDC 2005 / Congo TM 18','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4056','RGRDC 2005 / Congo TM 20','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4057','RGRDC 2005 / Congo TM 22','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4058','RGRDC 2005 / Congo TM 24','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4059','RGRDC 2005 / Congo TM 26','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4060','RGRDC 2005 / Congo TM 28','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4061','RGRDC 2005 / UTM 33S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4062','RGRDC 2005 / UTM 34S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4063','RGRDC 2005 / UTM 35S','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4975','306','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4975','SIRGAS95 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4170','318','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4170','SIRGAS95 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4988','384','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4988','SIRGAS2000 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4989','313','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4989','SIRGAS2000 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4674','398','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4674','SIRGAS2000 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8972','322','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8972','SIRGAS-CON DGF00P01 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8916','454','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8916','SIRGAS-CON DGF00P01 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8915','376','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8915','SIRGAS-CON DGF00P01 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8973','422','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8973','SIRGAS-CON DGF01P01 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8918','363','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8918','SIRGAS-CON DGF01P01 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8917','411','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8917','SIRGAS-CON DGF01P01 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8974','321','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8974','SIRGAS-CON DGF01P02 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8920','348','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8920','SIRGAS-CON DGF01P02 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8919','332','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8919','SIRGAS-CON DGF01P02 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8975','418','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8975','SIRGAS-CON DGF02P01 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8922','346','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8922','SIRGAS-CON DGF02P01 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8921','283','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8921','SIRGAS-CON DGF02P01 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8976','366','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8976','SIRGAS-CON DGF04P01 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8924','212','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8924','SIRGAS-CON DGF04P01 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8923','345','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8923','SIRGAS-CON DGF04P01 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8977','290','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8977','SIRGAS-CON DGF05P01 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8926','220','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8926','SIRGAS-CON DGF05P01 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8925','373','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4541','CGCS2000 / 3GK CM 96E','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8925','SIRGAS-CON DGF05P01 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4534','CGCS2000 / 3GK CM 75E','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8978','223','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4535','CGCS2000 / 3GK CM 78E','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8978','SIRGAS-CON DGF06P01 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4536','CGCS2000 / 3GK CM 81E','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8928','250','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4537','CGCS2000 / 3GK CM 84E','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8928','SIRGAS-CON DGF06P01 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8927','359','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8927','SIRGAS-CON DGF06P01 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4538','CGCS2000 / 3GK CM 87E','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8979','296','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4539','CGCS2000 / 3GK CM 90E','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8979','SIRGAS-CON DGF07P01 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4540','CGCS2000 / 3GK CM 93E','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8930','271','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8930','SIRGAS-CON DGF07P01 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4542','CGCS2000 / 3GK CM 99E','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8929','324','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8929','SIRGAS-CON DGF07P01 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4543','CGCS2000 / 3GK CM 102E','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8980','276','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8980','SIRGAS-CON DGF08P01 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4544','CGCS2000 / 3GK CM 105E','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8932','232','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8932','SIRGAS-CON DGF08P01 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4545','CGCS2000 / 3GK CM 108E','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8931','442','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8931','SIRGAS-CON DGF08P01 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4546','CGCS2000 / 3GK CM 111E','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8981','395','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8981','SIRGAS-CON SIR09P01 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4547','CGCS2000 / 3GK CM 114E','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8934','451','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8934','SIRGAS-CON SIR09P01 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4548','CGCS2000 / 3GK CM 117E','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8933','319','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8933','SIRGAS-CON SIR09P01 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4549','CGCS2000 / 3GK CM 120E','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8982','456','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8982','SIRGAS-CON SIR10P01 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4550','CGCS2000 / 3GK CM 123E','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8936','211','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8936','SIRGAS-CON SIR10P01 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4551','CGCS2000 / 3GK CM 126E','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8935','427','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8935','SIRGAS-CON SIR10P01 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4552','CGCS2000 / 3GK CM 129E','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8983','307','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8983','SIRGAS-CON SIR11P01 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4553','CGCS2000 / 3GK CM 132E','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8938','303','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8938','SIRGAS-CON SIR11P01 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4554','CGCS2000 / 3GK CM 135E','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8937','218','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8937','SIRGAS-CON SIR11P01 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8984','240','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8984','SIRGAS-CON SIR13P01 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8940','399','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8940','SIRGAS-CON SIR13P01 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8939','413','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8939','SIRGAS-CON SIR13P01 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8985','380','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8985','SIRGAS-CON SIR14P01 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8942','203','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8942','SIRGAS-CON SIR14P01 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8941','257','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8941','SIRGAS-CON SIR14P01 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8986','249','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8986','SIRGAS-CON SIR15P01 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8944','235','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8944','SIRGAS-CON SIR15P01 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8943','446','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8943','SIRGAS-CON SIR15P01 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8987','396','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8987','SIRGAS-CON SIR17P01 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8946','312','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8946','SIRGAS-CON SIR17P01 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8945','273','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8945','SIRGAS-CON SIR17P01 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4202','298','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4202','AGD66 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4203','350','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4203','AGD84 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7844','284','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7844','GDA2020 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7843','329','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7843','GDA2020 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7842','404','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7842','GDA2020 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4283','368','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4283','GDA94 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4939','288','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4939','GDA94 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4938','362','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4938','GDA94 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7663','216','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8888','WGS 84 (Original)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9053','374','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9053','WGS 84 (G730) - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8888','426','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8888','WGS 84 TRANSIT - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9054','208','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9054','WGS 84 (G873) - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9055','228','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9055','WGS 84 (G1150) - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9056','237','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9056','WGS 84 (G1674) - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9057','265','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4568','New Beijing / 6-degree Gauss-Kruger zone 13','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9057','WGS 84 (G1762) - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4568','New Beijing / G-K zone 13','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7663','WGS 84 (G1674) - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4569','New Beijing / 6-degree Gauss-Kruger zone 14','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7662','334','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4569','New Beijing / G-K zone 14','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7662','WGS 84 (G1674) - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7816','342','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4570','New Beijing / 6-degree Gauss-Kruger zone 15','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4570','New Beijing / G-K zone 15','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7816','WGS 84 TRANSIT - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7815','272','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7815','WGS 84 TRANSIT - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7657','458','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7657','WGS 84 (G730) - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4571','New Beijing / G-K zone 16','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4571','New Beijing / 6-degree Gauss-Kruger zone 16','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7656','314','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4572','New Beijing / 6-degree Gauss-Kruger zone 17','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7656','WGS 84 (G730) - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4572','New Beijing / G-K zone 17','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7659','344','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7659','WGS 84 (G873) - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4573','New Beijing / 6-degree Gauss-Kruger zone 18','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4573','New Beijing / G-K zone 18','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7658','325','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7658','WGS 84 (G873) - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4574','New Beijing / G-K zone 19','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4574','New Beijing / 6-degree Gauss-Kruger zone 19','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7661','414','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7661','WGS 84 (G1150) - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4575','New Beijing / G-K zone 20','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4575','New Beijing / 6-degree Gauss-Kruger zone 20','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7660','210','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4576','New Beijing / 6-degree Gauss-Kruger zone 21','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7660','WGS 84 (G1150) - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4576','New Beijing / G-K zone 21','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7665','405','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4577','New Beijing / 6-degree Gauss-Kruger zone 22','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7665','WGS 84 (G1762) - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4577','New Beijing / G-K zone 22','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7664','215','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7664','WGS 84 (G1762) - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4578','New Beijing / G-K zone 23','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4578','New Beijing / 6-degree Gauss-Kruger zone 23','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','3855','436','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2333','Xian 1980 / G-K zone 19','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','3855','WGS 84 EGM2008 - OHt','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4497','CGCS2000 / G-K zone 19','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4497','CGCS2000 / 6-degree Gauss-Kruger zone 19','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5798','383','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5798','WGS 84 EGM84 - OHt','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5773','444','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5773','WGS 84 EGM96 - OHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8237','285','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4579','New Beijing / 6-degree Gauss-Kruger CM 75E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4579','New Beijing / G-K CM 75E','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8237','NAD83(CSRS) v2 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8235','437','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8235','NAD83(CSRS) v2 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4580','New Beijing / 6-degree Gauss-Kruger CM 81E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4580','New Beijing / G-K CM 81E','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8233','432','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8233','NAD83(CSRS) v2 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8231','331','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4581','New Beijing / 6-degree Gauss-Kruger CM 87E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4581','New Beijing / G-K CM 87E','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8231','NAD83(CSRS96) v1 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8231','NAD83(CSRS)v1','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8230','447','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4582','New Beijing / 6-degree Gauss-Kruger CM 93E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4582','New Beijing / G-K CM 93E','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8230','NAD83(CSRS96) v1 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8230','NAD83(CSRS96)v1','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8232','230','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4583','New Beijing / 6-degree Gauss-Kruger CM 99E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4583','New Beijing / G-K CM 99E','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8232','NAD83(CSRS96) v1 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8232','NAD83(CSRS96)v1','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8253','448','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4584','New Beijing / 6-degree Gauss-Kruger CM 105E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4584','New Beijing / G-K CM 105E','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8253','NAD83(CSRS) v7 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8254','243','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8254','NAD83(CSRS) v7 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4585','New Beijing / 6-degree Gauss-Kruger CM 111E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4585','New Beijing / G-K CM 111E','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8255','236','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8255','NAD83(CSRS) v7 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8250','372','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4586','New Beijing / 6-degree Gauss-Kruger CM 117E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4586','New Beijing / G-K CM 117E','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8250','NAD83(CSRS) v6 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8251','327','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8251','NAD83(CSRS) v6 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4587','New Beijing / 6-degree Gauss-Kruger CM 123E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4587','New Beijing / G-K CM 123E','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8252','292','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8252','NAD83(CSRS) v6 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8247','429','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4588','New Beijing / 6-degree Gauss-Kruger CM 129E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4588','New Beijing / G-K CM 129E','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8247','NAD83(CSRS) v5 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8248','431','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8248','NAD83(CSRS) v5 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4589','New Beijing / 6-degree Gauss-Kruger CM 135E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4589','New Beijing / G-K CM 135E','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8249','377','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8249','NAD83(CSRS) v5 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8242','245','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8242','NAD83(CSRS) v4 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8244','347','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8244','NAD83(CSRS) v4 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8246','340','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8246','NAD83(CSRS) v4 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8238','386','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8238','NAD83(CSRS) v3 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8239','375','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8239','NAD83(CSRS) v3 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8240','320','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8240','NAD83(CSRS) v3 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5713','323','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5713','CGVD28 - NOHt','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','6647','423','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','6647','CGVD2013(CGG2013) - OHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4122','225','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4122','ATS77 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9191','WGS 84 / New Zealand Albers','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5899','VN-2000 / TM-3 Da Nang zone','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5621','409','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5621','EVRF2007 - NHt','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5730','238','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5730','EVRF2000 - NHt','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5621','EVRF2007_AMST / NH','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7928','433','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7928','ETRF97 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7929','330','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7929','ETRF97 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7926','356','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7926','ETRF96 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7927','421','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7927','ETRF96 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7924','206','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7924','ETRF94 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7925','407','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7925','ETRF94 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7922','244','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7922','ETRF93 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7923','358','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7923','ETRF93 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7920','221','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7920','ETRF92 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7921','369','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7921','ETRF92 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7918','213','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7918','ETRF91 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7919','371','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7919','ETRF91 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7916','435','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7916','ETRF90 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7917','353','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7917','ETRF90 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7914','308','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7914','ETRF89 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7915','217','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7915','ETRF89 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8397','378','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8397','ETRF2005 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8399','317','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8399','ETRF2005 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7930','260','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7930','ETRF2000 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7931','457','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7931','ETRF2000 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4612','394','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4612','JGD2000 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9059','247','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9059','ETRF89 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9061','343','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9061','ETRF91 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9060','224','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9060','ETRF90 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9062','410','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9062','ETRF92 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9068','279','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9068','ETRF2005 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9063','430','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9063','ETRF93 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9064','ETRF94 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9064','255','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4947','281','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4947','JGD2000 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9065','266','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9065','ETRF96 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4946','251','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4946','JGD2000 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9066','267','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9066','ETRF97 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','6694','390','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','6694','JGD2000 (vertical) - OHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9067','234','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9067','ETRF2000 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','6695','428','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','6695','JGD2011 (vertical) - OHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','6666','229','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','6666','JGD2011 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','6667','365','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','6667','JGD2011 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','6668','416','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','6668','JGD2011 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4272','443','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4272','NZGD1949 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4167','293','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4167','NZGD2000 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4959','287','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4959','NZGD2000 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4958','222','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4958','NZGD2000 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','4440','438','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','4440','NZVD2009 - NOHt','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','7839','328','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','7839','NZVD2016 - NOHt','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','6642','219','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','6642','VIVD09 - NOHt','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','6641','360','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','6641','PRVD02 - NOHt','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','6640','391','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','6640','NMVD03 - NOHt','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5702','214','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5702','NGVD29 - NOHt','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5703','256','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5703','NAVD88 - OHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4608','248','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4608','NAD27(MAY76) - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4609','304','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4609','NAD27(CGQ77) - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4267','246','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4267','NAD27 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4269','270','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4269','NAD 83 (1986) - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4269','NAD83(Original)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','6320','449','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','6320','NAD 83 (PA11) Epoch 2010 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','6321','445','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','6321','NAD 83 (PA11) Epoch 2010 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','6322','207','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','6322','NAD 83 (PA11) Epoch 2010 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','6323','379','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','6323','NAD 83 (MA11) Epoch 2010 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','6324','231','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','6324','NAD 83 (MA11) Epoch 2010 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','6325','393','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','6325','NAD 83 (MA11) Epoch 2010 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8544','204','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8544','NAD 83 (HARN) CORRECTED - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8545','291','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8545','NAD 83 (HARN) CORRECTED - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4957','351','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','6644','GUVD04 - NOHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4957','NAD 83 (HARN) - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4152','341','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4152','NAD 83 (HARN) - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8542','412','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8542','NAD 83 (FBN) - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8860','262','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8860','NAD 83 (FBN) - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','6781','311','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','6781','NAD 83 (CORS96) Epoch 1997.0 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','6781','294','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','6781','NAD 83 (CORS96) Epoch 2002.0 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','6782','385','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','6782','NAD 83 (CORS96) Epoch 1997.0 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','6782','354','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','6782','NAD 83 (CORS96) Epoch 2002.0 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','6783','302','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','6783','NAD 83 (CORS96) Epoch 1997.0 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','6783','338','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','6783','NAD 83 (CORS96) Epoch 2002.0 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','6317','252','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','6317','NAD 83 (2011) Epoch 2010 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','6319','239','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','6319','NAD 83 (2011) Epoch 2010 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','6318','415','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','6318','NAD 83 (2011) Epoch 2010 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4893','397','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4893','NAD 83 (2007) - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4759','316','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4759','NAD 83 (2007) - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','6644','253','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','6643','452','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','6643','ASVD02 - NOHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9070','408','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9070','NAD 83 (MARP00) - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9071','305','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9071','NAD 83 (MARP00) - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9072','280','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9072','NAD 83 (MARP00) - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9073','315','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9073','NAD 83 (PACP00) - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9074','455','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9074','NAD 83 (PACP00) - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9075','357','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9075','NAD 83 (PACP00) - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4491','CGCS2000 / 6-degree Gauss-Kruger zone 13','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4491','CGCS2000 / G-K zone 13','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5897','VN-2000 / TM-3 105-00','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4511','CGCS2000 / 6-degree Gauss-Kruger CM 129E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4511','CGCS2000 / G-K CM 129E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5898','VN-2000 / TM-3 108-00','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4531','CGCS2000 / 3GK zone 43','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4533','CGCS2000 / 3GK zone 45','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4532','CGCS2000 / 3GK zone 44','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4530','CGCS2000 / 3GK zone 42','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4528','CGCS2000 / 3GK zone 40','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4526','CGCS2000 / 3GK zone 38','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4494','CGCS2000 / 6-degree Gauss-Kruger zone 16','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4494','CGCS2000 / G-K zone 16','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4527','CGCS2000 / 3GK zone 39','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4525','CGCS2000 / 3GK zone 37','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4524','CGCS2000 / 3GK zone 36','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4523','CGCS2000 / 3GK zone 35','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4522','CGCS2000 / 3GK zone 34','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4521','CGCS2000 / 3GK zone 33','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4520','CGCS2000 / 3GK zone 32','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4519','CGCS2000 / 3GK zone 31','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4518','CGCS2000 / 3GK zone 30','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4517','CGCS2000 / 3GK zone 29','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4516','CGCS2000 / 3GK zone 28','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4515','CGCS2000 / 3GK zone 27','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4514','CGCS2000 / 3GK zone 26','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4513','CGCS2000 / 3GK zone 25','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','5359','SIRGAS-Chile','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8947','SIRGAS-Chile epoch 2010','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8947','SIRGAS-Chile realización 2','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8948','SIRGAS-Chile realización 2','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8948','SIRGAS-Chile epoch 2010.00','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8949','SIRGAS-Chile realización 2','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9141','KOSOVAREF01 / Transverse Mercator (TM)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8949','SIRGAS-Chile epoch 2010.00','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','5359','SIRGAS-Chile realización 1','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','5359','SIRGAS-Chile 2002.00','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','5360','SIRGAS-Chile realización 1','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','5360','SIRGAS-Chile epoch 2002.00','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','5360','SIRGAS-Chile','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','5358','SIRGAS-Chile','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9146','SIRGAS-Chile epoch 2013','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9146','SIRGAS-Chile realización 3','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9153','SIRGAS-Chile realización 4','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9153','SIRGAS-Chile epoch 2016.00','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9147','SIRGAS-Chile realización 3','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9147','SIRGAS-Chile epoch 2013.00','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9148','SIRGAS-Chile realización 3','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9148','SIRGAS-Chile epoch 2013.00','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4782','New Beijing / 3GK CM 75E','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9151','SIRGAS-Chile epoch 2016','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4783','New Beijing / 3GK CM 78E','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9151','SIRGAS-Chile realización 4','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4784','New Beijing / 3GK CM 81E','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9152','SIRGAS-Chile realización 4','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4785','New Beijing / 3GK CM 84E','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9152','SIRGAS-Chile epoch 2016.00','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4786','New Beijing / 3GK CM 87E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4787','New Beijing / 3GK CM 90E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4788','New Beijing / 3GK CM 93E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4789','New Beijing / 3GK CM 96E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4790','New Beijing / 3GK CM 99E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4791','New Beijing / 3GK CM 102E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4792','New Beijing / 3GK CM 105E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4793','New Beijing / 3GK CM 108E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4794','New Beijing / 3GK CM 111E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4795','New Beijing / 3GK CM 114E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4796','New Beijing / 3GK CM 117E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4797','New Beijing / 3GK CM 120E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4798','New Beijing / 3GK CM 123E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4799','New Beijing / 3GK CM 126E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4800','New Beijing / 3GK CM 129E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4812','New Beijing / 3GK CM 132E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4822','New Beijing / 3GK CM 135E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9221','Hartebeesthoek94 / RSA BSU Albers 25E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9221','Hartebeesthoek94 / ZA BSU Albers 25E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4652','New Beijing / 3GK zone 25','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4653','New Beijing / 3GK zone 26','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9222','Hartebeesthoek94 / RSA BSU Albers 44E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4654','New Beijing / 3GK zone 27','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9222','Hartebeesthoek94 / ZA BSU Albers 44E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4655','New Beijing / 3GK zone 28','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4656','New Beijing / 3GK zone 29','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4766','New Beijing / 3GK zone 30','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4767','New Beijing / 3GK zone 31','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4768','New Beijing / 3GK zone 32','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4769','New Beijing / 3GK zone 33','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4770','New Beijing / 3GK zone 34','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4771','New Beijing / 3GK zone 35','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4772','New Beijing / 3GK zone 36','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4773','New Beijing / 3GK zone 37','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4774','New Beijing / 3GK zone 38','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4775','New Beijing / 3GK zone 39','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4778','New Beijing / 3GK zone 42','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4779','New Beijing / 3GK zone 43','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4780','New Beijing / 3GK zone 44','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4781','New Beijing / 3GK zone 45','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4529','CGCS2000 / 3GK zone 41','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4160','Quini-Huao','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9249','Tapi Aike / Argentina zone 1','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9249','Tapi Aike / Gauss-Kruger zone 1','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4160','Quiñi-Huao','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9250','Tapi Aike / Argentina zone 2','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9250','Tapi Aike / Gauss-Kruger zone 2','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2081','Quiñi-Huao / Argentina 2','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9252','MMN / Argentina zone 2','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9252','MMN / Gauss-Kruger zone 2','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2081','Quini-Huao / Argentina 2','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9254','MMS / Argentina zone 2','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9254','MMS / Gauss-Kruger zone 2','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9284','Pampa del Castillo / Gauss-Kruger zone 1','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','28992','Stelsel van de Rijksdriehoeksmeting','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9284','Pampa del Castillo / Argentina zone 1','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9285','Pampa del Castillo / Gauss-Kruger zone 3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9285','Pampa del Castillo / Argentina zone 3','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4289','RD Bessel','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4776','New Beijing / 3GK zone 40','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','9306','ETRS89 / HS2P1+14 Snake + HS2-VRF height','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4777','New Beijing / 3GK zone 41','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9300','ETRS89 / HS2P1+14 Snake','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5753','NGNC height','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9311','US National Atlas EA','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9354','WGS 84 / IBCSO PS','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5709','Hoogte boven Normaal Amsterdams Peil','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','9394','REDNAP height','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31289','MGI (Ferro) / M31','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31290','MGI (Ferro) / M34','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31288','MGI (Ferro) / M28','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','9402','REDNAP height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','9395','REDNAP height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','9396','REDNAP height','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9403','Pico de las Nieves 1968','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','9397','REDNAP height','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4903','Madrid','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','9398','REDNAP height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','9399','REDNAP height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','8690','SI_KOP / NH','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5195','SI_TRIE / NOH','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5195','MK_TRIE / NOH','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4826','WGS 84 / Cape Verde New','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4479','732','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4480','733','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','9450','ETRS89 + Belfast Lough height','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4480','CGCS 2000 - LatLonEht','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4490','734','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4490','CGCS 2000 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5737','735','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5737','National Height Datum 1985 - NHt','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9456','ETRS89 / GBK19 SnakeGrid','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','9457','ETRS89 / GBK19 SnakeGrid + Newlyn height','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9473','GDA2020 / Aus Albers','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4647','ETRS89 / UTM zone 32N','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4824','Morro do Papagaio','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4824','Island of Principe datum','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4823','Fortaleza','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4823','Island of Sao Tome datum','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4481','Red Geodesica Nacional ITRF92','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4482','Red Geodesica Nacional ITRF92','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4483','Red Geodesica Nacional ITRF92','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3034','ETRS89 / LCC','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3034','ETRS89-LCC','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3038','ETRS89 / ETRS-TM26','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3038','ETRS89-TM26','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4411','NAD27 / UTM zone 11N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4412','NAD27 / UTM zone 12N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4413','NAD27 / UTM zone 13N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4419','NAD27 / UTM zone 19N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4401','NAD27 / UTM zone 1N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4402','NAD27 / UTM zone 2N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4403','NAD27 / UTM zone 3N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4404','NAD27 / UTM zone 4N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4405','NAD27 / UTM zone 5N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4407','NAD27 / UTM zone 7N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4408','NAD27 / UTM zone 8N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4409','NAD27 / UTM zone 9N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4430','NAD83 / UTM zone 10N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4431','NAD83 / UTM zone 11N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4432','NAD83 / UTM zone 12N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4433','NAD83 / UTM zone 13N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4438','NAD83 / UTM zone 18N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4439','NAD83 / UTM zone 19N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4421','NAD83 / UTM zone 1N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4422','NAD83 / UTM zone 2N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4423','NAD83 / UTM zone 3N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4424','NAD83 / UTM zone 4N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4425','NAD83 / UTM zone 5N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4426','NAD83 / UTM zone 6N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4427','NAD83 / UTM zone 7N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4428','NAD83 / UTM zone 8N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4429','NAD83 / UTM zone 9N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4410','NAD27 / UTM zone 10N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4418','NAD27 / UTM zone 18N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4406','NAD27 / UTM zone 6N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4399','NAD27 / UTM zone 59N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4217','NAD83 / UTM zone 59N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4420','NAD83 / UTM zone 60N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4400','NAD27 / UTM zone 60N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3035','ETRS89 / LAEA','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3035','ETRS89-LAEA','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3039','ETRS89 / ETRS-TM27','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3039','ETRS89-TM27','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3051','ETRS89 / ETRS-TM39','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3051','ETRS89-TM39','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3050','ETRS89 / ETRS-TM38','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3050','ETRS89-TM38','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3049','ETRS89 / TM37','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3049','ETRS89-TM37','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3048','ETRS89 / TM36','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3048','ETRS89-TM36','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3047','ETRS89 / TM35','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3047','ETRS89-TM35','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3046','ETRS89 / TM34','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3046','ETRS89-TM34','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3045','ETRS89 / TM33','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3045','ETRS89-TM33','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3044','ETRS89 / TM32','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3044','ETRS89-TM32','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3043','ETRS89 / TM31','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3043','ETRS89-TM31','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3042','ETRS89 / TM30','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3042','ETRS89-TM30','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3041','ETRS89 / TM29','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3041','ETRS89-TM29','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3040','ETRS89-TM28','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4936','ETRS89-XYZ','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4937','ETRS89-GRS80h','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4258','ETRS89-GRS80','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4839','ETRS89-LCC-DE(N-E)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4855','ETRF89 / NTM zone 5','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4855','EUREF89 / NTM zone 5','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4856','ETRF89 / NTM zone 6','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4856','EUREF89 / NTM zone 6','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4857','ETRF89 / NTM zone 7','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4857','EUREF89 / NTM zone 7','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4858','ETRF89 / NTM zone 8','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4858','EUREF89 / NTM zone 8','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4859','ETRF89 / NTM zone 9','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4859','EUREF89 / NTM zone 9','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4880','ETRF89 / NTM zone 30','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4880','EUREF89 / NTM zone 30','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4879','ETRF89 / NTM zone 29','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4879','EUREF89 / NTM zone 29','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4878','ETRF89 / NTM zone 28','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4878','EUREF89 / NTM zone 28','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4877','ETRF89 / NTM zone 27','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4877','EUREF89 / NTM zone 27','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4876','ETRF89 / NTM zone 26','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4876','EUREF89 / NTM zone 26','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4875','ETRF89 / NTM zone 25','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4875','EUREF89 / NTM zone 25','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4874','ETRF89 / NTM zone 24','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4874','EUREF89 / NTM zone 24','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4873','ETRF89 / NTM zone 23','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4873','EUREF89 / NTM zone 23','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4872','ETRF89 / NTM zone 22','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4872','EUREF89 / NTM zone 22','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4871','ETRF89 / NTM zone 21','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4871','EUREF89 / NTM zone 21','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4870','ETRF89 / NTM zone 20','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4870','EUREF89 / NTM zone 20','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4869','ETRF89 / NTM zone 19','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4869','EUREF89 / NTM zone 19','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4868','ETRF89 / NTM zone 18','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4868','EUREF89 / NTM zone 18','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4867','ETRF89 / NTM zone 17','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4867','EUREF89 / NTM zone 17','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4866','ETRF89 / NTM zone 16','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4866','EUREF89 / NTM zone 16','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4865','ETRF89 / NTM zone 15','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4865','EUREF89 / NTM zone 15','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4864','ETRF89 / NTM zone 14','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4864','EUREF89 / NTM zone 14','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4863','ETRF89 / NTM zone 13','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4863','EUREF89 / NTM zone 13','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4862','ETRF89 / NTM zone 12','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4862','EUREF89 / NTM zone 12','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4861','ETRF89 / NTM zone 11','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4861','EUREF89 / NTM zone 11','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4860','ETRF89 / NTM zone 10','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4860','EUREF89 / NTM zone 10','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4160','Chos Malal','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4254','Hito XVIII','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2082','Pampa Cas / Argentina 2','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2082','Pampa del Castillo / Argentina zone 2','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4161','Pampa Cas','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22181','POSGAR 94 / Gauss-Kruger zone 1','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22182','POSGAR 94 / Gauss-Kruger zone 2','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22183','POSGAR 94 / Gauss-Kruger zone 3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22184','POSGAR 94 / Gauss-Kruger zone 4','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22185','POSGAR 94 / Gauss-Kruger zone 5','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22187','POSGAR 94 / Gauss-Kruger zone 7','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22172','POSGAR 98 / Gauss-Kruger zone 2','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22171','POSGAR 98 / Gauss-Kruger zone 1','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22173','POSGAR 98 / Gauss-Kruger zone 3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22174','POSGAR 98 / Gauss-Kruger zone 4','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22175','POSGAR 98 / Gauss-Kruger zone 5','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22176','POSGAR 98 / Gauss-Kruger zone 6','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22177','POSGAR 98 / Gauss-Kruger zone 7','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22186','POSGAR 94 / Gauss-Kruger zone 6','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5017','BB DLx','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5018','DLx','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5018','Lisbon 1937 / Portuguese Grid New','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2083','Hito XVIII / Argentina 2','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2081','Chos Malal / Argentina 2','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2393','YKJ','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','7405','OSGB 1936 / British National Grid + Newlyn height','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3067','ETRS-TM35FIN','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3067','ETRS-TM35','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3873','ETRS89 / GK19FIN','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3873','ETRS-GK19FIN','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3874','ETRS89 / GK20FIN','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3874','ETRS-GK20FIN','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3875','ETRS89 / GK21FIN','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3875','ETRS-GK21FIN','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3876','ETRS89 / GK22FIN','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3876','ETRS-GK22FIN','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3877','ETRS89 / GK23FIN','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3877','ETRS-GK23FIN','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3878','ETRS89 / GK24FIN','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3878','ETRS-GK24FIN','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3879','ETRS89 / GK25FIN','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3879','ETRS-GK25FIN','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3880','ETRS89 / GK26FIN','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3880','ETRS-GK26FIN','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3881','ETRS89 / GK27FIN','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3881','ETRS-GK27FIN','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3882','ETRS89 / GK28FIN','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3882','ETRS-GK28FIN','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3883','ETRS89 / GK29FIN','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3883','ETRS-GK29FIN','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3884','ETRS89 / GK30FIN','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3884','ETRS-GK30FIN','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3885','ETRS / GK31FIN','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3885','ETRS89-GK31FIN','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','3903','ETRS-TM35FIN(N,E)/N2000','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','3901','YKJ/N60','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','3901','YKJ + N60 height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','3900','N2000','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5717','N60','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5048','ETRS89 / TM35FIN(N,E)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5048','ETRS-TM35FIN(N,E)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','3902','ETRS89 / TM35FIN(N,E) + N60 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','3902','ETRS-TM35FIN(N,E)/N60','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5781','RO_CONST / NH','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5705','LT_KRON / NH','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5182','KGD2002 / Jeju','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5184','Korea 2000 / East Sea Belt','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5185','Korea 2000 / West Belt 2010','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5186','Korea 2000 / Central Belt 2010','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5187','Korea 2000 / East Belt 2010','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5188','Korea 2000 / East Sea Belt 2010','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5168','Korean 1985 / Jeju Belt','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5167','Korean 1985 / East Sea','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5174','Korean 1985 / Mod Cen','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5175','Korean 1985 / Mod Jeju','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5177','Korean 1985 / Mod E Sea','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4301','Tokyo 1918','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5173','Korean 1985 / Mod West','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5176','Korean 1985 / Mod East','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','5132','Tokyo 1898','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5169','Tokyo / Korea West Belt','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5169','Tokyo 1892 / Korea West','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5169','Tokyo 1898 / Korea West Belt','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5170','Tokyo / Korea Central Belt','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5170','Tokyo 1892 / Korea Cen','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5170','Tokyo 1898 / Korea Central Belt','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5171','Tokyo / Korea East Belt','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5171','Tokyo 1892 / Korea East','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5171','Tokyo 1898 / Korea East Belt','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5172','Tokyo / Korea East Sea Belt','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5172','Tokyo 1892 / Korea E Sea','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5172','Tokyo 1898 / Korea East Sea Belt','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5613','SE_AMST2000 / NH','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5214','IT_GENO / OH','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5195','HR_TRIE / NOH','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5243','ETRS89-LCC-DE(E-N)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5254','TUREF / 3-degree Gauss-Kruger CM 30E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5254','TR_TUREF / TR_TM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5256','TUREF / 3-degree Gauss-Kruger CM 36E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5256','TR_TUREF / TR_TM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5257','TUREF / 3-degree Gauss-Kruger CM 39E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5257','TR_TUREF / TR_TM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5258','TUREF / 3-degree Gauss-Kruger CM 42E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5258','TR_TUREF / TR_TM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5259','TUREF / 3-degree Gauss-Kruger CM 45E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5259','TR_TUREF / TR_TM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5269','TUREF / Turkey zone 9','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5270','TUREF / Turkey zone 10','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5271','TUREF / Turkey zone 11','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5272','TUREF / Turkey zone 12','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5273','TUREF / Turkey zone 13','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5274','TUREF / Turkey zone 14','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5275','TUREF / Turkey zone 15','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5255','TUREF / 3-degree Gauss-Kruger CM 33E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5255','TR_TUREF / TR_TM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5253','TUREF / 3-degree Gauss-Kruger CM 27E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5253','TR_TUREF / TR_TM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5247','GDBD2009 / Brunei RSO','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4484','Mexico 1992 / UTM 11N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4484','Red Geodesica Nacional ITRF92 / UTM 11N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4485','Mexico 1992 / UTM 12N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4485','Red Geodesica Nacional ITRF92 / UTM 12N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4486','Mexico 1992 / UTM 13N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4486','Red Geodesica Nacional ITRF92 / UTM 13N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4487','Mexico 1992 / UTM 14N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4487','Red Geodesica Nacional ITRF92 / UTM 14N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4488','Mexico 1992 / UTM 15N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4488','Red Geodesica Nacional ITRF92 / UTM 15N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4489','Red Geodesica Nacional ITRF92 / UTM 16N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4489','Mexico 1992 / UTM 16N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5266','DRUKREF 03 / Bhutan NG','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5302','DRUKREF 03 / S Jong TM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5309','DRUKREF 03 / Wangdue TM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5300','DRUKREF 03 / P-Gatsh TM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5306','DRUKREF 03 / T-Gang TM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5316','ETRS89 / FOTM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5224','S-JTSK/05 (F) / ModKrovak','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5225','S-JTSK/05 (F) / ModKrovakEN','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5221','S-JTSK (F) / Krovak EN','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5320','NAD83 / Teranet LC','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5321','NAD83(CSRS) / Teranet LC','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5330','Batavia (Jkt) / NEIEZ','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5331','Makassar (Jkt) / NEIEZ','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4248','La Canoa','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31254','AT_MGI / Austria GK','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31256','AT_MGI / Austria GK','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31255','AT_MGI / Austria GK','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3416','AT_ETRS89 / Austria Lambert','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5459','Ocotepeque / Guatemala S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5458','Ocotepeque / Guatemala N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5460','Ocotepeque / El Salvador','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5461','Ocotepeque / Nicaragua N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5462','Ocotepeque / Nicaragua S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5396','SIRGAS 2000 / UTM 26S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5466','Sibun Gorge / Colony Grd','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5469','Panama-Colon / Lambert','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5382','SIRGAS-ROU98 / UTM 21S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5383','SIRGAS-ROU98 / UTM 22S','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4309','ROU-USAMS','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5362','SIRGAS-Chile / UTM 18S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5361','SIRGAS-Chile / UTM 19S','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','5341','Marco de Referencia Geodésico Nacional Oficial','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','5342','Marco de Referencia Geodésico Nacional Oficial','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','5340','Marco de Referencia Geodésico Nacional Oficial','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5349','POSGAR 07 / Argentina 7','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5349','POSGAR 2007 / Gauss-Kruger zone 7','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5348','POSGAR 07 / Argentina 6','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5348','POSGAR 2007 / Gauss-Kruger zone 6','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5347','POSGAR 07 / Argentina 5','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5347','POSGAR 2007 / Gauss-Kruger zone 5','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5346','POSGAR 07 / Argentina 4','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5346','POSGAR 2007 / Gauss-Kruger zone 4','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5345','POSGAR 07 / Argentina 3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5345','POSGAR 2007 / Gauss-Kruger zone 3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5344','POSGAR 2007 / Gauss-Kruger zone 2','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5344','POSGAR 07 / Argentina 2','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5343','POSGAR 07 / Argentina 1','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5343','POSGAR 2007 / Gauss-Kruger zone 1','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5456','Ocotepeque / Costa Rica N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5457','Ocotepeque / C R Sur','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5515','S-JTSK/05 / ModKrovak','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5516','S-JTSK/05 / ModKrovakEN','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5514','S-JTSK / Krovak EN (G)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5518','CI1971 / CIMG','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32183','NAD83 / SCoPQ zone 3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32184','NAD83 / SCoPQ zone 4','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32185','NAD83 / SCoPQ zone 5','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32186','NAD83 / SCoPQ zone 6','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32187','NAD83 / SCoPQ zone 7','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32188','NAD83 / SCoPQ zone 8','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32189','NAD83 / SCoPQ zone 9','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32190','NAD83 / SCoPQ zone 10','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5519','CI1979 / CIMG','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5520','DHDN / 3GK zone 1','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4646','IGN50','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5523','WGS 84 / GTM 2011','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5223','WGS 84 / GTM','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4225','Corrego Alegre','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22524','Corrego Alegre / UTM zone 24S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22523','Corrego Alegre / UTM zone 23S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22522','Corrego Alegre / UTM zone 22S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22521','Corrego Alegre / UTM zone 21S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22525','Corrego Alegre / UTM zone 25S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5536','Corrego Aleg61 / UTM 21S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5537','Corrego Aleg61 / UTM 22S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5538','Corrego Aleg61 / UTM 23S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5539','Corrego Aleg61 / UTM 24S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5472','Panama-Colon / Polyconic','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4674','SIRGAS2000','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4988','SIRGAS2000','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4989','SIRGAS2000','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5396','SIRGAS2000 / UTM zone 26S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31985','SIRGAS2000 / UTM zone 25S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31984','SIRGAS2000 / UTM zone 24S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31983','SIRGAS2000 / UTM zone 23S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31982','SIRGAS2000 / UTM zone 22S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31981','SIRGAS2000 / UTM zone 21S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31980','SIRGAS2000 / UTM zone 20S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31979','SIRGAS2000 / UTM zone 19S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31978','SIRGAS2000 / UTM zone 18S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5559','Ocotepeque / Guatemala N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5589','Sibun Gorge / Colony Grd','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5566','UCS-2000 / 6-degree Gauss-Kruger CM 21E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5566','UCS-2000 / 6GK 21E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5567','UCS-2000 / 6-degree Gauss-Kruger CM 27E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5567','UCS-2000 / 6GK 27E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5568','UCS-2000 / 6-degree Gauss-Kruger CM 33E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5568','UCS-2000 / 6GK 33E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5569','UCS-2000 / 6-degree Gauss-Kruger CM 39E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5569','UCS-2000 / 6GK 39E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5583','UCS-2000 / 3GK 39E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5582','UCS-2000 / 3GK 36E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5581','UCS-2000 / 3GK 33E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5580','UCS-2000 / 3GK 30E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5578','UCS-2000 / 3GK 24E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5579','UCS-2000 / 3GK 27E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5570','UCS-2000 / 3GK zn 7','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5571','UCS-2000 / 3GK zn 8','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5572','UCS-2000 / 3GK zn 9','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5573','UCS-2000 / 3GK zn 10','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5574','UCS-2000 / 3GK zn 11','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5575','UCS-2000 / 3GK zn 12','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5576','UCS-2000 / 3GK zn 13','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5562','UCS-2000 / 6-degree Gauss-Kruger zone 4','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5562','UCS-2000 zone 4','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5563','UCS-2000 / 6-degree Gauss-Kruger zone 5','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5563','UCS-2000 zone 5','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5564','UCS-2000 / 6-degree Gauss-Kruger zone 6','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5564','UCS-2000 zone 6','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5565','UCS-2000 / 6-degree Gauss-Kruger zone 7','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5565','UCS-2000 zone 7','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5577','UCS-2000 / 3GK 21E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5588','NAD27 / NB Stereographic','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5596','Fehmarnbelt Coordinate System','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5596','FCS','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','5598','FCS + FCSVR10 height','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5624','NAD27 / Michigan Old Cen','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3040','ETRS89 / TM28','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5646','NAD83 / VT (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5654','NAD83(HPGN) / Vermont (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5654','NAD83(HARN) / VT (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5655','NAD83(NSRS) / VT (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5680','DHDN / 3GK zone 1','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5676','DHDN / Gauss-Kruger zone 2','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5676','DE_DHDN / GK_3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5676','DHDN / 3GK zone 2','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5677','DHDN / Gauss-Kruger zone 3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5677','DE_DHDN / GK_3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5677','DHDN / 3GK zone 3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5678','DHDN / Gauss-Kruger zone 4','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5678','DE_DHDN / GK_3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5678','DHDN / 3GK zone 4','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5679','DHDN / Gauss-Kruger zone 5','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5679','DE_DHDN / GK_3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5679','DHDN / 3GK zone 5','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5666','DHDN / 3GK zone 3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5666','DE_PD/83 / GK_3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5666','PD/83 / Gauss-Kruger zone 3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5666','PD/83 / 3GK zone 3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5667','DHDN / 3GK zone 4','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5667','DE_PD/83 / GK_3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5667','PD/83 / Gauss-Kruger zone 4','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5667','PD/83 / 3GK zone 4','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5670','Pulkovo 42(58) / 3GK zn3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5671','Pulkovo 42(58) / 3GK zn4','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5672','System 1942/15 (3)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5672','Pulkovo 42(58) / 3GK zn5','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5631','Pulkovo 1942(58) / 6-degree Gauss-Kruger zone 2','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5631','S-42 zone 2','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5631','Pulkovo 42(58) / GK zn 2','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5663','System 1942/15 (6)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5663','S-42 zone 3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5663','Pulkovo 1942(58) / 6-degree Gauss-Kruger zone 3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5663','Pulkovo 42(58) / GK zn 3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5673','Pulkovo 1942(83) / Gauss-Kruger zone 3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5673','DE_42/83 / GK_3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5673','Pulkovo 42(83) / 3GK zn3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5674','Pulkovo 1942(83) / Gauss-Kruger zone 4','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5674','DE_42/83 / GK_3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5674','Pulkovo 42(83) / 3GK zn4','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5675','Pulkovo 1942(83) / Gauss-Kruger zone 5','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5675','DE_42/83 / GK_3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5675','Pulkovo 42(83) / 3GK zn5','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5664','Pulkovo 1942(83) / 6-degree Gauss-Kruger zone 2','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5664','S-42 zone 2','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5664','Pulkovo 42(83) / GK zn 2','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5665','Pulkovo 1942(83) / 6-degree Gauss-Kruger zone 3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5665','S-42 zone 3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5665','Pulkovo 42(83) / GK zn 3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5668','DHDN / Gauss-Kruger zone 4','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5668','DE_RD/83 / GK_3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5668','RD/83 / Gauss-Kruger zone 4','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5668','RD/83 / 3GK zone 4','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5669','DHDN / Gauss-Kruger zone 5','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5669','DE_RD/83 / GK_3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5669','RD/83 / Gauss-Kruger zone 5','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5669','RD/83 / 3GK zone 5','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5825','AGD66 / ACT SGC','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5683','DB_REF2003 / 3GK zone 3 E-N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5682','DB_REF2003 / 3GK zone 2 E-N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5684','DB_REF2003 / 3GK zone 4 E-N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5685','DB_REF2003 / 3GK zone 5 E-N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5836','Yemen NGN96 / UTM 37N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5837','Yemen NGN96 / UTM 40N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','30791','Nord Sahara 1959 / Lambert Algerie Nord','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','30791','Nord Sahara 1959 / LAN','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','30792','Nord Sahara 1959 / Lambert Algerie Sud','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','30792','Nord Sahara 1959 / LAS','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','30791','Nord Sahara 1959 / Voirol Unifie Nord','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','30792','Nord Sahara 1959 / Voirol Unifie Sud','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5659','Monte Mario / UTMRER','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5844','RGRDC 2005 / Congo TM 30','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5868','Mean High Water height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5869','Mean Higher High Water height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5870','Mean High Water Spring Tides height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5871','Higher High Water Large Tide height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5872','Highest Astronomical Tide height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5829','sea level height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5865','Mean Low Water Spring Tides depth','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5867','Mean Low Water depth','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5864','Mean Lower Low Water Spring Tides depth','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5866','Mean Lower Low Water depth','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5831','sea level depth','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5863','Indian Spring Low Water depth','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5863','Indian Tidal Plane depth','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5861','Lowest Astronomical Tide depth','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5862','Lower Low Water Large Tide depth','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5874','High Tide height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5873','Low Tide depth','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5649','ETRS89 / UTM zone 31','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5651','ETRS89 / UTM zone 31','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5652','ETRS89 / UTM zone 32N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5650','ETRS89 / UTM zone 33N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5653','ETRS89 / UTM zone 33N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5879','Cadastre 1997 / UTM 38S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5887','TGD2005 / TMG','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5890','JAXA Snow Depth PS North','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5932','WGS 84 / EPSG Arctic C2','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5931','WGS 84 / EPSG Arctic C1','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5933','WGS 84 / EPSG Arctic C3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5934','WGS 84 / EPSG Arctic C4','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5935','WGS 84 / EPSG Arctic C5','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5921','WGS 84 / EPSG Arctic A1','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5922','WGS 84 / EPSG Arctic A2','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5923','WGS 84 / EPSG Arctic A3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5924','WGS 84 / EPSG Arctic A4','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5925','WGS 84 / EPSG Arctic A5','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5926','WGS 84 / EPSG Arctic B1','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5927','WGS 84 / EPSG Arctic B2','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5928','WGS 84 / EPSG Arctic B3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5929','WGS 84 / EPSG Arctic B4','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5936','WGS 84 / EPSG Alaska PS','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5937','WGS 84 / EPSG Canada PS','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5938','WGS 84 / EPSG Greenland PS','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5939','WGS 84 / EPSG Norway PS','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5940','WGS 84 / EPSG Russia PS','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5930','WGS 84 / EPSG Arctic B5','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','5973','ETRS89 / UTM zone 33N + NN2000 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','5975','ETRS89 / UTM zone 35N + NN2000 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','5974','ETRS89 / UTM zone 34N + NN2000 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','5976','ETRS89 / UTM zone 36N + NN2000 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','5972','ETRS89 / UTM zone 32N + NN2000 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','5971','ETRS89 / UTM zone 31N + NN2000 height','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6050','GR96 / EPSG Arctic 1-25','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6051','GR96 / EPSG Arctic 2-18','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6052','GR96 / EPSG Arctic 2-20','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6053','GR96 / EPSG Arctic 3-29','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6054','GR96 / EPSG Arctic 3-31','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6055','GR96 / EPSG Arctic 3-33','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6056','GR96 / EPSG Arctic 4-20','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6059','GR96 / EPSG Arctic 5-41','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6060','GR96 / EPSG Arctic 5-43','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6061','GR96 / EPSG Arctic 5-45','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6062','GR96 / EPSG Arctic 6-26','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6063','GR96 / EPSG Arctic 6-28','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6067','GR96 / EPSG Arctic 8-20','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6068','GR96 / EPSG Arctic 8-22','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6070','ETRS89 / EPSG Arctic 3-11','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6071','ETRS89 / EPSG Arctic 4-26','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6073','ETRS89 / EPSG Arctic 5-11','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6074','ETRS89 / EPSG Arctic 5-13','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6075','WGS 84 / EPSG Arctic 2-24','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6076','WGS 84 / EPSG Arctic 2-26','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6077','WGS 84 / EPSG Arctic 3-13','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6078','WGS 84 / EPSG Arctic 3-15','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6079','WGS 84 / EPSG Arctic 3-17','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6069','ETRS89 / EPSG Arctic 2-22','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6080','WGS 84 / EPSG Arctic 3-19','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6087','WGS 84 / EPSG Arctic 5-15','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6088','WGS 84 / EPSG Arctic 5-17','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6089','WGS 84 / EPSG Arctic 5-19','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6090','WGS 84 / EPSG Arctic 5-21','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6091','WGS 84 / EPSG Arctic 5-23','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6092','WGS 84 / EPSG Arctic 5-25','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6093','WGS 84 / EPSG Arctic 5-27','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6095','NAD83(NSRS2007) / EPSG Arctic 5-31','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6096','NAD83(NSRS2007) / EPSG Arctic 6-14','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6097','NAD83(NSRS2007) / EPSG Arctic 6-16','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6098','NAD83(CSRS) / EPSG Arctic 1-23','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6099','NAD83(CSRS) / EPSG Arctic 2-14','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6100','NAD83(CSRS) / EPSG Arctic 2-16','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6101','NAD83(CSRS) / EPSG Arctic 3-25','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6102','NAD83(CSRS) / EPSG Arctic 3-27','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6103','NAD83(CSRS) / EPSG Arctic 3-29','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6105','NAD83(CSRS) / EPSG Arctic 4-16','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6106','NAD83(CSRS) / EPSG Arctic 4-18','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6107','NAD83(CSRS) / EPSG Arctic 5-33','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6108','NAD83(CSRS) / EPSG Arctic 5-35','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6109','NAD83(CSRS) / EPSG Arctic 5-37','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6110','NAD83(CSRS) / EPSG Arctic 5-39','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6111','NAD83(CSRS) / EPSG Arctic 6-18','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6112','NAD83(CSRS) / EPSG Arctic 6-20','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6113','NAD83(CSRS) / EPSG Arctic 6-22','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6114','NAD83(CSRS) / EPSG Arctic 6-24','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6115','WGS 84 / EPSG Arctic 1-27','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6116','WGS 84 / EPSG Arctic 1-29','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6117','WGS 84 / EPSG Arctic 1-31','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6118','WGS 84 / EPSG Arctic 1-21','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6119','WGS 84 / EPSG Arctic 2-28','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6121','WGS 84 / EPSG Arctic 2-12','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6120','WGS 84 / EPSG Arctic 2-10','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6122','WGS 84 / EPSG Arctic 3-21','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6123','WGS 84 / EPSG Arctic 3-23','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6064','GR96 / EPSG Arctic 6-30','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6065','GR96 / EPSG Arctic 7-11','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6066','GR96 / EPSG Arctic 7-13','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6128','GCNG59','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6129','SING61','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6086','WGS 84 / EPSG Arctic 4-40','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6085','WGS 84 / EPSG Arctic 4-38','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6084','WGS 84 / EPSG Arctic 4-36','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6083','WGS 84 / EPSG Arctic 4-34','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6082','WGS 84 / EPSG Arctic 4-32','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6081','WGS 84 / EPSG Arctic 4-30','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6124','WGS 84 / EPSG Arctic 4-12','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6125','ETRS89 / EPSG Arctic 5-47','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6072','ETRS89 / EPSG Arctic 4-28','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6104','NAD83(CSRS) / EPSG Arctic 4-14','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6141','CING11','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4723','Grand Cayman 1959','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4726','Little Cayman 1961','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','6171','ETRS89 / UTM zone 31N + NN54 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','6172','ETRS89 / UTM zone 32N + NN54 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','6173','ETRS89 / UTM zone 33N + NN54 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','6174','ETRS89 / UTM zone 34N + NN54 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','6175','ETRS89 / UTM zone 35N + NN54 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','6176','ETRS89 / UTM zone 36N + NN54 height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5710','BE_OOST / UNCOR','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5710','Oostende height','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6057','GR96 / EPSG Arctic 4-22','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6058','GR96 / EPSG Arctic 4-24','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6094','NAD83(NSRS2007) / EPSG Arctic 5-29','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6210','SIRGAS 2000 / UTM 23N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6211','SIRGAS 2000 / UTM 24N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6244','MAGNA-SIRGAS / Arauca','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6245','MAGNA-SIRGAS / Armenia','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6247','MAGNA-SIRGAS / Bogota DC','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6248','MAGNA-SIRGAS / Bucaramanga','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6251','MAGNA-SIRGAS / Cucuta','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6250','MAGNA-SIRGAS / Cartagena','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6252','MAGNA-SIRGAS / Florencia','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6253','MAGNA-SIRGAS / Ibague','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6254','MAGNA-SIRGAS / Inirida','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6255','MAGNA-SIRGAS / Leticia','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6256','MAGNA-SIRGAS / Manizales','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6257','MAGNA-SIRGAS / Medellin','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6258','MAGNA-SIRGAS / Mitu','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6259','MAGNA-SIRGAS / Mocoa','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6260','MAGNA-SIRGAS / Monteria','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6261','MAGNA-SIRGAS / Neiva','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6262','MAGNA-SIRGAS / Pasto','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6263','MAGNA-SIRGAS / Pereira','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6264','MAGNA-SIRGAS / Popayan','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6265','MAGNA-SIRGAS / Puerto Carreno','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6266','MAGNA-SIRGAS / Quibdo','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6267','MAGNA-SIRGAS / Riohacha','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6268','MAGNA-SIRGAS / San Andres','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6269','MAGNA-SIRGAS / San Jose','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6270','MAGNA-SIRGAS / Santa Marta','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6271','MAGNA-SIRGAS / Sucre','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6272','MAGNA-SIRGAS / Tunja','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6273','MAGNA-SIRGAS / Valledupar','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6274','MAGNA-SIRGAS / Villavicencio','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6275','MAGNA-SIRGAS / Yopal','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6249','MAGNA-SIRGAS / Cali','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6246','MAGNA-SIRGAS / Barranquilla','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6204','MGI 1901 / Macedonia GK','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6204','MGI 1901 / Macedonia Gauss-Kruger','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6204','MSCS','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6316','MSCS zone 7','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6351','NAD83(2011) / EPSG Arctic 5-29','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6352','NAD83(2011) / EPSG Arctic 5-31','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6353','NAD83(2011) / EPSG Arctic 6-14','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6354','NAD83(2011) / EPSG Arctic 6-16','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','6357','North American Vertical Datum of 1988 depth (m)','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','6359','National Geodetic Vertical Datum of 1929 depth (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','6360','North American Vertical Datum of 1988 height (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','6358','North American Vertical Datum of 1988 depth (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4481','Mexican Datum of 1993','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4482','Mexican Datum of 1993','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4483','Mexican Datum of 1993','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4484','Mexican Datum of 1993 / UTM zone 11N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4485','Mexican Datum of 1993 / UTM zone 12N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4486','Mexican Datum of 1993 / UTM zone 13N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4487','Mexican Datum of 1993 / UTM zone 14N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4489','Mexican Datum of 1993 / UTM zone 16N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','4488','Mexican Datum of 1993 / UTM zone 15N','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','6365','Red Geodesica Nacional ITRF2008','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6371','Red Geodesica Nacional ITRF2008 / UTM 16N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6371','Mexico 2008 / UTM 16N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6370','Mexico 2008 / UTM 15N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6370','Red Geodesica Nacional ITRF2008 / UTM 15N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6369','Mexico 2008 / UTM 14N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6369','Red Geodesica Nacional ITRF2008 / UTM 14N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6368','Mexico 2008 / UTM 13N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6368','Red Geodesica Nacional ITRF2008 / UTM 13N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6367','Mexico 2008 / UTM 12N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6367','Red Geodesica Nacional ITRF2008 / UTM 12N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6366','Mexico 2008 / UTM 11N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6366','Red Geodesica Nacional ITRF2008 / UTM 11N','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','6364','Red Geodesica Nacional ITRF2008','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','6363','Red Geodesica Nacional ITRF2008','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6381','UCS-2000 / 3GK 21E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6382','UCS-2000 / 3GK 24E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6384','UCS-2000 / 3GK 30E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6383','UCS-2000 / 3GK 27E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6385','UCS-2000 / 3GK 33E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6386','UCS-2000 / 3GK 36E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6387','UCS-2000 / 3GK 39E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6362','Red Geodesica Nacional ITRF92 / CCL','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6372','Red Geodesica Nacional ITRF2008 / CCL','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6391','CING11','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6395','NAD83(2011) / Alaska zone 2 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6395','NAD83(2011) / AK 2 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6355','NAD83(2011) / Alabama East (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6355','NAD83(2011) / AL E (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6356','NAD83(2011) / Alabama West (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6356','NAD83(2011) / AL W (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6393','NAD83(2011) / AK Alb','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6394','NAD83(2011) / Alaska zone 1 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6394','NAD83(2011) / AK 1 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6396','NAD83(2011) / Alaska zone 3 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6396','NAD83(2011) / AK 3 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6397','NAD83(2011) / Alaska zone 4 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6397','NAD83(2011) / AK 4 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6398','NAD83(2011) / Alaska zone 5 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6398','NAD83(2011) / AK 5 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6399','NAD83(2011) / Alaska zone 6 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6399','NAD83(2011) / AK 6 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6400','NAD83(2011) / Alaska zone 7 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6400','NAD83(2011) / AK 7 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6401','NAD83(2011) / Alaska zone 8 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6401','NAD83(2011) / AK 8 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6402','NAD83(2011) / Alaska zone 9 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6402','NAD83(2011) / AK 9 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6403','NAD83(2011) / Alaska zone 10 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6403','NAD83(2011) / AK 10 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6404','NAD83(2011) / Arizona Central (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6404','NAD83(2011) / AZ C (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6405','NAD83(2011) / AZ C (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6406','NAD83(2011) / Arizona East (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6406','NAD83(2011) / AZ E (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6407','NAD83(2011) / AZ E (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6408','NAD83(2011) / Arizona West (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6408','NAD83(2011) / AZ W (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6409','NAD83(2011) / AZ W (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6410','NAD83(2011) / Arkansas North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6410','NAD83(2011) / AR N (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6411','NAD83(2011) / AR N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6412','NAD83(2011) / Arkansas South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6412','NAD83(2011) / AR S (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6413','NAD83(2011) / AR S (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6414','NAD83(2011) / CA Albers','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6416','NAD83(2011) / CA 1 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6415','NAD83(2011) / California zone 1 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6415','NAD83(2011) / CA 1 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6417','NAD83(2011) / California zone 2 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6417','NAD83(2011) / CA 2 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6418','NAD83(2011) / CA 2 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6419','NAD83(2011) / California zone 3 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6419','NAD83(2011) / CA 3 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6420','NAD83(2011) / CA 3 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6421','NAD83(2011) / California zone 4 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6421','NAD83(2011) / CA 4 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6422','NAD83(2011) / CA 4 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6423','NAD83(2011) / California zone 5 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6423','NAD83(2011) / CA 5 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6424','NAD83(2011) / CA 5 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6425','NAD83(2011) / California zone 6 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6425','NAD83(2011) / CA 6 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6426','NAD83(2011) / CA 6 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6427','NAD83(2011) / Colorado Central (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6427','NAD83(2011) / CO C (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6428','NAD83(2011) / CO C (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6429','NAD83(2011) / Colorado North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6429','NAD83(2011) / CO N (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6430','NAD83(2011) / CO N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6431','NAD83(2011) / Colorado South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6431','NAD83(2011) / CO S (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6432','NAD83(2011) / CO S (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6433','NAD83(2011) / Connecticut (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6433','NAD83(2011) / CT (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6434','NAD83(2011) / CT (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6435','NAD83(2011) / Delaware (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6435','NAD83(2011) / DE (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6436','NAD83(2011) / DE (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6437','NAD83(2011) / Florida East (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6437','NAD83(2011) / FL E (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6438','NAD83(2011) / FL E (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6439','NAD83(2011) / FL GDL AEA','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6440','NAD83(2011) / Florida North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6440','NAD83(2011) / FL N (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6441','NAD83(2011) / FL N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6442','NAD83(2011) / Florida West (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6442','NAD83(2011) / FL W (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6443','NAD83(2011) / FL W (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6444','NAD83(2011) / Georgia East (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6444','NAD83(2011) / GA E (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6445','NAD83(2011) / GA E (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6446','NAD83(2011) / Georgia West (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6446','NAD83(2011) / GA W (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6447','NAD83(2011) / GA W (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6448','NAD83(2011) / Idaho Central (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6448','NAD83(2011) / ID C (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6449','NAD83(2011) / ID C (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6450','NAD83(2011) / Idaho East (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6450','NAD83(2011) / ID E (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6451','NAD83(2011) / ID E (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6452','NAD83(2011) / Idaho West (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6452','NAD83(2011) / ID W (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6453','NAD83(2011) / ID W (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6454','NAD83(2011) / Illinois East (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6454','NAD83(2011) / IL E (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6455','NAD83(2011) / IL E (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6456','NAD83(2011) / Illinois West (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6456','NAD83(2011) / IL W (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6457','NAD83(2011) / IL W (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6458','NAD83(2011) / Indiana East (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6458','NAD83(2011) / IN E (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6459','NAD83(2011) / IN E (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6460','NAD83(2011) / Indiana West (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6460','NAD83(2011) / IN W (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6461','NAD83(2011) / IN W (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6462','NAD83(2011) / Iowa North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6462','NAD83(2011) / IA N (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6464','NAD83(2011) / Iowa South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6464','NAD83(2011) / IA S (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6466','NAD83(2011) / Kansas North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6466','NAD83(2011) / KS N (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6468','NAD83(2011) / Kansas South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6468','NAD83(2011) / KS S (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6470','NAD83(2011) / Kentucky North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6470','NAD83(2011) / KY N (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6471','NAD83(2011) / KY N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6472','NAD83(2011) / Kentucky Single Zone (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6472','KY1Z','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6472','NAD83(2011) / KY1Z (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6473','NAD83(2011) / KY1Z (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6473','KY1Z','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6474','NAD83(2011) / Kentucky South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6474','NAD83(2011) / KY S (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6475','NAD83(2011) / KY S (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6476','NAD83(2011) / Louisiana North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6476','NAD83(2011) / LA N (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6477','NAD83(2011) / LA N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6478','NAD83(2011) / Louisiana South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6478','NAD83(2011) / LA S (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6479','NAD83(2011) / LA S (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6480','NAD83(2011) / Maine CS2000 Central (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6480','Maine Coordinate System of 2000 Central Zone','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6480','NAD83(2011) / ME 2000 C','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6481','NAD83(2011) / Maine CS2000 East (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6481','Maine Coordinate System of 2000 East Zone','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6481','NAD83(2011) / ME 2000 E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6482','NAD83(2011) / Maine CS2000 West (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6482','Maine Coordinate System of 2000 West Zone','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6482','NAD83(2011) / ME 2000 W','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6483','NAD83(2011) / Maine CS83 East (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6483','Maine Coordinate System of 1983 East Zone','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6483','NAD83(2011) / ME83 E (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6484','NAD83(2011) / Maine CS83 East (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6484','NAD83(2011) / ME E (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6485','NAD83(2011) / Maine West (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6485','Maine Coordinate System of 1983 West Zone','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6485','NAD83(2011) / ME83 W (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6486','NAD83(2011) / Maine CS83 West (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6486','NAD83(2011) / ME W (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6487','NAD83(2011) / Maryland (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6487','NAD83(2011) / MD (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6488','NAD83(2011) / MD (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6489','NAD83(2011) / Massachusetts Island (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6489','NAD83(2011) / MA Is (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6490','NAD83(2011) / MA Is (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6491','NAD83(2011) / Massachusetts Mainland (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6491','NAD83(2011) / MA md (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6492','NAD83(2011) / MA md (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6493','NAD83(2011) / Michigan Central (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6493','NAD83(2011) / MI C (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6494','NAD83(2011) / MI C (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6495','NAD83(2011) / Michigan North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6495','NAD83(2011) / MI N (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6496','NAD83(2011) / MI N (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6497','NAD83(2011) / MI Georef','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6498','NAD83(2011) / Michigan South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6498','NAD83(2011) / MI S (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6499','NAD83(2011) / MI S (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6500','NAD83(2011) / Minnesota Central (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6500','NAD83(2011) / MN C (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6501','NAD83(2011) / MN C (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6502','NAD83(2011) / Minnesota North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6502','NAD83(2011) / MN N (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6503','NAD83(2011) / MN N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6504','NAD83(2011) / Minnesota South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6504','NAD83(2011) / MN S (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6505','NAD83(2011) / MN S (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6506','NAD83(2011) / Mississippi East (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6506','NAD83(2011) / MS E (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6507','NAD83(2011) / MS E (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6508','NAD83(2011) / MSTM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6509','NAD83(2011) / Mississippi West (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6509','NAD83(2011) / MS W (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6510','NAD83(2011) / MS W (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6511','NAD83(2011) / Missouri Central (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6511','NAD83(2011) / MO C (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6512','NAD83(2011) / Missouri East (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6512','NAD83(2011) / MO E (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6513','NAD83(2011) / Missouri West (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6513','NAD83(2011) / MO W (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6514','NAD83(2011) / Montana (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6514','NAD83(2011) / MT (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6515','NAD83(2011) / MT (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6518','NAD83(2011) / Nevada Central (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6518','NAD83(2011) / NV C (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6520','NAD83(2011) / Nevada East (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6520','NAD83(2011) / NV E (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6522','NAD83(2011) / Nevada West (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6522','NAD83(2011) / NV W (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6524','NAD83(2011) / New Hampshire (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6524','NAD83(2011) / NH (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6525','NAD83(2011) / NH (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6526','NAD83(2011) / New Jersey (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6526','NAD83(2011) / NJ (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6528','NAD83(2011) / New Mexico Central (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6528','NAD83(2011) / NM C (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6529','NAD83(2011) / NM C (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6530','NAD83(2011) / New Mexico East (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6530','NAD83(2011) / NM E (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6531','NAD83(2011) / NM E (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6532','NAD83(2011) / New Mexico West (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6532','NAD83(2011) / NM W (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6533','NAD83(2011) / NM W (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6534','NAD83(2011) / New York Central (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6534','NAD83(2011) / NY C (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6535','NAD83(2011) / NY C (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6536','NAD83(2011) / New York East (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6536','NAD83(2011) / NY E (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6537','NAD83(2011) / NY E (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6538','NAD83(2011) / New York Long Island (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6538','NAD83(2011) / NY LI (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6539','NAD83(2011) / NY LI (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6540','NAD83(2011) / New York West (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6540','NAD83(2011) / NY W (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6541','NAD83(2011) / NY W (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6542','NAD83(2011) / North Carolina (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6542','NAD83(2011) / NC (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6543','NAD83(2011) / NC (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6544','NAD83(2011) / North Dakota North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6544','NAD83(2011) / ND N (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6545','NAD83(2011) / ND N (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6546','NAD83(2011) / North Dakota South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6546','NAD83(2011) / ND S (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6547','NAD83(2011) / ND S (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6548','NAD83(2011) / Ohio North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6549','NAD83(2011) / OH N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6550','NAD83(2011) / Ohio South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6551','NAD83(2011) / OH S (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6552','NAD83(2011) / Oklahoma North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6552','NAD83(2011) / OK N (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6553','NAD83(2011) / OK N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6554','NAD83(2011) / Oklahoma South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6554','NAD83(2011) / OK S (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6555','NAD83(2011) / OK S (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6557','NAD83(2011) / OR GIC Lam (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6558','NAD83(2011) / Oregon North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6558','NAD83(2011) / OR N (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6559','NAD83(2011) / OR N (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6560','NAD83(2011) / Oregon South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6560','NAD83(2011) / OR S (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6561','NAD83(2011) / OR S (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6562','NAD83(2011) / Pennsylvania North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6562','NAD83(2011) / PA N (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6563','NAD83(2011) / PA N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6564','NAD83(2011) / Pennsylvania South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6564','NAD83(2011) / PA S (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6565','NAD83(2011) / PA S (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6566','NAD83(2011) / PR and VI','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6567','NAD83(2011) / Rhode Island (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6567','NAD83(2011) / RI (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6568','NAD83(2011) / RI (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6569','NAD83(2011) / South Carolina (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6569','NAD83(2011) / SC (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6570','NAD83(2011) / SC (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6571','NAD83(2011) / South Dakota North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6571','NAD83(2011) / SD N (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6572','NAD83(2011) / SD N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6573','NAD83(2011) / South Dakota South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6573','NAD83(2011) / SD S (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6574','NAD83(2011) / SD S (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6575','NAD83(2011) / Tennessee (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6575','NAD83(2011) / TN (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6576','NAD83(2011) / TN (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6577','NAD83(2011) / Texas Central (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6577','NAD83(2011) / TX C (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6578','NAD83(2011) / TX C (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6579','NAD83(2011) / TX Albers','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6580','NAD83(2011) / TX LC','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6581','NAD83(2011) / Texas North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6581','NAD83(2011) / TX N (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6582','NAD83(2011) / TX N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6583','NAD83(2011) / Texas North Central (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6583','NAD83(2011) / TX NC (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6584','NAD83(2011) / TX NC (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6585','NAD83(2011) / Texas South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6585','NAD83(2011) / TX S (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6586','NAD83(2011) / TX S (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6587','NAD83(2011) / Texas South Central (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6587','NAD83(2011) / TX SC (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6588','NAD83(2011) / TX SC (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6589','NAD83(2011) / Vermont (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6589','NAD83(2011) / VT (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6590','NAD83(2011) / VT (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6591','NAD83(2011) / VA LCC','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6592','NAD83(2011) / Virginia North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6592','NAD83(2011) / VA N (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6593','NAD83(2011) / VA N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6594','NAD83(2011) / Virginia South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6594','NAD83(2011) / VA S (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6595','NAD83(2011) / VA S (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6596','NAD83(2011) / Washington North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6596','NAD83(2011) / WA N (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6597','NAD83(2011) / WA N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6598','NAD83(2011) / Washington South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6598','NAD83(2011) / WA S (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6599','NAD83(2011) / WA S (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6600','NAD83(2011) / West Virginia North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6600','NAD83(2011) / WV N (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6601','NAD83(2011) / WV N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6602','NAD83(2011) / West Virginia South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6602','NAD83(2011) / WV S (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6603','NAD83(2011) / WV S (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6604','NAD83(2011) / Wisconsin Central (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6604','NAD83(2011) / WI C (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6605','NAD83(2011) / WI C (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6606','NAD83(2011) / Wisconsin North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6606','NAD83(2011) / WI N (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6607','NAD83(2011) / WI N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6608','NAD83(2011) / Wisconsin South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6608','NAD83(2011) / WI S (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6609','NAD83(2011) / WI S (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6610','NAD83(2011) / WTM 83','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6611','NAD83(2011) / Wyoming East (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6611','NAD83(2011) / WY E (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6612','NAD83(2011) / WY E (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6613','NAD83(2011) / Wyoming East Central (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6613','NAD83(2011) / WY EC (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6614','NAD83(2011) / WY EC (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6615','NAD83(2011) / Wyoming West (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6615','NAD83(2011) / WY W (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6618','NAD83(2011) / WY WC (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6616','NAD83(2011) / WY W (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6628','NAD83(PA11) / Hawaii zone 1 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6628','NAD83(PA11) / HI 1 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6629','NAD83(PA11) / Hawaii zone 2 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6629','NAD83(PA11) / HI 2 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6630','NAD83(PA11) / Hawaii zone 3 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6630','NAD83(PA11) / HI 3 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6631','NAD83(PA11) / Hawaii zone 4 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6631','NAD83(PA11) / HI 4 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6633','NAD83(PA11) / HI 3 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6634','NAD83(PA11) / UTM 4N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6632','NAD83(PA11) / Hawaii zone 5 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6632','NAD83(PA11) / HI 5 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6635','NAD83(PA11) / UTM 5N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6636','NAD83(PA11) / UTM 2S','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','6640','Northern Marianas Vertical Datum of 2003 height (m)','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','6642','Virgin Islands Vertical Datum of 2009 height (m)','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','6643','American Samoa Vertical Datum of 2002 height (m)','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','6638','Tutuila Vertical Datum of 1962 height (m)','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','6644','Guam Vertical Datum of 2004 height (m)','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','6639','Guam Vertical Datum of 1963 height (m)','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','6641','Puerto Rico Vertical Datum of 2002 height (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6619','NAD83(2011) / Utah Central (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6619','NAD83(2011) / UT C (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6620','NAD83(2011) / Utah North (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6620','NAD83(2011) / UT N (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6621','NAD83(2011) / Utah South (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6621','NAD83(2011) / UT S (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6625','NAD83(2011) / UT C (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6626','NAD83(2011) / UT N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6627','NAD83(2011) / UT S (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6463','NAD83(2011) / IA N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6465','NAD83(2011) / IA S (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6467','NAD83(2011) / KS N (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6469','NAD83(2011) / KS S (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6516','NAD83(2011) / Nebraska (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6516','NAD83(2011) / NE (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6517','NAD83(2011) / NE (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6519','NAD83(2011) / NV C (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6521','NAD83(2011) / NV E (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6523','NAD83(2011) / NV W (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6527','NAD83(2011) / NJ (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6617','NAD83(2011) / Wyoming West Central (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6617','NAD83(2011) / WY WC (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6646','Karbala 1979 (Polservice) / Iraq National Grid','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6646','Karbala / Iraq Nat. Grid','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','6647','CGVD2013 height','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6669','JGD2011 / Japan zone 1','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6670','JGD2011 / Japan zone 2','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6671','JGD2011 / Japan zone 3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6672','JGD2011 / Japan zone 4','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6673','JGD2011 / Japan zone 5','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6674','JGD2011 / Japan zone 6','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6675','JGD2011 / Japan zone 7','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6676','JGD2011 / Japan zone 8','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6678','JGD2011 / Japan zone 10','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6679','JGD2011 / Japan zone 11','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6680','JGD2011 / Japan zone 12','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6681','JGD2011 / Japan zone 13','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6682','JGD2011 / Japan zone 14','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6687','JGD2011 / Japan zone 19','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6683','JGD2011 / Japan zone 15','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6684','JGD2011 / Japan zone 16','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6685','JGD2011 / Japan zone 17','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6686','JGD2011 / Japan zone 18','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5723','JSLD height','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6677','JGD2011 / Japan zone 9','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','6693','Japan Levelling Datum height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','6693','Japanese Standard Levelling Datum height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','6693','JSLD height','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2062','Madrid (Madrid) / Spain LCC','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2062','Madrid - LCC','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9380','725','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9380','IGb14 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5711','Australian Height Datum height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5711','AHD71 height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5711','AHD-TAS83 height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5711','339','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5711','AHD - NOHt','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','9502','CIGD11 + CBVD61 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','9504','CIGD11 + LCVD61 height','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','29873','Timbalai / Borneo (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','29873','BT68 / RSO Borneo (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','29873','Timbalai 1968 / RSO Borneo (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','29874','BT68 / RSO Sarawak LSD (m)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','6705','ETRF2000 epoca 2008.0','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','6704','ETRF2000 epoca 2008.0','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','6706','ETRF2000 epoca 2008.0','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6708','RDN2008 / TM33','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6708','ETRF2000 epoca 2008.0 fuso 33','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6707','RDN2008 / TM32','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6707','ETRF2000 epoca 2008.0 fuso 32','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6709','RDN2008 / TM34','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6709','ETRF2000 epoca 2008.0 fuso 34','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6875','RDN2008 / Fuso Italia (N-E)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6875','ETRF2000 epoca 2008.0 fuso Italia','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6876','RDN2008 / Fuso 12 (N-E)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6876','ETRF2000 epoca 2008.0 fuso 12','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','9503','CIGD11 + GCVD54 height','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9498','CABA-P07','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','9389','EVRF2019_AMST / NH','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','9390','EVRF2019mean_AMST / NH','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','9675','741','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','9675','LT at Pago Pago - NOHt','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3106','Gulshan 303 / Bangladesh Transverse Mercator','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3106','Gulshan 303 / BUTM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9678','Gulshan / Bangladesh TM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9680','WGS 84 / Bangladesh Universal Transverse Mercator','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9680','WGS 84 / BUTM 2010','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9549','LTF2004(G) / LTF2004(C)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','7956','St. Helena Map Grid 2015 + SHVD2015 height','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2176','ETRS89 / Poland CS2000 zone 5','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2177','ETRS89 / Poland CS2000 zone 6','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2178','ETRS89 / Poland CS2000 zone 7','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2179','ETRS89 / Poland CS2000 zone 8','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2180','ETRS89 / Poland CS92','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','9650','PL_KRON / NH','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','9650','Kronzstad 86','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','9650','Kronstadt 1986 height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','9651','PL_AMST / NH','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','9651','PL-EVRF2007-NH','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9700','PL-ETRF2000','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9701','PL-ETRF2000','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8401','748','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8401','ETRF2014 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8403','747','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8403','ETRF2014 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9069','746','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9069','ETRF2014 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','9672','CD Norway','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','9656','PL-ETRF2000 + Kronzstad 86','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','9657','PL-ETRF2000 + Kronzstad 86','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4277','OSGB 1936','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','27700','OSGB 1936 / British National Grid','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9702','PL-ETRF2000','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9713','NAD83(CSRS) / UTM 24N','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4265','Roma40','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4982','ETRF89 IT','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4983','ETRF89 IT','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4670','ETRF89 IT','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3065','ETRF89 IT / UTM zone 33N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3064','ETRF89 IT / UTM zone 32N','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5214','Genoa height','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9741','ETRS89 / EOS21 SnakeGrid','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','9742','ETRS89 / EOS21 SnakeGrid + Newlyn height','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9716','ETRF89 IT / UTM zone 34N','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5608','770','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5608','IGLD (1955) - DHt','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5609','771','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5609','IGLD (1985) - DHt','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','9390','766','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','9390','EVRF2019mean - NHt','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','9389','765','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','9389','EVRF2019 - NHt','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9749','NAD83(2011) / AL W (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9748','NAD83(2011) / AL E (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20904','GSK-2011 / 6-degree Gauss-Kruger zone 4','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20906','GSK-2011 / 6-degree Gauss-Kruger zone 6','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20907','GSK-2011 / 6-degree Gauss-Kruger zone 7','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20905','GSK-2011 / 6-degree Gauss-Kruger zone 5','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20908','GSK-2011 / 6-degree Gauss-Kruger zone 8','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20909','GSK-2011 / 6-degree Gauss-Kruger zone 9','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20910','GSK-2011 / 6-degree Gauss-Kruger zone 10','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20911','GSK-2011 / 6-degree Gauss-Kruger zone 11','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20912','GSK-2011 / 6-degree Gauss-Kruger zone 12','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20913','GSK-2011 / 6-degree Gauss-Kruger zone 13','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20914','GSK-2011 / 6-degree Gauss-Kruger zone 14','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20915','GSK-2011 / 6-degree Gauss-Kruger zone 15','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20916','GSK-2011 / 6-degree Gauss-Kruger zone 16','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20917','GSK-2011 / 6-degree Gauss-Kruger zone 17','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20919','GSK-2011 / 6-degree Gauss-Kruger zone 19','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20920','GSK-2011 / 6-degree Gauss-Kruger zone 20','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20921','GSK-2011 / 6-degree Gauss-Kruger zone 21','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20922','GSK-2011 / 6-degree Gauss-Kruger zone 22','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20923','GSK-2011 / 6-degree Gauss-Kruger zone 23','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20924','GSK-2011 / 6-degree Gauss-Kruger zone 24','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20925','GSK-2011 / 6-degree Gauss-Kruger zone 25','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20926','GSK-2011 / 6-degree Gauss-Kruger zone 26','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20927','GSK-2011 / 6-degree Gauss-Kruger zone 27','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20929','GSK-2011 / 6-degree Gauss-Kruger zone 29','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20930','GSK-2011 / 6-degree Gauss-Kruger zone 30','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20931','GSK-2011 / 6-degree Gauss-Kruger zone 31','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20932','GSK-2011 / 6-degree Gauss-Kruger zone 32','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20928','GSK-2011 / 6-degree Gauss-Kruger zone 28','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21005','GSK-2011 / 6-degree Gauss-Kruger CM 27E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21006','GSK-2011 / 6-degree Gauss-Kruger CM 33E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21007','GSK-2011 / 6-degree Gauss-Kruger CM 39E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21008','GSK-2011 / 6-degree Gauss-Kruger CM 45E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21010','GSK-2011 / 6-degree Gauss-Kruger CM 57E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21011','GSK-2011 / 6-degree Gauss-Kruger CM 63E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21012','GSK-2011 / 6-degree Gauss-Kruger CM 69E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21013','GSK-2011 / 6-degree Gauss-Kruger CM 75E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21014','GSK-2011 / 6-degree Gauss-Kruger CM 81E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21015','GSK-2011 / 6-degree Gauss-Kruger CM 87E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21016','GSK-2011 / 6-degree Gauss-Kruger CM 93E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21017','GSK-2011 / 6-degree Gauss-Kruger CM 99E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21018','GSK-2011 / 6-degree Gauss-Kruger CM 105E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21019','GSK-2011 / 6-degree Gauss-Kruger CM 111E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21020','GSK-2011 / 6-degree Gauss-Kruger CM 117E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21021','GSK-2011 / 6-degree Gauss-Kruger CM 123E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21022','GSK-2011 / 6-degree Gauss-Kruger CM 129E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21023','GSK-2011 / 6-degree Gauss-Kruger CM 135E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21024','GSK-2011 / 6-degree Gauss-Kruger CM 141E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21025','GSK-2011 / 6-degree Gauss-Kruger CM 147E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21026','GSK-2011 / 6-degree Gauss-Kruger CM 153E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21027','GSK-2011 / 6-degree Gauss-Kruger CM 159E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21028','GSK-2011 / 6-degree Gauss-Kruger CM 165E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21029','GSK-2011 / 6-degree Gauss-Kruger CM 171E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21030','GSK-2011 / 6-degree Gauss-Kruger CM 177E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21031','GSK-2011 / 6-degree Gauss-Kruger CM 177W','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21032','GSK-2011 / 6-degree Gauss-Kruger CM 171W','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21004','GSK-2011 / 6-degree Gauss-Kruger CM 21E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21009','GSK-2011 / 6-degree Gauss-Kruger CM 51E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20918','GSK-2011 / 6-degree Gauss-Kruger zone 18','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21500','Belge 1950 (Brussels) / Belge Lambert 50','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31300','Belge 1972 / Belge Lambert 72','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','31370','Belge 1972 / Belgian Lambert 72','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','6190','Belge 1972 / Belgian Lambert 72 + Ostend height','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7664','WGS 84 (G1762'')','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7665','WGS 84 (G1762'')','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9057','WGS 84 (G1762'')','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9761','ETRS89 / ECML14_NB SnakeGrid','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','9762','ETRS89 / ECML14_NB SnakeGrid + Newlyn height','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9766','ETRS89 / EWR2 SnakeGrid','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','9767','ETRS89 / EWR2 SnakeGrid + Newlyn height','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4932','CHTRF95','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4933','CHTRF95','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4151','CHTRF95','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4964','RGF93','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4965','RGF93','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4171','RGF93','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7042','RGF93 (lon-lat)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','7084','RGF93 (lon-lat)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','9538','RGF93 + NGF-IGN69 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','9539','RGF93 + NGF-IGN78 height','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2154','RGF93 / Lambert-93','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3942','RGF93 / CC42','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3943','RGF93 / CC43','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3944','RGF93 / CC44','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3945','RGF93 / CC45','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3946','RGF93 / CC46','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3947','RGF93 / CC47','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3948','RGF93 / CC48','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3949','RGF93 / CC49','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3950','RGF93 / CC50','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','5698','RGF93 / Lambert-93 + NGF-IGN69 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','5699','RGF93 / Lambert-93 + NGF-IGN78 height','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9793','(FR_RGF93 / LAMB93 - see alias remarks)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9776','RGF93 v2 (lat-lon)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9777','RGF93 v2 (lat-lon)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9781','RGF93 v2b (lat-lon)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9782','RGF93 v2b (lat-lon)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8233','NAD83(CSRS) 1997','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8235','NAD83(CSRS) 1997','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8237','NAD83(CSRS) 1997','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8238','NAD83(CSRS) 1997','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8239','NAD83(CSRS) 1997','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8240','NAD83(CSRS) 1997','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8242','NAD83(CSRS) 2002','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8244','NAD83(CSRS) 2002','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8246','NAD83(CSRS) 2002','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8247','NAD83(CSRS) 2005','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8248','NAD83(CSRS) 2005','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8249','NAD83(CSRS) 2005','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8250','NAD83(CSRS) 2010','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8251','NAD83(CSRS) 2010','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8252','NAD83(CSRS) 2010','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8253','NAD83(CSRS) 2010','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8254','NAD83(CSRS) 2010','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8255','NAD83(CSRS) 2010','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9831','UA_UCS_2000 / LCS_01','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9832','UA_UCS_2000 / LCS_05','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9833','UA_UCS_2000 / LCS_07','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9834','UA_UCS_2000 / LCS_12','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9836','UA_UCS_2000 / LCS_18','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9835','UA_UCS_2000 / LCS_14','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9837','UA_UCS_2000 / LCS_21','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9838','UA_UCS_2000 / LCS_23','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9839','UA_UCS_2000 / LCS_26','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9840','UA_UCS_2000 / LCS_35','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9841','UA_UCS_2000 / LCS_44','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9851','UA_UCS_2000 / LCS_46','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9852','UA_UCS_2000 / LCS_48','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9853','UA_UCS_2000 / LCS_51','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9856','UA_UCS_2000 / LCS_59','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9854','UA_UCS_2000 / LCS_53','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9331','777','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9331','KSA-GRF17 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9332','778','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9332','KSA-GRF17 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9333','779','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9333','KSA-GRF17 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','9335','780','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','9335','KSA-VRF14 - OHt','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9858','UA_UCS_2000 / LCS_63','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9857','UA_UCS_2000 / LCS_61','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9859','UA_UCS_2000 / LCS_65','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9865','UA_UCS_2000 / LCS_85','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9863','UA_UCS_2000 / LCS_74','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9862','UA_UCS_2000 / LCS_73','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9861','UA_UCS_2000 / LCS_71','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9821','UA_UCS_2000 / LCS_32','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9860','UA_UCS_2000 / LCS_68','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9855','UA_UCS_2000 / LCS_56','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9864','UA_UCS_2000 / LCS_80','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9869','ETRS89 / MRH21 SnakeGrid','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','9870','ETRS89 / MRH21 SnakeGrid + Newlyn height','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9880','ETRS89 / MOLDOR11 SnakeGrid','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','9881','ETRS89 / MOLDOR11 SnakeGrid + Newlyn height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5774','NG-L height','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','5558','UA_UCS-2000 / XYZ','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','5560','UA_UCS-2000 / Lat_Lon_h','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5562','UA_UCS-2000 / GK_6','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5563','UA_UCS-2000 / GK_6','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5564','UA_UCS-2000 / GK_6','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5565','UA_UCS-2000 / GK_6','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6381','UA_UCS-2000 / UA_TM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6382','UA_UCS-2000 / UA_TM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6383','UA_UCS-2000 / UA_TM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6384','UA_UCS-2000 / UA_TM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6385','UA_UCS-2000 / UA_TM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6386','UA_UCS-2000 / UA_TM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','6387','UA_UCS-2000 / UA_TM','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9307','785','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9307','ATRF2014 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9308','786','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9308','ATRF2014 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9309','787','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9309','ATRF2014 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','9458','788','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','9458','AVWS - NHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9753','796','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9753','WGS 84 (G2139) - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9754','797','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9754','WGS 84 (G2139) - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9755','798','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9755','WGS 84 (G2139) - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5705','LV_KRON / NH','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5705','UA_KRON / NH','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5793','SHOM 1953 height','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4181','Luxembourg 1930','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2169','LUREF / Gauss Luxembourg','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','9923','GNTRANS2003 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','9932','DB_REF / 3-degree Gauss-Kruger zone 2 (E-N) + GNTRANS2016 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','9933','DB_REF / 3-degree Gauss-Kruger zone 3 (E-N) + GNTRANS2016 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','9934','DB_REF / 3-degree Gauss-Kruger zone 4 (E-N) + GNTRANS2016 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','9935','DB_REF / 3-degree Gauss-Kruger zone 5 (E-N) + GNTRANS2016 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','9928','DB_REF / 3-degree Gauss-Kruger zone 2 (E-N) + GNTRANS2003 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','9929','DB_REF / 3-degree Gauss-Kruger zone 3 (E-N) + GNTRANS2003 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','9930','DB_REF / 3-degree Gauss-Kruger zone 4 (E-N) + GNTRANS2003 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','9931','DB_REF / 3-degree Gauss-Kruger zone 5 (E-N) + GNTRANS2003 height','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','5681','DB_REF2003','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','5681','DB_REF2016','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','5830','DB_REF2003','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','5830','DB_REF2016','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','5828','DB_REF2003','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','5828','DB_REF2016','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','9920','British National Grid + BI height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','9922','IRENET95 / ITM + BI height','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9943','ETRS89 / EBBWV14 SnakeGrid','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','9944','ETRS89 / EBBWV14 SnakeGrid + Newlyn height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','6190','BD72 / Belgian Lambert 72 + Oostende height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','9907','ETRS89 + Oostende height','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','32159','NAD83 / WyLam','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9967','ETRS89 / HULLEE13 SnakeGrid','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','9968','ETRS89 / HULLEE13 SnakeGrid + Newlyn height','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9972','ETRS89 / SCM22 SnakeGrid','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','9973','ETRS89 / SCM22 SnakeGrid + Newlyn height','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9977','ETRS89 / FNL22 SnakeGrid','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','9978','ETRS89 / FNL22 SnakeGrid + Newlyn height','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9945','MGI 1901 / Macedonia GK truncated','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9945','MGI 1901 / Macedonia Gauss-Kruger truncated','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9945','MSCS truncated','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8082','NAD83(CSRS)v6 / MTM Nova Scotia zone 4','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8082','NAD83(CSRS) 2010 / MTM NS zone 4','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8083','NAD83(CSRS)v6 / MTM Nova Scotia zone 5','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8083','NAD83(CSRS) 2010 / MTM NS zone 5','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22351','NAD83(CSRS) 1997 / MTM zone 11','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22352','NAD83(CSRS) 1997 / MTM zone 12','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22353','NAD83(CSRS) 1997 / MTM zone 13','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22354','NAD83(CSRS) 1997 / MTM zone 14','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22355','NAD83(CSRS) 1997 / MTM zone 15','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22356','NAD83(CSRS) 1997 / MTM zone 16','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22357','NAD83(CSRS) 1997 / MTM zone 17','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22651','NAD83(CSRS) 2010 / MTM zone 11','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22652','NAD83(CSRS) 2010 / MTM zone 12','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22653','NAD83(CSRS) 2010 / MTM zone 13','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22654','NAD83(CSRS) 2010 / MTM zone 14','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22655','NAD83(CSRS) 2010 / MTM zone 15','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22656','NAD83(CSRS) 2010 / MTM zone 16','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22657','NAD83(CSRS) 2010 / MTM zone 17','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22462','NAD83(CSRS) 2002 / Alberta 3TM ref merid 111','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22463','NAD83(CSRS) 2002 / Alberta 3TM ref merid 114','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22464','NAD83(CSRS) 2002 / Alberta 3TM ref merid 117','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22465','NAD83(CSRS) 2002 / Alberta 3TM ref merid 120','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22762','NAD83(CSRS) 2010 / Alberta 3TM ref merid 111','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22763','NAD83(CSRS) 2010 / Alberta 3TM ref merid 114','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22764','NAD83(CSRS) 2010 / Alberta 3TM ref merid 117','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22765','NAD83(CSRS) 2010 / Alberta 3TM ref merid 120','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22641','NAD83(CSRS) 2010 / MTM zone 1','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22642','NAD83(CSRS) 2010 / MTM zone 2','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22643','NAD83(CSRS) 2010 / MTM zone 3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22644','NAD83(CSRS) 2010 / MTM zone 4','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22645','NAD83(CSRS) 2010 / MTM zone 5','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22646','NAD83(CSRS) 2010 / MTM zone 6','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22739','NAD83(CSRS) 2010 / PEI Stereographic','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22648','NAD83(CSRS) 2010 / MTM zone 8','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22649','NAD83(CSRS) 2010 / MTM zone 9','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22650','NAD83(CSRS) 2010 / MTM zone 10','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22307','NAD83(CSRS) 1997 / UTM zone 7N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22607','NAD83(CSRS) 2010 / UTM zone 7N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22707','NAD83(CSRS) 2010 / UTM zone 7N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22407','NAD83(CSRS) 2002 / UTM zone 7N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22409','NAD83(CSRS) 2002 / UTM zone 9N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22609','NAD83(CSRS) 2010 / UTM zone 9N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22709','NAD83(CSRS) 2010 / UTM zone 9N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22309','NAD83(CSRS) 1997 / UTM zone 9N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22311','NAD83(CSRS) 1997 / UTM zone 11N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22411','NAD83(CSRS) 2002 / UTM zone 11N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22611','NAD83(CSRS) 2010 / UTM zone 11N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22711','NAD83(CSRS) 2010 / UTM zone 11N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22312','NAD83(CSRS) 1997 / UTM zone 12N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22412','NAD83(CSRS) 2002 / UTM zone 12N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22712','NAD83(CSRS) 2010 / UTM zone 12N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22612','NAD83(CSRS) 2010 / UTM zone 12N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22313','NAD83(CSRS) 1997 / UTM zone 13N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22413','NAD83(CSRS) 2002 / UTM zone 13N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22613','NAD83(CSRS) 2010 / UTM zone 13N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22713','NAD83(CSRS) 2010 / UTM zone 13N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22314','NAD83(CSRS) 1997 / UTM zone 14N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22414','NAD83(CSRS) 2002 / UTM zone 14N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22614','NAD83(CSRS) 2010 / UTM zone 14N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22714','NAD83(CSRS) 2010 / UTM zone 14N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22315','NAD83(CSRS) 1997 / UTM zone 15N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22615','NAD83(CSRS) 2010 / UTM zone 15N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22715','NAD83(CSRS) 2010 / UTM zone 15N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22317','NAD83(CSRS) 1997 / UTM zone 17N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22417','NAD83(CSRS) 2002 / UTM zone 17N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22617','NAD83(CSRS) 2010 / UTM zone 17N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22717','NAD83(CSRS) 2010 / UTM zone 17N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22318','NAD83(CSRS) 1997 / UTM zone 18N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22418','NAD83(CSRS) 2002 / UTM zone 18N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22618','NAD83(CSRS) 2010 / UTM zone 18N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22718','NAD83(CSRS) 2010 / UTM zone 18N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22415','NAD83(CSRS) 2002 / UTM zone 15N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22316','NAD83(CSRS) 1997 / UTM zone 16N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22416','NAD83(CSRS) 2002 / UTM zone 16N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22616','NAD83(CSRS) 2010 / UTM zone 16N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22716','NAD83(CSRS) 2010 / UTM zone 16N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22308','NAD83(CSRS) 1997 / UTM zone 8N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22408','NAD83(CSRS) 2002 / UTM zone 8N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22608','NAD83(CSRS) 2010 / UTM zone 8N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22708','NAD83(CSRS) 2010 / UTM zone 8N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22310','NAD83(CSRS) 1997 / UTM zone 10N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22410','NAD83(CSRS) 2002 / UTM zone 10N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22610','NAD83(CSRS) 2010 / UTM zone 10N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22710','NAD83(CSRS) 2010 / UTM zone 10N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22319','NAD83(CSRS) 1997 / UTM zone 19N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22419','NAD83(CSRS) 2002 / UTM zone 19N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22619','NAD83(CSRS) 2010 / UTM zone 19N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22719','NAD83(CSRS) 2010 / UTM zone 19N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22320','NAD83(CSRS) 1997 / UTM zone 20N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22420','NAD83(CSRS) 2002 / UTM zone 20N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22620','NAD83(CSRS) 2010 / UTM zone 20N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22720','NAD83(CSRS) 2010 / UTM zone 20N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22321','NAD83(CSRS) 1997 / UTM zone 21N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22421','NAD83(CSRS) 2002 / UTM zone 21N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22621','NAD83(CSRS) 2010 / UTM zone 21N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22721','NAD83(CSRS) 2010 / UTM zone 21N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22322','NAD83(CSRS) 1997 / UTM zone 22N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22422','NAD83(CSRS) 2002 / UTM zone 22N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22622','NAD83(CSRS) 2010 / UTM zone 22N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22722','NAD83(CSRS) 2010 / UTM zone 22N','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','20000','SVD2006Height','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22219','NAD83(CSRS) 1997 / UTM zone 19N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22219','NAD83(CSRS98) / UTM zone 19N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22220','NAD83(CSRS) 1997 / UTM zone 20N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22220','NAD83(CSRS98) / UTM zone 20N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22221','NAD83(CSRS) 1997 / UTM zone 21N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22221','NAD83(CSRS98) / UTM zone 21N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22222','NAD83(CSRS) 1997 / UTM zone 22N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22222','NAD83(CSRS98) / UTM zone 22N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22213','NAD83(CSRS) 1997 / UTM zone 13N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22213','NAD83 / UTM extended zone 13','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22213','NAD83(CSRS98) / UTM zone 13N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22214','NAD83(CSRS) 1997 / UTM zone 14N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22214','NAD83(CSRS98) / UTM zone 14N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22215','NAD83(CSRS) 1997 / UTM zone 15N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22215','NAD83(CSRS98) / UTM zone 15N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22216','NAD83(CSRS) 1997 / UTM zone 16N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22216','NAD83(CSRS98) / UTM zone 16N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22217','NAD83(CSRS) 1997 / UTM zone 17N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22217','NAD83(CSRS98) / UTM zone 17N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22218','NAD83(CSRS) 1997 / UTM zone 18N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22218','NAD83(CSRS98) / UTM zone 18N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22207','NAD83(CSRS) 1997 / UTM zone 7N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22207','NAD83(CSRS98) / UTM zone 7N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22208','NAD83(CSRS) 1997 / UTM zone 8N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22208','NAD83(CSRS98) / UTM zone 8N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22209','NAD83(CSRS) 1997 / UTM zone 9N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22209','NAD83(CSRS98) / UTM zone 9N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22210','NAD83(CSRS) 1997 / UTM zone 10N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22210','NAD83(CSRS98) / UTM zone 10N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22211','NAD83(CSRS) 1997 / UTM zone 11N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22211','NAD83(CSRS98) / UTM zone 11N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22212','NAD83(CSRS) 1997 / UTM zone 12N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22212','NAD83(CSRS98) / UTM zone 12N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22262','NAD83(CSRS) 1997 / Alberta 3TM ref merid 111','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22262','NAD83(CSRS98) / Alberta 3TM ref merid 111','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22263','NAD83(CSRS) 1997 / Alberta 3TM ref merid 114','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22263','NAD83(CSRS98) / Alberta 3TM ref merid 114','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22264','NAD83(CSRS) 1997 / Alberta 3TM ref merid 117','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22264','NAD83(CSRS98) / Alberta 3TM ref merid 117','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22265','NAD83(CSRS) 1997 / Alberta 3TM ref merid 120','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22265','NAD83(CSRS98) / Alberta 3TM ref merid 120','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22337','NAD83(CSRS)v3 / MTM Nova Scotia zone 5','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22337','NAD83(CSRS) 1997 / MTM NS zone 5','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22338','NAD83(CSRS)v3 / MTM Nova Scotia zone 4','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22338','NAD83(CSRS) 1997 / MTM NS zone 4','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22239','NAD83(CSRS) 1997 / PEI Stereographic','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22239','NAD83(CSRS98) / PEI Stereographic','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22240','NAD83(CSRS) 1997 / NB Stereographic','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22240','NAD83(CSRS98) / NB Stereographic','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22243','NAD83(CSRS) 1997 / MTM zone 3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22243','NAD83(CSRS98) / SCoPQ zone 3','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22244','NAD83(CSRS) 1997 / MTM zone 4','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22244','NAD83(CSRS98) / SCoPQ zone 4','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22245','NAD83(CSRS) 1997 / MTM zone 5','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22245','NAD83(CSRS98) / SCoPQ zone 5','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22246','NAD83(CSRS) 1997 / MTM zone 6','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22246','NAD83(CSRS98) / SCoPQ zone 6','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22247','NAD83(CSRS) 1997 / MTM zone 7','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22247','NAD83(CSRS98) / SCoPQ zone 7','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22639','NAD83(CSRS) 2010 / PEI Stereographic','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22348','NAD83(CSRS) 1997 / MTM zone 8','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22349','NAD83(CSRS) 1997 / MTM zone 9','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22350','NAD83(CSRS) 1997 / MTM zone 10','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22248','NAD83(CSRS) 1997 / MTM zone 8','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22248','NAD83(CSRS98) / SCoPQ zone 8','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22249','NAD83(CSRS) 1997 / MTM zone 9','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22249','NAD83(CSRS98) / SCoPQ zone 9','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22250','NAD83(CSRS) 1997 / MTM zone 10','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22250','NAD83(CSRS98) / SCoPQ zone 10','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','9715','NAD83(CSRS) / UTM zone 15N + CGVD2013a height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','6649','NAD83(CSRS) + CGVD2013 height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','9245','CGVD2013a height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','9245','335','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','9245','CGVD2013(CGG2013a) Epoch 2010 - OHt','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','20037','NAD83(CSRS) 2002 + CGVD2013(CGG2013a) epoch 2002 height','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9988','802','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9988','ITRF2020 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9989','803','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9989','ITRF2020 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9990','804','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','9990','ITRF2020 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','9245','CGVD2013(CGG2013a) height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','9245','CGVD2013(CGG2013a) epoch 2010','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','9544','NAD83(CSRS)v6 + CGVD2013(CGG2013a) height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','20034','CGVD2013(CGG2013a) epoch 2002','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','20035','CGVD2013(CGG2013a) epoch 1997','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','20038','NAD83(CSRS) 1997 + CGVD2013(CGG2013a) epoch 1997 height','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20002','ETRS89 / MWC18 SnakeGrid','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','20003','ETRS89 / MWC18 SnakeGrid + Newlyn height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5721','IGN78C','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','9471','INAGeoid2020 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','9529','SRGI2013 + INAGeoid2020 height','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9356','955','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9356','KSA-GRF17 / UTM zone 36N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9357','956','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9357','KSA-GRF17 / UTM zone 37N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9358','957','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9358','KSA-GRF17 / UTM zone 38N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9359','958','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9359','KSA-GRF17 / UTM zone 39N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9360','959','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9360','KSA-GRF17 / UTM zone 40N','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','20035','965','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','20035','CGVD2013(CGG2013a) epoch 1997 - OHt','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','20034','966','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','20034','CGVD2013(CGG2013a) epoch 2002 - OHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','20039','SIRGAS-Chile epoch 2021','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','20039','SIRGAS-Chile realización 5','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','20040','SIRGAS-Chile realización 5','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','20040','SIRGAS-Chile epoch 2021.00','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','20041','SIRGAS-Chile realización 5','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','20041','SIRGAS-Chile epoch 2021.00','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20050','NAD83(2011) / Amtrak CS21 (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','20050','ACS21 (ft)','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5872','Approximate HAT height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5874','Approximate High Water height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5869','Approximate MHHW height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5868','Approximate MHW height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5870','Approximate MHWS height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5862','Approximate LLWLT depth','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5873','Approximate Low Water depth','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5863','Approximate ISLW depth','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5871','Approximate HHWLT height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5866','Approximate MLLW depth','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5864','Approximate MLLWS depth','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5867','Approximate MLW depth','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5865','Approximate MLWS depth','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5861','Approximate LAT depth','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5715','Approximate MSL depth','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','8053','Approximate MSL depth (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5714','Approximate MSL height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','8050','Approximate MSL height (ft)','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','8052','Approximate MSL height (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','8051','Approximate MSL depth (ft)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10183','ETRS89 / DoPw22 SnakeGrid','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10188','ETRS89 / ShAb07 SnakeGrid','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','10189','ETRS89 / ShAb07 SnakeGrid + Newlyn height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','10184','ETRS89 / DoPw22 SnakeGrid + Newlyn height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','10195','ETRS89 / CNH22 SnakeGrid + Newlyn height','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4479','CGCS 2000 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','10200','ETRS89 / CWS13 SnakeGrid + Newlyn height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','10208','ETRS89 / DIBA15 SnakeGrid + Newlyn height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','10213','ETRS89 / GWPBS22 SnakeGrid + Newlyn height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','10223','ETRS89 / GWWWA22 SnakeGrid + Newlyn height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','10228','ETRS89 / MALS09 SnakeGrid + Newlyn height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','10236','ETRS89 / OxWo08 SnakeGrid + Newlyn height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','10241','ETRS89 / SYC20 SnakeGrid + Newlyn height','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10194','ETRS89 / CNH22 SnakeGrid','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10199','ETRS89 / CWS13 SnakeGrid','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10207','ETRS89 / DIBA15 SnakeGrid','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10207','DIDBAN15 Grid','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','10218','ETRS89 / GWWAB22 SnakeGrid + Newlyn height','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10227','ETRS89 / MALS09 SnakeGrid','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10235','ETRS89 / OxWo08 SnakeGrid','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10240','ETRS89 / SYC20 SnakeGrid','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10212','ETRS89 / GWPBS22 SnakeGrid','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10217','ETRS89 / GWWAB22 SnakeGrid','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','9377','MAGNA-SIRGAS 2018 / CTM12','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10222','ETRS89 / GWWWA22 SnakeGrid','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','10246','D96/TM + SVS2010/Koper','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','10245','D96 + SVS2010/Koper','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','11114','MAGNA-SIRGAS 2018 / Col FW','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','11115','MAGNA-SIRGAS 2018 / Col W','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','11117','MAGNA-SIRGAS 2018 / Col EC','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','11118','MAGNA-SIRGAS 2018 / Col E','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','11116','MAGNA-SIRGAS 2018 / Col Bog','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10275','ETRS89 / SMITB20 SnakeGrid','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','10276','ETRS89 / SMITB20 SnakeGrid + Newlyn height','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10280','ETRS89 / RBEPP12 SnakeGrid','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','10281','ETRS89 / RBEPP12 SnakeGrid + Newlyn height','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','10284','ETRS89/DREF91/R16','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10286','ETRS89/DREF91/R16 / UTM 31(N-zE)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10287','ETRS89/DREF91/R16 / UTM 31(zE-N)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10288','ETRS89/DREF91/R16 / UTM 32(N-zE)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10289','ETRS89/DREF91/R16 / UTM 32(zE-N)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10290','ETRS89/DREF91/R16 / UTM 33(N-zE)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10291','ETRS89/DREF91/R16 / UTM 33(zE-N)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','10298','REPANGOL','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4948','LKS92','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4949','LKS92','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4661','LKS92','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3059','LKS-92 TM','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10306','LKS-2020 TM','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','10176','980','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','10176','IGS20 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','10177','981','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','10177','IGS20 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','10178','982','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','10178','IGS20 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10285','ETRS89/DREF91/R16 / 3GK zn3','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4749','RGNC91-93 (lat-lon)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4907','RGNC91-93 (lat-lon)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','10309','RGNC15 (lat-lon)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','10310','RGNC15 (lat-lon)','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','9351','NGNC height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','9540','RGNC91-93 + NGNC height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','10318','RGNC15 (lon-lat) + NGNC height','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10314','Lambert NC15','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10315','RGNC / UTM zone 57S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10316','RGNC / UTM zone 58S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10160','System 34 Jylland, østvendt','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10250','System 34 Sjælland, østvendt','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10254','System 45 Bornholm, østvendt','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10262','Generalstabens System Bornholm, østvendt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','10268','OS-IRF','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10258','Generalstabens System, østvendt','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10266','Københavns Kommunes System, østvendt','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','10352','REDNAP height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','10353','REDNAP height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','10354','REDNAP height','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4926','Korea 2000','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4927','Korea 2000','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4737','Korea 2000','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5180','Korea 2000 / West Belt','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5181','Korea 2000 / Central Belt','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5182','Korea 2000 / Central Belt Jeju','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5183','Korea 2000 / East Belt','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5179','Korea 2000 / Unified CS','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5193','Incheon height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','6178','Cais da Pontinha - Funchal height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','6179','Cais da Vila - Porto Santo height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','10349','CD Portugal depth','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','10412','993','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','10412','NAD83(CSRS) v8 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','10412','NAD83(CSRS) 2010','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','10413','992','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','10413','NAD83(CSRS) v8 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','10413','NAD83(CSRS) 2010','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','10414','994','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','10414','NAD83(CSRS) v8 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','10414','NAD83(CSRS) 2010','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22807','NAD83(CSRS) 2010 / UTM zone 7N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22808','NAD83(CSRS) 2010 / UTM zone 8N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22809','NAD83(CSRS) 2010 / UTM zone 9N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22810','NAD83(CSRS) 2010 / UTM zone 10N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22811','NAD83(CSRS) 2010 / UTM zone 11N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22812','NAD83(CSRS) 2010 / UTM zone 12N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22813','NAD83(CSRS) 2010 / UTM zone 13N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22814','NAD83(CSRS) 2010 / UTM zone 14N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22815','NAD83(CSRS) 2010 / UTM zone 15N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22816','NAD83(CSRS) 2010 / UTM zone 16N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22817','NAD83(CSRS) 2010 / UTM zone 17N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22818','NAD83(CSRS) 2010 / UTM zone 18N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22819','NAD83(CSRS) 2010 / UTM zone 19N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22820','NAD83(CSRS) 2010 / UTM zone 20N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22821','NAD83(CSRS) 2010 / UTM zone 21N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','22822','NAD83(CSRS) 2010 / UTM zone 22N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10471','ETRS89 / COV23 SnakeGrid','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','10472','ETRS89 / COV23 SnakeGrid + Newlyn height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5799','DVR90 ensemble height','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4162','1011','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4162','Korean 1985 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4926','1008','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4926','KGD2002 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4927','1007','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4927','KGD2002 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4737','1009','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4737','KGD2002 - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5193','1010','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5193','KVD1964 - NOHt','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10731','ETRS89/DREF91/R16 / UTM 31','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10733','ETRS89/DREF91/R16 / UTM 33','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10516','NAD83(2011) / WISCRS Adjusted Jackson (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10732','ETRS89/DREF91/R16 / UTM 32','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','10551','DKMSL ensemble depth','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','10552','DKLAT ensemble depth','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','7400','NTF (Paris) + NGF IGN69 height','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3844','Stereografic 1970','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10622','NAD83(2011) / SFO B18 grid (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','10627','ETRS89 / ECML14 SnakeGrid + Newlyn height','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10626','ETRS89 / ECML14 SnakeGrid','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10632','ETRS89 / WC05 SnakeGrid','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','10659','ETRS89 + EOMA 1980 height','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10665','Porto Alegre Cartographic Reference System (SCR-POA)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4801','Bern 1898 (Bern)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','21780','Bern 1898 (Bern) / LV03C','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','10588','CGVD28(HT2)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5682','DB_REF2016 / 3GK zone 2 E-N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5682','DB_REF2003 / 3-degree Gauss-Kruger zone 2 (E-N)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5683','DB_REF2016 / 3GK zone 3 E-N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5683','DB_REF2003 / 3-degree Gauss-Kruger zone 3 (E-N)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5683','DB_REF2016 / 3-degree Gauss-Kruger zone 3 (E-N)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5684','DB_REF2016 / 3GK zone 4 E-N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5684','DB_REF2003 / 3-degree Gauss-Kruger zone 4 (E-N)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5684','DB_REF2016 / 3-degree Gauss-Kruger zone 4 (E-N)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5685','DB_REF2016 / 3GK zone 5 E-N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5685','DB_REF2003 / 3-degree Gauss-Kruger zone 5 (E-N)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5685','DB_REF2016 / 3-degree Gauss-Kruger zone 5 (E-N)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5682','DB_REF2016 / 3-degree Gauss-Kruger zone 2 (E-N)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','10671','RGM23 (lat-lon)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','10670','RGM23 (lat-lon)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','10637','BES2020','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','10638','BES2020','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','10639','BES2020','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10641','Saba / Saba TM 2020','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','10645','Saba / Saba TM 2020 + Saba height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5779','SVS2000/Trst ','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','8690','SVS2010/Koper','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','10686','D96 + SVS2000/Trst','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','10687','D96/TM + SVS2000/Trst','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','3903','ETRS89 / TM35FIN(N,E) + N2000 height','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3126','ETRS89 / ETRS-GK19FIN','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3127','ETRS89 / ETRS-GK20FIN','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3128','ETRS89 / ETRS-GK21FIN','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3129','ETRS89 / ETRS-GK22FIN','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3130','ETRS89 / ETRS-GK23FIN','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3131','ETRS89 / ETRS-GK24FIN','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3132','ETRS89 / ETRS-GK25FIN','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3133','ETRS89 / ETRS-GK26FIN','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3134','ETRS89 / ETRS-GK27FIN','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3135','ETRS89 / ETRS-GK28FIN','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3136','ETRS89 / ETRS-GK29FIN','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3137','ETRS89 / ETRS-GK30FIN','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3138','ETRS89 / ETRS-GK31FIN','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','10688','EUREF-FIN-XYZ','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','10689','EUREF-FIN-GRS80h','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','10691','ETRS89 + N60 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','10692','ETRS89 + N2000 height','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3067','EUREF-FIN / UTM zone 35N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','3067','ETRS89 / TM35FIN(E,N)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10699','ETRS-TM34','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10699','ETRS89 / UTM zone 34N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10702','ETRS-TM36','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10702','ETRS89 / UTM zone 36N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10759','Bonaire / Bonaire TM','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','10763','Bonaire KADpeil','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','10690','EUREF-FIN-GRS80','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','10774','ETRS-TM35FIN/N2000','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','10737','BES2020','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','10738','BES2020','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','10739','BES2020','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','10746','Sint Eustatius / Sint Eustatius TM 2020 + Sint Eustatius height','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10745','Sint Eustatius / UTM zone 20N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10744','Sint Eustatius / Sint Eustatius TM 2020','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','10747','Sint Eustatius / UTM 20N + Sint Eustatius height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','10764','Bonaire DPnet + Bonaire KADpeil','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','10764','Bonaire / Bonaire TM + Bonaire height','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10773','SIRGAS 2000 / RPTM','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','10765','Bonaire 2004 + Bonaire KADpeil','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','10789','Uganda 2010','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','10790','Uganda 2010','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','10791','Uganda 2010','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10792','Uganda 2010 / UTM zone 35N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10793','Uganda 2010 / UTM zone 36N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10794','Uganda 2010 / UTM zone 36S','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10795','Uganda 2010 / UTM zone 35S','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','10688','ETRS89','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','10689','ETRS89','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','10690','ETRS89','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','10604','1255','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','10604','WGS 84 (G2296) - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','10605','1256','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','10605','WGS 84 (G2296) - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','10606','1257','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','10606','WGS 84 (G2296) - LatLon','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','10569','1240','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','10569','ETRF2020 - XYZ','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','10570','1239','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','10570','ETRF2020 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','10571','1238','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','10571','ETRF2020 - LatLonEHt','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','10826','ETRS89 + LAS-2000 height','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','25884','LKS-2020 / TM Baltic93','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','10839','ETRS89 + LAS-2000 height','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10851','ETRS89 / EWR3 SnakeGrid','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','10852','ETRS89 / EWR3 SnakeGrid + Newlyn height','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','10829','GGD','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','10830','GGD','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','10831','GGD','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10833','GGD / Lambert','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10836','GGD / TM37','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10837','GGD / TM38','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10863','ETRS89 / WSPG SnakeGrid','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','10864','ETRS89 / WSPG SnakeGrid + Newlyn height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','10588','1265','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','10588','CGVD28(HTv2.0) - OHt','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10899','Asse 2025 / Gauss-Kruger zone 4','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','10906','Asse 2025 / 3-degree Gauss-Kruger zone 4 (E-N) + Asse 2025 height','EPSG');
INSERT INTO "alias_name" VALUES('vertical_crs','EPSG','5705','QazVRF','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','4194','Qornoq 1927','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2216','Qornoq 1927 / UTM zone 22N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2217','Qornoq 1927 / UTM zone 23N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2299','Qornoq 1927 / Greenland zone 2 west','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2301','Qornoq 1927 / Greenland zone 3 west','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2303','Qornoq 1927 / Greenland zone 4 west','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2304','Qornoq 1927 / Greenland zone 5 west','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2305','Qornoq 1927 / Greenland zone 6 west','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','2306','Qornoq 1927 / Greenland zone 7 west','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10942','QazTRF23 / Gauss-Kruger zone 8','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10943','QazTRF23 / Gauss-Kruger zone 9','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10944','QazTRF23 / Gauss-Kruger zone 10','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10945','QazTRF23 / Gauss-Kruger zone 11','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10946','QazTRF23 / Gauss-Kruger zone 12','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10947','QazTRF23 / Gauss-Kruger zone 13','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10948','QazTRF23 / Gauss-Kruger zone 14','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10949','QazTRF23 / Gauss-Kruger zone 15','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','10952','CSRN2025 (ITRF2020)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','10951','CSRN2025 (ITRF2020)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','10950','CSRN2025 (ITRF2020)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10926','CSRN2025 (NAD83 2011) / California zone 6 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10925','CSRN2025 (NAD83 2011) / California zone 5 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10914','CSRN2025 (NAD83 2011) / CA 4 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10914','NAD83(CSRN2025) / CCS83 zone 4 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10913','CSRN2025 (NAD83 2011) / CA 3 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10913','NAD83(CSRN2025) / CCS83 zone 3 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10912','CSRN2025 (NAD83 2011) / CA 2 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10912','NAD83(CSRN2025) / CCS83 zone 2 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10911','CSRN2025 (NAD83 2011) / CA 1 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10911','NAD83(CSRN2025) / CCS83 zone 1 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','10910','CSRN2025 (NAD83 2011)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','10909','CSRN2025 (NAD83 2011)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','10908','CSRN2025 (NAD83 2011)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10915','CSRN2025 (NAD83 2011) / CA 5 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10915','NAD83(CSRN2025) / CCS83 zone 5 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10916','CSRN2025 (NAD83 2011) / CA 6 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10916','NAD83(CSRN2025) / CCS83 zone 6 (ftUS)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10917','CSRN2025 (NAD83 2011) / CA Albers','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10921','CSRN2025 (NAD83 2011) / CA zone 1 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10922','CSRN2025 (NAD83 2011) / CA zone 2 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10923','CSRN2025 (NAD83 2011) / CA zone 3 (m)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','10924','CSRN2025 (NAD83 2011) / California zone 4 (m)','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','10873','EUREF89','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','10874','EUREF89','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','10875','EUREF89','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5105','ETRS89 / NTM zone 5','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5106','ETRS89 / NTM zone 6','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5107','ETRS89 / NTM zone 7','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5108','ETRS89 / NTM zone 8','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5109','ETRS89 / NTM zone 9','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5110','ETRS89 / NTM zone 10','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5111','ETRS89 / NTM zone 11','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5112','ETRS89 / NTM zone 12','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5113','ETRS89 / NTM zone 13','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5114','ETRS89 / NTM zone 14','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5115','ETRS89 / NTM zone 15','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5116','ETRS89 / NTM zone 16','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5117','ETRS89 / NTM zone 17','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5118','ETRS89 / NTM zone 18','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5119','ETRS89 / NTM zone 19','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5120','ETRS89 / NTM zone 20','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5121','ETRS89 / NTM zone 21','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5122','ETRS89 / NTM zone 22','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5123','ETRS89 / NTM zone 23','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5124','ETRS89 / NTM zone 24','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5125','ETRS89 / NTM zone 25','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5126','ETRS89 / NTM zone 26','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5127','ETRS89 / NTM zone 27','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5128','ETRS89 / NTM zone 28','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5129','ETRS89 / NTM zone 29','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5130','ETRS89 / NTM zone 30','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','6144','ETRS89 + NN54 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','5942','ETRS89 + NN2000 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','20001','ETRS89 + SVD2006 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','9883','ETRS89 + CD Norway depth','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5105','EUREF89 / NTM zone 5','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5106','EUREF89 / NTM zone 6','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5107','EUREF89 / NTM zone 7','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5108','EUREF89 / NTM zone 8','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5109','EUREF89 / NTM zone 9','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5110','EUREF89 / NTM zone 10','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5111','EUREF89 / NTM zone 11','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5112','EUREF89 / NTM zone 12','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5113','EUREF89 / NTM zone 13','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5114','EUREF89 / NTM zone 14','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5115','EUREF89 / NTM zone 15','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5116','EUREF89 / NTM zone 16','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5117','EUREF89 / NTM zone 17','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5118','EUREF89 / NTM zone 18','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5119','EUREF89 / NTM zone 19','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5120','EUREF89 / NTM zone 20','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5121','EUREF89 / NTM zone 21','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5122','EUREF89 / NTM zone 22','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5123','EUREF89 / NTM zone 23','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5124','EUREF89 / NTM zone 24','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5125','EUREF89 / NTM zone 25','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5126','EUREF89 / NTM zone 26','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5127','EUREF89 / NTM zone 27','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5128','EUREF89 / NTM zone 28','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5129','EUREF89 / NTM zone 29','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','5130','EUREF89 / NTM zone 30','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','11012','EUREF89 / UTM zone 30 (N-E)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','11013','EUREF89 / UTM zone 31 (N-E)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','11014','EUREF89 / UTM zone 32 (N-E)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','11015','EUREF89 / UTM zone 33 (N-E)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','11016','EUREF89 / UTM zone 34 (N-E)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','11017','EUREF89 / UTM zone 35 (N-E)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','11018','EUREF89 / UTM zone 36 (N-E)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','11019','EUREF89 / UTM zone 37 (N-E)','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','11020','EUREF89 / UTM zone 30','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','11021','EUREF89 / UTM zone 31','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','11022','EUREF89 / UTM zone 32','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','11023','EUREF89 / UTM zone 33','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','11024','EUREF89 / UTM zone 34','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','11025','EUREF89 / UTM zone 35','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','11026','EUREF89 / UTM zone 36','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','20001','EUREF89 + SVD2006 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','11000','EUREF89 + SVD2024 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','6145','ETRS89 / NTM zone 5 + NN54 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','6145','EUREF89 / NTM zone 5 + NN54','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','5945','ETRS89 / NTM zone 5 + NN2000 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','5945','EUREF89 / NTM zone 5 + NN2000','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','6146','ETRS89 / NTM zone 6 + NN54 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','5946','ETRS89 / NTM zone 6 + NN2000 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','6147','ETRS89 / NTM zone 7 + NN54 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','6146','EUREF89 / NTM zone 6 + NN54 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','5946','EUREF89 / NTM zone 6 + NN2000 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','5947','ETRS89 / NTM zone 7 + NN2000 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','5947','EUREF89 / NTM zone 7 + NN2000 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','6148','ETRS89 / NTM zone 8 + NN54 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','5948','ETRS89 / NTM zone 8 + NN2000 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','5948','EUREF89 / NTM zone 8 + NN2000 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','6149','ETRS89 / NTM zone 9 + NN54 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','6149','EUREF89 / NTM zone 9 + NN54 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','5949','ETRS89 / NTM zone 9 + NN2000 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','5949','EUREF89 / NTM zone 9 + NN2000 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','6150','ETRS89 / NTM zone 10 + NN54 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','6150','EUREF89 / NTM zone 10 + NN54 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','5950','ETRS89 / NTM zone 10 + NN2000 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','5950','EUREF89 / NTM zone 10 + NN2000 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','6151','ETRS89 / NTM zone 11 + NN54 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','6151','EUREF89 / NTM zone 11 + NN54 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','5951','ETRS89 / NTM zone 11 + NN2000 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','5951','EUREF89 / NTM zone 11 + NN2000 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','6152','ETRS89 / NTM zone 12 + NN54 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','6152','EUREF89 / NTM zone 12 + NN54 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','5952','ETRS89 / NTM zone 12 + NN2000 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','5952','EUREF89 / NTM zone 12 + NN2000 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','6153','ETRS89 / NTM zone 13 + NN54 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','6153','EUREF89 / NTM zone 13 + NN54 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','5953','ETRS89 / NTM zone 13 + NN2000 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','5953','EUREF89 / NTM zone 13 + NN2000 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','6154','ETRS89 / NTM zone 14 + NN54 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','6154','EUREF89 / NTM zone 14 + NN54 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','5954','ETRS89 / NTM zone 14 + NN2000 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','5954','EUREF89 / NTM zone 14 + NN2000 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','6155','ETRS89 / NTM zone 15 + NN54 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','6155','EUREF89 / NTM zone 15 + NN54 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','5955','ETRS89 / NTM zone 15 + NN2000 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','5955','EUREF89 / NTM zone 15 + NN2000 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','6156','ETRS89 / NTM zone 16 + NN54 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','6156','EUREF89 / NTM zone 16 + NN54 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','5956','ETRS89 / NTM zone 16 + NN2000 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','5956','EUREF89 / NTM zone 16 + NN2000 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','6157','ETRS89 / NTM zone 17 + NN54 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','6157','EUREF89 / NTM zone 17 + NN54 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','5957','ETRS89 / NTM zone 17 + NN2000 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','5957','EUREF89 / NTM zone 17 + NN2000 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','6158','ETRS89 / NTM zone 18 + NN54 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','6158','EUREF89 / NTM zone 18 + NN54 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','5958','ETRS89 / NTM zone 18 + NN2000 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','6159','ETRS89 / NTM zone 19 + NN54 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','6159','EUREF89 / NTM zone 19 + NN54 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','5959','ETRS89 / NTM zone 19 + NN2000 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','5959','EUREF89 / NTM zone 19 + NN2000 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','6160','ETRS89 / NTM zone 20 + NN54 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','6160','EUREF89 / NTM zone 20 + NN54 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','5960','ETRS89 / NTM zone 20 + NN2000 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','5960','EUREF89 / NTM zone 20 + NN2000 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','6161','ETRS89 / NTM zone 21 + NN54 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','6161','EUREF89 / NTM zone 21 + NN54 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','5961','ETRS89 / NTM zone 21 + NN2000 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','5961','EUREF89 / NTM zone 21 + NN2000 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','6162','ETRS89 / NTM zone 22 + NN54 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','5962','ETRS89 / NTM zone 22 + NN2000 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','6163','ETRS89 / NTM zone 23 + NN54 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','6163','EUREF89 / NTM zone 23 + NN54 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','5963','ETRS89 / NTM zone 23 + NN2000 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','5963','EUREF89 / NTM zone 23 + NN2000 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','6164','ETRS89 / NTM zone 24 + NN54 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','6164','EUREF89 / NTM zone 24 + NN54 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','5964','ETRS89 / NTM zone 24 + NN2000 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','5964','EUREF89 / NTM zone 24 + NN2000 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','6165','ETRS89 / NTM zone 25 + NN54 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','6165','EUREF89 / NTM zone 25 + NN54 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','5965','ETRS89 / NTM zone 25 + NN2000 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','5965','EUREF89 / NTM zone 25 + NN2000 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','6166','ETRS89 / NTM zone 26 + NN54 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','6166','EUREF89 / NTM zone 26 + NN54 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','5966','ETRS89 / NTM zone 26 + NN2000 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','5966','EUREF89 / NTM zone 26 + NN2000 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','6167','ETRS89 / NTM zone 27 + NN54 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','6167','EUREF89 / NTM zone 27 + NN54 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','5967','ETRS89 / NTM zone 27 + NN2000 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','5967','EUREF89 / NTM zone 27 + NN2000 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','6168','ETRS89 / NTM zone 28 + NN54 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','6168','EUREF89 / NTM zone 28 + NN54 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','5968','ETRS89 / NTM zone 28 + NN2000 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','5968','EUREF89 / NTM zone 28 + NN2000 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','6169','ETRS89 / NTM zone 29 + NN54 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','6169','EUREF89 / NTM zone 29 + NN54 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','5969','ETRS89 / NTM zone 29 + NN2000 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','5969','EUREF89 / NTM zone 29 + NN2000 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','6170','ETRS89 / NTM zone 30 + NN54 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','6170','EUREF89 / NTM zone 30 + NN54 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','5970','ETRS89 / NTM zone 30 + NN2000 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','5970','EUREF89 / NTM zone 30 + NN2000 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','5958','EUREF89 / NTM zone 18 + NN2000 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','6162','EUREF89 / NTM zone 22 + NN54 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','5962','EUREF89 / NTM zone 22 + NN2000 height','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8905','CR-SIRGAS','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8906','CR-SIRGAS','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','8907','CR-SIRGAS','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8908','CR-SIRGAS / CRTM05','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','8912','CR-SIRGAS / CRTM05 + DACR52 height','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8909','CR-SIRGAS / UTM zone 16N','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','8910','CR-SIRGAS / UTM zone 17N','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','11041','CR19','EPSG');
INSERT INTO "alias_name" VALUES('geodetic_crs','EPSG','11042','CR19','EPSG');
INSERT INTO "alias_name" VALUES('projected_crs','EPSG','11027','EUREF89 / UTM zone 37','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','5971','EUREF89 / UTM zone 31 + NN2000 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','5972','EUREF89 / UTM zone 32 + NN2000 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','5973','EUREF89 / UTM zone 33 + NN2000 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','5974','EUREF89 / UTM zone 34 + NN2000 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','5975','EUREF89 / UTM zone 35 + NN2000 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','5976','EUREF89 / UTM zone 36 + NN2000 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','6171','EUREF89 / UTM zone 31 + NN54 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','6172','EUREF89 / UTM zone 32 + NN54 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','6173','EUREF89 / UTM zone 33 + NN54 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','6174','EUREF89 / UTM zone 34 + NN54 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','6175','EUREF89 / UTM zone 35 + NN54 height','EPSG');
INSERT INTO "alias_name" VALUES('compound_crs','EPSG','6176','EUREF89 / UTM zone 36 + NN54 height','EPSG');
