--- This file has been generated by scripts/build_nrcan.py. DO NOT EDIT !

INSERT INTO "grid_transformation" VALUES('NRCAN','HT2_1997_NAD83CSRSV7','NAD83(CSRS)v7 to CGVD28 height',NULL,'EPSG','1060','Geographic3D to GravityRelatedHeight (NRCan byn)','EPSG','8254','EPSG','5713',0.05,'EPSG','8666','Geoid (height correction) model file','HT2_1997.byn',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NRC-Can CGG2000',0);
INSERT INTO "usage" VALUES('NRCAN','USAGE_HT2_1997_NAD83CSRSV7','grid_transformation','NRCAN','HT2_1997_NAD83CSRSV7','EPSG','1289','EPSG','1133');
INSERT INTO "grid_transformation" VALUES('NRCAN','HT2_2002_NAD83CSRSV7','NAD83(CSRS)v7 to CGVD28 height',NULL,'EPSG','1060','Geographic3D to GravityRelatedHeight (NRCan byn)','EPSG','8254','EPSG','5713',0.05,'EPSG','8666','Geoid (height correction) model file','HT2_2002v70.byn',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NRC-Can CGG2000 2002',0);
INSERT INTO "usage" VALUES('NRCAN','USAGE_HT2_2002_NAD83CSRSV7','grid_transformation','NRCAN','HT2_2002_NAD83CSRSV7','EPSG','1289','EPSG','1133');
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_1997_MTM1_HT2_1997', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 1", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 1", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -53, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_1997", "id": {"authority": "NRCAN", "code": "HT2_1997_NAD83CSRSV7"}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / MTM zone 1 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Newfoundland - onshore east of 54\u00b030''W.", "bbox": {"south_latitude": 46.56, "west_longitude": -54.5, "north_latitude": 49.89, "east_longitude": -52.54}}', 1997.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_1997_MTM2_HT2_1997', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 2", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 2", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -56, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_1997", "id": {"authority": "NRCAN", "code": "HT2_1997_NAD83CSRSV7"}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / MTM zone 2 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Newfoundland and Labrador between 57\u00b030''W and 54\u00b030''W.", "bbox": {"south_latitude": 46.81, "west_longitude": -57.5, "north_latitude": 54.71, "east_longitude": -54.49}}', 1997.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_1997_MTM3_HT2_1997', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 3", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 3", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -58.5, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_1997", "id": {"authority": "NRCAN", "code": "HT2_1997_NAD83CSRSV7"}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / MTM zone 3 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Newfoundland west of 57\u00b030''W.", "bbox": {"south_latitude": 47.5, "west_longitude": -59.48, "north_latitude": 50.54, "east_longitude": -57.5}}', 1997.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_1997_MTM4_HT2_1997', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 4", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 4", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -61.5, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_1997", "id": {"authority": "NRCAN", "code": "HT2_1997_NAD83CSRSV7"}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / MTM zone 4 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Labrador between 63\u00b0W and 60\u00b0W.", "bbox": {"south_latitude": 52, "west_longitude": -63, "north_latitude": 58.92, "east_longitude": -60}}', 1997.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_1997_MTM5_HT2_1997', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 5", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 5", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -64.5, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_1997", "id": {"authority": "NRCAN", "code": "HT2_1997_NAD83CSRSV7"}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / MTM zone 5 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Labrador - 66\u00b0W to 63\u00b0W.", "bbox": {"south_latitude": 51.58, "west_longitude": -66, "north_latitude": 60.52, "east_longitude": -63}}', 1997.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_1997_MTM6_HT2_1997', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 6", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 6", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -67.5, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_1997", "id": {"authority": "NRCAN", "code": "HT2_1997_NAD83CSRSV7"}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / MTM zone 6 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Labrador - west of 66\u00b0W.", "bbox": {"south_latitude": 52.05, "west_longitude": -67.81, "north_latitude": 55.34, "east_longitude": -66}}', 1997.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_1997_MTM7_HT2_1997', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 7", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 7", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -70.5, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_1997", "id": {"authority": "NRCAN", "code": "HT2_1997_NAD83CSRSV7"}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / MTM zone 7 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Quebec - between 72\u00b0W and 69\u00b0W.", "bbox": {"south_latitude": 45.01, "west_longitude": -72, "north_latitude": 61.8, "east_longitude": -69}}', 1997.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_1997_MTM8_HT2_1997', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 8", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 8", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -73.5, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_1997", "id": {"authority": "NRCAN", "code": "HT2_1997_NAD83CSRSV7"}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / MTM zone 8 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Ontario - east of 75\u00b0W.", "bbox": {"south_latitude": 44.98, "west_longitude": -75, "north_latitude": 45.65, "east_longitude": -74.35}}', 1997.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_1997_MTM9_HT2_1997', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 9", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 9", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -76.5, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_1997", "id": {"authority": "NRCAN", "code": "HT2_1997_NAD83CSRSV7"}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / MTM zone 9 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Ontario - between 78\u00b0W and 75\u00b0W.", "bbox": {"south_latitude": 43.63, "west_longitude": -78, "north_latitude": 46.25, "east_longitude": -75}}', 1997.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_1997_MTM10_HT2_1997', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 10", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 10", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -79.5, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_1997", "id": {"authority": "NRCAN", "code": "HT2_1997_NAD83CSRSV7"}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / MTM zone 10 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Ontario - between 81\u00b0W and 78\u00b0W: south of 46\u00b0N in area to west of 80\u00b015''W, south of 47\u00b0N in area between 80\u00b015''W and 79\u00b030''W, entire province between 79\u00b030''W and 78\u00b0W.", "bbox": {"south_latitude": 42.26, "west_longitude": -81, "north_latitude": 47.33, "east_longitude": -77.99}}', 1997.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_1997_MTM11_HT2_1997', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 11", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 11", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -82.5, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_1997", "id": {"authority": "NRCAN", "code": "HT2_1997_NAD83CSRSV7"}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / MTM zone 11 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Ontario - south of 46\u00b0N and west of 81\u00b0W.", "bbox": {"south_latitude": 41.67, "west_longitude": -83.6, "north_latitude": 46, "east_longitude": -81}}', 1997.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_1997_MTM12_HT2_1997', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 12", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 12", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -81, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_1997", "id": {"authority": "NRCAN", "code": "HT2_1997_NAD83CSRSV7"}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / MTM zone 12 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Ontario - between 82\u00b030''W and 79\u00b030''W: north of 46\u00b0N in area between 82\u00b030''W and 80\u00b015''W, north of 47\u00b0N in area between 80\u00b015''W and 79\u00b030''W.", "bbox": {"south_latitude": 46, "west_longitude": -82.5, "north_latitude": 55.21, "east_longitude": -79.5}}', 1997.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_1997_MTM13_HT2_1997', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 13", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 13", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -84, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_1997", "id": {"authority": "NRCAN", "code": "HT2_1997_NAD83CSRSV7"}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / MTM zone 13 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Ontario - between 85\u00b030''W and 82\u00b030''W and north of 46\u00b0N.", "bbox": {"south_latitude": 46, "west_longitude": -85.5, "north_latitude": 55.59, "east_longitude": -82.5}}', 1997.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_1997_MTM14_HT2_1997', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 14", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 14", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -87, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_1997", "id": {"authority": "NRCAN", "code": "HT2_1997_NAD83CSRSV7"}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / MTM zone 14 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Ontario - between 88\u00b030''W and 85\u00b030''W.", "bbox": {"south_latitude": 47.17, "west_longitude": -88.5, "north_latitude": 56.7, "east_longitude": -85.5}}', 1997.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_1997_MTM15_HT2_1997', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 15", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 15", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -90, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_1997", "id": {"authority": "NRCAN", "code": "HT2_1997_NAD83CSRSV7"}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / MTM zone 15 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Ontario - between 91\u00b030''W and 88\u00b030''W.", "bbox": {"south_latitude": 47.97, "west_longitude": -91.5, "north_latitude": 56.9, "east_longitude": -88.5}}', 1997.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_1997_MTM16_HT2_1997', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 16", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 16", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -93, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_1997", "id": {"authority": "NRCAN", "code": "HT2_1997_NAD83CSRSV7"}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / MTM zone 16 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Ontario - between 94\u00b030''W and 91\u00b030''W.", "bbox": {"south_latitude": 48.06, "west_longitude": -94.5, "north_latitude": 55.2, "east_longitude": -91.5}}', 1997.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_1997_MTM17_HT2_1997', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 17", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 17", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -96, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_1997", "id": {"authority": "NRCAN", "code": "HT2_1997_NAD83CSRSV7"}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / MTM zone 17 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Ontario - west of 94\u00b030''W.", "bbox": {"south_latitude": 48.69, "west_longitude": -95.16, "north_latitude": 53.24, "east_longitude": -94.5}}', 1997.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2002_MTM1_HT2_2002', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 1", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 1", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -53, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_2002", "id": {"authority": "NRCAN", "code": "HT2_2002_NAD83CSRSV7"}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / MTM zone 1 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Newfoundland - onshore east of 54\u00b030''W.", "bbox": {"south_latitude": 46.56, "west_longitude": -54.5, "north_latitude": 49.89, "east_longitude": -52.54}}', 2002.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2002_MTM2_HT2_2002', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 2", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 2", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -56, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_2002", "id": {"authority": "NRCAN", "code": "HT2_2002_NAD83CSRSV7"}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / MTM zone 2 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Newfoundland and Labrador between 57\u00b030''W and 54\u00b030''W.", "bbox": {"south_latitude": 46.81, "west_longitude": -57.5, "north_latitude": 54.71, "east_longitude": -54.49}}', 2002.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2002_MTM3_HT2_2002', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 3", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 3", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -58.5, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_2002", "id": {"authority": "NRCAN", "code": "HT2_2002_NAD83CSRSV7"}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / MTM zone 3 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Newfoundland west of 57\u00b030''W.", "bbox": {"south_latitude": 47.5, "west_longitude": -59.48, "north_latitude": 50.54, "east_longitude": -57.5}}', 2002.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2002_MTM4_HT2_2002', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 4", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 4", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -61.5, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_2002", "id": {"authority": "NRCAN", "code": "HT2_2002_NAD83CSRSV7"}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / MTM zone 4 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Labrador between 63\u00b0W and 60\u00b0W.", "bbox": {"south_latitude": 52, "west_longitude": -63, "north_latitude": 58.92, "east_longitude": -60}}', 2002.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2002_MTM5_HT2_2002', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 5", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 5", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -64.5, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_2002", "id": {"authority": "NRCAN", "code": "HT2_2002_NAD83CSRSV7"}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / MTM zone 5 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Labrador - 66\u00b0W to 63\u00b0W.", "bbox": {"south_latitude": 51.58, "west_longitude": -66, "north_latitude": 60.52, "east_longitude": -63}}', 2002.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2002_MTM6_HT2_2002', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 6", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 6", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -67.5, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_2002", "id": {"authority": "NRCAN", "code": "HT2_2002_NAD83CSRSV7"}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / MTM zone 6 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Labrador - west of 66\u00b0W.", "bbox": {"south_latitude": 52.05, "west_longitude": -67.81, "north_latitude": 55.34, "east_longitude": -66}}', 2002.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2002_MTM7_HT2_2002', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 7", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 7", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -70.5, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_2002", "id": {"authority": "NRCAN", "code": "HT2_2002_NAD83CSRSV7"}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / MTM zone 7 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Quebec - between 72\u00b0W and 69\u00b0W.", "bbox": {"south_latitude": 45.01, "west_longitude": -72, "north_latitude": 61.8, "east_longitude": -69}}', 2002.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2002_MTM8_HT2_2002', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 8", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 8", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -73.5, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_2002", "id": {"authority": "NRCAN", "code": "HT2_2002_NAD83CSRSV7"}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / MTM zone 8 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Ontario - east of 75\u00b0W.", "bbox": {"south_latitude": 44.98, "west_longitude": -75, "north_latitude": 45.65, "east_longitude": -74.35}}', 2002.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2002_MTM9_HT2_2002', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 9", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 9", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -76.5, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_2002", "id": {"authority": "NRCAN", "code": "HT2_2002_NAD83CSRSV7"}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / MTM zone 9 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Ontario - between 78\u00b0W and 75\u00b0W.", "bbox": {"south_latitude": 43.63, "west_longitude": -78, "north_latitude": 46.25, "east_longitude": -75}}', 2002.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2002_MTM10_HT2_2002', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 10", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 10", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -79.5, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_2002", "id": {"authority": "NRCAN", "code": "HT2_2002_NAD83CSRSV7"}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / MTM zone 10 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Ontario - between 81\u00b0W and 78\u00b0W: south of 46\u00b0N in area to west of 80\u00b015''W, south of 47\u00b0N in area between 80\u00b015''W and 79\u00b030''W, entire province between 79\u00b030''W and 78\u00b0W.", "bbox": {"south_latitude": 42.26, "west_longitude": -81, "north_latitude": 47.33, "east_longitude": -77.99}}', 2002.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2002_MTM11_HT2_2002', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 11", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 11", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -82.5, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_2002", "id": {"authority": "NRCAN", "code": "HT2_2002_NAD83CSRSV7"}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / MTM zone 11 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Ontario - south of 46\u00b0N and west of 81\u00b0W.", "bbox": {"south_latitude": 41.67, "west_longitude": -83.6, "north_latitude": 46, "east_longitude": -81}}', 2002.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2002_MTM12_HT2_2002', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 12", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 12", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -81, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_2002", "id": {"authority": "NRCAN", "code": "HT2_2002_NAD83CSRSV7"}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / MTM zone 12 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Ontario - between 82\u00b030''W and 79\u00b030''W: north of 46\u00b0N in area between 82\u00b030''W and 80\u00b015''W, north of 47\u00b0N in area between 80\u00b015''W and 79\u00b030''W.", "bbox": {"south_latitude": 46, "west_longitude": -82.5, "north_latitude": 55.21, "east_longitude": -79.5}}', 2002.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2002_MTM13_HT2_2002', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 13", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 13", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -84, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_2002", "id": {"authority": "NRCAN", "code": "HT2_2002_NAD83CSRSV7"}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / MTM zone 13 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Ontario - between 85\u00b030''W and 82\u00b030''W and north of 46\u00b0N.", "bbox": {"south_latitude": 46, "west_longitude": -85.5, "north_latitude": 55.59, "east_longitude": -82.5}}', 2002.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2002_MTM14_HT2_2002', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 14", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 14", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -87, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_2002", "id": {"authority": "NRCAN", "code": "HT2_2002_NAD83CSRSV7"}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / MTM zone 14 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Ontario - between 88\u00b030''W and 85\u00b030''W.", "bbox": {"south_latitude": 47.17, "west_longitude": -88.5, "north_latitude": 56.7, "east_longitude": -85.5}}', 2002.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2002_MTM15_HT2_2002', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 15", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 15", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -90, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_2002", "id": {"authority": "NRCAN", "code": "HT2_2002_NAD83CSRSV7"}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / MTM zone 15 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Ontario - between 91\u00b030''W and 88\u00b030''W.", "bbox": {"south_latitude": 47.97, "west_longitude": -91.5, "north_latitude": 56.9, "east_longitude": -88.5}}', 2002.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2002_MTM16_HT2_2002', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 16", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 16", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -93, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_2002", "id": {"authority": "NRCAN", "code": "HT2_2002_NAD83CSRSV7"}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / MTM zone 16 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Ontario - between 94\u00b030''W and 91\u00b030''W.", "bbox": {"south_latitude": 48.06, "west_longitude": -94.5, "north_latitude": 55.2, "east_longitude": -91.5}}', 2002.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2002_MTM17_HT2_2002', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 17", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 17", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -96, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_2002", "id": {"authority": "NRCAN", "code": "HT2_2002_NAD83CSRSV7"}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / MTM zone 17 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Ontario - west of 94\u00b030''W.", "bbox": {"south_latitude": 48.69, "west_longitude": -95.16, "north_latitude": 53.24, "east_longitude": -94.5}}', 2002.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2010_MTM1_HT2_2010', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 1", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 1", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -53, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_2010", "id": {"authority": "EPSG", "code": 9987}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / MTM zone 1 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Newfoundland - onshore east of 54\u00b030''W.", "bbox": {"south_latitude": 46.56, "west_longitude": -54.5, "north_latitude": 49.89, "east_longitude": -52.54}}', 2010.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2010_MTM2_HT2_2010', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 2", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 2", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -56, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_2010", "id": {"authority": "EPSG", "code": 9987}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / MTM zone 2 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Newfoundland and Labrador between 57\u00b030''W and 54\u00b030''W.", "bbox": {"south_latitude": 46.81, "west_longitude": -57.5, "north_latitude": 54.71, "east_longitude": -54.49}}', 2010.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2010_MTM3_HT2_2010', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 3", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 3", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -58.5, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_2010", "id": {"authority": "EPSG", "code": 9987}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / MTM zone 3 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Newfoundland west of 57\u00b030''W.", "bbox": {"south_latitude": 47.5, "west_longitude": -59.48, "north_latitude": 50.54, "east_longitude": -57.5}}', 2010.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2010_MTM4_HT2_2010', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 4", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 4", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -61.5, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_2010", "id": {"authority": "EPSG", "code": 9987}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / MTM zone 4 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Labrador between 63\u00b0W and 60\u00b0W.", "bbox": {"south_latitude": 52, "west_longitude": -63, "north_latitude": 58.92, "east_longitude": -60}}', 2010.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2010_MTM5_HT2_2010', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 5", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 5", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -64.5, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_2010", "id": {"authority": "EPSG", "code": 9987}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / MTM zone 5 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Labrador - 66\u00b0W to 63\u00b0W.", "bbox": {"south_latitude": 51.58, "west_longitude": -66, "north_latitude": 60.52, "east_longitude": -63}}', 2010.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2010_MTM6_HT2_2010', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 6", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 6", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -67.5, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_2010", "id": {"authority": "EPSG", "code": 9987}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / MTM zone 6 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Labrador - west of 66\u00b0W.", "bbox": {"south_latitude": 52.05, "west_longitude": -67.81, "north_latitude": 55.34, "east_longitude": -66}}', 2010.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2010_MTM7_HT2_2010', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 7", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 7", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -70.5, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_2010", "id": {"authority": "EPSG", "code": 9987}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / MTM zone 7 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Quebec - between 72\u00b0W and 69\u00b0W.", "bbox": {"south_latitude": 45.01, "west_longitude": -72, "north_latitude": 61.8, "east_longitude": -69}}', 2010.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2010_MTM8_HT2_2010', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 8", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 8", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -73.5, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_2010", "id": {"authority": "EPSG", "code": 9987}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / MTM zone 8 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Ontario - east of 75\u00b0W.", "bbox": {"south_latitude": 44.98, "west_longitude": -75, "north_latitude": 45.65, "east_longitude": -74.35}}', 2010.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2010_MTM9_HT2_2010', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 9", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 9", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -76.5, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_2010", "id": {"authority": "EPSG", "code": 9987}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / MTM zone 9 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Ontario - between 78\u00b0W and 75\u00b0W.", "bbox": {"south_latitude": 43.63, "west_longitude": -78, "north_latitude": 46.25, "east_longitude": -75}}', 2010.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2010_MTM10_HT2_2010', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 10", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 10", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -79.5, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_2010", "id": {"authority": "EPSG", "code": 9987}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / MTM zone 10 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Ontario - between 81\u00b0W and 78\u00b0W: south of 46\u00b0N in area to west of 80\u00b015''W, south of 47\u00b0N in area between 80\u00b015''W and 79\u00b030''W, entire province between 79\u00b030''W and 78\u00b0W.", "bbox": {"south_latitude": 42.26, "west_longitude": -81, "north_latitude": 47.33, "east_longitude": -77.99}}', 2010.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2010_MTM11_HT2_2010', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 11", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 11", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -82.5, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_2010", "id": {"authority": "EPSG", "code": 9987}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / MTM zone 11 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Ontario - south of 46\u00b0N and west of 81\u00b0W.", "bbox": {"south_latitude": 41.67, "west_longitude": -83.6, "north_latitude": 46, "east_longitude": -81}}', 2010.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2010_MTM12_HT2_2010', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 12", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 12", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -81, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_2010", "id": {"authority": "EPSG", "code": 9987}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / MTM zone 12 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Ontario - between 82\u00b030''W and 79\u00b030''W: north of 46\u00b0N in area between 82\u00b030''W and 80\u00b015''W, north of 47\u00b0N in area between 80\u00b015''W and 79\u00b030''W.", "bbox": {"south_latitude": 46, "west_longitude": -82.5, "north_latitude": 55.21, "east_longitude": -79.5}}', 2010.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2010_MTM13_HT2_2010', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 13", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 13", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -84, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_2010", "id": {"authority": "EPSG", "code": 9987}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / MTM zone 13 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Ontario - between 85\u00b030''W and 82\u00b030''W and north of 46\u00b0N.", "bbox": {"south_latitude": 46, "west_longitude": -85.5, "north_latitude": 55.59, "east_longitude": -82.5}}', 2010.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2010_MTM14_HT2_2010', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 14", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 14", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -87, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_2010", "id": {"authority": "EPSG", "code": 9987}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / MTM zone 14 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Ontario - between 88\u00b030''W and 85\u00b030''W.", "bbox": {"south_latitude": 47.17, "west_longitude": -88.5, "north_latitude": 56.7, "east_longitude": -85.5}}', 2010.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2010_MTM15_HT2_2010', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 15", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 15", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -90, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_2010", "id": {"authority": "EPSG", "code": 9987}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / MTM zone 15 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Ontario - between 91\u00b030''W and 88\u00b030''W.", "bbox": {"south_latitude": 47.97, "west_longitude": -91.5, "north_latitude": 56.9, "east_longitude": -88.5}}', 2010.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2010_MTM16_HT2_2010', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 16", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 16", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -93, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_2010", "id": {"authority": "EPSG", "code": 9987}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / MTM zone 16 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Ontario - between 94\u00b030''W and 91\u00b030''W.", "bbox": {"south_latitude": 48.06, "west_longitude": -94.5, "north_latitude": 55.2, "east_longitude": -91.5}}', 2010.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2010_MTM17_HT2_2010', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 17", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 17", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -96, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_2010", "id": {"authority": "EPSG", "code": 9987}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / MTM zone 17 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Ontario - west of 94\u00b030''W.", "bbox": {"south_latitude": 48.69, "west_longitude": -95.16, "north_latitude": 53.24, "east_longitude": -94.5}}', 2010.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_1997_MTM1_CGVD2013_1997', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 1", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 1", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -53, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(1997) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 1997"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 20035}}], "name": "NAD83(CSRS)v7 / MTM zone 1 + CGVD2013a(1997) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Newfoundland - onshore east of 54\u00b030''W.", "bbox": {"south_latitude": 46.56, "west_longitude": -54.5, "north_latitude": 49.89, "east_longitude": -52.54}}', 1997.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_1997_MTM2_CGVD2013_1997', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 2", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 2", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -56, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(1997) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 1997"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 20035}}], "name": "NAD83(CSRS)v7 / MTM zone 2 + CGVD2013a(1997) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Newfoundland and Labrador between 57\u00b030''W and 54\u00b030''W.", "bbox": {"south_latitude": 46.81, "west_longitude": -57.5, "north_latitude": 54.71, "east_longitude": -54.49}}', 1997.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_1997_MTM3_CGVD2013_1997', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 3", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 3", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -58.5, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(1997) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 1997"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 20035}}], "name": "NAD83(CSRS)v7 / MTM zone 3 + CGVD2013a(1997) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Newfoundland west of 57\u00b030''W.", "bbox": {"south_latitude": 47.5, "west_longitude": -59.48, "north_latitude": 50.54, "east_longitude": -57.5}}', 1997.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_1997_MTM4_CGVD2013_1997', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 4", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 4", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -61.5, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(1997) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 1997"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 20035}}], "name": "NAD83(CSRS)v7 / MTM zone 4 + CGVD2013a(1997) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Labrador between 63\u00b0W and 60\u00b0W.", "bbox": {"south_latitude": 52, "west_longitude": -63, "north_latitude": 58.92, "east_longitude": -60}}', 1997.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_1997_MTM5_CGVD2013_1997', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 5", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 5", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -64.5, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(1997) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 1997"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 20035}}], "name": "NAD83(CSRS)v7 / MTM zone 5 + CGVD2013a(1997) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Labrador - 66\u00b0W to 63\u00b0W.", "bbox": {"south_latitude": 51.58, "west_longitude": -66, "north_latitude": 60.52, "east_longitude": -63}}', 1997.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_1997_MTM6_CGVD2013_1997', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 6", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 6", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -67.5, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(1997) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 1997"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 20035}}], "name": "NAD83(CSRS)v7 / MTM zone 6 + CGVD2013a(1997) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Labrador - west of 66\u00b0W.", "bbox": {"south_latitude": 52.05, "west_longitude": -67.81, "north_latitude": 55.34, "east_longitude": -66}}', 1997.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_1997_MTM7_CGVD2013_1997', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 7", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 7", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -70.5, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(1997) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 1997"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 20035}}], "name": "NAD83(CSRS)v7 / MTM zone 7 + CGVD2013a(1997) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Quebec - between 72\u00b0W and 69\u00b0W.", "bbox": {"south_latitude": 45.01, "west_longitude": -72, "north_latitude": 61.8, "east_longitude": -69}}', 1997.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_1997_MTM8_CGVD2013_1997', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 8", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 8", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -73.5, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(1997) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 1997"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 20035}}], "name": "NAD83(CSRS)v7 / MTM zone 8 + CGVD2013a(1997) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Ontario - east of 75\u00b0W.", "bbox": {"south_latitude": 44.98, "west_longitude": -75, "north_latitude": 45.65, "east_longitude": -74.35}}', 1997.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_1997_MTM9_CGVD2013_1997', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 9", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 9", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -76.5, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(1997) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 1997"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 20035}}], "name": "NAD83(CSRS)v7 / MTM zone 9 + CGVD2013a(1997) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Ontario - between 78\u00b0W and 75\u00b0W.", "bbox": {"south_latitude": 43.63, "west_longitude": -78, "north_latitude": 46.25, "east_longitude": -75}}', 1997.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_1997_MTM10_CGVD2013_1997', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 10", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 10", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -79.5, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(1997) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 1997"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 20035}}], "name": "NAD83(CSRS)v7 / MTM zone 10 + CGVD2013a(1997) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Ontario - between 81\u00b0W and 78\u00b0W: south of 46\u00b0N in area to west of 80\u00b015''W, south of 47\u00b0N in area between 80\u00b015''W and 79\u00b030''W, entire province between 79\u00b030''W and 78\u00b0W.", "bbox": {"south_latitude": 42.26, "west_longitude": -81, "north_latitude": 47.33, "east_longitude": -77.99}}', 1997.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_1997_MTM11_CGVD2013_1997', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 11", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 11", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -82.5, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(1997) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 1997"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 20035}}], "name": "NAD83(CSRS)v7 / MTM zone 11 + CGVD2013a(1997) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Ontario - south of 46\u00b0N and west of 81\u00b0W.", "bbox": {"south_latitude": 41.67, "west_longitude": -83.6, "north_latitude": 46, "east_longitude": -81}}', 1997.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_1997_MTM12_CGVD2013_1997', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 12", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 12", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -81, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(1997) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 1997"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 20035}}], "name": "NAD83(CSRS)v7 / MTM zone 12 + CGVD2013a(1997) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Ontario - between 82\u00b030''W and 79\u00b030''W: north of 46\u00b0N in area between 82\u00b030''W and 80\u00b015''W, north of 47\u00b0N in area between 80\u00b015''W and 79\u00b030''W.", "bbox": {"south_latitude": 46, "west_longitude": -82.5, "north_latitude": 55.21, "east_longitude": -79.5}}', 1997.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_1997_MTM13_CGVD2013_1997', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 13", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 13", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -84, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(1997) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 1997"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 20035}}], "name": "NAD83(CSRS)v7 / MTM zone 13 + CGVD2013a(1997) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Ontario - between 85\u00b030''W and 82\u00b030''W and north of 46\u00b0N.", "bbox": {"south_latitude": 46, "west_longitude": -85.5, "north_latitude": 55.59, "east_longitude": -82.5}}', 1997.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_1997_MTM14_CGVD2013_1997', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 14", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 14", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -87, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(1997) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 1997"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 20035}}], "name": "NAD83(CSRS)v7 / MTM zone 14 + CGVD2013a(1997) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Ontario - between 88\u00b030''W and 85\u00b030''W.", "bbox": {"south_latitude": 47.17, "west_longitude": -88.5, "north_latitude": 56.7, "east_longitude": -85.5}}', 1997.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_1997_MTM15_CGVD2013_1997', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 15", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 15", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -90, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(1997) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 1997"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 20035}}], "name": "NAD83(CSRS)v7 / MTM zone 15 + CGVD2013a(1997) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Ontario - between 91\u00b030''W and 88\u00b030''W.", "bbox": {"south_latitude": 47.97, "west_longitude": -91.5, "north_latitude": 56.9, "east_longitude": -88.5}}', 1997.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_1997_MTM16_CGVD2013_1997', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 16", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 16", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -93, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(1997) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 1997"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 20035}}], "name": "NAD83(CSRS)v7 / MTM zone 16 + CGVD2013a(1997) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Ontario - between 94\u00b030''W and 91\u00b030''W.", "bbox": {"south_latitude": 48.06, "west_longitude": -94.5, "north_latitude": 55.2, "east_longitude": -91.5}}', 1997.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_1997_MTM17_CGVD2013_1997', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 17", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 17", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -96, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(1997) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 1997"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 20035}}], "name": "NAD83(CSRS)v7 / MTM zone 17 + CGVD2013a(1997) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Ontario - west of 94\u00b030''W.", "bbox": {"south_latitude": 48.69, "west_longitude": -95.16, "north_latitude": 53.24, "east_longitude": -94.5}}', 1997.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2002_MTM1_CGVD2013_2002', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 1", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 1", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -53, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(2002) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 2002"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 20034}}], "name": "NAD83(CSRS)v7 / MTM zone 1 + CGVD2013a(2002) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Newfoundland - onshore east of 54\u00b030''W.", "bbox": {"south_latitude": 46.56, "west_longitude": -54.5, "north_latitude": 49.89, "east_longitude": -52.54}}', 2002.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2002_MTM2_CGVD2013_2002', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 2", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 2", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -56, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(2002) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 2002"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 20034}}], "name": "NAD83(CSRS)v7 / MTM zone 2 + CGVD2013a(2002) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Newfoundland and Labrador between 57\u00b030''W and 54\u00b030''W.", "bbox": {"south_latitude": 46.81, "west_longitude": -57.5, "north_latitude": 54.71, "east_longitude": -54.49}}', 2002.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2002_MTM3_CGVD2013_2002', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 3", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 3", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -58.5, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(2002) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 2002"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 20034}}], "name": "NAD83(CSRS)v7 / MTM zone 3 + CGVD2013a(2002) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Newfoundland west of 57\u00b030''W.", "bbox": {"south_latitude": 47.5, "west_longitude": -59.48, "north_latitude": 50.54, "east_longitude": -57.5}}', 2002.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2002_MTM4_CGVD2013_2002', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 4", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 4", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -61.5, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(2002) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 2002"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 20034}}], "name": "NAD83(CSRS)v7 / MTM zone 4 + CGVD2013a(2002) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Labrador between 63\u00b0W and 60\u00b0W.", "bbox": {"south_latitude": 52, "west_longitude": -63, "north_latitude": 58.92, "east_longitude": -60}}', 2002.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2002_MTM5_CGVD2013_2002', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 5", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 5", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -64.5, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(2002) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 2002"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 20034}}], "name": "NAD83(CSRS)v7 / MTM zone 5 + CGVD2013a(2002) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Labrador - 66\u00b0W to 63\u00b0W.", "bbox": {"south_latitude": 51.58, "west_longitude": -66, "north_latitude": 60.52, "east_longitude": -63}}', 2002.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2002_MTM6_CGVD2013_2002', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 6", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 6", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -67.5, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(2002) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 2002"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 20034}}], "name": "NAD83(CSRS)v7 / MTM zone 6 + CGVD2013a(2002) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Labrador - west of 66\u00b0W.", "bbox": {"south_latitude": 52.05, "west_longitude": -67.81, "north_latitude": 55.34, "east_longitude": -66}}', 2002.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2002_MTM7_CGVD2013_2002', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 7", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 7", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -70.5, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(2002) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 2002"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 20034}}], "name": "NAD83(CSRS)v7 / MTM zone 7 + CGVD2013a(2002) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Quebec - between 72\u00b0W and 69\u00b0W.", "bbox": {"south_latitude": 45.01, "west_longitude": -72, "north_latitude": 61.8, "east_longitude": -69}}', 2002.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2002_MTM8_CGVD2013_2002', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 8", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 8", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -73.5, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(2002) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 2002"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 20034}}], "name": "NAD83(CSRS)v7 / MTM zone 8 + CGVD2013a(2002) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Ontario - east of 75\u00b0W.", "bbox": {"south_latitude": 44.98, "west_longitude": -75, "north_latitude": 45.65, "east_longitude": -74.35}}', 2002.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2002_MTM9_CGVD2013_2002', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 9", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 9", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -76.5, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(2002) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 2002"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 20034}}], "name": "NAD83(CSRS)v7 / MTM zone 9 + CGVD2013a(2002) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Ontario - between 78\u00b0W and 75\u00b0W.", "bbox": {"south_latitude": 43.63, "west_longitude": -78, "north_latitude": 46.25, "east_longitude": -75}}', 2002.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2002_MTM10_CGVD2013_2002', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 10", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 10", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -79.5, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(2002) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 2002"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 20034}}], "name": "NAD83(CSRS)v7 / MTM zone 10 + CGVD2013a(2002) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Ontario - between 81\u00b0W and 78\u00b0W: south of 46\u00b0N in area to west of 80\u00b015''W, south of 47\u00b0N in area between 80\u00b015''W and 79\u00b030''W, entire province between 79\u00b030''W and 78\u00b0W.", "bbox": {"south_latitude": 42.26, "west_longitude": -81, "north_latitude": 47.33, "east_longitude": -77.99}}', 2002.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2002_MTM11_CGVD2013_2002', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 11", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 11", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -82.5, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(2002) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 2002"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 20034}}], "name": "NAD83(CSRS)v7 / MTM zone 11 + CGVD2013a(2002) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Ontario - south of 46\u00b0N and west of 81\u00b0W.", "bbox": {"south_latitude": 41.67, "west_longitude": -83.6, "north_latitude": 46, "east_longitude": -81}}', 2002.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2002_MTM12_CGVD2013_2002', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 12", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 12", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -81, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(2002) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 2002"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 20034}}], "name": "NAD83(CSRS)v7 / MTM zone 12 + CGVD2013a(2002) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Ontario - between 82\u00b030''W and 79\u00b030''W: north of 46\u00b0N in area between 82\u00b030''W and 80\u00b015''W, north of 47\u00b0N in area between 80\u00b015''W and 79\u00b030''W.", "bbox": {"south_latitude": 46, "west_longitude": -82.5, "north_latitude": 55.21, "east_longitude": -79.5}}', 2002.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2002_MTM13_CGVD2013_2002', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 13", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 13", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -84, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(2002) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 2002"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 20034}}], "name": "NAD83(CSRS)v7 / MTM zone 13 + CGVD2013a(2002) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Ontario - between 85\u00b030''W and 82\u00b030''W and north of 46\u00b0N.", "bbox": {"south_latitude": 46, "west_longitude": -85.5, "north_latitude": 55.59, "east_longitude": -82.5}}', 2002.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2002_MTM14_CGVD2013_2002', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 14", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 14", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -87, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(2002) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 2002"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 20034}}], "name": "NAD83(CSRS)v7 / MTM zone 14 + CGVD2013a(2002) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Ontario - between 88\u00b030''W and 85\u00b030''W.", "bbox": {"south_latitude": 47.17, "west_longitude": -88.5, "north_latitude": 56.7, "east_longitude": -85.5}}', 2002.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2002_MTM15_CGVD2013_2002', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 15", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 15", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -90, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(2002) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 2002"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 20034}}], "name": "NAD83(CSRS)v7 / MTM zone 15 + CGVD2013a(2002) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Ontario - between 91\u00b030''W and 88\u00b030''W.", "bbox": {"south_latitude": 47.97, "west_longitude": -91.5, "north_latitude": 56.9, "east_longitude": -88.5}}', 2002.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2002_MTM16_CGVD2013_2002', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 16", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 16", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -93, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(2002) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 2002"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 20034}}], "name": "NAD83(CSRS)v7 / MTM zone 16 + CGVD2013a(2002) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Ontario - between 94\u00b030''W and 91\u00b030''W.", "bbox": {"south_latitude": 48.06, "west_longitude": -94.5, "north_latitude": 55.2, "east_longitude": -91.5}}', 2002.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2002_MTM17_CGVD2013_2002', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 17", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 17", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -96, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(2002) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 2002"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 20034}}], "name": "NAD83(CSRS)v7 / MTM zone 17 + CGVD2013a(2002) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Ontario - west of 94\u00b030''W.", "bbox": {"south_latitude": 48.69, "west_longitude": -95.16, "north_latitude": 53.24, "east_longitude": -94.5}}', 2002.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2010_MTM1_CGVD2013_2010', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 1", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 1", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -53, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(2010) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 2010"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 9245}}], "name": "NAD83(CSRS)v7 / MTM zone 1 + CGVD2013a(2010) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Newfoundland - onshore east of 54\u00b030''W.", "bbox": {"south_latitude": 46.56, "west_longitude": -54.5, "north_latitude": 49.89, "east_longitude": -52.54}}', 2010.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2010_MTM2_CGVD2013_2010', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 2", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 2", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -56, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(2010) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 2010"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 9245}}], "name": "NAD83(CSRS)v7 / MTM zone 2 + CGVD2013a(2010) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Newfoundland and Labrador between 57\u00b030''W and 54\u00b030''W.", "bbox": {"south_latitude": 46.81, "west_longitude": -57.5, "north_latitude": 54.71, "east_longitude": -54.49}}', 2010.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2010_MTM3_CGVD2013_2010', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 3", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 3", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -58.5, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(2010) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 2010"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 9245}}], "name": "NAD83(CSRS)v7 / MTM zone 3 + CGVD2013a(2010) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Newfoundland west of 57\u00b030''W.", "bbox": {"south_latitude": 47.5, "west_longitude": -59.48, "north_latitude": 50.54, "east_longitude": -57.5}}', 2010.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2010_MTM4_CGVD2013_2010', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 4", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 4", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -61.5, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(2010) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 2010"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 9245}}], "name": "NAD83(CSRS)v7 / MTM zone 4 + CGVD2013a(2010) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Labrador between 63\u00b0W and 60\u00b0W.", "bbox": {"south_latitude": 52, "west_longitude": -63, "north_latitude": 58.92, "east_longitude": -60}}', 2010.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2010_MTM5_CGVD2013_2010', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 5", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 5", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -64.5, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(2010) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 2010"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 9245}}], "name": "NAD83(CSRS)v7 / MTM zone 5 + CGVD2013a(2010) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Labrador - 66\u00b0W to 63\u00b0W.", "bbox": {"south_latitude": 51.58, "west_longitude": -66, "north_latitude": 60.52, "east_longitude": -63}}', 2010.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2010_MTM6_CGVD2013_2010', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 6", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 6", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -67.5, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(2010) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 2010"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 9245}}], "name": "NAD83(CSRS)v7 / MTM zone 6 + CGVD2013a(2010) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Labrador - west of 66\u00b0W.", "bbox": {"south_latitude": 52.05, "west_longitude": -67.81, "north_latitude": 55.34, "east_longitude": -66}}', 2010.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2010_MTM7_CGVD2013_2010', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 7", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 7", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -70.5, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(2010) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 2010"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 9245}}], "name": "NAD83(CSRS)v7 / MTM zone 7 + CGVD2013a(2010) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Quebec - between 72\u00b0W and 69\u00b0W.", "bbox": {"south_latitude": 45.01, "west_longitude": -72, "north_latitude": 61.8, "east_longitude": -69}}', 2010.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2010_MTM8_CGVD2013_2010', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 8", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 8", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -73.5, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(2010) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 2010"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 9245}}], "name": "NAD83(CSRS)v7 / MTM zone 8 + CGVD2013a(2010) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Ontario - east of 75\u00b0W.", "bbox": {"south_latitude": 44.98, "west_longitude": -75, "north_latitude": 45.65, "east_longitude": -74.35}}', 2010.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2010_MTM9_CGVD2013_2010', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 9", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 9", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -76.5, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(2010) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 2010"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 9245}}], "name": "NAD83(CSRS)v7 / MTM zone 9 + CGVD2013a(2010) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Ontario - between 78\u00b0W and 75\u00b0W.", "bbox": {"south_latitude": 43.63, "west_longitude": -78, "north_latitude": 46.25, "east_longitude": -75}}', 2010.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2010_MTM10_CGVD2013_2010', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 10", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 10", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -79.5, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(2010) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 2010"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 9245}}], "name": "NAD83(CSRS)v7 / MTM zone 10 + CGVD2013a(2010) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Ontario - between 81\u00b0W and 78\u00b0W: south of 46\u00b0N in area to west of 80\u00b015''W, south of 47\u00b0N in area between 80\u00b015''W and 79\u00b030''W, entire province between 79\u00b030''W and 78\u00b0W.", "bbox": {"south_latitude": 42.26, "west_longitude": -81, "north_latitude": 47.33, "east_longitude": -77.99}}', 2010.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2010_MTM11_CGVD2013_2010', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 11", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 11", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -82.5, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(2010) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 2010"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 9245}}], "name": "NAD83(CSRS)v7 / MTM zone 11 + CGVD2013a(2010) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Ontario - south of 46\u00b0N and west of 81\u00b0W.", "bbox": {"south_latitude": 41.67, "west_longitude": -83.6, "north_latitude": 46, "east_longitude": -81}}', 2010.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2010_MTM12_CGVD2013_2010', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 12", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 12", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -81, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(2010) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 2010"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 9245}}], "name": "NAD83(CSRS)v7 / MTM zone 12 + CGVD2013a(2010) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Ontario - between 82\u00b030''W and 79\u00b030''W: north of 46\u00b0N in area between 82\u00b030''W and 80\u00b015''W, north of 47\u00b0N in area between 80\u00b015''W and 79\u00b030''W.", "bbox": {"south_latitude": 46, "west_longitude": -82.5, "north_latitude": 55.21, "east_longitude": -79.5}}', 2010.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2010_MTM13_CGVD2013_2010', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 13", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 13", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -84, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(2010) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 2010"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 9245}}], "name": "NAD83(CSRS)v7 / MTM zone 13 + CGVD2013a(2010) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Ontario - between 85\u00b030''W and 82\u00b030''W and north of 46\u00b0N.", "bbox": {"south_latitude": 46, "west_longitude": -85.5, "north_latitude": 55.59, "east_longitude": -82.5}}', 2010.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2010_MTM14_CGVD2013_2010', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 14", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 14", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -87, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(2010) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 2010"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 9245}}], "name": "NAD83(CSRS)v7 / MTM zone 14 + CGVD2013a(2010) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Ontario - between 88\u00b030''W and 85\u00b030''W.", "bbox": {"south_latitude": 47.17, "west_longitude": -88.5, "north_latitude": 56.7, "east_longitude": -85.5}}', 2010.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2010_MTM15_CGVD2013_2010', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 15", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 15", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -90, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(2010) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 2010"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 9245}}], "name": "NAD83(CSRS)v7 / MTM zone 15 + CGVD2013a(2010) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Ontario - between 91\u00b030''W and 88\u00b030''W.", "bbox": {"south_latitude": 47.97, "west_longitude": -91.5, "north_latitude": 56.9, "east_longitude": -88.5}}', 2010.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2010_MTM16_CGVD2013_2010', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 16", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 16", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -93, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(2010) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 2010"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 9245}}], "name": "NAD83(CSRS)v7 / MTM zone 16 + CGVD2013a(2010) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Ontario - between 94\u00b030''W and 91\u00b030''W.", "bbox": {"south_latitude": 48.06, "west_longitude": -94.5, "north_latitude": 55.2, "east_longitude": -91.5}}', 2010.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2010_MTM17_CGVD2013_2010', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / MTM zone 17", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "MTM zone 17", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -96, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9999, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 304800, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(2010) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 2010"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 9245}}], "name": "NAD83(CSRS)v7 / MTM zone 17 + CGVD2013a(2010) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada - Ontario - west of 94\u00b030''W.", "bbox": {"south_latitude": 48.69, "west_longitude": -95.16, "north_latitude": 53.24, "east_longitude": -94.5}}', 2010.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_1997_UTM7_HT2_1997', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 7", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 7N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -141, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_1997", "id": {"authority": "NRCAN", "code": "HT2_1997_NAD83CSRSV7"}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / UTM zone 7 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada west of 138\u00b0W, onshore and offshore south of 84\u00b0N - British Columbia, Yukon.", "bbox": {"south_latitude": 52.05, "west_longitude": -141.01, "north_latitude": 72.53, "east_longitude": -138}}', 1997.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_1997_UTM8_HT2_1997', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 8", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 8N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -135, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_1997", "id": {"authority": "NRCAN", "code": "HT2_1997_NAD83CSRSV7"}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / UTM zone 8 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 138\u00b0W and 132\u00b0W, onshore and offshore south of 84\u00b0N - British Columbia, Northwest Territories, Yukon.", "bbox": {"south_latitude": 48.06, "west_longitude": -138, "north_latitude": 79.42, "east_longitude": -132}}', 1997.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_1997_UTM9_HT2_1997', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 9", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 9N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -129, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_1997", "id": {"authority": "NRCAN", "code": "HT2_1997_NAD83CSRSV7"}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / UTM zone 9 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 138\u00b0W and 132\u00b0W, onshore and offshore south of 84\u00b0N - British Columbia, Northwest Territories, Yukon.", "bbox": {"south_latitude": 48.06, "west_longitude": -138, "north_latitude": 79.42, "east_longitude": -132}}', 1997.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_1997_UTM10_HT2_1997', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 10", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 10N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -123, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_1997", "id": {"authority": "NRCAN", "code": "HT2_1997_NAD83CSRSV7"}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / UTM zone 10 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 126\u00b0W and 120\u00b0W, onshore and offshore south of 84\u00b0N - British Columbia, Northwest Territories, Yukon.", "bbox": {"south_latitude": 48.13, "west_longitude": -126, "north_latitude": 81.8, "east_longitude": -120}}', 1997.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_1997_UTM11_HT2_1997', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 11", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 11N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -117, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_1997", "id": {"authority": "NRCAN", "code": "HT2_1997_NAD83CSRSV7"}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / UTM zone 11 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 120\u00b0W and 114\u00b0W onshore and offshore - Alberta, British Columbia, Northwest Territories, Nunavut.", "bbox": {"south_latitude": 48.99, "west_longitude": -120, "north_latitude": 83.5, "east_longitude": -114}}', 1997.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_1997_UTM12_HT2_1997', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 12", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 12N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -111, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_1997", "id": {"authority": "NRCAN", "code": "HT2_1997_NAD83CSRSV7"}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / UTM zone 12 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 114\u00b0W and 108\u00b0W onshore and offshore - Alberta, Northwest Territories, Nunavut, Saskatchewan.", "bbox": {"south_latitude": 48.99, "west_longitude": -114, "north_latitude": 84, "east_longitude": -108}}', 1997.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_1997_UTM13_HT2_1997', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 13", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 13N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -105, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_1997", "id": {"authority": "NRCAN", "code": "HT2_1997_NAD83CSRSV7"}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / UTM zone 13 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 108\u00b0W and 102\u00b0W onshore and offshore - Northwest Territories, Nunavut, Saskatchewan.", "bbox": {"south_latitude": 48.99, "west_longitude": -108, "north_latitude": 84, "east_longitude": -102}}', 1997.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_1997_UTM14_HT2_1997', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 14", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 14N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -99, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_1997", "id": {"authority": "NRCAN", "code": "HT2_1997_NAD83CSRSV7"}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / UTM zone 14 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 102\u00b0W and 96\u00b0W, onshore and offshore south of 84\u00b0N - Manitoba, Nunavut, Saskatchewan.", "bbox": {"south_latitude": 48.99, "west_longitude": -102, "north_latitude": 84, "east_longitude": -96}}', 1997.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_1997_UTM15_HT2_1997', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 15", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 15N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -93, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_1997", "id": {"authority": "NRCAN", "code": "HT2_1997_NAD83CSRSV7"}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / UTM zone 15 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 96\u00b0W and 90\u00b0W, onshore and offshore south of 84\u00b0N - Manitoba, Nunavut, Ontario.", "bbox": {"south_latitude": 48.03, "west_longitude": -96, "north_latitude": 84, "east_longitude": -90}}', 1997.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_1997_UTM16_HT2_1997', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 16", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 16N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -87, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_1997", "id": {"authority": "NRCAN", "code": "HT2_1997_NAD83CSRSV7"}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / UTM zone 16 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 90\u00b0W and 84\u00b0W, onshore and offshore south of 84\u00b0N - Manitoba, Nunavut, Ontario.", "bbox": {"south_latitude": 46.11, "west_longitude": -90, "north_latitude": 84, "east_longitude": -84}}', 1997.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_1997_UTM17_HT2_1997', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 17", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 17N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -81, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_1997", "id": {"authority": "NRCAN", "code": "HT2_1997_NAD83CSRSV7"}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / UTM zone 17 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 84\u00b0W and 78\u00b0W, onshore and offshore south of 84\u00b0N - Nunavut, Ontario and Quebec.", "bbox": {"south_latitude": 41.67, "west_longitude": -84, "north_latitude": 84, "east_longitude": -78}}', 1997.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_1997_UTM18_HT2_1997', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 18", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 18N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -75, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_1997", "id": {"authority": "NRCAN", "code": "HT2_1997_NAD83CSRSV7"}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / UTM zone 18 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 78\u00b0W and 72\u00b0W, onshore and offshore south of 84\u00b0N - Nunavut, Ontario and Quebec.", "bbox": {"south_latitude": 43.63, "west_longitude": -78, "north_latitude": 84, "east_longitude": -72}}', 1997.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_1997_UTM19_HT2_1997', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 19", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 19N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -69, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_1997", "id": {"authority": "NRCAN", "code": "HT2_1997_NAD83CSRSV7"}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / UTM zone 19 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 72\u00b0W and 66\u00b0W onshore and offshore - New Brunswick, Labrador, Nova Scotia, Nunavut, Quebec.", "bbox": {"south_latitude": 40.8, "west_longitude": -72, "north_latitude": 84, "east_longitude": -66}}', 1997.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_1997_UTM20_HT2_1997', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 20", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 20N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -63, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_1997", "id": {"authority": "NRCAN", "code": "HT2_1997_NAD83CSRSV7"}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / UTM zone 20 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 66\u00b0W and 60\u00b0W onshore and offshore - New Brunswick, Labrador, Nova Scotia, Nunavut, Prince Edward Island, Quebec.", "bbox": {"south_latitude": 40.04, "west_longitude": -66, "north_latitude": 84, "east_longitude": -60}}', 1997.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_1997_UTM21_HT2_1997', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 21", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 21N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -57, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_1997", "id": {"authority": "NRCAN", "code": "HT2_1997_NAD83CSRSV7"}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / UTM zone 21 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 60\u00b0W and 54\u00b0W - Newfoundland and Labrador; Nunavut; Quebec.", "bbox": {"south_latitude": 38.56, "west_longitude": -60, "north_latitude": 84, "east_longitude": -54}}', 1997.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_1997_UTM22_HT2_1997', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 22", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 22N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -51, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_1997", "id": {"authority": "NRCAN", "code": "HT2_1997_NAD83CSRSV7"}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / UTM zone 22 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 54\u00b0W and 48\u00b0W onshore and offshore - Newfoundland and Labrador.", "bbox": {"south_latitude": 39.5, "west_longitude": -54, "north_latitude": 57.65, "east_longitude": -47.99}}', 1997.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2002_UTM7_HT2_2002', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 7", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 7N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -141, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_2002", "id": {"authority": "NRCAN", "code": "HT2_2002_NAD83CSRSV7"}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / UTM zone 7 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada west of 138\u00b0W, onshore and offshore south of 84\u00b0N - British Columbia, Yukon.", "bbox": {"south_latitude": 52.05, "west_longitude": -141.01, "north_latitude": 72.53, "east_longitude": -138}}', 2002.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2002_UTM8_HT2_2002', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 8", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 8N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -135, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_2002", "id": {"authority": "NRCAN", "code": "HT2_2002_NAD83CSRSV7"}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / UTM zone 8 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 138\u00b0W and 132\u00b0W, onshore and offshore south of 84\u00b0N - British Columbia, Northwest Territories, Yukon.", "bbox": {"south_latitude": 48.06, "west_longitude": -138, "north_latitude": 79.42, "east_longitude": -132}}', 2002.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2002_UTM9_HT2_2002', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 9", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 9N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -129, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_2002", "id": {"authority": "NRCAN", "code": "HT2_2002_NAD83CSRSV7"}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / UTM zone 9 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 138\u00b0W and 132\u00b0W, onshore and offshore south of 84\u00b0N - British Columbia, Northwest Territories, Yukon.", "bbox": {"south_latitude": 48.06, "west_longitude": -138, "north_latitude": 79.42, "east_longitude": -132}}', 2002.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2002_UTM10_HT2_2002', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 10", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 10N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -123, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_2002", "id": {"authority": "NRCAN", "code": "HT2_2002_NAD83CSRSV7"}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / UTM zone 10 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 126\u00b0W and 120\u00b0W, onshore and offshore south of 84\u00b0N - British Columbia, Northwest Territories, Yukon.", "bbox": {"south_latitude": 48.13, "west_longitude": -126, "north_latitude": 81.8, "east_longitude": -120}}', 2002.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2002_UTM11_HT2_2002', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 11", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 11N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -117, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_2002", "id": {"authority": "NRCAN", "code": "HT2_2002_NAD83CSRSV7"}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / UTM zone 11 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 120\u00b0W and 114\u00b0W onshore and offshore - Alberta, British Columbia, Northwest Territories, Nunavut.", "bbox": {"south_latitude": 48.99, "west_longitude": -120, "north_latitude": 83.5, "east_longitude": -114}}', 2002.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2002_UTM12_HT2_2002', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 12", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 12N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -111, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_2002", "id": {"authority": "NRCAN", "code": "HT2_2002_NAD83CSRSV7"}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / UTM zone 12 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 114\u00b0W and 108\u00b0W onshore and offshore - Alberta, Northwest Territories, Nunavut, Saskatchewan.", "bbox": {"south_latitude": 48.99, "west_longitude": -114, "north_latitude": 84, "east_longitude": -108}}', 2002.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2002_UTM13_HT2_2002', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 13", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 13N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -105, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_2002", "id": {"authority": "NRCAN", "code": "HT2_2002_NAD83CSRSV7"}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / UTM zone 13 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 108\u00b0W and 102\u00b0W onshore and offshore - Northwest Territories, Nunavut, Saskatchewan.", "bbox": {"south_latitude": 48.99, "west_longitude": -108, "north_latitude": 84, "east_longitude": -102}}', 2002.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2002_UTM14_HT2_2002', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 14", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 14N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -99, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_2002", "id": {"authority": "NRCAN", "code": "HT2_2002_NAD83CSRSV7"}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / UTM zone 14 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 102\u00b0W and 96\u00b0W, onshore and offshore south of 84\u00b0N - Manitoba, Nunavut, Saskatchewan.", "bbox": {"south_latitude": 48.99, "west_longitude": -102, "north_latitude": 84, "east_longitude": -96}}', 2002.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2002_UTM15_HT2_2002', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 15", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 15N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -93, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_2002", "id": {"authority": "NRCAN", "code": "HT2_2002_NAD83CSRSV7"}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / UTM zone 15 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 96\u00b0W and 90\u00b0W, onshore and offshore south of 84\u00b0N - Manitoba, Nunavut, Ontario.", "bbox": {"south_latitude": 48.03, "west_longitude": -96, "north_latitude": 84, "east_longitude": -90}}', 2002.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2002_UTM16_HT2_2002', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 16", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 16N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -87, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_2002", "id": {"authority": "NRCAN", "code": "HT2_2002_NAD83CSRSV7"}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / UTM zone 16 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 90\u00b0W and 84\u00b0W, onshore and offshore south of 84\u00b0N - Manitoba, Nunavut, Ontario.", "bbox": {"south_latitude": 46.11, "west_longitude": -90, "north_latitude": 84, "east_longitude": -84}}', 2002.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2002_UTM17_HT2_2002', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 17", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 17N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -81, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_2002", "id": {"authority": "NRCAN", "code": "HT2_2002_NAD83CSRSV7"}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / UTM zone 17 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 84\u00b0W and 78\u00b0W, onshore and offshore south of 84\u00b0N - Nunavut, Ontario and Quebec.", "bbox": {"south_latitude": 41.67, "west_longitude": -84, "north_latitude": 84, "east_longitude": -78}}', 2002.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2002_UTM18_HT2_2002', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 18", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 18N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -75, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_2002", "id": {"authority": "NRCAN", "code": "HT2_2002_NAD83CSRSV7"}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / UTM zone 18 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 78\u00b0W and 72\u00b0W, onshore and offshore south of 84\u00b0N - Nunavut, Ontario and Quebec.", "bbox": {"south_latitude": 43.63, "west_longitude": -78, "north_latitude": 84, "east_longitude": -72}}', 2002.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2002_UTM19_HT2_2002', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 19", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 19N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -69, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_2002", "id": {"authority": "NRCAN", "code": "HT2_2002_NAD83CSRSV7"}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / UTM zone 19 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 72\u00b0W and 66\u00b0W onshore and offshore - New Brunswick, Labrador, Nova Scotia, Nunavut, Quebec.", "bbox": {"south_latitude": 40.8, "west_longitude": -72, "north_latitude": 84, "east_longitude": -66}}', 2002.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2002_UTM20_HT2_2002', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 20", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 20N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -63, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_2002", "id": {"authority": "NRCAN", "code": "HT2_2002_NAD83CSRSV7"}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / UTM zone 20 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 66\u00b0W and 60\u00b0W onshore and offshore - New Brunswick, Labrador, Nova Scotia, Nunavut, Prince Edward Island, Quebec.", "bbox": {"south_latitude": 40.04, "west_longitude": -66, "north_latitude": 84, "east_longitude": -60}}', 2002.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2002_UTM21_HT2_2002', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 21", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 21N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -57, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_2002", "id": {"authority": "NRCAN", "code": "HT2_2002_NAD83CSRSV7"}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / UTM zone 21 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 60\u00b0W and 54\u00b0W - Newfoundland and Labrador; Nunavut; Quebec.", "bbox": {"south_latitude": 38.56, "west_longitude": -60, "north_latitude": 84, "east_longitude": -54}}', 2002.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2002_UTM22_HT2_2002', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 22", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 22N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -51, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_2002", "id": {"authority": "NRCAN", "code": "HT2_2002_NAD83CSRSV7"}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / UTM zone 22 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 54\u00b0W and 48\u00b0W onshore and offshore - Newfoundland and Labrador.", "bbox": {"south_latitude": 39.5, "west_longitude": -54, "north_latitude": 57.65, "east_longitude": -47.99}}', 2002.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2010_UTM7_HT2_2010', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 7", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 7N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -141, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_2010", "id": {"authority": "EPSG", "code": 9987}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / UTM zone 7 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada west of 138\u00b0W, onshore and offshore south of 84\u00b0N - British Columbia, Yukon.", "bbox": {"south_latitude": 52.05, "west_longitude": -141.01, "north_latitude": 72.53, "east_longitude": -138}}', 2010.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2010_UTM8_HT2_2010', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 8", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 8N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -135, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_2010", "id": {"authority": "EPSG", "code": 9987}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / UTM zone 8 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 138\u00b0W and 132\u00b0W, onshore and offshore south of 84\u00b0N - British Columbia, Northwest Territories, Yukon.", "bbox": {"south_latitude": 48.06, "west_longitude": -138, "north_latitude": 79.42, "east_longitude": -132}}', 2010.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2010_UTM9_HT2_2010', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 9", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 9N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -129, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_2010", "id": {"authority": "EPSG", "code": 9987}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / UTM zone 9 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 138\u00b0W and 132\u00b0W, onshore and offshore south of 84\u00b0N - British Columbia, Northwest Territories, Yukon.", "bbox": {"south_latitude": 48.06, "west_longitude": -138, "north_latitude": 79.42, "east_longitude": -132}}', 2010.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2010_UTM10_HT2_2010', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 10", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 10N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -123, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_2010", "id": {"authority": "EPSG", "code": 9987}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / UTM zone 10 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 126\u00b0W and 120\u00b0W, onshore and offshore south of 84\u00b0N - British Columbia, Northwest Territories, Yukon.", "bbox": {"south_latitude": 48.13, "west_longitude": -126, "north_latitude": 81.8, "east_longitude": -120}}', 2010.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2010_UTM11_HT2_2010', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 11", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 11N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -117, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_2010", "id": {"authority": "EPSG", "code": 9987}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / UTM zone 11 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 120\u00b0W and 114\u00b0W onshore and offshore - Alberta, British Columbia, Northwest Territories, Nunavut.", "bbox": {"south_latitude": 48.99, "west_longitude": -120, "north_latitude": 83.5, "east_longitude": -114}}', 2010.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2010_UTM12_HT2_2010', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 12", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 12N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -111, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_2010", "id": {"authority": "EPSG", "code": 9987}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / UTM zone 12 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 114\u00b0W and 108\u00b0W onshore and offshore - Alberta, Northwest Territories, Nunavut, Saskatchewan.", "bbox": {"south_latitude": 48.99, "west_longitude": -114, "north_latitude": 84, "east_longitude": -108}}', 2010.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2010_UTM13_HT2_2010', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 13", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 13N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -105, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_2010", "id": {"authority": "EPSG", "code": 9987}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / UTM zone 13 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 108\u00b0W and 102\u00b0W onshore and offshore - Northwest Territories, Nunavut, Saskatchewan.", "bbox": {"south_latitude": 48.99, "west_longitude": -108, "north_latitude": 84, "east_longitude": -102}}', 2010.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2010_UTM14_HT2_2010', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 14", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 14N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -99, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_2010", "id": {"authority": "EPSG", "code": 9987}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / UTM zone 14 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 102\u00b0W and 96\u00b0W, onshore and offshore south of 84\u00b0N - Manitoba, Nunavut, Saskatchewan.", "bbox": {"south_latitude": 48.99, "west_longitude": -102, "north_latitude": 84, "east_longitude": -96}}', 2010.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2010_UTM15_HT2_2010', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 15", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 15N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -93, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_2010", "id": {"authority": "EPSG", "code": 9987}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / UTM zone 15 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 96\u00b0W and 90\u00b0W, onshore and offshore south of 84\u00b0N - Manitoba, Nunavut, Ontario.", "bbox": {"south_latitude": 48.03, "west_longitude": -96, "north_latitude": 84, "east_longitude": -90}}', 2010.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2010_UTM16_HT2_2010', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 16", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 16N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -87, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_2010", "id": {"authority": "EPSG", "code": 9987}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / UTM zone 16 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 90\u00b0W and 84\u00b0W, onshore and offshore south of 84\u00b0N - Manitoba, Nunavut, Ontario.", "bbox": {"south_latitude": 46.11, "west_longitude": -90, "north_latitude": 84, "east_longitude": -84}}', 2010.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2010_UTM17_HT2_2010', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 17", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 17N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -81, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_2010", "id": {"authority": "EPSG", "code": 9987}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / UTM zone 17 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 84\u00b0W and 78\u00b0W, onshore and offshore south of 84\u00b0N - Nunavut, Ontario and Quebec.", "bbox": {"south_latitude": 41.67, "west_longitude": -84, "north_latitude": 84, "east_longitude": -78}}', 2010.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2010_UTM18_HT2_2010', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 18", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 18N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -75, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_2010", "id": {"authority": "EPSG", "code": 9987}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / UTM zone 18 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 78\u00b0W and 72\u00b0W, onshore and offshore south of 84\u00b0N - Nunavut, Ontario and Quebec.", "bbox": {"south_latitude": 43.63, "west_longitude": -78, "north_latitude": 84, "east_longitude": -72}}', 2010.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2010_UTM19_HT2_2010', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 19", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 19N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -69, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_2010", "id": {"authority": "EPSG", "code": 9987}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / UTM zone 19 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 72\u00b0W and 66\u00b0W onshore and offshore - New Brunswick, Labrador, Nova Scotia, Nunavut, Quebec.", "bbox": {"south_latitude": 40.8, "west_longitude": -72, "north_latitude": 84, "east_longitude": -66}}', 2010.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2010_UTM20_HT2_2010', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 20", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 20N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -63, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_2010", "id": {"authority": "EPSG", "code": 9987}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / UTM zone 20 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 66\u00b0W and 60\u00b0W onshore and offshore - New Brunswick, Labrador, Nova Scotia, Nunavut, Prince Edward Island, Quebec.", "bbox": {"south_latitude": 40.04, "west_longitude": -66, "north_latitude": 84, "east_longitude": -60}}', 2010.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2010_UTM21_HT2_2010', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 21", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 21N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -57, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_2010", "id": {"authority": "EPSG", "code": 9987}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / UTM zone 21 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 60\u00b0W and 54\u00b0W - Newfoundland and Labrador; Nunavut; Quebec.", "bbox": {"south_latitude": 38.56, "west_longitude": -60, "north_latitude": 84, "east_longitude": -54}}', 2010.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2010_UTM22_HT2_2010', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 22", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 22N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -51, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD28 height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 1928"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "geoid_model": {"name": "HT2_2010", "id": {"authority": "EPSG", "code": 9987}}, "id": {"authority": "EPSG", "code": 5713}}], "name": "NAD83(CSRS)v7 / UTM zone 22 + CGVD28 height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 54\u00b0W and 48\u00b0W onshore and offshore - Newfoundland and Labrador.", "bbox": {"south_latitude": 39.5, "west_longitude": -54, "north_latitude": 57.65, "east_longitude": -47.99}}', 2010.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_1997_UTM7_CGVD2013_1997', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 7", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 7N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -141, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(1997) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 1997"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 20035}}], "name": "NAD83(CSRS)v7 / UTM zone 7 + CGVD2013a(1997) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada west of 138\u00b0W, onshore and offshore south of 84\u00b0N - British Columbia, Yukon.", "bbox": {"south_latitude": 52.05, "west_longitude": -141.01, "north_latitude": 72.53, "east_longitude": -138}}', 1997.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_1997_UTM8_CGVD2013_1997', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 8", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 8N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -135, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(1997) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 1997"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 20035}}], "name": "NAD83(CSRS)v7 / UTM zone 8 + CGVD2013a(1997) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 138\u00b0W and 132\u00b0W, onshore and offshore south of 84\u00b0N - British Columbia, Northwest Territories, Yukon.", "bbox": {"south_latitude": 48.06, "west_longitude": -138, "north_latitude": 79.42, "east_longitude": -132}}', 1997.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_1997_UTM9_CGVD2013_1997', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 9", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 9N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -129, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(1997) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 1997"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 20035}}], "name": "NAD83(CSRS)v7 / UTM zone 9 + CGVD2013a(1997) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 138\u00b0W and 132\u00b0W, onshore and offshore south of 84\u00b0N - British Columbia, Northwest Territories, Yukon.", "bbox": {"south_latitude": 48.06, "west_longitude": -138, "north_latitude": 79.42, "east_longitude": -132}}', 1997.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_1997_UTM10_CGVD2013_1997', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 10", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 10N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -123, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(1997) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 1997"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 20035}}], "name": "NAD83(CSRS)v7 / UTM zone 10 + CGVD2013a(1997) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 126\u00b0W and 120\u00b0W, onshore and offshore south of 84\u00b0N - British Columbia, Northwest Territories, Yukon.", "bbox": {"south_latitude": 48.13, "west_longitude": -126, "north_latitude": 81.8, "east_longitude": -120}}', 1997.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_1997_UTM11_CGVD2013_1997', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 11", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 11N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -117, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(1997) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 1997"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 20035}}], "name": "NAD83(CSRS)v7 / UTM zone 11 + CGVD2013a(1997) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 120\u00b0W and 114\u00b0W onshore and offshore - Alberta, British Columbia, Northwest Territories, Nunavut.", "bbox": {"south_latitude": 48.99, "west_longitude": -120, "north_latitude": 83.5, "east_longitude": -114}}', 1997.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_1997_UTM12_CGVD2013_1997', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 12", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 12N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -111, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(1997) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 1997"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 20035}}], "name": "NAD83(CSRS)v7 / UTM zone 12 + CGVD2013a(1997) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 114\u00b0W and 108\u00b0W onshore and offshore - Alberta, Northwest Territories, Nunavut, Saskatchewan.", "bbox": {"south_latitude": 48.99, "west_longitude": -114, "north_latitude": 84, "east_longitude": -108}}', 1997.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_1997_UTM13_CGVD2013_1997', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 13", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 13N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -105, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(1997) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 1997"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 20035}}], "name": "NAD83(CSRS)v7 / UTM zone 13 + CGVD2013a(1997) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 108\u00b0W and 102\u00b0W onshore and offshore - Northwest Territories, Nunavut, Saskatchewan.", "bbox": {"south_latitude": 48.99, "west_longitude": -108, "north_latitude": 84, "east_longitude": -102}}', 1997.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_1997_UTM14_CGVD2013_1997', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 14", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 14N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -99, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(1997) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 1997"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 20035}}], "name": "NAD83(CSRS)v7 / UTM zone 14 + CGVD2013a(1997) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 102\u00b0W and 96\u00b0W, onshore and offshore south of 84\u00b0N - Manitoba, Nunavut, Saskatchewan.", "bbox": {"south_latitude": 48.99, "west_longitude": -102, "north_latitude": 84, "east_longitude": -96}}', 1997.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_1997_UTM15_CGVD2013_1997', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 15", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 15N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -93, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(1997) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 1997"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 20035}}], "name": "NAD83(CSRS)v7 / UTM zone 15 + CGVD2013a(1997) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 96\u00b0W and 90\u00b0W, onshore and offshore south of 84\u00b0N - Manitoba, Nunavut, Ontario.", "bbox": {"south_latitude": 48.03, "west_longitude": -96, "north_latitude": 84, "east_longitude": -90}}', 1997.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_1997_UTM16_CGVD2013_1997', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 16", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 16N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -87, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(1997) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 1997"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 20035}}], "name": "NAD83(CSRS)v7 / UTM zone 16 + CGVD2013a(1997) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 90\u00b0W and 84\u00b0W, onshore and offshore south of 84\u00b0N - Manitoba, Nunavut, Ontario.", "bbox": {"south_latitude": 46.11, "west_longitude": -90, "north_latitude": 84, "east_longitude": -84}}', 1997.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_1997_UTM17_CGVD2013_1997', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 17", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 17N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -81, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(1997) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 1997"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 20035}}], "name": "NAD83(CSRS)v7 / UTM zone 17 + CGVD2013a(1997) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 84\u00b0W and 78\u00b0W, onshore and offshore south of 84\u00b0N - Nunavut, Ontario and Quebec.", "bbox": {"south_latitude": 41.67, "west_longitude": -84, "north_latitude": 84, "east_longitude": -78}}', 1997.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_1997_UTM18_CGVD2013_1997', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 18", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 18N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -75, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(1997) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 1997"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 20035}}], "name": "NAD83(CSRS)v7 / UTM zone 18 + CGVD2013a(1997) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 78\u00b0W and 72\u00b0W, onshore and offshore south of 84\u00b0N - Nunavut, Ontario and Quebec.", "bbox": {"south_latitude": 43.63, "west_longitude": -78, "north_latitude": 84, "east_longitude": -72}}', 1997.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_1997_UTM19_CGVD2013_1997', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 19", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 19N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -69, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(1997) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 1997"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 20035}}], "name": "NAD83(CSRS)v7 / UTM zone 19 + CGVD2013a(1997) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 72\u00b0W and 66\u00b0W onshore and offshore - New Brunswick, Labrador, Nova Scotia, Nunavut, Quebec.", "bbox": {"south_latitude": 40.8, "west_longitude": -72, "north_latitude": 84, "east_longitude": -66}}', 1997.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_1997_UTM20_CGVD2013_1997', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 20", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 20N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -63, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(1997) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 1997"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 20035}}], "name": "NAD83(CSRS)v7 / UTM zone 20 + CGVD2013a(1997) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 66\u00b0W and 60\u00b0W onshore and offshore - New Brunswick, Labrador, Nova Scotia, Nunavut, Prince Edward Island, Quebec.", "bbox": {"south_latitude": 40.04, "west_longitude": -66, "north_latitude": 84, "east_longitude": -60}}', 1997.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_1997_UTM21_CGVD2013_1997', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 21", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 21N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -57, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(1997) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 1997"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 20035}}], "name": "NAD83(CSRS)v7 / UTM zone 21 + CGVD2013a(1997) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 60\u00b0W and 54\u00b0W - Newfoundland and Labrador; Nunavut; Quebec.", "bbox": {"south_latitude": 38.56, "west_longitude": -60, "north_latitude": 84, "east_longitude": -54}}', 1997.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_1997_UTM22_CGVD2013_1997', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 22", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 22N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -51, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(1997) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 1997"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 20035}}], "name": "NAD83(CSRS)v7 / UTM zone 22 + CGVD2013a(1997) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 54\u00b0W and 48\u00b0W onshore and offshore - Newfoundland and Labrador.", "bbox": {"south_latitude": 39.5, "west_longitude": -54, "north_latitude": 57.65, "east_longitude": -47.99}}', 1997.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2002_UTM7_CGVD2013_2002', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 7", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 7N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -141, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(2002) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 2002"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 20034}}], "name": "NAD83(CSRS)v7 / UTM zone 7 + CGVD2013a(2002) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada west of 138\u00b0W, onshore and offshore south of 84\u00b0N - British Columbia, Yukon.", "bbox": {"south_latitude": 52.05, "west_longitude": -141.01, "north_latitude": 72.53, "east_longitude": -138}}', 2002.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2002_UTM8_CGVD2013_2002', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 8", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 8N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -135, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(2002) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 2002"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 20034}}], "name": "NAD83(CSRS)v7 / UTM zone 8 + CGVD2013a(2002) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 138\u00b0W and 132\u00b0W, onshore and offshore south of 84\u00b0N - British Columbia, Northwest Territories, Yukon.", "bbox": {"south_latitude": 48.06, "west_longitude": -138, "north_latitude": 79.42, "east_longitude": -132}}', 2002.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2002_UTM9_CGVD2013_2002', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 9", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 9N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -129, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(2002) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 2002"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 20034}}], "name": "NAD83(CSRS)v7 / UTM zone 9 + CGVD2013a(2002) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 138\u00b0W and 132\u00b0W, onshore and offshore south of 84\u00b0N - British Columbia, Northwest Territories, Yukon.", "bbox": {"south_latitude": 48.06, "west_longitude": -138, "north_latitude": 79.42, "east_longitude": -132}}', 2002.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2002_UTM10_CGVD2013_2002', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 10", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 10N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -123, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(2002) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 2002"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 20034}}], "name": "NAD83(CSRS)v7 / UTM zone 10 + CGVD2013a(2002) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 126\u00b0W and 120\u00b0W, onshore and offshore south of 84\u00b0N - British Columbia, Northwest Territories, Yukon.", "bbox": {"south_latitude": 48.13, "west_longitude": -126, "north_latitude": 81.8, "east_longitude": -120}}', 2002.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2002_UTM11_CGVD2013_2002', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 11", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 11N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -117, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(2002) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 2002"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 20034}}], "name": "NAD83(CSRS)v7 / UTM zone 11 + CGVD2013a(2002) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 120\u00b0W and 114\u00b0W onshore and offshore - Alberta, British Columbia, Northwest Territories, Nunavut.", "bbox": {"south_latitude": 48.99, "west_longitude": -120, "north_latitude": 83.5, "east_longitude": -114}}', 2002.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2002_UTM12_CGVD2013_2002', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 12", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 12N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -111, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(2002) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 2002"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 20034}}], "name": "NAD83(CSRS)v7 / UTM zone 12 + CGVD2013a(2002) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 114\u00b0W and 108\u00b0W onshore and offshore - Alberta, Northwest Territories, Nunavut, Saskatchewan.", "bbox": {"south_latitude": 48.99, "west_longitude": -114, "north_latitude": 84, "east_longitude": -108}}', 2002.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2002_UTM13_CGVD2013_2002', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 13", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 13N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -105, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(2002) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 2002"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 20034}}], "name": "NAD83(CSRS)v7 / UTM zone 13 + CGVD2013a(2002) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 108\u00b0W and 102\u00b0W onshore and offshore - Northwest Territories, Nunavut, Saskatchewan.", "bbox": {"south_latitude": 48.99, "west_longitude": -108, "north_latitude": 84, "east_longitude": -102}}', 2002.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2002_UTM14_CGVD2013_2002', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 14", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 14N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -99, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(2002) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 2002"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 20034}}], "name": "NAD83(CSRS)v7 / UTM zone 14 + CGVD2013a(2002) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 102\u00b0W and 96\u00b0W, onshore and offshore south of 84\u00b0N - Manitoba, Nunavut, Saskatchewan.", "bbox": {"south_latitude": 48.99, "west_longitude": -102, "north_latitude": 84, "east_longitude": -96}}', 2002.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2002_UTM15_CGVD2013_2002', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 15", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 15N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -93, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(2002) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 2002"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 20034}}], "name": "NAD83(CSRS)v7 / UTM zone 15 + CGVD2013a(2002) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 96\u00b0W and 90\u00b0W, onshore and offshore south of 84\u00b0N - Manitoba, Nunavut, Ontario.", "bbox": {"south_latitude": 48.03, "west_longitude": -96, "north_latitude": 84, "east_longitude": -90}}', 2002.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2002_UTM16_CGVD2013_2002', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 16", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 16N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -87, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(2002) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 2002"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 20034}}], "name": "NAD83(CSRS)v7 / UTM zone 16 + CGVD2013a(2002) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 90\u00b0W and 84\u00b0W, onshore and offshore south of 84\u00b0N - Manitoba, Nunavut, Ontario.", "bbox": {"south_latitude": 46.11, "west_longitude": -90, "north_latitude": 84, "east_longitude": -84}}', 2002.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2002_UTM17_CGVD2013_2002', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 17", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 17N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -81, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(2002) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 2002"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 20034}}], "name": "NAD83(CSRS)v7 / UTM zone 17 + CGVD2013a(2002) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 84\u00b0W and 78\u00b0W, onshore and offshore south of 84\u00b0N - Nunavut, Ontario and Quebec.", "bbox": {"south_latitude": 41.67, "west_longitude": -84, "north_latitude": 84, "east_longitude": -78}}', 2002.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2002_UTM18_CGVD2013_2002', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 18", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 18N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -75, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(2002) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 2002"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 20034}}], "name": "NAD83(CSRS)v7 / UTM zone 18 + CGVD2013a(2002) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 78\u00b0W and 72\u00b0W, onshore and offshore south of 84\u00b0N - Nunavut, Ontario and Quebec.", "bbox": {"south_latitude": 43.63, "west_longitude": -78, "north_latitude": 84, "east_longitude": -72}}', 2002.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2002_UTM19_CGVD2013_2002', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 19", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 19N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -69, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(2002) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 2002"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 20034}}], "name": "NAD83(CSRS)v7 / UTM zone 19 + CGVD2013a(2002) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 72\u00b0W and 66\u00b0W onshore and offshore - New Brunswick, Labrador, Nova Scotia, Nunavut, Quebec.", "bbox": {"south_latitude": 40.8, "west_longitude": -72, "north_latitude": 84, "east_longitude": -66}}', 2002.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2002_UTM20_CGVD2013_2002', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 20", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 20N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -63, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(2002) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 2002"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 20034}}], "name": "NAD83(CSRS)v7 / UTM zone 20 + CGVD2013a(2002) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 66\u00b0W and 60\u00b0W onshore and offshore - New Brunswick, Labrador, Nova Scotia, Nunavut, Prince Edward Island, Quebec.", "bbox": {"south_latitude": 40.04, "west_longitude": -66, "north_latitude": 84, "east_longitude": -60}}', 2002.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2002_UTM21_CGVD2013_2002', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 21", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 21N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -57, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(2002) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 2002"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 20034}}], "name": "NAD83(CSRS)v7 / UTM zone 21 + CGVD2013a(2002) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 60\u00b0W and 54\u00b0W - Newfoundland and Labrador; Nunavut; Quebec.", "bbox": {"south_latitude": 38.56, "west_longitude": -60, "north_latitude": 84, "east_longitude": -54}}', 2002.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2002_UTM22_CGVD2013_2002', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 22", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 22N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -51, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(2002) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 2002"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 20034}}], "name": "NAD83(CSRS)v7 / UTM zone 22 + CGVD2013a(2002) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 54\u00b0W and 48\u00b0W onshore and offshore - Newfoundland and Labrador.", "bbox": {"south_latitude": 39.5, "west_longitude": -54, "north_latitude": 57.65, "east_longitude": -47.99}}', 2002.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2010_UTM7_CGVD2013_2010', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 7", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 7N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -141, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(2010) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 2010"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 9245}}], "name": "NAD83(CSRS)v7 / UTM zone 7 + CGVD2013a(2010) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada west of 138\u00b0W, onshore and offshore south of 84\u00b0N - British Columbia, Yukon.", "bbox": {"south_latitude": 52.05, "west_longitude": -141.01, "north_latitude": 72.53, "east_longitude": -138}}', 2010.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2010_UTM8_CGVD2013_2010', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 8", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 8N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -135, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(2010) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 2010"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 9245}}], "name": "NAD83(CSRS)v7 / UTM zone 8 + CGVD2013a(2010) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 138\u00b0W and 132\u00b0W, onshore and offshore south of 84\u00b0N - British Columbia, Northwest Territories, Yukon.", "bbox": {"south_latitude": 48.06, "west_longitude": -138, "north_latitude": 79.42, "east_longitude": -132}}', 2010.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2010_UTM9_CGVD2013_2010', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 9", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 9N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -129, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(2010) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 2010"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 9245}}], "name": "NAD83(CSRS)v7 / UTM zone 9 + CGVD2013a(2010) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 138\u00b0W and 132\u00b0W, onshore and offshore south of 84\u00b0N - British Columbia, Northwest Territories, Yukon.", "bbox": {"south_latitude": 48.06, "west_longitude": -138, "north_latitude": 79.42, "east_longitude": -132}}', 2010.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2010_UTM10_CGVD2013_2010', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 10", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 10N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -123, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(2010) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 2010"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 9245}}], "name": "NAD83(CSRS)v7 / UTM zone 10 + CGVD2013a(2010) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 126\u00b0W and 120\u00b0W, onshore and offshore south of 84\u00b0N - British Columbia, Northwest Territories, Yukon.", "bbox": {"south_latitude": 48.13, "west_longitude": -126, "north_latitude": 81.8, "east_longitude": -120}}', 2010.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2010_UTM11_CGVD2013_2010', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 11", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 11N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -117, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(2010) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 2010"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 9245}}], "name": "NAD83(CSRS)v7 / UTM zone 11 + CGVD2013a(2010) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 120\u00b0W and 114\u00b0W onshore and offshore - Alberta, British Columbia, Northwest Territories, Nunavut.", "bbox": {"south_latitude": 48.99, "west_longitude": -120, "north_latitude": 83.5, "east_longitude": -114}}', 2010.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2010_UTM12_CGVD2013_2010', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 12", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 12N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -111, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(2010) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 2010"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 9245}}], "name": "NAD83(CSRS)v7 / UTM zone 12 + CGVD2013a(2010) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 114\u00b0W and 108\u00b0W onshore and offshore - Alberta, Northwest Territories, Nunavut, Saskatchewan.", "bbox": {"south_latitude": 48.99, "west_longitude": -114, "north_latitude": 84, "east_longitude": -108}}', 2010.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2010_UTM13_CGVD2013_2010', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 13", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 13N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -105, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(2010) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 2010"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 9245}}], "name": "NAD83(CSRS)v7 / UTM zone 13 + CGVD2013a(2010) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 108\u00b0W and 102\u00b0W onshore and offshore - Northwest Territories, Nunavut, Saskatchewan.", "bbox": {"south_latitude": 48.99, "west_longitude": -108, "north_latitude": 84, "east_longitude": -102}}', 2010.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2010_UTM14_CGVD2013_2010', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 14", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 14N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -99, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(2010) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 2010"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 9245}}], "name": "NAD83(CSRS)v7 / UTM zone 14 + CGVD2013a(2010) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 102\u00b0W and 96\u00b0W, onshore and offshore south of 84\u00b0N - Manitoba, Nunavut, Saskatchewan.", "bbox": {"south_latitude": 48.99, "west_longitude": -102, "north_latitude": 84, "east_longitude": -96}}', 2010.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2010_UTM15_CGVD2013_2010', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 15", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 15N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -93, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(2010) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 2010"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 9245}}], "name": "NAD83(CSRS)v7 / UTM zone 15 + CGVD2013a(2010) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 96\u00b0W and 90\u00b0W, onshore and offshore south of 84\u00b0N - Manitoba, Nunavut, Ontario.", "bbox": {"south_latitude": 48.03, "west_longitude": -96, "north_latitude": 84, "east_longitude": -90}}', 2010.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2010_UTM16_CGVD2013_2010', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 16", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 16N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -87, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(2010) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 2010"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 9245}}], "name": "NAD83(CSRS)v7 / UTM zone 16 + CGVD2013a(2010) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 90\u00b0W and 84\u00b0W, onshore and offshore south of 84\u00b0N - Manitoba, Nunavut, Ontario.", "bbox": {"south_latitude": 46.11, "west_longitude": -90, "north_latitude": 84, "east_longitude": -84}}', 2010.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2010_UTM17_CGVD2013_2010', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 17", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 17N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -81, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(2010) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 2010"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 9245}}], "name": "NAD83(CSRS)v7 / UTM zone 17 + CGVD2013a(2010) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 84\u00b0W and 78\u00b0W, onshore and offshore south of 84\u00b0N - Nunavut, Ontario and Quebec.", "bbox": {"south_latitude": 41.67, "west_longitude": -84, "north_latitude": 84, "east_longitude": -78}}', 2010.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2010_UTM18_CGVD2013_2010', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 18", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 18N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -75, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(2010) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 2010"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 9245}}], "name": "NAD83(CSRS)v7 / UTM zone 18 + CGVD2013a(2010) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 78\u00b0W and 72\u00b0W, onshore and offshore south of 84\u00b0N - Nunavut, Ontario and Quebec.", "bbox": {"south_latitude": 43.63, "west_longitude": -78, "north_latitude": 84, "east_longitude": -72}}', 2010.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2010_UTM19_CGVD2013_2010', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 19", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 19N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -69, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(2010) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 2010"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 9245}}], "name": "NAD83(CSRS)v7 / UTM zone 19 + CGVD2013a(2010) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 72\u00b0W and 66\u00b0W onshore and offshore - New Brunswick, Labrador, Nova Scotia, Nunavut, Quebec.", "bbox": {"south_latitude": 40.8, "west_longitude": -72, "north_latitude": 84, "east_longitude": -66}}', 2010.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2010_UTM20_CGVD2013_2010', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 20", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 20N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -63, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(2010) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 2010"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 9245}}], "name": "NAD83(CSRS)v7 / UTM zone 20 + CGVD2013a(2010) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 66\u00b0W and 60\u00b0W onshore and offshore - New Brunswick, Labrador, Nova Scotia, Nunavut, Prince Edward Island, Quebec.", "bbox": {"south_latitude": 40.04, "west_longitude": -66, "north_latitude": 84, "east_longitude": -60}}', 2010.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2010_UTM21_CGVD2013_2010', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 21", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 21N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -57, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(2010) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 2010"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 9245}}], "name": "NAD83(CSRS)v7 / UTM zone 21 + CGVD2013a(2010) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 60\u00b0W and 54\u00b0W - Newfoundland and Labrador; Nunavut; Quebec.", "bbox": {"south_latitude": 38.56, "west_longitude": -60, "north_latitude": 84, "east_longitude": -54}}', 2010.0, 0);
INSERT INTO coordinate_metadata VALUES('NRCAN', 'NAD83_CSRS_2010_UTM22_CGVD2013_2010', NULL, NULL, NULL, '{"type": "CompoundCRS", "components": [{"type": "ProjectedCRS", "name": "NAD83(CSRS)v7 / UTM zone 22", "base_crs": {"name": "NAD83(CSRS)v7", "datum": {"type": "GeodeticReferenceFrame", "name": "North American Datum of 1983 (CSRS) version 7", "ellipsoid": {"name": "GRS 1980", "semi_major_axis": 6378137, "inverse_flattening": 298.257222101}}, "coordinate_system": {"subtype": "ellipsoidal", "axis": [{"name": "Geodetic latitude", "abbreviation": "Lat", "direction": "north", "unit": "degree"}, {"name": "Geodetic longitude", "abbreviation": "Lon", "direction": "east", "unit": "degree"}]}}, "conversion": {"name": "UTM zone 22N", "method": {"name": "Transverse Mercator", "id": {"authority": "EPSG", "code": 9807}}, "parameters": [{"name": "Latitude of natural origin", "value": 0, "unit": "degree", "id": {"authority": "EPSG", "code": 8801}}, {"name": "Longitude of natural origin", "value": -51, "unit": "degree", "id": {"authority": "EPSG", "code": 8802}}, {"name": "Scale factor at natural origin", "value": 0.9996, "unit": "unity", "id": {"authority": "EPSG", "code": 8805}}, {"name": "False easting", "value": 500000, "unit": "metre", "id": {"authority": "EPSG", "code": 8806}}, {"name": "False northing", "value": 0, "unit": "metre", "id": {"authority": "EPSG", "code": 8807}}]}, "coordinate_system": {"subtype": "Cartesian", "axis": [{"name": "Easting", "abbreviation": "E(X)", "direction": "east", "unit": "metre"}, {"name": "Northing", "abbreviation": "N(Y)", "direction": "north", "unit": "metre"}]}}, {"type": "VerticalCRS", "name": "CGVD2013a(2010) height", "datum": {"type": "VerticalReferenceFrame", "name": "Canadian Geodetic Vertical Datum of 2013 (CGG2013a) epoch 2010"}, "coordinate_system": {"subtype": "vertical", "axis": [{"name": "Gravity-related height", "abbreviation": "H", "direction": "up", "unit": "metre"}]}, "id": {"authority": "EPSG", "code": 9245}}], "name": "NAD83(CSRS)v7 / UTM zone 22 + CGVD2013a(2010) height", "scope": "Engineering survey, topographic mapping.", "area": "Canada between 54\u00b0W and 48\u00b0W onshore and offshore - Newfoundland and Labrador.", "bbox": {"south_latitude": 39.5, "west_longitude": -54, "north_latitude": 57.65, "east_longitude": -47.99}}', 2010.0, 0);
