--- This file has been generated by scripts/build_db.py. DO NOT EDIT !

INSERT INTO "projected_crs" VALUES('EPSG','2000','Anguilla 1957 / British West Indies Grid',NULL,'EPSG','4400','EPSG','4600','EPSG','19942',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1024','projected_crs','EPSG','2000','EPSG','3214','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2001','Antigua 1943 / British West Indies Grid',NULL,'EPSG','4400','EPSG','4601','EPSG','19942',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1025','projected_crs','EPSG','2001','EPSG','1273','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2002','Dominica 1945 / British West Indies Grid',NULL,'EPSG','4400','EPSG','4602','EPSG','19942',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1026','projected_crs','EPSG','2002','EPSG','3239','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2003','Grenada 1953 / British West Indies Grid',NULL,'EPSG','4400','EPSG','4603','EPSG','19942',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1027','projected_crs','EPSG','2003','EPSG','1551','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2004','Montserrat 1958 / British West Indies Grid',NULL,'EPSG','4400','EPSG','4604','EPSG','19942',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1028','projected_crs','EPSG','2004','EPSG','3279','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2005','St. Kitts 1955 / British West Indies Grid',NULL,'EPSG','4400','EPSG','4605','EPSG','19942',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1029','projected_crs','EPSG','2005','EPSG','3297','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2006','St. Lucia 1955 / British West Indies Grid',NULL,'EPSG','4400','EPSG','4606','EPSG','19942',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1030','projected_crs','EPSG','2006','EPSG','3298','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2007','St. Vincent 45 / British West Indies Grid',NULL,'EPSG','4400','EPSG','4607','EPSG','19942',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1031','projected_crs','EPSG','2007','EPSG','3300','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2008','NAD27(CGQ77) / SCoPQ zone 2',NULL,'EPSG','4499','EPSG','4609','EPSG','17700',NULL,1);
INSERT INTO "usage" VALUES('EPSG','1032','projected_crs','EPSG','2008','EPSG','1420','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2009','NAD27(CGQ77) / SCoPQ zone 3',NULL,'EPSG','4499','EPSG','4609','EPSG','17703',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1033','projected_crs','EPSG','2009','EPSG','1446','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2010','NAD27(CGQ77) / SCoPQ zone 4',NULL,'EPSG','4499','EPSG','4609','EPSG','17704',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1034','projected_crs','EPSG','2010','EPSG','1422','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2011','NAD27(CGQ77) / SCoPQ zone 5',NULL,'EPSG','4499','EPSG','4609','EPSG','17705',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1035','projected_crs','EPSG','2011','EPSG','1423','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2012','NAD27(CGQ77) / SCoPQ zone 6',NULL,'EPSG','4499','EPSG','4609','EPSG','17706',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1036','projected_crs','EPSG','2012','EPSG','1424','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2013','NAD27(CGQ77) / SCoPQ zone 7',NULL,'EPSG','4499','EPSG','4609','EPSG','17707',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1037','projected_crs','EPSG','2013','EPSG','1425','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2014','NAD27(CGQ77) / SCoPQ zone 8',NULL,'EPSG','4499','EPSG','4609','EPSG','17708',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1038','projected_crs','EPSG','2014','EPSG','1426','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2015','NAD27(CGQ77) / SCoPQ zone 9',NULL,'EPSG','4499','EPSG','4609','EPSG','17709',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1039','projected_crs','EPSG','2015','EPSG','1427','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2016','NAD27(CGQ77) / SCoPQ zone 10',NULL,'EPSG','4499','EPSG','4609','EPSG','17710',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1040','projected_crs','EPSG','2016','EPSG','1428','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2017','NAD27(76) / MTM zone 8',NULL,'EPSG','4499','EPSG','4608','EPSG','17708',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1041','projected_crs','EPSG','2017','EPSG','1429','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2018','NAD27(76) / MTM zone 9',NULL,'EPSG','4499','EPSG','4608','EPSG','17709',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1042','projected_crs','EPSG','2018','EPSG','1430','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2019','NAD27(76) / MTM zone 10',NULL,'EPSG','4499','EPSG','4608','EPSG','17710',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1043','projected_crs','EPSG','2019','EPSG','1431','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2020','NAD27(76) / MTM zone 11',NULL,'EPSG','4400','EPSG','4608','EPSG','17711',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1044','projected_crs','EPSG','2020','EPSG','1432','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2021','NAD27(76) / MTM zone 12',NULL,'EPSG','4400','EPSG','4608','EPSG','17712',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1045','projected_crs','EPSG','2021','EPSG','1433','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2022','NAD27(76) / MTM zone 13',NULL,'EPSG','4400','EPSG','4608','EPSG','17713',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1046','projected_crs','EPSG','2022','EPSG','1434','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2023','NAD27(76) / MTM zone 14',NULL,'EPSG','4400','EPSG','4608','EPSG','17714',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1047','projected_crs','EPSG','2023','EPSG','1435','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2024','NAD27(76) / MTM zone 15',NULL,'EPSG','4400','EPSG','4608','EPSG','17715',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1048','projected_crs','EPSG','2024','EPSG','1436','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2025','NAD27(76) / MTM zone 16',NULL,'EPSG','4400','EPSG','4608','EPSG','17716',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1049','projected_crs','EPSG','2025','EPSG','1437','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2026','NAD27(76) / MTM zone 17',NULL,'EPSG','4400','EPSG','4608','EPSG','17717',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1050','projected_crs','EPSG','2026','EPSG','1438','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2027','NAD27(76) / UTM zone 15N',NULL,'EPSG','4400','EPSG','4608','EPSG','16015',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1051','projected_crs','EPSG','2027','EPSG','1439','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2028','NAD27(76) / UTM zone 16N',NULL,'EPSG','4400','EPSG','4608','EPSG','16016',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1052','projected_crs','EPSG','2028','EPSG','1440','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2029','NAD27(76) / UTM zone 17N',NULL,'EPSG','4400','EPSG','4608','EPSG','16017',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1053','projected_crs','EPSG','2029','EPSG','1441','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2030','NAD27(76) / UTM zone 18N',NULL,'EPSG','4400','EPSG','4608','EPSG','16018',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1054','projected_crs','EPSG','2030','EPSG','1442','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2031','NAD27(CGQ77) / UTM zone 17N',NULL,'EPSG','4400','EPSG','4609','EPSG','16017',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1055','projected_crs','EPSG','2031','EPSG','1428','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2032','NAD27(CGQ77) / UTM zone 18N',NULL,'EPSG','4400','EPSG','4609','EPSG','16018',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1056','projected_crs','EPSG','2032','EPSG','1443','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2033','NAD27(CGQ77) / UTM zone 19N',NULL,'EPSG','4400','EPSG','4609','EPSG','16019',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1057','projected_crs','EPSG','2033','EPSG','1444','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2034','NAD27(CGQ77) / UTM zone 20N',NULL,'EPSG','4400','EPSG','4609','EPSG','16020',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1058','projected_crs','EPSG','2034','EPSG','1445','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2035','NAD27(CGQ77) / UTM zone 21N',NULL,'EPSG','4400','EPSG','4609','EPSG','16021',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1059','projected_crs','EPSG','2035','EPSG','1446','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2036','NAD83(CSRS98) / New Brunswick Stereo',NULL,'EPSG','4500','EPSG','4140','EPSG','19946',NULL,1);
INSERT INTO "usage" VALUES('EPSG','1060','projected_crs','EPSG','2036','EPSG','1447','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2037','NAD83(CSRS98) / UTM zone 19N',NULL,'EPSG','4400','EPSG','4140','EPSG','16019',NULL,1);
INSERT INTO "usage" VALUES('EPSG','1061','projected_crs','EPSG','2037','EPSG','1448','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2038','NAD83(CSRS98) / UTM zone 20N',NULL,'EPSG','4400','EPSG','4140','EPSG','16020',NULL,1);
INSERT INTO "usage" VALUES('EPSG','1062','projected_crs','EPSG','2038','EPSG','1449','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2039','Israel 1993 / Israeli TM Grid',NULL,'EPSG','4400','EPSG','4141','EPSG','18204',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1063','projected_crs','EPSG','2039','EPSG','2603','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','2040','Locodjo 1965 / UTM zone 30N',NULL,'EPSG','4400','EPSG','4142','EPSG','16030',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1064','projected_crs','EPSG','2040','EPSG','1450','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2041','Abidjan 1987 / UTM zone 30N',NULL,'EPSG','4400','EPSG','4143','EPSG','16030',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1065','projected_crs','EPSG','2041','EPSG','1450','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2042','Locodjo 1965 / UTM zone 29N',NULL,'EPSG','4400','EPSG','4142','EPSG','16029',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1066','projected_crs','EPSG','2042','EPSG','1451','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2043','Abidjan 1987 / UTM zone 29N',NULL,'EPSG','4400','EPSG','4143','EPSG','16029',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1067','projected_crs','EPSG','2043','EPSG','1451','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2044','Hanoi 1972 / Gauss-Kruger zone 18',NULL,'EPSG','4530','EPSG','4147','EPSG','16218',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1068','projected_crs','EPSG','2044','EPSG','1452','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2045','Hanoi 1972 / Gauss-Kruger zone 19',NULL,'EPSG','4530','EPSG','4147','EPSG','16219',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1069','projected_crs','EPSG','2045','EPSG','1453','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2046','Hartebeesthoek94 / Lo15',NULL,'EPSG','6503','EPSG','4148','EPSG','17515',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1070','projected_crs','EPSG','2046','EPSG','1454','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','2047','Hartebeesthoek94 / Lo17',NULL,'EPSG','6503','EPSG','4148','EPSG','17517',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1071','projected_crs','EPSG','2047','EPSG','1455','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','2048','Hartebeesthoek94 / Lo19',NULL,'EPSG','6503','EPSG','4148','EPSG','17519',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1072','projected_crs','EPSG','2048','EPSG','1456','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','2049','Hartebeesthoek94 / Lo21',NULL,'EPSG','6503','EPSG','4148','EPSG','17521',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1073','projected_crs','EPSG','2049','EPSG','1457','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','2050','Hartebeesthoek94 / Lo23',NULL,'EPSG','6503','EPSG','4148','EPSG','17523',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1074','projected_crs','EPSG','2050','EPSG','1458','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','2051','Hartebeesthoek94 / Lo25',NULL,'EPSG','6503','EPSG','4148','EPSG','17525',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1075','projected_crs','EPSG','2051','EPSG','1459','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','2052','Hartebeesthoek94 / Lo27',NULL,'EPSG','6503','EPSG','4148','EPSG','17527',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1076','projected_crs','EPSG','2052','EPSG','1460','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','2053','Hartebeesthoek94 / Lo29',NULL,'EPSG','6503','EPSG','4148','EPSG','17529',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1077','projected_crs','EPSG','2053','EPSG','1461','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','2054','Hartebeesthoek94 / Lo31',NULL,'EPSG','6503','EPSG','4148','EPSG','17531',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1078','projected_crs','EPSG','2054','EPSG','1462','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','2055','Hartebeesthoek94 / Lo33',NULL,'EPSG','6503','EPSG','4148','EPSG','17533',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1079','projected_crs','EPSG','2055','EPSG','1463','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','2056','CH1903+ / LV95',NULL,'EPSG','4400','EPSG','4150','EPSG','19950',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1080','projected_crs','EPSG','2056','EPSG','1286','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','2057','Rassadiran / Nakhl e Taqi',NULL,'EPSG','4400','EPSG','4153','EPSG','19951',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1081','projected_crs','EPSG','2057','EPSG','1338','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','2058','ED50(ED77) / UTM zone 38N',NULL,'EPSG','4400','EPSG','4154','EPSG','16038',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1082','projected_crs','EPSG','2058','EPSG','1464','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2059','ED50(ED77) / UTM zone 39N',NULL,'EPSG','4400','EPSG','4154','EPSG','16039',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1083','projected_crs','EPSG','2059','EPSG','1465','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2060','ED50(ED77) / UTM zone 40N',NULL,'EPSG','4400','EPSG','4154','EPSG','16040',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1084','projected_crs','EPSG','2060','EPSG','1466','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2061','ED50(ED77) / UTM zone 41N',NULL,'EPSG','4400','EPSG','4154','EPSG','16041',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1085','projected_crs','EPSG','2061','EPSG','1467','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2062','Madrid 1870 (Madrid) / Spain LCC',NULL,'EPSG','4499','EPSG','4903','EPSG','19921',NULL,0);
INSERT INTO "usage" VALUES('EPSG','14190','projected_crs','EPSG','2062','EPSG','2366','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2063','Dabola 1981 / UTM zone 28N',NULL,'EPSG','4400','EPSG','4315','EPSG','16028',NULL,1);
INSERT INTO "usage" VALUES('EPSG','1087','projected_crs','EPSG','2063','EPSG','1468','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2064','Dabola 1981 / UTM zone 29N',NULL,'EPSG','4400','EPSG','4315','EPSG','16029',NULL,1);
INSERT INTO "usage" VALUES('EPSG','1088','projected_crs','EPSG','2064','EPSG','1469','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2065','S-JTSK (Ferro) / Krovak',NULL,'EPSG','6501','EPSG','4818','EPSG','19952',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1089','projected_crs','EPSG','2065','EPSG','1306','EPSG','1094');
INSERT INTO "projected_crs" VALUES('EPSG','2066','Mount Dillon / Tobago Grid',NULL,'EPSG','4407','EPSG','4157','EPSG','19924',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1090','projected_crs','EPSG','2066','EPSG','1322','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2067','Naparima 1955 / UTM zone 20N',NULL,'EPSG','4400','EPSG','4158','EPSG','16020',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1091','projected_crs','EPSG','2067','EPSG','3143','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2068','ELD79 / Libya zone 5',NULL,'EPSG','4499','EPSG','4159','EPSG','18240',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1092','projected_crs','EPSG','2068','EPSG','1470','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2069','ELD79 / Libya zone 6',NULL,'EPSG','4499','EPSG','4159','EPSG','18241',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1093','projected_crs','EPSG','2069','EPSG','1471','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2070','ELD79 / Libya zone 7',NULL,'EPSG','4499','EPSG','4159','EPSG','18242',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1094','projected_crs','EPSG','2070','EPSG','1472','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2071','ELD79 / Libya zone 8',NULL,'EPSG','4499','EPSG','4159','EPSG','18243',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1095','projected_crs','EPSG','2071','EPSG','1473','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2072','ELD79 / Libya zone 9',NULL,'EPSG','4499','EPSG','4159','EPSG','18244',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1096','projected_crs','EPSG','2072','EPSG','1474','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2073','ELD79 / Libya zone 10',NULL,'EPSG','4499','EPSG','4159','EPSG','18245',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1097','projected_crs','EPSG','2073','EPSG','1475','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2074','ELD79 / Libya zone 11',NULL,'EPSG','4499','EPSG','4159','EPSG','18246',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1098','projected_crs','EPSG','2074','EPSG','1476','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2075','ELD79 / Libya zone 12',NULL,'EPSG','4499','EPSG','4159','EPSG','18247',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1099','projected_crs','EPSG','2075','EPSG','1477','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2076','ELD79 / Libya zone 13',NULL,'EPSG','4499','EPSG','4159','EPSG','18248',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1100','projected_crs','EPSG','2076','EPSG','1478','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2077','ELD79 / UTM zone 32N',NULL,'EPSG','4400','EPSG','4159','EPSG','16032',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1101','projected_crs','EPSG','2077','EPSG','1479','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2078','ELD79 / UTM zone 33N',NULL,'EPSG','4400','EPSG','4159','EPSG','16033',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1102','projected_crs','EPSG','2078','EPSG','1480','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2079','ELD79 / UTM zone 34N',NULL,'EPSG','4400','EPSG','4159','EPSG','16034',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1103','projected_crs','EPSG','2079','EPSG','1481','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2080','ELD79 / UTM zone 35N',NULL,'EPSG','4400','EPSG','4159','EPSG','16035',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1104','projected_crs','EPSG','2080','EPSG','1478','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2081','Chos Malal 1914 / Argentina 2',NULL,'EPSG','4530','EPSG','4160','EPSG','18032',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1105','projected_crs','EPSG','2081','EPSG','1483','EPSG','1136');
INSERT INTO "projected_crs" VALUES('EPSG','2082','Pampa del Castillo / Argentina 2',NULL,'EPSG','4530','EPSG','4161','EPSG','18032',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1106','projected_crs','EPSG','2082','EPSG','1484','EPSG','1136');
INSERT INTO "projected_crs" VALUES('EPSG','2083','Hito XVIII 1963 / Argentina 2',NULL,'EPSG','4530','EPSG','4254','EPSG','18032',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1107','projected_crs','EPSG','2083','EPSG','1485','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2084','Hito XVIII 1963 / UTM zone 19S',NULL,'EPSG','4400','EPSG','4254','EPSG','16119',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1108','projected_crs','EPSG','2084','EPSG','2596','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2085','NAD27 / Cuba Norte',NULL,'EPSG','4532','EPSG','4267','EPSG','18061',NULL,1);
INSERT INTO "usage" VALUES('EPSG','1109','projected_crs','EPSG','2085','EPSG','1487','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2086','NAD27 / Cuba Sur',NULL,'EPSG','4532','EPSG','4267','EPSG','18062',NULL,1);
INSERT INTO "usage" VALUES('EPSG','1110','projected_crs','EPSG','2086','EPSG','1488','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2087','ELD79 / TM 12 NE',NULL,'EPSG','4400','EPSG','4159','EPSG','16412',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1111','projected_crs','EPSG','2087','EPSG','1482','EPSG','1136');
INSERT INTO "projected_crs" VALUES('EPSG','2088','Carthage / TM 11 NE',NULL,'EPSG','4400','EPSG','4223','EPSG','16411',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1112','projected_crs','EPSG','2088','EPSG','1489','EPSG','1136');
INSERT INTO "projected_crs" VALUES('EPSG','2089','Yemen NGN96 / UTM zone 38N',NULL,'EPSG','4400','EPSG','4163','EPSG','16038',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1113','projected_crs','EPSG','2089','EPSG','1490','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2090','Yemen NGN96 / UTM zone 39N',NULL,'EPSG','4400','EPSG','4163','EPSG','16039',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1114','projected_crs','EPSG','2090','EPSG','1491','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2091','South Yemen / Gauss Kruger zone 8',NULL,'EPSG','4530','EPSG','4164','EPSG','16208',NULL,1);
INSERT INTO "usage" VALUES('EPSG','1115','projected_crs','EPSG','2091','EPSG','1492','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2092','South Yemen / Gauss Kruger zone 9',NULL,'EPSG','4530','EPSG','4164','EPSG','16209',NULL,1);
INSERT INTO "usage" VALUES('EPSG','1116','projected_crs','EPSG','2092','EPSG','1493','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2093','Hanoi 1972 / GK 106 NE',NULL,'EPSG','4530','EPSG','4147','EPSG','16586',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1117','projected_crs','EPSG','2093','EPSG','1494','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2094','WGS 72BE / TM 106 NE',NULL,'EPSG','4400','EPSG','4324','EPSG','16506',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1118','projected_crs','EPSG','2094','EPSG','1495','EPSG','1136');
INSERT INTO "projected_crs" VALUES('EPSG','2095','Bissau / UTM zone 28N',NULL,'EPSG','4400','EPSG','4165','EPSG','16028',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1119','projected_crs','EPSG','2095','EPSG','3258','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2096','Korean 1985 / East Belt',NULL,'EPSG','4530','EPSG','4162','EPSG','18251',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1120','projected_crs','EPSG','2096','EPSG','1496','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2097','Korean 1985 / Central Belt',NULL,'EPSG','4530','EPSG','4162','EPSG','18252',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1121','projected_crs','EPSG','2097','EPSG','3730','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2098','Korean 1985 / West Belt',NULL,'EPSG','4530','EPSG','4162','EPSG','18253',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1122','projected_crs','EPSG','2098','EPSG','1498','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2099','Qatar 1948 / Qatar Grid',NULL,'EPSG','4400','EPSG','4286','EPSG','19953',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1123','projected_crs','EPSG','2099','EPSG','1346','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2100','GGRS87 / Greek Grid',NULL,'EPSG','4400','EPSG','4121','EPSG','19930',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1124','projected_crs','EPSG','2100','EPSG','3254','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2101','Lake / Maracaibo Grid M1',NULL,'EPSG','4499','EPSG','4249','EPSG','18260',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1125','projected_crs','EPSG','2101','EPSG','1319','EPSG','1136');
INSERT INTO "projected_crs" VALUES('EPSG','2102','Lake / Maracaibo Grid',NULL,'EPSG','4499','EPSG','4249','EPSG','18261',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1126','projected_crs','EPSG','2102','EPSG','1319','EPSG','1136');
INSERT INTO "projected_crs" VALUES('EPSG','2103','Lake / Maracaibo Grid M3',NULL,'EPSG','4499','EPSG','4249','EPSG','18262',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1127','projected_crs','EPSG','2103','EPSG','1319','EPSG','1136');
INSERT INTO "projected_crs" VALUES('EPSG','2104','Lake / Maracaibo La Rosa Grid',NULL,'EPSG','4499','EPSG','4249','EPSG','18263',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1128','projected_crs','EPSG','2104','EPSG','1499','EPSG','1136');
INSERT INTO "projected_crs" VALUES('EPSG','2105','NZGD2000 / Mount Eden 2000',NULL,'EPSG','4500','EPSG','4167','EPSG','17931',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1129','projected_crs','EPSG','2105','EPSG','3781','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','2106','NZGD2000 / Bay of Plenty 2000',NULL,'EPSG','4500','EPSG','4167','EPSG','17932',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1130','projected_crs','EPSG','2106','EPSG','3779','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','2107','NZGD2000 / Poverty Bay 2000',NULL,'EPSG','4500','EPSG','4167','EPSG','17933',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1131','projected_crs','EPSG','2107','EPSG','3780','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','2108','NZGD2000 / Hawkes Bay 2000',NULL,'EPSG','4500','EPSG','4167','EPSG','17934',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1132','projected_crs','EPSG','2108','EPSG','3772','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','2109','NZGD2000 / Taranaki 2000',NULL,'EPSG','4500','EPSG','4167','EPSG','17935',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1133','projected_crs','EPSG','2109','EPSG','3777','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','2110','NZGD2000 / Tuhirangi 2000',NULL,'EPSG','4500','EPSG','4167','EPSG','17936',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1134','projected_crs','EPSG','2110','EPSG','3778','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','2111','NZGD2000 / Wanganui 2000',NULL,'EPSG','4500','EPSG','4167','EPSG','17937',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1135','projected_crs','EPSG','2111','EPSG','3776','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','2112','NZGD2000 / Wairarapa 2000',NULL,'EPSG','4500','EPSG','4167','EPSG','17938',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1136','projected_crs','EPSG','2112','EPSG','3775','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','2113','NZGD2000 / Wellington 2000',NULL,'EPSG','4500','EPSG','4167','EPSG','17939',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1137','projected_crs','EPSG','2113','EPSG','3774','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','2114','NZGD2000 / Collingwood 2000',NULL,'EPSG','4500','EPSG','4167','EPSG','17940',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1138','projected_crs','EPSG','2114','EPSG','3782','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','2115','NZGD2000 / Nelson 2000',NULL,'EPSG','4500','EPSG','4167','EPSG','17941',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1139','projected_crs','EPSG','2115','EPSG','3784','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','2116','NZGD2000 / Karamea 2000',NULL,'EPSG','4500','EPSG','4167','EPSG','17942',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1140','projected_crs','EPSG','2116','EPSG','3783','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','2117','NZGD2000 / Buller 2000',NULL,'EPSG','4500','EPSG','4167','EPSG','17943',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1141','projected_crs','EPSG','2117','EPSG','3786','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','2118','NZGD2000 / Grey 2000',NULL,'EPSG','4500','EPSG','4167','EPSG','17944',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1142','projected_crs','EPSG','2118','EPSG','3787','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','2119','NZGD2000 / Amuri 2000',NULL,'EPSG','4500','EPSG','4167','EPSG','17945',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1143','projected_crs','EPSG','2119','EPSG','3788','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','2120','NZGD2000 / Marlborough 2000',NULL,'EPSG','4500','EPSG','4167','EPSG','17946',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1144','projected_crs','EPSG','2120','EPSG','3785','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','2121','NZGD2000 / Hokitika 2000',NULL,'EPSG','4500','EPSG','4167','EPSG','17947',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1145','projected_crs','EPSG','2121','EPSG','3789','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','2122','NZGD2000 / Okarito 2000',NULL,'EPSG','4500','EPSG','4167','EPSG','17948',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1146','projected_crs','EPSG','2122','EPSG','3791','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','2123','NZGD2000 / Jacksons Bay 2000',NULL,'EPSG','4500','EPSG','4167','EPSG','17949',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1147','projected_crs','EPSG','2123','EPSG','3794','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','2124','NZGD2000 / Mount Pleasant 2000',NULL,'EPSG','4500','EPSG','4167','EPSG','17950',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1148','projected_crs','EPSG','2124','EPSG','3790','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','2125','NZGD2000 / Gawler 2000',NULL,'EPSG','4500','EPSG','4167','EPSG','17951',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1149','projected_crs','EPSG','2125','EPSG','3792','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','2126','NZGD2000 / Timaru 2000',NULL,'EPSG','4500','EPSG','4167','EPSG','17952',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1150','projected_crs','EPSG','2126','EPSG','3793','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','2127','NZGD2000 / Lindis Peak 2000',NULL,'EPSG','4500','EPSG','4167','EPSG','17953',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1151','projected_crs','EPSG','2127','EPSG','3795','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','2128','NZGD2000 / Mount Nicholas 2000',NULL,'EPSG','4500','EPSG','4167','EPSG','17954',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1152','projected_crs','EPSG','2128','EPSG','3797','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','2129','NZGD2000 / Mount York 2000',NULL,'EPSG','4500','EPSG','4167','EPSG','17955',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1153','projected_crs','EPSG','2129','EPSG','3799','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','2130','NZGD2000 / Observation Point 2000',NULL,'EPSG','4500','EPSG','4167','EPSG','17956',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1154','projected_crs','EPSG','2130','EPSG','3796','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','2131','NZGD2000 / North Taieri 2000',NULL,'EPSG','4500','EPSG','4167','EPSG','17957',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1155','projected_crs','EPSG','2131','EPSG','3798','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','2132','NZGD2000 / Bluff 2000',NULL,'EPSG','4500','EPSG','4167','EPSG','17958',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1156','projected_crs','EPSG','2132','EPSG','3800','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','2133','NZGD2000 / UTM zone 58S',NULL,'EPSG','4400','EPSG','4167','EPSG','16158',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1157','projected_crs','EPSG','2133','EPSG','1502','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2134','NZGD2000 / UTM zone 59S',NULL,'EPSG','4400','EPSG','4167','EPSG','16159',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1158','projected_crs','EPSG','2134','EPSG','1503','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2135','NZGD2000 / UTM zone 60S',NULL,'EPSG','4400','EPSG','4167','EPSG','16160',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1159','projected_crs','EPSG','2135','EPSG','1504','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2136','Accra / Ghana National Grid',NULL,'EPSG','4404','EPSG','4168','EPSG','19959',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1160','projected_crs','EPSG','2136','EPSG','3252','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2137','Accra / TM 1 NW',NULL,'EPSG','4400','EPSG','4168','EPSG','17001',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1161','projected_crs','EPSG','2137','EPSG','1505','EPSG','1136');
INSERT INTO "projected_crs" VALUES('EPSG','2138','NAD27(CGQ77) / Quebec Lambert',NULL,'EPSG','4499','EPSG','4609','EPSG','19944',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1162','projected_crs','EPSG','2138','EPSG','1368','EPSG','1210');
INSERT INTO "projected_crs" VALUES('EPSG','2139','NAD83(CSRS98) / SCoPQ zone 2',NULL,'EPSG','4499','EPSG','4140','EPSG','17700',NULL,1);
INSERT INTO "usage" VALUES('EPSG','1163','projected_crs','EPSG','2139','EPSG','1420','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2140','NAD83(CSRS98) / MTM zone 3',NULL,'EPSG','4496','EPSG','4140','EPSG','17703',NULL,1);
INSERT INTO "usage" VALUES('EPSG','1164','projected_crs','EPSG','2140','EPSG','1421','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2141','NAD83(CSRS98) / MTM zone 4',NULL,'EPSG','4496','EPSG','4140','EPSG','17704',NULL,1);
INSERT INTO "usage" VALUES('EPSG','1165','projected_crs','EPSG','2141','EPSG','1422','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2142','NAD83(CSRS98) / MTM zone 5',NULL,'EPSG','4496','EPSG','4140','EPSG','17705',NULL,1);
INSERT INTO "usage" VALUES('EPSG','1166','projected_crs','EPSG','2142','EPSG','1423','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2143','NAD83(CSRS98) / MTM zone 6',NULL,'EPSG','4496','EPSG','4140','EPSG','17706',NULL,1);
INSERT INTO "usage" VALUES('EPSG','1167','projected_crs','EPSG','2143','EPSG','1424','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2144','NAD83(CSRS98) / MTM zone 7',NULL,'EPSG','4496','EPSG','4140','EPSG','17707',NULL,1);
INSERT INTO "usage" VALUES('EPSG','1168','projected_crs','EPSG','2144','EPSG','1425','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2145','NAD83(CSRS98) / MTM zone 8',NULL,'EPSG','4496','EPSG','4140','EPSG','17708',NULL,1);
INSERT INTO "usage" VALUES('EPSG','1169','projected_crs','EPSG','2145','EPSG','1426','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2146','NAD83(CSRS98) / MTM zone 9',NULL,'EPSG','4496','EPSG','4140','EPSG','17709',NULL,1);
INSERT INTO "usage" VALUES('EPSG','1170','projected_crs','EPSG','2146','EPSG','1427','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2147','NAD83(CSRS98) / MTM zone 10',NULL,'EPSG','4496','EPSG','4140','EPSG','17710',NULL,1);
INSERT INTO "usage" VALUES('EPSG','1171','projected_crs','EPSG','2147','EPSG','1428','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2148','NAD83(CSRS98) / UTM zone 21N',NULL,'EPSG','4400','EPSG','4140','EPSG','16021',NULL,1);
INSERT INTO "usage" VALUES('EPSG','1172','projected_crs','EPSG','2148','EPSG','1446','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2149','NAD83(CSRS98) / UTM zone 18N',NULL,'EPSG','4400','EPSG','4140','EPSG','16018',NULL,1);
INSERT INTO "usage" VALUES('EPSG','1173','projected_crs','EPSG','2149','EPSG','1443','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2150','NAD83(CSRS98) / UTM zone 17N',NULL,'EPSG','4400','EPSG','4140','EPSG','16017',NULL,1);
INSERT INTO "usage" VALUES('EPSG','1174','projected_crs','EPSG','2150','EPSG','1428','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2151','NAD83(CSRS98) / UTM zone 13N',NULL,'EPSG','4400','EPSG','4140','EPSG','16013',NULL,1);
INSERT INTO "usage" VALUES('EPSG','1175','projected_crs','EPSG','2151','EPSG','1506','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2152','NAD83(CSRS98) / UTM zone 12N',NULL,'EPSG','4400','EPSG','4140','EPSG','16012',NULL,1);
INSERT INTO "usage" VALUES('EPSG','1176','projected_crs','EPSG','2152','EPSG','1507','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2153','NAD83(CSRS98) / UTM zone 11N',NULL,'EPSG','4400','EPSG','4140','EPSG','16011',NULL,1);
INSERT INTO "usage" VALUES('EPSG','1177','projected_crs','EPSG','2153','EPSG','1508','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2154','RGF93 v1 / Lambert-93',NULL,'EPSG','4499','EPSG','4171','EPSG','18085',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1178','projected_crs','EPSG','2154','EPSG','1096','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2155','American Samoa 1962 / American Samoa Lambert',NULL,'EPSG','4497','EPSG','4169','EPSG','15300',NULL,1);
INSERT INTO "usage" VALUES('EPSG','1179','projected_crs','EPSG','2155','EPSG','1027','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2156','NAD83(HARN) / UTM zone 59S',NULL,'EPSG','4400','EPSG','4152','EPSG','16159',NULL,1);
INSERT INTO "usage" VALUES('EPSG','1180','projected_crs','EPSG','2156','EPSG','1027','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2157','IRENET95 / Irish Transverse Mercator',NULL,'EPSG','4400','EPSG','4173','EPSG','19962',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1181','projected_crs','EPSG','2157','EPSG','1305','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2158','IRENET95 / UTM zone 29N',NULL,'EPSG','4400','EPSG','4173','EPSG','16029',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1182','projected_crs','EPSG','2158','EPSG','1305','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2159','Sierra Leone 1924 / New Colony Grid',NULL,'EPSG','4404','EPSG','4174','EPSG','19963',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1183','projected_crs','EPSG','2159','EPSG','1342','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2160','Sierra Leone 1924 / New War Office Grid',NULL,'EPSG','4404','EPSG','4174','EPSG','19964',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1184','projected_crs','EPSG','2160','EPSG','1342','EPSG','1153');
INSERT INTO "projected_crs" VALUES('EPSG','2161','Sierra Leone 1968 / UTM zone 28N',NULL,'EPSG','4400','EPSG','4175','EPSG','16028',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1185','projected_crs','EPSG','2161','EPSG','1509','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2162','Sierra Leone 1968 / UTM zone 29N',NULL,'EPSG','4400','EPSG','4175','EPSG','16029',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1186','projected_crs','EPSG','2162','EPSG','1510','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2163','US National Atlas Equal Area',NULL,'EPSG','4499','EPSG','4052','EPSG','3899',NULL,1);
INSERT INTO "usage" VALUES('EPSG','1187','projected_crs','EPSG','2163','EPSG','1245','EPSG','1162');
INSERT INTO "projected_crs" VALUES('EPSG','2164','Locodjo 1965 / TM 5 NW',NULL,'EPSG','4400','EPSG','4142','EPSG','17005',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1188','projected_crs','EPSG','2164','EPSG','2296','EPSG','1136');
INSERT INTO "projected_crs" VALUES('EPSG','2165','Abidjan 1987 / TM 5 NW',NULL,'EPSG','4400','EPSG','4143','EPSG','17005',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1189','projected_crs','EPSG','2165','EPSG','2296','EPSG','1136');
INSERT INTO "projected_crs" VALUES('EPSG','2166','Pulkovo 1942(83) / Gauss Kruger zone 3',NULL,'EPSG','4530','EPSG','4178','EPSG','16263',NULL,1);
INSERT INTO "usage" VALUES('EPSG','1190','projected_crs','EPSG','2166','EPSG','1512','EPSG','1056');
INSERT INTO "projected_crs" VALUES('EPSG','2167','Pulkovo 1942(83) / Gauss Kruger zone 4',NULL,'EPSG','4530','EPSG','4178','EPSG','16264',NULL,1);
INSERT INTO "usage" VALUES('EPSG','1191','projected_crs','EPSG','2167','EPSG','1513','EPSG','1056');
INSERT INTO "projected_crs" VALUES('EPSG','2168','Pulkovo 1942(83) / Gauss Kruger zone 5',NULL,'EPSG','4530','EPSG','4178','EPSG','16265',NULL,1);
INSERT INTO "usage" VALUES('EPSG','1192','projected_crs','EPSG','2168','EPSG','1512','EPSG','1056');
INSERT INTO "projected_crs" VALUES('EPSG','2169','LUREF / Luxembourg TM',NULL,'EPSG','4530','EPSG','4181','EPSG','19966',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1193','projected_crs','EPSG','2169','EPSG','1146','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2170','MGI / Slovenia Grid',NULL,'EPSG','4530','EPSG','4312','EPSG','19967',NULL,1);
INSERT INTO "usage" VALUES('EPSG','1194','projected_crs','EPSG','2170','EPSG','1212','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2171','Pulkovo 1942(58) / Poland zone I',NULL,'EPSG','4530','EPSG','4179','EPSG','18281',NULL,1);
INSERT INTO "usage" VALUES('EPSG','1195','projected_crs','EPSG','2171','EPSG','1515','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2172','Pulkovo 1942(58) / Poland zone II',NULL,'EPSG','4530','EPSG','4179','EPSG','18282',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1196','projected_crs','EPSG','2172','EPSG','1516','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2173','Pulkovo 1942(58) / Poland zone III',NULL,'EPSG','4530','EPSG','4179','EPSG','18283',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1197','projected_crs','EPSG','2173','EPSG','1517','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2174','Pulkovo 1942(58) / Poland zone IV',NULL,'EPSG','4530','EPSG','4179','EPSG','18284',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1198','projected_crs','EPSG','2174','EPSG','1518','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2175','Pulkovo 1942(58) / Poland zone V',NULL,'EPSG','4530','EPSG','4179','EPSG','18285',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1199','projected_crs','EPSG','2175','EPSG','1519','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2176','ETRF2000-PL / CS2000/15',NULL,'EPSG','4531','EPSG','9702','EPSG','18305',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1200','projected_crs','EPSG','2176','EPSG','1520','EPSG','1061');
INSERT INTO "projected_crs" VALUES('EPSG','2177','ETRF2000-PL / CS2000/18',NULL,'EPSG','4531','EPSG','9702','EPSG','18306',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1201','projected_crs','EPSG','2177','EPSG','1521','EPSG','1061');
INSERT INTO "projected_crs" VALUES('EPSG','2178','ETRF2000-PL / CS2000/21',NULL,'EPSG','4531','EPSG','9702','EPSG','18307',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1202','projected_crs','EPSG','2178','EPSG','1522','EPSG','1061');
INSERT INTO "projected_crs" VALUES('EPSG','2179','ETRF2000-PL / CS2000/24',NULL,'EPSG','4531','EPSG','9702','EPSG','18308',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1203','projected_crs','EPSG','2179','EPSG','1523','EPSG','1061');
INSERT INTO "projected_crs" VALUES('EPSG','2180','ETRF2000-PL / CS92',NULL,'EPSG','4531','EPSG','9702','EPSG','18300',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1204','projected_crs','EPSG','2180','EPSG','1192','EPSG','1210');
INSERT INTO "projected_crs" VALUES('EPSG','2188','Azores Occidental 1939 / UTM zone 25N',NULL,'EPSG','4400','EPSG','4182','EPSG','16025',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1205','projected_crs','EPSG','2188','EPSG','1344','EPSG','1153');
INSERT INTO "projected_crs" VALUES('EPSG','2189','Azores Central 1948 / UTM zone 26N',NULL,'EPSG','4400','EPSG','4183','EPSG','16026',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1206','projected_crs','EPSG','2189','EPSG','1301','EPSG','1153');
INSERT INTO "projected_crs" VALUES('EPSG','2190','Azores Oriental 1940 / UTM zone 26N',NULL,'EPSG','4400','EPSG','4184','EPSG','16026',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1207','projected_crs','EPSG','2190','EPSG','1345','EPSG','1153');
INSERT INTO "projected_crs" VALUES('EPSG','2191','Madeira 1936 / UTM zone 28N',NULL,'EPSG','4400','EPSG','4185','EPSG','16028',NULL,1);
INSERT INTO "usage" VALUES('EPSG','1208','projected_crs','EPSG','2191','EPSG','1314','EPSG','1153');
INSERT INTO "projected_crs" VALUES('EPSG','2192','ED50 / France EuroLambert',NULL,'EPSG','4499','EPSG','4230','EPSG','18086',NULL,1);
INSERT INTO "usage" VALUES('EPSG','1209','projected_crs','EPSG','2192','EPSG','1326','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2193','NZGD2000 / New Zealand Transverse Mercator 2000',NULL,'EPSG','4500','EPSG','4167','EPSG','19971',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1210','projected_crs','EPSG','2193','EPSG','3973','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2194','American Samoa 1962 / American Samoa Lambert',NULL,'EPSG','4497','EPSG','4169','EPSG','15301',NULL,1);
INSERT INTO "usage" VALUES('EPSG','1211','projected_crs','EPSG','2194','EPSG','1027','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2195','NAD83(HARN) / UTM zone 2S',NULL,'EPSG','4400','EPSG','4152','EPSG','16102',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1212','projected_crs','EPSG','2195','EPSG','3110','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2196','ETRS89 / Kp2000 Jutland',NULL,'EPSG','4400','EPSG','4258','EPSG','18401',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1213','projected_crs','EPSG','2196','EPSG','2531','EPSG','1207');
INSERT INTO "projected_crs" VALUES('EPSG','2197','ETRS89 / Kp2000 Zealand',NULL,'EPSG','4400','EPSG','4258','EPSG','18402',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1214','projected_crs','EPSG','2197','EPSG','2532','EPSG','1207');
INSERT INTO "projected_crs" VALUES('EPSG','2198','ETRS89 / Kp2000 Bornholm',NULL,'EPSG','4400','EPSG','4258','EPSG','18403',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1215','projected_crs','EPSG','2198','EPSG','2533','EPSG','1207');
INSERT INTO "projected_crs" VALUES('EPSG','2199','Albanian 1987 / Gauss Kruger zone 4',NULL,'EPSG','4530','EPSG','4191','EPSG','16204',NULL,1);
INSERT INTO "usage" VALUES('EPSG','1216','projected_crs','EPSG','2199','EPSG','1025','EPSG','1056');
INSERT INTO "projected_crs" VALUES('EPSG','2200','ATS77 / New Brunswick Stereographic (ATS77)',NULL,'EPSG','4500','EPSG','4122','EPSG','19945',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1217','projected_crs','EPSG','2200','EPSG','1447','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2201','REGVEN / UTM zone 18N',NULL,'EPSG','4400','EPSG','4189','EPSG','16018',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1218','projected_crs','EPSG','2201','EPSG','1693','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2202','REGVEN / UTM zone 19N',NULL,'EPSG','4400','EPSG','4189','EPSG','16019',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1219','projected_crs','EPSG','2202','EPSG','3859','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2203','REGVEN / UTM zone 20N',NULL,'EPSG','4400','EPSG','4189','EPSG','16020',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1220','projected_crs','EPSG','2203','EPSG','3858','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2204','NAD27 / Tennessee',NULL,'EPSG','4497','EPSG','4267','EPSG','15302',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1221','projected_crs','EPSG','2204','EPSG','1411','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2205','NAD83 / Kentucky North',NULL,'EPSG','4499','EPSG','4269','EPSG','15303',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1222','projected_crs','EPSG','2205','EPSG','2202','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2206','ED50 / 3-degree Gauss-Kruger zone 9',NULL,'EPSG','4530','EPSG','4230','EPSG','16269',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1223','projected_crs','EPSG','2206','EPSG','1524','EPSG','1207');
INSERT INTO "projected_crs" VALUES('EPSG','2207','ED50 / 3-degree Gauss-Kruger zone 10',NULL,'EPSG','4530','EPSG','4230','EPSG','16270',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1224','projected_crs','EPSG','2207','EPSG','1525','EPSG','1207');
INSERT INTO "projected_crs" VALUES('EPSG','2208','ED50 / 3-degree Gauss-Kruger zone 11',NULL,'EPSG','4530','EPSG','4230','EPSG','16271',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1225','projected_crs','EPSG','2208','EPSG','1526','EPSG','1207');
INSERT INTO "projected_crs" VALUES('EPSG','2209','ED50 / 3-degree Gauss-Kruger zone 12',NULL,'EPSG','4530','EPSG','4230','EPSG','16272',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1226','projected_crs','EPSG','2209','EPSG','1527','EPSG','1207');
INSERT INTO "projected_crs" VALUES('EPSG','2210','ED50 / 3-degree Gauss-Kruger zone 13',NULL,'EPSG','4530','EPSG','4230','EPSG','16273',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1227','projected_crs','EPSG','2210','EPSG','1528','EPSG','1207');
INSERT INTO "projected_crs" VALUES('EPSG','2211','ED50 / 3-degree Gauss-Kruger zone 14',NULL,'EPSG','4530','EPSG','4230','EPSG','16274',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1228','projected_crs','EPSG','2211','EPSG','1529','EPSG','1207');
INSERT INTO "projected_crs" VALUES('EPSG','2212','ED50 / 3-degree Gauss-Kruger zone 15',NULL,'EPSG','4530','EPSG','4230','EPSG','16275',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1229','projected_crs','EPSG','2212','EPSG','1530','EPSG','1207');
INSERT INTO "projected_crs" VALUES('EPSG','2213','ETRS89 / TM 30 NE',NULL,'EPSG','4400','EPSG','4258','EPSG','16430',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1230','projected_crs','EPSG','2213','EPSG','2546','EPSG','1136');
INSERT INTO "projected_crs" VALUES('EPSG','2214','Douala 1948 / AOF west',NULL,'EPSG','4400','EPSG','4192','EPSG','18415',NULL,1);
INSERT INTO "usage" VALUES('EPSG','1231','projected_crs','EPSG','2214','EPSG','2555','EPSG','1210');
INSERT INTO "projected_crs" VALUES('EPSG','2215','Manoca 1962 / UTM zone 32N',NULL,'EPSG','4400','EPSG','4193','EPSG','16032',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1232','projected_crs','EPSG','2215','EPSG','2555','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2216','Qoornoq 1927 / UTM zone 22N',NULL,'EPSG','4400','EPSG','4194','EPSG','16022',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1233','projected_crs','EPSG','2216','EPSG','2573','EPSG','1153');
INSERT INTO "projected_crs" VALUES('EPSG','2217','Qoornoq 1927 / UTM zone 23N',NULL,'EPSG','4400','EPSG','4194','EPSG','16023',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1234','projected_crs','EPSG','2217','EPSG','2572','EPSG','1153');
INSERT INTO "projected_crs" VALUES('EPSG','2218','Scoresbysund 1952 / Greenland zone 5 east',NULL,'EPSG','1031','EPSG','4195','EPSG','18425',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1235','projected_crs','EPSG','2218','EPSG','3370','EPSG','1153');
INSERT INTO "projected_crs" VALUES('EPSG','2219','ATS77 / UTM zone 19N',NULL,'EPSG','4400','EPSG','4122','EPSG','16019',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1236','projected_crs','EPSG','2219','EPSG','1531','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2220','ATS77 / UTM zone 20N',NULL,'EPSG','4400','EPSG','4122','EPSG','16020',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1237','projected_crs','EPSG','2220','EPSG','1532','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2221','Scoresbysund 1952 / Greenland zone 6 east',NULL,'EPSG','1031','EPSG','4195','EPSG','18426',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1238','projected_crs','EPSG','2221','EPSG','3369','EPSG','1153');
INSERT INTO "projected_crs" VALUES('EPSG','2222','NAD83 / Arizona East (ft)',NULL,'EPSG','4495','EPSG','4269','EPSG','15304',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1239','projected_crs','EPSG','2222','EPSG','2167','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2223','NAD83 / Arizona Central (ft)',NULL,'EPSG','4495','EPSG','4269','EPSG','15305',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1240','projected_crs','EPSG','2223','EPSG','2166','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2224','NAD83 / Arizona West (ft)',NULL,'EPSG','4495','EPSG','4269','EPSG','15306',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1241','projected_crs','EPSG','2224','EPSG','2168','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2225','NAD83 / California zone 1 (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15307',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1242','projected_crs','EPSG','2225','EPSG','2175','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2226','NAD83 / California zone 2 (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15308',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1243','projected_crs','EPSG','2226','EPSG','2176','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2227','NAD83 / California zone 3 (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15309',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1244','projected_crs','EPSG','2227','EPSG','2177','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2228','NAD83 / California zone 4 (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15310',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1245','projected_crs','EPSG','2228','EPSG','2178','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2229','NAD83 / California zone 5 (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15311',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1246','projected_crs','EPSG','2229','EPSG','2182','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2230','NAD83 / California zone 6 (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15312',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1247','projected_crs','EPSG','2230','EPSG','2180','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2231','NAD83 / Colorado North (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15313',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1248','projected_crs','EPSG','2231','EPSG','2184','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2232','NAD83 / Colorado Central (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15314',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1249','projected_crs','EPSG','2232','EPSG','2183','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2233','NAD83 / Colorado South (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15315',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1250','projected_crs','EPSG','2233','EPSG','2185','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2234','NAD83 / Connecticut (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15316',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1251','projected_crs','EPSG','2234','EPSG','1377','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2235','NAD83 / Delaware (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15317',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1252','projected_crs','EPSG','2235','EPSG','1378','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2236','NAD83 / Florida East (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15318',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1253','projected_crs','EPSG','2236','EPSG','2186','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2237','NAD83 / Florida West (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15319',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1254','projected_crs','EPSG','2237','EPSG','2188','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2238','NAD83 / Florida North (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15320',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1255','projected_crs','EPSG','2238','EPSG','2187','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2239','NAD83 / Georgia East (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15321',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1256','projected_crs','EPSG','2239','EPSG','2189','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2240','NAD83 / Georgia West (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15322',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1257','projected_crs','EPSG','2240','EPSG','2190','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2241','NAD83 / Idaho East (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15323',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1258','projected_crs','EPSG','2241','EPSG','2192','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2242','NAD83 / Idaho Central (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15324',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1259','projected_crs','EPSG','2242','EPSG','2191','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2243','NAD83 / Idaho West (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15325',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1260','projected_crs','EPSG','2243','EPSG','2193','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2244','NAD83 / Indiana East (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15326',NULL,1);
INSERT INTO "usage" VALUES('EPSG','1261','projected_crs','EPSG','2244','EPSG','2196','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2245','NAD83 / Indiana West (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15327',NULL,1);
INSERT INTO "usage" VALUES('EPSG','1262','projected_crs','EPSG','2245','EPSG','2197','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2246','NAD83 / Kentucky North (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15328',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1263','projected_crs','EPSG','2246','EPSG','2202','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2247','NAD83 / Kentucky South (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15329',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1264','projected_crs','EPSG','2247','EPSG','2203','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2248','NAD83 / Maryland (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15330',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1265','projected_crs','EPSG','2248','EPSG','1389','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2249','NAD83 / Massachusetts Mainland (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15331',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1266','projected_crs','EPSG','2249','EPSG','2209','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2250','NAD83 / Massachusetts Island (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15332',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1267','projected_crs','EPSG','2250','EPSG','2208','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2251','NAD83 / Michigan North (ft)',NULL,'EPSG','4495','EPSG','4269','EPSG','15333',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1268','projected_crs','EPSG','2251','EPSG','1723','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2252','NAD83 / Michigan Central (ft)',NULL,'EPSG','4495','EPSG','4269','EPSG','15334',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1269','projected_crs','EPSG','2252','EPSG','1724','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2253','NAD83 / Michigan South (ft)',NULL,'EPSG','4495','EPSG','4269','EPSG','15335',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1270','projected_crs','EPSG','2253','EPSG','1725','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2254','NAD83 / Mississippi East (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15336',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1271','projected_crs','EPSG','2254','EPSG','2216','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2255','NAD83 / Mississippi West (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15337',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1272','projected_crs','EPSG','2255','EPSG','2217','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2256','NAD83 / Montana (ft)',NULL,'EPSG','4495','EPSG','4269','EPSG','15338',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1273','projected_crs','EPSG','2256','EPSG','1395','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2257','NAD83 / New Mexico East (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15339',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1274','projected_crs','EPSG','2257','EPSG','2228','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2258','NAD83 / New Mexico Central (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15340',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1275','projected_crs','EPSG','2258','EPSG','2231','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2259','NAD83 / New Mexico West (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15341',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1276','projected_crs','EPSG','2259','EPSG','2232','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2260','NAD83 / New York East (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15342',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1277','projected_crs','EPSG','2260','EPSG','2234','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2261','NAD83 / New York Central (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15343',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1278','projected_crs','EPSG','2261','EPSG','2233','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2262','NAD83 / New York West (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15344',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1279','projected_crs','EPSG','2262','EPSG','2236','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2263','NAD83 / New York Long Island (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15345',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1280','projected_crs','EPSG','2263','EPSG','2235','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2264','NAD83 / North Carolina (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15346',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1281','projected_crs','EPSG','2264','EPSG','1402','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2265','NAD83 / North Dakota North (ft)',NULL,'EPSG','4495','EPSG','4269','EPSG','15347',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1282','projected_crs','EPSG','2265','EPSG','2237','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2266','NAD83 / North Dakota South (ft)',NULL,'EPSG','4495','EPSG','4269','EPSG','15348',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1283','projected_crs','EPSG','2266','EPSG','2238','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2267','NAD83 / Oklahoma North (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15349',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1284','projected_crs','EPSG','2267','EPSG','2241','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2268','NAD83 / Oklahoma South (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15350',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1285','projected_crs','EPSG','2268','EPSG','2242','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2269','NAD83 / Oregon North (ft)',NULL,'EPSG','4495','EPSG','4269','EPSG','15351',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1286','projected_crs','EPSG','2269','EPSG','2243','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2270','NAD83 / Oregon South (ft)',NULL,'EPSG','4495','EPSG','4269','EPSG','15352',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1287','projected_crs','EPSG','2270','EPSG','2244','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2271','NAD83 / Pennsylvania North (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15353',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1288','projected_crs','EPSG','2271','EPSG','2245','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2272','NAD83 / Pennsylvania South (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15354',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1289','projected_crs','EPSG','2272','EPSG','2246','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2273','NAD83 / South Carolina (ft)',NULL,'EPSG','4495','EPSG','4269','EPSG','15355',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1290','projected_crs','EPSG','2273','EPSG','1409','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2274','NAD83 / Tennessee (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15356',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1291','projected_crs','EPSG','2274','EPSG','1411','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2275','NAD83 / Texas North (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15357',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1292','projected_crs','EPSG','2275','EPSG','2253','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2276','NAD83 / Texas North Central (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15358',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1293','projected_crs','EPSG','2276','EPSG','2254','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2277','NAD83 / Texas Central (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15359',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1294','projected_crs','EPSG','2277','EPSG','2252','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2278','NAD83 / Texas South Central (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15360',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1295','projected_crs','EPSG','2278','EPSG','2527','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2279','NAD83 / Texas South (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15361',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1296','projected_crs','EPSG','2279','EPSG','2528','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2280','NAD83 / Utah North (ft)',NULL,'EPSG','4495','EPSG','4269','EPSG','15362',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1297','projected_crs','EPSG','2280','EPSG','2258','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2281','NAD83 / Utah Central (ft)',NULL,'EPSG','4495','EPSG','4269','EPSG','15363',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1298','projected_crs','EPSG','2281','EPSG','2257','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2282','NAD83 / Utah South (ft)',NULL,'EPSG','4495','EPSG','4269','EPSG','15364',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1299','projected_crs','EPSG','2282','EPSG','2259','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2283','NAD83 / Virginia North (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15365',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1300','projected_crs','EPSG','2283','EPSG','2260','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2284','NAD83 / Virginia South (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15366',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1301','projected_crs','EPSG','2284','EPSG','2261','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2285','NAD83 / Washington North (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15367',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1302','projected_crs','EPSG','2285','EPSG','2273','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2286','NAD83 / Washington South (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15368',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1303','projected_crs','EPSG','2286','EPSG','2274','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2287','NAD83 / Wisconsin North (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15369',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1304','projected_crs','EPSG','2287','EPSG','2267','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2288','NAD83 / Wisconsin Central (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15370',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1305','projected_crs','EPSG','2288','EPSG','2266','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2289','NAD83 / Wisconsin South (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15371',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1306','projected_crs','EPSG','2289','EPSG','2268','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2290','ATS77 / Prince Edward Isl. Stereographic (ATS77)',NULL,'EPSG','4496','EPSG','4122','EPSG','19933',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1307','projected_crs','EPSG','2290','EPSG','1533','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2291','NAD83(CSRS98) / Prince Edward Isl. Stereographic (NAD83)',NULL,'EPSG','4496','EPSG','4122','EPSG','19960',NULL,1);
INSERT INTO "usage" VALUES('EPSG','1308','projected_crs','EPSG','2291','EPSG','1533','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2292','NAD83(CSRS98) / Prince Edward Isl. Stereographic (NAD83)',NULL,'EPSG','4496','EPSG','4140','EPSG','19960',NULL,1);
INSERT INTO "usage" VALUES('EPSG','1309','projected_crs','EPSG','2292','EPSG','1533','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2294','ATS77 / MTM Nova Scotia zone 4',NULL,'EPSG','4400','EPSG','4122','EPSG','17794',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1310','projected_crs','EPSG','2294','EPSG','1534','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2295','ATS77 / MTM Nova Scotia zone 5',NULL,'EPSG','4400','EPSG','4122','EPSG','17795',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1311','projected_crs','EPSG','2295','EPSG','1535','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2296','Ammassalik 1958 / Greenland zone 7 east',NULL,'EPSG','1031','EPSG','4196','EPSG','18427',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1312','projected_crs','EPSG','2296','EPSG','2571','EPSG','1153');
INSERT INTO "projected_crs" VALUES('EPSG','2297','Qornoq 1927 / Greenland zone 1 east',NULL,'EPSG','4501','EPSG','4194','EPSG','18421',NULL,1);
INSERT INTO "usage" VALUES('EPSG','1313','projected_crs','EPSG','2297','EPSG','2556','EPSG','1153');
INSERT INTO "projected_crs" VALUES('EPSG','2298','Qornoq 1927 / Greenland zone 2 east',NULL,'EPSG','4501','EPSG','4194','EPSG','18422',NULL,1);
INSERT INTO "usage" VALUES('EPSG','1314','projected_crs','EPSG','2298','EPSG','2557','EPSG','1153');
INSERT INTO "projected_crs" VALUES('EPSG','2299','Qoornoq 1927 / Greenland zone 2 west',NULL,'EPSG','1031','EPSG','4194','EPSG','18432',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1315','projected_crs','EPSG','2299','EPSG','3368','EPSG','1153');
INSERT INTO "projected_crs" VALUES('EPSG','2300','Qornoq 1927 / Greenland zone 3 east',NULL,'EPSG','4501','EPSG','4194','EPSG','18423',NULL,1);
INSERT INTO "usage" VALUES('EPSG','1316','projected_crs','EPSG','2300','EPSG','2558','EPSG','1153');
INSERT INTO "projected_crs" VALUES('EPSG','2301','Qoornoq 1927 / Greenland zone 3 west',NULL,'EPSG','1031','EPSG','4194','EPSG','18433',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1317','projected_crs','EPSG','2301','EPSG','3367','EPSG','1153');
INSERT INTO "projected_crs" VALUES('EPSG','2302','Qornoq 1927 / Greenland zone 4 east',NULL,'EPSG','4501','EPSG','4194','EPSG','18424',NULL,1);
INSERT INTO "usage" VALUES('EPSG','1318','projected_crs','EPSG','2302','EPSG','2559','EPSG','1153');
INSERT INTO "projected_crs" VALUES('EPSG','2303','Qoornoq 1927 / Greenland zone 4 west',NULL,'EPSG','1031','EPSG','4194','EPSG','18434',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1319','projected_crs','EPSG','2303','EPSG','3366','EPSG','1153');
INSERT INTO "projected_crs" VALUES('EPSG','2304','Qoornoq 1927 / Greenland zone 5 west',NULL,'EPSG','1031','EPSG','4194','EPSG','18435',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1320','projected_crs','EPSG','2304','EPSG','3365','EPSG','1153');
INSERT INTO "projected_crs" VALUES('EPSG','2305','Qoornoq 1927 / Greenland zone 6 west',NULL,'EPSG','1031','EPSG','4194','EPSG','18436',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1321','projected_crs','EPSG','2305','EPSG','3364','EPSG','1153');
INSERT INTO "projected_crs" VALUES('EPSG','2306','Qoornoq 1927 / Greenland zone 7 west',NULL,'EPSG','1031','EPSG','4194','EPSG','18437',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1322','projected_crs','EPSG','2306','EPSG','3363','EPSG','1153');
INSERT INTO "projected_crs" VALUES('EPSG','2307','Qoornoq 1927 / Greenland zone 8',NULL,'EPSG','1031','EPSG','4194','EPSG','18428',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1323','projected_crs','EPSG','2307','EPSG','3846','EPSG','1153');
INSERT INTO "projected_crs" VALUES('EPSG','2308','Batavia / TM 109 SE',NULL,'EPSG','4400','EPSG','4211','EPSG','16709',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1324','projected_crs','EPSG','2308','EPSG','2577','EPSG','1136');
INSERT INTO "projected_crs" VALUES('EPSG','2309','WGS 84 / TM 116 SE',NULL,'EPSG','4400','EPSG','4326','EPSG','16716',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1325','projected_crs','EPSG','2309','EPSG','2588','EPSG','1136');
INSERT INTO "projected_crs" VALUES('EPSG','2310','WGS 84 / TM 132 SE',NULL,'EPSG','4400','EPSG','4326','EPSG','16732',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1326','projected_crs','EPSG','2310','EPSG','2589','EPSG','1136');
INSERT INTO "projected_crs" VALUES('EPSG','2311','WGS 84 / TM 6 NE',NULL,'EPSG','4400','EPSG','4326','EPSG','16406',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1327','projected_crs','EPSG','2311','EPSG','2981','EPSG','1136');
INSERT INTO "projected_crs" VALUES('EPSG','2312','Garoua / UTM zone 33N',NULL,'EPSG','4400','EPSG','4197','EPSG','16033',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1328','projected_crs','EPSG','2312','EPSG','2590','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2313','Kousseri / UTM zone 33N',NULL,'EPSG','4400','EPSG','4198','EPSG','16033',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1329','projected_crs','EPSG','2313','EPSG','2591','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2314','Trinidad 1903 / Trinidad Grid (ftCla)',NULL,'EPSG','4403','EPSG','4302','EPSG','19975',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1330','projected_crs','EPSG','2314','EPSG','1339','EPSG','1136');
INSERT INTO "projected_crs" VALUES('EPSG','2315','Campo Inchauspe / UTM zone 19S',NULL,'EPSG','4400','EPSG','4221','EPSG','16119',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1331','projected_crs','EPSG','2315','EPSG','2596','EPSG','1216');
INSERT INTO "projected_crs" VALUES('EPSG','2316','Campo Inchauspe / UTM zone 20S',NULL,'EPSG','4400','EPSG','4221','EPSG','16120',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1332','projected_crs','EPSG','2316','EPSG','2597','EPSG','1216');
INSERT INTO "projected_crs" VALUES('EPSG','2317','PSAD56 / ICN Regional',NULL,'EPSG','4499','EPSG','4248','EPSG','19976',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1333','projected_crs','EPSG','2317','EPSG','3327','EPSG','1242');
INSERT INTO "projected_crs" VALUES('EPSG','2318','Ain el Abd / Aramco Lambert',NULL,'EPSG','4400','EPSG','4204','EPSG','19977',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1334','projected_crs','EPSG','2318','EPSG','3303','EPSG','1136');
INSERT INTO "projected_crs" VALUES('EPSG','2319','ED50 / TM27',NULL,'EPSG','4530','EPSG','4230','EPSG','16305',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1335','projected_crs','EPSG','2319','EPSG','1524','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2320','ED50 / TM30',NULL,'EPSG','4530','EPSG','4230','EPSG','16370',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1336','projected_crs','EPSG','2320','EPSG','1525','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2321','ED50 / TM33',NULL,'EPSG','4530','EPSG','4230','EPSG','16306',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1337','projected_crs','EPSG','2321','EPSG','1526','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2322','ED50 / TM36',NULL,'EPSG','4530','EPSG','4230','EPSG','16372',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1338','projected_crs','EPSG','2322','EPSG','1527','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2323','ED50 / TM39',NULL,'EPSG','4530','EPSG','4230','EPSG','16307',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1339','projected_crs','EPSG','2323','EPSG','1528','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2324','ED50 / TM42',NULL,'EPSG','4530','EPSG','4230','EPSG','16374',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1340','projected_crs','EPSG','2324','EPSG','1529','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2325','ED50 / TM45',NULL,'EPSG','4530','EPSG','4230','EPSG','16308',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1341','projected_crs','EPSG','2325','EPSG','1530','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2326','Hong Kong 1980 Grid System',NULL,'EPSG','4500','EPSG','4611','EPSG','19978',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1342','projected_crs','EPSG','2326','EPSG','1118','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2327','Xian 1980 / Gauss-Kruger zone 13',NULL,'EPSG','4530','EPSG','4610','EPSG','16213',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1343','projected_crs','EPSG','2327','EPSG','1587','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2328','Xian 1980 / Gauss-Kruger zone 14',NULL,'EPSG','4530','EPSG','4610','EPSG','16214',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1344','projected_crs','EPSG','2328','EPSG','1588','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2329','Xian 1980 / Gauss-Kruger zone 15',NULL,'EPSG','4530','EPSG','4610','EPSG','16215',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1345','projected_crs','EPSG','2329','EPSG','1589','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2330','Xian 1980 / Gauss-Kruger zone 16',NULL,'EPSG','4530','EPSG','4610','EPSG','16216',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1346','projected_crs','EPSG','2330','EPSG','1590','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2331','Xian 1980 / Gauss-Kruger zone 17',NULL,'EPSG','4530','EPSG','4610','EPSG','16217',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1347','projected_crs','EPSG','2331','EPSG','1591','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2332','Xian 1980 / Gauss-Kruger zone 18',NULL,'EPSG','4530','EPSG','4610','EPSG','16218',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1348','projected_crs','EPSG','2332','EPSG','1592','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2333','Xian 1980 / Gauss-Kruger zone 19',NULL,'EPSG','4530','EPSG','4610','EPSG','16219',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1349','projected_crs','EPSG','2333','EPSG','1593','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2334','Xian 1980 / Gauss-Kruger zone 20',NULL,'EPSG','4530','EPSG','4610','EPSG','16220',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1350','projected_crs','EPSG','2334','EPSG','1594','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2335','Xian 1980 / Gauss-Kruger zone 21',NULL,'EPSG','4530','EPSG','4610','EPSG','16221',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1351','projected_crs','EPSG','2335','EPSG','1595','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2336','Xian 1980 / Gauss-Kruger zone 22',NULL,'EPSG','4530','EPSG','4610','EPSG','16222',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1352','projected_crs','EPSG','2336','EPSG','1596','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2337','Xian 1980 / Gauss-Kruger zone 23',NULL,'EPSG','4530','EPSG','4610','EPSG','16223',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1353','projected_crs','EPSG','2337','EPSG','1597','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2338','Xian 1980 / Gauss-Kruger CM 75E',NULL,'EPSG','4530','EPSG','4610','EPSG','16313',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1354','projected_crs','EPSG','2338','EPSG','1587','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2339','Xian 1980 / Gauss-Kruger CM 81E',NULL,'EPSG','4530','EPSG','4610','EPSG','16314',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1355','projected_crs','EPSG','2339','EPSG','1588','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2340','Xian 1980 / Gauss-Kruger CM 87E',NULL,'EPSG','4530','EPSG','4610','EPSG','16315',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1356','projected_crs','EPSG','2340','EPSG','1589','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2341','Xian 1980 / Gauss-Kruger CM 93E',NULL,'EPSG','4530','EPSG','4610','EPSG','16316',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1357','projected_crs','EPSG','2341','EPSG','1590','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2342','Xian 1980 / Gauss-Kruger CM 99E',NULL,'EPSG','4530','EPSG','4610','EPSG','16317',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1358','projected_crs','EPSG','2342','EPSG','1591','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2343','Xian 1980 / Gauss-Kruger CM 105E',NULL,'EPSG','4530','EPSG','4610','EPSG','16318',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1359','projected_crs','EPSG','2343','EPSG','1592','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2344','Xian 1980 / Gauss-Kruger CM 111E',NULL,'EPSG','4530','EPSG','4610','EPSG','16319',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1360','projected_crs','EPSG','2344','EPSG','1593','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2345','Xian 1980 / Gauss-Kruger CM 117E',NULL,'EPSG','4530','EPSG','4610','EPSG','16320',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1361','projected_crs','EPSG','2345','EPSG','1594','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2346','Xian 1980 / Gauss-Kruger CM 123E',NULL,'EPSG','4530','EPSG','4610','EPSG','16321',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1362','projected_crs','EPSG','2346','EPSG','1595','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2347','Xian 1980 / Gauss-Kruger CM 129E',NULL,'EPSG','4530','EPSG','4610','EPSG','16322',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1363','projected_crs','EPSG','2347','EPSG','1596','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2348','Xian 1980 / Gauss-Kruger CM 135E',NULL,'EPSG','4530','EPSG','4610','EPSG','16323',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1364','projected_crs','EPSG','2348','EPSG','1597','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2349','Xian 1980 / 3-degree Gauss-Kruger zone 25',NULL,'EPSG','4530','EPSG','4610','EPSG','16285',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1365','projected_crs','EPSG','2349','EPSG','2711','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2350','Xian 1980 / 3-degree Gauss-Kruger zone 26',NULL,'EPSG','4530','EPSG','4610','EPSG','16286',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1366','projected_crs','EPSG','2350','EPSG','2712','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2351','Xian 1980 / 3-degree Gauss-Kruger zone 27',NULL,'EPSG','4530','EPSG','4610','EPSG','16287',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1367','projected_crs','EPSG','2351','EPSG','2713','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2352','Xian 1980 / 3-degree Gauss-Kruger zone 28',NULL,'EPSG','4530','EPSG','4610','EPSG','16288',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1368','projected_crs','EPSG','2352','EPSG','2714','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2353','Xian 1980 / 3-degree Gauss-Kruger zone 29',NULL,'EPSG','4530','EPSG','4610','EPSG','16289',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1369','projected_crs','EPSG','2353','EPSG','2715','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2354','Xian 1980 / 3-degree Gauss-Kruger zone 30',NULL,'EPSG','4530','EPSG','4610','EPSG','16290',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1370','projected_crs','EPSG','2354','EPSG','2716','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2355','Xian 1980 / 3-degree Gauss-Kruger zone 31',NULL,'EPSG','4530','EPSG','4610','EPSG','16291',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1371','projected_crs','EPSG','2355','EPSG','2717','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2356','Xian 1980 / 3-degree Gauss-Kruger zone 32',NULL,'EPSG','4530','EPSG','4610','EPSG','16292',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1372','projected_crs','EPSG','2356','EPSG','2718','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2357','Xian 1980 / 3-degree Gauss-Kruger zone 33',NULL,'EPSG','4530','EPSG','4610','EPSG','16293',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1373','projected_crs','EPSG','2357','EPSG','2719','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2358','Xian 1980 / 3-degree Gauss-Kruger zone 34',NULL,'EPSG','4530','EPSG','4610','EPSG','16294',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1374','projected_crs','EPSG','2358','EPSG','2720','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2359','Xian 1980 / 3-degree Gauss-Kruger zone 35',NULL,'EPSG','4530','EPSG','4610','EPSG','16295',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1375','projected_crs','EPSG','2359','EPSG','2721','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2360','Xian 1980 / 3-degree Gauss-Kruger zone 36',NULL,'EPSG','4530','EPSG','4610','EPSG','16296',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1376','projected_crs','EPSG','2360','EPSG','2722','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2361','Xian 1980 / 3-degree Gauss-Kruger zone 37',NULL,'EPSG','4530','EPSG','4610','EPSG','16297',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1377','projected_crs','EPSG','2361','EPSG','2723','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2362','Xian 1980 / 3-degree Gauss-Kruger zone 38',NULL,'EPSG','4530','EPSG','4610','EPSG','16298',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1378','projected_crs','EPSG','2362','EPSG','2724','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2363','Xian 1980 / 3-degree Gauss-Kruger zone 39',NULL,'EPSG','4530','EPSG','4610','EPSG','16299',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1379','projected_crs','EPSG','2363','EPSG','2725','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2364','Xian 1980 / 3-degree Gauss-Kruger zone 40',NULL,'EPSG','4530','EPSG','4610','EPSG','16070',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1380','projected_crs','EPSG','2364','EPSG','2726','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2365','Xian 1980 / 3-degree Gauss-Kruger zone 41',NULL,'EPSG','4530','EPSG','4610','EPSG','16071',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1381','projected_crs','EPSG','2365','EPSG','2727','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2366','Xian 1980 / 3-degree Gauss-Kruger zone 42',NULL,'EPSG','4530','EPSG','4610','EPSG','16072',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1382','projected_crs','EPSG','2366','EPSG','2728','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2367','Xian 1980 / 3-degree Gauss-Kruger zone 43',NULL,'EPSG','4530','EPSG','4610','EPSG','16073',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1383','projected_crs','EPSG','2367','EPSG','2729','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2368','Xian 1980 / 3-degree Gauss-Kruger zone 44',NULL,'EPSG','4530','EPSG','4610','EPSG','16074',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1384','projected_crs','EPSG','2368','EPSG','2730','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2369','Xian 1980 / 3-degree Gauss-Kruger zone 45',NULL,'EPSG','4530','EPSG','4610','EPSG','16075',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1385','projected_crs','EPSG','2369','EPSG','2731','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2370','Xian 1980 / 3-degree Gauss-Kruger CM 75E',NULL,'EPSG','4530','EPSG','4610','EPSG','16313',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1386','projected_crs','EPSG','2370','EPSG','2711','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2371','Xian 1980 / 3-degree Gauss-Kruger CM 78E',NULL,'EPSG','4530','EPSG','4610','EPSG','16386',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1387','projected_crs','EPSG','2371','EPSG','2712','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2372','Xian 1980 / 3-degree Gauss-Kruger CM 81E',NULL,'EPSG','4530','EPSG','4610','EPSG','16314',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1388','projected_crs','EPSG','2372','EPSG','2713','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2373','Xian 1980 / 3-degree Gauss-Kruger CM 84E',NULL,'EPSG','4530','EPSG','4610','EPSG','16388',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1389','projected_crs','EPSG','2373','EPSG','2714','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2374','Xian 1980 / 3-degree Gauss-Kruger CM 87E',NULL,'EPSG','4530','EPSG','4610','EPSG','16315',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1390','projected_crs','EPSG','2374','EPSG','2715','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2375','Xian 1980 / 3-degree Gauss-Kruger CM 90E',NULL,'EPSG','4530','EPSG','4610','EPSG','16390',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1391','projected_crs','EPSG','2375','EPSG','2716','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2376','Xian 1980 / 3-degree Gauss-Kruger CM 93E',NULL,'EPSG','4530','EPSG','4610','EPSG','16316',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1392','projected_crs','EPSG','2376','EPSG','2717','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2377','Xian 1980 / 3-degree Gauss-Kruger CM 96E',NULL,'EPSG','4530','EPSG','4610','EPSG','16392',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1393','projected_crs','EPSG','2377','EPSG','2718','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2378','Xian 1980 / 3-degree Gauss-Kruger CM 99E',NULL,'EPSG','4530','EPSG','4610','EPSG','16317',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1394','projected_crs','EPSG','2378','EPSG','2719','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2379','Xian 1980 / 3-degree Gauss-Kruger CM 102E',NULL,'EPSG','4530','EPSG','4610','EPSG','16394',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1395','projected_crs','EPSG','2379','EPSG','2720','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2380','Xian 1980 / 3-degree Gauss-Kruger CM 105E',NULL,'EPSG','4530','EPSG','4610','EPSG','16318',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1396','projected_crs','EPSG','2380','EPSG','2721','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2381','Xian 1980 / 3-degree Gauss-Kruger CM 108E',NULL,'EPSG','4530','EPSG','4610','EPSG','16396',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1397','projected_crs','EPSG','2381','EPSG','2722','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2382','Xian 1980 / 3-degree Gauss-Kruger CM 111E',NULL,'EPSG','4530','EPSG','4610','EPSG','16319',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1398','projected_crs','EPSG','2382','EPSG','2723','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2383','Xian 1980 / 3-degree Gauss-Kruger CM 114E',NULL,'EPSG','4530','EPSG','4610','EPSG','16398',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1399','projected_crs','EPSG','2383','EPSG','2724','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2384','Xian 1980 / 3-degree Gauss-Kruger CM 117E',NULL,'EPSG','4530','EPSG','4610','EPSG','16320',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1400','projected_crs','EPSG','2384','EPSG','2725','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2385','Xian 1980 / 3-degree Gauss-Kruger CM 120E',NULL,'EPSG','4530','EPSG','4610','EPSG','16170',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1401','projected_crs','EPSG','2385','EPSG','2726','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2386','Xian 1980 / 3-degree Gauss-Kruger CM 123E',NULL,'EPSG','4530','EPSG','4610','EPSG','16321',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1402','projected_crs','EPSG','2386','EPSG','2727','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2387','Xian 1980 / 3-degree Gauss-Kruger CM 126E',NULL,'EPSG','4530','EPSG','4610','EPSG','16172',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1403','projected_crs','EPSG','2387','EPSG','2728','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2388','Xian 1980 / 3-degree Gauss-Kruger CM 129E',NULL,'EPSG','4530','EPSG','4610','EPSG','16322',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1404','projected_crs','EPSG','2388','EPSG','2729','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2389','Xian 1980 / 3-degree Gauss-Kruger CM 132E',NULL,'EPSG','4530','EPSG','4610','EPSG','16174',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1405','projected_crs','EPSG','2389','EPSG','2730','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2390','Xian 1980 / 3-degree Gauss-Kruger CM 135E',NULL,'EPSG','4530','EPSG','4610','EPSG','16323',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1406','projected_crs','EPSG','2390','EPSG','2731','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2391','KKJ / Finland zone 1',NULL,'EPSG','4530','EPSG','4123','EPSG','18191',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1407','projected_crs','EPSG','2391','EPSG','1536','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2392','KKJ / Finland zone 2',NULL,'EPSG','4530','EPSG','4123','EPSG','18192',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1408','projected_crs','EPSG','2392','EPSG','1537','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2393','KKJ / Finland Uniform Coordinate System',NULL,'EPSG','4530','EPSG','4123','EPSG','18193',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1409','projected_crs','EPSG','2393','EPSG','1538','EPSG','1208');
INSERT INTO "usage" VALUES('EPSG','1410','projected_crs','EPSG','2393','EPSG','3333','EPSG','1241');
INSERT INTO "projected_crs" VALUES('EPSG','2394','KKJ / Finland zone 4',NULL,'EPSG','4530','EPSG','4123','EPSG','18194',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1411','projected_crs','EPSG','2394','EPSG','1539','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2395','South Yemen / Gauss-Kruger zone 8',NULL,'EPSG','4530','EPSG','4164','EPSG','16208',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1412','projected_crs','EPSG','2395','EPSG','1492','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2396','South Yemen / Gauss-Kruger zone 9',NULL,'EPSG','4530','EPSG','4164','EPSG','16209',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1413','projected_crs','EPSG','2396','EPSG','1493','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2397','Pulkovo 1942(83) / 3-degree Gauss-Kruger zone 3',NULL,'EPSG','4530','EPSG','4178','EPSG','16263',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1414','projected_crs','EPSG','2397','EPSG','1512','EPSG','1160');
INSERT INTO "projected_crs" VALUES('EPSG','2398','Pulkovo 1942(83) / 3-degree Gauss-Kruger zone 4',NULL,'EPSG','4530','EPSG','4178','EPSG','16264',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1415','projected_crs','EPSG','2398','EPSG','1513','EPSG','1160');
INSERT INTO "projected_crs" VALUES('EPSG','2399','Pulkovo 1942(83) / 3-degree Gauss-Kruger zone 5',NULL,'EPSG','4530','EPSG','4178','EPSG','16265',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1416','projected_crs','EPSG','2399','EPSG','1514','EPSG','1160');
INSERT INTO "projected_crs" VALUES('EPSG','2400','RT90 2.5 gon W',NULL,'EPSG','4530','EPSG','4124','EPSG','19929',NULL,1);
INSERT INTO "usage" VALUES('EPSG','1417','projected_crs','EPSG','2400','EPSG','1225','EPSG','1056');
INSERT INTO "projected_crs" VALUES('EPSG','2401','Beijing 1954 / 3-degree Gauss-Kruger zone 25',NULL,'EPSG','4530','EPSG','4214','EPSG','16285',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1418','projected_crs','EPSG','2401','EPSG','2711','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2402','Beijing 1954 / 3-degree Gauss-Kruger zone 26',NULL,'EPSG','4530','EPSG','4214','EPSG','16286',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1419','projected_crs','EPSG','2402','EPSG','2712','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2403','Beijing 1954 / 3-degree Gauss-Kruger zone 27',NULL,'EPSG','4530','EPSG','4214','EPSG','16287',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1420','projected_crs','EPSG','2403','EPSG','2713','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2404','Beijing 1954 / 3-degree Gauss-Kruger zone 28',NULL,'EPSG','4530','EPSG','4214','EPSG','16288',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1421','projected_crs','EPSG','2404','EPSG','2714','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2405','Beijing 1954 / 3-degree Gauss-Kruger zone 29',NULL,'EPSG','4530','EPSG','4214','EPSG','16289',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1422','projected_crs','EPSG','2405','EPSG','2715','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2406','Beijing 1954 / 3-degree Gauss-Kruger zone 30',NULL,'EPSG','4530','EPSG','4214','EPSG','16290',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1423','projected_crs','EPSG','2406','EPSG','2716','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2407','Beijing 1954 / 3-degree Gauss-Kruger zone 31',NULL,'EPSG','4530','EPSG','4214','EPSG','16291',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1424','projected_crs','EPSG','2407','EPSG','2717','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2408','Beijing 1954 / 3-degree Gauss-Kruger zone 32',NULL,'EPSG','4530','EPSG','4214','EPSG','16292',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1425','projected_crs','EPSG','2408','EPSG','2718','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2409','Beijing 1954 / 3-degree Gauss-Kruger zone 33',NULL,'EPSG','4530','EPSG','4214','EPSG','16293',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1426','projected_crs','EPSG','2409','EPSG','2719','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2410','Beijing 1954 / 3-degree Gauss-Kruger zone 34',NULL,'EPSG','4530','EPSG','4214','EPSG','16294',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1427','projected_crs','EPSG','2410','EPSG','2720','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2411','Beijing 1954 / 3-degree Gauss-Kruger zone 35',NULL,'EPSG','4530','EPSG','4214','EPSG','16295',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1428','projected_crs','EPSG','2411','EPSG','2721','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2412','Beijing 1954 / 3-degree Gauss-Kruger zone 36',NULL,'EPSG','4530','EPSG','4214','EPSG','16296',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1429','projected_crs','EPSG','2412','EPSG','2722','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2413','Beijing 1954 / 3-degree Gauss-Kruger zone 37',NULL,'EPSG','4530','EPSG','4214','EPSG','16297',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1430','projected_crs','EPSG','2413','EPSG','2723','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2414','Beijing 1954 / 3-degree Gauss-Kruger zone 38',NULL,'EPSG','4530','EPSG','4214','EPSG','16298',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1431','projected_crs','EPSG','2414','EPSG','2724','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2415','Beijing 1954 / 3-degree Gauss-Kruger zone 39',NULL,'EPSG','4530','EPSG','4214','EPSG','16299',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1432','projected_crs','EPSG','2415','EPSG','2725','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2416','Beijing 1954 / 3-degree Gauss-Kruger zone 40',NULL,'EPSG','4530','EPSG','4214','EPSG','16070',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1433','projected_crs','EPSG','2416','EPSG','2726','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2417','Beijing 1954 / 3-degree Gauss-Kruger zone 41',NULL,'EPSG','4530','EPSG','4214','EPSG','16071',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1434','projected_crs','EPSG','2417','EPSG','2727','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2418','Beijing 1954 / 3-degree Gauss-Kruger zone 42',NULL,'EPSG','4530','EPSG','4214','EPSG','16072',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1435','projected_crs','EPSG','2418','EPSG','2728','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2419','Beijing 1954 / 3-degree Gauss-Kruger zone 43',NULL,'EPSG','4530','EPSG','4214','EPSG','16073',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1436','projected_crs','EPSG','2419','EPSG','2729','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2420','Beijing 1954 / 3-degree Gauss-Kruger zone 44',NULL,'EPSG','4530','EPSG','4214','EPSG','16074',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1437','projected_crs','EPSG','2420','EPSG','2730','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2421','Beijing 1954 / 3-degree Gauss-Kruger zone 45',NULL,'EPSG','4530','EPSG','4214','EPSG','16075',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1438','projected_crs','EPSG','2421','EPSG','2731','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2422','Beijing 1954 / 3-degree Gauss-Kruger CM 75E',NULL,'EPSG','4530','EPSG','4214','EPSG','16313',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1439','projected_crs','EPSG','2422','EPSG','2711','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2423','Beijing 1954 / 3-degree Gauss-Kruger CM 78E',NULL,'EPSG','4530','EPSG','4214','EPSG','16386',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1440','projected_crs','EPSG','2423','EPSG','2712','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2424','Beijing 1954 / 3-degree Gauss-Kruger CM 81E',NULL,'EPSG','4530','EPSG','4214','EPSG','16314',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1441','projected_crs','EPSG','2424','EPSG','2713','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2425','Beijing 1954 / 3-degree Gauss-Kruger CM 84E',NULL,'EPSG','4530','EPSG','4214','EPSG','16388',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1442','projected_crs','EPSG','2425','EPSG','2714','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2426','Beijing 1954 / 3-degree Gauss-Kruger CM 87E',NULL,'EPSG','4530','EPSG','4214','EPSG','16315',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1443','projected_crs','EPSG','2426','EPSG','2715','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2427','Beijing 1954 / 3-degree Gauss-Kruger CM 90E',NULL,'EPSG','4530','EPSG','4214','EPSG','16390',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1444','projected_crs','EPSG','2427','EPSG','2716','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2428','Beijing 1954 / 3-degree Gauss-Kruger CM 93E',NULL,'EPSG','4530','EPSG','4214','EPSG','16316',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1445','projected_crs','EPSG','2428','EPSG','2717','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2429','Beijing 1954 / 3-degree Gauss-Kruger CM 96E',NULL,'EPSG','4530','EPSG','4214','EPSG','16392',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1446','projected_crs','EPSG','2429','EPSG','2718','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2430','Beijing 1954 / 3-degree Gauss-Kruger CM 99E',NULL,'EPSG','4530','EPSG','4214','EPSG','16317',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1447','projected_crs','EPSG','2430','EPSG','2719','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2431','Beijing 1954 / 3-degree Gauss-Kruger CM 102E',NULL,'EPSG','4530','EPSG','4214','EPSG','16394',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1448','projected_crs','EPSG','2431','EPSG','2720','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2432','Beijing 1954 / 3-degree Gauss-Kruger CM 105E',NULL,'EPSG','4530','EPSG','4214','EPSG','16318',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1449','projected_crs','EPSG','2432','EPSG','2721','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2433','Beijing 1954 / 3-degree Gauss-Kruger CM 108E',NULL,'EPSG','4530','EPSG','4214','EPSG','16396',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1450','projected_crs','EPSG','2433','EPSG','2722','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2434','Beijing 1954 / 3-degree Gauss-Kruger CM 111E',NULL,'EPSG','4530','EPSG','4214','EPSG','16319',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1451','projected_crs','EPSG','2434','EPSG','2723','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2435','Beijing 1954 / 3-degree Gauss-Kruger CM 114E',NULL,'EPSG','4530','EPSG','4214','EPSG','16398',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1452','projected_crs','EPSG','2435','EPSG','2724','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2436','Beijing 1954 / 3-degree Gauss-Kruger CM 117E',NULL,'EPSG','4530','EPSG','4214','EPSG','16320',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1453','projected_crs','EPSG','2436','EPSG','2725','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2437','Beijing 1954 / 3-degree Gauss-Kruger CM 120E',NULL,'EPSG','4530','EPSG','4214','EPSG','16170',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1454','projected_crs','EPSG','2437','EPSG','2726','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2438','Beijing 1954 / 3-degree Gauss-Kruger CM 123E',NULL,'EPSG','4530','EPSG','4214','EPSG','16321',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1455','projected_crs','EPSG','2438','EPSG','2727','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2439','Beijing 1954 / 3-degree Gauss-Kruger CM 126E',NULL,'EPSG','4530','EPSG','4214','EPSG','16172',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1456','projected_crs','EPSG','2439','EPSG','2728','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2440','Beijing 1954 / 3-degree Gauss-Kruger CM 129E',NULL,'EPSG','4530','EPSG','4214','EPSG','16322',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1457','projected_crs','EPSG','2440','EPSG','2729','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2441','Beijing 1954 / 3-degree Gauss-Kruger CM 132E',NULL,'EPSG','4530','EPSG','4214','EPSG','16174',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1458','projected_crs','EPSG','2441','EPSG','2730','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2442','Beijing 1954 / 3-degree Gauss-Kruger CM 135E',NULL,'EPSG','4530','EPSG','4214','EPSG','16323',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1459','projected_crs','EPSG','2442','EPSG','2731','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2443','JGD2000 / Japan Plane Rectangular CS I',NULL,'EPSG','4530','EPSG','4612','EPSG','17801',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1460','projected_crs','EPSG','2443','EPSG','1854','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','2444','JGD2000 / Japan Plane Rectangular CS II',NULL,'EPSG','4530','EPSG','4612','EPSG','17802',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1461','projected_crs','EPSG','2444','EPSG','1855','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','2445','JGD2000 / Japan Plane Rectangular CS III',NULL,'EPSG','4530','EPSG','4612','EPSG','17803',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1462','projected_crs','EPSG','2445','EPSG','1856','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','2446','JGD2000 / Japan Plane Rectangular CS IV',NULL,'EPSG','4530','EPSG','4612','EPSG','17804',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1463','projected_crs','EPSG','2446','EPSG','1857','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','2447','JGD2000 / Japan Plane Rectangular CS V',NULL,'EPSG','4530','EPSG','4612','EPSG','17805',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1464','projected_crs','EPSG','2447','EPSG','1858','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','2448','JGD2000 / Japan Plane Rectangular CS VI',NULL,'EPSG','4530','EPSG','4612','EPSG','17806',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1465','projected_crs','EPSG','2448','EPSG','1859','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','2449','JGD2000 / Japan Plane Rectangular CS VII',NULL,'EPSG','4530','EPSG','4612','EPSG','17807',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1466','projected_crs','EPSG','2449','EPSG','1860','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','2450','JGD2000 / Japan Plane Rectangular CS VIII',NULL,'EPSG','4530','EPSG','4612','EPSG','17808',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1467','projected_crs','EPSG','2450','EPSG','1861','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','2451','JGD2000 / Japan Plane Rectangular CS IX',NULL,'EPSG','4530','EPSG','4612','EPSG','17809',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1468','projected_crs','EPSG','2451','EPSG','1862','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','2452','JGD2000 / Japan Plane Rectangular CS X',NULL,'EPSG','4530','EPSG','4612','EPSG','17810',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1469','projected_crs','EPSG','2452','EPSG','1863','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','2453','JGD2000 / Japan Plane Rectangular CS XI',NULL,'EPSG','4530','EPSG','4612','EPSG','17811',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1470','projected_crs','EPSG','2453','EPSG','1864','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','2454','JGD2000 / Japan Plane Rectangular CS XII',NULL,'EPSG','4530','EPSG','4612','EPSG','17812',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1471','projected_crs','EPSG','2454','EPSG','1865','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','2455','JGD2000 / Japan Plane Rectangular CS XIII',NULL,'EPSG','4530','EPSG','4612','EPSG','17813',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1472','projected_crs','EPSG','2455','EPSG','1866','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','2456','JGD2000 / Japan Plane Rectangular CS XIV',NULL,'EPSG','4530','EPSG','4612','EPSG','17814',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1473','projected_crs','EPSG','2456','EPSG','1867','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','2457','JGD2000 / Japan Plane Rectangular CS XV',NULL,'EPSG','4530','EPSG','4612','EPSG','17815',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1474','projected_crs','EPSG','2457','EPSG','1868','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','2458','JGD2000 / Japan Plane Rectangular CS XVI',NULL,'EPSG','4530','EPSG','4612','EPSG','17816',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1475','projected_crs','EPSG','2458','EPSG','1869','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','2459','JGD2000 / Japan Plane Rectangular CS XVII',NULL,'EPSG','4530','EPSG','4612','EPSG','17817',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1476','projected_crs','EPSG','2459','EPSG','1870','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','2460','JGD2000 / Japan Plane Rectangular CS XVIII',NULL,'EPSG','4530','EPSG','4612','EPSG','17818',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1477','projected_crs','EPSG','2460','EPSG','1871','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','2461','JGD2000 / Japan Plane Rectangular CS XIX',NULL,'EPSG','4530','EPSG','4612','EPSG','17819',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1478','projected_crs','EPSG','2461','EPSG','1872','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','2462','Albanian 1987 / Gauss-Kruger zone 4',NULL,'EPSG','4530','EPSG','4191','EPSG','16204',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1479','projected_crs','EPSG','2462','EPSG','3212','EPSG','1056');
INSERT INTO "projected_crs" VALUES('EPSG','2463','Pulkovo 1995 / Gauss-Kruger CM 21E',NULL,'EPSG','4530','EPSG','4200','EPSG','16304',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1480','projected_crs','EPSG','2463','EPSG','1763','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2464','Pulkovo 1995 / Gauss-Kruger CM 27E',NULL,'EPSG','4530','EPSG','4200','EPSG','16305',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1481','projected_crs','EPSG','2464','EPSG','1764','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2465','Pulkovo 1995 / Gauss-Kruger CM 33E',NULL,'EPSG','4530','EPSG','4200','EPSG','16306',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1482','projected_crs','EPSG','2465','EPSG','1765','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2466','Pulkovo 1995 / Gauss-Kruger CM 39E',NULL,'EPSG','4530','EPSG','4200','EPSG','16307',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1483','projected_crs','EPSG','2466','EPSG','1766','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2467','Pulkovo 1995 / Gauss-Kruger CM 45E',NULL,'EPSG','4530','EPSG','4200','EPSG','16308',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1484','projected_crs','EPSG','2467','EPSG','1767','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2468','Pulkovo 1995 / Gauss-Kruger CM 51E',NULL,'EPSG','4530','EPSG','4200','EPSG','16309',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1485','projected_crs','EPSG','2468','EPSG','1768','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2469','Pulkovo 1995 / Gauss-Kruger CM 57E',NULL,'EPSG','4530','EPSG','4200','EPSG','16310',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1486','projected_crs','EPSG','2469','EPSG','1769','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2470','Pulkovo 1995 / Gauss-Kruger CM 63E',NULL,'EPSG','4530','EPSG','4200','EPSG','16311',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1487','projected_crs','EPSG','2470','EPSG','1770','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2471','Pulkovo 1995 / Gauss-Kruger CM 69E',NULL,'EPSG','4530','EPSG','4200','EPSG','16312',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1488','projected_crs','EPSG','2471','EPSG','1771','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2472','Pulkovo 1995 / Gauss-Kruger CM 75E',NULL,'EPSG','4530','EPSG','4200','EPSG','16313',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1489','projected_crs','EPSG','2472','EPSG','1772','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2473','Pulkovo 1995 / Gauss-Kruger CM 81E',NULL,'EPSG','4530','EPSG','4200','EPSG','16314',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1490','projected_crs','EPSG','2473','EPSG','1773','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2474','Pulkovo 1995 / Gauss-Kruger CM 87E',NULL,'EPSG','4530','EPSG','4200','EPSG','16315',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1491','projected_crs','EPSG','2474','EPSG','1774','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2475','Pulkovo 1995 / Gauss-Kruger CM 93E',NULL,'EPSG','4530','EPSG','4200','EPSG','16316',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1492','projected_crs','EPSG','2475','EPSG','1775','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2476','Pulkovo 1995 / Gauss-Kruger CM 99E',NULL,'EPSG','4530','EPSG','4200','EPSG','16317',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1493','projected_crs','EPSG','2476','EPSG','1776','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2477','Pulkovo 1995 / Gauss-Kruger CM 105E',NULL,'EPSG','4530','EPSG','4200','EPSG','16318',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1494','projected_crs','EPSG','2477','EPSG','1777','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2478','Pulkovo 1995 / Gauss-Kruger CM 111E',NULL,'EPSG','4530','EPSG','4200','EPSG','16319',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1495','projected_crs','EPSG','2478','EPSG','1778','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2479','Pulkovo 1995 / Gauss-Kruger CM 117E',NULL,'EPSG','4530','EPSG','4200','EPSG','16320',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1496','projected_crs','EPSG','2479','EPSG','1779','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2480','Pulkovo 1995 / Gauss-Kruger CM 123E',NULL,'EPSG','4530','EPSG','4200','EPSG','16321',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1497','projected_crs','EPSG','2480','EPSG','1780','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2481','Pulkovo 1995 / Gauss-Kruger CM 129E',NULL,'EPSG','4530','EPSG','4200','EPSG','16322',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1498','projected_crs','EPSG','2481','EPSG','1781','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2482','Pulkovo 1995 / Gauss-Kruger CM 135E',NULL,'EPSG','4530','EPSG','4200','EPSG','16323',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1499','projected_crs','EPSG','2482','EPSG','1782','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2483','Pulkovo 1995 / Gauss-Kruger CM 141E',NULL,'EPSG','4530','EPSG','4200','EPSG','16324',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1500','projected_crs','EPSG','2483','EPSG','1783','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2484','Pulkovo 1995 / Gauss-Kruger CM 147E',NULL,'EPSG','4530','EPSG','4200','EPSG','16325',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1501','projected_crs','EPSG','2484','EPSG','1784','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2485','Pulkovo 1995 / Gauss-Kruger CM 153E',NULL,'EPSG','4530','EPSG','4200','EPSG','16326',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1502','projected_crs','EPSG','2485','EPSG','1785','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2486','Pulkovo 1995 / Gauss-Kruger CM 159E',NULL,'EPSG','4530','EPSG','4200','EPSG','16327',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1503','projected_crs','EPSG','2486','EPSG','1786','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2487','Pulkovo 1995 / Gauss-Kruger CM 165E',NULL,'EPSG','4530','EPSG','4200','EPSG','16328',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1504','projected_crs','EPSG','2487','EPSG','1787','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2488','Pulkovo 1995 / Gauss-Kruger CM 171E',NULL,'EPSG','4530','EPSG','4200','EPSG','16329',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1505','projected_crs','EPSG','2488','EPSG','1788','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2489','Pulkovo 1995 / Gauss-Kruger CM 177E',NULL,'EPSG','4530','EPSG','4200','EPSG','16330',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1506','projected_crs','EPSG','2489','EPSG','1789','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2490','Pulkovo 1995 / Gauss-Kruger CM 177W',NULL,'EPSG','4530','EPSG','4200','EPSG','16331',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1507','projected_crs','EPSG','2490','EPSG','1790','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2491','Pulkovo 1995 / Gauss-Kruger CM 171W',NULL,'EPSG','4530','EPSG','4200','EPSG','16332',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1508','projected_crs','EPSG','2491','EPSG','1791','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2492','Pulkovo 1942 / Gauss-Kruger CM 9E',NULL,'EPSG','4530','EPSG','4284','EPSG','16302',NULL,1);
INSERT INTO "usage" VALUES('EPSG','1509','projected_crs','EPSG','2492','EPSG','1805','EPSG','1160');
INSERT INTO "projected_crs" VALUES('EPSG','2493','Pulkovo 1942 / Gauss-Kruger CM 15E',NULL,'EPSG','4530','EPSG','4284','EPSG','16303',NULL,1);
INSERT INTO "usage" VALUES('EPSG','1510','projected_crs','EPSG','2493','EPSG','1792','EPSG','1160');
INSERT INTO "projected_crs" VALUES('EPSG','2494','Pulkovo 1942 / Gauss-Kruger CM 21E',NULL,'EPSG','4530','EPSG','4284','EPSG','16304',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1511','projected_crs','EPSG','2494','EPSG','1793','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2495','Pulkovo 1942 / Gauss-Kruger CM 27E',NULL,'EPSG','4530','EPSG','4284','EPSG','16305',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1512','projected_crs','EPSG','2495','EPSG','1794','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2496','Pulkovo 1942 / Gauss-Kruger CM 33E',NULL,'EPSG','4530','EPSG','4284','EPSG','16306',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1513','projected_crs','EPSG','2496','EPSG','1795','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2497','Pulkovo 1942 / Gauss-Kruger CM 39E',NULL,'EPSG','4530','EPSG','4284','EPSG','16307',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1514','projected_crs','EPSG','2497','EPSG','1796','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2498','Pulkovo 1942 / Gauss-Kruger CM 45E',NULL,'EPSG','4530','EPSG','4284','EPSG','16308',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1515','projected_crs','EPSG','2498','EPSG','1797','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2499','Pulkovo 1942 / Gauss-Kruger CM 51E',NULL,'EPSG','4530','EPSG','4284','EPSG','16309',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1516','projected_crs','EPSG','2499','EPSG','1798','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2500','Pulkovo 1942 / Gauss-Kruger CM 57E',NULL,'EPSG','4530','EPSG','4284','EPSG','16310',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1517','projected_crs','EPSG','2500','EPSG','1799','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2501','Pulkovo 1942 / Gauss-Kruger CM 63E',NULL,'EPSG','4530','EPSG','4284','EPSG','16311',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1518','projected_crs','EPSG','2501','EPSG','1800','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2502','Pulkovo 1942 / Gauss-Kruger CM 69E',NULL,'EPSG','4530','EPSG','4284','EPSG','16312',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1519','projected_crs','EPSG','2502','EPSG','1801','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2503','Pulkovo 1942 / Gauss-Kruger CM 75E',NULL,'EPSG','4530','EPSG','4284','EPSG','16313',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1520','projected_crs','EPSG','2503','EPSG','1802','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2504','Pulkovo 1942 / Gauss-Kruger CM 81E',NULL,'EPSG','4530','EPSG','4284','EPSG','16314',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1521','projected_crs','EPSG','2504','EPSG','1803','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2505','Pulkovo 1942 / Gauss-Kruger CM 87E',NULL,'EPSG','4530','EPSG','4284','EPSG','16315',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1522','projected_crs','EPSG','2505','EPSG','1804','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2506','Pulkovo 1942 / Gauss-Kruger CM 93E',NULL,'EPSG','4530','EPSG','4284','EPSG','16316',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1523','projected_crs','EPSG','2506','EPSG','1775','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2507','Pulkovo 1942 / Gauss-Kruger CM 99E',NULL,'EPSG','4530','EPSG','4284','EPSG','16317',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1524','projected_crs','EPSG','2507','EPSG','1776','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2508','Pulkovo 1942 / Gauss-Kruger CM 105E',NULL,'EPSG','4530','EPSG','4284','EPSG','16318',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1525','projected_crs','EPSG','2508','EPSG','1777','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2509','Pulkovo 1942 / Gauss-Kruger CM 111E',NULL,'EPSG','4530','EPSG','4284','EPSG','16319',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1526','projected_crs','EPSG','2509','EPSG','1778','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2510','Pulkovo 1942 / Gauss-Kruger CM 117E',NULL,'EPSG','4530','EPSG','4284','EPSG','16320',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1527','projected_crs','EPSG','2510','EPSG','1779','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2511','Pulkovo 1942 / Gauss-Kruger CM 123E',NULL,'EPSG','4530','EPSG','4284','EPSG','16321',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1528','projected_crs','EPSG','2511','EPSG','1780','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2512','Pulkovo 1942 / Gauss-Kruger CM 129E',NULL,'EPSG','4530','EPSG','4284','EPSG','16322',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1529','projected_crs','EPSG','2512','EPSG','1781','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2513','Pulkovo 1942 / Gauss-Kruger CM 135E',NULL,'EPSG','4530','EPSG','4284','EPSG','16323',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1530','projected_crs','EPSG','2513','EPSG','1782','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2514','Pulkovo 1942 / Gauss-Kruger CM 141E',NULL,'EPSG','4530','EPSG','4284','EPSG','16324',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1531','projected_crs','EPSG','2514','EPSG','1783','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2515','Pulkovo 1942 / Gauss-Kruger CM 147E',NULL,'EPSG','4530','EPSG','4284','EPSG','16325',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1532','projected_crs','EPSG','2515','EPSG','1784','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2516','Pulkovo 1942 / Gauss-Kruger CM 153E',NULL,'EPSG','4530','EPSG','4284','EPSG','16326',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1533','projected_crs','EPSG','2516','EPSG','1785','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2517','Pulkovo 1942 / Gauss-Kruger CM 159E',NULL,'EPSG','4530','EPSG','4284','EPSG','16327',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1534','projected_crs','EPSG','2517','EPSG','1786','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2518','Pulkovo 1942 / Gauss-Kruger CM 165E',NULL,'EPSG','4530','EPSG','4284','EPSG','16328',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1535','projected_crs','EPSG','2518','EPSG','1787','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2519','Pulkovo 1942 / Gauss-Kruger CM 171E',NULL,'EPSG','4530','EPSG','4284','EPSG','16329',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1536','projected_crs','EPSG','2519','EPSG','1788','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2520','Pulkovo 1942 / Gauss-Kruger CM 177E',NULL,'EPSG','4530','EPSG','4284','EPSG','16330',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1537','projected_crs','EPSG','2520','EPSG','1789','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2521','Pulkovo 1942 / Gauss-Kruger CM 177W',NULL,'EPSG','4530','EPSG','4284','EPSG','16331',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1538','projected_crs','EPSG','2521','EPSG','1790','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2522','Pulkovo 1942 / Gauss-Kruger CM 171W',NULL,'EPSG','4530','EPSG','4284','EPSG','16332',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1539','projected_crs','EPSG','2522','EPSG','1791','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2523','Pulkovo 1942 / 3-degree Gauss-Kruger zone 7',NULL,'EPSG','4530','EPSG','4284','EPSG','16267',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1540','projected_crs','EPSG','2523','EPSG','2653','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2524','Pulkovo 1942 / 3-degree Gauss-Kruger zone 8',NULL,'EPSG','4530','EPSG','4284','EPSG','16268',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1541','projected_crs','EPSG','2524','EPSG','2654','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2525','Pulkovo 1942 / 3-degree Gauss-Kruger zone 9',NULL,'EPSG','4530','EPSG','4284','EPSG','16269',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1542','projected_crs','EPSG','2525','EPSG','2655','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2526','Pulkovo 1942 / 3-degree Gauss-Kruger zone 10',NULL,'EPSG','4530','EPSG','4284','EPSG','16270',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1543','projected_crs','EPSG','2526','EPSG','2656','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2527','Pulkovo 1942 / 3-degree Gauss-Kruger zone 11',NULL,'EPSG','4530','EPSG','4284','EPSG','16271',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1544','projected_crs','EPSG','2527','EPSG','2657','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2528','Pulkovo 1942 / 3-degree Gauss-Kruger zone 12',NULL,'EPSG','4530','EPSG','4284','EPSG','16272',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1545','projected_crs','EPSG','2528','EPSG','2658','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2529','Pulkovo 1942 / 3-degree Gauss-Kruger zone 13',NULL,'EPSG','4530','EPSG','4284','EPSG','16273',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1546','projected_crs','EPSG','2529','EPSG','2659','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2530','Pulkovo 1942 / 3-degree Gauss-Kruger zone 14',NULL,'EPSG','4530','EPSG','4284','EPSG','16274',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1547','projected_crs','EPSG','2530','EPSG','2660','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2531','Pulkovo 1942 / 3-degree Gauss-Kruger zone 15',NULL,'EPSG','4530','EPSG','4284','EPSG','16275',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1548','projected_crs','EPSG','2531','EPSG','2661','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2532','Pulkovo 1942 / 3-degree Gauss-Kruger zone 16',NULL,'EPSG','4530','EPSG','4284','EPSG','16276',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1549','projected_crs','EPSG','2532','EPSG','2662','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2533','Pulkovo 1942 / 3-degree Gauss-Kruger zone 17',NULL,'EPSG','4530','EPSG','4284','EPSG','16277',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1550','projected_crs','EPSG','2533','EPSG','2663','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2534','Pulkovo 1942 / 3-degree Gauss-Kruger zone 18',NULL,'EPSG','4530','EPSG','4284','EPSG','16278',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1551','projected_crs','EPSG','2534','EPSG','2664','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2535','Pulkovo 1942 / 3-degree Gauss-Kruger zone 19',NULL,'EPSG','4530','EPSG','4284','EPSG','16279',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1552','projected_crs','EPSG','2535','EPSG','2665','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2536','Pulkovo 1942 / 3-degree Gauss-Kruger zone 20',NULL,'EPSG','4530','EPSG','4284','EPSG','16280',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1553','projected_crs','EPSG','2536','EPSG','2666','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2537','Pulkovo 1942 / 3-degree Gauss-Kruger zone 21',NULL,'EPSG','4530','EPSG','4284','EPSG','16281',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1554','projected_crs','EPSG','2537','EPSG','2667','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2538','Pulkovo 1942 / 3-degree Gauss-Kruger zone 22',NULL,'EPSG','4530','EPSG','4284','EPSG','16282',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1555','projected_crs','EPSG','2538','EPSG','2668','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2539','Pulkovo 1942 / 3-degree Gauss-Kruger zone 23',NULL,'EPSG','4530','EPSG','4284','EPSG','16283',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1556','projected_crs','EPSG','2539','EPSG','2669','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2540','Pulkovo 1942 / 3-degree Gauss-Kruger zone 24',NULL,'EPSG','4530','EPSG','4284','EPSG','16284',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1557','projected_crs','EPSG','2540','EPSG','2670','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2541','Pulkovo 1942 / 3-degree Gauss-Kruger zone 25',NULL,'EPSG','4530','EPSG','4284','EPSG','16285',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1558','projected_crs','EPSG','2541','EPSG','2671','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2542','Pulkovo 1942 / 3-degree Gauss-Kruger zone 26',NULL,'EPSG','4530','EPSG','4284','EPSG','16286',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1559','projected_crs','EPSG','2542','EPSG','2672','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2543','Pulkovo 1942 / 3-degree Gauss-Kruger zone 27',NULL,'EPSG','4530','EPSG','4284','EPSG','16287',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1560','projected_crs','EPSG','2543','EPSG','2673','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2544','Pulkovo 1942 / 3-degree Gauss-Kruger zone 28',NULL,'EPSG','4530','EPSG','4284','EPSG','16288',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1561','projected_crs','EPSG','2544','EPSG','2674','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2545','Pulkovo 1942 / 3-degree Gauss-Kruger zone 29',NULL,'EPSG','4530','EPSG','4284','EPSG','16289',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1562','projected_crs','EPSG','2545','EPSG','2675','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2546','Pulkovo 1942 / 3-degree Gauss-Kruger zone 30',NULL,'EPSG','4530','EPSG','4284','EPSG','16290',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1563','projected_crs','EPSG','2546','EPSG','2676','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2547','Pulkovo 1942 / 3-degree Gauss-Kruger zone 31',NULL,'EPSG','4530','EPSG','4284','EPSG','16291',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1564','projected_crs','EPSG','2547','EPSG','2677','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2548','Pulkovo 1942 / 3-degree Gauss-Kruger zone 32',NULL,'EPSG','4530','EPSG','4284','EPSG','16292',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1565','projected_crs','EPSG','2548','EPSG','2678','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2549','Pulkovo 1942 / 3-degree Gauss-Kruger zone 33',NULL,'EPSG','4530','EPSG','4284','EPSG','16293',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1566','projected_crs','EPSG','2549','EPSG','2679','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2550','Samboja / UTM zone 50S',NULL,'EPSG','4400','EPSG','4125','EPSG','16150',NULL,1);
INSERT INTO "usage" VALUES('EPSG','1567','projected_crs','EPSG','2550','EPSG','1328','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2551','Pulkovo 1942 / 3-degree Gauss-Kruger zone 34',NULL,'EPSG','4530','EPSG','4284','EPSG','16294',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1568','projected_crs','EPSG','2551','EPSG','2680','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2552','Pulkovo 1942 / 3-degree Gauss-Kruger zone 35',NULL,'EPSG','4530','EPSG','4284','EPSG','16295',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1569','projected_crs','EPSG','2552','EPSG','2681','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2553','Pulkovo 1942 / 3-degree Gauss-Kruger zone 36',NULL,'EPSG','4530','EPSG','4284','EPSG','16296',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1570','projected_crs','EPSG','2553','EPSG','2682','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2554','Pulkovo 1942 / 3-degree Gauss-Kruger zone 37',NULL,'EPSG','4530','EPSG','4284','EPSG','16297',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1571','projected_crs','EPSG','2554','EPSG','2683','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2555','Pulkovo 1942 / 3-degree Gauss-Kruger zone 38',NULL,'EPSG','4530','EPSG','4284','EPSG','16298',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1572','projected_crs','EPSG','2555','EPSG','2684','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2556','Pulkovo 1942 / 3-degree Gauss-Kruger zone 39',NULL,'EPSG','4530','EPSG','4284','EPSG','16299',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1573','projected_crs','EPSG','2556','EPSG','2685','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2557','Pulkovo 1942 / 3-degree Gauss-Kruger zone 40',NULL,'EPSG','4530','EPSG','4284','EPSG','16070',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1574','projected_crs','EPSG','2557','EPSG','2686','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2558','Pulkovo 1942 / 3-degree Gauss-Kruger zone 41',NULL,'EPSG','4530','EPSG','4284','EPSG','16071',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1575','projected_crs','EPSG','2558','EPSG','2687','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2559','Pulkovo 1942 / 3-degree Gauss-Kruger zone 42',NULL,'EPSG','4530','EPSG','4284','EPSG','16072',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1576','projected_crs','EPSG','2559','EPSG','2688','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2560','Pulkovo 1942 / 3-degree Gauss-Kruger zone 43',NULL,'EPSG','4530','EPSG','4284','EPSG','16073',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1577','projected_crs','EPSG','2560','EPSG','2689','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2561','Pulkovo 1942 / 3-degree Gauss-Kruger zone 44',NULL,'EPSG','4530','EPSG','4284','EPSG','16074',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1578','projected_crs','EPSG','2561','EPSG','2690','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2562','Pulkovo 1942 / 3-degree Gauss-Kruger zone 45',NULL,'EPSG','4530','EPSG','4284','EPSG','16075',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1579','projected_crs','EPSG','2562','EPSG','2691','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2563','Pulkovo 1942 / 3-degree Gauss-Kruger zone 46',NULL,'EPSG','4530','EPSG','4284','EPSG','16076',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1580','projected_crs','EPSG','2563','EPSG','2692','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2564','Pulkovo 1942 / 3-degree Gauss-Kruger zone 47',NULL,'EPSG','4530','EPSG','4284','EPSG','16077',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1581','projected_crs','EPSG','2564','EPSG','2693','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2565','Pulkovo 1942 / 3-degree Gauss-Kruger zone 48',NULL,'EPSG','4530','EPSG','4284','EPSG','16078',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1582','projected_crs','EPSG','2565','EPSG','2694','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2566','Pulkovo 1942 / 3-degree Gauss-Kruger zone 49',NULL,'EPSG','4530','EPSG','4284','EPSG','16079',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1583','projected_crs','EPSG','2566','EPSG','2695','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2567','Pulkovo 1942 / 3-degree Gauss-Kruger zone 50',NULL,'EPSG','4530','EPSG','4284','EPSG','16080',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1584','projected_crs','EPSG','2567','EPSG','2696','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2568','Pulkovo 1942 / 3-degree Gauss-Kruger zone 51',NULL,'EPSG','4530','EPSG','4284','EPSG','16081',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1585','projected_crs','EPSG','2568','EPSG','2697','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2569','Pulkovo 1942 / 3-degree Gauss-Kruger zone 52',NULL,'EPSG','4530','EPSG','4284','EPSG','16082',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1586','projected_crs','EPSG','2569','EPSG','2698','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2570','Pulkovo 1942 / 3-degree Gauss-Kruger zone 53',NULL,'EPSG','4530','EPSG','4284','EPSG','16083',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1587','projected_crs','EPSG','2570','EPSG','2699','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2571','Pulkovo 1942 / 3-degree Gauss-Kruger zone 54',NULL,'EPSG','4530','EPSG','4284','EPSG','16084',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1588','projected_crs','EPSG','2571','EPSG','2700','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2572','Pulkovo 1942 / 3-degree Gauss-Kruger zone 55',NULL,'EPSG','4530','EPSG','4284','EPSG','16085',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1589','projected_crs','EPSG','2572','EPSG','2701','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2573','Pulkovo 1942 / 3-degree Gauss-Kruger zone 56',NULL,'EPSG','4530','EPSG','4284','EPSG','16086',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1590','projected_crs','EPSG','2573','EPSG','2702','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2574','Pulkovo 1942 / 3-degree Gauss-Kruger zone 57',NULL,'EPSG','4530','EPSG','4284','EPSG','16087',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1591','projected_crs','EPSG','2574','EPSG','2703','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2575','Pulkovo 1942 / 3-degree Gauss-Kruger zone 58',NULL,'EPSG','4530','EPSG','4284','EPSG','16088',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1592','projected_crs','EPSG','2575','EPSG','2704','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2576','Pulkovo 1942 / 3-degree Gauss-Kruger zone 59',NULL,'EPSG','4530','EPSG','4284','EPSG','16089',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1593','projected_crs','EPSG','2576','EPSG','2705','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2577','Pulkovo 1942 / 3-degree Gauss-Kruger zone 60',NULL,'EPSG','4530','EPSG','4284','EPSG','16090',NULL,1);
INSERT INTO "usage" VALUES('EPSG','1594','projected_crs','EPSG','2577','EPSG','2706','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2578','Pulkovo 1942 / 3-degree Gauss-Kruger zone 61',NULL,'EPSG','4530','EPSG','4284','EPSG','16091',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1595','projected_crs','EPSG','2578','EPSG','2707','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2579','Pulkovo 1942 / 3-degree Gauss-Kruger zone 62',NULL,'EPSG','4530','EPSG','4284','EPSG','16092',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1596','projected_crs','EPSG','2579','EPSG','2708','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2580','Pulkovo 1942 / 3-degree Gauss-Kruger zone 63',NULL,'EPSG','4530','EPSG','4284','EPSG','16093',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1597','projected_crs','EPSG','2580','EPSG','2709','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2581','Pulkovo 1942 / 3-degree Gauss-Kruger zone 64',NULL,'EPSG','4530','EPSG','4284','EPSG','16094',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1598','projected_crs','EPSG','2581','EPSG','2710','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2582','Pulkovo 1942 / 3-degree Gauss-Kruger CM 21E',NULL,'EPSG','4530','EPSG','4284','EPSG','16304',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1599','projected_crs','EPSG','2582','EPSG','2653','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2583','Pulkovo 1942 / 3-degree Gauss-Kruger CM 24E',NULL,'EPSG','4530','EPSG','4284','EPSG','16368',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1600','projected_crs','EPSG','2583','EPSG','2654','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2584','Pulkovo 1942 / 3-degree Gauss-Kruger CM 27E',NULL,'EPSG','4530','EPSG','4284','EPSG','16305',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1601','projected_crs','EPSG','2584','EPSG','2655','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2585','Pulkovo 1942 / 3-degree Gauss-Kruger CM 30E',NULL,'EPSG','4530','EPSG','4284','EPSG','16370',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1602','projected_crs','EPSG','2585','EPSG','2656','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2586','Pulkovo 1942 / 3-degree Gauss-Kruger CM 33E',NULL,'EPSG','4530','EPSG','4284','EPSG','16306',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1603','projected_crs','EPSG','2586','EPSG','2657','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2587','Pulkovo 1942 / 3-degree Gauss-Kruger CM 36E',NULL,'EPSG','4530','EPSG','4284','EPSG','16372',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1604','projected_crs','EPSG','2587','EPSG','2658','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2588','Pulkovo 1942 / 3-degree Gauss-Kruger CM 39E',NULL,'EPSG','4530','EPSG','4284','EPSG','16307',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1605','projected_crs','EPSG','2588','EPSG','2659','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2589','Pulkovo 1942 / 3-degree Gauss-Kruger CM 42E',NULL,'EPSG','4530','EPSG','4284','EPSG','16374',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1606','projected_crs','EPSG','2589','EPSG','2660','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2590','Pulkovo 1942 / 3-degree Gauss-Kruger CM 45E',NULL,'EPSG','4530','EPSG','4284','EPSG','16308',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1607','projected_crs','EPSG','2590','EPSG','2661','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2591','Pulkovo 1942 / 3-degree Gauss-Kruger CM 48E',NULL,'EPSG','4530','EPSG','4284','EPSG','16376',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1608','projected_crs','EPSG','2591','EPSG','2662','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2592','Pulkovo 1942 / 3-degree Gauss-Kruger CM 51E',NULL,'EPSG','4530','EPSG','4284','EPSG','16309',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1609','projected_crs','EPSG','2592','EPSG','2663','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2593','Pulkovo 1942 / 3-degree Gauss-Kruger CM 54E',NULL,'EPSG','4530','EPSG','4284','EPSG','16378',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1610','projected_crs','EPSG','2593','EPSG','2664','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2594','Pulkovo 1942 / 3-degree Gauss-Kruger CM 57E',NULL,'EPSG','4530','EPSG','4284','EPSG','16310',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1611','projected_crs','EPSG','2594','EPSG','2665','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2595','Pulkovo 1942 / 3-degree Gauss-Kruger CM 60E',NULL,'EPSG','4530','EPSG','4284','EPSG','16380',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1612','projected_crs','EPSG','2595','EPSG','2666','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2596','Pulkovo 1942 / 3-degree Gauss-Kruger CM 63E',NULL,'EPSG','4530','EPSG','4284','EPSG','16311',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1613','projected_crs','EPSG','2596','EPSG','2667','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2597','Pulkovo 1942 / 3-degree Gauss-Kruger CM 66E',NULL,'EPSG','4530','EPSG','4284','EPSG','16382',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1614','projected_crs','EPSG','2597','EPSG','2668','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2598','Pulkovo 1942 / 3-degree Gauss-Kruger CM 69E',NULL,'EPSG','4530','EPSG','4284','EPSG','16312',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1615','projected_crs','EPSG','2598','EPSG','2669','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2599','Pulkovo 1942 / 3-degree Gauss-Kruger CM 72E',NULL,'EPSG','4530','EPSG','4284','EPSG','16384',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1616','projected_crs','EPSG','2599','EPSG','2670','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2600','Lietuvos Koordinoei Sistema 1994',NULL,'EPSG','4530','EPSG','4669','EPSG','19934',NULL,1);
INSERT INTO "usage" VALUES('EPSG','1617','projected_crs','EPSG','2600','EPSG','1145','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2601','Pulkovo 1942 / 3-degree Gauss-Kruger CM 75E',NULL,'EPSG','4530','EPSG','4284','EPSG','16313',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1618','projected_crs','EPSG','2601','EPSG','2671','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2602','Pulkovo 1942 / 3-degree Gauss-Kruger CM 78E',NULL,'EPSG','4530','EPSG','4284','EPSG','16386',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1619','projected_crs','EPSG','2602','EPSG','2672','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2603','Pulkovo 1942 / 3-degree Gauss-Kruger CM 81E',NULL,'EPSG','4530','EPSG','4284','EPSG','16314',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1620','projected_crs','EPSG','2603','EPSG','2673','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2604','Pulkovo 1942 / 3-degree Gauss-Kruger CM 84E',NULL,'EPSG','4530','EPSG','4284','EPSG','16388',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1621','projected_crs','EPSG','2604','EPSG','2674','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2605','Pulkovo 1942 / 3-degree Gauss-Kruger CM 87E',NULL,'EPSG','4530','EPSG','4284','EPSG','16315',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1622','projected_crs','EPSG','2605','EPSG','2675','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2606','Pulkovo 1942 / 3-degree Gauss-Kruger CM 90E',NULL,'EPSG','4530','EPSG','4284','EPSG','16390',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1623','projected_crs','EPSG','2606','EPSG','2676','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2607','Pulkovo 1942 / 3-degree Gauss-Kruger CM 93E',NULL,'EPSG','4530','EPSG','4284','EPSG','16316',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1624','projected_crs','EPSG','2607','EPSG','2677','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2608','Pulkovo 1942 / 3-degree Gauss-Kruger CM 96E',NULL,'EPSG','4530','EPSG','4284','EPSG','16392',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1625','projected_crs','EPSG','2608','EPSG','2678','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2609','Pulkovo 1942 / 3-degree Gauss-Kruger CM 99E',NULL,'EPSG','4530','EPSG','4284','EPSG','16317',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1626','projected_crs','EPSG','2609','EPSG','2679','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2610','Pulkovo 1942 / 3-degree Gauss-Kruger CM 102E',NULL,'EPSG','4530','EPSG','4284','EPSG','16394',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1627','projected_crs','EPSG','2610','EPSG','2680','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2611','Pulkovo 1942 / 3-degree Gauss-Kruger CM 105E',NULL,'EPSG','4530','EPSG','4284','EPSG','16318',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1628','projected_crs','EPSG','2611','EPSG','2681','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2612','Pulkovo 1942 / 3-degree Gauss-Kruger CM 108E',NULL,'EPSG','4530','EPSG','4284','EPSG','16396',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1629','projected_crs','EPSG','2612','EPSG','2682','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2613','Pulkovo 1942 / 3-degree Gauss-Kruger CM 111E',NULL,'EPSG','4530','EPSG','4284','EPSG','16319',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1630','projected_crs','EPSG','2613','EPSG','2683','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2614','Pulkovo 1942 / 3-degree Gauss-Kruger CM 114E',NULL,'EPSG','4530','EPSG','4284','EPSG','16398',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1631','projected_crs','EPSG','2614','EPSG','2684','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2615','Pulkovo 1942 / 3-degree Gauss-Kruger CM 117E',NULL,'EPSG','4530','EPSG','4284','EPSG','16320',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1632','projected_crs','EPSG','2615','EPSG','2685','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2616','Pulkovo 1942 / 3-degree Gauss-Kruger CM 120E',NULL,'EPSG','4530','EPSG','4284','EPSG','16170',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1633','projected_crs','EPSG','2616','EPSG','2686','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2617','Pulkovo 1942 / 3-degree Gauss-Kruger CM 123E',NULL,'EPSG','4530','EPSG','4284','EPSG','16321',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1634','projected_crs','EPSG','2617','EPSG','2687','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2618','Pulkovo 1942 / 3-degree Gauss-Kruger CM 126E',NULL,'EPSG','4530','EPSG','4284','EPSG','16172',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1635','projected_crs','EPSG','2618','EPSG','2688','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2619','Pulkovo 1942 / 3-degree Gauss-Kruger CM 129E',NULL,'EPSG','4530','EPSG','4284','EPSG','16322',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1636','projected_crs','EPSG','2619','EPSG','2689','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2620','Pulkovo 1942 / 3-degree Gauss-Kruger CM 132E',NULL,'EPSG','4530','EPSG','4284','EPSG','16174',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1637','projected_crs','EPSG','2620','EPSG','2690','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2621','Pulkovo 1942 / 3-degree Gauss-Kruger CM 135E',NULL,'EPSG','4530','EPSG','4284','EPSG','16323',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1638','projected_crs','EPSG','2621','EPSG','2691','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2622','Pulkovo 1942 / 3-degree Gauss-Kruger CM 138E',NULL,'EPSG','4530','EPSG','4284','EPSG','16176',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1639','projected_crs','EPSG','2622','EPSG','2692','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2623','Pulkovo 1942 / 3-degree Gauss-Kruger CM 141E',NULL,'EPSG','4530','EPSG','4284','EPSG','16324',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1640','projected_crs','EPSG','2623','EPSG','2693','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2624','Pulkovo 1942 / 3-degree Gauss-Kruger CM 144E',NULL,'EPSG','4530','EPSG','4284','EPSG','16178',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1641','projected_crs','EPSG','2624','EPSG','2694','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2625','Pulkovo 1942 / 3-degree Gauss-Kruger CM 147E',NULL,'EPSG','4530','EPSG','4284','EPSG','16325',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1642','projected_crs','EPSG','2625','EPSG','2695','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2626','Pulkovo 1942 / 3-degree Gauss-Kruger CM 150E',NULL,'EPSG','4530','EPSG','4284','EPSG','16180',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1643','projected_crs','EPSG','2626','EPSG','2696','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2627','Pulkovo 1942 / 3-degree Gauss-Kruger CM 153E',NULL,'EPSG','4530','EPSG','4284','EPSG','16326',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1644','projected_crs','EPSG','2627','EPSG','2697','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2628','Pulkovo 1942 / 3-degree Gauss-Kruger CM 156E',NULL,'EPSG','4530','EPSG','4284','EPSG','16182',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1645','projected_crs','EPSG','2628','EPSG','2698','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2629','Pulkovo 1942 / 3-degree Gauss-Kruger CM 159E',NULL,'EPSG','4530','EPSG','4284','EPSG','16327',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1646','projected_crs','EPSG','2629','EPSG','2699','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2630','Pulkovo 1942 / 3-degree Gauss-Kruger CM 162E',NULL,'EPSG','4530','EPSG','4284','EPSG','16184',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1647','projected_crs','EPSG','2630','EPSG','2700','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2631','Pulkovo 1942 / 3-degree Gauss-Kruger CM 165E',NULL,'EPSG','4530','EPSG','4284','EPSG','16328',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1648','projected_crs','EPSG','2631','EPSG','2701','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2632','Pulkovo 1942 / 3-degree Gauss-Kruger CM 168E',NULL,'EPSG','4530','EPSG','4284','EPSG','16186',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1649','projected_crs','EPSG','2632','EPSG','2702','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2633','Pulkovo 1942 / 3-degree Gauss-Kruger CM 171E',NULL,'EPSG','4530','EPSG','4284','EPSG','16329',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1650','projected_crs','EPSG','2633','EPSG','2703','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2634','Pulkovo 1942 / 3-degree Gauss-Kruger CM 174E',NULL,'EPSG','4530','EPSG','4284','EPSG','16188',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1651','projected_crs','EPSG','2634','EPSG','2704','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2635','Pulkovo 1942 / 3-degree Gauss-Kruger CM 177E',NULL,'EPSG','4530','EPSG','4284','EPSG','16330',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1652','projected_crs','EPSG','2635','EPSG','2705','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2636','Pulkovo 1942 / 3-degree Gauss-Kruger CM 180E',NULL,'EPSG','4530','EPSG','4284','EPSG','16190',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1653','projected_crs','EPSG','2636','EPSG','2706','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2637','Pulkovo 1942 / 3-degree Gauss-Kruger CM 177W',NULL,'EPSG','4530','EPSG','4284','EPSG','16331',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1654','projected_crs','EPSG','2637','EPSG','2707','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2638','Pulkovo 1942 / 3-degree Gauss-Kruger CM 174W',NULL,'EPSG','4530','EPSG','4284','EPSG','16192',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1655','projected_crs','EPSG','2638','EPSG','2708','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2639','Pulkovo 1942 / 3-degree Gauss-Kruger CM 171W',NULL,'EPSG','4530','EPSG','4284','EPSG','16332',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1656','projected_crs','EPSG','2639','EPSG','2709','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2640','Pulkovo 1942 / 3-degree Gauss-Kruger CM 168W',NULL,'EPSG','4530','EPSG','4284','EPSG','16194',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1657','projected_crs','EPSG','2640','EPSG','2710','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2641','Pulkovo 1995 / 3-degree Gauss-Kruger zone 7',NULL,'EPSG','4530','EPSG','4200','EPSG','16267',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1658','projected_crs','EPSG','2641','EPSG','2747','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2642','Pulkovo 1995 / 3-degree Gauss-Kruger zone 8',NULL,'EPSG','4530','EPSG','4200','EPSG','16268',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1659','projected_crs','EPSG','2642','EPSG','2748','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2643','Pulkovo 1995 / 3-degree Gauss-Kruger zone 9',NULL,'EPSG','4530','EPSG','4200','EPSG','16269',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1660','projected_crs','EPSG','2643','EPSG','2749','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2644','Pulkovo 1995 / 3-degree Gauss-Kruger zone 10',NULL,'EPSG','4530','EPSG','4200','EPSG','16270',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1661','projected_crs','EPSG','2644','EPSG','2750','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2645','Pulkovo 1995 / 3-degree Gauss-Kruger zone 11',NULL,'EPSG','4530','EPSG','4200','EPSG','16271',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1662','projected_crs','EPSG','2645','EPSG','2751','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2646','Pulkovo 1995 / 3-degree Gauss-Kruger zone 12',NULL,'EPSG','4530','EPSG','4200','EPSG','16272',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1663','projected_crs','EPSG','2646','EPSG','2752','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2647','Pulkovo 1995 / 3-degree Gauss-Kruger zone 13',NULL,'EPSG','4530','EPSG','4200','EPSG','16273',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1664','projected_crs','EPSG','2647','EPSG','2753','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2648','Pulkovo 1995 / 3-degree Gauss-Kruger zone 14',NULL,'EPSG','4530','EPSG','4200','EPSG','16274',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1665','projected_crs','EPSG','2648','EPSG','2754','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2649','Pulkovo 1995 / 3-degree Gauss-Kruger zone 15',NULL,'EPSG','4530','EPSG','4200','EPSG','16275',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1666','projected_crs','EPSG','2649','EPSG','2755','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2650','Pulkovo 1995 / 3-degree Gauss-Kruger zone 16',NULL,'EPSG','4530','EPSG','4200','EPSG','16276',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1667','projected_crs','EPSG','2650','EPSG','2756','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2651','Pulkovo 1995 / 3-degree Gauss-Kruger zone 17',NULL,'EPSG','4530','EPSG','4200','EPSG','16277',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1668','projected_crs','EPSG','2651','EPSG','2757','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2652','Pulkovo 1995 / 3-degree Gauss-Kruger zone 18',NULL,'EPSG','4530','EPSG','4200','EPSG','16278',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1669','projected_crs','EPSG','2652','EPSG','2758','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2653','Pulkovo 1995 / 3-degree Gauss-Kruger zone 19',NULL,'EPSG','4530','EPSG','4200','EPSG','16279',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1670','projected_crs','EPSG','2653','EPSG','2759','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2654','Pulkovo 1995 / 3-degree Gauss-Kruger zone 20',NULL,'EPSG','4530','EPSG','4200','EPSG','16280',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1671','projected_crs','EPSG','2654','EPSG','2760','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2655','Pulkovo 1995 / 3-degree Gauss-Kruger zone 21',NULL,'EPSG','4530','EPSG','4200','EPSG','16281',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1672','projected_crs','EPSG','2655','EPSG','2761','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2656','Pulkovo 1995 / 3-degree Gauss-Kruger zone 22',NULL,'EPSG','4530','EPSG','4200','EPSG','16282',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1673','projected_crs','EPSG','2656','EPSG','2762','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2657','Pulkovo 1995 / 3-degree Gauss-Kruger zone 23',NULL,'EPSG','4530','EPSG','4200','EPSG','16283',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1674','projected_crs','EPSG','2657','EPSG','2763','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2658','Pulkovo 1995 / 3-degree Gauss-Kruger zone 24',NULL,'EPSG','4530','EPSG','4200','EPSG','16284',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1675','projected_crs','EPSG','2658','EPSG','2764','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2659','Pulkovo 1995 / 3-degree Gauss-Kruger zone 25',NULL,'EPSG','4530','EPSG','4200','EPSG','16285',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1676','projected_crs','EPSG','2659','EPSG','2765','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2660','Pulkovo 1995 / 3-degree Gauss-Kruger zone 26',NULL,'EPSG','4530','EPSG','4200','EPSG','16286',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1677','projected_crs','EPSG','2660','EPSG','2766','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2661','Pulkovo 1995 / 3-degree Gauss-Kruger zone 27',NULL,'EPSG','4530','EPSG','4200','EPSG','16287',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1678','projected_crs','EPSG','2661','EPSG','2767','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2662','Pulkovo 1995 / 3-degree Gauss-Kruger zone 28',NULL,'EPSG','4530','EPSG','4200','EPSG','16288',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1679','projected_crs','EPSG','2662','EPSG','2768','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2663','Pulkovo 1995 / 3-degree Gauss-Kruger zone 29',NULL,'EPSG','4530','EPSG','4200','EPSG','16289',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1680','projected_crs','EPSG','2663','EPSG','2769','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2664','Pulkovo 1995 / 3-degree Gauss-Kruger zone 30',NULL,'EPSG','4530','EPSG','4200','EPSG','16290',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1681','projected_crs','EPSG','2664','EPSG','2676','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2665','Pulkovo 1995 / 3-degree Gauss-Kruger zone 31',NULL,'EPSG','4530','EPSG','4200','EPSG','16291',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1682','projected_crs','EPSG','2665','EPSG','2677','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2666','Pulkovo 1995 / 3-degree Gauss-Kruger zone 32',NULL,'EPSG','4530','EPSG','4200','EPSG','16292',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1683','projected_crs','EPSG','2666','EPSG','2678','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2667','Pulkovo 1995 / 3-degree Gauss-Kruger zone 33',NULL,'EPSG','4530','EPSG','4200','EPSG','16293',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1684','projected_crs','EPSG','2667','EPSG','2679','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2668','Pulkovo 1995 / 3-degree Gauss-Kruger zone 34',NULL,'EPSG','4530','EPSG','4200','EPSG','16294',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1685','projected_crs','EPSG','2668','EPSG','2680','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2669','Pulkovo 1995 / 3-degree Gauss-Kruger zone 35',NULL,'EPSG','4530','EPSG','4200','EPSG','16295',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1686','projected_crs','EPSG','2669','EPSG','2681','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2670','Pulkovo 1995 / 3-degree Gauss-Kruger zone 36',NULL,'EPSG','4530','EPSG','4200','EPSG','16296',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1687','projected_crs','EPSG','2670','EPSG','2682','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2671','Pulkovo 1995 / 3-degree Gauss-Kruger zone 37',NULL,'EPSG','4530','EPSG','4200','EPSG','16297',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1688','projected_crs','EPSG','2671','EPSG','2683','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2672','Pulkovo 1995 / 3-degree Gauss-Kruger zone 38',NULL,'EPSG','4530','EPSG','4200','EPSG','16298',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1689','projected_crs','EPSG','2672','EPSG','2684','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2673','Pulkovo 1995 / 3-degree Gauss-Kruger zone 39',NULL,'EPSG','4530','EPSG','4200','EPSG','16299',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1690','projected_crs','EPSG','2673','EPSG','2685','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2674','Pulkovo 1995 / 3-degree Gauss-Kruger zone 40',NULL,'EPSG','4530','EPSG','4200','EPSG','16070',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1691','projected_crs','EPSG','2674','EPSG','2686','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2675','Pulkovo 1995 / 3-degree Gauss-Kruger zone 41',NULL,'EPSG','4530','EPSG','4200','EPSG','16071',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1692','projected_crs','EPSG','2675','EPSG','2687','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2676','Pulkovo 1995 / 3-degree Gauss-Kruger zone 42',NULL,'EPSG','4530','EPSG','4200','EPSG','16072',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1693','projected_crs','EPSG','2676','EPSG','2688','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2677','Pulkovo 1995 / 3-degree Gauss-Kruger zone 43',NULL,'EPSG','4530','EPSG','4200','EPSG','16073',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1694','projected_crs','EPSG','2677','EPSG','2689','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2678','Pulkovo 1995 / 3-degree Gauss-Kruger zone 44',NULL,'EPSG','4530','EPSG','4200','EPSG','16074',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1695','projected_crs','EPSG','2678','EPSG','2690','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2679','Pulkovo 1995 / 3-degree Gauss-Kruger zone 45',NULL,'EPSG','4530','EPSG','4200','EPSG','16075',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1696','projected_crs','EPSG','2679','EPSG','2691','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2680','Pulkovo 1995 / 3-degree Gauss-Kruger zone 46',NULL,'EPSG','4530','EPSG','4200','EPSG','16076',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1697','projected_crs','EPSG','2680','EPSG','2692','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2681','Pulkovo 1995 / 3-degree Gauss-Kruger zone 47',NULL,'EPSG','4530','EPSG','4200','EPSG','16077',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1698','projected_crs','EPSG','2681','EPSG','2693','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2682','Pulkovo 1995 / 3-degree Gauss-Kruger zone 48',NULL,'EPSG','4530','EPSG','4200','EPSG','16078',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1699','projected_crs','EPSG','2682','EPSG','2694','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2683','Pulkovo 1995 / 3-degree Gauss-Kruger zone 49',NULL,'EPSG','4530','EPSG','4200','EPSG','16079',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1700','projected_crs','EPSG','2683','EPSG','2695','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2684','Pulkovo 1995 / 3-degree Gauss-Kruger zone 50',NULL,'EPSG','4530','EPSG','4200','EPSG','16080',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1701','projected_crs','EPSG','2684','EPSG','2696','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2685','Pulkovo 1995 / 3-degree Gauss-Kruger zone 51',NULL,'EPSG','4530','EPSG','4200','EPSG','16081',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1702','projected_crs','EPSG','2685','EPSG','2697','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2686','Pulkovo 1995 / 3-degree Gauss-Kruger zone 52',NULL,'EPSG','4530','EPSG','4200','EPSG','16082',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1703','projected_crs','EPSG','2686','EPSG','2698','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2687','Pulkovo 1995 / 3-degree Gauss-Kruger zone 53',NULL,'EPSG','4530','EPSG','4200','EPSG','16083',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1704','projected_crs','EPSG','2687','EPSG','2699','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2688','Pulkovo 1995 / 3-degree Gauss-Kruger zone 54',NULL,'EPSG','4530','EPSG','4200','EPSG','16084',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1705','projected_crs','EPSG','2688','EPSG','2700','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2689','Pulkovo 1995 / 3-degree Gauss-Kruger zone 55',NULL,'EPSG','4530','EPSG','4200','EPSG','16085',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1706','projected_crs','EPSG','2689','EPSG','2701','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2690','Pulkovo 1995 / 3-degree Gauss-Kruger zone 56',NULL,'EPSG','4530','EPSG','4200','EPSG','16086',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1707','projected_crs','EPSG','2690','EPSG','2702','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2691','Pulkovo 1995 / 3-degree Gauss-Kruger zone 57',NULL,'EPSG','4530','EPSG','4200','EPSG','16087',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1708','projected_crs','EPSG','2691','EPSG','2703','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2692','Pulkovo 1995 / 3-degree Gauss-Kruger zone 58',NULL,'EPSG','4530','EPSG','4200','EPSG','16088',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1709','projected_crs','EPSG','2692','EPSG','2704','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2693','Pulkovo 1995 / 3-degree Gauss-Kruger zone 59',NULL,'EPSG','4530','EPSG','4200','EPSG','16089',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1710','projected_crs','EPSG','2693','EPSG','2705','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2694','Pulkovo 1995 / 3-degree Gauss-Kruger zone 60',NULL,'EPSG','4530','EPSG','4200','EPSG','16090',NULL,1);
INSERT INTO "usage" VALUES('EPSG','1711','projected_crs','EPSG','2694','EPSG','2706','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2695','Pulkovo 1995 / 3-degree Gauss-Kruger zone 61',NULL,'EPSG','4530','EPSG','4200','EPSG','16091',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1712','projected_crs','EPSG','2695','EPSG','2707','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2696','Pulkovo 1995 / 3-degree Gauss-Kruger zone 62',NULL,'EPSG','4530','EPSG','4200','EPSG','16092',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1713','projected_crs','EPSG','2696','EPSG','2708','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2697','Pulkovo 1995 / 3-degree Gauss-Kruger zone 63',NULL,'EPSG','4530','EPSG','4200','EPSG','16093',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1714','projected_crs','EPSG','2697','EPSG','2709','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2698','Pulkovo 1995 / 3-degree Gauss-Kruger zone 64',NULL,'EPSG','4530','EPSG','4200','EPSG','16094',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1715','projected_crs','EPSG','2698','EPSG','2710','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2699','Pulkovo 1995 / 3-degree Gauss-Kruger CM 21E',NULL,'EPSG','4530','EPSG','4200','EPSG','16304',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1716','projected_crs','EPSG','2699','EPSG','2747','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2700','Pulkovo 1995 / 3-degree Gauss-Kruger CM 24E',NULL,'EPSG','4530','EPSG','4200','EPSG','16368',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1717','projected_crs','EPSG','2700','EPSG','2748','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2701','Pulkovo 1995 / 3-degree Gauss-Kruger CM 27E',NULL,'EPSG','4530','EPSG','4200','EPSG','16305',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1718','projected_crs','EPSG','2701','EPSG','2749','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2702','Pulkovo 1995 / 3-degree Gauss-Kruger CM 30E',NULL,'EPSG','4530','EPSG','4200','EPSG','16370',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1719','projected_crs','EPSG','2702','EPSG','2750','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2703','Pulkovo 1995 / 3-degree Gauss-Kruger CM 33E',NULL,'EPSG','4530','EPSG','4200','EPSG','16306',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1720','projected_crs','EPSG','2703','EPSG','2751','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2704','Pulkovo 1995 / 3-degree Gauss-Kruger CM 36E',NULL,'EPSG','4530','EPSG','4200','EPSG','16372',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1721','projected_crs','EPSG','2704','EPSG','2752','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2705','Pulkovo 1995 / 3-degree Gauss-Kruger CM 39E',NULL,'EPSG','4530','EPSG','4200','EPSG','16307',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1722','projected_crs','EPSG','2705','EPSG','2753','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2706','Pulkovo 1995 / 3-degree Gauss-Kruger CM 42E',NULL,'EPSG','4530','EPSG','4200','EPSG','16374',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1723','projected_crs','EPSG','2706','EPSG','2754','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2707','Pulkovo 1995 / 3-degree Gauss-Kruger CM 45E',NULL,'EPSG','4530','EPSG','4200','EPSG','16308',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1724','projected_crs','EPSG','2707','EPSG','2755','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2708','Pulkovo 1995 / 3-degree Gauss-Kruger CM 48E',NULL,'EPSG','4530','EPSG','4200','EPSG','16376',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1725','projected_crs','EPSG','2708','EPSG','2756','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2709','Pulkovo 1995 / 3-degree Gauss-Kruger CM 51E',NULL,'EPSG','4530','EPSG','4200','EPSG','16309',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1726','projected_crs','EPSG','2709','EPSG','2757','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2710','Pulkovo 1995 / 3-degree Gauss-Kruger CM 54E',NULL,'EPSG','4530','EPSG','4200','EPSG','16378',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1727','projected_crs','EPSG','2710','EPSG','2758','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2711','Pulkovo 1995 / 3-degree Gauss-Kruger CM 57E',NULL,'EPSG','4530','EPSG','4200','EPSG','16310',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1728','projected_crs','EPSG','2711','EPSG','2759','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2712','Pulkovo 1995 / 3-degree Gauss-Kruger CM 60E',NULL,'EPSG','4530','EPSG','4200','EPSG','16380',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1729','projected_crs','EPSG','2712','EPSG','2760','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2713','Pulkovo 1995 / 3-degree Gauss-Kruger CM 63E',NULL,'EPSG','4530','EPSG','4200','EPSG','16311',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1730','projected_crs','EPSG','2713','EPSG','2761','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2714','Pulkovo 1995 / 3-degree Gauss-Kruger CM 66E',NULL,'EPSG','4530','EPSG','4200','EPSG','16382',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1731','projected_crs','EPSG','2714','EPSG','2762','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2715','Pulkovo 1995 / 3-degree Gauss-Kruger CM 69E',NULL,'EPSG','4530','EPSG','4200','EPSG','16312',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1732','projected_crs','EPSG','2715','EPSG','2763','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2716','Pulkovo 1995 / 3-degree Gauss-Kruger CM 72E',NULL,'EPSG','4530','EPSG','4200','EPSG','16384',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1733','projected_crs','EPSG','2716','EPSG','2764','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2717','Pulkovo 1995 / 3-degree Gauss-Kruger CM 75E',NULL,'EPSG','4530','EPSG','4200','EPSG','16313',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1734','projected_crs','EPSG','2717','EPSG','2765','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2718','Pulkovo 1995 / 3-degree Gauss-Kruger CM 78E',NULL,'EPSG','4530','EPSG','4200','EPSG','16386',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1735','projected_crs','EPSG','2718','EPSG','2766','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2719','Pulkovo 1995 / 3-degree Gauss-Kruger CM 81E',NULL,'EPSG','4530','EPSG','4200','EPSG','16314',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1736','projected_crs','EPSG','2719','EPSG','2767','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2720','Pulkovo 1995 / 3-degree Gauss-Kruger CM 84E',NULL,'EPSG','4530','EPSG','4200','EPSG','16388',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1737','projected_crs','EPSG','2720','EPSG','2768','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2721','Pulkovo 1995 / 3-degree Gauss-Kruger CM 87E',NULL,'EPSG','4530','EPSG','4200','EPSG','16315',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1738','projected_crs','EPSG','2721','EPSG','2769','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2722','Pulkovo 1995 / 3-degree Gauss-Kruger CM 90E',NULL,'EPSG','4530','EPSG','4200','EPSG','16390',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1739','projected_crs','EPSG','2722','EPSG','2676','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2723','Pulkovo 1995 / 3-degree Gauss-Kruger CM 93E',NULL,'EPSG','4530','EPSG','4200','EPSG','16316',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1740','projected_crs','EPSG','2723','EPSG','2677','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2724','Pulkovo 1995 / 3-degree Gauss-Kruger CM 96E',NULL,'EPSG','4530','EPSG','4200','EPSG','16392',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1741','projected_crs','EPSG','2724','EPSG','2678','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2725','Pulkovo 1995 / 3-degree Gauss-Kruger CM 99E',NULL,'EPSG','4530','EPSG','4200','EPSG','16317',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1742','projected_crs','EPSG','2725','EPSG','2679','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2726','Pulkovo 1995 / 3-degree Gauss-Kruger CM 102E',NULL,'EPSG','4530','EPSG','4200','EPSG','16394',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1743','projected_crs','EPSG','2726','EPSG','2680','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2727','Pulkovo 1995 / 3-degree Gauss-Kruger CM 105E',NULL,'EPSG','4530','EPSG','4200','EPSG','16318',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1744','projected_crs','EPSG','2727','EPSG','2681','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2728','Pulkovo 1995 / 3-degree Gauss-Kruger CM 108E',NULL,'EPSG','4530','EPSG','4200','EPSG','16396',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1745','projected_crs','EPSG','2728','EPSG','2682','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2729','Pulkovo 1995 / 3-degree Gauss-Kruger CM 111E',NULL,'EPSG','4530','EPSG','4200','EPSG','16319',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1746','projected_crs','EPSG','2729','EPSG','2683','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2730','Pulkovo 1995 / 3-degree Gauss-Kruger CM 114E',NULL,'EPSG','4530','EPSG','4200','EPSG','16398',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1747','projected_crs','EPSG','2730','EPSG','2684','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2731','Pulkovo 1995 / 3-degree Gauss-Kruger CM 117E',NULL,'EPSG','4530','EPSG','4200','EPSG','16320',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1748','projected_crs','EPSG','2731','EPSG','2685','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2732','Pulkovo 1995 / 3-degree Gauss-Kruger CM 120E',NULL,'EPSG','4530','EPSG','4200','EPSG','16170',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1749','projected_crs','EPSG','2732','EPSG','2686','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2733','Pulkovo 1995 / 3-degree Gauss-Kruger CM 123E',NULL,'EPSG','4530','EPSG','4200','EPSG','16321',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1750','projected_crs','EPSG','2733','EPSG','2687','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2734','Pulkovo 1995 / 3-degree Gauss-Kruger CM 126E',NULL,'EPSG','4530','EPSG','4200','EPSG','16172',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1751','projected_crs','EPSG','2734','EPSG','2688','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2735','Pulkovo 1995 / 3-degree Gauss-Kruger CM 129E',NULL,'EPSG','4530','EPSG','4200','EPSG','16322',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1752','projected_crs','EPSG','2735','EPSG','2689','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2736','Tete / UTM zone 36S',NULL,'EPSG','4400','EPSG','4127','EPSG','16136',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1753','projected_crs','EPSG','2736','EPSG','1540','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2737','Tete / UTM zone 37S',NULL,'EPSG','4400','EPSG','4127','EPSG','16137',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1754','projected_crs','EPSG','2737','EPSG','1541','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2738','Pulkovo 1995 / 3-degree Gauss-Kruger CM 132E',NULL,'EPSG','4530','EPSG','4200','EPSG','16174',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1755','projected_crs','EPSG','2738','EPSG','2690','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2739','Pulkovo 1995 / 3-degree Gauss-Kruger CM 135E',NULL,'EPSG','4530','EPSG','4200','EPSG','16323',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1756','projected_crs','EPSG','2739','EPSG','2691','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2740','Pulkovo 1995 / 3-degree Gauss-Kruger CM 138E',NULL,'EPSG','4530','EPSG','4200','EPSG','16176',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1757','projected_crs','EPSG','2740','EPSG','2692','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2741','Pulkovo 1995 / 3-degree Gauss-Kruger CM 141E',NULL,'EPSG','4530','EPSG','4200','EPSG','16324',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1758','projected_crs','EPSG','2741','EPSG','2693','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2742','Pulkovo 1995 / 3-degree Gauss-Kruger CM 144E',NULL,'EPSG','4530','EPSG','4200','EPSG','16178',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1759','projected_crs','EPSG','2742','EPSG','2694','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2743','Pulkovo 1995 / 3-degree Gauss-Kruger CM 147E',NULL,'EPSG','4530','EPSG','4200','EPSG','16325',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1760','projected_crs','EPSG','2743','EPSG','2695','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2744','Pulkovo 1995 / 3-degree Gauss-Kruger CM 150E',NULL,'EPSG','4530','EPSG','4200','EPSG','16180',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1761','projected_crs','EPSG','2744','EPSG','2696','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2745','Pulkovo 1995 / 3-degree Gauss-Kruger CM 153E',NULL,'EPSG','4530','EPSG','4200','EPSG','16326',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1762','projected_crs','EPSG','2745','EPSG','2697','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2746','Pulkovo 1995 / 3-degree Gauss-Kruger CM 156E',NULL,'EPSG','4530','EPSG','4200','EPSG','16182',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1763','projected_crs','EPSG','2746','EPSG','2698','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2747','Pulkovo 1995 / 3-degree Gauss-Kruger CM 159E',NULL,'EPSG','4530','EPSG','4200','EPSG','16327',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1764','projected_crs','EPSG','2747','EPSG','2699','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2748','Pulkovo 1995 / 3-degree Gauss-Kruger CM 162E',NULL,'EPSG','4530','EPSG','4200','EPSG','16184',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1765','projected_crs','EPSG','2748','EPSG','2700','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2749','Pulkovo 1995 / 3-degree Gauss-Kruger CM 165E',NULL,'EPSG','4530','EPSG','4200','EPSG','16328',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1766','projected_crs','EPSG','2749','EPSG','2701','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2750','Pulkovo 1995 / 3-degree Gauss-Kruger CM 168E',NULL,'EPSG','4530','EPSG','4200','EPSG','16186',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1767','projected_crs','EPSG','2750','EPSG','2702','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2751','Pulkovo 1995 / 3-degree Gauss-Kruger CM 171E',NULL,'EPSG','4530','EPSG','4200','EPSG','16329',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1768','projected_crs','EPSG','2751','EPSG','2703','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2752','Pulkovo 1995 / 3-degree Gauss-Kruger CM 174E',NULL,'EPSG','4530','EPSG','4200','EPSG','16188',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1769','projected_crs','EPSG','2752','EPSG','2704','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2753','Pulkovo 1995 / 3-degree Gauss-Kruger CM 177E',NULL,'EPSG','4530','EPSG','4200','EPSG','16330',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1770','projected_crs','EPSG','2753','EPSG','2705','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2754','Pulkovo 1995 / 3-degree Gauss-Kruger CM 180E',NULL,'EPSG','4530','EPSG','4200','EPSG','16190',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1771','projected_crs','EPSG','2754','EPSG','2706','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2755','Pulkovo 1995 / 3-degree Gauss-Kruger CM 177W',NULL,'EPSG','4530','EPSG','4200','EPSG','16331',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1772','projected_crs','EPSG','2755','EPSG','2707','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2756','Pulkovo 1995 / 3-degree Gauss-Kruger CM 174W',NULL,'EPSG','4530','EPSG','4200','EPSG','16192',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1773','projected_crs','EPSG','2756','EPSG','2708','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2757','Pulkovo 1995 / 3-degree Gauss-Kruger CM 171W',NULL,'EPSG','4530','EPSG','4200','EPSG','16332',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1774','projected_crs','EPSG','2757','EPSG','2709','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2758','Pulkovo 1995 / 3-degree Gauss-Kruger CM 168W',NULL,'EPSG','4530','EPSG','4200','EPSG','16194',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1775','projected_crs','EPSG','2758','EPSG','2710','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2759','NAD83(HARN) / Alabama East',NULL,'EPSG','4499','EPSG','4152','EPSG','10131',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1776','projected_crs','EPSG','2759','EPSG','2154','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2760','NAD83(HARN) / Alabama West',NULL,'EPSG','4499','EPSG','4152','EPSG','10132',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1777','projected_crs','EPSG','2760','EPSG','2155','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2761','NAD83(HARN) / Arizona East',NULL,'EPSG','4499','EPSG','4152','EPSG','10231',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1778','projected_crs','EPSG','2761','EPSG','2167','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2762','NAD83(HARN) / Arizona Central',NULL,'EPSG','4499','EPSG','4152','EPSG','10232',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1779','projected_crs','EPSG','2762','EPSG','2166','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2763','NAD83(HARN) / Arizona West',NULL,'EPSG','4499','EPSG','4152','EPSG','10233',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1780','projected_crs','EPSG','2763','EPSG','2168','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2764','NAD83(HARN) / Arkansas North',NULL,'EPSG','4499','EPSG','4152','EPSG','10331',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1781','projected_crs','EPSG','2764','EPSG','2169','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2765','NAD83(HARN) / Arkansas South',NULL,'EPSG','4499','EPSG','4152','EPSG','10332',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1782','projected_crs','EPSG','2765','EPSG','2170','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2766','NAD83(HARN) / California zone 1',NULL,'EPSG','4499','EPSG','4152','EPSG','10431',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1783','projected_crs','EPSG','2766','EPSG','2175','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2767','NAD83(HARN) / California zone 2',NULL,'EPSG','4499','EPSG','4152','EPSG','10432',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1784','projected_crs','EPSG','2767','EPSG','2176','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2768','NAD83(HARN) / California zone 3',NULL,'EPSG','4499','EPSG','4152','EPSG','10433',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1785','projected_crs','EPSG','2768','EPSG','2177','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2769','NAD83(HARN) / California zone 4',NULL,'EPSG','4499','EPSG','4152','EPSG','10434',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1786','projected_crs','EPSG','2769','EPSG','2178','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2770','NAD83(HARN) / California zone 5',NULL,'EPSG','4499','EPSG','4152','EPSG','10435',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1787','projected_crs','EPSG','2770','EPSG','2182','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2771','NAD83(HARN) / California zone 6',NULL,'EPSG','4499','EPSG','4152','EPSG','10436',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1788','projected_crs','EPSG','2771','EPSG','2180','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2772','NAD83(HARN) / Colorado North',NULL,'EPSG','4499','EPSG','4152','EPSG','10531',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1789','projected_crs','EPSG','2772','EPSG','2184','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2773','NAD83(HARN) / Colorado Central',NULL,'EPSG','4499','EPSG','4152','EPSG','10532',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1790','projected_crs','EPSG','2773','EPSG','2183','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2774','NAD83(HARN) / Colorado South',NULL,'EPSG','4499','EPSG','4152','EPSG','10533',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1791','projected_crs','EPSG','2774','EPSG','2185','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2775','NAD83(HARN) / Connecticut',NULL,'EPSG','4499','EPSG','4152','EPSG','10630',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1792','projected_crs','EPSG','2775','EPSG','1377','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2776','NAD83(HARN) / Delaware',NULL,'EPSG','4499','EPSG','4152','EPSG','10730',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1793','projected_crs','EPSG','2776','EPSG','1378','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2777','NAD83(HARN) / Florida East',NULL,'EPSG','4499','EPSG','4152','EPSG','10931',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1794','projected_crs','EPSG','2777','EPSG','2186','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2778','NAD83(HARN) / Florida West',NULL,'EPSG','4499','EPSG','4152','EPSG','10932',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1795','projected_crs','EPSG','2778','EPSG','2188','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2779','NAD83(HARN) / Florida North',NULL,'EPSG','4499','EPSG','4152','EPSG','10933',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1796','projected_crs','EPSG','2779','EPSG','2187','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2780','NAD83(HARN) / Georgia East',NULL,'EPSG','4499','EPSG','4152','EPSG','11031',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1797','projected_crs','EPSG','2780','EPSG','2189','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2781','NAD83(HARN) / Georgia West',NULL,'EPSG','4499','EPSG','4152','EPSG','11032',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1798','projected_crs','EPSG','2781','EPSG','2190','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2782','NAD83(HARN) / Hawaii zone 1',NULL,'EPSG','4499','EPSG','4152','EPSG','15131',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1799','projected_crs','EPSG','2782','EPSG','1546','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2783','NAD83(HARN) / Hawaii zone 2',NULL,'EPSG','4499','EPSG','4152','EPSG','15132',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1800','projected_crs','EPSG','2783','EPSG','1547','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2784','NAD83(HARN) / Hawaii zone 3',NULL,'EPSG','4499','EPSG','4152','EPSG','15133',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1801','projected_crs','EPSG','2784','EPSG','1548','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2785','NAD83(HARN) / Hawaii zone 4',NULL,'EPSG','4499','EPSG','4152','EPSG','15134',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1802','projected_crs','EPSG','2785','EPSG','1549','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2786','NAD83(HARN) / Hawaii zone 5',NULL,'EPSG','4499','EPSG','4152','EPSG','15135',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1803','projected_crs','EPSG','2786','EPSG','1550','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2787','NAD83(HARN) / Idaho East',NULL,'EPSG','4499','EPSG','4152','EPSG','11131',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1804','projected_crs','EPSG','2787','EPSG','2192','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2788','NAD83(HARN) / Idaho Central',NULL,'EPSG','4499','EPSG','4152','EPSG','11132',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1805','projected_crs','EPSG','2788','EPSG','2191','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2789','NAD83(HARN) / Idaho West',NULL,'EPSG','4499','EPSG','4152','EPSG','11133',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1806','projected_crs','EPSG','2789','EPSG','2193','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2790','NAD83(HARN) / Illinois East',NULL,'EPSG','4499','EPSG','4152','EPSG','11231',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1807','projected_crs','EPSG','2790','EPSG','2194','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2791','NAD83(HARN) / Illinois West',NULL,'EPSG','4499','EPSG','4152','EPSG','11232',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1808','projected_crs','EPSG','2791','EPSG','2195','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2792','NAD83(HARN) / Indiana East',NULL,'EPSG','4499','EPSG','4152','EPSG','11331',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1809','projected_crs','EPSG','2792','EPSG','2196','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2793','NAD83(HARN) / Indiana West',NULL,'EPSG','4499','EPSG','4152','EPSG','11332',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1810','projected_crs','EPSG','2793','EPSG','2197','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2794','NAD83(HARN) / Iowa North',NULL,'EPSG','4499','EPSG','4152','EPSG','11431',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1811','projected_crs','EPSG','2794','EPSG','2198','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2795','NAD83(HARN) / Iowa South',NULL,'EPSG','4499','EPSG','4152','EPSG','11432',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1812','projected_crs','EPSG','2795','EPSG','2199','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2796','NAD83(HARN) / Kansas North',NULL,'EPSG','4499','EPSG','4152','EPSG','11531',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1813','projected_crs','EPSG','2796','EPSG','2200','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2797','NAD83(HARN) / Kansas South',NULL,'EPSG','4499','EPSG','4152','EPSG','11532',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1814','projected_crs','EPSG','2797','EPSG','2201','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2798','NAD83(HARN) / Kentucky North',NULL,'EPSG','4499','EPSG','4152','EPSG','15303',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1815','projected_crs','EPSG','2798','EPSG','2202','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2799','NAD83(HARN) / Kentucky South',NULL,'EPSG','4499','EPSG','4152','EPSG','11632',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1816','projected_crs','EPSG','2799','EPSG','2203','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2800','NAD83(HARN) / Louisiana North',NULL,'EPSG','4499','EPSG','4152','EPSG','11731',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1817','projected_crs','EPSG','2800','EPSG','2204','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2801','NAD83(HARN) / Louisiana South',NULL,'EPSG','4499','EPSG','4152','EPSG','11732',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1818','projected_crs','EPSG','2801','EPSG','2529','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2802','NAD83(HARN) / Maine East',NULL,'EPSG','4499','EPSG','4152','EPSG','11831',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1819','projected_crs','EPSG','2802','EPSG','2206','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2803','NAD83(HARN) / Maine West',NULL,'EPSG','4499','EPSG','4152','EPSG','11832',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1820','projected_crs','EPSG','2803','EPSG','2207','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2804','NAD83(HARN) / Maryland',NULL,'EPSG','4499','EPSG','4152','EPSG','11930',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1821','projected_crs','EPSG','2804','EPSG','1389','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2805','NAD83(HARN) / Massachusetts Mainland',NULL,'EPSG','4499','EPSG','4152','EPSG','12031',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1822','projected_crs','EPSG','2805','EPSG','2209','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2806','NAD83(HARN) / Massachusetts Island',NULL,'EPSG','4499','EPSG','4152','EPSG','12032',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1823','projected_crs','EPSG','2806','EPSG','2208','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2807','NAD83(HARN) / Michigan North',NULL,'EPSG','4499','EPSG','4152','EPSG','12141',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1824','projected_crs','EPSG','2807','EPSG','1723','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2808','NAD83(HARN) / Michigan Central',NULL,'EPSG','4499','EPSG','4152','EPSG','12142',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1825','projected_crs','EPSG','2808','EPSG','1724','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2809','NAD83(HARN) / Michigan South',NULL,'EPSG','4499','EPSG','4152','EPSG','12143',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1826','projected_crs','EPSG','2809','EPSG','1725','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2810','NAD83(HARN) / Minnesota North',NULL,'EPSG','4499','EPSG','4152','EPSG','12231',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1827','projected_crs','EPSG','2810','EPSG','2214','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2811','NAD83(HARN) / Minnesota Central',NULL,'EPSG','4499','EPSG','4152','EPSG','12232',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1828','projected_crs','EPSG','2811','EPSG','2213','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2812','NAD83(HARN) / Minnesota South',NULL,'EPSG','4499','EPSG','4152','EPSG','12233',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1829','projected_crs','EPSG','2812','EPSG','2215','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2813','NAD83(HARN) / Mississippi East',NULL,'EPSG','4499','EPSG','4152','EPSG','12331',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1830','projected_crs','EPSG','2813','EPSG','2216','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2814','NAD83(HARN) / Mississippi West',NULL,'EPSG','4499','EPSG','4152','EPSG','12332',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1831','projected_crs','EPSG','2814','EPSG','2217','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2815','NAD83(HARN) / Missouri East',NULL,'EPSG','4499','EPSG','4152','EPSG','12431',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1832','projected_crs','EPSG','2815','EPSG','2219','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2816','NAD83(HARN) / Missouri Central',NULL,'EPSG','4499','EPSG','4152','EPSG','12432',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1833','projected_crs','EPSG','2816','EPSG','2218','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2817','NAD83(HARN) / Missouri West',NULL,'EPSG','4499','EPSG','4152','EPSG','12433',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1834','projected_crs','EPSG','2817','EPSG','2220','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2818','NAD83(HARN) / Montana',NULL,'EPSG','4499','EPSG','4152','EPSG','12530',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1835','projected_crs','EPSG','2818','EPSG','1395','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2819','NAD83(HARN) / Nebraska',NULL,'EPSG','4499','EPSG','4152','EPSG','12630',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1836','projected_crs','EPSG','2819','EPSG','1396','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2820','NAD83(HARN) / Nevada East',NULL,'EPSG','4499','EPSG','4152','EPSG','12731',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1837','projected_crs','EPSG','2820','EPSG','2224','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2821','NAD83(HARN) / Nevada Central',NULL,'EPSG','4499','EPSG','4152','EPSG','12732',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1838','projected_crs','EPSG','2821','EPSG','2223','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2822','NAD83(HARN) / Nevada West',NULL,'EPSG','4499','EPSG','4152','EPSG','12733',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1839','projected_crs','EPSG','2822','EPSG','2225','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2823','NAD83(HARN) / New Hampshire',NULL,'EPSG','4499','EPSG','4152','EPSG','12830',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1840','projected_crs','EPSG','2823','EPSG','1398','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2824','NAD83(HARN) / New Jersey',NULL,'EPSG','4499','EPSG','4152','EPSG','12930',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1841','projected_crs','EPSG','2824','EPSG','1399','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2825','NAD83(HARN) / New Mexico East',NULL,'EPSG','4499','EPSG','4152','EPSG','13031',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1842','projected_crs','EPSG','2825','EPSG','2228','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2826','NAD83(HARN) / New Mexico Central',NULL,'EPSG','4499','EPSG','4152','EPSG','13032',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1843','projected_crs','EPSG','2826','EPSG','2231','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2827','NAD83(HARN) / New Mexico West',NULL,'EPSG','4499','EPSG','4152','EPSG','13033',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1844','projected_crs','EPSG','2827','EPSG','2232','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2828','NAD83(HARN) / New York East',NULL,'EPSG','4499','EPSG','4152','EPSG','13131',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1845','projected_crs','EPSG','2828','EPSG','2234','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2829','NAD83(HARN) / New York Central',NULL,'EPSG','4499','EPSG','4152','EPSG','13132',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1846','projected_crs','EPSG','2829','EPSG','2233','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2830','NAD83(HARN) / New York West',NULL,'EPSG','4499','EPSG','4152','EPSG','13133',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1847','projected_crs','EPSG','2830','EPSG','2236','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2831','NAD83(HARN) / New York Long Island',NULL,'EPSG','4499','EPSG','4152','EPSG','13134',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1848','projected_crs','EPSG','2831','EPSG','2235','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2832','NAD83(HARN) / North Dakota North',NULL,'EPSG','4499','EPSG','4152','EPSG','13331',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1849','projected_crs','EPSG','2832','EPSG','2237','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2833','NAD83(HARN) / North Dakota South',NULL,'EPSG','4499','EPSG','4152','EPSG','13332',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1850','projected_crs','EPSG','2833','EPSG','2238','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2834','NAD83(HARN) / Ohio North',NULL,'EPSG','4499','EPSG','4152','EPSG','13431',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1851','projected_crs','EPSG','2834','EPSG','2239','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2835','NAD83(HARN) / Ohio South',NULL,'EPSG','4499','EPSG','4152','EPSG','13432',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1852','projected_crs','EPSG','2835','EPSG','2240','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2836','NAD83(HARN) / Oklahoma North',NULL,'EPSG','4499','EPSG','4152','EPSG','13531',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1853','projected_crs','EPSG','2836','EPSG','2241','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2837','NAD83(HARN) / Oklahoma South',NULL,'EPSG','4499','EPSG','4152','EPSG','13532',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1854','projected_crs','EPSG','2837','EPSG','2242','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2838','NAD83(HARN) / Oregon North',NULL,'EPSG','4499','EPSG','4152','EPSG','13631',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1855','projected_crs','EPSG','2838','EPSG','2243','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2839','NAD83(HARN) / Oregon South',NULL,'EPSG','4499','EPSG','4152','EPSG','13632',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1856','projected_crs','EPSG','2839','EPSG','2244','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2840','NAD83(HARN) / Rhode Island',NULL,'EPSG','4499','EPSG','4152','EPSG','13830',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1857','projected_crs','EPSG','2840','EPSG','1408','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2841','NAD83(HARN) / South Dakota North',NULL,'EPSG','4499','EPSG','4152','EPSG','14031',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1858','projected_crs','EPSG','2841','EPSG','2249','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2842','NAD83(HARN) / South Dakota South',NULL,'EPSG','4499','EPSG','4152','EPSG','14032',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1859','projected_crs','EPSG','2842','EPSG','2250','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2843','NAD83(HARN) / Tennessee',NULL,'EPSG','4499','EPSG','4152','EPSG','14130',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1860','projected_crs','EPSG','2843','EPSG','1411','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2844','NAD83(HARN) / Texas North',NULL,'EPSG','4499','EPSG','4152','EPSG','14231',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1861','projected_crs','EPSG','2844','EPSG','2253','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2845','NAD83(HARN) / Texas North Central',NULL,'EPSG','4499','EPSG','4152','EPSG','14232',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1862','projected_crs','EPSG','2845','EPSG','2254','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2846','NAD83(HARN) / Texas Central',NULL,'EPSG','4499','EPSG','4152','EPSG','14233',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1863','projected_crs','EPSG','2846','EPSG','2252','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2847','NAD83(HARN) / Texas South Central',NULL,'EPSG','4499','EPSG','4152','EPSG','14234',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1864','projected_crs','EPSG','2847','EPSG','2527','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2848','NAD83(HARN) / Texas South',NULL,'EPSG','4499','EPSG','4152','EPSG','14235',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1865','projected_crs','EPSG','2848','EPSG','2528','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2849','NAD83(HARN) / Utah North',NULL,'EPSG','4499','EPSG','4152','EPSG','14331',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1866','projected_crs','EPSG','2849','EPSG','2258','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2850','NAD83(HARN) / Utah Central',NULL,'EPSG','4499','EPSG','4152','EPSG','14332',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1867','projected_crs','EPSG','2850','EPSG','2257','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2851','NAD83(HARN) / Utah South',NULL,'EPSG','4499','EPSG','4152','EPSG','14333',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1868','projected_crs','EPSG','2851','EPSG','2259','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2852','NAD83(HARN) / Vermont',NULL,'EPSG','4499','EPSG','4152','EPSG','14430',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1869','projected_crs','EPSG','2852','EPSG','1414','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2853','NAD83(HARN) / Virginia North',NULL,'EPSG','4499','EPSG','4152','EPSG','14531',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1870','projected_crs','EPSG','2853','EPSG','2260','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2854','NAD83(HARN) / Virginia South',NULL,'EPSG','4499','EPSG','4152','EPSG','14532',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1871','projected_crs','EPSG','2854','EPSG','2261','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2855','NAD83(HARN) / Washington North',NULL,'EPSG','4499','EPSG','4152','EPSG','14631',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1872','projected_crs','EPSG','2855','EPSG','2273','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2856','NAD83(HARN) / Washington South',NULL,'EPSG','4499','EPSG','4152','EPSG','14632',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1873','projected_crs','EPSG','2856','EPSG','2274','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2857','NAD83(HARN) / West Virginia North',NULL,'EPSG','4499','EPSG','4152','EPSG','14731',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1874','projected_crs','EPSG','2857','EPSG','2264','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2858','NAD83(HARN) / West Virginia South',NULL,'EPSG','4499','EPSG','4152','EPSG','14732',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1875','projected_crs','EPSG','2858','EPSG','2265','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2859','NAD83(HARN) / Wisconsin North',NULL,'EPSG','4499','EPSG','4152','EPSG','14831',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1876','projected_crs','EPSG','2859','EPSG','2267','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2860','NAD83(HARN) / Wisconsin Central',NULL,'EPSG','4499','EPSG','4152','EPSG','14832',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1877','projected_crs','EPSG','2860','EPSG','2266','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2861','NAD83(HARN) / Wisconsin South',NULL,'EPSG','4499','EPSG','4152','EPSG','14833',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1878','projected_crs','EPSG','2861','EPSG','2268','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2862','NAD83(HARN) / Wyoming East',NULL,'EPSG','4499','EPSG','4152','EPSG','14931',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1879','projected_crs','EPSG','2862','EPSG','2269','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2863','NAD83(HARN) / Wyoming East Central',NULL,'EPSG','4499','EPSG','4152','EPSG','14932',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1880','projected_crs','EPSG','2863','EPSG','2270','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2864','NAD83(HARN) / Wyoming West Central',NULL,'EPSG','4499','EPSG','4152','EPSG','14933',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1881','projected_crs','EPSG','2864','EPSG','2272','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2865','NAD83(HARN) / Wyoming West',NULL,'EPSG','4499','EPSG','4152','EPSG','14934',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1882','projected_crs','EPSG','2865','EPSG','2271','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2866','NAD83(HARN) / Puerto Rico and Virgin Is.',NULL,'EPSG','4499','EPSG','4152','EPSG','15230',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1883','projected_crs','EPSG','2866','EPSG','3634','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2867','NAD83(HARN) / Arizona East (ft)',NULL,'EPSG','4495','EPSG','4152','EPSG','15304',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1884','projected_crs','EPSG','2867','EPSG','2167','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2868','NAD83(HARN) / Arizona Central (ft)',NULL,'EPSG','4495','EPSG','4152','EPSG','15305',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1885','projected_crs','EPSG','2868','EPSG','2166','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2869','NAD83(HARN) / Arizona West (ft)',NULL,'EPSG','4495','EPSG','4152','EPSG','15306',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1886','projected_crs','EPSG','2869','EPSG','2168','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2870','NAD83(HARN) / California zone 1 (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15307',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1887','projected_crs','EPSG','2870','EPSG','2175','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2871','NAD83(HARN) / California zone 2 (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15308',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1888','projected_crs','EPSG','2871','EPSG','2176','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2872','NAD83(HARN) / California zone 3 (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15309',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1889','projected_crs','EPSG','2872','EPSG','2177','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2873','NAD83(HARN) / California zone 4 (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15310',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1890','projected_crs','EPSG','2873','EPSG','2178','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2874','NAD83(HARN) / California zone 5 (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15311',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1891','projected_crs','EPSG','2874','EPSG','2182','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2875','NAD83(HARN) / California zone 6 (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15312',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1892','projected_crs','EPSG','2875','EPSG','2180','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2876','NAD83(HARN) / Colorado North (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15313',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1893','projected_crs','EPSG','2876','EPSG','2184','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2877','NAD83(HARN) / Colorado Central (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15314',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1894','projected_crs','EPSG','2877','EPSG','2183','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2878','NAD83(HARN) / Colorado South (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15315',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1895','projected_crs','EPSG','2878','EPSG','2185','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2879','NAD83(HARN) / Connecticut (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15316',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1896','projected_crs','EPSG','2879','EPSG','1377','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2880','NAD83(HARN) / Delaware (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15317',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1897','projected_crs','EPSG','2880','EPSG','1378','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2881','NAD83(HARN) / Florida East (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15318',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1898','projected_crs','EPSG','2881','EPSG','2186','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2882','NAD83(HARN) / Florida West (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15319',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1899','projected_crs','EPSG','2882','EPSG','2188','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2883','NAD83(HARN) / Florida North (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15320',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1900','projected_crs','EPSG','2883','EPSG','2187','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2884','NAD83(HARN) / Georgia East (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15321',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1901','projected_crs','EPSG','2884','EPSG','2189','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2885','NAD83(HARN) / Georgia West (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15322',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1902','projected_crs','EPSG','2885','EPSG','2190','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2886','NAD83(HARN) / Idaho East (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15323',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1903','projected_crs','EPSG','2886','EPSG','2192','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2887','NAD83(HARN) / Idaho Central (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15324',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1904','projected_crs','EPSG','2887','EPSG','2191','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2888','NAD83(HARN) / Idaho West (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15325',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1905','projected_crs','EPSG','2888','EPSG','2193','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2889','NAD83(HARN) / Indiana East (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15326',NULL,1);
INSERT INTO "usage" VALUES('EPSG','1906','projected_crs','EPSG','2889','EPSG','2196','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2890','NAD83(HARN) / Indiana West (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15327',NULL,1);
INSERT INTO "usage" VALUES('EPSG','1907','projected_crs','EPSG','2890','EPSG','2197','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2891','NAD83(HARN) / Kentucky North (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15328',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1908','projected_crs','EPSG','2891','EPSG','2202','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2892','NAD83(HARN) / Kentucky South (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15329',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1909','projected_crs','EPSG','2892','EPSG','2203','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2893','NAD83(HARN) / Maryland (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15330',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1910','projected_crs','EPSG','2893','EPSG','1389','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2894','NAD83(HARN) / Massachusetts Mainland (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15331',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1911','projected_crs','EPSG','2894','EPSG','2209','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2895','NAD83(HARN) / Massachusetts Island (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15332',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1912','projected_crs','EPSG','2895','EPSG','2208','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2896','NAD83(HARN) / Michigan North (ft)',NULL,'EPSG','4495','EPSG','4152','EPSG','15333',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1913','projected_crs','EPSG','2896','EPSG','1723','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2897','NAD83(HARN) / Michigan Central (ft)',NULL,'EPSG','4495','EPSG','4152','EPSG','15334',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1914','projected_crs','EPSG','2897','EPSG','1724','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2898','NAD83(HARN) / Michigan South (ft)',NULL,'EPSG','4495','EPSG','4152','EPSG','15335',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1915','projected_crs','EPSG','2898','EPSG','1725','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2899','NAD83(HARN) / Mississippi East (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15336',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1916','projected_crs','EPSG','2899','EPSG','2216','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2900','NAD83(HARN) / Mississippi West (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15337',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1917','projected_crs','EPSG','2900','EPSG','2217','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2901','NAD83(HARN) / Montana (ft)',NULL,'EPSG','4495','EPSG','4152','EPSG','15338',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1918','projected_crs','EPSG','2901','EPSG','1395','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2902','NAD83(HARN) / New Mexico East (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15339',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1919','projected_crs','EPSG','2902','EPSG','2228','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2903','NAD83(HARN) / New Mexico Central (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15340',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1920','projected_crs','EPSG','2903','EPSG','2231','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2904','NAD83(HARN) / New Mexico West (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15341',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1921','projected_crs','EPSG','2904','EPSG','2232','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2905','NAD83(HARN) / New York East (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15342',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1922','projected_crs','EPSG','2905','EPSG','2234','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2906','NAD83(HARN) / New York Central (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15343',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1923','projected_crs','EPSG','2906','EPSG','2233','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2907','NAD83(HARN) / New York West (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15344',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1924','projected_crs','EPSG','2907','EPSG','2236','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2908','NAD83(HARN) / New York Long Island (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15345',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1925','projected_crs','EPSG','2908','EPSG','2235','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2909','NAD83(HARN) / North Dakota North (ft)',NULL,'EPSG','4495','EPSG','4152','EPSG','15347',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1926','projected_crs','EPSG','2909','EPSG','2237','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2910','NAD83(HARN) / North Dakota South (ft)',NULL,'EPSG','4495','EPSG','4152','EPSG','15348',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1927','projected_crs','EPSG','2910','EPSG','2238','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2911','NAD83(HARN) / Oklahoma North (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15349',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1928','projected_crs','EPSG','2911','EPSG','2241','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2912','NAD83(HARN) / Oklahoma South (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15350',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1929','projected_crs','EPSG','2912','EPSG','2242','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2913','NAD83(HARN) / Oregon North (ft)',NULL,'EPSG','4495','EPSG','4152','EPSG','15351',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1930','projected_crs','EPSG','2913','EPSG','2243','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2914','NAD83(HARN) / Oregon South (ft)',NULL,'EPSG','4495','EPSG','4152','EPSG','15352',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1931','projected_crs','EPSG','2914','EPSG','2244','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2915','NAD83(HARN) / Tennessee (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15356',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1932','projected_crs','EPSG','2915','EPSG','1411','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2916','NAD83(HARN) / Texas North (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15357',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1933','projected_crs','EPSG','2916','EPSG','2253','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2917','NAD83(HARN) / Texas North Central (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15358',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1934','projected_crs','EPSG','2917','EPSG','2254','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2918','NAD83(HARN) / Texas Central (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15359',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1935','projected_crs','EPSG','2918','EPSG','2252','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2919','NAD83(HARN) / Texas South Central (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15360',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1936','projected_crs','EPSG','2919','EPSG','2527','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2920','NAD83(HARN) / Texas South (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15361',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1937','projected_crs','EPSG','2920','EPSG','2528','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2921','NAD83(HARN) / Utah North (ft)',NULL,'EPSG','4495','EPSG','4152','EPSG','15362',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1938','projected_crs','EPSG','2921','EPSG','2258','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2922','NAD83(HARN) / Utah Central (ft)',NULL,'EPSG','4495','EPSG','4152','EPSG','15363',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1939','projected_crs','EPSG','2922','EPSG','2257','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2923','NAD83(HARN) / Utah South (ft)',NULL,'EPSG','4495','EPSG','4152','EPSG','15364',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1940','projected_crs','EPSG','2923','EPSG','2259','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2924','NAD83(HARN) / Virginia North (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15365',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1941','projected_crs','EPSG','2924','EPSG','2260','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2925','NAD83(HARN) / Virginia South (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15366',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1942','projected_crs','EPSG','2925','EPSG','2261','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2926','NAD83(HARN) / Washington North (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15367',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1943','projected_crs','EPSG','2926','EPSG','2273','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2927','NAD83(HARN) / Washington South (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15368',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1944','projected_crs','EPSG','2927','EPSG','2274','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2928','NAD83(HARN) / Wisconsin North (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15369',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1945','projected_crs','EPSG','2928','EPSG','2267','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2929','NAD83(HARN) / Wisconsin Central (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15370',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1946','projected_crs','EPSG','2929','EPSG','2266','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2930','NAD83(HARN) / Wisconsin South (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15371',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1947','projected_crs','EPSG','2930','EPSG','2268','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2931','Beduaram / TM 13 NE',NULL,'EPSG','4499','EPSG','4213','EPSG','16413',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1948','projected_crs','EPSG','2931','EPSG','2771','EPSG','1136');
INSERT INTO "projected_crs" VALUES('EPSG','2932','QND95 / Qatar National Grid',NULL,'EPSG','4400','EPSG','4614','EPSG','19919',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1949','projected_crs','EPSG','2932','EPSG','1346','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2933','Segara / UTM zone 50S',NULL,'EPSG','4400','EPSG','4613','EPSG','16150',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1950','projected_crs','EPSG','2933','EPSG','1328','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2934','Segara (Jakarta) / NEIEZ',NULL,'EPSG','4499','EPSG','4820','EPSG','19905',NULL,1);
INSERT INTO "usage" VALUES('EPSG','1951','projected_crs','EPSG','2934','EPSG','1360','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2935','Pulkovo 1942 / CS63 zone A1',NULL,'EPSG','4530','EPSG','4284','EPSG','18441',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1952','projected_crs','EPSG','2935','EPSG','2772','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2936','Pulkovo 1942 / CS63 zone A2',NULL,'EPSG','4530','EPSG','4284','EPSG','18442',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1953','projected_crs','EPSG','2936','EPSG','2773','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2937','Pulkovo 1942 / CS63 zone A3',NULL,'EPSG','4530','EPSG','4284','EPSG','18443',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1954','projected_crs','EPSG','2937','EPSG','2774','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2938','Pulkovo 1942 / CS63 zone A4',NULL,'EPSG','4530','EPSG','4284','EPSG','18444',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1955','projected_crs','EPSG','2938','EPSG','2775','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2939','Pulkovo 1942 / CS63 zone K2',NULL,'EPSG','4530','EPSG','4284','EPSG','18446',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1956','projected_crs','EPSG','2939','EPSG','2776','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2940','Pulkovo 1942 / CS63 zone K3',NULL,'EPSG','4530','EPSG','4284','EPSG','18447',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1957','projected_crs','EPSG','2940','EPSG','2777','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2941','Pulkovo 1942 / CS63 zone K4',NULL,'EPSG','4530','EPSG','4284','EPSG','18448',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1958','projected_crs','EPSG','2941','EPSG','2778','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','2942','Porto Santo / UTM zone 28N',NULL,'EPSG','4400','EPSG','4615','EPSG','16028',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1959','projected_crs','EPSG','2942','EPSG','1314','EPSG','1153');
INSERT INTO "projected_crs" VALUES('EPSG','2943','Selvagem Grande / UTM zone 28N',NULL,'EPSG','4400','EPSG','4616','EPSG','16028',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1960','projected_crs','EPSG','2943','EPSG','2779','EPSG','1153');
INSERT INTO "projected_crs" VALUES('EPSG','2944','NAD83(CSRS) / SCoPQ zone 2',NULL,'EPSG','4499','EPSG','4617','EPSG','17700',NULL,1);
INSERT INTO "usage" VALUES('EPSG','1961','projected_crs','EPSG','2944','EPSG','1420','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2945','NAD83(CSRS) / MTM zone 3',NULL,'EPSG','4496','EPSG','4617','EPSG','17703',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1962','projected_crs','EPSG','2945','EPSG','2290','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2946','NAD83(CSRS) / MTM zone 4',NULL,'EPSG','4496','EPSG','4617','EPSG','17704',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1963','projected_crs','EPSG','2946','EPSG','2276','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2947','NAD83(CSRS) / MTM zone 5',NULL,'EPSG','4496','EPSG','4617','EPSG','17705',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1964','projected_crs','EPSG','2947','EPSG','2277','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2948','NAD83(CSRS) / MTM zone 6',NULL,'EPSG','4496','EPSG','4617','EPSG','17706',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1965','projected_crs','EPSG','2948','EPSG','2278','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2949','NAD83(CSRS) / MTM zone 7',NULL,'EPSG','4496','EPSG','4617','EPSG','17707',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1966','projected_crs','EPSG','2949','EPSG','1425','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2950','NAD83(CSRS) / MTM zone 8',NULL,'EPSG','4496','EPSG','4617','EPSG','17708',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1967','projected_crs','EPSG','2950','EPSG','2279','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2951','NAD83(CSRS) / MTM zone 9',NULL,'EPSG','4496','EPSG','4617','EPSG','17709',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1968','projected_crs','EPSG','2951','EPSG','2280','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2952','NAD83(CSRS) / MTM zone 10',NULL,'EPSG','4496','EPSG','4617','EPSG','17710',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1969','projected_crs','EPSG','2952','EPSG','2281','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2953','NAD83(CSRS) / New Brunswick Stereographic',NULL,'EPSG','4500','EPSG','4617','EPSG','19946',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1970','projected_crs','EPSG','2953','EPSG','1447','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2954','NAD83(CSRS) / Prince Edward Isl. Stereographic (NAD83)',NULL,'EPSG','4496','EPSG','4617','EPSG','19960',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1971','projected_crs','EPSG','2954','EPSG','1533','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2955','NAD83(CSRS) / UTM zone 11N',NULL,'EPSG','4400','EPSG','4617','EPSG','16011',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1972','projected_crs','EPSG','2955','EPSG','3528','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2956','NAD83(CSRS) / UTM zone 12N',NULL,'EPSG','4400','EPSG','4617','EPSG','16012',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1973','projected_crs','EPSG','2956','EPSG','3527','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2957','NAD83(CSRS) / UTM zone 13N',NULL,'EPSG','4400','EPSG','4617','EPSG','16013',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1974','projected_crs','EPSG','2957','EPSG','3526','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2958','NAD83(CSRS) / UTM zone 17N',NULL,'EPSG','4400','EPSG','4617','EPSG','16017',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1975','projected_crs','EPSG','2958','EPSG','3416','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2959','NAD83(CSRS) / UTM zone 18N',NULL,'EPSG','4400','EPSG','4617','EPSG','16018',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1976','projected_crs','EPSG','2959','EPSG','3417','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2960','NAD83(CSRS) / UTM zone 19N',NULL,'EPSG','4400','EPSG','4617','EPSG','16019',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1977','projected_crs','EPSG','2960','EPSG','3524','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2961','NAD83(CSRS) / UTM zone 20N',NULL,'EPSG','4400','EPSG','4617','EPSG','16020',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1978','projected_crs','EPSG','2961','EPSG','3525','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2962','NAD83(CSRS) / UTM zone 21N',NULL,'EPSG','4400','EPSG','4617','EPSG','16021',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1979','projected_crs','EPSG','2962','EPSG','2151','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2963','Lisbon 1890 (Lisbon) / Portugal Bonne',NULL,'EPSG','6509','EPSG','4904','EPSG','19979',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1980','projected_crs','EPSG','2963','EPSG','1294','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','2964','NAD27 / Alaska Albers',NULL,'EPSG','4497','EPSG','4267','EPSG','15020',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1981','projected_crs','EPSG','2964','EPSG','1330','EPSG','1241');
INSERT INTO "projected_crs" VALUES('EPSG','2965','NAD83 / Indiana East (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15372',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1982','projected_crs','EPSG','2965','EPSG','2196','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2966','NAD83 / Indiana West (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15373',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1983','projected_crs','EPSG','2966','EPSG','2197','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2967','NAD83(HARN) / Indiana East (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15372',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1984','projected_crs','EPSG','2967','EPSG','2196','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2968','NAD83(HARN) / Indiana West (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15373',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1985','projected_crs','EPSG','2968','EPSG','2197','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2969','Fort Marigot / UTM zone 20N',NULL,'EPSG','4400','EPSG','4621','EPSG','16020',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1986','projected_crs','EPSG','2969','EPSG','2828','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2970','Guadeloupe 1948 / UTM zone 20N',NULL,'EPSG','4400','EPSG','4622','EPSG','16020',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1987','projected_crs','EPSG','2970','EPSG','2829','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2971','CSG67 / UTM zone 22N',NULL,'EPSG','4400','EPSG','4623','EPSG','16022',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1988','projected_crs','EPSG','2971','EPSG','3766','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2972','RGFG95 / UTM zone 22N',NULL,'EPSG','4400','EPSG','4624','EPSG','16022',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1989','projected_crs','EPSG','2972','EPSG','3144','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2973','Martinique 1938 / UTM zone 20N',NULL,'EPSG','4400','EPSG','4625','EPSG','16020',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1990','projected_crs','EPSG','2973','EPSG','3276','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2975','RGR92 / UTM zone 40S',NULL,'EPSG','4400','EPSG','4627','EPSG','16140',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1991','projected_crs','EPSG','2975','EPSG','3911','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2976','Tahiti 52 / UTM zone 6S',NULL,'EPSG','4400','EPSG','4628','EPSG','16106',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1992','projected_crs','EPSG','2976','EPSG','2811','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2977','Tahaa 54 / UTM zone 5S',NULL,'EPSG','4400','EPSG','4629','EPSG','16105',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1993','projected_crs','EPSG','2977','EPSG','2812','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2978','IGN72 Nuku Hiva / UTM zone 7S',NULL,'EPSG','4400','EPSG','4630','EPSG','16107',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1994','projected_crs','EPSG','2978','EPSG','3129','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2979','K0 1949 / UTM zone 42S',NULL,'EPSG','4400','EPSG','4631','EPSG','16142',NULL,1);
INSERT INTO "usage" VALUES('EPSG','1995','projected_crs','EPSG','2979','EPSG','2816','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2980','Combani 1950 / UTM zone 38S',NULL,'EPSG','4400','EPSG','4632','EPSG','16138',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1996','projected_crs','EPSG','2980','EPSG','3340','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2981','IGN56 Lifou / UTM zone 58S',NULL,'EPSG','4400','EPSG','4633','EPSG','16158',NULL,0);
INSERT INTO "usage" VALUES('EPSG','1997','projected_crs','EPSG','2981','EPSG','2814','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2982','IGN72 Grand Terre / UTM zone 58S',NULL,'EPSG','4400','EPSG','4634','EPSG','16158',NULL,1);
INSERT INTO "usage" VALUES('EPSG','1998','projected_crs','EPSG','2982','EPSG','2822','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2983','ST87 Ouvea / UTM zone 58S',NULL,'EPSG','4400','EPSG','4635','EPSG','16158',NULL,1);
INSERT INTO "usage" VALUES('EPSG','1999','projected_crs','EPSG','2983','EPSG','2813','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2984','RGNC 1991 / Lambert New Caledonia',NULL,'EPSG','4499','EPSG','4645','EPSG','19981',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2000','projected_crs','EPSG','2984','EPSG','1174','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2985','Petrels 1972 / Terre Adelie Polar Stereographic',NULL,'EPSG','1025','EPSG','4636','EPSG','19983',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2001','projected_crs','EPSG','2985','EPSG','2817','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2986','Perroud 1950 / Terre Adelie Polar Stereographic',NULL,'EPSG','1025','EPSG','4637','EPSG','19983',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2002','projected_crs','EPSG','2986','EPSG','2818','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2987','Saint Pierre et Miquelon 1950 / UTM zone 21N',NULL,'EPSG','4400','EPSG','4638','EPSG','16021',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2003','projected_crs','EPSG','2987','EPSG','3299','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2988','MOP78 / UTM zone 1S',NULL,'EPSG','4400','EPSG','4639','EPSG','16101',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2004','projected_crs','EPSG','2988','EPSG','2815','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2989','RRAF 1991 / UTM zone 20N',NULL,'EPSG','4400','EPSG','4640','EPSG','16020',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2005','projected_crs','EPSG','2989','EPSG','2824','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2990','Reunion 1947 / TM Reunion',NULL,'EPSG','4499','EPSG','4626','EPSG','19982',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2006','projected_crs','EPSG','2990','EPSG','3337','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2991','NAD83 / Oregon LCC (m)',NULL,'EPSG','4499','EPSG','4269','EPSG','13633',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2007','projected_crs','EPSG','2991','EPSG','1406','EPSG','1212');
INSERT INTO "projected_crs" VALUES('EPSG','2992','NAD83 / Oregon GIC Lambert (ft)',NULL,'EPSG','4495','EPSG','4269','EPSG','15374',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2008','projected_crs','EPSG','2992','EPSG','1406','EPSG','1135');
INSERT INTO "projected_crs" VALUES('EPSG','2993','NAD83(HARN) / Oregon LCC (m)',NULL,'EPSG','4499','EPSG','4152','EPSG','13633',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2009','projected_crs','EPSG','2993','EPSG','1406','EPSG','1212');
INSERT INTO "projected_crs" VALUES('EPSG','2994','NAD83(HARN) / Oregon GIC Lambert (ft)',NULL,'EPSG','4495','EPSG','4152','EPSG','15374',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2010','projected_crs','EPSG','2994','EPSG','1406','EPSG','1135');
INSERT INTO "projected_crs" VALUES('EPSG','2995','IGN53 Mare / UTM zone 58S',NULL,'EPSG','4400','EPSG','4641','EPSG','16158',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2011','projected_crs','EPSG','2995','EPSG','3434','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2996','ST84 Ile des Pins / UTM zone 58S',NULL,'EPSG','4400','EPSG','4642','EPSG','16158',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2012','projected_crs','EPSG','2996','EPSG','2820','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2997','ST71 Belep / UTM zone 58S',NULL,'EPSG','4400','EPSG','4643','EPSG','16158',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2013','projected_crs','EPSG','2997','EPSG','2821','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2998','NEA74 Noumea / UTM zone 58S',NULL,'EPSG','4400','EPSG','4644','EPSG','16158',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2014','projected_crs','EPSG','2998','EPSG','2823','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','2999','Grand Comoros / UTM zone 38S',NULL,'EPSG','4400','EPSG','4646','EPSG','16138',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2015','projected_crs','EPSG','2999','EPSG','2807','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3000','Segara / NEIEZ',NULL,'EPSG','4499','EPSG','4613','EPSG','19905',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2016','projected_crs','EPSG','3000','EPSG','1360','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3001','Batavia / NEIEZ',NULL,'EPSG','4499','EPSG','4211','EPSG','19905',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2017','projected_crs','EPSG','3001','EPSG','1285','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3002','Makassar / NEIEZ',NULL,'EPSG','4499','EPSG','4257','EPSG','19905',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2018','projected_crs','EPSG','3002','EPSG','1316','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3003','Monte Mario / Italy zone 1',NULL,'EPSG','4499','EPSG','4265','EPSG','18121',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2019','projected_crs','EPSG','3003','EPSG','1718','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3004','Monte Mario / Italy zone 2',NULL,'EPSG','4499','EPSG','4265','EPSG','18122',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2020','projected_crs','EPSG','3004','EPSG','1719','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3005','NAD83 / BC Albers',NULL,'EPSG','4400','EPSG','4269','EPSG','19984',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2021','projected_crs','EPSG','3005','EPSG','2832','EPSG','1220');
INSERT INTO "projected_crs" VALUES('EPSG','3006','SWEREF99 TM',NULL,'EPSG','4500','EPSG','4619','EPSG','17333',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2022','projected_crs','EPSG','3006','EPSG','1225','EPSG','1210');
INSERT INTO "projected_crs" VALUES('EPSG','3007','SWEREF99 12 00',NULL,'EPSG','4500','EPSG','4619','EPSG','17321',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2023','projected_crs','EPSG','3007','EPSG','2833','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','3008','SWEREF99 13 30',NULL,'EPSG','4500','EPSG','4619','EPSG','17322',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2024','projected_crs','EPSG','3008','EPSG','2834','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','3009','SWEREF99 15 00',NULL,'EPSG','4500','EPSG','4619','EPSG','17323',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2025','projected_crs','EPSG','3009','EPSG','2835','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','3010','SWEREF99 16 30',NULL,'EPSG','4500','EPSG','4619','EPSG','17324',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2026','projected_crs','EPSG','3010','EPSG','2836','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','3011','SWEREF99 18 00',NULL,'EPSG','4500','EPSG','4619','EPSG','17325',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2027','projected_crs','EPSG','3011','EPSG','2837','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','3012','SWEREF99 14 15',NULL,'EPSG','4500','EPSG','4619','EPSG','17326',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2028','projected_crs','EPSG','3012','EPSG','2838','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','3013','SWEREF99 15 45',NULL,'EPSG','4500','EPSG','4619','EPSG','17327',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2029','projected_crs','EPSG','3013','EPSG','2839','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','3014','SWEREF99 17 15',NULL,'EPSG','4500','EPSG','4619','EPSG','17328',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2030','projected_crs','EPSG','3014','EPSG','2840','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','3015','SWEREF99 18 45',NULL,'EPSG','4500','EPSG','4619','EPSG','17329',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2031','projected_crs','EPSG','3015','EPSG','2841','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','3016','SWEREF99 20 15',NULL,'EPSG','4500','EPSG','4619','EPSG','17330',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2032','projected_crs','EPSG','3016','EPSG','2842','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','3017','SWEREF99 21 45',NULL,'EPSG','4500','EPSG','4619','EPSG','17331',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2033','projected_crs','EPSG','3017','EPSG','2843','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','3018','SWEREF99 23 15',NULL,'EPSG','4500','EPSG','4619','EPSG','17332',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2034','projected_crs','EPSG','3018','EPSG','2844','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','3019','RT90 7.5 gon V',NULL,'EPSG','4530','EPSG','4124','EPSG','17334',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2035','projected_crs','EPSG','3019','EPSG','2845','EPSG','1056');
INSERT INTO "projected_crs" VALUES('EPSG','3020','RT90 5 gon V',NULL,'EPSG','4530','EPSG','4124','EPSG','17335',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2036','projected_crs','EPSG','3020','EPSG','2846','EPSG','1056');
INSERT INTO "projected_crs" VALUES('EPSG','3021','RT90 2.5 gon V',NULL,'EPSG','4530','EPSG','4124','EPSG','19929',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2037','projected_crs','EPSG','3021','EPSG','2847','EPSG','1055');
INSERT INTO "usage" VALUES('EPSG','2038','projected_crs','EPSG','3021','EPSG','3313','EPSG','1210');
INSERT INTO "projected_crs" VALUES('EPSG','3022','RT90 0 gon',NULL,'EPSG','4530','EPSG','4124','EPSG','17336',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2039','projected_crs','EPSG','3022','EPSG','2848','EPSG','1056');
INSERT INTO "projected_crs" VALUES('EPSG','3023','RT90 2.5 gon O',NULL,'EPSG','4530','EPSG','4124','EPSG','17337',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2040','projected_crs','EPSG','3023','EPSG','2849','EPSG','1056');
INSERT INTO "projected_crs" VALUES('EPSG','3024','RT90 5 gon O',NULL,'EPSG','4530','EPSG','4124','EPSG','17338',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2041','projected_crs','EPSG','3024','EPSG','2850','EPSG','1056');
INSERT INTO "projected_crs" VALUES('EPSG','3025','RT38 7.5 gon V',NULL,'EPSG','4530','EPSG','4308','EPSG','17334',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2042','projected_crs','EPSG','3025','EPSG','2845','EPSG','1056');
INSERT INTO "projected_crs" VALUES('EPSG','3026','RT38 5 gon V',NULL,'EPSG','4530','EPSG','4308','EPSG','17335',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2043','projected_crs','EPSG','3026','EPSG','2846','EPSG','1056');
INSERT INTO "projected_crs" VALUES('EPSG','3027','RT38 2.5 gon V',NULL,'EPSG','4530','EPSG','4308','EPSG','19929',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2044','projected_crs','EPSG','3027','EPSG','2847','EPSG','1055');
INSERT INTO "usage" VALUES('EPSG','2045','projected_crs','EPSG','3027','EPSG','3313','EPSG','1210');
INSERT INTO "projected_crs" VALUES('EPSG','3028','RT38 0 gon',NULL,'EPSG','4530','EPSG','4308','EPSG','17336',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2046','projected_crs','EPSG','3028','EPSG','2848','EPSG','1056');
INSERT INTO "projected_crs" VALUES('EPSG','3029','RT38 2.5 gon O',NULL,'EPSG','4530','EPSG','4308','EPSG','17337',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2047','projected_crs','EPSG','3029','EPSG','2849','EPSG','1056');
INSERT INTO "projected_crs" VALUES('EPSG','3030','RT38 5 gon O',NULL,'EPSG','4530','EPSG','4308','EPSG','17338',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2048','projected_crs','EPSG','3030','EPSG','2850','EPSG','1056');
INSERT INTO "projected_crs" VALUES('EPSG','3031','WGS 84 / Antarctic Polar Stereographic',NULL,'EPSG','4490','EPSG','4326','EPSG','19992',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2049','projected_crs','EPSG','3031','EPSG','1031','EPSG','1254');
INSERT INTO "projected_crs" VALUES('EPSG','3032','WGS 84 / Australian Antarctic Polar Stereographic',NULL,'EPSG','4489','EPSG','4326','EPSG','19993',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2050','projected_crs','EPSG','3032','EPSG','1278','EPSG','1210');
INSERT INTO "projected_crs" VALUES('EPSG','3033','WGS 84 / Australian Antarctic Lambert',NULL,'EPSG','4400','EPSG','4326','EPSG','19994',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2051','projected_crs','EPSG','3033','EPSG','2880','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3034','ETRS89-extended / LCC Europe',NULL,'EPSG','4500','EPSG','4258','EPSG','19985',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2052','projected_crs','EPSG','3034','EPSG','2881','EPSG','1107');
INSERT INTO "projected_crs" VALUES('EPSG','3035','ETRS89-extended / LAEA Europe',NULL,'EPSG','4532','EPSG','4258','EPSG','19986',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2053','projected_crs','EPSG','3035','EPSG','2881','EPSG','1162');
INSERT INTO "projected_crs" VALUES('EPSG','3036','Moznet / UTM zone 36S',NULL,'EPSG','4400','EPSG','4130','EPSG','16136',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2054','projected_crs','EPSG','3036','EPSG','3929','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3037','Moznet / UTM zone 37S',NULL,'EPSG','4400','EPSG','4130','EPSG','16137',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2055','projected_crs','EPSG','3037','EPSG','3931','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3038','ETRS89 / TM26',NULL,'EPSG','4500','EPSG','4258','EPSG','16026',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2056','projected_crs','EPSG','3038','EPSG','2855','EPSG','1227');
INSERT INTO "projected_crs" VALUES('EPSG','3039','ETRS89 / TM27',NULL,'EPSG','4500','EPSG','4258','EPSG','16027',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2057','projected_crs','EPSG','3039','EPSG','2856','EPSG','1227');
INSERT INTO "projected_crs" VALUES('EPSG','3040','ETRS89 / UTM zone 28N (N-E)',NULL,'EPSG','4500','EPSG','4258','EPSG','16028',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2058','projected_crs','EPSG','3040','EPSG','2122','EPSG','1227');
INSERT INTO "projected_crs" VALUES('EPSG','3041','ETRS89 / UTM zone 29N (N-E)',NULL,'EPSG','4500','EPSG','4258','EPSG','16029',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2059','projected_crs','EPSG','3041','EPSG','4766','EPSG','1227');
INSERT INTO "projected_crs" VALUES('EPSG','3042','ETRS89 / UTM zone 30N (N-E)',NULL,'EPSG','4500','EPSG','4258','EPSG','16030',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2060','projected_crs','EPSG','3042','EPSG','4767','EPSG','1227');
INSERT INTO "projected_crs" VALUES('EPSG','3043','ETRS89 / UTM zone 31N (N-E)',NULL,'EPSG','4500','EPSG','4258','EPSG','16031',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2061','projected_crs','EPSG','3043','EPSG','4768','EPSG','1227');
INSERT INTO "projected_crs" VALUES('EPSG','3044','ETRS89 / UTM zone 32N (N-E)',NULL,'EPSG','4500','EPSG','4258','EPSG','16032',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2062','projected_crs','EPSG','3044','EPSG','4769','EPSG','1227');
INSERT INTO "projected_crs" VALUES('EPSG','3045','ETRS89 / UTM zone 33N (N-E)',NULL,'EPSG','4500','EPSG','4258','EPSG','16033',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2063','projected_crs','EPSG','3045','EPSG','4770','EPSG','1227');
INSERT INTO "projected_crs" VALUES('EPSG','3046','ETRS89 / UTM zone 34N (N-E)',NULL,'EPSG','4500','EPSG','4258','EPSG','16034',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2064','projected_crs','EPSG','3046','EPSG','4771','EPSG','1227');
INSERT INTO "projected_crs" VALUES('EPSG','3047','ETRS89 / UTM zone 35N (N-E)',NULL,'EPSG','4500','EPSG','4258','EPSG','16035',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2065','projected_crs','EPSG','3047','EPSG','4772','EPSG','1227');
INSERT INTO "projected_crs" VALUES('EPSG','3048','ETRS89 / UTM zone 36N (N-E)',NULL,'EPSG','4500','EPSG','4258','EPSG','16036',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2066','projected_crs','EPSG','3048','EPSG','4773','EPSG','1227');
INSERT INTO "projected_crs" VALUES('EPSG','3049','ETRS89 / UTM zone 37N (N-E)',NULL,'EPSG','4500','EPSG','4258','EPSG','16037',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2067','projected_crs','EPSG','3049','EPSG','2131','EPSG','1227');
INSERT INTO "projected_crs" VALUES('EPSG','3050','ETRS89 / TM38',NULL,'EPSG','4500','EPSG','4258','EPSG','16038',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2068','projected_crs','EPSG','3050','EPSG','2867','EPSG','1227');
INSERT INTO "projected_crs" VALUES('EPSG','3051','ETRS89 / TM39',NULL,'EPSG','4500','EPSG','4258','EPSG','16039',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2069','projected_crs','EPSG','3051','EPSG','2868','EPSG','1227');
INSERT INTO "projected_crs" VALUES('EPSG','3052','Reykjavik 1900 / Lambert 1900',NULL,'EPSG','4491','EPSG','4657','EPSG','19987',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2070','projected_crs','EPSG','3052','EPSG','3262','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3053','Hjorsey 1955 / Lambert 1955',NULL,'EPSG','4491','EPSG','4658','EPSG','19988',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2071','projected_crs','EPSG','3053','EPSG','3262','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3054','Hjorsey 1955 / UTM zone 26N',NULL,'EPSG','4400','EPSG','4658','EPSG','16026',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2072','projected_crs','EPSG','3054','EPSG','2851','EPSG','1063');
INSERT INTO "projected_crs" VALUES('EPSG','3055','Hjorsey 1955 / UTM zone 27N',NULL,'EPSG','4400','EPSG','4658','EPSG','16027',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2073','projected_crs','EPSG','3055','EPSG','2852','EPSG','1063');
INSERT INTO "projected_crs" VALUES('EPSG','3056','Hjorsey 1955 / UTM zone 28N',NULL,'EPSG','4400','EPSG','4658','EPSG','16028',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2074','projected_crs','EPSG','3056','EPSG','2853','EPSG','1063');
INSERT INTO "projected_crs" VALUES('EPSG','3057','ISN93 / Lambert 1993',NULL,'EPSG','4499','EPSG','4659','EPSG','19989',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2075','projected_crs','EPSG','3057','EPSG','1120','EPSG','1210');
INSERT INTO "projected_crs" VALUES('EPSG','3058','Helle 1954 / Jan Mayen Grid',NULL,'EPSG','4531','EPSG','4660','EPSG','19991',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2076','projected_crs','EPSG','3058','EPSG','2869','EPSG','1153');
INSERT INTO "projected_crs" VALUES('EPSG','3059','LKS-92 / Latvia TM',NULL,'EPSG','4530','EPSG','4661','EPSG','19990',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2077','projected_crs','EPSG','3059','EPSG','1139','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3060','IGN72 Grande Terre / UTM zone 58S',NULL,'EPSG','4400','EPSG','4662','EPSG','16158',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2078','projected_crs','EPSG','3060','EPSG','2822','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3061','Porto Santo 1995 / UTM zone 28N',NULL,'EPSG','4400','EPSG','4663','EPSG','16028',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2079','projected_crs','EPSG','3061','EPSG','1314','EPSG','1153');
INSERT INTO "projected_crs" VALUES('EPSG','3062','Azores Oriental 1995 / UTM zone 26N',NULL,'EPSG','4400','EPSG','4664','EPSG','16026',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2080','projected_crs','EPSG','3062','EPSG','1345','EPSG','1153');
INSERT INTO "projected_crs" VALUES('EPSG','3063','Azores Central 1995 / UTM zone 26N',NULL,'EPSG','4400','EPSG','4665','EPSG','16026',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2081','projected_crs','EPSG','3063','EPSG','1301','EPSG','1153');
INSERT INTO "projected_crs" VALUES('EPSG','3064','IGM95 / UTM zone 32N',NULL,'EPSG','4400','EPSG','4670','EPSG','16032',NULL,0);
INSERT INTO "usage" VALUES('EPSG','14405','projected_crs','EPSG','3064','EPSG','1718','EPSG','1027');
INSERT INTO "projected_crs" VALUES('EPSG','3065','IGM95 / UTM zone 33N',NULL,'EPSG','4400','EPSG','4670','EPSG','16033',NULL,0);
INSERT INTO "usage" VALUES('EPSG','14406','projected_crs','EPSG','3065','EPSG','4186','EPSG','1027');
INSERT INTO "projected_crs" VALUES('EPSG','3066','ED50 / Jordan TM',NULL,'EPSG','4400','EPSG','4230','EPSG','19995',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2084','projected_crs','EPSG','3066','EPSG','1130','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3067','EUREF-FIN / TM35FIN(E,N)',NULL,'EPSG','4400','EPSG','10690','EPSG','16065',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2085','projected_crs','EPSG','3067','EPSG','1095','EPSG','1142');
INSERT INTO "usage" VALUES('EPSG','21990','projected_crs','EPSG','3067','EPSG','4782','EPSG','1227');
INSERT INTO "projected_crs" VALUES('EPSG','3068','DHDN / Soldner Berlin',NULL,'EPSG','4531','EPSG','4314','EPSG','19996',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2086','projected_crs','EPSG','3068','EPSG','2898','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','3069','NAD27 / Wisconsin Transverse Mercator',NULL,'EPSG','4499','EPSG','4267','EPSG','14811',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2087','projected_crs','EPSG','3069','EPSG','1418','EPSG','1135');
INSERT INTO "projected_crs" VALUES('EPSG','3070','NAD83 / Wisconsin Transverse Mercator',NULL,'EPSG','4499','EPSG','4269','EPSG','14841',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2088','projected_crs','EPSG','3070','EPSG','1418','EPSG','1135');
INSERT INTO "projected_crs" VALUES('EPSG','3071','NAD83(HARN) / Wisconsin Transverse Mercator',NULL,'EPSG','4499','EPSG','4152','EPSG','14841',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2089','projected_crs','EPSG','3071','EPSG','1418','EPSG','1135');
INSERT INTO "projected_crs" VALUES('EPSG','3072','NAD83 / Maine CS2000 East',NULL,'EPSG','4499','EPSG','4269','EPSG','11851',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2090','projected_crs','EPSG','3072','EPSG','2960','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3073','NAD83 / Maine CS2000 Central',NULL,'EPSG','4499','EPSG','4269','EPSG','11852',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2091','projected_crs','EPSG','3073','EPSG','2959','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3074','NAD83 / Maine CS2000 West',NULL,'EPSG','4499','EPSG','4269','EPSG','11853',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2092','projected_crs','EPSG','3074','EPSG','2958','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3075','NAD83(HARN) / Maine CS2000 East',NULL,'EPSG','4499','EPSG','4152','EPSG','11851',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2093','projected_crs','EPSG','3075','EPSG','2960','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3076','NAD83(HARN) / Maine CS2000 Central',NULL,'EPSG','4499','EPSG','4152','EPSG','11852',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2094','projected_crs','EPSG','3076','EPSG','2959','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3077','NAD83(HARN) / Maine CS2000 West',NULL,'EPSG','4499','EPSG','4152','EPSG','11853',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2095','projected_crs','EPSG','3077','EPSG','2958','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3078','NAD83 / Michigan Oblique Mercator',NULL,'EPSG','4499','EPSG','4269','EPSG','12150',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2096','projected_crs','EPSG','3078','EPSG','1391','EPSG','1135');
INSERT INTO "projected_crs" VALUES('EPSG','3079','NAD83(HARN) / Michigan Oblique Mercator',NULL,'EPSG','4499','EPSG','4152','EPSG','12150',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2097','projected_crs','EPSG','3079','EPSG','1391','EPSG','1135');
INSERT INTO "projected_crs" VALUES('EPSG','3080','NAD27 / Shackleford',NULL,'EPSG','4495','EPSG','4267','EPSG','14252',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2098','projected_crs','EPSG','3080','EPSG','1412','EPSG','1135');
INSERT INTO "projected_crs" VALUES('EPSG','3081','NAD83 / Texas State Mapping System',NULL,'EPSG','4499','EPSG','4269','EPSG','14251',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2099','projected_crs','EPSG','3081','EPSG','1412','EPSG','1135');
INSERT INTO "projected_crs" VALUES('EPSG','3082','NAD83 / Texas Centric Lambert Conformal',NULL,'EPSG','4499','EPSG','4269','EPSG','14253',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2100','projected_crs','EPSG','3082','EPSG','1412','EPSG','1221');
INSERT INTO "projected_crs" VALUES('EPSG','3083','NAD83 / Texas Centric Albers Equal Area',NULL,'EPSG','4499','EPSG','4269','EPSG','14254',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2101','projected_crs','EPSG','3083','EPSG','1412','EPSG','1222');
INSERT INTO "projected_crs" VALUES('EPSG','3084','NAD83(HARN) / Texas Centric Lambert Conformal',NULL,'EPSG','4499','EPSG','4152','EPSG','14253',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2102','projected_crs','EPSG','3084','EPSG','1412','EPSG','1221');
INSERT INTO "projected_crs" VALUES('EPSG','3085','NAD83(HARN) / Texas Centric Albers Equal Area',NULL,'EPSG','4499','EPSG','4152','EPSG','14254',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2103','projected_crs','EPSG','3085','EPSG','1412','EPSG','1222');
INSERT INTO "projected_crs" VALUES('EPSG','3086','NAD83 / Florida GDL Albers',NULL,'EPSG','4499','EPSG','4269','EPSG','10934',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2104','projected_crs','EPSG','3086','EPSG','1379','EPSG','1135');
INSERT INTO "projected_crs" VALUES('EPSG','3087','NAD83(HARN) / Florida GDL Albers',NULL,'EPSG','4499','EPSG','4152','EPSG','10934',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2105','projected_crs','EPSG','3087','EPSG','1379','EPSG','1135');
INSERT INTO "projected_crs" VALUES('EPSG','3088','NAD83 / Kentucky Single Zone',NULL,'EPSG','4499','EPSG','4269','EPSG','11630',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2106','projected_crs','EPSG','3088','EPSG','1386','EPSG','1135');
INSERT INTO "projected_crs" VALUES('EPSG','3089','NAD83 / Kentucky Single Zone (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15375',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2107','projected_crs','EPSG','3089','EPSG','1386','EPSG','1135');
INSERT INTO "projected_crs" VALUES('EPSG','3090','NAD83(HARN) / Kentucky Single Zone',NULL,'EPSG','4499','EPSG','4152','EPSG','11630',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2108','projected_crs','EPSG','3090','EPSG','1386','EPSG','1135');
INSERT INTO "projected_crs" VALUES('EPSG','3091','NAD83(HARN) / Kentucky Single Zone (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15375',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2109','projected_crs','EPSG','3091','EPSG','1386','EPSG','1135');
INSERT INTO "projected_crs" VALUES('EPSG','3092','Tokyo / UTM zone 51N',NULL,'EPSG','4400','EPSG','4301','EPSG','16051',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2110','projected_crs','EPSG','3092','EPSG','2951','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3093','Tokyo / UTM zone 52N',NULL,'EPSG','4400','EPSG','4301','EPSG','16052',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2111','projected_crs','EPSG','3093','EPSG','2952','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3094','Tokyo / UTM zone 53N',NULL,'EPSG','4400','EPSG','4301','EPSG','16053',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2112','projected_crs','EPSG','3094','EPSG','2953','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3095','Tokyo / UTM zone 54N',NULL,'EPSG','4400','EPSG','4301','EPSG','16054',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2113','projected_crs','EPSG','3095','EPSG','2954','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3096','Tokyo / UTM zone 55N',NULL,'EPSG','4400','EPSG','4301','EPSG','16055',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2114','projected_crs','EPSG','3096','EPSG','2955','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3097','JGD2000 / UTM zone 51N',NULL,'EPSG','4400','EPSG','4612','EPSG','16051',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2115','projected_crs','EPSG','3097','EPSG','3959','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3098','JGD2000 / UTM zone 52N',NULL,'EPSG','4400','EPSG','4612','EPSG','16052',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2116','projected_crs','EPSG','3098','EPSG','3960','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3099','JGD2000 / UTM zone 53N',NULL,'EPSG','4400','EPSG','4612','EPSG','16053',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2117','projected_crs','EPSG','3099','EPSG','3961','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3100','JGD2000 / UTM zone 54N',NULL,'EPSG','4400','EPSG','4612','EPSG','16054',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2118','projected_crs','EPSG','3100','EPSG','3962','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3101','JGD2000 / UTM zone 55N',NULL,'EPSG','4400','EPSG','4612','EPSG','16055',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2119','projected_crs','EPSG','3101','EPSG','3963','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3102','American Samoa 1962 / American Samoa Lambert',NULL,'EPSG','4497','EPSG','4169','EPSG','15376',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2120','projected_crs','EPSG','3102','EPSG','3109','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3103','Mauritania 1999 / UTM zone 28N',NULL,'EPSG','4400','EPSG','4681','EPSG','16028',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2121','projected_crs','EPSG','3103','EPSG','2971','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3104','Mauritania 1999 / UTM zone 29N',NULL,'EPSG','4400','EPSG','4681','EPSG','16029',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2122','projected_crs','EPSG','3104','EPSG','2970','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3105','Mauritania 1999 / UTM zone 30N',NULL,'EPSG','4400','EPSG','4681','EPSG','16030',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2123','projected_crs','EPSG','3105','EPSG','2969','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3106','Gulshan 303 / TM 90 NE',NULL,'EPSG','4400','EPSG','4682','EPSG','16490',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2124','projected_crs','EPSG','3106','EPSG','3217','EPSG','1142');
INSERT INTO "usage" VALUES('EPSG','14846','projected_crs','EPSG','3106','EPSG','1041','EPSG','1136');
INSERT INTO "projected_crs" VALUES('EPSG','3107','GDA94 / SA Lambert',NULL,'EPSG','4400','EPSG','4283','EPSG','17359',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2125','projected_crs','EPSG','3107','EPSG','2986','EPSG','1135');
INSERT INTO "projected_crs" VALUES('EPSG','3108','ETRS89 / Guernsey Grid',NULL,'EPSG','4400','EPSG','4258','EPSG','19998',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2126','projected_crs','EPSG','3108','EPSG','2989','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3109','ETRS89 / Jersey Transverse Mercator',NULL,'EPSG','4400','EPSG','4258','EPSG','19999',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2127','projected_crs','EPSG','3109','EPSG','2988','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3110','AGD66 / Vicgrid66',NULL,'EPSG','4400','EPSG','4202','EPSG','17360',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2128','projected_crs','EPSG','3110','EPSG','2285','EPSG','1135');
INSERT INTO "projected_crs" VALUES('EPSG','3111','GDA94 / Vicgrid',NULL,'EPSG','4400','EPSG','4283','EPSG','17361',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2129','projected_crs','EPSG','3111','EPSG','2285','EPSG','1135');
INSERT INTO "projected_crs" VALUES('EPSG','3112','GDA94 / Geoscience Australia Lambert',NULL,'EPSG','4400','EPSG','4283','EPSG','17362',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2130','projected_crs','EPSG','3112','EPSG','2575','EPSG','1236');
INSERT INTO "projected_crs" VALUES('EPSG','3113','GDA94 / BCSG02',NULL,'EPSG','4400','EPSG','4283','EPSG','17363',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2131','projected_crs','EPSG','3113','EPSG','2990','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','3114','MAGNA-SIRGAS / Colombia Far West zone',NULL,'EPSG','4500','EPSG','4686','EPSG','18055',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2132','projected_crs','EPSG','3114','EPSG','3091','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3115','MAGNA-SIRGAS / Colombia West zone',NULL,'EPSG','4500','EPSG','4686','EPSG','18056',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2133','projected_crs','EPSG','3115','EPSG','3090','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3116','MAGNA-SIRGAS / Colombia Bogota zone',NULL,'EPSG','4500','EPSG','4686','EPSG','18057',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2134','projected_crs','EPSG','3116','EPSG','1599','EPSG','1208');
INSERT INTO "projected_crs" VALUES('EPSG','3117','MAGNA-SIRGAS / Colombia East Central zone',NULL,'EPSG','4500','EPSG','4686','EPSG','18058',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2136','projected_crs','EPSG','3117','EPSG','1600','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3118','MAGNA-SIRGAS / Colombia East zone',NULL,'EPSG','4500','EPSG','4686','EPSG','18059',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2137','projected_crs','EPSG','3118','EPSG','1601','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3119','Douala 1948 / AEF west',NULL,'EPSG','4400','EPSG','4192','EPSG','18415',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2138','projected_crs','EPSG','3119','EPSG','2555','EPSG','1210');
INSERT INTO "projected_crs" VALUES('EPSG','3120','Pulkovo 1942(58) / Poland zone I',NULL,'EPSG','4530','EPSG','4179','EPSG','18280',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2139','projected_crs','EPSG','3120','EPSG','1515','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3121','PRS92 / Philippines zone 1',NULL,'EPSG','4499','EPSG','4683','EPSG','18171',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2140','projected_crs','EPSG','3121','EPSG','1698','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3122','PRS92 / Philippines zone 2',NULL,'EPSG','4499','EPSG','4683','EPSG','18172',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2141','projected_crs','EPSG','3122','EPSG','1699','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3123','PRS92 / Philippines zone 3',NULL,'EPSG','4499','EPSG','4683','EPSG','18173',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2142','projected_crs','EPSG','3123','EPSG','1700','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3124','PRS92 / Philippines zone 4',NULL,'EPSG','4499','EPSG','4683','EPSG','18174',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2143','projected_crs','EPSG','3124','EPSG','1701','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3125','PRS92 / Philippines zone 5',NULL,'EPSG','4499','EPSG','4683','EPSG','18175',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2144','projected_crs','EPSG','3125','EPSG','1702','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3126','EUREF-FIN / ETRS-GK19FIN',NULL,'EPSG','4500','EPSG','10690','EPSG','18183',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2145','projected_crs','EPSG','3126','EPSG','3092','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','3127','EUREF-FIN / ETRS-GK20FIN',NULL,'EPSG','4500','EPSG','10690','EPSG','18184',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2146','projected_crs','EPSG','3127','EPSG','3093','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','3128','EUREF-FIN / ETRS-GK21FIN',NULL,'EPSG','4500','EPSG','10690','EPSG','18185',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2147','projected_crs','EPSG','3128','EPSG','3094','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','3129','EUREF-FIN / ETRS-GK22FIN',NULL,'EPSG','4500','EPSG','10690','EPSG','18186',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2148','projected_crs','EPSG','3129','EPSG','3095','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','3130','EUREF-FIN / ETRS-GK23FIN',NULL,'EPSG','4500','EPSG','10690','EPSG','18187',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2149','projected_crs','EPSG','3130','EPSG','3096','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','3131','EUREF-FIN / ETRS-GK24FIN',NULL,'EPSG','4500','EPSG','10690','EPSG','18188',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2150','projected_crs','EPSG','3131','EPSG','3097','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','3132','EUREF-FIN / ETRS-GK25FIN',NULL,'EPSG','4500','EPSG','10690','EPSG','18189',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2151','projected_crs','EPSG','3132','EPSG','3098','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','3133','EUREF-FIN / ETRS-GK26FIN',NULL,'EPSG','4500','EPSG','10690','EPSG','18190',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2152','projected_crs','EPSG','3133','EPSG','3099','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','3134','EUREF-FIN / ETRS-GK27FIN',NULL,'EPSG','4500','EPSG','10690','EPSG','18195',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2153','projected_crs','EPSG','3134','EPSG','3100','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','3135','EUREF-FIN / ETRS-GK28FIN',NULL,'EPSG','4500','EPSG','10690','EPSG','18196',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2154','projected_crs','EPSG','3135','EPSG','3101','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','3136','EUREF-FIN / ETRS-GK29FIN',NULL,'EPSG','4500','EPSG','10690','EPSG','18197',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2155','projected_crs','EPSG','3136','EPSG','3102','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','3137','EUREF-FIN / ETRS-GK30FIN',NULL,'EPSG','4500','EPSG','10690','EPSG','18198',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2156','projected_crs','EPSG','3137','EPSG','3103','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','3138','EUREF-FIN / ETRS-GK31FIN',NULL,'EPSG','4500','EPSG','10690','EPSG','18199',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2157','projected_crs','EPSG','3138','EPSG','3104','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','3139','Vanua Levu 1915 / Vanua Levu Grid',NULL,'EPSG','4533','EPSG','4748','EPSG','19878',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2158','projected_crs','EPSG','3139','EPSG','3401','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','3140','Viti Levu 1912 / Viti Levu Grid',NULL,'EPSG','4533','EPSG','4752','EPSG','19879',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2159','projected_crs','EPSG','3140','EPSG','3195','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','3141','Fiji 1956 / UTM zone 60S',NULL,'EPSG','4400','EPSG','4721','EPSG','16160',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2160','projected_crs','EPSG','3141','EPSG','3399','EPSG','1153');
INSERT INTO "projected_crs" VALUES('EPSG','3142','Fiji 1956 / UTM zone 1S',NULL,'EPSG','4400','EPSG','4721','EPSG','16101',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2161','projected_crs','EPSG','3142','EPSG','3400','EPSG','1153');
INSERT INTO "projected_crs" VALUES('EPSG','3143','Fiji 1986 / Fiji Map Grid',NULL,'EPSG','4400','EPSG','4720','EPSG','19880',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2162','projected_crs','EPSG','3143','EPSG','1094','EPSG','1056');
INSERT INTO "projected_crs" VALUES('EPSG','3144','FD54 / Faroe Lambert',NULL,'EPSG','1031','EPSG','4741','EPSG','19870',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2163','projected_crs','EPSG','3144','EPSG','3248','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','3145','ETRS89 / Faroe Lambert',NULL,'EPSG','1031','EPSG','4258','EPSG','19870',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2164','projected_crs','EPSG','3145','EPSG','3248','EPSG','1153');
INSERT INTO "projected_crs" VALUES('EPSG','3146','Pulkovo 1942 / 3-degree Gauss-Kruger zone 6',NULL,'EPSG','4530','EPSG','4284','EPSG','16266',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2165','projected_crs','EPSG','3146','EPSG','3403','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','3147','Pulkovo 1942 / 3-degree Gauss-Kruger CM 18E',NULL,'EPSG','4530','EPSG','4284','EPSG','16366',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2166','projected_crs','EPSG','3147','EPSG','3403','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','3148','Indian 1960 / UTM zone 48N',NULL,'EPSG','4400','EPSG','4131','EPSG','16048',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2167','projected_crs','EPSG','3148','EPSG','1542','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3149','Indian 1960 / UTM zone 49N',NULL,'EPSG','4400','EPSG','4131','EPSG','16049',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2168','projected_crs','EPSG','3149','EPSG','1453','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3150','Pulkovo 1995 / 3-degree Gauss-Kruger zone 6',NULL,'EPSG','4530','EPSG','4200','EPSG','16266',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2169','projected_crs','EPSG','3150','EPSG','3403','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','3151','Pulkovo 1995 / 3-degree Gauss-Kruger CM 18E',NULL,'EPSG','4530','EPSG','4200','EPSG','16366',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2170','projected_crs','EPSG','3151','EPSG','3403','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','3152','ST74',NULL,'EPSG','4531','EPSG','4619','EPSG','19876',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2171','projected_crs','EPSG','3152','EPSG','3408','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','3153','NAD83(CSRS) / BC Albers',NULL,'EPSG','4400','EPSG','4617','EPSG','19984',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2172','projected_crs','EPSG','3153','EPSG','2832','EPSG','1220');
INSERT INTO "projected_crs" VALUES('EPSG','3154','NAD83(CSRS) / UTM zone 7N',NULL,'EPSG','4400','EPSG','4617','EPSG','16007',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2173','projected_crs','EPSG','3154','EPSG','3409','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3155','NAD83(CSRS) / UTM zone 8N',NULL,'EPSG','4400','EPSG','4617','EPSG','16008',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2174','projected_crs','EPSG','3155','EPSG','3410','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3156','NAD83(CSRS) / UTM zone 9N',NULL,'EPSG','4400','EPSG','4617','EPSG','16009',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2175','projected_crs','EPSG','3156','EPSG','3411','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3157','NAD83(CSRS) / UTM zone 10N',NULL,'EPSG','4400','EPSG','4617','EPSG','16010',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2176','projected_crs','EPSG','3157','EPSG','3412','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3158','NAD83(CSRS) / UTM zone 14N',NULL,'EPSG','4400','EPSG','4617','EPSG','16014',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2177','projected_crs','EPSG','3158','EPSG','3413','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3159','NAD83(CSRS) / UTM zone 15N',NULL,'EPSG','4400','EPSG','4617','EPSG','16015',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2178','projected_crs','EPSG','3159','EPSG','3414','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3160','NAD83(CSRS) / UTM zone 16N',NULL,'EPSG','4400','EPSG','4617','EPSG','16016',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2179','projected_crs','EPSG','3160','EPSG','3415','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3161','NAD83 / Ontario MNR Lambert',NULL,'EPSG','4400','EPSG','4269','EPSG','19875',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2180','projected_crs','EPSG','3161','EPSG','1367','EPSG','1220');
INSERT INTO "projected_crs" VALUES('EPSG','3162','NAD83(CSRS) / Ontario MNR Lambert',NULL,'EPSG','4400','EPSG','4617','EPSG','19875',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2181','projected_crs','EPSG','3162','EPSG','1367','EPSG','1220');
INSERT INTO "projected_crs" VALUES('EPSG','3163','RGNC91-93 / Lambert New Caledonia',NULL,'EPSG','4499','EPSG','4749','EPSG','19981',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2182','projected_crs','EPSG','3163','EPSG','3430','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3164','ST87 Ouvea / UTM zone 58S',NULL,'EPSG','4400','EPSG','4750','EPSG','16158',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2183','projected_crs','EPSG','3164','EPSG','2813','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3165','NEA74 Noumea / Noumea Lambert',NULL,'EPSG','4499','EPSG','4644','EPSG','19873',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2184','projected_crs','EPSG','3165','EPSG','2823','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3166','NEA74 Noumea / Noumea Lambert 2',NULL,'EPSG','4499','EPSG','4644','EPSG','19874',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2185','projected_crs','EPSG','3166','EPSG','2823','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3167','Kertau (RSO) / RSO Malaya (ch)',NULL,'EPSG','4410','EPSG','4751','EPSG','19871',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2186','projected_crs','EPSG','3167','EPSG','1690','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3168','Kertau (RSO) / RSO Malaya (m)',NULL,'EPSG','4400','EPSG','4751','EPSG','19872',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2187','projected_crs','EPSG','3168','EPSG','1690','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3169','RGNC91-93 / UTM zone 57S',NULL,'EPSG','4400','EPSG','4749','EPSG','16157',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2188','projected_crs','EPSG','3169','EPSG','3431','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3170','RGNC91-93 / UTM zone 58S',NULL,'EPSG','4400','EPSG','4749','EPSG','16158',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2189','projected_crs','EPSG','3170','EPSG','3432','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3171','RGNC91-93 / UTM zone 59S',NULL,'EPSG','4400','EPSG','4749','EPSG','16159',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2190','projected_crs','EPSG','3171','EPSG','3433','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3172','IGN53 Mare / UTM zone 59S',NULL,'EPSG','4400','EPSG','4641','EPSG','16159',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2191','projected_crs','EPSG','3172','EPSG','3435','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3173','fk89 / Faroe Lambert FK89',NULL,'EPSG','1031','EPSG','4753','EPSG','19877',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2192','projected_crs','EPSG','3173','EPSG','3248','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','3174','NAD83 / Great Lakes Albers',NULL,'EPSG','4499','EPSG','4269','EPSG','15397',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2193','projected_crs','EPSG','3174','EPSG','3467','EPSG','1052');
INSERT INTO "projected_crs" VALUES('EPSG','3175','NAD83 / Great Lakes and St Lawrence Albers',NULL,'EPSG','4499','EPSG','4269','EPSG','15398',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2194','projected_crs','EPSG','3175','EPSG','3468','EPSG','1052');
INSERT INTO "projected_crs" VALUES('EPSG','3176','Indian 1960 / TM 106 NE',NULL,'EPSG','4400','EPSG','4131','EPSG','16506',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2195','projected_crs','EPSG','3176','EPSG','1495','EPSG','1136');
INSERT INTO "projected_crs" VALUES('EPSG','3177','LGD2006 / Libya TM',NULL,'EPSG','4499','EPSG','4754','EPSG','18319',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2196','projected_crs','EPSG','3177','EPSG','1143','EPSG','1241');
INSERT INTO "projected_crs" VALUES('EPSG','3178','GR96 / UTM zone 18N',NULL,'EPSG','4400','EPSG','4747','EPSG','16018',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2197','projected_crs','EPSG','3178','EPSG','3449','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3179','GR96 / UTM zone 19N',NULL,'EPSG','4400','EPSG','4747','EPSG','16019',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2198','projected_crs','EPSG','3179','EPSG','3450','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3180','GR96 / UTM zone 20N',NULL,'EPSG','4400','EPSG','4747','EPSG','16020',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2199','projected_crs','EPSG','3180','EPSG','3451','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3181','GR96 / UTM zone 21N',NULL,'EPSG','4400','EPSG','4747','EPSG','16021',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2200','projected_crs','EPSG','3181','EPSG','3452','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3182','GR96 / UTM zone 22N',NULL,'EPSG','4400','EPSG','4747','EPSG','16022',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2201','projected_crs','EPSG','3182','EPSG','3453','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3183','GR96 / UTM zone 23N',NULL,'EPSG','4400','EPSG','4747','EPSG','16023',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2202','projected_crs','EPSG','3183','EPSG','3454','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3184','GR96 / UTM zone 24N',NULL,'EPSG','4400','EPSG','4747','EPSG','16024',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2203','projected_crs','EPSG','3184','EPSG','3455','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3185','GR96 / UTM zone 25N',NULL,'EPSG','4400','EPSG','4747','EPSG','16025',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2204','projected_crs','EPSG','3185','EPSG','3456','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3186','GR96 / UTM zone 26N',NULL,'EPSG','4400','EPSG','4747','EPSG','16026',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2205','projected_crs','EPSG','3186','EPSG','3457','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3187','GR96 / UTM zone 27N',NULL,'EPSG','4400','EPSG','4747','EPSG','16027',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2206','projected_crs','EPSG','3187','EPSG','3458','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3188','GR96 / UTM zone 28N',NULL,'EPSG','4400','EPSG','4747','EPSG','16028',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2207','projected_crs','EPSG','3188','EPSG','3459','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3189','GR96 / UTM zone 29N',NULL,'EPSG','4400','EPSG','4747','EPSG','16029',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2208','projected_crs','EPSG','3189','EPSG','3460','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3190','LGD2006 / Libya TM zone 5',NULL,'EPSG','4499','EPSG','4754','EPSG','18310',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2209','projected_crs','EPSG','3190','EPSG','1470','EPSG','1207');
INSERT INTO "projected_crs" VALUES('EPSG','3191','LGD2006 / Libya TM zone 6',NULL,'EPSG','4499','EPSG','4754','EPSG','18311',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2210','projected_crs','EPSG','3191','EPSG','1471','EPSG','1207');
INSERT INTO "projected_crs" VALUES('EPSG','3192','LGD2006 / Libya TM zone 7',NULL,'EPSG','4499','EPSG','4754','EPSG','18312',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2211','projected_crs','EPSG','3192','EPSG','1472','EPSG','1207');
INSERT INTO "projected_crs" VALUES('EPSG','3193','LGD2006 / Libya TM zone 8',NULL,'EPSG','4499','EPSG','4754','EPSG','18313',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2212','projected_crs','EPSG','3193','EPSG','1473','EPSG','1207');
INSERT INTO "projected_crs" VALUES('EPSG','3194','LGD2006 / Libya TM zone 9',NULL,'EPSG','4499','EPSG','4754','EPSG','18314',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2213','projected_crs','EPSG','3194','EPSG','1474','EPSG','1207');
INSERT INTO "projected_crs" VALUES('EPSG','3195','LGD2006 / Libya TM zone 10',NULL,'EPSG','4499','EPSG','4754','EPSG','18315',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2214','projected_crs','EPSG','3195','EPSG','1475','EPSG','1207');
INSERT INTO "projected_crs" VALUES('EPSG','3196','LGD2006 / Libya TM zone 11',NULL,'EPSG','4499','EPSG','4754','EPSG','18316',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2215','projected_crs','EPSG','3196','EPSG','1476','EPSG','1207');
INSERT INTO "projected_crs" VALUES('EPSG','3197','LGD2006 / Libya TM zone 12',NULL,'EPSG','4499','EPSG','4754','EPSG','18317',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2216','projected_crs','EPSG','3197','EPSG','1477','EPSG','1207');
INSERT INTO "projected_crs" VALUES('EPSG','3198','LGD2006 / Libya TM zone 13',NULL,'EPSG','4499','EPSG','4754','EPSG','18318',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2217','projected_crs','EPSG','3198','EPSG','1478','EPSG','1207');
INSERT INTO "projected_crs" VALUES('EPSG','3199','LGD2006 / UTM zone 32N',NULL,'EPSG','4400','EPSG','4754','EPSG','16032',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2218','projected_crs','EPSG','3199','EPSG','3949','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3200','FD58 / Iraq zone',NULL,'EPSG','4400','EPSG','4132','EPSG','19906',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2219','projected_crs','EPSG','3200','EPSG','1300','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3201','LGD2006 / UTM zone 33N',NULL,'EPSG','4400','EPSG','4754','EPSG','16033',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2220','projected_crs','EPSG','3201','EPSG','3950','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3202','LGD2006 / UTM zone 34N',NULL,'EPSG','4400','EPSG','4754','EPSG','16034',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2221','projected_crs','EPSG','3202','EPSG','3951','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3203','LGD2006 / UTM zone 35N',NULL,'EPSG','4400','EPSG','4754','EPSG','16035',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2222','projected_crs','EPSG','3203','EPSG','3941','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3204','WGS 84 / SCAR IMW SP19-20',NULL,'EPSG','4400','EPSG','4326','EPSG','17204',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2223','projected_crs','EPSG','3204','EPSG','2991','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3205','WGS 84 / SCAR IMW SP21-22',NULL,'EPSG','4400','EPSG','4326','EPSG','17205',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2224','projected_crs','EPSG','3205','EPSG','2992','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3206','WGS 84 / SCAR IMW SP23-24',NULL,'EPSG','4400','EPSG','4326','EPSG','17206',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2225','projected_crs','EPSG','3206','EPSG','2993','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3207','WGS 84 / SCAR IMW SQ01-02',NULL,'EPSG','4400','EPSG','4326','EPSG','17207',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2226','projected_crs','EPSG','3207','EPSG','2994','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3208','WGS 84 / SCAR IMW SQ19-20',NULL,'EPSG','4400','EPSG','4326','EPSG','17208',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2227','projected_crs','EPSG','3208','EPSG','2995','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3209','WGS 84 / SCAR IMW SQ21-22',NULL,'EPSG','4400','EPSG','4326','EPSG','17209',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2228','projected_crs','EPSG','3209','EPSG','2996','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3210','WGS 84 / SCAR IMW SQ37-38',NULL,'EPSG','4400','EPSG','4326','EPSG','17210',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2229','projected_crs','EPSG','3210','EPSG','2997','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3211','WGS 84 / SCAR IMW SQ39-40',NULL,'EPSG','4400','EPSG','4326','EPSG','17211',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2230','projected_crs','EPSG','3211','EPSG','2998','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3212','WGS 84 / SCAR IMW SQ41-42',NULL,'EPSG','4400','EPSG','4326','EPSG','17212',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2231','projected_crs','EPSG','3212','EPSG','2999','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3213','WGS 84 / SCAR IMW SQ43-44',NULL,'EPSG','4400','EPSG','4326','EPSG','17213',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2232','projected_crs','EPSG','3213','EPSG','3000','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3214','WGS 84 / SCAR IMW SQ45-46',NULL,'EPSG','4400','EPSG','4326','EPSG','17214',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2233','projected_crs','EPSG','3214','EPSG','3001','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3215','WGS 84 / SCAR IMW SQ47-48',NULL,'EPSG','4400','EPSG','4326','EPSG','17215',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2234','projected_crs','EPSG','3215','EPSG','3002','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3216','WGS 84 / SCAR IMW SQ49-50',NULL,'EPSG','4400','EPSG','4326','EPSG','17216',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2235','projected_crs','EPSG','3216','EPSG','3003','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3217','WGS 84 / SCAR IMW SQ51-52',NULL,'EPSG','4400','EPSG','4326','EPSG','17217',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2236','projected_crs','EPSG','3217','EPSG','3004','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3218','WGS 84 / SCAR IMW SQ53-54',NULL,'EPSG','4400','EPSG','4326','EPSG','17218',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2237','projected_crs','EPSG','3218','EPSG','3005','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3219','WGS 84 / SCAR IMW SQ55-56',NULL,'EPSG','4400','EPSG','4326','EPSG','17219',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2238','projected_crs','EPSG','3219','EPSG','3006','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3220','WGS 84 / SCAR IMW SQ57-58',NULL,'EPSG','4400','EPSG','4326','EPSG','17220',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2239','projected_crs','EPSG','3220','EPSG','3007','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3221','WGS 84 / SCAR IMW SR13-14',NULL,'EPSG','4400','EPSG','4326','EPSG','17221',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2240','projected_crs','EPSG','3221','EPSG','3008','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3222','WGS 84 / SCAR IMW SR15-16',NULL,'EPSG','4400','EPSG','4326','EPSG','17222',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2241','projected_crs','EPSG','3222','EPSG','3009','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3223','WGS 84 / SCAR IMW SR17-18',NULL,'EPSG','4400','EPSG','4326','EPSG','17223',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2242','projected_crs','EPSG','3223','EPSG','3010','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3224','WGS 84 / SCAR IMW SR19-20',NULL,'EPSG','4400','EPSG','4326','EPSG','17224',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2243','projected_crs','EPSG','3224','EPSG','3011','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3225','WGS 84 / SCAR IMW SR27-28',NULL,'EPSG','4400','EPSG','4326','EPSG','17225',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2244','projected_crs','EPSG','3225','EPSG','3012','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3226','WGS 84 / SCAR IMW SR29-30',NULL,'EPSG','4400','EPSG','4326','EPSG','17226',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2245','projected_crs','EPSG','3226','EPSG','3013','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3227','WGS 84 / SCAR IMW SR31-32',NULL,'EPSG','4400','EPSG','4326','EPSG','17227',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2246','projected_crs','EPSG','3227','EPSG','3014','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3228','WGS 84 / SCAR IMW SR33-34',NULL,'EPSG','4400','EPSG','4326','EPSG','17228',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2247','projected_crs','EPSG','3228','EPSG','3015','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3229','WGS 84 / SCAR IMW SR35-36',NULL,'EPSG','4400','EPSG','4326','EPSG','17229',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2248','projected_crs','EPSG','3229','EPSG','3016','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3230','WGS 84 / SCAR IMW SR37-38',NULL,'EPSG','4400','EPSG','4326','EPSG','17230',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2249','projected_crs','EPSG','3230','EPSG','3017','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3231','WGS 84 / SCAR IMW SR39-40',NULL,'EPSG','4400','EPSG','4326','EPSG','17231',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2250','projected_crs','EPSG','3231','EPSG','3018','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3232','WGS 84 / SCAR IMW SR41-42',NULL,'EPSG','4400','EPSG','4326','EPSG','17232',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2251','projected_crs','EPSG','3232','EPSG','3019','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3233','WGS 84 / SCAR IMW SR43-44',NULL,'EPSG','4400','EPSG','4326','EPSG','17233',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2252','projected_crs','EPSG','3233','EPSG','3020','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3234','WGS 84 / SCAR IMW SR45-46',NULL,'EPSG','4400','EPSG','4326','EPSG','17234',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2253','projected_crs','EPSG','3234','EPSG','3021','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3235','WGS 84 / SCAR IMW SR47-48',NULL,'EPSG','4400','EPSG','4326','EPSG','17235',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2254','projected_crs','EPSG','3235','EPSG','3022','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3236','WGS 84 / SCAR IMW SR49-50',NULL,'EPSG','4400','EPSG','4326','EPSG','17236',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2255','projected_crs','EPSG','3236','EPSG','3023','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3237','WGS 84 / SCAR IMW SR51-52',NULL,'EPSG','4400','EPSG','4326','EPSG','17237',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2256','projected_crs','EPSG','3237','EPSG','3024','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3238','WGS 84 / SCAR IMW SR53-54',NULL,'EPSG','4400','EPSG','4326','EPSG','17238',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2257','projected_crs','EPSG','3238','EPSG','3025','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3239','WGS 84 / SCAR IMW SR55-56',NULL,'EPSG','4400','EPSG','4326','EPSG','17239',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2258','projected_crs','EPSG','3239','EPSG','3026','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3240','WGS 84 / SCAR IMW SR57-58',NULL,'EPSG','4400','EPSG','4326','EPSG','17240',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2259','projected_crs','EPSG','3240','EPSG','3027','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3241','WGS 84 / SCAR IMW SR59-60',NULL,'EPSG','4400','EPSG','4326','EPSG','17241',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2260','projected_crs','EPSG','3241','EPSG','3028','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3242','WGS 84 / SCAR IMW SS04-06',NULL,'EPSG','4400','EPSG','4326','EPSG','17242',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2261','projected_crs','EPSG','3242','EPSG','3029','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3243','WGS 84 / SCAR IMW SS07-09',NULL,'EPSG','4400','EPSG','4326','EPSG','17243',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2262','projected_crs','EPSG','3243','EPSG','3030','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3244','WGS 84 / SCAR IMW SS10-12',NULL,'EPSG','4400','EPSG','4326','EPSG','17244',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2263','projected_crs','EPSG','3244','EPSG','3031','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3245','WGS 84 / SCAR IMW SS13-15',NULL,'EPSG','4400','EPSG','4326','EPSG','17245',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2264','projected_crs','EPSG','3245','EPSG','3032','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3246','WGS 84 / SCAR IMW SS16-18',NULL,'EPSG','4400','EPSG','4326','EPSG','17246',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2265','projected_crs','EPSG','3246','EPSG','3033','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3247','WGS 84 / SCAR IMW SS19-21',NULL,'EPSG','4400','EPSG','4326','EPSG','17247',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2266','projected_crs','EPSG','3247','EPSG','3034','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3248','WGS 84 / SCAR IMW SS25-27',NULL,'EPSG','4400','EPSG','4326','EPSG','17248',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2267','projected_crs','EPSG','3248','EPSG','3035','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3249','WGS 84 / SCAR IMW SS28-30',NULL,'EPSG','4400','EPSG','4326','EPSG','17249',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2268','projected_crs','EPSG','3249','EPSG','3036','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3250','WGS 84 / SCAR IMW SS31-33',NULL,'EPSG','4400','EPSG','4326','EPSG','17250',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2269','projected_crs','EPSG','3250','EPSG','3037','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3251','WGS 84 / SCAR IMW SS34-36',NULL,'EPSG','4400','EPSG','4326','EPSG','17251',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2270','projected_crs','EPSG','3251','EPSG','3038','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3252','WGS 84 / SCAR IMW SS37-39',NULL,'EPSG','4400','EPSG','4326','EPSG','17252',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2271','projected_crs','EPSG','3252','EPSG','3039','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3253','WGS 84 / SCAR IMW SS40-42',NULL,'EPSG','4400','EPSG','4326','EPSG','17253',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2272','projected_crs','EPSG','3253','EPSG','3040','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3254','WGS 84 / SCAR IMW SS43-45',NULL,'EPSG','4400','EPSG','4326','EPSG','17254',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2273','projected_crs','EPSG','3254','EPSG','3041','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3255','WGS 84 / SCAR IMW SS46-48',NULL,'EPSG','4400','EPSG','4326','EPSG','17255',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2274','projected_crs','EPSG','3255','EPSG','3042','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3256','WGS 84 / SCAR IMW SS49-51',NULL,'EPSG','4400','EPSG','4326','EPSG','17256',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2275','projected_crs','EPSG','3256','EPSG','3043','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3257','WGS 84 / SCAR IMW SS52-54',NULL,'EPSG','4400','EPSG','4326','EPSG','17257',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2276','projected_crs','EPSG','3257','EPSG','3044','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3258','WGS 84 / SCAR IMW SS55-57',NULL,'EPSG','4400','EPSG','4326','EPSG','17258',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2277','projected_crs','EPSG','3258','EPSG','3045','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3259','WGS 84 / SCAR IMW SS58-60',NULL,'EPSG','4400','EPSG','4326','EPSG','17259',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2278','projected_crs','EPSG','3259','EPSG','3046','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3260','WGS 84 / SCAR IMW ST01-04',NULL,'EPSG','4400','EPSG','4326','EPSG','17260',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2279','projected_crs','EPSG','3260','EPSG','3047','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3261','WGS 84 / SCAR IMW ST05-08',NULL,'EPSG','4400','EPSG','4326','EPSG','17261',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2280','projected_crs','EPSG','3261','EPSG','3048','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3262','WGS 84 / SCAR IMW ST09-12',NULL,'EPSG','4400','EPSG','4326','EPSG','17262',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2281','projected_crs','EPSG','3262','EPSG','3049','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3263','WGS 84 / SCAR IMW ST13-16',NULL,'EPSG','4400','EPSG','4326','EPSG','17263',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2282','projected_crs','EPSG','3263','EPSG','3050','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3264','WGS 84 / SCAR IMW ST17-20',NULL,'EPSG','4400','EPSG','4326','EPSG','17264',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2283','projected_crs','EPSG','3264','EPSG','3051','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3265','WGS 84 / SCAR IMW ST21-24',NULL,'EPSG','4400','EPSG','4326','EPSG','17265',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2284','projected_crs','EPSG','3265','EPSG','3052','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3266','WGS 84 / SCAR IMW ST25-28',NULL,'EPSG','4400','EPSG','4326','EPSG','17266',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2285','projected_crs','EPSG','3266','EPSG','3053','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3267','WGS 84 / SCAR IMW ST29-32',NULL,'EPSG','4400','EPSG','4326','EPSG','17267',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2286','projected_crs','EPSG','3267','EPSG','3054','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3268','WGS 84 / SCAR IMW ST33-36',NULL,'EPSG','4400','EPSG','4326','EPSG','17268',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2287','projected_crs','EPSG','3268','EPSG','3055','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3269','WGS 84 / SCAR IMW ST37-40',NULL,'EPSG','4400','EPSG','4326','EPSG','17269',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2288','projected_crs','EPSG','3269','EPSG','3056','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3270','WGS 84 / SCAR IMW ST41-44',NULL,'EPSG','4400','EPSG','4326','EPSG','17270',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2289','projected_crs','EPSG','3270','EPSG','3057','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3271','WGS 84 / SCAR IMW ST45-48',NULL,'EPSG','4400','EPSG','4326','EPSG','17271',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2290','projected_crs','EPSG','3271','EPSG','3058','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3272','WGS 84 / SCAR IMW ST49-52',NULL,'EPSG','4400','EPSG','4326','EPSG','17272',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2291','projected_crs','EPSG','3272','EPSG','3059','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3273','WGS 84 / SCAR IMW ST53-56',NULL,'EPSG','4400','EPSG','4326','EPSG','17273',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2292','projected_crs','EPSG','3273','EPSG','3060','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3274','WGS 84 / SCAR IMW ST57-60',NULL,'EPSG','4400','EPSG','4326','EPSG','17274',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2293','projected_crs','EPSG','3274','EPSG','3061','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3275','WGS 84 / SCAR IMW SU01-05',NULL,'EPSG','4471','EPSG','4326','EPSG','17275',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2294','projected_crs','EPSG','3275','EPSG','3062','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3276','WGS 84 / SCAR IMW SU06-10',NULL,'EPSG','4473','EPSG','4326','EPSG','17276',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2295','projected_crs','EPSG','3276','EPSG','3063','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3277','WGS 84 / SCAR IMW SU11-15',NULL,'EPSG','4474','EPSG','4326','EPSG','17277',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2296','projected_crs','EPSG','3277','EPSG','3064','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3278','WGS 84 / SCAR IMW SU16-20',NULL,'EPSG','4476','EPSG','4326','EPSG','17278',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2297','projected_crs','EPSG','3278','EPSG','3065','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3279','WGS 84 / SCAR IMW SU21-25',NULL,'EPSG','4477','EPSG','4326','EPSG','17279',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2298','projected_crs','EPSG','3279','EPSG','3066','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3280','WGS 84 / SCAR IMW SU26-30',NULL,'EPSG','4479','EPSG','4326','EPSG','17280',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2299','projected_crs','EPSG','3280','EPSG','3067','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3281','WGS 84 / SCAR IMW SU31-35',NULL,'EPSG','4480','EPSG','4326','EPSG','17281',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2300','projected_crs','EPSG','3281','EPSG','3068','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3282','WGS 84 / SCAR IMW SU36-40',NULL,'EPSG','4482','EPSG','4326','EPSG','17282',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2301','projected_crs','EPSG','3282','EPSG','3069','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3283','WGS 84 / SCAR IMW SU41-45',NULL,'EPSG','4483','EPSG','4326','EPSG','17283',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2302','projected_crs','EPSG','3283','EPSG','3070','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3284','WGS 84 / SCAR IMW SU46-50',NULL,'EPSG','4485','EPSG','4326','EPSG','17284',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2303','projected_crs','EPSG','3284','EPSG','3071','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3285','WGS 84 / SCAR IMW SU51-55',NULL,'EPSG','4486','EPSG','4326','EPSG','17285',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2304','projected_crs','EPSG','3285','EPSG','3072','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3286','WGS 84 / SCAR IMW SU56-60',NULL,'EPSG','4488','EPSG','4326','EPSG','17286',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2305','projected_crs','EPSG','3286','EPSG','3073','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3287','WGS 84 / SCAR IMW SV01-10',NULL,'EPSG','4472','EPSG','4326','EPSG','17287',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2306','projected_crs','EPSG','3287','EPSG','3074','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3288','WGS 84 / SCAR IMW SV11-20',NULL,'EPSG','4475','EPSG','4326','EPSG','17288',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2307','projected_crs','EPSG','3288','EPSG','3075','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3289','WGS 84 / SCAR IMW SV21-30',NULL,'EPSG','4478','EPSG','4326','EPSG','17289',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2308','projected_crs','EPSG','3289','EPSG','3076','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3290','WGS 84 / SCAR IMW SV31-40',NULL,'EPSG','4481','EPSG','4326','EPSG','17290',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2309','projected_crs','EPSG','3290','EPSG','3077','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3291','WGS 84 / SCAR IMW SV41-50',NULL,'EPSG','4484','EPSG','4326','EPSG','17291',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2310','projected_crs','EPSG','3291','EPSG','3078','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3292','WGS 84 / SCAR IMW SV51-60',NULL,'EPSG','4487','EPSG','4326','EPSG','17292',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2311','projected_crs','EPSG','3292','EPSG','3079','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3293','WGS 84 / SCAR IMW SW01-60',NULL,'EPSG','4490','EPSG','4326','EPSG','17293',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2312','projected_crs','EPSG','3293','EPSG','3080','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3294','WGS 84 / USGS Transantarctic Mountains',NULL,'EPSG','4400','EPSG','4326','EPSG','17294',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2313','projected_crs','EPSG','3294','EPSG','3081','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3295','Guam 1963 / Yap Islands',NULL,'EPSG','4499','EPSG','4675','EPSG','15399',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2314','projected_crs','EPSG','3295','EPSG','3108','EPSG','1153');
INSERT INTO "projected_crs" VALUES('EPSG','3296','RGPF / UTM zone 5S',NULL,'EPSG','4400','EPSG','4687','EPSG','16105',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2315','projected_crs','EPSG','3296','EPSG','3120','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3297','RGPF / UTM zone 6S',NULL,'EPSG','4400','EPSG','4687','EPSG','16106',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2316','projected_crs','EPSG','3297','EPSG','3121','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3298','RGPF / UTM zone 7S',NULL,'EPSG','4400','EPSG','4687','EPSG','16107',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2317','projected_crs','EPSG','3298','EPSG','3122','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3299','RGPF / UTM zone 8S',NULL,'EPSG','4400','EPSG','4687','EPSG','16108',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2318','projected_crs','EPSG','3299','EPSG','3123','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3300','Estonian Coordinate System of 1992',NULL,'EPSG','4530','EPSG','4133','EPSG','19938',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2319','projected_crs','EPSG','3300','EPSG','3246','EPSG','1209');
INSERT INTO "projected_crs" VALUES('EPSG','3301','Estonian Coordinate System of 1997',NULL,'EPSG','4530','EPSG','4180','EPSG','19938',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2320','projected_crs','EPSG','3301','EPSG','1090','EPSG','1209');
INSERT INTO "projected_crs" VALUES('EPSG','3302','IGN63 Hiva Oa / UTM zone 7S',NULL,'EPSG','4400','EPSG','4689','EPSG','16107',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2321','projected_crs','EPSG','3302','EPSG','3130','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3303','Fatu Iva 72 / UTM zone 7S',NULL,'EPSG','4400','EPSG','4688','EPSG','16107',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2322','projected_crs','EPSG','3303','EPSG','3133','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3304','Tahiti 79 / UTM zone 6S',NULL,'EPSG','4400','EPSG','4690','EPSG','16106',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2323','projected_crs','EPSG','3304','EPSG','3124','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3305','Moorea 87 / UTM zone 6S',NULL,'EPSG','4400','EPSG','4691','EPSG','16106',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2324','projected_crs','EPSG','3305','EPSG','3125','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3306','Maupiti 83 / UTM zone 5S',NULL,'EPSG','4400','EPSG','4692','EPSG','16105',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2325','projected_crs','EPSG','3306','EPSG','3126','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3307','Nakhl-e Ghanem / UTM zone 39N',NULL,'EPSG','4400','EPSG','4693','EPSG','16039',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2326','projected_crs','EPSG','3307','EPSG','2362','EPSG','1140');
INSERT INTO "projected_crs" VALUES('EPSG','3308','GDA94 / NSW Lambert',NULL,'EPSG','4400','EPSG','4283','EPSG','17364',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2327','projected_crs','EPSG','3308','EPSG','3139','EPSG','1135');
INSERT INTO "projected_crs" VALUES('EPSG','3309','NAD27 / California Albers',NULL,'EPSG','4499','EPSG','4267','EPSG','10420',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2328','projected_crs','EPSG','3309','EPSG','1375','EPSG','1135');
INSERT INTO "projected_crs" VALUES('EPSG','3310','NAD83 / California Albers',NULL,'EPSG','4499','EPSG','4269','EPSG','10420',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2329','projected_crs','EPSG','3310','EPSG','1375','EPSG','1135');
INSERT INTO "projected_crs" VALUES('EPSG','3311','NAD83(HARN) / California Albers',NULL,'EPSG','4499','EPSG','4152','EPSG','10420',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2330','projected_crs','EPSG','3311','EPSG','1375','EPSG','1135');
INSERT INTO "projected_crs" VALUES('EPSG','3312','CSG67 / UTM zone 21N',NULL,'EPSG','4400','EPSG','4623','EPSG','16021',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2331','projected_crs','EPSG','3312','EPSG','3765','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3313','RGFG95 / UTM zone 21N',NULL,'EPSG','4400','EPSG','4624','EPSG','16021',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2332','projected_crs','EPSG','3313','EPSG','3145','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3314','Katanga 1955 / Katanga Lambert',NULL,'EPSG','4400','EPSG','4695','EPSG','17401',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2333','projected_crs','EPSG','3314','EPSG','3147','EPSG','1210');
INSERT INTO "projected_crs" VALUES('EPSG','3315','Katanga 1955 / Katanga TM',NULL,'EPSG','4400','EPSG','4695','EPSG','17402',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2334','projected_crs','EPSG','3315','EPSG','3147','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','3316','Kasai 1953 / Congo TM zone 22',NULL,'EPSG','4400','EPSG','4696','EPSG','17422',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2335','projected_crs','EPSG','3316','EPSG','4587','EPSG','1057');
INSERT INTO "projected_crs" VALUES('EPSG','3317','Kasai 1953 / Congo TM zone 24',NULL,'EPSG','4400','EPSG','4696','EPSG','17424',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2336','projected_crs','EPSG','3317','EPSG','3164','EPSG','1057');
INSERT INTO "projected_crs" VALUES('EPSG','3318','IGC 1962 / Congo TM zone 12',NULL,'EPSG','4400','EPSG','4697','EPSG','17412',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2337','projected_crs','EPSG','3318','EPSG','3150','EPSG','1057');
INSERT INTO "projected_crs" VALUES('EPSG','3319','IGC 1962 / Congo TM zone 14',NULL,'EPSG','4400','EPSG','4697','EPSG','17414',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2338','projected_crs','EPSG','3319','EPSG','3151','EPSG','1057');
INSERT INTO "projected_crs" VALUES('EPSG','3320','IGC 1962 / Congo TM zone 16',NULL,'EPSG','4400','EPSG','4697','EPSG','17416',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2339','projected_crs','EPSG','3320','EPSG','3160','EPSG','1057');
INSERT INTO "projected_crs" VALUES('EPSG','3321','IGC 1962 / Congo TM zone 18',NULL,'EPSG','4400','EPSG','4697','EPSG','17418',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2340','projected_crs','EPSG','3321','EPSG','3161','EPSG','1057');
INSERT INTO "projected_crs" VALUES('EPSG','3322','IGC 1962 / Congo TM zone 20',NULL,'EPSG','4400','EPSG','4697','EPSG','17420',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2341','projected_crs','EPSG','3322','EPSG','3162','EPSG','1057');
INSERT INTO "projected_crs" VALUES('EPSG','3323','IGC 1962 / Congo TM zone 22',NULL,'EPSG','4400','EPSG','4697','EPSG','17422',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2342','projected_crs','EPSG','3323','EPSG','3163','EPSG','1057');
INSERT INTO "projected_crs" VALUES('EPSG','3324','IGC 1962 / Congo TM zone 24',NULL,'EPSG','4400','EPSG','4697','EPSG','17424',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2343','projected_crs','EPSG','3324','EPSG','3164','EPSG','1057');
INSERT INTO "projected_crs" VALUES('EPSG','3325','IGC 1962 / Congo TM zone 26',NULL,'EPSG','4400','EPSG','4697','EPSG','17426',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2344','projected_crs','EPSG','3325','EPSG','3165','EPSG','1057');
INSERT INTO "projected_crs" VALUES('EPSG','3326','IGC 1962 / Congo TM zone 28',NULL,'EPSG','4400','EPSG','4697','EPSG','17428',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2345','projected_crs','EPSG','3326','EPSG','3166','EPSG','1057');
INSERT INTO "projected_crs" VALUES('EPSG','3327','IGC 1962 / Congo TM zone 30',NULL,'EPSG','4400','EPSG','4697','EPSG','17430',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2346','projected_crs','EPSG','3327','EPSG','3167','EPSG','1057');
INSERT INTO "projected_crs" VALUES('EPSG','3328','Pulkovo 1942(58) / GUGiK-80',NULL,'EPSG','4530','EPSG','4179','EPSG','18286',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2347','projected_crs','EPSG','3328','EPSG','3293','EPSG','1241');
INSERT INTO "projected_crs" VALUES('EPSG','3329','Pulkovo 1942(58) / 3-degree Gauss-Kruger zone 5',NULL,'EPSG','4530','EPSG','4179','EPSG','16265',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2348','projected_crs','EPSG','3329','EPSG','3580','EPSG','1160');
INSERT INTO "projected_crs" VALUES('EPSG','3330','Pulkovo 1942(58) / 3-degree Gauss-Kruger zone 6',NULL,'EPSG','4530','EPSG','4179','EPSG','16266',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2349','projected_crs','EPSG','3330','EPSG','3581','EPSG','1160');
INSERT INTO "projected_crs" VALUES('EPSG','3331','Pulkovo 1942(58) / 3-degree Gauss-Kruger zone 7',NULL,'EPSG','4530','EPSG','4179','EPSG','16267',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2350','projected_crs','EPSG','3331','EPSG','3583','EPSG','1160');
INSERT INTO "projected_crs" VALUES('EPSG','3332','Pulkovo 1942(58) / 3-degree Gauss-Kruger zone 8',NULL,'EPSG','4530','EPSG','4179','EPSG','16268',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2351','projected_crs','EPSG','3332','EPSG','3585','EPSG','1160');
INSERT INTO "projected_crs" VALUES('EPSG','3333','Pulkovo 1942(58) / Gauss-Kruger zone 3',NULL,'EPSG','4530','EPSG','4179','EPSG','16203',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2352','projected_crs','EPSG','3333','EPSG','1792','EPSG','1160');
INSERT INTO "projected_crs" VALUES('EPSG','3334','Pulkovo 1942(58) / Gauss-Kruger zone 4',NULL,'EPSG','4530','EPSG','4179','EPSG','16204',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2353','projected_crs','EPSG','3334','EPSG','3577','EPSG','1160');
INSERT INTO "projected_crs" VALUES('EPSG','3335','Pulkovo 1942(58) / Gauss-Kruger zone 5',NULL,'EPSG','4530','EPSG','4179','EPSG','16205',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2354','projected_crs','EPSG','3335','EPSG','3579','EPSG','1160');
INSERT INTO "projected_crs" VALUES('EPSG','3336','IGN 1962 Kerguelen / UTM zone 42S',NULL,'EPSG','4400','EPSG','4698','EPSG','16142',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2355','projected_crs','EPSG','3336','EPSG','2816','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3337','Le Pouce 1934 / Mauritius Grid',NULL,'EPSG','4400','EPSG','4699','EPSG','19899',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2356','projected_crs','EPSG','3337','EPSG','3209','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3338','NAD83 / Alaska Albers',NULL,'EPSG','4499','EPSG','4269','EPSG','15021',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2357','projected_crs','EPSG','3338','EPSG','1330','EPSG','1241');
INSERT INTO "projected_crs" VALUES('EPSG','3339','IGCB 1955 / Congo TM zone 12',NULL,'EPSG','4400','EPSG','4701','EPSG','17412',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2358','projected_crs','EPSG','3339','EPSG','3150','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3340','IGCB 1955 / Congo TM zone 14',NULL,'EPSG','4400','EPSG','4701','EPSG','17414',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2359','projected_crs','EPSG','3340','EPSG','3151','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3341','IGCB 1955 / Congo TM zone 16',NULL,'EPSG','4400','EPSG','4701','EPSG','17416',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2360','projected_crs','EPSG','3341','EPSG','4012','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3342','IGCB 1955 / UTM zone 33S',NULL,'EPSG','4400','EPSG','4701','EPSG','16133',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2361','projected_crs','EPSG','3342','EPSG','3171','EPSG','1136');
INSERT INTO "projected_crs" VALUES('EPSG','3343','Mauritania 1999 / UTM zone 28N',NULL,'EPSG','4400','EPSG','4702','EPSG','16028',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2362','projected_crs','EPSG','3343','EPSG','3938','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3344','Mauritania 1999 / UTM zone 29N',NULL,'EPSG','4400','EPSG','4702','EPSG','16029',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2363','projected_crs','EPSG','3344','EPSG','2970','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3345','Mauritania 1999 / UTM zone 30N',NULL,'EPSG','4400','EPSG','4702','EPSG','16030',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2364','projected_crs','EPSG','3345','EPSG','2969','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3346','LKS94 / Lithuania TM',NULL,'EPSG','4530','EPSG','4669','EPSG','19934',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2365','projected_crs','EPSG','3346','EPSG','1145','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3347','NAD83 / Statistics Canada Lambert',NULL,'EPSG','4400','EPSG','4269','EPSG','19897',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2366','projected_crs','EPSG','3347','EPSG','1061','EPSG','1241');
INSERT INTO "projected_crs" VALUES('EPSG','3348','NAD83(CSRS) / Statistics Canada Lambert',NULL,'EPSG','4400','EPSG','4617','EPSG','19897',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2367','projected_crs','EPSG','3348','EPSG','1061','EPSG','1241');
INSERT INTO "projected_crs" VALUES('EPSG','3349','WGS 84 / PDC Mercator',NULL,'EPSG','4400','EPSG','4326','EPSG','19898',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2368','projected_crs','EPSG','3349','EPSG','3172','EPSG','1241');
INSERT INTO "projected_crs" VALUES('EPSG','3350','Pulkovo 1942 / CS63 zone C0',NULL,'EPSG','4530','EPSG','4284','EPSG','18450',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2369','projected_crs','EPSG','3350','EPSG','3173','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3351','Pulkovo 1942 / CS63 zone C1',NULL,'EPSG','4530','EPSG','4284','EPSG','18451',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2370','projected_crs','EPSG','3351','EPSG','3174','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3352','Pulkovo 1942 / CS63 zone C2',NULL,'EPSG','4530','EPSG','4284','EPSG','18452',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2371','projected_crs','EPSG','3352','EPSG','3175','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3353','Mhast (onshore) / UTM zone 32S',NULL,'EPSG','4400','EPSG','4704','EPSG','16132',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2372','projected_crs','EPSG','3353','EPSG','3179','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3354','Mhast (offshore) / UTM zone 32S',NULL,'EPSG','4400','EPSG','4705','EPSG','16132',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2373','projected_crs','EPSG','3354','EPSG','3180','EPSG','1136');
INSERT INTO "projected_crs" VALUES('EPSG','3355','Egypt Gulf of Suez S-650 TL / Red Belt',NULL,'EPSG','4400','EPSG','4706','EPSG','18072',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2374','projected_crs','EPSG','3355','EPSG','2341','EPSG','1136');
INSERT INTO "projected_crs" VALUES('EPSG','3356','Grand Cayman 1959 / UTM zone 17N',NULL,'EPSG','4400','EPSG','4723','EPSG','16017',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2375','projected_crs','EPSG','3356','EPSG','3185','EPSG','1056');
INSERT INTO "projected_crs" VALUES('EPSG','3357','Little Cayman 1961 / UTM zone 17N',NULL,'EPSG','4400','EPSG','4726','EPSG','16017',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2376','projected_crs','EPSG','3357','EPSG','3186','EPSG','1056');
INSERT INTO "projected_crs" VALUES('EPSG','3358','NAD83(HARN) / North Carolina',NULL,'EPSG','4499','EPSG','4152','EPSG','13230',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2377','projected_crs','EPSG','3358','EPSG','1402','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3359','NAD83(HARN) / North Carolina (ftUS)',NULL,'EPSG','4495','EPSG','4152','EPSG','15346',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2378','projected_crs','EPSG','3359','EPSG','1402','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3360','NAD83(HARN) / South Carolina',NULL,'EPSG','4499','EPSG','4152','EPSG','13930',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2379','projected_crs','EPSG','3360','EPSG','1409','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3361','NAD83(HARN) / South Carolina (ft)',NULL,'EPSG','4495','EPSG','4152','EPSG','15355',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2380','projected_crs','EPSG','3361','EPSG','1409','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3362','NAD83(HARN) / Pennsylvania North',NULL,'EPSG','4499','EPSG','4152','EPSG','13731',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2381','projected_crs','EPSG','3362','EPSG','2245','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3363','NAD83(HARN) / Pennsylvania North (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15353',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2382','projected_crs','EPSG','3363','EPSG','2245','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3364','NAD83(HARN) / Pennsylvania South',NULL,'EPSG','4499','EPSG','4152','EPSG','13732',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2383','projected_crs','EPSG','3364','EPSG','2246','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3365','NAD83(HARN) / Pennsylvania South (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15354',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2384','projected_crs','EPSG','3365','EPSG','2246','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3366','Hong Kong 1963 Grid System',NULL,'EPSG','4500','EPSG','4738','EPSG','19896',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2385','projected_crs','EPSG','3366','EPSG','1118','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','3367','IGN Astro 1960 / UTM zone 28N',NULL,'EPSG','4400','EPSG','4700','EPSG','16028',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2386','projected_crs','EPSG','3367','EPSG','2971','EPSG','1241');
INSERT INTO "projected_crs" VALUES('EPSG','3368','IGN Astro 1960 / UTM zone 29N',NULL,'EPSG','4400','EPSG','4700','EPSG','16029',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2387','projected_crs','EPSG','3368','EPSG','2970','EPSG','1241');
INSERT INTO "projected_crs" VALUES('EPSG','3369','IGN Astro 1960 / UTM zone 30N',NULL,'EPSG','4400','EPSG','4700','EPSG','16030',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2388','projected_crs','EPSG','3369','EPSG','2969','EPSG','1241');
INSERT INTO "projected_crs" VALUES('EPSG','3370','NAD27 / UTM zone 59N',NULL,'EPSG','4400','EPSG','4267','EPSG','16059',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2389','projected_crs','EPSG','3370','EPSG','3372','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3371','NAD27 / UTM zone 60N',NULL,'EPSG','4400','EPSG','4267','EPSG','16060',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2390','projected_crs','EPSG','3371','EPSG','3373','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3372','NAD83 / UTM zone 59N',NULL,'EPSG','4400','EPSG','4269','EPSG','16059',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2391','projected_crs','EPSG','3372','EPSG','3372','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3373','NAD83 / UTM zone 60N',NULL,'EPSG','4400','EPSG','4269','EPSG','16060',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2392','projected_crs','EPSG','3373','EPSG','3373','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3374','FD54 / UTM zone 29N',NULL,'EPSG','4400','EPSG','4741','EPSG','16029',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2393','projected_crs','EPSG','3374','EPSG','3248','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3375','GDM2000 / Peninsula RSO',NULL,'EPSG','4400','EPSG','4742','EPSG','19895',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2394','projected_crs','EPSG','3375','EPSG','3955','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3376','GDM2000 / East Malaysia BRSO',NULL,'EPSG','4400','EPSG','4742','EPSG','19894',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2395','projected_crs','EPSG','3376','EPSG','3977','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3377','GDM2000 / Johor Grid',NULL,'EPSG','4400','EPSG','4742','EPSG','19893',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2396','projected_crs','EPSG','3377','EPSG','3376','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','3378','GDM2000 / Sembilan and Melaka Grid',NULL,'EPSG','4400','EPSG','4742','EPSG','19892',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2397','projected_crs','EPSG','3378','EPSG','3377','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','3379','GDM2000 / Pahang Grid',NULL,'EPSG','4400','EPSG','4742','EPSG','19891',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2398','projected_crs','EPSG','3379','EPSG','3378','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','3380','GDM2000 / Selangor Grid',NULL,'EPSG','4400','EPSG','4742','EPSG','19890',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2399','projected_crs','EPSG','3380','EPSG','3379','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','3381','GDM2000 / Terengganu Grid',NULL,'EPSG','4400','EPSG','4742','EPSG','19889',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2400','projected_crs','EPSG','3381','EPSG','3380','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','3382','GDM2000 / Pinang Grid',NULL,'EPSG','4400','EPSG','4742','EPSG','19888',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2401','projected_crs','EPSG','3382','EPSG','3381','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','3383','GDM2000 / Kedah and Perlis Grid',NULL,'EPSG','4400','EPSG','4742','EPSG','19887',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2402','projected_crs','EPSG','3383','EPSG','3382','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','3384','GDM2000 / Perak Grid',NULL,'EPSG','4400','EPSG','4742','EPSG','19886',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2403','projected_crs','EPSG','3384','EPSG','3383','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','3385','GDM2000 / Kelantan Grid',NULL,'EPSG','4400','EPSG','4742','EPSG','19885',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2404','projected_crs','EPSG','3385','EPSG','3384','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','3386','KKJ / Finland zone 0',NULL,'EPSG','4530','EPSG','4123','EPSG','18180',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2405','projected_crs','EPSG','3386','EPSG','3092','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3387','KKJ / Finland zone 5',NULL,'EPSG','4530','EPSG','4123','EPSG','18205',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2406','projected_crs','EPSG','3387','EPSG','3385','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3388','Pulkovo 1942 / Caspian Sea Mercator',NULL,'EPSG','4534','EPSG','4284','EPSG','19884',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2407','projected_crs','EPSG','3388','EPSG','1291','EPSG','1198');
INSERT INTO "projected_crs" VALUES('EPSG','3389','Pulkovo 1942 / 3-degree Gauss-Kruger zone 60',NULL,'EPSG','4530','EPSG','4284','EPSG','16099',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2408','projected_crs','EPSG','3389','EPSG','2706','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','3390','Pulkovo 1995 / 3-degree Gauss-Kruger zone 60',NULL,'EPSG','4530','EPSG','4200','EPSG','16099',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2409','projected_crs','EPSG','3390','EPSG','2706','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','3391','Karbala 1979 / UTM zone 37N',NULL,'EPSG','4400','EPSG','4743','EPSG','16037',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2410','projected_crs','EPSG','3391','EPSG','3387','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3392','Karbala 1979 / UTM zone 38N',NULL,'EPSG','4400','EPSG','4743','EPSG','16038',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2411','projected_crs','EPSG','3392','EPSG','3388','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3393','Karbala 1979 / UTM zone 39N',NULL,'EPSG','4400','EPSG','4743','EPSG','16039',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2412','projected_crs','EPSG','3393','EPSG','3389','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3394','Nahrwan 1934 / Iraq zone',NULL,'EPSG','4400','EPSG','4744','EPSG','19906',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2413','projected_crs','EPSG','3394','EPSG','4238','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3395','WGS 84 / World Mercator',NULL,'EPSG','4400','EPSG','4326','EPSG','19883',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2414','projected_crs','EPSG','3395','EPSG','3391','EPSG','1228');
INSERT INTO "projected_crs" VALUES('EPSG','3396','PD/83 / 3-degree Gauss-Kruger zone 3',NULL,'EPSG','4530','EPSG','4746','EPSG','16263',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2415','projected_crs','EPSG','3396','EPSG','3392','EPSG','1091');
INSERT INTO "projected_crs" VALUES('EPSG','3397','PD/83 / 3-degree Gauss-Kruger zone 4',NULL,'EPSG','4530','EPSG','4746','EPSG','16264',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2416','projected_crs','EPSG','3397','EPSG','3393','EPSG','1091');
INSERT INTO "projected_crs" VALUES('EPSG','3398','RD/83 / 3-degree Gauss-Kruger zone 4',NULL,'EPSG','4530','EPSG','4745','EPSG','16264',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2417','projected_crs','EPSG','3398','EPSG','3395','EPSG','1091');
INSERT INTO "projected_crs" VALUES('EPSG','3399','RD/83 / 3-degree Gauss-Kruger zone 5',NULL,'EPSG','4530','EPSG','4745','EPSG','16265',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2418','projected_crs','EPSG','3399','EPSG','3394','EPSG','1091');
INSERT INTO "projected_crs" VALUES('EPSG','3400','NAD83 / Alberta 10-TM (Forest)',NULL,'EPSG','4400','EPSG','4269','EPSG','19881',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2419','projected_crs','EPSG','3400','EPSG','2376','EPSG','1220');
INSERT INTO "projected_crs" VALUES('EPSG','3401','NAD83 / Alberta 10-TM (Resource)',NULL,'EPSG','4400','EPSG','4269','EPSG','19882',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2420','projected_crs','EPSG','3401','EPSG','2376','EPSG','1220');
INSERT INTO "projected_crs" VALUES('EPSG','3402','NAD83(CSRS) / Alberta 10-TM (Forest)',NULL,'EPSG','4400','EPSG','4617','EPSG','19881',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2421','projected_crs','EPSG','3402','EPSG','2376','EPSG','1220');
INSERT INTO "projected_crs" VALUES('EPSG','3403','NAD83(CSRS) / Alberta 10-TM (Resource)',NULL,'EPSG','4400','EPSG','4617','EPSG','19882',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2422','projected_crs','EPSG','3403','EPSG','2376','EPSG','1220');
INSERT INTO "projected_crs" VALUES('EPSG','3404','NAD83(HARN) / North Carolina (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15346',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2423','projected_crs','EPSG','3404','EPSG','1402','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3405','VN-2000 / UTM zone 48N',NULL,'EPSG','4400','EPSG','4756','EPSG','16048',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2424','projected_crs','EPSG','3405','EPSG','1452','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3406','VN-2000 / UTM zone 49N',NULL,'EPSG','4400','EPSG','4756','EPSG','16049',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2425','projected_crs','EPSG','3406','EPSG','1453','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3407','Hong Kong 1963 Grid System',NULL,'EPSG','4502','EPSG','4738','EPSG','19896',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2426','projected_crs','EPSG','3407','EPSG','1118','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','3408','NSIDC EASE-Grid North',NULL,'EPSG','4469','EPSG','10346','EPSG','3897',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2427','projected_crs','EPSG','3408','EPSG','3475','EPSG','1195');
INSERT INTO "projected_crs" VALUES('EPSG','3409','NSIDC EASE-Grid South',NULL,'EPSG','4470','EPSG','10346','EPSG','3898',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2428','projected_crs','EPSG','3409','EPSG','3474','EPSG','1195');
INSERT INTO "projected_crs" VALUES('EPSG','3410','NSIDC EASE-Grid Global',NULL,'EPSG','4499','EPSG','10346','EPSG','19869',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2429','projected_crs','EPSG','3410','EPSG','3463','EPSG','1195');
INSERT INTO "projected_crs" VALUES('EPSG','3411','NSIDC Sea Ice Polar Stereographic North',NULL,'EPSG','4468','EPSG','10345','EPSG','19865',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2430','projected_crs','EPSG','3411','EPSG','1996','EPSG','1040');
INSERT INTO "projected_crs" VALUES('EPSG','3412','NSIDC Sea Ice Polar Stereographic South',NULL,'EPSG','4470','EPSG','10345','EPSG','19866',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2431','projected_crs','EPSG','3412','EPSG','1997','EPSG','1040');
INSERT INTO "projected_crs" VALUES('EPSG','3413','WGS 84 / NSIDC Sea Ice Polar Stereographic North',NULL,'EPSG','4468','EPSG','4326','EPSG','19865',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2432','projected_crs','EPSG','3413','EPSG','1996','EPSG','1040');
INSERT INTO "projected_crs" VALUES('EPSG','3414','SVY21 / Singapore TM',NULL,'EPSG','4500','EPSG','4757','EPSG','19864',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2433','projected_crs','EPSG','3414','EPSG','1210','EPSG','1056');
INSERT INTO "projected_crs" VALUES('EPSG','3415','WGS 72BE / South China Sea Lambert',NULL,'EPSG','4400','EPSG','4324','EPSG','19863',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2434','projected_crs','EPSG','3415','EPSG','3470','EPSG','1136');
INSERT INTO "projected_crs" VALUES('EPSG','3416','ETRS89 / Austria Lambert',NULL,'EPSG','4530','EPSG','4258','EPSG','19947',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2435','projected_crs','EPSG','3416','EPSG','1037','EPSG','1210');
INSERT INTO "projected_crs" VALUES('EPSG','3417','NAD83 / Iowa North (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15377',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2436','projected_crs','EPSG','3417','EPSG','2198','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3418','NAD83 / Iowa South (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15378',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2437','projected_crs','EPSG','3418','EPSG','2199','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3419','NAD83 / Kansas North (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15379',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2438','projected_crs','EPSG','3419','EPSG','2200','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3420','NAD83 / Kansas South (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15380',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2439','projected_crs','EPSG','3420','EPSG','2201','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3421','NAD83 / Nevada East (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15381',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2440','projected_crs','EPSG','3421','EPSG','2224','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3422','NAD83 / Nevada Central (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15382',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2441','projected_crs','EPSG','3422','EPSG','2223','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3423','NAD83 / Nevada West (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15383',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2442','projected_crs','EPSG','3423','EPSG','2225','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3424','NAD83 / New Jersey (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15384',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2443','projected_crs','EPSG','3424','EPSG','1399','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3425','NAD83(HARN) / Iowa North (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15377',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2444','projected_crs','EPSG','3425','EPSG','2198','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3426','NAD83(HARN) / Iowa South (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15378',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2445','projected_crs','EPSG','3426','EPSG','2199','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3427','NAD83(HARN) / Kansas North (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15379',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2446','projected_crs','EPSG','3427','EPSG','2200','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3428','NAD83(HARN) / Kansas South (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15380',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2447','projected_crs','EPSG','3428','EPSG','2201','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3429','NAD83(HARN) / Nevada East (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15381',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2448','projected_crs','EPSG','3429','EPSG','2224','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3430','NAD83(HARN) / Nevada Central (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15382',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2449','projected_crs','EPSG','3430','EPSG','2223','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3431','NAD83(HARN) / Nevada West (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15383',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2450','projected_crs','EPSG','3431','EPSG','2225','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3432','NAD83(HARN) / New Jersey (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15384',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2451','projected_crs','EPSG','3432','EPSG','1399','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3433','NAD83 / Arkansas North (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15385',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2452','projected_crs','EPSG','3433','EPSG','2169','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3434','NAD83 / Arkansas South (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15386',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2453','projected_crs','EPSG','3434','EPSG','2170','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3435','NAD83 / Illinois East (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15387',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2454','projected_crs','EPSG','3435','EPSG','2194','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3436','NAD83 / Illinois West (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15388',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2455','projected_crs','EPSG','3436','EPSG','2195','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3437','NAD83 / New Hampshire (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15389',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2456','projected_crs','EPSG','3437','EPSG','1398','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3438','NAD83 / Rhode Island (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15390',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2457','projected_crs','EPSG','3438','EPSG','1408','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3439','PSD93 / UTM zone 39N',NULL,'EPSG','4400','EPSG','4134','EPSG','16039',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2458','projected_crs','EPSG','3439','EPSG','1544','EPSG','1136');
INSERT INTO "projected_crs" VALUES('EPSG','3440','PSD93 / UTM zone 40N',NULL,'EPSG','4400','EPSG','4134','EPSG','16040',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2459','projected_crs','EPSG','3440','EPSG','1545','EPSG','1136');
INSERT INTO "projected_crs" VALUES('EPSG','3441','NAD83(HARN) / Arkansas North (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15385',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2460','projected_crs','EPSG','3441','EPSG','2169','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3442','NAD83(HARN) / Arkansas South (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15386',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2461','projected_crs','EPSG','3442','EPSG','2170','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3443','NAD83(HARN) / Illinois East (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15387',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2462','projected_crs','EPSG','3443','EPSG','2194','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3444','NAD83(HARN) / Illinois West (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15388',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2463','projected_crs','EPSG','3444','EPSG','2195','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3445','NAD83(HARN) / New Hampshire (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15389',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2464','projected_crs','EPSG','3445','EPSG','1398','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3446','NAD83(HARN) / Rhode Island (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15390',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2465','projected_crs','EPSG','3446','EPSG','1408','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3447','ETRS89 / Belgian Lambert 2005',NULL,'EPSG','4499','EPSG','4258','EPSG','19862',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2466','projected_crs','EPSG','3447','EPSG','1347','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3448','JAD2001 / Jamaica Metric Grid',NULL,'EPSG','4400','EPSG','4758','EPSG','19860',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2467','projected_crs','EPSG','3448','EPSG','3342','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','3449','JAD2001 / UTM zone 17N',NULL,'EPSG','4400','EPSG','4758','EPSG','16017',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2468','projected_crs','EPSG','3449','EPSG','3478','EPSG','1201');
INSERT INTO "projected_crs" VALUES('EPSG','3450','JAD2001 / UTM zone 18N',NULL,'EPSG','4400','EPSG','4758','EPSG','16018',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2469','projected_crs','EPSG','3450','EPSG','3479','EPSG','1201');
INSERT INTO "projected_crs" VALUES('EPSG','3451','NAD83 / Louisiana North (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15391',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2470','projected_crs','EPSG','3451','EPSG','2204','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3452','NAD83 / Louisiana South (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15392',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2471','projected_crs','EPSG','3452','EPSG','2529','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3453','NAD83 / Louisiana Offshore (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15393',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2472','projected_crs','EPSG','3453','EPSG','4821','EPSG','1212');
INSERT INTO "projected_crs" VALUES('EPSG','3454','NAD83 / South Dakota North (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15395',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2473','projected_crs','EPSG','3454','EPSG','2249','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3455','NAD83 / South Dakota South (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15395',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2474','projected_crs','EPSG','3455','EPSG','2250','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3456','NAD83(HARN) / Louisiana North (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15391',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2475','projected_crs','EPSG','3456','EPSG','2204','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3457','NAD83(HARN) / Louisiana South (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15392',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2476','projected_crs','EPSG','3457','EPSG','2529','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3458','NAD83(HARN) / South Dakota North (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15394',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2477','projected_crs','EPSG','3458','EPSG','2249','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3459','NAD83(HARN) / South Dakota South (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15395',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2478','projected_crs','EPSG','3459','EPSG','2250','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3460','Fiji 1986 / Fiji Map Grid',NULL,'EPSG','4400','EPSG','4720','EPSG','19859',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2479','projected_crs','EPSG','3460','EPSG','1094','EPSG','1056');
INSERT INTO "projected_crs" VALUES('EPSG','3461','Dabola 1981 / UTM zone 28N',NULL,'EPSG','4400','EPSG','4155','EPSG','16028',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2480','projected_crs','EPSG','3461','EPSG','1468','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3462','Dabola 1981 / UTM zone 29N',NULL,'EPSG','4400','EPSG','4155','EPSG','16029',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2481','projected_crs','EPSG','3462','EPSG','1469','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3463','NAD83 / Maine CS2000 Central',NULL,'EPSG','4499','EPSG','4269','EPSG','11854',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2482','projected_crs','EPSG','3463','EPSG','2959','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3464','NAD83(HARN) / Maine CS2000 Central',NULL,'EPSG','4499','EPSG','4152','EPSG','11854',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2483','projected_crs','EPSG','3464','EPSG','2959','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3465','NAD83(NSRS2007) / Alabama East',NULL,'EPSG','4499','EPSG','4759','EPSG','10131',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2484','projected_crs','EPSG','3465','EPSG','2154','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3466','NAD83(NSRS2007) / Alabama West',NULL,'EPSG','4499','EPSG','4759','EPSG','10132',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2485','projected_crs','EPSG','3466','EPSG','2155','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3467','NAD83(NSRS2007) / Alaska Albers',NULL,'EPSG','4499','EPSG','4759','EPSG','15021',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2486','projected_crs','EPSG','3467','EPSG','1330','EPSG','1241');
INSERT INTO "projected_crs" VALUES('EPSG','3468','NAD83(NSRS2007) / Alaska zone 1',NULL,'EPSG','4499','EPSG','4759','EPSG','15031',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2487','projected_crs','EPSG','3468','EPSG','2156','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3469','NAD83(NSRS2007) / Alaska zone 2',NULL,'EPSG','4499','EPSG','4759','EPSG','15032',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2488','projected_crs','EPSG','3469','EPSG','2158','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3470','NAD83(NSRS2007) / Alaska zone 3',NULL,'EPSG','4499','EPSG','4759','EPSG','15033',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2489','projected_crs','EPSG','3470','EPSG','2159','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3471','NAD83(NSRS2007) / Alaska zone 4',NULL,'EPSG','4499','EPSG','4759','EPSG','15034',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2490','projected_crs','EPSG','3471','EPSG','2160','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3472','NAD83(NSRS2007) / Alaska zone 5',NULL,'EPSG','4499','EPSG','4759','EPSG','15035',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2491','projected_crs','EPSG','3472','EPSG','2161','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3473','NAD83(NSRS2007) / Alaska zone 6',NULL,'EPSG','4499','EPSG','4759','EPSG','15036',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2492','projected_crs','EPSG','3473','EPSG','2162','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3474','NAD83(NSRS2007) / Alaska zone 7',NULL,'EPSG','4499','EPSG','4759','EPSG','15037',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2493','projected_crs','EPSG','3474','EPSG','2163','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3475','NAD83(NSRS2007) / Alaska zone 8',NULL,'EPSG','4499','EPSG','4759','EPSG','15038',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2494','projected_crs','EPSG','3475','EPSG','2164','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3476','NAD83(NSRS2007) / Alaska zone 9',NULL,'EPSG','4499','EPSG','4759','EPSG','15039',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2495','projected_crs','EPSG','3476','EPSG','2165','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3477','NAD83(NSRS2007) / Alaska zone 10',NULL,'EPSG','4499','EPSG','4759','EPSG','15040',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2496','projected_crs','EPSG','3477','EPSG','2157','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3478','NAD83(NSRS2007) / Arizona Central',NULL,'EPSG','4499','EPSG','4759','EPSG','10232',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2497','projected_crs','EPSG','3478','EPSG','2166','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3479','NAD83(NSRS2007) / Arizona Central (ft)',NULL,'EPSG','4495','EPSG','4759','EPSG','15305',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2498','projected_crs','EPSG','3479','EPSG','2166','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3480','NAD83(NSRS2007) / Arizona East',NULL,'EPSG','4499','EPSG','4759','EPSG','10231',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2499','projected_crs','EPSG','3480','EPSG','2167','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3481','NAD83(NSRS2007) / Arizona East (ft)',NULL,'EPSG','4495','EPSG','4759','EPSG','15304',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2500','projected_crs','EPSG','3481','EPSG','2167','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3482','NAD83(NSRS2007) / Arizona West',NULL,'EPSG','4499','EPSG','4759','EPSG','10233',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2501','projected_crs','EPSG','3482','EPSG','2168','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3483','NAD83(NSRS2007) / Arizona West (ft)',NULL,'EPSG','4495','EPSG','4759','EPSG','15306',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2502','projected_crs','EPSG','3483','EPSG','2168','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3484','NAD83(NSRS2007) / Arkansas North',NULL,'EPSG','4499','EPSG','4759','EPSG','10331',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2503','projected_crs','EPSG','3484','EPSG','2169','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3485','NAD83(NSRS2007) / Arkansas North (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15385',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2504','projected_crs','EPSG','3485','EPSG','2169','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3486','NAD83(NSRS2007) / Arkansas South',NULL,'EPSG','4499','EPSG','4759','EPSG','10332',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2505','projected_crs','EPSG','3486','EPSG','2170','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3487','NAD83(NSRS2007) / Arkansas South (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15386',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2506','projected_crs','EPSG','3487','EPSG','2170','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3488','NAD83(NSRS2007) / California Albers',NULL,'EPSG','4499','EPSG','4759','EPSG','10420',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2507','projected_crs','EPSG','3488','EPSG','1375','EPSG','1135');
INSERT INTO "projected_crs" VALUES('EPSG','3489','NAD83(NSRS2007) / California zone 1',NULL,'EPSG','4499','EPSG','4759','EPSG','10431',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2508','projected_crs','EPSG','3489','EPSG','2175','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3490','NAD83(NSRS2007) / California zone 1 (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15307',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2509','projected_crs','EPSG','3490','EPSG','2175','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3491','NAD83(NSRS2007) / California zone 2',NULL,'EPSG','4499','EPSG','4759','EPSG','10432',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2510','projected_crs','EPSG','3491','EPSG','2176','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3492','NAD83(NSRS2007) / California zone 2 (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15308',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2511','projected_crs','EPSG','3492','EPSG','2176','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3493','NAD83(NSRS2007) / California zone 3',NULL,'EPSG','4499','EPSG','4759','EPSG','10433',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2512','projected_crs','EPSG','3493','EPSG','2177','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3494','NAD83(NSRS2007) / California zone 3 (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15309',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2513','projected_crs','EPSG','3494','EPSG','2177','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3495','NAD83(NSRS2007) / California zone 4',NULL,'EPSG','4499','EPSG','4759','EPSG','10434',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2514','projected_crs','EPSG','3495','EPSG','2178','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3496','NAD83(NSRS2007) / California zone 4 (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15310',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2515','projected_crs','EPSG','3496','EPSG','2178','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3497','NAD83(NSRS2007) / California zone 5',NULL,'EPSG','4499','EPSG','4759','EPSG','10435',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2516','projected_crs','EPSG','3497','EPSG','2182','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3498','NAD83(NSRS2007) / California zone 5 (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15311',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2517','projected_crs','EPSG','3498','EPSG','2182','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3499','NAD83(NSRS2007) / California zone 6',NULL,'EPSG','4499','EPSG','4759','EPSG','10436',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2518','projected_crs','EPSG','3499','EPSG','2180','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3500','NAD83(NSRS2007) / California zone 6 (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15312',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2519','projected_crs','EPSG','3500','EPSG','2180','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3501','NAD83(NSRS2007) / Colorado Central',NULL,'EPSG','4499','EPSG','4759','EPSG','10532',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2520','projected_crs','EPSG','3501','EPSG','2183','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3502','NAD83(NSRS2007) / Colorado Central (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15314',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2521','projected_crs','EPSG','3502','EPSG','2183','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3503','NAD83(NSRS2007) / Colorado North',NULL,'EPSG','4499','EPSG','4759','EPSG','10531',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2522','projected_crs','EPSG','3503','EPSG','2184','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3504','NAD83(NSRS2007) / Colorado North (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15313',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2523','projected_crs','EPSG','3504','EPSG','2184','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3505','NAD83(NSRS2007) / Colorado South',NULL,'EPSG','4499','EPSG','4759','EPSG','10533',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2524','projected_crs','EPSG','3505','EPSG','2185','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3506','NAD83(NSRS2007) / Colorado South (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15315',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2525','projected_crs','EPSG','3506','EPSG','2185','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3507','NAD83(NSRS2007) / Connecticut',NULL,'EPSG','4499','EPSG','4759','EPSG','10630',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2526','projected_crs','EPSG','3507','EPSG','1377','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3508','NAD83(NSRS2007) / Connecticut (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15316',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2527','projected_crs','EPSG','3508','EPSG','1377','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3509','NAD83(NSRS2007) / Delaware',NULL,'EPSG','4499','EPSG','4759','EPSG','10730',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2528','projected_crs','EPSG','3509','EPSG','1378','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3510','NAD83(NSRS2007) / Delaware (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15317',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2529','projected_crs','EPSG','3510','EPSG','1378','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3511','NAD83(NSRS2007) / Florida East',NULL,'EPSG','4499','EPSG','4759','EPSG','10931',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2530','projected_crs','EPSG','3511','EPSG','2186','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3512','NAD83(NSRS2007) / Florida East (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15318',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2531','projected_crs','EPSG','3512','EPSG','2186','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3513','NAD83(NSRS2007) / Florida GDL Albers',NULL,'EPSG','4499','EPSG','4759','EPSG','10934',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2532','projected_crs','EPSG','3513','EPSG','1379','EPSG','1135');
INSERT INTO "projected_crs" VALUES('EPSG','3514','NAD83(NSRS2007) / Florida North',NULL,'EPSG','4499','EPSG','4759','EPSG','10933',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2533','projected_crs','EPSG','3514','EPSG','2187','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3515','NAD83(NSRS2007) / Florida North (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15320',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2534','projected_crs','EPSG','3515','EPSG','2187','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3516','NAD83(NSRS2007) / Florida West',NULL,'EPSG','4499','EPSG','4759','EPSG','10932',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2535','projected_crs','EPSG','3516','EPSG','2188','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3517','NAD83(NSRS2007) / Florida West (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15319',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2536','projected_crs','EPSG','3517','EPSG','2188','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3518','NAD83(NSRS2007) / Georgia East',NULL,'EPSG','4499','EPSG','4759','EPSG','11031',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2537','projected_crs','EPSG','3518','EPSG','2189','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3519','NAD83(NSRS2007) / Georgia East (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15321',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2538','projected_crs','EPSG','3519','EPSG','2189','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3520','NAD83(NSRS2007) / Georgia West',NULL,'EPSG','4499','EPSG','4759','EPSG','11032',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2539','projected_crs','EPSG','3520','EPSG','2190','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3521','NAD83(NSRS2007) / Georgia West (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15322',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2540','projected_crs','EPSG','3521','EPSG','2190','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3522','NAD83(NSRS2007) / Idaho Central',NULL,'EPSG','4499','EPSG','4759','EPSG','11132',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2541','projected_crs','EPSG','3522','EPSG','2191','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3523','NAD83(NSRS2007) / Idaho Central (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15324',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2542','projected_crs','EPSG','3523','EPSG','2191','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3524','NAD83(NSRS2007) / Idaho East',NULL,'EPSG','4499','EPSG','4759','EPSG','11131',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2543','projected_crs','EPSG','3524','EPSG','2192','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3525','NAD83(NSRS2007) / Idaho East (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15323',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2544','projected_crs','EPSG','3525','EPSG','2192','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3526','NAD83(NSRS2007) / Idaho West',NULL,'EPSG','4499','EPSG','4759','EPSG','11133',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2545','projected_crs','EPSG','3526','EPSG','2193','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3527','NAD83(NSRS2007) / Idaho West (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15325',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2546','projected_crs','EPSG','3527','EPSG','2193','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3528','NAD83(NSRS2007) / Illinois East',NULL,'EPSG','4499','EPSG','4759','EPSG','11231',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2547','projected_crs','EPSG','3528','EPSG','2194','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3529','NAD83(NSRS2007) / Illinois East (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15387',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2548','projected_crs','EPSG','3529','EPSG','2194','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3530','NAD83(NSRS2007) / Illinois West',NULL,'EPSG','4499','EPSG','4759','EPSG','11232',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2549','projected_crs','EPSG','3530','EPSG','2195','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3531','NAD83(NSRS2007) / Illinois West (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15388',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2550','projected_crs','EPSG','3531','EPSG','2195','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3532','NAD83(NSRS2007) / Indiana East',NULL,'EPSG','4499','EPSG','4759','EPSG','11331',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2551','projected_crs','EPSG','3532','EPSG','2196','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3533','NAD83(NSRS2007) / Indiana East (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15372',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2552','projected_crs','EPSG','3533','EPSG','2196','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3534','NAD83(NSRS2007) / Indiana West',NULL,'EPSG','4499','EPSG','4759','EPSG','11332',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2553','projected_crs','EPSG','3534','EPSG','2197','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3535','NAD83(NSRS2007) / Indiana West (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15373',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2554','projected_crs','EPSG','3535','EPSG','2197','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3536','NAD83(NSRS2007) / Iowa North',NULL,'EPSG','4499','EPSG','4759','EPSG','11431',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2555','projected_crs','EPSG','3536','EPSG','2198','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3537','NAD83(NSRS2007) / Iowa North (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15377',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2556','projected_crs','EPSG','3537','EPSG','2198','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3538','NAD83(NSRS2007) / Iowa South',NULL,'EPSG','4499','EPSG','4759','EPSG','11432',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2557','projected_crs','EPSG','3538','EPSG','2199','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3539','NAD83(NSRS2007) / Iowa South (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15378',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2558','projected_crs','EPSG','3539','EPSG','2199','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3540','NAD83(NSRS2007) / Kansas North',NULL,'EPSG','4499','EPSG','4759','EPSG','11531',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2559','projected_crs','EPSG','3540','EPSG','2200','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3541','NAD83(NSRS2007) / Kansas North (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15379',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2560','projected_crs','EPSG','3541','EPSG','2200','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3542','NAD83(NSRS2007) / Kansas South',NULL,'EPSG','4499','EPSG','4759','EPSG','11532',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2561','projected_crs','EPSG','3542','EPSG','2201','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3543','NAD83(NSRS2007) / Kansas South (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15380',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2562','projected_crs','EPSG','3543','EPSG','2201','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3544','NAD83(NSRS2007) / Kentucky North',NULL,'EPSG','4499','EPSG','4759','EPSG','15303',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2563','projected_crs','EPSG','3544','EPSG','2202','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3545','NAD83(NSRS2007) / Kentucky North (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15328',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2564','projected_crs','EPSG','3545','EPSG','2202','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3546','NAD83(NSRS2007) / Kentucky Single Zone',NULL,'EPSG','4499','EPSG','4759','EPSG','11630',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2565','projected_crs','EPSG','3546','EPSG','1386','EPSG','1135');
INSERT INTO "projected_crs" VALUES('EPSG','3547','NAD83(NSRS2007) / Kentucky Single Zone (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15375',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2566','projected_crs','EPSG','3547','EPSG','1386','EPSG','1135');
INSERT INTO "projected_crs" VALUES('EPSG','3548','NAD83(NSRS2007) / Kentucky South',NULL,'EPSG','4499','EPSG','4759','EPSG','11632',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2567','projected_crs','EPSG','3548','EPSG','2203','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3549','NAD83(NSRS2007) / Kentucky South (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15329',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2568','projected_crs','EPSG','3549','EPSG','2203','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3550','NAD83(NSRS2007) / Louisiana North',NULL,'EPSG','4499','EPSG','4759','EPSG','11731',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2569','projected_crs','EPSG','3550','EPSG','2204','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3551','NAD83(NSRS2007) / Louisiana North (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15391',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2570','projected_crs','EPSG','3551','EPSG','2204','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3552','NAD83(NSRS2007) / Louisiana South',NULL,'EPSG','4499','EPSG','4759','EPSG','11732',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2571','projected_crs','EPSG','3552','EPSG','2529','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3553','NAD83(NSRS2007) / Louisiana South (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15392',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2572','projected_crs','EPSG','3553','EPSG','2529','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3554','NAD83(NSRS2007) / Maine CS2000 Central',NULL,'EPSG','4499','EPSG','4759','EPSG','11854',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2573','projected_crs','EPSG','3554','EPSG','2959','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3555','NAD83(NSRS2007) / Maine CS2000 East',NULL,'EPSG','4499','EPSG','4759','EPSG','11851',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2574','projected_crs','EPSG','3555','EPSG','2960','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3556','NAD83(NSRS2007) / Maine CS2000 West',NULL,'EPSG','4499','EPSG','4759','EPSG','11853',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2575','projected_crs','EPSG','3556','EPSG','2958','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3557','NAD83(NSRS2007) / Maine East',NULL,'EPSG','4499','EPSG','4759','EPSG','11831',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2576','projected_crs','EPSG','3557','EPSG','2206','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3558','NAD83(NSRS2007) / Maine West',NULL,'EPSG','4499','EPSG','4759','EPSG','11832',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2577','projected_crs','EPSG','3558','EPSG','2207','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3559','NAD83(NSRS2007) / Maryland',NULL,'EPSG','4499','EPSG','4759','EPSG','11930',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2578','projected_crs','EPSG','3559','EPSG','1389','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3560','NAD83 / Utah North (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15297',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2579','projected_crs','EPSG','3560','EPSG','2258','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3561','Old Hawaiian / Hawaii zone 1',NULL,'EPSG','4497','EPSG','4135','EPSG','15101',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2580','projected_crs','EPSG','3561','EPSG','1546','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3562','Old Hawaiian / Hawaii zone 2',NULL,'EPSG','4497','EPSG','4135','EPSG','15102',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2581','projected_crs','EPSG','3562','EPSG','1547','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3563','Old Hawaiian / Hawaii zone 3',NULL,'EPSG','4497','EPSG','4135','EPSG','15103',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2582','projected_crs','EPSG','3563','EPSG','1548','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3564','Old Hawaiian / Hawaii zone 4',NULL,'EPSG','4497','EPSG','4135','EPSG','15104',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2583','projected_crs','EPSG','3564','EPSG','1549','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3565','Old Hawaiian / Hawaii zone 5',NULL,'EPSG','4497','EPSG','4135','EPSG','15105',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2584','projected_crs','EPSG','3565','EPSG','1550','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3566','NAD83 / Utah Central (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15298',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2585','projected_crs','EPSG','3566','EPSG','2257','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3567','NAD83 / Utah South (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15299',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2586','projected_crs','EPSG','3567','EPSG','2259','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3568','NAD83(HARN) / Utah North (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15297',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2587','projected_crs','EPSG','3568','EPSG','2258','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3569','NAD83(HARN) / Utah Central (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15298',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2588','projected_crs','EPSG','3569','EPSG','2257','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3570','NAD83(HARN) / Utah South (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15299',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2589','projected_crs','EPSG','3570','EPSG','2259','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3571','WGS 84 / North Pole LAEA Bering Sea',NULL,'EPSG','4464','EPSG','4326','EPSG','17295',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2590','projected_crs','EPSG','3571','EPSG','3480','EPSG','1040');
INSERT INTO "projected_crs" VALUES('EPSG','3572','WGS 84 / North Pole LAEA Alaska',NULL,'EPSG','4467','EPSG','4326','EPSG','17296',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2591','projected_crs','EPSG','3572','EPSG','3480','EPSG','1040');
INSERT INTO "projected_crs" VALUES('EPSG','3573','WGS 84 / North Pole LAEA Canada',NULL,'EPSG','4466','EPSG','4326','EPSG','17297',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2592','projected_crs','EPSG','3573','EPSG','3480','EPSG','1040');
INSERT INTO "projected_crs" VALUES('EPSG','3574','WGS 84 / North Pole LAEA Atlantic',NULL,'EPSG','4465','EPSG','4326','EPSG','17298',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2593','projected_crs','EPSG','3574','EPSG','3480','EPSG','1040');
INSERT INTO "projected_crs" VALUES('EPSG','3575','WGS 84 / North Pole LAEA Europe',NULL,'EPSG','4463','EPSG','4326','EPSG','17299',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2594','projected_crs','EPSG','3575','EPSG','3480','EPSG','1040');
INSERT INTO "projected_crs" VALUES('EPSG','3576','WGS 84 / North Pole LAEA Russia',NULL,'EPSG','1035','EPSG','4326','EPSG','17300',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2595','projected_crs','EPSG','3576','EPSG','3480','EPSG','1040');
INSERT INTO "projected_crs" VALUES('EPSG','3577','GDA94 / Australian Albers',NULL,'EPSG','4400','EPSG','4283','EPSG','17365',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2596','projected_crs','EPSG','3577','EPSG','2575','EPSG','1162');
INSERT INTO "projected_crs" VALUES('EPSG','3578','NAD83 / Yukon Albers',NULL,'EPSG','4400','EPSG','4269','EPSG','19858',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2597','projected_crs','EPSG','3578','EPSG','2417','EPSG','1238');
INSERT INTO "projected_crs" VALUES('EPSG','3579','NAD83(CSRS) / Yukon Albers',NULL,'EPSG','4400','EPSG','4617','EPSG','19858',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2598','projected_crs','EPSG','3579','EPSG','2417','EPSG','1238');
INSERT INTO "projected_crs" VALUES('EPSG','3580','NAD83 / NWT Lambert',NULL,'EPSG','4400','EPSG','4269','EPSG','19857',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2599','projected_crs','EPSG','3580','EPSG','3481','EPSG','1238');
INSERT INTO "projected_crs" VALUES('EPSG','3581','NAD83(CSRS) / NWT Lambert',NULL,'EPSG','4400','EPSG','4617','EPSG','19857',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2600','projected_crs','EPSG','3581','EPSG','3481','EPSG','1238');
INSERT INTO "projected_crs" VALUES('EPSG','3582','NAD83(NSRS2007) / Maryland (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15330',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2601','projected_crs','EPSG','3582','EPSG','1389','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3583','NAD83(NSRS2007) / Massachusetts Island',NULL,'EPSG','4499','EPSG','4759','EPSG','12032',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2602','projected_crs','EPSG','3583','EPSG','2208','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3584','NAD83(NSRS2007) / Massachusetts Island (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15332',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2603','projected_crs','EPSG','3584','EPSG','2208','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3585','NAD83(NSRS2007) / Massachusetts Mainland',NULL,'EPSG','4499','EPSG','4759','EPSG','12031',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2604','projected_crs','EPSG','3585','EPSG','2209','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3586','NAD83(NSRS2007) / Massachusetts Mainland (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15331',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2605','projected_crs','EPSG','3586','EPSG','2209','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3587','NAD83(NSRS2007) / Michigan Central',NULL,'EPSG','4499','EPSG','4759','EPSG','12142',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2606','projected_crs','EPSG','3587','EPSG','1724','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3588','NAD83(NSRS2007) / Michigan Central (ft)',NULL,'EPSG','4495','EPSG','4759','EPSG','15334',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2607','projected_crs','EPSG','3588','EPSG','1724','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3589','NAD83(NSRS2007) / Michigan North',NULL,'EPSG','4499','EPSG','4759','EPSG','12141',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2608','projected_crs','EPSG','3589','EPSG','1723','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3590','NAD83(NSRS2007) / Michigan North (ft)',NULL,'EPSG','4495','EPSG','4759','EPSG','15333',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2609','projected_crs','EPSG','3590','EPSG','1723','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3591','NAD83(NSRS2007) / Michigan Oblique Mercator',NULL,'EPSG','4499','EPSG','4759','EPSG','12150',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2610','projected_crs','EPSG','3591','EPSG','1391','EPSG','1135');
INSERT INTO "projected_crs" VALUES('EPSG','3592','NAD83(NSRS2007) / Michigan South',NULL,'EPSG','4499','EPSG','4759','EPSG','12143',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2611','projected_crs','EPSG','3592','EPSG','1725','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3593','NAD83(NSRS2007) / Michigan South (ft)',NULL,'EPSG','4495','EPSG','4759','EPSG','15335',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2612','projected_crs','EPSG','3593','EPSG','1725','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3594','NAD83(NSRS2007) / Minnesota Central',NULL,'EPSG','4499','EPSG','4759','EPSG','12232',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2613','projected_crs','EPSG','3594','EPSG','2213','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3595','NAD83(NSRS2007) / Minnesota North',NULL,'EPSG','4499','EPSG','4759','EPSG','12231',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2614','projected_crs','EPSG','3595','EPSG','2214','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3596','NAD83(NSRS2007) / Minnesota South',NULL,'EPSG','4499','EPSG','4759','EPSG','12233',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2615','projected_crs','EPSG','3596','EPSG','2215','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3597','NAD83(NSRS2007) / Mississippi East',NULL,'EPSG','4499','EPSG','4759','EPSG','12331',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2616','projected_crs','EPSG','3597','EPSG','2216','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3598','NAD83(NSRS2007) / Mississippi East (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15336',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2617','projected_crs','EPSG','3598','EPSG','2216','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3599','NAD83(NSRS2007) / Mississippi West',NULL,'EPSG','4499','EPSG','4759','EPSG','12332',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2618','projected_crs','EPSG','3599','EPSG','2217','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3600','NAD83(NSRS2007) / Mississippi West (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15337',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2619','projected_crs','EPSG','3600','EPSG','2217','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3601','NAD83(NSRS2007) / Missouri Central',NULL,'EPSG','4499','EPSG','4759','EPSG','12432',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2620','projected_crs','EPSG','3601','EPSG','2218','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3602','NAD83(NSRS2007) / Missouri East',NULL,'EPSG','4499','EPSG','4759','EPSG','12431',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2621','projected_crs','EPSG','3602','EPSG','2219','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3603','NAD83(NSRS2007) / Missouri West',NULL,'EPSG','4499','EPSG','4759','EPSG','12433',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2622','projected_crs','EPSG','3603','EPSG','2220','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3604','NAD83(NSRS2007) / Montana',NULL,'EPSG','4499','EPSG','4759','EPSG','12530',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2623','projected_crs','EPSG','3604','EPSG','1395','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3605','NAD83(NSRS2007) / Montana (ft)',NULL,'EPSG','4495','EPSG','4759','EPSG','15338',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2624','projected_crs','EPSG','3605','EPSG','1395','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3606','NAD83(NSRS2007) / Nebraska',NULL,'EPSG','4499','EPSG','4759','EPSG','12630',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2625','projected_crs','EPSG','3606','EPSG','1396','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3607','NAD83(NSRS2007) / Nevada Central',NULL,'EPSG','4499','EPSG','4759','EPSG','12732',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2626','projected_crs','EPSG','3607','EPSG','2223','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3608','NAD83(NSRS2007) / Nevada Central (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15382',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2627','projected_crs','EPSG','3608','EPSG','2223','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3609','NAD83(NSRS2007) / Nevada East',NULL,'EPSG','4499','EPSG','4759','EPSG','12731',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2628','projected_crs','EPSG','3609','EPSG','2224','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3610','NAD83(NSRS2007) / Nevada East (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15381',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2629','projected_crs','EPSG','3610','EPSG','2224','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3611','NAD83(NSRS2007) / Nevada West',NULL,'EPSG','4499','EPSG','4759','EPSG','12733',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2630','projected_crs','EPSG','3611','EPSG','2225','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3612','NAD83(NSRS2007) / Nevada West (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15383',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2631','projected_crs','EPSG','3612','EPSG','2225','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3613','NAD83(NSRS2007) / New Hampshire',NULL,'EPSG','4499','EPSG','4759','EPSG','12830',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2632','projected_crs','EPSG','3613','EPSG','1398','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3614','NAD83(NSRS2007) / New Hampshire (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15389',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2633','projected_crs','EPSG','3614','EPSG','1398','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3615','NAD83(NSRS2007) / New Jersey',NULL,'EPSG','4499','EPSG','4759','EPSG','12930',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2634','projected_crs','EPSG','3615','EPSG','1399','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3616','NAD83(NSRS2007) / New Jersey (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15384',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2635','projected_crs','EPSG','3616','EPSG','1399','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3617','NAD83(NSRS2007) / New Mexico Central',NULL,'EPSG','4499','EPSG','4759','EPSG','13032',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2636','projected_crs','EPSG','3617','EPSG','2231','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3618','NAD83(NSRS2007) / New Mexico Central (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15340',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2637','projected_crs','EPSG','3618','EPSG','2231','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3619','NAD83(NSRS2007) / New Mexico East',NULL,'EPSG','4499','EPSG','4759','EPSG','13031',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2638','projected_crs','EPSG','3619','EPSG','2228','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3620','NAD83(NSRS2007) / New Mexico East (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15339',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2639','projected_crs','EPSG','3620','EPSG','2228','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3621','NAD83(NSRS2007) / New Mexico West',NULL,'EPSG','4499','EPSG','4759','EPSG','13033',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2640','projected_crs','EPSG','3621','EPSG','2232','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3622','NAD83(NSRS2007) / New Mexico West (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15341',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2641','projected_crs','EPSG','3622','EPSG','2232','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3623','NAD83(NSRS2007) / New York Central',NULL,'EPSG','4499','EPSG','4759','EPSG','13132',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2642','projected_crs','EPSG','3623','EPSG','2233','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3624','NAD83(NSRS2007) / New York Central (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15343',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2643','projected_crs','EPSG','3624','EPSG','2233','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3625','NAD83(NSRS2007) / New York East',NULL,'EPSG','4499','EPSG','4759','EPSG','13131',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2644','projected_crs','EPSG','3625','EPSG','2234','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3626','NAD83(NSRS2007) / New York East (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15342',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2645','projected_crs','EPSG','3626','EPSG','2234','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3627','NAD83(NSRS2007) / New York Long Island',NULL,'EPSG','4499','EPSG','4759','EPSG','13134',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2646','projected_crs','EPSG','3627','EPSG','2235','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3628','NAD83(NSRS2007) / New York Long Island (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15345',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2647','projected_crs','EPSG','3628','EPSG','2235','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3629','NAD83(NSRS2007) / New York West',NULL,'EPSG','4499','EPSG','4759','EPSG','13133',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2648','projected_crs','EPSG','3629','EPSG','2236','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3630','NAD83(NSRS2007) / New York West (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15344',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2649','projected_crs','EPSG','3630','EPSG','2236','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3631','NAD83(NSRS2007) / North Carolina',NULL,'EPSG','4499','EPSG','4759','EPSG','13230',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2650','projected_crs','EPSG','3631','EPSG','1402','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3632','NAD83(NSRS2007) / North Carolina (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15346',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2651','projected_crs','EPSG','3632','EPSG','1402','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3633','NAD83(NSRS2007) / North Dakota North',NULL,'EPSG','4499','EPSG','4759','EPSG','13331',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2652','projected_crs','EPSG','3633','EPSG','2237','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3634','NAD83(NSRS2007) / North Dakota North (ft)',NULL,'EPSG','4495','EPSG','4759','EPSG','15347',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2653','projected_crs','EPSG','3634','EPSG','2237','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3635','NAD83(NSRS2007) / North Dakota South',NULL,'EPSG','4499','EPSG','4759','EPSG','13332',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2654','projected_crs','EPSG','3635','EPSG','2238','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3636','NAD83(NSRS2007) / North Dakota South (ft)',NULL,'EPSG','4495','EPSG','4759','EPSG','15348',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2655','projected_crs','EPSG','3636','EPSG','2238','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3637','NAD83(NSRS2007) / Ohio North',NULL,'EPSG','4499','EPSG','4759','EPSG','13431',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2656','projected_crs','EPSG','3637','EPSG','2239','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3638','NAD83(NSRS2007) / Ohio South',NULL,'EPSG','4499','EPSG','4759','EPSG','13432',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2657','projected_crs','EPSG','3638','EPSG','2240','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3639','NAD83(NSRS2007) / Oklahoma North',NULL,'EPSG','4499','EPSG','4759','EPSG','13531',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2658','projected_crs','EPSG','3639','EPSG','2241','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3640','NAD83(NSRS2007) / Oklahoma North (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15349',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2659','projected_crs','EPSG','3640','EPSG','2241','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3641','NAD83(NSRS2007) / Oklahoma South',NULL,'EPSG','4499','EPSG','4759','EPSG','13532',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2660','projected_crs','EPSG','3641','EPSG','2242','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3642','NAD83(NSRS2007) / Oklahoma South (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15350',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2661','projected_crs','EPSG','3642','EPSG','2242','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3643','NAD83(NSRS2007) / Oregon LCC (m)',NULL,'EPSG','4499','EPSG','4759','EPSG','13633',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2662','projected_crs','EPSG','3643','EPSG','1406','EPSG','1212');
INSERT INTO "projected_crs" VALUES('EPSG','3644','NAD83(NSRS2007) / Oregon GIC Lambert (ft)',NULL,'EPSG','4495','EPSG','4759','EPSG','15374',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2663','projected_crs','EPSG','3644','EPSG','1406','EPSG','1135');
INSERT INTO "projected_crs" VALUES('EPSG','3645','NAD83(NSRS2007) / Oregon North',NULL,'EPSG','4499','EPSG','4759','EPSG','13631',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2664','projected_crs','EPSG','3645','EPSG','2243','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3646','NAD83(NSRS2007) / Oregon North (ft)',NULL,'EPSG','4495','EPSG','4759','EPSG','15351',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2665','projected_crs','EPSG','3646','EPSG','2243','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3647','NAD83(NSRS2007) / Oregon South',NULL,'EPSG','4499','EPSG','4759','EPSG','13632',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2666','projected_crs','EPSG','3647','EPSG','2244','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3648','NAD83(NSRS2007) / Oregon South (ft)',NULL,'EPSG','4495','EPSG','4759','EPSG','15352',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2667','projected_crs','EPSG','3648','EPSG','2244','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3649','NAD83(NSRS2007) / Pennsylvania North',NULL,'EPSG','4499','EPSG','4759','EPSG','13731',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2668','projected_crs','EPSG','3649','EPSG','2245','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3650','NAD83(NSRS2007) / Pennsylvania North (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15353',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2669','projected_crs','EPSG','3650','EPSG','2245','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3651','NAD83(NSRS2007) / Pennsylvania South',NULL,'EPSG','4499','EPSG','4759','EPSG','13732',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2670','projected_crs','EPSG','3651','EPSG','2246','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3652','NAD83(NSRS2007) / Pennsylvania South (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15354',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2671','projected_crs','EPSG','3652','EPSG','2246','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3653','NAD83(NSRS2007) / Rhode Island',NULL,'EPSG','4499','EPSG','4759','EPSG','13830',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2672','projected_crs','EPSG','3653','EPSG','1408','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3654','NAD83(NSRS2007) / Rhode Island (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15390',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2673','projected_crs','EPSG','3654','EPSG','1408','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3655','NAD83(NSRS2007) / South Carolina',NULL,'EPSG','4499','EPSG','4759','EPSG','13930',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2674','projected_crs','EPSG','3655','EPSG','1409','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3656','NAD83(NSRS2007) / South Carolina (ft)',NULL,'EPSG','4495','EPSG','4759','EPSG','15355',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2675','projected_crs','EPSG','3656','EPSG','1409','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3657','NAD83(NSRS2007) / South Dakota North',NULL,'EPSG','4499','EPSG','4759','EPSG','14031',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2676','projected_crs','EPSG','3657','EPSG','2249','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3658','NAD83(NSRS2007) / South Dakota North (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15394',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2677','projected_crs','EPSG','3658','EPSG','2249','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3659','NAD83(NSRS2007) / South Dakota South',NULL,'EPSG','4499','EPSG','4759','EPSG','14032',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2678','projected_crs','EPSG','3659','EPSG','2250','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3660','NAD83(NSRS2007) / South Dakota South (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15395',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2679','projected_crs','EPSG','3660','EPSG','2250','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3661','NAD83(NSRS2007) / Tennessee',NULL,'EPSG','4499','EPSG','4759','EPSG','14130',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2680','projected_crs','EPSG','3661','EPSG','1411','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3662','NAD83(NSRS2007) / Tennessee (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15356',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2681','projected_crs','EPSG','3662','EPSG','1411','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3663','NAD83(NSRS2007) / Texas Central',NULL,'EPSG','4499','EPSG','4759','EPSG','14233',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2682','projected_crs','EPSG','3663','EPSG','2252','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3664','NAD83(NSRS2007) / Texas Central (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15359',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2683','projected_crs','EPSG','3664','EPSG','2252','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3665','NAD83(NSRS2007) / Texas Centric Albers Equal Area',NULL,'EPSG','4499','EPSG','4759','EPSG','14254',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2684','projected_crs','EPSG','3665','EPSG','1412','EPSG','1222');
INSERT INTO "projected_crs" VALUES('EPSG','3666','NAD83(NSRS2007) / Texas Centric Lambert Conformal',NULL,'EPSG','4499','EPSG','4759','EPSG','14253',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2685','projected_crs','EPSG','3666','EPSG','1412','EPSG','1221');
INSERT INTO "projected_crs" VALUES('EPSG','3667','NAD83(NSRS2007) / Texas North',NULL,'EPSG','4499','EPSG','4759','EPSG','14231',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2686','projected_crs','EPSG','3667','EPSG','2253','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3668','NAD83(NSRS2007) / Texas North (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15357',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2687','projected_crs','EPSG','3668','EPSG','2253','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3669','NAD83(NSRS2007) / Texas North Central',NULL,'EPSG','4499','EPSG','4759','EPSG','14232',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2688','projected_crs','EPSG','3669','EPSG','2254','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3670','NAD83(NSRS2007) / Texas North Central (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15358',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2689','projected_crs','EPSG','3670','EPSG','2254','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3671','NAD83(NSRS2007) / Texas South',NULL,'EPSG','4499','EPSG','4759','EPSG','14235',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2690','projected_crs','EPSG','3671','EPSG','2528','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3672','NAD83(NSRS2007) / Texas South (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15361',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2691','projected_crs','EPSG','3672','EPSG','2528','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3673','NAD83(NSRS2007) / Texas South Central',NULL,'EPSG','4499','EPSG','4759','EPSG','14234',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2692','projected_crs','EPSG','3673','EPSG','2527','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3674','NAD83(NSRS2007) / Texas South Central (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15360',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2693','projected_crs','EPSG','3674','EPSG','2527','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3675','NAD83(NSRS2007) / Utah Central',NULL,'EPSG','4499','EPSG','4759','EPSG','14332',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2694','projected_crs','EPSG','3675','EPSG','2257','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3676','NAD83(NSRS2007) / Utah Central (ft)',NULL,'EPSG','4495','EPSG','4759','EPSG','15363',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2695','projected_crs','EPSG','3676','EPSG','2257','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3677','NAD83(NSRS2007) / Utah Central (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15298',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2696','projected_crs','EPSG','3677','EPSG','2257','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3678','NAD83(NSRS2007) / Utah North',NULL,'EPSG','4499','EPSG','4759','EPSG','14331',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2697','projected_crs','EPSG','3678','EPSG','2258','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3679','NAD83(NSRS2007) / Utah North (ft)',NULL,'EPSG','4495','EPSG','4759','EPSG','15362',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2698','projected_crs','EPSG','3679','EPSG','2258','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3680','NAD83(NSRS2007) / Utah North (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15297',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2699','projected_crs','EPSG','3680','EPSG','2258','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3681','NAD83(NSRS2007) / Utah South',NULL,'EPSG','4499','EPSG','4759','EPSG','14333',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2700','projected_crs','EPSG','3681','EPSG','2259','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3682','NAD83(NSRS2007) / Utah South (ft)',NULL,'EPSG','4495','EPSG','4759','EPSG','15364',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2701','projected_crs','EPSG','3682','EPSG','2259','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3683','NAD83(NSRS2007) / Utah South (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15299',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2702','projected_crs','EPSG','3683','EPSG','2259','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3684','NAD83(NSRS2007) / Vermont',NULL,'EPSG','4499','EPSG','4759','EPSG','14430',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2703','projected_crs','EPSG','3684','EPSG','1414','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3685','NAD83(NSRS2007) / Virginia North',NULL,'EPSG','4499','EPSG','4759','EPSG','14531',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2704','projected_crs','EPSG','3685','EPSG','2260','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3686','NAD83(NSRS2007) / Virginia North (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15365',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2705','projected_crs','EPSG','3686','EPSG','2260','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3687','NAD83(NSRS2007) / Virginia South',NULL,'EPSG','4499','EPSG','4759','EPSG','14532',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2706','projected_crs','EPSG','3687','EPSG','2261','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3688','NAD83(NSRS2007) / Virginia South (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15366',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2707','projected_crs','EPSG','3688','EPSG','2261','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3689','NAD83(NSRS2007) / Washington North',NULL,'EPSG','4499','EPSG','4759','EPSG','14631',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2708','projected_crs','EPSG','3689','EPSG','2273','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3690','NAD83(NSRS2007) / Washington North (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15367',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2709','projected_crs','EPSG','3690','EPSG','2273','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3691','NAD83(NSRS2007) / Washington South',NULL,'EPSG','4499','EPSG','4759','EPSG','14632',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2710','projected_crs','EPSG','3691','EPSG','2274','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3692','NAD83(NSRS2007) / Washington South (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15368',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2711','projected_crs','EPSG','3692','EPSG','2274','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3693','NAD83(NSRS2007) / West Virginia North',NULL,'EPSG','4499','EPSG','4759','EPSG','14731',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2712','projected_crs','EPSG','3693','EPSG','2264','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3694','NAD83(NSRS2007) / West Virginia South',NULL,'EPSG','4499','EPSG','4759','EPSG','14732',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2713','projected_crs','EPSG','3694','EPSG','2265','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3695','NAD83(NSRS2007) / Wisconsin Central',NULL,'EPSG','4499','EPSG','4759','EPSG','14832',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2714','projected_crs','EPSG','3695','EPSG','2266','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3696','NAD83(NSRS2007) / Wisconsin Central (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15370',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2715','projected_crs','EPSG','3696','EPSG','2266','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3697','NAD83(NSRS2007) / Wisconsin North',NULL,'EPSG','4499','EPSG','4759','EPSG','14831',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2716','projected_crs','EPSG','3697','EPSG','2267','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3698','NAD83(NSRS2007) / Wisconsin North (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15369',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2717','projected_crs','EPSG','3698','EPSG','2267','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3699','NAD83(NSRS2007) / Wisconsin South',NULL,'EPSG','4499','EPSG','4759','EPSG','14833',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2718','projected_crs','EPSG','3699','EPSG','2268','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3700','NAD83(NSRS2007) / Wisconsin South (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15371',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2719','projected_crs','EPSG','3700','EPSG','2268','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3701','NAD83(NSRS2007) / Wisconsin Transverse Mercator',NULL,'EPSG','4499','EPSG','4759','EPSG','14841',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2720','projected_crs','EPSG','3701','EPSG','1418','EPSG','1135');
INSERT INTO "projected_crs" VALUES('EPSG','3702','NAD83(NSRS2007) / Wyoming East',NULL,'EPSG','4499','EPSG','4759','EPSG','14931',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2721','projected_crs','EPSG','3702','EPSG','2269','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3703','NAD83(NSRS2007) / Wyoming East Central',NULL,'EPSG','4499','EPSG','4759','EPSG','14932',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2722','projected_crs','EPSG','3703','EPSG','2270','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3704','NAD83(NSRS2007) / Wyoming West Central',NULL,'EPSG','4499','EPSG','4759','EPSG','14933',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2723','projected_crs','EPSG','3704','EPSG','2272','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3705','NAD83(NSRS2007) / Wyoming West',NULL,'EPSG','4499','EPSG','4759','EPSG','14934',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2724','projected_crs','EPSG','3705','EPSG','2271','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3706','NAD83(NSRS2007) / UTM zone 59N',NULL,'EPSG','4400','EPSG','4759','EPSG','16059',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2725','projected_crs','EPSG','3706','EPSG','3372','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3707','NAD83(NSRS2007) / UTM zone 60N',NULL,'EPSG','4400','EPSG','4759','EPSG','16060',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2726','projected_crs','EPSG','3707','EPSG','3373','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3708','NAD83(NSRS2007) / UTM zone 1N',NULL,'EPSG','4400','EPSG','4759','EPSG','16001',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2727','projected_crs','EPSG','3708','EPSG','3374','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3709','NAD83(NSRS2007) / UTM zone 2N',NULL,'EPSG','4400','EPSG','4759','EPSG','16002',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2728','projected_crs','EPSG','3709','EPSG','3375','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3710','NAD83(NSRS2007) / UTM zone 3N',NULL,'EPSG','4400','EPSG','4759','EPSG','16003',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2729','projected_crs','EPSG','3710','EPSG','2133','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3711','NAD83(NSRS2007) / UTM zone 4N',NULL,'EPSG','4400','EPSG','4759','EPSG','16004',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2730','projected_crs','EPSG','3711','EPSG','2134','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3712','NAD83(NSRS2007) / UTM zone 5N',NULL,'EPSG','4400','EPSG','4759','EPSG','16005',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2731','projected_crs','EPSG','3712','EPSG','2135','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3713','NAD83(NSRS2007) / UTM zone 6N',NULL,'EPSG','4400','EPSG','4759','EPSG','16006',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2732','projected_crs','EPSG','3713','EPSG','2136','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3714','NAD83(NSRS2007) / UTM zone 7N',NULL,'EPSG','4400','EPSG','4759','EPSG','16007',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2733','projected_crs','EPSG','3714','EPSG','3494','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3715','NAD83(NSRS2007) / UTM zone 8N',NULL,'EPSG','4400','EPSG','4759','EPSG','16008',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2734','projected_crs','EPSG','3715','EPSG','3495','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3716','NAD83(NSRS2007) / UTM zone 9N',NULL,'EPSG','4400','EPSG','4759','EPSG','16009',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2735','projected_crs','EPSG','3716','EPSG','3496','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3717','NAD83(NSRS2007) / UTM zone 10N',NULL,'EPSG','4400','EPSG','4759','EPSG','16010',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2736','projected_crs','EPSG','3717','EPSG','3497','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3718','NAD83(NSRS2007) / UTM zone 11N',NULL,'EPSG','4400','EPSG','4759','EPSG','16011',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2737','projected_crs','EPSG','3718','EPSG','3498','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3719','NAD83(NSRS2007) / UTM zone 12N',NULL,'EPSG','4400','EPSG','4759','EPSG','16012',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2738','projected_crs','EPSG','3719','EPSG','3499','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3720','NAD83(NSRS2007) / UTM zone 13N',NULL,'EPSG','4400','EPSG','4759','EPSG','16013',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2739','projected_crs','EPSG','3720','EPSG','3500','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3721','NAD83(NSRS2007) / UTM zone 14N',NULL,'EPSG','4400','EPSG','4759','EPSG','16014',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2740','projected_crs','EPSG','3721','EPSG','3501','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3722','NAD83(NSRS2007) / UTM zone 15N',NULL,'EPSG','4400','EPSG','4759','EPSG','16015',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2741','projected_crs','EPSG','3722','EPSG','3502','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3723','NAD83(NSRS2007) / UTM zone 16N',NULL,'EPSG','4400','EPSG','4759','EPSG','16016',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2742','projected_crs','EPSG','3723','EPSG','3503','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3724','NAD83(NSRS2007) / UTM zone 17N',NULL,'EPSG','4400','EPSG','4759','EPSG','16017',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2743','projected_crs','EPSG','3724','EPSG','3504','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3725','NAD83(NSRS2007) / UTM zone 18N',NULL,'EPSG','4400','EPSG','4759','EPSG','16018',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2744','projected_crs','EPSG','3725','EPSG','3505','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3726','NAD83(NSRS2007) / UTM zone 19N',NULL,'EPSG','4400','EPSG','4759','EPSG','16019',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2745','projected_crs','EPSG','3726','EPSG','3506','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3727','Reunion 1947 / TM Reunion',NULL,'EPSG','4499','EPSG','4626','EPSG','19856',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2746','projected_crs','EPSG','3727','EPSG','3337','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3728','NAD83(NSRS2007) / Ohio North (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','13433',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2747','projected_crs','EPSG','3728','EPSG','2239','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3729','NAD83(NSRS2007) / Ohio South (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','13434',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2748','projected_crs','EPSG','3729','EPSG','2240','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3730','NAD83(NSRS2007) / Wyoming East (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','14935',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2749','projected_crs','EPSG','3730','EPSG','2269','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3731','NAD83(NSRS2007) / Wyoming East Central (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','14936',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2750','projected_crs','EPSG','3731','EPSG','2270','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3732','NAD83(NSRS2007) / Wyoming West Central (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','14937',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2751','projected_crs','EPSG','3732','EPSG','2272','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3733','NAD83(NSRS2007) / Wyoming West (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','14938',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2752','projected_crs','EPSG','3733','EPSG','2271','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3734','NAD83 / Ohio North (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','13433',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2753','projected_crs','EPSG','3734','EPSG','2239','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3735','NAD83 / Ohio South (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','13434',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2754','projected_crs','EPSG','3735','EPSG','2240','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3736','NAD83 / Wyoming East (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','14935',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2755','projected_crs','EPSG','3736','EPSG','2269','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3737','NAD83 / Wyoming East Central (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','14936',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2756','projected_crs','EPSG','3737','EPSG','2270','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3738','NAD83 / Wyoming West Central (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','14937',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2757','projected_crs','EPSG','3738','EPSG','2272','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3739','NAD83 / Wyoming West (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','14938',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2758','projected_crs','EPSG','3739','EPSG','2271','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3740','NAD83(HARN) / UTM zone 10N',NULL,'EPSG','4400','EPSG','4152','EPSG','16010',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2759','projected_crs','EPSG','3740','EPSG','3857','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3741','NAD83(HARN) / UTM zone 11N',NULL,'EPSG','4400','EPSG','4152','EPSG','16011',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2760','projected_crs','EPSG','3741','EPSG','3852','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3742','NAD83(HARN) / UTM zone 12N',NULL,'EPSG','4400','EPSG','4152','EPSG','16012',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2761','projected_crs','EPSG','3742','EPSG','3499','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3743','NAD83(HARN) / UTM zone 13N',NULL,'EPSG','4400','EPSG','4152','EPSG','16013',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2762','projected_crs','EPSG','3743','EPSG','3500','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3744','NAD83(HARN) / UTM zone 14N',NULL,'EPSG','4400','EPSG','4152','EPSG','16014',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2763','projected_crs','EPSG','3744','EPSG','3860','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3745','NAD83(HARN) / UTM zone 15N',NULL,'EPSG','4400','EPSG','4152','EPSG','16015',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2764','projected_crs','EPSG','3745','EPSG','3861','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3746','NAD83(HARN) / UTM zone 16N',NULL,'EPSG','4400','EPSG','4152','EPSG','16016',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2765','projected_crs','EPSG','3746','EPSG','3862','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3747','NAD83(HARN) / UTM zone 17N',NULL,'EPSG','4400','EPSG','4152','EPSG','16017',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2766','projected_crs','EPSG','3747','EPSG','3863','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3748','NAD83(HARN) / UTM zone 18N',NULL,'EPSG','4400','EPSG','4152','EPSG','16018',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2767','projected_crs','EPSG','3748','EPSG','3868','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3749','NAD83(HARN) / UTM zone 19N',NULL,'EPSG','4400','EPSG','4152','EPSG','16019',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2768','projected_crs','EPSG','3749','EPSG','3871','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3750','NAD83(HARN) / UTM zone 4N',NULL,'EPSG','4400','EPSG','4152','EPSG','16004',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2769','projected_crs','EPSG','3750','EPSG','3488','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3751','NAD83(HARN) / UTM zone 5N',NULL,'EPSG','4400','EPSG','4152','EPSG','16005',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2770','projected_crs','EPSG','3751','EPSG','3491','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3752','WGS 84 / Mercator 41',NULL,'EPSG','4499','EPSG','4326','EPSG','19855',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2771','projected_crs','EPSG','3752','EPSG','3508','EPSG','1210');
INSERT INTO "projected_crs" VALUES('EPSG','3753','NAD83(HARN) / Ohio North (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','13433',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2772','projected_crs','EPSG','3753','EPSG','2239','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3754','NAD83(HARN) / Ohio South (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','13434',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2773','projected_crs','EPSG','3754','EPSG','2240','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3755','NAD83(HARN) / Wyoming East (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','14935',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2774','projected_crs','EPSG','3755','EPSG','2269','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3756','NAD83(HARN) / Wyoming East Central (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','14936',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2775','projected_crs','EPSG','3756','EPSG','2270','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3757','NAD83(HARN) / Wyoming West Central (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','14937',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2776','projected_crs','EPSG','3757','EPSG','2272','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3758','NAD83(HARN) / Wyoming West (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','14938',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2777','projected_crs','EPSG','3758','EPSG','2271','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3759','NAD83 / Hawaii zone 3 (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15138',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2778','projected_crs','EPSG','3759','EPSG','1548','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3760','NAD83(HARN) / Hawaii zone 3 (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15138',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2779','projected_crs','EPSG','3760','EPSG','1548','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3761','NAD83(CSRS) / UTM zone 22N',NULL,'EPSG','4400','EPSG','4617','EPSG','16022',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2780','projected_crs','EPSG','3761','EPSG','2152','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3762','WGS 84 / South Georgia Lambert',NULL,'EPSG','4400','EPSG','4326','EPSG','19854',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2781','projected_crs','EPSG','3762','EPSG','3529','EPSG','1189');
INSERT INTO "projected_crs" VALUES('EPSG','3763','ETRS89 / Portugal TM06',NULL,'EPSG','4499','EPSG','4258','EPSG','19853',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2782','projected_crs','EPSG','3763','EPSG','1294','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','3764','NZGD2000 / Chatham Island Circuit 2000',NULL,'EPSG','4500','EPSG','4167','EPSG','17959',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2783','projected_crs','EPSG','3764','EPSG','2889','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','3765','HTRS96 / Croatia TM',NULL,'EPSG','4400','EPSG','4761','EPSG','19851',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2784','projected_crs','EPSG','3765','EPSG','3234','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','3766','HTRS96 / Croatia LCC',NULL,'EPSG','4400','EPSG','4761','EPSG','19852',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2785','projected_crs','EPSG','3766','EPSG','1076','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','3767','HTRS96 / UTM zone 33N',NULL,'EPSG','4400','EPSG','4761','EPSG','16033',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2786','projected_crs','EPSG','3767','EPSG','3539','EPSG','1160');
INSERT INTO "projected_crs" VALUES('EPSG','3768','HTRS96 / UTM zone 34N',NULL,'EPSG','4400','EPSG','4761','EPSG','16034',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2787','projected_crs','EPSG','3768','EPSG','3538','EPSG','1160');
INSERT INTO "projected_crs" VALUES('EPSG','3769','Bermuda 1957 / UTM zone 20N',NULL,'EPSG','4400','EPSG','4216','EPSG','16020',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2788','projected_crs','EPSG','3769','EPSG','3221','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','3770','BDA2000 / Bermuda 2000 National Grid',NULL,'EPSG','4400','EPSG','4762','EPSG','19849',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2789','projected_crs','EPSG','3770','EPSG','1047','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','3771','NAD27 / Alberta 3TM ref merid 111 W',NULL,'EPSG','4400','EPSG','4267','EPSG','17722',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2790','projected_crs','EPSG','3771','EPSG','3543','EPSG','1096');
INSERT INTO "projected_crs" VALUES('EPSG','3772','NAD27 / Alberta 3TM ref merid 114 W',NULL,'EPSG','4400','EPSG','4267','EPSG','17723',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2791','projected_crs','EPSG','3772','EPSG','3542','EPSG','1096');
INSERT INTO "projected_crs" VALUES('EPSG','3773','NAD27 / Alberta 3TM ref merid 117 W',NULL,'EPSG','4400','EPSG','4267','EPSG','17724',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2792','projected_crs','EPSG','3773','EPSG','3541','EPSG','1096');
INSERT INTO "projected_crs" VALUES('EPSG','3774','NAD27 / Alberta 3TM ref merid 120 W',NULL,'EPSG','4400','EPSG','4267','EPSG','17725',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2793','projected_crs','EPSG','3774','EPSG','3540','EPSG','1096');
INSERT INTO "projected_crs" VALUES('EPSG','3775','NAD83 / Alberta 3TM ref merid 111 W',NULL,'EPSG','4400','EPSG','4269','EPSG','17722',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2794','projected_crs','EPSG','3775','EPSG','3543','EPSG','1096');
INSERT INTO "projected_crs" VALUES('EPSG','3776','NAD83 / Alberta 3TM ref merid 114 W',NULL,'EPSG','4400','EPSG','4269','EPSG','17723',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2795','projected_crs','EPSG','3776','EPSG','3542','EPSG','1096');
INSERT INTO "projected_crs" VALUES('EPSG','3777','NAD83 / Alberta 3TM ref merid 117 W',NULL,'EPSG','4400','EPSG','4269','EPSG','17724',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2796','projected_crs','EPSG','3777','EPSG','3541','EPSG','1096');
INSERT INTO "projected_crs" VALUES('EPSG','3778','NAD83 / Alberta 3TM ref merid 120 W',NULL,'EPSG','4400','EPSG','4269','EPSG','17725',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2797','projected_crs','EPSG','3778','EPSG','3540','EPSG','1096');
INSERT INTO "projected_crs" VALUES('EPSG','3779','NAD83(CSRS) / Alberta 3TM ref merid 111 W',NULL,'EPSG','4400','EPSG','4617','EPSG','17722',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2798','projected_crs','EPSG','3779','EPSG','3543','EPSG','1096');
INSERT INTO "projected_crs" VALUES('EPSG','3780','NAD83(CSRS) / Alberta 3TM ref merid 114 W',NULL,'EPSG','4400','EPSG','4617','EPSG','17723',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2799','projected_crs','EPSG','3780','EPSG','3542','EPSG','1096');
INSERT INTO "projected_crs" VALUES('EPSG','3781','NAD83(CSRS) / Alberta 3TM ref merid 117 W',NULL,'EPSG','4400','EPSG','4617','EPSG','17724',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2800','projected_crs','EPSG','3781','EPSG','3541','EPSG','1096');
INSERT INTO "projected_crs" VALUES('EPSG','3782','NAD83(CSRS) / Alberta 3TM ref merid 120 W',NULL,'EPSG','4400','EPSG','4617','EPSG','17725',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2801','projected_crs','EPSG','3782','EPSG','3540','EPSG','1096');
INSERT INTO "projected_crs" VALUES('EPSG','3783','Pitcairn 2006 / Pitcairn TM 2006',NULL,'EPSG','4400','EPSG','4763','EPSG','19848',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2802','projected_crs','EPSG','3783','EPSG','3208','EPSG','1056');
INSERT INTO "projected_crs" VALUES('EPSG','3784','Pitcairn 1967 / UTM zone 9S',NULL,'EPSG','4400','EPSG','4729','EPSG','16109',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2803','projected_crs','EPSG','3784','EPSG','3208','EPSG','1153');
INSERT INTO "projected_crs" VALUES('EPSG','3785','Popular Visualisation CRS / Mercator',NULL,'EPSG','4499','EPSG','4055','EPSG','19847',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2804','projected_crs','EPSG','3785','EPSG','3544','EPSG','1098');
INSERT INTO "projected_crs" VALUES('EPSG','3786','World Equidistant Cylindrical (Sphere)',NULL,'EPSG','4499','EPSG','4047','EPSG','19968',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2805','projected_crs','EPSG','3786','EPSG','1262','EPSG','1098');
INSERT INTO "projected_crs" VALUES('EPSG','3787','MGI / Slovene National Grid',NULL,'EPSG','4498','EPSG','4312','EPSG','19845',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2806','projected_crs','EPSG','3787','EPSG','1212','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3788','NZGD2000 / Auckland Islands TM 2000',NULL,'EPSG','4500','EPSG','4167','EPSG','17960',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2807','projected_crs','EPSG','3788','EPSG','3554','EPSG','1056');
INSERT INTO "projected_crs" VALUES('EPSG','3789','NZGD2000 / Campbell Island TM 2000',NULL,'EPSG','4500','EPSG','4167','EPSG','17961',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2808','projected_crs','EPSG','3789','EPSG','3555','EPSG','1056');
INSERT INTO "projected_crs" VALUES('EPSG','3790','NZGD2000 / Antipodes Islands TM 2000',NULL,'EPSG','4500','EPSG','4167','EPSG','17962',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2809','projected_crs','EPSG','3790','EPSG','3556','EPSG','1056');
INSERT INTO "projected_crs" VALUES('EPSG','3791','NZGD2000 / Raoul Island TM 2000',NULL,'EPSG','4500','EPSG','4167','EPSG','17963',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2810','projected_crs','EPSG','3791','EPSG','3557','EPSG','1056');
INSERT INTO "projected_crs" VALUES('EPSG','3793','NZGD2000 / Chatham Islands TM 2000',NULL,'EPSG','4500','EPSG','4167','EPSG','17965',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2811','projected_crs','EPSG','3793','EPSG','2889','EPSG','1056');
INSERT INTO "projected_crs" VALUES('EPSG','3794','Slovenia 1996 / Slovene National Grid',NULL,'EPSG','4400','EPSG','4765','EPSG','19845',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2812','projected_crs','EPSG','3794','EPSG','1212','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3795','NAD27 / Cuba Norte',NULL,'EPSG','4532','EPSG','4267','EPSG','18063',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2813','projected_crs','EPSG','3795','EPSG','1487','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3796','NAD27 / Cuba Sur',NULL,'EPSG','4532','EPSG','4267','EPSG','18064',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2814','projected_crs','EPSG','3796','EPSG','1488','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3797','NAD27 / MTQ Lambert',NULL,'EPSG','4499','EPSG','4267','EPSG','19844',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2815','projected_crs','EPSG','3797','EPSG','1368','EPSG','1220');
INSERT INTO "projected_crs" VALUES('EPSG','3798','NAD83 / MTQ Lambert',NULL,'EPSG','4499','EPSG','4269','EPSG','19844',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2816','projected_crs','EPSG','3798','EPSG','1368','EPSG','1220');
INSERT INTO "projected_crs" VALUES('EPSG','3799','NAD83(CSRS) / MTQ Lambert',NULL,'EPSG','4499','EPSG','4617','EPSG','19844',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2817','projected_crs','EPSG','3799','EPSG','1368','EPSG','1220');
INSERT INTO "projected_crs" VALUES('EPSG','3800','NAD27 / Alberta 3TM ref merid 120 W',NULL,'EPSG','4400','EPSG','4267','EPSG','17726',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2818','projected_crs','EPSG','3800','EPSG','3540','EPSG','1096');
INSERT INTO "projected_crs" VALUES('EPSG','3801','NAD83 / Alberta 3TM ref merid 120 W',NULL,'EPSG','4400','EPSG','4269','EPSG','17726',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2819','projected_crs','EPSG','3801','EPSG','3540','EPSG','1096');
INSERT INTO "projected_crs" VALUES('EPSG','3802','NAD83(CSRS) / Alberta 3TM ref merid 120 W',NULL,'EPSG','4400','EPSG','4617','EPSG','17726',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2820','projected_crs','EPSG','3802','EPSG','3540','EPSG','1096');
INSERT INTO "projected_crs" VALUES('EPSG','3812','ETRS89 / Belgian Lambert 2008',NULL,'EPSG','4499','EPSG','4258','EPSG','3811',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2821','projected_crs','EPSG','3812','EPSG','1347','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3814','NAD83 / Mississippi TM',NULL,'EPSG','4499','EPSG','4269','EPSG','3813',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2822','projected_crs','EPSG','3814','EPSG','1393','EPSG','1135');
INSERT INTO "projected_crs" VALUES('EPSG','3815','NAD83(HARN) / Mississippi TM',NULL,'EPSG','4499','EPSG','4152','EPSG','3813',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2823','projected_crs','EPSG','3815','EPSG','1393','EPSG','1135');
INSERT INTO "projected_crs" VALUES('EPSG','3816','NAD83(NSRS2007) / Mississippi TM',NULL,'EPSG','4499','EPSG','4759','EPSG','3813',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2824','projected_crs','EPSG','3816','EPSG','1393','EPSG','1135');
INSERT INTO "projected_crs" VALUES('EPSG','3825','TWD97 / TM2 zone 119',NULL,'EPSG','4499','EPSG','3824','EPSG','3818',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2830','projected_crs','EPSG','3825','EPSG','3563','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3826','TWD97 / TM2 zone 121',NULL,'EPSG','4499','EPSG','3824','EPSG','3820',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2831','projected_crs','EPSG','3826','EPSG','3562','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3827','TWD67 / TM2 zone 119',NULL,'EPSG','4499','EPSG','3821','EPSG','3818',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2832','projected_crs','EPSG','3827','EPSG','3591','EPSG','1056');
INSERT INTO "projected_crs" VALUES('EPSG','3828','TWD67 / TM2 zone 121',NULL,'EPSG','4499','EPSG','3821','EPSG','3820',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2833','projected_crs','EPSG','3828','EPSG','3982','EPSG','1056');
INSERT INTO "projected_crs" VALUES('EPSG','3829','Hu Tzu Shan 1950 / UTM zone 51N',NULL,'EPSG','4400','EPSG','4236','EPSG','16051',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2834','projected_crs','EPSG','3829','EPSG','3315','EPSG','1160');
INSERT INTO "projected_crs" VALUES('EPSG','3832','WGS 84 / PDC Mercator',NULL,'EPSG','4400','EPSG','4326','EPSG','3831',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2835','projected_crs','EPSG','3832','EPSG','3172','EPSG','1241');
INSERT INTO "projected_crs" VALUES('EPSG','3833','Pulkovo 1942(58) / Gauss-Kruger zone 2',NULL,'EPSG','4530','EPSG','4179','EPSG','16202',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2836','projected_crs','EPSG','3833','EPSG','3575','EPSG','1160');
INSERT INTO "projected_crs" VALUES('EPSG','3834','Pulkovo 1942(83) / Gauss-Kruger zone 2',NULL,'EPSG','4530','EPSG','4178','EPSG','16202',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2837','projected_crs','EPSG','3834','EPSG','3575','EPSG','1160');
INSERT INTO "projected_crs" VALUES('EPSG','3835','Pulkovo 1942(83) / Gauss-Kruger zone 3',NULL,'EPSG','4530','EPSG','4178','EPSG','16203',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2838','projected_crs','EPSG','3835','EPSG','3576','EPSG','1160');
INSERT INTO "projected_crs" VALUES('EPSG','3836','Pulkovo 1942(83) / Gauss-Kruger zone 4',NULL,'EPSG','4530','EPSG','4178','EPSG','16204',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2839','projected_crs','EPSG','3836','EPSG','3578','EPSG','1160');
INSERT INTO "projected_crs" VALUES('EPSG','3837','Pulkovo 1942(58) / 3-degree Gauss-Kruger zone 3',NULL,'EPSG','4530','EPSG','4179','EPSG','16263',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2840','projected_crs','EPSG','3837','EPSG','1512','EPSG','1160');
INSERT INTO "projected_crs" VALUES('EPSG','3838','Pulkovo 1942(58) / 3-degree Gauss-Kruger zone 4',NULL,'EPSG','4530','EPSG','4179','EPSG','16264',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2841','projected_crs','EPSG','3838','EPSG','1513','EPSG','1160');
INSERT INTO "projected_crs" VALUES('EPSG','3839','Pulkovo 1942(58) / 3-degree Gauss-Kruger zone 9',NULL,'EPSG','4530','EPSG','4179','EPSG','16269',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2842','projected_crs','EPSG','3839','EPSG','3587','EPSG','1160');
INSERT INTO "projected_crs" VALUES('EPSG','3840','Pulkovo 1942(58) / 3-degree Gauss-Kruger zone 10',NULL,'EPSG','4530','EPSG','4179','EPSG','16270',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2843','projected_crs','EPSG','3840','EPSG','3588','EPSG','1160');
INSERT INTO "projected_crs" VALUES('EPSG','3841','Pulkovo 1942(83) / 3-degree Gauss-Kruger zone 6',NULL,'EPSG','4530','EPSG','4178','EPSG','16266',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2844','projected_crs','EPSG','3841','EPSG','3582','EPSG','1160');
INSERT INTO "projected_crs" VALUES('EPSG','3842','Pulkovo 1942(83) / 3-degree Gauss-Kruger zone 7',NULL,'EPSG','4530','EPSG','4178','EPSG','16266',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2845','projected_crs','EPSG','3842','EPSG','3584','EPSG','1160');
INSERT INTO "projected_crs" VALUES('EPSG','3843','Pulkovo 1942(83) / 3-degree Gauss-Kruger zone 8',NULL,'EPSG','4530','EPSG','4178','EPSG','16266',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2846','projected_crs','EPSG','3843','EPSG','3586','EPSG','1160');
INSERT INTO "projected_crs" VALUES('EPSG','3844','Pulkovo 1942(58) / Stereo70',NULL,'EPSG','4530','EPSG','4179','EPSG','19926',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2847','projected_crs','EPSG','3844','EPSG','1197','EPSG','1056');
INSERT INTO "projected_crs" VALUES('EPSG','3845','SWEREF99 / RT90 7.5 gon V emulation',NULL,'EPSG','4530','EPSG','4619','EPSG','17339',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2848','projected_crs','EPSG','3845','EPSG','2845','EPSG','1230');
INSERT INTO "projected_crs" VALUES('EPSG','3846','SWEREF99 / RT90 5 gon V emulation',NULL,'EPSG','4530','EPSG','4619','EPSG','17340',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2849','projected_crs','EPSG','3846','EPSG','2846','EPSG','1230');
INSERT INTO "projected_crs" VALUES('EPSG','3847','SWEREF99 / RT90 2.5 gon V emulation',NULL,'EPSG','4530','EPSG','4619','EPSG','17341',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2850','projected_crs','EPSG','3847','EPSG','2847','EPSG','1230');
INSERT INTO "projected_crs" VALUES('EPSG','3848','SWEREF99 / RT90 0 gon emulation',NULL,'EPSG','4530','EPSG','4619','EPSG','17342',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2851','projected_crs','EPSG','3848','EPSG','2848','EPSG','1230');
INSERT INTO "projected_crs" VALUES('EPSG','3849','SWEREF99 / RT90 2.5 gon O emulation',NULL,'EPSG','4530','EPSG','4619','EPSG','17343',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2852','projected_crs','EPSG','3849','EPSG','2849','EPSG','1230');
INSERT INTO "projected_crs" VALUES('EPSG','3850','SWEREF99 / RT90 5 gon O emulation',NULL,'EPSG','4530','EPSG','4619','EPSG','17344',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2853','projected_crs','EPSG','3850','EPSG','2850','EPSG','1230');
INSERT INTO "projected_crs" VALUES('EPSG','3851','NZGD2000 / NZCS2000',NULL,'EPSG','4500','EPSG','4167','EPSG','17964',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2854','projected_crs','EPSG','3851','EPSG','3593','EPSG','1026');
INSERT INTO "projected_crs" VALUES('EPSG','3852','RSRGD2000 / DGLC2000',NULL,'EPSG','4500','EPSG','4764','EPSG','17966',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2855','projected_crs','EPSG','3852','EPSG','3592','EPSG','1236');
INSERT INTO "projected_crs" VALUES('EPSG','3854','County ST74',NULL,'EPSG','4531','EPSG','4619','EPSG','3853',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2856','projected_crs','EPSG','3854','EPSG','3608','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','3857','WGS 84 / Pseudo-Mercator',NULL,'EPSG','4499','EPSG','4326','EPSG','3856',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2858','projected_crs','EPSG','3857','EPSG','3544','EPSG','1098');
INSERT INTO "projected_crs" VALUES('EPSG','3873','EUREF-FIN / GK19FIN',NULL,'EPSG','4500','EPSG','10690','EPSG','3860',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2859','projected_crs','EPSG','3873','EPSG','3595','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','3874','EUREF-FIN / GK20FIN',NULL,'EPSG','4500','EPSG','10690','EPSG','3861',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2860','projected_crs','EPSG','3874','EPSG','3596','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','3875','EUREF-FIN / GK21FIN',NULL,'EPSG','4500','EPSG','10690','EPSG','3862',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2861','projected_crs','EPSG','3875','EPSG','3597','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','3876','EUREF-FIN / GK22FIN',NULL,'EPSG','4500','EPSG','10690','EPSG','3863',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2862','projected_crs','EPSG','3876','EPSG','3598','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','3877','EUREF-FIN / GK23FIN',NULL,'EPSG','4500','EPSG','10690','EPSG','3864',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2863','projected_crs','EPSG','3877','EPSG','3599','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','3878','EUREF-FIN / GK24FIN',NULL,'EPSG','4500','EPSG','10690','EPSG','3865',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2864','projected_crs','EPSG','3878','EPSG','3600','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','3879','EUREF-FIN / GK25FIN',NULL,'EPSG','4500','EPSG','10690','EPSG','3866',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2865','projected_crs','EPSG','3879','EPSG','3601','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','3880','EUREF-FIN / GK26FIN',NULL,'EPSG','4500','EPSG','10690','EPSG','3867',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2866','projected_crs','EPSG','3880','EPSG','3602','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','3881','EUREF-FIN / GK27FIN',NULL,'EPSG','4500','EPSG','10690','EPSG','3868',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2867','projected_crs','EPSG','3881','EPSG','3603','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','3882','EUREF-FIN / GK28FIN',NULL,'EPSG','4500','EPSG','10690','EPSG','3869',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2868','projected_crs','EPSG','3882','EPSG','3604','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','3883','EUREF-FIN / GK29FIN',NULL,'EPSG','4500','EPSG','10690','EPSG','3870',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2869','projected_crs','EPSG','3883','EPSG','3605','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','3884','EUREF-FIN / GK30FIN',NULL,'EPSG','4500','EPSG','10690','EPSG','3871',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2870','projected_crs','EPSG','3884','EPSG','3606','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','3885','EUREF-FIN / GK31FIN',NULL,'EPSG','4500','EPSG','10690','EPSG','3872',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2871','projected_crs','EPSG','3885','EPSG','3607','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','3890','IGRS / UTM zone 37N',NULL,'EPSG','4400','EPSG','3889','EPSG','16037',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2876','projected_crs','EPSG','3890','EPSG','3387','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3891','IGRS / UTM zone 38N',NULL,'EPSG','4400','EPSG','3889','EPSG','16038',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2877','projected_crs','EPSG','3891','EPSG','3388','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3892','IGRS / UTM zone 39N',NULL,'EPSG','4400','EPSG','3889','EPSG','16039',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2878','projected_crs','EPSG','3892','EPSG','3956','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3893','ED50 / Iraq National Grid',NULL,'EPSG','4400','EPSG','4230','EPSG','19907',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2879','projected_crs','EPSG','3893','EPSG','3625','EPSG','1160');
INSERT INTO "projected_crs" VALUES('EPSG','3907','MGI 1901 / Balkans zone 5',NULL,'EPSG','4530','EPSG','3906','EPSG','18275',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2885','projected_crs','EPSG','3907','EPSG','1709','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3908','MGI 1901 / Balkans zone 6',NULL,'EPSG','4530','EPSG','3906','EPSG','18276',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2886','projected_crs','EPSG','3908','EPSG','1710','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3909','MGI 1901 / Balkans zone 7',NULL,'EPSG','4530','EPSG','3906','EPSG','18277',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2887','projected_crs','EPSG','3909','EPSG','1711','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3910','MGI 1901 / Balkans zone 8',NULL,'EPSG','4530','EPSG','3906','EPSG','18278',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2888','projected_crs','EPSG','3910','EPSG','1712','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3911','MGI 1901 / Slovenia Grid',NULL,'EPSG','4530','EPSG','3906','EPSG','19967',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2889','projected_crs','EPSG','3911','EPSG','1212','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3912','MGI 1901 / Slovene National Grid',NULL,'EPSG','4498','EPSG','3906','EPSG','19845',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2890','projected_crs','EPSG','3912','EPSG','1212','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','3920','Puerto Rico / UTM zone 20N',NULL,'EPSG','4400','EPSG','4139','EPSG','16020',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2891','projected_crs','EPSG','3920','EPSG','3329','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3942','RGF93 v1 / CC42',NULL,'EPSG','4499','EPSG','4171','EPSG','18101',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2892','projected_crs','EPSG','3942','EPSG','3545','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','3943','RGF93 v1 / CC43',NULL,'EPSG','4499','EPSG','4171','EPSG','18102',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2893','projected_crs','EPSG','3943','EPSG','3546','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','3944','RGF93 v1 / CC44',NULL,'EPSG','4499','EPSG','4171','EPSG','18103',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2894','projected_crs','EPSG','3944','EPSG','3547','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','3945','RGF93 v1 / CC45',NULL,'EPSG','4499','EPSG','4171','EPSG','18104',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2895','projected_crs','EPSG','3945','EPSG','3548','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','3946','RGF93 v1 / CC46',NULL,'EPSG','4499','EPSG','4171','EPSG','18105',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2896','projected_crs','EPSG','3946','EPSG','3549','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','3947','RGF93 v1 / CC47',NULL,'EPSG','4499','EPSG','4171','EPSG','18106',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2897','projected_crs','EPSG','3947','EPSG','3550','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','3948','RGF93 v1 / CC48',NULL,'EPSG','4499','EPSG','4171','EPSG','18107',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2898','projected_crs','EPSG','3948','EPSG','3551','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','3949','RGF93 v1 / CC49',NULL,'EPSG','4499','EPSG','4171','EPSG','18108',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2899','projected_crs','EPSG','3949','EPSG','3552','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','3950','RGF93 v1 / CC50',NULL,'EPSG','4499','EPSG','4171','EPSG','18109',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2900','projected_crs','EPSG','3950','EPSG','3553','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','3968','NAD83 / Virginia Lambert',NULL,'EPSG','4499','EPSG','4269','EPSG','3967',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2901','projected_crs','EPSG','3968','EPSG','1415','EPSG','1135');
INSERT INTO "projected_crs" VALUES('EPSG','3969','NAD83(HARN) / Virginia Lambert',NULL,'EPSG','4499','EPSG','4152','EPSG','3967',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2902','projected_crs','EPSG','3969','EPSG','1415','EPSG','1135');
INSERT INTO "projected_crs" VALUES('EPSG','3970','NAD83(NSRS2007) / Virginia Lambert',NULL,'EPSG','4499','EPSG','4759','EPSG','3967',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2903','projected_crs','EPSG','3970','EPSG','1415','EPSG','1135');
INSERT INTO "projected_crs" VALUES('EPSG','3973','WGS 84 / NSIDC EASE-Grid North',NULL,'EPSG','4469','EPSG','4326','EPSG','3897',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2904','projected_crs','EPSG','3973','EPSG','3475','EPSG','1195');
INSERT INTO "projected_crs" VALUES('EPSG','3974','WGS 84 / NSIDC EASE-Grid South',NULL,'EPSG','4470','EPSG','4326','EPSG','3898',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2905','projected_crs','EPSG','3974','EPSG','3474','EPSG','1195');
INSERT INTO "projected_crs" VALUES('EPSG','3975','WGS 84 / NSIDC EASE-Grid Global',NULL,'EPSG','4499','EPSG','4326','EPSG','19869',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2906','projected_crs','EPSG','3975','EPSG','3463','EPSG','1195');
INSERT INTO "projected_crs" VALUES('EPSG','3976','WGS 84 / NSIDC Sea Ice Polar Stereographic South',NULL,'EPSG','4470','EPSG','4326','EPSG','19866',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2907','projected_crs','EPSG','3976','EPSG','1997','EPSG','1040');
INSERT INTO "projected_crs" VALUES('EPSG','3978','NAD83 / Canada Atlas Lambert',NULL,'EPSG','4400','EPSG','4269','EPSG','3977',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2908','projected_crs','EPSG','3978','EPSG','1061','EPSG','1045');
INSERT INTO "projected_crs" VALUES('EPSG','3979','NAD83(CSRS) / Canada Atlas Lambert',NULL,'EPSG','4400','EPSG','4617','EPSG','3977',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2909','projected_crs','EPSG','3979','EPSG','1061','EPSG','1045');
INSERT INTO "projected_crs" VALUES('EPSG','3985','Katanga 1955 / Katanga Lambert',NULL,'EPSG','4499','EPSG','4695','EPSG','3980',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2910','projected_crs','EPSG','3985','EPSG','3147','EPSG','1210');
INSERT INTO "projected_crs" VALUES('EPSG','3986','Katanga 1955 / Katanga Gauss zone A',NULL,'EPSG','4499','EPSG','4695','EPSG','3981',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2911','projected_crs','EPSG','3986','EPSG','3612','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','3987','Katanga 1955 / Katanga Gauss zone B',NULL,'EPSG','4499','EPSG','4695','EPSG','3982',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2912','projected_crs','EPSG','3987','EPSG','3611','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','3988','Katanga 1955 / Katanga Gauss zone C',NULL,'EPSG','4499','EPSG','4695','EPSG','3983',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2913','projected_crs','EPSG','3988','EPSG','3610','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','3989','Katanga 1955 / Katanga Gauss zone D',NULL,'EPSG','4499','EPSG','4695','EPSG','3984',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2914','projected_crs','EPSG','3989','EPSG','3609','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','3991','Puerto Rico State Plane CS of 1927',NULL,'EPSG','4497','EPSG','4139','EPSG','15201',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2915','projected_crs','EPSG','3991','EPSG','3294','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3992','Puerto Rico / St. Croix',NULL,'EPSG','4497','EPSG','4139','EPSG','15202',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2916','projected_crs','EPSG','3992','EPSG','3330','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','3993','Guam 1963 / Guam SPCS',NULL,'EPSG','4499','EPSG','4675','EPSG','15400',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2917','projected_crs','EPSG','3993','EPSG','3255','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','3994','WGS 84 / Mercator 41',NULL,'EPSG','4499','EPSG','4326','EPSG','19843',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2918','projected_crs','EPSG','3994','EPSG','3508','EPSG','1210');
INSERT INTO "projected_crs" VALUES('EPSG','3995','WGS 84 / Arctic Polar Stereographic',NULL,'EPSG','4469','EPSG','4326','EPSG','19842',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2919','projected_crs','EPSG','3995','EPSG','1996','EPSG','1040');
INSERT INTO "projected_crs" VALUES('EPSG','3996','WGS 84 / IBCAO Polar Stereographic',NULL,'EPSG','4469','EPSG','4326','EPSG','19840',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2920','projected_crs','EPSG','3996','EPSG','1996','EPSG','1198');
INSERT INTO "projected_crs" VALUES('EPSG','3997','WGS 84 / Dubai Local TM',NULL,'EPSG','4400','EPSG','4326','EPSG','19839',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2921','projected_crs','EPSG','3997','EPSG','3531','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','4026','MOLDREF99 / Moldova TM',NULL,'EPSG','4530','EPSG','4023','EPSG','3999',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2948','projected_crs','EPSG','4026','EPSG','1162','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4037','WGS 84 / TMzn35N',NULL,'EPSG','4500','EPSG','4326','EPSG','16035',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2959','projected_crs','EPSG','4037','EPSG','3615','EPSG','1210');
INSERT INTO "projected_crs" VALUES('EPSG','4038','WGS 84 / TMzn36N',NULL,'EPSG','4500','EPSG','4326','EPSG','16036',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2960','projected_crs','EPSG','4038','EPSG','3616','EPSG','1210');
INSERT INTO "projected_crs" VALUES('EPSG','4048','RGRDC 2005 / Congo TM zone 12',NULL,'EPSG','4499','EPSG','4046','EPSG','17412',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2970','projected_crs','EPSG','4048','EPSG','3937','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','4049','RGRDC 2005 / Congo TM zone 14',NULL,'EPSG','4499','EPSG','4046','EPSG','17414',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2971','projected_crs','EPSG','4049','EPSG','3151','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','4050','RGRDC 2005 / Congo TM zone 16',NULL,'EPSG','4499','EPSG','4046','EPSG','17416',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2972','projected_crs','EPSG','4050','EPSG','3617','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','4051','RGRDC 2005 / Congo TM zone 18',NULL,'EPSG','4499','EPSG','4046','EPSG','17418',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2973','projected_crs','EPSG','4051','EPSG','3618','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','4056','RGRDC 2005 / Congo TM zone 20',NULL,'EPSG','4499','EPSG','4046','EPSG','17420',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2978','projected_crs','EPSG','4056','EPSG','3620','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','4057','RGRDC 2005 / Congo TM zone 22',NULL,'EPSG','4499','EPSG','4046','EPSG','17422',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2979','projected_crs','EPSG','4057','EPSG','3621','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','4058','RGRDC 2005 / Congo TM zone 24',NULL,'EPSG','4499','EPSG','4046','EPSG','17424',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2980','projected_crs','EPSG','4058','EPSG','3622','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','4059','RGRDC 2005 / Congo TM zone 26',NULL,'EPSG','4499','EPSG','4046','EPSG','17426',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2981','projected_crs','EPSG','4059','EPSG','3623','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','4060','RGRDC 2005 / Congo TM zone 28',NULL,'EPSG','4499','EPSG','4046','EPSG','17428',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2982','projected_crs','EPSG','4060','EPSG','3624','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','4061','RGRDC 2005 / UTM zone 33S',NULL,'EPSG','4499','EPSG','4046','EPSG','16133',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2983','projected_crs','EPSG','4061','EPSG','3626','EPSG','1153');
INSERT INTO "projected_crs" VALUES('EPSG','4062','RGRDC 2005 / UTM zone 34S',NULL,'EPSG','4499','EPSG','4046','EPSG','16134',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2984','projected_crs','EPSG','4062','EPSG','3627','EPSG','1153');
INSERT INTO "projected_crs" VALUES('EPSG','4063','RGRDC 2005 / UTM zone 35S',NULL,'EPSG','4499','EPSG','4046','EPSG','16135',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2985','projected_crs','EPSG','4063','EPSG','3628','EPSG','1153');
INSERT INTO "projected_crs" VALUES('EPSG','4071','Chua / UTM zone 23S',NULL,'EPSG','4400','EPSG','4224','EPSG','16123',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2986','projected_crs','EPSG','4071','EPSG','3619','EPSG','1143');
INSERT INTO "projected_crs" VALUES('EPSG','4082','REGCAN95 / UTM zone 27N',NULL,'EPSG','4400','EPSG','4081','EPSG','16027',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2993','projected_crs','EPSG','4082','EPSG','3629','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','4083','REGCAN95 / UTM zone 28N',NULL,'EPSG','4400','EPSG','4081','EPSG','16028',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2994','projected_crs','EPSG','4083','EPSG','3630','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','4087','WGS 84 / World Equidistant Cylindrical',NULL,'EPSG','4499','EPSG','4326','EPSG','4085',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2995','projected_crs','EPSG','4087','EPSG','1262','EPSG','1191');
INSERT INTO "projected_crs" VALUES('EPSG','4088','World Equidistant Cylindrical (Sphere)',NULL,'EPSG','4499','EPSG','4047','EPSG','4086',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2996','projected_crs','EPSG','4088','EPSG','1262','EPSG','1098');
INSERT INTO "projected_crs" VALUES('EPSG','4093','ETRS89 / DKTM1',NULL,'EPSG','4400','EPSG','4258','EPSG','4089',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2997','projected_crs','EPSG','4093','EPSG','3631','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','4094','ETRS89 / DKTM2',NULL,'EPSG','4400','EPSG','4258','EPSG','4090',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2998','projected_crs','EPSG','4094','EPSG','3632','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','4095','ETRS89 / DKTM3',NULL,'EPSG','4400','EPSG','4258','EPSG','4091',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2999','projected_crs','EPSG','4095','EPSG','2532','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','4096','ETRS89 / DKTM4',NULL,'EPSG','4400','EPSG','4258','EPSG','4092',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3000','projected_crs','EPSG','4096','EPSG','2533','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','4217','NAD83 / BLM 59N (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','4186',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3099','projected_crs','EPSG','4217','EPSG','3372','EPSG','1212');
INSERT INTO "projected_crs" VALUES('EPSG','4390','Kertau 1968 / Johor Grid',NULL,'EPSG','4400','EPSG','4245','EPSG','4114',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3266','projected_crs','EPSG','4390','EPSG','3376','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','4391','Kertau 1968 / Sembilan and Melaka Grid',NULL,'EPSG','4400','EPSG','4245','EPSG','4115',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3267','projected_crs','EPSG','4391','EPSG','3377','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','4392','Kertau 1968 / Pahang Grid',NULL,'EPSG','4400','EPSG','4245','EPSG','4116',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3268','projected_crs','EPSG','4392','EPSG','3378','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','4393','Kertau 1968 / Selangor Grid',NULL,'EPSG','4400','EPSG','4245','EPSG','4117',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3269','projected_crs','EPSG','4393','EPSG','3379','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','4394','Kertau 1968 / Terengganu Grid',NULL,'EPSG','4400','EPSG','4245','EPSG','4177',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3270','projected_crs','EPSG','4394','EPSG','3380','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','4395','Kertau 1968 / Pinang Grid',NULL,'EPSG','4400','EPSG','4245','EPSG','4305',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3271','projected_crs','EPSG','4395','EPSG','3381','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','4396','Kertau 1968 / Kedah and Perlis Grid',NULL,'EPSG','4400','EPSG','4245','EPSG','4320',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3272','projected_crs','EPSG','4396','EPSG','3382','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','4397','Kertau 1968 / Perak Revised Grid',NULL,'EPSG','4400','EPSG','4245','EPSG','4321',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3273','projected_crs','EPSG','4397','EPSG','3383','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','4398','Kertau 1968 / Kelantan Grid',NULL,'EPSG','4400','EPSG','4245','EPSG','4323',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3274','projected_crs','EPSG','4398','EPSG','3384','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','4399','NAD27 / BLM 59N (ftUS)',NULL,'EPSG','4497','EPSG','4267','EPSG','4186',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3275','projected_crs','EPSG','4399','EPSG','3372','EPSG','1212');
INSERT INTO "projected_crs" VALUES('EPSG','4400','NAD27 / BLM 60N (ftUS)',NULL,'EPSG','4497','EPSG','4267','EPSG','4187',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3276','projected_crs','EPSG','4400','EPSG','3373','EPSG','1212');
INSERT INTO "projected_crs" VALUES('EPSG','4401','NAD27 / BLM 1N (ftUS)',NULL,'EPSG','4497','EPSG','4267','EPSG','4101',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3277','projected_crs','EPSG','4401','EPSG','3374','EPSG','1212');
INSERT INTO "projected_crs" VALUES('EPSG','4402','NAD27 / BLM 2N (ftUS)',NULL,'EPSG','4497','EPSG','4267','EPSG','4102',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3278','projected_crs','EPSG','4402','EPSG','3375','EPSG','1212');
INSERT INTO "projected_crs" VALUES('EPSG','4403','NAD27 / BLM 3N (ftUS)',NULL,'EPSG','4497','EPSG','4267','EPSG','4103',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3279','projected_crs','EPSG','4403','EPSG','2133','EPSG','1212');
INSERT INTO "projected_crs" VALUES('EPSG','4404','NAD27 / BLM 4N (ftUS)',NULL,'EPSG','4497','EPSG','4267','EPSG','4104',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3280','projected_crs','EPSG','4404','EPSG','2134','EPSG','1212');
INSERT INTO "projected_crs" VALUES('EPSG','4405','NAD27 / BLM 5N (ftUS)',NULL,'EPSG','4497','EPSG','4267','EPSG','4105',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3281','projected_crs','EPSG','4405','EPSG','2135','EPSG','1212');
INSERT INTO "projected_crs" VALUES('EPSG','4406','NAD27 / BLM 6N (ftUS)',NULL,'EPSG','4497','EPSG','4267','EPSG','4106',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3282','projected_crs','EPSG','4406','EPSG','2136','EPSG','1212');
INSERT INTO "projected_crs" VALUES('EPSG','4407','NAD27 / BLM 7N (ftUS)',NULL,'EPSG','4497','EPSG','4267','EPSG','4107',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3283','projected_crs','EPSG','4407','EPSG','3494','EPSG','1212');
INSERT INTO "projected_crs" VALUES('EPSG','4408','NAD27 / BLM 8N (ftUS)',NULL,'EPSG','4497','EPSG','4267','EPSG','4108',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3284','projected_crs','EPSG','4408','EPSG','3495','EPSG','1212');
INSERT INTO "projected_crs" VALUES('EPSG','4409','NAD27 / BLM 9N (ftUS)',NULL,'EPSG','4497','EPSG','4267','EPSG','4109',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3285','projected_crs','EPSG','4409','EPSG','3496','EPSG','1212');
INSERT INTO "projected_crs" VALUES('EPSG','4410','NAD27 / BLM 10N (ftUS)',NULL,'EPSG','4497','EPSG','4267','EPSG','4110',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3286','projected_crs','EPSG','4410','EPSG','3497','EPSG','1212');
INSERT INTO "projected_crs" VALUES('EPSG','4411','NAD27 / BLM 11N (ftUS)',NULL,'EPSG','4497','EPSG','4267','EPSG','4111',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3287','projected_crs','EPSG','4411','EPSG','3498','EPSG','1212');
INSERT INTO "projected_crs" VALUES('EPSG','4412','NAD27 / BLM 12N (ftUS)',NULL,'EPSG','4497','EPSG','4267','EPSG','4112',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3288','projected_crs','EPSG','4412','EPSG','3499','EPSG','1212');
INSERT INTO "projected_crs" VALUES('EPSG','4413','NAD27 / BLM 13N (ftUS)',NULL,'EPSG','4497','EPSG','4267','EPSG','4113',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3289','projected_crs','EPSG','4413','EPSG','3500','EPSG','1212');
INSERT INTO "projected_crs" VALUES('EPSG','4414','NAD83(HARN) / Guam Map Grid',NULL,'EPSG','4499','EPSG','4152','EPSG','4325',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3290','projected_crs','EPSG','4414','EPSG','3255','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','4415','Katanga 1955 / Katanga Lambert',NULL,'EPSG','4499','EPSG','4695','EPSG','4416',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3291','projected_crs','EPSG','4415','EPSG','3147','EPSG','1210');
INSERT INTO "projected_crs" VALUES('EPSG','4417','Pulkovo 1942(83) / 3-degree Gauss-Kruger zone 7',NULL,'EPSG','4530','EPSG','4178','EPSG','16267',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3292','projected_crs','EPSG','4417','EPSG','3584','EPSG','1160');
INSERT INTO "projected_crs" VALUES('EPSG','4418','NAD27 / BLM 18N (ftUS)',NULL,'EPSG','4497','EPSG','4267','EPSG','4118',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3293','projected_crs','EPSG','4418','EPSG','3505','EPSG','1212');
INSERT INTO "projected_crs" VALUES('EPSG','4419','NAD27 / BLM 19N (ftUS)',NULL,'EPSG','4497','EPSG','4267','EPSG','4119',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3294','projected_crs','EPSG','4419','EPSG','3506','EPSG','1212');
INSERT INTO "projected_crs" VALUES('EPSG','4420','NAD83 / BLM 60N (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','4187',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3295','projected_crs','EPSG','4420','EPSG','3373','EPSG','1212');
INSERT INTO "projected_crs" VALUES('EPSG','4421','NAD83 / BLM 1N (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','4101',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3296','projected_crs','EPSG','4421','EPSG','3374','EPSG','1212');
INSERT INTO "projected_crs" VALUES('EPSG','4422','NAD83 / BLM 2N (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','4102',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3297','projected_crs','EPSG','4422','EPSG','3375','EPSG','1212');
INSERT INTO "projected_crs" VALUES('EPSG','4423','NAD83 / BLM 3N (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','4103',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3298','projected_crs','EPSG','4423','EPSG','2133','EPSG','1212');
INSERT INTO "projected_crs" VALUES('EPSG','4424','NAD83 / BLM 4N (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','4104',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3299','projected_crs','EPSG','4424','EPSG','2134','EPSG','1212');
INSERT INTO "projected_crs" VALUES('EPSG','4425','NAD83 / BLM 5N (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','4105',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3300','projected_crs','EPSG','4425','EPSG','2135','EPSG','1212');
INSERT INTO "projected_crs" VALUES('EPSG','4426','NAD83 / BLM 6N (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','4106',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3301','projected_crs','EPSG','4426','EPSG','2136','EPSG','1212');
INSERT INTO "projected_crs" VALUES('EPSG','4427','NAD83 / BLM 7N (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','4107',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3302','projected_crs','EPSG','4427','EPSG','3494','EPSG','1212');
INSERT INTO "projected_crs" VALUES('EPSG','4428','NAD83 / BLM 8N (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','4108',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3303','projected_crs','EPSG','4428','EPSG','3495','EPSG','1212');
INSERT INTO "projected_crs" VALUES('EPSG','4429','NAD83 / BLM 9N (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','4109',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3304','projected_crs','EPSG','4429','EPSG','3496','EPSG','1212');
INSERT INTO "projected_crs" VALUES('EPSG','4430','NAD83 / BLM 10N (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','4110',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3305','projected_crs','EPSG','4430','EPSG','3497','EPSG','1212');
INSERT INTO "projected_crs" VALUES('EPSG','4431','NAD83 / BLM 11N (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','4111',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3306','projected_crs','EPSG','4431','EPSG','3498','EPSG','1212');
INSERT INTO "projected_crs" VALUES('EPSG','4432','NAD83 / BLM 12N (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','4112',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3307','projected_crs','EPSG','4432','EPSG','3499','EPSG','1212');
INSERT INTO "projected_crs" VALUES('EPSG','4433','NAD83 / BLM 13N (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','4113',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3308','projected_crs','EPSG','4433','EPSG','3500','EPSG','1212');
INSERT INTO "projected_crs" VALUES('EPSG','4434','Pulkovo 1942(83) / 3-degree Gauss-Kruger zone 8',NULL,'EPSG','4530','EPSG','4178','EPSG','16268',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3309','projected_crs','EPSG','4434','EPSG','3586','EPSG','1160');
INSERT INTO "projected_crs" VALUES('EPSG','4437','NAD83(NSRS2007) / Puerto Rico and Virgin Is.',NULL,'EPSG','4499','EPSG','4759','EPSG','15230',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3310','projected_crs','EPSG','4437','EPSG','3634','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','4438','NAD83 / BLM 18N (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','4118',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3311','projected_crs','EPSG','4438','EPSG','3505','EPSG','1212');
INSERT INTO "projected_crs" VALUES('EPSG','4439','NAD83 / BLM 19N (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','4119',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3312','projected_crs','EPSG','4439','EPSG','3506','EPSG','1212');
INSERT INTO "projected_crs" VALUES('EPSG','4455','NAD27 / Pennsylvania South',NULL,'EPSG','4497','EPSG','4267','EPSG','4436',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3314','projected_crs','EPSG','4455','EPSG','2246','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','4456','NAD27 / New York Long Island',NULL,'EPSG','4497','EPSG','4267','EPSG','4454',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3315','projected_crs','EPSG','4456','EPSG','2235','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','4457','NAD83 / South Dakota North (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15394',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3316','projected_crs','EPSG','4457','EPSG','2249','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','4462','WGS 84 / Australian Centre for Remote Sensing Lambert',NULL,'EPSG','4400','EPSG','4326','EPSG','4460',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3318','projected_crs','EPSG','4462','EPSG','2575','EPSG','1047');
INSERT INTO "projected_crs" VALUES('EPSG','4467','RGSPM06 / UTM zone 21N',NULL,'EPSG','4400','EPSG','4463','EPSG','16021',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3322','projected_crs','EPSG','4467','EPSG','1220','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','4471','RGM04 / UTM zone 38S',NULL,'EPSG','4400','EPSG','4470','EPSG','16138',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3326','projected_crs','EPSG','4471','EPSG','1159','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','4474','Cadastre 1997 / UTM zone 38S',NULL,'EPSG','4400','EPSG','4632','EPSG','16138',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3329','projected_crs','EPSG','4474','EPSG','3340','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','4484','Mexico ITRF92 / UTM zone 11N',NULL,'EPSG','4400','EPSG','4483','EPSG','16011',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3336','projected_crs','EPSG','4484','EPSG','3423','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','4485','Mexico ITRF92 / UTM zone 12N',NULL,'EPSG','4400','EPSG','4483','EPSG','16012',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3337','projected_crs','EPSG','4485','EPSG','3424','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','4486','Mexico ITRF92 / UTM zone 13N',NULL,'EPSG','4400','EPSG','4483','EPSG','16013',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3338','projected_crs','EPSG','4486','EPSG','3425','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','4487','Mexico ITRF92 / UTM zone 14N',NULL,'EPSG','4400','EPSG','4483','EPSG','16014',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3339','projected_crs','EPSG','4487','EPSG','3426','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','4488','Mexico ITRF92 / UTM zone 15N',NULL,'EPSG','4400','EPSG','4483','EPSG','16015',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3340','projected_crs','EPSG','4488','EPSG','3633','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','4489','Mexico ITRF92 / UTM zone 16N',NULL,'EPSG','4400','EPSG','4483','EPSG','16016',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3341','projected_crs','EPSG','4489','EPSG','3635','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','4491','CGCS2000 / Gauss-Kruger zone 13',NULL,'EPSG','4530','EPSG','4490','EPSG','16213',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3343','projected_crs','EPSG','4491','EPSG','1587','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','4492','CGCS2000 / Gauss-Kruger zone 14',NULL,'EPSG','4530','EPSG','4490','EPSG','16214',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3344','projected_crs','EPSG','4492','EPSG','1588','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','4493','CGCS2000 / Gauss-Kruger zone 15',NULL,'EPSG','4530','EPSG','4490','EPSG','16215',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3345','projected_crs','EPSG','4493','EPSG','1589','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','4494','CGCS2000 / Gauss-Kruger zone 16',NULL,'EPSG','4530','EPSG','4490','EPSG','16216',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3346','projected_crs','EPSG','4494','EPSG','1590','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','4495','CGCS2000 / Gauss-Kruger zone 17',NULL,'EPSG','4530','EPSG','4490','EPSG','16217',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3347','projected_crs','EPSG','4495','EPSG','1591','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','4496','CGCS2000 / Gauss-Kruger zone 18',NULL,'EPSG','4530','EPSG','4490','EPSG','16218',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3348','projected_crs','EPSG','4496','EPSG','3944','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','4497','CGCS2000 / Gauss-Kruger zone 19',NULL,'EPSG','4530','EPSG','4490','EPSG','16219',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3349','projected_crs','EPSG','4497','EPSG','3945','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','4498','CGCS2000 / Gauss-Kruger zone 20',NULL,'EPSG','4530','EPSG','4490','EPSG','16220',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3350','projected_crs','EPSG','4498','EPSG','3946','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','4499','CGCS2000 / Gauss-Kruger zone 21',NULL,'EPSG','4530','EPSG','4490','EPSG','16221',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3351','projected_crs','EPSG','4499','EPSG','3947','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','4500','CGCS2000 / Gauss-Kruger zone 22',NULL,'EPSG','4530','EPSG','4490','EPSG','16222',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3352','projected_crs','EPSG','4500','EPSG','3948','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','4501','CGCS2000 / Gauss-Kruger zone 23',NULL,'EPSG','4530','EPSG','4490','EPSG','16223',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3353','projected_crs','EPSG','4501','EPSG','1597','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','4502','CGCS2000 / Gauss-Kruger CM 75E',NULL,'EPSG','4530','EPSG','4490','EPSG','16313',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3354','projected_crs','EPSG','4502','EPSG','1587','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','4503','CGCS2000 / Gauss-Kruger CM 81E',NULL,'EPSG','4530','EPSG','4490','EPSG','16314',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3355','projected_crs','EPSG','4503','EPSG','1588','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','4504','CGCS2000 / Gauss-Kruger CM 87E',NULL,'EPSG','4530','EPSG','4490','EPSG','16315',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3356','projected_crs','EPSG','4504','EPSG','1589','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','4505','CGCS2000 / Gauss-Kruger CM 93E',NULL,'EPSG','4530','EPSG','4490','EPSG','16316',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3357','projected_crs','EPSG','4505','EPSG','1590','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','4506','CGCS2000 / Gauss-Kruger CM 99E',NULL,'EPSG','4530','EPSG','4490','EPSG','16317',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3358','projected_crs','EPSG','4506','EPSG','1591','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','4507','CGCS2000 / Gauss-Kruger CM 105E',NULL,'EPSG','4530','EPSG','4490','EPSG','16318',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3359','projected_crs','EPSG','4507','EPSG','3944','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','4508','CGCS2000 / Gauss-Kruger CM 111E',NULL,'EPSG','4530','EPSG','4490','EPSG','16319',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3360','projected_crs','EPSG','4508','EPSG','3945','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','4509','CGCS2000 / Gauss-Kruger CM 117E',NULL,'EPSG','4530','EPSG','4490','EPSG','16320',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3361','projected_crs','EPSG','4509','EPSG','3946','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','4510','CGCS2000 / Gauss-Kruger CM 123E',NULL,'EPSG','4530','EPSG','4490','EPSG','16321',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3362','projected_crs','EPSG','4510','EPSG','3947','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','4511','CGCS2000 / Gauss-Kruger CM 129E',NULL,'EPSG','4530','EPSG','4490','EPSG','16322',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3363','projected_crs','EPSG','4511','EPSG','3948','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','4512','CGCS2000 / Gauss-Kruger CM 135E',NULL,'EPSG','4530','EPSG','4490','EPSG','16323',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3364','projected_crs','EPSG','4512','EPSG','1597','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','4513','CGCS2000 / 3-degree Gauss-Kruger zone 25',NULL,'EPSG','4530','EPSG','4490','EPSG','16285',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3365','projected_crs','EPSG','4513','EPSG','2711','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4514','CGCS2000 / 3-degree Gauss-Kruger zone 26',NULL,'EPSG','4530','EPSG','4490','EPSG','16286',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3366','projected_crs','EPSG','4514','EPSG','2712','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4515','CGCS2000 / 3-degree Gauss-Kruger zone 27',NULL,'EPSG','4530','EPSG','4490','EPSG','16287',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3367','projected_crs','EPSG','4515','EPSG','2713','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4516','CGCS2000 / 3-degree Gauss-Kruger zone 28',NULL,'EPSG','4530','EPSG','4490','EPSG','16288',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3368','projected_crs','EPSG','4516','EPSG','2714','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4517','CGCS2000 / 3-degree Gauss-Kruger zone 29',NULL,'EPSG','4530','EPSG','4490','EPSG','16289',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3369','projected_crs','EPSG','4517','EPSG','2715','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4518','CGCS2000 / 3-degree Gauss-Kruger zone 30',NULL,'EPSG','4530','EPSG','4490','EPSG','16290',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3370','projected_crs','EPSG','4518','EPSG','2716','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4519','CGCS2000 / 3-degree Gauss-Kruger zone 31',NULL,'EPSG','4530','EPSG','4490','EPSG','16291',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3371','projected_crs','EPSG','4519','EPSG','2717','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4520','CGCS2000 / 3-degree Gauss-Kruger zone 32',NULL,'EPSG','4530','EPSG','4490','EPSG','16292',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3372','projected_crs','EPSG','4520','EPSG','2718','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4521','CGCS2000 / 3-degree Gauss-Kruger zone 33',NULL,'EPSG','4530','EPSG','4490','EPSG','16293',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3373','projected_crs','EPSG','4521','EPSG','2719','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4522','CGCS2000 / 3-degree Gauss-Kruger zone 34',NULL,'EPSG','4530','EPSG','4490','EPSG','16294',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3374','projected_crs','EPSG','4522','EPSG','2720','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4523','CGCS2000 / 3-degree Gauss-Kruger zone 35',NULL,'EPSG','4530','EPSG','4490','EPSG','16295',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3375','projected_crs','EPSG','4523','EPSG','2721','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4524','CGCS2000 / 3-degree Gauss-Kruger zone 36',NULL,'EPSG','4530','EPSG','4490','EPSG','16296',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3376','projected_crs','EPSG','4524','EPSG','2722','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4525','CGCS2000 / 3-degree Gauss-Kruger zone 37',NULL,'EPSG','4530','EPSG','4490','EPSG','16297',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3377','projected_crs','EPSG','4525','EPSG','2723','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4526','CGCS2000 / 3-degree Gauss-Kruger zone 38',NULL,'EPSG','4530','EPSG','4490','EPSG','16298',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3378','projected_crs','EPSG','4526','EPSG','2724','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4527','CGCS2000 / 3-degree Gauss-Kruger zone 39',NULL,'EPSG','4530','EPSG','4490','EPSG','16299',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3379','projected_crs','EPSG','4527','EPSG','2725','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4528','CGCS2000 / 3-degree Gauss-Kruger zone 40',NULL,'EPSG','4530','EPSG','4490','EPSG','16070',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3380','projected_crs','EPSG','4528','EPSG','2726','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4529','CGCS2000 / 3-degree Gauss-Kruger zone 41',NULL,'EPSG','4530','EPSG','4490','EPSG','16071',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3381','projected_crs','EPSG','4529','EPSG','2727','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4530','CGCS2000 / 3-degree Gauss-Kruger zone 42',NULL,'EPSG','4530','EPSG','4490','EPSG','16072',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3382','projected_crs','EPSG','4530','EPSG','2728','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4531','CGCS2000 / 3-degree Gauss-Kruger zone 43',NULL,'EPSG','4530','EPSG','4490','EPSG','16073',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3383','projected_crs','EPSG','4531','EPSG','2729','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4532','CGCS2000 / 3-degree Gauss-Kruger zone 44',NULL,'EPSG','4530','EPSG','4490','EPSG','16074',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3384','projected_crs','EPSG','4532','EPSG','2730','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4533','CGCS2000 / 3-degree Gauss-Kruger zone 45',NULL,'EPSG','4530','EPSG','4490','EPSG','16075',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3385','projected_crs','EPSG','4533','EPSG','2731','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4534','CGCS2000 / 3-degree Gauss-Kruger CM 75E',NULL,'EPSG','4530','EPSG','4490','EPSG','16313',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3386','projected_crs','EPSG','4534','EPSG','2711','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4535','CGCS2000 / 3-degree Gauss-Kruger CM 78E',NULL,'EPSG','4530','EPSG','4490','EPSG','16386',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3387','projected_crs','EPSG','4535','EPSG','2712','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4536','CGCS2000 / 3-degree Gauss-Kruger CM 81E',NULL,'EPSG','4530','EPSG','4490','EPSG','16314',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3388','projected_crs','EPSG','4536','EPSG','2713','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4537','CGCS2000 / 3-degree Gauss-Kruger CM 84E',NULL,'EPSG','4530','EPSG','4490','EPSG','16388',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3389','projected_crs','EPSG','4537','EPSG','2714','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4538','CGCS2000 / 3-degree Gauss-Kruger CM 87E',NULL,'EPSG','4530','EPSG','4490','EPSG','16315',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3390','projected_crs','EPSG','4538','EPSG','2715','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4539','CGCS2000 / 3-degree Gauss-Kruger CM 90E',NULL,'EPSG','4530','EPSG','4490','EPSG','16390',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3391','projected_crs','EPSG','4539','EPSG','2716','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4540','CGCS2000 / 3-degree Gauss-Kruger CM 93E',NULL,'EPSG','4530','EPSG','4490','EPSG','16316',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3392','projected_crs','EPSG','4540','EPSG','2717','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4541','CGCS2000 / 3-degree Gauss-Kruger CM 96E',NULL,'EPSG','4530','EPSG','4490','EPSG','16392',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3393','projected_crs','EPSG','4541','EPSG','2718','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4542','CGCS2000 / 3-degree Gauss-Kruger CM 99E',NULL,'EPSG','4530','EPSG','4490','EPSG','16317',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3394','projected_crs','EPSG','4542','EPSG','2719','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4543','CGCS2000 / 3-degree Gauss-Kruger CM 102E',NULL,'EPSG','4530','EPSG','4490','EPSG','16394',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3395','projected_crs','EPSG','4543','EPSG','2720','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4544','CGCS2000 / 3-degree Gauss-Kruger CM 105E',NULL,'EPSG','4530','EPSG','4490','EPSG','16318',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3396','projected_crs','EPSG','4544','EPSG','2721','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4545','CGCS2000 / 3-degree Gauss-Kruger CM 108E',NULL,'EPSG','4530','EPSG','4490','EPSG','16396',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3397','projected_crs','EPSG','4545','EPSG','2722','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4546','CGCS2000 / 3-degree Gauss-Kruger CM 111E',NULL,'EPSG','4530','EPSG','4490','EPSG','16319',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3398','projected_crs','EPSG','4546','EPSG','2723','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4547','CGCS2000 / 3-degree Gauss-Kruger CM 114E',NULL,'EPSG','4530','EPSG','4490','EPSG','16398',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3399','projected_crs','EPSG','4547','EPSG','2724','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4548','CGCS2000 / 3-degree Gauss-Kruger CM 117E',NULL,'EPSG','4530','EPSG','4490','EPSG','16320',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3400','projected_crs','EPSG','4548','EPSG','2725','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4549','CGCS2000 / 3-degree Gauss-Kruger CM 120E',NULL,'EPSG','4530','EPSG','4490','EPSG','16170',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3401','projected_crs','EPSG','4549','EPSG','2726','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4550','CGCS2000 / 3-degree Gauss-Kruger CM 123E',NULL,'EPSG','4530','EPSG','4490','EPSG','16321',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3402','projected_crs','EPSG','4550','EPSG','2727','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4551','CGCS2000 / 3-degree Gauss-Kruger CM 126E',NULL,'EPSG','4530','EPSG','4490','EPSG','16172',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3403','projected_crs','EPSG','4551','EPSG','2728','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4552','CGCS2000 / 3-degree Gauss-Kruger CM 129E',NULL,'EPSG','4530','EPSG','4490','EPSG','16322',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3404','projected_crs','EPSG','4552','EPSG','2729','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4553','CGCS2000 / 3-degree Gauss-Kruger CM 132E',NULL,'EPSG','4530','EPSG','4490','EPSG','16174',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3405','projected_crs','EPSG','4553','EPSG','2730','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4554','CGCS2000 / 3-degree Gauss-Kruger CM 135E',NULL,'EPSG','4530','EPSG','4490','EPSG','16323',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3406','projected_crs','EPSG','4554','EPSG','2731','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4559','RRAF 1991 / UTM zone 20N',NULL,'EPSG','4400','EPSG','4558','EPSG','16020',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3411','projected_crs','EPSG','4559','EPSG','3825','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','4568','New Beijing / Gauss-Kruger zone 13',NULL,'EPSG','4530','EPSG','4555','EPSG','16213',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3412','projected_crs','EPSG','4568','EPSG','1587','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','4569','New Beijing / Gauss-Kruger zone 14',NULL,'EPSG','4530','EPSG','4555','EPSG','16214',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3413','projected_crs','EPSG','4569','EPSG','1588','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','4570','New Beijing / Gauss-Kruger zone 15',NULL,'EPSG','4530','EPSG','4555','EPSG','16215',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3414','projected_crs','EPSG','4570','EPSG','1589','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','4571','New Beijing / Gauss-Kruger zone 16',NULL,'EPSG','4530','EPSG','4555','EPSG','16216',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3415','projected_crs','EPSG','4571','EPSG','1590','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','4572','New Beijing / Gauss-Kruger zone 17',NULL,'EPSG','4530','EPSG','4555','EPSG','16217',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3416','projected_crs','EPSG','4572','EPSG','1591','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','4573','New Beijing / Gauss-Kruger zone 18',NULL,'EPSG','4530','EPSG','4555','EPSG','16218',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3417','projected_crs','EPSG','4573','EPSG','1592','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','4574','New Beijing / Gauss-Kruger zone 19',NULL,'EPSG','4530','EPSG','4555','EPSG','16219',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3418','projected_crs','EPSG','4574','EPSG','1593','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','4575','New Beijing / Gauss-Kruger zone 20',NULL,'EPSG','4530','EPSG','4555','EPSG','16220',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3419','projected_crs','EPSG','4575','EPSG','1594','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','4576','New Beijing / Gauss-Kruger zone 21',NULL,'EPSG','4530','EPSG','4555','EPSG','16221',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3420','projected_crs','EPSG','4576','EPSG','1595','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','4577','New Beijing / Gauss-Kruger zone 22',NULL,'EPSG','4530','EPSG','4555','EPSG','16222',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3421','projected_crs','EPSG','4577','EPSG','1596','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','4578','New Beijing / Gauss-Kruger zone 23',NULL,'EPSG','4530','EPSG','4555','EPSG','16223',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3422','projected_crs','EPSG','4578','EPSG','1597','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','4579','New Beijing / Gauss-Kruger CM 75E',NULL,'EPSG','4530','EPSG','4555','EPSG','16313',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3423','projected_crs','EPSG','4579','EPSG','1587','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','4580','New Beijing / Gauss-Kruger CM 81E',NULL,'EPSG','4530','EPSG','4555','EPSG','16314',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3424','projected_crs','EPSG','4580','EPSG','1588','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','4581','New Beijing / Gauss-Kruger CM 87E',NULL,'EPSG','4530','EPSG','4555','EPSG','16315',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3425','projected_crs','EPSG','4581','EPSG','1589','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','4582','New Beijing / Gauss-Kruger CM 93E',NULL,'EPSG','4530','EPSG','4555','EPSG','16316',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3426','projected_crs','EPSG','4582','EPSG','1590','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','4583','New Beijing / Gauss-Kruger CM 99E',NULL,'EPSG','4530','EPSG','4555','EPSG','16317',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3427','projected_crs','EPSG','4583','EPSG','1591','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','4584','New Beijing / Gauss-Kruger CM 105E',NULL,'EPSG','4530','EPSG','4555','EPSG','16318',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3428','projected_crs','EPSG','4584','EPSG','1592','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','4585','New Beijing / Gauss-Kruger CM 111E',NULL,'EPSG','4530','EPSG','4555','EPSG','16319',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3429','projected_crs','EPSG','4585','EPSG','1593','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','4586','New Beijing / Gauss-Kruger CM 117E',NULL,'EPSG','4530','EPSG','4555','EPSG','16320',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3430','projected_crs','EPSG','4586','EPSG','1594','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','4587','New Beijing / Gauss-Kruger CM 123E',NULL,'EPSG','4530','EPSG','4555','EPSG','16321',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3431','projected_crs','EPSG','4587','EPSG','1595','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','4588','New Beijing / Gauss-Kruger CM 129E',NULL,'EPSG','4530','EPSG','4555','EPSG','16322',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3432','projected_crs','EPSG','4588','EPSG','1596','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','4589','New Beijing / Gauss-Kruger CM 135E',NULL,'EPSG','4530','EPSG','4555','EPSG','16323',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3433','projected_crs','EPSG','4589','EPSG','1597','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','4647','ETRS89 / UTM zone 32N (zE-N)',NULL,'EPSG','4400','EPSG','4258','EPSG','4648',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3481','projected_crs','EPSG','4647','EPSG','2861','EPSG','1153');
INSERT INTO "projected_crs" VALUES('EPSG','4652','New Beijing / 3-degree Gauss-Kruger zone 25',NULL,'EPSG','4530','EPSG','4555','EPSG','16285',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3482','projected_crs','EPSG','4652','EPSG','2711','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4653','New Beijing / 3-degree Gauss-Kruger zone 26',NULL,'EPSG','4530','EPSG','4555','EPSG','16286',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3483','projected_crs','EPSG','4653','EPSG','2712','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4654','New Beijing / 3-degree Gauss-Kruger zone 27',NULL,'EPSG','4530','EPSG','4555','EPSG','16287',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3484','projected_crs','EPSG','4654','EPSG','2713','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4655','New Beijing / 3-degree Gauss-Kruger zone 28',NULL,'EPSG','4530','EPSG','4555','EPSG','16288',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3485','projected_crs','EPSG','4655','EPSG','2714','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4656','New Beijing / 3-degree Gauss-Kruger zone 29',NULL,'EPSG','4530','EPSG','4555','EPSG','16289',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3486','projected_crs','EPSG','4656','EPSG','2715','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4766','New Beijing / 3-degree Gauss-Kruger zone 30',NULL,'EPSG','4530','EPSG','4555','EPSG','16290',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3596','projected_crs','EPSG','4766','EPSG','2716','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4767','New Beijing / 3-degree Gauss-Kruger zone 31',NULL,'EPSG','4530','EPSG','4555','EPSG','16291',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3597','projected_crs','EPSG','4767','EPSG','2717','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4768','New Beijing / 3-degree Gauss-Kruger zone 32',NULL,'EPSG','4530','EPSG','4555','EPSG','16292',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3598','projected_crs','EPSG','4768','EPSG','2718','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4769','New Beijing / 3-degree Gauss-Kruger zone 33',NULL,'EPSG','4530','EPSG','4555','EPSG','16293',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3599','projected_crs','EPSG','4769','EPSG','2719','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4770','New Beijing / 3-degree Gauss-Kruger zone 34',NULL,'EPSG','4530','EPSG','4555','EPSG','16294',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3600','projected_crs','EPSG','4770','EPSG','2720','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4771','New Beijing / 3-degree Gauss-Kruger zone 35',NULL,'EPSG','4530','EPSG','4555','EPSG','16295',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3601','projected_crs','EPSG','4771','EPSG','2721','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4772','New Beijing / 3-degree Gauss-Kruger zone 36',NULL,'EPSG','4530','EPSG','4555','EPSG','16296',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3602','projected_crs','EPSG','4772','EPSG','2722','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4773','New Beijing / 3-degree Gauss-Kruger zone 37',NULL,'EPSG','4530','EPSG','4555','EPSG','16297',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3603','projected_crs','EPSG','4773','EPSG','2723','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4774','New Beijing / 3-degree Gauss-Kruger zone 38',NULL,'EPSG','4530','EPSG','4555','EPSG','16298',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3604','projected_crs','EPSG','4774','EPSG','2724','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4775','New Beijing / 3-degree Gauss-Kruger zone 39',NULL,'EPSG','4530','EPSG','4555','EPSG','16299',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3605','projected_crs','EPSG','4775','EPSG','2725','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4776','New Beijing / 3-degree Gauss-Kruger zone 40',NULL,'EPSG','4530','EPSG','4555','EPSG','16070',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3606','projected_crs','EPSG','4776','EPSG','2726','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4777','New Beijing / 3-degree Gauss-Kruger zone 41',NULL,'EPSG','4530','EPSG','4555','EPSG','16071',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3607','projected_crs','EPSG','4777','EPSG','2727','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4778','New Beijing / 3-degree Gauss-Kruger zone 42',NULL,'EPSG','4530','EPSG','4555','EPSG','16072',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3608','projected_crs','EPSG','4778','EPSG','2728','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4779','New Beijing / 3-degree Gauss-Kruger zone 43',NULL,'EPSG','4530','EPSG','4555','EPSG','16073',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3609','projected_crs','EPSG','4779','EPSG','2729','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4780','New Beijing / 3-degree Gauss-Kruger zone 44',NULL,'EPSG','4530','EPSG','4555','EPSG','16074',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3610','projected_crs','EPSG','4780','EPSG','2730','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4781','New Beijing / 3-degree Gauss-Kruger zone 45',NULL,'EPSG','4530','EPSG','4555','EPSG','16075',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3611','projected_crs','EPSG','4781','EPSG','2731','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4782','New Beijing / 3-degree Gauss-Kruger CM 75E',NULL,'EPSG','4530','EPSG','4555','EPSG','16313',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3612','projected_crs','EPSG','4782','EPSG','2711','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4783','New Beijing / 3-degree Gauss-Kruger CM 78E',NULL,'EPSG','4530','EPSG','4555','EPSG','16386',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3613','projected_crs','EPSG','4783','EPSG','2712','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4784','New Beijing / 3-degree Gauss-Kruger CM 81E',NULL,'EPSG','4530','EPSG','4555','EPSG','16314',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3614','projected_crs','EPSG','4784','EPSG','2713','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4785','New Beijing / 3-degree Gauss-Kruger CM 84E',NULL,'EPSG','4530','EPSG','4555','EPSG','16388',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3615','projected_crs','EPSG','4785','EPSG','2714','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4786','New Beijing / 3-degree Gauss-Kruger CM 87E',NULL,'EPSG','4530','EPSG','4555','EPSG','16315',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3616','projected_crs','EPSG','4786','EPSG','2715','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4787','New Beijing / 3-degree Gauss-Kruger CM 90E',NULL,'EPSG','4530','EPSG','4555','EPSG','16390',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3617','projected_crs','EPSG','4787','EPSG','2716','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4788','New Beijing / 3-degree Gauss-Kruger CM 93E',NULL,'EPSG','4530','EPSG','4555','EPSG','16316',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3618','projected_crs','EPSG','4788','EPSG','2717','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4789','New Beijing / 3-degree Gauss-Kruger CM 96E',NULL,'EPSG','4530','EPSG','4555','EPSG','16392',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3619','projected_crs','EPSG','4789','EPSG','2718','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4790','New Beijing / 3-degree Gauss-Kruger CM 99E',NULL,'EPSG','4530','EPSG','4555','EPSG','16317',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3620','projected_crs','EPSG','4790','EPSG','2719','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4791','New Beijing / 3-degree Gauss-Kruger CM 102E',NULL,'EPSG','4530','EPSG','4555','EPSG','16394',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3621','projected_crs','EPSG','4791','EPSG','2720','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4792','New Beijing / 3-degree Gauss-Kruger CM 105E',NULL,'EPSG','4530','EPSG','4555','EPSG','16318',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3622','projected_crs','EPSG','4792','EPSG','2721','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4793','New Beijing / 3-degree Gauss-Kruger CM 108E',NULL,'EPSG','4530','EPSG','4555','EPSG','16396',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3623','projected_crs','EPSG','4793','EPSG','2722','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4794','New Beijing / 3-degree Gauss-Kruger CM 111E',NULL,'EPSG','4530','EPSG','4555','EPSG','16319',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3624','projected_crs','EPSG','4794','EPSG','2723','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4795','New Beijing / 3-degree Gauss-Kruger CM 114E',NULL,'EPSG','4530','EPSG','4555','EPSG','16398',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3625','projected_crs','EPSG','4795','EPSG','2724','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4796','New Beijing / 3-degree Gauss-Kruger CM 117E',NULL,'EPSG','4530','EPSG','4555','EPSG','16320',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3626','projected_crs','EPSG','4796','EPSG','2725','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4797','New Beijing / 3-degree Gauss-Kruger CM 120E',NULL,'EPSG','4530','EPSG','4555','EPSG','16170',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3627','projected_crs','EPSG','4797','EPSG','2726','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4798','New Beijing / 3-degree Gauss-Kruger CM 123E',NULL,'EPSG','4530','EPSG','4555','EPSG','16321',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3628','projected_crs','EPSG','4798','EPSG','2727','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4799','New Beijing / 3-degree Gauss-Kruger CM 126E',NULL,'EPSG','4530','EPSG','4555','EPSG','16172',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3629','projected_crs','EPSG','4799','EPSG','2728','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4800','New Beijing / 3-degree Gauss-Kruger CM 129E',NULL,'EPSG','4530','EPSG','4555','EPSG','16322',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3630','projected_crs','EPSG','4800','EPSG','2729','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4812','New Beijing / 3-degree Gauss-Kruger CM 132E',NULL,'EPSG','4530','EPSG','4555','EPSG','16174',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3642','projected_crs','EPSG','4812','EPSG','2730','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4822','New Beijing / 3-degree Gauss-Kruger CM 135E',NULL,'EPSG','4530','EPSG','4555','EPSG','16323',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3652','projected_crs','EPSG','4822','EPSG','2731','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','4826','WGS 84 / Cape Verde National',NULL,'EPSG','1024','EPSG','4326','EPSG','4825',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3655','projected_crs','EPSG','4826','EPSG','1062','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','4839','ETRS89 / LCC Germany (N-E)',NULL,'EPSG','4500','EPSG','4258','EPSG','4838',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3656','projected_crs','EPSG','4839','EPSG','3339','EPSG','1241');
INSERT INTO "projected_crs" VALUES('EPSG','4855','ETRS89 / NTM zone 5',NULL,'EPSG','4500','EPSG','4258','EPSG','4845',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3657','projected_crs','EPSG','4855','EPSG','3636','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','4856','ETRS89 / NTM zone 6',NULL,'EPSG','4500','EPSG','4258','EPSG','4846',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3658','projected_crs','EPSG','4856','EPSG','3639','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','4857','ETRS89 / NTM zone 7',NULL,'EPSG','4500','EPSG','4258','EPSG','4847',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3659','projected_crs','EPSG','4857','EPSG','3647','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','4858','ETRS89 / NTM zone 8',NULL,'EPSG','4500','EPSG','4258','EPSG','4848',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3660','projected_crs','EPSG','4858','EPSG','3648','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','4859','ETRS89 / NTM zone 9',NULL,'EPSG','4500','EPSG','4258','EPSG','4849',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3661','projected_crs','EPSG','4859','EPSG','3649','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','4860','ETRS89 / NTM zone 10',NULL,'EPSG','4500','EPSG','4258','EPSG','4850',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3662','projected_crs','EPSG','4860','EPSG','3650','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','4861','ETRS89 / NTM zone 11',NULL,'EPSG','4500','EPSG','4258','EPSG','4851',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3663','projected_crs','EPSG','4861','EPSG','3651','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','4862','ETRS89 / NTM zone 12',NULL,'EPSG','4500','EPSG','4258','EPSG','4852',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3664','projected_crs','EPSG','4862','EPSG','3653','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','4863','ETRS89 / NTM zone 13',NULL,'EPSG','4500','EPSG','4258','EPSG','4853',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3665','projected_crs','EPSG','4863','EPSG','3654','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','4864','ETRS89 / NTM zone 14',NULL,'EPSG','4500','EPSG','4258','EPSG','4854',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3666','projected_crs','EPSG','4864','EPSG','3655','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','4865','ETRS89 / NTM zone 15',NULL,'EPSG','4500','EPSG','4258','EPSG','4841',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3667','projected_crs','EPSG','4865','EPSG','3656','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','4866','ETRS89 / NTM zone 16',NULL,'EPSG','4500','EPSG','4258','EPSG','4842',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3668','projected_crs','EPSG','4866','EPSG','3657','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','4867','ETRS89 / NTM zone 17',NULL,'EPSG','4500','EPSG','4258','EPSG','4843',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3669','projected_crs','EPSG','4867','EPSG','3658','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','4868','ETRS89 / NTM zone 18',NULL,'EPSG','4500','EPSG','4258','EPSG','4844',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3670','projected_crs','EPSG','4868','EPSG','3660','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','4869','ETRS89 / NTM zone 19',NULL,'EPSG','4500','EPSG','4258','EPSG','4881',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3671','projected_crs','EPSG','4869','EPSG','3661','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','4870','ETRS89 / NTM zone 20',NULL,'EPSG','4500','EPSG','4258','EPSG','5000',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3672','projected_crs','EPSG','4870','EPSG','3662','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','4871','ETRS89 / NTM zone 21',NULL,'EPSG','4500','EPSG','4258','EPSG','5001',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3673','projected_crs','EPSG','4871','EPSG','3663','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','4872','ETRS89 / NTM zone 22',NULL,'EPSG','4500','EPSG','4258','EPSG','5002',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3674','projected_crs','EPSG','4872','EPSG','3665','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','4873','ETRS89 / NTM zone 23',NULL,'EPSG','4500','EPSG','4258','EPSG','5003',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3675','projected_crs','EPSG','4873','EPSG','3667','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','4874','ETRS89 / NTM zone 24',NULL,'EPSG','4500','EPSG','4258','EPSG','5004',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3676','projected_crs','EPSG','4874','EPSG','3668','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','4875','ETRS89 / NTM zone 25',NULL,'EPSG','4500','EPSG','4258','EPSG','5005',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3677','projected_crs','EPSG','4875','EPSG','3669','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','4876','ETRS89 / NTM zone 26',NULL,'EPSG','4500','EPSG','4258','EPSG','5006',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3678','projected_crs','EPSG','4876','EPSG','3671','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','4877','ETRS89 / NTM zone 27',NULL,'EPSG','4500','EPSG','4258','EPSG','5007',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3679','projected_crs','EPSG','4877','EPSG','3672','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','4878','ETRS89 / NTM zone 28',NULL,'EPSG','4500','EPSG','4258','EPSG','5008',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3680','projected_crs','EPSG','4878','EPSG','3673','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','4879','ETRS89 / NTM zone 29',NULL,'EPSG','4500','EPSG','4258','EPSG','5009',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3681','projected_crs','EPSG','4879','EPSG','3674','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','4880','ETRS89 / NTM zone 30',NULL,'EPSG','4500','EPSG','4258','EPSG','5010',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3682','projected_crs','EPSG','4880','EPSG','3676','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','5014','PTRA08 / UTM zone 25N',NULL,'EPSG','4400','EPSG','5013','EPSG','16025',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3803','projected_crs','EPSG','5014','EPSG','3682','EPSG','1153');
INSERT INTO "projected_crs" VALUES('EPSG','5015','PTRA08 / UTM zone 26N',NULL,'EPSG','4400','EPSG','5013','EPSG','16026',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3804','projected_crs','EPSG','5015','EPSG','3677','EPSG','1153');
INSERT INTO "projected_crs" VALUES('EPSG','5016','PTRA08 / UTM zone 28N',NULL,'EPSG','4400','EPSG','5013','EPSG','16028',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3805','projected_crs','EPSG','5016','EPSG','3678','EPSG','1153');
INSERT INTO "projected_crs" VALUES('EPSG','5017','Lisbon 1890 / Portugal Bonne New',NULL,'EPSG','6509','EPSG','4666','EPSG','5019',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3806','projected_crs','EPSG','5017','EPSG','1294','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','5018','Lisbon / Portuguese Grid New',NULL,'EPSG','4499','EPSG','4207','EPSG','5020',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3807','projected_crs','EPSG','5018','EPSG','1294','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','5041','WGS 84 / UPS North (E,N)',NULL,'EPSG','1026','EPSG','4326','EPSG','16061',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3808','projected_crs','EPSG','5041','EPSG','1996','EPSG','1160');
INSERT INTO "projected_crs" VALUES('EPSG','5042','WGS 84 / UPS South (E,N)',NULL,'EPSG','1027','EPSG','4326','EPSG','16161',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3809','projected_crs','EPSG','5042','EPSG','1997','EPSG','1160');
INSERT INTO "projected_crs" VALUES('EPSG','5048','EUREF-FIN / TM35FIN(N,E)',NULL,'EPSG','4500','EPSG','10690','EPSG','16065',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3810','projected_crs','EPSG','5048','EPSG','1095','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','5069','NAD27 / Conus Albers',NULL,'EPSG','4499','EPSG','4267','EPSG','5068',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3811','projected_crs','EPSG','5069','EPSG','1323','EPSG','1109');
INSERT INTO "projected_crs" VALUES('EPSG','5070','NAD83 / Conus Albers',NULL,'EPSG','4499','EPSG','4269','EPSG','5068',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3812','projected_crs','EPSG','5070','EPSG','1323','EPSG','1109');
INSERT INTO "projected_crs" VALUES('EPSG','5071','NAD83(HARN) / Conus Albers',NULL,'EPSG','4499','EPSG','4152','EPSG','5068',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3813','projected_crs','EPSG','5071','EPSG','1323','EPSG','1109');
INSERT INTO "projected_crs" VALUES('EPSG','5072','NAD83(NSRS2007) / Conus Albers',NULL,'EPSG','4499','EPSG','4759','EPSG','5068',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3814','projected_crs','EPSG','5072','EPSG','1323','EPSG','1109');
INSERT INTO "projected_crs" VALUES('EPSG','5105','ETRS89-NOR [EUREF89] / NTM zone 5',NULL,'EPSG','4500','EPSG','10875','EPSG','5135',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3815','projected_crs','EPSG','5105','EPSG','3636','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','5106','ETRS89-NOR [EUREF89] / NTM zone 6',NULL,'EPSG','4500','EPSG','10875','EPSG','5136',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3816','projected_crs','EPSG','5106','EPSG','3639','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','5107','ETRS89-NOR [EUREF89] / NTM zone 7',NULL,'EPSG','4500','EPSG','10875','EPSG','5137',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3817','projected_crs','EPSG','5107','EPSG','3647','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','5108','ETRS89-NOR [EUREF89] / NTM zone 8',NULL,'EPSG','4500','EPSG','10875','EPSG','5138',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3818','projected_crs','EPSG','5108','EPSG','3648','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','5109','ETRS89-NOR [EUREF89] / NTM zone 9',NULL,'EPSG','4500','EPSG','10875','EPSG','5139',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3819','projected_crs','EPSG','5109','EPSG','3649','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','5110','ETRS89-NOR [EUREF89] / NTM zone 10',NULL,'EPSG','4500','EPSG','10875','EPSG','5140',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3820','projected_crs','EPSG','5110','EPSG','3650','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','5111','ETRS89-NOR [EUREF89] / NTM zone 11',NULL,'EPSG','4500','EPSG','10875','EPSG','5141',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3821','projected_crs','EPSG','5111','EPSG','3651','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','5112','ETRS89-NOR [EUREF89] / NTM zone 12',NULL,'EPSG','4500','EPSG','10875','EPSG','5142',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3822','projected_crs','EPSG','5112','EPSG','3653','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','5113','ETRS89-NOR [EUREF89] / NTM zone 13',NULL,'EPSG','4500','EPSG','10875','EPSG','5143',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3823','projected_crs','EPSG','5113','EPSG','3654','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','5114','ETRS89-NOR [EUREF89] / NTM zone 14',NULL,'EPSG','4500','EPSG','10875','EPSG','5144',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3824','projected_crs','EPSG','5114','EPSG','3655','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','5115','ETRS89-NOR [EUREF89] / NTM zone 15',NULL,'EPSG','4500','EPSG','10875','EPSG','5145',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3825','projected_crs','EPSG','5115','EPSG','3656','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','5116','ETRS89-NOR [EUREF89] / NTM zone 16',NULL,'EPSG','4500','EPSG','10875','EPSG','5146',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3826','projected_crs','EPSG','5116','EPSG','3657','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','5117','ETRS89-NOR [EUREF89] / NTM zone 17',NULL,'EPSG','4500','EPSG','10875','EPSG','5147',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3827','projected_crs','EPSG','5117','EPSG','3658','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','5118','ETRS89-NOR [EUREF89] / NTM zone 18',NULL,'EPSG','4500','EPSG','10875','EPSG','5148',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3828','projected_crs','EPSG','5118','EPSG','3660','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','5119','ETRS89-NOR [EUREF89] / NTM zone 19',NULL,'EPSG','4500','EPSG','10875','EPSG','5149',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3829','projected_crs','EPSG','5119','EPSG','3661','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','5120','ETRS89-NOR [EUREF89] / NTM zone 20',NULL,'EPSG','4500','EPSG','10875','EPSG','5150',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3830','projected_crs','EPSG','5120','EPSG','3662','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','5121','ETRS89-NOR [EUREF89] / NTM zone 21',NULL,'EPSG','4500','EPSG','10875','EPSG','5151',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3831','projected_crs','EPSG','5121','EPSG','3663','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','5122','ETRS89-NOR [EUREF89] / NTM zone 22',NULL,'EPSG','4500','EPSG','10875','EPSG','5152',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3832','projected_crs','EPSG','5122','EPSG','3665','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','5123','ETRS89-NOR [EUREF89] / NTM zone 23',NULL,'EPSG','4500','EPSG','10875','EPSG','5153',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3833','projected_crs','EPSG','5123','EPSG','3667','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','5124','ETRS89-NOR [EUREF89] / NTM zone 24',NULL,'EPSG','4500','EPSG','10875','EPSG','5154',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3834','projected_crs','EPSG','5124','EPSG','3668','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','5125','ETRS89-NOR [EUREF89] / NTM zone 25',NULL,'EPSG','4500','EPSG','10875','EPSG','5155',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3835','projected_crs','EPSG','5125','EPSG','3669','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','5126','ETRS89-NOR [EUREF89] / NTM zone 26',NULL,'EPSG','4500','EPSG','10875','EPSG','5156',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3836','projected_crs','EPSG','5126','EPSG','3671','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','5127','ETRS89-NOR [EUREF89] / NTM zone 27',NULL,'EPSG','4500','EPSG','10875','EPSG','5157',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3837','projected_crs','EPSG','5127','EPSG','3672','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','5128','ETRS89-NOR [EUREF89] / NTM zone 28',NULL,'EPSG','4500','EPSG','10875','EPSG','5158',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3838','projected_crs','EPSG','5128','EPSG','3673','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','5129','ETRS89-NOR [EUREF89] / NTM zone 29',NULL,'EPSG','4500','EPSG','10875','EPSG','5159',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3839','projected_crs','EPSG','5129','EPSG','3674','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','5130','ETRS89-NOR [EUREF89] / NTM zone 30',NULL,'EPSG','4500','EPSG','10875','EPSG','5160',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3840','projected_crs','EPSG','5130','EPSG','3676','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','5167','Korean 1985 / East Sea Belt',NULL,'EPSG','4530','EPSG','4162','EPSG','5049',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3842','projected_crs','EPSG','5167','EPSG','3720','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','5168','Korean 1985 / Central Belt Jeju',NULL,'EPSG','4530','EPSG','4162','EPSG','5131',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3843','projected_crs','EPSG','5168','EPSG','3721','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','5169','Tokyo 1892 / Korea West Belt',NULL,'EPSG','4530','EPSG','5132','EPSG','18253',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3844','projected_crs','EPSG','5169','EPSG','3713','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','5170','Tokyo 1892 / Korea Central Belt',NULL,'EPSG','4530','EPSG','5132','EPSG','18252',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3845','projected_crs','EPSG','5170','EPSG','3716','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','5171','Tokyo 1892 / Korea East Belt',NULL,'EPSG','4530','EPSG','5132','EPSG','18251',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3846','projected_crs','EPSG','5171','EPSG','3726','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','5172','Tokyo 1892 / Korea East Sea Belt',NULL,'EPSG','4530','EPSG','5132','EPSG','5049',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3847','projected_crs','EPSG','5172','EPSG','3727','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','5173','Korean 1985 / Modified West Belt',NULL,'EPSG','4530','EPSG','4162','EPSG','5161',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3848','projected_crs','EPSG','5173','EPSG','1498','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','5174','Korean 1985 / Modified Central Belt',NULL,'EPSG','4530','EPSG','4162','EPSG','5162',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3849','projected_crs','EPSG','5174','EPSG','3730','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','5175','Korean 1985 / Modified Central Belt Jeju',NULL,'EPSG','4530','EPSG','4162','EPSG','5163',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3850','projected_crs','EPSG','5175','EPSG','3721','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','5176','Korean 1985 / Modified East Belt',NULL,'EPSG','4530','EPSG','4162','EPSG','5164',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3851','projected_crs','EPSG','5176','EPSG','1496','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','5177','Korean 1985 / Modified East Sea Belt',NULL,'EPSG','4530','EPSG','4162','EPSG','5165',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3852','projected_crs','EPSG','5177','EPSG','3720','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','5178','Korean 1985 / Unified CS',NULL,'EPSG','4530','EPSG','4162','EPSG','5100',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3853','projected_crs','EPSG','5178','EPSG','3266','EPSG','1241');
INSERT INTO "projected_crs" VALUES('EPSG','5179','KGD2002 / Unified CS',NULL,'EPSG','4530','EPSG','4737','EPSG','5100',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3854','projected_crs','EPSG','5179','EPSG','1135','EPSG','1241');
INSERT INTO "projected_crs" VALUES('EPSG','5180','KGD2002 / West Belt',NULL,'EPSG','4530','EPSG','4737','EPSG','18253',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3855','projected_crs','EPSG','5180','EPSG','1498','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','5181','KGD2002 / Central Belt',NULL,'EPSG','4530','EPSG','4737','EPSG','18252',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3856','projected_crs','EPSG','5181','EPSG','3730','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','5182','KGD2002 / Central Belt Jeju',NULL,'EPSG','4530','EPSG','4737','EPSG','5131',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3857','projected_crs','EPSG','5182','EPSG','3721','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','5183','KGD2002 / East Belt',NULL,'EPSG','4530','EPSG','4737','EPSG','18251',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3858','projected_crs','EPSG','5183','EPSG','1496','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','5184','KGD2002 / East Sea Belt',NULL,'EPSG','4530','EPSG','4737','EPSG','5049',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3859','projected_crs','EPSG','5184','EPSG','3720','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','5185','KGD2002 / West Belt 2010',NULL,'EPSG','4530','EPSG','4737','EPSG','5101',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3860','projected_crs','EPSG','5185','EPSG','1498','EPSG','1091');
INSERT INTO "projected_crs" VALUES('EPSG','5186','KGD2002 / Central Belt 2010',NULL,'EPSG','4530','EPSG','4737','EPSG','5102',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3861','projected_crs','EPSG','5186','EPSG','1497','EPSG','1091');
INSERT INTO "projected_crs" VALUES('EPSG','5187','KGD2002 / East Belt 2010',NULL,'EPSG','4530','EPSG','4737','EPSG','5103',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3862','projected_crs','EPSG','5187','EPSG','1496','EPSG','1091');
INSERT INTO "projected_crs" VALUES('EPSG','5188','KGD2002 / East Sea Belt 2010',NULL,'EPSG','4530','EPSG','4737','EPSG','5104',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3863','projected_crs','EPSG','5188','EPSG','3720','EPSG','1091');
INSERT INTO "projected_crs" VALUES('EPSG','5221','S-JTSK (Ferro) / Krovak East North',NULL,'EPSG','4499','EPSG','4818','EPSG','5218',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3867','projected_crs','EPSG','5221','EPSG','1306','EPSG','1189');
INSERT INTO "projected_crs" VALUES('EPSG','5223','WGS 84 / Gabon TM',NULL,'EPSG','4499','EPSG','4326','EPSG','5222',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3868','projected_crs','EPSG','5223','EPSG','3249','EPSG','1165');
INSERT INTO "projected_crs" VALUES('EPSG','5224','S-JTSK/05 (Ferro) / Modified Krovak',NULL,'EPSG','6501','EPSG','5229','EPSG','5219',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3869','projected_crs','EPSG','5224','EPSG','1079','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','5225','S-JTSK/05 (Ferro) / Modified Krovak East North',NULL,'EPSG','4499','EPSG','5229','EPSG','5220',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3870','projected_crs','EPSG','5225','EPSG','1079','EPSG','1189');
INSERT INTO "projected_crs" VALUES('EPSG','5234','Kandawala / Sri Lanka Grid',NULL,'EPSG','4400','EPSG','4244','EPSG','5231',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3874','projected_crs','EPSG','5234','EPSG','3310','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','5235','SLD99 / Sri Lanka Grid 1999',NULL,'EPSG','4400','EPSG','5233','EPSG','5232',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3875','projected_crs','EPSG','5235','EPSG','3310','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','5243','ETRS89 / LCC Germany (E-N)',NULL,'EPSG','4400','EPSG','4258','EPSG','4838',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3877','projected_crs','EPSG','5243','EPSG','3339','EPSG','1241');
INSERT INTO "projected_crs" VALUES('EPSG','5247','GDBD2009 / Brunei BRSO',NULL,'EPSG','4400','EPSG','5246','EPSG','19894',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3881','projected_crs','EPSG','5247','EPSG','1055','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','5253','TUREF / TM27',NULL,'EPSG','4530','EPSG','5252','EPSG','16305',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3885','projected_crs','EPSG','5253','EPSG','1524','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','5254','TUREF / TM30',NULL,'EPSG','4530','EPSG','5252','EPSG','16370',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3886','projected_crs','EPSG','5254','EPSG','1525','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','5255','TUREF / TM33',NULL,'EPSG','4530','EPSG','5252','EPSG','16306',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3887','projected_crs','EPSG','5255','EPSG','1526','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','5256','TUREF / TM36',NULL,'EPSG','4530','EPSG','5252','EPSG','16372',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3888','projected_crs','EPSG','5256','EPSG','1527','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','5257','TUREF / TM39',NULL,'EPSG','4530','EPSG','5252','EPSG','16307',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3889','projected_crs','EPSG','5257','EPSG','1528','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','5258','TUREF / TM42',NULL,'EPSG','4530','EPSG','5252','EPSG','16374',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3890','projected_crs','EPSG','5258','EPSG','1529','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','5259','TUREF / TM45',NULL,'EPSG','4530','EPSG','5252','EPSG','16308',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3891','projected_crs','EPSG','5259','EPSG','1530','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','5266','DRUKREF 03 / Bhutan National Grid',NULL,'EPSG','4400','EPSG','5264','EPSG','5265',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3895','projected_crs','EPSG','5266','EPSG','1048','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','5269','TUREF / 3-degree Gauss-Kruger zone 9',NULL,'EPSG','4530','EPSG','5252','EPSG','16269',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3896','projected_crs','EPSG','5269','EPSG','1524','EPSG','1207');
INSERT INTO "projected_crs" VALUES('EPSG','5270','TUREF / 3-degree Gauss-Kruger zone 10',NULL,'EPSG','4530','EPSG','5252','EPSG','16270',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3897','projected_crs','EPSG','5270','EPSG','1525','EPSG','1207');
INSERT INTO "projected_crs" VALUES('EPSG','5271','TUREF / 3-degree Gauss-Kruger zone 11',NULL,'EPSG','4530','EPSG','5252','EPSG','16271',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3898','projected_crs','EPSG','5271','EPSG','1526','EPSG','1207');
INSERT INTO "projected_crs" VALUES('EPSG','5272','TUREF / 3-degree Gauss-Kruger zone 12',NULL,'EPSG','4530','EPSG','5252','EPSG','16272',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3899','projected_crs','EPSG','5272','EPSG','1527','EPSG','1207');
INSERT INTO "projected_crs" VALUES('EPSG','5273','TUREF / 3-degree Gauss-Kruger zone 13',NULL,'EPSG','4530','EPSG','5252','EPSG','16273',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3900','projected_crs','EPSG','5273','EPSG','1528','EPSG','1207');
INSERT INTO "projected_crs" VALUES('EPSG','5274','TUREF / 3-degree Gauss-Kruger zone 14',NULL,'EPSG','4530','EPSG','5252','EPSG','16274',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3901','projected_crs','EPSG','5274','EPSG','1529','EPSG','1207');
INSERT INTO "projected_crs" VALUES('EPSG','5275','TUREF / 3-degree Gauss-Kruger zone 15',NULL,'EPSG','4530','EPSG','5252','EPSG','16275',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3902','projected_crs','EPSG','5275','EPSG','1530','EPSG','1207');
INSERT INTO "projected_crs" VALUES('EPSG','5292','DRUKREF 03 / Bumthang TM',NULL,'EPSG','4400','EPSG','5264','EPSG','5268',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3903','projected_crs','EPSG','5292','EPSG','3734','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','5293','DRUKREF 03 / Chhukha TM',NULL,'EPSG','4400','EPSG','5264','EPSG','5276',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3904','projected_crs','EPSG','5293','EPSG','3737','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','5294','DRUKREF 03 / Dagana TM',NULL,'EPSG','4400','EPSG','5264','EPSG','5277',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3905','projected_crs','EPSG','5294','EPSG','3738','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','5295','DRUKREF 03 / Gasa TM',NULL,'EPSG','4400','EPSG','5264','EPSG','5278',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3906','projected_crs','EPSG','5295','EPSG','3740','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','5296','DRUKREF 03 / Ha TM',NULL,'EPSG','4400','EPSG','5264','EPSG','5279',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3907','projected_crs','EPSG','5296','EPSG','3742','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','5297','DRUKREF 03 / Lhuentse TM',NULL,'EPSG','4400','EPSG','5264','EPSG','5280',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3908','projected_crs','EPSG','5297','EPSG','3743','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','5298','DRUKREF 03 / Mongar TM',NULL,'EPSG','4400','EPSG','5264','EPSG','5281',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3909','projected_crs','EPSG','5298','EPSG','3745','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','5299','DRUKREF 03 / Paro TM',NULL,'EPSG','4400','EPSG','5264','EPSG','5282',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3910','projected_crs','EPSG','5299','EPSG','3746','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','5300','DRUKREF 03 / Pemagatshel TM',NULL,'EPSG','4400','EPSG','5264','EPSG','5283',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3911','projected_crs','EPSG','5300','EPSG','3747','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','5301','DRUKREF 03 / Punakha TM',NULL,'EPSG','4400','EPSG','5264','EPSG','5313',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3912','projected_crs','EPSG','5301','EPSG','3749','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','5302','DRUKREF 03 / Samdrup Jongkhar TM',NULL,'EPSG','4400','EPSG','5264','EPSG','5285',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3913','projected_crs','EPSG','5302','EPSG','3750','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','5303','DRUKREF 03 / Samtse TM',NULL,'EPSG','4400','EPSG','5264','EPSG','5286',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3914','projected_crs','EPSG','5303','EPSG','3751','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','5304','DRUKREF 03 / Sarpang TM',NULL,'EPSG','4400','EPSG','5264','EPSG','5287',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3915','projected_crs','EPSG','5304','EPSG','3752','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','5305','DRUKREF 03 / Thimphu TM',NULL,'EPSG','4400','EPSG','5264','EPSG','5312',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3916','projected_crs','EPSG','5305','EPSG','3753','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','5306','DRUKREF 03 / Trashigang TM',NULL,'EPSG','4400','EPSG','5264','EPSG','5289',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3917','projected_crs','EPSG','5306','EPSG','3754','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','5307','DRUKREF 03 / Trongsa TM',NULL,'EPSG','4400','EPSG','5264','EPSG','5290',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3918','projected_crs','EPSG','5307','EPSG','3755','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','5308','DRUKREF 03 / Tsirang TM',NULL,'EPSG','4400','EPSG','5264','EPSG','5284',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3919','projected_crs','EPSG','5308','EPSG','3757','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','5309','DRUKREF 03 / Wangdue Phodrang TM',NULL,'EPSG','4400','EPSG','5264','EPSG','5288',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3920','projected_crs','EPSG','5309','EPSG','3758','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','5310','DRUKREF 03 / Yangtse TM',NULL,'EPSG','4400','EPSG','5264','EPSG','5314',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3921','projected_crs','EPSG','5310','EPSG','3760','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','5311','DRUKREF 03 / Zhemgang TM',NULL,'EPSG','4400','EPSG','5264','EPSG','5291',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3922','projected_crs','EPSG','5311','EPSG','3761','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','5316','ETRS89 / Faroe TM',NULL,'EPSG','4400','EPSG','4258','EPSG','5315',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3923','projected_crs','EPSG','5316','EPSG','1093','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','5320','NAD83 / Teranet Ontario Lambert',NULL,'EPSG','4499','EPSG','4269','EPSG','5319',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3926','projected_crs','EPSG','5320','EPSG','1367','EPSG','1220');
INSERT INTO "projected_crs" VALUES('EPSG','5321','NAD83(CSRS) / Teranet Ontario Lambert',NULL,'EPSG','4499','EPSG','4617','EPSG','5319',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3927','projected_crs','EPSG','5321','EPSG','1367','EPSG','1220');
INSERT INTO "projected_crs" VALUES('EPSG','5325','ISN2004 / Lambert 2004',NULL,'EPSG','4499','EPSG','5324','EPSG','5326',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3931','projected_crs','EPSG','5325','EPSG','1120','EPSG','1210');
INSERT INTO "projected_crs" VALUES('EPSG','5329','Segara (Jakarta) / NEIEZ',NULL,'EPSG','4499','EPSG','4820','EPSG','5328',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3932','projected_crs','EPSG','5329','EPSG','1360','EPSG','1027');
INSERT INTO "projected_crs" VALUES('EPSG','5330','Batavia (Jakarta) / NEIEZ',NULL,'EPSG','4499','EPSG','4813','EPSG','5328',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3933','projected_crs','EPSG','5330','EPSG','1285','EPSG','1027');
INSERT INTO "projected_crs" VALUES('EPSG','5331','Makassar (Jakarta) / NEIEZ',NULL,'EPSG','4499','EPSG','4804','EPSG','5328',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3934','projected_crs','EPSG','5331','EPSG','1316','EPSG','1027');
INSERT INTO "projected_crs" VALUES('EPSG','5337','Aratu / UTM zone 25S',NULL,'EPSG','4400','EPSG','4208','EPSG','16125',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3937','projected_crs','EPSG','5337','EPSG','3808','EPSG','1136');
INSERT INTO "projected_crs" VALUES('EPSG','5343','POSGAR 2007 / Argentina 1',NULL,'EPSG','4530','EPSG','5340','EPSG','18031',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3941','projected_crs','EPSG','5343','EPSG','1608','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','5344','POSGAR 2007 / Argentina 2',NULL,'EPSG','4530','EPSG','5340','EPSG','18032',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3942','projected_crs','EPSG','5344','EPSG','1609','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','5345','POSGAR 2007 / Argentina 3',NULL,'EPSG','4530','EPSG','5340','EPSG','18033',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3943','projected_crs','EPSG','5345','EPSG','1610','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','5346','POSGAR 2007 / Argentina 4',NULL,'EPSG','4530','EPSG','5340','EPSG','18034',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3944','projected_crs','EPSG','5346','EPSG','1611','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','5347','POSGAR 2007 / Argentina 5',NULL,'EPSG','4530','EPSG','5340','EPSG','18035',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3945','projected_crs','EPSG','5347','EPSG','1612','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','5348','POSGAR 2007 / Argentina 6',NULL,'EPSG','4530','EPSG','5340','EPSG','18036',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3946','projected_crs','EPSG','5348','EPSG','1613','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','5349','POSGAR 2007 / Argentina 7',NULL,'EPSG','4530','EPSG','5340','EPSG','18037',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3947','projected_crs','EPSG','5349','EPSG','1614','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','5355','MARGEN / UTM zone 20S',NULL,'EPSG','4400','EPSG','5354','EPSG','16120',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3951','projected_crs','EPSG','5355','EPSG','1761','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','5356','MARGEN / UTM zone 19S',NULL,'EPSG','4400','EPSG','5354','EPSG','16119',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3952','projected_crs','EPSG','5356','EPSG','3827','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','5357','MARGEN / UTM zone 21S',NULL,'EPSG','4400','EPSG','5354','EPSG','16121',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3953','projected_crs','EPSG','5357','EPSG','3733','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','5361','SIRGAS-Chile 2002 / UTM zone 19S',NULL,'EPSG','4400','EPSG','5360','EPSG','16119',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3957','projected_crs','EPSG','5361','EPSG','3811','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','5362','SIRGAS-Chile 2002 / UTM zone 18S',NULL,'EPSG','4400','EPSG','5360','EPSG','16118',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3958','projected_crs','EPSG','5362','EPSG','3829','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','5367','CR05 / CRTM05',NULL,'EPSG','4500','EPSG','5365','EPSG','5366',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3962','projected_crs','EPSG','5367','EPSG','3849','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','5382','SIRGAS-ROU98 / UTM zone 21S',NULL,'EPSG','4400','EPSG','5381','EPSG','16121',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3972','projected_crs','EPSG','5382','EPSG','3826','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','5383','SIRGAS-ROU98 / UTM zone 22S',NULL,'EPSG','4400','EPSG','5381','EPSG','16122',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3973','projected_crs','EPSG','5383','EPSG','3828','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','5387','Peru96 / UTM zone 18S',NULL,'EPSG','4400','EPSG','5373','EPSG','16118',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3974','projected_crs','EPSG','5387','EPSG','3838','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','5388','Peru96 / UTM zone 17S',NULL,'EPSG','4400','EPSG','5373','EPSG','16017',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3975','projected_crs','EPSG','5388','EPSG','3837','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','5389','Peru96 / UTM zone 19S',NULL,'EPSG','4400','EPSG','5373','EPSG','16119',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3976','projected_crs','EPSG','5389','EPSG','3836','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','5396','SIRGAS 2000 / UTM zone 26S',NULL,'EPSG','4400','EPSG','4674','EPSG','16126',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3980','projected_crs','EPSG','5396','EPSG','3842','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','5456','Ocotepeque 1935 / Costa Rica Norte',NULL,'EPSG','4499','EPSG','5451','EPSG','5390',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3982','projected_crs','EPSG','5456','EPSG','3869','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','5457','Ocotepeque 1935 / Costa Rica Sur',NULL,'EPSG','4499','EPSG','5451','EPSG','5394',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3983','projected_crs','EPSG','5457','EPSG','3870','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','5458','Ocotepeque 1935 / Guatemala Norte',NULL,'EPSG','4499','EPSG','4267','EPSG','18211',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3984','projected_crs','EPSG','5458','EPSG','2120','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','5459','Ocotepeque 1935 / Guatemala Sur',NULL,'EPSG','4499','EPSG','5451','EPSG','18212',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3985','projected_crs','EPSG','5459','EPSG','2121','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','5460','Ocotepeque 1935 / El Salvador Lambert',NULL,'EPSG','4499','EPSG','5451','EPSG','5399',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3986','projected_crs','EPSG','5460','EPSG','3243','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','5461','Ocotepeque 1935 / Nicaragua Norte',NULL,'EPSG','4499','EPSG','5451','EPSG','5439',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3987','projected_crs','EPSG','5461','EPSG','3844','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','5462','Ocotepeque 1935 / Nicaragua Sur',NULL,'EPSG','4499','EPSG','5451','EPSG','5444',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3988','projected_crs','EPSG','5462','EPSG','3847','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','5463','SAD69 / UTM zone 17N',NULL,'EPSG','4400','EPSG','4618','EPSG','16017',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3989','projected_crs','EPSG','5463','EPSG','3830','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','5466','Sibun Gorge 1922 / Colony Grid',NULL,'EPSG','4499','EPSG','5464','EPSG','5465',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3991','projected_crs','EPSG','5466','EPSG','3219','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','5469','Panama-Colon 1911 / Panama Lambert',NULL,'EPSG','4499','EPSG','5467','EPSG','5468',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3993','projected_crs','EPSG','5469','EPSG','3290','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','5472','Panama-Colon 1911 / Panama Polyconic',NULL,'EPSG','1028','EPSG','5467','EPSG','5471',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3994','projected_crs','EPSG','5472','EPSG','3290','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','5479','RSRGD2000 / MSLC2000',NULL,'EPSG','4500','EPSG','4764','EPSG','5475',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3995','projected_crs','EPSG','5479','EPSG','3853','EPSG','1236');
INSERT INTO "projected_crs" VALUES('EPSG','5480','RSRGD2000 / BCLC2000',NULL,'EPSG','4500','EPSG','4764','EPSG','5476',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3996','projected_crs','EPSG','5480','EPSG','3854','EPSG','1236');
INSERT INTO "projected_crs" VALUES('EPSG','5481','RSRGD2000 / PCLC2000',NULL,'EPSG','4500','EPSG','4764','EPSG','5477',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3997','projected_crs','EPSG','5481','EPSG','3855','EPSG','1236');
INSERT INTO "projected_crs" VALUES('EPSG','5482','RSRGD2000 / RSPS2000',NULL,'EPSG','1044','EPSG','4764','EPSG','5478',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3998','projected_crs','EPSG','5482','EPSG','3856','EPSG','1236');
INSERT INTO "projected_crs" VALUES('EPSG','5490','RGAF09 / UTM zone 20N',NULL,'EPSG','4400','EPSG','5489','EPSG','16020',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4002','projected_crs','EPSG','5490','EPSG','3825','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','5513','S-JTSK / Krovak',NULL,'EPSG','6501','EPSG','4156','EPSG','5509',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4006','projected_crs','EPSG','5513','EPSG','1306','EPSG','1095');
INSERT INTO "projected_crs" VALUES('EPSG','5514','S-JTSK / Krovak East North',NULL,'EPSG','4499','EPSG','4156','EPSG','5510',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4007','projected_crs','EPSG','5514','EPSG','1306','EPSG','1189');
INSERT INTO "projected_crs" VALUES('EPSG','5515','S-JTSK/05 / Modified Krovak',NULL,'EPSG','6501','EPSG','5228','EPSG','5511',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4008','projected_crs','EPSG','5515','EPSG','1079','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','5516','S-JTSK/05 / Modified Krovak East North',NULL,'EPSG','4499','EPSG','5228','EPSG','5512',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4009','projected_crs','EPSG','5516','EPSG','1079','EPSG','1189');
INSERT INTO "projected_crs" VALUES('EPSG','5518','CI1971 / Chatham Islands Map Grid',NULL,'EPSG','4500','EPSG','4672','EPSG','5517',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4010','projected_crs','EPSG','5518','EPSG','2889','EPSG','1056');
INSERT INTO "projected_crs" VALUES('EPSG','5519','CI1979 / Chatham Islands Map Grid',NULL,'EPSG','4500','EPSG','4673','EPSG','5517',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4011','projected_crs','EPSG','5519','EPSG','2889','EPSG','1056');
INSERT INTO "projected_crs" VALUES('EPSG','5520','DHDN / 3-degree Gauss-Kruger zone 1',NULL,'EPSG','4530','EPSG','4314','EPSG','16261',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4012','projected_crs','EPSG','5520','EPSG','3892','EPSG','1190');
INSERT INTO "projected_crs" VALUES('EPSG','5523','WGS 84 / Gabon TM 2011',NULL,'EPSG','4499','EPSG','4326','EPSG','5522',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4013','projected_crs','EPSG','5523','EPSG','1100','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','5530','SAD69(96) / Brazil Polyconic',NULL,'EPSG','4499','EPSG','5527','EPSG','19941',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4016','projected_crs','EPSG','5530','EPSG','1053','EPSG','1241');
INSERT INTO "projected_crs" VALUES('EPSG','5531','SAD69(96) / UTM zone 21S',NULL,'EPSG','4400','EPSG','5527','EPSG','16121',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4017','projected_crs','EPSG','5531','EPSG','3881','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','5532','SAD69(96) / UTM zone 22S',NULL,'EPSG','4400','EPSG','4618','EPSG','16122',NULL,1);
INSERT INTO "usage" VALUES('EPSG','4018','projected_crs','EPSG','5532','EPSG','3878','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','5533','SAD69(96) / UTM zone 23S',NULL,'EPSG','4400','EPSG','5527','EPSG','16123',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4019','projected_crs','EPSG','5533','EPSG','3445','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','5534','SAD69(96) / UTM zone 24S',NULL,'EPSG','4400','EPSG','5527','EPSG','16124',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4020','projected_crs','EPSG','5534','EPSG','3446','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','5535','SAD69(96) / UTM zone 25S',NULL,'EPSG','4400','EPSG','5527','EPSG','16125',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4021','projected_crs','EPSG','5535','EPSG','3447','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','5536','Corrego Alegre 1961 / UTM zone 21S',NULL,'EPSG','4400','EPSG','5524','EPSG','16121',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4022','projected_crs','EPSG','5536','EPSG','4574','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','5537','Corrego Alegre 1961 / UTM zone 22S',NULL,'EPSG','4400','EPSG','5524','EPSG','16122',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4023','projected_crs','EPSG','5537','EPSG','4576','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','5538','Corrego Alegre 1961 / UTM zone 23S',NULL,'EPSG','4400','EPSG','5524','EPSG','16123',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4024','projected_crs','EPSG','5538','EPSG','3177','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','5539','Corrego Alegre 1961 / UTM zone 24S',NULL,'EPSG','4400','EPSG','5524','EPSG','16124',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4025','projected_crs','EPSG','5539','EPSG','3877','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','5550','PNG94 / PNGMG94 zone 54',NULL,'EPSG','4400','EPSG','5546','EPSG','5547',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4029','projected_crs','EPSG','5550','EPSG','3882','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','5551','PNG94 / PNGMG94 zone 55',NULL,'EPSG','4400','EPSG','5546','EPSG','5548',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4030','projected_crs','EPSG','5551','EPSG','3885','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','5552','PNG94 / PNGMG94 zone 56',NULL,'EPSG','4400','EPSG','5546','EPSG','5549',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4031','projected_crs','EPSG','5552','EPSG','3888','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','5559','Ocotepeque 1935 / Guatemala Norte',NULL,'EPSG','4499','EPSG','5451','EPSG','18211',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4036','projected_crs','EPSG','5559','EPSG','2120','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','5562','UCS-2000 / Gauss-Kruger zone 4',NULL,'EPSG','4530','EPSG','5561','EPSG','16204',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4039','projected_crs','EPSG','5562','EPSG','3895','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','5563','UCS-2000 / Gauss-Kruger zone 5',NULL,'EPSG','4530','EPSG','5561','EPSG','16205',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4040','projected_crs','EPSG','5563','EPSG','3898','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','5564','UCS-2000 / Gauss-Kruger zone 6',NULL,'EPSG','4530','EPSG','5561','EPSG','16206',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4041','projected_crs','EPSG','5564','EPSG','3903','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','5565','UCS-2000 / Gauss-Kruger zone 7',NULL,'EPSG','4530','EPSG','5561','EPSG','16207',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4042','projected_crs','EPSG','5565','EPSG','3905','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','5566','UCS-2000 / Gauss-Kruger CM 21E',NULL,'EPSG','4530','EPSG','5561','EPSG','16304',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4043','projected_crs','EPSG','5566','EPSG','3895','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','5567','UCS-2000 / Gauss-Kruger CM 27E',NULL,'EPSG','4530','EPSG','5561','EPSG','16305',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4044','projected_crs','EPSG','5567','EPSG','3898','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','5568','UCS-2000 / Gauss-Kruger CM 33E',NULL,'EPSG','4530','EPSG','5561','EPSG','16306',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4045','projected_crs','EPSG','5568','EPSG','3903','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','5569','UCS-2000 / Gauss-Kruger CM 39E',NULL,'EPSG','4530','EPSG','5561','EPSG','16307',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4046','projected_crs','EPSG','5569','EPSG','3905','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','5570','UCS-2000 / 3-degree Gauss-Kruger zone 7',NULL,'EPSG','4530','EPSG','5561','EPSG','16267',NULL,1);
INSERT INTO "usage" VALUES('EPSG','4047','projected_crs','EPSG','5570','EPSG','3906','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','5571','UCS-2000 / 3-degree Gauss-Kruger zone 8',NULL,'EPSG','4530','EPSG','5561','EPSG','16268',NULL,1);
INSERT INTO "usage" VALUES('EPSG','4048','projected_crs','EPSG','5571','EPSG','3907','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','5572','UCS-2000 / 3-degree Gauss-Kruger zone 9',NULL,'EPSG','4530','EPSG','5561','EPSG','16269',NULL,1);
INSERT INTO "usage" VALUES('EPSG','4049','projected_crs','EPSG','5572','EPSG','3908','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','5573','UCS-2000 / 3-degree Gauss-Kruger zone 10',NULL,'EPSG','4530','EPSG','5561','EPSG','16270',NULL,1);
INSERT INTO "usage" VALUES('EPSG','4050','projected_crs','EPSG','5573','EPSG','3909','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','5574','UCS-2000 / 3-degree Gauss-Kruger zone 11',NULL,'EPSG','4530','EPSG','5561','EPSG','16271',NULL,1);
INSERT INTO "usage" VALUES('EPSG','4051','projected_crs','EPSG','5574','EPSG','3910','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','5575','UCS-2000 / 3-degree Gauss-Kruger zone 12',NULL,'EPSG','4530','EPSG','5561','EPSG','16272',NULL,1);
INSERT INTO "usage" VALUES('EPSG','4052','projected_crs','EPSG','5575','EPSG','3912','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','5576','UCS-2000 / 3-degree Gauss-Kruger zone 13',NULL,'EPSG','4530','EPSG','5561','EPSG','16273',NULL,1);
INSERT INTO "usage" VALUES('EPSG','4053','projected_crs','EPSG','5576','EPSG','3913','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','5577','UCS-2000 / 3-degree Gauss-Kruger CM 21E',NULL,'EPSG','4530','EPSG','5561','EPSG','16304',NULL,1);
INSERT INTO "usage" VALUES('EPSG','4054','projected_crs','EPSG','5577','EPSG','3906','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','5578','UCS-2000 / 3-degree Gauss-Kruger CM 24E',NULL,'EPSG','4530','EPSG','5561','EPSG','16368',NULL,1);
INSERT INTO "usage" VALUES('EPSG','4055','projected_crs','EPSG','5578','EPSG','3907','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','5579','UCS-2000 / 3-degree Gauss-Kruger CM 27E',NULL,'EPSG','4530','EPSG','5561','EPSG','16305',NULL,1);
INSERT INTO "usage" VALUES('EPSG','4056','projected_crs','EPSG','5579','EPSG','3908','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','5580','UCS-2000 / 3-degree Gauss-Kruger CM 30E',NULL,'EPSG','4530','EPSG','5561','EPSG','16370',NULL,1);
INSERT INTO "usage" VALUES('EPSG','4057','projected_crs','EPSG','5580','EPSG','3909','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','5581','UCS-2000 / 3-degree Gauss-Kruger CM 33E',NULL,'EPSG','4530','EPSG','5561','EPSG','16306',NULL,1);
INSERT INTO "usage" VALUES('EPSG','4058','projected_crs','EPSG','5581','EPSG','3910','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','5582','UCS-2000 / 3-degree Gauss-Kruger CM 36E',NULL,'EPSG','4530','EPSG','5561','EPSG','16372',NULL,1);
INSERT INTO "usage" VALUES('EPSG','4059','projected_crs','EPSG','5582','EPSG','3912','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','5583','UCS-2000 / 3-degree Gauss-Kruger CM 39E',NULL,'EPSG','4530','EPSG','5561','EPSG','16307',NULL,1);
INSERT INTO "usage" VALUES('EPSG','4060','projected_crs','EPSG','5583','EPSG','3913','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','5588','NAD27 / New Brunswick Stereographic (NAD27)',NULL,'EPSG','1029','EPSG','4267','EPSG','5587',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4061','projected_crs','EPSG','5588','EPSG','1447','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','5589','Sibun Gorge 1922 / Colony Grid',NULL,'EPSG','4403','EPSG','5464','EPSG','5465',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4062','projected_crs','EPSG','5589','EPSG','3219','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','5596','FEH2010 / Fehmarnbelt TM',NULL,'EPSG','4400','EPSG','5593','EPSG','5595',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4066','projected_crs','EPSG','5596','EPSG','3889','EPSG','1139');
INSERT INTO "projected_crs" VALUES('EPSG','5623','NAD27 / Michigan East',NULL,'EPSG','4497','EPSG','4267','EPSG','12101',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4091','projected_crs','EPSG','5623','EPSG','1720','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','5624','NAD27 / Michigan Old Central',NULL,'EPSG','4497','EPSG','4267','EPSG','12102',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4092','projected_crs','EPSG','5624','EPSG','1721','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','5625','NAD27 / Michigan West',NULL,'EPSG','4497','EPSG','4267','EPSG','12103',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4093','projected_crs','EPSG','5625','EPSG','3652','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','5627','ED50 / TM 6 NE',NULL,'EPSG','4400','EPSG','4230','EPSG','16406',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4094','projected_crs','EPSG','5627','EPSG','3897','EPSG','1136');
INSERT INTO "projected_crs" VALUES('EPSG','5629','Moznet / UTM zone 38S',NULL,'EPSG','4400','EPSG','4130','EPSG','16138',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4096','projected_crs','EPSG','5629','EPSG','1541','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','5631','Pulkovo 1942(58) / Gauss-Kruger zone 2 (E-N)',NULL,'EPSG','4400','EPSG','4179','EPSG','16202',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4097','projected_crs','EPSG','5631','EPSG','3575','EPSG','1189');
INSERT INTO "projected_crs" VALUES('EPSG','5632','PTRA08 / LCC Europe',NULL,'EPSG','4500','EPSG','5013','EPSG','19985',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4098','projected_crs','EPSG','5632','EPSG','3670','EPSG','1107');
INSERT INTO "projected_crs" VALUES('EPSG','5633','PTRA08 / LAEA Europe',NULL,'EPSG','4532','EPSG','5013','EPSG','19986',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4099','projected_crs','EPSG','5633','EPSG','3670','EPSG','1162');
INSERT INTO "projected_crs" VALUES('EPSG','5634','REGCAN95 / LCC Europe',NULL,'EPSG','4500','EPSG','4081','EPSG','19985',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4100','projected_crs','EPSG','5634','EPSG','3199','EPSG','1107');
INSERT INTO "projected_crs" VALUES('EPSG','5635','REGCAN95 / LAEA Europe',NULL,'EPSG','4500','EPSG','4081','EPSG','19986',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4101','projected_crs','EPSG','5635','EPSG','3199','EPSG','1162');
INSERT INTO "projected_crs" VALUES('EPSG','5636','TUREF / LAEA Europe',NULL,'EPSG','4532','EPSG','5252','EPSG','19986',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4102','projected_crs','EPSG','5636','EPSG','1237','EPSG','1162');
INSERT INTO "projected_crs" VALUES('EPSG','5637','TUREF / LCC Europe',NULL,'EPSG','4500','EPSG','5252','EPSG','19985',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4103','projected_crs','EPSG','5637','EPSG','1237','EPSG','1107');
INSERT INTO "projected_crs" VALUES('EPSG','5638','ISN2004 / LAEA Europe',NULL,'EPSG','4532','EPSG','5324','EPSG','19986',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4104','projected_crs','EPSG','5638','EPSG','1120','EPSG','1162');
INSERT INTO "projected_crs" VALUES('EPSG','5639','ISN2004 / LCC Europe',NULL,'EPSG','4500','EPSG','5324','EPSG','19985',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4105','projected_crs','EPSG','5639','EPSG','1120','EPSG','1107');
INSERT INTO "projected_crs" VALUES('EPSG','5641','SIRGAS 2000 / Brazil Mercator',NULL,'EPSG','4499','EPSG','4674','EPSG','5640',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4106','projected_crs','EPSG','5641','EPSG','3896','EPSG','1136');
INSERT INTO "projected_crs" VALUES('EPSG','5643','ED50 / SPBA LCC',NULL,'EPSG','4400','EPSG','4230','EPSG','5642',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4107','projected_crs','EPSG','5643','EPSG','3899','EPSG','1190');
INSERT INTO "projected_crs" VALUES('EPSG','5644','RGR92 / UTM zone 39S',NULL,'EPSG','4400','EPSG','4627','EPSG','16139',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4108','projected_crs','EPSG','5644','EPSG','3915','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','5646','NAD83 / Vermont (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','5645',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4109','projected_crs','EPSG','5646','EPSG','1414','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','5649','ETRS89 / UTM zone 31N (zE-N)',NULL,'EPSG','4400','EPSG','4258','EPSG','5647',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4110','projected_crs','EPSG','5649','EPSG','2860','EPSG','1153');
INSERT INTO "projected_crs" VALUES('EPSG','5650','ETRS89 / UTM zone 33N (zE-N)',NULL,'EPSG','4400','EPSG','4258','EPSG','5648',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4111','projected_crs','EPSG','5650','EPSG','2862','EPSG','1153');
INSERT INTO "projected_crs" VALUES('EPSG','5651','ETRS89 / UTM zone 31N (N-zE)',NULL,'EPSG','4500','EPSG','4258','EPSG','5647',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4112','projected_crs','EPSG','5651','EPSG','2860','EPSG','1153');
INSERT INTO "projected_crs" VALUES('EPSG','5652','ETRS89 / UTM zone 32N (N-zE)',NULL,'EPSG','4500','EPSG','4258','EPSG','4648',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4113','projected_crs','EPSG','5652','EPSG','2861','EPSG','1153');
INSERT INTO "projected_crs" VALUES('EPSG','5653','ETRS89 / UTM zone 33N (N-zE)',NULL,'EPSG','4500','EPSG','4258','EPSG','5648',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4114','projected_crs','EPSG','5653','EPSG','2862','EPSG','1153');
INSERT INTO "projected_crs" VALUES('EPSG','5654','NAD83(HARN) / Vermont (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','5645',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4115','projected_crs','EPSG','5654','EPSG','1414','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','5655','NAD83(NSRS2007) / Vermont (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','5645',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4116','projected_crs','EPSG','5655','EPSG','1414','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','5659','Monte Mario / TM Emilia-Romagna',NULL,'EPSG','4499','EPSG','4265','EPSG','5658',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4117','projected_crs','EPSG','5659','EPSG','4035','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','5663','Pulkovo 1942(58) / Gauss-Kruger zone 3 (E-N)',NULL,'EPSG','4400','EPSG','4179','EPSG','16203',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4118','projected_crs','EPSG','5663','EPSG','1792','EPSG','1189');
INSERT INTO "projected_crs" VALUES('EPSG','5664','Pulkovo 1942(83) / Gauss-Kruger zone 2 (E-N)',NULL,'EPSG','4400','EPSG','4178','EPSG','16202',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4119','projected_crs','EPSG','5664','EPSG','3575','EPSG','1189');
INSERT INTO "projected_crs" VALUES('EPSG','5665','Pulkovo 1942(83) / Gauss-Kruger zone 3 (E-N)',NULL,'EPSG','4400','EPSG','4178','EPSG','16203',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4120','projected_crs','EPSG','5665','EPSG','3576','EPSG','1189');
INSERT INTO "projected_crs" VALUES('EPSG','5666','PD/83 / 3-degree Gauss-Kruger zone 3 (E-N)',NULL,'EPSG','4400','EPSG','4746','EPSG','16263',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4121','projected_crs','EPSG','5666','EPSG','3392','EPSG','1189');
INSERT INTO "projected_crs" VALUES('EPSG','5667','PD/83 / 3-degree Gauss-Kruger zone 4 (E-N)',NULL,'EPSG','4400','EPSG','4746','EPSG','16264',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4122','projected_crs','EPSG','5667','EPSG','3393','EPSG','1189');
INSERT INTO "projected_crs" VALUES('EPSG','5668','RD/83 / 3-degree Gauss-Kruger zone 4 (E-N)',NULL,'EPSG','4400','EPSG','4745','EPSG','16264',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4123','projected_crs','EPSG','5668','EPSG','3395','EPSG','1189');
INSERT INTO "projected_crs" VALUES('EPSG','5669','RD/83 / 3-degree Gauss-Kruger zone 5 (E-N)',NULL,'EPSG','4400','EPSG','4745','EPSG','16265',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4124','projected_crs','EPSG','5669','EPSG','3394','EPSG','1189');
INSERT INTO "projected_crs" VALUES('EPSG','5670','Pulkovo 1942(58) / 3-degree Gauss-Kruger zone 3 (E-N)',NULL,'EPSG','4400','EPSG','4179','EPSG','16263',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4125','projected_crs','EPSG','5670','EPSG','1512','EPSG','1189');
INSERT INTO "projected_crs" VALUES('EPSG','5671','Pulkovo 1942(58) / 3-degree Gauss-Kruger zone 4 (E-N)',NULL,'EPSG','4400','EPSG','4179','EPSG','16264',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4126','projected_crs','EPSG','5671','EPSG','1513','EPSG','1189');
INSERT INTO "projected_crs" VALUES('EPSG','5672','Pulkovo 1942(58) / 3-degree Gauss-Kruger zone 5 (E-N)',NULL,'EPSG','4400','EPSG','4179','EPSG','16265',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4127','projected_crs','EPSG','5672','EPSG','3580','EPSG','1189');
INSERT INTO "projected_crs" VALUES('EPSG','5673','Pulkovo 1942(83) / 3-degree Gauss-Kruger zone 3 (E-N)',NULL,'EPSG','4400','EPSG','4178','EPSG','16263',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4128','projected_crs','EPSG','5673','EPSG','1512','EPSG','1189');
INSERT INTO "projected_crs" VALUES('EPSG','5674','Pulkovo 1942(83) / 3-degree Gauss-Kruger zone 4 (E-N)',NULL,'EPSG','4400','EPSG','4178','EPSG','16264',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4129','projected_crs','EPSG','5674','EPSG','1513','EPSG','1189');
INSERT INTO "projected_crs" VALUES('EPSG','5675','Pulkovo 1942(83) / 3-degree Gauss-Kruger zone 5 (E-N)',NULL,'EPSG','4400','EPSG','4178','EPSG','16265',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4130','projected_crs','EPSG','5675','EPSG','1514','EPSG','1160');
INSERT INTO "projected_crs" VALUES('EPSG','5676','DHDN / 3-degree Gauss-Kruger zone 2 (E-N)',NULL,'EPSG','4400','EPSG','4314','EPSG','16262',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4131','projected_crs','EPSG','5676','EPSG','1624','EPSG','1189');
INSERT INTO "projected_crs" VALUES('EPSG','5677','DHDN / 3-degree Gauss-Kruger zone 3 (E-N)',NULL,'EPSG','4400','EPSG','4314','EPSG','16263',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4132','projected_crs','EPSG','5677','EPSG','1625','EPSG','1189');
INSERT INTO "projected_crs" VALUES('EPSG','5678','DHDN / 3-degree Gauss-Kruger zone 4 (E-N)',NULL,'EPSG','4400','EPSG','4314','EPSG','16264',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4133','projected_crs','EPSG','5678','EPSG','1626','EPSG','1189');
INSERT INTO "projected_crs" VALUES('EPSG','5679','DHDN / 3-degree Gauss-Kruger zone 5 (E-N)',NULL,'EPSG','4400','EPSG','4314','EPSG','16265',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4134','projected_crs','EPSG','5679','EPSG','1627','EPSG','1189');
INSERT INTO "projected_crs" VALUES('EPSG','5680','DHDN / 3-degree Gauss-Kruger zone 1 (E-N)',NULL,'EPSG','4400','EPSG','4314','EPSG','16261',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4135','projected_crs','EPSG','5680','EPSG','3892','EPSG','1189');
INSERT INTO "projected_crs" VALUES('EPSG','5682','DB_REF / 3-degree Gauss-Kruger zone 2 (E-N)',NULL,'EPSG','4400','EPSG','5681','EPSG','16262',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4137','projected_crs','EPSG','5682','EPSG','1624','EPSG','1141');
INSERT INTO "projected_crs" VALUES('EPSG','5683','DB_REF / 3-degree Gauss-Kruger zone 3 (E-N)',NULL,'EPSG','4400','EPSG','5681','EPSG','16263',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4138','projected_crs','EPSG','5683','EPSG','3993','EPSG','1141');
INSERT INTO "projected_crs" VALUES('EPSG','5684','DB_REF / 3-degree Gauss-Kruger zone 4 (E-N)',NULL,'EPSG','4400','EPSG','5681','EPSG','16264',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4139','projected_crs','EPSG','5684','EPSG','3996','EPSG','1141');
INSERT INTO "projected_crs" VALUES('EPSG','5685','DB_REF / 3-degree Gauss-Kruger zone 5 (E-N)',NULL,'EPSG','4400','EPSG','5681','EPSG','16265',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4140','projected_crs','EPSG','5685','EPSG','3998','EPSG','1141');
INSERT INTO "projected_crs" VALUES('EPSG','5700','NZGD2000 / UTM zone 1S',NULL,'EPSG','4400','EPSG','4167','EPSG','16101',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4143','projected_crs','EPSG','5700','EPSG','3992','EPSG','1153');
INSERT INTO "projected_crs" VALUES('EPSG','5825','AGD66 / ACT Standard Grid',NULL,'EPSG','4400','EPSG','4202','EPSG','5824',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4265','projected_crs','EPSG','5825','EPSG','2283','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','5836','Yemen NGN96 / UTM zone 37N',NULL,'EPSG','4400','EPSG','4163','EPSG','16037',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4274','projected_crs','EPSG','5836','EPSG','4006','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','5837','Yemen NGN96 / UTM zone 40N',NULL,'EPSG','4400','EPSG','4163','EPSG','16040',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4275','projected_crs','EPSG','5837','EPSG','4002','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','5839','Peru96 / UTM zone 17S',NULL,'EPSG','4400','EPSG','5373','EPSG','16117',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4276','projected_crs','EPSG','5839','EPSG','3837','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','5842','WGS 84 / TM 12 SE',NULL,'EPSG','4400','EPSG','4326','EPSG','16612',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4277','projected_crs','EPSG','5842','EPSG','4025','EPSG','1080');
INSERT INTO "projected_crs" VALUES('EPSG','5844','RGRDC 2005 / Congo TM zone 30',NULL,'EPSG','4499','EPSG','4046','EPSG','17430',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4279','projected_crs','EPSG','5844','EPSG','4018','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','5858','SAD69(96) / UTM zone 22S',NULL,'EPSG','4400','EPSG','5527','EPSG','16122',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4293','projected_crs','EPSG','5858','EPSG','3878','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','5875','SAD69(96) / UTM zone 18S',NULL,'EPSG','4400','EPSG','5527','EPSG','16118',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4309','projected_crs','EPSG','5875','EPSG','4023','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','5876','SAD69(96) / UTM zone 19S',NULL,'EPSG','4400','EPSG','5527','EPSG','16119',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4310','projected_crs','EPSG','5876','EPSG','4024','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','5877','SAD69(96) / UTM zone 20S',NULL,'EPSG','4400','EPSG','5527','EPSG','16120',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4311','projected_crs','EPSG','5877','EPSG','4026','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','5879','Cadastre 1997 / UTM zone 38S',NULL,'EPSG','4400','EPSG','4475','EPSG','16138',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4312','projected_crs','EPSG','5879','EPSG','3340','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','5880','SIRGAS 2000 / Brazil Polyconic',NULL,'EPSG','4499','EPSG','4674','EPSG','19941',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4313','projected_crs','EPSG','5880','EPSG','1053','EPSG','1241');
INSERT INTO "projected_crs" VALUES('EPSG','5887','TGD2005 / Tonga Map Grid',NULL,'EPSG','4400','EPSG','5886','EPSG','5883',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4317','projected_crs','EPSG','5887','EPSG','1234','EPSG','1181');
INSERT INTO "projected_crs" VALUES('EPSG','5890','JAXA Snow Depth Polar Stereographic North',NULL,'EPSG','1035','EPSG','4054','EPSG','5889',NULL,1);
INSERT INTO "usage" VALUES('EPSG','4318','projected_crs','EPSG','5890','EPSG','1996','EPSG','1040');
INSERT INTO "projected_crs" VALUES('EPSG','5896','VN-2000 / TM-3 zone 481',NULL,'EPSG','4400','EPSG','4756','EPSG','5892',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4319','projected_crs','EPSG','5896','EPSG','4193','EPSG','1208');
INSERT INTO "projected_crs" VALUES('EPSG','5897','VN-2000 / TM-3 zone 482',NULL,'EPSG','4400','EPSG','4756','EPSG','5893',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4320','projected_crs','EPSG','5897','EPSG','4215','EPSG','1208');
INSERT INTO "usage" VALUES('EPSG','4321','projected_crs','EPSG','5897','EPSG','4547','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','5898','VN-2000 / TM-3 zone 491',NULL,'EPSG','4400','EPSG','4756','EPSG','5894',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4322','projected_crs','EPSG','5898','EPSG','4217','EPSG','1208');
INSERT INTO "usage" VALUES('EPSG','4323','projected_crs','EPSG','5898','EPSG','4558','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','5899','VN-2000 / TM-3 107-45',NULL,'EPSG','4400','EPSG','4756','EPSG','5895',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4324','projected_crs','EPSG','5899','EPSG','4218','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','5921','WGS 84 / EPSG Arctic Regional zone A1',NULL,'EPSG','4400','EPSG','4326','EPSG','5906',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4325','projected_crs','EPSG','5921','EPSG','4019','EPSG','1246');
INSERT INTO "projected_crs" VALUES('EPSG','5922','WGS 84 / EPSG Arctic Regional zone A2',NULL,'EPSG','4400','EPSG','4326','EPSG','5907',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4326','projected_crs','EPSG','5922','EPSG','4027','EPSG','1246');
INSERT INTO "projected_crs" VALUES('EPSG','5923','WGS 84 / EPSG Arctic Regional zone A3',NULL,'EPSG','4400','EPSG','4326','EPSG','5908',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4327','projected_crs','EPSG','5923','EPSG','4028','EPSG','1246');
INSERT INTO "projected_crs" VALUES('EPSG','5924','WGS 84 / EPSG Arctic Regional zone A4',NULL,'EPSG','4400','EPSG','4326','EPSG','5909',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4328','projected_crs','EPSG','5924','EPSG','4029','EPSG','1246');
INSERT INTO "projected_crs" VALUES('EPSG','5925','WGS 84 / EPSG Arctic Regional zone A5',NULL,'EPSG','4400','EPSG','4326','EPSG','5910',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4329','projected_crs','EPSG','5925','EPSG','4031','EPSG','1246');
INSERT INTO "projected_crs" VALUES('EPSG','5926','WGS 84 / EPSG Arctic Regional zone B1',NULL,'EPSG','4400','EPSG','4326','EPSG','5911',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4330','projected_crs','EPSG','5926','EPSG','4032','EPSG','1246');
INSERT INTO "projected_crs" VALUES('EPSG','5927','WGS 84 / EPSG Arctic Regional zone B2',NULL,'EPSG','4400','EPSG','4326','EPSG','5912',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4331','projected_crs','EPSG','5927','EPSG','4033','EPSG','1246');
INSERT INTO "projected_crs" VALUES('EPSG','5928','WGS 84 / EPSG Arctic Regional zone B3',NULL,'EPSG','4400','EPSG','4326','EPSG','5913',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4332','projected_crs','EPSG','5928','EPSG','4034','EPSG','1246');
INSERT INTO "projected_crs" VALUES('EPSG','5929','WGS 84 / EPSG Arctic Regional zone B4',NULL,'EPSG','4400','EPSG','4326','EPSG','5914',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4333','projected_crs','EPSG','5929','EPSG','4037','EPSG','1246');
INSERT INTO "projected_crs" VALUES('EPSG','5930','WGS 84 / EPSG Arctic Regional zone B5',NULL,'EPSG','4400','EPSG','4326','EPSG','5915',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4334','projected_crs','EPSG','5930','EPSG','4038','EPSG','1246');
INSERT INTO "projected_crs" VALUES('EPSG','5931','WGS 84 / EPSG Arctic Regional zone C1',NULL,'EPSG','4400','EPSG','4326','EPSG','5916',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4335','projected_crs','EPSG','5931','EPSG','4040','EPSG','1246');
INSERT INTO "projected_crs" VALUES('EPSG','5932','WGS 84 / EPSG Arctic Regional zone C2',NULL,'EPSG','4400','EPSG','4326','EPSG','5917',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4336','projected_crs','EPSG','5932','EPSG','4041','EPSG','1246');
INSERT INTO "projected_crs" VALUES('EPSG','5933','WGS 84 / EPSG Arctic Regional zone C3',NULL,'EPSG','4400','EPSG','4326','EPSG','5918',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4337','projected_crs','EPSG','5933','EPSG','4042','EPSG','1246');
INSERT INTO "projected_crs" VALUES('EPSG','5934','WGS 84 / EPSG Arctic Regional zone C4',NULL,'EPSG','4400','EPSG','4326','EPSG','5919',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4338','projected_crs','EPSG','5934','EPSG','4043','EPSG','1246');
INSERT INTO "projected_crs" VALUES('EPSG','5935','WGS 84 / EPSG Arctic Regional zone C5',NULL,'EPSG','4400','EPSG','4326','EPSG','5920',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4339','projected_crs','EPSG','5935','EPSG','4045','EPSG','1246');
INSERT INTO "projected_crs" VALUES('EPSG','5936','WGS 84 / EPSG Alaska Polar Stereographic',NULL,'EPSG','4467','EPSG','4326','EPSG','5901',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4340','projected_crs','EPSG','5936','EPSG','1996','EPSG','1071');
INSERT INTO "projected_crs" VALUES('EPSG','5937','WGS 84 / EPSG Canada Polar Stereographic',NULL,'EPSG','4466','EPSG','4326','EPSG','5902',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4341','projected_crs','EPSG','5937','EPSG','1996','EPSG','1072');
INSERT INTO "projected_crs" VALUES('EPSG','5938','WGS 84 / EPSG Greenland Polar Stereographic',NULL,'EPSG','1036','EPSG','4326','EPSG','5903',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4342','projected_crs','EPSG','5938','EPSG','1996','EPSG','1073');
INSERT INTO "projected_crs" VALUES('EPSG','5939','WGS 84 / EPSG Norway Polar Stereographic',NULL,'EPSG','1037','EPSG','4326','EPSG','5904',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4343','projected_crs','EPSG','5939','EPSG','1996','EPSG','1074');
INSERT INTO "projected_crs" VALUES('EPSG','5940','WGS 84 / EPSG Russia Polar Stereographic',NULL,'EPSG','1038','EPSG','4326','EPSG','5905',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4344','projected_crs','EPSG','5940','EPSG','1996','EPSG','1075');
INSERT INTO "projected_crs" VALUES('EPSG','6050','GR96 / EPSG Arctic zone 1-25',NULL,'EPSG','4400','EPSG','4747','EPSG','5979',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4379','projected_crs','EPSG','6050','EPSG','4048','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6051','GR96 / EPSG Arctic zone 2-18',NULL,'EPSG','4400','EPSG','4747','EPSG','5987',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4380','projected_crs','EPSG','6051','EPSG','4039','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6052','GR96 / EPSG Arctic zone 2-20',NULL,'EPSG','4400','EPSG','4747','EPSG','5988',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4381','projected_crs','EPSG','6052','EPSG','4046','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6053','GR96 / EPSG Arctic zone 3-29',NULL,'EPSG','4400','EPSG','4747','EPSG','6002',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4382','projected_crs','EPSG','6053','EPSG','4073','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6054','GR96 / EPSG Arctic zone 3-31',NULL,'EPSG','4400','EPSG','4747','EPSG','6003',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4383','projected_crs','EPSG','6054','EPSG','4074','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6055','GR96 / EPSG Arctic zone 3-33',NULL,'EPSG','4400','EPSG','4747','EPSG','6004',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4384','projected_crs','EPSG','6055','EPSG','4075','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6056','GR96 / EPSG Arctic zone 4-20',NULL,'EPSG','4400','EPSG','4747','EPSG','6009',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4385','projected_crs','EPSG','6056','EPSG','4080','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6057','GR96 / EPSG Arctic zone 4-22',NULL,'EPSG','4400','EPSG','4747','EPSG','6010',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4386','projected_crs','EPSG','6057','EPSG','4081','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6058','GR96 / EPSG Arctic zone 4-24',NULL,'EPSG','4400','EPSG','4747','EPSG','6011',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4387','projected_crs','EPSG','6058','EPSG','4082','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6059','GR96 / EPSG Arctic zone 5-41',NULL,'EPSG','4400','EPSG','4747','EPSG','6035',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4388','projected_crs','EPSG','6059','EPSG','4106','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6060','GR96 / EPSG Arctic zone 5-43',NULL,'EPSG','4400','EPSG','4747','EPSG','6036',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4389','projected_crs','EPSG','6060','EPSG','4107','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6061','GR96 / EPSG Arctic zone 5-45',NULL,'EPSG','4400','EPSG','4747','EPSG','6037',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4390','projected_crs','EPSG','6061','EPSG','4108','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6062','GR96 / EPSG Arctic zone 6-26',NULL,'EPSG','4400','EPSG','4747','EPSG','6045',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4391','projected_crs','EPSG','6062','EPSG','4116','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6063','GR96 / EPSG Arctic zone 6-28',NULL,'EPSG','4400','EPSG','4747','EPSG','6046',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4392','projected_crs','EPSG','6063','EPSG','4117','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6064','GR96 / EPSG Arctic zone 6-30',NULL,'EPSG','4400','EPSG','4747','EPSG','6047',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4393','projected_crs','EPSG','6064','EPSG','4118','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6065','GR96 / EPSG Arctic zone 7-11',NULL,'EPSG','4400','EPSG','4747','EPSG','6048',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4394','projected_crs','EPSG','6065','EPSG','4119','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6066','GR96 / EPSG Arctic zone 7-13',NULL,'EPSG','4400','EPSG','4747','EPSG','6049',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4395','projected_crs','EPSG','6066','EPSG','4120','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6067','GR96 / EPSG Arctic zone 8-20',NULL,'EPSG','4400','EPSG','4747','EPSG','5943',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4396','projected_crs','EPSG','6067','EPSG','4123','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6068','GR96 / EPSG Arctic zone 8-22',NULL,'EPSG','4400','EPSG','4747','EPSG','5944',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4397','projected_crs','EPSG','6068','EPSG','4124','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6069','ETRS89 / EPSG Arctic zone 2-22',NULL,'EPSG','4400','EPSG','4258','EPSG','5989',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4398','projected_crs','EPSG','6069','EPSG','4053','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6070','ETRS89 / EPSG Arctic zone 3-11',NULL,'EPSG','4400','EPSG','4258','EPSG','5993',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4399','projected_crs','EPSG','6070','EPSG','4058','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6071','ETRS89 / EPSG Arctic zone 4-26',NULL,'EPSG','4400','EPSG','4258','EPSG','6012',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4400','projected_crs','EPSG','6071','EPSG','4083','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6072','ETRS89 / EPSG Arctic zone 4-28',NULL,'EPSG','4400','EPSG','4258','EPSG','6013',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4401','projected_crs','EPSG','6072','EPSG','4084','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6073','ETRS89 / EPSG Arctic zone 5-11',NULL,'EPSG','4400','EPSG','4258','EPSG','6020',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4402','projected_crs','EPSG','6073','EPSG','4091','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6074','ETRS89 / EPSG Arctic zone 5-13',NULL,'EPSG','4400','EPSG','4258','EPSG','6021',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4403','projected_crs','EPSG','6074','EPSG','4092','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6075','WGS 84 / EPSG Arctic zone 2-24',NULL,'EPSG','4400','EPSG','4326','EPSG','5990',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4404','projected_crs','EPSG','6075','EPSG','4054','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6076','WGS 84 / EPSG Arctic zone 2-26',NULL,'EPSG','4400','EPSG','4326','EPSG','5991',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4405','projected_crs','EPSG','6076','EPSG','4055','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6077','WGS 84 / EPSG Arctic zone 3-13',NULL,'EPSG','4400','EPSG','4326','EPSG','5994',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4406','projected_crs','EPSG','6077','EPSG','4059','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6078','WGS 84 / EPSG Arctic zone 3-15',NULL,'EPSG','4400','EPSG','4326','EPSG','5995',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4407','projected_crs','EPSG','6078','EPSG','4060','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6079','WGS 84 / EPSG Arctic zone 3-17',NULL,'EPSG','4400','EPSG','4326','EPSG','5996',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4408','projected_crs','EPSG','6079','EPSG','4061','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6080','WGS 84 / EPSG Arctic zone 3-19',NULL,'EPSG','4400','EPSG','4326','EPSG','5997',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4409','projected_crs','EPSG','6080','EPSG','4062','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6081','WGS 84 / EPSG Arctic zone 4-30',NULL,'EPSG','4400','EPSG','4326','EPSG','6014',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4410','projected_crs','EPSG','6081','EPSG','4085','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6082','WGS 84 / EPSG Arctic zone 4-32',NULL,'EPSG','4400','EPSG','4326','EPSG','6015',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4411','projected_crs','EPSG','6082','EPSG','4086','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6083','WGS 84 / EPSG Arctic zone 4-34',NULL,'EPSG','4400','EPSG','4326','EPSG','6016',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4412','projected_crs','EPSG','6083','EPSG','4087','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6084','WGS 84 / EPSG Arctic zone 4-36',NULL,'EPSG','4400','EPSG','4326','EPSG','6017',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4413','projected_crs','EPSG','6084','EPSG','4088','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6085','WGS 84 / EPSG Arctic zone 4-38',NULL,'EPSG','4400','EPSG','4326','EPSG','6018',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4414','projected_crs','EPSG','6085','EPSG','4089','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6086','WGS 84 / EPSG Arctic zone 4-40',NULL,'EPSG','4400','EPSG','4326','EPSG','6019',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4415','projected_crs','EPSG','6086','EPSG','4090','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6087','WGS 84 / EPSG Arctic zone 5-15',NULL,'EPSG','4400','EPSG','4326','EPSG','6022',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4416','projected_crs','EPSG','6087','EPSG','4093','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6088','WGS 84 / EPSG Arctic zone 5-17',NULL,'EPSG','4400','EPSG','4326','EPSG','6023',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4417','projected_crs','EPSG','6088','EPSG','4094','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6089','WGS 84 / EPSG Arctic zone 5-19',NULL,'EPSG','4400','EPSG','4326','EPSG','6024',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4418','projected_crs','EPSG','6089','EPSG','4095','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6090','WGS 84 / EPSG Arctic zone 5-21',NULL,'EPSG','4400','EPSG','4326','EPSG','6025',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4419','projected_crs','EPSG','6090','EPSG','4096','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6091','WGS 84 / EPSG Arctic zone 5-23',NULL,'EPSG','4400','EPSG','4326','EPSG','6026',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4420','projected_crs','EPSG','6091','EPSG','4097','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6092','WGS 84 / EPSG Arctic zone 5-25',NULL,'EPSG','4400','EPSG','4326','EPSG','6027',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4421','projected_crs','EPSG','6092','EPSG','4098','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6093','WGS 84 / EPSG Arctic zone 5-27',NULL,'EPSG','4400','EPSG','4326','EPSG','6028',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4422','projected_crs','EPSG','6093','EPSG','4099','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6094','NAD83(NSRS2007) / EPSG Arctic zone 5-29',NULL,'EPSG','4400','EPSG','4759','EPSG','6029',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4423','projected_crs','EPSG','6094','EPSG','4100','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6095','NAD83(NSRS2007) / EPSG Arctic zone 5-31',NULL,'EPSG','4400','EPSG','4759','EPSG','6030',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4424','projected_crs','EPSG','6095','EPSG','4101','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6096','NAD83(NSRS2007) / EPSG Arctic zone 6-14',NULL,'EPSG','4400','EPSG','4759','EPSG','6039',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4425','projected_crs','EPSG','6096','EPSG','4110','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6097','NAD83(NSRS2007) / EPSG Arctic zone 6-16',NULL,'EPSG','4400','EPSG','4759','EPSG','6040',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4426','projected_crs','EPSG','6097','EPSG','4111','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6098','NAD83(CSRS) / EPSG Arctic zone 1-23',NULL,'EPSG','4400','EPSG','4617','EPSG','5978',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4427','projected_crs','EPSG','6098','EPSG','4047','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6099','NAD83(CSRS) / EPSG Arctic zone 2-14',NULL,'EPSG','4400','EPSG','4617','EPSG','5985',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4428','projected_crs','EPSG','6099','EPSG','4030','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6100','NAD83(CSRS) / EPSG Arctic zone 2-16',NULL,'EPSG','4400','EPSG','4617','EPSG','5986',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4429','projected_crs','EPSG','6100','EPSG','4036','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6101','NAD83(CSRS) / EPSG Arctic zone 3-25',NULL,'EPSG','4400','EPSG','4617','EPSG','6000',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4430','projected_crs','EPSG','6101','EPSG','4065','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6102','NAD83(CSRS) / EPSG Arctic zone 3-27',NULL,'EPSG','4400','EPSG','4617','EPSG','6001',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4431','projected_crs','EPSG','6102','EPSG','4070','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6103','NAD83(CSRS) / EPSG Arctic zone 3-29',NULL,'EPSG','4400','EPSG','4617','EPSG','6002',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4432','projected_crs','EPSG','6103','EPSG','4072','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6104','NAD83(CSRS) / EPSG Arctic zone 4-14',NULL,'EPSG','4400','EPSG','4617','EPSG','6006',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4433','projected_crs','EPSG','6104','EPSG','4077','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6105','NAD83(CSRS) / EPSG Arctic zone 4-16',NULL,'EPSG','4400','EPSG','4617','EPSG','6007',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4434','projected_crs','EPSG','6105','EPSG','4078','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6106','NAD83(CSRS) / EPSG Arctic zone 4-18',NULL,'EPSG','4400','EPSG','4617','EPSG','6008',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4435','projected_crs','EPSG','6106','EPSG','4079','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6107','NAD83(CSRS) / EPSG Arctic zone 5-33',NULL,'EPSG','4400','EPSG','4617','EPSG','6031',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4436','projected_crs','EPSG','6107','EPSG','4102','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6108','NAD83(CSRS) / EPSG Arctic zone 5-35',NULL,'EPSG','4400','EPSG','4617','EPSG','6032',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4437','projected_crs','EPSG','6108','EPSG','4103','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6109','NAD83(CSRS) / EPSG Arctic zone 5-37',NULL,'EPSG','4400','EPSG','4617','EPSG','6033',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4438','projected_crs','EPSG','6109','EPSG','4104','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6110','NAD83(CSRS) / EPSG Arctic zone 5-39',NULL,'EPSG','4400','EPSG','4617','EPSG','6034',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4439','projected_crs','EPSG','6110','EPSG','4105','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6111','NAD83(CSRS) / EPSG Arctic zone 6-18',NULL,'EPSG','4400','EPSG','4617','EPSG','6041',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4440','projected_crs','EPSG','6111','EPSG','4112','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6112','NAD83(CSRS) / EPSG Arctic zone 6-20',NULL,'EPSG','4400','EPSG','4617','EPSG','6042',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4441','projected_crs','EPSG','6112','EPSG','4113','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6113','NAD83(CSRS) / EPSG Arctic zone 6-22',NULL,'EPSG','4400','EPSG','4617','EPSG','6043',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4442','projected_crs','EPSG','6113','EPSG','4114','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6114','NAD83(CSRS) / EPSG Arctic zone 6-24',NULL,'EPSG','4400','EPSG','4617','EPSG','6044',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4443','projected_crs','EPSG','6114','EPSG','4115','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6115','WGS 84 / EPSG Arctic zone 1-27',NULL,'EPSG','4400','EPSG','4326','EPSG','5980',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4444','projected_crs','EPSG','6115','EPSG','4049','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6116','WGS 84 / EPSG Arctic zone 1-29',NULL,'EPSG','4400','EPSG','4326','EPSG','5981',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4445','projected_crs','EPSG','6116','EPSG','4050','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6117','WGS 84 / EPSG Arctic zone 1-31',NULL,'EPSG','4400','EPSG','4326','EPSG','5982',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4446','projected_crs','EPSG','6117','EPSG','4051','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6118','WGS 84 / EPSG Arctic zone 1-21',NULL,'EPSG','4400','EPSG','4326','EPSG','5977',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4447','projected_crs','EPSG','6118','EPSG','4044','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6119','WGS 84 / EPSG Arctic zone 2-28',NULL,'EPSG','4400','EPSG','4326','EPSG','5992',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4448','projected_crs','EPSG','6119','EPSG','4056','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6120','WGS 84 / EPSG Arctic zone 2-10',NULL,'EPSG','4400','EPSG','4326','EPSG','5983',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4449','projected_crs','EPSG','6120','EPSG','4057','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6121','WGS 84 / EPSG Arctic zone 2-12',NULL,'EPSG','4400','EPSG','4326','EPSG','5984',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4450','projected_crs','EPSG','6121','EPSG','4052','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6122','WGS 84 / EPSG Arctic zone 3-21',NULL,'EPSG','4400','EPSG','4326','EPSG','5998',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4451','projected_crs','EPSG','6122','EPSG','4063','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6123','WGS 84 / EPSG Arctic zone 3-23',NULL,'EPSG','4400','EPSG','4326','EPSG','5999',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4452','projected_crs','EPSG','6123','EPSG','4064','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6124','WGS 84 / EPSG Arctic zone 4-12',NULL,'EPSG','4400','EPSG','4326','EPSG','6005',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4453','projected_crs','EPSG','6124','EPSG','4076','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6125','ETRS89 / EPSG Arctic zone 5-47',NULL,'EPSG','4400','EPSG','4258','EPSG','6038',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4454','projected_crs','EPSG','6125','EPSG','4109','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6128','Grand Cayman National Grid 1959',NULL,'EPSG','1039','EPSG','4723','EPSG','6127',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4455','projected_crs','EPSG','6128','EPSG','3185','EPSG','1056');
INSERT INTO "projected_crs" VALUES('EPSG','6129','Sister Islands National Grid 1961',NULL,'EPSG','1039','EPSG','4726','EPSG','6127',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4456','projected_crs','EPSG','6129','EPSG','3186','EPSG','1056');
INSERT INTO "projected_crs" VALUES('EPSG','6141','Cayman Islands National Grid 2011',NULL,'EPSG','1039','EPSG','6135','EPSG','6126',NULL,1);
INSERT INTO "usage" VALUES('EPSG','4463','projected_crs','EPSG','6141','EPSG','1063','EPSG','1027');
INSERT INTO "projected_crs" VALUES('EPSG','6200','NAD27 / Michigan North',NULL,'EPSG','4497','EPSG','4267','EPSG','6197',NULL,1);
INSERT INTO "usage" VALUES('EPSG','4508','projected_crs','EPSG','6200','EPSG','1723','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6201','NAD27 / Michigan Central',NULL,'EPSG','4497','EPSG','4267','EPSG','6198',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4509','projected_crs','EPSG','6201','EPSG','1724','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6202','NAD27 / Michigan South',NULL,'EPSG','4497','EPSG','4267','EPSG','6199',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4510','projected_crs','EPSG','6202','EPSG','1725','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6204','Macedonia State Coordinate System',NULL,'EPSG','4498','EPSG','3906','EPSG','6203',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4511','projected_crs','EPSG','6204','EPSG','1148','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6210','SIRGAS 2000 / UTM zone 23N',NULL,'EPSG','4400','EPSG','4674','EPSG','16023',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4513','projected_crs','EPSG','6210','EPSG','4129','EPSG','1216');
INSERT INTO "projected_crs" VALUES('EPSG','6211','SIRGAS 2000 / UTM zone 24N',NULL,'EPSG','4400','EPSG','4674','EPSG','16024',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4514','projected_crs','EPSG','6211','EPSG','4133','EPSG','1216');
INSERT INTO "projected_crs" VALUES('EPSG','6244','MAGNA-SIRGAS / Arauca urban grid',NULL,'EPSG','4500','EPSG','4686','EPSG','6212',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4515','projected_crs','EPSG','6244','EPSG','4122','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','6245','MAGNA-SIRGAS / Armenia urban grid',NULL,'EPSG','4500','EPSG','4686','EPSG','6213',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4516','projected_crs','EPSG','6245','EPSG','4132','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','6246','MAGNA-SIRGAS / Barranquilla urban grid',NULL,'EPSG','4500','EPSG','4686','EPSG','6214',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4517','projected_crs','EPSG','6246','EPSG','4134','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','6247','MAGNA-SIRGAS / Bogota urban grid',NULL,'EPSG','4500','EPSG','4686','EPSG','6215',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4518','projected_crs','EPSG','6247','EPSG','4135','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','6248','MAGNA-SIRGAS / Bucaramanga urban grid',NULL,'EPSG','4500','EPSG','4686','EPSG','6216',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4519','projected_crs','EPSG','6248','EPSG','4136','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','6249','MAGNA-SIRGAS / Cali urban grid',NULL,'EPSG','4500','EPSG','4686','EPSG','6217',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4520','projected_crs','EPSG','6249','EPSG','4137','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','6250','MAGNA-SIRGAS / Cartagena urban grid',NULL,'EPSG','4500','EPSG','4686','EPSG','6218',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4521','projected_crs','EPSG','6250','EPSG','4138','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','6251','MAGNA-SIRGAS / Cucuta urban grid',NULL,'EPSG','4500','EPSG','4686','EPSG','6219',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4522','projected_crs','EPSG','6251','EPSG','4139','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','6252','MAGNA-SIRGAS / Florencia urban grid',NULL,'EPSG','4500','EPSG','4686','EPSG','6220',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4523','projected_crs','EPSG','6252','EPSG','4140','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','6253','MAGNA-SIRGAS / Ibague urban grid',NULL,'EPSG','4500','EPSG','4686','EPSG','6221',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4524','projected_crs','EPSG','6253','EPSG','4141','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','6254','MAGNA-SIRGAS / Inirida urban grid',NULL,'EPSG','4500','EPSG','4686','EPSG','6222',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4525','projected_crs','EPSG','6254','EPSG','4142','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','6255','MAGNA-SIRGAS / Leticia urban grid',NULL,'EPSG','4500','EPSG','4686','EPSG','6223',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4526','projected_crs','EPSG','6255','EPSG','4143','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','6256','MAGNA-SIRGAS / Manizales urban grid',NULL,'EPSG','4500','EPSG','4686','EPSG','6224',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4527','projected_crs','EPSG','6256','EPSG','4144','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','6257','MAGNA-SIRGAS / Medellin urban grid',NULL,'EPSG','4500','EPSG','4686','EPSG','6225',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4528','projected_crs','EPSG','6257','EPSG','4145','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','6258','MAGNA-SIRGAS / Mitu urban grid',NULL,'EPSG','4500','EPSG','4686','EPSG','6226',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4529','projected_crs','EPSG','6258','EPSG','4146','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','6259','MAGNA-SIRGAS / Mocoa urban grid',NULL,'EPSG','4500','EPSG','4686','EPSG','6227',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4530','projected_crs','EPSG','6259','EPSG','4147','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','6260','MAGNA-SIRGAS / Monteria urban grid',NULL,'EPSG','4500','EPSG','4686','EPSG','6228',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4531','projected_crs','EPSG','6260','EPSG','4148','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','6261','MAGNA-SIRGAS / Neiva urban grid',NULL,'EPSG','4500','EPSG','4686','EPSG','6229',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4532','projected_crs','EPSG','6261','EPSG','4149','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','6262','MAGNA-SIRGAS / Pasto urban grid',NULL,'EPSG','4500','EPSG','4686','EPSG','6230',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4533','projected_crs','EPSG','6262','EPSG','4150','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','6263','MAGNA-SIRGAS / Pereira urban grid',NULL,'EPSG','4500','EPSG','4686','EPSG','6231',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4534','projected_crs','EPSG','6263','EPSG','4151','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','6264','MAGNA-SIRGAS / Popayan urban grid',NULL,'EPSG','4500','EPSG','4686','EPSG','6232',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4535','projected_crs','EPSG','6264','EPSG','4152','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','6265','MAGNA-SIRGAS / Puerto Carreno urban grid',NULL,'EPSG','4500','EPSG','4686','EPSG','6233',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4536','projected_crs','EPSG','6265','EPSG','4153','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','6266','MAGNA-SIRGAS / Quibdo urban grid',NULL,'EPSG','4500','EPSG','4686','EPSG','6234',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4537','projected_crs','EPSG','6266','EPSG','4154','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','6267','MAGNA-SIRGAS / Riohacha urban grid',NULL,'EPSG','4500','EPSG','4686','EPSG','6235',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4538','projected_crs','EPSG','6267','EPSG','4155','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','6268','MAGNA-SIRGAS / San Andres urban grid',NULL,'EPSG','4500','EPSG','4686','EPSG','6236',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4539','projected_crs','EPSG','6268','EPSG','4156','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','6269','MAGNA-SIRGAS / San Jose del Guaviare urban grid',NULL,'EPSG','4500','EPSG','4686','EPSG','6237',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4540','projected_crs','EPSG','6269','EPSG','4157','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','6270','MAGNA-SIRGAS / Santa Marta urban grid',NULL,'EPSG','4500','EPSG','4686','EPSG','6238',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4541','projected_crs','EPSG','6270','EPSG','4128','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','6271','MAGNA-SIRGAS / Sucre urban grid',NULL,'EPSG','4500','EPSG','4686','EPSG','6239',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4542','projected_crs','EPSG','6271','EPSG','4130','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','6272','MAGNA-SIRGAS / Tunja urban grid',NULL,'EPSG','4500','EPSG','4686','EPSG','6240',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4543','projected_crs','EPSG','6272','EPSG','4131','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','6273','MAGNA-SIRGAS / Valledupar urban grid',NULL,'EPSG','4500','EPSG','4686','EPSG','6241',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4544','projected_crs','EPSG','6273','EPSG','4158','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','6274','MAGNA-SIRGAS / Villavicencio urban grid',NULL,'EPSG','4500','EPSG','4686','EPSG','6242',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4545','projected_crs','EPSG','6274','EPSG','4159','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','6275','MAGNA-SIRGAS / Yopal urban grid',NULL,'EPSG','4500','EPSG','4686','EPSG','6243',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4546','projected_crs','EPSG','6275','EPSG','4160','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','6307','NAD83(CORS96) / Puerto Rico and Virgin Is.',NULL,'EPSG','4499','EPSG','6783','EPSG','15230',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4547','projected_crs','EPSG','6307','EPSG','3634','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6312','CGRS93 / Cyprus Local Transverse Mercator',NULL,'EPSG','4400','EPSG','6311','EPSG','6308',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4551','projected_crs','EPSG','6312','EPSG','3236','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','6316','MGI 1901 / Balkans zone 7',NULL,'EPSG','4498','EPSG','3906','EPSG','18277',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4552','projected_crs','EPSG','6316','EPSG','1711','EPSG','1092');
INSERT INTO "usage" VALUES('EPSG','17354','projected_crs','EPSG','6316','EPSG','1148','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','6328','NAD83(2011) / UTM zone 59N',NULL,'EPSG','4400','EPSG','6318','EPSG','16059',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4562','projected_crs','EPSG','6328','EPSG','3372','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','6329','NAD83(2011) / UTM zone 60N',NULL,'EPSG','4400','EPSG','6318','EPSG','16060',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4563','projected_crs','EPSG','6329','EPSG','3373','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','6330','NAD83(2011) / UTM zone 1N',NULL,'EPSG','4400','EPSG','6318','EPSG','16001',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4564','projected_crs','EPSG','6330','EPSG','3374','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','6331','NAD83(2011) / UTM zone 2N',NULL,'EPSG','4400','EPSG','6318','EPSG','16002',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4565','projected_crs','EPSG','6331','EPSG','3375','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','6332','NAD83(2011) / UTM zone 3N',NULL,'EPSG','4400','EPSG','6318','EPSG','16003',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4566','projected_crs','EPSG','6332','EPSG','2133','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6333','NAD83(2011) / UTM zone 4N',NULL,'EPSG','4400','EPSG','6318','EPSG','16004',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4567','projected_crs','EPSG','6333','EPSG','2134','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6334','NAD83(2011) / UTM zone 5N',NULL,'EPSG','4400','EPSG','6318','EPSG','16005',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4568','projected_crs','EPSG','6334','EPSG','2135','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6335','NAD83(2011) / UTM zone 6N',NULL,'EPSG','4400','EPSG','6318','EPSG','16006',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4569','projected_crs','EPSG','6335','EPSG','2136','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6336','NAD83(2011) / UTM zone 7N',NULL,'EPSG','4400','EPSG','6318','EPSG','16007',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4570','projected_crs','EPSG','6336','EPSG','3494','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6337','NAD83(2011) / UTM zone 8N',NULL,'EPSG','4400','EPSG','6318','EPSG','16008',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4571','projected_crs','EPSG','6337','EPSG','3495','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6338','NAD83(2011) / UTM zone 9N',NULL,'EPSG','4400','EPSG','6318','EPSG','16009',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4572','projected_crs','EPSG','6338','EPSG','3496','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6339','NAD83(2011) / UTM zone 10N',NULL,'EPSG','4400','EPSG','6318','EPSG','16010',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4573','projected_crs','EPSG','6339','EPSG','3497','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6340','NAD83(2011) / UTM zone 11N',NULL,'EPSG','4400','EPSG','6318','EPSG','16011',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4574','projected_crs','EPSG','6340','EPSG','3498','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6341','NAD83(2011) / UTM zone 12N',NULL,'EPSG','4400','EPSG','6318','EPSG','16012',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4575','projected_crs','EPSG','6341','EPSG','3499','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6342','NAD83(2011) / UTM zone 13N',NULL,'EPSG','4400','EPSG','6318','EPSG','16013',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4576','projected_crs','EPSG','6342','EPSG','3500','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6343','NAD83(2011) / UTM zone 14N',NULL,'EPSG','4400','EPSG','6318','EPSG','16014',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4577','projected_crs','EPSG','6343','EPSG','3501','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6344','NAD83(2011) / UTM zone 15N',NULL,'EPSG','4400','EPSG','6318','EPSG','16015',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4578','projected_crs','EPSG','6344','EPSG','3502','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6345','NAD83(2011) / UTM zone 16N',NULL,'EPSG','4400','EPSG','6318','EPSG','16016',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4579','projected_crs','EPSG','6345','EPSG','3503','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6346','NAD83(2011) / UTM zone 17N',NULL,'EPSG','4400','EPSG','6318','EPSG','16017',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4580','projected_crs','EPSG','6346','EPSG','3504','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6347','NAD83(2011) / UTM zone 18N',NULL,'EPSG','4400','EPSG','6318','EPSG','16018',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4581','projected_crs','EPSG','6347','EPSG','3505','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6348','NAD83(2011) / UTM zone 19N',NULL,'EPSG','4400','EPSG','6318','EPSG','16019',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4582','projected_crs','EPSG','6348','EPSG','3506','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6350','NAD83(2011) / Conus Albers',NULL,'EPSG','4499','EPSG','6318','EPSG','5068',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4584','projected_crs','EPSG','6350','EPSG','1323','EPSG','1109');
INSERT INTO "projected_crs" VALUES('EPSG','6351','NAD83(2011) / EPSG Arctic zone 5-29',NULL,'EPSG','4400','EPSG','6318','EPSG','6029',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4585','projected_crs','EPSG','6351','EPSG','4100','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6352','NAD83(2011) / EPSG Arctic zone 5-31',NULL,'EPSG','4400','EPSG','6318','EPSG','6030',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4586','projected_crs','EPSG','6352','EPSG','4101','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6353','NAD83(2011) / EPSG Arctic zone 6-14',NULL,'EPSG','4400','EPSG','6318','EPSG','6039',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4587','projected_crs','EPSG','6353','EPSG','4110','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6354','NAD83(2011) / EPSG Arctic zone 6-16',NULL,'EPSG','4400','EPSG','6318','EPSG','6040',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4588','projected_crs','EPSG','6354','EPSG','4111','EPSG','1148');
INSERT INTO "projected_crs" VALUES('EPSG','6355','NAD83(2011) / Alabama East',NULL,'EPSG','4499','EPSG','6318','EPSG','10131',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4589','projected_crs','EPSG','6355','EPSG','2154','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6356','NAD83(2011) / Alabama West',NULL,'EPSG','4499','EPSG','6318','EPSG','10132',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4590','projected_crs','EPSG','6356','EPSG','2155','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6362','Mexico ITRF92 / LCC',NULL,'EPSG','4500','EPSG','4483','EPSG','6361',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4595','projected_crs','EPSG','6362','EPSG','1160','EPSG','1241');
INSERT INTO "projected_crs" VALUES('EPSG','6366','Mexico ITRF2008 / UTM zone 11N',NULL,'EPSG','4400','EPSG','6365','EPSG','16011',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4599','projected_crs','EPSG','6366','EPSG','3423','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6367','Mexico ITRF2008 / UTM zone 12N',NULL,'EPSG','4400','EPSG','6365','EPSG','16012',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4600','projected_crs','EPSG','6367','EPSG','3424','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6368','Mexico ITRF2008 / UTM zone 13N',NULL,'EPSG','4400','EPSG','6365','EPSG','16013',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4601','projected_crs','EPSG','6368','EPSG','3425','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6369','Mexico ITRF2008 / UTM zone 14N',NULL,'EPSG','4400','EPSG','6365','EPSG','16014',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4602','projected_crs','EPSG','6369','EPSG','3426','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6370','Mexico ITRF2008 / UTM zone 15N',NULL,'EPSG','4400','EPSG','6365','EPSG','16015',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4603','projected_crs','EPSG','6370','EPSG','3633','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6371','Mexico ITRF2008 / UTM zone 16N',NULL,'EPSG','4400','EPSG','6365','EPSG','16016',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4604','projected_crs','EPSG','6371','EPSG','3635','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6372','Mexico ITRF2008 / LCC',NULL,'EPSG','4500','EPSG','6365','EPSG','6361',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4605','projected_crs','EPSG','6372','EPSG','1160','EPSG','1241');
INSERT INTO "projected_crs" VALUES('EPSG','6381','UCS-2000 / Ukraine TM zone 7',NULL,'EPSG','4530','EPSG','5561','EPSG','6374',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4606','projected_crs','EPSG','6381','EPSG','3906','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','6382','UCS-2000 / Ukraine TM zone 8',NULL,'EPSG','4530','EPSG','5561','EPSG','6375',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4607','projected_crs','EPSG','6382','EPSG','3907','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','6383','UCS-2000 / Ukraine TM zone 9',NULL,'EPSG','4530','EPSG','5561','EPSG','6376',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4608','projected_crs','EPSG','6383','EPSG','3908','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','6384','UCS-2000 / Ukraine TM zone 10',NULL,'EPSG','4530','EPSG','5561','EPSG','6377',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4609','projected_crs','EPSG','6384','EPSG','3909','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','6385','UCS-2000 / Ukraine TM zone 11',NULL,'EPSG','4530','EPSG','5561','EPSG','6378',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4610','projected_crs','EPSG','6385','EPSG','3910','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','6386','UCS-2000 / Ukraine TM zone 12',NULL,'EPSG','4530','EPSG','5561','EPSG','6379',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4611','projected_crs','EPSG','6386','EPSG','3912','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','6387','UCS-2000 / Ukraine TM zone 13',NULL,'EPSG','4530','EPSG','5561','EPSG','6380',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4612','projected_crs','EPSG','6387','EPSG','3913','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','6391','Cayman Islands National Grid 2011',NULL,'EPSG','1039','EPSG','6135','EPSG','6390',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4613','projected_crs','EPSG','6391','EPSG','1063','EPSG','1027');
INSERT INTO "projected_crs" VALUES('EPSG','6393','NAD83(2011) / Alaska Albers',NULL,'EPSG','4499','EPSG','6318','EPSG','15021',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4614','projected_crs','EPSG','6393','EPSG','1330','EPSG','1241');
INSERT INTO "projected_crs" VALUES('EPSG','6394','NAD83(2011) / Alaska zone 1',NULL,'EPSG','4499','EPSG','6318','EPSG','15031',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4615','projected_crs','EPSG','6394','EPSG','2156','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6395','NAD83(2011) / Alaska zone 2',NULL,'EPSG','4499','EPSG','6318','EPSG','15032',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4616','projected_crs','EPSG','6395','EPSG','2158','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6396','NAD83(2011) / Alaska zone 3',NULL,'EPSG','4499','EPSG','6318','EPSG','15033',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4617','projected_crs','EPSG','6396','EPSG','2159','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6397','NAD83(2011) / Alaska zone 4',NULL,'EPSG','4499','EPSG','6318','EPSG','15034',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4618','projected_crs','EPSG','6397','EPSG','2160','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6398','NAD83(2011) / Alaska zone 5',NULL,'EPSG','4499','EPSG','6318','EPSG','15035',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4619','projected_crs','EPSG','6398','EPSG','2161','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6399','NAD83(2011) / Alaska zone 6',NULL,'EPSG','4499','EPSG','6318','EPSG','15036',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4620','projected_crs','EPSG','6399','EPSG','2162','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6400','NAD83(2011) / Alaska zone 7',NULL,'EPSG','4499','EPSG','6318','EPSG','15037',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4621','projected_crs','EPSG','6400','EPSG','2163','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6401','NAD83(2011) / Alaska zone 8',NULL,'EPSG','4499','EPSG','6318','EPSG','15038',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4622','projected_crs','EPSG','6401','EPSG','2164','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6402','NAD83(2011) / Alaska zone 9',NULL,'EPSG','4499','EPSG','6318','EPSG','15039',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4623','projected_crs','EPSG','6402','EPSG','2165','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6403','NAD83(2011) / Alaska zone 10',NULL,'EPSG','4499','EPSG','6318','EPSG','15040',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4624','projected_crs','EPSG','6403','EPSG','2157','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6404','NAD83(2011) / Arizona Central',NULL,'EPSG','4499','EPSG','6318','EPSG','10232',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4625','projected_crs','EPSG','6404','EPSG','2166','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6405','NAD83(2011) / Arizona Central (ft)',NULL,'EPSG','4495','EPSG','6318','EPSG','15305',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4626','projected_crs','EPSG','6405','EPSG','2166','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6406','NAD83(2011) / Arizona East',NULL,'EPSG','4499','EPSG','6318','EPSG','10231',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4627','projected_crs','EPSG','6406','EPSG','2167','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6407','NAD83(2011) / Arizona East (ft)',NULL,'EPSG','4495','EPSG','6318','EPSG','15304',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4628','projected_crs','EPSG','6407','EPSG','2167','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6408','NAD83(2011) / Arizona West',NULL,'EPSG','4499','EPSG','6318','EPSG','10233',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4629','projected_crs','EPSG','6408','EPSG','2168','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6409','NAD83(2011) / Arizona West (ft)',NULL,'EPSG','4495','EPSG','6318','EPSG','15306',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4630','projected_crs','EPSG','6409','EPSG','2168','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6410','NAD83(2011) / Arkansas North',NULL,'EPSG','4499','EPSG','6318','EPSG','10331',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4631','projected_crs','EPSG','6410','EPSG','2169','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6411','NAD83(2011) / Arkansas North (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15385',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4632','projected_crs','EPSG','6411','EPSG','2169','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6412','NAD83(2011) / Arkansas South',NULL,'EPSG','4499','EPSG','6318','EPSG','10332',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4633','projected_crs','EPSG','6412','EPSG','2170','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6413','NAD83(2011) / Arkansas South (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15386',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4634','projected_crs','EPSG','6413','EPSG','2170','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6414','NAD83(2011) / California Albers',NULL,'EPSG','4499','EPSG','6318','EPSG','10420',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4635','projected_crs','EPSG','6414','EPSG','1375','EPSG','1135');
INSERT INTO "projected_crs" VALUES('EPSG','6415','NAD83(2011) / California zone 1',NULL,'EPSG','4499','EPSG','6318','EPSG','10431',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4636','projected_crs','EPSG','6415','EPSG','2175','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6416','NAD83(2011) / California zone 1 (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15307',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4637','projected_crs','EPSG','6416','EPSG','2175','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6417','NAD83(2011) / California zone 2',NULL,'EPSG','4499','EPSG','6318','EPSG','10432',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4638','projected_crs','EPSG','6417','EPSG','2176','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6418','NAD83(2011) / California zone 2 (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15308',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4639','projected_crs','EPSG','6418','EPSG','2176','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6419','NAD83(2011) / California zone 3',NULL,'EPSG','4499','EPSG','6318','EPSG','10433',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4640','projected_crs','EPSG','6419','EPSG','2177','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6420','NAD83(2011) / California zone 3 (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15309',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4641','projected_crs','EPSG','6420','EPSG','2177','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6421','NAD83(2011) / California zone 4',NULL,'EPSG','4499','EPSG','6318','EPSG','10434',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4642','projected_crs','EPSG','6421','EPSG','2178','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6422','NAD83(2011) / California zone 4 (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15310',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4643','projected_crs','EPSG','6422','EPSG','2178','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6423','NAD83(2011) / California zone 5',NULL,'EPSG','4499','EPSG','6318','EPSG','10435',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4644','projected_crs','EPSG','6423','EPSG','2182','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6424','NAD83(2011) / California zone 5 (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15311',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4645','projected_crs','EPSG','6424','EPSG','2182','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6425','NAD83(2011) / California zone 6',NULL,'EPSG','4499','EPSG','6318','EPSG','10436',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4646','projected_crs','EPSG','6425','EPSG','2180','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6426','NAD83(2011) / California zone 6 (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15312',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4647','projected_crs','EPSG','6426','EPSG','2180','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6427','NAD83(2011) / Colorado Central',NULL,'EPSG','4499','EPSG','6318','EPSG','10532',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4648','projected_crs','EPSG','6427','EPSG','2183','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6428','NAD83(2011) / Colorado Central (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15314',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4649','projected_crs','EPSG','6428','EPSG','2183','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6429','NAD83(2011) / Colorado North',NULL,'EPSG','4499','EPSG','6318','EPSG','10531',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4650','projected_crs','EPSG','6429','EPSG','2184','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6430','NAD83(2011) / Colorado North (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15313',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4651','projected_crs','EPSG','6430','EPSG','2184','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6431','NAD83(2011) / Colorado South',NULL,'EPSG','4499','EPSG','6318','EPSG','10533',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4652','projected_crs','EPSG','6431','EPSG','2185','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6432','NAD83(2011) / Colorado South (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15315',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4653','projected_crs','EPSG','6432','EPSG','2185','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6433','NAD83(2011) / Connecticut',NULL,'EPSG','4499','EPSG','6318','EPSG','10630',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4654','projected_crs','EPSG','6433','EPSG','1377','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6434','NAD83(2011) / Connecticut (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15316',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4655','projected_crs','EPSG','6434','EPSG','1377','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6435','NAD83(2011) / Delaware',NULL,'EPSG','4499','EPSG','6318','EPSG','10730',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4656','projected_crs','EPSG','6435','EPSG','1378','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6436','NAD83(2011) / Delaware (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15317',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4657','projected_crs','EPSG','6436','EPSG','1378','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6437','NAD83(2011) / Florida East',NULL,'EPSG','4499','EPSG','6318','EPSG','10931',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4658','projected_crs','EPSG','6437','EPSG','2186','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6438','NAD83(2011) / Florida East (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15318',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4659','projected_crs','EPSG','6438','EPSG','2186','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6439','NAD83(2011) / Florida GDL Albers',NULL,'EPSG','4499','EPSG','6318','EPSG','10934',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4660','projected_crs','EPSG','6439','EPSG','1379','EPSG','1135');
INSERT INTO "projected_crs" VALUES('EPSG','6440','NAD83(2011) / Florida North',NULL,'EPSG','4499','EPSG','6318','EPSG','10933',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4661','projected_crs','EPSG','6440','EPSG','2187','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6441','NAD83(2011) / Florida North (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15320',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4662','projected_crs','EPSG','6441','EPSG','2187','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6442','NAD83(2011) / Florida West',NULL,'EPSG','4499','EPSG','6318','EPSG','10932',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4663','projected_crs','EPSG','6442','EPSG','2188','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6443','NAD83(2011) / Florida West (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15319',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4664','projected_crs','EPSG','6443','EPSG','2188','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6444','NAD83(2011) / Georgia East',NULL,'EPSG','4499','EPSG','6318','EPSG','11031',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4665','projected_crs','EPSG','6444','EPSG','2189','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6445','NAD83(2011) / Georgia East (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15321',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4666','projected_crs','EPSG','6445','EPSG','2189','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6446','NAD83(2011) / Georgia West',NULL,'EPSG','4499','EPSG','6318','EPSG','11032',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4667','projected_crs','EPSG','6446','EPSG','2190','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6447','NAD83(2011) / Georgia West (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15322',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4668','projected_crs','EPSG','6447','EPSG','2190','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6448','NAD83(2011) / Idaho Central',NULL,'EPSG','4499','EPSG','6318','EPSG','11132',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4669','projected_crs','EPSG','6448','EPSG','2191','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6449','NAD83(2011) / Idaho Central (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15324',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4670','projected_crs','EPSG','6449','EPSG','2191','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6450','NAD83(2011) / Idaho East',NULL,'EPSG','4499','EPSG','6318','EPSG','11131',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4671','projected_crs','EPSG','6450','EPSG','2192','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6451','NAD83(2011) / Idaho East (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15323',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4672','projected_crs','EPSG','6451','EPSG','2192','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6452','NAD83(2011) / Idaho West',NULL,'EPSG','4499','EPSG','6318','EPSG','11133',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4673','projected_crs','EPSG','6452','EPSG','2193','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6453','NAD83(2011) / Idaho West (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15325',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4674','projected_crs','EPSG','6453','EPSG','2193','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6454','NAD83(2011) / Illinois East',NULL,'EPSG','4499','EPSG','6318','EPSG','11231',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4675','projected_crs','EPSG','6454','EPSG','2194','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6455','NAD83(2011) / Illinois East (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15387',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4676','projected_crs','EPSG','6455','EPSG','2194','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6456','NAD83(2011) / Illinois West',NULL,'EPSG','4499','EPSG','6318','EPSG','11232',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4677','projected_crs','EPSG','6456','EPSG','2195','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6457','NAD83(2011) / Illinois West (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15388',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4678','projected_crs','EPSG','6457','EPSG','2195','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6458','NAD83(2011) / Indiana East',NULL,'EPSG','4499','EPSG','6318','EPSG','11331',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4679','projected_crs','EPSG','6458','EPSG','2196','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6459','NAD83(2011) / Indiana East (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15372',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4680','projected_crs','EPSG','6459','EPSG','2196','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6460','NAD83(2011) / Indiana West',NULL,'EPSG','4499','EPSG','6318','EPSG','11332',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4681','projected_crs','EPSG','6460','EPSG','2197','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6461','NAD83(2011) / Indiana West (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15373',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4682','projected_crs','EPSG','6461','EPSG','2197','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6462','NAD83(2011) / Iowa North',NULL,'EPSG','4499','EPSG','6318','EPSG','11431',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4683','projected_crs','EPSG','6462','EPSG','2198','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6463','NAD83(2011) / Iowa North (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15377',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4684','projected_crs','EPSG','6463','EPSG','2198','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6464','NAD83(2011) / Iowa South',NULL,'EPSG','4499','EPSG','6318','EPSG','11432',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4685','projected_crs','EPSG','6464','EPSG','2199','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6465','NAD83(2011) / Iowa South (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15378',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4686','projected_crs','EPSG','6465','EPSG','2199','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6466','NAD83(2011) / Kansas North',NULL,'EPSG','4499','EPSG','6318','EPSG','11531',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4687','projected_crs','EPSG','6466','EPSG','2200','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6467','NAD83(2011) / Kansas North (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15379',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4688','projected_crs','EPSG','6467','EPSG','2200','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6468','NAD83(2011) / Kansas South',NULL,'EPSG','4499','EPSG','6318','EPSG','11532',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4689','projected_crs','EPSG','6468','EPSG','2201','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6469','NAD83(2011) / Kansas South (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15380',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4690','projected_crs','EPSG','6469','EPSG','2201','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6470','NAD83(2011) / Kentucky North',NULL,'EPSG','4499','EPSG','6318','EPSG','15303',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4691','projected_crs','EPSG','6470','EPSG','2202','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6471','NAD83(2011) / Kentucky North (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15328',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4692','projected_crs','EPSG','6471','EPSG','2202','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6472','NAD83(2011) / Kentucky Single Zone',NULL,'EPSG','4499','EPSG','6318','EPSG','11630',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4693','projected_crs','EPSG','6472','EPSG','1386','EPSG','1135');
INSERT INTO "projected_crs" VALUES('EPSG','6473','NAD83(2011) / Kentucky Single Zone (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15375',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4694','projected_crs','EPSG','6473','EPSG','1386','EPSG','1135');
INSERT INTO "projected_crs" VALUES('EPSG','6474','NAD83(2011) / Kentucky South',NULL,'EPSG','4499','EPSG','6318','EPSG','11632',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4695','projected_crs','EPSG','6474','EPSG','2203','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6475','NAD83(2011) / Kentucky South (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15329',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4696','projected_crs','EPSG','6475','EPSG','2203','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6476','NAD83(2011) / Louisiana North',NULL,'EPSG','4499','EPSG','6318','EPSG','11731',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4697','projected_crs','EPSG','6476','EPSG','2204','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6477','NAD83(2011) / Louisiana North (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15391',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4698','projected_crs','EPSG','6477','EPSG','2204','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6478','NAD83(2011) / Louisiana South',NULL,'EPSG','4499','EPSG','6318','EPSG','11732',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4699','projected_crs','EPSG','6478','EPSG','2529','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6479','NAD83(2011) / Louisiana South (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15392',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4700','projected_crs','EPSG','6479','EPSG','2529','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6480','NAD83(2011) / Maine CS2000 Central',NULL,'EPSG','4499','EPSG','6318','EPSG','11854',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4701','projected_crs','EPSG','6480','EPSG','2959','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6481','NAD83(2011) / Maine CS2000 East',NULL,'EPSG','4499','EPSG','6318','EPSG','11851',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4702','projected_crs','EPSG','6481','EPSG','2960','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6482','NAD83(2011) / Maine CS2000 West',NULL,'EPSG','4499','EPSG','6318','EPSG','11853',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4703','projected_crs','EPSG','6482','EPSG','2958','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6483','NAD83(2011) / Maine East',NULL,'EPSG','4499','EPSG','6318','EPSG','11831',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4704','projected_crs','EPSG','6483','EPSG','2206','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6484','NAD83(2011) / Maine East (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','11833',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4705','projected_crs','EPSG','6484','EPSG','2206','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6485','NAD83(2011) / Maine West',NULL,'EPSG','4499','EPSG','6318','EPSG','11832',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4706','projected_crs','EPSG','6485','EPSG','2207','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6486','NAD83(2011) / Maine West (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','11834',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4707','projected_crs','EPSG','6486','EPSG','2207','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6487','NAD83(2011) / Maryland',NULL,'EPSG','4499','EPSG','6318','EPSG','11930',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4708','projected_crs','EPSG','6487','EPSG','1389','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6488','NAD83(2011) / Maryland (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15330',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4709','projected_crs','EPSG','6488','EPSG','1389','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6489','NAD83(2011) / Massachusetts Island',NULL,'EPSG','4499','EPSG','6318','EPSG','12032',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4710','projected_crs','EPSG','6489','EPSG','2208','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6490','NAD83(2011) / Massachusetts Island (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15332',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4711','projected_crs','EPSG','6490','EPSG','2208','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6491','NAD83(2011) / Massachusetts Mainland',NULL,'EPSG','4499','EPSG','6318','EPSG','12031',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4712','projected_crs','EPSG','6491','EPSG','2209','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6492','NAD83(2011) / Massachusetts Mainland (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15331',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4713','projected_crs','EPSG','6492','EPSG','2209','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6493','NAD83(2011) / Michigan Central',NULL,'EPSG','4499','EPSG','6318','EPSG','12142',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4714','projected_crs','EPSG','6493','EPSG','1724','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6494','NAD83(2011) / Michigan Central (ft)',NULL,'EPSG','4495','EPSG','6318','EPSG','15334',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4715','projected_crs','EPSG','6494','EPSG','1724','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6495','NAD83(2011) / Michigan North',NULL,'EPSG','4499','EPSG','6318','EPSG','12141',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4716','projected_crs','EPSG','6495','EPSG','1723','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6496','NAD83(2011) / Michigan North (ft)',NULL,'EPSG','4495','EPSG','6318','EPSG','15333',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4717','projected_crs','EPSG','6496','EPSG','1723','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6497','NAD83(2011) / Michigan Oblique Mercator',NULL,'EPSG','4499','EPSG','6318','EPSG','12150',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4718','projected_crs','EPSG','6497','EPSG','1391','EPSG','1135');
INSERT INTO "projected_crs" VALUES('EPSG','6498','NAD83(2011) / Michigan South',NULL,'EPSG','4499','EPSG','6318','EPSG','12143',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4719','projected_crs','EPSG','6498','EPSG','1725','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6499','NAD83(2011) / Michigan South (ft)',NULL,'EPSG','4495','EPSG','6318','EPSG','15335',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4720','projected_crs','EPSG','6499','EPSG','1725','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6500','NAD83(2011) / Minnesota Central',NULL,'EPSG','4499','EPSG','6318','EPSG','12232',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4721','projected_crs','EPSG','6500','EPSG','2213','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6501','NAD83(2011) / Minnesota Central (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','12235',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4722','projected_crs','EPSG','6501','EPSG','2213','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6502','NAD83(2011) / Minnesota North',NULL,'EPSG','4499','EPSG','6318','EPSG','12231',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4723','projected_crs','EPSG','6502','EPSG','2214','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6503','NAD83(2011) / Minnesota North (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','12234',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4724','projected_crs','EPSG','6503','EPSG','2214','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6504','NAD83(2011) / Minnesota South',NULL,'EPSG','4499','EPSG','6318','EPSG','12233',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4725','projected_crs','EPSG','6504','EPSG','2215','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6505','NAD83(2011) / Minnesota South (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','12236',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4726','projected_crs','EPSG','6505','EPSG','2215','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6506','NAD83(2011) / Mississippi East',NULL,'EPSG','4499','EPSG','6318','EPSG','12331',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4727','projected_crs','EPSG','6506','EPSG','2216','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6507','NAD83(2011) / Mississippi East (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15336',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4728','projected_crs','EPSG','6507','EPSG','2216','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6508','NAD83(2011) / Mississippi TM',NULL,'EPSG','4499','EPSG','6318','EPSG','3813',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4729','projected_crs','EPSG','6508','EPSG','1393','EPSG','1135');
INSERT INTO "projected_crs" VALUES('EPSG','6509','NAD83(2011) / Mississippi West',NULL,'EPSG','4499','EPSG','6318','EPSG','12332',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4730','projected_crs','EPSG','6509','EPSG','2217','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6510','NAD83(2011) / Mississippi West (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15337',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4731','projected_crs','EPSG','6510','EPSG','2217','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6511','NAD83(2011) / Missouri Central',NULL,'EPSG','4499','EPSG','6318','EPSG','12432',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4732','projected_crs','EPSG','6511','EPSG','2218','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6512','NAD83(2011) / Missouri East',NULL,'EPSG','4499','EPSG','6318','EPSG','12431',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4733','projected_crs','EPSG','6512','EPSG','2219','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6513','NAD83(2011) / Missouri West',NULL,'EPSG','4499','EPSG','6318','EPSG','12433',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4734','projected_crs','EPSG','6513','EPSG','2220','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6514','NAD83(2011) / Montana',NULL,'EPSG','4499','EPSG','6318','EPSG','12530',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4735','projected_crs','EPSG','6514','EPSG','1395','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6515','NAD83(2011) / Montana (ft)',NULL,'EPSG','4495','EPSG','6318','EPSG','15338',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4736','projected_crs','EPSG','6515','EPSG','1395','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6516','NAD83(2011) / Nebraska',NULL,'EPSG','4499','EPSG','6318','EPSG','12630',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4737','projected_crs','EPSG','6516','EPSG','1396','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6517','NAD83(2011) / Nebraska (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15396',NULL,1);
INSERT INTO "usage" VALUES('EPSG','4738','projected_crs','EPSG','6517','EPSG','1396','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6518','NAD83(2011) / Nevada Central',NULL,'EPSG','4499','EPSG','6318','EPSG','12732',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4739','projected_crs','EPSG','6518','EPSG','2223','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6519','NAD83(2011) / Nevada Central (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15382',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4740','projected_crs','EPSG','6519','EPSG','2223','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6520','NAD83(2011) / Nevada East',NULL,'EPSG','4499','EPSG','6318','EPSG','12731',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4741','projected_crs','EPSG','6520','EPSG','2224','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6521','NAD83(2011) / Nevada East (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15381',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4742','projected_crs','EPSG','6521','EPSG','2224','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6522','NAD83(2011) / Nevada West',NULL,'EPSG','4499','EPSG','6318','EPSG','12733',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4743','projected_crs','EPSG','6522','EPSG','2225','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6523','NAD83(2011) / Nevada West (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15383',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4744','projected_crs','EPSG','6523','EPSG','2225','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6524','NAD83(2011) / New Hampshire',NULL,'EPSG','4499','EPSG','6318','EPSG','12830',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4745','projected_crs','EPSG','6524','EPSG','1398','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6525','NAD83(2011) / New Hampshire (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15389',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4746','projected_crs','EPSG','6525','EPSG','1398','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6526','NAD83(2011) / New Jersey',NULL,'EPSG','4499','EPSG','6318','EPSG','12930',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4747','projected_crs','EPSG','6526','EPSG','1399','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6527','NAD83(2011) / New Jersey (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15384',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4748','projected_crs','EPSG','6527','EPSG','1399','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6528','NAD83(2011) / New Mexico Central',NULL,'EPSG','4499','EPSG','6318','EPSG','13032',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4749','projected_crs','EPSG','6528','EPSG','2231','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6529','NAD83(2011) / New Mexico Central (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15340',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4750','projected_crs','EPSG','6529','EPSG','2231','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6530','NAD83(2011) / New Mexico East',NULL,'EPSG','4499','EPSG','6318','EPSG','13031',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4751','projected_crs','EPSG','6530','EPSG','2228','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6531','NAD83(2011) / New Mexico East (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15339',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4752','projected_crs','EPSG','6531','EPSG','2228','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6532','NAD83(2011) / New Mexico West',NULL,'EPSG','4499','EPSG','6318','EPSG','13033',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4753','projected_crs','EPSG','6532','EPSG','2232','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6533','NAD83(2011) / New Mexico West (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15341',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4754','projected_crs','EPSG','6533','EPSG','2232','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6534','NAD83(2011) / New York Central',NULL,'EPSG','4499','EPSG','6318','EPSG','13132',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4755','projected_crs','EPSG','6534','EPSG','2233','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6535','NAD83(2011) / New York Central (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15343',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4756','projected_crs','EPSG','6535','EPSG','2233','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6536','NAD83(2011) / New York East',NULL,'EPSG','4499','EPSG','6318','EPSG','13131',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4757','projected_crs','EPSG','6536','EPSG','2234','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6537','NAD83(2011) / New York East (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15342',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4758','projected_crs','EPSG','6537','EPSG','2234','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6538','NAD83(2011) / New York Long Island',NULL,'EPSG','4499','EPSG','6318','EPSG','13134',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4759','projected_crs','EPSG','6538','EPSG','2235','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6539','NAD83(2011) / New York Long Island (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15345',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4760','projected_crs','EPSG','6539','EPSG','2235','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6540','NAD83(2011) / New York West',NULL,'EPSG','4499','EPSG','6318','EPSG','13133',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4761','projected_crs','EPSG','6540','EPSG','2236','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6541','NAD83(2011) / New York West (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15344',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4762','projected_crs','EPSG','6541','EPSG','2236','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6542','NAD83(2011) / North Carolina',NULL,'EPSG','4499','EPSG','6318','EPSG','13230',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4763','projected_crs','EPSG','6542','EPSG','1402','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6543','NAD83(2011) / North Carolina (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15346',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4764','projected_crs','EPSG','6543','EPSG','1402','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6544','NAD83(2011) / North Dakota North',NULL,'EPSG','4499','EPSG','6318','EPSG','13331',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4765','projected_crs','EPSG','6544','EPSG','2237','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6545','NAD83(2011) / North Dakota North (ft)',NULL,'EPSG','4495','EPSG','6318','EPSG','15347',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4766','projected_crs','EPSG','6545','EPSG','2237','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6546','NAD83(2011) / North Dakota South',NULL,'EPSG','4499','EPSG','6318','EPSG','13332',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4767','projected_crs','EPSG','6546','EPSG','2238','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6547','NAD83(2011) / North Dakota South (ft)',NULL,'EPSG','4495','EPSG','6318','EPSG','15348',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4768','projected_crs','EPSG','6547','EPSG','2238','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6548','NAD83(2011) / Ohio North',NULL,'EPSG','4499','EPSG','6318','EPSG','13431',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4769','projected_crs','EPSG','6548','EPSG','2239','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6549','NAD83(2011) / Ohio North (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','13433',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4770','projected_crs','EPSG','6549','EPSG','2239','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6550','NAD83(2011) / Ohio South',NULL,'EPSG','4499','EPSG','6318','EPSG','13432',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4771','projected_crs','EPSG','6550','EPSG','2240','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6551','NAD83(2011) / Ohio South (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','13434',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4772','projected_crs','EPSG','6551','EPSG','2240','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6552','NAD83(2011) / Oklahoma North',NULL,'EPSG','4499','EPSG','6318','EPSG','13531',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4773','projected_crs','EPSG','6552','EPSG','2241','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6553','NAD83(2011) / Oklahoma North (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15349',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4774','projected_crs','EPSG','6553','EPSG','2241','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6554','NAD83(2011) / Oklahoma South',NULL,'EPSG','4499','EPSG','6318','EPSG','13532',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4775','projected_crs','EPSG','6554','EPSG','2242','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6555','NAD83(2011) / Oklahoma South (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15350',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4776','projected_crs','EPSG','6555','EPSG','2242','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6556','NAD83(2011) / Oregon LCC (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','13633',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4777','projected_crs','EPSG','6556','EPSG','1406','EPSG','1212');
INSERT INTO "projected_crs" VALUES('EPSG','6557','NAD83(2011) / Oregon GIC Lambert (ft)',NULL,'EPSG','4495','EPSG','6318','EPSG','15374',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4778','projected_crs','EPSG','6557','EPSG','1406','EPSG','1135');
INSERT INTO "projected_crs" VALUES('EPSG','6558','NAD83(2011) / Oregon North',NULL,'EPSG','4499','EPSG','6318','EPSG','13631',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4779','projected_crs','EPSG','6558','EPSG','2243','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6559','NAD83(2011) / Oregon North (ft)',NULL,'EPSG','4495','EPSG','6318','EPSG','15351',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4780','projected_crs','EPSG','6559','EPSG','2243','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6560','NAD83(2011) / Oregon South',NULL,'EPSG','4499','EPSG','6318','EPSG','13632',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4781','projected_crs','EPSG','6560','EPSG','2244','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6561','NAD83(2011) / Oregon South (ft)',NULL,'EPSG','4495','EPSG','6318','EPSG','15352',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4782','projected_crs','EPSG','6561','EPSG','2244','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6562','NAD83(2011) / Pennsylvania North',NULL,'EPSG','4499','EPSG','6318','EPSG','13731',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4783','projected_crs','EPSG','6562','EPSG','2245','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6563','NAD83(2011) / Pennsylvania North (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15353',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4784','projected_crs','EPSG','6563','EPSG','2245','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6564','NAD83(2011) / Pennsylvania South',NULL,'EPSG','4499','EPSG','6318','EPSG','13732',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4785','projected_crs','EPSG','6564','EPSG','2246','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6565','NAD83(2011) / Pennsylvania South (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15354',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4786','projected_crs','EPSG','6565','EPSG','2246','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6566','NAD83(2011) / Puerto Rico and Virgin Is.',NULL,'EPSG','4499','EPSG','6318','EPSG','15230',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4787','projected_crs','EPSG','6566','EPSG','3634','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6567','NAD83(2011) / Rhode Island',NULL,'EPSG','4499','EPSG','6318','EPSG','13830',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4788','projected_crs','EPSG','6567','EPSG','1408','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6568','NAD83(2011) / Rhode Island (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15390',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4789','projected_crs','EPSG','6568','EPSG','1408','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6569','NAD83(2011) / South Carolina',NULL,'EPSG','4499','EPSG','6318','EPSG','13930',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4790','projected_crs','EPSG','6569','EPSG','1409','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6570','NAD83(2011) / South Carolina (ft)',NULL,'EPSG','4495','EPSG','6318','EPSG','15355',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4791','projected_crs','EPSG','6570','EPSG','1409','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6571','NAD83(2011) / South Dakota North',NULL,'EPSG','4499','EPSG','6318','EPSG','14031',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4792','projected_crs','EPSG','6571','EPSG','2249','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6572','NAD83(2011) / South Dakota North (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15394',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4793','projected_crs','EPSG','6572','EPSG','2249','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6573','NAD83(2011) / South Dakota South',NULL,'EPSG','4499','EPSG','6318','EPSG','14032',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4794','projected_crs','EPSG','6573','EPSG','2250','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6574','NAD83(2011) / South Dakota South (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15395',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4795','projected_crs','EPSG','6574','EPSG','2250','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6575','NAD83(2011) / Tennessee',NULL,'EPSG','4499','EPSG','6318','EPSG','14130',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4796','projected_crs','EPSG','6575','EPSG','1411','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6576','NAD83(2011) / Tennessee (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15356',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4797','projected_crs','EPSG','6576','EPSG','1411','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6577','NAD83(2011) / Texas Central',NULL,'EPSG','4499','EPSG','6318','EPSG','14233',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4798','projected_crs','EPSG','6577','EPSG','2252','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6578','NAD83(2011) / Texas Central (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15359',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4799','projected_crs','EPSG','6578','EPSG','2252','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6579','NAD83(2011) / Texas Centric Albers Equal Area',NULL,'EPSG','4499','EPSG','6318','EPSG','14254',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4800','projected_crs','EPSG','6579','EPSG','1412','EPSG','1222');
INSERT INTO "projected_crs" VALUES('EPSG','6580','NAD83(2011) / Texas Centric Lambert Conformal',NULL,'EPSG','4499','EPSG','6318','EPSG','14253',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4801','projected_crs','EPSG','6580','EPSG','1412','EPSG','1221');
INSERT INTO "projected_crs" VALUES('EPSG','6581','NAD83(2011) / Texas North',NULL,'EPSG','4499','EPSG','6318','EPSG','14231',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4802','projected_crs','EPSG','6581','EPSG','2253','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6582','NAD83(2011) / Texas North (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15357',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4803','projected_crs','EPSG','6582','EPSG','2253','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6583','NAD83(2011) / Texas North Central',NULL,'EPSG','4499','EPSG','6318','EPSG','14232',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4804','projected_crs','EPSG','6583','EPSG','2254','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6584','NAD83(2011) / Texas North Central (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15358',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4805','projected_crs','EPSG','6584','EPSG','2254','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6585','NAD83(2011) / Texas South',NULL,'EPSG','4499','EPSG','6318','EPSG','14235',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4806','projected_crs','EPSG','6585','EPSG','2528','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6586','NAD83(2011) / Texas South (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15361',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4807','projected_crs','EPSG','6586','EPSG','2528','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6587','NAD83(2011) / Texas South Central',NULL,'EPSG','4499','EPSG','6318','EPSG','14234',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4808','projected_crs','EPSG','6587','EPSG','2527','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6588','NAD83(2011) / Texas South Central (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15360',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4809','projected_crs','EPSG','6588','EPSG','2527','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6589','NAD83(2011) / Vermont',NULL,'EPSG','4499','EPSG','6318','EPSG','14430',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4810','projected_crs','EPSG','6589','EPSG','1414','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6590','NAD83(2011) / Vermont (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','5645',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4811','projected_crs','EPSG','6590','EPSG','1414','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6591','NAD83(2011) / Virginia Lambert',NULL,'EPSG','4499','EPSG','6318','EPSG','3967',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4812','projected_crs','EPSG','6591','EPSG','1415','EPSG','1135');
INSERT INTO "projected_crs" VALUES('EPSG','6592','NAD83(2011) / Virginia North',NULL,'EPSG','4499','EPSG','6318','EPSG','14531',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4813','projected_crs','EPSG','6592','EPSG','2260','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6593','NAD83(2011) / Virginia North (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15365',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4814','projected_crs','EPSG','6593','EPSG','2260','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6594','NAD83(2011) / Virginia South',NULL,'EPSG','4499','EPSG','6318','EPSG','14532',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4815','projected_crs','EPSG','6594','EPSG','2261','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6595','NAD83(2011) / Virginia South (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15366',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4816','projected_crs','EPSG','6595','EPSG','2261','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6596','NAD83(2011) / Washington North',NULL,'EPSG','4499','EPSG','6318','EPSG','14631',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4817','projected_crs','EPSG','6596','EPSG','2273','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6597','NAD83(2011) / Washington North (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15367',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4818','projected_crs','EPSG','6597','EPSG','2273','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6598','NAD83(2011) / Washington South',NULL,'EPSG','4499','EPSG','6318','EPSG','14632',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4819','projected_crs','EPSG','6598','EPSG','2274','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6599','NAD83(2011) / Washington South (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15368',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4820','projected_crs','EPSG','6599','EPSG','2274','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6600','NAD83(2011) / West Virginia North',NULL,'EPSG','4499','EPSG','6318','EPSG','14731',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4821','projected_crs','EPSG','6600','EPSG','2264','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6601','NAD83(2011) / West Virginia North (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','14735',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4822','projected_crs','EPSG','6601','EPSG','2264','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6602','NAD83(2011) / West Virginia South',NULL,'EPSG','4499','EPSG','6318','EPSG','14732',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4823','projected_crs','EPSG','6602','EPSG','2265','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6603','NAD83(2011) / West Virginia South (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','14736',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4824','projected_crs','EPSG','6603','EPSG','2265','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6604','NAD83(2011) / Wisconsin Central',NULL,'EPSG','4499','EPSG','4759','EPSG','14832',NULL,1);
INSERT INTO "usage" VALUES('EPSG','4825','projected_crs','EPSG','6604','EPSG','2266','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6605','NAD83(2011) / Wisconsin Central (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15370',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4826','projected_crs','EPSG','6605','EPSG','2266','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6606','NAD83(2011) / Wisconsin North',NULL,'EPSG','4499','EPSG','6318','EPSG','14831',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4827','projected_crs','EPSG','6606','EPSG','2267','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6607','NAD83(2011) / Wisconsin North (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15369',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4828','projected_crs','EPSG','6607','EPSG','2267','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6608','NAD83(2011) / Wisconsin South',NULL,'EPSG','4499','EPSG','6318','EPSG','14833',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4829','projected_crs','EPSG','6608','EPSG','2268','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6609','NAD83(2011) / Wisconsin South (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15371',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4830','projected_crs','EPSG','6609','EPSG','2268','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6610','NAD83(2011) / Wisconsin Transverse Mercator',NULL,'EPSG','4499','EPSG','6318','EPSG','14841',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4831','projected_crs','EPSG','6610','EPSG','1418','EPSG','1135');
INSERT INTO "projected_crs" VALUES('EPSG','6611','NAD83(2011) / Wyoming East',NULL,'EPSG','4499','EPSG','6318','EPSG','14931',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4832','projected_crs','EPSG','6611','EPSG','2269','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6612','NAD83(2011) / Wyoming East (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','14935',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4833','projected_crs','EPSG','6612','EPSG','2269','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6613','NAD83(2011) / Wyoming East Central',NULL,'EPSG','4499','EPSG','6318','EPSG','14932',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4834','projected_crs','EPSG','6613','EPSG','2270','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6614','NAD83(2011) / Wyoming East Central (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','14936',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4835','projected_crs','EPSG','6614','EPSG','2270','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6615','NAD83(2011) / Wyoming West',NULL,'EPSG','4499','EPSG','6318','EPSG','14934',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4836','projected_crs','EPSG','6615','EPSG','2271','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6616','NAD83(2011) / Wyoming West (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','14938',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4837','projected_crs','EPSG','6616','EPSG','2271','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6617','NAD83(2011) / Wyoming West Central',NULL,'EPSG','4499','EPSG','6318','EPSG','14933',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4838','projected_crs','EPSG','6617','EPSG','2272','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6618','NAD83(2011) / Wyoming West Central (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','14937',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4839','projected_crs','EPSG','6618','EPSG','2272','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6619','NAD83(2011) / Utah Central',NULL,'EPSG','4499','EPSG','6318','EPSG','14332',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4840','projected_crs','EPSG','6619','EPSG','2257','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6620','NAD83(2011) / Utah North',NULL,'EPSG','4499','EPSG','6318','EPSG','14331',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4841','projected_crs','EPSG','6620','EPSG','2258','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6621','NAD83(2011) / Utah South',NULL,'EPSG','4499','EPSG','6318','EPSG','14333',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4842','projected_crs','EPSG','6621','EPSG','2259','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6622','NAD83(CSRS)v2 / Quebec Lambert',NULL,'EPSG','4499','EPSG','8237','EPSG','19944',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4843','projected_crs','EPSG','6622','EPSG','1368','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6623','NAD83 / Quebec Albers',NULL,'EPSG','4499','EPSG','4269','EPSG','6645',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4844','projected_crs','EPSG','6623','EPSG','1368','EPSG','1210');
INSERT INTO "projected_crs" VALUES('EPSG','6624','NAD83(CSRS)v2 / Quebec Albers',NULL,'EPSG','4499','EPSG','8237','EPSG','6645',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4845','projected_crs','EPSG','6624','EPSG','1368','EPSG','1220');
INSERT INTO "projected_crs" VALUES('EPSG','6625','NAD83(2011) / Utah Central (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15298',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4846','projected_crs','EPSG','6625','EPSG','2257','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6626','NAD83(2011) / Utah North (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15297',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4847','projected_crs','EPSG','6626','EPSG','2258','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6627','NAD83(2011) / Utah South (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15299',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4848','projected_crs','EPSG','6627','EPSG','2259','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6628','NAD83(PA11) / Hawaii zone 1',NULL,'EPSG','4499','EPSG','6322','EPSG','15131',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4849','projected_crs','EPSG','6628','EPSG','1546','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6629','NAD83(PA11) / Hawaii zone 2',NULL,'EPSG','4499','EPSG','6322','EPSG','15132',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4850','projected_crs','EPSG','6629','EPSG','1547','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6630','NAD83(PA11) / Hawaii zone 3',NULL,'EPSG','4499','EPSG','6322','EPSG','15133',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4851','projected_crs','EPSG','6630','EPSG','1548','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6631','NAD83(PA11) / Hawaii zone 4',NULL,'EPSG','4499','EPSG','6322','EPSG','15134',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4852','projected_crs','EPSG','6631','EPSG','1549','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6632','NAD83(PA11) / Hawaii zone 5',NULL,'EPSG','4499','EPSG','6322','EPSG','15135',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4853','projected_crs','EPSG','6632','EPSG','1550','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6633','NAD83(PA11) / Hawaii zone 3 (ftUS)',NULL,'EPSG','4497','EPSG','6322','EPSG','15138',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4854','projected_crs','EPSG','6633','EPSG','1548','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6634','NAD83(PA11) / UTM zone 4N',NULL,'EPSG','4400','EPSG','6322','EPSG','16004',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4855','projected_crs','EPSG','6634','EPSG','3488','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6635','NAD83(PA11) / UTM zone 5N',NULL,'EPSG','4400','EPSG','6322','EPSG','16005',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4856','projected_crs','EPSG','6635','EPSG','3491','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6636','NAD83(PA11) / UTM zone 2S',NULL,'EPSG','4400','EPSG','6322','EPSG','16102',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4857','projected_crs','EPSG','6636','EPSG','3110','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6637','NAD83(MA11) / Guam Map Grid',NULL,'EPSG','4499','EPSG','6325','EPSG','4325',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4858','projected_crs','EPSG','6637','EPSG','3255','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','6646','Karbala 1979 / Iraq National Grid',NULL,'EPSG','4400','EPSG','4743','EPSG','19907',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4866','projected_crs','EPSG','6646','EPSG','3625','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','6669','JGD2011 / Japan Plane Rectangular CS I',NULL,'EPSG','4530','EPSG','6668','EPSG','17801',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4888','projected_crs','EPSG','6669','EPSG','1854','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','6670','JGD2011 / Japan Plane Rectangular CS II',NULL,'EPSG','4530','EPSG','6668','EPSG','17802',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4889','projected_crs','EPSG','6670','EPSG','1855','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','6671','JGD2011 / Japan Plane Rectangular CS III',NULL,'EPSG','4530','EPSG','6668','EPSG','17803',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4890','projected_crs','EPSG','6671','EPSG','1856','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','6672','JGD2011 / Japan Plane Rectangular CS IV',NULL,'EPSG','4530','EPSG','6668','EPSG','17804',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4891','projected_crs','EPSG','6672','EPSG','1857','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','6673','JGD2011 / Japan Plane Rectangular CS V',NULL,'EPSG','4530','EPSG','6668','EPSG','17805',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4892','projected_crs','EPSG','6673','EPSG','1858','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','6674','JGD2011 / Japan Plane Rectangular CS VI',NULL,'EPSG','4530','EPSG','6668','EPSG','17806',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4893','projected_crs','EPSG','6674','EPSG','1859','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','6675','JGD2011 / Japan Plane Rectangular CS VII',NULL,'EPSG','4530','EPSG','6668','EPSG','17807',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4894','projected_crs','EPSG','6675','EPSG','1860','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','6676','JGD2011 / Japan Plane Rectangular CS VIII',NULL,'EPSG','4530','EPSG','6668','EPSG','17808',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4895','projected_crs','EPSG','6676','EPSG','1861','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','6677','JGD2011 / Japan Plane Rectangular CS IX',NULL,'EPSG','4530','EPSG','6668','EPSG','17809',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4896','projected_crs','EPSG','6677','EPSG','1862','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','6678','JGD2011 / Japan Plane Rectangular CS X',NULL,'EPSG','4530','EPSG','6668','EPSG','17810',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4897','projected_crs','EPSG','6678','EPSG','1863','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','6679','JGD2011 / Japan Plane Rectangular CS XI',NULL,'EPSG','4530','EPSG','6668','EPSG','17811',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4898','projected_crs','EPSG','6679','EPSG','1864','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','6680','JGD2011 / Japan Plane Rectangular CS XII',NULL,'EPSG','4530','EPSG','6668','EPSG','17812',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4899','projected_crs','EPSG','6680','EPSG','1865','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','6681','JGD2011 / Japan Plane Rectangular CS XIII',NULL,'EPSG','4530','EPSG','6668','EPSG','17813',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4900','projected_crs','EPSG','6681','EPSG','1866','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','6682','JGD2011 / Japan Plane Rectangular CS XIV',NULL,'EPSG','4530','EPSG','6668','EPSG','17814',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4901','projected_crs','EPSG','6682','EPSG','1867','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','6683','JGD2011 / Japan Plane Rectangular CS XV',NULL,'EPSG','4530','EPSG','6668','EPSG','17815',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4902','projected_crs','EPSG','6683','EPSG','1868','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','6684','JGD2011 / Japan Plane Rectangular CS XVI',NULL,'EPSG','4530','EPSG','6668','EPSG','17816',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4903','projected_crs','EPSG','6684','EPSG','1869','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','6685','JGD2011 / Japan Plane Rectangular CS XVII',NULL,'EPSG','4530','EPSG','6668','EPSG','17817',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4904','projected_crs','EPSG','6685','EPSG','1870','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','6686','JGD2011 / Japan Plane Rectangular CS XVIII',NULL,'EPSG','4530','EPSG','6668','EPSG','17818',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4905','projected_crs','EPSG','6686','EPSG','1871','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','6687','JGD2011 / Japan Plane Rectangular CS XIX',NULL,'EPSG','4530','EPSG','6668','EPSG','17819',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4906','projected_crs','EPSG','6687','EPSG','1872','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','6688','JGD2011 / UTM zone 51N',NULL,'EPSG','4400','EPSG','6668','EPSG','16051',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4907','projected_crs','EPSG','6688','EPSG','3959','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','6689','JGD2011 / UTM zone 52N',NULL,'EPSG','4400','EPSG','6668','EPSG','16052',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4908','projected_crs','EPSG','6689','EPSG','3960','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','6690','JGD2011 / UTM zone 53N',NULL,'EPSG','4400','EPSG','6668','EPSG','16053',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4909','projected_crs','EPSG','6690','EPSG','3961','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','6691','JGD2011 / UTM zone 54N',NULL,'EPSG','4400','EPSG','6668','EPSG','16054',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4910','projected_crs','EPSG','6691','EPSG','3962','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','6692','JGD2011 / UTM zone 55N',NULL,'EPSG','4400','EPSG','6668','EPSG','16055',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4911','projected_crs','EPSG','6692','EPSG','3963','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','6703','WGS 84 / TM 60 SW',NULL,'EPSG','4400','EPSG','4326','EPSG','6702',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4918','projected_crs','EPSG','6703','EPSG','4172','EPSG','1136');
INSERT INTO "projected_crs" VALUES('EPSG','6707','RDN2008 / UTM zone 32N (N-E)',NULL,'EPSG','4500','EPSG','6706','EPSG','16032',NULL,0);
INSERT INTO "usage" VALUES('EPSG','14415','projected_crs','EPSG','6707','EPSG','1718','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6708','RDN2008 / UTM zone 33N (N-E)',NULL,'EPSG','4500','EPSG','6706','EPSG','16033',NULL,0);
INSERT INTO "usage" VALUES('EPSG','14414','projected_crs','EPSG','6708','EPSG','4186','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6709','RDN2008 / UTM zone 34N (N-E)',NULL,'EPSG','4500','EPSG','6706','EPSG','16034',NULL,0);
INSERT INTO "usage" VALUES('EPSG','14416','projected_crs','EPSG','6709','EPSG','4187','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6720','WGS 84 / CIG92',NULL,'EPSG','4400','EPSG','4326','EPSG','6716',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4926','projected_crs','EPSG','6720','EPSG','4169','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6721','GDA94 / CIG94',NULL,'EPSG','4400','EPSG','4283','EPSG','6717',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4927','projected_crs','EPSG','6721','EPSG','4169','EPSG','1093');
INSERT INTO "projected_crs" VALUES('EPSG','6722','WGS 84 / CKIG92',NULL,'EPSG','4400','EPSG','4326','EPSG','6718',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4928','projected_crs','EPSG','6722','EPSG','1069','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6723','GDA94 / CKIG94',NULL,'EPSG','4400','EPSG','4283','EPSG','6719',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4929','projected_crs','EPSG','6723','EPSG','1069','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','6732','GDA94 / MGA zone 41',NULL,'EPSG','4400','EPSG','4283','EPSG','6725',NULL,1);
INSERT INTO "usage" VALUES('EPSG','4930','projected_crs','EPSG','6732','EPSG','4173','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6733','GDA94 / MGA zone 42',NULL,'EPSG','4400','EPSG','4283','EPSG','6726',NULL,1);
INSERT INTO "usage" VALUES('EPSG','4931','projected_crs','EPSG','6733','EPSG','4181','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6734','GDA94 / MGA zone 43',NULL,'EPSG','4400','EPSG','4283','EPSG','6727',NULL,1);
INSERT INTO "usage" VALUES('EPSG','4932','projected_crs','EPSG','6734','EPSG','4184','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6735','GDA94 / MGA zone 44',NULL,'EPSG','4400','EPSG','4283','EPSG','6728',NULL,1);
INSERT INTO "usage" VALUES('EPSG','4933','projected_crs','EPSG','6735','EPSG','4185','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6736','GDA94 / MGA zone 46',NULL,'EPSG','4400','EPSG','4283','EPSG','6729',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4934','projected_crs','EPSG','6736','EPSG','4189','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6737','GDA94 / MGA zone 47',NULL,'EPSG','4400','EPSG','4283','EPSG','6730',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4935','projected_crs','EPSG','6737','EPSG','4190','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6738','GDA94 / MGA zone 59',NULL,'EPSG','4400','EPSG','4283','EPSG','6731',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4936','projected_crs','EPSG','6738','EPSG','4179','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6784','NAD83(CORS96) / Oregon Baker zone (m)',NULL,'EPSG','4499','EPSG','6783','EPSG','6741',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4940','projected_crs','EPSG','6784','EPSG','4180','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6785','NAD83(CORS96) / Oregon Baker zone (ft)',NULL,'EPSG','4495','EPSG','6783','EPSG','6742',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4941','projected_crs','EPSG','6785','EPSG','4180','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6786','NAD83(2011) / Oregon Baker zone (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','6741',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4942','projected_crs','EPSG','6786','EPSG','4180','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6787','NAD83(2011) / Oregon Baker zone (ft)',NULL,'EPSG','4495','EPSG','6318','EPSG','6742',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4943','projected_crs','EPSG','6787','EPSG','4180','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6788','NAD83(CORS96) / Oregon Bend-Klamath Falls zone (m)',NULL,'EPSG','4499','EPSG','6783','EPSG','6743',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4944','projected_crs','EPSG','6788','EPSG','4192','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6789','NAD83(CORS96) / Oregon Bend-Klamath Falls zone (ft)',NULL,'EPSG','4495','EPSG','6783','EPSG','6744',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4945','projected_crs','EPSG','6789','EPSG','4192','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6790','NAD83(2011) / Oregon Bend-Klamath Falls zone (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','6743',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4946','projected_crs','EPSG','6790','EPSG','4192','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6791','NAD83(2011) / Oregon Bend-Klamath Falls zone (ft)',NULL,'EPSG','4495','EPSG','6318','EPSG','6744',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4947','projected_crs','EPSG','6791','EPSG','4192','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6792','NAD83(CORS96) / Oregon Bend-Redmond-Prineville zone (m)',NULL,'EPSG','4499','EPSG','6783','EPSG','6745',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4948','projected_crs','EPSG','6792','EPSG','4195','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6793','NAD83(CORS96) / Oregon Bend-Redmond-Prineville zone (ft)',NULL,'EPSG','4495','EPSG','6783','EPSG','6746',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4949','projected_crs','EPSG','6793','EPSG','4195','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6794','NAD83(2011) / Oregon Bend-Redmond-Prineville zone (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','6745',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4950','projected_crs','EPSG','6794','EPSG','4195','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6795','NAD83(2011) / Oregon Bend-Redmond-Prineville zone (ft)',NULL,'EPSG','4495','EPSG','6318','EPSG','6746',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4951','projected_crs','EPSG','6795','EPSG','4195','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6796','NAD83(CORS96) / Oregon Bend-Burns zone (m)',NULL,'EPSG','4499','EPSG','6783','EPSG','6747',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4952','projected_crs','EPSG','6796','EPSG','4182','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6797','NAD83(CORS96) / Oregon Bend-Burns zone (ft)',NULL,'EPSG','4495','EPSG','6783','EPSG','6748',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4953','projected_crs','EPSG','6797','EPSG','4182','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6798','NAD83(2011) / Oregon Bend-Burns zone (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','6747',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4954','projected_crs','EPSG','6798','EPSG','4182','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6799','NAD83(2011) / Oregon Bend-Burns zone (ft)',NULL,'EPSG','4495','EPSG','6318','EPSG','6748',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4955','projected_crs','EPSG','6799','EPSG','4182','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6800','NAD83(CORS96) / Oregon Canyonville-Grants Pass zone (m)',NULL,'EPSG','4499','EPSG','6783','EPSG','6749',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4956','projected_crs','EPSG','6800','EPSG','4199','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6801','NAD83(CORS96) / Oregon Canyonville-Grants Pass zone (ft)',NULL,'EPSG','4495','EPSG','6783','EPSG','6750',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4957','projected_crs','EPSG','6801','EPSG','4199','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6802','NAD83(2011) / Oregon Canyonville-Grants Pass zone (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','6749',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4958','projected_crs','EPSG','6802','EPSG','4199','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6803','NAD83(2011) / Oregon Canyonville-Grants Pass zone (ft)',NULL,'EPSG','4495','EPSG','6318','EPSG','6750',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4959','projected_crs','EPSG','6803','EPSG','4199','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6804','NAD83(CORS96) / Oregon Columbia River East zone (m)',NULL,'EPSG','4499','EPSG','6783','EPSG','6751',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4960','projected_crs','EPSG','6804','EPSG','4200','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6805','NAD83(CORS96) / Oregon Columbia River East zone (ft)',NULL,'EPSG','4495','EPSG','6783','EPSG','6752',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4961','projected_crs','EPSG','6805','EPSG','4200','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6806','NAD83(2011) / Oregon Columbia River East zone (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','6751',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4962','projected_crs','EPSG','6806','EPSG','4200','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6807','NAD83(2011) / Oregon Columbia River East zone (ft)',NULL,'EPSG','4495','EPSG','6318','EPSG','6752',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4963','projected_crs','EPSG','6807','EPSG','4200','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6808','NAD83(CORS96) / Oregon Columbia River West zone (m)',NULL,'EPSG','4499','EPSG','6783','EPSG','6753',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4964','projected_crs','EPSG','6808','EPSG','4202','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6809','NAD83(CORS96) / Oregon Columbia River West zone (ft)',NULL,'EPSG','4495','EPSG','6783','EPSG','6754',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4965','projected_crs','EPSG','6809','EPSG','4202','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6810','NAD83(2011) / Oregon Columbia River West zone (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','6753',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4966','projected_crs','EPSG','6810','EPSG','4202','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6811','NAD83(2011) / Oregon Columbia River West zone (ft)',NULL,'EPSG','4495','EPSG','6318','EPSG','6754',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4967','projected_crs','EPSG','6811','EPSG','4202','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6812','NAD83(CORS96) / Oregon Cottage Grove-Canyonville zone (m)',NULL,'EPSG','4499','EPSG','6783','EPSG','6755',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4968','projected_crs','EPSG','6812','EPSG','4203','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6813','NAD83(CORS96) / Oregon Cottage Grove-Canyonville zone (ft)',NULL,'EPSG','4495','EPSG','6783','EPSG','6756',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4969','projected_crs','EPSG','6813','EPSG','4203','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6814','NAD83(2011) / Oregon Cottage Grove-Canyonville zone (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','6755',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4970','projected_crs','EPSG','6814','EPSG','4203','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6815','NAD83(2011) / Oregon Cottage Grove-Canyonville zone (ft)',NULL,'EPSG','4495','EPSG','6318','EPSG','6756',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4971','projected_crs','EPSG','6815','EPSG','4203','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6816','NAD83(CORS96) / Oregon Dufur-Madras zone (m)',NULL,'EPSG','4499','EPSG','6783','EPSG','6757',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4972','projected_crs','EPSG','6816','EPSG','4204','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6817','NAD83(CORS96) / Oregon Dufur-Madras zone (ft)',NULL,'EPSG','4495','EPSG','6783','EPSG','6758',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4973','projected_crs','EPSG','6817','EPSG','4204','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6818','NAD83(2011) / Oregon Dufur-Madras zone (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','6757',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4974','projected_crs','EPSG','6818','EPSG','4204','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6819','NAD83(2011) / Oregon Dufur-Madras zone (ft)',NULL,'EPSG','4495','EPSG','6318','EPSG','6758',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4975','projected_crs','EPSG','6819','EPSG','4204','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6820','NAD83(CORS96) / Oregon Eugene zone (m)',NULL,'EPSG','4499','EPSG','6783','EPSG','6759',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4976','projected_crs','EPSG','6820','EPSG','4197','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6821','NAD83(CORS96) / Oregon Eugene zone (ft)',NULL,'EPSG','4495','EPSG','6783','EPSG','6760',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4977','projected_crs','EPSG','6821','EPSG','4197','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6822','NAD83(2011) / Oregon Eugene zone (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','6759',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4978','projected_crs','EPSG','6822','EPSG','4197','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6823','NAD83(2011) / Oregon Eugene zone (ft)',NULL,'EPSG','4495','EPSG','6318','EPSG','6760',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4979','projected_crs','EPSG','6823','EPSG','4197','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6824','NAD83(CORS96) / Oregon Grants Pass-Ashland zone (m)',NULL,'EPSG','4499','EPSG','6783','EPSG','6761',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4980','projected_crs','EPSG','6824','EPSG','4198','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6825','NAD83(CORS96) / Oregon Grants Pass-Ashland zone (ft)',NULL,'EPSG','4495','EPSG','6783','EPSG','6762',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4981','projected_crs','EPSG','6825','EPSG','4198','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6826','NAD83(2011) / Oregon Grants Pass-Ashland zone (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','6761',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4982','projected_crs','EPSG','6826','EPSG','4198','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6827','NAD83(2011) / Oregon Grants Pass-Ashland zone (ft)',NULL,'EPSG','4495','EPSG','6318','EPSG','6762',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4983','projected_crs','EPSG','6827','EPSG','4198','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6828','NAD83(CORS96) / Oregon Gresham-Warm Springs zone (m)',NULL,'EPSG','4499','EPSG','6783','EPSG','6763',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4984','projected_crs','EPSG','6828','EPSG','4201','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6829','NAD83(CORS96) / Oregon Gresham-Warm Springs zone (ft)',NULL,'EPSG','4495','EPSG','6783','EPSG','6764',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4985','projected_crs','EPSG','6829','EPSG','4201','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6830','NAD83(2011) / Oregon Gresham-Warm Springs zone (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','6763',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4986','projected_crs','EPSG','6830','EPSG','4201','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6831','NAD83(2011) / Oregon Gresham-Warm Springs zone (ft)',NULL,'EPSG','4495','EPSG','6318','EPSG','6764',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4987','projected_crs','EPSG','6831','EPSG','4201','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6832','NAD83(CORS96) / Oregon La Grande zone (m)',NULL,'EPSG','4499','EPSG','6783','EPSG','6765',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4988','projected_crs','EPSG','6832','EPSG','4206','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6833','NAD83(CORS96) / Oregon La Grande zone (ft)',NULL,'EPSG','4495','EPSG','6783','EPSG','6766',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4989','projected_crs','EPSG','6833','EPSG','4206','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6834','NAD83(2011) / Oregon La Grande zone (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','6765',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4990','projected_crs','EPSG','6834','EPSG','4206','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6835','NAD83(2011) / Oregon La Grande zone (ft)',NULL,'EPSG','4495','EPSG','6318','EPSG','6766',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4991','projected_crs','EPSG','6835','EPSG','4206','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6836','NAD83(CORS96) / Oregon Ontario zone (m)',NULL,'EPSG','4499','EPSG','6783','EPSG','6767',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4992','projected_crs','EPSG','6836','EPSG','4207','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6837','NAD83(CORS96) / Oregon Ontario zone (ft)',NULL,'EPSG','4495','EPSG','6783','EPSG','6768',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4993','projected_crs','EPSG','6837','EPSG','4207','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6838','NAD83(2011) / Oregon Ontario zone (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','6767',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4994','projected_crs','EPSG','6838','EPSG','4207','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6839','NAD83(2011) / Oregon Ontario zone (ft)',NULL,'EPSG','4495','EPSG','6318','EPSG','6768',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4995','projected_crs','EPSG','6839','EPSG','4207','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6840','NAD83(CORS96) / Oregon Coast zone (m)',NULL,'EPSG','4499','EPSG','6783','EPSG','6769',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4996','projected_crs','EPSG','6840','EPSG','4208','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6841','NAD83(CORS96) / Oregon Coast zone (ft)',NULL,'EPSG','4495','EPSG','6783','EPSG','6770',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4997','projected_crs','EPSG','6841','EPSG','4208','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6842','NAD83(2011) / Oregon Coast zone (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','6769',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4998','projected_crs','EPSG','6842','EPSG','4208','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6843','NAD83(2011) / Oregon Coast zone (ft)',NULL,'EPSG','4495','EPSG','6318','EPSG','6770',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4999','projected_crs','EPSG','6843','EPSG','4208','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6844','NAD83(CORS96) / Oregon Pendleton zone (m)',NULL,'EPSG','4499','EPSG','6783','EPSG','6771',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5000','projected_crs','EPSG','6844','EPSG','4209','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6845','NAD83(CORS96) / Oregon Pendleton zone (ft)',NULL,'EPSG','4495','EPSG','6783','EPSG','6772',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5001','projected_crs','EPSG','6845','EPSG','4209','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6846','NAD83(2011) / Oregon Pendleton zone (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','6771',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5002','projected_crs','EPSG','6846','EPSG','4209','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6847','NAD83(2011) / Oregon Pendleton zone (ft)',NULL,'EPSG','4495','EPSG','6318','EPSG','6772',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5003','projected_crs','EPSG','6847','EPSG','4209','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6848','NAD83(CORS96) / Oregon Pendleton-La Grande zone (m)',NULL,'EPSG','4499','EPSG','6783','EPSG','6773',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5004','projected_crs','EPSG','6848','EPSG','4210','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6849','NAD83(CORS96) / Oregon Pendleton-La Grande zone (ft)',NULL,'EPSG','4495','EPSG','6783','EPSG','6774',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5005','projected_crs','EPSG','6849','EPSG','4210','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6850','NAD83(2011) / Oregon Pendleton-La Grande zone (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','6773',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5006','projected_crs','EPSG','6850','EPSG','4210','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6851','NAD83(2011) / Oregon Pendleton-La Grande zone (ft)',NULL,'EPSG','4495','EPSG','6318','EPSG','6774',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5007','projected_crs','EPSG','6851','EPSG','4210','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6852','NAD83(CORS96) / Oregon Portland zone (m)',NULL,'EPSG','4499','EPSG','6783','EPSG','6775',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5008','projected_crs','EPSG','6852','EPSG','4211','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6853','NAD83(CORS96) / Oregon Portland zone (ft)',NULL,'EPSG','4495','EPSG','6783','EPSG','6776',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5009','projected_crs','EPSG','6853','EPSG','4211','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6854','NAD83(2011) / Oregon Portland zone (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','6775',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5010','projected_crs','EPSG','6854','EPSG','4211','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6855','NAD83(2011) / Oregon Portland zone (ft)',NULL,'EPSG','4495','EPSG','6318','EPSG','6776',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5011','projected_crs','EPSG','6855','EPSG','4211','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6856','NAD83(CORS96) / Oregon Salem zone (m)',NULL,'EPSG','4499','EPSG','6783','EPSG','6777',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5012','projected_crs','EPSG','6856','EPSG','4212','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6857','NAD83(CORS96) / Oregon Salem zone (ft)',NULL,'EPSG','4495','EPSG','6783','EPSG','6778',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5013','projected_crs','EPSG','6857','EPSG','4212','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6858','NAD83(2011) / Oregon Salem zone (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','6777',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5014','projected_crs','EPSG','6858','EPSG','4212','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6859','NAD83(2011) / Oregon Salem zone (ft)',NULL,'EPSG','4495','EPSG','6318','EPSG','6778',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5015','projected_crs','EPSG','6859','EPSG','4212','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6860','NAD83(CORS96) / Oregon Santiam Pass zone (m)',NULL,'EPSG','4499','EPSG','6783','EPSG','6779',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5016','projected_crs','EPSG','6860','EPSG','4213','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6861','NAD83(CORS96) / Oregon Santiam Pass zone (ft)',NULL,'EPSG','4495','EPSG','6783','EPSG','6780',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5017','projected_crs','EPSG','6861','EPSG','4213','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6862','NAD83(2011) / Oregon Santiam Pass zone (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','6779',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5018','projected_crs','EPSG','6862','EPSG','4213','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6863','NAD83(2011) / Oregon Santiam Pass zone (ft)',NULL,'EPSG','4495','EPSG','6318','EPSG','6780',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5019','projected_crs','EPSG','6863','EPSG','4213','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','6867','NAD83(CORS96) / Oregon LCC (m)',NULL,'EPSG','4499','EPSG','6783','EPSG','13633',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5020','projected_crs','EPSG','6867','EPSG','1406','EPSG','1212');
INSERT INTO "projected_crs" VALUES('EPSG','6868','NAD83(CORS96) / Oregon GIC Lambert (ft)',NULL,'EPSG','4495','EPSG','6783','EPSG','15374',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5021','projected_crs','EPSG','6868','EPSG','1406','EPSG','1135');
INSERT INTO "projected_crs" VALUES('EPSG','6870','ETRS89 / Albania TM 2010',NULL,'EPSG','4530','EPSG','4258','EPSG','6869',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5022','projected_crs','EPSG','6870','EPSG','3212','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','6875','RDN2008 / Italy zone (N-E)',NULL,'EPSG','4500','EPSG','6706','EPSG','6877',NULL,0);
INSERT INTO "usage" VALUES('EPSG','14417','projected_crs','EPSG','6875','EPSG','1127','EPSG','1241');
INSERT INTO "projected_crs" VALUES('EPSG','6876','RDN2008 / Zone 12 (N-E)',NULL,'EPSG','4500','EPSG','6706','EPSG','6878',NULL,0);
INSERT INTO "usage" VALUES('EPSG','14418','projected_crs','EPSG','6876','EPSG','1127','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6879','NAD83(2011) / Wisconsin Central',NULL,'EPSG','4499','EPSG','6318','EPSG','14832',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5026','projected_crs','EPSG','6879','EPSG','2266','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6880','NAD83(2011) / Nebraska (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','15396',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5027','projected_crs','EPSG','6880','EPSG','1396','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6884','NAD83(CORS96) / Oregon North',NULL,'EPSG','4499','EPSG','6783','EPSG','13631',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5031','projected_crs','EPSG','6884','EPSG','2243','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6885','NAD83(CORS96) / Oregon North (ft)',NULL,'EPSG','4495','EPSG','6783','EPSG','15351',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5032','projected_crs','EPSG','6885','EPSG','2243','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6886','NAD83(CORS96) / Oregon South',NULL,'EPSG','4499','EPSG','6783','EPSG','13632',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5033','projected_crs','EPSG','6886','EPSG','2244','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6887','NAD83(CORS96) / Oregon South (ft)',NULL,'EPSG','4495','EPSG','6783','EPSG','15352',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5034','projected_crs','EPSG','6887','EPSG','2244','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6915','South East Island 1943 / UTM zone 40N',NULL,'EPSG','4400','EPSG','6892','EPSG','16040',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5038','projected_crs','EPSG','6915','EPSG','4183','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','6922','NAD83 / Kansas LCC',NULL,'EPSG','4499','EPSG','4269','EPSG','6920',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5041','projected_crs','EPSG','6922','EPSG','1385','EPSG','1241');
INSERT INTO "projected_crs" VALUES('EPSG','6923','NAD83 / Kansas LCC (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','6921',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5042','projected_crs','EPSG','6923','EPSG','1385','EPSG','1241');
INSERT INTO "projected_crs" VALUES('EPSG','6924','NAD83(2011) / Kansas LCC',NULL,'EPSG','4499','EPSG','6318','EPSG','6920',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5043','projected_crs','EPSG','6924','EPSG','1385','EPSG','1241');
INSERT INTO "projected_crs" VALUES('EPSG','6925','NAD83(2011) / Kansas LCC (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','6921',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5044','projected_crs','EPSG','6925','EPSG','1385','EPSG','1241');
INSERT INTO "projected_crs" VALUES('EPSG','6931','WGS 84 / NSIDC EASE-Grid 2.0 North',NULL,'EPSG','4469','EPSG','4326','EPSG','6929',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5046','projected_crs','EPSG','6931','EPSG','3475','EPSG','1195');
INSERT INTO "projected_crs" VALUES('EPSG','6932','WGS 84 / NSIDC EASE-Grid 2.0 South',NULL,'EPSG','4470','EPSG','4326','EPSG','6930',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5047','projected_crs','EPSG','6932','EPSG','3474','EPSG','1195');
INSERT INTO "projected_crs" VALUES('EPSG','6933','WGS 84 / NSIDC EASE-Grid 2.0 Global',NULL,'EPSG','4499','EPSG','4326','EPSG','6928',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5048','projected_crs','EPSG','6933','EPSG','3463','EPSG','1195');
INSERT INTO "projected_crs" VALUES('EPSG','6956','VN-2000 / TM-3 zone 481',NULL,'EPSG','4400','EPSG','4756','EPSG','6952',NULL,1);
INSERT INTO "usage" VALUES('EPSG','5050','projected_crs','EPSG','6956','EPSG','4193','EPSG','1208');
INSERT INTO "projected_crs" VALUES('EPSG','6957','VN-2000 / TM-3 zone 482',NULL,'EPSG','4400','EPSG','4756','EPSG','6953',NULL,1);
INSERT INTO "usage" VALUES('EPSG','5051','projected_crs','EPSG','6957','EPSG','4215','EPSG','1208');
INSERT INTO "projected_crs" VALUES('EPSG','6958','VN-2000 / TM-3 zone 491',NULL,'EPSG','4400','EPSG','4756','EPSG','6954',NULL,1);
INSERT INTO "usage" VALUES('EPSG','5052','projected_crs','EPSG','6958','EPSG','4217','EPSG','1208');
INSERT INTO "projected_crs" VALUES('EPSG','6959','VN-2000 / TM-3 Da Nang zone',NULL,'EPSG','4400','EPSG','4756','EPSG','6955',NULL,1);
INSERT INTO "usage" VALUES('EPSG','5053','projected_crs','EPSG','6959','EPSG','4218','EPSG','1208');
INSERT INTO "projected_crs" VALUES('EPSG','6962','ETRS89 / Albania LCC 2010',NULL,'EPSG','4530','EPSG','4258','EPSG','6961',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5054','projected_crs','EPSG','6962','EPSG','3212','EPSG','1241');
INSERT INTO "projected_crs" VALUES('EPSG','6966','NAD27 / Michigan North',NULL,'EPSG','4497','EPSG','4267','EPSG','6965',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5055','projected_crs','EPSG','6966','EPSG','1723','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','6984','Israeli Grid 05',NULL,'EPSG','4400','EPSG','6983','EPSG','18204',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5062','projected_crs','EPSG','6984','EPSG','2603','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','6991','Israeli Grid 05/12',NULL,'EPSG','4400','EPSG','6990','EPSG','18204',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5069','projected_crs','EPSG','6991','EPSG','2603','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','6996','NAD83(2011) / San Francisco CS13',NULL,'EPSG','4499','EPSG','6318','EPSG','6994',NULL,1);
INSERT INTO "usage" VALUES('EPSG','5070','projected_crs','EPSG','6996','EPSG','4228','EPSG','1194');
INSERT INTO "projected_crs" VALUES('EPSG','6997','NAD83(2011) / San Francisco CS13 (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','6995',NULL,1);
INSERT INTO "usage" VALUES('EPSG','5071','projected_crs','EPSG','6997','EPSG','4228','EPSG','1194');
INSERT INTO "projected_crs" VALUES('EPSG','7005','Nahrwan 1934 / UTM zone 37N',NULL,'EPSG','4400','EPSG','4744','EPSG','16037',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5072','projected_crs','EPSG','7005','EPSG','3387','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','7006','Nahrwan 1934 / UTM zone 38N',NULL,'EPSG','4400','EPSG','4744','EPSG','16038',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5073','projected_crs','EPSG','7006','EPSG','3388','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','7007','Nahrwan 1934 / UTM zone 39N',NULL,'EPSG','4400','EPSG','4744','EPSG','16039',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5074','projected_crs','EPSG','7007','EPSG','3956','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','7057','NAD83(2011) / IaRCS zone 1',NULL,'EPSG','4497','EPSG','6318','EPSG','7043',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5084','projected_crs','EPSG','7057','EPSG','4164','EPSG','1196');
INSERT INTO "projected_crs" VALUES('EPSG','7058','NAD83(2011) / IaRCS zone 2',NULL,'EPSG','4497','EPSG','6318','EPSG','7044',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5085','projected_crs','EPSG','7058','EPSG','4219','EPSG','1196');
INSERT INTO "projected_crs" VALUES('EPSG','7059','NAD83(2011) / IaRCS zone 3',NULL,'EPSG','4497','EPSG','6318','EPSG','7045',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5086','projected_crs','EPSG','7059','EPSG','4230','EPSG','1196');
INSERT INTO "projected_crs" VALUES('EPSG','7060','NAD83(2011) / IaRCS zone 4',NULL,'EPSG','4497','EPSG','6318','EPSG','7046',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5087','projected_crs','EPSG','7060','EPSG','4233','EPSG','1196');
INSERT INTO "projected_crs" VALUES('EPSG','7061','NAD83(2011) / IaRCS zone 5',NULL,'EPSG','4497','EPSG','6318','EPSG','7047',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5088','projected_crs','EPSG','7061','EPSG','4234','EPSG','1196');
INSERT INTO "projected_crs" VALUES('EPSG','7062','NAD83(2011) / IaRCS zone 6',NULL,'EPSG','4497','EPSG','6318','EPSG','7048',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5089','projected_crs','EPSG','7062','EPSG','4235','EPSG','1196');
INSERT INTO "projected_crs" VALUES('EPSG','7063','NAD83(2011) / IaRCS zone 7',NULL,'EPSG','4497','EPSG','6318','EPSG','7049',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5090','projected_crs','EPSG','7063','EPSG','4236','EPSG','1196');
INSERT INTO "projected_crs" VALUES('EPSG','7064','NAD83(2011) / IaRCS zone 8',NULL,'EPSG','4497','EPSG','6318','EPSG','7050',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5091','projected_crs','EPSG','7064','EPSG','4237','EPSG','1196');
INSERT INTO "projected_crs" VALUES('EPSG','7065','NAD83(2011) / IaRCS zone 9',NULL,'EPSG','4497','EPSG','6318','EPSG','7051',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5092','projected_crs','EPSG','7065','EPSG','4239','EPSG','1196');
INSERT INTO "projected_crs" VALUES('EPSG','7066','NAD83(2011) / IaRCS zone 10',NULL,'EPSG','4497','EPSG','6318','EPSG','7052',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5093','projected_crs','EPSG','7066','EPSG','4240','EPSG','1196');
INSERT INTO "projected_crs" VALUES('EPSG','7067','NAD83(2011) / IaRCS zone 11',NULL,'EPSG','4497','EPSG','6318','EPSG','7053',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5094','projected_crs','EPSG','7067','EPSG','4241','EPSG','1196');
INSERT INTO "projected_crs" VALUES('EPSG','7068','NAD83(2011) / IaRCS zone 12',NULL,'EPSG','4497','EPSG','6318','EPSG','7054',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5095','projected_crs','EPSG','7068','EPSG','4242','EPSG','1196');
INSERT INTO "projected_crs" VALUES('EPSG','7069','NAD83(2011) / IaRCS zone 13',NULL,'EPSG','4497','EPSG','6318','EPSG','7055',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5096','projected_crs','EPSG','7069','EPSG','4243','EPSG','1196');
INSERT INTO "projected_crs" VALUES('EPSG','7070','NAD83(2011) / IaRCS zone 14',NULL,'EPSG','4497','EPSG','6318','EPSG','7056',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5097','projected_crs','EPSG','7070','EPSG','4244','EPSG','1196');
INSERT INTO "projected_crs" VALUES('EPSG','7074','RGTAAF07 / UTM zone 37S',NULL,'EPSG','4400','EPSG','7073','EPSG','16137',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5101','projected_crs','EPSG','7074','EPSG','3934','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','7075','RGTAAF07 / UTM zone 38S',NULL,'EPSG','4400','EPSG','7073','EPSG','16138',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5102','projected_crs','EPSG','7075','EPSG','4245','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','7076','RGTAAF07 / UTM zone 39S',NULL,'EPSG','4400','EPSG','7073','EPSG','16139',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5103','projected_crs','EPSG','7076','EPSG','4247','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','7077','RGTAAF07 / UTM zone 40S',NULL,'EPSG','4400','EPSG','7073','EPSG','16140',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5104','projected_crs','EPSG','7077','EPSG','4248','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','7078','RGTAAF07 / UTM zone 41S',NULL,'EPSG','4400','EPSG','7073','EPSG','16141',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5105','projected_crs','EPSG','7078','EPSG','4249','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','7079','RGTAAF07 / UTM zone 42S',NULL,'EPSG','4400','EPSG','7073','EPSG','16142',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5106','projected_crs','EPSG','7079','EPSG','4250','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','7080','RGTAAF07 / UTM zone 43S',NULL,'EPSG','4400','EPSG','7073','EPSG','16143',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5107','projected_crs','EPSG','7080','EPSG','4251','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','7081','RGTAAF07 / UTM zone 44S',NULL,'EPSG','4400','EPSG','7073','EPSG','16144',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5108','projected_crs','EPSG','7081','EPSG','4252','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','7082','RGTAAF07 / Terre Adelie Polar Stereographic',NULL,'EPSG','4400','EPSG','7073','EPSG','19983',NULL,1);
INSERT INTO "usage" VALUES('EPSG','5109','projected_crs','EPSG','7082','EPSG','2818','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','7109','NAD83(2011) / RMTCRS St Mary (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7089',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5115','projected_crs','EPSG','7109','EPSG','4310','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7110','NAD83(2011) / RMTCRS Blackfeet (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7091',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5116','projected_crs','EPSG','7110','EPSG','4311','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7111','NAD83(2011) / RMTCRS Milk River (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7093',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5117','projected_crs','EPSG','7111','EPSG','4312','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7112','NAD83(2011) / RMTCRS Fort Belknap (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7095',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5118','projected_crs','EPSG','7112','EPSG','4313','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7113','NAD83(2011) / RMTCRS Fort Peck Assiniboine (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7097',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5119','projected_crs','EPSG','7113','EPSG','4314','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7114','NAD83(2011) / RMTCRS Fort Peck Sioux (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7099',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5120','projected_crs','EPSG','7114','EPSG','4315','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7115','NAD83(2011) / RMTCRS Crow (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7101',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5121','projected_crs','EPSG','7115','EPSG','4316','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7116','NAD83(2011) / RMTCRS Bobcat (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7103',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5122','projected_crs','EPSG','7116','EPSG','4317','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7117','NAD83(2011) / RMTCRS Billings (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7105',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5123','projected_crs','EPSG','7117','EPSG','4318','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7118','NAD83(2011) / RMTCRS Wind River (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7107',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5124','projected_crs','EPSG','7118','EPSG','4319','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7119','NAD83(2011) / RMTCRS St Mary (ft)',NULL,'EPSG','4495','EPSG','6318','EPSG','7090',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5125','projected_crs','EPSG','7119','EPSG','4310','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7120','NAD83(2011) / RMTCRS Blackfeet (ft)',NULL,'EPSG','4495','EPSG','6318','EPSG','7092',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5126','projected_crs','EPSG','7120','EPSG','4311','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7121','NAD83(2011) / RMTCRS Milk River (ft)',NULL,'EPSG','4495','EPSG','6318','EPSG','7094',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5127','projected_crs','EPSG','7121','EPSG','4312','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7122','NAD83(2011) / RMTCRS Fort Belknap (ft)',NULL,'EPSG','4495','EPSG','6318','EPSG','7096',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5128','projected_crs','EPSG','7122','EPSG','4313','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7123','NAD83(2011) / RMTCRS Fort Peck Assiniboine (ft)',NULL,'EPSG','4495','EPSG','6318','EPSG','7098',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5129','projected_crs','EPSG','7123','EPSG','4314','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7124','NAD83(2011) / RMTCRS Fort Peck Sioux (ft)',NULL,'EPSG','4495','EPSG','6318','EPSG','7100',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5130','projected_crs','EPSG','7124','EPSG','4315','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7125','NAD83(2011) / RMTCRS Crow (ft)',NULL,'EPSG','4495','EPSG','6318','EPSG','7102',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5131','projected_crs','EPSG','7125','EPSG','4316','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7126','NAD83(2011) / RMTCRS Bobcat (ft)',NULL,'EPSG','4495','EPSG','6318','EPSG','7104',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5132','projected_crs','EPSG','7126','EPSG','4317','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7127','NAD83(2011) / RMTCRS Billings (ft)',NULL,'EPSG','4495','EPSG','6318','EPSG','7106',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5133','projected_crs','EPSG','7127','EPSG','4318','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7128','NAD83(2011) / RMTCRS Wind River (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7108',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5134','projected_crs','EPSG','7128','EPSG','4319','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7131','NAD83(2011) / San Francisco CS13',NULL,'EPSG','4499','EPSG','6318','EPSG','7129',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5135','projected_crs','EPSG','7131','EPSG','4228','EPSG','1194');
INSERT INTO "projected_crs" VALUES('EPSG','7132','NAD83(2011) / San Francisco CS13 (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7130',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5136','projected_crs','EPSG','7132','EPSG','4228','EPSG','1194');
INSERT INTO "projected_crs" VALUES('EPSG','7142','Palestine 1923 / Palestine Grid modified',NULL,'EPSG','4400','EPSG','4281','EPSG','7141',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5144','projected_crs','EPSG','7142','EPSG','1356','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','7257','NAD83(2011) / InGCS Adams (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7143',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5145','projected_crs','EPSG','7257','EPSG','4289','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7258','NAD83(2011) / InGCS Adams (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7144',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5146','projected_crs','EPSG','7258','EPSG','4289','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7259','NAD83(2011) / InGCS Allen (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7145',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5147','projected_crs','EPSG','7259','EPSG','4285','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7260','NAD83(2011) / InGCS Allen (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7146',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5148','projected_crs','EPSG','7260','EPSG','4285','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7261','NAD83(2011) / InGCS Bartholomew (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7147',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5149','projected_crs','EPSG','7261','EPSG','4302','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7262','NAD83(2011) / InGCS Bartholomew (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7148',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5150','projected_crs','EPSG','7262','EPSG','4302','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7263','NAD83(2011) / InGCS Benton (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7149',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5151','projected_crs','EPSG','7263','EPSG','4256','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7264','NAD83(2011) / InGCS Benton (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7150',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5152','projected_crs','EPSG','7264','EPSG','4256','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7265','NAD83(2011) / InGCS Blackford-Delaware (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7151',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5153','projected_crs','EPSG','7265','EPSG','4291','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7266','NAD83(2011) / InGCS Blackford-Delaware (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7152',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5154','projected_crs','EPSG','7266','EPSG','4291','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7267','NAD83(2011) / InGCS Boone-Hendricks (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7153',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5155','projected_crs','EPSG','7267','EPSG','4263','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7268','NAD83(2011) / InGCS Boone-Hendricks (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7154',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5156','projected_crs','EPSG','7268','EPSG','4263','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7269','NAD83(2011) / InGCS Brown (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7155',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5157','projected_crs','EPSG','7269','EPSG','4301','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7270','NAD83(2011) / InGCS Brown (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7156',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5158','projected_crs','EPSG','7270','EPSG','4301','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7271','NAD83(2011) / InGCS Carroll (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7157',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5159','projected_crs','EPSG','7271','EPSG','4258','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7272','NAD83(2011) / InGCS Carroll (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7158',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5160','projected_crs','EPSG','7272','EPSG','4258','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7273','NAD83(2011) / InGCS Cass (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7159',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5161','projected_crs','EPSG','7273','EPSG','4286','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7274','NAD83(2011) / InGCS Cass (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7160',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5162','projected_crs','EPSG','7274','EPSG','4286','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7275','NAD83(2011) / InGCS Clark-Floyd-Scott (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7161',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5163','projected_crs','EPSG','7275','EPSG','4308','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7276','NAD83(2011) / InGCS Clark-Floyd-Scott (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7162',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5164','projected_crs','EPSG','7276','EPSG','4308','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7277','NAD83(2011) / InGCS Clay (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7163',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5165','projected_crs','EPSG','7277','EPSG','4265','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7278','NAD83(2011) / InGCS Clay (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7164',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5166','projected_crs','EPSG','7278','EPSG','4265','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7279','NAD83(2011) / InGCS Clinton (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7165',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5167','projected_crs','EPSG','7279','EPSG','4260','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7280','NAD83(2011) / InGCS Clinton (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7166',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5168','projected_crs','EPSG','7280','EPSG','4260','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7281','NAD83(2011) / InGCS Crawford-Lawrence-Orange (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7167',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5169','projected_crs','EPSG','7281','EPSG','4272','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7282','NAD83(2011) / InGCS Crawford-Lawrence-Orange (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7168',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5170','projected_crs','EPSG','7282','EPSG','4272','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7283','NAD83(2011) / InGCS Daviess-Greene (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7169',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5171','projected_crs','EPSG','7283','EPSG','4269','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7284','NAD83(2011) / InGCS Daviess-Greene (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7170',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5172','projected_crs','EPSG','7284','EPSG','4269','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7285','NAD83(2011) / InGCS Dearborn-Ohio-Switzerland (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7171',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5173','projected_crs','EPSG','7285','EPSG','4306','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7286','NAD83(2011) / InGCS Dearborn-Ohio-Switzerland (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7172',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5174','projected_crs','EPSG','7286','EPSG','4306','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7287','NAD83(2011) / InGCS Decatur-Rush (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7173',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5175','projected_crs','EPSG','7287','EPSG','4299','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7288','NAD83(2011) / InGCS Decatur-Rush (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7174',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5176','projected_crs','EPSG','7288','EPSG','4299','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7289','NAD83(2011) / InGCS DeKalb (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7175',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5177','projected_crs','EPSG','7289','EPSG','4283','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7290','NAD83(2011) / InGCS DeKalb (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7176',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5178','projected_crs','EPSG','7290','EPSG','4283','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7291','NAD83(2011) / InGCS Dubois-Martin (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7177',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5179','projected_crs','EPSG','7291','EPSG','4271','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7292','NAD83(2011) / InGCS Dubois-Martin (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7178',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5180','projected_crs','EPSG','7292','EPSG','4271','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7293','NAD83(2011) / InGCS Elkhart-Kosciusko-Wabash (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7179',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5181','projected_crs','EPSG','7293','EPSG','4280','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7294','NAD83(2011) / InGCS Elkhart-Kosciusko-Wabash (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7180',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5182','projected_crs','EPSG','7294','EPSG','4280','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7295','NAD83(2011) / InGCS Fayette-Franklin-Union (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7181',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5183','projected_crs','EPSG','7295','EPSG','4300','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7296','NAD83(2011) / InGCS Fayette-Franklin-Union (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7182',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5184','projected_crs','EPSG','7296','EPSG','4300','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7297','NAD83(2011) / InGCS Fountain-Warren (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7183',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5185','projected_crs','EPSG','7297','EPSG','4259','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7298','NAD83(2011) / InGCS Fountain-Warren (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7184',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5186','projected_crs','EPSG','7298','EPSG','4259','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7299','NAD83(2011) / InGCS Fulton-Marshall-St. Joseph (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7185',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5187','projected_crs','EPSG','7299','EPSG','4279','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7300','NAD83(2011) / InGCS Fulton-Marshall-St. Joseph (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7186',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5188','projected_crs','EPSG','7300','EPSG','4279','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7301','NAD83(2011) / InGCS Gibson (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7187',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5189','projected_crs','EPSG','7301','EPSG','4273','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7302','NAD83(2011) / InGCS Gibson (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7188',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5190','projected_crs','EPSG','7302','EPSG','4273','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7303','NAD83(2011) / InGCS Grant (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7189',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5191','projected_crs','EPSG','7303','EPSG','4290','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7304','NAD83(2011) / InGCS Grant (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7190',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5192','projected_crs','EPSG','7304','EPSG','4290','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7305','NAD83(2011) / InGCS Hamilton-Tipton (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7191',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5193','projected_crs','EPSG','7305','EPSG','4293','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7306','NAD83(2011) / InGCS Hamilton-Tipton (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7192',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5194','projected_crs','EPSG','7306','EPSG','4293','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7307','NAD83(2011) / InGCS Hancock-Madison (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7193',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5195','projected_crs','EPSG','7307','EPSG','4294','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7308','NAD83(2011) / InGCS Hancock-Madison (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7194',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5196','projected_crs','EPSG','7308','EPSG','4294','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7309','NAD83(2011) / InGCS Harrison-Washington (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7195',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5197','projected_crs','EPSG','7309','EPSG','4307','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7310','NAD83(2011) / InGCS Harrison-Washington (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7196',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5198','projected_crs','EPSG','7310','EPSG','4307','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7311','NAD83(2011) / InGCS Henry (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7197',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5199','projected_crs','EPSG','7311','EPSG','4296','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7312','NAD83(2011) / InGCS Henry (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7198',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5200','projected_crs','EPSG','7312','EPSG','4296','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7313','NAD83(2011) / InGCS Howard-Miami (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7199',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5201','projected_crs','EPSG','7313','EPSG','4287','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7314','NAD83(2011) / InGCS Howard-Miami (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7200',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5202','projected_crs','EPSG','7314','EPSG','4287','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7315','NAD83(2011) / InGCS Huntington-Whitley (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7201',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5203','projected_crs','EPSG','7315','EPSG','4284','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7316','NAD83(2011) / InGCS Huntington-Whitley (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7202',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5204','projected_crs','EPSG','7316','EPSG','4284','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7317','NAD83(2011) / InGCS Jackson (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7203',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5205','projected_crs','EPSG','7317','EPSG','4303','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7318','NAD83(2011) / InGCS Jackson (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7204',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5206','projected_crs','EPSG','7318','EPSG','4303','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7319','NAD83(2011) / InGCS Jasper-Porter (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7205',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5207','projected_crs','EPSG','7319','EPSG','4254','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7320','NAD83(2011) / InGCS Jasper-Porter (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7206',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5208','projected_crs','EPSG','7320','EPSG','4254','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7321','NAD83(2011) / InGCS Jay (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7207',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5209','projected_crs','EPSG','7321','EPSG','4292','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7322','NAD83(2011) / InGCS Jay (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7208',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5210','projected_crs','EPSG','7322','EPSG','4292','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7323','NAD83(2011) / InGCS Jefferson (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7209',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5211','projected_crs','EPSG','7323','EPSG','4309','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7324','NAD83(2011) / InGCS Jefferson (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7210',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5212','projected_crs','EPSG','7324','EPSG','4309','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7325','NAD83(2011) / InGCS Jennings (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7211',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5213','projected_crs','EPSG','7325','EPSG','4304','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7326','NAD83(2011) / InGCS Jennings (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7212',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5214','projected_crs','EPSG','7326','EPSG','4304','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7327','NAD83(2011) / InGCS Johnson-Marion (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7213',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5215','projected_crs','EPSG','7327','EPSG','4297','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7328','NAD83(2011) / InGCS Johnson-Marion (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7214',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5216','projected_crs','EPSG','7328','EPSG','4297','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7329','NAD83(2011) / InGCS Knox (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7215',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5217','projected_crs','EPSG','7329','EPSG','4270','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7330','NAD83(2011) / InGCS Knox (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7216',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5218','projected_crs','EPSG','7330','EPSG','4270','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7331','NAD83(2011) / InGCS LaGrange-Noble (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7217',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5219','projected_crs','EPSG','7331','EPSG','4281','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7332','NAD83(2011) / InGCS LaGrange-Noble (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7218',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5220','projected_crs','EPSG','7332','EPSG','4281','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7333','NAD83(2011) / InGCS Lake-Newton (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7219',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5221','projected_crs','EPSG','7333','EPSG','4253','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7334','NAD83(2011) / InGCS Lake-Newton (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7220',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5222','projected_crs','EPSG','7334','EPSG','4253','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7335','NAD83(2011) / InGCS LaPorte-Pulaski-Starke (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7221',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5223','projected_crs','EPSG','7335','EPSG','4255','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7336','NAD83(2011) / InGCS LaPorte-Pulaski-Starke (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7222',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5224','projected_crs','EPSG','7336','EPSG','4255','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7337','NAD83(2011) / InGCS Monroe-Morgan (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7223',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5225','projected_crs','EPSG','7337','EPSG','4267','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7338','NAD83(2011) / InGCS Monroe-Morgan (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7224',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5226','projected_crs','EPSG','7338','EPSG','4267','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7339','NAD83(2011) / InGCS Montgomery-Putnam (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7225',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5227','projected_crs','EPSG','7339','EPSG','4262','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7340','NAD83(2011) / InGCS Montgomery-Putnam (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7226',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5228','projected_crs','EPSG','7340','EPSG','4262','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7341','NAD83(2011) / InGCS Owen (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7227',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5229','projected_crs','EPSG','7341','EPSG','4266','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7342','NAD83(2011) / InGCS Owen (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7228',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5230','projected_crs','EPSG','7342','EPSG','4266','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7343','NAD83(2011) / InGCS Parke-Vermillion (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7229',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5231','projected_crs','EPSG','7343','EPSG','4261','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7344','NAD83(2011) / InGCS Parke-Vermillion (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7230',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5232','projected_crs','EPSG','7344','EPSG','4261','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7345','NAD83(2011) / InGCS Perry (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7231',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5233','projected_crs','EPSG','7345','EPSG','4278','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7346','NAD83(2011) / InGCS Perry (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7232',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5234','projected_crs','EPSG','7346','EPSG','4278','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7347','NAD83(2011) / InGCS Pike-Warrick (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7233',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5235','projected_crs','EPSG','7347','EPSG','4274','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7348','NAD83(2011) / InGCS Pike-Warrick (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7234',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5236','projected_crs','EPSG','7348','EPSG','4274','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7349','NAD83(2011) / InGCS Posey (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7235',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5237','projected_crs','EPSG','7349','EPSG','4275','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7350','NAD83(2011) / InGCS Posey (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7236',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5238','projected_crs','EPSG','7350','EPSG','4275','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7351','NAD83(2011) / InGCS Randolph-Wayne (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7237',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5239','projected_crs','EPSG','7351','EPSG','4295','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7352','NAD83(2011) / InGCS Randolph-Wayne (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7238',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5240','projected_crs','EPSG','7352','EPSG','4295','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7353','NAD83(2011) / InGCS Ripley (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7239',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5241','projected_crs','EPSG','7353','EPSG','4305','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7354','NAD83(2011) / InGCS Ripley (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7240',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5242','projected_crs','EPSG','7354','EPSG','4305','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7355','NAD83(2011) / InGCS Shelby (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7241',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5243','projected_crs','EPSG','7355','EPSG','4298','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7356','NAD83(2011) / InGCS Shelby (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7242',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5244','projected_crs','EPSG','7356','EPSG','4298','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7357','NAD83(2011) / InGCS Spencer (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7243',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5245','projected_crs','EPSG','7357','EPSG','4277','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7358','NAD83(2011) / InGCS Spencer (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7244',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5246','projected_crs','EPSG','7358','EPSG','4277','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7359','NAD83(2011) / InGCS Steuben (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7245',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5247','projected_crs','EPSG','7359','EPSG','4282','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7360','NAD83(2011) / InGCS Steuben (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7246',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5248','projected_crs','EPSG','7360','EPSG','4282','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7361','NAD83(2011) / InGCS Sullivan (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7247',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5249','projected_crs','EPSG','7361','EPSG','4268','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7362','NAD83(2011) / InGCS Sullivan (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7248',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5250','projected_crs','EPSG','7362','EPSG','4268','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7363','NAD83(2011) / InGCS Tippecanoe-White (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7249',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5251','projected_crs','EPSG','7363','EPSG','4257','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7364','NAD83(2011) / InGCS Tippecanoe-White (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7250',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5252','projected_crs','EPSG','7364','EPSG','4257','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7365','NAD83(2011) / InGCS Vanderburgh (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7251',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5253','projected_crs','EPSG','7365','EPSG','4276','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7366','NAD83(2011) / InGCS Vanderburgh (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7252',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5254','projected_crs','EPSG','7366','EPSG','4276','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7367','NAD83(2011) / InGCS Vigo (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7253',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5255','projected_crs','EPSG','7367','EPSG','4264','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7368','NAD83(2011) / InGCS Vigo (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7254',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5256','projected_crs','EPSG','7368','EPSG','4264','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7369','NAD83(2011) / InGCS Wells (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7255',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5257','projected_crs','EPSG','7369','EPSG','4288','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7370','NAD83(2011) / InGCS Wells (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7256',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5258','projected_crs','EPSG','7370','EPSG','4288','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7374','ONGD14 / UTM zone 39N',NULL,'EPSG','4400','EPSG','7373','EPSG','16039',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5262','projected_crs','EPSG','7374','EPSG','4322','EPSG','1153');
INSERT INTO "projected_crs" VALUES('EPSG','7375','ONGD14 / UTM zone 40N',NULL,'EPSG','4400','EPSG','7373','EPSG','16040',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5263','projected_crs','EPSG','7375','EPSG','4323','EPSG','1153');
INSERT INTO "projected_crs" VALUES('EPSG','7376','ONGD14 / UTM zone 41N',NULL,'EPSG','4400','EPSG','7373','EPSG','16041',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5264','projected_crs','EPSG','7376','EPSG','4324','EPSG','1153');
INSERT INTO "projected_crs" VALUES('EPSG','7528','NAD83(2011) / WISCRS Adams and Juneau (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7484',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5291','projected_crs','EPSG','7528','EPSG','4360','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7529','NAD83(2011) / WISCRS Ashland (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7378',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5292','projected_crs','EPSG','7529','EPSG','4320','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7530','NAD83(2011) / WISCRS Barron (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7426',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5293','projected_crs','EPSG','7530','EPSG','4331','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7531','NAD83(2011) / WISCRS Bayfield (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7380',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5294','projected_crs','EPSG','7531','EPSG','4321','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7532','NAD83(2011) / WISCRS Brown (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7428',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5295','projected_crs','EPSG','7532','EPSG','4336','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7533','NAD83(2011) / WISCRS Buffalo (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7430',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5296','projected_crs','EPSG','7533','EPSG','4337','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7534','NAD83(2011) / WISCRS Burnett (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7382',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5297','projected_crs','EPSG','7534','EPSG','4325','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7535','NAD83(2011) / WISCRS Calumet, Fond du Lac, Outagamie and Winnebago (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7486',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5298','projected_crs','EPSG','7535','EPSG','4361','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7536','NAD83(2011) / WISCRS Chippewa (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7432',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5299','projected_crs','EPSG','7536','EPSG','4338','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7537','NAD83(2011) / WISCRS Clark (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7434',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5300','projected_crs','EPSG','7537','EPSG','4339','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7538','NAD83(2011) / WISCRS Columbia (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7488',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5301','projected_crs','EPSG','7538','EPSG','4362','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7539','NAD83(2011) / WISCRS Crawford (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7490',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5302','projected_crs','EPSG','7539','EPSG','4363','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7540','NAD83(2011) / WISCRS Dane (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7492',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5303','projected_crs','EPSG','7540','EPSG','4364','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7541','NAD83(2011) / WISCRS Dodge and Jefferson (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7494',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5304','projected_crs','EPSG','7541','EPSG','4365','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7542','NAD83(2011) / WISCRS Door (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7436',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5305','projected_crs','EPSG','7542','EPSG','4340','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7543','NAD83(2011) / WISCRS Douglas (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7384',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5306','projected_crs','EPSG','7543','EPSG','4326','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7544','NAD83(2011) / WISCRS Dunn (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7438',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5307','projected_crs','EPSG','7544','EPSG','4341','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7545','NAD83(2011) / WISCRS Eau Claire (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7440',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5308','projected_crs','EPSG','7545','EPSG','4342','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7546','NAD83(2011) / WISCRS Florence (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7386',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5309','projected_crs','EPSG','7546','EPSG','4327','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7547','NAD83(2011) / WISCRS Forest (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7388',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5310','projected_crs','EPSG','7547','EPSG','4328','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7548','NAD83(2011) / WISCRS Grant (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7496',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5311','projected_crs','EPSG','7548','EPSG','4366','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7549','NAD83(2011) / WISCRS Green and Lafayette (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7498',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5312','projected_crs','EPSG','7549','EPSG','4367','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7550','NAD83(2011) / WISCRS Green Lake and Marquette (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7500',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5313','projected_crs','EPSG','7550','EPSG','4368','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7551','NAD83(2011) / WISCRS Iowa (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7502',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5314','projected_crs','EPSG','7551','EPSG','4369','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7552','NAD83(2011) / WISCRS Iron (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7390',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5315','projected_crs','EPSG','7552','EPSG','4329','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7553','NAD83(2011) / WISCRS Jackson (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7450',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5316','projected_crs','EPSG','7553','EPSG','4343','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7554','NAD83(2011) / WISCRS Kenosha, Milwaukee, Ozaukee and Racine (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7504',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5317','projected_crs','EPSG','7554','EPSG','4370','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7555','NAD83(2011) / WISCRS Kewaunee, Manitowoc and Sheboygan (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7506',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5318','projected_crs','EPSG','7555','EPSG','4371','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7556','NAD83(2011) / WISCRS La Crosse (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7508',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5319','projected_crs','EPSG','7556','EPSG','4372','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7557','NAD83(2011) / WISCRS Langlade (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7452',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5320','projected_crs','EPSG','7557','EPSG','4344','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7558','NAD83(2011) / WISCRS Lincoln (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7454',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5321','projected_crs','EPSG','7558','EPSG','4345','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7559','NAD83(2011) / WISCRS Marathon (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7456',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5322','projected_crs','EPSG','7559','EPSG','4346','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7560','NAD83(2011) / WISCRS Marinette (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7458',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5323','projected_crs','EPSG','7560','EPSG','4347','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7561','NAD83(2011) / WISCRS Menominee (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7460',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5324','projected_crs','EPSG','7561','EPSG','4348','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7562','NAD83(2011) / WISCRS Monroe (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7510',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5325','projected_crs','EPSG','7562','EPSG','4373','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7563','NAD83(2011) / WISCRS Oconto (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7462',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5326','projected_crs','EPSG','7563','EPSG','4349','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7564','NAD83(2011) / WISCRS Oneida (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7392',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5327','projected_crs','EPSG','7564','EPSG','4330','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7565','NAD83(2011) / WISCRS Pepin and Pierce (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7464',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5328','projected_crs','EPSG','7565','EPSG','4350','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7566','NAD83(2011) / WISCRS Polk (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7466',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5329','projected_crs','EPSG','7566','EPSG','4351','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7567','NAD83(2011) / WISCRS Portage (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7468',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5330','projected_crs','EPSG','7567','EPSG','4352','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7568','NAD83(2011) / WISCRS Price (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7394',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5331','projected_crs','EPSG','7568','EPSG','4332','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7569','NAD83(2011) / WISCRS Richland (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7512',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5332','projected_crs','EPSG','7569','EPSG','4374','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7570','NAD83(2011) / WISCRS Rock (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7514',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5333','projected_crs','EPSG','7570','EPSG','4375','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7571','NAD83(2011) / WISCRS Rusk (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7470',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5334','projected_crs','EPSG','7571','EPSG','4353','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7572','NAD83(2011) / WISCRS Sauk (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7516',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5335','projected_crs','EPSG','7572','EPSG','4376','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7573','NAD83(2011) / WISCRS Sawyer (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7396',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5336','projected_crs','EPSG','7573','EPSG','4333','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7574','NAD83(2011) / WISCRS Shawano (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7472',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5337','projected_crs','EPSG','7574','EPSG','4354','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7575','NAD83(2011) / WISCRS St. Croix (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7474',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5338','projected_crs','EPSG','7575','EPSG','4355','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7576','NAD83(2011) / WISCRS Taylor (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7476',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5339','projected_crs','EPSG','7576','EPSG','4356','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7577','NAD83(2011) / WISCRS Trempealeau (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7478',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5340','projected_crs','EPSG','7577','EPSG','4357','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7578','NAD83(2011) / WISCRS Vernon (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7518',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5341','projected_crs','EPSG','7578','EPSG','4377','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7579','NAD83(2011) / WISCRS Vilas (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7398',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5342','projected_crs','EPSG','7579','EPSG','4334','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7580','NAD83(2011) / WISCRS Walworth (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7520',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5343','projected_crs','EPSG','7580','EPSG','4378','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7581','NAD83(2011) / WISCRS Washburn (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7424',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5344','projected_crs','EPSG','7581','EPSG','4335','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7582','NAD83(2011) / WISCRS Washington (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7522',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5345','projected_crs','EPSG','7582','EPSG','4379','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7583','NAD83(2011) / WISCRS Waukesha (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7524',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5346','projected_crs','EPSG','7583','EPSG','4380','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7584','NAD83(2011) / WISCRS Waupaca (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7480',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5347','projected_crs','EPSG','7584','EPSG','4358','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7585','NAD83(2011) / WISCRS Waushara (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7526',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5348','projected_crs','EPSG','7585','EPSG','4381','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7586','NAD83(2011) / WISCRS Wood (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','7482',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5349','projected_crs','EPSG','7586','EPSG','4359','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7587','NAD83(2011) / WISCRS Adams and Juneau (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7485',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5350','projected_crs','EPSG','7587','EPSG','4360','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7588','NAD83(2011) / WISCRS Ashland (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7379',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5351','projected_crs','EPSG','7588','EPSG','4320','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7589','NAD83(2011) / WISCRS Barron (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7427',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5352','projected_crs','EPSG','7589','EPSG','4331','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7590','NAD83(2011) / WISCRS Bayfield (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7381',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5353','projected_crs','EPSG','7590','EPSG','4321','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7591','NAD83(2011) / WISCRS Brown (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7429',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5354','projected_crs','EPSG','7591','EPSG','4336','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7592','NAD83(2011) / WISCRS Buffalo (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7431',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5355','projected_crs','EPSG','7592','EPSG','4337','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7593','NAD83(2011) / WISCRS Burnett (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7383',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5356','projected_crs','EPSG','7593','EPSG','4325','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7594','NAD83(2011) / WISCRS Calumet, Fond du Lac, Outagamie and Winnebago (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7487',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5357','projected_crs','EPSG','7594','EPSG','4361','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7595','NAD83(2011) / WISCRS Chippewa (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7433',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5358','projected_crs','EPSG','7595','EPSG','4338','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7596','NAD83(2011) / WISCRS Clark (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7435',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5359','projected_crs','EPSG','7596','EPSG','4339','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7597','NAD83(2011) / WISCRS Columbia (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7489',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5360','projected_crs','EPSG','7597','EPSG','4362','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7598','NAD83(2011) / WISCRS Crawford (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7491',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5361','projected_crs','EPSG','7598','EPSG','4363','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7599','NAD83(2011) / WISCRS Dane (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7493',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5362','projected_crs','EPSG','7599','EPSG','4364','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7600','NAD83(2011) / WISCRS Dodge and Jefferson (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7495',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5363','projected_crs','EPSG','7600','EPSG','4365','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7601','NAD83(2011) / WISCRS Door (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7437',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5364','projected_crs','EPSG','7601','EPSG','4340','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7602','NAD83(2011) / WISCRS Douglas (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7385',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5365','projected_crs','EPSG','7602','EPSG','4326','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7603','NAD83(2011) / WISCRS Dunn (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7439',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5366','projected_crs','EPSG','7603','EPSG','4341','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7604','NAD83(2011) / WISCRS Eau Claire (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7441',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5367','projected_crs','EPSG','7604','EPSG','4342','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7605','NAD83(2011) / WISCRS Florence (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7387',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5368','projected_crs','EPSG','7605','EPSG','4327','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7606','NAD83(2011) / WISCRS Forest (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7389',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5369','projected_crs','EPSG','7606','EPSG','4328','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7607','NAD83(2011) / WISCRS Grant (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7497',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5370','projected_crs','EPSG','7607','EPSG','4366','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7608','NAD83(2011) / WISCRS Green and Lafayette (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7499',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5371','projected_crs','EPSG','7608','EPSG','4367','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7609','NAD83(2011) / WISCRS Green Lake and Marquette (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7501',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5372','projected_crs','EPSG','7609','EPSG','4368','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7610','NAD83(2011) / WISCRS Iowa (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7503',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5373','projected_crs','EPSG','7610','EPSG','4369','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7611','NAD83(2011) / WISCRS Iron (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7391',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5374','projected_crs','EPSG','7611','EPSG','4329','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7612','NAD83(2011) / WISCRS Jackson (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7451',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5375','projected_crs','EPSG','7612','EPSG','4343','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7613','NAD83(2011) / WISCRS Kenosha, Milwaukee, Ozaukee and Racine (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7505',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5376','projected_crs','EPSG','7613','EPSG','4370','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7614','NAD83(2011) / WISCRS Kewaunee, Manitowoc and Sheboygan (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7507',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5377','projected_crs','EPSG','7614','EPSG','4371','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7615','NAD83(2011) / WISCRS La Crosse (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7509',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5378','projected_crs','EPSG','7615','EPSG','4372','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7616','NAD83(2011) / WISCRS Langlade (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7453',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5379','projected_crs','EPSG','7616','EPSG','4344','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7617','NAD83(2011) / WISCRS Lincoln (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7455',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5380','projected_crs','EPSG','7617','EPSG','4345','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7618','NAD83(2011) / WISCRS Marathon (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7457',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5381','projected_crs','EPSG','7618','EPSG','4346','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7619','NAD83(2011) / WISCRS Marinette (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7459',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5382','projected_crs','EPSG','7619','EPSG','4347','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7620','NAD83(2011) / WISCRS Menominee (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7461',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5383','projected_crs','EPSG','7620','EPSG','4348','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7621','NAD83(2011) / WISCRS Monroe (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7511',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5384','projected_crs','EPSG','7621','EPSG','4373','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7622','NAD83(2011) / WISCRS Oconto (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7463',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5385','projected_crs','EPSG','7622','EPSG','4349','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7623','NAD83(2011) / WISCRS Oneida (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7393',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5386','projected_crs','EPSG','7623','EPSG','4330','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7624','NAD83(2011) / WISCRS Pepin and Pierce (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7465',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5387','projected_crs','EPSG','7624','EPSG','4350','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7625','NAD83(2011) / WISCRS Polk (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7467',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5388','projected_crs','EPSG','7625','EPSG','4351','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7626','NAD83(2011) / WISCRS Portage (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7469',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5389','projected_crs','EPSG','7626','EPSG','4352','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7627','NAD83(2011) / WISCRS Price (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7395',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5390','projected_crs','EPSG','7627','EPSG','4332','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7628','NAD83(2011) / WISCRS Richland (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7513',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5391','projected_crs','EPSG','7628','EPSG','4374','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7629','NAD83(2011) / WISCRS Rock (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7515',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5392','projected_crs','EPSG','7629','EPSG','4375','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7630','NAD83(2011) / WISCRS Rusk (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7471',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5393','projected_crs','EPSG','7630','EPSG','4353','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7631','NAD83(2011) / WISCRS Sauk (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7517',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5394','projected_crs','EPSG','7631','EPSG','4376','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7632','NAD83(2011) / WISCRS Sawyer (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7397',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5395','projected_crs','EPSG','7632','EPSG','4333','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7633','NAD83(2011) / WISCRS Shawano (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7473',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5396','projected_crs','EPSG','7633','EPSG','4354','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7634','NAD83(2011) / WISCRS St. Croix (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7475',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5397','projected_crs','EPSG','7634','EPSG','4355','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7635','NAD83(2011) / WISCRS Taylor (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7477',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5398','projected_crs','EPSG','7635','EPSG','4356','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7636','NAD83(2011) / WISCRS Trempealeau (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7479',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5399','projected_crs','EPSG','7636','EPSG','4357','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7637','NAD83(2011) / WISCRS Vernon (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7519',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5400','projected_crs','EPSG','7637','EPSG','4377','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7638','NAD83(2011) / WISCRS Vilas (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7399',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5401','projected_crs','EPSG','7638','EPSG','4334','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7639','NAD83(2011) / WISCRS Walworth (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7521',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5402','projected_crs','EPSG','7639','EPSG','4378','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7640','NAD83(2011) / WISCRS Washburn (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7425',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5403','projected_crs','EPSG','7640','EPSG','4335','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7641','NAD83(2011) / WISCRS Washington (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7523',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5404','projected_crs','EPSG','7641','EPSG','4379','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7642','NAD83(2011) / WISCRS Waukesha (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7525',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5405','projected_crs','EPSG','7642','EPSG','4380','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7643','NAD83(2011) / WISCRS Waupaca (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7481',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5406','projected_crs','EPSG','7643','EPSG','4358','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7644','NAD83(2011) / WISCRS Waushara (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7527',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5407','projected_crs','EPSG','7644','EPSG','4381','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7645','NAD83(2011) / WISCRS Wood (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','7483',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5408','projected_crs','EPSG','7645','EPSG','4359','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','7692','Kyrg-06 / zone 1',NULL,'EPSG','4400','EPSG','7686','EPSG','7687',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5431','projected_crs','EPSG','7692','EPSG','4385','EPSG','1056');
INSERT INTO "projected_crs" VALUES('EPSG','7693','Kyrg-06 / zone 2',NULL,'EPSG','4400','EPSG','7686','EPSG','7688',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5432','projected_crs','EPSG','7693','EPSG','4386','EPSG','1056');
INSERT INTO "projected_crs" VALUES('EPSG','7694','Kyrg-06 / zone 3',NULL,'EPSG','4400','EPSG','7686','EPSG','7689',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5433','projected_crs','EPSG','7694','EPSG','4387','EPSG','1056');
INSERT INTO "projected_crs" VALUES('EPSG','7695','Kyrg-06 / zone 4',NULL,'EPSG','4400','EPSG','7686','EPSG','7690',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5434','projected_crs','EPSG','7695','EPSG','4388','EPSG','1056');
INSERT INTO "projected_crs" VALUES('EPSG','7696','Kyrg-06 / zone 5',NULL,'EPSG','4400','EPSG','7686','EPSG','7691',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5435','projected_crs','EPSG','7696','EPSG','4389','EPSG','1056');
INSERT INTO "projected_crs" VALUES('EPSG','7755','WGS 84 / India NSF LCC',NULL,'EPSG','4499','EPSG','4326','EPSG','7722',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5440','projected_crs','EPSG','7755','EPSG','1121','EPSG','1219');
INSERT INTO "projected_crs" VALUES('EPSG','7756','WGS 84 / Andhra Pradesh',NULL,'EPSG','4499','EPSG','4326','EPSG','7723',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5441','projected_crs','EPSG','7756','EPSG','4394','EPSG','1219');
INSERT INTO "projected_crs" VALUES('EPSG','7757','WGS 84 / Arunachal Pradesh',NULL,'EPSG','4499','EPSG','4326','EPSG','7724',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5442','projected_crs','EPSG','7757','EPSG','4395','EPSG','1219');
INSERT INTO "projected_crs" VALUES('EPSG','7758','WGS 84 / Assam',NULL,'EPSG','4499','EPSG','4326','EPSG','7725',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5443','projected_crs','EPSG','7758','EPSG','4396','EPSG','1219');
INSERT INTO "projected_crs" VALUES('EPSG','7759','WGS 84 / Bihar',NULL,'EPSG','4499','EPSG','4326','EPSG','7726',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5444','projected_crs','EPSG','7759','EPSG','4397','EPSG','1219');
INSERT INTO "projected_crs" VALUES('EPSG','7760','WGS 84 / Delhi',NULL,'EPSG','4499','EPSG','4326','EPSG','7727',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5445','projected_crs','EPSG','7760','EPSG','4422','EPSG','1219');
INSERT INTO "projected_crs" VALUES('EPSG','7761','WGS 84 / Gujarat',NULL,'EPSG','4499','EPSG','4326','EPSG','7728',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5446','projected_crs','EPSG','7761','EPSG','4400','EPSG','1219');
INSERT INTO "projected_crs" VALUES('EPSG','7762','WGS 84 / Haryana',NULL,'EPSG','4499','EPSG','4326','EPSG','7729',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5447','projected_crs','EPSG','7762','EPSG','4401','EPSG','1219');
INSERT INTO "projected_crs" VALUES('EPSG','7763','WGS 84 / Himachal Pradesh',NULL,'EPSG','4499','EPSG','4326','EPSG','7730',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5448','projected_crs','EPSG','7763','EPSG','4402','EPSG','1219');
INSERT INTO "projected_crs" VALUES('EPSG','7764','WGS 84 / Jammu and Kashmir',NULL,'EPSG','4499','EPSG','4326','EPSG','7731',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5449','projected_crs','EPSG','7764','EPSG','4403','EPSG','1219');
INSERT INTO "projected_crs" VALUES('EPSG','7765','WGS 84 / Jharkhand',NULL,'EPSG','4499','EPSG','4326','EPSG','7732',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5450','projected_crs','EPSG','7765','EPSG','4404','EPSG','1219');
INSERT INTO "projected_crs" VALUES('EPSG','7766','WGS 84 / Madhya Pradesh',NULL,'EPSG','4499','EPSG','4326','EPSG','7733',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5451','projected_crs','EPSG','7766','EPSG','4407','EPSG','1219');
INSERT INTO "projected_crs" VALUES('EPSG','7767','WGS 84 / Maharashtra',NULL,'EPSG','4499','EPSG','4326','EPSG','7734',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5452','projected_crs','EPSG','7767','EPSG','4408','EPSG','1219');
INSERT INTO "projected_crs" VALUES('EPSG','7768','WGS 84 / Manipur',NULL,'EPSG','4499','EPSG','4326','EPSG','7735',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5453','projected_crs','EPSG','7768','EPSG','4409','EPSG','1219');
INSERT INTO "projected_crs" VALUES('EPSG','7769','WGS 84 / Meghalaya',NULL,'EPSG','4499','EPSG','4326','EPSG','7736',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5454','projected_crs','EPSG','7769','EPSG','4410','EPSG','1219');
INSERT INTO "projected_crs" VALUES('EPSG','7770','WGS 84 / Nagaland',NULL,'EPSG','4499','EPSG','4326','EPSG','7737',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5455','projected_crs','EPSG','7770','EPSG','4412','EPSG','1219');
INSERT INTO "projected_crs" VALUES('EPSG','7771','WGS 84 / India Northeast',NULL,'EPSG','4499','EPSG','4326','EPSG','7738',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5456','projected_crs','EPSG','7771','EPSG','4392','EPSG','1219');
INSERT INTO "projected_crs" VALUES('EPSG','7772','WGS 84 / Orissa',NULL,'EPSG','4499','EPSG','4326','EPSG','7739',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5457','projected_crs','EPSG','7772','EPSG','4413','EPSG','1219');
INSERT INTO "projected_crs" VALUES('EPSG','7773','WGS 84 / Punjab',NULL,'EPSG','4499','EPSG','4326','EPSG','7740',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5458','projected_crs','EPSG','7773','EPSG','4414','EPSG','1219');
INSERT INTO "projected_crs" VALUES('EPSG','7774','WGS 84 / Rajasthan',NULL,'EPSG','4499','EPSG','4326','EPSG','7741',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5459','projected_crs','EPSG','7774','EPSG','4415','EPSG','1219');
INSERT INTO "projected_crs" VALUES('EPSG','7775','WGS 84 / Uttar Pradesh',NULL,'EPSG','4499','EPSG','4326','EPSG','7742',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5460','projected_crs','EPSG','7775','EPSG','4419','EPSG','1219');
INSERT INTO "projected_crs" VALUES('EPSG','7776','WGS 84 / Uttaranchal',NULL,'EPSG','4499','EPSG','4326','EPSG','7743',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5461','projected_crs','EPSG','7776','EPSG','4420','EPSG','1219');
INSERT INTO "projected_crs" VALUES('EPSG','7777','WGS 84 / Andaman and Nicobar',NULL,'EPSG','4499','EPSG','4326','EPSG','7744',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5462','projected_crs','EPSG','7777','EPSG','4423','EPSG','1219');
INSERT INTO "projected_crs" VALUES('EPSG','7778','WGS 84 / Chhattisgarh',NULL,'EPSG','4499','EPSG','4326','EPSG','7745',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5463','projected_crs','EPSG','7778','EPSG','4398','EPSG','1219');
INSERT INTO "projected_crs" VALUES('EPSG','7779','WGS 84 / Goa',NULL,'EPSG','4499','EPSG','4326','EPSG','7746',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5464','projected_crs','EPSG','7779','EPSG','4399','EPSG','1219');
INSERT INTO "projected_crs" VALUES('EPSG','7780','WGS 84 / Karnataka',NULL,'EPSG','4499','EPSG','4326','EPSG','7747',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5465','projected_crs','EPSG','7780','EPSG','4405','EPSG','1219');
INSERT INTO "projected_crs" VALUES('EPSG','7781','WGS 84 / Kerala',NULL,'EPSG','4499','EPSG','4326','EPSG','7748',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5466','projected_crs','EPSG','7781','EPSG','4406','EPSG','1219');
INSERT INTO "projected_crs" VALUES('EPSG','7782','WGS 84 / Lakshadweep',NULL,'EPSG','4499','EPSG','4326','EPSG','7749',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5467','projected_crs','EPSG','7782','EPSG','4424','EPSG','1219');
INSERT INTO "projected_crs" VALUES('EPSG','7783','WGS 84 / Mizoram',NULL,'EPSG','4499','EPSG','4326','EPSG','7750',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5468','projected_crs','EPSG','7783','EPSG','4411','EPSG','1219');
INSERT INTO "projected_crs" VALUES('EPSG','7784','WGS 84 / Sikkim',NULL,'EPSG','4499','EPSG','4326','EPSG','7751',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5469','projected_crs','EPSG','7784','EPSG','4416','EPSG','1219');
INSERT INTO "projected_crs" VALUES('EPSG','7785','WGS 84 / Tamil Nadu',NULL,'EPSG','4499','EPSG','4326','EPSG','7752',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5470','projected_crs','EPSG','7785','EPSG','4417','EPSG','1219');
INSERT INTO "projected_crs" VALUES('EPSG','7786','WGS 84 / Tripura',NULL,'EPSG','4499','EPSG','4326','EPSG','7753',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5471','projected_crs','EPSG','7786','EPSG','4418','EPSG','1219');
INSERT INTO "projected_crs" VALUES('EPSG','7787','WGS 84 / West Bengal',NULL,'EPSG','4499','EPSG','4326','EPSG','7754',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5472','projected_crs','EPSG','7787','EPSG','4421','EPSG','1219');
INSERT INTO "projected_crs" VALUES('EPSG','7791','RDN2008 / UTM zone 32N',NULL,'EPSG','4400','EPSG','6706','EPSG','16032',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5474','projected_crs','EPSG','7791','EPSG','1718','EPSG','1189');
INSERT INTO "projected_crs" VALUES('EPSG','7792','RDN2008 / UTM zone 33N',NULL,'EPSG','4400','EPSG','6706','EPSG','16033',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5475','projected_crs','EPSG','7792','EPSG','4186','EPSG','1189');
INSERT INTO "projected_crs" VALUES('EPSG','7793','RDN2008 / UTM zone 34N',NULL,'EPSG','4400','EPSG','6706','EPSG','16034',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5476','projected_crs','EPSG','7793','EPSG','4187','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','7794','RDN2008 / Italy zone (E-N)',NULL,'EPSG','4400','EPSG','6706','EPSG','6877',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5477','projected_crs','EPSG','7794','EPSG','1127','EPSG','1241');
INSERT INTO "projected_crs" VALUES('EPSG','7795','RDN2008 / Zone 12 (E-N)',NULL,'EPSG','4400','EPSG','6706','EPSG','6878',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5478','projected_crs','EPSG','7795','EPSG','1127','EPSG','1189');
INSERT INTO "projected_crs" VALUES('EPSG','7799','BGS2005 / UTM zone 34N (N-E)',NULL,'EPSG','4531','EPSG','7798','EPSG','16034',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5482','projected_crs','EPSG','7799','EPSG','4428','EPSG','1210');
INSERT INTO "projected_crs" VALUES('EPSG','7800','BGS2005 / UTM zone 35N (N-E)',NULL,'EPSG','4531','EPSG','7798','EPSG','16035',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5483','projected_crs','EPSG','7800','EPSG','4427','EPSG','1210');
INSERT INTO "projected_crs" VALUES('EPSG','7801','BGS2005 / CCS2005',NULL,'EPSG','4531','EPSG','7798','EPSG','7802',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5484','projected_crs','EPSG','7801','EPSG','3224','EPSG','1061');
INSERT INTO "projected_crs" VALUES('EPSG','7803','BGS2005 / UTM zone 34N',NULL,'EPSG','4400','EPSG','7798','EPSG','16034',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5485','projected_crs','EPSG','7803','EPSG','4428','EPSG','1153');
INSERT INTO "projected_crs" VALUES('EPSG','7804','BGS2005 / UTM zone 35N',NULL,'EPSG','4400','EPSG','7798','EPSG','16034',NULL,1);
INSERT INTO "usage" VALUES('EPSG','5486','projected_crs','EPSG','7804','EPSG','4427','EPSG','1153');
INSERT INTO "projected_crs" VALUES('EPSG','7805','BGS2005 / UTM zone 36N',NULL,'EPSG','4400','EPSG','7798','EPSG','16036',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5487','projected_crs','EPSG','7805','EPSG','4426','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','7825','Pulkovo 1942 / CS63 zone X1',NULL,'EPSG','4530','EPSG','4284','EPSG','7818',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5490','projected_crs','EPSG','7825','EPSG','4435','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','7826','Pulkovo 1942 / CS63 zone X2',NULL,'EPSG','4530','EPSG','4284','EPSG','7819',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5491','projected_crs','EPSG','7826','EPSG','4429','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','7827','Pulkovo 1942 / CS63 zone X3',NULL,'EPSG','4530','EPSG','4284','EPSG','7820',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5492','projected_crs','EPSG','7827','EPSG','4430','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','7828','Pulkovo 1942 / CS63 zone X4',NULL,'EPSG','4530','EPSG','4284','EPSG','7821',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5493','projected_crs','EPSG','7828','EPSG','4431','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','7829','Pulkovo 1942 / CS63 zone X5',NULL,'EPSG','4530','EPSG','4284','EPSG','7822',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5494','projected_crs','EPSG','7829','EPSG','4432','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','7830','Pulkovo 1942 / CS63 zone X6',NULL,'EPSG','4530','EPSG','4284','EPSG','7823',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5495','projected_crs','EPSG','7830','EPSG','4433','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','7831','Pulkovo 1942 / CS63 zone X7',NULL,'EPSG','4530','EPSG','4284','EPSG','7824',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5496','projected_crs','EPSG','7831','EPSG','4434','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','7845','GDA2020 / GA LCC',NULL,'EPSG','4400','EPSG','7844','EPSG','17362',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5504','projected_crs','EPSG','7845','EPSG','2575','EPSG','1236');
INSERT INTO "projected_crs" VALUES('EPSG','7846','GDA2020 / MGA zone 46',NULL,'EPSG','4400','EPSG','7844','EPSG','6729',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5505','projected_crs','EPSG','7846','EPSG','4189','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','7847','GDA2020 / MGA zone 47',NULL,'EPSG','4400','EPSG','7844','EPSG','6730',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5506','projected_crs','EPSG','7847','EPSG','4190','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','7848','GDA2020 / MGA zone 48',NULL,'EPSG','4400','EPSG','7844','EPSG','17348',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5507','projected_crs','EPSG','7848','EPSG','4191','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','7849','GDA2020 / MGA zone 49',NULL,'EPSG','4400','EPSG','7844','EPSG','17349',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5508','projected_crs','EPSG','7849','EPSG','4176','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','7850','GDA2020 / MGA zone 50',NULL,'EPSG','4400','EPSG','7844','EPSG','17350',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5509','projected_crs','EPSG','7850','EPSG','4178','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','7851','GDA2020 / MGA zone 51',NULL,'EPSG','4400','EPSG','7844','EPSG','17351',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5510','projected_crs','EPSG','7851','EPSG','1559','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','7852','GDA2020 / MGA zone 52',NULL,'EPSG','4400','EPSG','7844','EPSG','17352',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5511','projected_crs','EPSG','7852','EPSG','1560','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','7853','GDA2020 / MGA zone 53',NULL,'EPSG','4400','EPSG','7844','EPSG','17353',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5512','projected_crs','EPSG','7853','EPSG','1561','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','7854','GDA2020 / MGA zone 54',NULL,'EPSG','4400','EPSG','7844','EPSG','17354',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5513','projected_crs','EPSG','7854','EPSG','1562','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','7855','GDA2020 / MGA zone 55',NULL,'EPSG','4400','EPSG','7844','EPSG','17355',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5514','projected_crs','EPSG','7855','EPSG','1563','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','7856','GDA2020 / MGA zone 56',NULL,'EPSG','4400','EPSG','7844','EPSG','17356',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5515','projected_crs','EPSG','7856','EPSG','1564','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','7857','GDA2020 / MGA zone 57',NULL,'EPSG','4400','EPSG','7844','EPSG','17357',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5516','projected_crs','EPSG','7857','EPSG','4196','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','7858','GDA2020 / MGA zone 58',NULL,'EPSG','4400','EPSG','7844','EPSG','17358',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5517','projected_crs','EPSG','7858','EPSG','4175','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','7859','GDA2020 / MGA zone 59',NULL,'EPSG','4400','EPSG','7844','EPSG','6731',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5518','projected_crs','EPSG','7859','EPSG','4179','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','7877','Astro DOS 71 / SHLG71',NULL,'EPSG','4400','EPSG','4710','EPSG','7875',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5519','projected_crs','EPSG','7877','EPSG','3183','EPSG','1153');
INSERT INTO "projected_crs" VALUES('EPSG','7878','Astro DOS 71 / UTM zone 30S',NULL,'EPSG','4400','EPSG','4710','EPSG','16130',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5520','projected_crs','EPSG','7878','EPSG','3183','EPSG','1153');
INSERT INTO "projected_crs" VALUES('EPSG','7882','St. Helena Tritan / SHLG(Tritan)',NULL,'EPSG','4400','EPSG','7881','EPSG','7876',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5524','projected_crs','EPSG','7882','EPSG','3183','EPSG','1146');
INSERT INTO "projected_crs" VALUES('EPSG','7883','St. Helena Tritan / UTM zone 30S',NULL,'EPSG','4400','EPSG','7881','EPSG','16130',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5525','projected_crs','EPSG','7883','EPSG','3183','EPSG','1147');
INSERT INTO "projected_crs" VALUES('EPSG','7887','SHMG2015',NULL,'EPSG','4400','EPSG','7886','EPSG','16130',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5529','projected_crs','EPSG','7887','EPSG','3183','EPSG','1147');
INSERT INTO "projected_crs" VALUES('EPSG','7899','GDA2020 / Vicgrid',NULL,'EPSG','4400','EPSG','7844','EPSG','17361',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5533','projected_crs','EPSG','7899','EPSG','2285','EPSG','1135');
INSERT INTO "projected_crs" VALUES('EPSG','7991','NAD27 / MTM zone 10',NULL,'EPSG','4499','EPSG','4267','EPSG','17710',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5572','projected_crs','EPSG','7991','EPSG','1431','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','7992','Malongo 1987 / UTM zone 33S',NULL,'EPSG','4400','EPSG','4259','EPSG','16133',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5573','projected_crs','EPSG','7992','EPSG','4447','EPSG','1136');
INSERT INTO "projected_crs" VALUES('EPSG','8013','GDA2020 / ALB2020',NULL,'EPSG','4400','EPSG','7844','EPSG','7993',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5574','projected_crs','EPSG','8013','EPSG','4439','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','8014','GDA2020 / BIO2020',NULL,'EPSG','4400','EPSG','7844','EPSG','7994',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5575','projected_crs','EPSG','8014','EPSG','4438','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','8015','GDA2020 / BRO2020',NULL,'EPSG','4400','EPSG','7844','EPSG','7995',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5576','projected_crs','EPSG','8015','EPSG','4441','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','8016','GDA2020 / BCG2020',NULL,'EPSG','4400','EPSG','7844','EPSG','7996',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5577','projected_crs','EPSG','8016','EPSG','4437','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','8017','GDA2020 / CARN2020',NULL,'EPSG','4400','EPSG','7844','EPSG','7997',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5578','projected_crs','EPSG','8017','EPSG','4442','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','8018','GDA2020 / CIG2020',NULL,'EPSG','4400','EPSG','7844','EPSG','7998',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5579','projected_crs','EPSG','8018','EPSG','4169','EPSG','1093');
INSERT INTO "projected_crs" VALUES('EPSG','8019','GDA2020 / CKIG2020',NULL,'EPSG','4400','EPSG','7844','EPSG','7999',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5580','projected_crs','EPSG','8019','EPSG','1069','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','8020','GDA2020 / COL2020',NULL,'EPSG','4400','EPSG','7844','EPSG','8000',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5581','projected_crs','EPSG','8020','EPSG','4443','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','8021','GDA2020 / ESP2020',NULL,'EPSG','4400','EPSG','7844','EPSG','8001',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5582','projected_crs','EPSG','8021','EPSG','4445','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','8022','GDA2020 / EXM2020',NULL,'EPSG','4400','EPSG','7844','EPSG','8002',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5583','projected_crs','EPSG','8022','EPSG','4448','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','8023','GDA2020 / GCG2020',NULL,'EPSG','4400','EPSG','7844','EPSG','8003',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5584','projected_crs','EPSG','8023','EPSG','4449','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','8024','GDA2020 / GOLD2020',NULL,'EPSG','4400','EPSG','7844','EPSG','8004',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5585','projected_crs','EPSG','8024','EPSG','4436','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','8025','GDA2020 / JCG2020',NULL,'EPSG','4400','EPSG','7844','EPSG','8005',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5586','projected_crs','EPSG','8025','EPSG','4440','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','8026','GDA2020 / KALB2020',NULL,'EPSG','4400','EPSG','7844','EPSG','8006',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5587','projected_crs','EPSG','8026','EPSG','4444','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','8027','GDA2020 / KAR2020',NULL,'EPSG','4400','EPSG','7844','EPSG','8007',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5588','projected_crs','EPSG','8027','EPSG','4451','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','8028','GDA2020 / KUN2020',NULL,'EPSG','4400','EPSG','7844','EPSG','8008',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5589','projected_crs','EPSG','8028','EPSG','4452','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','8029','GDA2020 / LCG2020',NULL,'EPSG','4400','EPSG','7844','EPSG','8009',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5590','projected_crs','EPSG','8029','EPSG','4453','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','8030','GDA2020 / MRCG2020',NULL,'EPSG','4400','EPSG','7844','EPSG','8010',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5591','projected_crs','EPSG','8030','EPSG','4457','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','8031','GDA2020 / PCG2020',NULL,'EPSG','4400','EPSG','7844','EPSG','8011',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5592','projected_crs','EPSG','8031','EPSG','4462','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','8032','GDA2020 / PHG2020',NULL,'EPSG','4400','EPSG','7844','EPSG','8012',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5593','projected_crs','EPSG','8032','EPSG','4466','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','8035','WGS 84 / TM Zone 20N (ftUS)',NULL,'EPSG','4497','EPSG','4326','EPSG','8033',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5594','projected_crs','EPSG','8035','EPSG','4467','EPSG','1136');
INSERT INTO "projected_crs" VALUES('EPSG','8036','WGS 84 / TM Zone 21N (ftUS)',NULL,'EPSG','4497','EPSG','4326','EPSG','8034',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5595','projected_crs','EPSG','8036','EPSG','4468','EPSG','1136');
INSERT INTO "projected_crs" VALUES('EPSG','8044','Gusterberg Grid (Ferro)',NULL,'EPSG','6501','EPSG','8042','EPSG','8040',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5598','projected_crs','EPSG','8044','EPSG','4455','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','8045','St. Stephen Grid (Ferro)',NULL,'EPSG','6501','EPSG','8043','EPSG','8041',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5599','projected_crs','EPSG','8045','EPSG','4456','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','8058','GDA2020 / NSW Lambert',NULL,'EPSG','4400','EPSG','7844','EPSG','17364',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5604','projected_crs','EPSG','8058','EPSG','3139','EPSG','1135');
INSERT INTO "projected_crs" VALUES('EPSG','8059','GDA2020 / SA Lambert',NULL,'EPSG','4400','EPSG','7844','EPSG','17359',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5605','projected_crs','EPSG','8059','EPSG','2986','EPSG','1135');
INSERT INTO "projected_crs" VALUES('EPSG','8065','NAD83(2011) / PCCS zone 1 (ft)',NULL,'EPSG','4495','EPSG','6318','EPSG','8061',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5606','projected_crs','EPSG','8065','EPSG','4472','EPSG','1207');
INSERT INTO "projected_crs" VALUES('EPSG','8066','NAD83(2011) / PCCS zone 2 (ft)',NULL,'EPSG','4495','EPSG','6318','EPSG','8062',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5607','projected_crs','EPSG','8066','EPSG','4460','EPSG','1207');
INSERT INTO "projected_crs" VALUES('EPSG','8067','NAD83(2011) / PCCS zone 3 (ft)',NULL,'EPSG','4495','EPSG','6318','EPSG','8063',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5608','projected_crs','EPSG','8067','EPSG','4450','EPSG','1207');
INSERT INTO "projected_crs" VALUES('EPSG','8068','NAD83(2011) / PCCS zone 4 (ft)',NULL,'EPSG','4495','EPSG','6318','EPSG','8064',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5609','projected_crs','EPSG','8068','EPSG','4473','EPSG','1207');
INSERT INTO "projected_crs" VALUES('EPSG','8082','NAD83(CSRS)v6 / MTM NS 2010 zone 4',NULL,'EPSG','4400','EPSG','8252','EPSG','8080',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5610','projected_crs','EPSG','8082','EPSG','1534','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','8083','NAD83(CSRS)v6 / MTM NS 2010 zone 5',NULL,'EPSG','4400','EPSG','8252','EPSG','8081',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5611','projected_crs','EPSG','8083','EPSG','1535','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','8088','ISN2016 / Lambert 2016',NULL,'EPSG','4499','EPSG','8086','EPSG','8087',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5615','projected_crs','EPSG','8088','EPSG','1120','EPSG','1210');
INSERT INTO "projected_crs" VALUES('EPSG','8090','NAD83(HARN) / WISCRS Florence (m)',NULL,'EPSG','4499','EPSG','4152','EPSG','7386',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5617','projected_crs','EPSG','8090','EPSG','4327','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8091','NAD83(HARN) / WISCRS Florence (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','7387',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5618','projected_crs','EPSG','8091','EPSG','4327','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8092','NAD83(HARN) / WISCRS Eau Claire (m)',NULL,'EPSG','4499','EPSG','4152','EPSG','7440',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5619','projected_crs','EPSG','8092','EPSG','4342','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8093','NAD83(HARN) / WISCRS Eau Claire (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','7441',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5620','projected_crs','EPSG','8093','EPSG','4342','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8095','NAD83(HARN) / WISCRS Wood (m)',NULL,'EPSG','4499','EPSG','4152','EPSG','7482',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5621','projected_crs','EPSG','8095','EPSG','4359','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8096','NAD83(HARN) / WISCRS Wood (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','7483',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5622','projected_crs','EPSG','8096','EPSG','4359','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8097','NAD83(HARN) / WISCRS Waushara (m)',NULL,'EPSG','4499','EPSG','4152','EPSG','7526',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5623','projected_crs','EPSG','8097','EPSG','4381','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8098','NAD83(HARN) / WISCRS Waushara (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','7527',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5624','projected_crs','EPSG','8098','EPSG','4381','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8099','NAD83(HARN) / WISCRS Waupaca (m)',NULL,'EPSG','4499','EPSG','4152','EPSG','7480',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5625','projected_crs','EPSG','8099','EPSG','4358','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8100','NAD83(HARN) / WISCRS Waupaca (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','7481',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5626','projected_crs','EPSG','8100','EPSG','4358','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8101','NAD83(HARN) / WISCRS Waukesha (m)',NULL,'EPSG','4499','EPSG','4152','EPSG','7524',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5627','projected_crs','EPSG','8101','EPSG','4380','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8102','NAD83(HARN) / WISCRS Waukesha (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','7525',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5628','projected_crs','EPSG','8102','EPSG','4380','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8103','NAD83(HARN) / WISCRS Washington (m)',NULL,'EPSG','4499','EPSG','4152','EPSG','7522',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5629','projected_crs','EPSG','8103','EPSG','4379','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8104','NAD83(HARN) / WISCRS Washington (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','7523',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5630','projected_crs','EPSG','8104','EPSG','4379','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8105','NAD83(HARN) / WISCRS Washburn (m)',NULL,'EPSG','4499','EPSG','4152','EPSG','7424',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5631','projected_crs','EPSG','8105','EPSG','4335','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8106','NAD83(HARN) / WISCRS Washburn (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','7425',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5632','projected_crs','EPSG','8106','EPSG','4335','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8107','NAD83(HARN) / WISCRS Walworth (m)',NULL,'EPSG','4499','EPSG','4152','EPSG','7520',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5633','projected_crs','EPSG','8107','EPSG','4378','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8108','NAD83(HARN) / WISCRS Walworth (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','7521',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5634','projected_crs','EPSG','8108','EPSG','4378','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8109','NAD83(HARN) / WISCRS Vilas (m)',NULL,'EPSG','4499','EPSG','4152','EPSG','7398',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5635','projected_crs','EPSG','8109','EPSG','4334','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8110','NAD83(HARN) / WISCRS Vilas (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','7399',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5636','projected_crs','EPSG','8110','EPSG','4334','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8111','NAD83(HARN) / WISCRS Vernon (m)',NULL,'EPSG','4499','EPSG','4152','EPSG','7518',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5637','projected_crs','EPSG','8111','EPSG','4377','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8112','NAD83(HARN) / WISCRS Vernon (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','7519',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5638','projected_crs','EPSG','8112','EPSG','4377','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8113','NAD83(HARN) / WISCRS Trempealeau (m)',NULL,'EPSG','4499','EPSG','4152','EPSG','7478',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5639','projected_crs','EPSG','8113','EPSG','4357','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8114','NAD83(HARN) / WISCRS Trempealeau (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','7479',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5640','projected_crs','EPSG','8114','EPSG','4357','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8115','NAD83(HARN) / WISCRS Taylor (m)',NULL,'EPSG','4499','EPSG','4152','EPSG','7476',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5641','projected_crs','EPSG','8115','EPSG','4356','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8116','NAD83(HARN) / WISCRS Taylor (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','7477',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5642','projected_crs','EPSG','8116','EPSG','4356','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8117','NAD83(HARN) / WISCRS St. Croix (m)',NULL,'EPSG','4499','EPSG','4152','EPSG','7474',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5643','projected_crs','EPSG','8117','EPSG','4355','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8118','NAD83(HARN) / WISCRS St. Croix (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','7475',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5644','projected_crs','EPSG','8118','EPSG','4355','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8119','NAD83(HARN) / WISCRS Shawano (m)',NULL,'EPSG','4499','EPSG','4152','EPSG','7472',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5645','projected_crs','EPSG','8119','EPSG','4354','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8120','NAD83(HARN) / WISCRS Shawano (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','7473',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5646','projected_crs','EPSG','8120','EPSG','4354','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8121','NAD83(HARN) / WISCRS Sawyer (m)',NULL,'EPSG','4499','EPSG','4152','EPSG','7396',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5647','projected_crs','EPSG','8121','EPSG','4333','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8122','NAD83(HARN) / WISCRS Sawyer (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','7397',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5648','projected_crs','EPSG','8122','EPSG','4333','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8123','NAD83(HARN) / WISCRS Sauk (m)',NULL,'EPSG','4499','EPSG','4152','EPSG','7516',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5649','projected_crs','EPSG','8123','EPSG','4376','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8124','NAD83(HARN) / WISCRS Sauk (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','7517',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5650','projected_crs','EPSG','8124','EPSG','4376','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8125','NAD83(HARN) / WISCRS Rusk (m)',NULL,'EPSG','4499','EPSG','4152','EPSG','7470',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5651','projected_crs','EPSG','8125','EPSG','4353','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8126','NAD83(HARN) / WISCRS Rusk (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','7471',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5652','projected_crs','EPSG','8126','EPSG','4353','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8127','NAD83(HARN) / WISCRS Rock (m)',NULL,'EPSG','4499','EPSG','4152','EPSG','7514',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5653','projected_crs','EPSG','8127','EPSG','4375','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8128','NAD83(HARN) / WISCRS Rock (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','7515',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5654','projected_crs','EPSG','8128','EPSG','4375','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8129','NAD83(HARN) / WISCRS Richland (m)',NULL,'EPSG','4499','EPSG','4152','EPSG','7512',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5655','projected_crs','EPSG','8129','EPSG','4374','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8130','NAD83(HARN) / WISCRS Richland (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','7513',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5656','projected_crs','EPSG','8130','EPSG','4374','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8131','NAD83(HARN) / WISCRS Price (m)',NULL,'EPSG','4499','EPSG','4152','EPSG','7394',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5657','projected_crs','EPSG','8131','EPSG','4332','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8132','NAD83(HARN) / WISCRS Price (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','7395',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5658','projected_crs','EPSG','8132','EPSG','4332','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8133','NAD83(HARN) / WISCRS Portage (m)',NULL,'EPSG','4499','EPSG','4152','EPSG','7468',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5659','projected_crs','EPSG','8133','EPSG','4352','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8134','NAD83(HARN) / WISCRS Portage (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','7469',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5660','projected_crs','EPSG','8134','EPSG','4352','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8135','NAD83(HARN) / WISCRS Polk (m)',NULL,'EPSG','4499','EPSG','4152','EPSG','7466',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5661','projected_crs','EPSG','8135','EPSG','4351','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8136','NAD83(HARN) / WISCRS Polk (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','7467',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5662','projected_crs','EPSG','8136','EPSG','4351','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8137','NAD83(HARN) / WISCRS Pepin and Pierce (m)',NULL,'EPSG','4499','EPSG','4152','EPSG','7464',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5663','projected_crs','EPSG','8137','EPSG','4350','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8138','NAD83(HARN) / WISCRS Pepin and Pierce (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','7465',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5664','projected_crs','EPSG','8138','EPSG','4350','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8139','NAD83(HARN) / WISCRS Oneida (m)',NULL,'EPSG','4499','EPSG','4152','EPSG','7392',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5665','projected_crs','EPSG','8139','EPSG','4330','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8140','NAD83(HARN) / WISCRS Oneida (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','7393',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5666','projected_crs','EPSG','8140','EPSG','4330','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8141','NAD83(HARN) / WISCRS Oconto (m)',NULL,'EPSG','4499','EPSG','4152','EPSG','7462',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5667','projected_crs','EPSG','8141','EPSG','4349','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8142','NAD83(HARN) / WISCRS Oconto (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','7463',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5668','projected_crs','EPSG','8142','EPSG','4349','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8143','NAD83(HARN) / WISCRS Monroe (m)',NULL,'EPSG','4499','EPSG','4152','EPSG','7510',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5669','projected_crs','EPSG','8143','EPSG','4373','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8144','NAD83(HARN) / WISCRS Monroe (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','7511',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5670','projected_crs','EPSG','8144','EPSG','4373','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8145','NAD83(HARN) / WISCRS Menominee (m)',NULL,'EPSG','4499','EPSG','4152','EPSG','7460',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5671','projected_crs','EPSG','8145','EPSG','4348','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8146','NAD83(HARN) / WISCRS Menominee (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','7461',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5672','projected_crs','EPSG','8146','EPSG','4348','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8147','NAD83(HARN) / WISCRS Marinette (m)',NULL,'EPSG','4499','EPSG','4152','EPSG','7458',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5673','projected_crs','EPSG','8147','EPSG','4347','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8148','NAD83(HARN) / WISCRS Marinette (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','7459',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5674','projected_crs','EPSG','8148','EPSG','4347','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8149','NAD83(HARN) / WISCRS Marathon (m)',NULL,'EPSG','4499','EPSG','4152','EPSG','7456',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5675','projected_crs','EPSG','8149','EPSG','4346','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8150','NAD83(HARN) / WISCRS Marathon (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','7457',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5676','projected_crs','EPSG','8150','EPSG','4346','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8151','NAD83(HARN) / WISCRS Lincoln (m)',NULL,'EPSG','4499','EPSG','4152','EPSG','7454',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5677','projected_crs','EPSG','8151','EPSG','4345','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8152','NAD83(HARN) / WISCRS Lincoln (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','7455',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5678','projected_crs','EPSG','8152','EPSG','4345','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8153','NAD83(HARN) / WISCRS Langlade (m)',NULL,'EPSG','4499','EPSG','4152','EPSG','7452',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5679','projected_crs','EPSG','8153','EPSG','4344','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8154','NAD83(HARN) / WISCRS Langlade (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','7453',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5680','projected_crs','EPSG','8154','EPSG','4344','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8155','NAD83(HARN) / WISCRS La Crosse (m)',NULL,'EPSG','4499','EPSG','4152','EPSG','7508',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5681','projected_crs','EPSG','8155','EPSG','4372','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8156','NAD83(HARN) / WISCRS La Crosse (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','7509',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5682','projected_crs','EPSG','8156','EPSG','4372','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8157','NAD83(HARN) / WISCRS Kewaunee, Manitowoc and Sheboygan (m)',NULL,'EPSG','4499','EPSG','4152','EPSG','7506',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5683','projected_crs','EPSG','8157','EPSG','4371','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8158','NAD83(HARN) / WISCRS Kewaunee, Manitowoc and Sheboygan (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','7507',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5684','projected_crs','EPSG','8158','EPSG','4371','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8159','NAD83(HARN) / WISCRS Kenosha, Milwaukee, Ozaukee and Racine (m)',NULL,'EPSG','4499','EPSG','4152','EPSG','7504',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5685','projected_crs','EPSG','8159','EPSG','4370','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8160','NAD83(HARN) / WISCRS Kenosha, Milwaukee, Ozaukee and Racine (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','7505',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5686','projected_crs','EPSG','8160','EPSG','4370','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8161','NAD83(HARN) / WISCRS Jackson (m)',NULL,'EPSG','4499','EPSG','4152','EPSG','7450',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5687','projected_crs','EPSG','8161','EPSG','4343','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8162','NAD83(HARN) / WISCRS Jackson (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','7451',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5688','projected_crs','EPSG','8162','EPSG','4343','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8163','NAD83(HARN) / WISCRS Iron (m)',NULL,'EPSG','4499','EPSG','4152','EPSG','7390',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5689','projected_crs','EPSG','8163','EPSG','4329','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8164','NAD83(HARN) / WISCRS Iron (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','7391',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5690','projected_crs','EPSG','8164','EPSG','4329','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8165','NAD83(HARN) / WISCRS Iowa (m)',NULL,'EPSG','4499','EPSG','4152','EPSG','7502',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5691','projected_crs','EPSG','8165','EPSG','4369','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8166','NAD83(HARN) / WISCRS Iowa (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','7503',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5692','projected_crs','EPSG','8166','EPSG','4369','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8167','NAD83(HARN) / WISCRS Green Lake and Marquette (m)',NULL,'EPSG','4499','EPSG','4152','EPSG','7500',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5693','projected_crs','EPSG','8167','EPSG','4368','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8168','NAD83(HARN) / WISCRS Green Lake and Marquette (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','7501',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5694','projected_crs','EPSG','8168','EPSG','4368','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8169','NAD83(HARN) / WISCRS Green and Lafayette (m)',NULL,'EPSG','4499','EPSG','4152','EPSG','7498',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5695','projected_crs','EPSG','8169','EPSG','4367','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8170','NAD83(HARN) / WISCRS Green and Lafayette (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','7499',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5696','projected_crs','EPSG','8170','EPSG','4367','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8171','NAD83(HARN) / WISCRS Grant (m)',NULL,'EPSG','4499','EPSG','4152','EPSG','7496',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5697','projected_crs','EPSG','8171','EPSG','4366','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8172','NAD83(HARN) / WISCRS Grant (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','7497',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5698','projected_crs','EPSG','8172','EPSG','4366','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8173','NAD83(HARN) / WISCRS Forest (m)',NULL,'EPSG','4499','EPSG','4152','EPSG','7388',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5699','projected_crs','EPSG','8173','EPSG','4328','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8177','NAD83(HARN) / WISCRS Forest (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','7389',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5700','projected_crs','EPSG','8177','EPSG','4328','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8179','NAD83(HARN) / WISCRS Dunn (m)',NULL,'EPSG','4499','EPSG','4152','EPSG','7438',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5701','projected_crs','EPSG','8179','EPSG','4341','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8180','NAD83(HARN) / WISCRS Dunn (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','7439',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5702','projected_crs','EPSG','8180','EPSG','4341','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8181','NAD83(HARN) / WISCRS Douglas (m)',NULL,'EPSG','4499','EPSG','4152','EPSG','7384',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5703','projected_crs','EPSG','8181','EPSG','4326','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8182','NAD83(HARN) / WISCRS Douglas (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','7385',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5704','projected_crs','EPSG','8182','EPSG','4326','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8184','NAD83(HARN) / WISCRS Door (m)',NULL,'EPSG','4499','EPSG','4152','EPSG','7436',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5705','projected_crs','EPSG','8184','EPSG','4340','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8185','NAD83(HARN) / WISCRS Door (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','7437',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5706','projected_crs','EPSG','8185','EPSG','4340','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8187','NAD83(HARN) / WISCRS Dodge and Jefferson (m)',NULL,'EPSG','4499','EPSG','4152','EPSG','7494',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5707','projected_crs','EPSG','8187','EPSG','4365','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8189','NAD83(HARN) / WISCRS Dodge and Jefferson (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','7495',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5708','projected_crs','EPSG','8189','EPSG','4365','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8191','NAD83(HARN) / WISCRS Dane (m)',NULL,'EPSG','4499','EPSG','4152','EPSG','7492',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5709','projected_crs','EPSG','8191','EPSG','4364','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8193','NAD83(HARN) / WISCRS Dane (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','7493',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5710','projected_crs','EPSG','8193','EPSG','4364','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8196','NAD83(HARN) / WISCRS Crawford (m)',NULL,'EPSG','4499','EPSG','4152','EPSG','7490',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5711','projected_crs','EPSG','8196','EPSG','4363','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8197','NAD83(HARN) / WISCRS Crawford (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','7491',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5712','projected_crs','EPSG','8197','EPSG','4363','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8198','NAD83(HARN) / WISCRS Columbia (m)',NULL,'EPSG','4499','EPSG','4152','EPSG','7488',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5713','projected_crs','EPSG','8198','EPSG','4362','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8200','NAD83(HARN) / WISCRS Columbia (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','7489',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5714','projected_crs','EPSG','8200','EPSG','4362','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8201','NAD83(HARN) / WISCRS Clark (m)',NULL,'EPSG','4499','EPSG','4152','EPSG','7434',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5715','projected_crs','EPSG','8201','EPSG','4339','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8202','NAD83(HARN) / WISCRS Clark (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','7435',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5716','projected_crs','EPSG','8202','EPSG','4339','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8203','NAD83(HARN) / WISCRS Chippewa (m)',NULL,'EPSG','4499','EPSG','4152','EPSG','7432',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5717','projected_crs','EPSG','8203','EPSG','4338','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8204','NAD83(HARN) / WISCRS Chippewa (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','7433',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5718','projected_crs','EPSG','8204','EPSG','4338','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8205','NAD83(HARN) / WISCRS Calumet, Fond du Lac, Outagamie and Winnebago (m)',NULL,'EPSG','4499','EPSG','4152','EPSG','7486',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5719','projected_crs','EPSG','8205','EPSG','4361','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8206','NAD83(HARN) / WISCRS Calumet, Fond du Lac, Outagamie and Winnebago (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','7487',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5720','projected_crs','EPSG','8206','EPSG','4361','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8207','NAD83(HARN) / WISCRS Burnett (m)',NULL,'EPSG','4499','EPSG','4152','EPSG','7382',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5721','projected_crs','EPSG','8207','EPSG','4325','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8208','NAD83(HARN) / WISCRS Burnett (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','7383',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5722','projected_crs','EPSG','8208','EPSG','4325','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8209','NAD83(HARN) / WISCRS Buffalo (m)',NULL,'EPSG','4499','EPSG','4152','EPSG','7430',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5723','projected_crs','EPSG','8209','EPSG','4337','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8210','NAD83(HARN) / WISCRS Buffalo (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','7431',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5724','projected_crs','EPSG','8210','EPSG','4337','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8212','NAD83(HARN) / WISCRS Brown (m)',NULL,'EPSG','4499','EPSG','4152','EPSG','7428',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5725','projected_crs','EPSG','8212','EPSG','4336','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8213','NAD83(HARN) / WISCRS Brown (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','7429',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5726','projected_crs','EPSG','8213','EPSG','4336','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8214','NAD83(HARN) / WISCRS Bayfield (m)',NULL,'EPSG','4499','EPSG','4152','EPSG','7380',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5727','projected_crs','EPSG','8214','EPSG','4321','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8216','NAD83(HARN) / WISCRS Bayfield (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','7381',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5728','projected_crs','EPSG','8216','EPSG','4321','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8218','NAD83(HARN) / WISCRS Barron (m)',NULL,'EPSG','4499','EPSG','4152','EPSG','7426',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5729','projected_crs','EPSG','8218','EPSG','4331','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8220','NAD83(HARN) / WISCRS Barron (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','7427',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5730','projected_crs','EPSG','8220','EPSG','4331','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8222','NAD83(HARN) / WISCRS Ashland (m)',NULL,'EPSG','4499','EPSG','4152','EPSG','7378',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5731','projected_crs','EPSG','8222','EPSG','4320','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8224','NAD83(HARN) / WISCRS Ashland (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','7379',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5732','projected_crs','EPSG','8224','EPSG','4320','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8225','NAD83(HARN) / WISCRS Adams and Juneau (m)',NULL,'EPSG','4499','EPSG','4152','EPSG','7484',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5733','projected_crs','EPSG','8225','EPSG','4360','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8226','NAD83(HARN) / WISCRS Adams and Juneau (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','7485',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5734','projected_crs','EPSG','8226','EPSG','4360','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8311','NAD83(2011) / Oregon Burns-Harper zone (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','8273',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5760','projected_crs','EPSG','8311','EPSG','4459','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8312','NAD83(2011) / Oregon Burns-Harper zone (ft)',NULL,'EPSG','4495','EPSG','6318','EPSG','8274',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5761','projected_crs','EPSG','8312','EPSG','4459','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8313','NAD83(2011) / Oregon Canyon City-Burns zone (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','8275',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5762','projected_crs','EPSG','8313','EPSG','4465','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8314','NAD83(2011) / Oregon Canyon City-Burns zone (ft)',NULL,'EPSG','4495','EPSG','6318','EPSG','8276',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5763','projected_crs','EPSG','8314','EPSG','4465','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8315','NAD83(2011) / Oregon Coast Range North zone (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','8277',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5764','projected_crs','EPSG','8315','EPSG','4471','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8316','NAD83(2011) / Oregon Coast Range North zone (ft)',NULL,'EPSG','4495','EPSG','6318','EPSG','8278',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5765','projected_crs','EPSG','8316','EPSG','4471','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8317','NAD83(2011) / Oregon Dayville-Prairie City zone (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','8279',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5766','projected_crs','EPSG','8317','EPSG','4474','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8318','NAD83(2011) / Oregon Dayville-Prairie City zone (ft)',NULL,'EPSG','4495','EPSG','6318','EPSG','8280',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5767','projected_crs','EPSG','8318','EPSG','4474','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8319','NAD83(2011) / Oregon Denio-Burns zone (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','8281',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5768','projected_crs','EPSG','8319','EPSG','4475','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8320','NAD83(2011) / Oregon Denio-Burns zone (ft)',NULL,'EPSG','4495','EPSG','6318','EPSG','8282',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5769','projected_crs','EPSG','8320','EPSG','4475','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8321','NAD83(2011) / Oregon Halfway zone (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','8283',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5770','projected_crs','EPSG','8321','EPSG','4476','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8322','NAD83(2011) / Oregon Halfway zone (ft)',NULL,'EPSG','4495','EPSG','6318','EPSG','8284',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5771','projected_crs','EPSG','8322','EPSG','4476','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8323','NAD83(2011) / Oregon Medford-Diamond Lake zone (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','8285',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5772','projected_crs','EPSG','8323','EPSG','4477','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8324','NAD83(2011) / Oregon Medford-Diamond Lake zone (ft)',NULL,'EPSG','4495','EPSG','6318','EPSG','8286',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5773','projected_crs','EPSG','8324','EPSG','4477','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8325','NAD83(2011) / Oregon Mitchell zone (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','8287',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5774','projected_crs','EPSG','8325','EPSG','4478','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8326','NAD83(2011) / Oregon Mitchell zone (ft)',NULL,'EPSG','4495','EPSG','6318','EPSG','8288',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5775','projected_crs','EPSG','8326','EPSG','4478','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8327','NAD83(2011) / Oregon North Central zone (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','8289',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5776','projected_crs','EPSG','8327','EPSG','4479','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8328','NAD83(2011) / Oregon North Central zone (ft)',NULL,'EPSG','4495','EPSG','6318','EPSG','8290',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5777','projected_crs','EPSG','8328','EPSG','4479','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8329','NAD83(2011) / Oregon Ochoco Summit zone (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','8291',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5778','projected_crs','EPSG','8329','EPSG','4481','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8330','NAD83(2011) / Oregon Ochoco Summit zone (ft)',NULL,'EPSG','4495','EPSG','6318','EPSG','8292',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5779','projected_crs','EPSG','8330','EPSG','4481','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8331','NAD83(2011) / Oregon Owyhee zone (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','8293',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5780','projected_crs','EPSG','8331','EPSG','4482','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8332','NAD83(2011) / Oregon Owyhee zone (ft)',NULL,'EPSG','4495','EPSG','6318','EPSG','8294',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5781','projected_crs','EPSG','8332','EPSG','4482','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8333','NAD83(2011) / Oregon Pilot Rock-Ukiah zone (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','8295',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5782','projected_crs','EPSG','8333','EPSG','4483','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8334','NAD83(2011) / Oregon Pilot Rock-Ukiah zone (ft)',NULL,'EPSG','4495','EPSG','6318','EPSG','8296',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5783','projected_crs','EPSG','8334','EPSG','4483','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8335','NAD83(2011) / Oregon Prairie City-Brogan zone (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','8297',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5784','projected_crs','EPSG','8335','EPSG','4484','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8336','NAD83(2011) / Oregon Prairie City-Brogan zone (ft)',NULL,'EPSG','4495','EPSG','6318','EPSG','8298',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5785','projected_crs','EPSG','8336','EPSG','4484','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8337','NAD83(2011) / Oregon Riley-Lakeview zone (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','8299',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5786','projected_crs','EPSG','8337','EPSG','4458','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8338','NAD83(2011) / Oregon Riley-Lakeview zone (ft)',NULL,'EPSG','4495','EPSG','6318','EPSG','8300',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5787','projected_crs','EPSG','8338','EPSG','4458','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8339','NAD83(2011) / Oregon Siskiyou Pass zone (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','8301',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5788','projected_crs','EPSG','8339','EPSG','4463','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8340','NAD83(2011) / Oregon Siskiyou Pass zone (ft)',NULL,'EPSG','4495','EPSG','6318','EPSG','8302',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5789','projected_crs','EPSG','8340','EPSG','4463','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8341','NAD83(2011) / Oregon Ukiah-Fox zone (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','8303',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5790','projected_crs','EPSG','8341','EPSG','4470','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8342','NAD83(2011) / Oregon Ukiah-Fox zone (ft)',NULL,'EPSG','4495','EPSG','6318','EPSG','8304',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5791','projected_crs','EPSG','8342','EPSG','4470','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8343','NAD83(2011) / Oregon Wallowa zone (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','8305',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5792','projected_crs','EPSG','8343','EPSG','4480','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8344','NAD83(2011) / Oregon Wallowa zone (ft)',NULL,'EPSG','4495','EPSG','6318','EPSG','8306',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5793','projected_crs','EPSG','8344','EPSG','4480','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8345','NAD83(2011) / Oregon Warner Highway zone (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','8307',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5794','projected_crs','EPSG','8345','EPSG','4486','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8346','NAD83(2011) / Oregon Warner Highway zone (ft)',NULL,'EPSG','4495','EPSG','6318','EPSG','8308',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5795','projected_crs','EPSG','8346','EPSG','4486','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8347','NAD83(2011) / Oregon Willamette Pass zone (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','8309',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5796','projected_crs','EPSG','8347','EPSG','4488','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8348','NAD83(2011) / Oregon Willamette Pass zone (ft)',NULL,'EPSG','4495','EPSG','6318','EPSG','8310',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5797','projected_crs','EPSG','8348','EPSG','4488','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8352','S-JTSK [JTSK03] / Krovak',NULL,'EPSG','6501','EPSG','8351','EPSG','5509',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5801','projected_crs','EPSG','8352','EPSG','1211','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','8353','S-JTSK [JTSK03] / Krovak East North',NULL,'EPSG','4499','EPSG','8351','EPSG','5510',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5802','projected_crs','EPSG','8353','EPSG','1211','EPSG','1189');
INSERT INTO "projected_crs" VALUES('EPSG','8379','NAD83 / NCRS Las Vegas (m)',NULL,'EPSG','4499','EPSG','4269','EPSG','8373',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5809','projected_crs','EPSG','8379','EPSG','4485','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8380','NAD83 / NCRS Las Vegas (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','8374',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5810','projected_crs','EPSG','8380','EPSG','4485','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8381','NAD83 / NCRS Las Vegas high (m)',NULL,'EPSG','4499','EPSG','4269','EPSG','8375',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5811','projected_crs','EPSG','8381','EPSG','4487','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8382','NAD83 / NCRS Las Vegas high (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','8376',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5812','projected_crs','EPSG','8382','EPSG','4487','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8383','NAD83(2011) / NCRS Las Vegas (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','8373',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5813','projected_crs','EPSG','8383','EPSG','4485','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8384','NAD83(2011) / NCRS Las Vegas (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','8374',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5814','projected_crs','EPSG','8384','EPSG','4485','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8385','NAD83(2011) / NCRS Las Vegas high (m)',NULL,'EPSG','4499','EPSG','6318','EPSG','8375',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5815','projected_crs','EPSG','8385','EPSG','4487','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8387','NAD83(2011) / NCRS Las Vegas high (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','8376',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5816','projected_crs','EPSG','8387','EPSG','4487','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','8391','GDA94 / WEIPA94',NULL,'EPSG','4400','EPSG','4283','EPSG','8389',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5817','projected_crs','EPSG','8391','EPSG','4491','EPSG','1249');
INSERT INTO "projected_crs" VALUES('EPSG','8395','ETRS89 / Gauss-Kruger CM 9E',NULL,'EPSG','4400','EPSG','4258','EPSG','16302',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5818','projected_crs','EPSG','8395','EPSG','4490','EPSG','1143');
INSERT INTO "projected_crs" VALUES('EPSG','8433','Macao 1920 / Macao Grid',NULL,'EPSG','4500','EPSG','8428','EPSG','8432',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5830','projected_crs','EPSG','8433','EPSG','1147','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','8441','Tananarive / Laborde Grid',NULL,'EPSG','4530','EPSG','4297','EPSG','8440',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5832','projected_crs','EPSG','8441','EPSG','3273','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','8455','RGTAAF07 / UTM zone 53S',NULL,'EPSG','4400','EPSG','7073','EPSG','16153',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5834','projected_crs','EPSG','8455','EPSG','4489','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','8456','RGTAAF07 / UTM zone 54S',NULL,'EPSG','4400','EPSG','7073','EPSG','16154',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5835','projected_crs','EPSG','8456','EPSG','4492','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','8518','NAD83(2011) / KS RCS zone 1',NULL,'EPSG','4497','EPSG','6318','EPSG','8458',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5836','projected_crs','EPSG','8518','EPSG','4495','EPSG','1196');
INSERT INTO "projected_crs" VALUES('EPSG','8519','NAD83(2011) / KS RCS zone 2',NULL,'EPSG','4497','EPSG','6318','EPSG','8459',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5837','projected_crs','EPSG','8519','EPSG','4496','EPSG','1196');
INSERT INTO "projected_crs" VALUES('EPSG','8520','NAD83(2011) / KS RCS zone 3',NULL,'EPSG','4497','EPSG','6318','EPSG','8490',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5838','projected_crs','EPSG','8520','EPSG','4497','EPSG','1196');
INSERT INTO "projected_crs" VALUES('EPSG','8521','NAD83(2011) / KS RCS zone 4',NULL,'EPSG','4497','EPSG','6318','EPSG','8491',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5839','projected_crs','EPSG','8521','EPSG','4494','EPSG','1196');
INSERT INTO "projected_crs" VALUES('EPSG','8522','NAD83(2011) / KS RCS zone 5',NULL,'EPSG','4497','EPSG','6318','EPSG','8492',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5840','projected_crs','EPSG','8522','EPSG','4498','EPSG','1196');
INSERT INTO "projected_crs" VALUES('EPSG','8523','NAD83(2011) / KS RCS zone 6',NULL,'EPSG','4497','EPSG','6318','EPSG','8493',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5841','projected_crs','EPSG','8523','EPSG','4499','EPSG','1196');
INSERT INTO "projected_crs" VALUES('EPSG','8524','NAD83(2011) / KS RCS zone 7',NULL,'EPSG','4497','EPSG','6318','EPSG','8494',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5842','projected_crs','EPSG','8524','EPSG','4500','EPSG','1196');
INSERT INTO "projected_crs" VALUES('EPSG','8525','NAD83(2011) / KS RCS zone 8',NULL,'EPSG','4497','EPSG','6318','EPSG','8495',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5843','projected_crs','EPSG','8525','EPSG','4501','EPSG','1196');
INSERT INTO "projected_crs" VALUES('EPSG','8526','NAD83(2011) / KS RCS zone 9',NULL,'EPSG','4497','EPSG','6318','EPSG','8498',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5844','projected_crs','EPSG','8526','EPSG','4502','EPSG','1196');
INSERT INTO "projected_crs" VALUES('EPSG','8527','NAD83(2011) / KS RCS zone 10',NULL,'EPSG','4497','EPSG','6318','EPSG','8499',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5845','projected_crs','EPSG','8527','EPSG','4503','EPSG','1196');
INSERT INTO "projected_crs" VALUES('EPSG','8528','NAD83(2011) / KS RCS zone 11',NULL,'EPSG','4497','EPSG','6318','EPSG','8500',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5846','projected_crs','EPSG','8528','EPSG','4504','EPSG','1196');
INSERT INTO "projected_crs" VALUES('EPSG','8529','NAD83(2011) / KS RCS zone 12',NULL,'EPSG','4497','EPSG','6318','EPSG','8501',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5847','projected_crs','EPSG','8529','EPSG','4505','EPSG','1196');
INSERT INTO "projected_crs" VALUES('EPSG','8531','NAD83(2011) / KS RCS zone 13',NULL,'EPSG','4497','EPSG','6318','EPSG','8502',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5848','projected_crs','EPSG','8531','EPSG','4506','EPSG','1196');
INSERT INTO "projected_crs" VALUES('EPSG','8533','NAD83(2011) / KS RCS zone 14',NULL,'EPSG','4497','EPSG','6318','EPSG','8503',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5849','projected_crs','EPSG','8533','EPSG','4507','EPSG','1196');
INSERT INTO "projected_crs" VALUES('EPSG','8534','NAD83(2011) / KS RCS zone 15',NULL,'EPSG','4497','EPSG','6318','EPSG','8504',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5850','projected_crs','EPSG','8534','EPSG','4508','EPSG','1196');
INSERT INTO "projected_crs" VALUES('EPSG','8535','NAD83(2011) / KS RCS zone 16',NULL,'EPSG','4497','EPSG','6318','EPSG','8505',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5851','projected_crs','EPSG','8535','EPSG','4509','EPSG','1196');
INSERT INTO "projected_crs" VALUES('EPSG','8536','NAD83(2011) / KS RCS zone 17',NULL,'EPSG','4497','EPSG','6318','EPSG','8506',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5852','projected_crs','EPSG','8536','EPSG','4510','EPSG','1196');
INSERT INTO "projected_crs" VALUES('EPSG','8538','NAD83(2011) / KS RCS zone 18',NULL,'EPSG','4497','EPSG','6318','EPSG','8507',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5853','projected_crs','EPSG','8538','EPSG','4511','EPSG','1196');
INSERT INTO "projected_crs" VALUES('EPSG','8539','NAD83(2011) / KS RCS zone 19',NULL,'EPSG','4497','EPSG','6318','EPSG','8515',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5854','projected_crs','EPSG','8539','EPSG','4512','EPSG','1196');
INSERT INTO "projected_crs" VALUES('EPSG','8540','NAD83(2011) / KS RCS zone 20',NULL,'EPSG','4497','EPSG','6318','EPSG','8516',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5855','projected_crs','EPSG','8540','EPSG','4513','EPSG','1196');
INSERT INTO "projected_crs" VALUES('EPSG','8677','MGI 1901 / Balkans zone 5',NULL,'EPSG','4498','EPSG','3906','EPSG','18275',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5862','projected_crs','EPSG','8677','EPSG','1709','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','8678','MGI 1901 / Balkans zone 6',NULL,'EPSG','4498','EPSG','3906','EPSG','18276',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5863','projected_crs','EPSG','8678','EPSG','1710','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','8679','MGI 1901 / Balkans zone 8',NULL,'EPSG','4498','EPSG','3906','EPSG','18278',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5864','projected_crs','EPSG','8679','EPSG','1712','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','8682','SRB_ETRS89 / UTM zone 34N',NULL,'EPSG','4400','EPSG','8685','EPSG','16034',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5865','projected_crs','EPSG','8682','EPSG','4543','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','8686','MGI 1901 / Slovenia Grid',NULL,'EPSG','4498','EPSG','3906','EPSG','19967',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5869','projected_crs','EPSG','8686','EPSG','1212','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','8687','Slovenia 1996 / UTM zone 33N',NULL,'EPSG','4400','EPSG','4765','EPSG','16033',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5870','projected_crs','EPSG','8687','EPSG','1212','EPSG','1160');
INSERT INTO "projected_crs" VALUES('EPSG','8692','NAD83(MA11) / UTM zone 54N',NULL,'EPSG','4400','EPSG','6325','EPSG','16054',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5873','projected_crs','EPSG','8692','EPSG','4514','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','8693','NAD83(MA11) / UTM zone 55N',NULL,'EPSG','4400','EPSG','6325','EPSG','16055',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5874','projected_crs','EPSG','8693','EPSG','4518','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','8826','NAD83 / Idaho Transverse Mercator',NULL,'EPSG','4499','EPSG','4269','EPSG','8825',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6000','projected_crs','EPSG','8826','EPSG','1381','EPSG','1135');
INSERT INTO "projected_crs" VALUES('EPSG','8836','MTRF-2000 / UTM zone 36N',NULL,'EPSG','4400','EPSG','8818','EPSG','16036',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6001','projected_crs','EPSG','8836','EPSG','4524','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','8837','MTRF-2000 / UTM zone 37N',NULL,'EPSG','4400','EPSG','8818','EPSG','16037',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6002','projected_crs','EPSG','8837','EPSG','4526','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','8838','MTRF-2000 / UTM zone 38N',NULL,'EPSG','4400','EPSG','8818','EPSG','16038',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6003','projected_crs','EPSG','8838','EPSG','4527','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','8839','MTRF-2000 / UTM zone 39N',NULL,'EPSG','4400','EPSG','8818','EPSG','16039',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6004','projected_crs','EPSG','8839','EPSG','4528','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','8840','MTRF-2000 / UTM zone 40N',NULL,'EPSG','4400','EPSG','8818','EPSG','16040',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6005','projected_crs','EPSG','8840','EPSG','3106','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','8857','WGS 84 / Equal Earth Greenwich',NULL,'EPSG','4400','EPSG','4326','EPSG','8854',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6007','projected_crs','EPSG','8857','EPSG','1262','EPSG','1225');
INSERT INTO "projected_crs" VALUES('EPSG','8858','WGS 84 / Equal Earth Americas',NULL,'EPSG','4400','EPSG','4326','EPSG','8855',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6008','projected_crs','EPSG','8858','EPSG','4520','EPSG','1223');
INSERT INTO "projected_crs" VALUES('EPSG','8859','WGS 84 / Equal Earth Asia-Pacific',NULL,'EPSG','4400','EPSG','4326','EPSG','8856',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6009','projected_crs','EPSG','8859','EPSG','4523','EPSG','1224');
INSERT INTO "projected_crs" VALUES('EPSG','8903','RGWF96 / UTM zone 1S',NULL,'EPSG','4400','EPSG','8900','EPSG','16101',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6019','projected_crs','EPSG','8903','EPSG','1255','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','8908','CR-SIRGAS epoch 2014.59 / CRTM05',NULL,'EPSG','4400','EPSG','8907','EPSG','5366',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6024','projected_crs','EPSG','8908','EPSG','3232','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','8909','CR-SIRGAS epoch 2014.59 / UTM zone 16N',NULL,'EPSG','4400','EPSG','8907','EPSG','16016',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6025','projected_crs','EPSG','8909','EPSG','4532','EPSG','1136');
INSERT INTO "projected_crs" VALUES('EPSG','8910','CR-SIRGAS epoch 2014.59 / UTM zone 17N',NULL,'EPSG','4400','EPSG','8907','EPSG','16017',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6026','projected_crs','EPSG','8910','EPSG','4531','EPSG','1136');
INSERT INTO "projected_crs" VALUES('EPSG','8950','SIRGAS-Chile 2010 / UTM zone 18S',NULL,'EPSG','4400','EPSG','8949','EPSG','16118',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6064','projected_crs','EPSG','8950','EPSG','3829','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','8951','SIRGAS-Chile 2010 / UTM zone 19S',NULL,'EPSG','4400','EPSG','8949','EPSG','16119',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6065','projected_crs','EPSG','8951','EPSG','3811','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','9039','ISN2016 / LAEA Europe',NULL,'EPSG','4532','EPSG','8086','EPSG','19986',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6114','projected_crs','EPSG','9039','EPSG','1120','EPSG','1162');
INSERT INTO "projected_crs" VALUES('EPSG','9040','ISN2016 / LCC Europe',NULL,'EPSG','4532','EPSG','8086','EPSG','19985',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6115','projected_crs','EPSG','9040','EPSG','1120','EPSG','1107');
INSERT INTO "projected_crs" VALUES('EPSG','9141','KOSOVAREF01 / Balkans zone 7',NULL,'EPSG','4400','EPSG','9140','EPSG','18277',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6142','projected_crs','EPSG','9141','EPSG','4542','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','9149','SIRGAS-Chile 2013 / UTM zone 18S',NULL,'EPSG','4400','EPSG','9148','EPSG','16118',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6146','projected_crs','EPSG','9149','EPSG','3829','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','9150','SIRGAS-Chile 2013 / UTM zone 19S',NULL,'EPSG','4400','EPSG','9148','EPSG','16119',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6147','projected_crs','EPSG','9150','EPSG','3811','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','9154','SIRGAS-Chile 2016 / UTM zone 18S',NULL,'EPSG','4400','EPSG','9153','EPSG','16118',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6151','projected_crs','EPSG','9154','EPSG','3829','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','9155','SIRGAS-Chile 2016 / UTM zone 19S',NULL,'EPSG','4400','EPSG','9153','EPSG','16119',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6152','projected_crs','EPSG','9155','EPSG','3811','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','9156','RSAO13 / UTM zone 32S',NULL,'EPSG','4400','EPSG','8699','EPSG','16132',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6153','projected_crs','EPSG','9156','EPSG','4551','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','9157','RSAO13 / UTM zone 33S',NULL,'EPSG','4400','EPSG','8699','EPSG','16133',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6154','projected_crs','EPSG','9157','EPSG','4555','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','9158','RSAO13 / UTM zone 34S',NULL,'EPSG','4400','EPSG','8699','EPSG','16134',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6155','projected_crs','EPSG','9158','EPSG','4539','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','9159','RSAO13 / TM 12 SE',NULL,'EPSG','4400','EPSG','8699','EPSG','16612',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6156','projected_crs','EPSG','9159','EPSG','1604','EPSG','1136');
INSERT INTO "projected_crs" VALUES('EPSG','9191','WGS 84 / NIWA Albers',NULL,'EPSG','4400','EPSG','4326','EPSG','9190',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6160','projected_crs','EPSG','9191','EPSG','3508','EPSG','1247');
INSERT INTO "projected_crs" VALUES('EPSG','9205','VN-2000 / TM-3 103-00',NULL,'EPSG','4400','EPSG','4756','EPSG','9058',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6161','projected_crs','EPSG','9205','EPSG','4541','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','9206','VN-2000 / TM-3 104-00',NULL,'EPSG','4400','EPSG','4756','EPSG','9192',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6162','projected_crs','EPSG','9206','EPSG','4538','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','9207','VN-2000 / TM-3 104-30',NULL,'EPSG','4400','EPSG','4756','EPSG','9193',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6163','projected_crs','EPSG','9207','EPSG','4545','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','9208','VN-2000 / TM-3 104-45',NULL,'EPSG','4400','EPSG','4756','EPSG','9194',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6164','projected_crs','EPSG','9208','EPSG','4546','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','9209','VN-2000 / TM-3 105-30',NULL,'EPSG','4400','EPSG','4756','EPSG','9195',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6165','projected_crs','EPSG','9209','EPSG','4548','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','9210','VN-2000 / TM-3 105-45',NULL,'EPSG','4400','EPSG','4756','EPSG','9196',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6166','projected_crs','EPSG','9210','EPSG','4549','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','9211','VN-2000 / TM-3 106-00',NULL,'EPSG','4400','EPSG','4756','EPSG','9197',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6167','projected_crs','EPSG','9211','EPSG','4550','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','9212','VN-2000 / TM-3 106-15',NULL,'EPSG','4400','EPSG','4756','EPSG','9198',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6168','projected_crs','EPSG','9212','EPSG','4552','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','9213','VN-2000 / TM-3 106-30',NULL,'EPSG','4400','EPSG','4756','EPSG','9199',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6169','projected_crs','EPSG','9213','EPSG','4553','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','9214','VN-2000 / TM-3 107-00',NULL,'EPSG','4400','EPSG','4756','EPSG','9200',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6170','projected_crs','EPSG','9214','EPSG','4554','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','9215','VN-2000 / TM-3 107-15',NULL,'EPSG','4400','EPSG','4756','EPSG','9201',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6171','projected_crs','EPSG','9215','EPSG','4556','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','9216','VN-2000 / TM-3 107-30',NULL,'EPSG','4400','EPSG','4756','EPSG','9202',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6172','projected_crs','EPSG','9216','EPSG','4557','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','9217','VN-2000 / TM-3 108-15',NULL,'EPSG','4400','EPSG','4756','EPSG','9203',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6173','projected_crs','EPSG','9217','EPSG','4559','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','9218','VN-2000 / TM-3 108-30',NULL,'EPSG','4400','EPSG','4756','EPSG','9204',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6174','projected_crs','EPSG','9218','EPSG','4560','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','9221','Hartebeesthoek94 / ZAF BSU Albers 25E',NULL,'EPSG','4500','EPSG','4148','EPSG','9219',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6175','projected_crs','EPSG','9221','EPSG','4567','EPSG','1240');
INSERT INTO "projected_crs" VALUES('EPSG','9222','Hartebeesthoek94 / ZAF BSU Albers 44E',NULL,'EPSG','4500','EPSG','4148','EPSG','9220',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6176','projected_crs','EPSG','9222','EPSG','4568','EPSG','1240');
INSERT INTO "projected_crs" VALUES('EPSG','9249','Tapi Aike / Argentina 1',NULL,'EPSG','4530','EPSG','9248','EPSG','18031',NULL,0);
INSERT INTO "usage" VALUES('EPSG','13902','projected_crs','EPSG','9249','EPSG','4570','EPSG','1136');
INSERT INTO "projected_crs" VALUES('EPSG','9250','Tapi Aike / Argentina 2',NULL,'EPSG','4530','EPSG','9248','EPSG','18032',NULL,0);
INSERT INTO "usage" VALUES('EPSG','13903','projected_crs','EPSG','9250','EPSG','4571','EPSG','1136');
INSERT INTO "projected_crs" VALUES('EPSG','9252','MMN / Argentina 2',NULL,'EPSG','4530','EPSG','9251','EPSG','18032',NULL,0);
INSERT INTO "usage" VALUES('EPSG','13905','projected_crs','EPSG','9252','EPSG','2357','EPSG','1136');
INSERT INTO "projected_crs" VALUES('EPSG','9254','MMS / Argentina 2',NULL,'EPSG','4530','EPSG','9253','EPSG','18032',NULL,0);
INSERT INTO "usage" VALUES('EPSG','13907','projected_crs','EPSG','9254','EPSG','2357','EPSG','1136');
INSERT INTO "projected_crs" VALUES('EPSG','9265','POSGAR 2007 / UTM zone 19S',NULL,'EPSG','4400','EPSG','5340','EPSG','16119',NULL,0);
INSERT INTO "usage" VALUES('EPSG','13909','projected_crs','EPSG','9265','EPSG','2596','EPSG','1136');
INSERT INTO "projected_crs" VALUES('EPSG','9271','MGI / Austria West',NULL,'EPSG','4530','EPSG','4312','EPSG','9268',NULL,0);
INSERT INTO "usage" VALUES('EPSG','13912','projected_crs','EPSG','9271','EPSG','1706','EPSG','1258');
INSERT INTO "projected_crs" VALUES('EPSG','9272','MGI / Austria Central',NULL,'EPSG','4530','EPSG','4312','EPSG','9269',NULL,0);
INSERT INTO "usage" VALUES('EPSG','13913','projected_crs','EPSG','9272','EPSG','1707','EPSG','1258');
INSERT INTO "projected_crs" VALUES('EPSG','9273','MGI / Austria East',NULL,'EPSG','4530','EPSG','4312','EPSG','9270',NULL,0);
INSERT INTO "usage" VALUES('EPSG','13914','projected_crs','EPSG','9273','EPSG','1708','EPSG','1258');
INSERT INTO "projected_crs" VALUES('EPSG','9284','Pampa del Castillo / Argentina 1',NULL,'EPSG','4530','EPSG','4161','EPSG','18031',NULL,0);
INSERT INTO "usage" VALUES('EPSG','13917','projected_crs','EPSG','9284','EPSG','4564','EPSG','1136');
INSERT INTO "projected_crs" VALUES('EPSG','9285','Pampa del Castillo / Argentina 3',NULL,'EPSG','4530','EPSG','4161','EPSG','18033',NULL,0);
INSERT INTO "usage" VALUES('EPSG','13918','projected_crs','EPSG','9285','EPSG','4565','EPSG','1136');
INSERT INTO "projected_crs" VALUES('EPSG','9295','ONGD17 / UTM zone 39N',NULL,'EPSG','4400','EPSG','9294','EPSG','16039',NULL,0);
INSERT INTO "usage" VALUES('EPSG','13966','projected_crs','EPSG','9295','EPSG','4322','EPSG','1153');
INSERT INTO "projected_crs" VALUES('EPSG','9296','ONGD17 / UTM zone 40N',NULL,'EPSG','4400','EPSG','9294','EPSG','16040',NULL,0);
INSERT INTO "usage" VALUES('EPSG','13967','projected_crs','EPSG','9296','EPSG','4323','EPSG','1153');
INSERT INTO "projected_crs" VALUES('EPSG','9297','ONGD17 / UTM zone 41N',NULL,'EPSG','4400','EPSG','9294','EPSG','16041',NULL,0);
INSERT INTO "usage" VALUES('EPSG','13968','projected_crs','EPSG','9297','EPSG','4324','EPSG','1153');
INSERT INTO "projected_crs" VALUES('EPSG','9300','HS2 Survey Grid',NULL,'EPSG','4400','EPSG','9299','EPSG','9301',NULL,0);
INSERT INTO "usage" VALUES('EPSG','14048','projected_crs','EPSG','9300','EPSG','4582','EPSG','1260');
INSERT INTO "projected_crs" VALUES('EPSG','9311','NAD27 / US National Atlas Equal Area',NULL,'EPSG','4499','EPSG','4267','EPSG','3899',NULL,0);
INSERT INTO "usage" VALUES('EPSG','13969','projected_crs','EPSG','9311','EPSG','1245','EPSG','1162');
INSERT INTO "projected_crs" VALUES('EPSG','9354','WGS 84 / IBCSO Polar Stereographic',NULL,'EPSG','4470','EPSG','4326','EPSG','9353',NULL,0);
INSERT INTO "usage" VALUES('EPSG','13978','projected_crs','EPSG','9354','EPSG','4586','EPSG','1198');
INSERT INTO "projected_crs" VALUES('EPSG','9356','KSA-GRF17 / UTM zone 36N',NULL,'EPSG','4400','EPSG','9333','EPSG','16036',NULL,0);
INSERT INTO "usage" VALUES('EPSG','14173','projected_crs','EPSG','9356','EPSG','4524','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','9357','KSA-GRF17 / UTM zone 37N',NULL,'EPSG','4400','EPSG','9333','EPSG','16037',NULL,0);
INSERT INTO "usage" VALUES('EPSG','14174','projected_crs','EPSG','9357','EPSG','4526','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','9358','KSA-GRF17 / UTM zone 38N',NULL,'EPSG','4400','EPSG','9333','EPSG','16038',NULL,0);
INSERT INTO "usage" VALUES('EPSG','14175','projected_crs','EPSG','9358','EPSG','4527','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','9359','KSA-GRF17 / UTM zone 39N',NULL,'EPSG','4400','EPSG','9333','EPSG','16039',NULL,0);
INSERT INTO "usage" VALUES('EPSG','14176','projected_crs','EPSG','9359','EPSG','4528','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','9360','KSA-GRF17 / UTM zone 40N',NULL,'EPSG','4400','EPSG','9333','EPSG','16040',NULL,0);
INSERT INTO "usage" VALUES('EPSG','14177','projected_crs','EPSG','9360','EPSG','3106','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','9367','TPEN11 Grid',NULL,'EPSG','4400','EPSG','9364','EPSG','9366',NULL,0);
INSERT INTO "usage" VALUES('EPSG','13975','projected_crs','EPSG','9367','EPSG','4583','EPSG','1141');
INSERT INTO "projected_crs" VALUES('EPSG','9373','MML07 Grid',NULL,'EPSG','4400','EPSG','9372','EPSG','9370',NULL,0);
INSERT INTO "usage" VALUES('EPSG','13996','projected_crs','EPSG','9373','EPSG','4588','EPSG','1141');
INSERT INTO "projected_crs" VALUES('EPSG','9377','MAGNA-SIRGAS 2018 / Origen-Nacional',NULL,'EPSG','4500','EPSG','20046','EPSG','9376',NULL,0);
INSERT INTO "usage" VALUES('EPSG','13998','projected_crs','EPSG','9377','EPSG','1070','EPSG','1091');
INSERT INTO "projected_crs" VALUES('EPSG','9387','AbInvA96_2020 Grid',NULL,'EPSG','4400','EPSG','9384','EPSG','9385',NULL,0);
INSERT INTO "usage" VALUES('EPSG','14029','projected_crs','EPSG','9387','EPSG','4589','EPSG','1196');
INSERT INTO "projected_crs" VALUES('EPSG','9391','BGS2005 / UTM zone 35N',NULL,'EPSG','4400','EPSG','7798','EPSG','16035',NULL,0);
INSERT INTO "usage" VALUES('EPSG','14051','projected_crs','EPSG','9391','EPSG','4427','EPSG','1153');
INSERT INTO "projected_crs" VALUES('EPSG','9404','PN68 / UTM zone 27N',NULL,'EPSG','4400','EPSG','9403','EPSG','16027',NULL,0);
INSERT INTO "usage" VALUES('EPSG','14043','projected_crs','EPSG','9404','EPSG','4599','EPSG','1178');
INSERT INTO "projected_crs" VALUES('EPSG','9405','PN68 / UTM zone 28N',NULL,'EPSG','4400','EPSG','9403','EPSG','16028',NULL,0);
INSERT INTO "usage" VALUES('EPSG','14044','projected_crs','EPSG','9405','EPSG','4600','EPSG','1178');
INSERT INTO "projected_crs" VALUES('EPSG','9406','PN84 / UTM zone 27N',NULL,'EPSG','4400','EPSG','4728','EPSG','16027',NULL,0);
INSERT INTO "usage" VALUES('EPSG','14045','projected_crs','EPSG','9406','EPSG','4599','EPSG','1178');
INSERT INTO "projected_crs" VALUES('EPSG','9407','PN84 / UTM zone 28N',NULL,'EPSG','4400','EPSG','4728','EPSG','16028',NULL,0);
INSERT INTO "usage" VALUES('EPSG','14046','projected_crs','EPSG','9407','EPSG','4601','EPSG','1178');
INSERT INTO "projected_crs" VALUES('EPSG','9456','GBK19 Grid',NULL,'EPSG','4400','EPSG','9453','EPSG','9455',NULL,0);
INSERT INTO "usage" VALUES('EPSG','14131','projected_crs','EPSG','9456','EPSG','4607','EPSG','1141');
INSERT INTO "projected_crs" VALUES('EPSG','9473','GDA2020 / Australian Albers',NULL,'EPSG','4400','EPSG','7844','EPSG','17365',NULL,0);
INSERT INTO "usage" VALUES('EPSG','14155','projected_crs','EPSG','9473','EPSG','2575','EPSG','1162');
INSERT INTO "projected_crs" VALUES('EPSG','9476','SRGI2013 / UTM zone 46N',NULL,'EPSG','4400','EPSG','9470','EPSG','16046',NULL,0);
INSERT INTO "usage" VALUES('EPSG','14158','projected_crs','EPSG','9476','EPSG','1647','EPSG','1266');
INSERT INTO "projected_crs" VALUES('EPSG','9477','SRGI2013 / UTM zone 47N',NULL,'EPSG','4400','EPSG','9470','EPSG','16047',NULL,0);
INSERT INTO "usage" VALUES('EPSG','14159','projected_crs','EPSG','9477','EPSG','1649','EPSG','1266');
INSERT INTO "projected_crs" VALUES('EPSG','9478','SRGI2013 / UTM zone 48N',NULL,'EPSG','4400','EPSG','9470','EPSG','16048',NULL,0);
INSERT INTO "usage" VALUES('EPSG','14160','projected_crs','EPSG','9478','EPSG','1651','EPSG','1266');
INSERT INTO "projected_crs" VALUES('EPSG','9479','SRGI2013 / UTM zone 49N',NULL,'EPSG','4400','EPSG','9470','EPSG','16049',NULL,0);
INSERT INTO "usage" VALUES('EPSG','14161','projected_crs','EPSG','9479','EPSG','1653','EPSG','1266');
INSERT INTO "projected_crs" VALUES('EPSG','9480','SRGI2013 / UTM zone 50N',NULL,'EPSG','4400','EPSG','9470','EPSG','16050',NULL,0);
INSERT INTO "usage" VALUES('EPSG','14162','projected_crs','EPSG','9480','EPSG','1655','EPSG','1266');
INSERT INTO "projected_crs" VALUES('EPSG','9481','SRGI2013 / UTM zone 51N',NULL,'EPSG','4400','EPSG','9470','EPSG','16051',NULL,0);
INSERT INTO "usage" VALUES('EPSG','14163','projected_crs','EPSG','9481','EPSG','1657','EPSG','1266');
INSERT INTO "projected_crs" VALUES('EPSG','9482','SRGI2013 / UTM zone 52N',NULL,'EPSG','4400','EPSG','9470','EPSG','16052',NULL,0);
INSERT INTO "usage" VALUES('EPSG','14164','projected_crs','EPSG','9482','EPSG','1659','EPSG','1266');
INSERT INTO "projected_crs" VALUES('EPSG','9487','SRGI2013 / UTM zone 47S',NULL,'EPSG','4400','EPSG','9470','EPSG','16147',NULL,0);
INSERT INTO "usage" VALUES('EPSG','14165','projected_crs','EPSG','9487','EPSG','1650','EPSG','1266');
INSERT INTO "projected_crs" VALUES('EPSG','9488','SRGI2013 / UTM zone 48S',NULL,'EPSG','4400','EPSG','9470','EPSG','16148',NULL,0);
INSERT INTO "usage" VALUES('EPSG','14166','projected_crs','EPSG','9488','EPSG','1652','EPSG','1266');
INSERT INTO "projected_crs" VALUES('EPSG','9489','SRGI2013 / UTM zone 49S',NULL,'EPSG','4400','EPSG','9470','EPSG','16149',NULL,0);
INSERT INTO "usage" VALUES('EPSG','14167','projected_crs','EPSG','9489','EPSG','1654','EPSG','1266');
INSERT INTO "projected_crs" VALUES('EPSG','9490','SRGI2013 / UTM zone 50S',NULL,'EPSG','4400','EPSG','9470','EPSG','16150',NULL,0);
INSERT INTO "usage" VALUES('EPSG','14168','projected_crs','EPSG','9490','EPSG','1656','EPSG','1266');
INSERT INTO "projected_crs" VALUES('EPSG','9491','SRGI2013 / UTM zone 51S',NULL,'EPSG','4400','EPSG','9470','EPSG','16151',NULL,0);
INSERT INTO "usage" VALUES('EPSG','14169','projected_crs','EPSG','9491','EPSG','1658','EPSG','1266');
INSERT INTO "projected_crs" VALUES('EPSG','9492','SRGI2013 / UTM zone 52S',NULL,'EPSG','4400','EPSG','9470','EPSG','16152',NULL,0);
INSERT INTO "usage" VALUES('EPSG','14170','projected_crs','EPSG','9492','EPSG','1660','EPSG','1266');
INSERT INTO "projected_crs" VALUES('EPSG','9493','SRGI2013 / UTM zone 53S',NULL,'EPSG','4400','EPSG','9470','EPSG','16153',NULL,0);
INSERT INTO "usage" VALUES('EPSG','14171','projected_crs','EPSG','9493','EPSG','1662','EPSG','1266');
INSERT INTO "projected_crs" VALUES('EPSG','9494','SRGI2013 / UTM zone 54S',NULL,'EPSG','4400','EPSG','9470','EPSG','16154',NULL,0);
INSERT INTO "usage" VALUES('EPSG','14172','projected_crs','EPSG','9494','EPSG','1663','EPSG','1266');
INSERT INTO "projected_crs" VALUES('EPSG','9498','POSGAR 2007 / CABA 2019',NULL,'EPSG','4530','EPSG','5340','EPSG','9497',NULL,0);
INSERT INTO "usage" VALUES('EPSG','14513','projected_crs','EPSG','9498','EPSG','4610','EPSG','1056');
INSERT INTO "projected_crs" VALUES('EPSG','9549','LTF2004(C)',NULL,'EPSG','4400','EPSG','9547','EPSG','9548',NULL,0);
INSERT INTO "usage" VALUES('EPSG','14928','projected_crs','EPSG','9549','EPSG','4613','EPSG','1271');
INSERT INTO "projected_crs" VALUES('EPSG','9674','NAD83 / USFS R6 Albers',NULL,'EPSG','4400','EPSG','4269','EPSG','9673',NULL,0);
INSERT INTO "usage" VALUES('EPSG','14787','projected_crs','EPSG','9674','EPSG','2381','EPSG','1165');
INSERT INTO "projected_crs" VALUES('EPSG','9678','Gulshan 303 / Bangladesh Transverse Mercator',NULL,'EPSG','4400','EPSG','4682','EPSG','9677',NULL,0);
INSERT INTO "usage" VALUES('EPSG','14849','projected_crs','EPSG','9678','EPSG','3217','EPSG','1274');
INSERT INTO "projected_crs" VALUES('EPSG','9680','WGS 84 / TM 90 NE',NULL,'EPSG','4400','EPSG','4326','EPSG','16490',NULL,0);
INSERT INTO "usage" VALUES('EPSG','14880','projected_crs','EPSG','9680','EPSG','1041','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','9697','REDGEOMIN / UTM zone 12S',NULL,'EPSG','4400','EPSG','9696','EPSG','16112',NULL,0);
INSERT INTO "usage" VALUES('EPSG','15026','projected_crs','EPSG','9697','EPSG','3188','EPSG','1181');
INSERT INTO "projected_crs" VALUES('EPSG','9698','REDGEOMIN / UTM zone 18S',NULL,'EPSG','4400','EPSG','9696','EPSG','16118',NULL,0);
INSERT INTO "usage" VALUES('EPSG','14943','projected_crs','EPSG','9698','EPSG','3829','EPSG','1181');
INSERT INTO "projected_crs" VALUES('EPSG','9699','REDGEOMIN / UTM zone 19S',NULL,'EPSG','4400','EPSG','9696','EPSG','16119',NULL,0);
INSERT INTO "usage" VALUES('EPSG','14945','projected_crs','EPSG','9699','EPSG','3811','EPSG','1181');
INSERT INTO "projected_crs" VALUES('EPSG','9709','NAD83(CSRS) / UTM zone 23N',NULL,'EPSG','4400','EPSG','4617','EPSG','16023',NULL,0);
INSERT INTO "usage" VALUES('EPSG','15204','projected_crs','EPSG','9709','EPSG','2153','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','9712','NAD83 / UTM zone 24N',NULL,'EPSG','4400','EPSG','4269','EPSG','16024',NULL,0);
INSERT INTO "usage" VALUES('EPSG','15196','projected_crs','EPSG','9712','EPSG','4617','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','9713','NAD83(CSRS) / UTM zone 24N',NULL,'EPSG','4400','EPSG','4617','EPSG','16024',NULL,0);
INSERT INTO "usage" VALUES('EPSG','15195','projected_crs','EPSG','9713','EPSG','4617','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','9716','IGM95 / UTM zone 34N',NULL,'EPSG','4400','EPSG','4670','EPSG','16034',NULL,0);
INSERT INTO "usage" VALUES('EPSG','15325','projected_crs','EPSG','9716','EPSG','4187','EPSG','1027');
INSERT INTO "projected_crs" VALUES('EPSG','9741','EOS21 Grid',NULL,'EPSG','4400','EPSG','9739','EPSG','9738',NULL,0);
INSERT INTO "usage" VALUES('EPSG','15317','projected_crs','EPSG','9741','EPSG','4620','EPSG','1141');
INSERT INTO "projected_crs" VALUES('EPSG','9748','NAD83(2011) / Alabama East (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','9746',NULL,0);
INSERT INTO "usage" VALUES('EPSG','15376','projected_crs','EPSG','9748','EPSG','2154','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','9749','NAD83(2011) / Alabama West (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','9747',NULL,0);
INSERT INTO "usage" VALUES('EPSG','15375','projected_crs','EPSG','9749','EPSG','2155','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','9761','ECML14_NB Grid',NULL,'EPSG','4400','EPSG','9758','EPSG','9760',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16498','projected_crs','EPSG','9761','EPSG','4621','EPSG','1141');
INSERT INTO "projected_crs" VALUES('EPSG','9766','EWR2 Grid',NULL,'EPSG','4400','EPSG','9763','EPSG','9765',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16509','projected_crs','EPSG','9766','EPSG','4622','EPSG','1141');
INSERT INTO "projected_crs" VALUES('EPSG','9793','RGF93 v2 / Lambert-93',NULL,'EPSG','4499','EPSG','9777','EPSG','18085',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16614','projected_crs','EPSG','9793','EPSG','1096','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','9794','RGF93 v2b / Lambert-93',NULL,'EPSG','4499','EPSG','9782','EPSG','18085',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16615','projected_crs','EPSG','9794','EPSG','1096','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','9821','UCS-2000 / LCS-32 Kyiv region',NULL,'EPSG','4531','EPSG','5561','EPSG','9796',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16867','projected_crs','EPSG','9821','EPSG','4633','EPSG','1207');
INSERT INTO "projected_crs" VALUES('EPSG','9822','RGF93 v2 / CC42',NULL,'EPSG','4499','EPSG','9777','EPSG','18101',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16616','projected_crs','EPSG','9822','EPSG','3545','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','9823','RGF93 v2 / CC43',NULL,'EPSG','4499','EPSG','9777','EPSG','18102',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16617','projected_crs','EPSG','9823','EPSG','3546','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','9824','RGF93 v2 / CC44',NULL,'EPSG','4499','EPSG','9777','EPSG','18103',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16618','projected_crs','EPSG','9824','EPSG','3547','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','9825','RGF93 v2 / CC45',NULL,'EPSG','4499','EPSG','9777','EPSG','18104',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16619','projected_crs','EPSG','9825','EPSG','3548','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','9826','RGF93 v2 / CC46',NULL,'EPSG','4499','EPSG','9777','EPSG','18105',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16620','projected_crs','EPSG','9826','EPSG','3549','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','9827','RGF93 v2 / CC47',NULL,'EPSG','4499','EPSG','9777','EPSG','18106',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16621','projected_crs','EPSG','9827','EPSG','3550','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','9828','RGF93 v2 / CC48',NULL,'EPSG','4499','EPSG','9777','EPSG','18107',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16622','projected_crs','EPSG','9828','EPSG','3551','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','9829','RGF93 v2 / CC49',NULL,'EPSG','4499','EPSG','9777','EPSG','18108',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16623','projected_crs','EPSG','9829','EPSG','3552','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','9830','RGF93 v2 / CC50',NULL,'EPSG','4499','EPSG','9777','EPSG','18109',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16624','projected_crs','EPSG','9830','EPSG','3553','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','9831','UCS-2000 / LCS-01 Crimea',NULL,'EPSG','4531','EPSG','5561','EPSG','9797',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16766','projected_crs','EPSG','9831','EPSG','4648','EPSG','1207');
INSERT INTO "projected_crs" VALUES('EPSG','9832','UCS-2000 / LCS-05 Vinnytsia',NULL,'EPSG','4531','EPSG','5561','EPSG','9798',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16767','projected_crs','EPSG','9832','EPSG','4643','EPSG','1207');
INSERT INTO "projected_crs" VALUES('EPSG','9833','UCS-2000 / LCS-07 Volyn',NULL,'EPSG','4531','EPSG','5561','EPSG','9799',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16768','projected_crs','EPSG','9833','EPSG','4644','EPSG','1207');
INSERT INTO "projected_crs" VALUES('EPSG','9834','UCS-2000 / LCS-12 Dnipropetrovsk',NULL,'EPSG','4531','EPSG','5561','EPSG','9800',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16769','projected_crs','EPSG','9834','EPSG','4627','EPSG','1207');
INSERT INTO "projected_crs" VALUES('EPSG','9835','UCS-2000 / LCS-14 Donetsk',NULL,'EPSG','4531','EPSG','5561','EPSG','9801',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16772','projected_crs','EPSG','9835','EPSG','4628','EPSG','1207');
INSERT INTO "projected_crs" VALUES('EPSG','9836','UCS-2000 / LCS-18 Zhytomyr',NULL,'EPSG','4531','EPSG','5561','EPSG','9802',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16771','projected_crs','EPSG','9836','EPSG','4647','EPSG','1207');
INSERT INTO "projected_crs" VALUES('EPSG','9837','UCS-2000 / LCS-21 Zakarpattia',NULL,'EPSG','4531','EPSG','5561','EPSG','9803',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16773','projected_crs','EPSG','9837','EPSG','4645','EPSG','1207');
INSERT INTO "projected_crs" VALUES('EPSG','9838','UCS-2000 / LCS-23 Zaporizhzhia',NULL,'EPSG','4531','EPSG','5561','EPSG','9804',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16774','projected_crs','EPSG','9838','EPSG','4646','EPSG','1207');
INSERT INTO "projected_crs" VALUES('EPSG','9839','UCS-2000 / LCS-26 Ivano-Frankivsk',NULL,'EPSG','4531','EPSG','5561','EPSG','9805',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16775','projected_crs','EPSG','9839','EPSG','4629','EPSG','1207');
INSERT INTO "projected_crs" VALUES('EPSG','9840','UCS-2000 / LCS-35 Kirovohrad',NULL,'EPSG','4531','EPSG','5561','EPSG','9806',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16776','projected_crs','EPSG','9840','EPSG','4634','EPSG','1207');
INSERT INTO "projected_crs" VALUES('EPSG','9841','UCS-2000 / LCS-44 Luhansk',NULL,'EPSG','4531','EPSG','5561','EPSG','9807',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16777','projected_crs','EPSG','9841','EPSG','4635','EPSG','1207');
INSERT INTO "projected_crs" VALUES('EPSG','9842','RGF93 v2b / CC42',NULL,'EPSG','4499','EPSG','9782','EPSG','18101',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16625','projected_crs','EPSG','9842','EPSG','3545','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','9843','RGF93 v2b / CC43',NULL,'EPSG','4499','EPSG','9782','EPSG','18102',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16626','projected_crs','EPSG','9843','EPSG','3546','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','9844','RGF93 v2b / CC44',NULL,'EPSG','4499','EPSG','9782','EPSG','18103',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16627','projected_crs','EPSG','9844','EPSG','3547','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','9845','RGF93 v2b / CC45',NULL,'EPSG','4499','EPSG','9782','EPSG','18104',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16628','projected_crs','EPSG','9845','EPSG','3548','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','9846','RGF93 v2b / CC46',NULL,'EPSG','4499','EPSG','9782','EPSG','18105',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16629','projected_crs','EPSG','9846','EPSG','3549','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','9847','RGF93 v2b / CC47',NULL,'EPSG','4499','EPSG','9782','EPSG','18106',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16630','projected_crs','EPSG','9847','EPSG','3550','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','9848','RGF93 v2b / CC48',NULL,'EPSG','4499','EPSG','9782','EPSG','18107',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16631','projected_crs','EPSG','9848','EPSG','3551','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','9849','RGF93 v2b / CC49',NULL,'EPSG','4499','EPSG','9782','EPSG','18108',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16632','projected_crs','EPSG','9849','EPSG','3552','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','9850','RGF93 v2b / CC50',NULL,'EPSG','4499','EPSG','9782','EPSG','18109',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16633','projected_crs','EPSG','9850','EPSG','3553','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','9851','UCS-2000 / LCS-46 Lviv',NULL,'EPSG','4531','EPSG','5561','EPSG','9808',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16778','projected_crs','EPSG','9851','EPSG','4636','EPSG','1207');
INSERT INTO "projected_crs" VALUES('EPSG','9852','UCS-2000 / LCS-48 Mykolaiv',NULL,'EPSG','4531','EPSG','5561','EPSG','9809',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16779','projected_crs','EPSG','9852','EPSG','4637','EPSG','1207');
INSERT INTO "projected_crs" VALUES('EPSG','9853','UCS-2000 / LCS-51 Odessa',NULL,'EPSG','4531','EPSG','5561','EPSG','9810',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16780','projected_crs','EPSG','9853','EPSG','4638','EPSG','1207');
INSERT INTO "projected_crs" VALUES('EPSG','9854','UCS-2000 / LCS-53 Poltava',NULL,'EPSG','4531','EPSG','5561','EPSG','9811',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16786','projected_crs','EPSG','9854','EPSG','4639','EPSG','1207');
INSERT INTO "projected_crs" VALUES('EPSG','9855','UCS-2000 / LCS-56 Rivne',NULL,'EPSG','4531','EPSG','5561','EPSG','9812',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16869','projected_crs','EPSG','9855','EPSG','4640','EPSG','1207');
INSERT INTO "projected_crs" VALUES('EPSG','9856','UCS-2000 / LCS-59 Sumy',NULL,'EPSG','4531','EPSG','5561','EPSG','9813',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16785','projected_crs','EPSG','9856','EPSG','4641','EPSG','1207');
INSERT INTO "projected_crs" VALUES('EPSG','9857','UCS-2000 / LCS-61 Ternopil',NULL,'EPSG','4531','EPSG','5561','EPSG','9814',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16789','projected_crs','EPSG','9857','EPSG','4642','EPSG','1207');
INSERT INTO "projected_crs" VALUES('EPSG','9858','UCS-2000 / LCS-63 Kharkiv',NULL,'EPSG','4531','EPSG','5561','EPSG','9815',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16788','projected_crs','EPSG','9858','EPSG','4630','EPSG','1207');
INSERT INTO "projected_crs" VALUES('EPSG','9859','UCS-2000 / LCS-65 Kherson',NULL,'EPSG','4531','EPSG','5561','EPSG','9816',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16790','projected_crs','EPSG','9859','EPSG','4631','EPSG','1207');
INSERT INTO "projected_crs" VALUES('EPSG','9860','UCS-2000 / LCS-68 Khmelnytsky',NULL,'EPSG','4531','EPSG','5561','EPSG','9812',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16868','projected_crs','EPSG','9860','EPSG','4632','EPSG','1207');
INSERT INTO "projected_crs" VALUES('EPSG','9861','UCS-2000 / LCS-71 Cherkasy',NULL,'EPSG','4531','EPSG','5561','EPSG','9817',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16842','projected_crs','EPSG','9861','EPSG','4624','EPSG','1207');
INSERT INTO "projected_crs" VALUES('EPSG','9862','UCS-2000 / LCS-73 Chernivtsi',NULL,'EPSG','4531','EPSG','5561','EPSG','9818',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16841','projected_crs','EPSG','9862','EPSG','4626','EPSG','1207');
INSERT INTO "projected_crs" VALUES('EPSG','9863','UCS-2000 / LCS-74 Chernihiv',NULL,'EPSG','4531','EPSG','5561','EPSG','9819',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16840','projected_crs','EPSG','9863','EPSG','4625','EPSG','1207');
INSERT INTO "projected_crs" VALUES('EPSG','9864','UCS-2000 / LCS-80 Kyiv city',NULL,'EPSG','4531','EPSG','5561','EPSG','9796',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16922','projected_crs','EPSG','9864','EPSG','4623','EPSG','1207');
INSERT INTO "projected_crs" VALUES('EPSG','9865','UCS-2000 / LCS-85 Sevastopol',NULL,'EPSG','4531','EPSG','5561','EPSG','9820',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16836','projected_crs','EPSG','9865','EPSG','4649','EPSG','1207');
INSERT INTO "projected_crs" VALUES('EPSG','9869','MRH21 Grid',NULL,'EPSG','4400','EPSG','9866','EPSG','9868',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16939','projected_crs','EPSG','9869','EPSG','4652','EPSG','1141');
INSERT INTO "projected_crs" VALUES('EPSG','9874','PNG94 / PNGMG94 zone 57',NULL,'EPSG','4400','EPSG','5546','EPSG','9872',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16919','projected_crs','EPSG','9874','EPSG','4653','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','9875','PNG94 / PNGMG94 zone 58',NULL,'EPSG','4400','EPSG','5546','EPSG','9873',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16920','projected_crs','EPSG','9875','EPSG','4654','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','9880','MOLDOR11 Grid',NULL,'EPSG','4400','EPSG','9871','EPSG','9879',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16961','projected_crs','EPSG','9880','EPSG','4655','EPSG','1141');
INSERT INTO "projected_crs" VALUES('EPSG','9895','LUREF / Luxembourg TM (3D)',NULL,'EPSG','1046','EPSG','9893','EPSG','9894',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17310','projected_crs','EPSG','9895','EPSG','1146','EPSG','1091');
INSERT INTO "projected_crs" VALUES('EPSG','9943','EBBWV14 Grid',NULL,'EPSG','4400','EPSG','9939','EPSG','9942',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17349','projected_crs','EPSG','9943','EPSG','4661','EPSG','1141');
INSERT INTO "projected_crs" VALUES('EPSG','9945','Macedonia State Coordinate System truncated',NULL,'EPSG','4498','EPSG','3906','EPSG','9911',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17461','projected_crs','EPSG','9945','EPSG','1148','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','9947','ISN2004 / LAEA Iceland',NULL,'EPSG','4400','EPSG','5324','EPSG','9946',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17376','projected_crs','EPSG','9947','EPSG','1120','EPSG','1162');
INSERT INTO "projected_crs" VALUES('EPSG','9967','HULLEE13 Grid',NULL,'EPSG','4400','EPSG','9964','EPSG','9966',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17429','projected_crs','EPSG','9967','EPSG','4663','EPSG','1141');
INSERT INTO "projected_crs" VALUES('EPSG','9972','SCM22 Grid',NULL,'EPSG','4400','EPSG','9969','EPSG','9971',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17441','projected_crs','EPSG','9972','EPSG','4665','EPSG','1141');
INSERT INTO "projected_crs" VALUES('EPSG','9977','FNL22 Grid',NULL,'EPSG','4400','EPSG','9974','EPSG','9976',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17456','projected_crs','EPSG','9977','EPSG','4664','EPSG','1141');
INSERT INTO "projected_crs" VALUES('EPSG','10160','S34J reconstruction east-orientated',NULL,'EPSG','4400','EPSG','10158','EPSG','10159',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19661','projected_crs','EPSG','10160','EPSG','2531','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','10183','DoPw22 Grid',NULL,'EPSG','4400','EPSG','10175','EPSG','10182',NULL,0);
INSERT INTO "usage" VALUES('EPSG','18655','projected_crs','EPSG','10183','EPSG','4686','EPSG','1141');
INSERT INTO "projected_crs" VALUES('EPSG','10188','ShAb07 Grid',NULL,'EPSG','4400','EPSG','10185','EPSG','10187',NULL,0);
INSERT INTO "usage" VALUES('EPSG','18667','projected_crs','EPSG','10188','EPSG','4687','EPSG','1141');
INSERT INTO "projected_crs" VALUES('EPSG','10194','CNH22 Grid',NULL,'EPSG','4400','EPSG','10191','EPSG','10193',NULL,0);
INSERT INTO "usage" VALUES('EPSG','18766','projected_crs','EPSG','10194','EPSG','4677','EPSG','1141');
INSERT INTO "projected_crs" VALUES('EPSG','10199','CWS13 Grid',NULL,'EPSG','4400','EPSG','10196','EPSG','10198',NULL,0);
INSERT INTO "usage" VALUES('EPSG','18767','projected_crs','EPSG','10199','EPSG','4678','EPSG','1141');
INSERT INTO "projected_crs" VALUES('EPSG','10207','DIBA15 Grid',NULL,'EPSG','4400','EPSG','10204','EPSG','10206',NULL,0);
INSERT INTO "usage" VALUES('EPSG','18771','projected_crs','EPSG','10207','EPSG','4681','EPSG','1141');
INSERT INTO "projected_crs" VALUES('EPSG','10212','GWPBS22 Grid',NULL,'EPSG','4400','EPSG','10209','EPSG','10211',NULL,0);
INSERT INTO "usage" VALUES('EPSG','18797','projected_crs','EPSG','10212','EPSG','4685','EPSG','1141');
INSERT INTO "projected_crs" VALUES('EPSG','10217','GWWAB22 Grid',NULL,'EPSG','4400','EPSG','10214','EPSG','10211',NULL,0);
INSERT INTO "usage" VALUES('EPSG','18802','projected_crs','EPSG','10217','EPSG','4684','EPSG','1141');
INSERT INTO "projected_crs" VALUES('EPSG','10222','GWWWA22 Grid',NULL,'EPSG','4400','EPSG','10219','EPSG','10211',NULL,0);
INSERT INTO "usage" VALUES('EPSG','18838','projected_crs','EPSG','10222','EPSG','4683','EPSG','1141');
INSERT INTO "projected_crs" VALUES('EPSG','10227','MALS09 Grid',NULL,'EPSG','4400','EPSG','10224','EPSG','10226',NULL,0);
INSERT INTO "usage" VALUES('EPSG','18782','projected_crs','EPSG','10227','EPSG','4682','EPSG','1141');
INSERT INTO "projected_crs" VALUES('EPSG','10235','OxWo08 Grid',NULL,'EPSG','4400','EPSG','10229','EPSG','10234',NULL,0);
INSERT INTO "usage" VALUES('EPSG','18786','projected_crs','EPSG','10235','EPSG','4680','EPSG','1141');
INSERT INTO "projected_crs" VALUES('EPSG','10240','SYC20 Grid',NULL,'EPSG','4400','EPSG','10237','EPSG','10239',NULL,0);
INSERT INTO "usage" VALUES('EPSG','18788','projected_crs','EPSG','10240','EPSG','4679','EPSG','1141');
INSERT INTO "projected_crs" VALUES('EPSG','10250','S34S reconstruction east-orientated',NULL,'EPSG','4400','EPSG','10249','EPSG','10159',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19665','projected_crs','EPSG','10250','EPSG','2532','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','10254','S45B reconstruction east-orientated',NULL,'EPSG','4400','EPSG','10252','EPSG','10253',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19674','projected_crs','EPSG','10254','EPSG','2533','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','10258','GS reconstruction east-orientated',NULL,'EPSG','4400','EPSG','10256','EPSG','10257',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19709','projected_crs','EPSG','10258','EPSG','4575','EPSG','1153');
INSERT INTO "projected_crs" VALUES('EPSG','10262','GSB reconstruction east-orientated',NULL,'EPSG','4400','EPSG','10260','EPSG','10261',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19685','projected_crs','EPSG','10262','EPSG','2533','EPSG','1180');
INSERT INTO "projected_crs" VALUES('EPSG','10266','KK reconstruction east-orientated',NULL,'EPSG','4400','EPSG','10265','EPSG','10257',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19710','projected_crs','EPSG','10266','EPSG','4693','EPSG','1153');
INSERT INTO "projected_crs" VALUES('EPSG','10270','Ostenfeld reconstruction',NULL,'EPSG','4400','EPSG','10268','EPSG','10269',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19698','projected_crs','EPSG','10270','EPSG','4694','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','10275','SMITB20 Grid',NULL,'EPSG','4400','EPSG','10272','EPSG','10274',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19178','projected_crs','EPSG','10275','EPSG','4688','EPSG','1141');
INSERT INTO "projected_crs" VALUES('EPSG','10280','RBEPP12 Grid',NULL,'EPSG','4400','EPSG','10277','EPSG','10279',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19184','projected_crs','EPSG','10280','EPSG','4689','EPSG','1141');
INSERT INTO "projected_crs" VALUES('EPSG','10285','ETRS89/DREF91/2016 / 3-degree Gauss-Kruger zone 3',NULL,'EPSG','4400','EPSG','10284','EPSG','16263',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19567','projected_crs','EPSG','10285','EPSG','4490','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','10286','ETRS89/DREF91/2016 / UTM zone 31N (N-zE)',NULL,'EPSG','4500','EPSG','10284','EPSG','5647',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19247','projected_crs','EPSG','10286','EPSG','2860','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','10287','ETRS89/DREF91/2016 / UTM zone 31N (zE-N)',NULL,'EPSG','4400','EPSG','10284','EPSG','5647',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19248','projected_crs','EPSG','10287','EPSG','2860','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','10288','ETRS89/DREF91/2016 / UTM zone 32N (N-zE)',NULL,'EPSG','4500','EPSG','10284','EPSG','4648',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19249','projected_crs','EPSG','10288','EPSG','2861','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','10289','ETRS89/DREF91/2016 / UTM zone 32N (zE-N)',NULL,'EPSG','4400','EPSG','10284','EPSG','4648',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19250','projected_crs','EPSG','10289','EPSG','2861','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','10290','ETRS89/DREF91/2016 / UTM zone 33N (N-zE)',NULL,'EPSG','4500','EPSG','10284','EPSG','5648',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19251','projected_crs','EPSG','10290','EPSG','2862','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','10291','ETRS89/DREF91/2016 / UTM zone 33N (zE-N)',NULL,'EPSG','4400','EPSG','10284','EPSG','5648',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19252','projected_crs','EPSG','10291','EPSG','2862','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','10306','LKS-2020 / Latvia TM',NULL,'EPSG','4530','EPSG','10305','EPSG','19990',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19561','projected_crs','EPSG','10306','EPSG','1139','EPSG','1056');
INSERT INTO "projected_crs" VALUES('EPSG','10314','RGNC15 / Lambert New Caledonia 2015',NULL,'EPSG','4400','EPSG','10310','EPSG','10313',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19643','projected_crs','EPSG','10314','EPSG','3430','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','10315','RGNC15 / UTM zone 57S',NULL,'EPSG','4400','EPSG','10310','EPSG','16157',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19648','projected_crs','EPSG','10315','EPSG','3431','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','10316','RGNC15 / UTM zone 58S',NULL,'EPSG','4400','EPSG','10310','EPSG','16158',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19649','projected_crs','EPSG','10316','EPSG','3432','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','10317','RGNC15 / UTM zone 59S',NULL,'EPSG','4400','EPSG','10310','EPSG','16159',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19650','projected_crs','EPSG','10317','EPSG','3433','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','10329','BH_ETRS89 / TM',NULL,'EPSG','4500','EPSG','10328','EPSG','10325',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19776','projected_crs','EPSG','10329','EPSG','1050','EPSG','1056');
INSERT INTO "projected_crs" VALUES('EPSG','10448','GDA94 / ALB94',NULL,'EPSG','4400','EPSG','4283','EPSG','10424',NULL,0);
INSERT INTO "usage" VALUES('EPSG','20274','projected_crs','EPSG','10448','EPSG','4439','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','10449','GDA94 / BIO94',NULL,'EPSG','4400','EPSG','4283','EPSG','10425',NULL,0);
INSERT INTO "usage" VALUES('EPSG','20273','projected_crs','EPSG','10449','EPSG','4438','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','10450','GDA94 / BRO94',NULL,'EPSG','4400','EPSG','4283','EPSG','10426',NULL,0);
INSERT INTO "usage" VALUES('EPSG','20272','projected_crs','EPSG','10450','EPSG','4441','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','10451','GDA94 / BCG94',NULL,'EPSG','4400','EPSG','4283','EPSG','10427',NULL,0);
INSERT INTO "usage" VALUES('EPSG','20271','projected_crs','EPSG','10451','EPSG','4437','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','10452','GDA94 / CARN94',NULL,'EPSG','4400','EPSG','4283','EPSG','10428',NULL,0);
INSERT INTO "usage" VALUES('EPSG','20270','projected_crs','EPSG','10452','EPSG','4442','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','10453','GDA94 / COL94',NULL,'EPSG','4400','EPSG','4283','EPSG','10429',NULL,0);
INSERT INTO "usage" VALUES('EPSG','20269','projected_crs','EPSG','10453','EPSG','4443','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','10454','GDA94 / ESP94',NULL,'EPSG','4400','EPSG','4283','EPSG','10430',NULL,0);
INSERT INTO "usage" VALUES('EPSG','20268','projected_crs','EPSG','10454','EPSG','4445','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','10455','GDA94 / EXM94',NULL,'EPSG','4400','EPSG','4283','EPSG','10437',NULL,0);
INSERT INTO "usage" VALUES('EPSG','20267','projected_crs','EPSG','10455','EPSG','4448','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','10456','GDA94 / GCG94',NULL,'EPSG','4400','EPSG','4283','EPSG','10438',NULL,0);
INSERT INTO "usage" VALUES('EPSG','20266','projected_crs','EPSG','10456','EPSG','4449','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','10457','GDA94 / GOLD94',NULL,'EPSG','4400','EPSG','4283','EPSG','10439',NULL,0);
INSERT INTO "usage" VALUES('EPSG','20265','projected_crs','EPSG','10457','EPSG','4436','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','10458','GDA94 / JCG94',NULL,'EPSG','4400','EPSG','4283','EPSG','10440',NULL,0);
INSERT INTO "usage" VALUES('EPSG','20264','projected_crs','EPSG','10458','EPSG','4440','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','10459','GDA94 / KALB94',NULL,'EPSG','4400','EPSG','4283','EPSG','10441',NULL,0);
INSERT INTO "usage" VALUES('EPSG','20263','projected_crs','EPSG','10459','EPSG','4444','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','10460','GDA94 / KAR94',NULL,'EPSG','4400','EPSG','4283','EPSG','10442',NULL,0);
INSERT INTO "usage" VALUES('EPSG','20262','projected_crs','EPSG','10460','EPSG','4451','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','10461','GDA94 / KUN94',NULL,'EPSG','4400','EPSG','4283','EPSG','10443',NULL,0);
INSERT INTO "usage" VALUES('EPSG','20261','projected_crs','EPSG','10461','EPSG','4452','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','10462','GDA94 / LCG94',NULL,'EPSG','4400','EPSG','4283','EPSG','10444',NULL,0);
INSERT INTO "usage" VALUES('EPSG','20260','projected_crs','EPSG','10462','EPSG','4453','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','10463','GDA94 / MRCG94',NULL,'EPSG','4400','EPSG','4283','EPSG','10445',NULL,0);
INSERT INTO "usage" VALUES('EPSG','20259','projected_crs','EPSG','10463','EPSG','4457','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','10464','GDA94 / PCG94',NULL,'EPSG','4400','EPSG','4283','EPSG','10446',NULL,0);
INSERT INTO "usage" VALUES('EPSG','20258','projected_crs','EPSG','10464','EPSG','4462','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','10465','GDA94 / PHG94',NULL,'EPSG','4400','EPSG','4283','EPSG','10447',NULL,0);
INSERT INTO "usage" VALUES('EPSG','20257','projected_crs','EPSG','10465','EPSG','4466','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','10471','COV23 Grid',NULL,'EPSG','4400','EPSG','10468','EPSG','10470',NULL,0);
INSERT INTO "usage" VALUES('EPSG','20313','projected_crs','EPSG','10471','EPSG','4743','EPSG','1141');
INSERT INTO "projected_crs" VALUES('EPSG','10477','BBT2000 / BBT-TM',NULL,'EPSG','4400','EPSG','10475','EPSG','10476',NULL,0);
INSERT INTO "usage" VALUES('EPSG','20341','projected_crs','EPSG','10477','EPSG','4744','EPSG','1285');
INSERT INTO "projected_crs" VALUES('EPSG','10480','NAD83 / TWDB GM',NULL,'EPSG','4499','EPSG','4269','EPSG','10479',NULL,1);
INSERT INTO "usage" VALUES('EPSG','20355','projected_crs','EPSG','10480','EPSG','1412','EPSG','1286');
INSERT INTO "projected_crs" VALUES('EPSG','10481','NAD83 / TWDB GM',NULL,'EPSG','4497','EPSG','4269','EPSG','10479',NULL,0);
INSERT INTO "usage" VALUES('EPSG','20356','projected_crs','EPSG','10481','EPSG','1412','EPSG','1286');
INSERT INTO "projected_crs" VALUES('EPSG','10516','NAD83(2011) / Adjusted Jackson (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','10515',NULL,0);
INSERT INTO "usage" VALUES('EPSG','20747','projected_crs','EPSG','10516','EPSG','4343','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','10592','WGS 84 / GLANCE Africa',NULL,'EPSG','4400','EPSG','4326','EPSG','10591',NULL,0);
INSERT INTO "usage" VALUES('EPSG','21204','projected_crs','EPSG','10592','EPSG','4745','EPSG','1287');
INSERT INTO "projected_crs" VALUES('EPSG','10594','WGS 84 / GLANCE Asia',NULL,'EPSG','4400','EPSG','4326','EPSG','10593',NULL,0);
INSERT INTO "usage" VALUES('EPSG','21206','projected_crs','EPSG','10594','EPSG','4747','EPSG','1287');
INSERT INTO "projected_crs" VALUES('EPSG','10596','WGS 84 / GLANCE Europe',NULL,'EPSG','4400','EPSG','4326','EPSG','10595',NULL,0);
INSERT INTO "usage" VALUES('EPSG','21316','projected_crs','EPSG','10596','EPSG','4748','EPSG','1287');
INSERT INTO "projected_crs" VALUES('EPSG','10598','WGS 84 / GLANCE North America',NULL,'EPSG','4400','EPSG','4326','EPSG','10597',NULL,0);
INSERT INTO "usage" VALUES('EPSG','21211','projected_crs','EPSG','10598','EPSG','4749','EPSG','1287');
INSERT INTO "projected_crs" VALUES('EPSG','10601','WGS 84 / GLANCE Oceania',NULL,'EPSG','4400','EPSG','4326','EPSG','10599',NULL,0);
INSERT INTO "usage" VALUES('EPSG','21213','projected_crs','EPSG','10601','EPSG','4751','EPSG','1287');
INSERT INTO "projected_crs" VALUES('EPSG','10603','WGS 84 / GLANCE South America',NULL,'EPSG','4400','EPSG','4326','EPSG','10602',NULL,0);
INSERT INTO "usage" VALUES('EPSG','21215','projected_crs','EPSG','10603','EPSG','4750','EPSG','1287');
INSERT INTO "projected_crs" VALUES('EPSG','10622','NAD83(2011) / San Francisco SFO-B18 (ftUS)',NULL,'EPSG','4497','EPSG','6318','EPSG','10621',NULL,0);
INSERT INTO "usage" VALUES('EPSG','21353','projected_crs','EPSG','10622','EPSG','4529','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','10626','ECML14 Grid',NULL,'EPSG','4400','EPSG','10623','EPSG','10625',NULL,0);
INSERT INTO "usage" VALUES('EPSG','21389','projected_crs','EPSG','10626','EPSG','4774','EPSG','1141');
INSERT INTO "projected_crs" VALUES('EPSG','10632','WC05 Grid',NULL,'EPSG','4400','EPSG','10628','EPSG','10631',NULL,0);
INSERT INTO "usage" VALUES('EPSG','21392','projected_crs','EPSG','10632','EPSG','4775','EPSG','1141');
INSERT INTO "projected_crs" VALUES('EPSG','10641','Saba DPnet',NULL,'EPSG','1054','EPSG','10636','EPSG','10640',NULL,0);
INSERT INTO "usage" VALUES('EPSG','21871','projected_crs','EPSG','10641','EPSG','4757','EPSG','1056');
INSERT INTO "projected_crs" VALUES('EPSG','10665','SIRGAS 2000 / Porto Alegre TM',NULL,'EPSG','4400','EPSG','4674','EPSG','10664',NULL,0);
INSERT INTO "usage" VALUES('EPSG','21625','projected_crs','EPSG','10665','EPSG','4777','EPSG','1056');
INSERT INTO "projected_crs" VALUES('EPSG','10674','RGM23 / UTM zone 38S',NULL,'EPSG','4400','EPSG','10671','EPSG','16138',NULL,0);
INSERT INTO "usage" VALUES('EPSG','21843','projected_crs','EPSG','10674','EPSG','1159','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','10699','EUREF-FIN / UTM zone 34N',NULL,'EPSG','4400','EPSG','10690','EPSG','16034',NULL,0);
INSERT INTO "usage" VALUES('EPSG','22020','projected_crs','EPSG','10699','EPSG','4781','EPSG','1227');
INSERT INTO "projected_crs" VALUES('EPSG','10702','EUREF-FIN / UTM zone 36N',NULL,'EPSG','4400','EPSG','10690','EPSG','16036',NULL,0);
INSERT INTO "usage" VALUES('EPSG','22021','projected_crs','EPSG','10702','EPSG','4783','EPSG','1227');
INSERT INTO "projected_crs" VALUES('EPSG','10726','UZGD2024 / UzREF24 zone 40',NULL,'EPSG','4400','EPSG','10725','EPSG','10719',NULL,0);
INSERT INTO "usage" VALUES('EPSG','22096','projected_crs','EPSG','10726','EPSG','4784','EPSG','1207');
INSERT INTO "projected_crs" VALUES('EPSG','10727','UZGD2024 / UzREF24 zone 41',NULL,'EPSG','4400','EPSG','10725','EPSG','10720',NULL,0);
INSERT INTO "usage" VALUES('EPSG','22097','projected_crs','EPSG','10727','EPSG','4785','EPSG','1207');
INSERT INTO "projected_crs" VALUES('EPSG','10728','UZGD2024 / UzREF24 zone 42',NULL,'EPSG','4400','EPSG','10725','EPSG','10721',NULL,0);
INSERT INTO "usage" VALUES('EPSG','22098','projected_crs','EPSG','10728','EPSG','4786','EPSG','1207');
INSERT INTO "projected_crs" VALUES('EPSG','10729','UZGD2024 / UzREF24 zone 43',NULL,'EPSG','4400','EPSG','10725','EPSG','10722',NULL,0);
INSERT INTO "usage" VALUES('EPSG','22099','projected_crs','EPSG','10729','EPSG','4787','EPSG','1207');
INSERT INTO "projected_crs" VALUES('EPSG','10731','ETRS89/DREF91/2016 / UTM zone 31N',NULL,'EPSG','4400','EPSG','10284','EPSG','16031',NULL,0);
INSERT INTO "usage" VALUES('EPSG','20592','projected_crs','EPSG','10731','EPSG','2860','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','10732','ETRS89/DREF91/2016 / UTM zone 32N',NULL,'EPSG','4400','EPSG','10284','EPSG','16032',NULL,0);
INSERT INTO "usage" VALUES('EPSG','20896','projected_crs','EPSG','10732','EPSG','2861','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','10733','ETRS89/DREF91/2016 / UTM zone 33N',NULL,'EPSG','4400','EPSG','10284','EPSG','16033',NULL,0);
INSERT INTO "usage" VALUES('EPSG','20594','projected_crs','EPSG','10733','EPSG','2862','EPSG','1054');
INSERT INTO "projected_crs" VALUES('EPSG','10744','Sint Eustatius DPnet short',NULL,'EPSG','1054','EPSG','10736','EPSG','10743',NULL,0);
INSERT INTO "usage" VALUES('EPSG','22317','projected_crs','EPSG','10744','EPSG','4788','EPSG','1056');
INSERT INTO "projected_crs" VALUES('EPSG','10745','Sint Eustatius DPnet long',NULL,'EPSG','1054','EPSG','10736','EPSG','16020',NULL,0);
INSERT INTO "usage" VALUES('EPSG','22316','projected_crs','EPSG','10745','EPSG','4788','EPSG','1056');
INSERT INTO "projected_crs" VALUES('EPSG','10759','Bonaire DPnet',NULL,'EPSG','1054','EPSG','10758','EPSG','10757',NULL,0);
INSERT INTO "usage" VALUES('EPSG','22169','projected_crs','EPSG','10759','EPSG','3822','EPSG','1056');
INSERT INTO "projected_crs" VALUES('EPSG','10773','SIRGAS 2000 / Ribeirao Preto Local TM',NULL,'EPSG','4400','EPSG','4674','EPSG','10772',NULL,0);
INSERT INTO "usage" VALUES('EPSG','22381','projected_crs','EPSG','10773','EPSG','4789','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','10788','WGS 84 / Agriculture Canada Albers',NULL,'EPSG','4400','EPSG','4326','EPSG','10751',NULL,1);
INSERT INTO "usage" VALUES('EPSG','22473','projected_crs','EPSG','10788','EPSG','4752','EPSG','1292');
INSERT INTO "projected_crs" VALUES('EPSG','10792','UGRF / UTM zone 35N',NULL,'EPSG','4400','EPSG','10791','EPSG','16035',NULL,0);
INSERT INTO "usage" VALUES('EPSG','22460','projected_crs','EPSG','10792','EPSG','1578','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','10793','UGRF / UTM zone 36N',NULL,'EPSG','4400','EPSG','10791','EPSG','16036',NULL,0);
INSERT INTO "usage" VALUES('EPSG','22461','projected_crs','EPSG','10793','EPSG','4790','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','10794','UGRF / UTM zone 36S',NULL,'EPSG','4400','EPSG','10791','EPSG','16136',NULL,0);
INSERT INTO "usage" VALUES('EPSG','22462','projected_crs','EPSG','10794','EPSG','4791','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','10795','UGRF / UTM zone 35S',NULL,'EPSG','4400','EPSG','10791','EPSG','16135',NULL,0);
INSERT INTO "usage" VALUES('EPSG','22463','projected_crs','EPSG','10795','EPSG','4792','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','10801','LibRef21 / UTM zone 28N',NULL,'EPSG','4400','EPSG','10800','EPSG','16028',NULL,0);
INSERT INTO "usage" VALUES('EPSG','22447','projected_crs','EPSG','10801','EPSG','4794','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','10802','LibRef21 / UTM zone 29N',NULL,'EPSG','4400','EPSG','10800','EPSG','16029',NULL,0);
INSERT INTO "usage" VALUES('EPSG','22448','projected_crs','EPSG','10802','EPSG','4793','EPSG','1056');
INSERT INTO "projected_crs" VALUES('EPSG','10820','WGS 84 / Agriculture Canada Albers',NULL,'EPSG','4400','EPSG','4326','EPSG','10819',NULL,0);
INSERT INTO "usage" VALUES('EPSG','22545','projected_crs','EPSG','10820','EPSG','4752','EPSG','1292');
INSERT INTO "projected_crs" VALUES('EPSG','10833','Georgia Geodetic Datum / Lambert',NULL,'EPSG','4500','EPSG','10831','EPSG','10832',NULL,0);
INSERT INTO "usage" VALUES('EPSG','22793','projected_crs','EPSG','10833','EPSG','3251','EPSG','1091');
INSERT INTO "projected_crs" VALUES('EPSG','10836','Georgia Geodetic Datum / UTM zone 37N (N-E)',NULL,'EPSG','4500','EPSG','10831','EPSG','16037',NULL,0);
INSERT INTO "usage" VALUES('EPSG','22796','projected_crs','EPSG','10836','EPSG','4797','EPSG','1026');
INSERT INTO "projected_crs" VALUES('EPSG','10837','Georgia Geodetic Datum / UTM zone 38N (N-E)',NULL,'EPSG','4500','EPSG','10831','EPSG','16038',NULL,0);
INSERT INTO "usage" VALUES('EPSG','22797','projected_crs','EPSG','10837','EPSG','4798','EPSG','1026');
INSERT INTO "projected_crs" VALUES('EPSG','10851','EWR3 Grid',NULL,'EPSG','4400','EPSG','10849','EPSG','9765',NULL,0);
INSERT INTO "usage" VALUES('EPSG','22761','projected_crs','EPSG','10851','EPSG','4799','EPSG','1141');
INSERT INTO "projected_crs" VALUES('EPSG','10857','SIRGAS 2000 / Brazil Albers',NULL,'EPSG','4400','EPSG','4674','EPSG','10856',NULL,0);
INSERT INTO "usage" VALUES('EPSG','22838','projected_crs','EPSG','10857','EPSG','3223','EPSG','1162');
INSERT INTO "projected_crs" VALUES('EPSG','10863','WSPG Grid',NULL,'EPSG','4400','EPSG','10860','EPSG','10862',NULL,0);
INSERT INTO "usage" VALUES('EPSG','22835','projected_crs','EPSG','10863','EPSG','4801','EPSG','1293');
INSERT INTO "projected_crs" VALUES('EPSG','10899','Asse 2025 / 3-degree Gauss-Kruger zone 4 (E-N)',NULL,'EPSG','4400','EPSG','10898','EPSG','16264',NULL,0);
INSERT INTO "usage" VALUES('EPSG','23102','projected_crs','EPSG','10899','EPSG','4805','EPSG','1144');
INSERT INTO "projected_crs" VALUES('EPSG','10911','CSRN2025 (NAD83 2011) / California zone 1 (ftUS)',NULL,'EPSG','4497','EPSG','10910','EPSG','15307',NULL,0);
INSERT INTO "usage" VALUES('EPSG','23583','projected_crs','EPSG','10911','EPSG','2175','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','10912','CSRN2025 (NAD83 2011) / California zone 2 (ftUS)',NULL,'EPSG','4497','EPSG','10910','EPSG','15308',NULL,0);
INSERT INTO "usage" VALUES('EPSG','23582','projected_crs','EPSG','10912','EPSG','2176','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','10913','CSRN2025 (NAD83 2011) / California zone 3 (ftUS)',NULL,'EPSG','4497','EPSG','10910','EPSG','15309',NULL,0);
INSERT INTO "usage" VALUES('EPSG','23581','projected_crs','EPSG','10913','EPSG','2177','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','10914','CSRN2025 (NAD83 2011) / California zone 4 (ftUS)',NULL,'EPSG','4497','EPSG','10910','EPSG','15310',NULL,0);
INSERT INTO "usage" VALUES('EPSG','23580','projected_crs','EPSG','10914','EPSG','2178','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','10915','CSRN2025 (NAD83 2011) / California zone 5 (ftUS)',NULL,'EPSG','4497','EPSG','10910','EPSG','15311',NULL,0);
INSERT INTO "usage" VALUES('EPSG','23588','projected_crs','EPSG','10915','EPSG','2182','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','10916','CSRN2025 (NAD83 2011) / California zone 6 (ftUS)',NULL,'EPSG','4497','EPSG','10910','EPSG','15312',NULL,0);
INSERT INTO "usage" VALUES('EPSG','23589','projected_crs','EPSG','10916','EPSG','2180','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','10917','CSRN2025 (NAD83 2011) / California Albers',NULL,'EPSG','4499','EPSG','10910','EPSG','10420',NULL,0);
INSERT INTO "usage" VALUES('EPSG','23590','projected_crs','EPSG','10917','EPSG','1375','EPSG','1135');
INSERT INTO "projected_crs" VALUES('EPSG','10921','CSRN2025 (NAD83 2011) / California zone 1',NULL,'EPSG','4499','EPSG','10910','EPSG','10431',NULL,0);
INSERT INTO "usage" VALUES('EPSG','23591','projected_crs','EPSG','10921','EPSG','2175','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','10922','CSRN2025 (NAD83 2011) / California zone 2',NULL,'EPSG','4499','EPSG','10910','EPSG','10432',NULL,0);
INSERT INTO "usage" VALUES('EPSG','23592','projected_crs','EPSG','10922','EPSG','2176','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','10923','CSRN2025 (NAD83 2011) / California zone 3',NULL,'EPSG','4499','EPSG','10910','EPSG','10433',NULL,0);
INSERT INTO "usage" VALUES('EPSG','23593','projected_crs','EPSG','10923','EPSG','2177','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','10924','CSRN2025 (NAD83 2011) / California zone 4',NULL,'EPSG','4499','EPSG','10910','EPSG','10434',NULL,0);
INSERT INTO "usage" VALUES('EPSG','23594','projected_crs','EPSG','10924','EPSG','2178','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','10925','CSRN2025 (NAD83 2011) / California zone 5',NULL,'EPSG','4499','EPSG','10910','EPSG','10435',NULL,0);
INSERT INTO "usage" VALUES('EPSG','23579','projected_crs','EPSG','10925','EPSG','2182','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','10926','CSRN2025 (NAD83 2011) / California zone 6',NULL,'EPSG','4499','EPSG','10910','EPSG','10436',NULL,0);
INSERT INTO "usage" VALUES('EPSG','23578','projected_crs','EPSG','10926','EPSG','2180','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','10942','QazTRF-23 / Gauss-Kruger zone 8',NULL,'EPSG','4400','EPSG','10941','EPSG','16208',NULL,0);
INSERT INTO "usage" VALUES('EPSG','23491','projected_crs','EPSG','10942','EPSG','4813','EPSG','1056');
INSERT INTO "projected_crs" VALUES('EPSG','10943','QazTRF-23 / Gauss-Kruger zone 9',NULL,'EPSG','4400','EPSG','10941','EPSG','16209',NULL,0);
INSERT INTO "usage" VALUES('EPSG','23492','projected_crs','EPSG','10943','EPSG','4814','EPSG','1056');
INSERT INTO "projected_crs" VALUES('EPSG','10944','QazTRF-23 / Gauss-Kruger zone 10',NULL,'EPSG','4400','EPSG','10941','EPSG','16210',NULL,0);
INSERT INTO "usage" VALUES('EPSG','23493','projected_crs','EPSG','10944','EPSG','4815','EPSG','1056');
INSERT INTO "projected_crs" VALUES('EPSG','10945','QazTRF-23 / Gauss-Kruger zone 11',NULL,'EPSG','4400','EPSG','10941','EPSG','16211',NULL,0);
INSERT INTO "usage" VALUES('EPSG','23494','projected_crs','EPSG','10945','EPSG','4816','EPSG','1056');
INSERT INTO "projected_crs" VALUES('EPSG','10946','QazTRF-23 / Gauss-Kruger zone 12',NULL,'EPSG','4400','EPSG','10941','EPSG','16212',NULL,0);
INSERT INTO "usage" VALUES('EPSG','23495','projected_crs','EPSG','10946','EPSG','4817','EPSG','1056');
INSERT INTO "projected_crs" VALUES('EPSG','10947','QazTRF-23 / Gauss-Kruger zone 13',NULL,'EPSG','4400','EPSG','10941','EPSG','16213',NULL,0);
INSERT INTO "usage" VALUES('EPSG','23496','projected_crs','EPSG','10947','EPSG','4818','EPSG','1056');
INSERT INTO "projected_crs" VALUES('EPSG','10948','QazTRF-23 / Gauss-Kruger zone 14',NULL,'EPSG','4400','EPSG','10941','EPSG','16214',NULL,0);
INSERT INTO "usage" VALUES('EPSG','23497','projected_crs','EPSG','10948','EPSG','4819','EPSG','1056');
INSERT INTO "projected_crs" VALUES('EPSG','10949','QazTRF-23 / Gauss-Kruger zone 15',NULL,'EPSG','4400','EPSG','10941','EPSG','16215',NULL,0);
INSERT INTO "usage" VALUES('EPSG','23498','projected_crs','EPSG','10949','EPSG','4820','EPSG','1056');
INSERT INTO "projected_crs" VALUES('EPSG','10979','NATRF2022 / Gulf',NULL,'EPSG','4500','EPSG','10968','EPSG','10970',NULL,0);
INSERT INTO "usage" VALUES('EPSG','23662','projected_crs','EPSG','10979','EPSG','3357','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','10980','NATRF2022 / Gulf (ft)',NULL,'EPSG','1029','EPSG','10968','EPSG','10971',NULL,0);
INSERT INTO "usage" VALUES('EPSG','23636','projected_crs','EPSG','10980','EPSG','3357','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','10981','NATRF2022 / Gulf Louisiana Shelf (ft)',NULL,'EPSG','4495','EPSG','10968','EPSG','10972',NULL,0);
INSERT INTO "usage" VALUES('EPSG','23677','projected_crs','EPSG','10981','EPSG','4809','EPSG','1294');
INSERT INTO "projected_crs" VALUES('EPSG','10982','NATRF2022 / Gulf Texas Shelf North (ft)',NULL,'EPSG','4495','EPSG','10968','EPSG','10973',NULL,0);
INSERT INTO "usage" VALUES('EPSG','23678','projected_crs','EPSG','10982','EPSG','4808','EPSG','1294');
INSERT INTO "projected_crs" VALUES('EPSG','10983','NATRF2022 / Gulf Texas Shelf South (ft)',NULL,'EPSG','4495','EPSG','10968','EPSG','10974',NULL,0);
INSERT INTO "usage" VALUES('EPSG','23679','projected_crs','EPSG','10983','EPSG','4807','EPSG','1294');
INSERT INTO "projected_crs" VALUES('EPSG','10984','NATRF2022 / Gulf West (ft)',NULL,'EPSG','4495','EPSG','10968','EPSG','10975',NULL,0);
INSERT INTO "usage" VALUES('EPSG','23680','projected_crs','EPSG','10984','EPSG','4810','EPSG','1294');
INSERT INTO "projected_crs" VALUES('EPSG','10985','NATRF2022 / Gulf West Central (ft)',NULL,'EPSG','4495','EPSG','10968','EPSG','10976',NULL,0);
INSERT INTO "usage" VALUES('EPSG','23681','projected_crs','EPSG','10985','EPSG','4811','EPSG','1294');
INSERT INTO "projected_crs" VALUES('EPSG','10986','NATRF2022 / Gulf East Central (ft)',NULL,'EPSG','4495','EPSG','10968','EPSG','10977',NULL,0);
INSERT INTO "usage" VALUES('EPSG','23682','projected_crs','EPSG','10986','EPSG','4812','EPSG','1294');
INSERT INTO "projected_crs" VALUES('EPSG','10987','NATRF2022 / Gulf East (ft)',NULL,'EPSG','4495','EPSG','10968','EPSG','10978',NULL,0);
INSERT INTO "usage" VALUES('EPSG','23683','projected_crs','EPSG','10987','EPSG','2174','EPSG','1294');
INSERT INTO "projected_crs" VALUES('EPSG','11012','ETRS89-NOR [EUREF89] / UTM zone 30N (N-E)',NULL,'EPSG','4500','EPSG','10875','EPSG','16030',NULL,0);
INSERT INTO "usage" VALUES('EPSG','24106','projected_crs','EPSG','11012','EPSG','4823','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','11013','ETRS89-NOR [EUREF89] / UTM zone 31N (N-E)',NULL,'EPSG','4500','EPSG','10875','EPSG','16031',NULL,0);
INSERT INTO "usage" VALUES('EPSG','24107','projected_crs','EPSG','11013','EPSG','4824','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','11014','ETRS89-NOR [EUREF89] / UTM zone 32N (N-E)',NULL,'EPSG','4500','EPSG','10875','EPSG','16032',NULL,0);
INSERT INTO "usage" VALUES('EPSG','24108','projected_crs','EPSG','11014','EPSG','4825','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','11015','ETRS89-NOR [EUREF89] / UTM zone 33N (N-E)',NULL,'EPSG','4500','EPSG','10875','EPSG','16033',NULL,0);
INSERT INTO "usage" VALUES('EPSG','24109','projected_crs','EPSG','11015','EPSG','4826','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','11016','ETRS89-NOR [EUREF89] / UTM zone 34N (N-E)',NULL,'EPSG','4500','EPSG','10875','EPSG','16034',NULL,0);
INSERT INTO "usage" VALUES('EPSG','24110','projected_crs','EPSG','11016','EPSG','4827','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','11017','ETRS89-NOR [EUREF89] / UTM zone 35N (N-E)',NULL,'EPSG','4500','EPSG','10875','EPSG','16035',NULL,0);
INSERT INTO "usage" VALUES('EPSG','24111','projected_crs','EPSG','11017','EPSG','4828','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','11018','ETRS89-NOR [EUREF89] / UTM zone 36N (N-E)',NULL,'EPSG','4500','EPSG','10875','EPSG','16036',NULL,0);
INSERT INTO "usage" VALUES('EPSG','24112','projected_crs','EPSG','11018','EPSG','4829','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','11019','ETRS89-NOR [EUREF89] / UTM zone 37N (N-E)',NULL,'EPSG','4500','EPSG','10875','EPSG','16037',NULL,0);
INSERT INTO "usage" VALUES('EPSG','24113','projected_crs','EPSG','11019','EPSG','4830','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','11020','ETRS89-NOR [EUREF89] / UTM zone 30N',NULL,'EPSG','4400','EPSG','10875','EPSG','16030',NULL,0);
INSERT INTO "usage" VALUES('EPSG','24114','projected_crs','EPSG','11020','EPSG','4823','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','11021','ETRS89-NOR [EUREF89] / UTM zone 31N',NULL,'EPSG','4400','EPSG','10875','EPSG','16031',NULL,0);
INSERT INTO "usage" VALUES('EPSG','24115','projected_crs','EPSG','11021','EPSG','4824','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','11022','ETRS89-NOR [EUREF89] / UTM zone 32N',NULL,'EPSG','4400','EPSG','10875','EPSG','16032',NULL,0);
INSERT INTO "usage" VALUES('EPSG','24117','projected_crs','EPSG','11022','EPSG','4825','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','11023','ETRS89-NOR [EUREF89] / UTM zone 33N',NULL,'EPSG','4400','EPSG','10875','EPSG','16033',NULL,0);
INSERT INTO "usage" VALUES('EPSG','24118','projected_crs','EPSG','11023','EPSG','4826','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','11024','ETRS89-NOR [EUREF89] / UTM zone 34N',NULL,'EPSG','4400','EPSG','10875','EPSG','16034',NULL,0);
INSERT INTO "usage" VALUES('EPSG','24119','projected_crs','EPSG','11024','EPSG','4827','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','11025','ETRS89-NOR [EUREF89] / UTM zone 35N',NULL,'EPSG','4400','EPSG','10875','EPSG','16035',NULL,0);
INSERT INTO "usage" VALUES('EPSG','24120','projected_crs','EPSG','11025','EPSG','4828','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','11026','ETRS89-NOR [EUREF89] / UTM zone 36N',NULL,'EPSG','4400','EPSG','10875','EPSG','16036',NULL,0);
INSERT INTO "usage" VALUES('EPSG','24121','projected_crs','EPSG','11026','EPSG','4829','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','11027','ETRS89-NOR [EUREF89] / UTM zone 37N',NULL,'EPSG','4400','EPSG','10875','EPSG','16037',NULL,0);
INSERT INTO "usage" VALUES('EPSG','24396','projected_crs','EPSG','11027','EPSG','4830','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','11114','MAGNA-SIRGAS 2018 / Colombia Far West zone',NULL,'EPSG','4500','EPSG','20046','EPSG','18065',NULL,0);
INSERT INTO "usage" VALUES('EPSG','18947','projected_crs','EPSG','11114','EPSG','3091','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','11115','MAGNA-SIRGAS 2018 / Colombia West zone',NULL,'EPSG','4500','EPSG','20046','EPSG','18066',NULL,0);
INSERT INTO "usage" VALUES('EPSG','18948','projected_crs','EPSG','11115','EPSG','3090','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','11116','MAGNA-SIRGAS 2018 / Colombia Bogota zone',NULL,'EPSG','4500','EPSG','20046','EPSG','18067',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19037','projected_crs','EPSG','11116','EPSG','1599','EPSG','1208');
INSERT INTO "projected_crs" VALUES('EPSG','11117','MAGNA-SIRGAS 2018 / Colombia East Central zone',NULL,'EPSG','4500','EPSG','20046','EPSG','18068',NULL,0);
INSERT INTO "usage" VALUES('EPSG','18950','projected_crs','EPSG','11117','EPSG','1600','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','11118','MAGNA-SIRGAS 2018 / Colombia East zone',NULL,'EPSG','4500','EPSG','20046','EPSG','18069',NULL,0);
INSERT INTO "usage" VALUES('EPSG','18951','projected_crs','EPSG','11118','EPSG','1601','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','20002','MWC18 Grid',NULL,'EPSG','4400','EPSG','20033','EPSG','10127',NULL,0);
INSERT INTO "usage" VALUES('EPSG','18336','projected_crs','EPSG','20002','EPSG','4666','EPSG','1141');
INSERT INTO "projected_crs" VALUES('EPSG','20004','Pulkovo 1995 / Gauss-Kruger zone 4',NULL,'EPSG','4530','EPSG','4200','EPSG','16204',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6177','projected_crs','EPSG','20004','EPSG','1763','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','20005','Pulkovo 1995 / Gauss-Kruger zone 5',NULL,'EPSG','4530','EPSG','4200','EPSG','16205',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6178','projected_crs','EPSG','20005','EPSG','1764','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','20006','Pulkovo 1995 / Gauss-Kruger zone 6',NULL,'EPSG','4530','EPSG','4200','EPSG','16206',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6179','projected_crs','EPSG','20006','EPSG','1765','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','20007','Pulkovo 1995 / Gauss-Kruger zone 7',NULL,'EPSG','4530','EPSG','4200','EPSG','16207',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6180','projected_crs','EPSG','20007','EPSG','1766','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','20008','Pulkovo 1995 / Gauss-Kruger zone 8',NULL,'EPSG','4530','EPSG','4200','EPSG','16208',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6181','projected_crs','EPSG','20008','EPSG','1767','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','20009','Pulkovo 1995 / Gauss-Kruger zone 9',NULL,'EPSG','4530','EPSG','4200','EPSG','16209',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6182','projected_crs','EPSG','20009','EPSG','1768','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','20010','Pulkovo 1995 / Gauss-Kruger zone 10',NULL,'EPSG','4530','EPSG','4200','EPSG','16210',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6183','projected_crs','EPSG','20010','EPSG','1769','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','20011','Pulkovo 1995 / Gauss-Kruger zone 11',NULL,'EPSG','4530','EPSG','4200','EPSG','16211',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6184','projected_crs','EPSG','20011','EPSG','1770','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','20012','Pulkovo 1995 / Gauss-Kruger zone 12',NULL,'EPSG','4530','EPSG','4200','EPSG','16212',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6185','projected_crs','EPSG','20012','EPSG','1771','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','20013','Pulkovo 1995 / Gauss-Kruger zone 13',NULL,'EPSG','4530','EPSG','4200','EPSG','16213',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6186','projected_crs','EPSG','20013','EPSG','1772','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','20014','Pulkovo 1995 / Gauss-Kruger zone 14',NULL,'EPSG','4530','EPSG','4200','EPSG','16214',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6187','projected_crs','EPSG','20014','EPSG','1773','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','20015','Pulkovo 1995 / Gauss-Kruger zone 15',NULL,'EPSG','4530','EPSG','4200','EPSG','16215',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6188','projected_crs','EPSG','20015','EPSG','1774','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','20016','Pulkovo 1995 / Gauss-Kruger zone 16',NULL,'EPSG','4530','EPSG','4200','EPSG','16216',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6189','projected_crs','EPSG','20016','EPSG','1775','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','20017','Pulkovo 1995 / Gauss-Kruger zone 17',NULL,'EPSG','4530','EPSG','4200','EPSG','16217',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6190','projected_crs','EPSG','20017','EPSG','1776','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','20018','Pulkovo 1995 / Gauss-Kruger zone 18',NULL,'EPSG','4530','EPSG','4200','EPSG','16218',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6191','projected_crs','EPSG','20018','EPSG','1777','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','20019','Pulkovo 1995 / Gauss-Kruger zone 19',NULL,'EPSG','4530','EPSG','4200','EPSG','16219',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6192','projected_crs','EPSG','20019','EPSG','1778','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','20020','Pulkovo 1995 / Gauss-Kruger zone 20',NULL,'EPSG','4530','EPSG','4200','EPSG','16220',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6193','projected_crs','EPSG','20020','EPSG','1779','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','20021','Pulkovo 1995 / Gauss-Kruger zone 21',NULL,'EPSG','4530','EPSG','4200','EPSG','16221',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6194','projected_crs','EPSG','20021','EPSG','1780','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','20022','Pulkovo 1995 / Gauss-Kruger zone 22',NULL,'EPSG','4530','EPSG','4200','EPSG','16222',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6195','projected_crs','EPSG','20022','EPSG','1781','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','20023','Pulkovo 1995 / Gauss-Kruger zone 23',NULL,'EPSG','4530','EPSG','4200','EPSG','16223',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6196','projected_crs','EPSG','20023','EPSG','1782','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','20024','Pulkovo 1995 / Gauss-Kruger zone 24',NULL,'EPSG','4530','EPSG','4200','EPSG','16224',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6197','projected_crs','EPSG','20024','EPSG','1783','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','20025','Pulkovo 1995 / Gauss-Kruger zone 25',NULL,'EPSG','4530','EPSG','4200','EPSG','16225',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6198','projected_crs','EPSG','20025','EPSG','1784','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','20026','Pulkovo 1995 / Gauss-Kruger zone 26',NULL,'EPSG','4530','EPSG','4200','EPSG','16226',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6199','projected_crs','EPSG','20026','EPSG','1785','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','20027','Pulkovo 1995 / Gauss-Kruger zone 27',NULL,'EPSG','4530','EPSG','4200','EPSG','16227',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6200','projected_crs','EPSG','20027','EPSG','1786','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','20028','Pulkovo 1995 / Gauss-Kruger zone 28',NULL,'EPSG','4530','EPSG','4200','EPSG','16228',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6201','projected_crs','EPSG','20028','EPSG','1787','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','20029','Pulkovo 1995 / Gauss-Kruger zone 29',NULL,'EPSG','4530','EPSG','4200','EPSG','16229',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6202','projected_crs','EPSG','20029','EPSG','1788','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','20030','Pulkovo 1995 / Gauss-Kruger zone 30',NULL,'EPSG','4530','EPSG','4200','EPSG','16230',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6203','projected_crs','EPSG','20030','EPSG','1789','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','20031','Pulkovo 1995 / Gauss-Kruger zone 31',NULL,'EPSG','4530','EPSG','4200','EPSG','16231',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6204','projected_crs','EPSG','20031','EPSG','1790','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','20032','Pulkovo 1995 / Gauss-Kruger zone 32',NULL,'EPSG','4530','EPSG','4200','EPSG','16232',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6205','projected_crs','EPSG','20032','EPSG','1791','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','20042','SIRGAS-Chile 2021 / UTM zone 12S',NULL,'EPSG','4400','EPSG','20041','EPSG','16112',NULL,0);
INSERT INTO "usage" VALUES('EPSG','18383','projected_crs','EPSG','20042','EPSG','4667','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','20047','GDA2020 / BCSG2020',NULL,'EPSG','4400','EPSG','7844','EPSG','10147',NULL,0);
INSERT INTO "usage" VALUES('EPSG','18477','projected_crs','EPSG','20047','EPSG','2990','EPSG','1029');
INSERT INTO "projected_crs" VALUES('EPSG','20048','SIRGAS-Chile 2021 / UTM zone 18S',NULL,'EPSG','4400','EPSG','20041','EPSG','16118',NULL,0);
INSERT INTO "usage" VALUES('EPSG','18384','projected_crs','EPSG','20048','EPSG','3829','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','20049','SIRGAS-Chile 2021 / UTM zone 19S',NULL,'EPSG','4400','EPSG','20041','EPSG','16119',NULL,0);
INSERT INTO "usage" VALUES('EPSG','18385','projected_crs','EPSG','20049','EPSG','3811','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','20050','NAD83(2011) / Amtrak NECCS21 (ft)',NULL,'EPSG','4495','EPSG','6318','EPSG','10148',NULL,0);
INSERT INTO "usage" VALUES('EPSG','18480','projected_crs','EPSG','20050','EPSG','4669','EPSG','1141');
INSERT INTO "projected_crs" VALUES('EPSG','20064','Pulkovo 1995 / Gauss-Kruger 4N',NULL,'EPSG','4530','EPSG','4200','EPSG','16304',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6206','projected_crs','EPSG','20064','EPSG','1763','EPSG','1209');
INSERT INTO "projected_crs" VALUES('EPSG','20065','Pulkovo 1995 / Gauss-Kruger 5N',NULL,'EPSG','4530','EPSG','4200','EPSG','16305',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6207','projected_crs','EPSG','20065','EPSG','1764','EPSG','1209');
INSERT INTO "projected_crs" VALUES('EPSG','20066','Pulkovo 1995 / Gauss-Kruger 6N',NULL,'EPSG','4530','EPSG','4200','EPSG','16306',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6208','projected_crs','EPSG','20066','EPSG','1765','EPSG','1209');
INSERT INTO "projected_crs" VALUES('EPSG','20067','Pulkovo 1995 / Gauss-Kruger 7N',NULL,'EPSG','4530','EPSG','4200','EPSG','16307',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6209','projected_crs','EPSG','20067','EPSG','1766','EPSG','1209');
INSERT INTO "projected_crs" VALUES('EPSG','20068','Pulkovo 1995 / Gauss-Kruger 8N',NULL,'EPSG','4530','EPSG','4200','EPSG','16308',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6210','projected_crs','EPSG','20068','EPSG','1767','EPSG','1209');
INSERT INTO "projected_crs" VALUES('EPSG','20069','Pulkovo 1995 / Gauss-Kruger 9N',NULL,'EPSG','4530','EPSG','4200','EPSG','16309',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6211','projected_crs','EPSG','20069','EPSG','1768','EPSG','1209');
INSERT INTO "projected_crs" VALUES('EPSG','20070','Pulkovo 1995 / Gauss-Kruger 10N',NULL,'EPSG','4530','EPSG','4200','EPSG','16310',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6212','projected_crs','EPSG','20070','EPSG','1769','EPSG','1209');
INSERT INTO "projected_crs" VALUES('EPSG','20071','Pulkovo 1995 / Gauss-Kruger 11N',NULL,'EPSG','4530','EPSG','4200','EPSG','16311',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6213','projected_crs','EPSG','20071','EPSG','1770','EPSG','1209');
INSERT INTO "projected_crs" VALUES('EPSG','20072','Pulkovo 1995 / Gauss-Kruger 12N',NULL,'EPSG','4530','EPSG','4200','EPSG','16312',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6214','projected_crs','EPSG','20072','EPSG','1771','EPSG','1209');
INSERT INTO "projected_crs" VALUES('EPSG','20073','Pulkovo 1995 / Gauss-Kruger 13N',NULL,'EPSG','4530','EPSG','4200','EPSG','16313',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6215','projected_crs','EPSG','20073','EPSG','1772','EPSG','1209');
INSERT INTO "projected_crs" VALUES('EPSG','20074','Pulkovo 1995 / Gauss-Kruger 14N',NULL,'EPSG','4530','EPSG','4200','EPSG','16314',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6216','projected_crs','EPSG','20074','EPSG','1773','EPSG','1209');
INSERT INTO "projected_crs" VALUES('EPSG','20075','Pulkovo 1995 / Gauss-Kruger 15N',NULL,'EPSG','4530','EPSG','4200','EPSG','16315',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6217','projected_crs','EPSG','20075','EPSG','1774','EPSG','1209');
INSERT INTO "projected_crs" VALUES('EPSG','20076','Pulkovo 1995 / Gauss-Kruger 16N',NULL,'EPSG','4530','EPSG','4200','EPSG','16316',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6218','projected_crs','EPSG','20076','EPSG','1775','EPSG','1209');
INSERT INTO "projected_crs" VALUES('EPSG','20077','Pulkovo 1995 / Gauss-Kruger 17N',NULL,'EPSG','4530','EPSG','4200','EPSG','16317',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6219','projected_crs','EPSG','20077','EPSG','1776','EPSG','1209');
INSERT INTO "projected_crs" VALUES('EPSG','20078','Pulkovo 1995 / Gauss-Kruger 18N',NULL,'EPSG','4530','EPSG','4200','EPSG','16318',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6220','projected_crs','EPSG','20078','EPSG','1777','EPSG','1209');
INSERT INTO "projected_crs" VALUES('EPSG','20079','Pulkovo 1995 / Gauss-Kruger 19N',NULL,'EPSG','4530','EPSG','4200','EPSG','16319',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6221','projected_crs','EPSG','20079','EPSG','1778','EPSG','1209');
INSERT INTO "projected_crs" VALUES('EPSG','20080','Pulkovo 1995 / Gauss-Kruger 20N',NULL,'EPSG','4530','EPSG','4200','EPSG','16320',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6222','projected_crs','EPSG','20080','EPSG','1779','EPSG','1209');
INSERT INTO "projected_crs" VALUES('EPSG','20081','Pulkovo 1995 / Gauss-Kruger 21N',NULL,'EPSG','4530','EPSG','4200','EPSG','16321',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6223','projected_crs','EPSG','20081','EPSG','1780','EPSG','1209');
INSERT INTO "projected_crs" VALUES('EPSG','20082','Pulkovo 1995 / Gauss-Kruger 22N',NULL,'EPSG','4530','EPSG','4200','EPSG','16322',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6224','projected_crs','EPSG','20082','EPSG','1781','EPSG','1209');
INSERT INTO "projected_crs" VALUES('EPSG','20083','Pulkovo 1995 / Gauss-Kruger 23N',NULL,'EPSG','4530','EPSG','4200','EPSG','16323',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6225','projected_crs','EPSG','20083','EPSG','1782','EPSG','1209');
INSERT INTO "projected_crs" VALUES('EPSG','20084','Pulkovo 1995 / Gauss-Kruger 24N',NULL,'EPSG','4530','EPSG','4200','EPSG','16324',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6226','projected_crs','EPSG','20084','EPSG','1783','EPSG','1209');
INSERT INTO "projected_crs" VALUES('EPSG','20085','Pulkovo 1995 / Gauss-Kruger 25N',NULL,'EPSG','4530','EPSG','4200','EPSG','16325',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6227','projected_crs','EPSG','20085','EPSG','1784','EPSG','1209');
INSERT INTO "projected_crs" VALUES('EPSG','20086','Pulkovo 1995 / Gauss-Kruger 26N',NULL,'EPSG','4530','EPSG','4200','EPSG','16326',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6228','projected_crs','EPSG','20086','EPSG','1785','EPSG','1209');
INSERT INTO "projected_crs" VALUES('EPSG','20087','Pulkovo 1995 / Gauss-Kruger 27N',NULL,'EPSG','4530','EPSG','4200','EPSG','16327',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6229','projected_crs','EPSG','20087','EPSG','1786','EPSG','1209');
INSERT INTO "projected_crs" VALUES('EPSG','20088','Pulkovo 1995 / Gauss-Kruger 28N',NULL,'EPSG','4530','EPSG','4200','EPSG','16328',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6230','projected_crs','EPSG','20088','EPSG','1787','EPSG','1209');
INSERT INTO "projected_crs" VALUES('EPSG','20089','Pulkovo 1995 / Gauss-Kruger 29N',NULL,'EPSG','4530','EPSG','4200','EPSG','16329',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6231','projected_crs','EPSG','20089','EPSG','1788','EPSG','1209');
INSERT INTO "projected_crs" VALUES('EPSG','20090','Pulkovo 1995 / Gauss-Kruger 30N',NULL,'EPSG','4530','EPSG','4200','EPSG','16330',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6232','projected_crs','EPSG','20090','EPSG','1789','EPSG','1209');
INSERT INTO "projected_crs" VALUES('EPSG','20091','Pulkovo 1995 / Gauss-Kruger 31N',NULL,'EPSG','4530','EPSG','4200','EPSG','16331',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6233','projected_crs','EPSG','20091','EPSG','1790','EPSG','1209');
INSERT INTO "projected_crs" VALUES('EPSG','20092','Pulkovo 1995 / Gauss-Kruger 32N',NULL,'EPSG','4530','EPSG','4200','EPSG','16332',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6234','projected_crs','EPSG','20092','EPSG','1791','EPSG','1209');
INSERT INTO "projected_crs" VALUES('EPSG','20135','Adindan / UTM zone 35N',NULL,'EPSG','4400','EPSG','4201','EPSG','16035',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6235','projected_crs','EPSG','20135','EPSG','2827','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','20136','Adindan / UTM zone 36N',NULL,'EPSG','4400','EPSG','4201','EPSG','16036',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6236','projected_crs','EPSG','20136','EPSG','2825','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','20137','Adindan / UTM zone 37N',NULL,'EPSG','4400','EPSG','4201','EPSG','16037',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6237','projected_crs','EPSG','20137','EPSG','1552','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','20138','Adindan / UTM zone 38N',NULL,'EPSG','4400','EPSG','4201','EPSG','16038',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6238','projected_crs','EPSG','20138','EPSG','1553','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','20248','AGD66 / AMG zone 48',NULL,'EPSG','4400','EPSG','4202','EPSG','17448',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6239','projected_crs','EPSG','20248','EPSG','1556','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','20249','AGD66 / AMG zone 49',NULL,'EPSG','4400','EPSG','4202','EPSG','17449',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6240','projected_crs','EPSG','20249','EPSG','1557','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','20250','AGD66 / AMG zone 50',NULL,'EPSG','4400','EPSG','4202','EPSG','17450',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6241','projected_crs','EPSG','20250','EPSG','1558','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','20251','AGD66 / AMG zone 51',NULL,'EPSG','4400','EPSG','4202','EPSG','17451',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6242','projected_crs','EPSG','20251','EPSG','1559','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','20252','AGD66 / AMG zone 52',NULL,'EPSG','4400','EPSG','4202','EPSG','17452',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6243','projected_crs','EPSG','20252','EPSG','1560','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','20253','AGD66 / AMG zone 53',NULL,'EPSG','4400','EPSG','4202','EPSG','17453',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6244','projected_crs','EPSG','20253','EPSG','1561','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','20254','AGD66 / AMG zone 54',NULL,'EPSG','4400','EPSG','4202','EPSG','17454',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6245','projected_crs','EPSG','20254','EPSG','1567','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','20255','AGD66 / AMG zone 55',NULL,'EPSG','4400','EPSG','4202','EPSG','17455',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6246','projected_crs','EPSG','20255','EPSG','1568','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','20256','AGD66 / AMG zone 56',NULL,'EPSG','4400','EPSG','4202','EPSG','17456',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6247','projected_crs','EPSG','20256','EPSG','2291','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','20257','AGD66 / AMG zone 57',NULL,'EPSG','4400','EPSG','4202','EPSG','17457',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6248','projected_crs','EPSG','20257','EPSG','1565','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','20258','AGD66 / AMG zone 58',NULL,'EPSG','4400','EPSG','4202','EPSG','17458',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6249','projected_crs','EPSG','20258','EPSG','1566','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','20348','AGD84 / AMG zone 48',NULL,'EPSG','4400','EPSG','4203','EPSG','17448',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6250','projected_crs','EPSG','20348','EPSG','1556','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','20349','AGD84 / AMG zone 49',NULL,'EPSG','4400','EPSG','4203','EPSG','17449',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6251','projected_crs','EPSG','20349','EPSG','1557','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','20350','AGD84 / AMG zone 50',NULL,'EPSG','4400','EPSG','4203','EPSG','17450',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6252','projected_crs','EPSG','20350','EPSG','1558','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','20351','AGD84 / AMG zone 51',NULL,'EPSG','4400','EPSG','4203','EPSG','17451',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6253','projected_crs','EPSG','20351','EPSG','1559','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','20352','AGD84 / AMG zone 52',NULL,'EPSG','4400','EPSG','4203','EPSG','17452',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6254','projected_crs','EPSG','20352','EPSG','3687','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','20353','AGD84 / AMG zone 53',NULL,'EPSG','4400','EPSG','4203','EPSG','17453',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6255','projected_crs','EPSG','20353','EPSG','3688','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','20354','AGD84 / AMG zone 54',NULL,'EPSG','4400','EPSG','4203','EPSG','17454',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6256','projected_crs','EPSG','20354','EPSG','3689','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','20355','AGD84 / AMG zone 55',NULL,'EPSG','4400','EPSG','4203','EPSG','17455',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6257','projected_crs','EPSG','20355','EPSG','3690','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','20356','AGD84 / AMG zone 56',NULL,'EPSG','4400','EPSG','4203','EPSG','17456',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6258','projected_crs','EPSG','20356','EPSG','3691','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','20357','AGD84 / AMG zone 57',NULL,'EPSG','4400','EPSG','4203','EPSG','17457',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6259','projected_crs','EPSG','20357','EPSG','1565','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','20358','AGD84 / AMG zone 58',NULL,'EPSG','4400','EPSG','4203','EPSG','17458',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6260','projected_crs','EPSG','20358','EPSG','1566','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','20436','Ain el Abd / UTM zone 36N',NULL,'EPSG','4400','EPSG','4204','EPSG','16036',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6261','projected_crs','EPSG','20436','EPSG','3107','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','20437','Ain el Abd / UTM zone 37N',NULL,'EPSG','4400','EPSG','4204','EPSG','16037',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6262','projected_crs','EPSG','20437','EPSG','1569','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','20438','Ain el Abd / UTM zone 38N',NULL,'EPSG','4400','EPSG','4204','EPSG','16038',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6263','projected_crs','EPSG','20438','EPSG','1571','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','20439','Ain el Abd / UTM zone 39N',NULL,'EPSG','4400','EPSG','4204','EPSG','16039',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6264','projected_crs','EPSG','20439','EPSG','1570','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','20440','Ain el Abd / UTM zone 40N',NULL,'EPSG','4400','EPSG','4204','EPSG','16040',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6265','projected_crs','EPSG','20440','EPSG','3106','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','20499','Ain el Abd / Bahrain Grid',NULL,'EPSG','4400','EPSG','4204','EPSG','19900',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6266','projected_crs','EPSG','20499','EPSG','3943','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','20538','Afgooye / UTM zone 38N',NULL,'EPSG','4400','EPSG','4205','EPSG','16038',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6267','projected_crs','EPSG','20538','EPSG','1554','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','20539','Afgooye / UTM zone 39N',NULL,'EPSG','4400','EPSG','4205','EPSG','16039',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6268','projected_crs','EPSG','20539','EPSG','1555','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','20790','Lisbon (Lisbon) / Portuguese National Grid',NULL,'EPSG','4499','EPSG','4803','EPSG','19936',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6269','projected_crs','EPSG','20790','EPSG','1294','EPSG','1160');
INSERT INTO "projected_crs" VALUES('EPSG','20791','Lisbon (Lisbon) / Portuguese Grid',NULL,'EPSG','4499','EPSG','4803','EPSG','19969',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6270','projected_crs','EPSG','20791','EPSG','1294','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','20822','Aratu / UTM zone 22S',NULL,'EPSG','4400','EPSG','4208','EPSG','16122',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6271','projected_crs','EPSG','20822','EPSG','1572','EPSG','1136');
INSERT INTO "projected_crs" VALUES('EPSG','20823','Aratu / UTM zone 23S',NULL,'EPSG','4400','EPSG','4208','EPSG','16123',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6272','projected_crs','EPSG','20823','EPSG','1573','EPSG','1136');
INSERT INTO "projected_crs" VALUES('EPSG','20824','Aratu / UTM zone 24S',NULL,'EPSG','4400','EPSG','4208','EPSG','16124',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6273','projected_crs','EPSG','20824','EPSG','1574','EPSG','1136');
INSERT INTO "projected_crs" VALUES('EPSG','20904','GSK-2011 / Gauss-Kruger zone 4',NULL,'EPSG','4530','EPSG','7683','EPSG','16204',NULL,0);
INSERT INTO "usage" VALUES('EPSG','15388','projected_crs','EPSG','20904','EPSG','1763','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','20905','GSK-2011 / Gauss-Kruger zone 5',NULL,'EPSG','4530','EPSG','7683','EPSG','16205',NULL,0);
INSERT INTO "usage" VALUES('EPSG','15473','projected_crs','EPSG','20905','EPSG','1764','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','20906','GSK-2011 / Gauss-Kruger zone 6',NULL,'EPSG','4530','EPSG','7683','EPSG','16206',NULL,0);
INSERT INTO "usage" VALUES('EPSG','15447','projected_crs','EPSG','20906','EPSG','1765','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','20907','GSK-2011 / Gauss-Kruger zone 7',NULL,'EPSG','4530','EPSG','7683','EPSG','16207',NULL,0);
INSERT INTO "usage" VALUES('EPSG','15448','projected_crs','EPSG','20907','EPSG','1766','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','20908','GSK-2011 / Gauss-Kruger zone 8',NULL,'EPSG','4530','EPSG','7683','EPSG','16208',NULL,0);
INSERT INTO "usage" VALUES('EPSG','15474','projected_crs','EPSG','20908','EPSG','1767','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','20909','GSK-2011 / Gauss-Kruger zone 9',NULL,'EPSG','4530','EPSG','7683','EPSG','16209',NULL,0);
INSERT INTO "usage" VALUES('EPSG','15475','projected_crs','EPSG','20909','EPSG','1768','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','20910','GSK-2011 / Gauss-Kruger zone 10',NULL,'EPSG','4530','EPSG','7683','EPSG','16210',NULL,0);
INSERT INTO "usage" VALUES('EPSG','15476','projected_crs','EPSG','20910','EPSG','1769','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','20911','GSK-2011 / Gauss-Kruger zone 11',NULL,'EPSG','4530','EPSG','7683','EPSG','16211',NULL,0);
INSERT INTO "usage" VALUES('EPSG','15477','projected_crs','EPSG','20911','EPSG','1770','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','20912','GSK-2011 / Gauss-Kruger zone 12',NULL,'EPSG','4530','EPSG','7683','EPSG','16212',NULL,0);
INSERT INTO "usage" VALUES('EPSG','15478','projected_crs','EPSG','20912','EPSG','1771','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','20913','GSK-2011 / Gauss-Kruger zone 13',NULL,'EPSG','4530','EPSG','7683','EPSG','16213',NULL,0);
INSERT INTO "usage" VALUES('EPSG','15479','projected_crs','EPSG','20913','EPSG','1772','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','20914','GSK-2011 / Gauss-Kruger zone 14',NULL,'EPSG','4530','EPSG','7683','EPSG','16214',NULL,0);
INSERT INTO "usage" VALUES('EPSG','15480','projected_crs','EPSG','20914','EPSG','1773','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','20915','GSK-2011 / Gauss-Kruger zone 15',NULL,'EPSG','4530','EPSG','7683','EPSG','16215',NULL,0);
INSERT INTO "usage" VALUES('EPSG','15481','projected_crs','EPSG','20915','EPSG','1774','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','20916','GSK-2011 / Gauss-Kruger zone 16',NULL,'EPSG','4530','EPSG','7683','EPSG','16216',NULL,0);
INSERT INTO "usage" VALUES('EPSG','15482','projected_crs','EPSG','20916','EPSG','1775','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','20917','GSK-2011 / Gauss-Kruger zone 17',NULL,'EPSG','4530','EPSG','7683','EPSG','16217',NULL,0);
INSERT INTO "usage" VALUES('EPSG','15483','projected_crs','EPSG','20917','EPSG','1776','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','20918','GSK-2011 / Gauss-Kruger zone 18',NULL,'EPSG','4530','EPSG','7683','EPSG','16218',NULL,0);
INSERT INTO "usage" VALUES('EPSG','15567','projected_crs','EPSG','20918','EPSG','1777','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','20919','GSK-2011 / Gauss-Kruger zone 19',NULL,'EPSG','4530','EPSG','7683','EPSG','16219',NULL,0);
INSERT INTO "usage" VALUES('EPSG','15485','projected_crs','EPSG','20919','EPSG','1778','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','20920','GSK-2011 / Gauss-Kruger zone 20',NULL,'EPSG','4530','EPSG','7683','EPSG','16220',NULL,0);
INSERT INTO "usage" VALUES('EPSG','15487','projected_crs','EPSG','20920','EPSG','1779','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','20921','GSK-2011 / Gauss-Kruger zone 21',NULL,'EPSG','4530','EPSG','7683','EPSG','16221',NULL,0);
INSERT INTO "usage" VALUES('EPSG','15488','projected_crs','EPSG','20921','EPSG','1780','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','20922','GSK-2011 / Gauss-Kruger zone 22',NULL,'EPSG','4530','EPSG','7683','EPSG','16222',NULL,0);
INSERT INTO "usage" VALUES('EPSG','15489','projected_crs','EPSG','20922','EPSG','1781','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','20923','GSK-2011 / Gauss-Kruger zone 23',NULL,'EPSG','4530','EPSG','7683','EPSG','16223',NULL,0);
INSERT INTO "usage" VALUES('EPSG','15490','projected_crs','EPSG','20923','EPSG','1782','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','20924','GSK-2011 / Gauss-Kruger zone 24',NULL,'EPSG','4530','EPSG','7683','EPSG','16224',NULL,0);
INSERT INTO "usage" VALUES('EPSG','15491','projected_crs','EPSG','20924','EPSG','1783','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','20925','GSK-2011 / Gauss-Kruger zone 25',NULL,'EPSG','4530','EPSG','7683','EPSG','16225',NULL,0);
INSERT INTO "usage" VALUES('EPSG','15492','projected_crs','EPSG','20925','EPSG','1784','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','20926','GSK-2011 / Gauss-Kruger zone 26',NULL,'EPSG','4530','EPSG','7683','EPSG','16226',NULL,0);
INSERT INTO "usage" VALUES('EPSG','15493','projected_crs','EPSG','20926','EPSG','1785','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','20927','GSK-2011 / Gauss-Kruger zone 27',NULL,'EPSG','4530','EPSG','7683','EPSG','16227',NULL,0);
INSERT INTO "usage" VALUES('EPSG','15494','projected_crs','EPSG','20927','EPSG','1786','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','20928','GSK-2011 / Gauss-Kruger zone 28',NULL,'EPSG','4530','EPSG','7683','EPSG','16228',NULL,0);
INSERT INTO "usage" VALUES('EPSG','15499','projected_crs','EPSG','20928','EPSG','1787','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','20929','GSK-2011 / Gauss-Kruger zone 29',NULL,'EPSG','4530','EPSG','7683','EPSG','16229',NULL,0);
INSERT INTO "usage" VALUES('EPSG','15495','projected_crs','EPSG','20929','EPSG','1788','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','20930','GSK-2011 / Gauss-Kruger zone 30',NULL,'EPSG','4530','EPSG','7683','EPSG','16230',NULL,0);
INSERT INTO "usage" VALUES('EPSG','15496','projected_crs','EPSG','20930','EPSG','1789','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','20931','GSK-2011 / Gauss-Kruger zone 31',NULL,'EPSG','4530','EPSG','7683','EPSG','16231',NULL,0);
INSERT INTO "usage" VALUES('EPSG','15497','projected_crs','EPSG','20931','EPSG','1790','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','20932','GSK-2011 / Gauss-Kruger zone 32',NULL,'EPSG','4530','EPSG','7683','EPSG','16232',NULL,0);
INSERT INTO "usage" VALUES('EPSG','15498','projected_crs','EPSG','20932','EPSG','1791','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','20934','Arc 1950 / UTM zone 34S',NULL,'EPSG','4400','EPSG','4209','EPSG','16134',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6274','projected_crs','EPSG','20934','EPSG','1575','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','20935','Arc 1950 / UTM zone 35S',NULL,'EPSG','4400','EPSG','4209','EPSG','16135',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6275','projected_crs','EPSG','20935','EPSG','1576','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','20936','Arc 1950 / UTM zone 36S',NULL,'EPSG','4400','EPSG','4209','EPSG','16136',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6276','projected_crs','EPSG','20936','EPSG','1577','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','21004','GSK-2011 / Gauss-Kruger CM 21E',NULL,'EPSG','4530','EPSG','7683','EPSG','16304',NULL,0);
INSERT INTO "usage" VALUES('EPSG','15563','projected_crs','EPSG','21004','EPSG','1763','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','21005','GSK-2011 / Gauss-Kruger CM 27E',NULL,'EPSG','4530','EPSG','7683','EPSG','16305',NULL,0);
INSERT INTO "usage" VALUES('EPSG','15535','projected_crs','EPSG','21005','EPSG','1764','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','21006','GSK-2011 / Gauss-Kruger CM 33E',NULL,'EPSG','4530','EPSG','7683','EPSG','16306',NULL,0);
INSERT INTO "usage" VALUES('EPSG','15536','projected_crs','EPSG','21006','EPSG','1765','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','21007','GSK-2011 / Gauss-Kruger CM 39E',NULL,'EPSG','4530','EPSG','7683','EPSG','16307',NULL,0);
INSERT INTO "usage" VALUES('EPSG','15537','projected_crs','EPSG','21007','EPSG','1766','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','21008','GSK-2011 / Gauss-Kruger CM 45E',NULL,'EPSG','4530','EPSG','7683','EPSG','16308',NULL,0);
INSERT INTO "usage" VALUES('EPSG','15538','projected_crs','EPSG','21008','EPSG','1767','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','21009','GSK-2011 / Gauss-Kruger CM 51E',NULL,'EPSG','4530','EPSG','7683','EPSG','16309',NULL,0);
INSERT INTO "usage" VALUES('EPSG','15564','projected_crs','EPSG','21009','EPSG','1768','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','21010','GSK-2011 / Gauss-Kruger CM 57E',NULL,'EPSG','4530','EPSG','7683','EPSG','16310',NULL,0);
INSERT INTO "usage" VALUES('EPSG','15540','projected_crs','EPSG','21010','EPSG','1769','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','21011','GSK-2011 / Gauss-Kruger CM 63E',NULL,'EPSG','4530','EPSG','7683','EPSG','16311',NULL,0);
INSERT INTO "usage" VALUES('EPSG','15541','projected_crs','EPSG','21011','EPSG','1770','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','21012','GSK-2011 / Gauss-Kruger CM 69E',NULL,'EPSG','4530','EPSG','7683','EPSG','16312',NULL,0);
INSERT INTO "usage" VALUES('EPSG','15542','projected_crs','EPSG','21012','EPSG','1771','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','21013','GSK-2011 / Gauss-Kruger CM 75E',NULL,'EPSG','4530','EPSG','7683','EPSG','16313',NULL,0);
INSERT INTO "usage" VALUES('EPSG','15543','projected_crs','EPSG','21013','EPSG','1772','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','21014','GSK-2011 / Gauss-Kruger CM 81E',NULL,'EPSG','4530','EPSG','7683','EPSG','16314',NULL,0);
INSERT INTO "usage" VALUES('EPSG','15544','projected_crs','EPSG','21014','EPSG','1773','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','21015','GSK-2011 / Gauss-Kruger CM 87E',NULL,'EPSG','4530','EPSG','7683','EPSG','16315',NULL,0);
INSERT INTO "usage" VALUES('EPSG','15545','projected_crs','EPSG','21015','EPSG','1774','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','21016','GSK-2011 / Gauss-Kruger CM 93E',NULL,'EPSG','4530','EPSG','7683','EPSG','16316',NULL,0);
INSERT INTO "usage" VALUES('EPSG','15546','projected_crs','EPSG','21016','EPSG','1775','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','21017','GSK-2011 / Gauss-Kruger CM 99E',NULL,'EPSG','4530','EPSG','7683','EPSG','16317',NULL,0);
INSERT INTO "usage" VALUES('EPSG','15547','projected_crs','EPSG','21017','EPSG','1776','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','21018','GSK-2011 / Gauss-Kruger CM 105E',NULL,'EPSG','4530','EPSG','7683','EPSG','16318',NULL,0);
INSERT INTO "usage" VALUES('EPSG','15548','projected_crs','EPSG','21018','EPSG','1777','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','21019','GSK-2011 / Gauss-Kruger CM 111E',NULL,'EPSG','4530','EPSG','7683','EPSG','16319',NULL,0);
INSERT INTO "usage" VALUES('EPSG','15549','projected_crs','EPSG','21019','EPSG','1778','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','21020','GSK-2011 / Gauss-Kruger CM 117E',NULL,'EPSG','4530','EPSG','7683','EPSG','16320',NULL,0);
INSERT INTO "usage" VALUES('EPSG','15550','projected_crs','EPSG','21020','EPSG','1779','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','21021','GSK-2011 / Gauss-Kruger CM 123E',NULL,'EPSG','4530','EPSG','7683','EPSG','16321',NULL,0);
INSERT INTO "usage" VALUES('EPSG','15551','projected_crs','EPSG','21021','EPSG','1780','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','21022','GSK-2011 / Gauss-Kruger CM 129E',NULL,'EPSG','4530','EPSG','7683','EPSG','16322',NULL,0);
INSERT INTO "usage" VALUES('EPSG','15552','projected_crs','EPSG','21022','EPSG','1781','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','21023','GSK-2011 / Gauss-Kruger CM 135E',NULL,'EPSG','4530','EPSG','7683','EPSG','16323',NULL,0);
INSERT INTO "usage" VALUES('EPSG','15553','projected_crs','EPSG','21023','EPSG','1782','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','21024','GSK-2011 / Gauss-Kruger CM 141E',NULL,'EPSG','4530','EPSG','7683','EPSG','16324',NULL,0);
INSERT INTO "usage" VALUES('EPSG','15554','projected_crs','EPSG','21024','EPSG','1783','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','21025','GSK-2011 / Gauss-Kruger CM 147E',NULL,'EPSG','4530','EPSG','7683','EPSG','16325',NULL,0);
INSERT INTO "usage" VALUES('EPSG','15555','projected_crs','EPSG','21025','EPSG','1784','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','21026','GSK-2011 / Gauss-Kruger CM 153E',NULL,'EPSG','4530','EPSG','7683','EPSG','16326',NULL,0);
INSERT INTO "usage" VALUES('EPSG','15556','projected_crs','EPSG','21026','EPSG','1785','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','21027','GSK-2011 / Gauss-Kruger CM 159E',NULL,'EPSG','4530','EPSG','7683','EPSG','16327',NULL,0);
INSERT INTO "usage" VALUES('EPSG','15557','projected_crs','EPSG','21027','EPSG','1786','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','21028','GSK-2011 / Gauss-Kruger CM 165E',NULL,'EPSG','4530','EPSG','7683','EPSG','16328',NULL,0);
INSERT INTO "usage" VALUES('EPSG','15558','projected_crs','EPSG','21028','EPSG','1787','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','21029','GSK-2011 / Gauss-Kruger CM 171E',NULL,'EPSG','4530','EPSG','7683','EPSG','16329',NULL,0);
INSERT INTO "usage" VALUES('EPSG','15559','projected_crs','EPSG','21029','EPSG','1788','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','21030','GSK-2011 / Gauss-Kruger CM 177E',NULL,'EPSG','4530','EPSG','7683','EPSG','16330',NULL,0);
INSERT INTO "usage" VALUES('EPSG','15560','projected_crs','EPSG','21030','EPSG','1789','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','21031','GSK-2011 / Gauss-Kruger CM 177W',NULL,'EPSG','4530','EPSG','7683','EPSG','16331',NULL,0);
INSERT INTO "usage" VALUES('EPSG','15561','projected_crs','EPSG','21031','EPSG','1790','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','21032','GSK-2011 / Gauss-Kruger CM 171W',NULL,'EPSG','4530','EPSG','7683','EPSG','16332',NULL,0);
INSERT INTO "usage" VALUES('EPSG','15562','projected_crs','EPSG','21032','EPSG','1791','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','21035','Arc 1960 / UTM zone 35S',NULL,'EPSG','4400','EPSG','4210','EPSG','16135',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6277','projected_crs','EPSG','21035','EPSG','1579','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','21036','Arc 1960 / UTM zone 36S',NULL,'EPSG','4400','EPSG','4210','EPSG','16136',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6278','projected_crs','EPSG','21036','EPSG','1581','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','21037','Arc 1960 / UTM zone 37S',NULL,'EPSG','4400','EPSG','4210','EPSG','16137',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6279','projected_crs','EPSG','21037','EPSG','1583','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','21095','Arc 1960 / UTM zone 35N',NULL,'EPSG','4400','EPSG','4210','EPSG','16035',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6280','projected_crs','EPSG','21095','EPSG','1578','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','21096','Arc 1960 / UTM zone 36N',NULL,'EPSG','4400','EPSG','4210','EPSG','16036',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6281','projected_crs','EPSG','21096','EPSG','1580','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','21097','Arc 1960 / UTM zone 37N',NULL,'EPSG','4400','EPSG','4210','EPSG','16037',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6282','projected_crs','EPSG','21097','EPSG','1582','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','21100','Batavia (Jakarta) / NEIEZ',NULL,'EPSG','4499','EPSG','4813','EPSG','19905',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6283','projected_crs','EPSG','21100','EPSG','1285','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','21148','Batavia / UTM zone 48S',NULL,'EPSG','4400','EPSG','4211','EPSG','16148',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6284','projected_crs','EPSG','21148','EPSG','1584','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','21149','Batavia / UTM zone 49S',NULL,'EPSG','4400','EPSG','4211','EPSG','16149',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6285','projected_crs','EPSG','21149','EPSG','1586','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','21150','Batavia / UTM zone 50S',NULL,'EPSG','4400','EPSG','4211','EPSG','16150',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6286','projected_crs','EPSG','21150','EPSG','1585','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','21207','GSK-2011 / GSK 3GK zone 7',NULL,'EPSG','4530','EPSG','7683','EPSG','16907',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16450','projected_crs','EPSG','21207','EPSG','2747','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21208','GSK-2011 / GSK 3GK zone 8',NULL,'EPSG','4530','EPSG','7683','EPSG','16908',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16452','projected_crs','EPSG','21208','EPSG','2748','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21209','GSK-2011 / GSK 3GK zone 9',NULL,'EPSG','4530','EPSG','7683','EPSG','16909',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16453','projected_crs','EPSG','21209','EPSG','2749','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21210','GSK-2011 / GSK 3GK zone 10',NULL,'EPSG','4530','EPSG','7683','EPSG','16910',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16454','projected_crs','EPSG','21210','EPSG','2750','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21211','GSK-2011 / GSK 3GK zone 11',NULL,'EPSG','4530','EPSG','7683','EPSG','16911',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16455','projected_crs','EPSG','21211','EPSG','2751','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21212','GSK-2011 / GSK 3GK zone 12',NULL,'EPSG','4530','EPSG','7683','EPSG','16912',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16456','projected_crs','EPSG','21212','EPSG','2752','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21213','GSK-2011 / GSK 3GK zone 13',NULL,'EPSG','4530','EPSG','7683','EPSG','16913',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16457','projected_crs','EPSG','21213','EPSG','2753','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21214','GSK-2011 / GSK 3GK zone 14',NULL,'EPSG','4530','EPSG','7683','EPSG','16914',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16458','projected_crs','EPSG','21214','EPSG','2754','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21215','GSK-2011 / GSK 3GK zone 15',NULL,'EPSG','4530','EPSG','7683','EPSG','16915',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16459','projected_crs','EPSG','21215','EPSG','2755','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21216','GSK-2011 / GSK 3GK zone 16',NULL,'EPSG','4530','EPSG','7683','EPSG','16916',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16460','projected_crs','EPSG','21216','EPSG','2756','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21217','GSK-2011 / GSK 3GK zone 17',NULL,'EPSG','4530','EPSG','7683','EPSG','16917',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16461','projected_crs','EPSG','21217','EPSG','2757','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21218','GSK-2011 / GSK 3GK zone 18',NULL,'EPSG','4530','EPSG','7683','EPSG','16918',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16462','projected_crs','EPSG','21218','EPSG','2758','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21219','GSK-2011 / GSK 3GK zone 19',NULL,'EPSG','4530','EPSG','7683','EPSG','16919',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16463','projected_crs','EPSG','21219','EPSG','2759','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21220','GSK-2011 / GSK 3GK zone 20',NULL,'EPSG','4530','EPSG','7683','EPSG','16920',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16464','projected_crs','EPSG','21220','EPSG','2760','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21221','GSK-2011 / GSK 3GK zone 21',NULL,'EPSG','4530','EPSG','7683','EPSG','16921',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16465','projected_crs','EPSG','21221','EPSG','2761','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21222','GSK-2011 / GSK 3GK zone 22',NULL,'EPSG','4530','EPSG','7683','EPSG','16922',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16420','projected_crs','EPSG','21222','EPSG','2762','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21223','GSK-2011 / GSK 3GK zone 23',NULL,'EPSG','4530','EPSG','7683','EPSG','16923',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16421','projected_crs','EPSG','21223','EPSG','2763','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21224','GSK-2011 / GSK 3GK zone 24',NULL,'EPSG','4530','EPSG','7683','EPSG','16924',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16422','projected_crs','EPSG','21224','EPSG','2764','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21225','GSK-2011 / GSK 3GK zone 25',NULL,'EPSG','4530','EPSG','7683','EPSG','16925',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16423','projected_crs','EPSG','21225','EPSG','2765','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21226','GSK-2011 / GSK 3GK zone 26',NULL,'EPSG','4530','EPSG','7683','EPSG','16926',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16424','projected_crs','EPSG','21226','EPSG','2766','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21227','GSK-2011 / GSK 3GK zone 27',NULL,'EPSG','4530','EPSG','7683','EPSG','16927',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16425','projected_crs','EPSG','21227','EPSG','2767','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21228','GSK-2011 / GSK 3GK zone 28',NULL,'EPSG','4530','EPSG','7683','EPSG','16928',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16426','projected_crs','EPSG','21228','EPSG','2768','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21229','GSK-2011 / GSK 3GK zone 29',NULL,'EPSG','4530','EPSG','7683','EPSG','16929',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16501','projected_crs','EPSG','21229','EPSG','2769','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21230','GSK-2011 / GSK 3GK zone 30',NULL,'EPSG','4530','EPSG','7683','EPSG','16930',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16502','projected_crs','EPSG','21230','EPSG','2676','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21231','GSK-2011 / GSK 3GK zone 31',NULL,'EPSG','4530','EPSG','7683','EPSG','16931',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16442','projected_crs','EPSG','21231','EPSG','2677','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21232','GSK-2011 / GSK 3GK zone 32',NULL,'EPSG','4530','EPSG','7683','EPSG','16932',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16443','projected_crs','EPSG','21232','EPSG','2678','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21233','GSK-2011 / GSK 3GK zone 33',NULL,'EPSG','4530','EPSG','7683','EPSG','16933',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16444','projected_crs','EPSG','21233','EPSG','2679','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21234','GSK-2011 / GSK 3GK zone 34',NULL,'EPSG','4530','EPSG','7683','EPSG','16934',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16445','projected_crs','EPSG','21234','EPSG','2680','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21235','GSK-2011 / GSK 3GK zone 35',NULL,'EPSG','4530','EPSG','7683','EPSG','16935',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16446','projected_crs','EPSG','21235','EPSG','2681','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21236','GSK-2011 / GSK 3GK zone 36',NULL,'EPSG','4530','EPSG','7683','EPSG','16936',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16447','projected_crs','EPSG','21236','EPSG','2682','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21237','GSK-2011 / GSK 3GK zone 37',NULL,'EPSG','4530','EPSG','7683','EPSG','16937',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16392','projected_crs','EPSG','21237','EPSG','2683','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21238','GSK-2011 / GSK 3GK zone 38',NULL,'EPSG','4530','EPSG','7683','EPSG','16938',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16393','projected_crs','EPSG','21238','EPSG','2684','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21239','GSK-2011 / GSK 3GK zone 39',NULL,'EPSG','4530','EPSG','7683','EPSG','16939',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16394','projected_crs','EPSG','21239','EPSG','2685','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21240','GSK-2011 / GSK 3GK zone 40',NULL,'EPSG','4530','EPSG','7683','EPSG','16940',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16395','projected_crs','EPSG','21240','EPSG','2686','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21241','GSK-2011 / GSK 3GK zone 41',NULL,'EPSG','4530','EPSG','7683','EPSG','16941',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16396','projected_crs','EPSG','21241','EPSG','2687','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21242','GSK-2011 / GSK 3GK zone 42',NULL,'EPSG','4530','EPSG','7683','EPSG','16942',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16397','projected_crs','EPSG','21242','EPSG','2688','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21243','GSK-2011 / GSK 3GK zone 43',NULL,'EPSG','4530','EPSG','7683','EPSG','16943',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16398','projected_crs','EPSG','21243','EPSG','2689','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21244','GSK-2011 / GSK 3GK zone 44',NULL,'EPSG','4530','EPSG','7683','EPSG','16944',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16399','projected_crs','EPSG','21244','EPSG','2690','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21245','GSK-2011 / GSK 3GK zone 45',NULL,'EPSG','4530','EPSG','7683','EPSG','16945',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16400','projected_crs','EPSG','21245','EPSG','2691','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21246','GSK-2011 / GSK 3GK zone 46',NULL,'EPSG','4530','EPSG','7683','EPSG','16946',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16401','projected_crs','EPSG','21246','EPSG','2692','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21247','GSK-2011 / GSK 3GK zone 47',NULL,'EPSG','4530','EPSG','7683','EPSG','16947',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16402','projected_crs','EPSG','21247','EPSG','2693','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21248','GSK-2011 / GSK 3GK zone 48',NULL,'EPSG','4530','EPSG','7683','EPSG','16948',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16403','projected_crs','EPSG','21248','EPSG','2694','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21249','GSK-2011 / GSK 3GK zone 49',NULL,'EPSG','4530','EPSG','7683','EPSG','16949',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16404','projected_crs','EPSG','21249','EPSG','2695','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21250','GSK-2011 / GSK 3GK zone 50',NULL,'EPSG','4530','EPSG','7683','EPSG','16950',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16405','projected_crs','EPSG','21250','EPSG','2696','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21251','GSK-2011 / GSK 3GK zone 51',NULL,'EPSG','4530','EPSG','7683','EPSG','16951',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16365','projected_crs','EPSG','21251','EPSG','2697','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21252','GSK-2011 / GSK 3GK zone 52',NULL,'EPSG','4530','EPSG','7683','EPSG','16952',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16366','projected_crs','EPSG','21252','EPSG','2698','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21253','GSK-2011 / GSK 3GK zone 53',NULL,'EPSG','4530','EPSG','7683','EPSG','16953',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16367','projected_crs','EPSG','21253','EPSG','2699','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21254','GSK-2011 / GSK 3GK zone 54',NULL,'EPSG','4530','EPSG','7683','EPSG','16954',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16368','projected_crs','EPSG','21254','EPSG','2700','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21255','GSK-2011 / GSK 3GK zone 55',NULL,'EPSG','4530','EPSG','7683','EPSG','16955',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16369','projected_crs','EPSG','21255','EPSG','2701','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21256','GSK-2011 / GSK 3GK zone 56',NULL,'EPSG','4530','EPSG','7683','EPSG','16956',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16370','projected_crs','EPSG','21256','EPSG','2702','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21257','GSK-2011 / GSK 3GK zone 57',NULL,'EPSG','4530','EPSG','7683','EPSG','16957',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16371','projected_crs','EPSG','21257','EPSG','2703','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21258','GSK-2011 / GSK 3GK zone 58',NULL,'EPSG','4530','EPSG','7683','EPSG','16958',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16372','projected_crs','EPSG','21258','EPSG','2704','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21259','GSK-2011 / GSK 3GK zone 59',NULL,'EPSG','4530','EPSG','7683','EPSG','16959',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16373','projected_crs','EPSG','21259','EPSG','2705','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21260','GSK-2011 / GSK 3GK zone 60',NULL,'EPSG','4530','EPSG','7683','EPSG','16960',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16375','projected_crs','EPSG','21260','EPSG','2706','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21261','GSK-2011 / GSK 3GK zone 61',NULL,'EPSG','4530','EPSG','7683','EPSG','16961',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16376','projected_crs','EPSG','21261','EPSG','2707','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21262','GSK-2011 / GSK 3GK zone 62',NULL,'EPSG','4530','EPSG','7683','EPSG','16962',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16377','projected_crs','EPSG','21262','EPSG','2708','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21263','GSK-2011 / GSK 3GK zone 63',NULL,'EPSG','4530','EPSG','7683','EPSG','16963',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16378','projected_crs','EPSG','21263','EPSG','2709','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21264','GSK-2011 / GSK 3GK zone 64',NULL,'EPSG','4530','EPSG','7683','EPSG','16964',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16379','projected_crs','EPSG','21264','EPSG','2710','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21291','Barbados 1938 / British West Indies Grid',NULL,'EPSG','4400','EPSG','4212','EPSG','19942',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6287','projected_crs','EPSG','21291','EPSG','3218','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','21292','Barbados 1938 / Barbados National Grid',NULL,'EPSG','4400','EPSG','4212','EPSG','19943',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6288','projected_crs','EPSG','21292','EPSG','3218','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','21307','GSK-2011 / GSK 3GK CM 21E',NULL,'EPSG','4530','EPSG','7683','EPSG','17107',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16451','projected_crs','EPSG','21307','EPSG','2747','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21308','GSK-2011 / GSK 3GK CM 24E',NULL,'EPSG','4530','EPSG','7683','EPSG','17108',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16466','projected_crs','EPSG','21308','EPSG','2748','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21309','GSK-2011 / GSK 3GK CM 27E',NULL,'EPSG','4530','EPSG','7683','EPSG','17109',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16467','projected_crs','EPSG','21309','EPSG','2749','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21310','GSK-2011 / GSK 3GK CM 30E',NULL,'EPSG','4530','EPSG','7683','EPSG','17110',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16468','projected_crs','EPSG','21310','EPSG','2750','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21311','GSK-2011 / GSK 3GK CM 33E',NULL,'EPSG','4530','EPSG','7683','EPSG','17111',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16469','projected_crs','EPSG','21311','EPSG','2751','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21312','GSK-2011 / GSK 3GK CM 36E',NULL,'EPSG','4530','EPSG','7683','EPSG','17112',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16470','projected_crs','EPSG','21312','EPSG','2752','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21313','GSK-2011 / GSK 3GK CM 39E',NULL,'EPSG','4530','EPSG','7683','EPSG','17113',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16479','projected_crs','EPSG','21313','EPSG','2753','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21314','GSK-2011 / GSK 3GK CM 42E',NULL,'EPSG','4530','EPSG','7683','EPSG','17114',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16471','projected_crs','EPSG','21314','EPSG','2754','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21315','GSK-2011 / GSK 3GK CM 45E',NULL,'EPSG','4530','EPSG','7683','EPSG','17115',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16472','projected_crs','EPSG','21315','EPSG','2755','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21316','GSK-2011 / GSK 3GK CM 48E',NULL,'EPSG','4530','EPSG','7683','EPSG','17116',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16473','projected_crs','EPSG','21316','EPSG','2756','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21317','GSK-2011 / GSK 3GK CM 51E',NULL,'EPSG','4530','EPSG','7683','EPSG','17117',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16474','projected_crs','EPSG','21317','EPSG','2757','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21318','GSK-2011 / GSK 3GK CM 54E',NULL,'EPSG','4530','EPSG','7683','EPSG','17118',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16475','projected_crs','EPSG','21318','EPSG','2758','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21319','GSK-2011 / GSK 3GK CM 57E',NULL,'EPSG','4530','EPSG','7683','EPSG','17119',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16476','projected_crs','EPSG','21319','EPSG','2759','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21320','GSK-2011 / GSK 3GK CM 60E',NULL,'EPSG','4530','EPSG','7683','EPSG','17120',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16477','projected_crs','EPSG','21320','EPSG','2760','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21321','GSK-2011 / GSK 3GK CM 63E',NULL,'EPSG','4530','EPSG','7683','EPSG','17121',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16478','projected_crs','EPSG','21321','EPSG','2761','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21322','GSK-2011 / GSK 3GK CM 66E',NULL,'EPSG','4530','EPSG','7683','EPSG','17122',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16448','projected_crs','EPSG','21322','EPSG','2762','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21323','GSK-2011 / GSK 3GK CM 69E',NULL,'EPSG','4530','EPSG','7683','EPSG','17123',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16430','projected_crs','EPSG','21323','EPSG','2763','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21324','GSK-2011 / GSK 3GK CM 72E',NULL,'EPSG','4530','EPSG','7683','EPSG','17124',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16449','projected_crs','EPSG','21324','EPSG','2764','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21325','GSK-2011 / GSK 3GK CM 75E',NULL,'EPSG','4530','EPSG','7683','EPSG','17125',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16431','projected_crs','EPSG','21325','EPSG','2765','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21326','GSK-2011 / GSK 3GK CM 78E',NULL,'EPSG','4530','EPSG','7683','EPSG','17126',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16432','projected_crs','EPSG','21326','EPSG','2766','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21327','GSK-2011 / GSK 3GK CM 81E',NULL,'EPSG','4530','EPSG','7683','EPSG','17127',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16433','projected_crs','EPSG','21327','EPSG','2767','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21328','GSK-2011 / GSK 3GK CM 84E',NULL,'EPSG','4530','EPSG','7683','EPSG','17128',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16434','projected_crs','EPSG','21328','EPSG','2768','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21329','GSK-2011 / GSK 3GK CM 87E',NULL,'EPSG','4530','EPSG','7683','EPSG','17129',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16435','projected_crs','EPSG','21329','EPSG','2769','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21330','GSK-2011 / GSK 3GK CM 90E',NULL,'EPSG','4530','EPSG','7683','EPSG','17130',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16436','projected_crs','EPSG','21330','EPSG','2676','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21331','GSK-2011 / GSK 3GK CM 93E',NULL,'EPSG','4530','EPSG','7683','EPSG','17131',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16437','projected_crs','EPSG','21331','EPSG','2677','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21332','GSK-2011 / GSK 3GK CM 96E',NULL,'EPSG','4530','EPSG','7683','EPSG','17132',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16429','projected_crs','EPSG','21332','EPSG','2678','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21333','GSK-2011 / GSK 3GK CM 99E',NULL,'EPSG','4530','EPSG','7683','EPSG','17133',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16438','projected_crs','EPSG','21333','EPSG','2679','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21334','GSK-2011 / GSK 3GK CM 102E',NULL,'EPSG','4530','EPSG','7683','EPSG','17134',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16439','projected_crs','EPSG','21334','EPSG','2680','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21335','GSK-2011 / GSK 3GK CM 105E',NULL,'EPSG','4530','EPSG','7683','EPSG','17135',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16440','projected_crs','EPSG','21335','EPSG','2681','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21336','GSK-2011 / GSK 3GK CM 108E',NULL,'EPSG','4530','EPSG','7683','EPSG','17136',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16441','projected_crs','EPSG','21336','EPSG','2682','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21337','GSK-2011 / GSK 3GK CM 111E',NULL,'EPSG','4530','EPSG','7683','EPSG','17137',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16406','projected_crs','EPSG','21337','EPSG','2683','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21338','GSK-2011 / GSK 3GK CM 114E',NULL,'EPSG','4530','EPSG','7683','EPSG','17138',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16407','projected_crs','EPSG','21338','EPSG','2684','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21339','GSK-2011 / GSK 3GK CM 117E',NULL,'EPSG','4530','EPSG','7683','EPSG','17139',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16408','projected_crs','EPSG','21339','EPSG','2685','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21340','GSK-2011 / GSK 3GK CM 120E',NULL,'EPSG','4530','EPSG','7683','EPSG','17140',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16409','projected_crs','EPSG','21340','EPSG','2686','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21341','GSK-2011 / GSK 3GK CM 123E',NULL,'EPSG','4530','EPSG','7683','EPSG','17141',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16481','projected_crs','EPSG','21341','EPSG','2687','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21342','GSK-2011 / GSK 3GK CM 126E',NULL,'EPSG','4530','EPSG','7683','EPSG','17142',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16411','projected_crs','EPSG','21342','EPSG','2688','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21343','GSK-2011 / GSK 3GK CM 129E',NULL,'EPSG','4530','EPSG','7683','EPSG','17143',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16412','projected_crs','EPSG','21343','EPSG','2689','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21344','GSK-2011 / GSK 3GK CM 132E',NULL,'EPSG','4530','EPSG','7683','EPSG','17144',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16413','projected_crs','EPSG','21344','EPSG','2690','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21345','GSK-2011 / GSK 3GK CM 135E',NULL,'EPSG','4530','EPSG','7683','EPSG','17145',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16414','projected_crs','EPSG','21345','EPSG','2691','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21346','GSK-2011 / GSK 3GK CM 138E',NULL,'EPSG','4530','EPSG','7683','EPSG','17146',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16415','projected_crs','EPSG','21346','EPSG','2692','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21347','GSK-2011 / GSK 3GK CM 141E',NULL,'EPSG','4530','EPSG','7683','EPSG','17147',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16416','projected_crs','EPSG','21347','EPSG','2693','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21348','GSK-2011 / GSK 3GK CM 144E',NULL,'EPSG','4530','EPSG','7683','EPSG','17148',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16417','projected_crs','EPSG','21348','EPSG','2694','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21349','GSK-2011 / GSK 3GK CM 147E',NULL,'EPSG','4530','EPSG','7683','EPSG','17149',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16418','projected_crs','EPSG','21349','EPSG','2695','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21350','GSK-2011 / GSK 3GK CM 150E',NULL,'EPSG','4530','EPSG','7683','EPSG','17150',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16419','projected_crs','EPSG','21350','EPSG','2696','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21351','GSK-2011 / GSK 3GK CM 153E',NULL,'EPSG','4530','EPSG','7683','EPSG','17151',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16380','projected_crs','EPSG','21351','EPSG','2697','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21352','GSK-2011 / GSK 3GK CM 156E',NULL,'EPSG','4530','EPSG','7683','EPSG','17152',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16381','projected_crs','EPSG','21352','EPSG','2698','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21353','GSK-2011 / GSK 3GK CM 159E',NULL,'EPSG','4530','EPSG','7683','EPSG','17153',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16382','projected_crs','EPSG','21353','EPSG','2699','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21354','GSK-2011 / GSK 3GK CM 162E',NULL,'EPSG','4530','EPSG','7683','EPSG','17154',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16383','projected_crs','EPSG','21354','EPSG','2700','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21355','GSK-2011 / GSK 3GK CM 165E',NULL,'EPSG','4530','EPSG','7683','EPSG','17155',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16384','projected_crs','EPSG','21355','EPSG','2701','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21356','GSK-2011 / GSK 3GK CM 168E',NULL,'EPSG','4530','EPSG','7683','EPSG','17156',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16385','projected_crs','EPSG','21356','EPSG','2702','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21357','GSK-2011 / GSK 3GK CM 171E',NULL,'EPSG','4530','EPSG','7683','EPSG','17157',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16386','projected_crs','EPSG','21357','EPSG','2703','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21358','GSK-2011 / GSK 3GK CM 174E',NULL,'EPSG','4530','EPSG','7683','EPSG','17158',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16374','projected_crs','EPSG','21358','EPSG','2704','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21359','GSK-2011 / GSK 3GK CM 177E',NULL,'EPSG','4530','EPSG','7683','EPSG','17159',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16387','projected_crs','EPSG','21359','EPSG','2705','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21360','GSK-2011 / GSK 3GK CM 180E',NULL,'EPSG','4530','EPSG','7683','EPSG','17160',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16388','projected_crs','EPSG','21360','EPSG','2706','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21361','GSK-2011 / GSK 3GK CM 177W',NULL,'EPSG','4530','EPSG','7683','EPSG','17161',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16389','projected_crs','EPSG','21361','EPSG','2707','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21362','GSK-2011 / GSK 3GK CM 174W',NULL,'EPSG','4530','EPSG','7683','EPSG','17162',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16390','projected_crs','EPSG','21362','EPSG','2708','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21363','GSK-2011 / GSK 3GK CM 171W',NULL,'EPSG','4530','EPSG','7683','EPSG','17163',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16483','projected_crs','EPSG','21363','EPSG','2709','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21364','GSK-2011 / GSK 3GK CM 168W',NULL,'EPSG','4530','EPSG','7683','EPSG','17164',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16484','projected_crs','EPSG','21364','EPSG','2710','EPSG','1055');
INSERT INTO "projected_crs" VALUES('EPSG','21413','Beijing 1954 / Gauss-Kruger zone 13',NULL,'EPSG','4530','EPSG','4214','EPSG','16213',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6289','projected_crs','EPSG','21413','EPSG','1587','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','21414','Beijing 1954 / Gauss-Kruger zone 14',NULL,'EPSG','4530','EPSG','4214','EPSG','16214',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6290','projected_crs','EPSG','21414','EPSG','1588','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','21415','Beijing 1954 / Gauss-Kruger zone 15',NULL,'EPSG','4530','EPSG','4214','EPSG','16215',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6291','projected_crs','EPSG','21415','EPSG','1589','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','21416','Beijing 1954 / Gauss-Kruger zone 16',NULL,'EPSG','4530','EPSG','4214','EPSG','16216',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6292','projected_crs','EPSG','21416','EPSG','1590','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','21417','Beijing 1954 / Gauss-Kruger zone 17',NULL,'EPSG','4530','EPSG','4214','EPSG','16217',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6293','projected_crs','EPSG','21417','EPSG','1591','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','21418','Beijing 1954 / Gauss-Kruger zone 18',NULL,'EPSG','4530','EPSG','4214','EPSG','16218',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6294','projected_crs','EPSG','21418','EPSG','1592','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','21419','Beijing 1954 / Gauss-Kruger zone 19',NULL,'EPSG','4530','EPSG','4214','EPSG','16219',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6295','projected_crs','EPSG','21419','EPSG','1593','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','21420','Beijing 1954 / Gauss-Kruger zone 20',NULL,'EPSG','4530','EPSG','4214','EPSG','16220',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6296','projected_crs','EPSG','21420','EPSG','1594','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','21421','Beijing 1954 / Gauss-Kruger zone 21',NULL,'EPSG','4530','EPSG','4214','EPSG','16221',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6297','projected_crs','EPSG','21421','EPSG','1595','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','21422','Beijing 1954 / Gauss-Kruger zone 22',NULL,'EPSG','4530','EPSG','4214','EPSG','16222',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6298','projected_crs','EPSG','21422','EPSG','1596','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','21423','Beijing 1954 / Gauss-Kruger zone 23',NULL,'EPSG','4530','EPSG','4214','EPSG','16223',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6299','projected_crs','EPSG','21423','EPSG','1597','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','21453','Beijing 1954 / Gauss-Kruger CM 75E',NULL,'EPSG','4530','EPSG','4214','EPSG','16313',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6300','projected_crs','EPSG','21453','EPSG','1587','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','21454','Beijing 1954 / Gauss-Kruger CM 81E',NULL,'EPSG','4530','EPSG','4214','EPSG','16314',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6301','projected_crs','EPSG','21454','EPSG','1588','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','21455','Beijing 1954 / Gauss-Kruger CM 87E',NULL,'EPSG','4530','EPSG','4214','EPSG','16315',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6302','projected_crs','EPSG','21455','EPSG','1589','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','21456','Beijing 1954 / Gauss-Kruger CM 93E',NULL,'EPSG','4530','EPSG','4214','EPSG','16316',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6303','projected_crs','EPSG','21456','EPSG','1590','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','21457','Beijing 1954 / Gauss-Kruger CM 99E',NULL,'EPSG','4530','EPSG','4214','EPSG','16317',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6304','projected_crs','EPSG','21457','EPSG','1591','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','21458','Beijing 1954 / Gauss-Kruger CM 105E',NULL,'EPSG','4530','EPSG','4214','EPSG','16318',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6305','projected_crs','EPSG','21458','EPSG','1592','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','21459','Beijing 1954 / Gauss-Kruger CM 111E',NULL,'EPSG','4530','EPSG','4214','EPSG','16319',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6306','projected_crs','EPSG','21459','EPSG','1593','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','21460','Beijing 1954 / Gauss-Kruger CM 117E',NULL,'EPSG','4530','EPSG','4214','EPSG','16320',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6307','projected_crs','EPSG','21460','EPSG','1594','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','21461','Beijing 1954 / Gauss-Kruger CM 123E',NULL,'EPSG','4530','EPSG','4214','EPSG','16321',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6308','projected_crs','EPSG','21461','EPSG','1595','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','21462','Beijing 1954 / Gauss-Kruger CM 129E',NULL,'EPSG','4530','EPSG','4214','EPSG','16322',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6309','projected_crs','EPSG','21462','EPSG','1596','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','21463','Beijing 1954 / Gauss-Kruger CM 135E',NULL,'EPSG','4530','EPSG','4214','EPSG','16323',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6310','projected_crs','EPSG','21463','EPSG','1597','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','21473','Beijing 1954 / Gauss-Kruger 13N',NULL,'EPSG','4530','EPSG','4214','EPSG','16313',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6311','projected_crs','EPSG','21473','EPSG','1587','EPSG','1209');
INSERT INTO "projected_crs" VALUES('EPSG','21474','Beijing 1954 / Gauss-Kruger 14N',NULL,'EPSG','4530','EPSG','4214','EPSG','16314',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6312','projected_crs','EPSG','21474','EPSG','1588','EPSG','1209');
INSERT INTO "projected_crs" VALUES('EPSG','21475','Beijing 1954 / Gauss-Kruger 15N',NULL,'EPSG','4530','EPSG','4214','EPSG','16315',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6313','projected_crs','EPSG','21475','EPSG','1589','EPSG','1209');
INSERT INTO "projected_crs" VALUES('EPSG','21476','Beijing 1954 / Gauss-Kruger 16N',NULL,'EPSG','4530','EPSG','4214','EPSG','16316',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6314','projected_crs','EPSG','21476','EPSG','1590','EPSG','1209');
INSERT INTO "projected_crs" VALUES('EPSG','21477','Beijing 1954 / Gauss-Kruger 17N',NULL,'EPSG','4530','EPSG','4214','EPSG','16317',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6315','projected_crs','EPSG','21477','EPSG','1591','EPSG','1209');
INSERT INTO "projected_crs" VALUES('EPSG','21478','Beijing 1954 / Gauss-Kruger 18N',NULL,'EPSG','4530','EPSG','4214','EPSG','16318',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6316','projected_crs','EPSG','21478','EPSG','1592','EPSG','1209');
INSERT INTO "projected_crs" VALUES('EPSG','21479','Beijing 1954 / Gauss-Kruger 19N',NULL,'EPSG','4530','EPSG','4214','EPSG','16319',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6317','projected_crs','EPSG','21479','EPSG','1593','EPSG','1209');
INSERT INTO "projected_crs" VALUES('EPSG','21480','Beijing 1954 / Gauss-Kruger 20N',NULL,'EPSG','4530','EPSG','4214','EPSG','16320',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6318','projected_crs','EPSG','21480','EPSG','1594','EPSG','1209');
INSERT INTO "projected_crs" VALUES('EPSG','21481','Beijing 1954 / Gauss-Kruger 21N',NULL,'EPSG','4530','EPSG','4214','EPSG','16321',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6319','projected_crs','EPSG','21481','EPSG','1595','EPSG','1209');
INSERT INTO "projected_crs" VALUES('EPSG','21482','Beijing 1954 / Gauss-Kruger 22N',NULL,'EPSG','4530','EPSG','4214','EPSG','16322',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6320','projected_crs','EPSG','21482','EPSG','1596','EPSG','1209');
INSERT INTO "projected_crs" VALUES('EPSG','21483','Beijing 1954 / Gauss-Kruger 23N',NULL,'EPSG','4530','EPSG','4214','EPSG','16323',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6321','projected_crs','EPSG','21483','EPSG','1597','EPSG','1209');
INSERT INTO "projected_crs" VALUES('EPSG','21500','BD50 (Brussels) / Belge Lambert 50',NULL,'EPSG','4499','EPSG','4809','EPSG','19901',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6322','projected_crs','EPSG','21500','EPSG','1347','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','21780','CH1903 (Bern) / LV03C',NULL,'EPSG','4498','EPSG','4801','EPSG','19923',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6323','projected_crs','EPSG','21780','EPSG','1286','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','21781','CH1903 / LV03',NULL,'EPSG','4498','EPSG','4149','EPSG','19922',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6324','projected_crs','EPSG','21781','EPSG','1286','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','21782','CH1903 / LV03C-G',NULL,'EPSG','4498','EPSG','4149','EPSG','19841',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6325','projected_crs','EPSG','21782','EPSG','1144','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','21817','Bogota 1975 / UTM zone 17N',NULL,'EPSG','4400','EPSG','4218','EPSG','16017',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6326','projected_crs','EPSG','21817','EPSG','1602','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','21818','Bogota 1975 / UTM zone 18N',NULL,'EPSG','4400','EPSG','4218','EPSG','16018',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6327','projected_crs','EPSG','21818','EPSG','1603','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','21891','Bogota 1975 / Colombia West zone',NULL,'EPSG','4499','EPSG','4218','EPSG','18051',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6328','projected_crs','EPSG','21891','EPSG','1598','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','21892','Bogota 1975 / Colombia Bogota zone',NULL,'EPSG','4499','EPSG','4218','EPSG','18052',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6329','projected_crs','EPSG','21892','EPSG','1599','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','21893','Bogota 1975 / Colombia East Central zone',NULL,'EPSG','4499','EPSG','4218','EPSG','18053',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6330','projected_crs','EPSG','21893','EPSG','1600','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','21894','Bogota 1975 / Colombia East',NULL,'EPSG','4499','EPSG','4218','EPSG','18054',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6331','projected_crs','EPSG','21894','EPSG','1601','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','21896','Bogota 1975 / Colombia West zone',NULL,'EPSG','4530','EPSG','4218','EPSG','18051',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6332','projected_crs','EPSG','21896','EPSG','1598','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','21897','Bogota 1975 / Colombia Bogota zone',NULL,'EPSG','4530','EPSG','4218','EPSG','18052',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6333','projected_crs','EPSG','21897','EPSG','1599','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','21898','Bogota 1975 / Colombia East Central zone',NULL,'EPSG','4530','EPSG','4218','EPSG','18053',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6334','projected_crs','EPSG','21898','EPSG','1600','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','21899','Bogota 1975 / Colombia East zone',NULL,'EPSG','4530','EPSG','4218','EPSG','18054',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6335','projected_crs','EPSG','21899','EPSG','1601','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22032','Camacupa 1948 / UTM zone 32S',NULL,'EPSG','4400','EPSG','4220','EPSG','16132',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6336','projected_crs','EPSG','22032','EPSG','1606','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22033','Camacupa 1948 / UTM zone 33S',NULL,'EPSG','4400','EPSG','4220','EPSG','16133',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6337','projected_crs','EPSG','22033','EPSG','1607','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22091','Camacupa 1948 / TM 11.30 SE',NULL,'EPSG','4400','EPSG','4220','EPSG','16611',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6338','projected_crs','EPSG','22091','EPSG','1605','EPSG','1136');
INSERT INTO "projected_crs" VALUES('EPSG','22092','Camacupa 1948 / TM 12 SE',NULL,'EPSG','4400','EPSG','4220','EPSG','16612',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6339','projected_crs','EPSG','22092','EPSG','1604','EPSG','1136');
INSERT INTO "projected_crs" VALUES('EPSG','22171','POSGAR 98 / Argentina 1',NULL,'EPSG','4530','EPSG','4190','EPSG','18031',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6340','projected_crs','EPSG','22171','EPSG','1608','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22172','POSGAR 98 / Argentina 2',NULL,'EPSG','4530','EPSG','4190','EPSG','18032',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6341','projected_crs','EPSG','22172','EPSG','1609','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22173','POSGAR 98 / Argentina 3',NULL,'EPSG','4530','EPSG','4190','EPSG','18033',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6342','projected_crs','EPSG','22173','EPSG','1610','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22174','POSGAR 98 / Argentina 4',NULL,'EPSG','4530','EPSG','4190','EPSG','18034',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6343','projected_crs','EPSG','22174','EPSG','1611','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22175','POSGAR 98 / Argentina 5',NULL,'EPSG','4530','EPSG','4190','EPSG','18035',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6344','projected_crs','EPSG','22175','EPSG','1612','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22176','POSGAR 98 / Argentina 6',NULL,'EPSG','4530','EPSG','4190','EPSG','18036',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6345','projected_crs','EPSG','22176','EPSG','1613','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22177','POSGAR 98 / Argentina 7',NULL,'EPSG','4530','EPSG','4190','EPSG','18037',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6346','projected_crs','EPSG','22177','EPSG','1614','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22181','POSGAR 94 / Argentina 1',NULL,'EPSG','4530','EPSG','4694','EPSG','18031',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6347','projected_crs','EPSG','22181','EPSG','1608','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22182','POSGAR 94 / Argentina 2',NULL,'EPSG','4530','EPSG','4694','EPSG','18032',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6348','projected_crs','EPSG','22182','EPSG','1609','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22183','POSGAR 94 / Argentina 3',NULL,'EPSG','4530','EPSG','4694','EPSG','18033',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6349','projected_crs','EPSG','22183','EPSG','1610','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22184','POSGAR 94 / Argentina 4',NULL,'EPSG','4530','EPSG','4694','EPSG','18034',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6350','projected_crs','EPSG','22184','EPSG','1611','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22185','POSGAR 94 / Argentina 5',NULL,'EPSG','4530','EPSG','4694','EPSG','18035',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6351','projected_crs','EPSG','22185','EPSG','1612','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22186','POSGAR 94 / Argentina 6',NULL,'EPSG','4530','EPSG','4694','EPSG','18036',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6352','projected_crs','EPSG','22186','EPSG','1613','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22187','POSGAR 94 / Argentina 7',NULL,'EPSG','4530','EPSG','4694','EPSG','18037',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6353','projected_crs','EPSG','22187','EPSG','1614','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22191','Campo Inchauspe / Argentina 1',NULL,'EPSG','4530','EPSG','4221','EPSG','18031',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6354','projected_crs','EPSG','22191','EPSG','1608','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22192','Campo Inchauspe / Argentina 2',NULL,'EPSG','4530','EPSG','4221','EPSG','18032',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6355','projected_crs','EPSG','22192','EPSG','4577','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22193','Campo Inchauspe / Argentina 3',NULL,'EPSG','4530','EPSG','4221','EPSG','18033',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6356','projected_crs','EPSG','22193','EPSG','4578','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22194','Campo Inchauspe / Argentina 4',NULL,'EPSG','4530','EPSG','4221','EPSG','18034',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6357','projected_crs','EPSG','22194','EPSG','4579','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22195','Campo Inchauspe / Argentina 5',NULL,'EPSG','4530','EPSG','4221','EPSG','18035',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6358','projected_crs','EPSG','22195','EPSG','1612','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22196','Campo Inchauspe / Argentina 6',NULL,'EPSG','4530','EPSG','4221','EPSG','18036',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6359','projected_crs','EPSG','22196','EPSG','1613','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22197','Campo Inchauspe / Argentina 7',NULL,'EPSG','4530','EPSG','4221','EPSG','18037',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6360','projected_crs','EPSG','22197','EPSG','1614','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22207','NAD83(CSRS)v2 / UTM zone 7N',NULL,'EPSG','4400','EPSG','8237','EPSG','16007',NULL,0);
INSERT INTO "usage" VALUES('EPSG','18003','projected_crs','EPSG','22207','EPSG','3409','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22208','NAD83(CSRS)v2 / UTM zone 8N',NULL,'EPSG','4400','EPSG','8237','EPSG','16008',NULL,0);
INSERT INTO "usage" VALUES('EPSG','18004','projected_crs','EPSG','22208','EPSG','3410','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22209','NAD83(CSRS)v2 / UTM zone 9N',NULL,'EPSG','4400','EPSG','8237','EPSG','16009',NULL,0);
INSERT INTO "usage" VALUES('EPSG','18005','projected_crs','EPSG','22209','EPSG','3411','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22210','NAD83(CSRS)v2 / UTM zone 10N',NULL,'EPSG','4400','EPSG','8237','EPSG','16010',NULL,0);
INSERT INTO "usage" VALUES('EPSG','18006','projected_crs','EPSG','22210','EPSG','3412','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22211','NAD83(CSRS)v2 / UTM zone 11N',NULL,'EPSG','4400','EPSG','8237','EPSG','16011',NULL,0);
INSERT INTO "usage" VALUES('EPSG','18007','projected_crs','EPSG','22211','EPSG','3528','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22212','NAD83(CSRS)v2 / UTM zone 12N',NULL,'EPSG','4400','EPSG','8237','EPSG','16012',NULL,0);
INSERT INTO "usage" VALUES('EPSG','18008','projected_crs','EPSG','22212','EPSG','3527','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22213','NAD83(CSRS)v2 / UTM zone 13N',NULL,'EPSG','4400','EPSG','8237','EPSG','16013',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17996','projected_crs','EPSG','22213','EPSG','3526','EPSG','1142');
INSERT INTO "usage" VALUES('EPSG','17997','projected_crs','EPSG','22213','EPSG','2375','EPSG','1220');
INSERT INTO "projected_crs" VALUES('EPSG','22214','NAD83(CSRS)v2 / UTM zone 14N',NULL,'EPSG','4400','EPSG','8237','EPSG','16014',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17998','projected_crs','EPSG','22214','EPSG','3413','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22215','NAD83(CSRS)v2 / UTM zone 15N',NULL,'EPSG','4400','EPSG','8237','EPSG','16015',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17999','projected_crs','EPSG','22215','EPSG','3414','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22216','NAD83(CSRS)v2 / UTM zone 16N',NULL,'EPSG','4400','EPSG','8237','EPSG','16016',NULL,0);
INSERT INTO "usage" VALUES('EPSG','18000','projected_crs','EPSG','22216','EPSG','3415','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22217','NAD83(CSRS)v2 / UTM zone 17N',NULL,'EPSG','4400','EPSG','8237','EPSG','16017',NULL,0);
INSERT INTO "usage" VALUES('EPSG','18001','projected_crs','EPSG','22217','EPSG','3416','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22218','NAD83(CSRS)v2 / UTM zone 18N',NULL,'EPSG','4400','EPSG','8237','EPSG','16018',NULL,0);
INSERT INTO "usage" VALUES('EPSG','18002','projected_crs','EPSG','22218','EPSG','3417','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22219','NAD83(CSRS)v2 / UTM zone 19N',NULL,'EPSG','4400','EPSG','8237','EPSG','16019',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17992','projected_crs','EPSG','22219','EPSG','3524','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22220','NAD83(CSRS)v2 / UTM zone 20N',NULL,'EPSG','4400','EPSG','8237','EPSG','16020',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17993','projected_crs','EPSG','22220','EPSG','3525','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22221','NAD83(CSRS)v2 / UTM zone 21N',NULL,'EPSG','4400','EPSG','8237','EPSG','16021',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17994','projected_crs','EPSG','22221','EPSG','2151','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22222','NAD83(CSRS)v2 / UTM zone 22N',NULL,'EPSG','4400','EPSG','8237','EPSG','16022',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17995','projected_crs','EPSG','22222','EPSG','2152','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22229','RGSH2020 / UTM zone 29N',NULL,'EPSG','4400','EPSG','10299','EPSG','16029',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19537','projected_crs','EPSG','22229','EPSG','1735','EPSG','1136');
INSERT INTO "projected_crs" VALUES('EPSG','22230','RGSH2020 / UTM zone 30N',NULL,'EPSG','4400','EPSG','10299','EPSG','16030',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19538','projected_crs','EPSG','22230','EPSG','3952','EPSG','1136');
INSERT INTO "projected_crs" VALUES('EPSG','22231','RGSH2020 / UTM zone 31N',NULL,'EPSG','4400','EPSG','10299','EPSG','16031',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19539','projected_crs','EPSG','22231','EPSG','3953','EPSG','1136');
INSERT INTO "projected_crs" VALUES('EPSG','22232','RGSH2020 / UTM zone 32N',NULL,'EPSG','4400','EPSG','10299','EPSG','16032',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19541','projected_crs','EPSG','22232','EPSG','3954','EPSG','1136');
INSERT INTO "projected_crs" VALUES('EPSG','22234','Cape / UTM zone 34S',NULL,'EPSG','4400','EPSG','4222','EPSG','16134',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6361','projected_crs','EPSG','22234','EPSG','1615','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22235','Cape / UTM zone 35S',NULL,'EPSG','4400','EPSG','4222','EPSG','16135',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6362','projected_crs','EPSG','22235','EPSG','1617','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22236','Cape / UTM zone 36S',NULL,'EPSG','4400','EPSG','4222','EPSG','16136',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6363','projected_crs','EPSG','22236','EPSG','1616','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22239','NAD83(CSRS)v2 / PEI Stereographic',NULL,'EPSG','4496','EPSG','8237','EPSG','19960',NULL,0);
INSERT INTO "usage" VALUES('EPSG','18015','projected_crs','EPSG','22239','EPSG','1533','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22240','NAD83(CSRS)v2 / NB Stereographic',NULL,'EPSG','4500','EPSG','8237','EPSG','19946',NULL,0);
INSERT INTO "usage" VALUES('EPSG','18016','projected_crs','EPSG','22240','EPSG','1447','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22243','NAD83(CSRS)v2 / SCoPQ zone 3',NULL,'EPSG','4499','EPSG','8237','EPSG','17703',NULL,0);
INSERT INTO "usage" VALUES('EPSG','18017','projected_crs','EPSG','22243','EPSG','1446','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22244','NAD83(CSRS)v2 / SCoPQ zone 4',NULL,'EPSG','4499','EPSG','8237','EPSG','17704',NULL,0);
INSERT INTO "usage" VALUES('EPSG','18018','projected_crs','EPSG','22244','EPSG','1422','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22245','NAD83(CSRS)v2 / SCoPQ zone 5',NULL,'EPSG','4499','EPSG','8237','EPSG','17705',NULL,0);
INSERT INTO "usage" VALUES('EPSG','18019','projected_crs','EPSG','22245','EPSG','1423','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22246','NAD83(CSRS)v2 / SCoPQ zone 6',NULL,'EPSG','4499','EPSG','8237','EPSG','17706',NULL,0);
INSERT INTO "usage" VALUES('EPSG','18020','projected_crs','EPSG','22246','EPSG','1424','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22247','NAD83(CSRS)v2 / SCoPQ zone 7',NULL,'EPSG','4499','EPSG','8237','EPSG','17707',NULL,0);
INSERT INTO "usage" VALUES('EPSG','18021','projected_crs','EPSG','22247','EPSG','1425','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22248','NAD83(CSRS)v2 / SCoPQ zone 8',NULL,'EPSG','4499','EPSG','8237','EPSG','17708',NULL,0);
INSERT INTO "usage" VALUES('EPSG','18074','projected_crs','EPSG','22248','EPSG','1426','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22249','NAD83(CSRS)v2 / SCoPQ zone 9',NULL,'EPSG','4499','EPSG','8237','EPSG','17709',NULL,0);
INSERT INTO "usage" VALUES('EPSG','18076','projected_crs','EPSG','22249','EPSG','1427','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22250','NAD83(CSRS)v2 / SCoPQ zone 10',NULL,'EPSG','4499','EPSG','8237','EPSG','17710',NULL,0);
INSERT INTO "usage" VALUES('EPSG','18077','projected_crs','EPSG','22250','EPSG','1428','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22262','NAD83(CSRS)v2 / Alberta 3TM ref merid 111 W',NULL,'EPSG','4400','EPSG','8237','EPSG','17722',NULL,0);
INSERT INTO "usage" VALUES('EPSG','18009','projected_crs','EPSG','22262','EPSG','3543','EPSG','1096');
INSERT INTO "projected_crs" VALUES('EPSG','22263','NAD83(CSRS)v2 / Alberta 3TM ref merid 114 W',NULL,'EPSG','4400','EPSG','8237','EPSG','17723',NULL,0);
INSERT INTO "usage" VALUES('EPSG','18010','projected_crs','EPSG','22263','EPSG','3542','EPSG','1096');
INSERT INTO "projected_crs" VALUES('EPSG','22264','NAD83(CSRS)v2 / Alberta 3TM ref merid 117 W',NULL,'EPSG','4400','EPSG','8237','EPSG','17724',NULL,0);
INSERT INTO "usage" VALUES('EPSG','18011','projected_crs','EPSG','22264','EPSG','3541','EPSG','1096');
INSERT INTO "projected_crs" VALUES('EPSG','22265','NAD83(CSRS)v2 / Alberta 3TM ref merid 120 W',NULL,'EPSG','4400','EPSG','8237','EPSG','17726',NULL,0);
INSERT INTO "usage" VALUES('EPSG','18012','projected_crs','EPSG','22265','EPSG','3540','EPSG','1096');
INSERT INTO "projected_crs" VALUES('EPSG','22275','Cape / Lo15',NULL,'EPSG','6503','EPSG','4222','EPSG','17515',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6364','projected_crs','EPSG','22275','EPSG','1454','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','22277','Cape / Lo17',NULL,'EPSG','6503','EPSG','4222','EPSG','17517',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6365','projected_crs','EPSG','22277','EPSG','1455','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','22279','Cape / Lo19',NULL,'EPSG','6503','EPSG','4222','EPSG','17519',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6366','projected_crs','EPSG','22279','EPSG','1456','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','22281','Cape / Lo21',NULL,'EPSG','6503','EPSG','4222','EPSG','17521',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6367','projected_crs','EPSG','22281','EPSG','1457','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','22283','Cape / Lo23',NULL,'EPSG','6503','EPSG','4222','EPSG','17523',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6368','projected_crs','EPSG','22283','EPSG','1458','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','22285','Cape / Lo25',NULL,'EPSG','6503','EPSG','4222','EPSG','17525',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6369','projected_crs','EPSG','22285','EPSG','1459','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','22287','Cape / Lo27',NULL,'EPSG','6503','EPSG','4222','EPSG','17527',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6370','projected_crs','EPSG','22287','EPSG','1460','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','22289','Cape / Lo29',NULL,'EPSG','6503','EPSG','4222','EPSG','17529',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6371','projected_crs','EPSG','22289','EPSG','1461','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','22291','Cape / Lo31',NULL,'EPSG','6503','EPSG','4222','EPSG','17531',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6372','projected_crs','EPSG','22291','EPSG','1462','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','22293','Cape / Lo33',NULL,'EPSG','6503','EPSG','4222','EPSG','17533',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6373','projected_crs','EPSG','22293','EPSG','1463','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','22300','Carthage (Paris) / Tunisia Mining Grid',NULL,'EPSG','4406','EPSG','4816','EPSG','19937',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6374','projected_crs','EPSG','22300','EPSG','1618','EPSG','1249');
INSERT INTO "projected_crs" VALUES('EPSG','22307','NAD83(CSRS)v3 / UTM zone 7N',NULL,'EPSG','4400','EPSG','8240','EPSG','16007',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17752','projected_crs','EPSG','22307','EPSG','3409','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22308','NAD83(CSRS)v3 / UTM zone 8N',NULL,'EPSG','4400','EPSG','8240','EPSG','16008',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17834','projected_crs','EPSG','22308','EPSG','3410','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22309','NAD83(CSRS)v3 / UTM zone 9N',NULL,'EPSG','4400','EPSG','8240','EPSG','16009',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17771','projected_crs','EPSG','22309','EPSG','3411','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22310','NAD83(CSRS)v3 / UTM zone 10N',NULL,'EPSG','4400','EPSG','8240','EPSG','16010',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17839','projected_crs','EPSG','22310','EPSG','3412','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22311','NAD83(CSRS)v3 / UTM zone 11N',NULL,'EPSG','4400','EPSG','8240','EPSG','16011',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17777','projected_crs','EPSG','22311','EPSG','3528','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22312','NAD83(CSRS)v3 / UTM zone 12N',NULL,'EPSG','4400','EPSG','8240','EPSG','16012',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17783','projected_crs','EPSG','22312','EPSG','3527','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22313','NAD83(CSRS)v3 / UTM zone 13N',NULL,'EPSG','4400','EPSG','8240','EPSG','16013',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17789','projected_crs','EPSG','22313','EPSG','3526','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22314','NAD83(CSRS)v3 / UTM zone 14N',NULL,'EPSG','4400','EPSG','8240','EPSG','16014',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17795','projected_crs','EPSG','22314','EPSG','3413','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22315','NAD83(CSRS)v3 / UTM zone 15N',NULL,'EPSG','4400','EPSG','8240','EPSG','16015',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17801','projected_crs','EPSG','22315','EPSG','3414','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22316','NAD83(CSRS)v3 / UTM zone 16N',NULL,'EPSG','4400','EPSG','8240','EPSG','16016',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17829','projected_crs','EPSG','22316','EPSG','3415','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22317','NAD83(CSRS)v3 / UTM zone 17N',NULL,'EPSG','4400','EPSG','8240','EPSG','16017',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17816','projected_crs','EPSG','22317','EPSG','3416','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22318','NAD83(CSRS)v3 / UTM zone 18N',NULL,'EPSG','4400','EPSG','8240','EPSG','16018',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17822','projected_crs','EPSG','22318','EPSG','3417','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22319','NAD83(CSRS)v3 / UTM zone 19N',NULL,'EPSG','4400','EPSG','8240','EPSG','16019',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17844','projected_crs','EPSG','22319','EPSG','3524','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22320','NAD83(CSRS)v3 / UTM zone 20N',NULL,'EPSG','4400','EPSG','8240','EPSG','16020',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17850','projected_crs','EPSG','22320','EPSG','3525','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22321','NAD83(CSRS)v3 / UTM zone 21N',NULL,'EPSG','4400','EPSG','8240','EPSG','16021',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17857','projected_crs','EPSG','22321','EPSG','2151','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22322','NAD83(CSRS)v3 / UTM zone 22N',NULL,'EPSG','4400','EPSG','8240','EPSG','16022',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17863','projected_crs','EPSG','22322','EPSG','2152','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22332','Carthage / UTM zone 32N',NULL,'EPSG','4400','EPSG','4223','EPSG','16032',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6375','projected_crs','EPSG','22332','EPSG','1489','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22337','NAD83(CSRS)v3 / MTM NS 1997 zone 5',NULL,'EPSG','4400','EPSG','8240','EPSG','9982',NULL,0);
INSERT INTO "usage" VALUES('EPSG','18013','projected_crs','EPSG','22337','EPSG','1535','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22338','NAD83(CSRS)v3 / MTM NS 1997 zone 4',NULL,'EPSG','4400','EPSG','8240','EPSG','9981',NULL,0);
INSERT INTO "usage" VALUES('EPSG','18014','projected_crs','EPSG','22338','EPSG','1534','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22348','NAD83(CSRS)v3 / MTM zone 8',NULL,'EPSG','4496','EPSG','8240','EPSG','17708',NULL,0);
INSERT INTO "usage" VALUES('EPSG','18071','projected_crs','EPSG','22348','EPSG','1429','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22349','NAD83(CSRS)v3 / MTM zone 9',NULL,'EPSG','4496','EPSG','8240','EPSG','17709',NULL,0);
INSERT INTO "usage" VALUES('EPSG','18072','projected_crs','EPSG','22349','EPSG','1430','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22350','NAD83(CSRS)v3 / MTM zone 10',NULL,'EPSG','4496','EPSG','8240','EPSG','17710',NULL,0);
INSERT INTO "usage" VALUES('EPSG','18073','projected_crs','EPSG','22350','EPSG','1431','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22351','NAD83(CSRS)v3 / MTM zone 11',NULL,'EPSG','4400','EPSG','8240','EPSG','17711',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17639','projected_crs','EPSG','22351','EPSG','1432','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22352','NAD83(CSRS)v3 / MTM zone 12',NULL,'EPSG','4400','EPSG','8240','EPSG','17712',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17640','projected_crs','EPSG','22352','EPSG','1433','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22353','NAD83(CSRS)v3 / MTM zone 13',NULL,'EPSG','4400','EPSG','8240','EPSG','17713',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17641','projected_crs','EPSG','22353','EPSG','1434','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22354','NAD83(CSRS)v3 / MTM zone 14',NULL,'EPSG','4400','EPSG','8240','EPSG','17714',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17642','projected_crs','EPSG','22354','EPSG','1435','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22355','NAD83(CSRS)v3 / MTM zone 15',NULL,'EPSG','4400','EPSG','8240','EPSG','17715',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17643','projected_crs','EPSG','22355','EPSG','1436','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22356','NAD83(CSRS)v3 / MTM zone 16',NULL,'EPSG','4400','EPSG','8240','EPSG','17716',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17644','projected_crs','EPSG','22356','EPSG','1437','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22357','NAD83(CSRS)v3 / MTM zone 17',NULL,'EPSG','4400','EPSG','8240','EPSG','17717',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17645','projected_crs','EPSG','22357','EPSG','1438','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22391','Carthage / Nord Tunisie',NULL,'EPSG','4499','EPSG','4223','EPSG','18181',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6376','projected_crs','EPSG','22391','EPSG','1619','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22392','Carthage / Sud Tunisie',NULL,'EPSG','4499','EPSG','4223','EPSG','18182',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6377','projected_crs','EPSG','22392','EPSG','1620','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22407','NAD83(CSRS)v4 / UTM zone 7N',NULL,'EPSG','4400','EPSG','8246','EPSG','16007',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17761','projected_crs','EPSG','22407','EPSG','3409','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22408','NAD83(CSRS)v4 / UTM zone 8N',NULL,'EPSG','4400','EPSG','8246','EPSG','16008',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17835','projected_crs','EPSG','22408','EPSG','3410','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22409','NAD83(CSRS)v4 / UTM zone 9N',NULL,'EPSG','4400','EPSG','8246','EPSG','16009',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17766','projected_crs','EPSG','22409','EPSG','3411','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22410','NAD83(CSRS)v4 / UTM zone 10N',NULL,'EPSG','4400','EPSG','8246','EPSG','16010',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17840','projected_crs','EPSG','22410','EPSG','3412','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22411','NAD83(CSRS)v4 / UTM zone 11N',NULL,'EPSG','4400','EPSG','8246','EPSG','16011',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17778','projected_crs','EPSG','22411','EPSG','3528','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22412','NAD83(CSRS)v4 / UTM zone 12N',NULL,'EPSG','4400','EPSG','8246','EPSG','16012',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17784','projected_crs','EPSG','22412','EPSG','3527','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22413','NAD83(CSRS)v4 / UTM zone 13N',NULL,'EPSG','4400','EPSG','8246','EPSG','16013',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17790','projected_crs','EPSG','22413','EPSG','3526','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22414','NAD83(CSRS)v4 / UTM zone 14N',NULL,'EPSG','4400','EPSG','8246','EPSG','16014',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17796','projected_crs','EPSG','22414','EPSG','3413','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22415','NAD83(CSRS)v4 / UTM zone 15N',NULL,'EPSG','4400','EPSG','8246','EPSG','16015',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17827','projected_crs','EPSG','22415','EPSG','3414','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22416','NAD83(CSRS)v4 / UTM zone 16N',NULL,'EPSG','4400','EPSG','8246','EPSG','16016',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17830','projected_crs','EPSG','22416','EPSG','3415','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22417','NAD83(CSRS)v4 / UTM zone 17N',NULL,'EPSG','4400','EPSG','8246','EPSG','16017',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17817','projected_crs','EPSG','22417','EPSG','3416','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22418','NAD83(CSRS)v4 / UTM zone 18N',NULL,'EPSG','4400','EPSG','8246','EPSG','16018',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17823','projected_crs','EPSG','22418','EPSG','3417','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22419','NAD83(CSRS)v4 / UTM zone 19N',NULL,'EPSG','4400','EPSG','8246','EPSG','16019',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17845','projected_crs','EPSG','22419','EPSG','3524','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22420','NAD83(CSRS)v4 / UTM zone 20N',NULL,'EPSG','4400','EPSG','8246','EPSG','16020',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17851','projected_crs','EPSG','22420','EPSG','3525','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22421','NAD83(CSRS)v4 / UTM zone 21N',NULL,'EPSG','4400','EPSG','8246','EPSG','16021',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17858','projected_crs','EPSG','22421','EPSG','2151','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22422','NAD83(CSRS)v4 / UTM zone 22N',NULL,'EPSG','4400','EPSG','8246','EPSG','16022',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17864','projected_crs','EPSG','22422','EPSG','2152','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22462','NAD83(CSRS)v4 / Alberta 3TM ref merid 111 W',NULL,'EPSG','4400','EPSG','8246','EPSG','17722',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17657','projected_crs','EPSG','22462','EPSG','3543','EPSG','1096');
INSERT INTO "projected_crs" VALUES('EPSG','22463','NAD83(CSRS)v4 / Alberta 3TM ref merid 114 W',NULL,'EPSG','4400','EPSG','8246','EPSG','17723',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17658','projected_crs','EPSG','22463','EPSG','3542','EPSG','1096');
INSERT INTO "projected_crs" VALUES('EPSG','22464','NAD83(CSRS)v4 / Alberta 3TM ref merid 117 W',NULL,'EPSG','4400','EPSG','8246','EPSG','17724',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17659','projected_crs','EPSG','22464','EPSG','3541','EPSG','1096');
INSERT INTO "projected_crs" VALUES('EPSG','22465','NAD83(CSRS)v4 / Alberta 3TM ref merid 120 W',NULL,'EPSG','4400','EPSG','8246','EPSG','17726',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17660','projected_crs','EPSG','22465','EPSG','3540','EPSG','1096');
INSERT INTO "projected_crs" VALUES('EPSG','22521','Corrego Alegre 1970-72 / UTM zone 21S',NULL,'EPSG','4400','EPSG','4225','EPSG','16121',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6378','projected_crs','EPSG','22521','EPSG','3355','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22522','Corrego Alegre 1970-72 / UTM zone 22S',NULL,'EPSG','4400','EPSG','4225','EPSG','16122',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6379','projected_crs','EPSG','22522','EPSG','3176','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22523','Corrego Alegre 1970-72 / UTM zone 23S',NULL,'EPSG','4400','EPSG','4225','EPSG','16123',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6380','projected_crs','EPSG','22523','EPSG','3177','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22524','Corrego Alegre 1970-72 / UTM zone 24S',NULL,'EPSG','4400','EPSG','4225','EPSG','16124',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6381','projected_crs','EPSG','22524','EPSG','1818','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22525','Corrego Alegre 1970-72 / UTM zone 25S',NULL,'EPSG','4400','EPSG','4225','EPSG','16125',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6382','projected_crs','EPSG','22525','EPSG','3178','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22607','NAD83(CSRS)v6 / UTM zone 7N',NULL,'EPSG','4400','EPSG','8252','EPSG','16007',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17755','projected_crs','EPSG','22607','EPSG','3409','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22608','NAD83(CSRS)v6 / UTM zone 8N',NULL,'EPSG','4400','EPSG','8252','EPSG','16008',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17836','projected_crs','EPSG','22608','EPSG','3410','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22609','NAD83(CSRS)v6 / UTM zone 9N',NULL,'EPSG','4400','EPSG','8252','EPSG','16009',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17768','projected_crs','EPSG','22609','EPSG','3411','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22610','NAD83(CSRS)v6 / UTM zone 10N',NULL,'EPSG','4400','EPSG','8252','EPSG','16010',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17841','projected_crs','EPSG','22610','EPSG','3412','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22611','NAD83(CSRS)v6 / UTM zone 11N',NULL,'EPSG','4400','EPSG','8252','EPSG','16011',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17779','projected_crs','EPSG','22611','EPSG','3528','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22612','NAD83(CSRS)v6 / UTM zone 12N',NULL,'EPSG','4400','EPSG','8252','EPSG','16012',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17786','projected_crs','EPSG','22612','EPSG','3527','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22613','NAD83(CSRS)v6 / UTM zone 13N',NULL,'EPSG','4400','EPSG','8252','EPSG','16013',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17791','projected_crs','EPSG','22613','EPSG','3526','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22614','NAD83(CSRS)v6 / UTM zone 14N',NULL,'EPSG','4400','EPSG','8252','EPSG','16014',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17797','projected_crs','EPSG','22614','EPSG','3413','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22615','NAD83(CSRS)v6 / UTM zone 15N',NULL,'EPSG','4400','EPSG','8252','EPSG','16015',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17803','projected_crs','EPSG','22615','EPSG','3414','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22616','NAD83(CSRS)v6 / UTM zone 16N',NULL,'EPSG','4400','EPSG','8252','EPSG','16016',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17831','projected_crs','EPSG','22616','EPSG','3415','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22617','NAD83(CSRS)v6 / UTM zone 17N',NULL,'EPSG','4400','EPSG','8252','EPSG','16017',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17818','projected_crs','EPSG','22617','EPSG','3416','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22618','NAD83(CSRS)v6 / UTM zone 18N',NULL,'EPSG','4400','EPSG','8252','EPSG','16018',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17824','projected_crs','EPSG','22618','EPSG','3417','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22619','NAD83(CSRS)v6 / UTM zone 19N',NULL,'EPSG','4400','EPSG','8252','EPSG','16019',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17846','projected_crs','EPSG','22619','EPSG','3524','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22620','NAD83(CSRS)v6 / UTM zone 20N',NULL,'EPSG','4400','EPSG','8252','EPSG','16020',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17852','projected_crs','EPSG','22620','EPSG','3525','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22621','NAD83(CSRS)v6 / UTM zone 21N',NULL,'EPSG','4400','EPSG','8252','EPSG','16021',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17859','projected_crs','EPSG','22621','EPSG','2151','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22622','NAD83(CSRS)v6 / UTM zone 22N',NULL,'EPSG','4400','EPSG','8252','EPSG','16022',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17865','projected_crs','EPSG','22622','EPSG','2152','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22639','NAD83(CSRS)v6 / PEI Stereographic',NULL,'EPSG','4496','EPSG','8252','EPSG','19960',NULL,0);
INSERT INTO "usage" VALUES('EPSG','18070','projected_crs','EPSG','22639','EPSG','1533','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22641','NAD83(CSRS)v6 / MTM zone 1',NULL,'EPSG','4496','EPSG','8252','EPSG','17701',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17665','projected_crs','EPSG','22641','EPSG','2226','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22642','NAD83(CSRS)v6 / MTM zone 2',NULL,'EPSG','4496','EPSG','8252','EPSG','17702',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17666','projected_crs','EPSG','22642','EPSG','2227','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22643','NAD83(CSRS)v6 / MTM zone 3',NULL,'EPSG','4496','EPSG','8252','EPSG','17703',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17667','projected_crs','EPSG','22643','EPSG','2275','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22644','NAD83(CSRS)v6 / MTM zone 4',NULL,'EPSG','4496','EPSG','8252','EPSG','17704',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17668','projected_crs','EPSG','22644','EPSG','3875','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22645','NAD83(CSRS)v6 / MTM zone 5',NULL,'EPSG','4496','EPSG','8252','EPSG','17705',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17669','projected_crs','EPSG','22645','EPSG','3865','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22646','NAD83(CSRS)v6 / MTM zone 6',NULL,'EPSG','4496','EPSG','8252','EPSG','17706',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17670','projected_crs','EPSG','22646','EPSG','3880','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22648','NAD83(CSRS)v6 / MTM zone 8',NULL,'EPSG','4496','EPSG','8252','EPSG','17708',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17688','projected_crs','EPSG','22648','EPSG','1429','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22649','NAD83(CSRS)v6 / MTM zone 9',NULL,'EPSG','4496','EPSG','8252','EPSG','17709',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17689','projected_crs','EPSG','22649','EPSG','1430','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22650','NAD83(CSRS)v6 / MTM zone 10',NULL,'EPSG','4496','EPSG','8252','EPSG','17710',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17690','projected_crs','EPSG','22650','EPSG','1431','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22651','NAD83(CSRS)v6 / MTM zone 11',NULL,'EPSG','4400','EPSG','8252','EPSG','17711',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17646','projected_crs','EPSG','22651','EPSG','1432','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22652','NAD83(CSRS)v6 / MTM zone 12',NULL,'EPSG','4400','EPSG','8252','EPSG','17712',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17647','projected_crs','EPSG','22652','EPSG','1433','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22653','NAD83(CSRS)v6 / MTM zone 13',NULL,'EPSG','4400','EPSG','8252','EPSG','17713',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17648','projected_crs','EPSG','22653','EPSG','1434','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22654','NAD83(CSRS)v6 / MTM zone 14',NULL,'EPSG','4400','EPSG','8252','EPSG','17714',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17649','projected_crs','EPSG','22654','EPSG','1435','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22655','NAD83(CSRS)v6 / MTM zone 15',NULL,'EPSG','4400','EPSG','8252','EPSG','17715',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17650','projected_crs','EPSG','22655','EPSG','1436','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22656','NAD83(CSRS)v6 / MTM zone 16',NULL,'EPSG','4400','EPSG','8252','EPSG','17716',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17651','projected_crs','EPSG','22656','EPSG','1437','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22657','NAD83(CSRS)v6 / MTM zone 17',NULL,'EPSG','4400','EPSG','8252','EPSG','17717',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17652','projected_crs','EPSG','22657','EPSG','1438','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22700','Deir ez Zor / Levant Zone',NULL,'EPSG','4499','EPSG','4227','EPSG','19940',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6383','projected_crs','EPSG','22700','EPSG','1623','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22707','NAD83(CSRS)v7 / UTM zone 7N',NULL,'EPSG','4400','EPSG','8255','EPSG','16007',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17756','projected_crs','EPSG','22707','EPSG','3409','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22708','NAD83(CSRS)v7 / UTM zone 8N',NULL,'EPSG','4400','EPSG','8255','EPSG','16008',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17837','projected_crs','EPSG','22708','EPSG','3410','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22709','NAD83(CSRS)v7 / UTM zone 9N',NULL,'EPSG','4400','EPSG','8255','EPSG','16009',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17769','projected_crs','EPSG','22709','EPSG','3411','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22710','NAD83(CSRS)v7 / UTM zone 10N',NULL,'EPSG','4400','EPSG','8255','EPSG','16010',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17842','projected_crs','EPSG','22710','EPSG','3412','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22711','NAD83(CSRS)v7 / UTM zone 11N',NULL,'EPSG','4400','EPSG','8255','EPSG','16011',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17781','projected_crs','EPSG','22711','EPSG','3528','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22712','NAD83(CSRS)v7 / UTM zone 12N',NULL,'EPSG','4400','EPSG','8255','EPSG','16012',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17785','projected_crs','EPSG','22712','EPSG','3527','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22713','NAD83(CSRS)v7 / UTM zone 13N',NULL,'EPSG','4400','EPSG','8255','EPSG','16013',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17792','projected_crs','EPSG','22713','EPSG','3526','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22714','NAD83(CSRS)v7 / UTM zone 14N',NULL,'EPSG','4400','EPSG','8255','EPSG','16014',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17798','projected_crs','EPSG','22714','EPSG','3413','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22715','NAD83(CSRS)v7 / UTM zone 15N',NULL,'EPSG','4400','EPSG','8255','EPSG','16015',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17804','projected_crs','EPSG','22715','EPSG','3414','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22716','NAD83(CSRS)v7 / UTM zone 16N',NULL,'EPSG','4400','EPSG','8255','EPSG','16016',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17832','projected_crs','EPSG','22716','EPSG','3415','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22717','NAD83(CSRS)v7 / UTM zone 17N',NULL,'EPSG','4400','EPSG','8255','EPSG','16017',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17819','projected_crs','EPSG','22717','EPSG','3416','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22718','NAD83(CSRS)v7 / UTM zone 18N',NULL,'EPSG','4400','EPSG','8255','EPSG','16018',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17825','projected_crs','EPSG','22718','EPSG','3417','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22719','NAD83(CSRS)v7 / UTM zone 19N',NULL,'EPSG','4400','EPSG','8255','EPSG','16019',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17847','projected_crs','EPSG','22719','EPSG','3524','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22720','NAD83(CSRS)v7 / UTM zone 20N',NULL,'EPSG','4400','EPSG','8255','EPSG','16020',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17855','projected_crs','EPSG','22720','EPSG','3525','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22721','NAD83(CSRS)v7 / UTM zone 21N',NULL,'EPSG','4400','EPSG','8255','EPSG','16021',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17860','projected_crs','EPSG','22721','EPSG','2151','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22722','NAD83(CSRS)v7 / UTM zone 22N',NULL,'EPSG','4400','EPSG','8255','EPSG','16022',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17866','projected_crs','EPSG','22722','EPSG','2152','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22739','NAD83(CSRS)v7 / PEI Stereographic',NULL,'EPSG','4496','EPSG','8255','EPSG','19960',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17675','projected_crs','EPSG','22739','EPSG','1533','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22762','NAD83(CSRS)v7 / Alberta 3TM ref merid 111 W',NULL,'EPSG','4400','EPSG','8255','EPSG','17722',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17661','projected_crs','EPSG','22762','EPSG','3543','EPSG','1096');
INSERT INTO "projected_crs" VALUES('EPSG','22763','NAD83(CSRS)v7 / Alberta 3TM ref merid 114 W',NULL,'EPSG','4400','EPSG','8255','EPSG','17723',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17662','projected_crs','EPSG','22763','EPSG','3542','EPSG','1096');
INSERT INTO "projected_crs" VALUES('EPSG','22764','NAD83(CSRS)v7 / Alberta 3TM ref merid 117 W',NULL,'EPSG','4400','EPSG','8255','EPSG','17724',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17663','projected_crs','EPSG','22764','EPSG','3541','EPSG','1096');
INSERT INTO "projected_crs" VALUES('EPSG','22765','NAD83(CSRS)v7 / Alberta 3TM ref merid 120 W',NULL,'EPSG','4400','EPSG','8255','EPSG','17726',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17664','projected_crs','EPSG','22765','EPSG','3540','EPSG','1096');
INSERT INTO "projected_crs" VALUES('EPSG','22770','Deir ez Zor / Syria Lambert',NULL,'EPSG','4499','EPSG','4227','EPSG','19948',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6384','projected_crs','EPSG','22770','EPSG','1623','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22780','Deir ez Zor / Levant Stereographic',NULL,'EPSG','4499','EPSG','4227','EPSG','19949',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6385','projected_crs','EPSG','22780','EPSG','1623','EPSG','1207');
INSERT INTO "projected_crs" VALUES('EPSG','22807','NAD83(CSRS)v8 / UTM zone 7N',NULL,'EPSG','4400','EPSG','10414','EPSG','16007',NULL,0);
INSERT INTO "usage" VALUES('EPSG','20167','projected_crs','EPSG','22807','EPSG','3409','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22808','NAD83(CSRS)v8 / UTM zone 8N',NULL,'EPSG','4400','EPSG','10414','EPSG','16008',NULL,0);
INSERT INTO "usage" VALUES('EPSG','20168','projected_crs','EPSG','22808','EPSG','3410','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22809','NAD83(CSRS)v8 / UTM zone 9N',NULL,'EPSG','4400','EPSG','10414','EPSG','16009',NULL,0);
INSERT INTO "usage" VALUES('EPSG','20169','projected_crs','EPSG','22809','EPSG','3411','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22810','NAD83(CSRS)v8 / UTM zone 10N',NULL,'EPSG','4400','EPSG','10414','EPSG','16010',NULL,0);
INSERT INTO "usage" VALUES('EPSG','20170','projected_crs','EPSG','22810','EPSG','3412','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22811','NAD83(CSRS)v8 / UTM zone 11N',NULL,'EPSG','4400','EPSG','10414','EPSG','16011',NULL,0);
INSERT INTO "usage" VALUES('EPSG','20171','projected_crs','EPSG','22811','EPSG','3528','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22812','NAD83(CSRS)v8 / UTM zone 12N',NULL,'EPSG','4400','EPSG','10414','EPSG','16012',NULL,0);
INSERT INTO "usage" VALUES('EPSG','20172','projected_crs','EPSG','22812','EPSG','3527','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22813','NAD83(CSRS)v8 / UTM zone 13N',NULL,'EPSG','4400','EPSG','10414','EPSG','16013',NULL,0);
INSERT INTO "usage" VALUES('EPSG','20173','projected_crs','EPSG','22813','EPSG','3526','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22814','NAD83(CSRS)v8 / UTM zone 14N',NULL,'EPSG','4400','EPSG','10414','EPSG','16014',NULL,0);
INSERT INTO "usage" VALUES('EPSG','20174','projected_crs','EPSG','22814','EPSG','3413','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22815','NAD83(CSRS)v8 / UTM zone 15N',NULL,'EPSG','4400','EPSG','10414','EPSG','16015',NULL,0);
INSERT INTO "usage" VALUES('EPSG','20175','projected_crs','EPSG','22815','EPSG','3414','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22816','NAD83(CSRS)v8 / UTM zone 16N',NULL,'EPSG','4400','EPSG','10414','EPSG','16016',NULL,0);
INSERT INTO "usage" VALUES('EPSG','20176','projected_crs','EPSG','22816','EPSG','3415','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22817','NAD83(CSRS)v8 / UTM zone 17N',NULL,'EPSG','4400','EPSG','10414','EPSG','16017',NULL,0);
INSERT INTO "usage" VALUES('EPSG','20177','projected_crs','EPSG','22817','EPSG','3416','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22818','NAD83(CSRS)v8 / UTM zone 18N',NULL,'EPSG','4400','EPSG','10414','EPSG','16018',NULL,0);
INSERT INTO "usage" VALUES('EPSG','20178','projected_crs','EPSG','22818','EPSG','3417','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22819','NAD83(CSRS)v8 / UTM zone 19N',NULL,'EPSG','4400','EPSG','10414','EPSG','16019',NULL,0);
INSERT INTO "usage" VALUES('EPSG','20179','projected_crs','EPSG','22819','EPSG','3524','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22820','NAD83(CSRS)v8 / UTM zone 20N',NULL,'EPSG','4400','EPSG','10414','EPSG','16020',NULL,0);
INSERT INTO "usage" VALUES('EPSG','20180','projected_crs','EPSG','22820','EPSG','3525','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22821','NAD83(CSRS)v8 / UTM zone 21N',NULL,'EPSG','4400','EPSG','10414','EPSG','16021',NULL,0);
INSERT INTO "usage" VALUES('EPSG','20181','projected_crs','EPSG','22821','EPSG','2151','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22822','NAD83(CSRS)v8 / UTM zone 22N',NULL,'EPSG','4400','EPSG','10414','EPSG','16022',NULL,0);
INSERT INTO "usage" VALUES('EPSG','20182','projected_crs','EPSG','22822','EPSG','2152','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22832','Douala / UTM zone 32N',NULL,'EPSG','4400','EPSG','4228','EPSG','16032',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6386','projected_crs','EPSG','22832','EPSG','1060','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22991','Egypt 1907 / Blue Belt',NULL,'EPSG','4400','EPSG','4229','EPSG','18071',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6387','projected_crs','EPSG','22991','EPSG','1642','EPSG','1160');
INSERT INTO "projected_crs" VALUES('EPSG','22992','Egypt 1907 / Red Belt',NULL,'EPSG','4400','EPSG','4229','EPSG','18072',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6388','projected_crs','EPSG','22992','EPSG','1643','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22993','Egypt 1907 / Purple Belt',NULL,'EPSG','4400','EPSG','4229','EPSG','18073',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6389','projected_crs','EPSG','22993','EPSG','1644','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','22994','Egypt 1907 / Extended Purple Belt',NULL,'EPSG','4400','EPSG','4229','EPSG','18074',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6390','projected_crs','EPSG','22994','EPSG','1645','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','23028','ED50 / UTM zone 28N',NULL,'EPSG','4400','EPSG','4230','EPSG','16028',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6391','projected_crs','EPSG','23028','EPSG','1631','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','23029','ED50 / UTM zone 29N',NULL,'EPSG','4400','EPSG','4230','EPSG','16029',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6392','projected_crs','EPSG','23029','EPSG','1632','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','23030','ED50 / UTM zone 30N',NULL,'EPSG','4400','EPSG','4230','EPSG','16030',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6393','projected_crs','EPSG','23030','EPSG','1633','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','23031','ED50 / UTM zone 31N',NULL,'EPSG','4400','EPSG','4230','EPSG','16031',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6394','projected_crs','EPSG','23031','EPSG','1634','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','23032','ED50 / UTM zone 32N',NULL,'EPSG','4400','EPSG','4230','EPSG','16032',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6395','projected_crs','EPSG','23032','EPSG','1635','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','23033','ED50 / UTM zone 33N',NULL,'EPSG','4400','EPSG','4230','EPSG','16033',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6396','projected_crs','EPSG','23033','EPSG','1636','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','23034','ED50 / UTM zone 34N',NULL,'EPSG','4400','EPSG','4230','EPSG','16034',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6397','projected_crs','EPSG','23034','EPSG','1637','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','23035','ED50 / UTM zone 35N',NULL,'EPSG','4400','EPSG','4230','EPSG','16035',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6398','projected_crs','EPSG','23035','EPSG','1638','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','23036','ED50 / UTM zone 36N',NULL,'EPSG','4400','EPSG','4230','EPSG','16036',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6399','projected_crs','EPSG','23036','EPSG','1639','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','23037','ED50 / UTM zone 37N',NULL,'EPSG','4400','EPSG','4230','EPSG','16037',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6400','projected_crs','EPSG','23037','EPSG','1640','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','23038','ED50 / UTM zone 38N',NULL,'EPSG','4400','EPSG','4230','EPSG','16038',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6401','projected_crs','EPSG','23038','EPSG','1641','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','23090','ED50 / TM 0 N',NULL,'EPSG','4400','EPSG','4230','EPSG','16400',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6402','projected_crs','EPSG','23090','EPSG','1629','EPSG','1136');
INSERT INTO "projected_crs" VALUES('EPSG','23095','ED50 / TM 5 NE',NULL,'EPSG','4400','EPSG','4230','EPSG','16405',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6403','projected_crs','EPSG','23095','EPSG','1630','EPSG','1136');
INSERT INTO "projected_crs" VALUES('EPSG','23239','Fahud / UTM zone 39N',NULL,'EPSG','4400','EPSG','4232','EPSG','16039',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6404','projected_crs','EPSG','23239','EPSG','1544','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','23240','Fahud / UTM zone 40N',NULL,'EPSG','4400','EPSG','4232','EPSG','16040',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6405','projected_crs','EPSG','23240','EPSG','4008','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','23301','NAD83(2011) / ICS83-Freeport (ftUS)',NULL,'EPSG','1053','EPSG','6318','EPSG','11264',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19282','projected_crs','EPSG','23301','EPSG','4701','EPSG','1266');
INSERT INTO "projected_crs" VALUES('EPSG','23302','NAD83(2011) / ICS83-Rockford (ftUS)',NULL,'EPSG','1053','EPSG','6318','EPSG','11265',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19283','projected_crs','EPSG','23302','EPSG','4702','EPSG','1266');
INSERT INTO "projected_crs" VALUES('EPSG','23303','NAD83(2011) / ICS83-Aurora (ftUS)',NULL,'EPSG','1053','EPSG','6318','EPSG','11233',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19284','projected_crs','EPSG','23303','EPSG','4703','EPSG','1266');
INSERT INTO "projected_crs" VALUES('EPSG','23304','NAD83(2011) / ICS83-Chicago (ftUS)',NULL,'EPSG','1053','EPSG','6318','EPSG','11234',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19285','projected_crs','EPSG','23304','EPSG','4704','EPSG','1266');
INSERT INTO "projected_crs" VALUES('EPSG','23305','NAD83(2011) / ICS83-Moline (ftUS)',NULL,'EPSG','1053','EPSG','6318','EPSG','11235',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19286','projected_crs','EPSG','23305','EPSG','4705','EPSG','1266');
INSERT INTO "projected_crs" VALUES('EPSG','23306','NAD83(2011) / ICS83-Sterling (ftUS)',NULL,'EPSG','1053','EPSG','6318','EPSG','11236',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19287','projected_crs','EPSG','23306','EPSG','4706','EPSG','1266');
INSERT INTO "projected_crs" VALUES('EPSG','23307','NAD83(2011) / ICS83-Ottawa (ftUS)',NULL,'EPSG','1053','EPSG','6318','EPSG','11237',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19288','projected_crs','EPSG','23307','EPSG','4707','EPSG','1266');
INSERT INTO "projected_crs" VALUES('EPSG','23308','NAD83(2011) / ICS83-Joliet (ftUS)',NULL,'EPSG','1053','EPSG','6318','EPSG','11238',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19289','projected_crs','EPSG','23308','EPSG','4708','EPSG','1266');
INSERT INTO "projected_crs" VALUES('EPSG','23309','NAD83(2011) / ICS83-Monmouth (ftUS)',NULL,'EPSG','1053','EPSG','6318','EPSG','11239',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19290','projected_crs','EPSG','23309','EPSG','4709','EPSG','1266');
INSERT INTO "projected_crs" VALUES('EPSG','23310','NAD83(2011) / ICS83-Galesburg (ftUS)',NULL,'EPSG','1053','EPSG','6318','EPSG','11240',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19291','projected_crs','EPSG','23310','EPSG','4710','EPSG','1266');
INSERT INTO "projected_crs" VALUES('EPSG','23311','NAD83(2011) / ICS83-Peoria (ftUS)',NULL,'EPSG','1053','EPSG','6318','EPSG','11241',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19292','projected_crs','EPSG','23311','EPSG','4711','EPSG','1266');
INSERT INTO "projected_crs" VALUES('EPSG','23312','NAD83(2011) / ICS83-Eureka (ftUS)',NULL,'EPSG','1053','EPSG','6318','EPSG','11242',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19293','projected_crs','EPSG','23312','EPSG','4712','EPSG','1266');
INSERT INTO "projected_crs" VALUES('EPSG','23313','NAD83(2011) / ICS83-Bloomington (ftUS)',NULL,'EPSG','1053','EPSG','6318','EPSG','11243',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19294','projected_crs','EPSG','23313','EPSG','4713','EPSG','1266');
INSERT INTO "projected_crs" VALUES('EPSG','23314','NAD83(2011) / ICS83-Pontiac (ftUS)',NULL,'EPSG','1053','EPSG','6318','EPSG','11244',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19295','projected_crs','EPSG','23314','EPSG','4714','EPSG','1266');
INSERT INTO "projected_crs" VALUES('EPSG','23315','NAD83(2011) / ICS83-Watseka (ftUS)',NULL,'EPSG','1053','EPSG','6318','EPSG','11245',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19296','projected_crs','EPSG','23315','EPSG','4715','EPSG','1266');
INSERT INTO "projected_crs" VALUES('EPSG','23316','NAD83(2011) / ICS83-Quincy (ftUS)',NULL,'EPSG','1053','EPSG','6318','EPSG','11246',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19297','projected_crs','EPSG','23316','EPSG','4716','EPSG','1266');
INSERT INTO "projected_crs" VALUES('EPSG','23317','NAD83(2011) / ICS83-Macomb (ftUS)',NULL,'EPSG','1053','EPSG','6318','EPSG','11247',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19298','projected_crs','EPSG','23317','EPSG','4717','EPSG','1266');
INSERT INTO "projected_crs" VALUES('EPSG','23318','NAD83(2011) / ICS83-Lincoln (ftUS)',NULL,'EPSG','1053','EPSG','6318','EPSG','11248',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19299','projected_crs','EPSG','23318','EPSG','4718','EPSG','1266');
INSERT INTO "projected_crs" VALUES('EPSG','23319','NAD83(2011) / ICS83-Decatur (ftUS)',NULL,'EPSG','1053','EPSG','6318','EPSG','11249',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19300','projected_crs','EPSG','23319','EPSG','4719','EPSG','1266');
INSERT INTO "projected_crs" VALUES('EPSG','23320','NAD83(2011) / ICS83-Champaign (ftUS)',NULL,'EPSG','1053','EPSG','6318','EPSG','11250',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19301','projected_crs','EPSG','23320','EPSG','4720','EPSG','1266');
INSERT INTO "projected_crs" VALUES('EPSG','23321','NAD83(2011) / ICS83-Jacksonville (ftUS)',NULL,'EPSG','1053','EPSG','6318','EPSG','11251',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19302','projected_crs','EPSG','23321','EPSG','4721','EPSG','1266');
INSERT INTO "projected_crs" VALUES('EPSG','23322','NAD83(2011) / ICS83-Springfield (ftUS)',NULL,'EPSG','1053','EPSG','6318','EPSG','11252',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19303','projected_crs','EPSG','23322','EPSG','4722','EPSG','1266');
INSERT INTO "projected_crs" VALUES('EPSG','23323','NAD83(2011) / ICS83-Charleston (ftUS)',NULL,'EPSG','1053','EPSG','6318','EPSG','11253',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19304','projected_crs','EPSG','23323','EPSG','4723','EPSG','1266');
INSERT INTO "projected_crs" VALUES('EPSG','23324','NAD83(2011) / ICS83-Jerseyville (ftUS)',NULL,'EPSG','1053','EPSG','6318','EPSG','11254',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19305','projected_crs','EPSG','23324','EPSG','4724','EPSG','1266');
INSERT INTO "projected_crs" VALUES('EPSG','23325','NAD83(2011) / ICS83-Carlinville (ftUS)',NULL,'EPSG','1053','EPSG','6318','EPSG','11255',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19306','projected_crs','EPSG','23325','EPSG','4725','EPSG','1266');
INSERT INTO "projected_crs" VALUES('EPSG','23326','NAD83(2011) / ICS83-Taylorville (ftUS)',NULL,'EPSG','1053','EPSG','6318','EPSG','11256',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19307','projected_crs','EPSG','23326','EPSG','4726','EPSG','1266');
INSERT INTO "projected_crs" VALUES('EPSG','23327','NAD83(2011) / ICS83-Effingham (ftUS)',NULL,'EPSG','1053','EPSG','6318','EPSG','11257',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19308','projected_crs','EPSG','23327','EPSG','4727','EPSG','1266');
INSERT INTO "projected_crs" VALUES('EPSG','23328','NAD83(2011) / ICS83-Robinson (ftUS)',NULL,'EPSG','1053','EPSG','6318','EPSG','11258',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19309','projected_crs','EPSG','23328','EPSG','4728','EPSG','1266');
INSERT INTO "projected_crs" VALUES('EPSG','23329','NAD83(2011) / ICS83-Belleville (ftUS)',NULL,'EPSG','1053','EPSG','6318','EPSG','11259',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19310','projected_crs','EPSG','23329','EPSG','4729','EPSG','1266');
INSERT INTO "projected_crs" VALUES('EPSG','23330','NAD83(2011) / ICS83-Mount Vernon (ftUS)',NULL,'EPSG','1053','EPSG','6318','EPSG','11260',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19311','projected_crs','EPSG','23330','EPSG','4730','EPSG','1266');
INSERT INTO "projected_crs" VALUES('EPSG','23331','NAD83(2011) / ICS83-Olney (ftUS)',NULL,'EPSG','1053','EPSG','6318','EPSG','11261',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19312','projected_crs','EPSG','23331','EPSG','4731','EPSG','1266');
INSERT INTO "projected_crs" VALUES('EPSG','23332','NAD83(2011) / ICS83-Carbondale (ftUS)',NULL,'EPSG','1053','EPSG','6318','EPSG','11262',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19313','projected_crs','EPSG','23332','EPSG','4732','EPSG','1266');
INSERT INTO "projected_crs" VALUES('EPSG','23333','NAD83(2011) / ICS83-Metropolis (ftUS)',NULL,'EPSG','1053','EPSG','6318','EPSG','11263',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19314','projected_crs','EPSG','23333','EPSG','4733','EPSG','1266');
INSERT INTO "projected_crs" VALUES('EPSG','23433','Garoua / UTM zone 33N',NULL,'EPSG','4400','EPSG','4234','EPSG','16033',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6406','projected_crs','EPSG','23433','EPSG','1060','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','23700','HD72 / EOV',NULL,'EPSG','4498','EPSG','4237','EPSG','19931',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6407','projected_crs','EPSG','23700','EPSG','1119','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','23830','DGN95 / Indonesia TM-3 zone 46.2',NULL,'EPSG','4499','EPSG','4755','EPSG','17432',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6408','projected_crs','EPSG','23830','EPSG','3976','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','23831','DGN95 / Indonesia TM-3 zone 47.1',NULL,'EPSG','4499','EPSG','4755','EPSG','17433',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6409','projected_crs','EPSG','23831','EPSG','3510','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','23832','DGN95 / Indonesia TM-3 zone 47.2',NULL,'EPSG','4499','EPSG','4755','EPSG','17434',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6410','projected_crs','EPSG','23832','EPSG','3511','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','23833','DGN95 / Indonesia TM-3 zone 48.1',NULL,'EPSG','4499','EPSG','4755','EPSG','17435',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6411','projected_crs','EPSG','23833','EPSG','3512','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','23834','DGN95 / Indonesia TM-3 zone 48.2',NULL,'EPSG','4499','EPSG','4755','EPSG','17436',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6412','projected_crs','EPSG','23834','EPSG','3513','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','23835','DGN95 / Indonesia TM-3 zone 49.1',NULL,'EPSG','4499','EPSG','4755','EPSG','17437',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6413','projected_crs','EPSG','23835','EPSG','3514','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','23836','DGN95 / Indonesia TM-3 zone 49.2',NULL,'EPSG','4499','EPSG','4755','EPSG','17438',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6414','projected_crs','EPSG','23836','EPSG','3515','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','23837','DGN95 / Indonesia TM-3 zone 50.1',NULL,'EPSG','4499','EPSG','4755','EPSG','17439',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6415','projected_crs','EPSG','23837','EPSG','3516','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','23838','DGN95 / Indonesia TM-3 zone 50.2',NULL,'EPSG','4499','EPSG','4755','EPSG','17440',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6416','projected_crs','EPSG','23838','EPSG','3517','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','23839','DGN95 / Indonesia TM-3 zone 51.1',NULL,'EPSG','4499','EPSG','4755','EPSG','17441',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6417','projected_crs','EPSG','23839','EPSG','3518','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','23840','DGN95 / Indonesia TM-3 zone 51.2',NULL,'EPSG','4499','EPSG','4755','EPSG','17442',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6418','projected_crs','EPSG','23840','EPSG','3519','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','23841','DGN95 / Indonesia TM-3 zone 52.1',NULL,'EPSG','4499','EPSG','4755','EPSG','17443',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6419','projected_crs','EPSG','23841','EPSG','3520','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','23842','DGN95 / Indonesia TM-3 zone 52.2',NULL,'EPSG','4499','EPSG','4755','EPSG','17444',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6420','projected_crs','EPSG','23842','EPSG','3521','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','23843','DGN95 / Indonesia TM-3 zone 53.1',NULL,'EPSG','4499','EPSG','4755','EPSG','17445',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6421','projected_crs','EPSG','23843','EPSG','3522','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','23844','DGN95 / Indonesia TM-3 zone 53.2',NULL,'EPSG','4499','EPSG','4755','EPSG','17446',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6422','projected_crs','EPSG','23844','EPSG','3523','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','23845','DGN95 / Indonesia TM-3 zone 54.1',NULL,'EPSG','4499','EPSG','4755','EPSG','17447',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6423','projected_crs','EPSG','23845','EPSG','3975','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','23846','ID74 / UTM zone 46N',NULL,'EPSG','4400','EPSG','4238','EPSG','16046',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6424','projected_crs','EPSG','23846','EPSG','3976','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','23847','ID74 / UTM zone 47N',NULL,'EPSG','4400','EPSG','4238','EPSG','16047',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6425','projected_crs','EPSG','23847','EPSG','3978','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','23848','ID74 / UTM zone 48N',NULL,'EPSG','4400','EPSG','4238','EPSG','16048',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6426','projected_crs','EPSG','23848','EPSG','3979','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','23849','ID74 / UTM zone 49N',NULL,'EPSG','4400','EPSG','4238','EPSG','16049',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6427','projected_crs','EPSG','23849','EPSG','3980','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','23850','ID74 / UTM zone 50N',NULL,'EPSG','4400','EPSG','4238','EPSG','16050',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6428','projected_crs','EPSG','23850','EPSG','3981','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','23851','ID74 / UTM zone 51N',NULL,'EPSG','4400','EPSG','4238','EPSG','16051',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6429','projected_crs','EPSG','23851','EPSG','3983','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','23852','ID74 / UTM zone 52N',NULL,'EPSG','4400','EPSG','4238','EPSG','16052',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6430','projected_crs','EPSG','23852','EPSG','3984','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','23853','ID74 / UTM zone 53N',NULL,'EPSG','4400','EPSG','4238','EPSG','16053',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6431','projected_crs','EPSG','23853','EPSG','1661','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','23866','DGN95 / UTM zone 46N',NULL,'EPSG','4400','EPSG','4755','EPSG','16046',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6432','projected_crs','EPSG','23866','EPSG','1647','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','23867','DGN95 / UTM zone 47N',NULL,'EPSG','4400','EPSG','4755','EPSG','16047',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6433','projected_crs','EPSG','23867','EPSG','1649','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','23868','DGN95 / UTM zone 48N',NULL,'EPSG','4400','EPSG','4755','EPSG','16048',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6434','projected_crs','EPSG','23868','EPSG','1651','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','23869','DGN95 / UTM zone 49N',NULL,'EPSG','4400','EPSG','4755','EPSG','16049',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6435','projected_crs','EPSG','23869','EPSG','1653','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','23870','DGN95 / UTM zone 50N',NULL,'EPSG','4400','EPSG','4755','EPSG','16050',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6436','projected_crs','EPSG','23870','EPSG','1655','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','23871','DGN95 / UTM zone 51N',NULL,'EPSG','4400','EPSG','4755','EPSG','16051',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6437','projected_crs','EPSG','23871','EPSG','1657','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','23872','DGN95 / UTM zone 52N',NULL,'EPSG','4400','EPSG','4755','EPSG','16052',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6438','projected_crs','EPSG','23872','EPSG','1659','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','23877','DGN95 / UTM zone 47S',NULL,'EPSG','4400','EPSG','4755','EPSG','16147',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6439','projected_crs','EPSG','23877','EPSG','1650','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','23878','DGN95 / UTM zone 48S',NULL,'EPSG','4400','EPSG','4755','EPSG','16148',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6440','projected_crs','EPSG','23878','EPSG','1652','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','23879','DGN95 / UTM zone 49S',NULL,'EPSG','4400','EPSG','4755','EPSG','16149',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6441','projected_crs','EPSG','23879','EPSG','1654','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','23880','DGN95 / UTM zone 50S',NULL,'EPSG','4400','EPSG','4755','EPSG','16150',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6442','projected_crs','EPSG','23880','EPSG','1656','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','23881','DGN95 / UTM zone 51S',NULL,'EPSG','4400','EPSG','4755','EPSG','16151',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6443','projected_crs','EPSG','23881','EPSG','1658','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','23882','DGN95 / UTM zone 52S',NULL,'EPSG','4400','EPSG','4755','EPSG','16152',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6444','projected_crs','EPSG','23882','EPSG','1660','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','23883','DGN95 / UTM zone 53S',NULL,'EPSG','4400','EPSG','4755','EPSG','16153',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6445','projected_crs','EPSG','23883','EPSG','1662','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','23884','DGN95 / UTM zone 54S',NULL,'EPSG','4400','EPSG','4755','EPSG','16154',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6446','projected_crs','EPSG','23884','EPSG','1663','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','23886','ID74 / UTM zone 46S',NULL,'EPSG','4400','EPSG','4238','EPSG','16146',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6447','projected_crs','EPSG','23886','EPSG','1648','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','23887','ID74 / UTM zone 47S',NULL,'EPSG','4400','EPSG','4238','EPSG','16147',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6448','projected_crs','EPSG','23887','EPSG','3985','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','23888','ID74 / UTM zone 48S',NULL,'EPSG','4400','EPSG','4238','EPSG','16148',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6449','projected_crs','EPSG','23888','EPSG','3986','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','23889','ID74 / UTM zone 49S',NULL,'EPSG','4400','EPSG','4238','EPSG','16149',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6450','projected_crs','EPSG','23889','EPSG','3987','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','23890','ID74 / UTM zone 50S',NULL,'EPSG','4400','EPSG','4238','EPSG','16150',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6451','projected_crs','EPSG','23890','EPSG','3988','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','23891','ID74 / UTM zone 51S',NULL,'EPSG','4400','EPSG','4238','EPSG','16151',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6452','projected_crs','EPSG','23891','EPSG','3989','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','23892','ID74 / UTM zone 52S',NULL,'EPSG','4400','EPSG','4238','EPSG','16152',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6453','projected_crs','EPSG','23892','EPSG','3990','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','23893','ID74 / UTM zone 53S',NULL,'EPSG','4400','EPSG','4238','EPSG','16153',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6454','projected_crs','EPSG','23893','EPSG','3991','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','23894','ID74 / UTM zone 54S',NULL,'EPSG','4400','EPSG','4238','EPSG','16154',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6455','projected_crs','EPSG','23894','EPSG','3975','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','23946','Indian 1954 / UTM zone 46N',NULL,'EPSG','4400','EPSG','4239','EPSG','16046',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6456','projected_crs','EPSG','23946','EPSG','1664','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','23947','Indian 1954 / UTM zone 47N',NULL,'EPSG','4400','EPSG','4239','EPSG','16047',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6457','projected_crs','EPSG','23947','EPSG','1665','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','23948','Indian 1954 / UTM zone 48N',NULL,'EPSG','4400','EPSG','4239','EPSG','16048',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6458','projected_crs','EPSG','23948','EPSG','3735','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','24047','Indian 1975 / UTM zone 47N',NULL,'EPSG','4400','EPSG','4240','EPSG','16047',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6459','projected_crs','EPSG','24047','EPSG','1667','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','24048','Indian 1975 / UTM zone 48N',NULL,'EPSG','4400','EPSG','4240','EPSG','16048',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6460','projected_crs','EPSG','24048','EPSG','1666','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','24100','Jamaica 1875 / Jamaica (Old Grid)',NULL,'EPSG','4403','EPSG','4241','EPSG','19909',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6461','projected_crs','EPSG','24100','EPSG','3342','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','24200','JAD69 / Jamaica National Grid',NULL,'EPSG','4400','EPSG','4242','EPSG','19910',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6462','projected_crs','EPSG','24200','EPSG','3342','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','24305','Kalianpur 1937 / UTM zone 45N',NULL,'EPSG','4400','EPSG','4144','EPSG','16045',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6463','projected_crs','EPSG','24305','EPSG','1674','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','24306','Kalianpur 1937 / UTM zone 46N',NULL,'EPSG','4400','EPSG','4144','EPSG','16046',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6464','projected_crs','EPSG','24306','EPSG','1675','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','24311','Kalianpur 1962 / UTM zone 41N',NULL,'EPSG','4400','EPSG','4145','EPSG','16041',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6465','projected_crs','EPSG','24311','EPSG','1687','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','24312','Kalianpur 1962 / UTM zone 42N',NULL,'EPSG','4400','EPSG','4145','EPSG','16042',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6466','projected_crs','EPSG','24312','EPSG','1688','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','24313','Kalianpur 1962 / UTM zone 43N',NULL,'EPSG','4400','EPSG','4145','EPSG','16043',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6467','projected_crs','EPSG','24313','EPSG','1689','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','24342','Kalianpur 1975 / UTM zone 42N',NULL,'EPSG','4400','EPSG','4146','EPSG','16042',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6468','projected_crs','EPSG','24342','EPSG','1679','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','24343','Kalianpur 1975 / UTM zone 43N',NULL,'EPSG','4400','EPSG','4146','EPSG','16043',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6469','projected_crs','EPSG','24343','EPSG','1680','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','24344','Kalianpur 1975 / UTM zone 44N',NULL,'EPSG','4400','EPSG','4146','EPSG','16044',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6470','projected_crs','EPSG','24344','EPSG','1681','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','24345','Kalianpur 1975 / UTM zone 45N',NULL,'EPSG','4400','EPSG','4146','EPSG','16045',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6471','projected_crs','EPSG','24345','EPSG','1682','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','24346','Kalianpur 1975 / UTM zone 46N',NULL,'EPSG','4400','EPSG','4146','EPSG','16046',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6472','projected_crs','EPSG','24346','EPSG','1683','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','24347','Kalianpur 1975 / UTM zone 47N',NULL,'EPSG','4400','EPSG','4146','EPSG','16047',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6473','projected_crs','EPSG','24347','EPSG','1684','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','24370','Kalianpur 1880 / India zone 0',NULL,'EPSG','4408','EPSG','4243','EPSG','18110',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6474','projected_crs','EPSG','24370','EPSG','1668','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','24371','Kalianpur 1880 / India zone I',NULL,'EPSG','4408','EPSG','4243','EPSG','18111',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6475','projected_crs','EPSG','24371','EPSG','1669','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','24372','Kalianpur 1880 / India zone IIa',NULL,'EPSG','4408','EPSG','4243','EPSG','18112',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6476','projected_crs','EPSG','24372','EPSG','1670','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','24373','Kalianpur 1880 / India zone IIIa',NULL,'EPSG','4408','EPSG','4243','EPSG','18114',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6477','projected_crs','EPSG','24373','EPSG','1672','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','24374','Kalianpur 1880 / India zone IVa',NULL,'EPSG','4408','EPSG','4243','EPSG','18116',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6478','projected_crs','EPSG','24374','EPSG','1673','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','24375','Kalianpur 1937 / India zone IIb',NULL,'EPSG','4400','EPSG','4144','EPSG','18238',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6479','projected_crs','EPSG','24375','EPSG','3217','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','24376','Kalianpur 1962 / India zone I',NULL,'EPSG','4400','EPSG','4145','EPSG','18236',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6480','projected_crs','EPSG','24376','EPSG','1685','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','24377','Kalianpur 1962 / India zone IIa',NULL,'EPSG','4400','EPSG','4145','EPSG','18237',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6481','projected_crs','EPSG','24377','EPSG','1686','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','24378','Kalianpur 1975 / India zone I',NULL,'EPSG','4400','EPSG','4146','EPSG','18231',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6482','projected_crs','EPSG','24378','EPSG','1676','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','24379','Kalianpur 1975 / India zone IIa',NULL,'EPSG','4400','EPSG','4146','EPSG','18232',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6483','projected_crs','EPSG','24379','EPSG','1677','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','24380','Kalianpur 1975 / India zone IIb',NULL,'EPSG','4400','EPSG','4146','EPSG','18235',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6484','projected_crs','EPSG','24380','EPSG','1678','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','24381','Kalianpur 1975 / India zone IIIa',NULL,'EPSG','4400','EPSG','4146','EPSG','18233',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6485','projected_crs','EPSG','24381','EPSG','1672','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','24382','Kalianpur 1880 / India zone IIb',NULL,'EPSG','4408','EPSG','4243','EPSG','18113',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6486','projected_crs','EPSG','24382','EPSG','1671','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','24383','Kalianpur 1975 / India zone IVa',NULL,'EPSG','4400','EPSG','4146','EPSG','18234',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6487','projected_crs','EPSG','24383','EPSG','1673','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','24500','Kertau 1968 / Singapore Grid',NULL,'EPSG','4400','EPSG','4245','EPSG','19920',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6488','projected_crs','EPSG','24500','EPSG','1210','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','24547','Kertau 1968 / UTM zone 47N',NULL,'EPSG','4400','EPSG','4245','EPSG','16047',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6489','projected_crs','EPSG','24547','EPSG','1691','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','24548','Kertau 1968 / UTM zone 48N',NULL,'EPSG','4400','EPSG','4245','EPSG','16048',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6490','projected_crs','EPSG','24548','EPSG','1692','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','24571','Kertau / R.S.O. Malaya (ch)',NULL,'EPSG','4401','EPSG','4245','EPSG','19935',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6491','projected_crs','EPSG','24571','EPSG','1690','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','24600','KOC Lambert',NULL,'EPSG','4400','EPSG','4246','EPSG','19906',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6492','projected_crs','EPSG','24600','EPSG','3267','EPSG','1216');
INSERT INTO "projected_crs" VALUES('EPSG','24718','La Canoa / UTM zone 18N',NULL,'EPSG','4400','EPSG','4247','EPSG','16018',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6493','projected_crs','EPSG','24718','EPSG','1693','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','24719','La Canoa / UTM zone 19N',NULL,'EPSG','4400','EPSG','4247','EPSG','16019',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6494','projected_crs','EPSG','24719','EPSG','1694','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','24720','La Canoa / UTM zone 20N',NULL,'EPSG','4400','EPSG','4247','EPSG','16020',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6495','projected_crs','EPSG','24720','EPSG','1695','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','24817','PSAD56 / UTM zone 17N',NULL,'EPSG','4400','EPSG','4248','EPSG','16017',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6496','projected_crs','EPSG','24817','EPSG','3112','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','24818','PSAD56 / UTM zone 18N',NULL,'EPSG','4400','EPSG','4248','EPSG','16018',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6497','projected_crs','EPSG','24818','EPSG','1756','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','24819','PSAD56 / UTM zone 19N',NULL,'EPSG','4400','EPSG','4248','EPSG','16019',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6498','projected_crs','EPSG','24819','EPSG','1758','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','24820','PSAD56 / UTM zone 20N',NULL,'EPSG','4400','EPSG','4248','EPSG','16020',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6499','projected_crs','EPSG','24820','EPSG','1760','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','24821','PSAD56 / UTM zone 21N',NULL,'EPSG','4400','EPSG','4248','EPSG','16021',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6500','projected_crs','EPSG','24821','EPSG','1762','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','24877','PSAD56 / UTM zone 17S',NULL,'EPSG','4400','EPSG','4248','EPSG','16117',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6501','projected_crs','EPSG','24877','EPSG','1755','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','24878','PSAD56 / UTM zone 18S',NULL,'EPSG','4400','EPSG','4248','EPSG','16118',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6502','projected_crs','EPSG','24878','EPSG','1757','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','24879','PSAD56 / UTM zone 19S',NULL,'EPSG','4400','EPSG','4248','EPSG','16119',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6503','projected_crs','EPSG','24879','EPSG','1759','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','24880','PSAD56 / UTM zone 20S',NULL,'EPSG','4400','EPSG','4248','EPSG','16120',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6504','projected_crs','EPSG','24880','EPSG','1761','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','24881','PSAD56 / UTM zone 21S',NULL,'EPSG','4400','EPSG','4248','EPSG','16121',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6505','projected_crs','EPSG','24881','EPSG','3733','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','24882','PSAD56 / UTM zone 22S',NULL,'EPSG','4400','EPSG','4248','EPSG','16122',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6506','projected_crs','EPSG','24882','EPSG','1754','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','24891','PSAD56 / Peru west zone',NULL,'EPSG','4499','EPSG','4248','EPSG','18161',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6507','projected_crs','EPSG','24891','EPSG','1753','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','24892','PSAD56 / Peru central zone',NULL,'EPSG','4499','EPSG','4248','EPSG','18162',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6508','projected_crs','EPSG','24892','EPSG','1752','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','24893','PSAD56 / Peru east zone',NULL,'EPSG','4499','EPSG','4248','EPSG','18163',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6509','projected_crs','EPSG','24893','EPSG','1751','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','25000','Leigon / Ghana Metre Grid',NULL,'EPSG','4400','EPSG','4250','EPSG','19904',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6510','projected_crs','EPSG','25000','EPSG','1104','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','25231','Lome / UTM zone 31N',NULL,'EPSG','4400','EPSG','4252','EPSG','16031',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6511','projected_crs','EPSG','25231','EPSG','1232','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','25391','Luzon 1911 / Philippines zone I',NULL,'EPSG','4499','EPSG','4253','EPSG','18171',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6512','projected_crs','EPSG','25391','EPSG','3958','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','25392','Luzon 1911 / Philippines zone II',NULL,'EPSG','4499','EPSG','4253','EPSG','18172',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6513','projected_crs','EPSG','25392','EPSG','3964','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','25393','Luzon 1911 / Philippines zone III',NULL,'EPSG','4499','EPSG','4253','EPSG','18173',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6514','projected_crs','EPSG','25393','EPSG','3965','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','25394','Luzon 1911 / Philippines zone IV',NULL,'EPSG','4499','EPSG','4253','EPSG','18174',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6515','projected_crs','EPSG','25394','EPSG','3966','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','25395','Luzon 1911 / Philippines zone V',NULL,'EPSG','4499','EPSG','4253','EPSG','18175',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6516','projected_crs','EPSG','25395','EPSG','3967','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','25700','Makassar (Jakarta) / NEIEZ',NULL,'EPSG','4499','EPSG','4804','EPSG','19905',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6517','projected_crs','EPSG','25700','EPSG','1316','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','25828','ETRS89 / UTM zone 28N',NULL,'EPSG','4400','EPSG','4258','EPSG','16028',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6518','projected_crs','EPSG','25828','EPSG','2122','EPSG','1290');
INSERT INTO "projected_crs" VALUES('EPSG','25829','ETRS89 / UTM zone 29N',NULL,'EPSG','4400','EPSG','4258','EPSG','16029',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6519','projected_crs','EPSG','25829','EPSG','2123','EPSG','1142');
INSERT INTO "usage" VALUES('EPSG','21368','projected_crs','EPSG','25829','EPSG','4765','EPSG','1227');
INSERT INTO "projected_crs" VALUES('EPSG','25830','ETRS89 / UTM zone 30N',NULL,'EPSG','4400','EPSG','4258','EPSG','16030',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6520','projected_crs','EPSG','25830','EPSG','2124','EPSG','1142');
INSERT INTO "usage" VALUES('EPSG','21361','projected_crs','EPSG','25830','EPSG','4758','EPSG','1227');
INSERT INTO "projected_crs" VALUES('EPSG','25831','ETRS89 / UTM zone 31N',NULL,'EPSG','4400','EPSG','4258','EPSG','16031',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6521','projected_crs','EPSG','25831','EPSG','2125','EPSG','1142');
INSERT INTO "usage" VALUES('EPSG','21362','projected_crs','EPSG','25831','EPSG','4759','EPSG','1227');
INSERT INTO "projected_crs" VALUES('EPSG','25832','ETRS89 / UTM zone 32N',NULL,'EPSG','4400','EPSG','4258','EPSG','16032',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6522','projected_crs','EPSG','25832','EPSG','2126','EPSG','1142');
INSERT INTO "usage" VALUES('EPSG','21363','projected_crs','EPSG','25832','EPSG','4760','EPSG','1227');
INSERT INTO "projected_crs" VALUES('EPSG','25833','ETRS89 / UTM zone 33N',NULL,'EPSG','4400','EPSG','4258','EPSG','16033',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6523','projected_crs','EPSG','25833','EPSG','2127','EPSG','1142');
INSERT INTO "usage" VALUES('EPSG','21364','projected_crs','EPSG','25833','EPSG','4761','EPSG','1227');
INSERT INTO "projected_crs" VALUES('EPSG','25834','ETRS89 / UTM zone 34N',NULL,'EPSG','4400','EPSG','4258','EPSG','16034',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6524','projected_crs','EPSG','25834','EPSG','2128','EPSG','1142');
INSERT INTO "usage" VALUES('EPSG','21365','projected_crs','EPSG','25834','EPSG','4762','EPSG','1227');
INSERT INTO "projected_crs" VALUES('EPSG','25835','ETRS89 / UTM zone 35N',NULL,'EPSG','4400','EPSG','4258','EPSG','16035',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6525','projected_crs','EPSG','25835','EPSG','2129','EPSG','1142');
INSERT INTO "usage" VALUES('EPSG','21366','projected_crs','EPSG','25835','EPSG','4763','EPSG','1227');
INSERT INTO "projected_crs" VALUES('EPSG','25836','ETRS89 / UTM zone 36N',NULL,'EPSG','4400','EPSG','4258','EPSG','16036',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6526','projected_crs','EPSG','25836','EPSG','2130','EPSG','1142');
INSERT INTO "usage" VALUES('EPSG','21367','projected_crs','EPSG','25836','EPSG','4764','EPSG','1227');
INSERT INTO "projected_crs" VALUES('EPSG','25837','ETRS89 / UTM zone 37N',NULL,'EPSG','4400','EPSG','4258','EPSG','16037',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6527','projected_crs','EPSG','25837','EPSG','2131','EPSG','1290');
INSERT INTO "projected_crs" VALUES('EPSG','25838','ETRS89 / UTM zone 38N',NULL,'EPSG','4400','EPSG','4258','EPSG','16038',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6528','projected_crs','EPSG','25838','EPSG','2132','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','25884','ETRS89 / TM Baltic93',NULL,'EPSG','4530','EPSG','4258','EPSG','19939',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6529','projected_crs','EPSG','25884','EPSG','1646','EPSG','1210');
INSERT INTO "projected_crs" VALUES('EPSG','25932','Malongo 1987 / UTM zone 32S',NULL,'EPSG','4400','EPSG','4259','EPSG','16132',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6530','projected_crs','EPSG','25932','EPSG','3180','EPSG','1136');
INSERT INTO "projected_crs" VALUES('EPSG','26191','Merchich / Nord Maroc',NULL,'EPSG','4499','EPSG','4261','EPSG','18131',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6531','projected_crs','EPSG','26191','EPSG','1703','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26192','Merchich / Sud Maroc',NULL,'EPSG','4499','EPSG','4261','EPSG','18132',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6532','projected_crs','EPSG','26192','EPSG','2787','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26193','Merchich / Sahara',NULL,'EPSG','4499','EPSG','4261','EPSG','18133',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6533','projected_crs','EPSG','26193','EPSG','1705','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26194','Merchich / Sahara Nord',NULL,'EPSG','4499','EPSG','4261','EPSG','18134',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6534','projected_crs','EPSG','26194','EPSG','2788','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26195','Merchich / Sahara Sud',NULL,'EPSG','4499','EPSG','4261','EPSG','18135',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6535','projected_crs','EPSG','26195','EPSG','2789','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26237','Massawa / UTM zone 37N',NULL,'EPSG','4400','EPSG','4262','EPSG','16037',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6536','projected_crs','EPSG','26237','EPSG','1089','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26331','Minna / UTM zone 31N',NULL,'EPSG','4400','EPSG','4263','EPSG','16031',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6537','projected_crs','EPSG','26331','EPSG','1716','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26332','Minna / UTM zone 32N',NULL,'EPSG','4400','EPSG','4263','EPSG','16032',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6538','projected_crs','EPSG','26332','EPSG','3812','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26391','Minna / Nigeria West Belt',NULL,'EPSG','4400','EPSG','4263','EPSG','18151',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6539','projected_crs','EPSG','26391','EPSG','1715','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26392','Minna / Nigeria Mid Belt',NULL,'EPSG','4400','EPSG','4263','EPSG','18152',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6540','projected_crs','EPSG','26392','EPSG','1714','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26393','Minna / Nigeria East Belt',NULL,'EPSG','4400','EPSG','4263','EPSG','18153',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6541','projected_crs','EPSG','26393','EPSG','1713','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26432','Mhast / UTM zone 32S',NULL,'EPSG','4400','EPSG','4264','EPSG','16132',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6542','projected_crs','EPSG','26432','EPSG','1318','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26591','Monte Mario (Rome) / Italy zone 1',NULL,'EPSG','4499','EPSG','4806','EPSG','18121',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6543','projected_crs','EPSG','26591','EPSG','1718','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26592','Monte Mario (Rome) / Italy zone 2',NULL,'EPSG','4499','EPSG','4806','EPSG','18122',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6544','projected_crs','EPSG','26592','EPSG','1719','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26632','M''poraloko / UTM zone 32N',NULL,'EPSG','4400','EPSG','4266','EPSG','16032',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6545','projected_crs','EPSG','26632','EPSG','1696','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26692','M''poraloko / UTM zone 32S',NULL,'EPSG','4400','EPSG','4266','EPSG','16132',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6546','projected_crs','EPSG','26692','EPSG','1697','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26701','NAD27 / UTM zone 1N',NULL,'EPSG','4400','EPSG','4267','EPSG','16001',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6547','projected_crs','EPSG','26701','EPSG','3374','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26702','NAD27 / UTM zone 2N',NULL,'EPSG','4400','EPSG','4267','EPSG','16002',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6548','projected_crs','EPSG','26702','EPSG','3375','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26703','NAD27 / UTM zone 3N',NULL,'EPSG','4400','EPSG','4267','EPSG','16003',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6549','projected_crs','EPSG','26703','EPSG','2133','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26704','NAD27 / UTM zone 4N',NULL,'EPSG','4400','EPSG','4267','EPSG','16004',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6550','projected_crs','EPSG','26704','EPSG','2134','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26705','NAD27 / UTM zone 5N',NULL,'EPSG','4400','EPSG','4267','EPSG','16005',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6551','projected_crs','EPSG','26705','EPSG','2135','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26706','NAD27 / UTM zone 6N',NULL,'EPSG','4400','EPSG','4267','EPSG','16006',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6552','projected_crs','EPSG','26706','EPSG','2136','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26707','NAD27 / UTM zone 7N',NULL,'EPSG','4400','EPSG','4267','EPSG','16007',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6553','projected_crs','EPSG','26707','EPSG','2137','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26708','NAD27 / UTM zone 8N',NULL,'EPSG','4400','EPSG','4267','EPSG','16008',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6554','projected_crs','EPSG','26708','EPSG','2138','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26709','NAD27 / UTM zone 9N',NULL,'EPSG','4400','EPSG','4267','EPSG','16009',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6555','projected_crs','EPSG','26709','EPSG','2139','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26710','NAD27 / UTM zone 10N',NULL,'EPSG','4400','EPSG','4267','EPSG','16010',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6556','projected_crs','EPSG','26710','EPSG','2140','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26711','NAD27 / UTM zone 11N',NULL,'EPSG','4400','EPSG','4267','EPSG','16011',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6557','projected_crs','EPSG','26711','EPSG','2141','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26712','NAD27 / UTM zone 12N',NULL,'EPSG','4400','EPSG','4267','EPSG','16012',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6558','projected_crs','EPSG','26712','EPSG','2142','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26713','NAD27 / UTM zone 13N',NULL,'EPSG','4400','EPSG','4267','EPSG','16013',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6559','projected_crs','EPSG','26713','EPSG','2143','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26714','NAD27 / UTM zone 14N',NULL,'EPSG','4400','EPSG','4267','EPSG','16014',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6560','projected_crs','EPSG','26714','EPSG','2144','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26715','NAD27 / UTM zone 15N',NULL,'EPSG','4400','EPSG','4267','EPSG','16015',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6561','projected_crs','EPSG','26715','EPSG','2145','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26716','NAD27 / UTM zone 16N',NULL,'EPSG','4400','EPSG','4267','EPSG','16016',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6562','projected_crs','EPSG','26716','EPSG','2146','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26717','NAD27 / UTM zone 17N',NULL,'EPSG','4400','EPSG','4267','EPSG','16017',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6563','projected_crs','EPSG','26717','EPSG','2147','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26718','NAD27 / UTM zone 18N',NULL,'EPSG','4400','EPSG','4267','EPSG','16018',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6564','projected_crs','EPSG','26718','EPSG','2148','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26719','NAD27 / UTM zone 19N',NULL,'EPSG','4400','EPSG','4267','EPSG','16019',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6565','projected_crs','EPSG','26719','EPSG','2149','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26720','NAD27 / UTM zone 20N',NULL,'EPSG','4400','EPSG','4267','EPSG','16020',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6566','projected_crs','EPSG','26720','EPSG','2150','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26721','NAD27 / UTM zone 21N',NULL,'EPSG','4400','EPSG','4267','EPSG','16021',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6567','projected_crs','EPSG','26721','EPSG','3891','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26722','NAD27 / UTM zone 22N',NULL,'EPSG','4400','EPSG','4267','EPSG','16022',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6568','projected_crs','EPSG','26722','EPSG','2152','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26729','NAD27 / Alabama East',NULL,'EPSG','4497','EPSG','4267','EPSG','10101',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6569','projected_crs','EPSG','26729','EPSG','2154','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26730','NAD27 / Alabama West',NULL,'EPSG','4497','EPSG','4267','EPSG','10102',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6570','projected_crs','EPSG','26730','EPSG','2155','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26731','NAD27 / Alaska zone 1',NULL,'EPSG','4497','EPSG','4267','EPSG','15001',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6571','projected_crs','EPSG','26731','EPSG','2156','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26732','NAD27 / Alaska zone 2',NULL,'EPSG','4497','EPSG','4267','EPSG','15002',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6572','projected_crs','EPSG','26732','EPSG','2158','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26733','NAD27 / Alaska zone 3',NULL,'EPSG','4497','EPSG','4267','EPSG','15003',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6573','projected_crs','EPSG','26733','EPSG','2159','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26734','NAD27 / Alaska zone 4',NULL,'EPSG','4497','EPSG','4267','EPSG','15004',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6574','projected_crs','EPSG','26734','EPSG','2160','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26735','NAD27 / Alaska zone 5',NULL,'EPSG','4497','EPSG','4267','EPSG','15005',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6575','projected_crs','EPSG','26735','EPSG','2161','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26736','NAD27 / Alaska zone 6',NULL,'EPSG','4497','EPSG','4267','EPSG','15006',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6576','projected_crs','EPSG','26736','EPSG','2162','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26737','NAD27 / Alaska zone 7',NULL,'EPSG','4497','EPSG','4267','EPSG','15007',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6577','projected_crs','EPSG','26737','EPSG','2163','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26738','NAD27 / Alaska zone 8',NULL,'EPSG','4497','EPSG','4267','EPSG','15008',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6578','projected_crs','EPSG','26738','EPSG','2164','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26739','NAD27 / Alaska zone 9',NULL,'EPSG','4497','EPSG','4267','EPSG','15009',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6579','projected_crs','EPSG','26739','EPSG','2165','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26740','NAD27 / Alaska zone 10',NULL,'EPSG','4497','EPSG','4267','EPSG','15010',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6580','projected_crs','EPSG','26740','EPSG','2157','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26741','NAD27 / California zone I',NULL,'EPSG','4497','EPSG','4267','EPSG','10401',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6581','projected_crs','EPSG','26741','EPSG','2175','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26742','NAD27 / California zone II',NULL,'EPSG','4497','EPSG','4267','EPSG','10402',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6582','projected_crs','EPSG','26742','EPSG','2176','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26743','NAD27 / California zone III',NULL,'EPSG','4497','EPSG','4267','EPSG','10403',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6583','projected_crs','EPSG','26743','EPSG','2177','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26744','NAD27 / California zone IV',NULL,'EPSG','4497','EPSG','4267','EPSG','10404',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6584','projected_crs','EPSG','26744','EPSG','2178','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26745','NAD27 / California zone V',NULL,'EPSG','4497','EPSG','4267','EPSG','10405',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6585','projected_crs','EPSG','26745','EPSG','2179','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26746','NAD27 / California zone VI',NULL,'EPSG','4497','EPSG','4267','EPSG','10406',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6586','projected_crs','EPSG','26746','EPSG','2180','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26747','NAD27 / California zone VII',NULL,'EPSG','4497','EPSG','4267','EPSG','10407',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6587','projected_crs','EPSG','26747','EPSG','2181','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26748','NAD27 / Arizona East',NULL,'EPSG','4497','EPSG','4267','EPSG','10201',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6588','projected_crs','EPSG','26748','EPSG','2167','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26749','NAD27 / Arizona Central',NULL,'EPSG','4497','EPSG','4267','EPSG','10202',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6589','projected_crs','EPSG','26749','EPSG','2166','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26750','NAD27 / Arizona West',NULL,'EPSG','4497','EPSG','4267','EPSG','10203',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6590','projected_crs','EPSG','26750','EPSG','2168','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26751','NAD27 / Arkansas North',NULL,'EPSG','4497','EPSG','4267','EPSG','10301',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6591','projected_crs','EPSG','26751','EPSG','2169','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26752','NAD27 / Arkansas South',NULL,'EPSG','4497','EPSG','4267','EPSG','10302',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6592','projected_crs','EPSG','26752','EPSG','2170','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26753','NAD27 / Colorado North',NULL,'EPSG','4497','EPSG','4267','EPSG','10501',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6593','projected_crs','EPSG','26753','EPSG','2184','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26754','NAD27 / Colorado Central',NULL,'EPSG','4497','EPSG','4267','EPSG','10502',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6594','projected_crs','EPSG','26754','EPSG','2183','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26755','NAD27 / Colorado South',NULL,'EPSG','4497','EPSG','4267','EPSG','10503',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6595','projected_crs','EPSG','26755','EPSG','2185','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26756','NAD27 / Connecticut',NULL,'EPSG','4497','EPSG','4267','EPSG','10600',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6596','projected_crs','EPSG','26756','EPSG','1377','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26757','NAD27 / Delaware',NULL,'EPSG','4497','EPSG','4267','EPSG','10700',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6597','projected_crs','EPSG','26757','EPSG','1378','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26758','NAD27 / Florida East',NULL,'EPSG','4497','EPSG','4267','EPSG','10901',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6598','projected_crs','EPSG','26758','EPSG','2186','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26759','NAD27 / Florida West',NULL,'EPSG','4497','EPSG','4267','EPSG','10902',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6599','projected_crs','EPSG','26759','EPSG','2188','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26760','NAD27 / Florida North',NULL,'EPSG','4497','EPSG','4267','EPSG','10903',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6600','projected_crs','EPSG','26760','EPSG','2187','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26766','NAD27 / Georgia East',NULL,'EPSG','4497','EPSG','4267','EPSG','11001',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6601','projected_crs','EPSG','26766','EPSG','2189','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26767','NAD27 / Georgia West',NULL,'EPSG','4497','EPSG','4267','EPSG','11002',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6602','projected_crs','EPSG','26767','EPSG','2190','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26768','NAD27 / Idaho East',NULL,'EPSG','4497','EPSG','4267','EPSG','11101',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6603','projected_crs','EPSG','26768','EPSG','2192','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26769','NAD27 / Idaho Central',NULL,'EPSG','4497','EPSG','4267','EPSG','11102',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6604','projected_crs','EPSG','26769','EPSG','2191','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26770','NAD27 / Idaho West',NULL,'EPSG','4497','EPSG','4267','EPSG','11103',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6605','projected_crs','EPSG','26770','EPSG','2193','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26771','NAD27 / Illinois East',NULL,'EPSG','4497','EPSG','4267','EPSG','11201',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6606','projected_crs','EPSG','26771','EPSG','2194','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26772','NAD27 / Illinois West',NULL,'EPSG','4497','EPSG','4267','EPSG','11202',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6607','projected_crs','EPSG','26772','EPSG','2195','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26773','NAD27 / Indiana East',NULL,'EPSG','4497','EPSG','4267','EPSG','11301',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6608','projected_crs','EPSG','26773','EPSG','2196','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26774','NAD27 / Indiana West',NULL,'EPSG','4497','EPSG','4267','EPSG','11302',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6609','projected_crs','EPSG','26774','EPSG','2197','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26775','NAD27 / Iowa North',NULL,'EPSG','4497','EPSG','4267','EPSG','11401',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6610','projected_crs','EPSG','26775','EPSG','2198','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26776','NAD27 / Iowa South',NULL,'EPSG','4497','EPSG','4267','EPSG','11402',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6611','projected_crs','EPSG','26776','EPSG','2199','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26777','NAD27 / Kansas North',NULL,'EPSG','4497','EPSG','4267','EPSG','11501',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6612','projected_crs','EPSG','26777','EPSG','2200','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26778','NAD27 / Kansas South',NULL,'EPSG','4497','EPSG','4267','EPSG','11502',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6613','projected_crs','EPSG','26778','EPSG','2201','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26779','NAD27 / Kentucky North',NULL,'EPSG','4497','EPSG','4267','EPSG','11601',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6614','projected_crs','EPSG','26779','EPSG','2202','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26780','NAD27 / Kentucky South',NULL,'EPSG','4497','EPSG','4267','EPSG','11602',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6615','projected_crs','EPSG','26780','EPSG','2203','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26781','NAD27 / Louisiana North',NULL,'EPSG','4497','EPSG','4267','EPSG','11701',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6616','projected_crs','EPSG','26781','EPSG','2204','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26782','NAD27 / Louisiana South',NULL,'EPSG','4497','EPSG','4267','EPSG','11702',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6617','projected_crs','EPSG','26782','EPSG','2205','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26783','NAD27 / Maine East',NULL,'EPSG','4497','EPSG','4267','EPSG','11801',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6618','projected_crs','EPSG','26783','EPSG','2206','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26784','NAD27 / Maine West',NULL,'EPSG','4497','EPSG','4267','EPSG','11802',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6619','projected_crs','EPSG','26784','EPSG','2207','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26785','NAD27 / Maryland',NULL,'EPSG','4497','EPSG','4267','EPSG','11900',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6620','projected_crs','EPSG','26785','EPSG','1389','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26786','NAD27 / Massachusetts Mainland',NULL,'EPSG','4497','EPSG','4267','EPSG','12001',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6621','projected_crs','EPSG','26786','EPSG','2209','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26787','NAD27 / Massachusetts Island',NULL,'EPSG','4497','EPSG','4267','EPSG','12002',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6622','projected_crs','EPSG','26787','EPSG','2208','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26791','NAD27 / Minnesota North',NULL,'EPSG','4497','EPSG','4267','EPSG','12201',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6623','projected_crs','EPSG','26791','EPSG','2214','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26792','NAD27 / Minnesota Central',NULL,'EPSG','4497','EPSG','4267','EPSG','12202',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6624','projected_crs','EPSG','26792','EPSG','2213','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26793','NAD27 / Minnesota South',NULL,'EPSG','4497','EPSG','4267','EPSG','12203',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6625','projected_crs','EPSG','26793','EPSG','2215','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26794','NAD27 / Mississippi East',NULL,'EPSG','4497','EPSG','4267','EPSG','12301',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6626','projected_crs','EPSG','26794','EPSG','2216','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26795','NAD27 / Mississippi West',NULL,'EPSG','4497','EPSG','4267','EPSG','12302',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6627','projected_crs','EPSG','26795','EPSG','2217','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26796','NAD27 / Missouri East',NULL,'EPSG','4497','EPSG','4267','EPSG','12401',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6628','projected_crs','EPSG','26796','EPSG','2219','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26797','NAD27 / Missouri Central',NULL,'EPSG','4497','EPSG','4267','EPSG','12402',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6629','projected_crs','EPSG','26797','EPSG','2218','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26798','NAD27 / Missouri West',NULL,'EPSG','4497','EPSG','4267','EPSG','12403',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6630','projected_crs','EPSG','26798','EPSG','2220','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26799','NAD27 / California zone VII',NULL,'EPSG','4497','EPSG','4267','EPSG','10408',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6631','projected_crs','EPSG','26799','EPSG','2181','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26801','NAD Michigan / Michigan East',NULL,'EPSG','4497','EPSG','4268','EPSG','12101',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6632','projected_crs','EPSG','26801','EPSG','1720','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26802','NAD Michigan / Michigan Old Central',NULL,'EPSG','4497','EPSG','4268','EPSG','12102',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6633','projected_crs','EPSG','26802','EPSG','1721','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26803','NAD Michigan / Michigan West',NULL,'EPSG','4497','EPSG','4268','EPSG','12103',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6634','projected_crs','EPSG','26803','EPSG','3652','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26811','NAD Michigan / Michigan North',NULL,'EPSG','4497','EPSG','4268','EPSG','12111',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6635','projected_crs','EPSG','26811','EPSG','1723','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26812','NAD Michigan / Michigan Central',NULL,'EPSG','4497','EPSG','4268','EPSG','12112',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6636','projected_crs','EPSG','26812','EPSG','1724','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26813','NAD Michigan / Michigan South',NULL,'EPSG','4497','EPSG','4268','EPSG','12113',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6637','projected_crs','EPSG','26813','EPSG','1725','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26814','NAD83 / Maine East (ftUS)',NULL,'EPSG','4499','EPSG','4269','EPSG','11833',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6638','projected_crs','EPSG','26814','EPSG','2206','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26815','NAD83 / Maine West (ftUS)',NULL,'EPSG','4499','EPSG','4269','EPSG','11834',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6639','projected_crs','EPSG','26815','EPSG','2207','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26819','NAD83 / Minnesota North (ftUS)',NULL,'EPSG','4499','EPSG','4269','EPSG','12234',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6640','projected_crs','EPSG','26819','EPSG','2214','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26820','NAD83 / Minnesota Central (ftUS)',NULL,'EPSG','4499','EPSG','4269','EPSG','12235',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6641','projected_crs','EPSG','26820','EPSG','2213','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26821','NAD83 / Minnesota South (ftUS)',NULL,'EPSG','4499','EPSG','4269','EPSG','12236',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6642','projected_crs','EPSG','26821','EPSG','2215','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26822','NAD83 / Nebraska (ftUS)',NULL,'EPSG','4499','EPSG','4269','EPSG','15396',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6643','projected_crs','EPSG','26822','EPSG','1396','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26823','NAD83 / West Virginia North (ftUS)',NULL,'EPSG','4499','EPSG','4269','EPSG','14733',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6644','projected_crs','EPSG','26823','EPSG','2264','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26824','NAD83 / West Virginia South (ftUS)',NULL,'EPSG','4499','EPSG','4269','EPSG','14734',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6645','projected_crs','EPSG','26824','EPSG','2265','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26825','NAD83(HARN) / Maine East (ftUS)',NULL,'EPSG','4499','EPSG','4152','EPSG','11833',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6646','projected_crs','EPSG','26825','EPSG','2206','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26826','NAD83(HARN) / Maine West (ftUS)',NULL,'EPSG','4499','EPSG','4152','EPSG','11834',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6647','projected_crs','EPSG','26826','EPSG','2207','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26830','NAD83(HARN) / Minnesota North (ftUS)',NULL,'EPSG','4499','EPSG','4152','EPSG','12234',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6648','projected_crs','EPSG','26830','EPSG','2214','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26831','NAD83(HARN) / Minnesota Central (ftUS)',NULL,'EPSG','4499','EPSG','4152','EPSG','12235',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6649','projected_crs','EPSG','26831','EPSG','2213','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26832','NAD83(HARN) / Minnesota South (ftUS)',NULL,'EPSG','4499','EPSG','4152','EPSG','12236',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6650','projected_crs','EPSG','26832','EPSG','2215','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26833','NAD83(HARN) / Nebraska (ftUS)',NULL,'EPSG','4499','EPSG','4152','EPSG','15396',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6651','projected_crs','EPSG','26833','EPSG','1396','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26834','NAD83(HARN) / West Virginia North (ftUS)',NULL,'EPSG','4499','EPSG','4152','EPSG','14733',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6652','projected_crs','EPSG','26834','EPSG','2264','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26835','NAD83(HARN) / West Virginia South (ftUS)',NULL,'EPSG','4499','EPSG','4152','EPSG','14734',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6653','projected_crs','EPSG','26835','EPSG','2265','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26836','NAD83(NSRS2007) / Maine East (ftUS)',NULL,'EPSG','4499','EPSG','4759','EPSG','11833',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6654','projected_crs','EPSG','26836','EPSG','2206','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26837','NAD83(NSRS2007) / Maine West (ftUS)',NULL,'EPSG','4499','EPSG','4759','EPSG','11834',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6655','projected_crs','EPSG','26837','EPSG','2207','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26841','NAD83(NSRS2007) / Minnesota North (ftUS)',NULL,'EPSG','4499','EPSG','4759','EPSG','12234',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6656','projected_crs','EPSG','26841','EPSG','2214','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26842','NAD83(NSRS2007) / Minnesota Central (ftUS)',NULL,'EPSG','4499','EPSG','4759','EPSG','12235',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6657','projected_crs','EPSG','26842','EPSG','2213','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26843','NAD83(NSRS2007) / Minnesota South (ftUS)',NULL,'EPSG','4499','EPSG','4759','EPSG','12236',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6658','projected_crs','EPSG','26843','EPSG','2215','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26844','NAD83(NSRS2007) / Nebraska (ftUS)',NULL,'EPSG','4499','EPSG','4759','EPSG','15396',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6659','projected_crs','EPSG','26844','EPSG','1396','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26845','NAD83(NSRS2007) / West Virginia North (ftUS)',NULL,'EPSG','4499','EPSG','4759','EPSG','14733',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6660','projected_crs','EPSG','26845','EPSG','2264','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26846','NAD83(NSRS2007) / West Virginia South (ftUS)',NULL,'EPSG','4499','EPSG','4759','EPSG','14734',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6661','projected_crs','EPSG','26846','EPSG','2265','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26847','NAD83 / Maine East (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','11833',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6662','projected_crs','EPSG','26847','EPSG','2206','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26848','NAD83 / Maine West (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','11834',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6663','projected_crs','EPSG','26848','EPSG','2207','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26849','NAD83 / Minnesota North (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','12234',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6664','projected_crs','EPSG','26849','EPSG','2214','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26850','NAD83 / Minnesota Central (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','12235',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6665','projected_crs','EPSG','26850','EPSG','2213','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26851','NAD83 / Minnesota South (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','12236',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6666','projected_crs','EPSG','26851','EPSG','2215','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26852','NAD83 / Nebraska (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15396',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6667','projected_crs','EPSG','26852','EPSG','1396','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26853','NAD83 / West Virginia North (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','14735',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6668','projected_crs','EPSG','26853','EPSG','2264','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26854','NAD83 / West Virginia South (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','14736',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6669','projected_crs','EPSG','26854','EPSG','2265','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26855','NAD83(HARN) / Maine East (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','11833',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6670','projected_crs','EPSG','26855','EPSG','2206','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26856','NAD83(HARN) / Maine West (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','11834',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6671','projected_crs','EPSG','26856','EPSG','2207','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26857','NAD83(HARN) / Minnesota North (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','12234',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6672','projected_crs','EPSG','26857','EPSG','2214','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26858','NAD83(HARN) / Minnesota Central (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','12235',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6673','projected_crs','EPSG','26858','EPSG','2213','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26859','NAD83(HARN) / Minnesota South (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','12236',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6674','projected_crs','EPSG','26859','EPSG','2215','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26860','NAD83(HARN) / Nebraska (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','15396',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6675','projected_crs','EPSG','26860','EPSG','1396','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26861','NAD83(HARN) / West Virginia North (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','14735',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6676','projected_crs','EPSG','26861','EPSG','2264','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26862','NAD83(HARN) / West Virginia South (ftUS)',NULL,'EPSG','4497','EPSG','4152','EPSG','14736',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6677','projected_crs','EPSG','26862','EPSG','2265','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26863','NAD83(NSRS2007) / Maine East (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','11833',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6678','projected_crs','EPSG','26863','EPSG','2206','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26864','NAD83(NSRS2007) / Maine West (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','11834',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6679','projected_crs','EPSG','26864','EPSG','2207','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26865','NAD83(NSRS2007) / Minnesota North (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','12234',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6680','projected_crs','EPSG','26865','EPSG','2214','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26866','NAD83(NSRS2007) / Minnesota Central (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','12235',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6681','projected_crs','EPSG','26866','EPSG','2213','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26867','NAD83(NSRS2007) / Minnesota South (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','12236',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6682','projected_crs','EPSG','26867','EPSG','2215','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26868','NAD83(NSRS2007) / Nebraska (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','15396',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6683','projected_crs','EPSG','26868','EPSG','1396','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26869','NAD83(NSRS2007) / West Virginia North (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','14735',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6684','projected_crs','EPSG','26869','EPSG','2264','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26870','NAD83(NSRS2007) / West Virginia South (ftUS)',NULL,'EPSG','4497','EPSG','4759','EPSG','14736',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6685','projected_crs','EPSG','26870','EPSG','2265','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26891','NAD83(CSRS) / MTM zone 11',NULL,'EPSG','4400','EPSG','4617','EPSG','17711',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6686','projected_crs','EPSG','26891','EPSG','1432','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26892','NAD83(CSRS) / MTM zone 12',NULL,'EPSG','4400','EPSG','4617','EPSG','17712',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6687','projected_crs','EPSG','26892','EPSG','1433','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26893','NAD83(CSRS) / MTM zone 13',NULL,'EPSG','4400','EPSG','4617','EPSG','17713',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6688','projected_crs','EPSG','26893','EPSG','1434','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26894','NAD83(CSRS) / MTM zone 14',NULL,'EPSG','4400','EPSG','4617','EPSG','17714',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6689','projected_crs','EPSG','26894','EPSG','1435','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26895','NAD83(CSRS) / MTM zone 15',NULL,'EPSG','4400','EPSG','4617','EPSG','17715',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6690','projected_crs','EPSG','26895','EPSG','1436','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26896','NAD83(CSRS) / MTM zone 16',NULL,'EPSG','4400','EPSG','4617','EPSG','17716',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6691','projected_crs','EPSG','26896','EPSG','1437','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26897','NAD83(CSRS) / MTM zone 17',NULL,'EPSG','4400','EPSG','4617','EPSG','17717',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6692','projected_crs','EPSG','26897','EPSG','1438','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26898','NAD83(CSRS) / MTM zone 1',NULL,'EPSG','4496','EPSG','4617','EPSG','17701',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6693','projected_crs','EPSG','26898','EPSG','2226','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26899','NAD83(CSRS) / MTM zone 2',NULL,'EPSG','4496','EPSG','4617','EPSG','17702',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6694','projected_crs','EPSG','26899','EPSG','2227','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26901','NAD83 / UTM zone 1N',NULL,'EPSG','4400','EPSG','4269','EPSG','16001',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6695','projected_crs','EPSG','26901','EPSG','3374','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26902','NAD83 / UTM zone 2N',NULL,'EPSG','4400','EPSG','4269','EPSG','16002',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6696','projected_crs','EPSG','26902','EPSG','3375','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26903','NAD83 / UTM zone 3N',NULL,'EPSG','4400','EPSG','4269','EPSG','16003',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6697','projected_crs','EPSG','26903','EPSG','2133','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26904','NAD83 / UTM zone 4N',NULL,'EPSG','4400','EPSG','4269','EPSG','16004',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6698','projected_crs','EPSG','26904','EPSG','3489','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26905','NAD83 / UTM zone 5N',NULL,'EPSG','4400','EPSG','4269','EPSG','16005',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6699','projected_crs','EPSG','26905','EPSG','3492','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26906','NAD83 / UTM zone 6N',NULL,'EPSG','4400','EPSG','4269','EPSG','16006',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6700','projected_crs','EPSG','26906','EPSG','2136','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26907','NAD83 / UTM zone 7N',NULL,'EPSG','4400','EPSG','4269','EPSG','16007',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6701','projected_crs','EPSG','26907','EPSG','3872','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26908','NAD83 / UTM zone 8N',NULL,'EPSG','4400','EPSG','4269','EPSG','16008',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6702','projected_crs','EPSG','26908','EPSG','3867','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26909','NAD83 / UTM zone 9N',NULL,'EPSG','4400','EPSG','4269','EPSG','16009',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6703','projected_crs','EPSG','26909','EPSG','3866','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26910','NAD83 / UTM zone 10N',NULL,'EPSG','4400','EPSG','4269','EPSG','16010',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6704','projected_crs','EPSG','26910','EPSG','3864','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26911','NAD83 / UTM zone 11N',NULL,'EPSG','4400','EPSG','4269','EPSG','16011',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6705','projected_crs','EPSG','26911','EPSG','3404','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26912','NAD83 / UTM zone 12N',NULL,'EPSG','4400','EPSG','4269','EPSG','16012',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6706','projected_crs','EPSG','26912','EPSG','3405','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26913','NAD83 / UTM zone 13N',NULL,'EPSG','4400','EPSG','4269','EPSG','16013',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6707','projected_crs','EPSG','26913','EPSG','3406','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26914','NAD83 / UTM zone 14N',NULL,'EPSG','4400','EPSG','4269','EPSG','16014',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6708','projected_crs','EPSG','26914','EPSG','3407','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26915','NAD83 / UTM zone 15N',NULL,'EPSG','4400','EPSG','4269','EPSG','16015',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6709','projected_crs','EPSG','26915','EPSG','3114','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26916','NAD83 / UTM zone 16N',NULL,'EPSG','4400','EPSG','4269','EPSG','16016',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6710','projected_crs','EPSG','26916','EPSG','3115','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26917','NAD83 / UTM zone 17N',NULL,'EPSG','4400','EPSG','4269','EPSG','16017',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6711','projected_crs','EPSG','26917','EPSG','3116','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26918','NAD83 / UTM zone 18N',NULL,'EPSG','4400','EPSG','4269','EPSG','16018',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6712','projected_crs','EPSG','26918','EPSG','3117','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26919','NAD83 / UTM zone 19N',NULL,'EPSG','4400','EPSG','4269','EPSG','16019',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6713','projected_crs','EPSG','26919','EPSG','3419','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26920','NAD83 / UTM zone 20N',NULL,'EPSG','4400','EPSG','4269','EPSG','16020',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6714','projected_crs','EPSG','26920','EPSG','3420','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26921','NAD83 / UTM zone 21N',NULL,'EPSG','4400','EPSG','4269','EPSG','16021',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6715','projected_crs','EPSG','26921','EPSG','2151','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26922','NAD83 / UTM zone 22N',NULL,'EPSG','4400','EPSG','4269','EPSG','16022',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6716','projected_crs','EPSG','26922','EPSG','2152','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26923','NAD83 / UTM zone 23N',NULL,'EPSG','4400','EPSG','4269','EPSG','16023',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6717','projected_crs','EPSG','26923','EPSG','2153','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26929','NAD83 / Alabama East',NULL,'EPSG','4499','EPSG','4269','EPSG','10131',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6718','projected_crs','EPSG','26929','EPSG','2154','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26930','NAD83 / Alabama West',NULL,'EPSG','4499','EPSG','4269','EPSG','10132',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6719','projected_crs','EPSG','26930','EPSG','2155','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26931','NAD83 / Alaska zone 1',NULL,'EPSG','4499','EPSG','4269','EPSG','15031',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6720','projected_crs','EPSG','26931','EPSG','2156','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26932','NAD83 / Alaska zone 2',NULL,'EPSG','4499','EPSG','4269','EPSG','15032',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6721','projected_crs','EPSG','26932','EPSG','2158','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26933','NAD83 / Alaska zone 3',NULL,'EPSG','4499','EPSG','4269','EPSG','15033',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6722','projected_crs','EPSG','26933','EPSG','2159','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26934','NAD83 / Alaska zone 4',NULL,'EPSG','4499','EPSG','4269','EPSG','15034',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6723','projected_crs','EPSG','26934','EPSG','2160','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26935','NAD83 / Alaska zone 5',NULL,'EPSG','4499','EPSG','4269','EPSG','15035',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6724','projected_crs','EPSG','26935','EPSG','2161','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26936','NAD83 / Alaska zone 6',NULL,'EPSG','4499','EPSG','4269','EPSG','15036',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6725','projected_crs','EPSG','26936','EPSG','2162','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26937','NAD83 / Alaska zone 7',NULL,'EPSG','4499','EPSG','4269','EPSG','15037',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6726','projected_crs','EPSG','26937','EPSG','2163','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26938','NAD83 / Alaska zone 8',NULL,'EPSG','4499','EPSG','4269','EPSG','15038',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6727','projected_crs','EPSG','26938','EPSG','2164','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26939','NAD83 / Alaska zone 9',NULL,'EPSG','4499','EPSG','4269','EPSG','15039',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6728','projected_crs','EPSG','26939','EPSG','2165','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26940','NAD83 / Alaska zone 10',NULL,'EPSG','4499','EPSG','4269','EPSG','15040',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6729','projected_crs','EPSG','26940','EPSG','2157','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26941','NAD83 / California zone 1',NULL,'EPSG','4499','EPSG','4269','EPSG','10431',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6730','projected_crs','EPSG','26941','EPSG','2175','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26942','NAD83 / California zone 2',NULL,'EPSG','4499','EPSG','4269','EPSG','10432',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6731','projected_crs','EPSG','26942','EPSG','2176','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26943','NAD83 / California zone 3',NULL,'EPSG','4499','EPSG','4269','EPSG','10433',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6732','projected_crs','EPSG','26943','EPSG','2177','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26944','NAD83 / California zone 4',NULL,'EPSG','4499','EPSG','4269','EPSG','10434',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6733','projected_crs','EPSG','26944','EPSG','2178','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26945','NAD83 / California zone 5',NULL,'EPSG','4499','EPSG','4269','EPSG','10435',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6734','projected_crs','EPSG','26945','EPSG','2182','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26946','NAD83 / California zone 6',NULL,'EPSG','4499','EPSG','4269','EPSG','10436',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6735','projected_crs','EPSG','26946','EPSG','2180','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26948','NAD83 / Arizona East',NULL,'EPSG','4499','EPSG','4269','EPSG','10231',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6736','projected_crs','EPSG','26948','EPSG','2167','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26949','NAD83 / Arizona Central',NULL,'EPSG','4499','EPSG','4269','EPSG','10232',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6737','projected_crs','EPSG','26949','EPSG','2166','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26950','NAD83 / Arizona West',NULL,'EPSG','4499','EPSG','4269','EPSG','10233',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6738','projected_crs','EPSG','26950','EPSG','2168','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26951','NAD83 / Arkansas North',NULL,'EPSG','4499','EPSG','4269','EPSG','10331',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6739','projected_crs','EPSG','26951','EPSG','2169','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26952','NAD83 / Arkansas South',NULL,'EPSG','4499','EPSG','4269','EPSG','10332',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6740','projected_crs','EPSG','26952','EPSG','2170','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26953','NAD83 / Colorado North',NULL,'EPSG','4499','EPSG','4269','EPSG','10531',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6741','projected_crs','EPSG','26953','EPSG','2184','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26954','NAD83 / Colorado Central',NULL,'EPSG','4499','EPSG','4269','EPSG','10532',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6742','projected_crs','EPSG','26954','EPSG','2183','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26955','NAD83 / Colorado South',NULL,'EPSG','4499','EPSG','4269','EPSG','10533',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6743','projected_crs','EPSG','26955','EPSG','2185','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26956','NAD83 / Connecticut',NULL,'EPSG','4499','EPSG','4269','EPSG','10630',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6744','projected_crs','EPSG','26956','EPSG','1377','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26957','NAD83 / Delaware',NULL,'EPSG','4499','EPSG','4269','EPSG','10730',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6745','projected_crs','EPSG','26957','EPSG','1378','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26958','NAD83 / Florida East',NULL,'EPSG','4499','EPSG','4269','EPSG','10931',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6746','projected_crs','EPSG','26958','EPSG','2186','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26959','NAD83 / Florida West',NULL,'EPSG','4499','EPSG','4269','EPSG','10932',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6747','projected_crs','EPSG','26959','EPSG','2188','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26960','NAD83 / Florida North',NULL,'EPSG','4499','EPSG','4269','EPSG','10933',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6748','projected_crs','EPSG','26960','EPSG','2187','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26961','NAD83 / Hawaii zone 1',NULL,'EPSG','4499','EPSG','4269','EPSG','15131',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6749','projected_crs','EPSG','26961','EPSG','1546','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26962','NAD83 / Hawaii zone 2',NULL,'EPSG','4499','EPSG','4269','EPSG','15132',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6750','projected_crs','EPSG','26962','EPSG','1547','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26963','NAD83 / Hawaii zone 3',NULL,'EPSG','4499','EPSG','4269','EPSG','15133',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6751','projected_crs','EPSG','26963','EPSG','1548','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26964','NAD83 / Hawaii zone 4',NULL,'EPSG','4499','EPSG','4269','EPSG','15134',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6752','projected_crs','EPSG','26964','EPSG','1549','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26965','NAD83 / Hawaii zone 5',NULL,'EPSG','4499','EPSG','4269','EPSG','15135',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6753','projected_crs','EPSG','26965','EPSG','1550','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26966','NAD83 / Georgia East',NULL,'EPSG','4499','EPSG','4269','EPSG','11031',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6754','projected_crs','EPSG','26966','EPSG','2189','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26967','NAD83 / Georgia West',NULL,'EPSG','4499','EPSG','4269','EPSG','11032',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6755','projected_crs','EPSG','26967','EPSG','2190','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26968','NAD83 / Idaho East',NULL,'EPSG','4499','EPSG','4269','EPSG','11131',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6756','projected_crs','EPSG','26968','EPSG','2192','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26969','NAD83 / Idaho Central',NULL,'EPSG','4499','EPSG','4269','EPSG','11132',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6757','projected_crs','EPSG','26969','EPSG','2191','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26970','NAD83 / Idaho West',NULL,'EPSG','4499','EPSG','4269','EPSG','11133',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6758','projected_crs','EPSG','26970','EPSG','2193','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26971','NAD83 / Illinois East',NULL,'EPSG','4499','EPSG','4269','EPSG','11231',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6759','projected_crs','EPSG','26971','EPSG','2194','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26972','NAD83 / Illinois West',NULL,'EPSG','4499','EPSG','4269','EPSG','11232',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6760','projected_crs','EPSG','26972','EPSG','2195','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26973','NAD83 / Indiana East',NULL,'EPSG','4499','EPSG','4269','EPSG','11331',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6761','projected_crs','EPSG','26973','EPSG','2196','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26974','NAD83 / Indiana West',NULL,'EPSG','4499','EPSG','4269','EPSG','11332',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6762','projected_crs','EPSG','26974','EPSG','2197','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26975','NAD83 / Iowa North',NULL,'EPSG','4499','EPSG','4269','EPSG','11431',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6763','projected_crs','EPSG','26975','EPSG','2198','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26976','NAD83 / Iowa South',NULL,'EPSG','4499','EPSG','4269','EPSG','11432',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6764','projected_crs','EPSG','26976','EPSG','2199','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26977','NAD83 / Kansas North',NULL,'EPSG','4499','EPSG','4269','EPSG','11531',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6765','projected_crs','EPSG','26977','EPSG','2200','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26978','NAD83 / Kansas South',NULL,'EPSG','4499','EPSG','4269','EPSG','11532',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6766','projected_crs','EPSG','26978','EPSG','2201','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26979','NAD83 / Kentucky North',NULL,'EPSG','4499','EPSG','4269','EPSG','11631',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6767','projected_crs','EPSG','26979','EPSG','2202','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26980','NAD83 / Kentucky South',NULL,'EPSG','4499','EPSG','4269','EPSG','11632',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6768','projected_crs','EPSG','26980','EPSG','2203','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26981','NAD83 / Louisiana North',NULL,'EPSG','4499','EPSG','4269','EPSG','11731',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6769','projected_crs','EPSG','26981','EPSG','2204','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26982','NAD83 / Louisiana South',NULL,'EPSG','4499','EPSG','4269','EPSG','11732',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6770','projected_crs','EPSG','26982','EPSG','2529','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26983','NAD83 / Maine East',NULL,'EPSG','4499','EPSG','4269','EPSG','11831',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6771','projected_crs','EPSG','26983','EPSG','2206','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26984','NAD83 / Maine West',NULL,'EPSG','4499','EPSG','4269','EPSG','11832',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6772','projected_crs','EPSG','26984','EPSG','2207','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26985','NAD83 / Maryland',NULL,'EPSG','4499','EPSG','4269','EPSG','11930',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6773','projected_crs','EPSG','26985','EPSG','1389','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26986','NAD83 / Massachusetts Mainland',NULL,'EPSG','4499','EPSG','4269','EPSG','12031',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6774','projected_crs','EPSG','26986','EPSG','2209','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26987','NAD83 / Massachusetts Island',NULL,'EPSG','4499','EPSG','4269','EPSG','12032',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6775','projected_crs','EPSG','26987','EPSG','2208','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26988','NAD83 / Michigan North',NULL,'EPSG','4499','EPSG','4269','EPSG','12141',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6776','projected_crs','EPSG','26988','EPSG','1723','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26989','NAD83 / Michigan Central',NULL,'EPSG','4499','EPSG','4269','EPSG','12142',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6777','projected_crs','EPSG','26989','EPSG','1724','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26990','NAD83 / Michigan South',NULL,'EPSG','4499','EPSG','4269','EPSG','12143',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6778','projected_crs','EPSG','26990','EPSG','1725','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26991','NAD83 / Minnesota North',NULL,'EPSG','4499','EPSG','4269','EPSG','12231',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6779','projected_crs','EPSG','26991','EPSG','2214','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26992','NAD83 / Minnesota Central',NULL,'EPSG','4499','EPSG','4269','EPSG','12232',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6780','projected_crs','EPSG','26992','EPSG','2213','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26993','NAD83 / Minnesota South',NULL,'EPSG','4499','EPSG','4269','EPSG','12233',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6781','projected_crs','EPSG','26993','EPSG','2215','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26994','NAD83 / Mississippi East',NULL,'EPSG','4499','EPSG','4269','EPSG','12331',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6782','projected_crs','EPSG','26994','EPSG','2216','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26995','NAD83 / Mississippi West',NULL,'EPSG','4499','EPSG','4269','EPSG','12332',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6783','projected_crs','EPSG','26995','EPSG','2217','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26996','NAD83 / Missouri East',NULL,'EPSG','4499','EPSG','4269','EPSG','12431',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6784','projected_crs','EPSG','26996','EPSG','2219','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26997','NAD83 / Missouri Central',NULL,'EPSG','4499','EPSG','4269','EPSG','12432',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6785','projected_crs','EPSG','26997','EPSG','2218','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','26998','NAD83 / Missouri West',NULL,'EPSG','4499','EPSG','4269','EPSG','12433',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6786','projected_crs','EPSG','26998','EPSG','2220','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','27037','Nahrwan 1967 / UTM zone 37N',NULL,'EPSG','4400','EPSG','4270','EPSG','16037',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6787','projected_crs','EPSG','27037','EPSG','3387','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','27038','Nahrwan 1967 / UTM zone 38N',NULL,'EPSG','4400','EPSG','4270','EPSG','16038',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6788','projected_crs','EPSG','27038','EPSG','3386','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','27039','Nahrwan 1967 / UTM zone 39N',NULL,'EPSG','4400','EPSG','4270','EPSG','16039',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6789','projected_crs','EPSG','27039','EPSG','1749','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','27040','Nahrwan 1967 / UTM zone 40N',NULL,'EPSG','4400','EPSG','4270','EPSG','16040',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6790','projected_crs','EPSG','27040','EPSG','1750','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','27120','Naparima 1972 / UTM zone 20N',NULL,'EPSG','4400','EPSG','4271','EPSG','16020',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6791','projected_crs','EPSG','27120','EPSG','1322','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','27200','NZGD49 / New Zealand Map Grid',NULL,'EPSG','4400','EPSG','4272','EPSG','19917',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6792','projected_crs','EPSG','27200','EPSG','3973','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','27205','NZGD49 / Mount Eden Circuit',NULL,'EPSG','4500','EPSG','4272','EPSG','17901',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6793','projected_crs','EPSG','27205','EPSG','3781','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','27206','NZGD49 / Bay of Plenty Circuit',NULL,'EPSG','4500','EPSG','4272','EPSG','17902',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6794','projected_crs','EPSG','27206','EPSG','3779','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','27207','NZGD49 / Poverty Bay Circuit',NULL,'EPSG','4500','EPSG','4272','EPSG','17903',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6795','projected_crs','EPSG','27207','EPSG','3780','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','27208','NZGD49 / Hawkes Bay Circuit',NULL,'EPSG','4500','EPSG','4272','EPSG','17904',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6796','projected_crs','EPSG','27208','EPSG','3772','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','27209','NZGD49 / Taranaki Circuit',NULL,'EPSG','4500','EPSG','4272','EPSG','17905',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6797','projected_crs','EPSG','27209','EPSG','3777','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','27210','NZGD49 / Tuhirangi Circuit',NULL,'EPSG','4500','EPSG','4272','EPSG','17906',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6798','projected_crs','EPSG','27210','EPSG','3778','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','27211','NZGD49 / Wanganui Circuit',NULL,'EPSG','4500','EPSG','4272','EPSG','17907',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6799','projected_crs','EPSG','27211','EPSG','3776','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','27212','NZGD49 / Wairarapa Circuit',NULL,'EPSG','4500','EPSG','4272','EPSG','17908',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6800','projected_crs','EPSG','27212','EPSG','3775','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','27213','NZGD49 / Wellington Circuit',NULL,'EPSG','4500','EPSG','4272','EPSG','17909',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6801','projected_crs','EPSG','27213','EPSG','3774','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','27214','NZGD49 / Collingwood Circuit',NULL,'EPSG','4500','EPSG','4272','EPSG','17910',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6802','projected_crs','EPSG','27214','EPSG','3782','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','27215','NZGD49 / Nelson Circuit',NULL,'EPSG','4500','EPSG','4272','EPSG','17911',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6803','projected_crs','EPSG','27215','EPSG','3784','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','27216','NZGD49 / Karamea Circuit',NULL,'EPSG','4500','EPSG','4272','EPSG','17912',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6804','projected_crs','EPSG','27216','EPSG','3783','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','27217','NZGD49 / Buller Circuit',NULL,'EPSG','4500','EPSG','4272','EPSG','17913',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6805','projected_crs','EPSG','27217','EPSG','3786','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','27218','NZGD49 / Grey Circuit',NULL,'EPSG','4500','EPSG','4272','EPSG','17914',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6806','projected_crs','EPSG','27218','EPSG','3787','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','27219','NZGD49 / Amuri Circuit',NULL,'EPSG','4500','EPSG','4272','EPSG','17915',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6807','projected_crs','EPSG','27219','EPSG','3788','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','27220','NZGD49 / Marlborough Circuit',NULL,'EPSG','4500','EPSG','4272','EPSG','17916',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6808','projected_crs','EPSG','27220','EPSG','3785','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','27221','NZGD49 / Hokitika Circuit',NULL,'EPSG','4500','EPSG','4272','EPSG','17917',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6809','projected_crs','EPSG','27221','EPSG','3789','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','27222','NZGD49 / Okarito Circuit',NULL,'EPSG','4500','EPSG','4272','EPSG','17918',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6810','projected_crs','EPSG','27222','EPSG','3791','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','27223','NZGD49 / Jacksons Bay Circuit',NULL,'EPSG','4500','EPSG','4272','EPSG','17919',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6811','projected_crs','EPSG','27223','EPSG','3794','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','27224','NZGD49 / Mount Pleasant Circuit',NULL,'EPSG','4500','EPSG','4272','EPSG','17920',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6812','projected_crs','EPSG','27224','EPSG','3790','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','27225','NZGD49 / Gawler Circuit',NULL,'EPSG','4500','EPSG','4272','EPSG','17921',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6813','projected_crs','EPSG','27225','EPSG','3792','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','27226','NZGD49 / Timaru Circuit',NULL,'EPSG','4500','EPSG','4272','EPSG','17922',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6814','projected_crs','EPSG','27226','EPSG','3793','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','27227','NZGD49 / Lindis Peak Circuit',NULL,'EPSG','4500','EPSG','4272','EPSG','17923',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6815','projected_crs','EPSG','27227','EPSG','3795','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','27228','NZGD49 / Mount Nicholas Circuit',NULL,'EPSG','4500','EPSG','4272','EPSG','17924',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6816','projected_crs','EPSG','27228','EPSG','3797','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','27229','NZGD49 / Mount York Circuit',NULL,'EPSG','4500','EPSG','4272','EPSG','17925',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6817','projected_crs','EPSG','27229','EPSG','3799','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','27230','NZGD49 / Observation Point Circuit',NULL,'EPSG','4500','EPSG','4272','EPSG','17926',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6818','projected_crs','EPSG','27230','EPSG','3796','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','27231','NZGD49 / North Taieri Circuit',NULL,'EPSG','4500','EPSG','4272','EPSG','17927',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6819','projected_crs','EPSG','27231','EPSG','3798','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','27232','NZGD49 / Bluff Circuit',NULL,'EPSG','4500','EPSG','4272','EPSG','17928',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6820','projected_crs','EPSG','27232','EPSG','3800','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','27258','NZGD49 / UTM zone 58S',NULL,'EPSG','4400','EPSG','4272','EPSG','16158',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6821','projected_crs','EPSG','27258','EPSG','3970','EPSG','1136');
INSERT INTO "projected_crs" VALUES('EPSG','27259','NZGD49 / UTM zone 59S',NULL,'EPSG','4400','EPSG','4272','EPSG','16159',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6822','projected_crs','EPSG','27259','EPSG','3971','EPSG','1136');
INSERT INTO "projected_crs" VALUES('EPSG','27260','NZGD49 / UTM zone 60S',NULL,'EPSG','4400','EPSG','4272','EPSG','16160',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6823','projected_crs','EPSG','27260','EPSG','3972','EPSG','1136');
INSERT INTO "projected_crs" VALUES('EPSG','27291','NZGD49 / North Island Grid',NULL,'EPSG','4409','EPSG','4272','EPSG','18141',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6824','projected_crs','EPSG','27291','EPSG','1500','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','27292','NZGD49 / South Island Grid',NULL,'EPSG','4409','EPSG','4272','EPSG','18142',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6825','projected_crs','EPSG','27292','EPSG','3344','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','27391','NGO 1948 (Oslo) / NGO zone I',NULL,'EPSG','4531','EPSG','4817','EPSG','18221',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6826','projected_crs','EPSG','27391','EPSG','1741','EPSG','1091');
INSERT INTO "projected_crs" VALUES('EPSG','27392','NGO 1948 (Oslo) / NGO zone II',NULL,'EPSG','4531','EPSG','4817','EPSG','18222',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6827','projected_crs','EPSG','27392','EPSG','1742','EPSG','1091');
INSERT INTO "projected_crs" VALUES('EPSG','27393','NGO 1948 (Oslo) / NGO zone III',NULL,'EPSG','4531','EPSG','4817','EPSG','18223',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6828','projected_crs','EPSG','27393','EPSG','1743','EPSG','1091');
INSERT INTO "projected_crs" VALUES('EPSG','27394','NGO 1948 (Oslo) / NGO zone IV',NULL,'EPSG','4531','EPSG','4817','EPSG','18224',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6829','projected_crs','EPSG','27394','EPSG','1744','EPSG','1091');
INSERT INTO "projected_crs" VALUES('EPSG','27395','NGO 1948 (Oslo) / NGO zone V',NULL,'EPSG','4531','EPSG','4817','EPSG','18225',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6830','projected_crs','EPSG','27395','EPSG','1745','EPSG','1091');
INSERT INTO "projected_crs" VALUES('EPSG','27396','NGO 1948 (Oslo) / NGO zone VI',NULL,'EPSG','4531','EPSG','4817','EPSG','18226',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6831','projected_crs','EPSG','27396','EPSG','1746','EPSG','1091');
INSERT INTO "projected_crs" VALUES('EPSG','27397','NGO 1948 (Oslo) / NGO zone VII',NULL,'EPSG','4531','EPSG','4817','EPSG','18227',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6832','projected_crs','EPSG','27397','EPSG','1747','EPSG','1091');
INSERT INTO "projected_crs" VALUES('EPSG','27398','NGO 1948 (Oslo) / NGO zone VIII',NULL,'EPSG','4531','EPSG','4817','EPSG','18228',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6833','projected_crs','EPSG','27398','EPSG','1748','EPSG','1091');
INSERT INTO "projected_crs" VALUES('EPSG','27429','Datum 73 / UTM zone 29N',NULL,'EPSG','4400','EPSG','4274','EPSG','16029',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6834','projected_crs','EPSG','27429','EPSG','1294','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','27492','Datum 73 / Modified Portuguese Grid',NULL,'EPSG','4530','EPSG','4274','EPSG','19974',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6835','projected_crs','EPSG','27492','EPSG','1294','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','27493','Datum 73 / Modified Portuguese Grid',NULL,'EPSG','4499','EPSG','4274','EPSG','19974',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6836','projected_crs','EPSG','27493','EPSG','1294','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','27500','ATF (Paris) / Nord de Guerre',NULL,'EPSG','4499','EPSG','4901','EPSG','19903',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6837','projected_crs','EPSG','27500','EPSG','1369','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','27561','NTF (Paris) / Lambert Nord France',NULL,'EPSG','4499','EPSG','4807','EPSG','18091',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6838','projected_crs','EPSG','27561','EPSG','1731','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','27562','NTF (Paris) / Lambert Centre France',NULL,'EPSG','4499','EPSG','4807','EPSG','18092',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6839','projected_crs','EPSG','27562','EPSG','1732','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','27563','NTF (Paris) / Lambert Sud France',NULL,'EPSG','4499','EPSG','4807','EPSG','18093',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6840','projected_crs','EPSG','27563','EPSG','1733','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','27564','NTF (Paris) / Lambert Corse',NULL,'EPSG','4499','EPSG','4807','EPSG','18094',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6841','projected_crs','EPSG','27564','EPSG','1327','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','27571','NTF (Paris) / Lambert zone I',NULL,'EPSG','4499','EPSG','4807','EPSG','18081',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6842','projected_crs','EPSG','27571','EPSG','1731','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','27572','NTF (Paris) / Lambert zone II',NULL,'EPSG','4499','EPSG','4807','EPSG','18082',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6843','projected_crs','EPSG','27572','EPSG','1734','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','27573','NTF (Paris) / Lambert zone III',NULL,'EPSG','4499','EPSG','4807','EPSG','18083',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6844','projected_crs','EPSG','27573','EPSG','1733','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','27574','NTF (Paris) / Lambert zone IV',NULL,'EPSG','4499','EPSG','4807','EPSG','18084',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6845','projected_crs','EPSG','27574','EPSG','1327','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','27581','NTF (Paris) / France I',NULL,'EPSG','4499','EPSG','4807','EPSG','18081',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6846','projected_crs','EPSG','27581','EPSG','1731','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','27582','NTF (Paris) / France II',NULL,'EPSG','4499','EPSG','4807','EPSG','18082',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6847','projected_crs','EPSG','27582','EPSG','1734','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','27583','NTF (Paris) / France III',NULL,'EPSG','4499','EPSG','4807','EPSG','18083',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6848','projected_crs','EPSG','27583','EPSG','1733','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','27584','NTF (Paris) / France IV',NULL,'EPSG','4499','EPSG','4807','EPSG','18084',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6849','projected_crs','EPSG','27584','EPSG','1327','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','27591','NTF (Paris) / Nord France',NULL,'EPSG','4499','EPSG','4807','EPSG','18091',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6850','projected_crs','EPSG','27591','EPSG','1731','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','27592','NTF (Paris) / Centre France',NULL,'EPSG','4499','EPSG','4807','EPSG','18092',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6851','projected_crs','EPSG','27592','EPSG','1732','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','27593','NTF (Paris) / Sud France',NULL,'EPSG','4499','EPSG','4807','EPSG','18093',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6852','projected_crs','EPSG','27593','EPSG','1733','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','27594','NTF (Paris) / Corse',NULL,'EPSG','4499','EPSG','4807','EPSG','18094',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6853','projected_crs','EPSG','27594','EPSG','1327','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','27700','OSGB36 / British National Grid',NULL,'EPSG','4400','EPSG','4277','EPSG','19916',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6854','projected_crs','EPSG','27700','EPSG','4390','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','27701','WGS 84 / Equi7 Africa',NULL,'EPSG','4400','EPSG','4326','EPSG','17771',NULL,0);
INSERT INTO "usage" VALUES('EPSG','20884','projected_crs','EPSG','27701','EPSG','4745','EPSG','1287');
INSERT INTO "projected_crs" VALUES('EPSG','27702','WGS 84 / Equi7 Antarctica',NULL,'EPSG','1027','EPSG','4326','EPSG','17772',NULL,0);
INSERT INTO "usage" VALUES('EPSG','20885','projected_crs','EPSG','27702','EPSG','4746','EPSG','1287');
INSERT INTO "projected_crs" VALUES('EPSG','27703','WGS 84 / Equi7 Asia',NULL,'EPSG','4400','EPSG','4326','EPSG','17773',NULL,0);
INSERT INTO "usage" VALUES('EPSG','20886','projected_crs','EPSG','27703','EPSG','4747','EPSG','1287');
INSERT INTO "projected_crs" VALUES('EPSG','27704','WGS 84 / Equi7 Europe',NULL,'EPSG','4400','EPSG','4326','EPSG','17774',NULL,0);
INSERT INTO "usage" VALUES('EPSG','20883','projected_crs','EPSG','27704','EPSG','4748','EPSG','1287');
INSERT INTO "projected_crs" VALUES('EPSG','27705','WGS 84 / Equi7 North America',NULL,'EPSG','4400','EPSG','4326','EPSG','17775',NULL,0);
INSERT INTO "usage" VALUES('EPSG','20887','projected_crs','EPSG','27705','EPSG','4749','EPSG','1287');
INSERT INTO "projected_crs" VALUES('EPSG','27706','WGS 84 / Equi7 Oceania',NULL,'EPSG','4400','EPSG','4326','EPSG','17776',NULL,0);
INSERT INTO "usage" VALUES('EPSG','20889','projected_crs','EPSG','27706','EPSG','4751','EPSG','1287');
INSERT INTO "projected_crs" VALUES('EPSG','27707','WGS 84 / Equi7 South America',NULL,'EPSG','4400','EPSG','4326','EPSG','17777',NULL,0);
INSERT INTO "usage" VALUES('EPSG','20895','projected_crs','EPSG','27707','EPSG','4750','EPSG','1287');
INSERT INTO "projected_crs" VALUES('EPSG','28191','Palestine 1923 / Palestine Grid',NULL,'EPSG','4400','EPSG','4281','EPSG','18201',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6855','projected_crs','EPSG','28191','EPSG','1356','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','28192','Palestine 1923 / Palestine Belt',NULL,'EPSG','4400','EPSG','4281','EPSG','18202',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6856','projected_crs','EPSG','28192','EPSG','1356','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','28193','Palestine 1923 / Israeli CS Grid',NULL,'EPSG','4400','EPSG','4281','EPSG','18203',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6857','projected_crs','EPSG','28193','EPSG','2603','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','28232','Pointe Noire / UTM zone 32S',NULL,'EPSG','4400','EPSG','4282','EPSG','16132',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6858','projected_crs','EPSG','28232','EPSG','1072','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','28348','GDA94 / MGA zone 48',NULL,'EPSG','4400','EPSG','4283','EPSG','17348',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6859','projected_crs','EPSG','28348','EPSG','4191','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','28349','GDA94 / MGA zone 49',NULL,'EPSG','4400','EPSG','4283','EPSG','17349',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6860','projected_crs','EPSG','28349','EPSG','4176','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','28350','GDA94 / MGA zone 50',NULL,'EPSG','4400','EPSG','4283','EPSG','17350',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6861','projected_crs','EPSG','28350','EPSG','4178','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','28351','GDA94 / MGA zone 51',NULL,'EPSG','4400','EPSG','4283','EPSG','17351',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6862','projected_crs','EPSG','28351','EPSG','1559','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','28352','GDA94 / MGA zone 52',NULL,'EPSG','4400','EPSG','4283','EPSG','17352',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6863','projected_crs','EPSG','28352','EPSG','1560','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','28353','GDA94 / MGA zone 53',NULL,'EPSG','4400','EPSG','4283','EPSG','17353',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6864','projected_crs','EPSG','28353','EPSG','1561','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','28354','GDA94 / MGA zone 54',NULL,'EPSG','4400','EPSG','4283','EPSG','17354',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6865','projected_crs','EPSG','28354','EPSG','1562','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','28355','GDA94 / MGA zone 55',NULL,'EPSG','4400','EPSG','4283','EPSG','17355',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6866','projected_crs','EPSG','28355','EPSG','1563','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','28356','GDA94 / MGA zone 56',NULL,'EPSG','4400','EPSG','4283','EPSG','17356',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6867','projected_crs','EPSG','28356','EPSG','1564','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','28357','GDA94 / MGA zone 57',NULL,'EPSG','4400','EPSG','4283','EPSG','17357',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6868','projected_crs','EPSG','28357','EPSG','4196','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','28358','GDA94 / MGA zone 58',NULL,'EPSG','4400','EPSG','4283','EPSG','17358',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6869','projected_crs','EPSG','28358','EPSG','4175','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','28402','Pulkovo 1942 / Gauss-Kruger zone 2',NULL,'EPSG','4530','EPSG','4284','EPSG','16202',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6870','projected_crs','EPSG','28402','EPSG','1805','EPSG','1160');
INSERT INTO "projected_crs" VALUES('EPSG','28403','Pulkovo 1942 / Gauss-Kruger zone 3',NULL,'EPSG','4530','EPSG','4284','EPSG','16203',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6871','projected_crs','EPSG','28403','EPSG','1792','EPSG','1160');
INSERT INTO "projected_crs" VALUES('EPSG','28404','Pulkovo 1942 / Gauss-Kruger zone 4',NULL,'EPSG','4530','EPSG','4284','EPSG','16204',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6872','projected_crs','EPSG','28404','EPSG','1793','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','28405','Pulkovo 1942 / Gauss-Kruger zone 5',NULL,'EPSG','4530','EPSG','4284','EPSG','16205',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6873','projected_crs','EPSG','28405','EPSG','1794','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','28406','Pulkovo 1942 / Gauss-Kruger zone 6',NULL,'EPSG','4530','EPSG','4284','EPSG','16206',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6874','projected_crs','EPSG','28406','EPSG','1795','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','28407','Pulkovo 1942 / Gauss-Kruger zone 7',NULL,'EPSG','4530','EPSG','4284','EPSG','16207',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6875','projected_crs','EPSG','28407','EPSG','1796','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','28408','Pulkovo 1942 / Gauss-Kruger zone 8',NULL,'EPSG','4530','EPSG','4284','EPSG','16208',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6876','projected_crs','EPSG','28408','EPSG','1797','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','28409','Pulkovo 1942 / Gauss-Kruger zone 9',NULL,'EPSG','4530','EPSG','4284','EPSG','16209',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6877','projected_crs','EPSG','28409','EPSG','1798','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','28410','Pulkovo 1942 / Gauss-Kruger zone 10',NULL,'EPSG','4530','EPSG','4284','EPSG','16210',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6878','projected_crs','EPSG','28410','EPSG','1799','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','28411','Pulkovo 1942 / Gauss-Kruger zone 11',NULL,'EPSG','4530','EPSG','4284','EPSG','16211',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6879','projected_crs','EPSG','28411','EPSG','1800','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','28412','Pulkovo 1942 / Gauss-Kruger zone 12',NULL,'EPSG','4530','EPSG','4284','EPSG','16212',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6880','projected_crs','EPSG','28412','EPSG','1801','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','28413','Pulkovo 1942 / Gauss-Kruger zone 13',NULL,'EPSG','4530','EPSG','4284','EPSG','16213',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6881','projected_crs','EPSG','28413','EPSG','1802','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','28414','Pulkovo 1942 / Gauss-Kruger zone 14',NULL,'EPSG','4530','EPSG','4284','EPSG','16214',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6882','projected_crs','EPSG','28414','EPSG','1803','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','28415','Pulkovo 1942 / Gauss-Kruger zone 15',NULL,'EPSG','4530','EPSG','4284','EPSG','16215',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6883','projected_crs','EPSG','28415','EPSG','1804','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','28416','Pulkovo 1942 / Gauss-Kruger zone 16',NULL,'EPSG','4530','EPSG','4284','EPSG','16216',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6884','projected_crs','EPSG','28416','EPSG','1775','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','28417','Pulkovo 1942 / Gauss-Kruger zone 17',NULL,'EPSG','4530','EPSG','4284','EPSG','16217',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6885','projected_crs','EPSG','28417','EPSG','1776','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','28418','Pulkovo 1942 / Gauss-Kruger zone 18',NULL,'EPSG','4530','EPSG','4284','EPSG','16218',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6886','projected_crs','EPSG','28418','EPSG','1777','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','28419','Pulkovo 1942 / Gauss-Kruger zone 19',NULL,'EPSG','4530','EPSG','4284','EPSG','16219',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6887','projected_crs','EPSG','28419','EPSG','1778','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','28420','Pulkovo 1942 / Gauss-Kruger zone 20',NULL,'EPSG','4530','EPSG','4284','EPSG','16220',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6888','projected_crs','EPSG','28420','EPSG','1779','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','28421','Pulkovo 1942 / Gauss-Kruger zone 21',NULL,'EPSG','4530','EPSG','4284','EPSG','16221',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6889','projected_crs','EPSG','28421','EPSG','1780','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','28422','Pulkovo 1942 / Gauss-Kruger zone 22',NULL,'EPSG','4530','EPSG','4284','EPSG','16222',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6890','projected_crs','EPSG','28422','EPSG','1781','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','28423','Pulkovo 1942 / Gauss-Kruger zone 23',NULL,'EPSG','4530','EPSG','4284','EPSG','16223',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6891','projected_crs','EPSG','28423','EPSG','1782','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','28424','Pulkovo 1942 / Gauss-Kruger zone 24',NULL,'EPSG','4530','EPSG','4284','EPSG','16224',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6892','projected_crs','EPSG','28424','EPSG','1783','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','28425','Pulkovo 1942 / Gauss-Kruger zone 25',NULL,'EPSG','4530','EPSG','4284','EPSG','16225',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6893','projected_crs','EPSG','28425','EPSG','1784','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','28426','Pulkovo 1942 / Gauss-Kruger zone 26',NULL,'EPSG','4530','EPSG','4284','EPSG','16226',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6894','projected_crs','EPSG','28426','EPSG','1785','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','28427','Pulkovo 1942 / Gauss-Kruger zone 27',NULL,'EPSG','4530','EPSG','4284','EPSG','16227',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6895','projected_crs','EPSG','28427','EPSG','1786','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','28428','Pulkovo 1942 / Gauss-Kruger zone 28',NULL,'EPSG','4530','EPSG','4284','EPSG','16228',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6896','projected_crs','EPSG','28428','EPSG','1787','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','28429','Pulkovo 1942 / Gauss-Kruger zone 29',NULL,'EPSG','4530','EPSG','4284','EPSG','16229',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6897','projected_crs','EPSG','28429','EPSG','1788','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','28430','Pulkovo 1942 / Gauss-Kruger zone 30',NULL,'EPSG','4530','EPSG','4284','EPSG','16230',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6898','projected_crs','EPSG','28430','EPSG','1789','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','28431','Pulkovo 1942 / Gauss-Kruger zone 31',NULL,'EPSG','4530','EPSG','4284','EPSG','16231',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6899','projected_crs','EPSG','28431','EPSG','1790','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','28432','Pulkovo 1942 / Gauss-Kruger zone 32',NULL,'EPSG','4530','EPSG','4284','EPSG','16232',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6900','projected_crs','EPSG','28432','EPSG','1791','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','28462','Pulkovo 1942 / Gauss-Kruger 2N',NULL,'EPSG','4530','EPSG','4284','EPSG','16302',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6901','projected_crs','EPSG','28462','EPSG','1805','EPSG','1209');
INSERT INTO "projected_crs" VALUES('EPSG','28463','Pulkovo 1942 / Gauss-Kruger 3N',NULL,'EPSG','4530','EPSG','4284','EPSG','16303',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6902','projected_crs','EPSG','28463','EPSG','1792','EPSG','1160');
INSERT INTO "projected_crs" VALUES('EPSG','28464','Pulkovo 1942 / Gauss-Kruger 4N',NULL,'EPSG','4530','EPSG','4284','EPSG','16304',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6903','projected_crs','EPSG','28464','EPSG','1793','EPSG','1160');
INSERT INTO "projected_crs" VALUES('EPSG','28465','Pulkovo 1942 / Gauss-Kruger 5N',NULL,'EPSG','4530','EPSG','4284','EPSG','16305',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6904','projected_crs','EPSG','28465','EPSG','1794','EPSG','1209');
INSERT INTO "projected_crs" VALUES('EPSG','28466','Pulkovo 1942 / Gauss-Kruger 6N',NULL,'EPSG','4530','EPSG','4284','EPSG','16306',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6905','projected_crs','EPSG','28466','EPSG','1795','EPSG','1209');
INSERT INTO "projected_crs" VALUES('EPSG','28467','Pulkovo 1942 / Gauss-Kruger 7N',NULL,'EPSG','4530','EPSG','4284','EPSG','16307',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6906','projected_crs','EPSG','28467','EPSG','1796','EPSG','1209');
INSERT INTO "projected_crs" VALUES('EPSG','28468','Pulkovo 1942 / Gauss-Kruger 8N',NULL,'EPSG','4530','EPSG','4284','EPSG','16308',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6907','projected_crs','EPSG','28468','EPSG','1797','EPSG','1209');
INSERT INTO "projected_crs" VALUES('EPSG','28469','Pulkovo 1942 / Gauss-Kruger 9N',NULL,'EPSG','4530','EPSG','4284','EPSG','16309',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6908','projected_crs','EPSG','28469','EPSG','1798','EPSG','1209');
INSERT INTO "projected_crs" VALUES('EPSG','28470','Pulkovo 1942 / Gauss-Kruger 10N',NULL,'EPSG','4530','EPSG','4284','EPSG','16310',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6909','projected_crs','EPSG','28470','EPSG','1799','EPSG','1209');
INSERT INTO "projected_crs" VALUES('EPSG','28471','Pulkovo 1942 / Gauss-Kruger 11N',NULL,'EPSG','4530','EPSG','4284','EPSG','16311',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6910','projected_crs','EPSG','28471','EPSG','1800','EPSG','1209');
INSERT INTO "projected_crs" VALUES('EPSG','28472','Pulkovo 1942 / Gauss-Kruger 12N',NULL,'EPSG','4530','EPSG','4284','EPSG','16312',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6911','projected_crs','EPSG','28472','EPSG','1801','EPSG','1209');
INSERT INTO "projected_crs" VALUES('EPSG','28473','Pulkovo 1942 / Gauss-Kruger 13N',NULL,'EPSG','4530','EPSG','4284','EPSG','16313',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6912','projected_crs','EPSG','28473','EPSG','1802','EPSG','1209');
INSERT INTO "projected_crs" VALUES('EPSG','28474','Pulkovo 1942 / Gauss-Kruger 14N',NULL,'EPSG','4530','EPSG','4284','EPSG','16314',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6913','projected_crs','EPSG','28474','EPSG','1803','EPSG','1209');
INSERT INTO "projected_crs" VALUES('EPSG','28475','Pulkovo 1942 / Gauss-Kruger 15N',NULL,'EPSG','4530','EPSG','4284','EPSG','16315',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6914','projected_crs','EPSG','28475','EPSG','1804','EPSG','1209');
INSERT INTO "projected_crs" VALUES('EPSG','28476','Pulkovo 1942 / Gauss-Kruger 16N',NULL,'EPSG','4530','EPSG','4284','EPSG','16316',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6915','projected_crs','EPSG','28476','EPSG','1775','EPSG','1209');
INSERT INTO "projected_crs" VALUES('EPSG','28477','Pulkovo 1942 / Gauss-Kruger 17N',NULL,'EPSG','4530','EPSG','4284','EPSG','16317',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6916','projected_crs','EPSG','28477','EPSG','1776','EPSG','1209');
INSERT INTO "projected_crs" VALUES('EPSG','28478','Pulkovo 1942 / Gauss-Kruger 18N',NULL,'EPSG','4530','EPSG','4284','EPSG','16318',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6917','projected_crs','EPSG','28478','EPSG','1777','EPSG','1209');
INSERT INTO "projected_crs" VALUES('EPSG','28479','Pulkovo 1942 / Gauss-Kruger 19N',NULL,'EPSG','4530','EPSG','4284','EPSG','16319',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6918','projected_crs','EPSG','28479','EPSG','1778','EPSG','1209');
INSERT INTO "projected_crs" VALUES('EPSG','28480','Pulkovo 1942 / Gauss-Kruger 20N',NULL,'EPSG','4530','EPSG','4284','EPSG','16320',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6919','projected_crs','EPSG','28480','EPSG','1779','EPSG','1209');
INSERT INTO "projected_crs" VALUES('EPSG','28481','Pulkovo 1942 / Gauss-Kruger 21N',NULL,'EPSG','4530','EPSG','4284','EPSG','16321',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6920','projected_crs','EPSG','28481','EPSG','1780','EPSG','1209');
INSERT INTO "projected_crs" VALUES('EPSG','28482','Pulkovo 1942 / Gauss-Kruger 22N',NULL,'EPSG','4530','EPSG','4284','EPSG','16322',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6921','projected_crs','EPSG','28482','EPSG','1781','EPSG','1209');
INSERT INTO "projected_crs" VALUES('EPSG','28483','Pulkovo 1942 / Gauss-Kruger 23N',NULL,'EPSG','4530','EPSG','4284','EPSG','16323',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6922','projected_crs','EPSG','28483','EPSG','1782','EPSG','1209');
INSERT INTO "projected_crs" VALUES('EPSG','28484','Pulkovo 1942 / Gauss-Kruger 24N',NULL,'EPSG','4530','EPSG','4284','EPSG','16324',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6923','projected_crs','EPSG','28484','EPSG','1783','EPSG','1209');
INSERT INTO "projected_crs" VALUES('EPSG','28485','Pulkovo 1942 / Gauss-Kruger 25N',NULL,'EPSG','4530','EPSG','4284','EPSG','16325',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6924','projected_crs','EPSG','28485','EPSG','1784','EPSG','1209');
INSERT INTO "projected_crs" VALUES('EPSG','28486','Pulkovo 1942 / Gauss-Kruger 26N',NULL,'EPSG','4530','EPSG','4284','EPSG','16326',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6925','projected_crs','EPSG','28486','EPSG','1785','EPSG','1209');
INSERT INTO "projected_crs" VALUES('EPSG','28487','Pulkovo 1942 / Gauss-Kruger 27N',NULL,'EPSG','4530','EPSG','4284','EPSG','16327',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6926','projected_crs','EPSG','28487','EPSG','1786','EPSG','1209');
INSERT INTO "projected_crs" VALUES('EPSG','28488','Pulkovo 1942 / Gauss-Kruger 28N',NULL,'EPSG','4530','EPSG','4284','EPSG','16328',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6927','projected_crs','EPSG','28488','EPSG','1787','EPSG','1209');
INSERT INTO "projected_crs" VALUES('EPSG','28489','Pulkovo 1942 / Gauss-Kruger 29N',NULL,'EPSG','4530','EPSG','4284','EPSG','16329',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6928','projected_crs','EPSG','28489','EPSG','1788','EPSG','1209');
INSERT INTO "projected_crs" VALUES('EPSG','28490','Pulkovo 1942 / Gauss-Kruger 30N',NULL,'EPSG','4530','EPSG','4284','EPSG','16330',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6929','projected_crs','EPSG','28490','EPSG','1789','EPSG','1209');
INSERT INTO "projected_crs" VALUES('EPSG','28491','Pulkovo 1942 / Gauss-Kruger 31N',NULL,'EPSG','4530','EPSG','4284','EPSG','16331',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6930','projected_crs','EPSG','28491','EPSG','1790','EPSG','1209');
INSERT INTO "projected_crs" VALUES('EPSG','28492','Pulkovo 1942 / Gauss-Kruger 32N',NULL,'EPSG','4530','EPSG','4284','EPSG','16332',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6931','projected_crs','EPSG','28492','EPSG','1791','EPSG','1209');
INSERT INTO "projected_crs" VALUES('EPSG','28600','Qatar 1974 / Qatar National Grid',NULL,'EPSG','4400','EPSG','4285','EPSG','19919',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6932','projected_crs','EPSG','28600','EPSG','1346','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','28991','Amersfoort / RD Old',NULL,'EPSG','4499','EPSG','4289','EPSG','19913',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6933','projected_crs','EPSG','28991','EPSG','1275','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','28992','Amersfoort / RD New',NULL,'EPSG','4499','EPSG','4289','EPSG','19914',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6934','projected_crs','EPSG','28992','EPSG','1275','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','29100','SAD69 / Brazil Polyconic',NULL,'EPSG','4499','EPSG','4291','EPSG','19941',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6935','projected_crs','EPSG','29100','EPSG','1053','EPSG','1241');
INSERT INTO "projected_crs" VALUES('EPSG','29101','SAD69 / Brazil Polyconic',NULL,'EPSG','4499','EPSG','4618','EPSG','19941',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6936','projected_crs','EPSG','29101','EPSG','1053','EPSG','1241');
INSERT INTO "projected_crs" VALUES('EPSG','29118','SAD69 / UTM zone 18N',NULL,'EPSG','4400','EPSG','4291','EPSG','16018',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6937','projected_crs','EPSG','29118','EPSG','1807','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','29119','SAD69 / UTM zone 19N',NULL,'EPSG','4400','EPSG','4291','EPSG','16019',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6938','projected_crs','EPSG','29119','EPSG','1809','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','29120','SAD69 / UTM zone 20N',NULL,'EPSG','4400','EPSG','4291','EPSG','16020',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6939','projected_crs','EPSG','29120','EPSG','1811','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','29121','SAD69 / UTM zone 21N',NULL,'EPSG','4400','EPSG','4291','EPSG','16021',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6940','projected_crs','EPSG','29121','EPSG','1813','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','29122','SAD69 / UTM zone 22N',NULL,'EPSG','4400','EPSG','4291','EPSG','16022',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6941','projected_crs','EPSG','29122','EPSG','1815','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','29168','SAD69 / UTM zone 18N',NULL,'EPSG','4400','EPSG','4618','EPSG','16018',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6942','projected_crs','EPSG','29168','EPSG','3832','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','29169','SAD69 / UTM zone 19N',NULL,'EPSG','4400','EPSG','4618','EPSG','16019',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6943','projected_crs','EPSG','29169','EPSG','3834','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','29170','SAD69 / UTM zone 20N',NULL,'EPSG','4400','EPSG','4618','EPSG','16020',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6944','projected_crs','EPSG','29170','EPSG','3839','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','29171','SAD69 / UTM zone 21N',NULL,'EPSG','4400','EPSG','4618','EPSG','16021',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6945','projected_crs','EPSG','29171','EPSG','3841','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','29172','SAD69 / UTM zone 22N',NULL,'EPSG','4400','EPSG','4618','EPSG','16022',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6946','projected_crs','EPSG','29172','EPSG','1815','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','29177','SAD69 / UTM zone 17S',NULL,'EPSG','4400','EPSG','4291','EPSG','16117',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6947','projected_crs','EPSG','29177','EPSG','1806','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','29178','SAD69 / UTM zone 18S',NULL,'EPSG','4400','EPSG','4291','EPSG','16118',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6948','projected_crs','EPSG','29178','EPSG','1808','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','29179','SAD69 / UTM zone 19S',NULL,'EPSG','4400','EPSG','4291','EPSG','16119',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6949','projected_crs','EPSG','29179','EPSG','1810','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','29180','SAD69 / UTM zone 20S',NULL,'EPSG','4400','EPSG','4291','EPSG','16120',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6950','projected_crs','EPSG','29180','EPSG','1812','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','29181','SAD69 / UTM zone 21S',NULL,'EPSG','4400','EPSG','4291','EPSG','16121',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6951','projected_crs','EPSG','29181','EPSG','1814','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','29182','SAD69 / UTM zone 22S',NULL,'EPSG','4400','EPSG','4291','EPSG','16122',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6952','projected_crs','EPSG','29182','EPSG','1816','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','29183','SAD69 / UTM zone 23S',NULL,'EPSG','4400','EPSG','4291','EPSG','16123',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6953','projected_crs','EPSG','29183','EPSG','1817','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','29184','SAD69 / UTM zone 24S',NULL,'EPSG','4400','EPSG','4291','EPSG','16124',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6954','projected_crs','EPSG','29184','EPSG','1818','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','29185','SAD69 / UTM zone 25S',NULL,'EPSG','4400','EPSG','4291','EPSG','16125',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6955','projected_crs','EPSG','29185','EPSG','1819','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','29187','SAD69 / UTM zone 17S',NULL,'EPSG','4400','EPSG','4618','EPSG','16117',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6956','projected_crs','EPSG','29187','EPSG','3831','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','29188','SAD69 / UTM zone 18S',NULL,'EPSG','4400','EPSG','4618','EPSG','16118',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6957','projected_crs','EPSG','29188','EPSG','3833','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','29189','SAD69 / UTM zone 19S',NULL,'EPSG','4400','EPSG','4618','EPSG','16119',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6958','projected_crs','EPSG','29189','EPSG','3835','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','29190','SAD69 / UTM zone 20S',NULL,'EPSG','4400','EPSG','4618','EPSG','16120',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6959','projected_crs','EPSG','29190','EPSG','3840','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','29191','SAD69 / UTM zone 21S',NULL,'EPSG','4400','EPSG','4618','EPSG','16121',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6960','projected_crs','EPSG','29191','EPSG','1814','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','29192','SAD69 / UTM zone 22S',NULL,'EPSG','4400','EPSG','4618','EPSG','16122',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6961','projected_crs','EPSG','29192','EPSG','1816','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','29193','SAD69 / UTM zone 23S',NULL,'EPSG','4400','EPSG','4618','EPSG','16123',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6962','projected_crs','EPSG','29193','EPSG','3445','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','29194','SAD69 / UTM zone 24S',NULL,'EPSG','4400','EPSG','4618','EPSG','16124',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6963','projected_crs','EPSG','29194','EPSG','3446','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','29195','SAD69 / UTM zone 25S',NULL,'EPSG','4400','EPSG','4618','EPSG','16125',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6964','projected_crs','EPSG','29195','EPSG','3447','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','29220','Sapper Hill 1943 / UTM zone 20S',NULL,'EPSG','4400','EPSG','4292','EPSG','16120',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6965','projected_crs','EPSG','29220','EPSG','1820','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','29221','Sapper Hill 1943 / UTM zone 21S',NULL,'EPSG','4400','EPSG','4292','EPSG','16121',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6966','projected_crs','EPSG','29221','EPSG','1821','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','29333','Schwarzeck / UTM zone 33S',NULL,'EPSG','4400','EPSG','4293','EPSG','16133',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6967','projected_crs','EPSG','29333','EPSG','1822','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','29371','Schwarzeck / Lo22/11',NULL,'EPSG','6502','EPSG','4293','EPSG','17611',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6968','projected_crs','EPSG','29371','EPSG','1838','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','29373','Schwarzeck / Lo22/13',NULL,'EPSG','6502','EPSG','4293','EPSG','17613',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6969','projected_crs','EPSG','29373','EPSG','1839','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','29375','Schwarzeck / Lo22/15',NULL,'EPSG','6502','EPSG','4293','EPSG','17615',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6970','projected_crs','EPSG','29375','EPSG','1840','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','29377','Schwarzeck / Lo22/17',NULL,'EPSG','6502','EPSG','4293','EPSG','17617',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6971','projected_crs','EPSG','29377','EPSG','1841','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','29379','Schwarzeck / Lo22/19',NULL,'EPSG','6502','EPSG','4293','EPSG','17619',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6972','projected_crs','EPSG','29379','EPSG','1842','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','29381','Schwarzeck / Lo22/21',NULL,'EPSG','6502','EPSG','4293','EPSG','17621',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6973','projected_crs','EPSG','29381','EPSG','1843','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','29383','Schwarzeck / Lo22/23',NULL,'EPSG','6502','EPSG','4293','EPSG','17623',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6974','projected_crs','EPSG','29383','EPSG','1844','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','29385','Schwarzeck / Lo22/25',NULL,'EPSG','6502','EPSG','4293','EPSG','17625',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6975','projected_crs','EPSG','29385','EPSG','1845','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','29635','Sudan / UTM zone 35N',NULL,'EPSG','4400','EPSG','4296','EPSG','16035',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6976','projected_crs','EPSG','29635','EPSG','1846','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','29636','Sudan / UTM zone 36N',NULL,'EPSG','4400','EPSG','4296','EPSG','16036',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6977','projected_crs','EPSG','29636','EPSG','1847','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','29700','Tananarive (Paris) / Laborde Grid',NULL,'EPSG','4499','EPSG','4810','EPSG','19911',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6978','projected_crs','EPSG','29700','EPSG','3273','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','29701','Tananarive (Paris) / Laborde Grid',NULL,'EPSG','4530','EPSG','4810','EPSG','19861',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6979','projected_crs','EPSG','29701','EPSG','3273','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','29702','Tananarive (Paris) / Laborde Grid approximation',NULL,'EPSG','4530','EPSG','4810','EPSG','19911',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6980','projected_crs','EPSG','29702','EPSG','3273','EPSG','1211');
INSERT INTO "projected_crs" VALUES('EPSG','29738','Tananarive / UTM zone 38S',NULL,'EPSG','4400','EPSG','4297','EPSG','16138',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6981','projected_crs','EPSG','29738','EPSG','1848','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','29739','Tananarive / UTM zone 39S',NULL,'EPSG','4400','EPSG','4297','EPSG','16139',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6982','projected_crs','EPSG','29739','EPSG','1849','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','29849','Timbalai 1948 / UTM zone 49N',NULL,'EPSG','4400','EPSG','4298','EPSG','16049',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6983','projected_crs','EPSG','29849','EPSG','1852','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','29850','Timbalai 1948 / UTM zone 50N',NULL,'EPSG','4400','EPSG','4298','EPSG','16050',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6984','projected_crs','EPSG','29850','EPSG','1853','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','29871','Timbalai 1948 / RSO Borneo (ch)',NULL,'EPSG','4402','EPSG','4298','EPSG','19956',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6985','projected_crs','EPSG','29871','EPSG','1362','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','29872','Timbalai 1948 / RSO Borneo (ftSe)',NULL,'EPSG','4405','EPSG','4298','EPSG','19957',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6986','projected_crs','EPSG','29872','EPSG','3977','EPSG','1136');
INSERT INTO "projected_crs" VALUES('EPSG','29873','Timbalai 1948 / RSO Borneo (m)',NULL,'EPSG','4400','EPSG','4298','EPSG','19958',NULL,0);
INSERT INTO "usage" VALUES('EPSG','14397','projected_crs','EPSG','29873','EPSG','1362','EPSG','1149');
INSERT INTO "projected_crs" VALUES('EPSG','29874','Timbalai 1948 / RSO Sarawak LSD (m)',NULL,'EPSG','4400','EPSG','4298','EPSG','19838',NULL,0);
INSERT INTO "usage" VALUES('EPSG','14400','projected_crs','EPSG','29874','EPSG','4611','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','29900','TM65 / Irish National Grid',NULL,'EPSG','4400','EPSG','4299','EPSG','19908',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6988','projected_crs','EPSG','29900','EPSG','1305','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','29901','OSNI 1952 / Irish National Grid',NULL,'EPSG','4400','EPSG','4188','EPSG','19973',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6989','projected_crs','EPSG','29901','EPSG','2530','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','29902','TM65 / Irish Grid',NULL,'EPSG','4400','EPSG','4299','EPSG','19972',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6990','projected_crs','EPSG','29902','EPSG','3767','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','29903','TM75 / Irish Grid',NULL,'EPSG','4400','EPSG','4300','EPSG','19972',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6991','projected_crs','EPSG','29903','EPSG','1305','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','30161','Tokyo / Japan Plane Rectangular CS I',NULL,'EPSG','4530','EPSG','4301','EPSG','17801',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6992','projected_crs','EPSG','30161','EPSG','1854','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','30162','Tokyo / Japan Plane Rectangular CS II',NULL,'EPSG','4530','EPSG','4301','EPSG','17802',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6993','projected_crs','EPSG','30162','EPSG','1855','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','30163','Tokyo / Japan Plane Rectangular CS III',NULL,'EPSG','4530','EPSG','4301','EPSG','17803',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6994','projected_crs','EPSG','30163','EPSG','1856','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','30164','Tokyo / Japan Plane Rectangular CS IV',NULL,'EPSG','4530','EPSG','4301','EPSG','17804',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6995','projected_crs','EPSG','30164','EPSG','1857','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','30165','Tokyo / Japan Plane Rectangular CS V',NULL,'EPSG','4530','EPSG','4301','EPSG','17805',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6996','projected_crs','EPSG','30165','EPSG','1858','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','30166','Tokyo / Japan Plane Rectangular CS VI',NULL,'EPSG','4530','EPSG','4301','EPSG','17806',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6997','projected_crs','EPSG','30166','EPSG','1859','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','30167','Tokyo / Japan Plane Rectangular CS VII',NULL,'EPSG','4530','EPSG','4301','EPSG','17807',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6998','projected_crs','EPSG','30167','EPSG','1860','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','30168','Tokyo / Japan Plane Rectangular CS VIII',NULL,'EPSG','4530','EPSG','4301','EPSG','17808',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6999','projected_crs','EPSG','30168','EPSG','1861','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','30169','Tokyo / Japan Plane Rectangular CS IX',NULL,'EPSG','4530','EPSG','4301','EPSG','17809',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7000','projected_crs','EPSG','30169','EPSG','1862','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','30170','Tokyo / Japan Plane Rectangular CS X',NULL,'EPSG','4530','EPSG','4301','EPSG','17810',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7001','projected_crs','EPSG','30170','EPSG','1863','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','30171','Tokyo / Japan Plane Rectangular CS XI',NULL,'EPSG','4530','EPSG','4301','EPSG','17811',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7002','projected_crs','EPSG','30171','EPSG','1864','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','30172','Tokyo / Japan Plane Rectangular CS XII',NULL,'EPSG','4530','EPSG','4301','EPSG','17812',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7003','projected_crs','EPSG','30172','EPSG','1865','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','30173','Tokyo / Japan Plane Rectangular CS XIII',NULL,'EPSG','4530','EPSG','4301','EPSG','17813',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7004','projected_crs','EPSG','30173','EPSG','1866','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','30174','Tokyo / Japan Plane Rectangular CS XIV',NULL,'EPSG','4530','EPSG','4301','EPSG','17814',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7005','projected_crs','EPSG','30174','EPSG','1867','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','30175','Tokyo / Japan Plane Rectangular CS XV',NULL,'EPSG','4530','EPSG','4301','EPSG','17815',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7006','projected_crs','EPSG','30175','EPSG','1868','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','30176','Tokyo / Japan Plane Rectangular CS XVI',NULL,'EPSG','4530','EPSG','4301','EPSG','17816',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7007','projected_crs','EPSG','30176','EPSG','1869','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','30177','Tokyo / Japan Plane Rectangular CS XVII',NULL,'EPSG','4530','EPSG','4301','EPSG','17817',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7008','projected_crs','EPSG','30177','EPSG','1870','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','30178','Tokyo / Japan Plane Rectangular CS XVIII',NULL,'EPSG','4530','EPSG','4301','EPSG','17818',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7009','projected_crs','EPSG','30178','EPSG','1871','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','30179','Tokyo / Japan Plane Rectangular CS XIX',NULL,'EPSG','4530','EPSG','4301','EPSG','17819',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7010','projected_crs','EPSG','30179','EPSG','1872','EPSG','1092');
INSERT INTO "projected_crs" VALUES('EPSG','30200','Trinidad 1903 / Trinidad Grid',NULL,'EPSG','4407','EPSG','4302','EPSG','19925',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7011','projected_crs','EPSG','30200','EPSG','1339','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','30339','TC(1948) / UTM zone 39N',NULL,'EPSG','4400','EPSG','4303','EPSG','16039',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7012','projected_crs','EPSG','30339','EPSG','1850','EPSG','1136');
INSERT INTO "projected_crs" VALUES('EPSG','30340','TC(1948) / UTM zone 40N',NULL,'EPSG','4400','EPSG','4303','EPSG','16040',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7013','projected_crs','EPSG','30340','EPSG','4022','EPSG','1136');
INSERT INTO "projected_crs" VALUES('EPSG','30491','Voirol 1875 / Nord Algerie (ancienne)',NULL,'EPSG','4499','EPSG','4304','EPSG','18011',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7014','projected_crs','EPSG','30491','EPSG','1728','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','30492','Voirol 1875 / Sud Algerie (ancienne)',NULL,'EPSG','4499','EPSG','4304','EPSG','18012',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7015','projected_crs','EPSG','30492','EPSG','4519','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','30493','Voirol 1879 / Nord Algerie (ancienne)',NULL,'EPSG','4499','EPSG','4671','EPSG','18011',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7016','projected_crs','EPSG','30493','EPSG','1728','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','30494','Voirol 1879 / Sud Algerie (ancienne)',NULL,'EPSG','4499','EPSG','4671','EPSG','18012',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7017','projected_crs','EPSG','30494','EPSG','4519','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','30729','Nord Sahara 1959 / UTM zone 29N',NULL,'EPSG','4400','EPSG','4307','EPSG','16029',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7018','projected_crs','EPSG','30729','EPSG','1735','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','30730','Nord Sahara 1959 / UTM zone 30N',NULL,'EPSG','4400','EPSG','4307','EPSG','16030',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7019','projected_crs','EPSG','30730','EPSG','3952','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','30731','Nord Sahara 1959 / UTM zone 31N',NULL,'EPSG','4400','EPSG','4307','EPSG','16031',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7020','projected_crs','EPSG','30731','EPSG','3953','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','30732','Nord Sahara 1959 / UTM zone 32N',NULL,'EPSG','4400','EPSG','4307','EPSG','16032',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7021','projected_crs','EPSG','30732','EPSG','3954','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','30791','Nord Sahara 1959 / Nord Algerie',NULL,'EPSG','4499','EPSG','4307','EPSG','18021',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7022','projected_crs','EPSG','30791','EPSG','1728','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','30792','Nord Sahara 1959 / Sud Algerie',NULL,'EPSG','4499','EPSG','4307','EPSG','18022',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7023','projected_crs','EPSG','30792','EPSG','1729','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','30800','RT38 2.5 gon W',NULL,'EPSG','4530','EPSG','4308','EPSG','19929',NULL,1);
INSERT INTO "usage" VALUES('EPSG','7024','projected_crs','EPSG','30800','EPSG','1225','EPSG','1056');
INSERT INTO "projected_crs" VALUES('EPSG','31028','Yoff / UTM zone 28N',NULL,'EPSG','4400','EPSG','4310','EPSG','16028',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7025','projected_crs','EPSG','31028','EPSG','1207','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31121','Zanderij / UTM zone 21N',NULL,'EPSG','4400','EPSG','4311','EPSG','16021',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7026','projected_crs','EPSG','31121','EPSG','1222','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31154','Zanderij / TM 54 NW',NULL,'EPSG','4400','EPSG','4311','EPSG','17054',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7027','projected_crs','EPSG','31154','EPSG','1727','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31170','Zanderij / Suriname Old TM',NULL,'EPSG','4400','EPSG','4311','EPSG','19954',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7028','projected_crs','EPSG','31170','EPSG','3312','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31171','Zanderij / Suriname TM',NULL,'EPSG','4400','EPSG','4311','EPSG','19955',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7029','projected_crs','EPSG','31171','EPSG','3312','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31251','MGI (Ferro) / Austria GK West Zone',NULL,'EPSG','4530','EPSG','4805','EPSG','18001',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7030','projected_crs','EPSG','31251','EPSG','1706','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31252','MGI (Ferro) / Austria GK Central Zone',NULL,'EPSG','4530','EPSG','4805','EPSG','18002',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7031','projected_crs','EPSG','31252','EPSG','1707','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31253','MGI (Ferro) / Austria GK East Zone',NULL,'EPSG','4530','EPSG','4805','EPSG','18003',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7032','projected_crs','EPSG','31253','EPSG','1708','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31254','MGI / Austria GK West',NULL,'EPSG','4530','EPSG','4312','EPSG','18004',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7033','projected_crs','EPSG','31254','EPSG','1706','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31255','MGI / Austria GK Central',NULL,'EPSG','4530','EPSG','4312','EPSG','18005',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7034','projected_crs','EPSG','31255','EPSG','1707','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31256','MGI / Austria GK East',NULL,'EPSG','4530','EPSG','4312','EPSG','18006',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7035','projected_crs','EPSG','31256','EPSG','1708','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31257','MGI / Austria GK M28',NULL,'EPSG','4530','EPSG','4312','EPSG','18007',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7036','projected_crs','EPSG','31257','EPSG','1706','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31258','MGI / Austria GK M31',NULL,'EPSG','4530','EPSG','4312','EPSG','18008',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7037','projected_crs','EPSG','31258','EPSG','1707','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31259','MGI / Austria GK M34',NULL,'EPSG','4530','EPSG','4312','EPSG','18009',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7038','projected_crs','EPSG','31259','EPSG','1708','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31265','MGI / 3-degree Gauss zone 5',NULL,'EPSG','4499','EPSG','4312','EPSG','16265',NULL,1);
INSERT INTO "usage" VALUES('EPSG','7039','projected_crs','EPSG','31265','EPSG','1709','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31266','MGI / 3-degree Gauss zone 6',NULL,'EPSG','4499','EPSG','4312','EPSG','16266',NULL,1);
INSERT INTO "usage" VALUES('EPSG','7040','projected_crs','EPSG','31266','EPSG','1710','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31267','MGI / 3-degree Gauss zone 7',NULL,'EPSG','4499','EPSG','4312','EPSG','16267',NULL,1);
INSERT INTO "usage" VALUES('EPSG','7041','projected_crs','EPSG','31267','EPSG','1711','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31268','MGI / 3-degree Gauss zone 8',NULL,'EPSG','4499','EPSG','4312','EPSG','16268',NULL,1);
INSERT INTO "usage" VALUES('EPSG','7042','projected_crs','EPSG','31268','EPSG','1712','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31275','MGI / Balkans zone 5',NULL,'EPSG','4530','EPSG','4312','EPSG','18275',NULL,1);
INSERT INTO "usage" VALUES('EPSG','7043','projected_crs','EPSG','31275','EPSG','1709','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31276','MGI / Balkans zone 6',NULL,'EPSG','4530','EPSG','4312','EPSG','18276',NULL,1);
INSERT INTO "usage" VALUES('EPSG','7044','projected_crs','EPSG','31276','EPSG','1710','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31277','MGI / Balkans zone 7',NULL,'EPSG','4530','EPSG','4312','EPSG','18277',NULL,1);
INSERT INTO "usage" VALUES('EPSG','7045','projected_crs','EPSG','31277','EPSG','1711','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31278','MGI / Balkans zone 8',NULL,'EPSG','4530','EPSG','4312','EPSG','18277',NULL,1);
INSERT INTO "usage" VALUES('EPSG','7046','projected_crs','EPSG','31278','EPSG','1712','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31279','MGI / Balkans zone 8',NULL,'EPSG','4530','EPSG','4312','EPSG','18278',NULL,1);
INSERT INTO "usage" VALUES('EPSG','7047','projected_crs','EPSG','31279','EPSG','1712','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31281','MGI (Ferro) / Austria West Zone',NULL,'EPSG','4530','EPSG','4805','EPSG','18041',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7048','projected_crs','EPSG','31281','EPSG','1706','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','31282','MGI (Ferro) / Austria Central Zone',NULL,'EPSG','4530','EPSG','4805','EPSG','18042',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7049','projected_crs','EPSG','31282','EPSG','1707','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','31283','MGI (Ferro) / Austria East Zone',NULL,'EPSG','4530','EPSG','4805','EPSG','18043',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7050','projected_crs','EPSG','31283','EPSG','1708','EPSG','1028');
INSERT INTO "projected_crs" VALUES('EPSG','31284','MGI / Austria M28',NULL,'EPSG','4530','EPSG','4312','EPSG','18044',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7051','projected_crs','EPSG','31284','EPSG','1706','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31285','MGI / Austria M31',NULL,'EPSG','4530','EPSG','4312','EPSG','18045',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7052','projected_crs','EPSG','31285','EPSG','1707','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31286','MGI / Austria M34',NULL,'EPSG','4530','EPSG','4312','EPSG','18046',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7053','projected_crs','EPSG','31286','EPSG','1708','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31287','MGI / Austria Lambert',NULL,'EPSG','4530','EPSG','4312','EPSG','19947',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7054','projected_crs','EPSG','31287','EPSG','1037','EPSG','1210');
INSERT INTO "projected_crs" VALUES('EPSG','31288','MGI (Ferro) / Austria zone M28',NULL,'EPSG','4530','EPSG','4805','EPSG','18047',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7055','projected_crs','EPSG','31288','EPSG','1706','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31289','MGI (Ferro) / Austria zone M31',NULL,'EPSG','4530','EPSG','4805','EPSG','18048',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7056','projected_crs','EPSG','31289','EPSG','1707','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31290','MGI (Ferro) / Austria zone M34',NULL,'EPSG','4530','EPSG','4805','EPSG','18049',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7057','projected_crs','EPSG','31290','EPSG','1708','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31291','MGI (Ferro) / Austria West Zone',NULL,'EPSG','4499','EPSG','4805','EPSG','18041',NULL,1);
INSERT INTO "usage" VALUES('EPSG','7058','projected_crs','EPSG','31291','EPSG','1706','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31292','MGI (Ferro) / Austria Central Zone',NULL,'EPSG','4499','EPSG','4805','EPSG','18042',NULL,1);
INSERT INTO "usage" VALUES('EPSG','7059','projected_crs','EPSG','31292','EPSG','1708','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31293','MGI (Ferro) / Austria East Zone',NULL,'EPSG','4499','EPSG','4805','EPSG','18043',NULL,1);
INSERT INTO "usage" VALUES('EPSG','7060','projected_crs','EPSG','31293','EPSG','1707','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31294','MGI / M28',NULL,'EPSG','4499','EPSG','4312','EPSG','18044',NULL,1);
INSERT INTO "usage" VALUES('EPSG','7061','projected_crs','EPSG','31294','EPSG','1706','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31295','MGI / M31',NULL,'EPSG','4499','EPSG','4312','EPSG','18045',NULL,1);
INSERT INTO "usage" VALUES('EPSG','7062','projected_crs','EPSG','31295','EPSG','1707','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31296','MGI / M34',NULL,'EPSG','4499','EPSG','4312','EPSG','18046',NULL,1);
INSERT INTO "usage" VALUES('EPSG','7063','projected_crs','EPSG','31296','EPSG','1708','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31297','MGI / Austria Lambert',NULL,'EPSG','4499','EPSG','4312','EPSG','19947',NULL,1);
INSERT INTO "usage" VALUES('EPSG','7064','projected_crs','EPSG','31297','EPSG','1037','EPSG','1210');
INSERT INTO "projected_crs" VALUES('EPSG','31300','BD72 / Belge Lambert 72',NULL,'EPSG','4499','EPSG','4313','EPSG','19902',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7065','projected_crs','EPSG','31300','EPSG','1347','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31370','BD72 / Belgian Lambert 72',NULL,'EPSG','4499','EPSG','4313','EPSG','19961',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7066','projected_crs','EPSG','31370','EPSG','1347','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31461','DHDN / 3-degree Gauss zone 1',NULL,'EPSG','4499','EPSG','4314','EPSG','16261',NULL,1);
INSERT INTO "usage" VALUES('EPSG','7067','projected_crs','EPSG','31461','EPSG','1628','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31462','DHDN / 3-degree Gauss zone 2',NULL,'EPSG','4499','EPSG','4314','EPSG','16262',NULL,1);
INSERT INTO "usage" VALUES('EPSG','7068','projected_crs','EPSG','31462','EPSG','1624','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31463','DHDN / 3-degree Gauss zone 3',NULL,'EPSG','4499','EPSG','4314','EPSG','16263',NULL,1);
INSERT INTO "usage" VALUES('EPSG','7069','projected_crs','EPSG','31463','EPSG','1625','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31464','DHDN / 3-degree Gauss zone 4',NULL,'EPSG','4499','EPSG','4314','EPSG','16264',NULL,1);
INSERT INTO "usage" VALUES('EPSG','7070','projected_crs','EPSG','31464','EPSG','1626','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31465','DHDN / 3-degree Gauss zone 5',NULL,'EPSG','4499','EPSG','4314','EPSG','16265',NULL,1);
INSERT INTO "usage" VALUES('EPSG','7071','projected_crs','EPSG','31465','EPSG','1627','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31466','DHDN / 3-degree Gauss-Kruger zone 2',NULL,'EPSG','4530','EPSG','4314','EPSG','16262',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7072','projected_crs','EPSG','31466','EPSG','1624','EPSG','1056');
INSERT INTO "projected_crs" VALUES('EPSG','31467','DHDN / 3-degree Gauss-Kruger zone 3',NULL,'EPSG','4530','EPSG','4314','EPSG','16263',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7073','projected_crs','EPSG','31467','EPSG','1625','EPSG','1056');
INSERT INTO "projected_crs" VALUES('EPSG','31468','DHDN / 3-degree Gauss-Kruger zone 4',NULL,'EPSG','4530','EPSG','4314','EPSG','16264',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7074','projected_crs','EPSG','31468','EPSG','1626','EPSG','1056');
INSERT INTO "projected_crs" VALUES('EPSG','31469','DHDN / 3-degree Gauss-Kruger zone 5',NULL,'EPSG','4530','EPSG','4314','EPSG','16265',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7075','projected_crs','EPSG','31469','EPSG','1627','EPSG','1056');
INSERT INTO "projected_crs" VALUES('EPSG','31528','Conakry 1905 / UTM zone 28N',NULL,'EPSG','4400','EPSG','4315','EPSG','16028',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7076','projected_crs','EPSG','31528','EPSG','1468','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31529','Conakry 1905 / UTM zone 29N',NULL,'EPSG','4400','EPSG','4315','EPSG','16029',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7077','projected_crs','EPSG','31529','EPSG','1469','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31600','Dealul Piscului 1930 / Stereo 33',NULL,'EPSG','4499','EPSG','4316','EPSG','19927',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7078','projected_crs','EPSG','31600','EPSG','3295','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31700','Dealul Piscului 1970/ Stereo 70',NULL,'EPSG','4530','EPSG','4317','EPSG','19926',NULL,1);
INSERT INTO "usage" VALUES('EPSG','7079','projected_crs','EPSG','31700','EPSG','1197','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31838','NGN / UTM zone 38N',NULL,'EPSG','4400','EPSG','4318','EPSG','16038',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7080','projected_crs','EPSG','31838','EPSG','1739','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31839','NGN / UTM zone 39N',NULL,'EPSG','4400','EPSG','4318','EPSG','16039',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7081','projected_crs','EPSG','31839','EPSG','1740','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31900','KUDAMS / KTM',NULL,'EPSG','4400','EPSG','4319','EPSG','19928',NULL,1);
INSERT INTO "usage" VALUES('EPSG','7082','projected_crs','EPSG','31900','EPSG','1310','EPSG','1208');
INSERT INTO "projected_crs" VALUES('EPSG','31901','KUDAMS / KTM',NULL,'EPSG','4400','EPSG','4319','EPSG','19997',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7083','projected_crs','EPSG','31901','EPSG','1310','EPSG','1208');
INSERT INTO "projected_crs" VALUES('EPSG','31965','SIRGAS 2000 / UTM zone 11N',NULL,'EPSG','4400','EPSG','4674','EPSG','16011',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7084','projected_crs','EPSG','31965','EPSG','3748','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31966','SIRGAS 2000 / UTM zone 12N',NULL,'EPSG','4400','EPSG','4674','EPSG','16012',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7085','projected_crs','EPSG','31966','EPSG','3756','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31967','SIRGAS 2000 / UTM zone 13N',NULL,'EPSG','4400','EPSG','4674','EPSG','16013',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7086','projected_crs','EPSG','31967','EPSG','3759','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31968','SIRGAS 2000 / UTM zone 14N',NULL,'EPSG','4400','EPSG','4674','EPSG','16014',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7087','projected_crs','EPSG','31968','EPSG','3763','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31969','SIRGAS 2000 / UTM zone 15N',NULL,'EPSG','4400','EPSG','4674','EPSG','16015',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7088','projected_crs','EPSG','31969','EPSG','3427','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31970','SIRGAS 2000 / UTM zone 16N',NULL,'EPSG','4400','EPSG','4674','EPSG','16016',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7089','projected_crs','EPSG','31970','EPSG','3428','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31971','SIRGAS 2000 / UTM zone 17N',NULL,'EPSG','4400','EPSG','4674','EPSG','16017',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7090','projected_crs','EPSG','31971','EPSG','3421','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31972','SIRGAS 2000 / UTM zone 18N',NULL,'EPSG','4400','EPSG','4674','EPSG','16018',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7091','projected_crs','EPSG','31972','EPSG','3422','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31973','SIRGAS 2000 / UTM zone 19N',NULL,'EPSG','4400','EPSG','4674','EPSG','16019',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7092','projected_crs','EPSG','31973','EPSG','3436','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31974','SIRGAS 2000 / UTM zone 20N',NULL,'EPSG','4400','EPSG','4674','EPSG','16020',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7093','projected_crs','EPSG','31974','EPSG','3437','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31975','SIRGAS 2000 / UTM zone 21N',NULL,'EPSG','4400','EPSG','4674','EPSG','16021',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7094','projected_crs','EPSG','31975','EPSG','3438','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31976','SIRGAS 2000 / UTM zone 22N',NULL,'EPSG','4400','EPSG','4674','EPSG','16022',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7095','projected_crs','EPSG','31976','EPSG','3439','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31977','SIRGAS 2000 / UTM zone 17S',NULL,'EPSG','4400','EPSG','4674','EPSG','16117',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7096','projected_crs','EPSG','31977','EPSG','1824','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31978','SIRGAS 2000 / UTM zone 18S',NULL,'EPSG','4400','EPSG','4674','EPSG','16118',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7097','projected_crs','EPSG','31978','EPSG','3440','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31979','SIRGAS 2000 / UTM zone 19S',NULL,'EPSG','4400','EPSG','4674','EPSG','16119',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7098','projected_crs','EPSG','31979','EPSG','3441','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31980','SIRGAS 2000 / UTM zone 20S',NULL,'EPSG','4400','EPSG','4674','EPSG','16120',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7099','projected_crs','EPSG','31980','EPSG','3442','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31981','SIRGAS 2000 / UTM zone 21S',NULL,'EPSG','4400','EPSG','4674','EPSG','16121',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7100','projected_crs','EPSG','31981','EPSG','3443','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31982','SIRGAS 2000 / UTM zone 22S',NULL,'EPSG','4400','EPSG','4674','EPSG','16122',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7101','projected_crs','EPSG','31982','EPSG','3444','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31983','SIRGAS 2000 / UTM zone 23S',NULL,'EPSG','4400','EPSG','4674','EPSG','16123',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7102','projected_crs','EPSG','31983','EPSG','3445','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31984','SIRGAS 2000 / UTM zone 24S',NULL,'EPSG','4400','EPSG','4674','EPSG','16124',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7103','projected_crs','EPSG','31984','EPSG','3446','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31985','SIRGAS 2000 / UTM zone 25S',NULL,'EPSG','4400','EPSG','4674','EPSG','16125',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7104','projected_crs','EPSG','31985','EPSG','3447','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31986','SIRGAS 1995 / UTM zone 17N',NULL,'EPSG','4400','EPSG','4170','EPSG','16017',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7105','projected_crs','EPSG','31986','EPSG','1823','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31987','SIRGAS 1995 / UTM zone 18N',NULL,'EPSG','4400','EPSG','4170','EPSG','16018',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7106','projected_crs','EPSG','31987','EPSG','1825','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31988','SIRGAS 1995 / UTM zone 19N',NULL,'EPSG','4400','EPSG','4170','EPSG','16019',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7107','projected_crs','EPSG','31988','EPSG','1827','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31989','SIRGAS 1995 / UTM zone 20N',NULL,'EPSG','4400','EPSG','4170','EPSG','16020',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7108','projected_crs','EPSG','31989','EPSG','1829','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31990','SIRGAS 1995 / UTM zone 21N',NULL,'EPSG','4400','EPSG','4170','EPSG','16021',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7109','projected_crs','EPSG','31990','EPSG','1831','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31991','SIRGAS 1995 / UTM zone 22N',NULL,'EPSG','4400','EPSG','4170','EPSG','16022',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7110','projected_crs','EPSG','31991','EPSG','1833','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31992','SIRGAS 1995 / UTM zone 17S',NULL,'EPSG','4400','EPSG','4170','EPSG','16117',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7111','projected_crs','EPSG','31992','EPSG','3638','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31993','SIRGAS 1995 / UTM zone 18S',NULL,'EPSG','4400','EPSG','4170','EPSG','16118',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7112','projected_crs','EPSG','31993','EPSG','1826','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31994','SIRGAS 1995 / UTM zone 19S',NULL,'EPSG','4400','EPSG','4170','EPSG','16119',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7113','projected_crs','EPSG','31994','EPSG','1828','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31995','SIRGAS 1995 / UTM zone 20S',NULL,'EPSG','4400','EPSG','4170','EPSG','16120',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7114','projected_crs','EPSG','31995','EPSG','1830','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31996','SIRGAS 1995 / UTM zone 21S',NULL,'EPSG','4400','EPSG','4170','EPSG','16121',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7115','projected_crs','EPSG','31996','EPSG','1832','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31997','SIRGAS 1995 / UTM zone 22S',NULL,'EPSG','4400','EPSG','4170','EPSG','16122',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7116','projected_crs','EPSG','31997','EPSG','1834','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31998','SIRGAS 1995 / UTM zone 23S',NULL,'EPSG','4400','EPSG','4170','EPSG','16123',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7117','projected_crs','EPSG','31998','EPSG','1835','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','31999','SIRGAS 1995 / UTM zone 24S',NULL,'EPSG','4400','EPSG','4170','EPSG','16124',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7118','projected_crs','EPSG','31999','EPSG','1836','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32000','SIRGAS 1995 / UTM zone 25S',NULL,'EPSG','4400','EPSG','4170','EPSG','16125',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7119','projected_crs','EPSG','32000','EPSG','1837','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32001','NAD27 / Montana North',NULL,'EPSG','4497','EPSG','4267','EPSG','12501',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7120','projected_crs','EPSG','32001','EPSG','2211','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32002','NAD27 / Montana Central',NULL,'EPSG','4497','EPSG','4267','EPSG','12502',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7121','projected_crs','EPSG','32002','EPSG','2210','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32003','NAD27 / Montana South',NULL,'EPSG','4497','EPSG','4267','EPSG','12503',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7122','projected_crs','EPSG','32003','EPSG','2212','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32005','NAD27 / Nebraska North',NULL,'EPSG','4497','EPSG','4267','EPSG','12601',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7123','projected_crs','EPSG','32005','EPSG','2221','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32006','NAD27 / Nebraska South',NULL,'EPSG','4497','EPSG','4267','EPSG','12602',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7124','projected_crs','EPSG','32006','EPSG','2222','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32007','NAD27 / Nevada East',NULL,'EPSG','4497','EPSG','4267','EPSG','12701',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7125','projected_crs','EPSG','32007','EPSG','2224','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32008','NAD27 / Nevada Central',NULL,'EPSG','4497','EPSG','4267','EPSG','12702',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7126','projected_crs','EPSG','32008','EPSG','2223','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32009','NAD27 / Nevada West',NULL,'EPSG','4497','EPSG','4267','EPSG','12703',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7127','projected_crs','EPSG','32009','EPSG','2225','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32010','NAD27 / New Hampshire',NULL,'EPSG','4497','EPSG','4267','EPSG','12800',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7128','projected_crs','EPSG','32010','EPSG','1398','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32011','NAD27 / New Jersey',NULL,'EPSG','4497','EPSG','4267','EPSG','12900',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7129','projected_crs','EPSG','32011','EPSG','1399','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32012','NAD27 / New Mexico East',NULL,'EPSG','4497','EPSG','4267','EPSG','13001',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7130','projected_crs','EPSG','32012','EPSG','2228','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32013','NAD27 / New Mexico Central',NULL,'EPSG','4497','EPSG','4267','EPSG','13002',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7131','projected_crs','EPSG','32013','EPSG','2229','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32014','NAD27 / New Mexico West',NULL,'EPSG','4497','EPSG','4267','EPSG','13003',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7132','projected_crs','EPSG','32014','EPSG','2230','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32015','NAD27 / New York East',NULL,'EPSG','4497','EPSG','4267','EPSG','13101',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7133','projected_crs','EPSG','32015','EPSG','2234','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32016','NAD27 / New York Central',NULL,'EPSG','4497','EPSG','4267','EPSG','13102',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7134','projected_crs','EPSG','32016','EPSG','2233','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32017','NAD27 / New York West',NULL,'EPSG','4497','EPSG','4267','EPSG','13103',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7135','projected_crs','EPSG','32017','EPSG','2236','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32018','NAD27 / New York Long Island',NULL,'EPSG','4497','EPSG','4267','EPSG','13104',NULL,1);
INSERT INTO "usage" VALUES('EPSG','7136','projected_crs','EPSG','32018','EPSG','2235','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32019','NAD27 / North Carolina',NULL,'EPSG','4497','EPSG','4267','EPSG','13200',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7137','projected_crs','EPSG','32019','EPSG','1402','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32020','NAD27 / North Dakota North',NULL,'EPSG','4497','EPSG','4267','EPSG','13301',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7138','projected_crs','EPSG','32020','EPSG','2237','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32021','NAD27 / North Dakota South',NULL,'EPSG','4497','EPSG','4267','EPSG','13302',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7139','projected_crs','EPSG','32021','EPSG','2238','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32022','NAD27 / Ohio North',NULL,'EPSG','4497','EPSG','4267','EPSG','13401',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7140','projected_crs','EPSG','32022','EPSG','2239','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32023','NAD27 / Ohio South',NULL,'EPSG','4497','EPSG','4267','EPSG','13402',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7141','projected_crs','EPSG','32023','EPSG','2240','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32024','NAD27 / Oklahoma North',NULL,'EPSG','4497','EPSG','4267','EPSG','13501',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7142','projected_crs','EPSG','32024','EPSG','2241','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32025','NAD27 / Oklahoma South',NULL,'EPSG','4497','EPSG','4267','EPSG','13502',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7143','projected_crs','EPSG','32025','EPSG','2242','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32026','NAD27 / Oregon North',NULL,'EPSG','4497','EPSG','4267','EPSG','13601',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7144','projected_crs','EPSG','32026','EPSG','2243','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32027','NAD27 / Oregon South',NULL,'EPSG','4497','EPSG','4267','EPSG','13602',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7145','projected_crs','EPSG','32027','EPSG','2244','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32028','NAD27 / Pennsylvania North',NULL,'EPSG','4497','EPSG','4267','EPSG','13701',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7146','projected_crs','EPSG','32028','EPSG','2245','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32029','NAD27 / Pennsylvania South',NULL,'EPSG','4497','EPSG','4267','EPSG','13702',NULL,1);
INSERT INTO "usage" VALUES('EPSG','7147','projected_crs','EPSG','32029','EPSG','2246','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32030','NAD27 / Rhode Island',NULL,'EPSG','4497','EPSG','4267','EPSG','13800',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7148','projected_crs','EPSG','32030','EPSG','1408','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32031','NAD27 / South Carolina North',NULL,'EPSG','4497','EPSG','4267','EPSG','13901',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7149','projected_crs','EPSG','32031','EPSG','2247','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32033','NAD27 / South Carolina South',NULL,'EPSG','4497','EPSG','4267','EPSG','13902',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7150','projected_crs','EPSG','32033','EPSG','2248','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32034','NAD27 / South Dakota North',NULL,'EPSG','4497','EPSG','4267','EPSG','14001',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7151','projected_crs','EPSG','32034','EPSG','2249','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32035','NAD27 / South Dakota South',NULL,'EPSG','4497','EPSG','4267','EPSG','14002',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7152','projected_crs','EPSG','32035','EPSG','2250','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32036','NAD27 / Tennessee',NULL,'EPSG','4497','EPSG','4267','EPSG','14100',NULL,1);
INSERT INTO "usage" VALUES('EPSG','7153','projected_crs','EPSG','32036','EPSG','1411','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32037','NAD27 / Texas North',NULL,'EPSG','4497','EPSG','4267','EPSG','14201',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7154','projected_crs','EPSG','32037','EPSG','2253','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32038','NAD27 / Texas North Central',NULL,'EPSG','4497','EPSG','4267','EPSG','14202',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7155','projected_crs','EPSG','32038','EPSG','2254','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32039','NAD27 / Texas Central',NULL,'EPSG','4497','EPSG','4267','EPSG','14203',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7156','projected_crs','EPSG','32039','EPSG','2252','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32040','NAD27 / Texas South Central',NULL,'EPSG','4497','EPSG','4267','EPSG','14204',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7157','projected_crs','EPSG','32040','EPSG','2256','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32041','NAD27 / Texas South',NULL,'EPSG','4497','EPSG','4267','EPSG','14205',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7158','projected_crs','EPSG','32041','EPSG','2255','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32042','NAD27 / Utah North',NULL,'EPSG','4497','EPSG','4267','EPSG','14301',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7159','projected_crs','EPSG','32042','EPSG','2258','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32043','NAD27 / Utah Central',NULL,'EPSG','4497','EPSG','4267','EPSG','14302',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7160','projected_crs','EPSG','32043','EPSG','2257','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32044','NAD27 / Utah South',NULL,'EPSG','4497','EPSG','4267','EPSG','14303',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7161','projected_crs','EPSG','32044','EPSG','2259','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32045','NAD27 / Vermont',NULL,'EPSG','4497','EPSG','4267','EPSG','14400',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7162','projected_crs','EPSG','32045','EPSG','1414','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32046','NAD27 / Virginia North',NULL,'EPSG','4497','EPSG','4267','EPSG','14501',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7163','projected_crs','EPSG','32046','EPSG','2260','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32047','NAD27 / Virginia South',NULL,'EPSG','4497','EPSG','4267','EPSG','14502',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7164','projected_crs','EPSG','32047','EPSG','2261','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32048','NAD27 / Washington North',NULL,'EPSG','4497','EPSG','4267','EPSG','14601',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7165','projected_crs','EPSG','32048','EPSG','2262','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32049','NAD27 / Washington South',NULL,'EPSG','4497','EPSG','4267','EPSG','14602',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7166','projected_crs','EPSG','32049','EPSG','2263','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32050','NAD27 / West Virginia North',NULL,'EPSG','4497','EPSG','4267','EPSG','14701',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7167','projected_crs','EPSG','32050','EPSG','2264','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32051','NAD27 / West Virginia South',NULL,'EPSG','4497','EPSG','4267','EPSG','14702',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7168','projected_crs','EPSG','32051','EPSG','2265','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32052','NAD27 / Wisconsin North',NULL,'EPSG','4497','EPSG','4267','EPSG','14801',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7169','projected_crs','EPSG','32052','EPSG','2267','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32053','NAD27 / Wisconsin Central',NULL,'EPSG','4497','EPSG','4267','EPSG','14802',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7170','projected_crs','EPSG','32053','EPSG','2266','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32054','NAD27 / Wisconsin South',NULL,'EPSG','4497','EPSG','4267','EPSG','14803',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7171','projected_crs','EPSG','32054','EPSG','2268','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32055','NAD27 / Wyoming East',NULL,'EPSG','4497','EPSG','4267','EPSG','14901',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7172','projected_crs','EPSG','32055','EPSG','2269','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32056','NAD27 / Wyoming East Central',NULL,'EPSG','4497','EPSG','4267','EPSG','14902',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7173','projected_crs','EPSG','32056','EPSG','2270','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32057','NAD27 / Wyoming West Central',NULL,'EPSG','4497','EPSG','4267','EPSG','14903',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7174','projected_crs','EPSG','32057','EPSG','2272','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32058','NAD27 / Wyoming West',NULL,'EPSG','4497','EPSG','4267','EPSG','14904',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7175','projected_crs','EPSG','32058','EPSG','2271','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32061','NAD27 / Guatemala Norte',NULL,'EPSG','4499','EPSG','4267','EPSG','18211',NULL,1);
INSERT INTO "usage" VALUES('EPSG','7176','projected_crs','EPSG','32061','EPSG','2120','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32062','NAD27 / Guatemala Sur',NULL,'EPSG','4499','EPSG','4267','EPSG','18212',NULL,1);
INSERT INTO "usage" VALUES('EPSG','7177','projected_crs','EPSG','32062','EPSG','2121','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32064','NAD27 / BLM 14N (ftUS)',NULL,'EPSG','4497','EPSG','4267','EPSG','15914',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7178','projected_crs','EPSG','32064','EPSG','3637','EPSG','1249');
INSERT INTO "projected_crs" VALUES('EPSG','32065','NAD27 / BLM 15N (ftUS)',NULL,'EPSG','4497','EPSG','4267','EPSG','15915',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7179','projected_crs','EPSG','32065','EPSG','3640','EPSG','1249');
INSERT INTO "projected_crs" VALUES('EPSG','32066','NAD27 / BLM 16N (ftUS)',NULL,'EPSG','4497','EPSG','4267','EPSG','15916',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7180','projected_crs','EPSG','32066','EPSG','3641','EPSG','1249');
INSERT INTO "projected_crs" VALUES('EPSG','32067','NAD27 / BLM 17N (ftUS)',NULL,'EPSG','4497','EPSG','4267','EPSG','15917',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7181','projected_crs','EPSG','32067','EPSG','3642','EPSG','1249');
INSERT INTO "projected_crs" VALUES('EPSG','32074','NAD27 / BLM 14N (feet)',NULL,'EPSG','4497','EPSG','4267','EPSG','15914',NULL,1);
INSERT INTO "usage" VALUES('EPSG','7182','projected_crs','EPSG','32074','EPSG','2171','EPSG','1249');
INSERT INTO "projected_crs" VALUES('EPSG','32075','NAD27 / BLM 15N (feet)',NULL,'EPSG','4497','EPSG','4267','EPSG','15915',NULL,1);
INSERT INTO "usage" VALUES('EPSG','7183','projected_crs','EPSG','32075','EPSG','2172','EPSG','1249');
INSERT INTO "projected_crs" VALUES('EPSG','32076','NAD27 / BLM 16N (feet)',NULL,'EPSG','4497','EPSG','4267','EPSG','15916',NULL,1);
INSERT INTO "usage" VALUES('EPSG','7184','projected_crs','EPSG','32076','EPSG','2173','EPSG','1249');
INSERT INTO "projected_crs" VALUES('EPSG','32077','NAD27 / BLM 17N (feet)',NULL,'EPSG','4497','EPSG','4267','EPSG','15917',NULL,1);
INSERT INTO "usage" VALUES('EPSG','7185','projected_crs','EPSG','32077','EPSG','2174','EPSG','1249');
INSERT INTO "projected_crs" VALUES('EPSG','32081','NAD27 / MTM zone 1',NULL,'EPSG','4400','EPSG','4267','EPSG','17701',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7186','projected_crs','EPSG','32081','EPSG','2226','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32082','NAD27 / MTM zone 2',NULL,'EPSG','4400','EPSG','4267','EPSG','17702',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7187','projected_crs','EPSG','32082','EPSG','2227','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32083','NAD27 / MTM zone 3',NULL,'EPSG','4400','EPSG','4267','EPSG','17703',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7188','projected_crs','EPSG','32083','EPSG','2275','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32084','NAD27 / MTM zone 4',NULL,'EPSG','4400','EPSG','4267','EPSG','17704',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7189','projected_crs','EPSG','32084','EPSG','3875','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32085','NAD27 / MTM zone 5',NULL,'EPSG','4400','EPSG','4267','EPSG','17705',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7190','projected_crs','EPSG','32085','EPSG','3865','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32086','NAD27 / MTM zone 6',NULL,'EPSG','4400','EPSG','4267','EPSG','17706',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7191','projected_crs','EPSG','32086','EPSG','3880','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32098','NAD27 / Quebec Lambert',NULL,'EPSG','4499','EPSG','4267','EPSG','19944',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7192','projected_crs','EPSG','32098','EPSG','1368','EPSG','1210');
INSERT INTO "projected_crs" VALUES('EPSG','32099','NAD27 / Louisiana Offshore',NULL,'EPSG','4497','EPSG','4267','EPSG','11703',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7193','projected_crs','EPSG','32099','EPSG','4821','EPSG','1212');
INSERT INTO "projected_crs" VALUES('EPSG','32100','NAD83 / Montana',NULL,'EPSG','4499','EPSG','4269','EPSG','12530',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7194','projected_crs','EPSG','32100','EPSG','1395','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32104','NAD83 / Nebraska',NULL,'EPSG','4499','EPSG','4269','EPSG','12630',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7195','projected_crs','EPSG','32104','EPSG','1396','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32107','NAD83 / Nevada East',NULL,'EPSG','4499','EPSG','4269','EPSG','12731',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7196','projected_crs','EPSG','32107','EPSG','2224','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32108','NAD83 / Nevada Central',NULL,'EPSG','4499','EPSG','4269','EPSG','12732',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7197','projected_crs','EPSG','32108','EPSG','2223','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32109','NAD83 / Nevada West',NULL,'EPSG','4499','EPSG','4269','EPSG','12733',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7198','projected_crs','EPSG','32109','EPSG','2225','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32110','NAD83 / New Hampshire',NULL,'EPSG','4499','EPSG','4269','EPSG','12830',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7199','projected_crs','EPSG','32110','EPSG','1398','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32111','NAD83 / New Jersey',NULL,'EPSG','4499','EPSG','4269','EPSG','12930',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7200','projected_crs','EPSG','32111','EPSG','1399','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32112','NAD83 / New Mexico East',NULL,'EPSG','4499','EPSG','4269','EPSG','13031',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7201','projected_crs','EPSG','32112','EPSG','2228','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32113','NAD83 / New Mexico Central',NULL,'EPSG','4499','EPSG','4269','EPSG','13032',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7202','projected_crs','EPSG','32113','EPSG','2231','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32114','NAD83 / New Mexico West',NULL,'EPSG','4499','EPSG','4269','EPSG','13033',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7203','projected_crs','EPSG','32114','EPSG','2232','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32115','NAD83 / New York East',NULL,'EPSG','4499','EPSG','4269','EPSG','13131',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7204','projected_crs','EPSG','32115','EPSG','2234','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32116','NAD83 / New York Central',NULL,'EPSG','4499','EPSG','4269','EPSG','13132',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7205','projected_crs','EPSG','32116','EPSG','2233','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32117','NAD83 / New York West',NULL,'EPSG','4499','EPSG','4269','EPSG','13133',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7206','projected_crs','EPSG','32117','EPSG','2236','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32118','NAD83 / New York Long Island',NULL,'EPSG','4499','EPSG','4269','EPSG','13134',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7207','projected_crs','EPSG','32118','EPSG','2235','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32119','NAD83 / North Carolina',NULL,'EPSG','4499','EPSG','4269','EPSG','13230',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7208','projected_crs','EPSG','32119','EPSG','1402','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32120','NAD83 / North Dakota North',NULL,'EPSG','4499','EPSG','4269','EPSG','13331',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7209','projected_crs','EPSG','32120','EPSG','2237','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32121','NAD83 / North Dakota South',NULL,'EPSG','4499','EPSG','4269','EPSG','13332',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7210','projected_crs','EPSG','32121','EPSG','2238','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32122','NAD83 / Ohio North',NULL,'EPSG','4499','EPSG','4269','EPSG','13431',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7211','projected_crs','EPSG','32122','EPSG','2239','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32123','NAD83 / Ohio South',NULL,'EPSG','4499','EPSG','4269','EPSG','13432',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7212','projected_crs','EPSG','32123','EPSG','2240','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32124','NAD83 / Oklahoma North',NULL,'EPSG','4499','EPSG','4269','EPSG','13531',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7213','projected_crs','EPSG','32124','EPSG','2241','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32125','NAD83 / Oklahoma South',NULL,'EPSG','4499','EPSG','4269','EPSG','13532',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7214','projected_crs','EPSG','32125','EPSG','2242','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32126','NAD83 / Oregon North',NULL,'EPSG','4499','EPSG','4269','EPSG','13631',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7215','projected_crs','EPSG','32126','EPSG','2243','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32127','NAD83 / Oregon South',NULL,'EPSG','4499','EPSG','4269','EPSG','13632',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7216','projected_crs','EPSG','32127','EPSG','2244','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32128','NAD83 / Pennsylvania North',NULL,'EPSG','4499','EPSG','4269','EPSG','13731',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7217','projected_crs','EPSG','32128','EPSG','2245','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32129','NAD83 / Pennsylvania South',NULL,'EPSG','4499','EPSG','4269','EPSG','13732',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7218','projected_crs','EPSG','32129','EPSG','2246','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32130','NAD83 / Rhode Island',NULL,'EPSG','4499','EPSG','4269','EPSG','13830',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7219','projected_crs','EPSG','32130','EPSG','1408','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32133','NAD83 / South Carolina',NULL,'EPSG','4499','EPSG','4269','EPSG','13930',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7220','projected_crs','EPSG','32133','EPSG','1409','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32134','NAD83 / South Dakota North',NULL,'EPSG','4499','EPSG','4269','EPSG','14031',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7221','projected_crs','EPSG','32134','EPSG','2249','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32135','NAD83 / South Dakota South',NULL,'EPSG','4499','EPSG','4269','EPSG','14032',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7222','projected_crs','EPSG','32135','EPSG','2250','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32136','NAD83 / Tennessee',NULL,'EPSG','4499','EPSG','4269','EPSG','14130',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7223','projected_crs','EPSG','32136','EPSG','1411','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32137','NAD83 / Texas North',NULL,'EPSG','4499','EPSG','4269','EPSG','14231',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7224','projected_crs','EPSG','32137','EPSG','2253','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32138','NAD83 / Texas North Central',NULL,'EPSG','4499','EPSG','4269','EPSG','14232',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7225','projected_crs','EPSG','32138','EPSG','2254','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32139','NAD83 / Texas Central',NULL,'EPSG','4499','EPSG','4269','EPSG','14233',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7226','projected_crs','EPSG','32139','EPSG','2252','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32140','NAD83 / Texas South Central',NULL,'EPSG','4499','EPSG','4269','EPSG','14234',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7227','projected_crs','EPSG','32140','EPSG','2527','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32141','NAD83 / Texas South',NULL,'EPSG','4499','EPSG','4269','EPSG','14235',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7228','projected_crs','EPSG','32141','EPSG','2528','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32142','NAD83 / Utah North',NULL,'EPSG','4499','EPSG','4269','EPSG','14331',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7229','projected_crs','EPSG','32142','EPSG','2258','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32143','NAD83 / Utah Central',NULL,'EPSG','4499','EPSG','4269','EPSG','14332',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7230','projected_crs','EPSG','32143','EPSG','2257','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32144','NAD83 / Utah South',NULL,'EPSG','4499','EPSG','4269','EPSG','14333',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7231','projected_crs','EPSG','32144','EPSG','2259','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32145','NAD83 / Vermont',NULL,'EPSG','4499','EPSG','4269','EPSG','14430',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7232','projected_crs','EPSG','32145','EPSG','1414','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32146','NAD83 / Virginia North',NULL,'EPSG','4499','EPSG','4269','EPSG','14531',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7233','projected_crs','EPSG','32146','EPSG','2260','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32147','NAD83 / Virginia South',NULL,'EPSG','4499','EPSG','4269','EPSG','14532',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7234','projected_crs','EPSG','32147','EPSG','2261','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32148','NAD83 / Washington North',NULL,'EPSG','4499','EPSG','4269','EPSG','14631',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7235','projected_crs','EPSG','32148','EPSG','2273','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32149','NAD83 / Washington South',NULL,'EPSG','4499','EPSG','4269','EPSG','14632',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7236','projected_crs','EPSG','32149','EPSG','2274','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32150','NAD83 / West Virginia North',NULL,'EPSG','4499','EPSG','4269','EPSG','14731',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7237','projected_crs','EPSG','32150','EPSG','2264','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32151','NAD83 / West Virginia South',NULL,'EPSG','4499','EPSG','4269','EPSG','14732',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7238','projected_crs','EPSG','32151','EPSG','2265','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32152','NAD83 / Wisconsin North',NULL,'EPSG','4499','EPSG','4269','EPSG','14831',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7239','projected_crs','EPSG','32152','EPSG','2267','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32153','NAD83 / Wisconsin Central',NULL,'EPSG','4499','EPSG','4269','EPSG','14832',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7240','projected_crs','EPSG','32153','EPSG','2266','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32154','NAD83 / Wisconsin South',NULL,'EPSG','4499','EPSG','4269','EPSG','14833',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7241','projected_crs','EPSG','32154','EPSG','2268','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32155','NAD83 / Wyoming East',NULL,'EPSG','4499','EPSG','4269','EPSG','14931',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7242','projected_crs','EPSG','32155','EPSG','2269','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32156','NAD83 / Wyoming East Central',NULL,'EPSG','4499','EPSG','4269','EPSG','14932',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7243','projected_crs','EPSG','32156','EPSG','2270','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32157','NAD83 / Wyoming West Central',NULL,'EPSG','4499','EPSG','4269','EPSG','14933',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7244','projected_crs','EPSG','32157','EPSG','2272','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32158','NAD83 / Wyoming West',NULL,'EPSG','4499','EPSG','4269','EPSG','14934',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7245','projected_crs','EPSG','32158','EPSG','2271','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32159','NAD83 / Wyoming Lambert',NULL,'EPSG','4499','EPSG','4269','EPSG','14930',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17409','projected_crs','EPSG','32159','EPSG','1419','EPSG','1135');
INSERT INTO "projected_crs" VALUES('EPSG','32161','NAD83 / Puerto Rico & Virgin Is.',NULL,'EPSG','4499','EPSG','4269','EPSG','15230',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7246','projected_crs','EPSG','32161','EPSG','2251','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32164','NAD83 / BLM 14N (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15914',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7247','projected_crs','EPSG','32164','EPSG','3637','EPSG','1212');
INSERT INTO "projected_crs" VALUES('EPSG','32165','NAD83 / BLM 15N (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15915',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7248','projected_crs','EPSG','32165','EPSG','3640','EPSG','1212');
INSERT INTO "projected_crs" VALUES('EPSG','32166','NAD83 / BLM 16N (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15916',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7249','projected_crs','EPSG','32166','EPSG','3641','EPSG','1212');
INSERT INTO "projected_crs" VALUES('EPSG','32167','NAD83 / BLM 17N (ftUS)',NULL,'EPSG','4497','EPSG','4269','EPSG','15917',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7250','projected_crs','EPSG','32167','EPSG','3642','EPSG','1212');
INSERT INTO "projected_crs" VALUES('EPSG','32180','NAD83 / SCoPQ zone 2',NULL,'EPSG','4499','EPSG','4269','EPSG','17700',NULL,1);
INSERT INTO "usage" VALUES('EPSG','7251','projected_crs','EPSG','32180','EPSG','1420','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32181','NAD83 / MTM zone 1',NULL,'EPSG','4496','EPSG','4269','EPSG','17701',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7252','projected_crs','EPSG','32181','EPSG','2226','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32182','NAD83 / MTM zone 2',NULL,'EPSG','4496','EPSG','4269','EPSG','17702',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7253','projected_crs','EPSG','32182','EPSG','2227','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32183','NAD83 / MTM zone 3',NULL,'EPSG','4496','EPSG','4269','EPSG','17703',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7254','projected_crs','EPSG','32183','EPSG','2290','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32184','NAD83 / MTM zone 4',NULL,'EPSG','4496','EPSG','4269','EPSG','17704',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7255','projected_crs','EPSG','32184','EPSG','2276','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32185','NAD83 / MTM zone 5',NULL,'EPSG','4496','EPSG','4269','EPSG','17705',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7256','projected_crs','EPSG','32185','EPSG','2277','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32186','NAD83 / MTM zone 6',NULL,'EPSG','4496','EPSG','4269','EPSG','17706',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7257','projected_crs','EPSG','32186','EPSG','2278','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32187','NAD83 / MTM zone 7',NULL,'EPSG','4496','EPSG','4269','EPSG','17707',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7258','projected_crs','EPSG','32187','EPSG','1425','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32188','NAD83 / MTM zone 8',NULL,'EPSG','4496','EPSG','4269','EPSG','17708',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7259','projected_crs','EPSG','32188','EPSG','2279','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32189','NAD83 / MTM zone 9',NULL,'EPSG','4496','EPSG','4269','EPSG','17709',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7260','projected_crs','EPSG','32189','EPSG','2280','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32190','NAD83 / MTM zone 10',NULL,'EPSG','4496','EPSG','4269','EPSG','17710',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7261','projected_crs','EPSG','32190','EPSG','2281','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32191','NAD83 / MTM zone 11',NULL,'EPSG','4400','EPSG','4269','EPSG','17711',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7262','projected_crs','EPSG','32191','EPSG','1432','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32192','NAD83 / MTM zone 12',NULL,'EPSG','4400','EPSG','4269','EPSG','17712',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7263','projected_crs','EPSG','32192','EPSG','1433','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32193','NAD83 / MTM zone 13',NULL,'EPSG','4400','EPSG','4269','EPSG','17713',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7264','projected_crs','EPSG','32193','EPSG','1434','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32194','NAD83 / MTM zone 14',NULL,'EPSG','4400','EPSG','4269','EPSG','17714',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7265','projected_crs','EPSG','32194','EPSG','1435','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32195','NAD83 / MTM zone 15',NULL,'EPSG','4400','EPSG','4269','EPSG','17715',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7266','projected_crs','EPSG','32195','EPSG','1436','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32196','NAD83 / MTM zone 16',NULL,'EPSG','4400','EPSG','4269','EPSG','17716',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7267','projected_crs','EPSG','32196','EPSG','1437','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32197','NAD83 / MTM zone 17',NULL,'EPSG','4400','EPSG','4269','EPSG','17717',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7268','projected_crs','EPSG','32197','EPSG','1438','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32198','NAD83 / Quebec Lambert',NULL,'EPSG','4499','EPSG','4269','EPSG','19944',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7269','projected_crs','EPSG','32198','EPSG','1368','EPSG','1210');
INSERT INTO "projected_crs" VALUES('EPSG','32199','NAD83 / Louisiana Offshore',NULL,'EPSG','4499','EPSG','4269','EPSG','11733',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7270','projected_crs','EPSG','32199','EPSG','4821','EPSG','1212');
INSERT INTO "projected_crs" VALUES('EPSG','32201','WGS 72 / UTM zone 1N',NULL,'EPSG','4400','EPSG','4322','EPSG','16001',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7271','projected_crs','EPSG','32201','EPSG','1873','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32202','WGS 72 / UTM zone 2N',NULL,'EPSG','4400','EPSG','4322','EPSG','16002',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7272','projected_crs','EPSG','32202','EPSG','1875','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32203','WGS 72 / UTM zone 3N',NULL,'EPSG','4400','EPSG','4322','EPSG','16003',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7273','projected_crs','EPSG','32203','EPSG','1877','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32204','WGS 72 / UTM zone 4N',NULL,'EPSG','4400','EPSG','4322','EPSG','16004',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7274','projected_crs','EPSG','32204','EPSG','1879','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32205','WGS 72 / UTM zone 5N',NULL,'EPSG','4400','EPSG','4322','EPSG','16005',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7275','projected_crs','EPSG','32205','EPSG','1881','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32206','WGS 72 / UTM zone 6N',NULL,'EPSG','4400','EPSG','4322','EPSG','16006',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7276','projected_crs','EPSG','32206','EPSG','1883','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32207','WGS 72 / UTM zone 7N',NULL,'EPSG','4400','EPSG','4322','EPSG','16007',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7277','projected_crs','EPSG','32207','EPSG','1885','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32208','WGS 72 / UTM zone 8N',NULL,'EPSG','4400','EPSG','4322','EPSG','16008',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7278','projected_crs','EPSG','32208','EPSG','1887','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32209','WGS 72 / UTM zone 9N',NULL,'EPSG','4400','EPSG','4322','EPSG','16009',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7279','projected_crs','EPSG','32209','EPSG','1889','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32210','WGS 72 / UTM zone 10N',NULL,'EPSG','4400','EPSG','4322','EPSG','16010',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7280','projected_crs','EPSG','32210','EPSG','1891','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32211','WGS 72 / UTM zone 11N',NULL,'EPSG','4400','EPSG','4322','EPSG','16011',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7281','projected_crs','EPSG','32211','EPSG','1893','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32212','WGS 72 / UTM zone 12N',NULL,'EPSG','4400','EPSG','4322','EPSG','16012',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7282','projected_crs','EPSG','32212','EPSG','1895','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32213','WGS 72 / UTM zone 13N',NULL,'EPSG','4400','EPSG','4322','EPSG','16013',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7283','projected_crs','EPSG','32213','EPSG','1897','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32214','WGS 72 / UTM zone 14N',NULL,'EPSG','4400','EPSG','4322','EPSG','16014',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7284','projected_crs','EPSG','32214','EPSG','1899','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32215','WGS 72 / UTM zone 15N',NULL,'EPSG','4400','EPSG','4322','EPSG','16015',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7285','projected_crs','EPSG','32215','EPSG','1901','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32216','WGS 72 / UTM zone 16N',NULL,'EPSG','4400','EPSG','4322','EPSG','16016',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7286','projected_crs','EPSG','32216','EPSG','1903','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32217','WGS 72 / UTM zone 17N',NULL,'EPSG','4400','EPSG','4322','EPSG','16017',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7287','projected_crs','EPSG','32217','EPSG','1905','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32218','WGS 72 / UTM zone 18N',NULL,'EPSG','4400','EPSG','4322','EPSG','16018',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7288','projected_crs','EPSG','32218','EPSG','1907','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32219','WGS 72 / UTM zone 19N',NULL,'EPSG','4400','EPSG','4322','EPSG','16019',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7289','projected_crs','EPSG','32219','EPSG','1909','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32220','WGS 72 / UTM zone 20N',NULL,'EPSG','4400','EPSG','4322','EPSG','16020',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7290','projected_crs','EPSG','32220','EPSG','1911','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32221','WGS 72 / UTM zone 21N',NULL,'EPSG','4400','EPSG','4322','EPSG','16021',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7291','projected_crs','EPSG','32221','EPSG','1913','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32222','WGS 72 / UTM zone 22N',NULL,'EPSG','4400','EPSG','4322','EPSG','16022',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7292','projected_crs','EPSG','32222','EPSG','1915','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32223','WGS 72 / UTM zone 23N',NULL,'EPSG','4400','EPSG','4322','EPSG','16023',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7293','projected_crs','EPSG','32223','EPSG','1917','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32224','WGS 72 / UTM zone 24N',NULL,'EPSG','4400','EPSG','4322','EPSG','16024',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7294','projected_crs','EPSG','32224','EPSG','1919','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32225','WGS 72 / UTM zone 25N',NULL,'EPSG','4400','EPSG','4322','EPSG','16025',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7295','projected_crs','EPSG','32225','EPSG','1921','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32226','WGS 72 / UTM zone 26N',NULL,'EPSG','4400','EPSG','4322','EPSG','16026',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7296','projected_crs','EPSG','32226','EPSG','1923','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32227','WGS 72 / UTM zone 27N',NULL,'EPSG','4400','EPSG','4322','EPSG','16027',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7297','projected_crs','EPSG','32227','EPSG','1925','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32228','WGS 72 / UTM zone 28N',NULL,'EPSG','4400','EPSG','4322','EPSG','16028',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7298','projected_crs','EPSG','32228','EPSG','1927','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32229','WGS 72 / UTM zone 29N',NULL,'EPSG','4400','EPSG','4322','EPSG','16029',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7299','projected_crs','EPSG','32229','EPSG','1929','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32230','WGS 72 / UTM zone 30N',NULL,'EPSG','4400','EPSG','4322','EPSG','16030',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7300','projected_crs','EPSG','32230','EPSG','1931','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32231','WGS 72 / UTM zone 31N',NULL,'EPSG','4400','EPSG','4322','EPSG','16031',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7301','projected_crs','EPSG','32231','EPSG','1933','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32232','WGS 72 / UTM zone 32N',NULL,'EPSG','4400','EPSG','4322','EPSG','16032',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7302','projected_crs','EPSG','32232','EPSG','1935','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32233','WGS 72 / UTM zone 33N',NULL,'EPSG','4400','EPSG','4322','EPSG','16033',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7303','projected_crs','EPSG','32233','EPSG','1937','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32234','WGS 72 / UTM zone 34N',NULL,'EPSG','4400','EPSG','4322','EPSG','16034',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7304','projected_crs','EPSG','32234','EPSG','1939','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32235','WGS 72 / UTM zone 35N',NULL,'EPSG','4400','EPSG','4322','EPSG','16035',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7305','projected_crs','EPSG','32235','EPSG','1941','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32236','WGS 72 / UTM zone 36N',NULL,'EPSG','4400','EPSG','4322','EPSG','16036',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7306','projected_crs','EPSG','32236','EPSG','1943','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32237','WGS 72 / UTM zone 37N',NULL,'EPSG','4400','EPSG','4322','EPSG','16037',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7307','projected_crs','EPSG','32237','EPSG','1945','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32238','WGS 72 / UTM zone 38N',NULL,'EPSG','4400','EPSG','4322','EPSG','16038',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7308','projected_crs','EPSG','32238','EPSG','1947','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32239','WGS 72 / UTM zone 39N',NULL,'EPSG','4400','EPSG','4322','EPSG','16039',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7309','projected_crs','EPSG','32239','EPSG','1949','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32240','WGS 72 / UTM zone 40N',NULL,'EPSG','4400','EPSG','4322','EPSG','16040',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7310','projected_crs','EPSG','32240','EPSG','1951','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32241','WGS 72 / UTM zone 41N',NULL,'EPSG','4400','EPSG','4322','EPSG','16041',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7311','projected_crs','EPSG','32241','EPSG','1953','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32242','WGS 72 / UTM zone 42N',NULL,'EPSG','4400','EPSG','4322','EPSG','16042',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7312','projected_crs','EPSG','32242','EPSG','1955','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32243','WGS 72 / UTM zone 43N',NULL,'EPSG','4400','EPSG','4322','EPSG','16043',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7313','projected_crs','EPSG','32243','EPSG','1957','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32244','WGS 72 / UTM zone 44N',NULL,'EPSG','4400','EPSG','4322','EPSG','16044',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7314','projected_crs','EPSG','32244','EPSG','1959','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32245','WGS 72 / UTM zone 45N',NULL,'EPSG','4400','EPSG','4322','EPSG','16045',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7315','projected_crs','EPSG','32245','EPSG','1961','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32246','WGS 72 / UTM zone 46N',NULL,'EPSG','4400','EPSG','4322','EPSG','16046',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7316','projected_crs','EPSG','32246','EPSG','1963','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32247','WGS 72 / UTM zone 47N',NULL,'EPSG','4400','EPSG','4322','EPSG','16047',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7317','projected_crs','EPSG','32247','EPSG','1965','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32248','WGS 72 / UTM zone 48N',NULL,'EPSG','4400','EPSG','4322','EPSG','16048',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7318','projected_crs','EPSG','32248','EPSG','1967','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32249','WGS 72 / UTM zone 49N',NULL,'EPSG','4400','EPSG','4322','EPSG','16049',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7319','projected_crs','EPSG','32249','EPSG','1969','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32250','WGS 72 / UTM zone 50N',NULL,'EPSG','4400','EPSG','4322','EPSG','16050',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7320','projected_crs','EPSG','32250','EPSG','1971','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32251','WGS 72 / UTM zone 51N',NULL,'EPSG','4400','EPSG','4322','EPSG','16051',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7321','projected_crs','EPSG','32251','EPSG','1973','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32252','WGS 72 / UTM zone 52N',NULL,'EPSG','4400','EPSG','4322','EPSG','16052',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7322','projected_crs','EPSG','32252','EPSG','1975','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32253','WGS 72 / UTM zone 53N',NULL,'EPSG','4400','EPSG','4322','EPSG','16053',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7323','projected_crs','EPSG','32253','EPSG','1977','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32254','WGS 72 / UTM zone 54N',NULL,'EPSG','4400','EPSG','4322','EPSG','16054',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7324','projected_crs','EPSG','32254','EPSG','1979','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32255','WGS 72 / UTM zone 55N',NULL,'EPSG','4400','EPSG','4322','EPSG','16055',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7325','projected_crs','EPSG','32255','EPSG','1981','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32256','WGS 72 / UTM zone 56N',NULL,'EPSG','4400','EPSG','4322','EPSG','16056',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7326','projected_crs','EPSG','32256','EPSG','1983','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32257','WGS 72 / UTM zone 57N',NULL,'EPSG','4400','EPSG','4322','EPSG','16057',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7327','projected_crs','EPSG','32257','EPSG','1985','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32258','WGS 72 / UTM zone 58N',NULL,'EPSG','4400','EPSG','4322','EPSG','16058',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7328','projected_crs','EPSG','32258','EPSG','1987','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32259','WGS 72 / UTM zone 59N',NULL,'EPSG','4400','EPSG','4322','EPSG','16059',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7329','projected_crs','EPSG','32259','EPSG','1989','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32260','WGS 72 / UTM zone 60N',NULL,'EPSG','4400','EPSG','4322','EPSG','16060',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7330','projected_crs','EPSG','32260','EPSG','1991','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32301','WGS 72 / UTM zone 1S',NULL,'EPSG','4400','EPSG','4322','EPSG','16101',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7331','projected_crs','EPSG','32301','EPSG','1874','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32302','WGS 72 / UTM zone 2S',NULL,'EPSG','4400','EPSG','4322','EPSG','16102',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7332','projected_crs','EPSG','32302','EPSG','1876','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32303','WGS 72 / UTM zone 3S',NULL,'EPSG','4400','EPSG','4322','EPSG','16103',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7333','projected_crs','EPSG','32303','EPSG','1878','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32304','WGS 72 / UTM zone 4S',NULL,'EPSG','4400','EPSG','4322','EPSG','16104',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7334','projected_crs','EPSG','32304','EPSG','1880','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32305','WGS 72 / UTM zone 5S',NULL,'EPSG','4400','EPSG','4322','EPSG','16105',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7335','projected_crs','EPSG','32305','EPSG','1882','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32306','WGS 72 / UTM zone 6S',NULL,'EPSG','4400','EPSG','4322','EPSG','16106',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7336','projected_crs','EPSG','32306','EPSG','1884','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32307','WGS 72 / UTM zone 7S',NULL,'EPSG','4400','EPSG','4322','EPSG','16107',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7337','projected_crs','EPSG','32307','EPSG','1886','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32308','WGS 72 / UTM zone 8S',NULL,'EPSG','4400','EPSG','4322','EPSG','16108',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7338','projected_crs','EPSG','32308','EPSG','1888','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32309','WGS 72 / UTM zone 9S',NULL,'EPSG','4400','EPSG','4322','EPSG','16109',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7339','projected_crs','EPSG','32309','EPSG','1890','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32310','WGS 72 / UTM zone 10S',NULL,'EPSG','4400','EPSG','4322','EPSG','16110',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7340','projected_crs','EPSG','32310','EPSG','1892','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32311','WGS 72 / UTM zone 11S',NULL,'EPSG','4400','EPSG','4322','EPSG','16111',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7341','projected_crs','EPSG','32311','EPSG','1894','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32312','WGS 72 / UTM zone 12S',NULL,'EPSG','4400','EPSG','4322','EPSG','16112',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7342','projected_crs','EPSG','32312','EPSG','1896','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32313','WGS 72 / UTM zone 13S',NULL,'EPSG','4400','EPSG','4322','EPSG','16113',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7343','projected_crs','EPSG','32313','EPSG','1898','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32314','WGS 72 / UTM zone 14S',NULL,'EPSG','4400','EPSG','4322','EPSG','16114',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7344','projected_crs','EPSG','32314','EPSG','1900','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32315','WGS 72 / UTM zone 15S',NULL,'EPSG','4400','EPSG','4322','EPSG','16115',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7345','projected_crs','EPSG','32315','EPSG','1902','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32316','WGS 72 / UTM zone 16S',NULL,'EPSG','4400','EPSG','4322','EPSG','16116',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7346','projected_crs','EPSG','32316','EPSG','1904','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32317','WGS 72 / UTM zone 17S',NULL,'EPSG','4400','EPSG','4322','EPSG','16117',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7347','projected_crs','EPSG','32317','EPSG','1906','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32318','WGS 72 / UTM zone 18S',NULL,'EPSG','4400','EPSG','4322','EPSG','16118',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7348','projected_crs','EPSG','32318','EPSG','1908','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32319','WGS 72 / UTM zone 19S',NULL,'EPSG','4400','EPSG','4322','EPSG','16119',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7349','projected_crs','EPSG','32319','EPSG','1910','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32320','WGS 72 / UTM zone 20S',NULL,'EPSG','4400','EPSG','4322','EPSG','16120',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7350','projected_crs','EPSG','32320','EPSG','1912','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32321','WGS 72 / UTM zone 21S',NULL,'EPSG','4400','EPSG','4322','EPSG','16121',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7351','projected_crs','EPSG','32321','EPSG','1914','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32322','WGS 72 / UTM zone 22S',NULL,'EPSG','4400','EPSG','4322','EPSG','16122',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7352','projected_crs','EPSG','32322','EPSG','1916','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32323','WGS 72 / UTM zone 23S',NULL,'EPSG','4400','EPSG','4322','EPSG','16123',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7353','projected_crs','EPSG','32323','EPSG','1918','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32324','WGS 72 / UTM zone 24S',NULL,'EPSG','4400','EPSG','4322','EPSG','16124',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7354','projected_crs','EPSG','32324','EPSG','1920','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32325','WGS 72 / UTM zone 25S',NULL,'EPSG','4400','EPSG','4322','EPSG','16125',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7355','projected_crs','EPSG','32325','EPSG','1922','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32326','WGS 72 / UTM zone 26S',NULL,'EPSG','4400','EPSG','4322','EPSG','16126',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7356','projected_crs','EPSG','32326','EPSG','1924','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32327','WGS 72 / UTM zone 27S',NULL,'EPSG','4400','EPSG','4322','EPSG','16127',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7357','projected_crs','EPSG','32327','EPSG','1926','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32328','WGS 72 / UTM zone 28S',NULL,'EPSG','4400','EPSG','4322','EPSG','16128',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7358','projected_crs','EPSG','32328','EPSG','1928','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32329','WGS 72 / UTM zone 29S',NULL,'EPSG','4400','EPSG','4322','EPSG','16129',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7359','projected_crs','EPSG','32329','EPSG','1930','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32330','WGS 72 / UTM zone 30S',NULL,'EPSG','4400','EPSG','4322','EPSG','16130',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7360','projected_crs','EPSG','32330','EPSG','1932','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32331','WGS 72 / UTM zone 31S',NULL,'EPSG','4400','EPSG','4322','EPSG','16131',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7361','projected_crs','EPSG','32331','EPSG','1934','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32332','WGS 72 / UTM zone 32S',NULL,'EPSG','4400','EPSG','4322','EPSG','16132',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7362','projected_crs','EPSG','32332','EPSG','1936','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32333','WGS 72 / UTM zone 33S',NULL,'EPSG','4400','EPSG','4322','EPSG','16133',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7363','projected_crs','EPSG','32333','EPSG','1938','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32334','WGS 72 / UTM zone 34S',NULL,'EPSG','4400','EPSG','4322','EPSG','16134',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7364','projected_crs','EPSG','32334','EPSG','1940','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32335','WGS 72 / UTM zone 35S',NULL,'EPSG','4400','EPSG','4322','EPSG','16135',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7365','projected_crs','EPSG','32335','EPSG','1942','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32336','WGS 72 / UTM zone 36S',NULL,'EPSG','4400','EPSG','4322','EPSG','16136',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7366','projected_crs','EPSG','32336','EPSG','1944','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32337','WGS 72 / UTM zone 37S',NULL,'EPSG','4400','EPSG','4322','EPSG','16137',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7367','projected_crs','EPSG','32337','EPSG','1946','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32338','WGS 72 / UTM zone 38S',NULL,'EPSG','4400','EPSG','4322','EPSG','16138',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7368','projected_crs','EPSG','32338','EPSG','1948','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32339','WGS 72 / UTM zone 39S',NULL,'EPSG','4400','EPSG','4322','EPSG','16139',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7369','projected_crs','EPSG','32339','EPSG','1950','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32340','WGS 72 / UTM zone 40S',NULL,'EPSG','4400','EPSG','4322','EPSG','16140',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7370','projected_crs','EPSG','32340','EPSG','1952','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32341','WGS 72 / UTM zone 41S',NULL,'EPSG','4400','EPSG','4322','EPSG','16141',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7371','projected_crs','EPSG','32341','EPSG','1954','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32342','WGS 72 / UTM zone 42S',NULL,'EPSG','4400','EPSG','4322','EPSG','16142',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7372','projected_crs','EPSG','32342','EPSG','1956','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32343','WGS 72 / UTM zone 43S',NULL,'EPSG','4400','EPSG','4322','EPSG','16143',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7373','projected_crs','EPSG','32343','EPSG','1958','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32344','WGS 72 / UTM zone 44S',NULL,'EPSG','4400','EPSG','4322','EPSG','16144',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7374','projected_crs','EPSG','32344','EPSG','1960','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32345','WGS 72 / UTM zone 45S',NULL,'EPSG','4400','EPSG','4322','EPSG','16145',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7375','projected_crs','EPSG','32345','EPSG','1962','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32346','WGS 72 / UTM zone 46S',NULL,'EPSG','4400','EPSG','4322','EPSG','16146',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7376','projected_crs','EPSG','32346','EPSG','1964','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32347','WGS 72 / UTM zone 47S',NULL,'EPSG','4400','EPSG','4322','EPSG','16147',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7377','projected_crs','EPSG','32347','EPSG','1966','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32348','WGS 72 / UTM zone 48S',NULL,'EPSG','4400','EPSG','4322','EPSG','16148',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7378','projected_crs','EPSG','32348','EPSG','1968','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32349','WGS 72 / UTM zone 49S',NULL,'EPSG','4400','EPSG','4322','EPSG','16149',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7379','projected_crs','EPSG','32349','EPSG','1970','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32350','WGS 72 / UTM zone 50S',NULL,'EPSG','4400','EPSG','4322','EPSG','16150',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7380','projected_crs','EPSG','32350','EPSG','1972','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32351','WGS 72 / UTM zone 51S',NULL,'EPSG','4400','EPSG','4322','EPSG','16151',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7381','projected_crs','EPSG','32351','EPSG','1974','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32352','WGS 72 / UTM zone 52S',NULL,'EPSG','4400','EPSG','4322','EPSG','16152',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7382','projected_crs','EPSG','32352','EPSG','1976','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32353','WGS 72 / UTM zone 53S',NULL,'EPSG','4400','EPSG','4322','EPSG','16153',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7383','projected_crs','EPSG','32353','EPSG','1978','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32354','WGS 72 / UTM zone 54S',NULL,'EPSG','4400','EPSG','4322','EPSG','16154',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7384','projected_crs','EPSG','32354','EPSG','1980','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32355','WGS 72 / UTM zone 55S',NULL,'EPSG','4400','EPSG','4322','EPSG','16155',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7385','projected_crs','EPSG','32355','EPSG','1982','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32356','WGS 72 / UTM zone 56S',NULL,'EPSG','4400','EPSG','4322','EPSG','16156',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7386','projected_crs','EPSG','32356','EPSG','1984','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32357','WGS 72 / UTM zone 57S',NULL,'EPSG','4400','EPSG','4322','EPSG','16157',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7387','projected_crs','EPSG','32357','EPSG','1986','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32358','WGS 72 / UTM zone 58S',NULL,'EPSG','4400','EPSG','4322','EPSG','16158',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7388','projected_crs','EPSG','32358','EPSG','1988','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32359','WGS 72 / UTM zone 59S',NULL,'EPSG','4400','EPSG','4322','EPSG','16159',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7389','projected_crs','EPSG','32359','EPSG','1990','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32360','WGS 72 / UTM zone 60S',NULL,'EPSG','4400','EPSG','4322','EPSG','16160',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7390','projected_crs','EPSG','32360','EPSG','1992','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32401','WGS 72BE / UTM zone 1N',NULL,'EPSG','4400','EPSG','4324','EPSG','16001',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7391','projected_crs','EPSG','32401','EPSG','1873','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32402','WGS 72BE / UTM zone 2N',NULL,'EPSG','4400','EPSG','4324','EPSG','16002',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7392','projected_crs','EPSG','32402','EPSG','1875','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32403','WGS 72BE / UTM zone 3N',NULL,'EPSG','4400','EPSG','4324','EPSG','16003',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7393','projected_crs','EPSG','32403','EPSG','1877','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32404','WGS 72BE / UTM zone 4N',NULL,'EPSG','4400','EPSG','4324','EPSG','16004',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7394','projected_crs','EPSG','32404','EPSG','1879','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32405','WGS 72BE / UTM zone 5N',NULL,'EPSG','4400','EPSG','4324','EPSG','16005',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7395','projected_crs','EPSG','32405','EPSG','1881','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32406','WGS 72BE / UTM zone 6N',NULL,'EPSG','4400','EPSG','4324','EPSG','16006',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7396','projected_crs','EPSG','32406','EPSG','1883','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32407','WGS 72BE / UTM zone 7N',NULL,'EPSG','4400','EPSG','4324','EPSG','16007',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7397','projected_crs','EPSG','32407','EPSG','1885','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32408','WGS 72BE / UTM zone 8N',NULL,'EPSG','4400','EPSG','4324','EPSG','16008',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7398','projected_crs','EPSG','32408','EPSG','1887','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32409','WGS 72BE / UTM zone 9N',NULL,'EPSG','4400','EPSG','4324','EPSG','16009',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7399','projected_crs','EPSG','32409','EPSG','1889','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32410','WGS 72BE / UTM zone 10N',NULL,'EPSG','4400','EPSG','4324','EPSG','16010',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7400','projected_crs','EPSG','32410','EPSG','1891','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32411','WGS 72BE / UTM zone 11N',NULL,'EPSG','4400','EPSG','4324','EPSG','16011',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7401','projected_crs','EPSG','32411','EPSG','1893','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32412','WGS 72BE / UTM zone 12N',NULL,'EPSG','4400','EPSG','4324','EPSG','16012',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7402','projected_crs','EPSG','32412','EPSG','1895','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32413','WGS 72BE / UTM zone 13N',NULL,'EPSG','4400','EPSG','4324','EPSG','16013',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7403','projected_crs','EPSG','32413','EPSG','1897','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32414','WGS 72BE / UTM zone 14N',NULL,'EPSG','4400','EPSG','4324','EPSG','16014',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7404','projected_crs','EPSG','32414','EPSG','1899','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32415','WGS 72BE / UTM zone 15N',NULL,'EPSG','4400','EPSG','4324','EPSG','16015',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7405','projected_crs','EPSG','32415','EPSG','1901','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32416','WGS 72BE / UTM zone 16N',NULL,'EPSG','4400','EPSG','4324','EPSG','16016',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7406','projected_crs','EPSG','32416','EPSG','1903','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32417','WGS 72BE / UTM zone 17N',NULL,'EPSG','4400','EPSG','4324','EPSG','16017',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7407','projected_crs','EPSG','32417','EPSG','1905','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32418','WGS 72BE / UTM zone 18N',NULL,'EPSG','4400','EPSG','4324','EPSG','16018',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7408','projected_crs','EPSG','32418','EPSG','1907','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32419','WGS 72BE / UTM zone 19N',NULL,'EPSG','4400','EPSG','4324','EPSG','16019',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7409','projected_crs','EPSG','32419','EPSG','1909','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32420','WGS 72BE / UTM zone 20N',NULL,'EPSG','4400','EPSG','4324','EPSG','16020',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7410','projected_crs','EPSG','32420','EPSG','1911','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32421','WGS 72BE / UTM zone 21N',NULL,'EPSG','4400','EPSG','4324','EPSG','16021',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7411','projected_crs','EPSG','32421','EPSG','1913','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32422','WGS 72BE / UTM zone 22N',NULL,'EPSG','4400','EPSG','4324','EPSG','16022',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7412','projected_crs','EPSG','32422','EPSG','1915','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32423','WGS 72BE / UTM zone 23N',NULL,'EPSG','4400','EPSG','4324','EPSG','16023',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7413','projected_crs','EPSG','32423','EPSG','1917','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32424','WGS 72BE / UTM zone 24N',NULL,'EPSG','4400','EPSG','4324','EPSG','16024',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7414','projected_crs','EPSG','32424','EPSG','1919','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32425','WGS 72BE / UTM zone 25N',NULL,'EPSG','4400','EPSG','4324','EPSG','16025',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7415','projected_crs','EPSG','32425','EPSG','1921','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32426','WGS 72BE / UTM zone 26N',NULL,'EPSG','4400','EPSG','4324','EPSG','16026',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7416','projected_crs','EPSG','32426','EPSG','1923','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32427','WGS 72BE / UTM zone 27N',NULL,'EPSG','4400','EPSG','4324','EPSG','16027',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7417','projected_crs','EPSG','32427','EPSG','1925','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32428','WGS 72BE / UTM zone 28N',NULL,'EPSG','4400','EPSG','4324','EPSG','16028',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7418','projected_crs','EPSG','32428','EPSG','1927','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32429','WGS 72BE / UTM zone 29N',NULL,'EPSG','4400','EPSG','4324','EPSG','16029',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7419','projected_crs','EPSG','32429','EPSG','1929','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32430','WGS 72BE / UTM zone 30N',NULL,'EPSG','4400','EPSG','4324','EPSG','16030',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7420','projected_crs','EPSG','32430','EPSG','1931','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32431','WGS 72BE / UTM zone 31N',NULL,'EPSG','4400','EPSG','4324','EPSG','16031',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7421','projected_crs','EPSG','32431','EPSG','1933','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32432','WGS 72BE / UTM zone 32N',NULL,'EPSG','4400','EPSG','4324','EPSG','16032',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7422','projected_crs','EPSG','32432','EPSG','1935','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32433','WGS 72BE / UTM zone 33N',NULL,'EPSG','4400','EPSG','4324','EPSG','16033',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7423','projected_crs','EPSG','32433','EPSG','3464','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32434','WGS 72BE / UTM zone 34N',NULL,'EPSG','4400','EPSG','4324','EPSG','16034',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7424','projected_crs','EPSG','32434','EPSG','3465','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32435','WGS 72BE / UTM zone 35N',NULL,'EPSG','4400','EPSG','4324','EPSG','16035',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7425','projected_crs','EPSG','32435','EPSG','1941','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32436','WGS 72BE / UTM zone 36N',NULL,'EPSG','4400','EPSG','4324','EPSG','16036',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7426','projected_crs','EPSG','32436','EPSG','1943','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32437','WGS 72BE / UTM zone 37N',NULL,'EPSG','4400','EPSG','4324','EPSG','16037',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7427','projected_crs','EPSG','32437','EPSG','1945','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32438','WGS 72BE / UTM zone 38N',NULL,'EPSG','4400','EPSG','4324','EPSG','16038',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7428','projected_crs','EPSG','32438','EPSG','1947','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32439','WGS 72BE / UTM zone 39N',NULL,'EPSG','4400','EPSG','4324','EPSG','16039',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7429','projected_crs','EPSG','32439','EPSG','1949','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32440','WGS 72BE / UTM zone 40N',NULL,'EPSG','4400','EPSG','4324','EPSG','16040',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7430','projected_crs','EPSG','32440','EPSG','1951','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32441','WGS 72BE / UTM zone 41N',NULL,'EPSG','4400','EPSG','4324','EPSG','16041',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7431','projected_crs','EPSG','32441','EPSG','1953','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32442','WGS 72BE / UTM zone 42N',NULL,'EPSG','4400','EPSG','4324','EPSG','16042',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7432','projected_crs','EPSG','32442','EPSG','1955','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32443','WGS 72BE / UTM zone 43N',NULL,'EPSG','4400','EPSG','4324','EPSG','16043',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7433','projected_crs','EPSG','32443','EPSG','1957','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32444','WGS 72BE / UTM zone 44N',NULL,'EPSG','4400','EPSG','4324','EPSG','16044',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7434','projected_crs','EPSG','32444','EPSG','1959','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32445','WGS 72BE / UTM zone 45N',NULL,'EPSG','4400','EPSG','4324','EPSG','16045',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7435','projected_crs','EPSG','32445','EPSG','1961','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32446','WGS 72BE / UTM zone 46N',NULL,'EPSG','4400','EPSG','4324','EPSG','16046',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7436','projected_crs','EPSG','32446','EPSG','1963','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32447','WGS 72BE / UTM zone 47N',NULL,'EPSG','4400','EPSG','4324','EPSG','16047',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7437','projected_crs','EPSG','32447','EPSG','1965','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32448','WGS 72BE / UTM zone 48N',NULL,'EPSG','4400','EPSG','4324','EPSG','16048',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7438','projected_crs','EPSG','32448','EPSG','1993','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32449','WGS 72BE / UTM zone 49N',NULL,'EPSG','4400','EPSG','4324','EPSG','16049',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7439','projected_crs','EPSG','32449','EPSG','1994','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32450','WGS 72BE / UTM zone 50N',NULL,'EPSG','4400','EPSG','4324','EPSG','16050',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7440','projected_crs','EPSG','32450','EPSG','1971','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32451','WGS 72BE / UTM zone 51N',NULL,'EPSG','4400','EPSG','4324','EPSG','16051',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7441','projected_crs','EPSG','32451','EPSG','1973','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32452','WGS 72BE / UTM zone 52N',NULL,'EPSG','4400','EPSG','4324','EPSG','16052',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7442','projected_crs','EPSG','32452','EPSG','1975','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32453','WGS 72BE / UTM zone 53N',NULL,'EPSG','4400','EPSG','4324','EPSG','16053',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7443','projected_crs','EPSG','32453','EPSG','1977','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32454','WGS 72BE / UTM zone 54N',NULL,'EPSG','4400','EPSG','4324','EPSG','16054',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7444','projected_crs','EPSG','32454','EPSG','1979','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32455','WGS 72BE / UTM zone 55N',NULL,'EPSG','4400','EPSG','4324','EPSG','16055',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7445','projected_crs','EPSG','32455','EPSG','1981','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32456','WGS 72BE / UTM zone 56N',NULL,'EPSG','4400','EPSG','4324','EPSG','16056',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7446','projected_crs','EPSG','32456','EPSG','1983','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32457','WGS 72BE / UTM zone 57N',NULL,'EPSG','4400','EPSG','4324','EPSG','16057',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7447','projected_crs','EPSG','32457','EPSG','1985','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32458','WGS 72BE / UTM zone 58N',NULL,'EPSG','4400','EPSG','4324','EPSG','16058',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7448','projected_crs','EPSG','32458','EPSG','1987','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32459','WGS 72BE / UTM zone 59N',NULL,'EPSG','4400','EPSG','4324','EPSG','16059',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7449','projected_crs','EPSG','32459','EPSG','1989','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32460','WGS 72BE / UTM zone 60N',NULL,'EPSG','4400','EPSG','4324','EPSG','16060',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7450','projected_crs','EPSG','32460','EPSG','1991','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32501','WGS 72BE / UTM zone 1S',NULL,'EPSG','4400','EPSG','4324','EPSG','16101',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7451','projected_crs','EPSG','32501','EPSG','1874','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32502','WGS 72BE / UTM zone 2S',NULL,'EPSG','4400','EPSG','4324','EPSG','16102',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7452','projected_crs','EPSG','32502','EPSG','1876','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32503','WGS 72BE / UTM zone 3S',NULL,'EPSG','4400','EPSG','4324','EPSG','16103',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7453','projected_crs','EPSG','32503','EPSG','1878','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32504','WGS 72BE / UTM zone 4S',NULL,'EPSG','4400','EPSG','4324','EPSG','16104',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7454','projected_crs','EPSG','32504','EPSG','1880','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32505','WGS 72BE / UTM zone 5S',NULL,'EPSG','4400','EPSG','4324','EPSG','16105',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7455','projected_crs','EPSG','32505','EPSG','1882','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32506','WGS 72BE / UTM zone 6S',NULL,'EPSG','4400','EPSG','4324','EPSG','16106',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7456','projected_crs','EPSG','32506','EPSG','1884','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32507','WGS 72BE / UTM zone 7S',NULL,'EPSG','4400','EPSG','4324','EPSG','16107',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7457','projected_crs','EPSG','32507','EPSG','1886','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32508','WGS 72BE / UTM zone 8S',NULL,'EPSG','4400','EPSG','4324','EPSG','16108',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7458','projected_crs','EPSG','32508','EPSG','1888','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32509','WGS 72BE / UTM zone 9S',NULL,'EPSG','4400','EPSG','4324','EPSG','16109',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7459','projected_crs','EPSG','32509','EPSG','1890','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32510','WGS 72BE / UTM zone 10S',NULL,'EPSG','4400','EPSG','4324','EPSG','16110',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7460','projected_crs','EPSG','32510','EPSG','1892','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32511','WGS 72BE / UTM zone 11S',NULL,'EPSG','4400','EPSG','4324','EPSG','16111',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7461','projected_crs','EPSG','32511','EPSG','1894','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32512','WGS 72BE / UTM zone 12S',NULL,'EPSG','4400','EPSG','4324','EPSG','16112',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7462','projected_crs','EPSG','32512','EPSG','1896','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32513','WGS 72BE / UTM zone 13S',NULL,'EPSG','4400','EPSG','4324','EPSG','16113',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7463','projected_crs','EPSG','32513','EPSG','1898','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32514','WGS 72BE / UTM zone 14S',NULL,'EPSG','4400','EPSG','4324','EPSG','16114',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7464','projected_crs','EPSG','32514','EPSG','1900','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32515','WGS 72BE / UTM zone 15S',NULL,'EPSG','4400','EPSG','4324','EPSG','16115',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7465','projected_crs','EPSG','32515','EPSG','1902','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32516','WGS 72BE / UTM zone 16S',NULL,'EPSG','4400','EPSG','4324','EPSG','16116',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7466','projected_crs','EPSG','32516','EPSG','1904','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32517','WGS 72BE / UTM zone 17S',NULL,'EPSG','4400','EPSG','4324','EPSG','16117',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7467','projected_crs','EPSG','32517','EPSG','1906','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32518','WGS 72BE / UTM zone 18S',NULL,'EPSG','4400','EPSG','4324','EPSG','16118',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7468','projected_crs','EPSG','32518','EPSG','1908','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32519','WGS 72BE / UTM zone 19S',NULL,'EPSG','4400','EPSG','4324','EPSG','16119',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7469','projected_crs','EPSG','32519','EPSG','1910','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32520','WGS 72BE / UTM zone 20S',NULL,'EPSG','4400','EPSG','4324','EPSG','16120',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7470','projected_crs','EPSG','32520','EPSG','1912','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32521','WGS 72BE / UTM zone 21S',NULL,'EPSG','4400','EPSG','4324','EPSG','16121',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7471','projected_crs','EPSG','32521','EPSG','1914','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32522','WGS 72BE / UTM zone 22S',NULL,'EPSG','4400','EPSG','4324','EPSG','16122',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7472','projected_crs','EPSG','32522','EPSG','1916','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32523','WGS 72BE / UTM zone 23S',NULL,'EPSG','4400','EPSG','4324','EPSG','16123',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7473','projected_crs','EPSG','32523','EPSG','1918','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32524','WGS 72BE / UTM zone 24S',NULL,'EPSG','4400','EPSG','4324','EPSG','16124',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7474','projected_crs','EPSG','32524','EPSG','1920','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32525','WGS 72BE / UTM zone 25S',NULL,'EPSG','4400','EPSG','4324','EPSG','16125',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7475','projected_crs','EPSG','32525','EPSG','1922','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32526','WGS 72BE / UTM zone 26S',NULL,'EPSG','4400','EPSG','4324','EPSG','16126',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7476','projected_crs','EPSG','32526','EPSG','1924','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32527','WGS 72BE / UTM zone 27S',NULL,'EPSG','4400','EPSG','4324','EPSG','16127',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7477','projected_crs','EPSG','32527','EPSG','1926','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32528','WGS 72BE / UTM zone 28S',NULL,'EPSG','4400','EPSG','4324','EPSG','16128',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7478','projected_crs','EPSG','32528','EPSG','1928','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32529','WGS 72BE / UTM zone 29S',NULL,'EPSG','4400','EPSG','4324','EPSG','16129',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7479','projected_crs','EPSG','32529','EPSG','1930','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32530','WGS 72BE / UTM zone 30S',NULL,'EPSG','4400','EPSG','4324','EPSG','16130',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7480','projected_crs','EPSG','32530','EPSG','1932','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32531','WGS 72BE / UTM zone 31S',NULL,'EPSG','4400','EPSG','4324','EPSG','16131',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7481','projected_crs','EPSG','32531','EPSG','1934','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32532','WGS 72BE / UTM zone 32S',NULL,'EPSG','4400','EPSG','4324','EPSG','16132',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7482','projected_crs','EPSG','32532','EPSG','1936','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32533','WGS 72BE / UTM zone 33S',NULL,'EPSG','4400','EPSG','4324','EPSG','16133',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7483','projected_crs','EPSG','32533','EPSG','1938','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32534','WGS 72BE / UTM zone 34S',NULL,'EPSG','4400','EPSG','4324','EPSG','16134',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7484','projected_crs','EPSG','32534','EPSG','1940','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32535','WGS 72BE / UTM zone 35S',NULL,'EPSG','4400','EPSG','4324','EPSG','16135',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7485','projected_crs','EPSG','32535','EPSG','1942','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32536','WGS 72BE / UTM zone 36S',NULL,'EPSG','4400','EPSG','4324','EPSG','16136',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7486','projected_crs','EPSG','32536','EPSG','1944','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32537','WGS 72BE / UTM zone 37S',NULL,'EPSG','4400','EPSG','4324','EPSG','16137',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7487','projected_crs','EPSG','32537','EPSG','1946','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32538','WGS 72BE / UTM zone 38S',NULL,'EPSG','4400','EPSG','4324','EPSG','16138',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7488','projected_crs','EPSG','32538','EPSG','1948','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32539','WGS 72BE / UTM zone 39S',NULL,'EPSG','4400','EPSG','4324','EPSG','16139',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7489','projected_crs','EPSG','32539','EPSG','1950','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32540','WGS 72BE / UTM zone 40S',NULL,'EPSG','4400','EPSG','4324','EPSG','16140',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7490','projected_crs','EPSG','32540','EPSG','1952','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32541','WGS 72BE / UTM zone 41S',NULL,'EPSG','4400','EPSG','4324','EPSG','16141',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7491','projected_crs','EPSG','32541','EPSG','1954','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32542','WGS 72BE / UTM zone 42S',NULL,'EPSG','4400','EPSG','4324','EPSG','16142',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7492','projected_crs','EPSG','32542','EPSG','1956','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32543','WGS 72BE / UTM zone 43S',NULL,'EPSG','4400','EPSG','4324','EPSG','16143',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7493','projected_crs','EPSG','32543','EPSG','1958','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32544','WGS 72BE / UTM zone 44S',NULL,'EPSG','4400','EPSG','4324','EPSG','16144',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7494','projected_crs','EPSG','32544','EPSG','1960','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32545','WGS 72BE / UTM zone 45S',NULL,'EPSG','4400','EPSG','4324','EPSG','16145',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7495','projected_crs','EPSG','32545','EPSG','1962','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32546','WGS 72BE / UTM zone 46S',NULL,'EPSG','4400','EPSG','4324','EPSG','16146',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7496','projected_crs','EPSG','32546','EPSG','1964','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32547','WGS 72BE / UTM zone 47S',NULL,'EPSG','4400','EPSG','4324','EPSG','16147',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7497','projected_crs','EPSG','32547','EPSG','1966','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32548','WGS 72BE / UTM zone 48S',NULL,'EPSG','4400','EPSG','4324','EPSG','16148',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7498','projected_crs','EPSG','32548','EPSG','1968','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32549','WGS 72BE / UTM zone 49S',NULL,'EPSG','4400','EPSG','4324','EPSG','16149',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7499','projected_crs','EPSG','32549','EPSG','1995','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32550','WGS 72BE / UTM zone 50S',NULL,'EPSG','4400','EPSG','4324','EPSG','16150',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7500','projected_crs','EPSG','32550','EPSG','1972','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32551','WGS 72BE / UTM zone 51S',NULL,'EPSG','4400','EPSG','4324','EPSG','16151',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7501','projected_crs','EPSG','32551','EPSG','1974','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32552','WGS 72BE / UTM zone 52S',NULL,'EPSG','4400','EPSG','4324','EPSG','16152',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7502','projected_crs','EPSG','32552','EPSG','1976','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32553','WGS 72BE / UTM zone 53S',NULL,'EPSG','4400','EPSG','4324','EPSG','16153',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7503','projected_crs','EPSG','32553','EPSG','1978','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32554','WGS 72BE / UTM zone 54S',NULL,'EPSG','4400','EPSG','4324','EPSG','16154',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7504','projected_crs','EPSG','32554','EPSG','1980','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32555','WGS 72BE / UTM zone 55S',NULL,'EPSG','4400','EPSG','4324','EPSG','16155',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7505','projected_crs','EPSG','32555','EPSG','1982','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32556','WGS 72BE / UTM zone 56S',NULL,'EPSG','4400','EPSG','4324','EPSG','16156',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7506','projected_crs','EPSG','32556','EPSG','1984','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32557','WGS 72BE / UTM zone 57S',NULL,'EPSG','4400','EPSG','4324','EPSG','16157',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7507','projected_crs','EPSG','32557','EPSG','1986','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32558','WGS 72BE / UTM zone 58S',NULL,'EPSG','4400','EPSG','4324','EPSG','16158',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7508','projected_crs','EPSG','32558','EPSG','1988','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32559','WGS 72BE / UTM zone 59S',NULL,'EPSG','4400','EPSG','4324','EPSG','16159',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7509','projected_crs','EPSG','32559','EPSG','1990','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32560','WGS 72BE / UTM zone 60S',NULL,'EPSG','4400','EPSG','4324','EPSG','16160',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7510','projected_crs','EPSG','32560','EPSG','1992','EPSG','1142');
INSERT INTO "projected_crs" VALUES('EPSG','32600','WGS 84 / UTM grid system (northern hemisphere)',NULL,'EPSG','4400','EPSG','4326','EPSG','16000',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7511','projected_crs','EPSG','32600','EPSG','1998','EPSG','1163');
INSERT INTO "projected_crs" VALUES('EPSG','32601','WGS 84 / UTM zone 1N',NULL,'EPSG','4400','EPSG','4326','EPSG','16001',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7512','projected_crs','EPSG','32601','EPSG','2000','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32602','WGS 84 / UTM zone 2N',NULL,'EPSG','4400','EPSG','4326','EPSG','16002',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7513','projected_crs','EPSG','32602','EPSG','2002','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32603','WGS 84 / UTM zone 3N',NULL,'EPSG','4400','EPSG','4326','EPSG','16003',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7514','projected_crs','EPSG','32603','EPSG','2004','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32604','WGS 84 / UTM zone 4N',NULL,'EPSG','4400','EPSG','4326','EPSG','16004',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7515','projected_crs','EPSG','32604','EPSG','2006','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32605','WGS 84 / UTM zone 5N',NULL,'EPSG','4400','EPSG','4326','EPSG','16005',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7516','projected_crs','EPSG','32605','EPSG','2008','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32606','WGS 84 / UTM zone 6N',NULL,'EPSG','4400','EPSG','4326','EPSG','16006',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7517','projected_crs','EPSG','32606','EPSG','2010','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32607','WGS 84 / UTM zone 7N',NULL,'EPSG','4400','EPSG','4326','EPSG','16007',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7518','projected_crs','EPSG','32607','EPSG','2012','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32608','WGS 84 / UTM zone 8N',NULL,'EPSG','4400','EPSG','4326','EPSG','16008',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7519','projected_crs','EPSG','32608','EPSG','2014','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32609','WGS 84 / UTM zone 9N',NULL,'EPSG','4400','EPSG','4326','EPSG','16009',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7520','projected_crs','EPSG','32609','EPSG','2016','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32610','WGS 84 / UTM zone 10N',NULL,'EPSG','4400','EPSG','4326','EPSG','16010',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7521','projected_crs','EPSG','32610','EPSG','2018','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32611','WGS 84 / UTM zone 11N',NULL,'EPSG','4400','EPSG','4326','EPSG','16011',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7522','projected_crs','EPSG','32611','EPSG','2020','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32612','WGS 84 / UTM zone 12N',NULL,'EPSG','4400','EPSG','4326','EPSG','16012',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7523','projected_crs','EPSG','32612','EPSG','2022','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32613','WGS 84 / UTM zone 13N',NULL,'EPSG','4400','EPSG','4326','EPSG','16013',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7524','projected_crs','EPSG','32613','EPSG','2024','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32614','WGS 84 / UTM zone 14N',NULL,'EPSG','4400','EPSG','4326','EPSG','16014',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7525','projected_crs','EPSG','32614','EPSG','2026','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32615','WGS 84 / UTM zone 15N',NULL,'EPSG','4400','EPSG','4326','EPSG','16015',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7526','projected_crs','EPSG','32615','EPSG','2028','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32616','WGS 84 / UTM zone 16N',NULL,'EPSG','4400','EPSG','4326','EPSG','16016',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7527','projected_crs','EPSG','32616','EPSG','2030','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32617','WGS 84 / UTM zone 17N',NULL,'EPSG','4400','EPSG','4326','EPSG','16017',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7528','projected_crs','EPSG','32617','EPSG','2032','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32618','WGS 84 / UTM zone 18N',NULL,'EPSG','4400','EPSG','4326','EPSG','16018',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7529','projected_crs','EPSG','32618','EPSG','2034','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32619','WGS 84 / UTM zone 19N',NULL,'EPSG','4400','EPSG','4326','EPSG','16019',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7530','projected_crs','EPSG','32619','EPSG','2036','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32620','WGS 84 / UTM zone 20N',NULL,'EPSG','4400','EPSG','4326','EPSG','16020',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7531','projected_crs','EPSG','32620','EPSG','2038','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32621','WGS 84 / UTM zone 21N',NULL,'EPSG','4400','EPSG','4326','EPSG','16021',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7532','projected_crs','EPSG','32621','EPSG','2040','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32622','WGS 84 / UTM zone 22N',NULL,'EPSG','4400','EPSG','4326','EPSG','16022',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7533','projected_crs','EPSG','32622','EPSG','2042','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32623','WGS 84 / UTM zone 23N',NULL,'EPSG','4400','EPSG','4326','EPSG','16023',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7534','projected_crs','EPSG','32623','EPSG','2044','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32624','WGS 84 / UTM zone 24N',NULL,'EPSG','4400','EPSG','4326','EPSG','16024',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7535','projected_crs','EPSG','32624','EPSG','2046','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32625','WGS 84 / UTM zone 25N',NULL,'EPSG','4400','EPSG','4326','EPSG','16025',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7536','projected_crs','EPSG','32625','EPSG','2048','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32626','WGS 84 / UTM zone 26N',NULL,'EPSG','4400','EPSG','4326','EPSG','16026',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7537','projected_crs','EPSG','32626','EPSG','2050','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32627','WGS 84 / UTM zone 27N',NULL,'EPSG','4400','EPSG','4326','EPSG','16027',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7538','projected_crs','EPSG','32627','EPSG','2052','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32628','WGS 84 / UTM zone 28N',NULL,'EPSG','4400','EPSG','4326','EPSG','16028',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7539','projected_crs','EPSG','32628','EPSG','2054','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32629','WGS 84 / UTM zone 29N',NULL,'EPSG','4400','EPSG','4326','EPSG','16029',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7540','projected_crs','EPSG','32629','EPSG','2056','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32630','WGS 84 / UTM zone 30N',NULL,'EPSG','4400','EPSG','4326','EPSG','16030',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7541','projected_crs','EPSG','32630','EPSG','2058','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32631','WGS 84 / UTM zone 31N',NULL,'EPSG','4400','EPSG','4326','EPSG','16031',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7542','projected_crs','EPSG','32631','EPSG','2060','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32632','WGS 84 / UTM zone 32N',NULL,'EPSG','4400','EPSG','4326','EPSG','16032',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7543','projected_crs','EPSG','32632','EPSG','2062','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32633','WGS 84 / UTM zone 33N',NULL,'EPSG','4400','EPSG','4326','EPSG','16033',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7544','projected_crs','EPSG','32633','EPSG','2064','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32634','WGS 84 / UTM zone 34N',NULL,'EPSG','4400','EPSG','4326','EPSG','16034',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7545','projected_crs','EPSG','32634','EPSG','2066','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32635','WGS 84 / UTM zone 35N',NULL,'EPSG','4400','EPSG','4326','EPSG','16035',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7546','projected_crs','EPSG','32635','EPSG','2068','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32636','WGS 84 / UTM zone 36N',NULL,'EPSG','4400','EPSG','4326','EPSG','16036',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7547','projected_crs','EPSG','32636','EPSG','2070','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32637','WGS 84 / UTM zone 37N',NULL,'EPSG','4400','EPSG','4326','EPSG','16037',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7548','projected_crs','EPSG','32637','EPSG','2072','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32638','WGS 84 / UTM zone 38N',NULL,'EPSG','4400','EPSG','4326','EPSG','16038',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7549','projected_crs','EPSG','32638','EPSG','2074','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32639','WGS 84 / UTM zone 39N',NULL,'EPSG','4400','EPSG','4326','EPSG','16039',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7550','projected_crs','EPSG','32639','EPSG','2076','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32640','WGS 84 / UTM zone 40N',NULL,'EPSG','4400','EPSG','4326','EPSG','16040',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7551','projected_crs','EPSG','32640','EPSG','2078','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32641','WGS 84 / UTM zone 41N',NULL,'EPSG','4400','EPSG','4326','EPSG','16041',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7552','projected_crs','EPSG','32641','EPSG','2080','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32642','WGS 84 / UTM zone 42N',NULL,'EPSG','4400','EPSG','4326','EPSG','16042',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7553','projected_crs','EPSG','32642','EPSG','2082','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32643','WGS 84 / UTM zone 43N',NULL,'EPSG','4400','EPSG','4326','EPSG','16043',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7554','projected_crs','EPSG','32643','EPSG','2084','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32644','WGS 84 / UTM zone 44N',NULL,'EPSG','4400','EPSG','4326','EPSG','16044',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7555','projected_crs','EPSG','32644','EPSG','2086','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32645','WGS 84 / UTM zone 45N',NULL,'EPSG','4400','EPSG','4326','EPSG','16045',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7556','projected_crs','EPSG','32645','EPSG','2088','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32646','WGS 84 / UTM zone 46N',NULL,'EPSG','4400','EPSG','4326','EPSG','16046',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7557','projected_crs','EPSG','32646','EPSG','2090','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32647','WGS 84 / UTM zone 47N',NULL,'EPSG','4400','EPSG','4326','EPSG','16047',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7558','projected_crs','EPSG','32647','EPSG','2092','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32648','WGS 84 / UTM zone 48N',NULL,'EPSG','4400','EPSG','4326','EPSG','16048',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7559','projected_crs','EPSG','32648','EPSG','2094','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32649','WGS 84 / UTM zone 49N',NULL,'EPSG','4400','EPSG','4326','EPSG','16049',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7560','projected_crs','EPSG','32649','EPSG','2096','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32650','WGS 84 / UTM zone 50N',NULL,'EPSG','4400','EPSG','4326','EPSG','16050',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7561','projected_crs','EPSG','32650','EPSG','2098','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32651','WGS 84 / UTM zone 51N',NULL,'EPSG','4400','EPSG','4326','EPSG','16051',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7562','projected_crs','EPSG','32651','EPSG','2100','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32652','WGS 84 / UTM zone 52N',NULL,'EPSG','4400','EPSG','4326','EPSG','16052',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7563','projected_crs','EPSG','32652','EPSG','2102','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32653','WGS 84 / UTM zone 53N',NULL,'EPSG','4400','EPSG','4326','EPSG','16053',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7564','projected_crs','EPSG','32653','EPSG','2104','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32654','WGS 84 / UTM zone 54N',NULL,'EPSG','4400','EPSG','4326','EPSG','16054',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7565','projected_crs','EPSG','32654','EPSG','2106','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32655','WGS 84 / UTM zone 55N',NULL,'EPSG','4400','EPSG','4326','EPSG','16055',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7566','projected_crs','EPSG','32655','EPSG','2108','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32656','WGS 84 / UTM zone 56N',NULL,'EPSG','4400','EPSG','4326','EPSG','16056',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7567','projected_crs','EPSG','32656','EPSG','2110','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32657','WGS 84 / UTM zone 57N',NULL,'EPSG','4400','EPSG','4326','EPSG','16057',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7568','projected_crs','EPSG','32657','EPSG','2112','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32658','WGS 84 / UTM zone 58N',NULL,'EPSG','4400','EPSG','4326','EPSG','16058',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7569','projected_crs','EPSG','32658','EPSG','2114','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32659','WGS 84 / UTM zone 59N',NULL,'EPSG','4400','EPSG','4326','EPSG','16059',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7570','projected_crs','EPSG','32659','EPSG','2116','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32660','WGS 84 / UTM zone 60N',NULL,'EPSG','4400','EPSG','4326','EPSG','16060',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7571','projected_crs','EPSG','32660','EPSG','2118','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32661','WGS 84 / UPS North (N,E)',NULL,'EPSG','4493','EPSG','4326','EPSG','16061',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7572','projected_crs','EPSG','32661','EPSG','1996','EPSG','1160');
INSERT INTO "projected_crs" VALUES('EPSG','32662','WGS 84 / Plate Carree',NULL,'EPSG','4499','EPSG','4326','EPSG','19968',NULL,1);
INSERT INTO "usage" VALUES('EPSG','7573','projected_crs','EPSG','32662','EPSG','1262','EPSG','1191');
INSERT INTO "projected_crs" VALUES('EPSG','32663','WGS 84 / World Equidistant Cylindrical',NULL,'EPSG','4499','EPSG','4326','EPSG','19846',NULL,1);
INSERT INTO "usage" VALUES('EPSG','7574','projected_crs','EPSG','32663','EPSG','1262','EPSG','1191');
INSERT INTO "projected_crs" VALUES('EPSG','32664','WGS 84 / BLM 14N (ftUS)',NULL,'EPSG','4497','EPSG','4326','EPSG','15914',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7575','projected_crs','EPSG','32664','EPSG','2171','EPSG','1212');
INSERT INTO "projected_crs" VALUES('EPSG','32665','WGS 84 / BLM 15N (ftUS)',NULL,'EPSG','4497','EPSG','4326','EPSG','15915',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7576','projected_crs','EPSG','32665','EPSG','2172','EPSG','1212');
INSERT INTO "projected_crs" VALUES('EPSG','32666','WGS 84 / BLM 16N (ftUS)',NULL,'EPSG','4497','EPSG','4326','EPSG','15916',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7577','projected_crs','EPSG','32666','EPSG','2173','EPSG','1212');
INSERT INTO "projected_crs" VALUES('EPSG','32667','WGS 84 / BLM 17N (ftUS)',NULL,'EPSG','4497','EPSG','4326','EPSG','15917',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7578','projected_crs','EPSG','32667','EPSG','2174','EPSG','1212');
INSERT INTO "projected_crs" VALUES('EPSG','32700','WGS 84 / UTM grid system (southern hemisphere)',NULL,'EPSG','4400','EPSG','4326','EPSG','16100',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7579','projected_crs','EPSG','32700','EPSG','1999','EPSG','1163');
INSERT INTO "projected_crs" VALUES('EPSG','32701','WGS 84 / UTM zone 1S',NULL,'EPSG','4400','EPSG','4326','EPSG','16101',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7580','projected_crs','EPSG','32701','EPSG','2001','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32702','WGS 84 / UTM zone 2S',NULL,'EPSG','4400','EPSG','4326','EPSG','16102',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7581','projected_crs','EPSG','32702','EPSG','2003','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32703','WGS 84 / UTM zone 3S',NULL,'EPSG','4400','EPSG','4326','EPSG','16103',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7582','projected_crs','EPSG','32703','EPSG','2005','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32704','WGS 84 / UTM zone 4S',NULL,'EPSG','4400','EPSG','4326','EPSG','16104',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7583','projected_crs','EPSG','32704','EPSG','2007','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32705','WGS 84 / UTM zone 5S',NULL,'EPSG','4400','EPSG','4326','EPSG','16105',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7584','projected_crs','EPSG','32705','EPSG','2009','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32706','WGS 84 / UTM zone 6S',NULL,'EPSG','4400','EPSG','4326','EPSG','16106',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7585','projected_crs','EPSG','32706','EPSG','2011','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32707','WGS 84 / UTM zone 7S',NULL,'EPSG','4400','EPSG','4326','EPSG','16107',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7586','projected_crs','EPSG','32707','EPSG','2013','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32708','WGS 84 / UTM zone 8S',NULL,'EPSG','4400','EPSG','4326','EPSG','16108',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7587','projected_crs','EPSG','32708','EPSG','2015','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32709','WGS 84 / UTM zone 9S',NULL,'EPSG','4400','EPSG','4326','EPSG','16109',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7588','projected_crs','EPSG','32709','EPSG','2017','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32710','WGS 84 / UTM zone 10S',NULL,'EPSG','4400','EPSG','4326','EPSG','16110',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7589','projected_crs','EPSG','32710','EPSG','2019','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32711','WGS 84 / UTM zone 11S',NULL,'EPSG','4400','EPSG','4326','EPSG','16111',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7590','projected_crs','EPSG','32711','EPSG','2021','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32712','WGS 84 / UTM zone 12S',NULL,'EPSG','4400','EPSG','4326','EPSG','16112',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7591','projected_crs','EPSG','32712','EPSG','2023','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32713','WGS 84 / UTM zone 13S',NULL,'EPSG','4400','EPSG','4326','EPSG','16113',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7592','projected_crs','EPSG','32713','EPSG','2025','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32714','WGS 84 / UTM zone 14S',NULL,'EPSG','4400','EPSG','4326','EPSG','16114',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7593','projected_crs','EPSG','32714','EPSG','2027','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32715','WGS 84 / UTM zone 15S',NULL,'EPSG','4400','EPSG','4326','EPSG','16115',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7594','projected_crs','EPSG','32715','EPSG','2029','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32716','WGS 84 / UTM zone 16S',NULL,'EPSG','4400','EPSG','4326','EPSG','16116',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7595','projected_crs','EPSG','32716','EPSG','2031','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32717','WGS 84 / UTM zone 17S',NULL,'EPSG','4400','EPSG','4326','EPSG','16117',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7596','projected_crs','EPSG','32717','EPSG','2033','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32718','WGS 84 / UTM zone 18S',NULL,'EPSG','4400','EPSG','4326','EPSG','16118',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7597','projected_crs','EPSG','32718','EPSG','2035','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32719','WGS 84 / UTM zone 19S',NULL,'EPSG','4400','EPSG','4326','EPSG','16119',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7598','projected_crs','EPSG','32719','EPSG','2037','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32720','WGS 84 / UTM zone 20S',NULL,'EPSG','4400','EPSG','4326','EPSG','16120',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7599','projected_crs','EPSG','32720','EPSG','2039','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32721','WGS 84 / UTM zone 21S',NULL,'EPSG','4400','EPSG','4326','EPSG','16121',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7600','projected_crs','EPSG','32721','EPSG','2041','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32722','WGS 84 / UTM zone 22S',NULL,'EPSG','4400','EPSG','4326','EPSG','16122',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7601','projected_crs','EPSG','32722','EPSG','2043','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32723','WGS 84 / UTM zone 23S',NULL,'EPSG','4400','EPSG','4326','EPSG','16123',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7602','projected_crs','EPSG','32723','EPSG','2045','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32724','WGS 84 / UTM zone 24S',NULL,'EPSG','4400','EPSG','4326','EPSG','16124',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7603','projected_crs','EPSG','32724','EPSG','2047','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32725','WGS 84 / UTM zone 25S',NULL,'EPSG','4400','EPSG','4326','EPSG','16125',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7604','projected_crs','EPSG','32725','EPSG','2049','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32726','WGS 84 / UTM zone 26S',NULL,'EPSG','4400','EPSG','4326','EPSG','16126',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7605','projected_crs','EPSG','32726','EPSG','2051','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32727','WGS 84 / UTM zone 27S',NULL,'EPSG','4400','EPSG','4326','EPSG','16127',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7606','projected_crs','EPSG','32727','EPSG','2053','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32728','WGS 84 / UTM zone 28S',NULL,'EPSG','4400','EPSG','4326','EPSG','16128',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7607','projected_crs','EPSG','32728','EPSG','2055','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32729','WGS 84 / UTM zone 29S',NULL,'EPSG','4400','EPSG','4326','EPSG','16129',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7608','projected_crs','EPSG','32729','EPSG','2057','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32730','WGS 84 / UTM zone 30S',NULL,'EPSG','4400','EPSG','4326','EPSG','16130',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7609','projected_crs','EPSG','32730','EPSG','2059','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32731','WGS 84 / UTM zone 31S',NULL,'EPSG','4400','EPSG','4326','EPSG','16131',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7610','projected_crs','EPSG','32731','EPSG','2061','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32732','WGS 84 / UTM zone 32S',NULL,'EPSG','4400','EPSG','4326','EPSG','16132',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7611','projected_crs','EPSG','32732','EPSG','2063','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32733','WGS 84 / UTM zone 33S',NULL,'EPSG','4400','EPSG','4326','EPSG','16133',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7612','projected_crs','EPSG','32733','EPSG','2065','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32734','WGS 84 / UTM zone 34S',NULL,'EPSG','4400','EPSG','4326','EPSG','16134',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7613','projected_crs','EPSG','32734','EPSG','2067','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32735','WGS 84 / UTM zone 35S',NULL,'EPSG','4400','EPSG','4326','EPSG','16135',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7614','projected_crs','EPSG','32735','EPSG','2069','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32736','WGS 84 / UTM zone 36S',NULL,'EPSG','4400','EPSG','4326','EPSG','16136',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7615','projected_crs','EPSG','32736','EPSG','2071','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32737','WGS 84 / UTM zone 37S',NULL,'EPSG','4400','EPSG','4326','EPSG','16137',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7616','projected_crs','EPSG','32737','EPSG','2073','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32738','WGS 84 / UTM zone 38S',NULL,'EPSG','4400','EPSG','4326','EPSG','16138',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7617','projected_crs','EPSG','32738','EPSG','2075','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32739','WGS 84 / UTM zone 39S',NULL,'EPSG','4400','EPSG','4326','EPSG','16139',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7618','projected_crs','EPSG','32739','EPSG','2077','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32740','WGS 84 / UTM zone 40S',NULL,'EPSG','4400','EPSG','4326','EPSG','16140',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7619','projected_crs','EPSG','32740','EPSG','2079','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32741','WGS 84 / UTM zone 41S',NULL,'EPSG','4400','EPSG','4326','EPSG','16141',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7620','projected_crs','EPSG','32741','EPSG','2081','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32742','WGS 84 / UTM zone 42S',NULL,'EPSG','4400','EPSG','4326','EPSG','16142',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7621','projected_crs','EPSG','32742','EPSG','2083','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32743','WGS 84 / UTM zone 43S',NULL,'EPSG','4400','EPSG','4326','EPSG','16143',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7622','projected_crs','EPSG','32743','EPSG','2085','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32744','WGS 84 / UTM zone 44S',NULL,'EPSG','4400','EPSG','4326','EPSG','16144',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7623','projected_crs','EPSG','32744','EPSG','2087','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32745','WGS 84 / UTM zone 45S',NULL,'EPSG','4400','EPSG','4326','EPSG','16145',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7624','projected_crs','EPSG','32745','EPSG','2089','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32746','WGS 84 / UTM zone 46S',NULL,'EPSG','4400','EPSG','4326','EPSG','16146',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7625','projected_crs','EPSG','32746','EPSG','2091','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32747','WGS 84 / UTM zone 47S',NULL,'EPSG','4400','EPSG','4326','EPSG','16147',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7626','projected_crs','EPSG','32747','EPSG','2093','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32748','WGS 84 / UTM zone 48S',NULL,'EPSG','4400','EPSG','4326','EPSG','16148',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7627','projected_crs','EPSG','32748','EPSG','2095','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32749','WGS 84 / UTM zone 49S',NULL,'EPSG','4400','EPSG','4326','EPSG','16149',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7628','projected_crs','EPSG','32749','EPSG','2097','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32750','WGS 84 / UTM zone 50S',NULL,'EPSG','4400','EPSG','4326','EPSG','16150',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7629','projected_crs','EPSG','32750','EPSG','2099','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32751','WGS 84 / UTM zone 51S',NULL,'EPSG','4400','EPSG','4326','EPSG','16151',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7630','projected_crs','EPSG','32751','EPSG','2101','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32752','WGS 84 / UTM zone 52S',NULL,'EPSG','4400','EPSG','4326','EPSG','16152',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7631','projected_crs','EPSG','32752','EPSG','2103','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32753','WGS 84 / UTM zone 53S',NULL,'EPSG','4400','EPSG','4326','EPSG','16153',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7632','projected_crs','EPSG','32753','EPSG','2105','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32754','WGS 84 / UTM zone 54S',NULL,'EPSG','4400','EPSG','4326','EPSG','16154',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7633','projected_crs','EPSG','32754','EPSG','2107','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32755','WGS 84 / UTM zone 55S',NULL,'EPSG','4400','EPSG','4326','EPSG','16155',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7634','projected_crs','EPSG','32755','EPSG','2109','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32756','WGS 84 / UTM zone 56S',NULL,'EPSG','4400','EPSG','4326','EPSG','16156',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7635','projected_crs','EPSG','32756','EPSG','2111','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32757','WGS 84 / UTM zone 57S',NULL,'EPSG','4400','EPSG','4326','EPSG','16157',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7636','projected_crs','EPSG','32757','EPSG','2113','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32758','WGS 84 / UTM zone 58S',NULL,'EPSG','4400','EPSG','4326','EPSG','16158',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7637','projected_crs','EPSG','32758','EPSG','2115','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32759','WGS 84 / UTM zone 59S',NULL,'EPSG','4400','EPSG','4326','EPSG','16159',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7638','projected_crs','EPSG','32759','EPSG','2117','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32760','WGS 84 / UTM zone 60S',NULL,'EPSG','4400','EPSG','4326','EPSG','16160',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7639','projected_crs','EPSG','32760','EPSG','2119','EPSG','1279');
INSERT INTO "projected_crs" VALUES('EPSG','32761','WGS 84 / UPS South (N,E)',NULL,'EPSG','4494','EPSG','4326','EPSG','16161',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7640','projected_crs','EPSG','32761','EPSG','1997','EPSG','1160');
INSERT INTO "projected_crs" VALUES('EPSG','32766','WGS 84 / TM 36 SE',NULL,'EPSG','4400','EPSG','4326','EPSG','16636',NULL,0);
INSERT INTO "usage" VALUES('EPSG','7641','projected_crs','EPSG','32766','EPSG','1726','EPSG','1136');
