--- This file has been generated by scripts/build_db.py. DO NOT EDIT !

INSERT INTO "conversion" VALUES('EPSG','3811','Belgian Lambert 2008','Replaces Lambert 2005.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',50.4752134,'EPSG','9110','EPSG','8822','Longitude of false origin',4.2133177,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',49.5,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',51.1,'EPSG','9110','EPSG','8826','Easting at false origin',649328.0,'EPSG','9001','EPSG','8827','Northing at false origin',665262.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','8939','conversion','EPSG','3811','EPSG','1347','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','3813','Mississippi Transverse Mercator','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',32.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-89.45,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9998335,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',1300000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','8940','conversion','EPSG','3813','EPSG','1393','EPSG','1135');
INSERT INTO "conversion" VALUES('EPSG','3818','Taiwan 2-degree TM zone 119','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',119.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','8942','conversion','EPSG','3818','EPSG','3563','EPSG','1056');
INSERT INTO "conversion" VALUES('EPSG','3820','Taiwan 2-degree TM zone 121','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',121.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','8943','conversion','EPSG','3820','EPSG','3562','EPSG','1056');
INSERT INTO "conversion" VALUES('EPSG','3831','Pacific Disaster Center Mercator','','EPSG','9804','Mercator (variant A)','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',150.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','8945','conversion','EPSG','3831','EPSG','3172','EPSG','1241');
INSERT INTO "conversion" VALUES('EPSG','3853','County ST74','In Stockholm commune, replaces ST74 (proj code 19876).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',18.0328332,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.99999506,'EPSG','9201','EPSG','8806','False easting',100182.7406,'EPSG','9001','EPSG','8807','False northing',-6500620.1207,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','8946','conversion','EPSG','3853','EPSG','3608','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','3856','Popular Visualisation Pseudo-Mercator','','EPSG','1024','Popular Visualisation Pseudo Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',0.0,'EPSG','9102','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','8947','conversion','EPSG','3856','EPSG','1262','EPSG','1098');
INSERT INTO "conversion" VALUES('EPSG','3860','Finland Gauss-Kruger zone 19','Replaces Finland ETRS-GK19 (proj code 18183).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',19.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',19500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','8950','conversion','EPSG','3860','EPSG','3595','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','3861','Finland Gauss-Kruger zone 20','Replaces Finland ETRS-GK20 (proj code 18184).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',20.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',20500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','8951','conversion','EPSG','3861','EPSG','3596','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','3862','Finland Gauss-Kruger zone 21','Replaces Finland ETRS-GK21 (proj code 18185).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',21.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',21500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','8952','conversion','EPSG','3862','EPSG','3597','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','3863','Finland Gauss-Kruger zone 22','Replaces Finland ETRS-GK22 (proj code 18186).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',22.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',22500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','8953','conversion','EPSG','3863','EPSG','3598','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','3864','Finland Gauss-Kruger zone 23','Replaces Finland ETRS-GK23 (proj code 18187).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',23.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',23500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','8954','conversion','EPSG','3864','EPSG','3599','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','3865','Finland Gauss-Kruger zone 24','Replaces Finland ETRS-GK24 (proj code 18188).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',24.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',24500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','8955','conversion','EPSG','3865','EPSG','3600','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','3866','Finland Gauss-Kruger zone 25','Replaces Finland ETRS-GK25 (proj code 18189).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',25.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',25500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','8956','conversion','EPSG','3866','EPSG','3601','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','3867','Finland Gauss-Kruger zone 26','Replaces Finland ETRS-GK26 (proj code 18190).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',26.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',26500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','8957','conversion','EPSG','3867','EPSG','3602','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','3868','Finland Gauss-Kruger zone 27','Replaces Finland ETRS-GK27 (proj code 18195).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',27.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',27500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','8958','conversion','EPSG','3868','EPSG','3603','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','3869','Finland Gauss-Kruger zone 28','Replaces Finland ETRS-GK28 (proj code 18196).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',28.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',28500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','8959','conversion','EPSG','3869','EPSG','3604','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','3870','Finland Gauss-Kruger zone 29','Replaces Finland ETRS-GK29 (proj code 18197).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',29.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',29500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','8960','conversion','EPSG','3870','EPSG','3605','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','3871','Finland Gauss-Kruger zone 30','Replaces Finland ETRS-GK30 (proj code 18198).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',30.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',30500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','8961','conversion','EPSG','3871','EPSG','3606','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','3872','Finland Gauss-Kruger zone 31','Replaces Finland ETRS-GK31 (proj code 18199).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',31.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',31500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','8962','conversion','EPSG','3872','EPSG','3607','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','3897','US NSIDC Equal Area north projection','See information source for equations to define Equal-Area Scalable Earth Grid (EASE-Grid) overlay.','EPSG','1027','Lambert Azimuthal Equal Area (Spherical)','EPSG','8801','Latitude of natural origin',90.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',0.0,'EPSG','9102','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','8966','conversion','EPSG','3897','EPSG','3475','EPSG','1195');
INSERT INTO "conversion" VALUES('EPSG','3898','US NSIDC Equal Area south projection','See information source for equations to define Equal-Area Scalable Earth Grid (EASE-Grid) overlay.','EPSG','1027','Lambert Azimuthal Equal Area (Spherical)','EPSG','8801','Latitude of natural origin',-90.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',0.0,'EPSG','9102','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','8967','conversion','EPSG','3898','EPSG','3474','EPSG','1195');
INSERT INTO "conversion" VALUES('EPSG','3899','US National Atlas Equal Area','','EPSG','1027','Lambert Azimuthal Equal Area (Spherical)','EPSG','8801','Latitude of natural origin',45.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-100.0,'EPSG','9102','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','8968','conversion','EPSG','3899','EPSG','1245','EPSG','1162');
INSERT INTO "conversion" VALUES('EPSG','3967','Virginia Lambert Conic Conformal','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',36.0,'EPSG','9102','EPSG','8822','Longitude of false origin',-79.5,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',37.0,'EPSG','9102','EPSG','8824','Latitude of 2nd standard parallel',39.5,'EPSG','9102','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9015','conversion','EPSG','3967','EPSG','1415','EPSG','1135');
INSERT INTO "conversion" VALUES('EPSG','3977','Canada Atlas Lambert','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',49.0,'EPSG','9102','EPSG','8822','Longitude of false origin',-95.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',49.0,'EPSG','9102','EPSG','8824','Latitude of 2nd standard parallel',77.0,'EPSG','9102','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9018','conversion','EPSG','3977','EPSG','1061','EPSG','1046');
INSERT INTO "conversion" VALUES('EPSG','3980','Katanga Lambert','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',9.0,'EPSG','9102','EPSG','8822','Longitude of false origin',26.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-6.5,'EPSG','9102','EPSG','8824','Latitude of 2nd standard parallel',-11.5,'EPSG','9102','EPSG','8826','Easting at false origin',500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','9019','conversion','EPSG','3980','EPSG','3147','EPSG','1210');
INSERT INTO "conversion" VALUES('EPSG','3981','Katanga Gauss zone A','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-9.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',30.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9020','conversion','EPSG','3981','EPSG','3612','EPSG','1054');
INSERT INTO "conversion" VALUES('EPSG','3982','Katanga Gauss zone B','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-9.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',28.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9021','conversion','EPSG','3982','EPSG','3611','EPSG','1054');
INSERT INTO "conversion" VALUES('EPSG','3983','Katanga Gauss zone C','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-9.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',26.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9022','conversion','EPSG','3983','EPSG','3610','EPSG','1054');
INSERT INTO "conversion" VALUES('EPSG','3984','Katanga Gauss zone D','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-9.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',24.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9023','conversion','EPSG','3984','EPSG','3609','EPSG','1054');
INSERT INTO "conversion" VALUES('EPSG','3999','Moldova Transverse Mercator','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',28.24,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.99994,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',-5000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9026','conversion','EPSG','3999','EPSG','1162','EPSG','1054');
INSERT INTO "conversion" VALUES('EPSG','4085','World Equidistant Cylindrical','Origin at intersection of equator and prime meridian. See projection code 4086 for spherical development.','EPSG','1028','Equidistant Cylindrical','EPSG','8823','Latitude of 1st standard parallel',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',0.0,'EPSG','9102','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9039','conversion','EPSG','4085','EPSG','1262','EPSG','1192');
INSERT INTO "conversion" VALUES('EPSG','4086','World Equidistant Cylindrical (Sphere)','Origin at intersection of equator and prime meridian.','EPSG','1029','Equidistant Cylindrical (Spherical)','EPSG','8823','Latitude of 1st standard parallel',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',0.0,'EPSG','9102','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9040','conversion','EPSG','4086','EPSG','1262','EPSG','1192');
INSERT INTO "conversion" VALUES('EPSG','4089','DKTM1','Introduced in 2009.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',9.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.99998,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',-5000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9041','conversion','EPSG','4089','EPSG','3631','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','4090','DKTM2','Introduced in 2009.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',10.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.99998,'EPSG','9201','EPSG','8806','False easting',400000.0,'EPSG','9001','EPSG','8807','False northing',-5000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9042','conversion','EPSG','4090','EPSG','3632','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','4091','DKTM3','Introduced in 2009.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',11.75,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.99998,'EPSG','9201','EPSG','8806','False easting',600000.0,'EPSG','9001','EPSG','8807','False northing',-5000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9043','conversion','EPSG','4091','EPSG','2532','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','4092','DKTM4','Introduced in 2009.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',15.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',800000.0,'EPSG','9001','EPSG','8807','False northing',-5000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9044','conversion','EPSG','4092','EPSG','2533','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','4101','BLM zone 1N (US survey foot)','US survey foot form of UTM zone 1N. Sometimes locally referred to as "UTM zone 1".','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-177.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',1640416.67,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9045','conversion','EPSG','4101','EPSG','3374','EPSG','1153');
INSERT INTO "conversion" VALUES('EPSG','4102','BLM zone 2N (US survey foot)','US survey foot form of UTM zone 2N. Sometimes locally referred to as "UTM zone 2".','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-171.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',1640416.67,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9046','conversion','EPSG','4102','EPSG','3375','EPSG','1153');
INSERT INTO "conversion" VALUES('EPSG','4103','BLM zone 3N (US survey foot)','US survey foot form of UTM zone 3N. Sometimes locally referred to as "UTM zone 3".','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-165.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',1640416.67,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9047','conversion','EPSG','4103','EPSG','2133','EPSG','1153');
INSERT INTO "conversion" VALUES('EPSG','4104','BLM zone 4N (US survey foot)','US survey foot form of UTM zone 4N. Sometimes locally referred to as "UTM zone 4".','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-159.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',1640416.67,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9048','conversion','EPSG','4104','EPSG','2134','EPSG','1153');
INSERT INTO "conversion" VALUES('EPSG','4105','BLM zone 5N (US survey foot)','US survey foot form of UTM zone 5N. Sometimes locally referred to as "UTM zone 5".','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-153.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',1640416.67,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9049','conversion','EPSG','4105','EPSG','2135','EPSG','1153');
INSERT INTO "conversion" VALUES('EPSG','4106','BLM zone 6N (US survey foot)','US survey foot form of UTM zone 6N. Sometimes locally referred to as "UTM zone 6".','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-147.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',1640416.67,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9050','conversion','EPSG','4106','EPSG','2136','EPSG','1153');
INSERT INTO "conversion" VALUES('EPSG','4107','BLM zone 7N (US survey foot)','US survey foot form of UTM zone 7N. Sometimes locally referred to as "UTM zone 7".','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-141.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',1640416.67,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9051','conversion','EPSG','4107','EPSG','3494','EPSG','1153');
INSERT INTO "conversion" VALUES('EPSG','4108','BLM zone 8N (US survey foot)','US survey foot form of UTM zone 8N. Sometimes locally referred to as "UTM zone 8".','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-135.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',1640416.67,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9052','conversion','EPSG','4108','EPSG','3495','EPSG','1153');
INSERT INTO "conversion" VALUES('EPSG','4109','BLM zone 9N (US survey foot)','US survey foot form of UTM zone 9N. Sometimes locally referred to as "UTM zone 9".','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-129.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',1640416.67,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9053','conversion','EPSG','4109','EPSG','3496','EPSG','1153');
INSERT INTO "conversion" VALUES('EPSG','4110','BLM zone 10N (US survey foot)','US survey foot form of UTM zone 10N. Sometimes locally referred to as "UTM zone 10".','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-123.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',1640416.67,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9054','conversion','EPSG','4110','EPSG','3497','EPSG','1153');
INSERT INTO "conversion" VALUES('EPSG','4111','BLM zone 11N (US survey foot)','US survey foot form of UTM zone 11N. Sometimes locally referred to as "UTM zone 11".','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-117.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',1640416.67,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9055','conversion','EPSG','4111','EPSG','3498','EPSG','1153');
INSERT INTO "conversion" VALUES('EPSG','4112','BLM zone 12N (US survey foot)','US survey foot form of UTM zone 12N. Sometimes locally referred to as "UTM zone 12".','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-111.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',1640416.67,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9056','conversion','EPSG','4112','EPSG','3499','EPSG','1153');
INSERT INTO "conversion" VALUES('EPSG','4113','BLM zone 13N (US survey foot)','US survey foot form of UTM zone 13N. Sometimes locally referred to as "UTM zone 13".','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-105.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',1640416.67,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9057','conversion','EPSG','4113','EPSG','3500','EPSG','1153');
INSERT INTO "conversion" VALUES('EPSG','4114','Johor Cassini Grid','Origin is Gunung Blumut. Replaced by GDM2000 Johor grid (code 19893).','EPSG','9806','Cassini-Soldner','EPSG','8801','Latitude of natural origin',2.02333,'EPSG','9110','EPSG','8802','Longitude of natural origin',103.334593,'EPSG','9110','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9058','conversion','EPSG','4114','EPSG','3376','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','4115','Sembilan and Melaka Cassini Grid','Origin is Gun Hill. Replaced by GDM2000 Sembilan and Melaka grid (code 19892).','EPSG','9806','Cassini-Soldner','EPSG','8801','Latitude of natural origin',2.424422,'EPSG','9110','EPSG','8802','Longitude of natural origin',101.56282,'EPSG','9110','EPSG','8806','False easting',-242.005,'EPSG','9001','EPSG','8807','False northing',-948.547,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9059','conversion','EPSG','4115','EPSG','3377','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','4116','Pahang Cassini Grid','Origin is Gunung Sinyum. Replaced by GDM2000 Pahang grid (code 19891).','EPSG','9806','Cassini-Soldner','EPSG','8801','Latitude of natural origin',3.42395,'EPSG','9110','EPSG','8802','Longitude of natural origin',102.261024,'EPSG','9110','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9060','conversion','EPSG','4116','EPSG','3378','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','4117','Selangor Cassini Grid','Origin is Bukit Asa. Replaced by GDM2000 Selangor grid (code 19890).','EPSG','9806','Cassini-Soldner','EPSG','8801','Latitude of natural origin',3.404924,'EPSG','9110','EPSG','8802','Longitude of natural origin',101.302968,'EPSG','9110','EPSG','8806','False easting',-21759.438,'EPSG','9001','EPSG','8807','False northing',55960.906,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9061','conversion','EPSG','4117','EPSG','3379','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','4118','BLM zone 18N (US survey foot)','Sometimes locally referred to as "UTM zone 18".','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-75.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',1640416.67,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9062','conversion','EPSG','4118','EPSG','3505','EPSG','1153');
INSERT INTO "conversion" VALUES('EPSG','4119','BLM zone 19N (US survey foot)','Sometimes locally referred to as "UTM zone 19".','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-69.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',1640416.67,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9063','conversion','EPSG','4119','EPSG','3506','EPSG','1153');
INSERT INTO "conversion" VALUES('EPSG','4177','Terengganu Cassini Grid','Origin is Gunung Gajah Trom. Replaced by GDM2000 Terengganu grid (code 19889).','EPSG','9806','Cassini-Soldner','EPSG','8801','Latitude of natural origin',4.564611,'EPSG','9110','EPSG','8802','Longitude of natural origin',102.534275,'EPSG','9110','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9064','conversion','EPSG','4177','EPSG','3380','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','4186','BLM zone 59N (US survey foot)','Sometimes locally referred to as "UTM zone 59".','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',171.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',1640416.67,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9065','conversion','EPSG','4186','EPSG','3372','EPSG','1153');
INSERT INTO "conversion" VALUES('EPSG','4187','BLM zone 60N (US survey foot)','Sometimes locally referred to as "UTM zone 60".','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',177.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',1640416.67,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9066','conversion','EPSG','4187','EPSG','3373','EPSG','1153');
INSERT INTO "conversion" VALUES('EPSG','4305','Pinang Cassini Grid','Origin is Fort Cornwallis. Replaced by GDM2000 Pinang grid (code 19888).','EPSG','9806','Cassini-Soldner','EPSG','8801','Latitude of natural origin',5.251677,'EPSG','9110','EPSG','8802','Longitude of natural origin',100.204513,'EPSG','9110','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9068','conversion','EPSG','4305','EPSG','3381','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','4320','Kedah and Perlis Cassini Grid','Origin is Gunung Perak. Replaced by GDM2000 Kedah and Perlis grid (code 19887).','EPSG','9806','Cassini-Soldner','EPSG','8801','Latitude of natural origin',5.575453,'EPSG','9110','EPSG','8802','Longitude of natural origin',100.381534,'EPSG','9110','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9069','conversion','EPSG','4320','EPSG','3382','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','4321','Perak Revised Cassini Grid','Origin is Gunung Hijau Larut. Replaces Perak Cassini grid. Replaced by GDM2000 Perak grid (code 19886).','EPSG','9806','Cassini-Soldner','EPSG','8801','Latitude of natural origin',4.513377,'EPSG','9110','EPSG','8802','Longitude of natural origin',100.490036,'EPSG','9110','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',133453.669,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9070','conversion','EPSG','4321','EPSG','3383','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','4323','Kelantan Cassini Grid','Origin is Bukit Panau. Replaced by GDM2000 Kelantan grid (code 19885).','EPSG','9806','Cassini-Soldner','EPSG','8801','Latitude of natural origin',5.533812,'EPSG','9110','EPSG','8802','Longitude of natural origin',102.103825,'EPSG','9110','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9071','conversion','EPSG','4323','EPSG','3384','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','4325','Guam Map Grid','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',13.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',144.45,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',100000.0,'EPSG','9001','EPSG','8807','False northing',200000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9072','conversion','EPSG','4325','EPSG','3255','EPSG','1056');
INSERT INTO "conversion" VALUES('EPSG','4416','Katanga Lambert','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-9.0,'EPSG','9102','EPSG','8822','Longitude of false origin',26.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-6.5,'EPSG','9102','EPSG','8824','Latitude of 2nd standard parallel',-11.5,'EPSG','9102','EPSG','8826','Easting at false origin',500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9073','conversion','EPSG','4416','EPSG','3147','EPSG','1210');
INSERT INTO "conversion" VALUES('EPSG','4436','Pennsylvania CS27 South zone','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',39.2,'EPSG','9110','EPSG','8822','Longitude of false origin',-77.45,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',40.58,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',39.56,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9075','conversion','EPSG','4436','EPSG','2246','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','4454','New York CS27 Long Island zone','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',40.3,'EPSG','9110','EPSG','8822','Longitude of false origin',-74.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',41.02,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',40.4,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',100000.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9089','conversion','EPSG','4454','EPSG','2235','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','4460','Australian Centre for Remote Sensing Lambert Conformal Projection','Created by AUSLIG prior to incorporation into Geoscience Australia. See also GALCC (proj code 17362).','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-27.0,'EPSG','9102','EPSG','8822','Longitude of false origin',132.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-18.0,'EPSG','9102','EPSG','8824','Latitude of 2nd standard parallel',-36.0,'EPSG','9102','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9091','conversion','EPSG','4460','EPSG','2575','EPSG','1047');
INSERT INTO "conversion" VALUES('EPSG','4648','UTM zone 32N with prefix','Variant of UTM zone 32N (code 16032) with zone number prefixed to easting.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',9.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',32500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9115','conversion','EPSG','4648','EPSG','2861','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','4825','Cape Verde National','Adopted October 2004.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',15.5,'EPSG','9110','EPSG','8822','Longitude of false origin',-24.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',15.0,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',16.4,'EPSG','9110','EPSG','8826','Easting at false origin',161587.83,'EPSG','9001','EPSG','8827','Northing at false origin',128511.202,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9119','conversion','EPSG','4825','EPSG','1062','EPSG','1153');
INSERT INTO "conversion" VALUES('EPSG','4838','LCC Germany','Variant for Germany of LCC Europe (proj code 19985). Gauss-Kruger used for applications at scales larger than 1:500,000.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',51.0,'EPSG','9102','EPSG','8822','Longitude of false origin',10.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',48.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',53.4,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9131','conversion','EPSG','4838','EPSG','3339','EPSG','1241');
INSERT INTO "conversion" VALUES('EPSG','4841','Norway TM zone 15','UTM used as official map projection. This is an official alternative for construction projects.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',15.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',100000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','9133','conversion','EPSG','4841','EPSG','3656','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','4842','Norway TM zone 16','UTM used as official map projection. This is an official alternative for construction projects.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',16.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',100000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','9134','conversion','EPSG','4842','EPSG','3657','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','4843','Norway TM zone 17','UTM used as official map projection. This is an official alternative for construction projects.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',17.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',100000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','9135','conversion','EPSG','4843','EPSG','3658','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','4844','Norway TM zone 18','UTM used as official map projection. This is an official alternative for construction projects.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',18.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',100000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','9136','conversion','EPSG','4844','EPSG','3660','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','4845','Norway TM zone 5','UTM used as official map projection. This is an official alternative for construction projects.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',5.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',100000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','9137','conversion','EPSG','4845','EPSG','3636','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','4846','Norway TM zone 6','UTM used as official map projection. This is an official alternative for construction projects.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',6.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',100000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','9138','conversion','EPSG','4846','EPSG','3639','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','4847','Norway TM zone 7','UTM used as official map projection. This is an official alternative for construction projects.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',7.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',100000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','9139','conversion','EPSG','4847','EPSG','3647','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','4848','Norway TM zone 8','UTM used as official map projection. This is an official alternative for construction projects.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',8.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',100000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','9140','conversion','EPSG','4848','EPSG','3648','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','4849','Norway TM zone 9','UTM used as official map projection. This is an official alternative for construction projects.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',9.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',100000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','9141','conversion','EPSG','4849','EPSG','3649','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','4850','Norway TM zone 10','UTM used as official map projection. This is an official alternative for construction projects.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',10.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',100000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','9142','conversion','EPSG','4850','EPSG','3650','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','4851','Norway TM zone 11','UTM used as official map projection. This is an official alternative for construction projects.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',11.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',100000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','9143','conversion','EPSG','4851','EPSG','3651','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','4852','Norway TM zone 12','UTM used as official map projection. This is an official alternative for construction projects.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',12.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',100000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','9144','conversion','EPSG','4852','EPSG','3653','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','4853','Norway TM zone 13','UTM used as official map projection. This is an official alternative for construction projects.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',13.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',100000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','9145','conversion','EPSG','4853','EPSG','3654','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','4854','Norway TM zone 14','UTM used as official map projection. This is an official alternative for construction projects.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',14.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',100000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','9146','conversion','EPSG','4854','EPSG','3655','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','4881','Norway TM zone 19','UTM used as official map projection. This is an official alternative for construction projects.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',19.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',100000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','9147','conversion','EPSG','4881','EPSG','3661','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','5000','Norway TM zone 20','UTM used as official map projection. This is an official alternative for construction projects.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',20.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',100000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','9149','conversion','EPSG','5000','EPSG','3662','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','5001','Norway TM zone 21','UTM used as official map projection. This is an official alternative for construction projects.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',21.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',100000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','9150','conversion','EPSG','5001','EPSG','3663','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','5002','Norway TM zone 22','UTM used as official map projection. This is an official alternative for construction projects.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',22.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',100000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','9151','conversion','EPSG','5002','EPSG','3665','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','5003','Norway TM zone 23','UTM used as official map projection. This is an official alternative for construction projects.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',23.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',100000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','9152','conversion','EPSG','5003','EPSG','3667','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','5004','Norway TM zone 24','UTM used as official map projection. This is an official alternative for construction projects.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',24.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',100000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','9153','conversion','EPSG','5004','EPSG','3668','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','5005','Norway TM zone 25','UTM used as official map projection. This is an official alternative for construction projects.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',25.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',100000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','9154','conversion','EPSG','5005','EPSG','3669','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','5006','Norway TM zone 26','UTM used as official map projection. This is an official alternative for construction projects.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',26.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',100000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','9155','conversion','EPSG','5006','EPSG','3671','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','5007','Norway TM zone 27','UTM used as official map projection. This is an official alternative for construction projects.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',27.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',100000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','9156','conversion','EPSG','5007','EPSG','3672','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','5008','Norway TM zone 28','UTM used as official map projection. This is an official alternative for construction projects.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',28.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',100000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','9157','conversion','EPSG','5008','EPSG','3673','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','5009','Norway TM zone 29','UTM used as official map projection. This is an official alternative for construction projects.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',29.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',100000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','9158','conversion','EPSG','5009','EPSG','3674','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','5010','Norway TM zone 30','UTM used as official map projection. This is an official alternative for construction projects.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',30.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',100000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','9159','conversion','EPSG','5010','EPSG','3676','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','5019','Portugal Bonne New','Replaces original definition referenced to Lisbon meridian (proj code 19979).','EPSG','9828','Bonne (South Orientated)','EPSG','8801','Latitude of natural origin',39.4,'EPSG','9110','EPSG','8802','Longitude of natural origin',-8.0754862,'EPSG','9110','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9160','conversion','EPSG','5019','EPSG','1294','EPSG','1062');
INSERT INTO "conversion" VALUES('EPSG','5020','Portuguese Grid New','Replaces original definition referenced to Lisbon meridian (proj code 19969).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',39.4,'EPSG','9110','EPSG','8802','Longitude of natural origin',-8.0754862,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9161','conversion','EPSG','5020','EPSG','1294','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','5049','Korea East Sea Belt','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',38.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',131.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9187','conversion','EPSG','5049','EPSG','3727','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','5068','Conus Albers','','EPSG','9822','Albers Equal Area','EPSG','8821','Latitude of false origin',23.0,'EPSG','9102','EPSG','8822','Longitude of false origin',-96.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',29.3,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',45.3,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9206','conversion','EPSG','5068','EPSG','1323','EPSG','1241');
INSERT INTO "conversion" VALUES('EPSG','5100','Korea Unified Belt','Introduced in 2004.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',38.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',127.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',1000000.0,'EPSG','9001','EPSG','8807','False northing',2000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9234','conversion','EPSG','5100','EPSG','1135','EPSG','1241');
INSERT INTO "conversion" VALUES('EPSG','5101','Korea West Belt 2010','Introduced from 2010-01-01. Officially defined as extending between 124°E and 126°E.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',38.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',125.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',600000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9235','conversion','EPSG','5101','EPSG','1498','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','5102','Korea Central Belt 2010','Introduced from 2010-01-01.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',38.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',127.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',600000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9236','conversion','EPSG','5102','EPSG','1497','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','5103','Korea East Belt 2010','Introduced from 2010-01-01.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',38.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',129.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',600000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9237','conversion','EPSG','5103','EPSG','1496','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','5104','Korea East Sea Belt 2010','Introduced from 2010-01-01. Officially defined as extending between 130°E and 132°E.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',38.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',131.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',600000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9238','conversion','EPSG','5104','EPSG','3720','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','5131','Korea Central Belt Jeju','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',38.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',127.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',550000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9239','conversion','EPSG','5131','EPSG','3721','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','5135','Norway TM zone 5','UTM used as official map projection. This is an official alternative for construction projects.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',58.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',5.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',100000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9242','conversion','EPSG','5135','EPSG','3636','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','5136','Norway TM zone 6','UTM used as official map projection. This is an official alternative for construction projects.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',58.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',6.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',100000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9243','conversion','EPSG','5136','EPSG','3639','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','5137','Norway TM zone 7','UTM used as official map projection. This is an official alternative for construction projects.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',58.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',7.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',100000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9244','conversion','EPSG','5137','EPSG','3647','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','5138','Norway TM zone 8','UTM used as official map projection. This is an official alternative for construction projects.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',58.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',8.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',100000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9245','conversion','EPSG','5138','EPSG','3648','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','5139','Norway TM zone 9','UTM used as official map projection. This is an official alternative for construction projects.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',58.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',9.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',100000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9246','conversion','EPSG','5139','EPSG','3649','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','5140','Norway TM zone 10','UTM used as official map projection. This is an official alternative for construction projects.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',58.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',10.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',100000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9247','conversion','EPSG','5140','EPSG','3650','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','5141','Norway TM zone 11','UTM used as official map projection. This is an official alternative for construction projects.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',58.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',11.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',100000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9248','conversion','EPSG','5141','EPSG','3651','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','5142','Norway TM zone 12','UTM used as official map projection. This is an official alternative for construction projects.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',58.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',12.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',100000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9249','conversion','EPSG','5142','EPSG','3653','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','5143','Norway TM zone 13','UTM used as official map projection. This is an official alternative for construction projects.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',58.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',13.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',100000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9250','conversion','EPSG','5143','EPSG','3654','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','5144','Norway TM zone 14','UTM used as official map projection. This is an official alternative for construction projects.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',58.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',14.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',100000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9251','conversion','EPSG','5144','EPSG','3655','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','5145','Norway TM zone 15','UTM used as official map projection. This is an official alternative for construction projects.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',58.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',15.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',100000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9252','conversion','EPSG','5145','EPSG','3656','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','5146','Norway TM zone 16','UTM used as official map projection. This is an official alternative for construction projects.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',58.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',16.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',100000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9253','conversion','EPSG','5146','EPSG','3657','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','5147','Norway TM zone 17','UTM used as official map projection. This is an official alternative for construction projects.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',58.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',17.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',100000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9254','conversion','EPSG','5147','EPSG','3658','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','5148','Norway TM zone 18','UTM used as official map projection. This is an official alternative for construction projects.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',58.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',18.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',100000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9255','conversion','EPSG','5148','EPSG','3660','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','5149','Norway TM zone 19','UTM used as official map projection. This is an official alternative for construction projects.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',58.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',19.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',100000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9256','conversion','EPSG','5149','EPSG','3661','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','5150','Norway TM zone 20','UTM used as official map projection. This is an official alternative for construction projects.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',58.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',20.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',100000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9257','conversion','EPSG','5150','EPSG','3662','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','5151','Norway TM zone 21','UTM used as official map projection. This is an official alternative for construction projects.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',58.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',21.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',100000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9258','conversion','EPSG','5151','EPSG','3663','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','5152','Norway TM zone 22','UTM used as official map projection. This is an official alternative for construction projects.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',58.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',22.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',100000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9259','conversion','EPSG','5152','EPSG','3665','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','5153','Norway TM zone 23','UTM used as official map projection. This is an official alternative for construction projects.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',58.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',23.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',100000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9260','conversion','EPSG','5153','EPSG','3667','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','5154','Norway TM zone 24','UTM used as official map projection. This is an official alternative for construction projects.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',58.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',24.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',100000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9261','conversion','EPSG','5154','EPSG','3668','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','5155','Norway TM zone 25','UTM used as official map projection. This is an official alternative for construction projects.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',58.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',25.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',100000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9262','conversion','EPSG','5155','EPSG','3669','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','5156','Norway TM zone 26','UTM used as official map projection. This is an official alternative for construction projects.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',58.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',26.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',100000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9263','conversion','EPSG','5156','EPSG','3671','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','5157','Norway TM zone 27','UTM used as official map projection. This is an official alternative for construction projects.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',58.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',27.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',100000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9264','conversion','EPSG','5157','EPSG','3672','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','5158','Norway TM zone 28','UTM used as official map projection. This is an official alternative for construction projects.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',58.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',28.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',100000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9265','conversion','EPSG','5158','EPSG','3673','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','5159','Norway TM zone 29','UTM used as official map projection. This is an official alternative for construction projects.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',58.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',29.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',100000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9266','conversion','EPSG','5159','EPSG','3674','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','5160','Norway TM zone 30','UTM used as official map projection. This is an official alternative for construction projects.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',58.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',30.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',100000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9267','conversion','EPSG','5160','EPSG','3676','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','5161','Korea Modified West Belt','Introduced in 1998.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',38.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',125.0010405,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9268','conversion','EPSG','5161','EPSG','1498','EPSG','1092');
INSERT INTO "conversion" VALUES('EPSG','5162','Korea Modified Central Belt','Introduced in 1998.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',38.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',127.0010405,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9269','conversion','EPSG','5162','EPSG','1497','EPSG','1092');
INSERT INTO "conversion" VALUES('EPSG','5163','Korea Modified Central Belt Jeju','Introduced in 1998.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',38.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',127.0010405,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',550000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9270','conversion','EPSG','5163','EPSG','3721','EPSG','1092');
INSERT INTO "conversion" VALUES('EPSG','5164','Korea Modified East Belt','Introduced in 1998.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',38.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',129.0010405,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9271','conversion','EPSG','5164','EPSG','1496','EPSG','1092');
INSERT INTO "conversion" VALUES('EPSG','5165','Korea Modified East Sea Belt','Introduced in 1998.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',38.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',131.0010405,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9272','conversion','EPSG','5165','EPSG','3720','EPSG','1092');
INSERT INTO "conversion" VALUES('EPSG','5218','Krovak East North','Longitude is referenced to the Ferro meridian. North-orientated alternative to south-orientated Krovak projection (code 19952) introduced for GIS purposes.','EPSG','1041','Krovak (North Orientated)','EPSG','8811','Latitude of projection centre',49.3,'EPSG','9110','EPSG','8833','Longitude of origin',42.3,'EPSG','9110','EPSG','1036','Co-latitude of cone axis',30.171730311,'EPSG','9110','EPSG','8818','Latitude of pseudo standard parallel',78.3,'EPSG','9110','EPSG','8819','Scale factor on pseudo standard parallel',0.9999,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',0);
INSERT INTO "usage" VALUES('EPSG','9300','conversion','EPSG','5218','EPSG','1306','EPSG','1189');
INSERT INTO "conversion" VALUES('EPSG','5219','Modified Krovak','Longitude is referenced to the Ferro meridian.','EPSG','1042','Krovak Modified','EPSG','8811','Latitude of projection centre',49.3,'EPSG','9110','EPSG','8833','Longitude of origin',42.3,'EPSG','9110','EPSG','1036','Co-latitude of cone axis',30.1717303,'EPSG','9110','EPSG','8818','Latitude of pseudo standard parallel',78.3,'EPSG','9110','EPSG','8819','Scale factor on pseudo standard parallel',0.9999,'EPSG','9201','EPSG','8806','False easting',5000000.0,'EPSG','9001','EPSG','8807','False northing',5000000.0,'EPSG','9001',0);
INSERT INTO "usage" VALUES('EPSG','9301','conversion','EPSG','5219','EPSG','1079','EPSG','1092');
INSERT INTO "conversion" VALUES('EPSG','5220','Modified Krovak East North','Longitude is referenced to the Ferro meridian. Replaces Modified Krovak south-orientated projection (code 5219) for GIS purposes.','EPSG','1043','Krovak Modified (North Orientated)','EPSG','8811','Latitude of projection centre',49.3,'EPSG','9110','EPSG','8833','Longitude of origin',42.3,'EPSG','9110','EPSG','1036','Co-latitude of cone axis',30.1717303,'EPSG','9110','EPSG','8818','Latitude of pseudo standard parallel',78.3,'EPSG','9110','EPSG','8819','Scale factor on pseudo standard parallel',0.9999,'EPSG','9201','EPSG','8806','False easting',5000000.0,'EPSG','9001','EPSG','8807','False northing',5000000.0,'EPSG','9001',0);
INSERT INTO "usage" VALUES('EPSG','9302','conversion','EPSG','5220','EPSG','1079','EPSG','1189');
INSERT INTO "conversion" VALUES('EPSG','5222','Gabon Transverse Mercator','For topographic mapping and survey applications see projection code 5522.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',12.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9303','conversion','EPSG','5222','EPSG','3249','EPSG','1165');
INSERT INTO "conversion" VALUES('EPSG','5231','Sri Lanka Grid','Origin is Piturutalagala triangulation point. Used with Kandawala datum.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',7.0001729,'EPSG','9110','EPSG','8802','Longitude of natural origin',80.461816,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999238418,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',200000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9307','conversion','EPSG','5231','EPSG','3310','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','5232','Sri Lanka Grid 1999','Origin is Piturutalagala triangulation point. Used with SLD99.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',7.00016975,'EPSG','9110','EPSG','8802','Longitude of natural origin',80.46181671,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999238418,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9308','conversion','EPSG','5232','EPSG','3310','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','5265','Bhutan National Grid','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',90.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9319','conversion','EPSG','5265','EPSG','1048','EPSG','1178');
INSERT INTO "conversion" VALUES('EPSG','5268','Bumthang TM','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',90.44,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',-2500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9321','conversion','EPSG','5268','EPSG','3734','EPSG','1054');
INSERT INTO "conversion" VALUES('EPSG','5276','Chhukha TM','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',89.33,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',-2500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9322','conversion','EPSG','5276','EPSG','3737','EPSG','1054');
INSERT INTO "conversion" VALUES('EPSG','5277','Dagana TM','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',89.51,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',-2500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9323','conversion','EPSG','5277','EPSG','3738','EPSG','1054');
INSERT INTO "conversion" VALUES('EPSG','5278','Gasa TM','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',90.02,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',-2500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9324','conversion','EPSG','5278','EPSG','3740','EPSG','1054');
INSERT INTO "conversion" VALUES('EPSG','5279','Ha TM','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',90.09,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',-2500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9325','conversion','EPSG','5279','EPSG','3742','EPSG','1054');
INSERT INTO "conversion" VALUES('EPSG','5280','Lhuentse TM','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',91.08,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',-2500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9326','conversion','EPSG','5280','EPSG','3743','EPSG','1054');
INSERT INTO "conversion" VALUES('EPSG','5281','Mongar TM','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',91.14,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',-2500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9327','conversion','EPSG','5281','EPSG','3745','EPSG','1054');
INSERT INTO "conversion" VALUES('EPSG','5282','Paro TM','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',89.21,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',-2500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9328','conversion','EPSG','5282','EPSG','3746','EPSG','1054');
INSERT INTO "conversion" VALUES('EPSG','5283','Pemagatshel TM','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',91.21,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',-2500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9329','conversion','EPSG','5283','EPSG','3747','EPSG','1054');
INSERT INTO "conversion" VALUES('EPSG','5284','Tsirang TM','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',90.1,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',-2500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9330','conversion','EPSG','5284','EPSG','3757','EPSG','1054');
INSERT INTO "conversion" VALUES('EPSG','5285','Samdrup Jongkhar TM','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',91.34,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',-2500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9331','conversion','EPSG','5285','EPSG','3750','EPSG','1054');
INSERT INTO "conversion" VALUES('EPSG','5286','Samtse TM','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',89.04,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',-2500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9332','conversion','EPSG','5286','EPSG','3751','EPSG','1054');
INSERT INTO "conversion" VALUES('EPSG','5287','Sarpang TM','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',90.16,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',-2500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9333','conversion','EPSG','5287','EPSG','3752','EPSG','1054');
INSERT INTO "conversion" VALUES('EPSG','5288','Wangdue Phodrang TM','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',90.07,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',-2500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9334','conversion','EPSG','5288','EPSG','3758','EPSG','1054');
INSERT INTO "conversion" VALUES('EPSG','5289','Trashigang TM','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',91.45,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',-2500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9335','conversion','EPSG','5289','EPSG','3754','EPSG','1054');
INSERT INTO "conversion" VALUES('EPSG','5290','Trongsa TM','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',90.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',-2500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9336','conversion','EPSG','5290','EPSG','3755','EPSG','1054');
INSERT INTO "conversion" VALUES('EPSG','5291','Zhemgang TM','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',90.52,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',-2500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9337','conversion','EPSG','5291','EPSG','3761','EPSG','1054');
INSERT INTO "conversion" VALUES('EPSG','5312','Thimphu TM','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',89.33,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',-2500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9338','conversion','EPSG','5312','EPSG','3753','EPSG','1054');
INSERT INTO "conversion" VALUES('EPSG','5313','Punakha TM','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',89.51,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',-2500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9339','conversion','EPSG','5313','EPSG','3749','EPSG','1054');
INSERT INTO "conversion" VALUES('EPSG','5314','Yangtse TM','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',91.34,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',-2500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9340','conversion','EPSG','5314','EPSG','3760','EPSG','1054');
INSERT INTO "conversion" VALUES('EPSG','5315','Faroe Transverse Mercator','Introduced in 2010.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-7.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.999997,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',-6000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9341','conversion','EPSG','5315','EPSG','1093','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','5319','Teranet Ontario Lambert','Used for Province of Ontario land registration system.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',0.0,'EPSG','9102','EPSG','8822','Longitude of false origin',-84.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',44.3,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',54.3,'EPSG','9110','EPSG','8826','Easting at false origin',1000000.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9342','conversion','EPSG','5319','EPSG','1367','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','5326','Iceland Lambert 2004','Replaces Iceland Lambert 1993 (code 19989). Used only with ISN2004 geogCRS.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',65.0,'EPSG','9110','EPSG','8822','Longitude of false origin',-19.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',64.15,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',65.45,'EPSG','9110','EPSG','8826','Easting at false origin',1700000.0,'EPSG','9001','EPSG','8827','Northing at false origin',300000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9343','conversion','EPSG','5326','EPSG','1120','EPSG','1241');
INSERT INTO "conversion" VALUES('EPSG','5328','Netherlands East Indies Equatorial Zone (Jkt)','Longitude is referenced to the Jakarta meridian. Replaced by NEIEZ (code 19905).','EPSG','9804','Mercator (variant A)','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',3.113221,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.997,'EPSG','9201','EPSG','8806','False easting',3900000.0,'EPSG','9001','EPSG','8807','False northing',900000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9345','conversion','EPSG','5328','EPSG','4020','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','5366','Costa Rica TM 2005','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-84.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9353','conversion','EPSG','5366','EPSG','3849','EPSG','1092');
INSERT INTO "conversion" VALUES('EPSG','5390','Costa Rica Norte','Costa Rica recognises a ''common zone'' between 9°32''N and 9°56''N where either Costa Rica Norte or Costa Rica Sur Lambert zones may be used.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',10.28,'EPSG','9110','EPSG','8802','Longitude of natural origin',-84.2,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.99995696,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',271820.522,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9362','conversion','EPSG','5390','EPSG','3869','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','5394','Costa Rica Sur','Costa Rica recognises a ''common zone'' between 9°32''N and 9°56''N where either Costa Rica Norte or Costa Rica Sur Lambert zones may be used.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',9.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-83.4,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.99995696,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',327987.436,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9363','conversion','EPSG','5394','EPSG','3870','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','5397','Honduras Norte','','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',15.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-86.1,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.99993273,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',296917.439,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9365','conversion','EPSG','5397','EPSG','3848','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','5398','Honduras Sur','','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',13.47,'EPSG','9110','EPSG','8802','Longitude of natural origin',-86.1,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999514,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',296215.903,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9366','conversion','EPSG','5398','EPSG','3850','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','5399','El Salvador Lambert','','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',13.47,'EPSG','9110','EPSG','8802','Longitude of natural origin',-89.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.99996704,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',295809.184,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9367','conversion','EPSG','5399','EPSG','3243','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','5439','Nicaragua Norte','Typographical error in value for false northing in the information source has been corrected in this record.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',13.52,'EPSG','9110','EPSG','8802','Longitude of natural origin',-85.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.99990314,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',359891.816,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9407','conversion','EPSG','5439','EPSG','3844','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','5444','Nicaragua Sur','Typographical error in value for scale factor at natural origin in the information source has been corrected in this record.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',11.44,'EPSG','9110','EPSG','8802','Longitude of natural origin',-85.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.99992228,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',288876.327,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9412','conversion','EPSG','5444','EPSG','3847','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','5465','Belize Colony Grid','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',17.0340471,'EPSG','9110','EPSG','8802','Longitude of natural origin',-88.3754687,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',217259.26,'EPSG','9005','EPSG','8807','False northing',445474.83,'EPSG','9005',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9423','conversion','EPSG','5465','EPSG','3219','EPSG','1209');
INSERT INTO "conversion" VALUES('EPSG','5468','Panama Lambert','Replaces Panama Polyconic projection from 1940s.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',8.25,'EPSG','9110','EPSG','8802','Longitude of natural origin',-80.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.99989909,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',294865.303,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9424','conversion','EPSG','5468','EPSG','3290','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','5471','Panama Polyconic','Replaced by Panama Lambert projection from 1940s.','EPSG','9818','American Polyconic','EPSG','8801','Latitude of natural origin',8.15,'EPSG','9110','EPSG','8802','Longitude of natural origin',-81.0,'EPSG','9110','EPSG','8806','False easting',1000000.0,'EPSG','9037','EPSG','8807','False northing',1092972.1,'EPSG','9037',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9426','conversion','EPSG','5471','EPSG','3290','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','5475','McMurdo Sound Lambert Conformal 2000','Replaces Darwin Glacier LC2000 (proj code 17966) from March 2011.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-78.0,'EPSG','9110','EPSG','8822','Longitude of false origin',163.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',-76.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-79.2,'EPSG','9110','EPSG','8826','Easting at false origin',7000000.0,'EPSG','9001','EPSG','8827','Northing at false origin',5000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9429','conversion','EPSG','5475','EPSG','3853','EPSG','1236');
INSERT INTO "conversion" VALUES('EPSG','5476','Borchgrevink Coast Lambert Conformal 2000','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-74.3,'EPSG','9110','EPSG','8822','Longitude of false origin',165.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',-73.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-75.2,'EPSG','9110','EPSG','8826','Easting at false origin',5000000.0,'EPSG','9001','EPSG','8827','Northing at false origin',3000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9430','conversion','EPSG','5476','EPSG','3854','EPSG','1236');
INSERT INTO "conversion" VALUES('EPSG','5477','Pennell Coast Lambert Conformal 2000','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-71.3,'EPSG','9110','EPSG','8822','Longitude of false origin',166.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',-70.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-72.2,'EPSG','9110','EPSG','8826','Easting at false origin',3000000.0,'EPSG','9001','EPSG','8827','Northing at false origin',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9431','conversion','EPSG','5477','EPSG','3855','EPSG','1236');
INSERT INTO "conversion" VALUES('EPSG','5478','Ross Sea Polar Stereographic 2000','','EPSG','9810','Polar Stereographic (variant A)','EPSG','8801','Latitude of natural origin',-90.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',180.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.994,'EPSG','9201','EPSG','8806','False easting',5000000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9432','conversion','EPSG','5478','EPSG','3856','EPSG','1236');
INSERT INTO "conversion" VALUES('EPSG','5509','Krovak (Greenwich)','Longitude is referenced to the Greenwich meridian. See projection code 19952 for original definition referenced to Ferro.','EPSG','9819','Krovak','EPSG','8811','Latitude of projection centre',49.3,'EPSG','9110','EPSG','8833','Longitude of origin',24.5,'EPSG','9110','EPSG','1036','Co-latitude of cone axis',30.171730311,'EPSG','9110','EPSG','8818','Latitude of pseudo standard parallel',78.3,'EPSG','9110','EPSG','8819','Scale factor on pseudo standard parallel',0.9999,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',0);
INSERT INTO "usage" VALUES('EPSG','9452','conversion','EPSG','5509','EPSG','1306','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','5510','Krovak East North (Greenwich)','Greenwich-based alternative to projection code 5218.','EPSG','1041','Krovak (North Orientated)','EPSG','8811','Latitude of projection centre',49.3,'EPSG','9110','EPSG','8833','Longitude of origin',24.5,'EPSG','9110','EPSG','1036','Co-latitude of cone axis',30.171730311,'EPSG','9110','EPSG','8818','Latitude of pseudo standard parallel',78.3,'EPSG','9110','EPSG','8819','Scale factor on pseudo standard parallel',0.9999,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',0);
INSERT INTO "usage" VALUES('EPSG','9453','conversion','EPSG','5510','EPSG','1306','EPSG','1189');
INSERT INTO "conversion" VALUES('EPSG','5511','Modified Krovak (Greenwich)','Greenwich-based alternative to projection code 5219.','EPSG','1042','Krovak Modified','EPSG','8811','Latitude of projection centre',49.3,'EPSG','9110','EPSG','8833','Longitude of origin',24.5,'EPSG','9110','EPSG','1036','Co-latitude of cone axis',30.1717303,'EPSG','9110','EPSG','8818','Latitude of pseudo standard parallel',78.3,'EPSG','9110','EPSG','8819','Scale factor on pseudo standard parallel',0.9999,'EPSG','9201','EPSG','8806','False easting',5000000.0,'EPSG','9001','EPSG','8807','False northing',5000000.0,'EPSG','9001',0);
INSERT INTO "usage" VALUES('EPSG','9454','conversion','EPSG','5511','EPSG','1079','EPSG','1092');
INSERT INTO "conversion" VALUES('EPSG','5512','Modified Krovak East North (Greenwich)','Greenwich-based alternative to projection code 5220.','EPSG','1043','Krovak Modified (North Orientated)','EPSG','8811','Latitude of projection centre',49.3,'EPSG','9110','EPSG','8833','Longitude of origin',24.5,'EPSG','9110','EPSG','1036','Co-latitude of cone axis',30.1717303,'EPSG','9110','EPSG','8818','Latitude of pseudo standard parallel',78.3,'EPSG','9110','EPSG','8819','Scale factor on pseudo standard parallel',0.9999,'EPSG','9201','EPSG','8806','False easting',5000000.0,'EPSG','9001','EPSG','8807','False northing',5000000.0,'EPSG','9001',0);
INSERT INTO "usage" VALUES('EPSG','9455','conversion','EPSG','5512','EPSG','1079','EPSG','1189');
INSERT INTO "conversion" VALUES('EPSG','5517','Chatham Islands Map Grid','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-44.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-176.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',350000.0,'EPSG','9001','EPSG','8807','False northing',650000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9456','conversion','EPSG','5517','EPSG','2889','EPSG','1056');
INSERT INTO "conversion" VALUES('EPSG','5522','Gabon Transverse Mercator 2011','For forestry applications see projection code 5222.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',11.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',1500000.0,'EPSG','9001','EPSG','8807','False northing',5500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9458','conversion','EPSG','5522','EPSG','1100','EPSG','1092');
INSERT INTO "conversion" VALUES('EPSG','5547','Papua New Guinea Map Grid 1994 zone 54','Grid convergence uses opposite sign convention to UTM.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',141.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9467','conversion','EPSG','5547','EPSG','3882','EPSG','1092');
INSERT INTO "conversion" VALUES('EPSG','5548','Papua New Guinea Map Grid 1994 zone 55','Grid convergence uses opposite sign convention to UTM.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',147.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9468','conversion','EPSG','5548','EPSG','3885','EPSG','1092');
INSERT INTO "conversion" VALUES('EPSG','5549','Papua New Guinea Map Grid 1994 zone 56','Grid convergence uses opposite sign convention to UTM.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',153.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9469','conversion','EPSG','5549','EPSG','3885','EPSG','1092');
INSERT INTO "conversion" VALUES('EPSG','5587','New Brunswick Stereographic (NAD27)','In use until 1979.','EPSG','9809','Oblique Stereographic','EPSG','8801','Latitude of natural origin',46.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-66.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999912,'EPSG','9201','EPSG','8806','False easting',1000000.0,'EPSG','9002','EPSG','8807','False northing',1000000.0,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9475','conversion','EPSG','5587','EPSG','1447','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','5595','Fehmarnbelt TM','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',11.2,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',1000000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9478','conversion','EPSG','5595','EPSG','3889','EPSG','1139');
INSERT INTO "conversion" VALUES('EPSG','5640','Petrobras Mercator','','EPSG','9805','Mercator (variant B)','EPSG','8823','Latitude of 1st standard parallel',-2.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-43.0,'EPSG','9102','EPSG','8806','False easting',5000000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9483','conversion','EPSG','5640','EPSG','3896','EPSG','1136');
INSERT INTO "conversion" VALUES('EPSG','5642','Southern Permian Basin Atlas Lambert','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',48.0,'EPSG','9102','EPSG','8822','Longitude of false origin',10.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',52.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',54.2,'EPSG','9110','EPSG','8826','Easting at false origin',815000.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9484','conversion','EPSG','5642','EPSG','3899','EPSG','1190');
INSERT INTO "conversion" VALUES('EPSG','5645','SPCS83 Vermont zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 14430.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',42.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-72.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999964286,'EPSG','9201','EPSG','8806','False easting',1640416.6667,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9485','conversion','EPSG','5645','EPSG','1414','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','5647','UTM zone 31N with prefix','Variant of UTM zone 31N (code 16031) with zone number prefixed to easting.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',3.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',31500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9486','conversion','EPSG','5647','EPSG','2860','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','5648','UTM zone 33N with prefix','Variant of UTM zone 33N (code 16033) with zone number prefixed to easting.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',15.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',33500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9487','conversion','EPSG','5648','EPSG','2862','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','5658','TM Emilia-Romagna','When applied to Monte Mario datum, this projection emulates superseded CRS ED50 / UTMA in which FE=500000m and FN=-4000000m exactly. This projection''s FE and FN values include approximations for the transformation from Monte Mario to ED50.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',9.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500053.0,'EPSG','9001','EPSG','8807','False northing',-3999820.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9490','conversion','EPSG','5658','EPSG','4035','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','5824','ACT Standard Grid','Origin at AGD66 coordinates of Mt. Stromlo triangulation station. Scale factor includes reduction to average height of area covered.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-35.19038506,'EPSG','9110','EPSG','8802','Longitude of natural origin',149.003346139,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000086,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',600000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9508','conversion','EPSG','5824','EPSG','2283','EPSG','1054');
INSERT INTO "conversion" VALUES('EPSG','5883','Tonga Map Grid','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-177.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',1500000.0,'EPSG','9001','EPSG','8807','False northing',5000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9518','conversion','EPSG','5883','EPSG','1234','EPSG','1181');
INSERT INTO "conversion" VALUES('EPSG','5889','JAXA Snow Depth Polar Stereographic North','','EPSG','9829','Polar Stereographic (variant B)','EPSG','8832','Latitude of standard parallel',70.0,'EPSG','9102','EPSG','8833','Longitude of origin',90.0,'EPSG','9102','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9520','conversion','EPSG','5889','EPSG','1996','EPSG','1040');
INSERT INTO "conversion" VALUES('EPSG','5892','Vietnam TM-3 zone 481','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',102.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9522','conversion','EPSG','5892','EPSG','4193','EPSG','1207');
INSERT INTO "conversion" VALUES('EPSG','5893','Vietnam TM-3 zone 482','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',105.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9523','conversion','EPSG','5893','EPSG','4215','EPSG','1208');
INSERT INTO "usage" VALUES('EPSG','9524','conversion','EPSG','5893','EPSG','4547','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','5894','Vietnam TM-3 zone 491','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',108.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9525','conversion','EPSG','5894','EPSG','4217','EPSG','1208');
INSERT INTO "usage" VALUES('EPSG','9526','conversion','EPSG','5894','EPSG','4558','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','5895','Vietnam TM-3 107-45','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',107.45,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9527','conversion','EPSG','5895','EPSG','4218','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','5901','EPSG Alaska Polar Stereographic','','EPSG','9810','Polar Stereographic (variant A)','EPSG','8801','Latitude of natural origin',90.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-150.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.994,'EPSG','9201','EPSG','8806','False easting',2000000.0,'EPSG','9001','EPSG','8807','False northing',2000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9529','conversion','EPSG','5901','EPSG','1996','EPSG','1071');
INSERT INTO "conversion" VALUES('EPSG','5902','EPSG Canada Polar Stereographic','','EPSG','9810','Polar Stereographic (variant A)','EPSG','8801','Latitude of natural origin',90.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-100.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.994,'EPSG','9201','EPSG','8806','False easting',2000000.0,'EPSG','9001','EPSG','8807','False northing',2000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9530','conversion','EPSG','5902','EPSG','1996','EPSG','1072');
INSERT INTO "conversion" VALUES('EPSG','5903','EPSG Greenland Polar Stereographic','','EPSG','9810','Polar Stereographic (variant A)','EPSG','8801','Latitude of natural origin',90.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-33.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.994,'EPSG','9201','EPSG','8806','False easting',2000000.0,'EPSG','9001','EPSG','8807','False northing',2000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9531','conversion','EPSG','5903','EPSG','1996','EPSG','1073');
INSERT INTO "conversion" VALUES('EPSG','5904','EPSG Norway Polar Stereographic','','EPSG','9810','Polar Stereographic (variant A)','EPSG','8801','Latitude of natural origin',90.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',18.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.994,'EPSG','9201','EPSG','8806','False easting',2000000.0,'EPSG','9001','EPSG','8807','False northing',2000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9532','conversion','EPSG','5904','EPSG','1996','EPSG','1074');
INSERT INTO "conversion" VALUES('EPSG','5905','EPSG Russia Polar Stereographic','','EPSG','9810','Polar Stereographic (variant A)','EPSG','8801','Latitude of natural origin',90.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',105.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.994,'EPSG','9201','EPSG','8806','False easting',2000000.0,'EPSG','9001','EPSG','8807','False northing',2000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9533','conversion','EPSG','5905','EPSG','1996','EPSG','1075');
INSERT INTO "conversion" VALUES('EPSG','5906','EPSG Arctic Regional LCC zone A1','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.997505803.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',81.19020136,'EPSG','9110','EPSG','8822','Longitude of false origin',-111.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',85.0,'EPSG','9102','EPSG','8824','Latitude of 2nd standard parallel',77.0,'EPSG','9102','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9534','conversion','EPSG','5906','EPSG','4019','EPSG','1246');
INSERT INTO "conversion" VALUES('EPSG','5907','EPSG Arctic Regional LCC zone A2','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.997505803.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',81.19020136,'EPSG','9110','EPSG','8822','Longitude of false origin',-39.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',85.0,'EPSG','9102','EPSG','8824','Latitude of 2nd standard parallel',77.0,'EPSG','9102','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9535','conversion','EPSG','5907','EPSG','4027','EPSG','1246');
INSERT INTO "conversion" VALUES('EPSG','5908','EPSG Arctic Regional LCC zone A3','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.997505803.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',81.19020136,'EPSG','9110','EPSG','8822','Longitude of false origin',33.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',85.0,'EPSG','9102','EPSG','8824','Latitude of 2nd standard parallel',77.0,'EPSG','9102','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9536','conversion','EPSG','5908','EPSG','4028','EPSG','1246');
INSERT INTO "conversion" VALUES('EPSG','5909','EPSG Arctic Regional LCC zone A4','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.997505803.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',81.19020136,'EPSG','9110','EPSG','8822','Longitude of false origin',105.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',85.0,'EPSG','9102','EPSG','8824','Latitude of 2nd standard parallel',77.0,'EPSG','9102','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9537','conversion','EPSG','5909','EPSG','4029','EPSG','1246');
INSERT INTO "conversion" VALUES('EPSG','5910','EPSG Arctic Regional LCC zone A5','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.997505803.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',81.19020136,'EPSG','9110','EPSG','8822','Longitude of false origin',177.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',85.0,'EPSG','9102','EPSG','8824','Latitude of 2nd standard parallel',77.0,'EPSG','9102','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9538','conversion','EPSG','5910','EPSG','4031','EPSG','1246');
INSERT INTO "conversion" VALUES('EPSG','5911','EPSG Arctic Regional LCC zone B1','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.997550978.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',73.09206671,'EPSG','9110','EPSG','8822','Longitude of false origin',-111.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',77.0,'EPSG','9102','EPSG','8824','Latitude of 2nd standard parallel',69.0,'EPSG','9102','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9539','conversion','EPSG','5911','EPSG','4032','EPSG','1246');
INSERT INTO "conversion" VALUES('EPSG','5912','EPSG Arctic Regional LCC zone B2','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.997550978.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',73.09206671,'EPSG','9110','EPSG','8822','Longitude of false origin',-39.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',77.0,'EPSG','9102','EPSG','8824','Latitude of 2nd standard parallel',69.0,'EPSG','9102','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9540','conversion','EPSG','5912','EPSG','4033','EPSG','1246');
INSERT INTO "conversion" VALUES('EPSG','5913','EPSG Arctic Regional LCC zone B3','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.997550978.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',73.09206671,'EPSG','9110','EPSG','8822','Longitude of false origin',33.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',77.0,'EPSG','9102','EPSG','8824','Latitude of 2nd standard parallel',69.0,'EPSG','9102','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9541','conversion','EPSG','5913','EPSG','4034','EPSG','1246');
INSERT INTO "conversion" VALUES('EPSG','5914','EPSG Arctic Regional LCC zone B4','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.997550978.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',73.09206671,'EPSG','9110','EPSG','8822','Longitude of false origin',105.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',77.0,'EPSG','9102','EPSG','8824','Latitude of 2nd standard parallel',69.0,'EPSG','9102','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9542','conversion','EPSG','5914','EPSG','4037','EPSG','1246');
INSERT INTO "conversion" VALUES('EPSG','5915','EPSG Arctic Regional LCC zone B5','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.997550978.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',73.09206671,'EPSG','9110','EPSG','8822','Longitude of false origin',177.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',77.0,'EPSG','9102','EPSG','8824','Latitude of 2nd standard parallel',69.0,'EPSG','9102','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9543','conversion','EPSG','5915','EPSG','4038','EPSG','1246');
INSERT INTO "conversion" VALUES('EPSG','5916','EPSG Arctic Regional LCC zone C1','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.997560876.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',65.06045752,'EPSG','9110','EPSG','8822','Longitude of false origin',-111.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',69.0,'EPSG','9102','EPSG','8824','Latitude of 2nd standard parallel',61.0,'EPSG','9102','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9544','conversion','EPSG','5916','EPSG','4040','EPSG','1246');
INSERT INTO "conversion" VALUES('EPSG','5917','EPSG Arctic Regional LCC zone C2','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.997560876.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',65.06045752,'EPSG','9110','EPSG','8822','Longitude of false origin',-39.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',69.0,'EPSG','9102','EPSG','8824','Latitude of 2nd standard parallel',61.0,'EPSG','9102','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9545','conversion','EPSG','5917','EPSG','4041','EPSG','1246');
INSERT INTO "conversion" VALUES('EPSG','5918','EPSG Arctic Regional LCC zone C3','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.997560876.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',65.06045752,'EPSG','9110','EPSG','8822','Longitude of false origin',33.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',69.0,'EPSG','9102','EPSG','8824','Latitude of 2nd standard parallel',61.0,'EPSG','9102','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9546','conversion','EPSG','5918','EPSG','4042','EPSG','1246');
INSERT INTO "conversion" VALUES('EPSG','5919','EPSG Arctic Regional LCC zone C4','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.997560876.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',65.06045752,'EPSG','9110','EPSG','8822','Longitude of false origin',105.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',69.0,'EPSG','9102','EPSG','8824','Latitude of 2nd standard parallel',61.0,'EPSG','9102','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9547','conversion','EPSG','5919','EPSG','4043','EPSG','1246');
INSERT INTO "conversion" VALUES('EPSG','5920','EPSG Arctic Regional LCC zone C5','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.997560876.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',65.06045752,'EPSG','9110','EPSG','8822','Longitude of false origin',177.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',69.0,'EPSG','9102','EPSG','8824','Latitude of 2nd standard parallel',61.0,'EPSG','9102','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9548','conversion','EPSG','5920','EPSG','4045','EPSG','1246');
INSERT INTO "conversion" VALUES('EPSG','5943','EPSG Arctic LCC zone 8-20','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995774379.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',62.00551048,'EPSG','9110','EPSG','8822','Longitude of false origin',-52.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',63.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',60.2,'EPSG','9110','EPSG','8826','Easting at false origin',20500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',8500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9549','conversion','EPSG','5943','EPSG','4123','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','5944','EPSG Arctic LCC zone 8-22','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995774379.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',62.00551048,'EPSG','9110','EPSG','8822','Longitude of false origin',-37.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',63.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',60.2,'EPSG','9110','EPSG','8826','Easting at false origin',22500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',8500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9550','conversion','EPSG','5944','EPSG','4124','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','5977','EPSG Arctic LCC zone 1-21','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995705769.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',85.2613626,'EPSG','9110','EPSG','8822','Longitude of false origin',-150.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',87.0,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',83.4,'EPSG','9110','EPSG','8826','Easting at false origin',21500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',1500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9551','conversion','EPSG','5977','EPSG','4044','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','5978','EPSG Arctic LCC zone 1-23','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995705769.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',85.2613626,'EPSG','9110','EPSG','8822','Longitude of false origin',-90.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',87.0,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',83.4,'EPSG','9110','EPSG','8826','Easting at false origin',23500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',1500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9552','conversion','EPSG','5978','EPSG','4047','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','5979','EPSG Arctic LCC zone 1-25','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995705769.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',85.2613626,'EPSG','9110','EPSG','8822','Longitude of false origin',-30.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',87.0,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',83.4,'EPSG','9110','EPSG','8826','Easting at false origin',25500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',1500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9553','conversion','EPSG','5979','EPSG','4048','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','5980','EPSG Arctic LCC zone 1-27','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995705769.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',85.2613626,'EPSG','9110','EPSG','8822','Longitude of false origin',30.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',87.0,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',83.4,'EPSG','9110','EPSG','8826','Easting at false origin',27500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',1500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9554','conversion','EPSG','5980','EPSG','4049','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','5981','EPSG Arctic LCC zone 1-29','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995705769.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',85.2613626,'EPSG','9110','EPSG','8822','Longitude of false origin',90.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',87.0,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',83.4,'EPSG','9110','EPSG','8826','Easting at false origin',29500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',1500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9555','conversion','EPSG','5981','EPSG','4050','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','5982','EPSG Arctic LCC zone 1-31','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995705769.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',85.2613626,'EPSG','9110','EPSG','8822','Longitude of false origin',150.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',87.0,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',83.4,'EPSG','9110','EPSG','8826','Easting at false origin',31500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',1500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9556','conversion','EPSG','5982','EPSG','4051','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','5983','EPSG Arctic LCC zone 2-10','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995749475.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',82.03303296,'EPSG','9110','EPSG','8822','Longitude of false origin',166.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',83.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',80.2,'EPSG','9110','EPSG','8826','Easting at false origin',10500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',2500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9557','conversion','EPSG','5983','EPSG','4057','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','5984','EPSG Arctic LCC zone 2-12','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995749475.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',82.03303296,'EPSG','9110','EPSG','8822','Longitude of false origin',-154.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',83.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',80.2,'EPSG','9110','EPSG','8826','Easting at false origin',12500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',2500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9558','conversion','EPSG','5984','EPSG','4052','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','5985','EPSG Arctic LCC zone 2-14','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995749475.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',82.03303296,'EPSG','9110','EPSG','8822','Longitude of false origin',-115.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',83.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',80.2,'EPSG','9110','EPSG','8826','Easting at false origin',14500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',2500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9559','conversion','EPSG','5985','EPSG','4030','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','5986','EPSG Arctic LCC zone 2-16','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995749475.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',82.03303296,'EPSG','9110','EPSG','8822','Longitude of false origin',-75.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',83.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',80.2,'EPSG','9110','EPSG','8826','Easting at false origin',16500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',2500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9560','conversion','EPSG','5986','EPSG','4036','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','5987','EPSG Arctic LCC zone 2-18','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995749475.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',82.03303296,'EPSG','9110','EPSG','8822','Longitude of false origin',-52.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',83.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',80.2,'EPSG','9110','EPSG','8826','Easting at false origin',18500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',2500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9561','conversion','EPSG','5987','EPSG','4039','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','5988','EPSG Arctic LCC zone 2-20','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995749475.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',82.03303296,'EPSG','9110','EPSG','8822','Longitude of false origin',-12.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',83.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',80.2,'EPSG','9110','EPSG','8826','Easting at false origin',20500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',2500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9562','conversion','EPSG','5988','EPSG','4046','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','5989','EPSG Arctic LCC zone 2-22','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995749475.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',82.03303296,'EPSG','9110','EPSG','8822','Longitude of false origin',16.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',83.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',80.2,'EPSG','9110','EPSG','8826','Easting at false origin',22500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',2500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9563','conversion','EPSG','5989','EPSG','4053','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','5990','EPSG Arctic LCC zone 2-24','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995749475.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',82.03303296,'EPSG','9110','EPSG','8822','Longitude of false origin',53.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',83.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',80.2,'EPSG','9110','EPSG','8826','Easting at false origin',24500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',2500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9564','conversion','EPSG','5990','EPSG','4054','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','5991','EPSG Arctic LCC zone 2-26','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995749475.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',82.03303296,'EPSG','9110','EPSG','8822','Longitude of false origin',93.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',83.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',80.2,'EPSG','9110','EPSG','8826','Easting at false origin',26500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',2500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9565','conversion','EPSG','5991','EPSG','4055','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','5992','EPSG Arctic LCC zone 2-28','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995749475.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',82.03303296,'EPSG','9110','EPSG','8822','Longitude of false origin',133.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',83.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',80.2,'EPSG','9110','EPSG','8826','Easting at false origin',28500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',2500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9566','conversion','EPSG','5992','EPSG','4056','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','5993','EPSG Arctic LCC zone 3-11','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995760608.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',78.42264151,'EPSG','9110','EPSG','8822','Longitude of false origin',21.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',80.2,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',77.0,'EPSG','9110','EPSG','8826','Easting at false origin',11500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',3500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9567','conversion','EPSG','5993','EPSG','4058','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','5994','EPSG Arctic LCC zone 3-13','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995760608.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',78.42264151,'EPSG','9110','EPSG','8822','Longitude of false origin',52.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',80.2,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',77.0,'EPSG','9110','EPSG','8826','Easting at false origin',13500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',3500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9568','conversion','EPSG','5994','EPSG','4059','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','5995','EPSG Arctic LCC zone 3-15','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995760608.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',78.42264151,'EPSG','9110','EPSG','8822','Longitude of false origin',83.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',80.2,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',77.0,'EPSG','9110','EPSG','8826','Easting at false origin',15500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',3500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9569','conversion','EPSG','5995','EPSG','4060','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','5996','EPSG Arctic LCC zone 3-17','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995760608.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',78.42264151,'EPSG','9110','EPSG','8822','Longitude of false origin',114.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',80.2,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',77.0,'EPSG','9110','EPSG','8826','Easting at false origin',17500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',3500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9570','conversion','EPSG','5996','EPSG','4061','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','5997','EPSG Arctic LCC zone 3-19','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995760608.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',78.42264151,'EPSG','9110','EPSG','8822','Longitude of false origin',145.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',80.2,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',77.0,'EPSG','9110','EPSG','8826','Easting at false origin',19500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',3500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9571','conversion','EPSG','5997','EPSG','4062','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','5998','EPSG Arctic LCC zone 3-21','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995760608.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',78.42264151,'EPSG','9110','EPSG','8822','Longitude of false origin',176.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',80.2,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',77.0,'EPSG','9110','EPSG','8826','Easting at false origin',21500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',3500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9572','conversion','EPSG','5998','EPSG','4063','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','5999','EPSG Arctic LCC zone 3-23','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995760608.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',78.42264151,'EPSG','9110','EPSG','8822','Longitude of false origin',-153.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',80.2,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',77.0,'EPSG','9110','EPSG','8826','Easting at false origin',23500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',3500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9573','conversion','EPSG','5999','EPSG','4064','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','6000','EPSG Arctic LCC zone 3-25','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995760608.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',78.42264151,'EPSG','9110','EPSG','8822','Longitude of false origin',-129.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',80.2,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',77.0,'EPSG','9110','EPSG','8826','Easting at false origin',25500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',3500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9574','conversion','EPSG','6000','EPSG','4065','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','6001','EPSG Arctic LCC zone 3-27','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995760608.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',78.42264151,'EPSG','9110','EPSG','8822','Longitude of false origin',-99.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',80.2,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',77.0,'EPSG','9110','EPSG','8826','Easting at false origin',27500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',3500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9575','conversion','EPSG','6001','EPSG','4070','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','6002','EPSG Arctic LCC zone 3-29','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995760608.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',78.42264151,'EPSG','9110','EPSG','8822','Longitude of false origin',-69.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',80.2,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',77.0,'EPSG','9110','EPSG','8826','Easting at false origin',29500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',3500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9576','conversion','EPSG','6002','EPSG','4071','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','6003','EPSG Arctic LCC zone 3-31','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995760608.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',78.42264151,'EPSG','9110','EPSG','8822','Longitude of false origin',-39.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',80.2,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',77.0,'EPSG','9110','EPSG','8826','Easting at false origin',31500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',3500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9577','conversion','EPSG','6003','EPSG','4074','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','6004','EPSG Arctic LCC zone 3-33','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995760608.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',78.42264151,'EPSG','9110','EPSG','8822','Longitude of false origin',-10.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',80.2,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',77.0,'EPSG','9110','EPSG','8826','Easting at false origin',33500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',3500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9578','conversion','EPSG','6004','EPSG','4075','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','6005','EPSG Arctic LCC zone 4-12','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995765501.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',75.21518519,'EPSG','9110','EPSG','8822','Longitude of false origin',-155.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',77.0,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',73.4,'EPSG','9110','EPSG','8826','Easting at false origin',12500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',4500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9579','conversion','EPSG','6005','EPSG','4076','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','6006','EPSG Arctic LCC zone 4-14','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995765501.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',75.21518519,'EPSG','9110','EPSG','8822','Longitude of false origin',-129.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',77.0,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',73.4,'EPSG','9110','EPSG','8826','Easting at false origin',14500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',4500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9580','conversion','EPSG','6006','EPSG','4077','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','6007','EPSG Arctic LCC zone 4-16','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995765501.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',75.21518519,'EPSG','9110','EPSG','8822','Longitude of false origin',-104.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',77.0,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',73.4,'EPSG','9110','EPSG','8826','Easting at false origin',16500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',4500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9581','conversion','EPSG','6007','EPSG','4078','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','6008','EPSG Arctic LCC zone 4-18','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995765501.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',75.21518519,'EPSG','9110','EPSG','8822','Longitude of false origin',-79.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',77.0,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',73.4,'EPSG','9110','EPSG','8826','Easting at false origin',18500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',4500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9582','conversion','EPSG','6008','EPSG','4079','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','6009','EPSG Arctic LCC zone 4-20','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995765501.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',75.21518519,'EPSG','9110','EPSG','8822','Longitude of false origin',-64.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',77.0,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',73.4,'EPSG','9110','EPSG','8826','Easting at false origin',20500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',4500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9583','conversion','EPSG','6009','EPSG','4080','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','6010','EPSG Arctic LCC zone 4-22','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995765501.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',75.21518519,'EPSG','9110','EPSG','8822','Longitude of false origin',-39.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',77.0,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',73.4,'EPSG','9110','EPSG','8826','Easting at false origin',22500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',4500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9584','conversion','EPSG','6010','EPSG','4081','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','6011','EPSG Arctic LCC zone 4-24','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995765501.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',75.21518519,'EPSG','9110','EPSG','8822','Longitude of false origin',-14.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',77.0,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',73.4,'EPSG','9110','EPSG','8826','Easting at false origin',24500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',4500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9585','conversion','EPSG','6011','EPSG','4082','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','6012','EPSG Arctic LCC zone 4-26','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995765501.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',75.21518519,'EPSG','9110','EPSG','8822','Longitude of false origin',10.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',77.0,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',73.4,'EPSG','9110','EPSG','8826','Easting at false origin',26500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',4500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9586','conversion','EPSG','6012','EPSG','4083','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','6013','EPSG Arctic LCC zone 4-28','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995765501.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',75.21518519,'EPSG','9110','EPSG','8822','Longitude of false origin',34.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',77.0,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',73.4,'EPSG','9110','EPSG','8826','Easting at false origin',28500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',4500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9587','conversion','EPSG','6013','EPSG','4084','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','6014','EPSG Arctic LCC zone 4-30','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995765501.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',75.21518519,'EPSG','9110','EPSG','8822','Longitude of false origin',58.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',77.0,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',73.4,'EPSG','9110','EPSG','8826','Easting at false origin',30500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',4500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9588','conversion','EPSG','6014','EPSG','4085','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','6015','EPSG Arctic LCC zone 4-32','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995765501.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',75.21518519,'EPSG','9110','EPSG','8822','Longitude of false origin',82.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',77.0,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',73.4,'EPSG','9110','EPSG','8826','Easting at false origin',32500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',4500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9589','conversion','EPSG','6015','EPSG','4086','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','6016','EPSG Arctic LCC zone 4-34','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995765501.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',75.21518519,'EPSG','9110','EPSG','8822','Longitude of false origin',106.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',77.0,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',73.4,'EPSG','9110','EPSG','8826','Easting at false origin',34500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',4500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9590','conversion','EPSG','6016','EPSG','4087','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','6017','EPSG Arctic LCC zone 4-36','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995765501.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',75.21518519,'EPSG','9110','EPSG','8822','Longitude of false origin',130.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',77.0,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',73.4,'EPSG','9110','EPSG','8826','Easting at false origin',36500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',4500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9591','conversion','EPSG','6017','EPSG','4088','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','6018','EPSG Arctic LCC zone 4-38','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995765501.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',75.21518519,'EPSG','9110','EPSG','8822','Longitude of false origin',154.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',77.0,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',73.4,'EPSG','9110','EPSG','8826','Easting at false origin',38500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',4500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9592','conversion','EPSG','6018','EPSG','4089','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','6019','EPSG Arctic LCC zone 4-40','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995765501.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',75.21518519,'EPSG','9110','EPSG','8822','Longitude of false origin',179.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',77.0,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',73.4,'EPSG','9110','EPSG','8826','Easting at false origin',40500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',4500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9593','conversion','EPSG','6019','EPSG','4090','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','6020','EPSG Arctic LCC zone 5-11','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995768455.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',72.01300331,'EPSG','9110','EPSG','8822','Longitude of false origin',14.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',73.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',70.2,'EPSG','9110','EPSG','8826','Easting at false origin',11500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',5500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9594','conversion','EPSG','6020','EPSG','4091','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','6021','EPSG Arctic LCC zone 5-13','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995768455.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',72.01300331,'EPSG','9110','EPSG','8822','Longitude of false origin',34.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',73.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',70.2,'EPSG','9110','EPSG','8826','Easting at false origin',13500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',5500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9595','conversion','EPSG','6021','EPSG','4092','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','6022','EPSG Arctic LCC zone 5-15','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995768455.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',72.01300331,'EPSG','9110','EPSG','8822','Longitude of false origin',54.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',73.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',70.2,'EPSG','9110','EPSG','8826','Easting at false origin',15500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',5500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9596','conversion','EPSG','6022','EPSG','4093','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','6023','EPSG Arctic LCC zone 5-17','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995768455.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',72.01300331,'EPSG','9110','EPSG','8822','Longitude of false origin',74.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',73.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',70.2,'EPSG','9110','EPSG','8826','Easting at false origin',17500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',5500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9597','conversion','EPSG','6023','EPSG','4094','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','6024','EPSG Arctic LCC zone 5-19','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995768455.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',72.01300331,'EPSG','9110','EPSG','8822','Longitude of false origin',95.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',73.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',70.2,'EPSG','9110','EPSG','8826','Easting at false origin',19500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',5500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9598','conversion','EPSG','6024','EPSG','4095','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','6025','EPSG Arctic LCC zone 5-21','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995768455.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',72.01300331,'EPSG','9110','EPSG','8822','Longitude of false origin',116.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',73.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',70.2,'EPSG','9110','EPSG','8826','Easting at false origin',21500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',5500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9599','conversion','EPSG','6025','EPSG','4096','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','6026','EPSG Arctic LCC zone 5-23','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995768455.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',72.01300331,'EPSG','9110','EPSG','8822','Longitude of false origin',137.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',73.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',70.2,'EPSG','9110','EPSG','8826','Easting at false origin',23500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',5500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9600','conversion','EPSG','6026','EPSG','4097','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','6027','EPSG Arctic LCC zone 5-25','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995768455.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',72.01300331,'EPSG','9110','EPSG','8822','Longitude of false origin',158.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',73.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',70.2,'EPSG','9110','EPSG','8826','Easting at false origin',25500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',5500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9601','conversion','EPSG','6027','EPSG','4098','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','6028','EPSG Arctic LCC zone 5-27','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995768455.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',72.01300331,'EPSG','9110','EPSG','8822','Longitude of false origin',179.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',73.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',70.2,'EPSG','9110','EPSG','8826','Easting at false origin',27500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',5500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9602','conversion','EPSG','6028','EPSG','4099','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','6029','EPSG Arctic LCC zone 5-29','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995768455.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',72.01300331,'EPSG','9110','EPSG','8822','Longitude of false origin',-163.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',73.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',70.2,'EPSG','9110','EPSG','8826','Easting at false origin',29500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',5500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9603','conversion','EPSG','6029','EPSG','4100','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','6030','EPSG Arctic LCC zone 5-31','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995768455.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',72.01300331,'EPSG','9110','EPSG','8822','Longitude of false origin',-147.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',73.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',70.2,'EPSG','9110','EPSG','8826','Easting at false origin',31500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',5500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9604','conversion','EPSG','6030','EPSG','4101','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','6031','EPSG Arctic LCC zone 5-33','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995768455.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',72.01300331,'EPSG','9110','EPSG','8822','Longitude of false origin',-131.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',73.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',70.2,'EPSG','9110','EPSG','8826','Easting at false origin',33500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',5500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9605','conversion','EPSG','6031','EPSG','4102','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','6032','EPSG Arctic LCC zone 5-35','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995768455.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',72.01300331,'EPSG','9110','EPSG','8822','Longitude of false origin',-111.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',73.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',70.2,'EPSG','9110','EPSG','8826','Easting at false origin',35500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',5500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9606','conversion','EPSG','6032','EPSG','4103','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','6033','EPSG Arctic LCC zone 5-37','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995768455.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',72.01300331,'EPSG','9110','EPSG','8822','Longitude of false origin',-91.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',73.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',70.2,'EPSG','9110','EPSG','8826','Easting at false origin',37500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',5500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9607','conversion','EPSG','6033','EPSG','4104','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','6034','EPSG Arctic LCC zone 5-39','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995768455.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',72.01300331,'EPSG','9110','EPSG','8822','Longitude of false origin',-71.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',73.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',70.2,'EPSG','9110','EPSG','8826','Easting at false origin',39500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',5500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9608','conversion','EPSG','6034','EPSG','4105','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','6035','EPSG Arctic LCC zone 5-41','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995768455.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',72.01300331,'EPSG','9110','EPSG','8822','Longitude of false origin',-62.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',73.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',70.2,'EPSG','9110','EPSG','8826','Easting at false origin',41500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',5500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9609','conversion','EPSG','6035','EPSG','4106','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','6036','EPSG Arctic LCC zone 5-43','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995768455.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',72.01300331,'EPSG','9110','EPSG','8822','Longitude of false origin',-42.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',73.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',70.2,'EPSG','9110','EPSG','8826','Easting at false origin',43500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',5500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9610','conversion','EPSG','6036','EPSG','4107','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','6037','EPSG Arctic LCC zone 5-45','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995768455.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',72.01300331,'EPSG','9110','EPSG','8822','Longitude of false origin',-22.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',73.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',70.2,'EPSG','9110','EPSG','8826','Easting at false origin',45500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',5500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9611','conversion','EPSG','6037','EPSG','4108','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','6038','EPSG Arctic LCC zone 5-47','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995768455.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',72.01300331,'EPSG','9110','EPSG','8822','Longitude of false origin',-5.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',73.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',70.2,'EPSG','9110','EPSG','8826','Easting at false origin',47500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',5500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9612','conversion','EPSG','6038','EPSG','4109','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','6039','EPSG Arctic LCC zone 6-14','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995770671.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',68.4114912,'EPSG','9110','EPSG','8822','Longitude of false origin',-165.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',70.2,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',67.0,'EPSG','9110','EPSG','8826','Easting at false origin',14500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',6500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9613','conversion','EPSG','6039','EPSG','4110','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','6040','EPSG Arctic LCC zone 6-16','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995770671.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',68.4114912,'EPSG','9110','EPSG','8822','Longitude of false origin',-147.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',70.2,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',67.0,'EPSG','9110','EPSG','8826','Easting at false origin',16500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',6500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9614','conversion','EPSG','6040','EPSG','4111','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','6041','EPSG Arctic LCC zone 6-18','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995770671.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',68.4114912,'EPSG','9110','EPSG','8822','Longitude of false origin',-132.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',70.2,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',67.0,'EPSG','9110','EPSG','8826','Easting at false origin',18500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',6500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9615','conversion','EPSG','6041','EPSG','4112','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','6042','EPSG Arctic LCC zone 6-20','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995770671.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',68.4114912,'EPSG','9110','EPSG','8822','Longitude of false origin',-113.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',70.2,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',67.0,'EPSG','9110','EPSG','8826','Easting at false origin',20500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',6500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9616','conversion','EPSG','6042','EPSG','4113','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','6043','EPSG Arctic LCC zone 6-22','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995770671.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',68.4114912,'EPSG','9110','EPSG','8822','Longitude of false origin',-94.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',70.2,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',67.0,'EPSG','9110','EPSG','8826','Easting at false origin',22500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',6500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9617','conversion','EPSG','6043','EPSG','4114','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','6044','EPSG Arctic LCC zone 6-24','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995770671.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',68.4114912,'EPSG','9110','EPSG','8822','Longitude of false origin',-75.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',70.2,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',67.0,'EPSG','9110','EPSG','8826','Easting at false origin',24500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',6500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9618','conversion','EPSG','6044','EPSG','4115','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','6045','EPSG Arctic LCC zone 6-26','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995770671.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',68.4114912,'EPSG','9110','EPSG','8822','Longitude of false origin',-56.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',70.2,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',67.0,'EPSG','9110','EPSG','8826','Easting at false origin',26500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',6500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9619','conversion','EPSG','6045','EPSG','4116','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','6046','EPSG Arctic LCC zone 6-28','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995770671.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',68.4114912,'EPSG','9110','EPSG','8822','Longitude of false origin',-38.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',70.2,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',67.0,'EPSG','9110','EPSG','8826','Easting at false origin',28500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',6500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9620','conversion','EPSG','6046','EPSG','4117','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','6047','EPSG Arctic LCC zone 6-30','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995770671.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',68.4114912,'EPSG','9110','EPSG','8822','Longitude of false origin',-20.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',70.2,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',67.0,'EPSG','9110','EPSG','8826','Easting at false origin',30500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',6500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9621','conversion','EPSG','6047','EPSG','4118','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','6048','EPSG Arctic LCC zone 7-11','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995772585.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',65.21037415,'EPSG','9110','EPSG','8822','Longitude of false origin',-51.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',67.0,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',63.4,'EPSG','9110','EPSG','8826','Easting at false origin',11500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',7500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9622','conversion','EPSG','6048','EPSG','4119','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','6049','EPSG Arctic LCC zone 7-13','May be defined using LCC (1SP) method (EPSG method code 9801) with value of latitude for natural origin as latitude at false origin here and scale factor = 0.9995772585.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',65.21037415,'EPSG','9110','EPSG','8822','Longitude of false origin',-34.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',67.0,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',63.4,'EPSG','9110','EPSG','8826','Easting at false origin',13500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',7500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9623','conversion','EPSG','6049','EPSG','4120','EPSG','1148');
INSERT INTO "conversion" VALUES('EPSG','6126','Cayman Islands LCC (ft)','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',19.2,'EPSG','9110','EPSG','8822','Longitude of false origin',80.34,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',19.2,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',19.42,'EPSG','9110','EPSG','8826','Easting at false origin',2950000.0,'EPSG','9002','EPSG','8827','Northing at false origin',1900000.0,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','9624','conversion','EPSG','6126','EPSG','1063','EPSG','1027');
INSERT INTO "conversion" VALUES('EPSG','6127','Cayman Islands TM (ft)','Sometimes locally referred to as "UTM zone 17".','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-81.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',1640419.9475,'EPSG','9002','EPSG','8807','False northing',0.0,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9625','conversion','EPSG','6127','EPSG','1063','EPSG','1056');
INSERT INTO "conversion" VALUES('EPSG','6197','Michigan CS27 North zone','','EPSG','1051','Lambert Conic Conformal (2SP Michigan)','EPSG','8821','Latitude of false origin',44.87,'EPSG','9110','EPSG','8822','Longitude of false origin',-87.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',45.29,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',47.05,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003','EPSG','1038','Ellipsoid scaling factor',1.0000382,'EPSG','9201',1);
INSERT INTO "usage" VALUES('EPSG','9642','conversion','EPSG','6197','EPSG','1723','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','6198','Michigan CS27 Central zone','','EPSG','1051','Lambert Conic Conformal (2SP Michigan)','EPSG','8821','Latitude of false origin',43.19,'EPSG','9110','EPSG','8822','Longitude of false origin',-84.2,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',44.11,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',45.42,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003','EPSG','1038','Ellipsoid scaling factor',1.0000382,'EPSG','9201',0);
INSERT INTO "usage" VALUES('EPSG','9643','conversion','EPSG','6198','EPSG','1724','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','6199','Michigan CS27 South zone','','EPSG','1051','Lambert Conic Conformal (2SP Michigan)','EPSG','8821','Latitude of false origin',41.3,'EPSG','9110','EPSG','8822','Longitude of false origin',-84.2,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',42.06,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',43.4,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003','EPSG','1038','Ellipsoid scaling factor',1.0000382,'EPSG','9201',0);
INSERT INTO "usage" VALUES('EPSG','9644','conversion','EPSG','6199','EPSG','1725','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','6203','Macedonia Gauss-Kruger','A version of this projection with FN reduced by 4000000 is sometimes used for cadastre - see projection code 9911.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',21.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','17410','conversion','EPSG','6203','EPSG','1148','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','6212','Arauca urban grid','','EPSG','1052','Colombia Urban','EPSG','8801','Latitude of natural origin',7.051538301,'EPSG','9110','EPSG','8802','Longitude of natural origin',-70.452991476,'EPSG','9110','EPSG','8806','False easting',1035263.443,'EPSG','9001','EPSG','8807','False northing',1275526.621,'EPSG','9001','EPSG','1039','Projection plane origin height',100.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9650','conversion','EPSG','6212','EPSG','4122','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','6213','Armenia urban grid','','EPSG','1052','Colombia Urban','EPSG','8801','Latitude of natural origin',4.315637,'EPSG','9110','EPSG','8802','Longitude of natural origin',-75.4024561,'EPSG','9110','EPSG','8806','False easting',1155824.666,'EPSG','9001','EPSG','8807','False northing',993087.465,'EPSG','9001','EPSG','1039','Projection plane origin height',1470.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9651','conversion','EPSG','6213','EPSG','4132','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','6214','Barranquilla urban grid','','EPSG','1052','Colombia Urban','EPSG','8801','Latitude of natural origin',10.55234591,'EPSG','9110','EPSG','8802','Longitude of natural origin',-74.50035928,'EPSG','9110','EPSG','8806','False easting',917264.406,'EPSG','9001','EPSG','8807','False northing',1699839.935,'EPSG','9001','EPSG','1039','Projection plane origin height',100.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9652','conversion','EPSG','6214','EPSG','4134','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','6215','Bogota urban grid','','EPSG','1052','Colombia Urban','EPSG','8801','Latitude of natural origin',4.404975,'EPSG','9110','EPSG','8802','Longitude of natural origin',-74.084773,'EPSG','9110','EPSG','8806','False easting',92334.879,'EPSG','9001','EPSG','8807','False northing',109320.965,'EPSG','9001','EPSG','1039','Projection plane origin height',2550.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9653','conversion','EPSG','6215','EPSG','4135','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','6216','Bucaramanga urban grid','','EPSG','1052','Colombia Urban','EPSG','8801','Latitude of natural origin',7.044399371,'EPSG','9110','EPSG','8802','Longitude of natural origin',-73.11504356,'EPSG','9110','EPSG','8806','False easting',1097241.305,'EPSG','9001','EPSG','8807','False northing',1274642.278,'EPSG','9001','EPSG','1039','Projection plane origin height',931.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9654','conversion','EPSG','6216','EPSG','4136','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','6217','Cali urban grid','','EPSG','1052','Colombia Urban','EPSG','8801','Latitude of natural origin',3.263078,'EPSG','9110','EPSG','8802','Longitude of natural origin',-76.3114025,'EPSG','9110','EPSG','8806','False easting',1061900.18,'EPSG','9001','EPSG','8807','False northing',872364.63,'EPSG','9001','EPSG','1039','Projection plane origin height',1000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9655','conversion','EPSG','6217','EPSG','4137','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','6218','Cartagena urban grid','','EPSG','1052','Colombia Urban','EPSG','8801','Latitude of natural origin',10.2349371,'EPSG','9110','EPSG','8802','Longitude of natural origin',-75.3040345,'EPSG','9110','EPSG','8806','False easting',842981.41,'EPSG','9001','EPSG','8807','False northing',1641887.09,'EPSG','9001','EPSG','1039','Projection plane origin height',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9656','conversion','EPSG','6218','EPSG','4138','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','6219','Cucuta urban grid','','EPSG','1052','Colombia Urban','EPSG','8801','Latitude of natural origin',7.532017225,'EPSG','9110','EPSG','8802','Longitude of natural origin',-72.301033542,'EPSG','9110','EPSG','8806','False easting',842805.406,'EPSG','9001','EPSG','8807','False northing',1364404.57,'EPSG','9001','EPSG','1039','Projection plane origin height',308.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9657','conversion','EPSG','6219','EPSG','4139','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','6220','Florencia urban grid','','EPSG','1052','Colombia Urban','EPSG','8801','Latitude of natural origin',1.371564426,'EPSG','9110','EPSG','8802','Longitude of natural origin',-75.370882337,'EPSG','9110','EPSG','8806','False easting',1162300.348,'EPSG','9001','EPSG','8807','False northing',671068.716,'EPSG','9001','EPSG','1039','Projection plane origin height',300.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9658','conversion','EPSG','6220','EPSG','4140','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','6221','Ibague urban grid','','EPSG','1052','Colombia Urban','EPSG','8801','Latitude of natural origin',4.250988618,'EPSG','9110','EPSG','8802','Longitude of natural origin',-75.104773336,'EPSG','9110','EPSG','8806','False easting',877634.33,'EPSG','9001','EPSG','8807','False northing',980541.348,'EPSG','9001','EPSG','1039','Projection plane origin height',1100.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9659','conversion','EPSG','6221','EPSG','4141','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','6222','Inirida urban grid','','EPSG','1052','Colombia Urban','EPSG','8801','Latitude of natural origin',3.504357746,'EPSG','9110','EPSG','8802','Longitude of natural origin',-67.541883552,'EPSG','9110','EPSG','8806','False easting',1019177.687,'EPSG','9001','EPSG','8807','False northing',491791.326,'EPSG','9001','EPSG','1039','Projection plane origin height',96.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9660','conversion','EPSG','6222','EPSG','4142','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','6223','Leticia urban grid','','EPSG','1052','Colombia Urban','EPSG','8801','Latitude of natural origin',-4.115166257,'EPSG','9110','EPSG','8802','Longitude of natural origin',-69.563411981,'EPSG','9110','EPSG','8806','False easting',25978.217,'EPSG','9001','EPSG','8807','False northing',27501.365,'EPSG','9001','EPSG','1039','Projection plane origin height',89.7,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9661','conversion','EPSG','6223','EPSG','4143','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','6224','Manizales urban grid','','EPSG','1052','Colombia Urban','EPSG','8801','Latitude of natural origin',5.0405354,'EPSG','9110','EPSG','8802','Longitude of natural origin',-75.3039941,'EPSG','9110','EPSG','8806','False easting',1173727.04,'EPSG','9001','EPSG','8807','False northing',1052391.13,'EPSG','9001','EPSG','1039','Projection plane origin height',2100.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9662','conversion','EPSG','6224','EPSG','4144','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','6225','Medellin urban grid','','EPSG','1052','Colombia Urban','EPSG','8801','Latitude of natural origin',6.1345152,'EPSG','9110','EPSG','8802','Longitude of natural origin',-75.3353593,'EPSG','9110','EPSG','8806','False easting',835378.647,'EPSG','9001','EPSG','8807','False northing',1180816.875,'EPSG','9001','EPSG','1039','Projection plane origin height',1510.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9663','conversion','EPSG','6225','EPSG','4145','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','6226','Mitu urban grid','','EPSG','1052','Colombia Urban','EPSG','8801','Latitude of natural origin',1.145988972,'EPSG','9110','EPSG','8802','Longitude of natural origin',-70.140766196,'EPSG','9110','EPSG','8806','False easting',1093717.398,'EPSG','9001','EPSG','8807','False northing',629997.236,'EPSG','9001','EPSG','1039','Projection plane origin height',170.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9664','conversion','EPSG','6226','EPSG','4146','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','6227','Mocoa urban grid','','EPSG','1052','Colombia Urban','EPSG','8801','Latitude of natural origin',1.082408409,'EPSG','9110','EPSG','8802','Longitude of natural origin',-76.390367639,'EPSG','9110','EPSG','8806','False easting',1047467.388,'EPSG','9001','EPSG','8807','False northing',617828.474,'EPSG','9001','EPSG','1039','Projection plane origin height',655.2,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9665','conversion','EPSG','6227','EPSG','4147','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','6228','Monteria urban grid','','EPSG','1052','Colombia Urban','EPSG','8801','Latitude of natural origin',8.462310872,'EPSG','9110','EPSG','8802','Longitude of natural origin',-75.524639199,'EPSG','9110','EPSG','8806','False easting',1131814.934,'EPSG','9001','EPSG','8807','False northing',1462131.119,'EPSG','9001','EPSG','1039','Projection plane origin height',15.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9666','conversion','EPSG','6228','EPSG','4148','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','6229','Neiva urban grid','','EPSG','1052','Colombia Urban','EPSG','8801','Latitude of natural origin',2.56326942,'EPSG','9110','EPSG','8802','Longitude of natural origin',-75.17471722,'EPSG','9110','EPSG','8806','False easting',864476.923,'EPSG','9001','EPSG','8807','False northing',817199.827,'EPSG','9001','EPSG','1039','Projection plane origin height',430.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9667','conversion','EPSG','6229','EPSG','4149','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','6230','Pasto urban grid','','EPSG','1052','Colombia Urban','EPSG','8801','Latitude of natural origin',1.120356225,'EPSG','9110','EPSG','8802','Longitude of natural origin',-77.151125228,'EPSG','9110','EPSG','8806','False easting',980469.695,'EPSG','9001','EPSG','8807','False northing',624555.332,'EPSG','9001','EPSG','1039','Projection plane origin height',2530.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9668','conversion','EPSG','6230','EPSG','4150','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','6231','Pereira urban grid','','EPSG','1052','Colombia Urban','EPSG','8801','Latitude of natural origin',4.4848937,'EPSG','9110','EPSG','8802','Longitude of natural origin',-75.4138225,'EPSG','9110','EPSG','8806','False easting',1153492.012,'EPSG','9001','EPSG','8807','False northing',1024195.255,'EPSG','9001','EPSG','1039','Projection plane origin height',1500.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9669','conversion','EPSG','6231','EPSG','4151','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','6232','Popayan urban grid','','EPSG','1052','Colombia Urban','EPSG','8801','Latitude of natural origin',2.272217558,'EPSG','9110','EPSG','8802','Longitude of natural origin',-76.362192989,'EPSG','9110','EPSG','8806','False easting',1052430.525,'EPSG','9001','EPSG','8807','False northing',763366.548,'EPSG','9001','EPSG','1039','Projection plane origin height',1740.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9670','conversion','EPSG','6232','EPSG','4152','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','6233','Puerto Carreno urban grid','','EPSG','1052','Colombia Urban','EPSG','8801','Latitude of natural origin',6.105059709,'EPSG','9110','EPSG','8802','Longitude of natural origin',-67.300270089,'EPSG','9110','EPSG','8806','False easting',1063834.703,'EPSG','9001','EPSG','8807','False northing',1175257.481,'EPSG','9001','EPSG','1039','Projection plane origin height',51.58,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9671','conversion','EPSG','6233','EPSG','4153','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','6234','Quibdo urban grid','','EPSG','1052','Colombia Urban','EPSG','8801','Latitude of natural origin',5.413929158,'EPSG','9110','EPSG','8802','Longitude of natural origin',-76.390271389,'EPSG','9110','EPSG','8806','False easting',1047273.617,'EPSG','9001','EPSG','8807','False northing',1121443.09,'EPSG','9001','EPSG','1039','Projection plane origin height',44.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9672','conversion','EPSG','6234','EPSG','4154','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','6235','Riohacha urban grid','','EPSG','1052','Colombia Urban','EPSG','8801','Latitude of natural origin',11.321288798,'EPSG','9110','EPSG','8802','Longitude of natural origin',-72.540996793,'EPSG','9110','EPSG','8806','False easting',1128154.73,'EPSG','9001','EPSG','8807','False northing',1767887.914,'EPSG','9001','EPSG','1039','Projection plane origin height',6.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9673','conversion','EPSG','6235','EPSG','4155','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','6236','San Andres urban grid','','EPSG','1052','Colombia Urban','EPSG','8801','Latitude of natural origin',12.312565957,'EPSG','9110','EPSG','8802','Longitude of natural origin',-81.434575342,'EPSG','9110','EPSG','8806','False easting',820439.298,'EPSG','9001','EPSG','8807','False northing',1877357.828,'EPSG','9001','EPSG','1039','Projection plane origin height',6.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9674','conversion','EPSG','6236','EPSG','4156','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','6237','San Jose del Guaviare urban grid','','EPSG','1052','Colombia Urban','EPSG','8801','Latitude of natural origin',2.335068419,'EPSG','9110','EPSG','8802','Longitude of natural origin',-72.382411997,'EPSG','9110','EPSG','8806','False easting',1159876.62,'EPSG','9001','EPSG','8807','False northing',775380.342,'EPSG','9001','EPSG','1039','Projection plane origin height',185.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9675','conversion','EPSG','6237','EPSG','4157','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','6238','Santa Marta urban grid','','EPSG','1052','Colombia Urban','EPSG','8801','Latitude of natural origin',11.1310715,'EPSG','9110','EPSG','8802','Longitude of natural origin',-74.1330019,'EPSG','9110','EPSG','8806','False easting',983892.409,'EPSG','9001','EPSG','8807','False northing',1732533.518,'EPSG','9001','EPSG','1039','Projection plane origin height',29.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9676','conversion','EPSG','6238','EPSG','4128','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','6239','Sucre urban grid','','EPSG','1052','Colombia Urban','EPSG','8801','Latitude of natural origin',8.483798132,'EPSG','9110','EPSG','8802','Longitude of natural origin',-74.432088057,'EPSG','9110','EPSG','8806','False easting',929043.607,'EPSG','9001','EPSG','8807','False northing',1466125.658,'EPSG','9001','EPSG','1039','Projection plane origin height',20.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9677','conversion','EPSG','6239','EPSG','4130','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','6240','Tunja urban grid','','EPSG','1052','Colombia Urban','EPSG','8801','Latitude of natural origin',5.320310106,'EPSG','9110','EPSG','8802','Longitude of natural origin',-73.210698004,'EPSG','9110','EPSG','8806','False easting',1080514.91,'EPSG','9001','EPSG','8807','False northing',1103772.028,'EPSG','9001','EPSG','1039','Projection plane origin height',2800.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9678','conversion','EPSG','6240','EPSG','4131','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','6241','Valledupar urban grid','','EPSG','1052','Colombia Urban','EPSG','8801','Latitude of natural origin',10.265014,'EPSG','9110','EPSG','8802','Longitude of natural origin',-73.1447657,'EPSG','9110','EPSG','8806','False easting',1090979.66,'EPSG','9001','EPSG','8807','False northing',1647208.93,'EPSG','9001','EPSG','1039','Projection plane origin height',200.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9679','conversion','EPSG','6241','EPSG','4158','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','6242','Villavicencio urban grid','','EPSG','1052','Colombia Urban','EPSG','8801','Latitude of natural origin',4.091935036,'EPSG','9110','EPSG','8802','Longitude of natural origin',-73.372814955,'EPSG','9110','EPSG','8806','False easting',1050678.757,'EPSG','9001','EPSG','8807','False northing',950952.124,'EPSG','9001','EPSG','1039','Projection plane origin height',427.19,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9680','conversion','EPSG','6242','EPSG','4159','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','6243','Yopal urban grid','','EPSG','1052','Colombia Urban','EPSG','8801','Latitude of natural origin',5.2114138,'EPSG','9110','EPSG','8802','Longitude of natural origin',-72.2512145,'EPSG','9110','EPSG','8806','False easting',851184.177,'EPSG','9001','EPSG','8807','False northing',1083954.137,'EPSG','9001','EPSG','1039','Projection plane origin height',300.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9681','conversion','EPSG','6243','EPSG','4160','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','6308','Cyprus Transverse Mercator','Introduced in 1993. Replaces Cassini grid.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',33.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.99995,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',-3500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9713','conversion','EPSG','6308','EPSG','3236','EPSG','1056');
INSERT INTO "conversion" VALUES('EPSG','6361','Mexico LCC','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',12.0,'EPSG','9102','EPSG','8822','Longitude of false origin',-102.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',17.5,'EPSG','9102','EPSG','8824','Latitude of 2nd standard parallel',29.5,'EPSG','9102','EPSG','8826','Easting at false origin',2500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9719','conversion','EPSG','6361','EPSG','1160','EPSG','1241');
INSERT INTO "conversion" VALUES('EPSG','6374','Ukraine TM zone 7','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',21.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',300000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9721','conversion','EPSG','6374','EPSG','3906','EPSG','1207');
INSERT INTO "conversion" VALUES('EPSG','6375','Ukraine TM zone 8','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',24.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',300000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9722','conversion','EPSG','6375','EPSG','3907','EPSG','1207');
INSERT INTO "conversion" VALUES('EPSG','6376','Ukraine TM zone 9','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',27.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',300000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9723','conversion','EPSG','6376','EPSG','3908','EPSG','1207');
INSERT INTO "conversion" VALUES('EPSG','6377','Ukraine TM zone 10','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',30.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',300000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9724','conversion','EPSG','6377','EPSG','3909','EPSG','1207');
INSERT INTO "conversion" VALUES('EPSG','6378','Ukraine TM zone 11','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',33.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',300000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9725','conversion','EPSG','6378','EPSG','3910','EPSG','1207');
INSERT INTO "conversion" VALUES('EPSG','6379','Ukraine TM zone 12','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',36.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',300000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9726','conversion','EPSG','6379','EPSG','3912','EPSG','1207');
INSERT INTO "conversion" VALUES('EPSG','6380','Ukraine TM zone 13','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',39.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',300000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9727','conversion','EPSG','6380','EPSG','3913','EPSG','1207');
INSERT INTO "conversion" VALUES('EPSG','6390','Cayman Islands LCC (ft)','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',19.2,'EPSG','9110','EPSG','8822','Longitude of false origin',-80.34,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',19.2,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',19.42,'EPSG','9110','EPSG','8826','Easting at false origin',2950000.0,'EPSG','9002','EPSG','8827','Northing at false origin',1900000.0,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9730','conversion','EPSG','6390','EPSG','1063','EPSG','1027');
INSERT INTO "conversion" VALUES('EPSG','6645','Quebec Albers Projection','','EPSG','9822','Albers Equal Area','EPSG','8821','Latitude of false origin',44.0,'EPSG','9110','EPSG','8822','Longitude of false origin',-68.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',60.0,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',46.0,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9732','conversion','EPSG','6645','EPSG','1368','EPSG','1210');
INSERT INTO "conversion" VALUES('EPSG','6702','TM 60 SW','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-60.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9737','conversion','EPSG','6702','EPSG','4172','EPSG','1136');
INSERT INTO "conversion" VALUES('EPSG','6716','Christmas Island Grid 1992','Replaces CIG85. Replaced by CIG94.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',105.373,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000024,'EPSG','9201','EPSG','8806','False easting',50000.0,'EPSG','9001','EPSG','8807','False northing',1300000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9743','conversion','EPSG','6716','EPSG','4169','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','6717','Christmas Island Grid 1994','Replaces CIG85 and CIG92. Replaced by CIG2020.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',105.373,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00002514,'EPSG','9201','EPSG','8806','False easting',50000.0,'EPSG','9001','EPSG','8807','False northing',1300000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9744','conversion','EPSG','6717','EPSG','4169','EPSG','1054');
INSERT INTO "conversion" VALUES('EPSG','6718','Cocos Island Grid 1992','Replaces CKIG65. Replaced by CKIG94.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',96.523,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',50000.0,'EPSG','9001','EPSG','8807','False northing',1400000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9745','conversion','EPSG','6718','EPSG','1069','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','6719','Cocos Island Grid 1994','Replaces CKIG92. Replaced by CKIG2020.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',96.523,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.99999387,'EPSG','9201','EPSG','8806','False easting',50000.0,'EPSG','9001','EPSG','8807','False northing',1500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9746','conversion','EPSG','6719','EPSG','1069','EPSG','1054');
INSERT INTO "conversion" VALUES('EPSG','6725','Map Grid of Australia zone 41','Grid convergence uses opposite sign convention to UTM.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',63.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','9748','conversion','EPSG','6725','EPSG','4173','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','6726','Map Grid of Australia zone 42','Grid convergence uses opposite sign convention to UTM.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',69.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','9749','conversion','EPSG','6726','EPSG','4181','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','6727','Map Grid of Australia zone 43','Grid convergence uses opposite sign convention to UTM.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',75.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','9750','conversion','EPSG','6727','EPSG','4184','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','6728','Map Grid of Australia zone 44','Grid convergence uses opposite sign convention to UTM.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',81.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','9751','conversion','EPSG','6728','EPSG','4185','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','6729','Map Grid of Australia zone 46','Grid convergence uses opposite sign convention to UTM.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',93.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9752','conversion','EPSG','6729','EPSG','4189','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','6730','Map Grid of Australia zone 47','Grid convergence uses opposite sign convention to UTM.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',99.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9753','conversion','EPSG','6730','EPSG','4190','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','6731','Map Grid of Australia zone 59','Grid convergence uses opposite sign convention to UTM.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',171.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9754','conversion','EPSG','6731','EPSG','4179','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','6741','Oregon Baker zone (meter)','See code 6742 for equivalent non-metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',44.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-117.5,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00016,'EPSG','9201','EPSG','8806','False easting',40000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9757','conversion','EPSG','6741','EPSG','4180','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','6742','Oregon Baker zone (international foot)','See code 6741 for equivalent metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',44.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-117.5,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00016,'EPSG','9201','EPSG','8806','False easting',131233.5958,'EPSG','9002','EPSG','8807','False northing',0.0,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9758','conversion','EPSG','6742','EPSG','4180','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','6743','Oregon Bend-Klamath Falls zone (meter)','See code 6744 for equivalent non-metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',41.45,'EPSG','9110','EPSG','8802','Longitude of natural origin',-121.45,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0002,'EPSG','9201','EPSG','8806','False easting',80000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9759','conversion','EPSG','6743','EPSG','4192','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','6744','Oregon Bend-Klamath Falls zone (international foot)','See code 6743 for equivalent metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',41.45,'EPSG','9110','EPSG','8802','Longitude of natural origin',-121.45,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0002,'EPSG','9201','EPSG','8806','False easting',262467.1916,'EPSG','9002','EPSG','8807','False northing',0.0,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9760','conversion','EPSG','6744','EPSG','4192','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','6745','Oregon Bend-Redmond-Prineville zone (meter)','See code 6746 for equivalent non-metric definition.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',44.4,'EPSG','9110','EPSG','8802','Longitude of natural origin',-121.15,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00012,'EPSG','9201','EPSG','8806','False easting',80000.0,'EPSG','9001','EPSG','8807','False northing',130000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9761','conversion','EPSG','6745','EPSG','4195','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','6746','Oregon Bend-Redmond-Prineville zone (international foot)','See code 6745 for equivalent metric definition.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',44.4,'EPSG','9110','EPSG','8802','Longitude of natural origin',-121.15,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00012,'EPSG','9201','EPSG','8806','False easting',262467.1916,'EPSG','9002','EPSG','8807','False northing',426509.1864,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9762','conversion','EPSG','6746','EPSG','4195','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','6747','Oregon Bend-Burns zone (meter)','See code 6748 for equivalent non-metric definition.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',43.4,'EPSG','9110','EPSG','8802','Longitude of natural origin',-119.45,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0002,'EPSG','9201','EPSG','8806','False easting',120000.0,'EPSG','9001','EPSG','8807','False northing',60000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9763','conversion','EPSG','6747','EPSG','4182','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','6748','Oregon Bend-Burns zone (international foot)','See code 6747 for equivalent metric definition.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',43.4,'EPSG','9110','EPSG','8802','Longitude of natural origin',-119.45,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0002,'EPSG','9201','EPSG','8806','False easting',393700.7874,'EPSG','9002','EPSG','8807','False northing',196850.3937,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9764','conversion','EPSG','6748','EPSG','4182','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','6749','Oregon Canyonville-Grants Pass zone (meter)','See code 6750 for equivalent non-metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',42.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-123.2,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00007,'EPSG','9201','EPSG','8806','False easting',40000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9765','conversion','EPSG','6749','EPSG','4199','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','6750','Oregon Canyonville-Grants Pass zone (international foot)','See code 6749 for equivalent metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',42.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-123.2,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00007,'EPSG','9201','EPSG','8806','False easting',131233.5958,'EPSG','9002','EPSG','8807','False northing',0.0,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9766','conversion','EPSG','6750','EPSG','4199','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','6751','Oregon Columbia River East zone (meter)','See code 6752 for equivalent non-metric definition.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',45.4,'EPSG','9110','EPSG','8802','Longitude of natural origin',-120.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000008,'EPSG','9201','EPSG','8806','False easting',150000.0,'EPSG','9001','EPSG','8807','False northing',30000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9767','conversion','EPSG','6751','EPSG','4200','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','6752','Oregon Columbia River East zone (international foot)','See code 6751 for equivalent metric definition.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',45.4,'EPSG','9110','EPSG','8802','Longitude of natural origin',-120.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000008,'EPSG','9201','EPSG','8806','False easting',492125.9843,'EPSG','9002','EPSG','8807','False northing',98425.1969,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9768','conversion','EPSG','6752','EPSG','4200','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','6753','Oregon Columbia River West zone (meter)','If using Hotine Oblique Mercator (variant B) method (code 9815), Ec=168300.419 m, Nc=185673.833 m. See code 6754 for equivalent non-metric definition.','EPSG','9812','Hotine Oblique Mercator (variant A)','EPSG','8811','Latitude of projection centre',45.55,'EPSG','9110','EPSG','8812','Longitude of projection centre',-123.0,'EPSG','9110','EPSG','8813','Azimuth at projection centre',295.0,'EPSG','9102','EPSG','8814','Angle from Rectified to Skew Grid',295.0,'EPSG','9102','EPSG','8815','Scale factor at projection centre',1.0,'EPSG','9201','EPSG','8806','False easting',7000000.0,'EPSG','9001','EPSG','8807','False northing',-3000000.0,'EPSG','9001',0);
INSERT INTO "usage" VALUES('EPSG','9769','conversion','EPSG','6753','EPSG','4202','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','6754','Oregon Columbia River West zone (international foot)','If using Hotine Oblique Mercator (variant B) method (code 9815), Ec=552166.730 ft, Nc=609166.118 ft. See code 6753 for equivalent metric definition.','EPSG','9812','Hotine Oblique Mercator (variant A)','EPSG','8811','Latitude of projection centre',45.55,'EPSG','9110','EPSG','8812','Longitude of projection centre',-123.0,'EPSG','9110','EPSG','8813','Azimuth at projection centre',295.0,'EPSG','9102','EPSG','8814','Angle from Rectified to Skew Grid',295.0,'EPSG','9102','EPSG','8815','Scale factor at projection centre',1.0,'EPSG','9201','EPSG','8806','False easting',22965879.2651,'EPSG','9002','EPSG','8807','False northing',-9842519.685,'EPSG','9002',0);
INSERT INTO "usage" VALUES('EPSG','9770','conversion','EPSG','6754','EPSG','4202','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','6755','Oregon Cottage Grove-Canyonville zone (meter)','See code 6756 for equivalent non-metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',42.5,'EPSG','9110','EPSG','8802','Longitude of natural origin',-123.2,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000023,'EPSG','9201','EPSG','8806','False easting',50000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9771','conversion','EPSG','6755','EPSG','4203','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','6756','Oregon Cottage Grove-Canyonville zone (international foot)','See code 6755 for equivalent metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',42.5,'EPSG','9110','EPSG','8802','Longitude of natural origin',-123.2,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000023,'EPSG','9201','EPSG','8806','False easting',164041.9948,'EPSG','9002','EPSG','8807','False northing',0.0,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9772','conversion','EPSG','6756','EPSG','4203','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','6757','Oregon Dufur-Madras zone (meter)','See code 6758 for equivalent non-metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',44.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-121.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00011,'EPSG','9201','EPSG','8806','False easting',80000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9773','conversion','EPSG','6757','EPSG','4204','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','6758','Oregon Dufur-Madras zone (international foot)','See code 6757 for equivalent metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',44.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-121.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00011,'EPSG','9201','EPSG','8806','False easting',262467.1916,'EPSG','9002','EPSG','8807','False northing',0.0,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9774','conversion','EPSG','6758','EPSG','4204','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','6759','Oregon Eugene zone (meter)','See code 6760 for equivalent non-metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',43.45,'EPSG','9110','EPSG','8802','Longitude of natural origin',-123.1,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000015,'EPSG','9201','EPSG','8806','False easting',50000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9775','conversion','EPSG','6759','EPSG','4197','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','6760','Oregon Eugene zone (international foot)','See code 6759 for equivalent metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',43.45,'EPSG','9110','EPSG','8802','Longitude of natural origin',-123.1,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000015,'EPSG','9201','EPSG','8806','False easting',164041.9948,'EPSG','9002','EPSG','8807','False northing',0.0,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9776','conversion','EPSG','6760','EPSG','4197','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','6761','Oregon Grants Pass-Ashland zone (meter)','See code 6762 for equivalent non-metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',41.45,'EPSG','9110','EPSG','8802','Longitude of natural origin',-123.2,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000043,'EPSG','9201','EPSG','8806','False easting',50000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9777','conversion','EPSG','6761','EPSG','4198','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','6762','Oregon Grants Pass-Ashland zone (international foot)','See code 6761 for equivalent metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',41.45,'EPSG','9110','EPSG','8802','Longitude of natural origin',-123.2,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000043,'EPSG','9201','EPSG','8806','False easting',164041.9948,'EPSG','9002','EPSG','8807','False northing',0.0,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9778','conversion','EPSG','6762','EPSG','4198','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','6763','Oregon Gresham-Warm Springs zone (meter)','See code 6764 for equivalent non-metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',45.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-122.2,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00005,'EPSG','9201','EPSG','8806','False easting',10000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9779','conversion','EPSG','6763','EPSG','4201','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','6764','Oregon Gresham-Warm Springs zone (international foot)','See code 6763 for equivalent metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',45.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-122.2,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00005,'EPSG','9201','EPSG','8806','False easting',32808.399,'EPSG','9002','EPSG','8807','False northing',0.0,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9780','conversion','EPSG','6764','EPSG','4201','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','6765','Oregon La Grande zone (meter)','See code 6766 for equivalent non-metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',45.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-118.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.00013,'EPSG','9201','EPSG','8806','False easting',40000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9781','conversion','EPSG','6765','EPSG','4206','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','6766','Oregon La Grande zone (international foot)','See code 6765 for equivalent metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',45.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-118.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.00013,'EPSG','9201','EPSG','8806','False easting',131233.5958,'EPSG','9002','EPSG','8807','False northing',0.0,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9782','conversion','EPSG','6766','EPSG','4206','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','6767','Oregon Ontario zone (meter)','See code 6768 for equivalent non-metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',43.15,'EPSG','9110','EPSG','8802','Longitude of natural origin',-117.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0001,'EPSG','9201','EPSG','8806','False easting',80000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9783','conversion','EPSG','6767','EPSG','4207','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','6768','Oregon Ontario zone (international foot)','See code 6767 for equivalent metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',43.15,'EPSG','9110','EPSG','8802','Longitude of natural origin',-117.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0001,'EPSG','9201','EPSG','8806','False easting',262467.1916,'EPSG','9002','EPSG','8807','False northing',0.0,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9784','conversion','EPSG','6768','EPSG','4207','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','6769','Oregon Coast zone (meter)','If using Hotine Oblique Mercator (variant B) method (code 9815), Ec=134743.332 m, Nc=369139.028 m. See code 6770 for equivalent non-metric definition.','EPSG','9812','Hotine Oblique Mercator (variant A)','EPSG','8811','Latitude of projection centre',44.45,'EPSG','9110','EPSG','8812','Longitude of projection centre',-124.03,'EPSG','9110','EPSG','8813','Azimuth at projection centre',5.0,'EPSG','9102','EPSG','8814','Angle from Rectified to Skew Grid',5.0,'EPSG','9102','EPSG','8815','Scale factor at projection centre',1.0,'EPSG','9201','EPSG','8806','False easting',-300000.0,'EPSG','9001','EPSG','8807','False northing',-4600000.0,'EPSG','9001',0);
INSERT INTO "usage" VALUES('EPSG','9785','conversion','EPSG','6769','EPSG','4208','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','6770','Oregon Coast zone (international foot)','If using Hotine Oblique Mercator (variant B) method (code 9815), Ec=442071.300 ft, Nc=1211086.049 ft. See code 6769 for equivalent metric definition.','EPSG','9812','Hotine Oblique Mercator (variant A)','EPSG','8811','Latitude of projection centre',44.45,'EPSG','9110','EPSG','8812','Longitude of projection centre',-124.03,'EPSG','9110','EPSG','8813','Azimuth at projection centre',5.0,'EPSG','9102','EPSG','8814','Angle from Rectified to Skew Grid',5.0,'EPSG','9102','EPSG','8815','Scale factor at projection centre',1.0,'EPSG','9201','EPSG','8806','False easting',-984251.9685,'EPSG','9002','EPSG','8807','False northing',-15091863.5171,'EPSG','9002',0);
INSERT INTO "usage" VALUES('EPSG','9786','conversion','EPSG','6770','EPSG','4208','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','6771','Oregon Pendleton zone (meter)','See code 6772 for equivalent non-metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',45.15,'EPSG','9110','EPSG','8802','Longitude of natural origin',-119.1,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000045,'EPSG','9201','EPSG','8806','False easting',60000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9787','conversion','EPSG','6771','EPSG','4209','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','6772','Oregon Pendleton zone (international foot)','See code 6771 for equivalent metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',45.15,'EPSG','9110','EPSG','8802','Longitude of natural origin',-119.1,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000045,'EPSG','9201','EPSG','8806','False easting',196850.3937,'EPSG','9002','EPSG','8807','False northing',0.0,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9788','conversion','EPSG','6772','EPSG','4209','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','6773','Oregon Pendleton-La Grande zone (meter)','See code 6774 for equivalent non-metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',45.05,'EPSG','9110','EPSG','8802','Longitude of natural origin',-118.2,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000175,'EPSG','9201','EPSG','8806','False easting',30000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9789','conversion','EPSG','6773','EPSG','4210','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','6774','Oregon Pendleton-La Grande zone (international foot)','See code 6773 for equivalent metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',45.05,'EPSG','9110','EPSG','8802','Longitude of natural origin',-118.2,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000175,'EPSG','9201','EPSG','8806','False easting',98425.1969,'EPSG','9002','EPSG','8807','False northing',0.0,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9790','conversion','EPSG','6774','EPSG','4210','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','6775','Oregon Portland zone (meter)','See code 6776 for equivalent non-metric definition.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',45.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-122.45,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000002,'EPSG','9201','EPSG','8806','False easting',100000.0,'EPSG','9001','EPSG','8807','False northing',50000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9791','conversion','EPSG','6775','EPSG','4211','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','6776','Oregon Portland zone (international foot)','See code 6775 for equivalent metric definition.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',45.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-122.45,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000002,'EPSG','9201','EPSG','8806','False easting',328083.9895,'EPSG','9002','EPSG','8807','False northing',164041.9948,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9792','conversion','EPSG','6776','EPSG','4211','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','6777','Oregon Salem zone (meter)','See code 6778 for equivalent non-metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',44.2,'EPSG','9110','EPSG','8802','Longitude of natural origin',-123.05,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00001,'EPSG','9201','EPSG','8806','False easting',50000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9793','conversion','EPSG','6777','EPSG','4212','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','6778','Oregon Salem zone (international foot)','See code 6777 for equivalent metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',44.2,'EPSG','9110','EPSG','8802','Longitude of natural origin',-123.05,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00001,'EPSG','9201','EPSG','8806','False easting',164041.9948,'EPSG','9002','EPSG','8807','False northing',0.0,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9794','conversion','EPSG','6778','EPSG','4212','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','6779','Oregon Santiam Pass zone (meter)','See code 6780 for equivalent non-metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',44.05,'EPSG','9110','EPSG','8802','Longitude of natural origin',-122.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000155,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9795','conversion','EPSG','6779','EPSG','4213','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','6780','Oregon Santiam Pass zone (international foot)','See code 6779 for equivalent metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',44.05,'EPSG','9110','EPSG','8802','Longitude of natural origin',-122.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000155,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9002','EPSG','8807','False northing',0.0,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9796','conversion','EPSG','6780','EPSG','4213','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','6869','Albania TM 2010','Planimetric component of Albanian Geodetic Reference Framework (KRGJSH) 2010. Albania LCC 2010 projection (code 6961) used for mapping at scales of 1:500,000 and smaller.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',20.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9800','conversion','EPSG','6869','EPSG','3212','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','6877','Italy zone','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',12.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9985,'EPSG','9201','EPSG','8806','False easting',7000000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9804','conversion','EPSG','6877','EPSG','1127','EPSG','1241');
INSERT INTO "conversion" VALUES('EPSG','6878','Italy zone 12','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',12.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',3000000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9805','conversion','EPSG','6878','EPSG','1127','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','6920','Kansas DOT Lambert (meter)','See code 6921 for equivalent non-metric definition.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',36.0,'EPSG','9110','EPSG','8822','Longitude of false origin',-98.15,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',39.3,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',37.3,'EPSG','9110','EPSG','8826','Easting at false origin',400000.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9832','conversion','EPSG','6920','EPSG','1385','EPSG','1135');
INSERT INTO "conversion" VALUES('EPSG','6921','Kansas DOT Lambert (US survey foot)','See code 6920 for equivalent metric definition.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',36.0,'EPSG','9110','EPSG','8822','Longitude of false origin',-98.15,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',39.3,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',37.3,'EPSG','9110','EPSG','8826','Easting at false origin',1312333.3333,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9833','conversion','EPSG','6921','EPSG','1385','EPSG','1135');
INSERT INTO "conversion" VALUES('EPSG','6928','US NSIDC EASE-Grid 2.0 Global','','EPSG','9835','Lambert Cylindrical Equal Area','EPSG','8823','Latitude of 1st standard parallel',30.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',0.0,'EPSG','9102','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9835','conversion','EPSG','6928','EPSG','3463','EPSG','1195');
INSERT INTO "conversion" VALUES('EPSG','6929','US NSIDC EASE-Grid 2.0 North','','EPSG','9820','Lambert Azimuthal Equal Area','EPSG','8801','Latitude of natural origin',90.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',0.0,'EPSG','9102','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9836','conversion','EPSG','6929','EPSG','3475','EPSG','1195');
INSERT INTO "conversion" VALUES('EPSG','6930','US NSIDC EASE-Grid 2.0 South','','EPSG','9820','Lambert Azimuthal Equal Area','EPSG','8801','Latitude of natural origin',-90.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',0.0,'EPSG','9102','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9837','conversion','EPSG','6930','EPSG','3474','EPSG','1195');
INSERT INTO "conversion" VALUES('EPSG','6952','Vietnam TM-3 zone 481','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',102.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','9855','conversion','EPSG','6952','EPSG','4193','EPSG','1207');
INSERT INTO "conversion" VALUES('EPSG','6953','Vietnam TM-3 zone 482','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',105.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','9856','conversion','EPSG','6953','EPSG','4215','EPSG','1207');
INSERT INTO "conversion" VALUES('EPSG','6954','Vietnam TM-3 zone 491','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',108.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','9857','conversion','EPSG','6954','EPSG','4217','EPSG','1207');
INSERT INTO "conversion" VALUES('EPSG','6955','Vietnam TM-3 Da Nang zone','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',107.45,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','9858','conversion','EPSG','6955','EPSG','4218','EPSG','1207');
INSERT INTO "conversion" VALUES('EPSG','6961','Albania LCC 2010','Albania TM 2010 projection (code 6869) used for applications at scales larger than 1:500,000.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',41.0,'EPSG','9102','EPSG','8822','Longitude of false origin',20.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',39.0,'EPSG','9102','EPSG','8824','Latitude of 2nd standard parallel',43.0,'EPSG','9102','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9860','conversion','EPSG','6961','EPSG','1025','EPSG','1107');
INSERT INTO "conversion" VALUES('EPSG','6965','Michigan CS27 North zone','','EPSG','1051','Lambert Conic Conformal (2SP Michigan)','EPSG','8821','Latitude of false origin',44.47,'EPSG','9110','EPSG','8822','Longitude of false origin',-87.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',45.29,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',47.05,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003','EPSG','1038','Ellipsoid scaling factor',1.0000382,'EPSG','9201',0);
INSERT INTO "usage" VALUES('EPSG','9863','conversion','EPSG','6965','EPSG','1723','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','6994','City and County of San Francisco CS13 (meters)','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',37.75,'EPSG','9102','EPSG','8802','Longitude of natural origin',-122.45,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.000007,'EPSG','9202','EPSG','8806','False easting',48000.0,'EPSG','9001','EPSG','8807','False northing',24000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','9877','conversion','EPSG','6994','EPSG','4228','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','6995','City and County of San Francisco CS13 (US Survey feet)','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',37.75,'EPSG','9102','EPSG','8802','Longitude of natural origin',-122.45,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.000007,'EPSG','9202','EPSG','8806','False easting',157480.0,'EPSG','9003','EPSG','8807','False northing',78740.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','9878','conversion','EPSG','6995','EPSG','4228','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','7043','Iowa regional zone 1 Spencer','','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',43.12,'EPSG','9110','EPSG','8802','Longitude of natural origin',-95.15,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000052,'EPSG','9201','EPSG','8806','False easting',11500000.0,'EPSG','9003','EPSG','8807','False northing',9600000.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9912','conversion','EPSG','7043','EPSG','4164','EPSG','1196');
INSERT INTO "conversion" VALUES('EPSG','7044','Iowa regional zone 2 Mason City','','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',43.1,'EPSG','9110','EPSG','8802','Longitude of natural origin',-92.45,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000043,'EPSG','9201','EPSG','8806','False easting',12500000.0,'EPSG','9003','EPSG','8807','False northing',9800000.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9913','conversion','EPSG','7044','EPSG','4219','EPSG','1196');
INSERT INTO "conversion" VALUES('EPSG','7045','Iowa regional zone 3 Elkader','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',40.15,'EPSG','9110','EPSG','8802','Longitude of natural origin',-91.12,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000035,'EPSG','9201','EPSG','8806','False easting',13500000.0,'EPSG','9003','EPSG','8807','False northing',8300000.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9914','conversion','EPSG','7045','EPSG','4230','EPSG','1196');
INSERT INTO "conversion" VALUES('EPSG','7046','Iowa regional zone 4 Sioux City-Iowa Falls','','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',42.32,'EPSG','9110','EPSG','8802','Longitude of natural origin',-94.5,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000045,'EPSG','9201','EPSG','8806','False easting',14500000.0,'EPSG','9003','EPSG','8807','False northing',8600000.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9915','conversion','EPSG','7046','EPSG','4233','EPSG','1196');
INSERT INTO "conversion" VALUES('EPSG','7047','Iowa regional zone 5 Waterloo','','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',42.39,'EPSG','9110','EPSG','8802','Longitude of natural origin',-92.15,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000032,'EPSG','9201','EPSG','8806','False easting',15500000.0,'EPSG','9003','EPSG','8807','False northing',8900000.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9916','conversion','EPSG','7047','EPSG','4234','EPSG','1196');
INSERT INTO "conversion" VALUES('EPSG','7048','Iowa regional zone 6 Council Bluffs','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',40.15,'EPSG','9110','EPSG','8802','Longitude of natural origin',-95.44,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000039,'EPSG','9201','EPSG','8806','False easting',16500000.0,'EPSG','9003','EPSG','8807','False northing',6600000.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9917','conversion','EPSG','7048','EPSG','4235','EPSG','1196');
INSERT INTO "conversion" VALUES('EPSG','7049','Iowa regional zone 7 Carroll-Atlantic','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',40.15,'EPSG','9110','EPSG','8802','Longitude of natural origin',-94.38,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000045,'EPSG','9201','EPSG','8806','False easting',17500000.0,'EPSG','9003','EPSG','8807','False northing',6800000.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9918','conversion','EPSG','7049','EPSG','4236','EPSG','1196');
INSERT INTO "conversion" VALUES('EPSG','7050','Iowa regional zone 8 Ames-Des Moines','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',40.15,'EPSG','9110','EPSG','8802','Longitude of natural origin',-93.43,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000033,'EPSG','9201','EPSG','8806','False easting',18500000.0,'EPSG','9003','EPSG','8807','False northing',7000000.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9919','conversion','EPSG','7050','EPSG','4237','EPSG','1196');
INSERT INTO "conversion" VALUES('EPSG','7051','Iowa regional zone 9 Newton','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',40.15,'EPSG','9110','EPSG','8802','Longitude of natural origin',-92.49,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000027,'EPSG','9201','EPSG','8806','False easting',19500000.0,'EPSG','9003','EPSG','8807','False northing',7200000.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9920','conversion','EPSG','7051','EPSG','4239','EPSG','1196');
INSERT INTO "conversion" VALUES('EPSG','7052','Iowa regional zone 10 Cedar Rapids','','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',41.5,'EPSG','9110','EPSG','8802','Longitude of natural origin',-91.4,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00002,'EPSG','9201','EPSG','8806','False easting',20500000.0,'EPSG','9003','EPSG','8807','False northing',8000000.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9921','conversion','EPSG','7052','EPSG','4240','EPSG','1196');
INSERT INTO "conversion" VALUES('EPSG','7053','Iowa regional zone 11 Dubuque-Davenport','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',40.15,'EPSG','9110','EPSG','8802','Longitude of natural origin',-90.32,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000027,'EPSG','9201','EPSG','8806','False easting',21500000.0,'EPSG','9003','EPSG','8807','False northing',7600000.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9922','conversion','EPSG','7053','EPSG','4241','EPSG','1196');
INSERT INTO "conversion" VALUES('EPSG','7054','Iowa regional zone 12 Red Oak-Ottumwa','','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',40.55,'EPSG','9110','EPSG','8802','Longitude of natural origin',-93.45,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000037,'EPSG','9201','EPSG','8806','False easting',22500000.0,'EPSG','9003','EPSG','8807','False northing',6200000.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9923','conversion','EPSG','7054','EPSG','4242','EPSG','1196');
INSERT INTO "conversion" VALUES('EPSG','7055','Iowa regional zone 13 Fairfield','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',40.15,'EPSG','9110','EPSG','8802','Longitude of natural origin',-91.55,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00002,'EPSG','9201','EPSG','8806','False easting',23500000.0,'EPSG','9003','EPSG','8807','False northing',6400000.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9924','conversion','EPSG','7055','EPSG','4243','EPSG','1196');
INSERT INTO "conversion" VALUES('EPSG','7056','Iowa regional zone 14 Burlington','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',40.15,'EPSG','9110','EPSG','8802','Longitude of natural origin',-91.15,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000018,'EPSG','9201','EPSG','8806','False easting',24500000.0,'EPSG','9003','EPSG','8807','False northing',6200000.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9925','conversion','EPSG','7056','EPSG','4244','EPSG','1196');
INSERT INTO "conversion" VALUES('EPSG','7089','Montana Blackfeet St Mary Valley (meter)','See code 7090 for equivalent non-metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',48.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-112.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00016,'EPSG','9201','EPSG','8806','False easting',150000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9927','conversion','EPSG','7089','EPSG','4310','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7090','Montana Blackfeet St Mary Valley (international foot)','For authoritative metric definition see code 7089. Working unit is International feet (note: not US Survey feet).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',48.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-112.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00016,'EPSG','9201','EPSG','8806','False easting',492125.9843,'EPSG','9002','EPSG','8807','False northing',0.0,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9928','conversion','EPSG','7090','EPSG','4310','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7091','Montana Blackfeet (meter)','See code 7092 for equivalent non-metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',48.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-112.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00019,'EPSG','9201','EPSG','8806','False easting',100000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9929','conversion','EPSG','7091','EPSG','4311','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7092','Montana Blackfeet (international foot)','For authoritative metric definition see code 7091. Working unit is International feet (note: not US Survey feet).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',48.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-112.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00019,'EPSG','9201','EPSG','8806','False easting',328083.9895,'EPSG','9002','EPSG','8807','False northing',0.0,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9930','conversion','EPSG','7092','EPSG','4311','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7093','Montana Milk River (meter)','See code 7094 for equivalent non-metric definition.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',48.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-111.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000145,'EPSG','9201','EPSG','8806','False easting',150000.0,'EPSG','9001','EPSG','8807','False northing',200000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9931','conversion','EPSG','7093','EPSG','4312','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7094','Montana Milk River (international foot)','For authoritative metric definition see code 7093. Working unit is International feet (note: not US Survey feet).','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',48.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-111.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000145,'EPSG','9201','EPSG','8806','False easting',492125.9843,'EPSG','9002','EPSG','8807','False northing',656167.979,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9932','conversion','EPSG','7094','EPSG','4312','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7095','Montana Fort Belknap (meter)','See code 7096 for equivalent non-metric definition.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',48.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-108.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00012,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',150000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9933','conversion','EPSG','7095','EPSG','4313','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7096','Montana Fort Belknap (international foot)','For authoritative metric definition see code 7095. Working unit is International feet (note: not US Survey feet).','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',48.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-108.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00012,'EPSG','9201','EPSG','8806','False easting',656167.979,'EPSG','9002','EPSG','8807','False northing',492125.9843,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9934','conversion','EPSG','7096','EPSG','4313','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7097','Montana Fort Peck Assiniboine (meter)','See code 7098 for equivalent non-metric definition.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',48.2,'EPSG','9110','EPSG','8802','Longitude of natural origin',-105.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00012,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',100000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9935','conversion','EPSG','7097','EPSG','4314','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7098','Montana Fort Peck Assiniboine (international foot)','For authoritative metric definition see code 7097. Working unit is International feet (note: not US Survey feet).','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',48.2,'EPSG','9110','EPSG','8802','Longitude of natural origin',-105.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00012,'EPSG','9201','EPSG','8806','False easting',656167.979,'EPSG','9002','EPSG','8807','False northing',328083.9895,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9936','conversion','EPSG','7098','EPSG','4314','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7099','Montana Fort Peck Sioux (meter)','See code 7100 for equivalent non-metric definition.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',48.2,'EPSG','9110','EPSG','8802','Longitude of natural origin',-105.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00009,'EPSG','9201','EPSG','8806','False easting',100000.0,'EPSG','9001','EPSG','8807','False northing',50000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9937','conversion','EPSG','7099','EPSG','4315','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7100','Montana Fort Peck Sioux (international foot)','For authoritative metric definition see code 7099. Working unit is International feet (note: not US Survey feet).','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',48.2,'EPSG','9110','EPSG','8802','Longitude of natural origin',-105.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00009,'EPSG','9201','EPSG','8806','False easting',328083.9895,'EPSG','9002','EPSG','8807','False northing',164041.9938,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9938','conversion','EPSG','7100','EPSG','4315','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7101','Montana Crow (meter)','See code 7102 for equivalent non-metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',44.45,'EPSG','9110','EPSG','8802','Longitude of natural origin',-107.45,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000148,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9939','conversion','EPSG','7101','EPSG','4316','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7102','Montana Crow (international foot)','For authoritative metric definition see code 7101. Working unit is International feet (note: not US Survey feet).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',44.45,'EPSG','9110','EPSG','8802','Longitude of natural origin',-107.45,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000148,'EPSG','9201','EPSG','8806','False easting',656167.979,'EPSG','9002','EPSG','8807','False northing',0.0,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9940','conversion','EPSG','7102','EPSG','4316','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7103','Montana Bobcat (meter)','See code 7104 for equivalent non-metric definition.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',46.15,'EPSG','9110','EPSG','8802','Longitude of natural origin',-111.15,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000185,'EPSG','9201','EPSG','8806','False easting',100000.0,'EPSG','9001','EPSG','8807','False northing',100000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9941','conversion','EPSG','7103','EPSG','4317','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7104','Montana Bobcat (international foot)','For authoritative metric definition see code 7103. Working unit is International feet (note: not US Survey feet).','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',46.15,'EPSG','9110','EPSG','8802','Longitude of natural origin',-111.15,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000185,'EPSG','9201','EPSG','8806','False easting',328083.9895,'EPSG','9002','EPSG','8807','False northing',328083.9895,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9942','conversion','EPSG','7104','EPSG','4317','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7105','Montana Billings (meter)','See code 7106 for equivalent non-metric definition.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',45.47,'EPSG','9110','EPSG','8802','Longitude of natural origin',-108.25,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0001515,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',50000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9943','conversion','EPSG','7105','EPSG','4318','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7106','Montana Billings (international foot)','For authoritative metric definition see code 7105. Working unit is International feet (note: not US Survey feet).','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',45.47,'EPSG','9110','EPSG','8802','Longitude of natural origin',-108.25,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0001515,'EPSG','9201','EPSG','8806','False easting',656167.979,'EPSG','9002','EPSG','8807','False northing',164041.9948,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9944','conversion','EPSG','7106','EPSG','4318','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7107','Wyoming Wind River (meter)','See code 7108 for equivalent non-metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',42.4,'EPSG','9110','EPSG','8802','Longitude of natural origin',-108.2,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00024,'EPSG','9201','EPSG','8806','False easting',100000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9945','conversion','EPSG','7107','EPSG','4319','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7108','Wyoming Wind River (US survey foot)','For authoritative metric definition see code 7107. Working unit is US Survey feet.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',42.4,'EPSG','9110','EPSG','8802','Longitude of natural origin',-108.2,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00024,'EPSG','9201','EPSG','8806','False easting',328083.3333,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9946','conversion','EPSG','7108','EPSG','4319','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7129','City and County of San Francisco CS13 (meter)','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',37.75,'EPSG','9102','EPSG','8802','Longitude of natural origin',-122.45,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.000007,'EPSG','9201','EPSG','8806','False easting',48000.0,'EPSG','9001','EPSG','8807','False northing',24000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9947','conversion','EPSG','7129','EPSG','4228','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','7130','City and County of San Francisco CS13 (US survey foot)','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',37.75,'EPSG','9102','EPSG','8802','Longitude of natural origin',-122.45,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.000007,'EPSG','9201','EPSG','8806','False easting',157480.0,'EPSG','9003','EPSG','8807','False northing',78740.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9948','conversion','EPSG','7130','EPSG','4228','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','7141','Palestine Grid modified','Modification of Palestine Grid (proj 18201) using TM rather than Cassini projection method. The difference in coordinates caused by change of method is under 2m within Israel but can be over 200m in eastern Jordan.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',31.4402749,'EPSG','9110','EPSG','8802','Longitude of natural origin',35.124349,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',170251.555,'EPSG','9001','EPSG','8807','False northing',126867.909,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9950','conversion','EPSG','7141','EPSG','1356','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','7143','InGCS Adams (m)','For equivalent non-metric definition see proj code 7144.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',40.33,'EPSG','9110','EPSG','8802','Longitude of natural origin',-84.57,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000034,'EPSG','9201','EPSG','8806','False easting',240000.0,'EPSG','9001','EPSG','8807','False northing',36000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9951','conversion','EPSG','7143','EPSG','4289','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7144','InGCS Adams (ftUS)','See proj code 7143 for authoritative metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',40.33,'EPSG','9110','EPSG','8802','Longitude of natural origin',-84.57,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000034,'EPSG','9201','EPSG','8806','False easting',787400.0,'EPSG','9003','EPSG','8807','False northing',118110.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9952','conversion','EPSG','7144','EPSG','4289','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7145','InGCS Allen (m)','For equivalent non-metric definition see proj code 7146.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',40.54,'EPSG','9110','EPSG','8802','Longitude of natural origin',-85.03,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000031,'EPSG','9201','EPSG','8806','False easting',240000.0,'EPSG','9001','EPSG','8807','False northing',36000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9953','conversion','EPSG','7145','EPSG','4285','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7146','InGCS Allen (ftUS)','See proj code 7145 for authoritative metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',40.54,'EPSG','9110','EPSG','8802','Longitude of natural origin',-85.03,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000031,'EPSG','9201','EPSG','8806','False easting',787400.0,'EPSG','9003','EPSG','8807','False northing',118110.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9954','conversion','EPSG','7146','EPSG','4285','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7147','InGCS Bartholomew (m)','For equivalent non-metric definition see proj code 7148.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',39.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-85.51,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000026,'EPSG','9201','EPSG','8806','False easting',240000.0,'EPSG','9001','EPSG','8807','False northing',36000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9955','conversion','EPSG','7147','EPSG','4302','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7148','InGCS Bartholomew (ftUS)','See proj code 7147 for authoritative metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',39.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-85.51,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000026,'EPSG','9201','EPSG','8806','False easting',787400.0,'EPSG','9003','EPSG','8807','False northing',118110.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9956','conversion','EPSG','7148','EPSG','4302','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7149','InGCS Benton (m)','For equivalent non-metric definition see proj code 7150.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',40.27,'EPSG','9110','EPSG','8802','Longitude of natural origin',-87.18,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000029,'EPSG','9201','EPSG','8806','False easting',240000.0,'EPSG','9001','EPSG','8807','False northing',36000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9957','conversion','EPSG','7149','EPSG','4256','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7150','InGCS Benton (ftUS)','See proj code 7149 for authoritative metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',40.27,'EPSG','9110','EPSG','8802','Longitude of natural origin',-87.18,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000029,'EPSG','9201','EPSG','8806','False easting',787400.0,'EPSG','9003','EPSG','8807','False northing',118110.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9958','conversion','EPSG','7150','EPSG','4256','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7151','InGCS Blackford-Delaware (m)','For equivalent non-metric definition see proj code 7152. Defined by Indiana DOT as two separate zones (one for each county) each having identical projection parameter values: see info source.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',40.03,'EPSG','9110','EPSG','8802','Longitude of natural origin',-85.24,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000038,'EPSG','9201','EPSG','8806','False easting',240000.0,'EPSG','9001','EPSG','8807','False northing',36000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9959','conversion','EPSG','7151','EPSG','4291','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7152','InGCS Blackford-Delaware (ftUS)','See proj code 7151 for authoritative metric definition. Defined by Indiana DOT as two separate zones (one for each county) each having identical defining parameter values: see info source.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',40.03,'EPSG','9110','EPSG','8802','Longitude of natural origin',-85.24,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000038,'EPSG','9201','EPSG','8806','False easting',787400.0,'EPSG','9003','EPSG','8807','False northing',118110.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9960','conversion','EPSG','7152','EPSG','4291','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7153','InGCS Boone-Hendricks (m)','For equivalent non-metric definition see proj code 7154. Defined by Indiana DOT as two separate zones (one for each county) each having identical projection parameter values: see info source.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',39.36,'EPSG','9110','EPSG','8802','Longitude of natural origin',-86.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000036,'EPSG','9201','EPSG','8806','False easting',240000.0,'EPSG','9001','EPSG','8807','False northing',36000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9961','conversion','EPSG','7153','EPSG','4263','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7154','InGCS Boone-Hendricks (ftUS)','See proj code 7153 for authoritative metric definition. Defined by Indiana DOT as two separate zones (one for each county) each having identical defining parameter values: see info source.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',39.36,'EPSG','9110','EPSG','8802','Longitude of natural origin',-86.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000036,'EPSG','9201','EPSG','8806','False easting',787400.0,'EPSG','9003','EPSG','8807','False northing',118110.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9962','conversion','EPSG','7154','EPSG','4263','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7155','InGCS Brown (m)','For equivalent non-metric definition see proj code 7156.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',39.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-86.18,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00003,'EPSG','9201','EPSG','8806','False easting',240000.0,'EPSG','9001','EPSG','8807','False northing',36000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9963','conversion','EPSG','7155','EPSG','4301','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7156','InGCS Brown (ftUS)','See proj code 7155 for authoritative metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',39.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-86.18,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00003,'EPSG','9201','EPSG','8806','False easting',787400.0,'EPSG','9003','EPSG','8807','False northing',118110.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9964','conversion','EPSG','7156','EPSG','4301','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7157','InGCS Carroll (m)','For equivalent non-metric definition see proj code 7158.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',40.24,'EPSG','9110','EPSG','8802','Longitude of natural origin',-86.39,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000026,'EPSG','9201','EPSG','8806','False easting',240000.0,'EPSG','9001','EPSG','8807','False northing',36000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9965','conversion','EPSG','7157','EPSG','4258','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7158','InGCS Carroll (ftUS)','See proj code 7157 for authoritative metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',40.24,'EPSG','9110','EPSG','8802','Longitude of natural origin',-86.39,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000026,'EPSG','9201','EPSG','8806','False easting',787400.0,'EPSG','9003','EPSG','8807','False northing',118110.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9966','conversion','EPSG','7158','EPSG','4258','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7159','InGCS Cass (m)','For equivalent non-metric definition see proj code 7160.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',40.33,'EPSG','9110','EPSG','8802','Longitude of natural origin',-86.24,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000028,'EPSG','9201','EPSG','8806','False easting',240000.0,'EPSG','9001','EPSG','8807','False northing',36000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9967','conversion','EPSG','7159','EPSG','4286','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7160','InGCS Cass (ftUS)','See proj code 7159 for authoritative metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',40.33,'EPSG','9110','EPSG','8802','Longitude of natural origin',-86.24,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000028,'EPSG','9201','EPSG','8806','False easting',787400.0,'EPSG','9003','EPSG','8807','False northing',118110.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9968','conversion','EPSG','7160','EPSG','4286','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7161','InGCS Clark-Floyd-Scott (m)','For equivalent non-metric definition see proj code 7162. Defined by Indiana DOT as three separate zones (one for each county) each having identical projection parameter values: see info source.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',38.09,'EPSG','9110','EPSG','8802','Longitude of natural origin',-85.36,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000021,'EPSG','9201','EPSG','8806','False easting',240000.0,'EPSG','9001','EPSG','8807','False northing',36000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9969','conversion','EPSG','7161','EPSG','4308','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7162','InGCS Clark-Floyd-Scott (ftUS)','See proj code 7161 for authoritative metric definition. Defined by Indiana DOT as three separate zones (one for each county) each having identical defining parameter values: see info source.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',38.09,'EPSG','9110','EPSG','8802','Longitude of natural origin',-85.36,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000021,'EPSG','9201','EPSG','8806','False easting',787400.0,'EPSG','9003','EPSG','8807','False northing',118110.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9970','conversion','EPSG','7162','EPSG','4308','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7163','InGCS Clay (m)','For equivalent non-metric definition see proj code 7164.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',39.09,'EPSG','9110','EPSG','8802','Longitude of natural origin',-87.09,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000024,'EPSG','9201','EPSG','8806','False easting',240000.0,'EPSG','9001','EPSG','8807','False northing',36000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9971','conversion','EPSG','7163','EPSG','4265','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7164','InGCS Clay (ftUS)','See proj code 7163 for authoritative metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',39.09,'EPSG','9110','EPSG','8802','Longitude of natural origin',-87.09,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000024,'EPSG','9201','EPSG','8806','False easting',787400.0,'EPSG','9003','EPSG','8807','False northing',118110.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9972','conversion','EPSG','7164','EPSG','4265','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7165','InGCS Clinton (m)','For equivalent non-metric definition see proj code 7166.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',40.09,'EPSG','9110','EPSG','8802','Longitude of natural origin',-86.36,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000032,'EPSG','9201','EPSG','8806','False easting',240000.0,'EPSG','9001','EPSG','8807','False northing',36000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9973','conversion','EPSG','7165','EPSG','4260','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7166','InGCS Clinton (ftUS)','See proj code 7165 for authoritative metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',40.09,'EPSG','9110','EPSG','8802','Longitude of natural origin',-86.36,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000032,'EPSG','9201','EPSG','8806','False easting',787400.0,'EPSG','9003','EPSG','8807','False northing',118110.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9974','conversion','EPSG','7166','EPSG','4260','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7167','InGCS Crawford-Lawrence-Orange (m)','For equivalent non-metric definition see proj code 7168. Defined by Indiana DOT as three separate zones (one for each county) each having identical projection parameter values: see info source.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',38.06,'EPSG','9110','EPSG','8802','Longitude of natural origin',-86.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000025,'EPSG','9201','EPSG','8806','False easting',240000.0,'EPSG','9001','EPSG','8807','False northing',36000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9975','conversion','EPSG','7167','EPSG','4272','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7168','InGCS Crawford-Lawrence-Orange (ftUS)','See proj code 7167 for authoritative metric definition. Defined by Indiana DOT as three separate zones (one for each county) each having identical defining parameter values: see info source.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',38.06,'EPSG','9110','EPSG','8802','Longitude of natural origin',-86.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000025,'EPSG','9201','EPSG','8806','False easting',787400.0,'EPSG','9003','EPSG','8807','False northing',118110.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9976','conversion','EPSG','7168','EPSG','4272','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7169','InGCS Daviess-Greene (m)','For equivalent non-metric definition see proj code 7170. Defined by Indiana DOT as two separate zones (one for each county) each having identical projection parameter values: see info source.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',38.27,'EPSG','9110','EPSG','8802','Longitude of natural origin',-87.06,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000018,'EPSG','9201','EPSG','8806','False easting',240000.0,'EPSG','9001','EPSG','8807','False northing',36000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9977','conversion','EPSG','7169','EPSG','4269','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7170','InGCS Daviess-Greene (ftUS)','See proj code 7169 for authoritative metric definition. Defined by Indiana DOT as two separate zones (one for each county) each having identical defining parameter values: see info source.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',38.27,'EPSG','9110','EPSG','8802','Longitude of natural origin',-87.06,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000018,'EPSG','9201','EPSG','8806','False easting',787400.0,'EPSG','9003','EPSG','8807','False northing',118110.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9978','conversion','EPSG','7170','EPSG','4269','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7171','InGCS Dearborn-Ohio-Switzerland (m)','For equivalent non-metric definition see proj code 7172. Defined by Indiana DOT as three separate zones (one for each county) each having identical projection parameter values: see info source.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',38.39,'EPSG','9110','EPSG','8802','Longitude of natural origin',-84.54,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000029,'EPSG','9201','EPSG','8806','False easting',240000.0,'EPSG','9001','EPSG','8807','False northing',36000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9979','conversion','EPSG','7171','EPSG','4306','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7172','InGCS Dearborn-Ohio-Switzerland (ftUS)','See proj code 7171 for authoritative metric definition. Defined by Indiana DOT as three separate zones (one for each county) each having identical defining parameter values: see info source.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',38.39,'EPSG','9110','EPSG','8802','Longitude of natural origin',-84.54,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000029,'EPSG','9201','EPSG','8806','False easting',787400.0,'EPSG','9003','EPSG','8807','False northing',118110.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9980','conversion','EPSG','7172','EPSG','4306','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7173','InGCS Decatur-Rush (m)','For equivalent non-metric definition see proj code 7174. Defined by Indiana DOT as two separate zones (one for each county) each having identical projection parameter values: see info source.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',39.06,'EPSG','9110','EPSG','8802','Longitude of natural origin',-85.39,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000036,'EPSG','9201','EPSG','8806','False easting',240000.0,'EPSG','9001','EPSG','8807','False northing',36000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9981','conversion','EPSG','7173','EPSG','4299','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7174','InGCS Decatur-Rush (ftUS)','See proj code 7173 for authoritative metric definition. Defined by Indiana DOT as two separate zones (one for each county) each having identical defining parameter values: see info source.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',39.06,'EPSG','9110','EPSG','8802','Longitude of natural origin',-85.39,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000036,'EPSG','9201','EPSG','8806','False easting',787400.0,'EPSG','9003','EPSG','8807','False northing',118110.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9982','conversion','EPSG','7174','EPSG','4299','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7175','InGCS DeKalb (m)','For equivalent non-metric definition see proj code 7176.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',41.15,'EPSG','9110','EPSG','8802','Longitude of natural origin',-84.57,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000036,'EPSG','9201','EPSG','8806','False easting',240000.0,'EPSG','9001','EPSG','8807','False northing',36000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9983','conversion','EPSG','7175','EPSG','4283','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7176','InGCS DeKalb (ftUS)','See proj code 7175 for authoritative metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',41.15,'EPSG','9110','EPSG','8802','Longitude of natural origin',-84.57,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000036,'EPSG','9201','EPSG','8806','False easting',787400.0,'EPSG','9003','EPSG','8807','False northing',118110.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9984','conversion','EPSG','7176','EPSG','4283','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7177','InGCS Dubois-Martin (m)','For equivalent non-metric definition see proj code 7178. Defined by Indiana DOT as two separate zones (one for each county) each having identical projection parameter values: see info source.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',38.12,'EPSG','9110','EPSG','8802','Longitude of natural origin',-86.57,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00002,'EPSG','9201','EPSG','8806','False easting',240000.0,'EPSG','9001','EPSG','8807','False northing',36000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9985','conversion','EPSG','7177','EPSG','4271','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7178','InGCS Dubois-Martin (ftUS)','See proj code 7177 for authoritative metric definition. Defined by Indiana DOT as two separate zones (one for each county) each having identical defining parameter values: see info source.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',38.12,'EPSG','9110','EPSG','8802','Longitude of natural origin',-86.57,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00002,'EPSG','9201','EPSG','8806','False easting',787400.0,'EPSG','9003','EPSG','8807','False northing',118110.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9986','conversion','EPSG','7178','EPSG','4271','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7179','InGCS Elkhart-Kosciusko-Wabash (m)','For equivalent non-metric definition see proj code 7180. Defined by Indiana DOT as three separate zones (one for each county) each having identical projection parameter values: see info source.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',40.39,'EPSG','9110','EPSG','8802','Longitude of natural origin',-85.51,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000033,'EPSG','9201','EPSG','8806','False easting',240000.0,'EPSG','9001','EPSG','8807','False northing',36000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9987','conversion','EPSG','7179','EPSG','4280','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7180','InGCS Elkhart-Kosciusko-Wabash (ftUS)','See proj code 7179 for authoritative metric definition. Defined by Indiana DOT as three separate zones (one for each county) each having identical defining parameter values: see info source.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',40.39,'EPSG','9110','EPSG','8802','Longitude of natural origin',-85.51,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000033,'EPSG','9201','EPSG','8806','False easting',787400.0,'EPSG','9003','EPSG','8807','False northing',118110.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9988','conversion','EPSG','7180','EPSG','4280','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7181','InGCS Fayette-Franklin-Union (m)','For equivalent non-metric definition see proj code 7182. Defined by Indiana DOT as three separate zones (one for each county) each having identical projection parameter values: see info source.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',39.15,'EPSG','9110','EPSG','8802','Longitude of natural origin',-85.03,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000038,'EPSG','9201','EPSG','8806','False easting',240000.0,'EPSG','9001','EPSG','8807','False northing',36000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9989','conversion','EPSG','7181','EPSG','4300','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7182','InGCS Fayette-Franklin-Union (ftUS)','See proj code 7181 for authoritative metric definition. Defined by Indiana DOT as three separate zones (one for each county) each having identical defining parameter values: see info source.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',39.15,'EPSG','9110','EPSG','8802','Longitude of natural origin',-85.03,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000038,'EPSG','9201','EPSG','8806','False easting',787400.0,'EPSG','9003','EPSG','8807','False northing',118110.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9990','conversion','EPSG','7182','EPSG','4300','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7183','InGCS Fountain-Warren (m)','For equivalent non-metric definition see proj code 7184. Defined by Indiana DOT as two separate zones (one for each county) each having identical projection parameter values: see info source.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',39.57,'EPSG','9110','EPSG','8802','Longitude of natural origin',-87.18,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000025,'EPSG','9201','EPSG','8806','False easting',240000.0,'EPSG','9001','EPSG','8807','False northing',36000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9991','conversion','EPSG','7183','EPSG','4259','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7184','InGCS Fountain-Warren (ftUS)','See proj code 7183 for authoritative metric definition. Defined by Indiana DOT as two separate zones (one for each county) each having identical defining parameter values: see info source.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',39.57,'EPSG','9110','EPSG','8802','Longitude of natural origin',-87.18,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000025,'EPSG','9201','EPSG','8806','False easting',787400.0,'EPSG','9003','EPSG','8807','False northing',118110.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9992','conversion','EPSG','7184','EPSG','4259','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7185','InGCS Fulton-Marshall-St. Joseph (m)','For equivalent non-metric definition see proj code 7186. Defined by Indiana DOT as three separate zones (one for each county) each having identical projection parameter values: see info source.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',40.54,'EPSG','9110','EPSG','8802','Longitude of natural origin',-86.18,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000031,'EPSG','9201','EPSG','8806','False easting',240000.0,'EPSG','9001','EPSG','8807','False northing',36000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9993','conversion','EPSG','7185','EPSG','4279','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7186','InGCS Fulton-Marshall-St. Joseph (ftUS)','See proj code 7185 for authoritative metric definition. Defined by Indiana DOT as three separate zones (one for each county) each having identical defining parameter values: see info source.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',40.54,'EPSG','9110','EPSG','8802','Longitude of natural origin',-86.18,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000031,'EPSG','9201','EPSG','8806','False easting',787400.0,'EPSG','9003','EPSG','8807','False northing',118110.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9994','conversion','EPSG','7186','EPSG','4279','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7187','InGCS Gibson (m)','For equivalent non-metric definition see proj code 7188.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',38.09,'EPSG','9110','EPSG','8802','Longitude of natural origin',-87.39,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000013,'EPSG','9201','EPSG','8806','False easting',240000.0,'EPSG','9001','EPSG','8807','False northing',36000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9995','conversion','EPSG','7187','EPSG','4273','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7188','InGCS Gibson (ftUS)','See proj code 7187 for authoritative metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',38.09,'EPSG','9110','EPSG','8802','Longitude of natural origin',-87.39,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000013,'EPSG','9201','EPSG','8806','False easting',787400.0,'EPSG','9003','EPSG','8807','False northing',118110.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9996','conversion','EPSG','7188','EPSG','4273','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7189','InGCS Grant (m)','For equivalent non-metric definition see proj code 7190.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',40.21,'EPSG','9110','EPSG','8802','Longitude of natural origin',-85.42,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000034,'EPSG','9201','EPSG','8806','False easting',240000.0,'EPSG','9001','EPSG','8807','False northing',36000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9997','conversion','EPSG','7189','EPSG','4290','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7190','InGCS Grant (ftUS)','See proj code 7189 for authoritative metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',40.21,'EPSG','9110','EPSG','8802','Longitude of natural origin',-85.42,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000034,'EPSG','9201','EPSG','8806','False easting',787400.0,'EPSG','9003','EPSG','8807','False northing',118110.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9998','conversion','EPSG','7190','EPSG','4290','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7191','InGCS Hamilton-Tipton (m)','For equivalent non-metric definition see proj code 7192. Defined by Indiana DOT as two separate zones (one for each county) each having identical projection parameter values: see info source.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',39.54,'EPSG','9110','EPSG','8802','Longitude of natural origin',-86.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000034,'EPSG','9201','EPSG','8806','False easting',240000.0,'EPSG','9001','EPSG','8807','False northing',36000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','9999','conversion','EPSG','7191','EPSG','4293','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7192','InGCS Hamilton-Tipton (ftUS)','See proj code 7191 for authoritative metric definition. Defined by Indiana DOT as two separate zones (one for each county) each having identical defining parameter values: see info source.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',39.54,'EPSG','9110','EPSG','8802','Longitude of natural origin',-86.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000034,'EPSG','9201','EPSG','8806','False easting',787400.0,'EPSG','9003','EPSG','8807','False northing',118110.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10000','conversion','EPSG','7192','EPSG','4293','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7193','InGCS Hancock-Madison (m)','For equivalent non-metric definition see proj code 7194. Defined by Indiana DOT as two separate zones (one for each county) each having identical projection parameter values: see info source.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',39.39,'EPSG','9110','EPSG','8802','Longitude of natural origin',-85.48,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000036,'EPSG','9201','EPSG','8806','False easting',240000.0,'EPSG','9001','EPSG','8807','False northing',36000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10001','conversion','EPSG','7193','EPSG','4294','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7194','InGCS Hancock-Madison (ftUS)','See proj code 7193 for authoritative metric definition. Defined by Indiana DOT as two separate zones (one for each county) each having identical defining parameter values: see info source.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',39.39,'EPSG','9110','EPSG','8802','Longitude of natural origin',-85.48,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000036,'EPSG','9201','EPSG','8806','False easting',787400.0,'EPSG','9003','EPSG','8807','False northing',118110.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10002','conversion','EPSG','7194','EPSG','4294','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7195','InGCS Harrison-Washington (m)','For equivalent non-metric definition see proj code 7196. Defined by Indiana DOT as two separate zones (one for each county) each having identical projection parameter values: see info source.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',37.57,'EPSG','9110','EPSG','8802','Longitude of natural origin',-86.09,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000027,'EPSG','9201','EPSG','8806','False easting',240000.0,'EPSG','9001','EPSG','8807','False northing',36000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10003','conversion','EPSG','7195','EPSG','4307','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7196','InGCS Harrison-Washington (ftUS)','See proj code 7195 for authoritative metric definition. Defined by Indiana DOT as two separate zones (one for each county) each having identical defining parameter values: see info source.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',37.57,'EPSG','9110','EPSG','8802','Longitude of natural origin',-86.09,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000027,'EPSG','9201','EPSG','8806','False easting',787400.0,'EPSG','9003','EPSG','8807','False northing',118110.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10004','conversion','EPSG','7196','EPSG','4307','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7197','InGCS Henry (m)','For equivalent non-metric definition see proj code 7198.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',39.45,'EPSG','9110','EPSG','8802','Longitude of natural origin',-85.27,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000043,'EPSG','9201','EPSG','8806','False easting',240000.0,'EPSG','9001','EPSG','8807','False northing',36000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10005','conversion','EPSG','7197','EPSG','4296','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7198','InGCS Henry (ftUS)','See proj code 7197 for authoritative metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',39.45,'EPSG','9110','EPSG','8802','Longitude of natural origin',-85.27,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000043,'EPSG','9201','EPSG','8806','False easting',787400.0,'EPSG','9003','EPSG','8807','False northing',118110.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10006','conversion','EPSG','7198','EPSG','4296','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7199','InGCS Howard-Miami (m)','For equivalent non-metric definition see proj code 7200. Defined by Indiana DOT as two separate zones (one for each county) each having identical projection parameter values: see info source.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',40.21,'EPSG','9110','EPSG','8802','Longitude of natural origin',-86.09,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000031,'EPSG','9201','EPSG','8806','False easting',240000.0,'EPSG','9001','EPSG','8807','False northing',36000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10007','conversion','EPSG','7199','EPSG','4287','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7200','InGCS Howard-Miami (ftUS)','See proj code 7199 for authoritative metric definition. Defined by Indiana DOT as two separate zones (one for each county) each having identical defining parameter values: see info source.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',40.21,'EPSG','9110','EPSG','8802','Longitude of natural origin',-86.09,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000031,'EPSG','9201','EPSG','8806','False easting',787400.0,'EPSG','9003','EPSG','8807','False northing',118110.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10008','conversion','EPSG','7200','EPSG','4287','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7201','InGCS Huntington-Whitley (m)','For equivalent non-metric definition see proj code 7202. Defined by Indiana DOT as two separate zones (one for each county) each having identical projection parameter values: see info source.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',40.39,'EPSG','9110','EPSG','8802','Longitude of natural origin',-85.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000034,'EPSG','9201','EPSG','8806','False easting',240000.0,'EPSG','9001','EPSG','8807','False northing',36000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10009','conversion','EPSG','7201','EPSG','4284','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7202','InGCS Huntington-Whitley (ftUS)','See proj code 7201 for authoritative metric definition. Defined by Indiana DOT as two separate zones (one for each county) each having identical defining parameter values: see info source.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',40.39,'EPSG','9110','EPSG','8802','Longitude of natural origin',-85.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000034,'EPSG','9201','EPSG','8806','False easting',787400.0,'EPSG','9003','EPSG','8807','False northing',118110.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10010','conversion','EPSG','7202','EPSG','4284','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7203','InGCS Jackson (m)','For equivalent non-metric definition see proj code 7204.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',38.42,'EPSG','9110','EPSG','8802','Longitude of natural origin',-85.57,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000022,'EPSG','9201','EPSG','8806','False easting',240000.0,'EPSG','9001','EPSG','8807','False northing',36000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10011','conversion','EPSG','7203','EPSG','4303','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7204','InGCS Jackson (ftUS)','See proj code 7203 for authoritative metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',38.42,'EPSG','9110','EPSG','8802','Longitude of natural origin',-85.57,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000022,'EPSG','9201','EPSG','8806','False easting',787400.0,'EPSG','9003','EPSG','8807','False northing',118110.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10012','conversion','EPSG','7204','EPSG','4303','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7205','InGCS Jasper-Porter (m)','For equivalent non-metric definition see proj code 7206. Defined by Indiana DOT as two separate zones (one for each county) each having identical projection parameter values: see info source.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',40.42,'EPSG','9110','EPSG','8802','Longitude of natural origin',-87.06,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000027,'EPSG','9201','EPSG','8806','False easting',240000.0,'EPSG','9001','EPSG','8807','False northing',36000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10013','conversion','EPSG','7205','EPSG','4254','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7206','InGCS Jasper-Porter (ftUS)','See proj code 7205 for authoritative metric definition. Defined by Indiana DOT as two separate zones (one for each county) each having identical defining parameter values: see info source.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',40.42,'EPSG','9110','EPSG','8802','Longitude of natural origin',-87.06,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000027,'EPSG','9201','EPSG','8806','False easting',787400.0,'EPSG','9003','EPSG','8807','False northing',118110.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10014','conversion','EPSG','7206','EPSG','4254','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7207','InGCS Jay (m)','For equivalent non-metric definition see proj code 7208.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',40.18,'EPSG','9110','EPSG','8802','Longitude of natural origin',-85.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000038,'EPSG','9201','EPSG','8806','False easting',240000.0,'EPSG','9001','EPSG','8807','False northing',36000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10015','conversion','EPSG','7207','EPSG','4292','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7208','InGCS Jay (ftUS)','See proj code 7207 for authoritative metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',40.18,'EPSG','9110','EPSG','8802','Longitude of natural origin',-85.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000038,'EPSG','9201','EPSG','8806','False easting',787400.0,'EPSG','9003','EPSG','8807','False northing',118110.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10016','conversion','EPSG','7208','EPSG','4292','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7209','InGCS Jefferson (m)','For equivalent non-metric definition see proj code 7210.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',38.33,'EPSG','9110','EPSG','8802','Longitude of natural origin',-85.21,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000028,'EPSG','9201','EPSG','8806','False easting',240000.0,'EPSG','9001','EPSG','8807','False northing',36000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10017','conversion','EPSG','7209','EPSG','4309','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7210','InGCS Jefferson (ftUS)','See proj code 7209 for authoritative metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',38.33,'EPSG','9110','EPSG','8802','Longitude of natural origin',-85.21,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000028,'EPSG','9201','EPSG','8806','False easting',787400.0,'EPSG','9003','EPSG','8807','False northing',118110.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10018','conversion','EPSG','7210','EPSG','4309','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7211','InGCS Jennings (m)','For equivalent non-metric definition see proj code 7212.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',38.48,'EPSG','9110','EPSG','8802','Longitude of natural origin',-85.48,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000025,'EPSG','9201','EPSG','8806','False easting',240000.0,'EPSG','9001','EPSG','8807','False northing',36000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10019','conversion','EPSG','7211','EPSG','4304','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7212','InGCS Jennings (ftUS)','See proj code 7211 for authoritative metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',38.48,'EPSG','9110','EPSG','8802','Longitude of natural origin',-85.48,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000025,'EPSG','9201','EPSG','8806','False easting',787400.0,'EPSG','9003','EPSG','8807','False northing',118110.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10020','conversion','EPSG','7212','EPSG','4304','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7213','InGCS Johnson-Marion (m)','For equivalent non-metric definition see proj code 7214. Defined by Indiana DOT as two separate zones (one for each county) each having identical projection parameter values: see info source.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',39.18,'EPSG','9110','EPSG','8802','Longitude of natural origin',-86.09,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000031,'EPSG','9201','EPSG','8806','False easting',240000.0,'EPSG','9001','EPSG','8807','False northing',36000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10021','conversion','EPSG','7213','EPSG','4297','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7214','InGCS Johnson-Marion (ftUS)','See proj code 7213 for authoritative metric definition. Defined by Indiana DOT as two separate zones (one for each county) each having identical defining parameter values: see info source.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',39.18,'EPSG','9110','EPSG','8802','Longitude of natural origin',-86.09,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000031,'EPSG','9201','EPSG','8806','False easting',787400.0,'EPSG','9003','EPSG','8807','False northing',118110.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10022','conversion','EPSG','7214','EPSG','4297','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7215','InGCS Knox (m)','For equivalent non-metric definition see proj code 7216.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',38.24,'EPSG','9110','EPSG','8802','Longitude of natural origin',-87.27,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000015,'EPSG','9201','EPSG','8806','False easting',240000.0,'EPSG','9001','EPSG','8807','False northing',36000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10023','conversion','EPSG','7215','EPSG','4270','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7216','InGCS Knox (ftUS)','See proj code 7215 for authoritative metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',38.24,'EPSG','9110','EPSG','8802','Longitude of natural origin',-87.27,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000015,'EPSG','9201','EPSG','8806','False easting',787400.0,'EPSG','9003','EPSG','8807','False northing',118110.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10024','conversion','EPSG','7216','EPSG','4270','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7217','InGCS LaGrange-Noble (m)','For equivalent non-metric definition see proj code 7218. Defined by Indiana DOT as two separate zones (one for each county) each having identical projection parameter values: see info source.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',41.15,'EPSG','9110','EPSG','8802','Longitude of natural origin',-85.27,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000037,'EPSG','9201','EPSG','8806','False easting',240000.0,'EPSG','9001','EPSG','8807','False northing',36000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10025','conversion','EPSG','7217','EPSG','4281','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7218','InGCS LaGrange-Noble (ftUS)','See proj code 7217 for authoritative metric definition. Defined by Indiana DOT as two separate zones (one for each county) each having identical defining parameter values: see info source.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',41.15,'EPSG','9110','EPSG','8802','Longitude of natural origin',-85.27,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000037,'EPSG','9201','EPSG','8806','False easting',787400.0,'EPSG','9003','EPSG','8807','False northing',118110.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10026','conversion','EPSG','7218','EPSG','4281','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7219','InGCS Lake-Newton (m)','For equivalent non-metric definition see proj code 7220. Defined by Indiana DOT as two separate zones (one for each county) each having identical projection parameter values: see info source.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',40.42,'EPSG','9110','EPSG','8802','Longitude of natural origin',-87.24,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000026,'EPSG','9201','EPSG','8806','False easting',240000.0,'EPSG','9001','EPSG','8807','False northing',36000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10027','conversion','EPSG','7219','EPSG','4253','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7220','InGCS Lake-Newton (ftUS)','See proj code 7219 for authoritative metric definition. Defined by Indiana DOT as two separate zones (one for each county) each having identical defining parameter values: see info source.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',40.42,'EPSG','9110','EPSG','8802','Longitude of natural origin',-87.24,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000026,'EPSG','9201','EPSG','8806','False easting',787400.0,'EPSG','9003','EPSG','8807','False northing',118110.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10028','conversion','EPSG','7220','EPSG','4253','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7221','InGCS LaPorte-Pulaski-Starke (m)','For equivalent non-metric definition see proj code 7222. Defined by Indiana DOT as three separate zones (one for each county) each having identical projection parameter values: see info source.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',40.54,'EPSG','9110','EPSG','8802','Longitude of natural origin',-86.45,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000027,'EPSG','9201','EPSG','8806','False easting',240000.0,'EPSG','9001','EPSG','8807','False northing',36000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10029','conversion','EPSG','7221','EPSG','4255','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7222','InGCS LaPorte-Pulaski-Starke (ftUS)','See proj code 7221 for authoritative metric definition. Defined by Indiana DOT as three separate zones (one for each county) each having identical defining parameter values: see info source.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',40.54,'EPSG','9110','EPSG','8802','Longitude of natural origin',-86.45,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000027,'EPSG','9201','EPSG','8806','False easting',787400.0,'EPSG','9003','EPSG','8807','False northing',118110.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10030','conversion','EPSG','7222','EPSG','4255','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7223','InGCS Monroe-Morgan (m)','For equivalent non-metric definition see proj code 7224. Defined by Indiana DOT as two separate zones (one for each county) each having identical projection parameter values: see info source.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',38.57,'EPSG','9110','EPSG','8802','Longitude of natural origin',-86.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000028,'EPSG','9201','EPSG','8806','False easting',240000.0,'EPSG','9001','EPSG','8807','False northing',36000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10031','conversion','EPSG','7223','EPSG','4267','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7224','InGCS Monroe-Morgan (ftUS)','See proj code 7223 for authoritative metric definition. Defined by Indiana DOT as two separate zones (one for each county) each having identical defining parameter values: see info source.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',38.57,'EPSG','9110','EPSG','8802','Longitude of natural origin',-86.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000028,'EPSG','9201','EPSG','8806','False easting',787400.0,'EPSG','9003','EPSG','8807','False northing',118110.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10032','conversion','EPSG','7224','EPSG','4267','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7225','InGCS Montgomery-Putnam (m)','For equivalent non-metric definition see proj code 7226. Defined by Indiana DOT as two separate zones (one for each county) each having identical projection parameter values: see info source.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',39.27,'EPSG','9110','EPSG','8802','Longitude of natural origin',-86.57,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000031,'EPSG','9201','EPSG','8806','False easting',240000.0,'EPSG','9001','EPSG','8807','False northing',36000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10033','conversion','EPSG','7225','EPSG','4262','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7226','InGCS Montgomery-Putnam (ftUS)','See proj code 7225 for authoritative metric definition. Defined by Indiana DOT as two separate zones (one for each county) each having identical defining parameter values: see info source.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',39.27,'EPSG','9110','EPSG','8802','Longitude of natural origin',-86.57,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000031,'EPSG','9201','EPSG','8806','False easting',787400.0,'EPSG','9003','EPSG','8807','False northing',118110.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10034','conversion','EPSG','7226','EPSG','4262','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7227','InGCS Owen (m)','For equivalent non-metric definition see proj code 7228.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',39.09,'EPSG','9110','EPSG','8802','Longitude of natural origin',-86.54,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000026,'EPSG','9201','EPSG','8806','False easting',240000.0,'EPSG','9001','EPSG','8807','False northing',36000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10035','conversion','EPSG','7227','EPSG','4266','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7228','InGCS Owen (ftUS)','See proj code 7227 for authoritative metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',39.09,'EPSG','9110','EPSG','8802','Longitude of natural origin',-86.54,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000026,'EPSG','9201','EPSG','8806','False easting',787400.0,'EPSG','9003','EPSG','8807','False northing',118110.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10036','conversion','EPSG','7228','EPSG','4266','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7229','InGCS Parke-Vermillion (m)','For equivalent non-metric definition see proj code 7230. Defined by Indiana DOT as two separate zones (one for each county) each having identical projection parameter values: see info source.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',39.36,'EPSG','9110','EPSG','8802','Longitude of natural origin',-87.21,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000022,'EPSG','9201','EPSG','8806','False easting',240000.0,'EPSG','9001','EPSG','8807','False northing',36000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10037','conversion','EPSG','7229','EPSG','4261','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7230','InGCS Parke-Vermillion (ftUS)','See proj code 7229 for authoritative metric definition. Defined by Indiana DOT as two separate zones (one for each county) each having identical defining parameter values: see info source.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',39.36,'EPSG','9110','EPSG','8802','Longitude of natural origin',-87.21,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000022,'EPSG','9201','EPSG','8806','False easting',787400.0,'EPSG','9003','EPSG','8807','False northing',118110.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10038','conversion','EPSG','7230','EPSG','4261','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7231','InGCS Perry (m)','For equivalent non-metric definition see proj code 7232.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',37.48,'EPSG','9110','EPSG','8802','Longitude of natural origin',-86.42,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00002,'EPSG','9201','EPSG','8806','False easting',240000.0,'EPSG','9001','EPSG','8807','False northing',36000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10039','conversion','EPSG','7231','EPSG','4278','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7232','InGCS Perry (ftUS)','See proj code 7231 for authoritative metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',37.48,'EPSG','9110','EPSG','8802','Longitude of natural origin',-86.42,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00002,'EPSG','9201','EPSG','8806','False easting',787400.0,'EPSG','9003','EPSG','8807','False northing',118110.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10040','conversion','EPSG','7232','EPSG','4278','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7233','InGCS Pike-Warrick (m)','For equivalent non-metric definition see proj code 7234. Defined by Indiana DOT as two separate zones (one for each county) each having identical projection parameter values: see info source.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',37.51,'EPSG','9110','EPSG','8802','Longitude of natural origin',-87.18,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000015,'EPSG','9201','EPSG','8806','False easting',240000.0,'EPSG','9001','EPSG','8807','False northing',36000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10041','conversion','EPSG','7233','EPSG','4274','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7234','InGCS Pike-Warrick (ftUS)','See proj code 7233 for authoritative metric definition. Defined by Indiana DOT as two separate zones (one for each county) each having identical defining parameter values: see info source.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',37.51,'EPSG','9110','EPSG','8802','Longitude of natural origin',-87.18,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000015,'EPSG','9201','EPSG','8806','False easting',787400.0,'EPSG','9003','EPSG','8807','False northing',118110.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10042','conversion','EPSG','7234','EPSG','4274','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7235','InGCS Posey (m)','For equivalent non-metric definition see proj code 7236.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',37.45,'EPSG','9110','EPSG','8802','Longitude of natural origin',-87.57,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000013,'EPSG','9201','EPSG','8806','False easting',240000.0,'EPSG','9001','EPSG','8807','False northing',36000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10043','conversion','EPSG','7235','EPSG','4275','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7236','InGCS Posey (ftUS)','See proj code 7235 for authoritative metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',37.45,'EPSG','9110','EPSG','8802','Longitude of natural origin',-87.57,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000013,'EPSG','9201','EPSG','8806','False easting',787400.0,'EPSG','9003','EPSG','8807','False northing',118110.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10044','conversion','EPSG','7236','EPSG','4275','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7237','InGCS Randolph-Wayne (m)','For equivalent non-metric definition see proj code 7238. Defined by Indiana DOT as two separate zones (one for each county) each having identical projection parameter values: see info source.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',39.42,'EPSG','9110','EPSG','8802','Longitude of natural origin',-85.03,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000044,'EPSG','9201','EPSG','8806','False easting',240000.0,'EPSG','9001','EPSG','8807','False northing',36000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10045','conversion','EPSG','7237','EPSG','4295','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7238','InGCS Randolph-Wayne (ftUS)','See proj code 7237 for authoritative metric definition. Defined by Indiana DOT as two separate zones (one for each county) each having identical defining parameter values: see info source.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',39.42,'EPSG','9110','EPSG','8802','Longitude of natural origin',-85.03,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000044,'EPSG','9201','EPSG','8806','False easting',787400.0,'EPSG','9003','EPSG','8807','False northing',118110.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10046','conversion','EPSG','7238','EPSG','4295','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7239','InGCS Ripley (m)','For equivalent non-metric definition see proj code 7240.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',38.54,'EPSG','9110','EPSG','8802','Longitude of natural origin',-85.18,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000038,'EPSG','9201','EPSG','8806','False easting',240000.0,'EPSG','9001','EPSG','8807','False northing',36000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10047','conversion','EPSG','7239','EPSG','4305','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7240','InGCS Ripley (ftUS)','See proj code 7239 for authoritative metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',38.54,'EPSG','9110','EPSG','8802','Longitude of natural origin',-85.18,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000038,'EPSG','9201','EPSG','8806','False easting',787400.0,'EPSG','9003','EPSG','8807','False northing',118110.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10048','conversion','EPSG','7240','EPSG','4305','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7241','InGCS Shelby (m)','For equivalent non-metric definition see proj code 7242.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',39.18,'EPSG','9110','EPSG','8802','Longitude of natural origin',-85.54,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00003,'EPSG','9201','EPSG','8806','False easting',240000.0,'EPSG','9001','EPSG','8807','False northing',36000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10049','conversion','EPSG','7241','EPSG','4298','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7242','InGCS Shelby (ftUS)','See proj code 7241 for authoritative metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',39.18,'EPSG','9110','EPSG','8802','Longitude of natural origin',-85.54,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00003,'EPSG','9201','EPSG','8806','False easting',787400.0,'EPSG','9003','EPSG','8807','False northing',118110.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10050','conversion','EPSG','7242','EPSG','4298','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7243','InGCS Spencer (m)','For equivalent non-metric definition see proj code 7244.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',37.45,'EPSG','9110','EPSG','8802','Longitude of natural origin',-87.03,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000014,'EPSG','9201','EPSG','8806','False easting',240000.0,'EPSG','9001','EPSG','8807','False northing',36000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10051','conversion','EPSG','7243','EPSG','4277','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7244','InGCS Spencer (ftUS)','See proj code 7243 for authoritative metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',37.45,'EPSG','9110','EPSG','8802','Longitude of natural origin',-87.03,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000014,'EPSG','9201','EPSG','8806','False easting',787400.0,'EPSG','9003','EPSG','8807','False northing',118110.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10052','conversion','EPSG','7244','EPSG','4277','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7245','InGCS Steuben (m)','For equivalent non-metric definition see proj code 7246.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',41.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-85.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000041,'EPSG','9201','EPSG','8806','False easting',240000.0,'EPSG','9001','EPSG','8807','False northing',36000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10053','conversion','EPSG','7245','EPSG','4282','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7246','InGCS Steuben (ftUS)','See proj code 7245 for authoritative metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',41.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-85.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000041,'EPSG','9201','EPSG','8806','False easting',787400.0,'EPSG','9003','EPSG','8807','False northing',118110.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10054','conversion','EPSG','7246','EPSG','4282','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7247','InGCS Sullivan (m)','For equivalent non-metric definition see proj code 7248.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',38.54,'EPSG','9110','EPSG','8802','Longitude of natural origin',-87.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000017,'EPSG','9201','EPSG','8806','False easting',240000.0,'EPSG','9001','EPSG','8807','False northing',36000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10055','conversion','EPSG','7247','EPSG','4268','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7248','InGCS Sullivan (ftUS)','See proj code 7247 for authoritative metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',38.54,'EPSG','9110','EPSG','8802','Longitude of natural origin',-87.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000017,'EPSG','9201','EPSG','8806','False easting',787400.0,'EPSG','9003','EPSG','8807','False northing',118110.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10056','conversion','EPSG','7248','EPSG','4268','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7249','InGCS Tippecanoe-White (m)','For equivalent non-metric definition see proj code 7250. Defined by Indiana DOT as two separate zones (one for each county) each having identical projection parameter values: see info source.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',40.12,'EPSG','9110','EPSG','8802','Longitude of natural origin',-86.54,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000026,'EPSG','9201','EPSG','8806','False easting',240000.0,'EPSG','9001','EPSG','8807','False northing',36000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10057','conversion','EPSG','7249','EPSG','4257','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7250','InGCS Tippecanoe-White (ftUS)','See proj code 7249 for authoritative metric definition. Defined by Indiana DOT as two separate zones (one for each county) each having identical defining parameter values: see info source.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',40.12,'EPSG','9110','EPSG','8802','Longitude of natural origin',-86.54,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000026,'EPSG','9201','EPSG','8806','False easting',787400.0,'EPSG','9003','EPSG','8807','False northing',118110.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10058','conversion','EPSG','7250','EPSG','4257','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7251','InGCS Vanderburgh (m)','For equivalent non-metric definition see proj code 7252.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',37.48,'EPSG','9110','EPSG','8802','Longitude of natural origin',-87.33,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000015,'EPSG','9201','EPSG','8806','False easting',240000.0,'EPSG','9001','EPSG','8807','False northing',36000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10059','conversion','EPSG','7251','EPSG','4276','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7252','InGCS Vanderburgh (ftUS)','See proj code 7251 for authoritative metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',37.48,'EPSG','9110','EPSG','8802','Longitude of natural origin',-87.33,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000015,'EPSG','9201','EPSG','8806','False easting',787400.0,'EPSG','9003','EPSG','8807','False northing',118110.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10060','conversion','EPSG','7252','EPSG','4276','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7253','InGCS Vigo (m)','For equivalent non-metric definition see proj code 7254.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',39.15,'EPSG','9110','EPSG','8802','Longitude of natural origin',-87.27,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00002,'EPSG','9201','EPSG','8806','False easting',240000.0,'EPSG','9001','EPSG','8807','False northing',36000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10061','conversion','EPSG','7253','EPSG','4264','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7254','InGCS Vigo (ftUS)','See proj code 7253 for authoritative metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',39.15,'EPSG','9110','EPSG','8802','Longitude of natural origin',-87.27,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00002,'EPSG','9201','EPSG','8806','False easting',787400.0,'EPSG','9003','EPSG','8807','False northing',118110.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10062','conversion','EPSG','7254','EPSG','4264','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7255','InGCS Wells (m)','For equivalent non-metric definition see proj code 7256.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',40.33,'EPSG','9110','EPSG','8802','Longitude of natural origin',-85.15,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000034,'EPSG','9201','EPSG','8806','False easting',240000.0,'EPSG','9001','EPSG','8807','False northing',36000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10063','conversion','EPSG','7255','EPSG','4288','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7256','InGCS Wells (ftUS)','See proj code 7255 for authoritative metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',40.33,'EPSG','9110','EPSG','8802','Longitude of natural origin',-85.15,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000034,'EPSG','9201','EPSG','8806','False easting',787400.0,'EPSG','9003','EPSG','8807','False northing',118110.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10064','conversion','EPSG','7256','EPSG','4288','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7378','WISCRS Ashland County (m)','Replaces and emulates WCCS zone. For equivalent non-metric definition see proj code 7379.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',45.4222,'EPSG','9110','EPSG','8802','Longitude of natural origin',-90.372,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000495683,'EPSG','9201','EPSG','8806','False easting',172821.9461,'EPSG','9001','EPSG','8807','False northing',0.0017,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10066','conversion','EPSG','7378','EPSG','4320','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7379','WISCRS Ashland County (ftUS)','See proj code 7378 for authoritative metric definition. Replaces and emulates WCCS zone.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',45.4222,'EPSG','9110','EPSG','8802','Longitude of natural origin',-90.372,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000495683,'EPSG','9201','EPSG','8806','False easting',567000.001,'EPSG','9003','EPSG','8807','False northing',0.006,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10067','conversion','EPSG','7379','EPSG','4320','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7380','WISCRS Bayfield County (m)','Replaces and emulates WCCS zone. For equivalent non-metric definition see proj code 7381.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',46.4010734158,'EPSG','9110','EPSG','8802','Longitude of natural origin',-91.091,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000331195,'EPSG','9201','EPSG','8806','False easting',228600.4575,'EPSG','9001','EPSG','8807','False northing',148551.4837,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10068','conversion','EPSG','7380','EPSG','4321','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7381','WISCRS Bayfield County (ftUS)','See proj code 7380 for authoritative metric definition. Replaces and emulates WCCS zone.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',46.4010734158,'EPSG','9110','EPSG','8802','Longitude of natural origin',-91.091,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000331195,'EPSG','9201','EPSG','8806','False easting',750000.001,'EPSG','9003','EPSG','8807','False northing',487372.659,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10069','conversion','EPSG','7381','EPSG','4321','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7382','WISCRS Burnett County (m)','Replaces and emulates WCCS zone. For equivalent non-metric definition see proj code 7383.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',45.5355373517,'EPSG','9110','EPSG','8802','Longitude of natural origin',-92.2728,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000383841,'EPSG','9201','EPSG','8806','False easting',64008.1276,'EPSG','9001','EPSG','8807','False northing',59445.9043,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10070','conversion','EPSG','7382','EPSG','4325','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7383','WISCRS Burnett County (ftUS)','See proj code 7382 for authoritative metric definition. Replaces and emulates WCCS zone.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',45.5355373517,'EPSG','9110','EPSG','8802','Longitude of natural origin',-92.2728,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000383841,'EPSG','9201','EPSG','8806','False easting',209999.999,'EPSG','9003','EPSG','8807','False northing',195032.104,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10071','conversion','EPSG','7383','EPSG','4325','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7384','WISCRS Douglas County (m)','Replaces and emulates WCCS zone. For equivalent non-metric definition see proj code 7385.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',45.53,'EPSG','9110','EPSG','8802','Longitude of natural origin',-91.55,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000385418,'EPSG','9201','EPSG','8806','False easting',59131.3183,'EPSG','9001','EPSG','8807','False northing',0.0041,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10072','conversion','EPSG','7384','EPSG','4326','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7385','WISCRS Douglas County (ftUS)','See proj code 7384 for authoritative metric definition. Replaces and emulates WCCS zone.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',45.53,'EPSG','9110','EPSG','8802','Longitude of natural origin',-91.55,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000385418,'EPSG','9201','EPSG','8806','False easting',194000.0,'EPSG','9003','EPSG','8807','False northing',0.013,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10073','conversion','EPSG','7385','EPSG','4326','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7386','WISCRS Florence County (m)','Replaces and emulates WCCS zone. For equivalent non-metric definition see proj code 7387.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',45.262,'EPSG','9110','EPSG','8802','Longitude of natural origin',-88.083,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000552095,'EPSG','9201','EPSG','8806','False easting',133502.6683,'EPSG','9001','EPSG','8807','False northing',0.0063,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10074','conversion','EPSG','7386','EPSG','4327','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7387','WISCRS Florence County (ftUS)','See proj code 7386 for authoritative metric definition. Replaces and emulates WCCS zone.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',45.262,'EPSG','9110','EPSG','8802','Longitude of natural origin',-88.083,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000552095,'EPSG','9201','EPSG','8806','False easting',438000.004,'EPSG','9003','EPSG','8807','False northing',0.021,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10075','conversion','EPSG','7387','EPSG','4327','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7388','WISCRS Forest County (m)','Replaces and emulates WCCS zone. For equivalent non-metric definition see proj code 7389.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',44.002,'EPSG','9110','EPSG','8802','Longitude of natural origin',-88.38,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000673004,'EPSG','9201','EPSG','8806','False easting',275844.5533,'EPSG','9001','EPSG','8807','False northing',0.0157,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10076','conversion','EPSG','7388','EPSG','4328','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7389','WISCRS Forest County (ftUS)','See proj code 7388 for authoritative metric definition. Replaces and emulates WCCS zone.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',44.002,'EPSG','9110','EPSG','8802','Longitude of natural origin',-88.38,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000673004,'EPSG','9201','EPSG','8806','False easting',905000.005,'EPSG','9003','EPSG','8807','False northing',0.052,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10077','conversion','EPSG','7389','EPSG','4328','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7390','WISCRS Iron County (m)','Replaces and emulates WCCS zone. For equivalent non-metric definition see proj code 7391.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',45.26,'EPSG','9110','EPSG','8802','Longitude of natural origin',-90.152,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000677153,'EPSG','9201','EPSG','8806','False easting',220980.4419,'EPSG','9001','EPSG','8807','False northing',0.0085,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10078','conversion','EPSG','7390','EPSG','4329','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7391','WISCRS Iron County (ftUS)','See proj code 7390 for authoritative metric definition. Replaces and emulates WCCS zone.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',45.26,'EPSG','9110','EPSG','8802','Longitude of natural origin',-90.152,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000677153,'EPSG','9201','EPSG','8806','False easting',725000.0,'EPSG','9003','EPSG','8807','False northing',0.028,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10079','conversion','EPSG','7391','EPSG','4329','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7392','WISCRS Oneida County (m)','Replaces and emulates WCCS zone. For equivalent non-metric definition see proj code 7393.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',45.4215205573,'EPSG','9110','EPSG','8802','Longitude of natural origin',-89.324,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000686968,'EPSG','9201','EPSG','8806','False easting',70104.1401,'EPSG','9001','EPSG','8807','False northing',57588.0346,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10080','conversion','EPSG','7392','EPSG','4330','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7393','WISCRS Oneida County (ftUS)','See proj code 7392 for authoritative metric definition. Replaces and emulates WCCS zone.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',45.4215205573,'EPSG','9110','EPSG','8802','Longitude of natural origin',-89.324,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000686968,'EPSG','9201','EPSG','8806','False easting',230000.0,'EPSG','9003','EPSG','8807','False northing',188936.744,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10081','conversion','EPSG','7393','EPSG','4330','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7394','WISCRS Price County (m)','Replaces and emulates WCCS zone. For equivalent non-metric definition see proj code 7395.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',44.332,'EPSG','9110','EPSG','8802','Longitude of natural origin',-90.292,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000649554,'EPSG','9201','EPSG','8806','False easting',227990.8546,'EPSG','9001','EPSG','8807','False northing',0.0109,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10082','conversion','EPSG','7394','EPSG','4332','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7395','WISCRS Price County (ftUS)','See proj code 7394 for authoritative metric definition. Replaces and emulates WCCS zone.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',44.332,'EPSG','9110','EPSG','8802','Longitude of natural origin',-90.292,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000649554,'EPSG','9201','EPSG','8806','False easting',747999.995,'EPSG','9003','EPSG','8807','False northing',0.036,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10083','conversion','EPSG','7395','EPSG','4332','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7396','WISCRS Sawyer County (m)','Replaces and emulates WCCS zone. For equivalent non-metric definition see proj code 7397.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',45.5400356873,'EPSG','9110','EPSG','8802','Longitude of natural origin',-91.07,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000573461,'EPSG','9201','EPSG','8806','False easting',216713.2336,'EPSG','9001','EPSG','8807','False northing',120734.1631,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10084','conversion','EPSG','7396','EPSG','4333','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7397','WISCRS Sawyer County (ftUS)','See proj code 7396 for authoritative metric definition. Replaces and emulates WCCS zone.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',45.5400356873,'EPSG','9110','EPSG','8802','Longitude of natural origin',-91.07,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000573461,'EPSG','9201','EPSG','8806','False easting',711000.001,'EPSG','9003','EPSG','8807','False northing',396108.667,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10085','conversion','EPSG','7397','EPSG','4333','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7398','WISCRS Vilas County (m)','Replaces and emulates WCCS zone. For equivalent non-metric definition see proj code 7399.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',46.0440238726,'EPSG','9110','EPSG','8802','Longitude of natural origin',-89.292,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000730142,'EPSG','9201','EPSG','8806','False easting',134417.0689,'EPSG','9001','EPSG','8807','False northing',50337.1092,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10086','conversion','EPSG','7398','EPSG','4334','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7399','WISCRS Vilas County (ftUS)','See proj code 7398 for authoritative metric definition. Replaces and emulates WCCS zone.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',46.0440238726,'EPSG','9110','EPSG','8802','Longitude of natural origin',-89.292,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000730142,'EPSG','9201','EPSG','8806','False easting',441000.0,'EPSG','9003','EPSG','8807','False northing',165147.666,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10087','conversion','EPSG','7399','EPSG','4334','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7424','WISCRS Washburn County (m)','Replaces and emulates WCCS zone. For equivalent non-metric definition see proj code 7425.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',45.57403914,'EPSG','9110','EPSG','8802','Longitude of natural origin',-91.47,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000475376,'EPSG','9201','EPSG','8806','False easting',234086.8682,'EPSG','9001','EPSG','8807','False northing',188358.6058,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10088','conversion','EPSG','7424','EPSG','4335','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7425','WISCRS Washburn County (ftUS)','See proj code 7424 for authoritative metric definition. Replaces and emulates WCCS zone.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',45.57403914,'EPSG','9110','EPSG','8802','Longitude of natural origin',-91.47,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000475376,'EPSG','9201','EPSG','8806','False easting',768000.0,'EPSG','9003','EPSG','8807','False northing',617973.193,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10089','conversion','EPSG','7425','EPSG','4335','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7426','WISCRS Barron County (m)','Replaces and emulates WCCS zone. For equivalent non-metric definition see proj code 7427.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',45.08,'EPSG','9110','EPSG','8802','Longitude of natural origin',-91.51,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000486665,'EPSG','9201','EPSG','8806','False easting',93150.0,'EPSG','9001','EPSG','8807','False northing',0.0029,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10090','conversion','EPSG','7426','EPSG','4331','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7427','WISCRS Barron County (ftUS)','See proj code 7426 for authoritative metric definition. Replaces and emulates WCCS zone.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',45.08,'EPSG','9110','EPSG','8802','Longitude of natural origin',-91.51,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000486665,'EPSG','9201','EPSG','8806','False easting',305609.625,'EPSG','9003','EPSG','8807','False northing',0.01,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10091','conversion','EPSG','7427','EPSG','4331','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7428','WISCRS Brown County (m)','Replaces and emulates WCCS zone. For equivalent non-metric definition see proj code 7429.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',43.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-88.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.00002,'EPSG','9201','EPSG','8806','False easting',31600.0,'EPSG','9001','EPSG','8807','False northing',4600.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10092','conversion','EPSG','7428','EPSG','4336','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7429','WISCRS Brown County (ftUS)','See proj code 7428 for authoritative metric definition. Replaces and emulates WCCS zone.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',43.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-88.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00002,'EPSG','9201','EPSG','8806','False easting',103674.333,'EPSG','9003','EPSG','8807','False northing',15091.833,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10093','conversion','EPSG','7429','EPSG','4336','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7430','WISCRS Buffalo County (m)','Replaces and emulates WCCS zone. For equivalent non-metric definition see proj code 7431.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',43.2853,'EPSG','9110','EPSG','8802','Longitude of natural origin',-91.475,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000382778,'EPSG','9201','EPSG','8806','False easting',175260.3502,'EPSG','9001','EPSG','8807','False northing',0.0048,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10094','conversion','EPSG','7430','EPSG','4337','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7431','WISCRS Buffalo County (ftUS)','See proj code 7430 for authoritative metric definition. Replaces and emulates WCCS zone.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',43.2853,'EPSG','9110','EPSG','8802','Longitude of natural origin',-91.475,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000382778,'EPSG','9201','EPSG','8806','False easting',574999.999,'EPSG','9003','EPSG','8807','False northing',0.016,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10095','conversion','EPSG','7431','EPSG','4337','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7432','WISCRS Chippewa County (m)','Replaces and emulates WCCS zone. For equivalent non-metric definition see proj code 7433.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',44.5840284835,'EPSG','9110','EPSG','8802','Longitude of natural origin',-91.174,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000391127,'EPSG','9201','EPSG','8806','False easting',60045.72,'EPSG','9001','EPSG','8807','False northing',44091.4346,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10096','conversion','EPSG','7432','EPSG','4338','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7433','WISCRS Chippewa County (ftUS)','See proj code 7432 for authoritative metric definition. Replaces and emulates WCCS zone.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',44.5840284835,'EPSG','9110','EPSG','8802','Longitude of natural origin',-91.174,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000391127,'EPSG','9201','EPSG','8806','False easting',197000.0,'EPSG','9003','EPSG','8807','False northing',144656.648,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10097','conversion','EPSG','7433','EPSG','4338','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7434','WISCRS Clark County (m)','Replaces and emulates WCCS zone. For equivalent non-metric definition see proj code 7435.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',43.36,'EPSG','9110','EPSG','8802','Longitude of natural origin',-90.423,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000463003,'EPSG','9201','EPSG','8806','False easting',199949.1989,'EPSG','9001','EPSG','8807','False northing',0.0086,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10098','conversion','EPSG','7434','EPSG','4339','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7435','WISCRS Clark County (ftUS)','See proj code 7434 for authoritative metric definition. Replaces and emulates WCCS zone.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',43.36,'EPSG','9110','EPSG','8802','Longitude of natural origin',-90.423,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000463003,'EPSG','9201','EPSG','8806','False easting',655999.997,'EPSG','9003','EPSG','8807','False northing',0.028,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10099','conversion','EPSG','7435','EPSG','4339','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7436','WISCRS Door County (m)','Replaces and emulates WCCS zone. For equivalent non-metric definition see proj code 7437.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',44.24,'EPSG','9110','EPSG','8802','Longitude of natural origin',-87.162,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000187521,'EPSG','9201','EPSG','8806','False easting',158801.1176,'EPSG','9001','EPSG','8807','False northing',0.0023,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10100','conversion','EPSG','7436','EPSG','4340','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7437','WISCRS Door County (ftUS)','See proj code 7436 for authoritative metric definition. Replaces and emulates WCCS zone.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',44.24,'EPSG','9110','EPSG','8802','Longitude of natural origin',-87.162,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000187521,'EPSG','9201','EPSG','8806','False easting',521000.0,'EPSG','9003','EPSG','8807','False northing',0.008,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10101','conversion','EPSG','7437','EPSG','4340','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7438','WISCRS Dunn County (m)','Replaces and emulates WCCS zone. For equivalent non-metric definition see proj code 7439.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',44.243,'EPSG','9110','EPSG','8802','Longitude of natural origin',-91.534,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000410324,'EPSG','9201','EPSG','8806','False easting',51816.104,'EPSG','9001','EPSG','8807','False northing',0.003,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10102','conversion','EPSG','7438','EPSG','4341','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7439','WISCRS Dunn County (ftUS)','See proj code 7438 for authoritative metric definition. Replaces and emulates WCCS zone.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',44.243,'EPSG','9110','EPSG','8802','Longitude of natural origin',-91.534,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000410324,'EPSG','9201','EPSG','8806','False easting',170000.001,'EPSG','9003','EPSG','8807','False northing',0.01,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10103','conversion','EPSG','7439','EPSG','4341','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7440','WISCRS Eau Claire County (m)','Replaces and emulates WCCS zone. For equivalent non-metric definition see proj code 7441.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',44.5220212055,'EPSG','9110','EPSG','8802','Longitude of natural origin',-91.172,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000035079,'EPSG','9201','EPSG','8806','False easting',120091.4402,'EPSG','9001','EPSG','8807','False northing',91687.9239,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10104','conversion','EPSG','7440','EPSG','4342','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7441','WISCRS Eau Claire County (ftUS)','See proj code 7440 for authoritative metric definition. Replaces and emulates WCCS zone.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',44.5220212055,'EPSG','9110','EPSG','8802','Longitude of natural origin',-91.172,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000035079,'EPSG','9201','EPSG','8806','False easting',394000.0,'EPSG','9003','EPSG','8807','False northing',300812.797,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10105','conversion','EPSG','7441','EPSG','4342','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7450','WISCRS Jackson County (m)','For equivalent non-metric definition see proj code 7451.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',44.151200646,'EPSG','9110','EPSG','8802','Longitude of natural origin',-90.503946747,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000353,'EPSG','9201','EPSG','8806','False easting',27000.0,'EPSG','9001','EPSG','8807','False northing',25000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10112','conversion','EPSG','7450','EPSG','4343','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7451','WISCRS Jackson County (ftUS)','See proj code 7450 for authoritative metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',44.151200646,'EPSG','9110','EPSG','8802','Longitude of natural origin',-90.503946747,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000353,'EPSG','9201','EPSG','8806','False easting',88582.5,'EPSG','9003','EPSG','8807','False northing',82020.833,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10113','conversion','EPSG','7451','EPSG','4343','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7452','WISCRS Langlade County (m)','Replaces and emulates WCCS zone. For equivalent non-metric definition see proj code 7453.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',45.0915253579,'EPSG','9110','EPSG','8802','Longitude of natural origin',-89.02,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000627024,'EPSG','9201','EPSG','8806','False easting',198425.197,'EPSG','9001','EPSG','8807','False northing',105279.7829,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10114','conversion','EPSG','7452','EPSG','4344','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7453','WISCRS Langlade County (ftUS)','See proj code 7452 for authoritative metric definition. Replaces and emulates WCCS zone.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',45.0915253579,'EPSG','9110','EPSG','8802','Longitude of natural origin',-89.02,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000627024,'EPSG','9201','EPSG','8806','False easting',651000.0,'EPSG','9003','EPSG','8807','False northing',345405.421,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10115','conversion','EPSG','7453','EPSG','4344','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7454','WISCRS Lincoln County (m)','Replaces and emulates WCCS zone. For equivalent non-metric definition see proj code 7455.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',44.504,'EPSG','9110','EPSG','8802','Longitude of natural origin',-89.44,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000599003,'EPSG','9201','EPSG','8806','False easting',116129.0323,'EPSG','9001','EPSG','8807','False northing',0.0058,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10116','conversion','EPSG','7454','EPSG','4345','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7455','WISCRS Lincoln County (ftUS)','See proj code 7454 for authoritative metric definition. Replaces and emulates WCCS zone.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',44.504,'EPSG','9110','EPSG','8802','Longitude of natural origin',-89.44,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000599003,'EPSG','9201','EPSG','8806','False easting',381000.0,'EPSG','9003','EPSG','8807','False northing',0.019,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10117','conversion','EPSG','7455','EPSG','4345','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7456','WISCRS Marathon County (m)','Replaces and emulates WCCS zone. For equivalent non-metric definition see proj code 7457.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',44.5403255925,'EPSG','9110','EPSG','8802','Longitude of natural origin',-89.4612,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000053289,'EPSG','9201','EPSG','8806','False easting',74676.1493,'EPSG','9001','EPSG','8807','False northing',55049.2669,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10118','conversion','EPSG','7456','EPSG','4346','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7457','WISCRS Marathon County (ftUS)','See proj code 7456 for authoritative metric definition. Replaces and emulates WCCS zone.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',44.5403255925,'EPSG','9110','EPSG','8802','Longitude of natural origin',-89.4612,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000053289,'EPSG','9201','EPSG','8806','False easting',245000.0,'EPSG','9003','EPSG','8807','False northing',180607.47,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10119','conversion','EPSG','7457','EPSG','4346','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7458','WISCRS Marinette County (m)','Replaces and emulates WCCS zone. For equivalent non-metric definition see proj code 7459.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',44.413,'EPSG','9110','EPSG','8802','Longitude of natural origin',-87.424,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000234982,'EPSG','9201','EPSG','8806','False easting',238658.8794,'EPSG','9001','EPSG','8807','False northing',0.0032,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10120','conversion','EPSG','7458','EPSG','4347','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7459','WISCRS Marinette County (ftUS)','See proj code 7458 for authoritative metric definition. Replaces and emulates WCCS zone.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',44.413,'EPSG','9110','EPSG','8802','Longitude of natural origin',-87.424,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000234982,'EPSG','9201','EPSG','8806','False easting',783000.007,'EPSG','9003','EPSG','8807','False northing',0.01,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10121','conversion','EPSG','7459','EPSG','4347','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7460','WISCRS Menominee County (m)','Replaces and emulates WCCS zone. For equivalent non-metric definition see proj code 7461.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',44.43,'EPSG','9110','EPSG','8802','Longitude of natural origin',-88.25,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000362499,'EPSG','9201','EPSG','8806','False easting',105461.0121,'EPSG','9001','EPSG','8807','False northing',0.0029,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10122','conversion','EPSG','7460','EPSG','4348','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7461','WISCRS Menominee County (ftUS)','See proj code 7460 for authoritative metric definition. Replaces and emulates WCCS zone.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',44.43,'EPSG','9110','EPSG','8802','Longitude of natural origin',-88.25,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000362499,'EPSG','9201','EPSG','8806','False easting',346000.004,'EPSG','9003','EPSG','8807','False northing',0.01,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10123','conversion','EPSG','7461','EPSG','4348','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7462','WISCRS Oconto County (m)','Replaces and emulates WCCS zone. For equivalent non-metric definition see proj code 7463.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',44.235,'EPSG','9110','EPSG','8802','Longitude of natural origin',-87.543,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000236869,'EPSG','9201','EPSG','8806','False easting',182880.3676,'EPSG','9001','EPSG','8807','False northing',0.0033,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10124','conversion','EPSG','7462','EPSG','4349','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7463','WISCRS Oconto County (ftUS)','See proj code 7462 for authoritative metric definition. Replaces and emulates WCCS zone.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',44.235,'EPSG','9110','EPSG','8802','Longitude of natural origin',-87.543,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000236869,'EPSG','9201','EPSG','8806','False easting',600000.006,'EPSG','9003','EPSG','8807','False northing',0.011,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10125','conversion','EPSG','7463','EPSG','4349','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7464','WISCRS Pepin and Pierce Counties (m)','Replaces and emulates WCCS zone. For equivalent non-metric definition see proj code 7465. Defined by Wisconsin DOT as two separate zones (one for each county) each having identical projection parameter values: see info source.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',44.3810135939,'EPSG','9110','EPSG','8802','Longitude of natural origin',-92.134,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000362977,'EPSG','9201','EPSG','8806','False easting',167640.3354,'EPSG','9001','EPSG','8807','False northing',86033.0876,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10126','conversion','EPSG','7464','EPSG','4350','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7465','WISCRS Pepin and Pierce Counties (ftUS)','See proj code 7464 for authoritative metric definition. Replaces and emulates WCCS zone. Defined by Wisconsin DOT as two separate zones (one for each county) each having identical projection parameter values: see info source.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',44.3810135939,'EPSG','9110','EPSG','8802','Longitude of natural origin',-92.134,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000362977,'EPSG','9201','EPSG','8806','False easting',550000.0,'EPSG','9003','EPSG','8807','False northing',282260.222,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10127','conversion','EPSG','7465','EPSG','4350','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7466','WISCRS Polk County (m)','Replaces and emulates WCCS zone. For equivalent non-metric definition see proj code 7467.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',44.394,'EPSG','9110','EPSG','8802','Longitude of natural origin',-92.38,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000433849,'EPSG','9201','EPSG','8806','False easting',141732.2823,'EPSG','9001','EPSG','8807','False northing',0.0059,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10128','conversion','EPSG','7466','EPSG','4351','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7467','WISCRS Polk County (ftUS)','See proj code 7466 for authoritative metric definition. Replaces and emulates WCCS zone.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',44.394,'EPSG','9110','EPSG','8802','Longitude of natural origin',-92.38,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000433849,'EPSG','9201','EPSG','8806','False easting',464999.996,'EPSG','9003','EPSG','8807','False northing',0.019,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10129','conversion','EPSG','7467','EPSG','4351','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7468','WISCRS Portage County (m)','Replaces and emulates WCCS zone. For equivalent non-metric definition see proj code 7469.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',44.2500566311,'EPSG','9110','EPSG','8802','Longitude of natural origin',-89.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000039936,'EPSG','9201','EPSG','8806','False easting',56388.1128,'EPSG','9001','EPSG','8807','False northing',50022.1874,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10130','conversion','EPSG','7468','EPSG','4352','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7469','WISCRS Portage County (ftUS)','See proj code 7468 for authoritative metric definition. Replaces and emulates WCCS zone.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',44.2500566311,'EPSG','9110','EPSG','8802','Longitude of natural origin',-89.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000039936,'EPSG','9201','EPSG','8806','False easting',185000.0,'EPSG','9003','EPSG','8807','False northing',164114.46,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10131','conversion','EPSG','7469','EPSG','4352','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7470','WISCRS Rusk County (m)','Replaces and emulates WCCS zone. For equivalent non-metric definition see proj code 7471.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',43.551,'EPSG','9110','EPSG','8802','Longitude of natural origin',-91.04,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000495976,'EPSG','9201','EPSG','8806','False easting',250546.1013,'EPSG','9001','EPSG','8807','False northing',0.0234,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10132','conversion','EPSG','7470','EPSG','4353','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7471','WISCRS Rusk County (ftUS)','See proj code 7470 for authoritative metric definition. Replaces and emulates WCCS zone.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',43.551,'EPSG','9110','EPSG','8802','Longitude of natural origin',-91.04,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000495976,'EPSG','9201','EPSG','8806','False easting',822000.001,'EPSG','9003','EPSG','8807','False northing',0.077,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10133','conversion','EPSG','7471','EPSG','4353','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7472','WISCRS Shawano County (m)','Replaces and emulates WCCS zone. For equivalent non-metric definition see proj code 7473.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',44.021,'EPSG','9110','EPSG','8802','Longitude of natural origin',-88.362,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000032144,'EPSG','9201','EPSG','8806','False easting',262433.3253,'EPSG','9001','EPSG','8807','False northing',0.0096,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10134','conversion','EPSG','7472','EPSG','4354','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7473','WISCRS Shawano County (ftUS)','See proj code 7472 for authoritative metric definition. Replaces and emulates WCCS zone.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',44.021,'EPSG','9110','EPSG','8802','Longitude of natural origin',-88.362,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000032144,'EPSG','9201','EPSG','8806','False easting',861000.001,'EPSG','9003','EPSG','8807','False northing',0.031,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10135','conversion','EPSG','7473','EPSG','4354','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7474','WISCRS St. Croix County (m)','Replaces and emulates WCCS zone. For equivalent non-metric definition see proj code 7475.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',44.021,'EPSG','9110','EPSG','8802','Longitude of natural origin',-92.38,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000381803,'EPSG','9201','EPSG','8806','False easting',165506.7302,'EPSG','9001','EPSG','8807','False northing',0.0103,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10136','conversion','EPSG','7474','EPSG','4355','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7475','WISCRS St. Croix County (ftUS)','See proj code 7474 for authoritative metric definition. Replaces and emulates WCCS zone.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',44.021,'EPSG','9110','EPSG','8802','Longitude of natural origin',-92.38,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000381803,'EPSG','9201','EPSG','8806','False easting',542999.997,'EPSG','9003','EPSG','8807','False northing',0.034,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10137','conversion','EPSG','7475','EPSG','4355','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7476','WISCRS Taylor County (m)','Replaces and emulates WCCS zone. For equivalent non-metric definition see proj code 7477.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',45.1040159509,'EPSG','9110','EPSG','8802','Longitude of natural origin',-90.29,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000597566,'EPSG','9201','EPSG','8806','False easting',187147.5744,'EPSG','9001','EPSG','8807','False northing',107746.7522,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10138','conversion','EPSG','7476','EPSG','4356','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7477','WISCRS Taylor County (ftUS)','See proj code 7476 for authoritative metric definition. Replaces and emulates WCCS zone.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',45.1040159509,'EPSG','9110','EPSG','8802','Longitude of natural origin',-90.29,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000597566,'EPSG','9201','EPSG','8806','False easting',614000.0,'EPSG','9003','EPSG','8807','False northing',353499.136,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10139','conversion','EPSG','7477','EPSG','4356','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7478','WISCRS Trempealeau County (m)','Replaces and emulates WCCS zone. For equivalent non-metric definition see proj code 7479.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',43.094,'EPSG','9110','EPSG','8802','Longitude of natural origin',-91.22,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000361538,'EPSG','9201','EPSG','8806','False easting',256946.9138,'EPSG','9001','EPSG','8807','False northing',0.0041,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10140','conversion','EPSG','7478','EPSG','4357','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7479','WISCRS Trempealeau County (ftUS)','See proj code 7478 for authoritative metric definition. Replaces and emulates WCCS zone.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',43.094,'EPSG','9110','EPSG','8802','Longitude of natural origin',-91.22,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000361538,'EPSG','9201','EPSG','8806','False easting',843000.0,'EPSG','9003','EPSG','8807','False northing',0.013,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10141','conversion','EPSG','7479','EPSG','4357','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7480','WISCRS Waupaca County (m)','Replaces and emulates WCCS zone. For equivalent non-metric definition see proj code 7481.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',43.2513,'EPSG','9110','EPSG','8802','Longitude of natural origin',-88.49,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000333645,'EPSG','9201','EPSG','8806','False easting',185013.9709,'EPSG','9001','EPSG','8807','False northing',0.007,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10142','conversion','EPSG','7480','EPSG','4358','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7481','WISCRS Waupaca County (ftUS)','See proj code 7480 for authoritative metric definition. Replaces and emulates WCCS zone.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',43.2513,'EPSG','9110','EPSG','8802','Longitude of natural origin',-88.49,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000333645,'EPSG','9201','EPSG','8806','False easting',607000.003,'EPSG','9003','EPSG','8807','False northing',0.023,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10143','conversion','EPSG','7481','EPSG','4358','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7482','WISCRS Wood County (m)','Replaces and emulates WCCS zone. For equivalent non-metric definition see proj code 7483.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',44.214534369,'EPSG','9110','EPSG','8802','Longitude of natural origin',-90.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000421209,'EPSG','9201','EPSG','8806','False easting',208483.6173,'EPSG','9001','EPSG','8807','False northing',134589.754,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10144','conversion','EPSG','7482','EPSG','4359','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7483','WISCRS Wood County (ftUS)','See proj code 7482 for authoritative metric definition. Replaces and emulates WCCS zone.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',44.214534369,'EPSG','9110','EPSG','8802','Longitude of natural origin',-90.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000421209,'EPSG','9201','EPSG','8806','False easting',684000.001,'EPSG','9003','EPSG','8807','False northing',441566.551,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10145','conversion','EPSG','7483','EPSG','4359','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7484','WISCRS Adams and Juneau Counties (m)','Replaces and emulates WCCS zone. For equivalent non-metric definition see proj code 7485. Defined by Wisconsin DOT as two separate zones (one for each county) each having identical projection parameter values: see info source.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',43.22,'EPSG','9110','EPSG','8802','Longitude of natural origin',-90.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000365285,'EPSG','9201','EPSG','8806','False easting',147218.6942,'EPSG','9001','EPSG','8807','False northing',0.0037,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10146','conversion','EPSG','7484','EPSG','4360','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7485','WISCRS Adams and Juneau Counties (ftUS)','See proj code 7484 for authoritative metric definition. Replaces and emulates WCCS zone. Defined by Wisconsin DOT as two separate zones (one for each county) each having identical projection parameter values: see info source.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',43.22,'EPSG','9110','EPSG','8802','Longitude of natural origin',-90.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000365285,'EPSG','9201','EPSG','8806','False easting',482999.999,'EPSG','9003','EPSG','8807','False northing',0.012,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10147','conversion','EPSG','7485','EPSG','4360','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7486','WISCRS Calumet, Fond du Lac, Outagamie and Winnebago Counties (m)','Replaces and emulates WCCS zone. For equivalent non-metric definition see proj code 7487. Defined by Wisconsin DOT as four separate zones (one for each county) each having identical projection parameter values: see info source.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',42.431,'EPSG','9110','EPSG','8802','Longitude of natural origin',-88.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000286569,'EPSG','9201','EPSG','8806','False easting',244754.8893,'EPSG','9001','EPSG','8807','False northing',0.0049,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10148','conversion','EPSG','7486','EPSG','4361','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7487','WISCRS Calumet, Fond du Lac, Outagamie and Winnebago Counties (ftUS)','See proj code 7486 for authoritative metric definition. Replaces and emulates WCCS zone. Defined by Wisconsin DOT as four separate zones (one for each county) each having identical projection parameter values: see info source.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',42.431,'EPSG','9110','EPSG','8802','Longitude of natural origin',-88.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000286569,'EPSG','9201','EPSG','8806','False easting',802999.999,'EPSG','9003','EPSG','8807','False northing',0.016,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10149','conversion','EPSG','7487','EPSG','4361','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7488','WISCRS Columbia County (m)','Replaces and emulates WCCS zone. For equivalent non-metric definition see proj code 7489.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',43.2745167925,'EPSG','9110','EPSG','8802','Longitude of natural origin',-89.234,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00003498,'EPSG','9201','EPSG','8806','False easting',169164.3381,'EPSG','9001','EPSG','8807','False northing',111569.6134,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10150','conversion','EPSG','7488','EPSG','4362','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7489','WISCRS Columbia County (ftUS)','See proj code 7488 for authoritative metric definition. Replaces and emulates WCCS zone.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',43.2745167925,'EPSG','9110','EPSG','8802','Longitude of natural origin',-89.234,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00003498,'EPSG','9201','EPSG','8806','False easting',554999.999,'EPSG','9003','EPSG','8807','False northing',366041.307,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10151','conversion','EPSG','7489','EPSG','4362','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7490','WISCRS Crawford County (m)','Replaces and emulates WCCS zone. For equivalent non-metric definition see proj code 7491.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',43.1200200178,'EPSG','9110','EPSG','8802','Longitude of natural origin',-90.562,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000349151,'EPSG','9201','EPSG','8806','False easting',113690.6274,'EPSG','9001','EPSG','8807','False northing',53703.1201,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10152','conversion','EPSG','7490','EPSG','4363','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7491','WISCRS Crawford County (ftUS)','See proj code 7490 for authoritative metric definition. Replaces and emulates WCCS zone.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',43.1200200178,'EPSG','9110','EPSG','8802','Longitude of natural origin',-90.562,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000349151,'EPSG','9201','EPSG','8806','False easting',373000.0,'EPSG','9003','EPSG','8807','False northing',176190.987,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10153','conversion','EPSG','7491','EPSG','4363','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7492','WISCRS Dane County (m)','Replaces and emulates WCCS zone. For equivalent non-metric definition see proj code 7493.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',43.0410257735,'EPSG','9110','EPSG','8802','Longitude of natural origin',-89.252,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000384786,'EPSG','9201','EPSG','8806','False easting',247193.2944,'EPSG','9001','EPSG','8807','False northing',146591.9896,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10154','conversion','EPSG','7492','EPSG','4364','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7493','WISCRS Dane County (ftUS)','See proj code 7492 for authoritative metric definition. Replaces and emulates WCCS zone.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',43.0410257735,'EPSG','9110','EPSG','8802','Longitude of natural origin',-89.252,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000384786,'EPSG','9201','EPSG','8806','False easting',811000.0,'EPSG','9003','EPSG','8807','False northing',480943.886,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10155','conversion','EPSG','7493','EPSG','4364','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7494','WISCRS Dodge and Jefferson Counties (m)','Replaces and emulates WCCS zone. For equivalent non-metric definition see proj code 7495. Defined by Wisconsin DOT as two separate zones (one for each county) each having identical projection parameter values: see info source.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',41.282,'EPSG','9110','EPSG','8802','Longitude of natural origin',-88.463,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000346418,'EPSG','9201','EPSG','8806','False easting',263347.7263,'EPSG','9001','EPSG','8807','False northing',0.0076,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10156','conversion','EPSG','7494','EPSG','4365','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7495','WISCRS Dodge and Jefferson Counties (ftUS)','See proj code 7494 for authoritative metric definition. Replaces and emulates WCCS zone. Defined by Wisconsin DOT as two separate zones (one for each county) each having identical projection parameter values: see info source.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',41.282,'EPSG','9110','EPSG','8802','Longitude of natural origin',-88.463,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000346418,'EPSG','9201','EPSG','8806','False easting',863999.999,'EPSG','9003','EPSG','8807','False northing',0.025,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10157','conversion','EPSG','7495','EPSG','4365','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7496','WISCRS Grant County (m)','Replaces and emulates WCCS zone. For equivalent non-metric definition see proj code 7497.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',41.244,'EPSG','9110','EPSG','8802','Longitude of natural origin',-90.48,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000349452,'EPSG','9201','EPSG','8806','False easting',242316.4841,'EPSG','9001','EPSG','8807','False northing',0.01,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10158','conversion','EPSG','7496','EPSG','4366','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7497','WISCRS Grant County (ftUS)','See proj code 7496 for authoritative metric definition. Replaces and emulates WCCS zone.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',41.244,'EPSG','9110','EPSG','8802','Longitude of natural origin',-90.48,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000349452,'EPSG','9201','EPSG','8806','False easting',794999.998,'EPSG','9003','EPSG','8807','False northing',0.033,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10159','conversion','EPSG','7497','EPSG','4366','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7498','WISCRS Green and Lafayette Counties (m)','Replaces and emulates WCCS zone. For equivalent non-metric definition see proj code 7499.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',42.3815224197,'EPSG','9110','EPSG','8802','Longitude of natural origin',-89.502,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000390487,'EPSG','9201','EPSG','8806','False easting',170078.7403,'EPSG','9001','EPSG','8807','False northing',45830.2947,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10160','conversion','EPSG','7498','EPSG','4367','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7499','WISCRS Green and Lafayette Counties (ftUS)','See proj code 7498 for authoritative metric definition. Replaces and emulates WCCS zone.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',42.3815224197,'EPSG','9110','EPSG','8802','Longitude of natural origin',-89.502,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000390487,'EPSG','9201','EPSG','8806','False easting',558000.0,'EPSG','9003','EPSG','8807','False northing',150361.559,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10161','conversion','EPSG','7499','EPSG','4367','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7500','WISCRS Green Lake and Marquette Counties (m)','Replaces and emulates WCCS zone. For equivalent non-metric definition see proj code 7501.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',43.4825200424,'EPSG','9110','EPSG','8802','Longitude of natural origin',-89.143,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000344057,'EPSG','9201','EPSG','8806','False easting',150876.3018,'EPSG','9001','EPSG','8807','False northing',79170.7795,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10162','conversion','EPSG','7500','EPSG','4368','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7501','WISCRS Green Lake and Marquette Counties (ftUS)','See proj code 7500 for authoritative metric definition. Replaces and emulates WCCS zone.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',43.4825200424,'EPSG','9110','EPSG','8802','Longitude of natural origin',-89.143,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000344057,'EPSG','9201','EPSG','8806','False easting',495000.0,'EPSG','9003','EPSG','8807','False northing',259746.132,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10163','conversion','EPSG','7501','EPSG','4368','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7502','WISCRS Iowa County (m)','Replaces and emulates WCCS zone. For equivalent non-metric definition see proj code 7503.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',42.322,'EPSG','9110','EPSG','8802','Longitude of natural origin',-90.094,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000394961,'EPSG','9201','EPSG','8806','False easting',113081.0261,'EPSG','9001','EPSG','8807','False northing',0.0045,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10164','conversion','EPSG','7502','EPSG','4369','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7503','WISCRS Iowa County (ftUS)','See proj code 7502 for authoritative metric definition. Replaces and emulates WCCS zone.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',42.322,'EPSG','9110','EPSG','8802','Longitude of natural origin',-90.094,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000394961,'EPSG','9201','EPSG','8806','False easting',371000.0,'EPSG','9003','EPSG','8807','False northing',0.015,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10165','conversion','EPSG','7503','EPSG','4369','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7504','WISCRS Kenosha, Milwaukee, Ozaukee and Racine Counties (m)','Replaces and emulates WCCS zone. For equivalent non-metric definition see proj code 7505. Defined by Wisconsin DOT as four separate zones (one for each county) each having identical projection parameter values: see info source.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',42.13,'EPSG','9110','EPSG','8802','Longitude of natural origin',-87.534,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000260649,'EPSG','9201','EPSG','8806','False easting',185928.3728,'EPSG','9001','EPSG','8807','False northing',0.0009,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10166','conversion','EPSG','7504','EPSG','4370','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7505','WISCRS Kenosha, Milwaukee, Ozaukee and Racine Counties (ftUS)','See proj code 7504 for authoritative metric definition. Replaces and emulates WCCS zone. Defined by Wisconsin DOT as four separate zones (one for each county) each having identical projection parameter values: see info source.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',42.13,'EPSG','9110','EPSG','8802','Longitude of natural origin',-87.534,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000260649,'EPSG','9201','EPSG','8806','False easting',610000.003,'EPSG','9003','EPSG','8807','False northing',0.003,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10167','conversion','EPSG','7505','EPSG','4370','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7506','WISCRS Kewaunee, Manitowoc and Sheboygan Counties (m)','Replaces and emulates WCCS zone. For equivalent non-metric definition see proj code 7507. Defined by Wisconsin DOT as three separate zones (one for each county) each having identical projection parameter values: see info source.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',43.16,'EPSG','9110','EPSG','8802','Longitude of natural origin',-87.33,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000233704,'EPSG','9201','EPSG','8806','False easting',79857.7614,'EPSG','9001','EPSG','8807','False northing',0.0012,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10168','conversion','EPSG','7506','EPSG','4371','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7507','WISCRS Kewaunee, Manitowoc and Sheboygan Counties (ftUS)','See proj code 7506 for authoritative metric definition. Replaces and emulates WCCS zone. Defined by Wisconsin DOT as three separate zones (one for each county) each having identical projection parameter values: see info source.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',43.16,'EPSG','9110','EPSG','8802','Longitude of natural origin',-87.33,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000233704,'EPSG','9201','EPSG','8806','False easting',262000.006,'EPSG','9003','EPSG','8807','False northing',0.004,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10169','conversion','EPSG','7507','EPSG','4371','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7508','WISCRS La Crosse County (m)','Replaces and emulates WCCS zone. For equivalent non-metric definition see proj code 7509.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',43.2704,'EPSG','9110','EPSG','8802','Longitude of natural origin',-91.19,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000319985,'EPSG','9201','EPSG','8806','False easting',130454.6598,'EPSG','9001','EPSG','8807','False northing',0.0033,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10170','conversion','EPSG','7508','EPSG','4372','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7509','WISCRS La Crosse County (ftUS)','See proj code 7508 for authoritative metric definition. Replaces and emulates WCCS zone.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',43.2704,'EPSG','9110','EPSG','8802','Longitude of natural origin',-91.19,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000319985,'EPSG','9201','EPSG','8806','False easting',427999.996,'EPSG','9003','EPSG','8807','False northing',0.011,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10171','conversion','EPSG','7509','EPSG','4372','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7510','WISCRS Monroe County (m)','Replaces and emulates WCCS zone. For equivalent non-metric definition see proj code 7511.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',44.0000266143,'EPSG','9110','EPSG','8802','Longitude of natural origin',-90.383,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000434122,'EPSG','9201','EPSG','8806','False easting',204521.209,'EPSG','9001','EPSG','8807','False northing',121923.9861,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10172','conversion','EPSG','7510','EPSG','4373','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7511','WISCRS Monroe County (ftUS)','See proj code 7510 for authoritative metric definition. Replaces and emulates WCCS zone.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',44.0000266143,'EPSG','9110','EPSG','8802','Longitude of natural origin',-90.383,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000434122,'EPSG','9201','EPSG','8806','False easting',671000.0,'EPSG','9003','EPSG','8807','False northing',400012.278,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10173','conversion','EPSG','7511','EPSG','4373','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7512','WISCRS Richland County (m)','Replaces and emulates WCCS zone. For equivalent non-metric definition see proj code 7513.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',43.1920326539,'EPSG','9110','EPSG','8802','Longitude of natural origin',-90.255,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000375653,'EPSG','9201','EPSG','8806','False easting',202387.6048,'EPSG','9001','EPSG','8807','False northing',134255.4253,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10174','conversion','EPSG','7512','EPSG','4374','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7513','WISCRS Richland County (ftUS)','See proj code 7512 for authoritative metric definition. Replaces and emulates WCCS zone.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',43.1920326539,'EPSG','9110','EPSG','8802','Longitude of natural origin',-90.255,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000375653,'EPSG','9201','EPSG','8806','False easting',664000.0,'EPSG','9003','EPSG','8807','False northing',440469.675,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10175','conversion','EPSG','7513','EPSG','4374','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7514','WISCRS Rock County (m)','Replaces and emulates WCCS zone. For equivalent non-metric definition see proj code 7515.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',41.564,'EPSG','9110','EPSG','8802','Longitude of natural origin',-89.042,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000337311,'EPSG','9201','EPSG','8806','False easting',146304.2926,'EPSG','9001','EPSG','8807','False northing',0.0068,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10176','conversion','EPSG','7514','EPSG','4375','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7515','WISCRS Rock County (ftUS)','See proj code 7514 for authoritative metric definition. Replaces and emulates WCCS zone.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',41.564,'EPSG','9110','EPSG','8802','Longitude of natural origin',-89.042,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000337311,'EPSG','9201','EPSG','8806','False easting',480000.0,'EPSG','9003','EPSG','8807','False northing',0.022,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10177','conversion','EPSG','7515','EPSG','4375','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7516','WISCRS Sauk County (m)','Replaces and emulates WCCS zone. For equivalent non-metric definition see proj code 7517.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',42.491,'EPSG','9110','EPSG','8802','Longitude of natural origin',-89.54,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000373868,'EPSG','9201','EPSG','8806','False easting',185623.5716,'EPSG','9001','EPSG','8807','False northing',0.0051,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10178','conversion','EPSG','7516','EPSG','4376','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7517','WISCRS Sauk County (ftUS)','See proj code 7516 for authoritative metric definition. Replaces and emulates WCCS zone.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',42.491,'EPSG','9110','EPSG','8802','Longitude of natural origin',-89.54,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000373868,'EPSG','9201','EPSG','8806','False easting',609000.001,'EPSG','9003','EPSG','8807','False northing',0.017,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10179','conversion','EPSG','7517','EPSG','4376','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7518','WISCRS Vernon County (m)','Replaces and emulates WCCS zone. For equivalent non-metric definition see proj code 7519.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',43.3430118583,'EPSG','9110','EPSG','8802','Longitude of natural origin',-90.47,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000408158,'EPSG','9201','EPSG','8806','False easting',222504.4451,'EPSG','9001','EPSG','8807','False northing',47532.0602,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10180','conversion','EPSG','7518','EPSG','4377','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7519','WISCRS Vernon County (ftUS)','See proj code 7518 for authoritative metric definition. Replaces and emulates WCCS zone.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',43.3430118583,'EPSG','9110','EPSG','8802','Longitude of natural origin',-90.47,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000408158,'EPSG','9201','EPSG','8806','False easting',730000.0,'EPSG','9003','EPSG','8807','False northing',155944.768,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10181','conversion','EPSG','7519','EPSG','4377','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7520','WISCRS Walworth County (m)','Replaces and emulates WCCS zone. For equivalent non-metric definition see proj code 7521.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',42.4010063549,'EPSG','9110','EPSG','8802','Longitude of natural origin',-88.323,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000367192,'EPSG','9201','EPSG','8806','False easting',232562.8651,'EPSG','9001','EPSG','8807','False northing',111088.2224,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10182','conversion','EPSG','7520','EPSG','4378','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7521','WISCRS Walworth County (ftUS)','See proj code 7520 for authoritative metric definition. Replaces and emulates WCCS zone.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',42.4010063549,'EPSG','9110','EPSG','8802','Longitude of natural origin',-88.323,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000367192,'EPSG','9201','EPSG','8806','False easting',763000.0,'EPSG','9003','EPSG','8807','False northing',364461.943,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10183','conversion','EPSG','7521','EPSG','4378','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7522','WISCRS Washington County (m)','Replaces and emulates WCCS zone. For equivalent non-metric definition see proj code 7523.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',42.5505,'EPSG','9110','EPSG','8802','Longitude of natural origin',-88.035,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00003738,'EPSG','9201','EPSG','8806','False easting',120091.4415,'EPSG','9001','EPSG','8807','False northing',0.003,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10184','conversion','EPSG','7522','EPSG','4379','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7523','WISCRS Washington County (ftUS)','See proj code 7522 for authoritative metric definition. Replaces and emulates WCCS zone.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',42.5505,'EPSG','9110','EPSG','8802','Longitude of natural origin',-88.035,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00003738,'EPSG','9201','EPSG','8806','False easting',394000.004,'EPSG','9003','EPSG','8807','False northing',0.01,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10185','conversion','EPSG','7523','EPSG','4379','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7524','WISCRS Waukesha County (m)','Replaces and emulates WCCS zone. For equivalent non-metric definition see proj code 7525.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',42.341,'EPSG','9110','EPSG','8802','Longitude of natural origin',-88.133,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000346179,'EPSG','9201','EPSG','8806','False easting',208788.418,'EPSG','9001','EPSG','8807','False northing',0.0034,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10186','conversion','EPSG','7524','EPSG','4380','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7525','WISCRS Waukesha County (ftUS)','See proj code 7524 for authoritative metric definition. Replaces and emulates WCCS zone.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',42.341,'EPSG','9110','EPSG','8802','Longitude of natural origin',-88.133,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000346179,'EPSG','9201','EPSG','8806','False easting',685000.001,'EPSG','9003','EPSG','8807','False northing',0.011,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10187','conversion','EPSG','7525','EPSG','4380','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7526','WISCRS Waushara County (m)','Replaces and emulates WCCS zone. For equivalent non-metric definition see proj code 7527.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',44.0650198565,'EPSG','9110','EPSG','8802','Longitude of natural origin',-89.143,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000392096,'EPSG','9201','EPSG','8806','False easting',120091.4402,'EPSG','9001','EPSG','8807','False northing',45069.7587,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10188','conversion','EPSG','7526','EPSG','4381','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7527','WISCRS Waushara County (ftUS)','See proj code 7526 for authoritative metric definition. Replaces and emulates WCCS zone.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',44.0650198565,'EPSG','9110','EPSG','8802','Longitude of natural origin',-89.143,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000392096,'EPSG','9201','EPSG','8806','False easting',394000.0,'EPSG','9003','EPSG','8807','False northing',147866.367,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10189','conversion','EPSG','7527','EPSG','4381','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','7687','Kyrgyzstan zone 1','Established by government resolution N235 of 2010-10-07.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',68.31,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',1300000.0,'EPSG','9001','EPSG','8807','False northing',14743.5,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10209','conversion','EPSG','7687','EPSG','4385','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','7688','Kyrgyzstan zone 2','Established by government resolution N235 of 2010-10-07.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',71.31,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',2300000.0,'EPSG','9001','EPSG','8807','False northing',14743.5,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10210','conversion','EPSG','7688','EPSG','4386','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','7689','Kyrgyzstan zone 3','Established by government resolution N235 of 2010-10-07.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',74.31,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',3300000.0,'EPSG','9001','EPSG','8807','False northing',14743.5,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10211','conversion','EPSG','7689','EPSG','4387','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','7690','Kyrgyzstan zone 4','Established by government resolution N235 of 2010-10-07.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',77.31,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',4300000.0,'EPSG','9001','EPSG','8807','False northing',14743.5,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10212','conversion','EPSG','7690','EPSG','4388','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','7691','Kyrgyzstan zone 5','Established by government resolution N235 of 2010-10-07.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',80.31,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',5300000.0,'EPSG','9001','EPSG','8807','False northing',14743.5,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10213','conversion','EPSG','7691','EPSG','4389','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','7722','Survey of India Lambert','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',24.0,'EPSG','9102','EPSG','8822','Longitude of false origin',80.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',12.2822638,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',35.1022096,'EPSG','9110','EPSG','8826','Easting at false origin',4000000.0,'EPSG','9001','EPSG','8827','Northing at false origin',4000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10235','conversion','EPSG','7722','EPSG','1121','EPSG','1218');
INSERT INTO "conversion" VALUES('EPSG','7723','Andhra Pradesh NSF LCC','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',16.25543298,'EPSG','9102','EPSG','8822','Longitude of false origin',80.875,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',13.75,'EPSG','9102','EPSG','8824','Latitude of 2nd standard parallel',18.75,'EPSG','9102','EPSG','8826','Easting at false origin',1000000.0,'EPSG','9001','EPSG','8827','Northing at false origin',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10236','conversion','EPSG','7723','EPSG','4394','EPSG','1219');
INSERT INTO "conversion" VALUES('EPSG','7724','Arunachal Pradesh NSF LCC','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',28.00157897,'EPSG','9102','EPSG','8822','Longitude of false origin',94.5,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',27.0,'EPSG','9102','EPSG','8824','Latitude of 2nd standard parallel',29.0,'EPSG','9102','EPSG','8826','Easting at false origin',1000000.0,'EPSG','9001','EPSG','8827','Northing at false origin',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10237','conversion','EPSG','7724','EPSG','4395','EPSG','1219');
INSERT INTO "conversion" VALUES('EPSG','7725','Assam NSF LCC','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',26.00257703,'EPSG','9102','EPSG','8822','Longitude of false origin',92.75,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',24.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',27.2,'EPSG','9110','EPSG','8826','Easting at false origin',1000000.0,'EPSG','9001','EPSG','8827','Northing at false origin',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10238','conversion','EPSG','7725','EPSG','4396','EPSG','1219');
INSERT INTO "conversion" VALUES('EPSG','7726','Bihar NSF LCC','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',25.87725247,'EPSG','9102','EPSG','8822','Longitude of false origin',85.875,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',24.625,'EPSG','9102','EPSG','8824','Latitude of 2nd standard parallel',27.125,'EPSG','9102','EPSG','8826','Easting at false origin',1000000.0,'EPSG','9001','EPSG','8827','Northing at false origin',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10239','conversion','EPSG','7726','EPSG','4397','EPSG','1219');
INSERT INTO "conversion" VALUES('EPSG','7727','Delhi NSF LCC','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',28.62510126,'EPSG','9102','EPSG','8822','Longitude of false origin',77.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',28.223,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',28.523,'EPSG','9110','EPSG','8826','Easting at false origin',1000000.0,'EPSG','9001','EPSG','8827','Northing at false origin',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10240','conversion','EPSG','7727','EPSG','4422','EPSG','1219');
INSERT INTO "conversion" VALUES('EPSG','7728','Gujarat NSF LCC','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',22.37807121,'EPSG','9102','EPSG','8822','Longitude of false origin',71.375,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',20.473,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',23.573,'EPSG','9110','EPSG','8826','Easting at false origin',1000000.0,'EPSG','9001','EPSG','8827','Northing at false origin',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10241','conversion','EPSG','7728','EPSG','4400','EPSG','1219');
INSERT INTO "conversion" VALUES('EPSG','7729','Haryana NSF LCC','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',29.25226266,'EPSG','9102','EPSG','8822','Longitude of false origin',76.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',28.05,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',30.25,'EPSG','9110','EPSG','8826','Easting at false origin',1000000.0,'EPSG','9001','EPSG','8827','Northing at false origin',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10242','conversion','EPSG','7729','EPSG','4401','EPSG','1219');
INSERT INTO "conversion" VALUES('EPSG','7730','Himachal Pradesh NSF LCC','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',31.75183497,'EPSG','9102','EPSG','8822','Longitude of false origin',77.375,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',30.75,'EPSG','9102','EPSG','8824','Latitude of 2nd standard parallel',32.75,'EPSG','9102','EPSG','8826','Easting at false origin',1000000.0,'EPSG','9001','EPSG','8827','Northing at false origin',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10243','conversion','EPSG','7730','EPSG','4402','EPSG','1219');
INSERT INTO "conversion" VALUES('EPSG','7731','Jammu and Kashmir NSF LCC','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',34.75570874,'EPSG','9102','EPSG','8822','Longitude of false origin',76.5,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',33.05,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',36.25,'EPSG','9110','EPSG','8826','Easting at false origin',1000000.0,'EPSG','9001','EPSG','8827','Northing at false origin',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10244','conversion','EPSG','7731','EPSG','4403','EPSG','1219');
INSERT INTO "conversion" VALUES('EPSG','7732','Jharkhand NSF LCC','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',23.62652682,'EPSG','9102','EPSG','8822','Longitude of false origin',85.625,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',22.323,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',24.423,'EPSG','9110','EPSG','8826','Easting at false origin',1000000.0,'EPSG','9001','EPSG','8827','Northing at false origin',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10245','conversion','EPSG','7732','EPSG','4404','EPSG','1219');
INSERT INTO "conversion" VALUES('EPSG','7733','Madhya Pradesh NSF LCC','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',24.00529821,'EPSG','9102','EPSG','8822','Longitude of false origin',78.375,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',22.0,'EPSG','9102','EPSG','8824','Latitude of 2nd standard parallel',26.0,'EPSG','9102','EPSG','8826','Easting at false origin',1000000.0,'EPSG','9001','EPSG','8827','Northing at false origin',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10246','conversion','EPSG','7733','EPSG','4407','EPSG','1219');
INSERT INTO "conversion" VALUES('EPSG','7734','Maharashtra NSF LCC','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',18.88015774,'EPSG','9102','EPSG','8822','Longitude of false origin',76.75,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',16.373,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',21.073,'EPSG','9110','EPSG','8826','Easting at false origin',1000000.0,'EPSG','9001','EPSG','8827','Northing at false origin',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10247','conversion','EPSG','7734','EPSG','4408','EPSG','1219');
INSERT INTO "conversion" VALUES('EPSG','7735','Manipur NSF LCC','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',24.75060911,'EPSG','9102','EPSG','8822','Longitude of false origin',94.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',24.05,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',25.25,'EPSG','9110','EPSG','8826','Easting at false origin',1000000.0,'EPSG','9001','EPSG','8827','Northing at false origin',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10248','conversion','EPSG','7735','EPSG','4409','EPSG','1219');
INSERT INTO "conversion" VALUES('EPSG','7736','Meghalaya NSF LCC','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',25.62524747,'EPSG','9102','EPSG','8822','Longitude of false origin',91.375,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',25.123,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',26.023,'EPSG','9110','EPSG','8826','Easting at false origin',1000000.0,'EPSG','9001','EPSG','8827','Northing at false origin',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10249','conversion','EPSG','7736','EPSG','4410','EPSG','1219');
INSERT INTO "conversion" VALUES('EPSG','7737','Nagaland NSF LCC','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',26.12581974,'EPSG','9102','EPSG','8822','Longitude of false origin',94.375,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',25.223,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',26.523,'EPSG','9110','EPSG','8826','Easting at false origin',1000000.0,'EPSG','9001','EPSG','8827','Northing at false origin',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10250','conversion','EPSG','7737','EPSG','4412','EPSG','1219');
INSERT INTO "conversion" VALUES('EPSG','7738','Northeast India NSF LCC','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',25.63452135,'EPSG','9102','EPSG','8822','Longitude of false origin',93.5,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',23.023,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',28.123,'EPSG','9110','EPSG','8826','Easting at false origin',1000000.0,'EPSG','9001','EPSG','8827','Northing at false origin',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10251','conversion','EPSG','7738','EPSG','4392','EPSG','1219');
INSERT INTO "conversion" VALUES('EPSG','7739','Orissa NSF LCC','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',20.25305174,'EPSG','9102','EPSG','8822','Longitude of false origin',84.375,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',18.35,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',21.55,'EPSG','9110','EPSG','8826','Easting at false origin',1000000.0,'EPSG','9001','EPSG','8827','Northing at false origin',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10252','conversion','EPSG','7739','EPSG','4413','EPSG','1219');
INSERT INTO "conversion" VALUES('EPSG','7740','Punjab NSF LCC','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',31.00178226,'EPSG','9102','EPSG','8822','Longitude of false origin',75.375,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',30.0,'EPSG','9102','EPSG','8824','Latitude of 2nd standard parallel',32.0,'EPSG','9102','EPSG','8826','Easting at false origin',1000000.0,'EPSG','9001','EPSG','8827','Northing at false origin',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10253','conversion','EPSG','7740','EPSG','4414','EPSG','1219');
INSERT INTO "conversion" VALUES('EPSG','7741','Rajasthan NSF LCC','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',26.88505546,'EPSG','9102','EPSG','8822','Longitude of false origin',73.875,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',24.173,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',29.273,'EPSG','9110','EPSG','8826','Easting at false origin',1000000.0,'EPSG','9001','EPSG','8827','Northing at false origin',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10254','conversion','EPSG','7741','EPSG','4415','EPSG','1219');
INSERT INTO "conversion" VALUES('EPSG','7742','Uttar Pradesh NSF LCC','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',27.13270823,'EPSG','9102','EPSG','8822','Longitude of false origin',80.875,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',24.523,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',29.223,'EPSG','9110','EPSG','8826','Easting at false origin',1000000.0,'EPSG','9001','EPSG','8827','Northing at false origin',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10255','conversion','EPSG','7742','EPSG','4419','EPSG','1219');
INSERT INTO "conversion" VALUES('EPSG','7743','Uttaranchal NSF LCC','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',30.0017132,'EPSG','9102','EPSG','8822','Longitude of false origin',79.375,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',29.0,'EPSG','9102','EPSG','8824','Latitude of 2nd standard parallel',31.0,'EPSG','9102','EPSG','8826','Easting at false origin',1000000.0,'EPSG','9001','EPSG','8827','Northing at false origin',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10256','conversion','EPSG','7743','EPSG','4420','EPSG','1219');
INSERT INTO "conversion" VALUES('EPSG','7744','Andaman and Nicobar NSF TM','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',10.25,'EPSG','9102','EPSG','8802','Longitude of natural origin',93.25,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999428,'EPSG','9201','EPSG','8806','False easting',1000000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10257','conversion','EPSG','7744','EPSG','4423','EPSG','1219');
INSERT INTO "conversion" VALUES('EPSG','7745','Chhattisgarh NSF TM','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',21.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',82.25,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9998332,'EPSG','9201','EPSG','8806','False easting',1000000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10258','conversion','EPSG','7745','EPSG','4398','EPSG','1219');
INSERT INTO "conversion" VALUES('EPSG','7746','Goa NSF TM','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',15.375,'EPSG','9102','EPSG','8802','Longitude of natural origin',74.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999913,'EPSG','9201','EPSG','8806','False easting',1000000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10259','conversion','EPSG','7746','EPSG','4399','EPSG','1219');
INSERT INTO "conversion" VALUES('EPSG','7747','Karnataka NSF TM','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',15.125,'EPSG','9102','EPSG','8802','Longitude of natural origin',76.375,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9998012,'EPSG','9201','EPSG','8806','False easting',1000000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10260','conversion','EPSG','7747','EPSG','4405','EPSG','1219');
INSERT INTO "conversion" VALUES('EPSG','7748','Kerala NSF TM','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',10.5,'EPSG','9102','EPSG','8802','Longitude of natural origin',76.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999177,'EPSG','9201','EPSG','8806','False easting',1000000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10261','conversion','EPSG','7748','EPSG','4406','EPSG','1219');
INSERT INTO "conversion" VALUES('EPSG','7749','Lakshadweep NSF TM','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',10.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',73.125,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999536,'EPSG','9201','EPSG','8806','False easting',1000000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10262','conversion','EPSG','7749','EPSG','4424','EPSG','1219');
INSERT INTO "conversion" VALUES('EPSG','7750','Mizoram NSF TM','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',23.125,'EPSG','9102','EPSG','8802','Longitude of natural origin',92.75,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999821,'EPSG','9201','EPSG','8806','False easting',1000000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10263','conversion','EPSG','7750','EPSG','4411','EPSG','1219');
INSERT INTO "conversion" VALUES('EPSG','7751','Sikkim NSF TM','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',27.625,'EPSG','9102','EPSG','8802','Longitude of natural origin',88.5,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999926,'EPSG','9201','EPSG','8806','False easting',1000000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10264','conversion','EPSG','7751','EPSG','4416','EPSG','1219');
INSERT INTO "conversion" VALUES('EPSG','7752','Tamil Nadu NSF TM','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',10.875,'EPSG','9102','EPSG','8802','Longitude of natural origin',78.375,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9997942,'EPSG','9201','EPSG','8806','False easting',1000000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10265','conversion','EPSG','7752','EPSG','4417','EPSG','1219');
INSERT INTO "conversion" VALUES('EPSG','7753','Tripura NSF TM','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',23.75,'EPSG','9102','EPSG','8802','Longitude of natural origin',91.75,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999822,'EPSG','9201','EPSG','8806','False easting',1000000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10266','conversion','EPSG','7753','EPSG','4418','EPSG','1219');
INSERT INTO "conversion" VALUES('EPSG','7754','West Bengal NSF TM','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',24.375,'EPSG','9102','EPSG','8802','Longitude of natural origin',87.875,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9998584,'EPSG','9201','EPSG','8806','False easting',1000000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10267','conversion','EPSG','7754','EPSG','4421','EPSG','1219');
INSERT INTO "conversion" VALUES('EPSG','7802','Cadastral Coordinate System 2005','Also given with equivalent definition having parameter values latitude of false origin = 42°N and FN = 4651640.214m. Note that at latitude 42°40''04.35246"N scale is minimum but not unity.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',42.400435246,'EPSG','9110','EPSG','8822','Longitude of false origin',25.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',42.0,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',43.2,'EPSG','9110','EPSG','8826','Easting at false origin',500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',4725824.3591,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10270','conversion','EPSG','7802','EPSG','3224','EPSG','1061');
INSERT INTO "conversion" VALUES('EPSG','7812','Height <> Depth Conversion','This is a parameter-less conversion to reverse the positive direction of the axis of a vertical CRS. Source and target CRSs must both have the same unit of measure.','EPSG','1068','Height Depth Reversal',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10277','conversion','EPSG','7812','EPSG','1262','EPSG','1156');
INSERT INTO "conversion" VALUES('EPSG','7813','Vertical Axis Unit Conversion','Source and target CRSs both must have the same axis direction (both up or both down).','EPSG','1104','Change of Vertical Unit',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10278','conversion','EPSG','7813','EPSG','1262','EPSG','1101');
INSERT INTO "conversion" VALUES('EPSG','7818','CS63 zone X1','On the Krassowsky 1940 ellipsoid an alternative but equivalent definition uses latitude of natural origin = 0° and false northing = -9214.692m.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.05,'EPSG','9110','EPSG','8802','Longitude of natural origin',23.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',1300000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10281','conversion','EPSG','7818','EPSG','4435','EPSG','1208');
INSERT INTO "conversion" VALUES('EPSG','7819','CS63 zone X2','On the Krassowsky 1940 ellipsoid an alternative but equivalent definition uses latitude of natural origin = 0° and false northing = -9214.692m.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.05,'EPSG','9110','EPSG','8802','Longitude of natural origin',26.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',2300000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10282','conversion','EPSG','7819','EPSG','4429','EPSG','1208');
INSERT INTO "conversion" VALUES('EPSG','7820','CS63 zone X3','On the Krassowsky 1940 ellipsoid an alternative but equivalent definition uses latitude of natural origin = 0° and false northing = -9214.692m.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.05,'EPSG','9110','EPSG','8802','Longitude of natural origin',29.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',3300000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10283','conversion','EPSG','7820','EPSG','4430','EPSG','1208');
INSERT INTO "conversion" VALUES('EPSG','7821','CS63 zone X4','On the Krassowsky 1940 ellipsoid an alternative but equivalent definition uses latitude of natural origin = 0° and false northing = -9214.692m.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.05,'EPSG','9110','EPSG','8802','Longitude of natural origin',32.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',4300000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10284','conversion','EPSG','7821','EPSG','4431','EPSG','1208');
INSERT INTO "conversion" VALUES('EPSG','7822','CS63 zone X5','On the Krassowsky 1940 ellipsoid an alternative but equivalent definition uses latitude of natural origin = 0° and false northing = -9214.692m.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.05,'EPSG','9110','EPSG','8802','Longitude of natural origin',35.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',5300000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10285','conversion','EPSG','7822','EPSG','4432','EPSG','1208');
INSERT INTO "conversion" VALUES('EPSG','7823','CS63 zone X6','On the Krassowsky 1940 ellipsoid an alternative but equivalent definition uses latitude of natural origin = 0° and false northing = -9214.692m.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.05,'EPSG','9110','EPSG','8802','Longitude of natural origin',38.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',6300000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10286','conversion','EPSG','7823','EPSG','4433','EPSG','1208');
INSERT INTO "conversion" VALUES('EPSG','7824','CS63 zone X7','On the Krassowsky 1940 ellipsoid an alternative but equivalent definition uses latitude of natural origin = 0° and false northing = -9214.692m.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.05,'EPSG','9110','EPSG','8802','Longitude of natural origin',41.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',7300000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10287','conversion','EPSG','7824','EPSG','4434','EPSG','1208');
INSERT INTO "conversion" VALUES('EPSG','7875','St. Helena Local Grid 1971','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-15.58,'EPSG','9110','EPSG','8802','Longitude of natural origin',-5.43,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',300000.0,'EPSG','9001','EPSG','8807','False northing',2000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10309','conversion','EPSG','7875','EPSG','3183','EPSG','1153');
INSERT INTO "conversion" VALUES('EPSG','7876','St. Helena Local Grid (Tritan)','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-15.58,'EPSG','9110','EPSG','8802','Longitude of natural origin',-5.43,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',299483.737,'EPSG','9001','EPSG','8807','False northing',2000527.879,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10310','conversion','EPSG','7876','EPSG','3183','EPSG','1146');
INSERT INTO "conversion" VALUES('EPSG','7993','Albany Grid 2020','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',117.53,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000044,'EPSG','9201','EPSG','8806','False easting',50000.0,'EPSG','9001','EPSG','8807','False northing',4100000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10372','conversion','EPSG','7993','EPSG','4439','EPSG','1054');
INSERT INTO "conversion" VALUES('EPSG','7994','Barrow Island Grid 2020','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',115.15,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000022,'EPSG','9201','EPSG','8806','False easting',60000.0,'EPSG','9001','EPSG','8807','False northing',2700000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10373','conversion','EPSG','7994','EPSG','4438','EPSG','1054');
INSERT INTO "conversion" VALUES('EPSG','7995','Broome Grid 2020','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',122.2,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00000298,'EPSG','9201','EPSG','8806','False easting',50000.0,'EPSG','9001','EPSG','8807','False northing',2300000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10374','conversion','EPSG','7995','EPSG','4441','EPSG','1054');
INSERT INTO "conversion" VALUES('EPSG','7996','Busselton Coastal Grid 2020','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',115.26,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.99999592,'EPSG','9201','EPSG','8806','False easting',50000.0,'EPSG','9001','EPSG','8807','False northing',4000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10375','conversion','EPSG','7996','EPSG','4437','EPSG','1054');
INSERT INTO "conversion" VALUES('EPSG','7997','Carnarvon Grid 2020','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',113.4,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.99999796,'EPSG','9201','EPSG','8806','False easting',50000.0,'EPSG','9001','EPSG','8807','False northing',3050000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10376','conversion','EPSG','7997','EPSG','4442','EPSG','1054');
INSERT INTO "conversion" VALUES('EPSG','7998','Christmas Island Grid 2020','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',105.373,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00002514,'EPSG','9201','EPSG','8806','False easting',50000.0,'EPSG','9001','EPSG','8807','False northing',1400000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10377','conversion','EPSG','7998','EPSG','4169','EPSG','1054');
INSERT INTO "conversion" VALUES('EPSG','7999','Cocos Island Grid 2020','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',96.523,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.99999387,'EPSG','9201','EPSG','8806','False easting',50000.0,'EPSG','9001','EPSG','8807','False northing',1600000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10378','conversion','EPSG','7999','EPSG','1069','EPSG','1054');
INSERT INTO "conversion" VALUES('EPSG','8000','Collie Grid 2020','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',115.56,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000019,'EPSG','9201','EPSG','8806','False easting',40000.0,'EPSG','9001','EPSG','8807','False northing',4100000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10379','conversion','EPSG','8000','EPSG','4443','EPSG','1054');
INSERT INTO "conversion" VALUES('EPSG','8001','Esperance Grid 2020','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',121.53,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000055,'EPSG','9201','EPSG','8806','False easting',50000.0,'EPSG','9001','EPSG','8807','False northing',4050000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10380','conversion','EPSG','8001','EPSG','4445','EPSG','1054');
INSERT INTO "conversion" VALUES('EPSG','8002','Exmouth Grid 2020','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',114.04,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00000236,'EPSG','9201','EPSG','8806','False easting',50000.0,'EPSG','9001','EPSG','8807','False northing',2750000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10381','conversion','EPSG','8002','EPSG','4448','EPSG','1054');
INSERT INTO "conversion" VALUES('EPSG','8003','Geraldton Coastal Grid 2020','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',114.35,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00000628,'EPSG','9201','EPSG','8806','False easting',50000.0,'EPSG','9001','EPSG','8807','False northing',3450000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10382','conversion','EPSG','8003','EPSG','4449','EPSG','1054');
INSERT INTO "conversion" VALUES('EPSG','8004','Goldfields Grid 2020','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',121.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00004949,'EPSG','9201','EPSG','8806','False easting',60000.0,'EPSG','9001','EPSG','8807','False northing',3800000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10383','conversion','EPSG','8004','EPSG','4436','EPSG','1054');
INSERT INTO "conversion" VALUES('EPSG','8005','Jurien Coastal Grid 2020','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',114.59,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00000314,'EPSG','9201','EPSG','8806','False easting',50000.0,'EPSG','9001','EPSG','8807','False northing',3650000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10384','conversion','EPSG','8005','EPSG','4440','EPSG','1054');
INSERT INTO "conversion" VALUES('EPSG','8006','Kalbarri Grid 2020','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',114.1855,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000014,'EPSG','9201','EPSG','8806','False easting',55000.0,'EPSG','9001','EPSG','8807','False northing',3700000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10385','conversion','EPSG','8006','EPSG','4444','EPSG','1054');
INSERT INTO "conversion" VALUES('EPSG','8007','Karratha Grid 2020','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',116.56,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999989,'EPSG','9201','EPSG','8806','False easting',50000.0,'EPSG','9001','EPSG','8807','False northing',2550000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10386','conversion','EPSG','8007','EPSG','4451','EPSG','1054');
INSERT INTO "conversion" VALUES('EPSG','8008','Kununurra Grid 2020','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',128.45,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000165,'EPSG','9201','EPSG','8806','False easting',50000.0,'EPSG','9001','EPSG','8807','False northing',2100000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10387','conversion','EPSG','8008','EPSG','4452','EPSG','1054');
INSERT INTO "conversion" VALUES('EPSG','8009','Lancelin Coastal Grid 2020','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',115.22,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00000157,'EPSG','9201','EPSG','8806','False easting',50000.0,'EPSG','9001','EPSG','8807','False northing',3750000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10388','conversion','EPSG','8009','EPSG','4453','EPSG','1054');
INSERT INTO "conversion" VALUES('EPSG','8010','Margaret River Coastal Grid 2020','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',115.1,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000055,'EPSG','9201','EPSG','8806','False easting',50000.0,'EPSG','9001','EPSG','8807','False northing',4050000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10389','conversion','EPSG','8010','EPSG','4457','EPSG','1054');
INSERT INTO "conversion" VALUES('EPSG','8011','Perth Coastal Grid 2020','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',115.49,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.99999906,'EPSG','9201','EPSG','8806','False easting',50000.0,'EPSG','9001','EPSG','8807','False northing',3900000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10390','conversion','EPSG','8011','EPSG','4462','EPSG','1054');
INSERT INTO "conversion" VALUES('EPSG','8012','Port Hedland Grid 2020','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',118.36,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00000135,'EPSG','9201','EPSG','8806','False easting',50000.0,'EPSG','9001','EPSG','8807','False northing',2500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10391','conversion','EPSG','8012','EPSG','4466','EPSG','1054');
INSERT INTO "conversion" VALUES('EPSG','8033','TM Zone 20N (US survey foot)','US survey foot form of UTM zone 20N. Sometimes locally referred to as "UTM zone 20".','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-63.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',1640416.667,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10392','conversion','EPSG','8033','EPSG','4467','EPSG','1153');
INSERT INTO "conversion" VALUES('EPSG','8034','TM Zone 21N (US survey foot)','US survey foot form of UTM zone 21N. Sometimes locally referred to as "UTM zone 21".','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-57.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',1640416.667,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10393','conversion','EPSG','8034','EPSG','4468','EPSG','1153');
INSERT INTO "conversion" VALUES('EPSG','8040','Gusterberg Grid','Longitude is referenced to the Ferro meridian.','EPSG','9806','Cassini-Soldner','EPSG','8801','Latitude of natural origin',48.021847,'EPSG','9110','EPSG','8802','Longitude of natural origin',31.481505,'EPSG','9110','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10397','conversion','EPSG','8040','EPSG','4455','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','8041','St. Stephen Grid','Longitude is referenced to the Ferro meridian.','EPSG','9806','Cassini-Soldner','EPSG','8801','Latitude of natural origin',48.123154,'EPSG','9110','EPSG','8802','Longitude of natural origin',34.022732,'EPSG','9110','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10398','conversion','EPSG','8041','EPSG','4456','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','8061','Pima County zone 1 East (ft)','Grid unit is International feet (note: not US Survey feet).','EPSG','9815','Hotine Oblique Mercator (variant B)','EPSG','8811','Latitude of projection centre',32.15,'EPSG','9110','EPSG','8812','Longitude of projection centre',-111.24,'EPSG','9110','EPSG','8813','Azimuth at projection centre',45.0,'EPSG','9102','EPSG','8814','Angle from Rectified to Skew Grid',45.0,'EPSG','9102','EPSG','8815','Scale factor at projection centre',1.00011,'EPSG','9201','EPSG','8816','Easting at projection centre',160000.0,'EPSG','9002','EPSG','8817','Northing at projection centre',800000.0,'EPSG','9002',0);
INSERT INTO "usage" VALUES('EPSG','10408','conversion','EPSG','8061','EPSG','4472','EPSG','1207');
INSERT INTO "conversion" VALUES('EPSG','8062','Pima County zone 2 Central (ft)','Grid unit is International feet (note: not US Survey feet).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',31.15,'EPSG','9110','EPSG','8802','Longitude of natural origin',-112.1,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00009,'EPSG','9201','EPSG','8806','False easting',1800000.0,'EPSG','9002','EPSG','8807','False northing',1000000.0,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10409','conversion','EPSG','8062','EPSG','4460','EPSG','1207');
INSERT INTO "conversion" VALUES('EPSG','8063','Pima County zone 3 West (ft)','Grid unit is International feet (note: not US Survey feet).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',31.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-113.1,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000055,'EPSG','9201','EPSG','8806','False easting',600000.0,'EPSG','9002','EPSG','8807','False northing',0.0,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10410','conversion','EPSG','8063','EPSG','4450','EPSG','1207');
INSERT INTO "conversion" VALUES('EPSG','8064','Pima County zone 4 Mt. Lemmon (ft)','Grid unit is International feet (note: not US Survey feet).','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',30.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-110.45,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9998,'EPSG','9201','EPSG','8806','False easting',30000.0,'EPSG','9002','EPSG','8807','False northing',-620000.0,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10411','conversion','EPSG','8064','EPSG','4473','EPSG','1207');
INSERT INTO "conversion" VALUES('EPSG','8080','MTM Nova Scotia 2010 zone 4','Introduced for use with NAD83(CSRS)v6.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-61.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',24500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10423','conversion','EPSG','8080','EPSG','1534','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','8081','MTM Nova Scotia 2010 zone 5','Introduced for use with NAD83(CSRS)v6.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-64.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',25500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10424','conversion','EPSG','8081','EPSG','1535','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','8087','Iceland Lambert 2016','Replaces Iceland Lambert 2004 (code 5326). Used only with ISN2016 geogCRS.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',65.0,'EPSG','9110','EPSG','8822','Longitude of false origin',-19.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',64.15,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',65.45,'EPSG','9110','EPSG','8826','Easting at false origin',2700000.0,'EPSG','9001','EPSG','8827','Northing at false origin',300000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10425','conversion','EPSG','8087','EPSG','1120','EPSG','1241');
INSERT INTO "conversion" VALUES('EPSG','8273','Oregon Burns-Harper zone (meter)','See code 8274 for equivalent non-metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',43.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-117.4,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00014,'EPSG','9201','EPSG','8806','False easting',90000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10466','conversion','EPSG','8273','EPSG','4459','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','8274','Oregon Burns-Harper zone (international foot)','See code 8273 for equivalent metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',43.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-117.4,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00014,'EPSG','9201','EPSG','8806','False easting',295275.5906,'EPSG','9002','EPSG','8807','False northing',0.0,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10467','conversion','EPSG','8274','EPSG','4459','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','8275','Oregon Canyon City-Burns zone (meter)','See code 8276 for equivalent non-metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',43.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-119.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00022,'EPSG','9201','EPSG','8806','False easting',20000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10468','conversion','EPSG','8275','EPSG','4465','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','8276','Oregon Canyon City-Burns zone (international foot)','See code 8275 for equivalent metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',43.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-119.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00022,'EPSG','9201','EPSG','8806','False easting',65616.7979,'EPSG','9002','EPSG','8807','False northing',0.0,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10469','conversion','EPSG','8276','EPSG','4465','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','8277','Oregon Coast Range North zone (meter)','See code 8278 for equivalent non-metric definition.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',45.35,'EPSG','9110','EPSG','8802','Longitude of natural origin',-123.25,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000045,'EPSG','9201','EPSG','8806','False easting',30000.0,'EPSG','9001','EPSG','8807','False northing',20000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10470','conversion','EPSG','8277','EPSG','4471','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','8278','Oregon Coast Range North zone (international foot)','See code 8277 for equivalent metric definition.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',45.35,'EPSG','9110','EPSG','8802','Longitude of natural origin',-123.25,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000045,'EPSG','9201','EPSG','8806','False easting',98425.1969,'EPSG','9002','EPSG','8807','False northing',65616.7979,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10471','conversion','EPSG','8278','EPSG','4471','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','8279','Oregon Dayville-Prairie City zone (meter)','See code 8280 for equivalent non-metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',44.15,'EPSG','9110','EPSG','8802','Longitude of natural origin',-119.38,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00012,'EPSG','9201','EPSG','8806','False easting',20000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10472','conversion','EPSG','8279','EPSG','4474','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','8280','Oregon Dayville-Prairie City zone (international foot)','See code 8279 for equivalent metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',44.15,'EPSG','9110','EPSG','8802','Longitude of natural origin',-119.38,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00012,'EPSG','9201','EPSG','8806','False easting',65616.7979,'EPSG','9002','EPSG','8807','False northing',0.0,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10473','conversion','EPSG','8280','EPSG','4474','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','8281','Oregon Denio-Burns zone (meter)','See code 8282 for equivalent non-metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',41.45,'EPSG','9110','EPSG','8802','Longitude of natural origin',-118.25,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00019,'EPSG','9201','EPSG','8806','False easting',80000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10474','conversion','EPSG','8281','EPSG','4475','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','8282','Oregon Denio-Burns zone (international foot)','See code 8281 for equivalent metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',41.45,'EPSG','9110','EPSG','8802','Longitude of natural origin',-118.25,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00019,'EPSG','9201','EPSG','8806','False easting',262467.1916,'EPSG','9002','EPSG','8807','False northing',0.0,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10475','conversion','EPSG','8282','EPSG','4475','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','8283','Oregon Halfway zone (meter)','See code 8284 for equivalent non-metric definition.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',45.15,'EPSG','9110','EPSG','8802','Longitude of natural origin',-117.15,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000085,'EPSG','9201','EPSG','8806','False easting',40000.0,'EPSG','9001','EPSG','8807','False northing',70000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10476','conversion','EPSG','8283','EPSG','4476','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','8284','Oregon Halfway zone (international foot)','See code 8283 for equivalent metric definition.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',45.15,'EPSG','9110','EPSG','8802','Longitude of natural origin',-117.15,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000085,'EPSG','9201','EPSG','8806','False easting',131233.5958,'EPSG','9002','EPSG','8807','False northing',229658.7927,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10477','conversion','EPSG','8284','EPSG','4476','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','8285','Oregon Medford-Diamond Lake zone (meter)','See code 8286 for equivalent non-metric definition.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',42.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-122.15,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00004,'EPSG','9201','EPSG','8806','False easting',60000.0,'EPSG','9001','EPSG','8807','False northing',-60000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10478','conversion','EPSG','8285','EPSG','4477','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','8286','Oregon Medford-Diamond Lake zone (international foot)','See code 8285 for equivalent metric definition.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',42.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-122.15,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00004,'EPSG','9201','EPSG','8806','False easting',196850.3937,'EPSG','9002','EPSG','8807','False northing',-196850.3937,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10479','conversion','EPSG','8286','EPSG','4477','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','8287','Oregon Mitchell zone (meter)','See code 8288 for equivalent non-metric definition.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',47.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-120.15,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.99927,'EPSG','9201','EPSG','8806','False easting',30000.0,'EPSG','9001','EPSG','8807','False northing',290000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10480','conversion','EPSG','8287','EPSG','4478','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','8288','Oregon Mitchell zone (international foot)','See code 8287 for equivalent metric definition.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',47.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-120.15,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.99927,'EPSG','9201','EPSG','8806','False easting',98425.1969,'EPSG','9002','EPSG','8807','False northing',951443.5696,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10481','conversion','EPSG','8288','EPSG','4478','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','8289','Oregon North Central zone (meter)','See code 8290 for equivalent non-metric definition.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',46.1,'EPSG','9110','EPSG','8802','Longitude of natural origin',-120.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',100000.0,'EPSG','9001','EPSG','8807','False northing',140000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10482','conversion','EPSG','8289','EPSG','4479','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','8290','Oregon North Central zone (international foot)','See code 8289 for equivalent metric definition.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',46.1,'EPSG','9110','EPSG','8802','Longitude of natural origin',-120.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',328083.9895,'EPSG','9002','EPSG','8807','False northing',459317.5853,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10483','conversion','EPSG','8290','EPSG','4479','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','8291','Oregon Ochoco Summit zone (meter)','See code 8292 for equivalent non-metric definition.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',43.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-120.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00006,'EPSG','9201','EPSG','8806','False easting',40000.0,'EPSG','9001','EPSG','8807','False northing',-80000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10484','conversion','EPSG','8291','EPSG','4481','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','8292','Oregon Ochoco Summit zone (international foot)','See code 8291 for equivalent metric definition.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',43.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-120.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00006,'EPSG','9201','EPSG','8806','False easting',131233.5958,'EPSG','9002','EPSG','8807','False northing',-262467.1916,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10485','conversion','EPSG','8292','EPSG','4481','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','8293','Oregon Owyhee zone (meter)','See code 8294 for equivalent non-metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',41.45,'EPSG','9110','EPSG','8802','Longitude of natural origin',-117.35,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00018,'EPSG','9201','EPSG','8806','False easting',70000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10486','conversion','EPSG','8293','EPSG','4482','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','8294','Oregon Owyhee zone (international foot)','See code 8293 for equivalent metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',41.45,'EPSG','9110','EPSG','8802','Longitude of natural origin',-117.35,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00018,'EPSG','9201','EPSG','8806','False easting',229658.7927,'EPSG','9002','EPSG','8807','False northing',0.0,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10487','conversion','EPSG','8294','EPSG','4482','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','8295','Oregon Pilot Rock-Ukiah zone (meter)','See code 8296 for equivalent non-metric definition.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',46.1,'EPSG','9110','EPSG','8802','Longitude of natural origin',-119.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000025,'EPSG','9201','EPSG','8806','False easting',50000.0,'EPSG','9001','EPSG','8807','False northing',130000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10488','conversion','EPSG','8295','EPSG','4483','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','8296','Oregon Pilot Rock-Ukiah zone (international foot)','See code 8295 for equivalent metric definition.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',46.1,'EPSG','9110','EPSG','8802','Longitude of natural origin',-119.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000025,'EPSG','9201','EPSG','8806','False easting',164041.9948,'EPSG','9002','EPSG','8807','False northing',426509.1864,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10489','conversion','EPSG','8296','EPSG','4483','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','8297','Oregon Prairie City-Brogan zone (meter)','See code 8298 for equivalent non-metric definition.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',44.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-118.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00017,'EPSG','9201','EPSG','8806','False easting',60000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10490','conversion','EPSG','8297','EPSG','4484','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','8298','Oregon Prairie City-Brogan zone (international foot)','See code 8297 for equivalent metric definition.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',44.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-118.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00017,'EPSG','9201','EPSG','8806','False easting',196850.3937,'EPSG','9002','EPSG','8807','False northing',0.0,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10491','conversion','EPSG','8298','EPSG','4484','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','8299','Oregon Riley-Lakeview zone (meter)','See code 8300 for equivalent non-metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',41.45,'EPSG','9110','EPSG','8802','Longitude of natural origin',-120.2,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000215,'EPSG','9201','EPSG','8806','False easting',70000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10492','conversion','EPSG','8299','EPSG','4458','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','8300','Oregon Riley-Lakeview zone (international foot)','See code 8299 for equivalent metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',41.45,'EPSG','9110','EPSG','8802','Longitude of natural origin',-120.2,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000215,'EPSG','9201','EPSG','8806','False easting',229658.7927,'EPSG','9002','EPSG','8807','False northing',0.0,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10493','conversion','EPSG','8300','EPSG','4458','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','8301','Oregon Siskiyou Pass zone (meter)','See code 8302 for equivalent non-metric definition.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',42.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-122.35,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00015,'EPSG','9201','EPSG','8806','False easting',10000.0,'EPSG','9001','EPSG','8807','False northing',60000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10494','conversion','EPSG','8301','EPSG','4463','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','8302','Oregon Siskiyou Pass zone (international foot)','See code 8301 for equivalent metric definition.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',42.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-122.35,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00015,'EPSG','9201','EPSG','8806','False easting',32808.399,'EPSG','9002','EPSG','8807','False northing',196850.3937,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10495','conversion','EPSG','8302','EPSG','4463','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','8303','Oregon Ukiah-Fox zone (meter)','See code 8304 for equivalent non-metric definition.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',45.15,'EPSG','9110','EPSG','8802','Longitude of natural origin',-119.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00014,'EPSG','9201','EPSG','8806','False easting',30000.0,'EPSG','9001','EPSG','8807','False northing',90000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10496','conversion','EPSG','8303','EPSG','4470','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','8304','Oregon Ukiah-Fox zone (international foot)','See code 8303 for equivalent metric definition.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',45.15,'EPSG','9110','EPSG','8802','Longitude of natural origin',-119.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00014,'EPSG','9201','EPSG','8806','False easting',98425.1969,'EPSG','9002','EPSG','8807','False northing',295275.5906,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10497','conversion','EPSG','8304','EPSG','4470','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','8305','Oregon Wallowa zone (meter)','See code 8306 for equivalent non-metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',45.15,'EPSG','9110','EPSG','8802','Longitude of natural origin',-117.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000195,'EPSG','9201','EPSG','8806','False easting',60000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10498','conversion','EPSG','8305','EPSG','4480','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','8306','Oregon Wallowa zone (international foot)','See code 8305 for equivalent metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',45.15,'EPSG','9110','EPSG','8802','Longitude of natural origin',-117.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000195,'EPSG','9201','EPSG','8806','False easting',196850.3937,'EPSG','9002','EPSG','8807','False northing',0.0,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10499','conversion','EPSG','8306','EPSG','4480','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','8307','Oregon Warner Highway zone (meter)','See code 8308 for equivalent non-metric definition.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',42.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-120.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000245,'EPSG','9201','EPSG','8806','False easting',40000.0,'EPSG','9001','EPSG','8807','False northing',60000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10500','conversion','EPSG','8307','EPSG','4486','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','8308','Oregon Warner Highway zone (international foot)','See code 8307 for equivalent metric definition.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',42.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-120.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000245,'EPSG','9201','EPSG','8806','False easting',131233.5958,'EPSG','9002','EPSG','8807','False northing',196850.3937,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10501','conversion','EPSG','8308','EPSG','4486','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','8309','Oregon Willamette Pass zone (meter)','See code 8310 for equivalent non-metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',43.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-122.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000223,'EPSG','9201','EPSG','8806','False easting',20000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10502','conversion','EPSG','8309','EPSG','4488','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','8310','Oregon Willamette Pass zone (international foot)','See code 8309 for equivalent metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',43.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-122.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000223,'EPSG','9201','EPSG','8806','False easting',65616.7979,'EPSG','9002','EPSG','8807','False northing',0.0,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10503','conversion','EPSG','8310','EPSG','4488','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','8373','NCRS Las Vegas zone (m)','See proj code 8374 for equivalent non-metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',36.15,'EPSG','9110','EPSG','8802','Longitude of natural origin',-114.58,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0001,'EPSG','9201','EPSG','8806','False easting',100000.0,'EPSG','9001','EPSG','8807','False northing',200000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10519','conversion','EPSG','8373','EPSG','4485','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','8374','NCRS Las Vegas zone (ftUS)','See proj code 8373 for authoritative metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',36.15,'EPSG','9110','EPSG','8802','Longitude of natural origin',-114.58,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0001,'EPSG','9201','EPSG','8806','False easting',328083.3333,'EPSG','9003','EPSG','8807','False northing',656166.6667,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10520','conversion','EPSG','8374','EPSG','4485','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','8375','NCRS Las Vegas high elevation zone (m)','See proj code 8375 for authoritative metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',36.15,'EPSG','9110','EPSG','8802','Longitude of natural origin',-114.58,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000135,'EPSG','9201','EPSG','8806','False easting',300000.0,'EPSG','9001','EPSG','8807','False northing',400000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10521','conversion','EPSG','8375','EPSG','4487','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','8376','NCRS Las Vegas high elevation zone (ftUS)','See proj code 8375 for authoritative metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',36.15,'EPSG','9110','EPSG','8802','Longitude of natural origin',-114.58,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000135,'EPSG','9201','EPSG','8806','False easting',984250.0,'EPSG','9003','EPSG','8807','False northing',1312333.3333,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10522','conversion','EPSG','8376','EPSG','4487','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','8389','WEIPA94','Grid convergence uses opposite sign convention to UTM. The projection has reduced scale distortion compared to Map Grid of Australia zone 54 (code 17354).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',141.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.999929,'EPSG','9201','EPSG','8806','False easting',300000.0,'EPSG','9001','EPSG','8807','False northing',2000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10525','conversion','EPSG','8389','EPSG','4491','EPSG','1249');
INSERT INTO "conversion" VALUES('EPSG','8432','Macau Grid','Grid origin is Base W triangulation station.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',22.124463,'EPSG','9110','EPSG','8802','Longitude of natural origin',113.321129,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',20000.0,'EPSG','9001','EPSG','8807','False northing',20000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10555','conversion','EPSG','8432','EPSG','1147','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','8440','Laborde Grid (Greenwich)','See Laborde Grid (projection code 19861) for original definition in grads with respect to the Paris meridian. This is equivalent.','EPSG','9813','Laborde Oblique Mercator','EPSG','8811','Latitude of projection centre',-18.54,'EPSG','9110','EPSG','8812','Longitude of projection centre',46.2614025,'EPSG','9110','EPSG','8813','Azimuth at projection centre',18.54,'EPSG','9110','EPSG','8815','Scale factor at projection centre',0.9995,'EPSG','9201','EPSG','8806','False easting',400000.0,'EPSG','9001','EPSG','8807','False northing',800000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10561','conversion','EPSG','8440','EPSG','1149','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','8458','Kansas regional zone 1 Goodland (ftUS)','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',37.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-101.36,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000156,'EPSG','9201','EPSG','8806','False easting',1500000.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10575','conversion','EPSG','8458','EPSG','4495','EPSG','1196');
INSERT INTO "conversion" VALUES('EPSG','8459','Kansas regional zone 2 Colby (ftUS)','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',37.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-100.57,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000134,'EPSG','9201','EPSG','8806','False easting',2500000.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10576','conversion','EPSG','8459','EPSG','4496','EPSG','1196');
INSERT INTO "conversion" VALUES('EPSG','8490','Kansas regional zone 3 Oberlin (ftUS)','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',37.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-100.21,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000116,'EPSG','9201','EPSG','8806','False easting',3500000.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10607','conversion','EPSG','8490','EPSG','4497','EPSG','1196');
INSERT INTO "conversion" VALUES('EPSG','8491','Kansas regional zone 4 Hays (ftUS)','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',37.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-99.27,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000082,'EPSG','9201','EPSG','8806','False easting',4500000.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10608','conversion','EPSG','8491','EPSG','4494','EPSG','1196');
INSERT INTO "conversion" VALUES('EPSG','8492','Kansas regional zone 5 Great Bend (ftUS)','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',37.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-98.4,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000078,'EPSG','9201','EPSG','8806','False easting',5500000.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10609','conversion','EPSG','8492','EPSG','4498','EPSG','1196');
INSERT INTO "conversion" VALUES('EPSG','8493','Kansas regional zone 6 Beliot (ftUS)','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',37.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-98.09,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000068,'EPSG','9201','EPSG','8806','False easting',6500000.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10610','conversion','EPSG','8493','EPSG','4499','EPSG','1196');
INSERT INTO "conversion" VALUES('EPSG','8494','Kansas regional zone 7 Salina (ftUS)','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',37.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-97.2,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000049,'EPSG','9201','EPSG','8806','False easting',7500000.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10611','conversion','EPSG','8494','EPSG','4500','EPSG','1196');
INSERT INTO "conversion" VALUES('EPSG','8495','Kansas regional zone 8 Manhattan (ftUS)','','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',39.1,'EPSG','9110','EPSG','8802','Longitude of natural origin',-96.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000044,'EPSG','9201','EPSG','8806','False easting',8500000.0,'EPSG','9003','EPSG','8807','False northing',600000.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10612','conversion','EPSG','8495','EPSG','4501','EPSG','1196');
INSERT INTO "conversion" VALUES('EPSG','8498','Kansas regional zone 9 Emporia (ftUS)','','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',38.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-96.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00005,'EPSG','9201','EPSG','8806','False easting',9500000.0,'EPSG','9003','EPSG','8807','False northing',300000.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10615','conversion','EPSG','8498','EPSG','4502','EPSG','1196');
INSERT INTO "conversion" VALUES('EPSG','8499','Kansas regional zone 10 Atchison (ftUS)','','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',39.38,'EPSG','9110','EPSG','8802','Longitude of natural origin',-95.45,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00004,'EPSG','9201','EPSG','8806','False easting',10500000.0,'EPSG','9003','EPSG','8807','False northing',700000.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10616','conversion','EPSG','8499','EPSG','4503','EPSG','1196');
INSERT INTO "conversion" VALUES('EPSG','8500','Kansas regional zone 11 Kansas City (ftUS)','','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',39.06,'EPSG','9110','EPSG','8802','Longitude of natural origin',-95.15,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000033,'EPSG','9201','EPSG','8806','False easting',11500000.0,'EPSG','9003','EPSG','8807','False northing',600000.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10617','conversion','EPSG','8500','EPSG','4504','EPSG','1196');
INSERT INTO "conversion" VALUES('EPSG','8501','Kansas regional zone 12 Ulysses (ftUS)','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',36.45,'EPSG','9110','EPSG','8802','Longitude of natural origin',-101.25,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00014,'EPSG','9201','EPSG','8806','False easting',12500000.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10618','conversion','EPSG','8501','EPSG','4505','EPSG','1196');
INSERT INTO "conversion" VALUES('EPSG','8502','Kansas regional zone 13 Garden City (ftUS)','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',36.45,'EPSG','9110','EPSG','8802','Longitude of natural origin',-100.24,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000109,'EPSG','9201','EPSG','8806','False easting',13500000.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10619','conversion','EPSG','8502','EPSG','4506','EPSG','1196');
INSERT INTO "conversion" VALUES('EPSG','8503','Kansas regional zone 14 Dodge City (ftUS)','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',36.45,'EPSG','9110','EPSG','8802','Longitude of natural origin',-99.4,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000097,'EPSG','9201','EPSG','8806','False easting',14500000.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10620','conversion','EPSG','8503','EPSG','4507','EPSG','1196');
INSERT INTO "conversion" VALUES('EPSG','8504','Kansas regional zone 15 Larned (ftUS)','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',36.45,'EPSG','9110','EPSG','8802','Longitude of natural origin',-99.12,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000087,'EPSG','9201','EPSG','8806','False easting',15500000.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10621','conversion','EPSG','8504','EPSG','4508','EPSG','1196');
INSERT INTO "conversion" VALUES('EPSG','8505','Kansas regional zone 16 Pratt (ftUS)','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',36.45,'EPSG','9110','EPSG','8802','Longitude of natural origin',-98.33,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000069,'EPSG','9201','EPSG','8806','False easting',16500000.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10622','conversion','EPSG','8505','EPSG','4509','EPSG','1196');
INSERT INTO "conversion" VALUES('EPSG','8506','Kansas regional zone 17 Wichita (ftUS)','','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',37.46,'EPSG','9110','EPSG','8802','Longitude of natural origin',-97.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000059,'EPSG','9201','EPSG','8806','False easting',17500000.0,'EPSG','9003','EPSG','8807','False northing',400000.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10623','conversion','EPSG','8506','EPSG','4510','EPSG','1196');
INSERT INTO "conversion" VALUES('EPSG','8507','Kansas regional zone 18 Arkansas City (ftUS)','','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',37.11,'EPSG','9110','EPSG','8802','Longitude of natural origin',-97.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000055,'EPSG','9201','EPSG','8806','False easting',18500000.0,'EPSG','9003','EPSG','8807','False northing',200000.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10624','conversion','EPSG','8507','EPSG','4511','EPSG','1196');
INSERT INTO "conversion" VALUES('EPSG','8515','Kansas regional zone 19 Coffeyville (ftUS)','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',36.45,'EPSG','9110','EPSG','8802','Longitude of natural origin',-95.58,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000034,'EPSG','9201','EPSG','8806','False easting',19500000.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10632','conversion','EPSG','8515','EPSG','4512','EPSG','1196');
INSERT INTO "conversion" VALUES('EPSG','8516','Kansas regional zone 20 Pittsburg (ftUS)','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',36.45,'EPSG','9110','EPSG','8802','Longitude of natural origin',-95.05,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000031,'EPSG','9201','EPSG','8806','False easting',20500000.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10633','conversion','EPSG','8516','EPSG','4513','EPSG','1196');
INSERT INTO "conversion" VALUES('EPSG','8825','Idaho Transverse Mercator','Replaces IDTM27.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',42.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-114.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',2500000.0,'EPSG','9001','EPSG','8807','False northing',1200000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10778','conversion','EPSG','8825','EPSG','1381','EPSG','1135');
INSERT INTO "conversion" VALUES('EPSG','8854','Equal Earth Greenwich','','EPSG','1078','Equal Earth','EPSG','8802','Longitude of natural origin',0.0,'EPSG','9102','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10800','conversion','EPSG','8854','EPSG','1262','EPSG','1225');
INSERT INTO "conversion" VALUES('EPSG','8855','Equal Earth Americas','','EPSG','1078','Equal Earth','EPSG','8802','Longitude of natural origin',-90.0,'EPSG','9102','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10801','conversion','EPSG','8855','EPSG','4520','EPSG','1223');
INSERT INTO "conversion" VALUES('EPSG','8856','Equal Earth Asia-Pacific','','EPSG','1078','Equal Earth','EPSG','8802','Longitude of natural origin',150.0,'EPSG','9102','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10802','conversion','EPSG','8856','EPSG','4523','EPSG','1224');
INSERT INTO "conversion" VALUES('EPSG','9058','Vietnam TM-3 103-00','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',103.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10889','conversion','EPSG','9058','EPSG','4541','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','9190','NIWA Albers','See NZCS2000 (projection code 17964) and Mercator 41 (projection code 19843) for spatial referencing and conformal mapping.','EPSG','9822','Albers Equal Area','EPSG','8821','Latitude of false origin',-40.0,'EPSG','9102','EPSG','8822','Longitude of false origin',175.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-30.0,'EPSG','9102','EPSG','8824','Latitude of 2nd standard parallel',-50.0,'EPSG','9102','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10982','conversion','EPSG','9190','EPSG','3508','EPSG','1247');
INSERT INTO "conversion" VALUES('EPSG','9192','Vietnam TM-3 104-00','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',104.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10983','conversion','EPSG','9192','EPSG','4538','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','9193','Vietnam TM-3 104-30','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',104.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10984','conversion','EPSG','9193','EPSG','4545','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','9194','Vietnam TM-3 104-45','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',104.45,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10985','conversion','EPSG','9194','EPSG','4546','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','9195','Vietnam TM-3 105-30','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',105.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10986','conversion','EPSG','9195','EPSG','4548','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','9196','Vietnam TM-3 105-45','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',105.45,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10987','conversion','EPSG','9196','EPSG','4549','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','9197','Vietnam TM-3 106-00','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',106.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10988','conversion','EPSG','9197','EPSG','4550','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','9198','Vietnam TM-3 106-15','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',106.15,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10989','conversion','EPSG','9198','EPSG','4552','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','9199','Vietnam TM-3 106-30','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',106.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10990','conversion','EPSG','9199','EPSG','4553','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','9200','Vietnam TM-3 107-00','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',107.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10991','conversion','EPSG','9200','EPSG','4554','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','9201','Vietnam TM-3 107-15','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',107.15,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10992','conversion','EPSG','9201','EPSG','4556','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','9202','Vietnam TM-3 107-30','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',107.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10993','conversion','EPSG','9202','EPSG','4557','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','9203','Vietnam TM-3 108-15','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',108.15,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10994','conversion','EPSG','9203','EPSG','4559','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','9204','Vietnam TM-3 108-30','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',108.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10995','conversion','EPSG','9204','EPSG','4560','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','9219','South Africa Basic Survey Unit Albers 25E','Came into use from 2019-01-01. Use is extended northwards into neighbouring countries to include the river catchment areas of the Orange/Gariep and Limpopo rivers.','EPSG','9822','Albers Equal Area','EPSG','8821','Latitude of false origin',-30.0,'EPSG','9102','EPSG','8822','Longitude of false origin',25.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-22.0,'EPSG','9102','EPSG','8824','Latitude of 2nd standard parallel',-38.0,'EPSG','9102','EPSG','8826','Easting at false origin',1400000.0,'EPSG','9001','EPSG','8827','Northing at false origin',1300000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10996','conversion','EPSG','9219','EPSG','4567','EPSG','1106');
INSERT INTO "conversion" VALUES('EPSG','9220','South Africa Basic Survey Unit Albers 44E','Came into use from 2019-01-01.','EPSG','9822','Albers Equal Area','EPSG','8821','Latitude of false origin',-42.0,'EPSG','9102','EPSG','8822','Longitude of false origin',44.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-34.0,'EPSG','9102','EPSG','8824','Latitude of 2nd standard parallel',-50.0,'EPSG','9102','EPSG','8826','Easting at false origin',1200000.0,'EPSG','9001','EPSG','8827','Northing at false origin',1300000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','10997','conversion','EPSG','9220','EPSG','4568','EPSG','1106');
INSERT INTO "conversion" VALUES('EPSG','9268','Austria West','Greenwich equivalent of Austria West Zone (EPSG code 18041).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',10.2,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13898','conversion','EPSG','9268','EPSG','1706','EPSG','1258');
INSERT INTO "conversion" VALUES('EPSG','9269','Austria Central','Greenwich equivalent of Austria Central Zone (EPSG code 18042).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',13.2,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13899','conversion','EPSG','9269','EPSG','1707','EPSG','1258');
INSERT INTO "conversion" VALUES('EPSG','9270','Austria East','Greenwich equivalent of Austria East Zone (EPSG code 18043).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',16.2,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13900','conversion','EPSG','9270','EPSG','1708','EPSG','1258');
INSERT INTO "conversion" VALUES('EPSG','9301','HS2-TM','In conjunction with transformation HS2TN15 (code 9302) emulates the zero-distortion Snake projection HS2P1+14. Emulation derived at 350,000 locations, RMS position difference is 0.15 mm and max position difference is 0.6 mm; this is considered errorless.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',52.3,'EPSG','9102','EPSG','8802','Longitude of natural origin',-1.5,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',198873.0046,'EPSG','9001','EPSG','8807','False northing',375064.3871,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','14059','conversion','EPSG','9301','EPSG','4582','EPSG','1260');
INSERT INTO "conversion" VALUES('EPSG','9353','IBCSO Polar Stereographic','Used for the International Bathymetric Chart of the Southern Ocean.','EPSG','9829','Polar Stereographic (variant B)','EPSG','8832','Latitude of standard parallel',-65.0,'EPSG','9102','EPSG','8833','Longitude of origin',0.0,'EPSG','9102','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13982','conversion','EPSG','9353','EPSG','4586','EPSG','1198');
INSERT INTO "conversion" VALUES('EPSG','9366','TPEN11-TM','In conjunction with transformation ETRS89 to TPEN11-IRF (1) (code 9365), emulates the TPEN11 Snake and TPEN11ext Snake projections.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',53.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-2.15,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',203252.175,'EPSG','9001','EPSG','8807','False northing',407512.765,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13985','conversion','EPSG','9366','EPSG','4583','EPSG','1141');
INSERT INTO "conversion" VALUES('EPSG','9370','MML07-TM','In conjunction with transformation ETRS89 to MML07-IRF (1) (code 9369), emulates the MML07 Snake projection.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',52.27,'EPSG','9110','EPSG','8802','Longitude of natural origin',-0.51,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',49350.157,'EPSG','9001','EPSG','8807','False northing',108398.212,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','14003','conversion','EPSG','9370','EPSG','4588','EPSG','1141');
INSERT INTO "conversion" VALUES('EPSG','9376','Colombia Transverse Mercator','Adopted by resolution IGAC 471/May 14, 2020. Supports the Spatial Data Infrastructure for Land Administration (IDE-AT) and its GIS applications. Used to fulfill all the requirements of the activities required for the Multipurpose Cadastre.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',4.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-73.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9992,'EPSG','9201','EPSG','8806','False easting',5000000.0,'EPSG','9001','EPSG','8807','False northing',2000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','14004','conversion','EPSG','9376','EPSG','1070','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','9385','AbInvA96_2020-TM','In conjunction with transformation ETRS89 to AbInvA96_2020-IRF (1) (code 9386), emulates the AbInvA96_2020 Snake projection.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',57.24,'EPSG','9110','EPSG','8802','Longitude of natural origin',-3.12,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',155828.702,'EPSG','9001','EPSG','8807','False northing',115225.707,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','14062','conversion','EPSG','9385','EPSG','4589','EPSG','1196');
INSERT INTO "conversion" VALUES('EPSG','9455','GBK19-TM','In conjunction with transformation ETRS89 to GBK19-IRF (1) (code 9454), emulates the GBK19 Snake projection.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',55.45,'EPSG','9110','EPSG','8802','Longitude of natural origin',-4.21,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',93720.394,'EPSG','9001','EPSG','8807','False northing',113870.493,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','14130','conversion','EPSG','9455','EPSG','4607','EPSG','1141');
INSERT INTO "conversion" VALUES('EPSG','9497','Gauss-Kruger CABA 2019','Projection created in 2017 for the purpose of modernizing the city''s cadastre and linking it to modern reference frames. Origin approximates the 1919 origin of the cross of the main tower of the San José de Flores church ("0 de Flores" plane grid).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-34.374536,'EPSG','9110','EPSG','8802','Longitude of natural origin',-58.274791,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',20000.0,'EPSG','9001','EPSG','8807','False northing',70000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','14242','conversion','EPSG','9497','EPSG','4610','EPSG','1056');
INSERT INTO "conversion" VALUES('EPSG','9548','Lyon Turin Ferroviaire 2004 (C)','May be emulated using LCC 1SP variant A with false northing of -39367.968 metres at natural origin of 44°22''45"N.','EPSG','1102','Lambert Conic Conformal (1SP variant B)','EPSG','8801','Latitude of natural origin',44.2245,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8821','Latitude of false origin',45.11,'EPSG','9110','EPSG','8822','Longitude of false origin',6.49,'EPSG','9110','EPSG','8826','Easting at false origin',150000.0,'EPSG','9001','EPSG','8827','Northing at false origin',50000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','14927','conversion','EPSG','9548','EPSG','4613','EPSG','1271');
INSERT INTO "conversion" VALUES('EPSG','9673','US Forest Service region 6 Albers','','EPSG','9822','Albers Equal Area','EPSG','8821','Latitude of false origin',34.0,'EPSG','9102','EPSG','8822','Longitude of false origin',-120.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',43.0,'EPSG','9102','EPSG','8824','Latitude of 2nd standard parallel',48.0,'EPSG','9102','EPSG','8826','Easting at false origin',600000.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','14786','conversion','EPSG','9673','EPSG','2381','EPSG','1165');
INSERT INTO "conversion" VALUES('EPSG','9677','Bangladesh Transverse Mercator','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',90.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',-2000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','14878','conversion','EPSG','9677','EPSG','3217','EPSG','1274');
INSERT INTO "conversion" VALUES('EPSG','9738','EOS21-TM','In conjunction with transformation ETRS89 to EOS-IRF (1) (code 9740), emulates the EOS21 Snake projection.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',56.21,'EPSG','9110','EPSG','8802','Longitude of natural origin',-2.45,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',74996.927,'EPSG','9001','EPSG','8807','False northing',133508.35,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15339','conversion','EPSG','9738','EPSG','4620','EPSG','1141');
INSERT INTO "conversion" VALUES('EPSG','9746','SPCS83 Alabama East zone (US survey foot)','This map projection is not formally recognised by the NGS because there is no State legislation for NAD83 defining the foot to be used. See code 10131 for official metric definition. ftUS required by AL DOT and used by other professional practitioners.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',30.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-85.5,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.99996,'EPSG','9201','EPSG','8806','False easting',656166.667,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15382','conversion','EPSG','9746','EPSG','2154','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','9747','SPCS83 Alabama West zone (US survey foot)','This map projection is not formally recognised by the NGS because there is no State legislation for NAD83 defining the foot to be used. See code 10132 for official metric definition. ftUS required by AL DOT and used by other professional practitioners.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',30.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-87.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999933333,'EPSG','9201','EPSG','8806','False easting',1968500.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15381','conversion','EPSG','9747','EPSG','2155','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','9760','ECML14_NB-TM','In conjunction with transformation ETRS89 to ECML14_NB-IRF (1) (code 9759), emulates the ECML14_NB Snake projection.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',55.03,'EPSG','9110','EPSG','8802','Longitude of natural origin',-1.33,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',112242.8512,'EPSG','9001','EPSG','8807','False northing',402313.7432,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','16497','conversion','EPSG','9760','EPSG','4621','EPSG','1141');
INSERT INTO "conversion" VALUES('EPSG','9765','EWR3-TM','In conjunction with transformation ETRS89 to EWR2-IRF (1) (code 9764), emulates the EWR2 Snake projection. In conjunction with transformation ETRS89 to EWR3-IRF (1) (code 10850), emulates the EWR3 Snake projection. In 2024 the EWR2 Grid (ETRS89 / EWR2 SnakeGrid) was extended eastwards as the EWR3 Grid (ETRS89 / EWR3 SnakeGrid). In the emulation of the Snake projection, the NTv2 file supporting the transformation is extended but this map projection is unchanged and applies to both EWR2 and EWR3.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',51.57,'EPSG','9110','EPSG','8802','Longitude of natural origin',-0.54,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',192519.9715,'EPSG','9001','EPSG','8807','False northing',146942.6806,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','16511','conversion','EPSG','9765','EPSG','4799','EPSG','1141');
INSERT INTO "conversion" VALUES('EPSG','9796','Local coordinate system of Kyiv','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',30.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',300000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','16909','conversion','EPSG','9796','EPSG','4650','EPSG','1207');
INSERT INTO "conversion" VALUES('EPSG','9797','Local coordinate system of Crimea region','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',34.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',300000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','16876','conversion','EPSG','9797','EPSG','4648','EPSG','1207');
INSERT INTO "conversion" VALUES('EPSG','9798','Local coordinate system of Vinnytsia region','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',28.4,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',300000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','16877','conversion','EPSG','9798','EPSG','4643','EPSG','1207');
INSERT INTO "conversion" VALUES('EPSG','9799','Local coordinate system of Volyn region','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',24.5,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',300000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','16878','conversion','EPSG','9799','EPSG','4644','EPSG','1207');
INSERT INTO "conversion" VALUES('EPSG','9800','Local coordinate system of Dnipropetrovsk region','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',35.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',300000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','16879','conversion','EPSG','9800','EPSG','4627','EPSG','1207');
INSERT INTO "conversion" VALUES('EPSG','9801','Local coordinate system of Donetsk region','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',37.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',300000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','16880','conversion','EPSG','9801','EPSG','4628','EPSG','1207');
INSERT INTO "conversion" VALUES('EPSG','9802','Local coordinate system of Zhytomyr region','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',28.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',300000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','16881','conversion','EPSG','9802','EPSG','4647','EPSG','1207');
INSERT INTO "conversion" VALUES('EPSG','9803','Local coordinate system of Zakarpattia region','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',23.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',300000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','16882','conversion','EPSG','9803','EPSG','4645','EPSG','1207');
INSERT INTO "conversion" VALUES('EPSG','9804','Local coordinate system of Zaporizhzhia region','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',36.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',300000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','16883','conversion','EPSG','9804','EPSG','4646','EPSG','1207');
INSERT INTO "conversion" VALUES('EPSG','9805','Local coordinate system of Ivano-Frankivsk region','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',24.45,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',300000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','16884','conversion','EPSG','9805','EPSG','4629','EPSG','1207');
INSERT INTO "conversion" VALUES('EPSG','9806','Local coordinate system of Kirovohrad region','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',32.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',300000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','16885','conversion','EPSG','9806','EPSG','4634','EPSG','1207');
INSERT INTO "conversion" VALUES('EPSG','9807','Local coordinate system of Luhansk region','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',39.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',300000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','16886','conversion','EPSG','9807','EPSG','4635','EPSG','1207');
INSERT INTO "conversion" VALUES('EPSG','9808','Local coordinate system of Lviv region','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',24.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',300000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','16887','conversion','EPSG','9808','EPSG','4636','EPSG','1207');
INSERT INTO "conversion" VALUES('EPSG','9809','Local coordinate system of Mykolaiv region','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',31.5,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',300000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','16888','conversion','EPSG','9809','EPSG','4637','EPSG','1207');
INSERT INTO "conversion" VALUES('EPSG','9810','Local coordinate system of Odessa region','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',30.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',300000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','16889','conversion','EPSG','9810','EPSG','4638','EPSG','1207');
INSERT INTO "conversion" VALUES('EPSG','9811','Local coordinate system of Poltava region','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',33.5,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',300000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','16890','conversion','EPSG','9811','EPSG','4639','EPSG','1207');
INSERT INTO "conversion" VALUES('EPSG','9812','Local coordinate system of Rivne and Khmelnytsky regions','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',27.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',300000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','16908','conversion','EPSG','9812','EPSG','4651','EPSG','1207');
INSERT INTO "conversion" VALUES('EPSG','9813','Local coordinate system of Sumy region','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',34.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',300000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','16895','conversion','EPSG','9813','EPSG','4641','EPSG','1207');
INSERT INTO "conversion" VALUES('EPSG','9814','Local coordinate system of Ternopil region','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',25.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',300000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','16896','conversion','EPSG','9814','EPSG','4642','EPSG','1207');
INSERT INTO "conversion" VALUES('EPSG','9815','Local coordinate system of Kharkiv region','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',36.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',300000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','16897','conversion','EPSG','9815','EPSG','4630','EPSG','1207');
INSERT INTO "conversion" VALUES('EPSG','9816','Local coordinate system of Kherson region','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',33.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',300000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','16898','conversion','EPSG','9816','EPSG','4631','EPSG','1207');
INSERT INTO "conversion" VALUES('EPSG','9817','Local coordinate system of Cherkasy region','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',31.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',300000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','16899','conversion','EPSG','9817','EPSG','4624','EPSG','1207');
INSERT INTO "conversion" VALUES('EPSG','9818','Local coordinate system of Chernivtsi region','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',26.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',300000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','16900','conversion','EPSG','9818','EPSG','4626','EPSG','1207');
INSERT INTO "conversion" VALUES('EPSG','9819','Local coordinate system of Chernihiv region','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',32.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',300000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','16901','conversion','EPSG','9819','EPSG','4625','EPSG','1207');
INSERT INTO "conversion" VALUES('EPSG','9820','Local coordinate system of Sevastopol city','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',33.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',300000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','16902','conversion','EPSG','9820','EPSG','4649','EPSG','1207');
INSERT INTO "conversion" VALUES('EPSG','9868','MRH21-TM','In conjunction with transformation ETRS89 to MHR21-IRF (1) (code 9867), emulates the MHR21 Snake projection.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',52.18,'EPSG','9110','EPSG','8802','Longitude of natural origin',-1.48,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',227286.9881,'EPSG','9001','EPSG','8807','False northing',265751.2874,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','16937','conversion','EPSG','9868','EPSG','4652','EPSG','1141');
INSERT INTO "conversion" VALUES('EPSG','9872','Papua New Guinea Map Grid 1994 zone 57','Grid convergence uses opposite sign convention to UTM.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',159.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','16917','conversion','EPSG','9872','EPSG','4653','EPSG','1092');
INSERT INTO "conversion" VALUES('EPSG','9873','Papua New Guinea Map Grid 1994 zone 58','Grid convergence uses opposite sign convention to UTM.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',165.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','16921','conversion','EPSG','9873','EPSG','4654','EPSG','1092');
INSERT INTO "conversion" VALUES('EPSG','9879','MOLDOR11-TM','In conjunction with transformation ETRS89 to MOLDOR11-IRF (1) (code 9878), emulates the MOLDOR11 Snake projection.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',53.21,'EPSG','9110','EPSG','8802','Longitude of natural origin',-1.54,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',226574.2032,'EPSG','9001','EPSG','8807','False northing',390894.838,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','16956','conversion','EPSG','9879','EPSG','4655','EPSG','1141');
INSERT INTO "conversion" VALUES('EPSG','9894','Luxembourg TM (3D)','','EPSG','1111','Transverse Mercator 3D','EPSG','8801','Latitude of natural origin',49.5,'EPSG','9110','EPSG','8802','Longitude of natural origin',6.1,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',80000.0,'EPSG','9001','EPSG','8807','False northing',100000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','17109','conversion','EPSG','9894','EPSG','1146','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','9911','Macedonia Gauss-Kruger truncated','Truncated version of Macedonia Gauss-Kruger, with 4000000m removed from northings.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',21.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',-4000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','17400','conversion','EPSG','9911','EPSG','1148','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','9942','EBBWV14-TM','In conjunction with transformation ETRS89 to EBBWV14-IRF (1) (code 9941), emulates the EBBWV14 Snake projection.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',51.45,'EPSG','9110','EPSG','8802','Longitude of natural origin',-3.06,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',106702.326,'EPSG','9001','EPSG','8807','False northing',119968.1395,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','17347','conversion','EPSG','9942','EPSG','4661','EPSG','1141');
INSERT INTO "conversion" VALUES('EPSG','9946','Iceland Lambert Azimuthal Equal Area','','EPSG','9820','Lambert Azimuthal Equal Area','EPSG','8801','Latitude of natural origin',65.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-19.0,'EPSG','9102','EPSG','8806','False easting',1700000.0,'EPSG','9001','EPSG','8807','False northing',1300000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','17463','conversion','EPSG','9946','EPSG','1120','EPSG','1162');
INSERT INTO "conversion" VALUES('EPSG','9966','HULLEE13-TM','In conjunction with transformation ETRS89 to HULLEE13-IRF (1) (code 9965), emulates the HULLEE13 Snake projection.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',53.45,'EPSG','9110','EPSG','8802','Longitude of natural origin',-0.57,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',140859.7394,'EPSG','9001','EPSG','8807','False northing',247512.2812,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','17427','conversion','EPSG','9966','EPSG','4663','EPSG','1141');
INSERT INTO "conversion" VALUES('EPSG','9971','SCM22-TM','In conjunction with transformation ETRS89 to SCM22-IRF (1) (code 9970), emulates the SCM22 Snake projection.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',56.36,'EPSG','9110','EPSG','8802','Longitude of natural origin',-3.51,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',108600.972,'EPSG','9001','EPSG','8807','False northing',239087.349,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','17440','conversion','EPSG','9971','EPSG','4665','EPSG','1141');
INSERT INTO "conversion" VALUES('EPSG','9976','FNL22-TM','In conjunction with transformation ETRS89 to FNL22-IRF (1) (code 9975), emulates the FNL22 Snake projection.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',58.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-3.48,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',139618.9493,'EPSG','9001','EPSG','8807','False northing',183110.794,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','17455','conversion','EPSG','9976','EPSG','4664','EPSG','1141');
INSERT INTO "conversion" VALUES('EPSG','9981','MTM Nova Scotia 1997 zone 4','Introduced for use with NAD83(CSRS)v3.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-61.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',14500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','17527','conversion','EPSG','9981','EPSG','1534','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','9982','MTM Nova Scotia 1997 zone 5','Introduced for use with NAD83(CSRS)v3.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-64.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',15500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','17525','conversion','EPSG','9982','EPSG','1535','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','10101','Alabama CS27 East zone','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',30.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-85.5,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.99996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11101','conversion','EPSG','10101','EPSG','2154','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','10102','Alabama CS27 West zone','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',30.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-87.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999933333,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11102','conversion','EPSG','10102','EPSG','2155','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','10127','MWC18-TM','In conjunction with transformation ETRS89 to MWC18-IRF (1) (code 10108), emulates the MWC18 Snake projection.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',53.21,'EPSG','9110','EPSG','8802','Longitude of natural origin',-2.33,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',171975.9382,'EPSG','9001','EPSG','8807','False northing',116744.6938,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','18333','conversion','EPSG','10127','EPSG','4666','EPSG','1141');
INSERT INTO "conversion" VALUES('EPSG','10131','SPCS83 Alabama East zone (meter)','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',30.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-85.5,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.99996,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11103','conversion','EPSG','10131','EPSG','2154','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','10132','SPCS83 Alabama West zone (meter)','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',30.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-87.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999933333,'EPSG','9201','EPSG','8806','False easting',600000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11104','conversion','EPSG','10132','EPSG','2155','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','10147','Brisbane City Survey Grid 2020','Replaces Brisbane City Survey Grid 02 (code 17363) from 2022-07-01.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-28.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',153.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.99999,'EPSG','9201','EPSG','8806','False easting',150000.0,'EPSG','9001','EPSG','8807','False northing',200000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','18476','conversion','EPSG','10147','EPSG','2990','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','10148','Amtrak North East Corridor Coordinate System 2021 (international foot)','If using Hotine Oblique Mercator (variant A) method (code 9812), FE = -16,589,516.268572... ft, FN = -9,803,584.307096... ft.','EPSG','9815','Hotine Oblique Mercator (variant B)','EPSG','8811','Latitude of projection centre',40.5,'EPSG','9110','EPSG','8812','Longitude of projection centre',-74.0,'EPSG','9110','EPSG','8813','Azimuth at projection centre',58.0,'EPSG','9110','EPSG','8814','Angle from Rectified to Skew Grid',58.0,'EPSG','9110','EPSG','8815','Scale factor at projection centre',0.99999,'EPSG','9201','EPSG','8816','Easting at projection centre',1500000.0,'EPSG','9002','EPSG','8817','Northing at projection centre',1500000.0,'EPSG','9002',0);
INSERT INTO "usage" VALUES('EPSG','18487','conversion','EPSG','10148','EPSG','4669','EPSG','1141');
INSERT INTO "conversion" VALUES('EPSG','10159','S34-reconstruction','Created in 2022. In conjunction with transformation ETRS89 to S34J-IRF (1) for Jutland or  ETRS89 to S34S-IRF (1) for Zealand (codes 10161 and 10251), defines the relationship of ETRS89 to the historic S34J and S34S CRSs using a modern parameterisation.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',10.37,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',-210327.0,'EPSG','9001','EPSG','8807','False northing',-6034310.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','19660','conversion','EPSG','10159','EPSG','4575','EPSG','1203');
INSERT INTO "conversion" VALUES('EPSG','10182','DoPw22-TM','In conjunction with transformation ETRS89 to DoPw22-IRF (1) (code 10181), emulates the DoPw22 Snake projection.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',52.42,'EPSG','9110','EPSG','8802','Longitude of natural origin',-4.09,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',64859.6557,'EPSG','9001','EPSG','8807','False northing',122266.5277,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','18659','conversion','EPSG','10182','EPSG','4686','EPSG','1141');
INSERT INTO "conversion" VALUES('EPSG','10187','ShAb07-TM','In conjunction with transformation ETRS89 to ShAb07-IRF (1) (code 10186), emulates the ShAb07 Snake projection.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',52.36,'EPSG','9110','EPSG','8802','Longitude of natural origin',-3.21,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',56023.5377,'EPSG','9001','EPSG','8807','False northing',24567.6764,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','18665','conversion','EPSG','10187','EPSG','4687','EPSG','1141');
INSERT INTO "conversion" VALUES('EPSG','10193','CNH22-LCC','In conjunction with transformation ETRS89 to CNH22-IRF (1) (code 10192), emulates the CNH22 Snake projection.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',53.15,'EPSG','9110','EPSG','8822','Longitude of false origin',-3.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',53.06,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',53.24,'EPSG','9110','EPSG','8826','Easting at false origin',212548.8756,'EPSG','9001','EPSG','8827','Northing at false origin',495230.9254,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','18720','conversion','EPSG','10193','EPSG','4677','EPSG','1141');
INSERT INTO "conversion" VALUES('EPSG','10198','CWS13-TM','In conjunction with transformation ETRS89 to CWS13-IRF (1) (code 10197), emulates the CWS13 Snake projection.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',52.24,'EPSG','9110','EPSG','8802','Longitude of natural origin',-2.54,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',199668.0926,'EPSG','9001','EPSG','8807','False northing',89354.3229,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','18704','conversion','EPSG','10198','EPSG','4678','EPSG','1141');
INSERT INTO "conversion" VALUES('EPSG','10201','Arizona Coordinate System East zone','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',31.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-110.1,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11105','conversion','EPSG','10201','EPSG','2167','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','10202','Arizona Coordinate System Central zone','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',31.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-111.55,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11106','conversion','EPSG','10202','EPSG','2166','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','10203','Arizona Coordinate System West zone','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',31.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-113.45,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999933333,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11107','conversion','EPSG','10203','EPSG','2168','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','10206','DIBA15-TM','In conjunction with transformation ETRS89 to DIBA15-IRF (1) (code 10205), emulates the DIBA15 Snake projection.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',51.51,'EPSG','9110','EPSG','8802','Longitude of natural origin',-1.18,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',511622.854,'EPSG','9001','EPSG','8807','False northing',324737.4633,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','18770','conversion','EPSG','10206','EPSG','4681','EPSG','1141');
INSERT INTO "conversion" VALUES('EPSG','10211','GW22-LCC','In conjunction with transformation ETRS89 to GWPBS22-IRF (1) (code 10210) emulates the GWPBS22 Snake projection, with ETRS89 to GWWAB22-IRF (1) (code 10215) emulates GWWAB22 and with ETRS89 to GWWWA22-IRF (1) (code 10220) emulates the GWWWA22 Snake proj.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',51.39,'EPSG','9110','EPSG','8822','Longitude of false origin',-2.39,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',51.24,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',51.54,'EPSG','9110','EPSG','8826','Easting at false origin',168854.016,'EPSG','9001','EPSG','8827','Northing at false origin',193447.117,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','18794','conversion','EPSG','10211','EPSG','4690','EPSG','1141');
INSERT INTO "conversion" VALUES('EPSG','10226','MALS09-TM','In conjunction with transformation ETRS89 to MALS09-IRF (1) (code 10225), emulates the MALS09 Snake projection.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',52.12,'EPSG','9110','EPSG','8802','Longitude of natural origin',-1.09,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',175262.1809,'EPSG','9001','EPSG','8807','False northing',174688.2508,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','18743','conversion','EPSG','10226','EPSG','4682','EPSG','1141');
INSERT INTO "conversion" VALUES('EPSG','10231','SPCS83 Arizona East zone (meter)','State law defines origin in International feet. FE = 700000ft. See code 15304 for equivalent non-metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',31.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-110.1,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',213360.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11108','conversion','EPSG','10231','EPSG','2167','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','10232','SPCS83 Arizona Central zone (meter)','State law defines origin in International feet. FE = 700000ft. See code 15305 for equivalent non-metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',31.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-111.55,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',213360.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11109','conversion','EPSG','10232','EPSG','2166','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','10233','SPCS83 Arizona West zone (meter)','State law defines origin in International feet. FE = 700000ft. See code 15306 for equivalent non-metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',31.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-113.45,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999933333,'EPSG','9201','EPSG','8806','False easting',213360.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11110','conversion','EPSG','10233','EPSG','2168','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','10234','OxWo08-TM','In conjunction with transformation ETRS89 to OxWo08-IRF (1) (code 10230), emulates the OxWo08 Snake projection.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',51.57,'EPSG','9110','EPSG','8802','Longitude of natural origin',-1.42,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',134791.6965,'EPSG','9001','EPSG','8807','False northing',121872.5056,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','18761','conversion','EPSG','10234','EPSG','4680','EPSG','1141');
INSERT INTO "conversion" VALUES('EPSG','10239','SYC20-TM','In conjunction with transformation ETRS89 to SYC20-IRF (1) (code 10238), emulates the SYC20 Snake projection.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',52.54,'EPSG','9110','EPSG','8802','Longitude of natural origin',-2.36,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',110094.4312,'EPSG','9001','EPSG','8807','False northing',120623.8396,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','18762','conversion','EPSG','10239','EPSG','4679','EPSG','1141');
INSERT INTO "conversion" VALUES('EPSG','10253','S45B-reconstruction','Created in 2022. In conjunction with transformation ETRS89 to S45B-IRF (1) (code  10255) defines the relationship of ETRS89 to the historic S45B CRS using a modern parameterisation.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',55.11171,'EPSG','9102','EPSG','8802','Longitude of natural origin',14.88927,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',-50000.0,'EPSG','9001','EPSG','8807','False northing',50000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','19778','conversion','EPSG','10253','EPSG','2533','EPSG','1203');
INSERT INTO "conversion" VALUES('EPSG','10257','Generalstabens System LCC','Created in 2022. Together with transformation ETRS89 to GS-IRF (1) for Jutland and Zealand or ETRS89 to KK-IRF (1) for Copenhagen (codes 10259 and 10267), defines the relationship of ETRS89 to the historic GS and KK CRSs using a modern parameterisation.','EPSG','1102','Lambert Conic Conformal (1SP variant B)','EPSG','8801','Latitude of natural origin',56.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8821','Latitude of false origin',55.0,'EPSG','9102','EPSG','8822','Longitude of false origin',10.37775,'EPSG','9102','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','19706','conversion','EPSG','10257','EPSG','4575','EPSG','1203');
INSERT INTO "conversion" VALUES('EPSG','10261','GSB-reconstruction','Created in 2022. In conjunction with transformation ETRS89 to GSB-IRF (1) (code 10263) defines the relationship of ETRS89 to the historic Generalstaben projCRS using a modern parameterisation.','EPSG','1102','Lambert Conic Conformal (1SP variant B)','EPSG','8801','Latitude of natural origin',56.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8821','Latitude of false origin',55.0,'EPSG','9102','EPSG','8822','Longitude of false origin',14.92775,'EPSG','9102','EPSG','8826','Easting at false origin',-18831.46,'EPSG','9001','EPSG','8827','Northing at false origin',5614.621,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','19777','conversion','EPSG','10261','EPSG','2533','EPSG','1203');
INSERT INTO "conversion" VALUES('EPSG','10269','Ostenfeld-reconstruction','Created in 2022. In conjunction with transformation ETRS89 to Ostenfeld-IRF (1) (code 10271) defines the relationship of ETRS89 to the historic Ostenfeld CRS of Prussia using a modern parameterisation.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',54.4685,'EPSG','9102','EPSG','8802','Longitude of natural origin',9.233,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','19694','conversion','EPSG','10269','EPSG','4694','EPSG','1203');
INSERT INTO "conversion" VALUES('EPSG','10274','SMITB20-TM','In conjunction with transformation ETRS89 to SMITB20-IRF (1) (code 10273), emulates the SMITB20 Snake projection.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',50.45,'EPSG','9110','EPSG','8802','Longitude of natural origin',-3.51,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',110693.666,'EPSG','9001','EPSG','8807','False northing',102089.2943,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','19177','conversion','EPSG','10274','EPSG','4688','EPSG','1141');
INSERT INTO "conversion" VALUES('EPSG','10279','RBEPP12-LCC','In conjunction with transformation ETRS89 to RBEPP12-IRF (1) (code 10278), emulates the RBEPP12 Snake projection.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',50.51,'EPSG','9110','EPSG','8822','Longitude of false origin',-3.15,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',50.18,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',51.27,'EPSG','9110','EPSG','8826','Easting at false origin',372382.8292,'EPSG','9001','EPSG','8827','Northing at false origin',217764.7796,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','19186','conversion','EPSG','10279','EPSG','4689','EPSG','1141');
INSERT INTO "conversion" VALUES('EPSG','10301','Arkansas CS27 North','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',34.2,'EPSG','9110','EPSG','8822','Longitude of false origin',-92.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',36.14,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',34.56,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11111','conversion','EPSG','10301','EPSG','2169','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','10302','Arkansas CS27 South','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',32.4,'EPSG','9110','EPSG','8822','Longitude of false origin',-92.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',34.46,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',33.18,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11112','conversion','EPSG','10302','EPSG','2170','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','10313','Lambert New Caledonia 2015','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-21.3,'EPSG','9110','EPSG','8822','Longitude of false origin',166.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',-20.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-22.2,'EPSG','9110','EPSG','8826','Easting at false origin',2400000.0,'EPSG','9001','EPSG','8827','Northing at false origin',2300000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','19602','conversion','EPSG','10313','EPSG','3430','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','10325','Bosnia and Herzegovina Transverse Mercator','Replaces Balkans TM zones in Bosnia and Herzegovina.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',18.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','19739','conversion','EPSG','10325','EPSG','1050','EPSG','1056');
INSERT INTO "conversion" VALUES('EPSG','10331','SPCS83 Arkansas North zone (meter)','See code 15385 for equivalent non-metric definition.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',34.2,'EPSG','9110','EPSG','8822','Longitude of false origin',-92.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',36.14,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',34.56,'EPSG','9110','EPSG','8826','Easting at false origin',400000.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11113','conversion','EPSG','10331','EPSG','2169','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','10332','SPCS83 Arkansas South zone (meter)','See code 15386 for equivalent non-metric definition.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',32.4,'EPSG','9110','EPSG','8822','Longitude of false origin',-92.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',34.46,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',33.18,'EPSG','9110','EPSG','8826','Easting at false origin',400000.0,'EPSG','9001','EPSG','8827','Northing at false origin',400000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11114','conversion','EPSG','10332','EPSG','2170','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','10401','California CS27 zone I','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',39.2,'EPSG','9110','EPSG','8822','Longitude of false origin',-122.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',41.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',40.0,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11115','conversion','EPSG','10401','EPSG','2175','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','10402','California CS27 zone II','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',37.4,'EPSG','9110','EPSG','8822','Longitude of false origin',-122.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',39.5,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',38.2,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11116','conversion','EPSG','10402','EPSG','2176','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','10403','California CS27 zone III','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',36.3,'EPSG','9110','EPSG','8822','Longitude of false origin',-120.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',38.26,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',37.04,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11117','conversion','EPSG','10403','EPSG','2177','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','10404','California CS27 zone IV','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',35.2,'EPSG','9110','EPSG','8822','Longitude of false origin',-119.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',37.15,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',36.0,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11118','conversion','EPSG','10404','EPSG','2178','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','10405','California CS27 zone V','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',33.3,'EPSG','9110','EPSG','8822','Longitude of false origin',-118.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',35.28,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',34.02,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11119','conversion','EPSG','10405','EPSG','2179','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','10406','California CS27 zone VI','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',32.1,'EPSG','9110','EPSG','8822','Longitude of false origin',-116.15,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',33.53,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',32.47,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11120','conversion','EPSG','10406','EPSG','2180','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','10407','California CS27 zone VII','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',34.08,'EPSG','9110','EPSG','8822','Longitude of false origin',-118.2,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',34.25,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',33.52,'EPSG','9110','EPSG','8826','Easting at false origin',4186692.58,'EPSG','9003','EPSG','8827','Northing at false origin',416926.74,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11121','conversion','EPSG','10407','EPSG','2181','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','10408','California CS27 zone VII','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',34.08,'EPSG','9110','EPSG','8822','Longitude of false origin',-118.2,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',34.25,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',33.52,'EPSG','9110','EPSG','8826','Easting at false origin',4186692.58,'EPSG','9003','EPSG','8827','Northing at false origin',4160926.74,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11122','conversion','EPSG','10408','EPSG','2181','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','10420','California Albers','Created at the Stephen P Teale Data Center.','EPSG','9822','Albers Equal Area','EPSG','8821','Latitude of false origin',0.0,'EPSG','9102','EPSG','8822','Longitude of false origin',-120.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',34.0,'EPSG','9102','EPSG','8824','Latitude of 2nd standard parallel',40.5,'EPSG','9102','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',-4000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11123','conversion','EPSG','10420','EPSG','1375','EPSG','1135');
INSERT INTO "conversion" VALUES('EPSG','10424','Albany Grid 1994','Replaced by ALB2020.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',117.53,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000044,'EPSG','9201','EPSG','8806','False easting',50000.0,'EPSG','9001','EPSG','8807','False northing',4000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','20290','conversion','EPSG','10424','EPSG','4439','EPSG','1054');
INSERT INTO "conversion" VALUES('EPSG','10425','Barrow Island Grid 1994','Replaced by BIO2020.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',115.15,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000022,'EPSG','9201','EPSG','8806','False easting',60000.0,'EPSG','9001','EPSG','8807','False northing',2600000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','20256','conversion','EPSG','10425','EPSG','4438','EPSG','1054');
INSERT INTO "conversion" VALUES('EPSG','10426','Broome Grid 1994','Replaced by BRO2020.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',122.2,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00000298,'EPSG','9201','EPSG','8806','False easting',50000.0,'EPSG','9001','EPSG','8807','False northing',2200000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','20289','conversion','EPSG','10426','EPSG','4441','EPSG','1054');
INSERT INTO "conversion" VALUES('EPSG','10427','Busselton Coastal Grid 1994','Replaced by BCG2020.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',115.26,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.99999592,'EPSG','9201','EPSG','8806','False easting',50000.0,'EPSG','9001','EPSG','8807','False northing',3900000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','20288','conversion','EPSG','10427','EPSG','4437','EPSG','1054');
INSERT INTO "conversion" VALUES('EPSG','10428','Carnarvon Grid 1994','Replaced by CARN2020.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',113.4,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.99999796,'EPSG','9201','EPSG','8806','False easting',50000.0,'EPSG','9001','EPSG','8807','False northing',2950000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','20287','conversion','EPSG','10428','EPSG','4442','EPSG','1054');
INSERT INTO "conversion" VALUES('EPSG','10429','Collie Grid 1994','Replaced by COL2020.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',115.56,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000019,'EPSG','9201','EPSG','8806','False easting',40000.0,'EPSG','9001','EPSG','8807','False northing',4000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','20286','conversion','EPSG','10429','EPSG','4443','EPSG','1054');
INSERT INTO "conversion" VALUES('EPSG','10430','Esperance Grid 1994','Replaced by ESP2020.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',121.53,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000055,'EPSG','9201','EPSG','8806','False easting',50000.0,'EPSG','9001','EPSG','8807','False northing',3950000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','20285','conversion','EPSG','10430','EPSG','4445','EPSG','1054');
INSERT INTO "conversion" VALUES('EPSG','10431','SPCS83 California zone 1 (meter)','See code 15307 for equivalent non-metric definition.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',39.2,'EPSG','9110','EPSG','8822','Longitude of false origin',-122.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',41.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',40.0,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9001','EPSG','8827','Northing at false origin',500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11124','conversion','EPSG','10431','EPSG','2175','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','10432','SPCS83 California zone 2 (meter)','See code 15308 for equivalent non-metric definition.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',37.4,'EPSG','9110','EPSG','8822','Longitude of false origin',-122.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',39.5,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',38.2,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9001','EPSG','8827','Northing at false origin',500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11125','conversion','EPSG','10432','EPSG','2176','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','10433','SPCS83 California zone 3 (meter)','See code 15309 for equivalent non-metric definition.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',36.3,'EPSG','9110','EPSG','8822','Longitude of false origin',-120.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',38.26,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',37.04,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9001','EPSG','8827','Northing at false origin',500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11126','conversion','EPSG','10433','EPSG','2177','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','10434','SPCS83 California zone 4 (meter)','See code 15310 for equivalent non-metric definition.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',35.2,'EPSG','9110','EPSG','8822','Longitude of false origin',-119.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',37.15,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',36.0,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9001','EPSG','8827','Northing at false origin',500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11127','conversion','EPSG','10434','EPSG','2178','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','10435','SPCS83 California zone 5 (meter)','See code 15311 for equivalent non-metric definition.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',33.3,'EPSG','9110','EPSG','8822','Longitude of false origin',-118.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',35.28,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',34.02,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9001','EPSG','8827','Northing at false origin',500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11128','conversion','EPSG','10435','EPSG','2182','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','10436','SPCS83 California zone 6 (meter)','See code 15312 for equivalent non-metric definition.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',32.1,'EPSG','9110','EPSG','8822','Longitude of false origin',-116.15,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',33.53,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',32.47,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9001','EPSG','8827','Northing at false origin',500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11129','conversion','EPSG','10436','EPSG','2180','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','10437','Exmouth Grid 1994','Replaced by EXM2020.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',114.04,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00000236,'EPSG','9201','EPSG','8806','False easting',50000.0,'EPSG','9001','EPSG','8807','False northing',2650000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','20284','conversion','EPSG','10437','EPSG','4448','EPSG','1054');
INSERT INTO "conversion" VALUES('EPSG','10438','Geraldton Coastal Grid 1994','Replaced by GCG2020.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',114.35,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00000628,'EPSG','9201','EPSG','8806','False easting',50000.0,'EPSG','9001','EPSG','8807','False northing',3350000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','20283','conversion','EPSG','10438','EPSG','4449','EPSG','1054');
INSERT INTO "conversion" VALUES('EPSG','10439','Goldfields Grid 1994','Replaced by GOLD2020.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',121.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00004949,'EPSG','9201','EPSG','8806','False easting',60000.0,'EPSG','9001','EPSG','8807','False northing',3700000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','20282','conversion','EPSG','10439','EPSG','4436','EPSG','1054');
INSERT INTO "conversion" VALUES('EPSG','10440','Jurien Coastal Grid 1994','Replaced by JCG2020.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',114.59,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00000314,'EPSG','9201','EPSG','8806','False easting',50000.0,'EPSG','9001','EPSG','8807','False northing',3550000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','20281','conversion','EPSG','10440','EPSG','4440','EPSG','1054');
INSERT INTO "conversion" VALUES('EPSG','10441','Kalbarri Grid 1994','Replaced by KALB2020.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',114.1855,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000014,'EPSG','9201','EPSG','8806','False easting',55000.0,'EPSG','9001','EPSG','8807','False northing',3600000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','20291','conversion','EPSG','10441','EPSG','4444','EPSG','1054');
INSERT INTO "conversion" VALUES('EPSG','10442','Karratha Grid 1994','Replaced by KAR2020.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',116.56,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999989,'EPSG','9201','EPSG','8806','False easting',50000.0,'EPSG','9001','EPSG','8807','False northing',2450000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','20280','conversion','EPSG','10442','EPSG','4451','EPSG','1054');
INSERT INTO "conversion" VALUES('EPSG','10443','Kununurra Grid 1994','Replaced by KUN2020.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',128.45,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000165,'EPSG','9201','EPSG','8806','False easting',50000.0,'EPSG','9001','EPSG','8807','False northing',2000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','20279','conversion','EPSG','10443','EPSG','4452','EPSG','1054');
INSERT INTO "conversion" VALUES('EPSG','10444','Lancelin Coastal Grid 1994','Replaced by LCG2020.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',115.22,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00000157,'EPSG','9201','EPSG','8806','False easting',50000.0,'EPSG','9001','EPSG','8807','False northing',3650000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','20278','conversion','EPSG','10444','EPSG','4453','EPSG','1054');
INSERT INTO "conversion" VALUES('EPSG','10445','Margaret River Coastal Grid 1994','Replaced by MRCG2020.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',115.1,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000055,'EPSG','9201','EPSG','8806','False easting',50000.0,'EPSG','9001','EPSG','8807','False northing',3950000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','20277','conversion','EPSG','10445','EPSG','4457','EPSG','1054');
INSERT INTO "conversion" VALUES('EPSG','10446','Perth Coastal Grid 1994','Replaced by PCG2020.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',115.49,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.99999906,'EPSG','9201','EPSG','8806','False easting',50000.0,'EPSG','9001','EPSG','8807','False northing',3800000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','20276','conversion','EPSG','10446','EPSG','4462','EPSG','1054');
INSERT INTO "conversion" VALUES('EPSG','10447','Port Hedland Grid 1994','Replaced by PHG2020.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',118.36,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00000135,'EPSG','9201','EPSG','8806','False easting',50000.0,'EPSG','9001','EPSG','8807','False northing',2400000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','20275','conversion','EPSG','10447','EPSG','4466','EPSG','1054');
INSERT INTO "conversion" VALUES('EPSG','10470','COV23-TM','In conjunction with transformation ETRS89 to COV23-IRF (1) (code 10469), emulates the COV23 Snake projection.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',52.24,'EPSG','9110','EPSG','8802','Longitude of natural origin',-1.33,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',116887.9989,'EPSG','9001','EPSG','8807','False northing',102194.9369,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','20314','conversion','EPSG','10470','EPSG','4743','EPSG','1141');
INSERT INTO "conversion" VALUES('EPSG','10476','Brenner Base Tunnel TM','Also defined with latitude of natural origin at equator and false northing = -5105739.717m.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',46.58507947,'EPSG','9110','EPSG','8802','Longitude of natural origin',11.31425775,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000121,'EPSG','9201','EPSG','8806','False easting',20000.0,'EPSG','9001','EPSG','8807','False northing',100000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','20340','conversion','EPSG','10476','EPSG','4744','EPSG','1285');
INSERT INTO "conversion" VALUES('EPSG','10479','TWDB Groundwater Modeling (US survey foot)','','EPSG','9822','Albers Equal Area','EPSG','8821','Latitude of false origin',31.25,'EPSG','9102','EPSG','8822','Longitude of false origin',-100.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',27.5,'EPSG','9102','EPSG','8824','Latitude of 2nd standard parallel',35.0,'EPSG','9102','EPSG','8826','Easting at false origin',4921250.0,'EPSG','9003','EPSG','8827','Northing at false origin',19685000.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','20353','conversion','EPSG','10479','EPSG','1412','EPSG','1286');
INSERT INTO "conversion" VALUES('EPSG','10501','Colorado CS27 North zone','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',39.2,'EPSG','9110','EPSG','8822','Longitude of false origin',-105.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',39.43,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',40.47,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11130','conversion','EPSG','10501','EPSG','2184','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','10502','Colorado CS27 Central zone','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',37.5,'EPSG','9110','EPSG','8822','Longitude of false origin',-105.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',39.45,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',38.27,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11131','conversion','EPSG','10502','EPSG','2183','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','10503','Colorado CS27 South zone','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',36.4,'EPSG','9110','EPSG','8822','Longitude of false origin',-105.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',38.26,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',37.14,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11132','conversion','EPSG','10503','EPSG','2185','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','10515','WISCRS Adjusted Jackson County (ftUS)','Modification of WISCRS Jackson County (ftUS) (code 7451), designed to enable positioning using the WISCORS network referenced to NAD83(2011) to be used for referencing to NAD83(HARN) by approximating the NAD83(2011) to NAD83(HARN) NADCON5 transformation.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',44.151200716,'EPSG','9110','EPSG','8802','Longitude of natural origin',-90.503946745,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000353,'EPSG','9201','EPSG','8806','False easting',88582.5,'EPSG','9003','EPSG','8807','False northing',82020.833,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','20746','conversion','EPSG','10515','EPSG','4343','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','10531','SPCS83 Colorado North zone (meter)','See code 15313 for equivalent non-metric definition.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',39.2,'EPSG','9110','EPSG','8822','Longitude of false origin',-105.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',40.47,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',39.43,'EPSG','9110','EPSG','8826','Easting at false origin',914401.8289,'EPSG','9001','EPSG','8827','Northing at false origin',304800.6096,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11133','conversion','EPSG','10531','EPSG','2184','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','10532','SPCS83 Colorado Central zone (meter)','See code 15314 for equivalent non-metric definition.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',37.5,'EPSG','9110','EPSG','8822','Longitude of false origin',-105.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',39.45,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',38.27,'EPSG','9110','EPSG','8826','Easting at false origin',914401.8289,'EPSG','9001','EPSG','8827','Northing at false origin',304800.6096,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11134','conversion','EPSG','10532','EPSG','2183','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','10533','SPCS83 Colorado South zone (meter)','See code 15315 for equivalent non-metric definition.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',36.4,'EPSG','9110','EPSG','8822','Longitude of false origin',-105.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',38.26,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',37.14,'EPSG','9110','EPSG','8826','Easting at false origin',914401.8289,'EPSG','9001','EPSG','8827','Northing at false origin',304800.6096,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11135','conversion','EPSG','10533','EPSG','2185','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','10591','GLANCE projection - Africa','Basis for the GLANCE continental tiling system for Africa, used for the GLANCE land cover product in archival process at https://lpdaac.usgs.gov/products/glance30v001/.','EPSG','9820','Lambert Azimuthal Equal Area','EPSG','8801','Latitude of natural origin',5.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',20.0,'EPSG','9102','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','21201','conversion','EPSG','10591','EPSG','4745','EPSG','1287');
INSERT INTO "conversion" VALUES('EPSG','10593','GLANCE projection - Asia','Basis for the GLANCE continental tiling system for Asia, used for the GLANCE land cover product in archival process at https://lpdaac.usgs.gov/products/glance30v001/.','EPSG','9820','Lambert Azimuthal Equal Area','EPSG','8801','Latitude of natural origin',45.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',100.0,'EPSG','9102','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','21205','conversion','EPSG','10593','EPSG','4747','EPSG','1287');
INSERT INTO "conversion" VALUES('EPSG','10595','GLANCE projection - Europe','Basis for the GLANCE continental tiling system for Europe, used for the GLANCE land cover product (https://lpdaac.usgs.gov/products/glance30v001/). For compatibility with EU INSPIRE regulations, Europe Equal Area 2001 (code 19986) should be preferred.','EPSG','9820','Lambert Azimuthal Equal Area','EPSG','8801','Latitude of natural origin',55.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',20.0,'EPSG','9102','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','21318','conversion','EPSG','10595','EPSG','4748','EPSG','1287');
INSERT INTO "conversion" VALUES('EPSG','10597','GLANCE projection - North America','Basis for the GLANCE continental tiling system for North America, used for the GLANCE land cover product in archival process at https://lpdaac.usgs.gov/products/glance30v001/.','EPSG','9820','Lambert Azimuthal Equal Area','EPSG','8801','Latitude of natural origin',50.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-100.0,'EPSG','9102','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','21209','conversion','EPSG','10597','EPSG','4749','EPSG','1287');
INSERT INTO "conversion" VALUES('EPSG','10599','GLANCE projection - Oceania','Basis for the GLANCE continental tiling system for Oceania, used for the GLANCE land cover product in archival process at https://lpdaac.usgs.gov/products/glance30v001/.','EPSG','9820','Lambert Azimuthal Equal Area','EPSG','8801','Latitude of natural origin',-15.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',135.0,'EPSG','9102','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','21212','conversion','EPSG','10599','EPSG','4751','EPSG','1287');
INSERT INTO "conversion" VALUES('EPSG','10600','Connecticut CS27','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',40.5,'EPSG','9110','EPSG','8822','Longitude of false origin',-72.45,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',41.52,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',41.12,'EPSG','9110','EPSG','8826','Easting at false origin',600000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11136','conversion','EPSG','10600','EPSG','1377','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','10602','GLANCE projection - South America','Basis for the GLANCE continental tiling system for South America, used for the GLANCE land cover product in archival process at https://lpdaac.usgs.gov/products/glance30v001/.','EPSG','9820','Lambert Azimuthal Equal Area','EPSG','8801','Latitude of natural origin',-15.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-60.0,'EPSG','9102','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','21214','conversion','EPSG','10602','EPSG','4750','EPSG','1287');
INSERT INTO "conversion" VALUES('EPSG','10621','San Francisco International Airport B18 grid (ftUS)','','EPSG','1130','Local Orthographic','EPSG','8811','Latitude of projection centre',37.6289686531,'EPSG','9102','EPSG','8812','Longitude of projection centre',-122.3939412704,'EPSG','9102','EPSG','8813','Azimuth at projection centre',27.7928209333,'EPSG','9102','EPSG','8815','Scale factor at projection centre',0.9999968,'EPSG','9201','EPSG','8816','Easting at projection centre',0.0,'EPSG','9003','EPSG','8817','Northing at projection centre',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','21559','conversion','EPSG','10621','EPSG','4529','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','10625','ECML14-TM','In conjunction with transformation ETRS89 to ECML14-IRF (1) (code 10624), emulates the ECML14 Snake projection.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',53.48,'EPSG','9110','EPSG','8802','Longitude of natural origin',-1.36,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',108021.121,'EPSG','9001','EPSG','8807','False northing',263196.8721,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','21388','conversion','EPSG','10625','EPSG','4774','EPSG','1141');
INSERT INTO "conversion" VALUES('EPSG','10630','SPCS83 Connecticut zone (meter)','See code 15316 for equivalent non-metric definition.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',40.5,'EPSG','9110','EPSG','8822','Longitude of false origin',-72.45,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',41.52,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',41.12,'EPSG','9110','EPSG','8826','Easting at false origin',304800.6096,'EPSG','9001','EPSG','8827','Northing at false origin',152400.3048,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11137','conversion','EPSG','10630','EPSG','1377','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','10631','WC05-TM','In conjunction with transformation ETRS89 to WC05-IRF (1) (code 10629), emulates the WC05 Snake projection.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',53.27,'EPSG','9110','EPSG','8802','Longitude of natural origin',-2.09,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',209900.2337,'EPSG','9001','EPSG','8807','False northing',401958.2494,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','21391','conversion','EPSG','10631','EPSG','4775','EPSG','1141');
INSERT INTO "conversion" VALUES('EPSG','10640','Saba Transverse Mercator 2020','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-63.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',29973.97,'EPSG','9001','EPSG','8807','False northing',-1947925.94,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','21870','conversion','EPSG','10640','EPSG','4757','EPSG','1056');
INSERT INTO "conversion" VALUES('EPSG','10664','Porto Alegre Transverse Mercator','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-51.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999995,'EPSG','9201','EPSG','8806','False easting',300000.0,'EPSG','9001','EPSG','8807','False northing',5000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','21622','conversion','EPSG','10664','EPSG','4777','EPSG','1056');
INSERT INTO "conversion" VALUES('EPSG','10700','Delaware CS27','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',38.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-75.25,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999995,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11138','conversion','EPSG','10700','EPSG','1378','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','10719','Uzbekistan coordinate reference system 2024 zone 40','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',40.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',57.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999984965,'EPSG','9201','EPSG','8806','False easting',350000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','22083','conversion','EPSG','10719','EPSG','4784','EPSG','1207');
INSERT INTO "conversion" VALUES('EPSG','10720','Uzbekistan coordinate reference system 2024 zone 41','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',37.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',63.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.99999730738,'EPSG','9201','EPSG','8806','False easting',350000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','22199','conversion','EPSG','10720','EPSG','4785','EPSG','1207');
INSERT INTO "conversion" VALUES('EPSG','10721','Uzbekistan coordinate reference system 2024 zone 42','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',36.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',69.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999962402,'EPSG','9201','EPSG','8806','False easting',350000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','22081','conversion','EPSG','10721','EPSG','4786','EPSG','1207');
INSERT INTO "conversion" VALUES('EPSG','10722','Uzbekistan coordinate reference system 2024 zone 43','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',39.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',75.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999995546,'EPSG','9201','EPSG','8806','False easting',350000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','22080','conversion','EPSG','10722','EPSG','4787','EPSG','1207');
INSERT INTO "conversion" VALUES('EPSG','10730','SPCS83 Delaware zone (meter)','See code 15317 for equivalent non-metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',38.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-75.25,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999995,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11139','conversion','EPSG','10730','EPSG','1378','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','10743','Sint Eustatius Transverse Mercator 2020','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-63.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',578.55,'EPSG','9001','EPSG','8807','False northing',-1930396.26,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','22291','conversion','EPSG','10743','EPSG','4788','EPSG','1056');
INSERT INTO "conversion" VALUES('EPSG','10751','Agriculture Canada Albers','','EPSG','9822','Albers Equal Area','EPSG','8821','Latitude of false origin',40.0,'EPSG','9102','EPSG','8822','Longitude of false origin',-96.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',44.75,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',55.75,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','22472','conversion','EPSG','10751','EPSG','4752','EPSG','1292');
INSERT INTO "conversion" VALUES('EPSG','10757','Bonaire Transverse Mercator','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',12.105037123,'EPSG','9110','EPSG','8802','Longitude of natural origin',-68.150648821,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',23209.56,'EPSG','9001','EPSG','8807','False northing',21423.99,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','22167','conversion','EPSG','10757','EPSG','3822','EPSG','1056');
INSERT INTO "conversion" VALUES('EPSG','10772','Ribeirao Preto Local TM','PBD-Projeção de Baixa Deformação (low distortion projection).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-21.12,'EPSG','9110','EPSG','8802','Longitude of natural origin',-47.47,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000092,'EPSG','9201','EPSG','8806','False easting',15000.0,'EPSG','9001','EPSG','8807','False northing',25000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','22213','conversion','EPSG','10772','EPSG','4789','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','10819','Agriculture Canada Albers','','EPSG','9822','Albers Equal Area','EPSG','8821','Latitude of false origin',40.0,'EPSG','9102','EPSG','8822','Longitude of false origin',-96.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',44.75,'EPSG','9102','EPSG','8824','Latitude of 2nd standard parallel',55.75,'EPSG','9102','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','22544','conversion','EPSG','10819','EPSG','4752','EPSG','1292');
INSERT INTO "conversion" VALUES('EPSG','10832','Georgia Lambert','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',42.3,'EPSG','9110','EPSG','8822','Longitude of false origin',43.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',41.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',42.4,'EPSG','9110','EPSG','8826','Easting at false origin',1300000.0,'EPSG','9001','EPSG','8827','Northing at false origin',400000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','22792','conversion','EPSG','10832','EPSG','3251','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','10856','Brazil Albers','','EPSG','9822','Albers Equal Area','EPSG','8821','Latitude of false origin',-12.0,'EPSG','9102','EPSG','8822','Longitude of false origin',-54.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-2.0,'EPSG','9102','EPSG','8824','Latitude of 2nd standard parallel',-22.0,'EPSG','9102','EPSG','8826','Easting at false origin',5000000.0,'EPSG','9001','EPSG','8827','Northing at false origin',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','22837','conversion','EPSG','10856','EPSG','3223','EPSG','1162');
INSERT INTO "conversion" VALUES('EPSG','10862','WSPG-TM','In conjunction with transformation ETRS89 to WSPG-IRF (1) (code 9764), emulates the WSPG Snake projection.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',54.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-0.51,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',39630.9213,'EPSG','9001','EPSG','8807','False northing',36016.293,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','22834','conversion','EPSG','10862','EPSG','4801','EPSG','1293');
INSERT INTO "conversion" VALUES('EPSG','10901','Florida CS27 East zone','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',24.2,'EPSG','9110','EPSG','8802','Longitude of natural origin',-81.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999941177,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11140','conversion','EPSG','10901','EPSG','2186','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','10902','Florida CS27 West zone','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',24.2,'EPSG','9110','EPSG','8802','Longitude of natural origin',-82.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999941177,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11141','conversion','EPSG','10902','EPSG','2188','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','10903','Florida CS27 North zone','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',29.0,'EPSG','9110','EPSG','8822','Longitude of false origin',-84.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',30.45,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',29.35,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11142','conversion','EPSG','10903','EPSG','2187','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','10931','SPCS83 Florida East zone (meter)','See code 15318 for equivalent non-metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',24.2,'EPSG','9110','EPSG','8802','Longitude of natural origin',-81.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999941177,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11143','conversion','EPSG','10931','EPSG','2186','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','10932','SPCS83 Florida West zone (meter)','See code 15319 for equivalent non-metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',24.2,'EPSG','9110','EPSG','8802','Longitude of natural origin',-82.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999941177,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11144','conversion','EPSG','10932','EPSG','2188','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','10933','SPCS83 Florida North zone (meter)','See code 15320 for equivalent non-metric definition.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',29.0,'EPSG','9110','EPSG','8822','Longitude of false origin',-84.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',30.45,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',29.35,'EPSG','9110','EPSG','8826','Easting at false origin',600000.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11145','conversion','EPSG','10933','EPSG','2187','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','10934','Florida GDL Albers (meter)','','EPSG','9822','Albers Equal Area','EPSG','8821','Latitude of false origin',24.0,'EPSG','9110','EPSG','8822','Longitude of false origin',-84.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',24.0,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',31.3,'EPSG','9110','EPSG','8826','Easting at false origin',400000.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11146','conversion','EPSG','10934','EPSG','1379','EPSG','1135');
INSERT INTO "conversion" VALUES('EPSG','10970','SPCS2022 Gulf zone','See code 10971 for equivalent non-metric definition.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',28.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-90.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',1600200.0,'EPSG','9001','EPSG','8807','False northing',533400.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','23626','conversion','EPSG','10970','EPSG','3357','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','10971','SPCS2022 Gulf zone (ft)','See code 10970 for equivalent metric definition.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',28.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-90.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',5250000.0,'EPSG','9002','EPSG','8807','False northing',1750000.0,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','23659','conversion','EPSG','10971','EPSG','3357','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','10972','SPCS2022 Gulf Louisiana Shelf zone (ft)','Emulates offshore part of Louisiana CS27 South zone. NGS definition in meters has FE=609610.287m FN=147761.325m.
','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',30.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-91.2,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999165,'EPSG','9201','EPSG','8806','False easting',2000033.75,'EPSG','9002','EPSG','8807','False northing',484781.25,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','23660','conversion','EPSG','10972','EPSG','4809','EPSG','1294');
INSERT INTO "conversion" VALUES('EPSG','10973','SPCS2022 Gulf Texas Shelf North zone (ft)','Emulates offshore part of Texas CS27 South Central zone. NGS definition in meters has FE=609633.909m FN=166114.095m.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',29.1957,'EPSG','9110','EPSG','8802','Longitude of natural origin',-99.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9998499,'EPSG','9201','EPSG','8806','False easting',2000111.25,'EPSG','9002','EPSG','8807','False northing',544993.75,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','23629','conversion','EPSG','10973','EPSG','4808','EPSG','1294');
INSERT INTO "conversion" VALUES('EPSG','10974','SPCS2022 Gulf Texas Shelf South zone (ft)','Emulates offshore part of Texas CS27 South zone. NGS definition in meters has FE=609630.480m FN=147679.410m.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',27.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-98.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9998844,'EPSG','9201','EPSG','8806','False easting',2000100.0,'EPSG','9002','EPSG','8807','False northing',484512.5,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','23630','conversion','EPSG','10974','EPSG','4807','EPSG','1294');
INSERT INTO "conversion" VALUES('EPSG','10975','SPCS2022 Gulf West zone (ft)','Emulates offshore deep water part of BLM zone 14N (US survey foot). NGS definition in meters has FE=500032.020m FN=-161.163m.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-99.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999584,'EPSG','9201','EPSG','8806','False easting',1640525.0,'EPSG','9002','EPSG','8807','False northing',-528.75,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','23631','conversion','EPSG','10975','EPSG','4810','EPSG','1294');
INSERT INTO "conversion" VALUES('EPSG','10976','SPCS2022 Gulf West Central zone (ft)','Emulates offshore deep water part of BLM zone 15N (US survey foot). NGS definition in meters has FE=500012.589m FN=-167.259m.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-93.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9995867,'EPSG','9201','EPSG','8806','False easting',1640461.25,'EPSG','9002','EPSG','8807','False northing',-548.75,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','23661','conversion','EPSG','10976','EPSG','4811','EPSG','1294');
INSERT INTO "conversion" VALUES('EPSG','10977','SPCS2022 Gulf East Central zone (ft)','Emulates offshore part of BLM zone 16N (US survey foot). NGS definition in meters has FE=499995.063m FN=-178.308m.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-87.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9995904,'EPSG','9201','EPSG','8806','False easting',1640403.75,'EPSG','9002','EPSG','8807','False northing',-585.0,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','23633','conversion','EPSG','10977','EPSG','4812','EPSG','1294');
INSERT INTO "conversion" VALUES('EPSG','10978','SPCS2022 Gulf East zone (ft)','Emulates offshore part of BLM zone 17N (US survey foot). NGS definition in meters has FE=499980.204m FN=-206.121m.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-81.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9995987,'EPSG','9201','EPSG','8806','False easting',1640355.0,'EPSG','9002','EPSG','8807','False northing',-676.25,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','23634','conversion','EPSG','10978','EPSG','2174','EPSG','1294');
INSERT INTO "conversion" VALUES('EPSG','11001','Georgia CS27 East zone','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',30.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-82.1,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11147','conversion','EPSG','11001','EPSG','2189','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','11002','Georgia CS27 West zone','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',30.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-84.1,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11148','conversion','EPSG','11002','EPSG','2190','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','11031','SPCS83 Georgia East zone (meter)','See code 15321 for equivalent non-metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',30.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-82.1,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11149','conversion','EPSG','11031','EPSG','2189','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','11032','SPCS83 Georgia West zone (meter)','See code 15322 for equivalent non-metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',30.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-84.1,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',700000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11150','conversion','EPSG','11032','EPSG','2190','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','11101','Idaho CS27 East zone','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',41.4,'EPSG','9110','EPSG','8802','Longitude of natural origin',-112.1,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999947368,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11151','conversion','EPSG','11101','EPSG','2192','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','11102','Idaho CS27 Central zone','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',41.4,'EPSG','9110','EPSG','8802','Longitude of natural origin',-114.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999947368,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11152','conversion','EPSG','11102','EPSG','2191','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','11103','Idaho CS27 West zone','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',41.4,'EPSG','9110','EPSG','8802','Longitude of natural origin',-115.45,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999933333,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11153','conversion','EPSG','11103','EPSG','2193','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','11131','SPCS83 Idaho East zone (meter)','See code 15323 for equivalent non-metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',41.4,'EPSG','9110','EPSG','8802','Longitude of natural origin',-112.1,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999947368,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11154','conversion','EPSG','11131','EPSG','2192','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','11132','SPCS83 Idaho Central zone (meter)','See code 15324 for equivalent non-metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',41.4,'EPSG','9110','EPSG','8802','Longitude of natural origin',-114.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999947368,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11155','conversion','EPSG','11132','EPSG','2191','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','11133','SPCS83 Idaho West zone (meter)','See code 15325 for equivalent non-metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',41.4,'EPSG','9110','EPSG','8802','Longitude of natural origin',-115.45,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999933333,'EPSG','9201','EPSG','8806','False easting',800000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11156','conversion','EPSG','11133','EPSG','2193','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','11201','Illinois CS27 East zone','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',36.4,'EPSG','9110','EPSG','8802','Longitude of natural origin',-88.2,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999975,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11157','conversion','EPSG','11201','EPSG','2194','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','11202','Illinois CS27 West zone','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',36.4,'EPSG','9110','EPSG','8802','Longitude of natural origin',-90.1,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999941177,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11158','conversion','EPSG','11202','EPSG','2195','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','11231','SPCS83 Illinois East zone (meter)','See code 15387 for equivalent non-metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',36.4,'EPSG','9110','EPSG','8802','Longitude of natural origin',-88.2,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999975,'EPSG','9201','EPSG','8806','False easting',300000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11159','conversion','EPSG','11231','EPSG','2194','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','11232','SPCS83 Illinois West zone (meter)','See code 15388 for equivalent non-metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',36.4,'EPSG','9110','EPSG','8802','Longitude of natural origin',-90.1,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999941177,'EPSG','9201','EPSG','8806','False easting',700000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11160','conversion','EPSG','11232','EPSG','2195','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','11233','Illinois Coordinate System of 1983 Aurora zone (US survey foot)','Part of the ICS83 33-zone system. Longitude of origin also given by information source as 271°30''E. State law defines grid unit as US survey foot. No equivalent metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',41.15,'EPSG','9110','EPSG','8802','Longitude of natural origin',-88.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00003,'EPSG','9201','EPSG','8806','False easting',3773000.0,'EPSG','9003','EPSG','8807','False northing',492000.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','19048','conversion','EPSG','11233','EPSG','4703','EPSG','1266');
INSERT INTO "conversion" VALUES('EPSG','11234','Illinois Coordinate System of 1983 Chicago zone (US survey foot)','Part of the ICS83 33-zone system. Longitude of origin also given by information source as 272°12''E. State law defines grid unit as US survey foot. No equivalent metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',41.15,'EPSG','9110','EPSG','8802','Longitude of natural origin',-87.48,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000023,'EPSG','9201','EPSG','8806','False easting',4757000.0,'EPSG','9003','EPSG','8807','False northing',492000.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','19049','conversion','EPSG','11234','EPSG','4704','EPSG','1266');
INSERT INTO "conversion" VALUES('EPSG','11235','Illinois Coordinate System of 1983 Moline zone (US survey foot)','Part of the ICS83 33-zone system. Longitude of origin also given by information source as 269°24''E. State law defines grid unit as US survey foot. No equivalent metric definition.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',41.33,'EPSG','9110','EPSG','8802','Longitude of natural origin',-90.36,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000024,'EPSG','9201','EPSG','8806','False easting',5741000.0,'EPSG','9003','EPSG','8807','False northing',755000.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','19050','conversion','EPSG','11235','EPSG','4705','EPSG','1266');
INSERT INTO "conversion" VALUES('EPSG','11236','Illinois Coordinate System of 1983 Sterling zone (US survey foot)','Part of the ICS83 33-zone system. Longitude of origin also given by information source as 269°57''E. State law defines grid unit as US survey foot. No equivalent metric definition.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',41.33,'EPSG','9110','EPSG','8802','Longitude of natural origin',-90.03,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00002,'EPSG','9201','EPSG','8806','False easting',6726000.0,'EPSG','9003','EPSG','8807','False northing',755000.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','19051','conversion','EPSG','11236','EPSG','4706','EPSG','1266');
INSERT INTO "conversion" VALUES('EPSG','11237','Illinois Coordinate System of 1983 Ottawa zone (US survey foot)','Part of the ICS83 33-zone system. Longitude of origin also given by information source as 270°57''E. State law defines grid unit as US survey foot. No equivalent metric definition.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',41.18,'EPSG','9110','EPSG','8802','Longitude of natural origin',-89.03,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000023,'EPSG','9201','EPSG','8806','False easting',7743000.0,'EPSG','9003','EPSG','8807','False northing',755000.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','19052','conversion','EPSG','11237','EPSG','4707','EPSG','1266');
INSERT INTO "conversion" VALUES('EPSG','11238','Illinois Coordinate System of 1983 Joliet zone (US survey foot)','Part of the ICS83 33-zone system. Longitude of origin also given by information source as 272°00''E. State law defines grid unit as US survey foot. No equivalent metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',40.33,'EPSG','9110','EPSG','8802','Longitude of natural origin',-88.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000022,'EPSG','9201','EPSG','8806','False easting',8694000.0,'EPSG','9003','EPSG','8807','False northing',492000.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','19053','conversion','EPSG','11238','EPSG','4708','EPSG','1266');
INSERT INTO "conversion" VALUES('EPSG','11239','Illinois Coordinate System of 1983 Monmouth zone (US survey foot)','Part of the ICS83 33-zone system. Longitude of origin also given by information source as 269°09''E. State law defines grid unit as US survey foot. No equivalent metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',40.12,'EPSG','9110','EPSG','8802','Longitude of natural origin',-90.51,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000024,'EPSG','9201','EPSG','8806','False easting',9678000.0,'EPSG','9003','EPSG','8807','False northing',492000.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','19054','conversion','EPSG','11239','EPSG','4709','EPSG','1266');
INSERT INTO "conversion" VALUES('EPSG','11240','Illinois Coordinate System of 1983 Galesburg zone (US survey foot)','Part of the ICS83 33-zone system. Longitude of origin also given by information source as 269°54''E. State law defines grid unit as US survey foot. No equivalent metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',37.15,'EPSG','9110','EPSG','8802','Longitude of natural origin',-90.06,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000023,'EPSG','9201','EPSG','8806','False easting',230000.0,'EPSG','9003','EPSG','8807','False northing',492000.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','19187','conversion','EPSG','11240','EPSG','4710','EPSG','1266');
INSERT INTO "conversion" VALUES('EPSG','11241','Illinois Coordinate System of 1983 Peoria zone (US survey foot)','Part of the ICS83 33-zone system. Longitude of origin also given by information source as 270°21''E. State law defines grid unit as US survey foot. No equivalent metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',37.24,'EPSG','9110','EPSG','8802','Longitude of natural origin',-89.39,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000023,'EPSG','9201','EPSG','8806','False easting',1378000.0,'EPSG','9003','EPSG','8807','False northing',622000.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','19056','conversion','EPSG','11241','EPSG','4711','EPSG','1266');
INSERT INTO "conversion" VALUES('EPSG','11242','Illinois Coordinate System of 1983 Eureka zone (US survey foot)','Part of the ICS83 33-zone system. Longitude of origin also given by information source as 270°42''E. State law defines grid unit as US survey foot. No equivalent metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',37.27,'EPSG','9110','EPSG','8802','Longitude of natural origin',-89.18,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000025,'EPSG','9201','EPSG','8806','False easting',2756000.0,'EPSG','9003','EPSG','8807','False northing',427000.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','19057','conversion','EPSG','11242','EPSG','4712','EPSG','1266');
INSERT INTO "conversion" VALUES('EPSG','11243','Illinois Coordinate System of 1983 Bloomington zone (US survey foot)','Part of the ICS83 33-zone system. Longitude of origin also given by information source as 271°09''E. State law defines grid unit as US survey foot. No equivalent metric definition.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',40.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-88.51,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000031,'EPSG','9201','EPSG','8806','False easting',3773000.0,'EPSG','9003','EPSG','8807','False northing',1739000.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','19058','conversion','EPSG','11243','EPSG','4713','EPSG','1266');
INSERT INTO "conversion" VALUES('EPSG','11244','Illinois Coordinate System of 1983 Pontiac zone (US survey foot)','Part of the ICS83 33-zone system. Longitude of origin also given by information source as 271°27''E. State law defines grid unit as US survey foot. No equivalent metric definition.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',40.54,'EPSG','9110','EPSG','8802','Longitude of natural origin',-88.33,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000025,'EPSG','9201','EPSG','8806','False easting',4757000.0,'EPSG','9003','EPSG','8807','False northing',1739000.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','19059','conversion','EPSG','11244','EPSG','4714','EPSG','1266');
INSERT INTO "conversion" VALUES('EPSG','11245','Illinois Coordinate System of 1983 Watseka zone (US survey foot)','Part of the ICS83 33-zone system. Longitude of origin also given by information source as 272°03''E. State law defines grid unit as US survey foot. No equivalent metric definition.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',40.45,'EPSG','9110','EPSG','8802','Longitude of natural origin',-87.57,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000024,'EPSG','9201','EPSG','8806','False easting',5741000.0,'EPSG','9003','EPSG','8807','False northing',1739000.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','19188','conversion','EPSG','11245','EPSG','4715','EPSG','1266');
INSERT INTO "conversion" VALUES('EPSG','11246','Illinois Coordinate System of 1983 Quincy zone (US survey foot)','Part of the ICS83 33-zone system. Longitude of origin also given by information source as 268°45''E. State law defines grid unit as US survey foot. No equivalent metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',36.45,'EPSG','9110','EPSG','8802','Longitude of natural origin',-91.15,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000023,'EPSG','9201','EPSG','8806','False easting',6726000.0,'EPSG','9003','EPSG','8807','False northing',492000.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','19061','conversion','EPSG','11246','EPSG','4716','EPSG','1266');
INSERT INTO "conversion" VALUES('EPSG','11247','Illinois Coordinate System of 1983 Macomb zone (US survey foot)','Part of the ICS83 33-zone system. Longitude of origin also given by information source as 269°24''E. State law defines grid unit as US survey foot. No equivalent metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',36.48,'EPSG','9110','EPSG','8802','Longitude of natural origin',-90.36,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000024,'EPSG','9201','EPSG','8806','False easting',7710000.0,'EPSG','9003','EPSG','8807','False northing',492000.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','19062','conversion','EPSG','11247','EPSG','4717','EPSG','1266');
INSERT INTO "conversion" VALUES('EPSG','11248','Illinois Coordinate System of 1983 Lincoln zone (US survey foot)','Part of the ICS83 33-zone system. Longitude of origin also given by information source as 270°12''E. State law defines grid unit as US survey foot. No equivalent metric definition.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',40.09,'EPSG','9110','EPSG','8802','Longitude of natural origin',-89.48,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000018,'EPSG','9201','EPSG','8806','False easting',8760000.0,'EPSG','9003','EPSG','8807','False northing',1739000.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','19063','conversion','EPSG','11248','EPSG','4718','EPSG','1266');
INSERT INTO "conversion" VALUES('EPSG','11249','Illinois Coordinate System of 1983 Decatur zone (US survey foot)','Part of the ICS83 33-zone system. Longitude of origin also given by information source as 271°12''E. State law defines grid unit as US survey foot. No equivalent metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',36.18,'EPSG','9110','EPSG','8802','Longitude of natural origin',-88.48,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000024,'EPSG','9201','EPSG','8806','False easting',9678000.0,'EPSG','9003','EPSG','8807','False northing',492000.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','19064','conversion','EPSG','11249','EPSG','4719','EPSG','1266');
INSERT INTO "conversion" VALUES('EPSG','11250','Illinois Coordinate System of 1983 Champaign zone (US survey foot)','Part of the ICS83 33-zone system. Longitude of origin also given by information source as 272°00''E. State law defines grid unit as US survey foot. No equivalent metric definition.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',40.09,'EPSG','9110','EPSG','8802','Longitude of natural origin',-88.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000026,'EPSG','9201','EPSG','8806','False easting',328000.0,'EPSG','9003','EPSG','8807','False northing',2822000.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','19065','conversion','EPSG','11250','EPSG','4720','EPSG','1266');
INSERT INTO "conversion" VALUES('EPSG','11251','Illinois Coordinate System of 1983 Jacksonville zone (US survey foot)','Part of the ICS83 33-zone system. Longitude of origin also given by information source as 269°24''E. State law defines grid unit as US survey foot. No equivalent metric definition.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',39.39,'EPSG','9110','EPSG','8802','Longitude of natural origin',-90.36,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000023,'EPSG','9201','EPSG','8806','False easting',1247000.0,'EPSG','9003','EPSG','8807','False northing',2822000.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','19066','conversion','EPSG','11251','EPSG','4721','EPSG','1266');
INSERT INTO "conversion" VALUES('EPSG','11252','Illinois Coordinate System of 1983 Springfield zone (US survey foot)','Part of the ICS83 33-zone system. Longitude of origin also given by information source as 270°21''E. State law defines grid unit as US survey foot. No equivalent metric definition.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',39.45,'EPSG','9110','EPSG','8802','Longitude of natural origin',-89.39,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000022,'EPSG','9201','EPSG','8806','False easting',2329000.0,'EPSG','9003','EPSG','8807','False northing',2887000.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','19067','conversion','EPSG','11252','EPSG','4722','EPSG','1266');
INSERT INTO "conversion" VALUES('EPSG','11253','Illinois Coordinate System of 1983 Charleston zone (US survey foot)','Part of the ICS83 33-zone system. Longitude of origin also given by information source as 272°00''E. State law defines grid unit as US survey foot. No equivalent metric definition.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',39.39,'EPSG','9110','EPSG','8802','Longitude of natural origin',-88.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000024,'EPSG','9201','EPSG','8806','False easting',3773000.0,'EPSG','9003','EPSG','8807','False northing',2756000.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','19068','conversion','EPSG','11253','EPSG','4723','EPSG','1266');
INSERT INTO "conversion" VALUES('EPSG','11254','Illinois Coordinate System of 1983 Jerseyville zone (US survey foot)','Part of the ICS83 33-zone system. Longitude of origin also given by information source as 269°30''E. State law defines grid unit as US survey foot. No equivalent metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',32.51,'EPSG','9110','EPSG','8802','Longitude of natural origin',-90.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000019,'EPSG','9201','EPSG','8806','False easting',4757000.0,'EPSG','9003','EPSG','8807','False northing',492000.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','19069','conversion','EPSG','11254','EPSG','4724','EPSG','1266');
INSERT INTO "conversion" VALUES('EPSG','11255','Illinois Coordinate System of 1983 Carlinville zone (US survey foot)','Part of the ICS83 33-zone system. Longitude of origin also given by information source as 269°51''E. State law defines grid unit as US survey foot. No equivalent metric definition.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',39.18,'EPSG','9110','EPSG','8802','Longitude of natural origin',-90.09,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00002,'EPSG','9201','EPSG','8806','False easting',5741000.0,'EPSG','9003','EPSG','8807','False northing',2756000.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','19070','conversion','EPSG','11255','EPSG','4725','EPSG','1266');
INSERT INTO "conversion" VALUES('EPSG','11256','Illinois Coordinate System of 1983 Taylorville zone (US survey foot)','Part of the ICS83 33-zone system. Longitude of origin also given by information source as 270°36''E. State law defines grid unit as US survey foot. No equivalent metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',33.09,'EPSG','9110','EPSG','8802','Longitude of natural origin',-89.24,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000023,'EPSG','9201','EPSG','8806','False easting',6726000.0,'EPSG','9003','EPSG','8807','False northing',492000.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','19071','conversion','EPSG','11256','EPSG','4726','EPSG','1266');
INSERT INTO "conversion" VALUES('EPSG','11257','Illinois Coordinate System of 1983 Effingham zone (US survey foot)','Part of the ICS83 33-zone system. Longitude of origin also given by information source as 271°00''E. State law defines grid unit as US survey foot. No equivalent metric definition.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',38.57,'EPSG','9110','EPSG','8802','Longitude of natural origin',-89.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000019,'EPSG','9201','EPSG','8806','False easting',7710000.0,'EPSG','9003','EPSG','8807','False northing',2756000.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','19072','conversion','EPSG','11257','EPSG','4727','EPSG','1266');
INSERT INTO "conversion" VALUES('EPSG','11258','Illinois Coordinate System of 1983 Robinson zone (US survey foot)','Part of the ICS83 33-zone system. Longitude of origin also given by information source as 272°00''E. State law defines grid unit as US survey foot. No equivalent metric definition.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',39.06,'EPSG','9110','EPSG','8802','Longitude of natural origin',-88.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000017,'EPSG','9201','EPSG','8806','False easting',8694000.0,'EPSG','9003','EPSG','8807','False northing',2756000.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','19073','conversion','EPSG','11258','EPSG','4728','EPSG','1266');
INSERT INTO "conversion" VALUES('EPSG','11259','Illinois Coordinate System of 1983 Belleville zone (US survey foot)','Part of the ICS83 33-zone system. Longitude of origin also given by information source as 270°00''E. State law defines grid unit as US survey foot. No equivalent metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',32.21,'EPSG','9110','EPSG','8802','Longitude of natural origin',-90.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000016,'EPSG','9201','EPSG','8806','False easting',9678000.0,'EPSG','9003','EPSG','8807','False northing',492000.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','19074','conversion','EPSG','11259','EPSG','4729','EPSG','1266');
INSERT INTO "conversion" VALUES('EPSG','11260','Illinois Coordinate System of 1983 Mount Vernon zone (US survey foot)','Part of the ICS83 33-zone system. Longitude of origin also given by information source as 270°51''E. State law defines grid unit as US survey foot. No equivalent metric definition.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',38.27,'EPSG','9110','EPSG','8802','Longitude of natural origin',-89.09,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000015,'EPSG','9201','EPSG','8806','False easting',328000.0,'EPSG','9003','EPSG','8807','False northing',3773000.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','19075','conversion','EPSG','11260','EPSG','4730','EPSG','1266');
INSERT INTO "conversion" VALUES('EPSG','11261','Illinois Coordinate System of 1983 Olney zone (US survey foot)','Part of the ICS83 33-zone system. Longitude of origin also given by information source as 271°51''E. State law defines grid unit as US survey foot. No equivalent metric definition.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',38.33,'EPSG','9110','EPSG','8802','Longitude of natural origin',-88.09,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000013,'EPSG','9201','EPSG','8806','False easting',1247000.0,'EPSG','9003','EPSG','8807','False northing',3773000.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','19076','conversion','EPSG','11261','EPSG','4731','EPSG','1266');
INSERT INTO "conversion" VALUES('EPSG','11262','Illinois Coordinate System of 1983 Carbondale zone (US survey foot)','Part of the ICS83 33-zone system. Longitude of origin also given by information source as 271°03''E. State law defines grid unit as US survey foot. No equivalent metric definition.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',37.54,'EPSG','9110','EPSG','8802','Longitude of natural origin',-88.57,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000012,'EPSG','9201','EPSG','8806','False easting',2395000.0,'EPSG','9003','EPSG','8807','False northing',3773000.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','19077','conversion','EPSG','11262','EPSG','4732','EPSG','1266');
INSERT INTO "conversion" VALUES('EPSG','11263','Illinois Coordinate System of 1983 Metropolis zone (US survey foot)','Part of the ICS83 33-zone system. Longitude of origin also given by information source as 271°06''E. State law defines grid unit as US survey foot. No equivalent metric definition.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',37.12,'EPSG','9110','EPSG','8802','Longitude of natural origin',-88.54,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00001,'EPSG','9201','EPSG','8806','False easting',3642000.0,'EPSG','9003','EPSG','8807','False northing',3839000.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','19078','conversion','EPSG','11263','EPSG','4733','EPSG','1266');
INSERT INTO "conversion" VALUES('EPSG','11264','Illinois Coordinate System of 1983 Freeport zone (US survey foot)','Part of the ICS83 33-zone system. Longitude of origin also given by information source as 270°03''E. State law defines grid unit as US survey foot. No equivalent metric definition.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',42.12,'EPSG','9110','EPSG','8802','Longitude of natural origin',-89.57,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000029,'EPSG','9201','EPSG','8806','False easting',1804000.0,'EPSG','9003','EPSG','8807','False northing',755000.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','19046','conversion','EPSG','11264','EPSG','4701','EPSG','1266');
INSERT INTO "conversion" VALUES('EPSG','11265','Illinois Coordinate System of 1983 Rockford zone (US survey foot)','Part of the ICS83 33-zone system. Longitude of origin also given by information source as 270°45''E. State law defines grid unit as US survey foot. No equivalent metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',41.15,'EPSG','9110','EPSG','8802','Longitude of natural origin',-89.15,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000029,'EPSG','9201','EPSG','8806','False easting',2822000.0,'EPSG','9003','EPSG','8807','False northing',492000.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','19047','conversion','EPSG','11265','EPSG','4702','EPSG','1266');
INSERT INTO "conversion" VALUES('EPSG','11301','Indiana CS27 East zone','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',37.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-85.4,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999966667,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11161','conversion','EPSG','11301','EPSG','2196','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','11302','Indiana CS27 West zone','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',37.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-87.05,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999966667,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11162','conversion','EPSG','11302','EPSG','2197','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','11331','SPCS83 Indiana East zone (meter)','See code 15372 for equivalent non-metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',37.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-85.4,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999966667,'EPSG','9201','EPSG','8806','False easting',100000.0,'EPSG','9001','EPSG','8807','False northing',250000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11163','conversion','EPSG','11331','EPSG','2196','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','11332','SPCS83 Indiana West zone (meter)','See code 15373 for equivalent non-metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',37.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-87.05,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999966667,'EPSG','9201','EPSG','8806','False easting',900000.0,'EPSG','9001','EPSG','8807','False northing',250000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11164','conversion','EPSG','11332','EPSG','2197','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','11401','Iowa CS27 North zone','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',41.3,'EPSG','9110','EPSG','8822','Longitude of false origin',-93.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',43.16,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',42.04,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11165','conversion','EPSG','11401','EPSG','2198','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','11402','Iowa CS27 South zone','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',40.0,'EPSG','9110','EPSG','8822','Longitude of false origin',-93.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',41.47,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',40.37,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11166','conversion','EPSG','11402','EPSG','2199','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','11431','SPCS83 Iowa North zone (meter)','See code 15377 for equivalent non-metric definition.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',41.3,'EPSG','9110','EPSG','8822','Longitude of false origin',-93.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',43.16,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',42.04,'EPSG','9110','EPSG','8826','Easting at false origin',1500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11167','conversion','EPSG','11431','EPSG','2198','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','11432','SPCS83 Iowa South zone (meter)','See code 15378 for equivalent non-metric definition.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',40.0,'EPSG','9110','EPSG','8822','Longitude of false origin',-93.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',41.47,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',40.37,'EPSG','9110','EPSG','8826','Easting at false origin',500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11168','conversion','EPSG','11432','EPSG','2199','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','11501','Kansas CS27 North zone','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',38.2,'EPSG','9110','EPSG','8822','Longitude of false origin',-98.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',39.47,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',38.43,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11169','conversion','EPSG','11501','EPSG','2200','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','11502','Kansas CS27 South zone','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',36.4,'EPSG','9110','EPSG','8822','Longitude of false origin',-98.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',38.34,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',37.16,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11170','conversion','EPSG','11502','EPSG','2201','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','11531','SPCS83 Kansas North zone (meter)','See code 15379 for equivalent non-metric definition.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',38.2,'EPSG','9110','EPSG','8822','Longitude of false origin',-98.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',39.47,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',38.43,'EPSG','9110','EPSG','8826','Easting at false origin',400000.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11171','conversion','EPSG','11531','EPSG','2200','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','11532','SPCS83 Kansas South zone (meter)','See code 15380 for equivalent non-metric definition.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',36.4,'EPSG','9110','EPSG','8822','Longitude of false origin',-98.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',38.34,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',37.16,'EPSG','9110','EPSG','8826','Easting at false origin',400000.0,'EPSG','9001','EPSG','8827','Northing at false origin',400000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11172','conversion','EPSG','11532','EPSG','2201','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','11601','Kentucky CS27 North zone','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',37.3,'EPSG','9110','EPSG','8822','Longitude of false origin',-84.15,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',37.58,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',38.58,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11173','conversion','EPSG','11601','EPSG','2202','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','11602','Kentucky CS27 South zone','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',36.2,'EPSG','9110','EPSG','8822','Longitude of false origin',-85.45,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',36.44,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',37.56,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11174','conversion','EPSG','11602','EPSG','2203','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','11630','SPCS83 Kentucky Single Zone (meter)','See code 15375 for equivalent non-metric definition.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',36.2,'EPSG','9110','EPSG','8822','Longitude of false origin',-85.45,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',37.05,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',38.4,'EPSG','9110','EPSG','8826','Easting at false origin',1500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11175','conversion','EPSG','11630','EPSG','1386','EPSG','1135');
INSERT INTO "conversion" VALUES('EPSG','11631','Kentucky CS83 North zone','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',37.3,'EPSG','9110','EPSG','8822','Longitude of false origin',-84.15,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',37.58,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',37.58,'EPSG','9110','EPSG','8826','Easting at false origin',500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11176','conversion','EPSG','11631','EPSG','2202','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','11632','SPCS83 Kentucky South zone (meter)','See code 15329 for equivalent non-metric definition.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',36.2,'EPSG','9110','EPSG','8822','Longitude of false origin',-85.45,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',37.56,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',36.44,'EPSG','9110','EPSG','8826','Easting at false origin',500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11177','conversion','EPSG','11632','EPSG','2203','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','11701','Louisiana CS27 North zone','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',30.4,'EPSG','9110','EPSG','8822','Longitude of false origin',-92.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',31.1,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',32.4,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11178','conversion','EPSG','11701','EPSG','2204','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','11702','Louisiana CS27 South zone','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',28.4,'EPSG','9110','EPSG','8822','Longitude of false origin',-91.2,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',29.18,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',30.42,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11179','conversion','EPSG','11702','EPSG','2205','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','11703','Louisiana CS27 Offshore zone','This projection is NOT used for oil industry purposes - use Louisiana CS27 Offshore zone (proj 11702) on shelf and BLM (proj 15915-16) in deep water protraction areas.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',25.4,'EPSG','9110','EPSG','8822','Longitude of false origin',-91.2,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',27.5,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',26.1,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11180','conversion','EPSG','11703','EPSG','4821','EPSG','1212');
INSERT INTO "conversion" VALUES('EPSG','11731','SPCS83 Louisiana North zone (meter)','See code 15391 for equivalent non-metric definition.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',30.3,'EPSG','9110','EPSG','8822','Longitude of false origin',-92.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',32.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',31.1,'EPSG','9110','EPSG','8826','Easting at false origin',1000000.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11181','conversion','EPSG','11731','EPSG','2204','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','11732','SPCS83 Louisiana South zone (meter)','See code 15392 for equivalent non-metric definition.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',28.3,'EPSG','9110','EPSG','8822','Longitude of false origin',-91.2,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',30.42,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',29.18,'EPSG','9110','EPSG','8826','Easting at false origin',1000000.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11182','conversion','EPSG','11732','EPSG','2529','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','11733','SPCS83 Louisiana Offshore zone (meter)','This projection is NOT used for oil industry purposes. See code 15393 for equivalent non-metric definition.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',25.3,'EPSG','9110','EPSG','8822','Longitude of false origin',-91.2,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',27.5,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',26.1,'EPSG','9110','EPSG','8826','Easting at false origin',1000000.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11183','conversion','EPSG','11733','EPSG','4821','EPSG','1212');
INSERT INTO "conversion" VALUES('EPSG','11801','Maine CS27 East zone','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',43.5,'EPSG','9110','EPSG','8802','Longitude of natural origin',-68.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11184','conversion','EPSG','11801','EPSG','2206','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','11802','Maine CS27 West zone','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',42.5,'EPSG','9110','EPSG','8802','Longitude of natural origin',-70.1,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999966667,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11185','conversion','EPSG','11802','EPSG','2207','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','11831','SPCS83 Maine East zone (meter)','See code 11833 for equivalent non-metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',43.4,'EPSG','9110','EPSG','8802','Longitude of natural origin',-68.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',300000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11186','conversion','EPSG','11831','EPSG','2206','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','11832','SPCS83 Maine West zone (meter)','See code 11834 for equivalent non-metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',42.5,'EPSG','9110','EPSG','8802','Longitude of natural origin',-70.1,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999966667,'EPSG','9201','EPSG','8806','False easting',900000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11187','conversion','EPSG','11832','EPSG','2207','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','11833','SPCS83 Maine East zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 11831.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',43.4,'EPSG','9110','EPSG','8802','Longitude of natural origin',-68.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',984250.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11188','conversion','EPSG','11833','EPSG','2206','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','11834','SPCS83 Maine West zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 11832.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',42.5,'EPSG','9110','EPSG','8802','Longitude of natural origin',-70.1,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999966667,'EPSG','9201','EPSG','8806','False easting',2952750.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11189','conversion','EPSG','11834','EPSG','2207','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','11851','Maine CS2000 East zone (meter)','In Maine Department of Transportation and other State agencies replaces CS27 and SPCS83 from 1/1/2001.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',43.5,'EPSG','9110','EPSG','8802','Longitude of natural origin',-67.523,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.99998,'EPSG','9201','EPSG','8806','False easting',700000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11190','conversion','EPSG','11851','EPSG','2960','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','11852','Maine CS2000 Central zone','Supersedes CS27 and CS83 from 1/1/2001.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',43.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-69.073,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.99998,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11191','conversion','EPSG','11852','EPSG','2959','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','11853','Maine CS2000 West zone (meter)','In Maine Department of Transportation and other State agencies replaces CS27 and SPCS83 from 1/1/2001.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',42.5,'EPSG','9110','EPSG','8802','Longitude of natural origin',-70.223,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.99998,'EPSG','9201','EPSG','8806','False easting',300000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11192','conversion','EPSG','11853','EPSG','2958','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','11854','Maine CS2000 Central zone (meter)','In Maine Department of Transportation and other State agencies replaces CS27 and SPCS83 from 1/1/2001.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',43.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-69.073,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.99998,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11193','conversion','EPSG','11854','EPSG','2959','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','11900','Maryland CS27','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',37.5,'EPSG','9110','EPSG','8822','Longitude of false origin',-77.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',38.18,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',39.27,'EPSG','9110','EPSG','8826','Easting at false origin',800000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11194','conversion','EPSG','11900','EPSG','1389','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','11930','SPCS83 Maryland zone (meter)','See code 15330 for equivalent non-metric definition.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',37.4,'EPSG','9110','EPSG','8822','Longitude of false origin',-77.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',39.27,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',38.18,'EPSG','9110','EPSG','8826','Easting at false origin',400000.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11195','conversion','EPSG','11930','EPSG','1389','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','12001','Massachusetts CS27 Mainland zone','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',41.0,'EPSG','9110','EPSG','8822','Longitude of false origin',-71.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',41.43,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',42.41,'EPSG','9110','EPSG','8826','Easting at false origin',600000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11196','conversion','EPSG','12001','EPSG','2209','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','12002','Massachusetts CS27 Island zone','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',41.0,'EPSG','9110','EPSG','8822','Longitude of false origin',-70.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',41.17,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',41.29,'EPSG','9110','EPSG','8826','Easting at false origin',200000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11197','conversion','EPSG','12002','EPSG','2208','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','12031','SPCS83 Massachusetts Mainland zone (meter)','See code 15331 for equivalent non-metric definition.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',41.0,'EPSG','9110','EPSG','8822','Longitude of false origin',-71.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',42.41,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',41.43,'EPSG','9110','EPSG','8826','Easting at false origin',200000.0,'EPSG','9001','EPSG','8827','Northing at false origin',750000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11198','conversion','EPSG','12031','EPSG','2209','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','12032','SPCS83 Massachusetts Island zone (meter)','See code 15332 for equivalent non-metric definition.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',41.0,'EPSG','9110','EPSG','8822','Longitude of false origin',-70.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',41.29,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',41.17,'EPSG','9110','EPSG','8826','Easting at false origin',500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11199','conversion','EPSG','12032','EPSG','2208','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','12101','Michigan State Plane East zone','Replaced by central and south zones.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',41.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-83.4,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999942857,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11200','conversion','EPSG','12101','EPSG','1720','EPSG','1215');
INSERT INTO "conversion" VALUES('EPSG','12102','Michigan State Plane Old Central zone','Replaced by central and south zones.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',41.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-85.45,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999909091,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11201','conversion','EPSG','12102','EPSG','1721','EPSG','1215');
INSERT INTO "conversion" VALUES('EPSG','12103','Michigan State Plane West zone','Replaced by North zone.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',41.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-88.45,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999909091,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11202','conversion','EPSG','12103','EPSG','3652','EPSG','1215');
INSERT INTO "conversion" VALUES('EPSG','12111','Michigan CS27 North zone','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',44.47,'EPSG','9110','EPSG','8822','Longitude of false origin',-87.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',45.29,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',47.05,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11203','conversion','EPSG','12111','EPSG','1723','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','12112','Michigan CS27 Central zone','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',43.19,'EPSG','9110','EPSG','8822','Longitude of false origin',-84.2,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',44.11,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',45.42,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11204','conversion','EPSG','12112','EPSG','1724','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','12113','Michigan CS27 South zone','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',41.3,'EPSG','9110','EPSG','8822','Longitude of false origin',-84.2,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',42.06,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',43.4,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11205','conversion','EPSG','12113','EPSG','1725','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','12141','SPCS83 Michigan North zone (meter)','See code 15333 for equivalent non-metric definition.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',44.47,'EPSG','9110','EPSG','8822','Longitude of false origin',-87.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',47.05,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',45.29,'EPSG','9110','EPSG','8826','Easting at false origin',8000000.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11206','conversion','EPSG','12141','EPSG','1723','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','12142','SPCS83 Michigan Central zone (meter)','See code 15334 for equivalent non-metric definition.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',43.19,'EPSG','9110','EPSG','8822','Longitude of false origin',-84.22,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',45.42,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',44.11,'EPSG','9110','EPSG','8826','Easting at false origin',6000000.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11207','conversion','EPSG','12142','EPSG','1724','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','12143','SPCS83 Michigan South zone (meter)','See code 15335 for equivalent non-metric definition.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',41.3,'EPSG','9110','EPSG','8822','Longitude of false origin',-84.22,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',43.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',42.06,'EPSG','9110','EPSG','8826','Easting at false origin',4000000.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11208','conversion','EPSG','12143','EPSG','1725','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','12150','Michigan Oblique Mercator (meter)','If using Hotine Oblique Mercator (variant B) method (code 9815), Ec=499840.252 m, Nc=528600.303 m.','EPSG','9812','Hotine Oblique Mercator (variant A)','EPSG','8811','Latitude of projection centre',45.1833,'EPSG','9110','EPSG','8812','Longitude of projection centre',-86.0,'EPSG','9110','EPSG','8813','Azimuth at projection centre',337.25556,'EPSG','9102','EPSG','8814','Angle from Rectified to Skew Grid',337.25556,'EPSG','9102','EPSG','8815','Scale factor at projection centre',0.9996,'EPSG','9201','EPSG','8806','False easting',2546731.496,'EPSG','9001','EPSG','8807','False northing',-4354009.816,'EPSG','9001',0);
INSERT INTO "usage" VALUES('EPSG','11209','conversion','EPSG','12150','EPSG','1391','EPSG','1135');
INSERT INTO "conversion" VALUES('EPSG','12201','Minnesota CS27 North zone','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',46.3,'EPSG','9110','EPSG','8822','Longitude of false origin',-93.06,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',47.02,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',48.38,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11210','conversion','EPSG','12201','EPSG','2214','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','12202','Minnesota CS27 Central zone','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',45.0,'EPSG','9110','EPSG','8822','Longitude of false origin',-94.15,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',45.37,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',47.03,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11211','conversion','EPSG','12202','EPSG','2213','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','12203','Minnesota CS27 South zone','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',43.0,'EPSG','9110','EPSG','8822','Longitude of false origin',-94.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',43.47,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',45.13,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11212','conversion','EPSG','12203','EPSG','2215','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','12231','SPCS83 Minnesota North zone (meter)','See code 12234 for equivalent non-metric definition.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',46.3,'EPSG','9110','EPSG','8822','Longitude of false origin',-93.06,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',48.38,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',47.02,'EPSG','9110','EPSG','8826','Easting at false origin',800000.0,'EPSG','9001','EPSG','8827','Northing at false origin',100000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11213','conversion','EPSG','12231','EPSG','2214','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','12232','SPCS83 Minnesota Central zone (meter)','See code 12235 for equivalent non-metric definition.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',45.0,'EPSG','9110','EPSG','8822','Longitude of false origin',-94.15,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',47.03,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',45.37,'EPSG','9110','EPSG','8826','Easting at false origin',800000.0,'EPSG','9001','EPSG','8827','Northing at false origin',100000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11214','conversion','EPSG','12232','EPSG','2213','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','12233','SPCS83 Minnesota South zone (meter)','See code 12236 for equivalent non-metric definition.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',43.0,'EPSG','9110','EPSG','8822','Longitude of false origin',-94.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',45.13,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',43.47,'EPSG','9110','EPSG','8826','Easting at false origin',800000.0,'EPSG','9001','EPSG','8827','Northing at false origin',100000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11215','conversion','EPSG','12233','EPSG','2215','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','12234','SPCS83 Minnesota North zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 12231.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',46.3,'EPSG','9110','EPSG','8822','Longitude of false origin',-93.06,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',48.38,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',47.02,'EPSG','9110','EPSG','8826','Easting at false origin',2624666.6667,'EPSG','9003','EPSG','8827','Northing at false origin',328083.3333,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11216','conversion','EPSG','12234','EPSG','2214','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','12235','SPCS83 Minnesota Central zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 12232.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',45.0,'EPSG','9110','EPSG','8822','Longitude of false origin',-94.15,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',47.03,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',45.37,'EPSG','9110','EPSG','8826','Easting at false origin',2624666.6667,'EPSG','9003','EPSG','8827','Northing at false origin',328083.3333,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11217','conversion','EPSG','12235','EPSG','2213','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','12236','SPCS83 Minnesota South zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 12233.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',43.0,'EPSG','9110','EPSG','8822','Longitude of false origin',-94.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',45.13,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',43.47,'EPSG','9110','EPSG','8826','Easting at false origin',2624666.6667,'EPSG','9003','EPSG','8827','Northing at false origin',328083.3333,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11218','conversion','EPSG','12236','EPSG','2215','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','12301','Mississippi CS27 East zone','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',29.4,'EPSG','9110','EPSG','8802','Longitude of natural origin',-88.5,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.99996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11219','conversion','EPSG','12301','EPSG','2216','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','12302','Mississippi CS27 West zone','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',30.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-90.2,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999941177,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11220','conversion','EPSG','12302','EPSG','2217','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','12331','SPCS83 Mississippi East zone (meter)','See code 15336 for equivalent non-metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',29.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-88.5,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.99995,'EPSG','9201','EPSG','8806','False easting',300000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11221','conversion','EPSG','12331','EPSG','2216','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','12332','SPCS83 Mississippi West zone (meter)','See code 15337 for equivalent non-metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',29.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-90.2,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.99995,'EPSG','9201','EPSG','8806','False easting',700000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11222','conversion','EPSG','12332','EPSG','2217','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','12401','Missouri CS27 East zone','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',35.5,'EPSG','9110','EPSG','8802','Longitude of natural origin',-90.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999933333,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11223','conversion','EPSG','12401','EPSG','2219','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','12402','Missouri CS27 Central zone','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',35.5,'EPSG','9110','EPSG','8802','Longitude of natural origin',-92.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999933333,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11224','conversion','EPSG','12402','EPSG','2218','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','12403','Missouri CS27 West zone','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',36.1,'EPSG','9110','EPSG','8802','Longitude of natural origin',-94.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999941177,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11225','conversion','EPSG','12403','EPSG','2220','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','12431','SPCS83 Missouri East zone (meter)','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',35.5,'EPSG','9110','EPSG','8802','Longitude of natural origin',-90.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999933333,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11226','conversion','EPSG','12431','EPSG','2219','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','12432','SPCS83 Missouri Central zone (meter)','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',35.5,'EPSG','9110','EPSG','8802','Longitude of natural origin',-92.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999933333,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11227','conversion','EPSG','12432','EPSG','2218','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','12433','SPCS83 Missouri West zone (meter)','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',36.1,'EPSG','9110','EPSG','8802','Longitude of natural origin',-94.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999941177,'EPSG','9201','EPSG','8806','False easting',850000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11228','conversion','EPSG','12433','EPSG','2220','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','12501','Montana CS27 North zone','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',47.0,'EPSG','9110','EPSG','8822','Longitude of false origin',-109.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',48.43,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',47.51,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11229','conversion','EPSG','12501','EPSG','2211','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','12502','Montana CS27 Central zone','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',45.5,'EPSG','9110','EPSG','8822','Longitude of false origin',-109.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',47.53,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',46.27,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11230','conversion','EPSG','12502','EPSG','2210','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','12503','Montana CS27 South zone','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',44.0,'EPSG','9110','EPSG','8822','Longitude of false origin',-109.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',46.24,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',44.52,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11231','conversion','EPSG','12503','EPSG','2212','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','12530','SPCS83 Montana zone (meter)','See code 15338 for equivalent non-metric definition.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',44.15,'EPSG','9110','EPSG','8822','Longitude of false origin',-109.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',49.0,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',45.0,'EPSG','9110','EPSG','8826','Easting at false origin',600000.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11232','conversion','EPSG','12530','EPSG','1395','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','12601','Nebraska CS27 North zone','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',41.2,'EPSG','9110','EPSG','8822','Longitude of false origin',-100.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',41.51,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',42.49,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11233','conversion','EPSG','12601','EPSG','2221','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','12602','Nebraska CS27 South zone','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',39.4,'EPSG','9110','EPSG','8822','Longitude of false origin',-99.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',40.17,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',41.43,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11234','conversion','EPSG','12602','EPSG','2222','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','12630','SPCS83 Nebraska zone (meter)','See code 15396 for equivalent non-metric definition.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',39.5,'EPSG','9110','EPSG','8822','Longitude of false origin',-100.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',43.0,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',40.0,'EPSG','9110','EPSG','8826','Easting at false origin',500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11235','conversion','EPSG','12630','EPSG','1396','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','12701','Nevada CS27 East zone','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',34.45,'EPSG','9110','EPSG','8802','Longitude of natural origin',-115.35,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11236','conversion','EPSG','12701','EPSG','2224','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','12702','Nevada CS27 Central zone','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',34.45,'EPSG','9110','EPSG','8802','Longitude of natural origin',-116.4,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11237','conversion','EPSG','12702','EPSG','2223','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','12703','Nevada CS27 West zone','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',34.45,'EPSG','9110','EPSG','8802','Longitude of natural origin',-118.35,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11238','conversion','EPSG','12703','EPSG','2225','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','12731','SPCS83 Nevada East zone (meter)','See code 15381 for equivalent non-metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',34.45,'EPSG','9110','EPSG','8802','Longitude of natural origin',-115.35,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',8000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11239','conversion','EPSG','12731','EPSG','2224','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','12732','SPCS83 Nevada Central zone (meter)','See code 15382 for equivalent non-metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',34.45,'EPSG','9110','EPSG','8802','Longitude of natural origin',-116.4,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',6000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11240','conversion','EPSG','12732','EPSG','2223','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','12733','SPCS83 Nevada West zone (meter)','See code 15383 for equivalent non-metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',34.45,'EPSG','9110','EPSG','8802','Longitude of natural origin',-118.35,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',800000.0,'EPSG','9001','EPSG','8807','False northing',4000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11241','conversion','EPSG','12733','EPSG','2225','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','12800','New Hampshire CS27','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',42.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-71.4,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999966667,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11242','conversion','EPSG','12800','EPSG','1398','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','12830','SPCS83 New Hampshire zone (meter)','See code 15389 for equivalent non-metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',42.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-71.4,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999966667,'EPSG','9201','EPSG','8806','False easting',300000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11243','conversion','EPSG','12830','EPSG','1398','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','12900','New Jersey CS27','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',38.5,'EPSG','9110','EPSG','8802','Longitude of natural origin',-74.4,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999975,'EPSG','9201','EPSG','8806','False easting',2000000.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11244','conversion','EPSG','12900','EPSG','1399','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','12930','SPCS83 New Jersey zone (meter)','See code 15384 for equivalent non-metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',38.5,'EPSG','9110','EPSG','8802','Longitude of natural origin',-74.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',150000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11245','conversion','EPSG','12930','EPSG','1399','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','13001','New Mexico CS27 East zone','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',31.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-104.2,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999909091,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11246','conversion','EPSG','13001','EPSG','2228','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','13002','New Mexico CS27 Central zone','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',31.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-106.15,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11247','conversion','EPSG','13002','EPSG','2229','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','13003','New Mexico CS27 West zone','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',31.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-107.5,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999916667,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11248','conversion','EPSG','13003','EPSG','2230','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','13031','SPCS83 New Mexico East zone (meter)','See code 15339 for equivalent non-metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',31.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-104.2,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999909091,'EPSG','9201','EPSG','8806','False easting',165000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11249','conversion','EPSG','13031','EPSG','2228','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','13032','SPCS83 New Mexico Central zone (meter)','See code 15340 for equivalent non-metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',31.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-106.15,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11250','conversion','EPSG','13032','EPSG','2231','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','13033','SPCS83 New Mexico West zone (meter)','See code 15341 for equivalent non-metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',31.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-107.5,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999916667,'EPSG','9201','EPSG','8806','False easting',830000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11251','conversion','EPSG','13033','EPSG','2232','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','13101','New York CS27 East zone','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',40.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-74.2,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999966667,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11252','conversion','EPSG','13101','EPSG','2234','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','13102','New York CS27 Central zone','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',40.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-76.35,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999375,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11253','conversion','EPSG','13102','EPSG','2233','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','13103','New York CS27 West zone','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',40.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-78.35,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999375,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11254','conversion','EPSG','13103','EPSG','2236','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','13104','New York CS27 Long Island zone','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',40.3,'EPSG','9110','EPSG','8822','Longitude of false origin',-74.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',41.02,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',40.4,'EPSG','9110','EPSG','8826','Easting at false origin',1000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11255','conversion','EPSG','13104','EPSG','2235','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','13131','SPCS83 New York East zone (meter)','See code 15342 for equivalent non-metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',38.5,'EPSG','9110','EPSG','8802','Longitude of natural origin',-74.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',150000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11256','conversion','EPSG','13131','EPSG','2234','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','13132','SPCS83 New York Central zone (meter)','See code 15343 for equivalent non-metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',40.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-76.35,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999375,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11257','conversion','EPSG','13132','EPSG','2233','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','13133','SPCS83 New York West zone (meter)','See code 15344 for equivalent non-metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',40.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-78.35,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999375,'EPSG','9201','EPSG','8806','False easting',350000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11258','conversion','EPSG','13133','EPSG','2236','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','13134','SPCS83 New York Long Island zone (meter)','See code 15345 for equivalent non-metric definition.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',40.1,'EPSG','9110','EPSG','8822','Longitude of false origin',-74.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',41.02,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',40.4,'EPSG','9110','EPSG','8826','Easting at false origin',300000.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11259','conversion','EPSG','13134','EPSG','2235','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','13200','North Carolina CS27','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',33.45,'EPSG','9110','EPSG','8822','Longitude of false origin',-79.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',34.2,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',36.1,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11260','conversion','EPSG','13200','EPSG','1402','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','13230','SPCS83 North Carolina zone (meter)','See code 15346 for equivalent non-metric definition.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',33.45,'EPSG','9110','EPSG','8822','Longitude of false origin',-79.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',36.1,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',34.2,'EPSG','9110','EPSG','8826','Easting at false origin',609601.22,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11261','conversion','EPSG','13230','EPSG','1402','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','13301','North Dakota CS27 North zone','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',47.0,'EPSG','9110','EPSG','8822','Longitude of false origin',-100.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',47.26,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',48.44,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11262','conversion','EPSG','13301','EPSG','2237','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','13302','North Dakota CS27 South zone','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',45.4,'EPSG','9110','EPSG','8822','Longitude of false origin',-100.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',46.11,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',47.29,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11263','conversion','EPSG','13302','EPSG','2238','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','13331','SPCS83 North Dakota North zone (meter)','See code 15347 for equivalent non-metric definition.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',47.0,'EPSG','9110','EPSG','8822','Longitude of false origin',-100.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',48.44,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',47.26,'EPSG','9110','EPSG','8826','Easting at false origin',600000.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11264','conversion','EPSG','13331','EPSG','2237','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','13332','SPCS83 North Dakota South zone (meter)','See code 15348 for equivalent non-metric definition.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',45.4,'EPSG','9110','EPSG','8822','Longitude of false origin',-100.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',47.29,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',46.11,'EPSG','9110','EPSG','8826','Easting at false origin',600000.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11265','conversion','EPSG','13332','EPSG','2238','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','13401','Ohio CS27 North zone','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',39.4,'EPSG','9110','EPSG','8822','Longitude of false origin',-82.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',40.26,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',41.42,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11266','conversion','EPSG','13401','EPSG','2239','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','13402','Ohio CS27 South zone','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',38.0,'EPSG','9110','EPSG','8822','Longitude of false origin',-82.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',38.44,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',40.02,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11267','conversion','EPSG','13402','EPSG','2240','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','13431','SPCS83 Ohio North zone (meter)','See code 13433 for equivalent non-metric definition.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',39.4,'EPSG','9110','EPSG','8822','Longitude of false origin',-82.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',41.42,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',40.26,'EPSG','9110','EPSG','8826','Easting at false origin',600000.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11268','conversion','EPSG','13431','EPSG','2239','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','13432','SPCS83 Ohio South zone (meter)','See code 13434 for equivalent non-metric definition.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',38.0,'EPSG','9110','EPSG','8822','Longitude of false origin',-82.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',40.02,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',38.44,'EPSG','9110','EPSG','8826','Easting at false origin',600000.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11269','conversion','EPSG','13432','EPSG','2240','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','13433','SPCS83 Ohio North zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 13431.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',39.4,'EPSG','9110','EPSG','8822','Longitude of false origin',-82.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',41.42,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',40.26,'EPSG','9110','EPSG','8826','Easting at false origin',1968500.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11270','conversion','EPSG','13433','EPSG','2239','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','13434','SPCS83 Ohio South zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 13432.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',38.0,'EPSG','9110','EPSG','8822','Longitude of false origin',-82.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',40.02,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',38.44,'EPSG','9110','EPSG','8826','Easting at false origin',1968500.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11271','conversion','EPSG','13434','EPSG','2240','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','13501','Oklahoma CS27 North zone','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',35.0,'EPSG','9110','EPSG','8822','Longitude of false origin',-98.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',35.34,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',36.46,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11272','conversion','EPSG','13501','EPSG','2241','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','13502','Oklahoma CS27 South zone','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',33.2,'EPSG','9110','EPSG','8822','Longitude of false origin',-98.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',33.56,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',35.14,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11273','conversion','EPSG','13502','EPSG','2242','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','13531','SPCS83 Oklahoma North zone (meter)','See code 15349 for equivalent non-metric definition.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',35.0,'EPSG','9110','EPSG','8822','Longitude of false origin',-98.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',36.46,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',35.34,'EPSG','9110','EPSG','8826','Easting at false origin',600000.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11274','conversion','EPSG','13531','EPSG','2241','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','13532','SPCS83 Oklahoma South zone (meter)','See code 15350 for equivalent non-metric definition.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',33.2,'EPSG','9110','EPSG','8822','Longitude of false origin',-98.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',35.14,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',33.56,'EPSG','9110','EPSG','8826','Easting at false origin',600000.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11275','conversion','EPSG','13532','EPSG','2242','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','13601','Oregon CS27 North zone','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',43.4,'EPSG','9110','EPSG','8822','Longitude of false origin',-120.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',44.2,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',46.0,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11276','conversion','EPSG','13601','EPSG','2243','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','13602','Oregon CS27 South zone','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',41.4,'EPSG','9110','EPSG','8822','Longitude of false origin',-120.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',42.2,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',44.0,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11277','conversion','EPSG','13602','EPSG','2244','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','13631','SPCS83 Oregon North zone (meter)','See code 15351 for equivalent non-metric definition.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',43.4,'EPSG','9110','EPSG','8822','Longitude of false origin',-120.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',46.0,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',44.2,'EPSG','9110','EPSG','8826','Easting at false origin',2500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11278','conversion','EPSG','13631','EPSG','2243','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','13632','SPCS83 Oregon South zone (meter)','See code 15352 for equivalent non-metric definition.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',41.4,'EPSG','9110','EPSG','8822','Longitude of false origin',-120.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',44.0,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',42.2,'EPSG','9110','EPSG','8826','Easting at false origin',1500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11279','conversion','EPSG','13632','EPSG','2244','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','13633','Oregon Lambert (meter)','Initial metric definition of projection 15374. This projection is not used in metric form by state agencies. See proj code 15375 for equivalent non-metric definition recommended by Oregon Geographic Information Council (GIC).','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',41.45,'EPSG','9110','EPSG','8822','Longitude of false origin',-120.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',43.0,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',45.3,'EPSG','9110','EPSG','8826','Easting at false origin',400000.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11280','conversion','EPSG','13633','EPSG','1406','EPSG','1135');
INSERT INTO "conversion" VALUES('EPSG','13701','Pennsylvania CS27 North zone','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',40.1,'EPSG','9110','EPSG','8822','Longitude of false origin',-77.45,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',40.53,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',41.57,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11281','conversion','EPSG','13701','EPSG','2245','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','13702','Pennsylvania CS27 South zone','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',39.2,'EPSG','9110','EPSG','8822','Longitude of false origin',-77.45,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',39.56,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',40.48,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11282','conversion','EPSG','13702','EPSG','2246','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','13731','SPCS83 Pennsylvania North zone (meter)','See code 15353 for equivalent non-metric definition.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',40.1,'EPSG','9110','EPSG','8822','Longitude of false origin',-77.45,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',41.57,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',40.53,'EPSG','9110','EPSG','8826','Easting at false origin',600000.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11283','conversion','EPSG','13731','EPSG','2245','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','13732','SPCS83 Pennsylvania South zone (meter)','See code 15354 for equivalent non-metric definition.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',39.2,'EPSG','9110','EPSG','8822','Longitude of false origin',-77.45,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',40.58,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',39.56,'EPSG','9110','EPSG','8826','Easting at false origin',600000.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11284','conversion','EPSG','13732','EPSG','2246','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','13800','Rhode Island CS27','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',41.05,'EPSG','9110','EPSG','8802','Longitude of natural origin',-71.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999938,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11285','conversion','EPSG','13800','EPSG','1408','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','13830','SPCS83 Rhode Island zone (meter)','See code 15390 for equivalent non-metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',41.05,'EPSG','9110','EPSG','8802','Longitude of natural origin',-71.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.99999375,'EPSG','9201','EPSG','8806','False easting',100000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11286','conversion','EPSG','13830','EPSG','1408','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','13901','South Carolina CS27 North zone','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',33.0,'EPSG','9110','EPSG','8822','Longitude of false origin',-81.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',33.46,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',34.58,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11287','conversion','EPSG','13901','EPSG','2247','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','13902','South Carolina CS27 South zone','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',31.5,'EPSG','9110','EPSG','8822','Longitude of false origin',-81.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',32.2,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',33.4,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11288','conversion','EPSG','13902','EPSG','2248','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','13930','SPCS83 South Carolina zone (meter)','See code 15355 for equivalent non-metric definition.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',31.5,'EPSG','9110','EPSG','8822','Longitude of false origin',-81.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',34.5,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',32.3,'EPSG','9110','EPSG','8826','Easting at false origin',609600.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11289','conversion','EPSG','13930','EPSG','1409','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','14001','South Dakota CS27 North zone','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',43.5,'EPSG','9110','EPSG','8822','Longitude of false origin',-100.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',44.25,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',45.41,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11290','conversion','EPSG','14001','EPSG','2249','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','14002','South Dakota CS27 South zone','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',42.2,'EPSG','9110','EPSG','8822','Longitude of false origin',-100.2,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',42.5,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',44.24,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11291','conversion','EPSG','14002','EPSG','2250','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','14031','SPCS83 South Dakota North zone (meter)','See code 15394 for equivalent non-metric definition.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',43.5,'EPSG','9110','EPSG','8822','Longitude of false origin',-100.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',45.41,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',44.25,'EPSG','9110','EPSG','8826','Easting at false origin',600000.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11292','conversion','EPSG','14031','EPSG','2249','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','14032','SPCS83 South Dakota South zone (meter)','See code 15395 for equivalent non-metric definition.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',42.2,'EPSG','9110','EPSG','8822','Longitude of false origin',-100.2,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',44.24,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',42.5,'EPSG','9110','EPSG','8826','Easting at false origin',600000.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11293','conversion','EPSG','14032','EPSG','2250','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','14100','Tennessee CS27','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',34.4,'EPSG','9110','EPSG','8822','Longitude of false origin',-86.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',35.15,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',36.25,'EPSG','9110','EPSG','8826','Easting at false origin',100000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11294','conversion','EPSG','14100','EPSG','1411','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','14130','SPCS83 Tennessee zone (meter)','See code 15356 for equivalent non-metric definition.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',34.2,'EPSG','9110','EPSG','8822','Longitude of false origin',-86.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',36.25,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',35.15,'EPSG','9110','EPSG','8826','Easting at false origin',600000.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11295','conversion','EPSG','14130','EPSG','1411','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','14201','Texas CS27 North zone','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',34.0,'EPSG','9110','EPSG','8822','Longitude of false origin',-101.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',34.39,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',36.11,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11296','conversion','EPSG','14201','EPSG','2253','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','14202','Texas CS27 North Central zone','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',31.4,'EPSG','9110','EPSG','8822','Longitude of false origin',-97.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',32.08,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',33.58,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11297','conversion','EPSG','14202','EPSG','2254','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','14203','Texas CS27 Central zone','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',29.4,'EPSG','9110','EPSG','8822','Longitude of false origin',-100.2,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',30.07,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',31.53,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11298','conversion','EPSG','14203','EPSG','2252','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','14204','Texas CS27 South Central zone','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',27.5,'EPSG','9110','EPSG','8822','Longitude of false origin',-99.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',28.23,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',30.17,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11299','conversion','EPSG','14204','EPSG','2256','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','14205','Texas CS27 South zone','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',25.4,'EPSG','9110','EPSG','8822','Longitude of false origin',-98.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',26.1,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',27.5,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11300','conversion','EPSG','14205','EPSG','2255','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','14231','SPCS83 Texas North zone (meter)','See code 15357 for equivalent non-metric definition.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',34.0,'EPSG','9110','EPSG','8822','Longitude of false origin',-101.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',36.11,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',34.39,'EPSG','9110','EPSG','8826','Easting at false origin',200000.0,'EPSG','9001','EPSG','8827','Northing at false origin',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11301','conversion','EPSG','14231','EPSG','2253','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','14232','SPCS83 Texas North Central zone (meter)','See code 15358 for equivalent non-metric definition.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',31.4,'EPSG','9110','EPSG','8822','Longitude of false origin',-98.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',33.58,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',32.08,'EPSG','9110','EPSG','8826','Easting at false origin',600000.0,'EPSG','9001','EPSG','8827','Northing at false origin',2000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11302','conversion','EPSG','14232','EPSG','2254','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','14233','SPCS83 Texas Central zone (meter)','See code 15359 for equivalent non-metric definition.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',29.4,'EPSG','9110','EPSG','8822','Longitude of false origin',-100.2,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',31.53,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',30.07,'EPSG','9110','EPSG','8826','Easting at false origin',700000.0,'EPSG','9001','EPSG','8827','Northing at false origin',3000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11303','conversion','EPSG','14233','EPSG','2252','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','14234','SPCS83 Texas South Central zone (meter)','See code 15360 for equivalent non-metric definition.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',27.5,'EPSG','9110','EPSG','8822','Longitude of false origin',-99.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',30.17,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',28.23,'EPSG','9110','EPSG','8826','Easting at false origin',600000.0,'EPSG','9001','EPSG','8827','Northing at false origin',4000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11304','conversion','EPSG','14234','EPSG','2527','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','14235','SPCS83 Texas South zone (meter)','See code 15361 for equivalent non-metric definition.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',25.4,'EPSG','9110','EPSG','8822','Longitude of false origin',-98.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',27.5,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',26.1,'EPSG','9110','EPSG','8826','Easting at false origin',300000.0,'EPSG','9001','EPSG','8827','Northing at false origin',5000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11305','conversion','EPSG','14235','EPSG','2528','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','14251','Texas State Mapping System (meter)','Replaces Shackleford.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',31.1,'EPSG','9110','EPSG','8822','Longitude of false origin',-100.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',27.25,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',34.55,'EPSG','9110','EPSG','8826','Easting at false origin',1000000.0,'EPSG','9001','EPSG','8827','Northing at false origin',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11306','conversion','EPSG','14251','EPSG','1412','EPSG','1135');
INSERT INTO "conversion" VALUES('EPSG','14252','Shackleford','Replaced by TSMS. Care: survey data in Texas uses the US survey foot, not the International foot used by this projection.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',31.1,'EPSG','9110','EPSG','8822','Longitude of false origin',-100.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',27.25,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',34.55,'EPSG','9110','EPSG','8826','Easting at false origin',3000000.0,'EPSG','9002','EPSG','8827','Northing at false origin',3000000.0,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11307','conversion','EPSG','14252','EPSG','1412','EPSG','1135');
INSERT INTO "conversion" VALUES('EPSG','14253','Texas Centric Lambert Conformal','Use TCMC/AEA for applications requiring true area measurement.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',18.0,'EPSG','9110','EPSG','8822','Longitude of false origin',-100.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',27.3,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',35.0,'EPSG','9110','EPSG','8826','Easting at false origin',1500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',5000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11308','conversion','EPSG','14253','EPSG','1412','EPSG','1221');
INSERT INTO "conversion" VALUES('EPSG','14254','Texas Centric Albers Equal Area','Use TCMC/LC for applications requiring true shape preservation.','EPSG','9822','Albers Equal Area','EPSG','8821','Latitude of false origin',18.0,'EPSG','9110','EPSG','8822','Longitude of false origin',-100.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',27.3,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',35.0,'EPSG','9110','EPSG','8826','Easting at false origin',1500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',6000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11309','conversion','EPSG','14254','EPSG','1412','EPSG','1222');
INSERT INTO "conversion" VALUES('EPSG','14301','Utah CS27 North zone','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',40.2,'EPSG','9110','EPSG','8822','Longitude of false origin',-111.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',40.43,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',41.47,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11310','conversion','EPSG','14301','EPSG','2258','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','14302','Utah CS27 Central zone','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',38.2,'EPSG','9110','EPSG','8822','Longitude of false origin',-111.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',39.01,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',40.39,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11311','conversion','EPSG','14302','EPSG','2257','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','14303','Utah CS27 South zone','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',36.4,'EPSG','9110','EPSG','8822','Longitude of false origin',-111.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',37.13,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',38.21,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11312','conversion','EPSG','14303','EPSG','2259','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','14331','SPCS83 Utah North zone (meter)','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',40.2,'EPSG','9110','EPSG','8822','Longitude of false origin',-111.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',41.47,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',40.43,'EPSG','9110','EPSG','8826','Easting at false origin',500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11313','conversion','EPSG','14331','EPSG','2258','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','14332','SPCS83 Utah Central zone (meter)','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',38.2,'EPSG','9110','EPSG','8822','Longitude of false origin',-111.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',40.39,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',39.01,'EPSG','9110','EPSG','8826','Easting at false origin',500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',2000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11314','conversion','EPSG','14332','EPSG','2257','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','14333','SPCS83 Utah South zone (meter)','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',36.4,'EPSG','9110','EPSG','8822','Longitude of false origin',-111.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',38.21,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',37.13,'EPSG','9110','EPSG','8826','Easting at false origin',500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',3000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11315','conversion','EPSG','14333','EPSG','2259','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','14400','Vermont CS27','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',42.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-72.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999964286,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11316','conversion','EPSG','14400','EPSG','1414','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','14430','SPCS83 Vermont zone (meter)','See code 5645 for equivalent non-metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',42.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-72.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999964286,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11317','conversion','EPSG','14430','EPSG','1414','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','14501','Virginia CS27 North zone','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',37.4,'EPSG','9110','EPSG','8822','Longitude of false origin',-78.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',38.02,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',39.12,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11318','conversion','EPSG','14501','EPSG','2260','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','14502','Virginia CS27 South zone','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',36.2,'EPSG','9110','EPSG','8822','Longitude of false origin',-78.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',36.46,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',37.58,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11319','conversion','EPSG','14502','EPSG','2261','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','14531','SPCS83 Virginia North zone (meter)','See code 15365 for equivalent non-metric definition.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',37.4,'EPSG','9110','EPSG','8822','Longitude of false origin',-78.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',39.12,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',38.02,'EPSG','9110','EPSG','8826','Easting at false origin',3500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',2000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11320','conversion','EPSG','14531','EPSG','2260','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','14532','SPCS83 Virginia South zone (meter)','See code 15366 for equivalent non-metric definition.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',36.2,'EPSG','9110','EPSG','8822','Longitude of false origin',-78.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',37.58,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',36.46,'EPSG','9110','EPSG','8826','Easting at false origin',3500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11321','conversion','EPSG','14532','EPSG','2261','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','14601','Washington CS27 North zone','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',47.0,'EPSG','9110','EPSG','8822','Longitude of false origin',-120.5,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',47.3,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',48.44,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11322','conversion','EPSG','14601','EPSG','2262','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','14602','Washington CS27 South zone','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',45.2,'EPSG','9110','EPSG','8822','Longitude of false origin',-120.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',45.5,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',47.2,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11323','conversion','EPSG','14602','EPSG','2263','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','14631','SPCS83 Washington North zone (meter)','See code 15367 for equivalent non-metric definition.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',47.0,'EPSG','9110','EPSG','8822','Longitude of false origin',-120.5,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',48.44,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',47.3,'EPSG','9110','EPSG','8826','Easting at false origin',500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11324','conversion','EPSG','14631','EPSG','2273','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','14632','SPCS83 Washington South zone (meter)','See code 15368 for equivalent non-metric definition.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',45.2,'EPSG','9110','EPSG','8822','Longitude of false origin',-120.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',47.2,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',45.5,'EPSG','9110','EPSG','8826','Easting at false origin',500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11325','conversion','EPSG','14632','EPSG','2274','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','14701','West Virginia CS27 North zone','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',38.3,'EPSG','9110','EPSG','8822','Longitude of false origin',-79.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',39.0,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',40.15,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11326','conversion','EPSG','14701','EPSG','2264','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','14702','West Virginia CS27 South zone','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',37.0,'EPSG','9110','EPSG','8822','Longitude of false origin',-81.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',37.29,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',38.53,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11327','conversion','EPSG','14702','EPSG','2265','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','14731','SPCS83 West Virginia North zone (meter)','See projection code 14735 for equivalent non-metric definition.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',38.3,'EPSG','9110','EPSG','8822','Longitude of false origin',-79.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',40.15,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',39.0,'EPSG','9110','EPSG','8826','Easting at false origin',600000.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11328','conversion','EPSG','14731','EPSG','2264','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','14732','SPCS83 West Virginia South zone (meter)','See projection code 14736 for equivalent non-metric definition.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',37.0,'EPSG','9110','EPSG','8822','Longitude of false origin',-81.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',38.53,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',37.29,'EPSG','9110','EPSG','8826','Easting at false origin',600000.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11329','conversion','EPSG','14732','EPSG','2265','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','14733','SPCS83 West Virginia North zone (US Survey feet)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 14731.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',38.3,'EPSG','9110','EPSG','8822','Longitude of false origin',-79.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',40.15,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',39.0,'EPSG','9110','EPSG','8826','Easting at false origin',1968500.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11330','conversion','EPSG','14733','EPSG','2264','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','14734','SPCS83 West Virginia South zone (US Survey feet)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 14732.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',37.0,'EPSG','9110','EPSG','8822','Longitude of false origin',-81.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',38.53,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',37.29,'EPSG','9110','EPSG','8826','Easting at false origin',1968500.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11331','conversion','EPSG','14734','EPSG','2265','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','14735','SPCS83 West Virginia North zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 14731.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',38.3,'EPSG','9110','EPSG','8822','Longitude of false origin',-79.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',40.15,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',39.0,'EPSG','9110','EPSG','8826','Easting at false origin',1968500.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11332','conversion','EPSG','14735','EPSG','2264','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','14736','SPCS83 West Virginia South zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 14732.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',37.0,'EPSG','9110','EPSG','8822','Longitude of false origin',-81.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',38.53,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',37.29,'EPSG','9110','EPSG','8826','Easting at false origin',1968500.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11333','conversion','EPSG','14736','EPSG','2265','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','14801','Wisconsin CS27 North zone','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',45.1,'EPSG','9110','EPSG','8822','Longitude of false origin',-90.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',45.34,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',46.46,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11334','conversion','EPSG','14801','EPSG','2267','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','14802','Wisconsin CS27 Central zone','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',43.5,'EPSG','9110','EPSG','8822','Longitude of false origin',-90.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',44.15,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',45.3,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11335','conversion','EPSG','14802','EPSG','2266','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','14803','Wisconsin CS27 South zone','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',42.0,'EPSG','9110','EPSG','8822','Longitude of false origin',-90.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',42.44,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',44.04,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11336','conversion','EPSG','14803','EPSG','2268','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','14811','Wisconsin Transverse Mercator 27','Designed to cover the state in a single zone.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-90.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',-4500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11337','conversion','EPSG','14811','EPSG','1418','EPSG','1135');
INSERT INTO "conversion" VALUES('EPSG','14831','SPCS83 Wisconsin North zone (meter)','See code 15369 for equivalent non-metric definition.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',45.1,'EPSG','9110','EPSG','8822','Longitude of false origin',-90.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',46.46,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',45.34,'EPSG','9110','EPSG','8826','Easting at false origin',600000.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11338','conversion','EPSG','14831','EPSG','2267','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','14832','SPCS83 Wisconsin Central zone (meter)','See code 15370 for equivalent non-metric definition.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',43.5,'EPSG','9110','EPSG','8822','Longitude of false origin',-90.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',45.3,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',44.15,'EPSG','9110','EPSG','8826','Easting at false origin',600000.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11339','conversion','EPSG','14832','EPSG','2266','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','14833','SPCS83 Wisconsin South zone (meter)','See code 15371 for equivalent non-metric definition.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',42.0,'EPSG','9110','EPSG','8822','Longitude of false origin',-90.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',44.04,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',42.44,'EPSG','9110','EPSG','8826','Easting at false origin',600000.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11340','conversion','EPSG','14833','EPSG','2268','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','14841','Wisconsin Transverse Mercator 83','Designed to cover the state in a single zone.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-90.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',520000.0,'EPSG','9001','EPSG','8807','False northing',-4480000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11341','conversion','EPSG','14841','EPSG','1418','EPSG','1135');
INSERT INTO "conversion" VALUES('EPSG','14901','Wyoming CS27 East zone','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',40.4,'EPSG','9110','EPSG','8802','Longitude of natural origin',-105.1,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999941177,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11342','conversion','EPSG','14901','EPSG','2269','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','14902','Wyoming CS27 East Central zone','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',40.4,'EPSG','9110','EPSG','8802','Longitude of natural origin',-107.2,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999941177,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11343','conversion','EPSG','14902','EPSG','2270','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','14903','Wyoming CS27 West Central zone','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',40.4,'EPSG','9110','EPSG','8802','Longitude of natural origin',-108.45,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999941177,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11344','conversion','EPSG','14903','EPSG','2272','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','14904','Wyoming CS27 West zone','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',40.4,'EPSG','9110','EPSG','8802','Longitude of natural origin',-110.05,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999941177,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11345','conversion','EPSG','14904','EPSG','2271','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','14930','Wyoming Lambert','Source originally defining the projection is unclear - possibly the Wyoming GIS Center.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',41.0,'EPSG','9110','EPSG','8822','Longitude of false origin',-107.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',41.0,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',45.0,'EPSG','9110','EPSG','8826','Easting at false origin',500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',200000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','17408','conversion','EPSG','14930','EPSG','1419','EPSG','1135');
INSERT INTO "conversion" VALUES('EPSG','14931','SPCS83 Wyoming East zone (meter)','See code 14935 for equivalent non-metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',40.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-105.1,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999375,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11346','conversion','EPSG','14931','EPSG','2269','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','14932','SPCS83 Wyoming East Central zone (meter)','See code 14936 for equivalent non-metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',40.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-107.2,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999375,'EPSG','9201','EPSG','8806','False easting',400000.0,'EPSG','9001','EPSG','8807','False northing',100000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11347','conversion','EPSG','14932','EPSG','2270','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','14933','SPCS83 Wyoming West Central zone (meter)','See code 14937 for equivalent non-metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',40.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-108.45,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999375,'EPSG','9201','EPSG','8806','False easting',600000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11348','conversion','EPSG','14933','EPSG','2272','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','14934','SPCS83 Wyoming West zone (meter)','See code 14938 for equivalent non-metric definition.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',40.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-110.05,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999375,'EPSG','9201','EPSG','8806','False easting',800000.0,'EPSG','9001','EPSG','8807','False northing',100000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11349','conversion','EPSG','14934','EPSG','2271','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','14935','SPCS83 Wyoming East zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 14931.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',40.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-105.1,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999375,'EPSG','9201','EPSG','8806','False easting',656166.6667,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11350','conversion','EPSG','14935','EPSG','2269','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','14936','SPCS83 Wyoming East Central zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 14932.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',40.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-107.2,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999375,'EPSG','9201','EPSG','8806','False easting',1312333.3333,'EPSG','9003','EPSG','8807','False northing',328083.3333,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11351','conversion','EPSG','14936','EPSG','2270','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','14937','SPCS83 Wyoming West Central zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 14933.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',40.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-108.45,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999375,'EPSG','9201','EPSG','8806','False easting',1968500.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11352','conversion','EPSG','14937','EPSG','2272','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','14938','SPCS83 Wyoming West zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 14934.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',40.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-110.05,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999375,'EPSG','9201','EPSG','8806','False easting',2624666.6667,'EPSG','9003','EPSG','8807','False northing',328083.3333,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11353','conversion','EPSG','14938','EPSG','2271','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15001','Alaska CS27 zone 1','If using Hotine Oblique Mercator (variant B) method (code 9815), Ec=2685642.82 ftUS, Nc=1887198.47 ftUS.','EPSG','9812','Hotine Oblique Mercator (variant A)','EPSG','8811','Latitude of projection centre',57.0,'EPSG','9110','EPSG','8812','Longitude of projection centre',-133.4,'EPSG','9110','EPSG','8813','Azimuth at projection centre',323.07483685,'EPSG','9110','EPSG','8814','Angle from Rectified to Skew Grid',323.07483685,'EPSG','9110','EPSG','8815','Scale factor at projection centre',0.9999,'EPSG','9201','EPSG','8806','False easting',16404166.67,'EPSG','9003','EPSG','8807','False northing',-16404166.67,'EPSG','9003',0);
INSERT INTO "usage" VALUES('EPSG','11354','conversion','EPSG','15001','EPSG','2156','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15002','Alaska CS27 zone 2','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',54.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-142.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11355','conversion','EPSG','15002','EPSG','2158','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15003','Alaska CS27 zone 3','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',54.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-146.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11356','conversion','EPSG','15003','EPSG','2159','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15004','Alaska CS27 zone 4','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',54.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-150.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11357','conversion','EPSG','15004','EPSG','2160','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15005','Alaska CS27 zone 5','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',54.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-154.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11358','conversion','EPSG','15005','EPSG','2161','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15006','Alaska CS27 zone 6','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',54.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-158.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11359','conversion','EPSG','15006','EPSG','2162','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15007','Alaska CS27 zone 7','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',54.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-162.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',700000.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11360','conversion','EPSG','15007','EPSG','2163','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15008','Alaska CS27 zone 8','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',54.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-166.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11361','conversion','EPSG','15008','EPSG','2164','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15009','Alaska CS27 zone 9','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',54.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-170.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',600000.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11362','conversion','EPSG','15009','EPSG','2165','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15010','Alaska CS27 zone 10','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',51.0,'EPSG','9110','EPSG','8822','Longitude of false origin',-176.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',53.5,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',51.5,'EPSG','9110','EPSG','8826','Easting at false origin',3000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11363','conversion','EPSG','15010','EPSG','2157','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15020','Alaska Albers','','EPSG','9822','Albers Equal Area','EPSG','8821','Latitude of false origin',50.0,'EPSG','9102','EPSG','8822','Longitude of false origin',-154.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',55.0,'EPSG','9102','EPSG','8824','Latitude of 2nd standard parallel',65.0,'EPSG','9102','EPSG','8826','Easting at false origin',0.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11364','conversion','EPSG','15020','EPSG','1330','EPSG','1241');
INSERT INTO "conversion" VALUES('EPSG','15021','Alaska Albers (meter)','','EPSG','9822','Albers Equal Area','EPSG','8821','Latitude of false origin',50.0,'EPSG','9102','EPSG','8822','Longitude of false origin',-154.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',55.0,'EPSG','9102','EPSG','8824','Latitude of 2nd standard parallel',65.0,'EPSG','9102','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11365','conversion','EPSG','15021','EPSG','1330','EPSG','1241');
INSERT INTO "conversion" VALUES('EPSG','15031','SPCS83 Alaska zone 1 (meter)','If using Hotine Oblique Mercator (variant B) method (code 9815), Ec=818585.57 m, Nc=575219.25 m.','EPSG','9812','Hotine Oblique Mercator (variant A)','EPSG','8811','Latitude of projection centre',57.0,'EPSG','9110','EPSG','8812','Longitude of projection centre',-133.4,'EPSG','9110','EPSG','8813','Azimuth at projection centre',323.07483685,'EPSG','9110','EPSG','8814','Angle from Rectified to Skew Grid',323.07483685,'EPSG','9110','EPSG','8815','Scale factor at projection centre',0.9999,'EPSG','9201','EPSG','8806','False easting',5000000.0,'EPSG','9001','EPSG','8807','False northing',-5000000.0,'EPSG','9001',0);
INSERT INTO "usage" VALUES('EPSG','11366','conversion','EPSG','15031','EPSG','2156','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15032','SPCS83 Alaska zone 2 (meter)','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',54.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-142.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11367','conversion','EPSG','15032','EPSG','2158','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15033','SPCS83 Alaska zone 3 (meter)','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',54.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-146.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11368','conversion','EPSG','15033','EPSG','2159','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15034','SPCS83 Alaska zone 4 (meter)','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',54.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-150.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11369','conversion','EPSG','15034','EPSG','2160','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15035','SPCS83 Alaska zone 5 (meter)','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',54.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-154.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11370','conversion','EPSG','15035','EPSG','2161','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15036','SPCS83 Alaska zone 6 (meter)','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',54.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-158.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11371','conversion','EPSG','15036','EPSG','2162','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15037','SPCS83 Alaska zone 7 (meter)','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',54.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-162.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11372','conversion','EPSG','15037','EPSG','2163','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15038','SPCS83 Alaska zone 8 (meter)','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',54.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-166.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11373','conversion','EPSG','15038','EPSG','2164','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15039','SPCS83 Alaska zone 9 (meter)','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',54.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-170.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11374','conversion','EPSG','15039','EPSG','2165','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15040','SPCS83 Alaska zone 10 (meter)','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',51.0,'EPSG','9110','EPSG','8822','Longitude of false origin',-176.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',53.5,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',51.5,'EPSG','9110','EPSG','8826','Easting at false origin',1000000.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11375','conversion','EPSG','15040','EPSG','2157','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15101','Hawaii CS27 zone 1','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',18.5,'EPSG','9110','EPSG','8802','Longitude of natural origin',-155.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999966667,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11376','conversion','EPSG','15101','EPSG','1546','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15102','Hawaii CS27 zone 2','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',20.2,'EPSG','9110','EPSG','8802','Longitude of natural origin',-156.4,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999966667,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11377','conversion','EPSG','15102','EPSG','1547','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15103','Hawaii CS27 zone 3','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',21.1,'EPSG','9110','EPSG','8802','Longitude of natural origin',-158.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.99999,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11378','conversion','EPSG','15103','EPSG','1548','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15104','Hawaii CS27 zone 4','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',21.5,'EPSG','9110','EPSG','8802','Longitude of natural origin',-159.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.99999,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11379','conversion','EPSG','15104','EPSG','1549','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15105','Hawaii CS27 zone 5','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',21.4,'EPSG','9110','EPSG','8802','Longitude of natural origin',-160.1,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11380','conversion','EPSG','15105','EPSG','1550','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15131','SPCS83 Hawaii zone 1 (meter)','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',18.5,'EPSG','9110','EPSG','8802','Longitude of natural origin',-155.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999966667,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11381','conversion','EPSG','15131','EPSG','1546','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15132','SPCS83 Hawaii zone 2 (meter)','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',20.2,'EPSG','9110','EPSG','8802','Longitude of natural origin',-156.4,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999966667,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11382','conversion','EPSG','15132','EPSG','1547','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15133','SPCS83 Hawaii zone 3 (meter)','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',21.1,'EPSG','9110','EPSG','8802','Longitude of natural origin',-158.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.99999,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11383','conversion','EPSG','15133','EPSG','1548','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15134','SPCS83 Hawaii zone 4 (meter)','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',21.5,'EPSG','9110','EPSG','8802','Longitude of natural origin',-159.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.99999,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11384','conversion','EPSG','15134','EPSG','1549','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15135','SPCS83 Hawaii zone 5 (meter)','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',21.4,'EPSG','9110','EPSG','8802','Longitude of natural origin',-160.1,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11385','conversion','EPSG','15135','EPSG','1550','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15138','SPCS83 Hawaii zone 3 (US survey foot)','Used by City and County of Honolulu. Not recognised by Federal authorities because there is no State law defining grid unit. For equivalent metric Federal definition see code 15133.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',21.1,'EPSG','9110','EPSG','8802','Longitude of natural origin',-158.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.99999,'EPSG','9201','EPSG','8806','False easting',1640416.6667,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11386','conversion','EPSG','15138','EPSG','1548','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15201','Puerto Rico CS27','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',17.5,'EPSG','9110','EPSG','8822','Longitude of false origin',-66.26,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',18.26,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',18.02,'EPSG','9110','EPSG','8826','Easting at false origin',500000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11387','conversion','EPSG','15201','EPSG','3294','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15202','St. Croix CS27','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',17.5,'EPSG','9110','EPSG','8822','Longitude of false origin',-66.26,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',18.26,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',18.02,'EPSG','9110','EPSG','8826','Easting at false origin',500000.0,'EPSG','9003','EPSG','8827','Northing at false origin',100000.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11388','conversion','EPSG','15202','EPSG','3330','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15230','SPCS83 Puerto Rico & Virgin Islands zone (meter)','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',17.5,'EPSG','9110','EPSG','8822','Longitude of false origin',-66.26,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',18.26,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',18.02,'EPSG','9110','EPSG','8826','Easting at false origin',200000.0,'EPSG','9001','EPSG','8827','Northing at false origin',200000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11389','conversion','EPSG','15230','EPSG','3634','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15297','SPCS83 Utah North zone (US survey foot)','State law defining grid unit as International feet (note: not US Survey feet) has been withdrawn. For equivalent metric Federal definition see code 14331. For equivalent International foot definition see code 15362.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',40.2,'EPSG','9110','EPSG','8822','Longitude of false origin',-111.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',41.47,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',40.43,'EPSG','9110','EPSG','8826','Easting at false origin',1640416.6667,'EPSG','9003','EPSG','8827','Northing at false origin',3280833.3333,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11390','conversion','EPSG','15297','EPSG','2258','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15298','SPCS83 Utah Central zone (US survey foot)','State law defining grid unit as International feet (note: not US Survey feet) has been withdrawn. For equivalent metric Federal definition see code 14332. For equivalent International foot definition see code 15363.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',38.2,'EPSG','9110','EPSG','8822','Longitude of false origin',-111.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',40.39,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',39.01,'EPSG','9110','EPSG','8826','Easting at false origin',1640416.6667,'EPSG','9003','EPSG','8827','Northing at false origin',6561666.6667,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11391','conversion','EPSG','15298','EPSG','2257','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15299','SPCS83 Utah South zone (US survey foot)','State law defining grid unit as International feet (note: not US Survey feet) has been withdrawn. For equivalent metric Federal definition see code 14333. For equivalent International foot definition see code 15364.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',36.4,'EPSG','9110','EPSG','8822','Longitude of false origin',-111.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',38.21,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',37.13,'EPSG','9110','EPSG','8826','Easting at false origin',1640416.6667,'EPSG','9003','EPSG','8827','Northing at false origin',9842500.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11392','conversion','EPSG','15299','EPSG','2259','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15300','American Samoa Lambert','Per Snyder: Map Projections - a Working Manual: At origin x=500000 ft; y=o but radius to latitude of origin = -82000000 feet. US National Geodetic Survey confirms use of zero for False Northing.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',-14.16,'EPSG','9110','EPSG','8802','Longitude of natural origin',170.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11393','conversion','EPSG','15300','EPSG','1027','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15301','American Samoa Lambert','Per Snyder: Map Projections - a Working Manual: At origin x=500000 ft; y=0 but radius to latitude of origin = -82000000 feet. Thus False Northing = 312234.65 ftUS. NGS confirms value of FN, which is encoded within their algorithms but not published.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',-14.16,'EPSG','9110','EPSG','8802','Longitude of natural origin',-170.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11394','conversion','EPSG','15301','EPSG','1027','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15302','Tennessee CS27','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',34.4,'EPSG','9110','EPSG','8822','Longitude of false origin',-86.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',35.15,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',36.25,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',100000.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11395','conversion','EPSG','15302','EPSG','1411','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15303','SPCS83 Kentucky North zone (meter)','See code 15328 for equivalent non-metric definition.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',37.3,'EPSG','9110','EPSG','8822','Longitude of false origin',-84.15,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',37.58,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',38.58,'EPSG','9110','EPSG','8826','Easting at false origin',500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11396','conversion','EPSG','15303','EPSG','2202','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15304','SPCS83 Arizona East zone (international foot)','State law defines grid unit as International feet (note: not US Survey feet). For equivalent metric Federal definition see code 10231.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',31.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-110.1,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',700000.0,'EPSG','9002','EPSG','8807','False northing',0.0,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11397','conversion','EPSG','15304','EPSG','2167','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15305','SPCS83 Arizona Central zone (international foot)','State law defines grid unit as International feet (note: not US Survey feet). For equivalent metric Federal definition see code 10232.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',31.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-111.55,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',700000.0,'EPSG','9002','EPSG','8807','False northing',0.0,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11398','conversion','EPSG','15305','EPSG','2166','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15306','SPCS83 Arizona West zone (international foot)','State law defines grid unit as International feet (note: not US Survey feet). For equivalent metric Federal definition see code 10233.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',31.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-113.45,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999933333,'EPSG','9201','EPSG','8806','False easting',700000.0,'EPSG','9002','EPSG','8807','False northing',0.0,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11399','conversion','EPSG','15306','EPSG','2168','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15307','SPCS83 California zone 1 (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 10431.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',39.2,'EPSG','9110','EPSG','8822','Longitude of false origin',-122.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',41.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',40.0,'EPSG','9110','EPSG','8826','Easting at false origin',6561666.667,'EPSG','9003','EPSG','8827','Northing at false origin',1640416.667,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11400','conversion','EPSG','15307','EPSG','2175','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15308','SPCS83 California zone 2 (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 10432.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',37.4,'EPSG','9110','EPSG','8822','Longitude of false origin',-122.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',39.5,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',38.2,'EPSG','9110','EPSG','8826','Easting at false origin',6561666.667,'EPSG','9003','EPSG','8827','Northing at false origin',1640416.667,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11401','conversion','EPSG','15308','EPSG','2176','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15309','SPCS83 California zone 3 (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 10433.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',36.3,'EPSG','9110','EPSG','8822','Longitude of false origin',-120.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',38.26,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',37.04,'EPSG','9110','EPSG','8826','Easting at false origin',6561666.667,'EPSG','9003','EPSG','8827','Northing at false origin',1640416.667,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11402','conversion','EPSG','15309','EPSG','2177','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15310','SPCS83 California zone 4 (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 10434.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',35.2,'EPSG','9110','EPSG','8822','Longitude of false origin',-119.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',37.15,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',36.0,'EPSG','9110','EPSG','8826','Easting at false origin',6561666.667,'EPSG','9003','EPSG','8827','Northing at false origin',1640416.667,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11403','conversion','EPSG','15310','EPSG','2178','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15311','SPCS83 California zone 5 (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 10435.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',33.3,'EPSG','9110','EPSG','8822','Longitude of false origin',-118.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',35.28,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',34.02,'EPSG','9110','EPSG','8826','Easting at false origin',6561666.667,'EPSG','9003','EPSG','8827','Northing at false origin',1640416.667,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11404','conversion','EPSG','15311','EPSG','2182','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15312','SPCS83 California zone 6 (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 10436.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',32.1,'EPSG','9110','EPSG','8822','Longitude of false origin',-116.15,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',33.53,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',32.47,'EPSG','9110','EPSG','8826','Easting at false origin',6561666.667,'EPSG','9003','EPSG','8827','Northing at false origin',1640416.667,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11405','conversion','EPSG','15312','EPSG','2180','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15313','SPCS83 Colorado North zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 10531.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',39.2,'EPSG','9110','EPSG','8822','Longitude of false origin',-105.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',40.47,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',39.43,'EPSG','9110','EPSG','8826','Easting at false origin',3000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',1000000.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11406','conversion','EPSG','15313','EPSG','2184','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15314','SPCS83 Colorado Central zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 10532.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',37.5,'EPSG','9110','EPSG','8822','Longitude of false origin',-105.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',39.45,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',38.27,'EPSG','9110','EPSG','8826','Easting at false origin',3000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',1000000.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11407','conversion','EPSG','15314','EPSG','2183','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15315','SPCS83 Colorado South zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 10533.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',36.4,'EPSG','9110','EPSG','8822','Longitude of false origin',-105.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',38.26,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',37.14,'EPSG','9110','EPSG','8826','Easting at false origin',3000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',1000000.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11408','conversion','EPSG','15315','EPSG','2185','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15316','SPCS83 Connecticut zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 10630.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',40.5,'EPSG','9110','EPSG','8822','Longitude of false origin',-72.45,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',41.52,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',41.12,'EPSG','9110','EPSG','8826','Easting at false origin',1000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',500000.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11409','conversion','EPSG','15316','EPSG','1377','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15317','SPCS83 Delaware zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 10730.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',38.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-75.25,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999995,'EPSG','9201','EPSG','8806','False easting',656166.667,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11410','conversion','EPSG','15317','EPSG','1378','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15318','SPCS83 Florida East zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 10931.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',24.2,'EPSG','9110','EPSG','8802','Longitude of natural origin',-81.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999941177,'EPSG','9201','EPSG','8806','False easting',656166.667,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11411','conversion','EPSG','15318','EPSG','2186','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15319','SPCS83 Florida West zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 10932.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',24.2,'EPSG','9110','EPSG','8802','Longitude of natural origin',-82.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999941177,'EPSG','9201','EPSG','8806','False easting',656166.667,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11412','conversion','EPSG','15319','EPSG','2188','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15320','SPCS83 Florida North zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 10933.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',29.0,'EPSG','9110','EPSG','8822','Longitude of false origin',-84.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',30.45,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',29.35,'EPSG','9110','EPSG','8826','Easting at false origin',1968500.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11413','conversion','EPSG','15320','EPSG','2187','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15321','SPCS83 Georgia East zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 11031.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',30.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-82.1,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',656166.667,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11414','conversion','EPSG','15321','EPSG','2189','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15322','SPCS83 Georgia West zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 11032.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',30.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-84.1,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',2296583.333,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11415','conversion','EPSG','15322','EPSG','2190','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15323','SPCS83 Idaho East zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 11131.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',41.4,'EPSG','9110','EPSG','8802','Longitude of natural origin',-112.1,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999947368,'EPSG','9201','EPSG','8806','False easting',656166.667,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11416','conversion','EPSG','15323','EPSG','2192','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15324','SPCS83 Idaho Central zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 11132.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',41.4,'EPSG','9110','EPSG','8802','Longitude of natural origin',-114.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999947368,'EPSG','9201','EPSG','8806','False easting',1640416.667,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11417','conversion','EPSG','15324','EPSG','2191','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15325','SPCS83 Idaho West zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 11133.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',41.4,'EPSG','9110','EPSG','8802','Longitude of natural origin',-115.45,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999933333,'EPSG','9201','EPSG','8806','False easting',2624666.667,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11418','conversion','EPSG','15325','EPSG','2193','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15326','SPCS83 Indiana East zone (US Survey feet)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 11331.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',37.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-85.4,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999966667,'EPSG','9201','EPSG','8806','False easting',328083.333,'EPSG','9003','EPSG','8807','False northing',818125.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11419','conversion','EPSG','15326','EPSG','2196','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15327','SPCS83 Indiana West zone (US Survey feet)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 11332.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',37.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-87.05,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999966667,'EPSG','9201','EPSG','8806','False easting',2952750.0,'EPSG','9003','EPSG','8807','False northing',818125.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11420','conversion','EPSG','15327','EPSG','2197','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15328','SPCS83 Kentucky North zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 15303.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',37.3,'EPSG','9110','EPSG','8822','Longitude of false origin',-84.15,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',37.58,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',38.58,'EPSG','9110','EPSG','8826','Easting at false origin',1640416.667,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11421','conversion','EPSG','15328','EPSG','2202','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15329','SPCS83 Kentucky South zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 11632.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',36.2,'EPSG','9110','EPSG','8822','Longitude of false origin',-85.45,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',37.56,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',36.44,'EPSG','9110','EPSG','8826','Easting at false origin',1640416.667,'EPSG','9003','EPSG','8827','Northing at false origin',1640416.667,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11422','conversion','EPSG','15329','EPSG','2203','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15330','SPCS83 Maryland zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 11930.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',37.4,'EPSG','9110','EPSG','8822','Longitude of false origin',-77.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',39.27,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',38.18,'EPSG','9110','EPSG','8826','Easting at false origin',1312333.333,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11423','conversion','EPSG','15330','EPSG','1389','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15331','SPCS83 Massachusetts Mainland zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 12031.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',41.0,'EPSG','9110','EPSG','8822','Longitude of false origin',-71.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',42.41,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',41.43,'EPSG','9110','EPSG','8826','Easting at false origin',656166.667,'EPSG','9003','EPSG','8827','Northing at false origin',2460625.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11424','conversion','EPSG','15331','EPSG','2209','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15332','SPCS83 Massachusetts Island zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 12032.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',41.0,'EPSG','9110','EPSG','8822','Longitude of false origin',-70.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',41.29,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',41.17,'EPSG','9110','EPSG','8826','Easting at false origin',1640416.667,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11425','conversion','EPSG','15332','EPSG','2208','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15333','SPCS83 Michigan North zone (international foot)','State law defines grid unit as International feet (note: not US Survey feet). For equivalent metric Federal definition see code 12141.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',44.47,'EPSG','9110','EPSG','8822','Longitude of false origin',-87.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',47.05,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',45.29,'EPSG','9110','EPSG','8826','Easting at false origin',26246719.16,'EPSG','9002','EPSG','8827','Northing at false origin',0.0,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11426','conversion','EPSG','15333','EPSG','1723','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15334','SPCS83 Michigan Central zone (international foot)','State law defines grid unit as International feet (note: not US Survey feet). For equivalent metric Federal definition see code 12142.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',43.19,'EPSG','9110','EPSG','8822','Longitude of false origin',-84.22,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',45.42,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',44.11,'EPSG','9110','EPSG','8826','Easting at false origin',19685039.37,'EPSG','9002','EPSG','8827','Northing at false origin',0.0,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11427','conversion','EPSG','15334','EPSG','1724','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15335','SPCS83 Michigan South zone (international foot)','State law defines grid unit as International feet (note: not US Survey feet). For equivalent metric Federal definition see code 12143.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',41.3,'EPSG','9110','EPSG','8822','Longitude of false origin',-84.22,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',43.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',42.06,'EPSG','9110','EPSG','8826','Easting at false origin',13123359.58,'EPSG','9002','EPSG','8827','Northing at false origin',0.0,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11428','conversion','EPSG','15335','EPSG','1725','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15336','SPCS83 Mississippi East zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 12331.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',29.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-88.5,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.99995,'EPSG','9201','EPSG','8806','False easting',984250.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11429','conversion','EPSG','15336','EPSG','2216','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15337','SPCS83 Mississippi West zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 12332.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',29.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-90.2,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.99995,'EPSG','9201','EPSG','8806','False easting',2296583.333,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11430','conversion','EPSG','15337','EPSG','2217','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15338','SPCS83 Montana zone (international foot)','State law defines grid unit as International feet (note: not US Survey feet). For equivalent metric Federal definition see code 12530.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',44.15,'EPSG','9110','EPSG','8822','Longitude of false origin',-109.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',49.0,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',45.0,'EPSG','9110','EPSG','8826','Easting at false origin',1968503.937,'EPSG','9002','EPSG','8827','Northing at false origin',0.0,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11431','conversion','EPSG','15338','EPSG','1395','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15339','SPCS83 New Mexico East zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 13031.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',31.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-104.2,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999909091,'EPSG','9201','EPSG','8806','False easting',541337.5,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11432','conversion','EPSG','15339','EPSG','2228','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15340','SPCS83 New Mexico Central zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 13032.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',31.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-106.15,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',1640416.667,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11433','conversion','EPSG','15340','EPSG','2231','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15341','SPCS83 New Mexico West zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 13033.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',31.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-107.5,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999916667,'EPSG','9201','EPSG','8806','False easting',2723091.667,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11434','conversion','EPSG','15341','EPSG','2232','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15342','SPCS83 New York East zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 13131.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',38.5,'EPSG','9110','EPSG','8802','Longitude of natural origin',-74.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',492125.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11435','conversion','EPSG','15342','EPSG','2234','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15343','SPCS83 New York Central zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 13132.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',40.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-76.35,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999375,'EPSG','9201','EPSG','8806','False easting',820208.333,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11436','conversion','EPSG','15343','EPSG','2233','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15344','SPCS83 New York West zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 13133.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',40.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-78.35,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999375,'EPSG','9201','EPSG','8806','False easting',1148291.667,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11437','conversion','EPSG','15344','EPSG','2236','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15345','SPCS83 New York Long Island zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 13134.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',40.1,'EPSG','9110','EPSG','8822','Longitude of false origin',-74.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',41.02,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',40.4,'EPSG','9110','EPSG','8826','Easting at false origin',984250.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11438','conversion','EPSG','15345','EPSG','2235','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15346','SPCS83 North Carolina zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 13230.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',33.45,'EPSG','9110','EPSG','8822','Longitude of false origin',-79.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',36.1,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',34.2,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11439','conversion','EPSG','15346','EPSG','1402','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15347','SPCS83 North Dakota North zone (international foot)','State law defines grid unit as International feet (note: not US Survey feet). For equivalent metric Federal definition see code 13331.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',47.0,'EPSG','9110','EPSG','8822','Longitude of false origin',-100.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',48.44,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',47.26,'EPSG','9110','EPSG','8826','Easting at false origin',1968503.937,'EPSG','9002','EPSG','8827','Northing at false origin',0.0,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11440','conversion','EPSG','15347','EPSG','2237','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15348','SPCS83 North Dakota South zone (international foot)','State law defines grid unit as International feet (note: not US Survey feet). For equivalent metric Federal definition see code 13332.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',45.4,'EPSG','9110','EPSG','8822','Longitude of false origin',-100.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',47.29,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',46.11,'EPSG','9110','EPSG','8826','Easting at false origin',1968503.937,'EPSG','9002','EPSG','8827','Northing at false origin',0.0,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11441','conversion','EPSG','15348','EPSG','2238','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15349','SPCS83 Oklahoma North zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 13531.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',35.0,'EPSG','9110','EPSG','8822','Longitude of false origin',-98.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',36.46,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',35.34,'EPSG','9110','EPSG','8826','Easting at false origin',1968500.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11442','conversion','EPSG','15349','EPSG','2241','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15350','SPCS83 Oklahoma South zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 13532.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',33.2,'EPSG','9110','EPSG','8822','Longitude of false origin',-98.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',35.14,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',33.56,'EPSG','9110','EPSG','8826','Easting at false origin',1968500.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11443','conversion','EPSG','15350','EPSG','2242','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15351','SPCS83 Oregon North zone (international foot)','State law defines grid unit as International feet (note: not US Survey feet). For equivalent metric Federal definition see code 13631.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',43.4,'EPSG','9110','EPSG','8822','Longitude of false origin',-120.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',46.0,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',44.2,'EPSG','9110','EPSG','8826','Easting at false origin',8202099.738,'EPSG','9002','EPSG','8827','Northing at false origin',0.0,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11444','conversion','EPSG','15351','EPSG','2243','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15352','SPCS83 Oregon South zone (international foot)','State law defines grid unit as International feet (note: not US Survey feet). For equivalent metric Federal definition see code 13632.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',41.4,'EPSG','9110','EPSG','8822','Longitude of false origin',-120.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',44.0,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',42.2,'EPSG','9110','EPSG','8826','Easting at false origin',4921259.843,'EPSG','9002','EPSG','8827','Northing at false origin',0.0,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11445','conversion','EPSG','15352','EPSG','2244','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15353','SPCS83 Pennsylvania North zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 13731.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',40.1,'EPSG','9110','EPSG','8822','Longitude of false origin',-77.45,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',41.57,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',40.53,'EPSG','9110','EPSG','8826','Easting at false origin',1968500.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11446','conversion','EPSG','15353','EPSG','2245','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15354','SPCS83 Pennsylvania South zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 13732.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',39.2,'EPSG','9110','EPSG','8822','Longitude of false origin',-77.45,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',40.58,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',39.56,'EPSG','9110','EPSG','8826','Easting at false origin',1968500.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11447','conversion','EPSG','15354','EPSG','2246','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15355','SPCS83 South Carolina zone (international foot)','State law defines grid unit as International feet (note: not US Survey feet). For equivalent metric Federal definition see code 13930.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',31.5,'EPSG','9110','EPSG','8822','Longitude of false origin',-81.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',34.5,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',32.3,'EPSG','9110','EPSG','8826','Easting at false origin',2000000.0,'EPSG','9002','EPSG','8827','Northing at false origin',0.0,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11448','conversion','EPSG','15355','EPSG','1409','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15356','SPCS83 Tennessee zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 14130.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',34.2,'EPSG','9110','EPSG','8822','Longitude of false origin',-86.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',36.25,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',35.15,'EPSG','9110','EPSG','8826','Easting at false origin',1968500.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11449','conversion','EPSG','15356','EPSG','1411','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15357','SPCS83 Texas North zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 14231.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',34.0,'EPSG','9110','EPSG','8822','Longitude of false origin',-101.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',36.11,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',34.39,'EPSG','9110','EPSG','8826','Easting at false origin',656166.667,'EPSG','9003','EPSG','8827','Northing at false origin',3280833.333,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11450','conversion','EPSG','15357','EPSG','2253','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15358','SPCS83 Texas North Central zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 14232.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',31.4,'EPSG','9110','EPSG','8822','Longitude of false origin',-98.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',33.58,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',32.08,'EPSG','9110','EPSG','8826','Easting at false origin',1968500.0,'EPSG','9003','EPSG','8827','Northing at false origin',6561666.667,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11451','conversion','EPSG','15358','EPSG','2254','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15359','SPCS83 Texas Central zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 14233.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',29.4,'EPSG','9110','EPSG','8822','Longitude of false origin',-100.2,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',31.53,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',30.07,'EPSG','9110','EPSG','8826','Easting at false origin',2296583.333,'EPSG','9003','EPSG','8827','Northing at false origin',9842500.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11452','conversion','EPSG','15359','EPSG','2252','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15360','SPCS83 Texas South Central zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 14234.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',27.5,'EPSG','9110','EPSG','8822','Longitude of false origin',-99.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',30.17,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',28.23,'EPSG','9110','EPSG','8826','Easting at false origin',1968500.0,'EPSG','9003','EPSG','8827','Northing at false origin',13123333.333,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11453','conversion','EPSG','15360','EPSG','2527','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15361','SPCS83 Texas South zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 14235.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',25.4,'EPSG','9110','EPSG','8822','Longitude of false origin',-98.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',27.5,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',26.1,'EPSG','9110','EPSG','8826','Easting at false origin',984250.0,'EPSG','9003','EPSG','8827','Northing at false origin',16404166.667,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11454','conversion','EPSG','15361','EPSG','2528','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15362','SPCS83 Utah North zone (international foot)','State law defining grid unit as International feet (note: not US Survey feet) has been withdrawn. For equivalent metric Federal definition see code 14331. For equivalent US Survey foot definition see code 15297.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',40.2,'EPSG','9110','EPSG','8822','Longitude of false origin',-111.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',41.47,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',40.43,'EPSG','9110','EPSG','8826','Easting at false origin',1640419.948,'EPSG','9002','EPSG','8827','Northing at false origin',3280839.895,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11455','conversion','EPSG','15362','EPSG','2258','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15363','SPCS83 Utah Central zone (international foot)','State law defining grid unit as International feet (note: not US Survey feet) has been withdrawn. For equivalent metric Federal definition see code 14332. For equivalent US Survey foot definition see code 15298.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',38.2,'EPSG','9110','EPSG','8822','Longitude of false origin',-111.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',40.39,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',39.01,'EPSG','9110','EPSG','8826','Easting at false origin',1640419.948,'EPSG','9002','EPSG','8827','Northing at false origin',6561679.79,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11456','conversion','EPSG','15363','EPSG','2257','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15364','SPCS83 Utah South zone (international foot)','State law defining grid unit as International feet (note: not US Survey feet) has been withdrawn. For equivalent metric Federal definition see code 14333. For equivalent US Survey foot definition see code 15299.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',36.4,'EPSG','9110','EPSG','8822','Longitude of false origin',-111.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',38.21,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',37.13,'EPSG','9110','EPSG','8826','Easting at false origin',1640419.948,'EPSG','9002','EPSG','8827','Northing at false origin',9842519.685,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11457','conversion','EPSG','15364','EPSG','2259','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15365','SPCS83 Virginia North zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 14531.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',37.4,'EPSG','9110','EPSG','8822','Longitude of false origin',-78.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',39.12,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',38.02,'EPSG','9110','EPSG','8826','Easting at false origin',11482916.667,'EPSG','9003','EPSG','8827','Northing at false origin',6561666.667,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11458','conversion','EPSG','15365','EPSG','2260','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15366','SPCS83 Virginia South zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 14532.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',36.2,'EPSG','9110','EPSG','8822','Longitude of false origin',-78.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',37.58,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',36.46,'EPSG','9110','EPSG','8826','Easting at false origin',11482916.667,'EPSG','9003','EPSG','8827','Northing at false origin',3280833.333,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11459','conversion','EPSG','15366','EPSG','2261','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15367','SPCS83 Washington North zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 14631.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',47.0,'EPSG','9110','EPSG','8822','Longitude of false origin',-120.5,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',48.44,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',47.3,'EPSG','9110','EPSG','8826','Easting at false origin',1640416.667,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11460','conversion','EPSG','15367','EPSG','2273','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15368','SPCS83 Washington South zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 14632.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',45.2,'EPSG','9110','EPSG','8822','Longitude of false origin',-120.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',47.2,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',45.5,'EPSG','9110','EPSG','8826','Easting at false origin',1640416.667,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11461','conversion','EPSG','15368','EPSG','2274','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15369','SPCS83 Wisconsin North zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 14831.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',45.1,'EPSG','9110','EPSG','8822','Longitude of false origin',-90.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',46.46,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',45.34,'EPSG','9110','EPSG','8826','Easting at false origin',1968500.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11462','conversion','EPSG','15369','EPSG','2267','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15370','SPCS83 Wisconsin Central zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 14832.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',43.5,'EPSG','9110','EPSG','8822','Longitude of false origin',-90.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',45.3,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',44.15,'EPSG','9110','EPSG','8826','Easting at false origin',1968500.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11463','conversion','EPSG','15370','EPSG','2266','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15371','SPCS83 Wisconsin South zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 14833.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',42.0,'EPSG','9110','EPSG','8822','Longitude of false origin',-90.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',44.04,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',42.44,'EPSG','9110','EPSG','8826','Easting at false origin',1968500.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11464','conversion','EPSG','15371','EPSG','2268','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15372','SPCS83 Indiana East zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 11331.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',37.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-85.4,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999966667,'EPSG','9201','EPSG','8806','False easting',328083.333,'EPSG','9003','EPSG','8807','False northing',820208.333,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11465','conversion','EPSG','15372','EPSG','2196','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15373','SPCS83 Indiana West zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 11332.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',37.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-87.05,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999966667,'EPSG','9201','EPSG','8806','False easting',2952750.0,'EPSG','9003','EPSG','8807','False northing',820208.333,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11466','conversion','EPSG','15373','EPSG','2197','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15374','Oregon GIC Lambert (international foot)','State law defines grid unit as International feet (note: not US Survey feet). For original metric definition (not used by Oregon state agencies) see proj code 13633.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',41.45,'EPSG','9110','EPSG','8822','Longitude of false origin',-120.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',43.0,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',45.3,'EPSG','9110','EPSG','8826','Easting at false origin',1312335.958,'EPSG','9002','EPSG','8827','Northing at false origin',0.0,'EPSG','9002',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11467','conversion','EPSG','15374','EPSG','1406','EPSG','1135');
INSERT INTO "conversion" VALUES('EPSG','15375','SPCS83 Kentucky Single Zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 11630.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',36.2,'EPSG','9110','EPSG','8822','Longitude of false origin',-85.45,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',37.05,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',38.4,'EPSG','9110','EPSG','8826','Easting at false origin',4921250.0,'EPSG','9003','EPSG','8827','Northing at false origin',3280833.333,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11468','conversion','EPSG','15375','EPSG','1386','EPSG','1135');
INSERT INTO "conversion" VALUES('EPSG','15376','American Samoa Lambert','Per Snyder: Map Projections - a Working Manual: At origin x=500000 ft; y=0 but radius to latitude of origin = -82000000 feet. US National Geodetic Survey confirms value for False Northing.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',-14.16,'EPSG','9110','EPSG','8802','Longitude of natural origin',-170.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9003','EPSG','8807','False northing',312234.65,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11469','conversion','EPSG','15376','EPSG','3109','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15377','SPCS83 Iowa North zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 11431.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',41.3,'EPSG','9110','EPSG','8822','Longitude of false origin',-93.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',43.16,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',42.04,'EPSG','9110','EPSG','8826','Easting at false origin',4921250.0,'EPSG','9003','EPSG','8827','Northing at false origin',3280833.3333,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11470','conversion','EPSG','15377','EPSG','2198','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15378','SPCS83 Iowa South zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 11432.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',40.0,'EPSG','9110','EPSG','8822','Longitude of false origin',-93.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',41.47,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',40.37,'EPSG','9110','EPSG','8826','Easting at false origin',1640416.6667,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11471','conversion','EPSG','15378','EPSG','2199','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15379','SPCS83 Kansas North zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 11531.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',38.2,'EPSG','9110','EPSG','8822','Longitude of false origin',-98.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',39.47,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',38.43,'EPSG','9110','EPSG','8826','Easting at false origin',1312333.3333,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11472','conversion','EPSG','15379','EPSG','2200','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15380','SPCS83 Kansas South zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 11532.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',36.4,'EPSG','9110','EPSG','8822','Longitude of false origin',-98.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',38.34,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',37.16,'EPSG','9110','EPSG','8826','Easting at false origin',1312333.3333,'EPSG','9003','EPSG','8827','Northing at false origin',1312333.3333,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11473','conversion','EPSG','15380','EPSG','2201','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15381','SPCS83 Nevada East zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 12731.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',34.45,'EPSG','9110','EPSG','8802','Longitude of natural origin',-115.35,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',656166.6667,'EPSG','9003','EPSG','8807','False northing',26246666.6667,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11474','conversion','EPSG','15381','EPSG','2224','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15382','SPCS83 Nevada Central zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 12732.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',34.45,'EPSG','9110','EPSG','8802','Longitude of natural origin',-116.4,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',1640416.6667,'EPSG','9003','EPSG','8807','False northing',19685000.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11475','conversion','EPSG','15382','EPSG','2223','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15383','SPCS83 Nevada West zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 12733.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',34.45,'EPSG','9110','EPSG','8802','Longitude of natural origin',-118.35,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',2624666.6667,'EPSG','9003','EPSG','8807','False northing',13123333.3333,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11476','conversion','EPSG','15383','EPSG','2225','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15384','SPCS83 New Jersey zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 12930.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',38.5,'EPSG','9110','EPSG','8802','Longitude of natural origin',-74.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',492125.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11477','conversion','EPSG','15384','EPSG','1399','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15385','SPCS83 Arkansas North zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 10331.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',34.2,'EPSG','9110','EPSG','8822','Longitude of false origin',-92.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',36.14,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',34.56,'EPSG','9110','EPSG','8826','Easting at false origin',1312333.3333,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11478','conversion','EPSG','15385','EPSG','2169','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15386','SPCS83 Arkansas South zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 10332.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',32.4,'EPSG','9110','EPSG','8822','Longitude of false origin',-92.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',34.46,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',33.18,'EPSG','9110','EPSG','8826','Easting at false origin',1312333.3333,'EPSG','9003','EPSG','8827','Northing at false origin',1312333.3333,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11479','conversion','EPSG','15386','EPSG','2170','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15387','SPCS83 Illinois East zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 11231.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',36.4,'EPSG','9110','EPSG','8802','Longitude of natural origin',-88.2,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999975,'EPSG','9201','EPSG','8806','False easting',984250.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11480','conversion','EPSG','15387','EPSG','2194','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15388','SPCS83 Illinois West zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 11232.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',36.4,'EPSG','9110','EPSG','8802','Longitude of natural origin',-90.1,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999941177,'EPSG','9201','EPSG','8806','False easting',2296583.3333,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11481','conversion','EPSG','15388','EPSG','2195','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15389','SPCS83 New Hampshire zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 12830.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',42.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-71.4,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999966667,'EPSG','9201','EPSG','8806','False easting',984250.0,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11482','conversion','EPSG','15389','EPSG','1398','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15390','SPCS83 Rhode Island zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 13830.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',41.05,'EPSG','9110','EPSG','8802','Longitude of natural origin',-71.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.99999375,'EPSG','9201','EPSG','8806','False easting',328083.3333,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11483','conversion','EPSG','15390','EPSG','1408','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15391','SPCS83 Louisiana North zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 11731.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',30.3,'EPSG','9110','EPSG','8822','Longitude of false origin',-92.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',32.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',31.1,'EPSG','9110','EPSG','8826','Easting at false origin',3280833.3333,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11484','conversion','EPSG','15391','EPSG','2204','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15392','SPCS83 Louisiana South zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 11732.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',28.3,'EPSG','9110','EPSG','8822','Longitude of false origin',-91.2,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',30.42,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',29.18,'EPSG','9110','EPSG','8826','Easting at false origin',3280833.3333,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11485','conversion','EPSG','15392','EPSG','2529','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15393','SPCS83 Louisiana Offshore zone (US survey foot)','This projection is NOT used for oil industry purposes. State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 11733.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',25.3,'EPSG','9110','EPSG','8822','Longitude of false origin',-91.2,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',27.5,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',26.1,'EPSG','9110','EPSG','8826','Easting at false origin',3280833.3333,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11486','conversion','EPSG','15393','EPSG','4821','EPSG','1212');
INSERT INTO "conversion" VALUES('EPSG','15394','SPCS83 South Dakota North zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 14031.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',43.5,'EPSG','9110','EPSG','8822','Longitude of false origin',-100.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',45.41,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',44.25,'EPSG','9110','EPSG','8826','Easting at false origin',1968500.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11487','conversion','EPSG','15394','EPSG','2249','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15395','SPCS83 South Dakota South zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 14032.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',42.2,'EPSG','9110','EPSG','8822','Longitude of false origin',-100.2,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',44.24,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',42.5,'EPSG','9110','EPSG','8826','Easting at false origin',1968500.0,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11488','conversion','EPSG','15395','EPSG','2250','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15396','SPCS83 Nebraska zone (US survey foot)','State law defines grid unit as US Survey feet. For equivalent metric Federal definition see code 12630.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',39.5,'EPSG','9110','EPSG','8822','Longitude of false origin',-100.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',43.0,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',40.0,'EPSG','9110','EPSG','8826','Easting at false origin',1640416.6667,'EPSG','9003','EPSG','8827','Northing at false origin',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11489','conversion','EPSG','15396','EPSG','1396','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','15397','Great Lakes Albers','','EPSG','9822','Albers Equal Area','EPSG','8821','Latitude of false origin',45.568977,'EPSG','9102','EPSG','8822','Longitude of false origin',-84.455955,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',42.122774,'EPSG','9102','EPSG','8824','Latitude of 2nd standard parallel',49.01518,'EPSG','9102','EPSG','8826','Easting at false origin',1000000.0,'EPSG','9001','EPSG','8827','Northing at false origin',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11490','conversion','EPSG','15397','EPSG','3467','EPSG','1052');
INSERT INTO "conversion" VALUES('EPSG','15398','Great Lakes and St Lawrence Albers','','EPSG','9822','Albers Equal Area','EPSG','8821','Latitude of false origin',45.568977,'EPSG','9102','EPSG','8822','Longitude of false origin',-83.248627,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',42.122774,'EPSG','9102','EPSG','8824','Latitude of 2nd standard parallel',49.01518,'EPSG','9102','EPSG','8826','Easting at false origin',1000000.0,'EPSG','9001','EPSG','8827','Northing at false origin',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11491','conversion','EPSG','15398','EPSG','3468','EPSG','1052');
INSERT INTO "conversion" VALUES('EPSG','15399','Yap Islands','Origin is station YAP SECOR AMS 1965.','EPSG','9832','Modified Azimuthal Equidistant','EPSG','8801','Latitude of natural origin',9.324815,'EPSG','9110','EPSG','8802','Longitude of natural origin',138.100748,'EPSG','9110','EPSG','8806','False easting',40000.0,'EPSG','9001','EPSG','8807','False northing',60000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11492','conversion','EPSG','15399','EPSG','3108','EPSG','1153');
INSERT INTO "conversion" VALUES('EPSG','15400','Guam SPCS','','EPSG','9831','Guam Projection','EPSG','8801','Latitude of natural origin',13.282087887,'EPSG','9110','EPSG','8802','Longitude of natural origin',144.445550254,'EPSG','9110','EPSG','8806','False easting',50000.0,'EPSG','9001','EPSG','8807','False northing',50000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11493','conversion','EPSG','15400','EPSG','3255','EPSG','1092');
INSERT INTO "conversion" VALUES('EPSG','15498','axis order change (2D)','This conversion allows the creation of a user-defined geographic 2D CRS with axis order longitude, latitude from an EPSG geographic 2D CRS.','EPSG','9843','Axis Order Reversal (2D)',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11509','conversion','EPSG','15498','EPSG','1262','EPSG','1098');
INSERT INTO "conversion" VALUES('EPSG','15499','axis order change (geographic3D horizontal)','This conversion allows the creation of a user-defined geographic 3D CRS with axis order longitude, latitude, ellipsoid height from an EPSG geographic 3D CRS.','EPSG','9844','Axis Order Reversal (Geographic3D horizontal)',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11510','conversion','EPSG','15499','EPSG','1262','EPSG','1098');
INSERT INTO "conversion" VALUES('EPSG','15500','Australian Antarctic geocentric to geog3D','','EPSG','9602','Geographic/geocentric conversions',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11511','conversion','EPSG','15500','EPSG','1278','EPSG','1154');
INSERT INTO "conversion" VALUES('EPSG','15501','Australian Antarctic geog3D to geog2D','See EPSG Guidance Note #7 or Coordinate Operation Method description for techniques for handling reverse conversion.','EPSG','9659','Geographic3D to 2D conversion',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11512','conversion','EPSG','15501','EPSG','1278','EPSG','1155');
INSERT INTO "conversion" VALUES('EPSG','15502','CHTRF95 geocentric to geog3D','','EPSG','9602','Geographic/geocentric conversions',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11513','conversion','EPSG','15502','EPSG','1286','EPSG','1154');
INSERT INTO "conversion" VALUES('EPSG','15503','CHTRF95 geog3D to geog2D','See EPSG Guidance Note #7 or Coordinate Operation Method description for techniques for handling reverse conversion.','EPSG','9659','Geographic3D to 2D conversion',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11514','conversion','EPSG','15503','EPSG','1286','EPSG','1155');
INSERT INTO "conversion" VALUES('EPSG','15504','EST97 geocentric to geog3D','','EPSG','9602','Geographic/geocentric conversions',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11515','conversion','EPSG','15504','EPSG','1090','EPSG','1154');
INSERT INTO "conversion" VALUES('EPSG','15505','EST97 geog3D to geog2D','See EPSG Guidance Note #7 or Coordinate Operation Method description for techniques for handling reverse conversion.','EPSG','9659','Geographic3D to 2D conversion',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11516','conversion','EPSG','15505','EPSG','1090','EPSG','1155');
INSERT INTO "conversion" VALUES('EPSG','15506','ETRS89 geocentric to geog3D','','EPSG','9602','Geographic/geocentric conversions',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11517','conversion','EPSG','15506','EPSG','1298','EPSG','1154');
INSERT INTO "conversion" VALUES('EPSG','15507','ETRS89 geog3D to geog2D','See EPSG Guidance Note #7 or Coordinate Operation Method description for techniques for handling reverse conversion.','EPSG','9659','Geographic3D to 2D conversion',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11518','conversion','EPSG','15507','EPSG','1298','EPSG','1155');
INSERT INTO "conversion" VALUES('EPSG','15508','GDA94 geocentric to geog3D','','EPSG','9602','Geographic/geocentric conversions',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11519','conversion','EPSG','15508','EPSG','2575','EPSG','1154');
INSERT INTO "conversion" VALUES('EPSG','15509','GDA94 geog3D to geog2D','See EPSG Guidance Note #7 or Coordinate Operation Method description for techniques for handling reverse conversion.','EPSG','9659','Geographic3D to 2D conversion',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11520','conversion','EPSG','15509','EPSG','2575','EPSG','1155');
INSERT INTO "conversion" VALUES('EPSG','15510','Hartebeesthoek94 geocentric to geog3D','','EPSG','9602','Geographic/geocentric conversions',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11521','conversion','EPSG','15510','EPSG','1215','EPSG','1154');
INSERT INTO "conversion" VALUES('EPSG','15511','Hartebeesthoek94 geog3D to geog2D','See EPSG Guidance Note #7 or Coordinate Operation Method description for techniques for handling reverse conversion.','EPSG','9659','Geographic3D to 2D conversion',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11522','conversion','EPSG','15511','EPSG','1215','EPSG','1155');
INSERT INTO "conversion" VALUES('EPSG','15512','IRENET95 geocentric to geog3D','','EPSG','9602','Geographic/geocentric conversions',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11523','conversion','EPSG','15512','EPSG','1305','EPSG','1154');
INSERT INTO "conversion" VALUES('EPSG','15513','IRENET95 geog3D to geog2D','See EPSG Guidance Note #7 or Coordinate Operation Method description for techniques for handling reverse conversion.','EPSG','9659','Geographic3D to 2D conversion',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11524','conversion','EPSG','15513','EPSG','1305','EPSG','1155');
INSERT INTO "conversion" VALUES('EPSG','15514','ISN93 geocentric to geog3D','','EPSG','9602','Geographic/geocentric conversions',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11525','conversion','EPSG','15514','EPSG','1120','EPSG','1154');
INSERT INTO "conversion" VALUES('EPSG','15515','ISN93 geog3D to geog2D','See EPSG Guidance Note #7 or Coordinate Operation Method description for techniques for handling reverse conversion.','EPSG','9659','Geographic3D to 2D conversion',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11526','conversion','EPSG','15515','EPSG','1120','EPSG','1155');
INSERT INTO "conversion" VALUES('EPSG','15516','JGD2000 geocentric to geog3D','','EPSG','9602','Geographic/geocentric conversions',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11527','conversion','EPSG','15516','EPSG','1129','EPSG','1154');
INSERT INTO "conversion" VALUES('EPSG','15517','JGD2000 geog3D to geog2D','See EPSG Guidance Note #7 or Coordinate Operation Method description for techniques for handling reverse conversion.','EPSG','9659','Geographic3D to 2D conversion',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11528','conversion','EPSG','15517','EPSG','1129','EPSG','1155');
INSERT INTO "conversion" VALUES('EPSG','15518','LKS92 geocentric to geog3D','','EPSG','9602','Geographic/geocentric conversions',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11529','conversion','EPSG','15518','EPSG','1139','EPSG','1154');
INSERT INTO "conversion" VALUES('EPSG','15519','LKS92 geog3D to geog2D','See EPSG Guidance Note #7 or Coordinate Operation Method description for techniques for handling reverse conversion.','EPSG','9659','Geographic3D to 2D conversion',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11530','conversion','EPSG','15519','EPSG','1139','EPSG','1155');
INSERT INTO "conversion" VALUES('EPSG','15520','LKS94 geocentric to geog3D','','EPSG','9602','Geographic/geocentric conversions',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11531','conversion','EPSG','15520','EPSG','1145','EPSG','1154');
INSERT INTO "conversion" VALUES('EPSG','15521','LKS94 geocentric to geog3D','See EPSG Guidance Note #7 or Coordinate Operation Method description for techniques for handling reverse conversion.','EPSG','9659','Geographic3D to 2D conversion',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11532','conversion','EPSG','15521','EPSG','1145','EPSG','1155');
INSERT INTO "conversion" VALUES('EPSG','15522','Moznet geocentric to geog3D','','EPSG','9602','Geographic/geocentric conversions',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11533','conversion','EPSG','15522','EPSG','1167','EPSG','1154');
INSERT INTO "conversion" VALUES('EPSG','15523','Moznet geog3D to geog2D','See EPSG Guidance Note #7 or Coordinate Operation Method description for techniques for handling reverse conversion.','EPSG','9659','Geographic3D to 2D conversion',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11534','conversion','EPSG','15523','EPSG','1167','EPSG','1155');
INSERT INTO "conversion" VALUES('EPSG','15524','NAD83(CSRS) geocentric to geog3D','','EPSG','9602','Geographic/geocentric conversions',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11535','conversion','EPSG','15524','EPSG','1061','EPSG','1154');
INSERT INTO "conversion" VALUES('EPSG','15525','NAD83(CSRS) geog3D to geog2D','See EPSG Guidance Note #7 or Coordinate Operation Method description for techniques for handling reverse conversion.','EPSG','9659','Geographic3D to 2D conversion',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11536','conversion','EPSG','15525','EPSG','1061','EPSG','1155');
INSERT INTO "conversion" VALUES('EPSG','15526','NAD83(HARN) geocentric to geog3D','','EPSG','9602','Geographic/geocentric conversions',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11537','conversion','EPSG','15526','EPSG','1337','EPSG','1154');
INSERT INTO "conversion" VALUES('EPSG','15527','NAD83(HARN) geog3D to geog2D','See EPSG Guidance Note #7 or Coordinate Operation Method description for techniques for handling reverse conversion.','EPSG','9659','Geographic3D to 2D conversion',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11538','conversion','EPSG','15527','EPSG','1337','EPSG','1155');
INSERT INTO "conversion" VALUES('EPSG','15528','NZGD2000 geocentric to geog3D','','EPSG','9602','Geographic/geocentric conversions',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11539','conversion','EPSG','15528','EPSG','1175','EPSG','1154');
INSERT INTO "conversion" VALUES('EPSG','15529','NZGD2000 geog3D to geog2D','See EPSG Guidance Note #7 or Coordinate Operation Method description for techniques for handling reverse conversion.','EPSG','9659','Geographic3D to 2D conversion',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11540','conversion','EPSG','15529','EPSG','1175','EPSG','1155');
INSERT INTO "conversion" VALUES('EPSG','15530','POSGAR 98 geocentric to geog3D','','EPSG','9602','Geographic/geocentric conversions',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11541','conversion','EPSG','15530','EPSG','1033','EPSG','1154');
INSERT INTO "conversion" VALUES('EPSG','15531','POSGAR 98 geog3D to geog2D','See EPSG Guidance Note #7 or Coordinate Operation Method description for techniques for handling reverse conversion.','EPSG','9659','Geographic3D to 2D conversion',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11542','conversion','EPSG','15531','EPSG','1033','EPSG','1155');
INSERT INTO "conversion" VALUES('EPSG','15532','REGVEN geocentric to geog3D','','EPSG','9602','Geographic/geocentric conversions',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11543','conversion','EPSG','15532','EPSG','1251','EPSG','1154');
INSERT INTO "conversion" VALUES('EPSG','15533','REGVEN geog3D to geog2D','See EPSG Guidance Note #7 or Coordinate Operation Method description for techniques for handling reverse conversion.','EPSG','9659','Geographic3D to 2D conversion',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11544','conversion','EPSG','15533','EPSG','1251','EPSG','1155');
INSERT INTO "conversion" VALUES('EPSG','15534','RGF93 geocentric to geog3D','','EPSG','9602','Geographic/geocentric conversions',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11545','conversion','EPSG','15534','EPSG','1096','EPSG','1154');
INSERT INTO "conversion" VALUES('EPSG','15535','RGF93 geog3D to geog2D','See EPSG Guidance Note #7 or Coordinate Operation Method description for techniques for handling reverse conversion.','EPSG','9659','Geographic3D to 2D conversion',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11546','conversion','EPSG','15535','EPSG','1096','EPSG','1155');
INSERT INTO "conversion" VALUES('EPSG','15536','RGFG95 geocentric to geog3D','','EPSG','9602','Geographic/geocentric conversions',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11547','conversion','EPSG','15536','EPSG','1097','EPSG','1154');
INSERT INTO "conversion" VALUES('EPSG','15537','RGFG95 geog3D to geog2D','See EPSG Guidance Note #7 or Coordinate Operation Method description for techniques for handling reverse conversion.','EPSG','9659','Geographic3D to 2D conversion',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11548','conversion','EPSG','15537','EPSG','1097','EPSG','1155');
INSERT INTO "conversion" VALUES('EPSG','15538','RGNC91-93 geocentric to geog3D','','EPSG','9602','Geographic/geocentric conversions',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11549','conversion','EPSG','15538','EPSG','1174','EPSG','1154');
INSERT INTO "conversion" VALUES('EPSG','15539','RGNC91-93 geog3D to geog2D','See EPSG Guidance Note #7 or Coordinate Operation Method description for techniques for handling reverse conversion.','EPSG','9659','Geographic3D to 2D conversion',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11550','conversion','EPSG','15539','EPSG','1174','EPSG','1155');
INSERT INTO "conversion" VALUES('EPSG','15540','RGR92 geocentric to geog3D','','EPSG','9602','Geographic/geocentric conversions',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11551','conversion','EPSG','15540','EPSG','1196','EPSG','1154');
INSERT INTO "conversion" VALUES('EPSG','15541','RGR92 geog3D to geog2D','See EPSG Guidance Note #7 or Coordinate Operation Method description for techniques for handling reverse conversion.','EPSG','9659','Geographic3D to 2D conversion',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11552','conversion','EPSG','15541','EPSG','1196','EPSG','1155');
INSERT INTO "conversion" VALUES('EPSG','15542','RRAF 1991 geocentric to geog3D','','EPSG','9602','Geographic/geocentric conversions',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11553','conversion','EPSG','15542','EPSG','2824','EPSG','1154');
INSERT INTO "conversion" VALUES('EPSG','15543','RRAF 1991 geog3D to geog2D','See EPSG Guidance Note #7 or Coordinate Operation Method description for techniques for handling reverse conversion.','EPSG','9659','Geographic3D to 2D conversion',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11554','conversion','EPSG','15543','EPSG','2824','EPSG','1155');
INSERT INTO "conversion" VALUES('EPSG','15544','SIRGAS geocentric to geog3D','','EPSG','9602','Geographic/geocentric conversions',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11555','conversion','EPSG','15544','EPSG','3448','EPSG','1154');
INSERT INTO "conversion" VALUES('EPSG','15545','SIRGAS geog3D to geog2D','See EPSG Guidance Note #7 or Coordinate Operation Method description for techniques for handling reverse conversion.','EPSG','9659','Geographic3D to 2D conversion',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11556','conversion','EPSG','15545','EPSG','3448','EPSG','1155');
INSERT INTO "conversion" VALUES('EPSG','15546','SWEREF99 geocentric to geog3D','','EPSG','9602','Geographic/geocentric conversions',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11557','conversion','EPSG','15546','EPSG','1225','EPSG','1154');
INSERT INTO "conversion" VALUES('EPSG','15547','SWEREF99 geog3D to geog2D','See EPSG Guidance Note #7 or Coordinate Operation Method description for techniques for handling reverse conversion.','EPSG','9659','Geographic3D to 2D conversion',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11558','conversion','EPSG','15547','EPSG','1225','EPSG','1155');
INSERT INTO "conversion" VALUES('EPSG','15548','WGS 84 geocentric to geog3D','','EPSG','9602','Geographic/geocentric conversions',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11559','conversion','EPSG','15548','EPSG','1262','EPSG','1154');
INSERT INTO "conversion" VALUES('EPSG','15549','WGS 84 geog3D to geog2D','See EPSG Guidance Note #7 or Coordinate Operation Method description for techniques for handling reverse conversion.','EPSG','9659','Geographic3D to 2D conversion',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11560','conversion','EPSG','15549','EPSG','1262','EPSG','1155');
INSERT INTO "conversion" VALUES('EPSG','15550','Yemen NGN96 geocentric to geog3D','','EPSG','9602','Geographic/geocentric conversions',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11561','conversion','EPSG','15550','EPSG','1257','EPSG','1154');
INSERT INTO "conversion" VALUES('EPSG','15551','Yemen NGN96 geog3D to geog2D','See EPSG Guidance Note #7 or Coordinate Operation Method description for techniques for handling reverse conversion.','EPSG','9659','Geographic3D to 2D conversion',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11562','conversion','EPSG','15551','EPSG','1257','EPSG','1155');
INSERT INTO "conversion" VALUES('EPSG','15552','IGM95 geocentric to geog3D','','EPSG','9602','Geographic/geocentric conversions',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11563','conversion','EPSG','15552','EPSG','1127','EPSG','1154');
INSERT INTO "conversion" VALUES('EPSG','15553','IGM95 geog3D to geog2D','See EPSG Guidance Note #7 or Coordinate Operation Method description for techniques for handling reverse conversion.','EPSG','9659','Geographic3D to 2D conversion',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11564','conversion','EPSG','15553','EPSG','1127','EPSG','1155');
INSERT INTO "conversion" VALUES('EPSG','15554','WGS 72 geocentric to geog3D','','EPSG','9602','Geographic/geocentric conversions',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11565','conversion','EPSG','15554','EPSG','1262','EPSG','1154');
INSERT INTO "conversion" VALUES('EPSG','15555','WGS 72 geog3D to geog2D','See EPSG Guidance Note #7 or Coordinate Operation Method description for techniques for handling reverse conversion.','EPSG','9659','Geographic3D to 2D conversion',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11566','conversion','EPSG','15555','EPSG','1262','EPSG','1155');
INSERT INTO "conversion" VALUES('EPSG','15556','WGS 72BE geocentric to geog3D','','EPSG','9602','Geographic/geocentric conversions',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11567','conversion','EPSG','15556','EPSG','1262','EPSG','1154');
INSERT INTO "conversion" VALUES('EPSG','15557','WGS 72BE geog3D to geog2D','See EPSG Guidance Note #7 or Coordinate Operation Method description for techniques for handling reverse conversion.','EPSG','9659','Geographic3D to 2D conversion',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11568','conversion','EPSG','15557','EPSG','1262','EPSG','1155');
INSERT INTO "conversion" VALUES('EPSG','15558','SIRGAS 2000 geocentric to geog3D','','EPSG','9602','Geographic/geocentric conversions',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11569','conversion','EPSG','15558','EPSG','3418','EPSG','1154');
INSERT INTO "conversion" VALUES('EPSG','15559','SIRGAS 2000 geog3D to geog2D','See EPSG Guidance Note #7 or Coordinate Operation Method description for techniques for handling reverse conversion.','EPSG','9659','Geographic3D to 2D conversion',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11570','conversion','EPSG','15559','EPSG','3418','EPSG','1155');
INSERT INTO "conversion" VALUES('EPSG','15560','Lao 1993 geocentric to geog3D','','EPSG','9602','Geographic/geocentric conversions',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11571','conversion','EPSG','15560','EPSG','1138','EPSG','1154');
INSERT INTO "conversion" VALUES('EPSG','15561','Lao 1993 geog3D to geog2D','See EPSG Guidance Note #7 or Coordinate Operation Method description for techniques for handling reverse conversion.','EPSG','9659','Geographic3D to 2D conversion',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11572','conversion','EPSG','15561','EPSG','1138','EPSG','1155');
INSERT INTO "conversion" VALUES('EPSG','15562','Lao 1997 geocentric to geog3D','','EPSG','9602','Geographic/geocentric conversions',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11573','conversion','EPSG','15562','EPSG','1138','EPSG','1154');
INSERT INTO "conversion" VALUES('EPSG','15563','Lao 1997 geog3D to geog2D','See EPSG Guidance Note #7 or Coordinate Operation Method description for techniques for handling reverse conversion.','EPSG','9659','Geographic3D to 2D conversion',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11574','conversion','EPSG','15563','EPSG','1138','EPSG','1155');
INSERT INTO "conversion" VALUES('EPSG','15564','PRS92 geocentric to geog3D','','EPSG','9602','Geographic/geocentric conversions',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11575','conversion','EPSG','15564','EPSG','1190','EPSG','1154');
INSERT INTO "conversion" VALUES('EPSG','15565','PRS92 geog3D to geog2D','See EPSG Guidance Note #7 or Coordinate Operation Method description for techniques for handling reverse conversion.','EPSG','9659','Geographic3D to 2D conversion',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11576','conversion','EPSG','15565','EPSG','1190','EPSG','1155');
INSERT INTO "conversion" VALUES('EPSG','15566','MAGNA-SIRGAS geocentric to geog3D','','EPSG','9602','Geographic/geocentric conversions',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11577','conversion','EPSG','15566','EPSG','1070','EPSG','1154');
INSERT INTO "conversion" VALUES('EPSG','15567','MAGNA-SIRGAS geog3D to geog2D','See EPSG Guidance Note #7 or Coordinate Operation Method description for techniques for handling reverse conversion.','EPSG','9659','Geographic3D to 2D conversion',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11578','conversion','EPSG','15567','EPSG','1070','EPSG','1155');
INSERT INTO "conversion" VALUES('EPSG','15568','RGPF geocentric to geog3D','','EPSG','9602','Geographic/geocentric conversions',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11579','conversion','EPSG','15568','EPSG','1098','EPSG','1154');
INSERT INTO "conversion" VALUES('EPSG','15569','RGPF geog3D to geog2D','See EPSG Guidance Note #7 or Coordinate Operation Method description for techniques for handling reverse conversion.','EPSG','9659','Geographic3D to 2D conversion',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11580','conversion','EPSG','15569','EPSG','1098','EPSG','1155');
INSERT INTO "conversion" VALUES('EPSG','15570','POSGAR 94 geocentric to geog3D','','EPSG','9602','Geographic/geocentric conversions',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11581','conversion','EPSG','15570','EPSG','1033','EPSG','1154');
INSERT INTO "conversion" VALUES('EPSG','15571','POSGAR 94 geog3D to geog2D','See EPSG Guidance Note #7 or Coordinate Operation Method description for techniques for handling reverse conversion.','EPSG','9659','Geographic3D to 2D conversion',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11582','conversion','EPSG','15571','EPSG','1033','EPSG','1155');
INSERT INTO "conversion" VALUES('EPSG','15572','Korean 2000 geocentric to geog3D','','EPSG','9602','Geographic/geocentric conversions',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11583','conversion','EPSG','15572','EPSG','1135','EPSG','1154');
INSERT INTO "conversion" VALUES('EPSG','15573','Korean 2000 geog3D to geog2D','See EPSG Guidance Note #7 or Coordinate Operation Method description for techniques for handling reverse conversion.','EPSG','9659','Geographic3D to 2D conversion',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11584','conversion','EPSG','15573','EPSG','1135','EPSG','1155');
INSERT INTO "conversion" VALUES('EPSG','15574','Mauritania 1999 geocentric to geog3D','','EPSG','9602','Geographic/geocentric conversions',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11585','conversion','EPSG','15574','EPSG','1157','EPSG','1154');
INSERT INTO "conversion" VALUES('EPSG','15575','Mauritania 1999 geog3D to geog2D','See EPSG Guidance Note #7 or Coordinate Operation Method description for techniques for handling reverse conversion.','EPSG','9659','Geographic3D to 2D conversion',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11586','conversion','EPSG','15575','EPSG','1157','EPSG','1155');
INSERT INTO "conversion" VALUES('EPSG','15576','PZ-90 geocentric to geog3D','','EPSG','9602','Geographic/geocentric conversions',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11587','conversion','EPSG','15576','EPSG','1262','EPSG','1154');
INSERT INTO "conversion" VALUES('EPSG','15577','PZ-90 geog3D to geog2D','See EPSG Guidance Note #7 or Coordinate Operation Method description for techniques for handling reverse conversion.','EPSG','9659','Geographic3D to 2D conversion',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11588','conversion','EPSG','15577','EPSG','1262','EPSG','1155');
INSERT INTO "conversion" VALUES('EPSG','15578','GDM2000 geocentric to geog3D','','EPSG','9602','Geographic/geocentric conversions',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11589','conversion','EPSG','15578','EPSG','1151','EPSG','1154');
INSERT INTO "conversion" VALUES('EPSG','15579','GDM2000 geog3D to geog2D','See EPSG Guidance Note #7 or Coordinate Operation Method description for techniques for handling reverse conversion.','EPSG','9659','Geographic3D to 2D conversion',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11590','conversion','EPSG','15579','EPSG','1151','EPSG','1155');
INSERT INTO "conversion" VALUES('EPSG','15580','GR96 geocentric to geog3D','','EPSG','9602','Geographic/geocentric conversions',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11591','conversion','EPSG','15580','EPSG','1107','EPSG','1154');
INSERT INTO "conversion" VALUES('EPSG','15581','GR96 geog3D to geog2D','See EPSG Guidance Note #7 or Coordinate Operation Method description for techniques for handling reverse conversion.','EPSG','9659','Geographic3D to 2D conversion',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11592','conversion','EPSG','15581','EPSG','1107','EPSG','1155');
INSERT INTO "conversion" VALUES('EPSG','15582','LGD2006 geocentric to geog3D','','EPSG','9602','Geographic/geocentric conversions',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11593','conversion','EPSG','15582','EPSG','1143','EPSG','1154');
INSERT INTO "conversion" VALUES('EPSG','15583','LGD2006 geog3D to geog2D','See EPSG Guidance Note #7 or Coordinate Operation Method description for techniques for handling reverse conversion.','EPSG','9659','Geographic3D to 2D conversion',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11594','conversion','EPSG','15583','EPSG','1143','EPSG','1155');
INSERT INTO "conversion" VALUES('EPSG','15584','DGN95 geocentric to geog3D','','EPSG','9602','Geographic/geocentric conversions',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11595','conversion','EPSG','15584','EPSG','1122','EPSG','1154');
INSERT INTO "conversion" VALUES('EPSG','15585','DGN95 geog3D to geog2D','See EPSG Guidance Note #7 or Coordinate Operation Method description for techniques for handling reverse conversion.','EPSG','9659','Geographic3D to 2D conversion',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11596','conversion','EPSG','15585','EPSG','1122','EPSG','1155');
INSERT INTO "conversion" VALUES('EPSG','15586','JAD2001 geocentric to geog3D','','EPSG','9602','Geographic/geocentric conversions',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11597','conversion','EPSG','15586','EPSG','1128','EPSG','1154');
INSERT INTO "conversion" VALUES('EPSG','15587','JAD2001 geog3D to geog2D','See EPSG Guidance Note #7 or Coordinate Operation Method description for techniques for handling reverse conversion.','EPSG','9659','Geographic3D to 2D conversion',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11598','conversion','EPSG','15587','EPSG','1128','EPSG','1155');
INSERT INTO "conversion" VALUES('EPSG','15588','NAD83(NSRS2007) geocentric to geog3D','','EPSG','9602','Geographic/geocentric conversions',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11599','conversion','EPSG','15588','EPSG','1511','EPSG','1154');
INSERT INTO "conversion" VALUES('EPSG','15589','NAD83(NSRS2007) geog3D to geog2D','See EPSG Guidance Note #7 or Coordinate Operation Method description for techniques for handling reverse conversion.','EPSG','9659','Geographic3D to 2D conversion',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11600','conversion','EPSG','15589','EPSG','1511','EPSG','1155');
INSERT INTO "conversion" VALUES('EPSG','15590','WGS 66 geocentric to geog3D','','EPSG','9602','Geographic/geocentric conversions',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11601','conversion','EPSG','15590','EPSG','1262','EPSG','1154');
INSERT INTO "conversion" VALUES('EPSG','15591','WGS 66 geog3D to geog2D','See EPSG Guidance Note #7 or Coordinate Operation Method description for techniques for handling reverse conversion.','EPSG','9659','Geographic3D to 2D conversion',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','11602','conversion','EPSG','15591','EPSG','1262','EPSG','1155');
INSERT INTO "conversion" VALUES('EPSG','15592','geocentric to geographic3D','','EPSG','9602','Geographic/geocentric conversions',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11603','conversion','EPSG','15592','EPSG','1262','EPSG','1154');
INSERT INTO "conversion" VALUES('EPSG','15593','geographic3D to geographic2D','See EPSG Guidance Note #7 or Coordinate Operation Method description for techniques for handling reverse conversion.','EPSG','9659','Geographic3D to 2D conversion',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11604','conversion','EPSG','15593','EPSG','1262','EPSG','1155');
INSERT INTO "conversion" VALUES('EPSG','15594','EPSG topocentric example A','Example only.','EPSG','9837','Geographic/topocentric conversions','EPSG','8834','Latitude of topocentric origin',55.0,'EPSG','9102','EPSG','8835','Longitude of topocentric origin',5.0,'EPSG','9102','EPSG','8836','Ellipsoidal height of topocentric origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11605','conversion','EPSG','15594','EPSG','1263','EPSG','1117');
INSERT INTO "conversion" VALUES('EPSG','15595','EPSG topocentric example B','Example only.','EPSG','9836','Geocentric/topocentric conversions','EPSG','8837','Geocentric X of topocentric origin',3771793.97,'EPSG','9001','EPSG','8838','Geocentric Y of topocentric origin',140253.34,'EPSG','9001','EPSG','8839','Geocentric Z of topocentric origin',5124304.35,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11606','conversion','EPSG','15595','EPSG','1263','EPSG','1118');
INSERT INTO "conversion" VALUES('EPSG','15914','BLM zone 14N (US survey foot)','US survey foot form of UTM zone 14N. Sometimes locally referred to as "UTM zone 14".','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-99.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',1640416.67,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11925','conversion','EPSG','15914','EPSG','3637','EPSG','1153');
INSERT INTO "conversion" VALUES('EPSG','15915','BLM zone 15N (US survey foot)','US survey foot form of UTM zone 15N. Sometimes locally referred to as "UTM zone 15".','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-93.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',1640416.67,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11926','conversion','EPSG','15915','EPSG','3640','EPSG','1153');
INSERT INTO "conversion" VALUES('EPSG','15916','BLM zone 16N (US survey foot)','US survey foot form of UTM zone 16N. Sometimes locally referred to as "UTM zone 16".','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-87.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',1640416.67,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11927','conversion','EPSG','15916','EPSG','3641','EPSG','1153');
INSERT INTO "conversion" VALUES('EPSG','15917','BLM zone 17N (US survey foot)','US survey foot form of UTM zone 17N. Sometimes locally referred to as "UTM zone 17".','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-81.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',1640416.67,'EPSG','9003','EPSG','8807','False northing',0.0,'EPSG','9003',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','11928','conversion','EPSG','15917','EPSG','3642','EPSG','1153');
INSERT INTO "conversion" VALUES('EPSG','16000','UTM grid system (northern hemisphere)','Use UTM zone xx N (codes 16001-16060) for use outwith zone boundary or when easting is not prefixed by zone number.','EPSG','9824','Transverse Mercator Zoned Grid System','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8830','Initial longitude',-180.0,'EPSG','9102','EPSG','8831','Zone width',6.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12010','conversion','EPSG','16000','EPSG','1998','EPSG','1163');
INSERT INTO "conversion" VALUES('EPSG','16001','UTM zone 1N','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-177.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12011','conversion','EPSG','16001','EPSG','1873','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16002','UTM zone 2N','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-171.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12012','conversion','EPSG','16002','EPSG','1875','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16003','UTM zone 3N','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-165.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12013','conversion','EPSG','16003','EPSG','1877','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16004','UTM zone 4N','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-159.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12014','conversion','EPSG','16004','EPSG','1879','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16005','UTM zone 5N','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-153.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12015','conversion','EPSG','16005','EPSG','1881','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16006','UTM zone 6N','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-147.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12016','conversion','EPSG','16006','EPSG','1883','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16007','UTM zone 7N','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-141.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12017','conversion','EPSG','16007','EPSG','1885','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16008','UTM zone 8N','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-135.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12018','conversion','EPSG','16008','EPSG','1887','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16009','UTM zone 9N','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-129.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12019','conversion','EPSG','16009','EPSG','1889','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16010','UTM zone 10N','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-123.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12020','conversion','EPSG','16010','EPSG','1891','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16011','UTM zone 11N','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-117.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12021','conversion','EPSG','16011','EPSG','1893','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16012','UTM zone 12N','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-111.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12022','conversion','EPSG','16012','EPSG','1895','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16013','UTM zone 13N','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-105.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12023','conversion','EPSG','16013','EPSG','1897','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16014','UTM zone 14N','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-99.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12024','conversion','EPSG','16014','EPSG','1899','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16015','UTM zone 15N','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-93.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12025','conversion','EPSG','16015','EPSG','1901','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16016','UTM zone 16N','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-87.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12026','conversion','EPSG','16016','EPSG','1903','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16017','UTM zone 17N','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-81.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12027','conversion','EPSG','16017','EPSG','1905','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16018','UTM zone 18N','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-75.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12028','conversion','EPSG','16018','EPSG','1907','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16019','UTM zone 19N','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-69.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12029','conversion','EPSG','16019','EPSG','1909','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16020','UTM zone 20N','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-63.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12030','conversion','EPSG','16020','EPSG','1911','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16021','UTM zone 21N','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-57.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12031','conversion','EPSG','16021','EPSG','1913','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16022','UTM zone 22N','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-51.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12032','conversion','EPSG','16022','EPSG','1915','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16023','UTM zone 23N','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-45.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12033','conversion','EPSG','16023','EPSG','1917','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16024','UTM zone 24N','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-39.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12034','conversion','EPSG','16024','EPSG','1919','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16025','UTM zone 25N','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-33.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12035','conversion','EPSG','16025','EPSG','1921','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16026','UTM zone 26N','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-27.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12036','conversion','EPSG','16026','EPSG','1923','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16027','UTM zone 27N','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-21.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12037','conversion','EPSG','16027','EPSG','1925','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16028','UTM zone 28N','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-15.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12038','conversion','EPSG','16028','EPSG','1927','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16029','UTM zone 29N','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-9.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12039','conversion','EPSG','16029','EPSG','1929','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16030','UTM zone 30N','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-3.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12040','conversion','EPSG','16030','EPSG','1931','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16031','UTM zone 31N','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',3.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12041','conversion','EPSG','16031','EPSG','1933','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16032','UTM zone 32N','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',9.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12042','conversion','EPSG','16032','EPSG','1935','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16033','UTM zone 33N','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',15.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12043','conversion','EPSG','16033','EPSG','1937','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16034','UTM zone 34N','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',21.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12044','conversion','EPSG','16034','EPSG','1939','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16035','UTM zone 35N','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',27.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12045','conversion','EPSG','16035','EPSG','1941','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16036','UTM zone 36N','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',33.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12046','conversion','EPSG','16036','EPSG','1943','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16037','UTM zone 37N','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',39.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12047','conversion','EPSG','16037','EPSG','1945','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16038','UTM zone 38N','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',45.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12048','conversion','EPSG','16038','EPSG','1947','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16039','UTM zone 39N','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',51.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12049','conversion','EPSG','16039','EPSG','1949','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16040','UTM zone 40N','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',57.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12050','conversion','EPSG','16040','EPSG','1951','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16041','UTM zone 41N','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',63.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12051','conversion','EPSG','16041','EPSG','1953','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16042','UTM zone 42N','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',69.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12052','conversion','EPSG','16042','EPSG','1955','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16043','UTM zone 43N','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',75.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12053','conversion','EPSG','16043','EPSG','1957','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16044','UTM zone 44N','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',81.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12054','conversion','EPSG','16044','EPSG','1959','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16045','UTM zone 45N','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',87.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12055','conversion','EPSG','16045','EPSG','1961','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16046','UTM zone 46N','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',93.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12056','conversion','EPSG','16046','EPSG','1963','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16047','UTM zone 47N','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',99.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12057','conversion','EPSG','16047','EPSG','1965','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16048','UTM zone 48N','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',105.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12058','conversion','EPSG','16048','EPSG','1967','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16049','UTM zone 49N','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',111.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12059','conversion','EPSG','16049','EPSG','1969','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16050','UTM zone 50N','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',117.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12060','conversion','EPSG','16050','EPSG','1971','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16051','UTM zone 51N','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',123.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12061','conversion','EPSG','16051','EPSG','1973','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16052','UTM zone 52N','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',129.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12062','conversion','EPSG','16052','EPSG','1975','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16053','UTM zone 53N','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',135.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12063','conversion','EPSG','16053','EPSG','1977','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16054','UTM zone 54N','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',141.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12064','conversion','EPSG','16054','EPSG','1979','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16055','UTM zone 55N','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',147.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12065','conversion','EPSG','16055','EPSG','1981','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16056','UTM zone 56N','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',153.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12066','conversion','EPSG','16056','EPSG','1983','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16057','UTM zone 57N','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',159.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12067','conversion','EPSG','16057','EPSG','1985','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16058','UTM zone 58N','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',165.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12068','conversion','EPSG','16058','EPSG','1987','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16059','UTM zone 59N','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',171.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12069','conversion','EPSG','16059','EPSG','1989','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16060','UTM zone 60N','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',177.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12070','conversion','EPSG','16060','EPSG','1991','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16061','Universal Polar Stereographic North','','EPSG','9810','Polar Stereographic (variant A)','EPSG','8801','Latitude of natural origin',90.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',0.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.994,'EPSG','9201','EPSG','8806','False easting',2000000.0,'EPSG','9001','EPSG','8807','False northing',2000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12071','conversion','EPSG','16061','EPSG','1996','EPSG','1160');
INSERT INTO "conversion" VALUES('EPSG','16065','TM35FIN','Identical to UTM zone 35N (code 16035) except for area of use. TM35FIN is used in conjunction with EUREF-FIN for mapping all Finnish territory (from 19°E to 31.6°E) in a single zone.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',27.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12072','conversion','EPSG','16065','EPSG','1095','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16070','3-degree Gauss-Kruger zone 40','Also found with zone truncated from false easting: see 3-degree Gauss-Kruger cm 120E (code 16170). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',120.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',40500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12073','conversion','EPSG','16070','EPSG','2628','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16071','3-degree Gauss-Kruger zone 41','Also found with zone truncated from false easting: see Gauss-Kruger cm 123E (code 16321). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',123.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',41500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12074','conversion','EPSG','16071','EPSG','2629','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16072','3-degree Gauss-Kruger zone 42','Also found with zone truncated from false easting: see 3-degree Gauss-Kruger cm 126E (code 16172). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',126.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',42500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12075','conversion','EPSG','16072','EPSG','2630','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16073','3-degree Gauss-Kruger zone 43','Also found with zone truncated from false easting: see Gauss-Kruger cm 129E (code 16322). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',129.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',43500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12076','conversion','EPSG','16073','EPSG','2631','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16074','3-degree Gauss-Kruger zone 44','Also found with zone truncated from false easting: see 3-degree Gauss-Kruger cm 132E (code 16174). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',132.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',44500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12077','conversion','EPSG','16074','EPSG','2632','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16075','3-degree Gauss-Kruger zone 45','Also found with zone truncated from false easting: see Gauss-Kruger cm 135E (code 16323). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',135.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',45500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12078','conversion','EPSG','16075','EPSG','2633','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16076','3-degree Gauss-Kruger zone 46','Also found with zone truncated from false easting: see 3-degree Gauss-Kruger cm 138E (code 16176). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',138.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',46500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12079','conversion','EPSG','16076','EPSG','2634','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16077','3-degree Gauss-Kruger zone 47','Also found with zone truncated from false easting: see Gauss-Kruger cm 141E (code 16324). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',141.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',47500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12080','conversion','EPSG','16077','EPSG','2635','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16078','3-degree Gauss-Kruger zone 48','Also found with zone truncated from false easting: see 3-degree Gauss-Kruger cm 144E (code 16178). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',144.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',48500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12081','conversion','EPSG','16078','EPSG','2636','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16079','3-degree Gauss-Kruger zone 49','Also found with zone truncated from false easting: see Gauss-Kruger cm 147E (code 16325). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',147.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',49500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12082','conversion','EPSG','16079','EPSG','2637','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16080','3-degree Gauss-Kruger zone 50','Also found with zone truncated from false easting: see 3-degree Gauss-Kruger cm 150E (code 16180). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',150.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',50500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12083','conversion','EPSG','16080','EPSG','2638','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16081','3-degree Gauss-Kruger zone 51','Also found with zone truncated from false easting: see Gauss-Kruger cm 153E (code 16326). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',153.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',51500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12084','conversion','EPSG','16081','EPSG','2639','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16082','3-degree Gauss-Kruger zone 52','Also found with zone truncated from false easting: see 3-degree Gauss-Kruger cm 156E (code 16182). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',156.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',52500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12085','conversion','EPSG','16082','EPSG','2640','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16083','3-degree Gauss-Kruger zone 53','Also found with zone truncated from false easting: see Gauss-Kruger cm 159E (code 16327). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',159.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',53500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12086','conversion','EPSG','16083','EPSG','2641','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16084','3-degree Gauss-Kruger zone 54','Also found with zone truncated from false easting: see 3-degree Gauss-Kruger cm 162E (code 16184). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',162.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',54500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12087','conversion','EPSG','16084','EPSG','2642','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16085','3-degree Gauss-Kruger zone 55','Also found with zone truncated from false easting: see Gauss-Kruger cm 165E (code 16328). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',165.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',55500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12088','conversion','EPSG','16085','EPSG','2643','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16086','3-degree Gauss-Kruger zone 56','Also found with zone truncated from false easting: see 3-degree Gauss-Kruger cm 168E (code 16186). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',168.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',56500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12089','conversion','EPSG','16086','EPSG','2644','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16087','3-degree Gauss-Kruger zone 57','Also found with zone truncated from false easting: see Gauss-Kruger cm 171E (code 16329). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',171.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',57500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12090','conversion','EPSG','16087','EPSG','2645','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16088','3-degree Gauss-Kruger zone 58','Also found with zone truncated from false easting: see 3-degree Gauss-Kruger cm 174E (code 16188). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',174.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',58500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12091','conversion','EPSG','16088','EPSG','2646','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16089','3-degree Gauss-Kruger zone 59','Also found with zone truncated from false easting: see Gauss-Kruger cm 177E (code 16330). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',177.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',59500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12092','conversion','EPSG','16089','EPSG','2647','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16090','3-degree Gauss-Kruger zone 60','Also found with zone truncated from false easting: see 3-degree Gauss-Kruger cm 180 (code 16190). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',180.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',60000000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','12093','conversion','EPSG','16090','EPSG','2648','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16091','3-degree Gauss-Kruger zone 61','Also found with zone truncated from false easting: see Gauss-Kruger cm 177W (code 16331). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-177.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',61500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12094','conversion','EPSG','16091','EPSG','2649','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16092','3-degree Gauss-Kruger zone 62','Also found with zone truncated from false easting: see 3-degree Gauss-Kruger cm 174W (code 16192). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-174.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',62500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12095','conversion','EPSG','16092','EPSG','2650','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16093','3-degree Gauss-Kruger zone 63','Also found with zone truncated from false easting: see Gauss-Kruger cm 171W (code 16332). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-171.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',63500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12096','conversion','EPSG','16093','EPSG','2651','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16094','3-degree Gauss-Kruger zone 64','Also found with zone truncated from false easting: see 3-degree Gauss-Kruger cm 168W (code 16194). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-168.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',64500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12097','conversion','EPSG','16094','EPSG','2652','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16099','3-degree Gauss-Kruger zone 60','Also found with zone truncated from false easting: see 3-degree Gauss-Kruger CM 180E (code 16190).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',180.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',60500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12098','conversion','EPSG','16099','EPSG','2648','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16100','UTM grid system (southern hemisphere)','Use UTM zone xx S (codes 16101-16160) for use outwith zone boundary or when easting is not prefixed by zone number.','EPSG','9824','Transverse Mercator Zoned Grid System','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8830','Initial longitude',-180.0,'EPSG','9102','EPSG','8831','Zone width',6.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12099','conversion','EPSG','16100','EPSG','1999','EPSG','1163');
INSERT INTO "conversion" VALUES('EPSG','16101','UTM zone 1S','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-177.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12100','conversion','EPSG','16101','EPSG','1874','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16102','UTM zone 2S','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-171.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12101','conversion','EPSG','16102','EPSG','1876','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16103','UTM zone 3S','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-165.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12102','conversion','EPSG','16103','EPSG','1878','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16104','UTM zone 4S','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-159.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12103','conversion','EPSG','16104','EPSG','1880','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16105','UTM zone 5S','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-153.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12104','conversion','EPSG','16105','EPSG','1882','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16106','UTM zone 6S','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-147.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12105','conversion','EPSG','16106','EPSG','1884','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16107','UTM zone 7S','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-141.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12106','conversion','EPSG','16107','EPSG','1886','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16108','UTM zone 8S','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-135.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12107','conversion','EPSG','16108','EPSG','1888','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16109','UTM zone 9S','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-129.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12108','conversion','EPSG','16109','EPSG','1890','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16110','UTM zone 10S','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-123.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12109','conversion','EPSG','16110','EPSG','1892','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16111','UTM zone 11S','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-117.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12110','conversion','EPSG','16111','EPSG','1894','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16112','UTM zone 12S','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-111.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12111','conversion','EPSG','16112','EPSG','1896','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16113','UTM zone 13S','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-105.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12112','conversion','EPSG','16113','EPSG','1898','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16114','UTM zone 14S','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-99.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12113','conversion','EPSG','16114','EPSG','1900','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16115','UTM zone 15S','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-93.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12114','conversion','EPSG','16115','EPSG','1902','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16116','UTM zone 16S','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-87.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12115','conversion','EPSG','16116','EPSG','1904','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16117','UTM zone 17S','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-81.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12116','conversion','EPSG','16117','EPSG','1906','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16118','UTM zone 18S','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-75.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12117','conversion','EPSG','16118','EPSG','1908','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16119','UTM zone 19S','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-69.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12118','conversion','EPSG','16119','EPSG','1910','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16120','UTM zone 20S','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-63.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12119','conversion','EPSG','16120','EPSG','1912','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16121','UTM zone 21S','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-57.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12120','conversion','EPSG','16121','EPSG','1914','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16122','UTM zone 22S','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-51.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12121','conversion','EPSG','16122','EPSG','1916','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16123','UTM zone 23S','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-45.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12122','conversion','EPSG','16123','EPSG','1918','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16124','UTM zone 24S','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-39.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12123','conversion','EPSG','16124','EPSG','1920','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16125','UTM zone 25S','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-33.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12124','conversion','EPSG','16125','EPSG','1922','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16126','UTM zone 26S','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-27.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12125','conversion','EPSG','16126','EPSG','1924','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16127','UTM zone 27S','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-21.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12126','conversion','EPSG','16127','EPSG','1926','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16128','UTM zone 28S','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-15.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12127','conversion','EPSG','16128','EPSG','1928','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16129','UTM zone 29S','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-9.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12128','conversion','EPSG','16129','EPSG','1930','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16130','UTM zone 30S','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-3.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12129','conversion','EPSG','16130','EPSG','1932','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16131','UTM zone 31S','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',3.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12130','conversion','EPSG','16131','EPSG','1934','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16132','UTM zone 32S','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',9.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12131','conversion','EPSG','16132','EPSG','1936','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16133','UTM zone 33S','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',15.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12132','conversion','EPSG','16133','EPSG','1938','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16134','UTM zone 34S','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',21.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12133','conversion','EPSG','16134','EPSG','1940','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16135','UTM zone 35S','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',27.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12134','conversion','EPSG','16135','EPSG','1942','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16136','UTM zone 36S','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',33.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12135','conversion','EPSG','16136','EPSG','1944','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16137','UTM zone 37S','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',39.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12136','conversion','EPSG','16137','EPSG','1946','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16138','UTM zone 38S','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',45.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12137','conversion','EPSG','16138','EPSG','1948','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16139','UTM zone 39S','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',51.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12138','conversion','EPSG','16139','EPSG','1950','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16140','UTM zone 40S','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',57.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12139','conversion','EPSG','16140','EPSG','1952','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16141','UTM zone 41S','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',63.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12140','conversion','EPSG','16141','EPSG','1954','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16142','UTM zone 42S','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',69.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12141','conversion','EPSG','16142','EPSG','1956','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16143','UTM zone 43S','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',75.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12142','conversion','EPSG','16143','EPSG','1958','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16144','UTM zone 44S','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',81.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12143','conversion','EPSG','16144','EPSG','1960','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16145','UTM zone 45S','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',87.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12144','conversion','EPSG','16145','EPSG','1962','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16146','UTM zone 46S','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',93.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12145','conversion','EPSG','16146','EPSG','1964','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16147','UTM zone 47S','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',99.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12146','conversion','EPSG','16147','EPSG','1966','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16148','UTM zone 48S','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',105.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12147','conversion','EPSG','16148','EPSG','1968','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16149','UTM zone 49S','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',111.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12148','conversion','EPSG','16149','EPSG','1970','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16150','UTM zone 50S','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',117.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12149','conversion','EPSG','16150','EPSG','1972','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16151','UTM zone 51S','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',123.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12150','conversion','EPSG','16151','EPSG','1974','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16152','UTM zone 52S','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',129.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12151','conversion','EPSG','16152','EPSG','1976','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16153','UTM zone 53S','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',135.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12152','conversion','EPSG','16153','EPSG','1978','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16154','UTM zone 54S','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',141.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12153','conversion','EPSG','16154','EPSG','1980','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16155','UTM zone 55S','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',147.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12154','conversion','EPSG','16155','EPSG','1982','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16156','UTM zone 56S','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',153.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12155','conversion','EPSG','16156','EPSG','1984','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16157','UTM zone 57S','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',159.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12156','conversion','EPSG','16157','EPSG','1986','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16158','UTM zone 58S','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',165.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12157','conversion','EPSG','16158','EPSG','1988','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16159','UTM zone 59S','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',171.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12158','conversion','EPSG','16159','EPSG','1990','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16160','UTM zone 60S','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',177.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12159','conversion','EPSG','16160','EPSG','1992','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','16161','Universal Polar Stereographic South','','EPSG','9810','Polar Stereographic (variant A)','EPSG','8801','Latitude of natural origin',-90.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',0.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.994,'EPSG','9201','EPSG','8806','False easting',2000000.0,'EPSG','9001','EPSG','8807','False northing',2000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12160','conversion','EPSG','16161','EPSG','1997','EPSG','1160');
INSERT INTO "conversion" VALUES('EPSG','16170','3-degree Gauss-Kruger CM 120E','3-degree Gauss-Kruger zone 40N (code 16070) without zone prefix to false easting value. Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',120.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12161','conversion','EPSG','16170','EPSG','2628','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16171','3-degree Gauss-Kruger CM 123E','3-degree Gauss-Kruger zone 41N (code 16071) without zone prefix to false easting value. Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',123.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','12162','conversion','EPSG','16171','EPSG','2629','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16172','3-degree Gauss-Kruger CM 126E','3-degree Gauss-Kruger zone 42N (code 16072) without zone prefix to false easting value. Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',126.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12163','conversion','EPSG','16172','EPSG','2630','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16173','3-degree Gauss-Kruger CM 129E','3-degree Gauss-Kruger zone 43N (code 16073) without zone prefix to false easting value. Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',129.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','12164','conversion','EPSG','16173','EPSG','2631','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16174','3-degree Gauss-Kruger CM 132E','3-degree Gauss-Kruger zone 44N (code 16074) without zone prefix to false easting value. Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',132.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12165','conversion','EPSG','16174','EPSG','2632','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16175','3-degree Gauss-Kruger CM 135E','3-degree Gauss-Kruger zone 45N (code 16075) without zone prefix to false easting value. Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',135.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','12166','conversion','EPSG','16175','EPSG','2633','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16176','3-degree Gauss-Kruger CM 138E','3-degree Gauss-Kruger zone 46N (code 16076) without zone prefix to false easting value. Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',138.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12167','conversion','EPSG','16176','EPSG','2634','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16177','3-degree Gauss-Kruger CM 141E','3-degree Gauss-Kruger zone 47N (code 16077) without zone prefix to false easting value. Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',141.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','12168','conversion','EPSG','16177','EPSG','2635','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16178','3-degree Gauss-Kruger CM 144E','3-degree Gauss-Kruger zone 48N (code 16078) without zone prefix to false easting value. Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',144.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12169','conversion','EPSG','16178','EPSG','2636','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16179','3-degree Gauss-Kruger CM 147E','3-degree Gauss-Kruger zone 49N (code 16079) without zone prefix to false easting value. Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',147.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','12170','conversion','EPSG','16179','EPSG','2637','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16180','3-degree Gauss-Kruger CM 150E','3-degree Gauss-Kruger zone 50N (code 16080) without zone prefix to false easting value. Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',150.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12171','conversion','EPSG','16180','EPSG','2638','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16181','3-degree Gauss-Kruger CM 153E','3-degree Gauss-Kruger zone 51N (code 16081) without zone prefix to false easting value. Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',153.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','12172','conversion','EPSG','16181','EPSG','2639','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16182','3-degree Gauss-Kruger CM 156E','3-degree Gauss-Kruger zone 52N (code 16082) without zone prefix to false easting value. Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',156.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12173','conversion','EPSG','16182','EPSG','2640','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16183','3-degree Gauss-Kruger CM 159E','3-degree Gauss-Kruger zone 53N (code 16083) without zone prefix to false easting value. Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',159.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','12174','conversion','EPSG','16183','EPSG','2641','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16184','3-degree Gauss-Kruger CM 162E','3-degree Gauss-Kruger zone 54N (code 16084) without zone prefix to false easting value. Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',162.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12175','conversion','EPSG','16184','EPSG','2642','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16185','3-degree Gauss-Kruger CM 165E','3-degree Gauss-Kruger zone 55N (code 16085) without zone prefix to false easting value. Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',165.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','12176','conversion','EPSG','16185','EPSG','2643','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16186','3-degree Gauss-Kruger CM 168E','3-degree Gauss-Kruger zone 56N (code 16086) without zone prefix to false easting value. Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',168.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12177','conversion','EPSG','16186','EPSG','2644','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16187','3-degree Gauss-Kruger CM 171E','3-degree Gauss-Kruger zone 57N (code 16087) without zone prefix to false easting value. Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',171.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','12178','conversion','EPSG','16187','EPSG','2645','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16188','3-degree Gauss-Kruger CM 174E','3-degree Gauss-Kruger zone 58N (code 16088) without zone prefix to false easting value. Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',174.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12179','conversion','EPSG','16188','EPSG','2646','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16189','3-degree Gauss-Kruger CM 177E','3-degree Gauss-Kruger zone 59N (code 16089) without zone prefix to false easting value. Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',177.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','12180','conversion','EPSG','16189','EPSG','2647','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16190','3-degree Gauss-Kruger CM 180E','3-degree Gauss-Kruger zone 60N (code 16099) without zone prefix to false easting value.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',180.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12181','conversion','EPSG','16190','EPSG','2648','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16191','3-degree Gauss-Kruger CM 177W','3-degree Gauss-Kruger zone 61N (code 16091) without zone prefix to false easting value. Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-177.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','12182','conversion','EPSG','16191','EPSG','2649','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16192','3-degree Gauss-Kruger CM 174W','3-degree Gauss-Kruger zone 62N (code 16092) without zone prefix to false easting value. Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-174.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12183','conversion','EPSG','16192','EPSG','2650','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16193','3-degree Gauss-Kruger CM 171W','3-degree Gauss-Kruger zone 63N (code 16093) without zone prefix to false easting value. Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-171.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','12184','conversion','EPSG','16193','EPSG','2651','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16194','3-degree Gauss-Kruger CM 168W','3-degree Gauss-Kruger zone 64N (code 16094) without zone prefix to false easting value. Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-168.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12185','conversion','EPSG','16194','EPSG','2652','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16201','6-degree Gauss-Kruger zone 1','Also found with zone truncated from false easting: see 6-degree Gauss-Kruger cm 3E (code 16301). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',3.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',1500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12186','conversion','EPSG','16201','EPSG','1933','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16202','6-degree Gauss-Kruger zone 2','Also found with zone truncated from false easting: see 6-degree Gauss-Kruger CM 9E (code 16302).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',9.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',2500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12187','conversion','EPSG','16202','EPSG','2741','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16203','6-degree Gauss-Kruger zone 3','Also found with zone truncated from false easting: see 6-degree Gauss-Kruger CM 15E (code 16303).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',15.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',3500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12188','conversion','EPSG','16203','EPSG','2742','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16204','6-degree Gauss-Kruger zone 4','Also found with zone truncated from false easting: see 6-degree Gauss-Kruger CM 21E (code 16304).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',21.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',4500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12189','conversion','EPSG','16204','EPSG','2743','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16205','6-degree Gauss-Kruger zone 5','Also found with zone truncated from false easting: see 6-degree Gauss-Kruger CM 27E (code 16305).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',27.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',5500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12190','conversion','EPSG','16205','EPSG','2744','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16206','6-degree Gauss-Kruger zone 6','Also found with zone truncated from false easting: see 6-degree Gauss-Kruger CM 33E (code 16306).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',33.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',6500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12191','conversion','EPSG','16206','EPSG','2745','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16207','6-degree Gauss-Kruger zone 7','Also found with zone truncated from false easting: see 6-degree Gauss-Kruger CM 39E (code 16307).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',39.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',7500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12192','conversion','EPSG','16207','EPSG','2746','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16208','6-degree Gauss-Kruger zone 8','Also found with zone truncated from false easting: see 6-degree Gauss-Kruger CM 45E (code 16308).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',45.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',8500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12193','conversion','EPSG','16208','EPSG','1947','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16209','6-degree Gauss-Kruger zone 9','Also found with zone truncated from false easting: see 6-degree Gauss-Kruger CM 51E (code 16309).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',51.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',9500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12194','conversion','EPSG','16209','EPSG','1949','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16210','6-degree Gauss-Kruger zone 10','Also found with zone truncated from false easting: see 6-degree Gauss-Kruger CM 57E (code 16310).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',57.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',10500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12195','conversion','EPSG','16210','EPSG','1951','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16211','6-degree Gauss-Kruger zone 11','Also found with zone truncated from false easting: see 6-degree Gauss-Kruger CM 63E (code 16311).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',63.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',11500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12196','conversion','EPSG','16211','EPSG','1953','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16212','6-degree Gauss-Kruger zone 12','Also found with zone truncated from false easting: see 6-degree Gauss-Kruger CM 69E (code 16312).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',69.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',12500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12197','conversion','EPSG','16212','EPSG','1955','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16213','6-degree Gauss-Kruger zone 13','Also found with zone truncated from false easting: see 6-degree Gauss-Kruger CM 75E (code 16313).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',75.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',13500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12198','conversion','EPSG','16213','EPSG','1957','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16214','6-degree Gauss-Kruger zone 14','Also found with zone truncated from false easting: see 6-degree Gauss-Kruger CM 81E (code 16314).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',81.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',14500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12199','conversion','EPSG','16214','EPSG','1959','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16215','6-degree Gauss-Kruger zone 15','Also found with zone truncated from false easting: see 6-degree Gauss-Kruger CM 87E (code 16315).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',87.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',15500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12200','conversion','EPSG','16215','EPSG','1961','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16216','6-degree Gauss-Kruger zone 16','Also found with zone truncated from false easting: see 6-degree Gauss-Kruger CM 93E (code 16316).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',93.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',16500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12201','conversion','EPSG','16216','EPSG','1963','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16217','6-degree Gauss-Kruger zone 17','Also found with zone truncated from false easting: see 6-degree Gauss-Kruger CM 99E (code 16317).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',99.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',17500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12202','conversion','EPSG','16217','EPSG','1965','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16218','6-degree Gauss-Kruger zone 18','Also found with zone truncated from false easting: see 6-degree Gauss-Kruger CM 105E (code 16318).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',105.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',18500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12203','conversion','EPSG','16218','EPSG','1967','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16219','6-degree Gauss-Kruger zone 19','Also found with zone truncated from false easting: see 6-degree Gauss-Kruger CM 111E (code 16319).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',111.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',19500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12204','conversion','EPSG','16219','EPSG','1969','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16220','6-degree Gauss-Kruger zone 20','Also found with zone truncated from false easting: see 6-degree Gauss-Kruger CM 117E (code 16320).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',117.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',20500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12205','conversion','EPSG','16220','EPSG','1971','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16221','6-degree Gauss-Kruger zone 21','Also found with zone truncated from false easting: see 6-degree Gauss-Kruger CM 123E (code 16321).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',123.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',21500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12206','conversion','EPSG','16221','EPSG','1973','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16222','6-degree Gauss-Kruger zone 22','Also found with zone truncated from false easting: see 6-degree Gauss-Kruger CM 129E (code 16322).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',129.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',22500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12207','conversion','EPSG','16222','EPSG','1975','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16223','6-degree Gauss-Kruger zone 23','Also found with zone truncated from false easting: see 6-degree Gauss-Kruger CM 135E (code 16323).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',135.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',23500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12208','conversion','EPSG','16223','EPSG','1977','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16224','6-degree Gauss-Kruger zone 24','Also found with zone truncated from false easting: see 6-degree Gauss-Kruger CM 141E (code 16324).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',141.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',24500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12209','conversion','EPSG','16224','EPSG','1979','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16225','6-degree Gauss-Kruger zone 25','Also found with zone truncated from false easting: see 6-degree Gauss-Kruger CM 147E (code 16325).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',147.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',25500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12210','conversion','EPSG','16225','EPSG','1981','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16226','6-degree Gauss-Kruger zone 26','Also found with zone truncated from false easting: see 6-degree Gauss-Kruger CM 153E (code 16326).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',153.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',26500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12211','conversion','EPSG','16226','EPSG','1983','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16227','6-degree Gauss-Kruger zone 27','Also found with zone truncated from false easting: see 6-degree Gauss-Kruger CM 159E (code 16327).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',159.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',27500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12212','conversion','EPSG','16227','EPSG','1985','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16228','6-degree Gauss-Kruger zone 28','Also found with zone truncated from false easting: see 6-degree Gauss-Kruger CM 165E (code 16328).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',165.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',28500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12213','conversion','EPSG','16228','EPSG','1987','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16229','6-degree Gauss-Kruger zone 29','Also found with zone truncated from false easting: see 6-degree Gauss-Kruger CM 171E (code 16329).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',171.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',29500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12214','conversion','EPSG','16229','EPSG','1989','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16230','6-degree Gauss-Kruger zone 30','Also found with zone truncated from false easting: see 6-degree Gauss-Kruger CM 177E (code 16330).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',177.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',30500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12215','conversion','EPSG','16230','EPSG','1991','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16231','6-degree Gauss-Kruger zone 31','Also found with zone truncated from false easting: see 6-degree Gauss-Kruger CM 177W (code 16331).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-177.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',31500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12216','conversion','EPSG','16231','EPSG','1873','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16232','6-degree Gauss-Kruger zone 32','Also found with zone truncated from false easting: see 6-degree Gauss-Kruger CM 171W (code 16332).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-171.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',32500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12217','conversion','EPSG','16232','EPSG','1875','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16233','6-degree Gauss-Kruger zone 33','Also found with zone truncated from false easting: see 6-degree Gauss-Kruger cm 165W (code 16333). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-165.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',33500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12218','conversion','EPSG','16233','EPSG','1877','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16234','6-degree Gauss-Kruger zone 34','Also found with zone truncated from false easting: see 6-degree Gauss-Kruger cm 159W (code 16334). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-159.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',34500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12219','conversion','EPSG','16234','EPSG','1879','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16235','6-degree Gauss-Kruger zone 35','Also found with zone truncated from false easting: see 6-degree Gauss-Kruger cm 153W (code 16335). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-153.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',35500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12220','conversion','EPSG','16235','EPSG','1881','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16236','6-degree Gauss-Kruger zone 36','Also found with zone truncated from false easting: see 6-degree Gauss-Kruger cm 147W (code 16336). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-147.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',36500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12221','conversion','EPSG','16236','EPSG','1883','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16237','6-degree Gauss-Kruger zone 37','Also found with zone truncated from false easting: see 6-degree Gauss-Kruger cm 141W (code 16337). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-141.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',37500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12222','conversion','EPSG','16237','EPSG','1885','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16238','6-degree Gauss-Kruger zone 38','Also found with zone truncated from false easting: see 6-degree Gauss-Kruger cm 135W (code 16338). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-135.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',38500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12223','conversion','EPSG','16238','EPSG','1887','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16239','6-degree Gauss-Kruger zone 39','Also found with zone truncated from false easting: see 6-degree Gauss-Kruger cm 129W (code 16339). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-129.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',39500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12224','conversion','EPSG','16239','EPSG','1889','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16240','6-degree Gauss-Kruger zone 40','Also found with zone truncated from false easting: see 6-degree Gauss-Kruger cm 123W (code 16340). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-123.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',40500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12225','conversion','EPSG','16240','EPSG','1891','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16241','6-degree Gauss-Kruger zone 41','Also found with zone truncated from false easting: see 6-degree Gauss-Kruger cm 117W (code 16341). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-117.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',41500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12226','conversion','EPSG','16241','EPSG','1893','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16242','6-degree Gauss-Kruger zone 42','Also found with zone truncated from false easting: see 6-degree Gauss-Kruger cm 111W (code 16342). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-111.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',42500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12227','conversion','EPSG','16242','EPSG','1895','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16243','6-degree Gauss-Kruger zone 43','Also found with zone truncated from false easting: see 6-degree Gauss-Kruger cm 105W (code 16343). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-105.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',43500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12228','conversion','EPSG','16243','EPSG','1897','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16244','6-degree Gauss-Kruger zone 44','Also found with zone truncated from false easting: see 6-degree Gauss-Kruger cm 99W (code 16344). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-99.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',44500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12229','conversion','EPSG','16244','EPSG','1899','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16245','6-degree Gauss-Kruger zone 45','Also found with zone truncated from false easting: see 6-degree Gauss-Kruger cm 93W (code 16345). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-93.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',45500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12230','conversion','EPSG','16245','EPSG','1901','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16246','6-degree Gauss-Kruger zone 46','Also found with zone truncated from false easting: see 6-degree Gauss-Kruger cm 87W (code 16346). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-87.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',46500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12231','conversion','EPSG','16246','EPSG','1903','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16247','6-degree Gauss-Kruger zone 47','Also found with zone truncated from false easting: see 6-degree Gauss-Kruger cm 81W (code 16347). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-81.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',47500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12232','conversion','EPSG','16247','EPSG','2732','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16248','6-degree Gauss-Kruger zone 48','Also found with zone truncated from false easting: see 6-degree Gauss-Kruger cm 75W (code 16348). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-75.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',48500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12233','conversion','EPSG','16248','EPSG','2733','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16249','6-degree Gauss-Kruger zone 49','Also found with zone truncated from false easting: see 6-degree Gauss-Kruger cm 69W (code 16349). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-69.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',49500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12234','conversion','EPSG','16249','EPSG','2734','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16250','6-degree Gauss-Kruger zone 50','Also found with zone truncated from false easting: see 6-degree Gauss-Kruger cm 63W (code 16350). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-63.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',50500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12235','conversion','EPSG','16250','EPSG','2735','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16251','6-degree Gauss-Kruger zone 51','Also found with zone truncated from false easting: see 6-degree Gauss-Kruger cm 57W (code 16351). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-57.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',51500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12236','conversion','EPSG','16251','EPSG','2736','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16252','6-degree Gauss-Kruger zone 52','Also found with zone truncated from false easting: see 6-degree Gauss-Kruger cm 51W (code 16352). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-51.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',52500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12237','conversion','EPSG','16252','EPSG','2737','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16253','6-degree Gauss-Kruger zone 53','Also found with zone truncated from false easting: see 6-degree Gauss-Kruger cm 45W (code 16353). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-45.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',53500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12238','conversion','EPSG','16253','EPSG','2738','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16254','6-degree Gauss-Kruger zone 54','Also found with zone truncated from false easting: see 6-degree Gauss-Kruger cm 39W (code 16354). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-39.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',54500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12239','conversion','EPSG','16254','EPSG','2739','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16255','6-degree Gauss-Kruger zone 55','Also found with zone truncated from false easting: see 6-degree Gauss-Kruger cm 33W (code 16355). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-33.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',55500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12240','conversion','EPSG','16255','EPSG','2740','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16256','6-degree Gauss-Kruger zone 56','Also found with zone truncated from false easting: see 6-degree Gauss-Kruger cm 27W (code 16356). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-27.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',56500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12241','conversion','EPSG','16256','EPSG','1923','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16257','6-degree Gauss-Kruger zone 57','Also found with zone truncated from false easting: see 6-degree Gauss-Kruger cm 21W (code 16357). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-21.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',57500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12242','conversion','EPSG','16257','EPSG','1925','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16258','6-degree Gauss-Kruger zone 58','Also found with zone truncated from false easting: see 6-degree Gauss-Kruger cm 15W (code 16358). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-15.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',58500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12243','conversion','EPSG','16258','EPSG','1927','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16259','6-degree Gauss-Kruger zone 59','Also found with zone truncated from false easting: see 6-degree Gauss-Kruger cm 9W (code 16359). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-9.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',59500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12244','conversion','EPSG','16259','EPSG','1929','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16260','6-degree Gauss-Kruger zone 60','Also found with zone truncated from false easting: see 6-degree Gauss-Kruger cm 3W (code 16360). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-3.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',60500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12245','conversion','EPSG','16260','EPSG','1931','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16261','3-degree Gauss-Kruger zone 1','Also found with zone truncated from false easting: see Gauss-Kruger CM 3E (code 16301).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',3.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',1500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12246','conversion','EPSG','16261','EPSG','2299','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16262','3-degree Gauss-Kruger zone 2','Also found with zone truncated from false easting: see 3-degree Gauss-Kruger CM 6E (code 16362).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',6.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',2500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12247','conversion','EPSG','16262','EPSG','2300','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16263','3-degree Gauss-Kruger zone 3','Also found with zone truncated from false easting: see Gauss-Kruger CM 9E (code 16302).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',9.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',3500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12248','conversion','EPSG','16263','EPSG','2301','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16264','3-degree Gauss-Kruger zone 4','Also found with zone truncated from false easting: see 3-degree Gauss-Kruger CM 12E (code 16364).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',12.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',4500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12249','conversion','EPSG','16264','EPSG','2302','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16265','3-degree Gauss-Kruger zone 5','Also found with zone truncated from false easting: see Gauss-Kruger CM 15E (code 16303).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',15.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',5500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12250','conversion','EPSG','16265','EPSG','2303','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16266','3-degree Gauss-Kruger zone 6','Also found with zone truncated from false easting: see 3-degree Gauss-Kruger CM 18E (code 16366).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',18.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',6500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12251','conversion','EPSG','16266','EPSG','2304','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16267','3-degree Gauss-Kruger zone 7','Also found with zone truncated from false easting: see Gauss-Kruger CM 21E (code 16304).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',21.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',7500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12252','conversion','EPSG','16267','EPSG','2305','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16268','3-degree Gauss-Kruger zone 8','Also found with zone truncated from false easting: see 3-degree Gauss-Kruger CM 24E (code 16368).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',24.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',8500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12253','conversion','EPSG','16268','EPSG','2306','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16269','3-degree Gauss-Kruger zone 9','Also found with zone truncated from false easting: see Gauss-Kruger CM 27E (code 16305).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',27.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',9500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12254','conversion','EPSG','16269','EPSG','2534','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16270','3-degree Gauss-Kruger zone 10','Also found with zone truncated from false easting: see 3-degree Gauss-Kruger CM 30E (code 16370).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',30.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',10500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12255','conversion','EPSG','16270','EPSG','2535','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16271','3-degree Gauss-Kruger zone 11','Also found with zone truncated from false easting: see Gauss-Kruger CM 33E (code 16306).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',33.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',11500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12256','conversion','EPSG','16271','EPSG','2536','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16272','3-degree Gauss-Kruger zone 12','Also found with zone truncated from false easting: see 3-degree Gauss-Kruger CM 36E (code 16372).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',36.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',12500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12257','conversion','EPSG','16272','EPSG','2537','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16273','3-degree Gauss-Kruger zone 13','Also found with zone truncated from false easting: see Gauss-Kruger CM 39E (code 16307).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',39.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',13500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12258','conversion','EPSG','16273','EPSG','2538','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16274','3-degree Gauss-Kruger zone 14','Also found with zone truncated from false easting: see 3-degree Gauss-Kruger CM 42E (code 16374).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',42.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',14500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12259','conversion','EPSG','16274','EPSG','2539','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16275','3-degree Gauss-Kruger zone 15','Also found with zone truncated from false easting: see Gauss-Kruger CM 45E (code 16308).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',45.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',15500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12260','conversion','EPSG','16275','EPSG','2540','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16276','3-degree Gauss-Kruger zone 16','Also found with zone truncated from false easting: see 3-degree Gauss-Kruger cm 48E (code 16376). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',48.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',16500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12261','conversion','EPSG','16276','EPSG','2604','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16277','3-degree Gauss-Kruger zone 17','Also found with zone truncated from false easting: see Gauss-Kruger cm 51E (code 16309). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',51.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',17500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12262','conversion','EPSG','16277','EPSG','2605','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16278','3-degree Gauss-Kruger zone 18','Also found with zone truncated from false easting: see 3-degree Gauss-Kruger cm 54E (code 16378). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',54.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',18500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12263','conversion','EPSG','16278','EPSG','2606','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16279','3-degree Gauss-Kruger zone 19','Also found with zone truncated from false easting: see Gauss-Kruger cm 57E (code 16310). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',57.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',19500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12264','conversion','EPSG','16279','EPSG','2607','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16280','3-degree Gauss-Kruger zone 20','Also found with zone truncated from false easting: see 3-degree Gauss-Kruger cm 60E (code 16380). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',60.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',20500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12265','conversion','EPSG','16280','EPSG','2608','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16281','3-degree Gauss-Kruger zone 21','Also found with zone truncated from false easting: see Gauss-Kruger cm 63E (code 16311). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',63.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',21500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12266','conversion','EPSG','16281','EPSG','2609','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16282','3-degree Gauss-Kruger zone 22','Also found with zone truncated from false easting: see 3-degree Gauss-Kruger cm 66E (code 16382). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',66.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',22500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12267','conversion','EPSG','16282','EPSG','2610','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16283','3-degree Gauss-Kruger zone 23','Also found with zone truncated from false easting: see Gauss-Kruger cm 69E (code 16312). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',69.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',23500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12268','conversion','EPSG','16283','EPSG','2611','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16284','3-degree Gauss-Kruger zone 24','Also found with zone truncated from false easting: see 3-degree Gauss-Kruger cm 72E (code 16384). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',72.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',24500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12269','conversion','EPSG','16284','EPSG','2612','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16285','3-degree Gauss-Kruger zone 25','Also found with zone truncated from false easting: see Gauss-Kruger cm 75E (code 16313). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',75.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',25500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12270','conversion','EPSG','16285','EPSG','2613','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16286','3-degree Gauss-Kruger zone 26','Also found with zone truncated from false easting: see 3-degree Gauss-Kruger cm 78E (code 16386). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',78.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',26500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12271','conversion','EPSG','16286','EPSG','2614','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16287','3-degree Gauss-Kruger zone 27','Also found with zone truncated from false easting: see Gauss-Kruger cm 81E (code 16314). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',81.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',27500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12272','conversion','EPSG','16287','EPSG','2615','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16288','3-degree Gauss-Kruger zone 28','Also found with zone truncated from false easting: see 3-degree Gauss-Kruger cm 84E (code 16388). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',84.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',28500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12273','conversion','EPSG','16288','EPSG','2616','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16289','3-degree Gauss-Kruger zone 29','Also found with zone truncated from false easting: see Gauss-Kruger cm 87E (code 16315). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',87.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',29500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12274','conversion','EPSG','16289','EPSG','2617','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16290','3-degree Gauss-Kruger zone 30','Also found with zone truncated from false easting: see 3-degree Gauss-Kruger cm 90E (code 16390). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',90.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',30500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12275','conversion','EPSG','16290','EPSG','2618','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16291','3-degree Gauss-Kruger zone 31','Also found with zone truncated from false easting: see Gauss-Kruger cm 93E (code 16316). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',93.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',31500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12276','conversion','EPSG','16291','EPSG','2619','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16292','3-degree Gauss-Kruger zone 32','Also found with zone truncated from false easting: see 3-degree Gauss-Kruger cm 96E (code 16392). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',96.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',32500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12277','conversion','EPSG','16292','EPSG','2620','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16293','3-degree Gauss-Kruger zone 33','Also found with zone truncated from false easting: see Gauss-Kruger cm 99E (code 16317). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',99.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',33500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12278','conversion','EPSG','16293','EPSG','2621','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16294','3-degree Gauss-Kruger zone 34','Also found with zone truncated from false easting: see 3-degree Gauss-Kruger cm 102E (code 16394). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',102.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',34500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12279','conversion','EPSG','16294','EPSG','2622','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16295','3-degree Gauss-Kruger zone 35','Also found with zone truncated from false easting: see Gauss-Kruger cm 105E (code 16318). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',105.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',35500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12280','conversion','EPSG','16295','EPSG','2623','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16296','3-degree Gauss-Kruger zone 36','Also found with zone truncated from false easting: see 3-degree Gauss-Kruger cm 108E (code 16396). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',108.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',36500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12281','conversion','EPSG','16296','EPSG','2624','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16297','3-degree Gauss-Kruger zone 37','Also found with zone truncated from false easting: see Gauss-Kruger cm 111E (code 16319). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',111.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',37500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12282','conversion','EPSG','16297','EPSG','2625','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16298','3-degree Gauss-Kruger zone 38','Also found with zone truncated from false easting: see 3-degree Gauss-Kruger cm 114E (code 16398). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',114.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',38500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12283','conversion','EPSG','16298','EPSG','2626','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16299','3-degree Gauss-Kruger zone 39','Also found with zone truncated from false easting: see 3-degree Gauss-Kruger cm 117E (code 16320). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',117.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',39500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12284','conversion','EPSG','16299','EPSG','2627','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16301','Gauss-Kruger CM 3E','Usually used with zone prefix to FE - see 6-degree Gauss-Kruger zone 1N (code 16201), or for a restricted longitude range 3-degree Gauss-Kruger zone 1N (code 16261). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',3.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12285','conversion','EPSG','16301','EPSG','1933','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16302','Gauss-Kruger CM 9E','Usually used with zone prefix to FE - see 6-degree Gauss-Kruger zone 2N (code 16202).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',9.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12286','conversion','EPSG','16302','EPSG','1935','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16303','Gauss-Kruger CM 15E','Usually used with zone prefix to FE - see 6-degree Gauss-Kruger zone 3N (code 16203).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',15.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12287','conversion','EPSG','16303','EPSG','1937','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16304','Gauss-Kruger CM 21E','Usually used with zone prefix to FE - see 6-degree Gauss-Kruger zone 4N (code 16204).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',21.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12288','conversion','EPSG','16304','EPSG','1939','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16305','Gauss-Kruger CM 27E','Usually used with zone prefix to FE - see 6-degree Gauss-Kruger zone 5N (code 16205).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',27.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12289','conversion','EPSG','16305','EPSG','1941','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16306','Gauss-Kruger CM 33E','Usually used with zone prefix to FE - see 6-degree Gauss-Kruger zone 6N (code 16206).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',33.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12290','conversion','EPSG','16306','EPSG','1943','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16307','Gauss-Kruger CM 39E','Usually used with zone prefix to FE - see 6-degree Gauss-Kruger zone 7N (code 16207).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',39.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12291','conversion','EPSG','16307','EPSG','1945','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16308','Gauss-Kruger CM 45E','Usually used with zone prefix to FE - see 6-degree Gauss-Kruger zone 8N (code 16208).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',45.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12292','conversion','EPSG','16308','EPSG','1947','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16309','Gauss-Kruger CM 51E','Usually used with zone prefix to FE - see 6-degree Gauss-Kruger zone 9N (code 16209).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',51.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12293','conversion','EPSG','16309','EPSG','1949','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16310','Gauss-Kruger CM 57E','Usually used with zone prefix to FE - see 6-degree Gauss-Kruger zone 10N (code 16210).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',57.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12294','conversion','EPSG','16310','EPSG','1951','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16311','Gauss-Kruger CM 63E','Usually used with zone prefix to FE - see 6-degree Gauss-Kruger zone 11N (code 16211).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',63.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12295','conversion','EPSG','16311','EPSG','1953','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16312','Gauss-Kruger CM 69E','Usually used with zone prefix to FE - see 6-degree Gauss-Kruger zone 12N (code 16212).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',69.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12296','conversion','EPSG','16312','EPSG','1955','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16313','Gauss-Kruger CM 75E','Usually used with zone prefix to FE - see 6-degree Gauss-Kruger zone 13N (code 16213).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',75.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12297','conversion','EPSG','16313','EPSG','1957','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16314','Gauss-Kruger CM 81E','Usually used with zone prefix to FE - see 6-degree Gauss-Kruger zone 14N (code 16214).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',81.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12298','conversion','EPSG','16314','EPSG','1959','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16315','Gauss-Kruger CM 87E','Usually used with zone prefix to FE - see 6-degree Gauss-Kruger zone 15N (code 16215).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',87.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12299','conversion','EPSG','16315','EPSG','1961','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16316','Gauss-Kruger CM 93E','Usually used with zone prefix to FE - see 6-degree Gauss-Kruger zone 16N (code 16216).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',93.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12300','conversion','EPSG','16316','EPSG','1963','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16317','Gauss-Kruger CM 99E','Usually used with zone prefix to FE - see 6-degree Gauss-Kruger zone 17N (code 16217).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',99.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12301','conversion','EPSG','16317','EPSG','1965','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16318','Gauss-Kruger CM 105E','Usually used with zone prefix to FE - see 6-degree Gauss-Kruger zone 18N (code 16218).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',105.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12302','conversion','EPSG','16318','EPSG','1967','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16319','Gauss-Kruger CM 111E','Usually used with zone prefix to FE - see 6-degree Gauss-Kruger zone 19N (code 16219).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',111.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12303','conversion','EPSG','16319','EPSG','1969','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16320','Gauss-Kruger CM 117E','Usually used with zone prefix to FE - see 6-degree Gauss-Kruger zone 20N (code 16220).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',117.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12304','conversion','EPSG','16320','EPSG','1971','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16321','Gauss-Kruger CM 123E','Usually used with zone prefix to FE - see 6-degree Gauss-Kruger zone 21N (code 16221).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',123.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12305','conversion','EPSG','16321','EPSG','1973','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16322','Gauss-Kruger CM 129E','Usually used with zone prefix to FE - see 6-degree Gauss-Kruger zone 22N (code 16222).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',129.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12306','conversion','EPSG','16322','EPSG','1975','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16323','Gauss-Kruger CM 135E','Usually used with zone prefix to FE - see 6-degree Gauss-Kruger zone 23N (code 16223).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',135.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12307','conversion','EPSG','16323','EPSG','1977','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16324','Gauss-Kruger CM 141E','Usually used with zone prefix to FE - see 6-degree Gauss-Kruger zone 24N (code 16224).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',141.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12308','conversion','EPSG','16324','EPSG','1979','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16325','Gauss-Kruger CM 147E','Usually used with zone prefix to FE - see 6-degree Gauss-Kruger zone 25N (code 16225).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',147.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12309','conversion','EPSG','16325','EPSG','1981','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16326','Gauss-Kruger CM 153E','Usually used with zone prefix to FE - see 6-degree Gauss-Kruger zone 26N (code 16226).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',153.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12310','conversion','EPSG','16326','EPSG','1983','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16327','Gauss-Kruger CM 159E','Usually used with zone prefix to FE - see 6-degree Gauss-Kruger zone 27N (code 16227).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',159.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12311','conversion','EPSG','16327','EPSG','1985','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16328','Gauss-Kruger CM 165E','Usually used with zone prefix to FE - see 6-degree Gauss-Kruger zone 28N (code 16228).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',165.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12312','conversion','EPSG','16328','EPSG','1987','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16329','Gauss-Kruger CM 171E','Usually used with zone prefix to FE - see 6-degree Gauss-Kruger zone 29N (code 16229).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',171.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12313','conversion','EPSG','16329','EPSG','1989','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16330','Gauss-Kruger CM 177E','Usually used with zone prefix to FE - see 6-degree Gauss-Kruger zone 30N (code 16230).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',177.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12314','conversion','EPSG','16330','EPSG','1991','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16331','Gauss-Kruger CM 177W','Usually used with zone prefix to FE - see 6-degree Gauss-Kruger zone 31N (code 16231).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-177.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12315','conversion','EPSG','16331','EPSG','1873','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16332','Gauss-Kruger CM 171W','Usually used with zone prefix to FE - see 6-degree Gauss-Kruger zone 32N (code 16232).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-171.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12316','conversion','EPSG','16332','EPSG','1875','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16333','Gauss-Kruger CM 165W','Usually used with zone prefix to FE - see 6-degree Gauss-Kruger zone 33N (code 16233). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-165.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12317','conversion','EPSG','16333','EPSG','1877','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16334','Gauss-Kruger CM 159W','Usually used with zone prefix to FE - see 6-degree Gauss-Kruger zone 34N (code 16234). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-159.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12318','conversion','EPSG','16334','EPSG','1879','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16335','Gauss-Kruger CM 153W','Usually used with zone prefix to FE - see 6-degree Gauss-Kruger zone 35N (code 16235). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-153.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12319','conversion','EPSG','16335','EPSG','1881','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16336','Gauss-Kruger CM 147W','Usually used with zone prefix to FE - see 6-degree Gauss-Kruger zone 36N (code 16236). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-147.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12320','conversion','EPSG','16336','EPSG','1883','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16337','Gauss-Kruger CM 141W','Usually used with zone prefix to FE - see 6-degree Gauss-Kruger zone 37N (code 16237). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-141.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12321','conversion','EPSG','16337','EPSG','1885','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16338','Gauss-Kruger CM 135W','Usually used with zone prefix to FE - see 6-degree Gauss-Kruger zone 38N (code 16238). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-135.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12322','conversion','EPSG','16338','EPSG','1887','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16339','Gauss-Kruger CM 129W','Usually used with zone prefix to FE - see 6-degree Gauss-Kruger zone 39N (code 16239). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-129.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12323','conversion','EPSG','16339','EPSG','1889','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16340','Gauss-Kruger CM 123W','Usually used with zone prefix to FE - see 6-degree Gauss-Kruger zone 40N (code 16240). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-123.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12324','conversion','EPSG','16340','EPSG','1891','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16341','Gauss-Kruger CM 117W','Usually used with zone prefix to FE - see 6-degree Gauss-Kruger zone 41N (code 16241). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-117.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12325','conversion','EPSG','16341','EPSG','1893','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16342','Gauss-Kruger CM 111W','Usually used with zone prefix to FE - see 6-degree Gauss-Kruger zone 42N (code 16242). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-111.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12326','conversion','EPSG','16342','EPSG','1895','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16343','Gauss-Kruger CM 105W','Usually used with zone prefix to FE - see 6-degree Gauss-Kruger zone 43N (code 16243). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-105.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12327','conversion','EPSG','16343','EPSG','1897','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16344','Gauss-Kruger CM 99W','Usually used with zone prefix to FE - see 6-degree Gauss-Kruger zone 44N (code 16244). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-99.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12328','conversion','EPSG','16344','EPSG','1899','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16345','Gauss-Kruger CM 93W','Usually used with zone prefix to FE - see 6-degree Gauss-Kruger zone 45N (code 16245). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-93.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12329','conversion','EPSG','16345','EPSG','1901','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16346','Gauss-Kruger CM 87W','Usually used with zone prefix to FE - see 6-degree Gauss-Kruger zone 46N (code 16246). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-87.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12330','conversion','EPSG','16346','EPSG','1903','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16347','Gauss-Kruger CM 81W','Usually used with zone prefix to FE - see 6-degree Gauss-Kruger zone 47N (code 16247). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-81.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12331','conversion','EPSG','16347','EPSG','1905','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16348','Gauss-Kruger CM 75W','Usually used with zone prefix to FE - see 6-degree Gauss-Kruger zone 48N (code 16248). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-75.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12332','conversion','EPSG','16348','EPSG','1907','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16349','Gauss-Kruger CM 69W','Usually used with zone prefix to FE - see 6-degree Gauss-Kruger zone 49N (code 16249). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-69.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12333','conversion','EPSG','16349','EPSG','1909','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16350','Gauss-Kruger CM 63W','Usually used with zone prefix to FE - see 6-degree Gauss-Kruger zone 50N (code 16250). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-63.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12334','conversion','EPSG','16350','EPSG','1911','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16351','Gauss-Kruger CM 57W','Usually used with zone prefix to FE - see 6-degree Gauss-Kruger zone 51N (code 16251). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-57.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12335','conversion','EPSG','16351','EPSG','1913','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16352','Gauss-Kruger CM 51W','Usually used with zone prefix to FE - see 6-degree Gauss-Kruger zone 52N (code 16252). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-51.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12336','conversion','EPSG','16352','EPSG','1915','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16353','Gauss-Kruger CM 45W','Usually used with zone prefix to FE - see 6-degree Gauss-Kruger zone 53N (code 16253). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-45.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12337','conversion','EPSG','16353','EPSG','1917','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16354','Gauss-Kruger CM 39W','Usually used with zone prefix to FE - see 6-degree Gauss-Kruger zone 54N (code 16254). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-39.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12338','conversion','EPSG','16354','EPSG','1919','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16355','Gauss-Kruger CM 33W','Usually used with zone prefix to FE - see 6-degree Gauss-Kruger zone 55N (code 16255). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-33.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12339','conversion','EPSG','16355','EPSG','1921','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16356','Gauss-Kruger CM 27W','Usually used with zone prefix to FE - see 6-degree Gauss-Kruger zone 56N (code 16256). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-27.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12340','conversion','EPSG','16356','EPSG','1923','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16357','Gauss-Kruger CM 21W','Usually used with zone prefix to FE - see 6-degree Gauss-Kruger zone 57N (code 16257). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-21.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12341','conversion','EPSG','16357','EPSG','1925','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16358','Gauss-Kruger CM 15W','Usually used with zone prefix to FE - see 6-degree Gauss-Kruger zone 58N (code 16258). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-15.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12342','conversion','EPSG','16358','EPSG','1927','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16359','Gauss-Kruger CM 9W','Usually used with zone prefix to FE - see 6-degree Gauss-Kruger zone 59N (code 16259). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-9.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12343','conversion','EPSG','16359','EPSG','1929','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16360','Gauss-Kruger CM 3W','Usually used with zone prefix to FE - see 6-degree Gauss-Kruger zone 60N (code 16260). Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-3.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12344','conversion','EPSG','16360','EPSG','1931','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','16361','3-degree Gauss-Kruger CM 3E','3-degree Gauss-Kruger zone 1N (code 16261) without zone prefix to false easting value. Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',3.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','12345','conversion','EPSG','16361','EPSG','2299','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16362','3-degree Gauss-Kruger CM 6E','3-degree Gauss-Kruger zone 2N (code 16262) without zone prefix to false easting value. Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',6.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12346','conversion','EPSG','16362','EPSG','2300','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16363','3-degree Gauss-Kruger CM 9E','3-degree Gauss-Kruger zone 3N (code 16263) without zone prefix to false easting value. Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',9.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','12347','conversion','EPSG','16363','EPSG','2301','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16364','3-degree Gauss-Kruger CM 12E','3-degree Gauss-Kruger zone 4N (code 16264) without zone prefix to false easting value. Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',12.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12348','conversion','EPSG','16364','EPSG','2302','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16365','3-degree Gauss-Kruger CM 15E','3-degree Gauss-Kruger zone 5N (code 16265) without zone prefix to false easting value. Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',15.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','12349','conversion','EPSG','16365','EPSG','2303','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16366','3-degree Gauss-Kruger CM 18E','3-degree Gauss-Kruger zone 6N (code 16266) without zone prefix to false easting value.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',18.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12350','conversion','EPSG','16366','EPSG','2304','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16367','3-degree Gauss-Kruger CM 21E','3-degree Gauss-Kruger zone 7N (code 16267) without zone prefix to false easting value. Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',21.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','12351','conversion','EPSG','16367','EPSG','2305','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16368','3-degree Gauss-Kruger CM 24E','3-degree Gauss-Kruger zone 8N (code 16268) without zone prefix to false easting value.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',24.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12352','conversion','EPSG','16368','EPSG','2306','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16369','3-degree Gauss-Kruger CM 27E','3-degree Gauss-Kruger zone 9N (code 16269) without zone prefix to false easting value. Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',27.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','12353','conversion','EPSG','16369','EPSG','2534','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16370','3-degree Gauss-Kruger CM 30E','3-degree Gauss-Kruger zone 10N (code 16270) without zone prefix to false easting value.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',30.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12354','conversion','EPSG','16370','EPSG','2535','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16371','3-degree Gauss-Kruger CM 33E','3-degree Gauss-Kruger zone 11N (code 16271) without zone prefix to false easting value. Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',33.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','12355','conversion','EPSG','16371','EPSG','2536','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16372','3-degree Gauss-Kruger CM 36E','3-degree Gauss-Kruger zone 12N (code 16272) without zone prefix to false easting value.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',36.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12356','conversion','EPSG','16372','EPSG','2537','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16373','3-degree Gauss-Kruger CM 39E','3-degree Gauss-Kruger zone 13N (code 16273) without zone prefix to false easting value. Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',39.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','12357','conversion','EPSG','16373','EPSG','2538','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16374','3-degree Gauss-Kruger CM 42E','3-degree Gauss-Kruger zone 14N (code 16274) without zone prefix to false easting value.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',42.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12358','conversion','EPSG','16374','EPSG','2539','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16375','3-degree Gauss-Kruger CM 45E','3-degree Gauss-Kruger zone 15N (code 16275) without zone prefix to false easting value. Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',45.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','12359','conversion','EPSG','16375','EPSG','2540','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16376','3-degree Gauss-Kruger CM 48E','3-degree Gauss-Kruger zone 16N (code 16276) without zone prefix to false easting value. Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',48.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12360','conversion','EPSG','16376','EPSG','2604','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16377','3-degree Gauss-Kruger CM 51E','3-degree Gauss-Kruger zone 17N (code 16277) without zone prefix to false easting value. Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',51.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','12361','conversion','EPSG','16377','EPSG','2605','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16378','3-degree Gauss-Kruger CM 54E','3-degree Gauss-Kruger zone 18N (code 16278) without zone prefix to false easting value. Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',54.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12362','conversion','EPSG','16378','EPSG','2606','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16379','3-degree Gauss-Kruger CM 57E','3-degree Gauss-Kruger zone 19N (code 16279) without zone prefix to false easting value. Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',57.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','12363','conversion','EPSG','16379','EPSG','2607','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16380','3-degree Gauss-Kruger CM 60E','3-degree Gauss-Kruger zone 20N (code 16280) without zone prefix to false easting value. Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',60.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12364','conversion','EPSG','16380','EPSG','2608','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16381','3-degree Gauss-Kruger CM 63E','3-degree Gauss-Kruger zone 21N (code 16281) without zone prefix to false easting value. Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',63.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','12365','conversion','EPSG','16381','EPSG','2609','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16382','3-degree Gauss-Kruger CM 66E','3-degree Gauss-Kruger zone 22N (code 16282) without zone prefix to false easting value. Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',66.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12366','conversion','EPSG','16382','EPSG','2610','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16383','3-degree Gauss-Kruger CM 69E','3-degree Gauss-Kruger zone 23N (code 16283) without zone prefix to false easting value. Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',69.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','12367','conversion','EPSG','16383','EPSG','2611','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16384','3-degree Gauss-Kruger CM 72E','3-degree Gauss-Kruger zone 24N (code 16284) without zone prefix to false easting value. Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',72.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12368','conversion','EPSG','16384','EPSG','2612','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16385','3-degree Gauss-Kruger CM 75E','3-degree Gauss-Kruger zone 25N (code 16285) without zone prefix to false easting value. Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',75.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','12369','conversion','EPSG','16385','EPSG','2613','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16386','3-degree Gauss-Kruger CM 78E','3-degree Gauss-Kruger zone 26N (code 16286) without zone prefix to false easting value. Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',78.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12370','conversion','EPSG','16386','EPSG','2614','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16387','3-degree Gauss-Kruger CM 81E','3-degree Gauss-Kruger zone 27N (code 16287) without zone prefix to false easting value. Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',81.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','12371','conversion','EPSG','16387','EPSG','2615','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16388','3-degree Gauss-Kruger CM 84E','3-degree Gauss-Kruger zone 28N (code 16288) without zone prefix to false easting value. Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',84.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12372','conversion','EPSG','16388','EPSG','2616','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16389','3-degree Gauss-Kruger CM 87E','3-degree Gauss-Kruger zone 29N (code 16289) without zone prefix to false easting value. Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',87.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','12373','conversion','EPSG','16389','EPSG','2617','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16390','3-degree Gauss-Kruger CM 90E','3-degree Gauss-Kruger zone 30N (code 16290) without zone prefix to false easting value. Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',90.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12374','conversion','EPSG','16390','EPSG','2618','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16391','3-degree Gauss-Kruger CM 93E','3-degree Gauss-Kruger zone 31N (code 16291) without zone prefix to false easting value. Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',93.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','12375','conversion','EPSG','16391','EPSG','2619','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16392','3-degree Gauss-Kruger CM 96E','3-degree Gauss-Kruger zone 32N (code 16292) without zone prefix to false easting value. Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',96.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12376','conversion','EPSG','16392','EPSG','2620','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16393','3-degree Gauss-Kruger CM 99E','3-degree Gauss-Kruger zone 33N (code 16293) without zone prefix to false easting value. Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',99.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','12377','conversion','EPSG','16393','EPSG','2621','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16394','3-degree Gauss-Kruger CM 102E','3-degree Gauss-Kruger zone 34N (code 16294) without zone prefix to false easting value. Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',102.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12378','conversion','EPSG','16394','EPSG','2622','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16395','3-degree Gauss-Kruger CM 105E','3-degree Gauss-Kruger zone 35N (code 16295) without zone prefix to false easting value. Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',105.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','12379','conversion','EPSG','16395','EPSG','2623','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16396','3-degree Gauss-Kruger CM 108E','3-degree Gauss-Kruger zone 36N (code 16296) without zone prefix to false easting value. Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',108.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12380','conversion','EPSG','16396','EPSG','2624','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16397','3-degree Gauss-Kruger CM 111E','3-degree Gauss-Kruger zone 37N (code 16297) without zone prefix to false easting value. Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',111.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','12381','conversion','EPSG','16397','EPSG','2625','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16398','3-degree Gauss-Kruger CM 114E','3-degree Gauss-Kruger zone 38N (code 16298) without zone prefix to false easting value. Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',114.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12382','conversion','EPSG','16398','EPSG','2626','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16399','3-degree Gauss-Kruger CM 117E','3-degree Gauss-Kruger zone 39N (code 16299) without zone prefix to false easting value. Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',117.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','12383','conversion','EPSG','16399','EPSG','2627','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16400','TM 0 N','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',0.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12384','conversion','EPSG','16400','EPSG','1629','EPSG','1136');
INSERT INTO "conversion" VALUES('EPSG','16405','TM 5 NE','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',5.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12385','conversion','EPSG','16405','EPSG','1630','EPSG','1136');
INSERT INTO "conversion" VALUES('EPSG','16406','TM 6 NE','Used by ExxonMobil in Nigeria beyond the contintental shelf. Used by Total in French Mediterranean and with effect from March 2004 all areas offshore Nigeria.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',6.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12386','conversion','EPSG','16406','EPSG','3914','EPSG','1136');
INSERT INTO "conversion" VALUES('EPSG','16411','TM 11 NE','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',11.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12387','conversion','EPSG','16411','EPSG','1489','EPSG','1136');
INSERT INTO "conversion" VALUES('EPSG','16412','TM 12 NE','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',12.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12388','conversion','EPSG','16412','EPSG','1482','EPSG','1136');
INSERT INTO "conversion" VALUES('EPSG','16413','TM 13 NE','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',13.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12389','conversion','EPSG','16413','EPSG','2771','EPSG','1136');
INSERT INTO "conversion" VALUES('EPSG','16430','TM 30 NE','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',30.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12390','conversion','EPSG','16430','EPSG','2546','EPSG','1136');
INSERT INTO "conversion" VALUES('EPSG','16490','TM 90 NE','Not part of the global UTM grid system.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',90.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12391','conversion','EPSG','16490','EPSG','1041','EPSG','1136');
INSERT INTO "usage" VALUES('EPSG','14845','conversion','EPSG','16490','EPSG','3217','EPSG','1153');
INSERT INTO "conversion" VALUES('EPSG','16506','TM 106 NE','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',106.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12392','conversion','EPSG','16506','EPSG','1495','EPSG','1136');
INSERT INTO "conversion" VALUES('EPSG','16586','GK 106 NE','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',106.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12393','conversion','EPSG','16586','EPSG','1494','EPSG','1136');
INSERT INTO "conversion" VALUES('EPSG','16611','TM 11.30 SE','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',11.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12394','conversion','EPSG','16611','EPSG','1605','EPSG','1136');
INSERT INTO "conversion" VALUES('EPSG','16612','TM 12 SE','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',12.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12395','conversion','EPSG','16612','EPSG','1604','EPSG','1136');
INSERT INTO "conversion" VALUES('EPSG','16636','TM 36 SE','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',36.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12396','conversion','EPSG','16636','EPSG','1726','EPSG','1136');
INSERT INTO "conversion" VALUES('EPSG','16709','TM 109 SE','Used by Arco and BP for ONWJ.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',109.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12397','conversion','EPSG','16709','EPSG','2577','EPSG','1136');
INSERT INTO "conversion" VALUES('EPSG','16716','TM 116 SE','Used by BP for Terang-Sirasun.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',116.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12398','conversion','EPSG','16716','EPSG','2588','EPSG','1136');
INSERT INTO "conversion" VALUES('EPSG','16732','TM 132 SE','Used for Tangguh developments.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',132.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12399','conversion','EPSG','16732','EPSG','2589','EPSG','1136');
INSERT INTO "conversion" VALUES('EPSG','16907','GSK 3-degree Gauss-Kruger zone 7','Also found with zone truncated from false easting: see GSK 3-degree Gauss-Kruger CM 21E (code 17107).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',21.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',7250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','16011','conversion','EPSG','16907','EPSG','2305','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16908','GSK 3-degree Gauss-Kruger zone 8','Also found with zone truncated from false easting: see GSK 3-degree Gauss-Kruger CM 24E (code 17108).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',24.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',8250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','16015','conversion','EPSG','16908','EPSG','2748','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16909','GSK 3-degree Gauss-Kruger zone 9','Also found with zone truncated from false easting: see GSK 3-degree Gauss-Kruger CM 27E (code 17109).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',27.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',9250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','16016','conversion','EPSG','16909','EPSG','2749','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16910','GSK 3-degree Gauss-Kruger zone 10','Also found with zone truncated from false easting: see GSK 3-degree Gauss-Kruger CM 30E (code 17110).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',30.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',10250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','16017','conversion','EPSG','16910','EPSG','2535','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16911','GSK 3-degree Gauss-Kruger zone 11','Also found with zone truncated from false easting: see GSK 3-degree Gauss-Kruger CM 33E (code 17111).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',33.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',11250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','16018','conversion','EPSG','16911','EPSG','2751','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16912','GSK 3-degree Gauss-Kruger zone 12','Also found with zone truncated from false easting: see GSK 3-degree Gauss-Kruger CM 36E (code 17112).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',36.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',12250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','16019','conversion','EPSG','16912','EPSG','2752','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16913','GSK 3-degree Gauss-Kruger zone 13','Also found with zone truncated from false easting: see GSK 3-degree Gauss-Kruger CM 39E (code 17113).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',39.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',13250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','16020','conversion','EPSG','16913','EPSG','2753','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16914','GSK 3-degree Gauss-Kruger zone 14','Also found with zone truncated from false easting: see GSK 3-degree Gauss-Kruger CM 42E (code 17114).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',42.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',14250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','16021','conversion','EPSG','16914','EPSG','2754','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16915','GSK 3-degree Gauss-Kruger zone 15','Also found with zone truncated from false easting: see GSK 3-degree Gauss-Kruger CM 45E (code 17115).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',45.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',15250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','16022','conversion','EPSG','16915','EPSG','2755','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16916','GSK 3-degree Gauss-Kruger zone 16','Also found with zone truncated from false easting: see GSK 3-degree Gauss-Kruger CM 48E (code 17116).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',48.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',16250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','16023','conversion','EPSG','16916','EPSG','2756','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16917','GSK 3-degree Gauss-Kruger zone 17','Also found with zone truncated from false easting: see GSK 3-degree Gauss-Kruger CM 51E (code 17117).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',51.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',17250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','16024','conversion','EPSG','16917','EPSG','2757','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16918','GSK 3-degree Gauss-Kruger zone 18','Also found with zone truncated from false easting: see GSK 3-degree Gauss-Kruger CM 54E (code 17118).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',54.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',18250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','16025','conversion','EPSG','16918','EPSG','2758','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16919','GSK 3-degree Gauss-Kruger zone 19','Also found with zone truncated from false easting: see GSK 3-degree Gauss-Kruger CM 57E (code 17119).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',57.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',19250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','16026','conversion','EPSG','16919','EPSG','2759','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16920','GSK 3-degree Gauss-Kruger zone 20','Also found with zone truncated from false easting: see GSK 3-degree Gauss-Kruger CM 60E (code 17120).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',60.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',20250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','16027','conversion','EPSG','16920','EPSG','2760','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16921','GSK 3-degree Gauss-Kruger zone 21','Also found with zone truncated from false easting: see GSK 3-degree Gauss-Kruger CM 63E (code 17121).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',63.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',21250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','16028','conversion','EPSG','16921','EPSG','2609','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16922','GSK 3-degree Gauss-Kruger zone 22','Also found with zone truncated from false easting: see GSK 3-degree Gauss-Kruger CM 66E (code 17122).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',66.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',22250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15917','conversion','EPSG','16922','EPSG','2762','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16923','GSK 3-degree Gauss-Kruger zone 23','Also found with zone truncated from false easting: see GSK 3-degree Gauss-Kruger CM 69E (code 17123).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',69.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',23250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15916','conversion','EPSG','16923','EPSG','2763','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16924','GSK 3-degree Gauss-Kruger zone 24','Also found with zone truncated from false easting: see GSK 3-degree Gauss-Kruger CM 72E (code 17124).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',72.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',24250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15915','conversion','EPSG','16924','EPSG','2764','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16925','GSK 3-degree Gauss-Kruger zone 25','Also found with zone truncated from false easting: see GSK 3-degree Gauss-Kruger CM 75E (code 17125).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',75.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',25250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15914','conversion','EPSG','16925','EPSG','2765','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16926','GSK 3-degree Gauss-Kruger zone 26','Also found with zone truncated from false easting: see GSK 3-degree Gauss-Kruger CM 78E (code 17126).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',78.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',26250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15913','conversion','EPSG','16926','EPSG','2766','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16927','GSK 3-degree Gauss-Kruger zone 27','Also found with zone truncated from false easting: see GSK 3-degree Gauss-Kruger CM 81E (code 17127).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',81.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',27250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15912','conversion','EPSG','16927','EPSG','2767','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16928','GSK 3-degree Gauss-Kruger zone 28','Also found with zone truncated from false easting: see GSK 3-degree Gauss-Kruger CM 84E (code 17128).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',84.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',28250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15911','conversion','EPSG','16928','EPSG','2768','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16929','GSK 3-degree Gauss-Kruger zone 29','Also found with zone truncated from false easting: see GSK 3-degree Gauss-Kruger CM 87E (code 17129).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',87.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',29250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15910','conversion','EPSG','16929','EPSG','2769','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16930','GSK 3-degree Gauss-Kruger zone 30','Also found with zone truncated from false easting: see GSK 3-degree Gauss-Kruger CM 90E (code 17130).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',90.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',30250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15909','conversion','EPSG','16930','EPSG','2676','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16931','GSK 3-degree Gauss-Kruger zone 31','Also found with zone truncated from false easting: see GSK 3-degree Gauss-Kruger CM 93E (code 17131).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',93.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',31250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15908','conversion','EPSG','16931','EPSG','2677','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16932','GSK 3-degree Gauss-Kruger zone 32','Also found with zone truncated from false easting: see GSK 3-degree Gauss-Kruger CM 96E (code 17132).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',96.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',32250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15907','conversion','EPSG','16932','EPSG','2678','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16933','GSK 3-degree Gauss-Kruger zone 33','Also found with zone truncated from false easting: see GSK 3-degree Gauss-Kruger CM 99E (code 17133).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',99.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',33250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15925','conversion','EPSG','16933','EPSG','2679','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16934','GSK 3-degree Gauss-Kruger zone 34','Also found with zone truncated from false easting: see GSK 3-degree Gauss-Kruger CM 102E (code 17134).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',102.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',34250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15926','conversion','EPSG','16934','EPSG','2680','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16935','GSK 3-degree Gauss-Kruger zone 35','Also found with zone truncated from false easting: see GSK 3-degree Gauss-Kruger CM 105E (code 17135).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',105.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',35250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15927','conversion','EPSG','16935','EPSG','2681','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16936','GSK 3-degree Gauss-Kruger zone 36','Also found with zone truncated from false easting: see GSK 3-degree Gauss-Kruger CM 108E (code 17136).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',108.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',36250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15928','conversion','EPSG','16936','EPSG','2682','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16937','GSK 3-degree Gauss-Kruger zone 37','Also found with zone truncated from false easting: see GSK 3-degree Gauss-Kruger CM 111E (code 17137).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',111.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',37250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15938','conversion','EPSG','16937','EPSG','2683','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16938','GSK 3-degree Gauss-Kruger zone 38','Also found with zone truncated from false easting: see GSK 3-degree Gauss-Kruger CM 114E (code 17138).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',114.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',38250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15939','conversion','EPSG','16938','EPSG','2684','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16939','GSK 3-degree Gauss-Kruger zone 39','Also found with zone truncated from false easting: see GSK 3-degree Gauss-Kruger CM 117E (code 17139).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',117.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',39250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15940','conversion','EPSG','16939','EPSG','2685','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16940','GSK 3-degree Gauss-Kruger zone 40','Also found with zone truncated from false easting: see GSK 3-degree Gauss-Kruger CM 120E (code 17140).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',120.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',40250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15941','conversion','EPSG','16940','EPSG','2686','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16941','GSK 3-degree Gauss-Kruger zone 41','Also found with zone truncated from false easting: see GSK 3-degree Gauss-Kruger CM 123E (code 17141).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',123.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',41250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15942','conversion','EPSG','16941','EPSG','2687','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16942','GSK 3-degree Gauss-Kruger zone 42','Also found with zone truncated from false easting: see GSK 3-degree Gauss-Kruger CM 126E (code 17142).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',126.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',42250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15943','conversion','EPSG','16942','EPSG','2688','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16943','GSK 3-degree Gauss-Kruger zone 43','Also found with zone truncated from false easting: see GSK 3-degree Gauss-Kruger CM 129E (code 17143).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',129.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',43250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15944','conversion','EPSG','16943','EPSG','2689','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16944','GSK 3-degree Gauss-Kruger zone 44','Also found with zone truncated from false easting: see GSK 3-degree Gauss-Kruger CM 132E (code 17144).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',132.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',44250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15945','conversion','EPSG','16944','EPSG','2690','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16945','GSK 3-degree Gauss-Kruger zone 45','Also found with zone truncated from false easting: see GSK 3-degree Gauss-Kruger CM 135E (code 17145).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',135.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',45250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15946','conversion','EPSG','16945','EPSG','2691','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16946','GSK 3-degree Gauss-Kruger zone 46','Also found with zone truncated from false easting: see GSK 3-degree Gauss-Kruger CM 138E (code 17146).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',138.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',46250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15947','conversion','EPSG','16946','EPSG','2692','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16947','GSK 3-degree Gauss-Kruger zone 47','Also found with zone truncated from false easting: see GSK 3-degree Gauss-Kruger CM 141E (code 17147).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',141.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',47250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15948','conversion','EPSG','16947','EPSG','2693','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16948','GSK 3-degree Gauss-Kruger zone 48','Also found with zone truncated from false easting: see GSK 3-degree Gauss-Kruger CM 144E (code 17148).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',144.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',48250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15949','conversion','EPSG','16948','EPSG','2694','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16949','GSK 3-degree Gauss-Kruger zone 49','Also found with zone truncated from false easting: see GSK 3-degree Gauss-Kruger CM 147E (code 17149).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',147.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',49250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15950','conversion','EPSG','16949','EPSG','2695','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16950','GSK 3-degree Gauss-Kruger zone 50','Also found with zone truncated from false easting: see GSK 3-degree Gauss-Kruger CM 150E (code 17150).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',150.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',50250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15951','conversion','EPSG','16950','EPSG','2696','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16951','GSK 3-degree Gauss-Kruger zone 51','Also found with zone truncated from false easting: see GSK 3-degree Gauss-Kruger CM 153E (code 17151).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',153.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',51250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15983','conversion','EPSG','16951','EPSG','2697','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16952','GSK 3-degree Gauss-Kruger zone 52','Also found with zone truncated from false easting: see GSK 3-degree Gauss-Kruger CM 156E (code 17152).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',156.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',52250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15984','conversion','EPSG','16952','EPSG','2698','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16953','GSK 3-degree Gauss-Kruger zone 53','Also found with zone truncated from false easting: see GSK 3-degree Gauss-Kruger CM 159E (code 17153).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',159.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',53250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15985','conversion','EPSG','16953','EPSG','2699','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16954','GSK 3-degree Gauss-Kruger zone 54','Also found with zone truncated from false easting: see GSK 3-degree Gauss-Kruger CM 162E (code 17154).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',162.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',54250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15986','conversion','EPSG','16954','EPSG','2700','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16955','GSK 3-degree Gauss-Kruger zone 55','Also found with zone truncated from false easting: see GSK 3-degree Gauss-Kruger CM 165E (code 17155).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',165.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',55250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15987','conversion','EPSG','16955','EPSG','2701','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16956','GSK 3-degree Gauss-Kruger zone 56','Also found with zone truncated from false easting: see GSK 3-degree Gauss-Kruger CM 168E (code 17156).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',168.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',56250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15988','conversion','EPSG','16956','EPSG','2702','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16957','GSK 3-degree Gauss-Kruger zone 57','Also found with zone truncated from false easting: see GSK 3-degree Gauss-Kruger CM 171E (code 17157).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',171.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',57250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15989','conversion','EPSG','16957','EPSG','2703','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16958','GSK 3-degree Gauss-Kruger zone 58','Also found with zone truncated from false easting: see GSK 3-degree Gauss-Kruger CM 174E (code 17158).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',174.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',58250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15990','conversion','EPSG','16958','EPSG','2704','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16959','GSK 3-degree Gauss-Kruger zone 59','Also found with zone truncated from false easting: see GSK 3-degree Gauss-Kruger CM 177E (code 17159).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',177.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',59250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15994','conversion','EPSG','16959','EPSG','2705','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16960','GSK 3-degree Gauss-Kruger zone 60','Also found with zone truncated from false easting: see GSK 3-degree Gauss-Kruger CM 180 (code 17160).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',180.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',60250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15991','conversion','EPSG','16960','EPSG','2706','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16961','GSK 3-degree Gauss-Kruger zone 61','Also found with zone truncated from false easting: see GSK 3-degree Gauss-Kruger CM 177W (code 17161).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-177.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',61250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15992','conversion','EPSG','16961','EPSG','2707','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16962','GSK 3-degree Gauss-Kruger zone 62','Also found with zone truncated from false easting: see GSK 3-degree Gauss-Kruger CM 174W (code 17162).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-174.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',62250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15993','conversion','EPSG','16962','EPSG','2708','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16963','GSK 3-degree Gauss-Kruger zone 63','Also found with zone truncated from false easting: see GSK 3-degree Gauss-Kruger CM 171W (code 17163).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-171.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',63250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15995','conversion','EPSG','16963','EPSG','2709','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','16964','GSK 3-degree Gauss-Kruger zone 64','Also found with zone truncated from false easting: see GSK 3-degree Gauss-Kruger CM 168W (code 17164).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-168.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',64250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15996','conversion','EPSG','16964','EPSG','2710','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','17001','TM 1 NW','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-1.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12400','conversion','EPSG','17001','EPSG','1505','EPSG','1136');
INSERT INTO "conversion" VALUES('EPSG','17005','TM 5 NW','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-5.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12401','conversion','EPSG','17005','EPSG','2296','EPSG','1136');
INSERT INTO "conversion" VALUES('EPSG','17054','TM 54 NW','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-54.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12402','conversion','EPSG','17054','EPSG','1727','EPSG','1136');
INSERT INTO "conversion" VALUES('EPSG','17107','GSK 3-degree Gauss-Kruger CM 21E','GSK 3-degree Gauss-Kruger zone 7N (code 16907) without zone prefix to false easting value.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',21.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','16012','conversion','EPSG','17107','EPSG','2305','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','17108','GSK 3-degree Gauss-Kruger CM 24E','GSK 3-degree Gauss-Kruger zone 8N (code 16908) without zone prefix to false easting value.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',24.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','16029','conversion','EPSG','17108','EPSG','2748','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','17109','GSK 3-degree Gauss-Kruger CM 27E','GSK 3-degree Gauss-Kruger zone 9N (code 16909) without zone prefix to false easting value.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',27.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','16030','conversion','EPSG','17109','EPSG','2749','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','17110','GSK 3-degree Gauss-Kruger CM 30E','GSK 3-degree Gauss-Kruger zone 10N (code 16910) without zone prefix to false easting value.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',30.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','16031','conversion','EPSG','17110','EPSG','2750','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','17111','GSK 3-degree Gauss-Kruger CM 33E','GSK 3-degree Gauss-Kruger zone 11N (code 16911) without zone prefix to false easting value.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',33.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','16032','conversion','EPSG','17111','EPSG','2751','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','17112','GSK 3-degree Gauss-Kruger CM 36E','GSK 3-degree Gauss-Kruger zone 12N (code 16912) without zone prefix to false easting value.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',36.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','16033','conversion','EPSG','17112','EPSG','2752','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','17113','GSK 3-degree Gauss-Kruger CM 39E','GSK 3-degree Gauss-Kruger zone 13N (code 16913) without zone prefix to false easting value.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',39.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','16034','conversion','EPSG','17113','EPSG','2753','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','17114','GSK 3-degree Gauss-Kruger CM 42E','GSK 3-degree Gauss-Kruger zone 14N (code 16914) without zone prefix to false easting value.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',42.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','16035','conversion','EPSG','17114','EPSG','2754','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','17115','GSK 3-degree Gauss-Kruger CM 45E','GSK 3-degree Gauss-Kruger zone 15N (code 16915) without zone prefix to false easting value.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',45.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','16036','conversion','EPSG','17115','EPSG','2755','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','17116','GSK 3-degree Gauss-Kruger CM 48E','GSK 3-degree Gauss-Kruger zone 16N (code 16916) without zone prefix to false easting value.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',48.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','16037','conversion','EPSG','17116','EPSG','2756','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','17117','GSK 3-degree Gauss-Kruger CM 51E','GSK 3-degree Gauss-Kruger zone 17N (code 16917) without zone prefix to false easting value.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',51.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','16040','conversion','EPSG','17117','EPSG','2757','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','17118','GSK 3-degree Gauss-Kruger CM 54E','GSK 3-degree Gauss-Kruger zone 18N (code 16918) without zone prefix to false easting value.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',54.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','16041','conversion','EPSG','17118','EPSG','2758','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','17119','GSK 3-degree Gauss-Kruger CM 57E','GSK 3-degree Gauss-Kruger zone 19N (code 16919) without zone prefix to false easting value.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',57.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','16039','conversion','EPSG','17119','EPSG','2759','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','17120','GSK 3-degree Gauss-Kruger CM 60E','GSK 3-degree Gauss-Kruger zone 20N (code 16920) without zone prefix to false easting value.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',60.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','16038','conversion','EPSG','17120','EPSG','2760','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','17121','GSK 3-degree Gauss-Kruger CM 63E','GSK 3-degree Gauss-Kruger zone 21N (code 16921) without zone prefix to false easting value.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',63.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','16042','conversion','EPSG','17121','EPSG','2761','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','17122','GSK 3-degree Gauss-Kruger CM 66E','GSK 3-degree Gauss-Kruger zone 22N (code 16922) without zone prefix to false easting value.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',66.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15933','conversion','EPSG','17122','EPSG','2762','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','17123','GSK 3-degree Gauss-Kruger CM 69E','GSK 3-degree Gauss-Kruger zone 23N (code 16923) without zone prefix to false easting value.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',69.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15937','conversion','EPSG','17123','EPSG','2763','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','17124','GSK 3-degree Gauss-Kruger CM 72E','GSK 3-degree Gauss-Kruger zone 24N (code 16924) without zone prefix to false easting value.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',72.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15936','conversion','EPSG','17124','EPSG','2764','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','17125','GSK 3-degree Gauss-Kruger CM 75E','GSK 3-degree Gauss-Kruger zone 25N (code 16925) without zone prefix to false easting value.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',75.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15935','conversion','EPSG','17125','EPSG','2765','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','17126','GSK 3-degree Gauss-Kruger CM 78E','GSK 3-degree Gauss-Kruger zone 26N (code 16926) without zone prefix to false easting value.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',78.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15934','conversion','EPSG','17126','EPSG','2766','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','17127','GSK 3-degree Gauss-Kruger CM 81E','GSK 3-degree Gauss-Kruger zone 27N (code 16927) without zone prefix to false easting value.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',81.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15932','conversion','EPSG','17127','EPSG','2767','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','17128','GSK 3-degree Gauss-Kruger CM 84E','GSK 3-degree Gauss-Kruger zone 28N (code 16928) without zone prefix to false easting value.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',84.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15931','conversion','EPSG','17128','EPSG','2768','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','17129','GSK 3-degree Gauss-Kruger CM 87E','GSK 3-degree Gauss-Kruger zone 29N (code 16929) without zone prefix to false easting value.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',87.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15930','conversion','EPSG','17129','EPSG','2769','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','17130','GSK 3-degree Gauss-Kruger CM 90E','GSK 3-degree Gauss-Kruger zone 30N (code 16930) without zone prefix to false easting value.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',90.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15929','conversion','EPSG','17130','EPSG','2676','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','17131','GSK 3-degree Gauss-Kruger CM 93E','GSK 3-degree Gauss-Kruger zone 31N (code 16931) without zone prefix to false easting value.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',93.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15923','conversion','EPSG','17131','EPSG','2677','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','17132','GSK 3-degree Gauss-Kruger CM 96E','GSK 3-degree Gauss-Kruger zone 32N (code 16932) without zone prefix to false easting value.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',96.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15924','conversion','EPSG','17132','EPSG','2678','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','17133','GSK 3-degree Gauss-Kruger CM 99E','GSK 3-degree Gauss-Kruger zone 33N (code 16933) without zone prefix to false easting value.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',99.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15921','conversion','EPSG','17133','EPSG','2679','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','17134','GSK 3-degree Gauss-Kruger CM 102E','GSK 3-degree Gauss-Kruger zone 34N (code 16934) without zone prefix to false easting value.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',102.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15920','conversion','EPSG','17134','EPSG','2680','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','17135','GSK 3-degree Gauss-Kruger CM 105E','GSK 3-degree Gauss-Kruger zone 35N (code 16935) without zone prefix to false easting value.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',105.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15919','conversion','EPSG','17135','EPSG','2681','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','17136','GSK 3-degree Gauss-Kruger CM 108E','GSK 3-degree Gauss-Kruger zone 36N (code 16936) without zone prefix to false easting value.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',108.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15918','conversion','EPSG','17136','EPSG','2682','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','17137','GSK 3-degree Gauss-Kruger CM 111E','GSK 3-degree Gauss-Kruger zone 37N (code 16937) without zone prefix to false easting value.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',111.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15953','conversion','EPSG','17137','EPSG','2683','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','17138','GSK 3-degree Gauss-Kruger CM 114E','GSK 3-degree Gauss-Kruger zone 38N (code 16938) without zone prefix to false easting value.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',114.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15954','conversion','EPSG','17138','EPSG','2684','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','17139','GSK 3-degree Gauss-Kruger CM 117E','GSK 3-degree Gauss-Kruger zone 39N (code 16939) without zone prefix to false easting value.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',117.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15955','conversion','EPSG','17139','EPSG','2685','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','17140','GSK 3-degree Gauss-Kruger CM 120E','GSK 3-degree Gauss-Kruger zone 40N (code 16940) without zone prefix to false easting value.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',120.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15956','conversion','EPSG','17140','EPSG','2686','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','17141','GSK 3-degree Gauss-Kruger CM 123E','GSK 3-degree Gauss-Kruger zone 41N (code 16941) without zone prefix to false easting value.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',123.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15957','conversion','EPSG','17141','EPSG','2687','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','17142','GSK 3-degree Gauss-Kruger CM 126E','GSK 3-degree Gauss-Kruger zone 42N (code 16942) without zone prefix to false easting value.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',126.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15958','conversion','EPSG','17142','EPSG','2688','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','17143','GSK 3-degree Gauss-Kruger CM 129E','GSK 3-degree Gauss-Kruger zone 43N (code 16943) without zone prefix to false easting value.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',129.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15959','conversion','EPSG','17143','EPSG','2689','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','17144','GSK 3-degree Gauss-Kruger CM 132E','GSK 3-degree Gauss-Kruger zone 44N (code 16944) without zone prefix to false easting value.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',132.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15960','conversion','EPSG','17144','EPSG','2690','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','17145','GSK 3-degree Gauss-Kruger CM 135E','GSK 3-degree Gauss-Kruger zone 45N (code 16945) without zone prefix to false easting value.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',135.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15961','conversion','EPSG','17145','EPSG','2691','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','17146','GSK 3-degree Gauss-Kruger CM 138E','GSK 3-degree Gauss-Kruger zone 46N (code 16946) without zone prefix to false easting value.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',138.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15962','conversion','EPSG','17146','EPSG','2692','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','17147','GSK 3-degree Gauss-Kruger CM 141E','GSK 3-degree Gauss-Kruger zone 47N (code 16947) without zone prefix to false easting value.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',141.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15963','conversion','EPSG','17147','EPSG','2693','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','17148','GSK 3-degree Gauss-Kruger CM 144E','GSK 3-degree Gauss-Kruger zone 48N (code 16948) without zone prefix to false easting value.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',144.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15964','conversion','EPSG','17148','EPSG','2694','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','17149','GSK 3-degree Gauss-Kruger CM 147E','GSK 3-degree Gauss-Kruger zone 49N (code 16949) without zone prefix to false easting value.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',147.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15965','conversion','EPSG','17149','EPSG','2695','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','17150','GSK 3-degree Gauss-Kruger CM 150E','GSK 3-degree Gauss-Kruger zone 50N (code 16950) without zone prefix to false easting value.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',150.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15966','conversion','EPSG','17150','EPSG','2696','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','17151','GSK 3-degree Gauss-Kruger CM 153E','GSK 3-degree Gauss-Kruger zone 51N (code 16951) without zone prefix to false easting value.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',153.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15997','conversion','EPSG','17151','EPSG','2697','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','17152','GSK 3-degree Gauss-Kruger CM 156E','GSK 3-degree Gauss-Kruger zone 52N (code 16952) without zone prefix to false easting value.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',156.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15998','conversion','EPSG','17152','EPSG','2698','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','17153','GSK 3-degree Gauss-Kruger CM 159E','GSK 3-degree Gauss-Kruger zone 53N (code 16953) without zone prefix to false easting value.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',159.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','15999','conversion','EPSG','17153','EPSG','2699','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','17154','GSK 3-degree Gauss-Kruger CM 162E','GSK 3-degree Gauss-Kruger zone 54N (code 16954) without zone prefix to false easting value.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',162.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','16000','conversion','EPSG','17154','EPSG','2700','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','17155','GSK 3-degree Gauss-Kruger CM 165E','GSK 3-degree Gauss-Kruger zone 55N (code 16955) without zone prefix to false easting value.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',165.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','16001','conversion','EPSG','17155','EPSG','2701','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','17156','GSK 3-degree Gauss-Kruger CM 168E','GSK 3-degree Gauss-Kruger zone 56N (code 16956) without zone prefix to false easting value.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',168.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','16002','conversion','EPSG','17156','EPSG','2702','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','17157','GSK 3-degree Gauss-Kruger CM 171E','GSK 3-degree Gauss-Kruger zone 57N (code 16957) without zone prefix to false easting value.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',171.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','16003','conversion','EPSG','17157','EPSG','2703','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','17158','GSK 3-degree Gauss-Kruger CM 174E','GSK 3-degree Gauss-Kruger zone 58N (code 16958) without zone prefix to false easting value.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',174.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','16004','conversion','EPSG','17158','EPSG','2704','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','17159','GSK 3-degree Gauss-Kruger CM 177E','GSK 3-degree Gauss-Kruger zone 59N (code 16959) without zone prefix to false easting value.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',177.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','16005','conversion','EPSG','17159','EPSG','2705','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','17160','GSK 3-degree Gauss-Kruger CM 180E','GSK 3-degree Gauss-Kruger zone 60N (code 16960) without zone prefix to false easting value.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',180.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','16482','conversion','EPSG','17160','EPSG','2706','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','17161','GSK 3-degree Gauss-Kruger CM 177W','GSK 3-degree Gauss-Kruger zone 61N (code 16961) without zone prefix to false easting value.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-177.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','16007','conversion','EPSG','17161','EPSG','2707','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','17162','GSK 3-degree Gauss-Kruger CM 174W','GSK 3-degree Gauss-Kruger zone 62N (code 16962) without zone prefix to false easting value.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-174.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','16008','conversion','EPSG','17162','EPSG','2708','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','17163','GSK 3-degree Gauss-Kruger CM 171W','GSK 3-degree Gauss-Kruger zone 63N (code 16963) without zone prefix to false easting value.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-171.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','16009','conversion','EPSG','17163','EPSG','2709','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','17164','GSK 3-degree Gauss-Kruger CM 168W','GSK 3-degree Gauss-Kruger zone 64N (code 16964) without zone prefix to false easting value.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-168.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','16010','conversion','EPSG','17164','EPSG','2710','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','17204','SCAR IMW SP19-20','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-90.0,'EPSG','9102','EPSG','8822','Longitude of false origin',-66.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-60.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-63.2,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12403','conversion','EPSG','17204','EPSG','2991','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17205','SCAR IMW SP21-22','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-90.0,'EPSG','9102','EPSG','8822','Longitude of false origin',-54.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-60.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-63.2,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12404','conversion','EPSG','17205','EPSG','2992','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17206','SCAR IMW SP23-24','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-90.0,'EPSG','9102','EPSG','8822','Longitude of false origin',-42.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-60.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-63.2,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12405','conversion','EPSG','17206','EPSG','2993','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17207','SCAR IMW SQ01-02','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-90.0,'EPSG','9102','EPSG','8822','Longitude of false origin',-174.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-64.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-67.2,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12406','conversion','EPSG','17207','EPSG','2994','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17208','SCAR IMW SQ19-20','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-90.0,'EPSG','9102','EPSG','8822','Longitude of false origin',-66.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-64.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-67.2,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12407','conversion','EPSG','17208','EPSG','2995','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17209','SCAR IMW SQ21-22','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-90.0,'EPSG','9102','EPSG','8822','Longitude of false origin',-54.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-64.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-67.2,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12408','conversion','EPSG','17209','EPSG','2996','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17210','SCAR IMW SQ37-38','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-90.0,'EPSG','9102','EPSG','8822','Longitude of false origin',42.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-64.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-67.2,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12409','conversion','EPSG','17210','EPSG','2997','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17211','SCAR IMW SQ39-40','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-90.0,'EPSG','9102','EPSG','8822','Longitude of false origin',54.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-64.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-67.2,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12410','conversion','EPSG','17211','EPSG','2998','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17212','SCAR IMW SQ41-42','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-90.0,'EPSG','9102','EPSG','8822','Longitude of false origin',66.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-64.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-67.2,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12411','conversion','EPSG','17212','EPSG','2999','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17213','SCAR IMW SQ43-44','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-90.0,'EPSG','9102','EPSG','8822','Longitude of false origin',78.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-64.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-67.2,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12412','conversion','EPSG','17213','EPSG','3000','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17214','SCAR IMW SQ45-46','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-90.0,'EPSG','9102','EPSG','8822','Longitude of false origin',90.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-64.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-67.2,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12413','conversion','EPSG','17214','EPSG','3001','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17215','SCAR IMW SQ47-48','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-90.0,'EPSG','9102','EPSG','8822','Longitude of false origin',102.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-64.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-67.2,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12414','conversion','EPSG','17215','EPSG','3002','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17216','SCAR IMW SQ49-50','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-90.0,'EPSG','9102','EPSG','8822','Longitude of false origin',114.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-64.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-67.2,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12415','conversion','EPSG','17216','EPSG','3003','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17217','SCAR IMW SQ51-52','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-90.0,'EPSG','9102','EPSG','8822','Longitude of false origin',126.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-64.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-67.2,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12416','conversion','EPSG','17217','EPSG','3004','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17218','SCAR IMW SQ53-54','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-90.0,'EPSG','9102','EPSG','8822','Longitude of false origin',138.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-64.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-67.2,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12417','conversion','EPSG','17218','EPSG','3005','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17219','SCAR IMW SQ55-56','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-90.0,'EPSG','9102','EPSG','8822','Longitude of false origin',150.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-64.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-67.2,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12418','conversion','EPSG','17219','EPSG','3006','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17220','SCAR IMW SQ57-58','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-90.0,'EPSG','9102','EPSG','8822','Longitude of false origin',162.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-64.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-67.2,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12419','conversion','EPSG','17220','EPSG','3007','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17221','SCAR IMW SR13-14','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-90.0,'EPSG','9102','EPSG','8822','Longitude of false origin',-102.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-68.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-71.2,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12420','conversion','EPSG','17221','EPSG','3008','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17222','SCAR IMW SR15-16','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-90.0,'EPSG','9102','EPSG','8822','Longitude of false origin',-90.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-68.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-71.2,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12421','conversion','EPSG','17222','EPSG','3009','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17223','SCAR IMW SR17-18','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-90.0,'EPSG','9102','EPSG','8822','Longitude of false origin',-78.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-68.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-71.2,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12422','conversion','EPSG','17223','EPSG','3010','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17224','SCAR IMW SR19-20','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-90.0,'EPSG','9102','EPSG','8822','Longitude of false origin',-66.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-68.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-71.2,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12423','conversion','EPSG','17224','EPSG','3011','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17225','SCAR IMW SR27-28','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-90.0,'EPSG','9102','EPSG','8822','Longitude of false origin',-18.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-68.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-71.2,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12424','conversion','EPSG','17225','EPSG','3012','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17226','SCAR IMW SR29-30','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-90.0,'EPSG','9102','EPSG','8822','Longitude of false origin',-6.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-68.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-71.2,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12425','conversion','EPSG','17226','EPSG','3013','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17227','SCAR IMW SR31-32','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-90.0,'EPSG','9102','EPSG','8822','Longitude of false origin',6.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-68.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-71.2,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12426','conversion','EPSG','17227','EPSG','3014','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17228','SCAR IMW SR33-34','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-90.0,'EPSG','9102','EPSG','8822','Longitude of false origin',18.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-68.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-71.2,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12427','conversion','EPSG','17228','EPSG','3015','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17229','SCAR IMW SR35-36','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-90.0,'EPSG','9102','EPSG','8822','Longitude of false origin',30.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-68.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-71.2,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12428','conversion','EPSG','17229','EPSG','3016','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17230','SCAR IMW SR37-38','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-90.0,'EPSG','9102','EPSG','8822','Longitude of false origin',42.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-68.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-71.2,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12429','conversion','EPSG','17230','EPSG','3017','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17231','SCAR IMW SR39-40','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-90.0,'EPSG','9102','EPSG','8822','Longitude of false origin',54.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-68.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-71.2,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12430','conversion','EPSG','17231','EPSG','3018','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17232','SCAR IMW SR41-42','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-90.0,'EPSG','9102','EPSG','8822','Longitude of false origin',66.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-68.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-71.2,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12431','conversion','EPSG','17232','EPSG','3019','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17233','SCAR IMW SR43-44','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-90.0,'EPSG','9102','EPSG','8822','Longitude of false origin',78.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-68.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-71.2,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12432','conversion','EPSG','17233','EPSG','3020','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17234','SCAR IMW SR45-46','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-90.0,'EPSG','9102','EPSG','8822','Longitude of false origin',90.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-68.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-71.2,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12433','conversion','EPSG','17234','EPSG','3021','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17235','SCAR IMW SR47-48','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-90.0,'EPSG','9102','EPSG','8822','Longitude of false origin',102.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-68.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-71.2,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12434','conversion','EPSG','17235','EPSG','3022','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17236','SCAR IMW SR49-50','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-90.0,'EPSG','9102','EPSG','8822','Longitude of false origin',114.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-68.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-71.2,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12435','conversion','EPSG','17236','EPSG','3023','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17237','SCAR IMW SR51-52','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-90.0,'EPSG','9102','EPSG','8822','Longitude of false origin',126.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-68.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-71.2,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12436','conversion','EPSG','17237','EPSG','3024','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17238','SCAR IMW SR53-54','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-90.0,'EPSG','9102','EPSG','8822','Longitude of false origin',138.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-68.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-71.2,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12437','conversion','EPSG','17238','EPSG','3025','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17239','SCAR IMW SR55-56','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-90.0,'EPSG','9102','EPSG','8822','Longitude of false origin',150.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-68.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-71.2,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12438','conversion','EPSG','17239','EPSG','3026','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17240','SCAR IMW SR57-58','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-90.0,'EPSG','9102','EPSG','8822','Longitude of false origin',162.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-68.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-71.2,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12439','conversion','EPSG','17240','EPSG','3027','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17241','SCAR IMW SR59-60','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-90.0,'EPSG','9102','EPSG','8822','Longitude of false origin',174.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-68.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-71.2,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12440','conversion','EPSG','17241','EPSG','3028','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17242','SCAR IMW SS04-06','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-90.0,'EPSG','9102','EPSG','8822','Longitude of false origin',-153.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-72.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-75.2,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12441','conversion','EPSG','17242','EPSG','3029','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17243','SCAR IMW SS07-09','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-90.0,'EPSG','9102','EPSG','8822','Longitude of false origin',-135.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-72.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-75.2,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12442','conversion','EPSG','17243','EPSG','3030','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17244','SCAR IMW SS10-12','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-90.0,'EPSG','9102','EPSG','8822','Longitude of false origin',-117.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-72.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-75.2,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12443','conversion','EPSG','17244','EPSG','3031','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17245','SCAR IMW SS13-15','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-90.0,'EPSG','9102','EPSG','8822','Longitude of false origin',-99.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-72.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-75.2,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12444','conversion','EPSG','17245','EPSG','3032','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17246','SCAR IMW SS16-18','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-90.0,'EPSG','9102','EPSG','8822','Longitude of false origin',-81.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-72.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-75.2,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12445','conversion','EPSG','17246','EPSG','3033','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17247','SCAR IMW SS19-21','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-90.0,'EPSG','9102','EPSG','8822','Longitude of false origin',-63.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-72.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-75.2,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12446','conversion','EPSG','17247','EPSG','3034','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17248','SCAR IMW SS25-27','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-90.0,'EPSG','9102','EPSG','8822','Longitude of false origin',-27.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-72.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-75.2,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12447','conversion','EPSG','17248','EPSG','3035','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17249','SCAR IMW SS28-30','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-90.0,'EPSG','9102','EPSG','8822','Longitude of false origin',-9.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-72.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-75.2,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12448','conversion','EPSG','17249','EPSG','3036','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17250','SCAR IMW SS31-33','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-90.0,'EPSG','9102','EPSG','8822','Longitude of false origin',9.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-72.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-75.2,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12449','conversion','EPSG','17250','EPSG','3037','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17251','SCAR IMW SS34-36','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-90.0,'EPSG','9102','EPSG','8822','Longitude of false origin',27.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-72.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-75.2,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12450','conversion','EPSG','17251','EPSG','3038','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17252','SCAR IMW SS37-39','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-90.0,'EPSG','9102','EPSG','8822','Longitude of false origin',45.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-72.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-75.2,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12451','conversion','EPSG','17252','EPSG','3039','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17253','SCAR IMW SS40-42','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-90.0,'EPSG','9102','EPSG','8822','Longitude of false origin',63.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-72.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-75.2,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12452','conversion','EPSG','17253','EPSG','3040','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17254','SCAR IMW SS43-45','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-90.0,'EPSG','9102','EPSG','8822','Longitude of false origin',81.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-72.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-75.2,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12453','conversion','EPSG','17254','EPSG','3041','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17255','SCAR IMW SS46-48','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-90.0,'EPSG','9102','EPSG','8822','Longitude of false origin',99.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-72.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-75.2,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12454','conversion','EPSG','17255','EPSG','3042','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17256','SCAR IMW SS49-51','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-90.0,'EPSG','9102','EPSG','8822','Longitude of false origin',117.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-72.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-75.2,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12455','conversion','EPSG','17256','EPSG','3043','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17257','SCAR IMW SS52-54','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-90.0,'EPSG','9102','EPSG','8822','Longitude of false origin',135.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-72.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-75.2,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12456','conversion','EPSG','17257','EPSG','3044','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17258','SCAR IMW SS55-57','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-90.0,'EPSG','9102','EPSG','8822','Longitude of false origin',153.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-72.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-75.2,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12457','conversion','EPSG','17258','EPSG','3045','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17259','SCAR IMW SS58-60','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-90.0,'EPSG','9102','EPSG','8822','Longitude of false origin',171.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-72.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-75.2,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12458','conversion','EPSG','17259','EPSG','3046','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17260','SCAR IMW ST01-04','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-90.0,'EPSG','9102','EPSG','8822','Longitude of false origin',-168.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-76.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-79.2,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12459','conversion','EPSG','17260','EPSG','3047','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17261','SCAR IMW ST05-08','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-90.0,'EPSG','9102','EPSG','8822','Longitude of false origin',-144.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-76.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-79.2,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12460','conversion','EPSG','17261','EPSG','3048','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17262','SCAR IMW ST09-12','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-90.0,'EPSG','9102','EPSG','8822','Longitude of false origin',-120.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-76.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-79.2,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12461','conversion','EPSG','17262','EPSG','3049','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17263','SCAR IMW ST13-16','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-90.0,'EPSG','9102','EPSG','8822','Longitude of false origin',-96.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-76.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-79.2,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12462','conversion','EPSG','17263','EPSG','3050','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17264','SCAR IMW ST17-20','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-90.0,'EPSG','9102','EPSG','8822','Longitude of false origin',-72.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-76.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-79.2,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12463','conversion','EPSG','17264','EPSG','3051','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17265','SCAR IMW ST21-24','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-90.0,'EPSG','9102','EPSG','8822','Longitude of false origin',-48.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-76.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-79.2,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12464','conversion','EPSG','17265','EPSG','3052','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17266','SCAR IMW ST25-28','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-90.0,'EPSG','9102','EPSG','8822','Longitude of false origin',-24.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-76.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-79.2,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12465','conversion','EPSG','17266','EPSG','3053','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17267','SCAR IMW ST29-32','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-90.0,'EPSG','9102','EPSG','8822','Longitude of false origin',0.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-76.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-79.2,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12466','conversion','EPSG','17267','EPSG','3054','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17268','SCAR IMW ST33-36','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-90.0,'EPSG','9102','EPSG','8822','Longitude of false origin',24.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-76.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-79.2,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12467','conversion','EPSG','17268','EPSG','3055','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17269','SCAR IMW ST37-40','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-90.0,'EPSG','9102','EPSG','8822','Longitude of false origin',48.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-76.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-79.2,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12468','conversion','EPSG','17269','EPSG','3056','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17270','SCAR IMW ST41-44','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-90.0,'EPSG','9102','EPSG','8822','Longitude of false origin',72.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-76.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-79.2,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12469','conversion','EPSG','17270','EPSG','3057','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17271','SCAR IMW ST45-48','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-90.0,'EPSG','9102','EPSG','8822','Longitude of false origin',96.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-76.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-79.2,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12470','conversion','EPSG','17271','EPSG','3058','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17272','SCAR IMW ST49-52','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-90.0,'EPSG','9102','EPSG','8822','Longitude of false origin',120.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-76.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-79.2,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12471','conversion','EPSG','17272','EPSG','3059','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17273','SCAR IMW ST53-56','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-90.0,'EPSG','9102','EPSG','8822','Longitude of false origin',144.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-76.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-79.2,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12472','conversion','EPSG','17273','EPSG','3060','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17274','SCAR IMW ST57-60','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-90.0,'EPSG','9102','EPSG','8822','Longitude of false origin',168.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-76.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-79.2,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12473','conversion','EPSG','17274','EPSG','3061','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17275','SCAR IMW SU01-05','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9829','Polar Stereographic (variant B)','EPSG','8832','Latitude of standard parallel',-80.1419,'EPSG','9110','EPSG','8833','Longitude of origin',-165.0,'EPSG','9102','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12474','conversion','EPSG','17275','EPSG','3062','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17276','SCAR IMW SU06-10','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9829','Polar Stereographic (variant B)','EPSG','8832','Latitude of standard parallel',-80.1419,'EPSG','9110','EPSG','8833','Longitude of origin',-135.0,'EPSG','9102','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12475','conversion','EPSG','17276','EPSG','3063','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17277','SCAR IMW SU11-15','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9829','Polar Stereographic (variant B)','EPSG','8832','Latitude of standard parallel',-80.1419,'EPSG','9110','EPSG','8833','Longitude of origin',-105.0,'EPSG','9102','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12476','conversion','EPSG','17277','EPSG','3064','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17278','SCAR IMW SU16-20','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9829','Polar Stereographic (variant B)','EPSG','8832','Latitude of standard parallel',-80.1419,'EPSG','9110','EPSG','8833','Longitude of origin',-75.0,'EPSG','9102','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12477','conversion','EPSG','17278','EPSG','3065','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17279','SCAR IMW SU21-25','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9829','Polar Stereographic (variant B)','EPSG','8832','Latitude of standard parallel',-80.1419,'EPSG','9110','EPSG','8833','Longitude of origin',-45.0,'EPSG','9102','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12478','conversion','EPSG','17279','EPSG','3066','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17280','SCAR IMW SU26-30','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9829','Polar Stereographic (variant B)','EPSG','8832','Latitude of standard parallel',-80.1419,'EPSG','9110','EPSG','8833','Longitude of origin',-15.0,'EPSG','9102','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12479','conversion','EPSG','17280','EPSG','3067','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17281','SCAR IMW SU31-35','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9829','Polar Stereographic (variant B)','EPSG','8832','Latitude of standard parallel',-80.1419,'EPSG','9110','EPSG','8833','Longitude of origin',15.0,'EPSG','9102','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12480','conversion','EPSG','17281','EPSG','3068','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17282','SCAR IMW SU36-40','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9829','Polar Stereographic (variant B)','EPSG','8832','Latitude of standard parallel',-80.1419,'EPSG','9110','EPSG','8833','Longitude of origin',45.0,'EPSG','9102','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12481','conversion','EPSG','17282','EPSG','3069','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17283','SCAR IMW SU41-45','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9829','Polar Stereographic (variant B)','EPSG','8832','Latitude of standard parallel',-80.1419,'EPSG','9110','EPSG','8833','Longitude of origin',75.0,'EPSG','9102','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12482','conversion','EPSG','17283','EPSG','3070','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17284','SCAR IMW SU46-50','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9829','Polar Stereographic (variant B)','EPSG','8832','Latitude of standard parallel',-80.1419,'EPSG','9110','EPSG','8833','Longitude of origin',105.0,'EPSG','9102','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12483','conversion','EPSG','17284','EPSG','3071','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17285','SCAR IMW SU51-55','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9829','Polar Stereographic (variant B)','EPSG','8832','Latitude of standard parallel',-80.1419,'EPSG','9110','EPSG','8833','Longitude of origin',135.0,'EPSG','9102','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12484','conversion','EPSG','17285','EPSG','3072','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17286','SCAR IMW SU56-60','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9829','Polar Stereographic (variant B)','EPSG','8832','Latitude of standard parallel',-80.1419,'EPSG','9110','EPSG','8833','Longitude of origin',165.0,'EPSG','9102','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12485','conversion','EPSG','17286','EPSG','3073','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17287','SCAR IMW SV01-10','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9829','Polar Stereographic (variant B)','EPSG','8832','Latitude of standard parallel',-80.1419,'EPSG','9110','EPSG','8833','Longitude of origin',-150.0,'EPSG','9102','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12486','conversion','EPSG','17287','EPSG','3074','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17288','SCAR IMW SV11-20','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9829','Polar Stereographic (variant B)','EPSG','8832','Latitude of standard parallel',-80.1419,'EPSG','9110','EPSG','8833','Longitude of origin',-90.0,'EPSG','9102','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12487','conversion','EPSG','17288','EPSG','3075','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17289','SCAR IMW SV21-30','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9829','Polar Stereographic (variant B)','EPSG','8832','Latitude of standard parallel',-80.1419,'EPSG','9110','EPSG','8833','Longitude of origin',-30.0,'EPSG','9102','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12488','conversion','EPSG','17289','EPSG','3076','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17290','SCAR IMW SV31-40','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9829','Polar Stereographic (variant B)','EPSG','8832','Latitude of standard parallel',-80.1419,'EPSG','9110','EPSG','8833','Longitude of origin',30.0,'EPSG','9102','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12489','conversion','EPSG','17290','EPSG','3077','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17291','SCAR IMW SV41-50','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9829','Polar Stereographic (variant B)','EPSG','8832','Latitude of standard parallel',-80.1419,'EPSG','9110','EPSG','8833','Longitude of origin',90.0,'EPSG','9102','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12490','conversion','EPSG','17291','EPSG','3078','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17292','SCAR IMW SV51-60','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9829','Polar Stereographic (variant B)','EPSG','8832','Latitude of standard parallel',-80.1419,'EPSG','9110','EPSG','8833','Longitude of origin',150.0,'EPSG','9102','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12491','conversion','EPSG','17292','EPSG','3079','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17293','SCAR IMW SW01-60','After: Sievers, J. and H. Bennat (1989). "Reference systems of maps and geographic information systems of Antarctica." Antarctic Science 1(4): 351-362.','EPSG','9829','Polar Stereographic (variant B)','EPSG','8832','Latitude of standard parallel',-80.1419,'EPSG','9110','EPSG','8833','Longitude of origin',0.0,'EPSG','9102','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12492','conversion','EPSG','17293','EPSG','3080','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','17294','USGS Transantarctic Mountains','USGS mapping of Transantarctic mountains.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-78.0,'EPSG','9102','EPSG','8822','Longitude of false origin',162.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-76.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-79.2,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12493','conversion','EPSG','17294','EPSG','3081','EPSG','1153');
INSERT INTO "conversion" VALUES('EPSG','17295','North Pole Lambert Azimuthal Equal Area (Bering Sea)','For studies of Bering Sea area.','EPSG','9820','Lambert Azimuthal Equal Area','EPSG','8801','Latitude of natural origin',90.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',180.0,'EPSG','9102','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12494','conversion','EPSG','17295','EPSG','3480','EPSG','1040');
INSERT INTO "conversion" VALUES('EPSG','17296','North Pole Lambert Azimuthal Equal Area (Alaska)','For studies of Alaskan area.','EPSG','9820','Lambert Azimuthal Equal Area','EPSG','8801','Latitude of natural origin',90.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-150.0,'EPSG','9102','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12495','conversion','EPSG','17296','EPSG','3480','EPSG','1040');
INSERT INTO "conversion" VALUES('EPSG','17297','North Pole Lambert Azimuthal Equal Area (Canada)','For studies of Canadian area.','EPSG','9820','Lambert Azimuthal Equal Area','EPSG','8801','Latitude of natural origin',90.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-100.0,'EPSG','9102','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12496','conversion','EPSG','17297','EPSG','3480','EPSG','1040');
INSERT INTO "conversion" VALUES('EPSG','17298','North Pole Lambert Azimuthal Equal Area (Atlantic)','For studies of North Atlantic and Greenland area.','EPSG','9820','Lambert Azimuthal Equal Area','EPSG','8801','Latitude of natural origin',90.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-40.0,'EPSG','9102','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12497','conversion','EPSG','17298','EPSG','3480','EPSG','1040');
INSERT INTO "conversion" VALUES('EPSG','17299','North Pole Lambert Azimuthal Equal Area (Europe)','For studies of north European area.','EPSG','9820','Lambert Azimuthal Equal Area','EPSG','8801','Latitude of natural origin',90.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',10.0,'EPSG','9102','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12498','conversion','EPSG','17299','EPSG','3480','EPSG','1040');
INSERT INTO "conversion" VALUES('EPSG','17300','North Pole Lambert Azimuthal Equal Area (Russia)','For studies of Russian area.','EPSG','9820','Lambert Azimuthal Equal Area','EPSG','8801','Latitude of natural origin',90.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',90.0,'EPSG','9102','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12499','conversion','EPSG','17300','EPSG','3480','EPSG','1040');
INSERT INTO "conversion" VALUES('EPSG','17321','SWEREF99 12 00','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',12.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',150000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12500','conversion','EPSG','17321','EPSG','2833','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','17322','SWEREF99 13 30','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',13.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',150000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12501','conversion','EPSG','17322','EPSG','2834','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','17323','SWEREF99 15 00','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',15.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',150000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12502','conversion','EPSG','17323','EPSG','2835','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','17324','SWEREF99 16 30','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',16.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',150000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12503','conversion','EPSG','17324','EPSG','2836','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','17325','SWEREF99 18 00','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',18.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',150000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12504','conversion','EPSG','17325','EPSG','2837','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','17326','SWEREF99 14 15','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',14.15,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',150000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12505','conversion','EPSG','17326','EPSG','2838','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','17327','SWEREF99 15 45','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',15.45,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',150000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12506','conversion','EPSG','17327','EPSG','2839','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','17328','SWEREF99 17 15','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',17.15,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',150000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12507','conversion','EPSG','17328','EPSG','2840','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','17329','SWEREF99 18 45','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',18.45,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',150000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12508','conversion','EPSG','17329','EPSG','2841','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','17330','SWEREF99 20 15','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',20.15,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',150000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12509','conversion','EPSG','17330','EPSG','2842','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','17331','SWEREF99 21 45','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',21.45,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',150000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12510','conversion','EPSG','17331','EPSG','2843','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','17332','SWEREF99 23 15','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',23.15,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',150000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12511','conversion','EPSG','17332','EPSG','2844','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','17333','SWEREF99 TM','Projection parameters are identical to UTM zone 33N. Unlike UTM zone 33N, the SWREF99 TM is used throughout all Sweden.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',15.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12512','conversion','EPSG','17333','EPSG','1225','EPSG','1210');
INSERT INTO "conversion" VALUES('EPSG','17334','Sweden zone 7.5 gon V','At the municipal level alternative projections are found defined with different sets of False Northing and Easting, based on 100 km grid squares. This is denoted by the last part of the name.  For example 61:-1 means FN = -6100000 and FE = 100000 m.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',11.18298,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',1500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12513','conversion','EPSG','17334','EPSG','2845','EPSG','1056');
INSERT INTO "conversion" VALUES('EPSG','17335','Sweden zone 5 gon V','At the municipal level alternative projections are found defined with different sets of False Northing and Easting, based on 100 km grid squares. This is denoted by the last part of the name.  For example 61:-1 means FN = -6100000 and FE = 100000 m.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',13.33298,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',1500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12514','conversion','EPSG','17335','EPSG','2846','EPSG','1056');
INSERT INTO "conversion" VALUES('EPSG','17336','Sweden zone 0 gon','At the municipal level alternative projections are found defined with different sets of False Northing and Easting, based on 100 km grid squares. This is denoted by the last part of the name.  For example 61:-1 means FN = -6100000 and FE = 100000 m.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',18.03298,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',1500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12515','conversion','EPSG','17336','EPSG','2848','EPSG','1056');
INSERT INTO "conversion" VALUES('EPSG','17337','Sweden zone 2.5 gon O','At the municipal level alternative projections are found defined with different sets of False Northing and Easting, based on 100 km grid squares. This is denoted by the last part of the name.  For example 61:-1 means FN = -6100000 and FE = 100000 m.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',20.18298,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',1500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12516','conversion','EPSG','17337','EPSG','2849','EPSG','1056');
INSERT INTO "conversion" VALUES('EPSG','17338','Sweden zone 5 gon O','At the municipal level alternative projections are found defined with different sets of False Northing and Easting, based on 100 km grid squares. This is denoted by the last part of the name.  For example 61:-1 means FN = -6100000 and FE = 100000 m.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',22.33298,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',1500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12517','conversion','EPSG','17338','EPSG','2850','EPSG','1056');
INSERT INTO "conversion" VALUES('EPSG','17339','RT90 zone 7.5 gon V emulation','This projection embeds an approximation to 0.2m accuracy of the Sweref99 and WGS 84 to RT90 transformations (tfm codes 1895 and 1896).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',11.18225,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000006,'EPSG','9201','EPSG','8806','False easting',1500025.141,'EPSG','9001','EPSG','8807','False northing',-667.282,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12518','conversion','EPSG','17339','EPSG','2845','EPSG','1230');
INSERT INTO "conversion" VALUES('EPSG','17340','RT90 zone 5 gon V emulation','This projection embeds an approximation to 0.2m accuracy of the Sweref99 and WGS 84 to RT90 transformations (tfm codes 1895 and 1896).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',13.332256,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000058,'EPSG','9201','EPSG','8806','False easting',1500044.695,'EPSG','9001','EPSG','8807','False northing',-667.13,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12519','conversion','EPSG','17340','EPSG','2846','EPSG','1230');
INSERT INTO "conversion" VALUES('EPSG','17341','RT90 zone 2.5 gon V emulation','This projection embeds an approximation to 0.2m accuracy of the Sweref99 and WGS 84 to RT90 transformations (tfm codes 1895 and 1896).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',15.4822624306,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.00000561024,'EPSG','9201','EPSG','8806','False easting',1500064.274,'EPSG','9001','EPSG','8807','False northing',-667.711,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12520','conversion','EPSG','17341','EPSG','2847','EPSG','1230');
INSERT INTO "conversion" VALUES('EPSG','17342','RT90 zone 0 gon emulation','This projection embeds an approximation to 0.2m accuracy of the Sweref99 and WGS 84 to RT90 transformations (tfm codes 1895 and 1896).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',18.032268,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000054,'EPSG','9201','EPSG','8806','False easting',1500083.521,'EPSG','9001','EPSG','8807','False northing',-668.844,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12521','conversion','EPSG','17342','EPSG','2848','EPSG','1230');
INSERT INTO "conversion" VALUES('EPSG','17343','RT90 zone 2.5 gon O emulation','This projection embeds an approximation to 0.2m accuracy of the Sweref99 and WGS 84 to RT90 transformations (tfm codes 1895 and 1896).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',20.182274,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000052,'EPSG','9201','EPSG','8806','False easting',1500102.765,'EPSG','9001','EPSG','8807','False northing',-670.706,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12522','conversion','EPSG','17343','EPSG','2849','EPSG','1230');
INSERT INTO "conversion" VALUES('EPSG','17344','RT90 zone 5 gon O emulation','This projection embeds an approximation to 0.2m accuracy of the Sweref99 and WGS 84 to RT90 transformations (tfm codes 1895 and 1896).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',22.33228,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000049,'EPSG','9201','EPSG','8806','False easting',1500121.846,'EPSG','9001','EPSG','8807','False northing',-672.557,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12523','conversion','EPSG','17344','EPSG','2850','EPSG','1230');
INSERT INTO "conversion" VALUES('EPSG','17348','Map Grid of Australia zone 48','Grid convergence uses opposite sign convention to UTM. Note: From 2012-01 to 2014-02 (v7.9.4 to v8.3.3) this record status = deprecated as area falls outside EEZ. Reinstated as valid on ratification of limit of Australian continental shelf beyond EEZ.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',105.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12524','conversion','EPSG','17348','EPSG','4191','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','17349','Map Grid of Australia zone 49','Grid convergence uses opposite sign convention to UTM.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',111.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12525','conversion','EPSG','17349','EPSG','4176','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','17350','Map Grid of Australia zone 50','Grid convergence uses opposite sign convention to UTM.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',117.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12526','conversion','EPSG','17350','EPSG','4178','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','17351','Map Grid of Australia zone 51','Grid convergence uses opposite sign convention to UTM.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',123.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12527','conversion','EPSG','17351','EPSG','1559','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','17352','Map Grid of Australia zone 52','Grid convergence uses opposite sign convention to UTM.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',129.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12528','conversion','EPSG','17352','EPSG','1560','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','17353','Map Grid of Australia zone 53','Grid convergence uses opposite sign convention to UTM.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',135.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12529','conversion','EPSG','17353','EPSG','1561','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','17354','Map Grid of Australia zone 54','Grid convergence uses opposite sign convention to UTM.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',141.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12530','conversion','EPSG','17354','EPSG','1562','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','17355','Map Grid of Australia zone 55','Grid convergence uses opposite sign convention to UTM.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',147.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12531','conversion','EPSG','17355','EPSG','1563','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','17356','Map Grid of Australia zone 56','Grid convergence uses opposite sign convention to UTM.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',153.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12532','conversion','EPSG','17356','EPSG','1564','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','17357','Map Grid of Australia zone 57','Grid convergence uses opposite sign convention to UTM.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',159.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12533','conversion','EPSG','17357','EPSG','4196','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','17358','Map Grid of Australia zone 58','Grid convergence uses opposite sign convention to UTM.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',165.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12534','conversion','EPSG','17358','EPSG','4175','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','17359','South Australia Lambert','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-32.0,'EPSG','9102','EPSG','8822','Longitude of false origin',135.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-28.0,'EPSG','9102','EPSG','8824','Latitude of 2nd standard parallel',-36.0,'EPSG','9102','EPSG','8826','Easting at false origin',1000000.0,'EPSG','9001','EPSG','8827','Northing at false origin',2000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12535','conversion','EPSG','17359','EPSG','2986','EPSG','1135');
INSERT INTO "conversion" VALUES('EPSG','17360','Vicgrid66','May also be used to display Victoria, New South Wales, Tasmania and South Australia as a single entity. Replaced by Vicgrid94 with effect from 2nd February 2000. Prior to 2000 this projection was known as VICGRID.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-37.0,'EPSG','9102','EPSG','8822','Longitude of false origin',145.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-36.0,'EPSG','9102','EPSG','8824','Latitude of 2nd standard parallel',-38.0,'EPSG','9102','EPSG','8826','Easting at false origin',2500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',4500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12536','conversion','EPSG','17360','EPSG','2285','EPSG','1135');
INSERT INTO "conversion" VALUES('EPSG','17361','Vicgrid','May also be used to display Victoria, New South Wales, Tasmania and South Australia as a single entity. Replaces Vicgrid66 with effect from 2nd February 2000.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-37.0,'EPSG','9102','EPSG','8822','Longitude of false origin',145.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-36.0,'EPSG','9102','EPSG','8824','Latitude of 2nd standard parallel',-38.0,'EPSG','9102','EPSG','8826','Easting at false origin',2500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',2500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12537','conversion','EPSG','17361','EPSG','2285','EPSG','1135');
INSERT INTO "conversion" VALUES('EPSG','17362','Geoscience Australia Standard National Scale Lambert Projection','Created by Australian Geological Survey Organisation prior to incorporation into Auslig. See also ACRESLC (proj code 4460).','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',0.0,'EPSG','9102','EPSG','8822','Longitude of false origin',134.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-18.0,'EPSG','9102','EPSG','8824','Latitude of 2nd standard parallel',-36.0,'EPSG','9102','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12538','conversion','EPSG','17362','EPSG','2575','EPSG','1236');
INSERT INTO "conversion" VALUES('EPSG','17363','Brisbane City Survey Grid 02','Replaced by BCGS2020 (code 10147) from 2022-07-01.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-28.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',153.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.99999,'EPSG','9201','EPSG','8806','False easting',50000.0,'EPSG','9001','EPSG','8807','False northing',100000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12539','conversion','EPSG','17363','EPSG','2990','EPSG','1029');
INSERT INTO "conversion" VALUES('EPSG','17364','New South Wales Lambert','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-33.25,'EPSG','9102','EPSG','8822','Longitude of false origin',147.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-30.75,'EPSG','9102','EPSG','8824','Latitude of 2nd standard parallel',-35.75,'EPSG','9102','EPSG','8826','Easting at false origin',9300000.0,'EPSG','9001','EPSG','8827','Northing at false origin',4500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12540','conversion','EPSG','17364','EPSG','3139','EPSG','1135');
INSERT INTO "conversion" VALUES('EPSG','17365','Australian Albers','','EPSG','9822','Albers Equal Area','EPSG','8821','Latitude of false origin',0.0,'EPSG','9102','EPSG','8822','Longitude of false origin',132.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-18.0,'EPSG','9102','EPSG','8824','Latitude of 2nd standard parallel',-36.0,'EPSG','9102','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12541','conversion','EPSG','17365','EPSG','2575','EPSG','1162');
INSERT INTO "conversion" VALUES('EPSG','17401','Katanga Lambert Conformal','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',0.0,'EPSG','9102','EPSG','8822','Longitude of false origin',26.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-6.5,'EPSG','9102','EPSG','8824','Latitude of 2nd standard parallel',-11.5,'EPSG','9102','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','12542','conversion','EPSG','17401','EPSG','3147','EPSG','1210');
INSERT INTO "conversion" VALUES('EPSG','17402','Katanga Transverse Mercator','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-9.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',26.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9998,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','12543','conversion','EPSG','17402','EPSG','3147','EPSG','1054');
INSERT INTO "conversion" VALUES('EPSG','17412','Congo Transverse Mercator zone 12','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',12.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12544','conversion','EPSG','17412','EPSG','3937','EPSG','1056');
INSERT INTO "conversion" VALUES('EPSG','17414','Congo Transverse Mercator zone 14','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',14.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12545','conversion','EPSG','17414','EPSG','3151','EPSG','1056');
INSERT INTO "conversion" VALUES('EPSG','17416','Congo Transverse Mercator zone 16','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',16.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12546','conversion','EPSG','17416','EPSG','3152','EPSG','1056');
INSERT INTO "conversion" VALUES('EPSG','17418','Congo Transverse Mercator zone 18','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',18.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12547','conversion','EPSG','17418','EPSG','3153','EPSG','1056');
INSERT INTO "conversion" VALUES('EPSG','17420','Congo Transverse Mercator zone 20','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',20.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12548','conversion','EPSG','17420','EPSG','3154','EPSG','1056');
INSERT INTO "conversion" VALUES('EPSG','17422','Congo Transverse Mercator zone 22','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',22.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12549','conversion','EPSG','17422','EPSG','3155','EPSG','1056');
INSERT INTO "conversion" VALUES('EPSG','17424','Congo Transverse Mercator zone 24','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',24.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12550','conversion','EPSG','17424','EPSG','3156','EPSG','1056');
INSERT INTO "conversion" VALUES('EPSG','17426','Congo Transverse Mercator zone 26','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',26.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12551','conversion','EPSG','17426','EPSG','3157','EPSG','1056');
INSERT INTO "conversion" VALUES('EPSG','17428','Congo Transverse Mercator zone 28','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',28.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12552','conversion','EPSG','17428','EPSG','3158','EPSG','1056');
INSERT INTO "conversion" VALUES('EPSG','17430','Congo Transverse Mercator zone 30','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',30.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12553','conversion','EPSG','17430','EPSG','3159','EPSG','1056');
INSERT INTO "conversion" VALUES('EPSG','17432','Indonesia TM-3 zone 46.2','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',94.5,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',1500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12554','conversion','EPSG','17432','EPSG','3976','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','17433','Indonesia TM-3 zone 47.1','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',97.5,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',1500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12555','conversion','EPSG','17433','EPSG','3510','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','17434','Indonesia TM-3 zone 47.2','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',100.5,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',1500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12556','conversion','EPSG','17434','EPSG','3511','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','17435','Indonesia TM-3 zone 48.1','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',103.5,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',1500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12557','conversion','EPSG','17435','EPSG','3512','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','17436','Indonesia TM-3 zone 48.2','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',106.5,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',1500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12558','conversion','EPSG','17436','EPSG','3513','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','17437','Indonesia TM-3 zone 49.1','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',109.5,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',1500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12559','conversion','EPSG','17437','EPSG','3514','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','17438','Indonesia TM-3 zone 49.2','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',112.5,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',1500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12560','conversion','EPSG','17438','EPSG','3515','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','17439','Indonesia TM-3 zone 50.1','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',115.5,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',1500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12561','conversion','EPSG','17439','EPSG','3516','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','17440','Indonesia TM-3 zone 50.2','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',118.5,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',1500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12562','conversion','EPSG','17440','EPSG','3517','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','17441','Indonesia TM-3 zone 51.1','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',121.5,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',1500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12563','conversion','EPSG','17441','EPSG','3518','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','17442','Indonesia TM-3 zone 51.2','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',124.5,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',1500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12564','conversion','EPSG','17442','EPSG','3519','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','17443','Indonesia TM-3 zone 52.1','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',127.5,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',1500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12565','conversion','EPSG','17443','EPSG','3520','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','17444','Indonesia TM-3 zone 52.2','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',130.5,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',1500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12566','conversion','EPSG','17444','EPSG','3521','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','17445','Indonesia TM-3 zone 53.1','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',133.5,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',1500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12567','conversion','EPSG','17445','EPSG','3522','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','17446','Indonesia TM-3 zone 53.2','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',136.5,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',1500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12568','conversion','EPSG','17446','EPSG','3523','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','17447','Indonesia TM-3 zone 54.1','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',139.5,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',1500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12569','conversion','EPSG','17447','EPSG','3975','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','17448','Australian Map Grid zone 48','Grid convergence uses opposite sign convention to UTM','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',105.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','12570','conversion','EPSG','17448','EPSG','1556','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','17449','Australian Map Grid zone 49','Grid convergence uses opposite sign convention to UTM','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',111.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12571','conversion','EPSG','17449','EPSG','1557','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','17450','Australian Map Grid zone 50','Grid convergence uses opposite sign convention to UTM','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',117.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12572','conversion','EPSG','17450','EPSG','1558','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','17451','Australian Map Grid zone 51','Grid convergence uses opposite sign convention to UTM','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',123.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12573','conversion','EPSG','17451','EPSG','1559','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','17452','Australian Map Grid zone 52','Grid convergence uses opposite sign convention to UTM','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',129.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12574','conversion','EPSG','17452','EPSG','1560','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','17453','Australian Map Grid zone 53','Grid convergence uses opposite sign convention to UTM','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',135.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12575','conversion','EPSG','17453','EPSG','1561','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','17454','Australian Map Grid zone 54','Grid convergence uses opposite sign convention to UTM','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',141.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12576','conversion','EPSG','17454','EPSG','1567','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','17455','Australian Map Grid zone 55','Grid convergence uses opposite sign convention to UTM','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',147.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12577','conversion','EPSG','17455','EPSG','1568','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','17456','Australian Map Grid zone 56','Grid convergence uses opposite sign convention to UTM','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',153.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12578','conversion','EPSG','17456','EPSG','2291','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','17457','Australian Map Grid zone 57','Grid convergence uses opposite sign convention to UTM','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',159.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12579','conversion','EPSG','17457','EPSG','1565','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','17458','Australian Map Grid zone 58','Grid convergence uses opposite sign convention to UTM','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',165.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12580','conversion','EPSG','17458','EPSG','1566','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','17515','South African Survey Grid zone 15','Transformation method sometimes described as "Gauss conform".','EPSG','9808','Transverse Mercator (South Orientated)','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',15.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12581','conversion','EPSG','17515','EPSG','1454','EPSG','1092');
INSERT INTO "conversion" VALUES('EPSG','17517','South African Survey Grid zone 17','Transformation method sometimes described as "Gauss conform".','EPSG','9808','Transverse Mercator (South Orientated)','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',17.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12582','conversion','EPSG','17517','EPSG','1455','EPSG','1092');
INSERT INTO "conversion" VALUES('EPSG','17519','South African Survey Grid zone 19','Transformation method sometimes described as "Gauss conform".','EPSG','9808','Transverse Mercator (South Orientated)','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',19.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12583','conversion','EPSG','17519','EPSG','1456','EPSG','1092');
INSERT INTO "conversion" VALUES('EPSG','17521','South African Survey Grid zone 21','Transformation method sometimes described as "Gauss conform".','EPSG','9808','Transverse Mercator (South Orientated)','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',21.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12584','conversion','EPSG','17521','EPSG','1457','EPSG','1092');
INSERT INTO "conversion" VALUES('EPSG','17523','South African Survey Grid zone 23','Transformation method sometimes described as "Gauss conform".','EPSG','9808','Transverse Mercator (South Orientated)','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',23.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12585','conversion','EPSG','17523','EPSG','1458','EPSG','1092');
INSERT INTO "conversion" VALUES('EPSG','17525','South African Survey Grid zone 25','Transformation method sometimes described as "Gauss conform".','EPSG','9808','Transverse Mercator (South Orientated)','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',25.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12586','conversion','EPSG','17525','EPSG','1459','EPSG','1092');
INSERT INTO "conversion" VALUES('EPSG','17527','South African Survey Grid zone 27','Transformation method sometimes described as "Gauss conform".','EPSG','9808','Transverse Mercator (South Orientated)','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',27.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12587','conversion','EPSG','17527','EPSG','1460','EPSG','1092');
INSERT INTO "conversion" VALUES('EPSG','17529','South African Survey Grid zone 29','Transformation method sometimes described as "Gauss conform".','EPSG','9808','Transverse Mercator (South Orientated)','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',29.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12588','conversion','EPSG','17529','EPSG','1461','EPSG','1092');
INSERT INTO "conversion" VALUES('EPSG','17531','South African Survey Grid zone 31','Transformation method sometimes described as "Gauss conform".','EPSG','9808','Transverse Mercator (South Orientated)','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',31.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12589','conversion','EPSG','17531','EPSG','1462','EPSG','1092');
INSERT INTO "conversion" VALUES('EPSG','17533','South African Survey Grid zone 33','Transformation method sometimes described as "Gauss conform".','EPSG','9808','Transverse Mercator (South Orientated)','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',33.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12590','conversion','EPSG','17533','EPSG','1463','EPSG','1092');
INSERT INTO "conversion" VALUES('EPSG','17611','South West African Survey Grid zone 11','Transformation method sometimes described as "Gauss conform".','EPSG','9808','Transverse Mercator (South Orientated)','EPSG','8801','Latitude of natural origin',-22.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',11.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9031','EPSG','8807','False northing',0.0,'EPSG','9031',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12591','conversion','EPSG','17611','EPSG','1838','EPSG','1092');
INSERT INTO "conversion" VALUES('EPSG','17613','South West African Survey Grid zone 13','Transformation method sometimes described as "Gauss conform".','EPSG','9808','Transverse Mercator (South Orientated)','EPSG','8801','Latitude of natural origin',-22.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',13.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9031','EPSG','8807','False northing',0.0,'EPSG','9031',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12592','conversion','EPSG','17613','EPSG','1839','EPSG','1092');
INSERT INTO "conversion" VALUES('EPSG','17615','South West African Survey Grid zone 15','Transformation method sometimes described as "Gauss conform".','EPSG','9808','Transverse Mercator (South Orientated)','EPSG','8801','Latitude of natural origin',-22.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',15.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9031','EPSG','8807','False northing',0.0,'EPSG','9031',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12593','conversion','EPSG','17615','EPSG','1840','EPSG','1092');
INSERT INTO "conversion" VALUES('EPSG','17617','South West African Survey Grid zone 17','Transformation method sometimes described as "Gauss conform".','EPSG','9808','Transverse Mercator (South Orientated)','EPSG','8801','Latitude of natural origin',-22.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',17.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9031','EPSG','8807','False northing',0.0,'EPSG','9031',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12594','conversion','EPSG','17617','EPSG','1841','EPSG','1092');
INSERT INTO "conversion" VALUES('EPSG','17619','South West African Survey Grid zone 19','Transformation method sometimes described as "Gauss conform".','EPSG','9808','Transverse Mercator (South Orientated)','EPSG','8801','Latitude of natural origin',-22.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',19.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9031','EPSG','8807','False northing',0.0,'EPSG','9031',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12595','conversion','EPSG','17619','EPSG','1842','EPSG','1092');
INSERT INTO "conversion" VALUES('EPSG','17621','South West African Survey Grid zone 21','Transformation method sometimes described as "Gauss conform".','EPSG','9808','Transverse Mercator (South Orientated)','EPSG','8801','Latitude of natural origin',-22.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',21.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9031','EPSG','8807','False northing',0.0,'EPSG','9031',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12596','conversion','EPSG','17621','EPSG','1843','EPSG','1092');
INSERT INTO "conversion" VALUES('EPSG','17623','South West African Survey Grid zone 23','Transformation method sometimes described as "Gauss conform".','EPSG','9808','Transverse Mercator (South Orientated)','EPSG','8801','Latitude of natural origin',-22.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',23.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9031','EPSG','8807','False northing',0.0,'EPSG','9031',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12597','conversion','EPSG','17623','EPSG','1844','EPSG','1092');
INSERT INTO "conversion" VALUES('EPSG','17625','South West African Survey Grid zone 25','Transformation method sometimes described as "Gauss conform".','EPSG','9808','Transverse Mercator (South Orientated)','EPSG','8801','Latitude of natural origin',-22.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',25.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9031','EPSG','8807','False northing',0.0,'EPSG','9031',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12598','conversion','EPSG','17625','EPSG','1845','EPSG','1092');
INSERT INTO "conversion" VALUES('EPSG','17700','MTM Quebec zone 2','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-55.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',304800.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','12599','conversion','EPSG','17700','EPSG','1420','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','17701','MTM zone 1','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-53.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',304800.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12600','conversion','EPSG','17701','EPSG','2226','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','17702','MTM zone 2','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-56.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',304800.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12601','conversion','EPSG','17702','EPSG','2227','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','17703','MTM zone 3','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-58.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',304800.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12602','conversion','EPSG','17703','EPSG','2290','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','17704','MTM zone 4','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-61.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',304800.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12603','conversion','EPSG','17704','EPSG','2276','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','17705','MTM zone 5','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-64.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',304800.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12604','conversion','EPSG','17705','EPSG','2277','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','17706','MTM zone 6','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-67.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',304800.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12605','conversion','EPSG','17706','EPSG','2278','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','17707','MTM zone 7','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-70.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',304800.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12606','conversion','EPSG','17707','EPSG','1425','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','17708','MTM zone 8','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-73.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',304800.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12607','conversion','EPSG','17708','EPSG','2279','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','17709','MTM zone 9','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-76.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',304800.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12608','conversion','EPSG','17709','EPSG','2280','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','17710','MTM zone 10','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-79.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',304800.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12609','conversion','EPSG','17710','EPSG','2281','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','17711','MTM zone 11','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-82.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',304800.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12610','conversion','EPSG','17711','EPSG','1432','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','17712','MTM zone 12','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-81.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',304800.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12611','conversion','EPSG','17712','EPSG','1433','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','17713','MTM zone 13','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-84.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',304800.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12612','conversion','EPSG','17713','EPSG','1434','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','17714','MTM zone 14','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-87.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',304800.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12613','conversion','EPSG','17714','EPSG','1435','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','17715','MTM zone 15','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-90.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',304800.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12614','conversion','EPSG','17715','EPSG','1436','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','17716','MTM zone 16','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-93.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',304800.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12615','conversion','EPSG','17716','EPSG','1437','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','17717','MTM zone 17','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-96.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',304800.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12616','conversion','EPSG','17717','EPSG','1438','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','17722','Alberta 3-degree TM reference meridian 111 W','If used for rural area control markers, area of use is amended to east of 112°W; however use of UTM encouraged in these areas.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-111.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12617','conversion','EPSG','17722','EPSG','3543','EPSG','1096');
INSERT INTO "conversion" VALUES('EPSG','17723','Alberta 3-degree TM reference meridian 114 W','If used for rural area control markers, area of use is amended to between 112° and 116°W; however use of UTM encouraged in these areas.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-114.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12618','conversion','EPSG','17723','EPSG','3542','EPSG','1096');
INSERT INTO "conversion" VALUES('EPSG','17724','Alberta 3-degree TM reference meridian 117 W','If used for rural area control markers, area of use is amended to between 116° and 118° W; however use of UTM encouraged in these areas.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-117.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12619','conversion','EPSG','17724','EPSG','3541','EPSG','1096');
INSERT INTO "conversion" VALUES('EPSG','17725','Alberta 3-degree TM reference meridian 120 W','If used for rural area control markers, area of use is amended to west of 118 deg W; however use of UTM encouraged in these areas.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-120.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9001','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','12620','conversion','EPSG','17725','EPSG','3540','EPSG','1096');
INSERT INTO "conversion" VALUES('EPSG','17726','Alberta 3-degree TM reference meridian 120 W','If used for rural area control markers, area of use is amended to west of 118°W; however use of UTM encouraged in these areas.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-120.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12621','conversion','EPSG','17726','EPSG','3540','EPSG','1096');
INSERT INTO "conversion" VALUES('EPSG','17771','Equi7 projection - Africa','Basis for the Equi7 grid for Africa. Designed in 2014 to efficiently handle the archiving, processing, and displaying of high resolution raster data over land while preserving geometric accuracy. The projection is neither conformal nor equal-area.','EPSG','1125','Azimuthal Equidistant','EPSG','8801','Latitude of natural origin',8.5,'EPSG','9102','EPSG','8802','Longitude of natural origin',21.5,'EPSG','9102','EPSG','8806','False easting',5621452.02,'EPSG','9001','EPSG','8807','False northing',5990638.423,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','20872','conversion','EPSG','17771','EPSG','4745','EPSG','1287');
INSERT INTO "conversion" VALUES('EPSG','17772','Equi7 projection - Antarctica','Basis for the Equi7 grid for Antarctica. Designed in 2014 to efficiently handle the archiving, processing, and displaying of high resolution raster data over land while preserving geometric accuracy. The projection is neither conformal nor equal-area.','EPSG','1125','Azimuthal Equidistant','EPSG','8801','Latitude of natural origin',-90.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',0.0,'EPSG','9102','EPSG','8806','False easting',3714266.977,'EPSG','9001','EPSG','8807','False northing',3402016.506,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','20873','conversion','EPSG','17772','EPSG','4746','EPSG','1287');
INSERT INTO "conversion" VALUES('EPSG','17773','Equi7 projection - Asia','Basis for the Equi7 grid for Asia. Designed in 2014 to efficiently handle the archiving, processing, and displaying of high resolution raster data over land while preserving geometric accuracy. The projection is neither conformal nor equal-area.','EPSG','1125','Azimuthal Equidistant','EPSG','8801','Latitude of natural origin',47.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',94.0,'EPSG','9102','EPSG','8806','False easting',4340913.848,'EPSG','9001','EPSG','8807','False northing',4812712.923,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','20874','conversion','EPSG','17773','EPSG','4747','EPSG','1287');
INSERT INTO "conversion" VALUES('EPSG','17774','Equi7 projection - Europe','Basis for the Equi7 grid for Europe. Designed in 2014 to efficiently handle the archiving, processing, and displaying of high resolution raster data over land while preserving geometric accuracy. The projection is neither conformal nor equal-area.','EPSG','1125','Azimuthal Equidistant','EPSG','8801','Latitude of natural origin',53.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',24.0,'EPSG','9102','EPSG','8806','False easting',5837287.82,'EPSG','9001','EPSG','8807','False northing',2121415.696,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','20875','conversion','EPSG','17774','EPSG','4748','EPSG','1287');
INSERT INTO "conversion" VALUES('EPSG','17775','Equi7 projection - North America','Basis for the Equi7 grid for North America. Designed in 2014 to efficiently handle the archiving, processing, and displaying of high resolution raster data over land while preserving geometric accuracy. The projection is neither conformal nor equal-area.','EPSG','1125','Azimuthal Equidistant','EPSG','8801','Latitude of natural origin',52.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-97.5,'EPSG','9102','EPSG','8806','False easting',8264722.177,'EPSG','9001','EPSG','8807','False northing',4867518.353,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','20876','conversion','EPSG','17775','EPSG','4749','EPSG','1287');
INSERT INTO "conversion" VALUES('EPSG','17776','Equi7 projection - Oceania','Basis for the Equi7 grid for Oceania. Designed in 2014 to efficiently handle the archiving, processing, and displaying of high resolution raster data over land while preserving geometric accuracy. The projection is neither conformal nor equal-area.','EPSG','1125','Azimuthal Equidistant','EPSG','8801','Latitude of natural origin',-19.5,'EPSG','9102','EPSG','8802','Longitude of natural origin',131.5,'EPSG','9102','EPSG','8806','False easting',6988408.536,'EPSG','9001','EPSG','8807','False northing',7654884.537,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','20877','conversion','EPSG','17776','EPSG','4751','EPSG','1287');
INSERT INTO "conversion" VALUES('EPSG','17777','Equi7 projection - South America','Basis for the Equi7 grid for South America. Designed in 2014 to efficiently handle the archiving, processing, and displaying of high resolution raster data over land while preserving geometric accuracy. The projection is neither conformal nor equal-area.','EPSG','1125','Azimuthal Equidistant','EPSG','8801','Latitude of natural origin',-14.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-60.5,'EPSG','9102','EPSG','8806','False easting',7257179.236,'EPSG','9001','EPSG','8807','False northing',5592024.446,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','20878','conversion','EPSG','17777','EPSG','4750','EPSG','1287');
INSERT INTO "conversion" VALUES('EPSG','17794','MTM Nova Scotia zone 4','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-61.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',4500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12622','conversion','EPSG','17794','EPSG','1534','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','17795','MTM Nova Scotia zone 5','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-64.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',5500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12623','conversion','EPSG','17795','EPSG','1535','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','17801','Japan Plane Rectangular CS zone I','Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',33.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',129.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12624','conversion','EPSG','17801','EPSG','1854','EPSG','1092');
INSERT INTO "conversion" VALUES('EPSG','17802','Japan Plane Rectangular CS zone II','Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',33.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',131.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12625','conversion','EPSG','17802','EPSG','1855','EPSG','1092');
INSERT INTO "conversion" VALUES('EPSG','17803','Japan Plane Rectangular CS zone III','Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',36.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',132.1,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12626','conversion','EPSG','17803','EPSG','1856','EPSG','1092');
INSERT INTO "conversion" VALUES('EPSG','17804','Japan Plane Rectangular CS zone IV','Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',33.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',133.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12627','conversion','EPSG','17804','EPSG','1857','EPSG','1092');
INSERT INTO "conversion" VALUES('EPSG','17805','Japan Plane Rectangular CS zone V','Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',36.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',134.2,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12628','conversion','EPSG','17805','EPSG','1858','EPSG','1092');
INSERT INTO "conversion" VALUES('EPSG','17806','Japan Plane Rectangular CS zone VI','Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',36.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',136.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12629','conversion','EPSG','17806','EPSG','1859','EPSG','1092');
INSERT INTO "conversion" VALUES('EPSG','17807','Japan Plane Rectangular CS zone VII','Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',36.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',137.1,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12630','conversion','EPSG','17807','EPSG','1860','EPSG','1092');
INSERT INTO "conversion" VALUES('EPSG','17808','Japan Plane Rectangular CS zone VIII','Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',36.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',138.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12631','conversion','EPSG','17808','EPSG','1861','EPSG','1092');
INSERT INTO "conversion" VALUES('EPSG','17809','Japan Plane Rectangular CS zone IX','Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',36.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',139.5,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12632','conversion','EPSG','17809','EPSG','1862','EPSG','1092');
INSERT INTO "conversion" VALUES('EPSG','17810','Japan Plane Rectangular CS zone X','Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',40.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',140.5,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12633','conversion','EPSG','17810','EPSG','1863','EPSG','1092');
INSERT INTO "conversion" VALUES('EPSG','17811','Japan Plane Rectangular CS zone XI','Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',44.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',140.15,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12634','conversion','EPSG','17811','EPSG','1864','EPSG','1092');
INSERT INTO "conversion" VALUES('EPSG','17812','Japan Plane Rectangular CS zone XII','Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',44.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',142.15,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12635','conversion','EPSG','17812','EPSG','1865','EPSG','1092');
INSERT INTO "conversion" VALUES('EPSG','17813','Japan Plane Rectangular CS zone XIII','Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',44.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',144.15,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12636','conversion','EPSG','17813','EPSG','1866','EPSG','1092');
INSERT INTO "conversion" VALUES('EPSG','17814','Japan Plane Rectangular CS zone XIV','Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',26.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',142.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12637','conversion','EPSG','17814','EPSG','1867','EPSG','1092');
INSERT INTO "conversion" VALUES('EPSG','17815','Japan Plane Rectangular CS zone XV','Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',26.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',127.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12638','conversion','EPSG','17815','EPSG','1868','EPSG','1092');
INSERT INTO "conversion" VALUES('EPSG','17816','Japan Plane Rectangular CS zone XVI','Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',26.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',124.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12639','conversion','EPSG','17816','EPSG','1869','EPSG','1092');
INSERT INTO "conversion" VALUES('EPSG','17817','Japan Plane Rectangular CS zone XVII','Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',26.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',131.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12640','conversion','EPSG','17817','EPSG','1870','EPSG','1092');
INSERT INTO "conversion" VALUES('EPSG','17818','Japan Plane Rectangular CS zone XVIII','Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',20.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',136.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12641','conversion','EPSG','17818','EPSG','1871','EPSG','1092');
INSERT INTO "conversion" VALUES('EPSG','17819','Japan Plane Rectangular CS zone XIX','Original transformation by Gauss-Kruger formula.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',26.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',154.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12642','conversion','EPSG','17819','EPSG','1872','EPSG','1092');
INSERT INTO "conversion" VALUES('EPSG','17901','Mount Eden Circuit','Replaced Imperial measure circuit in 1972. Replaced by Mount Eden 2000 (code 17931) from March 2000.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-36.5247515,'EPSG','9110','EPSG','8802','Longitude of natural origin',174.45516217,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',300000.0,'EPSG','9001','EPSG','8807','False northing',700000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12643','conversion','EPSG','17901','EPSG','3781','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','17902','Bay of Plenty Circuit','Replaced Imperial measure circuit in 1972. Replaced by Bay of Plenty 2000 (code 17932) from March 2000.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-37.45404993,'EPSG','9110','EPSG','8802','Longitude of natural origin',176.27583101,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',300000.0,'EPSG','9001','EPSG','8807','False northing',700000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12644','conversion','EPSG','17902','EPSG','3779','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','17903','Poverty Bay Circuit','Replaced Imperial measure circuit in 1972. Replaced by Poverty Bay 2000 (code 17933) from March 2000.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-38.372893,'EPSG','9110','EPSG','8802','Longitude of natural origin',177.53082906,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',300000.0,'EPSG','9001','EPSG','8807','False northing',700000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12645','conversion','EPSG','17903','EPSG','3780','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','17904','Hawkes Bay Circuit','Replaced Imperial measure circuit in 1972. Replaced by Hawkes Bay 2000 (code 17934) from March 2000.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-39.39033455,'EPSG','9110','EPSG','8802','Longitude of natural origin',176.40252499,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',300000.0,'EPSG','9001','EPSG','8807','False northing',700000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12646','conversion','EPSG','17904','EPSG','3772','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','17905','Taranaki Circuit','Replaced Imperial measure circuit in 1972. Replaced by Taranaki 2000 (code 17935) from March 2000.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-39.08087299,'EPSG','9110','EPSG','8802','Longitude of natural origin',174.13408423,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',300000.0,'EPSG','9001','EPSG','8807','False northing',700000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12647','conversion','EPSG','17905','EPSG','3777','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','17906','Tuhirangi Circuit','Replaced Imperial measure circuit in 1972. Replaced by Tuhirangi 2000 (code 17936) from March 2000.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-39.30448934,'EPSG','9110','EPSG','8802','Longitude of natural origin',175.38241325,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',300000.0,'EPSG','9001','EPSG','8807','False northing',700000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12648','conversion','EPSG','17906','EPSG','3778','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','17907','Wanganui Circuit','Replaced Imperial measure circuit in 1972. Replaced by Wanganui 2000 (code 17937) from March 2000.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-40.14310097,'EPSG','9110','EPSG','8802','Longitude of natural origin',175.29171586,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',300000.0,'EPSG','9001','EPSG','8807','False northing',700000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12649','conversion','EPSG','17907','EPSG','3776','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','17908','Wairarapa Circuit','Replaced Imperial measure circuit in 1972. Replaced by Wairarapa 2000 (code 17938) from March 2000.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-40.55319175,'EPSG','9110','EPSG','8802','Longitude of natural origin',175.38504588,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',300000.0,'EPSG','9001','EPSG','8807','False northing',700000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12650','conversion','EPSG','17908','EPSG','3775','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','17909','Wellington Circuit','Replaced Imperial measure circuit in 1972. Replaced by Wellington 2000 (code 17939) from March 2000.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-41.18047507,'EPSG','9110','EPSG','8802','Longitude of natural origin',174.46358432,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',300000.0,'EPSG','9001','EPSG','8807','False northing',700000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12651','conversion','EPSG','17909','EPSG','3774','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','17910','Collingwood Circuit','Replaced Imperial measure circuit in 1972. Replaced by Collingwood 2000 (code 17940) from March 2000.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-40.42531326,'EPSG','9110','EPSG','8802','Longitude of natural origin',172.40193674,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',300000.0,'EPSG','9001','EPSG','8807','False northing',700000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12652','conversion','EPSG','17910','EPSG','3782','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','17911','Nelson Circuit','Replaced Imperial measure circuit in 1972. Replaced by Nelson 2000 (code 17941) from March 2000.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-41.1628361,'EPSG','9110','EPSG','8802','Longitude of natural origin',173.17575405,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',300000.0,'EPSG','9001','EPSG','8807','False northing',700000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12653','conversion','EPSG','17911','EPSG','3784','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','17912','Karamea Circuit','Replaced Imperial measure circuit in 1972. Replaced by Karamea 2000 (code 17942) from March 2000.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-41.17236815,'EPSG','9110','EPSG','8802','Longitude of natural origin',172.06325015,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',300000.0,'EPSG','9001','EPSG','8807','False northing',700000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12654','conversion','EPSG','17912','EPSG','3783','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','17913','Buller Circuit','Replaced Imperial measure circuit in 1972. Replaced by Buller 2000 (code 17943) from March 2000.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-41.48388903,'EPSG','9110','EPSG','8802','Longitude of natural origin',171.34525362,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',300000.0,'EPSG','9001','EPSG','8807','False northing',700000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12655','conversion','EPSG','17913','EPSG','3786','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','17914','Grey Circuit','Replaced Imperial measure circuit in 1972. Replaced by Grey 2000 (code 17944) from March 2000.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-42.20012994,'EPSG','9110','EPSG','8802','Longitude of natural origin',171.32591767,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',300000.0,'EPSG','9001','EPSG','8807','False northing',700000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12656','conversion','EPSG','17914','EPSG','3787','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','17915','Amuri Circuit','Replaced Imperial measure circuit in 1972. Replaced by Amuri 2000 (code 17945) from March 2000.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-42.41208197,'EPSG','9110','EPSG','8802','Longitude of natural origin',173.00364802,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',300000.0,'EPSG','9001','EPSG','8807','False northing',700000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12657','conversion','EPSG','17915','EPSG','3788','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','17916','Marlborough Circuit','Replaced Imperial measure circuit in 1972. Replaced by Marlborough 2000 (code 17946) from March 2000.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-41.3240152,'EPSG','9110','EPSG','8802','Longitude of natural origin',173.48074668,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',300000.0,'EPSG','9001','EPSG','8807','False northing',700000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12658','conversion','EPSG','17916','EPSG','3785','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','17917','Hokitika Circuit','Replaced Imperial measure circuit in 1972. Replaced by Hokitika 2000 (code 17947) from March 2000.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-42.53107605,'EPSG','9110','EPSG','8802','Longitude of natural origin',170.58479766,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',300000.0,'EPSG','9001','EPSG','8807','False northing',700000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12659','conversion','EPSG','17917','EPSG','3789','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','17918','Okarito Circuit','Replaced Imperial measure circuit in 1972. Replaced by Okarito 2000 (code 17948) from March 2000.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-43.06364613,'EPSG','9110','EPSG','8802','Longitude of natural origin',170.1539333,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',300000.0,'EPSG','9001','EPSG','8807','False northing',700000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12660','conversion','EPSG','17918','EPSG','3791','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','17919','Jacksons Bay Circuit','Replaced Imperial measure circuit in 1972. Replaced by Jacksons Bay 2000 (code 17949) from March 2000.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-43.58400904,'EPSG','9110','EPSG','8802','Longitude of natural origin',168.36225612,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',300000.0,'EPSG','9001','EPSG','8807','False northing',700000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12661','conversion','EPSG','17919','EPSG','3794','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','17920','Mount Pleasant Circuit','Replaced Imperial measure circuit in 1972. Replaced by Mount Pleasant 2000 (code 17950) from March 2000.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-43.35262953,'EPSG','9110','EPSG','8802','Longitude of natural origin',172.43378969,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',300000.0,'EPSG','9001','EPSG','8807','False northing',700000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12662','conversion','EPSG','17920','EPSG','3790','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','17921','Gawler Circuit','Replaced Imperial measure circuit in 1972. Replaced by Gawler 2000 (code 17951) from March 2000.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-43.44553616,'EPSG','9110','EPSG','8802','Longitude of natural origin',171.21386945,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',300000.0,'EPSG','9001','EPSG','8807','False northing',700000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12663','conversion','EPSG','17921','EPSG','3792','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','17922','Timaru Circuit','Replaced Imperial measure circuit in 1972. Replaced by Timaru 2000 (code 17952) from March 2000.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-44.24079933,'EPSG','9110','EPSG','8802','Longitude of natural origin',171.0326103,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',300000.0,'EPSG','9001','EPSG','8807','False northing',700000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12664','conversion','EPSG','17922','EPSG','3793','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','17923','Lindis Peak Circuit','Replaced Imperial measure circuit in 1972. Replaced by Lindis Peak 2000 (code 17953) from March 2000.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-44.44069647,'EPSG','9110','EPSG','8802','Longitude of natural origin',169.28039183,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',300000.0,'EPSG','9001','EPSG','8807','False northing',700000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12665','conversion','EPSG','17923','EPSG','3795','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','17924','Mount Nicholas Circuit','Replaced Imperial measure circuit in 1972. Replaced by Mount Nicholas 2000 (code 17954) from March 2000.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-45.07584493,'EPSG','9110','EPSG','8802','Longitude of natural origin',168.23551083,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',300000.0,'EPSG','9001','EPSG','8807','False northing',700000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12666','conversion','EPSG','17924','EPSG','3797','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','17925','Mount York Circuit','Replaced Imperial measure circuit in 1972. Replaced by Mount York 2000 (code 17955) from March 2000.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-45.33494142,'EPSG','9110','EPSG','8802','Longitude of natural origin',167.44199024,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',300000.0,'EPSG','9001','EPSG','8807','False northing',700000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12667','conversion','EPSG','17925','EPSG','3799','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','17926','Observation Point Circuit','Replaced Imperial measure circuit in 1972. Replaced by Observation Point 2000 (code 17956) from March 2000.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-45.48583078,'EPSG','9110','EPSG','8802','Longitude of natural origin',170.37429426,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',300000.0,'EPSG','9001','EPSG','8807','False northing',700000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12668','conversion','EPSG','17926','EPSG','3796','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','17927','North Taieri Circuit','Replaced Imperial measure circuit in 1972. Replaced by North Taieri 2000 (code 17957) from March 2000.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-45.51414481,'EPSG','9110','EPSG','8802','Longitude of natural origin',170.16573208,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.99996,'EPSG','9201','EPSG','8806','False easting',300000.0,'EPSG','9001','EPSG','8807','False northing',700000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12669','conversion','EPSG','17927','EPSG','3798','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','17928','Bluff Circuit','Replaced Imperial measure circuit in 1972. Replaced by Bluff 2000 (code 17958) from March 2000.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-46.36000346,'EPSG','9110','EPSG','8802','Longitude of natural origin',168.20343392,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',300002.66,'EPSG','9001','EPSG','8807','False northing',699999.58,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12670','conversion','EPSG','17928','EPSG','3800','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','17931','Mount Eden 2000','Replaces Mount Eden Circuit (code 17901) after 1st March 2000.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-36.5247,'EPSG','9110','EPSG','8802','Longitude of natural origin',174.4551,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',400000.0,'EPSG','9001','EPSG','8807','False northing',800000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12671','conversion','EPSG','17931','EPSG','3781','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','17932','Bay of Plenty 2000','Replaces Bay of Plenty Circuit (code 17902) after 1st March 2000.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-37.454,'EPSG','9110','EPSG','8802','Longitude of natural origin',176.2758,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',400000.0,'EPSG','9001','EPSG','8807','False northing',800000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12672','conversion','EPSG','17932','EPSG','3779','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','17933','Poverty Bay 2000','Replaces Poverty Bay Circuit (code 17903) after 1st March 2000.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-38.3728,'EPSG','9110','EPSG','8802','Longitude of natural origin',177.5308,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',400000.0,'EPSG','9001','EPSG','8807','False northing',800000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12673','conversion','EPSG','17933','EPSG','3780','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','17934','Hawkes Bay 2000','Replaces Hawkes Bay Circuit (code 17904) after 1st March 2000.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-39.3903,'EPSG','9110','EPSG','8802','Longitude of natural origin',176.4025,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',400000.0,'EPSG','9001','EPSG','8807','False northing',800000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12674','conversion','EPSG','17934','EPSG','3772','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','17935','Taranaki 2000','Replaces Taranaki Circuit (code 17905) after 1st March 2000.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-39.0808,'EPSG','9110','EPSG','8802','Longitude of natural origin',174.134,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',400000.0,'EPSG','9001','EPSG','8807','False northing',800000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12675','conversion','EPSG','17935','EPSG','3777','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','17936','Tuhirangi 2000','Replaces Tuhirangi Circuit (code 17906) after 1st March 2000.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-39.3044,'EPSG','9110','EPSG','8802','Longitude of natural origin',175.3824,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',400000.0,'EPSG','9001','EPSG','8807','False northing',800000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12676','conversion','EPSG','17936','EPSG','3778','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','17937','Wanganui 2000','Replaces Wanganui Circuit (code 17907) after 1st March 2000.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-40.1431,'EPSG','9110','EPSG','8802','Longitude of natural origin',175.2917,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',400000.0,'EPSG','9001','EPSG','8807','False northing',800000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12677','conversion','EPSG','17937','EPSG','3776','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','17938','Wairarapa 2000','Replaces Wairarapa Circuit (code 17908) after 1st March 2000.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-40.5531,'EPSG','9110','EPSG','8802','Longitude of natural origin',175.385,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',400000.0,'EPSG','9001','EPSG','8807','False northing',800000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12678','conversion','EPSG','17938','EPSG','3775','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','17939','Wellington 2000','Replaces Wellington Circuit (code 17909) after 1st March 2000.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-41.1804,'EPSG','9110','EPSG','8802','Longitude of natural origin',174.4635,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',400000.0,'EPSG','9001','EPSG','8807','False northing',800000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12679','conversion','EPSG','17939','EPSG','3774','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','17940','Collingwood 2000','Replaces Collingwood Circuit (code 17910) after 1st March 2000.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-40.4253,'EPSG','9110','EPSG','8802','Longitude of natural origin',172.4019,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',400000.0,'EPSG','9001','EPSG','8807','False northing',800000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12680','conversion','EPSG','17940','EPSG','3782','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','17941','Nelson 2000','Replaces Nelson Circuit (code 17911) after 1st March 2000.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-41.1628,'EPSG','9110','EPSG','8802','Longitude of natural origin',173.1757,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',400000.0,'EPSG','9001','EPSG','8807','False northing',800000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12681','conversion','EPSG','17941','EPSG','3784','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','17942','Karamea 2000','Replaces Karamea Circuit (code 17912) after 1st March 2000.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-41.1723,'EPSG','9110','EPSG','8802','Longitude of natural origin',172.0632,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',400000.0,'EPSG','9001','EPSG','8807','False northing',800000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12682','conversion','EPSG','17942','EPSG','3783','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','17943','Buller 2000','Replaces Buller Circuit (code 17913) after 1st March 2000.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-41.4838,'EPSG','9110','EPSG','8802','Longitude of natural origin',171.3452,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',400000.0,'EPSG','9001','EPSG','8807','False northing',800000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12683','conversion','EPSG','17943','EPSG','3786','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','17944','Grey 2000','Replaces Grey Circuit (code 17914) after 1st March 2000.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-42.2001,'EPSG','9110','EPSG','8802','Longitude of natural origin',171.3259,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',400000.0,'EPSG','9001','EPSG','8807','False northing',800000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12684','conversion','EPSG','17944','EPSG','3787','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','17945','Amuri 2000','Replaces Amuri Circuit (code 17915) after 1st March 2000.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-42.412,'EPSG','9110','EPSG','8802','Longitude of natural origin',173.0036,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',400000.0,'EPSG','9001','EPSG','8807','False northing',800000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12685','conversion','EPSG','17945','EPSG','3788','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','17946','Marlborough 2000','Replaces Marlborough Circuit (code 17916) after 1st March 2000.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-41.324,'EPSG','9110','EPSG','8802','Longitude of natural origin',173.4807,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',400000.0,'EPSG','9001','EPSG','8807','False northing',800000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12686','conversion','EPSG','17946','EPSG','3785','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','17947','Hokitika 2000','Replaces Hokitika Circuit (code 17917) after 1st March 2000.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-42.531,'EPSG','9110','EPSG','8802','Longitude of natural origin',170.5847,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',400000.0,'EPSG','9001','EPSG','8807','False northing',800000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12687','conversion','EPSG','17947','EPSG','3789','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','17948','Okarito 2000','Replaces Okarito Circuit (code 17918) after 1st March 2000.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-43.0636,'EPSG','9110','EPSG','8802','Longitude of natural origin',170.1539,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',400000.0,'EPSG','9001','EPSG','8807','False northing',800000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12688','conversion','EPSG','17948','EPSG','3791','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','17949','Jacksons Bay 2000','Replaces Jacksons Bay Circuit (code 17919) after 1st March 2000.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-43.584,'EPSG','9110','EPSG','8802','Longitude of natural origin',168.3622,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',400000.0,'EPSG','9001','EPSG','8807','False northing',800000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12689','conversion','EPSG','17949','EPSG','3794','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','17950','Mount Pleasant 2000','Replaces Mount Pleasant Circuit (code 17920) after 1st March 2000.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-43.3526,'EPSG','9110','EPSG','8802','Longitude of natural origin',172.4337,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',400000.0,'EPSG','9001','EPSG','8807','False northing',800000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12690','conversion','EPSG','17950','EPSG','3790','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','17951','Gawler 2000','Replaces Gawler Circuit (code 17921) after 1st March 2000.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-43.4455,'EPSG','9110','EPSG','8802','Longitude of natural origin',171.2138,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',400000.0,'EPSG','9001','EPSG','8807','False northing',800000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12691','conversion','EPSG','17951','EPSG','3792','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','17952','Timaru 2000','Replaces Timaru Circuit (code 17922) after 1st March 2000.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-44.2407,'EPSG','9110','EPSG','8802','Longitude of natural origin',171.0326,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',400000.0,'EPSG','9001','EPSG','8807','False northing',800000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12692','conversion','EPSG','17952','EPSG','3793','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','17953','Lindis Peak 2000','Replaces Lindis Peak Circuit (code 17923) after 1st March 2000.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-44.4406,'EPSG','9110','EPSG','8802','Longitude of natural origin',169.2803,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',400000.0,'EPSG','9001','EPSG','8807','False northing',800000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12693','conversion','EPSG','17953','EPSG','3795','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','17954','Mount Nicholas 2000','Replaces Mount Nicholas Circuit (code 17924) after 1st March 2000.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-45.0758,'EPSG','9110','EPSG','8802','Longitude of natural origin',168.2355,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',400000.0,'EPSG','9001','EPSG','8807','False northing',800000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12694','conversion','EPSG','17954','EPSG','3797','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','17955','Mount York 2000','Replaces Mount York Circuit (code 17925) after 1st March 2000.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-45.3349,'EPSG','9110','EPSG','8802','Longitude of natural origin',167.4419,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',400000.0,'EPSG','9001','EPSG','8807','False northing',800000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12695','conversion','EPSG','17955','EPSG','3799','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','17956','Observation Point 2000','Replaces Observation Point Circuit (code 17926) after 1st March 2000.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-45.4858,'EPSG','9110','EPSG','8802','Longitude of natural origin',170.3742,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',400000.0,'EPSG','9001','EPSG','8807','False northing',800000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12696','conversion','EPSG','17956','EPSG','3796','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','17957','North Taieri 2000','Replaces North Taieri Circuit (code 17927) after 1st March 2000.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-45.5141,'EPSG','9110','EPSG','8802','Longitude of natural origin',170.1657,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.99996,'EPSG','9201','EPSG','8806','False easting',400000.0,'EPSG','9001','EPSG','8807','False northing',800000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12697','conversion','EPSG','17957','EPSG','3798','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','17958','Bluff 2000','Replaces Bluff Circuit (code 17928) after 1st March 2000.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-46.36,'EPSG','9110','EPSG','8802','Longitude of natural origin',168.2034,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',400000.0,'EPSG','9001','EPSG','8807','False northing',800000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12698','conversion','EPSG','17958','EPSG','3800','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','17959','Chatham Island Circuit 2000','Officially discontinued 6 June 2006. Replaced by Chatham Islands Transverse Mercator 2000 (Code 17965).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-44.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-176.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',400000.0,'EPSG','9001','EPSG','8807','False northing',800000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12699','conversion','EPSG','17959','EPSG','2889','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','17960','Auckland Islands Transverse Mercator 2000','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',166.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',3500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12700','conversion','EPSG','17960','EPSG','3554','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','17961','Campbell Island Transverse Mercator 2000','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',169.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',3500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12701','conversion','EPSG','17961','EPSG','3555','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','17962','Antipodes Islands Transverse Mercator 2000','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',179.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',3500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12702','conversion','EPSG','17962','EPSG','3556','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','17963','Raoul Island Transverse Mercator 2000','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-178.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',3500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12703','conversion','EPSG','17963','EPSG','3557','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','17964','New Zealand Continental Shelf Lambert Conformal 2000','See NIWA Albers (projection code 9190) for oceanic data statistical analysis.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-41.0,'EPSG','9110','EPSG','8822','Longitude of false origin',173.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',-37.3,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-44.3,'EPSG','9110','EPSG','8826','Easting at false origin',3000000.0,'EPSG','9001','EPSG','8827','Northing at false origin',7000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12704','conversion','EPSG','17964','EPSG','3593','EPSG','1026');
INSERT INTO "conversion" VALUES('EPSG','17965','Chatham Islands Transverse Mercator 2000','Replaces Chatham Island Circuit 2000 (code 17959).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-176.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',3500000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12705','conversion','EPSG','17965','EPSG','2889','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','17966','Darwin Glacier Lambert Conformal 2000','Replaced by McMurdo Sound Lambert Conformal 2000 (proj code 5475) from March 2011. LINZ S20007 withdrawn at this date.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-90.0,'EPSG','9110','EPSG','8822','Longitude of false origin',157.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',-76.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-79.2,'EPSG','9110','EPSG','8826','Easting at false origin',500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12706','conversion','EPSG','17966','EPSG','3592','EPSG','1236');
INSERT INTO "conversion" VALUES('EPSG','18001','Austria Gauss-Kruger West Zone','Austria West Zone (EPSG proj code 18041) with truncated northing. Longitude is referenced to the Ferro meridian. See code 18004 for equivalent projection referenced to the Greenwich meridian.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',28.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',-5000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12707','conversion','EPSG','18001','EPSG','1706','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18002','Austria Gauss-Kruger Central Zone','Austria Central Zone (EPSG proj code 18042) with truncated northing. Longitude is referenced to the Ferro meridian. See code 18005 for equivalent projection referenced to the Greenwich meridian.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',31.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',-5000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12708','conversion','EPSG','18002','EPSG','1707','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18003','Austria Gauss-Kruger East Zone','Austria East Zone (EPSG proj code 18043) with truncated northing. Longitude is referenced to the Ferro meridian. See code 18006 for equivalent projection referenced to the Greenwich meridian.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',34.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',-5000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12709','conversion','EPSG','18003','EPSG','1708','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18004','Austria Gauss-Kruger West','Greenwich equivalent of Austria GK West zone (EPSG code 18001).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',10.2,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',-5000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12710','conversion','EPSG','18004','EPSG','1706','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18005','Austria Gauss-Kruger Central','Greenwich equivalent of Austria GK Central zone (EPSG code 18002).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',13.2,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',-5000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12711','conversion','EPSG','18005','EPSG','1707','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18006','Austria Gauss-Kruger East','Greenwich equivalent of Austria GK East zone (EPSG code 18003).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',16.2,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',-5000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12712','conversion','EPSG','18006','EPSG','1708','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18007','Austria Gauss-Kruger M28','Proj code 18044 but with truncated northing.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',10.2,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',150000.0,'EPSG','9001','EPSG','8807','False northing',-5000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12713','conversion','EPSG','18007','EPSG','1706','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18008','Austria Gauss-Kruger M31','Proj code 18045 but with truncated northing.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',13.2,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',450000.0,'EPSG','9001','EPSG','8807','False northing',-5000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12714','conversion','EPSG','18008','EPSG','1707','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18009','Austria Gauss-Kruger M34','Proj code 18046 but with truncated northing.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',16.2,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',750000.0,'EPSG','9001','EPSG','8807','False northing',-5000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12715','conversion','EPSG','18009','EPSG','1708','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18011','Nord Algerie (ancienne)','Used with Voirol 1875 datum - now replaced.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',40.0,'EPSG','9105','EPSG','8802','Longitude of natural origin',3.0,'EPSG','9105','EPSG','8805','Scale factor at natural origin',0.999625544,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',300000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12716','conversion','EPSG','18011','EPSG','1728','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18012','Sud Algerie (ancienne)','Used with Voirol 1875 datum - now replaced.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',37.0,'EPSG','9105','EPSG','8802','Longitude of natural origin',3.0,'EPSG','9105','EPSG','8805','Scale factor at natural origin',0.999625769,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',300000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12717','conversion','EPSG','18012','EPSG','1729','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18021','Nord Algerie','Use with Nord Sahara 1959 datum.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',40.0,'EPSG','9105','EPSG','8802','Longitude of natural origin',3.0,'EPSG','9105','EPSG','8805','Scale factor at natural origin',0.999625544,'EPSG','9201','EPSG','8806','False easting',500135.0,'EPSG','9001','EPSG','8807','False northing',300090.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12718','conversion','EPSG','18021','EPSG','1728','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18022','Sud Algerie','Use with Nord Sahara 1959 datum. INCT uses this projection to the southern limit of Algeria but due to distance from origin OGP does not endorse this practice and south of 31°30''N recommends use of UTM.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',37.0,'EPSG','9105','EPSG','8802','Longitude of natural origin',3.0,'EPSG','9105','EPSG','8805','Scale factor at natural origin',0.999625769,'EPSG','9201','EPSG','8806','False easting',500135.0,'EPSG','9001','EPSG','8807','False northing',300090.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12719','conversion','EPSG','18022','EPSG','1729','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18031','Argentina zone 1','Original transformation by Gauss-Kruger formula','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-90.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-72.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',1500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12720','conversion','EPSG','18031','EPSG','1608','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18032','Argentina zone 2','Original transformation by Gauss-Kruger formula','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-90.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-69.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',2500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12721','conversion','EPSG','18032','EPSG','1609','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18033','Argentina zone 3','Original transformation by Gauss-Kruger formula','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-90.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-66.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',3500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12722','conversion','EPSG','18033','EPSG','1610','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18034','Argentina zone 4','Original transformation by Gauss-Kruger formula','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-90.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-63.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',4500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12723','conversion','EPSG','18034','EPSG','1611','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18035','Argentina zone 5','Original transformation by Gauss-Kruger formula','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-90.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-60.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',5500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12724','conversion','EPSG','18035','EPSG','1612','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18036','Argentina zone 6','Original transformation by Gauss-Kruger formula','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-90.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-57.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',6500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12725','conversion','EPSG','18036','EPSG','1613','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18037','Argentina zone 7','Original transformation by Gauss-Kruger formula','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-90.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-54.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',7500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12726','conversion','EPSG','18037','EPSG','1614','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18041','Austria West Zone','Longitude is referenced to the Ferro meridian. See code 9268 for equivalent referenced to Greenwich meridian. Replaced by Austria zone M28 (EPSG code 18047) for large and medium scale topographic mapping and engineering survey.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',28.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12727','conversion','EPSG','18041','EPSG','1706','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','18042','Austria Central Zone','Longitude is referenced to the Ferro meridian. See code 9269 for equivalent referenced to Greenwich meridian. Replaced by Austria zone M31 (EPSG code 18048) for large and medium scale topographic mapping and engineering survey.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',31.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12728','conversion','EPSG','18042','EPSG','1707','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','18043','Austria East Zone','Longitude is referenced to the Ferro meridian. See code 9270 for equivalent referenced to Greenwich meridian. Replaced by Austria zone M34 (EPSG code 18049) for large and medium scale topographic mapping and engineering survey.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',34.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12729','conversion','EPSG','18043','EPSG','1708','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','18044','Austria M28','Greenwich equivalent of Austria zone M28 (EPSG code 18047).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',10.2,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',150000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12730','conversion','EPSG','18044','EPSG','1706','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18045','Austria M31','Greenwich equivalent of Austria zone M31 (EPSG code 18048).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',13.2,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',450000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12731','conversion','EPSG','18045','EPSG','1707','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18046','Austria M34','Greenwich equivalent of Austria zone M34 (EPSG code 18049).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',16.2,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',750000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12732','conversion','EPSG','18046','EPSG','1708','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18047','Austria zone M28','Longitude is referenced to the Ferro meridian. See code 18044 for equivalent referenced to Greenwich meridian. Replaces Austria West Zone (EPSG code 18041) for large and medium scale topographic mapping and engineering survey but not cadastral survey.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',28.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',150000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12733','conversion','EPSG','18047','EPSG','1706','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18048','Austria zone M31','Longitude is referenced to the Ferro meridian. See code 18045 for equivalent referenced to Greenwich meridian. Replaces Austria Central Zone (EPSG code 18042) for large and medium scale topographic mapping and engineering survey but not cadastral survey.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',31.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',450000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12734','conversion','EPSG','18048','EPSG','1707','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18049','Austria zone M34','Longitude is referenced to the Ferro meridian. See code 18046 for equivalent referenced to Greenwich meridian. Replaces Austria East Zone (EPSG code 18043) for large and medium scale topographic mapping and engineering survey but not cadastral survey.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',34.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',750000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12735','conversion','EPSG','18049','EPSG','1708','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18051','Colombia West zone','Original transformation by Gauss-Kruger formula. Zone name sometimes referred to as "3 west". Associated with Bogota datum.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',4.355657,'EPSG','9110','EPSG','8802','Longitude of natural origin',-77.04513,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',1000000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12736','conversion','EPSG','18051','EPSG','1598','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18052','Colombia Bogota zone','Original transformation by Gauss-Kruger formula. Associated with Bogota datum.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',4.355657,'EPSG','9110','EPSG','8802','Longitude of natural origin',-74.04513,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',1000000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12737','conversion','EPSG','18052','EPSG','1599','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18053','Colombia East Central zone','Original transformation by Gauss-Kruger formula. Zone name sometimes referred to as "3 east". Associated with Bogota datum.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',4.355657,'EPSG','9110','EPSG','8802','Longitude of natural origin',-71.04513,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',1000000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12738','conversion','EPSG','18053','EPSG','1600','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18054','Colombia East zone','Original transformation by Gauss-Kruger formula. Zone name sometimes referred to as "6 east". Associated with Bogota datum.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',4.355657,'EPSG','9110','EPSG','8802','Longitude of natural origin',-68.04513,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',1000000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12739','conversion','EPSG','18054','EPSG','1601','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18055','Colombia MAGNA Far West zone','Zone name sometimes referred to as "6 west". Associated with MAGNA-SIRGAS datum.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',4.35463215,'EPSG','9110','EPSG','8802','Longitude of natural origin',-80.04390285,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',1000000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12740','conversion','EPSG','18055','EPSG','3091','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18056','Colombia MAGNA West zone','Zone name sometimes referred to as "3 west". Associated with MAGNA-SIRGAS datum.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',4.35463215,'EPSG','9110','EPSG','8802','Longitude of natural origin',-77.04390285,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',1000000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12741','conversion','EPSG','18056','EPSG','3090','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18057','Colombia MAGNA Bogota zone','Associated with MAGNA-SIRGAS datum.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',4.35463215,'EPSG','9110','EPSG','8802','Longitude of natural origin',-74.04390285,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',1000000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12742','conversion','EPSG','18057','EPSG','1599','EPSG','1208');
INSERT INTO "usage" VALUES('EPSG','13874','conversion','EPSG','18057','EPSG','3229','EPSG','1241');
INSERT INTO "conversion" VALUES('EPSG','18058','Colombia MAGNA East Central zone','Zone name sometimes referred to as "3 east". Associated with MAGNA-SIRGAS datum.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',4.35463215,'EPSG','9110','EPSG','8802','Longitude of natural origin',-71.04390285,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',1000000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12743','conversion','EPSG','18058','EPSG','1600','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18059','Colombia MAGNA East zone','Zone name sometimes referred to as "6 east". Associated with MAGNA-SIRGAS datum.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',4.35463215,'EPSG','9110','EPSG','8802','Longitude of natural origin',-68.04390285,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',1000000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12744','conversion','EPSG','18059','EPSG','1601','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18061','Cuba Norte','','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',22.21,'EPSG','9110','EPSG','8802','Longitude of natural origin',-81.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.99993602,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',280296.016,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','12745','conversion','EPSG','18061','EPSG','1487','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18062','Cuba Sur','','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',20.43,'EPSG','9110','EPSG','8802','Longitude of natural origin',-76.5,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.99994848,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',229126.939,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','12746','conversion','EPSG','18062','EPSG','1488','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18063','Cuba Norte','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',22.21,'EPSG','9110','EPSG','8822','Longitude of false origin',-81.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',23.0,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',21.42,'EPSG','9110','EPSG','8826','Easting at false origin',500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',280296.016,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12747','conversion','EPSG','18063','EPSG','1487','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18064','Cuba Sur','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',20.43,'EPSG','9110','EPSG','8822','Longitude of false origin',-76.5,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',21.18,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',20.08,'EPSG','9110','EPSG','8826','Easting at false origin',500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',229126.939,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12748','conversion','EPSG','18064','EPSG','1488','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18065','Colombia MAGNA 2018 Far West zone','Associated with MAGNA-SIRGAS 2018 reference frame.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',4.35463316,'EPSG','9110','EPSG','8802','Longitude of natural origin',-80.043902797,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',1000000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','18936','conversion','EPSG','18065','EPSG','3091','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18066','Colombia MAGNA 2018 West zone','Associated with MAGNA-SIRGAS 2018 reference frame.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',4.35463316,'EPSG','9110','EPSG','8802','Longitude of natural origin',-77.043902797,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',1000000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','18937','conversion','EPSG','18066','EPSG','3090','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18067','Colombia MAGNA 2018 Bogota zone','Associated with MAGNA-SIRGAS 2018 reference frame.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',4.35463316,'EPSG','9110','EPSG','8802','Longitude of natural origin',-74.043902797,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',1000000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','18938','conversion','EPSG','18067','EPSG','1599','EPSG','1208');
INSERT INTO "conversion" VALUES('EPSG','18068','Colombia MAGNA 2018 East Central zone','Associated with MAGNA-SIRGAS 2018 reference frame.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',4.35463316,'EPSG','9110','EPSG','8802','Longitude of natural origin',-71.043902797,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',1000000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','18939','conversion','EPSG','18068','EPSG','1600','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18069','Colombia MAGNA 2018 East zone','Associated with MAGNA-SIRGAS 2018 reference frame.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',4.35463316,'EPSG','9110','EPSG','8802','Longitude of natural origin',-68.043902797,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',1000000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','18940','conversion','EPSG','18069','EPSG','1601','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18071','Egypt Blue Belt','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',30.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',35.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',300000.0,'EPSG','9001','EPSG','8807','False northing',1100000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12749','conversion','EPSG','18071','EPSG','1642','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18072','Egypt Red Belt','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',30.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',31.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',615000.0,'EPSG','9001','EPSG','8807','False northing',810000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12750','conversion','EPSG','18072','EPSG','1643','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18073','Egypt Purple Belt','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',30.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',27.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',700000.0,'EPSG','9001','EPSG','8807','False northing',200000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12751','conversion','EPSG','18073','EPSG','1644','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18074','Egypt Extended Purple Belt','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',30.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',27.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',700000.0,'EPSG','9001','EPSG','8807','False northing',1200000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12752','conversion','EPSG','18074','EPSG','1645','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18081','Lambert zone I','Longitude is referenced to the Paris meridian. Introduced 1972. Replaces Lambert Nord France (code 18091).','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',55.0,'EPSG','9105','EPSG','8802','Longitude of natural origin',0.0,'EPSG','9105','EPSG','8805','Scale factor at natural origin',0.999877341,'EPSG','9201','EPSG','8806','False easting',600000.0,'EPSG','9001','EPSG','8807','False northing',1200000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12753','conversion','EPSG','18081','EPSG','1731','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18082','Lambert zone II','Longitude is referenced to the Paris meridian. Introduced 1972. Replaces Lambert Centre France (code 18092).','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',52.0,'EPSG','9105','EPSG','8802','Longitude of natural origin',0.0,'EPSG','9105','EPSG','8805','Scale factor at natural origin',0.99987742,'EPSG','9201','EPSG','8806','False easting',600000.0,'EPSG','9001','EPSG','8807','False northing',2200000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12754','conversion','EPSG','18082','EPSG','1734','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18083','Lambert zone III','Longitude is referenced to the Paris meridian. Introduced 1972. Replaces Lambert Sud France (code 18093).','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',49.0,'EPSG','9105','EPSG','8802','Longitude of natural origin',0.0,'EPSG','9105','EPSG','8805','Scale factor at natural origin',0.999877499,'EPSG','9201','EPSG','8806','False easting',600000.0,'EPSG','9001','EPSG','8807','False northing',3200000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12755','conversion','EPSG','18083','EPSG','1733','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18084','Lambert zone IV','Longitude is referenced to the Paris meridian. Introduced 1972. Replaces Lambert Corse (code 18094).','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',46.85,'EPSG','9105','EPSG','8802','Longitude of natural origin',0.0,'EPSG','9105','EPSG','8805','Scale factor at natural origin',0.99994471,'EPSG','9201','EPSG','8806','False easting',234.358,'EPSG','9001','EPSG','8807','False northing',4185861.369,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12756','conversion','EPSG','18084','EPSG','1327','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18085','Lambert-93','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',46.3,'EPSG','9110','EPSG','8822','Longitude of false origin',3.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',49.0,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',44.0,'EPSG','9110','EPSG','8826','Easting at false origin',700000.0,'EPSG','9001','EPSG','8827','Northing at false origin',6600000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12757','conversion','EPSG','18085','EPSG','1096','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18086','France EuroLambert','This is Lambert zone II (code 18082) parameters converted from grads/Paris to degrees/Greenwich for use with ED50.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',46.48,'EPSG','9110','EPSG','8802','Longitude of natural origin',2.2014025,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.99987742,'EPSG','9201','EPSG','8806','False easting',600000.0,'EPSG','9001','EPSG','8807','False northing',2200000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','12758','conversion','EPSG','18086','EPSG','1326','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','18091','Lambert Nord France','Longitude is referenced to the Paris meridian. Replaced in 1972 by Lambert zone I (code 18081).','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',55.0,'EPSG','9105','EPSG','8802','Longitude of natural origin',0.0,'EPSG','9105','EPSG','8805','Scale factor at natural origin',0.999877341,'EPSG','9201','EPSG','8806','False easting',600000.0,'EPSG','9001','EPSG','8807','False northing',200000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12759','conversion','EPSG','18091','EPSG','1731','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18092','Lambert Centre France','Longitude is referenced to the Paris meridian. Replaced in 1972 by Lambert zone II (code 18082).','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',52.0,'EPSG','9105','EPSG','8802','Longitude of natural origin',0.0,'EPSG','9105','EPSG','8805','Scale factor at natural origin',0.99987742,'EPSG','9201','EPSG','8806','False easting',600000.0,'EPSG','9001','EPSG','8807','False northing',200000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12760','conversion','EPSG','18092','EPSG','1734','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18093','Lambert Sud France','Longitude is referenced to the Paris meridian. Replaced in 1972 by Lambert zone III (code 18083).','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',49.0,'EPSG','9105','EPSG','8802','Longitude of natural origin',0.0,'EPSG','9105','EPSG','8805','Scale factor at natural origin',0.999877499,'EPSG','9201','EPSG','8806','False easting',600000.0,'EPSG','9001','EPSG','8807','False northing',200000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12761','conversion','EPSG','18093','EPSG','1733','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18094','Lambert Corse','Longitude is referenced to the Paris meridian. Replaced in 1972 by Lambert zone IV (code 18084).','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',46.85,'EPSG','9105','EPSG','8802','Longitude of natural origin',0.0,'EPSG','9105','EPSG','8805','Scale factor at natural origin',0.99994471,'EPSG','9201','EPSG','8806','False easting',234.358,'EPSG','9001','EPSG','8807','False northing',185861.369,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12762','conversion','EPSG','18094','EPSG','1327','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18101','France Conic Conformal zone 1','Compliments Lambert-93. Scale factor ranges between -9 and +7 cm/km. Area of use overlaps adjacent zone.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',42.0,'EPSG','9102','EPSG','8822','Longitude of false origin',3.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',41.25,'EPSG','9102','EPSG','8824','Latitude of 2nd standard parallel',42.75,'EPSG','9102','EPSG','8826','Easting at false origin',1700000.0,'EPSG','9001','EPSG','8827','Northing at false origin',1200000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12763','conversion','EPSG','18101','EPSG','3545','EPSG','1207');
INSERT INTO "conversion" VALUES('EPSG','18102','France Conic Conformal zone 2','Compliments Lambert-93. Scale factor ranges between -9 and +7 cm/km. Area of use overlaps adjacent zones.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',43.0,'EPSG','9102','EPSG','8822','Longitude of false origin',3.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',42.25,'EPSG','9102','EPSG','8824','Latitude of 2nd standard parallel',43.75,'EPSG','9102','EPSG','8826','Easting at false origin',1700000.0,'EPSG','9001','EPSG','8827','Northing at false origin',2200000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12764','conversion','EPSG','18102','EPSG','3546','EPSG','1207');
INSERT INTO "conversion" VALUES('EPSG','18103','France Conic Conformal zone 3','Compliments Lambert-93. Scale factor ranges between -9 and +7 cm/km. Area of use overlaps adjacent zones.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',44.0,'EPSG','9102','EPSG','8822','Longitude of false origin',3.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',43.25,'EPSG','9102','EPSG','8824','Latitude of 2nd standard parallel',44.75,'EPSG','9102','EPSG','8826','Easting at false origin',1700000.0,'EPSG','9001','EPSG','8827','Northing at false origin',3200000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12765','conversion','EPSG','18103','EPSG','3547','EPSG','1207');
INSERT INTO "conversion" VALUES('EPSG','18104','France Conic Conformal zone 4','Compliments Lambert-93. Scale factor ranges between -9 and +7 cm/km. Area of use overlaps adjacent zones.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',45.0,'EPSG','9102','EPSG','8822','Longitude of false origin',3.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',44.25,'EPSG','9102','EPSG','8824','Latitude of 2nd standard parallel',45.75,'EPSG','9102','EPSG','8826','Easting at false origin',1700000.0,'EPSG','9001','EPSG','8827','Northing at false origin',4200000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12766','conversion','EPSG','18104','EPSG','3548','EPSG','1207');
INSERT INTO "conversion" VALUES('EPSG','18105','France Conic Conformal zone 5','Compliments Lambert-93. Scale factor ranges between -9 and +7 cm/km. Area of use overlaps adjacent zones.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',46.0,'EPSG','9102','EPSG','8822','Longitude of false origin',3.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',45.25,'EPSG','9102','EPSG','8824','Latitude of 2nd standard parallel',46.75,'EPSG','9102','EPSG','8826','Easting at false origin',1700000.0,'EPSG','9001','EPSG','8827','Northing at false origin',5200000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12767','conversion','EPSG','18105','EPSG','3549','EPSG','1207');
INSERT INTO "conversion" VALUES('EPSG','18106','France Conic Conformal zone 6','Compliments Lambert-93. Scale factor ranges between -9 and +7 cm/km. Area of use overlaps adjacent zones.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',47.0,'EPSG','9102','EPSG','8822','Longitude of false origin',3.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',46.25,'EPSG','9102','EPSG','8824','Latitude of 2nd standard parallel',47.75,'EPSG','9102','EPSG','8826','Easting at false origin',1700000.0,'EPSG','9001','EPSG','8827','Northing at false origin',6200000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12768','conversion','EPSG','18106','EPSG','3550','EPSG','1207');
INSERT INTO "conversion" VALUES('EPSG','18107','France Conic Conformal zone 7','Compliments Lambert-93. Scale factor ranges between -9 and +7 cm/km. Area of use overlaps adjacent zones.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',48.0,'EPSG','9102','EPSG','8822','Longitude of false origin',3.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',47.25,'EPSG','9102','EPSG','8824','Latitude of 2nd standard parallel',48.75,'EPSG','9102','EPSG','8826','Easting at false origin',1700000.0,'EPSG','9001','EPSG','8827','Northing at false origin',7200000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12769','conversion','EPSG','18107','EPSG','3551','EPSG','1207');
INSERT INTO "conversion" VALUES('EPSG','18108','France Conic Conformal zone 8','Compliments Lambert-93. Scale factor ranges between -9 and +7 cm/km. Area of use overlaps adjacent zones.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',49.0,'EPSG','9102','EPSG','8822','Longitude of false origin',3.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',48.25,'EPSG','9102','EPSG','8824','Latitude of 2nd standard parallel',49.75,'EPSG','9102','EPSG','8826','Easting at false origin',1700000.0,'EPSG','9001','EPSG','8827','Northing at false origin',8200000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12770','conversion','EPSG','18108','EPSG','3552','EPSG','1207');
INSERT INTO "conversion" VALUES('EPSG','18109','France Conic Conformal zone 9','Compliments Lambert-93. Scale factor ranges between -9 and +7 cm/km. Area of use overlaps adjacent zone.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',50.0,'EPSG','9102','EPSG','8822','Longitude of false origin',3.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',49.25,'EPSG','9102','EPSG','8824','Latitude of 2nd standard parallel',50.75,'EPSG','9102','EPSG','8826','Easting at false origin',1700000.0,'EPSG','9001','EPSG','8827','Northing at false origin',9200000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12771','conversion','EPSG','18109','EPSG','3553','EPSG','1207');
INSERT INTO "conversion" VALUES('EPSG','18110','India zone 0','BEWARE ! Different yard to metre conversion values have been used in different parts of south Asia. Some areas have changed conversion value with time.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',39.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',68.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.99846154,'EPSG','9201','EPSG','8806','False easting',2355500.0,'EPSG','9084','EPSG','8807','False northing',2590000.0,'EPSG','9084',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12772','conversion','EPSG','18110','EPSG','1668','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18111','India zone I','BEWARE ! Different yard to metre conversion values have been used in different parts of south Asia. Some areas have changed conversion value with time.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',32.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',68.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.99878641,'EPSG','9201','EPSG','8806','False easting',3000000.0,'EPSG','9084','EPSG','8807','False northing',1000000.0,'EPSG','9084',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12773','conversion','EPSG','18111','EPSG','1669','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18112','India zone IIa','BEWARE ! Different yard to metre conversion values have been used in different parts of south Asia. Some areas have changed conversion value with time.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',26.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',74.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.99878641,'EPSG','9201','EPSG','8806','False easting',3000000.0,'EPSG','9084','EPSG','8807','False northing',1000000.0,'EPSG','9084',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12774','conversion','EPSG','18112','EPSG','1670','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18113','India zone IIb','BEWARE ! Different yard to metre conversion values have been used in different parts of south Asia. Some areas have changed conversion value with time.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',26.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',90.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.99878641,'EPSG','9201','EPSG','8806','False easting',3000000.0,'EPSG','9084','EPSG','8807','False northing',1000000.0,'EPSG','9084',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12775','conversion','EPSG','18113','EPSG','1671','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18114','India zone IIIa','BEWARE ! Different yard to metre conversion values have been used in different parts of south Asia. Some areas have changed conversion value with time.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',19.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',80.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.99878641,'EPSG','9201','EPSG','8806','False easting',3000000.0,'EPSG','9084','EPSG','8807','False northing',1000000.0,'EPSG','9084',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12776','conversion','EPSG','18114','EPSG','1672','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18115','India zone IIIb','BEWARE ! Different yard to metre conversion values have been used in different parts of south Asia. Some areas have changed conversion value with time.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',19.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',100.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.99878641,'EPSG','9201','EPSG','8806','False easting',3000000.0,'EPSG','9084','EPSG','8807','False northing',1000000.0,'EPSG','9084',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12777','conversion','EPSG','18115','EPSG','2292','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18116','India zone IVa','BEWARE ! Different yard to metre conversion values have been used in different parts of south Asia. Some areas have changed conversion value with time.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',12.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',80.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.99878641,'EPSG','9201','EPSG','8806','False easting',3000000.0,'EPSG','9084','EPSG','8807','False northing',1000000.0,'EPSG','9084',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12778','conversion','EPSG','18116','EPSG','1673','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18117','India zone IVb','BEWARE ! Different yard to metre conversion values have been used in different parts of south Asia. Some areas have changed conversion value with time.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',12.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',100.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.99878641,'EPSG','9201','EPSG','8806','False easting',3000000.0,'EPSG','9084','EPSG','8807','False northing',1000000.0,'EPSG','9084',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12779','conversion','EPSG','18117','EPSG','2293','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18121','Italy zone 1','Original transformation by Gauss-Boaga formula','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',9.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',1500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12780','conversion','EPSG','18121','EPSG','1718','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18122','Italy zone 2','Original transformation by Gauss-Boaga formula','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',15.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',2520000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12781','conversion','EPSG','18122','EPSG','1719','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18131','Nord Maroc','A projection with the same parameter values used the Lambert Conic Near-Conformal method (EPSG code 9817) prior to 1953.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',37.0,'EPSG','9105','EPSG','8802','Longitude of natural origin',-6.0,'EPSG','9105','EPSG','8805','Scale factor at natural origin',0.999625769,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',300000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12782','conversion','EPSG','18131','EPSG','1703','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18132','Sud Maroc','A projection with the same parameter values used the Lambert Conic Near-Conformal method (EPSG code 9817) prior to 1953.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',33.0,'EPSG','9105','EPSG','8802','Longitude of natural origin',-6.0,'EPSG','9105','EPSG','8805','Scale factor at natural origin',0.999615596,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',300000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12783','conversion','EPSG','18132','EPSG','2787','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18133','Sahara','Created in 1977 to cover Sahara Marocain (ex Spanish Sahara)','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',29.0,'EPSG','9105','EPSG','8802','Longitude of natural origin',-6.0,'EPSG','9105','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',1200000.0,'EPSG','9001','EPSG','8807','False northing',400000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','12784','conversion','EPSG','18133','EPSG','1705','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18134','Sahara Nord','Created in 1977.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',29.0,'EPSG','9105','EPSG','8802','Longitude of natural origin',-6.0,'EPSG','9105','EPSG','8805','Scale factor at natural origin',0.999616304,'EPSG','9201','EPSG','8806','False easting',1200000.0,'EPSG','9001','EPSG','8807','False northing',400000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12785','conversion','EPSG','18134','EPSG','2788','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18135','Sahara Sud','','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',25.0,'EPSG','9105','EPSG','8802','Longitude of natural origin',-6.0,'EPSG','9105','EPSG','8805','Scale factor at natural origin',0.999616437,'EPSG','9201','EPSG','8806','False easting',1500000.0,'EPSG','9001','EPSG','8807','False northing',400000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12786','conversion','EPSG','18135','EPSG','2789','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18141','New Zealand North Island National Grid','Replaced by New Zealand Map Grid (code 19917). Used for topographic mapping.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-39.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',175.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',300000.0,'EPSG','9040','EPSG','8807','False northing',400000.0,'EPSG','9040',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12787','conversion','EPSG','18141','EPSG','1500','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18142','New Zealand South Island National Grid','Replaced by New Zealand Map Grid (code 19917). Used for topographic mapping.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-44.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',171.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9040','EPSG','8807','False northing',500000.0,'EPSG','9040',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12788','conversion','EPSG','18142','EPSG','3344','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18151','Nigeria West Belt','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',4.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',4.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.99975,'EPSG','9201','EPSG','8806','False easting',230738.26,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12789','conversion','EPSG','18151','EPSG','1715','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18152','Nigeria Mid Belt','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',4.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',8.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.99975,'EPSG','9201','EPSG','8806','False easting',670553.98,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12790','conversion','EPSG','18152','EPSG','1714','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18153','Nigeria East Belt','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',4.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',12.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.99975,'EPSG','9201','EPSG','8806','False easting',1110369.7,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12791','conversion','EPSG','18153','EPSG','1713','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18161','Peru west zone','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-6.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-80.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.99983008,'EPSG','9201','EPSG','8806','False easting',222000.0,'EPSG','9001','EPSG','8807','False northing',1426834.743,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12792','conversion','EPSG','18161','EPSG','1753','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18162','Peru central zone','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-9.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-76.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.99932994,'EPSG','9201','EPSG','8806','False easting',720000.0,'EPSG','9001','EPSG','8807','False northing',1039979.159,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12793','conversion','EPSG','18162','EPSG','1752','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18163','Peru east zone','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-9.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-70.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.99952992,'EPSG','9201','EPSG','8806','False easting',1324000.0,'EPSG','9001','EPSG','8807','False northing',1040084.558,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12794','conversion','EPSG','18163','EPSG','1751','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18171','Philippines zone I','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',117.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.99995,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12795','conversion','EPSG','18171','EPSG','1698','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18172','Philippines zone II','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',119.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.99995,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12796','conversion','EPSG','18172','EPSG','1699','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18173','Philippines zone III','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',121.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.99995,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12797','conversion','EPSG','18173','EPSG','1700','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18174','Philippines zone IV','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',123.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.99995,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12798','conversion','EPSG','18174','EPSG','1701','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18175','Philippines zone V','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',125.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.99995,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12799','conversion','EPSG','18175','EPSG','1702','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18180','Finland zone 0','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',18.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12800','conversion','EPSG','18180','EPSG','3092','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18181','Nord Tunisie','','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',40.0,'EPSG','9105','EPSG','8802','Longitude of natural origin',11.0,'EPSG','9105','EPSG','8805','Scale factor at natural origin',0.999625544,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',300000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12801','conversion','EPSG','18181','EPSG','1619','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18182','Sud Tunisie','','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',37.0,'EPSG','9105','EPSG','8802','Longitude of natural origin',11.0,'EPSG','9105','EPSG','8805','Scale factor at natural origin',0.999625769,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',300000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12802','conversion','EPSG','18182','EPSG','1620','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18183','Finland ETRS-GK19','Replaced by GK19FIN (proj code 3860).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',19.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12803','conversion','EPSG','18183','EPSG','3092','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','18184','Finland ETRS-GK20','Replaced by GK20FIN (proj code 3861).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',20.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12804','conversion','EPSG','18184','EPSG','3093','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','18185','Finland ETRS-GK21','Replaced by GK21FIN (proj code 3862).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',21.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12805','conversion','EPSG','18185','EPSG','3094','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','18186','Finland ETRS-GK22','Replaced by GK22FIN (proj code 3863).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',22.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12806','conversion','EPSG','18186','EPSG','3095','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','18187','Finland ETRS-GK23','Replaced by GK23FIN (proj code 3864).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',23.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12807','conversion','EPSG','18187','EPSG','3096','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','18188','Finland ETRS-GK24','Replaced by GK24FIN (proj code 3865).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',24.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12808','conversion','EPSG','18188','EPSG','3097','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','18189','Finland ETRS-GK25','Replaced by GK25FIN (proj code 3866).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',25.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12809','conversion','EPSG','18189','EPSG','3098','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','18190','Finland ETRS-GK26','Replaced by GK26FIN (proj code 3867).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',26.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12810','conversion','EPSG','18190','EPSG','3099','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','18191','Finland zone 1','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',21.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',1500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12811','conversion','EPSG','18191','EPSG','1536','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18192','Finland zone 2','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',24.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',2500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12812','conversion','EPSG','18192','EPSG','1537','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18193','Finland Uniform Coordinate System','Used by Uniform Coordinate System over all country and also by zone 3 of Basic Coordinate System at larger scales.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',27.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',3500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12813','conversion','EPSG','18193','EPSG','1538','EPSG','1208');
INSERT INTO "usage" VALUES('EPSG','13875','conversion','EPSG','18193','EPSG','3333','EPSG','1241');
INSERT INTO "conversion" VALUES('EPSG','18194','Finland zone 4','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',30.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',4500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12814','conversion','EPSG','18194','EPSG','1539','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18195','Finland ETRS-GK27','Replaced by GK27FIN (proj code 3868).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',27.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12815','conversion','EPSG','18195','EPSG','3100','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','18196','Finland ETRS-GK28','Replaced by GK28FIN (proj code 3869).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',28.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12816','conversion','EPSG','18196','EPSG','3101','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','18197','Finland ETRS-GK29','Replaced by GK29FIN (proj code 3870).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',29.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12817','conversion','EPSG','18197','EPSG','3102','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','18198','Finland ETRS-GK30','Replaced by GK30FIN (proj code 3871).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',30.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12818','conversion','EPSG','18198','EPSG','3103','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','18199','Finland ETRS-GK31','Replaced by GK31FIN (proj code 3872).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',31.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12819','conversion','EPSG','18199','EPSG','3104','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','18201','Palestine Grid','Replaced (i) in Israel by Israeli CS (proj code 18203) which adds 1 million to FN; (ii) for AMS by Palestine Belt (code18202) which adds 1 million to FN and changes method to TM. Also encountered as a TM projection without change to FN (see code 7141).','EPSG','9806','Cassini-Soldner','EPSG','8801','Latitude of natural origin',31.4402749,'EPSG','9110','EPSG','8802','Longitude of natural origin',35.124349,'EPSG','9110','EPSG','8806','False easting',170251.555,'EPSG','9001','EPSG','8807','False northing',126867.909,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12820','conversion','EPSG','18201','EPSG','1356','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18202','Palestine Belt','Replaces Palestine Grid (code 18201). See also Israeli CS (proj code 18203) which has identical parameter values: the difference in coordinates between the Palestine Belt and the Israeli CS caused by their different methods is under 2m within Israel.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',31.4402749,'EPSG','9110','EPSG','8802','Longitude of natural origin',35.124349,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',170251.555,'EPSG','9001','EPSG','8807','False northing',1126867.909,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12821','conversion','EPSG','18202','EPSG','1356','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18203','Israeli CS','Based on proj 18201 but with 1 million added to FN. Note: Palestine Belt (proj code 18202) has identical parameter values: the difference in conversion between the Israeli CS and the Palestine Belt caused by their different methods is under 2m in Israel.','EPSG','9806','Cassini-Soldner','EPSG','8801','Latitude of natural origin',31.4402749,'EPSG','9110','EPSG','8802','Longitude of natural origin',35.124349,'EPSG','9110','EPSG','8806','False easting',170251.555,'EPSG','9001','EPSG','8807','False northing',1126867.909,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12822','conversion','EPSG','18203','EPSG','2603','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18204','Israeli TM','Designed to approximate Israeli CRS grid in north-central Israel.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',31.4403817,'EPSG','9110','EPSG','8802','Longitude of natural origin',35.1216261,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0000067,'EPSG','9201','EPSG','8806','False easting',219529.584,'EPSG','9001','EPSG','8807','False northing',626907.39,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12823','conversion','EPSG','18204','EPSG','2603','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18205','Finland zone 5','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',33.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',5500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12824','conversion','EPSG','18205','EPSG','3385','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18211','Guatemala Norte','','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',16.49,'EPSG','9110','EPSG','8802','Longitude of natural origin',-90.2,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.99992226,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',292209.579,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12825','conversion','EPSG','18211','EPSG','2120','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18212','Guatemala Sur','','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',14.54,'EPSG','9110','EPSG','8802','Longitude of natural origin',-90.2,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.99989906,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',325992.681,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12826','conversion','EPSG','18212','EPSG','2121','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18221','NGO zone I','Longitude is referenced to the Oslo meridian.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',58.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-4.4,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12827','conversion','EPSG','18221','EPSG','1741','EPSG','1092');
INSERT INTO "conversion" VALUES('EPSG','18222','NGO zone II','Longitude is referenced to the Oslo meridian.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',58.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-2.2,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12828','conversion','EPSG','18222','EPSG','1742','EPSG','1092');
INSERT INTO "conversion" VALUES('EPSG','18223','NGO zone III','Longitude is referenced to the Oslo meridian.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',58.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',0.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12829','conversion','EPSG','18223','EPSG','1743','EPSG','1092');
INSERT INTO "conversion" VALUES('EPSG','18224','NGO zone IV','Longitude is referenced to the Oslo meridian.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',58.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',2.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12830','conversion','EPSG','18224','EPSG','1744','EPSG','1092');
INSERT INTO "conversion" VALUES('EPSG','18225','NGO zone V','Longitude is referenced to the Oslo meridian.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',58.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',6.1,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12831','conversion','EPSG','18225','EPSG','1745','EPSG','1092');
INSERT INTO "conversion" VALUES('EPSG','18226','NGO zone VI','Longitude is referenced to the Oslo meridian.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',58.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',10.1,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12832','conversion','EPSG','18226','EPSG','1746','EPSG','1092');
INSERT INTO "conversion" VALUES('EPSG','18227','NGO zone VII','Longitude is referenced to the Oslo meridian.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',58.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',14.1,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12833','conversion','EPSG','18227','EPSG','1747','EPSG','1092');
INSERT INTO "conversion" VALUES('EPSG','18228','NGO zone VIII','Longitude is referenced to the Oslo meridian.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',58.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',18.2,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12834','conversion','EPSG','18228','EPSG','1748','EPSG','1092');
INSERT INTO "conversion" VALUES('EPSG','18231','India zone I (1975 metres)','BEWARE ! Different yard to metre conversion values have been used in different parts of south Asia. Some areas have changed conversion value with time.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',32.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',68.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.99878641,'EPSG','9201','EPSG','8806','False easting',2743195.5,'EPSG','9001','EPSG','8807','False northing',914398.5,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12835','conversion','EPSG','18231','EPSG','1676','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18232','India zone IIa (1975 metres)','BEWARE ! Different yard to metre conversion values have been used in different parts of south Asia. Some areas have changed conversion value with time.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',26.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',74.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.99878641,'EPSG','9201','EPSG','8806','False easting',2743195.5,'EPSG','9001','EPSG','8807','False northing',914398.5,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12836','conversion','EPSG','18232','EPSG','1677','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18233','India zone IIIa (1975 metres)','BEWARE ! Different yard to metre conversion values have been used in different parts of south Asia. Some areas have changed conversion value with time.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',19.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',80.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.99878641,'EPSG','9201','EPSG','8806','False easting',2743195.5,'EPSG','9001','EPSG','8807','False northing',914398.5,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12837','conversion','EPSG','18233','EPSG','1672','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18234','India zone IVa (1975 metres)','BEWARE ! Different yard to metre conversion values have been used in different parts of south Asia. Some areas have changed conversion value with time.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',12.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',80.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.99878641,'EPSG','9201','EPSG','8806','False easting',2743195.5,'EPSG','9001','EPSG','8807','False northing',914398.5,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12838','conversion','EPSG','18234','EPSG','1673','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18235','India zone IIb (1975 metres)','BEWARE ! Different yard to metre conversion values have been used in different parts of south Asia. Some areas have changed conversion value with time.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',26.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',90.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.99878641,'EPSG','9201','EPSG','8806','False easting',2743195.5,'EPSG','9001','EPSG','8807','False northing',914398.5,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12839','conversion','EPSG','18235','EPSG','1678','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18236','India zone I (1962 metres)','BEWARE ! Different yard to metre conversion values have been used in different parts of south Asia. Some areas have changed conversion value with time.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',32.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',68.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.99878641,'EPSG','9201','EPSG','8806','False easting',2743196.4,'EPSG','9001','EPSG','8807','False northing',914398.8,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12840','conversion','EPSG','18236','EPSG','1685','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18237','India zone IIa (1962 metres)','BEWARE ! Different yard to metre conversion values have been used in different parts of south Asia. Some areas have changed conversion value with time.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',26.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',74.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.99878641,'EPSG','9201','EPSG','8806','False easting',2743196.4,'EPSG','9001','EPSG','8807','False northing',914398.8,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12841','conversion','EPSG','18237','EPSG','1686','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18238','India zone IIb (1937 metres)','BEWARE ! Different yard to metre conversion values have been used in different parts of south Asia. Some areas have changed conversion value with time.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',26.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',90.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.99878641,'EPSG','9201','EPSG','8806','False easting',2743185.69,'EPSG','9001','EPSG','8807','False northing',914395.23,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12842','conversion','EPSG','18238','EPSG','3217','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18240','Libya zone 5','Replaced by Libya TM zone 5 (code 18310).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',9.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12843','conversion','EPSG','18240','EPSG','1470','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18241','Libya zone 6','Replaced by Libya TM zone 6 (code 18311).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',11.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12844','conversion','EPSG','18241','EPSG','1471','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18242','Libya zone 7','Replaced by Libya TM zone 7 (code 18312).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',13.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12845','conversion','EPSG','18242','EPSG','1472','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18243','Libya zone 8','Replaced by Libya TM zone 8 (code 18313).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',15.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12846','conversion','EPSG','18243','EPSG','1473','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18244','Libya zone 9','Replaced by Libya TM zone 9 (code 18314).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',17.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12847','conversion','EPSG','18244','EPSG','1474','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18245','Libya zone 10','Replaced by Libya TM zone 10 (code 18315).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',19.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12848','conversion','EPSG','18245','EPSG','1475','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18246','Libya zone 11','Replaced by Libya TM zone 11 (code 18316).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',21.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12849','conversion','EPSG','18246','EPSG','1476','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18247','Libya zone 12','Replaced by Libya TM zone 12 (code 18317).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',23.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12850','conversion','EPSG','18247','EPSG','1477','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18248','Libya zone 13','Replaced by Libya TM zone 13 (code 18318).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',25.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12851','conversion','EPSG','18248','EPSG','1478','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18251','Korea East Belt','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',38.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',129.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12852','conversion','EPSG','18251','EPSG','3726','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18252','Korea Central Belt','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',38.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',127.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12853','conversion','EPSG','18252','EPSG','3716','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18253','Korea West Belt','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',38.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',125.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12854','conversion','EPSG','18253','EPSG','3713','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18260','Maracaibo Grid (M1)','Grid coordinates are (0 0) at Maracaibo Cathedral.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',10.1,'EPSG','9110','EPSG','8802','Longitude of natural origin',-71.3620224,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',-52684.972,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12855','conversion','EPSG','18260','EPSG','1319','EPSG','1136');
INSERT INTO "conversion" VALUES('EPSG','18261','Maracaibo Grid','Grid coordinates are (200000 200000) at Maracaibo Cathedral.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',10.1,'EPSG','9110','EPSG','8802','Longitude of natural origin',-71.3620224,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',147315.028,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12856','conversion','EPSG','18261','EPSG','1319','EPSG','1136');
INSERT INTO "conversion" VALUES('EPSG','18262','Maracaibo Grid (M3)','Grid coordinates are (500000 500000) at Maracaibo Cathedral.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',10.1,'EPSG','9110','EPSG','8802','Longitude of natural origin',-71.3620224,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',447315.028,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12857','conversion','EPSG','18262','EPSG','1319','EPSG','1136');
INSERT INTO "conversion" VALUES('EPSG','18263','Maracaibo La Rosa Grid','Grid coordinates are (-17044E 29545N) at Maracaibo Cathedral.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',10.1,'EPSG','9110','EPSG','8802','Longitude of natural origin',-71.3620224,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',-17044.0,'EPSG','9001','EPSG','8807','False northing',-23139.97,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12858','conversion','EPSG','18263','EPSG','1319','EPSG','1136');
INSERT INTO "conversion" VALUES('EPSG','18275','Balkans zone 5','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',15.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',5500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12859','conversion','EPSG','18275','EPSG','1709','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18276','Balkans zone 6','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',18.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',6500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12860','conversion','EPSG','18276','EPSG','1710','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18277','Balkans zone 7','In Macedonia replaced by MSCS (map projection code 6203) from 2008 but from 2013 this projection used for AREC electronic cadastral database (E-kat) across whole country.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',21.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',7500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12861','conversion','EPSG','18277','EPSG','1711','EPSG','1142');
INSERT INTO "usage" VALUES('EPSG','17355','conversion','EPSG','18277','EPSG','1148','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','18278','Balkans zone 8','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',24.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',8500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12862','conversion','EPSG','18278','EPSG','1712','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','18280','Poland zone I','','EPSG','9809','Oblique Stereographic','EPSG','8801','Latitude of natural origin',50.373,'EPSG','9110','EPSG','8802','Longitude of natural origin',21.05,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9998,'EPSG','9201','EPSG','8806','False easting',4637000.0,'EPSG','9001','EPSG','8807','False northing',5467000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12863','conversion','EPSG','18280','EPSG','1515','EPSG','1102');
INSERT INTO "conversion" VALUES('EPSG','18281','Poland zone I','','EPSG','9809','Oblique Stereographic','EPSG','8801','Latitude of natural origin',50.373,'EPSG','9110','EPSG','8802','Longitude of natural origin',21.05,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9998,'EPSG','9201','EPSG','8806','False easting',4637000.0,'EPSG','9001','EPSG','8807','False northing',5647000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','12864','conversion','EPSG','18281','EPSG','1515','EPSG','1102');
INSERT INTO "conversion" VALUES('EPSG','18282','Poland zone II','','EPSG','9809','Oblique Stereographic','EPSG','8801','Latitude of natural origin',53.0007,'EPSG','9110','EPSG','8802','Longitude of natural origin',21.301,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9998,'EPSG','9201','EPSG','8806','False easting',4603000.0,'EPSG','9001','EPSG','8807','False northing',5806000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12865','conversion','EPSG','18282','EPSG','1516','EPSG','1102');
INSERT INTO "conversion" VALUES('EPSG','18283','Poland zone III','','EPSG','9809','Oblique Stereographic','EPSG','8801','Latitude of natural origin',53.35,'EPSG','9110','EPSG','8802','Longitude of natural origin',17.003,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9998,'EPSG','9201','EPSG','8806','False easting',3501000.0,'EPSG','9001','EPSG','8807','False northing',5999000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12866','conversion','EPSG','18283','EPSG','1517','EPSG','1102');
INSERT INTO "conversion" VALUES('EPSG','18284','Poland zone IV','','EPSG','9809','Oblique Stereographic','EPSG','8801','Latitude of natural origin',51.4015,'EPSG','9110','EPSG','8802','Longitude of natural origin',16.402,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9998,'EPSG','9201','EPSG','8806','False easting',3703000.0,'EPSG','9001','EPSG','8807','False northing',5627000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12867','conversion','EPSG','18284','EPSG','1518','EPSG','1102');
INSERT INTO "conversion" VALUES('EPSG','18285','Poland zone V','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',18.573,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999983,'EPSG','9201','EPSG','8806','False easting',237000.0,'EPSG','9001','EPSG','8807','False northing',-4700000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12868','conversion','EPSG','18285','EPSG','1519','EPSG','1102');
INSERT INTO "conversion" VALUES('EPSG','18286','GUGiK-80','','EPSG','9809','Oblique Stereographic','EPSG','8801','Latitude of natural origin',52.1,'EPSG','9110','EPSG','8802','Longitude of natural origin',19.1,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999714,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12869','conversion','EPSG','18286','EPSG','1192','EPSG','1241');
INSERT INTO "conversion" VALUES('EPSG','18300','Poland CS92','See Poland CS2000 zones (codes 18305-08) for cadastral survey and mapping at larger scales.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',19.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9993,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',-5300000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12870','conversion','EPSG','18300','EPSG','1192','EPSG','1210');
INSERT INTO "conversion" VALUES('EPSG','18305','Poland CS2000 zone 5','See Poland CS92 (code 18300) for mapping at 1:10,000 and smaller scales.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',15.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.999923,'EPSG','9201','EPSG','8806','False easting',5500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12871','conversion','EPSG','18305','EPSG','1520','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','18306','Poland CS2000 zone 6','See Poland CS92 (code 18300) for mapping at 1:10,000 and smaller scales.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',18.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.999923,'EPSG','9201','EPSG','8806','False easting',6500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12872','conversion','EPSG','18306','EPSG','1521','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','18307','Poland CS2000 zone 7','See Poland CS92 (code 18300) for mapping at 1:10,000 and smaller scales.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',21.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.999923,'EPSG','9201','EPSG','8806','False easting',7500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12873','conversion','EPSG','18307','EPSG','1522','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','18308','Poland CS2000 zone 8','See Poland CS92 (code 18300) for mapping at 1:10,000 and smaller scales.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',24.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.999923,'EPSG','9201','EPSG','8806','False easting',8500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12874','conversion','EPSG','18308','EPSG','1523','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','18310','Libya TM zone 5','Replaces Libya zone 5 (code 18240).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',9.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.99995,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12875','conversion','EPSG','18310','EPSG','1470','EPSG','1207');
INSERT INTO "conversion" VALUES('EPSG','18311','Libya TM zone 6','Replaces Libya zone 6 (code 18241).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',11.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.99995,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12876','conversion','EPSG','18311','EPSG','1471','EPSG','1207');
INSERT INTO "conversion" VALUES('EPSG','18312','Libya TM zone 7','Replaces Libya zone 7 (code 18242).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',13.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.99995,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12877','conversion','EPSG','18312','EPSG','1472','EPSG','1207');
INSERT INTO "conversion" VALUES('EPSG','18313','Libya TM zone 8','Replaces Libya zone 8 (code 18243).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',15.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.99995,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12878','conversion','EPSG','18313','EPSG','1473','EPSG','1207');
INSERT INTO "conversion" VALUES('EPSG','18314','Libya TM zone 9','Replaces Libya zone 9 (code 18244).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',17.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.99995,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12879','conversion','EPSG','18314','EPSG','1474','EPSG','1207');
INSERT INTO "conversion" VALUES('EPSG','18315','Libya TM zone 10','Replaces Libya zone 10 (code 18245).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',19.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.99995,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12880','conversion','EPSG','18315','EPSG','1475','EPSG','1207');
INSERT INTO "conversion" VALUES('EPSG','18316','Libya TM zone 11','Replaces Libya zone 11 (code 18246).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',21.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.99995,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12881','conversion','EPSG','18316','EPSG','1476','EPSG','1207');
INSERT INTO "conversion" VALUES('EPSG','18317','Libya TM zone 12','Replaces Libya zone 12 (code 18247).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',23.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.99995,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12882','conversion','EPSG','18317','EPSG','1477','EPSG','1207');
INSERT INTO "conversion" VALUES('EPSG','18318','Libya TM zone 13','Replaces Libya zone 13 (code 18248).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',25.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.99995,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12883','conversion','EPSG','18318','EPSG','1478','EPSG','1207');
INSERT INTO "conversion" VALUES('EPSG','18319','Libya TM','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',17.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9965,'EPSG','9201','EPSG','8806','False easting',1000000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12884','conversion','EPSG','18319','EPSG','1143','EPSG','1241');
INSERT INTO "conversion" VALUES('EPSG','18401','Kp2000 Jylland og Fyn','See UTM zone 32N (code 16032) for mapping at 1:10,000 and smaller scales.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',9.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.99995,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12885','conversion','EPSG','18401','EPSG','2531','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','18402','Kp2000 Sjaelland','See UTM zone 32N (code 16032) for mapping at 1:10,000 and smaller scales.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',12.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.99995,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12886','conversion','EPSG','18402','EPSG','2532','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','18403','Kp2000 Bornholm','See UTM zone 33N (code 16033) for mapping at 1:10,000 and smaller scales.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',15.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',900000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12887','conversion','EPSG','18403','EPSG','2533','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','18411','French West Africa Senegal zone','Replaced in 1950 by UTM.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-13.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999,'EPSG','9201','EPSG','8806','False easting',1000000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12888','conversion','EPSG','18411','EPSG','2548','EPSG','1241');
INSERT INTO "conversion" VALUES('EPSG','18412','French West Africa Ivory Coast zone','Replaced in 1950 by UTM.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-6.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999,'EPSG','9201','EPSG','8806','False easting',1000000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12889','conversion','EPSG','18412','EPSG','2549','EPSG','1241');
INSERT INTO "conversion" VALUES('EPSG','18413','French West Africa Dahomey zone','Replaced in 1950 by UTM.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',0.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999,'EPSG','9201','EPSG','8806','False easting',1000000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12890','conversion','EPSG','18413','EPSG','2550','EPSG','1241');
INSERT INTO "conversion" VALUES('EPSG','18414','French West Africa Niger zone','Replaced in 1950 by UTM.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',7.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999,'EPSG','9201','EPSG','8806','False easting',1000000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12891','conversion','EPSG','18414','EPSG','2551','EPSG','1241');
INSERT INTO "conversion" VALUES('EPSG','18415','French Equatorial Africa west zone','Replaced in 1950 by UTM.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',10.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999,'EPSG','9201','EPSG','8806','False easting',1000000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12892','conversion','EPSG','18415','EPSG','2552','EPSG','1241');
INSERT INTO "conversion" VALUES('EPSG','18416','French Equatorial Africa central zone','Replaced in 1950 by UTM.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',17.4,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999,'EPSG','9201','EPSG','8806','False easting',1000000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12893','conversion','EPSG','18416','EPSG','2553','EPSG','1241');
INSERT INTO "conversion" VALUES('EPSG','18417','French Equatorial Africa east zone','Replaced in 1950 by UTM.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',24.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999,'EPSG','9201','EPSG','8806','False easting',1000000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12894','conversion','EPSG','18417','EPSG','2554','EPSG','1241');
INSERT INTO "conversion" VALUES('EPSG','18421','Greenland zone 1','','EPSG','9826','Lambert Conic Conformal (West Orientated)','EPSG','8801','Latitude of natural origin',82.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-40.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12895','conversion','EPSG','18421','EPSG','2556','EPSG','1153');
INSERT INTO "conversion" VALUES('EPSG','18422','Greenland zone 2 east','','EPSG','9826','Lambert Conic Conformal (West Orientated)','EPSG','8801','Latitude of natural origin',79.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-24.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12896','conversion','EPSG','18422','EPSG','2557','EPSG','1153');
INSERT INTO "conversion" VALUES('EPSG','18423','Greenland zone 3 east','','EPSG','9826','Lambert Conic Conformal (West Orientated)','EPSG','8801','Latitude of natural origin',76.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-20.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12897','conversion','EPSG','18423','EPSG','2558','EPSG','1153');
INSERT INTO "conversion" VALUES('EPSG','18424','Greenland zone 4 east','','EPSG','9826','Lambert Conic Conformal (West Orientated)','EPSG','8801','Latitude of natural origin',73.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-24.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12898','conversion','EPSG','18424','EPSG','2559','EPSG','1153');
INSERT INTO "conversion" VALUES('EPSG','18425','Greenland zone 5 east','','EPSG','9826','Lambert Conic Conformal (West Orientated)','EPSG','8801','Latitude of natural origin',70.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-24.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12899','conversion','EPSG','18425','EPSG','2560','EPSG','1153');
INSERT INTO "conversion" VALUES('EPSG','18426','Greenland zone 6 east','','EPSG','9826','Lambert Conic Conformal (West Orientated)','EPSG','8801','Latitude of natural origin',67.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-32.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12900','conversion','EPSG','18426','EPSG','2561','EPSG','1153');
INSERT INTO "conversion" VALUES('EPSG','18427','Greenland zone 7 east','','EPSG','9826','Lambert Conic Conformal (West Orientated)','EPSG','8801','Latitude of natural origin',64.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-40.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12901','conversion','EPSG','18427','EPSG','2562','EPSG','1153');
INSERT INTO "conversion" VALUES('EPSG','18428','Greenland zone 8','','EPSG','9826','Lambert Conic Conformal (West Orientated)','EPSG','8801','Latitude of natural origin',61.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-48.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12902','conversion','EPSG','18428','EPSG','2569','EPSG','1153');
INSERT INTO "conversion" VALUES('EPSG','18432','Greenland zone 2 west','','EPSG','9826','Lambert Conic Conformal (West Orientated)','EPSG','8801','Latitude of natural origin',79.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-64.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12903','conversion','EPSG','18432','EPSG','2563','EPSG','1153');
INSERT INTO "conversion" VALUES('EPSG','18433','Greenland zone 3 west','','EPSG','9826','Lambert Conic Conformal (West Orientated)','EPSG','8801','Latitude of natural origin',76.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-64.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12904','conversion','EPSG','18433','EPSG','2564','EPSG','1153');
INSERT INTO "conversion" VALUES('EPSG','18434','Greenland zone 4 west','','EPSG','9826','Lambert Conic Conformal (West Orientated)','EPSG','8801','Latitude of natural origin',73.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-52.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12905','conversion','EPSG','18434','EPSG','2565','EPSG','1153');
INSERT INTO "conversion" VALUES('EPSG','18435','Greenland zone 5 west','','EPSG','9826','Lambert Conic Conformal (West Orientated)','EPSG','8801','Latitude of natural origin',70.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-52.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12906','conversion','EPSG','18435','EPSG','2566','EPSG','1153');
INSERT INTO "conversion" VALUES('EPSG','18436','Greenland zone 6 west','','EPSG','9826','Lambert Conic Conformal (West Orientated)','EPSG','8801','Latitude of natural origin',67.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-52.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12907','conversion','EPSG','18436','EPSG','2567','EPSG','1153');
INSERT INTO "conversion" VALUES('EPSG','18437','Greenland zone 7 west','','EPSG','9826','Lambert Conic Conformal (West Orientated)','EPSG','8801','Latitude of natural origin',64.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-52.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12908','conversion','EPSG','18437','EPSG','2568','EPSG','1153');
INSERT INTO "conversion" VALUES('EPSG','18441','CS63 zone A1','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.07,'EPSG','9110','EPSG','8802','Longitude of natural origin',41.32,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',1300000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12909','conversion','EPSG','18441','EPSG','2772','EPSG','1207');
INSERT INTO "conversion" VALUES('EPSG','18442','CS63 zone A2','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.07,'EPSG','9110','EPSG','8802','Longitude of natural origin',44.32,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',2300000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12910','conversion','EPSG','18442','EPSG','2773','EPSG','1207');
INSERT INTO "conversion" VALUES('EPSG','18443','CS63 zone A3','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.07,'EPSG','9110','EPSG','8802','Longitude of natural origin',47.32,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',3300000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12911','conversion','EPSG','18443','EPSG','2774','EPSG','1207');
INSERT INTO "conversion" VALUES('EPSG','18444','CS63 zone A4','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.07,'EPSG','9110','EPSG','8802','Longitude of natural origin',50.32,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',4300000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12912','conversion','EPSG','18444','EPSG','2775','EPSG','1207');
INSERT INTO "conversion" VALUES('EPSG','18446','CS63 zone K2','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.08,'EPSG','9110','EPSG','8802','Longitude of natural origin',50.46,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',2300000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12913','conversion','EPSG','18446','EPSG','2776','EPSG','1207');
INSERT INTO "conversion" VALUES('EPSG','18447','CS63 zone K3','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.08,'EPSG','9110','EPSG','8802','Longitude of natural origin',53.46,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',3300000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12914','conversion','EPSG','18447','EPSG','2777','EPSG','1207');
INSERT INTO "conversion" VALUES('EPSG','18448','CS63 zone K4','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.08,'EPSG','9110','EPSG','8802','Longitude of natural origin',56.46,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',4300000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12915','conversion','EPSG','18448','EPSG','2778','EPSG','1207');
INSERT INTO "conversion" VALUES('EPSG','18450','CS63 zone C0','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.06,'EPSG','9110','EPSG','8802','Longitude of natural origin',21.57,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12916','conversion','EPSG','18450','EPSG','3173','EPSG','1207');
INSERT INTO "conversion" VALUES('EPSG','18451','CS63 zone C1','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.06,'EPSG','9110','EPSG','8802','Longitude of natural origin',24.57,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',1250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12917','conversion','EPSG','18451','EPSG','3174','EPSG','1207');
INSERT INTO "conversion" VALUES('EPSG','18452','CS63 zone C2','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.06,'EPSG','9110','EPSG','8802','Longitude of natural origin',27.57,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',2250000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12918','conversion','EPSG','18452','EPSG','3175','EPSG','1207');
INSERT INTO "conversion" VALUES('EPSG','19838','Rectified Skew Orthomorphic Sarawak LSD (metre)','Used by the Sarawak Land and Survey Department. Conversion 19958 but using Hotine Oblique Mercator (variant A) and with FE increased by 2,000,000 m and FN by 5,000,000 m. If using variant B method (code 9815) FE = 2590476.871 m and FN = 5442857.653 m.','EPSG','9812','Hotine Oblique Mercator (variant A)','EPSG','8811','Latitude of projection centre',4.0,'EPSG','9110','EPSG','8812','Longitude of projection centre',115.0,'EPSG','9110','EPSG','8813','Azimuth at projection centre',53.18569537,'EPSG','9110','EPSG','8814','Angle from Rectified to Skew Grid',53.07483685,'EPSG','9110','EPSG','8815','Scale factor at projection centre',0.99984,'EPSG','9201','EPSG','8806','False easting',2000000.0,'EPSG','9001','EPSG','8807','False northing',5000000.0,'EPSG','9001',0);
INSERT INTO "usage" VALUES('EPSG','14399','conversion','EPSG','19838','EPSG','4611','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19839','Dubai Local Transverse Mercator','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',55.2,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12919','conversion','EPSG','19839','EPSG','3531','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','19840','IBCAO Polar Stereographic','Used for the International Bathymetric Chart of Arctic Ocean.','EPSG','9829','Polar Stereographic (variant B)','EPSG','8832','Latitude of standard parallel',75.0,'EPSG','9102','EPSG','8833','Longitude of origin',0.0,'EPSG','9102','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12920','conversion','EPSG','19840','EPSG','1996','EPSG','1201');
INSERT INTO "conversion" VALUES('EPSG','19841','Swiss Oblique Mercator 1903C (Greenwich)','Greenwich-based equivalent of LV03C, proj code 19923.','EPSG','9815','Hotine Oblique Mercator (variant B)','EPSG','8811','Latitude of projection centre',46.570866,'EPSG','9110','EPSG','8812','Longitude of projection centre',7.26225,'EPSG','9110','EPSG','8813','Azimuth at projection centre',90.0,'EPSG','9110','EPSG','8814','Angle from Rectified to Skew Grid',90.0,'EPSG','9110','EPSG','8815','Scale factor at projection centre',1.0,'EPSG','9201','EPSG','8816','Easting at projection centre',0.0,'EPSG','9001','EPSG','8817','Northing at projection centre',0.0,'EPSG','9001',0);
INSERT INTO "usage" VALUES('EPSG','12921','conversion','EPSG','19841','EPSG','1144','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19842','Arctic Polar Stereographic','Mirror of Antarctic Polar Stereographic (proj code 19992).','EPSG','9829','Polar Stereographic (variant B)','EPSG','8832','Latitude of standard parallel',71.0,'EPSG','9102','EPSG','8833','Longitude of origin',0.0,'EPSG','9102','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12922','conversion','EPSG','19842','EPSG','1996','EPSG','1040');
INSERT INTO "conversion" VALUES('EPSG','19843','Mercator 41','For projects extending within the 12 nautical mile line or onshore, NIWA recommends using NZCS2000 (projection code 17964). For offshore statistical analysis see NIWA Albers (projection code 9190).','EPSG','9805','Mercator (variant B)','EPSG','8823','Latitude of 1st standard parallel',-41.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',100.0,'EPSG','9102','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12923','conversion','EPSG','19843','EPSG','3508','EPSG','1210');
INSERT INTO "conversion" VALUES('EPSG','19844','Ministry of Transport of Quebec Lambert','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',44.0,'EPSG','9102','EPSG','8822','Longitude of false origin',-70.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',50.0,'EPSG','9102','EPSG','8824','Latitude of 2nd standard parallel',46.0,'EPSG','9102','EPSG','8826','Easting at false origin',800000.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12924','conversion','EPSG','19844','EPSG','1368','EPSG','1220');
INSERT INTO "conversion" VALUES('EPSG','19845','Slovene National Grid','Replaced Slovenia Grid soon after Slovenian independence.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',15.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',-5000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12925','conversion','EPSG','19845','EPSG','1212','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19846','World Equidistant Cylindrical','Origin at intersection of equator and prime meridian. See projection code 19968 for spherical development.','EPSG','9842','Equidistant Cylindrical','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8822','Longitude of false origin',0.0,'EPSG','9102','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','12926','conversion','EPSG','19846','EPSG','1262','EPSG','1192');
INSERT INTO "conversion" VALUES('EPSG','19847','Popular Visualisation Mercator','Uses spherical development. Compared to ellipsoidal development errors of up to 800 metres may arise.','EPSG','9841','Mercator (1SP) (Spherical)','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',0.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','12927','conversion','EPSG','19847','EPSG','1262','EPSG','1098');
INSERT INTO "conversion" VALUES('EPSG','19848','Pitcairn TM 2006','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-25.04067894,'EPSG','9110','EPSG','8802','Longitude of natural origin',-130.06466816,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',14200.0,'EPSG','9001','EPSG','8807','False northing',15500.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12928','conversion','EPSG','19848','EPSG','3208','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','19849','Bermuda 2000 National Grid','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',32.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-64.45,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',550000.0,'EPSG','9001','EPSG','8807','False northing',100000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12929','conversion','EPSG','19849','EPSG','1047','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','19850','EPSG vertical perspective example','Example only.','EPSG','9838','Vertical Perspective','EPSG','8834','Latitude of topocentric origin',55.0,'EPSG','9102','EPSG','8835','Longitude of topocentric origin',5.0,'EPSG','9102','EPSG','8836','Ellipsoidal height of topocentric origin',200.0,'EPSG','9001','EPSG','8840','Viewpoint height',5900.0,'EPSG','9036',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12930','conversion','EPSG','19850','EPSG','1263','EPSG','1030');
INSERT INTO "conversion" VALUES('EPSG','19851','Croatia Transverse Mercator','Croatia LCC (code 19852) used for medium and small scale mapping.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',16.5,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12931','conversion','EPSG','19851','EPSG','1076','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','19852','Croatia Lambert Conformal Conic','Croatia TM (code 19851) used for cadastre, large scale topographic mapping and engineering survey.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',0.0,'EPSG','9102','EPSG','8822','Longitude of false origin',16.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',45.55,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',43.05,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12932','conversion','EPSG','19852','EPSG','1076','EPSG','1210');
INSERT INTO "conversion" VALUES('EPSG','19853','Portugual TM06','Applied to ETRS89.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',39.400573,'EPSG','9110','EPSG','8802','Longitude of natural origin',-8.075919,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12933','conversion','EPSG','19853','EPSG','1294','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19854','South Georgia Lambert','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-55.0,'EPSG','9102','EPSG','8822','Longitude of false origin',-37.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',-54.0,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-54.45,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12934','conversion','EPSG','19854','EPSG','3529','EPSG','1189');
INSERT INTO "conversion" VALUES('EPSG','19855','Mercator 41','','EPSG','9804','Mercator (variant A)','EPSG','8801','Latitude of natural origin',-41.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',100.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','12935','conversion','EPSG','19855','EPSG','3508','EPSG','1210');
INSERT INTO "conversion" VALUES('EPSG','19856','TM Reunion','Replaces Gauss Laborde Reunion.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-21.07,'EPSG','9110','EPSG','8802','Longitude of natural origin',55.32,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',160000.0,'EPSG','9001','EPSG','8807','False northing',50000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12936','conversion','EPSG','19856','EPSG','3337','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19857','Northwest Territories Lambert','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',0.0,'EPSG','9102','EPSG','8822','Longitude of false origin',-112.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',62.0,'EPSG','9102','EPSG','8824','Latitude of 2nd standard parallel',70.0,'EPSG','9102','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12937','conversion','EPSG','19857','EPSG','3481','EPSG','1238');
INSERT INTO "conversion" VALUES('EPSG','19858','Yukon Albers','','EPSG','9822','Albers Equal Area','EPSG','8821','Latitude of false origin',59.0,'EPSG','9110','EPSG','8822','Longitude of false origin',-132.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',61.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',68.0,'EPSG','9110','EPSG','8826','Easting at false origin',500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12938','conversion','EPSG','19858','EPSG','2417','EPSG','1238');
INSERT INTO "conversion" VALUES('EPSG','19859','Fiji Map Grid','Replaces Viti Levu Grid and Vanua Levu Grid (codes 19878-79).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-17.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',178.45,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.99985,'EPSG','9201','EPSG','8806','False easting',2000000.0,'EPSG','9001','EPSG','8807','False northing',4000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12939','conversion','EPSG','19859','EPSG','1094','EPSG','1056');
INSERT INTO "conversion" VALUES('EPSG','19860','Jamaica Metric Grid 2001','Replaces Jamaica National Grid (proj code 19910).','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',18.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-77.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',750000.0,'EPSG','9001','EPSG','8807','False northing',650000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12940','conversion','EPSG','19860','EPSG','3342','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19861','Laborde Grid','Longitude is referenced to the Paris meridian. See Laborde Grid (Greenwich) (projection code 8440) for equivalent in degrees referenced to Greenwich. Within a few hundred km of origin, may be approximated by Oblique Mercator method - see proj code 19911.','EPSG','9813','Laborde Oblique Mercator','EPSG','8811','Latitude of projection centre',-21.0,'EPSG','9105','EPSG','8812','Longitude of projection centre',49.0,'EPSG','9105','EPSG','8813','Azimuth at projection centre',21.0,'EPSG','9105','EPSG','8815','Scale factor at projection centre',0.9995,'EPSG','9201','EPSG','8806','False easting',400000.0,'EPSG','9001','EPSG','8807','False northing',800000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12941','conversion','EPSG','19861','EPSG','1149','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19862','Belgian Lambert 2005','Introduced in 2005.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',50.4752134,'EPSG','9110','EPSG','8822','Longitude of false origin',4.2133177,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',49.5,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',51.1,'EPSG','9110','EPSG','8826','Easting at false origin',150328.0,'EPSG','9001','EPSG','8827','Northing at false origin',166262.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12942','conversion','EPSG','19862','EPSG','1347','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19863','South China Sea Lambert','Originally defined with FN=500000 at true origin. This is at 21° 00'' 37.0619"N. But operators assumed that it was at 21°N exactly. Thus by common practice Nf=500000 at 21°N exactly. The potential ambiguity is 1138m.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',21.0,'EPSG','9102','EPSG','8822','Longitude of false origin',114.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',18.0,'EPSG','9102','EPSG','8824','Latitude of 2nd standard parallel',24.0,'EPSG','9102','EPSG','8826','Easting at false origin',500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12943','conversion','EPSG','19863','EPSG','3470','EPSG','1136');
INSERT INTO "conversion" VALUES('EPSG','19864','Singapore Transverse Mercator','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',1.22,'EPSG','9110','EPSG','8802','Longitude of natural origin',103.5,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',28001.642,'EPSG','9001','EPSG','8807','False northing',38744.572,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12944','conversion','EPSG','19864','EPSG','1210','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','19865','US NSIDC Sea Ice polar stereographic north','Used in polar research.','EPSG','9829','Polar Stereographic (variant B)','EPSG','8832','Latitude of standard parallel',70.0,'EPSG','9102','EPSG','8833','Longitude of origin',-45.0,'EPSG','9102','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12945','conversion','EPSG','19865','EPSG','1996','EPSG','1110');
INSERT INTO "conversion" VALUES('EPSG','19866','US NSIDC Sea Ice polar stereographic south','Used in polar research.','EPSG','9829','Polar Stereographic (variant B)','EPSG','8832','Latitude of standard parallel',-70.0,'EPSG','9102','EPSG','8833','Longitude of origin',0.0,'EPSG','9102','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12946','conversion','EPSG','19866','EPSG','1997','EPSG','1110');
INSERT INTO "conversion" VALUES('EPSG','19867','US NSIDC Equal Area north projection','See information source for equations to define Equal-Area Scalable Earth Grid (EASE-Grid) overlay.','EPSG','9821','Lambert Azimuthal Equal Area (Spherical)','EPSG','8828','Spherical latitude of origin',90.0,'EPSG','9102','EPSG','8829','Spherical longitude of origin',0.0,'EPSG','9102','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','12947','conversion','EPSG','19867','EPSG','1996','EPSG','1195');
INSERT INTO "conversion" VALUES('EPSG','19868','US NSIDC Equal Area south projection','See information source for equations to define Equal-Area Scalable Earth Grid (EASE-Grid) overlay.','EPSG','9821','Lambert Azimuthal Equal Area (Spherical)','EPSG','8828','Spherical latitude of origin',-90.0,'EPSG','9102','EPSG','8829','Spherical longitude of origin',0.0,'EPSG','9102','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','12948','conversion','EPSG','19868','EPSG','1997','EPSG','1195');
INSERT INTO "conversion" VALUES('EPSG','19869','US NSIDC Equal Area global projection','See information source for equations to define Equal-Area Scalable Earth Grid (EASE-Grid) overlay.','EPSG','9834','Lambert Cylindrical Equal Area (Spherical)','EPSG','8823','Latitude of 1st standard parallel',30.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',0.0,'EPSG','9102','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12949','conversion','EPSG','19869','EPSG','3463','EPSG','1195');
INSERT INTO "conversion" VALUES('EPSG','19870','Faroe Lambert','','EPSG','9826','Lambert Conic Conformal (West Orientated)','EPSG','8801','Latitude of natural origin',62.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-9.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12950','conversion','EPSG','19870','EPSG','3248','EPSG','1091');
INSERT INTO "conversion" VALUES('EPSG','19871','Rectified Skew Orthomorphic Malaya Grid (chain)','If using Hotine Oblique Mercator (variant B) method (code 9815), Ec=23505.515 chSe(T), Nc=21992.646 chSe(T).','EPSG','9812','Hotine Oblique Mercator (variant A)','EPSG','8811','Latitude of projection centre',4.0,'EPSG','9110','EPSG','8812','Longitude of projection centre',102.15,'EPSG','9110','EPSG','8813','Azimuth at projection centre',323.01328458,'EPSG','9110','EPSG','8814','Angle from Rectified to Skew Grid',323.07483685,'EPSG','9110','EPSG','8815','Scale factor at projection centre',0.99984,'EPSG','9201','EPSG','8806','False easting',40000.0,'EPSG','9301','EPSG','8807','False northing',0.0,'EPSG','9301',0);
INSERT INTO "usage" VALUES('EPSG','12951','conversion','EPSG','19871','EPSG','1690','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19872','Rectified Skew Orthomorphic Malaya Grid (metre)','Uses metric conversion factor of 0.914398 metres per yard exactly. If using Hotine Oblique Mercator (variant B) method (code 9815), Ec=472854.710m, Nc=442420.693m.','EPSG','9812','Hotine Oblique Mercator (variant A)','EPSG','8811','Latitude of projection centre',4.0,'EPSG','9110','EPSG','8812','Longitude of projection centre',102.15,'EPSG','9110','EPSG','8813','Azimuth at projection centre',323.01328458,'EPSG','9110','EPSG','8814','Angle from Rectified to Skew Grid',323.07483685,'EPSG','9110','EPSG','8815','Scale factor at projection centre',0.99984,'EPSG','9201','EPSG','8806','False easting',804670.24,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',0);
INSERT INTO "usage" VALUES('EPSG','12952','conversion','EPSG','19872','EPSG','1690','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19873','Noumea Lambert','Applications unable to define parameter values in decimal seconds should use the Noumea Lambert 2 projection which gives the same conversion results.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-22.16108903,'EPSG','9110','EPSG','8822','Longitude of false origin',166.26327327,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',-22.14408903,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-22.17408903,'EPSG','9110','EPSG','8826','Easting at false origin',0.66,'EPSG','9001','EPSG','8827','Northing at false origin',1.02,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12953','conversion','EPSG','19873','EPSG','2823','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','19874','Noumea Lambert 2','Variant of Noumea Lambert (proj code 19873) defined for applications unable to define parameter values in decimal seconds. Gives same conversion results.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-22.1611,'EPSG','9110','EPSG','8822','Longitude of false origin',166.2633,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',-22.1441,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-22.1741,'EPSG','9110','EPSG','8826','Easting at false origin',8.313,'EPSG','9001','EPSG','8827','Northing at false origin',-2.354,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12954','conversion','EPSG','19874','EPSG','2823','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','19875','Ontario MNR Lambert','One of a number of similar projections used by Ontario MNR.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',0.0,'EPSG','9102','EPSG','8822','Longitude of false origin',-85.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',44.5,'EPSG','9102','EPSG','8824','Latitude of 2nd standard parallel',53.5,'EPSG','9102','EPSG','8826','Easting at false origin',930000.0,'EPSG','9001','EPSG','8827','Northing at false origin',6430000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12955','conversion','EPSG','19875','EPSG','1367','EPSG','1220');
INSERT INTO "conversion" VALUES('EPSG','19876','ST74','Replaced by County ST74 (proj code 3853).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',18.0328044,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.99999425,'EPSG','9201','EPSG','8806','False easting',100178.1808,'EPSG','9001','EPSG','8807','False northing',-6500614.7836,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12956','conversion','EPSG','19876','EPSG','3408','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','19877','Faroe Lambert fk89','','EPSG','9826','Lambert Conic Conformal (West Orientated)','EPSG','8801','Latitude of natural origin',62.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-9.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',700000.0,'EPSG','9001','EPSG','8807','False northing',700000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12957','conversion','EPSG','19877','EPSG','3248','EPSG','1028');
INSERT INTO "conversion" VALUES('EPSG','19878','Vanua Levu Grid','Original definition is in chains (1 chain = 100 links).','EPSG','9833','Hyperbolic Cassini-Soldner','EPSG','8801','Latitude of natural origin',-16.15,'EPSG','9110','EPSG','8802','Longitude of natural origin',179.2,'EPSG','9110','EPSG','8806','False easting',1251331.8,'EPSG','9098','EPSG','8807','False northing',1662888.5,'EPSG','9098',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12958','conversion','EPSG','19878','EPSG','3401','EPSG','1056');
INSERT INTO "conversion" VALUES('EPSG','19879','Viti Levu Grid','Original definition is in chains (1 chain = 100 links).','EPSG','9806','Cassini-Soldner','EPSG','8801','Latitude of natural origin',-18.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',178.0,'EPSG','9102','EPSG','8806','False easting',544000.0,'EPSG','9098','EPSG','8807','False northing',704000.0,'EPSG','9098',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12959','conversion','EPSG','19879','EPSG','3195','EPSG','1056');
INSERT INTO "conversion" VALUES('EPSG','19880','Fiji Map Grid','Supersedes Viti Levu Grid and Vanua Levu Grid (codes 19878-79).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-17.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',178.45,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.99985,'EPSG','9001','EPSG','8806','False easting',2000000.0,'EPSG','9001','EPSG','8807','False northing',4000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','12960','conversion','EPSG','19880','EPSG','1094','EPSG','1056');
INSERT INTO "conversion" VALUES('EPSG','19881','Alberta 10-degree TM (Forest)','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-115.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9992,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12961','conversion','EPSG','19881','EPSG','2376','EPSG','1220');
INSERT INTO "conversion" VALUES('EPSG','19882','Alberta 10-degree TM (Resource)','Has negative easting coordinates in western Alberta. For an alternative with positive coordinates see Alberta 10-degree TM (Forest) (code 19881).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-115.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9992,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12962','conversion','EPSG','19882','EPSG','2376','EPSG','1220');
INSERT INTO "conversion" VALUES('EPSG','19883','World Mercator','','EPSG','9804','Mercator (variant A)','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',0.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12963','conversion','EPSG','19883','EPSG','3391','EPSG','1228');
INSERT INTO "conversion" VALUES('EPSG','19884','Caspian Sea Mercator','','EPSG','9805','Mercator (variant B)','EPSG','8823','Latitude of 1st standard parallel',42.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',51.0,'EPSG','9102','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12964','conversion','EPSG','19884','EPSG','1291','EPSG','1198');
INSERT INTO "conversion" VALUES('EPSG','19885','Kelantan Grid','Origin is station P243 at B. Polis Melor, Kota Bahuru. Offset from old grid origin: 13227.851m east, 8739.894m north.','EPSG','9806','Cassini-Soldner','EPSG','8801','Latitude of natural origin',5.582115717,'EPSG','9110','EPSG','8802','Longitude of natural origin',102.174287001,'EPSG','9110','EPSG','8806','False easting',13227.851,'EPSG','9001','EPSG','8807','False northing',8739.894,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12965','conversion','EPSG','19885','EPSG','3384','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19886','Perak Grid','Origin is station TG26 at Gunung Larut Hiijau, Taiping. Offset from old grid origin is -1.769m east, 0.994m north.','EPSG','9806','Cassini-Soldner','EPSG','8801','Latitude of natural origin',4.513262688,'EPSG','9110','EPSG','8802','Longitude of natural origin',100.485547811,'EPSG','9110','EPSG','8806','False easting',-1.769,'EPSG','9001','EPSG','8807','False northing',133454.779,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12966','conversion','EPSG','19886','EPSG','3383','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19887','Kedah and Perlis Grid','Origin is station TG35 at Gunung Perak, Kuala Muda.','EPSG','9806','Cassini-Soldner','EPSG','8801','Latitude of natural origin',5.575282177,'EPSG','9110','EPSG','8802','Longitude of natural origin',100.3810936,'EPSG','9110','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12967','conversion','EPSG','19887','EPSG','3382','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19888','Pinang Grid','Origin is station P314 at TLDM Georgetown. Offset from old grid origin is -23.414m east, 62.2832m north.','EPSG','9806','Cassini-Soldner','EPSG','8801','Latitude of natural origin',5.251746315,'EPSG','9110','EPSG','8802','Longitude of natural origin',100.203975707,'EPSG','9110','EPSG','8806','False easting',-23.414,'EPSG','9001','EPSG','8807','False northing',62.283,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12968','conversion','EPSG','19888','EPSG','3381','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19889','Terengganu Grid','Origin is station P253 at Kg. Matang, Hulu Terenganu. Offset from old grid origin: 19594.245m east, 3371.895m north.','EPSG','9806','Cassini-Soldner','EPSG','8801','Latitude of natural origin',4.583462672,'EPSG','9110','EPSG','8802','Longitude of natural origin',103.041299225,'EPSG','9110','EPSG','8806','False easting',19594.245,'EPSG','9001','EPSG','8807','False northing',3371.895,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12969','conversion','EPSG','19889','EPSG','3380','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19890','Selangor Grid','Origin is station 251D at Felda Soeharto, K. Kuba Baharu. Offset from old grid origin is -13076.704m east, 503.095m north.','EPSG','9806','Cassini-Soldner','EPSG','8801','Latitude of natural origin',3.410473658,'EPSG','9110','EPSG','8802','Longitude of natural origin',101.232078849,'EPSG','9110','EPSG','8806','False easting',-34836.161,'EPSG','9001','EPSG','8807','False northing',56464.049,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12970','conversion','EPSG','19890','EPSG','3379','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19891','Pahang Grid','Origin is station GP31 at Sek. Ren. Keb. Kuala Mai, Jerantut. Offset from old grid origin is -7368.228m east, 6485.858m north.','EPSG','9806','Cassini-Soldner','EPSG','8801','Latitude of natural origin',3.460979712,'EPSG','9110','EPSG','8802','Longitude of natural origin',102.220587634,'EPSG','9110','EPSG','8806','False easting',-7368.228,'EPSG','9001','EPSG','8807','False northing',6485.858,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12971','conversion','EPSG','19891','EPSG','3378','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19892','Sembilan and Melaka Grid','Origin is station GP10 at K. Perindustrian Senawang, Seremban. Offset from old grid origin: 3915.790m east, -3292.026m north.','EPSG','9806','Cassini-Soldner','EPSG','8801','Latitude of natural origin',2.405645149,'EPSG','9110','EPSG','8802','Longitude of natural origin',101.582965815,'EPSG','9110','EPSG','8806','False easting',3673.785,'EPSG','9001','EPSG','8807','False northing',-4240.573,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12972','conversion','EPSG','19892','EPSG','3377','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19893','Johor Grid','Origin is station GP58 at Institut Haiwan, Kluang. Offset from old grid origin is -14810.562m east, 8758.320m north.','EPSG','9806','Cassini-Soldner','EPSG','8801','Latitude of natural origin',2.071804708,'EPSG','9110','EPSG','8802','Longitude of natural origin',103.254057045,'EPSG','9110','EPSG','8806','False easting',-14810.562,'EPSG','9001','EPSG','8807','False northing',8758.32,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12973','conversion','EPSG','19893','EPSG','3376','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19894','Borneo RSO','Replaces RSO Borneo (proj codes 19956-58) for use with ITRF-based geodetic CRSs. If using Hotine Oblique Mercator (variant B) method (code 9815), Ec=590521.147 m, Nc=442890.861 m.','EPSG','9812','Hotine Oblique Mercator (variant A)','EPSG','8811','Latitude of projection centre',4.0,'EPSG','9110','EPSG','8812','Longitude of projection centre',115.0,'EPSG','9110','EPSG','8813','Azimuth at projection centre',53.185691582,'EPSG','9110','EPSG','8814','Angle from Rectified to Skew Grid',53.07483685,'EPSG','9110','EPSG','8815','Scale factor at projection centre',0.99984,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',0);
INSERT INTO "usage" VALUES('EPSG','12974','conversion','EPSG','19894','EPSG','1362','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19895','Peninsular RSO','If using Hotine Oblique Mercator (variant B) method (code 9815), Ec=472830.426 m, Nc=442454.099 m.','EPSG','9812','Hotine Oblique Mercator (variant A)','EPSG','8811','Latitude of projection centre',4.0,'EPSG','9110','EPSG','8812','Longitude of projection centre',102.15,'EPSG','9110','EPSG','8813','Azimuth at projection centre',323.013286728,'EPSG','9110','EPSG','8814','Angle from Rectified to Skew Grid',323.07483685,'EPSG','9110','EPSG','8815','Scale factor at projection centre',0.99984,'EPSG','9201','EPSG','8806','False easting',804671.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',0);
INSERT INTO "usage" VALUES('EPSG','12975','conversion','EPSG','19895','EPSG','3955','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19896','Hong Kong 1963 Grid','Replaced by HK1980 Grid.','EPSG','9806','Cassini-Soldner','EPSG','8801','Latitude of natural origin',22.184368,'EPSG','9110','EPSG','8802','Longitude of natural origin',114.10428,'EPSG','9110','EPSG','8806','False easting',132033.92,'EPSG','9005','EPSG','8807','False northing',62565.96,'EPSG','9005',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12976','conversion','EPSG','19896','EPSG','1118','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','19897','Statistics Canada Lambert','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',63.390675,'EPSG','9102','EPSG','8822','Longitude of false origin',-91.52,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',49.0,'EPSG','9102','EPSG','8824','Latitude of 2nd standard parallel',77.0,'EPSG','9102','EPSG','8826','Easting at false origin',6200000.0,'EPSG','9001','EPSG','8827','Northing at false origin',3000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12977','conversion','EPSG','19897','EPSG','1061','EPSG','1241');
INSERT INTO "conversion" VALUES('EPSG','19898','Pacific Disaster Center Mercator','','EPSG','9804','Mercator (variant A)','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-150.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','12978','conversion','EPSG','19898','EPSG','3172','EPSG','1241');
INSERT INTO "conversion" VALUES('EPSG','19899','Mauritius Grid','','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',-20.114225,'EPSG','9110','EPSG','8802','Longitude of natural origin',57.311858,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',1000000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12979','conversion','EPSG','19899','EPSG','3209','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19900','Bahrain State Grid','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',51.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12980','conversion','EPSG','19900','EPSG','1040','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19901','Belge Lambert 50','Longitude is referenced to the Brussels meridian. If software cannot handle latitude of false origin of 90°N, use latitude of false origin = 50°30''00.0"N with northing at false origin = 131983.890 m.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',90.0,'EPSG','9110','EPSG','8822','Longitude of false origin',0.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',49.5,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',51.1,'EPSG','9110','EPSG','8826','Easting at false origin',150000.0,'EPSG','9001','EPSG','8827','Northing at false origin',5400000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12981','conversion','EPSG','19901','EPSG','1347','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19902','Belge Lambert 72','Rotation from Belge Lambert 50 to Belge Lambert 72 is +29.2985sec. An equivalent using the conventional Lambert Conic Conformal (2SP) method (Belgian Lambert 72, code 19961) was introduced in 2000.','EPSG','9803','Lambert Conic Conformal (2SP Belgium)','EPSG','8821','Latitude of false origin',90.0,'EPSG','9110','EPSG','8822','Longitude of false origin',4.2124983,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',49.5,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',51.1,'EPSG','9110','EPSG','8826','Easting at false origin',150000.01256,'EPSG','9001','EPSG','8827','Northing at false origin',5400088.4378,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12982','conversion','EPSG','19902','EPSG','1347','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19903','Nord de Guerre','Longitude is referenced to the Paris meridian.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',55.0,'EPSG','9105','EPSG','8802','Longitude of natural origin',6.0,'EPSG','9105','EPSG','8805','Scale factor at natural origin',0.99950908,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',300000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12983','conversion','EPSG','19903','EPSG','1369','EPSG','1215');
INSERT INTO "conversion" VALUES('EPSG','19904','Ghana Metre Grid','Replaces Ghana National Grid (code 19959) from 1978. British foot (Sears 1922) used to convert projection defining parameters.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',4.4,'EPSG','9110','EPSG','8802','Longitude of natural origin',-1.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.99975,'EPSG','9201','EPSG','8806','False easting',274319.51,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12984','conversion','EPSG','19904','EPSG','1104','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19905','Netherlands East Indies Equatorial Zone','','EPSG','9804','Mercator (variant A)','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',110.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.997,'EPSG','9201','EPSG','8806','False easting',3900000.0,'EPSG','9001','EPSG','8807','False northing',900000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12985','conversion','EPSG','19905','EPSG','4020','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19906','Iraq zone','','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',32.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',45.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9987864078,'EPSG','9201','EPSG','8806','False easting',1500000.0,'EPSG','9001','EPSG','8807','False northing',1166200.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12986','conversion','EPSG','19906','EPSG','2294','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19907','Iraq National Grid','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',29.0134566,'EPSG','9110','EPSG','8802','Longitude of natural origin',46.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9994,'EPSG','9201','EPSG','8806','False easting',800000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12987','conversion','EPSG','19907','EPSG','3625','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19908','Irish National Grid','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',53.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-8.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000035,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',250000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','12988','conversion','EPSG','19908','EPSG','1305','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19909','Jamaica (Old Grid)','Replaced by Jamaica National Grid (proj code 19910).','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',18.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-77.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',550000.0,'EPSG','9005','EPSG','8807','False northing',400000.0,'EPSG','9005',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12989','conversion','EPSG','19909','EPSG','3342','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19910','Jamaica National Grid','Replaces Jamaica (Old Grid) (proj code 19909). Replaced by Jamaica Metric Grid 2001 (proj code 19860).','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',18.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-77.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',250000.0,'EPSG','9001','EPSG','8807','False northing',150000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12990','conversion','EPSG','19910','EPSG','3342','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19911','Laborde Grid approximation','Longitude is referenced to the Paris meridian. The conversion method is a good approximation (better than 5cm) to original formula (see proj code 19861) within a few hundred km of origin, but farther away the approximation may be no better than 1m.','EPSG','9815','Hotine Oblique Mercator (variant B)','EPSG','8811','Latitude of projection centre',-21.0,'EPSG','9105','EPSG','8812','Longitude of projection centre',49.0,'EPSG','9105','EPSG','8813','Azimuth at projection centre',21.0,'EPSG','9105','EPSG','8814','Angle from Rectified to Skew Grid',21.0,'EPSG','9105','EPSG','8815','Scale factor at projection centre',0.9995,'EPSG','9201','EPSG','8816','Easting at projection centre',400000.0,'EPSG','9001','EPSG','8817','Northing at projection centre',800000.0,'EPSG','9001',0);
INSERT INTO "usage" VALUES('EPSG','12991','conversion','EPSG','19911','EPSG','1149','EPSG','1211');
INSERT INTO "conversion" VALUES('EPSG','19913','RD Old','','EPSG','9809','Oblique Stereographic','EPSG','8801','Latitude of natural origin',52.0922178,'EPSG','9110','EPSG','8802','Longitude of natural origin',5.23155,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999079,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12992','conversion','EPSG','19913','EPSG','1275','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19914','RD New','','EPSG','9809','Oblique Stereographic','EPSG','8801','Latitude of natural origin',52.0922178,'EPSG','9110','EPSG','8802','Longitude of natural origin',5.23155,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999079,'EPSG','9201','EPSG','8806','False easting',155000.0,'EPSG','9001','EPSG','8807','False northing',463000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12993','conversion','EPSG','19914','EPSG','1275','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19915','Aden Zone','','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',15.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',45.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.999365678,'EPSG','9201','EPSG','8806','False easting',1500000.0,'EPSG','9001','EPSG','8807','False northing',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12994','conversion','EPSG','19915','EPSG','3332','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19916','British National Grid','With the introduction of OSTN15, the area for the British National Grid has effectively been extended from Britain to cover the adjacent UK Continental Shelf.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',49.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-2.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996012717,'EPSG','9201','EPSG','8806','False easting',400000.0,'EPSG','9001','EPSG','8807','False northing',-100000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12995','conversion','EPSG','19916','EPSG','4390','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19917','New Zealand Map Grid','Replaces North and South Island National Grids (codes 18141-2). Used for topographic mapping.','EPSG','9811','New Zealand Map Grid','EPSG','8801','Latitude of natural origin',-41.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',173.0,'EPSG','9102','EPSG','8806','False easting',2510000.0,'EPSG','9001','EPSG','8807','False northing',6023150.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12996','conversion','EPSG','19917','EPSG','3973','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19919','Qatar National Grid','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',24.27,'EPSG','9110','EPSG','8802','Longitude of natural origin',51.13,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.99999,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',300000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12997','conversion','EPSG','19919','EPSG','1195','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19920','Singapore Grid','','EPSG','9806','Cassini-Soldner','EPSG','8801','Latitude of natural origin',1.1715528,'EPSG','9110','EPSG','8802','Longitude of natural origin',103.5110808,'EPSG','9110','EPSG','8806','False easting',30000.0,'EPSG','9001','EPSG','8807','False northing',30000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12998','conversion','EPSG','19920','EPSG','1210','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19921','Spain','Replaced by UTM. Longitude is referenced to the Madrid meridian.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',40.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',0.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9988085293,'EPSG','9201','EPSG','8806','False easting',600000.0,'EPSG','9001','EPSG','8807','False northing',600000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','12999','conversion','EPSG','19921','EPSG','2366','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19922','Swiss Oblique Mercator 1903M','','EPSG','9815','Hotine Oblique Mercator (variant B)','EPSG','8811','Latitude of projection centre',46.570866,'EPSG','9110','EPSG','8812','Longitude of projection centre',7.26225,'EPSG','9110','EPSG','8813','Azimuth at projection centre',90.0,'EPSG','9110','EPSG','8814','Angle from Rectified to Skew Grid',90.0,'EPSG','9110','EPSG','8815','Scale factor at projection centre',1.0,'EPSG','9201','EPSG','8816','Easting at projection centre',600000.0,'EPSG','9001','EPSG','8817','Northing at projection centre',200000.0,'EPSG','9001',0);
INSERT INTO "usage" VALUES('EPSG','13000','conversion','EPSG','19922','EPSG','1286','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19923','Swiss Oblique Mercator 1903C','In Switzerland, replaced by new grid LV03 (proj code 19922). Longitude is referenced to the Bern meridian. See proj code 19841 for Greenwich-based equivalent.','EPSG','9815','Hotine Oblique Mercator (variant B)','EPSG','8811','Latitude of projection centre',46.570866,'EPSG','9110','EPSG','8812','Longitude of projection centre',0.0,'EPSG','9110','EPSG','8813','Azimuth at projection centre',90.0,'EPSG','9110','EPSG','8814','Angle from Rectified to Skew Grid',90.0,'EPSG','9110','EPSG','8815','Scale factor at projection centre',1.0,'EPSG','9201','EPSG','8816','Easting at projection centre',0.0,'EPSG','9001','EPSG','8817','Northing at projection centre',0.0,'EPSG','9001',0);
INSERT INTO "usage" VALUES('EPSG','13001','conversion','EPSG','19923','EPSG','1286','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19924','Tobago Grid','','EPSG','9806','Cassini-Soldner','EPSG','8801','Latitude of natural origin',11.1507843,'EPSG','9110','EPSG','8802','Longitude of natural origin',-60.4109632,'EPSG','9110','EPSG','8806','False easting',187500.0,'EPSG','9039','EPSG','8807','False northing',180000.0,'EPSG','9039',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13002','conversion','EPSG','19924','EPSG','1322','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19925','Trinidad Grid','','EPSG','9806','Cassini-Soldner','EPSG','8801','Latitude of natural origin',10.263,'EPSG','9110','EPSG','8802','Longitude of natural origin',-61.2,'EPSG','9110','EPSG','8806','False easting',430000.0,'EPSG','9039','EPSG','8807','False northing',325000.0,'EPSG','9039',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13003','conversion','EPSG','19925','EPSG','1339','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19926','Stereo 70','Replaces Stereo 33 (code 19927).','EPSG','9809','Oblique Stereographic','EPSG','8801','Latitude of natural origin',46.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',25.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.99975,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13004','conversion','EPSG','19926','EPSG','1197','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19927','Stereo 33','Replaced by Stereo 70 (code 19926)','EPSG','9809','Oblique Stereographic','EPSG','8801','Latitude of natural origin',45.54,'EPSG','9110','EPSG','8802','Longitude of natural origin',25.23328772,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9996667,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13005','conversion','EPSG','19927','EPSG','1197','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19928','Kuwait TM','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',48.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','13006','conversion','EPSG','19928','EPSG','1310','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19929','Sweden zone 2.5 gon V','At the municipal level alternative projections are found defined with different sets of False Northing and Easting, based on 100 km grid squares. This is denoted by the last part of the name.  For example 61:-1 means FN = -6100000 and FE = 100000 m.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',15.48298,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',1500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13007','conversion','EPSG','19929','EPSG','2847','EPSG','1055');
INSERT INTO "usage" VALUES('EPSG','13008','conversion','EPSG','19929','EPSG','3313','EPSG','1210');
INSERT INTO "conversion" VALUES('EPSG','19930','Greek Grid','Created for use with GGRS87.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',24.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13009','conversion','EPSG','19930','EPSG','3254','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19931','Egyseges Orszagos Vetuleti','EOV = Uniform National Projection','EPSG','9815','Hotine Oblique Mercator (variant B)','EPSG','8811','Latitude of projection centre',47.08398174,'EPSG','9110','EPSG','8812','Longitude of projection centre',19.02548584,'EPSG','9110','EPSG','8813','Azimuth at projection centre',90.0,'EPSG','9110','EPSG','8814','Angle from Rectified to Skew Grid',90.0,'EPSG','9110','EPSG','8815','Scale factor at projection centre',0.99993,'EPSG','9201','EPSG','8816','Easting at projection centre',650000.0,'EPSG','9001','EPSG','8817','Northing at projection centre',200000.0,'EPSG','9001',0);
INSERT INTO "usage" VALUES('EPSG','13010','conversion','EPSG','19931','EPSG','1119','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19933','Prince Edward Island Stereographic (ATS77)','In use from 1979. To be phased out in late 1990''s.','EPSG','9809','Oblique Stereographic','EPSG','8801','Latitude of natural origin',47.15,'EPSG','9110','EPSG','8802','Longitude of natural origin',-63.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999912,'EPSG','9201','EPSG','8806','False easting',700000.0,'EPSG','9001','EPSG','8807','False northing',400000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13011','conversion','EPSG','19933','EPSG','1533','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19934','Lithuania 1994','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',24.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9998,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13012','conversion','EPSG','19934','EPSG','1145','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19935','Rectified Skew Orthomorphic Malaya Grid','If using Oblique Mercator method (code 9815), Ec=23505.515 chSe, Nc=21992.646 chSe.','EPSG','9812','Hotine Oblique Mercator (variant A)','EPSG','8811','Latitude of projection centre',4.0,'EPSG','9110','EPSG','8812','Longitude of projection centre',102.15,'EPSG','9110','EPSG','8813','Azimuth at projection centre',323.01328458,'EPSG','9110','EPSG','8814','Angle from Rectified to Skew Grid',323.07483685,'EPSG','9110','EPSG','8815','Scale factor at projection centre',0.99984,'EPSG','9201','EPSG','8806','False easting',40000.0,'EPSG','9062','EPSG','8807','False northing',0.0,'EPSG','9062',1);
INSERT INTO "usage" VALUES('EPSG','13013','conversion','EPSG','19935','EPSG','1690','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19936','Portuguese National Grid','Original transformation by Gauss-Kruger formula. Longitude is referenced to the Lisbon meridian.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',39.4,'EPSG','9110','EPSG','8802','Longitude of natural origin',1.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',300000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13014','conversion','EPSG','19936','EPSG','1294','EPSG','1160');
INSERT INTO "conversion" VALUES('EPSG','19937','Tunisia Mining Grid','The origin of the grid is at 38.81973gN, 7.83445gE of Paris, near Djebel Kebar, at which the grid reference is 270582.','EPSG','9816','Tunisia Mining Grid','EPSG','8821','Latitude of false origin',36.5964,'EPSG','9105','EPSG','8822','Longitude of false origin',7.83445,'EPSG','9105','EPSG','8826','Easting at false origin',270.0,'EPSG','9036','EPSG','8827','Northing at false origin',360.0,'EPSG','9036',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13015','conversion','EPSG','19937','EPSG','1618','EPSG','1249');
INSERT INTO "conversion" VALUES('EPSG','19938','Estonian National Grid','Coordinates at the projection origin match those of TM Baltic 93.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',57.310319415,'EPSG','9110','EPSG','8822','Longitude of false origin',24.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',59.2,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',58.0,'EPSG','9110','EPSG','8826','Easting at false origin',500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',6375000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13016','conversion','EPSG','19938','EPSG','1090','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19939','TM Baltic 93','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',24.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13017','conversion','EPSG','19939','EPSG','1646','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19940','Levant Zone','Replaced by projection using full Lambert formula (EPSG code 19948) from 1973.','EPSG','9817','Lambert Conic Near-Conformal','EPSG','8801','Latitude of natural origin',34.39,'EPSG','9110','EPSG','8802','Longitude of natural origin',37.21,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9996256,'EPSG','9201','EPSG','8806','False easting',300000.0,'EPSG','9001','EPSG','8807','False northing',300000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13018','conversion','EPSG','19940','EPSG','1623','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19941','Brazil Polyconic','','EPSG','9818','American Polyconic','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-54.0,'EPSG','9102','EPSG','8806','False easting',5000000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13019','conversion','EPSG','19941','EPSG','1053','EPSG','1241');
INSERT INTO "conversion" VALUES('EPSG','19942','British West Indies Grid','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-62.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9995,'EPSG','9201','EPSG','8806','False easting',400000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13020','conversion','EPSG','19942','EPSG','2295','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19943','Barbados National Grid','Replaced British West Indies Grid (19942) after 1983.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',13.1035,'EPSG','9110','EPSG','8802','Longitude of natural origin',-59.3335,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999986,'EPSG','9201','EPSG','8806','False easting',30000.0,'EPSG','9001','EPSG','8807','False northing',75000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13021','conversion','EPSG','19943','EPSG','3218','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19944','Quebec Lambert Projection','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',44.0,'EPSG','9110','EPSG','8822','Longitude of false origin',-68.3,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',60.0,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',46.0,'EPSG','9110','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13022','conversion','EPSG','19944','EPSG','1368','EPSG','1210');
INSERT INTO "conversion" VALUES('EPSG','19945','New Brunswick Stereographic (ATS77)','In use from 1979. To be phased out in late 1990''s.','EPSG','9809','Oblique Stereographic','EPSG','8801','Latitude of natural origin',46.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-66.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999912,'EPSG','9201','EPSG','8806','False easting',300000.0,'EPSG','9001','EPSG','8807','False northing',800000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13023','conversion','EPSG','19945','EPSG','1447','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19946','New Brunswick Stereographic (NAD83)','In use from 1999.','EPSG','9809','Oblique Stereographic','EPSG','8801','Latitude of natural origin',46.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-66.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999912,'EPSG','9201','EPSG','8806','False easting',2500000.0,'EPSG','9001','EPSG','8807','False northing',7500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13024','conversion','EPSG','19946','EPSG','1447','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19947','Austria Lambert','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',47.3,'EPSG','9110','EPSG','8822','Longitude of false origin',13.2,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',49.0,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',46.0,'EPSG','9110','EPSG','8826','Easting at false origin',400000.0,'EPSG','9001','EPSG','8827','Northing at false origin',400000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13025','conversion','EPSG','19947','EPSG','1037','EPSG','1210');
INSERT INTO "conversion" VALUES('EPSG','19948','Syria Lambert','Replaced Levant zone using same parameters but truncated near-conformal method (code 19940) from 1973.','EPSG','9801','Lambert Conic Conformal (1SP)','EPSG','8801','Latitude of natural origin',34.39,'EPSG','9110','EPSG','8802','Longitude of natural origin',37.21,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9996256,'EPSG','9201','EPSG','8806','False easting',300000.0,'EPSG','9001','EPSG','8807','False northing',300000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13026','conversion','EPSG','19948','EPSG','1623','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19949','Levant Stereographic','Used prior to World War II for cadastral and large scale topographic mapping.','EPSG','9809','Oblique Stereographic','EPSG','8801','Latitude of natural origin',38.0,'EPSG','9105','EPSG','8802','Longitude of natural origin',43.5,'EPSG','9105','EPSG','8805','Scale factor at natural origin',0.9995341,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13027','conversion','EPSG','19949','EPSG','1623','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19950','Swiss Oblique Mercator 1995','','EPSG','9815','Hotine Oblique Mercator (variant B)','EPSG','8811','Latitude of projection centre',46.570866,'EPSG','9110','EPSG','8812','Longitude of projection centre',7.26225,'EPSG','9110','EPSG','8813','Azimuth at projection centre',90.0,'EPSG','9110','EPSG','8814','Angle from Rectified to Skew Grid',90.0,'EPSG','9110','EPSG','8815','Scale factor at projection centre',1.0,'EPSG','9201','EPSG','8816','Easting at projection centre',2600000.0,'EPSG','9001','EPSG','8817','Northing at projection centre',1200000.0,'EPSG','9001',0);
INSERT INTO "usage" VALUES('EPSG','13028','conversion','EPSG','19950','EPSG','1286','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19951','Nakhl e Taqi Oblique Mercator','Used only for terminal site.','EPSG','9815','Hotine Oblique Mercator (variant B)','EPSG','8811','Latitude of projection centre',27.31077837,'EPSG','9110','EPSG','8812','Longitude of projection centre',52.3612741,'EPSG','9110','EPSG','8813','Azimuth at projection centre',0.34179803,'EPSG','9110','EPSG','8814','Angle from Rectified to Skew Grid',0.34179803,'EPSG','9110','EPSG','8815','Scale factor at projection centre',0.999895934,'EPSG','9201','EPSG','8816','Easting at projection centre',658377.437,'EPSG','9001','EPSG','8817','Northing at projection centre',3044969.194,'EPSG','9001',0);
INSERT INTO "usage" VALUES('EPSG','13029','conversion','EPSG','19951','EPSG','1338','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19952','Krovak','Original definition. Longitude is referenced to the Ferro meridian. See projection code 5509 for alternative referenced to the Greenwich meridian and code 5218 for north-orientated alternative introduced for GIS purposes.','EPSG','9819','Krovak','EPSG','8811','Latitude of projection centre',49.3,'EPSG','9110','EPSG','8833','Longitude of origin',42.3,'EPSG','9110','EPSG','1036','Co-latitude of cone axis',30.171730311,'EPSG','9110','EPSG','8818','Latitude of pseudo standard parallel',78.3,'EPSG','9110','EPSG','8819','Scale factor on pseudo standard parallel',0.9999,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',0);
INSERT INTO "usage" VALUES('EPSG','13030','conversion','EPSG','19952','EPSG','1306','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19953','Qatar Grid','','EPSG','9806','Cassini-Soldner','EPSG','8801','Latitude of natural origin',25.22565,'EPSG','9110','EPSG','8802','Longitude of natural origin',50.4541,'EPSG','9110','EPSG','8806','False easting',100000.0,'EPSG','9001','EPSG','8807','False northing',100000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13031','conversion','EPSG','19953','EPSG','1346','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19954','Suriname Old TM','Introduced in 1975. Replaced by Suriname TM in 1979.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-55.41,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13032','conversion','EPSG','19954','EPSG','1222','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19955','Suriname TM','Replaced Suriname Old TM in 1979 (scale factor changed).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-55.41,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13033','conversion','EPSG','19955','EPSG','1222','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19956','Rectified Skew Orthomorphic Borneo Grid (chain)','See 19957 and 19958 for feet and metres versions. If using Hotine Oblique Mercator (variant A) method (code 9812) FE = FN = 0 chSe. Being replaced by metric version (code 19958).','EPSG','9815','Hotine Oblique Mercator (variant B)','EPSG','8811','Latitude of projection centre',4.0,'EPSG','9110','EPSG','8812','Longitude of projection centre',115.0,'EPSG','9110','EPSG','8813','Azimuth at projection centre',53.18569537,'EPSG','9110','EPSG','8814','Angle from Rectified to Skew Grid',53.07483685,'EPSG','9110','EPSG','8815','Scale factor at projection centre',0.99984,'EPSG','9201','EPSG','8816','Easting at projection centre',29352.4763,'EPSG','9042','EPSG','8817','Northing at projection centre',22014.3572,'EPSG','9042',0);
INSERT INTO "usage" VALUES('EPSG','13034','conversion','EPSG','19956','EPSG','1362','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19957','Rectified Skew Orthomorphic Borneo Grid (foot)','See 19956 and 19958 for chains and metres versions. If using Hotine Oblique Mercator (variant A) method (code 9812) FE = FN = 0 ftSe. Being replaced by metric version (code 19958).','EPSG','9815','Hotine Oblique Mercator (variant B)','EPSG','8811','Latitude of projection centre',4.0,'EPSG','9110','EPSG','8812','Longitude of projection centre',115.0,'EPSG','9110','EPSG','8813','Azimuth at projection centre',53.18569537,'EPSG','9110','EPSG','8814','Angle from Rectified to Skew Grid',53.07483685,'EPSG','9110','EPSG','8815','Scale factor at projection centre',0.99984,'EPSG','9201','EPSG','8816','Easting at projection centre',1937263.44,'EPSG','9041','EPSG','8817','Northing at projection centre',1452947.58,'EPSG','9041',0);
INSERT INTO "usage" VALUES('EPSG','13035','conversion','EPSG','19957','EPSG','3977','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19958','Rectified Skew Orthomorphic Borneo Grid (metre)','See 19956 and 19957 for chains and feet versions. Uses Sear''s 1922 British yard-metre ratio as given by Bomford as 39.370147 inches per metre. If using Hotine Oblique Mercator (variant A) method (code 9812) FE = FN = 0 m.','EPSG','9815','Hotine Oblique Mercator (variant B)','EPSG','8811','Latitude of projection centre',4.0,'EPSG','9110','EPSG','8812','Longitude of projection centre',115.0,'EPSG','9110','EPSG','8813','Azimuth at projection centre',53.18569537,'EPSG','9110','EPSG','8814','Angle from Rectified to Skew Grid',53.07483685,'EPSG','9110','EPSG','8815','Scale factor at projection centre',0.99984,'EPSG','9201','EPSG','8816','Easting at projection centre',590476.87,'EPSG','9001','EPSG','8817','Northing at projection centre',442857.65,'EPSG','9001',0);
INSERT INTO "usage" VALUES('EPSG','13036','conversion','EPSG','19958','EPSG','1362','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19959','Ghana National Grid','Replaced by Ghana metric grid (code 19904).','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',4.4,'EPSG','9110','EPSG','8802','Longitude of natural origin',-1.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.99975,'EPSG','9201','EPSG','8806','False easting',900000.0,'EPSG','9094','EPSG','8807','False northing',0.0,'EPSG','9094',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13037','conversion','EPSG','19959','EPSG','1104','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19960','Prince Edward Isl. Stereographic (NAD83)','False Easting and False Northing changed from values used with ATS77 (which were FE=700000m; FN=400000m) to these new values when used with NAD83 (CSRS). New values are FE=400000m; FN=800000m; adopted in 2000.','EPSG','9809','Oblique Stereographic','EPSG','8801','Latitude of natural origin',47.15,'EPSG','9110','EPSG','8802','Longitude of natural origin',-63.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999912,'EPSG','9201','EPSG','8806','False easting',400000.0,'EPSG','9001','EPSG','8807','False northing',800000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13038','conversion','EPSG','19960','EPSG','1533','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19961','Belgian Lambert 72','Introduced in 2000. Equivalent to Belge Lambert 72 (code 19902). If software cannot handle latitude of false origin of 90°N, use latitude of false origin = 50°47''57.704"N with northing at false origin = 165 372.956 m.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',90.0,'EPSG','9110','EPSG','8822','Longitude of false origin',4.2202952,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',51.100000204,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',49.500000204,'EPSG','9110','EPSG','8826','Easting at false origin',150000.013,'EPSG','9001','EPSG','8827','Northing at false origin',5400088.438,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13039','conversion','EPSG','19961','EPSG','1347','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19962','Irish Transverse Mercator','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',53.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-8.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.99982,'EPSG','9201','EPSG','8806','False easting',600000.0,'EPSG','9001','EPSG','8807','False northing',750000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13040','conversion','EPSG','19962','EPSG','1305','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19963','Sierra Leone New Colony Grid','Replaces the Sierra Leone Colony Grid. New grid is 422.3 ft west and 112.1 ft south of old grid.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',6.4,'EPSG','9110','EPSG','8802','Longitude of natural origin',-12.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9094','EPSG','8807','False northing',0.0,'EPSG','9094',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13041','conversion','EPSG','19963','EPSG','1342','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19964','New War Office Sierra Leone Grid','Replaces the War Office Sierra Leone Grid. New grid is 422.3 ft west and 112.1 ft south of old grid.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',6.4,'EPSG','9110','EPSG','8802','Longitude of natural origin',-12.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',800000.0,'EPSG','9094','EPSG','8807','False northing',600000.0,'EPSG','9094',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13042','conversion','EPSG','19964','EPSG','1342','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19965','US National Atlas Equal Area','','EPSG','9821','Lambert Azimuthal Equal Area (Spherical)','EPSG','8828','Spherical latitude of origin',45.0,'EPSG','9102','EPSG','8829','Spherical longitude of origin',-100.0,'EPSG','9102','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','13043','conversion','EPSG','19965','EPSG','1245','EPSG','1162');
INSERT INTO "conversion" VALUES('EPSG','19966','Luxembourg TM','Name changed as part of redefinition of system in 2020.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',49.5,'EPSG','9110','EPSG','8802','Longitude of natural origin',6.1,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',80000.0,'EPSG','9001','EPSG','8807','False northing',100000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13044','conversion','EPSG','19966','EPSG','1146','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19967','Slovenia Grid','Replaced by Slovene National Grid (code 19845) soon after Slovenian independence.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',15.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.9999,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13045','conversion','EPSG','19967','EPSG','1212','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19968','World Equidistant Cylindrical (Sphere)','Origin at intersection of equator and prime meridian.','EPSG','9823','Equidistant Cylindrical (Spherical)','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',0.0,'EPSG','9102','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','13046','conversion','EPSG','19968','EPSG','1262','EPSG','1192');
INSERT INTO "conversion" VALUES('EPSG','19969','Portuguese Grid','Original transformation by Gauss-Kruger formula. Longitude is referenced to the Lisbon meridian. Replaced by definition using Greenwich meridian, proj code 5020.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',39.4,'EPSG','9110','EPSG','8802','Longitude of natural origin',1.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13047','conversion','EPSG','19969','EPSG','1294','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19971','New Zealand Transverse Mercator 2000','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',173.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',1600000.0,'EPSG','9001','EPSG','8807','False northing',10000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13048','conversion','EPSG','19971','EPSG','3973','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19972','Irish Grid','Defined as part of the 1965 and 1975 mapping adjustments in which the scale factor was introduced as a best fit to retain existing grid coordinates.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',53.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-8.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.000035,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',250000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13049','conversion','EPSG','19972','EPSG','1305','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19973','Irish National Grid','Used only with the 1952 geodetic adjustment. Replaced by the 1975 Mapping Adjustment: see code 19972.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',53.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-8.0,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',200000.0,'EPSG','9001','EPSG','8807','False northing',250000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13050','conversion','EPSG','19973','EPSG','2530','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19974','Modified Portuguese Grid','Applied to Datum 73. Grid position at origin is coincident with the unmodified grid applied to Lisbon datum.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',39.4,'EPSG','9110','EPSG','8802','Longitude of natural origin',-8.0754862,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',180.598,'EPSG','9001','EPSG','8807','False northing',-86.99,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13051','conversion','EPSG','19974','EPSG','1294','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19975','Trinidad Grid (Clarke''s foot)','Foot version of EPSG code 19925. Not an official system, but used by some US-based organisations including Amoco Trinidad.','EPSG','9806','Cassini-Soldner','EPSG','8801','Latitude of natural origin',10.263,'EPSG','9110','EPSG','8802','Longitude of natural origin',-61.2,'EPSG','9110','EPSG','8806','False easting',283800.0,'EPSG','9005','EPSG','8807','False northing',214500.0,'EPSG','9005',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13052','conversion','EPSG','19975','EPSG','1339','EPSG','1136');
INSERT INTO "conversion" VALUES('EPSG','19976','ICN Regional','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',6.0,'EPSG','9102','EPSG','8822','Longitude of false origin',-66.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',9.0,'EPSG','9102','EPSG','8824','Latitude of 2nd standard parallel',3.0,'EPSG','9102','EPSG','8826','Easting at false origin',1000000.0,'EPSG','9001','EPSG','8827','Northing at false origin',1000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13053','conversion','EPSG','19976','EPSG','1251','EPSG','1242');
INSERT INTO "conversion" VALUES('EPSG','19977','Aramco Lambert','Used by Saudi Aramco when area of interest crosses UTM zone boundary. Adopted by partners for Core Venture 1 (South Ghawar) area.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',25.0522236,'EPSG','9110','EPSG','8822','Longitude of false origin',48.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',17.0,'EPSG','9102','EPSG','8824','Latitude of 2nd standard parallel',33.0,'EPSG','9102','EPSG','8826','Easting at false origin',0.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13054','conversion','EPSG','19977','EPSG','3303','EPSG','1136');
INSERT INTO "conversion" VALUES('EPSG','19978','Hong Kong 1980 Grid','Grid origin is Partiridge Hill triangulation station (old trig "2").','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',22.184368,'EPSG','9110','EPSG','8802','Longitude of natural origin',114.10428,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',836694.05,'EPSG','9001','EPSG','8807','False northing',819069.8,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13055','conversion','EPSG','19978','EPSG','1118','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','19979','Portugal Bonne','Longitude is referenced to the Lisbon meridian. Replaced by definition using Greenwich meridian, proj code 5019.','EPSG','9828','Bonne (South Orientated)','EPSG','8801','Latitude of natural origin',39.4,'EPSG','9110','EPSG','8802','Longitude of natural origin',1.0,'EPSG','9110','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13056','conversion','EPSG','19979','EPSG','1294','EPSG','1062');
INSERT INTO "conversion" VALUES('EPSG','19981','Lambert New Caledonia','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-21.3,'EPSG','9110','EPSG','8822','Longitude of false origin',166.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',-20.4,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-22.2,'EPSG','9110','EPSG','8826','Easting at false origin',400000.0,'EPSG','9001','EPSG','8827','Northing at false origin',300000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13057','conversion','EPSG','19981','EPSG','3430','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19982','TM Reunion','Supersedes Gauss Laborde Reunion.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',-21.07,'EPSG','9110','EPSG','8802','Longitude of natural origin',55.32,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',50000.0,'EPSG','9001','EPSG','8807','False northing',160000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1);
INSERT INTO "usage" VALUES('EPSG','13058','conversion','EPSG','19982','EPSG','1196','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19983','Terre Adelie Polar Stereographic','','EPSG','9830','Polar Stereographic (variant C)','EPSG','8832','Latitude of standard parallel',-67.0,'EPSG','9102','EPSG','8833','Longitude of origin',140.0,'EPSG','9102','EPSG','8826','Easting at false origin',300000.0,'EPSG','9001','EPSG','8827','Northing at false origin',200000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13059','conversion','EPSG','19983','EPSG','2818','EPSG','1153');
INSERT INTO "conversion" VALUES('EPSG','19984','British Columbia Albers','','EPSG','9822','Albers Equal Area','EPSG','8821','Latitude of false origin',45.0,'EPSG','9102','EPSG','8822','Longitude of false origin',-126.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',50.0,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',58.3,'EPSG','9110','EPSG','8826','Easting at false origin',1000000.0,'EPSG','9001','EPSG','8827','Northing at false origin',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13060','conversion','EPSG','19984','EPSG','2832','EPSG','1220');
INSERT INTO "conversion" VALUES('EPSG','19985','Europe Conformal 2001','TMzn used for applications at scales larger than 1:500,000. LAEA (code 19986) used for statistical mapping.','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',52.0,'EPSG','9102','EPSG','8822','Longitude of false origin',10.0,'EPSG','9102','EPSG','8823','Latitude of 1st standard parallel',35.0,'EPSG','9102','EPSG','8824','Latitude of 2nd standard parallel',65.0,'EPSG','9102','EPSG','8826','Easting at false origin',4000000.0,'EPSG','9001','EPSG','8827','Northing at false origin',2800000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13061','conversion','EPSG','19985','EPSG','2881','EPSG','1241');
INSERT INTO "conversion" VALUES('EPSG','19986','Europe Equal Area 2001','LCC (code 19985) used for conformal mapping.','EPSG','9820','Lambert Azimuthal Equal Area','EPSG','8801','Latitude of natural origin',52.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',10.0,'EPSG','9102','EPSG','8806','False easting',4321000.0,'EPSG','9001','EPSG','8807','False northing',3210000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13062','conversion','EPSG','19986','EPSG','2881','EPSG','1162');
INSERT INTO "conversion" VALUES('EPSG','19987','Iceland Lambert 1900','Used only with Reykjavik geogCRS. Longitude of origin originally defined as 31°30'' west of Copenhagen.','EPSG','9826','Lambert Conic Conformal (West Orientated)','EPSG','8801','Latitude of natural origin',65.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-19.011965,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13063','conversion','EPSG','19987','EPSG','3262','EPSG','1241');
INSERT INTO "conversion" VALUES('EPSG','19988','Iceland Lambert 1955','Used only with Hjorsey geogCRS.','EPSG','9826','Lambert Conic Conformal (West Orientated)','EPSG','8801','Latitude of natural origin',65.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',-18.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13064','conversion','EPSG','19988','EPSG','3262','EPSG','1241');
INSERT INTO "conversion" VALUES('EPSG','19989','Iceland Lambert 1993','Used only with ISN93 geogCRS. Replaced by Iceland Lambert 2004 (code 5326).','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',65.0,'EPSG','9110','EPSG','8822','Longitude of false origin',-19.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',64.15,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',65.45,'EPSG','9110','EPSG','8826','Easting at false origin',500000.0,'EPSG','9001','EPSG','8827','Northing at false origin',500000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13065','conversion','EPSG','19989','EPSG','1120','EPSG','1241');
INSERT INTO "conversion" VALUES('EPSG','19990','Latvian Transverse Mercator','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',24.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9996,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',-6000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13066','conversion','EPSG','19990','EPSG','1139','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19991','Jan Mayen Grid','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9110','EPSG','8802','Longitude of natural origin',-8.3,'EPSG','9110','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',50000.0,'EPSG','9001','EPSG','8807','False northing',-7800000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13067','conversion','EPSG','19991','EPSG','2869','EPSG','1153');
INSERT INTO "conversion" VALUES('EPSG','19992','Antarctic Polar Stereographic','Special studies may use a different projection using an alternative longitude of origin. See for example projection code 19993.','EPSG','9829','Polar Stereographic (variant B)','EPSG','8832','Latitude of standard parallel',-71.0,'EPSG','9102','EPSG','8833','Longitude of origin',0.0,'EPSG','9102','EPSG','8806','False easting',0.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13068','conversion','EPSG','19992','EPSG','1031','EPSG','1254');
INSERT INTO "conversion" VALUES('EPSG','19993','Australian Antarctic Polar Stereographic','','EPSG','9829','Polar Stereographic (variant B)','EPSG','8832','Latitude of standard parallel',-71.0,'EPSG','9102','EPSG','8833','Longitude of origin',70.0,'EPSG','9102','EPSG','8806','False easting',6000000.0,'EPSG','9001','EPSG','8807','False northing',6000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13069','conversion','EPSG','19993','EPSG','1278','EPSG','1210');
INSERT INTO "conversion" VALUES('EPSG','19994','Australian Antarctic Lambert','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',-50.0,'EPSG','9110','EPSG','8822','Longitude of false origin',70.0,'EPSG','9110','EPSG','8823','Latitude of 1st standard parallel',-68.3,'EPSG','9110','EPSG','8824','Latitude of 2nd standard parallel',-74.3,'EPSG','9110','EPSG','8826','Easting at false origin',6000000.0,'EPSG','9001','EPSG','8827','Northing at false origin',6000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13070','conversion','EPSG','19994','EPSG','2880','EPSG','1210');
INSERT INTO "conversion" VALUES('EPSG','19995','Jordan Transverse Mercator','Information has not been confirmed by National Mapping Agency.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',37.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9998,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',-3000000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13071','conversion','EPSG','19995','EPSG','1130','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19996','Soldner Berlin','Origin is trigonometrical station Müggelberg. Effective under city ordnance dated 5th November 1991.','EPSG','9806','Cassini-Soldner','EPSG','8801','Latitude of natural origin',52.25071338,'EPSG','9110','EPSG','8802','Longitude of natural origin',13.37379332,'EPSG','9110','EPSG','8806','False easting',40000.0,'EPSG','9001','EPSG','8807','False northing',10000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13072','conversion','EPSG','19996','EPSG','2898','EPSG','1055');
INSERT INTO "conversion" VALUES('EPSG','19997','Kuwait Transverse Mercator','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0.0,'EPSG','9102','EPSG','8802','Longitude of natural origin',48.0,'EPSG','9102','EPSG','8805','Scale factor at natural origin',1.0,'EPSG','9201','EPSG','8806','False easting',500000.0,'EPSG','9001','EPSG','8807','False northing',0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13073','conversion','EPSG','19997','EPSG','1310','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19998','Guernsey Grid','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',49.3,'EPSG','9110','EPSG','8802','Longitude of natural origin',-2.25,'EPSG','9110','EPSG','8805','Scale factor at natural origin',0.999997,'EPSG','9201','EPSG','8806','False easting',47000.0,'EPSG','9001','EPSG','8807','False northing',50000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13074','conversion','EPSG','19998','EPSG','2989','EPSG','1142');
INSERT INTO "conversion" VALUES('EPSG','19999','Jersey Transverse Mercator','Effective from 1st January 2005.','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',49.225,'EPSG','9102','EPSG','8802','Longitude of natural origin',-2.135,'EPSG','9102','EPSG','8805','Scale factor at natural origin',0.9999999,'EPSG','9201','EPSG','8806','False easting',40000.0,'EPSG','9001','EPSG','8807','False northing',70000.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO "usage" VALUES('EPSG','13075','conversion','EPSG','19999','EPSG','2988','EPSG','1142');
