--- This file has been generated by scripts/build_db_from_iau.py. DO NOT EDIT !

INSERT INTO coordinate_system VALUES('PROJ','OCENTRIC_LAT_LON','spherical',2);
INSERT INTO axis VALUES('PROJ','OCENTRIC_LAT_LON_LAT','Planetocentric latitude','U','north','PROJ','OCENTRIC_LAT_LON',1,'EPSG','9122');
INSERT INTO axis VALUES('PROJ','OCENTRIC_LAT_LON_LON','Planetocentric longitude','V','east','PROJ','OCENTRIC_LAT_LON',2,'EPSG','9122');
INSERT INTO coordinate_system VALUES('PROJ','OGRAPHIC_NORTH_WEST','ellipsoidal',2);
INSERT INTO axis VALUES('PROJ','OGRAPHIC_NORTH_WEST_LAT','Geodetic latitude','Lat','north','PROJ','OGRAPHIC_NORTH_WEST',1,'EPSG','9122');
INSERT INTO axis VALUES('PROJ','OGRAPHIC_NORTH_WEST_LON','Geodetic longitude','Lon','west','PROJ','OGRAPHIC_NORTH_WEST',2,'EPSG','9122');
INSERT INTO coordinate_system VALUES('PROJ','PROJECTED_WEST_NORTH','Cartesian',2);
INSERT INTO axis VALUES('PROJ','PROJECTED_WEST_NORTH_W','Westing','W','west','PROJ','PROJECTED_WEST_NORTH',1,'EPSG','9001');
INSERT INTO axis VALUES('PROJ','PROJECTED_WEST_NORTH_N','Northing','N','north','PROJ','PROJECTED_WEST_NORTH',2,'EPSG','9001');
INSERT INTO conversion VALUES ('IAU_2015',10,'Equirectangular, clon = 0','','EPSG','1028','Equidistant Cylindrical','EPSG','8823','Latitude of 1st standard parallel',0,'EPSG',9122,'EPSG','8802','Longitude of natural origin',0,'EPSG',9122,'EPSG','8806','False easting',0,'EPSG',9001,'EPSG','8807','False northing',0,'EPSG',9001,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','CONV_10','conversion','IAU_2015',10,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO conversion VALUES ('IAU_2015',15,'Equirectangular, clon = 180','','EPSG','1028','Equidistant Cylindrical','EPSG','8823','Latitude of 1st standard parallel',0,'EPSG',9122,'EPSG','8802','Longitude of natural origin',180,'EPSG',9122,'EPSG','8806','False easting',0,'EPSG',9001,'EPSG','8807','False northing',0,'EPSG',9001,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','CONV_15','conversion','IAU_2015',15,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO conversion VALUES ('IAU_2015',20,'Sinusoidal, clon = 0','','PROJ','SINUSOIDAL','Sinusoidal','EPSG','8802','Longitude of natural origin',0,'EPSG',9122,'EPSG','8806','False easting',0,'EPSG',9001,'EPSG','8807','False northing',0,'EPSG',9001,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','CONV_20','conversion','IAU_2015',20,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO conversion VALUES ('IAU_2015',25,'Sinusoidal, clon = 180','','PROJ','SINUSOIDAL','Sinusoidal','EPSG','8802','Longitude of natural origin',180,'EPSG',9122,'EPSG','8806','False easting',0,'EPSG',9001,'EPSG','8807','False northing',0,'EPSG',9001,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','CONV_25','conversion','IAU_2015',25,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO conversion VALUES ('IAU_2015',30,'North Polar','','EPSG','9810','Polar Stereographic (variant A)','EPSG','8801','Latitude of natural origin',90,'EPSG',9122,'EPSG','8802','Longitude of natural origin',0,'EPSG',9122,'EPSG','8805','Scale factor at natural origin',1,'EPSG',9201,'EPSG','8806','False easting',0,'EPSG',9001,'EPSG','8807','False northing',0,'EPSG',9001,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','CONV_30','conversion','IAU_2015',30,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO conversion VALUES ('IAU_2015',35,'South Polar','','EPSG','9810','Polar Stereographic (variant A)','EPSG','8801','Latitude of natural origin',-90,'EPSG',9122,'EPSG','8802','Longitude of natural origin',0,'EPSG',9122,'EPSG','8805','Scale factor at natural origin',1,'EPSG',9201,'EPSG','8806','False easting',0,'EPSG',9001,'EPSG','8807','False northing',0,'EPSG',9001,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','CONV_35','conversion','IAU_2015',35,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO conversion VALUES ('IAU_2015',40,'Mollweide, clon = 0','','PROJ','MOLLWEIDE','Mollweide','EPSG','8802','Longitude of natural origin',0,'EPSG',9122,'EPSG','8806','False easting',0,'EPSG',9001,'EPSG','8807','False northing',0,'EPSG',9001,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','CONV_40','conversion','IAU_2015',40,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO conversion VALUES ('IAU_2015',45,'Mollweide, clon = 180','','PROJ','MOLLWEIDE','Mollweide','EPSG','8802','Longitude of natural origin',180,'EPSG',9122,'EPSG','8806','False easting',0,'EPSG',9001,'EPSG','8807','False northing',0,'EPSG',9001,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','CONV_45','conversion','IAU_2015',45,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO conversion VALUES ('IAU_2015',50,'Robinson, clon = 0','','PROJ','ROBINSON','Robinson','EPSG','8802','Longitude of natural origin',0,'EPSG',9122,'EPSG','8806','False easting',0,'EPSG',9001,'EPSG','8807','False northing',0,'EPSG',9001,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','CONV_50','conversion','IAU_2015',50,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO conversion VALUES ('IAU_2015',55,'Robinson, clon = 180','','PROJ','ROBINSON','Robinson','EPSG','8802','Longitude of natural origin',180,'EPSG',9122,'EPSG','8806','False easting',0,'EPSG',9001,'EPSG','8807','False northing',0,'EPSG',9001,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','CONV_55','conversion','IAU_2015',55,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO conversion VALUES ('IAU_2015',60,'Transverse Mercator','','EPSG','9807','Transverse Mercator','EPSG','8801','Latitude of natural origin',0,'EPSG',9122,'EPSG','8802','Longitude of natural origin',0,'EPSG',9122,'EPSG','8805','Scale factor at natural origin',1.0,'EPSG',9201,'EPSG','8806','False easting',0,'EPSG',9001,'EPSG','8807','False northing',0,'EPSG',9001,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','CONV_60','conversion','IAU_2015',60,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO conversion VALUES ('IAU_2015',65,'Orthographic, clon = 0','','EPSG','9840','Orthographic','EPSG','8801','Latitude of natural origin',0,'EPSG',9122,'EPSG','8802','Longitude of natural origin',0,'EPSG',9122,'EPSG','8806','False easting',0,'EPSG',9001,'EPSG','8807','False northing',0,'EPSG',9001,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','CONV_65','conversion','IAU_2015',65,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO conversion VALUES ('IAU_2015',70,'Orthographic, clon = 180','','EPSG','9840','Orthographic','EPSG','8801','Latitude of natural origin',0,'EPSG',9122,'EPSG','8802','Longitude of natural origin',180,'EPSG',9122,'EPSG','8806','False easting',0,'EPSG',9001,'EPSG','8807','False northing',0,'EPSG',9001,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','CONV_70','conversion','IAU_2015',70,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO conversion VALUES ('IAU_2015',75,'Lambert Conic Conformal','','EPSG','9802','Lambert Conic Conformal (2SP)','EPSG','8821','Latitude of false origin',40,'EPSG',9122,'EPSG','8822','Longitude of false origin',0,'EPSG',9122,'EPSG','8823','Latitude of 1st standard parallel',20,'EPSG',9122,'EPSG','8824','Latitude of 2nd standard parallel',60,'EPSG',9122,'EPSG','8826','Easting at false origin',0,'EPSG',9001,'EPSG','8827','Northing at false origin',0,'EPSG',9001,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','CONV_75','conversion','IAU_2015',75,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO conversion VALUES ('IAU_2015',80,'Lambert Azimuthal Equal Area','','EPSG','9820','Lambert Azimuthal Equal Area','EPSG','8801','Latitude of natural origin',40,'EPSG',9122,'EPSG','8802','Longitude of natural origin',0,'EPSG',9122,'EPSG','8806','False easting',0,'EPSG',9001,'EPSG','8807','False northing',0,'EPSG',9001,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','CONV_80','conversion','IAU_2015',80,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO conversion VALUES ('IAU_2015',85,'Albers Equal Area','','EPSG','9822','Albers Equal Area','EPSG','8821','Latitude of false origin',40,'EPSG',9122,'EPSG','8822','Longitude of false origin',0,'EPSG',9122,'EPSG','8823','Latitude of 1st standard parallel',20,'EPSG',9122,'EPSG','8824','Latitude of 2nd standard parallel',60,'EPSG',9122,'EPSG','8826','Easting at false origin',0,'EPSG',9001,'EPSG','8827','Northing at false origin',0,'EPSG',9001,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','CONV_85','conversion','IAU_2015',85,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO conversion VALUES ('IAU_2015',90,'Mercator','','EPSG','1026','Mercator (Spherical)','EPSG','8801','Latitude of natural origin',0,'EPSG',9122,'EPSG','8802','Longitude of natural origin',0,'EPSG',9122,'EPSG','8806','False easting',0,'EPSG',9001,'EPSG','8807','False northing',0,'EPSG',9001,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','CONV_90','conversion','IAU_2015',90,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 10, 'Sun', 695700000.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 1000, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',1000,'Sun (2015) - Sphere',NULL,'IAU_2015',10,695700000.000000,'EPSG','9001',NULL,695700000.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',1000,'Sun (2015) - Sphere','','IAU_2015',1000,'IAU_2015',1000,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_1000','geodetic_datum','IAU_2015',1000,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',1000,'Sun (2015) - Sphere / Ocentric','Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',1000,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_1000','geodetic_crs','IAU_2015',1000,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',1010,'Sun (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',1000,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_1010','projected_crs','IAU_2015',1010,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',1015,'Sun (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',1000,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_1015','projected_crs','IAU_2015',1015,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',1020,'Sun (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',1000,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_1020','projected_crs','IAU_2015',1020,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',1025,'Sun (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',1000,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_1025','projected_crs','IAU_2015',1025,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',1030,'Sun (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',1000,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_1030','projected_crs','IAU_2015',1030,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',1035,'Sun (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',1000,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_1035','projected_crs','IAU_2015',1035,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',1040,'Sun (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',1000,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_1040','projected_crs','IAU_2015',1040,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',1045,'Sun (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',1000,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_1045','projected_crs','IAU_2015',1045,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',1050,'Sun (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',1000,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_1050','projected_crs','IAU_2015',1050,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',1055,'Sun (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',1000,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_1055','projected_crs','IAU_2015',1055,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',1060,'Sun (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',1000,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_1060','projected_crs','IAU_2015',1060,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',1065,'Sun (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',1000,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_1065','projected_crs','IAU_2015',1065,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',1070,'Sun (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',1000,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_1070','projected_crs','IAU_2015',1070,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',1075,'Sun (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',1000,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_1075','projected_crs','IAU_2015',1075,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',1080,'Sun (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',1000,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_1080','projected_crs','IAU_2015',1080,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',1085,'Sun (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',1000,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_1085','projected_crs','IAU_2015',1085,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',1090,'Sun (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',1000,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_1090','projected_crs','IAU_2015',1090,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 199, 'Mercury', 2440530.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 19900, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',19900,'Mercury (2015) - Sphere',NULL,'IAU_2015',199,2440530.000000,'EPSG','9001',NULL,2440530.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',19900,'Mercury (2015) - Sphere','','IAU_2015',19900,'IAU_2015',19900,NULL,NULL,NULL,'Hun Kal: 20 W.0',NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_19900','geodetic_datum','IAU_2015',19900,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',19900,'Mercury (2015) - Sphere / Ocentric','Use semi-major radius as sphere for interoperability. Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',19900,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_19900','geodetic_crs','IAU_2015',19900,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',19910,'Mercury (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',19900,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_19910','projected_crs','IAU_2015',19910,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',19915,'Mercury (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',19900,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_19915','projected_crs','IAU_2015',19915,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',19920,'Mercury (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',19900,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_19920','projected_crs','IAU_2015',19920,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',19925,'Mercury (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',19900,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_19925','projected_crs','IAU_2015',19925,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',19930,'Mercury (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',19900,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_19930','projected_crs','IAU_2015',19930,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',19935,'Mercury (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',19900,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_19935','projected_crs','IAU_2015',19935,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',19940,'Mercury (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',19900,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_19940','projected_crs','IAU_2015',19940,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',19945,'Mercury (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',19900,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_19945','projected_crs','IAU_2015',19945,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',19950,'Mercury (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',19900,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_19950','projected_crs','IAU_2015',19950,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',19955,'Mercury (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',19900,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_19955','projected_crs','IAU_2015',19955,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',19960,'Mercury (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',19900,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_19960','projected_crs','IAU_2015',19960,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',19965,'Mercury (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',19900,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_19965','projected_crs','IAU_2015',19965,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',19970,'Mercury (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',19900,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_19970','projected_crs','IAU_2015',19970,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',19975,'Mercury (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',19900,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_19975','projected_crs','IAU_2015',19975,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',19980,'Mercury (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',19900,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_19980','projected_crs','IAU_2015',19980,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',19985,'Mercury (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',19900,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_19985','projected_crs','IAU_2015',19985,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',19990,'Mercury (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',19900,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_19990','projected_crs','IAU_2015',19990,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO ellipsoid VALUES('IAU_2015',19901,'Mercury (2015)',NULL,'IAU_2015',199,2440530.000000,'EPSG','9001',NULL,2438260.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',19901,'Mercury (2015)','','IAU_2015',19901,'IAU_2015',19900,NULL,NULL,NULL,'Hun Kal: 20 W.0',NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_19901','geodetic_datum','IAU_2015',19901,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',19901,'Mercury (2015) / Ographic','Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','PROJ','OGRAPHIC_NORTH_WEST','IAU_2015',19901,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_19901','geodetic_crs','IAU_2015',19901,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',19911,'Mercury (2015) / Ographic / Equirectangular, clon = 0',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',19901,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_19911','projected_crs','IAU_2015',19911,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',19916,'Mercury (2015) / Ographic / Equirectangular, clon = 180',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',19901,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_19916','projected_crs','IAU_2015',19916,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',19921,'Mercury (2015) / Ographic / Sinusoidal, clon = 0',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',19901,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_19921','projected_crs','IAU_2015',19921,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',19926,'Mercury (2015) / Ographic / Sinusoidal, clon = 180',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',19901,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_19926','projected_crs','IAU_2015',19926,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',19931,'Mercury (2015) / Ographic / North Polar',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',19901,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_19931','projected_crs','IAU_2015',19931,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',19936,'Mercury (2015) / Ographic / South Polar',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',19901,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_19936','projected_crs','IAU_2015',19936,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',19941,'Mercury (2015) / Ographic / Mollweide, clon = 0',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',19901,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_19941','projected_crs','IAU_2015',19941,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',19946,'Mercury (2015) / Ographic / Mollweide, clon = 180',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',19901,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_19946','projected_crs','IAU_2015',19946,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',19951,'Mercury (2015) / Ographic / Robinson, clon = 0',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',19901,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_19951','projected_crs','IAU_2015',19951,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',19956,'Mercury (2015) / Ographic / Robinson, clon = 180',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',19901,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_19956','projected_crs','IAU_2015',19956,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',19961,'Mercury (2015) / Ographic / Transverse Mercator',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',19901,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_19961','projected_crs','IAU_2015',19961,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',19966,'Mercury (2015) / Ographic / Orthographic, clon = 0',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',19901,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_19966','projected_crs','IAU_2015',19966,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',19971,'Mercury (2015) / Ographic / Orthographic, clon = 180',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',19901,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_19971','projected_crs','IAU_2015',19971,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',19976,'Mercury (2015) / Ographic / Lambert Conic Conformal',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',19901,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_19976','projected_crs','IAU_2015',19976,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',19981,'Mercury (2015) / Ographic / Lambert Azimuthal Equal Area',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',19901,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_19981','projected_crs','IAU_2015',19981,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',19986,'Mercury (2015) / Ographic / Albers Equal Area',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',19901,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_19986','projected_crs','IAU_2015',19986,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',19991,'Mercury (2015) / Ographic / Mercator',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',19901,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_19991','projected_crs','IAU_2015',19991,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',19902,'Mercury (2015) / Ocentric','Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','other','PROJ','OCENTRIC_LAT_LON','IAU_2015',19901,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_19902','geodetic_crs','IAU_2015',19902,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',19912,'Mercury (2015) / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',19902,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_19912','projected_crs','IAU_2015',19912,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',19917,'Mercury (2015) / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',19902,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_19917','projected_crs','IAU_2015',19917,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',19922,'Mercury (2015) / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',19902,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_19922','projected_crs','IAU_2015',19922,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',19927,'Mercury (2015) / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',19902,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_19927','projected_crs','IAU_2015',19927,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',19932,'Mercury (2015) / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',19902,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_19932','projected_crs','IAU_2015',19932,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',19937,'Mercury (2015) / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',19902,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_19937','projected_crs','IAU_2015',19937,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',19942,'Mercury (2015) / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',19902,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_19942','projected_crs','IAU_2015',19942,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',19947,'Mercury (2015) / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',19902,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_19947','projected_crs','IAU_2015',19947,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',19952,'Mercury (2015) / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',19902,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_19952','projected_crs','IAU_2015',19952,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',19957,'Mercury (2015) / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',19902,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_19957','projected_crs','IAU_2015',19957,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',19962,'Mercury (2015) / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',19902,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_19962','projected_crs','IAU_2015',19962,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',19967,'Mercury (2015) / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',19902,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_19967','projected_crs','IAU_2015',19967,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',19972,'Mercury (2015) / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',19902,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_19972','projected_crs','IAU_2015',19972,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',19977,'Mercury (2015) / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',19902,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_19977','projected_crs','IAU_2015',19977,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',19982,'Mercury (2015) / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',19902,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_19982','projected_crs','IAU_2015',19982,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',19987,'Mercury (2015) / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',19902,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_19987','projected_crs','IAU_2015',19987,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',19992,'Mercury (2015) / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',19902,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_19992','projected_crs','IAU_2015',19992,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 299, 'Venus', 6051800.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 29900, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',29900,'Venus (2015) - Sphere',NULL,'IAU_2015',299,6051800.000000,'EPSG','9001',NULL,6051800.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',29900,'Venus (2015) - Sphere','','IAU_2015',29900,'IAU_2015',29900,NULL,NULL,NULL,'Ariadne: 0.0',NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_29900','geodetic_datum','IAU_2015',29900,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',29900,'Venus (2015) - Sphere / Ocentric','Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',29900,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_29900','geodetic_crs','IAU_2015',29900,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',29910,'Venus (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',29900,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_29910','projected_crs','IAU_2015',29910,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',29915,'Venus (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',29900,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_29915','projected_crs','IAU_2015',29915,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',29920,'Venus (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',29900,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_29920','projected_crs','IAU_2015',29920,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',29925,'Venus (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',29900,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_29925','projected_crs','IAU_2015',29925,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',29930,'Venus (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',29900,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_29930','projected_crs','IAU_2015',29930,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',29935,'Venus (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',29900,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_29935','projected_crs','IAU_2015',29935,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',29940,'Venus (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',29900,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_29940','projected_crs','IAU_2015',29940,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',29945,'Venus (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',29900,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_29945','projected_crs','IAU_2015',29945,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',29950,'Venus (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',29900,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_29950','projected_crs','IAU_2015',29950,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',29955,'Venus (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',29900,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_29955','projected_crs','IAU_2015',29955,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',29960,'Venus (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',29900,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_29960','projected_crs','IAU_2015',29960,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',29965,'Venus (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',29900,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_29965','projected_crs','IAU_2015',29965,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',29970,'Venus (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',29900,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_29970','projected_crs','IAU_2015',29970,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',29975,'Venus (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',29900,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_29975','projected_crs','IAU_2015',29975,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',29980,'Venus (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',29900,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_29980','projected_crs','IAU_2015',29980,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',29985,'Venus (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',29900,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_29985','projected_crs','IAU_2015',29985,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',29990,'Venus (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',29900,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_29990','projected_crs','IAU_2015',29990,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 399, 'Earth', 6378136.600000);
INSERT INTO prime_meridian VALUES('IAU_2015', 39900, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',39900,'Earth (2015) - Sphere',NULL,'IAU_2015',399,6378136.600000,'EPSG','9001',NULL,6378136.600000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',39900,'Earth (2015) - Sphere','','IAU_2015',39900,'IAU_2015',39900,NULL,NULL,NULL,'Greenwich: 0.0',NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_39900','geodetic_datum','IAU_2015',39900,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',39900,'Earth (2015) - Sphere / Ocentric','Use semi-major radius as sphere for interoperability. Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',39900,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_39900','geodetic_crs','IAU_2015',39900,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',39910,'Earth (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',39900,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_39910','projected_crs','IAU_2015',39910,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',39915,'Earth (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',39900,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_39915','projected_crs','IAU_2015',39915,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',39920,'Earth (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',39900,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_39920','projected_crs','IAU_2015',39920,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',39925,'Earth (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',39900,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_39925','projected_crs','IAU_2015',39925,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',39930,'Earth (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',39900,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_39930','projected_crs','IAU_2015',39930,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',39935,'Earth (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',39900,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_39935','projected_crs','IAU_2015',39935,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',39940,'Earth (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',39900,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_39940','projected_crs','IAU_2015',39940,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',39945,'Earth (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',39900,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_39945','projected_crs','IAU_2015',39945,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',39950,'Earth (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',39900,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_39950','projected_crs','IAU_2015',39950,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',39955,'Earth (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',39900,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_39955','projected_crs','IAU_2015',39955,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',39960,'Earth (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',39900,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_39960','projected_crs','IAU_2015',39960,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',39965,'Earth (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',39900,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_39965','projected_crs','IAU_2015',39965,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',39970,'Earth (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',39900,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_39970','projected_crs','IAU_2015',39970,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',39975,'Earth (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',39900,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_39975','projected_crs','IAU_2015',39975,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',39980,'Earth (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',39900,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_39980','projected_crs','IAU_2015',39980,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',39985,'Earth (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',39900,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_39985','projected_crs','IAU_2015',39985,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',39990,'Earth (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',39900,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_39990','projected_crs','IAU_2015',39990,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO ellipsoid VALUES('IAU_2015',39901,'Earth (2015)',NULL,'IAU_2015',399,6378136.600000,'EPSG','9001',NULL,6356751.900000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',39901,'Earth (2015)','','IAU_2015',39901,'IAU_2015',39900,NULL,NULL,NULL,'Greenwich: 0.0',NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_39901','geodetic_datum','IAU_2015',39901,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',39901,'Earth (2015) / Ographic','Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',39901,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_39901','geodetic_crs','IAU_2015',39901,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',39911,'Earth (2015) / Ographic / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',39901,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_39911','projected_crs','IAU_2015',39911,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',39916,'Earth (2015) / Ographic / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',39901,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_39916','projected_crs','IAU_2015',39916,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',39921,'Earth (2015) / Ographic / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',39901,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_39921','projected_crs','IAU_2015',39921,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',39926,'Earth (2015) / Ographic / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',39901,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_39926','projected_crs','IAU_2015',39926,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',39931,'Earth (2015) / Ographic / North Polar',NULL,'EPSG','4469','IAU_2015',39901,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_39931','projected_crs','IAU_2015',39931,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',39936,'Earth (2015) / Ographic / South Polar',NULL,'EPSG','4490','IAU_2015',39901,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_39936','projected_crs','IAU_2015',39936,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',39941,'Earth (2015) / Ographic / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',39901,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_39941','projected_crs','IAU_2015',39941,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',39946,'Earth (2015) / Ographic / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',39901,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_39946','projected_crs','IAU_2015',39946,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',39951,'Earth (2015) / Ographic / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',39901,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_39951','projected_crs','IAU_2015',39951,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',39956,'Earth (2015) / Ographic / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',39901,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_39956','projected_crs','IAU_2015',39956,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',39961,'Earth (2015) / Ographic / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',39901,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_39961','projected_crs','IAU_2015',39961,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',39966,'Earth (2015) / Ographic / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',39901,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_39966','projected_crs','IAU_2015',39966,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',39971,'Earth (2015) / Ographic / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',39901,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_39971','projected_crs','IAU_2015',39971,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',39976,'Earth (2015) / Ographic / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',39901,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_39976','projected_crs','IAU_2015',39976,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',39981,'Earth (2015) / Ographic / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',39901,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_39981','projected_crs','IAU_2015',39981,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',39986,'Earth (2015) / Ographic / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',39901,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_39986','projected_crs','IAU_2015',39986,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',39991,'Earth (2015) / Ographic / Mercator',NULL,'EPSG','4400','IAU_2015',39901,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_39991','projected_crs','IAU_2015',39991,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',39902,'Earth (2015) / Ocentric','Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','other','PROJ','OCENTRIC_LAT_LON','IAU_2015',39901,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_39902','geodetic_crs','IAU_2015',39902,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 301, 'Moon', 1737400.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 30100, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',30100,'Moon (2015) - Sphere',NULL,'IAU_2015',301,1737400.000000,'EPSG','9001',NULL,1737400.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',30100,'Moon (2015) - Sphere','','IAU_2015',30100,'IAU_2015',30100,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_30100','geodetic_datum','IAU_2015',30100,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',30100,'Moon (2015) - Sphere / Ocentric','Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',30100,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_30100','geodetic_crs','IAU_2015',30100,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',30110,'Moon (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',30100,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_30110','projected_crs','IAU_2015',30110,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',30115,'Moon (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',30100,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_30115','projected_crs','IAU_2015',30115,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',30120,'Moon (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',30100,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_30120','projected_crs','IAU_2015',30120,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',30125,'Moon (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',30100,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_30125','projected_crs','IAU_2015',30125,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',30130,'Moon (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',30100,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_30130','projected_crs','IAU_2015',30130,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',30135,'Moon (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',30100,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_30135','projected_crs','IAU_2015',30135,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',30140,'Moon (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',30100,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_30140','projected_crs','IAU_2015',30140,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',30145,'Moon (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',30100,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_30145','projected_crs','IAU_2015',30145,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',30150,'Moon (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',30100,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_30150','projected_crs','IAU_2015',30150,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',30155,'Moon (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',30100,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_30155','projected_crs','IAU_2015',30155,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',30160,'Moon (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',30100,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_30160','projected_crs','IAU_2015',30160,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',30165,'Moon (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',30100,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_30165','projected_crs','IAU_2015',30165,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',30170,'Moon (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',30100,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_30170','projected_crs','IAU_2015',30170,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',30175,'Moon (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',30100,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_30175','projected_crs','IAU_2015',30175,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',30180,'Moon (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',30100,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_30180','projected_crs','IAU_2015',30180,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',30185,'Moon (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',30100,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_30185','projected_crs','IAU_2015',30185,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',30190,'Moon (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',30100,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_30190','projected_crs','IAU_2015',30190,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 499, 'Mars', 3396190.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 49900, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',49900,'Mars (2015) - Sphere',NULL,'IAU_2015',499,3396190.000000,'EPSG','9001',NULL,3396190.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',49900,'Mars (2015) - Sphere','','IAU_2015',49900,'IAU_2015',49900,NULL,NULL,NULL,'Viking 1 lander: 47.95137 W',NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_49900','geodetic_datum','IAU_2015',49900,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',49900,'Mars (2015) - Sphere / Ocentric','Use semi-major radius as sphere for interoperability. Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',49900,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_49900','geodetic_crs','IAU_2015',49900,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',49910,'Mars (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',49900,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_49910','projected_crs','IAU_2015',49910,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',49915,'Mars (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',49900,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_49915','projected_crs','IAU_2015',49915,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',49920,'Mars (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',49900,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_49920','projected_crs','IAU_2015',49920,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',49925,'Mars (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',49900,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_49925','projected_crs','IAU_2015',49925,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',49930,'Mars (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',49900,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_49930','projected_crs','IAU_2015',49930,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',49935,'Mars (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',49900,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_49935','projected_crs','IAU_2015',49935,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',49940,'Mars (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',49900,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_49940','projected_crs','IAU_2015',49940,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',49945,'Mars (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',49900,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_49945','projected_crs','IAU_2015',49945,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',49950,'Mars (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',49900,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_49950','projected_crs','IAU_2015',49950,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',49955,'Mars (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',49900,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_49955','projected_crs','IAU_2015',49955,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',49960,'Mars (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',49900,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_49960','projected_crs','IAU_2015',49960,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',49965,'Mars (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',49900,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_49965','projected_crs','IAU_2015',49965,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',49970,'Mars (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',49900,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_49970','projected_crs','IAU_2015',49970,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',49975,'Mars (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',49900,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_49975','projected_crs','IAU_2015',49975,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',49980,'Mars (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',49900,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_49980','projected_crs','IAU_2015',49980,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',49985,'Mars (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',49900,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_49985','projected_crs','IAU_2015',49985,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',49990,'Mars (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',49900,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_49990','projected_crs','IAU_2015',49990,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO ellipsoid VALUES('IAU_2015',49901,'Mars (2015)',NULL,'IAU_2015',499,3396190.000000,'EPSG','9001',NULL,3376200.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',49901,'Mars (2015)','','IAU_2015',49901,'IAU_2015',49900,NULL,NULL,NULL,'Viking 1 lander: 47.95137 W',NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_49901','geodetic_datum','IAU_2015',49901,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',49901,'Mars (2015) / Ographic','Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','PROJ','OGRAPHIC_NORTH_WEST','IAU_2015',49901,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_49901','geodetic_crs','IAU_2015',49901,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',49911,'Mars (2015) / Ographic / Equirectangular, clon = 0',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',49901,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_49911','projected_crs','IAU_2015',49911,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',49916,'Mars (2015) / Ographic / Equirectangular, clon = 180',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',49901,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_49916','projected_crs','IAU_2015',49916,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',49921,'Mars (2015) / Ographic / Sinusoidal, clon = 0',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',49901,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_49921','projected_crs','IAU_2015',49921,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',49926,'Mars (2015) / Ographic / Sinusoidal, clon = 180',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',49901,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_49926','projected_crs','IAU_2015',49926,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',49931,'Mars (2015) / Ographic / North Polar',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',49901,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_49931','projected_crs','IAU_2015',49931,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',49936,'Mars (2015) / Ographic / South Polar',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',49901,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_49936','projected_crs','IAU_2015',49936,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',49941,'Mars (2015) / Ographic / Mollweide, clon = 0',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',49901,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_49941','projected_crs','IAU_2015',49941,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',49946,'Mars (2015) / Ographic / Mollweide, clon = 180',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',49901,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_49946','projected_crs','IAU_2015',49946,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',49951,'Mars (2015) / Ographic / Robinson, clon = 0',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',49901,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_49951','projected_crs','IAU_2015',49951,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',49956,'Mars (2015) / Ographic / Robinson, clon = 180',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',49901,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_49956','projected_crs','IAU_2015',49956,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',49961,'Mars (2015) / Ographic / Transverse Mercator',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',49901,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_49961','projected_crs','IAU_2015',49961,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',49966,'Mars (2015) / Ographic / Orthographic, clon = 0',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',49901,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_49966','projected_crs','IAU_2015',49966,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',49971,'Mars (2015) / Ographic / Orthographic, clon = 180',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',49901,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_49971','projected_crs','IAU_2015',49971,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',49976,'Mars (2015) / Ographic / Lambert Conic Conformal',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',49901,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_49976','projected_crs','IAU_2015',49976,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',49981,'Mars (2015) / Ographic / Lambert Azimuthal Equal Area',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',49901,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_49981','projected_crs','IAU_2015',49981,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',49986,'Mars (2015) / Ographic / Albers Equal Area',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',49901,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_49986','projected_crs','IAU_2015',49986,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',49991,'Mars (2015) / Ographic / Mercator',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',49901,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_49991','projected_crs','IAU_2015',49991,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',49902,'Mars (2015) / Ocentric','Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','other','PROJ','OCENTRIC_LAT_LON','IAU_2015',49901,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_49902','geodetic_crs','IAU_2015',49902,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',49912,'Mars (2015) / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',49902,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_49912','projected_crs','IAU_2015',49912,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',49917,'Mars (2015) / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',49902,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_49917','projected_crs','IAU_2015',49917,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',49922,'Mars (2015) / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',49902,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_49922','projected_crs','IAU_2015',49922,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',49927,'Mars (2015) / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',49902,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_49927','projected_crs','IAU_2015',49927,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',49932,'Mars (2015) / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',49902,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_49932','projected_crs','IAU_2015',49932,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',49937,'Mars (2015) / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',49902,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_49937','projected_crs','IAU_2015',49937,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',49942,'Mars (2015) / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',49902,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_49942','projected_crs','IAU_2015',49942,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',49947,'Mars (2015) / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',49902,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_49947','projected_crs','IAU_2015',49947,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',49952,'Mars (2015) / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',49902,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_49952','projected_crs','IAU_2015',49952,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',49957,'Mars (2015) / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',49902,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_49957','projected_crs','IAU_2015',49957,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',49962,'Mars (2015) / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',49902,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_49962','projected_crs','IAU_2015',49962,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',49967,'Mars (2015) / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',49902,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_49967','projected_crs','IAU_2015',49967,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',49972,'Mars (2015) / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',49902,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_49972','projected_crs','IAU_2015',49972,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',49977,'Mars (2015) / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',49902,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_49977','projected_crs','IAU_2015',49977,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',49982,'Mars (2015) / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',49902,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_49982','projected_crs','IAU_2015',49982,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',49987,'Mars (2015) / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',49902,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_49987','projected_crs','IAU_2015',49987,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',49992,'Mars (2015) / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',49902,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_49992','projected_crs','IAU_2015',49992,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 401, 'Phobos', 11080.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 40100, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',40100,'Phobos (2015) - Sphere',NULL,'IAU_2015',401,11080.000000,'EPSG','9001',NULL,11080.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',40100,'Phobos (2015) - Sphere','','IAU_2015',40100,'IAU_2015',40100,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_40100','geodetic_datum','IAU_2015',40100,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',40100,'Phobos (2015) - Sphere / Ocentric','Use mean radius as sphere radius for interoperability. Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',40100,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_40100','geodetic_crs','IAU_2015',40100,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',40110,'Phobos (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',40100,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_40110','projected_crs','IAU_2015',40110,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',40115,'Phobos (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',40100,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_40115','projected_crs','IAU_2015',40115,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',40120,'Phobos (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',40100,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_40120','projected_crs','IAU_2015',40120,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',40125,'Phobos (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',40100,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_40125','projected_crs','IAU_2015',40125,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',40130,'Phobos (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',40100,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_40130','projected_crs','IAU_2015',40130,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',40135,'Phobos (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',40100,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_40135','projected_crs','IAU_2015',40135,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',40140,'Phobos (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',40100,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_40140','projected_crs','IAU_2015',40140,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',40145,'Phobos (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',40100,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_40145','projected_crs','IAU_2015',40145,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',40150,'Phobos (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',40100,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_40150','projected_crs','IAU_2015',40150,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',40155,'Phobos (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',40100,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_40155','projected_crs','IAU_2015',40155,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',40160,'Phobos (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',40100,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_40160','projected_crs','IAU_2015',40160,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',40165,'Phobos (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',40100,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_40165','projected_crs','IAU_2015',40165,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',40170,'Phobos (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',40100,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_40170','projected_crs','IAU_2015',40170,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',40175,'Phobos (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',40100,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_40175','projected_crs','IAU_2015',40175,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',40180,'Phobos (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',40100,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_40180','projected_crs','IAU_2015',40180,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',40185,'Phobos (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',40100,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_40185','projected_crs','IAU_2015',40185,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',40190,'Phobos (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',40100,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_40190','projected_crs','IAU_2015',40190,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 402, 'Deimos', 6200.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 40200, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',40200,'Deimos (2015) - Sphere',NULL,'IAU_2015',402,6200.000000,'EPSG','9001',NULL,6200.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',40200,'Deimos (2015) - Sphere','','IAU_2015',40200,'IAU_2015',40200,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_40200','geodetic_datum','IAU_2015',40200,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',40200,'Deimos (2015) - Sphere / Ocentric','Use mean radius as sphere radius for interoperability. Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',40200,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_40200','geodetic_crs','IAU_2015',40200,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',40210,'Deimos (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',40200,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_40210','projected_crs','IAU_2015',40210,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',40215,'Deimos (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',40200,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_40215','projected_crs','IAU_2015',40215,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',40220,'Deimos (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',40200,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_40220','projected_crs','IAU_2015',40220,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',40225,'Deimos (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',40200,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_40225','projected_crs','IAU_2015',40225,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',40230,'Deimos (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',40200,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_40230','projected_crs','IAU_2015',40230,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',40235,'Deimos (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',40200,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_40235','projected_crs','IAU_2015',40235,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',40240,'Deimos (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',40200,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_40240','projected_crs','IAU_2015',40240,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',40245,'Deimos (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',40200,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_40245','projected_crs','IAU_2015',40245,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',40250,'Deimos (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',40200,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_40250','projected_crs','IAU_2015',40250,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',40255,'Deimos (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',40200,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_40255','projected_crs','IAU_2015',40255,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',40260,'Deimos (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',40200,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_40260','projected_crs','IAU_2015',40260,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',40265,'Deimos (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',40200,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_40265','projected_crs','IAU_2015',40265,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',40270,'Deimos (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',40200,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_40270','projected_crs','IAU_2015',40270,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',40275,'Deimos (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',40200,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_40275','projected_crs','IAU_2015',40275,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',40280,'Deimos (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',40200,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_40280','projected_crs','IAU_2015',40280,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',40285,'Deimos (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',40200,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_40285','projected_crs','IAU_2015',40285,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',40290,'Deimos (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',40200,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_40290','projected_crs','IAU_2015',40290,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 599, 'Jupiter', 71492000.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 59900, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',59900,'Jupiter (2015) - Sphere',NULL,'IAU_2015',599,71492000.000000,'EPSG','9001',NULL,71492000.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',59900,'Jupiter (2015) - Sphere','','IAU_2015',59900,'IAU_2015',59900,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_59900','geodetic_datum','IAU_2015',59900,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',59900,'Jupiter (2015) - Sphere / Ocentric','Use semi-major radius as sphere for interoperability. Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',59900,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_59900','geodetic_crs','IAU_2015',59900,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',59910,'Jupiter (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',59900,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_59910','projected_crs','IAU_2015',59910,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',59915,'Jupiter (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',59900,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_59915','projected_crs','IAU_2015',59915,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',59920,'Jupiter (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',59900,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_59920','projected_crs','IAU_2015',59920,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',59925,'Jupiter (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',59900,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_59925','projected_crs','IAU_2015',59925,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',59930,'Jupiter (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',59900,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_59930','projected_crs','IAU_2015',59930,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',59935,'Jupiter (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',59900,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_59935','projected_crs','IAU_2015',59935,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',59940,'Jupiter (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',59900,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_59940','projected_crs','IAU_2015',59940,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',59945,'Jupiter (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',59900,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_59945','projected_crs','IAU_2015',59945,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',59950,'Jupiter (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',59900,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_59950','projected_crs','IAU_2015',59950,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',59955,'Jupiter (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',59900,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_59955','projected_crs','IAU_2015',59955,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',59960,'Jupiter (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',59900,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_59960','projected_crs','IAU_2015',59960,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',59965,'Jupiter (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',59900,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_59965','projected_crs','IAU_2015',59965,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',59970,'Jupiter (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',59900,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_59970','projected_crs','IAU_2015',59970,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',59975,'Jupiter (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',59900,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_59975','projected_crs','IAU_2015',59975,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',59980,'Jupiter (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',59900,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_59980','projected_crs','IAU_2015',59980,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',59985,'Jupiter (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',59900,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_59985','projected_crs','IAU_2015',59985,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',59990,'Jupiter (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',59900,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_59990','projected_crs','IAU_2015',59990,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO ellipsoid VALUES('IAU_2015',59901,'Jupiter (2015)',NULL,'IAU_2015',599,71492000.000000,'EPSG','9001',NULL,66854000.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',59901,'Jupiter (2015)','','IAU_2015',59901,'IAU_2015',59900,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_59901','geodetic_datum','IAU_2015',59901,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',59901,'Jupiter (2015) / Ographic','Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','PROJ','OGRAPHIC_NORTH_WEST','IAU_2015',59901,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_59901','geodetic_crs','IAU_2015',59901,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',59911,'Jupiter (2015) / Ographic / Equirectangular, clon = 0',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',59901,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_59911','projected_crs','IAU_2015',59911,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',59916,'Jupiter (2015) / Ographic / Equirectangular, clon = 180',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',59901,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_59916','projected_crs','IAU_2015',59916,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',59921,'Jupiter (2015) / Ographic / Sinusoidal, clon = 0',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',59901,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_59921','projected_crs','IAU_2015',59921,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',59926,'Jupiter (2015) / Ographic / Sinusoidal, clon = 180',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',59901,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_59926','projected_crs','IAU_2015',59926,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',59931,'Jupiter (2015) / Ographic / North Polar',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',59901,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_59931','projected_crs','IAU_2015',59931,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',59936,'Jupiter (2015) / Ographic / South Polar',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',59901,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_59936','projected_crs','IAU_2015',59936,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',59941,'Jupiter (2015) / Ographic / Mollweide, clon = 0',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',59901,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_59941','projected_crs','IAU_2015',59941,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',59946,'Jupiter (2015) / Ographic / Mollweide, clon = 180',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',59901,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_59946','projected_crs','IAU_2015',59946,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',59951,'Jupiter (2015) / Ographic / Robinson, clon = 0',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',59901,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_59951','projected_crs','IAU_2015',59951,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',59956,'Jupiter (2015) / Ographic / Robinson, clon = 180',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',59901,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_59956','projected_crs','IAU_2015',59956,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',59961,'Jupiter (2015) / Ographic / Transverse Mercator',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',59901,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_59961','projected_crs','IAU_2015',59961,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',59966,'Jupiter (2015) / Ographic / Orthographic, clon = 0',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',59901,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_59966','projected_crs','IAU_2015',59966,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',59971,'Jupiter (2015) / Ographic / Orthographic, clon = 180',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',59901,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_59971','projected_crs','IAU_2015',59971,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',59976,'Jupiter (2015) / Ographic / Lambert Conic Conformal',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',59901,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_59976','projected_crs','IAU_2015',59976,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',59981,'Jupiter (2015) / Ographic / Lambert Azimuthal Equal Area',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',59901,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_59981','projected_crs','IAU_2015',59981,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',59986,'Jupiter (2015) / Ographic / Albers Equal Area',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',59901,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_59986','projected_crs','IAU_2015',59986,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',59991,'Jupiter (2015) / Ographic / Mercator',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',59901,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_59991','projected_crs','IAU_2015',59991,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',59902,'Jupiter (2015) / Ocentric','Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','other','PROJ','OCENTRIC_LAT_LON','IAU_2015',59901,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_59902','geodetic_crs','IAU_2015',59902,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',59912,'Jupiter (2015) / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',59902,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_59912','projected_crs','IAU_2015',59912,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',59917,'Jupiter (2015) / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',59902,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_59917','projected_crs','IAU_2015',59917,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',59922,'Jupiter (2015) / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',59902,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_59922','projected_crs','IAU_2015',59922,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',59927,'Jupiter (2015) / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',59902,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_59927','projected_crs','IAU_2015',59927,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',59932,'Jupiter (2015) / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',59902,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_59932','projected_crs','IAU_2015',59932,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',59937,'Jupiter (2015) / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',59902,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_59937','projected_crs','IAU_2015',59937,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',59942,'Jupiter (2015) / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',59902,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_59942','projected_crs','IAU_2015',59942,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',59947,'Jupiter (2015) / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',59902,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_59947','projected_crs','IAU_2015',59947,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',59952,'Jupiter (2015) / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',59902,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_59952','projected_crs','IAU_2015',59952,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',59957,'Jupiter (2015) / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',59902,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_59957','projected_crs','IAU_2015',59957,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',59962,'Jupiter (2015) / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',59902,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_59962','projected_crs','IAU_2015',59962,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',59967,'Jupiter (2015) / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',59902,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_59967','projected_crs','IAU_2015',59967,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',59972,'Jupiter (2015) / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',59902,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_59972','projected_crs','IAU_2015',59972,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',59977,'Jupiter (2015) / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',59902,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_59977','projected_crs','IAU_2015',59977,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',59982,'Jupiter (2015) / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',59902,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_59982','projected_crs','IAU_2015',59982,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',59987,'Jupiter (2015) / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',59902,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_59987','projected_crs','IAU_2015',59987,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',59992,'Jupiter (2015) / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',59902,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_59992','projected_crs','IAU_2015',59992,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 501, 'Io', 1821490.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 50100, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',50100,'Io (2015) - Sphere',NULL,'IAU_2015',501,1821490.000000,'EPSG','9001',NULL,1821490.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',50100,'Io (2015) - Sphere','','IAU_2015',50100,'IAU_2015',50100,NULL,NULL,NULL,'The mean sub-Jovian direction: 0.0',NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_50100','geodetic_datum','IAU_2015',50100,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',50100,'Io (2015) - Sphere / Ocentric','Use mean radius as sphere radius for interoperability. Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',50100,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_50100','geodetic_crs','IAU_2015',50100,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50110,'Io (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',50100,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50110','projected_crs','IAU_2015',50110,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50115,'Io (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',50100,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50115','projected_crs','IAU_2015',50115,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50120,'Io (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',50100,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50120','projected_crs','IAU_2015',50120,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50125,'Io (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',50100,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50125','projected_crs','IAU_2015',50125,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50130,'Io (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',50100,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50130','projected_crs','IAU_2015',50130,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50135,'Io (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',50100,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50135','projected_crs','IAU_2015',50135,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50140,'Io (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',50100,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50140','projected_crs','IAU_2015',50140,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50145,'Io (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',50100,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50145','projected_crs','IAU_2015',50145,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50150,'Io (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',50100,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50150','projected_crs','IAU_2015',50150,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50155,'Io (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',50100,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50155','projected_crs','IAU_2015',50155,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50160,'Io (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',50100,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50160','projected_crs','IAU_2015',50160,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50165,'Io (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',50100,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50165','projected_crs','IAU_2015',50165,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50170,'Io (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',50100,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50170','projected_crs','IAU_2015',50170,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50175,'Io (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',50100,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50175','projected_crs','IAU_2015',50175,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50180,'Io (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',50100,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50180','projected_crs','IAU_2015',50180,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50185,'Io (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',50100,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50185','projected_crs','IAU_2015',50185,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50190,'Io (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',50100,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50190','projected_crs','IAU_2015',50190,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 502, 'Europa', 1560800.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 50200, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',50200,'Europa (2015) - Sphere',NULL,'IAU_2015',502,1560800.000000,'EPSG','9001',NULL,1560800.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',50200,'Europa (2015) - Sphere','','IAU_2015',50200,'IAU_2015',50200,NULL,NULL,NULL,'Cilix: 182 W.0',NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_50200','geodetic_datum','IAU_2015',50200,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',50200,'Europa (2015) - Sphere / Ocentric','Use mean radius as sphere radius for interoperability. Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',50200,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_50200','geodetic_crs','IAU_2015',50200,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50210,'Europa (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',50200,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50210','projected_crs','IAU_2015',50210,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50215,'Europa (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',50200,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50215','projected_crs','IAU_2015',50215,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50220,'Europa (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',50200,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50220','projected_crs','IAU_2015',50220,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50225,'Europa (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',50200,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50225','projected_crs','IAU_2015',50225,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50230,'Europa (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',50200,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50230','projected_crs','IAU_2015',50230,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50235,'Europa (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',50200,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50235','projected_crs','IAU_2015',50235,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50240,'Europa (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',50200,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50240','projected_crs','IAU_2015',50240,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50245,'Europa (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',50200,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50245','projected_crs','IAU_2015',50245,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50250,'Europa (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',50200,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50250','projected_crs','IAU_2015',50250,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50255,'Europa (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',50200,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50255','projected_crs','IAU_2015',50255,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50260,'Europa (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',50200,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50260','projected_crs','IAU_2015',50260,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50265,'Europa (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',50200,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50265','projected_crs','IAU_2015',50265,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50270,'Europa (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',50200,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50270','projected_crs','IAU_2015',50270,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50275,'Europa (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',50200,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50275','projected_crs','IAU_2015',50275,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50280,'Europa (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',50200,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50280','projected_crs','IAU_2015',50280,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50285,'Europa (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',50200,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50285','projected_crs','IAU_2015',50285,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50290,'Europa (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',50200,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50290','projected_crs','IAU_2015',50290,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 503, 'Ganymede', 2631200.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 50300, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',50300,'Ganymede (2015) - Sphere',NULL,'IAU_2015',503,2631200.000000,'EPSG','9001',NULL,2631200.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',50300,'Ganymede (2015) - Sphere','','IAU_2015',50300,'IAU_2015',50300,NULL,NULL,NULL,'Anat: 128 W.0',NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_50300','geodetic_datum','IAU_2015',50300,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',50300,'Ganymede (2015) - Sphere / Ocentric','Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',50300,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_50300','geodetic_crs','IAU_2015',50300,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50310,'Ganymede (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',50300,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50310','projected_crs','IAU_2015',50310,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50315,'Ganymede (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',50300,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50315','projected_crs','IAU_2015',50315,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50320,'Ganymede (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',50300,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50320','projected_crs','IAU_2015',50320,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50325,'Ganymede (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',50300,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50325','projected_crs','IAU_2015',50325,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50330,'Ganymede (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',50300,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50330','projected_crs','IAU_2015',50330,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50335,'Ganymede (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',50300,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50335','projected_crs','IAU_2015',50335,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50340,'Ganymede (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',50300,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50340','projected_crs','IAU_2015',50340,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50345,'Ganymede (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',50300,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50345','projected_crs','IAU_2015',50345,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50350,'Ganymede (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',50300,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50350','projected_crs','IAU_2015',50350,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50355,'Ganymede (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',50300,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50355','projected_crs','IAU_2015',50355,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50360,'Ganymede (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',50300,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50360','projected_crs','IAU_2015',50360,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50365,'Ganymede (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',50300,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50365','projected_crs','IAU_2015',50365,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50370,'Ganymede (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',50300,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50370','projected_crs','IAU_2015',50370,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50375,'Ganymede (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',50300,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50375','projected_crs','IAU_2015',50375,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50380,'Ganymede (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',50300,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50380','projected_crs','IAU_2015',50380,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50385,'Ganymede (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',50300,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50385','projected_crs','IAU_2015',50385,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50390,'Ganymede (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',50300,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50390','projected_crs','IAU_2015',50390,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO ellipsoid VALUES('IAU_2015',50301,'Ganymede (2015)',NULL,'IAU_2015',503,2631200.000000,'EPSG','9001',NULL,2631200.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',50301,'Ganymede (2015)','','IAU_2015',50301,'IAU_2015',50300,NULL,NULL,NULL,'Anat: 128 W.0',NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_50301','geodetic_datum','IAU_2015',50301,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',50301,'Ganymede (2015) / Ographic','Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','PROJ','OGRAPHIC_NORTH_WEST','IAU_2015',50301,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_50301','geodetic_crs','IAU_2015',50301,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50311,'Ganymede (2015) / Ographic / Equirectangular, clon = 0',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',50301,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50311','projected_crs','IAU_2015',50311,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50316,'Ganymede (2015) / Ographic / Equirectangular, clon = 180',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',50301,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50316','projected_crs','IAU_2015',50316,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50321,'Ganymede (2015) / Ographic / Sinusoidal, clon = 0',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',50301,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50321','projected_crs','IAU_2015',50321,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50326,'Ganymede (2015) / Ographic / Sinusoidal, clon = 180',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',50301,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50326','projected_crs','IAU_2015',50326,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50331,'Ganymede (2015) / Ographic / North Polar',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',50301,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50331','projected_crs','IAU_2015',50331,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50336,'Ganymede (2015) / Ographic / South Polar',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',50301,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50336','projected_crs','IAU_2015',50336,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50341,'Ganymede (2015) / Ographic / Mollweide, clon = 0',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',50301,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50341','projected_crs','IAU_2015',50341,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50346,'Ganymede (2015) / Ographic / Mollweide, clon = 180',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',50301,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50346','projected_crs','IAU_2015',50346,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50351,'Ganymede (2015) / Ographic / Robinson, clon = 0',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',50301,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50351','projected_crs','IAU_2015',50351,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50356,'Ganymede (2015) / Ographic / Robinson, clon = 180',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',50301,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50356','projected_crs','IAU_2015',50356,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50361,'Ganymede (2015) / Ographic / Transverse Mercator',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',50301,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50361','projected_crs','IAU_2015',50361,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50366,'Ganymede (2015) / Ographic / Orthographic, clon = 0',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',50301,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50366','projected_crs','IAU_2015',50366,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50371,'Ganymede (2015) / Ographic / Orthographic, clon = 180',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',50301,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50371','projected_crs','IAU_2015',50371,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50376,'Ganymede (2015) / Ographic / Lambert Conic Conformal',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',50301,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50376','projected_crs','IAU_2015',50376,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50381,'Ganymede (2015) / Ographic / Lambert Azimuthal Equal Area',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',50301,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50381','projected_crs','IAU_2015',50381,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50386,'Ganymede (2015) / Ographic / Albers Equal Area',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',50301,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50386','projected_crs','IAU_2015',50386,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50391,'Ganymede (2015) / Ographic / Mercator',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',50301,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50391','projected_crs','IAU_2015',50391,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 504, 'Callisto', 2410300.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 50400, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',50400,'Callisto (2015) - Sphere',NULL,'IAU_2015',504,2410300.000000,'EPSG','9001',NULL,2410300.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',50400,'Callisto (2015) - Sphere','','IAU_2015',50400,'IAU_2015',50400,NULL,NULL,NULL,'Saga: 326 W.0',NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_50400','geodetic_datum','IAU_2015',50400,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',50400,'Callisto (2015) - Sphere / Ocentric','Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',50400,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_50400','geodetic_crs','IAU_2015',50400,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50410,'Callisto (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',50400,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50410','projected_crs','IAU_2015',50410,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50415,'Callisto (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',50400,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50415','projected_crs','IAU_2015',50415,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50420,'Callisto (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',50400,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50420','projected_crs','IAU_2015',50420,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50425,'Callisto (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',50400,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50425','projected_crs','IAU_2015',50425,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50430,'Callisto (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',50400,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50430','projected_crs','IAU_2015',50430,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50435,'Callisto (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',50400,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50435','projected_crs','IAU_2015',50435,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50440,'Callisto (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',50400,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50440','projected_crs','IAU_2015',50440,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50445,'Callisto (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',50400,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50445','projected_crs','IAU_2015',50445,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50450,'Callisto (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',50400,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50450','projected_crs','IAU_2015',50450,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50455,'Callisto (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',50400,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50455','projected_crs','IAU_2015',50455,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50460,'Callisto (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',50400,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50460','projected_crs','IAU_2015',50460,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50465,'Callisto (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',50400,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50465','projected_crs','IAU_2015',50465,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50470,'Callisto (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',50400,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50470','projected_crs','IAU_2015',50470,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50475,'Callisto (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',50400,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50475','projected_crs','IAU_2015',50475,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50480,'Callisto (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',50400,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50480','projected_crs','IAU_2015',50480,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50485,'Callisto (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',50400,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50485','projected_crs','IAU_2015',50485,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50490,'Callisto (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',50400,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50490','projected_crs','IAU_2015',50490,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO ellipsoid VALUES('IAU_2015',50401,'Callisto (2015)',NULL,'IAU_2015',504,2410300.000000,'EPSG','9001',NULL,2410300.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',50401,'Callisto (2015)','','IAU_2015',50401,'IAU_2015',50400,NULL,NULL,NULL,'Saga: 326 W.0',NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_50401','geodetic_datum','IAU_2015',50401,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',50401,'Callisto (2015) / Ographic','Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','PROJ','OGRAPHIC_NORTH_WEST','IAU_2015',50401,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_50401','geodetic_crs','IAU_2015',50401,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50411,'Callisto (2015) / Ographic / Equirectangular, clon = 0',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',50401,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50411','projected_crs','IAU_2015',50411,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50416,'Callisto (2015) / Ographic / Equirectangular, clon = 180',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',50401,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50416','projected_crs','IAU_2015',50416,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50421,'Callisto (2015) / Ographic / Sinusoidal, clon = 0',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',50401,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50421','projected_crs','IAU_2015',50421,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50426,'Callisto (2015) / Ographic / Sinusoidal, clon = 180',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',50401,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50426','projected_crs','IAU_2015',50426,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50431,'Callisto (2015) / Ographic / North Polar',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',50401,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50431','projected_crs','IAU_2015',50431,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50436,'Callisto (2015) / Ographic / South Polar',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',50401,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50436','projected_crs','IAU_2015',50436,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50441,'Callisto (2015) / Ographic / Mollweide, clon = 0',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',50401,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50441','projected_crs','IAU_2015',50441,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50446,'Callisto (2015) / Ographic / Mollweide, clon = 180',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',50401,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50446','projected_crs','IAU_2015',50446,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50451,'Callisto (2015) / Ographic / Robinson, clon = 0',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',50401,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50451','projected_crs','IAU_2015',50451,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50456,'Callisto (2015) / Ographic / Robinson, clon = 180',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',50401,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50456','projected_crs','IAU_2015',50456,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50461,'Callisto (2015) / Ographic / Transverse Mercator',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',50401,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50461','projected_crs','IAU_2015',50461,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50466,'Callisto (2015) / Ographic / Orthographic, clon = 0',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',50401,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50466','projected_crs','IAU_2015',50466,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50471,'Callisto (2015) / Ographic / Orthographic, clon = 180',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',50401,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50471','projected_crs','IAU_2015',50471,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50476,'Callisto (2015) / Ographic / Lambert Conic Conformal',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',50401,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50476','projected_crs','IAU_2015',50476,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50481,'Callisto (2015) / Ographic / Lambert Azimuthal Equal Area',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',50401,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50481','projected_crs','IAU_2015',50481,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50486,'Callisto (2015) / Ographic / Albers Equal Area',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',50401,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50486','projected_crs','IAU_2015',50486,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50491,'Callisto (2015) / Ographic / Mercator',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',50401,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50491','projected_crs','IAU_2015',50491,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 505, 'Amalthea', 83500.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 50500, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',50500,'Amalthea (2015) - Sphere',NULL,'IAU_2015',505,83500.000000,'EPSG','9001',NULL,83500.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',50500,'Amalthea (2015) - Sphere','','IAU_2015',50500,'IAU_2015',50500,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_50500','geodetic_datum','IAU_2015',50500,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',50500,'Amalthea (2015) - Sphere / Ocentric','Use mean radius as sphere radius for interoperability. Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',50500,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_50500','geodetic_crs','IAU_2015',50500,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50510,'Amalthea (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',50500,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50510','projected_crs','IAU_2015',50510,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50515,'Amalthea (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',50500,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50515','projected_crs','IAU_2015',50515,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50520,'Amalthea (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',50500,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50520','projected_crs','IAU_2015',50520,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50525,'Amalthea (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',50500,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50525','projected_crs','IAU_2015',50525,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50530,'Amalthea (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',50500,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50530','projected_crs','IAU_2015',50530,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50535,'Amalthea (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',50500,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50535','projected_crs','IAU_2015',50535,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50540,'Amalthea (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',50500,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50540','projected_crs','IAU_2015',50540,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50545,'Amalthea (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',50500,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50545','projected_crs','IAU_2015',50545,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50550,'Amalthea (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',50500,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50550','projected_crs','IAU_2015',50550,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50555,'Amalthea (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',50500,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50555','projected_crs','IAU_2015',50555,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50560,'Amalthea (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',50500,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50560','projected_crs','IAU_2015',50560,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50565,'Amalthea (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',50500,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50565','projected_crs','IAU_2015',50565,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50570,'Amalthea (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',50500,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50570','projected_crs','IAU_2015',50570,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50575,'Amalthea (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',50500,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50575','projected_crs','IAU_2015',50575,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50580,'Amalthea (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',50500,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50580','projected_crs','IAU_2015',50580,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50585,'Amalthea (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',50500,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50585','projected_crs','IAU_2015',50585,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50590,'Amalthea (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',50500,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50590','projected_crs','IAU_2015',50590,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 506, 'Himalia', 85000.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 50600, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',50600,'Himalia (2015) - Sphere',NULL,'IAU_2015',506,85000.000000,'EPSG','9001',NULL,85000.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',50600,'Himalia (2015) - Sphere','','IAU_2015',50600,'IAU_2015',50600,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_50600','geodetic_datum','IAU_2015',50600,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',50600,'Himalia (2015) - Sphere / Ocentric','Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',50600,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_50600','geodetic_crs','IAU_2015',50600,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50610,'Himalia (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',50600,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50610','projected_crs','IAU_2015',50610,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50615,'Himalia (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',50600,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50615','projected_crs','IAU_2015',50615,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50620,'Himalia (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',50600,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50620','projected_crs','IAU_2015',50620,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50625,'Himalia (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',50600,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50625','projected_crs','IAU_2015',50625,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50630,'Himalia (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',50600,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50630','projected_crs','IAU_2015',50630,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50635,'Himalia (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',50600,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50635','projected_crs','IAU_2015',50635,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50640,'Himalia (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',50600,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50640','projected_crs','IAU_2015',50640,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50645,'Himalia (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',50600,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50645','projected_crs','IAU_2015',50645,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50650,'Himalia (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',50600,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50650','projected_crs','IAU_2015',50650,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50655,'Himalia (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',50600,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50655','projected_crs','IAU_2015',50655,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50660,'Himalia (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',50600,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50660','projected_crs','IAU_2015',50660,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50665,'Himalia (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',50600,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50665','projected_crs','IAU_2015',50665,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50670,'Himalia (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',50600,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50670','projected_crs','IAU_2015',50670,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50675,'Himalia (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',50600,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50675','projected_crs','IAU_2015',50675,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50680,'Himalia (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',50600,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50680','projected_crs','IAU_2015',50680,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50685,'Himalia (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',50600,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50685','projected_crs','IAU_2015',50685,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50690,'Himalia (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',50600,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50690','projected_crs','IAU_2015',50690,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 507, 'Elara', 40000.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 50700, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',50700,'Elara (2015) - Sphere',NULL,'IAU_2015',507,40000.000000,'EPSG','9001',NULL,40000.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',50700,'Elara (2015) - Sphere','','IAU_2015',50700,'IAU_2015',50700,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_50700','geodetic_datum','IAU_2015',50700,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',50700,'Elara (2015) - Sphere / Ocentric','Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',50700,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_50700','geodetic_crs','IAU_2015',50700,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50710,'Elara (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',50700,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50710','projected_crs','IAU_2015',50710,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50715,'Elara (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',50700,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50715','projected_crs','IAU_2015',50715,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50720,'Elara (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',50700,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50720','projected_crs','IAU_2015',50720,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50725,'Elara (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',50700,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50725','projected_crs','IAU_2015',50725,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50730,'Elara (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',50700,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50730','projected_crs','IAU_2015',50730,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50735,'Elara (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',50700,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50735','projected_crs','IAU_2015',50735,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50740,'Elara (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',50700,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50740','projected_crs','IAU_2015',50740,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50745,'Elara (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',50700,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50745','projected_crs','IAU_2015',50745,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50750,'Elara (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',50700,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50750','projected_crs','IAU_2015',50750,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50755,'Elara (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',50700,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50755','projected_crs','IAU_2015',50755,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50760,'Elara (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',50700,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50760','projected_crs','IAU_2015',50760,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50765,'Elara (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',50700,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50765','projected_crs','IAU_2015',50765,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50770,'Elara (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',50700,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50770','projected_crs','IAU_2015',50770,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50775,'Elara (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',50700,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50775','projected_crs','IAU_2015',50775,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50780,'Elara (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',50700,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50780','projected_crs','IAU_2015',50780,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50785,'Elara (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',50700,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50785','projected_crs','IAU_2015',50785,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50790,'Elara (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',50700,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50790','projected_crs','IAU_2015',50790,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 508, 'Pasiphae', 18000.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 50800, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',50800,'Pasiphae (2015) - Sphere',NULL,'IAU_2015',508,18000.000000,'EPSG','9001',NULL,18000.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',50800,'Pasiphae (2015) - Sphere','','IAU_2015',50800,'IAU_2015',50800,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_50800','geodetic_datum','IAU_2015',50800,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',50800,'Pasiphae (2015) - Sphere / Ocentric','Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',50800,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_50800','geodetic_crs','IAU_2015',50800,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50810,'Pasiphae (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',50800,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50810','projected_crs','IAU_2015',50810,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50815,'Pasiphae (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',50800,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50815','projected_crs','IAU_2015',50815,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50820,'Pasiphae (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',50800,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50820','projected_crs','IAU_2015',50820,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50825,'Pasiphae (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',50800,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50825','projected_crs','IAU_2015',50825,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50830,'Pasiphae (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',50800,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50830','projected_crs','IAU_2015',50830,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50835,'Pasiphae (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',50800,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50835','projected_crs','IAU_2015',50835,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50840,'Pasiphae (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',50800,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50840','projected_crs','IAU_2015',50840,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50845,'Pasiphae (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',50800,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50845','projected_crs','IAU_2015',50845,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50850,'Pasiphae (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',50800,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50850','projected_crs','IAU_2015',50850,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50855,'Pasiphae (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',50800,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50855','projected_crs','IAU_2015',50855,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50860,'Pasiphae (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',50800,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50860','projected_crs','IAU_2015',50860,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50865,'Pasiphae (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',50800,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50865','projected_crs','IAU_2015',50865,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50870,'Pasiphae (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',50800,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50870','projected_crs','IAU_2015',50870,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50875,'Pasiphae (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',50800,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50875','projected_crs','IAU_2015',50875,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50880,'Pasiphae (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',50800,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50880','projected_crs','IAU_2015',50880,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50885,'Pasiphae (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',50800,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50885','projected_crs','IAU_2015',50885,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50890,'Pasiphae (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',50800,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50890','projected_crs','IAU_2015',50890,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 509, 'Sinope', 14000.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 50900, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',50900,'Sinope (2015) - Sphere',NULL,'IAU_2015',509,14000.000000,'EPSG','9001',NULL,14000.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',50900,'Sinope (2015) - Sphere','','IAU_2015',50900,'IAU_2015',50900,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_50900','geodetic_datum','IAU_2015',50900,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',50900,'Sinope (2015) - Sphere / Ocentric','Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',50900,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_50900','geodetic_crs','IAU_2015',50900,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50910,'Sinope (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',50900,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50910','projected_crs','IAU_2015',50910,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50915,'Sinope (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',50900,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50915','projected_crs','IAU_2015',50915,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50920,'Sinope (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',50900,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50920','projected_crs','IAU_2015',50920,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50925,'Sinope (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',50900,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50925','projected_crs','IAU_2015',50925,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50930,'Sinope (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',50900,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50930','projected_crs','IAU_2015',50930,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50935,'Sinope (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',50900,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50935','projected_crs','IAU_2015',50935,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50940,'Sinope (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',50900,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50940','projected_crs','IAU_2015',50940,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50945,'Sinope (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',50900,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50945','projected_crs','IAU_2015',50945,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50950,'Sinope (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',50900,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50950','projected_crs','IAU_2015',50950,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50955,'Sinope (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',50900,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50955','projected_crs','IAU_2015',50955,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50960,'Sinope (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',50900,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50960','projected_crs','IAU_2015',50960,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50965,'Sinope (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',50900,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50965','projected_crs','IAU_2015',50965,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50970,'Sinope (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',50900,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50970','projected_crs','IAU_2015',50970,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50975,'Sinope (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',50900,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50975','projected_crs','IAU_2015',50975,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50980,'Sinope (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',50900,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50980','projected_crs','IAU_2015',50980,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50985,'Sinope (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',50900,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50985','projected_crs','IAU_2015',50985,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',50990,'Sinope (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',50900,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_50990','projected_crs','IAU_2015',50990,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 510, 'Lysithea', 12000.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 51000, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',51000,'Lysithea (2015) - Sphere',NULL,'IAU_2015',510,12000.000000,'EPSG','9001',NULL,12000.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',51000,'Lysithea (2015) - Sphere','','IAU_2015',51000,'IAU_2015',51000,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_51000','geodetic_datum','IAU_2015',51000,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',51000,'Lysithea (2015) - Sphere / Ocentric','Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',51000,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_51000','geodetic_crs','IAU_2015',51000,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51010,'Lysithea (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',51000,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51010','projected_crs','IAU_2015',51010,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51015,'Lysithea (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',51000,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51015','projected_crs','IAU_2015',51015,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51020,'Lysithea (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',51000,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51020','projected_crs','IAU_2015',51020,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51025,'Lysithea (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',51000,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51025','projected_crs','IAU_2015',51025,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51030,'Lysithea (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',51000,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51030','projected_crs','IAU_2015',51030,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51035,'Lysithea (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',51000,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51035','projected_crs','IAU_2015',51035,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51040,'Lysithea (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',51000,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51040','projected_crs','IAU_2015',51040,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51045,'Lysithea (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',51000,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51045','projected_crs','IAU_2015',51045,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51050,'Lysithea (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',51000,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51050','projected_crs','IAU_2015',51050,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51055,'Lysithea (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',51000,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51055','projected_crs','IAU_2015',51055,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51060,'Lysithea (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',51000,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51060','projected_crs','IAU_2015',51060,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51065,'Lysithea (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',51000,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51065','projected_crs','IAU_2015',51065,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51070,'Lysithea (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',51000,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51070','projected_crs','IAU_2015',51070,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51075,'Lysithea (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',51000,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51075','projected_crs','IAU_2015',51075,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51080,'Lysithea (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',51000,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51080','projected_crs','IAU_2015',51080,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51085,'Lysithea (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',51000,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51085','projected_crs','IAU_2015',51085,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51090,'Lysithea (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',51000,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51090','projected_crs','IAU_2015',51090,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 511, 'Carme', 15000.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 51100, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',51100,'Carme (2015) - Sphere',NULL,'IAU_2015',511,15000.000000,'EPSG','9001',NULL,15000.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',51100,'Carme (2015) - Sphere','','IAU_2015',51100,'IAU_2015',51100,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_51100','geodetic_datum','IAU_2015',51100,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',51100,'Carme (2015) - Sphere / Ocentric','Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',51100,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_51100','geodetic_crs','IAU_2015',51100,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51110,'Carme (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',51100,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51110','projected_crs','IAU_2015',51110,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51115,'Carme (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',51100,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51115','projected_crs','IAU_2015',51115,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51120,'Carme (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',51100,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51120','projected_crs','IAU_2015',51120,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51125,'Carme (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',51100,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51125','projected_crs','IAU_2015',51125,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51130,'Carme (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',51100,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51130','projected_crs','IAU_2015',51130,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51135,'Carme (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',51100,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51135','projected_crs','IAU_2015',51135,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51140,'Carme (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',51100,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51140','projected_crs','IAU_2015',51140,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51145,'Carme (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',51100,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51145','projected_crs','IAU_2015',51145,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51150,'Carme (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',51100,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51150','projected_crs','IAU_2015',51150,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51155,'Carme (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',51100,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51155','projected_crs','IAU_2015',51155,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51160,'Carme (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',51100,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51160','projected_crs','IAU_2015',51160,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51165,'Carme (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',51100,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51165','projected_crs','IAU_2015',51165,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51170,'Carme (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',51100,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51170','projected_crs','IAU_2015',51170,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51175,'Carme (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',51100,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51175','projected_crs','IAU_2015',51175,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51180,'Carme (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',51100,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51180','projected_crs','IAU_2015',51180,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51185,'Carme (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',51100,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51185','projected_crs','IAU_2015',51185,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51190,'Carme (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',51100,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51190','projected_crs','IAU_2015',51190,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 512, 'Ananke', 10000.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 51200, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',51200,'Ananke (2015) - Sphere',NULL,'IAU_2015',512,10000.000000,'EPSG','9001',NULL,10000.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',51200,'Ananke (2015) - Sphere','','IAU_2015',51200,'IAU_2015',51200,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_51200','geodetic_datum','IAU_2015',51200,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',51200,'Ananke (2015) - Sphere / Ocentric','Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',51200,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_51200','geodetic_crs','IAU_2015',51200,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51210,'Ananke (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',51200,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51210','projected_crs','IAU_2015',51210,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51215,'Ananke (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',51200,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51215','projected_crs','IAU_2015',51215,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51220,'Ananke (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',51200,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51220','projected_crs','IAU_2015',51220,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51225,'Ananke (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',51200,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51225','projected_crs','IAU_2015',51225,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51230,'Ananke (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',51200,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51230','projected_crs','IAU_2015',51230,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51235,'Ananke (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',51200,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51235','projected_crs','IAU_2015',51235,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51240,'Ananke (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',51200,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51240','projected_crs','IAU_2015',51240,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51245,'Ananke (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',51200,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51245','projected_crs','IAU_2015',51245,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51250,'Ananke (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',51200,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51250','projected_crs','IAU_2015',51250,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51255,'Ananke (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',51200,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51255','projected_crs','IAU_2015',51255,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51260,'Ananke (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',51200,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51260','projected_crs','IAU_2015',51260,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51265,'Ananke (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',51200,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51265','projected_crs','IAU_2015',51265,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51270,'Ananke (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',51200,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51270','projected_crs','IAU_2015',51270,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51275,'Ananke (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',51200,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51275','projected_crs','IAU_2015',51275,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51280,'Ananke (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',51200,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51280','projected_crs','IAU_2015',51280,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51285,'Ananke (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',51200,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51285','projected_crs','IAU_2015',51285,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51290,'Ananke (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',51200,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51290','projected_crs','IAU_2015',51290,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 513, 'Leda', 5000.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 51300, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',51300,'Leda (2015) - Sphere',NULL,'IAU_2015',513,5000.000000,'EPSG','9001',NULL,5000.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',51300,'Leda (2015) - Sphere','','IAU_2015',51300,'IAU_2015',51300,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_51300','geodetic_datum','IAU_2015',51300,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',51300,'Leda (2015) - Sphere / Ocentric','Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',51300,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_51300','geodetic_crs','IAU_2015',51300,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51310,'Leda (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',51300,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51310','projected_crs','IAU_2015',51310,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51315,'Leda (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',51300,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51315','projected_crs','IAU_2015',51315,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51320,'Leda (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',51300,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51320','projected_crs','IAU_2015',51320,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51325,'Leda (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',51300,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51325','projected_crs','IAU_2015',51325,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51330,'Leda (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',51300,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51330','projected_crs','IAU_2015',51330,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51335,'Leda (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',51300,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51335','projected_crs','IAU_2015',51335,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51340,'Leda (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',51300,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51340','projected_crs','IAU_2015',51340,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51345,'Leda (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',51300,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51345','projected_crs','IAU_2015',51345,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51350,'Leda (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',51300,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51350','projected_crs','IAU_2015',51350,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51355,'Leda (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',51300,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51355','projected_crs','IAU_2015',51355,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51360,'Leda (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',51300,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51360','projected_crs','IAU_2015',51360,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51365,'Leda (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',51300,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51365','projected_crs','IAU_2015',51365,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51370,'Leda (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',51300,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51370','projected_crs','IAU_2015',51370,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51375,'Leda (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',51300,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51375','projected_crs','IAU_2015',51375,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51380,'Leda (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',51300,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51380','projected_crs','IAU_2015',51380,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51385,'Leda (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',51300,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51385','projected_crs','IAU_2015',51385,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51390,'Leda (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',51300,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51390','projected_crs','IAU_2015',51390,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 514, 'Thebe', 49300.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 51400, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',51400,'Thebe (2015) - Sphere',NULL,'IAU_2015',514,49300.000000,'EPSG','9001',NULL,49300.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',51400,'Thebe (2015) - Sphere','','IAU_2015',51400,'IAU_2015',51400,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_51400','geodetic_datum','IAU_2015',51400,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',51400,'Thebe (2015) - Sphere / Ocentric','Use mean radius as sphere radius for interoperability. Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',51400,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_51400','geodetic_crs','IAU_2015',51400,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51410,'Thebe (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',51400,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51410','projected_crs','IAU_2015',51410,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51415,'Thebe (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',51400,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51415','projected_crs','IAU_2015',51415,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51420,'Thebe (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',51400,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51420','projected_crs','IAU_2015',51420,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51425,'Thebe (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',51400,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51425','projected_crs','IAU_2015',51425,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51430,'Thebe (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',51400,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51430','projected_crs','IAU_2015',51430,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51435,'Thebe (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',51400,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51435','projected_crs','IAU_2015',51435,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51440,'Thebe (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',51400,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51440','projected_crs','IAU_2015',51440,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51445,'Thebe (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',51400,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51445','projected_crs','IAU_2015',51445,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51450,'Thebe (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',51400,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51450','projected_crs','IAU_2015',51450,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51455,'Thebe (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',51400,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51455','projected_crs','IAU_2015',51455,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51460,'Thebe (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',51400,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51460','projected_crs','IAU_2015',51460,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51465,'Thebe (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',51400,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51465','projected_crs','IAU_2015',51465,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51470,'Thebe (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',51400,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51470','projected_crs','IAU_2015',51470,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51475,'Thebe (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',51400,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51475','projected_crs','IAU_2015',51475,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51480,'Thebe (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',51400,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51480','projected_crs','IAU_2015',51480,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51485,'Thebe (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',51400,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51485','projected_crs','IAU_2015',51485,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51490,'Thebe (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',51400,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51490','projected_crs','IAU_2015',51490,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 515, 'Adrastea', 8200.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 51500, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',51500,'Adrastea (2015) - Sphere',NULL,'IAU_2015',515,8200.000000,'EPSG','9001',NULL,8200.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',51500,'Adrastea (2015) - Sphere','','IAU_2015',51500,'IAU_2015',51500,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_51500','geodetic_datum','IAU_2015',51500,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',51500,'Adrastea (2015) - Sphere / Ocentric','Use mean radius as sphere radius for interoperability. Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',51500,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_51500','geodetic_crs','IAU_2015',51500,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51510,'Adrastea (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',51500,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51510','projected_crs','IAU_2015',51510,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51515,'Adrastea (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',51500,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51515','projected_crs','IAU_2015',51515,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51520,'Adrastea (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',51500,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51520','projected_crs','IAU_2015',51520,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51525,'Adrastea (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',51500,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51525','projected_crs','IAU_2015',51525,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51530,'Adrastea (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',51500,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51530','projected_crs','IAU_2015',51530,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51535,'Adrastea (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',51500,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51535','projected_crs','IAU_2015',51535,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51540,'Adrastea (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',51500,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51540','projected_crs','IAU_2015',51540,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51545,'Adrastea (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',51500,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51545','projected_crs','IAU_2015',51545,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51550,'Adrastea (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',51500,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51550','projected_crs','IAU_2015',51550,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51555,'Adrastea (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',51500,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51555','projected_crs','IAU_2015',51555,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51560,'Adrastea (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',51500,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51560','projected_crs','IAU_2015',51560,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51565,'Adrastea (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',51500,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51565','projected_crs','IAU_2015',51565,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51570,'Adrastea (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',51500,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51570','projected_crs','IAU_2015',51570,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51575,'Adrastea (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',51500,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51575','projected_crs','IAU_2015',51575,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51580,'Adrastea (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',51500,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51580','projected_crs','IAU_2015',51580,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51585,'Adrastea (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',51500,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51585','projected_crs','IAU_2015',51585,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51590,'Adrastea (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',51500,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51590','projected_crs','IAU_2015',51590,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 516, 'Metis', 21500.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 51600, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',51600,'Metis (2015) - Sphere',NULL,'IAU_2015',516,21500.000000,'EPSG','9001',NULL,21500.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',51600,'Metis (2015) - Sphere','','IAU_2015',51600,'IAU_2015',51600,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_51600','geodetic_datum','IAU_2015',51600,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',51600,'Metis (2015) - Sphere / Ocentric','Use mean radius as sphere radius for interoperability. Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',51600,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_51600','geodetic_crs','IAU_2015',51600,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51610,'Metis (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',51600,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51610','projected_crs','IAU_2015',51610,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51615,'Metis (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',51600,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51615','projected_crs','IAU_2015',51615,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51620,'Metis (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',51600,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51620','projected_crs','IAU_2015',51620,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51625,'Metis (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',51600,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51625','projected_crs','IAU_2015',51625,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51630,'Metis (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',51600,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51630','projected_crs','IAU_2015',51630,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51635,'Metis (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',51600,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51635','projected_crs','IAU_2015',51635,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51640,'Metis (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',51600,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51640','projected_crs','IAU_2015',51640,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51645,'Metis (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',51600,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51645','projected_crs','IAU_2015',51645,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51650,'Metis (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',51600,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51650','projected_crs','IAU_2015',51650,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51655,'Metis (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',51600,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51655','projected_crs','IAU_2015',51655,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51660,'Metis (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',51600,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51660','projected_crs','IAU_2015',51660,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51665,'Metis (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',51600,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51665','projected_crs','IAU_2015',51665,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51670,'Metis (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',51600,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51670','projected_crs','IAU_2015',51670,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51675,'Metis (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',51600,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51675','projected_crs','IAU_2015',51675,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51680,'Metis (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',51600,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51680','projected_crs','IAU_2015',51680,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51685,'Metis (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',51600,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51685','projected_crs','IAU_2015',51685,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',51690,'Metis (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',51600,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_51690','projected_crs','IAU_2015',51690,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
-- Skip Callirrhoe has it lacks all axis information
-- Skip Themisto has it lacks all axis information
-- Skip Magaclite has it lacks all axis information
-- Skip Taygete has it lacks all axis information
-- Skip Chaldene has it lacks all axis information
-- Skip Harpalyke has it lacks all axis information
-- Skip Kalyke has it lacks all axis information
-- Skip Iocaste has it lacks all axis information
-- Skip Erinome has it lacks all axis information
-- Skip Isonoe has it lacks all axis information
-- Skip Praxidike has it lacks all axis information
-- Skip Autonoe has it lacks all axis information
-- Skip Thyone has it lacks all axis information
-- Skip Hermippe has it lacks all axis information
-- Skip Aitne has it lacks all axis information
-- Skip Eurydome has it lacks all axis information
-- Skip Euanthe has it lacks all axis information
-- Skip Euporie has it lacks all axis information
-- Skip Orthosie has it lacks all axis information
-- Skip Sponde has it lacks all axis information
-- Skip Kale has it lacks all axis information
-- Skip Pasithee has it lacks all axis information
-- Skip Hegemone has it lacks all axis information
-- Skip Mneme has it lacks all axis information
-- Skip Aoede has it lacks all axis information
-- Skip Thelxinoe has it lacks all axis information
-- Skip Arche has it lacks all axis information
-- Skip Kallichore has it lacks all axis information
-- Skip Helike has it lacks all axis information
-- Skip Carpo has it lacks all axis information
-- Skip Eukelade has it lacks all axis information
-- Skip Cyllene has it lacks all axis information
-- Skip Kore has it lacks all axis information
-- Skip Herse has it lacks all axis information
INSERT INTO celestial_body VALUES('IAU_2015', 699, 'Saturn', 60268000.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 69900, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',69900,'Saturn (2015) - Sphere',NULL,'IAU_2015',699,60268000.000000,'EPSG','9001',NULL,60268000.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',69900,'Saturn (2015) - Sphere','','IAU_2015',69900,'IAU_2015',69900,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_69900','geodetic_datum','IAU_2015',69900,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',69900,'Saturn (2015) - Sphere / Ocentric','Use semi-major radius as sphere for interoperability. Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',69900,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_69900','geodetic_crs','IAU_2015',69900,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',69910,'Saturn (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',69900,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_69910','projected_crs','IAU_2015',69910,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',69915,'Saturn (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',69900,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_69915','projected_crs','IAU_2015',69915,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',69920,'Saturn (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',69900,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_69920','projected_crs','IAU_2015',69920,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',69925,'Saturn (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',69900,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_69925','projected_crs','IAU_2015',69925,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',69930,'Saturn (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',69900,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_69930','projected_crs','IAU_2015',69930,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',69935,'Saturn (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',69900,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_69935','projected_crs','IAU_2015',69935,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',69940,'Saturn (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',69900,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_69940','projected_crs','IAU_2015',69940,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',69945,'Saturn (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',69900,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_69945','projected_crs','IAU_2015',69945,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',69950,'Saturn (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',69900,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_69950','projected_crs','IAU_2015',69950,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',69955,'Saturn (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',69900,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_69955','projected_crs','IAU_2015',69955,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',69960,'Saturn (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',69900,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_69960','projected_crs','IAU_2015',69960,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',69965,'Saturn (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',69900,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_69965','projected_crs','IAU_2015',69965,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',69970,'Saturn (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',69900,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_69970','projected_crs','IAU_2015',69970,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',69975,'Saturn (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',69900,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_69975','projected_crs','IAU_2015',69975,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',69980,'Saturn (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',69900,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_69980','projected_crs','IAU_2015',69980,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',69985,'Saturn (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',69900,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_69985','projected_crs','IAU_2015',69985,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',69990,'Saturn (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',69900,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_69990','projected_crs','IAU_2015',69990,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO ellipsoid VALUES('IAU_2015',69901,'Saturn (2015)',NULL,'IAU_2015',699,60268000.000000,'EPSG','9001',NULL,54364000.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',69901,'Saturn (2015)','','IAU_2015',69901,'IAU_2015',69900,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_69901','geodetic_datum','IAU_2015',69901,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',69901,'Saturn (2015) / Ographic','Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','PROJ','OGRAPHIC_NORTH_WEST','IAU_2015',69901,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_69901','geodetic_crs','IAU_2015',69901,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',69911,'Saturn (2015) / Ographic / Equirectangular, clon = 0',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',69901,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_69911','projected_crs','IAU_2015',69911,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',69916,'Saturn (2015) / Ographic / Equirectangular, clon = 180',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',69901,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_69916','projected_crs','IAU_2015',69916,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',69921,'Saturn (2015) / Ographic / Sinusoidal, clon = 0',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',69901,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_69921','projected_crs','IAU_2015',69921,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',69926,'Saturn (2015) / Ographic / Sinusoidal, clon = 180',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',69901,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_69926','projected_crs','IAU_2015',69926,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',69931,'Saturn (2015) / Ographic / North Polar',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',69901,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_69931','projected_crs','IAU_2015',69931,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',69936,'Saturn (2015) / Ographic / South Polar',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',69901,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_69936','projected_crs','IAU_2015',69936,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',69941,'Saturn (2015) / Ographic / Mollweide, clon = 0',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',69901,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_69941','projected_crs','IAU_2015',69941,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',69946,'Saturn (2015) / Ographic / Mollweide, clon = 180',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',69901,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_69946','projected_crs','IAU_2015',69946,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',69951,'Saturn (2015) / Ographic / Robinson, clon = 0',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',69901,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_69951','projected_crs','IAU_2015',69951,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',69956,'Saturn (2015) / Ographic / Robinson, clon = 180',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',69901,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_69956','projected_crs','IAU_2015',69956,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',69961,'Saturn (2015) / Ographic / Transverse Mercator',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',69901,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_69961','projected_crs','IAU_2015',69961,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',69966,'Saturn (2015) / Ographic / Orthographic, clon = 0',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',69901,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_69966','projected_crs','IAU_2015',69966,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',69971,'Saturn (2015) / Ographic / Orthographic, clon = 180',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',69901,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_69971','projected_crs','IAU_2015',69971,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',69976,'Saturn (2015) / Ographic / Lambert Conic Conformal',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',69901,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_69976','projected_crs','IAU_2015',69976,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',69981,'Saturn (2015) / Ographic / Lambert Azimuthal Equal Area',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',69901,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_69981','projected_crs','IAU_2015',69981,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',69986,'Saturn (2015) / Ographic / Albers Equal Area',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',69901,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_69986','projected_crs','IAU_2015',69986,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',69991,'Saturn (2015) / Ographic / Mercator',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',69901,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_69991','projected_crs','IAU_2015',69991,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',69902,'Saturn (2015) / Ocentric','Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','other','PROJ','OCENTRIC_LAT_LON','IAU_2015',69901,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_69902','geodetic_crs','IAU_2015',69902,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',69912,'Saturn (2015) / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',69902,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_69912','projected_crs','IAU_2015',69912,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',69917,'Saturn (2015) / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',69902,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_69917','projected_crs','IAU_2015',69917,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',69922,'Saturn (2015) / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',69902,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_69922','projected_crs','IAU_2015',69922,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',69927,'Saturn (2015) / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',69902,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_69927','projected_crs','IAU_2015',69927,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',69932,'Saturn (2015) / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',69902,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_69932','projected_crs','IAU_2015',69932,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',69937,'Saturn (2015) / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',69902,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_69937','projected_crs','IAU_2015',69937,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',69942,'Saturn (2015) / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',69902,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_69942','projected_crs','IAU_2015',69942,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',69947,'Saturn (2015) / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',69902,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_69947','projected_crs','IAU_2015',69947,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',69952,'Saturn (2015) / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',69902,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_69952','projected_crs','IAU_2015',69952,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',69957,'Saturn (2015) / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',69902,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_69957','projected_crs','IAU_2015',69957,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',69962,'Saturn (2015) / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',69902,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_69962','projected_crs','IAU_2015',69962,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',69967,'Saturn (2015) / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',69902,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_69967','projected_crs','IAU_2015',69967,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',69972,'Saturn (2015) / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',69902,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_69972','projected_crs','IAU_2015',69972,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',69977,'Saturn (2015) / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',69902,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_69977','projected_crs','IAU_2015',69977,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',69982,'Saturn (2015) / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',69902,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_69982','projected_crs','IAU_2015',69982,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',69987,'Saturn (2015) / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',69902,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_69987','projected_crs','IAU_2015',69987,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',69992,'Saturn (2015) / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',69902,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_69992','projected_crs','IAU_2015',69992,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 601, 'Mimas', 198200.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 60100, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',60100,'Mimas (2015) - Sphere',NULL,'IAU_2015',601,198200.000000,'EPSG','9001',NULL,198200.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',60100,'Mimas (2015) - Sphere','','IAU_2015',60100,'IAU_2015',60100,NULL,NULL,NULL,'Palomides: 162 W.0',NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_60100','geodetic_datum','IAU_2015',60100,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',60100,'Mimas (2015) - Sphere / Ocentric','Use mean radius as sphere radius for interoperability. Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',60100,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_60100','geodetic_crs','IAU_2015',60100,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60110,'Mimas (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',60100,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60110','projected_crs','IAU_2015',60110,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60115,'Mimas (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',60100,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60115','projected_crs','IAU_2015',60115,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60120,'Mimas (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',60100,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60120','projected_crs','IAU_2015',60120,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60125,'Mimas (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',60100,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60125','projected_crs','IAU_2015',60125,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60130,'Mimas (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',60100,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60130','projected_crs','IAU_2015',60130,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60135,'Mimas (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',60100,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60135','projected_crs','IAU_2015',60135,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60140,'Mimas (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',60100,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60140','projected_crs','IAU_2015',60140,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60145,'Mimas (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',60100,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60145','projected_crs','IAU_2015',60145,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60150,'Mimas (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',60100,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60150','projected_crs','IAU_2015',60150,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60155,'Mimas (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',60100,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60155','projected_crs','IAU_2015',60155,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60160,'Mimas (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',60100,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60160','projected_crs','IAU_2015',60160,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60165,'Mimas (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',60100,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60165','projected_crs','IAU_2015',60165,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60170,'Mimas (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',60100,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60170','projected_crs','IAU_2015',60170,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60175,'Mimas (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',60100,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60175','projected_crs','IAU_2015',60175,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60180,'Mimas (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',60100,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60180','projected_crs','IAU_2015',60180,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60185,'Mimas (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',60100,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60185','projected_crs','IAU_2015',60185,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60190,'Mimas (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',60100,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60190','projected_crs','IAU_2015',60190,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 602, 'Enceladus', 252100.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 60200, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',60200,'Enceladus (2015) - Sphere',NULL,'IAU_2015',602,252100.000000,'EPSG','9001',NULL,252100.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',60200,'Enceladus (2015) - Sphere','','IAU_2015',60200,'IAU_2015',60200,NULL,NULL,NULL,'Salih: 5 W.0',NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_60200','geodetic_datum','IAU_2015',60200,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',60200,'Enceladus (2015) - Sphere / Ocentric','Use mean radius as sphere radius for interoperability. Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',60200,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_60200','geodetic_crs','IAU_2015',60200,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60210,'Enceladus (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',60200,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60210','projected_crs','IAU_2015',60210,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60215,'Enceladus (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',60200,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60215','projected_crs','IAU_2015',60215,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60220,'Enceladus (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',60200,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60220','projected_crs','IAU_2015',60220,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60225,'Enceladus (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',60200,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60225','projected_crs','IAU_2015',60225,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60230,'Enceladus (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',60200,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60230','projected_crs','IAU_2015',60230,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60235,'Enceladus (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',60200,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60235','projected_crs','IAU_2015',60235,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60240,'Enceladus (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',60200,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60240','projected_crs','IAU_2015',60240,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60245,'Enceladus (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',60200,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60245','projected_crs','IAU_2015',60245,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60250,'Enceladus (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',60200,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60250','projected_crs','IAU_2015',60250,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60255,'Enceladus (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',60200,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60255','projected_crs','IAU_2015',60255,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60260,'Enceladus (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',60200,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60260','projected_crs','IAU_2015',60260,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60265,'Enceladus (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',60200,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60265','projected_crs','IAU_2015',60265,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60270,'Enceladus (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',60200,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60270','projected_crs','IAU_2015',60270,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60275,'Enceladus (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',60200,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60275','projected_crs','IAU_2015',60275,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60280,'Enceladus (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',60200,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60280','projected_crs','IAU_2015',60280,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60285,'Enceladus (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',60200,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60285','projected_crs','IAU_2015',60285,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60290,'Enceladus (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',60200,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60290','projected_crs','IAU_2015',60290,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 603, 'Tethys', 531000.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 60300, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',60300,'Tethys (2015) - Sphere',NULL,'IAU_2015',603,531000.000000,'EPSG','9001',NULL,531000.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',60300,'Tethys (2015) - Sphere','','IAU_2015',60300,'IAU_2015',60300,NULL,NULL,NULL,'Arete: 299 W.0',NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_60300','geodetic_datum','IAU_2015',60300,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',60300,'Tethys (2015) - Sphere / Ocentric','Use mean radius as sphere radius for interoperability. Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',60300,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_60300','geodetic_crs','IAU_2015',60300,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60310,'Tethys (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',60300,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60310','projected_crs','IAU_2015',60310,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60315,'Tethys (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',60300,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60315','projected_crs','IAU_2015',60315,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60320,'Tethys (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',60300,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60320','projected_crs','IAU_2015',60320,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60325,'Tethys (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',60300,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60325','projected_crs','IAU_2015',60325,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60330,'Tethys (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',60300,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60330','projected_crs','IAU_2015',60330,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60335,'Tethys (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',60300,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60335','projected_crs','IAU_2015',60335,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60340,'Tethys (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',60300,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60340','projected_crs','IAU_2015',60340,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60345,'Tethys (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',60300,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60345','projected_crs','IAU_2015',60345,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60350,'Tethys (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',60300,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60350','projected_crs','IAU_2015',60350,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60355,'Tethys (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',60300,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60355','projected_crs','IAU_2015',60355,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60360,'Tethys (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',60300,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60360','projected_crs','IAU_2015',60360,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60365,'Tethys (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',60300,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60365','projected_crs','IAU_2015',60365,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60370,'Tethys (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',60300,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60370','projected_crs','IAU_2015',60370,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60375,'Tethys (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',60300,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60375','projected_crs','IAU_2015',60375,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60380,'Tethys (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',60300,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60380','projected_crs','IAU_2015',60380,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60385,'Tethys (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',60300,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60385','projected_crs','IAU_2015',60385,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60390,'Tethys (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',60300,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60390','projected_crs','IAU_2015',60390,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 604, 'Dione', 561400.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 60400, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',60400,'Dione (2015) - Sphere',NULL,'IAU_2015',604,561400.000000,'EPSG','9001',NULL,561400.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',60400,'Dione (2015) - Sphere','','IAU_2015',60400,'IAU_2015',60400,NULL,NULL,NULL,'Palinurus: 63 W.0',NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_60400','geodetic_datum','IAU_2015',60400,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',60400,'Dione (2015) - Sphere / Ocentric','Use mean radius as sphere radius for interoperability. Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',60400,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_60400','geodetic_crs','IAU_2015',60400,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60410,'Dione (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',60400,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60410','projected_crs','IAU_2015',60410,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60415,'Dione (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',60400,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60415','projected_crs','IAU_2015',60415,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60420,'Dione (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',60400,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60420','projected_crs','IAU_2015',60420,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60425,'Dione (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',60400,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60425','projected_crs','IAU_2015',60425,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60430,'Dione (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',60400,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60430','projected_crs','IAU_2015',60430,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60435,'Dione (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',60400,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60435','projected_crs','IAU_2015',60435,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60440,'Dione (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',60400,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60440','projected_crs','IAU_2015',60440,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60445,'Dione (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',60400,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60445','projected_crs','IAU_2015',60445,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60450,'Dione (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',60400,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60450','projected_crs','IAU_2015',60450,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60455,'Dione (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',60400,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60455','projected_crs','IAU_2015',60455,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60460,'Dione (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',60400,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60460','projected_crs','IAU_2015',60460,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60465,'Dione (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',60400,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60465','projected_crs','IAU_2015',60465,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60470,'Dione (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',60400,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60470','projected_crs','IAU_2015',60470,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60475,'Dione (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',60400,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60475','projected_crs','IAU_2015',60475,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60480,'Dione (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',60400,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60480','projected_crs','IAU_2015',60480,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60485,'Dione (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',60400,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60485','projected_crs','IAU_2015',60485,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60490,'Dione (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',60400,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60490','projected_crs','IAU_2015',60490,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 605, 'Rhea', 763500.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 60500, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',60500,'Rhea (2015) - Sphere',NULL,'IAU_2015',605,763500.000000,'EPSG','9001',NULL,763500.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',60500,'Rhea (2015) - Sphere','','IAU_2015',60500,'IAU_2015',60500,NULL,NULL,NULL,'Tore: 340 W.0',NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_60500','geodetic_datum','IAU_2015',60500,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',60500,'Rhea (2015) - Sphere / Ocentric','Use mean radius as sphere radius for interoperability. Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',60500,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_60500','geodetic_crs','IAU_2015',60500,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60510,'Rhea (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',60500,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60510','projected_crs','IAU_2015',60510,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60515,'Rhea (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',60500,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60515','projected_crs','IAU_2015',60515,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60520,'Rhea (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',60500,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60520','projected_crs','IAU_2015',60520,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60525,'Rhea (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',60500,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60525','projected_crs','IAU_2015',60525,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60530,'Rhea (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',60500,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60530','projected_crs','IAU_2015',60530,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60535,'Rhea (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',60500,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60535','projected_crs','IAU_2015',60535,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60540,'Rhea (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',60500,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60540','projected_crs','IAU_2015',60540,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60545,'Rhea (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',60500,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60545','projected_crs','IAU_2015',60545,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60550,'Rhea (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',60500,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60550','projected_crs','IAU_2015',60550,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60555,'Rhea (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',60500,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60555','projected_crs','IAU_2015',60555,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60560,'Rhea (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',60500,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60560','projected_crs','IAU_2015',60560,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60565,'Rhea (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',60500,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60565','projected_crs','IAU_2015',60565,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60570,'Rhea (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',60500,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60570','projected_crs','IAU_2015',60570,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60575,'Rhea (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',60500,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60575','projected_crs','IAU_2015',60575,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60580,'Rhea (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',60500,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60580','projected_crs','IAU_2015',60580,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60585,'Rhea (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',60500,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60585','projected_crs','IAU_2015',60585,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60590,'Rhea (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',60500,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60590','projected_crs','IAU_2015',60590,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 606, 'Titan', 2575000.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 60600, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',60600,'Titan (2015) - Sphere',NULL,'IAU_2015',606,2575000.000000,'EPSG','9001',NULL,2575000.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',60600,'Titan (2015) - Sphere','','IAU_2015',60600,'IAU_2015',60600,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_60600','geodetic_datum','IAU_2015',60600,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',60600,'Titan (2015) - Sphere / Ocentric','Use mean radius as sphere radius for interoperability. Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',60600,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_60600','geodetic_crs','IAU_2015',60600,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60610,'Titan (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',60600,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60610','projected_crs','IAU_2015',60610,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60615,'Titan (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',60600,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60615','projected_crs','IAU_2015',60615,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60620,'Titan (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',60600,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60620','projected_crs','IAU_2015',60620,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60625,'Titan (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',60600,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60625','projected_crs','IAU_2015',60625,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60630,'Titan (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',60600,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60630','projected_crs','IAU_2015',60630,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60635,'Titan (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',60600,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60635','projected_crs','IAU_2015',60635,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60640,'Titan (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',60600,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60640','projected_crs','IAU_2015',60640,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60645,'Titan (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',60600,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60645','projected_crs','IAU_2015',60645,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60650,'Titan (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',60600,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60650','projected_crs','IAU_2015',60650,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60655,'Titan (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',60600,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60655','projected_crs','IAU_2015',60655,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60660,'Titan (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',60600,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60660','projected_crs','IAU_2015',60660,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60665,'Titan (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',60600,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60665','projected_crs','IAU_2015',60665,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60670,'Titan (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',60600,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60670','projected_crs','IAU_2015',60670,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60675,'Titan (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',60600,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60675','projected_crs','IAU_2015',60675,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60680,'Titan (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',60600,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60680','projected_crs','IAU_2015',60680,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60685,'Titan (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',60600,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60685','projected_crs','IAU_2015',60685,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60690,'Titan (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',60600,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60690','projected_crs','IAU_2015',60690,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 607, 'Hyperion', 135000.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 60700, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',60700,'Hyperion (2015) - Sphere',NULL,'IAU_2015',607,135000.000000,'EPSG','9001',NULL,135000.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',60700,'Hyperion (2015) - Sphere','','IAU_2015',60700,'IAU_2015',60700,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_60700','geodetic_datum','IAU_2015',60700,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',60700,'Hyperion (2015) - Sphere / Ocentric','Use mean radius as sphere radius for interoperability. Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',60700,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_60700','geodetic_crs','IAU_2015',60700,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60710,'Hyperion (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',60700,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60710','projected_crs','IAU_2015',60710,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60715,'Hyperion (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',60700,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60715','projected_crs','IAU_2015',60715,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60720,'Hyperion (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',60700,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60720','projected_crs','IAU_2015',60720,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60725,'Hyperion (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',60700,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60725','projected_crs','IAU_2015',60725,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60730,'Hyperion (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',60700,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60730','projected_crs','IAU_2015',60730,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60735,'Hyperion (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',60700,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60735','projected_crs','IAU_2015',60735,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60740,'Hyperion (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',60700,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60740','projected_crs','IAU_2015',60740,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60745,'Hyperion (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',60700,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60745','projected_crs','IAU_2015',60745,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60750,'Hyperion (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',60700,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60750','projected_crs','IAU_2015',60750,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60755,'Hyperion (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',60700,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60755','projected_crs','IAU_2015',60755,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60760,'Hyperion (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',60700,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60760','projected_crs','IAU_2015',60760,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60765,'Hyperion (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',60700,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60765','projected_crs','IAU_2015',60765,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60770,'Hyperion (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',60700,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60770','projected_crs','IAU_2015',60770,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60775,'Hyperion (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',60700,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60775','projected_crs','IAU_2015',60775,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60780,'Hyperion (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',60700,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60780','projected_crs','IAU_2015',60780,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60785,'Hyperion (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',60700,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60785','projected_crs','IAU_2015',60785,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60790,'Hyperion (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',60700,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60790','projected_crs','IAU_2015',60790,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 608, 'Iapetus', 745700.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 60800, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',60800,'Iapetus (2015) - Sphere',NULL,'IAU_2015',608,745700.000000,'EPSG','9001',NULL,745700.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',60800,'Iapetus (2015) - Sphere','','IAU_2015',60800,'IAU_2015',60800,NULL,NULL,NULL,'Almeric: 276 W.0',NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_60800','geodetic_datum','IAU_2015',60800,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',60800,'Iapetus (2015) - Sphere / Ocentric','Use semi-major radius as sphere for interoperability. Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',60800,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_60800','geodetic_crs','IAU_2015',60800,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60810,'Iapetus (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',60800,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60810','projected_crs','IAU_2015',60810,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60815,'Iapetus (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',60800,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60815','projected_crs','IAU_2015',60815,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60820,'Iapetus (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',60800,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60820','projected_crs','IAU_2015',60820,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60825,'Iapetus (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',60800,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60825','projected_crs','IAU_2015',60825,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60830,'Iapetus (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',60800,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60830','projected_crs','IAU_2015',60830,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60835,'Iapetus (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',60800,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60835','projected_crs','IAU_2015',60835,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60840,'Iapetus (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',60800,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60840','projected_crs','IAU_2015',60840,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60845,'Iapetus (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',60800,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60845','projected_crs','IAU_2015',60845,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60850,'Iapetus (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',60800,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60850','projected_crs','IAU_2015',60850,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60855,'Iapetus (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',60800,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60855','projected_crs','IAU_2015',60855,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60860,'Iapetus (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',60800,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60860','projected_crs','IAU_2015',60860,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60865,'Iapetus (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',60800,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60865','projected_crs','IAU_2015',60865,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60870,'Iapetus (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',60800,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60870','projected_crs','IAU_2015',60870,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60875,'Iapetus (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',60800,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60875','projected_crs','IAU_2015',60875,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60880,'Iapetus (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',60800,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60880','projected_crs','IAU_2015',60880,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60885,'Iapetus (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',60800,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60885','projected_crs','IAU_2015',60885,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60890,'Iapetus (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',60800,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60890','projected_crs','IAU_2015',60890,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO ellipsoid VALUES('IAU_2015',60801,'Iapetus (2015)',NULL,'IAU_2015',608,745700.000000,'EPSG','9001',NULL,712100.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',60801,'Iapetus (2015)','','IAU_2015',60801,'IAU_2015',60800,NULL,NULL,NULL,'Almeric: 276 W.0',NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_60801','geodetic_datum','IAU_2015',60801,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',60801,'Iapetus (2015) / Ographic','Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','PROJ','OGRAPHIC_NORTH_WEST','IAU_2015',60801,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_60801','geodetic_crs','IAU_2015',60801,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60811,'Iapetus (2015) / Ographic / Equirectangular, clon = 0',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',60801,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60811','projected_crs','IAU_2015',60811,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60816,'Iapetus (2015) / Ographic / Equirectangular, clon = 180',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',60801,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60816','projected_crs','IAU_2015',60816,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60821,'Iapetus (2015) / Ographic / Sinusoidal, clon = 0',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',60801,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60821','projected_crs','IAU_2015',60821,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60826,'Iapetus (2015) / Ographic / Sinusoidal, clon = 180',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',60801,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60826','projected_crs','IAU_2015',60826,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60831,'Iapetus (2015) / Ographic / North Polar',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',60801,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60831','projected_crs','IAU_2015',60831,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60836,'Iapetus (2015) / Ographic / South Polar',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',60801,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60836','projected_crs','IAU_2015',60836,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60841,'Iapetus (2015) / Ographic / Mollweide, clon = 0',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',60801,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60841','projected_crs','IAU_2015',60841,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60846,'Iapetus (2015) / Ographic / Mollweide, clon = 180',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',60801,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60846','projected_crs','IAU_2015',60846,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60851,'Iapetus (2015) / Ographic / Robinson, clon = 0',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',60801,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60851','projected_crs','IAU_2015',60851,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60856,'Iapetus (2015) / Ographic / Robinson, clon = 180',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',60801,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60856','projected_crs','IAU_2015',60856,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60861,'Iapetus (2015) / Ographic / Transverse Mercator',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',60801,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60861','projected_crs','IAU_2015',60861,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60866,'Iapetus (2015) / Ographic / Orthographic, clon = 0',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',60801,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60866','projected_crs','IAU_2015',60866,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60871,'Iapetus (2015) / Ographic / Orthographic, clon = 180',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',60801,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60871','projected_crs','IAU_2015',60871,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60876,'Iapetus (2015) / Ographic / Lambert Conic Conformal',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',60801,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60876','projected_crs','IAU_2015',60876,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60881,'Iapetus (2015) / Ographic / Lambert Azimuthal Equal Area',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',60801,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60881','projected_crs','IAU_2015',60881,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60886,'Iapetus (2015) / Ographic / Albers Equal Area',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',60801,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60886','projected_crs','IAU_2015',60886,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60891,'Iapetus (2015) / Ographic / Mercator',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',60801,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60891','projected_crs','IAU_2015',60891,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',60802,'Iapetus (2015) / Ocentric','Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','other','PROJ','OCENTRIC_LAT_LON','IAU_2015',60801,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_60802','geodetic_crs','IAU_2015',60802,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60812,'Iapetus (2015) / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',60802,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60812','projected_crs','IAU_2015',60812,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60817,'Iapetus (2015) / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',60802,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60817','projected_crs','IAU_2015',60817,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60822,'Iapetus (2015) / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',60802,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60822','projected_crs','IAU_2015',60822,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60827,'Iapetus (2015) / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',60802,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60827','projected_crs','IAU_2015',60827,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60832,'Iapetus (2015) / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',60802,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60832','projected_crs','IAU_2015',60832,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60837,'Iapetus (2015) / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',60802,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60837','projected_crs','IAU_2015',60837,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60842,'Iapetus (2015) / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',60802,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60842','projected_crs','IAU_2015',60842,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60847,'Iapetus (2015) / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',60802,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60847','projected_crs','IAU_2015',60847,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60852,'Iapetus (2015) / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',60802,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60852','projected_crs','IAU_2015',60852,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60857,'Iapetus (2015) / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',60802,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60857','projected_crs','IAU_2015',60857,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60862,'Iapetus (2015) / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',60802,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60862','projected_crs','IAU_2015',60862,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60867,'Iapetus (2015) / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',60802,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60867','projected_crs','IAU_2015',60867,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60872,'Iapetus (2015) / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',60802,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60872','projected_crs','IAU_2015',60872,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60877,'Iapetus (2015) / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',60802,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60877','projected_crs','IAU_2015',60877,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60882,'Iapetus (2015) / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',60802,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60882','projected_crs','IAU_2015',60882,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60887,'Iapetus (2015) / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',60802,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60887','projected_crs','IAU_2015',60887,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60892,'Iapetus (2015) / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',60802,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60892','projected_crs','IAU_2015',60892,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 609, 'Phoebe', 106500.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 60900, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',60900,'Phoebe (2015) - Sphere',NULL,'IAU_2015',609,106500.000000,'EPSG','9001',NULL,106500.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',60900,'Phoebe (2015) - Sphere','','IAU_2015',60900,'IAU_2015',60900,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_60900','geodetic_datum','IAU_2015',60900,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',60900,'Phoebe (2015) - Sphere / Ocentric','Use mean radius as sphere radius for interoperability. Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',60900,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_60900','geodetic_crs','IAU_2015',60900,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60910,'Phoebe (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',60900,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60910','projected_crs','IAU_2015',60910,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60915,'Phoebe (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',60900,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60915','projected_crs','IAU_2015',60915,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60920,'Phoebe (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',60900,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60920','projected_crs','IAU_2015',60920,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60925,'Phoebe (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',60900,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60925','projected_crs','IAU_2015',60925,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60930,'Phoebe (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',60900,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60930','projected_crs','IAU_2015',60930,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60935,'Phoebe (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',60900,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60935','projected_crs','IAU_2015',60935,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60940,'Phoebe (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',60900,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60940','projected_crs','IAU_2015',60940,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60945,'Phoebe (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',60900,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60945','projected_crs','IAU_2015',60945,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60950,'Phoebe (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',60900,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60950','projected_crs','IAU_2015',60950,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60955,'Phoebe (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',60900,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60955','projected_crs','IAU_2015',60955,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60960,'Phoebe (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',60900,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60960','projected_crs','IAU_2015',60960,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60965,'Phoebe (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',60900,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60965','projected_crs','IAU_2015',60965,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60970,'Phoebe (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',60900,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60970','projected_crs','IAU_2015',60970,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60975,'Phoebe (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',60900,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60975','projected_crs','IAU_2015',60975,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60980,'Phoebe (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',60900,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60980','projected_crs','IAU_2015',60980,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60985,'Phoebe (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',60900,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60985','projected_crs','IAU_2015',60985,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',60990,'Phoebe (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',60900,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_60990','projected_crs','IAU_2015',60990,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 610, 'Janus', 89200.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 61000, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',61000,'Janus (2015) - Sphere',NULL,'IAU_2015',610,89200.000000,'EPSG','9001',NULL,89200.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',61000,'Janus (2015) - Sphere','','IAU_2015',61000,'IAU_2015',61000,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_61000','geodetic_datum','IAU_2015',61000,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',61000,'Janus (2015) - Sphere / Ocentric','Use mean radius as sphere radius for interoperability. Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',61000,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_61000','geodetic_crs','IAU_2015',61000,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61010,'Janus (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',61000,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61010','projected_crs','IAU_2015',61010,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61015,'Janus (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',61000,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61015','projected_crs','IAU_2015',61015,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61020,'Janus (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',61000,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61020','projected_crs','IAU_2015',61020,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61025,'Janus (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',61000,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61025','projected_crs','IAU_2015',61025,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61030,'Janus (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',61000,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61030','projected_crs','IAU_2015',61030,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61035,'Janus (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',61000,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61035','projected_crs','IAU_2015',61035,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61040,'Janus (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',61000,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61040','projected_crs','IAU_2015',61040,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61045,'Janus (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',61000,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61045','projected_crs','IAU_2015',61045,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61050,'Janus (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',61000,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61050','projected_crs','IAU_2015',61050,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61055,'Janus (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',61000,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61055','projected_crs','IAU_2015',61055,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61060,'Janus (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',61000,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61060','projected_crs','IAU_2015',61060,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61065,'Janus (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',61000,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61065','projected_crs','IAU_2015',61065,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61070,'Janus (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',61000,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61070','projected_crs','IAU_2015',61070,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61075,'Janus (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',61000,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61075','projected_crs','IAU_2015',61075,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61080,'Janus (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',61000,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61080','projected_crs','IAU_2015',61080,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61085,'Janus (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',61000,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61085','projected_crs','IAU_2015',61085,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61090,'Janus (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',61000,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61090','projected_crs','IAU_2015',61090,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 611, 'Epimetheus', 58200.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 61100, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',61100,'Epimetheus (2015) - Sphere',NULL,'IAU_2015',611,58200.000000,'EPSG','9001',NULL,58200.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',61100,'Epimetheus (2015) - Sphere','','IAU_2015',61100,'IAU_2015',61100,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_61100','geodetic_datum','IAU_2015',61100,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',61100,'Epimetheus (2015) - Sphere / Ocentric','Use mean radius as sphere radius for interoperability. Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',61100,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_61100','geodetic_crs','IAU_2015',61100,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61110,'Epimetheus (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',61100,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61110','projected_crs','IAU_2015',61110,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61115,'Epimetheus (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',61100,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61115','projected_crs','IAU_2015',61115,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61120,'Epimetheus (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',61100,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61120','projected_crs','IAU_2015',61120,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61125,'Epimetheus (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',61100,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61125','projected_crs','IAU_2015',61125,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61130,'Epimetheus (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',61100,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61130','projected_crs','IAU_2015',61130,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61135,'Epimetheus (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',61100,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61135','projected_crs','IAU_2015',61135,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61140,'Epimetheus (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',61100,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61140','projected_crs','IAU_2015',61140,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61145,'Epimetheus (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',61100,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61145','projected_crs','IAU_2015',61145,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61150,'Epimetheus (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',61100,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61150','projected_crs','IAU_2015',61150,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61155,'Epimetheus (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',61100,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61155','projected_crs','IAU_2015',61155,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61160,'Epimetheus (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',61100,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61160','projected_crs','IAU_2015',61160,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61165,'Epimetheus (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',61100,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61165','projected_crs','IAU_2015',61165,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61170,'Epimetheus (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',61100,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61170','projected_crs','IAU_2015',61170,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61175,'Epimetheus (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',61100,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61175','projected_crs','IAU_2015',61175,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61180,'Epimetheus (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',61100,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61180','projected_crs','IAU_2015',61180,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61185,'Epimetheus (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',61100,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61185','projected_crs','IAU_2015',61185,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61190,'Epimetheus (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',61100,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61190','projected_crs','IAU_2015',61190,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 612, 'Helene', 18000.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 61200, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',61200,'Helene (2015) - Sphere',NULL,'IAU_2015',612,18000.000000,'EPSG','9001',NULL,18000.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',61200,'Helene (2015) - Sphere','','IAU_2015',61200,'IAU_2015',61200,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_61200','geodetic_datum','IAU_2015',61200,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',61200,'Helene (2015) - Sphere / Ocentric','Use mean radius as sphere radius for interoperability. Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',61200,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_61200','geodetic_crs','IAU_2015',61200,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61210,'Helene (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',61200,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61210','projected_crs','IAU_2015',61210,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61215,'Helene (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',61200,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61215','projected_crs','IAU_2015',61215,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61220,'Helene (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',61200,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61220','projected_crs','IAU_2015',61220,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61225,'Helene (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',61200,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61225','projected_crs','IAU_2015',61225,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61230,'Helene (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',61200,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61230','projected_crs','IAU_2015',61230,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61235,'Helene (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',61200,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61235','projected_crs','IAU_2015',61235,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61240,'Helene (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',61200,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61240','projected_crs','IAU_2015',61240,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61245,'Helene (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',61200,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61245','projected_crs','IAU_2015',61245,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61250,'Helene (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',61200,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61250','projected_crs','IAU_2015',61250,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61255,'Helene (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',61200,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61255','projected_crs','IAU_2015',61255,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61260,'Helene (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',61200,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61260','projected_crs','IAU_2015',61260,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61265,'Helene (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',61200,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61265','projected_crs','IAU_2015',61265,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61270,'Helene (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',61200,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61270','projected_crs','IAU_2015',61270,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61275,'Helene (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',61200,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61275','projected_crs','IAU_2015',61275,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61280,'Helene (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',61200,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61280','projected_crs','IAU_2015',61280,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61285,'Helene (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',61200,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61285','projected_crs','IAU_2015',61285,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61290,'Helene (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',61200,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61290','projected_crs','IAU_2015',61290,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 613, 'Telesto', 12400.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 61300, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',61300,'Telesto (2015) - Sphere',NULL,'IAU_2015',613,12400.000000,'EPSG','9001',NULL,12400.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',61300,'Telesto (2015) - Sphere','','IAU_2015',61300,'IAU_2015',61300,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_61300','geodetic_datum','IAU_2015',61300,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',61300,'Telesto (2015) - Sphere / Ocentric','Use mean radius as sphere radius for interoperability. Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',61300,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_61300','geodetic_crs','IAU_2015',61300,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61310,'Telesto (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',61300,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61310','projected_crs','IAU_2015',61310,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61315,'Telesto (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',61300,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61315','projected_crs','IAU_2015',61315,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61320,'Telesto (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',61300,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61320','projected_crs','IAU_2015',61320,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61325,'Telesto (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',61300,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61325','projected_crs','IAU_2015',61325,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61330,'Telesto (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',61300,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61330','projected_crs','IAU_2015',61330,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61335,'Telesto (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',61300,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61335','projected_crs','IAU_2015',61335,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61340,'Telesto (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',61300,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61340','projected_crs','IAU_2015',61340,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61345,'Telesto (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',61300,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61345','projected_crs','IAU_2015',61345,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61350,'Telesto (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',61300,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61350','projected_crs','IAU_2015',61350,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61355,'Telesto (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',61300,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61355','projected_crs','IAU_2015',61355,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61360,'Telesto (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',61300,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61360','projected_crs','IAU_2015',61360,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61365,'Telesto (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',61300,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61365','projected_crs','IAU_2015',61365,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61370,'Telesto (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',61300,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61370','projected_crs','IAU_2015',61370,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61375,'Telesto (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',61300,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61375','projected_crs','IAU_2015',61375,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61380,'Telesto (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',61300,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61380','projected_crs','IAU_2015',61380,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61385,'Telesto (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',61300,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61385','projected_crs','IAU_2015',61385,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61390,'Telesto (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',61300,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61390','projected_crs','IAU_2015',61390,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 614, 'Calypso', 9600.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 61400, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',61400,'Calypso (2015) - Sphere',NULL,'IAU_2015',614,9600.000000,'EPSG','9001',NULL,9600.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',61400,'Calypso (2015) - Sphere','','IAU_2015',61400,'IAU_2015',61400,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_61400','geodetic_datum','IAU_2015',61400,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',61400,'Calypso (2015) - Sphere / Ocentric','Use mean radius as sphere radius for interoperability. Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',61400,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_61400','geodetic_crs','IAU_2015',61400,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61410,'Calypso (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',61400,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61410','projected_crs','IAU_2015',61410,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61415,'Calypso (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',61400,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61415','projected_crs','IAU_2015',61415,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61420,'Calypso (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',61400,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61420','projected_crs','IAU_2015',61420,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61425,'Calypso (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',61400,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61425','projected_crs','IAU_2015',61425,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61430,'Calypso (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',61400,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61430','projected_crs','IAU_2015',61430,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61435,'Calypso (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',61400,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61435','projected_crs','IAU_2015',61435,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61440,'Calypso (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',61400,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61440','projected_crs','IAU_2015',61440,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61445,'Calypso (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',61400,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61445','projected_crs','IAU_2015',61445,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61450,'Calypso (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',61400,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61450','projected_crs','IAU_2015',61450,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61455,'Calypso (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',61400,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61455','projected_crs','IAU_2015',61455,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61460,'Calypso (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',61400,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61460','projected_crs','IAU_2015',61460,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61465,'Calypso (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',61400,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61465','projected_crs','IAU_2015',61465,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61470,'Calypso (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',61400,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61470','projected_crs','IAU_2015',61470,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61475,'Calypso (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',61400,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61475','projected_crs','IAU_2015',61475,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61480,'Calypso (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',61400,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61480','projected_crs','IAU_2015',61480,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61485,'Calypso (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',61400,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61485','projected_crs','IAU_2015',61485,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61490,'Calypso (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',61400,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61490','projected_crs','IAU_2015',61490,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 615, 'Atlas', 15100.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 61500, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',61500,'Atlas (2015) - Sphere',NULL,'IAU_2015',615,15100.000000,'EPSG','9001',NULL,15100.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',61500,'Atlas (2015) - Sphere','','IAU_2015',61500,'IAU_2015',61500,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_61500','geodetic_datum','IAU_2015',61500,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',61500,'Atlas (2015) - Sphere / Ocentric','Use mean radius as sphere radius for interoperability. Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',61500,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_61500','geodetic_crs','IAU_2015',61500,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61510,'Atlas (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',61500,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61510','projected_crs','IAU_2015',61510,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61515,'Atlas (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',61500,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61515','projected_crs','IAU_2015',61515,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61520,'Atlas (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',61500,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61520','projected_crs','IAU_2015',61520,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61525,'Atlas (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',61500,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61525','projected_crs','IAU_2015',61525,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61530,'Atlas (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',61500,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61530','projected_crs','IAU_2015',61530,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61535,'Atlas (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',61500,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61535','projected_crs','IAU_2015',61535,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61540,'Atlas (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',61500,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61540','projected_crs','IAU_2015',61540,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61545,'Atlas (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',61500,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61545','projected_crs','IAU_2015',61545,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61550,'Atlas (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',61500,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61550','projected_crs','IAU_2015',61550,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61555,'Atlas (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',61500,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61555','projected_crs','IAU_2015',61555,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61560,'Atlas (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',61500,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61560','projected_crs','IAU_2015',61560,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61565,'Atlas (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',61500,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61565','projected_crs','IAU_2015',61565,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61570,'Atlas (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',61500,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61570','projected_crs','IAU_2015',61570,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61575,'Atlas (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',61500,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61575','projected_crs','IAU_2015',61575,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61580,'Atlas (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',61500,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61580','projected_crs','IAU_2015',61580,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61585,'Atlas (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',61500,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61585','projected_crs','IAU_2015',61585,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61590,'Atlas (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',61500,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61590','projected_crs','IAU_2015',61590,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 616, 'Prometheus', 43100.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 61600, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',61600,'Prometheus (2015) - Sphere',NULL,'IAU_2015',616,43100.000000,'EPSG','9001',NULL,43100.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',61600,'Prometheus (2015) - Sphere','','IAU_2015',61600,'IAU_2015',61600,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_61600','geodetic_datum','IAU_2015',61600,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',61600,'Prometheus (2015) - Sphere / Ocentric','Use mean radius as sphere radius for interoperability. Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',61600,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_61600','geodetic_crs','IAU_2015',61600,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61610,'Prometheus (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',61600,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61610','projected_crs','IAU_2015',61610,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61615,'Prometheus (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',61600,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61615','projected_crs','IAU_2015',61615,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61620,'Prometheus (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',61600,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61620','projected_crs','IAU_2015',61620,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61625,'Prometheus (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',61600,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61625','projected_crs','IAU_2015',61625,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61630,'Prometheus (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',61600,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61630','projected_crs','IAU_2015',61630,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61635,'Prometheus (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',61600,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61635','projected_crs','IAU_2015',61635,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61640,'Prometheus (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',61600,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61640','projected_crs','IAU_2015',61640,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61645,'Prometheus (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',61600,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61645','projected_crs','IAU_2015',61645,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61650,'Prometheus (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',61600,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61650','projected_crs','IAU_2015',61650,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61655,'Prometheus (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',61600,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61655','projected_crs','IAU_2015',61655,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61660,'Prometheus (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',61600,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61660','projected_crs','IAU_2015',61660,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61665,'Prometheus (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',61600,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61665','projected_crs','IAU_2015',61665,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61670,'Prometheus (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',61600,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61670','projected_crs','IAU_2015',61670,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61675,'Prometheus (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',61600,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61675','projected_crs','IAU_2015',61675,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61680,'Prometheus (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',61600,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61680','projected_crs','IAU_2015',61680,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61685,'Prometheus (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',61600,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61685','projected_crs','IAU_2015',61685,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61690,'Prometheus (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',61600,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61690','projected_crs','IAU_2015',61690,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 617, 'Pandora', 40600.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 61700, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',61700,'Pandora (2015) - Sphere',NULL,'IAU_2015',617,40600.000000,'EPSG','9001',NULL,40600.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',61700,'Pandora (2015) - Sphere','','IAU_2015',61700,'IAU_2015',61700,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_61700','geodetic_datum','IAU_2015',61700,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',61700,'Pandora (2015) - Sphere / Ocentric','Use mean radius as sphere radius for interoperability. Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',61700,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_61700','geodetic_crs','IAU_2015',61700,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61710,'Pandora (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',61700,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61710','projected_crs','IAU_2015',61710,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61715,'Pandora (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',61700,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61715','projected_crs','IAU_2015',61715,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61720,'Pandora (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',61700,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61720','projected_crs','IAU_2015',61720,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61725,'Pandora (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',61700,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61725','projected_crs','IAU_2015',61725,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61730,'Pandora (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',61700,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61730','projected_crs','IAU_2015',61730,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61735,'Pandora (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',61700,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61735','projected_crs','IAU_2015',61735,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61740,'Pandora (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',61700,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61740','projected_crs','IAU_2015',61740,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61745,'Pandora (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',61700,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61745','projected_crs','IAU_2015',61745,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61750,'Pandora (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',61700,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61750','projected_crs','IAU_2015',61750,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61755,'Pandora (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',61700,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61755','projected_crs','IAU_2015',61755,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61760,'Pandora (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',61700,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61760','projected_crs','IAU_2015',61760,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61765,'Pandora (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',61700,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61765','projected_crs','IAU_2015',61765,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61770,'Pandora (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',61700,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61770','projected_crs','IAU_2015',61770,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61775,'Pandora (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',61700,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61775','projected_crs','IAU_2015',61775,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61780,'Pandora (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',61700,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61780','projected_crs','IAU_2015',61780,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61785,'Pandora (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',61700,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61785','projected_crs','IAU_2015',61785,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61790,'Pandora (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',61700,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61790','projected_crs','IAU_2015',61790,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 618, 'Pan', 14000.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 61800, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',61800,'Pan (2015) - Sphere',NULL,'IAU_2015',618,14000.000000,'EPSG','9001',NULL,14000.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',61800,'Pan (2015) - Sphere','','IAU_2015',61800,'IAU_2015',61800,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_61800','geodetic_datum','IAU_2015',61800,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',61800,'Pan (2015) - Sphere / Ocentric','Use mean radius as sphere radius for interoperability. Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',61800,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_61800','geodetic_crs','IAU_2015',61800,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61810,'Pan (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',61800,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61810','projected_crs','IAU_2015',61810,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61815,'Pan (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',61800,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61815','projected_crs','IAU_2015',61815,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61820,'Pan (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',61800,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61820','projected_crs','IAU_2015',61820,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61825,'Pan (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',61800,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61825','projected_crs','IAU_2015',61825,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61830,'Pan (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',61800,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61830','projected_crs','IAU_2015',61830,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61835,'Pan (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',61800,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61835','projected_crs','IAU_2015',61835,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61840,'Pan (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',61800,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61840','projected_crs','IAU_2015',61840,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61845,'Pan (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',61800,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61845','projected_crs','IAU_2015',61845,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61850,'Pan (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',61800,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61850','projected_crs','IAU_2015',61850,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61855,'Pan (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',61800,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61855','projected_crs','IAU_2015',61855,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61860,'Pan (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',61800,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61860','projected_crs','IAU_2015',61860,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61865,'Pan (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',61800,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61865','projected_crs','IAU_2015',61865,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61870,'Pan (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',61800,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61870','projected_crs','IAU_2015',61870,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61875,'Pan (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',61800,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61875','projected_crs','IAU_2015',61875,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61880,'Pan (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',61800,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61880','projected_crs','IAU_2015',61880,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61885,'Pan (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',61800,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61885','projected_crs','IAU_2015',61885,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',61890,'Pan (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',61800,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_61890','projected_crs','IAU_2015',61890,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
-- Skip Ymir has it lacks all axis information
-- Skip Paaliaq has it lacks all axis information
-- Skip Tarvos has it lacks all axis information
-- Skip Ijiraq has it lacks all axis information
-- Skip Suttungr has it lacks all axis information
-- Skip Kiviuq has it lacks all axis information
-- Skip Mundilfari has it lacks all axis information
-- Skip Albiorix has it lacks all axis information
-- Skip Skathi has it lacks all axis information
-- Skip Erriapo has it lacks all axis information
-- Skip Siarnaq has it lacks all axis information
-- Skip Thrymr has it lacks all axis information
-- Skip Narvi has it lacks all axis information
INSERT INTO celestial_body VALUES('IAU_2015', 632, 'Methone', 1450.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 63200, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',63200,'Methone (2015) - Sphere',NULL,'IAU_2015',632,1450.000000,'EPSG','9001',NULL,1450.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',63200,'Methone (2015) - Sphere','','IAU_2015',63200,'IAU_2015',63200,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_63200','geodetic_datum','IAU_2015',63200,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',63200,'Methone (2015) - Sphere / Ocentric','Use mean radius as sphere radius for interoperability. Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',63200,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_63200','geodetic_crs','IAU_2015',63200,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',63210,'Methone (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',63200,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_63210','projected_crs','IAU_2015',63210,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',63215,'Methone (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',63200,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_63215','projected_crs','IAU_2015',63215,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',63220,'Methone (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',63200,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_63220','projected_crs','IAU_2015',63220,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',63225,'Methone (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',63200,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_63225','projected_crs','IAU_2015',63225,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',63230,'Methone (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',63200,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_63230','projected_crs','IAU_2015',63230,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',63235,'Methone (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',63200,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_63235','projected_crs','IAU_2015',63235,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',63240,'Methone (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',63200,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_63240','projected_crs','IAU_2015',63240,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',63245,'Methone (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',63200,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_63245','projected_crs','IAU_2015',63245,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',63250,'Methone (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',63200,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_63250','projected_crs','IAU_2015',63250,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',63255,'Methone (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',63200,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_63255','projected_crs','IAU_2015',63255,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',63260,'Methone (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',63200,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_63260','projected_crs','IAU_2015',63260,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',63265,'Methone (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',63200,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_63265','projected_crs','IAU_2015',63265,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',63270,'Methone (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',63200,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_63270','projected_crs','IAU_2015',63270,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',63275,'Methone (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',63200,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_63275','projected_crs','IAU_2015',63275,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',63280,'Methone (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',63200,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_63280','projected_crs','IAU_2015',63280,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',63285,'Methone (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',63200,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_63285','projected_crs','IAU_2015',63285,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',63290,'Methone (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',63200,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_63290','projected_crs','IAU_2015',63290,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 633, 'Pallene', 2230.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 63300, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',63300,'Pallene (2015) - Sphere',NULL,'IAU_2015',633,2230.000000,'EPSG','9001',NULL,2230.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',63300,'Pallene (2015) - Sphere','','IAU_2015',63300,'IAU_2015',63300,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_63300','geodetic_datum','IAU_2015',63300,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',63300,'Pallene (2015) - Sphere / Ocentric','Use mean radius as sphere radius for interoperability. Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',63300,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_63300','geodetic_crs','IAU_2015',63300,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',63310,'Pallene (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',63300,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_63310','projected_crs','IAU_2015',63310,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',63315,'Pallene (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',63300,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_63315','projected_crs','IAU_2015',63315,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',63320,'Pallene (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',63300,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_63320','projected_crs','IAU_2015',63320,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',63325,'Pallene (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',63300,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_63325','projected_crs','IAU_2015',63325,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',63330,'Pallene (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',63300,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_63330','projected_crs','IAU_2015',63330,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',63335,'Pallene (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',63300,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_63335','projected_crs','IAU_2015',63335,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',63340,'Pallene (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',63300,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_63340','projected_crs','IAU_2015',63340,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',63345,'Pallene (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',63300,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_63345','projected_crs','IAU_2015',63345,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',63350,'Pallene (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',63300,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_63350','projected_crs','IAU_2015',63350,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',63355,'Pallene (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',63300,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_63355','projected_crs','IAU_2015',63355,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',63360,'Pallene (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',63300,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_63360','projected_crs','IAU_2015',63360,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',63365,'Pallene (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',63300,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_63365','projected_crs','IAU_2015',63365,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',63370,'Pallene (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',63300,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_63370','projected_crs','IAU_2015',63370,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',63375,'Pallene (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',63300,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_63375','projected_crs','IAU_2015',63375,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',63380,'Pallene (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',63300,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_63380','projected_crs','IAU_2015',63380,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',63385,'Pallene (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',63300,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_63385','projected_crs','IAU_2015',63385,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',63390,'Pallene (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',63300,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_63390','projected_crs','IAU_2015',63390,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 634, 'Polydeuces', 1300.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 63400, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',63400,'Polydeuces (2015) - Sphere',NULL,'IAU_2015',634,1300.000000,'EPSG','9001',NULL,1300.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',63400,'Polydeuces (2015) - Sphere','','IAU_2015',63400,'IAU_2015',63400,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_63400','geodetic_datum','IAU_2015',63400,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',63400,'Polydeuces (2015) - Sphere / Ocentric','Use mean radius as sphere radius for interoperability. Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',63400,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_63400','geodetic_crs','IAU_2015',63400,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',63410,'Polydeuces (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',63400,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_63410','projected_crs','IAU_2015',63410,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',63415,'Polydeuces (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',63400,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_63415','projected_crs','IAU_2015',63415,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',63420,'Polydeuces (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',63400,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_63420','projected_crs','IAU_2015',63420,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',63425,'Polydeuces (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',63400,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_63425','projected_crs','IAU_2015',63425,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',63430,'Polydeuces (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',63400,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_63430','projected_crs','IAU_2015',63430,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',63435,'Polydeuces (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',63400,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_63435','projected_crs','IAU_2015',63435,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',63440,'Polydeuces (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',63400,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_63440','projected_crs','IAU_2015',63440,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',63445,'Polydeuces (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',63400,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_63445','projected_crs','IAU_2015',63445,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',63450,'Polydeuces (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',63400,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_63450','projected_crs','IAU_2015',63450,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',63455,'Polydeuces (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',63400,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_63455','projected_crs','IAU_2015',63455,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',63460,'Polydeuces (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',63400,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_63460','projected_crs','IAU_2015',63460,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',63465,'Polydeuces (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',63400,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_63465','projected_crs','IAU_2015',63465,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',63470,'Polydeuces (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',63400,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_63470','projected_crs','IAU_2015',63470,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',63475,'Polydeuces (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',63400,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_63475','projected_crs','IAU_2015',63475,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',63480,'Polydeuces (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',63400,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_63480','projected_crs','IAU_2015',63480,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',63485,'Polydeuces (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',63400,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_63485','projected_crs','IAU_2015',63485,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',63490,'Polydeuces (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',63400,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_63490','projected_crs','IAU_2015',63490,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 635, 'Daphnis', 3800.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 63500, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',63500,'Daphnis (2015) - Sphere',NULL,'IAU_2015',635,3800.000000,'EPSG','9001',NULL,3800.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',63500,'Daphnis (2015) - Sphere','','IAU_2015',63500,'IAU_2015',63500,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_63500','geodetic_datum','IAU_2015',63500,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',63500,'Daphnis (2015) - Sphere / Ocentric','Use mean radius as sphere radius for interoperability. Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',63500,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_63500','geodetic_crs','IAU_2015',63500,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',63510,'Daphnis (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',63500,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_63510','projected_crs','IAU_2015',63510,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',63515,'Daphnis (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',63500,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_63515','projected_crs','IAU_2015',63515,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',63520,'Daphnis (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',63500,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_63520','projected_crs','IAU_2015',63520,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',63525,'Daphnis (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',63500,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_63525','projected_crs','IAU_2015',63525,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',63530,'Daphnis (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',63500,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_63530','projected_crs','IAU_2015',63530,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',63535,'Daphnis (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',63500,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_63535','projected_crs','IAU_2015',63535,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',63540,'Daphnis (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',63500,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_63540','projected_crs','IAU_2015',63540,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',63545,'Daphnis (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',63500,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_63545','projected_crs','IAU_2015',63545,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',63550,'Daphnis (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',63500,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_63550','projected_crs','IAU_2015',63550,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',63555,'Daphnis (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',63500,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_63555','projected_crs','IAU_2015',63555,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',63560,'Daphnis (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',63500,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_63560','projected_crs','IAU_2015',63560,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',63565,'Daphnis (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',63500,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_63565','projected_crs','IAU_2015',63565,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',63570,'Daphnis (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',63500,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_63570','projected_crs','IAU_2015',63570,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',63575,'Daphnis (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',63500,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_63575','projected_crs','IAU_2015',63575,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',63580,'Daphnis (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',63500,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_63580','projected_crs','IAU_2015',63580,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',63585,'Daphnis (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',63500,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_63585','projected_crs','IAU_2015',63585,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',63590,'Daphnis (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',63500,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_63590','projected_crs','IAU_2015',63590,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
-- Skip Aegir has it lacks all axis information
-- Skip Bebhionn has it lacks all axis information
-- Skip Bergelmir has it lacks all axis information
-- Skip Bestla has it lacks all axis information
-- Skip Farbauti has it lacks all axis information
-- Skip Fenrir has it lacks all axis information
-- Skip Fornjot has it lacks all axis information
-- Skip Hati has it lacks all axis information
-- Skip Hyrrokkin has it lacks all axis information
-- Skip Kari has it lacks all axis information
-- Skip Loge has it lacks all axis information
-- Skip Skoll has it lacks all axis information
-- Skip Sutur has it lacks all axis information
INSERT INTO celestial_body VALUES('IAU_2015', 649, 'Anthe', 500.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 64900, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',64900,'Anthe (2015) - Sphere',NULL,'IAU_2015',649,500.000000,'EPSG','9001',NULL,500.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',64900,'Anthe (2015) - Sphere','','IAU_2015',64900,'IAU_2015',64900,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_64900','geodetic_datum','IAU_2015',64900,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',64900,'Anthe (2015) - Sphere / Ocentric','Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',64900,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_64900','geodetic_crs','IAU_2015',64900,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',64910,'Anthe (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',64900,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_64910','projected_crs','IAU_2015',64910,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',64915,'Anthe (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',64900,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_64915','projected_crs','IAU_2015',64915,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',64920,'Anthe (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',64900,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_64920','projected_crs','IAU_2015',64920,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',64925,'Anthe (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',64900,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_64925','projected_crs','IAU_2015',64925,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',64930,'Anthe (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',64900,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_64930','projected_crs','IAU_2015',64930,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',64935,'Anthe (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',64900,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_64935','projected_crs','IAU_2015',64935,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',64940,'Anthe (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',64900,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_64940','projected_crs','IAU_2015',64940,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',64945,'Anthe (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',64900,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_64945','projected_crs','IAU_2015',64945,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',64950,'Anthe (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',64900,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_64950','projected_crs','IAU_2015',64950,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',64955,'Anthe (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',64900,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_64955','projected_crs','IAU_2015',64955,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',64960,'Anthe (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',64900,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_64960','projected_crs','IAU_2015',64960,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',64965,'Anthe (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',64900,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_64965','projected_crs','IAU_2015',64965,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',64970,'Anthe (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',64900,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_64970','projected_crs','IAU_2015',64970,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',64975,'Anthe (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',64900,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_64975','projected_crs','IAU_2015',64975,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',64980,'Anthe (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',64900,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_64980','projected_crs','IAU_2015',64980,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',64985,'Anthe (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',64900,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_64985','projected_crs','IAU_2015',64985,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',64990,'Anthe (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',64900,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_64990','projected_crs','IAU_2015',64990,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
-- Skip Jarnsaxa has it lacks all axis information
-- Skip Greip has it lacks all axis information
-- Skip Tarqeq has it lacks all axis information
INSERT INTO celestial_body VALUES('IAU_2015', 653, 'Aegaeon', 330.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 65300, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',65300,'Aegaeon (2015) - Sphere',NULL,'IAU_2015',653,330.000000,'EPSG','9001',NULL,330.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',65300,'Aegaeon (2015) - Sphere','','IAU_2015',65300,'IAU_2015',65300,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_65300','geodetic_datum','IAU_2015',65300,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',65300,'Aegaeon (2015) - Sphere / Ocentric','Use mean radius as sphere radius for interoperability. Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',65300,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_65300','geodetic_crs','IAU_2015',65300,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',65310,'Aegaeon (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',65300,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_65310','projected_crs','IAU_2015',65310,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',65315,'Aegaeon (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',65300,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_65315','projected_crs','IAU_2015',65315,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',65320,'Aegaeon (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',65300,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_65320','projected_crs','IAU_2015',65320,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',65325,'Aegaeon (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',65300,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_65325','projected_crs','IAU_2015',65325,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',65330,'Aegaeon (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',65300,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_65330','projected_crs','IAU_2015',65330,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',65335,'Aegaeon (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',65300,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_65335','projected_crs','IAU_2015',65335,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',65340,'Aegaeon (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',65300,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_65340','projected_crs','IAU_2015',65340,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',65345,'Aegaeon (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',65300,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_65345','projected_crs','IAU_2015',65345,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',65350,'Aegaeon (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',65300,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_65350','projected_crs','IAU_2015',65350,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',65355,'Aegaeon (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',65300,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_65355','projected_crs','IAU_2015',65355,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',65360,'Aegaeon (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',65300,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_65360','projected_crs','IAU_2015',65360,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',65365,'Aegaeon (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',65300,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_65365','projected_crs','IAU_2015',65365,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',65370,'Aegaeon (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',65300,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_65370','projected_crs','IAU_2015',65370,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',65375,'Aegaeon (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',65300,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_65375','projected_crs','IAU_2015',65375,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',65380,'Aegaeon (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',65300,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_65380','projected_crs','IAU_2015',65380,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',65385,'Aegaeon (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',65300,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_65385','projected_crs','IAU_2015',65385,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',65390,'Aegaeon (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',65300,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_65390','projected_crs','IAU_2015',65390,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 799, 'Uranus', 25559000.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 79900, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',79900,'Uranus (2015) - Sphere',NULL,'IAU_2015',799,25559000.000000,'EPSG','9001',NULL,25559000.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',79900,'Uranus (2015) - Sphere','','IAU_2015',79900,'IAU_2015',79900,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_79900','geodetic_datum','IAU_2015',79900,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',79900,'Uranus (2015) - Sphere / Ocentric','Use semi-major radius as sphere for interoperability. Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',79900,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_79900','geodetic_crs','IAU_2015',79900,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',79910,'Uranus (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',79900,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_79910','projected_crs','IAU_2015',79910,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',79915,'Uranus (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',79900,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_79915','projected_crs','IAU_2015',79915,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',79920,'Uranus (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',79900,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_79920','projected_crs','IAU_2015',79920,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',79925,'Uranus (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',79900,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_79925','projected_crs','IAU_2015',79925,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',79930,'Uranus (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',79900,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_79930','projected_crs','IAU_2015',79930,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',79935,'Uranus (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',79900,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_79935','projected_crs','IAU_2015',79935,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',79940,'Uranus (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',79900,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_79940','projected_crs','IAU_2015',79940,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',79945,'Uranus (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',79900,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_79945','projected_crs','IAU_2015',79945,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',79950,'Uranus (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',79900,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_79950','projected_crs','IAU_2015',79950,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',79955,'Uranus (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',79900,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_79955','projected_crs','IAU_2015',79955,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',79960,'Uranus (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',79900,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_79960','projected_crs','IAU_2015',79960,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',79965,'Uranus (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',79900,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_79965','projected_crs','IAU_2015',79965,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',79970,'Uranus (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',79900,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_79970','projected_crs','IAU_2015',79970,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',79975,'Uranus (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',79900,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_79975','projected_crs','IAU_2015',79975,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',79980,'Uranus (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',79900,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_79980','projected_crs','IAU_2015',79980,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',79985,'Uranus (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',79900,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_79985','projected_crs','IAU_2015',79985,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',79990,'Uranus (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',79900,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_79990','projected_crs','IAU_2015',79990,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO ellipsoid VALUES('IAU_2015',79901,'Uranus (2015)',NULL,'IAU_2015',799,25559000.000000,'EPSG','9001',NULL,24973000.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',79901,'Uranus (2015)','','IAU_2015',79901,'IAU_2015',79900,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_79901','geodetic_datum','IAU_2015',79901,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',79901,'Uranus (2015) / Ographic','Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',79901,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_79901','geodetic_crs','IAU_2015',79901,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',79911,'Uranus (2015) / Ographic / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',79901,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_79911','projected_crs','IAU_2015',79911,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',79916,'Uranus (2015) / Ographic / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',79901,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_79916','projected_crs','IAU_2015',79916,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',79921,'Uranus (2015) / Ographic / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',79901,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_79921','projected_crs','IAU_2015',79921,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',79926,'Uranus (2015) / Ographic / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',79901,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_79926','projected_crs','IAU_2015',79926,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',79931,'Uranus (2015) / Ographic / North Polar',NULL,'EPSG','4469','IAU_2015',79901,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_79931','projected_crs','IAU_2015',79931,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',79936,'Uranus (2015) / Ographic / South Polar',NULL,'EPSG','4490','IAU_2015',79901,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_79936','projected_crs','IAU_2015',79936,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',79941,'Uranus (2015) / Ographic / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',79901,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_79941','projected_crs','IAU_2015',79941,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',79946,'Uranus (2015) / Ographic / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',79901,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_79946','projected_crs','IAU_2015',79946,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',79951,'Uranus (2015) / Ographic / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',79901,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_79951','projected_crs','IAU_2015',79951,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',79956,'Uranus (2015) / Ographic / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',79901,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_79956','projected_crs','IAU_2015',79956,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',79961,'Uranus (2015) / Ographic / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',79901,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_79961','projected_crs','IAU_2015',79961,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',79966,'Uranus (2015) / Ographic / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',79901,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_79966','projected_crs','IAU_2015',79966,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',79971,'Uranus (2015) / Ographic / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',79901,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_79971','projected_crs','IAU_2015',79971,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',79976,'Uranus (2015) / Ographic / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',79901,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_79976','projected_crs','IAU_2015',79976,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',79981,'Uranus (2015) / Ographic / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',79901,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_79981','projected_crs','IAU_2015',79981,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',79986,'Uranus (2015) / Ographic / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',79901,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_79986','projected_crs','IAU_2015',79986,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',79991,'Uranus (2015) / Ographic / Mercator',NULL,'EPSG','4400','IAU_2015',79901,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_79991','projected_crs','IAU_2015',79991,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',79902,'Uranus (2015) / Ocentric','Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','other','PROJ','OCENTRIC_LAT_LON','IAU_2015',79901,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_79902','geodetic_crs','IAU_2015',79902,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 701, 'Ariel', 578900.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 70100, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',70100,'Ariel (2015) - Sphere',NULL,'IAU_2015',701,578900.000000,'EPSG','9001',NULL,578900.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',70100,'Ariel (2015) - Sphere','','IAU_2015',70100,'IAU_2015',70100,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_70100','geodetic_datum','IAU_2015',70100,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',70100,'Ariel (2015) - Sphere / Ocentric','Use mean radius as sphere radius for interoperability. Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',70100,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_70100','geodetic_crs','IAU_2015',70100,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70110,'Ariel (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',70100,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70110','projected_crs','IAU_2015',70110,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70115,'Ariel (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',70100,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70115','projected_crs','IAU_2015',70115,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70120,'Ariel (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',70100,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70120','projected_crs','IAU_2015',70120,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70125,'Ariel (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',70100,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70125','projected_crs','IAU_2015',70125,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70130,'Ariel (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',70100,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70130','projected_crs','IAU_2015',70130,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70135,'Ariel (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',70100,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70135','projected_crs','IAU_2015',70135,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70140,'Ariel (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',70100,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70140','projected_crs','IAU_2015',70140,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70145,'Ariel (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',70100,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70145','projected_crs','IAU_2015',70145,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70150,'Ariel (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',70100,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70150','projected_crs','IAU_2015',70150,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70155,'Ariel (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',70100,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70155','projected_crs','IAU_2015',70155,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70160,'Ariel (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',70100,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70160','projected_crs','IAU_2015',70160,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70165,'Ariel (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',70100,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70165','projected_crs','IAU_2015',70165,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70170,'Ariel (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',70100,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70170','projected_crs','IAU_2015',70170,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70175,'Ariel (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',70100,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70175','projected_crs','IAU_2015',70175,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70180,'Ariel (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',70100,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70180','projected_crs','IAU_2015',70180,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70185,'Ariel (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',70100,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70185','projected_crs','IAU_2015',70185,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70190,'Ariel (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',70100,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70190','projected_crs','IAU_2015',70190,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 702, 'Umbriel', 584700.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 70200, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',70200,'Umbriel (2015) - Sphere',NULL,'IAU_2015',702,584700.000000,'EPSG','9001',NULL,584700.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',70200,'Umbriel (2015) - Sphere','','IAU_2015',70200,'IAU_2015',70200,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_70200','geodetic_datum','IAU_2015',70200,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',70200,'Umbriel (2015) - Sphere / Ocentric','Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',70200,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_70200','geodetic_crs','IAU_2015',70200,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70210,'Umbriel (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',70200,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70210','projected_crs','IAU_2015',70210,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70215,'Umbriel (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',70200,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70215','projected_crs','IAU_2015',70215,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70220,'Umbriel (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',70200,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70220','projected_crs','IAU_2015',70220,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70225,'Umbriel (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',70200,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70225','projected_crs','IAU_2015',70225,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70230,'Umbriel (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',70200,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70230','projected_crs','IAU_2015',70230,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70235,'Umbriel (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',70200,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70235','projected_crs','IAU_2015',70235,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70240,'Umbriel (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',70200,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70240','projected_crs','IAU_2015',70240,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70245,'Umbriel (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',70200,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70245','projected_crs','IAU_2015',70245,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70250,'Umbriel (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',70200,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70250','projected_crs','IAU_2015',70250,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70255,'Umbriel (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',70200,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70255','projected_crs','IAU_2015',70255,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70260,'Umbriel (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',70200,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70260','projected_crs','IAU_2015',70260,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70265,'Umbriel (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',70200,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70265','projected_crs','IAU_2015',70265,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70270,'Umbriel (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',70200,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70270','projected_crs','IAU_2015',70270,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70275,'Umbriel (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',70200,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70275','projected_crs','IAU_2015',70275,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70280,'Umbriel (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',70200,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70280','projected_crs','IAU_2015',70280,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70285,'Umbriel (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',70200,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70285','projected_crs','IAU_2015',70285,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70290,'Umbriel (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',70200,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70290','projected_crs','IAU_2015',70290,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 703, 'Titania', 788900.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 70300, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',70300,'Titania (2015) - Sphere',NULL,'IAU_2015',703,788900.000000,'EPSG','9001',NULL,788900.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',70300,'Titania (2015) - Sphere','','IAU_2015',70300,'IAU_2015',70300,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_70300','geodetic_datum','IAU_2015',70300,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',70300,'Titania (2015) - Sphere / Ocentric','Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',70300,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_70300','geodetic_crs','IAU_2015',70300,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70310,'Titania (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',70300,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70310','projected_crs','IAU_2015',70310,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70315,'Titania (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',70300,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70315','projected_crs','IAU_2015',70315,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70320,'Titania (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',70300,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70320','projected_crs','IAU_2015',70320,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70325,'Titania (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',70300,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70325','projected_crs','IAU_2015',70325,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70330,'Titania (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',70300,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70330','projected_crs','IAU_2015',70330,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70335,'Titania (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',70300,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70335','projected_crs','IAU_2015',70335,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70340,'Titania (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',70300,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70340','projected_crs','IAU_2015',70340,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70345,'Titania (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',70300,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70345','projected_crs','IAU_2015',70345,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70350,'Titania (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',70300,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70350','projected_crs','IAU_2015',70350,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70355,'Titania (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',70300,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70355','projected_crs','IAU_2015',70355,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70360,'Titania (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',70300,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70360','projected_crs','IAU_2015',70360,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70365,'Titania (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',70300,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70365','projected_crs','IAU_2015',70365,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70370,'Titania (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',70300,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70370','projected_crs','IAU_2015',70370,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70375,'Titania (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',70300,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70375','projected_crs','IAU_2015',70375,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70380,'Titania (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',70300,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70380','projected_crs','IAU_2015',70380,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70385,'Titania (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',70300,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70385','projected_crs','IAU_2015',70385,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70390,'Titania (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',70300,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70390','projected_crs','IAU_2015',70390,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 704, 'Oberon', 761400.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 70400, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',70400,'Oberon (2015) - Sphere',NULL,'IAU_2015',704,761400.000000,'EPSG','9001',NULL,761400.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',70400,'Oberon (2015) - Sphere','','IAU_2015',70400,'IAU_2015',70400,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_70400','geodetic_datum','IAU_2015',70400,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',70400,'Oberon (2015) - Sphere / Ocentric','Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',70400,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_70400','geodetic_crs','IAU_2015',70400,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70410,'Oberon (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',70400,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70410','projected_crs','IAU_2015',70410,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70415,'Oberon (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',70400,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70415','projected_crs','IAU_2015',70415,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70420,'Oberon (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',70400,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70420','projected_crs','IAU_2015',70420,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70425,'Oberon (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',70400,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70425','projected_crs','IAU_2015',70425,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70430,'Oberon (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',70400,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70430','projected_crs','IAU_2015',70430,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70435,'Oberon (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',70400,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70435','projected_crs','IAU_2015',70435,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70440,'Oberon (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',70400,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70440','projected_crs','IAU_2015',70440,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70445,'Oberon (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',70400,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70445','projected_crs','IAU_2015',70445,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70450,'Oberon (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',70400,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70450','projected_crs','IAU_2015',70450,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70455,'Oberon (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',70400,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70455','projected_crs','IAU_2015',70455,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70460,'Oberon (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',70400,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70460','projected_crs','IAU_2015',70460,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70465,'Oberon (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',70400,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70465','projected_crs','IAU_2015',70465,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70470,'Oberon (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',70400,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70470','projected_crs','IAU_2015',70470,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70475,'Oberon (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',70400,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70475','projected_crs','IAU_2015',70475,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70480,'Oberon (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',70400,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70480','projected_crs','IAU_2015',70480,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70485,'Oberon (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',70400,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70485','projected_crs','IAU_2015',70485,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70490,'Oberon (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',70400,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70490','projected_crs','IAU_2015',70490,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 705, 'Miranda', 235800.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 70500, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',70500,'Miranda (2015) - Sphere',NULL,'IAU_2015',705,235800.000000,'EPSG','9001',NULL,235800.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',70500,'Miranda (2015) - Sphere','','IAU_2015',70500,'IAU_2015',70500,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_70500','geodetic_datum','IAU_2015',70500,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',70500,'Miranda (2015) - Sphere / Ocentric','Use mean radius as sphere radius for interoperability. Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',70500,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_70500','geodetic_crs','IAU_2015',70500,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70510,'Miranda (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',70500,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70510','projected_crs','IAU_2015',70510,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70515,'Miranda (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',70500,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70515','projected_crs','IAU_2015',70515,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70520,'Miranda (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',70500,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70520','projected_crs','IAU_2015',70520,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70525,'Miranda (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',70500,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70525','projected_crs','IAU_2015',70525,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70530,'Miranda (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',70500,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70530','projected_crs','IAU_2015',70530,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70535,'Miranda (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',70500,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70535','projected_crs','IAU_2015',70535,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70540,'Miranda (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',70500,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70540','projected_crs','IAU_2015',70540,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70545,'Miranda (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',70500,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70545','projected_crs','IAU_2015',70545,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70550,'Miranda (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',70500,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70550','projected_crs','IAU_2015',70550,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70555,'Miranda (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',70500,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70555','projected_crs','IAU_2015',70555,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70560,'Miranda (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',70500,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70560','projected_crs','IAU_2015',70560,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70565,'Miranda (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',70500,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70565','projected_crs','IAU_2015',70565,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70570,'Miranda (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',70500,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70570','projected_crs','IAU_2015',70570,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70575,'Miranda (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',70500,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70575','projected_crs','IAU_2015',70575,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70580,'Miranda (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',70500,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70580','projected_crs','IAU_2015',70580,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70585,'Miranda (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',70500,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70585','projected_crs','IAU_2015',70585,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70590,'Miranda (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',70500,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70590','projected_crs','IAU_2015',70590,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 706, 'Cordelia', 13000.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 70600, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',70600,'Cordelia (2015) - Sphere',NULL,'IAU_2015',706,13000.000000,'EPSG','9001',NULL,13000.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',70600,'Cordelia (2015) - Sphere','','IAU_2015',70600,'IAU_2015',70600,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_70600','geodetic_datum','IAU_2015',70600,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',70600,'Cordelia (2015) - Sphere / Ocentric','Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',70600,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_70600','geodetic_crs','IAU_2015',70600,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70610,'Cordelia (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',70600,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70610','projected_crs','IAU_2015',70610,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70615,'Cordelia (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',70600,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70615','projected_crs','IAU_2015',70615,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70620,'Cordelia (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',70600,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70620','projected_crs','IAU_2015',70620,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70625,'Cordelia (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',70600,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70625','projected_crs','IAU_2015',70625,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70630,'Cordelia (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',70600,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70630','projected_crs','IAU_2015',70630,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70635,'Cordelia (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',70600,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70635','projected_crs','IAU_2015',70635,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70640,'Cordelia (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',70600,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70640','projected_crs','IAU_2015',70640,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70645,'Cordelia (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',70600,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70645','projected_crs','IAU_2015',70645,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70650,'Cordelia (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',70600,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70650','projected_crs','IAU_2015',70650,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70655,'Cordelia (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',70600,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70655','projected_crs','IAU_2015',70655,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70660,'Cordelia (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',70600,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70660','projected_crs','IAU_2015',70660,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70665,'Cordelia (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',70600,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70665','projected_crs','IAU_2015',70665,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70670,'Cordelia (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',70600,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70670','projected_crs','IAU_2015',70670,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70675,'Cordelia (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',70600,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70675','projected_crs','IAU_2015',70675,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70680,'Cordelia (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',70600,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70680','projected_crs','IAU_2015',70680,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70685,'Cordelia (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',70600,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70685','projected_crs','IAU_2015',70685,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70690,'Cordelia (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',70600,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70690','projected_crs','IAU_2015',70690,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 707, 'Ophelia', 15000.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 70700, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',70700,'Ophelia (2015) - Sphere',NULL,'IAU_2015',707,15000.000000,'EPSG','9001',NULL,15000.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',70700,'Ophelia (2015) - Sphere','','IAU_2015',70700,'IAU_2015',70700,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_70700','geodetic_datum','IAU_2015',70700,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',70700,'Ophelia (2015) - Sphere / Ocentric','Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',70700,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_70700','geodetic_crs','IAU_2015',70700,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70710,'Ophelia (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',70700,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70710','projected_crs','IAU_2015',70710,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70715,'Ophelia (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',70700,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70715','projected_crs','IAU_2015',70715,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70720,'Ophelia (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',70700,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70720','projected_crs','IAU_2015',70720,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70725,'Ophelia (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',70700,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70725','projected_crs','IAU_2015',70725,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70730,'Ophelia (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',70700,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70730','projected_crs','IAU_2015',70730,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70735,'Ophelia (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',70700,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70735','projected_crs','IAU_2015',70735,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70740,'Ophelia (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',70700,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70740','projected_crs','IAU_2015',70740,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70745,'Ophelia (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',70700,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70745','projected_crs','IAU_2015',70745,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70750,'Ophelia (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',70700,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70750','projected_crs','IAU_2015',70750,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70755,'Ophelia (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',70700,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70755','projected_crs','IAU_2015',70755,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70760,'Ophelia (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',70700,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70760','projected_crs','IAU_2015',70760,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70765,'Ophelia (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',70700,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70765','projected_crs','IAU_2015',70765,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70770,'Ophelia (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',70700,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70770','projected_crs','IAU_2015',70770,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70775,'Ophelia (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',70700,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70775','projected_crs','IAU_2015',70775,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70780,'Ophelia (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',70700,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70780','projected_crs','IAU_2015',70780,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70785,'Ophelia (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',70700,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70785','projected_crs','IAU_2015',70785,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70790,'Ophelia (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',70700,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70790','projected_crs','IAU_2015',70790,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 708, 'Bianca', 21000.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 70800, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',70800,'Bianca (2015) - Sphere',NULL,'IAU_2015',708,21000.000000,'EPSG','9001',NULL,21000.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',70800,'Bianca (2015) - Sphere','','IAU_2015',70800,'IAU_2015',70800,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_70800','geodetic_datum','IAU_2015',70800,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',70800,'Bianca (2015) - Sphere / Ocentric','Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',70800,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_70800','geodetic_crs','IAU_2015',70800,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70810,'Bianca (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',70800,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70810','projected_crs','IAU_2015',70810,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70815,'Bianca (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',70800,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70815','projected_crs','IAU_2015',70815,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70820,'Bianca (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',70800,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70820','projected_crs','IAU_2015',70820,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70825,'Bianca (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',70800,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70825','projected_crs','IAU_2015',70825,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70830,'Bianca (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',70800,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70830','projected_crs','IAU_2015',70830,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70835,'Bianca (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',70800,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70835','projected_crs','IAU_2015',70835,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70840,'Bianca (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',70800,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70840','projected_crs','IAU_2015',70840,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70845,'Bianca (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',70800,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70845','projected_crs','IAU_2015',70845,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70850,'Bianca (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',70800,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70850','projected_crs','IAU_2015',70850,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70855,'Bianca (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',70800,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70855','projected_crs','IAU_2015',70855,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70860,'Bianca (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',70800,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70860','projected_crs','IAU_2015',70860,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70865,'Bianca (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',70800,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70865','projected_crs','IAU_2015',70865,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70870,'Bianca (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',70800,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70870','projected_crs','IAU_2015',70870,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70875,'Bianca (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',70800,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70875','projected_crs','IAU_2015',70875,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70880,'Bianca (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',70800,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70880','projected_crs','IAU_2015',70880,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70885,'Bianca (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',70800,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70885','projected_crs','IAU_2015',70885,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70890,'Bianca (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',70800,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70890','projected_crs','IAU_2015',70890,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 709, 'Cressida', 31000.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 70900, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',70900,'Cressida (2015) - Sphere',NULL,'IAU_2015',709,31000.000000,'EPSG','9001',NULL,31000.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',70900,'Cressida (2015) - Sphere','','IAU_2015',70900,'IAU_2015',70900,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_70900','geodetic_datum','IAU_2015',70900,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',70900,'Cressida (2015) - Sphere / Ocentric','Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',70900,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_70900','geodetic_crs','IAU_2015',70900,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70910,'Cressida (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',70900,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70910','projected_crs','IAU_2015',70910,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70915,'Cressida (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',70900,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70915','projected_crs','IAU_2015',70915,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70920,'Cressida (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',70900,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70920','projected_crs','IAU_2015',70920,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70925,'Cressida (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',70900,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70925','projected_crs','IAU_2015',70925,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70930,'Cressida (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',70900,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70930','projected_crs','IAU_2015',70930,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70935,'Cressida (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',70900,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70935','projected_crs','IAU_2015',70935,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70940,'Cressida (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',70900,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70940','projected_crs','IAU_2015',70940,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70945,'Cressida (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',70900,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70945','projected_crs','IAU_2015',70945,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70950,'Cressida (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',70900,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70950','projected_crs','IAU_2015',70950,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70955,'Cressida (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',70900,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70955','projected_crs','IAU_2015',70955,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70960,'Cressida (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',70900,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70960','projected_crs','IAU_2015',70960,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70965,'Cressida (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',70900,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70965','projected_crs','IAU_2015',70965,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70970,'Cressida (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',70900,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70970','projected_crs','IAU_2015',70970,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70975,'Cressida (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',70900,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70975','projected_crs','IAU_2015',70975,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70980,'Cressida (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',70900,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70980','projected_crs','IAU_2015',70980,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70985,'Cressida (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',70900,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70985','projected_crs','IAU_2015',70985,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',70990,'Cressida (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',70900,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_70990','projected_crs','IAU_2015',70990,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 710, 'Desdemona', 27000.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 71000, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',71000,'Desdemona (2015) - Sphere',NULL,'IAU_2015',710,27000.000000,'EPSG','9001',NULL,27000.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',71000,'Desdemona (2015) - Sphere','','IAU_2015',71000,'IAU_2015',71000,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_71000','geodetic_datum','IAU_2015',71000,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',71000,'Desdemona (2015) - Sphere / Ocentric','Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',71000,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_71000','geodetic_crs','IAU_2015',71000,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71010,'Desdemona (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',71000,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71010','projected_crs','IAU_2015',71010,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71015,'Desdemona (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',71000,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71015','projected_crs','IAU_2015',71015,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71020,'Desdemona (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',71000,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71020','projected_crs','IAU_2015',71020,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71025,'Desdemona (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',71000,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71025','projected_crs','IAU_2015',71025,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71030,'Desdemona (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',71000,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71030','projected_crs','IAU_2015',71030,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71035,'Desdemona (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',71000,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71035','projected_crs','IAU_2015',71035,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71040,'Desdemona (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',71000,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71040','projected_crs','IAU_2015',71040,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71045,'Desdemona (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',71000,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71045','projected_crs','IAU_2015',71045,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71050,'Desdemona (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',71000,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71050','projected_crs','IAU_2015',71050,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71055,'Desdemona (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',71000,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71055','projected_crs','IAU_2015',71055,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71060,'Desdemona (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',71000,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71060','projected_crs','IAU_2015',71060,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71065,'Desdemona (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',71000,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71065','projected_crs','IAU_2015',71065,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71070,'Desdemona (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',71000,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71070','projected_crs','IAU_2015',71070,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71075,'Desdemona (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',71000,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71075','projected_crs','IAU_2015',71075,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71080,'Desdemona (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',71000,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71080','projected_crs','IAU_2015',71080,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71085,'Desdemona (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',71000,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71085','projected_crs','IAU_2015',71085,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71090,'Desdemona (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',71000,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71090','projected_crs','IAU_2015',71090,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 711, 'Juliet', 42000.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 71100, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',71100,'Juliet (2015) - Sphere',NULL,'IAU_2015',711,42000.000000,'EPSG','9001',NULL,42000.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',71100,'Juliet (2015) - Sphere','','IAU_2015',71100,'IAU_2015',71100,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_71100','geodetic_datum','IAU_2015',71100,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',71100,'Juliet (2015) - Sphere / Ocentric','Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',71100,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_71100','geodetic_crs','IAU_2015',71100,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71110,'Juliet (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',71100,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71110','projected_crs','IAU_2015',71110,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71115,'Juliet (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',71100,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71115','projected_crs','IAU_2015',71115,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71120,'Juliet (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',71100,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71120','projected_crs','IAU_2015',71120,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71125,'Juliet (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',71100,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71125','projected_crs','IAU_2015',71125,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71130,'Juliet (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',71100,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71130','projected_crs','IAU_2015',71130,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71135,'Juliet (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',71100,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71135','projected_crs','IAU_2015',71135,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71140,'Juliet (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',71100,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71140','projected_crs','IAU_2015',71140,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71145,'Juliet (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',71100,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71145','projected_crs','IAU_2015',71145,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71150,'Juliet (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',71100,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71150','projected_crs','IAU_2015',71150,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71155,'Juliet (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',71100,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71155','projected_crs','IAU_2015',71155,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71160,'Juliet (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',71100,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71160','projected_crs','IAU_2015',71160,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71165,'Juliet (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',71100,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71165','projected_crs','IAU_2015',71165,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71170,'Juliet (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',71100,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71170','projected_crs','IAU_2015',71170,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71175,'Juliet (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',71100,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71175','projected_crs','IAU_2015',71175,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71180,'Juliet (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',71100,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71180','projected_crs','IAU_2015',71180,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71185,'Juliet (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',71100,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71185','projected_crs','IAU_2015',71185,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71190,'Juliet (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',71100,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71190','projected_crs','IAU_2015',71190,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 712, 'Portia', 54000.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 71200, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',71200,'Portia (2015) - Sphere',NULL,'IAU_2015',712,54000.000000,'EPSG','9001',NULL,54000.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',71200,'Portia (2015) - Sphere','','IAU_2015',71200,'IAU_2015',71200,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_71200','geodetic_datum','IAU_2015',71200,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',71200,'Portia (2015) - Sphere / Ocentric','Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',71200,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_71200','geodetic_crs','IAU_2015',71200,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71210,'Portia (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',71200,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71210','projected_crs','IAU_2015',71210,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71215,'Portia (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',71200,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71215','projected_crs','IAU_2015',71215,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71220,'Portia (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',71200,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71220','projected_crs','IAU_2015',71220,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71225,'Portia (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',71200,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71225','projected_crs','IAU_2015',71225,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71230,'Portia (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',71200,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71230','projected_crs','IAU_2015',71230,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71235,'Portia (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',71200,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71235','projected_crs','IAU_2015',71235,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71240,'Portia (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',71200,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71240','projected_crs','IAU_2015',71240,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71245,'Portia (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',71200,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71245','projected_crs','IAU_2015',71245,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71250,'Portia (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',71200,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71250','projected_crs','IAU_2015',71250,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71255,'Portia (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',71200,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71255','projected_crs','IAU_2015',71255,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71260,'Portia (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',71200,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71260','projected_crs','IAU_2015',71260,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71265,'Portia (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',71200,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71265','projected_crs','IAU_2015',71265,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71270,'Portia (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',71200,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71270','projected_crs','IAU_2015',71270,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71275,'Portia (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',71200,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71275','projected_crs','IAU_2015',71275,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71280,'Portia (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',71200,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71280','projected_crs','IAU_2015',71280,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71285,'Portia (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',71200,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71285','projected_crs','IAU_2015',71285,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71290,'Portia (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',71200,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71290','projected_crs','IAU_2015',71290,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 713, 'Rosalind', 27000.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 71300, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',71300,'Rosalind (2015) - Sphere',NULL,'IAU_2015',713,27000.000000,'EPSG','9001',NULL,27000.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',71300,'Rosalind (2015) - Sphere','','IAU_2015',71300,'IAU_2015',71300,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_71300','geodetic_datum','IAU_2015',71300,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',71300,'Rosalind (2015) - Sphere / Ocentric','Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',71300,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_71300','geodetic_crs','IAU_2015',71300,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71310,'Rosalind (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',71300,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71310','projected_crs','IAU_2015',71310,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71315,'Rosalind (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',71300,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71315','projected_crs','IAU_2015',71315,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71320,'Rosalind (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',71300,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71320','projected_crs','IAU_2015',71320,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71325,'Rosalind (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',71300,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71325','projected_crs','IAU_2015',71325,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71330,'Rosalind (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',71300,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71330','projected_crs','IAU_2015',71330,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71335,'Rosalind (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',71300,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71335','projected_crs','IAU_2015',71335,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71340,'Rosalind (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',71300,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71340','projected_crs','IAU_2015',71340,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71345,'Rosalind (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',71300,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71345','projected_crs','IAU_2015',71345,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71350,'Rosalind (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',71300,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71350','projected_crs','IAU_2015',71350,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71355,'Rosalind (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',71300,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71355','projected_crs','IAU_2015',71355,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71360,'Rosalind (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',71300,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71360','projected_crs','IAU_2015',71360,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71365,'Rosalind (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',71300,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71365','projected_crs','IAU_2015',71365,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71370,'Rosalind (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',71300,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71370','projected_crs','IAU_2015',71370,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71375,'Rosalind (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',71300,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71375','projected_crs','IAU_2015',71375,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71380,'Rosalind (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',71300,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71380','projected_crs','IAU_2015',71380,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71385,'Rosalind (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',71300,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71385','projected_crs','IAU_2015',71385,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71390,'Rosalind (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',71300,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71390','projected_crs','IAU_2015',71390,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 714, 'Belinda', 33000.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 71400, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',71400,'Belinda (2015) - Sphere',NULL,'IAU_2015',714,33000.000000,'EPSG','9001',NULL,33000.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',71400,'Belinda (2015) - Sphere','','IAU_2015',71400,'IAU_2015',71400,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_71400','geodetic_datum','IAU_2015',71400,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',71400,'Belinda (2015) - Sphere / Ocentric','Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',71400,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_71400','geodetic_crs','IAU_2015',71400,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71410,'Belinda (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',71400,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71410','projected_crs','IAU_2015',71410,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71415,'Belinda (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',71400,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71415','projected_crs','IAU_2015',71415,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71420,'Belinda (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',71400,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71420','projected_crs','IAU_2015',71420,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71425,'Belinda (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',71400,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71425','projected_crs','IAU_2015',71425,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71430,'Belinda (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',71400,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71430','projected_crs','IAU_2015',71430,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71435,'Belinda (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',71400,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71435','projected_crs','IAU_2015',71435,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71440,'Belinda (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',71400,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71440','projected_crs','IAU_2015',71440,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71445,'Belinda (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',71400,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71445','projected_crs','IAU_2015',71445,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71450,'Belinda (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',71400,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71450','projected_crs','IAU_2015',71450,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71455,'Belinda (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',71400,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71455','projected_crs','IAU_2015',71455,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71460,'Belinda (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',71400,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71460','projected_crs','IAU_2015',71460,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71465,'Belinda (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',71400,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71465','projected_crs','IAU_2015',71465,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71470,'Belinda (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',71400,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71470','projected_crs','IAU_2015',71470,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71475,'Belinda (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',71400,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71475','projected_crs','IAU_2015',71475,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71480,'Belinda (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',71400,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71480','projected_crs','IAU_2015',71480,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71485,'Belinda (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',71400,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71485','projected_crs','IAU_2015',71485,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71490,'Belinda (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',71400,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71490','projected_crs','IAU_2015',71490,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 715, 'Puck', 77000.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 71500, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',71500,'Puck (2015) - Sphere',NULL,'IAU_2015',715,77000.000000,'EPSG','9001',NULL,77000.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',71500,'Puck (2015) - Sphere','','IAU_2015',71500,'IAU_2015',71500,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_71500','geodetic_datum','IAU_2015',71500,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',71500,'Puck (2015) - Sphere / Ocentric','Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',71500,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_71500','geodetic_crs','IAU_2015',71500,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71510,'Puck (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',71500,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71510','projected_crs','IAU_2015',71510,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71515,'Puck (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',71500,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71515','projected_crs','IAU_2015',71515,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71520,'Puck (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',71500,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71520','projected_crs','IAU_2015',71520,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71525,'Puck (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',71500,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71525','projected_crs','IAU_2015',71525,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71530,'Puck (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',71500,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71530','projected_crs','IAU_2015',71530,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71535,'Puck (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',71500,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71535','projected_crs','IAU_2015',71535,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71540,'Puck (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',71500,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71540','projected_crs','IAU_2015',71540,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71545,'Puck (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',71500,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71545','projected_crs','IAU_2015',71545,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71550,'Puck (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',71500,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71550','projected_crs','IAU_2015',71550,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71555,'Puck (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',71500,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71555','projected_crs','IAU_2015',71555,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71560,'Puck (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',71500,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71560','projected_crs','IAU_2015',71560,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71565,'Puck (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',71500,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71565','projected_crs','IAU_2015',71565,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71570,'Puck (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',71500,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71570','projected_crs','IAU_2015',71570,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71575,'Puck (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',71500,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71575','projected_crs','IAU_2015',71575,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71580,'Puck (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',71500,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71580','projected_crs','IAU_2015',71580,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71585,'Puck (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',71500,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71585','projected_crs','IAU_2015',71585,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',71590,'Puck (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',71500,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_71590','projected_crs','IAU_2015',71590,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
-- Skip Caliban has it lacks all axis information
-- Skip Sycorax has it lacks all axis information
-- Skip Prospero has it lacks all axis information
-- Skip Setebos has it lacks all axis information
-- Skip Stephano has it lacks all axis information
-- Skip Trinculo has it lacks all axis information
-- Skip Francisco has it lacks all axis information
-- Skip Margaret has it lacks all axis information
-- Skip Ferdinand has it lacks all axis information
-- Skip Perdita has it lacks all axis information
-- Skip Mab has it lacks all axis information
-- Skip Cupid has it lacks all axis information
INSERT INTO celestial_body VALUES('IAU_2015', 899, 'Neptune', 24764000.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 89900, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',89900,'Neptune (2015) - Sphere',NULL,'IAU_2015',899,24764000.000000,'EPSG','9001',NULL,24764000.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',89900,'Neptune (2015) - Sphere','','IAU_2015',89900,'IAU_2015',89900,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_89900','geodetic_datum','IAU_2015',89900,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',89900,'Neptune (2015) - Sphere / Ocentric','Use semi-major radius as sphere for interoperability. Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',89900,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_89900','geodetic_crs','IAU_2015',89900,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',89910,'Neptune (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',89900,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_89910','projected_crs','IAU_2015',89910,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',89915,'Neptune (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',89900,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_89915','projected_crs','IAU_2015',89915,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',89920,'Neptune (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',89900,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_89920','projected_crs','IAU_2015',89920,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',89925,'Neptune (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',89900,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_89925','projected_crs','IAU_2015',89925,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',89930,'Neptune (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',89900,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_89930','projected_crs','IAU_2015',89930,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',89935,'Neptune (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',89900,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_89935','projected_crs','IAU_2015',89935,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',89940,'Neptune (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',89900,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_89940','projected_crs','IAU_2015',89940,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',89945,'Neptune (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',89900,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_89945','projected_crs','IAU_2015',89945,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',89950,'Neptune (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',89900,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_89950','projected_crs','IAU_2015',89950,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',89955,'Neptune (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',89900,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_89955','projected_crs','IAU_2015',89955,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',89960,'Neptune (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',89900,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_89960','projected_crs','IAU_2015',89960,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',89965,'Neptune (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',89900,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_89965','projected_crs','IAU_2015',89965,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',89970,'Neptune (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',89900,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_89970','projected_crs','IAU_2015',89970,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',89975,'Neptune (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',89900,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_89975','projected_crs','IAU_2015',89975,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',89980,'Neptune (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',89900,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_89980','projected_crs','IAU_2015',89980,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',89985,'Neptune (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',89900,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_89985','projected_crs','IAU_2015',89985,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',89990,'Neptune (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',89900,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_89990','projected_crs','IAU_2015',89990,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO ellipsoid VALUES('IAU_2015',89901,'Neptune (2015)',NULL,'IAU_2015',899,24764000.000000,'EPSG','9001',NULL,24341000.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',89901,'Neptune (2015)','','IAU_2015',89901,'IAU_2015',89900,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_89901','geodetic_datum','IAU_2015',89901,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',89901,'Neptune (2015) / Ographic','Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','PROJ','OGRAPHIC_NORTH_WEST','IAU_2015',89901,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_89901','geodetic_crs','IAU_2015',89901,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',89911,'Neptune (2015) / Ographic / Equirectangular, clon = 0',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',89901,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_89911','projected_crs','IAU_2015',89911,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',89916,'Neptune (2015) / Ographic / Equirectangular, clon = 180',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',89901,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_89916','projected_crs','IAU_2015',89916,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',89921,'Neptune (2015) / Ographic / Sinusoidal, clon = 0',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',89901,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_89921','projected_crs','IAU_2015',89921,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',89926,'Neptune (2015) / Ographic / Sinusoidal, clon = 180',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',89901,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_89926','projected_crs','IAU_2015',89926,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',89931,'Neptune (2015) / Ographic / North Polar',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',89901,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_89931','projected_crs','IAU_2015',89931,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',89936,'Neptune (2015) / Ographic / South Polar',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',89901,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_89936','projected_crs','IAU_2015',89936,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',89941,'Neptune (2015) / Ographic / Mollweide, clon = 0',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',89901,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_89941','projected_crs','IAU_2015',89941,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',89946,'Neptune (2015) / Ographic / Mollweide, clon = 180',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',89901,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_89946','projected_crs','IAU_2015',89946,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',89951,'Neptune (2015) / Ographic / Robinson, clon = 0',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',89901,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_89951','projected_crs','IAU_2015',89951,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',89956,'Neptune (2015) / Ographic / Robinson, clon = 180',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',89901,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_89956','projected_crs','IAU_2015',89956,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',89961,'Neptune (2015) / Ographic / Transverse Mercator',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',89901,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_89961','projected_crs','IAU_2015',89961,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',89966,'Neptune (2015) / Ographic / Orthographic, clon = 0',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',89901,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_89966','projected_crs','IAU_2015',89966,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',89971,'Neptune (2015) / Ographic / Orthographic, clon = 180',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',89901,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_89971','projected_crs','IAU_2015',89971,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',89976,'Neptune (2015) / Ographic / Lambert Conic Conformal',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',89901,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_89976','projected_crs','IAU_2015',89976,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',89981,'Neptune (2015) / Ographic / Lambert Azimuthal Equal Area',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',89901,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_89981','projected_crs','IAU_2015',89981,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',89986,'Neptune (2015) / Ographic / Albers Equal Area',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',89901,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_89986','projected_crs','IAU_2015',89986,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',89991,'Neptune (2015) / Ographic / Mercator',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',89901,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_89991','projected_crs','IAU_2015',89991,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',89902,'Neptune (2015) / Ocentric','Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','other','PROJ','OCENTRIC_LAT_LON','IAU_2015',89901,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_89902','geodetic_crs','IAU_2015',89902,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',89912,'Neptune (2015) / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',89902,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_89912','projected_crs','IAU_2015',89912,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',89917,'Neptune (2015) / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',89902,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_89917','projected_crs','IAU_2015',89917,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',89922,'Neptune (2015) / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',89902,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_89922','projected_crs','IAU_2015',89922,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',89927,'Neptune (2015) / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',89902,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_89927','projected_crs','IAU_2015',89927,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',89932,'Neptune (2015) / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',89902,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_89932','projected_crs','IAU_2015',89932,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',89937,'Neptune (2015) / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',89902,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_89937','projected_crs','IAU_2015',89937,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',89942,'Neptune (2015) / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',89902,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_89942','projected_crs','IAU_2015',89942,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',89947,'Neptune (2015) / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',89902,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_89947','projected_crs','IAU_2015',89947,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',89952,'Neptune (2015) / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',89902,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_89952','projected_crs','IAU_2015',89952,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',89957,'Neptune (2015) / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',89902,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_89957','projected_crs','IAU_2015',89957,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',89962,'Neptune (2015) / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',89902,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_89962','projected_crs','IAU_2015',89962,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',89967,'Neptune (2015) / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',89902,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_89967','projected_crs','IAU_2015',89967,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',89972,'Neptune (2015) / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',89902,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_89972','projected_crs','IAU_2015',89972,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',89977,'Neptune (2015) / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',89902,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_89977','projected_crs','IAU_2015',89977,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',89982,'Neptune (2015) / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',89902,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_89982','projected_crs','IAU_2015',89982,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',89987,'Neptune (2015) / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',89902,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_89987','projected_crs','IAU_2015',89987,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',89992,'Neptune (2015) / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',89902,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_89992','projected_crs','IAU_2015',89992,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 801, 'Triton', 1352600.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 80100, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',80100,'Triton (2015) - Sphere',NULL,'IAU_2015',801,1352600.000000,'EPSG','9001',NULL,1352600.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',80100,'Triton (2015) - Sphere','','IAU_2015',80100,'IAU_2015',80100,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_80100','geodetic_datum','IAU_2015',80100,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',80100,'Triton (2015) - Sphere / Ocentric','Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',80100,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_80100','geodetic_crs','IAU_2015',80100,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80110,'Triton (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',80100,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80110','projected_crs','IAU_2015',80110,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80115,'Triton (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',80100,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80115','projected_crs','IAU_2015',80115,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80120,'Triton (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',80100,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80120','projected_crs','IAU_2015',80120,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80125,'Triton (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',80100,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80125','projected_crs','IAU_2015',80125,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80130,'Triton (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',80100,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80130','projected_crs','IAU_2015',80130,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80135,'Triton (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',80100,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80135','projected_crs','IAU_2015',80135,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80140,'Triton (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',80100,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80140','projected_crs','IAU_2015',80140,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80145,'Triton (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',80100,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80145','projected_crs','IAU_2015',80145,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80150,'Triton (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',80100,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80150','projected_crs','IAU_2015',80150,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80155,'Triton (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',80100,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80155','projected_crs','IAU_2015',80155,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80160,'Triton (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',80100,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80160','projected_crs','IAU_2015',80160,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80165,'Triton (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',80100,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80165','projected_crs','IAU_2015',80165,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80170,'Triton (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',80100,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80170','projected_crs','IAU_2015',80170,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80175,'Triton (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',80100,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80175','projected_crs','IAU_2015',80175,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80180,'Triton (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',80100,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80180','projected_crs','IAU_2015',80180,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80185,'Triton (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',80100,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80185','projected_crs','IAU_2015',80185,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80190,'Triton (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',80100,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80190','projected_crs','IAU_2015',80190,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 802, 'Nereid', 170000.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 80200, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',80200,'Nereid (2015) - Sphere',NULL,'IAU_2015',802,170000.000000,'EPSG','9001',NULL,170000.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',80200,'Nereid (2015) - Sphere','','IAU_2015',80200,'IAU_2015',80200,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_80200','geodetic_datum','IAU_2015',80200,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',80200,'Nereid (2015) - Sphere / Ocentric','Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',80200,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_80200','geodetic_crs','IAU_2015',80200,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80210,'Nereid (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',80200,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80210','projected_crs','IAU_2015',80210,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80215,'Nereid (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',80200,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80215','projected_crs','IAU_2015',80215,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80220,'Nereid (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',80200,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80220','projected_crs','IAU_2015',80220,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80225,'Nereid (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',80200,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80225','projected_crs','IAU_2015',80225,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80230,'Nereid (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',80200,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80230','projected_crs','IAU_2015',80230,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80235,'Nereid (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',80200,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80235','projected_crs','IAU_2015',80235,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80240,'Nereid (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',80200,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80240','projected_crs','IAU_2015',80240,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80245,'Nereid (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',80200,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80245','projected_crs','IAU_2015',80245,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80250,'Nereid (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',80200,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80250','projected_crs','IAU_2015',80250,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80255,'Nereid (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',80200,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80255','projected_crs','IAU_2015',80255,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80260,'Nereid (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',80200,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80260','projected_crs','IAU_2015',80260,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80265,'Nereid (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',80200,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80265','projected_crs','IAU_2015',80265,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80270,'Nereid (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',80200,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80270','projected_crs','IAU_2015',80270,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80275,'Nereid (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',80200,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80275','projected_crs','IAU_2015',80275,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80280,'Nereid (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',80200,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80280','projected_crs','IAU_2015',80280,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80285,'Nereid (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',80200,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80285','projected_crs','IAU_2015',80285,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80290,'Nereid (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',80200,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80290','projected_crs','IAU_2015',80290,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 803, 'Naiad', 29000.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 80300, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',80300,'Naiad (2015) - Sphere',NULL,'IAU_2015',803,29000.000000,'EPSG','9001',NULL,29000.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',80300,'Naiad (2015) - Sphere','','IAU_2015',80300,'IAU_2015',80300,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_80300','geodetic_datum','IAU_2015',80300,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',80300,'Naiad (2015) - Sphere / Ocentric','Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',80300,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_80300','geodetic_crs','IAU_2015',80300,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80310,'Naiad (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',80300,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80310','projected_crs','IAU_2015',80310,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80315,'Naiad (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',80300,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80315','projected_crs','IAU_2015',80315,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80320,'Naiad (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',80300,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80320','projected_crs','IAU_2015',80320,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80325,'Naiad (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',80300,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80325','projected_crs','IAU_2015',80325,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80330,'Naiad (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',80300,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80330','projected_crs','IAU_2015',80330,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80335,'Naiad (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',80300,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80335','projected_crs','IAU_2015',80335,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80340,'Naiad (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',80300,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80340','projected_crs','IAU_2015',80340,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80345,'Naiad (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',80300,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80345','projected_crs','IAU_2015',80345,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80350,'Naiad (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',80300,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80350','projected_crs','IAU_2015',80350,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80355,'Naiad (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',80300,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80355','projected_crs','IAU_2015',80355,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80360,'Naiad (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',80300,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80360','projected_crs','IAU_2015',80360,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80365,'Naiad (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',80300,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80365','projected_crs','IAU_2015',80365,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80370,'Naiad (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',80300,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80370','projected_crs','IAU_2015',80370,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80375,'Naiad (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',80300,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80375','projected_crs','IAU_2015',80375,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80380,'Naiad (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',80300,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80380','projected_crs','IAU_2015',80380,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80385,'Naiad (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',80300,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80385','projected_crs','IAU_2015',80385,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80390,'Naiad (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',80300,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80390','projected_crs','IAU_2015',80390,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO ellipsoid VALUES('IAU_2015',80301,'Naiad (2015)',NULL,'IAU_2015',803,29000.000000,'EPSG','9001',NULL,29000.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',80301,'Naiad (2015)','','IAU_2015',80301,'IAU_2015',80300,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_80301','geodetic_datum','IAU_2015',80301,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',80301,'Naiad (2015) / Ographic','Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','PROJ','OGRAPHIC_NORTH_WEST','IAU_2015',80301,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_80301','geodetic_crs','IAU_2015',80301,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80311,'Naiad (2015) / Ographic / Equirectangular, clon = 0',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80301,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80311','projected_crs','IAU_2015',80311,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80316,'Naiad (2015) / Ographic / Equirectangular, clon = 180',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80301,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80316','projected_crs','IAU_2015',80316,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80321,'Naiad (2015) / Ographic / Sinusoidal, clon = 0',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80301,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80321','projected_crs','IAU_2015',80321,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80326,'Naiad (2015) / Ographic / Sinusoidal, clon = 180',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80301,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80326','projected_crs','IAU_2015',80326,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80331,'Naiad (2015) / Ographic / North Polar',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80301,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80331','projected_crs','IAU_2015',80331,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80336,'Naiad (2015) / Ographic / South Polar',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80301,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80336','projected_crs','IAU_2015',80336,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80341,'Naiad (2015) / Ographic / Mollweide, clon = 0',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80301,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80341','projected_crs','IAU_2015',80341,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80346,'Naiad (2015) / Ographic / Mollweide, clon = 180',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80301,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80346','projected_crs','IAU_2015',80346,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80351,'Naiad (2015) / Ographic / Robinson, clon = 0',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80301,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80351','projected_crs','IAU_2015',80351,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80356,'Naiad (2015) / Ographic / Robinson, clon = 180',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80301,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80356','projected_crs','IAU_2015',80356,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80361,'Naiad (2015) / Ographic / Transverse Mercator',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80301,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80361','projected_crs','IAU_2015',80361,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80366,'Naiad (2015) / Ographic / Orthographic, clon = 0',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80301,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80366','projected_crs','IAU_2015',80366,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80371,'Naiad (2015) / Ographic / Orthographic, clon = 180',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80301,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80371','projected_crs','IAU_2015',80371,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80376,'Naiad (2015) / Ographic / Lambert Conic Conformal',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80301,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80376','projected_crs','IAU_2015',80376,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80381,'Naiad (2015) / Ographic / Lambert Azimuthal Equal Area',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80301,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80381','projected_crs','IAU_2015',80381,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80386,'Naiad (2015) / Ographic / Albers Equal Area',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80301,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80386','projected_crs','IAU_2015',80386,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80391,'Naiad (2015) / Ographic / Mercator',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80301,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80391','projected_crs','IAU_2015',80391,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 804, 'Thalassa', 40000.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 80400, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',80400,'Thalassa (2015) - Sphere',NULL,'IAU_2015',804,40000.000000,'EPSG','9001',NULL,40000.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',80400,'Thalassa (2015) - Sphere','','IAU_2015',80400,'IAU_2015',80400,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_80400','geodetic_datum','IAU_2015',80400,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',80400,'Thalassa (2015) - Sphere / Ocentric','Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',80400,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_80400','geodetic_crs','IAU_2015',80400,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80410,'Thalassa (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',80400,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80410','projected_crs','IAU_2015',80410,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80415,'Thalassa (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',80400,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80415','projected_crs','IAU_2015',80415,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80420,'Thalassa (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',80400,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80420','projected_crs','IAU_2015',80420,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80425,'Thalassa (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',80400,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80425','projected_crs','IAU_2015',80425,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80430,'Thalassa (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',80400,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80430','projected_crs','IAU_2015',80430,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80435,'Thalassa (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',80400,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80435','projected_crs','IAU_2015',80435,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80440,'Thalassa (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',80400,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80440','projected_crs','IAU_2015',80440,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80445,'Thalassa (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',80400,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80445','projected_crs','IAU_2015',80445,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80450,'Thalassa (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',80400,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80450','projected_crs','IAU_2015',80450,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80455,'Thalassa (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',80400,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80455','projected_crs','IAU_2015',80455,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80460,'Thalassa (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',80400,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80460','projected_crs','IAU_2015',80460,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80465,'Thalassa (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',80400,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80465','projected_crs','IAU_2015',80465,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80470,'Thalassa (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',80400,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80470','projected_crs','IAU_2015',80470,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80475,'Thalassa (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',80400,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80475','projected_crs','IAU_2015',80475,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80480,'Thalassa (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',80400,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80480','projected_crs','IAU_2015',80480,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80485,'Thalassa (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',80400,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80485','projected_crs','IAU_2015',80485,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80490,'Thalassa (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',80400,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80490','projected_crs','IAU_2015',80490,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO ellipsoid VALUES('IAU_2015',80401,'Thalassa (2015)',NULL,'IAU_2015',804,40000.000000,'EPSG','9001',NULL,40000.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',80401,'Thalassa (2015)','','IAU_2015',80401,'IAU_2015',80400,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_80401','geodetic_datum','IAU_2015',80401,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',80401,'Thalassa (2015) / Ographic','Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','PROJ','OGRAPHIC_NORTH_WEST','IAU_2015',80401,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_80401','geodetic_crs','IAU_2015',80401,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80411,'Thalassa (2015) / Ographic / Equirectangular, clon = 0',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80401,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80411','projected_crs','IAU_2015',80411,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80416,'Thalassa (2015) / Ographic / Equirectangular, clon = 180',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80401,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80416','projected_crs','IAU_2015',80416,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80421,'Thalassa (2015) / Ographic / Sinusoidal, clon = 0',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80401,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80421','projected_crs','IAU_2015',80421,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80426,'Thalassa (2015) / Ographic / Sinusoidal, clon = 180',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80401,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80426','projected_crs','IAU_2015',80426,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80431,'Thalassa (2015) / Ographic / North Polar',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80401,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80431','projected_crs','IAU_2015',80431,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80436,'Thalassa (2015) / Ographic / South Polar',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80401,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80436','projected_crs','IAU_2015',80436,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80441,'Thalassa (2015) / Ographic / Mollweide, clon = 0',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80401,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80441','projected_crs','IAU_2015',80441,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80446,'Thalassa (2015) / Ographic / Mollweide, clon = 180',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80401,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80446','projected_crs','IAU_2015',80446,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80451,'Thalassa (2015) / Ographic / Robinson, clon = 0',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80401,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80451','projected_crs','IAU_2015',80451,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80456,'Thalassa (2015) / Ographic / Robinson, clon = 180',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80401,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80456','projected_crs','IAU_2015',80456,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80461,'Thalassa (2015) / Ographic / Transverse Mercator',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80401,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80461','projected_crs','IAU_2015',80461,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80466,'Thalassa (2015) / Ographic / Orthographic, clon = 0',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80401,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80466','projected_crs','IAU_2015',80466,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80471,'Thalassa (2015) / Ographic / Orthographic, clon = 180',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80401,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80471','projected_crs','IAU_2015',80471,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80476,'Thalassa (2015) / Ographic / Lambert Conic Conformal',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80401,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80476','projected_crs','IAU_2015',80476,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80481,'Thalassa (2015) / Ographic / Lambert Azimuthal Equal Area',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80401,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80481','projected_crs','IAU_2015',80481,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80486,'Thalassa (2015) / Ographic / Albers Equal Area',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80401,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80486','projected_crs','IAU_2015',80486,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80491,'Thalassa (2015) / Ographic / Mercator',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80401,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80491','projected_crs','IAU_2015',80491,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 805, 'Despina', 74000.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 80500, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',80500,'Despina (2015) - Sphere',NULL,'IAU_2015',805,74000.000000,'EPSG','9001',NULL,74000.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',80500,'Despina (2015) - Sphere','','IAU_2015',80500,'IAU_2015',80500,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_80500','geodetic_datum','IAU_2015',80500,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',80500,'Despina (2015) - Sphere / Ocentric','Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',80500,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_80500','geodetic_crs','IAU_2015',80500,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80510,'Despina (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',80500,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80510','projected_crs','IAU_2015',80510,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80515,'Despina (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',80500,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80515','projected_crs','IAU_2015',80515,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80520,'Despina (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',80500,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80520','projected_crs','IAU_2015',80520,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80525,'Despina (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',80500,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80525','projected_crs','IAU_2015',80525,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80530,'Despina (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',80500,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80530','projected_crs','IAU_2015',80530,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80535,'Despina (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',80500,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80535','projected_crs','IAU_2015',80535,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80540,'Despina (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',80500,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80540','projected_crs','IAU_2015',80540,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80545,'Despina (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',80500,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80545','projected_crs','IAU_2015',80545,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80550,'Despina (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',80500,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80550','projected_crs','IAU_2015',80550,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80555,'Despina (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',80500,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80555','projected_crs','IAU_2015',80555,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80560,'Despina (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',80500,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80560','projected_crs','IAU_2015',80560,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80565,'Despina (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',80500,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80565','projected_crs','IAU_2015',80565,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80570,'Despina (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',80500,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80570','projected_crs','IAU_2015',80570,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80575,'Despina (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',80500,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80575','projected_crs','IAU_2015',80575,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80580,'Despina (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',80500,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80580','projected_crs','IAU_2015',80580,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80585,'Despina (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',80500,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80585','projected_crs','IAU_2015',80585,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80590,'Despina (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',80500,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80590','projected_crs','IAU_2015',80590,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO ellipsoid VALUES('IAU_2015',80501,'Despina (2015)',NULL,'IAU_2015',805,74000.000000,'EPSG','9001',NULL,74000.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',80501,'Despina (2015)','','IAU_2015',80501,'IAU_2015',80500,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_80501','geodetic_datum','IAU_2015',80501,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',80501,'Despina (2015) / Ographic','Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','PROJ','OGRAPHIC_NORTH_WEST','IAU_2015',80501,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_80501','geodetic_crs','IAU_2015',80501,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80511,'Despina (2015) / Ographic / Equirectangular, clon = 0',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80501,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80511','projected_crs','IAU_2015',80511,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80516,'Despina (2015) / Ographic / Equirectangular, clon = 180',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80501,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80516','projected_crs','IAU_2015',80516,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80521,'Despina (2015) / Ographic / Sinusoidal, clon = 0',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80501,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80521','projected_crs','IAU_2015',80521,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80526,'Despina (2015) / Ographic / Sinusoidal, clon = 180',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80501,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80526','projected_crs','IAU_2015',80526,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80531,'Despina (2015) / Ographic / North Polar',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80501,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80531','projected_crs','IAU_2015',80531,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80536,'Despina (2015) / Ographic / South Polar',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80501,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80536','projected_crs','IAU_2015',80536,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80541,'Despina (2015) / Ographic / Mollweide, clon = 0',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80501,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80541','projected_crs','IAU_2015',80541,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80546,'Despina (2015) / Ographic / Mollweide, clon = 180',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80501,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80546','projected_crs','IAU_2015',80546,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80551,'Despina (2015) / Ographic / Robinson, clon = 0',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80501,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80551','projected_crs','IAU_2015',80551,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80556,'Despina (2015) / Ographic / Robinson, clon = 180',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80501,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80556','projected_crs','IAU_2015',80556,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80561,'Despina (2015) / Ographic / Transverse Mercator',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80501,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80561','projected_crs','IAU_2015',80561,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80566,'Despina (2015) / Ographic / Orthographic, clon = 0',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80501,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80566','projected_crs','IAU_2015',80566,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80571,'Despina (2015) / Ographic / Orthographic, clon = 180',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80501,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80571','projected_crs','IAU_2015',80571,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80576,'Despina (2015) / Ographic / Lambert Conic Conformal',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80501,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80576','projected_crs','IAU_2015',80576,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80581,'Despina (2015) / Ographic / Lambert Azimuthal Equal Area',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80501,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80581','projected_crs','IAU_2015',80581,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80586,'Despina (2015) / Ographic / Albers Equal Area',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80501,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80586','projected_crs','IAU_2015',80586,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80591,'Despina (2015) / Ographic / Mercator',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80501,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80591','projected_crs','IAU_2015',80591,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 806, 'Galatea', 79000.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 80600, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',80600,'Galatea (2015) - Sphere',NULL,'IAU_2015',806,79000.000000,'EPSG','9001',NULL,79000.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',80600,'Galatea (2015) - Sphere','','IAU_2015',80600,'IAU_2015',80600,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_80600','geodetic_datum','IAU_2015',80600,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',80600,'Galatea (2015) - Sphere / Ocentric','Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',80600,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_80600','geodetic_crs','IAU_2015',80600,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80610,'Galatea (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',80600,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80610','projected_crs','IAU_2015',80610,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80615,'Galatea (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',80600,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80615','projected_crs','IAU_2015',80615,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80620,'Galatea (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',80600,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80620','projected_crs','IAU_2015',80620,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80625,'Galatea (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',80600,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80625','projected_crs','IAU_2015',80625,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80630,'Galatea (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',80600,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80630','projected_crs','IAU_2015',80630,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80635,'Galatea (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',80600,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80635','projected_crs','IAU_2015',80635,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80640,'Galatea (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',80600,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80640','projected_crs','IAU_2015',80640,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80645,'Galatea (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',80600,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80645','projected_crs','IAU_2015',80645,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80650,'Galatea (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',80600,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80650','projected_crs','IAU_2015',80650,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80655,'Galatea (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',80600,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80655','projected_crs','IAU_2015',80655,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80660,'Galatea (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',80600,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80660','projected_crs','IAU_2015',80660,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80665,'Galatea (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',80600,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80665','projected_crs','IAU_2015',80665,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80670,'Galatea (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',80600,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80670','projected_crs','IAU_2015',80670,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80675,'Galatea (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',80600,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80675','projected_crs','IAU_2015',80675,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80680,'Galatea (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',80600,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80680','projected_crs','IAU_2015',80680,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80685,'Galatea (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',80600,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80685','projected_crs','IAU_2015',80685,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80690,'Galatea (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',80600,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80690','projected_crs','IAU_2015',80690,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO ellipsoid VALUES('IAU_2015',80601,'Galatea (2015)',NULL,'IAU_2015',806,79000.000000,'EPSG','9001',NULL,79000.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',80601,'Galatea (2015)','','IAU_2015',80601,'IAU_2015',80600,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_80601','geodetic_datum','IAU_2015',80601,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',80601,'Galatea (2015) / Ographic','Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','PROJ','OGRAPHIC_NORTH_WEST','IAU_2015',80601,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_80601','geodetic_crs','IAU_2015',80601,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80611,'Galatea (2015) / Ographic / Equirectangular, clon = 0',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80601,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80611','projected_crs','IAU_2015',80611,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80616,'Galatea (2015) / Ographic / Equirectangular, clon = 180',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80601,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80616','projected_crs','IAU_2015',80616,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80621,'Galatea (2015) / Ographic / Sinusoidal, clon = 0',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80601,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80621','projected_crs','IAU_2015',80621,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80626,'Galatea (2015) / Ographic / Sinusoidal, clon = 180',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80601,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80626','projected_crs','IAU_2015',80626,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80631,'Galatea (2015) / Ographic / North Polar',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80601,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80631','projected_crs','IAU_2015',80631,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80636,'Galatea (2015) / Ographic / South Polar',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80601,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80636','projected_crs','IAU_2015',80636,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80641,'Galatea (2015) / Ographic / Mollweide, clon = 0',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80601,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80641','projected_crs','IAU_2015',80641,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80646,'Galatea (2015) / Ographic / Mollweide, clon = 180',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80601,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80646','projected_crs','IAU_2015',80646,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80651,'Galatea (2015) / Ographic / Robinson, clon = 0',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80601,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80651','projected_crs','IAU_2015',80651,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80656,'Galatea (2015) / Ographic / Robinson, clon = 180',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80601,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80656','projected_crs','IAU_2015',80656,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80661,'Galatea (2015) / Ographic / Transverse Mercator',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80601,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80661','projected_crs','IAU_2015',80661,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80666,'Galatea (2015) / Ographic / Orthographic, clon = 0',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80601,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80666','projected_crs','IAU_2015',80666,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80671,'Galatea (2015) / Ographic / Orthographic, clon = 180',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80601,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80671','projected_crs','IAU_2015',80671,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80676,'Galatea (2015) / Ographic / Lambert Conic Conformal',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80601,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80676','projected_crs','IAU_2015',80676,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80681,'Galatea (2015) / Ographic / Lambert Azimuthal Equal Area',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80601,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80681','projected_crs','IAU_2015',80681,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80686,'Galatea (2015) / Ographic / Albers Equal Area',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80601,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80686','projected_crs','IAU_2015',80686,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80691,'Galatea (2015) / Ographic / Mercator',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80601,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80691','projected_crs','IAU_2015',80691,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 807, 'Larissa', 96000.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 80700, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',80700,'Larissa (2015) - Sphere',NULL,'IAU_2015',807,96000.000000,'EPSG','9001',NULL,96000.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',80700,'Larissa (2015) - Sphere','','IAU_2015',80700,'IAU_2015',80700,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_80700','geodetic_datum','IAU_2015',80700,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',80700,'Larissa (2015) - Sphere / Ocentric','Use semi-major radius as sphere for interoperability. Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',80700,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_80700','geodetic_crs','IAU_2015',80700,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80710,'Larissa (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',80700,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80710','projected_crs','IAU_2015',80710,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80715,'Larissa (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',80700,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80715','projected_crs','IAU_2015',80715,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80720,'Larissa (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',80700,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80720','projected_crs','IAU_2015',80720,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80725,'Larissa (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',80700,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80725','projected_crs','IAU_2015',80725,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80730,'Larissa (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',80700,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80730','projected_crs','IAU_2015',80730,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80735,'Larissa (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',80700,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80735','projected_crs','IAU_2015',80735,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80740,'Larissa (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',80700,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80740','projected_crs','IAU_2015',80740,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80745,'Larissa (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',80700,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80745','projected_crs','IAU_2015',80745,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80750,'Larissa (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',80700,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80750','projected_crs','IAU_2015',80750,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80755,'Larissa (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',80700,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80755','projected_crs','IAU_2015',80755,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80760,'Larissa (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',80700,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80760','projected_crs','IAU_2015',80760,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80765,'Larissa (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',80700,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80765','projected_crs','IAU_2015',80765,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80770,'Larissa (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',80700,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80770','projected_crs','IAU_2015',80770,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80775,'Larissa (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',80700,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80775','projected_crs','IAU_2015',80775,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80780,'Larissa (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',80700,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80780','projected_crs','IAU_2015',80780,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80785,'Larissa (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',80700,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80785','projected_crs','IAU_2015',80785,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80790,'Larissa (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',80700,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80790','projected_crs','IAU_2015',80790,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO ellipsoid VALUES('IAU_2015',80701,'Larissa (2015)',NULL,'IAU_2015',807,96000.000000,'EPSG','9001',NULL,89000.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',80701,'Larissa (2015)','','IAU_2015',80701,'IAU_2015',80700,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_80701','geodetic_datum','IAU_2015',80701,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',80701,'Larissa (2015) / Ographic','Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','PROJ','OGRAPHIC_NORTH_WEST','IAU_2015',80701,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_80701','geodetic_crs','IAU_2015',80701,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80711,'Larissa (2015) / Ographic / Equirectangular, clon = 0',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80701,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80711','projected_crs','IAU_2015',80711,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80716,'Larissa (2015) / Ographic / Equirectangular, clon = 180',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80701,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80716','projected_crs','IAU_2015',80716,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80721,'Larissa (2015) / Ographic / Sinusoidal, clon = 0',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80701,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80721','projected_crs','IAU_2015',80721,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80726,'Larissa (2015) / Ographic / Sinusoidal, clon = 180',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80701,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80726','projected_crs','IAU_2015',80726,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80731,'Larissa (2015) / Ographic / North Polar',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80701,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80731','projected_crs','IAU_2015',80731,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80736,'Larissa (2015) / Ographic / South Polar',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80701,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80736','projected_crs','IAU_2015',80736,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80741,'Larissa (2015) / Ographic / Mollweide, clon = 0',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80701,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80741','projected_crs','IAU_2015',80741,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80746,'Larissa (2015) / Ographic / Mollweide, clon = 180',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80701,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80746','projected_crs','IAU_2015',80746,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80751,'Larissa (2015) / Ographic / Robinson, clon = 0',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80701,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80751','projected_crs','IAU_2015',80751,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80756,'Larissa (2015) / Ographic / Robinson, clon = 180',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80701,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80756','projected_crs','IAU_2015',80756,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80761,'Larissa (2015) / Ographic / Transverse Mercator',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80701,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80761','projected_crs','IAU_2015',80761,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80766,'Larissa (2015) / Ographic / Orthographic, clon = 0',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80701,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80766','projected_crs','IAU_2015',80766,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80771,'Larissa (2015) / Ographic / Orthographic, clon = 180',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80701,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80771','projected_crs','IAU_2015',80771,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80776,'Larissa (2015) / Ographic / Lambert Conic Conformal',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80701,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80776','projected_crs','IAU_2015',80776,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80781,'Larissa (2015) / Ographic / Lambert Azimuthal Equal Area',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80701,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80781','projected_crs','IAU_2015',80781,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80786,'Larissa (2015) / Ographic / Albers Equal Area',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80701,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80786','projected_crs','IAU_2015',80786,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80791,'Larissa (2015) / Ographic / Mercator',NULL,'PROJ','PROJECTED_WEST_NORTH','IAU_2015',80701,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80791','projected_crs','IAU_2015',80791,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',80702,'Larissa (2015) / Ocentric','Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','other','PROJ','OCENTRIC_LAT_LON','IAU_2015',80701,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_80702','geodetic_crs','IAU_2015',80702,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80712,'Larissa (2015) / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',80702,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80712','projected_crs','IAU_2015',80712,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80717,'Larissa (2015) / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',80702,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80717','projected_crs','IAU_2015',80717,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80722,'Larissa (2015) / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',80702,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80722','projected_crs','IAU_2015',80722,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80727,'Larissa (2015) / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',80702,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80727','projected_crs','IAU_2015',80727,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80732,'Larissa (2015) / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',80702,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80732','projected_crs','IAU_2015',80732,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80737,'Larissa (2015) / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',80702,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80737','projected_crs','IAU_2015',80737,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80742,'Larissa (2015) / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',80702,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80742','projected_crs','IAU_2015',80742,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80747,'Larissa (2015) / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',80702,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80747','projected_crs','IAU_2015',80747,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80752,'Larissa (2015) / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',80702,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80752','projected_crs','IAU_2015',80752,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80757,'Larissa (2015) / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',80702,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80757','projected_crs','IAU_2015',80757,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80762,'Larissa (2015) / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',80702,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80762','projected_crs','IAU_2015',80762,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80767,'Larissa (2015) / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',80702,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80767','projected_crs','IAU_2015',80767,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80772,'Larissa (2015) / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',80702,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80772','projected_crs','IAU_2015',80772,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80777,'Larissa (2015) / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',80702,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80777','projected_crs','IAU_2015',80777,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80782,'Larissa (2015) / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',80702,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80782','projected_crs','IAU_2015',80782,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80787,'Larissa (2015) / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',80702,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80787','projected_crs','IAU_2015',80787,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80792,'Larissa (2015) / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',80702,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80792','projected_crs','IAU_2015',80792,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 808, 'Proteus', 208000.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 80800, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',80800,'Proteus (2015) - Sphere',NULL,'IAU_2015',808,208000.000000,'EPSG','9001',NULL,208000.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',80800,'Proteus (2015) - Sphere','','IAU_2015',80800,'IAU_2015',80800,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_80800','geodetic_datum','IAU_2015',80800,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',80800,'Proteus (2015) - Sphere / Ocentric','Use mean radius as sphere radius for interoperability. Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',80800,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_80800','geodetic_crs','IAU_2015',80800,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80810,'Proteus (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',80800,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80810','projected_crs','IAU_2015',80810,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80815,'Proteus (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',80800,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80815','projected_crs','IAU_2015',80815,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80820,'Proteus (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',80800,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80820','projected_crs','IAU_2015',80820,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80825,'Proteus (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',80800,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80825','projected_crs','IAU_2015',80825,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80830,'Proteus (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',80800,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80830','projected_crs','IAU_2015',80830,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80835,'Proteus (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',80800,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80835','projected_crs','IAU_2015',80835,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80840,'Proteus (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',80800,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80840','projected_crs','IAU_2015',80840,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80845,'Proteus (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',80800,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80845','projected_crs','IAU_2015',80845,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80850,'Proteus (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',80800,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80850','projected_crs','IAU_2015',80850,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80855,'Proteus (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',80800,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80855','projected_crs','IAU_2015',80855,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80860,'Proteus (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',80800,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80860','projected_crs','IAU_2015',80860,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80865,'Proteus (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',80800,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80865','projected_crs','IAU_2015',80865,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80870,'Proteus (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',80800,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80870','projected_crs','IAU_2015',80870,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80875,'Proteus (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',80800,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80875','projected_crs','IAU_2015',80875,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80880,'Proteus (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',80800,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80880','projected_crs','IAU_2015',80880,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80885,'Proteus (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',80800,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80885','projected_crs','IAU_2015',80885,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',80890,'Proteus (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',80800,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_80890','projected_crs','IAU_2015',80890,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
-- Skip Halimede has it lacks all axis information
-- Skip Psamathe has it lacks all axis information
-- Skip Sao has it lacks all axis information
-- Skip Laomedeia has it lacks all axis information
-- Skip Neso has it lacks all axis information
INSERT INTO celestial_body VALUES('IAU_2015', 999, 'Pluto', 1188300.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 99900, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',99900,'Pluto (2015) - Sphere',NULL,'IAU_2015',999,1188300.000000,'EPSG','9001',NULL,1188300.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',99900,'Pluto (2015) - Sphere','','IAU_2015',99900,'IAU_2015',99900,NULL,NULL,NULL,'Mean sub-Charon meridian: 0.0',NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_99900','geodetic_datum','IAU_2015',99900,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',99900,'Pluto (2015) - Sphere / Ocentric','Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',99900,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_99900','geodetic_crs','IAU_2015',99900,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',99910,'Pluto (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',99900,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_99910','projected_crs','IAU_2015',99910,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',99915,'Pluto (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',99900,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_99915','projected_crs','IAU_2015',99915,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',99920,'Pluto (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',99900,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_99920','projected_crs','IAU_2015',99920,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',99925,'Pluto (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',99900,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_99925','projected_crs','IAU_2015',99925,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',99930,'Pluto (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',99900,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_99930','projected_crs','IAU_2015',99930,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',99935,'Pluto (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',99900,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_99935','projected_crs','IAU_2015',99935,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',99940,'Pluto (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',99900,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_99940','projected_crs','IAU_2015',99940,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',99945,'Pluto (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',99900,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_99945','projected_crs','IAU_2015',99945,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',99950,'Pluto (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',99900,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_99950','projected_crs','IAU_2015',99950,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',99955,'Pluto (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',99900,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_99955','projected_crs','IAU_2015',99955,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',99960,'Pluto (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',99900,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_99960','projected_crs','IAU_2015',99960,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',99965,'Pluto (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',99900,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_99965','projected_crs','IAU_2015',99965,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',99970,'Pluto (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',99900,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_99970','projected_crs','IAU_2015',99970,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',99975,'Pluto (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',99900,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_99975','projected_crs','IAU_2015',99975,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',99980,'Pluto (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',99900,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_99980','projected_crs','IAU_2015',99980,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',99985,'Pluto (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',99900,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_99985','projected_crs','IAU_2015',99985,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',99990,'Pluto (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',99900,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_99990','projected_crs','IAU_2015',99990,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 901, 'Charon', 606000.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 90100, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',90100,'Charon (2015) - Sphere',NULL,'IAU_2015',901,606000.000000,'EPSG','9001',NULL,606000.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',90100,'Charon (2015) - Sphere','','IAU_2015',90100,'IAU_2015',90100,NULL,NULL,NULL,'Mean sub-Pluto meridian: 0.0',NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_90100','geodetic_datum','IAU_2015',90100,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',90100,'Charon (2015) - Sphere / Ocentric','Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',90100,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_90100','geodetic_crs','IAU_2015',90100,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',90110,'Charon (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',90100,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_90110','projected_crs','IAU_2015',90110,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',90115,'Charon (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',90100,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_90115','projected_crs','IAU_2015',90115,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',90120,'Charon (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',90100,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_90120','projected_crs','IAU_2015',90120,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',90125,'Charon (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',90100,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_90125','projected_crs','IAU_2015',90125,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',90130,'Charon (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',90100,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_90130','projected_crs','IAU_2015',90130,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',90135,'Charon (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',90100,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_90135','projected_crs','IAU_2015',90135,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',90140,'Charon (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',90100,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_90140','projected_crs','IAU_2015',90140,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',90145,'Charon (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',90100,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_90145','projected_crs','IAU_2015',90145,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',90150,'Charon (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',90100,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_90150','projected_crs','IAU_2015',90150,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',90155,'Charon (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',90100,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_90155','projected_crs','IAU_2015',90155,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',90160,'Charon (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',90100,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_90160','projected_crs','IAU_2015',90160,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',90165,'Charon (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',90100,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_90165','projected_crs','IAU_2015',90165,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',90170,'Charon (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',90100,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_90170','projected_crs','IAU_2015',90170,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',90175,'Charon (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',90100,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_90175','projected_crs','IAU_2015',90175,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',90180,'Charon (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',90100,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_90180','projected_crs','IAU_2015',90180,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',90185,'Charon (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',90100,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_90185','projected_crs','IAU_2015',90185,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',90190,'Charon (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',90100,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_90190','projected_crs','IAU_2015',90190,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
-- Borrelly lacks value for semimajor and/or axisb and/or semiminor. Only consider its mean value.
INSERT INTO celestial_body VALUES('IAU_2015', 1000005, 'Borrelly', 4220.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 100000500, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',100000500,'Borrelly (2015) - Sphere',NULL,'IAU_2015',1000005,4220.000000,'EPSG','9001',NULL,4220.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',100000500,'Borrelly (2015) - Sphere','','IAU_2015',100000500,'IAU_2015',100000500,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_100000500','geodetic_datum','IAU_2015',100000500,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',100000500,'Borrelly (2015) - Sphere / Ocentric','Use mean radius as sphere radius for interoperability. Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',100000500,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_100000500','geodetic_crs','IAU_2015',100000500,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100000510,'Borrelly (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',100000500,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100000510','projected_crs','IAU_2015',100000510,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100000515,'Borrelly (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',100000500,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100000515','projected_crs','IAU_2015',100000515,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100000520,'Borrelly (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',100000500,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100000520','projected_crs','IAU_2015',100000520,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100000525,'Borrelly (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',100000500,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100000525','projected_crs','IAU_2015',100000525,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100000530,'Borrelly (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',100000500,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100000530','projected_crs','IAU_2015',100000530,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100000535,'Borrelly (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',100000500,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100000535','projected_crs','IAU_2015',100000535,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100000540,'Borrelly (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',100000500,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100000540','projected_crs','IAU_2015',100000540,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100000545,'Borrelly (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',100000500,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100000545','projected_crs','IAU_2015',100000545,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100000550,'Borrelly (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',100000500,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100000550','projected_crs','IAU_2015',100000550,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100000555,'Borrelly (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',100000500,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100000555','projected_crs','IAU_2015',100000555,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100000560,'Borrelly (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',100000500,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100000560','projected_crs','IAU_2015',100000560,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100000565,'Borrelly (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',100000500,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100000565','projected_crs','IAU_2015',100000565,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100000570,'Borrelly (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',100000500,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100000570','projected_crs','IAU_2015',100000570,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100000575,'Borrelly (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',100000500,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100000575','projected_crs','IAU_2015',100000575,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100000580,'Borrelly (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',100000500,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100000580','projected_crs','IAU_2015',100000580,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100000585,'Borrelly (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',100000500,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100000585','projected_crs','IAU_2015',100000585,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100000590,'Borrelly (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',100000500,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100000590','projected_crs','IAU_2015',100000590,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 1000012, 'Churyumov-Gerasimenko', 1650.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 100001200, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',100001200,'Churyumov-Gerasimenko (2015) - Sphere',NULL,'IAU_2015',1000012,1650.000000,'EPSG','9001',NULL,1650.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',100001200,'Churyumov-Gerasimenko (2015) - Sphere','','IAU_2015',100001200,'IAU_2015',100001200,NULL,NULL,NULL,'a large boulder called Cheops: 0.0',NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_100001200','geodetic_datum','IAU_2015',100001200,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',100001200,'Churyumov-Gerasimenko (2015) - Sphere / Ocentric','Use mean radius as sphere radius for interoperability. Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',100001200,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_100001200','geodetic_crs','IAU_2015',100001200,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100001210,'Churyumov-Gerasimenko (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',100001200,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100001210','projected_crs','IAU_2015',100001210,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100001215,'Churyumov-Gerasimenko (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',100001200,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100001215','projected_crs','IAU_2015',100001215,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100001220,'Churyumov-Gerasimenko (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',100001200,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100001220','projected_crs','IAU_2015',100001220,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100001225,'Churyumov-Gerasimenko (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',100001200,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100001225','projected_crs','IAU_2015',100001225,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100001230,'Churyumov-Gerasimenko (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',100001200,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100001230','projected_crs','IAU_2015',100001230,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100001235,'Churyumov-Gerasimenko (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',100001200,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100001235','projected_crs','IAU_2015',100001235,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100001240,'Churyumov-Gerasimenko (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',100001200,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100001240','projected_crs','IAU_2015',100001240,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100001245,'Churyumov-Gerasimenko (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',100001200,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100001245','projected_crs','IAU_2015',100001245,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100001250,'Churyumov-Gerasimenko (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',100001200,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100001250','projected_crs','IAU_2015',100001250,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100001255,'Churyumov-Gerasimenko (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',100001200,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100001255','projected_crs','IAU_2015',100001255,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100001260,'Churyumov-Gerasimenko (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',100001200,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100001260','projected_crs','IAU_2015',100001260,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100001265,'Churyumov-Gerasimenko (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',100001200,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100001265','projected_crs','IAU_2015',100001265,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100001270,'Churyumov-Gerasimenko (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',100001200,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100001270','projected_crs','IAU_2015',100001270,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100001275,'Churyumov-Gerasimenko (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',100001200,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100001275','projected_crs','IAU_2015',100001275,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100001280,'Churyumov-Gerasimenko (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',100001200,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100001280','projected_crs','IAU_2015',100001280,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100001285,'Churyumov-Gerasimenko (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',100001200,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100001285','projected_crs','IAU_2015',100001285,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100001290,'Churyumov-Gerasimenko (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',100001200,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100001290','projected_crs','IAU_2015',100001290,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 1000036, 'Halley', 8000.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 100003600, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',100003600,'Halley (2015) - Sphere',NULL,'IAU_2015',1000036,8000.000000,'EPSG','9001',NULL,8000.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',100003600,'Halley (2015) - Sphere','','IAU_2015',100003600,'IAU_2015',100003600,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_100003600','geodetic_datum','IAU_2015',100003600,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',100003600,'Halley (2015) - Sphere / Ocentric','Use semi-major radius as sphere for interoperability. Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',100003600,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_100003600','geodetic_crs','IAU_2015',100003600,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100003610,'Halley (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',100003600,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100003610','projected_crs','IAU_2015',100003610,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100003615,'Halley (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',100003600,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100003615','projected_crs','IAU_2015',100003615,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100003620,'Halley (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',100003600,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100003620','projected_crs','IAU_2015',100003620,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100003625,'Halley (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',100003600,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100003625','projected_crs','IAU_2015',100003625,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100003630,'Halley (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',100003600,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100003630','projected_crs','IAU_2015',100003630,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100003635,'Halley (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',100003600,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100003635','projected_crs','IAU_2015',100003635,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100003640,'Halley (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',100003600,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100003640','projected_crs','IAU_2015',100003640,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100003645,'Halley (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',100003600,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100003645','projected_crs','IAU_2015',100003645,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100003650,'Halley (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',100003600,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100003650','projected_crs','IAU_2015',100003650,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100003655,'Halley (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',100003600,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100003655','projected_crs','IAU_2015',100003655,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100003660,'Halley (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',100003600,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100003660','projected_crs','IAU_2015',100003660,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100003665,'Halley (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',100003600,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100003665','projected_crs','IAU_2015',100003665,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100003670,'Halley (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',100003600,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100003670','projected_crs','IAU_2015',100003670,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100003675,'Halley (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',100003600,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100003675','projected_crs','IAU_2015',100003675,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100003680,'Halley (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',100003600,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100003680','projected_crs','IAU_2015',100003680,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100003685,'Halley (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',100003600,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100003685','projected_crs','IAU_2015',100003685,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100003690,'Halley (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',100003600,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100003690','projected_crs','IAU_2015',100003690,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO ellipsoid VALUES('IAU_2015',100003601,'Halley (2015)',NULL,'IAU_2015',1000036,8000.000000,'EPSG','9001',NULL,4000.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',100003601,'Halley (2015)','','IAU_2015',100003601,'IAU_2015',100003600,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_100003601','geodetic_datum','IAU_2015',100003601,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',100003601,'Halley (2015) / Ographic','Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',100003601,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_100003601','geodetic_crs','IAU_2015',100003601,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100003611,'Halley (2015) / Ographic / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',100003601,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100003611','projected_crs','IAU_2015',100003611,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100003616,'Halley (2015) / Ographic / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',100003601,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100003616','projected_crs','IAU_2015',100003616,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100003621,'Halley (2015) / Ographic / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',100003601,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100003621','projected_crs','IAU_2015',100003621,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100003626,'Halley (2015) / Ographic / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',100003601,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100003626','projected_crs','IAU_2015',100003626,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100003631,'Halley (2015) / Ographic / North Polar',NULL,'EPSG','4469','IAU_2015',100003601,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100003631','projected_crs','IAU_2015',100003631,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100003636,'Halley (2015) / Ographic / South Polar',NULL,'EPSG','4490','IAU_2015',100003601,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100003636','projected_crs','IAU_2015',100003636,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100003641,'Halley (2015) / Ographic / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',100003601,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100003641','projected_crs','IAU_2015',100003641,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100003646,'Halley (2015) / Ographic / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',100003601,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100003646','projected_crs','IAU_2015',100003646,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100003651,'Halley (2015) / Ographic / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',100003601,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100003651','projected_crs','IAU_2015',100003651,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100003656,'Halley (2015) / Ographic / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',100003601,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100003656','projected_crs','IAU_2015',100003656,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100003661,'Halley (2015) / Ographic / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',100003601,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100003661','projected_crs','IAU_2015',100003661,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100003666,'Halley (2015) / Ographic / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',100003601,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100003666','projected_crs','IAU_2015',100003666,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100003671,'Halley (2015) / Ographic / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',100003601,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100003671','projected_crs','IAU_2015',100003671,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100003676,'Halley (2015) / Ographic / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',100003601,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100003676','projected_crs','IAU_2015',100003676,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100003681,'Halley (2015) / Ographic / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',100003601,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100003681','projected_crs','IAU_2015',100003681,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100003686,'Halley (2015) / Ographic / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',100003601,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100003686','projected_crs','IAU_2015',100003686,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100003691,'Halley (2015) / Ographic / Mercator',NULL,'EPSG','4400','IAU_2015',100003601,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100003691','projected_crs','IAU_2015',100003691,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',100003602,'Halley (2015) / Ocentric','Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','other','PROJ','OCENTRIC_LAT_LON','IAU_2015',100003601,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_100003602','geodetic_crs','IAU_2015',100003602,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
-- Hartley 2 has inconsistent values: semimajor < axisb or axis < semiminor. Only consider its mean value.
INSERT INTO celestial_body VALUES('IAU_2015', 1000041, 'Hartley 2', 580.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 100004100, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',100004100,'Hartley 2 (2015) - Sphere',NULL,'IAU_2015',1000041,580.000000,'EPSG','9001',NULL,580.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',100004100,'Hartley 2 (2015) - Sphere','','IAU_2015',100004100,'IAU_2015',100004100,NULL,NULL,NULL,'An isolated large mount on the waist near the large lobe: 0.0',NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_100004100','geodetic_datum','IAU_2015',100004100,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',100004100,'Hartley 2 (2015) - Sphere / Ocentric','Use mean radius as sphere radius for interoperability. Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',100004100,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_100004100','geodetic_crs','IAU_2015',100004100,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100004110,'Hartley 2 (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',100004100,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100004110','projected_crs','IAU_2015',100004110,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100004115,'Hartley 2 (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',100004100,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100004115','projected_crs','IAU_2015',100004115,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100004120,'Hartley 2 (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',100004100,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100004120','projected_crs','IAU_2015',100004120,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100004125,'Hartley 2 (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',100004100,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100004125','projected_crs','IAU_2015',100004125,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100004130,'Hartley 2 (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',100004100,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100004130','projected_crs','IAU_2015',100004130,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100004135,'Hartley 2 (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',100004100,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100004135','projected_crs','IAU_2015',100004135,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100004140,'Hartley 2 (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',100004100,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100004140','projected_crs','IAU_2015',100004140,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100004145,'Hartley 2 (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',100004100,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100004145','projected_crs','IAU_2015',100004145,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100004150,'Hartley 2 (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',100004100,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100004150','projected_crs','IAU_2015',100004150,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100004155,'Hartley 2 (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',100004100,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100004155','projected_crs','IAU_2015',100004155,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100004160,'Hartley 2 (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',100004100,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100004160','projected_crs','IAU_2015',100004160,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100004165,'Hartley 2 (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',100004100,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100004165','projected_crs','IAU_2015',100004165,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100004170,'Hartley 2 (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',100004100,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100004170','projected_crs','IAU_2015',100004170,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100004175,'Hartley 2 (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',100004100,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100004175','projected_crs','IAU_2015',100004175,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100004180,'Hartley 2 (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',100004100,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100004180','projected_crs','IAU_2015',100004180,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100004185,'Hartley 2 (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',100004100,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100004185','projected_crs','IAU_2015',100004185,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100004190,'Hartley 2 (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',100004100,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100004190','projected_crs','IAU_2015',100004190,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
-- Tempel 1 lacks value for semimajor and/or axisb and/or semiminor. Only consider its mean value.
INSERT INTO celestial_body VALUES('IAU_2015', 1000093, 'Tempel 1', 3000.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 100009300, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',100009300,'Tempel 1 (2015) - Sphere',NULL,'IAU_2015',1000093,3000.000000,'EPSG','9001',NULL,3000.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',100009300,'Tempel 1 (2015) - Sphere','','IAU_2015',100009300,'IAU_2015',100009300,NULL,NULL,NULL,'A 350 m diameter unnamed circular feature near the Deep Impactor impact site: 0.0',NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_100009300','geodetic_datum','IAU_2015',100009300,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',100009300,'Tempel 1 (2015) - Sphere / Ocentric','Use mean radius as sphere radius for interoperability. Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',100009300,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_100009300','geodetic_crs','IAU_2015',100009300,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100009310,'Tempel 1 (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',100009300,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100009310','projected_crs','IAU_2015',100009310,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100009315,'Tempel 1 (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',100009300,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100009315','projected_crs','IAU_2015',100009315,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100009320,'Tempel 1 (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',100009300,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100009320','projected_crs','IAU_2015',100009320,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100009325,'Tempel 1 (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',100009300,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100009325','projected_crs','IAU_2015',100009325,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100009330,'Tempel 1 (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',100009300,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100009330','projected_crs','IAU_2015',100009330,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100009335,'Tempel 1 (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',100009300,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100009335','projected_crs','IAU_2015',100009335,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100009340,'Tempel 1 (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',100009300,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100009340','projected_crs','IAU_2015',100009340,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100009345,'Tempel 1 (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',100009300,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100009345','projected_crs','IAU_2015',100009345,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100009350,'Tempel 1 (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',100009300,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100009350','projected_crs','IAU_2015',100009350,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100009355,'Tempel 1 (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',100009300,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100009355','projected_crs','IAU_2015',100009355,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100009360,'Tempel 1 (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',100009300,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100009360','projected_crs','IAU_2015',100009360,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100009365,'Tempel 1 (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',100009300,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100009365','projected_crs','IAU_2015',100009365,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100009370,'Tempel 1 (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',100009300,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100009370','projected_crs','IAU_2015',100009370,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100009375,'Tempel 1 (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',100009300,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100009375','projected_crs','IAU_2015',100009375,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100009380,'Tempel 1 (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',100009300,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100009380','projected_crs','IAU_2015',100009380,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100009385,'Tempel 1 (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',100009300,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100009385','projected_crs','IAU_2015',100009385,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100009390,'Tempel 1 (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',100009300,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100009390','projected_crs','IAU_2015',100009390,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 1000107, 'Wild 2', 1975.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 100010700, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',100010700,'Wild 2 (2015) - Sphere',NULL,'IAU_2015',1000107,1975.000000,'EPSG','9001',NULL,1975.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',100010700,'Wild 2 (2015) - Sphere','','IAU_2015',100010700,'IAU_2015',100010700,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_100010700','geodetic_datum','IAU_2015',100010700,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',100010700,'Wild 2 (2015) - Sphere / Ocentric','Use mean radius as sphere radius for interoperability. Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',100010700,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_100010700','geodetic_crs','IAU_2015',100010700,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100010710,'Wild 2 (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',100010700,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100010710','projected_crs','IAU_2015',100010710,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100010715,'Wild 2 (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',100010700,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100010715','projected_crs','IAU_2015',100010715,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100010720,'Wild 2 (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',100010700,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100010720','projected_crs','IAU_2015',100010720,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100010725,'Wild 2 (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',100010700,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100010725','projected_crs','IAU_2015',100010725,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100010730,'Wild 2 (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',100010700,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100010730','projected_crs','IAU_2015',100010730,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100010735,'Wild 2 (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',100010700,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100010735','projected_crs','IAU_2015',100010735,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100010740,'Wild 2 (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',100010700,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100010740','projected_crs','IAU_2015',100010740,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100010745,'Wild 2 (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',100010700,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100010745','projected_crs','IAU_2015',100010745,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100010750,'Wild 2 (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',100010700,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100010750','projected_crs','IAU_2015',100010750,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100010755,'Wild 2 (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',100010700,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100010755','projected_crs','IAU_2015',100010755,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100010760,'Wild 2 (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',100010700,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100010760','projected_crs','IAU_2015',100010760,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100010765,'Wild 2 (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',100010700,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100010765','projected_crs','IAU_2015',100010765,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100010770,'Wild 2 (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',100010700,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100010770','projected_crs','IAU_2015',100010770,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100010775,'Wild 2 (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',100010700,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100010775','projected_crs','IAU_2015',100010775,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100010780,'Wild 2 (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',100010700,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100010780','projected_crs','IAU_2015',100010780,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100010785,'Wild 2 (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',100010700,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100010785','projected_crs','IAU_2015',100010785,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',100010790,'Wild 2 (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',100010700,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_100010790','projected_crs','IAU_2015',100010790,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 9511010, 'Gaspra', 6100.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 951101000, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',951101000,'Gaspra (2015) - Sphere',NULL,'IAU_2015',9511010,6100.000000,'EPSG','9001',NULL,6100.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',951101000,'Gaspra (2015) - Sphere','','IAU_2015',951101000,'IAU_2015',951101000,NULL,NULL,NULL,'Charax: 0.0',NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_951101000','geodetic_datum','IAU_2015',951101000,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',951101000,'Gaspra (2015) - Sphere / Ocentric','Use mean radius as sphere radius for interoperability. Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',951101000,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_951101000','geodetic_crs','IAU_2015',951101000,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',951101010,'Gaspra (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',951101000,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_951101010','projected_crs','IAU_2015',951101010,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',951101015,'Gaspra (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',951101000,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_951101015','projected_crs','IAU_2015',951101015,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',951101020,'Gaspra (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',951101000,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_951101020','projected_crs','IAU_2015',951101020,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',951101025,'Gaspra (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',951101000,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_951101025','projected_crs','IAU_2015',951101025,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',951101030,'Gaspra (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',951101000,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_951101030','projected_crs','IAU_2015',951101030,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',951101035,'Gaspra (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',951101000,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_951101035','projected_crs','IAU_2015',951101035,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',951101040,'Gaspra (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',951101000,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_951101040','projected_crs','IAU_2015',951101040,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',951101045,'Gaspra (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',951101000,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_951101045','projected_crs','IAU_2015',951101045,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',951101050,'Gaspra (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',951101000,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_951101050','projected_crs','IAU_2015',951101050,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',951101055,'Gaspra (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',951101000,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_951101055','projected_crs','IAU_2015',951101055,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',951101060,'Gaspra (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',951101000,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_951101060','projected_crs','IAU_2015',951101060,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',951101065,'Gaspra (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',951101000,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_951101065','projected_crs','IAU_2015',951101065,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',951101070,'Gaspra (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',951101000,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_951101070','projected_crs','IAU_2015',951101070,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',951101075,'Gaspra (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',951101000,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_951101075','projected_crs','IAU_2015',951101075,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',951101080,'Gaspra (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',951101000,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_951101080','projected_crs','IAU_2015',951101080,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',951101085,'Gaspra (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',951101000,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_951101085','projected_crs','IAU_2015',951101085,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',951101090,'Gaspra (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',951101000,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_951101090','projected_crs','IAU_2015',951101090,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 2431010, 'Ida', 15650.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 243101000, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',243101000,'Ida (2015) - Sphere',NULL,'IAU_2015',2431010,15650.000000,'EPSG','9001',NULL,15650.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',243101000,'Ida (2015) - Sphere','','IAU_2015',243101000,'IAU_2015',243101000,NULL,NULL,NULL,'Afon: 0.0',NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_243101000','geodetic_datum','IAU_2015',243101000,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',243101000,'Ida (2015) - Sphere / Ocentric','Use mean radius as sphere radius for interoperability. Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',243101000,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_243101000','geodetic_crs','IAU_2015',243101000,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',243101010,'Ida (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',243101000,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_243101010','projected_crs','IAU_2015',243101010,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',243101015,'Ida (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',243101000,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_243101015','projected_crs','IAU_2015',243101015,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',243101020,'Ida (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',243101000,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_243101020','projected_crs','IAU_2015',243101020,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',243101025,'Ida (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',243101000,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_243101025','projected_crs','IAU_2015',243101025,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',243101030,'Ida (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',243101000,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_243101030','projected_crs','IAU_2015',243101030,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',243101035,'Ida (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',243101000,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_243101035','projected_crs','IAU_2015',243101035,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',243101040,'Ida (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',243101000,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_243101040','projected_crs','IAU_2015',243101040,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',243101045,'Ida (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',243101000,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_243101045','projected_crs','IAU_2015',243101045,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',243101050,'Ida (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',243101000,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_243101050','projected_crs','IAU_2015',243101050,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',243101055,'Ida (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',243101000,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_243101055','projected_crs','IAU_2015',243101055,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',243101060,'Ida (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',243101000,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_243101060','projected_crs','IAU_2015',243101060,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',243101065,'Ida (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',243101000,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_243101065','projected_crs','IAU_2015',243101065,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',243101070,'Ida (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',243101000,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_243101070','projected_crs','IAU_2015',243101070,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',243101075,'Ida (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',243101000,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_243101075','projected_crs','IAU_2015',243101075,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',243101080,'Ida (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',243101000,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_243101080','projected_crs','IAU_2015',243101080,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',243101085,'Ida (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',243101000,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_243101085','projected_crs','IAU_2015',243101085,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',243101090,'Ida (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',243101000,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_243101090','projected_crs','IAU_2015',243101090,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
-- Skip Dactyl has it lacks all axis information
INSERT INTO celestial_body VALUES('IAU_2015', 2000001, 'Ceres', 487300.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 200000100, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',200000100,'Ceres (2015) - Sphere',NULL,'IAU_2015',2000001,487300.000000,'EPSG','9001',NULL,487300.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',200000100,'Ceres (2015) - Sphere','','IAU_2015',200000100,'IAU_2015',200000100,NULL,NULL,NULL,'Kait: 0.0',NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_200000100','geodetic_datum','IAU_2015',200000100,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',200000100,'Ceres (2015) - Sphere / Ocentric','Use semi-major radius as sphere for interoperability. Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',200000100,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_200000100','geodetic_crs','IAU_2015',200000100,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200000110,'Ceres (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',200000100,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200000110','projected_crs','IAU_2015',200000110,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200000115,'Ceres (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',200000100,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200000115','projected_crs','IAU_2015',200000115,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200000120,'Ceres (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',200000100,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200000120','projected_crs','IAU_2015',200000120,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200000125,'Ceres (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',200000100,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200000125','projected_crs','IAU_2015',200000125,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200000130,'Ceres (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',200000100,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200000130','projected_crs','IAU_2015',200000130,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200000135,'Ceres (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',200000100,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200000135','projected_crs','IAU_2015',200000135,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200000140,'Ceres (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',200000100,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200000140','projected_crs','IAU_2015',200000140,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200000145,'Ceres (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',200000100,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200000145','projected_crs','IAU_2015',200000145,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200000150,'Ceres (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',200000100,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200000150','projected_crs','IAU_2015',200000150,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200000155,'Ceres (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',200000100,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200000155','projected_crs','IAU_2015',200000155,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200000160,'Ceres (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',200000100,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200000160','projected_crs','IAU_2015',200000160,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200000165,'Ceres (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',200000100,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200000165','projected_crs','IAU_2015',200000165,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200000170,'Ceres (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',200000100,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200000170','projected_crs','IAU_2015',200000170,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200000175,'Ceres (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',200000100,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200000175','projected_crs','IAU_2015',200000175,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200000180,'Ceres (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',200000100,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200000180','projected_crs','IAU_2015',200000180,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200000185,'Ceres (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',200000100,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200000185','projected_crs','IAU_2015',200000185,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200000190,'Ceres (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',200000100,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200000190','projected_crs','IAU_2015',200000190,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO ellipsoid VALUES('IAU_2015',200000101,'Ceres (2015)',NULL,'IAU_2015',2000001,487300.000000,'EPSG','9001',NULL,446000.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',200000101,'Ceres (2015)','','IAU_2015',200000101,'IAU_2015',200000100,NULL,NULL,NULL,'Kait: 0.0',NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_200000101','geodetic_datum','IAU_2015',200000101,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',200000101,'Ceres (2015) / Ographic','Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',200000101,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_200000101','geodetic_crs','IAU_2015',200000101,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200000111,'Ceres (2015) / Ographic / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',200000101,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200000111','projected_crs','IAU_2015',200000111,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200000116,'Ceres (2015) / Ographic / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',200000101,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200000116','projected_crs','IAU_2015',200000116,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200000121,'Ceres (2015) / Ographic / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',200000101,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200000121','projected_crs','IAU_2015',200000121,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200000126,'Ceres (2015) / Ographic / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',200000101,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200000126','projected_crs','IAU_2015',200000126,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200000131,'Ceres (2015) / Ographic / North Polar',NULL,'EPSG','4469','IAU_2015',200000101,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200000131','projected_crs','IAU_2015',200000131,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200000136,'Ceres (2015) / Ographic / South Polar',NULL,'EPSG','4490','IAU_2015',200000101,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200000136','projected_crs','IAU_2015',200000136,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200000141,'Ceres (2015) / Ographic / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',200000101,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200000141','projected_crs','IAU_2015',200000141,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200000146,'Ceres (2015) / Ographic / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',200000101,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200000146','projected_crs','IAU_2015',200000146,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200000151,'Ceres (2015) / Ographic / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',200000101,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200000151','projected_crs','IAU_2015',200000151,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200000156,'Ceres (2015) / Ographic / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',200000101,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200000156','projected_crs','IAU_2015',200000156,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200000161,'Ceres (2015) / Ographic / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',200000101,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200000161','projected_crs','IAU_2015',200000161,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200000166,'Ceres (2015) / Ographic / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',200000101,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200000166','projected_crs','IAU_2015',200000166,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200000171,'Ceres (2015) / Ographic / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',200000101,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200000171','projected_crs','IAU_2015',200000171,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200000176,'Ceres (2015) / Ographic / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',200000101,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200000176','projected_crs','IAU_2015',200000176,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200000181,'Ceres (2015) / Ographic / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',200000101,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200000181','projected_crs','IAU_2015',200000181,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200000186,'Ceres (2015) / Ographic / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',200000101,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200000186','projected_crs','IAU_2015',200000186,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200000191,'Ceres (2015) / Ographic / Mercator',NULL,'EPSG','4400','IAU_2015',200000101,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200000191','projected_crs','IAU_2015',200000191,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',200000102,'Ceres (2015) / Ocentric','Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','other','PROJ','OCENTRIC_LAT_LON','IAU_2015',200000101,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_200000102','geodetic_crs','IAU_2015',200000102,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
-- Skip Pallas has it lacks all axis information
INSERT INTO celestial_body VALUES('IAU_2015', 2000004, 'Vesta', 255000.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 200000400, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',200000400,'Vesta (2015) - Sphere',NULL,'IAU_2015',2000004,255000.000000,'EPSG','9001',NULL,255000.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',200000400,'Vesta (2015) - Sphere','','IAU_2015',200000400,'IAU_2015',200000400,NULL,NULL,NULL,'Claudia: 146.0',NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_200000400','geodetic_datum','IAU_2015',200000400,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',200000400,'Vesta (2015) - Sphere / Ocentric','Use mean radius as sphere radius for interoperability. Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',200000400,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_200000400','geodetic_crs','IAU_2015',200000400,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200000410,'Vesta (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',200000400,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200000410','projected_crs','IAU_2015',200000410,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200000415,'Vesta (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',200000400,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200000415','projected_crs','IAU_2015',200000415,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200000420,'Vesta (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',200000400,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200000420','projected_crs','IAU_2015',200000420,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200000425,'Vesta (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',200000400,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200000425','projected_crs','IAU_2015',200000425,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200000430,'Vesta (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',200000400,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200000430','projected_crs','IAU_2015',200000430,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200000435,'Vesta (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',200000400,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200000435','projected_crs','IAU_2015',200000435,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200000440,'Vesta (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',200000400,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200000440','projected_crs','IAU_2015',200000440,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200000445,'Vesta (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',200000400,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200000445','projected_crs','IAU_2015',200000445,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200000450,'Vesta (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',200000400,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200000450','projected_crs','IAU_2015',200000450,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200000455,'Vesta (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',200000400,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200000455','projected_crs','IAU_2015',200000455,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200000460,'Vesta (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',200000400,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200000460','projected_crs','IAU_2015',200000460,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200000465,'Vesta (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',200000400,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200000465','projected_crs','IAU_2015',200000465,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200000470,'Vesta (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',200000400,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200000470','projected_crs','IAU_2015',200000470,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200000475,'Vesta (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',200000400,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200000475','projected_crs','IAU_2015',200000475,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200000480,'Vesta (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',200000400,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200000480','projected_crs','IAU_2015',200000480,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200000485,'Vesta (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',200000400,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200000485','projected_crs','IAU_2015',200000485,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200000490,'Vesta (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',200000400,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200000490','projected_crs','IAU_2015',200000490,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 2000016, 'Psyche', 113000.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 200001600, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',200001600,'Psyche (2015) - Sphere',NULL,'IAU_2015',2000016,113000.000000,'EPSG','9001',NULL,113000.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',200001600,'Psyche (2015) - Sphere','','IAU_2015',200001600,'IAU_2015',200001600,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_200001600','geodetic_datum','IAU_2015',200001600,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',200001600,'Psyche (2015) - Sphere / Ocentric','Use mean radius as sphere radius for interoperability. Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',200001600,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_200001600','geodetic_crs','IAU_2015',200001600,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200001610,'Psyche (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',200001600,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200001610','projected_crs','IAU_2015',200001610,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200001615,'Psyche (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',200001600,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200001615','projected_crs','IAU_2015',200001615,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200001620,'Psyche (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',200001600,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200001620','projected_crs','IAU_2015',200001620,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200001625,'Psyche (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',200001600,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200001625','projected_crs','IAU_2015',200001625,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200001630,'Psyche (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',200001600,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200001630','projected_crs','IAU_2015',200001630,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200001635,'Psyche (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',200001600,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200001635','projected_crs','IAU_2015',200001635,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200001640,'Psyche (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',200001600,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200001640','projected_crs','IAU_2015',200001640,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200001645,'Psyche (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',200001600,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200001645','projected_crs','IAU_2015',200001645,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200001650,'Psyche (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',200001600,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200001650','projected_crs','IAU_2015',200001650,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200001655,'Psyche (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',200001600,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200001655','projected_crs','IAU_2015',200001655,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200001660,'Psyche (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',200001600,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200001660','projected_crs','IAU_2015',200001660,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200001665,'Psyche (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',200001600,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200001665','projected_crs','IAU_2015',200001665,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200001670,'Psyche (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',200001600,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200001670','projected_crs','IAU_2015',200001670,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200001675,'Psyche (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',200001600,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200001675','projected_crs','IAU_2015',200001675,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200001680,'Psyche (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',200001600,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200001680','projected_crs','IAU_2015',200001680,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200001685,'Psyche (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',200001600,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200001685','projected_crs','IAU_2015',200001685,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200001690,'Psyche (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',200001600,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200001690','projected_crs','IAU_2015',200001690,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 2000021, 'Lutetia', 52500.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 200002100, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',200002100,'Lutetia (2015) - Sphere',NULL,'IAU_2015',2000021,52500.000000,'EPSG','9001',NULL,52500.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',200002100,'Lutetia (2015) - Sphere','','IAU_2015',200002100,'IAU_2015',200002100,NULL,NULL,NULL,'Arbitrarily defined based on light curve information: 0.0',NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_200002100','geodetic_datum','IAU_2015',200002100,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',200002100,'Lutetia (2015) - Sphere / Ocentric','Use mean radius as sphere radius for interoperability. Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',200002100,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_200002100','geodetic_crs','IAU_2015',200002100,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200002110,'Lutetia (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',200002100,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200002110','projected_crs','IAU_2015',200002110,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200002115,'Lutetia (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',200002100,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200002115','projected_crs','IAU_2015',200002115,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200002120,'Lutetia (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',200002100,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200002120','projected_crs','IAU_2015',200002120,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200002125,'Lutetia (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',200002100,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200002125','projected_crs','IAU_2015',200002125,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200002130,'Lutetia (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',200002100,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200002130','projected_crs','IAU_2015',200002130,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200002135,'Lutetia (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',200002100,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200002135','projected_crs','IAU_2015',200002135,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200002140,'Lutetia (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',200002100,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200002140','projected_crs','IAU_2015',200002140,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200002145,'Lutetia (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',200002100,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200002145','projected_crs','IAU_2015',200002145,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200002150,'Lutetia (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',200002100,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200002150','projected_crs','IAU_2015',200002150,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200002155,'Lutetia (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',200002100,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200002155','projected_crs','IAU_2015',200002155,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200002160,'Lutetia (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',200002100,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200002160','projected_crs','IAU_2015',200002160,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200002165,'Lutetia (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',200002100,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200002165','projected_crs','IAU_2015',200002165,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200002170,'Lutetia (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',200002100,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200002170','projected_crs','IAU_2015',200002170,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200002175,'Lutetia (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',200002100,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200002175','projected_crs','IAU_2015',200002175,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200002180,'Lutetia (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',200002100,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200002180','projected_crs','IAU_2015',200002180,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200002185,'Lutetia (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',200002100,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200002185','projected_crs','IAU_2015',200002185,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200002190,'Lutetia (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',200002100,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200002190','projected_crs','IAU_2015',200002190,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 2000052, '52 Europa', 157500.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 200005200, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',200005200,'52 Europa (2015) - Sphere',NULL,'IAU_2015',2000052,157500.000000,'EPSG','9001',NULL,157500.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',200005200,'52 Europa (2015) - Sphere','','IAU_2015',200005200,'IAU_2015',200005200,NULL,NULL,NULL,'long axis that pointed toward the Earth on 2007 May28 8.3125 UT (light-time corrected): 0.0',NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_200005200','geodetic_datum','IAU_2015',200005200,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',200005200,'52 Europa (2015) - Sphere / Ocentric','Use mean radius as sphere radius for interoperability. Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',200005200,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_200005200','geodetic_crs','IAU_2015',200005200,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200005210,'52 Europa (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',200005200,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200005210','projected_crs','IAU_2015',200005210,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200005215,'52 Europa (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',200005200,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200005215','projected_crs','IAU_2015',200005215,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200005220,'52 Europa (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',200005200,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200005220','projected_crs','IAU_2015',200005220,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200005225,'52 Europa (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',200005200,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200005225','projected_crs','IAU_2015',200005225,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200005230,'52 Europa (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',200005200,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200005230','projected_crs','IAU_2015',200005230,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200005235,'52 Europa (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',200005200,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200005235','projected_crs','IAU_2015',200005235,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200005240,'52 Europa (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',200005200,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200005240','projected_crs','IAU_2015',200005240,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200005245,'52 Europa (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',200005200,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200005245','projected_crs','IAU_2015',200005245,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200005250,'52 Europa (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',200005200,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200005250','projected_crs','IAU_2015',200005250,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200005255,'52 Europa (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',200005200,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200005255','projected_crs','IAU_2015',200005255,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200005260,'52 Europa (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',200005200,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200005260','projected_crs','IAU_2015',200005260,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200005265,'52 Europa (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',200005200,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200005265','projected_crs','IAU_2015',200005265,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200005270,'52 Europa (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',200005200,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200005270','projected_crs','IAU_2015',200005270,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200005275,'52 Europa (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',200005200,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200005275','projected_crs','IAU_2015',200005275,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200005280,'52 Europa (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',200005200,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200005280','projected_crs','IAU_2015',200005280,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200005285,'52 Europa (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',200005200,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200005285','projected_crs','IAU_2015',200005285,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200005290,'52 Europa (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',200005200,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200005290','projected_crs','IAU_2015',200005290,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 2000216, 'Kleopatra', 65333.333333);
INSERT INTO prime_meridian VALUES('IAU_2015', 200021600, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',200021600,'Kleopatra (2015) - Sphere',NULL,'IAU_2015',2000216,65333.333333,'EPSG','9001',NULL,65333.333333,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',200021600,'Kleopatra (2015) - Sphere','','IAU_2015',200021600,'IAU_2015',200021600,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_200021600','geodetic_datum','IAU_2015',200021600,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',200021600,'Kleopatra (2015) - Sphere / Ocentric','Use R_m = (a+b+c)/3 as mean radius. Use mean radius as sphere radius for interoperability. Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',200021600,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_200021600','geodetic_crs','IAU_2015',200021600,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200021610,'Kleopatra (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',200021600,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200021610','projected_crs','IAU_2015',200021610,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200021615,'Kleopatra (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',200021600,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200021615','projected_crs','IAU_2015',200021615,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200021620,'Kleopatra (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',200021600,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200021620','projected_crs','IAU_2015',200021620,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200021625,'Kleopatra (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',200021600,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200021625','projected_crs','IAU_2015',200021625,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200021630,'Kleopatra (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',200021600,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200021630','projected_crs','IAU_2015',200021630,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200021635,'Kleopatra (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',200021600,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200021635','projected_crs','IAU_2015',200021635,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200021640,'Kleopatra (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',200021600,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200021640','projected_crs','IAU_2015',200021640,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200021645,'Kleopatra (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',200021600,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200021645','projected_crs','IAU_2015',200021645,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200021650,'Kleopatra (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',200021600,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200021650','projected_crs','IAU_2015',200021650,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200021655,'Kleopatra (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',200021600,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200021655','projected_crs','IAU_2015',200021655,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200021660,'Kleopatra (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',200021600,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200021660','projected_crs','IAU_2015',200021660,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200021665,'Kleopatra (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',200021600,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200021665','projected_crs','IAU_2015',200021665,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200021670,'Kleopatra (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',200021600,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200021670','projected_crs','IAU_2015',200021670,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200021675,'Kleopatra (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',200021600,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200021675','projected_crs','IAU_2015',200021675,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200021680,'Kleopatra (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',200021600,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200021680','projected_crs','IAU_2015',200021680,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200021685,'Kleopatra (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',200021600,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200021685','projected_crs','IAU_2015',200021685,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200021690,'Kleopatra (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',200021600,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200021690','projected_crs','IAU_2015',200021690,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 2000433, 'Eros', 17000.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 200043300, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',200043300,'Eros (2015) - Sphere',NULL,'IAU_2015',2000433,17000.000000,'EPSG','9001',NULL,17000.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',200043300,'Eros (2015) - Sphere','','IAU_2015',200043300,'IAU_2015',200043300,NULL,NULL,NULL,'unnamed crater: 0.0',NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_200043300','geodetic_datum','IAU_2015',200043300,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',200043300,'Eros (2015) - Sphere / Ocentric','Use semi-major radius as sphere for interoperability. Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',200043300,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_200043300','geodetic_crs','IAU_2015',200043300,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200043310,'Eros (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',200043300,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200043310','projected_crs','IAU_2015',200043310,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200043315,'Eros (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',200043300,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200043315','projected_crs','IAU_2015',200043315,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200043320,'Eros (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',200043300,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200043320','projected_crs','IAU_2015',200043320,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200043325,'Eros (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',200043300,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200043325','projected_crs','IAU_2015',200043325,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200043330,'Eros (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',200043300,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200043330','projected_crs','IAU_2015',200043330,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200043335,'Eros (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',200043300,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200043335','projected_crs','IAU_2015',200043335,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200043340,'Eros (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',200043300,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200043340','projected_crs','IAU_2015',200043340,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200043345,'Eros (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',200043300,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200043345','projected_crs','IAU_2015',200043345,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200043350,'Eros (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',200043300,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200043350','projected_crs','IAU_2015',200043350,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200043355,'Eros (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',200043300,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200043355','projected_crs','IAU_2015',200043355,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200043360,'Eros (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',200043300,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200043360','projected_crs','IAU_2015',200043360,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200043365,'Eros (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',200043300,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200043365','projected_crs','IAU_2015',200043365,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200043370,'Eros (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',200043300,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200043370','projected_crs','IAU_2015',200043370,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200043375,'Eros (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',200043300,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200043375','projected_crs','IAU_2015',200043375,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200043380,'Eros (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',200043300,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200043380','projected_crs','IAU_2015',200043380,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200043385,'Eros (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',200043300,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200043385','projected_crs','IAU_2015',200043385,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200043390,'Eros (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',200043300,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200043390','projected_crs','IAU_2015',200043390,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO ellipsoid VALUES('IAU_2015',200043301,'Eros (2015)',NULL,'IAU_2015',2000433,17000.000000,'EPSG','9001',NULL,5500.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',200043301,'Eros (2015)','','IAU_2015',200043301,'IAU_2015',200043300,NULL,NULL,NULL,'unnamed crater: 0.0',NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_200043301','geodetic_datum','IAU_2015',200043301,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',200043301,'Eros (2015) / Ographic','Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',200043301,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_200043301','geodetic_crs','IAU_2015',200043301,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200043311,'Eros (2015) / Ographic / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',200043301,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200043311','projected_crs','IAU_2015',200043311,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200043316,'Eros (2015) / Ographic / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',200043301,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200043316','projected_crs','IAU_2015',200043316,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200043321,'Eros (2015) / Ographic / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',200043301,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200043321','projected_crs','IAU_2015',200043321,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200043326,'Eros (2015) / Ographic / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',200043301,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200043326','projected_crs','IAU_2015',200043326,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200043331,'Eros (2015) / Ographic / North Polar',NULL,'EPSG','4469','IAU_2015',200043301,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200043331','projected_crs','IAU_2015',200043331,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200043336,'Eros (2015) / Ographic / South Polar',NULL,'EPSG','4490','IAU_2015',200043301,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200043336','projected_crs','IAU_2015',200043336,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200043341,'Eros (2015) / Ographic / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',200043301,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200043341','projected_crs','IAU_2015',200043341,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200043346,'Eros (2015) / Ographic / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',200043301,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200043346','projected_crs','IAU_2015',200043346,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200043351,'Eros (2015) / Ographic / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',200043301,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200043351','projected_crs','IAU_2015',200043351,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200043356,'Eros (2015) / Ographic / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',200043301,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200043356','projected_crs','IAU_2015',200043356,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200043361,'Eros (2015) / Ographic / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',200043301,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200043361','projected_crs','IAU_2015',200043361,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200043366,'Eros (2015) / Ographic / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',200043301,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200043366','projected_crs','IAU_2015',200043366,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200043371,'Eros (2015) / Ographic / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',200043301,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200043371','projected_crs','IAU_2015',200043371,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200043376,'Eros (2015) / Ographic / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',200043301,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200043376','projected_crs','IAU_2015',200043376,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200043381,'Eros (2015) / Ographic / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',200043301,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200043381','projected_crs','IAU_2015',200043381,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200043386,'Eros (2015) / Ographic / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',200043301,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200043386','projected_crs','IAU_2015',200043386,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200043391,'Eros (2015) / Ographic / Mercator',NULL,'EPSG','4400','IAU_2015',200043301,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200043391','projected_crs','IAU_2015',200043391,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',200043302,'Eros (2015) / Ocentric','Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','other','PROJ','OCENTRIC_LAT_LON','IAU_2015',200043301,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_200043302','geodetic_crs','IAU_2015',200043302,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 2000511, 'Davida', 150000.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 200051100, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',200051100,'Davida (2015) - Sphere',NULL,'IAU_2015',2000511,150000.000000,'EPSG','9001',NULL,150000.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',200051100,'Davida (2015) - Sphere','','IAU_2015',200051100,'IAU_2015',200051100,NULL,NULL,NULL,'the long axis that points toward the Earth on 2002 December27 7.83 UT: 0.0',NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_200051100','geodetic_datum','IAU_2015',200051100,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',200051100,'Davida (2015) - Sphere / Ocentric','Use mean radius as sphere radius for interoperability. Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',200051100,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_200051100','geodetic_crs','IAU_2015',200051100,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200051110,'Davida (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',200051100,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200051110','projected_crs','IAU_2015',200051110,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200051115,'Davida (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',200051100,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200051115','projected_crs','IAU_2015',200051115,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200051120,'Davida (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',200051100,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200051120','projected_crs','IAU_2015',200051120,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200051125,'Davida (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',200051100,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200051125','projected_crs','IAU_2015',200051125,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200051130,'Davida (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',200051100,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200051130','projected_crs','IAU_2015',200051130,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200051135,'Davida (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',200051100,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200051135','projected_crs','IAU_2015',200051135,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200051140,'Davida (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',200051100,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200051140','projected_crs','IAU_2015',200051140,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200051145,'Davida (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',200051100,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200051145','projected_crs','IAU_2015',200051145,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200051150,'Davida (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',200051100,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200051150','projected_crs','IAU_2015',200051150,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200051155,'Davida (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',200051100,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200051155','projected_crs','IAU_2015',200051155,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200051160,'Davida (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',200051100,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200051160','projected_crs','IAU_2015',200051160,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200051165,'Davida (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',200051100,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200051165','projected_crs','IAU_2015',200051165,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200051170,'Davida (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',200051100,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200051170','projected_crs','IAU_2015',200051170,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200051175,'Davida (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',200051100,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200051175','projected_crs','IAU_2015',200051175,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200051180,'Davida (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',200051100,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200051180','projected_crs','IAU_2015',200051180,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200051185,'Davida (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',200051100,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200051185','projected_crs','IAU_2015',200051185,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200051190,'Davida (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',200051100,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200051190','projected_crs','IAU_2015',200051190,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 2000253, 'Mathilde', 26500.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 200025300, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',200025300,'Mathilde (2015) - Sphere',NULL,'IAU_2015',2000253,26500.000000,'EPSG','9001',NULL,26500.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',200025300,'Mathilde (2015) - Sphere','','IAU_2015',200025300,'IAU_2015',200025300,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_200025300','geodetic_datum','IAU_2015',200025300,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',200025300,'Mathilde (2015) - Sphere / Ocentric','Use mean radius as sphere radius for interoperability. Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',200025300,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_200025300','geodetic_crs','IAU_2015',200025300,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200025310,'Mathilde (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',200025300,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200025310','projected_crs','IAU_2015',200025310,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200025315,'Mathilde (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',200025300,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200025315','projected_crs','IAU_2015',200025315,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200025320,'Mathilde (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',200025300,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200025320','projected_crs','IAU_2015',200025320,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200025325,'Mathilde (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',200025300,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200025325','projected_crs','IAU_2015',200025325,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200025330,'Mathilde (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',200025300,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200025330','projected_crs','IAU_2015',200025330,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200025335,'Mathilde (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',200025300,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200025335','projected_crs','IAU_2015',200025335,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200025340,'Mathilde (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',200025300,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200025340','projected_crs','IAU_2015',200025340,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200025345,'Mathilde (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',200025300,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200025345','projected_crs','IAU_2015',200025345,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200025350,'Mathilde (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',200025300,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200025350','projected_crs','IAU_2015',200025350,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200025355,'Mathilde (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',200025300,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200025355','projected_crs','IAU_2015',200025355,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200025360,'Mathilde (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',200025300,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200025360','projected_crs','IAU_2015',200025360,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200025365,'Mathilde (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',200025300,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200025365','projected_crs','IAU_2015',200025365,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200025370,'Mathilde (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',200025300,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200025370','projected_crs','IAU_2015',200025370,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200025375,'Mathilde (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',200025300,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200025375','projected_crs','IAU_2015',200025375,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200025380,'Mathilde (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',200025300,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200025380','projected_crs','IAU_2015',200025380,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200025385,'Mathilde (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',200025300,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200025385','projected_crs','IAU_2015',200025385,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200025390,'Mathilde (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',200025300,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200025390','projected_crs','IAU_2015',200025390,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 2002867, 'Steins', 2700.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 200286700, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',200286700,'Steins (2015) - Sphere',NULL,'IAU_2015',2002867,2700.000000,'EPSG','9001',NULL,2700.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',200286700,'Steins (2015) - Sphere','','IAU_2015',200286700,'IAU_2015',200286700,NULL,NULL,NULL,'Topaz: 0.0',NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_200286700','geodetic_datum','IAU_2015',200286700,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',200286700,'Steins (2015) - Sphere / Ocentric','Use mean radius as sphere radius for interoperability. Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',200286700,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_200286700','geodetic_crs','IAU_2015',200286700,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200286710,'Steins (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',200286700,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200286710','projected_crs','IAU_2015',200286710,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200286715,'Steins (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',200286700,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200286715','projected_crs','IAU_2015',200286715,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200286720,'Steins (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',200286700,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200286720','projected_crs','IAU_2015',200286720,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200286725,'Steins (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',200286700,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200286725','projected_crs','IAU_2015',200286725,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200286730,'Steins (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',200286700,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200286730','projected_crs','IAU_2015',200286730,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200286735,'Steins (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',200286700,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200286735','projected_crs','IAU_2015',200286735,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200286740,'Steins (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',200286700,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200286740','projected_crs','IAU_2015',200286740,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200286745,'Steins (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',200286700,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200286745','projected_crs','IAU_2015',200286745,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200286750,'Steins (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',200286700,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200286750','projected_crs','IAU_2015',200286750,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200286755,'Steins (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',200286700,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200286755','projected_crs','IAU_2015',200286755,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200286760,'Steins (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',200286700,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200286760','projected_crs','IAU_2015',200286760,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200286765,'Steins (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',200286700,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200286765','projected_crs','IAU_2015',200286765,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200286770,'Steins (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',200286700,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200286770','projected_crs','IAU_2015',200286770,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200286775,'Steins (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',200286700,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200286775','projected_crs','IAU_2015',200286775,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200286780,'Steins (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',200286700,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200286780','projected_crs','IAU_2015',200286780,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200286785,'Steins (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',200286700,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200286785','projected_crs','IAU_2015',200286785,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200286790,'Steins (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',200286700,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200286790','projected_crs','IAU_2015',200286790,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
-- Skip 1992KD has it lacks all axis information
-- Skip Braille has it lacks all axis information
-- Skip Wilson-Harrington has it lacks all axis information
INSERT INTO celestial_body VALUES('IAU_2015', 2004179, 'Toutatis', 1331.666667);
INSERT INTO prime_meridian VALUES('IAU_2015', 200417900, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',200417900,'Toutatis (2015) - Sphere',NULL,'IAU_2015',2004179,1331.666667,'EPSG','9001',NULL,1331.666667,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',200417900,'Toutatis (2015) - Sphere','','IAU_2015',200417900,'IAU_2015',200417900,NULL,NULL,NULL,NULL,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_200417900','geodetic_datum','IAU_2015',200417900,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',200417900,'Toutatis (2015) - Sphere / Ocentric','Use R_m = (a+b+c)/3 as mean radius. Use mean radius as sphere radius for interoperability. Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',200417900,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_200417900','geodetic_crs','IAU_2015',200417900,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200417910,'Toutatis (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',200417900,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200417910','projected_crs','IAU_2015',200417910,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200417915,'Toutatis (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',200417900,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200417915','projected_crs','IAU_2015',200417915,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200417920,'Toutatis (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',200417900,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200417920','projected_crs','IAU_2015',200417920,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200417925,'Toutatis (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',200417900,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200417925','projected_crs','IAU_2015',200417925,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200417930,'Toutatis (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',200417900,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200417930','projected_crs','IAU_2015',200417930,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200417935,'Toutatis (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',200417900,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200417935','projected_crs','IAU_2015',200417935,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200417940,'Toutatis (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',200417900,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200417940','projected_crs','IAU_2015',200417940,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200417945,'Toutatis (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',200417900,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200417945','projected_crs','IAU_2015',200417945,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200417950,'Toutatis (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',200417900,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200417950','projected_crs','IAU_2015',200417950,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200417955,'Toutatis (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',200417900,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200417955','projected_crs','IAU_2015',200417955,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200417960,'Toutatis (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',200417900,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200417960','projected_crs','IAU_2015',200417960,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200417965,'Toutatis (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',200417900,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200417965','projected_crs','IAU_2015',200417965,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200417970,'Toutatis (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',200417900,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200417970','projected_crs','IAU_2015',200417970,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200417975,'Toutatis (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',200417900,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200417975','projected_crs','IAU_2015',200417975,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200417980,'Toutatis (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',200417900,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200417980','projected_crs','IAU_2015',200417980,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200417985,'Toutatis (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',200417900,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200417985','projected_crs','IAU_2015',200417985,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',200417990,'Toutatis (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',200417900,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_200417990','projected_crs','IAU_2015',200417990,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO celestial_body VALUES('IAU_2015', 2025143, 'Itokawa', 173.000000);
INSERT INTO prime_meridian VALUES('IAU_2015', 202514300, 'Reference Meridian', 0.0, 'EPSG', 9102, 0);
INSERT INTO ellipsoid VALUES('IAU_2015',202514300,'Itokawa (2015) - Sphere',NULL,'IAU_2015',2025143,173.000000,'EPSG','9001',NULL,173.000000,0);
INSERT INTO geodetic_datum VALUES('IAU_2015',202514300,'Itokawa (2015) - Sphere','','IAU_2015',202514300,'IAU_2015',202514300,NULL,NULL,NULL,'deﬁned with W0=0: 0.0',NULL,0);
INSERT INTO usage VALUES('IAU_2015','GD_202514300','geodetic_datum','IAU_2015',202514300,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO geodetic_crs VALUES('IAU_2015',202514300,'Itokawa (2015) - Sphere / Ocentric','Use R_m = (a+b+c)/3 as mean radius. Use mean radius as sphere radius for interoperability. Source of IAU Coordinate systems: https://doi.org/10.1007/s10569-017-9805-5','geographic 2D','EPSG','6422','IAU_2015',202514300,NULL,0);
INSERT INTO usage VALUES('IAU_2015','GCRS_202514300','geodetic_crs','IAU_2015',202514300,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',202514310,'Itokawa (2015) - Sphere / Ocentric / Equirectangular, clon = 0',NULL,'EPSG','4400','IAU_2015',202514300,'IAU_2015',10,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_202514310','projected_crs','IAU_2015',202514310,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',202514315,'Itokawa (2015) - Sphere / Ocentric / Equirectangular, clon = 180',NULL,'EPSG','4400','IAU_2015',202514300,'IAU_2015',15,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_202514315','projected_crs','IAU_2015',202514315,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',202514320,'Itokawa (2015) - Sphere / Ocentric / Sinusoidal, clon = 0',NULL,'EPSG','4400','IAU_2015',202514300,'IAU_2015',20,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_202514320','projected_crs','IAU_2015',202514320,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',202514325,'Itokawa (2015) - Sphere / Ocentric / Sinusoidal, clon = 180',NULL,'EPSG','4400','IAU_2015',202514300,'IAU_2015',25,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_202514325','projected_crs','IAU_2015',202514325,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',202514330,'Itokawa (2015) - Sphere / Ocentric / North Polar',NULL,'EPSG','4469','IAU_2015',202514300,'IAU_2015',30,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_202514330','projected_crs','IAU_2015',202514330,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',202514335,'Itokawa (2015) - Sphere / Ocentric / South Polar',NULL,'EPSG','4490','IAU_2015',202514300,'IAU_2015',35,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_202514335','projected_crs','IAU_2015',202514335,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',202514340,'Itokawa (2015) - Sphere / Ocentric / Mollweide, clon = 0',NULL,'EPSG','4400','IAU_2015',202514300,'IAU_2015',40,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_202514340','projected_crs','IAU_2015',202514340,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',202514345,'Itokawa (2015) - Sphere / Ocentric / Mollweide, clon = 180',NULL,'EPSG','4400','IAU_2015',202514300,'IAU_2015',45,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_202514345','projected_crs','IAU_2015',202514345,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',202514350,'Itokawa (2015) - Sphere / Ocentric / Robinson, clon = 0',NULL,'EPSG','4400','IAU_2015',202514300,'IAU_2015',50,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_202514350','projected_crs','IAU_2015',202514350,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',202514355,'Itokawa (2015) - Sphere / Ocentric / Robinson, clon = 180',NULL,'EPSG','4400','IAU_2015',202514300,'IAU_2015',55,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_202514355','projected_crs','IAU_2015',202514355,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',202514360,'Itokawa (2015) - Sphere / Ocentric / Transverse Mercator',NULL,'EPSG','4400','IAU_2015',202514300,'IAU_2015',60,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_202514360','projected_crs','IAU_2015',202514360,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',202514365,'Itokawa (2015) - Sphere / Ocentric / Orthographic, clon = 0',NULL,'EPSG','4400','IAU_2015',202514300,'IAU_2015',65,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_202514365','projected_crs','IAU_2015',202514365,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',202514370,'Itokawa (2015) - Sphere / Ocentric / Orthographic, clon = 180',NULL,'EPSG','4400','IAU_2015',202514300,'IAU_2015',70,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_202514370','projected_crs','IAU_2015',202514370,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',202514375,'Itokawa (2015) - Sphere / Ocentric / Lambert Conic Conformal',NULL,'EPSG','4400','IAU_2015',202514300,'IAU_2015',75,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_202514375','projected_crs','IAU_2015',202514375,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',202514380,'Itokawa (2015) - Sphere / Ocentric / Lambert Azimuthal Equal Area',NULL,'EPSG','4400','IAU_2015',202514300,'IAU_2015',80,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_202514380','projected_crs','IAU_2015',202514380,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',202514385,'Itokawa (2015) - Sphere / Ocentric / Albers Equal Area',NULL,'EPSG','4400','IAU_2015',202514300,'IAU_2015',85,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_202514385','projected_crs','IAU_2015',202514385,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
INSERT INTO projected_crs VALUES('IAU_2015',202514390,'Itokawa (2015) - Sphere / Ocentric / Mercator',NULL,'EPSG','4400','IAU_2015',202514300,'IAU_2015',90,NULL,0);
INSERT INTO usage VALUES('IAU_2015','PCRS_202514390','projected_crs','IAU_2015',202514390,'PROJ','EXTENT_UNKNOWN','PROJ','SCOPE_UNKNOWN');
