--- This file has been generated by scripts/build_db.py. DO NOT EDIT !

INSERT INTO "helmert_transformation" VALUES('PROJ','GR96_TO_GR96_1996','GR96 to GR96(1996)','Accuracy 0.25 m, from datum ensemble definition','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4747','EPSG','10956',0.25,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'',0);
INSERT INTO "usage" VALUES('PROJ','GR96_TO_GR96_1996_USAGE','helmert_transformation','PROJ','GR96_TO_GR96_1996','EPSG','1107','EPSG','1024');
INSERT INTO "helmert_transformation" VALUES('PROJ','GR96_TO_GR96_2021','GR96 to GR96(2021)','Accuracy 0.25 m, from datum ensemble definition','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4747','EPSG','10959',0.25,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'',0);
INSERT INTO "usage" VALUES('PROJ','GR96_TO_GR96_2021_USAGE','helmert_transformation','PROJ','GR96_TO_GR96_2021','EPSG','1107','EPSG','1024');
INSERT INTO "helmert_transformation" VALUES('PROJ','ETRS89_TO_ETRF89','ETRS89 to ETRF89','Accuracy 0.1 m, from datum ensemble definition','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4258','EPSG','9059',0.1,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'',0);
INSERT INTO "usage" VALUES('PROJ','ETRS89_TO_ETRF89_USAGE','helmert_transformation','PROJ','ETRS89_TO_ETRF89','EPSG','1298','EPSG','1024');
INSERT INTO "helmert_transformation" VALUES('PROJ','ETRS89_TO_ETRF90','ETRS89 to ETRF90','Accuracy 0.1 m, from datum ensemble definition','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4258','EPSG','9060',0.1,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'',0);
INSERT INTO "usage" VALUES('PROJ','ETRS89_TO_ETRF90_USAGE','helmert_transformation','PROJ','ETRS89_TO_ETRF90','EPSG','1298','EPSG','1024');
INSERT INTO "helmert_transformation" VALUES('PROJ','ETRS89_TO_ETRF91','ETRS89 to ETRF91','Accuracy 0.1 m, from datum ensemble definition','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4258','EPSG','9061',0.1,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'',0);
INSERT INTO "usage" VALUES('PROJ','ETRS89_TO_ETRF91_USAGE','helmert_transformation','PROJ','ETRS89_TO_ETRF91','EPSG','1298','EPSG','1024');
INSERT INTO "helmert_transformation" VALUES('PROJ','ETRS89_TO_ETRF92','ETRS89 to ETRF92','Accuracy 0.1 m, from datum ensemble definition','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4258','EPSG','9062',0.1,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'',0);
INSERT INTO "usage" VALUES('PROJ','ETRS89_TO_ETRF92_USAGE','helmert_transformation','PROJ','ETRS89_TO_ETRF92','EPSG','1298','EPSG','1024');
INSERT INTO "helmert_transformation" VALUES('PROJ','ETRS89_TO_ETRF93','ETRS89 to ETRF93','Accuracy 0.1 m, from datum ensemble definition','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4258','EPSG','9063',0.1,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'',0);
INSERT INTO "usage" VALUES('PROJ','ETRS89_TO_ETRF93_USAGE','helmert_transformation','PROJ','ETRS89_TO_ETRF93','EPSG','1298','EPSG','1024');
INSERT INTO "helmert_transformation" VALUES('PROJ','ETRS89_TO_ETRF94','ETRS89 to ETRF94','Accuracy 0.1 m, from datum ensemble definition','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4258','EPSG','9064',0.1,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'',0);
INSERT INTO "usage" VALUES('PROJ','ETRS89_TO_ETRF94_USAGE','helmert_transformation','PROJ','ETRS89_TO_ETRF94','EPSG','1298','EPSG','1024');
INSERT INTO "helmert_transformation" VALUES('PROJ','ETRS89_TO_ETRF96','ETRS89 to ETRF96','Accuracy 0.1 m, from datum ensemble definition','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4258','EPSG','9065',0.1,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'',0);
INSERT INTO "usage" VALUES('PROJ','ETRS89_TO_ETRF96_USAGE','helmert_transformation','PROJ','ETRS89_TO_ETRF96','EPSG','1298','EPSG','1024');
INSERT INTO "helmert_transformation" VALUES('PROJ','ETRS89_TO_ETRF97','ETRS89 to ETRF97','Accuracy 0.1 m, from datum ensemble definition','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4258','EPSG','9066',0.1,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'',0);
INSERT INTO "usage" VALUES('PROJ','ETRS89_TO_ETRF97_USAGE','helmert_transformation','PROJ','ETRS89_TO_ETRF97','EPSG','1298','EPSG','1024');
INSERT INTO "helmert_transformation" VALUES('PROJ','ETRS89_TO_ETRF2000','ETRS89 to ETRF2000','Accuracy 0.1 m, from datum ensemble definition','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4258','EPSG','9067',0.1,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'',0);
INSERT INTO "usage" VALUES('PROJ','ETRS89_TO_ETRF2000_USAGE','helmert_transformation','PROJ','ETRS89_TO_ETRF2000','EPSG','1298','EPSG','1024');
INSERT INTO "helmert_transformation" VALUES('PROJ','ETRS89_TO_ETRF2005','ETRS89 to ETRF2005','Accuracy 0.1 m, from datum ensemble definition','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4258','EPSG','9068',0.1,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'',0);
INSERT INTO "usage" VALUES('PROJ','ETRS89_TO_ETRF2005_USAGE','helmert_transformation','PROJ','ETRS89_TO_ETRF2005','EPSG','1298','EPSG','1024');
INSERT INTO "helmert_transformation" VALUES('PROJ','ETRS89_TO_ETRF2014','ETRS89 to ETRF2014','Accuracy 0.1 m, from datum ensemble definition','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4258','EPSG','9069',0.1,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'',0);
INSERT INTO "usage" VALUES('PROJ','ETRS89_TO_ETRF2014_USAGE','helmert_transformation','PROJ','ETRS89_TO_ETRF2014','EPSG','1298','EPSG','1024');
INSERT INTO "helmert_transformation" VALUES('PROJ','ETRS89_TO_ETRF2020','ETRS89 to ETRF2020','Accuracy 0.1 m, from datum ensemble definition','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4258','EPSG','10571',0.1,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'',0);
INSERT INTO "usage" VALUES('PROJ','ETRS89_TO_ETRF2020_USAGE','helmert_transformation','PROJ','ETRS89_TO_ETRF2020','EPSG','1298','EPSG','1024');
INSERT INTO "helmert_transformation" VALUES('PROJ','WGS84_TO_WGS84_TRANSIT','WGS 84 to WGS 84 (Transit)','Accuracy 2.0 m, from datum ensemble definition','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4326','EPSG','8888',2.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'',0);
INSERT INTO "usage" VALUES('PROJ','WGS84_TO_WGS84_TRANSIT_USAGE','helmert_transformation','PROJ','WGS84_TO_WGS84_TRANSIT','EPSG','1262','EPSG','1024');
INSERT INTO "helmert_transformation" VALUES('PROJ','WGS84_TO_WGS84_G730','WGS 84 to WGS 84 (G730)','Accuracy 2.0 m, from datum ensemble definition','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4326','EPSG','9053',2.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'',0);
INSERT INTO "usage" VALUES('PROJ','WGS84_TO_WGS84_G730_USAGE','helmert_transformation','PROJ','WGS84_TO_WGS84_G730','EPSG','1262','EPSG','1024');
INSERT INTO "helmert_transformation" VALUES('PROJ','WGS84_TO_WGS84_G873','WGS 84 to WGS 84 (G873)','Accuracy 2.0 m, from datum ensemble definition','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4326','EPSG','9054',2.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'',0);
INSERT INTO "usage" VALUES('PROJ','WGS84_TO_WGS84_G873_USAGE','helmert_transformation','PROJ','WGS84_TO_WGS84_G873','EPSG','1262','EPSG','1024');
INSERT INTO "helmert_transformation" VALUES('PROJ','WGS84_TO_WGS84_G1150','WGS 84 to WGS 84 (G1150)','Accuracy 2.0 m, from datum ensemble definition','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4326','EPSG','9055',2.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'',0);
INSERT INTO "usage" VALUES('PROJ','WGS84_TO_WGS84_G1150_USAGE','helmert_transformation','PROJ','WGS84_TO_WGS84_G1150','EPSG','1262','EPSG','1024');
INSERT INTO "helmert_transformation" VALUES('PROJ','WGS84_TO_WGS84_G1674','WGS 84 to WGS 84 (G1674)','Accuracy 2.0 m, from datum ensemble definition','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4326','EPSG','9056',2.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'',0);
INSERT INTO "usage" VALUES('PROJ','WGS84_TO_WGS84_G1674_USAGE','helmert_transformation','PROJ','WGS84_TO_WGS84_G1674','EPSG','1262','EPSG','1024');
INSERT INTO "helmert_transformation" VALUES('PROJ','WGS84_TO_WGS84_G1762','WGS 84 to WGS 84 (G1762)','Accuracy 2.0 m, from datum ensemble definition','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4326','EPSG','9057',2.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'',0);
INSERT INTO "usage" VALUES('PROJ','WGS84_TO_WGS84_G1762_USAGE','helmert_transformation','PROJ','WGS84_TO_WGS84_G1762','EPSG','1262','EPSG','1024');
INSERT INTO "helmert_transformation" VALUES('PROJ','WGS84_TO_WGS84_G2139','WGS 84 to WGS 84 (G2139)','Accuracy 2.0 m, from datum ensemble definition','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4326','EPSG','9755',2.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'',0);
INSERT INTO "usage" VALUES('PROJ','WGS84_TO_WGS84_G2139_USAGE','helmert_transformation','PROJ','WGS84_TO_WGS84_G2139','EPSG','1262','EPSG','1024');
INSERT INTO "helmert_transformation" VALUES('PROJ','WGS84_TO_WGS84_G2296','WGS 84 to WGS 84 (G2296)','Accuracy 2.0 m, from datum ensemble definition','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4326','EPSG','10606',2.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'',0);
INSERT INTO "usage" VALUES('PROJ','WGS84_TO_WGS84_G2296_USAGE','helmert_transformation','PROJ','WGS84_TO_WGS84_G2296','EPSG','1262','EPSG','1024');
INSERT INTO "helmert_transformation" VALUES('EPSG','1024','MGI to ETRS89 (4)','Parameter values from MGI to WGS 84 (8) (tfm code 1194). Assumes ETRS89 and WGS 84 can be considered the same to within the accuracy of the transformation. Information source gives scale as -2.388739 ppm.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4312','EPSG','4258',1.0,601.705,84.263,485.227,'EPSG','9001',-4.7354,-1.3145,-5.393,'EPSG','9104',-2.3887,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'LBD-Aut Sty',0);
INSERT INTO "usage" VALUES('EPSG','7945','helmert_transformation','EPSG','1024','EPSG','1543','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1055','Ain el Abd to WGS 84 (3)','Derived at station K1.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4204','EPSG','4326',1.0,-145.7,-249.1,1.5,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'WGC-Kwt',0);
INSERT INTO "usage" VALUES('EPSG','7976','helmert_transformation','EPSG','1055','EPSG','3267','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','1056','Ain el Abd to WGS 84 (4)','Derivation is more precise, but no evidence that accuracy is better than Ain el Abd to WGS 84 (3). OGP recommends using Ain el Abd to WGS 84 (3).','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4204','EPSG','4326',1.0,-85.645,-273.077,-79.708,'EPSG','9001',-2.289,1.421,-2.532,'EPSG','9104',3.194,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'Par-Kwt',0);
INSERT INTO "usage" VALUES('EPSG','7977','helmert_transformation','EPSG','1056','EPSG','3267','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','1057','Ain el Abd to WGS 84 (5)','','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4204','EPSG','4326',1.0,-202.234,-168.351,-63.51,'EPSG','9001',-3.545,-0.659,1.945,'EPSG','9104',2.1,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'Par-Kwt N',0);
INSERT INTO "usage" VALUES('EPSG','7978','helmert_transformation','EPSG','1057','EPSG','2956','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','1058','Ain el Abd to WGS 84 (6)','','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4204','EPSG','4326',1.0,-18.944,-379.364,-24.063,'EPSG','9001',-0.04,0.764,-6.431,'EPSG','9104',3.657,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'Par-Kwt S',0);
INSERT INTO "usage" VALUES('EPSG','7979','helmert_transformation','EPSG','1058','EPSG','2957','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','1059','KOC to WGS 84 (1)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4246','EPSG','4326',1.0,-294.7,-200.1,525.5,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'WGC-Kwt',0);
INSERT INTO "usage" VALUES('EPSG','7980','helmert_transformation','EPSG','1059','EPSG','3267','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','1060','NGN to WGS 84 (1)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4318','EPSG','4326',1.0,-3.2,-5.7,2.8,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'Mun-Kwt',0);
INSERT INTO "usage" VALUES('EPSG','7981','helmert_transformation','EPSG','1060','EPSG','3267','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','1061','Kudams to WGS 84 (1)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4319','EPSG','4326',1.0,-20.8,11.3,2.4,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'Mun-Kwt',0);
INSERT INTO "usage" VALUES('EPSG','7982','helmert_transformation','EPSG','1061','EPSG','1310','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','1062','Kudams to WGS 84 (2)','','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4319','EPSG','4326',1.0,226.702,-193.337,-35.371,'EPSG','9001',2.229,4.391,-9.238,'EPSG','9104',0.9798,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'Par-Kwt',0);
INSERT INTO "usage" VALUES('EPSG','7983','helmert_transformation','EPSG','1062','EPSG','1310','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','1063','Vientiane 1982 to Lao 1997 (1)','Derived at 8 stations.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4676','EPSG','4678',2.0,-2.227,6.524,2.178,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NGD-Lao',0);
INSERT INTO "usage" VALUES('EPSG','7984','helmert_transformation','EPSG','1063','EPSG','1138','EPSG','1042');
INSERT INTO "helmert_transformation" VALUES('EPSG','1064','Lao 1993 to Lao 1997 (1)','Derived at 25 stations.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4677','EPSG','4678',0.15,-0.652,1.619,0.213,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NGD-Lao',0);
INSERT INTO "usage" VALUES('EPSG','7985','helmert_transformation','EPSG','1064','EPSG','1138','EPSG','1078');
INSERT INTO "helmert_transformation" VALUES('EPSG','1065','Lao 1997 to WGS 84 (1)','Derived at 25 stations.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4678','EPSG','4326',5.0,44.585,-131.212,-39.544,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NGD-Lao',0);
INSERT INTO "usage" VALUES('EPSG','7986','helmert_transformation','EPSG','1065','EPSG','1138','EPSG','1045');
INSERT INTO "helmert_transformation" VALUES('EPSG','1066','Amersfoort to ETRS89 (2)','Replaced by Amersfoort to ETRS89 (4) (tfm code 15740). Dutch sources also quote an equivalent transformation using the Coordinate Frame 7-parameter method - see tfm code 1751.','EPSG','9636','Molodensky-Badekas (CF geog2D domain)','EPSG','4289','EPSG','4258',0.5,593.032,26.0,478.741,'EPSG','9001',1.9848,-1.7439,9.0587,'EPSG','9109',4.0772,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,3903453.148,368135.313,5012970.306,'EPSG','9001','NCG-Nld 2000 MB',0);
INSERT INTO "usage" VALUES('EPSG','7987','helmert_transformation','EPSG','1066','EPSG','1275','EPSG','1035');
INSERT INTO "helmert_transformation" VALUES('EPSG','1067','Minna to WGS 84 (11)','Used by Statoil for deep water blocks 210, 213, 217 and 218. Parameter values interpolated from Racal Survey geocentric translation contour charts for each of these four blocks and then averaged.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4263','EPSG','4326',8.0,-92.1,-89.9,114.9,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'Stat-Nga',0);
INSERT INTO "usage" VALUES('EPSG','7988','helmert_transformation','EPSG','1067','EPSG','3817','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1070','Guam 1963 to WGS 84 (1)','Derived at 5 stations. Accuracy 3m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4675','EPSG','4326',6.0,-100.0,-248.0,259.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Gum',0);
INSERT INTO "usage" VALUES('EPSG','7991','helmert_transformation','EPSG','1070','EPSG','3255','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1071','Palestine 1923 to Israel 1993 (1)','For more accurate transformation contact Survey of Israel.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4281','EPSG','4141',1.5,-181.0,-122.0,225.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'SoI-Isr',0);
INSERT INTO "usage" VALUES('EPSG','14559','helmert_transformation','EPSG','1071','EPSG','2603','EPSG','1157');
INSERT INTO "helmert_transformation" VALUES('EPSG','1073','Israel 1993 to WGS 84 (1)','For more accurate transformation contact Survey of Israel.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4141','EPSG','4326',2.0,-48.0,55.0,52.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'SoI-Isr',0);
INSERT INTO "usage" VALUES('EPSG','14558','helmert_transformation','EPSG','1073','EPSG','2603','EPSG','1042');
INSERT INTO "helmert_transformation" VALUES('EPSG','1074','Palestine 1923 to WGS 84 (1)','Accuracy: 1m to north and 5m to south of east-west line through Beersheba (31°15''N). Not recognised by Survey of Israel. See Palestine 1923 to WGS 84 (2) (code 8650).','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4281','EPSG','4326',2.0,-275.7224,94.7824,340.8944,'EPSG','9001',-8.001,-4.42,-11.821,'EPSG','9104',1.0,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Isr',0);
INSERT INTO "usage" VALUES('EPSG','7995','helmert_transformation','EPSG','1074','EPSG','2603','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1075','ED50 to WGS 84 (38)','Derived in 1987 by Geodetic for TPAO. Used on BP 1991/92 2D seismic surveys in central and eastern Turkish sector of Black Sea. In Turkey, replaced by tfm code 1784. Also adopted for use offshore Israel.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4230','EPSG','4326',10.0,-89.05,-87.03,-124.56,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'TPAO-Tur',0);
INSERT INTO "usage" VALUES('EPSG','7996','helmert_transformation','EPSG','1075','EPSG','2896','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1078','LUREF to ETRS89 (2)','LUREF 1994 parameters. Replaced by LUREF 2006 parameters, see transformation LUREF to ETRS89 (4) (code 5483). May be taken as approximate transformation LUREF to WGS 84 - see CT code 1079.','EPSG','9636','Molodensky-Badekas (CF geog2D domain)','EPSG','4181','EPSG','4258',0.1,-265.983,76.918,20.182,'EPSG','9001',0.4099,2.9332,-2.6881,'EPSG','9104',0.43,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,4098647.674,442843.139,4851251.093,'EPSG','9001','ACT-Lux 0.1m',0);
INSERT INTO "usage" VALUES('EPSG','7999','helmert_transformation','EPSG','1078','EPSG','1146','EPSG','1031');
INSERT INTO "helmert_transformation" VALUES('EPSG','1079','LUREF to WGS 84 (2)','Parameter values from LUREF to ETRS89 (2) (code 1078). Assumes ETRS89 and WGS 84 can be considered the same to within the accuracy of the transformation.','EPSG','9636','Molodensky-Badekas (CF geog2D domain)','EPSG','4181','EPSG','4326',1.0,-265.983,76.918,20.182,'EPSG','9001',0.4099,2.9332,-2.6881,'EPSG','9104',0.43,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,4098647.674,442843.139,4851251.093,'EPSG','9001','EPSG-Lux 1994',0);
INSERT INTO "usage" VALUES('EPSG','8000','helmert_transformation','EPSG','1079','EPSG','1146','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1080','CI1971 to WGS 84 (1)','Derived at 4 stations. Accuracy 15m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4672','EPSG','4326',26.0,175.0,-38.0,113.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Nzl CI',0);
INSERT INTO "usage" VALUES('EPSG','8001','helmert_transformation','EPSG','1080','EPSG','2889','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1081','CI1979 to WGS 84 (1)','Derived at 4 stations using concatenation through WGS72. Parameter vales are also used to transform CI1979 to NZGD2000 - see tfm code 1082.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4673','EPSG','4326',2.0,174.05,-25.49,112.57,'EPSG','9001',0.0,0.0,-0.554,'EPSG','9104',0.2263,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OSG-Nzl CI',0);
INSERT INTO "usage" VALUES('EPSG','8002','helmert_transformation','EPSG','1081','EPSG','2889','EPSG','1042');
INSERT INTO "helmert_transformation" VALUES('EPSG','1082','CI1979 to NZGD2000 (1)','Parameter vales are from CI1979 to WGS 84 (1) (code 1081) assuming that WGS 84 is equivalent to NZGD2000 within the accuracy of the transformation.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4673','EPSG','4167',2.0,174.05,-25.49,112.57,'EPSG','9001',0.0,0.0,-0.554,'EPSG','9104',0.2263,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OSG-Nzl CI',0);
INSERT INTO "usage" VALUES('EPSG','8003','helmert_transformation','EPSG','1082','EPSG','2889','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1083','JAD69 to WGS 72 (2)','Derived in 1981 through Transit observations at 4 stations by Geodetic Survey for Petroleum Corporation of Jamaica.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4242','EPSG','4322',10.0,50.0,212.0,381.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'PC-Jam',0);
INSERT INTO "usage" VALUES('EPSG','8004','helmert_transformation','EPSG','1083','EPSG','3342','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1084','JAD69 to WGS 84 (1)','Derived via NAD27 and WGS 72. Preliminary values derived by Survey Department but not officially promulgated.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4242','EPSG','4326',5.0,70.0,207.0,389.5,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'UT-Jam 5m',0);
INSERT INTO "usage" VALUES('EPSG','8005','helmert_transformation','EPSG','1084','EPSG','3342','EPSG','1045');
INSERT INTO "helmert_transformation" VALUES('EPSG','1085','JAD69 to WGS 84 (2)','Derived at 4 stations, tested at a further 9.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4242','EPSG','4326',2.0,65.334,212.46,387.63,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'UT-Jam 2m',0);
INSERT INTO "usage" VALUES('EPSG','8006','helmert_transformation','EPSG','1085','EPSG','3342','EPSG','1042');
INSERT INTO "helmert_transformation" VALUES('EPSG','1086','JAD69 to WGS 84 (3)','Derived at 4 stations, tested at a further 9.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4242','EPSG','4326',1.0,-33.722,153.789,94.959,'EPSG','9001',8.581,4.478,-4.54,'EPSG','9104',-8.95,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'UT-Jam 1m',1);
INSERT INTO "usage" VALUES('EPSG','8007','helmert_transformation','EPSG','1086','EPSG','3342','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','1087','ED50 to WGS 84 (37)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4230','EPSG','4326',2.5,-112.0,-110.3,-140.2,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'RJGC-Jor',0);
INSERT INTO "usage" VALUES('EPSG','8008','helmert_transformation','EPSG','1087','EPSG','1130','EPSG','1153');
INSERT INTO "helmert_transformation" VALUES('EPSG','1088','Monte Mario to WGS 84 (5)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4265','EPSG','4326',10.0,-223.7,-67.38,1.34,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'ENI-Ita Adr N Anc',0);
INSERT INTO "usage" VALUES('EPSG','8009','helmert_transformation','EPSG','1088','EPSG','2882','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1089','Monte Mario to WGS 84 (6)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4265','EPSG','4326',10.0,-225.4,-67.7,7.85,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'ENI-Ita Adr Anc-Gar',0);
INSERT INTO "usage" VALUES('EPSG','8010','helmert_transformation','EPSG','1089','EPSG','2883','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1090','Monte Mario to WGS 84 (7)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4265','EPSG','4326',10.0,-227.1,-68.1,14.4,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'ENI-Ita Adr S Gar',0);
INSERT INTO "usage" VALUES('EPSG','8011','helmert_transformation','EPSG','1090','EPSG','2884','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1091','Monte Mario to WGS 84 (8)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4265','EPSG','4326',10.0,-231.61,-68.21,13.93,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'ENI-Ita Otr',0);
INSERT INTO "usage" VALUES('EPSG','8012','helmert_transformation','EPSG','1091','EPSG','2885','EPSG','1251');
INSERT INTO "helmert_transformation" VALUES('EPSG','1092','Monte Mario to WGS 84 (9)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4265','EPSG','4326',10.0,-225.06,-67.37,14.61,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'ENI-Ita N Jon',0);
INSERT INTO "usage" VALUES('EPSG','8013','helmert_transformation','EPSG','1092','EPSG','2886','EPSG','1251');
INSERT INTO "helmert_transformation" VALUES('EPSG','1093','Monte Mario to WGS 84 (10)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4265','EPSG','4326',10.0,-229.08,-65.73,20.21,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'ENI-Ita E Sic',0);
INSERT INTO "usage" VALUES('EPSG','8014','helmert_transformation','EPSG','1093','EPSG','2887','EPSG','1251');
INSERT INTO "helmert_transformation" VALUES('EPSG','1094','Monte Mario to WGS 84 (11)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4265','EPSG','4326',10.0,-230.47,-56.08,22.43,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'ENI-Ita W Sic',0);
INSERT INTO "usage" VALUES('EPSG','8015','helmert_transformation','EPSG','1094','EPSG','2888','EPSG','1251');
INSERT INTO "helmert_transformation" VALUES('EPSG','1095','PSAD56 to WGS 84 (13)','Parameter values are from PSAD56 to REGVEN (1) (code 1769) assuming that REGVEN is equivalent to WGS 84 within the accuracy of the transformation.','EPSG','9636','Molodensky-Badekas (CF geog2D domain)','EPSG','4248','EPSG','4326',15.0,-270.933,115.599,-360.226,'EPSG','9001',-5.266,-1.238,2.381,'EPSG','9104',-5.109,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,2464351.59,-5783466.61,974809.81,'EPSG','9001','EPSG-Ven',0);
INSERT INTO "usage" VALUES('EPSG','8016','helmert_transformation','EPSG','1095','EPSG','3327','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1096','La Canoa to WGS 84 (13)','Parameter values are from La Canoa to REGVEN (1) (code 1771) assuming that REGVEN is equivalent to WGS 84 within the accuracy of the transformation.','EPSG','9636','Molodensky-Badekas (CF geog2D domain)','EPSG','4247','EPSG','4326',15.0,-270.933,115.599,-360.226,'EPSG','9001',-5.266,-1.238,2.381,'EPSG','9104',-5.109,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,2464351.59,-5783466.61,974809.81,'EPSG','9001','EPSG-Ven',0);
INSERT INTO "usage" VALUES('EPSG','8017','helmert_transformation','EPSG','1096','EPSG','3327','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1097','Dealul Piscului 1970 to WGS 84 (2)','Parameter values taken from Pulkovo 1942 to WGS 84 (9) (code 1293) assuming that Pulkovo 1942 in Romania is equivalent to Dealul Piscului 1970.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4317','EPSG','4326',7.0,28.0,-121.0,-77.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Rom',1);
INSERT INTO "usage" VALUES('EPSG','8018','helmert_transformation','EPSG','1097','EPSG','1197','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1098','IGM95 to ETRS89 (1)','IGM95 is the first Italian realization of ETRS89. May be taken as approximate transformation IGM95 to WGS 84 - see code 1099. For accurate transformation to a later realization see IGM95 to RDN2008 (1) (CT code 9737).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4670','EPSG','4258',0.5,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGM-Ita',0);
INSERT INTO "usage" VALUES('EPSG','14407','helmert_transformation','EPSG','1098','EPSG','3343','EPSG','1161');
INSERT INTO "helmert_transformation" VALUES('EPSG','1099','IGM95 to WGS 84 (1)','Approximation at the +/- 1m level assuming that ETRS89 is equivalent to WGS 84. IGM95 is a national realization of ETRS89.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4670','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGM-Ita',0);
INSERT INTO "usage" VALUES('EPSG','14408','helmert_transformation','EPSG','1099','EPSG','3343','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1100','Adindan to WGS 84 (1)','Derived at 22 stations. Accuracy 5m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4201','EPSG','4326',9.0,-166.0,-15.0,204.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Eth Sud',0);
INSERT INTO "usage" VALUES('EPSG','8021','helmert_transformation','EPSG','1100','EPSG','1271','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1101','Adindan to WGS 84 (2)','Derived at 1 station connected to the Adindan (Blue Nile 1958) network through the 1968-69 12th parallel traverse. Accuracy 25m in each axis. Note: the Adindan (Blue Nile 1958) CRS is used in Ethiopia and Sudan, not Burkino Faso.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4201','EPSG','4326',44.0,-118.0,-14.0,218.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Bfa',0);
INSERT INTO "usage" VALUES('EPSG','8022','helmert_transformation','EPSG','1101','EPSG','1057','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1102','Adindan to WGS 84 (3)','Derived at 1 station connected to the Adindan (Blue Nile 1958) network through the 1968-69 12th parallel traverse. Accuracy 25m in each axis. Note: the Adindan (Blue Nile 1958) CRS is used in Ethiopia and Sudan, not Cameroon.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4201','EPSG','4326',44.0,-134.0,-2.0,210.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Cmr',0);
INSERT INTO "usage" VALUES('EPSG','8023','helmert_transformation','EPSG','1102','EPSG','3226','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1103','Adindan to WGS 84 (4)','Derived at 8 stations. Accuracy 3m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4201','EPSG','4326',6.0,-165.0,-11.0,206.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Eth',0);
INSERT INTO "usage" VALUES('EPSG','8024','helmert_transformation','EPSG','1103','EPSG','1091','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1104','Adindan to WGS 84 (5)','Derived at 1 station connected to the Adindan (Blue Nile 1958) network through the 1968-69 12th parallel traverse. Accuracy 25m in each axis. Note: the Adindan (Blue Nile 1958) CRS is used in Ethiopia and Sudan, not Mali.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4201','EPSG','4326',44.0,-123.0,-20.0,220.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Mli',0);
INSERT INTO "usage" VALUES('EPSG','8025','helmert_transformation','EPSG','1104','EPSG','1153','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1105','Adindan to WGS 84 (6)','Derived at 2 stations connected to the Adindan (Blue Nile 1958) network through the 1968-69 12th parallel traverse. Accuracy 25m in each axis. Note: The Adindan (Blue Nile 1958) CRS is used in Ethiopia and Sudan, not Senegal.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4201','EPSG','4326',44.0,-128.0,-18.0,224.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Sen',0);
INSERT INTO "usage" VALUES('EPSG','8026','helmert_transformation','EPSG','1105','EPSG','3304','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1106','Adindan to WGS 84 (7)','Derived at 14 stations. Accuracy 3m, 5m and 3m in X, Y and Z axes.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4201','EPSG','4326',7.0,-161.0,-14.0,205.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Sud',0);
INSERT INTO "usage" VALUES('EPSG','8027','helmert_transformation','EPSG','1106','EPSG','3311','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1107','Afgooye to WGS 84 (1)','Derived at 1 station. Accuracy 25m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4205','EPSG','4326',44.0,-43.0,-163.0,45.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Som',0);
INSERT INTO "usage" VALUES('EPSG','8028','helmert_transformation','EPSG','1107','EPSG','3308','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1108','AGD66 to WGS 84 (1)','Derived at 105 stations. Accuracy 3m in each axis. Replaced by AGD66 to WGS 84 (20) (code 6905).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4202','EPSG','4326',6.0,-133.0,-48.0,148.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Aus',0);
INSERT INTO "usage" VALUES('EPSG','8029','helmert_transformation','EPSG','1108','EPSG','2575','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1109','AGD84 to WGS 84 (1)','Derived at 90 stations. Accuracy 2m in each axis. Note: AGD84 officially adopted only in Queensland, South Australia and Western Australia.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4203','EPSG','4326',4.0,-134.0,-48.0,149.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Aus',0);
INSERT INTO "usage" VALUES('EPSG','8030','helmert_transformation','EPSG','1109','EPSG','2576','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1110','Ain el Abd to WGS 84 (1)','Derived at 2 stations. Accuracy 25m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4204','EPSG','4326',44.0,-150.0,-250.0,-1.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Bhr',0);
INSERT INTO "usage" VALUES('EPSG','8031','helmert_transformation','EPSG','1110','EPSG','3943','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1111','Ain el Abd to WGS 84 (2)','Derived at 9 stations. Accuracy 10m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4204','EPSG','4326',18.0,-143.0,-236.0,7.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Sau',0);
INSERT INTO "usage" VALUES('EPSG','8032','helmert_transformation','EPSG','1111','EPSG','3303','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1112','Amersfoort to WGS 84 (1)','Replaced by Amersfoort to WGS 84 (2) (code 1672).','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4289','EPSG','4326',1.0,593.16,26.15,478.54,'EPSG','9001',-6.3239,-0.5008,-5.5487,'EPSG','9109',4.0775,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NCG-Nld 93',0);
INSERT INTO "usage" VALUES('EPSG','8033','helmert_transformation','EPSG','1112','EPSG','1275','EPSG','1024');
INSERT INTO "helmert_transformation" VALUES('EPSG','1113','Arc 1950 to WGS 84 (1)','Derived at 41 stations. Accuracy 20m, 33m and 20m in X, Y and Z axes. Note: Eswatini and Lesotho do not use Arc 1950 but do use Cape (CRS code 4222); Cape and Arc 1950 datums share a common fundamental point.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4209','EPSG','4326',44.0,-143.0,-90.0,-294.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-mean',0);
INSERT INTO "usage" VALUES('EPSG','8034','helmert_transformation','EPSG','1113','EPSG','2312','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1114','Arc 1950 to WGS 84 (2)','Derived at 9 stations. Accuracy 3m, 5m and 3m in X, Y and Z axes.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4209','EPSG','4326',7.0,-138.0,-105.0,-289.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Bwa',0);
INSERT INTO "usage" VALUES('EPSG','8035','helmert_transformation','EPSG','1114','EPSG','1051','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1115','Arc 1950 to WGS 84 (3)','Derived at 3 stations.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4209','EPSG','4326',35.0,-153.0,-5.0,-292.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Bdi',1);
INSERT INTO "usage" VALUES('EPSG','8036','helmert_transformation','EPSG','1115','EPSG','1058','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1116','Arc 1950 to WGS 84 (4)','Derived at 5 stations. Accuracy 3m, 3m and 8m in X, Y and Z axes. Note: Lesotho does not use Arc 1950 but does use Cape (CRS code 4222); Cape and Arc 1950 datums share a common fundamental point.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4209','EPSG','4326',10.0,-125.0,-108.0,-295.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Lso',0);
INSERT INTO "usage" VALUES('EPSG','8037','helmert_transformation','EPSG','1116','EPSG','1141','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1117','Arc 1950 to WGS 84 (5)','Derived at 6 stations. Accuracy 9m, 24m and 8m in X, Y and Z axes.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4209','EPSG','4326',27.0,-161.0,-73.0,-317.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Mwi',0);
INSERT INTO "usage" VALUES('EPSG','8038','helmert_transformation','EPSG','1117','EPSG','1150','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1118','Arc 1950 to WGS 84 (6)','Derived at 4 stations. Accuracy 15m in each axis. Note: Eswatini does not use Arc 1950 but does use Cape (CRS code 4222); Cape and Arc 1950 datums share a common fundamental point.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4209','EPSG','4326',26.0,-134.0,-105.0,-295.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Swz',0);
INSERT INTO "usage" VALUES('EPSG','8039','helmert_transformation','EPSG','1118','EPSG','1224','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1119','Arc 1950 to WGS 84 (7)','Derived at 2 stations.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4209','EPSG','4326',25.0,-169.0,-19.0,-278.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Cod',1);
INSERT INTO "usage" VALUES('EPSG','8040','helmert_transformation','EPSG','1119','EPSG','1259','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1120','Arc 1950 to WGS 84 (8)','Derived at 5 stations. Accuracy 21m, 21m and 27m in X, Y and Z axes.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4209','EPSG','4326',41.0,-147.0,-74.0,-283.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Zmb',0);
INSERT INTO "usage" VALUES('EPSG','8041','helmert_transformation','EPSG','1120','EPSG','1260','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1121','Arc 1950 to WGS 84 (9)','Derived at 10 stations. Accuracy 5m, 8m and 11m in X, Y and Z axes. Replaced by Arc 1950 to WGS 84 (10), tfm code 6906.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4209','EPSG','4326',15.0,-142.0,-96.0,-293.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Zwe',0);
INSERT INTO "usage" VALUES('EPSG','8042','helmert_transformation','EPSG','1121','EPSG','1261','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1122','Arc 1960 to WGS 84 (1)','Derived at 25 stations. Accuracy 20m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4210','EPSG','4326',35.0,-160.0,-6.0,-302.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Ken Tza',0);
INSERT INTO "usage" VALUES('EPSG','8043','helmert_transformation','EPSG','1122','EPSG','2311','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1123','Batavia to WGS 84 (1)','Note: The area of use cited for this transformation (Sumatra) is not consistent with the area of use (Java) for the Batavia (Genuk) coordinate reference system. Derived at 5 stations.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4211','EPSG','4326',6.0,-377.0,681.0,-50.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Idn Sumatra',1);
INSERT INTO "usage" VALUES('EPSG','8044','helmert_transformation','EPSG','1123','EPSG','1355','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1124','Bermuda 1957 to WGS 84 (1)','Derived at 3 stations. Accuracy 20m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4216','EPSG','4326',35.0,-73.0,213.0,296.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Bmu',0);
INSERT INTO "usage" VALUES('EPSG','8045','helmert_transformation','EPSG','1124','EPSG','3221','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1125','Bogota 1975 to WGS 84 (1)','Derived in 1987 at 7 stations. Accuracy 6m, 5m and 6m in X, Y and Z axes.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4218','EPSG','4326',10.0,307.0,304.0,-318.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Col',0);
INSERT INTO "usage" VALUES('EPSG','8046','helmert_transformation','EPSG','1125','EPSG','3686','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1126','Bukit Rimpah to WGS 84 (1)','Accuracy estimate not available.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4219','EPSG','4326',999.0,-384.0,664.0,-48.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Idn BBI',0);
INSERT INTO "usage" VALUES('EPSG','8047','helmert_transformation','EPSG','1126','EPSG','1287','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1127','Campo Inchauspe to WGS 84 (1)','Derived at 20 stations. Accuracy 5m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4221','EPSG','4326',9.0,-148.0,136.0,90.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Arg',0);
INSERT INTO "usage" VALUES('EPSG','8048','helmert_transformation','EPSG','1127','EPSG','3843','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1128','Cape to WGS 84 (1)','Derived at 5 stations. Accuracy 3m, 6m and 6m in X, Y and Z axes.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4222','EPSG','4326',9.0,-136.0,-108.0,-292.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Zaf',0);
INSERT INTO "usage" VALUES('EPSG','8049','helmert_transformation','EPSG','1128','EPSG','3309','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1129','Cape to WGS 84 (2)','Parameter values are from Cape to Hartebeesthoek94 (1) (code 1504) assuming that Hartebeesthoek94 and WGS 84 are equivalent within the accuracy of the transformation. Residuals should not exceed 15 metres.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4222','EPSG','4326',15.0,-134.73,-110.92,-292.66,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DSLI-Zaf',0);
INSERT INTO "usage" VALUES('EPSG','8050','helmert_transformation','EPSG','1129','EPSG','3309','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1130','Carthage to WGS 84 (1)','Derived at 5 stations. Accuracy 6m, 9m and 8m in X, Y and Z axes.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4223','EPSG','4326',14.0,-263.0,6.0,431.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Tun',0);
INSERT INTO "usage" VALUES('EPSG','8051','helmert_transformation','EPSG','1130','EPSG','1236','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1131','Chua to WGS 84 (1)','Derived at 6 stations. Accuracy 6m, 9m and 5m in X, Y and Z axes.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4224','EPSG','4326',12.0,-134.0,229.0,-29.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Pry',0);
INSERT INTO "usage" VALUES('EPSG','8052','helmert_transformation','EPSG','1131','EPSG','3675','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1132','Corrego Alegre 1970-72 to WGS 84 (1)','Derived at 17 stations. Accuracy 5m, 3m and 5m in X, Y and Z axes.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4225','EPSG','4326',8.0,-206.0,172.0,-6.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Bra',0);
INSERT INTO "usage" VALUES('EPSG','8053','helmert_transformation','EPSG','1132','EPSG','1293','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1133','ED50 to WGS 84 (1)','Derived at 85 stations. Accuracy 3m, 8m and 5m in X, Y and Z axes. In Germany will be accepted by LBA for minerals management purposes as alternative to tfm 1052 or 1998.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4230','EPSG','4326',10.0,-87.0,-98.0,-121.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-mean',0);
INSERT INTO "usage" VALUES('EPSG','8054','helmert_transformation','EPSG','1133','EPSG','2420','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1134','ED50 to WGS 84 (2)','Derived at 52 stations. Accuracy 3m each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4230','EPSG','4326',6.0,-87.0,-96.0,-120.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-cenEur',0);
INSERT INTO "usage" VALUES('EPSG','8055','helmert_transformation','EPSG','1134','EPSG','2421','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1135','ED50 to WGS 84 (3)','Accuracy estimate not available. Note: ED50 is not used in Israel, Lebanon, Kuwait, Saudi Arabia or Syria.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4230','EPSG','4326',999.0,-103.0,-106.0,-141.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-midEast',0);
INSERT INTO "usage" VALUES('EPSG','8056','helmert_transformation','EPSG','1135','EPSG','2345','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1136','ED50 to WGS 84 (4)','Derived at 4 stations. Accuracy 15m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4230','EPSG','4326',26.0,-104.0,-101.0,-140.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Cyp',0);
INSERT INTO "usage" VALUES('EPSG','8057','helmert_transformation','EPSG','1136','EPSG','1078','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1137','ED50 to WGS 84 (5)','Derived at 14 stations. Accuracy 6m, 8m and 8m in X, Y and Z axes.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4230','EPSG','4326',13.0,-130.0,-117.0,-151.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Egy',0);
INSERT INTO "usage" VALUES('EPSG','8058','helmert_transformation','EPSG','1137','EPSG','2595','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1138','ED50 to WGS 84 (6)','Derived at 40 stations. Accuracy 3m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4230','EPSG','4326',6.0,-86.0,-96.0,-120.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Irl Gbr',0);
INSERT INTO "usage" VALUES('EPSG','8059','helmert_transformation','EPSG','1138','EPSG','2343','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1139','ED50 to WGS 84 (7)','Derived at 20 stations. Accuracy 3m, 5m and 3m in X, Y and Z axes.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4230','EPSG','4326',7.0,-87.0,-95.0,-120.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Fin Nor',0);
INSERT INTO "usage" VALUES('EPSG','8060','helmert_transformation','EPSG','1139','EPSG','2344','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1140','ED50 to WGS 84 (8)','Derived at 2 stations. Accuracy 25m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4230','EPSG','4326',44.0,-84.0,-95.0,-130.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Grc',0);
INSERT INTO "usage" VALUES('EPSG','8061','helmert_transformation','EPSG','1140','EPSG','3254','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1141','ED50(ED77) to WGS 84 (2)','Given by DMA as from ED50. OGP interpret that as ED50(ED77) in Iran. Derived at 27 stations. Accuracy 9m, 12m and 11m in X, Y and Z axes.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4154','EPSG','4326',19.0,-117.0,-132.0,-164.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Irn',0);
INSERT INTO "usage" VALUES('EPSG','8062','helmert_transformation','EPSG','1141','EPSG','1123','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1142','ED50 to WGS 84 (10)','Derived at 2 stations. Accuracy 25m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4230','EPSG','4326',44.0,-97.0,-103.0,-120.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Ita Sard',0);
INSERT INTO "usage" VALUES('EPSG','8063','helmert_transformation','EPSG','1142','EPSG','2339','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1143','ED50 to WGS 84 (11)','Derived at 3 stations. Accuracy 20m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4230','EPSG','4326',35.0,-97.0,-88.0,-135.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Ita Sic',0);
INSERT INTO "usage" VALUES('EPSG','8064','helmert_transformation','EPSG','1143','EPSG','2340','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1144','ED50 to WGS 84 (12)','Derived at 1 station. Accuracy 25m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4230','EPSG','4326',44.0,-107.0,-88.0,-149.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Mlt',0);
INSERT INTO "usage" VALUES('EPSG','8065','helmert_transformation','EPSG','1144','EPSG','3275','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1145','ED50 to WGS 84 (13)','Derived at 18 stations. Accuracy 5m, 6m and 3m in X, Y and Z axes.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4230','EPSG','4326',9.0,-84.0,-107.0,-120.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Prt Esp',0);
INSERT INTO "usage" VALUES('EPSG','8066','helmert_transformation','EPSG','1145','EPSG','2338','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1146','ED87 to WGS 84 (1)','','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4231','EPSG','4326',0.8,-82.981,-99.719,-110.709,'EPSG','9001',-0.5076,0.1503,0.3898,'EPSG','9109',-0.3143,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'5Nat-NSea-90',0);
INSERT INTO "usage" VALUES('EPSG','8067','helmert_transformation','EPSG','1146','EPSG','2330','EPSG','1024');
INSERT INTO "helmert_transformation" VALUES('EPSG','1147','ED50 to ED87 (2)','','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4230','EPSG','4231',1.0,-1.51,-0.84,-3.5,'EPSG','9001',-1.893,-0.687,-2.764,'EPSG','9109',0.609,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NMA-Nor N65',0);
INSERT INTO "usage" VALUES('EPSG','8068','helmert_transformation','EPSG','1147','EPSG','2332','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','1148','Egypt 1907 to WGS 84 (1)','Derived at 14 stations. Accuracy 3m, 6m and 8m in X, Y and Z axes.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4229','EPSG','4326',11.0,-130.0,110.0,-13.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Egy',0);
INSERT INTO "usage" VALUES('EPSG','8069','helmert_transformation','EPSG','1148','EPSG','1086','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1149','ETRS89 to WGS 84 (1)','ETRS89 and WGS 84 are realizations of ITRS coincident to within 1 metre. This transformation has an accuracy equal to the coincidence figure.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4258','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-eur',0);
INSERT INTO "usage" VALUES('EPSG','8070','helmert_transformation','EPSG','1149','EPSG','4755','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1150','GDA94 to WGS 84 (1)','Approximation at the 3m level assuming WGS 84 is equivalent to GDA94. Ignores the low accuracy of the WGS 84 ensemble and the inconsistent application of tectonic plate motion to WGS 84 data.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4283','EPSG','4326',3.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Aus',0);
INSERT INTO "usage" VALUES('EPSG','8071','helmert_transformation','EPSG','1150','EPSG','4177','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1151','NZGD49 to WGS 84 (1)','Derived at 14 stations. Accuracy 5m, 3m and 5m in X, Y and Z axes.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4272','EPSG','4326',8.0,84.0,-22.0,209.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Nzl',0);
INSERT INTO "usage" VALUES('EPSG','8072','helmert_transformation','EPSG','1151','EPSG','3285','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1152','Hu Tzu Shan 1950 to WGS 84 (1)','Derived at 4 stations. Accuracy 15m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4236','EPSG','4326',26.0,-637.0,-549.0,-203.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Twn',0);
INSERT INTO "usage" VALUES('EPSG','8073','helmert_transformation','EPSG','1152','EPSG','3315','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1153','Indian 1954 to WGS 84 (1)','Derived at 11 stations. Accuracy 15m, 6m and 12m in X, Y and Z axes.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4239','EPSG','4326',21.0,217.0,823.0,299.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Tha',0);
INSERT INTO "usage" VALUES('EPSG','8074','helmert_transformation','EPSG','1153','EPSG','3317','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1154','Indian 1975 to WGS 84 (1)','Derived at 62 stations. Accuracy 3m, 2m and 3m in X, Y and Z axes. Replaced by Indian 1975 to WGS 84 (2) (code 1304).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4240','EPSG','4326',5.0,209.0,818.0,290.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Tha',0);
INSERT INTO "usage" VALUES('EPSG','8075','helmert_transformation','EPSG','1154','EPSG','3741','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1155','Kalianpur 1937 to WGS 84 (1)','Derived at 6 stations. Accuracy 10m, 8m and 12m in X, Y and Z axes.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4144','EPSG','4326',18.0,282.0,726.0,254.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Bgd',0);
INSERT INTO "usage" VALUES('EPSG','8076','helmert_transformation','EPSG','1155','EPSG','3217','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1156','Kalianpur 1975 to WGS 84 (1)','Derived at 7 stations. Accuracy 12m, 10m and 15m in X, Y and Z axes. Care! DMA ellipsoid is inconsistent with EPSG ellipsoid - transformation parameter values may not be appropriate. Also source CRS may not apply to Nepal.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4146','EPSG','4326',22.0,295.0,736.0,257.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Ind Npl',0);
INSERT INTO "usage" VALUES('EPSG','8077','helmert_transformation','EPSG','1156','EPSG','2411','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1157','Kandawala to WGS 84 (1)','Derived at 3 stations. Accuracy 20m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4244','EPSG','4326',35.0,-97.0,787.0,86.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Lka',0);
INSERT INTO "usage" VALUES('EPSG','8078','helmert_transformation','EPSG','1157','EPSG','3310','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1158','Kertau 1968 to WGS 84 (1)','Derived at 6 stations. Accuracy 10m, 8m and 6m in X, Y and Z axes.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4245','EPSG','4326',15.0,-11.0,851.0,5.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Mys Sgp',0);
INSERT INTO "usage" VALUES('EPSG','8079','helmert_transformation','EPSG','1158','EPSG','4223','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1159','Leigon to WGS 84 (1)','Derived at 8 stations. Accuracy 2m, 3m and 2m in X, Y and Z axes.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4250','EPSG','4326',5.0,-130.0,29.0,364.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Gha',0);
INSERT INTO "usage" VALUES('EPSG','8080','helmert_transformation','EPSG','1159','EPSG','1104','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1160','Liberia 1964 to WGS 84 (1)','Derived at 4 stations. Accuracy 15m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4251','EPSG','4326',26.0,-90.0,40.0,88.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Lbr',0);
INSERT INTO "usage" VALUES('EPSG','8081','helmert_transformation','EPSG','1160','EPSG','3270','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1161','Luzon 1911 to WGS 84 (1)','Derived at 6 stations. Accuracy 8m, 11m and 9m in X, Y and Z axes.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4253','EPSG','4326',17.0,-133.0,-77.0,-51.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Phl N',0);
INSERT INTO "usage" VALUES('EPSG','8082','helmert_transformation','EPSG','1161','EPSG','2364','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1162','Luzon 1911 to WGS 84 (2)','Derived at 1 station. Accuracy 25m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4253','EPSG','4326',44.0,-133.0,-79.0,-72.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Phl Min',0);
INSERT INTO "usage" VALUES('EPSG','8083','helmert_transformation','EPSG','1162','EPSG','2365','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1163','M''poraloko to WGS 84 (1)','Derived at 1 station. Accuracy 25m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4266','EPSG','4326',44.0,-74.0,-130.0,42.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Gab',0);
INSERT INTO "usage" VALUES('EPSG','8084','helmert_transformation','EPSG','1163','EPSG','1100','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1164','Mahe 1971 to WGS 84 (1)','Derived at 1 station. Accuracy 25m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4256','EPSG','4326',44.0,41.0,-220.0,-134.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Syc',0);
INSERT INTO "usage" VALUES('EPSG','8085','helmert_transformation','EPSG','1164','EPSG','2369','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1165','Massawa to WGS 84 (1)','Derived at 1 station. Accuracy 25m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4262','EPSG','4326',44.0,639.0,405.0,60.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Eth',0);
INSERT INTO "usage" VALUES('EPSG','8086','helmert_transformation','EPSG','1165','EPSG','1089','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1166','Merchich to WGS 84 (1)','Derived at 9 stations. Accuracy 5m, 3m and 3m in X, Y and Z axes.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4261','EPSG','4326',7.0,31.0,146.0,47.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Mar',0);
INSERT INTO "usage" VALUES('EPSG','8087','helmert_transformation','EPSG','1166','EPSG','3280','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1167','Minna to WGS 84 (1)','Derived at 2 stations. Accuracy 25m in each axis. Note: Minna is used in Nigeria, not Cameroon.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4263','EPSG','4326',44.0,-81.0,-84.0,115.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Cmr',0);
INSERT INTO "usage" VALUES('EPSG','8088','helmert_transformation','EPSG','1167','EPSG','3226','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1168','Minna to WGS 84 (2)','Derived at 6 stations. Accuracy 3m, 6m and 5m in X, Y and Z axes.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4263','EPSG','4326',15.0,-92.0,-93.0,122.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Nga',0);
INSERT INTO "usage" VALUES('EPSG','8089','helmert_transformation','EPSG','1168','EPSG','1178','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1169','Monte Mario to WGS 84 (1)','Derived at 1 station. Accuracy 25m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4265','EPSG','4326',44.0,-225.0,-65.0,9.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Ita Sar',0);
INSERT INTO "usage" VALUES('EPSG','8090','helmert_transformation','EPSG','1169','EPSG','2339','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1170','NAD27 to WGS 84 (1)','Derived at 15 stations. Accuracy 3m, 9m and 12m in X, Y and Z axes.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4267','EPSG','4326',16.0,-3.0,142.0,183.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Carib',0);
INSERT INTO "usage" VALUES('EPSG','8091','helmert_transformation','EPSG','1170','EPSG','2418','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1171','NAD27 to WGS 84 (2)','Derived at 19 stations. Accuracy 8m, 3m and 5m in X, Y and Z axes.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4267','EPSG','4326',10.0,0.0,125.0,194.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Cen Am',0);
INSERT INTO "usage" VALUES('EPSG','8092','helmert_transformation','EPSG','1171','EPSG','2419','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1172','NAD27 to WGS 84 (3)','Derived at 112 stations. Accuracy 15m, 11m and 6m in X, Y and Z axes.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4267','EPSG','4326',20.0,-10.0,158.0,187.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Can',0);
INSERT INTO "usage" VALUES('EPSG','8093','helmert_transformation','EPSG','1172','EPSG','4517','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1173','NAD27 to WGS 84 (4)','Derived at 405 stations. Accuracy 5m, 5m and 6m in X, Y and Z axes.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4267','EPSG','4326',10.0,-8.0,160.0,176.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Conus',0);
INSERT INTO "usage" VALUES('EPSG','8094','helmert_transformation','EPSG','1173','EPSG','1323','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1174','NAD27 to WGS 84 (5)','Derived at 129 stations. Accuracy 5m, 5m and 8m in X, Y and Z axes.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4267','EPSG','4326',11.0,-9.0,161.0,179.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-ConusE',0);
INSERT INTO "usage" VALUES('EPSG','8095','helmert_transformation','EPSG','1174','EPSG','2389','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1175','NAD27 to WGS 84 (6)','Derived at 276 stations. Accuracy 5m, 3m and 3m in X, Y and Z axes.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4267','EPSG','4326',7.0,-8.0,159.0,175.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-ConusW',0);
INSERT INTO "usage" VALUES('EPSG','8096','helmert_transformation','EPSG','1175','EPSG','2390','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1176','NAD27 to WGS 84 (7)','Derived at 47 stations. Accuracy 5m, 9m and 5m in X, Y and Z axes.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4267','EPSG','4326',12.0,-5.0,135.0,172.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-USA AK',0);
INSERT INTO "usage" VALUES('EPSG','8097','helmert_transformation','EPSG','1176','EPSG','2412','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1177','NAD27 to WGS 84 (8)','Derived at 11 stations. Accuracy 5m, 3m and 5m in X, Y and Z axes.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4267','EPSG','4326',8.0,-4.0,154.0,178.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Bha xSalv',0);
INSERT INTO "usage" VALUES('EPSG','8098','helmert_transformation','EPSG','1177','EPSG','2413','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1178','NAD27 to WGS 84 (9)','Derived at 1 station. Accuracy 25m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4267','EPSG','4326',44.0,1.0,140.0,165.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Bha Salv',0);
INSERT INTO "usage" VALUES('EPSG','8099','helmert_transformation','EPSG','1178','EPSG','2414','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1179','NAD27 to WGS 84 (10)','Derived at 25 stations. Accuracy 8m, 8m and 6m in X, Y and Z axes.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4267','EPSG','4326',13.0,-7.0,162.0,188.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Can AB BC',0);
INSERT INTO "usage" VALUES('EPSG','8100','helmert_transformation','EPSG','1179','EPSG','2384','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1180','NAD27 to WGS 84 (11)','Derived at 25 stations. Accuracy 9m, 5m and 5m in X, Y and Z axes.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4267','EPSG','4326',12.0,-9.0,157.0,184.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Can MN ON',0);
INSERT INTO "usage" VALUES('EPSG','8101','helmert_transformation','EPSG','1180','EPSG','2415','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1181','NAD27 to WGS 84 (12)','Derived at 37 stations. Accuracy 6m, 6m and 3m in X, Y and Z axes.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4267','EPSG','4326',9.0,-22.0,160.0,190.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Can E',0);
INSERT INTO "usage" VALUES('EPSG','8102','helmert_transformation','EPSG','1181','EPSG','2416','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1182','NAD27 to WGS 84 (13)','Derived at 17 stations. Accuracy 5m, 5m and 3m in X, Y and Z axes.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4267','EPSG','4326',8.0,4.0,159.0,188.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Can NWT',0);
INSERT INTO "usage" VALUES('EPSG','8103','helmert_transformation','EPSG','1182','EPSG','2410','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1183','NAD27 to WGS 84 (14)','Derived at 8 stations. Accuracy 5m, 8m and 3m in X, Y and Z axes.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4267','EPSG','4326',10.0,-7.0,139.0,181.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Can Yuk',0);
INSERT INTO "usage" VALUES('EPSG','8104','helmert_transformation','EPSG','1183','EPSG','2417','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1184','NAD27 to WGS 84 (15)','Derived at 3 stations. Accuracy 20m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4267','EPSG','4326',35.0,0.0,125.0,201.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Pan',0);
INSERT INTO "usage" VALUES('EPSG','8105','helmert_transformation','EPSG','1184','EPSG','2385','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1185','NAD27 to WGS 84 (16)','Derived at 1 station. Accuracy 25m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4267','EPSG','4326',44.0,-9.0,152.0,178.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Cuba',0);
INSERT INTO "usage" VALUES('EPSG','8106','helmert_transformation','EPSG','1185','EPSG','3235','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1186','NAD27 to WGS 84 (17)','Derived at 2 stations. Accuracy 25m in each axis. Note: NAD27 is not used in Greenland.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4267','EPSG','4326',44.0,11.0,114.0,195.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Grl',0);
INSERT INTO "usage" VALUES('EPSG','8107','helmert_transformation','EPSG','1186','EPSG','2386','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1187','NAD27 to WGS 84 (18)','Derived at 22 stations. Accuracy 25m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4267','EPSG','4326',12.0,-12.0,130.0,190.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Mex',0);
INSERT INTO "usage" VALUES('EPSG','8108','helmert_transformation','EPSG','1187','EPSG','3278','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1188','NAD83 to WGS 84 (1)','Derived at 354 stations. Accuracy 2m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4269','EPSG','4326',4.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-N Am',0);
INSERT INTO "usage" VALUES('EPSG','8109','helmert_transformation','EPSG','1188','EPSG','1325','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1189','Nahrwan 1967 to WGS 84 (1)','Derived at 2 stations. Accuracy 25m in each axis. Note: Nahrwan 1967 is not used in Oman.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4270','EPSG','4326',44.0,-247.0,-148.0,369.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Omn Mas',0);
INSERT INTO "usage" VALUES('EPSG','8110','helmert_transformation','EPSG','1189','EPSG','2391','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1190','Nahrwan 1967 to WGS 84 (2)','Derived at 3 stations. Accuracy 20m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4270','EPSG','4326',35.0,-243.0,-192.0,477.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Sau',0);
INSERT INTO "usage" VALUES('EPSG','8111','helmert_transformation','EPSG','1190','EPSG','3968','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1191','Nahrwan 1967 to WGS 84 (3)','Derived at 2 stations. Accuracy 25m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4270','EPSG','4326',44.0,-249.0,-156.0,381.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-UAE',0);
INSERT INTO "usage" VALUES('EPSG','8112','helmert_transformation','EPSG','1191','EPSG','1243','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1192','Naparima 1972 to WGS 84 (1)','CAUTION: IOGP believes that the coordinates used to derive these parameter values include a blunder, leading to an error in the value of tX. If a transformation from DMA is required IOGP recommends use of the 1987 version (EPSG codes 1307 and 1556).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4271','EPSG','4326',33.0,-10.0,375.0,165.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Tto',0);
INSERT INTO "usage" VALUES('EPSG','8113','helmert_transformation','EPSG','1192','EPSG','1322','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1193','NTF to WGS 84 (1)','These same parameter values are used to transform to ETRS89. See NTF to ETRS89 (1) (code 1651).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4275','EPSG','4326',2.0,-168.0,-60.0,320.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Fra',0);
INSERT INTO "usage" VALUES('EPSG','8114','helmert_transformation','EPSG','1193','EPSG','3694','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1194','MGI to WGS 84 (8)','May be taken as approximate transformation MGI to ETRS89 assuming ETRS89 is equivalent to WGS 84 within the accuracy of the transformation - see tfm code 1024. Information source gives scale as -2.388739 ppm.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4312','EPSG','4326',0.5,601.705,84.263,485.227,'EPSG','9001',-4.7354,-1.3145,-5.393,'EPSG','9104',-2.3887,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'LBD-Aut Sty',0);
INSERT INTO "usage" VALUES('EPSG','8115','helmert_transformation','EPSG','1194','EPSG','1543','EPSG','1035');
INSERT INTO "helmert_transformation" VALUES('EPSG','1195','OSGB36 to WGS 84 (1)','Derived at 38 stations. Accuracy 10m, 10m and 15m in X, Y and Z axes.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4277','EPSG','4326',21.0,375.0,-111.0,431.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Gbr',0);
INSERT INTO "usage" VALUES('EPSG','8116','helmert_transformation','EPSG','1195','EPSG','1264','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1196','OSGB36 to WGS 84 (2)','Derived at 24 stations. Accuracy 5m, 5m and 6m in X, Y and Z axes.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4277','EPSG','4326',10.0,371.0,-112.0,434.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Gbr Eng',0);
INSERT INTO "usage" VALUES('EPSG','8117','helmert_transformation','EPSG','1196','EPSG','2395','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1197','OSGB36 to WGS 84 (3)','Derived at 25 stations. Accuracy 10m, 10m and 15m in X, Y and Z axes.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4277','EPSG','4326',21.0,371.0,-111.0,434.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Gbr E&W',0);
INSERT INTO "usage" VALUES('EPSG','8118','helmert_transformation','EPSG','1197','EPSG','2396','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1198','OSGB36 to WGS 84 (4)','Derived at 13 stations. Accuracy 10m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4277','EPSG','4326',18.0,384.0,-111.0,425.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Gbr Sco',0);
INSERT INTO "usage" VALUES('EPSG','8119','helmert_transformation','EPSG','1198','EPSG','2397','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1199','OSGB36 to WGS 84 (5)','Derived at 3 stations. Accuracy 20m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4277','EPSG','4326',35.0,370.0,-108.0,434.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Gbr Wal',0);
INSERT INTO "usage" VALUES('EPSG','8120','helmert_transformation','EPSG','1199','EPSG','2398','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1200','Pointe Noire to WGS 84 (1)','Derived at 1 station. Accuracy 25m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4282','EPSG','4326',44.0,-148.0,51.0,-291.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Cog',0);
INSERT INTO "usage" VALUES('EPSG','8121','helmert_transformation','EPSG','1200','EPSG','1072','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1201','PSAD56 to WGS 84 (1)','Derived at 63 stations. Accuracy 17m, 27m and 27m in X, Y and Z axes. DMA also lists Colombia as area of applicability but PSAD56 is not used in that country.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4248','EPSG','4326',42.0,-288.0,175.0,-376.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-mean',0);
INSERT INTO "usage" VALUES('EPSG','8122','helmert_transformation','EPSG','1201','EPSG','2399','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1202','PSAD56 to WGS 84 (2)','Derived at 5 stations. Accuracy 5m, 11m and 14m in X, Y and Z axes.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4248','EPSG','4326',19.0,-270.0,188.0,-388.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Bol',0);
INSERT INTO "usage" VALUES('EPSG','8123','helmert_transformation','EPSG','1202','EPSG','1049','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1203','PSAD56 to WGS 84 (3)','Derived at 1 station. Accuracy 25m in each axis. Replaced by PSAD56 to WGS 84 (15) (code 6971).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4248','EPSG','4326',44.0,-270.0,183.0,-390.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Chl N',0);
INSERT INTO "usage" VALUES('EPSG','8124','helmert_transformation','EPSG','1203','EPSG','2402','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1204','PSAD56 to WGS 84 (4)','Derived at 3 stations. Accuracy 20m in each axis. Replaced by PSAD56 to WGS 84 (17) (code 6973).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4248','EPSG','4326',35.0,-305.0,243.0,-442.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Chl S',0);
INSERT INTO "usage" VALUES('EPSG','8125','helmert_transformation','EPSG','1204','EPSG','2403','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1205','PSAD56 to WGS 84 (5)','Derived at 4 stations. Accuracy 15m in each axis. Note that although the PSAD56 network included Colombia the CRS is not used there: see Bogota 1975 (CRS code 4218).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4248','EPSG','4326',26.0,-282.0,169.0,-371.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Col',0);
INSERT INTO "usage" VALUES('EPSG','8126','helmert_transformation','EPSG','1205','EPSG','3229','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1206','PSAD56 to WGS 84 (6)','Derived at 11 stations. Accuracy 3m, 5m and 3m in X, Y and Z axes.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4248','EPSG','4326',7.0,-278.0,171.0,-367.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Ecu',0);
INSERT INTO "usage" VALUES('EPSG','8127','helmert_transformation','EPSG','1206','EPSG','3241','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1207','PSAD56 to WGS 84 (7)','Derived at 9 stations. Accuracy 6m, 14m and 5m in X, Y and Z axes.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4248','EPSG','4326',17.0,-298.0,159.0,-369.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Guy',0);
INSERT INTO "usage" VALUES('EPSG','8128','helmert_transformation','EPSG','1207','EPSG','1114','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1208','PSAD56 to WGS 84 (8)','Derived at 6 stations. Accuracy 6m, 8m and 12m in X, Y and Z axes.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4248','EPSG','4326',16.0,-279.0,175.0,-379.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Per',0);
INSERT INTO "usage" VALUES('EPSG','8129','helmert_transformation','EPSG','1208','EPSG','3292','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1209','PSAD56 to WGS 84 (9)','Derived at 24 stations. Accuracy 9m, 14m and 15m in X, Y and Z axes.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4248','EPSG','4326',23.0,-295.0,173.0,-371.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Ven',0);
INSERT INTO "usage" VALUES('EPSG','8130','helmert_transformation','EPSG','1209','EPSG','3327','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1210','POSGAR 94 to WGS 84 (1)','Approximation at the +/- 1m level assuming that POSGAR 94 is equivalent to WGS 84.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4694','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Arg',0);
INSERT INTO "usage" VALUES('EPSG','8131','helmert_transformation','EPSG','1210','EPSG','1033','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1211','Qornoq to WGS 84 (1)','Derived at 2 stations.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4287','EPSG','4326',NULL,164.0,138.0,-189.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Grl S',1);
INSERT INTO "usage" VALUES('EPSG','8132','helmert_transformation','EPSG','1211','EPSG','2407','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1212','SAD69 to WGS 84 (1)','Derived at 84 stations.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4291','EPSG','4326',NULL,-57.0,1.0,-41.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-mean',1);
INSERT INTO "usage" VALUES('EPSG','8133','helmert_transformation','EPSG','1212','EPSG','1341','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1213','SAD69 to WGS 84 (2)','Derived at 10 stations.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4291','EPSG','4326',NULL,-62.0,-1.0,-37.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Arg',1);
INSERT INTO "usage" VALUES('EPSG','8134','helmert_transformation','EPSG','1213','EPSG','1033','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1214','SAD69 to WGS 84 (3)','Derived at 4 stations.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4291','EPSG','4326',NULL,-61.0,2.0,-48.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Bol',1);
INSERT INTO "usage" VALUES('EPSG','8135','helmert_transformation','EPSG','1214','EPSG','1049','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1215','SAD69 to WGS 84 (4)','Derived at 22 stations.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4291','EPSG','4326',NULL,-60.0,-2.0,-41.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Bra',1);
INSERT INTO "usage" VALUES('EPSG','8136','helmert_transformation','EPSG','1215','EPSG','1053','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1216','SAD69 to WGS 84 (5)','Derived at 9 stations.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4291','EPSG','4326',NULL,-75.0,-1.0,-44.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Chile',1);
INSERT INTO "usage" VALUES('EPSG','8137','helmert_transformation','EPSG','1216','EPSG','1066','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1217','SAD69 to WGS 84 (6)','Derived at 7 stations.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4291','EPSG','4326',NULL,-44.0,6.0,-36.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Col',1);
INSERT INTO "usage" VALUES('EPSG','8138','helmert_transformation','EPSG','1217','EPSG','1070','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1218','SAD69 to WGS 84 (7)','Derived at 11 stations.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4291','EPSG','4326',NULL,-48.0,3.0,-44.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Ecu',1);
INSERT INTO "usage" VALUES('EPSG','8139','helmert_transformation','EPSG','1218','EPSG','1085','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1219','SAD69 to WGS 84 (8)','Derived at 1 station.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4291','EPSG','4326',NULL,-47.0,26.0,-42.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Ecu Gal',1);
INSERT INTO "usage" VALUES('EPSG','8140','helmert_transformation','EPSG','1219','EPSG','2356','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1220','SAD69 to WGS 84 (9)','Derived at 5 stations.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4291','EPSG','4326',NULL,-53.0,3.0,-47.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Guy',1);
INSERT INTO "usage" VALUES('EPSG','8141','helmert_transformation','EPSG','1220','EPSG','1114','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1221','SAD69 to WGS 84 (10)','Derived at 4 stations.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4291','EPSG','4326',NULL,-61.0,2.0,-33.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Pgy',1);
INSERT INTO "usage" VALUES('EPSG','8142','helmert_transformation','EPSG','1221','EPSG','1188','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1222','SAD69 to WGS 84 (11)','Derived at 6 stations.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4291','EPSG','4326',NULL,-58.0,0.0,-44.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Peru',1);
INSERT INTO "usage" VALUES('EPSG','8143','helmert_transformation','EPSG','1222','EPSG','1189','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1223','SAD69 to WGS 84 (12)','Derived at 1 station.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4291','EPSG','4326',NULL,-45.0,12.0,-33.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Tto',1);
INSERT INTO "usage" VALUES('EPSG','8144','helmert_transformation','EPSG','1223','EPSG','1235','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1224','SAD69 to WGS 84 (13)','Derived at 5 stations.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4291','EPSG','4326',NULL,-45.0,8.0,-33.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Ven',1);
INSERT INTO "usage" VALUES('EPSG','8145','helmert_transformation','EPSG','1224','EPSG','1251','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1225','Sapper Hill 1943 to WGS 84 (1)','Derived at 5 stations. Accuracy 1m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4292','EPSG','4326',2.0,-355.0,21.0,72.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Flk E',0);
INSERT INTO "usage" VALUES('EPSG','8146','helmert_transformation','EPSG','1225','EPSG','2355','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1226','Schwarzeck to WGS 84 (1)','Derived at 3 stations. Accuracy 20m in each axis. Beware! Source CRS uses German legal metres, transformation parameter values are in (International) metres. See tfm code 1271 for example.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4293','EPSG','4326',35.0,616.0,97.0,-251.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Nam',0);
INSERT INTO "usage" VALUES('EPSG','8147','helmert_transformation','EPSG','1226','EPSG','1169','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1227','Tananarive to WGS 84 (1)','Accuracy estimate not available.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4297','EPSG','4326',999.0,-189.0,-242.0,-91.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Mdg',0);
INSERT INTO "usage" VALUES('EPSG','8148','helmert_transformation','EPSG','1227','EPSG','1149','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1228','Timbalai 1948 to WGS 84 (1)','Derived at 8 stations. Accuracy 10m, 10m and 12m in X, Y and Z axes.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4298','EPSG','4326',19.0,-679.0,669.0,-48.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Borneo',0);
INSERT INTO "usage" VALUES('EPSG','8149','helmert_transformation','EPSG','1228','EPSG','1362','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1229','TM65 to WGS 84 (1)','Derived at 7 stations.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4299','EPSG','4326',NULL,506.0,-122.0,611.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Ire',1);
INSERT INTO "usage" VALUES('EPSG','8150','helmert_transformation','EPSG','1229','EPSG','1305','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1230','Tokyo to WGS 84 (1)','Derived at 31 stations. Accuracy 20m, 5m and 20m in X, Y and Z axes.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4301','EPSG','4326',29.0,-148.0,507.0,685.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Jpn Kor',0);
INSERT INTO "usage" VALUES('EPSG','8151','helmert_transformation','EPSG','1230','EPSG','2409','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1231','Tokyo to WGS 84 (2)','Derived at 16 stations. Accuracy 8m, 5m and 8m in X, Y and Z axes.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4301','EPSG','4326',13.0,-148.0,507.0,685.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Jpn',0);
INSERT INTO "usage" VALUES('EPSG','8152','helmert_transformation','EPSG','1231','EPSG','3995','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1232','Tokyo to WGS 84 (3)','Derived at 29 stations. Accuracy 8m, 5m and 8m in X, Y and Z axes. Replaced by Tokyo to WGS 84 (5) (code 1305).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4301','EPSG','4326',13.0,-146.0,507.0,687.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Kor',0);
INSERT INTO "usage" VALUES('EPSG','8153','helmert_transformation','EPSG','1232','EPSG','3266','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1233','Tokyo to WGS 84 (4)','Derived at 3 stations. Accuracy 20m, 5m and 20m in X, Y and Z axes.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4301','EPSG','4326',29.0,-158.0,507.0,676.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Jpn Ok',0);
INSERT INTO "usage" VALUES('EPSG','8154','helmert_transformation','EPSG','1233','EPSG','2408','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1234','Yacare to WGS 84 (1)','Accuracy estimate not available.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4309','EPSG','4326',999.0,-155.0,171.0,37.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Ury',0);
INSERT INTO "usage" VALUES('EPSG','8155','helmert_transformation','EPSG','1234','EPSG','3326','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1235','Zanderij to WGS 84 (1)','Derived at 5 stations. Accuracy 5m, 5m and 8m in X, Y and Z axes.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4311','EPSG','4326',11.0,-265.0,120.0,-358.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Sur',0);
INSERT INTO "usage" VALUES('EPSG','8156','helmert_transformation','EPSG','1235','EPSG','1222','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1236','AGD84 to WGS 84 (2)','"Higgins parameters". Replaced by AGD84 to GDA94 (2) (code 1280) and AGD84 to WGS 84 (7) (code 1669). Note: AGD84 officially adopted only in Queensland, South Australia and Western Australia.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4203','EPSG','4326',5.0,-116.0,-50.47,141.69,'EPSG','9001',-0.23,-0.39,-0.344,'EPSG','9104',0.0983,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'Auslig-Aus old',0);
INSERT INTO "usage" VALUES('EPSG','8157','helmert_transformation','EPSG','1236','EPSG','2576','EPSG','1204');
INSERT INTO "helmert_transformation" VALUES('EPSG','1237','WGS 72 to WGS 84 (1)','','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4322','EPSG','4326',2.0,0.0,0.0,4.5,'EPSG','9001',0.0,0.0,0.554,'EPSG','9104',0.2263,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA1',0);
INSERT INTO "usage" VALUES('EPSG','8158','helmert_transformation','EPSG','1237','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','1238','WGS 72 to WGS 84 (2)','','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4322','EPSG','4326',2.0,0.0,0.0,4.5,'EPSG','9001',0.0,0.0,0.554,'EPSG','9104',0.219,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA2',0);
INSERT INTO "usage" VALUES('EPSG','8159','helmert_transformation','EPSG','1238','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','1239','WGS 72BE to WGS 72 (1)','','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4324','EPSG','4322',2.0,0.0,0.0,-2.6,'EPSG','9001',0.0,0.0,0.26,'EPSG','9104',-0.6063,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA',0);
INSERT INTO "usage" VALUES('EPSG','8160','helmert_transformation','EPSG','1239','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','1240','WGS 72BE to WGS 84 (1)','','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4324','EPSG','4326',2.0,0.0,0.0,1.9,'EPSG','9001',0.0,0.0,0.814,'EPSG','9104',-0.38,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA',0);
INSERT INTO "usage" VALUES('EPSG','8161','helmert_transformation','EPSG','1240','EPSG','1262','EPSG','1027');
INSERT INTO "usage" VALUES('EPSG','21889','helmert_transformation','EPSG','1240','EPSG','4780','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1242','HD72 to WGS 84 (4)','Parameter value error in info source Hungarian text but correct in English summary. Replaces HD72 to WGS 84 (2) (code 1831).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4237','EPSG','4326',1.0,52.17,-71.82,-14.9,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'ELTE-Hun 2004',0);
INSERT INTO "usage" VALUES('EPSG','8163','helmert_transformation','EPSG','1242','EPSG','1119','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','1244','PZ-90 to WGS 84 (2)','Mandated for use in Russia by GosStandard of Russia Decree #327 of August 9, 2001. Republished but with one significant figure less precision to parameter values in GOST R 51794-2008 of December 18 2008.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4740','EPSG','4326',0.5,-1.08,-0.27,-0.9,'EPSG','9001',0.0,0.0,-0.16,'EPSG','9104',-0.12,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'GOST-Rus',0);
INSERT INTO "usage" VALUES('EPSG','8165','helmert_transformation','EPSG','1244','EPSG','1198','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','1245','ED50 to WGS 84 (16)','Derived at 4 stations. Accuracy 25m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4230','EPSG','4326',44.0,-112.0,-77.0,-145.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Tun',0);
INSERT INTO "usage" VALUES('EPSG','8166','helmert_transformation','EPSG','1245','EPSG','1236','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1246','Herat North to WGS 84 (1)','Accuracy estimate not available.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4255','EPSG','4326',999.0,-333.0,-222.0,114.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Afg',0);
INSERT INTO "usage" VALUES('EPSG','8167','helmert_transformation','EPSG','1246','EPSG','1024','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1247','Kalianpur 1962 to WGS 84 (1)','Care! DMA ellipsoid is inconsistent with EPSG ellipsoid - transformation parameter values may not be appropriate. No accuracy estimate available.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4145','EPSG','4326',999.0,283.0,682.0,231.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Pak',0);
INSERT INTO "usage" VALUES('EPSG','8168','helmert_transformation','EPSG','1247','EPSG','3289','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1248','ID74 to WGS 84 (1)','Derived at 1 station. Accuracy 25m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4238','EPSG','4326',44.0,-24.0,-15.0,5.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Idn',0);
INSERT INTO "usage" VALUES('EPSG','8169','helmert_transformation','EPSG','1248','EPSG','4020','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1249','NAD27 to WGS 84 (21)','Derived at 6 stations. Accuracy 6m, 8m and 10m in X, Y and Z axes.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4267','EPSG','4326',15.0,-2.0,152.0,149.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-AK AluE',0);
INSERT INTO "usage" VALUES('EPSG','8170','helmert_transformation','EPSG','1249','EPSG','2387','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1250','NAD27 to WGS 84 (22)','Derived at 5 stations. Accuracy 10m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4267','EPSG','4326',18.0,2.0,204.0,105.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-AK AluW',0);
INSERT INTO "usage" VALUES('EPSG','8171','helmert_transformation','EPSG','1250','EPSG','2388','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1251','NAD83 to WGS 84 (2)','Derived at 4 stations. Accuracy 5m, 2m and 5m in X, Y and Z axes.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4269','EPSG','4326',8.0,-2.0,0.0,4.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-AK Alu',0);
INSERT INTO "usage" VALUES('EPSG','8172','helmert_transformation','EPSG','1251','EPSG','2157','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1252','NAD83 to WGS 84 (3)','Derived at 6 stations. Accuracy 2m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4269','EPSG','4326',4.0,1.0,1.0,-1.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-USA Hi',0);
INSERT INTO "usage" VALUES('EPSG','8173','helmert_transformation','EPSG','1252','EPSG','3883','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1253','Nord Sahara 1959 to WGS 84 (1)','Derived at 3 stations. Accuracy 25m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4307','EPSG','4326',44.0,-186.0,-93.0,310.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Alg',0);
INSERT INTO "usage" VALUES('EPSG','8174','helmert_transformation','EPSG','1253','EPSG','3213','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1254','Pulkovo 1942 to WGS 84 (1)','Accuracy estimate not available.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4284','EPSG','4326',999.0,28.0,-130.0,-95.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Rus',0);
INSERT INTO "usage" VALUES('EPSG','8175','helmert_transformation','EPSG','1254','EPSG','3296','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1255','Nord Sahara 1959 to WGS 84 (2)','Derived at 2 stations. Accuracy 25m in each axis. CAUTION: DMA describe Source CRS as Voirol 1960. OGP believes that the data used in the derivation of these parameters contains a blunder. We recommend using CT North Sahara 1959 to WGS84 (1) (code 1253).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4307','EPSG','4326',44.0,-123.0,-206.0,219.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Dza N',0);
INSERT INTO "usage" VALUES('EPSG','8176','helmert_transformation','EPSG','1255','EPSG','1365','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1256','Fahud to WGS 84 (1)','Derived at 7 stations. Accuracy 3m, 3m and 9m in X, Y and Z axes. Replaced by Fahud to WGS 84 (3) (code 6908).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4232','EPSG','4326',10.0,-346.0,-1.0,224.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Omn',0);
INSERT INTO "usage" VALUES('EPSG','8177','helmert_transformation','EPSG','1256','EPSG','4009','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1257','Pulkovo 1995 to PZ-90 (1)','Mandated for use in Russia by GosStandard of Russia Decree #327 of August 9, 2001.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4200','EPSG','4740',1.0,25.9,-130.94,-81.76,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'GOST-Rus',0);
INSERT INTO "usage" VALUES('EPSG','8178','helmert_transformation','EPSG','1257','EPSG','1198','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','1267','Pulkovo 1942 to WGS 84 (17)','Derived through concatenation of Pulkovo 1942 to PZ-90 (1) (tfm code 15844) and PZ-90 to WGS 84 (2) (tfm code 1244. Mandated for use in Russia by GOST R 51794-2001, but this has been superseded by GOST R 51794-2008. Replaced by tfm code 5044.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4284','EPSG','4326',4.0,23.92,-141.27,-80.9,'EPSG','9001',0.0,-0.35,-0.82,'EPSG','9104',-0.12,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'GOST-Rus',0);
INSERT INTO "usage" VALUES('EPSG','8188','helmert_transformation','EPSG','1267','EPSG','3296','EPSG','1044');
INSERT INTO "helmert_transformation" VALUES('EPSG','1271','Schwarzeck to WGS 84 (2)','Beware! Source CRS uses GLM, tfm param in m. Example: Schwarzeck φ=19°35''46.952"S λ=20°41''50.649"E h=1185.99m; X=5623409.386 Y=2124618.003 Z=-2125847.632 GLM; X=5623485.84m Y=2124646.89m Z=-2125876.54m; WGS 84 X=5624101.48m Y=2124748.97m Z=-2126132.35m.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4293','EPSG','4326',999.0,615.64,102.08,-255.81,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'SLI-Nam',0);
INSERT INTO "usage" VALUES('EPSG','8192','helmert_transformation','EPSG','1271','EPSG','1169','EPSG','1024');
INSERT INTO "helmert_transformation" VALUES('EPSG','1272','GGRS87 to WGS 84 (1)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4121','EPSG','4326',1.0,-199.87,74.79,246.62,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'Hel-Grc',0);
INSERT INTO "usage" VALUES('EPSG','8193','helmert_transformation','EPSG','1272','EPSG','3254','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','1273','HD72 to ETRS89 (1)','May be taken as approximate transformation HD72 to WGS 84 - see code 1677.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4237','EPSG','4258',NULL,-56.0,75.77,15.31,'EPSG','9001',-0.37,-0.2,-0.21,'EPSG','9104',-1.01,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'ELTE-Hun',1);
INSERT INTO "usage" VALUES('EPSG','8194','helmert_transformation','EPSG','1273','EPSG','1119','EPSG','1025');
INSERT INTO "helmert_transformation" VALUES('EPSG','1274','Pulkovo 1942 to LKS94 (1)','May be taken as approximate transformation Pulkovo 1942 to WGS 84 - see code 1679.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4284','EPSG','4669',9.0,-40.595,-18.55,-69.339,'EPSG','9001',-2.508,-1.832,2.611,'EPSG','9104',-4.299,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'HNIT-Ltu',0);
INSERT INTO "usage" VALUES('EPSG','8195','helmert_transformation','EPSG','1274','EPSG','3272','EPSG','1049');
INSERT INTO "helmert_transformation" VALUES('EPSG','1275','ED50 to WGS 84 (17)','These same parameter values are used to transform to ETRS89. See ED50 to ETRS89 (10) (code 1650).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4230','EPSG','4326',2.0,-84.0,-97.0,-117.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Fra',0);
INSERT INTO "usage" VALUES('EPSG','8196','helmert_transformation','EPSG','1275','EPSG','1096','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1276','NTF to ED50 (1)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4275','EPSG','4230',2.0,-84.0,37.0,437.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Fra',0);
INSERT INTO "usage" VALUES('EPSG','8197','helmert_transformation','EPSG','1276','EPSG','3694','EPSG','1042');
INSERT INTO "helmert_transformation" VALUES('EPSG','1277','NTF to WGS 72 (1)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4275','EPSG','4322',2.0,-168.0,-72.0,314.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Fra',0);
INSERT INTO "usage" VALUES('EPSG','8198','helmert_transformation','EPSG','1277','EPSG','3694','EPSG','1042');
INSERT INTO "helmert_transformation" VALUES('EPSG','1278','AGD66 to GDA94 (1)','Given to greater precision but no better accuracy at http://www.dehaa.sa.gov.au For higher accuracy requirements see various regional transformations. May be taken as approximate transformation AGD66 to WGS 84 - see code 15788. Derived at 162 stations.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4202','EPSG','4283',5.0,-127.8,-52.3,152.9,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'Auslig-Aus 5m',0);
INSERT INTO "usage" VALUES('EPSG','8199','helmert_transformation','EPSG','1278','EPSG','2575','EPSG','1045');
INSERT INTO "helmert_transformation" VALUES('EPSG','1279','AGD84 to GDA94 (1)','Derived at 327 stations. May be taken as approximate transformation AGD84 to WGS 84 - see code 15789. For higher accuracy use AGD84 to GDA94 (2) (code 1280). Note: AGD84 officially adopted only in Queensland, South Australia and Western Australia.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4203','EPSG','4283',5.0,-128.5,-53.0,153.4,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'Auslig-Aus 5m',0);
INSERT INTO "usage" VALUES('EPSG','8200','helmert_transformation','EPSG','1279','EPSG','2576','EPSG','1045');
INSERT INTO "helmert_transformation" VALUES('EPSG','1280','AGD84 to GDA94 (2)','Replaces AGD84 to WGS 84 (2) (code 1236). May be taken as approximate transformation AGD84 to WGS 84 - see code 1669. Note: although applicable nationwide, AGD84 officially adopted only in Queensland, South Australia and Western Australia.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4203','EPSG','4283',1.0,-117.763,-51.51,139.061,'EPSG','9001',-0.292,-0.443,-0.277,'EPSG','9104',-0.191,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'Auslig-Aus 1m',0);
INSERT INTO "usage" VALUES('EPSG','14198','helmert_transformation','EPSG','1280','EPSG','2576','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','1281','Pulkovo 1995 to WGS 84 (1)','Derived through concatenation of Pulkovo 1995 to PZ-90 (1) (tfm code 1257) and PZ-90 to WGS 84 (2) (tfm code 1244). Mandated for use in Russia by GOST R 51794-2001, but this has been superseded by GOST R 51794-2008. Replaced by tfm code 5043.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4200','EPSG','4326',1.0,24.82,-131.21,-82.66,'EPSG','9001',0.0,0.0,-0.16,'EPSG','9104',-0.12,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'GOST-Rus',0);
INSERT INTO "usage" VALUES('EPSG','8202','helmert_transformation','EPSG','1281','EPSG','1198','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','1282','Samboja to WGS 84 (1)','Datum shift derived through ITRF93.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4125','EPSG','4326',NULL,-404.78,685.68,45.47,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'TOT-Idn Mah',1);
INSERT INTO "usage" VALUES('EPSG','8203','helmert_transformation','EPSG','1282','EPSG','1328','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1283','LKS94 to WGS 84 (1)','LKS94 is a national realization of ETRS89 and coincident to WGS 84 within 1 metre. This transformation has an accuracy equal to the coincidence figure.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4669','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'HNIT-Ltu',0);
INSERT INTO "usage" VALUES('EPSG','8204','helmert_transformation','EPSG','1283','EPSG','1145','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1284','Arc 1960 to WGS 84 (2)','Derived at 24 stations. Accuracy 4m, 3m and 3m in X, Y and Z axes.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4210','EPSG','4326',6.0,-157.0,-2.0,-299.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NIMA-Ken',0);
INSERT INTO "usage" VALUES('EPSG','8205','helmert_transformation','EPSG','1284','EPSG','3264','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1285','Arc 1960 to WGS 84 (3)','Derived at 12 stations. Accuracy 6m, 9m and 10m in X, Y and Z axes.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4210','EPSG','4326',15.0,-175.0,-23.0,-303.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NIMA-Tza',0);
INSERT INTO "usage" VALUES('EPSG','8206','helmert_transformation','EPSG','1285','EPSG','3316','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1286','Segora to WGS 84 (1)','Accuracy estimate not available.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4294','EPSG','4326',NULL,-403.0,684.0,41.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NIMA-Idn Kal',1);
INSERT INTO "usage" VALUES('EPSG','8207','helmert_transformation','EPSG','1286','EPSG','2354','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1287','Pulkovo 1942 to WGS 84 (3)','Derived at 5 stations.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4284','EPSG','4326',4.0,28.0,-121.0,-77.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NIMA-Hun',1);
INSERT INTO "usage" VALUES('EPSG','8208','helmert_transformation','EPSG','1287','EPSG','1119','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1288','Pulkovo 1942 to WGS 84 (4)','Derived at 11 stations.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4284','EPSG','4326',6.0,23.0,-124.0,-82.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NIMA-Pol',1);
INSERT INTO "usage" VALUES('EPSG','8209','helmert_transformation','EPSG','1288','EPSG','1192','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1289','Pulkovo 1942 to WGS 84 (5)','Derived at 6 stations.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4284','EPSG','4326',5.0,26.0,-121.0,-78.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NIMA-Cze',1);
INSERT INTO "usage" VALUES('EPSG','8210','helmert_transformation','EPSG','1289','EPSG','1306','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1290','Pulkovo 1942 to WGS 84 (6)','Derived at 5 stations.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4284','EPSG','4326',4.0,24.0,-124.0,-82.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NIMA-Lva',0);
INSERT INTO "usage" VALUES('EPSG','8211','helmert_transformation','EPSG','1290','EPSG','3268','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1291','Pulkovo 1942 to WGS 84 (7)','Derived at 2 stations.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4284','EPSG','4326',44.0,15.0,-130.0,-84.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NIMA-Kaz',0);
INSERT INTO "usage" VALUES('EPSG','8212','helmert_transformation','EPSG','1291','EPSG','1131','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1292','Pulkovo 1942 to WGS 84 (8)','Derived at 7 stations.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4284','EPSG','4326',6.0,24.0,-130.0,-92.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NIMA-Alb',1);
INSERT INTO "usage" VALUES('EPSG','8213','helmert_transformation','EPSG','1292','EPSG','1025','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1293','Pulkovo 1942 to WGS 84 (9)','Derived at 4 stations.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4284','EPSG','4326',7.0,28.0,-121.0,-77.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NIMA-Rom',1);
INSERT INTO "usage" VALUES('EPSG','8214','helmert_transformation','EPSG','1293','EPSG','1197','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1294','Voirol 1875 to WGS 84 (1)','Accuracy estimate not available.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4304','EPSG','4326',999.0,-73.0,-247.0,227.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NIMA-Dza N',0);
INSERT INTO "usage" VALUES('EPSG','8215','helmert_transformation','EPSG','1294','EPSG','1365','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1296','Trinidad 1903 to WGS 84 (1)','Derived in 1989 by ONI for Amoco.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4302','EPSG','4326',2.0,-61.702,284.488,472.052,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'Amoco-Tto Trin',0);
INSERT INTO "usage" VALUES('EPSG','8217','helmert_transformation','EPSG','1296','EPSG','1339','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1297','Tete to Moznet (1)','Mean of 32 stations. Residuals as high as 30 metres. To reduce the size of the residuals; four regional parameter sets (see codes 1298-1301) were developed. May be taken as approximate transformation Tete to WGS 84 - see code 1683.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4127','EPSG','4130',30.0,-115.064,-87.39,-101.716,'EPSG','9001',0.058,-4.001,2.062,'EPSG','9104',9.366,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DNGC-Moz',0);
INSERT INTO "usage" VALUES('EPSG','8218','helmert_transformation','EPSG','1297','EPSG','3281','EPSG','1157');
INSERT INTO "helmert_transformation" VALUES('EPSG','1298','Tete to Moznet (2)','Mean of 9 stations. Residuals are generally under 1 metre. May be taken as approximate transformation Tete to WGS 84 - see code 1684.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4127','EPSG','4130',1.0,-82.875,-57.097,-156.768,'EPSG','9001',2.158,-1.524,0.982,'EPSG','9104',-0.359,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DNGC-Moz A',0);
INSERT INTO "usage" VALUES('EPSG','8219','helmert_transformation','EPSG','1298','EPSG','2350','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','1299','Tete to Moznet (3)','Mean of 6 stations. Residuals are generally under 4 metres. May be taken as approximate transformation Tete to WGS 84 - see code 1685.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4127','EPSG','4130',4.0,-138.527,-91.999,-114.591,'EPSG','9001',0.14,-3.363,2.217,'EPSG','9104',11.748,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DNGC-Moz B',0);
INSERT INTO "usage" VALUES('EPSG','8220','helmert_transformation','EPSG','1299','EPSG','2351','EPSG','1044');
INSERT INTO "helmert_transformation" VALUES('EPSG','1300','Tete to Moznet (4)','Mean of 11 stations. Residuals are generally under 3 metres. May be taken as approximate transformation Tete to WGS 84 - see code 1686.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4127','EPSG','4130',3.0,-73.472,-51.66,-112.482,'EPSG','9001',-0.953,-4.6,2.368,'EPSG','9104',0.586,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DNGC-Moz C',0);
INSERT INTO "usage" VALUES('EPSG','8221','helmert_transformation','EPSG','1300','EPSG','2352','EPSG','1043');
INSERT INTO "helmert_transformation" VALUES('EPSG','1301','Tete to Moznet (5)','Mean of 7 stations. Residuals are 5-10 metres. May be taken as approximate transformation Tete to WGS 84 - see code 1687.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4127','EPSG','4130',10.0,219.315,168.975,-166.145,'EPSG','9001',-0.198,-5.926,2.356,'EPSG','9104',-57.104,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DNGC-Moz D',0);
INSERT INTO "usage" VALUES('EPSG','8222','helmert_transformation','EPSG','1301','EPSG','2353','EPSG','1050');
INSERT INTO "helmert_transformation" VALUES('EPSG','1302','Moznet to WGS 84 (1)','For many purposes Moznet can be considered to be coincident with WGS 84. Accuracy better than 1 metre.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4130','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',0.0,0.0,0.0,'EPSG','9104',0.0,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Moz',0);
INSERT INTO "usage" VALUES('EPSG','8223','helmert_transformation','EPSG','1302','EPSG','1167','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1303','Pulkovo 1942 to WGS 84 (10)','Mean of 13 stations along entire Kazak coastline.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4284','EPSG','4326',2.0,43.822,-108.842,-119.585,'EPSG','9001',1.455,-0.761,0.737,'EPSG','9104',0.549,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'KCS-Kaz Cas',0);
INSERT INTO "usage" VALUES('EPSG','8224','helmert_transformation','EPSG','1303','EPSG','2405','EPSG','1042');
INSERT INTO "helmert_transformation" VALUES('EPSG','1304','Indian 1975 to WGS 84 (2)','Derived at 62 stations. Accuracy 3m, 2m and 3m in X, Y and Z axes. Replaces Indian 1975 to WGS 84 (1) (code 1154).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4240','EPSG','4326',5.0,210.0,814.0,289.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NIMA-Tha',0);
INSERT INTO "usage" VALUES('EPSG','8225','helmert_transformation','EPSG','1304','EPSG','3741','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1305','Tokyo to WGS 84 (5)','Derived at 29 stations. Accuracy 2m in each axis. Replaces Tokyo to WGS 84 (3) (code 1232).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4301','EPSG','4326',4.0,-147.0,506.0,687.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NIMA-Kor',0);
INSERT INTO "usage" VALUES('EPSG','8226','helmert_transformation','EPSG','1305','EPSG','3266','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1306','MGI to WGS 84 (1)','Accuracy estimate not available.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4312','EPSG','4326',999.0,682.0,-203.0,480.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NIMA-balk',1);
INSERT INTO "usage" VALUES('EPSG','8227','helmert_transformation','EPSG','1306','EPSG','2370','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1307','Naparima 1972 to WGS 84 (3)','DMA does not differentiate between Naparima 1955 (Trinidad) and Naparima 1972 (Tobago). Consequently for Trinidad IOGP has duplicated this transformation as Naparima 1955 to WGS 84 (3) - see code 1556.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4271','EPSG','4326',26.0,-2.0,374.0,172.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NIMA-Tto Tob',0);
INSERT INTO "usage" VALUES('EPSG','8228','helmert_transformation','EPSG','1307','EPSG','1322','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1308','NAD83 to WGS 84 (4)','Strictly between NAD83 and ITRF94(1996.0). Superseded by NAD83 to WGS 84 (5) (code 1515).','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4269','EPSG','4326',NULL,-0.9738,1.9453,0.5486,'EPSG','9001',-1.3357e-07,-4.872e-08,-5.507e-08,'EPSG','9101',0.0,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NGS-Usa ITRF94',1);
INSERT INTO "usage" VALUES('EPSG','8229','helmert_transformation','EPSG','1308','EPSG','1323','EPSG','1197');
INSERT INTO "helmert_transformation" VALUES('EPSG','1309','DHDN to ETRS89 (1)','Mean of 69 stations. May be taken as approximate tfm DHDN to WGS 84 (code 1673). Replaced by DHDN to ETRS89 (2) (tfm code 1776) and regional higher accuracy tfms. Note: these later tfms have been published using the Position Vector method.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4314','EPSG','4258',5.0,582.0,105.0,414.0,'EPSG','9001',-1.04,-0.35,3.08,'EPSG','9104',8.3,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IfAG-Deu W',0);
INSERT INTO "usage" VALUES('EPSG','8230','helmert_transformation','EPSG','1309','EPSG','2326','EPSG','1045');
INSERT INTO "helmert_transformation" VALUES('EPSG','1310','Pulkovo 1942 to ETRS89 (1)','Mean of 20 stations.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4284','EPSG','4258',2.0,24.0,-123.0,-94.0,'EPSG','9001',-0.02,0.25,0.13,'EPSG','9104',1.1,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IfAG-Deu E',1);
INSERT INTO "usage" VALUES('EPSG','8231','helmert_transformation','EPSG','1310','EPSG','1343','EPSG','1042');
INSERT INTO "helmert_transformation" VALUES('EPSG','1311','ED50 to WGS 84 (18)','Recommended CT for UKCS and IrishCS petroleum purposes, known as "Common Offshore". Based on ED50 to WGS 72 (precise ephemeris) 6-nations agreement of 1981 with which precise to broadcast and broadcast to WGS 84 transformations have been concatenated.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4230','EPSG','4326',1.0,-89.5,-93.8,-123.1,'EPSG','9001',0.0,0.0,-0.156,'EPSG','9104',1.2,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'UKOOA-CO',0);
INSERT INTO "usage" VALUES('EPSG','8232','helmert_transformation','EPSG','1311','EPSG','2342','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1314','OSGB36 to WGS 84 (6)','Commonly referred to as the "OSGB Petroleum transformation". For a more accurate transformation see ETRS89 to OSGB36 / British National Grid (3) (code 7953).','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4277','EPSG','4326',2.0,446.448,-125.157,542.06,'EPSG','9001',0.15,0.247,0.842,'EPSG','9104',-20.489,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'UKOOA-Pet',0);
INSERT INTO "usage" VALUES('EPSG','8235','helmert_transformation','EPSG','1314','EPSG','1264','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1315','OSGB36 to ED50 (1)','Known as the "UKOOA landward/seaward transformation". This transformation is concatenated from OSGB36 to WGS 84 (6) (Petroleum) (code 1314) minus ED50 to WGS 84 (18) (Common Offshore) (code 1311). Accuracy better than 4m and generally better than 2m.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4277','EPSG','4230',2.0,535.948,-31.357,665.16,'EPSG','9001',0.15,0.247,0.998,'EPSG','9104',-21.689,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'UKOOA-UKCS',0);
INSERT INTO "usage" VALUES('EPSG','8236','helmert_transformation','EPSG','1315','EPSG','1264','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1316','Manoca to WGS 84 (1)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4260','EPSG','4326',999.0,-70.9,-151.8,-41.4,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'SCS-Cmr',1);
INSERT INTO "usage" VALUES('EPSG','8237','helmert_transformation','EPSG','1316','EPSG','1060','EPSG','1025');
INSERT INTO "helmert_transformation" VALUES('EPSG','1317','Camacupa 1948 to WGS 72BE (1)','Derived by Geophysical Services Inc. in 1979 from mean of Transit results at 7 stations.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4220','EPSG','4324',10.0,-37.2,-370.6,-228.5,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'GSI-Ago',0);
INSERT INTO "usage" VALUES('EPSG','8238','helmert_transformation','EPSG','1317','EPSG','1604','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1318','Camacupa 1948 to WGS 84 (1)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4220','EPSG','4326',10.0,-42.01,-332.21,-229.75,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'CON-Ago B5',0);
INSERT INTO "usage" VALUES('EPSG','8239','helmert_transformation','EPSG','1318','EPSG','2316','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1319','Camacupa 1948 to WGS 84 (2)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4220','EPSG','4326',25.0,-40.0,-354.0,-224.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'TEX-Ago B2',0);
INSERT INTO "usage" VALUES('EPSG','8240','helmert_transformation','EPSG','1319','EPSG','2317','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1320','Camacupa 1948 to WGS 84 (3)','Replaced by Camacupa 1948 to WGS 84 (9). Used by Shell prior to 1994.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4220','EPSG','4326',10.0,-37.2,-370.6,-224.0,'EPSG','9001',0.0,0.0,0.554,'EPSG','9104',0.219,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'SHL-Ago old',0);
INSERT INTO "usage" VALUES('EPSG','8241','helmert_transformation','EPSG','1320','EPSG','2321','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1321','Camacupa 1948 to WGS 84 (4)','Mean of 123 Transit passes at station Cabo Ledo NE base in November 1990. Used by Elf for block 7 up to December 1992 then replaced by Camacupa 1948 to WGS 84 (7). Used by Total in block 8, ExxonMobil block 24, Western Geophysical for spec. data.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4220','EPSG','4326',10.0,-41.8,-342.2,-228.2,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'GSI-Ago',0);
INSERT INTO "usage" VALUES('EPSG','8242','helmert_transformation','EPSG','1321','EPSG','2320','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1322','Camacupa 1948 to WGS 84 (5)','Derived at station Djeno during coordination of platform PAL F2 in February 1992. Used by Elf for block 3 up to December 1992 then replaced by Camacupa 1948 to WGS 84 (7).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4220','EPSG','4326',3.0,-55.5,-348.0,-229.2,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'ELF-Ago B3 old',0);
INSERT INTO "usage" VALUES('EPSG','8243','helmert_transformation','EPSG','1322','EPSG','2318','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1323','Camacupa 1948 to WGS 84 (6)','Derived at Luanda observatory December 1992.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4220','EPSG','4326',8.0,-43.0,-337.0,-233.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'ELF-Ago B7 old',0);
INSERT INTO "usage" VALUES('EPSG','8244','helmert_transformation','EPSG','1323','EPSG','2319','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1324','Camacupa 1948 to WGS 84 (7)','Derived at platform PAL F2 in December 1992. For use in blocks 3, 7 and 17, replaced by Camacupa 1948 to WGS 84 (10) (code 1327).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4220','EPSG','4326',3.0,-48.0,-345.0,-231.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'ELF-Ago B15',0);
INSERT INTO "usage" VALUES('EPSG','8245','helmert_transformation','EPSG','1324','EPSG','2322','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1325','Camacupa 1948 to WGS 84 (8)','Derived at platform PAL F2 in December 1992. Used by Total for block 2 between December 1992 and 1994 then replaced by Camacupa 1948 to WGS 84 (10).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4220','EPSG','4326',3.0,-48.6,-345.1,-230.8,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'ELF-Ago B2 old',0);
INSERT INTO "usage" VALUES('EPSG','8246','helmert_transformation','EPSG','1325','EPSG','2317','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1326','Camacupa 1948 to WGS 84 (9)','Derived by GPS on two Topnav DGPS reference stations at Djeno and Luanda. Replaces Camacupa 1948 to WGS 84 (3). In block 18 replaced by BP from 1999 by Camacupa 1948 to WGS 84 (10).','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4220','EPSG','4326',10.0,-41.057,-374.564,-226.287,'EPSG','9001',0.0,0.0,0.554,'EPSG','9104',0.219,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'SHL-Ago B16',0);
INSERT INTO "usage" VALUES('EPSG','8247','helmert_transformation','EPSG','1326','EPSG','2323','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1327','Camacupa 1948 to WGS 84 (10)','Derived at platform PAL F2 in 1994 by Topnav using Doris.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4220','EPSG','4326',5.0,-50.9,-347.6,-231.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'ELF-Ago N',0);
INSERT INTO "usage" VALUES('EPSG','8248','helmert_transformation','EPSG','1327','EPSG','2324','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1328','Malongo 1987 to Mhast (1)','Malongo 1987 is an offshore extension of Mhast adopted by Chevron in 1987.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4259','EPSG','4264',0.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'CHV-Ago Cab',1);
INSERT INTO "usage" VALUES('EPSG','8249','helmert_transformation','EPSG','1328','EPSG','1317','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1329','Mhast to WGS 84 (1)','Superseded in 1990 by trf Malongo 1987 to WGS 84 (2), code 1557. Malongo 1987 is an offshore extension of the Mhast cooordinate system.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4264','EPSG','4326',10.0,-252.95,-4.11,-96.38,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'CHV-Ago Cab',1);
INSERT INTO "usage" VALUES('EPSG','8250','helmert_transformation','EPSG','1329','EPSG','1317','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1330','Malongo 1987 to WGS 84 (1)','Derived at Station Y in April 1989 using 572 transit satellite passes. Computed value for dZ was -96.42 but -96.38 has been utilised. Replaced Malongo 1987 to WGS 84 (3) (code 15791) in 1989. Replaced by Malongo 1987 to WGS 84 (2) (code 1557) in 1990.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4259','EPSG','4326',10.0,-252.95,-4.11,-96.38,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'CHV-Ago Cab89',0);
INSERT INTO "usage" VALUES('EPSG','8251','helmert_transformation','EPSG','1330','EPSG','3180','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1331','EST92 to ETRS89 (1)','','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4133','EPSG','4258',0.1,0.0,0.0,0.0,'EPSG','9001',0.0,0.0,0.0,'EPSG','9104',0.0,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'UT-Est',0);
INSERT INTO "usage" VALUES('EPSG','8252','helmert_transformation','EPSG','1331','EPSG','3246','EPSG','1161');
INSERT INTO "helmert_transformation" VALUES('EPSG','1332','Pulkovo 1942 to EST92 (1)','','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4284','EPSG','4133',9.0,21.53219,-97.00027,-60.74046,'EPSG','9001',-0.99548,-0.58147,-0.2418,'EPSG','9104',-4.5981,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'UT-Est',0);
INSERT INTO "usage" VALUES('EPSG','8253','helmert_transformation','EPSG','1332','EPSG','3246','EPSG','1024');
INSERT INTO "helmert_transformation" VALUES('EPSG','1333','EST92 to WGS 84 (1)','','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4133','EPSG','4326',0.5,0.055,-0.541,-0.185,'EPSG','9001',-0.0183,0.0003,0.007,'EPSG','9104',-0.014,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'UT-Est',0);
INSERT INTO "usage" VALUES('EPSG','8254','helmert_transformation','EPSG','1333','EPSG','3246','EPSG','1024');
INSERT INTO "helmert_transformation" VALUES('EPSG','1334','Pulkovo 1942 to WGS 84 (12)','','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4284','EPSG','4326',9.0,21.58719,-97.54127,-60.92546,'EPSG','9001',-1.01378,-0.58117,-0.2348,'EPSG','9104',-4.6121,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'UT-Est',0);
INSERT INTO "usage" VALUES('EPSG','8255','helmert_transformation','EPSG','1334','EPSG','3246','EPSG','1024');
INSERT INTO "helmert_transformation" VALUES('EPSG','1437','RT90 to ETRS89 (1)','Derived at 22 points in 1993. Replaced by RT90 to SWEREF99 (1) (code 1895) from 2001. This transformation is actually between ETRS89 and RR92. RR92 is a geographic 3D CRS where the horizontal component is RT90.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4124','EPSG','4258',0.5,419.3836,99.3335,591.3451,'EPSG','9001',-0.850389,-1.817277,7.862238,'EPSG','9104',-0.99496,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NLS-Swe',0);
INSERT INTO "usage" VALUES('EPSG','8358','helmert_transformation','EPSG','1437','EPSG','1225','EPSG','1035');
INSERT INTO "helmert_transformation" VALUES('EPSG','1438','Fahud to WGS 84 (2)','','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4232','EPSG','4326',25.0,-333.102,-11.02,230.69,'EPSG','9001',0.0,0.0,0.554,'EPSG','9104',0.219,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'PDO-Omn',0);
INSERT INTO "usage" VALUES('EPSG','8359','helmert_transformation','EPSG','1438','EPSG','4009','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1439','PSD93 to WGS 84 (1)','Residuals 0.5m at 67% probability level. Replaced PSD93 to WGS 84 (2) (code 8581) in 1997.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4134','EPSG','4326',0.5,-180.624,-225.516,173.919,'EPSG','9001',-0.81,-1.898,8.336,'EPSG','9104',16.71006,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'PDO-Omn 97',0);
INSERT INTO "usage" VALUES('EPSG','8360','helmert_transformation','EPSG','1439','EPSG','3288','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1440','ED50 to WGS 84 (19)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4230','EPSG','4326',999.0,-86.0,-92.2,-127.5,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'HEL-Grc',0);
INSERT INTO "usage" VALUES('EPSG','8361','helmert_transformation','EPSG','1440','EPSG','3254','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1441','Antigua 1943 to WGS 84 (1)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4601','EPSG','4326',10.0,-255.0,-15.0,71.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DOS-Atg Ant',0);
INSERT INTO "usage" VALUES('EPSG','8362','helmert_transformation','EPSG','1441','EPSG','1273','EPSG','1024');
INSERT INTO "helmert_transformation" VALUES('EPSG','1442','Dominica 1945 to WGS 84 (1)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4602','EPSG','4326',10.0,725.0,685.0,536.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DOS-Dma',0);
INSERT INTO "usage" VALUES('EPSG','8363','helmert_transformation','EPSG','1442','EPSG','3239','EPSG','1024');
INSERT INTO "helmert_transformation" VALUES('EPSG','1443','Grenada 1953 to WGS 84 (1)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4603','EPSG','4326',10.0,72.0,213.7,93.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DOS-Grd',0);
INSERT INTO "usage" VALUES('EPSG','8364','helmert_transformation','EPSG','1443','EPSG','3118','EPSG','1024');
INSERT INTO "helmert_transformation" VALUES('EPSG','1444','Montserrat 1958 to WGS 84 (1)','Derived at 1 satellite station. Accuracy 25m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4604','EPSG','4326',44.0,174.0,359.0,365.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DOS-Msr',0);
INSERT INTO "usage" VALUES('EPSG','8365','helmert_transformation','EPSG','1444','EPSG','3279','EPSG','1157');
INSERT INTO "helmert_transformation" VALUES('EPSG','1445','St. Kitts 1955 to WGS 84 (1)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4605','EPSG','4326',10.0,9.0,183.0,236.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DOS-Kna',0);
INSERT INTO "usage" VALUES('EPSG','8366','helmert_transformation','EPSG','1445','EPSG','3297','EPSG','1024');
INSERT INTO "helmert_transformation" VALUES('EPSG','1446','St. Lucia 1955 to WGS 84 (1)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4606','EPSG','4326',10.0,-149.0,128.0,296.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DOS-Lca',0);
INSERT INTO "usage" VALUES('EPSG','8367','helmert_transformation','EPSG','1446','EPSG','3298','EPSG','1024');
INSERT INTO "helmert_transformation" VALUES('EPSG','1448','HD72 to WGS 84 (3)','Parameter values taken from HD72 to ETRS89 (2) (code 1449) assuming that ETRS89 is equivalent to WGS 84 within the accuracy of the transformation. Replaces HD72 to WGS 84 (1) (code 1830).','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4237','EPSG','4326',1.0,52.684,-71.194,-13.975,'EPSG','9001',0.312,0.1063,0.3729,'EPSG','9104',1.0191,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'ELTE-Hun 2003',0);
INSERT INTO "usage" VALUES('EPSG','8369','helmert_transformation','EPSG','1448','EPSG','1119','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1449','HD72 to ETRS89 (2)','Derived at 1153 stations of the Hungarian National GPS Network. Values here correct parameter errors given in Hungarian standard MSZ 7222:2002. Replaces HD72 to ETRS89 (1), code 1829. May be taken as approximate tfm HD72 to WGS 84 - see tfm code 1448.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4237','EPSG','4258',0.4,52.684,-71.194,-13.975,'EPSG','9001',0.312,0.1063,0.3729,'EPSG','9104',1.0191,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'MSZ-Hun',0);
INSERT INTO "usage" VALUES('EPSG','8370','helmert_transformation','EPSG','1449','EPSG','1119','EPSG','1034');
INSERT INTO "helmert_transformation" VALUES('EPSG','1458','AGD66 to GDA94 (2)','For higher accuracy requirements see AGD66 to GDA94 (11) (code 1803). May be taken as approximate transformation AGD66 to WGS 84 - see code 1665. See code 5827 for a marginally better accuracy transformation derived locally.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4202','EPSG','4283',1.0,-129.193,-41.212,130.73,'EPSG','9001',-0.246,-0.374,-0.329,'EPSG','9104',-2.955,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'Auslig-ACT 1m',0);
INSERT INTO "usage" VALUES('EPSG','8379','helmert_transformation','EPSG','1458','EPSG','2283','EPSG','1158');
INSERT INTO "helmert_transformation" VALUES('EPSG','1459','AGD66 to GDA94 (3)','Replaced in 2000 by AGD66 to GDA94 (8) (code 1594). Application gives result differences which are sub-metre.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4202','EPSG','4283',1.0,-120.695,-62.73,165.46,'EPSG','9001',-0.109,0.141,0.116,'EPSG','9104',2.733,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'Auslig-Tas 1m old',0);
INSERT INTO "usage" VALUES('EPSG','8380','helmert_transformation','EPSG','1459','EPSG','1282','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','1460','AGD66 to GDA94 (4)','For higher accuracy requirements see AGD66 to GDA94 (11) (code 1803). May be taken as approximate transformation AGD66 to WGS 84 - see code 1666.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4202','EPSG','4283',1.0,-119.353,-48.301,139.484,'EPSG','9001',-0.415,-0.26,-0.437,'EPSG','9104',-0.613,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'Auslig-NSW Vic 1m',0);
INSERT INTO "usage" VALUES('EPSG','8381','helmert_transformation','EPSG','1460','EPSG','2286','EPSG','1158');
INSERT INTO "helmert_transformation" VALUES('EPSG','1469','Locodjo 1965 to WGS 84 (1)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4142','EPSG','4326',15.0,-125.0,53.0,467.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Civ',0);
INSERT INTO "usage" VALUES('EPSG','8390','helmert_transformation','EPSG','1469','EPSG','2282','EPSG','1024');
INSERT INTO "helmert_transformation" VALUES('EPSG','1470','Abidjan 1987 to WGS 84 (1)','Derived in Abidjan for use in the immediate area, but used by E&P industry onshore and offshore. Accuracy is submetre in Abidjan but unknown elsewhere. A similar CT (tfm code 6872) was used by Western Geophysical for offshore surveys in the 1990s.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4143','EPSG','4326',2.0,-124.76,53.0,466.79,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Civ',0);
INSERT INTO "usage" VALUES('EPSG','8391','helmert_transformation','EPSG','1470','EPSG','1075','EPSG','1216');
INSERT INTO "helmert_transformation" VALUES('EPSG','1471','MGI to WGS 84 (2)','','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4312','EPSG','4326',NULL,-577.326,-90.129,-463.919,'EPSG','9001',-15.8537,-4.55,-16.3489,'EPSG','9113',-2.4232,'EPSG','9201',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'BEV-Aut',1);
INSERT INTO "usage" VALUES('EPSG','8392','helmert_transformation','EPSG','1471','EPSG','1037','EPSG','1151');
INSERT INTO "helmert_transformation" VALUES('EPSG','1473','NAD83(CSRS98) to WGS 84 (1)','For many purposes NAD83(CSRS98) can be considered to be coincident with WGS 84.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4140','EPSG','4326',NULL,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Can',1);
INSERT INTO "usage" VALUES('EPSG','8394','helmert_transformation','EPSG','1473','EPSG','1336','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1504','Cape to Hartebeesthoek94 (1)','Residuals should not exceed 15 metres. Also used to transform Cape to WGS 84 - see code 1129.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4222','EPSG','4148',15.0,-134.73,-110.92,-292.66,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DSM-Zaf',0);
INSERT INTO "usage" VALUES('EPSG','8425','helmert_transformation','EPSG','1504','EPSG','3309','EPSG','1157');
INSERT INTO "helmert_transformation" VALUES('EPSG','1505','Hartebeesthoek94 to WGS 84 (1)','For many purposes Hartebeesthoek94 datum can be considered to be coincident with WGS 84.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4148','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Zaf',0);
INSERT INTO "usage" VALUES('EPSG','8426','helmert_transformation','EPSG','1505','EPSG','4540','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1508','CH1903 to WGS 84 (1)','Implemented in Bundesamt für Landestopographie programme GRANIT.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4149','EPSG','4326',NULL,660.077,13.551,369.344,'EPSG','9001',2.484,1.783,2.939,'EPSG','9113',5.66,'EPSG','9201',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'BfL-CH 1',1);
INSERT INTO "usage" VALUES('EPSG','8429','helmert_transformation','EPSG','1508','EPSG','1286','EPSG','1025');
INSERT INTO "helmert_transformation" VALUES('EPSG','1509','CH1903+ to CHTRS95 (1)','This transformation is also given as CH1903+ to ETRS89 (1) (code 1647). CHTRS95 is a national realization of ETRS89.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4150','EPSG','4151',0.1,674.374,15.056,405.346,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'BfL-CH',0);
INSERT INTO "usage" VALUES('EPSG','8430','helmert_transformation','EPSG','1509','EPSG','1286','EPSG','1031');
INSERT INTO "helmert_transformation" VALUES('EPSG','1510','CH1903 to WGS 84 (2)','These parameters are strictly between CH1903+ and CHTRF95 but are used from CH1903 as an approximation which is within the accuracy of the distortions in the CH1903 network.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4149','EPSG','4326',NULL,674.374,15.056,405.346,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'BfL-CH 2',1);
INSERT INTO "usage" VALUES('EPSG','8431','helmert_transformation','EPSG','1510','EPSG','1286','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1511','CHTRS95 to WGS 84 (1)','For many purposes CHTRS95 can be considered to be coincident with WGS 84.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4151','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-CH',0);
INSERT INTO "usage" VALUES('EPSG','8432','helmert_transformation','EPSG','1511','EPSG','1286','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1512','Rassadiran to WGS 84 (1)','Derived in 1998 at Assaluyeh (Taheri refinery) by Geoid for Total. Used only for terminal site for South Pars phases 2 and 3.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4153','EPSG','4326',0.5,-133.63,-157.5,-158.62,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'TOT-Irn Taheri',0);
INSERT INTO "usage" VALUES('EPSG','8433','helmert_transformation','EPSG','1512','EPSG','1338','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1513','FD58 to WGS 84 (1)','Derived in 1998 in Kangan district by Geoid for Total. Used for South Pars phases 2 and 3.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4132','EPSG','4326',0.5,-241.54,-163.64,396.06,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'TOT-Irn Kangan',0);
INSERT INTO "usage" VALUES('EPSG','8434','helmert_transformation','EPSG','1513','EPSG','2362','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1514','ED50(ED77) to WGS 84 (1)','Used for South Pars phases 6, 7 and 8.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4154','EPSG','4326',1.0,-110.33,-97.73,-119.85,'EPSG','9001',0.3423,1.1634,0.2715,'EPSG','9104',0.063,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NCCI-Irn',0);
INSERT INTO "usage" VALUES('EPSG','8435','helmert_transformation','EPSG','1514','EPSG','1123','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','1515','NAD83 to WGS 84 (5)','Strictly between NAD83 and ITRF96(1997.0). Supersedes NAD83 to WGS 84 (4) (code 1308).','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4269','EPSG','4326',NULL,-0.991,1.9072,0.5129,'EPSG','9001',-1.25033e-07,-4.6785e-08,-5.6529e-08,'EPSG','9101',0.0,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NGS-Usa ITRF96',1);
INSERT INTO "usage" VALUES('EPSG','8436','helmert_transformation','EPSG','1515','EPSG','1323','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','1516','La Canoa to WGS 84 (18)','Parameter values estimated accuracy: ± 2.0m; ± 2.7m; ± 1.3m respectively. Also used for PSAD56 to WGS 84 transformations.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4247','EPSG','4326',2.5,-273.5,110.6,-357.9,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'LAG-Ven E',0);
INSERT INTO "usage" VALUES('EPSG','8437','helmert_transformation','EPSG','1516','EPSG','2363','EPSG','1043');
INSERT INTO "helmert_transformation" VALUES('EPSG','1517','Conakry 1905 to WGS 84 (1)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4315','EPSG','4326',30.0,-23.0,259.0,-9.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Gin',0);
INSERT INTO "usage" VALUES('EPSG','8438','helmert_transformation','EPSG','1517','EPSG','3257','EPSG','1024');
INSERT INTO "helmert_transformation" VALUES('EPSG','1518','Dabola 1981 to WGS 84 (1)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4155','EPSG','4326',25.0,-83.0,37.0,124.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Gin',0);
INSERT INTO "usage" VALUES('EPSG','8439','helmert_transformation','EPSG','1518','EPSG','3257','EPSG','1157');
INSERT INTO "helmert_transformation" VALUES('EPSG','1527','Campo Inchauspe to WGS 84 (2)','Derived through ties at 2 stations (Cerro Colorado and Chihuido Sur) to 4 IGS stations in February 1995','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4221','EPSG','4326',0.5,-154.5,150.7,100.4,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'TOT-Arg Neu',0);
INSERT INTO "usage" VALUES('EPSG','8448','helmert_transformation','EPSG','1527','EPSG','2325','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1528','Chos Malal 1914 to Campo Inchauspe (1)','Derived through common coordinates at 5 stations.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4160','EPSG','4221',10.0,160.0,26.0,41.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'TOT-Arg Neu',0);
INSERT INTO "usage" VALUES('EPSG','8449','helmert_transformation','EPSG','1528','EPSG','2325','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1529','Hito XVIII to WGS 84 (1)','Derived through ties at 3 stations (RC03, TOTAL11 and MP12) to 3 IGS stations in November 1995','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4254','EPSG','4326',0.5,18.38,192.45,96.82,'EPSG','9001',0.056,-0.142,-0.2,'EPSG','9104',-0.0013,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'TOT-Arg TdF',0);
INSERT INTO "usage" VALUES('EPSG','8450','helmert_transformation','EPSG','1529','EPSG','2357','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1530','NAD27 to WGS 84 (30)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4267','EPSG','4326',3.0,-4.2,135.4,181.9,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'ICH-Cub',0);
INSERT INTO "usage" VALUES('EPSG','8451','helmert_transformation','EPSG','1530','EPSG','1077','EPSG','1158');
INSERT INTO "helmert_transformation" VALUES('EPSG','1531','Nahrwan 1967 to WGS 84 (4)','Parameter values adopted by Total are mean of those derived by Oceonics and Geoid through ties at platform AK1 to 4 IGS stations in March 1995.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4270','EPSG','4326',0.5,-245.0,-153.9,382.8,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'TOT-UAE Abk',0);
INSERT INTO "usage" VALUES('EPSG','8452','helmert_transformation','EPSG','1531','EPSG','2392','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1532','M''poraloko to WGS 84 (2)','Derived as mean of Doris determinations at 3 stations in Port Gentil area in 1994.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4266','EPSG','4326',0.5,-80.7,-132.5,41.1,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'Elf-Gab94',0);
INSERT INTO "usage" VALUES('EPSG','8453','helmert_transformation','EPSG','1532','EPSG','1100','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1533','Kalianpur 1937 to WGS 84 (2)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4144','EPSG','4326',5.0,214.0,804.0,268.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'TOT-Mmr Moat',0);
INSERT INTO "usage" VALUES('EPSG','8454','helmert_transformation','EPSG','1533','EPSG','2361','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1534','Minna to WGS 84 (3)','','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4263','EPSG','4326',NULL,-111.92,-87.85,114.5,'EPSG','9001',1.875,0.202,0.219,'EPSG','9104',0.032,'EPSG','9201',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'SHL-Nig S',1);
INSERT INTO "usage" VALUES('EPSG','8455','helmert_transformation','EPSG','1534','EPSG','2371','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1536','Nahrwan 1967 to WGS 84 (5)','Derived by Brown & Root in 1992 for Qatar General Petroleum Corporation North Field development. Adopted by QGPC for all offshore Qatar.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4270','EPSG','4326',1.0,-250.2,-153.09,391.7,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'B&R-Qat off',0);
INSERT INTO "usage" VALUES('EPSG','8457','helmert_transformation','EPSG','1536','EPSG','2406','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1537','Indian 1975 to WGS 84 (3)','Derived in 1995 at point RTSD181.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4240','EPSG','4326',1.0,204.64,834.74,293.8,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'Fug-Tha',0);
INSERT INTO "usage" VALUES('EPSG','8458','helmert_transformation','EPSG','1537','EPSG','2358','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1538','Carthage to WGS 84 (2)','Derived at station Chaffar January 1995.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4223','EPSG','4326',1.0,-260.1,5.5,432.2,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'Elf-Tun',0);
INSERT INTO "usage" VALUES('EPSG','8459','helmert_transformation','EPSG','1538','EPSG','1489','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1539','South Yemen to Yemen NGN96 (1)','May be used as an approximate transformation to WGS 84 - see South Yemen to WGS 84 (1) (code 1682).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4164','EPSG','4163',5.0,-76.0,-138.0,67.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Yem South',0);
INSERT INTO "usage" VALUES('EPSG','8460','helmert_transformation','EPSG','1539','EPSG','1340','EPSG','1045');
INSERT INTO "helmert_transformation" VALUES('EPSG','1540','Yemen NGN96 to WGS 84 (1)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4163','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Yem',0);
INSERT INTO "usage" VALUES('EPSG','8461','helmert_transformation','EPSG','1540','EPSG','1257','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1541','Indian 1960 to WGS 72BE (1)','Derived in Vung Tau area by Technical Navigation for Deminex in 1978.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4131','EPSG','4324',25.0,199.0,931.0,317.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'PV-Vnm',0);
INSERT INTO "usage" VALUES('EPSG','8462','helmert_transformation','EPSG','1541','EPSG','1495','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1542','Indian 1960 to WGS 84 (2)','Derived at 2 stations. Accuracy 25m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4131','EPSG','4326',44.0,198.0,881.0,317.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Vnm 16N',0);
INSERT INTO "usage" VALUES('EPSG','8463','helmert_transformation','EPSG','1542','EPSG','2359','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1543','Indian 1960 to WGS 84 (3)','Derived at 1 station. Accuracy 25m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4131','EPSG','4326',44.0,182.0,915.0,344.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Vnm ConSon',0);
INSERT INTO "usage" VALUES('EPSG','8464','helmert_transformation','EPSG','1543','EPSG','2360','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1544','Hanoi 1972 to WGS 84 (1)','Derived in Vung Tau area.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4147','EPSG','4326',5.0,-17.51,-108.32,-62.39,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'BP-Vnm',0);
INSERT INTO "usage" VALUES('EPSG','8465','helmert_transformation','EPSG','1544','EPSG','1494','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1545','Egypt 1907 to WGS 72 (1)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4229','EPSG','4322',5.0,-121.8,98.1,-15.2,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'MCE-Egy',0);
INSERT INTO "usage" VALUES('EPSG','8466','helmert_transformation','EPSG','1545','EPSG','1086','EPSG','1024');
INSERT INTO "helmert_transformation" VALUES('EPSG','1546','Egypt 1907 to WGS 84 (3)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4229','EPSG','4326',30.0,-146.21,112.63,4.05,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'Racal-Egy GoS',1);
INSERT INTO "usage" VALUES('EPSG','8467','helmert_transformation','EPSG','1546','EPSG','2341','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1547','Bissau to WGS 84 (1)','Derived at 2 stations. Accuracy 25m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4165','EPSG','4326',25.0,-173.0,253.0,27.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Gnb',0);
INSERT INTO "usage" VALUES('EPSG','8468','helmert_transformation','EPSG','1547','EPSG','3258','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1548','SAD69 to WGS 84 (14)','Derived by Brazilean Institute of Geography and Statistics (IGBE) in 1989. Used by ANP.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4291','EPSG','4326',NULL,-66.87,4.37,-38.52,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGBE-Bra',1);
INSERT INTO "usage" VALUES('EPSG','8469','helmert_transformation','EPSG','1548','EPSG','1053','EPSG','1210');
INSERT INTO "helmert_transformation" VALUES('EPSG','1549','Aratu to WGS 84 (1)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4208','EPSG','4326',999.0,-158.0,315.0,-148.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'PB-Bra Camp',1);
INSERT INTO "usage" VALUES('EPSG','8470','helmert_transformation','EPSG','1549','EPSG','2307','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1550','Aratu to WGS 84 (2)','Replaced by Aratu to WGS 84 (18) (tfm code 5061) which Petrobras now recommends for the area.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4208','EPSG','4326',5.0,-139.62,290.53,-150.29,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'PB-Bra TucN',0);
INSERT INTO "usage" VALUES('EPSG','8471','helmert_transformation','EPSG','1550','EPSG','2308','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1551','Aratu to WGS 84 (3)','Replaced by Aratu to WGS 84 (18) (tfm code 5061) which Petrobras now recommends for the area.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4208','EPSG','4326',5.0,-141.15,293.44,-150.56,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'PB-Bra TucC',0);
INSERT INTO "usage" VALUES('EPSG','8472','helmert_transformation','EPSG','1551','EPSG','2309','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1552','Aratu to WGS 84 (4)','Replaced by Aratu to WGS 84 (18) (tfm code 5061) which Petrobras now recommends for the area.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4208','EPSG','4326',5.0,-142.48,296.03,-149.74,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'PB-Bra TucS',0);
INSERT INTO "usage" VALUES('EPSG','8473','helmert_transformation','EPSG','1552','EPSG','2310','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1555','Naparima 1955 to WGS 84 (2)','Derived in 1989 by ONI for Amoco.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4158','EPSG','4326',1.0,-0.465,372.095,171.736,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'Amoco-Tto Trin',0);
INSERT INTO "usage" VALUES('EPSG','8476','helmert_transformation','EPSG','1555','EPSG','3143','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1556','Naparima 1955 to WGS 84 (3)','DMA does not differentiate between Naparima 1955 (Trinidad) and Naparima 1972 (Tobago). Consequently for Trinidad IOGP has duplicated this transformation as Naparima 1972 to WGS 84 (3) - see code 1307.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4158','EPSG','4326',26.0,-2.0,374.0,172.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NIMA-Tto Trin',0);
INSERT INTO "usage" VALUES('EPSG','8477','helmert_transformation','EPSG','1556','EPSG','3143','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1557','Malongo 1987 to WGS 84 (2)','Derived at station Y in July 1990 through Transit single point positioning using 187 passes by Geodetic Survey Ltd. Replaces Malongo 1987 to WGS 84 (1) (trf code 1330).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4259','EPSG','4326',5.0,-254.1,-5.36,-100.29,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'CHV-Ago Cab90',0);
INSERT INTO "usage" VALUES('EPSG','8478','helmert_transformation','EPSG','1557','EPSG','3180','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1558','Korean 1995 to WGS 84 (1)','Derived at 5 stations. Accuracy 1m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4166','EPSG','4326',2.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NIMA-Kor',0);
INSERT INTO "usage" VALUES('EPSG','8479','helmert_transformation','EPSG','1558','EPSG','3266','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1560','Nord Sahara 1959 to WGS 72BE (1)','Derived at IGN monument CFP19 using Transit.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4307','EPSG','4324',8.0,-156.5,-87.2,285.9,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'CGG-Alg HM',0);
INSERT INTO "usage" VALUES('EPSG','8481','helmert_transformation','EPSG','1560','EPSG','2393','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1561','Qatar 1974 to WGS 84 (1)','Derived at 3 stations. Accuracy 20m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4285','EPSG','4326',35.0,-128.0,-283.0,22.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Qat',0);
INSERT INTO "usage" VALUES('EPSG','8482','helmert_transformation','EPSG','1561','EPSG','1346','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1562','Qatar 1974 to WGS 84 (2)','Derived by Brown & Root in 1992 for Qatar General Petroleum Corporation.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4285','EPSG','4326',1.0,-128.16,-282.42,21.93,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'B&R-Qat off',0);
INSERT INTO "usage" VALUES('EPSG','8483','helmert_transformation','EPSG','1562','EPSG','2406','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1563','Qatar 1974 to WGS 84 (3)','Derived by Qatar Centre for GIS. See Qatar 1974 to WGS 84 (2) (code 1562) for transformation used by QGPC for offshore petroleum industry.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4285','EPSG','4326',1.0,-128.033,-283.697,21.052,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'CGIS-Qat',0);
INSERT INTO "usage" VALUES('EPSG','8484','helmert_transformation','EPSG','1563','EPSG','1346','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1564','NZGD49 to WGS 84 (2)','These parameter values are taken from NZGD49 to NZGD2000 (2) (code 1701) and assume that NZGD2000 and WGS 84 are coincident to within the accuracy of the transformation. For improved accuracy use NZGD49 to WGS 84 (4) (code 1670).','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4272','EPSG','4326',4.0,59.47,-5.04,187.44,'EPSG','9001',-0.47,0.1,-1.024,'EPSG','9104',-4.5993,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OSG-Nzl 4m',0);
INSERT INTO "usage" VALUES('EPSG','8485','helmert_transformation','EPSG','1564','EPSG','3285','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1565','NZGD2000 to WGS 84 (1)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4167','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OSG-Nzl',0);
INSERT INTO "usage" VALUES('EPSG','8486','helmert_transformation','EPSG','1565','EPSG','1175','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1566','NZGD49 to NZGD2000 (1)','For better accuracy use NZGD49 to NZGD2000 (2) (code 1701) or NZGD49 to NZGD2000 (3) (code 1568).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4272','EPSG','4167',5.0,54.4,-20.1,183.1,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OSG-Nzl 5m',0);
INSERT INTO "usage" VALUES('EPSG','8487','helmert_transformation','EPSG','1566','EPSG','3285','EPSG','1045');
INSERT INTO "helmert_transformation" VALUES('EPSG','1567','NZGD49 to NZGD2000 (2)','4m accuracy. For better accuracy use NZGD49 to NZGD2000 (3) (code 1568)','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4272','EPSG','4167',NULL,59.47,-5.04,187.44,'EPSG','9001',-0.47,0.1,1.024,'EPSG','9104',-4.5993,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OSG-Nzl 4m',1);
INSERT INTO "usage" VALUES('EPSG','8488','helmert_transformation','EPSG','1567','EPSG','1175','EPSG','1044');
INSERT INTO "helmert_transformation" VALUES('EPSG','1569','Accra to WGS 84 (1)','Derived at 3 common points.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4168','EPSG','4326',25.0,-199.0,32.0,322.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'MCE-Gha',0);
INSERT INTO "usage" VALUES('EPSG','8490','helmert_transformation','EPSG','1569','EPSG','1104','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1570','Accra to WGS 72BE (1)','Derived be single point Transit observation at several locations.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4168','EPSG','4324',25.0,-171.16,17.29,323.31,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'GSI-Gha',0);
INSERT INTO "usage" VALUES('EPSG','8491','helmert_transformation','EPSG','1570','EPSG','1505','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1571','Amersfoort to ETRS89 (1)','Dutch sources also quote an equivalent transformation with parameter values dX=+593.032 dY=+26.000 dZ=+478.741m, rX rY rZ and dS as this tfm. These values belong to a different transformation method and cannot be used with the Coordinate Frame method.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4258','EPSG','4326',NULL,565.04,49.91,465.84,'EPSG','9001',1.9848,-1.7439,9.0587,'EPSG','9109',4.0772,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NCG-Nld 2000',1);
INSERT INTO "usage" VALUES('EPSG','8492','helmert_transformation','EPSG','1571','EPSG','1172','EPSG','1035');
INSERT INTO "helmert_transformation" VALUES('EPSG','1577','American Samoa 1962 to WGS 84 (1)','Transformation based on observations at 2 stations in 1993. One sigma uncertainty is 25m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4169','EPSG','4326',44.0,-115.0,118.0,426.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NIMA-Asm',0);
INSERT INTO "usage" VALUES('EPSG','8498','helmert_transformation','EPSG','1577','EPSG','3109','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1580','NAD83(HARN) to WGS 84 (1)','For many purposes NAD83(HARN) can be considered to be coincident with WGS 84.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4152','EPSG','4326',2.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Usa',0);
INSERT INTO "usage" VALUES('EPSG','8501','helmert_transformation','EPSG','1580','EPSG','1337','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1581','SIRGAS 1995 to WGS 84 (1)','Accuracy 1m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4170','EPSG','4326',2.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NIMA-S America',0);
INSERT INTO "usage" VALUES('EPSG','8502','helmert_transformation','EPSG','1581','EPSG','3448','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1582','PSAD56 to WGS 84 (10)','Derived May 1995 by Geoid for Total. OSU91A geoid model used.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4248','EPSG','4326',3.0,-259.73,173.12,-398.27,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'TOT-Bol  Mad',0);
INSERT INTO "usage" VALUES('EPSG','8503','helmert_transformation','EPSG','1582','EPSG','2400','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1583','PSAD56 to WGS 84 (11)','Derived July 1997 by Geoid from data recorded by UGA for Total. OSU91A geoid model used.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4248','EPSG','4326',0.5,-307.7,265.3,-363.5,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'TOT-Bol  B20',0);
INSERT INTO "usage" VALUES('EPSG','8504','helmert_transformation','EPSG','1583','EPSG','2401','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1584','Deir ez Zor to WGS 72BE (1)','Recomputed in 1991 by Elf from data derived in 1983 at station 254 Deir by Geco using Transit. Derivation of 1983 parameter values of dX=-163.2 dY=-12.7 dZ=+232.7 contained errors in geodetic parameters for Syria.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4227','EPSG','4324',5.0,-174.6,-3.1,236.2,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'GECO-Syr',0);
INSERT INTO "usage" VALUES('EPSG','8505','helmert_transformation','EPSG','1584','EPSG','2329','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1585','Deir ez Zor to WGS 84 (2)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4227','EPSG','4326',999.0,-177.5,14.1,237.6,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Syr',1);
INSERT INTO "usage" VALUES('EPSG','8506','helmert_transformation','EPSG','1585','EPSG','1227','EPSG','1025');
INSERT INTO "helmert_transformation" VALUES('EPSG','1586','Deir ez Zor to WGS 84 (3)','Derived in 1995 by CGG for Al Furat Petroleum Company. Can be approximated using geocentric translations of dX=-174.3m, dY=+14.1m, dZ=+237.6m.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4227','EPSG','4326',999.0,-175.09,1.218,238.831,'EPSG','9001',-0.047,0.019,0.808,'EPSG','9104',0.1698,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'SHL-Syr Whal',0);
INSERT INTO "usage" VALUES('EPSG','8507','helmert_transformation','EPSG','1586','EPSG','2327','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1587','Deir ez Zor to WGS 84 (4)','Derived at four stations by Topnav in 1997.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4227','EPSG','4326',1.0,-191.77,15.01,235.07,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'ELF-Syr Shad',0);
INSERT INTO "usage" VALUES('EPSG','8508','helmert_transformation','EPSG','1587','EPSG','2328','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1588','ED50 to ETRS89 (1)','Included in Statens Kartverk programme wsktrans from 1997. The same parameter values were adopted for ED50 to WGS84 (variant 23) transformation offshore Norway north of 62N from April 2001 - see code 1612.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4230','EPSG','4258',1.0,-116.641,-56.931,-110.559,'EPSG','9001',4.327,4.464,-4.444,'EPSG','9109',-3.52,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NMA-Nor N65 1997',0);
INSERT INTO "usage" VALUES('EPSG','8509','helmert_transformation','EPSG','1588','EPSG','2332','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','1591','RGF93 v1 to ETRS89 (1)','RGF93 v1 is a national realization of ETRS89. May be taken as approximate transformation RGF93 v1 to WGS 84 - see code 1671.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4171','EPSG','4258',0.1,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Fra',0);
INSERT INTO "usage" VALUES('EPSG','8512','helmert_transformation','EPSG','1591','EPSG','1096','EPSG','1161');
INSERT INTO "helmert_transformation" VALUES('EPSG','1592','Timbalai 1948 to WGS 84 (2)','Originally used by BSP offshore only, use extended to onshore in 2010.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4298','EPSG','4326',5.0,-678.0,670.0,-48.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'BSP-Brn',0);
INSERT INTO "usage" VALUES('EPSG','8513','helmert_transformation','EPSG','1592','EPSG','1055','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1594','AGD66 to GDA94 (8)','Replaces AGD66 to GDA94 (3) (code 1459) from August 2000. For higher accuracy requirements see AGD66 to GDA94 (11) (code 1803). May be taken as approximate transformation AGD66 to WGS 84 - see code 1667.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4202','EPSG','4283',1.0,-120.271,-64.543,161.632,'EPSG','9001',-0.217,0.067,0.129,'EPSG','9104',2.499,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OSG-Tas 1m',0);
INSERT INTO "usage" VALUES('EPSG','8515','helmert_transformation','EPSG','1594','EPSG','1282','EPSG','1158');
INSERT INTO "helmert_transformation" VALUES('EPSG','1595','AGD66 to GDA94 (9)','For higher accuracy requirements see AGD66 to GDA94 (11) (code 1803). May be taken as approximate transformation AGD66 to WGS 84 - see code 1668.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4202','EPSG','4283',1.0,-124.133,-42.003,137.4,'EPSG','9001',0.008,-0.557,-0.178,'EPSG','9104',-1.854,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'Auslig-NT 1m',0);
INSERT INTO "usage" VALUES('EPSG','8516','helmert_transformation','EPSG','1595','EPSG','2284','EPSG','1158');
INSERT INTO "helmert_transformation" VALUES('EPSG','1597','Bogota 1975 to WGS 84 (2)','Derived in 1995 by WGC at first order stations Recreo and Mena via multi-day ties to 4 IGS stations. Residuals under 20cm.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4218','EPSG','4326',0.2,304.5,306.5,-318.1,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'BP-Col CusCup',0);
INSERT INTO "usage" VALUES('EPSG','8518','helmert_transformation','EPSG','1597','EPSG','2315','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1598','POSGAR to WGS 84 (1)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4172','EPSG','4326',NULL,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Arg',1);
INSERT INTO "usage" VALUES('EPSG','8519','helmert_transformation','EPSG','1598','EPSG','1033','EPSG','1025');
INSERT INTO "helmert_transformation" VALUES('EPSG','1609','BD72 to WGS 84 (1)','Scale difference is given by information source as 0.999999. Given in this record in ppm to assist application usage. Very similar parameter values (to slightly less precision) used for BD72 to ETRS89: see code 1652.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4313','EPSG','4326',1.0,-99.059,53.322,-112.486,'EPSG','9001',-0.419,0.83,-1.885,'EPSG','9104',-1.0,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Bel 7',0);
INSERT INTO "usage" VALUES('EPSG','8530','helmert_transformation','EPSG','1609','EPSG','1347','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1610','BD72 to WGS 84 (2)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4313','EPSG','4326',5.0,-125.8,79.9,-100.5,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Bel 3',0);
INSERT INTO "usage" VALUES('EPSG','8531','helmert_transformation','EPSG','1610','EPSG','1347','EPSG','1045');
INSERT INTO "helmert_transformation" VALUES('EPSG','1611','IRENET95 to ETRS89 (1)','IRENET95 is a regional realization of ETRS89. May be taken as approximate transformation IRENET95 to WGS 84 - see code 1678.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4173','EPSG','4258',0.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OSI-Ire',0);
INSERT INTO "usage" VALUES('EPSG','8532','helmert_transformation','EPSG','1611','EPSG','1305','EPSG','1161');
INSERT INTO "helmert_transformation" VALUES('EPSG','1612','ED50 to WGS 84 (23)','Parameter values are taken from ED50 to ETRS89 (1), code 1588, (rotations converted from μrad). Adopted for ED50 to WGS 84 CTs offshore Norway north of 62°N from April 2001, replacing code 1590. Included in Statens Kartverk programme wsktrans from v4.0.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4230','EPSG','4326',1.0,-116.641,-56.931,-110.559,'EPSG','9001',0.893,0.921,-0.917,'EPSG','9104',-3.52,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Nor N62 2001',0);
INSERT INTO "usage" VALUES('EPSG','8533','helmert_transformation','EPSG','1612','EPSG','2601','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1613','ED50 to WGS 84 (24)','Approximation to 1 metre of concatenated transformation ED50 to WGS 84 (14), code 8653. 8653 remains the transformation promulgated by Statens Kartverk but 1613 recommended by EPSG for practical oil industry usage.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4230','EPSG','4326',1.0,-90.365,-101.13,-123.384,'EPSG','9001',0.333,0.077,0.894,'EPSG','9104',1.994,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Nor S62 2001',0);
INSERT INTO "usage" VALUES('EPSG','8534','helmert_transformation','EPSG','1613','EPSG','2334','EPSG','1253');
INSERT INTO "helmert_transformation" VALUES('EPSG','1614','Sierra Leone 1968 to WGS 84 (1)','Determined at 8 stations, accuracy +/- 15m in each axis. Info. Source has the source CRS as Sierra Leone 1960. Sierra Leone 1968 is a readjustment of the 1960 network: coordinates changed by less than 3 metres.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4175','EPSG','4326',26.0,-88.0,4.0,101.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NIMA-Sle',0);
INSERT INTO "usage" VALUES('EPSG','8535','helmert_transformation','EPSG','1614','EPSG','3306','EPSG','1076');
INSERT INTO "helmert_transformation" VALUES('EPSG','1615','Timbalai 1948 to WGS 84 (3)','CARE! Erroneous GPS data was used in the derivation of these parameters. They produce a coordinate difference of 10m horizontally and 50m vertically compared to Timbalai 1948 to WGS 84 (2) (code 1592).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4298','EPSG','4326',100.0,-726.282,703.611,-48.999,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'SD-Brn',0);
INSERT INTO "usage" VALUES('EPSG','8536','helmert_transformation','EPSG','1615','EPSG','2349','EPSG','1142');
INSERT INTO "helmert_transformation" VALUES('EPSG','1616','PSD93 to WGS 72 (1)','Residuals 1.2m at 67% probability level.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4134','EPSG','4322',1.2,-182.046,-225.604,168.884,'EPSG','9001',-0.616,-1.655,7.824,'EPSG','9104',16.641,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'PDO-Omn 93',0);
INSERT INTO "usage" VALUES('EPSG','8537','helmert_transformation','EPSG','1616','EPSG','3288','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1617','PSD93 to WGS 84 (3)','Accuracy better than 0.5m in block 4.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4134','EPSG','4326',0.5,-191.808,-250.512,167.861,'EPSG','9001',-0.792,-1.653,8.558,'EPSG','9104',20.703,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'Tot-Omn 95',0);
INSERT INTO "usage" VALUES('EPSG','8538','helmert_transformation','EPSG','1617','EPSG','2404','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1618','MGI to WGS 84 (3)','Same transformation parameters used for MGI to ETRS89 (1) (code 1619).','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4312','EPSG','4326',1.5,577.326,90.129,463.919,'EPSG','9001',5.137,1.474,5.297,'EPSG','9104',2.4232,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'BEV-Aut',0);
INSERT INTO "usage" VALUES('EPSG','8539','helmert_transformation','EPSG','1618','EPSG','1037','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1619','MGI to ETRS89 (1)','Same transformation parameters used for MGI to WGS 84 (3) (code 1618). Precision of parameter values in this record were increased effective 16-Dec-2006 (db v6.12): see change record 2006.971.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4312','EPSG','4258',1.5,577.326,90.129,463.919,'EPSG','9001',5.137,1.474,5.297,'EPSG','9104',2.4232,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'BEV-Aut',0);
INSERT INTO "usage" VALUES('EPSG','8540','helmert_transformation','EPSG','1619','EPSG','1037','EPSG','1151');
INSERT INTO "helmert_transformation" VALUES('EPSG','1620','MGI to ETRS89 (2)','May be taken as approximate transformation MGI to WGS 84 - see code 1621.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4312','EPSG','4258',1.0,551.7,162.9,467.9,'EPSG','9001',6.04,1.96,-11.38,'EPSG','9104',-4.82,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DGU-Hrv',1);
INSERT INTO "usage" VALUES('EPSG','8541','helmert_transformation','EPSG','1620','EPSG','1076','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','1621','MGI to WGS 84 (4)','Parameter values from MGI to ETRS89 (2) (code 1620). Assumes ETRS89 and WGS 84 can be considered the same to within the accuracy of the transformation.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4312','EPSG','4326',1.0,551.7,162.9,467.9,'EPSG','9001',6.04,1.96,-11.38,'EPSG','9104',-4.82,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Hrv',1);
INSERT INTO "usage" VALUES('EPSG','8542','helmert_transformation','EPSG','1621','EPSG','1076','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1622','S-JTSK to ETRS89 (1)','May be taken as approximate transformation S-JTSK to WGS 84 - see code 1623. Improved by S-JTSK/05 to ETRS89 (1) (code 5226) in 2009, where S-JTSK/05 is a scientific working system and S-JTSK remains the legal system in Czechia.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4156','EPSG','4258',1.0,570.8,85.7,462.8,'EPSG','9001',4.998,1.587,5.261,'EPSG','9104',3.56,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'CUZK-Cze',0);
INSERT INTO "usage" VALUES('EPSG','8543','helmert_transformation','EPSG','1622','EPSG','1079','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','1623','S-JTSK to WGS 84 (1)','Parameter values from S-JTSK to ETRS89 (1) (code 1622). Assumes ETRS89 and WGS 84 can be considered the same to within the accuracy of the transformation. Replaced by S-JTSK to WGS 84 (5) (code 5239).','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4156','EPSG','4326',1.0,570.8,85.7,462.8,'EPSG','9001',4.998,1.587,5.261,'EPSG','9104',3.56,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Cze',0);
INSERT INTO "usage" VALUES('EPSG','8544','helmert_transformation','EPSG','1623','EPSG','1079','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1624','S-JTSK to ETRS89 (2)','May be taken as approximate transformation S-JTSK to WGS 84 - see code 1625.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4156','EPSG','4258',1.0,559.0,68.7,451.5,'EPSG','9001',7.92,4.073,4.251,'EPSG','9104',5.71,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'UGKK-Svk',1);
INSERT INTO "usage" VALUES('EPSG','8545','helmert_transformation','EPSG','1624','EPSG','1211','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','1625','S-JTSK to WGS 84 (2)','Parameter values from S-JTSK to ETRS89 (2) (code 1624). Assumes ETRS89 and WGS 84 can be considered the same to within the accuracy of the transformation.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4156','EPSG','4326',1.0,559.0,68.7,451.5,'EPSG','9001',7.92,4.073,4.251,'EPSG','9104',5.71,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Svk',1);
INSERT INTO "usage" VALUES('EPSG','8546','helmert_transformation','EPSG','1625','EPSG','1211','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1626','ED50 to ETRS89 (4)','May be taken as approximate transformation ED50 to WGS 84 - see code 1627.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4230','EPSG','4258',1.0,-81.1,-89.4,-115.8,'EPSG','9001',0.485,0.024,0.413,'EPSG','9104',-0.54,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'KMS-Dnk',0);
INSERT INTO "usage" VALUES('EPSG','8547','helmert_transformation','EPSG','1626','EPSG','3237','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','1627','ED50 to WGS 84 (25)','Parameter values from ED50 to ETRS89 (4) (code 1626). Assumes ETRS89 and WGS 84 can be considered the same to within the accuracy of the transformation.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4230','EPSG','4326',1.0,-81.1,-89.4,-115.8,'EPSG','9001',0.485,0.024,0.413,'EPSG','9104',-0.54,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Dnk',0);
INSERT INTO "usage" VALUES('EPSG','8548','helmert_transformation','EPSG','1627','EPSG','3237','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1628','ED50 to ETRS89 (5)','May be taken as approximate transformation ED50 to WGS 84 - see code 1629.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4230','EPSG','4258',1.0,-116.8,-106.4,-154.4,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DGC-Gib',0);
INSERT INTO "usage" VALUES('EPSG','8549','helmert_transformation','EPSG','1628','EPSG','1105','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','1629','ED50 to WGS 84 (26)','Parameter values from ED50 to ETRS89 (5) (code 1628). Assumes ETRS89 and WGS 84 can be considered the same to within the accuracy of the transformation.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4230','EPSG','4326',1.0,-116.8,-106.4,-154.4,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Gib',0);
INSERT INTO "usage" VALUES('EPSG','8550','helmert_transformation','EPSG','1629','EPSG','1105','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1630','ED50 to ETRS89 (6)','May be taken as approximate transformation ED50 to WGS 84 - see code 1631.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4230','EPSG','4258',1.5,-181.5,-90.3,-187.2,'EPSG','9001',0.144,0.492,-0.394,'EPSG','9104',17.57,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'CNIG-Esp Bal',0);
INSERT INTO "usage" VALUES('EPSG','8551','helmert_transformation','EPSG','1630','EPSG','2335','EPSG','1151');
INSERT INTO "helmert_transformation" VALUES('EPSG','1631','ED50 to WGS 84 (27)','Parameter values from ED50 to ETRS89 (6) (code 1630). Assumes ETRS89 and WGS 84 can be considered the same to within the accuracy of the transformation.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4230','EPSG','4326',1.5,-181.5,-90.3,-187.2,'EPSG','9001',0.144,0.492,-0.394,'EPSG','9104',17.57,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Esp Bal',0);
INSERT INTO "usage" VALUES('EPSG','8552','helmert_transformation','EPSG','1631','EPSG','2335','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1632','ED50 to ETRS89 (7)','May be taken as approximate transformation ED50 to WGS 84 - see code 1633.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4230','EPSG','4258',1.5,-131.0,-100.3,-163.4,'EPSG','9001',-1.244,-0.02,-1.144,'EPSG','9104',9.39,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'CNIG-Esp',0);
INSERT INTO "usage" VALUES('EPSG','8553','helmert_transformation','EPSG','1632','EPSG','2336','EPSG','1151');
INSERT INTO "helmert_transformation" VALUES('EPSG','1633','ED50 to WGS 84 (28)','Parameter values from ED50 to ETRS89 (7) (code 1632). Assumes ETRS89 and WGS 84 can be considered the same to within the accuracy of the transformation.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4230','EPSG','4326',1.5,-131.0,-100.3,-163.4,'EPSG','9001',-1.244,-0.02,-1.144,'EPSG','9104',9.39,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Esp',0);
INSERT INTO "usage" VALUES('EPSG','8554','helmert_transformation','EPSG','1633','EPSG','2336','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1634','ED50 to ETRS89 (8)','May be taken as approximate transformation ED50 to WGS 84 - see code 1635.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4230','EPSG','4258',1.5,-178.4,-83.2,-221.3,'EPSG','9001',0.54,-0.532,-0.126,'EPSG','9104',21.2,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'CNIG-Esp NW',0);
INSERT INTO "usage" VALUES('EPSG','8555','helmert_transformation','EPSG','1634','EPSG','2337','EPSG','1151');
INSERT INTO "helmert_transformation" VALUES('EPSG','1635','ED50 to WGS 84 (29)','Parameter values from ED50 to ETRS89 (8) (code 1634). Assumes ETRS89 and WGS 84 can be considered the same to within the accuracy of the transformation.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4230','EPSG','4326',1.5,-178.4,-83.2,-221.3,'EPSG','9001',0.54,-0.532,-0.126,'EPSG','9104',21.2,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Esp NW',0);
INSERT INTO "usage" VALUES('EPSG','8556','helmert_transformation','EPSG','1635','EPSG','2337','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1638','KKJ to ETRS89 (1)','Parameter values derived from ETRF89 (EUREF-89) coordinates based on the Finnish GPS campaign in 1992 (“GPS-Suomi”). May be taken as approximate transformation KKJ to WGS 84 - see code 1639. Replaced by KKJ to EUREF-FIN (2) (code 10098).','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4123','EPSG','4258',1.5,-90.7,-106.1,-119.2,'EPSG','9001',4.09,0.218,-1.05,'EPSG','9104',1.37,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NLS-Fin',0);
INSERT INTO "usage" VALUES('EPSG','8559','helmert_transformation','EPSG','1638','EPSG','3333','EPSG','1151');
INSERT INTO "helmert_transformation" VALUES('EPSG','1639','KKJ to WGS 84 (1)','Parameter values from KKJ to ETRS89 (1) (code 1638). Assumes ETRS89 and WGS 84 can be considered the same to within the accuracy of the transformation. Replaced by KKJ to WGS 84 (2) (code 10099).','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4123','EPSG','4326',1.5,-90.7,-106.1,-119.2,'EPSG','9001',4.09,0.218,-1.05,'EPSG','9104',1.37,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Fin',0);
INSERT INTO "usage" VALUES('EPSG','8560','helmert_transformation','EPSG','1639','EPSG','3333','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1640','TM65 to ETRS89 (1)','May be taken as approximate transformation TM65 to WGS 84 - see code 1641.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4299','EPSG','4258',NULL,482.5,-130.6,564.6,'EPSG','9001',-1.042,-0.214,-0.631,'EPSG','9104',8.15,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OSI-Ire',1);
INSERT INTO "usage" VALUES('EPSG','8561','helmert_transformation','EPSG','1640','EPSG','1305','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','1641','TM65 to WGS 84 (2)','Parameter values from TM75 to ETRS89 (2) (code 1953). Assumes each pair of (i) TM65 and TM75, and (ii) ETRS89 and WGS 84, can be considered the same to within the accuracy of the transformation.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4299','EPSG','4326',1.0,482.5,-130.6,564.6,'EPSG','9001',-1.042,-0.214,-0.631,'EPSG','9104',8.15,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Ire',0);
INSERT INTO "usage" VALUES('EPSG','8562','helmert_transformation','EPSG','1641','EPSG','1305','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1642','LUREF to ETRS89 (1)','Derived by EuroGeographics from ACT transformation parameters (see CT code 1078). Not published by ACT. Replaced by LUREF to ETRS89 (3) (CT code 5485). May be taken as approximate transformation LUREF to WGS 84 - see CT code 1643.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4181','EPSG','4258',1.0,-193.0,13.7,-39.3,'EPSG','9001',-0.41,-2.933,2.688,'EPSG','9104',0.43,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'ACT-Lux',0);
INSERT INTO "usage" VALUES('EPSG','8563','helmert_transformation','EPSG','1642','EPSG','1146','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','1643','LUREF to WGS 84 (1)','Parameter values from LUREF to ETRS89 (1) (code 1642). Assumes ETRS89 and WGS 84 can be considered the same to within the accuracy of the transformation.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4181','EPSG','4326',1.0,-193.0,13.7,-39.3,'EPSG','9001',-0.41,-2.933,2.688,'EPSG','9104',0.43,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Lux',0);
INSERT INTO "usage" VALUES('EPSG','8564','helmert_transformation','EPSG','1643','EPSG','1146','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1644','Pulkovo 1942(58) to ETRS89 (1)','May be taken as approximate transformation Pulkovo 1942(58) to WGS 84 - see code 1645. Parameter values given to greater precision but to no better accuracy in GUGiK Technical Instruction G-2, Warsaw 2001.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4179','EPSG','4258',1.0,33.4,-146.6,-76.3,'EPSG','9001',-0.359,-0.053,0.844,'EPSG','9104',-0.84,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'GUGK-Pol',0);
INSERT INTO "usage" VALUES('EPSG','8565','helmert_transformation','EPSG','1644','EPSG','3293','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','1645','Pulkovo 1942(58) to WGS 84 (1)','Parameter values from Pulkovo 1942(58) to ETRS89 (1) (code 1644). Assumes ETRS89 and WGS 84 can be considered the same to within the accuracy of the transformation.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4179','EPSG','4326',1.0,33.4,-146.6,-76.3,'EPSG','9001',-0.359,-0.053,0.844,'EPSG','9104',-0.84,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Pol',0);
INSERT INTO "usage" VALUES('EPSG','8566','helmert_transformation','EPSG','1645','EPSG','3293','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1646','CH1903 to ETRS89 (1)','Parameter values from CH1903+ to ETRS89 (tfm code 1647). In EPSG db v5.2 to v8.9 given to 1dm; the difference in output of cms is considered by swisstopo to be insignificant given the tfm accuracy. Superseded by CH1903 to ETRS89 (2) (tfm code 7674).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4149','EPSG','4258',1.5,674.374,15.056,405.346,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'BfL-Che',0);
INSERT INTO "usage" VALUES('EPSG','8567','helmert_transformation','EPSG','1646','EPSG','1286','EPSG','1084');
INSERT INTO "helmert_transformation" VALUES('EPSG','1647','CH1903+ to ETRS89 (1)','This transformation is also given as CH1903+ to CHTRS95 (1) (code 1509). CHTRS95 is a national realization of ETRS89. May be taken as approximate transformation CH1903+ to WGS 84 - see code 1676.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4150','EPSG','4258',0.1,674.374,15.056,405.346,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'BfL-Che',0);
INSERT INTO "usage" VALUES('EPSG','8568','helmert_transformation','EPSG','1647','EPSG','1286','EPSG','1031');
INSERT INTO "helmert_transformation" VALUES('EPSG','1648','EST97 to ETRS89 (1)','EST97 is a national realization of ETRS89. May be taken as approximate transformation EST97 to WGS 84 - see code 1649.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4180','EPSG','4258',0.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NLB-Est',0);
INSERT INTO "usage" VALUES('EPSG','8569','helmert_transformation','EPSG','1648','EPSG','1090','EPSG','1161');
INSERT INTO "helmert_transformation" VALUES('EPSG','1649','EST97 to WGS 84 (1)','Approximation at the +/- 1m level assuming ETRS89 and WGS 84 can be considered the same to within the accuracy of the transformation. EST97 is a national densification of ETRS89.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4180','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Est',0);
INSERT INTO "usage" VALUES('EPSG','8570','helmert_transformation','EPSG','1649','EPSG','1090','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1650','ED50 to ETRS89 (10)','These same parameter values are used to transform to WGS 84. See ED50 to WGS 84 (17) (code 1275).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4230','EPSG','4258',2.0,-84.0,-97.0,-117.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Fra',0);
INSERT INTO "usage" VALUES('EPSG','8571','helmert_transformation','EPSG','1650','EPSG','1096','EPSG','1042');
INSERT INTO "helmert_transformation" VALUES('EPSG','1651','NTF to ETRS89 (1)','These same parameter values are used to transform to WGS 84. See NTF to WGS 84 (1) (code 1193).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4275','EPSG','4258',2.0,-168.0,-60.0,320.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Fra',0);
INSERT INTO "usage" VALUES('EPSG','8572','helmert_transformation','EPSG','1651','EPSG','3694','EPSG','1042');
INSERT INTO "helmert_transformation" VALUES('EPSG','1652','BD72 to ETRS89 (1)','May be taken as approximate transformation BD72 to WGS 84 - see code 1609.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4313','EPSG','4258',1.0,-99.1,53.3,-112.5,'EPSG','9001',0.419,-0.83,1.885,'EPSG','9104',-1.0,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Bel',0);
INSERT INTO "usage" VALUES('EPSG','8573','helmert_transformation','EPSG','1652','EPSG','1347','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','1653','NGO 1948 to ETRS89-NOR [EUREF89] (1)','May be taken as approximate transformation NGO 1948 to WGS 84 - see code 1654.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4273','EPSG','10875',3.0,278.3,93.0,474.5,'EPSG','9001',7.889,0.05,-6.61,'EPSG','9104',6.21,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'SKV-Nor',0);
INSERT INTO "usage" VALUES('EPSG','8574','helmert_transformation','EPSG','1653','EPSG','1352','EPSG','1043');
INSERT INTO "helmert_transformation" VALUES('EPSG','1654','NGO 1948 to WGS 84 (1)','Parameter values from NGO 1948 to ETRS89 (1) (code 1653). Assumes ETRS89 and WGS 84 can be considered the same to within the accuracy of the transformation.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4273','EPSG','4326',3.0,278.3,93.0,474.5,'EPSG','9001',7.889,0.05,-6.61,'EPSG','9104',6.21,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Nor',0);
INSERT INTO "usage" VALUES('EPSG','8575','helmert_transformation','EPSG','1654','EPSG','1352','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1655','Lisbon to ETRS89 (1)','Derived in 2000 at 8 stations. Replaced by 2001 derivation (tfm code 1997). May be taken as approximate transformation to WGS 84 - see tfm code 1656,','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4207','EPSG','4258',3.0,-280.9,-89.8,130.2,'EPSG','9001',-1.721,0.355,-0.371,'EPSG','9104',-5.92,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'ICC-Prt 2000',0);
INSERT INTO "usage" VALUES('EPSG','8576','helmert_transformation','EPSG','1655','EPSG','1294','EPSG','1043');
INSERT INTO "helmert_transformation" VALUES('EPSG','1656','Lisbon to WGS 84 (1)','Parameter values from Lisbon to ETRS89 (1) (code 1655). Assumes ETRS89 and WGS 84 can be considered the same to within the accuracy of the transformation. Replaced by Lisbon to WGS 84 (4) (code 1988).','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4207','EPSG','4326',3.0,-280.9,-89.8,130.2,'EPSG','9001',-1.721,0.355,-0.371,'EPSG','9104',-5.92,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Prt 2000',0);
INSERT INTO "usage" VALUES('EPSG','8577','helmert_transformation','EPSG','1656','EPSG','1294','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1657','Datum 73 to ETRS89 (1)','Derived in 2000 at 8 stations. Replaced by 2001 derivation (tfm code 1992). May be taken as approximate tfm to WGS 84 - see tfm 1658.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4274','EPSG','4258',2.0,-238.2,85.2,29.9,'EPSG','9001',0.166,0.046,1.248,'EPSG','9104',2.03,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'ICC-Prt 2000',0);
INSERT INTO "usage" VALUES('EPSG','8578','helmert_transformation','EPSG','1657','EPSG','1294','EPSG','1042');
INSERT INTO "helmert_transformation" VALUES('EPSG','1658','Datum 73 to WGS 84 (1)','Parameter values from Datum 73 to ETRS89 (1) (code 1657). Assumes ETRS89 and WGS 84 can be considered the same to within the accuracy of the transformation. Replaced by Datum 73 to WGS 84 (4) (tfm code 1987).','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4274','EPSG','4326',2.0,-238.2,85.2,29.9,'EPSG','9001',0.166,0.046,1.248,'EPSG','9104',2.03,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Prt 2000',0);
INSERT INTO "usage" VALUES('EPSG','8579','helmert_transformation','EPSG','1658','EPSG','1294','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1659','Monte Mario to ETRS89 (1)','May be taken as approximate transformation Monte Mario to WGS 84 - see code 1660. For more accurate transformations to explicit realizations of ETRS89 see Monte Mario to IGM95 (4) and Monte Mario to RDN2008 (5), CTs 9733 and 9734.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4265','EPSG','4258',4.0,-104.1,-49.1,-9.9,'EPSG','9001',0.971,-2.917,0.714,'EPSG','9104',-11.68,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGM-Ita main',0);
INSERT INTO "usage" VALUES('EPSG','8580','helmert_transformation','EPSG','1659','EPSG','2372','EPSG','1044');
INSERT INTO "helmert_transformation" VALUES('EPSG','1660','Monte Mario to WGS 84 (4)','Parameter values from Monte Mario to ETRS89 (1) (code 1659). Assumes ETRS89 and WGS 84 can be considered the same to within the accuracy of the transformation.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4265','EPSG','4326',4.0,-104.1,-49.1,-9.9,'EPSG','9001',0.971,-2.917,0.714,'EPSG','9104',-11.68,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Ita main',0);
INSERT INTO "usage" VALUES('EPSG','8581','helmert_transformation','EPSG','1660','EPSG','2372','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1661','Monte Mario to ETRS89 (2)','May be taken as approximate transformation Monte Mario to WGS 84 - see code 1662. For more accurate transformations to explicit realizations of ETRS89 see Monte Mario to IGM95 (4) and Monte Mario to RDN2008 (5), CTs 9733 and 9734.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4265','EPSG','4258',4.0,-168.6,-34.0,38.6,'EPSG','9001',-0.374,-0.679,-1.379,'EPSG','9104',-9.48,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGM-Ita Sar',0);
INSERT INTO "usage" VALUES('EPSG','8582','helmert_transformation','EPSG','1661','EPSG','2339','EPSG','1044');
INSERT INTO "helmert_transformation" VALUES('EPSG','1662','Monte Mario to WGS 84 (2)','Parameter values from Monte Mario to ETRS89 (2) (code 1661). Assumes ETRS89 and WGS 84 can be considered the same to within the accuracy of the transformation.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4265','EPSG','4326',4.0,-168.6,-34.0,38.6,'EPSG','9001',-0.374,-0.679,-1.379,'EPSG','9104',-9.48,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Ita Sar',0);
INSERT INTO "usage" VALUES('EPSG','8583','helmert_transformation','EPSG','1662','EPSG','2339','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1663','Monte Mario to ETRS89 (3)','May be taken as approximate transformation Monte Mario to WGS 84 - see code 1664. For more accurate transformations to explicit realizations of ETRS89 see Monte Mario to IGM95 (4) and Monte Mario to RDN2008 (5), CTs 9733 and 9734.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4265','EPSG','4258',4.0,-50.2,-50.4,84.8,'EPSG','9001',-0.69,-2.012,0.459,'EPSG','9104',-28.08,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGM-Ita Sic',0);
INSERT INTO "usage" VALUES('EPSG','8584','helmert_transformation','EPSG','1663','EPSG','2340','EPSG','1044');
INSERT INTO "helmert_transformation" VALUES('EPSG','1664','Monte Mario to WGS 84 (3)','Parameter values from Monte Mario to ETRS89 (3) (code 1663). Assumes ETRS89 and WGS 84 can be considered the same to within the accuracy of the transformation.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4265','EPSG','4326',4.0,-50.2,-50.4,84.8,'EPSG','9001',-0.69,-2.012,0.459,'EPSG','9104',-28.08,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Ita Sic',0);
INSERT INTO "usage" VALUES('EPSG','8585','helmert_transformation','EPSG','1664','EPSG','2340','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1665','AGD66 to WGS 84 (12)','Parameter values from AGD66 to GDA94 (2) (code 1458). Approximation assuming WGS 84 is equivalent to GDA94; ignores the low accuracy of the WGS 84 ensemble and the inconsistent application of tectonic plate motion to WGS 84 data.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4202','EPSG','4326',3.0,-129.193,-41.212,130.73,'EPSG','9001',-0.246,-0.374,-0.329,'EPSG','9104',-2.955,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-ACT 1m',0);
INSERT INTO "usage" VALUES('EPSG','8586','helmert_transformation','EPSG','1665','EPSG','2283','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1666','AGD66 to WGS 84 (13)','Parameter values from AGD66 to GDA94 (4) (code 1460). Approximation assuming WGS 84 is equivalent to GDA94; ignores the low accuracy of the WGS 84 ensemble and the inconsistent application of tectonic plate motion to WGS 84 data.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4202','EPSG','4326',3.0,-119.353,-48.301,139.484,'EPSG','9001',-0.415,-0.26,-0.437,'EPSG','9104',-0.613,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-NSW Vic 1m',0);
INSERT INTO "usage" VALUES('EPSG','8587','helmert_transformation','EPSG','1666','EPSG','2286','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1667','AGD66 to WGS 84 (14)','Parameter values from AGD66 to GDA94 (8) (code 1594). Approximation assuming WGS 84 is equivalent to GDA94; ignores the low accuracy of the WGS 84 ensemble and the inconsistent application of tectonic plate motion to WGS 84 data.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4202','EPSG','4326',3.0,-120.271,-64.543,161.632,'EPSG','9001',-0.217,0.067,0.129,'EPSG','9104',2.499,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Tas 1m',0);
INSERT INTO "usage" VALUES('EPSG','8588','helmert_transformation','EPSG','1667','EPSG','1282','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1668','AGD66 to WGS 84 (15)','Parameter values from AGD66 to GDA94 (9) (code 1595). Approximation assuming WGS 84 is equivalent to GDA94; ignores the low accuracy of the WGS 84 ensemble and the inconsistent application of tectonic plate motion to WGS 84 data.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4202','EPSG','4326',3.0,-124.133,-42.003,137.4,'EPSG','9001',0.008,-0.557,-0.178,'EPSG','9104',-1.854,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-NT 1m',0);
INSERT INTO "usage" VALUES('EPSG','8589','helmert_transformation','EPSG','1668','EPSG','2284','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1669','AGD84 to WGS 84 (7)','Parameter values from AGD84 to GDA94 (2) (code 1280). Approximation assuming WGS 84 is equivalent to GDA94; ignores low accuracy of the WGS 84 ensemble and inconsistent application of tectonic plate motion. Replaces AGD84 to WGS 84 (2) (code 1236).','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4203','EPSG','4326',3.0,-117.763,-51.51,139.061,'EPSG','9001',-0.292,-0.443,-0.277,'EPSG','9104',-0.191,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Aus 1m',0);
INSERT INTO "usage" VALUES('EPSG','8590','helmert_transformation','EPSG','1669','EPSG','2576','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1671','RGF93 v1 to WGS 84 (1)','Approximation at the 1m level assuming that ETRS89 is equivalent to WGS 84 within the accuracy of the transformation. (RGF93 v1 is a national realization of ETRS89).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4171','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Fra',0);
INSERT INTO "usage" VALUES('EPSG','8592','helmert_transformation','EPSG','1671','EPSG','1096','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1672','Amersfoort to WGS 84 (2)','Parameter values from Amersfoort to ETRS89 (1) (code 1751) assuming that ETRS89 is equivalent to WGS 84 within the accuracy of the transformation. Replaces Amersfoort to WGS 84 (1) (code 1112). Replaced by Amersfoort to WGS 84 (3) (code 15934).','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4289','EPSG','4326',1.0,565.04,49.91,465.84,'EPSG','9001',1.9848,-1.7439,9.0587,'EPSG','9109',4.0772,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Nld 2000',0);
INSERT INTO "usage" VALUES('EPSG','8593','helmert_transformation','EPSG','1672','EPSG','1275','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1673','DHDN to WGS 84 (1)','Parameter values from DHDN to ETRS89 (1) (code 1309) assuming that ETRS89 is equivalent to WGS 84 within the accuracy of the transformation. Replaced by DHDN to WGS 84 (2) (tfm code 1777).','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4314','EPSG','4326',5.0,582.0,105.0,414.0,'EPSG','9001',-1.04,-0.35,3.08,'EPSG','9104',8.3,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Deu W',0);
INSERT INTO "usage" VALUES('EPSG','8594','helmert_transformation','EPSG','1673','EPSG','2326','EPSG','1045');
INSERT INTO "helmert_transformation" VALUES('EPSG','1674','Pulkovo 1942(83) to ETRS89 (1)','Mean of 20 stations. May be taken as approximate transformation to WGS 84 - see code 1675. Also given by EuroGeographics at http://crs.ifag.de/ as a Position Vector transformation with changed values for rotations. In 2001 partially replaced by tfm 1775.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4178','EPSG','4258',2.0,24.0,-123.0,-94.0,'EPSG','9001',-0.02,0.25,0.13,'EPSG','9104',1.1,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IfAG-Deu E',0);
INSERT INTO "usage" VALUES('EPSG','8595','helmert_transformation','EPSG','1674','EPSG','1343','EPSG','1042');
INSERT INTO "helmert_transformation" VALUES('EPSG','1675','Pulkovo 1942(83) to WGS 84 (1)','Parameter values from Pulkovo 1942(83) to ETRS89 (1) (code 1674) assuming that ETRS89 is equivalent to WGS 84 within the accuracy of the transformation.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4178','EPSG','4326',2.0,24.0,-123.0,-94.0,'EPSG','9001',-0.02,0.25,0.13,'EPSG','9104',1.1,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Deu E',0);
INSERT INTO "usage" VALUES('EPSG','8596','helmert_transformation','EPSG','1675','EPSG','1343','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1676','CH1903+ to WGS 84 (1)','Parameter values are from CH1903+ to ETRS89 (1) (code 1647) assuming that ETRS89 is equivalent to WGS 84.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4150','EPSG','4326',1.0,674.374,15.056,405.346,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'BfL-CH',0);
INSERT INTO "usage" VALUES('EPSG','8597','helmert_transformation','EPSG','1676','EPSG','1286','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1677','HD72 to WGS 84 (1)','Parameter values taken from HD72 to ETRS89 (1) (code 1273) assuming that ETRS89 is equivalent to WGS 84 within the accuracy of the transformation.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4237','EPSG','4326',NULL,56.0,75.77,15.31,'EPSG','9001',-0.37,-0.2,-0.21,'EPSG','9104',-1.01,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Hun',1);
INSERT INTO "usage" VALUES('EPSG','8598','helmert_transformation','EPSG','1677','EPSG','1119','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','1678','IRENET95 to WGS 84 (1)','Assumes that ETRS89 is equivalent to WGS 84 within the accuracy of the transformation. IRENET95 is a regional realisation of ETRS89.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4173','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Ire',0);
INSERT INTO "usage" VALUES('EPSG','8599','helmert_transformation','EPSG','1678','EPSG','1305','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1679','Pulkovo 1942 to WGS 84 (2)','Parameter values taken from Pulkovo 1942 to LKS94(ETRS89) (1) (code 1274) assuming that LKS94(ETRS89) is equivalent to WGS 84 within the accuracy of the transformation.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4284','EPSG','4326',9.0,-40.595,-18.55,-69.339,'EPSG','9001',-2.508,-1.832,2.611,'EPSG','9104',-4.299,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Ltu',0);
INSERT INTO "usage" VALUES('EPSG','8600','helmert_transformation','EPSG','1679','EPSG','3272','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1680','RT90 to WGS 84 (1)','Parameter values from RT90 to ETRS89 (1) (code 1437) assuming that ETRS89 is equivalent to WGS 84 within the accuracy of the transformation. Replaced by RT90 to WGS 84 (2) (code 1896) from 2001.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4124','EPSG','4326',1.0,419.3836,99.3335,591.3451,'EPSG','9001',-0.850389,-1.817277,7.862238,'EPSG','9104',-0.99496,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Swe',0);
INSERT INTO "usage" VALUES('EPSG','8601','helmert_transformation','EPSG','1680','EPSG','1225','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1682','South Yemen to WGS 84 (1)','Parameter values taken from South Yemen to Yemen NGN96 (1) (code 1539) assuming that NGN96 is equivalent to WGS 84 within the accuracy of the transformation.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4164','EPSG','4326',5.0,-76.0,-138.0,67.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Yem South',0);
INSERT INTO "usage" VALUES('EPSG','8603','helmert_transformation','EPSG','1682','EPSG','1340','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1683','Tete to WGS 84 (1)','Parameter values taken from Tete to Moznet (1) (code 1297) assuming that Moznet is equivalent to WGS 84 within the accuracy of the transformation.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4127','EPSG','4326',30.0,-115.064,-87.39,-101.716,'EPSG','9001',0.058,-4.001,2.062,'EPSG','9104',9.366,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Moz',0);
INSERT INTO "usage" VALUES('EPSG','8604','helmert_transformation','EPSG','1683','EPSG','3281','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1684','Tete to WGS 84 (2)','Parameter values taken from Tete to Moznet (2) (code 1298) assuming that Moznet is equivalent to WGS 84 within the accuracy of the transformation.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4127','EPSG','4326',1.0,-82.875,-57.097,-156.768,'EPSG','9001',2.158,-1.524,0.982,'EPSG','9104',-0.359,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Moz A',0);
INSERT INTO "usage" VALUES('EPSG','8605','helmert_transformation','EPSG','1684','EPSG','2350','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1685','Tete to WGS 84 (3)','Parameter values taken from Tete to Moznet (3) (code 1299) assuming that Moznet is equivalent to WGS 84 within the accuracy of the transformation.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4127','EPSG','4326',4.0,-138.527,-91.999,-114.591,'EPSG','9001',0.14,-3.363,2.217,'EPSG','9104',11.748,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Moz B',0);
INSERT INTO "usage" VALUES('EPSG','8606','helmert_transformation','EPSG','1685','EPSG','2351','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1686','Tete to WGS 84 (4)','Parameter values taken from Tete to Moznet (4) (code 1300) assuming that Moznet is equivalent to WGS 84 within the accuracy of the transformation.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4127','EPSG','4326',3.0,-73.472,-51.66,-112.482,'EPSG','9001',-0.953,-4.6,2.368,'EPSG','9104',0.586,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Moz C',0);
INSERT INTO "usage" VALUES('EPSG','8607','helmert_transformation','EPSG','1686','EPSG','2352','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1687','Tete to WGS 84 (5)','Parameter values taken from Tete to Moznet (5) (code 1301) assuming that Moznet is equivalent to WGS 84 within the accuracy of the transformation.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4127','EPSG','4326',10.0,219.315,168.975,-166.145,'EPSG','9001',-0.198,-5.926,2.356,'EPSG','9104',-57.104,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Moz D',0);
INSERT INTO "usage" VALUES('EPSG','8608','helmert_transformation','EPSG','1687','EPSG','2353','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1701','NZGD49 to NZGD2000 (2)','For better accuracy use NZGD49 to NZGD2000 (3) (code 1568).','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4272','EPSG','4167',4.0,59.47,-5.04,187.44,'EPSG','9001',-0.47,0.1,-1.024,'EPSG','9104',-4.5993,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OSG-Nzl 4m',0);
INSERT INTO "usage" VALUES('EPSG','8622','helmert_transformation','EPSG','1701','EPSG','3285','EPSG','1044');
INSERT INTO "helmert_transformation" VALUES('EPSG','1751','Amersfoort to ETRS89 (1)','Replaced by Amersfoort to ETRS89 (3) (tfm code 15739). Dutch sources also quote an equivalent transformation using the Molodenski-Badekas 10-parameter method (M-B) - see tfm code 1066.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4289','EPSG','4258',0.5,565.04,49.91,465.84,'EPSG','9001',1.9848,-1.7439,9.0587,'EPSG','9109',4.0772,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NCG-Nld 2000 CF',0);
INSERT INTO "usage" VALUES('EPSG','8672','helmert_transformation','EPSG','1751','EPSG','1275','EPSG','1035');
INSERT INTO "helmert_transformation" VALUES('EPSG','1753','CH1903 to WGS 84 (1)','Implemented in Bundesamt für Landestopografie programme GRANIT. Used from 1987 to 1997. Not recommended for current usage - replaced by CH1903 to WGS 84 (2) (code 1766).','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4149','EPSG','4326',1.0,660.077,13.551,369.344,'EPSG','9001',2.484,1.783,2.939,'EPSG','9113',5.66,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'BfL-CH 1',0);
INSERT INTO "usage" VALUES('EPSG','8674','helmert_transformation','EPSG','1753','EPSG','1286','EPSG','1232');
INSERT INTO "helmert_transformation" VALUES('EPSG','1754','Minna to WGS 84 (3)','Derived at 8 stations across the Niger delta. Used by Shell SPDC throughout southern Nigeria onshore, delta and shallow offshore from 1994 and by Total in OPL246. Sometimes given with parameter values to greater resolution; values here are adequate.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4263','EPSG','4326',5.0,-111.92,-87.85,114.5,'EPSG','9001',1.875,0.202,0.219,'EPSG','9104',0.032,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'SHL-Nga S',0);
INSERT INTO "usage" VALUES('EPSG','8675','helmert_transformation','EPSG','1754','EPSG','2371','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1766','CH1903 to WGS 84 (2)','Parameters values from CH1903 to ETRS89 (1) (tfm code 1646) assuming ETRS89 and WGS 84 can be considered the same to within the accuracy of the transformation. Replaces CH1903 to WGS 84 (1) (code 1753). Replaced by CH1903 to WGS 84 (3) (code 7788).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4149','EPSG','4326',1.5,674.374,15.056,405.346,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'BfL-CH 2',0);
INSERT INTO "usage" VALUES('EPSG','8687','helmert_transformation','EPSG','1766','EPSG','1286','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1767','REGVEN to SIRGAS 1995 (1)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4189','EPSG','4170',0.02,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'CN-Ven',0);
INSERT INTO "usage" VALUES('EPSG','8688','helmert_transformation','EPSG','1767','EPSG','1251','EPSG','1255');
INSERT INTO "helmert_transformation" VALUES('EPSG','1768','REGVEN to WGS 84 (1)','Approximation at the +/- 1m level assuming that REGVEN is equivalent to WGS 84.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4189','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Ven',0);
INSERT INTO "usage" VALUES('EPSG','8689','helmert_transformation','EPSG','1768','EPSG','1251','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1769','PSAD56 to REGVEN (1)','May be taken as transformation to WGS 84 - see PSAD56 to WGS 84 (13) (code 1095).','EPSG','9636','Molodensky-Badekas (CF geog2D domain)','EPSG','4248','EPSG','4189',15.0,-270.933,115.599,-360.226,'EPSG','9001',-5.266,-1.238,2.381,'EPSG','9104',-5.109,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,2464351.59,-5783466.61,974809.81,'EPSG','9001','IGSB-Ven',0);
INSERT INTO "usage" VALUES('EPSG','8690','helmert_transformation','EPSG','1769','EPSG','3327','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','1770','PSAD56 to WGS84 (1)','Parameter vales are from PSAD56 to REGVEN (1) (code 1769) assuming that REGVEN is equivalent to WGS 84 within the accuracy of the transformation.','EPSG','9636','Molodensky-Badekas (CF geog2D domain)','EPSG','4248','EPSG','4326',NULL,-270.933,115.599,-360.226,'EPSG','9001',-5.266,-1.238,2.381,'EPSG','9104',-5.109,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,2464351.59,-5783466.61,974809.81,'EPSG','9001','EPSG-Ven',1);
INSERT INTO "usage" VALUES('EPSG','8691','helmert_transformation','EPSG','1770','EPSG','1251','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1771','La Canoa to REGVEN (1)','May be used as transformation to WGS 84 - see La Canoa to WGS 84 (13) (code 1096)','EPSG','9636','Molodensky-Badekas (CF geog2D domain)','EPSG','4247','EPSG','4189',15.0,-270.933,115.599,-360.226,'EPSG','9001',-5.266,-1.238,2.381,'EPSG','9104',-5.109,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,2464351.59,-5783466.61,974809.81,'EPSG','9001','IGSB-Ven',0);
INSERT INTO "usage" VALUES('EPSG','8692','helmert_transformation','EPSG','1771','EPSG','3327','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','1772','La Canoa to WGS84 (1)','Parameter values are from La Canoa to REGVEN (1) (code 1771) assuming that REGVEN is equivalent to WGS 84 within the accuracy of the transformation.','EPSG','9636','Molodensky-Badekas (CF geog2D domain)','EPSG','4247','EPSG','4326',NULL,-270.933,115.599,-360.226,'EPSG','9001',-5.266,-1.238,2.381,'EPSG','9104',-5.109,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,2464351.59,-5783466.61,974809.81,'EPSG','9001','EPSG-Ven',1);
INSERT INTO "usage" VALUES('EPSG','8693','helmert_transformation','EPSG','1772','EPSG','1251','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1773','POSGAR 98 to WGS 84 (1)','Approximation at the +/- 1m level assuming that POSGAR 98 is equivalent to WGS 84.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4190','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Arg',0);
INSERT INTO "usage" VALUES('EPSG','8694','helmert_transformation','EPSG','1773','EPSG','1033','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1774','POSGAR 98 to SIRGAS 1995 (1)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4190','EPSG','4170',0.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGM-Arg',0);
INSERT INTO "usage" VALUES('EPSG','8695','helmert_transformation','EPSG','1774','EPSG','1033','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','1775','Pulkovo 1942(83) to ETRS89 (2)','Derived at 35 points of the German GPS Network DREF. From 2001 replaces Pulkovo 1942(83) to ETRS89 (1) (code 1674) within Mecklenburg-Vorpommern and Sachsen-Anhalt. From 2009 replaces tfm 1674 in all other states of former East Germany.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4178','EPSG','4258',0.1,24.9,-126.4,-93.2,'EPSG','9001',-0.063,-0.247,-0.041,'EPSG','9104',1.01,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IfAG-Deu E 0.1m',0);
INSERT INTO "usage" VALUES('EPSG','8696','helmert_transformation','EPSG','1775','EPSG','1343','EPSG','1031');
INSERT INTO "helmert_transformation" VALUES('EPSG','1776','DHDN to ETRS89 (2)','Mean of 109 stations. Replaces DHDN to ETRS89 (1) (tfm code 1309). May be taken as approximate transformation DHDN to WGS 84 - see code 1777.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4314','EPSG','4258',3.0,598.1,73.7,418.2,'EPSG','9001',0.202,0.045,-2.455,'EPSG','9104',6.7,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IfAG-Deu W 3m',0);
INSERT INTO "usage" VALUES('EPSG','8697','helmert_transformation','EPSG','1776','EPSG','2326','EPSG','1043');
INSERT INTO "helmert_transformation" VALUES('EPSG','1777','DHDN to WGS 84 (2)','Parameter values from DHDN to ETRS89 (2) (code 1776) assuming that ETRS89 is equivalent to WGS 84 within the accuracy of the transformation. Replaces DHDN to WGS 84 (1) (tfm code 1673).','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4314','EPSG','4326',3.0,598.1,73.7,418.2,'EPSG','9001',0.202,0.045,-2.455,'EPSG','9104',6.7,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Deu W 3m',0);
INSERT INTO "usage" VALUES('EPSG','8698','helmert_transformation','EPSG','1777','EPSG','2326','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1778','DHDN to ETRS89 (3)','Derived in 2001 at 41 points of the German GPS Network DREF.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4314','EPSG','4258',1.0,597.1,71.4,412.1,'EPSG','9001',0.894,0.068,-1.563,'EPSG','9104',7.58,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IfAG-Deu W-S',0);
INSERT INTO "usage" VALUES('EPSG','8699','helmert_transformation','EPSG','1778','EPSG','2543','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','1779','DHDN to ETRS89 (4)','Derived at 27 points of the German GPS Network DREF.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4314','EPSG','4258',1.0,584.8,67.0,400.3,'EPSG','9001',0.105,0.013,-2.378,'EPSG','9104',10.29,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IfAG-Deu W-cen',0);
INSERT INTO "usage" VALUES('EPSG','8700','helmert_transformation','EPSG','1779','EPSG','2542','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','1780','DHDN to ETRS89 (5)','Derived at 21 points of the German GPS Network DREF.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4314','EPSG','4258',1.0,590.5,69.5,411.6,'EPSG','9001',-0.796,-0.052,-3.601,'EPSG','9104',8.3,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IfAG-Deu W-N',0);
INSERT INTO "usage" VALUES('EPSG','8701','helmert_transformation','EPSG','1780','EPSG','2541','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','1781','DHDN to ETRS89 (6)','Derived at 10 points of the German GPS Network DREF.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4314','EPSG','4258',0.1,599.4,72.4,419.2,'EPSG','9001',-0.062,-0.022,-2.723,'EPSG','9104',6.46,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IfAG-Deu Thur',1);
INSERT INTO "usage" VALUES('EPSG','8702','helmert_transformation','EPSG','1781','EPSG','2544','EPSG','1031');
INSERT INTO "helmert_transformation" VALUES('EPSG','1782','DHDN to ETRS89 (7)','Derived at 35 points of the German GPS Network DREF.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4314','EPSG','4258',0.1,612.4,77.0,440.2,'EPSG','9001',-0.054,0.057,-2.797,'EPSG','9104',2.55,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IfAG-Deu Sach',1);
INSERT INTO "usage" VALUES('EPSG','8703','helmert_transformation','EPSG','1782','EPSG','2545','EPSG','1031');
INSERT INTO "helmert_transformation" VALUES('EPSG','1783','ED50 to ETRS89 (9)','May be taken as approximate transformation ED50 to WGS 84 - see code 1784. Note: the ETRS89 CRS is not used in Turkey.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4230','EPSG','4258',2.0,-84.1,-101.8,-129.7,'EPSG','9001',0.0,0.0,0.468,'EPSG','9104',1.05,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'HGK-Tur',0);
INSERT INTO "usage" VALUES('EPSG','8704','helmert_transformation','EPSG','1783','EPSG','1237','EPSG','1042');
INSERT INTO "helmert_transformation" VALUES('EPSG','1784','ED50 to WGS 84 (30)','Parameter values from ED50 to ETRS89 (9) (code 1783). Assumes ETRS89 and WGS 84 can be considered the same to within the accuracy of the transformation.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4230','EPSG','4326',2.0,-84.1,-101.8,-129.7,'EPSG','9001',0.0,0.0,0.468,'EPSG','9104',1.05,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Tur',0);
INSERT INTO "usage" VALUES('EPSG','8705','helmert_transformation','EPSG','1784','EPSG','1237','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1785','MGI to ETRS89 (3)','May be taken as approximate transformation MGI to WGS 84 - see code 1786.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4312','EPSG','4258',1.0,426.9,142.6,460.1,'EPSG','9001',4.91,4.49,-12.42,'EPSG','9104',17.1,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'GURS-Svn',1);
INSERT INTO "usage" VALUES('EPSG','8706','helmert_transformation','EPSG','1785','EPSG','1212','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','1786','MGI to WGS 84 (5)','Parameter values from MGI to ETRS89 (3) (code 1785). Assumes ETRS89 and WGS 84 can be considered the same to within the accuracy of the transformation.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4312','EPSG','4326',1.0,426.9,142.6,460.1,'EPSG','9001',4.91,4.49,-12.42,'EPSG','9104',17.1,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Svn',1);
INSERT INTO "usage" VALUES('EPSG','8707','helmert_transformation','EPSG','1786','EPSG','1212','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','1787','RT90 to ETRS89 (2)','Derived at 165 points. Supersedes RT90 to ETRS89 (1) (code 1437). May be taken as approximate transformation RT90 to WGS 84 - see code 1787.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4124','EPSG','4258',NULL,414.1,41.3,603.1,'EPSG','9001',-0.855,2.141,-7.023,'EPSG','9104',0.0,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NLS-Swe 2001',1);
INSERT INTO "usage" VALUES('EPSG','8708','helmert_transformation','EPSG','1787','EPSG','1225','EPSG','1031');
INSERT INTO "helmert_transformation" VALUES('EPSG','1788','RT90 to WGS 84 (2)','Parameter values from RT90 to ETRS89 (1) (code 1787) assuming that ETRS89 is equivalent to WGS 84 within the accuracy of the transformation. Supersedes RT90 to WGS 84 (1) (code 1680).','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4124','EPSG','4326',NULL,414.1,41.3,603.1,'EPSG','9001',-0.855,2.141,-7.023,'EPSG','9104',0.0,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Swe 2001',1);
INSERT INTO "usage" VALUES('EPSG','8709','helmert_transformation','EPSG','1788','EPSG','1225','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','1789','Dealui Piscului 1933 to WGS 84 (1)','Parameter values taken from Pulkovo 1942 to WGS 84 (9) (code 1293) assuming that','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4316','EPSG','4326',NULL,103.25,-100.4,-307.19,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NAMR-Rom',1);
INSERT INTO "usage" VALUES('EPSG','8710','helmert_transformation','EPSG','1789','EPSG','1197','EPSG','1025');
INSERT INTO "helmert_transformation" VALUES('EPSG','1790','Lisbon to ETRS89 (2)','Derived in 2001. Supersedes Lisbon to ETRS89 (1) (code 1655).','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4207','EPSG','4258',NULL,-282.1,-72.2,120.0,'EPSG','9001',-1.592,0.145,-0.89,'EPSG','9104',-4.46,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'ICC-Prt 2001',1);
INSERT INTO "usage" VALUES('EPSG','8711','helmert_transformation','EPSG','1790','EPSG','1294','EPSG','1042');
INSERT INTO "helmert_transformation" VALUES('EPSG','1791','Lisbon to WGS 84 (2)','Parameter values from Lisbon to ETRS89 (2) (code 1790). Assumes ETRS89 and WGS 84 can be considered the same to within the accuracy of the transformation.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4207','EPSG','4258',NULL,-282.1,-72.2,120.0,'EPSG','9001',-1.592,0.145,-0.89,'EPSG','9104',-4.46,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Prt 2001',1);
INSERT INTO "usage" VALUES('EPSG','8712','helmert_transformation','EPSG','1791','EPSG','1294','EPSG','1042');
INSERT INTO "helmert_transformation" VALUES('EPSG','1792','Datum 73 to ETRS89 (2)','Derived in 2001. Supersedes Datum 73 to ETRS89 (1) (code 1657).','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4274','EPSG','4258',NULL,-231.0,102.6,29.8,'EPSG','9001',0.615,-0.198,0.881,'EPSG','9104',1.79,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'ICC-Prt 2001',1);
INSERT INTO "usage" VALUES('EPSG','8713','helmert_transformation','EPSG','1792','EPSG','1294','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','1793','Datum 73 to WGS 84 (2)','Parameter values from Datum 73 to ETRS89 (2) (code 1792). Assumes ETRS89 and WGS 84 can be considered the same to within the accuracy of the transformation.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4274','EPSG','4258',NULL,-231.0,102.6,29.8,'EPSG','9001',0.615,-0.198,0.881,'EPSG','9104',1.79,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Prt 2001',1);
INSERT INTO "usage" VALUES('EPSG','8714','helmert_transformation','EPSG','1793','EPSG','1294','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1794','MGI to WGS 84 (6)','For more accurate transformation see MGI to WGS 84 (7) (code 1795).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4312','EPSG','4326',999.0,695.5,-216.6,491.1,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'JPet-Yug',1);
INSERT INTO "usage" VALUES('EPSG','8715','helmert_transformation','EPSG','1794','EPSG','3536','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1795','MGI to WGS 84 (7)','','EPSG','9636','Molodensky-Badekas (CF geog2D domain)','EPSG','4312','EPSG','4326',999.0,408.0895,-288.9616,791.5498,'EPSG','9001',-4.078662,0.022669,9.825424,'EPSG','9104',94.060626,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,4444943.0248,1518098.4827,4302370.0765,'EPSG','9001','JPET-Yug MB',1);
INSERT INTO "usage" VALUES('EPSG','8716','helmert_transformation','EPSG','1795','EPSG','3536','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1796','Manoca 1962 to WGS 84 (1)','Derived at two points, checked at a third by Stolt Comex Seaway and Geoid for Elf.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4193','EPSG','4326',0.5,-70.9,-151.8,-41.4,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'ELF94-Cmr',0);
INSERT INTO "usage" VALUES('EPSG','8717','helmert_transformation','EPSG','1796','EPSG','2555','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1797','Qoornoq 1927 to WGS 84 (1)','Derived at 2 stations. Accuracy 25m, 25m and 32m in X, Y and Z axes.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4194','EPSG','4326',48.0,164.0,138.0,-189.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Grl S',0);
INSERT INTO "usage" VALUES('EPSG','8718','helmert_transformation','EPSG','1797','EPSG','3362','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1798','Qoornoq 1927 to WGS 84 (2)','','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4194','EPSG','4326',1.0,163.511,127.533,-159.789,'EPSG','9001',0.0,0.0,0.814,'EPSG','9104',-0.6,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'KMS-Grl',0);
INSERT INTO "usage" VALUES('EPSG','8719','helmert_transformation','EPSG','1798','EPSG','3362','EPSG','1153');
INSERT INTO "helmert_transformation" VALUES('EPSG','1799','Scoresbysund 1952 to WGS 84 (1)','','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4195','EPSG','4326',1.0,105.0,326.0,-102.5,'EPSG','9001',0.0,0.0,0.814,'EPSG','9104',-0.6,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'KMS-Grl Scosd',0);
INSERT INTO "usage" VALUES('EPSG','8720','helmert_transformation','EPSG','1799','EPSG','2570','EPSG','1153');
INSERT INTO "helmert_transformation" VALUES('EPSG','1800','Ammassalik 1958 to WGS 84 (1)','','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4196','EPSG','4326',1.0,-45.0,417.0,-3.5,'EPSG','9001',0.0,0.0,0.814,'EPSG','9104',-0.6,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'KMS-Grl Ammlk',0);
INSERT INTO "usage" VALUES('EPSG','8721','helmert_transformation','EPSG','1800','EPSG','2571','EPSG','1153');
INSERT INTO "helmert_transformation" VALUES('EPSG','1801','Pointe Noire to WGS 84 (2)','Derived in 1994 by CGG/Topnav using DORIS system on various stations along the coastline.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4282','EPSG','4326',4.0,-145.0,52.7,-291.6,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'CGG94-Cog',0);
INSERT INTO "usage" VALUES('EPSG','8722','helmert_transformation','EPSG','1801','EPSG','2574','EPSG','1024');
INSERT INTO "helmert_transformation" VALUES('EPSG','1802','Pointe Noire to WGS 84 (3)','Derived by Geoid for Elf in May 1995 using GPS and IGS data by tying 4 geodetic points to ITRF93 epoch 1995.4. Used for all offshore Congo operations.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4282','EPSG','4326',0.15,-178.3,-316.7,-131.5,'EPSG','9001',5.278,6.077,10.979,'EPSG','9104',19.166,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'ELF95-Cog',0);
INSERT INTO "usage" VALUES('EPSG','8723','helmert_transformation','EPSG','1802','EPSG','2574','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1805','Garoua to WGS 72BE (1)','Derived in 1981 by Decca Survey France for Elf Serepca.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4197','EPSG','4324',5.0,-56.1,-167.8,13.1,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'ELF-Cmr',0);
INSERT INTO "usage" VALUES('EPSG','8726','helmert_transformation','EPSG','1805','EPSG','2590','EPSG','1216');
INSERT INTO "helmert_transformation" VALUES('EPSG','1806','Kousseri to WGS 72BE (1)','Derived in 1981 by Decca Survey France for Elf Serepca.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4198','EPSG','4324',5.0,-104.4,-136.6,201.2,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'ELF-Cmr',0);
INSERT INTO "usage" VALUES('EPSG','8727','helmert_transformation','EPSG','1806','EPSG','2591','EPSG','1216');
INSERT INTO "helmert_transformation" VALUES('EPSG','1807','Pulkovo 1942 to WGS 84 (13)','Derived via WGS72 values taken from SOCAR Magnavox 1502 manual. Used by AIOC 1995-1997 then replaced by the AIOC97 values (tfm code 1808). Do not confuse with AIOC95 vertical datum as used in southern Caspian Sea and at Sangachal terminal by AIOC.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4284','EPSG','4326',10.0,27.0,-135.0,-84.5,'EPSG','9001',0.0,0.0,0.554,'EPSG','9104',0.2263,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'BP-Aze Aioc95',0);
INSERT INTO "usage" VALUES('EPSG','8728','helmert_transformation','EPSG','1807','EPSG','1038','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1808','Pulkovo 1942 to WGS 84 (14)','Mean of 3 stations in western Georgia, 4 stations in eastern Georgia and 4 stations in eastern Azerbaijan. Derived for use on AIOC early oil western export pipeline, but adopted for all AIOC work replacing the 1995 AIOC transformation (code 1807).','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4284','EPSG','4326',5.0,686.1,-123.5,-574.4,'EPSG','9001',8.045,-23.366,10.791,'EPSG','9104',-2.926,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'BP-Aze Aioc97',0);
INSERT INTO "usage" VALUES('EPSG','8729','helmert_transformation','EPSG','1808','EPSG','2593','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1809','Pulkovo 1942 to WGS 84 (15)','Parameter values calculated by Elf Exploration and Production based on geodetic survey carried out by Azerbaijan State Committee for Geodesy and Cartography.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4284','EPSG','4326',2.0,926.4,-715.9,-186.4,'EPSG','9001',-10.364,-20.78,26.452,'EPSG','9104',-7.224,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'TFE-Aze97',0);
INSERT INTO "usage" VALUES('EPSG','8730','helmert_transformation','EPSG','1809','EPSG','2594','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1810','ED50 to WGS 84 (31)','Derived via concatenation through WGS72. The ED50 to WGS72 step is the Sepplin 1974 value for all Europe.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4230','EPSG','4326',15.0,-84.0,-103.0,-122.5,'EPSG','9001',0.0,0.0,0.554,'EPSG','9104',0.2263,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'wgc72-Egy',0);
INSERT INTO "usage" VALUES('EPSG','8731','helmert_transformation','EPSG','1810','EPSG','2595','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1811','PSAD56 to WGS 84 (12)','Used by Petrobras for shelf operations.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4248','EPSG','4326',10.0,-291.87,106.37,-364.52,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'PB-Braz N',0);
INSERT INTO "usage" VALUES('EPSG','8732','helmert_transformation','EPSG','1811','EPSG','1754','EPSG','1216');
INSERT INTO "helmert_transformation" VALUES('EPSG','1812','Indian 1975 to WGS 84 (4)','','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4240','EPSG','4326',3.0,293.0,836.0,318.0,'EPSG','9001',0.5,1.6,-2.8,'EPSG','9104',2.1,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'Auslig-Tha',0);
INSERT INTO "usage" VALUES('EPSG','8733','helmert_transformation','EPSG','1812','EPSG','3317','EPSG','1028');
INSERT INTO "helmert_transformation" VALUES('EPSG','1813','Batavia to WGS 84 (2)','Used by ARCO offshore NW Java area.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4211','EPSG','4326',5.0,-378.873,676.002,-46.255,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'ARCO-Idn ONWJ',0);
INSERT INTO "usage" VALUES('EPSG','8734','helmert_transformation','EPSG','1813','EPSG','2577','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1814','Batavia to WGS 84 (3)','Used by PT Komaritim for Nippon Steel during East Java Gas Pipeline construction.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4211','EPSG','4326',5.0,-377.7,675.1,-52.2,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'KOM-Idn EJGP',0);
INSERT INTO "usage" VALUES('EPSG','8735','helmert_transformation','EPSG','1814','EPSG','2588','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1815','Nord Sahara 1959 to WGS 84 (4)','Used by BP in District 3 and In Salah Gas. May be taken as a transformation to RGSH2020 - see CT 10339.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4307','EPSG','4326',5.0,-152.9,43.8,358.3,'EPSG','9001',2.714,1.386,-2.788,'EPSG','9104',-6.743,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'BP-Alg D3',0);
INSERT INTO "usage" VALUES('EPSG','8736','helmert_transformation','EPSG','1815','EPSG','2598','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1816','Nord Sahara 1959 to WGS 84 (5)','Derived at astro station central to concession. Significant and varying differences (>100m) at 4 neighbouring astro stations. May be taken as a transformation to RGSH2020 - see CT 10340.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4307','EPSG','4326',100.0,-95.7,10.2,158.9,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'BPA-Alg InAm',0);
INSERT INTO "usage" VALUES('EPSG','8737','helmert_transformation','EPSG','1816','EPSG','2599','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1817','Nord Sahara 1959 to WGS 84 (6)','Derived at astro station Guerrara. May be taken as a transformation to RGSH2020 - see CT 10341.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4307','EPSG','4326',100.0,-165.914,-70.607,305.009,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'ARCO-Alg HBR',0);
INSERT INTO "usage" VALUES('EPSG','8738','helmert_transformation','EPSG','1817','EPSG','2600','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1818','Minna to WGS 84 (4)','Concatenated via WGS 72BE.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4263','EPSG','4326',12.0,-89.0,-112.0,125.9,'EPSG','9001',0.0,0.0,0.814,'EPSG','9104',-0.38,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'RSL-Nga',0);
INSERT INTO "usage" VALUES('EPSG','8739','helmert_transformation','EPSG','1818','EPSG','1717','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1819','Minna to WGS 84 (5)','Used by Shell in southern Nigeria and Total in OPL246.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4263','EPSG','4326',NULL,-111.92,-87.85,114.5,'EPSG','9001',1.875,0.202,0.219,'EPSG','9104',0.032,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'SPD-Nga S',1);
INSERT INTO "usage" VALUES('EPSG','8740','helmert_transformation','EPSG','1819','EPSG','2371','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1820','Minna to WGS 84 (6)','Derived by Nortech at station L40 Minna using NNPC 1989 GPS network tied to 4 ADOS stations. Used by Conoco in OPLs 219-220 to cm precision and ExxonMobil in OPL 209 to dm precision.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4263','EPSG','4326',12.0,-93.2,-93.31,121.156,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'CON89-Nga',0);
INSERT INTO "usage" VALUES('EPSG','8741','helmert_transformation','EPSG','1820','EPSG','3813','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1821','Minna to WGS 84 (7)','Derived by Elf Petroleum Nigeria in 1994 at 3 stations (M101 onshore, offshore platforms XSW06 and XSV39) and used in OMLs 99-102 and OPLs 222-223.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4263','EPSG','4326',6.0,-88.98,-83.23,113.55,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'ELF94-Nga',0);
INSERT INTO "usage" VALUES('EPSG','8742','helmert_transformation','EPSG','1821','EPSG','3814','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1822','Minna to WGS 84 (8)','Used by Shell SNEPCO for OPLs 209-213 and 316. Derived during 1990 Niger Delta control survey at 4 stations (XSU27, 30 31 and 35).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4263','EPSG','4326',10.0,-92.726,-90.304,115.735,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'SHL-Nga OPL W',0);
INSERT INTO "usage" VALUES('EPSG','8743','helmert_transformation','EPSG','1822','EPSG','3815','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1823','Minna to WGS 84 (9)','Used by Shell SNEPCO for OPLs 217-223. Derived during 1990 Niger Delta control survey at 4 stations (XSU38, 41, 44 and 45).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4263','EPSG','4326',8.0,-93.134,-86.647,114.196,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'SHL-Nga OPL S',0);
INSERT INTO "usage" VALUES('EPSG','8744','helmert_transformation','EPSG','1823','EPSG','3816','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1824','Minna to WGS 84 (10)','Used by Shell SNEPCO for Gongola basin.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4263','EPSG','4326',25.0,-93.0,-94.0,124.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'SHL-Nga Gongola',0);
INSERT INTO "usage" VALUES('EPSG','8745','helmert_transformation','EPSG','1824','EPSG','3824','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1825','Hong Kong 1980 to WGS 84 (1)','Derived via ITRF96 @ 1998.121. Published 2002-03-01. Parameter values from Hong Kong 1980 to Hong Kong Geodetic CS (1) (code 9913) assuming Hong Kong Geodetic CS and WGS 84 are equivalent within the accuracy of the CT.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4611','EPSG','4326',1.0,-162.619,-276.959,-161.764,'EPSG','9001',-0.067753,2.243648,1.158828,'EPSG','9104',-1.094246,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'LSD-HKG 2002',0);
INSERT INTO "usage" VALUES('EPSG','8746','helmert_transformation','EPSG','1825','EPSG','1118','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1826','JGD2000 to WGS 84 (1)','Excludes area affected by the 2011 Tohoku earthquake. For areas of northern Honshu affected by the earthquake see JGD2000 to WGS 84 (2) (code 9896).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4612','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Jpn',0);
INSERT INTO "usage" VALUES('EPSG','8747','helmert_transformation','EPSG','1826','EPSG','4163','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1828','Yoff to WGS 72 (1)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4310','EPSG','4322',25.0,-37.0,157.0,85.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-SEN',0);
INSERT INTO "usage" VALUES('EPSG','8749','helmert_transformation','EPSG','1828','EPSG','1207','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1829','HD72 to ETRS89 (1)','Derived at 5 stations. OGP recommends corrected Hungarian standard MSZ 7222 (tfm code 1449) be used in preference to this transformation. May be taken as approximate transformation HD72 to WGS 84 - see code 1830.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4237','EPSG','4258',0.5,56.0,-75.77,-15.31,'EPSG','9001',0.37,0.2,0.21,'EPSG','9104',1.01,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'FOMI-Hun',0);
INSERT INTO "usage" VALUES('EPSG','8750','helmert_transformation','EPSG','1829','EPSG','1119','EPSG','1031');
INSERT INTO "helmert_transformation" VALUES('EPSG','1830','HD72 to WGS 84 (1)','Parameter values taken from HD72 to ETRS89 (1) (code 1829) assuming that ETRS89 is equivalent to WGS 84 within the accuracy of the transformation. OGP recommends use of newer MSZ 7222 equivalent (tfm code 1448) in preference to this transformation.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4237','EPSG','4326',1.0,56.0,-75.77,-15.31,'EPSG','9001',0.37,0.2,0.21,'EPSG','9104',1.01,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Hun',0);
INSERT INTO "usage" VALUES('EPSG','8751','helmert_transformation','EPSG','1830','EPSG','1119','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1831','HD72 to WGS 84 (2)','Derived at fundamental point Szolohegy and tested at 99 stations throughout Hungary. Accuracy better than 1m in all three dimensions throughout Hungary. OGP recommends use of newer transformation (tfm code 1242) in preference to this transformation.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4237','EPSG','4326',1.0,57.01,-69.97,-9.29,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'ELTE-Hun',0);
INSERT INTO "usage" VALUES('EPSG','8752','helmert_transformation','EPSG','1831','EPSG','1119','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','1832','ID74 to WGS 84 (2)','Derived via coordinates of 2 Pulse8 stations. Use of ID74 to WGS 84 (3) (code 1833) is recommended.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4238','EPSG','4326',25.0,2.691,-14.757,4.724,'EPSG','9001',0.0,0.0,0.774,'EPSG','9104',-0.6,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'Rac91-Idn',0);
INSERT INTO "usage" VALUES('EPSG','8753','helmert_transformation','EPSG','1832','EPSG','4020','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1833','ID74 to WGS 84 (3)','Parameter values from ID74 to DGN95 (1) (code 15911) assuming that DGN95 is equivalent to WGS 84 within the accuracy of the transformation.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4238','EPSG','4326',3.0,-1.977,-13.06,-9.993,'EPSG','9001',-0.364,-0.254,-0.689,'EPSG','9104',-1.037,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'Bak-Idn',0);
INSERT INTO "usage" VALUES('EPSG','8754','helmert_transformation','EPSG','1833','EPSG','4020','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1834','Segara to WGS 84 (1)','Accuracy estimate not available.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4294','EPSG','4326',NULL,-403.0,684.0,41.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NIMA-Idn Kal',1);
INSERT INTO "usage" VALUES('EPSG','8755','helmert_transformation','EPSG','1834','EPSG','2354','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1835','Segara to WGS 84 (2)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4294','EPSG','4326',NULL,-387.06,636.53,46.29,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'Shl-Idn Kal E',1);
INSERT INTO "usage" VALUES('EPSG','8756','helmert_transformation','EPSG','1835','EPSG','1360','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1836','Segara to WGS 84 (3)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4294','EPSG','4326',NULL,-403.4,681.12,46.56,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'Shl-Idn Kal NE',1);
INSERT INTO "usage" VALUES('EPSG','8757','helmert_transformation','EPSG','1836','EPSG','2770','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1837','Makassar to WGS 84 (1)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4257','EPSG','4326',999.0,-587.8,519.75,145.76,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'Shl-Idn Sul SW',0);
INSERT INTO "usage" VALUES('EPSG','8758','helmert_transformation','EPSG','1837','EPSG','1316','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1838','Segara to WGS 84 (4)','Datum shift derived through ITRF93. In Pertamina Hulu Mahakam replaced by ITRF2014 to Segara (1) (CT code 10589).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4613','EPSG','4326',1.0,-404.78,685.68,45.47,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'TOT-Idn Mah',0);
INSERT INTO "usage" VALUES('EPSG','8759','helmert_transformation','EPSG','1838','EPSG','1328','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1839','Beduaram to WGS 72BE (1)','Derived by Elf in 1986.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4213','EPSG','4324',15.0,-101.0,-111.0,187.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'ELF-Ner SE',0);
INSERT INTO "usage" VALUES('EPSG','8760','helmert_transformation','EPSG','1839','EPSG','2771','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1840','QND95 to WGS 84 (1)','Transformation defines QND95. May be approximated to 1m throughout Qatar by geocentric translation transformation with dX=-127.78098m, dY=-283.37477m, dZ=+21.24081m.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4614','EPSG','4326',0.0,-119.4248,-303.65872,-11.00061,'EPSG','9001',1.164298,0.174458,1.096259,'EPSG','9104',3.657065,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'CGIS-Qat',0);
INSERT INTO "usage" VALUES('EPSG','8761','helmert_transformation','EPSG','1840','EPSG','1346','EPSG','1113');
INSERT INTO "helmert_transformation" VALUES('EPSG','1842','NAD83(CSRS) to WGS 84 (1)','For many purposes NAD83(CSRS) can be considered to be coincident with WGS 84.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4617','EPSG','4326',2.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Can',0);
INSERT INTO "usage" VALUES('EPSG','8763','helmert_transformation','EPSG','1842','EPSG','1061','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1852','Timbalai 1948 to WGS 84 (4)','Derived by Racal Survey for SSB at 24 coastal stations (including Timbalai fundamental point and 6 other primary triangulation stations) between in Sabah (Kudat southwards) and Sarawak (Sibu northwards).','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4298','EPSG','4326',5.0,-533.4,669.2,-52.5,'EPSG','9001',0.0,0.0,4.28,'EPSG','9104',9.4,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'SSB-Mys E',0);
INSERT INTO "usage" VALUES('EPSG','8773','helmert_transformation','EPSG','1852','EPSG','2780','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1853','ED50 to WGS 84 (39)','Derived at a single point in Galway docks.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4230','EPSG','4326',5.0,-82.31,-95.23,-114.96,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'Ent-Ire Corrib',0);
INSERT INTO "usage" VALUES('EPSG','8774','helmert_transformation','EPSG','1853','EPSG','2961','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1854','FD58 to WGS 84 (2)','Derived by Geoid for Elf in 1999. EGM96 geoid used.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4132','EPSG','4326',0.5,-239.1,-170.02,397.5,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'TFE-Irn Lavan',0);
INSERT INTO "usage" VALUES('EPSG','8775','helmert_transformation','EPSG','1854','EPSG','2782','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1855','FD58 to WGS 84 (3)','Derived by Geoid for Elf in 1999. EGM96 geoid used.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4132','EPSG','4326',0.5,-244.72,-162.773,400.75,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'TFE-Irn Kharg',0);
INSERT INTO "usage" VALUES('EPSG','8776','helmert_transformation','EPSG','1855','EPSG','2781','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1856','ED50(ED77) to WGS 84 (3)','Derived in Kangan district by Geoid for Total in 1998. Used for South Pars phases 2 and 3.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4154','EPSG','4326',0.5,-122.89,-159.08,-168.74,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'TFE-Irn SPars',0);
INSERT INTO "usage" VALUES('EPSG','8777','helmert_transformation','EPSG','1856','EPSG','2783','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1857','ED50(ED77) to WGS 84 (4)','Derived in 1999 on Lavan island by Geoid for Elf.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4154','EPSG','4326',0.5,-84.78,-107.55,-137.25,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'TFE-Irn Lavan',0);
INSERT INTO "usage" VALUES('EPSG','8778','helmert_transformation','EPSG','1857','EPSG','2782','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1858','ED50(ED77) to WGS 84 (5)','Derived by Geoid for Elf in 1999. EGM96 geoid used.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4154','EPSG','4326',0.5,-123.92,-155.515,-157.721,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'TFE-Irn Kharg',0);
INSERT INTO "usage" VALUES('EPSG','8779','helmert_transformation','EPSG','1858','EPSG','2781','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1859','ELD79 to WGS 84 (1)','Used by Repsol in Murzuq field, and PetroCanada and previous licence holders in NC177 and 72 (En Naga field). Reliability of connection to ELD79 questionned.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4159','EPSG','4326',20.0,-69.06,-90.71,-142.56,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'REP-Lby MZQ',0);
INSERT INTO "usage" VALUES('EPSG','8780','helmert_transformation','EPSG','1859','EPSG','2785','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1860','ELD79 to WGS 84 (2)','Derived December 2001 by NAGECO. 3-dimensional SD at 11 points is 0.5m. Connected to ITRF via Remsa 2000 data. Used by TotalFinaElf.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4159','EPSG','4326',0.5,-113.997,-97.076,-152.312,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'TFE-Lby MZQ',0);
INSERT INTO "usage" VALUES('EPSG','8781','helmert_transformation','EPSG','1860','EPSG','2785','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1861','ELD79 to WGS 84 (3)','Derived by GEOID in 1994 from Transit satellite data. Used by TotalFinaElf.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4159','EPSG','4326',2.0,-114.5,-96.1,-151.9,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'TFE-Lby MBK94',0);
INSERT INTO "usage" VALUES('EPSG','8782','helmert_transformation','EPSG','1861','EPSG','2786','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1862','ELD79 to WGS 84 (4)','Derived by Geoid in 2000 from ITRF connection by NAGECO for TotalFinaElf. For historic compatibility TFE use the 1994 tfm ELD79 to WGS 84 (3) (code 1861) rather than this transformation.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4159','EPSG','4326',0.5,-194.513,-63.978,-25.759,'EPSG','9001',-3.4027,3.756,-3.352,'EPSG','9104',-0.9175,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'TFE-Lby MBK00',0);
INSERT INTO "usage" VALUES('EPSG','8783','helmert_transformation','EPSG','1862','EPSG','2786','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1863','ELD79 to WGS 84 (5)','Derived for the Great Man-made River Authority (GMRA).','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4159','EPSG','4326',6.0,-389.691,64.502,210.209,'EPSG','9001',-0.086,-14.314,6.39,'EPSG','9104',0.9264,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'GMRA-Lby',0);
INSERT INTO "usage" VALUES('EPSG','8784','helmert_transformation','EPSG','1863','EPSG','2786','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1864','SAD69 to WGS 84 (1)','Derived at 84 stations. Accuracy 15m, 6m and 9m in X, Y and Z axes.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4618','EPSG','4326',19.0,-57.0,1.0,-41.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-mean',0);
INSERT INTO "usage" VALUES('EPSG','8785','helmert_transformation','EPSG','1864','EPSG','4016','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1865','SAD69 to WGS 84 (2)','Derived at 10 stations. Accuracy 5m in each axis. Note: SAD69 not adopted in Argentina: see Campo Inchauspe (CRS code 4221).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4618','EPSG','4326',9.0,-62.0,-1.0,-37.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Arg',0);
INSERT INTO "usage" VALUES('EPSG','8786','helmert_transformation','EPSG','1865','EPSG','3215','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1866','SAD69 to WGS 84 (3)','Derived at 4 stations. Accuracy 15m in each axis. Note: SAD69 not adopted in Bolivia: see PSAD56 (CRS code 4248).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4618','EPSG','4326',26.0,-61.0,2.0,-48.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Bol',0);
INSERT INTO "usage" VALUES('EPSG','8787','helmert_transformation','EPSG','1866','EPSG','1049','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1867','SAD69 to WGS 84 (4)','Derived at 22 stations. Accuracy 3m, 5m and 5m in X, Y and Z axes.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4618','EPSG','4326',8.0,-60.0,-2.0,-41.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Bra',0);
INSERT INTO "usage" VALUES('EPSG','8788','helmert_transformation','EPSG','1867','EPSG','3887','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1868','SAD69 to WGS 84 (5)','Derived at 9 stations. Accuracy 15m, 8m and 11m in X, Y and Z axes. Note: SAD69 not adopted in Chile north of 43°30''S. Replaced by SAD69 to WGS 84 (17) to (19) (codes 6974, 6975 and 6976).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4618','EPSG','4326',21.0,-75.0,-1.0,-44.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Chile',0);
INSERT INTO "usage" VALUES('EPSG','8789','helmert_transformation','EPSG','1868','EPSG','3227','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1869','SAD69 to WGS 84 (6)','Derived at 7 stations. Accuracy 6m, 6m and 5m in X, Y and Z axes. Note: SAD69 not adopted in Colombia: see Bogota 1975 (CRS code 4218).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4618','EPSG','4326',10.0,-44.0,6.0,-36.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Col',0);
INSERT INTO "usage" VALUES('EPSG','8790','helmert_transformation','EPSG','1869','EPSG','3229','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1870','SAD69 to WGS 84 (7)','Derived at 11 stations. Accuracy 3m in each axis. Note: SAD69 not adopted in Ecuador: see PSAD56 (CRS code 4248).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4618','EPSG','4326',6.0,-48.0,3.0,-44.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Ecu',0);
INSERT INTO "usage" VALUES('EPSG','8791','helmert_transformation','EPSG','1870','EPSG','3241','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1871','SAD69 to WGS 84 (8)','Derived at 1 station. Accuracy 25m in each axis. Note: SAD69 not adopted in Ecuador.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4618','EPSG','4326',44.0,-47.0,26.0,-42.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Ecu Gal',0);
INSERT INTO "usage" VALUES('EPSG','8792','helmert_transformation','EPSG','1871','EPSG','2356','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1872','SAD69 to WGS 84 (9)','Derived at 5 stations. Accuracy 9m, 5m and 5m in X, Y and Z axes. Note: SAD69 not adopted in Guyana.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4618','EPSG','4326',12.0,-53.0,3.0,-47.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Guy',0);
INSERT INTO "usage" VALUES('EPSG','8793','helmert_transformation','EPSG','1872','EPSG','3259','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1873','SAD69 to WGS 84 (10)','Derived at 4 stations. Accuracy 15m in each axis. Note: SAD69 not adopted in Paraguay.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4618','EPSG','4326',26.0,-61.0,2.0,-33.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Pgy',0);
INSERT INTO "usage" VALUES('EPSG','8794','helmert_transformation','EPSG','1873','EPSG','1188','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1874','SAD69 to WGS 84 (11)','Derived at 6 stations. Accuracy 5m in each axis. Note: SAD69 not adopted in Peru: see PSAD56 (CRS code 4248).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4618','EPSG','4326',9.0,-58.0,0.0,-44.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Peru',0);
INSERT INTO "usage" VALUES('EPSG','8795','helmert_transformation','EPSG','1874','EPSG','3292','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1875','SAD69 to WGS 84 (12)','Derived at 1 station. Accuracy 25m in each axis. Note: SAD69 not adopted in Trinidad and Tobago.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4618','EPSG','4326',44.0,-45.0,12.0,-33.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Tto',0);
INSERT INTO "usage" VALUES('EPSG','8796','helmert_transformation','EPSG','1875','EPSG','3143','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1876','SAD69 to WGS 84 (13)','Derived at 5 stations. Accuracy 3m, 6m and 3m in X, Y and Z axes. Note: SAD69 not adopted in Venezuela: see PSAD56 (CRS code 4248).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4618','EPSG','4326',8.0,-45.0,8.0,-33.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Ven',0);
INSERT INTO "usage" VALUES('EPSG','8797','helmert_transformation','EPSG','1876','EPSG','3327','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1877','SAD69 to WGS 84 (14)','Derived by Brazilian Institute of Geography and Statistics (IBGE) in 1989 at Chua origin point. In use by Shell throughout Brazil. For use by Petrobras and ANP, replaced by tfm code 5882 from 1994.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4618','EPSG','4326',5.0,-66.87,4.37,-38.52,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IBGE-Bra 5m 1989',0);
INSERT INTO "usage" VALUES('EPSG','8798','helmert_transformation','EPSG','1877','EPSG','1053','EPSG','1210');
INSERT INTO "helmert_transformation" VALUES('EPSG','1878','SWEREF99 to ETRS89 (1)','Can be taken as an approximate transformation SWEREF99 to WGS 84 - see code 1879.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4619','EPSG','4258',0.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NLS-Swe',0);
INSERT INTO "usage" VALUES('EPSG','8799','helmert_transformation','EPSG','1878','EPSG','1225','EPSG','1161');
INSERT INTO "helmert_transformation" VALUES('EPSG','1879','SWEREF99 to WGS 84 (1)','Approximation at the +/- 1m level assuming that ETRS89 is equivalent to WGS 84. SWEREF99 is a regional realization of ETRS89.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4619','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Swe',0);
INSERT INTO "usage" VALUES('EPSG','8800','helmert_transformation','EPSG','1879','EPSG','1225','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1880','Point 58 to WGS 84 (1)','Derived at one point in each of Burkina Faso and Niger. Accuracy 25m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4620','EPSG','4326',44.0,-106.0,-129.0,165.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NIMA-Bfa Ner',0);
INSERT INTO "usage" VALUES('EPSG','8801','helmert_transformation','EPSG','1880','EPSG','2791','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1885','Azores Oriental 1940 to WGS 84 (1)','Derived at 2 stations. Accuracy 25m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4184','EPSG','4326',44.0,-203.0,141.0,53.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Prt Az E',0);
INSERT INTO "usage" VALUES('EPSG','8806','helmert_transformation','EPSG','1885','EPSG','1345','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1886','Azores Central 1948 to WGS 84 (1)','Derived at 5 stations. Accuracy 3m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4183','EPSG','4326',6.0,-104.0,167.0,-38.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Prt Az C',0);
INSERT INTO "usage" VALUES('EPSG','8807','helmert_transformation','EPSG','1886','EPSG','1301','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1887','Azores Occidental 1939 to WGS 84 (1)','Derived at 3 stations. Accuracy 20m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4182','EPSG','4326',35.0,-425.0,-169.0,81.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Prt Az W',0);
INSERT INTO "usage" VALUES('EPSG','8808','helmert_transformation','EPSG','1887','EPSG','1344','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1888','Porto Santo to WGS 84 (1)','Derived at 2 stations. Accuracy 25m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4615','EPSG','4326',44.0,-499.0,-249.0,314.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Prt Mad',0);
INSERT INTO "usage" VALUES('EPSG','8809','helmert_transformation','EPSG','1888','EPSG','1314','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1889','Selvagen Grande to WGS 84 (1)','Derived at 1 station. Accuracy 25m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4616','EPSG','4326',NULL,-289.0,-124.0,60.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Prt Sel',1);
INSERT INTO "usage" VALUES('EPSG','8810','helmert_transformation','EPSG','1889','EPSG','2779','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1890','Australian Antarctic to WGS 84 (1)','For many purposes Australian Antarctic can be considered to be coincident with WGS 84.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4176','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Ata Aus',0);
INSERT INTO "usage" VALUES('EPSG','8811','helmert_transformation','EPSG','1890','EPSG','1278','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1892','Hito XVIII 1963 to WGS 84 (2)','Derived at 2 stations. As the source CRS was used for the border survey this transformation is probably also applicable to adjacent areas of Argentina.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4254','EPSG','4326',44.0,16.0,196.0,93.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NIMA-Chl',0);
INSERT INTO "usage" VALUES('EPSG','8813','helmert_transformation','EPSG','1892','EPSG','2805','EPSG','1157');
INSERT INTO "helmert_transformation" VALUES('EPSG','1893','Puerto Rico to WGS 84 (3)','Derived at 11 stations. Accuracy 3m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4139','EPSG','4326',6.0,11.0,72.0,-101.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NIMA-Pri',0);
INSERT INTO "usage" VALUES('EPSG','8814','helmert_transformation','EPSG','1893','EPSG','1335','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1894','Gandajika 1970 to WGS 84 (1)','Derived at 1 station. Accuracy 25m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4233','EPSG','4326',25.0,-133.0,-321.0,50.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NIMA-Mdv',1);
INSERT INTO "usage" VALUES('EPSG','8815','helmert_transformation','EPSG','1894','EPSG','1152','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1895','RT90 to SWEREF99 (1)','Derived at 165 points in 2001. Also given by EuroGeographics as RT90 to ETRS89 using the Position Vector transformation method. Replaces RT90 to ETRS89 (1) (code 1437). May be taken as approximate transformation RT90 to WGS 84 - see code 1896.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4124','EPSG','4619',0.1,414.1,41.3,603.1,'EPSG','9001',0.855,-2.141,7.023,'EPSG','9104',0.0,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NLS-Swe 2001',0);
INSERT INTO "usage" VALUES('EPSG','8816','helmert_transformation','EPSG','1895','EPSG','1225','EPSG','1031');
INSERT INTO "helmert_transformation" VALUES('EPSG','1896','RT90 to WGS 84 (2)','Parameter values from RT90 to SWEREF99 (1) (code 1895) assuming that SWEREF99 is equivalent to WGS 84 within the accuracy of the transformation. Replaces RT90 to WGS 84 (1) (code 1680).','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4124','EPSG','4326',1.0,414.1,41.3,603.1,'EPSG','9001',0.855,-2.141,7.023,'EPSG','9104',0.0,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Swe 2001',0);
INSERT INTO "usage" VALUES('EPSG','8817','helmert_transformation','EPSG','1896','EPSG','1225','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1897','Segara to WGS 84 (1)','Accuracy estimate not available.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4613','EPSG','4326',999.0,-403.0,684.0,41.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NIMA-Idn Kal',0);
INSERT INTO "usage" VALUES('EPSG','8818','helmert_transformation','EPSG','1897','EPSG','1360','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1898','Segara to WGS 84 (2)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4613','EPSG','4326',5.0,-387.06,636.53,46.29,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'Shl-Idn Kal E',0);
INSERT INTO "usage" VALUES('EPSG','8819','helmert_transformation','EPSG','1898','EPSG','1359','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1899','Segara to WGS 84 (3)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4613','EPSG','4326',10.0,-403.4,681.12,46.56,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'Shl-Idn Kal NE',0);
INSERT INTO "usage" VALUES('EPSG','8820','helmert_transformation','EPSG','1899','EPSG','2770','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1900','NAD83(HARN) to WGS 84 (2)','Approximation derived ignoring time-dependent parameters and assuming ITRF94(1996.0) and WGS 84, plus NAD83(CORS94) and NAD83(HARN), can be considered the same within the accuracy of the transformation. Replaced by NAD83(HARN) to WGS 84 (3) (code 1901).','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4152','EPSG','4326',1.0,-0.9738,1.9453,0.5486,'EPSG','9001',-1.3357e-07,-4.872e-08,-5.507e-08,'EPSG','9101',0.0,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NGS-Usa ITRF94',0);
INSERT INTO "usage" VALUES('EPSG','8821','helmert_transformation','EPSG','1900','EPSG','1323','EPSG','1159');
INSERT INTO "helmert_transformation" VALUES('EPSG','1901','NAD83(HARN) to WGS 84 (3)','Approximation derived from tfm code 6864 ignoring time-dependent parameters and assuming ITRF96(1997.0) and WGS 84, plus NAD83(CORS96) and NAD83(HARN), can be considered the same within the accuracy of the tfm. In USA only replaces tfm code 1900.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4152','EPSG','4326',1.0,-0.991,1.9072,0.5129,'EPSG','9001',-1.25033e-07,-4.6785e-08,-5.6529e-08,'EPSG','9101',0.0,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NGS-Usa ITRF96',0);
INSERT INTO "usage" VALUES('EPSG','8822','helmert_transformation','EPSG','1901','EPSG','1323','EPSG','1159');
INSERT INTO "helmert_transformation" VALUES('EPSG','1902','Manoca 1962 to WGS 72BE (1)','Derived at 6 stations using Transit in 1977.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4193','EPSG','4324',5.0,-56.7,-171.8,-40.6,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'GOC-Cmr',0);
INSERT INTO "usage" VALUES('EPSG','8823','helmert_transformation','EPSG','1902','EPSG','2555','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1903','Fort Marigot to WGS 84 (1)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4621','EPSG','4326',10.0,137.0,248.0,-430.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Glp',0);
INSERT INTO "usage" VALUES('EPSG','8824','helmert_transformation','EPSG','1903','EPSG','2828','EPSG','1050');
INSERT INTO "helmert_transformation" VALUES('EPSG','1904','Guadeloupe 1948 to WGS 84 (1)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4622','EPSG','4326',10.0,-467.0,-16.0,-300.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Glp 10m',0);
INSERT INTO "usage" VALUES('EPSG','8825','helmert_transformation','EPSG','1904','EPSG','2829','EPSG','1050');
INSERT INTO "helmert_transformation" VALUES('EPSG','1905','Guadeloupe 1948 to WGS 84 (2)','','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4622','EPSG','4326',0.1,-472.29,-5.63,-304.12,'EPSG','9001',0.4362,-0.8374,0.2563,'EPSG','9104',1.8984,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Glp 1m',0);
INSERT INTO "usage" VALUES('EPSG','8826','helmert_transformation','EPSG','1905','EPSG','2829','EPSG','1031');
INSERT INTO "helmert_transformation" VALUES('EPSG','1906','CSG67 to WGS 84 (1)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4623','EPSG','4326',10.0,-186.0,230.0,110.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Guf',0);
INSERT INTO "usage" VALUES('EPSG','8827','helmert_transformation','EPSG','1906','EPSG','3105','EPSG','1050');
INSERT INTO "helmert_transformation" VALUES('EPSG','1907','RGFG95 to WGS 84 (1)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4624','EPSG','4326',2.0,2.0,2.0,-2.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Guf',1);
INSERT INTO "usage" VALUES('EPSG','8828','helmert_transformation','EPSG','1907','EPSG','1097','EPSG','1042');
INSERT INTO "helmert_transformation" VALUES('EPSG','1908','CSG67 to RGFG95 (1)','Accuracy better than +/- 0.1 metre in the coastal area, better than +/- 1 metre in the interior.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4623','EPSG','4624',1.0,-193.066,236.993,105.447,'EPSG','9001',0.4814,-0.8074,0.1276,'EPSG','9104',1.5649,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Guf',0);
INSERT INTO "usage" VALUES('EPSG','8829','helmert_transformation','EPSG','1908','EPSG','3105','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','1909','Martinique 1938 to WGS 84 (1)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4625','EPSG','4326',10.0,186.0,482.0,151.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Mtq 10m',0);
INSERT INTO "usage" VALUES('EPSG','8830','helmert_transformation','EPSG','1909','EPSG','3276','EPSG','1050');
INSERT INTO "helmert_transformation" VALUES('EPSG','1910','Martinique 1938 to WGS 84 (2)','','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4625','EPSG','4326',0.1,126.93,547.94,130.41,'EPSG','9001',-2.7867,5.1612,-0.8584,'EPSG','9104',13.8227,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Mtq 1m',0);
INSERT INTO "usage" VALUES('EPSG','8831','helmert_transformation','EPSG','1910','EPSG','3276','EPSG','1031');
INSERT INTO "helmert_transformation" VALUES('EPSG','1911','Reunion 1947 to WGS 84 (1)','Derived at 1 station.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4626','EPSG','4326',30.0,94.0,-948.0,-1292.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Reu 30m',1);
INSERT INTO "usage" VALUES('EPSG','8832','helmert_transformation','EPSG','1911','EPSG','1196','EPSG','1077');
INSERT INTO "helmert_transformation" VALUES('EPSG','1912','RGR92 to WGS 84 (1)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4627','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Reu',0);
INSERT INTO "usage" VALUES('EPSG','8833','helmert_transformation','EPSG','1912','EPSG','3902','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1913','Tahaa 54 to WGS 84 (1)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4629','EPSG','4326',10.0,65.0,342.0,77.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Pyf Tahaa',0);
INSERT INTO "usage" VALUES('EPSG','8834','helmert_transformation','EPSG','1913','EPSG','2812','EPSG','1050');
INSERT INTO "helmert_transformation" VALUES('EPSG','1914','IGN72 Nuku Hiva to WGS 84 (1)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4630','EPSG','4326',10.0,84.0,274.0,65.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Pyf',0);
INSERT INTO "usage" VALUES('EPSG','8835','helmert_transformation','EPSG','1914','EPSG','3129','EPSG','1050');
INSERT INTO "helmert_transformation" VALUES('EPSG','1915','K0 1949 to WGS 84 (1)','Also published in US NIMA/NGA TR8350.2 which gives accuracy of +/-25m in each axis and states that derived at one station.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4631','EPSG','4326',10.0,145.0,-187.0,103.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Atf Kerg',1);
INSERT INTO "usage" VALUES('EPSG','8836','helmert_transformation','EPSG','1915','EPSG','2816','EPSG','1050');
INSERT INTO "helmert_transformation" VALUES('EPSG','1916','Combani 1950 to WGS 84 (1)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4632','EPSG','4326',10.0,-382.0,-59.0,-262.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Myt',0);
INSERT INTO "usage" VALUES('EPSG','8837','helmert_transformation','EPSG','1916','EPSG','3340','EPSG','1050');
INSERT INTO "helmert_transformation" VALUES('EPSG','1917','IGN56 Lifou to WGS 84 (1)','Withdrawn by information source and replaced by improved information from local authority - see tfm code 15902.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4633','EPSG','4326',10.0,336.0,223.0,-231.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Ncl',0);
INSERT INTO "usage" VALUES('EPSG','8838','helmert_transformation','EPSG','1917','EPSG','2814','EPSG','1050');
INSERT INTO "helmert_transformation" VALUES('EPSG','1918','IGN72 Grand Terre to WGS 84 (1)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4634','EPSG','4326',NULL,-13.0,-348.0,292.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Ncl',1);
INSERT INTO "usage" VALUES('EPSG','8839','helmert_transformation','EPSG','1918','EPSG','1174','EPSG','1050');
INSERT INTO "helmert_transformation" VALUES('EPSG','1919','ST87 Ouvea to WGS 84 (1)','','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4635','EPSG','4326',1.0,-122.383,-188.696,103.344,'EPSG','9001',3.5107,-4.9668,-5.7047,'EPSG','9104',4.4798,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'BGN-Ncl',1);
INSERT INTO "usage" VALUES('EPSG','8840','helmert_transformation','EPSG','1919','EPSG','2813','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','1920','RGNC 1991 to WGS 84 (1)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4645','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Ncl',1);
INSERT INTO "usage" VALUES('EPSG','8841','helmert_transformation','EPSG','1920','EPSG','1174','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','1921','Petrels 1972 to WGS 84 (1)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4636','EPSG','4326',10.0,365.0,194.0,166.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Ata Adel',0);
INSERT INTO "usage" VALUES('EPSG','8842','helmert_transformation','EPSG','1921','EPSG','2817','EPSG','1050');
INSERT INTO "helmert_transformation" VALUES('EPSG','1922','Perroud 1950 to WGS 84 (1)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4637','EPSG','4326',10.0,325.0,154.0,172.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Ata Adel',0);
INSERT INTO "usage" VALUES('EPSG','8843','helmert_transformation','EPSG','1922','EPSG','2818','EPSG','1050');
INSERT INTO "helmert_transformation" VALUES('EPSG','1923','Saint Pierre et Miquelon 1950 to WGS 84 (1)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4638','EPSG','4326',10.0,30.0,430.0,368.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Spm',0);
INSERT INTO "usage" VALUES('EPSG','8844','helmert_transformation','EPSG','1923','EPSG','3299','EPSG','1050');
INSERT INTO "helmert_transformation" VALUES('EPSG','1924','Tahiti 52 to WGS 84 (1)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4628','EPSG','4326',10.0,162.0,117.0,154.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Pyf',0);
INSERT INTO "usage" VALUES('EPSG','8845','helmert_transformation','EPSG','1924','EPSG','2811','EPSG','1050');
INSERT INTO "helmert_transformation" VALUES('EPSG','1925','MOP78 to WGS 84 (1)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4639','EPSG','4326',10.0,252.0,-132.0,-125.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Wlf Wallis',1);
INSERT INTO "usage" VALUES('EPSG','8846','helmert_transformation','EPSG','1925','EPSG','2815','EPSG','1050');
INSERT INTO "helmert_transformation" VALUES('EPSG','1926','Reunion 1947 to RGR92 (1)','Note: Because of the large rotation about the Y-axis this transformation is not reversible. Errors of up to 0.5m may occur. For the reverse transformation use RGR92 to Reunion 1947 [alias Piton des Neiges] (1) (code 1964).','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4626','EPSG','4627',0.1,789.524,-626.486,-89.904,'EPSG','9001',0.6006,76.7946,-10.5788,'EPSG','9104',-32.3241,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Reu 0.1m',0);
INSERT INTO "usage" VALUES('EPSG','8847','helmert_transformation','EPSG','1926','EPSG','3337','EPSG','1031');
INSERT INTO "helmert_transformation" VALUES('EPSG','1927','IGN56 Lifou to WGS 84 (2)','Withdrawn by information source and replaced by improved information - see CT code 15902.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4633','EPSG','4326',1.0,137.092,131.66,91.475,'EPSG','9001',-1.9436,-11.5993,-4.3321,'EPSG','9104',-7.4824,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'BGN-Ncl v2',0);
INSERT INTO "usage" VALUES('EPSG','8848','helmert_transformation','EPSG','1927','EPSG','2814','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','1928','IGN53 Mare to WGS 84 (1)','Withdrawn by information source and replaced by improved information - see CT code 15901.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4641','EPSG','4326',1.0,-408.809,366.856,-412.987,'EPSG','9001',1.8842,-0.5308,2.1655,'EPSG','9104',-121.0993,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'BGN-Ncl v1',0);
INSERT INTO "usage" VALUES('EPSG','8849','helmert_transformation','EPSG','1928','EPSG','2819','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','1929','IGN72 Grand Terre to WGS 84 (2)','','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4634','EPSG','4326',NULL,97.295,-263.247,310.882,'EPSG','9001',-1.5999,0.8386,3.1409,'EPSG','9104',13.3259,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'BGN-Ncl',1);
INSERT INTO "usage" VALUES('EPSG','8850','helmert_transformation','EPSG','1929','EPSG','2822','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','1930','ST84 Ile des Pins to WGS 84 (1)','','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4642','EPSG','4326',1.0,244.416,85.339,168.114,'EPSG','9001',-8.9353,7.7523,12.5953,'EPSG','9104',14.268,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'BGN-Ncl',1);
INSERT INTO "usage" VALUES('EPSG','8851','helmert_transformation','EPSG','1930','EPSG','2820','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','1931','ST71 Belep to WGS 84 (1)','','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4643','EPSG','4326',1.0,-480.26,-438.32,-643.429,'EPSG','9001',16.3119,20.1721,-4.0349,'EPSG','9104',-111.7002,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'BGN-Ncl',0);
INSERT INTO "usage" VALUES('EPSG','8852','helmert_transformation','EPSG','1931','EPSG','2821','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','1932','NEA74 Noumea to WGS 84 (1)','','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4644','EPSG','4326',1.0,-166.207,-154.777,254.831,'EPSG','9001',-37.5444,7.7011,-10.2025,'EPSG','9104',-30.8598,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'BGN-Ncl',1);
INSERT INTO "usage" VALUES('EPSG','8853','helmert_transformation','EPSG','1932','EPSG','2823','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','1933','RGR92 to Piton des Nieges (1)','Note: Because of the large rotation about the Y-axis this transformation is not reversible. For the reverse transformation see Piton des Nieges to RGR92 (1) (code 1926).','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4627','EPSG','4626',NULL,-789.99,627.333,89.685,'EPSG','9001',-0.6072,-76.8019,10.568,'EPSG','9104',32.2083,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Reu 0.1m',1);
INSERT INTO "usage" VALUES('EPSG','8854','helmert_transformation','EPSG','1933','EPSG','1196','EPSG','1031');
INSERT INTO "helmert_transformation" VALUES('EPSG','1934','RRAF 1991 to WGS 84 (1)','RRAF 1991 was defined to be WGS84 at a single point in Martinique during the 1988 Tango mission.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4640','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-FrAnt',1);
INSERT INTO "usage" VALUES('EPSG','8855','helmert_transformation','EPSG','1934','EPSG','2824','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','1935','ITRF97 to ITRF2000 (1)','At epoch 1997.0. Rates dX=0.0000 m/yr, dy=0.0006 m/yr, dZ=0.0014 m/yr, rX=rY=0.0"/yr, rZ=-0.00002"/yr, dS=-0.00001 ppm/yr.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4918','EPSG','4919',0.0,-0.0067,-0.0061,0.0185,'EPSG','9001',0.0,0.0,0.0,'EPSG','9104',-0.00155,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IERS',1);
INSERT INTO "usage" VALUES('EPSG','8856','helmert_transformation','EPSG','1935','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','1936','ITRF96 to ITRF2000 (1)','At epoch 1997.0. Rates dX=0.0000 m/yr, dy=0.0006 m/yr, dZ=0.0014 m/yr, rX=rY=0.0"/yr, rZ=-0.00002"/yr, dS=-0.00001 ppm/yr.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4917','EPSG','4919',0.0,-0.0067,-0.0061,0.0185,'EPSG','9001',0.0,0.0,0.0,'EPSG','9104',-0.00155,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IERS',1);
INSERT INTO "usage" VALUES('EPSG','8857','helmert_transformation','EPSG','1936','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','1937','ITRF94 to ITRF2000 (1)','At epoch 1997.0. Rates dX=0.0000 m/yr, dy=0.0006 m/yr, dZ=0.0014 m/yr, rX=rY=0.0"/yr, rZ=-0.00002"/yr, dS=-0.00001 ppm/yr.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4916','EPSG','4919',0.0,-0.0067,-0.0061,0.0185,'EPSG','9001',0.0,0.0,0.0,'EPSG','9104',-0.00155,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IERS',1);
INSERT INTO "usage" VALUES('EPSG','8858','helmert_transformation','EPSG','1937','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','1938','ITRF93 to ITRF2000 (1)','At epoch 1988.0. Rates dX=0.0029 m/yr, dy=0.0002 m/yr, dZ=0.0006 m/yr, rX=0.00011"/yr, rY=0.00019"/yr, rZ=-0.00007"/yr, dS=-0.00001 ppm/yr.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4915','EPSG','4919',0.0,-0.0127,-0.0065,0.0209,'EPSG','9001',0.00039,-0.0008,0.00114,'EPSG','9104',-0.00195,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IERS',1);
INSERT INTO "usage" VALUES('EPSG','8859','helmert_transformation','EPSG','1938','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','1939','ITRF92 to ITRF2000 (1)','At epoch 1988.0. Rates dX=0.0000 m/yr, dy=0.0006 m/yr, dZ=0.0014 m/yr, rX=rY=0.0"/yr, rZ=-0.00002"/yr, dS=-0.00001 ppm/yr.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4914','EPSG','4919',0.0,-0.0147,-0.0135,0.0139,'EPSG','9001',0.0,0.0,0.00018,'EPSG','9104',-0.00075,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IERS',1);
INSERT INTO "usage" VALUES('EPSG','8860','helmert_transformation','EPSG','1939','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','1940','ITRF91 to ITRF2000 (1)','At epoch 1988.0. Rates dX=0.0000 m/yr, dy=0.0006 m/yr, dZ=0.0014 m/yr, rX=rY=0.0"/yr, rZ=-0.00002"/yr, dS=-0.00001 ppm/yr.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4913','EPSG','4919',0.0,-0.0267,-0.0275,0.0199,'EPSG','9001',0.0,0.0,0.00018,'EPSG','9104',-0.00215,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IERS',1);
INSERT INTO "usage" VALUES('EPSG','8861','helmert_transformation','EPSG','1940','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','1941','ITRF90 to ITRF2000 (1)','At epoch 1988.0. Rates dX=0.0000 m/yr, dy=0.0006 m/yr, dZ=0.0014 m/yr, rX=rY=0.0"/yr, rZ=-0.00002"/yr, dS=-0.00001 ppm/yr.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4912','EPSG','4919',0.0,-0.0247,-0.0235,0.0359,'EPSG','9001',0.0,0.0,0.00018,'EPSG','9104',-0.00245,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IERS',1);
INSERT INTO "usage" VALUES('EPSG','8862','helmert_transformation','EPSG','1941','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','1942','ITRF89 to ITRF2000 (1)','At epoch 1988.0. Rates dX=0.0000 m/yr, dy=0.0006 m/yr, dZ=0.0014 m/yr, rX=rY=0.0"/yr, rZ=-0.00002"/yr, dS=-0.00001 ppm/yr.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4911','EPSG','4919',0.0,-0.0297,-0.0475,0.0739,'EPSG','9001',0.0,0.0,0.00018,'EPSG','9104',0.00585,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IERS',1);
INSERT INTO "usage" VALUES('EPSG','8863','helmert_transformation','EPSG','1942','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','1943','ITRF88 to ITRF2000 (1)','At epoch 1988.0. Rates dX=0.0000 m/yr, dy=0.0006 m/yr, dZ=0.0014 m/yr, rX=rY=0.0"/yr, rZ=-0.00002"/yr, dS=-0.00001 ppm/yr.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4910','EPSG','4919',0.0,-0.0247,-0.0115,0.0979,'EPSG','9001',-0.0001,0.0,0.00018,'EPSG','9104',-0.00895,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IERS',1);
INSERT INTO "usage" VALUES('EPSG','8864','helmert_transformation','EPSG','1943','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','1944','Lisbon to WGS 84 (2)','Parameter values from Lisbon to ETRS89 (2) (code 1790). Assumes ETRS89 and WGS 84 can be considered the same to within the accuracy of the transformation.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4207','EPSG','4326',NULL,-282.1,-72.2,120.0,'EPSG','9001',-1.592,0.145,-0.89,'EPSG','9104',-4.46,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Prt 2001',1);
INSERT INTO "usage" VALUES('EPSG','8865','helmert_transformation','EPSG','1944','EPSG','1294','EPSG','1042');
INSERT INTO "helmert_transformation" VALUES('EPSG','1945','Datum 73 to WGS 84 (2)','Parameter values from Datum 73 to ETRS89 (2) (code 1792). Assumes ETRS89 and WGS 84 can be considered the same to within the accuracy of the transformation.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4274','EPSG','4326',NULL,-231.0,102.6,29.8,'EPSG','9001',0.615,-0.198,0.881,'EPSG','9104',1.79,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Prt 2001',1);
INSERT INTO "usage" VALUES('EPSG','8866','helmert_transformation','EPSG','1945','EPSG','1294','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','1946','NAD83(CSRS) to WGS 84 (2)','Approximation derived from tfm code 6864 ignoring time-dependent parameters and assuming ITRF96(1997.0) and WGS 84 can be considered the same within the accuracy of the transformation.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4617','EPSG','4326',1.0,-0.991,1.9072,0.5129,'EPSG','9001',-1.25033e-07,-4.6785e-08,-5.6529e-08,'EPSG','9101',0.0,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NGS-Usa ITRF96',0);
INSERT INTO "usage" VALUES('EPSG','8867','helmert_transformation','EPSG','1946','EPSG','1061','EPSG','1159');
INSERT INTO "helmert_transformation" VALUES('EPSG','1950','NAD83 to NAD83(CSRS) (4)','Used as part of NAD27 to/from WGS 84 transformation for offshore oil operations - see code 8647.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4269','EPSG','4617',2.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Can E Off',0);
INSERT INTO "usage" VALUES('EPSG','8871','helmert_transformation','EPSG','1950','EPSG','2831','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1951','Hjorsey 1955 to WGS 84 (1)','Derived at 6 stations. Accuracy 3m, 3m and 5m in X, Y and Z axes. Replaced by Hjorsey 1955 to WGS 84 (2) (code 6909).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4658','EPSG','4326',7.0,-73.0,46.0,-86.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Isl',0);
INSERT INTO "usage" VALUES('EPSG','8872','helmert_transformation','EPSG','1951','EPSG','3262','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1952','ISN93 to WGS 84 (1)','For many purposes ISN93 can be considered to be coincident with WGS 84.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4659','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Isl',0);
INSERT INTO "usage" VALUES('EPSG','8873','helmert_transformation','EPSG','1952','EPSG','1120','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1953','TM75 to ETRS89 (2)','TM75 is based on the geodetic datum of 1965 which should not be confused with the mapping adjustment of 1965 (TM65). May be taken as approximate transformations TM75 to WGS 84, TM65 to WGS 84 and OSNI 1952 to WGS 84 - see codes 1954, 1641 and 1955.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4300','EPSG','4258',1.0,482.5,-130.6,564.6,'EPSG','9001',-1.042,-0.214,-0.631,'EPSG','9104',8.15,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OSI-Ire PV approximation',0);
INSERT INTO "usage" VALUES('EPSG','8874','helmert_transformation','EPSG','1953','EPSG','1305','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','1954','TM75 to WGS 84 (2)','Parameter values taken from TM65 to ETRS89 (2) (code 1953). Assumes ETRS89 and WGS 84 can be considered the same to within the accuracy of the transformation.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4300','EPSG','4326',1.0,482.5,-130.6,564.6,'EPSG','9001',-1.042,-0.214,-0.631,'EPSG','9104',8.15,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Ire PV approx',0);
INSERT INTO "usage" VALUES('EPSG','8875','helmert_transformation','EPSG','1954','EPSG','1305','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1955','OSNI 1952 to WGS 84 (1)','Parameter values from TM75 to ETRS89 (2) (code 1953). Assumes each pair of (i) OSNI 1952 and TM75, and (ii) ETRS89 and WGS 84, can be considered the same to within the accuracy of the transformation.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4188','EPSG','4326',1.0,482.5,-130.6,564.6,'EPSG','9001',-1.042,-0.214,-0.631,'EPSG','9104',8.15,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Ire',0);
INSERT INTO "usage" VALUES('EPSG','8876','helmert_transformation','EPSG','1955','EPSG','2530','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1956','TM75 to WGS 84 (3)','Derived at 7 stations. Accuracy 3m in each axis. TM75 is based on the geodetic datum of 1965 which should not be confused with the mapping adjustment of 1965 (TM65).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4300','EPSG','4326',6.0,506.0,-122.0,611.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Ire',0);
INSERT INTO "usage" VALUES('EPSG','8877','helmert_transformation','EPSG','1956','EPSG','1305','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1957','Helle 1954 to WGS 84 (1)','Derived at 3 stations. Residuals under 1m.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4660','EPSG','4326',1.0,982.6087,552.753,-540.873,'EPSG','9001',32.39344,-153.25684,-96.2266,'EPSG','9109',16.805,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'SKV-SJM Jan Mayen',0);
INSERT INTO "usage" VALUES('EPSG','8878','helmert_transformation','EPSG','1957','EPSG','2869','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','1958','LKS-92 to WGS 84 (1)','LKS-92 is a national realization of ETRS89 and coincident to WGS84 within 1 metre. This transformation has an accuracy equal to the coincidence figure.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4661','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'Vzd-Lva',0);
INSERT INTO "usage" VALUES('EPSG','8879','helmert_transformation','EPSG','1958','EPSG','1139','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1959','St. Vincent 1945 to WGS 84 (1)','Derived at 4 points.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4607','EPSG','4326',1.0,195.671,332.517,274.607,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'LSU-Vct',0);
INSERT INTO "usage" VALUES('EPSG','8880','helmert_transformation','EPSG','1959','EPSG','3300','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','1960','ED87 to WGS 84 (2)','','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4231','EPSG','4326',1.0,-83.11,-97.38,-117.22,'EPSG','9001',0.005693,-0.04469,0.04428,'EPSG','9104',1.218,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IERS-Eur',1);
INSERT INTO "usage" VALUES('EPSG','8881','helmert_transformation','EPSG','1960','EPSG','1297','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','1961','ED50 to WGS 84 (32)','Parameter values taken from ED87 to WGS 84 (2) (tfm code 1960) assuming that ED87 is identical to ED50. Errors caused by this assumption can reach 3m.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4230','EPSG','4326',NULL,-83.11,-97.38,-117.22,'EPSG','9001',0.005693,-0.04469,0.4428,'EPSG','9104',1.218,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NAM-Nld-Nsea',1);
INSERT INTO "usage" VALUES('EPSG','8882','helmert_transformation','EPSG','1961','EPSG','1630','EPSG','1217');
INSERT INTO "helmert_transformation" VALUES('EPSG','1962','IGN72 Grande Terre to WGS 84 (1)','Withdrawn by information source and replaced by improved information from local authority - see tfm code 15903.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4662','EPSG','4326',10.0,-13.0,-348.0,292.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Ncl',0);
INSERT INTO "usage" VALUES('EPSG','8883','helmert_transformation','EPSG','1962','EPSG','2822','EPSG','1050');
INSERT INTO "helmert_transformation" VALUES('EPSG','1963','IGN72 Grande Terre to WGS 84 (2)','Withdrawn by information source and replaced by improved information - see CT code 15903.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4662','EPSG','4326',1.0,97.295,-263.247,310.882,'EPSG','9001',-1.5999,0.8386,3.1409,'EPSG','9104',13.3259,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'BGN-Ncl v2',0);
INSERT INTO "usage" VALUES('EPSG','8884','helmert_transformation','EPSG','1963','EPSG','2822','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','1964','RGR92 to Reunion 1947 (1)','Note: Because of the large rotation about the Y-axis this transformation is not reversible. Errors of up to 0.5m may occur. For the reverse transformation use Reunion 1947 [alias Piton des Neiges] to RGR92 (1) (code 1926).','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4627','EPSG','4626',0.1,-789.99,627.333,89.685,'EPSG','9001',-0.6072,-76.8019,10.568,'EPSG','9104',32.2083,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Reu 0.1m',0);
INSERT INTO "usage" VALUES('EPSG','8885','helmert_transformation','EPSG','1964','EPSG','3337','EPSG','1031');
INSERT INTO "helmert_transformation" VALUES('EPSG','1965','Selvagem Grande to WGS 84 (1)','Derived at 1 station. Accuracy 25m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4616','EPSG','4326',44.0,-289.0,-124.0,60.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Prt Sel',0);
INSERT INTO "usage" VALUES('EPSG','8886','helmert_transformation','EPSG','1965','EPSG','2779','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','1966','Porto Santo 1995 to WGS 84 (2)','Derived at Forte de Sao Tiago.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4663','EPSG','4326',5.0,-502.862,-247.438,312.724,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'CGC-Prt Mad 5m',0);
INSERT INTO "usage" VALUES('EPSG','8887','helmert_transformation','EPSG','1966','EPSG','2870','EPSG','1157');
INSERT INTO "helmert_transformation" VALUES('EPSG','1967','Porto Santo 1995 to WGS 84 (3)','','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4663','EPSG','4326',1.0,-210.502,-66.902,-48.476,'EPSG','9001',-2.094,15.067,5.817,'EPSG','9104',0.485,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'CGC-Prt Mad 1m',0);
INSERT INTO "usage" VALUES('EPSG','8888','helmert_transformation','EPSG','1967','EPSG','2870','EPSG','1158');
INSERT INTO "helmert_transformation" VALUES('EPSG','1968','Azores Oriental 1995 to WGS 84 (2)','Calculated in 2001.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4664','EPSG','4326',5.0,-204.633,140.216,55.199,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'CGC-Prt Az Mig 5m',0);
INSERT INTO "usage" VALUES('EPSG','8889','helmert_transformation','EPSG','1968','EPSG','2871','EPSG','1157');
INSERT INTO "helmert_transformation" VALUES('EPSG','1969','Azores Oriental 1995 to WGS 84 (3)','Calculated in 2001.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4664','EPSG','4326',1.0,-211.939,137.626,58.3,'EPSG','9001',0.089,-0.251,-0.079,'EPSG','9104',0.384,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'CGC-Prt Az Mig 1m',0);
INSERT INTO "usage" VALUES('EPSG','8890','helmert_transformation','EPSG','1969','EPSG','2871','EPSG','1158');
INSERT INTO "helmert_transformation" VALUES('EPSG','1970','Azores Oriental 1995 to WGS 84 (4)','Mean for all islands in group.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4664','EPSG','4326',5.0,-204.619,140.176,55.226,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'CGC-Prt Az E 5m',0);
INSERT INTO "usage" VALUES('EPSG','8891','helmert_transformation','EPSG','1970','EPSG','1345','EPSG','1157');
INSERT INTO "helmert_transformation" VALUES('EPSG','1971','Azores Oriental 1995 to WGS 84 (5)','Mean for all islands in group.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4664','EPSG','4326',1.0,-208.719,129.685,52.092,'EPSG','9001',0.195,0.014,-0.327,'EPSG','9104',0.198,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'CGC-Prt Az E 1m',0);
INSERT INTO "usage" VALUES('EPSG','8892','helmert_transformation','EPSG','1971','EPSG','1345','EPSG','1158');
INSERT INTO "helmert_transformation" VALUES('EPSG','1972','Azores Central 1995 to WGS 84 (2)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4665','EPSG','4326',5.0,-106.301,166.27,-37.916,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'CGC-Prt Az Ter 5m',0);
INSERT INTO "usage" VALUES('EPSG','8893','helmert_transformation','EPSG','1972','EPSG','2872','EPSG','1157');
INSERT INTO "helmert_transformation" VALUES('EPSG','1973','Azores Central 1995 to WGS 84 (3)','','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4665','EPSG','4326',1.0,-105.854,165.589,-38.312,'EPSG','9001',0.003,0.026,-0.024,'EPSG','9104',-0.048,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'CGC-Prt Az Ter 1m',0);
INSERT INTO "usage" VALUES('EPSG','8894','helmert_transformation','EPSG','1973','EPSG','2872','EPSG','1158');
INSERT INTO "helmert_transformation" VALUES('EPSG','1974','Azores Central 1995 to WGS 84 (4)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4665','EPSG','4326',5.0,-106.248,166.244,-37.845,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'CGC-Prt Az Fai 5m',0);
INSERT INTO "usage" VALUES('EPSG','8895','helmert_transformation','EPSG','1974','EPSG','2873','EPSG','1157');
INSERT INTO "helmert_transformation" VALUES('EPSG','1975','Azores Central 1995 to WGS 84 (5)','','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4665','EPSG','4326',1.0,-104.0,162.924,-38.882,'EPSG','9001',0.075,0.071,-0.051,'EPSG','9104',-0.338,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'CGC-Prt Az Fai 1m',0);
INSERT INTO "usage" VALUES('EPSG','8896','helmert_transformation','EPSG','1975','EPSG','2873','EPSG','1158');
INSERT INTO "helmert_transformation" VALUES('EPSG','1976','Azores Central 1995 to WGS 84 (6)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4665','EPSG','4326',5.0,-106.044,166.655,-37.876,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'CGC-Prt Az Pic 5m',0);
INSERT INTO "usage" VALUES('EPSG','8897','helmert_transformation','EPSG','1976','EPSG','2874','EPSG','1157');
INSERT INTO "helmert_transformation" VALUES('EPSG','1977','Azores Central 1995 to WGS 84 (7)','','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4665','EPSG','4326',1.0,-95.323,166.098,-69.942,'EPSG','9001',0.215,1.031,-0.047,'EPSG','9104',1.922,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'CGC-Prt Az Pic 1m',0);
INSERT INTO "usage" VALUES('EPSG','8898','helmert_transformation','EPSG','1977','EPSG','2874','EPSG','1158');
INSERT INTO "helmert_transformation" VALUES('EPSG','1978','Azores Central 1995 to WGS 84 (8)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4665','EPSG','4326',5.0,-106.253,166.239,-37.854,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'CGC-Prt Az SJ 5m',0);
INSERT INTO "usage" VALUES('EPSG','8899','helmert_transformation','EPSG','1978','EPSG','2875','EPSG','1157');
INSERT INTO "helmert_transformation" VALUES('EPSG','1979','Azores Central 1995 to WGS 84 (9)','','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4665','EPSG','4326',1.0,-100.306,161.246,-48.761,'EPSG','9001',0.192,0.385,-0.076,'EPSG','9104',0.131,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'CGC-Prt Az SJ 1m',0);
INSERT INTO "usage" VALUES('EPSG','8900','helmert_transformation','EPSG','1979','EPSG','2875','EPSG','1158');
INSERT INTO "helmert_transformation" VALUES('EPSG','1980','Azores Central 1995 to WGS 84 (10)','Mean for all islands in group.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4665','EPSG','4326',5.0,-106.226,166.366,-37.893,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'CGC-Prt Az C 5m',0);
INSERT INTO "usage" VALUES('EPSG','8901','helmert_transformation','EPSG','1980','EPSG','1301','EPSG','1157');
INSERT INTO "helmert_transformation" VALUES('EPSG','1981','Azores Central 1995 to WGS 84 (11)','Mean for all islands in group.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4665','EPSG','4326',1.0,-103.088,162.481,-28.276,'EPSG','9001',-0.167,-0.082,-0.168,'EPSG','9104',-1.504,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'CGC-Prt Az C 1m',0);
INSERT INTO "usage" VALUES('EPSG','8902','helmert_transformation','EPSG','1981','EPSG','1301','EPSG','1158');
INSERT INTO "helmert_transformation" VALUES('EPSG','1982','Azores Occidental 1939 to WGS 84 (2)','Derived at 2 stations in 1999.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4182','EPSG','4326',5.0,-422.651,-172.995,84.02,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'CGC-Prt Az W',0);
INSERT INTO "usage" VALUES('EPSG','8903','helmert_transformation','EPSG','1982','EPSG','1344','EPSG','1157');
INSERT INTO "helmert_transformation" VALUES('EPSG','1983','Datum 73 to WGS 84 (3)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4274','EPSG','4326',5.0,-223.237,110.193,36.649,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'CGC-Prt 5m',0);
INSERT INTO "usage" VALUES('EPSG','8904','helmert_transformation','EPSG','1983','EPSG','1294','EPSG','1157');
INSERT INTO "helmert_transformation" VALUES('EPSG','1984','Lisbon to WGS 84 (3)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4207','EPSG','4326',5.0,-304.046,-60.576,103.64,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'CGC-Prt 5m',0);
INSERT INTO "usage" VALUES('EPSG','8905','helmert_transformation','EPSG','1984','EPSG','1294','EPSG','1157');
INSERT INTO "helmert_transformation" VALUES('EPSG','1985','ED50 to WGS 84 (33)','May be taken as a transformation from ED50 to ETRS89 - see tfm code 5040.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4230','EPSG','4326',5.0,-87.987,-108.639,-121.593,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'CGC-Prt 5m',0);
INSERT INTO "usage" VALUES('EPSG','8906','helmert_transformation','EPSG','1985','EPSG','1294','EPSG','1157');
INSERT INTO "helmert_transformation" VALUES('EPSG','1986','Lisbon 1890 to WGS 84 (1)','May be taken as a transformation from Lisbon 1890 to ETRS89 - see tfm code 5039.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4666','EPSG','4326',5.0,508.088,-191.042,565.223,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'CGC-Prt 5m',0);
INSERT INTO "usage" VALUES('EPSG','8907','helmert_transformation','EPSG','1986','EPSG','1294','EPSG','1157');
INSERT INTO "helmert_transformation" VALUES('EPSG','1987','Datum 73 to WGS 84 (4)','','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4274','EPSG','4326',1.0,-239.749,88.181,30.488,'EPSG','9001',-0.263,-0.082,-1.211,'EPSG','9104',2.229,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'CGC-Prt 1m',0);
INSERT INTO "usage" VALUES('EPSG','8908','helmert_transformation','EPSG','1987','EPSG','1294','EPSG','1158');
INSERT INTO "helmert_transformation" VALUES('EPSG','1988','Lisbon to WGS 84 (4)','','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4207','EPSG','4326',2.0,-288.885,-91.744,126.244,'EPSG','9001',1.691,-0.41,0.211,'EPSG','9104',-4.598,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'CGC-Prt 1m',0);
INSERT INTO "usage" VALUES('EPSG','8909','helmert_transformation','EPSG','1988','EPSG','1294','EPSG','1158');
INSERT INTO "helmert_transformation" VALUES('EPSG','1989','ED50 to WGS 84 (34)','','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4230','EPSG','4326',1.0,-74.292,-135.889,-104.967,'EPSG','9001',0.524,0.136,-0.61,'EPSG','9104',-3.761,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'CGC-Prt 1m',0);
INSERT INTO "usage" VALUES('EPSG','8910','helmert_transformation','EPSG','1989','EPSG','1294','EPSG','1158');
INSERT INTO "helmert_transformation" VALUES('EPSG','1990','Lisbon 1890 to WGS 84 (2)','','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4666','EPSG','4326',1.0,631.392,-66.551,481.442,'EPSG','9001',-1.09,4.445,4.487,'EPSG','9104',-4.43,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'CGC-Prt 1m',0);
INSERT INTO "usage" VALUES('EPSG','8911','helmert_transformation','EPSG','1990','EPSG','1294','EPSG','1158');
INSERT INTO "helmert_transformation" VALUES('EPSG','1992','Datum 73 to ETRS89 (3)','Parameters calculated in 1998 using 9 common stations. Published in 2001. Replaces Datum 73 to ETRS89 (1) (code 1657). Replaced by Datum 73 to ETRS89 (5) (code 5037).','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4274','EPSG','4258',1.0,-231.034,102.615,26.836,'EPSG','9001',-0.615,0.198,-0.881,'EPSG','9104',1.786,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'CGC-Prt 1m',0);
INSERT INTO "usage" VALUES('EPSG','8913','helmert_transformation','EPSG','1992','EPSG','1294','EPSG','1158');
INSERT INTO "helmert_transformation" VALUES('EPSG','1993','IKBD-92 to WGS 84 (4)','This transformation was exact in 1992, deteriorating with tectonic plate motion at several centimetres per year.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4667','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'UN-Irq Kwt',0);
INSERT INTO "usage" VALUES('EPSG','8914','helmert_transformation','EPSG','1993','EPSG','2876','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','1994','Reykjavik 1900 to WGS 84 (1)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4657','EPSG','4326',10.0,-28.0,199.0,5.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'LMI-Isl',0);
INSERT INTO "usage" VALUES('EPSG','8915','helmert_transformation','EPSG','1994','EPSG','3262','EPSG','1157');
INSERT INTO "helmert_transformation" VALUES('EPSG','1995','Dealul Piscului 1930 to WGS 84 (1)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4316','EPSG','4326',10.0,103.25,-100.4,-307.19,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NAMR-Rom',0);
INSERT INTO "usage" VALUES('EPSG','8916','helmert_transformation','EPSG','1995','EPSG','3295','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1996','Dealul Piscului 1970 to WGS 84 (1)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4317','EPSG','4326',10.0,44.107,-116.147,-54.648,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'Shell-Rom',1);
INSERT INTO "usage" VALUES('EPSG','8917','helmert_transformation','EPSG','1996','EPSG','1197','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1997','Lisbon to ETRS89 (2)','Derived in 2001. Replaces Lisbon to ETRS89 (1) (code 1655). Also given to greater precision but no more accuracy on ICC web site using Coordinate Frame method. Replaced by Lisbon to ETRS89 (3) (code 5038).','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4207','EPSG','4258',2.0,-282.1,-72.2,120.0,'EPSG','9001',-1.529,0.145,-0.89,'EPSG','9104',-4.46,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'ICC-Prt 2001',0);
INSERT INTO "usage" VALUES('EPSG','8918','helmert_transformation','EPSG','1997','EPSG','1294','EPSG','1042');
INSERT INTO "helmert_transformation" VALUES('EPSG','1998','ED50 to WGS 84 (36)','Approximation to better than 0.5m of CT adopted in June 2003 (see ED50 to WGS 84 (35), code 1052). Recommended for Germany North Sea petroleum purposes. Acceptable to Landesbergamt for Lower Saxony and Bundesanstalt für Seeschifffahrt und Hydrographie.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4230','EPSG','4326',1.0,-157.89,-17.16,-78.41,'EPSG','9001',2.118,2.697,-1.434,'EPSG','9104',-5.38,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Ger Nsea',0);
INSERT INTO "usage" VALUES('EPSG','8919','helmert_transformation','EPSG','1998','EPSG','2879','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','1999','ED50 to WGS 84 (32)','Parameter values taken from ED87 to WGS 84 (2) (tfm code 1960) assuming that ED87 is identical to ED50. Errors caused by this assumption can reach 3m.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4230','EPSG','4326',3.0,-83.11,-97.38,-117.22,'EPSG','9001',0.005693,-0.04469,0.04428,'EPSG','9104',1.218,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NAM-Nld-Nsea',1);
INSERT INTO "usage" VALUES('EPSG','8920','helmert_transformation','EPSG','1999','EPSG','1630','EPSG','1217');
INSERT INTO "helmert_transformation" VALUES('EPSG','3817','HD1909 to WGS 84 (1)','Horizontal coordinates of 66 points of the National Geodetic Network were used to compute this transformation.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','3819','EPSG','4326',3.0,595.48,121.69,515.35,'EPSG','9001',-4.115,2.9383,-0.853,'EPSG','9104',-3.408,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'ELTE-Hun',0);
INSERT INTO "usage" VALUES('EPSG','8941','helmert_transformation','EPSG','3817','EPSG','1119','EPSG','1153');
INSERT INTO "helmert_transformation" VALUES('EPSG','3830','TWD97 to WGS 84 (1)','Approximation at the +/- 1m level assuming that TWD97 is equivalent to WGS 84.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','3824','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-Twn',0);
INSERT INTO "usage" VALUES('EPSG','8944','helmert_transformation','EPSG','3830','EPSG','1228','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','3894','IGRS to WGS 84 (1)','Approximation at the +/- 1m level assuming that IGRS is equivalent to WGS 84 within the accuracy of the transformation.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','3889','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-Irq',0);
INSERT INTO "usage" VALUES('EPSG','8963','helmert_transformation','EPSG','3894','EPSG','1124','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','3904','ED50 to WGS 84 (32)','Parameter values from ED87 to ETRS89 (1) (tfm code 4078), assuming that ED50 is identical to ED87 and ETRS89 to WGS 84. Errors caused by this assumption can reach 3-5m. Used by NAM for offshore operations until mid 2004, then replaced by tfm code 1311.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4230','EPSG','4326',5.0,-83.11,-97.38,-117.22,'EPSG','9001',0.0276,-0.2167,0.2147,'EPSG','9109',0.1218,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'Rws-Nld-Nsea',0);
INSERT INTO "usage" VALUES('EPSG','8969','helmert_transformation','EPSG','3904','EPSG','1630','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','3905','ED87 to WGS 84 (2)','Parameter values taken from ED87 to ETRS89 (1) (tfm code 4078) assuming that ETRS89 is coincident with WGS 84 within the accuracy of the transformation. Used as a tfm between ED50 and WGS 84 - see code 3904.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4231','EPSG','4326',1.0,-83.11,-97.38,-117.22,'EPSG','9001',0.0276,-0.2167,0.2147,'EPSG','9109',0.1218,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-Eur',0);
INSERT INTO "usage" VALUES('EPSG','8970','helmert_transformation','EPSG','3905','EPSG','1297','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','3914','MGI 1901 to ETRS89 (3)','Derived at 11 points. May be taken as approximate transformation MGI 1901 to WGS 84 - see code 3915. Superseded by MGI 1901 to Slovenia 1996 (12) (code 8689).','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','3906','EPSG','4258',1.0,426.9,142.6,460.1,'EPSG','9001',4.91,4.49,-12.42,'EPSG','9104',17.1,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'GURS-Svn',0);
INSERT INTO "usage" VALUES('EPSG','8972','helmert_transformation','EPSG','3914','EPSG','3307','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','3915','MGI 1901 to WGS 84 (5)','Parameter values from MGI 1901 to ETRS89 (3) (code 3914). Assumes ETRS89 and WGS 84 can be considered the same to within the accuracy of the transformation.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','3906','EPSG','4326',1.0,426.9,142.6,460.1,'EPSG','9001',4.91,4.49,-12.42,'EPSG','9104',17.1,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Svn',0);
INSERT INTO "usage" VALUES('EPSG','8973','helmert_transformation','EPSG','3915','EPSG','3307','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','3916','MGI 1901 to Slovenia 1996 (1)','Info source also gives a separate reverse tfm with slightly different parameter values. Given the tfm accuracy these differences are not significant and this tfm can be considered reversible. May be taken as approx tfm MGI 1901 to WGS 84 (see code 3917).','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','3906','EPSG','4765',1.0,409.545,72.164,486.872,'EPSG','9001',-3.085957,-5.46911,11.020289,'EPSG','9104',17.919665,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'GuRS-Svn',0);
INSERT INTO "usage" VALUES('EPSG','8974','helmert_transformation','EPSG','3916','EPSG','3307','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','3917','MGI 1901 to WGS 84 (9)','Parameter values from MGI 1901 to Slovenia 1996 (1) (code 3916). Assumes Slovenia 1996 and WGS 84 can be considered the same to within the accuracy of the transformation.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','3906','EPSG','4326',1.0,409.545,72.164,486.872,'EPSG','9001',-3.085957,-5.46911,11.020289,'EPSG','9104',17.919665,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-Svn 96',0);
INSERT INTO "usage" VALUES('EPSG','8975','helmert_transformation','EPSG','3917','EPSG','3307','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','3918','MGI 1901 to Slovenia 1996 (2)','Info source also gives a separate reverse tfm with slightly different parameter values. Given the tfm accuracy these differences are not significant and this tfm can be considered reversible.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','3906','EPSG','4765',0.5,315.393,186.223,499.609,'EPSG','9001',-6.445954,-8.131631,13.208641,'EPSG','9104',23.449046,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'GuRS-Svn W',0);
INSERT INTO "usage" VALUES('EPSG','8976','helmert_transformation','EPSG','3918','EPSG','3564','EPSG','1035');
INSERT INTO "helmert_transformation" VALUES('EPSG','3919','MGI 1901 to Slovenia 1996 (3)','Info source also gives a separate reverse tfm with slightly different parameter values. Given the tfm accuracy these differences are not significant and this tfm can be considered reversible.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','3906','EPSG','4765',0.5,464.939,-21.478,504.497,'EPSG','9001',0.403,-4.228747,9.954942,'EPSG','9104',12.795378,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'GuRS-Svn NE',0);
INSERT INTO "usage" VALUES('EPSG','8977','helmert_transformation','EPSG','3919','EPSG','3565','EPSG','1035');
INSERT INTO "helmert_transformation" VALUES('EPSG','3921','MGI 1901 to Slovenia 1996 (4)','Info source also gives a separate reverse tfm with slightly different parameter values. Given the tfm accuracy these differences are not significant and this tfm can be considered reversible.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','3906','EPSG','4765',0.5,459.968,82.193,458.756,'EPSG','9001',-3.565234,-3.700593,10.860523,'EPSG','9104',15.507563,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'GuRS-Svn SE 0.5m',0);
INSERT INTO "usage" VALUES('EPSG','8978','helmert_transformation','EPSG','3921','EPSG','3566','EPSG','1035');
INSERT INTO "helmert_transformation" VALUES('EPSG','3922','MGI 1901 to Slovenia 1996 (5)','Info source also gives a separate reverse tfm with slightly different parameter values. Given the tfm accuracy these differences are not significant and this tfm can be considered reversible.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','3906','EPSG','4765',0.3,427.914,105.528,510.908,'EPSG','9001',-4.992523,-5.898813,10.306673,'EPSG','9104',12.431493,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'GuRS-Svn SE 0.3m',0);
INSERT INTO "usage" VALUES('EPSG','8979','helmert_transformation','EPSG','3922','EPSG','3567','EPSG','1033');
INSERT INTO "helmert_transformation" VALUES('EPSG','3923','MGI 1901 to Slovenia 1996 (6)','Info source also gives a separate reverse tfm with slightly different parameter values. Given the tfm accuracy these differences are not significant and this tfm can be considered reversible.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','3906','EPSG','4765',0.3,468.63,81.389,445.221,'EPSG','9001',-3.839242,-3.262525,10.566866,'EPSG','9104',16.132726,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'GuRS-Svn Dol',0);
INSERT INTO "usage" VALUES('EPSG','8980','helmert_transformation','EPSG','3923','EPSG','3568','EPSG','1033');
INSERT INTO "helmert_transformation" VALUES('EPSG','3924','MGI 1901 to Slovenia 1996 (7)','Info source also gives a separate reverse tfm with slightly different parameter values. Given the tfm accuracy these differences are not significant and this tfm can be considered reversible.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','3906','EPSG','4765',0.3,439.5,-11.77,494.976,'EPSG','9001',-0.026585,-4.65641,10.155824,'EPSG','9104',16.270002,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'GuRS-Svn Staj',0);
INSERT INTO "usage" VALUES('EPSG','8981','helmert_transformation','EPSG','3924','EPSG','3569','EPSG','1033');
INSERT INTO "helmert_transformation" VALUES('EPSG','3925','MGI 1901 to Slovenia 1996 (8)','Info source also gives a separate reverse tfm with slightly different parameter values. Given the tfm accuracy these differences are not significant and this tfm can be considered reversible.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','3906','EPSG','4765',0.3,524.442,3.275,519.002,'EPSG','9001',0.013287,-3.119714,10.232693,'EPSG','9104',4.184981,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'GuRS-Svn Pom',0);
INSERT INTO "usage" VALUES('EPSG','8982','helmert_transformation','EPSG','3925','EPSG','3570','EPSG','1033');
INSERT INTO "helmert_transformation" VALUES('EPSG','3926','MGI 1901 to Slovenia 1996 (9)','Info source also gives a separate reverse tfm with slightly different parameter values. Given the tfm accuracy these differences are not significant and this tfm can be considered reversible.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','3906','EPSG','4765',0.3,281.529,45.963,537.515,'EPSG','9001',-2.570437,-9.648271,10.759507,'EPSG','9104',26.465548,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'GuRS-Svn Gor',0);
INSERT INTO "usage" VALUES('EPSG','8983','helmert_transformation','EPSG','3926','EPSG','3571','EPSG','1033');
INSERT INTO "helmert_transformation" VALUES('EPSG','3927','MGI 1901 to Slovenia 1996 (10)','Info source also gives a separate reverse tfm with slightly different parameter values. Given the tfm accuracy these differences are not significant and this tfm can be considered reversible.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','3906','EPSG','4765',0.3,355.845,274.282,462.979,'EPSG','9001',-9.086933,-6.491055,14.502181,'EPSG','9104',20.888647,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'GuRS-Svn Prim',0);
INSERT INTO "usage" VALUES('EPSG','8984','helmert_transformation','EPSG','3927','EPSG','3572','EPSG','1033');
INSERT INTO "helmert_transformation" VALUES('EPSG','3928','MGI 1901 to Slovenia 1996 (11)','Info source also gives a separate reverse tfm with slightly different parameter values. Given the tfm accuracy these differences are not significant and this tfm can be considered reversible.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','3906','EPSG','4765',0.3,400.629,90.651,472.249,'EPSG','9001',-3.261138,-5.263404,11.83739,'EPSG','9104',20.022676,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'GuRS-Svn cen',0);
INSERT INTO "usage" VALUES('EPSG','8985','helmert_transformation','EPSG','3928','EPSG','3573','EPSG','1033');
INSERT INTO "helmert_transformation" VALUES('EPSG','3962','MGI 1901 to WGS 84 (1)','Accuracy estimate not available from information source but established empirically by OGP.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','3906','EPSG','4326',5.0,682.0,-203.0,480.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NIMA-balk',0);
INSERT INTO "usage" VALUES('EPSG','9010','helmert_transformation','EPSG','3962','EPSG','2370','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','3963','MGI 1901 to ETRS89 (2)','May be taken as approximate transformation MGI 1901 to WGS 84 - see code 3964.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','3906','EPSG','4258',1.0,551.7,162.9,467.9,'EPSG','9001',6.04,1.96,-11.38,'EPSG','9104',-4.82,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DGU-Hrv',0);
INSERT INTO "usage" VALUES('EPSG','9011','helmert_transformation','EPSG','3963','EPSG','3234','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','3964','MGI 1901 to WGS 84 (4)','Parameter values from MGI 1901 to ETRS89 (2) (code 3963). Assumes ETRS89 and WGS 84 can be considered the same to within the accuracy of the transformation.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','3906','EPSG','4326',1.0,551.7,162.9,467.9,'EPSG','9001',6.04,1.96,-11.38,'EPSG','9104',-4.82,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Hrv',0);
INSERT INTO "usage" VALUES('EPSG','9012','helmert_transformation','EPSG','3964','EPSG','3234','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','3965','MGI 1901 to WGS 84 (6)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','3906','EPSG','4326',10.0,695.5,-216.6,491.1,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'JPet-Yug',0);
INSERT INTO "usage" VALUES('EPSG','9013','helmert_transformation','EPSG','3965','EPSG','3536','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','3971','PSAD56 to SIRGAS 1995 (1)','Derived at 42 points. May be taken as transformation PSAD56 to WGS 84 - see code 3990.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4248','EPSG','4170',5.0,-60.31,245.935,31.008,'EPSG','9001',-12.324,-3.755,7.37,'EPSG','9104',0.447,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGM-Ecu',0);
INSERT INTO "usage" VALUES('EPSG','9016','helmert_transformation','EPSG','3971','EPSG','3241','EPSG','1045');
INSERT INTO "helmert_transformation" VALUES('EPSG','3972','Chua to WGS 84 (2)','Mandatory for SICAD use until 2005. Replaced by Chua to SIRGAS 2000 (tfm code 4069).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4224','EPSG','4326',5.0,-143.87,243.37,-33.52,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'SICAD-Bra DF pre 2000',0);
INSERT INTO "usage" VALUES('EPSG','9017','helmert_transformation','EPSG','3972','EPSG','3619','EPSG','1143');
INSERT INTO "helmert_transformation" VALUES('EPSG','3990','PSAD56 to WGS 84 (14)','Parameter values from PSAD56 to SIRGAS 1995 (1) (code 3971). Assumes SIRGAS 1995 and WGS 84 can be considered the same to within the accuracy of the transformation.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4248','EPSG','4326',5.0,-60.31,245.935,31.008,'EPSG','9001',-12.324,-3.755,7.37,'EPSG','9104',0.447,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGM-Ecu',0);
INSERT INTO "usage" VALUES('EPSG','9024','helmert_transformation','EPSG','3990','EPSG','3241','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','3998','Arc 1960 to WGS 84 (4)','Derived at 3 stations. Accuracy 20m in each axis. Info source gives source CRS as Arc 1950. From inspection of parameter values, analysis of TR8350.2 contour chart and geographic applicability of CRS, OGP believes that the this should be Arc 1960.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4210','EPSG','4326',35.0,-153.0,-5.0,-292.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Bdi',0);
INSERT INTO "usage" VALUES('EPSG','9025','helmert_transformation','EPSG','3998','EPSG','1058','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','4064','RGRDC 2005 to WGS 84 (1)','Approximation at the +/- 1m level assuming that RGRDC 2005 is equivalent to WGS 84 within the accuracy of the transformation.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4046','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-DUC',0);
INSERT INTO "usage" VALUES('EPSG','9027','helmert_transformation','EPSG','4064','EPSG','3613','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','4065','Katanga 1955 to RGRDC 2005 (1)','Derived at 4 stations in Lubumbashi area. May be taken as approximate transformation Katanga 1955 to WGS 84 - see code 4066.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4695','EPSG','4046',1.5,-103.746,-9.614,-255.95,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'Rec-DUC',0);
INSERT INTO "usage" VALUES('EPSG','9028','helmert_transformation','EPSG','4065','EPSG','3614','EPSG','1151');
INSERT INTO "helmert_transformation" VALUES('EPSG','4066','Katanga 1955 to WGS 84 (1)','Parameter values taken from Katanga 1955 to RGRDC 2005 (1) (code 4065) assuming that RGRDC 2005 is equivalent to WGS 84 within the accuracy of the transformation.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4695','EPSG','4326',1.5,-103.746,-9.614,-255.95,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'Rec-DUC',0);
INSERT INTO "usage" VALUES('EPSG','9029','helmert_transformation','EPSG','4066','EPSG','3614','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','4067','Katanga 1955 to RGRDC 2005 (2)','Derived at 5 stations across Lubumbashi-Likasi region. Used as transformation Katanga 1955 to WGS 84 - see code 4068.','EPSG','9636','Molodensky-Badekas (CF geog2D domain)','EPSG','4695','EPSG','4046',0.5,-102.283,-10.277,-257.396,'EPSG','9001',-3.976,-0.002,-6.203,'EPSG','9104',12.315,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,5580868.818,2826402.46,-1243557.996,'EPSG','9001','SDG-DUC',0);
INSERT INTO "usage" VALUES('EPSG','9030','helmert_transformation','EPSG','4067','EPSG','3614','EPSG','1035');
INSERT INTO "helmert_transformation" VALUES('EPSG','4068','Katanga 1955 to WGS 84 (2)','Parameter values taken from Katanga 1955 to RGRDC 2005 (2) (code 4067) assuming that RGRDC 2005 is equivalent to WGS 84 within the accuracy of the transformation.','EPSG','9636','Molodensky-Badekas (CF geog2D domain)','EPSG','4695','EPSG','4326',1.0,-102.283,-10.277,-257.396,'EPSG','9001',-3.976,-0.002,-6.203,'EPSG','9104',12.315,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,5580868.818,2826402.46,-1243557.996,'EPSG','9001','SDG-DUC',0);
INSERT INTO "usage" VALUES('EPSG','9031','helmert_transformation','EPSG','4068','EPSG','3614','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','4069','Chua to SIRGAS 2000 (1)','Mandatory for SICAD use. Replaces Chua to WGS 84 (2) (code 3972). May be used as a tfm to WGS 84 - see tfm code 4834.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4224','EPSG','4674',5.0,-144.35,242.88,-33.2,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'SICAD-Bra DF',0);
INSERT INTO "usage" VALUES('EPSG','9032','helmert_transformation','EPSG','4069','EPSG','3619','EPSG','1143');
INSERT INTO "helmert_transformation" VALUES('EPSG','4070','Chua to WGS 84 (3)','Parameter values from Chua to SIRGAS 2000 (1) (tfm code 4069) assuming that within the tfm accuracy SIRGAS 2000 is equivalent to WGS 84.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4224','EPSG','4674',5.0,-144.35,242.88,-33.2,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-Bra DF post 2000',1);
INSERT INTO "usage" VALUES('EPSG','9033','helmert_transformation','EPSG','4070','EPSG','1053','EPSG','1097');
INSERT INTO "helmert_transformation" VALUES('EPSG','4076','SREF98 to ETRS89 (1)','May be taken as approximate transformation SREF98 to WGS 84 - see code 4077.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4075','EPSG','4258',0.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-Srb',0);
INSERT INTO "usage" VALUES('EPSG','9035','helmert_transformation','EPSG','4076','EPSG','4543','EPSG','1161');
INSERT INTO "helmert_transformation" VALUES('EPSG','4077','SREF98 to WGS 84 (1)','Approximation at the +/- 1m level assuming that ETRS89 is equivalent to WGS 84. SREF98 is a regional realisation of ETRS89.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4075','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-Srb',0);
INSERT INTO "usage" VALUES('EPSG','9036','helmert_transformation','EPSG','4077','EPSG','4543','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','4078','ED87 to ETRS89 (1)','May be used as a transformation between ED87 and WGS 84 - see tfm code 3905.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4231','EPSG','4258',0.3,-83.11,-97.38,-117.22,'EPSG','9001',0.0276,-0.2167,0.2147,'EPSG','9109',0.1218,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IERS-Eur',0);
INSERT INTO "usage" VALUES('EPSG','9037','helmert_transformation','EPSG','4078','EPSG','1297','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','4084','REGCAN95 to WGS 84 (1)','Approximation at the +/- 1m level assuming that REGCAN95 is equivalent to WGS 84.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4081','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-esp',0);
INSERT INTO "usage" VALUES('EPSG','9038','helmert_transformation','EPSG','4084','EPSG','3199','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','4290','Cadastre 1997 to WGS 84 (1)','Parameter values taken from Cadastre 1997 to RGM04 (1) (transformation code 4478) assuming that RGM04 is coincident with WGS 84 within the accuracy of the transformation.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4475','EPSG','4326',1.0,-381.788,-57.501,-256.673,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-Myt',0);
INSERT INTO "usage" VALUES('EPSG','9067','helmert_transformation','EPSG','4290','EPSG','3340','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','4461','NAD83(HARN) to NAD83(NSRS2007) (1)','Accuracy 0.1 to 0.2m in California, 0.05-0.11 in Oregon, elsewhere better than 0.05m.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4152','EPSG','4759',0.1,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-USA conus',0);
INSERT INTO "usage" VALUES('EPSG','9092','helmert_transformation','EPSG','4461','EPSG','1323','EPSG','1032');
INSERT INTO "helmert_transformation" VALUES('EPSG','4476','RGM04 to WGS 84 (1)','Approximation at the +/- 1m level assuming that RGM04 is equivalent to WGS 84.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4470','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-Myt',0);
INSERT INTO "usage" VALUES('EPSG','9094','helmert_transformation','EPSG','4476','EPSG','1159','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','4477','RGSPM06 to WGS 84 (1)','Approximation at the +/- 1m level assuming that RGSPM06 is equivalent to WGS 84.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4463','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-SPM',0);
INSERT INTO "usage" VALUES('EPSG','9095','helmert_transformation','EPSG','4477','EPSG','1220','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','4478','Cadastre 1997 to RGM04 (1)','May be taken as approximate transformation Cadastre 1997 to WGS 84 - see transformation code 4290.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4475','EPSG','4470',0.1,-381.788,-57.501,-256.673,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'Certu-Myt',0);
INSERT INTO "usage" VALUES('EPSG','9096','helmert_transformation','EPSG','4478','EPSG','3340','EPSG','1031');
INSERT INTO "helmert_transformation" VALUES('EPSG','4560','RRAF 1991 to WGS 84 (1)','Approximation at the +/- 1m level assuming that RRAF91 is equivalent to WGS 84 within the accuracy of the transformation.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4558','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-FrAnt',0);
INSERT INTO "usage" VALUES('EPSG','9097','helmert_transformation','EPSG','4560','EPSG','2824','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','4590','ITRF88 to ITRF2000 (1)','At epoch 1988.0. Rates dX=0.0000 m/yr, dy=0.0006 m/yr, dZ=0.0014 m/yr, rX=rY=0.0"/yr, rZ=-0.00002"/yr, dS=-0.00001 ppm/yr.','EPSG','1033','Position Vector transformation (geocentric domain)','EPSG','4910','EPSG','4919',0.0,-0.0247,-0.0115,0.0979,'EPSG','9001',-0.0001,0.0,0.00018,'EPSG','9104',-0.00895,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IERS',1);
INSERT INTO "usage" VALUES('EPSG','9105','helmert_transformation','EPSG','4590','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','4591','ITRF89 to ITRF2000 (1)','At epoch 1988.0. Rates dX=0.0000 m/yr, dy=0.0006 m/yr, dZ=0.0014 m/yr, rX=rY=0.0"/yr, rZ=-0.00002"/yr, dS=-0.00001 ppm/yr.','EPSG','1033','Position Vector transformation (geocentric domain)','EPSG','4911','EPSG','4919',0.0,-0.0297,-0.0475,0.0739,'EPSG','9001',0.0,0.0,0.00018,'EPSG','9104',-0.00585,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IERS',1);
INSERT INTO "usage" VALUES('EPSG','9106','helmert_transformation','EPSG','4591','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','4592','ITRF90 to ITRF2000 (1)','At epoch 1988.0. Rates dX=0.0000 m/yr, dy=0.0006 m/yr, dZ=0.0014 m/yr, rX=rY=0.0"/yr, rZ=-0.00002"/yr, dS=-0.00001 ppm/yr.','EPSG','1033','Position Vector transformation (geocentric domain)','EPSG','4912','EPSG','4919',0.0,-0.0247,-0.0235,0.0359,'EPSG','9001',0.0,0.0,0.00018,'EPSG','9104',-0.00245,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IERS',1);
INSERT INTO "usage" VALUES('EPSG','9107','helmert_transformation','EPSG','4592','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','4593','ITRF91 to ITRF2000 (1)','At epoch 1988.0. Rates dX=0.0000 m/yr, dy=0.0006 m/yr, dZ=0.0014 m/yr, rX=rY=0.0"/yr, rZ=-0.00002"/yr, dS=-0.00001 ppm/yr.','EPSG','1033','Position Vector transformation (geocentric domain)','EPSG','4913','EPSG','4919',0.0,-0.0267,-0.0275,0.0199,'EPSG','9001',0.0,0.0,0.00018,'EPSG','9104',-0.00215,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IERS',1);
INSERT INTO "usage" VALUES('EPSG','9108','helmert_transformation','EPSG','4593','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','4594','ITRF92 to ITRF2000 (1)','At epoch 1988.0. Rates dX=0.0000 m/yr, dy=0.0006 m/yr, dZ=0.0014 m/yr, rX=rY=0.0"/yr, rZ=-0.00002"/yr, dS=-0.00001 ppm/yr.','EPSG','1033','Position Vector transformation (geocentric domain)','EPSG','4914','EPSG','4919',0.0,-0.0147,-0.0135,0.0139,'EPSG','9001',0.0,0.0,0.00018,'EPSG','9104',-0.00075,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IERS',1);
INSERT INTO "usage" VALUES('EPSG','9109','helmert_transformation','EPSG','4594','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','4595','ITRF93 to ITRF2000 (1)','At epoch 1988.0. Rates dX=0.0029 m/yr, dy=0.0002 m/yr, dZ=0.0006 m/yr, rX=0.00011"/yr, rY=0.00019"/yr, rZ=-0.00007"/yr, dS=-0.00001 ppm/yr.','EPSG','1033','Position Vector transformation (geocentric domain)','EPSG','4915','EPSG','4919',0.0,-0.0127,-0.0065,0.0209,'EPSG','9001',0.00039,-0.0008,0.00114,'EPSG','9104',-0.00195,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IERS',1);
INSERT INTO "usage" VALUES('EPSG','9110','helmert_transformation','EPSG','4595','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','4596','ITRF94 to ITRF2000 (1)','At epoch 1997.0. Rates dX=0.0000 m/yr, dy=0.0006 m/yr, dZ=0.0014 m/yr, rX=rY=0.0"/yr, rZ=-0.00002"/yr, dS=-0.00001 ppm/yr.','EPSG','1033','Position Vector transformation (geocentric domain)','EPSG','4916','EPSG','4919',0.0,-0.0067,-0.0061,0.0185,'EPSG','9001',0.0,0.0,0.0,'EPSG','9104',-0.00155,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IERS',1);
INSERT INTO "usage" VALUES('EPSG','9111','helmert_transformation','EPSG','4596','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','4597','ITRF96 to ITRF2000 (1)','At epoch 1997.0. Rates dX=0.0000 m/yr, dy=0.0006 m/yr, dZ=0.0014 m/yr, rX=rY=0.0"/yr, rZ=-0.00002"/yr, dS=-0.00001 ppm/yr.','EPSG','1033','Position Vector transformation (geocentric domain)','EPSG','4917','EPSG','4919',0.0,-0.0067,-0.0061,0.0185,'EPSG','9001',0.0,0.0,0.0,'EPSG','9104',-0.00155,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IERS',1);
INSERT INTO "usage" VALUES('EPSG','9112','helmert_transformation','EPSG','4597','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','4598','ITRF97 to ITRF2000 (1)','At epoch 1997.0. Rates dX=0.0000 m/yr, dy=0.0006 m/yr, dZ=0.0014 m/yr, rX=rY=0.0"/yr, rZ=-0.00002"/yr, dS=-0.00001 ppm/yr.','EPSG','1033','Position Vector transformation (geocentric domain)','EPSG','4918','EPSG','4919',0.0,-0.0067,-0.0061,0.0185,'EPSG','9001',0.0,0.0,0.0,'EPSG','9104',-0.00155,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IERS',1);
INSERT INTO "usage" VALUES('EPSG','9113','helmert_transformation','EPSG','4598','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','4599','ITRF2000 to ITRF2005 (1)','At epoch 2000.0. Rates dX=0.0002 m/yr, dy=-0.0001 m/yr, dZ=0.0018 m/yr, rX=rY=rZ=0.0"/yr, dS=-0.00008 ppm/yr.','EPSG','1033','Position Vector transformation (geocentric domain)','EPSG','4919','EPSG','4896',0.0,-0.0001,0.0008,0.0058,'EPSG','9001',0.0,0.0,0.0,'EPSG','9104',-0.0004,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IERS',1);
INSERT INTO "usage" VALUES('EPSG','9114','helmert_transformation','EPSG','4599','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','4827','S-JTSK to ETRS89 (4)','Derived at approximately 700 points. Scale difference incorporated into rotation matrix. Replaces S-JTSK to ETRS89 (3) (code 4829) to use more common method. May be taken as approximate transformation S-JTSK to WGS 84 - see code 4836.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4156','EPSG','4258',1.0,485.0,169.5,483.8,'EPSG','9001',7.786,4.398,4.103,'EPSG','9104',0.0,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'UGKK-Svk PV',0);
INSERT INTO "usage" VALUES('EPSG','9120','helmert_transformation','EPSG','4827','EPSG','1211','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','4828','S-JTSK to WGS 84 (4)','Parameter values from S-JTSK to ETRS89 (4) (code 4827). Assumes ETRS89 and WGS 84 can be considered the same to within the accuracy of the transformation.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4156','EPSG','4326',1.0,485.0,169.5,483.5,'EPSG','9001',7.786,4.398,4.103,'EPSG','9104',0.0,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Svk',1);
INSERT INTO "usage" VALUES('EPSG','9121','helmert_transformation','EPSG','4828','EPSG','1211','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','4829','S-JTSK to ETRS89 (3)','Replaced by S-JTSK to ETRS89 (4) (code 4827) to use more commonly encountered transformation method.','EPSG','9636','Molodensky-Badekas (CF geog2D domain)','EPSG','4156','EPSG','4258',0.5,558.7,68.8,452.2,'EPSG','9001',-8.025,-4.105,-4.295,'EPSG','9104',5.74,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,3977358.114,1407223.203,4765441.589,'EPSG','9001','UGKK-Svk MB',0);
INSERT INTO "usage" VALUES('EPSG','9122','helmert_transformation','EPSG','4829','EPSG','1211','EPSG','1035');
INSERT INTO "helmert_transformation" VALUES('EPSG','4830','Amersfoort to ETRS89 (5)','Replaces Amersfoort to ETRS89 (3) (tfm code 15739). Dutch sources also quote an equivalent transformation using the Molodenski-Badekas method - see tfm code 4831.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4289','EPSG','4258',0.5,565.4171,50.3319,465.5524,'EPSG','9001',1.9342,-1.6677,9.1019,'EPSG','9109',4.0725,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NCG-Nld 2008 CF',0);
INSERT INTO "usage" VALUES('EPSG','9123','helmert_transformation','EPSG','4830','EPSG','1275','EPSG','1035');
INSERT INTO "helmert_transformation" VALUES('EPSG','4831','Amersfoort to ETRS89 (6)','Replaces Amersfoort to ETRS89 (4) (tfm code 15740). Dutch sources also quote an equivalent transformation using the Coordinate Frame 7-parameter method - see tfm code 4830.','EPSG','9636','Molodensky-Badekas (CF geog2D domain)','EPSG','4289','EPSG','4258',0.5,593.0248,25.9984,478.7459,'EPSG','9001',1.9342,-1.6677,9.1019,'EPSG','9109',4.0725,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,3903453.1482,368135.3134,5012970.3051,'EPSG','9001','NCG-Nld 2008 MB',0);
INSERT INTO "usage" VALUES('EPSG','9124','helmert_transformation','EPSG','4831','EPSG','1275','EPSG','1035');
INSERT INTO "helmert_transformation" VALUES('EPSG','4832','Mexico ITRF92 to WGS 84 (1)','Approximation at the +/- 1m level assuming that Mexico ITRF92 is equivalent to WGS 84.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4483','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-Mex',0);
INSERT INTO "usage" VALUES('EPSG','9125','helmert_transformation','EPSG','4832','EPSG','1160','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','4833','Amersfoort to WGS 84 (4)','Parameter values from Amersfoort to ETRS89 (5) (tfm code 4830) assuming that ETRS89 is equivalent to WGS 84 within the accuracy of the transformation. Replaces Amersfoort to WGS 84 (3) (code 15934).','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4289','EPSG','4326',1.0,565.4171,50.3319,465.5524,'EPSG','9001',1.9342,-1.6677,9.1019,'EPSG','9109',4.0725,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-Nld 2008',0);
INSERT INTO "usage" VALUES('EPSG','9126','helmert_transformation','EPSG','4833','EPSG','1275','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','4834','Chua to WGS 84 (3)','Parameter values from Chua to SIRGAS 2000 (1) (tfm code 4069) assuming that within the tfm accuracy SIRGAS 2000 is equivalent to WGS 84.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4224','EPSG','4326',5.0,-144.35,242.88,-33.2,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-Bra DF post 2000',0);
INSERT INTO "usage" VALUES('EPSG','9127','helmert_transformation','EPSG','4834','EPSG','3619','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','4835','Tahiti 79 to WGS 84 (1)','Approximation at the +/- 1m level assuming that RGPF is equivalent to WGS 84. Parameter values taken from Tahiti 79 to RGPF (1) (tfm code 15756).','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4690','EPSG','4326',1.0,221.525,152.948,176.768,'EPSG','9001',2.3847,1.3896,0.877,'EPSG','9104',11.4741,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Pyf',1);
INSERT INTO "usage" VALUES('EPSG','9128','helmert_transformation','EPSG','4835','EPSG','3124','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','4836','S-JTSK to WGS 84 (4)','Parameter values from S-JTSK to ETRS89 (4) (code 4827). Assumes ETRS89 and WGS 84 can be considered the same to within the accuracy of the transformation.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4156','EPSG','4326',1.0,485.0,169.5,483.8,'EPSG','9001',7.786,4.398,4.103,'EPSG','9104',0.0,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Svk',0);
INSERT INTO "usage" VALUES('EPSG','9129','helmert_transformation','EPSG','4836','EPSG','1211','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','4840','RGFG95 to WGS 84 (2)','Replaces RGFG95 to WGS 84 (1) (code 1907) which was not put into official use but issued in error.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4624','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Guf 2',0);
INSERT INTO "usage" VALUES('EPSG','9132','helmert_transformation','EPSG','4840','EPSG','1097','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','4905','PTRA08 to WGS 84 (1)','Approximation at the +/- 1m level assuming that PTRA08 is equivalent to WGS 84.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','5013','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-pt RA',0);
INSERT INTO "usage" VALUES('EPSG','9148','helmert_transformation','EPSG','4905','EPSG','3670','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','5021','Porto Santo 1995 to PTRA08 (1)','Derived at 34 points in May 2009. Residuals at 25 test points within 0.5m horizontal and 2m vertical. Info source also provides a Position Vector tfm of similar accuracy. See codes 5022 and 5023 for preferred tfms for islands of Porto Santo and Maderia.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4663','EPSG','5013',2.0,-503.229,-247.375,312.582,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGP-Prt MadArch',0);
INSERT INTO "usage" VALUES('EPSG','9162','helmert_transformation','EPSG','5021','EPSG','1314','EPSG','1042');
INSERT INTO "helmert_transformation" VALUES('EPSG','5022','Porto Santo 1995 to PTRA08 (2)','Derived at 22 points in May 2009. Residuals at 17 test points within 0.1m horizontal and 1m vertical. Info source also provides a less accurate 3-parameter transformation (dX=-503.174m, dY=-247.255m, dZ=312.316m).','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4663','EPSG','5013',1.0,-303.956,224.556,214.306,'EPSG','9001',9.405,-6.626,-12.583,'EPSG','9104',1.327,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGP-Prt Mad',0);
INSERT INTO "usage" VALUES('EPSG','9163','helmert_transformation','EPSG','5022','EPSG','3679','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','5023','Porto Santo 1995 to PTRA08 (3)','Derived at 14 points in May 2009. Residuals at 6 test points within 0.1m horizontal and 0.2m vertical. Info source also provides a 7-parameter Position Vector transformation of similar accuracy.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4663','EPSG','5013',0.2,-503.3,-247.574,313.025,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGP-Prt Porto Santo',0);
INSERT INTO "usage" VALUES('EPSG','9164','helmert_transformation','EPSG','5023','EPSG','3680','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','5024','Azores Oriental 1995 to PTRA08 (1)','Derived at 53 points in May 2009. Residuals at 58 test points within 0.2m horizontal and 2m vertical. Info source also provides a Position Vector tfm of similar accuracy. See codes 5025-26 for preferred tfms for islands of Sao Miguel and Santa Maria.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4664','EPSG','5013',2.0,-204.926,140.353,55.063,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGP-Prt Az E',0);
INSERT INTO "usage" VALUES('EPSG','9165','helmert_transformation','EPSG','5024','EPSG','1345','EPSG','1042');
INSERT INTO "helmert_transformation" VALUES('EPSG','5025','Azores Oriental 1995 to PTRA08 (2)','Derived at 36 points in May 2009. Residuals at 43 test points within 0.2m horizontal and 0.3m vertical. Info source also provides a 7-parameter Position Vector transformation of similar accuracy.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4664','EPSG','5013',0.3,-204.519,140.159,55.404,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGP-Prt Az Mig',0);
INSERT INTO "usage" VALUES('EPSG','9166','helmert_transformation','EPSG','5025','EPSG','2871','EPSG','1033');
INSERT INTO "helmert_transformation" VALUES('EPSG','5026','Azores Oriental 1995 to PTRA08 (3)','Derived at 18 points in May 2009. Residuals at 14 test points within 0.1m. Info source also provides a 7-parameter Position Vector transformation of similar accuracy.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4664','EPSG','5013',0.1,-205.808,140.771,54.326,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGP-Prt Az S.Maria',0);
INSERT INTO "usage" VALUES('EPSG','9167','helmert_transformation','EPSG','5026','EPSG','3683','EPSG','1031');
INSERT INTO "helmert_transformation" VALUES('EPSG','5027','Azores Central 1995 to PTRA08 (1)','Derived at 112 points in May 2009. Residuals at 184 test points within 0.5m horizontal and 1m vertical. Info source also provides a Position Vector tfm of similar accuracy. See codes 5028-32 for preferred tfms for individual islands.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4665','EPSG','5013',2.0,-105.679,166.1,-37.322,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGP-Prt Az C',0);
INSERT INTO "usage" VALUES('EPSG','9168','helmert_transformation','EPSG','5027','EPSG','1301','EPSG','1042');
INSERT INTO "helmert_transformation" VALUES('EPSG','5028','Azores Central 1995 to PTRA08 (2)','Derived at 24 points in May 2009. Residuals at 37 test points within 0.1m horizontal and 0.3m vertical. Info source also provides a Position Vector tfm of similar accuracy.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4665','EPSG','5013',0.5,-105.377,165.769,-36.965,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGP-Prt Az Faial',0);
INSERT INTO "usage" VALUES('EPSG','9169','helmert_transformation','EPSG','5028','EPSG','2873','EPSG','1035');
INSERT INTO "helmert_transformation" VALUES('EPSG','5029','Azores Central 1995 to PTRA08 (3)','Derived at 11 points in May 2009. Residuals at 15 test points within 0.1m horizontal and 0.2m vertical. Info source also provides a Position Vector tfm of similar accuracy.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4665','EPSG','5013',0.2,-105.359,165.804,-37.05,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGP-Prt Az Graciosa',0);
INSERT INTO "usage" VALUES('EPSG','9170','helmert_transformation','EPSG','5029','EPSG','3681','EPSG','1032');
INSERT INTO "helmert_transformation" VALUES('EPSG','5030','Azores Central 1995 to PTRA08 (4)','Derived at 34 points in May 2009. Residuals at 38 test points within 0.2m horizontal and 1m vertical. Info source also provides a Position Vector tfm of similar accuracy.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4665','EPSG','5013',1.0,-105.531,166.39,-37.326,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGP-Prt Az Pico',0);
INSERT INTO "usage" VALUES('EPSG','9171','helmert_transformation','EPSG','5030','EPSG','2874','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','5031','Azores Central 1995 to PTRA08 (5)','Derived at 17 points in May 2009. Residuals at 60 test points within 0.1m horizontal and 0.8m vertical. Info source also provides a Position Vector tfm of similar accuracy.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4665','EPSG','5013',0.8,-105.756,165.972,-37.313,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGP-Prt Az S.Jorge',0);
INSERT INTO "usage" VALUES('EPSG','9172','helmert_transformation','EPSG','5031','EPSG','2875','EPSG','1038');
INSERT INTO "helmert_transformation" VALUES('EPSG','5032','Azores Central 1995 to PTRA08 (6)','Derived at 26 points in May 2009. Residuals at 34 test points within 0.1m horizontal and 0.6m vertical. Info source also provides a Position Vector tfm of similar accuracy.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4665','EPSG','5013',0.6,-106.235,166.236,-37.768,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGP-Prt Az Terceira',0);
INSERT INTO "usage" VALUES('EPSG','9173','helmert_transformation','EPSG','5032','EPSG','2872','EPSG','1036');
INSERT INTO "helmert_transformation" VALUES('EPSG','5033','Azores Occidental 1939 to PTRA08 (1)','Derived at 21 points in May 2009. Residuals at 18 test points within 0.1m horizontal and 0.2m vertical. Info source also provides a Position Vector tfm of similar accuracy. See codes 5034-35 for preferred tfms for islands of Flores and Corvo.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4182','EPSG','5013',0.5,-423.058,-172.868,83.772,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGP-Prt Az W',0);
INSERT INTO "usage" VALUES('EPSG','9174','helmert_transformation','EPSG','5033','EPSG','1344','EPSG','1035');
INSERT INTO "helmert_transformation" VALUES('EPSG','5034','Azores Occidental 1939 to PTRA08 (2)','Derived at 18 points in May 2009. Residuals at 15 test points within 0.1m horizontal and 0.2m vertical. Info source also provides a Position Vector tfm of similar accuracy.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4182','EPSG','5013',0.2,-423.053,-172.871,83.771,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGP-Prt Az Flores',0);
INSERT INTO "usage" VALUES('EPSG','9175','helmert_transformation','EPSG','5034','EPSG','3684','EPSG','1032');
INSERT INTO "helmert_transformation" VALUES('EPSG','5035','Azores Occidental 1939 to PTRA08 (3)','Derived at 3 points in May 2009. Residuals at these points within 0.1m horizontal and 0.3m vertical. Info source also provides a Position Vector tfm of similar accuracy.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4182','EPSG','5013',0.3,-423.024,-172.923,83.83,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGP-Prt Az Corvo',0);
INSERT INTO "usage" VALUES('EPSG','9176','helmert_transformation','EPSG','5035','EPSG','3685','EPSG','1033');
INSERT INTO "helmert_transformation" VALUES('EPSG','5036','Datum 73 to ETRS89 (4)','Derived in July 2009 from 119 common stations. Residuals at 833 test points under 3m.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4274','EPSG','4258',3.0,-223.15,110.132,36.711,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'CGC-Prt 2009 3m',0);
INSERT INTO "usage" VALUES('EPSG','9177','helmert_transformation','EPSG','5036','EPSG','1294','EPSG','1043');
INSERT INTO "helmert_transformation" VALUES('EPSG','5037','Datum 73 to ETRS89 (5)','Derived in July 2009 from 119 common stations. Residuals at 833 test points under 2m. Replaces Datum 73 to ETRS89 (3) (tfm code 1992).','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4274','EPSG','4258',2.0,-230.994,102.591,25.199,'EPSG','9001',0.633,-0.239,0.9,'EPSG','9104',1.95,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'CGC-Prt 2009 2m',0);
INSERT INTO "usage" VALUES('EPSG','9178','helmert_transformation','EPSG','5037','EPSG','1294','EPSG','1042');
INSERT INTO "helmert_transformation" VALUES('EPSG','5038','Lisbon to ETRS89 (3)','Derived in July 2009 from 119 common stations. Average residual at 833 test points 2.5m, maximum 7m. Info source also gives a Position Vector tfm which is of similar accuracy. Replaces Lisbon to ETRS89 (2) (tfm code 1997).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4207','EPSG','4258',2.5,-303.861,-60.693,103.607,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'CGC-Prt 2009 7m',0);
INSERT INTO "usage" VALUES('EPSG','9179','helmert_transformation','EPSG','5038','EPSG','1294','EPSG','1043');
INSERT INTO "helmert_transformation" VALUES('EPSG','5039','Lisbon 1890 to ETRS89 (1)','Parameter values taken from Lisbon 1890 to WGS 84 (1) (tfm code 1986) assuming that ETRS89 is equivalent to WGS 84 within the accuracy of the transformation.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4666','EPSG','4258',5.0,508.088,-191.042,565.223,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'CGC-Prt 5m',0);
INSERT INTO "usage" VALUES('EPSG','9180','helmert_transformation','EPSG','5039','EPSG','1294','EPSG','1157');
INSERT INTO "helmert_transformation" VALUES('EPSG','5040','ED50 to ETRS89 (13)','Parameter values taken from ED50 to WGS 84 (33) (tfm code 1985) assuming that ETRS89 and WGS 84 are the same within the accuracy of the transformation.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4230','EPSG','4258',5.0,-87.987,-108.639,-121.593,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'CGC-Prt 5m',0);
INSERT INTO "usage" VALUES('EPSG','9181','helmert_transformation','EPSG','5040','EPSG','1294','EPSG','1157');
INSERT INTO "helmert_transformation" VALUES('EPSG','5043','Pulkovo 1995 to WGS 84 (2)','Derived through concatenation of Pulkovo 1995 to PZ-90.02 to WGS 84. Replaces Pulkovo 1995 to WGS 84 (1), tfm code 1281.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4200','EPSG','4326',1.0,24.47,-130.89,-81.56,'EPSG','9001',0.0,0.0,-0.13,'EPSG','9104',-0.22,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'GOST-Rus 2008',0);
INSERT INTO "usage" VALUES('EPSG','9182','helmert_transformation','EPSG','5043','EPSG','1198','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','5044','Pulkovo 1942 to WGS 84 (20)','Derived through concatenation of Pulkovo 1942 to PZ-90.02 to WGS 84. Replaces Pulkovo 1942 to WGS 84 (17) (code 1267).','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4284','EPSG','4326',3.0,23.57,-140.95,-79.8,'EPSG','9001',0.0,-0.35,-0.79,'EPSG','9104',-0.22,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'GOST-Rus 2008',0);
INSERT INTO "usage" VALUES('EPSG','9183','helmert_transformation','EPSG','5044','EPSG','3296','EPSG','1043');
INSERT INTO "helmert_transformation" VALUES('EPSG','5050','Aratu to SIRGAS 2000 (1)','Derived in 2002. Petrobras preferred transformation for all purposes in the area. May be used as transformation between Aratu and WGS 84 - see tfm code 5051.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4208','EPSG','4674',0.5,-157.84,308.54,-146.6,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'PB-Bra BS 2002',0);
INSERT INTO "usage" VALUES('EPSG','9188','helmert_transformation','EPSG','5050','EPSG','3700','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','5051','Aratu to WGS 84 (13)','Parameters from Aratu to SIRGAS 2000 (1) (tfm code 5050) assuming that SIRGAS 2000 and WGS 84 are equal within the accuracy of the transformation. Petrobras preferred parameters for all purposes in the area. Replaces tfm codes 15711 and 15734.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4208','EPSG','4326',1.0,-157.84,308.54,-146.6,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'PB-Bra BS 2002',0);
INSERT INTO "usage" VALUES('EPSG','9189','helmert_transformation','EPSG','5051','EPSG','3700','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','5052','Aratu to SIRGAS 2000 (2)','Derived in 2002. Petrobras preferred transformation for all purposes in the area. May be used as transformation between Aratu and WGS 84 - see tfm code 5053.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4208','EPSG','4674',0.5,-160.31,314.82,-142.25,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'PB-Bra BC 2002',0);
INSERT INTO "usage" VALUES('EPSG','9190','helmert_transformation','EPSG','5052','EPSG','2963','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','5053','Aratu to WGS 84 (14)','Parameters from Aratu to SIRGAS 2000 (2) (tfm code 5052) assuming that SIRGAS 2000 and WGS 84 are equal within the accuracy of the transformation. Petrobras preferred parameters for all purposes in the area. Replaces tfm codes 15710 and 15754.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4208','EPSG','4326',1.0,-160.31,314.82,-142.25,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'PB-Bra BC 2002',0);
INSERT INTO "usage" VALUES('EPSG','9191','helmert_transformation','EPSG','5053','EPSG','2963','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','5054','Aratu to SIRGAS 2000 (3)','Derived in 2002. Petrobras preferred transformation for all purposes in the area. May be used as transformation between Aratu and WGS 84 - see tfm code 5055.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4208','EPSG','4674',0.5,-161.11,310.25,-144.64,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'PB-Bra ES 2002',0);
INSERT INTO "usage" VALUES('EPSG','9192','helmert_transformation','EPSG','5054','EPSG','2964','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','5055','Aratu to WGS 84 (15)','Parameters from Aratu to SIRGAS 2000 (3) (tfm code 5054) assuming that SIRGAS 2000 and WGS 84 are equal within the accuracy of the transformation. Petrobras preferred parameters for all purposes in the area. Replaces tfms 15712 and 15754.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4208','EPSG','4326',1.0,-161.11,310.25,-144.64,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'PB-Bra ES 2002',0);
INSERT INTO "usage" VALUES('EPSG','9193','helmert_transformation','EPSG','5055','EPSG','2964','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','5056','Aratu to SIRGAS 2000 (4)','Derived in 2002. Petrobras preferred transformation for all purposes in the area. May be used as transformation between Aratu and WGS 84 - see tfm code 5057.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4208','EPSG','4674',0.5,-160.4,302.29,-144.19,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'PB-Bra BSUL 2002',0);
INSERT INTO "usage" VALUES('EPSG','9194','helmert_transformation','EPSG','5056','EPSG','3699','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','5057','Aratu to WGS 84 (16)','Parameters from Aratu to SIRGAS 2000 (4) (tfm code 5056) assuming that SIRGAS 2000 and WGS 84 are equal within the accuracy of the transformation. Petrobras preferred parameters for all purposes in the area.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4208','EPSG','4326',1.0,-160.4,302.29,-144.19,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'PB-Bra BSUL 2002',0);
INSERT INTO "usage" VALUES('EPSG','9195','helmert_transformation','EPSG','5057','EPSG','3699','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','5058','Aratu to SIRGAS 2000 (5)','Derived in 2002. Petrobras preferred transformation for all purposes in the area. May be used as transformation between Aratu and WGS 84 - see tfm code 5059.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4208','EPSG','4674',0.5,-153.54,302.33,-152.37,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'PB-Bra BREC 2002',0);
INSERT INTO "usage" VALUES('EPSG','9196','helmert_transformation','EPSG','5058','EPSG','3692','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','5059','Aratu to WGS 84 (17)','Parameters from Aratu to SIRGAS 2000 (5) (tfm code 5058) assuming that SIRGAS 2000 and WGS 84 are equal within the accuracy of the transformation. Petrobras preferred parameters for all purposes in the area.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4208','EPSG','4326',1.0,-153.54,302.33,-152.37,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'PB-Bra BREC 2002',0);
INSERT INTO "usage" VALUES('EPSG','9197','helmert_transformation','EPSG','5059','EPSG','3692','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','5060','Aratu to SIRGAS 2000 (6)','Derived in 2002. Petrobras preferred transformation for all purposes in the area. May be used as transformation between Aratu and WGS 84 - see tfm code 5061.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4208','EPSG','4674',0.5,-151.5,300.09,-151.15,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'PB-Bra BTUC 2002',0);
INSERT INTO "usage" VALUES('EPSG','9198','helmert_transformation','EPSG','5060','EPSG','3693','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','5061','Aratu to WGS 84 (18)','Parameters from Aratu to SIRGAS 2000 (6) (tfm code 5060) assuming that SIRGAS 2000 and WGS 84 are equal within the accuracy of the transformation. Petrobras preferred parameters for all purposes in the area. Replaces tfms 1550-1552.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4208','EPSG','4326',1.0,-151.5,300.09,-151.15,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'PB-Bra BTUC 2002',0);
INSERT INTO "usage" VALUES('EPSG','9199','helmert_transformation','EPSG','5061','EPSG','3693','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','5062','Aratu to SIRGAS 2000 (7)','Derived in 2002. Petrobras preferred transformation for all purposes in the area. May be used as transformation between Aratu and WGS 84 - see tfm code 5063.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4208','EPSG','4674',0.5,-156.8,298.41,-147.41,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'PB-Bra SEAL 2002',0);
INSERT INTO "usage" VALUES('EPSG','9200','helmert_transformation','EPSG','5062','EPSG','3696','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','5063','Aratu to WGS 84 (19)','Parameters from Aratu to SIRGAS 2000 (7) (tfm code 5062) assuming that SIRGAS 2000 and WGS 84 are equal within the accuracy of the transformation. Petrobras preferred parameters for all purposes in the area.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4208','EPSG','4326',1.0,-156.8,298.41,-147.41,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'PB-Bra SEAL 2002',0);
INSERT INTO "usage" VALUES('EPSG','9201','helmert_transformation','EPSG','5063','EPSG','3696','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','5064','Aratu to SIRGAS 2000 (8)','Derived in 2002. Petrobras preferred transformation for all purposes in the area. May be used as transformation between Aratu and WGS 84 - see tfm code 5065.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4208','EPSG','4674',0.5,-157.4,295.05,-150.19,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'PB-Bra PEPB 2002',0);
INSERT INTO "usage" VALUES('EPSG','9202','helmert_transformation','EPSG','5064','EPSG','3697','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','5065','Aratu to WGS 84 (20)','Parameters from Aratu to SIRGAS 2000 (8) (tfm code 5064) assuming that SIRGAS 2000 and WGS 84 are equal within the accuracy of the transformation.Petrobras preferred parameters for all purposes in the area.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4208','EPSG','4326',1.0,-157.4,295.05,-150.19,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'PB-Bra PEPB 2002',0);
INSERT INTO "usage" VALUES('EPSG','9203','helmert_transformation','EPSG','5065','EPSG','3697','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','5066','Aratu to SIRGAS 2000 (9)','Derived in 2002. Petrobras preferred transformation for all purposes in the area. May be used as transformation between Aratu and WGS 84 - see tfm code 5067.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4208','EPSG','4674',0.5,-151.99,287.04,-147.45,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'PB-Bra RNCE 2002',0);
INSERT INTO "usage" VALUES('EPSG','9204','helmert_transformation','EPSG','5066','EPSG','3698','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','5067','Aratu to WGS 84 (21)','Parameters from Aratu to SIRGAS 2000 (9) (tfm code 5066) assuming that SIRGAS 2000 and WGS 84 are equal within the accuracy of the transformation. Petrobras preferred parameters for all purposes in the area.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4208','EPSG','4326',1.0,-151.99,287.04,-147.45,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'PB-Bra RNCE 2002',0);
INSERT INTO "usage" VALUES('EPSG','9205','helmert_transformation','EPSG','5067','EPSG','3698','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','5077','Karbala 1979 to IGRS (1)','Derived at 95 stations mostly south of Baghdad but including 3 in Kirkuk-Erbil area. Maximum residuals 0.3m. May be used as a tfm to WGS 84 - see tfm code 5078.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4743','EPSG','3889',0.3,70.995,-335.916,262.898,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'MWR-Irq 2009',0);
INSERT INTO "usage" VALUES('EPSG','9211','helmert_transformation','EPSG','5077','EPSG','3625','EPSG','1033');
INSERT INTO "helmert_transformation" VALUES('EPSG','5078','Karbala 1979 to WGS 84 (2)','Parameter values from Karbala 1979 to IGRS (1) (tfm code 5077) assuming that IGRS is equivalent to WGS 84 within the accuracy of the transformation. Replaces Karbala 1979 to WGS 84 (1) (tfm code 15872).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4743','EPSG','4326',1.0,70.995,-335.916,262.898,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-Irq 2009',0);
INSERT INTO "usage" VALUES('EPSG','9212','helmert_transformation','EPSG','5078','EPSG','3625','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','5189','Korean 1985 to KGD2002 (1)','For accuracies commensurate with mapping at 1/5000 scale. May be taken as approximate transformation Korean 1985 to WGS 84 - see code 5191.','EPSG','9636','Molodensky-Badekas (CF geog2D domain)','EPSG','4162','EPSG','4737',1.0,-145.907,505.034,685.756,'EPSG','9001',-1.162,2.347,1.592,'EPSG','9104',6.342,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,-3159521.31,4068151.32,3748113.85,'EPSG','9001','NGII-Kor',0);
INSERT INTO "usage" VALUES('EPSG','9274','helmert_transformation','EPSG','5189','EPSG','3266','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','5191','Korean 1985 to WGS 84 (1)','Parameter values from Korean 1985 to KGD2002 (1) (code 5189). Assumes KGD2002 and WGS 84 can be considered the same to within the accuracy of the transformation.','EPSG','9636','Molodensky-Badekas (CF geog2D domain)','EPSG','4162','EPSG','4326',1.0,-145.907,505.034,685.756,'EPSG','9001',-1.162,2.347,1.592,'EPSG','9104',6.342,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,-3159521.31,4068151.32,3748113.85,'EPSG','9001','OGP-Kor',0);
INSERT INTO "usage" VALUES('EPSG','9276','helmert_transformation','EPSG','5191','EPSG','3266','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','5194','VN-2000 to WGS 84 (1)','Used by Total in Mekong delta.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4756','EPSG','4326',1.0,-192.873,-39.382,-111.202,'EPSG','9001',0.00205,0.0005,-0.00335,'EPSG','9104',0.0188,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'HCMCTU-Vnm',0);
INSERT INTO "usage" VALUES('EPSG','9278','helmert_transformation','EPSG','5194','EPSG','3770','EPSG','1065');
INSERT INTO "helmert_transformation" VALUES('EPSG','5226','S-JTSK/05 to ETRS89 (1)','Derived from the relationship between the R05 realisation of ETRS89 and the astrogeodetic S-JTSK network. Improves CT code 1622. S-JTSK/05 is a scientific working system, S-JTSK the legal system. May be taken as approx. CT S-JTSK to WGS 84 (code 5227).','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','5228','EPSG','4258',0.0,572.213,85.334,461.94,'EPSG','9001',-4.9732,-1.529,-5.2484,'EPSG','9104',3.5378,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'CUZK-Cze 05',0);
INSERT INTO "usage" VALUES('EPSG','9304','helmert_transformation','EPSG','5226','EPSG','1079','EPSG','1114');
INSERT INTO "helmert_transformation" VALUES('EPSG','5227','S-JTSK/05 to WGS 84 (1)','Parameter values from S-JTSK/05 to ETRS89 (1) (code 5226). Assumes ETRS89 and WGS 84 can be considered the same to within the accuracy of the transformation. Replaces tfm code 1622.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','5228','EPSG','4326',1.0,572.213,85.334,461.94,'EPSG','9001',-4.9732,-1.529,-5.2484,'EPSG','9104',3.5378,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-Cze 05',0);
INSERT INTO "usage" VALUES('EPSG','9305','helmert_transformation','EPSG','5227','EPSG','1079','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','5236','SLD99 to WGS 84 (1)','Derived at 58 stations.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','5233','EPSG','4326',14.0,-0.293,766.95,87.713,'EPSG','9001',-0.195704,-1.695068,-3.473016,'EPSG','9104',-0.039338,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'GSU-Lka',0);
INSERT INTO "usage" VALUES('EPSG','9309','helmert_transformation','EPSG','5236','EPSG','3310','EPSG','1157');
INSERT INTO "helmert_transformation" VALUES('EPSG','5239','S-JTSK to WGS 84 (5)','Parameter values from S-JTSK/05 to WGS 84 (1) (code 5227). Assumes ETRS89 and WGS 84 can be considered the same to within the accuracy of the transformation. Replaces tfm code 1622.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4156','EPSG','4326',1.0,572.213,85.334,461.94,'EPSG','9001',-4.9732,-1.529,-5.2484,'EPSG','9104',3.5378,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-Cze 05',0);
INSERT INTO "usage" VALUES('EPSG','9311','helmert_transformation','EPSG','5239','EPSG','1079','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','5248','Timbalai 1948 to GDBD2009 (1)','','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4298','EPSG','4326',1.0,-689.5937,623.84046,-65.93566,'EPSG','9001',0.02331,-1.17094,0.80054,'EPSG','9104',5.88536,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'SD-Brn',1);
INSERT INTO "usage" VALUES('EPSG','9315','helmert_transformation','EPSG','5248','EPSG','1055','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','5249','Timbalai 1948 to WGS 84 (5)','Parameter values taken from Timbalai 1948 to GDBD2009 (1) (code 5878) assuming that GDBD2009 is equivalent to WGS 84 within the accuracy of the transformation.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4298','EPSG','4326',1.0,-689.5937,623.84046,-65.93566,'EPSG','9001',0.02331,-1.17094,0.80054,'EPSG','9104',5.88536,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-Brn',0);
INSERT INTO "usage" VALUES('EPSG','9316','helmert_transformation','EPSG','5249','EPSG','1055','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','5260','TUREF to ETRS89 (1)','Note: the ETRS89 CRS is not used in Turkey.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','5252','EPSG','4258',0.1,0.023,0.036,-0.068,'EPSG','9001',0.00176,0.00912,-0.01136,'EPSG','9104',0.00439,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'GCM-Tur',0);
INSERT INTO "usage" VALUES('EPSG','9317','helmert_transformation','EPSG','5260','EPSG','1237','EPSG','1031');
INSERT INTO "helmert_transformation" VALUES('EPSG','5261','TUREF to WGS 84 (1)','Approximation at the +/- 1m level assuming that TUREF is equivalent to WGS 84.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','5252','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-Tur',0);
INSERT INTO "usage" VALUES('EPSG','9318','helmert_transformation','EPSG','5261','EPSG','1237','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','5267','DRUKREF 03 to WGS 84 (1)','DRUKREF 03 and WGS 84 are both realisations of ITRS.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','5264','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-Btn',0);
INSERT INTO "usage" VALUES('EPSG','9320','helmert_transformation','EPSG','5267','EPSG','1048','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','5327','ISN2004 to WGS 84 (1)','For many purposes ISN2004 can be considered to be coincident with WGS 84.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','5324','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-Isl',0);
INSERT INTO "usage" VALUES('EPSG','9344','helmert_transformation','EPSG','5327','EPSG','1120','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','5333','ITRF2005 to ITRF2008 (1)','At epoch 2005.0. Rates dX=-0.0003 m/yr, dy=dz=0.000 m/yr, rX=rY=rZ=0.0"/yr, dS=0.0000 ppm/yr.','EPSG','1033','Position Vector transformation (geocentric domain)','EPSG','4896','EPSG','5332',0.0,0.0005,0.0009,0.0047,'EPSG','9001',0.0,0.0,0.0,'EPSG','9104',-0.00094,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IERS',1);
INSERT INTO "usage" VALUES('EPSG','9346','helmert_transformation','EPSG','5333','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','5350','Campo Inchauspe to POSGAR 2007 (1)','Adopted from U.S. Defense Mapping Agency values for Campo Inchauspe to WGS 84 (tfm code 1127) assuming that POSGAR 2007 is equivalent to WGS 84.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4221','EPSG','5340',5.0,-148.0,136.0,90.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGM-Arg',0);
INSERT INTO "usage" VALUES('EPSG','9351','helmert_transformation','EPSG','5350','EPSG','3215','EPSG','1045');
INSERT INTO "helmert_transformation" VALUES('EPSG','5351','POSGAR 2007 to WGS 84 (1)','Approximation at the +/- 1m level assuming that POSGAR 2007 is equivalent to WGS 84.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','5340','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Arg',0);
INSERT INTO "usage" VALUES('EPSG','9352','helmert_transformation','EPSG','5351','EPSG','1033','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','5374','MARGEN to WGS 84 (1)','Approximation at the +/- 1m level assuming that MARGEN is equivalent to WGS 84.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','5354','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-Bol',0);
INSERT INTO "usage" VALUES('EPSG','9354','helmert_transformation','EPSG','5374','EPSG','1049','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','5375','SIRGAS-Chile to WGS 84 (1)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','9184','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-Chl',1);
INSERT INTO "usage" VALUES('EPSG','9355','helmert_transformation','EPSG','5375','EPSG','1066','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','5376','CR05 to WGS 84 (1)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','5365','EPSG','4326',1.5,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-Cri',0);
INSERT INTO "usage" VALUES('EPSG','9356','helmert_transformation','EPSG','5376','EPSG','1074','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','5377','MACARIO SOLIS to WGS 84 (1)','Approximation at the +/- 1m level assuming that MACARIO SOLIS is equivalent to WGS 84.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','5371','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-Pan',0);
INSERT INTO "usage" VALUES('EPSG','9357','helmert_transformation','EPSG','5377','EPSG','1186','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','5378','Peru96 to WGS 84 (1)','Approximation at the +/- 1m level assuming that Peru96 is equivalent to WGS 84.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','5373','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-Per',0);
INSERT INTO "usage" VALUES('EPSG','9358','helmert_transformation','EPSG','5378','EPSG','1189','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','5384','SIRGAS-ROU98 to WGS 84 (1)','Approximation at the +/- 1m level assuming that SIRGAS-ROU98 is equivalent to WGS 84.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','5381','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-Ury',0);
INSERT INTO "usage" VALUES('EPSG','9359','helmert_transformation','EPSG','5384','EPSG','1247','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','5385','Yacare to SIRGAS-ROU98 (1)','Derived at 11 stations during 1998 densification of Uruguay control based on SIRGAS 1995. Accuracy at stations used for derivation: 0.13 to 1.17m.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4309','EPSG','5381',1.5,-124.45,183.74,44.64,'EPSG','9001',-0.4384,0.5446,-0.9706,'EPSG','9104',-2.1365,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'SGM-Ury',0);
INSERT INTO "usage" VALUES('EPSG','9360','helmert_transformation','EPSG','5385','EPSG','3326','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','5386','Yacare to WGS 84 (2)','Derived at 11 stations during 1998 densification of Uruguay control based on SIRGAS 1995. Accuracy at stations used for derivation: 0.13 to 1.17m.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4309','EPSG','4326',1.5,-124.45,183.74,44.64,'EPSG','9001',-0.4384,0.5446,-0.9706,'EPSG','9104',-2.1365,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'SGM-Ury',0);
INSERT INTO "usage" VALUES('EPSG','9361','helmert_transformation','EPSG','5386','EPSG','3326','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','5395','SIRGAS_ES2007.8 to WGS 84 (1)','Approximation at the +/- 1m level assuming that SIRGAS_ES2007.8 is equivalent to WGS 84.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','5393','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-Slv',0);
INSERT INTO "usage" VALUES('EPSG','9364','helmert_transformation','EPSG','5395','EPSG','1087','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','5470','Ocotepeque 1935 to WGS 84 (1)','Parameter values taken from Ocotepeque to CR05 (1) (tfm code 6890) assuming that CR05 is equivalent to WGS 84 within the accuracy of the transformation.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','5451','EPSG','4326',8.0,213.11,9.37,-74.95,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Cri',0);
INSERT INTO "usage" VALUES('EPSG','9425','helmert_transformation','EPSG','5470','EPSG','3232','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','5473','Ocotepeque 1935 to WGS 84 (2)','Rotations in original source given in radians are equivalent to Rx = 2.35", Ry = -0.06", Rz = 6.39".','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','5451','EPSG','4326',5.0,213.116,9.358,-74.946,'EPSG','9001',1.14e-05,-2.98e-07,3.1e-05,'EPSG','9101',-5.22,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'UNA-Cri',1);
INSERT INTO "usage" VALUES('EPSG','9427','helmert_transformation','EPSG','5473','EPSG','3232','EPSG','1153');
INSERT INTO "helmert_transformation" VALUES('EPSG','5474','Ocotepeque 1935 to NAD27 (1)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','5451','EPSG','4326',9.0,205.435,-29.099,292.202,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Cri',1);
INSERT INTO "usage" VALUES('EPSG','9428','helmert_transformation','EPSG','5474','EPSG','3876','EPSG','1153');
INSERT INTO "helmert_transformation" VALUES('EPSG','5483','LUREF to ETRS89 (4)','LUREF 2006 parameters. For an equivalent CT using the Coordinate Frame method see CT 5485. Defines LUREF between 2006 and 2014. Replaces LUREF 1994 parameters (CT 1078). May be taken as approximate transformation LUREF to WGS 84 - see CT code 5484.','EPSG','9636','Molodensky-Badekas (CF geog2D domain)','EPSG','4181','EPSG','4258',0.0,-265.8867,76.9851,20.2667,'EPSG','9001',0.33746,3.09264,-2.53861,'EPSG','9104',0.4598,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,4103620.3943,440486.4235,4846923.4558,'EPSG','9001','ACT-Lux MB 2011',0);
INSERT INTO "usage" VALUES('EPSG','9433','helmert_transformation','EPSG','5483','EPSG','1146','EPSG','1281');
INSERT INTO "helmert_transformation" VALUES('EPSG','5484','LUREF to WGS 84 (4)','Parameter values from LUREF to ETRS89 (4) (code 5483) assuming ETRS89 and WGS 84 are coincident within the one metre level. Replaces CT code 1079. For an equivalent transformation using the Coordinate Frame method see code 5486.','EPSG','9636','Molodensky-Badekas (CF geog2D domain)','EPSG','4181','EPSG','4326',1.0,-265.8867,76.9851,20.2667,'EPSG','9001',0.33746,3.09264,-2.53861,'EPSG','9104',0.4598,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,4103620.3943,440486.4235,4846923.4558,'EPSG','9001','OGP-Lux MB 2011',0);
INSERT INTO "usage" VALUES('EPSG','9434','helmert_transformation','EPSG','5484','EPSG','1146','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','5485','LUREF to ETRS89 (3)','LUREF 2006 parameters. For an equivalent CT using the Molodensky-Badekas method see code 5483. Defines LUREF between 2006 and 2014. Replaces LUREF to ETRS89 (1) (CT 1642). May be taken as approximate transformation LUREF to WGS 84 - see CT code 5486.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4181','EPSG','4258',0.0,-189.6806,18.3463,-42.7695,'EPSG','9001',0.33746,3.09264,-2.53861,'EPSG','9104',0.4598,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'ACT-Lux CF 2011',0);
INSERT INTO "usage" VALUES('EPSG','9435','helmert_transformation','EPSG','5485','EPSG','1146','EPSG','1281');
INSERT INTO "helmert_transformation" VALUES('EPSG','5486','LUREF to WGS 84 (3)','Parameter values from LUREF to ETRS89 (3) (code 5485) assuming ETRS89 and WGS 84 are coincident within the one metre level. Replaces CT code 1643. For an equivalent transformation using the Molodensky-Badekas method see code 5484.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4181','EPSG','4326',1.0,-189.6806,18.3463,-42.7695,'EPSG','9001',0.33746,3.09264,-2.53861,'EPSG','9104',0.4598,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-Lux CF 2011',0);
INSERT INTO "usage" VALUES('EPSG','9436','helmert_transformation','EPSG','5486','EPSG','1146','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','5491','Martinique 1938 to RGAF09 (1)','','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4625','EPSG','5489',0.1,127.744,547.069,118.359,'EPSG','9001',-3.1116,4.9509,-0.8837,'EPSG','9104',14.1012,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Mtq',0);
INSERT INTO "usage" VALUES('EPSG','9437','helmert_transformation','EPSG','5491','EPSG','3276','EPSG','1031');
INSERT INTO "helmert_transformation" VALUES('EPSG','5492','Guadeloupe 1948 to RGAF09 (1)','','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4622','EPSG','5489',10.0,-471.06,-3.212,-305.843,'EPSG','9001',0.4752,-0.9978,0.2068,'EPSG','9104',2.1353,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Glp',0);
INSERT INTO "usage" VALUES('EPSG','9438','helmert_transformation','EPSG','5492','EPSG','2829','EPSG','1050');
INSERT INTO "helmert_transformation" VALUES('EPSG','5493','Fort Marigot to RGAF09 (1)','','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4621','EPSG','5489',10.0,151.613,253.832,-429.084,'EPSG','9001',-0.0506,0.0958,-0.5974,'EPSG','9104',-0.3971,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Glp',0);
INSERT INTO "usage" VALUES('EPSG','9439','helmert_transformation','EPSG','5493','EPSG','2828','EPSG','1050');
INSERT INTO "helmert_transformation" VALUES('EPSG','5494','RRAF 1991 to RGAF09 (1)','','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4558','EPSG','5489',0.1,0.7696,-0.8692,-12.0631,'EPSG','9001',-0.32511,-0.21041,-0.0239,'EPSG','9104',0.2829,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Glp',0);
INSERT INTO "usage" VALUES('EPSG','9440','helmert_transformation','EPSG','5494','EPSG','1156','EPSG','1031');
INSERT INTO "helmert_transformation" VALUES('EPSG','5495','RRAF 1991 to RGAF09 (2)','','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4558','EPSG','5489',0.1,1.2239,2.4156,-1.7598,'EPSG','9001',0.038,-0.16101,-0.04925,'EPSG','9104',0.2387,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Glp GT',0);
INSERT INTO "usage" VALUES('EPSG','9441','helmert_transformation','EPSG','5495','EPSG','2829','EPSG','1031');
INSERT INTO "helmert_transformation" VALUES('EPSG','5496','RRAF 1991 to RGAF09 (3)','','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4558','EPSG','5489',0.1,14.6642,5.2493,0.1981,'EPSG','9001',-0.06838,0.09141,-0.58131,'EPSG','9104',-0.4067,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Glp SM',0);
INSERT INTO "usage" VALUES('EPSG','9442','helmert_transformation','EPSG','5496','EPSG','2828','EPSG','1031');
INSERT INTO "helmert_transformation" VALUES('EPSG','5497','POSGAR 2007 to SIRGAS 2000 (1)','Agreement at the decimeter level.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','5340','EPSG','4674',0.1,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGM-Arg',0);
INSERT INTO "usage" VALUES('EPSG','9443','helmert_transformation','EPSG','5497','EPSG','1033','EPSG','1031');
INSERT INTO "helmert_transformation" VALUES('EPSG','5501','RGAF09 to WGS 84 (1)','Approximation at the +/- 1m level assuming that RGAF09 is equivalent to WGS 84 within the accuracy of the transformation.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','5489','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-FrAnt',0);
INSERT INTO "usage" VALUES('EPSG','9444','helmert_transformation','EPSG','5501','EPSG','2824','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','5521','Grand Comoros to WGS 84 (1)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4646','EPSG','4326',999.0,-963.0,510.0,-359.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'SHOM-Com',0);
INSERT INTO "usage" VALUES('EPSG','9457','helmert_transformation','EPSG','5521','EPSG','2807','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','5553','PNG94 to WGS 84 (1)','Exact in 1994 but due to significant and variable tectonic activity in PNG, in 2011 PNG94 and WGS 84 differ generally by 2m but in areas of significant tectonic activity differences can exceed 9m.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','5546','EPSG','4326',2.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-PNG',0);
INSERT INTO "usage" VALUES('EPSG','9470','helmert_transformation','EPSG','5553','EPSG','1187','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','5584','MOLDREF99 to ETRS89 (1)','MOLDREF is a densification of ETRS89 in Moldova.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4023','EPSG','4258',0.1,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-Mda',0);
INSERT INTO "usage" VALUES('EPSG','9472','helmert_transformation','EPSG','5584','EPSG','1162','EPSG','1161');
INSERT INTO "helmert_transformation" VALUES('EPSG','5585','MOLDREF99 to WGS 84 (1)','Parameter values from MOLDREF99 to ETRS89 (1) (code 5584). Assumes ETRS89 and WGS 84 can be considered the same to within the accuracy of the transformation.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4023','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-Mda',0);
INSERT INTO "usage" VALUES('EPSG','9473','helmert_transformation','EPSG','5585','EPSG','1162','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','5586','Pulkovo 1942 to UCS-2000 (1)','UCS-2000 is defined to be approximately consistent with Pulkovo 1942 and this transformation''s accuracy is due to deformation of the Pulkovo system across Ukrainian territory.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4284','EPSG','5561',3.5,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-Ukr',0);
INSERT INTO "usage" VALUES('EPSG','9474','helmert_transformation','EPSG','5586','EPSG','1242','EPSG','1044');
INSERT INTO "helmert_transformation" VALUES('EPSG','5590','UCS-2000 to WGS 84 (1)','Derived through concatenation of UCS-2000 to S-42 (1) (tfm code 5586 reversed) [an approximation] and S-42 to WGS 84 (16) (tfm code 15865) [derived for whole FSU rather than Ukraine]. Replaced by UCS-2000 to WGS 84 (2) (tfm code 5840).','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','5561','EPSG','4326',5.0,25.0,-141.0,-78.5,'EPSG','9001',0.0,-0.35,-0.736,'EPSG','9104',0.0,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-Ukr',0);
INSERT INTO "usage" VALUES('EPSG','9476','helmert_transformation','EPSG','5590','EPSG','1242','EPSG','1045');
INSERT INTO "helmert_transformation" VALUES('EPSG','5599','FEH2010 to WGS 84 (1)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','5593','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-Dnk-Deu Feh',0);
INSERT INTO "usage" VALUES('EPSG','9479','helmert_transformation','EPSG','5599','EPSG','3889','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','5622','OSGB36 to WGS 84 (8)','Derived by CGG for 1994 3D seismic survey.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4277','EPSG','4326',3.0,370.936,-108.938,435.682,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'bp-Gbr WytchF',0);
INSERT INTO "usage" VALUES('EPSG','9480','helmert_transformation','EPSG','5622','EPSG','3893','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','5630','Nord Sahara 1959 to WGS 84 (8)','Derived at 1 station (L38).  May be taken as a transformation to RGSH2020 - see CT 10343.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4307','EPSG','4326',5.0,-168.52,-72.05,304.3,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'Tot-Dza Ahnet',0);
INSERT INTO "usage" VALUES('EPSG','9482','helmert_transformation','EPSG','5630','EPSG','3917','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','5660','Nord Sahara 1959 to WGS 84 (9)','Derived in 2006 at 45 points in north and central Algeria. Accuracy at 75 common points better than 1m.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4307','EPSG','4326',1.0,-209.3622,-87.8162,404.6198,'EPSG','9001',0.0046,3.4784,0.5805,'EPSG','9104',-1.4547,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'INCT-Dza',0);
INSERT INTO "usage" VALUES('EPSG','9491','helmert_transformation','EPSG','5660','EPSG','1026','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','5662','AGD66 to PNG94 (1)','Derived at 25 stations in 2007. Accuracy 2m in 2007. Replaced by AGD66 to PNG94 (4) (code 6939).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4202','EPSG','5546',2.0,-124.0,-60.0,153.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'Qcl-Png PFTB 2007',0);
INSERT INTO "usage" VALUES('EPSG','9493','helmert_transformation','EPSG','5662','EPSG','4013','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','5822','UCS-2000 to ITRF2005 (1)','May be taken as approximate transformation UCS-2000 to WGS 84 - see code 5840.','EPSG','1031','Geocentric translations (geocentric domain)','EPSG','5558','EPSG','4896',1.0,24.0,-121.0,-76.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'SSGC-Ukr',1);
INSERT INTO "usage" VALUES('EPSG','9506','helmert_transformation','EPSG','5822','EPSG','1242','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','5823','Ukraine 2000 to WGS 84 (1)','Parameter values taken from Ukraine 2000 to ITRF2005 (1) (code 5822) assuming that ITRS2005 is equivalent to WGS 84 within the accuracy of the transformation.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','5561','EPSG','4326',1.0,24.0,-121.0,-76.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-Ukr',1);
INSERT INTO "usage" VALUES('EPSG','9507','helmert_transformation','EPSG','5823','EPSG','1242','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','5826','DB_REF to ETRS89 (1)','Defines DB_REF so considered errorless. Defined with parameter values for forward and reverse CTs with rotation and scale to greater resolution. The OGP truncated values and reversible formula have no impact on calculation results at the micron level.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','5681','EPSG','4258',0.0,584.9636,107.7175,413.8067,'EPSG','9001',-1.1155,-0.2824,3.1384,'EPSG','9104',7.9922,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DB-Deu',0);
INSERT INTO "usage" VALUES('EPSG','9509','helmert_transformation','EPSG','5826','EPSG','3339','EPSG','1141');
INSERT INTO "helmert_transformation" VALUES('EPSG','5827','AGD66 to GDA94 (19)','Replaces nationally-derived transformation code 1458.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4202','EPSG','4283',0.5,-129.164,-41.188,130.718,'EPSG','9001',-0.246,-0.374,-0.329,'EPSG','9104',-2.955,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'PLA-ACT',0);
INSERT INTO "usage" VALUES('EPSG','9510','helmert_transformation','EPSG','5827','EPSG','2283','EPSG','1158');
INSERT INTO "helmert_transformation" VALUES('EPSG','5840','UCS-2000 to WGS 84 (2)','Rounded parameter values taken from UCS-2000 to ITRF2000 (1) (code 7817) assuming that WGS 84 is equivalent to ITRF2000 within the accuracy of the transformation. Replaces UCS-2000 to WGS 84 (1) (tfm code 5590).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','5561','EPSG','4326',1.0,24.0,-121.0,-76.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-Ukr SSGC',0);
INSERT INTO "usage" VALUES('EPSG','9512','helmert_transformation','EPSG','5840','EPSG','1242','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','5841','AGD66 to WGS 84 (19)','Derived at 25 stations in 2007. Accuracy 2m in 2007. Due to significant tectonic activity in PNG, AGD66 and WGS 84 are separating by approximately 7cm per year.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4202','EPSG','4326',2.0,-124.0,-60.0,154.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'Qcl-Png PFTB 2007',0);
INSERT INTO "usage" VALUES('EPSG','9513','helmert_transformation','EPSG','5841','EPSG','4013','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','5878','Timbalai 1948 to GDBD2009 (1)','May be taken as approximate transformation Timbalai 1948 to WGS 84 - see code 5249.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4298','EPSG','5246',1.0,-689.5937,623.84046,-65.93566,'EPSG','9001',0.02331,-1.17094,0.80054,'EPSG','9104',5.88536,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'SD-Brn',0);
INSERT INTO "usage" VALUES('EPSG','9515','helmert_transformation','EPSG','5878','EPSG','1055','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','5881','SAD69(96) to SIRGAS 2000 (2)','Parameter values from SAD69 to SIRGAS 2000 (1) (tfm code 15485) assuming that SAD69 and SAD69(96) are equal within the accuracy of the transformation. Used by Petrobras and ANP throughout Brazil from 1994.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','5527','EPSG','4674',5.0,-67.35,3.88,-38.22,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IBGE-Bra 5m 1994',0);
INSERT INTO "usage" VALUES('EPSG','9516','helmert_transformation','EPSG','5881','EPSG','1053','EPSG','1257');
INSERT INTO "helmert_transformation" VALUES('EPSG','5882','SAD69 to WGS 84 (16)','Parameter values taken from SAD69 to SIRGAS 2000 (1) (tfm code 15485) assuming that SIRGAS 2000 and WGS 84 are equal within the accuracy of the transformation. Used by ANP and Petrobras throughout Brazil from 1994, replacing use of tfm code 1877.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4618','EPSG','4326',5.0,-67.35,3.88,-38.22,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-Bra 5m 1994',0);
INSERT INTO "usage" VALUES('EPSG','9517','helmert_transformation','EPSG','5882','EPSG','1053','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','5888','Combani 1950 to RGM04 (1)','','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4632','EPSG','4470',0.3,-599.928,-275.552,-195.665,'EPSG','9001',-0.0835,-0.4715,0.0602,'EPSG','9104',49.2814,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Myt',0);
INSERT INTO "usage" VALUES('EPSG','9519','helmert_transformation','EPSG','5888','EPSG','3340','EPSG','1033');
INSERT INTO "helmert_transformation" VALUES('EPSG','5900','ITRF2005 to ETRF2005 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4896','EPSG','8397',0.0,56.0,48.0,-37.0,'EPSG','1025',0.0,0.0,0.0,'EPSG','1031',0.0,'EPSG','1028',0.0,0.0,0.0,'EPSG','1027',0.054,0.518,-0.781,'EPSG','1032',0.0,'EPSG','1030',1989.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'EUREF-Eur',0);
INSERT INTO "usage" VALUES('EPSG','9528','helmert_transformation','EPSG','5900','EPSG','1298','EPSG','1128');
INSERT INTO "helmert_transformation" VALUES('EPSG','6136','GCGD59 to CIGD11 (1)','May be taken as approximate transformation GCGD61 to WGS 84 - see code 6142.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4723','EPSG','6135',0.3,-179.483,-69.379,-27.584,'EPSG','9001',7.862,-8.163,-6.042,'EPSG','9104',-13.925,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'LSD-Cym',0);
INSERT INTO "usage" VALUES('EPSG','9626','helmert_transformation','EPSG','6136','EPSG','3185','EPSG','1033');
INSERT INTO "helmert_transformation" VALUES('EPSG','6137','SIGD61 to CIGD11 (1)','May be taken as approximate transformation SIGD61 to WGS 84 - see code 6143.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4726','EPSG','6135',0.15,8.853,-52.644,180.304,'EPSG','9001',0.393,2.323,-2.96,'EPSG','9104',-24.081,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'LSD-Cym',0);
INSERT INTO "usage" VALUES('EPSG','9627','helmert_transformation','EPSG','6137','EPSG','3186','EPSG','1078');
INSERT INTO "helmert_transformation" VALUES('EPSG','6142','GCGD59 to WGS 84 (2)','Parameter values are taken from GCGD59 to CIGD11 (1) (code 6136) assuming that CIGD11 and WGS 84 can be considered the same to within the accuracy of the transformation.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4723','EPSG','4326',1.0,-179.483,-69.379,-27.584,'EPSG','9001',7.862,-8.163,-6.042,'EPSG','9104',-13.925,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-Cym',0);
INSERT INTO "usage" VALUES('EPSG','9631','helmert_transformation','EPSG','6142','EPSG','3185','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','6143','SIGD61 to WGS 84 (3)','Parameter values are taken from SIGD59 to CIGD11 (1) (code 6137) assuming that CIGD11 and WGS 84 can be considered the same to within the accuracy of the transformation.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4726','EPSG','4326',1.0,8.853,-52.644,180.304,'EPSG','9001',0.393,2.323,-2.96,'EPSG','9104',-24.081,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-Cym',0);
INSERT INTO "usage" VALUES('EPSG','9632','helmert_transformation','EPSG','6143','EPSG','3186','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','6177','CIGD11 to WGS 84 (1)','Approximation at the +/- 1m level assuming that CIGD11 is equivalent to WGS 84.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','6135','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-Cym',0);
INSERT INTO "usage" VALUES('EPSG','9633','helmert_transformation','EPSG','6177','EPSG','1063','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','6191','Corrego Alegre 1970-72 to SAD69 (1)','Derived by Brazilian Institute of Geography and Statistics (IBGE) in 1983 at Chua origin point.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4225','EPSG','4618',5.0,-138.7,164.4,34.4,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGBE-Bra',0);
INSERT INTO "usage" VALUES('EPSG','9636','helmert_transformation','EPSG','6191','EPSG','1293','EPSG','1210');
INSERT INTO "helmert_transformation" VALUES('EPSG','6192','Corrego Alegre 1970-72 to WGS 84 (3)','Formed by concatenation of tfms codes 6191 and 1877. Used by Petrobras and ANP until February 2005 when replaced by Corrego Alegre 1970-72 to WGS 84 (4) (tfm code 6194).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4225','EPSG','4326',5.0,-205.57,168.77,-4.12,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'PBS-Bra 1983',0);
INSERT INTO "usage" VALUES('EPSG','9637','helmert_transformation','EPSG','6192','EPSG','1293','EPSG','1210');
INSERT INTO "helmert_transformation" VALUES('EPSG','6193','Corrego Alegre 1970-72 to SIRGAS 2000 (2)','Formed by concatenation of tfms codes 6191 and 15485. May be used as transformation between Corrego Alegre 1970-72 and WGS 84 - see tfm code 6194. Used by Petrobras and ANP from February 2005.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4225','EPSG','4674',5.0,-206.05,168.28,-3.82,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'PBS-Bra 2005',0);
INSERT INTO "usage" VALUES('EPSG','9638','helmert_transformation','EPSG','6193','EPSG','1293','EPSG','1210');
INSERT INTO "helmert_transformation" VALUES('EPSG','6194','Corrego Alegre 1970-72 to WGS 84 (4)','Parameter values from Corrego Alegre to SIRGAS 2000 (2) (tfm code 6193) assuming that SIRGAS 2000 and WGS 84 are equal within the accuracy of the transformation. Used by ANP and Petrobras from February 2005, replacing use of tfm code 6192.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4225','EPSG','4326',5.0,-206.05,168.28,-3.82,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'PBS-Bra 2005',0);
INSERT INTO "usage" VALUES('EPSG','9639','helmert_transformation','EPSG','6194','EPSG','1293','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','6195','SAD69(96) to WGS 84 (2)','Parameter values from SAD69(96) to SIRGAS 2000 (2)) (tfm code 5881) assuming that SIRGAS 2000 and WGS 84 are equal within the accuracy of the transformation, based on SAD69 to SIRGAS 2000 (1)) (tfm code 15485). Used by Petrobras and ANP from 1994.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','5527','EPSG','4326',5.0,-67.35,3.88,-38.22,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-Bra 5m 1994',0);
INSERT INTO "usage" VALUES('EPSG','9640','helmert_transformation','EPSG','6195','EPSG','1053','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','6196','Minna to WGS 84 (16)','Used by Addax for OPL 118 and OML 124. Derived in 1999 at 4 stations during extension into OPL 118 of control in Chevron block OML 53.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4263','EPSG','4326',5.0,-93.179,-87.124,114.338,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'ADX-Nga OPL 118',0);
INSERT INTO "usage" VALUES('EPSG','9641','helmert_transformation','EPSG','6196','EPSG','4127','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','6205','MGI 1901 to ETRS89 (5)','Derived at 31 stations in July 2010. Residuals generally less than +/- 0.7m horizontally.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','3906','EPSG','4258',1.0,517.4399,228.7318,579.7954,'EPSG','9001',-4.045,-4.304,15.612,'EPSG','9104',-8.312,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'KAT-Mkd',0);
INSERT INTO "usage" VALUES('EPSG','9646','helmert_transformation','EPSG','6205','EPSG','1148','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','6206','MGI 1901 to WGS 84 (10)','Derived at 13 stations. Residuals generally less than +/- 1m horizontally and vertically.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','3906','EPSG','4326',2.0,521.748,229.489,590.921,'EPSG','9001',-4.029,-4.488,15.521,'EPSG','9104',-9.78,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'Kat-Mkd',0);
INSERT INTO "usage" VALUES('EPSG','9647','helmert_transformation','EPSG','6206','EPSG','1148','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','6208','Nepal 1981 to WGS 84 (1)','Derived at 11 points. Accuracy 0.26m (1-sigma).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','6207','EPSG','4326',0.3,293.17,726.18,245.36,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'SD-Npl',0);
INSERT INTO "usage" VALUES('EPSG','9648','helmert_transformation','EPSG','6208','EPSG','1171','EPSG','1153');
INSERT INTO "helmert_transformation" VALUES('EPSG','6276','ITRF2008 to GDA94 (1)','RMS residuals 5mm north, 8mm east and 28mm vertical, maximum residuals 10mm north, 13mm east and 51mm vertical. Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1056','Time-dependent Coordinate Frame rotation (geocen)','EPSG','5332','EPSG','4938',0.03,-84.68,-19.42,32.01,'EPSG','1025',-0.4254,2.2578,2.4015,'EPSG','1031',9.71,'EPSG','1028',1.42,1.34,0.9,'EPSG','1027',1.5461,1.182,1.1551,'EPSG','1032',0.109,'EPSG','1030',1994.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'GA-Aus 2010',0);
INSERT INTO "usage" VALUES('EPSG','9682','helmert_transformation','EPSG','6276','EPSG','1036','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','6277','ITRF2005 to GDA94 (1)','RMS residuals 4mm north, 8mm east and 30mm vertical, maximum residuals 10mm north, 17 mm east and 61mm vertical. Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1056','Time-dependent Coordinate Frame rotation (geocen)','EPSG','4896','EPSG','4938',0.03,-79.73,-6.86,38.03,'EPSG','1025',-0.0351,2.1211,2.1411,'EPSG','1031',6.636,'EPSG','1028',2.25,-0.62,-0.56,'EPSG','1027',1.4707,1.1443,1.1701,'EPSG','1032',0.294,'EPSG','1030',1994.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'GA-Aus 2010',0);
INSERT INTO "usage" VALUES('EPSG','9683','helmert_transformation','EPSG','6277','EPSG','1036','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','6278','ITRF2000 to GDA94 (2)','RMS residuals 3mm N, 8mm E and 55mm up, maximum residuals 5mm N, 13mm E and 84mm up. Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m. Replaces 2001 transformation by Dawson and Steed, tfm code 6315.','EPSG','1056','Time-dependent Coordinate Frame rotation (geocen)','EPSG','4919','EPSG','4938',0.06,-45.91,-29.85,-20.37,'EPSG','1025',-1.6705,0.4594,1.9356,'EPSG','1031',7.07,'EPSG','1028',-4.66,3.55,11.24,'EPSG','1027',1.7454,1.4868,1.224,'EPSG','1032',0.249,'EPSG','1030',1994.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'GA-Aus 2010',0);
INSERT INTO "usage" VALUES('EPSG','9684','helmert_transformation','EPSG','6278','EPSG','1036','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','6279','ITRF97 to GDA94 (2)','RMS residuals 26mm N, 12mm E and 179mm up, maximum residuals 49mm N, 24mm E and 464mm up. Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m. Replaces 2001 transformation by Dawson and Steed, tfm code 6392.','EPSG','1056','Time-dependent Coordinate Frame rotation (geocen)','EPSG','4918','EPSG','4938',0.18,-14.63,-27.62,-25.32,'EPSG','1025',-1.7893,-0.6047,0.9962,'EPSG','1031',6.695,'EPSG','1028',-8.6,0.36,11.25,'EPSG','1027',1.6394,1.5198,1.3801,'EPSG','1032',0.007,'EPSG','1030',1994.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'GA-Aus 2010',0);
INSERT INTO "usage" VALUES('EPSG','9685','helmert_transformation','EPSG','6279','EPSG','1036','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','6280','ITRF96 to GDA94 (2)','RMS residuals 22mm N, 56mm E and 90mm up, maximum residuals 49mm N, 126mm E and 193mm up. Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m. Replaces 2001 transformation by Dawson and Steed, code 6313.','EPSG','1056','Time-dependent Coordinate Frame rotation (geocen)','EPSG','4917','EPSG','4938',0.11,24.54,-36.43,-68.12,'EPSG','1025',-2.7359,-2.0431,0.3731,'EPSG','1031',6.901,'EPSG','1028',-21.8,4.71,26.27,'EPSG','1027',2.0203,2.1735,1.629,'EPSG','1032',0.388,'EPSG','1030',1994.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'GA-Aus 2010',0);
INSERT INTO "usage" VALUES('EPSG','9686','helmert_transformation','EPSG','6280','EPSG','1036','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','6281','ITRF88 to ITRF2000 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4910','EPSG','4919',0.01,-2.47,-1.15,9.79,'EPSG','1033',-0.1,0.0,0.18,'EPSG','1031',-8.95,'EPSG','1028',0.0,0.06,0.14,'EPSG','1034',0.0,0.0,-0.02,'EPSG','1032',-0.01,'EPSG','1030',1988.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IERS-Wld',0);
INSERT INTO "usage" VALUES('EPSG','9687','helmert_transformation','EPSG','6281','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','6282','ITRF89 to ITRF2000 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4911','EPSG','4919',0.01,-2.97,-4.75,7.39,'EPSG','1033',0.0,0.0,0.18,'EPSG','1031',-5.85,'EPSG','1028',0.0,0.6,-3.2,'EPSG','1034',0.0,0.0,-0.02,'EPSG','1032',-0.01,'EPSG','1030',1988.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IERS-Wld',1);
INSERT INTO "usage" VALUES('EPSG','9688','helmert_transformation','EPSG','6282','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','6283','ITRF90 to ITRF2000 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4912','EPSG','4919',0.01,-2.47,-2.35,3.59,'EPSG','1033',0.0,0.0,0.18,'EPSG','1031',-2.45,'EPSG','1028',0.0,0.06,0.14,'EPSG','1034',0.0,0.0,-0.02,'EPSG','1032',-0.01,'EPSG','1030',1988.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IERS-Wld',0);
INSERT INTO "usage" VALUES('EPSG','9689','helmert_transformation','EPSG','6283','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','6284','ITRF91 to ITRF2000 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4913','EPSG','4919',0.01,-2.67,-2.75,1.99,'EPSG','1033',0.0,0.0,0.18,'EPSG','1031',-2.15,'EPSG','1028',0.0,0.06,0.14,'EPSG','1034',0.0,0.0,-0.02,'EPSG','1032',-0.01,'EPSG','1030',1988.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IERS-Wld',0);
INSERT INTO "usage" VALUES('EPSG','9690','helmert_transformation','EPSG','6284','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','6285','ITRF92 to ITRF2000 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4914','EPSG','4919',0.01,-1.47,-1.35,1.39,'EPSG','1033',0.0,0.0,0.18,'EPSG','1031',-0.75,'EPSG','1028',0.0,0.06,0.14,'EPSG','1034',0.0,0.0,-0.02,'EPSG','1032',-0.01,'EPSG','1030',1988.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IERS-Wld',0);
INSERT INTO "usage" VALUES('EPSG','9691','helmert_transformation','EPSG','6285','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','6286','ITRF93 to ITRF2000 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4915','EPSG','4919',0.01,-1.27,-0.65,2.09,'EPSG','1033',0.39,-0.8,1.14,'EPSG','1031',-1.95,'EPSG','1028',0.29,0.02,0.06,'EPSG','1034',0.11,0.19,-0.07,'EPSG','1032',-0.01,'EPSG','1030',1988.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IERS-Wld',0);
INSERT INTO "usage" VALUES('EPSG','9692','helmert_transformation','EPSG','6286','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','6287','ITRF94 to ITRF2000 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4916','EPSG','4919',0.01,-0.67,-0.61,1.85,'EPSG','1033',0.0,0.0,0.0,'EPSG','1031',-1.55,'EPSG','1028',0.0,0.06,0.14,'EPSG','1034',0.0,0.0,-0.02,'EPSG','1032',-0.01,'EPSG','1030',1997.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IERS-Wld',0);
INSERT INTO "usage" VALUES('EPSG','9693','helmert_transformation','EPSG','6287','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','6288','ITRF96 to ITRF2000 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4917','EPSG','4919',0.01,-0.67,-0.61,1.85,'EPSG','1033',0.0,0.0,0.0,'EPSG','1031',-1.55,'EPSG','1028',0.0,0.06,0.14,'EPSG','1034',0.0,0.0,-0.02,'EPSG','1032',-0.01,'EPSG','1030',1997.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IERS-Wld',0);
INSERT INTO "usage" VALUES('EPSG','9694','helmert_transformation','EPSG','6288','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','6289','ITRF97 to ITRF2000 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4918','EPSG','4919',0.01,-0.67,-0.61,1.85,'EPSG','1033',0.0,0.0,0.0,'EPSG','1031',-1.55,'EPSG','1028',0.0,0.06,0.14,'EPSG','1034',0.0,0.0,-0.02,'EPSG','1032',-0.01,'EPSG','1030',1997.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IERS-Wld',0);
INSERT INTO "usage" VALUES('EPSG','9695','helmert_transformation','EPSG','6289','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','6290','ITRF2000 to ITRF2005 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4919','EPSG','4896',0.01,-0.1,0.8,5.8,'EPSG','1025',0.0,0.0,0.0,'EPSG','1031',-0.4,'EPSG','1028',0.2,-0.1,0.18,'EPSG','1027',0.0,0.0,-0.02,'EPSG','1032',-0.08,'EPSG','1030',2000.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IERS-Wld',1);
INSERT INTO "usage" VALUES('EPSG','9696','helmert_transformation','EPSG','6290','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','6291','ITRF88 to ITRF2008 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4910','EPSG','5332',0.01,-22.8,-2.6,125.2,'EPSG','1025',-0.1,0.0,-0.06,'EPSG','1031',-10.41,'EPSG','1028',-0.1,0.5,3.2,'EPSG','1027',0.0,0.0,-0.02,'EPSG','1032',-0.09,'EPSG','1030',2000.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IERS-Wld',0);
INSERT INTO "usage" VALUES('EPSG','9697','helmert_transformation','EPSG','6291','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','6292','ITRF89 to ITRF2008 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4911','EPSG','5332',0.01,-27.8,-38.6,101.2,'EPSG','1025',0.0,0.0,-0.06,'EPSG','1031',-7.31,'EPSG','1028',-0.1,0.5,3.2,'EPSG','1027',0.0,0.0,-0.02,'EPSG','1032',-0.09,'EPSG','1030',2000.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IERS-Wld',0);
INSERT INTO "usage" VALUES('EPSG','9698','helmert_transformation','EPSG','6292','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','6293','ITRF90 to ITRF2008 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4912','EPSG','5332',0.01,-22.8,-14.6,63.2,'EPSG','1025',0.0,0.0,-0.06,'EPSG','1031',-3.91,'EPSG','1028',-0.1,0.5,3.2,'EPSG','1027',0.0,0.0,-0.02,'EPSG','1032',-0.09,'EPSG','1030',2000.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IERS-Wld',0);
INSERT INTO "usage" VALUES('EPSG','9699','helmert_transformation','EPSG','6293','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','6294','ITRF91 to ITRF2008 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4913','EPSG','5332',0.01,-24.8,-18.6,47.2,'EPSG','1025',0.0,0.0,-0.06,'EPSG','1031',-3.61,'EPSG','1028',-0.1,0.5,3.2,'EPSG','1027',0.0,0.0,-0.02,'EPSG','1032',-0.09,'EPSG','1030',2000.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IERS-Wld',0);
INSERT INTO "usage" VALUES('EPSG','9700','helmert_transformation','EPSG','6294','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','6295','ITRF92 to ITRF2008 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4914','EPSG','5332',0.01,-12.8,-4.6,41.2,'EPSG','1025',0.0,0.0,-0.06,'EPSG','1031',-2.21,'EPSG','1028',-0.1,0.5,3.2,'EPSG','1027',0.0,0.0,-0.02,'EPSG','1032',-0.09,'EPSG','1030',2000.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IERS-Wld',0);
INSERT INTO "usage" VALUES('EPSG','9701','helmert_transformation','EPSG','6295','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','6296','ITRF93 to ITRF2008 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4915','EPSG','5332',0.01,24.0,-2.4,38.6,'EPSG','1025',1.71,1.48,0.3,'EPSG','1031',-3.41,'EPSG','1028',2.8,0.1,2.4,'EPSG','1027',0.11,0.19,-0.07,'EPSG','1032',-0.09,'EPSG','1030',2000.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IERS-Wld',0);
INSERT INTO "usage" VALUES('EPSG','9702','helmert_transformation','EPSG','6296','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','6297','ITRF94 to ITRF2008 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4916','EPSG','5332',0.01,-4.8,-2.6,33.2,'EPSG','1025',0.0,0.0,-0.06,'EPSG','1031',-2.92,'EPSG','1028',-0.1,0.5,3.2,'EPSG','1027',0.0,0.0,-0.02,'EPSG','1032',-0.09,'EPSG','1030',2000.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IERS-Wld',0);
INSERT INTO "usage" VALUES('EPSG','9703','helmert_transformation','EPSG','6297','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','6298','ITRF96 to ITRF2008 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4917','EPSG','5332',0.01,-4.8,-2.6,33.2,'EPSG','1025',0.0,0.0,-0.06,'EPSG','1031',-2.92,'EPSG','1028',-0.1,0.5,3.2,'EPSG','1027',0.0,0.0,-0.02,'EPSG','1032',-0.09,'EPSG','1030',2000.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IERS-Wld',0);
INSERT INTO "usage" VALUES('EPSG','9704','helmert_transformation','EPSG','6298','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','6299','ITRF97 to ITRF2008 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4918','EPSG','5332',0.01,-4.8,-2.6,33.2,'EPSG','1025',0.0,0.0,-0.06,'EPSG','1031',-2.92,'EPSG','1028',-0.1,0.5,3.2,'EPSG','1027',0.0,0.0,-0.02,'EPSG','1032',-0.09,'EPSG','1030',2000.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IERS-Wld',0);
INSERT INTO "usage" VALUES('EPSG','9705','helmert_transformation','EPSG','6299','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','6300','ITRF2000 to ITRF2008 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4919','EPSG','5332',0.01,1.9,1.7,10.5,'EPSG','1025',0.0,0.0,0.0,'EPSG','1031',-1.34,'EPSG','1028',-0.1,-0.1,1.8,'EPSG','1027',0.0,0.0,0.0,'EPSG','1032',-0.08,'EPSG','1030',2000.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IERS-Wld',0);
INSERT INTO "usage" VALUES('EPSG','9706','helmert_transformation','EPSG','6300','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','6301','ITRF2005 to ITRF2008 (2)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4896','EPSG','5332',0.01,2.0,0.9,4.7,'EPSG','1025',0.0,0.0,0.0,'EPSG','1031',-0.94,'EPSG','1028',-0.1,-0.3,0.0,'EPSG','1027',0.0,0.0,0.0,'EPSG','1032',0.0,'EPSG','1030',2000.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IERS-Wld 2008',1);
INSERT INTO "usage" VALUES('EPSG','9707','helmert_transformation','EPSG','6301','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','6302','ITRF2000 to ITRF2005 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m. Estimated using 70 stations.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4919','EPSG','4896',0.01,-0.1,0.8,5.8,'EPSG','1025',0.0,0.0,0.0,'EPSG','1031',-0.4,'EPSG','1028',0.2,-0.1,1.8,'EPSG','1027',0.0,0.0,0.0,'EPSG','1032',-0.08,'EPSG','1030',2000.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IERS-Wld',0);
INSERT INTO "usage" VALUES('EPSG','9708','helmert_transformation','EPSG','6302','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','6313','ITRF96 to GDA94 (1)','Replaced by Dawson and Woods transformation of 2010, code 6280.','EPSG','1056','Time-dependent Coordinate Frame rotation (geocen)','EPSG','4917','EPSG','4938',0.1,-0.014,0.0431,0.201,'EPSG','9001',0.012464,0.012013,0.006434,'EPSG','9104',0.024607,'EPSG','9202',0.0411,0.0218,0.0383,'EPSG','1042',0.002542,0.001431,-0.000234,'EPSG','1043',0.005897,'EPSG','1041',2000.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'GA-Aus 2001',0);
INSERT INTO "usage" VALUES('EPSG','9714','helmert_transformation','EPSG','6313','EPSG','1036','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','6314','ITRF97 to GDA94 (1)','Replaced by Dawson and Woods transformation of 2010, tfm code 6279.','EPSG','1056','Time-dependent Coordinate Frame rotation (geocen)','EPSG','4918','EPSG','4938',999.0,-0.2088,0.0119,0.1805,'EPSG','9001',0.012059,0.013369,0.011825,'EPSG','9104',0.004559,'EPSG','9202',-0.022,0.0049,0.0169,'EPSG','1042',0.00204,0.001782,0.001697,'EPSG','1043',-0.00109,'EPSG','1041',2000.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'GA-Aus 2001',1);
INSERT INTO "usage" VALUES('EPSG','9715','helmert_transformation','EPSG','6314','EPSG','1036','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','6315','ITRF2000 to GDA94 (1)','Replaced by Dawson and Woods transformation of 2010, tfm code 6278.','EPSG','1056','Time-dependent Coordinate Frame rotation (geocen)','EPSG','4919','EPSG','4938',0.1,-0.0761,-0.0101,0.0444,'EPSG','9001',0.008765,0.009361,0.009325,'EPSG','9104',0.007935,'EPSG','9202',0.011,-0.0045,-0.0174,'EPSG','1042',0.001034,0.000671,0.001039,'EPSG','1043',-0.000538,'EPSG','1041',2000.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'GA-Aus 2001',0);
INSERT INTO "usage" VALUES('EPSG','9716','helmert_transformation','EPSG','6315','EPSG','1036','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','6373','Mexico ITRF2008 to WGS 84 (1)','Approximation at the +/- 1m level assuming that Mexico ITRF2008 is equivalent to WGS 84.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','6365','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-Mex',0);
INSERT INTO "usage" VALUES('EPSG','9720','helmert_transformation','EPSG','6373','EPSG','1160','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','6388','Ocotepeque 1935 to NAD27 (1)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','5451','EPSG','4267',9.0,205.435,-29.099,292.202,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Cri',1);
INSERT INTO "usage" VALUES('EPSG','9728','helmert_transformation','EPSG','6388','EPSG','3876','EPSG','1153');
INSERT INTO "helmert_transformation" VALUES('EPSG','6389','ITRF2005 to ITRF2008 (2)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m. IERS also publish parameter values for epoch 2005.0; because most rates are zero all values as above except tX=0.5mm. Estimated using 171 stations at 131 sites.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4896','EPSG','5332',0.01,2.0,0.9,4.7,'EPSG','1025',0.0,0.0,0.0,'EPSG','1031',-0.94,'EPSG','1028',-0.3,0.0,0.0,'EPSG','1027',0.0,0.0,0.0,'EPSG','1032',0.0,'EPSG','1030',2000.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IERS-Wld 2008',0);
INSERT INTO "usage" VALUES('EPSG','9729','helmert_transformation','EPSG','6389','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','6392','ITRF97 to GDA94 (1)','Replaced by Dawson and Woods transformation of 2010, tfm code 6279.','EPSG','1056','Time-dependent Coordinate Frame rotation (geocen)','EPSG','4918','EPSG','4938',0.1,-0.2088,0.0119,0.1855,'EPSG','9001',0.012059,0.013639,0.011825,'EPSG','9104',0.004559,'EPSG','9202',-0.022,0.0049,0.0169,'EPSG','1042',0.00204,0.001782,0.001697,'EPSG','1043',-0.00109,'EPSG','1041',2000.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'GA-Aus 2001',0);
INSERT INTO "usage" VALUES('EPSG','9731','helmert_transformation','EPSG','6392','EPSG','1036','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','6698','JGD2000 to JGD2011 (2)','Excludes areas of northern Honshu affected by 2008 Iwate-Miyagi and 2011 Tohoku earthquakes. For these areas use GSI PatchJGD application or JGD2000 to JGD2011 (1) (tfm code 6713).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4612','EPSG','6668',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-Jpn ex N Honshu',0);
INSERT INTO "usage" VALUES('EPSG','9734','helmert_transformation','EPSG','6698','EPSG','4163','EPSG','1253');
INSERT INTO "helmert_transformation" VALUES('EPSG','6701','GDBD2009 to WGS 84 (1)','Approximation at the +/- 1m level assuming that GDBD2009 is equivalent to WGS 84 within the accuracy of the transformation.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','5246','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-Brn',0);
INSERT INTO "usage" VALUES('EPSG','9736','helmert_transformation','EPSG','6701','EPSG','1055','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','6710','RDN2008 to ETRS89 (1)','RDN2008 is the second Italian realization of ETRS89. May be taken as approximate transformation RDN2008 to WGS 84 - see code 6711.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','6706','EPSG','4258',0.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-Ita',0);
INSERT INTO "usage" VALUES('EPSG','9738','helmert_transformation','EPSG','6710','EPSG','3343','EPSG','1161');
INSERT INTO "helmert_transformation" VALUES('EPSG','6711','RDN2008 to WGS 84 (1)','Approximation at the +/- 1m level assuming that ETRS89 is equivalent to WGS 84. RDN2008 is a regional realisation of ETRS89.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','6706','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-Ita',0);
INSERT INTO "usage" VALUES('EPSG','9739','helmert_transformation','EPSG','6711','EPSG','1127','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','6864','ITRF96 to NAD83(CORS96) (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m. Jointly derived by Canada and US at 12 North American VLBI stations. Replaced by CT code 6865 from 2000-01-01. See CT code 8259 for Canadian equivalent.','EPSG','1056','Time-dependent Coordinate Frame rotation (geocen)','EPSG','4917','EPSG','6781',0.0,0.991,-1.9072,-0.5129,'EPSG','9001',25.79,9.65,11.66,'EPSG','1031',0.0,'EPSG','1028',0.0,0.0,0.0,'EPSG','1042',0.0532,-0.7423,-0.0316,'EPSG','1032',0.0,'EPSG','1030',1997.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'NGS-US CORS96',0);
INSERT INTO "usage" VALUES('EPSG','9797','helmert_transformation','EPSG','6864','EPSG','1511','EPSG','1173');
INSERT INTO "helmert_transformation" VALUES('EPSG','6865','ITRF97 to NAD83(CORS96) (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m. 1996 derivation (see CT 6864) concatenated with IGS value of ITRF96>ITRF97. Replaced by CT code 6866 from 2002-01-01. See CT code 8260 for Canadian equivalent.','EPSG','1056','Time-dependent Coordinate Frame rotation (geocen)','EPSG','4918','EPSG','6781',0.0,0.9889,-1.9074,-0.503,'EPSG','9001',25.915,9.426,11.599,'EPSG','1031',-0.93,'EPSG','1028',0.0007,-0.0001,0.0019,'EPSG','1042',0.067,-0.757,-0.031,'EPSG','1032',-0.19,'EPSG','1030',1997.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'NGS-US CORS96',0);
INSERT INTO "usage" VALUES('EPSG','9798','helmert_transformation','EPSG','6865','EPSG','1511','EPSG','1174');
INSERT INTO "helmert_transformation" VALUES('EPSG','6866','ITRF2000 to NAD83(CORS96) (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m. Joint derivation by Canada and US (tfm 6864) concatenated with IGS value for ITRF96>97 and IERS ITRF97>2000 transformations. See tfm 8261 for Canadian equivalent.','EPSG','1056','Time-dependent Coordinate Frame rotation (geocen)','EPSG','4919','EPSG','6781',0.0,0.9956,-1.9013,-0.5215,'EPSG','9001',25.915,9.426,11.599,'EPSG','1031',0.62,'EPSG','1028',0.0007,-0.0007,0.0005,'EPSG','1042',0.067,-0.757,-0.051,'EPSG','1032',-0.18,'EPSG','1030',1997.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'NGS-US CORS96',0);
INSERT INTO "usage" VALUES('EPSG','9799','helmert_transformation','EPSG','6866','EPSG','1511','EPSG','1175');
INSERT INTO "helmert_transformation" VALUES('EPSG','6872','Abidjan 1987 to WGS 84 (2)','Derived and used by Western Geophysical for offshore surveys in the 1990s, but exact provenance uncertain. Used by OMV.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4143','EPSG','4326',2.0,-123.1,53.2,465.4,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'WGC-Civ',0);
INSERT INTO "usage" VALUES('EPSG','9801','helmert_transformation','EPSG','6872','EPSG','2296','EPSG','1216');
INSERT INTO "helmert_transformation" VALUES('EPSG','6873','Tananarive to WGS 84 (2)','Derived at 9 points throughout Madagascar. Adopted by OMV.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4297','EPSG','4326',3.0,-198.383,-240.517,-107.909,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'ROG-Mdg',0);
INSERT INTO "usage" VALUES('EPSG','9802','helmert_transformation','EPSG','6873','EPSG','1149','EPSG','1043');
INSERT INTO "helmert_transformation" VALUES('EPSG','6888','Ocotepeque 1935 to NAD27 (1)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','5451','EPSG','4267',9.0,205.435,-29.099,-292.202,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Cri',0);
INSERT INTO "usage" VALUES('EPSG','9806','helmert_transformation','EPSG','6888','EPSG','3876','EPSG','1153');
INSERT INTO "helmert_transformation" VALUES('EPSG','6889','Ocotepeque 1935 to WGS 84 (2)','Rotations in original source given in radians are equivalent to Rx = 2.35", Ry = -0.06", Rz = 6.39".','EPSG','1063','Molodensky-Badekas (PV geog2D domain)','EPSG','5451','EPSG','4326',5.0,213.116,9.358,-74.946,'EPSG','9001',1.14e-05,-2.98e-07,3.1e-05,'EPSG','9101',5.22,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,617749.7118,-6250547.7336,1102063.6099,'EPSG','9001','UNA-Cri',0);
INSERT INTO "usage" VALUES('EPSG','9807','helmert_transformation','EPSG','6889','EPSG','3232','EPSG','1153');
INSERT INTO "helmert_transformation" VALUES('EPSG','6890','Ocotepeque 1935 to CR05 (1)','May be taken as approximate transformation Ocotepeque to WGS 84 - see code 5470.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','5451','EPSG','5365',8.0,213.11,9.37,-74.95,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Cri',0);
INSERT INTO "usage" VALUES('EPSG','9808','helmert_transformation','EPSG','6890','EPSG','3232','EPSG','1153');
INSERT INTO "helmert_transformation" VALUES('EPSG','6891','Ocotepeque 1935 to WGS 84 (3)','Concatenation (via NAD27) of transformations 6888 and 1171. Accuracy not given, but accuracy of constituent transformations given as 9m and 10m respectively.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','5451','EPSG','4326',14.0,205.0,96.0,-98.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NGA-Cri',0);
INSERT INTO "usage" VALUES('EPSG','9809','helmert_transformation','EPSG','6891','EPSG','3876','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','6895','Viti Levu 1912 to WGS 84 (2)','Derived at 9 stations. Accuracy +/-3m in each axis. Replaces Viti Levu 1912 to WGS 84 (1) (code 15897).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4752','EPSG','4326',5.0,98.0,390.0,-22.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NGA-Fji GeoTrans3-4',0);
INSERT INTO "usage" VALUES('EPSG','9810','helmert_transformation','EPSG','6895','EPSG','3195','EPSG','1153');
INSERT INTO "helmert_transformation" VALUES('EPSG','6896','Accra to WGS 84 (4)','Derived at 4 stations. Accuracy 3m, 4m and 3m in X, Y and Z axes.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4168','EPSG','4326',6.0,-170.0,33.0,326.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NGA-Gha GeoTrans3-4',0);
INSERT INTO "usage" VALUES('EPSG','9811','helmert_transformation','EPSG','6896','EPSG','3252','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','6897','St. Lucia 1955 to WGS 84 (2)','Derived at 3 stations. Accuracy 1m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4606','EPSG','4326',2.0,-153.0,153.0,307.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NGA-Lca GeoTrans3-4',0);
INSERT INTO "usage" VALUES('EPSG','9812','helmert_transformation','EPSG','6897','EPSG','3298','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','6898','Lisbon to WGS 84 (5)','Derived at 1 station. Accuracy 25m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4207','EPSG','4326',43.0,-306.0,-62.0,105.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NGA-Prt GeoTrans3-4',0);
INSERT INTO "usage" VALUES('EPSG','9813','helmert_transformation','EPSG','6898','EPSG','1294','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','6899','Pulkovo 1942 to WGS 84 (21)','Derived at 19 stations. Accuracy 2m, 3m and 3m in X, Y and Z axes.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4284','EPSG','4326',5.0,22.0,-126.0,-85.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NGA-Est GeoTrans3-4',0);
INSERT INTO "usage" VALUES('EPSG','9814','helmert_transformation','EPSG','6899','EPSG','3246','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','6900','Observatario to WGS 84 (1)','Derived at 3 stations. Accuracy 10m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4129','EPSG','4326',17.0,-132.0,-110.0,-335.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NGA-Moz Geotrans3-4',0);
INSERT INTO "usage" VALUES('EPSG','9815','helmert_transformation','EPSG','6900','EPSG','1329','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','6901','Tete to WGS 84 (6)','Derived at 4 stations. Accuracy 10m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4127','EPSG','4326',17.0,-80.0,-100.0,-228.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NGA-Moz GeoTrans3-4',0);
INSERT INTO "usage" VALUES('EPSG','9816','helmert_transformation','EPSG','6901','EPSG','3281','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','6902','Timbalai 1948 to WGS 84 (6)','Derived at 9 stations. Accuracy 1m, 6m and 2m in X, Y and Z axes.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4298','EPSG','4326',6.0,-679.0,667.0,-49.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NGA-Brn GeoTrans3-4',0);
INSERT INTO "usage" VALUES('EPSG','9817','helmert_transformation','EPSG','6902','EPSG','2349','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','6903','Yoff to WGS 84 (2)','Derived at 7 stations. Accuracy 3m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4310','EPSG','4326',5.0,-30.0,190.0,89.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NGA-Sen GeoTrans3-4',0);
INSERT INTO "usage" VALUES('EPSG','9818','helmert_transformation','EPSG','6903','EPSG','1207','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','6904','Arc 1950 to WGS 84 (11)','Derived at 7 stations. Accuracy 13m, 25m and 7m in X, Y and Z axes. Info source gives source CRS as Arc 1960. From inspection of parameter values, comparison with other CTs and geographic applicability of CRS, OGP believes that this should be Arc 1950.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4209','EPSG','4326',29.0,-179.0,-81.0,-314.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NGA-Mwi GeoTrans3-4',0);
INSERT INTO "usage" VALUES('EPSG','9819','helmert_transformation','EPSG','6904','EPSG','1150','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','6905','AGD66 to WGS 84 (20)','Derived at 161 stations. Accuracy 5m in each axis. Replaces AGD66 to WGS 84 (1) (code 1108).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4202','EPSG','4326',9.0,-128.0,-52.0,153.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NGA-Aus GeoTrans3-4',0);
INSERT INTO "usage" VALUES('EPSG','9820','helmert_transformation','EPSG','6905','EPSG','2575','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','6906','Arc 1950 to WGS 84 (10)','Derived at 38 stations. Accuracy 10m in each of X, Y and Z axes. Replaces Arc 1950 to WGS 84 (9), tfm code 1121.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4209','EPSG','4326',17.0,-145.0,-97.0,-292.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NGA-Zwe GeoTrans3-4',0);
INSERT INTO "usage" VALUES('EPSG','9821','helmert_transformation','EPSG','6906','EPSG','1261','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','6907','Ayabelle Lighthouse to WGS 84 (2)','Derived at 2 stations. Accuracy 10m in each axis. Replaces Ayabelle Lighthouse to WGS 84 (1) (code 15800).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4713','EPSG','4326',17.0,-77.0,-128.0,142.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NGA-Dji GeoTrans3-4',0);
INSERT INTO "usage" VALUES('EPSG','9822','helmert_transformation','EPSG','6907','EPSG','3238','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','6908','Fahud to WGS 84 (3)','Derived at 11 stations. Accuracy 3m, 3m and 6m in X, Y and Z axes. Replaces Fahud to WGS 84 (1) (code 1256).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4232','EPSG','4326',7.0,-345.0,3.0,223.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NGA-Omn GeoTrans3-4',0);
INSERT INTO "usage" VALUES('EPSG','9823','helmert_transformation','EPSG','6908','EPSG','4009','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','6909','Hjorsey 1955 to WGS 84 (2)','Derived at 16 stations. Accuracy 3m, 3m and 6m in X, Y and Z axes. Replaces Hjorsey 1955 to WGS 84 (1) (code 1951).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4658','EPSG','4326',7.0,-73.0,47.0,-83.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NGA-Isl GeoTrans3-4',0);
INSERT INTO "usage" VALUES('EPSG','9824','helmert_transformation','EPSG','6909','EPSG','3262','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','6910','Aden 1925 to WGS 84 (1)','Derivation not given. Accuracy not specified.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','6881','EPSG','4326',999.0,-24.0,-203.0,268.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NGA-Yem GeoTrans3-4',0);
INSERT INTO "usage" VALUES('EPSG','9825','helmert_transformation','EPSG','6910','EPSG','1340','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','6911','Bekaa Valley 1920 to WGS 84 (1)','Derivation not given. Accuracy not specified.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','6882','EPSG','4326',999.0,-183.0,-15.0,273.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NGA-Lbn GeoTrans3-4',0);
INSERT INTO "usage" VALUES('EPSG','9826','helmert_transformation','EPSG','6911','EPSG','3269','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','6912','Bioko to WGS 84 (1)','Derived at 6 stations. Accuracy 5m, 17m and 38m in X, Y and Z axes.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','6883','EPSG','4326',42.0,-235.0,-110.0,393.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NGA-Gnq GeoTrans3-4',0);
INSERT INTO "usage" VALUES('EPSG','9827','helmert_transformation','EPSG','6912','EPSG','4220','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','6913','Gambia to WGS 84 (1)','Derived at 1 station. Accuracy 25m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','6894','EPSG','4326',43.0,-63.0,176.0,185.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NGA-Gmb GeoTrans3-4',0);
INSERT INTO "usage" VALUES('EPSG','9828','helmert_transformation','EPSG','6913','EPSG','3250','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','6914','South East Island 1943 to WGS 84 (1)','Derived by UK DOS at 10 stations in 1998, RMS ±0.314m. Also published by NGA in Standard 0036 v1.0.0 of 2014-07-08 and in GeoTrans v3.4 software with parameter values rounded to integer.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','6892','EPSG','4326',1.0,-43.685,-179.785,-267.721,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DOS-Syc 3-param',0);
INSERT INTO "usage" VALUES('EPSG','9829','helmert_transformation','EPSG','6914','EPSG','4183','EPSG','1153');
INSERT INTO "helmert_transformation" VALUES('EPSG','6926','South East Island 1943 to WGS 84 (2)','Derived by UKHO at 13 stations in 1999, RMS ±0.271m.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','6892','EPSG','4326',1.0,-76.269,-16.683,68.562,'EPSG','9001',-6.275,10.536,-4.286,'EPSG','9104',-13.686,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'UKHO-Syc 7-param',0);
INSERT INTO "usage" VALUES('EPSG','9834','helmert_transformation','EPSG','6926','EPSG','4183','EPSG','1198');
INSERT INTO "helmert_transformation" VALUES('EPSG','6935','IGS08 to IGRS (1)','Derived by least squares adjustment from the coordinates of the IRAQ-CORS network in both CRSs. Station Baghdad was excluded (high residuals). RMSE = 0.004 m. Application yields identical results to transformation 6936.','EPSG','1061','Molodensky-Badekas (PV geocentric domain)','EPSG','6934','EPSG','3887',0.05,0.208,-0.012,-0.229,'EPSG','9001',-0.01182,0.00811,-0.01677,'EPSG','9104',-0.0059,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,3777505.028,3779254.396,3471111.632,'EPSG','9001','IRQ-MB(PV)',0);
INSERT INTO "usage" VALUES('EPSG','9838','helmert_transformation','EPSG','6935','EPSG','1124','EPSG','1178');
INSERT INTO "helmert_transformation" VALUES('EPSG','6936','IGS08 to IGRS (2)','Derived by least squares adjustment from the coordinates of the IRAQ-CORS network in both CRSs. Station Baghdad was excluded (high residuals). RMSE = 0.004 m. Application yields identical results to transformation 6935.','EPSG','1033','Position Vector transformation (geocentric domain)','EPSG','6934','EPSG','3887',0.05,-0.214,0.119,0.156,'EPSG','9001',-0.01182,0.00811,-0.01677,'EPSG','9104',-0.0059,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IRQ-7PV',0);
INSERT INTO "usage" VALUES('EPSG','9839','helmert_transformation','EPSG','6936','EPSG','1124','EPSG','1178');
INSERT INTO "helmert_transformation" VALUES('EPSG','6937','AGD66 to PNG94 (2)','Derived in 2014 at 38 stations around the PNG mainland. Aligned to the Bevan Rapids Geodetic Origin AA 070 as required by the Papua New Guinea Oil and Gas Act 1998.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4202','EPSG','5546',1.0,-0.41,-2.37,2.0,'EPSG','9001',3.592,3.698,3.989,'EPSG','9104',8.843,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'Qcl-Png Mainland 1m',0);
INSERT INTO "usage" VALUES('EPSG','9840','helmert_transformation','EPSG','6937','EPSG','4214','EPSG','1158');
INSERT INTO "helmert_transformation" VALUES('EPSG','6938','AGD66 to PNG94 (3)','Derived in 2014 at 38 stations around the PNG mainland. See AGD66 to PNG94 (2) for a more accurate 7-parameter transformation. May be taken as an approximate transformation AGD66 to WGS 84 - see tfm code 6943.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4202','EPSG','5546',4.0,-129.0,-58.0,152.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'Qcl-Png Mainland 4m',0);
INSERT INTO "usage" VALUES('EPSG','9841','helmert_transformation','EPSG','6938','EPSG','4214','EPSG','1157');
INSERT INTO "helmert_transformation" VALUES('EPSG','6939','AGD66 to PNG94 (4)','Derived in 2014 at 23 stations around the Kutubu oilfields. Aligned to the Bevan Rapids Geodetic Origin AA 070 as required by the Papua New Guinea Oil and Gas Act 1998. Replaces AGD66 to PNG94 (1) (tfm code 5662).','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4202','EPSG','5546',1.0,-131.876,-54.554,453.346,'EPSG','9001',-5.2155,-8.2042,0.09,'EPSG','9104',5.02,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'Qcl-Png PFTB 2014 1m',0);
INSERT INTO "usage" VALUES('EPSG','9842','helmert_transformation','EPSG','6939','EPSG','4013','EPSG','1158');
INSERT INTO "helmert_transformation" VALUES('EPSG','6940','AGD66 to PNG94 (5)','Derived in 2014 at 23 stations around the Kutubu oilfields. See AGD66 to PNG94 (4) for a more accurate 7-parameter transformation. May be taken as an approximate transformation AGD66 to WGS 84 - see tfm code 6944.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4202','EPSG','5546',2.0,-131.3,-55.3,151.8,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'Qcl-Png PFTB 2014 2m',0);
INSERT INTO "usage" VALUES('EPSG','9843','helmert_transformation','EPSG','6940','EPSG','4013','EPSG','1157');
INSERT INTO "helmert_transformation" VALUES('EPSG','6941','AGD66 to PNG94 (6)','Derived in 2014 at 7 stations in Ningerum and Tabubil (North Fly District).','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4202','EPSG','5546',0.5,45.928,-177.212,336.867,'EPSG','9001',-4.6039,-3.0921,0.5729,'EPSG','9104',36.796,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'Qcl-Png North Fly 1m',0);
INSERT INTO "usage" VALUES('EPSG','9844','helmert_transformation','EPSG','6941','EPSG','4216','EPSG','1158');
INSERT INTO "helmert_transformation" VALUES('EPSG','6942','AGD66 to PNG94 (7)','Derived in 2014 at 7 stations in Ningerum and Tabubil (North Fly District). See AGD66 to PNG94 (6) for a more accurate 7-parameter transformation. May be taken as an approximate transformation AGD66 to WGS 84 - see tfm code 6945.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4202','EPSG','5546',2.5,-137.4,-58.9,150.4,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'Qcl-Png North Fly 3m',0);
INSERT INTO "usage" VALUES('EPSG','9845','helmert_transformation','EPSG','6942','EPSG','4216','EPSG','1157');
INSERT INTO "helmert_transformation" VALUES('EPSG','6943','AGD66 to WGS 84 (21)','Parameter values taken from AGD66 to PNG94 (3) (code 6938). Approximation at the +/- 5m level assuming that PNG94 is equivalent to WGS 84 within the accuracy of the transformation.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4202','EPSG','4326',5.0,-129.0,-58.0,152.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'Qcl-Png Mainland',0);
INSERT INTO "usage" VALUES('EPSG','9846','helmert_transformation','EPSG','6943','EPSG','4214','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','6944','AGD66 to WGS 84 (22)','Parameter values taken from AGD66 to PNG94 (5) (code 6940). Approximation at the +/- 4m level assuming that PNG94 is equivalent to WGS 84 within the accuracy of the transformation.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4202','EPSG','4326',4.0,-131.3,-55.3,151.8,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'Qcl-Png PFTB 2014',0);
INSERT INTO "usage" VALUES('EPSG','9847','helmert_transformation','EPSG','6944','EPSG','4013','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','6945','AGD66 to WGS 84 (23)','Parameter values taken from AGD66 to PNG94 (7) (code 6942). Approximation at the +/- 4m level assuming that PNG94 is equivalent to WGS 84 within the accuracy of the transformation.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4202','EPSG','4326',4.0,-137.4,-58.9,150.4,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'Qcl-Png North Fly',0);
INSERT INTO "usage" VALUES('EPSG','9848','helmert_transformation','EPSG','6945','EPSG','4216','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','6949','PSAD56 to SIRGAS-Chile 2002 (1)','Also used as a transformation from PSAD56 to WGS 84 - see code 6971.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4248','EPSG','5360',5.0,-302.0,272.0,-360.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGM-Chl A',0);
INSERT INTO "usage" VALUES('EPSG','9852','helmert_transformation','EPSG','6949','EPSG','4231','EPSG','1210');
INSERT INTO "helmert_transformation" VALUES('EPSG','6950','PSAD56 to SIRGAS-Chile 2002 (2)','Also used as a transformation from PSAD56 to WGS 84 - see code 6972.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4248','EPSG','5360',5.0,-328.0,340.0,-329.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGM-Chl B',0);
INSERT INTO "usage" VALUES('EPSG','9853','helmert_transformation','EPSG','6950','EPSG','4222','EPSG','1210');
INSERT INTO "helmert_transformation" VALUES('EPSG','6951','PSAD56 to SIRGAS-Chile 2002 (3)','Also used as a transformation from PSAD56 to WGS 84 - see code 6973.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4248','EPSG','5360',5.0,-352.0,403.0,-287.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGM-Chl C',0);
INSERT INTO "usage" VALUES('EPSG','9854','helmert_transformation','EPSG','6951','EPSG','4221','EPSG','1210');
INSERT INTO "helmert_transformation" VALUES('EPSG','6960','VN-2000 to WGS 84 (2)','','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4756','EPSG','4326',1.0,-191.90441429,-39.30318279,-111.45032835,'EPSG','9001',-0.00928836,0.01975479,-0.00427372,'EPSG','9104',0.252906278,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DoSM-Vnm',0);
INSERT INTO "usage" VALUES('EPSG','9859','helmert_transformation','EPSG','6960','EPSG','3328','EPSG','1065');
INSERT INTO "helmert_transformation" VALUES('EPSG','6963','Albanian 1987 to ETRS89 (1)','Derived using 90 stations, mse 18cm. May be taken as approximate transformation from Albanian 1987 to WGS 84 (see code 6964).','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4191','EPSG','4258',0.2,-44.183,-0.58,-38.489,'EPSG','9001',2.3867,2.7072,-3.5196,'EPSG','9104',-8.2703,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGM-Alb 2D',1);
INSERT INTO "usage" VALUES('EPSG','9861','helmert_transformation','EPSG','6963','EPSG','3212','EPSG','1032');
INSERT INTO "helmert_transformation" VALUES('EPSG','6964','Albanian 1987 to WGS 84 (1)','Parameter values from Albanian 1987 to ETRS89 (1) (code 6963). Assumes ETRS89 and WGS 84 can be considered the same to within the accuracy of the transformation.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4191','EPSG','4326',1.0,-44.183,-0.58,-38.489,'EPSG','9001',2.3867,2.7072,-3.5196,'EPSG','9104',-8.2703,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IOGP-Alb 2D',1);
INSERT INTO "usage" VALUES('EPSG','9862','helmert_transformation','EPSG','6964','EPSG','3212','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','6967','SAD69 to SIRGAS-Chile (1)','Also used as a transformation from SAD69 to WGS 84 - see code 6974. Note: SAD69 adopted in Chile only south of 43°30''S.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4618','EPSG','4326',5.0,-59.0,-11.0,-52.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGM-Chl A',1);
INSERT INTO "usage" VALUES('EPSG','9864','helmert_transformation','EPSG','6967','EPSG','4232','EPSG','1210');
INSERT INTO "helmert_transformation" VALUES('EPSG','6968','SAD69 to SIRGAS-Chile 2002 (2)','Also used as a transformation from SAD69 to WGS 84 - see code 6975. Note: SAD69 adopted in Chile only south of 43°30''S.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4618','EPSG','5360',5.0,-64.0,0.0,-32.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGM-Chl B',0);
INSERT INTO "usage" VALUES('EPSG','9865','helmert_transformation','EPSG','6968','EPSG','4224','EPSG','1210');
INSERT INTO "helmert_transformation" VALUES('EPSG','6969','SAD69 to SIRGAS-Chile (3)','Also used as a transformation from SAD69 to WGS 84 - see code 6976. Note: SAD69 adopted in Chile only south of 43°30''S.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4618','EPSG','4326',5.0,-72.0,10.0,-32.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGM-Chl C',1);
INSERT INTO "usage" VALUES('EPSG','9866','helmert_transformation','EPSG','6969','EPSG','4221','EPSG','1210');
INSERT INTO "helmert_transformation" VALUES('EPSG','6970','SAD69 to SIRGAS-Chile 2002 (4)','Also used as a transformation from SAD69 to WGS 84 - see code 6977.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4618','EPSG','5360',5.0,-79.0,13.0,-14.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGM-Chl D',0);
INSERT INTO "usage" VALUES('EPSG','9867','helmert_transformation','EPSG','6970','EPSG','2805','EPSG','1210');
INSERT INTO "helmert_transformation" VALUES('EPSG','6971','PSAD56 to WGS 84 (15)','Derived at 5 stations. Accuracy 10m in each axis. Replaces PSAD56 to WGS 84 (3) (code 1203). Also used as a transformation from PSAD56 to SIRGAS-Chile - see code 6949.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4248','EPSG','4326',17.0,-302.0,272.0,-360.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NGA-Chl N 2014',0);
INSERT INTO "usage" VALUES('EPSG','9868','helmert_transformation','EPSG','6971','EPSG','4231','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','6972','PSAD56 to WGS 84 (16)','Derived at 7 stations. Accuracy 10m in each axis. Also used as a transformation from PSAD56 to SIRGAS-Chile - see code 6950.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4248','EPSG','4326',17.0,-328.0,340.0,-329.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NGA-Chl Cen 2014',0);
INSERT INTO "usage" VALUES('EPSG','9869','helmert_transformation','EPSG','6972','EPSG','4222','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','6973','PSAD56 to WGS 84 (17)','Derived at 6 stations. Accuracy 10m in each axis. Replaces PSAD56 to WGS 84 (4) (code 1204). Info source gives S limit as 44°S but Chilean IGM states that PSAD56 limit is 43°30''S. Also used as a transformation from PSAD56 to SIRGAS-Chile - see code 6951.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4248','EPSG','4326',17.0,-352.0,403.0,-287.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NGA-Chl S 2014',0);
INSERT INTO "usage" VALUES('EPSG','9870','helmert_transformation','EPSG','6973','EPSG','4221','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','6974','SAD69 to WGS 84 (17)','Derived at 8 stations. Accuracy 2m in each axis. Along with CTs 6975 and 6976, replaces SAD69 to WGS 84 (5) (code 1868). Also used as a CT from SAD69 to SIRGAS-Chile - see code 7448. Note: SAD69 adopted by Chile authorities only south of 43°30''S.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4618','EPSG','4326',4.0,-59.0,-11.0,-52.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NGA-Chl 17-32',0);
INSERT INTO "usage" VALUES('EPSG','9871','helmert_transformation','EPSG','6974','EPSG','4232','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','6975','SAD69 to WGS 84 (18)','Derived at 6 stations. Accuracy 2m in each axis. Along with CTs 6974 and 6976, replaces SAD69 to WGS 84 (5) (code 1868). Also used as a CT from SAD69 to SIRGAS-Chile - see code 6968. Note: SAD69 adopted by Chile authorities only south of 43°30''S.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4618','EPSG','4326',4.0,-64.0,0.0,-32.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NGA-Chile 32-36',0);
INSERT INTO "usage" VALUES('EPSG','9872','helmert_transformation','EPSG','6975','EPSG','4224','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','6976','SAD69 to WGS 84 (19)','Derived at 4 stations. Accuracy 4m in each axis. Along with CTs 6974 and 6975, replaces SAD69 to WGS 84 (5) (code 1868). Also used as a CT from SAD69 to SIRGAS-Chile - see code 7449. Note: SAD69 adopted by Chile authorities only south of 43°30''S.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4618','EPSG','4326',7.0,-72.0,10.0,-32.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NGA-Chl 36-44',0);
INSERT INTO "usage" VALUES('EPSG','9873','helmert_transformation','EPSG','6976','EPSG','4221','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','6977','SAD69 to WGS 84 (20)','Derived at 6 stations. Accuracy 3m, 3m and 4m in X, Y and Z axes. Also used as a transformation from SAD69 to SIRGAS-Chile - see code 6970. Unlike IGM Chile, NGA extends use of this tfm to all Chile south of 44°S.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4618','EPSG','4326',6.0,-79.0,13.0,-14.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NGA-Chile 44-',0);
INSERT INTO "usage" VALUES('EPSG','9874','helmert_transformation','EPSG','6977','EPSG','2805','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','6992','IGD05 to IGD05/12','','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','7136','EPSG','7139',0.05,0.2255,-0.3709,-0.1171,'EPSG','9001',-0.00388,0.00063,-0.0182,'EPSG','9104',0.013443,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'SOI-Isr',0);
INSERT INTO "usage" VALUES('EPSG','9875','helmert_transformation','EPSG','6992','EPSG','1126','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','6993','IGD05/12 to IG05/12 Intermediate CRS','Replaces IGD05 transformation (code 7140). Defines the IG05/12 Intermediate CRS so is considered errorless. Israeli documentation refers to target CRS as "in GRS80". Use this CT for cadastre and precise engineering but see CTs 9186 or 9189 for GIS use.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','7139','EPSG','6990',0.0,-24.0024,-17.1032,-17.8444,'EPSG','9001',-0.33009,-1.85269,1.66969,'EPSG','9104',5.4248,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'SOI-Isr',0);
INSERT INTO "usage" VALUES('EPSG','9876','helmert_transformation','EPSG','6993','EPSG','2603','EPSG','1113');
INSERT INTO "helmert_transformation" VALUES('EPSG','6998','Nahrwan 1967 to WGS 84 (11)','Derived via WGS 72 but provenance uncertain. In ADMA replaces tfm code 15938. In ADCO replaced by tfm code 6999 from October 2013.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4270','EPSG','4326',5.0,-233.4,-160.7,381.5,'EPSG','9001',0.0,0.0,-0.554,'EPSG','9104',0.2263,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'ADNOC-UAE Abd',0);
INSERT INTO "usage" VALUES('EPSG','9879','helmert_transformation','EPSG','6998','EPSG','4226','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','6999','Nahrwan 1967 to WGS 84 (12)','Derived in October 2013 at four control points of the ADCO CRF and evaluated at four others. Estimated horizontal accuracy of 0.14 m at the 95% confidence level.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4270','EPSG','4326',0.15,-253.4392,-148.452,386.5267,'EPSG','9001',-0.15605,-0.43,0.1013,'EPSG','9104',-0.0424,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'ADCO-UAE Abd 2013',0);
INSERT INTO "usage" VALUES('EPSG','9880','helmert_transformation','EPSG','6999','EPSG','4225','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','7002','Nahrwan 1967 to WGS 84 (13)','','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4270','EPSG','4326',1.0,-246.1633,-152.9047,382.6047,'EPSG','9001',-0.0989,-0.1382,-0.0768,'EPSG','9104',2.1e-06,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'ADM-UAE Abd Isl',0);
INSERT INTO "usage" VALUES('EPSG','9883','helmert_transformation','EPSG','7002','EPSG','4229','EPSG','1064');
INSERT INTO "helmert_transformation" VALUES('EPSG','7003','Nahrwan 1967 to WGS 84 (14)','','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4270','EPSG','4326',1.0,-242.8907,-149.0671,384.416,'EPSG','9001',-0.19044,-0.24987,-0.13925,'EPSG','9104',0.0001746,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'ADM-UAE Abd U39',0);
INSERT INTO "usage" VALUES('EPSG','9884','helmert_transformation','EPSG','7003','EPSG','1850','EPSG','1064');
INSERT INTO "helmert_transformation" VALUES('EPSG','7004','Nahrwan 1967 to WGS 84 (15)','','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4270','EPSG','4326',1.0,-246.734,-153.4345,382.1477,'EPSG','9001',0.116617,0.165167,0.091327,'EPSG','9104',1.94e-05,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'ADM-UAE Abd U40',0);
INSERT INTO "usage" VALUES('EPSG','9885','helmert_transformation','EPSG','7004','EPSG','4227','EPSG','1064');
INSERT INTO "helmert_transformation" VALUES('EPSG','7033','Nahrwan 1934 to WGS 84 (6)','Derived by concatenation of parameter values published by IGN Paris from Nahrwan 1934 to WGS 72 at the Nahrwan SE Base station near Baghdad with DMA WGS 72 to WGS 84 parameter values. For more accurate transformation away from origin see codes 7008-7032.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4744','EPSG','4326',30.0,-242.2,-144.9,370.3,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Irq',0);
INSERT INTO "usage" VALUES('EPSG','9911','helmert_transformation','EPSG','7033','EPSG','3625','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','7083','Perroud 1950 to RGTAAF07 (1)','Derived at three point on Petrels island at which residuals about 20 cm.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4637','EPSG','7073',0.5,324.912,153.282,172.026,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN Ata Petrel',0);
INSERT INTO "usage" VALUES('EPSG','9926','helmert_transformation','EPSG','7083','EPSG','2817','EPSG','1035');
INSERT INTO "helmert_transformation" VALUES('EPSG','7140','IGD05 to IG05 Intermediate CRS','Defines the IG05 Intermediate CRS so is considered errorless. Target CRS is referred to in Israeli documentation as "in GRS80". Replaced by IG05/12 transformation (code 6993).','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','7136','EPSG','6983',0.0,-23.8085,-17.5937,-17.801,'EPSG','9001',-0.3306,-1.85706,1.64828,'EPSG','9104',5.4374,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'SOI-Isr',0);
INSERT INTO "usage" VALUES('EPSG','9949','helmert_transformation','EPSG','7140','EPSG','2603','EPSG','1113');
INSERT INTO "helmert_transformation" VALUES('EPSG','7377','ONGD14 to WGS 84 (1)','Translations given by information source in mm. Derived at 20 stations, RMS 0.0313m in northing, 0.0377m in easting and 0.0678m in height.','EPSG','1032','Coordinate Frame rotation (geocentric domain)','EPSG','7371','EPSG','4978',0.1,0.819,-0.5762,-1.6446,'EPSG','9001',0.00378,0.03317,-0.00318,'EPSG','9104',0.0693,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NSA-Omn',0);
INSERT INTO "usage" VALUES('EPSG','10065','helmert_transformation','EPSG','7377','EPSG','1183','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','7442','Nord Sahara 1959 to WGS 84 (10)','Derived at 1 astro station central to concession. Significant and varying differences (>100m) known to exist in neighbouring astro stations. May be taken as a transformation to RGSH2020 - see CT 10344.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4307','EPSG','4326',100.0,-181.7,64.7,247.2,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'Isa-Alg Ain Tsila',0);
INSERT INTO "usage" VALUES('EPSG','10106','helmert_transformation','EPSG','7442','EPSG','4382','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','7443','ONGD14 to WGS 84 (2)','Approximation at the +/- 2m level assuming that ONG14 is equivalent to WGS 84. See transformation code 7377 for authoritative values.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','7373','EPSG','4326',2.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IOGP-Omn',0);
INSERT INTO "usage" VALUES('EPSG','10107','helmert_transformation','EPSG','7443','EPSG','1183','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','7444','CGRS93 to ETRS89 (1)','Derived at 6 points at epoch 1993.1. May be taken as approximate transformation CGRS93 to WGS 84 - see code 7445.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','6311','EPSG','4258',0.1,8.846,-4.394,-1.122,'EPSG','9001',0.00237,0.146528,-0.130428,'EPSG','9104',0.783926,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DLS-Cyp',1);
INSERT INTO "usage" VALUES('EPSG','10108','helmert_transformation','EPSG','7444','EPSG','3236','EPSG','1243');
INSERT INTO "helmert_transformation" VALUES('EPSG','7445','CGRS93 to WGS 84 (1)','Parameter values from CGRS93 to ETRS89 (1) (code 7444). Approximation at the +/- 1m level assuming that ETRS89 is equivalent to WGS 84.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','6311','EPSG','4326',1.0,8.846,-4.394,-1.122,'EPSG','9001',0.00237,0.146528,-0.130428,'EPSG','9104',0.783926,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IOGP-Cyp',1);
INSERT INTO "usage" VALUES('EPSG','10109','helmert_transformation','EPSG','7445','EPSG','3236','EPSG','1243');
INSERT INTO "helmert_transformation" VALUES('EPSG','7448','SAD69 to SIRGAS-Chile 2002 (1)','Also used as a transformation from SAD69 to WGS 84 - see code 6974. Note: SAD69 adopted in Chile only south of 43°30''S.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4618','EPSG','5360',5.0,-59.0,-11.0,-52.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGM-Chl A',0);
INSERT INTO "usage" VALUES('EPSG','10110','helmert_transformation','EPSG','7448','EPSG','4232','EPSG','1210');
INSERT INTO "helmert_transformation" VALUES('EPSG','7449','SAD69 to SIRGAS-Chile 2002 (3)','Also used as a transformation from SAD69 to WGS 84 - see code 6976. Note: SAD69 adopted in Chile only south of 43°30''S.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4618','EPSG','5360',5.0,-72.0,10.0,-32.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGM-Chl C',0);
INSERT INTO "usage" VALUES('EPSG','10111','helmert_transformation','EPSG','7449','EPSG','4221','EPSG','1210');
INSERT INTO "helmert_transformation" VALUES('EPSG','7666','WGS 84 (G1762) to ITRF2008 (1)','Defined at epoch 2005.0. Scale difference in ppb where 1/billion = 1E-9 or nm/m.','EPSG','1032','Coordinate Frame rotation (geocentric domain)','EPSG','7664','EPSG','5332',0.01,0.0,0.0,0.0,'EPSG','1025',0.0,0.0,0.0,'EPSG','1031',0.0,'EPSG','1028',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NGA-Wld 2005.0',0);
INSERT INTO "usage" VALUES('EPSG','10198','helmert_transformation','EPSG','7666','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','7667','WGS 84 (G1674) to WGS 84 (G1762) (1)','Defined at epoch 2005.0. Scale difference in ppb where 1/billion = 1E-9 or nm/m.','EPSG','1032','Coordinate Frame rotation (geocentric domain)','EPSG','7662','EPSG','7664',0.01,-4.0,3.0,4.0,'EPSG','1025',0.27,-0.27,0.38,'EPSG','1031',-6.9,'EPSG','1028',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NGA-Wld 2005.0',0);
INSERT INTO "usage" VALUES('EPSG','10199','helmert_transformation','EPSG','7667','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','7668','WGS 84 (G1150) to WGS 84 (G1762) (1)','Defined at epoch 2001.0. Scale difference in ppb where 1/billion = 1E-9 or nm/m.','EPSG','1032','Coordinate Frame rotation (geocentric domain)','EPSG','7660','EPSG','7664',0.02,-6.0,5.0,20.0,'EPSG','1025',0.0,0.0,0.0,'EPSG','1031',-4.5,'EPSG','1028',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NGA-Wld 2001.0',0);
INSERT INTO "usage" VALUES('EPSG','10200','helmert_transformation','EPSG','7668','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','7669','WGS 84 (G1674) to ITRF2008 (1)','Defined at epoch 2005.0. Scale difference in ppb where 1/billion = 1E-9 or nm/m.','EPSG','1032','Coordinate Frame rotation (geocentric domain)','EPSG','7662','EPSG','5332',0.01,0.0,0.0,0.0,'EPSG','1025',0.0,0.0,0.0,'EPSG','1031',0.0,'EPSG','1028',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NGA-Wld 2005.0',0);
INSERT INTO "usage" VALUES('EPSG','10201','helmert_transformation','EPSG','7669','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','7670','WGS 84 (G1150) to ITRF2000 (1)','Defined at epoch 2001.0.','EPSG','1032','Coordinate Frame rotation (geocentric domain)','EPSG','7660','EPSG','4919',0.02,0.0,0.0,0.0,'EPSG','9001',0.0,0.0,0.0,'EPSG','9104',0.0,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NGA-Wld 2001.0',0);
INSERT INTO "usage" VALUES('EPSG','10202','helmert_transformation','EPSG','7670','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','7671','WGS 84 (G873) to ITRF92 (1)','Defined at epoch 1997.0.','EPSG','1032','Coordinate Frame rotation (geocentric domain)','EPSG','7658','EPSG','4914',0.1,0.0,0.0,0.0,'EPSG','9001',0.0,0.0,0.0,'EPSG','9104',0.0,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NGA-Wld 1997.0',1);
INSERT INTO "usage" VALUES('EPSG','10203','helmert_transformation','EPSG','7671','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','7672','WGS 84 (G730) to ITRF92 (1)','Defined at epoch 1994.0.','EPSG','1032','Coordinate Frame rotation (geocentric domain)','EPSG','7656','EPSG','4914',0.2,0.0,0.0,0.0,'EPSG','9001',0.0,0.0,0.0,'EPSG','9104',0.0,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NGA-Wld 1994.0',0);
INSERT INTO "usage" VALUES('EPSG','10204','helmert_transformation','EPSG','7672','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','7675','MGI 1901 to ETRS89 (6)','Derived at 5506 points across the Republic of Serbia. May be taken as approximate transformation MGI 1901 to WGS 84 assuming ETRS89 is equivalent to WGS 84 within the accuracy of the transformation - see tfm code 7676.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','3906','EPSG','4258',0.5,577.88891,165.22205,391.18289,'EPSG','9001',-4.9145,0.94729,13.05098,'EPSG','9104',7.78664,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'RGZ-Srb',0);
INSERT INTO "usage" VALUES('EPSG','10207','helmert_transformation','EPSG','7675','EPSG','4543','EPSG','1035');
INSERT INTO "helmert_transformation" VALUES('EPSG','7676','MGI 1901 to WGS 84 (11)','Parameter values from MGI 1901 to ETRS89 (6) (code 7675). Assumes ETRS89 and WGS 84 can be considered the same to within the accuracy of the transformation.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','3906','EPSG','4326',1.0,577.88891,165.22205,391.18289,'EPSG','9001',-4.9145,0.94729,13.05098,'EPSG','9104',7.78664,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IOGP-Srb',0);
INSERT INTO "usage" VALUES('EPSG','10208','helmert_transformation','EPSG','7676','EPSG','4543','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','7697','Egypt 1907 to WGS 84 (4)','Derived at 30 stations throughout Egypt 1907 network. Accuracy determined at 15 stations 0.7m in each axis. Unified transformation for whole country.','EPSG','9636','Molodensky-Badekas (CF geog2D domain)','EPSG','4229','EPSG','4326',1.2,-127.535,113.495,-12.7,'EPSG','9001',1.603747,-0.153612,-5.364408,'EPSG','9104',5.33745,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,4854969.728,2945552.013,2868447.61,'EPSG','9001','SRI-Egy',0);
INSERT INTO "usage" VALUES('EPSG','10214','helmert_transformation','EPSG','7697','EPSG','1086','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','7698','NAD27 to WGS 84 (89)','Derived at stations in the provinces of Colón, Panamá, Coclé, Veraguas, Herrera, Los Santos y Chiriquí. Standard deviation 0.871m in north and 0.531m in east.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4267','EPSG','4326',1.0,-32.3841359,180.4090461,120.8442577,'EPSG','9001',2.1545854,0.1498782,-0.5742915,'EPSG','9104',8.1049164,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGNTG-Pan',0);
INSERT INTO "usage" VALUES('EPSG','10215','helmert_transformation','EPSG','7698','EPSG','3290','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','7702','PZ-90 to PZ-90.02 (1)','','EPSG','1066','Time-specific Coordinate Frame rotation (geocen)','EPSG','4922','EPSG','7677',0.17,-1.07,-0.03,0.02,'EPSG','9001',0.0,0.0,-130.0,'EPSG','1031',-0.22,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,2002.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'MTD-Rus',0);
INSERT INTO "usage" VALUES('EPSG','10217','helmert_transformation','EPSG','7702','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','7703','PZ-90.02 to PZ-90.11 (1)','','EPSG','1066','Time-specific Coordinate Frame rotation (geocen)','EPSG','7677','EPSG','7679',0.07,-0.373,0.186,0.202,'EPSG','9001',-2.3,3.54,-4.21,'EPSG','1031',-0.008,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,2010.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'MTD-Rus',0);
INSERT INTO "usage" VALUES('EPSG','10218','helmert_transformation','EPSG','7703','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','7704','PZ-90 to PZ-90.11 (1)','Concatenation of transformations 7702 and 7703.','EPSG','1032','Coordinate Frame rotation (geocentric domain)','EPSG','4922','EPSG','7679',0.2,-1.443,0.156,0.222,'EPSG','9001',-2.3,3.54,-134.21,'EPSG','1031',-0.228,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'MTD-Rus',0);
INSERT INTO "usage" VALUES('EPSG','10219','helmert_transformation','EPSG','7704','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','7705','GSK-2011 to PZ-90.11 (1)','','EPSG','1066','Time-specific Coordinate Frame rotation (geocen)','EPSG','7681','EPSG','7679',0.03,0.0,0.014,-0.008,'EPSG','9001',-0.562,-0.019,0.053,'EPSG','1031',-0.0006,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,2011.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'MTD-Rus',0);
INSERT INTO "usage" VALUES('EPSG','10220','helmert_transformation','EPSG','7705','EPSG','1198','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','7720','CGRS93 to ETRS89 (1)','Derived at 6 points at epoch 1993.1. May be taken as approximate transformation CGRS93 to WGS 84 - see code 7721.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','6311','EPSG','4258',0.1,8.846,-4.394,-1.122,'EPSG','9001',0.00237,0.146528,-0.130428,'EPSG','9104',0.783926,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DLS-Cyp',0);
INSERT INTO "usage" VALUES('EPSG','10233','helmert_transformation','EPSG','7720','EPSG','3236','EPSG','1243');
INSERT INTO "helmert_transformation" VALUES('EPSG','7721','CGRS93 to WGS 84 (1)','Parameter values from CGRS93 to ETRS89 (1) (code 7720). Approximation at the +/- 1m level assuming that ETRS89 is equivalent to WGS 84.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','6311','EPSG','4326',1.0,8.846,-4.394,-1.122,'EPSG','9001',0.00237,0.146528,-0.130428,'EPSG','9104',0.783926,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IOGP-Cyp',0);
INSERT INTO "usage" VALUES('EPSG','10234','helmert_transformation','EPSG','7721','EPSG','3236','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','7790','ITRF2008 to ITRF2014 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m. Estimated using 127 stations at 125 sites.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','5332','EPSG','7789',0.01,-1.6,-1.9,-2.4,'EPSG','1025',0.0,0.0,0.0,'EPSG','1031',0.02,'EPSG','1028',0.0,0.0,0.1,'EPSG','1027',0.0,0.0,0.0,'EPSG','1032',-0.03,'EPSG','1030',2010.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IERS-Wld',0);
INSERT INTO "usage" VALUES('EPSG','10269','helmert_transformation','EPSG','7790','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','7806','Pulkovo 1942(83) to BGS2005 (1)','Official transformation for converting existing geodetic and cartographic materials to BGS2005. Older CRSs (CS30, CS50, CS70) must first be transformed to Pulkovo 1942(83) before this transformation is applied.','EPSG','1063','Molodensky-Badekas (PV geog2D domain)','EPSG','4178','EPSG','7798',5.0,5.0,-133.0,-104.0,'EPSG','9001',-1.4,-2.0,3.4,'EPSG','9104',-3.9901,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,4223032.0,2032778.0,4309209.0,'EPSG','9001','RD-Bul',0);
INSERT INTO "usage" VALUES('EPSG','10271','helmert_transformation','EPSG','7806','EPSG','3224','EPSG','1178');
INSERT INTO "helmert_transformation" VALUES('EPSG','7807','ITRF2008 to NAD83(2011) (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m. Joint derivation by Canada and US (tfm 6864) concatenated with IGS value for ITRF96>97 and IERS ITRF97>2008 transformations. See tfm 8264 for Canadian equivalent.','EPSG','1056','Time-dependent Coordinate Frame rotation (geocen)','EPSG','5332','EPSG','6317',0.0,0.99343,-1.90331,-0.52655,'EPSG','9001',25.91467,9.42645,11.59935,'EPSG','1031',1.71504,'EPSG','1028',0.00079,-0.0006,-0.00134,'EPSG','1042',0.06667,-0.75744,-0.05133,'EPSG','1032',-0.10201,'EPSG','1030',1997.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'NGS-NA',0);
INSERT INTO "usage" VALUES('EPSG','10272','helmert_transformation','EPSG','7807','EPSG','1511','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','7808','ITRF2008 to NAD83(PA11) (1)','Information source gives IGS08 as source CRS: for most practical purposes IGS08 is equivalent to ITRF2008.','EPSG','1056','Time-dependent Coordinate Frame rotation (geocen)','EPSG','5332','EPSG','6320',0.0,0.908,-2.0161,-0.5653,'EPSG','9001',27.741,13.469,2.712,'EPSG','1031',1.1,'EPSG','1028',0.0001,0.0001,-0.0018,'EPSG','1042',-0.384,1.007,-2.186,'EPSG','1032',0.08,'EPSG','1030',1997.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'NGS-PA',0);
INSERT INTO "usage" VALUES('EPSG','10273','helmert_transformation','EPSG','7808','EPSG','4162','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','7809','ITRF2008 to NAD83(MA11) (1)','Information source gives IGS08 as source CRS: for most practical purposes IGS08 is equivalent to ITRF2008.','EPSG','1056','Time-dependent Coordinate Frame rotation (geocen)','EPSG','5332','EPSG','6323',0.0,0.908,-2.0161,-0.5653,'EPSG','9001',28.971,10.42,8.928,'EPSG','1031',1.1,'EPSG','1028',0.0001,0.0001,-0.0018,'EPSG','1042',-0.02,0.105,-0.347,'EPSG','1032',0.08,'EPSG','1030',1997.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'NGS-MA',0);
INSERT INTO "usage" VALUES('EPSG','10274','helmert_transformation','EPSG','7809','EPSG','4167','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','7814','ITRF89 to ITRF2000 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4911','EPSG','4919',0.01,-2.97,-4.75,7.39,'EPSG','1033',0.0,0.0,0.18,'EPSG','1031',-5.85,'EPSG','1028',0.0,0.06,0.14,'EPSG','1034',0.0,0.0,-0.02,'EPSG','1032',-0.01,'EPSG','1030',1988.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IERS-Wld',0);
INSERT INTO "usage" VALUES('EPSG','10279','helmert_transformation','EPSG','7814','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','7817','UCS-2000 to ITRF2000 (1)','Derived for epoch 2005.0 at which time it defines UCS-2000 and is therefore exact (accuracy = 0) at this epoch. May be taken as approximate transformation UCS-2000 to ETRS89 and UCS-2000 to WGS 84 - see codes 9901 and 5840.','EPSG','1031','Geocentric translations (geocentric domain)','EPSG','5558','EPSG','4919',0.0,24.322,-121.372,-75.847,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'SSGC-Ukr',0);
INSERT INTO "usage" VALUES('EPSG','10280','helmert_transformation','EPSG','7817','EPSG','1242','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','7833','Albanian 1987 to ETRS89 (1)','Derived using 90 stations by IGM Italy on behalf of ASIG. mse = 18cm. Use only for horizontal coordinates; geoid heights must be calculated with ALBGEO3 software. May be taken as approximate transformation from Albanian 1987 to WGS 84 (see code 7834).','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4191','EPSG','4258',0.2,-44.183,-0.58,-38.489,'EPSG','9001',-2.3867,-2.7072,3.5196,'EPSG','9104',-8.2703,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGM-Alb 2D',0);
INSERT INTO "usage" VALUES('EPSG','10288','helmert_transformation','EPSG','7833','EPSG','3212','EPSG','1032');
INSERT INTO "helmert_transformation" VALUES('EPSG','7834','Albanian 1987 to WGS 84 (1)','Parameter values from Albanian 1987 to ETRS89 (1) (code 7833). Assumes ETRS89 and WGS 84 can be considered the same to within the accuracy of the transformation.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4191','EPSG','4326',1.0,-44.183,-0.58,-38.489,'EPSG','9001',-2.3867,-2.7072,3.5196,'EPSG','9104',-8.2703,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IOGP-Alb 2D',0);
INSERT INTO "usage" VALUES('EPSG','10289','helmert_transformation','EPSG','7834','EPSG','3212','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','7835','Pulkovo 1942(58) to WGS 84 (22)','Derived by Deminex for nearshore Rodoni block in 1991-1992. Used by Shell for onshore seismic in 1995.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4179','EPSG','4326',2.0,74.5,-112.5,-44.3,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'Dmnx-Alb',0);
INSERT INTO "usage" VALUES('EPSG','10290','helmert_transformation','EPSG','7835','EPSG','4446','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','7836','Pulkovo 1942(58) to Albanian 1987 (1)','Albanian 1987 may be considered to be approximately equivalent to Pulkovo 1942(58) at the +/- 1m level.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4179','EPSG','4191',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IOGP-Alb',0);
INSERT INTO "usage" VALUES('EPSG','10291','helmert_transformation','EPSG','7836','EPSG','1025','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','7892','SHGD2015 to WGS 84 (1)','SHGD2015 is realized by ITRF2008 at epoch 2015.0 and can be considered coincident with WGS 84 at epoch 2015.0 Accuracy 3 cm at 1/1/2015 then degrades by 3 cm/yr from 1/1/2015 depending upon epoch of WGS 84 due to motion of the Nubian Plate','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','7886','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'ENRD-Shn Hel',0);
INSERT INTO "usage" VALUES('EPSG','10312','helmert_transformation','EPSG','7892','EPSG','3183','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','7893','Astro DOS 71 to SHGD2015 (1)','Derived at 19 stations, RMS = 12cm. May be used as an approximate transformation to WGS 84 - see code 7894.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4710','EPSG','7886',0.15,-323.65,551.39,-491.22,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'ENRD-Shn Hel 0.15m',0);
INSERT INTO "usage" VALUES('EPSG','10313','helmert_transformation','EPSG','7893','EPSG','3183','EPSG','1158');
INSERT INTO "helmert_transformation" VALUES('EPSG','7894','Astro DOS 71 to WGS 84 (2)','Parameter values from Astro DOS 71 to SHGD2015 (1) (tfm code 7893). Assumes SHGD2015 and WGS 84 can be considered the same to within the accuracy of the transformation.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4710','EPSG','4326',1.0,-323.65,551.39,-491.22,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'ENRD-Shn Hel',0);
INSERT INTO "usage" VALUES('EPSG','10314','helmert_transformation','EPSG','7894','EPSG','3183','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','7895','Astro DOS 71 to SHGD2015 (2)','Derived at 19 stations, RMS = 6cm. Note: Because of the large rotations about the Y- and Z-axes this transformation is not reversible. For the reverse transformation use SHGD2015 to Astro DOS 71 (2) (code 9226).','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4710','EPSG','7886',0.1,-112.854,12.27,-18.913,'EPSG','9001',2.1692,16.8896,17.1961,'EPSG','9104',-19.54517,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'ENRD-Shn Hel 0.1m',0);
INSERT INTO "usage" VALUES('EPSG','10315','helmert_transformation','EPSG','7895','EPSG','3183','EPSG','1031');
INSERT INTO "helmert_transformation" VALUES('EPSG','7896','SHGD2015 to Astro DOS 71 (2)','Derived at 19 stations, RMS = 6cm. Note: Because of the large rotations about the Y- and Z-axes this transformation is not reversible. For the reverse transformation use Astro DOS 71 to SHGD2015 (2) (code 7895).','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4710','EPSG','4710',0.1,112.771,-12.282,18.935,'EPSG','9001',-2.1692,-16.8896,-17.1961,'EPSG','9104',19.54517,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'ENRD-Shn Hel 0.1m Rev',1);
INSERT INTO "usage" VALUES('EPSG','10316','helmert_transformation','EPSG','7896','EPSG','3183','EPSG','1031');
INSERT INTO "helmert_transformation" VALUES('EPSG','7897','St. Helena Tritan to SHGD2015 (1)','Derived at 19 stations, RMS = 5cm. May be used as an approximate transformation to WGS 84 - see code 7898.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','7881','EPSG','7886',0.05,-0.077,0.079,0.086,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'ENRD-Shn Hel',0);
INSERT INTO "usage" VALUES('EPSG','10317','helmert_transformation','EPSG','7897','EPSG','3183','EPSG','1079');
INSERT INTO "helmert_transformation" VALUES('EPSG','7898','St. Helena Tritan to WGS 84 (1)','Parameter values from Tritan St. Helena to SHGD2015 (1) (tfm code 7897). Assumes Tritan St. Helena and SHGD2015 can be considered the same to within the accuracy of the transformation.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','7881','EPSG','4326',1.0,-0.077,0.079,0.086,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'ENRD-Shn Hel',0);
INSERT INTO "usage" VALUES('EPSG','10318','helmert_transformation','EPSG','7898','EPSG','3183','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','7932','ITRF89 to ETRF89 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4911','EPSG','7914',0.0,0.0,0.0,0.0,'EPSG','1033',0.0,0.0,0.0,'EPSG','1031',0.0,'EPSG','1028',0.0,0.0,0.0,'EPSG','1034',0.11,0.57,-0.71,'EPSG','1032',0.0,'EPSG','1030',1989.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'EUREF-Eur',0);
INSERT INTO "usage" VALUES('EPSG','10320','helmert_transformation','EPSG','7932','EPSG','1298','EPSG','1119');
INSERT INTO "helmert_transformation" VALUES('EPSG','7933','ITRF90 to ETRF90 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4912','EPSG','7916',0.0,1.9,2.8,-2.3,'EPSG','1033',0.0,0.0,0.0,'EPSG','1031',0.0,'EPSG','1028',0.0,0.0,0.0,'EPSG','1034',0.11,0.57,-0.71,'EPSG','1032',0.0,'EPSG','1030',1989.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'EUREF-Eur',0);
INSERT INTO "usage" VALUES('EPSG','10321','helmert_transformation','EPSG','7933','EPSG','1298','EPSG','1120');
INSERT INTO "helmert_transformation" VALUES('EPSG','7934','ITRF91 to ETRF91 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4913','EPSG','7918',0.0,2.1,2.5,-3.7,'EPSG','1033',0.0,0.0,0.0,'EPSG','1031',0.0,'EPSG','1028',0.0,0.0,0.0,'EPSG','1034',0.21,0.52,-0.68,'EPSG','1032',0.0,'EPSG','1030',1989.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'EUREF-Eur',0);
INSERT INTO "usage" VALUES('EPSG','10322','helmert_transformation','EPSG','7934','EPSG','1298','EPSG','1121');
INSERT INTO "helmert_transformation" VALUES('EPSG','7935','ITRF92 to ETRF92 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4914','EPSG','7920',0.0,3.8,4.0,-3.7,'EPSG','1033',0.0,0.0,0.0,'EPSG','1031',0.0,'EPSG','1028',0.0,0.0,0.0,'EPSG','1034',0.21,0.52,-0.68,'EPSG','1032',0.0,'EPSG','1030',1989.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'EUREF-Eur',0);
INSERT INTO "usage" VALUES('EPSG','10323','helmert_transformation','EPSG','7935','EPSG','1298','EPSG','1122');
INSERT INTO "helmert_transformation" VALUES('EPSG','7936','ITRF93 to ETRF93 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4915','EPSG','7922',0.0,1.9,5.3,-2.1,'EPSG','1033',0.0,0.0,0.0,'EPSG','1031',0.0,'EPSG','1028',0.0,0.0,0.0,'EPSG','1034',0.32,0.78,-0.67,'EPSG','1032',0.0,'EPSG','1030',1989.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'EUREF-Eur',0);
INSERT INTO "usage" VALUES('EPSG','10324','helmert_transformation','EPSG','7936','EPSG','1298','EPSG','1123');
INSERT INTO "helmert_transformation" VALUES('EPSG','7937','ITRF94 to ETRF94 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4916','EPSG','7924',0.0,4.1,4.1,-4.9,'EPSG','1033',0.0,0.0,0.0,'EPSG','1031',0.0,'EPSG','1028',0.0,0.0,0.0,'EPSG','1034',0.2,0.5,-0.65,'EPSG','1032',0.0,'EPSG','1030',1989.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'EUREF-Eur',0);
INSERT INTO "usage" VALUES('EPSG','10325','helmert_transformation','EPSG','7937','EPSG','1298','EPSG','1124');
INSERT INTO "helmert_transformation" VALUES('EPSG','7938','ITRF96 to ETRF96 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4917','EPSG','7926',0.0,4.1,4.1,-4.9,'EPSG','1033',0.0,0.0,0.0,'EPSG','1031',0.0,'EPSG','1028',0.0,0.0,0.0,'EPSG','1034',0.2,0.5,-0.65,'EPSG','1032',0.0,'EPSG','1030',1989.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'EUREF-Eur',0);
INSERT INTO "usage" VALUES('EPSG','10326','helmert_transformation','EPSG','7938','EPSG','1298','EPSG','1125');
INSERT INTO "helmert_transformation" VALUES('EPSG','7939','ITRF97 to ETRF97 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4918','EPSG','7928',0.0,4.1,4.1,-4.9,'EPSG','1033',0.0,0.0,0.0,'EPSG','1031',0.0,'EPSG','1028',0.0,0.0,0.0,'EPSG','1034',0.2,0.5,-0.65,'EPSG','1032',0.0,'EPSG','1030',1989.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'EUREF-Eur',0);
INSERT INTO "usage" VALUES('EPSG','10327','helmert_transformation','EPSG','7939','EPSG','1298','EPSG','1126');
INSERT INTO "helmert_transformation" VALUES('EPSG','7940','ITRF2000 to ETRF2000 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m. See ITRF2000 to ETRF2000 (2) (code 7941) for an exactly equivalent transformation but with the transformation''s parameter values at epoch 2000.00.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4919','EPSG','7930',0.0,5.4,5.1,-4.8,'EPSG','1033',0.0,0.0,0.0,'EPSG','1031',0.0,'EPSG','1028',0.0,0.0,0.0,'EPSG','1034',0.081,0.49,-0.792,'EPSG','1032',0.0,'EPSG','1030',1989.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'EUREF-Eur',0);
INSERT INTO "usage" VALUES('EPSG','10328','helmert_transformation','EPSG','7940','EPSG','1298','EPSG','1127');
INSERT INTO "helmert_transformation" VALUES('EPSG','7941','ITRF2000 to ETRF2000 (2)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m. See ITRF2000 to ETRF2000 (1) (code 7940) for transformation which defines ETRF2000. 7941 is equivalent but with the transformation''s parameters at epoch 2000.00.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4919','EPSG','7930',0.0,54.0,51.0,-48.0,'EPSG','1025',0.891,5.39,-8.712,'EPSG','1031',0.0,'EPSG','1028',0.0,0.0,0.0,'EPSG','1027',0.081,0.49,-0.792,'EPSG','1032',0.0,'EPSG','1030',2000.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'EUREF-Eur 2000',0);
INSERT INTO "usage" VALUES('EPSG','10329','helmert_transformation','EPSG','7941','EPSG','1298','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','7942','ITRF89 to ETRF2000 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4911','EPSG','7930',0.0,24.3,10.7,42.7,'EPSG','1025',0.891,5.39,-8.772,'EPSG','1031',-5.97,'EPSG','1028',0.0,0.6,1.4,'EPSG','1027',0.081,0.49,-0.812,'EPSG','1032',-0.01,'EPSG','1030',2000.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'EUREF-Eur 2000',0);
INSERT INTO "usage" VALUES('EPSG','10330','helmert_transformation','EPSG','7942','EPSG','1298','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','7943','ITRF90 to ETRF2000 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4912','EPSG','7930',0.0,29.3,34.7,4.7,'EPSG','1025',0.891,5.39,-8.772,'EPSG','1031',-2.57,'EPSG','1028',0.0,0.6,1.4,'EPSG','1027',0.081,0.49,-0.812,'EPSG','1032',-0.01,'EPSG','1030',2000.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'EUREF-Eur 2000',0);
INSERT INTO "usage" VALUES('EPSG','10331','helmert_transformation','EPSG','7943','EPSG','1298','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','7944','ITRF91 to ETRF2000 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4913','EPSG','7930',0.0,27.3,30.7,-11.3,'EPSG','1025',0.891,5.39,-8.772,'EPSG','1031',-2.27,'EPSG','1028',0.0,0.6,1.4,'EPSG','1027',0.081,0.49,-0.812,'EPSG','1032',-0.01,'EPSG','1030',2000.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'EUREF-Eur 2000',0);
INSERT INTO "usage" VALUES('EPSG','10332','helmert_transformation','EPSG','7944','EPSG','1298','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','7945','ITRF92 to ETRF2000 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4914','EPSG','7930',0.0,39.3,44.7,-17.3,'EPSG','1025',0.891,5.39,-8.772,'EPSG','1031',-0.87,'EPSG','1028',0.0,0.6,1.4,'EPSG','1027',0.081,0.49,-0.812,'EPSG','1032',-0.01,'EPSG','1030',2000.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'EUREF-Eur 2000',0);
INSERT INTO "usage" VALUES('EPSG','10333','helmert_transformation','EPSG','7945','EPSG','1298','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','7946','ITRF93 to ETRF2000 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4915','EPSG','7930',0.0,76.1,46.9,-19.9,'EPSG','1025',2.601,6.87,-8.412,'EPSG','1031',-2.07,'EPSG','1028',2.9,0.2,0.6,'EPSG','1027',0.191,0.68,-0.862,'EPSG','1032',-0.01,'EPSG','1030',2000.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'EUREF-Eur 2000',0);
INSERT INTO "usage" VALUES('EPSG','10334','helmert_transformation','EPSG','7946','EPSG','1298','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','7947','ITRF94 to ETRF2000 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4916','EPSG','7930',0.0,47.3,46.7,-25.3,'EPSG','1025',0.891,5.39,-8.772,'EPSG','1031',-1.58,'EPSG','1028',0.0,0.6,1.4,'EPSG','1027',0.081,0.49,-0.812,'EPSG','1032',-0.01,'EPSG','1030',2000.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'EUREF-Eur 2000',0);
INSERT INTO "usage" VALUES('EPSG','10335','helmert_transformation','EPSG','7947','EPSG','1298','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','7948','ITRF96 to ETRF2000 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4917','EPSG','7930',0.0,47.3,46.7,-25.3,'EPSG','1025',0.891,5.39,-8.772,'EPSG','1031',-1.58,'EPSG','1028',0.0,0.6,1.4,'EPSG','1027',0.081,0.49,-0.812,'EPSG','1032',-0.01,'EPSG','1030',2000.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'EUREF-Eur 2000',0);
INSERT INTO "usage" VALUES('EPSG','10336','helmert_transformation','EPSG','7948','EPSG','1298','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','7949','ITRF97 to ETRF2000 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4918','EPSG','7930',0.0,47.3,46.7,-25.3,'EPSG','1025',0.891,5.39,-8.772,'EPSG','1031',-1.58,'EPSG','1028',0.0,0.6,1.4,'EPSG','1027',0.081,0.49,-0.812,'EPSG','1032',-0.01,'EPSG','1030',2000.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'EUREF-Eur 2000',0);
INSERT INTO "usage" VALUES('EPSG','10337','helmert_transformation','EPSG','7949','EPSG','1298','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','7950','ITRF2005 to ETRF2000 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4896','EPSG','7930',0.0,54.1,50.2,-53.8,'EPSG','1025',0.891,5.39,-8.712,'EPSG','1031',0.4,'EPSG','1028',-0.2,0.1,-1.8,'EPSG','1027',0.081,0.49,-0.792,'EPSG','1032',0.08,'EPSG','1030',2000.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'EUREF-Eur 2000',0);
INSERT INTO "usage" VALUES('EPSG','10338','helmert_transformation','EPSG','7950','EPSG','1298','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','7951','ITRF2008 to ETRF2000 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','5332','EPSG','7930',0.0,52.1,49.3,-58.5,'EPSG','1025',0.891,5.39,-8.712,'EPSG','1031',1.34,'EPSG','1028',0.1,0.1,-1.8,'EPSG','1027',0.081,0.49,-0.792,'EPSG','1032',0.08,'EPSG','1030',2000.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'EUREF-Eur 2000',0);
INSERT INTO "usage" VALUES('EPSG','10339','helmert_transformation','EPSG','7951','EPSG','1298','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','7960','PZ-90.11 to ITRF2008 (1)','','EPSG','1066','Time-specific Coordinate Frame rotation (geocen)','EPSG','7679','EPSG','5332',0.004,-0.003,-0.001,0.0,'EPSG','9001',0.019,-0.042,0.002,'EPSG','1031',0.0,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,2010.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'MTD-Rus',0);
INSERT INTO "usage" VALUES('EPSG','10345','helmert_transformation','EPSG','7960','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','7961','WGS 84 (G1150) to PZ-90.02 (1)','','EPSG','1066','Time-specific Coordinate Frame rotation (geocen)','EPSG','7660','EPSG','7677',0.17,0.36,-0.08,-0.18,'EPSG','9001',0.0,0.0,0.0,'EPSG','1031',0.0,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,2002.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'MTD-Rus',0);
INSERT INTO "usage" VALUES('EPSG','10346','helmert_transformation','EPSG','7961','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8048','GDA94 to GDA2020 (1)','Scale difference in ppb where 1/billion = 1E-9. See CT codes 8444-46 for NTv2 method giving equivalent results for Christmas Island, Cocos Islands and Australia respectively. See CT code 8447 for alternative including distortion model for Australia only.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4283','EPSG','7844',0.01,61.55,-10.87,-40.19,'EPSG','1025',-39.4924,-32.7221,-32.8979,'EPSG','1031',-9.994,'EPSG','1028',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'ICSM-Aus',0);
INSERT INTO "usage" VALUES('EPSG','10401','helmert_transformation','EPSG','8048','EPSG','4177','EPSG','1108');
INSERT INTO "helmert_transformation" VALUES('EPSG','8049','ITRF2014 to GDA2020 (1)','Derived at 109 stations of the Australian Regional GNSS network (ARGN). RMS residuals 26mm N, 12mm E and 179mm up, maximum residuals 49mm N, 24mm E and 464mm up. Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1056','Time-dependent Coordinate Frame rotation (geocen)','EPSG','7789','EPSG','7842',0.03,0.0,0.0,0.0,'EPSG','1025',0.0,0.0,0.0,'EPSG','1031',0.0,'EPSG','1028',0.0,0.0,0.0,'EPSG','1027',1.50379,1.18346,1.20716,'EPSG','1032',0.0,'EPSG','1030',2020.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'GA-Aus',0);
INSERT INTO "usage" VALUES('EPSG','10402','helmert_transformation','EPSG','8049','EPSG','4177','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8069','ITRF88 to ITRF2014 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m. Derived by IERS from previously published information.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4910','EPSG','7789',0.01,-25.4,0.5,154.8,'EPSG','1025',-0.1,0.0,-0.26,'EPSG','1031',-11.29,'EPSG','1028',-0.1,0.5,3.3,'EPSG','1027',0.0,0.0,-0.02,'EPSG','1032',-0.12,'EPSG','1030',2010.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IERS-Wld',0);
INSERT INTO "usage" VALUES('EPSG','10412','helmert_transformation','EPSG','8069','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8070','ITRF89 to ITRF2014 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m. Derived by IERS from previously published information.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4911','EPSG','7789',0.01,-30.4,-35.5,130.8,'EPSG','1025',0.0,0.0,-0.26,'EPSG','1031',-8.19,'EPSG','1028',-0.1,0.5,3.3,'EPSG','1027',0.0,0.0,-0.02,'EPSG','1032',-0.12,'EPSG','1030',2010.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IERS-Wld',0);
INSERT INTO "usage" VALUES('EPSG','10413','helmert_transformation','EPSG','8070','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8071','ITRF90 to ITRF2014 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m. Derived by IERS from previously published information.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4912','EPSG','7789',0.01,-25.4,-11.5,92.8,'EPSG','1025',0.0,0.0,-0.26,'EPSG','1031',-4.79,'EPSG','1028',-0.1,0.5,3.3,'EPSG','1027',0.0,0.0,-0.02,'EPSG','1032',-0.12,'EPSG','1030',2010.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IERS-Wld',0);
INSERT INTO "usage" VALUES('EPSG','10414','helmert_transformation','EPSG','8071','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8072','ITRF91 to ITRF2014 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m. Derived by IERS from previously published information.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4913','EPSG','7789',0.01,-27.4,-15.5,76.8,'EPSG','1025',0.0,0.0,-0.26,'EPSG','1031',-4.49,'EPSG','1028',-0.1,0.5,3.3,'EPSG','1027',0.0,0.0,-0.02,'EPSG','1032',-0.12,'EPSG','1030',2010.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IERS-Wld',0);
INSERT INTO "usage" VALUES('EPSG','10415','helmert_transformation','EPSG','8072','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8073','ITRF92 to ITRF2014 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m. Derived by IERS from previously published information.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4914','EPSG','7789',0.01,-15.4,-1.5,70.8,'EPSG','1025',0.0,0.0,-0.26,'EPSG','1031',-3.09,'EPSG','1028',-0.1,0.5,3.3,'EPSG','1027',0.0,0.0,-0.02,'EPSG','1032',-0.12,'EPSG','1030',2010.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IERS-Wld',0);
INSERT INTO "usage" VALUES('EPSG','10416','helmert_transformation','EPSG','8073','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8074','ITRF93 to ITRF2014 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m. Derived by IERS from previously published information.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4915','EPSG','7789',0.01,50.4,-3.3,60.2,'EPSG','1025',2.81,3.38,-0.4,'EPSG','1031',-4.29,'EPSG','1028',2.8,0.1,2.5,'EPSG','1027',0.11,0.19,-0.07,'EPSG','1032',-0.12,'EPSG','1030',2010.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IERS-Wld',0);
INSERT INTO "usage" VALUES('EPSG','10417','helmert_transformation','EPSG','8074','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8075','ITRF94 to ITRF2014 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m. Derived by IERS from previously published information.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4916','EPSG','7789',0.01,-7.4,0.5,62.8,'EPSG','1025',0.0,0.0,-0.26,'EPSG','1031',-3.8,'EPSG','1028',-0.1,0.5,3.3,'EPSG','1027',0.0,0.0,-0.02,'EPSG','1032',-0.12,'EPSG','1030',2010.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IERS-Wld',0);
INSERT INTO "usage" VALUES('EPSG','10418','helmert_transformation','EPSG','8075','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8076','ITRF96 to ITRF2014 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m. Derived by IERS from previously published information.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4917','EPSG','7789',0.01,-7.4,0.5,62.8,'EPSG','1025',0.0,0.0,-0.26,'EPSG','1031',-3.8,'EPSG','1028',-0.1,0.5,3.3,'EPSG','1027',0.0,0.0,-0.02,'EPSG','1032',-0.12,'EPSG','1030',2010.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IERS-Wld',0);
INSERT INTO "usage" VALUES('EPSG','10419','helmert_transformation','EPSG','8076','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8077','ITRF97 to ITRF2014 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m. Derived by IERS from previously published information.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4918','EPSG','7789',0.01,-7.4,0.5,62.8,'EPSG','1025',0.0,0.0,-0.26,'EPSG','1031',-3.8,'EPSG','1028',-0.1,0.5,3.3,'EPSG','1027',0.0,0.0,-0.02,'EPSG','1032',-0.12,'EPSG','1030',2010.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IERS-Wld',0);
INSERT INTO "usage" VALUES('EPSG','10420','helmert_transformation','EPSG','8077','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8078','ITRF2000 to ITRF2014 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m. Derived by IERS from previously published information.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4919','EPSG','7789',0.01,-0.7,-1.2,26.1,'EPSG','1025',0.0,0.0,0.0,'EPSG','1031',-2.12,'EPSG','1028',-0.1,-0.1,1.9,'EPSG','1027',0.0,0.0,0.0,'EPSG','1032',-0.11,'EPSG','1030',2010.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IERS-Wld',0);
INSERT INTO "usage" VALUES('EPSG','10421','helmert_transformation','EPSG','8078','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8079','ITRF2005 to ITRF2014 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m. Derived by IERS from previously published information.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4896','EPSG','7789',0.01,-2.6,-1.0,2.3,'EPSG','1025',0.0,0.0,0.0,'EPSG','1031',-0.92,'EPSG','1028',-0.3,0.0,0.1,'EPSG','1027',0.0,0.0,0.0,'EPSG','1032',-0.03,'EPSG','1030',2010.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IERS-Wld',0);
INSERT INTO "usage" VALUES('EPSG','10422','helmert_transformation','EPSG','8079','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8256','ITRF92 to NAD83(CSRS96) (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m. Superseded by CT from ITRF93 (see code 8257).','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4914','EPSG','8230',0.0,0.936,-1.984,-0.543,'EPSG','9001',-27.5,-15.5,-10.7,'EPSG','1031',5.0,'EPSG','1028',0.0,0.0,0.0,'EPSG','1042',-0.052,0.742,0.032,'EPSG','1032',0.0,'EPSG','1030',1988.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'NRC-Can CSRS96 92',0);
INSERT INTO "usage" VALUES('EPSG','10451','helmert_transformation','EPSG','8256','EPSG','1061','EPSG','1166');
INSERT INTO "helmert_transformation" VALUES('EPSG','8257','ITRF93 to NAD83(CSRS96) (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m. Superseded by CT from ITRF94 (see code 8258).','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4915','EPSG','8230',0.0,0.94,-1.979,-0.534,'EPSG','9001',-27.09,-16.22,-9.87,'EPSG','1031',4.1,'EPSG','1028',0.0023,0.0004,-0.0008,'EPSG','1042',0.078,0.962,-0.008,'EPSG','1032',0.11,'EPSG','1030',1988.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'NRC-Can CSRS96 93',0);
INSERT INTO "usage" VALUES('EPSG','10452','helmert_transformation','EPSG','8257','EPSG','1061','EPSG','1166');
INSERT INTO "helmert_transformation" VALUES('EPSG','8258','ITRF94 to NAD83(CSRS96) (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4916','EPSG','8230',0.0,0.942,-1.979,-0.534,'EPSG','9001',-27.3,-15.4,-10.7,'EPSG','1031',4.9,'EPSG','1028',-0.0004,0.0004,-0.0008,'EPSG','1042',-0.052,0.762,0.032,'EPSG','1032',0.0,'EPSG','1030',1988.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'NRC-Can CSRS96 94',0);
INSERT INTO "usage" VALUES('EPSG','10453','helmert_transformation','EPSG','8258','EPSG','1061','EPSG','1166');
INSERT INTO "helmert_transformation" VALUES('EPSG','8259','ITRF96 to NAD83(CSRS)v2 (1)','Jointly derived by Canada and US at 12 North American VLBI stations. Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m. See tfm code 6864 for US equivalent.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4917','EPSG','8233',0.0,0.991,-1.9072,-0.5129,'EPSG','9001',-25.79,-9.65,-11.66,'EPSG','1031',0.0,'EPSG','1028',0.0,0.0,0.0,'EPSG','1042',-0.0532,0.7423,0.0316,'EPSG','1032',0.0,'EPSG','1030',1997.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'NRC-Can CSRSv2',0);
INSERT INTO "usage" VALUES('EPSG','10454','helmert_transformation','EPSG','8259','EPSG','1061','EPSG','1167');
INSERT INTO "helmert_transformation" VALUES('EPSG','8260','ITRF97 to NAD83(CSRS)v3 (1)','Concatenation of joint Canada-US transformation NAD83>ITRF96 (see tfm code 8259) and IGS value for ITRF96>ITRF97 transformation. Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m. See tfm 6865 for US equivalent.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4918','EPSG','8238',0.0,0.9889,-1.9074,-0.503,'EPSG','9001',-25.915,-9.426,-11.599,'EPSG','1031',-0.935,'EPSG','1028',0.0007,-0.0001,0.0019,'EPSG','1042',-0.067,0.757,0.031,'EPSG','1032',-0.192,'EPSG','1030',1997.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'NRC-Can CSRSv3',0);
INSERT INTO "usage" VALUES('EPSG','10455','helmert_transformation','EPSG','8260','EPSG','1061','EPSG','1168');
INSERT INTO "helmert_transformation" VALUES('EPSG','8261','ITRF2000 to NAD83(CSRS)v4 (1)','Concatenation of joint Canada-US NAD83>ITRF96 tfm (code 8259) with IGS value of ITRF96>ITRF97 and IERS tfm ITRF97>ITRF2000. Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m. See tfm code 6866 for US equivalent.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4919','EPSG','8242',0.0,0.9956,-1.9013,-0.5214,'EPSG','9001',-25.915,-9.426,-11.599,'EPSG','1031',0.615,'EPSG','1028',0.0007,-0.0007,0.0005,'EPSG','1042',-0.067,0.757,0.051,'EPSG','1032',-0.182,'EPSG','1030',1997.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'NRC-Can CSRSv4',0);
INSERT INTO "usage" VALUES('EPSG','10456','helmert_transformation','EPSG','8261','EPSG','1061','EPSG','1169');
INSERT INTO "helmert_transformation" VALUES('EPSG','8262','ITRF2005 to NAD83(CSRS)v5 (1)','Concatenation of joint Canada-US NAD83>ITRF96 transformation (code 8259) with IGS value for ITRF96>ITRF97 and IERS transformations ITRF97>ITRF2005. Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4896','EPSG','8248',0.0,0.9963,-1.9024,-0.5219,'EPSG','9001',-25.915,-9.426,-11.599,'EPSG','1031',0.775,'EPSG','1028',0.0005,-0.0006,-0.0013,'EPSG','1042',-0.067,0.757,0.051,'EPSG','1032',-0.102,'EPSG','1030',1997.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'NRC-Can CSRSv5',1);
INSERT INTO "usage" VALUES('EPSG','10457','helmert_transformation','EPSG','8262','EPSG','1061','EPSG','1170');
INSERT INTO "helmert_transformation" VALUES('EPSG','8264','ITRF2008 to NAD83(CSRS)v6 (1)','Concatenation of joint Canada-US transformation NAD83>ITRF96 (code 8259) with IGS tfm ITRF96>97 and IERS tfms ITRF97>2008. Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m. See tfm code 7807 for US equivalent.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','5332','EPSG','8250',0.0,0.99343,-1.90331,-0.52655,'EPSG','9001',-25.91467,-9.42645,-11.59935,'EPSG','1031',1.71504,'EPSG','1028',0.00079,-0.0006,-0.00134,'EPSG','1042',-0.06667,0.75744,0.05133,'EPSG','1032',-0.102,'EPSG','1030',1997.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'NRC-Can CSRSv6',0);
INSERT INTO "usage" VALUES('EPSG','10459','helmert_transformation','EPSG','8264','EPSG','1061','EPSG','1171');
INSERT INTO "helmert_transformation" VALUES('EPSG','8265','ITRF2014 to NAD83(CSRS)v7 (1)','Concatenation of joint Canada-US tfm NAD83>ITRF96 (CT code 8259) with IGS value for ITRF96>ITRF97 and IERS values for ITRF97>ITRF2014 transformations. Scale difference in ppb and rate in ppb/yr where 1/billion = 1E-9. See tfm 8970 for US equivalent.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','7789','EPSG','8253',0.0,1.0053,-1.90921,-0.54157,'EPSG','9001',-26.78138,0.42027,-10.93206,'EPSG','1031',0.36891,'EPSG','1028',0.00079,-0.0006,-0.00144,'EPSG','1042',-0.06667,0.75744,0.05133,'EPSG','1032',-0.07201,'EPSG','1030',2010.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'NRC-Can CSRSv7',0);
INSERT INTO "usage" VALUES('EPSG','10460','helmert_transformation','EPSG','8265','EPSG','1061','EPSG','1172');
INSERT INTO "helmert_transformation" VALUES('EPSG','8270','Saint Pierre et Miquelon 1950 to WGS 84 (2)','Replaces Saint Pierre et Miquelon 1950 to WGS 84 (1) (code 1923) from March 2006. Accuracy +/- 0.5 to 1 metre.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4638','EPSG','4326',1.0,11.363,424.148,373.13,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Spm 2017',0);
INSERT INTO "usage" VALUES('EPSG','10463','helmert_transformation','EPSG','8270','EPSG','3299','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','8365','ETRS89 to S-JTSK [JTSK03] (1)','Derived at 684 points with known S-JTSK and ETRS89 (ETRF2000 realization) coordinates. Scale parameter was constrained to be zero. UGKK consider this transformation to not be reversible at the 1mm accuracy level: for reverse see transformation code 8367.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4258','EPSG','8351',0.001,-485.014055,-169.473618,-483.842943,'EPSG','9001',7.78625453,4.39770887,4.10248899,'EPSG','9104',0.0,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'UGKK-Svk',0);
INSERT INTO "usage" VALUES('EPSG','10512','helmert_transformation','EPSG','8365','EPSG','1211','EPSG','1115');
INSERT INTO "helmert_transformation" VALUES('EPSG','8366','ITRF2014 to ETRF2014 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m. See ITRF2014 to ETRF2014 (2) (code 8880) for an exactly equivalent transformation but with the transformation''s parameter values at epoch 2010.00.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','7789','EPSG','8401',0.0,0.0,0.0,0.0,'EPSG','1025',0.0,0.0,0.0,'EPSG','1031',0.0,'EPSG','1028',0.0,0.0,0.0,'EPSG','1027',0.085,0.531,-0.77,'EPSG','1032',0.0,'EPSG','1030',1989.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'EUREF-Eur',0);
INSERT INTO "usage" VALUES('EPSG','14203','helmert_transformation','EPSG','8366','EPSG','1298','EPSG','1129');
INSERT INTO "helmert_transformation" VALUES('EPSG','8367','S-JTSK [JTSK03] to ETRS89 (1)','Derived at 684 points. At the 1mm accuracy level this transformation is not reversible: for reverse see transformation code 8365. May be taken as approximate transformation to WGS 84 - see code 8368.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','8351','EPSG','4258',0.001,485.021,169.465,483.839,'EPSG','9001',-7.786342,-4.397554,-4.102655,'EPSG','9104',0.0,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'UGKK-Svk',0);
INSERT INTO "usage" VALUES('EPSG','10514','helmert_transformation','EPSG','8367','EPSG','1211','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8368','S-JTSK [JTSK03] to WGS 84 (1)','Parameter values taken from S-JTSK [JTSK03] to ETRS89 (1) (code 8367) assuming that ETRS89 (ETRF2000 realization) is coincident with WGS 84 within the accuracy of the transformation. Within the 1m accuracy of this transformation, it is reversible.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','8351','EPSG','4326',1.0,485.021,169.465,483.839,'EPSG','9001',-7.786342,-4.397554,-4.102655,'EPSG','9104',0.0,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'UGKK-Svk',0);
INSERT INTO "usage" VALUES('EPSG','10515','helmert_transformation','EPSG','8368','EPSG','1211','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','8393','Camacupa to WGS 84 (11)','Derived by Univ. of Lisbon for IGCA using 38 REPANGOL points in Angola (except SE) and Cabinda. Application differs from Camacupa to WGS 84 (1) to (10) by approx 25 m. Average horizontal error 1m, vertical 3m; max radial error 6m.  For onshore use only.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4220','EPSG','4326',3.0,-93.799,-132.737,-219.073,'EPSG','9001',1.844,-0.648,6.37,'EPSG','9104',-0.169,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGCA-Ago',1);
INSERT INTO "usage" VALUES('EPSG','10528','helmert_transformation','EPSG','8393','EPSG','4469','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8405','ITRF2014 to ETRF2000 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','7789','EPSG','7930',0.0,54.7,52.2,-74.1,'EPSG','1025',1.701,10.29,-16.632,'EPSG','1031',2.12,'EPSG','1028',0.1,0.1,-1.9,'EPSG','1027',0.081,0.49,-0.792,'EPSG','1032',0.11,'EPSG','1030',2010.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'EUREF-Eur 2000',0);
INSERT INTO "usage" VALUES('EPSG','10535','helmert_transformation','EPSG','8405','EPSG','1298','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8407','ITRF2014 to ETRF2014 (2)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9. See ITRF2014 to ETRF2014 (1) (code 8366) for transformation which defines ETRF2014. Transformation 8407 is equivalent to 8366 but with parameter values at epoch 2010.00.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','7789','EPSG','7930',0.0,0.0,0.0,0.0,'EPSG','1025',1.785,11.151,-16.17,'EPSG','1031',0.0,'EPSG','1028',0.0,0.0,0.0,'EPSG','1027',0.085,0.531,-0.77,'EPSG','1032',0.0,'EPSG','1030',2010.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'EUREF-Eur 2014',1);
INSERT INTO "usage" VALUES('EPSG','10537','helmert_transformation','EPSG','8407','EPSG','1298','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8409','ITRF2008 to ETRF2014 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','5332','EPSG','7789',0.0,-1.6,-1.9,-2.4,'EPSG','1025',1.785,11.151,-16.17,'EPSG','1031',0.02,'EPSG','1028',0.0,0.0,0.1,'EPSG','1027',0.085,0.531,-0.77,'EPSG','1032',-0.03,'EPSG','1030',2010.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'EUREF-Eur 2014',1);
INSERT INTO "usage" VALUES('EPSG','10539','helmert_transformation','EPSG','8409','EPSG','1298','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8410','ITRF2005 to ETRF2014 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4896','EPSG','7789',0.0,-2.6,-1.0,-2.3,'EPSG','1025',1.785,11.151,-16.17,'EPSG','1031',-0.92,'EPSG','1028',-0.3,0.0,0.1,'EPSG','1027',0.085,0.531,-0.77,'EPSG','1032',-0.03,'EPSG','1030',2010.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'EUREF-Eur 2014',1);
INSERT INTO "usage" VALUES('EPSG','10540','helmert_transformation','EPSG','8410','EPSG','1298','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8411','ITRF2000 to ETRF2014 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4919','EPSG','7789',0.0,-0.7,-1.2,26.1,'EPSG','1025',1.785,11.151,-16.17,'EPSG','1031',-2.12,'EPSG','1028',-0.1,-0.1,1.9,'EPSG','1027',0.085,0.531,-0.77,'EPSG','1032',-0.11,'EPSG','1030',2010.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'EUREF-Eur 2014',1);
INSERT INTO "usage" VALUES('EPSG','10541','helmert_transformation','EPSG','8411','EPSG','1298','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8412','ITRF97 to ETRF2014 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4918','EPSG','7789',0.0,-7.4,0.5,62.8,'EPSG','1025',1.785,11.151,-16.43,'EPSG','1031',-3.8,'EPSG','1028',-0.1,0.5,3.3,'EPSG','1027',0.085,0.531,-0.79,'EPSG','1032',-0.12,'EPSG','1030',2010.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'EUREF-Eur 2014',1);
INSERT INTO "usage" VALUES('EPSG','10542','helmert_transformation','EPSG','8412','EPSG','1298','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8413','ITRF96 to ETRF2014 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4917','EPSG','7789',0.0,-7.4,0.5,62.8,'EPSG','1025',1.785,11.151,-16.43,'EPSG','1031',-3.8,'EPSG','1028',-0.1,0.5,3.3,'EPSG','1027',0.085,0.531,-0.79,'EPSG','1032',-0.12,'EPSG','1030',2010.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'EUREF-Eur 2014',1);
INSERT INTO "usage" VALUES('EPSG','10543','helmert_transformation','EPSG','8413','EPSG','1298','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8414','ITRF94 to ETRF2014 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4916','EPSG','7789',0.0,-7.4,0.5,62.8,'EPSG','1025',1.785,11.151,-16.43,'EPSG','1031',-3.8,'EPSG','1028',-0.1,0.5,3.3,'EPSG','1027',0.085,0.531,-0.79,'EPSG','1032',-0.12,'EPSG','1030',2010.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'EUREF-Eur 2014',1);
INSERT INTO "usage" VALUES('EPSG','10544','helmert_transformation','EPSG','8414','EPSG','1298','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8415','ITRF93 to ETRF2014 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4915','EPSG','7789',0.0,50.4,-3.3,60.2,'EPSG','1025',4.595,14.531,-16.57,'EPSG','1031',-4.29,'EPSG','1028',2.8,0.1,2.5,'EPSG','1027',0.195,0.721,-0.84,'EPSG','1032',-0.12,'EPSG','1030',2010.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'EUREF-Eur 2014',1);
INSERT INTO "usage" VALUES('EPSG','10545','helmert_transformation','EPSG','8415','EPSG','1298','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8416','ITRF92 to ETRF2014 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4914','EPSG','7789',0.0,-15.4,-1.5,70.8,'EPSG','1025',1.785,11.151,-16.43,'EPSG','1031',-3.09,'EPSG','1028',-0.1,0.5,3.3,'EPSG','1027',0.085,0.531,-0.79,'EPSG','1032',-0.12,'EPSG','1030',2010.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'EUREF-Eur 2014',1);
INSERT INTO "usage" VALUES('EPSG','10546','helmert_transformation','EPSG','8416','EPSG','1298','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8417','ITRF91 to ETRF2014 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4913','EPSG','7789',0.0,-27.4,-15.5,76.8,'EPSG','1025',1.785,11.151,-16.43,'EPSG','1031',-4.49,'EPSG','1028',-0.1,0.5,3.3,'EPSG','1027',0.085,0.531,-0.79,'EPSG','1032',-0.12,'EPSG','1030',2010.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'EUREF-Eur 2014',1);
INSERT INTO "usage" VALUES('EPSG','10547','helmert_transformation','EPSG','8417','EPSG','1298','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8423','ITRF90 to ETRF2014 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4912','EPSG','7789',0.0,-25.4,-11.5,92.8,'EPSG','1025',1.785,11.151,-16.43,'EPSG','1031',-4.79,'EPSG','1028',-0.1,0.5,3.3,'EPSG','1027',0.085,0.531,-0.79,'EPSG','1032',-0.12,'EPSG','1030',2010.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'EUREF-Eur 2014',1);
INSERT INTO "usage" VALUES('EPSG','10553','helmert_transformation','EPSG','8423','EPSG','1298','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8424','ITRF89 to ETRF2014 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4911','EPSG','7789',0.0,-30.4,-35.5,130.8,'EPSG','1025',1.785,11.151,-16.43,'EPSG','1031',-8.19,'EPSG','1028',-0.1,0.5,3.3,'EPSG','1027',0.085,0.531,-0.79,'EPSG','1032',-0.12,'EPSG','1030',2010.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'EUREF-Eur 2014',1);
INSERT INTO "usage" VALUES('EPSG','10554','helmert_transformation','EPSG','8424','EPSG','1298','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8435','Macao 2008 to Macao 1920 (1)','Derived at 3 stations in 2008. Accuracy not stated.','EPSG','9636','Molodensky-Badekas (CF geog2D domain)','EPSG','8431','EPSG','8428',999.0,202.865,303.99,155.873,'EPSG','9001',34.067,-76.126,-32.647,'EPSG','9104',-6.096,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,-2361757.652,5417232.187,2391453.053,'EPSG','9001','DSCC-Mac',0);
INSERT INTO "usage" VALUES('EPSG','10556','helmert_transformation','EPSG','8435','EPSG','1147','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','8436','Macao 2008 to WGS 84 (1)','Approximation at the +/- 1m level assuming that Macao 2008 is equivalent to WGS 84.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','8431','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-Bmu',0);
INSERT INTO "usage" VALUES('EPSG','10557','helmert_transformation','EPSG','8436','EPSG','1147','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','8437','Hong Kong 1980 to Hong Kong Geodetic CS (1)','Also published as a transformation to WGS 84 using the position vector method - see Hong Kong 1980 to WGS 84 (1) (code 1825).','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4611','EPSG','8427',1.0,-162.619,-276.961,-161.763,'EPSG','9001',0.067741,-2.243649,-1.158827,'EPSG','9104',-1.094239,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'LSD-HKG 2002',1);
INSERT INTO "usage" VALUES('EPSG','10558','helmert_transformation','EPSG','8437','EPSG','1118','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','8438','Macao 1920 to WGS 84 (1)','Derived from Macao 2008 to Macao 1920 (1) (code 8435) (reversed) assuming that Macao 2008 is equivalent to WGS 84 within the accuracy of the transformation. Some parameter values differ in the reverse due to the high rotations.','EPSG','9636','Molodensky-Badekas (CF geog2D domain)','EPSG','8428','EPSG','4326',1.0,-202.865,-303.99,-155.873,'EPSG','9001',-34.079,76.126,32.66,'EPSG','9104',6.096,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,-2361554.788,5417536.177,2391608.926,'EPSG','9001','EPSG-Mac',0);
INSERT INTO "usage" VALUES('EPSG','10559','helmert_transformation','EPSG','8438','EPSG','1147','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','8439','Hong Kong Geodetic CS to WGS 84 (1)','Approximation at the +/- 1m level assuming that Hong Kong Geodetic CS is equivalent to WGS 84.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','8427','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Hkg',0);
INSERT INTO "usage" VALUES('EPSG','10560','helmert_transformation','EPSG','8439','EPSG','1118','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','8448','GDA2020 to WGS 84 (G1762) (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m. Parameter values taken from ITRF2014 to GDA2020 (1) (code 8049), assuming WGS 84 (G1762) is equivalent to ITRF2014 within the accuracy of the transformation.','EPSG','1056','Time-dependent Coordinate Frame rotation (geocen)','EPSG','7842','EPSG','7664',0.2,0.0,0.0,0.0,'EPSG','1025',0.0,0.0,0.0,'EPSG','1031',0.0,'EPSG','1028',0.0,0.0,0.0,'EPSG','1027',-1.50379,-1.18346,-1.20716,'EPSG','1032',0.0,'EPSG','1030',2020.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'GA-Aus 0.2m',0);
INSERT INTO "usage" VALUES('EPSG','10568','helmert_transformation','EPSG','8448','EPSG','4177','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8450','GDA2020 to WGS 84 (2)','Approximation at the 3m level assuming WGS 84 is equivalent to GDA2020. Ignores the low accuracy of the WGS 84 ensemble and the inconsistent application of tectonic plate motion to WGS 84 data.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','7844','EPSG','4326',3.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'GA-Aus 3m',0);
INSERT INTO "usage" VALUES('EPSG','10569','helmert_transformation','EPSG','8450','EPSG','4177','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','8452','Batavia to WGS 84 (1)','Derived at 5 stations. Note: U.S. DMA TR8350.2 September 1987 gives source CRS as Batavia and area as Sumatra. The Batavia (Genuk) CRS applies to Java and western Sumatra. EPSG presumes this CT applies to both. Sometimes found applied to all Sumatra.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4211','EPSG','4326',6.0,-377.0,681.0,-50.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Idn Sumatra',0);
INSERT INTO "usage" VALUES('EPSG','10571','helmert_transformation','EPSG','8452','EPSG','1285','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','8674','La Canoa to PSAD56 (1)','In Venezuela PSAD56 is coincident with La Canoa.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4247','EPSG','4248',0.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'LAG-Ven E',0);
INSERT INTO "usage" VALUES('EPSG','10766','helmert_transformation','EPSG','8674','EPSG','3327','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8680','MGI 1901 to ETRS89 (7)','Derived at 1385 points across the area of Bosnia and Herzegovina. May be taken as approximate transformation MGI 1901 to WGS 84 (see code 8823).','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','3906','EPSG','4258',1.0,489.88,183.912,533.711,'EPSG','9001',5.76545,4.69994,-12.58211,'EPSG','9104',1.00646,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'FGA-Bih',0);
INSERT INTO "usage" VALUES('EPSG','10768','helmert_transformation','EPSG','8680','EPSG','1050','EPSG','1056');
INSERT INTO "helmert_transformation" VALUES('EPSG','8681','MGI 1901 to WGS 84 (12)','Parameter values from MGI 1901 to ETRS89 (7) (code 8680). Assumes ETRS89 and WGS 84 can be considered the same to within the accuracy of the transformation.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','3906','EPSG','4258',1.0,489.88,183.912,533.711,'EPSG','9001',5.76545,4.69994,-12.58211,'EPSG','9104',1.00646,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'FGA-Bih',1);
INSERT INTO "usage" VALUES('EPSG','10769','helmert_transformation','EPSG','8681','EPSG','1050','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','8688','MGI 1901 to WGS 84 (16)','Parameter values from MGI to Slovenia 1996 (12) (tfm code 8689) assuming Slovenia 1996 and WGS 84 can be considered the same to within the accuracy of the transformation.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','3906','EPSG','4326',1.0,476.08,125.947,417.81,'EPSG','9001',-4.610862,-2.388137,11.942335,'EPSG','9104',9.896638,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'GuRS-Svn 2010',0);
INSERT INTO "usage" VALUES('EPSG','10770','helmert_transformation','EPSG','8688','EPSG','3307','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','8689','MGI 1901 to Slovenia 1996 (12)','Derived at 479 nodes of Delauney triangulation generated from 1958 control points. Replaces MGI 1901 to Slovenia 1996 (1) (code 3916). May be taken as approximate transformation MGI 1901 to WGS 84 (see code 8688).','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','3906','EPSG','4765',1.0,476.08,125.947,417.81,'EPSG','9001',-4.610862,-2.388137,11.942335,'EPSG','9104',9.896638,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'GuRS-Svn 2010',0);
INSERT INTO "usage" VALUES('EPSG','10771','helmert_transformation','EPSG','8689','EPSG','3307','EPSG','1158');
INSERT INTO "helmert_transformation" VALUES('EPSG','8695','Camacupa 1948 to Camacupa 2015 (1)','Concatenation of transformations 1327 and 8882.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4220','EPSG','8694',5.8,42.899,-214.863,-11.927,'EPSG','9001',-1.844,0.648,-6.37,'EPSG','9104',0.169,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IOGP-Ago',0);
INSERT INTO "usage" VALUES('EPSG','10772','helmert_transformation','EPSG','8695','EPSG','2324','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8696','Camacupa 1948 to Camacupa 2015 (2)','Concatenation of transformations 1324 and 8882.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4220','EPSG','8694',4.2,45.799,-212.263,-11.927,'EPSG','9001',-1.844,0.648,-6.37,'EPSG','9104',0.169,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IOGP-Ago B15',0);
INSERT INTO "usage" VALUES('EPSG','10773','helmert_transformation','EPSG','8696','EPSG','2322','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8819','RSAO13 to WGS 84 (1)','Approximation at the +/- 1m level assuming that RSAO13 is equivalent to WGS 84 within the accuracy of the transformation.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','8699','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IOGP-Ago',0);
INSERT INTO "usage" VALUES('EPSG','10774','helmert_transformation','EPSG','8819','EPSG','1029','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','8822','MTRF-2000 to WGS 84 (1)','Approximation at the +/- 1m level assuming that MTRF-2000 (ITRF2000 at epoch 2004.00) is equivalent to WGS 84.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','8818','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-Sau',0);
INSERT INTO "usage" VALUES('EPSG','10775','helmert_transformation','EPSG','8822','EPSG','1206','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','8823','MGI 1901 to WGS 84 (13)','Parameter values from MGI 1901 to ETRS89 (7) (code 8680). Assumes ETRS89 and WGS 84 can be considered the same to within the accuracy of the transformation.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','3906','EPSG','4326',1.0,489.88,183.912,533.711,'EPSG','9001',5.76545,4.69994,-12.58211,'EPSG','9104',1.00646,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'FGA-Bih',0);
INSERT INTO "usage" VALUES('EPSG','10776','helmert_transformation','EPSG','8823','EPSG','1050','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','8824','Ain el Abd to MTRF-2000 (1)','Nation-wide transformation. Accuracy given as ''several metres''. More precise national cellular transformation parameters were also determined. Software coded for these cellular transformations is available in MOMRA.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4204','EPSG','8818',5.0,-61.15,-315.86,-3.51,'EPSG','9001',0.41,0.74,-3.52,'EPSG','9104',1.36,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'MOMRA-Sau',0);
INSERT INTO "usage" VALUES('EPSG','10777','helmert_transformation','EPSG','8824','EPSG','3303','EPSG','1045');
INSERT INTO "helmert_transformation" VALUES('EPSG','8827','Camacupa 2015 to RSAO13 (1)','Derived by Univ. of Lisbon for CIDDEMA using 38 REPANGOL points in Angola (except SE) and Cabinda. Average horizontal error 1m, vertical 3m; max radial error 6m.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','8694','EPSG','8699',3.0,-93.799,-132.737,-219.073,'EPSG','9001',1.844,-0.648,6.37,'EPSG','9104',-0.169,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'CIDDEMA-Ago',0);
INSERT INTO "usage" VALUES('EPSG','10779','helmert_transformation','EPSG','8827','EPSG','1029','EPSG','1130');
INSERT INTO "helmert_transformation" VALUES('EPSG','8828','RGPF to WGS 84 (1)','SHOM report gives scale difference as 0.999 999 9907 (wrt unity). Transformation is to original Transit definition of WGS 84. It is consistent with later WGS 84 realisations G730, G873 and G1150 to no better than 1m.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4687','EPSG','4326',0.5,0.072,-0.507,-0.245,'EPSG','9001',0.0183,-0.0003,0.007,'EPSG','9104',-0.0093,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'URB-Pyf',0);
INSERT INTO "usage" VALUES('EPSG','10780','helmert_transformation','EPSG','8828','EPSG','1098','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8829','Tahiti 79 to RGPF (1)','','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4690','EPSG','4687',0.5,221.525,152.948,176.768,'EPSG','9001',2.3847,1.3896,0.877,'EPSG','9104',11.4741,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'URB-Pyf',0);
INSERT INTO "usage" VALUES('EPSG','10781','helmert_transformation','EPSG','8829','EPSG','3124','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8830','Tahiti 79 to WGS 84 (2)','Concatenation of Tahiti 79 to RGPF (1) and RGPF to WGS 84 (1) (CT codes 8829 and 8828).','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4690','EPSG','4326',1.0,221.597,152.441,176.523,'EPSG','9001',2.403,1.3893,0.884,'EPSG','9104',11.4648,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'URB-Pyf',0);
INSERT INTO "usage" VALUES('EPSG','10782','helmert_transformation','EPSG','8830','EPSG','3124','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8831','Moorea 87 to RGPF (2)','Recalculated in 2009 using corrected coordinates of deriving stations.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4691','EPSG','4687',0.5,218.697,151.257,176.995,'EPSG','9001',3.5048,2.004,1.281,'EPSG','9104',10.991,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'URB-Pyf',0);
INSERT INTO "usage" VALUES('EPSG','10783','helmert_transformation','EPSG','8831','EPSG','3125','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8832','Moorea 87 to WGS 84 (2)','Concatenation of Moorea 87 to RGPF (2) and RGPF to WGS 84 (1) (CT codes 8831 and 8828).','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4691','EPSG','4326',1.0,218.769,150.75,176.75,'EPSG','9001',3.5231,2.0037,1.288,'EPSG','9104',10.9817,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'URB-Pyf',0);
INSERT INTO "usage" VALUES('EPSG','10784','helmert_transformation','EPSG','8832','EPSG','3125','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8833','Tahaa 54 to RGPF (1)','','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4629','EPSG','4687',0.5,72.438,345.918,79.486,'EPSG','9001',-1.6045,-0.8823,-0.5565,'EPSG','9104',1.3746,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'URB-Pyf',0);
INSERT INTO "usage" VALUES('EPSG','10785','helmert_transformation','EPSG','8833','EPSG','2812','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8834','Tahaa 54 to WGS 84 (3)','Concatenation of Tahaa 54 to RGPF (1) and RGPF to WGS 84 (1) (CT codes 8833 and 8828).','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4629','EPSG','4326',1.0,72.51,345.411,79.241,'EPSG','9001',-1.5862,-0.8826,-0.5495,'EPSG','9104',1.3653,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'URB-Pyf',0);
INSERT INTO "usage" VALUES('EPSG','10786','helmert_transformation','EPSG','8834','EPSG','2812','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8835','Fatu Iva 72 to RGPF (1)','','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4688','EPSG','4687',2.0,347.103,1078.125,2623.922,'EPSG','9001',33.8875,-70.6773,9.3943,'EPSG','9104',186.074,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'URB-Pyf',0);
INSERT INTO "usage" VALUES('EPSG','10787','helmert_transformation','EPSG','8835','EPSG','3133','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8842','Fatu Iva 72 to WGS 84 (2)','Concatenation of Fatu Iva 72 to RGPF (1) and RGPF to WGS 84 (1) (CT codes 8835 and 8828).','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4688','EPSG','4326',2.0,347.175,1077.618,2623.677,'EPSG','9001',33.9058,-70.6776,9.4013,'EPSG','9104',186.0647,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'URB-Pyf',0);
INSERT INTO "usage" VALUES('EPSG','10788','helmert_transformation','EPSG','8842','EPSG','3133','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8843','IGN63 Hiva Oa to RGPF (1)','','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4689','EPSG','4687',0.5,410.721,55.049,80.746,'EPSG','9001',-2.5779,-2.3514,-0.6664,'EPSG','9104',17.3311,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'URB-Pyf HivaOa',0);
INSERT INTO "usage" VALUES('EPSG','10789','helmert_transformation','EPSG','8843','EPSG','3131','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8844','IGN63 Hiva Oa to WGS 84 (3)','Concatenation of IGN63 Hiva Oa to RGPF (1) and RGPF to WGS 84 (1) (CT codes 8843 and 8828).','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4689','EPSG','4326',2.0,410.793,54.542,80.501,'EPSG','9001',-2.5596,-2.3517,-0.6594,'EPSG','9104',17.3218,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'URB-Pyf HivaOa',0);
INSERT INTO "usage" VALUES('EPSG','10790','helmert_transformation','EPSG','8844','EPSG','3131','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8845','IGN63 Hiva Oa to RGPF (2)','','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4689','EPSG','4687',2.0,374.715,-58.407,-0.957,'EPSG','9001',-16.2111,-11.4626,-5.5357,'EPSG','9104',-0.5409,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'URB-Pyf Tahuata',0);
INSERT INTO "usage" VALUES('EPSG','10791','helmert_transformation','EPSG','8845','EPSG','3132','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8846','IGN63 Hiva Oa to WGS 84 (4)','Concatenation of TIGN63 Hiva Oa to RGPF (2) and RGPF to WGS 84 (1) (CT codes 8845 and 8828).','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4689','EPSG','4326',2.0,374.787,-58.914,-1.202,'EPSG','9001',-16.1928,-11.4629,-5.5287,'EPSG','9104',-0.5502,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'URB-Pyf Tahuata',0);
INSERT INTO "usage" VALUES('EPSG','10792','helmert_transformation','EPSG','8846','EPSG','3132','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8847','IGN72 Nuku Hiva to RGPF (1)','','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4630','EPSG','4687',0.5,165.732,216.72,180.505,'EPSG','9001',-0.6434,-0.4512,-0.0791,'EPSG','9104',7.4204,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'URB-Pyf NukuHiva',0);
INSERT INTO "usage" VALUES('EPSG','10793','helmert_transformation','EPSG','8847','EPSG','2810','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8848','IGN72 Nuku Hiva to WGS 84 (5)','Concatenation of IGN72 Nuku Hiva to RGPF (1) and RGPF to WGS 84 (1) (CT codes 8847 and 8828).','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4630','EPSG','4326',1.0,165.804,216.213,180.26,'EPSG','9001',-0.6251,-0.4515,-0.0721,'EPSG','9104',7.4111,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'URB-Pyf NukuHiva',0);
INSERT INTO "usage" VALUES('EPSG','10794','helmert_transformation','EPSG','8848','EPSG','2810','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8849','IGN72 Nuku Hiva to RGPF (2)','','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4630','EPSG','4687',2.0,1363.785,1362.687,398.811,'EPSG','9001',-4.5322,-6.7579,-1.0574,'EPSG','9104',268.361,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'URB-Pyf UaHuka',0);
INSERT INTO "usage" VALUES('EPSG','10795','helmert_transformation','EPSG','8849','EPSG','3127','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8850','IGN72 Nuku Hiva to WGS 84 (6)','Concatenation of IGN72 Nuku Hiva to RGPF (2) and RGPF to WGS 84 (1) (CT codes 8849 and 8828).','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4630','EPSG','4326',2.0,1363.857,1362.18,398.566,'EPSG','9001',-4.5139,-6.7582,-1.0504,'EPSG','9104',268.3517,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'URB-Pyf UaHuka',0);
INSERT INTO "usage" VALUES('EPSG','10796','helmert_transformation','EPSG','8850','EPSG','3127','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8851','IGN72 Nuku Hiva to RGPF (3)','','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4630','EPSG','4687',0.5,259.551,297.612,197.833,'EPSG','9001',1.4866,2.1224,0.4612,'EPSG','9104',27.0249,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'URB-Pyf UaPou',0);
INSERT INTO "usage" VALUES('EPSG','10797','helmert_transformation','EPSG','8851','EPSG','3128','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8852','IGN72 Nuku Hiva to WGS 84 (7)','Concatenation of IGN72 Nuku Hiva to RGPF (3) and RGPF to WGS 84 (1) (CT codes 8851 and 8828).','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4630','EPSG','4326',1.0,259.623,297.105,197.588,'EPSG','9001',1.5049,2.1221,0.4682,'EPSG','9104',27.0156,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'URB-Pyf UaPou',0);
INSERT INTO "usage" VALUES('EPSG','10798','helmert_transformation','EPSG','8852','EPSG','3128','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8853','Maupiti 83 to WGS 84 (2)','Concatenation of Maupiti 83 to RGPF (1) and RGPF to WGS 84 (1) (CT codes 15759 and 8828).','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4692','EPSG','4326',1.0,217.109,86.452,23.711,'EPSG','9001',0.0183,-0.0003,0.007,'EPSG','9104',-0.0093,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'URB-Pyf',0);
INSERT INTO "usage" VALUES('EPSG','10799','helmert_transformation','EPSG','8853','EPSG','3126','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8869','ITRF2008 to ETRF2014 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','5332','EPSG','8401',0.0,-1.6,-1.9,-2.4,'EPSG','1025',1.785,11.151,-16.17,'EPSG','1031',0.02,'EPSG','1028',0.0,0.0,0.1,'EPSG','1027',0.085,0.531,-0.77,'EPSG','1032',-0.03,'EPSG','1030',2010.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'EUREF-Eur 2014',0);
INSERT INTO "usage" VALUES('EPSG','10811','helmert_transformation','EPSG','8869','EPSG','1298','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8870','ITRF2005 to ETRF2014 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4896','EPSG','8401',0.0,-2.6,-1.0,2.3,'EPSG','1025',1.785,11.151,-16.17,'EPSG','1031',-0.92,'EPSG','1028',-0.3,0.0,0.1,'EPSG','1027',0.085,0.531,-0.77,'EPSG','1032',-0.03,'EPSG','1030',2010.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'EUREF-Eur 2014',0);
INSERT INTO "usage" VALUES('EPSG','10812','helmert_transformation','EPSG','8870','EPSG','1298','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8871','ITRF2000 to ETRF2014 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4919','EPSG','8401',0.0,-0.7,-1.2,26.1,'EPSG','1025',1.785,11.151,-16.17,'EPSG','1031',-2.12,'EPSG','1028',-0.1,-0.1,1.9,'EPSG','1027',0.085,0.531,-0.77,'EPSG','1032',-0.11,'EPSG','1030',2010.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'EUREF-Eur 2014',0);
INSERT INTO "usage" VALUES('EPSG','10813','helmert_transformation','EPSG','8871','EPSG','1298','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8872','ITRF97 to ETRF2014 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4918','EPSG','8401',0.0,-7.4,0.5,62.8,'EPSG','1025',1.785,11.151,-16.43,'EPSG','1031',-3.8,'EPSG','1028',-0.1,0.5,3.3,'EPSG','1027',0.085,0.531,-0.79,'EPSG','1032',-0.12,'EPSG','1030',2010.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'EUREF-Eur 2014',0);
INSERT INTO "usage" VALUES('EPSG','10814','helmert_transformation','EPSG','8872','EPSG','1298','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8873','ITRF96 to ETRF2014 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4917','EPSG','8401',0.0,-7.4,0.5,62.8,'EPSG','1025',1.785,11.151,-16.43,'EPSG','1031',-3.8,'EPSG','1028',-0.1,0.5,3.3,'EPSG','1027',0.085,0.531,-0.79,'EPSG','1032',-0.12,'EPSG','1030',2010.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'EUREF-Eur 2014',0);
INSERT INTO "usage" VALUES('EPSG','10815','helmert_transformation','EPSG','8873','EPSG','1298','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8874','ITRF94 to ETRF2014 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4916','EPSG','8401',0.0,-7.4,0.5,62.8,'EPSG','1025',1.785,11.151,-16.43,'EPSG','1031',-3.8,'EPSG','1028',-0.1,0.5,3.3,'EPSG','1027',0.085,0.531,-0.79,'EPSG','1032',-0.12,'EPSG','1030',2010.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'EUREF-Eur 2014',0);
INSERT INTO "usage" VALUES('EPSG','10816','helmert_transformation','EPSG','8874','EPSG','1298','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8875','ITRF93 to ETRF2014 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4915','EPSG','8401',0.0,50.4,-3.3,60.2,'EPSG','1025',4.595,14.531,-16.57,'EPSG','1031',-4.29,'EPSG','1028',2.8,0.1,2.5,'EPSG','1027',0.195,0.721,-0.84,'EPSG','1032',-0.12,'EPSG','1030',2010.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'EUREF-Eur 2014',0);
INSERT INTO "usage" VALUES('EPSG','10817','helmert_transformation','EPSG','8875','EPSG','1298','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8876','ITRF92 to ETRF2014 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4914','EPSG','8401',0.0,-15.4,-1.5,70.8,'EPSG','1025',1.785,11.151,-16.43,'EPSG','1031',-3.09,'EPSG','1028',-0.1,0.5,3.3,'EPSG','1027',0.085,0.531,-0.79,'EPSG','1032',-0.12,'EPSG','1030',2010.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'EUREF-Eur 2014',0);
INSERT INTO "usage" VALUES('EPSG','10818','helmert_transformation','EPSG','8876','EPSG','1298','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8877','ITRF91 to ETRF2014 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4913','EPSG','8401',0.0,-27.4,-15.5,76.8,'EPSG','1025',1.785,11.151,-16.43,'EPSG','1031',-4.49,'EPSG','1028',-0.1,0.5,3.3,'EPSG','1027',0.085,0.531,-0.79,'EPSG','1032',-0.12,'EPSG','1030',2010.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'EUREF-Eur 2014',0);
INSERT INTO "usage" VALUES('EPSG','10819','helmert_transformation','EPSG','8877','EPSG','1298','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8878','ITRF90 to ETRF2014 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4912','EPSG','8401',0.0,-25.4,-11.5,92.8,'EPSG','1025',1.785,11.151,-16.43,'EPSG','1031',-4.79,'EPSG','1028',-0.1,0.5,3.3,'EPSG','1027',0.085,0.531,-0.79,'EPSG','1032',-0.12,'EPSG','1030',2010.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'EUREF-Eur 2014',0);
INSERT INTO "usage" VALUES('EPSG','10820','helmert_transformation','EPSG','8878','EPSG','1298','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8879','ITRF89 to ETRF2014 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4911','EPSG','8401',0.0,-30.4,-35.5,130.8,'EPSG','1025',1.785,11.151,-16.43,'EPSG','1031',-8.19,'EPSG','1028',-0.1,0.5,3.3,'EPSG','1027',0.085,0.531,-0.79,'EPSG','1032',-0.12,'EPSG','1030',2010.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'EUREF-Eur 2014',0);
INSERT INTO "usage" VALUES('EPSG','10821','helmert_transformation','EPSG','8879','EPSG','1298','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8880','ITRF2014 to ETRF2014 (2)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9. See ITRF2014 to ETRF2014 (1) (code 8366) for transformation which defines ETRF2014. Transformation 8880 is equivalent to 8366 but with parameter values at epoch 2010.00.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','7789','EPSG','8401',0.0,0.0,0.0,0.0,'EPSG','1025',1.785,11.151,-16.17,'EPSG','1031',0.0,'EPSG','1028',0.0,0.0,0.0,'EPSG','1027',0.085,0.531,-0.77,'EPSG','1032',0.0,'EPSG','1030',2010.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'EUREF-Eur 2014',0);
INSERT INTO "usage" VALUES('EPSG','14204','helmert_transformation','EPSG','8880','EPSG','1298','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8882','Camacupa 2015 to WGS 84 (11)','Used by CIDDEMA for delimitation of Angola''s EEZ boundary. Derived by Univ. of Lisbon using 38 REPANGOL points. Average horizontal error 1m, vertical 3m; max radial error 6m. Application offshore differs from Camacupa 1948 to WGS 84 by approx 25m.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','8694','EPSG','4326',3.0,-93.799,-132.737,-219.073,'EPSG','9001',1.844,-0.648,6.37,'EPSG','9104',-0.169,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'CIDDEMA-Ago',0);
INSERT INTO "usage" VALUES('EPSG','10823','helmert_transformation','EPSG','8882','EPSG','1029','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','8883','Camacupa 1948 to RSAO13 (1)','Parameter values taken from Camacupa 1948 to WGS 84 (7) (code 1324) assuming that RSAO13 is coincident with WGS 84 within the accuracy of the transformation.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4220','EPSG','8699',3.0,-48.0,-345.0,-231.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IOGP-Ago B15',0);
INSERT INTO "usage" VALUES('EPSG','10824','helmert_transformation','EPSG','8883','EPSG','2322','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','8884','Camacupa 1948 to RSAO13 (2)','Parameter values taken from Camacupa 1948 to WGS 84 (10) (code 1327) assuming that RSAO13 is coincident with WGS 84 within the accuracy of the transformation.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4220','EPSG','8699',5.0,-50.9,-347.6,-231.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IOGP-Ago N',0);
INSERT INTO "usage" VALUES('EPSG','10825','helmert_transformation','EPSG','8884','EPSG','2324','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','8886','SVY21 to WGS 84 (1)','Considered exact at 1994-01-01 when SVY21 aligned to WGS 84 (Transit).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4757','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'SLA-sgp',0);
INSERT INTO "usage" VALUES('EPSG','10827','helmert_transformation','EPSG','8886','EPSG','1210','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','8887','GDA2020 to WGS 84 (Transit) (1)','Approximation at the 3m level assuming WGS 84 (Transit) is equivalent to ITRF2014 within the accuracy of the transformation.','EPSG','1031','Geocentric translations (geocentric domain)','EPSG','7842','EPSG','7815',3.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'GA-Aus 3m',0);
INSERT INTO "usage" VALUES('EPSG','10828','helmert_transformation','EPSG','8887','EPSG','4177','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','8889','BGS2005 to ETRS89 (1)','BGS2005 is a national realization of ETRS89. May be taken as approximate transformation BGS2005 to WGS 84 - see code 8890.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','7798','EPSG','4258',0.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IOGP-Bgr',0);
INSERT INTO "usage" VALUES('EPSG','10829','helmert_transformation','EPSG','8889','EPSG','1056','EPSG','1161');
INSERT INTO "helmert_transformation" VALUES('EPSG','8890','BGS2005 to WGS 84 (1)','Approximation at the +/- 1m level assuming that ETRS89 is equivalent to WGS 84. BGS2005 is a national realization of ETRS89.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','7798','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IOGP-Bgr',0);
INSERT INTO "usage" VALUES('EPSG','10830','helmert_transformation','EPSG','8890','EPSG','1056','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','8891','LKS-92 to ETRS89 (1)','LKS-92 is a national realization of ETRS89. May be taken as approximate transformation LKS92 to WGS 84 - see code 1958.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4661','EPSG','4258',0.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IOGP-Lva',0);
INSERT INTO "usage" VALUES('EPSG','10831','helmert_transformation','EPSG','8891','EPSG','1139','EPSG','1161');
INSERT INTO "helmert_transformation" VALUES('EPSG','8892','LKS94 to ETRS89 (1)','LKS94 is a national realization of ETRS89. May be taken as approximate transformation LKS94 to WGS 84 - see code 1283.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4669','EPSG','4258',0.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IOGP-Ltu',0);
INSERT INTO "usage" VALUES('EPSG','10832','helmert_transformation','EPSG','8892','EPSG','1145','EPSG','1161');
INSERT INTO "helmert_transformation" VALUES('EPSG','8893','SRB_ETRS89 to ETRS89 (1)','SRB_ETRS89 is a national realization of ETRS89. May be taken as approximate transformation SRB_ETRS89 to WGS 84 - see code 8894.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','8685','EPSG','4258',0.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IOGP-Sbr',0);
INSERT INTO "usage" VALUES('EPSG','10833','helmert_transformation','EPSG','8893','EPSG','4543','EPSG','1161');
INSERT INTO "helmert_transformation" VALUES('EPSG','8894','SRB_ETRS89 to WGS 84 (1)','Approximation at the +/- 1m level assuming that ETRS89 is equivalent to WGS 84. SRB_ETRS89 is a national realization of ETRS89.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','8685','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IOGP-Srb',0);
INSERT INTO "usage" VALUES('EPSG','10834','helmert_transformation','EPSG','8894','EPSG','4543','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','8895','CHTRS95 to ETRS89 (1)','CHTRS95 is a national realization of ETRS89. May be taken as approximate transformation CHTRS95 to WGS 84 - see code 1511.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4151','EPSG','4258',0.01,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IOGP-Che',0);
INSERT INTO "usage" VALUES('EPSG','10835','helmert_transformation','EPSG','8895','EPSG','1286','EPSG','1161');
INSERT INTO "helmert_transformation" VALUES('EPSG','8913','CR05 to CR-SIRGAS (1)','Derived at 16 stations. CR05 is static, CR-SIRGAS is dynamic: transformation is valid at epoch 2014.59 but accuracy will deteriorate due to tectonic motion. May be taken as an approximate transformation to ITRF08 / IGb08 / WGS 84 - see code 8914.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','5364','EPSG','8906',0.5,-0.16959,0.35312,0.51846,'EPSG','9001',-0.03385,0.16325,-0.03446,'EPSG','9104',0.03693,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Cri 2014',1);
INSERT INTO "usage" VALUES('EPSG','10836','helmert_transformation','EPSG','8913','EPSG','1074','EPSG','1035');
INSERT INTO "helmert_transformation" VALUES('EPSG','8914','CR05 to WGS 84 (2)','Parameter vales are from CR05 to CR-SIRGAS (1) (code 8913) assuming that CR-SIRGAS (ITRF08 (IGb08)@2014.59) is equivalent to WGS 84 within the accuracy of the transformation.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','5364','EPSG','4326',1.0,-0.16959,0.35312,0.51846,'EPSG','9001',-0.03385,0.16325,-0.03446,'EPSG','9104',0.03693,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Cri 2014',1);
INSERT INTO "usage" VALUES('EPSG','10837','helmert_transformation','EPSG','8914','EPSG','1074','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','8952','ITRF97 to SIRGAS-CON DGF00P01 (1)','Scale difference in ppb where 1/billion = 1E-9 or nm/m.','EPSG','1065','Time-specific Position Vector transform (geocen)','EPSG','4918','EPSG','8915',0.01,0.0,0.0,0.0,'EPSG','1025',0.0,0.0,0.0,'EPSG','1031',0.0,'EPSG','1028',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,2000.4,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'SIR-SC Am',0);
INSERT INTO "usage" VALUES('EPSG','10838','helmert_transformation','EPSG','8952','EPSG','4530','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8953','ITRF2000 to SIRGAS-CON DGF01P01 (1)','Scale difference in ppb where 1/billion = 1E-9 or nm/m.','EPSG','1065','Time-specific Position Vector transform (geocen)','EPSG','4919','EPSG','8917',0.01,0.0,0.0,0.0,'EPSG','1025',0.0,0.0,0.0,'EPSG','1031',0.0,'EPSG','1028',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,2000.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'SIR-SC Am',0);
INSERT INTO "usage" VALUES('EPSG','10839','helmert_transformation','EPSG','8953','EPSG','4530','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8954','ITRF2000 to SIRGAS-CON DGF01P02 (1)','Scale difference in ppb where 1/billion = 1E-9 or nm/m.','EPSG','1065','Time-specific Position Vector transform (geocen)','EPSG','4919','EPSG','8919',0.01,0.0,0.0,0.0,'EPSG','1025',0.0,0.0,0.0,'EPSG','1031',0.0,'EPSG','1028',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1998.4,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'SIR-SC Am',0);
INSERT INTO "usage" VALUES('EPSG','10840','helmert_transformation','EPSG','8954','EPSG','4530','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8955','ITRF2000 to SIRGAS-CON DGF02P01 (1)','Scale difference in ppb where 1/billion = 1E-9 or nm/m.','EPSG','1065','Time-specific Position Vector transform (geocen)','EPSG','4919','EPSG','8921',0.01,0.0,0.0,0.0,'EPSG','1025',0.0,0.0,0.0,'EPSG','1031',0.0,'EPSG','1028',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,2000.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'SIR-SC Am',0);
INSERT INTO "usage" VALUES('EPSG','10841','helmert_transformation','EPSG','8955','EPSG','4530','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8956','ITRF2000 to SIRGAS-CON DGF04P01 (1)','Scale difference in ppb where 1/billion = 1E-9 or nm/m.','EPSG','1065','Time-specific Position Vector transform (geocen)','EPSG','4919','EPSG','8923',0.01,0.0,0.0,0.0,'EPSG','1025',0.0,0.0,0.0,'EPSG','1031',0.0,'EPSG','1028',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,2003.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'SIR-SC Am',0);
INSERT INTO "usage" VALUES('EPSG','10842','helmert_transformation','EPSG','8956','EPSG','4530','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8957','ITRF2000 to SIRGAS-CON DGF05P01 (1)','Scale difference in ppb where 1/billion = 1E-9 or nm/m.','EPSG','1065','Time-specific Position Vector transform (geocen)','EPSG','4919','EPSG','8925',0.01,0.0,0.0,0.0,'EPSG','1025',0.0,0.0,0.0,'EPSG','1031',0.0,'EPSG','1028',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,2004.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'SIR-SC Am',0);
INSERT INTO "usage" VALUES('EPSG','10843','helmert_transformation','EPSG','8957','EPSG','4530','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8958','ITRF2000 to SIRGAS-CON DGF06P01 (1)','Scale difference in ppb where 1/billion = 1E-9 or nm/m.','EPSG','1065','Time-specific Position Vector transform (geocen)','EPSG','4919','EPSG','8927',0.01,0.0,0.0,0.0,'EPSG','1025',0.0,0.0,0.0,'EPSG','1031',0.0,'EPSG','1028',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,2004.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'SIR-SC Am',0);
INSERT INTO "usage" VALUES('EPSG','10844','helmert_transformation','EPSG','8958','EPSG','4530','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8959','IGS05 to SIRGAS-CON DGF07P01 (1)','Scale difference in ppb where 1/billion = 1E-9 or nm/m.','EPSG','1065','Time-specific Position Vector transform (geocen)','EPSG','9010','EPSG','8929',0.01,0.0,0.0,0.0,'EPSG','1025',0.0,0.0,0.0,'EPSG','1031',0.0,'EPSG','1028',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,2004.5,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'SIR-SC Am',0);
INSERT INTO "usage" VALUES('EPSG','10845','helmert_transformation','EPSG','8959','EPSG','4530','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8960','IGS05 to SIRGAS-CON DGF08P01 (1)','Scale difference in ppb where 1/billion = 1E-9 or nm/m.','EPSG','1065','Time-specific Position Vector transform (geocen)','EPSG','9010','EPSG','8931',0.01,0.0,0.0,0.0,'EPSG','1025',0.0,0.0,0.0,'EPSG','1031',0.0,'EPSG','1028',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,2004.5,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'SIR-SC Am',0);
INSERT INTO "usage" VALUES('EPSG','10846','helmert_transformation','EPSG','8960','EPSG','4530','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8961','IGS05 to SIRGAS-CON SIR09P01 (1)','Scale difference in ppb where 1/billion = 1E-9 or nm/m.','EPSG','1065','Time-specific Position Vector transform (geocen)','EPSG','9010','EPSG','8933',0.01,0.0,0.0,0.0,'EPSG','1025',0.0,0.0,0.0,'EPSG','1031',0.0,'EPSG','1028',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,2005.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'SIR-SC Am',0);
INSERT INTO "usage" VALUES('EPSG','10847','helmert_transformation','EPSG','8961','EPSG','4530','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8962','ITRF2008 to SIRGAS-CON SIR10P01 (1)','Scale difference in ppb where 1/billion = 1E-9 or nm/m.','EPSG','1065','Time-specific Position Vector transform (geocen)','EPSG','5332','EPSG','8935',0.01,0.0,0.0,0.0,'EPSG','1025',0.0,0.0,0.0,'EPSG','1031',0.0,'EPSG','1028',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,2005.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'SIR-SC Am',0);
INSERT INTO "usage" VALUES('EPSG','10848','helmert_transformation','EPSG','8962','EPSG','4530','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8963','ITRF2008 to SIRGAS-CON SIR11P01 (1)','Scale difference in ppb where 1/billion = 1E-9 or nm/m.','EPSG','1065','Time-specific Position Vector transform (geocen)','EPSG','5332','EPSG','8937',0.01,0.0,0.0,0.0,'EPSG','1025',0.0,0.0,0.0,'EPSG','1031',0.0,'EPSG','1028',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,2005.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'SIR-SC Am',0);
INSERT INTO "usage" VALUES('EPSG','10849','helmert_transformation','EPSG','8963','EPSG','4530','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8964','IGb08 to SIRGAS-CON SIR13P01 (1)','Scale difference in ppb where 1/billion = 1E-9 or nm/m.','EPSG','1065','Time-specific Position Vector transform (geocen)','EPSG','9015','EPSG','8939',0.01,0.0,0.0,0.0,'EPSG','1025',0.0,0.0,0.0,'EPSG','1031',0.0,'EPSG','1028',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,2012.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'SIR-SC Am',0);
INSERT INTO "usage" VALUES('EPSG','10850','helmert_transformation','EPSG','8964','EPSG','4530','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8965','IGb08 to SIRGAS-CON SIR14P01 (1)','Scale difference in ppb where 1/billion = 1E-9 or nm/m.','EPSG','1065','Time-specific Position Vector transform (geocen)','EPSG','9015','EPSG','8941',0.01,0.0,0.0,0.0,'EPSG','1025',0.0,0.0,0.0,'EPSG','1031',0.0,'EPSG','1028',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,2013.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'SIR-SC Am',0);
INSERT INTO "usage" VALUES('EPSG','10851','helmert_transformation','EPSG','8965','EPSG','4530','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8966','IGb08 to SIRGAS-CON SIR15P01 (1)','Scale difference in ppb where 1/billion = 1E-9 or nm/m.','EPSG','1065','Time-specific Position Vector transform (geocen)','EPSG','9015','EPSG','8943',0.01,0.0,0.0,0.0,'EPSG','1025',0.0,0.0,0.0,'EPSG','1031',0.0,'EPSG','1028',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,2013.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'SIR-SC Am',0);
INSERT INTO "usage" VALUES('EPSG','10852','helmert_transformation','EPSG','8966','EPSG','4530','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8967','IGS14 to SIRGAS-CON SIR17P01 (1)','Scale difference in ppb where 1/billion = 1E-9 or nm/m.','EPSG','1065','Time-specific Position Vector transform (geocen)','EPSG','8227','EPSG','8945',0.01,0.0,0.0,0.0,'EPSG','1025',0.0,0.0,0.0,'EPSG','1031',0.0,'EPSG','1028',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,2015.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'SIR-SC Am',0);
INSERT INTO "usage" VALUES('EPSG','10853','helmert_transformation','EPSG','8967','EPSG','4530','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','8968','CR05 to CR-SIRGAS (1)','Derived at 16 stations. CR05 is static, CR-SIRGAS is dynamic: transformation is valid at epoch 2014.59 but accuracy will deteriorate due to tectonic motion. May be taken as an approximate transformation to ITRF08 / IGb08 / WGS 84 - see code 8969.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','5365','EPSG','8907',0.5,-0.16959,0.35312,0.51846,'EPSG','9001',-0.03385,0.16325,-0.03446,'EPSG','9104',0.03693,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Cri 2014',1);
INSERT INTO "usage" VALUES('EPSG','10854','helmert_transformation','EPSG','8968','EPSG','1074','EPSG','1035');
INSERT INTO "helmert_transformation" VALUES('EPSG','8969','CR05 to WGS 84 (2)','Parameter vales are from CR05 to CR-SIRGAS (1) (code 8968) assuming that CR-SIRGAS (ITRF08 (IGb08)@2014.59) is equivalent to WGS 84 within the accuracy of the transformation.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','5365','EPSG','4326',1.0,-0.16959,0.35312,0.51846,'EPSG','9001',-0.03385,0.16325,-0.03446,'EPSG','9104',0.03693,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Cri 2014',1);
INSERT INTO "usage" VALUES('EPSG','10855','helmert_transformation','EPSG','8969','EPSG','1074','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','8970','ITRF2014 to NAD83(2011) (1)','Concatenation of joint US-Canada tfm NAD83(CORS96)>ITRF96 (CT code 6864) with IGS value for ITRF96>ITRF97 and IERS values for ITRF97>ITRF2014 CTs. Scale difference in ppb and rate in ppb/yr where 1/billion = 1E-9. See tfm 8265 for Canadian equivalent.','EPSG','1056','Time-dependent Coordinate Frame rotation (geocen)','EPSG','7789','EPSG','6317',0.0,1.0053,-1.90921,-0.54157,'EPSG','9001',26.78138,-0.42027,10.93206,'EPSG','1031',0.36891,'EPSG','1028',0.00079,-0.0006,-0.00144,'EPSG','1042',0.06667,-0.75744,-0.05133,'EPSG','1032',-0.07201,'EPSG','1030',2010.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'NGS-Usa CONUS-AK PRVI',0);
INSERT INTO "usage" VALUES('EPSG','10856','helmert_transformation','EPSG','8970','EPSG','1511','EPSG','1026');
INSERT INTO "helmert_transformation" VALUES('EPSG','8971','NAD83 to NAD83(2011) (1)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4269','EPSG','6318',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IOGP-Usa GoM',0);
INSERT INTO "usage" VALUES('EPSG','10857','helmert_transformation','EPSG','8971','EPSG','3357','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','9020','ITRF88 to ITRF89 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m. Superseded by parameter values derived from transformations to ITRF2020 (change in scale difference), see EPSG:10139.','EPSG','1033','Position Vector transformation (geocentric domain)','EPSG','4910','EPSG','4911',0.01,0.5,3.6,2.4,'EPSG','1033',-0.1,0.0,0.0,'EPSG','1031',-3.0,'EPSG','1028',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IERS-Wld',0);
INSERT INTO "usage" VALUES('EPSG','10858','helmert_transformation','EPSG','9020','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','9021','ITRF89 to ITRF90 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m. Superseded by parameter values derived from transformations to ITRF2020 (change in scale difference), see EPSG:10140.','EPSG','1033','Position Vector transformation (geocentric domain)','EPSG','4911','EPSG','4912',0.01,-0.5,-2.4,3.8,'EPSG','1033',0.0,0.0,0.0,'EPSG','1031',-3.0,'EPSG','1028',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IERS-Wld',0);
INSERT INTO "usage" VALUES('EPSG','10859','helmert_transformation','EPSG','9021','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','9022','ITRF90 to ITRF91 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m. Superseded by parameter values derived from transformations to ITRF2020 (change in tX), see EPSG:10141.','EPSG','1033','Position Vector transformation (geocentric domain)','EPSG','4912','EPSG','4913',0.007,-0.1,0.4,1.6,'EPSG','1033',0.0,0.0,0.0,'EPSG','1031',-0.3,'EPSG','1028',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IERS-Wld',0);
INSERT INTO "usage" VALUES('EPSG','10860','helmert_transformation','EPSG','9022','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','9023','ITRF91 to ITRF92 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m. Superseded by parameter values derived from transformations to ITRF2020 (change in tX), see EPSG:10142.','EPSG','1033','Position Vector transformation (geocentric domain)','EPSG','4913','EPSG','4914',0.005,-1.1,-1.4,0.6,'EPSG','1033',0.0,0.0,0.0,'EPSG','1031',-1.4,'EPSG','1028',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IERS-Wld',0);
INSERT INTO "usage" VALUES('EPSG','10861','helmert_transformation','EPSG','9023','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','9024','ITRF92 to ITRF93 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m. Note: info source gives translation rates in mm/year.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4914','EPSG','4915',0.003,-0.2,-0.7,-0.7,'EPSG','1033',-0.39,0.8,-0.96,'EPSG','1031',1.2,'EPSG','1028',-0.29,0.04,0.08,'EPSG','1034',-0.11,-0.19,0.05,'EPSG','1032',0.0,'EPSG','1030',1988.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IERS-Wld',0);
INSERT INTO "usage" VALUES('EPSG','10862','helmert_transformation','EPSG','9024','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','9025','ITRF93 to ITRF94 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m. Superseded by parameter values derived from transformations to ITRF2020 (change in scale difference), see EPSG:10143.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4915','EPSG','4916',0.01,-0.6,0.5,1.5,'EPSG','1033',0.39,-0.8,0.96,'EPSG','1031',-0.4,'EPSG','1028',0.29,-0.04,-0.08,'EPSG','1034',0.11,0.19,-0.05,'EPSG','1032',0.0,'EPSG','1030',1988.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IERS-Wld',0);
INSERT INTO "usage" VALUES('EPSG','10863','helmert_transformation','EPSG','9025','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','9026','ITRF94 to ITRF96 (1)','Scale difference in ppb where 1/billion = 1E-9 or nm/m. ITRF96 is by definition aligned with ITRF94.','EPSG','1065','Time-specific Position Vector transform (geocen)','EPSG','4916','EPSG','4917',0.0,0.0,0.0,0.0,'EPSG','1033',0.0,0.0,0.0,'EPSG','1031',0.0,'EPSG','1028',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1988.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IERS-Wld',0);
INSERT INTO "usage" VALUES('EPSG','10864','helmert_transformation','EPSG','9026','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','9027','ITRF96 to ITRF97 (1)','Scale difference in ppb where 1/billion = 1E-9 or nm/m. ITRF97 is by definition aligned with ITRF96.','EPSG','1065','Time-specific Position Vector transform (geocen)','EPSG','4917','EPSG','4918',0.0,0.0,0.0,0.0,'EPSG','1033',0.0,0.0,0.0,'EPSG','1031',0.0,'EPSG','1028',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1988.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IERS-Wld',0);
INSERT INTO "usage" VALUES('EPSG','10865','helmert_transformation','EPSG','9027','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','9028','ITRF97 to IGS97 (1)','Scale difference in ppb where 1/billion = 1E-9 or nm/m. IGS97 is by definition aligned with ITRF97.','EPSG','1065','Time-specific Position Vector transform (geocen)','EPSG','4918','EPSG','9001',0.0,0.0,0.0,0.0,'EPSG','1033',0.0,0.0,0.0,'EPSG','1031',0.0,'EPSG','1028',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1997.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IGS-Wld',0);
INSERT INTO "usage" VALUES('EPSG','10866','helmert_transformation','EPSG','9028','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','9029','ITRF2000 to IGS00 (1)','Scale difference in ppb where 1/billion = 1E-9 or nm/m. IGS00 is by definition aligned with ITRF2000.','EPSG','1065','Time-specific Position Vector transform (geocen)','EPSG','4919','EPSG','9004',0.0,0.0,0.0,0.0,'EPSG','1033',0.0,0.0,0.0,'EPSG','1031',0.0,'EPSG','1028',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1998.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IGS-Wld',0);
INSERT INTO "usage" VALUES('EPSG','10867','helmert_transformation','EPSG','9029','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','9030','ITRF2005 to IGS05 (1)','Scale difference in ppb where 1/billion = 1E-9 or nm/m. IGS05 is by definition aligned with ITRF2005.','EPSG','1065','Time-specific Position Vector transform (geocen)','EPSG','4896','EPSG','9010',0.0,0.0,0.0,0.0,'EPSG','1025',0.0,0.0,0.0,'EPSG','1031',0.0,'EPSG','1028',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,2000.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IGS-Wld',0);
INSERT INTO "usage" VALUES('EPSG','10868','helmert_transformation','EPSG','9030','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','9031','ITRF2008 to IGS08 (1)','Scale difference in ppb where 1/billion = 1E-9 or nm/m. IGS08 is by definition aligned with ITRF2008.','EPSG','1065','Time-specific Position Vector transform (geocen)','EPSG','5332','EPSG','6934',0.0,0.0,0.0,0.0,'EPSG','1033',0.0,0.0,0.0,'EPSG','1031',0.0,'EPSG','1028',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,2005.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IGS-Wld',0);
INSERT INTO "usage" VALUES('EPSG','10869','helmert_transformation','EPSG','9031','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','9032','ITRF2014 to IGS14 (1)','Scale difference in ppb where 1/billion = 1E-9 or nm/m. IGS14 is by definition aligned with ITRF2014.','EPSG','1065','Time-specific Position Vector transform (geocen)','EPSG','7789','EPSG','8227',0.0,0.0,0.0,0.0,'EPSG','1033',0.0,0.0,0.0,'EPSG','1031',0.0,'EPSG','1028',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,2010.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IGS-Wld',0);
INSERT INTO "usage" VALUES('EPSG','10870','helmert_transformation','EPSG','9032','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','9033','IGS97 to IGS00 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','9001','EPSG','9004',0.007,-6.0,-5.6,20.1,'EPSG','1025',-0.04,0.001,0.043,'EPSG','1031',-1.403,'EPSG','1028',0.4,0.8,1.5,'EPSG','1027',0.004,-0.001,-0.003,'EPSG','1032',-0.012,'EPSG','1030',1998.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IGS-Wld',0);
INSERT INTO "usage" VALUES('EPSG','10871','helmert_transformation','EPSG','9033','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','9034','IGS00 to IGb00 (1)','Scale difference in ppb where 1/billion = 1E-9 or nm/m. IGS00 and IGb00 are both by definition aligned with ITRF2000. The actual IGS00-IGb00 transformation parameter values are not null but are statistically insignificant and treated as null by IGS.','EPSG','1065','Time-specific Position Vector transform (geocen)','EPSG','9004','EPSG','9007',0.0,0.0,0.0,0.0,'EPSG','1025',0.0,0.0,0.0,'EPSG','1031',0.0,'EPSG','1028',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1998.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IGS-Wld',0);
INSERT INTO "usage" VALUES('EPSG','10872','helmert_transformation','EPSG','9034','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','9035','IGb00 to IGS05 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','9007','EPSG','9010',0.001,0.0,1.7,5.3,'EPSG','1025',0.0224,-0.0341,0.0099,'EPSG','1031',-0.8473,'EPSG','1028',0.4,-0.7,1.8,'EPSG','1027',-0.0033,0.0001,0.0161,'EPSG','1032',-0.1748,'EPSG','1030',2000.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IGS-Wld',0);
INSERT INTO "usage" VALUES('EPSG','10873','helmert_transformation','EPSG','9035','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','9036','IGS05 to IGS08 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','9010','EPSG','6934',0.001,1.5,0.0,5.8,'EPSG','1025',-0.012,0.014,0.014,'EPSG','1031',-1.04,'EPSG','1028',-0.1,0.0,-0.1,'EPSG','1027',-0.002,-0.003,0.001,'EPSG','1032',0.01,'EPSG','1030',2005.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IGS-Wld',0);
INSERT INTO "usage" VALUES('EPSG','10874','helmert_transformation','EPSG','9036','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','9037','IGS08 to IGb08 (1)','Scale difference in ppb where 1/billion = 1E-9 or nm/m. IGS08 and IGb08 are both by definition aligned with ITRF2008.','EPSG','1065','Time-specific Position Vector transform (geocen)','EPSG','6934','EPSG','9015',0.0,0.0,0.0,0.0,'EPSG','1025',0.0,0.0,0.0,'EPSG','1031',0.0,'EPSG','1028',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,2005.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IGS-Wld',0);
INSERT INTO "usage" VALUES('EPSG','10875','helmert_transformation','EPSG','9037','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','9038','IGb08 to IGS14 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m. Parameter values from ITRF2008 to ITRF2014 (1) (code 7790) as IGb08 is aligned to ITRF2008 and IGS14 is aligned to ITRF2014.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','9015','EPSG','8227',0.01,-1.6,-1.9,-2.4,'EPSG','1025',0.0,0.0,0.0,'EPSG','1031',0.02,'EPSG','1028',0.0,0.0,0.1,'EPSG','1027',0.0,0.0,0.0,'EPSG','1032',-0.03,'EPSG','1030',2010.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IGS-Wld',0);
INSERT INTO "usage" VALUES('EPSG','10876','helmert_transformation','EPSG','9038','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','9051','ITRF94 to SIRGAS 1995 (1)','Scale difference in ppb where 1/billion = 1E-9 or nm/m.','EPSG','1065','Time-specific Position Vector transform (geocen)','EPSG','4916','EPSG','4974',0.01,0.0,0.0,0.0,'EPSG','1025',0.0,0.0,0.0,'EPSG','1031',0.0,'EPSG','1028',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1995.4,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'SIR-S Am',0);
INSERT INTO "usage" VALUES('EPSG','10887','helmert_transformation','EPSG','9051','EPSG','3448','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','9052','ITRF2000 to SIRGAS 2000 (1)','Scale difference in ppb where 1/billion = 1E-9 or nm/m.','EPSG','1065','Time-specific Position Vector transform (geocen)','EPSG','4919','EPSG','4988',0.01,0.0,0.0,0.0,'EPSG','1025',0.0,0.0,0.0,'EPSG','1031',0.0,'EPSG','1028',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,2000.4,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'SIR-SC Am',0);
INSERT INTO "usage" VALUES('EPSG','10888','helmert_transformation','EPSG','9052','EPSG','3418','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','9076','WGS 84 (G873) to ITRF94 (1)','Defined at epoch 1997.0.','EPSG','1032','Coordinate Frame rotation (geocentric domain)','EPSG','7658','EPSG','4916',0.1,0.0,0.0,0.0,'EPSG','9001',0.0,0.0,0.0,'EPSG','9104',0.0,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NGA-Wld 1997.0',0);
INSERT INTO "usage" VALUES('EPSG','10890','helmert_transformation','EPSG','9076','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','9077','ITRF2000 to NAD83(MARP00) (1)','Defines NAD83(MARP00). Equivalent transformation published on NGS web site and used in HDTP with rX=28.971 mas, rY=10.420 mas and rZ=8.928 mas at epoch 1997.00.','EPSG','1056','Time-dependent Coordinate Frame rotation (geocen)','EPSG','4919','EPSG','9070',0.0,0.9102,-2.0141,-0.5602,'EPSG','9001',29.039,10.065,10.101,'EPSG','1031',0.0,'EPSG','1028',0.0,0.0,0.0,'EPSG','1042',-0.02,0.105,-0.347,'EPSG','1032',0.0,'EPSG','1030',1993.62,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'NGS-MA',0);
INSERT INTO "usage" VALUES('EPSG','10891','helmert_transformation','EPSG','9077','EPSG','4167','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','9078','ITRF2000 to NAD83(PACP00) (1)','Defines NAD83(PACP00). Equivalent transformation published on NGS web site and used in HDTP with rX=27.741 mas, rY=13.469 mas and rZ=2.712 mas at epoch 1997.00.','EPSG','1056','Time-dependent Coordinate Frame rotation (geocen)','EPSG','4919','EPSG','9073',0.0,0.9102,-2.0141,-0.5602,'EPSG','9001',29.039,10.065,10.101,'EPSG','1031',0.0,'EPSG','1028',0.0,0.0,0.0,'EPSG','1042',-0.384,1.007,-2.186,'EPSG','1032',0.0,'EPSG','1030',1993.62,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'NGS-PA',0);
INSERT INTO "usage" VALUES('EPSG','10892','helmert_transformation','EPSG','9078','EPSG','4162','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','9079','ITRF97 to ITRF96 (2)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m. Uses IGS determination. Used as first step in ITRF97 to NZGD2000 concatenated operations, followed by application of NZGD2000 deformation model.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4918','EPSG','4917',0.01,0.0,-0.51,15.53,'EPSG','1025',-0.16508,0.26897,0.05984,'EPSG','1031',-1.51099,'EPSG','1028',0.69,-0.1,1.86,'EPSG','1027',-0.01347,0.01514,-0.00027,'EPSG','1032',-0.19201,'EPSG','1030',2000.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'LINZ-Nzl 97',0);
INSERT INTO "usage" VALUES('EPSG','10893','helmert_transformation','EPSG','9079','EPSG','1175','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','9080','ITRF2000 to ITRF96 (2)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m. Used as first step in ITRF2000 to NZGD2000 concatenated operations, followed by application of NZGD2000 deformation model.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4919','EPSG','4917',0.01,6.7,3.79,-7.17,'EPSG','1025',-0.16508,0.26897,0.11984,'EPSG','1031',0.06901,'EPSG','1028',0.69,-0.7,0.46,'EPSG','1027',-0.01347,0.01514,0.01973,'EPSG','1032',-0.18201,'EPSG','1030',2000.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'LINZ-Nzl 2000',0);
INSERT INTO "usage" VALUES('EPSG','10894','helmert_transformation','EPSG','9080','EPSG','1175','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','9081','ITRF2005 to ITRF96 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m. Used as first step in ITRF2005 to NZGD2000 concatenated operations, followed by application of NZGD2000 deformation model.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4896','EPSG','4917',0.01,6.8,2.99,-12.97,'EPSG','1025',-0.16508,0.26897,0.11984,'EPSG','1031',0.46901,'EPSG','1028',0.49,-0.6,-1.34,'EPSG','1027',-0.01347,0.01514,0.01973,'EPSG','1032',-0.10201,'EPSG','1030',2000.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'LINZ-Nzl 2005',0);
INSERT INTO "usage" VALUES('EPSG','10895','helmert_transformation','EPSG','9081','EPSG','1175','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','9082','ITRF2008 to ITRF96 (2)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m. Used as first step in ITRF2008 to NZGD2000 concatenated operations, followed by application of NZGD2000 deformation model.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','5332','EPSG','4917',0.01,4.8,2.09,-17.67,'EPSG','1025',-0.16508,0.26897,0.11984,'EPSG','1031',1.40901,'EPSG','1028',0.79,-0.6,-1.34,'EPSG','1027',-0.01347,0.01514,0.01973,'EPSG','1032',-0.10201,'EPSG','1030',2000.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'LINZ-Nzl 2008',0);
INSERT INTO "usage" VALUES('EPSG','10896','helmert_transformation','EPSG','9082','EPSG','1175','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','9083','ITRF2014 to ITRF96 (2)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m. Used as first step in ITRF2014 to NZGD2000 concatenated operations, followed by application of NZGD2000 deformation model.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','7789','EPSG','4917',0.01,6.4,3.99,-14.27,'EPSG','1025',-0.16508,0.26897,0.11984,'EPSG','1031',1.08901,'EPSG','1028',0.79,-0.6,-1.44,'EPSG','1027',-0.01347,0.01514,0.01973,'EPSG','1032',-0.07201,'EPSG','1030',2000.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'LINZ-Nzl 2014',0);
INSERT INTO "usage" VALUES('EPSG','10897','helmert_transformation','EPSG','9083','EPSG','1175','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','9126','NAD83(CSRS)v2 to NAD83(CORS96) (1)','Scale difference in ppb where 1/billion = 1E-9 or nm/m. Source and target CRSs defined from ITRF96 by common transformations (codes 6864 and 8259). 6864 defines CORS96 from 1st January 1997 to 31st December 1999.','EPSG','1033','Position Vector transformation (geocentric domain)','EPSG','8233','EPSG','6781',0.0,0.0,0.0,0.0,'EPSG','9001',0.0,0.0,0.0,'EPSG','1031',0.0,'EPSG','1028',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'ISO-N Am',0);
INSERT INTO "usage" VALUES('EPSG','10940','helmert_transformation','EPSG','9126','EPSG','4544','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','9127','NAD83(CSRS)v3 to NAD83(CORS96) (1)','Scale difference in ppb where 1/billion = 1E-9 or nm/m. Source and target CRSs defined from ITRF97 by common transformations (codes 6865 and 8260). 6865 defines CORS96 from 1st January 2000 to 31st December 2001.','EPSG','1033','Position Vector transformation (geocentric domain)','EPSG','8238','EPSG','6781',0.0,0.0,0.0,0.0,'EPSG','9001',0.0,0.0,0.0,'EPSG','1031',0.0,'EPSG','1028',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'ISO-N Am',0);
INSERT INTO "usage" VALUES('EPSG','10941','helmert_transformation','EPSG','9127','EPSG','4544','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','9128','NAD83(CSRS)v4 to NAD83(CORS96) (1)','Scale difference in ppb where 1/billion = 1E-9 or nm/m. Source and target CRSs defined from ITRF2000 by common transformations (codes 6866 and 8261). 6866 defines CORS96 from 1st January 2002 to 6th September 2011.','EPSG','1033','Position Vector transformation (geocentric domain)','EPSG','8242','EPSG','6781',0.0,0.0,0.0,0.0,'EPSG','9001',0.0,0.0,0.0,'EPSG','1031',0.0,'EPSG','1028',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'ISO-N Am',0);
INSERT INTO "usage" VALUES('EPSG','10942','helmert_transformation','EPSG','9128','EPSG','4544','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','9129','NAD83(CSRS)v6 to NAD83(2011) (1)','Scale difference in ppb where 1/billion = 1E-9 or nm/m. Source and target CRSs defined from ITRF2008 by common transformations (codes 7807 and 8264).','EPSG','1033','Position Vector transformation (geocentric domain)','EPSG','8250','EPSG','6317',0.0,0.0,0.0,0.0,'EPSG','9001',0.0,0.0,0.0,'EPSG','1031',0.0,'EPSG','1028',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'ISO-N Am',0);
INSERT INTO "usage" VALUES('EPSG','10943','helmert_transformation','EPSG','9129','EPSG','4544','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','9142','MGI 1901 to KOSOVAREF01 (1)','Derived at 18 points across the area of Kosovo. May be taken as approximate transformation MGI 1901 to WGS 84 (see code 9143).','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','3906','EPSG','9140',1.0,628.54052,192.2538,498.43507,'EPSG','9001',-13.79189,-0.81467,41.21533,'EPSG','9104',-17.40368,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'KCA-Kos',0);
INSERT INTO "usage" VALUES('EPSG','10951','helmert_transformation','EPSG','9142','EPSG','4542','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','9143','MGI 1901 to WGS 84 (14)','Parameter values from MGI 1901 to KOSOVAREF01 (1) (code 9142). Assumes KOSOVAREF01 and WGS 84 can be considered the same to within the accuracy of the transformation.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','3906','EPSG','4326',1.0,628.54052,192.2538,498.43507,'EPSG','9001',-13.79189,-0.81467,41.21533,'EPSG','9104',-17.40368,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IOGP-Kos',0);
INSERT INTO "usage" VALUES('EPSG','10952','helmert_transformation','EPSG','9143','EPSG','4542','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','9144','KOSOVAREF01 to WGS 84 (1)','Approximation at the +/- 1m level assuming that ETRS89 is equivalent to WGS 84. KOSOVAREF01 is a national realization of ETRS89.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','9140','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IOGP-Kos',0);
INSERT INTO "usage" VALUES('EPSG','10953','helmert_transformation','EPSG','9144','EPSG','4542','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','9145','WGS 84 (Transit) to ITRF90 (1)','','EPSG','1033','Position Vector transformation (geocentric domain)','EPSG','7815','EPSG','4912',1.0,-0.06,0.517,0.223,'EPSG','9001',-0.0183,0.0003,-0.007,'EPSG','9104',0.011,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IERS-Wld',0);
INSERT INTO "usage" VALUES('EPSG','10954','helmert_transformation','EPSG','9145','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','9177','SIRGAS-Chile 2002 to ITRF2000 (1)','Transformation is exact at epoch 2002.00 but accuracy then decreasing with time.','EPSG','1065','Time-specific Position Vector transform (geocen)','EPSG','5358','EPSG','4919',0.1,0.0,0.0,0.0,'EPSG','1025',0.0,0.0,0.0,'EPSG','1031',0.0,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,2002.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IOGP-Chl',0);
INSERT INTO "usage" VALUES('EPSG','10972','helmert_transformation','EPSG','9177','EPSG','1066','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','9178','SIRGAS-Chile 2010 to IGS08 (1)','Transformation is exact at epoch 2010.00 but accuracy then decreasing with time.','EPSG','1065','Time-specific Position Vector transform (geocen)','EPSG','8947','EPSG','6934',0.1,0.0,0.0,0.0,'EPSG','1025',0.0,0.0,0.0,'EPSG','1031',0.0,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,2010.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IOGP-Chl',0);
INSERT INTO "usage" VALUES('EPSG','10973','helmert_transformation','EPSG','9178','EPSG','1066','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','9179','SIRGAS-Chile 2013 to IGb08 (1)','Transformation is exact at epoch 2013.00 but accuracy then decreasing with time.','EPSG','1065','Time-specific Position Vector transform (geocen)','EPSG','9146','EPSG','9015',0.1,0.0,0.0,0.0,'EPSG','1025',0.0,0.0,0.0,'EPSG','1031',0.0,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,2013.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IOGP-Chl',0);
INSERT INTO "usage" VALUES('EPSG','10974','helmert_transformation','EPSG','9179','EPSG','1066','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','9180','SIRGAS-Chile 2016 to IGb08 (1)','Transformation is exact at epoch 2016.00 but accuracy then decreasing with time.','EPSG','1065','Time-specific Position Vector transform (geocen)','EPSG','9151','EPSG','9015',0.1,0.0,0.0,0.0,'EPSG','1025',0.0,0.0,0.0,'EPSG','1031',0.0,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,2016.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IOGP-Chl',0);
INSERT INTO "usage" VALUES('EPSG','10975','helmert_transformation','EPSG','9180','EPSG','1066','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','9185','AGD66 to GDA2020 (1)','This transformation is for users wanting to apply a 7-parameter conformal transformation from AGD66 to GDA2020 in the ACT.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4202','EPSG','7844',0.05,-136.9703,-37.5638,124.4242,'EPSG','9001',-0.25676,-0.42966,-0.30077,'EPSG','9104',-4.61966,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPD-Aus ACT',0);
INSERT INTO "usage" VALUES('EPSG','10977','helmert_transformation','EPSG','9185','EPSG','2283','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','9186','ITRF2008 to IG05/12 Intermediate CRS','Derived from Israeli CORS on ITRF2008 at epoch 2018.50. Updates CT 6993 for approx. 40cm tectonic plate motion 2012 to 2019 for GIS purposes. CT 6993 remains the legal definition of IG05/12 and must be used for cadastral and precise engineering purposes.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','8999','EPSG','6990',0.05,-23.772,-17.49,-17.859,'EPSG','9001',-0.3132,-1.85274,1.67299,'EPSG','9104',5.4262,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'SOI-Isr 2019',0);
INSERT INTO "usage" VALUES('EPSG','10978','helmert_transformation','EPSG','9186','EPSG','2603','EPSG','1189');
INSERT INTO "helmert_transformation" VALUES('EPSG','9189','WGS 84 to IG05/12 Intermediate CRS','Parameter values from CT code 9186 assuming that WGS 84 is coincident with ITRF2008. Updates CT 6993 for approx 40cm tectonic plate motion for GIS purposes. CT 6993 remains the legal definition of IG05/12 and must be used for cadastre and engineering.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4326','EPSG','6990',0.1,-23.772,-17.49,-17.859,'EPSG','9001',-0.3132,-1.85274,1.67299,'EPSG','9104',5.4262,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'SOI-Isr 2019',0);
INSERT INTO "usage" VALUES('EPSG','10981','helmert_transformation','EPSG','9189','EPSG','2603','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','9223','WGS 84 to ETRS89 (2)','Parameter values derived from ITRF2008 to ETRF2000 (1), code 7951, as a time-specific transformation @2014.81, assuming ITRF2008 = WGS 84 (G1762) = WGS 84 and ETRF2000 = ETRS89.','EPSG','1065','Time-specific Position Vector transform (geocen)','EPSG','4936','EPSG','4978',0.1,0.054,0.051,-0.085,'EPSG','9001',0.0021,0.0126,-0.0204,'EPSG','9104',0.0025,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,2014.81,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'ONE-Deu/Nld 2014.81',1);
INSERT INTO "usage" VALUES('EPSG','10998','helmert_transformation','EPSG','9223','EPSG','4566','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','9224','ED50 to ETRS89 (15)','Parameter values from ED50 to WGS 84 (36) (CT 1998) assuming that ETRS89 is equivalent to WGS 84 at epoch 1989.00.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4230','EPSG','4258',1.0,-157.89,-17.16,-78.41,'EPSG','9001',2.118,2.697,-1.434,'EPSG','9104',-5.38,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'ONE-Ger Nsea',0);
INSERT INTO "usage" VALUES('EPSG','10999','helmert_transformation','EPSG','9224','EPSG','4566','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','9225','WGS 84 to ETRS89 (2)','Parameter values derived from ITRF2008 to ETRF2000 (1), code 7951, as a time-specific transformation @2014.81, assuming ITRF2008 = WGS 84 (G1762) = WGS 84 and ETRF2000 = ETRS89.','EPSG','1065','Time-specific Position Vector transform (geocen)','EPSG','4978','EPSG','4936',0.1,0.054,0.051,-0.085,'EPSG','9001',0.0021,0.0126,-0.0204,'EPSG','9104',0.0025,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,2014.81,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'ONE-Deu/Nld 2014.81',0);
INSERT INTO "usage" VALUES('EPSG','11000','helmert_transformation','EPSG','9225','EPSG','4566','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','9226','SHGD2015 to Astro DOS 71 (2)','Derived at 19 stations, RMS = 6cm. Note: Because of the large rotations about the Y- and Z-axes this transformation is not reversible. For the reverse transformation use Astro DOS 71 to SHGD2015 (2) (code 7895).','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','7886','EPSG','4710',0.1,112.771,-12.282,18.935,'EPSG','9001',-2.1692,-16.8896,-17.1961,'EPSG','9104',19.54517,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'ENRD-Shn Hel 0.1m Rev',0);
INSERT INTO "usage" VALUES('EPSG','13866','helmert_transformation','EPSG','9226','EPSG','3183','EPSG','1031');
INSERT INTO "helmert_transformation" VALUES('EPSG','9227','ITRF2005 to NAD83(CSRS)v5 (1)','Concatenation of joint Canada-US NAD83>ITRF96 transformation (code 8259) with IGS value for ITRF96>ITRF97 and IERS transformations ITRF97>ITRF2005. Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4896','EPSG','8247',0.0,0.9963,-1.9024,-0.5219,'EPSG','9001',-25.915,-9.426,-11.599,'EPSG','1031',0.775,'EPSG','1028',0.0005,-0.0006,-0.0013,'EPSG','1042',-0.067,0.757,0.051,'EPSG','1032',-0.102,'EPSG','1030',1997.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'NRC-Can CSRSv5',0);
INSERT INTO "usage" VALUES('EPSG','13867','helmert_transformation','EPSG','9227','EPSG','1061','EPSG','1170');
INSERT INTO "helmert_transformation" VALUES('EPSG','9234','Kalianpur 1962 to WGS 84 (5)','Derived by Western Geophysical for UTP 1996 East Sind 2D survey. Very similar parameter values (higher resolution but no better accuracy) were used by BGP for the ENI 2006 East Sind 2D survey.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4145','EPSG','4326',3.0,230.25,632.76,161.03,'EPSG','9001',-1.114,1.115,1.212,'EPSG','9104',12.584,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'utp-Pak E Sind',0);
INSERT INTO "usage" VALUES('EPSG','13923','helmert_transformation','EPSG','9234','EPSG','2983','EPSG','1216');
INSERT INTO "helmert_transformation" VALUES('EPSG','9257','Chos Malal 1914 to WGS 84 (2)','Derived through common coordinates at 13 stations.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4160','EPSG','4326',2.5,8.88,184.86,106.69,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'YPF-Arg Cuyo',0);
INSERT INTO "usage" VALUES('EPSG','13925','helmert_transformation','EPSG','9257','EPSG','4561','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','9258','Chos Malal 1914 to WGS 84 (3)','Derived through common coordinates at 43 stations.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4160','EPSG','4326',2.5,15.75,164.93,126.18,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'YPF-Arg Neuq',0);
INSERT INTO "usage" VALUES('EPSG','13926','helmert_transformation','EPSG','9258','EPSG','1292','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','9259','Pampa del Castillo to WGS 84 (2)','Derived through common coordinates at 22 stations. Used by YPF throughout the Golfo San Jorge basin.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4161','EPSG','4326',2.5,-233.43,6.65,173.64,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'YPF-Arg GSJB',0);
INSERT INTO "usage" VALUES('EPSG','13927','helmert_transformation','EPSG','9259','EPSG','4572','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','9260','Tapi Aike to WGS 84 (1)','Used by YPF throughout the Tapi Aike basin.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','9248','EPSG','4326',5.0,-192.26,65.72,132.08,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'YPF-Arg TapiAike',0);
INSERT INTO "usage" VALUES('EPSG','13928','helmert_transformation','EPSG','9260','EPSG','4569','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','9261','MMN to WGS 84 (1)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','9251','EPSG','4326',2.5,-9.5,122.9,138.2,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'YPF-Arg MMN',0);
INSERT INTO "usage" VALUES('EPSG','13929','helmert_transformation','EPSG','9261','EPSG','2357','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','9262','MMS to WGS 84 (1)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','9253','EPSG','4326',2.5,-78.1,101.6,133.3,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'YPF-Arg MMS',0);
INSERT INTO "usage" VALUES('EPSG','13930','helmert_transformation','EPSG','9262','EPSG','2357','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','9263','Hito XVIII 1963 to WGS 84 (3)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4254','EPSG','4326',2.5,18.2,190.7,100.9,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'YPF-Arg TdF Hito',0);
INSERT INTO "usage" VALUES('EPSG','13931','helmert_transformation','EPSG','9263','EPSG','2357','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','9264','POSGAR 2007 to WGS 84 (2)','Derived as average at all points common between the POSGAR 94 and POSGAR 2007 networks. Accuracy 0.1m in 1994 increasing by approximately 1.5cm per year due to tectonic plate motion.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','5340','EPSG','4326',0.5,-0.41,0.46,-0.35,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'YPF-Arg',0);
INSERT INTO "usage" VALUES('EPSG','13932','helmert_transformation','EPSG','9264','EPSG','1033','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','9281','Amersfoort to ETRS89 (8)','Replaces Amersfoort to ETRS89 (5) and (6) (tfm codes 4830 and 4831). Derived using ETRF2000. In official transformation used with an ellipsoidal height of 0m in Amersfoort or 43m in ETRS89 and with an additional correction grid (up to 0.25m).','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4289','EPSG','4258',0.25,565.7381,50.4018,465.2904,'EPSG','9001',1.91514,-1.60363,9.09546,'EPSG','9109',4.07244,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IOGP-Nld 2018',0);
INSERT INTO "usage" VALUES('EPSG','13938','helmert_transformation','EPSG','9281','EPSG','1275','EPSG','1048');
INSERT INTO "helmert_transformation" VALUES('EPSG','9291','ISN2016 to WGS 84 (1)','For many purposes ISN2016 can be considered to be coincident with WGS 84.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','8086','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'LMI-Isl',0);
INSERT INTO "usage" VALUES('EPSG','14126','helmert_transformation','EPSG','9291','EPSG','1120','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','9298','ONGD17 to WGS 84 (1)','','EPSG','1032','Coordinate Frame rotation (geocentric domain)','EPSG','9292','EPSG','4978',0.1,1.16835,-1.42001,-2.24431,'EPSG','9001',0.00822,0.05508,-0.01818,'EPSG','9104',0.23388,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NSA-Omn',0);
INSERT INTO "usage" VALUES('EPSG','13971','helmert_transformation','EPSG','9298','EPSG','1183','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','9334','ITRF2014 to KSA-GRF17 (1)','Arabian plate rotations derived from obs. at 41 sites with at least 2.5y of continuous observations. May be approximated by position vector CT with rX=8.393 mas, rY=-0.749 mas, rZ=10.276 mas, valid at 2017.0, and degrading by ~2.5cm per year (code 9383).','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','7789','EPSG','9331',0.001,0.0,0.0,0.0,'EPSG','1025',0.0,0.0,0.0,'EPSG','1031',0.0,'EPSG','1028',0.0,0.0,0.0,'EPSG','1027',-1.199,0.107,-1.468,'EPSG','1032',0.0,'EPSG','1030',2017.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'GCS-Sau',0);
INSERT INTO "usage" VALUES('EPSG','13960','helmert_transformation','EPSG','9334','EPSG','1206','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','9339','PSAD56 to SIRGAS-Chile 2010 (1)','Also used as a transformation from PSAD56 to other realizations of SIRGAS-Chile and to WGS 84 - see code 6971.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4248','EPSG','8949',5.0,-302.0,272.0,-360.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGM-Chl A',0);
INSERT INTO "usage" VALUES('EPSG','14107','helmert_transformation','EPSG','9339','EPSG','4231','EPSG','1210');
INSERT INTO "helmert_transformation" VALUES('EPSG','9340','PSAD56 to SIRGAS-Chile 2010 (2)','Also used as a transformation from PSAD56 to other realizations of SIRGAS-Chile and to WGS 84 - see code 6972.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4248','EPSG','8949',5.0,-328.0,340.0,-329.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGM-Chl B',0);
INSERT INTO "usage" VALUES('EPSG','14108','helmert_transformation','EPSG','9340','EPSG','4222','EPSG','1210');
INSERT INTO "helmert_transformation" VALUES('EPSG','9341','PSAD56 to SIRGAS-Chile 2010 (3)','Also used as a transformation from PSAD56 to other realizations of SIRGAS-Chile and to WGS 84 - see code 6973.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4248','EPSG','8949',5.0,-352.0,403.0,-287.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGM-Chl C',0);
INSERT INTO "usage" VALUES('EPSG','14109','helmert_transformation','EPSG','9341','EPSG','4221','EPSG','1210');
INSERT INTO "helmert_transformation" VALUES('EPSG','9342','PSAD56 to SIRGAS-Chile 2013 (1)','Also used as a transformation from PSAD56 to other realizations of SIRGAS-Chile and to WGS 84 - see code 6971.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4248','EPSG','9148',5.0,-302.0,272.0,-360.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGM-Chl A',0);
INSERT INTO "usage" VALUES('EPSG','14110','helmert_transformation','EPSG','9342','EPSG','4231','EPSG','1210');
INSERT INTO "helmert_transformation" VALUES('EPSG','9343','PSAD56 to SIRGAS-Chile 2013 (2)','Also used as a transformation from PSAD56 to other realizations of SIRGAS-Chile and to WGS 84 - see code 6972.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4248','EPSG','9148',5.0,-328.0,340.0,-329.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGM-Chl B',0);
INSERT INTO "usage" VALUES('EPSG','14111','helmert_transformation','EPSG','9343','EPSG','4222','EPSG','1210');
INSERT INTO "helmert_transformation" VALUES('EPSG','9344','PSAD56 to SIRGAS-Chile 2013 (3)','Also used as a transformation from PSAD56 to other realizations of SIRGAS-Chile and to WGS 84 - see code 6973.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4248','EPSG','9148',5.0,-352.0,403.0,-287.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGM-Chl C',0);
INSERT INTO "usage" VALUES('EPSG','14112','helmert_transformation','EPSG','9344','EPSG','4221','EPSG','1210');
INSERT INTO "helmert_transformation" VALUES('EPSG','9345','PSAD56 to SIRGAS-Chile 2016 (1)','Also used as a transformation from PSAD56 to other realizations of SIRGAS-Chile and to WGS 84 - see code 6971.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4248','EPSG','9153',5.0,-302.0,272.0,-360.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGM-Chl A',0);
INSERT INTO "usage" VALUES('EPSG','14113','helmert_transformation','EPSG','9345','EPSG','4231','EPSG','1210');
INSERT INTO "helmert_transformation" VALUES('EPSG','9346','PSAD56 to SIRGAS-Chile 2016 (2)','Also used as a transformation from PSAD56 to other realizations of SIRGAS-Chile and to WGS 84 - see code 6972.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4248','EPSG','9153',5.0,-328.0,340.0,-329.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGM-Chl B',0);
INSERT INTO "usage" VALUES('EPSG','14114','helmert_transformation','EPSG','9346','EPSG','4222','EPSG','1210');
INSERT INTO "helmert_transformation" VALUES('EPSG','9347','PSAD56 to SIRGAS-Chile 2016 (3)','Also used as a transformation from PSAD56 to other realizations of SIRGAS-Chile and to WGS 84 - see code 6973.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4248','EPSG','9153',5.0,-352.0,403.0,-287.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGM-Chl C',0);
INSERT INTO "usage" VALUES('EPSG','14115','helmert_transformation','EPSG','9347','EPSG','4221','EPSG','1210');
INSERT INTO "helmert_transformation" VALUES('EPSG','9348','SAD69 to SIRGAS-Chile 2010 (4)','Also used as a transformation from SAD69 to other realizations of SIRGAS-Chile and to WGS 84 - see code 6977.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4618','EPSG','8949',5.0,-79.0,13.0,-14.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGM-Chl D',0);
INSERT INTO "usage" VALUES('EPSG','14116','helmert_transformation','EPSG','9348','EPSG','2805','EPSG','1210');
INSERT INTO "helmert_transformation" VALUES('EPSG','9349','SAD69 to SIRGAS-Chile 2013 (4)','Also used as a transformation from SAD69 to other realizations of SIRGAS-Chile and to WGS 84 - see code 6977.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4618','EPSG','9148',5.0,-79.0,13.0,-14.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGM-Chl D',0);
INSERT INTO "usage" VALUES('EPSG','14117','helmert_transformation','EPSG','9349','EPSG','2805','EPSG','1210');
INSERT INTO "helmert_transformation" VALUES('EPSG','9350','SAD69 to SIRGAS-Chile 2016 (4)','Also used as a transformation from SAD69 to other realizations of SIRGAS-Chile and to WGS 84 - see code 6977.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4618','EPSG','9153',5.0,-79.0,13.0,-14.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGM-Chl D',0);
INSERT INTO "usage" VALUES('EPSG','14118','helmert_transformation','EPSG','9350','EPSG','2805','EPSG','1210');
INSERT INTO "helmert_transformation" VALUES('EPSG','9361','MTRF-2000 to KSA-GRF17 (1)','','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','8818','EPSG','9333',0.1,0.0469,-0.2827,0.0866,'EPSG','9001',0.00559,-0.004981,0.023108,'EPSG','9104',-0.008051,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'GCS-Sau',0);
INSERT INTO "usage" VALUES('EPSG','14008','helmert_transformation','EPSG','9361','EPSG','1206','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','9362','Ain el Abd to KSA-GRF17 (1)','For a more accurate transformation see Ain el Abd to KSA-GRF17 (2) (CT code 9363)','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4204','EPSG','9333',2.0,13.8714,-83.9721,101.674,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'GCS-Sau 2m',0);
INSERT INTO "usage" VALUES('EPSG','14009','helmert_transformation','EPSG','9362','EPSG','3303','EPSG','1026');
INSERT INTO "helmert_transformation" VALUES('EPSG','9381','ITRF2014 to IGb14 (1)','Scale difference in ppb where 1/billion = 1E-9 or nm/m. IGb14 is aligned with IGS14 which by definition aligned with ITRF2014.','EPSG','1065','Time-specific Position Vector transform (geocen)','EPSG','7789','EPSG','9378',0.0,0.0,0.0,0.0,'EPSG','1033',0.0,0.0,0.0,'EPSG','1031',0.0,'EPSG','1028',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,2010.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IGS-Wld',0);
INSERT INTO "usage" VALUES('EPSG','14005','helmert_transformation','EPSG','9381','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','9382','IGS14 to IGb14 (1)','Scale difference in ppb where 1/billion = 1E-9 or nm/m. IGS14 and IGb14 are both by definition aligned with ITRF2014.','EPSG','1065','Time-specific Position Vector transform (geocen)','EPSG','8227','EPSG','9378',0.0,0.0,0.0,0.0,'EPSG','1025',0.0,0.0,0.0,'EPSG','1031',0.0,'EPSG','1028',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,2010.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IGS-Wld',0);
INSERT INTO "usage" VALUES('EPSG','14006','helmert_transformation','EPSG','9382','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','9383','KSA-GRF17 to WGS 84 (1)','Derived from ITRF2014 to KSA-GRF17 (1) (CT code 9334). Valid at 2017.00 and degrading by approximately 2.5cm per year.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','9333','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',-8.393,0.749,-10.276,'EPSG','1031',0.0,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'GCS-Sau',0);
INSERT INTO "usage" VALUES('EPSG','14007','helmert_transformation','EPSG','9383','EPSG','1206','EPSG','1026');
INSERT INTO "helmert_transformation" VALUES('EPSG','9459','ATRF2014 to GDA2020 (1)','Describes movement of ATRF2014 relative to GDA2020 due to Australian tectonic plate motion. Derived at 109 stations of the Australian Regional GNSS network (ARGN). Scale difference (dS) in ppb and dS rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1056','Time-dependent Coordinate Frame rotation (geocen)','EPSG','9307','EPSG','7842',0.03,0.0,0.0,0.0,'EPSG','1025',0.0,0.0,0.0,'EPSG','1031',0.0,'EPSG','1028',0.0,0.0,0.0,'EPSG','1027',1.50379,1.18346,1.20716,'EPSG','1032',0.0,'EPSG','1030',2020.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'GA-Aus',0);
INSERT INTO "usage" VALUES('EPSG','14139','helmert_transformation','EPSG','9459','EPSG','4177','EPSG','1267');
INSERT INTO "helmert_transformation" VALUES('EPSG','9460','ITRF2014 to ATRF2014 (1)','ATRF2014 is a regional densification of ITRF2014 for the Australian region.','EPSG','1032','Coordinate Frame rotation (geocentric domain)','EPSG','7789','EPSG','9307',0.01,0.0,0.0,0.0,'EPSG','9001',0.0,0.0,0.0,'EPSG','9104',0.0,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'GA-Aus',0);
INSERT INTO "usage" VALUES('EPSG','14140','helmert_transformation','EPSG','9460','EPSG','4177','EPSG','1268');
INSERT INTO "helmert_transformation" VALUES('EPSG','9472','DGN95 to SRGI2013 (1)','Derived at 533 stations. Mean residual 0.218m. Note: information source gives rotations given in radians.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4755','EPSG','9470',0.2,-0.2773,0.0534,0.4819,'EPSG','9001',0.0935,-0.0286,0.00969,'EPSG','9109',-0.028,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'SRGI-Idn',0);
INSERT INTO "usage" VALUES('EPSG','14154','helmert_transformation','EPSG','9472','EPSG','1122','EPSG','1026');
INSERT INTO "helmert_transformation" VALUES('EPSG','9486','MGI 1901 to WGS 84 (15)','Parameter values from MGI 1901 to ETRS89 (8) (code 9495). Assumes SRB-ETRS89 and WGS 84 can be considered the same to within the accuracy of the transformation.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','3906','EPSG','4326',1.0,577.84843,165.45019,390.43652,'EPSG','9001',-4.93131,0.96052,13.05072,'EPSG','9104',7.86546,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IOGP-Srb 2020',0);
INSERT INTO "usage" VALUES('EPSG','14220','helmert_transformation','EPSG','9486','EPSG','4543','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','9495','MGI 1901 to SRB-ETRS89 (8)','','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','3906','EPSG','8685',0.46,577.84843,165.45019,390.43652,'EPSG','9001',-4.93131,0.96052,13.05072,'EPSG','9104',7.86546,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'RGZ-Srb 0.5m 2020',0);
INSERT INTO "usage" VALUES('EPSG','14227','helmert_transformation','EPSG','9495','EPSG','4543','EPSG','1153');
INSERT INTO "helmert_transformation" VALUES('EPSG','9676','Israel 1993 to WGS 84 (2)','Parameter values from CT code 9186 assuming that WGS 84 is coincident with ITRF2008 and New Israeli Grid 1993 is equivalent to Israeli Grid 05/12 within the accuracy of the transformation. Use in preference to Israel 1993 to WGS 84 (1) (code 1073).','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4141','EPSG','4326',0.5,23.772,17.49,17.859,'EPSG','9001',0.3132,1.85274,-1.67299,'EPSG','9104',-5.4262,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'SOI-Isr 2019',0);
INSERT INTO "usage" VALUES('EPSG','14833','helmert_transformation','EPSG','9676','EPSG','2603','EPSG','1189');
INSERT INTO "helmert_transformation" VALUES('EPSG','9679','Gulshan 303 to WGS 84 (2)','Derived at origin station in Dhaka.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4682','EPSG','4326',1.0,283.729,735.942,261.143,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'SB-Bgd',0);
INSERT INTO "usage" VALUES('EPSG','14836','helmert_transformation','EPSG','9679','EPSG','1041','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','9682','ITRF2014 to GDA94 (1)','Concatenation of ITRF2014 to GDA2020 (1) (CT 8049) and GDA94 to GDA2020 (1) (CT 8048). Scale difference in ppb and rate in ppb/yr where 1/billion = 1E-9 or nm/m. See ITRF2014 to GDA94 (2) (CT 9683) for alternative for Australia with distortion modelling.','EPSG','1056','Time-dependent Coordinate Frame rotation (geocen)','EPSG','7789','EPSG','4938',0.035,-61.55,10.87,40.19,'EPSG','1025',39.4924,32.7221,32.8979,'EPSG','1031',9.994,'EPSG','1028',0.0,0.0,0.0,'EPSG','1027',1.50379,1.18346,1.20716,'EPSG','1032',0.0,'EPSG','1030',2020.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'ICSM-Aus Conf',0);
INSERT INTO "usage" VALUES('EPSG','14950','helmert_transformation','EPSG','9682','EPSG','4177','EPSG','1108');
INSERT INTO "helmert_transformation" VALUES('EPSG','9684','ATRF2014 to GDA94 (1)','Concatenation of ATRF2014 to GDA2020 (1) (CT 9459) and GDA94 to GDA2020 (1) (CT 8048). Scale difference in ppb and rate in ppb/yr where 1/billion = 1E-9 or nm/m. See ATRF2014 to GDA94 (2) (CT 9685) for alternative for Australia with distortion modelling.','EPSG','1056','Time-dependent Coordinate Frame rotation (geocen)','EPSG','9307','EPSG','4938',0.035,-61.55,10.87,40.19,'EPSG','1025',39.4924,32.7221,32.8979,'EPSG','1031',9.994,'EPSG','1028',0.0,0.0,0.0,'EPSG','1027',1.50379,1.18346,1.20716,'EPSG','1032',0.0,'EPSG','1030',2020.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'ICSM-Aus Conf',0);
INSERT INTO "usage" VALUES('EPSG','14952','helmert_transformation','EPSG','9684','EPSG','4177','EPSG','1108');
INSERT INTO "helmert_transformation" VALUES('EPSG','9686','GDA94 to WGS 84 (G1762) (1)','Concatenation of transformations codes 8048 and 8448 through GDA2020. Scale difference in ppb and rate in ppb/yr where 1/billion = 1E-9 or nm/m. See GDA94 to WGS 84 (G1762) (2) (CT 9687) for alternative for Australia with distortion modelling.','EPSG','1056','Time-dependent Coordinate Frame rotation (geocen)','EPSG','4938','EPSG','7664',0.25,61.55,-10.87,-40.19,'EPSG','1025',-39.4924,-32.7221,-32.8979,'EPSG','1031',-9.994,'EPSG','1028',0.0,0.0,0.0,'EPSG','1027',-1.50379,-1.18346,-1.20716,'EPSG','1032',0.0,'EPSG','1030',2020.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'ICSM-Aus Conf',0);
INSERT INTO "usage" VALUES('EPSG','14956','helmert_transformation','EPSG','9686','EPSG','4177','EPSG','1108');
INSERT INTO "helmert_transformation" VALUES('EPSG','9688','GDA94 to WGS 84 (2)','Concatenation of CTs 8048 and 8450 through GDA2020. Scale difference in ppb where 1/billion = 1E-9. See GDA94 to WGS 84 (2) (CT code 9689) for alternative for Australia including distortion model.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4283','EPSG','4326',3.0,61.55,-10.87,-40.19,'EPSG','1025',-39.4924,-32.7221,-32.8979,'EPSG','1031',-9.994,'EPSG','1028',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'ICSM-Aus Conf',0);
INSERT INTO "usage" VALUES('EPSG','14921','helmert_transformation','EPSG','9688','EPSG','4177','EPSG','1275');
INSERT INTO "helmert_transformation" VALUES('EPSG','9690','WGS 84 to GDA2020 (3)','Concatenation of CTs 1150 and 8048 through GDA94. Scale difference in ppb where 1/billion = 1E-9. See WGS 84 to GDA2020 (4) (CT code 9691) for alternative for Australia including distortion model.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4326','EPSG','7844',3.0,61.55,-10.87,-40.19,'EPSG','1025',-39.4924,-32.7221,-32.8979,'EPSG','1031',-9.994,'EPSG','1028',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'ICSM-Aus Conf',0);
INSERT INTO "usage" VALUES('EPSG','14920','helmert_transformation','EPSG','9690','EPSG','4177','EPSG','1276');
INSERT INTO "helmert_transformation" VALUES('EPSG','9703','ETRF2000-PL to ETRS89 (1)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','9702','EPSG','4258',0.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IOGP-Pol',0);
INSERT INTO "usage" VALUES('EPSG','15011','helmert_transformation','EPSG','9703','EPSG','1192','EPSG','1161');
INSERT INTO "helmert_transformation" VALUES('EPSG','9743','PN68 to WGS 84 (1)','Determined at 1 satellite station. Accuracy +/- 25m in each axis. Unclear from information source whether the source CRS is PN68 or PN84 - see also CT code 15815. Given the accuracy, can be considered to apply to either.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','9403','EPSG','4326',44.0,-307.0,-92.0,127.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Esp Canary',0);
INSERT INTO "usage" VALUES('EPSG','15321','helmert_transformation','EPSG','9743','EPSG','3873','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','9751','CR05 to CR-SIRGAS epoch 2014.59 (1)','Derived at 16 stations. Transformation is valid at epoch 2014.59 but accuracy will deteriorate due to tectonic deformation. Note that the rotation convention (coordinate frame) differs from that used in the transformation from CR-SIRGAS epoch 2019.24 to CR-SIRGAS epoch 2014.59 (code 11044). May be taken as an approximate transformation to ITRF08 / IGb08 / WGS 84 - see code 9752.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','5365','EPSG','8907',0.5,-0.16959,0.35312,0.51846,'EPSG','9001',-0.03385,0.16325,-0.03446,'EPSG','9104',0.03693,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Cri 2014',0);
INSERT INTO "usage" VALUES('EPSG','15596','helmert_transformation','EPSG','9751','EPSG','1074','EPSG','1035');
INSERT INTO "helmert_transformation" VALUES('EPSG','9752','CR05 to WGS 84 (2)','Parameter values are from CR05 to CR-SIRGAS epoch 2014.59 (1) (code 9751) assuming that CR-SIRGAS (ITRF08 (IGb08)@2014.59) is equivalent to WGS 84 within the accuracy of the transformation.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','5365','EPSG','4326',1.0,-0.16959,0.35312,0.51846,'EPSG','9001',-0.03385,0.16325,-0.03446,'EPSG','9104',0.03693,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Cri 2014',0);
INSERT INTO "usage" VALUES('EPSG','15832','helmert_transformation','EPSG','9752','EPSG','1074','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','9756','WGS 84 (G1762) to WGS 84 (G2139) (1)','Scale difference in ppb where 1/billion = 1E-9 or nm/m.','EPSG','1032','Coordinate Frame rotation (geocentric domain)','EPSG','7664','EPSG','9753',0.01,0.0058,-0.0064,0.007,'EPSG','9001',0.08,0.04,0.12,'EPSG','1031',-4.4,'EPSG','1028',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NGA-Wld 2021.0',0);
INSERT INTO "usage" VALUES('EPSG','16692','helmert_transformation','EPSG','9756','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','9757','WGS 84 (G2139) to ITRF2014 (1)','Scale difference in ppb where 1/billion = 1E-9 or nm/m.','EPSG','1032','Coordinate Frame rotation (geocentric domain)','EPSG','9753','EPSG','7789',0.01,0.0,0.0,0.0,'EPSG','1025',0.0,0.0,0.0,'EPSG','1031',0.0,'EPSG','1028',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NGA-Wld 2021.0',0);
INSERT INTO "usage" VALUES('EPSG','16664','helmert_transformation','EPSG','9757','EPSG','1262','EPSG','1026');
INSERT INTO "helmert_transformation" VALUES('EPSG','9768','Kyrg-06 to WGS 84 (1)','For many purposes Kyrg-06 can be considered to be coincident with WGS 84. This approximation ignores the low accuracy of the WGS 84 ensemble and the application of tectonic plate motion to WGS 84 data.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','7686','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Kgz',0);
INSERT INTO "usage" VALUES('EPSG','16666','helmert_transformation','EPSG','9768','EPSG','1137','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','9769','RGWF96 to WGS 84 (1)','For many purposes RGWF96 can be considered to be coincident with WGS 84. This approximation ignores the low accuracy of the WGS 84 ensemble and the application of tectonic plate motion to WGS 84 data.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','8900','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Wlf',0);
INSERT INTO "usage" VALUES('EPSG','16667','helmert_transformation','EPSG','9769','EPSG','1255','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','9770','RGTAAF07 to WGS 84 (1)','For many purposes RGTAAF07 can be considered to be coincident with WGS 84. This approximation ignores the low accuracy of the WGS 84 ensemble and the application of tectonic plate motion to WGS 84 data.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','7073','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Atf',0);
INSERT INTO "usage" VALUES('EPSG','16668','helmert_transformation','EPSG','9770','EPSG','4246','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','9771','TGD2005 to WGS 84 (1)','For many purposes TGD2005 can be considered to be coincident with WGS 84. This approximation ignores the low accuracy of the WGS 84 ensemble and the application of tectonic plate motion to WGS 84 data.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','5886','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Ton',0);
INSERT INTO "usage" VALUES('EPSG','16669','helmert_transformation','EPSG','9771','EPSG','1234','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','9772','ETRF2000-PL to WGS 84 (1)','For many purposes ETRF2000-PL can be considered to be coincident with WGS 84. This approximation ignores the low accuracy of the WGS 84 ensemble and the application of tectonic plate motion to WGS 84 data.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','9702','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Pol',0);
INSERT INTO "usage" VALUES('EPSG','16670','helmert_transformation','EPSG','9772','EPSG','1192','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','9773','GSK-2011 to WGS 84 (1)','For many purposes GSK-2011 can be considered to be coincident with WGS 84. This approximation ignores the low accuracy of the WGS 84 ensemble and the application of tectonic plate motion to WGS 84 data.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','7683','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Rus',0);
INSERT INTO "usage" VALUES('EPSG','16671','helmert_transformation','EPSG','9773','EPSG','1198','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','9774','NAD83(2011) to WGS 84 (1)','For many purposes NAD83(2011) can be considered to be coincident with WGS 84. This approximation ignores the ~2.2m offset of NAD83 from the geocentre, the low accuracy of the WGS 84 ensemble and the application of tectonic plate motion to WGS 84 data.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','6318','EPSG','4326',2.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Usa Conus AK PRVI',0);
INSERT INTO "usage" VALUES('EPSG','16675','helmert_transformation','EPSG','9774','EPSG','1511','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','9788','RGF93 v2 to RGF93 v2b','Derived at 465 permanent GNSS stations. In use from 2021-01-05.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','9777','EPSG','9782',0.005,-0.017,0.058,0.009,'EPSG','9001',0.001305,0.00068,-0.001467,'EPSG','9104',-0.00072,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Fra',0);
INSERT INTO "usage" VALUES('EPSG','16601','helmert_transformation','EPSG','9788','EPSG','1096','EPSG','1179');
INSERT INTO "helmert_transformation" VALUES('EPSG','9789','RGF93 v2 to ETRS89 (1)','RGF93 v2 is a national realization of ETRS89. May be taken as approximate transformation RGF93 v2 to WGS 84 - see code 9791.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','9777','EPSG','4258',0.02,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Fra',0);
INSERT INTO "usage" VALUES('EPSG','16638','helmert_transformation','EPSG','9789','EPSG','1096','EPSG','1161');
INSERT INTO "helmert_transformation" VALUES('EPSG','9790','RGF93 v2b to ETRS89 (1)','RGF93 v2b is a national realization of ETRS89. May be taken as approximate transformation RGF93 v2b to WGS 84 - see CT code 9792.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','9782','EPSG','4258',0.01,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Fra',0);
INSERT INTO "usage" VALUES('EPSG','16656','helmert_transformation','EPSG','9790','EPSG','1096','EPSG','1161');
INSERT INTO "helmert_transformation" VALUES('EPSG','9791','RGF93 v2 to WGS 84 (1)','Approximation at the 1m level assuming that ETRS89 is equivalent to WGS 84 within the accuracy of the transformation. (RGF93 v2 is a national realization of ETRS89).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','9777','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Fra',0);
INSERT INTO "usage" VALUES('EPSG','16612','helmert_transformation','EPSG','9791','EPSG','1096','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','9792','RGF93 v2b to WGS 84 (1)','Approximation at the 1m level assuming that ETRS89 is equivalent to WGS 84 within the accuracy of the transformation. (RGF93 v2b is a national realization of ETRS89).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','9782','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Fra',0);
INSERT INTO "usage" VALUES('EPSG','16613','helmert_transformation','EPSG','9792','EPSG','1096','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','9882','RGF93 v1 to RGF93 v2 (1)','Maximum differences are 0.1m in both horizontal and vertical. Distortions in RGF93 v1 are irregular and IGN considers that no systematic transformation to the improved v2 adjustment is possible.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4171','EPSG','9777',0.05,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Fra',0);
INSERT INTO "usage" VALUES('EPSG','16986','helmert_transformation','EPSG','9882','EPSG','1096','EPSG','1079');
INSERT INTO "helmert_transformation" VALUES('EPSG','9898','LUREF to ETRS89 (5)','LUREF 2014 parameters. For an equivalent CT using the Coordinate Frame method see CT 9899. Defines LUREF between 2014 and 2020. Replaces LUREF 2006 parameters (CT 5483). Info source gives this CT in reverse direction. Derived using ETRF2000 @2013-01-27.','EPSG','9636','Molodensky-Badekas (CF geog2D domain)','EPSG','4181','EPSG','4258',0.0,-265.8979,76.9761,20.2504,'EPSG','9001',0.43335,3.11447,-2.63637,'EPSG','9104',0.4752,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,4103620.3891,440486.4152,4846923.4466,'EPSG','9001','ACT-Lux MB 2014',0);
INSERT INTO "usage" VALUES('EPSG','17305','helmert_transformation','EPSG','9898','EPSG','1146','EPSG','1282');
INSERT INTO "helmert_transformation" VALUES('EPSG','9899','LUREF to ETRS89 (6)','LUREF 2014 parameters. For an equivalent CT using the Molodensky-Badekas method see CT 9898. Defines LUREF between 2014 and 2020. Replaces LUREF 2006 parameters (CT 5485). Info source gives this CT in reverse direction. Derived using ETRF2000@2013-01-27.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4181','EPSG','4258',0.0,-189.033,14.1335,-43.0901,'EPSG','9001',0.43331,3.11448,-2.63636,'EPSG','9104',0.4752,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'ACT-Lux CF 2014',0);
INSERT INTO "usage" VALUES('EPSG','17306','helmert_transformation','EPSG','9899','EPSG','1146','EPSG','1282');
INSERT INTO "helmert_transformation" VALUES('EPSG','9901','UCS-2000 to ETRS89 (1)','Rounded parameter values taken from UCS-2000 to ITRF2000 (1) (code 7817) assuming that ETRS89 is equivalent to ITRF2000 within the accuracy of the transformation.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','5561','EPSG','4258',1.0,24.0,-121.0,-76.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-Ukr SSGC',0);
INSERT INTO "usage" VALUES('EPSG','17069','helmert_transformation','EPSG','9901','EPSG','1242','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','9904','Camacupa 1948 to RSAO13 (3)','Parameter values taken from Camacupa 1948 to WGS 84 (6) (code 1323) assuming that RSAO13 is coincident with WGS 84 within the accuracy of the transformation. Equivalent to concatenation of EPSG:1323 and EPSG:8819.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4220','EPSG','8699',8.0,-43.0,-337.0,-233.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IOGP-Ago B7 old',0);
INSERT INTO "usage" VALUES('EPSG','18357','helmert_transformation','EPSG','9904','EPSG','2319','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','9905','Camacupa 1948 to RSAO13 (4)','Parameter values taken from Camacupa 1948 to WGS 84 (9) (code 1326) assuming that RSAO13 is coincident with WGS 84 within the accuracy of the transformation. Equivalent to concatenation of EPSG:1326 and EPSG:8819.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4220','EPSG','8699',10.0,-41.057,-374.564,-226.287,'EPSG','9001',0.0,0.0,0.554,'EPSG','9104',0.219,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IOGP-Ago B16',0);
INSERT INTO "usage" VALUES('EPSG','18358','helmert_transformation','EPSG','9905','EPSG','2323','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','9906','Malongo 1987 to RSAO13 (1)','Parameter values taken from Malongo 1987 to WGS 84 (2) (code 1557) assuming that RSAO13 is coincident with WGS 84 within the accuracy of the transformation. Equivalent to concatenation of EPSG:1557 and EPSG:8819.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4259','EPSG','8699',5.0,-254.1,-5.36,-100.29,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IOGP-Ago Cab90',0);
INSERT INTO "usage" VALUES('EPSG','18359','helmert_transformation','EPSG','9906','EPSG','3180','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','9913','Hong Kong 1980 to Hong Kong Geodetic CS (1)','Derived via ITRF96 @ 1998.121. Published 2002-03-01. Also published as a transformation to WGS 84 - see Hong Kong 1980 to WGS 84 (1) (code 1825).','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4611','EPSG','8427',1.0,-162.619,-276.959,-161.764,'EPSG','9001',-0.067753,2.243648,1.158828,'EPSG','9104',-1.094246,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'LSD-HKG 2002',0);
INSERT INTO "usage" VALUES('EPSG','17339','helmert_transformation','EPSG','9913','EPSG','1118','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','9936','JGD2011 to WGS 84 (1)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','6668','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Jpn',0);
INSERT INTO "usage" VALUES('EPSG','17201','helmert_transformation','EPSG','9936','EPSG','1129','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','9937','LUREF to ETRS89 (7)','LUREF 2020 parameters. For an equivalent CT using the Coordinate Frame method see CT 9938. Defines LUREF from 2020. Replaces LUREF 2014 parameters (CT 9898). Info source gives this CT in reverse direction. Derived using ETRF2000 @2013-01-27.','EPSG','9636','Molodensky-Badekas (CF geog2D domain)','EPSG','4181','EPSG','4258',0.0,-265.9196,76.9506,20.2222,'EPSG','9001',0.48171,3.09948,-2.68639,'EPSG','9104',0.46346,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,4101567.0943,440245.0881,4848681.4115,'EPSG','9001','ACT-Lux MB 2020',0);
INSERT INTO "usage" VALUES('EPSG','17307','helmert_transformation','EPSG','9937','EPSG','1146','EPSG','1283');
INSERT INTO "helmert_transformation" VALUES('EPSG','9938','LUREF to ETRS89 (8)','LUREF 2020 parameters. For an equivalent CT using the Molodensky-Badekas method see CT 9937. Defines LUREF from 2020. Replaces LUREF 2014 parameters (CT 9899). Info source gives CT in reverse direction. Derived using ETRF2000 @2013-01-27.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4181','EPSG','4258',0.0,-189.228,12.0035,-42.6303,'EPSG','9001',0.48171,3.09948,-2.68639,'EPSG','9104',0.46346,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'ACT-Lux CF 2020',0);
INSERT INTO "usage" VALUES('EPSG','17308','helmert_transformation','EPSG','9938','EPSG','1146','EPSG','1283');
INSERT INTO "helmert_transformation" VALUES('EPSG','9960','WGS 84 (Transit) to WGS 84 (G730) (1)','Scale difference in ppb where 1/billion = 1E-9 or nm/m. Derived through ITRF90 and ITRF91 at epoch 1990.5. Parameter values valid at other epochs including 2005.0.','EPSG','1032','Coordinate Frame rotation (geocentric domain)','EPSG','7815','EPSG','7656',0.7,-58.0,521.0,239.0,'EPSG','1025',18.3,-0.3,7.0,'EPSG','1031',10.7,'EPSG','1028',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'KD2022-Wld',0);
INSERT INTO "usage" VALUES('EPSG','17491','helmert_transformation','EPSG','9960','EPSG','1262','EPSG','1026');
INSERT INTO "helmert_transformation" VALUES('EPSG','9961','WGS 84 (G730) to WGS 84 (G873) (1)','Scale difference in ppb where 1/billion = 1E-9 or nm/m. Derived via ITRF91 through ITRF94 at epoch 1995.5. Parameter values valid at other epochs including 2005.0.','EPSG','1032','Coordinate Frame rotation (geocentric domain)','EPSG','7656','EPSG','7658',0.04,-20.0,-16.0,14.0,'EPSG','1025',0.0,0.0,0.0,'EPSG','1031',-0.69,'EPSG','1028',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'KD2022-Wld',0);
INSERT INTO "usage" VALUES('EPSG','17492','helmert_transformation','EPSG','9961','EPSG','1262','EPSG','1026');
INSERT INTO "helmert_transformation" VALUES('EPSG','9962','WGS 84 (G873) to WGS 84 (G1150) (1)','Scale difference in ppb where 1/billion = 1E-9 or nm/m. Derived via ITRF94 through ITRF2000. May be approximated to 0.2m using 7-parameter CF method 1032 with parameters for epoch 1999.50 of tX=1.1mm tY=-8.0mm tZ=14.3mm rX=0 rY=0 rZ=0.05mas dS=1.505ppb.','EPSG','1056','Time-dependent Coordinate Frame rotation (geocen)','EPSG','7658','EPSG','7660',0.03,1.1,-4.7,22.0,'EPSG','1025',0.0,0.0,0.16,'EPSG','1031',1.45,'EPSG','1028',0.0,0.6,1.4,'EPSG','1027',0.0,0.0,0.02,'EPSG','1032',-0.01,'EPSG','1030',2005.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'KD2022-Wld',0);
INSERT INTO "usage" VALUES('EPSG','18078','helmert_transformation','EPSG','9962','EPSG','1262','EPSG','1026');
INSERT INTO "helmert_transformation" VALUES('EPSG','9963','WGS 84 (G1150) to WGS 84 (G1674) (1)','Scale difference in ppb where 1/billion = 1E-9 or nm/m. Derived via ITRF2005. May be approximated to 0.4m using 7-parameter CF method 1032 with parameters for epoch 2007.50 of tX=-2.65mm tY=1.35mm tZ=27.7mm rX=-0.27mas rY=027mas rZ=-0.38 mas dS=1.88ppb.','EPSG','1056','Time-dependent Coordinate Frame rotation (geocen)','EPSG','7660','EPSG','7662',0.02,-2.4,1.6,23.2,'EPSG','1025',-0.27,0.27,-0.38,'EPSG','1031',2.08,'EPSG','1028',-0.1,-0.1,1.8,'EPSG','1027',0.0,0.0,0.0,'EPSG','1032',-0.08,'EPSG','1030',2005.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'KD2022-Wld',0);
INSERT INTO "usage" VALUES('EPSG','17500','helmert_transformation','EPSG','9963','EPSG','1262','EPSG','1026');
INSERT INTO "helmert_transformation" VALUES('EPSG','9991','ITRF2014 to ITRF2020 (1)','IERS describes CT in opposite direction. Estimated using core network of 131 stations at 105 sites. Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','7789','EPSG','9988',0.001,1.4,0.9,-1.4,'EPSG','1025',0.0,0.0,0.0,'EPSG','1031',0.42,'EPSG','1028',0.0,0.1,-0.2,'EPSG','1027',0.0,0.0,0.0,'EPSG','1032',0.0,'EPSG','1030',2015.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IERS-Wld',0);
INSERT INTO "usage" VALUES('EPSG','17933','helmert_transformation','EPSG','9991','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','9992','ITRF2008 to ITRF2020 (1)','IERS describes CT in opposite direction. Parameter values derived from those already published between ITRF2020, ITRF2014 and ITRF2008. Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','5332','EPSG','9988',0.01,-0.2,-1.0,-3.3,'EPSG','1025',0.0,0.0,0.0,'EPSG','1031',0.29,'EPSG','1028',0.0,0.1,-0.1,'EPSG','1027',0.0,0.0,0.0,'EPSG','1032',-0.03,'EPSG','1030',2015.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IERS-Wld',0);
INSERT INTO "usage" VALUES('EPSG','17946','helmert_transformation','EPSG','9992','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','9993','ITRF2005 to ITRF2020 (1)','IERS describes CT in opposite direction. Parameter values derived from the those between ITRF2020, ITRF2014 and ITRF2008 and those published between earlier realizations. Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4896','EPSG','9988',0.01,-2.7,-0.1,1.4,'EPSG','1025',0.0,0.0,0.0,'EPSG','1031',-0.65,'EPSG','1028',-0.3,0.1,-0.1,'EPSG','1027',0.0,0.0,0.0,'EPSG','1032',-0.03,'EPSG','1030',2015.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IERS-Wld',0);
INSERT INTO "usage" VALUES('EPSG','17935','helmert_transformation','EPSG','9993','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','9994','ITRF2000 to ITRF2020 (1)','IERS describes CT in opposite direction. Parameter values derived from the those between ITRF2020, ITRF2014 and ITRF2008 and those published between earlier realizations. Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4919','EPSG','9988',0.01,0.2,-0.8,34.2,'EPSG','1025',0.0,0.0,0.0,'EPSG','1031',-2.25,'EPSG','1028',-0.1,0.0,1.7,'EPSG','1027',0.0,0.0,0.0,'EPSG','1032',-0.11,'EPSG','1030',2015.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IERS-Wld',0);
INSERT INTO "usage" VALUES('EPSG','17936','helmert_transformation','EPSG','9994','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','9995','ITRF97 to ITRF2020 (1)','IERS describes CT in opposite direction. Parameter values derived from the those between ITRF2020, ITRF2014 and ITRF2008 and those published between earlier realizations. Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4918','EPSG','9988',0.01,-6.5,3.9,77.9,'EPSG','1025',0.0,0.0,-0.36,'EPSG','1031',-3.98,'EPSG','1028',-0.1,0.6,3.1,'EPSG','1027',0.0,0.0,-0.02,'EPSG','1032',-0.12,'EPSG','1030',2015.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IERS-Wld',0);
INSERT INTO "usage" VALUES('EPSG','17937','helmert_transformation','EPSG','9995','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','9996','ITRF96 to ITRF2020 (1)','IERS describes CT in opposite direction. Parameter values derived from the those between ITRF2020, ITRF2014 and ITRF2008 and those published between earlier realizations. Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4917','EPSG','9988',0.01,-6.5,3.9,77.9,'EPSG','1025',0.0,0.0,-0.36,'EPSG','1031',-3.98,'EPSG','1028',-0.1,0.6,3.1,'EPSG','1027',0.0,0.0,-0.02,'EPSG','1032',-0.12,'EPSG','1030',2015.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IERS-Wld',0);
INSERT INTO "usage" VALUES('EPSG','17938','helmert_transformation','EPSG','9996','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','9997','ITRF94 to ITRF2020 (1)','IERS describes CT in opposite direction. Parameter values derived from the those between ITRF2020, ITRF2014 and ITRF2008 and those published between earlier realizations. Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4916','EPSG','9988',0.01,-6.5,3.9,77.9,'EPSG','1025',0.0,0.0,-0.36,'EPSG','1031',-3.98,'EPSG','1028',-0.1,0.6,3.1,'EPSG','1027',0.0,0.0,-0.02,'EPSG','1032',-0.12,'EPSG','1030',2015.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IERS-Wld',0);
INSERT INTO "usage" VALUES('EPSG','17939','helmert_transformation','EPSG','9997','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','9998','ITRF93 to ITRF2020 (1)','IERS describes CT in opposite direction. Parameter values derived from the those between ITRF2020, ITRF2014 and ITRF2008 and those published between earlier realizations. Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4915','EPSG','9988',0.01,65.8,-1.9,71.3,'EPSG','1025',3.36,4.33,-0.75,'EPSG','1031',-4.47,'EPSG','1028',2.8,0.2,2.3,'EPSG','1027',0.11,0.19,-0.07,'EPSG','1032',-0.12,'EPSG','1030',2015.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IERS-Wld',0);
INSERT INTO "usage" VALUES('EPSG','17947','helmert_transformation','EPSG','9998','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','9999','ITRF92 to ITRF2020 (1)','IERS describes CT in opposite direction. Parameter values derived from the those between ITRF2020, ITRF2014 and ITRF2008 and those published between earlier realizations. Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4914','EPSG','9988',0.01,-14.5,1.9,85.9,'EPSG','1025',0.0,0.0,-0.36,'EPSG','1031',-3.27,'EPSG','1028',-0.1,0.6,3.1,'EPSG','1027',0.0,0.0,-0.02,'EPSG','1032',-0.12,'EPSG','1030',2015.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IERS-Wld',0);
INSERT INTO "usage" VALUES('EPSG','17941','helmert_transformation','EPSG','9999','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','10085','Trinidad 1903 to WGS 84 (2)','Parameter values provided to EOG by Trinidad Ministry of Energy and Energy Industries. Used by EOG offshore Trinidad (including Pelican, Kiskadee and Ibis fields) since 1996.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4302','EPSG','4326',3.0,-61.0,285.2,471.6,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EOG-Tto Trin',0);
INSERT INTO "usage" VALUES('EPSG','11086','helmert_transformation','EPSG','10085','EPSG','1339','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','10086','JAD69 to WGS 72 (1)','Derived in 1977 through Transit observations at 2 stations by US DMA.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4242','EPSG','4322',15.0,48.0,208.0,382.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'SD-Jam',0);
INSERT INTO "usage" VALUES('EPSG','11087','helmert_transformation','EPSG','10086','EPSG','3342','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','10089','Aratu to WGS 84 (5)','Used by ExxonMobil for block BMS1. See WGS 84 (13) (tfm code 5051) which Petrobras now recommends for the area.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4208','EPSG','4326',7.0,-163.466,317.396,-147.538,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EXM-Bra Santos',0);
INSERT INTO "usage" VALUES('EPSG','11090','helmert_transformation','EPSG','10089','EPSG','2962','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','10090','Aratu to WGS 84 (6)','Used by ExxonMobil for block BC10. Derived from earlier Shell position vector tfm of dX = -181m, dY = +294m, dZ = -144.5m, rX = rY = 0, rZ = +0.554s, dS = +0.219 ppm. See Aratu to WGS 84 (14) (tfm code 5053) which Petrobras now recommends for the area.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4208','EPSG','4326',7.0,-170.0,305.0,-145.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EXM-Bra Campos',0);
INSERT INTO "usage" VALUES('EPSG','11091','helmert_transformation','EPSG','10090','EPSG','2963','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','10091','Aratu to WGS 84 (7)','Used by ExxonMobil for block BMES1. See Aratu to WGS 84 (15) (tfm code 5055) which Petrobras now recommends for the area.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4208','EPSG','4326',7.0,-162.904,312.531,-137.109,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EXM-Bra EspS',0);
INSERT INTO "usage" VALUES('EPSG','11092','helmert_transformation','EPSG','10091','EPSG','2964','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','10092','Aratu to WGS 84 (8)','Used by ExxonMobil for block BP1. Also used by BG as part of a concatenated tfm to SAD69 for offshore regional studies. See WGS 84 (13) (tfm code 5051) for transformation Petrobras now recommends for the area.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4208','EPSG','4326',7.0,-158.0,309.0,-151.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EXM-Bra Pel',0);
INSERT INTO "usage" VALUES('EPSG','11093','helmert_transformation','EPSG','10092','EPSG','2965','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','10093','Aratu to WGS 84 (9)','Used by ExxonMobil for offshore regional studies. See Aratu to WGS 84 (13) through (21) (tfm codes 5051-67 [odd numbers only]) which Petrobras now recommends for various areas.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4208','EPSG','4326',15.0,-161.0,308.0,-142.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EXM-Bra off',0);
INSERT INTO "usage" VALUES('EPSG','11094','helmert_transformation','EPSG','10093','EPSG','2966','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','10094','Nouakchott 1965 to WGS 84 (1)','Derived by IGN in 1992 at 7 stations within Nouakchott city.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4208','EPSG','4326',5.0,124.5,-63.5,-281.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Mau',1);
INSERT INTO "usage" VALUES('EPSG','11095','helmert_transformation','EPSG','10094','EPSG','2972','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','10098','KKJ to EUREF-FIN (2)','In most areas accuracy is approximately 0.5m although in some areas it is in the order of 2m. Replaces KKJ to ETRS89 (1) (code 1638). YKJ to ETRS-TM35FIN (code 10703) using the TINshift method is recommended in preference to this transformation.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4123','EPSG','10690',0.5,-96.062,-82.428,-121.753,'EPSG','9001',-4.801,-0.345,1.376,'EPSG','9104',1.496,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NLS-Fin JHS153',0);
INSERT INTO "usage" VALUES('EPSG','11099','helmert_transformation','EPSG','10098','EPSG','3333','EPSG','1035');
INSERT INTO "helmert_transformation" VALUES('EPSG','10099','KKJ to WGS 84 (2)','Parameter values from KKJ to ETRS89 (2) (code 10098). Assumes ETRS89 and WGS 84 can be considered the same to within the accuracy of the transformation. Replaces KKJ to WGS 84 (1) (code 1639).','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4123','EPSG','4326',1.0,-96.062,-82.428,-121.753,'EPSG','9001',-4.801,-0.345,1.376,'EPSG','9104',1.496,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Fin JHS153',0);
INSERT INTO "usage" VALUES('EPSG','11100','helmert_transformation','EPSG','10099','EPSG','3333','EPSG','1151');
INSERT INTO "helmert_transformation" VALUES('EPSG','10100','ITRF91 to ITRF2020 (1)','IERS describes CT in opposite direction. Parameter values derived from the those between ITRF2020, ITRF2014 and ITRF2008 and those published between earlier realizations. Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4913','EPSG','9988',0.01,-26.5,-12.1,91.9,'EPSG','1025',0.0,0.0,-0.36,'EPSG','1031',-4.67,'EPSG','1028',-0.1,0.6,3.1,'EPSG','1027',0.0,0.0,-0.02,'EPSG','1032',-0.12,'EPSG','1030',2015.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IERS-Wld',0);
INSERT INTO "usage" VALUES('EPSG','17942','helmert_transformation','EPSG','10100','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','10103','ITRF90 to ITRF2020 (1)','IERS describes CT in opposite direction. Parameter values derived from the those between ITRF2020, ITRF2014 and ITRF2008 and those published between earlier realizations. Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4912','EPSG','9988',0.01,-24.5,-8.1,107.9,'EPSG','1025',0.0,0.0,-0.36,'EPSG','1031',-4.97,'EPSG','1028',-0.1,0.6,3.1,'EPSG','1027',0.0,0.0,-0.02,'EPSG','1032',-0.12,'EPSG','1030',2015.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IERS-Wld',0);
INSERT INTO "usage" VALUES('EPSG','17943','helmert_transformation','EPSG','10103','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','10104','ITRF89 to ITRF2020 (1)','IERS describes CT in opposite direction. Parameter values derived from the those between ITRF2020, ITRF2014 and ITRF2008 and those published between earlier realizations. Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4911','EPSG','9988',0.01,-29.5,-32.1,145.9,'EPSG','1025',0.0,0.0,-0.36,'EPSG','1031',-8.37,'EPSG','1028',-0.1,0.6,3.1,'EPSG','1027',0.0,0.0,-0.02,'EPSG','1032',-0.12,'EPSG','1030',2015.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IERS-Wld',0);
INSERT INTO "usage" VALUES('EPSG','17944','helmert_transformation','EPSG','10104','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','10105','ITRF88 to ITRF2020 (1)','IERS describes CT in opposite direction. Parameter values derived from the those between ITRF2020, ITRF2014 and ITRF2008 and those published between earlier realizations. Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4910','EPSG','9988',0.01,-24.5,3.9,169.9,'EPSG','1025',-0.1,0.0,-0.36,'EPSG','1031',-11.47,'EPSG','1028',-0.1,0.6,3.1,'EPSG','1027',0.0,0.0,-0.02,'EPSG','1032',-0.12,'EPSG','1030',2015.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IERS-Wld',0);
INSERT INTO "usage" VALUES('EPSG','17948','helmert_transformation','EPSG','10105','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','10134','SIRGAS-Chile 2021 to ITRF2014 (1)','Transformation is exact at epoch 2021.00 but accuracy then decreasing with time.','EPSG','1065','Time-specific Position Vector transform (geocen)','EPSG','20039','EPSG','7789',0.1,0.0,0.0,0.0,'EPSG','1025',0.0,0.0,0.0,'EPSG','1031',0.0,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,2021.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IOGP-Chl',0);
INSERT INTO "usage" VALUES('EPSG','18457','helmert_transformation','EPSG','10134','EPSG','1066','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','10135','PSAD56 to SIRGAS-Chile 2021 (1)','Also used as a transformation from PSAD56 to other realizations of SIRGAS-Chile and to WGS 84 - see code 6971.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4248','EPSG','20041',5.0,-302.0,272.0,-360.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGM-Chl A',0);
INSERT INTO "usage" VALUES('EPSG','18374','helmert_transformation','EPSG','10135','EPSG','4231','EPSG','1210');
INSERT INTO "helmert_transformation" VALUES('EPSG','10136','PSAD56 to SIRGAS-Chile 2021 (2)','Also used as a transformation from PSAD56 to other realizations of SIRGAS-Chile and to WGS 84 - see code 6972.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4248','EPSG','20041',5.0,-328.0,340.0,-329.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGM-Chl B',0);
INSERT INTO "usage" VALUES('EPSG','18375','helmert_transformation','EPSG','10136','EPSG','4222','EPSG','1210');
INSERT INTO "helmert_transformation" VALUES('EPSG','10137','PSAD56 to SIRGAS-Chile 2021 (3)','Also used as a transformation from PSAD56 to other realizations of SIRGAS-Chile and to WGS 84 - see code 6973.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4248','EPSG','20041',5.0,-352.0,403.0,-287.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGM-Chl C',0);
INSERT INTO "usage" VALUES('EPSG','18376','helmert_transformation','EPSG','10137','EPSG','4221','EPSG','1210');
INSERT INTO "helmert_transformation" VALUES('EPSG','10138','SAD69 to SIRGAS-Chile 2021 (4)','Also used as a transformation from SAD69 to other realizations of SIRGAS-Chile and to WGS 84 - see code 6977.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4618','EPSG','20041',5.0,-79.0,13.0,-14.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGM-Chl D',0);
INSERT INTO "usage" VALUES('EPSG','18377','helmert_transformation','EPSG','10138','EPSG','2805','EPSG','1210');
INSERT INTO "helmert_transformation" VALUES('EPSG','10139','ITRF88 to ITRF89 (2)','Parameter values derived from differences in CTs to ITRF2020 (codes 10104 and 10105). These supersede all values published in past IERS/ITRF documentation (CT code 9020). Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9.','EPSG','1033','Position Vector transformation (geocentric domain)','EPSG','4910','EPSG','4911',0.01,0.5,3.6,2.4,'EPSG','1033',-0.1,0.0,0.0,'EPSG','1031',-3.1,'EPSG','1028',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IERS-Wld 2020',0);
INSERT INTO "usage" VALUES('EPSG','18427','helmert_transformation','EPSG','10139','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','10140','ITRF89 to ITRF90 (2)','Parameter values derived from differences in CTs to ITRF2020 (codes 10103 and 10104). These supersede all values published in past IERS/ITRF documentation (CT code 9021). Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9.','EPSG','1033','Position Vector transformation (geocentric domain)','EPSG','4911','EPSG','4912',0.01,-0.5,-2.4,3.8,'EPSG','1033',0.0,0.0,0.0,'EPSG','1031',-3.4,'EPSG','1028',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IERS-Wld 2020',0);
INSERT INTO "usage" VALUES('EPSG','18428','helmert_transformation','EPSG','10140','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','10141','ITRF90 to ITRF91 (2)','Parameter values derived from differences in CTs to ITRF2020 (codes 10100 and 10103). These supersede all values published in past IERS/ITRF documentation (CT code 9022). Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9.','EPSG','1033','Position Vector transformation (geocentric domain)','EPSG','4912','EPSG','4913',0.007,0.2,0.4,1.6,'EPSG','1033',0.0,0.0,0.0,'EPSG','1031',-0.3,'EPSG','1028',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IERS-Wld 2020',0);
INSERT INTO "usage" VALUES('EPSG','18432','helmert_transformation','EPSG','10141','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','10142','ITRF91 to ITRF92 (2)','Parameter values derived from differences in CTs to ITRF2020 (codes 9999 and 10100). These supersede all values published in past IERS/ITRF documentation (CT code 9023). Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9.','EPSG','1033','Position Vector transformation (geocentric domain)','EPSG','4913','EPSG','4914',0.005,-1.2,-1.4,0.6,'EPSG','1033',0.0,0.0,0.0,'EPSG','1031',-1.4,'EPSG','1028',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IERS-Wld 2020',0);
INSERT INTO "usage" VALUES('EPSG','18430','helmert_transformation','EPSG','10142','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','10143','ITRF93 to ITRF94 (2)','Parameter values derived from differences in CTs to ITRF2020 (codes 9997 and 9998). These supersede all values published in past IERS/ITRF documentation (CT code 9025). Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4915','EPSG','4916',0.01,-0.6,0.5,1.5,'EPSG','1033',0.39,-0.8,0.96,'EPSG','1031',-0.49,'EPSG','1028',0.29,-0.04,-0.08,'EPSG','1034',0.11,0.19,-0.05,'EPSG','1032',0.0,'EPSG','1030',1988.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IERS-Wld 2020',0);
INSERT INTO "usage" VALUES('EPSG','18431','helmert_transformation','EPSG','10143','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','10149','MAGNA-SIRGAS 2018 to WGS 84 (1)','See Datum 1329 remarks for conversion path between GeogCRSs "MAGNA-SIRGAS" and "MAGNA-SIRGAS 2018". CTs 10216, 10221, 10242-4 provide recommended conversions between ProjCRSs on "MAGNA-SIRGAS" and "MAGNA-SIRGAS 2018" using null Cartesian Offsets.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','20046','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Col',0);
INSERT INTO "usage" VALUES('EPSG','18935','helmert_transformation','EPSG','10149','EPSG','1070','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','10179','ITRF2020 to IGS20 (1)','Scale difference in ppb where 1/billion = 1E-9 or nm/m. IGS20 is by definition aligned with ITRF2020.','EPSG','1065','Time-specific Position Vector transform (geocen)','EPSG','9988','EPSG','10176',0.0,0.0,0.0,0.0,'EPSG','1033',0.0,0.0,0.0,'EPSG','1031',0.0,'EPSG','1028',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,2015.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IGS-Wld',0);
INSERT INTO "usage" VALUES('EPSG','18634','helmert_transformation','EPSG','10179','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','10180','IGb14 to IGS20 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m. Parameter values from ITRF2014 to ITRF2020 (1) (code 9991) as IGb14 is aligned to ITRF2014 and IGS20 is aligned to ITRF2020.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','9378','EPSG','10176',0.001,1.4,0.9,-1.4,'EPSG','1025',0.0,0.0,0.0,'EPSG','1031',0.42,'EPSG','1028',0.0,0.1,-0.2,'EPSG','1027',0.0,0.0,0.0,'EPSG','1032',0.0,'EPSG','1030',2015.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IGS-Wld',0);
INSERT INTO "usage" VALUES('EPSG','18633','helmert_transformation','EPSG','10180','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','10264','RGSH2020 to WGS 84 (1)','For many purposes RGSH2020 can be considered to be coincident with WGS 84. This approximation ignores the low accuracy of the WGS 84 ensemble and the application of tectonic plate motion to WGS 84 data.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','10299','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Dza',0);
INSERT INTO "usage" VALUES('EPSG','19536','helmert_transformation','EPSG','10264','EPSG','1026','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','10292','ETRS89/DREF91/2016 to ETRF2000 (1)','Accuracy 4.1mm N, 3.3mm E, 10.6mm up.','EPSG','1033','Position Vector transformation (geocentric domain)','EPSG','10282','EPSG','7930',0.1,0.0,0.0,0.0,'EPSG','9001',0.658,-0.208,0.755,'EPSG','1031',0.0,'EPSG','1028',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'AdV-Deu 2016',0);
INSERT INTO "usage" VALUES('EPSG','19279','helmert_transformation','EPSG','10292','EPSG','1103','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','10296','Nord Sahara 1959 to WGS 84 (11)','Given by information source in direction WGS 84 to Nord Sahara 1959. Defines official relationship between WGS 84 and national geodetic system; epoch unspecified. Derived at approximately 40 points in northern Algeria; accuracy applies only to this area.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4307','EPSG','4326',1.0,-267.407,-47.068,446.357,'EPSG','9001',-0.179423,5.577661,-1.27762,'EPSG','9104',1.204866,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'INCT-Dza 2022',0);
INSERT INTO "usage" VALUES('EPSG','19518','helmert_transformation','EPSG','10296','EPSG','1026','EPSG','1026');
INSERT INTO "helmert_transformation" VALUES('EPSG','10321','RGNC91-93 to RGNC15 (1)','See RGNC91-93 to RGNC15 (2) (transformation code 10322) for authoritative transformation between these CRSs. This transformation is used only as the standard transformation T0 within that transformation.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','10307','EPSG','10312',1.0,-0.584,-1.117,1.125,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'BGN-Ncl 1m',0);
INSERT INTO "usage" VALUES('EPSG','19653','helmert_transformation','EPSG','10321','EPSG','3430','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','10324','RGNC15 to WGS 84 (1)','Accuracy decreasing at approximately 6cm per year due to tectonic plate motion.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','10310','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IOGP-Ncl',0);
INSERT INTO "usage" VALUES('EPSG','19656','helmert_transformation','EPSG','10324','EPSG','1174','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','10330','BH_ETRS89 to ETRS89 (1)','BH_ETRS89 is a national realization of ETRS89. May be taken as approximate transformation BH_ETRS89 to WGS 84 - see code 10333.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','10328','EPSG','4258',0.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IOGP-Bih',0);
INSERT INTO "usage" VALUES('EPSG','19745','helmert_transformation','EPSG','10330','EPSG','1050','EPSG','1161');
INSERT INTO "helmert_transformation" VALUES('EPSG','10333','BH_ETRS89 to WGS 84 (1)','Approximation at the +/- 1m level assuming that ETRS89 is equivalent to WGS 84. BH_ETRS89 is a national realization of ETRS89.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','10328','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IOGP-Bih',0);
INSERT INTO "usage" VALUES('EPSG','19744','helmert_transformation','EPSG','10333','EPSG','1050','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','10334','ITRF2020 to NAD83(2011) (1)','Concatenation of joint US-Canada tfm NAD83(CORS96)>ITRF96 (CT code 6864) with IGS value for ITRF96>ITRF97 and IERS values for ITRF97>ITRF2020 CTs. Scale difference in ppb and rate in ppb/yr where 1/billion = 1E-9. See tfm 10415 for Canadian equivalent.','EPSG','1056','Time-dependent Coordinate Frame rotation (geocen)','EPSG','9988','EPSG','6317',0.0,1003.9,-1909.61,-541.17,'EPSG','1025',26.78138,-0.42027,10.93206,'EPSG','1031',-0.05109,'EPSG','1028',0.79,-0.7,-1.24,'EPSG','1027',0.06667,-0.75744,-0.05133,'EPSG','1032',-0.07201,'EPSG','1030',2010.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'NGS-Usa CONUS-AK PRVI',0);
INSERT INTO "usage" VALUES('EPSG','19772','helmert_transformation','EPSG','10334','EPSG','1511','EPSG','1026');
INSERT INTO "helmert_transformation" VALUES('EPSG','10335','ITRF2014 to NAD83(PA11) (1)','','EPSG','1056','Time-dependent Coordinate Frame rotation (geocen)','EPSG','7789','EPSG','6320',0.0,0.9109,-2.0129,-0.5863,'EPSG','9001',22.749,26.56,-25.706,'EPSG','1031',2.12,'EPSG','1028',0.0001,0.0001,-0.0019,'EPSG','1042',-0.384,1.007,-2.186,'EPSG','1032',0.11,'EPSG','1030',2010.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'NGS-PA',0);
INSERT INTO "usage" VALUES('EPSG','19769','helmert_transformation','EPSG','10335','EPSG','4162','EPSG','1026');
INSERT INTO "helmert_transformation" VALUES('EPSG','10336','ITRF2020 to NAD83(PA11) (1)','','EPSG','1056','Time-dependent Coordinate Frame rotation (geocen)','EPSG','9988','EPSG','6320',0.0,909.5,-2013.3,-585.9,'EPSG','1025',22.749,26.56,-25.706,'EPSG','1031',1.7,'EPSG','1028',0.1,0.0,-1.7,'EPSG','1027',-0.384,1.007,-2.186,'EPSG','1032',0.11,'EPSG','1030',2010.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'NGS-PA',0);
INSERT INTO "usage" VALUES('EPSG','19773','helmert_transformation','EPSG','10336','EPSG','4162','EPSG','1026');
INSERT INTO "helmert_transformation" VALUES('EPSG','10337','ITRF2014 to NAD83(MA11) (1)','','EPSG','1056','Time-dependent Coordinate Frame rotation (geocen)','EPSG','7789','EPSG','6323',0.0,0.9109,-2.0129,-0.5863,'EPSG','9001',28.711,11.785,4.417,'EPSG','1031',2.12,'EPSG','1028',0.0001,0.0001,-0.0019,'EPSG','1042',-0.02,0.105,-0.347,'EPSG','1032',0.11,'EPSG','1030',2010.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'NGS-MA',0);
INSERT INTO "usage" VALUES('EPSG','19770','helmert_transformation','EPSG','10337','EPSG','4167','EPSG','1026');
INSERT INTO "helmert_transformation" VALUES('EPSG','10338','ITRF2020 to NAD83(MA11) (1)','','EPSG','1056','Time-dependent Coordinate Frame rotation (geocen)','EPSG','9988','EPSG','6323',0.0,909.5,-2013.3,-585.9,'EPSG','1025',28.711,11.785,4.417,'EPSG','1031',1.7,'EPSG','1028',0.1,0.0,-1.7,'EPSG','1027',-0.02,0.105,-0.347,'EPSG','1032',0.11,'EPSG','1030',2010.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'NGS-MA',0);
INSERT INTO "usage" VALUES('EPSG','19774','helmert_transformation','EPSG','10338','EPSG','4167','EPSG','1026');
INSERT INTO "helmert_transformation" VALUES('EPSG','10339','Nord Sahara 1959 to RGSH2020 (1)','Parameter values taken from Nord Sahara 1959 to WGS 84 (4) (code 1815) assuming that RGSH2020 is coincident with WGS 84 within the accuracy of the transformation. CT 1815 used by BP in District 3 and In Salah Gas.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4307','EPSG','10299',5.0,-152.9,43.8,358.3,'EPSG','9001',2.714,1.386,-2.788,'EPSG','9104',-6.743,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IOGP-Alg D3',0);
INSERT INTO "usage" VALUES('EPSG','19855','helmert_transformation','EPSG','10339','EPSG','2598','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','10340','Nord Sahara 1959 to RGSH2020 (2)','Parameter values taken from Nord Sahara 1959 to WGS 84 (5) (code 1816) assuming that RGSH2020 is coincident with WGS 84 within the accuracy of the transformation. Significant and varying differences (>100m) at 4 neighbouring astro stations.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4307','EPSG','10299',100.0,-95.7,10.2,158.9,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IOGP-Alg InAm',0);
INSERT INTO "usage" VALUES('EPSG','19850','helmert_transformation','EPSG','10340','EPSG','2599','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','10341','Nord Sahara 1959 to RGSH2020 (3)','Parameter values taken from Nord Sahara 1959 to WGS 84 (6) (code 1817) assuming that RGSH2020 is coincident with WGS 84 within the accuracy of the transformation. Derived at astro station Guerrara.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4307','EPSG','10299',100.0,-165.914,-70.607,305.009,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IOGP-Alg HBR',0);
INSERT INTO "usage" VALUES('EPSG','19851','helmert_transformation','EPSG','10341','EPSG','2600','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','10342','Nord Sahara 1959 to RGSH2020 (4)','Parameter values taken from Nord Sahara 1959 to WGS 84 (7) (code 15874) assuming that RGSH2020 is coincident with WGS 84 within the accuracy of the transformation. Derived at 11 stations. CT 15874 used by Equinor in Hassi Mouina.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4307','EPSG','10299',5.0,-169.559,-72.34,303.102,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IOGP-Dza Mou',0);
INSERT INTO "usage" VALUES('EPSG','19852','helmert_transformation','EPSG','10342','EPSG','3402','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','10343','Nord Sahara 1959 to RGSH2020 (5)','Parameter values taken from Nord Sahara 1959 to WGS 84 (8) (code 5630) assuming that RGSH2020 is coincident with WGS 84 within the accuracy of the transformation. Derived at 1 station (L38).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4307','EPSG','10299',5.0,-168.52,-72.05,304.3,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IOGP-Dza Ahnet',0);
INSERT INTO "usage" VALUES('EPSG','19853','helmert_transformation','EPSG','10343','EPSG','3917','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','10344','Nord Sahara 1959 to RGSH2020 (6)','Parameter values taken from Nord Sahara 1959 to WGS 84 (10) (code 7442) assuming that RGSH2020 is coincident with WGS 84 within the accuracy of the transformation. Significant and varying differences (>100m) known to exist in neighbouring astro stations.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4307','EPSG','10299',100.0,-181.7,64.7,247.2,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IOGP-Alg Ain Tsila',0);
INSERT INTO "usage" VALUES('EPSG','19854','helmert_transformation','EPSG','10344','EPSG','4382','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','10415','ITRF2020 to NAD83(CSRS)v8 (1)','Concatenation of joint Canada-US tfm NAD83>ITRF96 (see tfm code 8259) with IGS value for ITRF96>ITRF97 and IERS values for ITRF97>ITRF2020 transformations. Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','9988','EPSG','10412',0.0,1.0039,-1.90961,-0.54117,'EPSG','9001',-26.78138,0.42027,-10.93206,'EPSG','1031',-0.05109,'EPSG','1028',0.00079,-0.0007,-0.00124,'EPSG','1042',-0.06667,0.75744,0.05133,'EPSG','1032',-0.07201,'EPSG','1030',2010.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'NRC-Can CSRSv8',0);
INSERT INTO "usage" VALUES('EPSG','20192','helmert_transformation','EPSG','10415','EPSG','1061','EPSG','1284');
INSERT INTO "helmert_transformation" VALUES('EPSG','10416','NAD83(CSRS)v7 to NAD83(2011) (1)','Scale difference in ppb where 1/billion = 1E-9 or nm/m. Source and target CRSs defined from ITRF2014 by common transformations (codes 8970 and 8265).','EPSG','1033','Position Vector transformation (geocentric domain)','EPSG','8253','EPSG','6317',0.0,0.0,0.0,0.0,'EPSG','9001',0.0,0.0,0.0,'EPSG','1031',0.0,'EPSG','1028',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'ISO-N Am',0);
INSERT INTO "usage" VALUES('EPSG','20197','helmert_transformation','EPSG','10416','EPSG','4544','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','10419','NAD83(CSRS)v8 to NAD83(2011) (1)','Scale difference in ppb where 1/billion = 1E-9 or nm/m. Source and target CRSs defined from ITRF2020 by common transformations (codes 10334 and 10415).','EPSG','1033','Position Vector transformation (geocentric domain)','EPSG','10412','EPSG','6317',0.0,0.0,0.0,0.0,'EPSG','9001',0.0,0.0,0.0,'EPSG','1031',0.0,'EPSG','1028',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'ISO-N Am',0);
INSERT INTO "usage" VALUES('EPSG','20195','helmert_transformation','EPSG','10419','EPSG','4544','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','10478','BBT2000 to WGS 84 (1)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','10475','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'BBT-Aut-Ita',0);
INSERT INTO "usage" VALUES('EPSG','20343','helmert_transformation','EPSG','10478','EPSG','4744','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','10511','NAD83(CSRS)v2 to NAD83(CSRS)v3 (3)','Scale difference in ppb where 1/billion = 1E-9 or nm/m. NAD83(CSRS)v2 and NAD83(CSRS)v3 are referenced to the same coordinate reference epoch (1997.00) and considered compatible with each other at the centimetre level.','EPSG','1033','Position Vector transformation (geocentric domain)','EPSG','8233','EPSG','8238',0.01,0.0,0.0,0.0,'EPSG','9001',0.0,0.0,0.0,'EPSG','1031',0.0,'EPSG','1028',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NRCan-Can 1997',0);
INSERT INTO "usage" VALUES('EPSG','20612','helmert_transformation','EPSG','10511','EPSG','1061','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','10512','NAD83(CSRS)v6 to NAD83(CSRS)v7 (1)','Scale difference in ppb where 1/billion = 1E-9 or nm/m. NAD83(CSRS)v6 and NAD83(CSRS)v7 are referenced to the same coordinate reference epoch (2010.00) and considered compatible with each other at the centimetre level.','EPSG','1033','Position Vector transformation (geocentric domain)','EPSG','8250','EPSG','8253',0.01,0.0,0.0,0.0,'EPSG','9001',0.0,0.0,0.0,'EPSG','1031',0.0,'EPSG','1028',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NRCan-Can 2010',0);
INSERT INTO "usage" VALUES('EPSG','20890','helmert_transformation','EPSG','10512','EPSG','1061','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','10513','NAD83(CSRS)v6 to NAD83(CSRS)v8 (1)','Scale difference in ppb where 1/billion = 1E-9 or nm/m. NAD83(CSRS)v6 and NAD83(CSRS)v8 are referenced to the same coordinate reference epoch (2010.00) and considered compatible with each other at the centimetre level.','EPSG','1033','Position Vector transformation (geocentric domain)','EPSG','8250','EPSG','10412',0.01,0.0,0.0,0.0,'EPSG','9001',0.0,0.0,0.0,'EPSG','1031',0.0,'EPSG','1028',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NRCan-Can 2010',0);
INSERT INTO "usage" VALUES('EPSG','20614','helmert_transformation','EPSG','10513','EPSG','1061','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','10514','NAD83(CSRS)v7 to NAD83(CSRS)v8 (1)','Scale difference in ppb where 1/billion = 1E-9 or nm/m. NAD83(CSRS)7 and NAD83(CSRS)v8 are referenced to the same coordinate reference epoch (2010.00) and considered compatible with each other at the millimetre level.','EPSG','1033','Position Vector transformation (geocentric domain)','EPSG','8253','EPSG','10412',0.01,0.0,0.0,0.0,'EPSG','9001',0.0,0.0,0.0,'EPSG','1031',0.0,'EPSG','1028',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NRCan-Can 2010',0);
INSERT INTO "usage" VALUES('EPSG','20891','helmert_transformation','EPSG','10514','EPSG','1061','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','10543','GDM2000 to WGS 84 (1)','Approximation at the +/- 1m level assuming that GDM2000 is equivalent to WGS 84 within the accuracy of the transformation.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4742','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IOGP-Mys 1m',0);
INSERT INTO "usage" VALUES('EPSG','20831','helmert_transformation','EPSG','10543','EPSG','1151','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','10572','ITRF2020 to ETRF2020 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m. See ITRF2020 to ETRF2020 (2) (code 10573) for an exactly equivalent transformation but with the transformation''s parameter values at epoch 2015.00.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','9988','EPSG','10569',0.0,0.0,0.0,0.0,'EPSG','1025',0.0,0.0,0.0,'EPSG','1031',0.0,'EPSG','1028',0.0,0.0,0.0,'EPSG','1027',0.086,0.519,-0.753,'EPSG','1032',0.0,'EPSG','1030',1989.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'EUREF-Eur',0);
INSERT INTO "usage" VALUES('EPSG','21229','helmert_transformation','EPSG','10572','EPSG','1298','EPSG','1289');
INSERT INTO "helmert_transformation" VALUES('EPSG','10573','ITRF2020 to ETRF2020 (2)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9. See ITRF2020 to ETRF2020 (1) (code 10572) for transformation which defines ETRF2020. This CT 10573 is equivalent to 10572 but with parameter values at epoch 2015.00.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','9988','EPSG','10569',0.0,0.0,0.0,0.0,'EPSG','1025',2.236,13.494,-19.578,'EPSG','1031',0.0,'EPSG','1028',0.0,0.0,0.0,'EPSG','1027',0.086,0.519,-0.753,'EPSG','1032',0.0,'EPSG','1030',2015.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'EUREF-Eur 2024',0);
INSERT INTO "usage" VALUES('EPSG','21230','helmert_transformation','EPSG','10573','EPSG','1298','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','10574','ITRF2014 to ETRF2020 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','7789','EPSG','10569',0.0,1.4,0.9,-1.4,'EPSG','1025',2.236,13.494,-19.578,'EPSG','1031',0.42,'EPSG','1028',0.0,0.1,-0.2,'EPSG','1027',0.086,0.519,-0.753,'EPSG','1032',0.0,'EPSG','1030',2015.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'EUREF-Eur 2024',0);
INSERT INTO "usage" VALUES('EPSG','21231','helmert_transformation','EPSG','10574','EPSG','1298','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','10575','ITRF2008 to ETRF2020 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','5332','EPSG','10569',0.0,-0.2,-1.0,-3.3,'EPSG','1025',2.236,13.494,-19.578,'EPSG','1031',0.29,'EPSG','1028',0.0,0.1,-0.1,'EPSG','1027',0.086,0.519,-0.753,'EPSG','1032',-0.03,'EPSG','1030',2015.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'EUREF-Eur 2024',0);
INSERT INTO "usage" VALUES('EPSG','21224','helmert_transformation','EPSG','10575','EPSG','1298','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','10576','ITRF2005 to ETRF2020 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4896','EPSG','10569',0.0,-2.7,-0.1,1.4,'EPSG','1025',2.236,13.494,-19.578,'EPSG','1031',-0.65,'EPSG','1028',-0.3,0.1,-0.1,'EPSG','1027',0.086,0.519,-0.753,'EPSG','1032',-0.03,'EPSG','1030',2015.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'EUREF-Eur 2024',0);
INSERT INTO "usage" VALUES('EPSG','21244','helmert_transformation','EPSG','10576','EPSG','1298','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','10577','ITRF2000 to ETRF2020 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4919','EPSG','10569',0.0,0.2,-0.8,34.2,'EPSG','1025',2.236,13.494,-19.578,'EPSG','1031',-2.25,'EPSG','1028',-0.1,0.0,1.7,'EPSG','1027',0.086,0.519,-0.753,'EPSG','1032',-0.11,'EPSG','1030',2015.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'EUREF-Eur 2024',0);
INSERT INTO "usage" VALUES('EPSG','21290','helmert_transformation','EPSG','10577','EPSG','1298','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','10578','ITRF97 to ETRF2020 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4918','EPSG','10569',0.0,-6.5,3.9,77.9,'EPSG','1025',2.236,13.494,-19.938,'EPSG','1031',-3.98,'EPSG','1028',-0.1,0.6,3.1,'EPSG','1027',0.086,0.519,-0.773,'EPSG','1032',-0.12,'EPSG','1030',2015.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'EUREF-Eur 2024',0);
INSERT INTO "usage" VALUES('EPSG','21246','helmert_transformation','EPSG','10578','EPSG','1298','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','10579','ITRF96 to ETRF2020 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4917','EPSG','10569',0.0,-6.5,3.9,77.9,'EPSG','1025',2.236,13.494,-19.938,'EPSG','1031',-3.98,'EPSG','1028',-0.1,0.6,3.1,'EPSG','1027',0.086,0.519,-0.773,'EPSG','1032',-0.12,'EPSG','1030',2015.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'EUREF-Eur 2024',0);
INSERT INTO "usage" VALUES('EPSG','21247','helmert_transformation','EPSG','10579','EPSG','1298','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','10580','ITRF94 to ETRF2020 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4916','EPSG','10569',0.0,-6.5,3.9,77.9,'EPSG','1025',2.236,13.494,-19.938,'EPSG','1031',-3.98,'EPSG','1028',-0.1,0.6,3.1,'EPSG','1027',0.086,0.519,-0.773,'EPSG','1032',-0.12,'EPSG','1030',2015.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'EUREF-Eur 2024',0);
INSERT INTO "usage" VALUES('EPSG','21248','helmert_transformation','EPSG','10580','EPSG','1298','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','10581','ITRF93 to ETRF2020 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4915','EPSG','10569',0.0,65.8,-1.9,71.3,'EPSG','1025',5.596,17.824,-20.328,'EPSG','1031',-4.47,'EPSG','1028',2.8,0.2,2.3,'EPSG','1027',0.196,0.709,-0.823,'EPSG','1032',-0.12,'EPSG','1030',2015.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'EUREF-Eur 2024',0);
INSERT INTO "usage" VALUES('EPSG','21249','helmert_transformation','EPSG','10581','EPSG','1298','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','10582','ITRF92 to ETRF2020 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4914','EPSG','10569',0.0,-14.5,1.9,85.9,'EPSG','1025',2.236,13.494,-19.938,'EPSG','1031',-3.27,'EPSG','1028',-0.1,0.6,3.1,'EPSG','1027',0.086,0.519,-0.773,'EPSG','1032',-0.12,'EPSG','1030',2015.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'EUREF-Eur 2024',0);
INSERT INTO "usage" VALUES('EPSG','21250','helmert_transformation','EPSG','10582','EPSG','1298','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','10583','ITRF91 to ETRF2020 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4913','EPSG','10569',0.0,-26.5,-12.1,91.9,'EPSG','1025',2.236,13.494,-19.938,'EPSG','1031',-4.67,'EPSG','1028',-0.1,0.6,3.1,'EPSG','1027',0.086,0.519,-0.773,'EPSG','1032',-0.12,'EPSG','1030',2015.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'EUREF-Eur 2024',0);
INSERT INTO "usage" VALUES('EPSG','21251','helmert_transformation','EPSG','10583','EPSG','1298','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','10584','ITRF90 to ETRF2020 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4912','EPSG','10569',0.0,-24.5,-8.1,107.9,'EPSG','1025',2.236,13.494,-19.938,'EPSG','1031',-4.97,'EPSG','1028',-0.1,0.6,3.1,'EPSG','1027',0.086,0.519,-0.773,'EPSG','1032',-0.12,'EPSG','1030',2015.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'EUREF-Eur 2024',0);
INSERT INTO "usage" VALUES('EPSG','21252','helmert_transformation','EPSG','10584','EPSG','1298','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','10585','ITRF89 to ETRF2020 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','4911','EPSG','10569',0.0,-29.5,-32.1,145.9,'EPSG','1025',2.236,13.494,-19.938,'EPSG','1031',-8.37,'EPSG','1028',-0.1,0.6,3.1,'EPSG','1027',0.086,0.519,-0.773,'EPSG','1032',-0.12,'EPSG','1030',2015.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'EUREF-Eur 2024',0);
INSERT INTO "usage" VALUES('EPSG','21253','helmert_transformation','EPSG','10585','EPSG','1298','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','10586','ITRF2020 to ETRF2000 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','9988','EPSG','7930',0.0,53.8,51.8,-82.2,'EPSG','1025',2.106,12.74,-20.592,'EPSG','1031',2.25,'EPSG','1028',0.1,0.0,-1.7,'EPSG','1027',0.081,0.49,-0.792,'EPSG','1032',0.11,'EPSG','1030',2015.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'EUREF-Eur 2024',0);
INSERT INTO "usage" VALUES('EPSG','21243','helmert_transformation','EPSG','10586','EPSG','1298','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','10587','ITRF2020 to ETRF2014 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','9988','EPSG','8401',0.0,-1.4,-0.9,1.4,'EPSG','1025',2.21,13.806,-20.02,'EPSG','1031',-0.42,'EPSG','1028',0.0,-0.1,0.2,'EPSG','1027',0.085,0.531,-0.77,'EPSG','1032',0.0,'EPSG','1030',2015.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'EUREF-Eur 2024',0);
INSERT INTO "usage" VALUES('EPSG','21242','helmert_transformation','EPSG','10587','EPSG','1298','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','10589','ITRF2014 to Segara (1)','Replaces transformation Segara to WGS 84 (4) (code 1838). Computed by PT Pageo Utama and PT Bhumi Warih Geohydromatics in May 2023. Transformation parameters were derived at epoch 2022.00.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','9000','EPSG','4613',0.2,407.379,-685.226,-52.577,'EPSG','9001',-0.318,0.107,-0.058,'EPSG','9104',0.207,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'PHM-Idn Mhk 2023',0);
INSERT INTO "usage" VALUES('EPSG','21255','helmert_transformation','EPSG','10589','EPSG','1328','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','10607','WGS 84 (G2139) to WGS 84 (G2296) (1)','Scale difference in ppb where 1/billion = 1E-9 or nm/m. Calculated by extrapolating the position and velocity of the seventeen WGS 84 GPS monitoring sites to the epoch 2024.00.','EPSG','1032','Coordinate Frame rotation (geocentric domain)','EPSG','9753','EPSG','10604',0.01,2.6,5.4,-0.9,'EPSG','1025',-0.01,-0.07,0.0,'EPSG','1031',0.06,'EPSG','1028',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NGA-Wld 2024.0',0);
INSERT INTO "usage" VALUES('EPSG','21197','helmert_transformation','EPSG','10607','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','10608','WGS 84 (G2296) to ITRF2020 (1)','Scale difference in ppb where 1/billion = 1E-9 or nm/m. Accuracy of 1cm applies at epoch 2024.0. Due to subsequent drift between the reference frames it may reach 2cm at other epochs.','EPSG','1032','Coordinate Frame rotation (geocentric domain)','EPSG','10604','EPSG','9988',0.01,0.0,0.0,0.0,'EPSG','1025',0.0,0.0,0.0,'EPSG','1031',0.0,'EPSG','1028',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NGA-Wld 2024.0',0);
INSERT INTO "usage" VALUES('EPSG','21198','helmert_transformation','EPSG','10608','EPSG','1262','EPSG','1026');
INSERT INTO "helmert_transformation" VALUES('EPSG','10646','Saba to BES2020 Saba (1)','For the reverse transformation from BES2020 Saba to Saba, method 1140 [used here] is reversible using the same parameter values with the reverse formulas (see GN7-2). However, the official BESTRANS document applies the same formulas for both the forward and reverse transformations, but uses different parameter values for the reverse transformation.','EPSG','1140','Coordinate Frame rotation full matrix (geog3D)','EPSG','10635','EPSG','10638',0.05,1138.7432,-2064.4761,110.7016,'EPSG','9001',-214.615206,479.360036,-164.703951,'EPSG','9104',-402.32073,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NSGI-Bes Saba 2020',0);
INSERT INTO "usage" VALUES('EPSG','21861','helmert_transformation','EPSG','10646','EPSG','4757','EPSG','1142');
INSERT INTO "helmert_transformation" VALUES('EPSG','10647','BES2020 Saba to ITRF2014 (1)','Time-dependent component of official transformation BESTRANS2020.','EPSG','1056','Time-dependent Coordinate Frame rotation (geocen)','EPSG','10637','EPSG','7789',0.05,0.0,0.0,0.0,'EPSG','9001',0.0,0.0,0.0,'EPSG','9104',0.0,'EPSG','9202',0.00726,0.00848,0.01353,'EPSG','1042',0.0,0.0,0.0,'EPSG','1043',0.0,'EPSG','1041',2020.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'NSGI-Bes Saba',0);
INSERT INTO "usage" VALUES('EPSG','21765','helmert_transformation','EPSG','10647','EPSG','4757','EPSG','1079');
INSERT INTO "helmert_transformation" VALUES('EPSG','10648','BES2020 Saba to WGS 84 (1)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','10639','EPSG','4326',0.5,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NSGI-Bes Saba',0);
INSERT INTO "usage" VALUES('EPSG','21743','helmert_transformation','EPSG','10648','EPSG','4757','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','10676','Saba to WGS 84 (1)','Parameter values taken from Saba to BES2020 Saba (1) (transformation code 10646) assuming that BES2020 Saba is coincident with WGS 84 within the accuracy of the transformation.','EPSG','1133','Coordinate Frame rotation full matrix (geog2D)','EPSG','10636','EPSG','4326',1.0,1138.7432,-2064.4761,110.7016,'EPSG','9001',-214.615206,479.360036,-164.703951,'EPSG','9104',-402.32073,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IOGP-Bes Saba',0);
INSERT INTO "usage" VALUES('EPSG','21783','helmert_transformation','EPSG','10676','EPSG','4757','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','10682','RGM04 to RGM23 (1)','Approximation at the +/- 0.1m level. For more accurate official transformation see RGM04 to RGM23 (2) (code 10683).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4470','EPSG','10671',0.1,-0.5377,0.3946,0.3608,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Myt 10cm',0);
INSERT INTO "usage" VALUES('EPSG','21834','helmert_transformation','EPSG','10682','EPSG','1159','EPSG','1189');
INSERT INTO "helmert_transformation" VALUES('EPSG','10684','RGM23 to WGS 84 (1)','Approximation at the +/- 1m level assuming that RGM23 is equivalent to WGS 84.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','10671','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IOGP-Myt',0);
INSERT INTO "usage" VALUES('EPSG','21859','helmert_transformation','EPSG','10684','EPSG','1159','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','10701','EUREF-FIN to WGS 84 (1)','Approximation at the +/- 1m level assuming that ETRS89 is equivalent to WGS 84. EUREF-FIN is a national realization of ETRS89.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','10690','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IOGP-Fin',0);
INSERT INTO "usage" VALUES('EPSG','22008','helmert_transformation','EPSG','10701','EPSG','1095','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','10706','EUREF-FIN to ETRS89 (1)','EUREF-FIN is a national realization of ETRS89. May be taken as approximate transformation EUREF-FIN to WGS 84 - see code 10701.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','10690','EPSG','4258',0.1,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IOGP-Fin',0);
INSERT INTO "usage" VALUES('EPSG','22009','helmert_transformation','EPSG','10706','EPSG','1095','EPSG','1161');
INSERT INTO "helmert_transformation" VALUES('EPSG','10748','BES2020 Sint Eustatius to ITRF2014 (1)','Time-dependent component of official transformation BESTRANS2020.','EPSG','1056','Time-dependent Coordinate Frame rotation (geocen)','EPSG','10737','EPSG','7789',0.05,0.0,0.0,0.0,'EPSG','9001',0.0,0.0,0.0,'EPSG','9104',0.0,'EPSG','9202',0.00743,0.00875,0.01402,'EPSG','1042',0.0,0.0,0.0,'EPSG','1043',0.0,'EPSG','1041',2020.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'NSGI-Bes SEu 2020',0);
INSERT INTO "usage" VALUES('EPSG','22303','helmert_transformation','EPSG','10748','EPSG','4788','EPSG','1079');
INSERT INTO "helmert_transformation" VALUES('EPSG','10749','BES2020 Sint Eustatius to WGS 84 (1)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','10739','EPSG','4326',0.5,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NSGI-Bes SEu 2020',0);
INSERT INTO "usage" VALUES('EPSG','22311','helmert_transformation','EPSG','10749','EPSG','4788','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','10750','Sint Eustatius to BES2020 Sint Eustatius (1)','For the reverse transformation from BES2020 Sint Eustatius to Sint Eustatius, method 1140 [used here] is reversible using the same parameter values with the reverse formulas (see GN7-2). However, the official BESTRANS document applies the same formulas for both the forward and reverse transformations, but uses different parameter values for the reverse transformation.','EPSG','1140','Coordinate Frame rotation full matrix (geog3D)','EPSG','10735','EPSG','10738',0.05,1276.2485,-2016.6406,667.4403,'EPSG','9001',-101.005288,212.913401,-68.43277,'EPSG','9104',-431.59604,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NSGI-Bes SEu 2020',0);
INSERT INTO "usage" VALUES('EPSG','22306','helmert_transformation','EPSG','10750','EPSG','4788','EPSG','1142');
INSERT INTO "helmert_transformation" VALUES('EPSG','10766','Bonaire to Bonaire 2004 (1)','For the reverse transformation from Bonaire 2004 to Bonaire, method 1133 [used here] is reversible using the same parameter values with the reverse formulas (see GN7-2). However, the official BESTRANS document applies the same formulas for both the forward and reverse transformations, but uses different parameter values for the reverse transformation.','EPSG','1133','Coordinate Frame rotation full matrix (geog2D)','EPSG','10758','EPSG','10762',0.05,-366.1939,-115.0688,-776.7039,'EPSG','9001',20.96308,16.462749,-14.276379,'EPSG','9104',-12.809,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NSGI-Bes Bon',0);
INSERT INTO "usage" VALUES('EPSG','22270','helmert_transformation','EPSG','10766','EPSG','3822','EPSG','1142');
INSERT INTO "helmert_transformation" VALUES('EPSG','10769','Bonaire to WGS 84 (1)','Parameter values taken from Bonaire to Bonaire 2004 (1) (transformation code 10766) assuming that Bonaire 2004 is coincident with WGS 84 within the accuracy of the transformation.','EPSG','1133','Coordinate Frame rotation full matrix (geog2D)','EPSG','10758','EPSG','4326',1.0,-366.1939,-115.0688,-776.7039,'EPSG','9001',20.96308,16.462749,-14.276379,'EPSG','9104',-12.809,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IOGP-Bes Bon',0);
INSERT INTO "usage" VALUES('EPSG','22274','helmert_transformation','EPSG','10769','EPSG','3822','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','10770','Bonaire 2004 to WGS 84 (1)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','10762','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NSGI-Bes Bon',0);
INSERT INTO "usage" VALUES('EPSG','22193','helmert_transformation','EPSG','10770','EPSG','3822','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','10771','UZGD2024 to WGS 84 (1)','Approximation at the +/- 1m level assuming that UZGD2024 is equivalent to WGS 84 within the accuracy of the transformation.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','10725','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IOGP-Uzb 1m',0);
INSERT INTO "usage" VALUES('EPSG','22195','helmert_transformation','EPSG','10771','EPSG','1248','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','10777','ITRF2020 to KSA-GRF17 (1)','Derived by combining the IERS transformation from ITRF2020 to ITRF2014 at epoch 2017.0 (code 9991) with ITRF2014 to KSA-GRF17 transformation (code 9334).','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','9988','EPSG','9331',0.001,-1.4,-1.1,1.8,'EPSG','1025',0.0,0.0,0.0,'EPSG','1031',-0.42,'EPSG','1028',0.0,-0.1,0.2,'EPSG','1027',-1.199,0.107,-1.468,'EPSG','1032',0.0,'EPSG','1030',2017.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'GEOSA-Sau',0);
INSERT INTO "usage" VALUES('EPSG','22256','helmert_transformation','EPSG','10777','EPSG','1206','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','10782','ITRF2020 to ITRF2020-u2023 (1)','The alignment of ITRF2020-u2023 to ITRF2020 implies that all transformation parameters between ITRF2020-u2023 and ITRF2020 are zero.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','9988','EPSG','10779',0.001,0.0,0.0,0.0,'EPSG','1025',0.0,0.0,0.0,'EPSG','1031',0.0,'EPSG','1028',0.0,0.0,0.0,'EPSG','1027',0.0,0.0,0.0,'EPSG','1032',0.0,'EPSG','1030',2015.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IERS-Wld',0);
INSERT INTO "usage" VALUES('EPSG','22374','helmert_transformation','EPSG','10782','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','10786','ITRF2020-u2023 to IGb20 (1)','Scale difference in ppb where 1/billion = 1E-9 or nm/m. IGb20 is by definition aligned with ITRF2020-u2023.','EPSG','1065','Time-specific Position Vector transform (geocen)','EPSG','10779','EPSG','10783',0.0,0.0,0.0,0.0,'EPSG','1033',0.0,0.0,0.0,'EPSG','1031',0.0,'EPSG','1028',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,2015.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IGS-Wld',0);
INSERT INTO "usage" VALUES('EPSG','22366','helmert_transformation','EPSG','10786','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','10787','IGS20 to IGb20 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m. Parameter values from ITRF2020 to ITRF2020-u2023 (1) (code 10782) as IGS20 is aligned to ITRF2020 and IGb20 is aligned to ITRF2020-u2023.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','10176','EPSG','10783',0.001,0.0,0.0,0.0,'EPSG','1025',0.0,0.0,0.0,'EPSG','1031',0.0,'EPSG','1028',0.0,0.0,0.0,'EPSG','1027',0.0,0.0,0.0,'EPSG','1032',0.0,'EPSG','1030',2015.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IGS-Wld',0);
INSERT INTO "usage" VALUES('EPSG','22369','helmert_transformation','EPSG','10787','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','10796','Arc 1960 to UGRF (1)','Derived in 2019 using 18 points. RMS 0.5m - 1m.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4210','EPSG','10791',1.0,-136.7231,-87.8654,20.1215,'EPSG','9001',4.966933,-9.01001,-2.72486,'EPSG','9104',7.86009,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'MLHUD-Uga',0);
INSERT INTO "usage" VALUES('EPSG','22467','helmert_transformation','EPSG','10796','EPSG','1241','EPSG','1151');
INSERT INTO "helmert_transformation" VALUES('EPSG','10797','UGRF to WGS 84 (1)','Approximation at the +/- 1m level assuming that UGRF is equivalent to WGS 84 within the accuracy of the transformation.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','10791','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Uga',0);
INSERT INTO "usage" VALUES('EPSG','22465','helmert_transformation','EPSG','10797','EPSG','1241','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','10803','LibRef21 to WGS 84 (1)','Approximation at the +/- 1m level assuming that LibRef21 is equivalent to WGS 84 within the accuracy of the transformation.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','10800','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IOGP-Lbr',0);
INSERT INTO "usage" VALUES('EPSG','22441','helmert_transformation','EPSG','10803','EPSG','1142','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','10804','Sint Eustatius to WGS 84 (1)','Parameter values taken from Sint Eustatius to BES2020 Sint Eustatius (1) (transformation code 10750) assuming that BES2020 Sint Eustatius is coincident with WGS 84 within the accuracy of the transformation.','EPSG','1133','Coordinate Frame rotation full matrix (geog2D)','EPSG','10736','EPSG','4326',1.0,1276.2485,-2016.6406,667.4403,'EPSG','9001',-101.005288,212.913401,-68.43277,'EPSG','9104',-431.59604,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IOGP-Bes Seu',0);
INSERT INTO "usage" VALUES('EPSG','22543','helmert_transformation','EPSG','10804','EPSG','4788','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','10834','Georgia Geodetic Datum to WGS 84 (1)','Derived at 5 stations. Note: information source states that the Helmert formula use the Position Vector convention, but using the station coordinates tabulated in the report IOGP believes that the parameter values are consistent with use of the coordinate frame rotation convention.','EPSG','1032','Coordinate Frame rotation (geocentric domain)','EPSG','10829','EPSG','4978',0.03,-2.0796,-0.3484,1.7009,'EPSG','9001',0.05465,-0.06718,0.06143,'EPSG','9104',0.0181,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NAPR-Geo 2013',0);
INSERT INTO "usage" VALUES('EPSG','23615','helmert_transformation','EPSG','10834','EPSG','1102','EPSG','1026');
INSERT INTO "helmert_transformation" VALUES('EPSG','10835','Pulkovo 1942 to Georgia Geodetic Datum (1)','Derived at 16 stations. Replaces transformation derived at 9 stations given in "Methodology and Parameters for Datum Transformation between the New and Old Reference Systems", European Union ENPI Programme for Georgia National Agency of Public Registry (NAPR), Ministry of Justice of Georgia, November 14th 2013. Note: information source states that the Helmert formula use the Position Vector convention, but using the station coordinates tabulated in the report IOGP believes that the parameter values are consistent with use of the coordinate frame rotation convention.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4284','EPSG','10831',0.1,-40.7436,-40.0018,-56.707,'EPSG','9001',-1.2753,-1.42112,2.69445,'EPSG','9104',-4.5284,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NAPR-Geo 2014',0);
INSERT INTO "usage" VALUES('EPSG','23614','helmert_transformation','EPSG','10835','EPSG','3251','EPSG','1026');
INSERT INTO "helmert_transformation" VALUES('EPSG','10840','LKS-2020 to WGS 84 (1)','LKS-2020 is the second national realization of ETRS89 and coincident to WGS 84 within 1 metre. This transformation has an accuracy equal to the coincidence figure.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','10305','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'Vzd-Lva',0);
INSERT INTO "usage" VALUES('EPSG','22702','helmert_transformation','EPSG','10840','EPSG','1139','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','10848','ITRF2014 to LibRef21 (1)','Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','7789','EPSG','10798',0.01,-47.48,11.76,-0.58,'EPSG','1025',0.0,0.0,0.0,'EPSG','1031',0.0,'EPSG','1028',0.0,0.0,0.0,'EPSG','1027',-0.099,0.614,-0.733,'EPSG','1032',0.0,'EPSG','1030',2021.112,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'LLA-Lbr 2021',0);
INSERT INTO "usage" VALUES('EPSG','22773','helmert_transformation','EPSG','10848','EPSG','1142','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','10853','ITRF2020 to PNG94 (1)','Transformation derived by inversion of seven site velocities within the transformation coverage area. Transformation accuracy at 2σ is 0.15 m between 1994.0 and 2035.0 for locations within Papuan Foreland. Transformation precision degrades in plate margin deforming zone (PFTB) near plate boundary with NGH and Woodlark plates. Transformation not valid outside stated extent.','EPSG','1056','Time-dependent Coordinate Frame rotation (geocen)','EPSG','9988','EPSG','5544',0.15,0.0,0.0,0.0,'EPSG','1025',0.0,0.0,0.0,'EPSG','1031',0.0,'EPSG','1028',0.0,0.0,0.0,'EPSG','1027',1.3242,1.2788,1.1675,'EPSG','1032',0.0,'EPSG','1030',1994.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'OSG-Png',0);
INSERT INTO "usage" VALUES('EPSG','22777','helmert_transformation','EPSG','10853','EPSG','4800','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','10888','ETRF2014 to ETRF2000 (1)','Derived from concatenation of ETRF2014 to ITRF2014 (CT code 8880, with direction reversed) reduced to epoch 2000.00 and ITRF2014 to ETRF2000 (CT code 8405) reduced to epoch 2000.00. Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','8401','EPSG','7930',0.01,53.7,51.2,-55.1,'EPSG','1025',-0.044,-0.451,-0.242,'EPSG','1031',1.02,'EPSG','1028',0.1,0.1,-1.9,'EPSG','1027',-0.004,-0.041,-0.022,'EPSG','1032',0.11,'EPSG','1030',2000.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IOGP-Eur 2000',0);
INSERT INTO "usage" VALUES('EPSG','24547','helmert_transformation','EPSG','10888','EPSG','1298','EPSG','1296');
INSERT INTO "helmert_transformation" VALUES('EPSG','10889','ETRF97 to ETRF2000 (1)','Derived from concatenation of ETRF97 to ITRF97 (CT code 7939, with direction reversed) reduced to epoch 2000.00 and ITRF97 to ETRF2000 (CT code 7949) reduced to epoch 2000.00. Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','7928','EPSG','7930',0.01,6.3,5.7,23.7,'EPSG','1025',-1.309,-0.11,-16.622,'EPSG','1031',-1.58,'EPSG','1028',0.0,0.6,1.4,'EPSG','1027',-0.119,-0.01,-0.162,'EPSG','1032',-0.01,'EPSG','1030',2000.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IOGP-Eur 2000',0);
INSERT INTO "usage" VALUES('EPSG','24548','helmert_transformation','EPSG','10889','EPSG','1298','EPSG','1296');
INSERT INTO "helmert_transformation" VALUES('EPSG','10905','ETRS89/DREF91/2016 to Asse 2025 + Asse 2025 height (1)','3-dimensional transformation defining the horizontal and vertical CRSs for the Asse mining area.','EPSG','1149','Coordinate Frame rotation (geog3D to compound)','EPSG','10283','EPSG','10904',0.04,-646.6552,-165.0859,-437.6858,'EPSG','9001',4.77773,-0.39139,-1.07485,'EPSG','9104',2.0025,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'BGE-Deu Asse',0);
INSERT INTO "usage" VALUES('EPSG','23490','helmert_transformation','EPSG','10905','EPSG','4805','EPSG','1144');
INSERT INTO "helmert_transformation" VALUES('EPSG','10907','ETRF96 to ETRF2000 (1)','Derived from concatenation of ETRF96 to ITRF96 (CT code 7938, with direction reversed) reduced to epoch 2000.00 and ITRF96 to ETRF2000 (CT code 7948) reduced to epoch 2000.00. Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','7926','EPSG','7930',0.01,6.3,5.7,23.7,'EPSG','1025',-1.309,-0.11,-1.622,'EPSG','1031',-1.58,'EPSG','1028',0.0,0.6,1.4,'EPSG','1027',-0.119,-0.01,-0.162,'EPSG','1032',-0.01,'EPSG','1030',2000.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'EUREF-Eur 2000',0);
INSERT INTO "usage" VALUES('EPSG','24502','helmert_transformation','EPSG','10907','EPSG','1298','EPSG','1296');
INSERT INTO "helmert_transformation" VALUES('EPSG','10930','CSRN2025.0 (NAD83 2011) to WGS 84 (1)','For many purposes CSRN2025.0 (NAD83 2011) can be considered to be coincident with WGS 84. This approximation ignores the ~2.2m offset of CSRN2025.0 (NAD83 2011) from the geocentre, the low accuracy of the WGS 84 ensemble and the application of tectonic plate motion to WGS 84 data.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','10910','EPSG','4326',2.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Usa CA',0);
INSERT INTO "usage" VALUES('EPSG','23227','helmert_transformation','EPSG','10930','EPSG','1375','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','10935','ETRF93 to ETRF2000 (1)','Derived from concatenation of ETRF93 to ITRF93 (CT code 7936, with direction reversed) reduced to epoch 2000.00 and ITRF93 to ETRF2000 (CT code 7946) reduced to epoch 2000.00. Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','7922','EPSG','7930',0.01,57.1,-6.1,1.1,'EPSG','1025',-0.919,-1.71,-1.042,'EPSG','1031',-2.07,'EPSG','1028',2.9,0.2,0.6,'EPSG','1027',-0.129,-0.1,-0.192,'EPSG','1032',-0.01,'EPSG','1030',2000.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'EUREF-Eur 2000',0);
INSERT INTO "usage" VALUES('EPSG','24549','helmert_transformation','EPSG','10935','EPSG','1298','EPSG','1296');
INSERT INTO "helmert_transformation" VALUES('EPSG','10953','CSRN2025 (ITRF2020) to CSRN2025 (NAD83 2011) (1)','Parameter values derived from time-dependent transformation ITRF2020 to NAD83(2011) (CT code 10334) reduced to the CSRN2025 (ITRF2020) and CSRN2025 (NAD83 2011) common anchor epoch 2025.00. Defines CSRN2025 (NAD83 2011).','EPSG','1032','Coordinate Frame rotation (geocentric domain)','EPSG','10950','EPSG','10908',0.0,1015.75,-1920.11,-559.77,'EPSG','1025',27.78143,-11.78187,10.16211,'EPSG','1031',-1.13124,'EPSG','1028',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'SOPAC-Usa CA',0);
INSERT INTO "usage" VALUES('EPSG','23567','helmert_transformation','EPSG','10953','EPSG','1375','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','10960','ITRF2020 to GR96(2021) (1)','Derived from the IGS20 coordinates at epoch 2021.6164 of 7 stations including the four passive stations used to define GR96(1996). Validated at 26 other stations.','EPSG','1056','Time-dependent Coordinate Frame rotation (geocen)','EPSG','9988','EPSG','10957',0.1,-0.30031,-1.17512,-0.30654,'EPSG','9001',0.041614,-0.026303,-0.011214,'EPSG','9104',-0.01626,'EPSG','9202',0.0,0.0,0.0,'EPSG','1042',4.5e-05,-0.000666,-9.8e-05,'EPSG','1043',0.0,'EPSG','1041',2021.6164,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'KDS-Grl',0);
INSERT INTO "usage" VALUES('EPSG','23427','helmert_transformation','EPSG','10960','EPSG','1107','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','10961','Ammassalik 1958 to GR96 (2)','Determined through the GR96(1996) coordinates of 34 points, standard deviation 0.2m. Replaces Ammassalik 1958 to GR96 (1) (code 15900).','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4196','EPSG','4747',0.4,308.9415,136.202,986.3661,'EPSG','9001',-3.8742,3.77827,-7.61345,'EPSG','9104',-171.67315,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'KDS-Grl 2021',0);
INSERT INTO "usage" VALUES('EPSG','23617','helmert_transformation','EPSG','10961','EPSG','2571','EPSG','1026');
INSERT INTO "helmert_transformation" VALUES('EPSG','10962','Qoornoq 1927 to GR96 (2)','Determined through the GR96(1996) coordinates of 3881 points, standard deviation 4.0m. Replaces Qoornoq 1927 to GR96 (1) (code 15957).','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4194','EPSG','4747',8.0,197.8579,146.5947,-108.8501,'EPSG','9001',-0.85735,-0.028492,0.38626,'EPSG','9104',-8.356137,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'KDS-Grl 2021',0);
INSERT INTO "usage" VALUES('EPSG','23618','helmert_transformation','EPSG','10962','EPSG','3362','EPSG','1026');
INSERT INTO "helmert_transformation" VALUES('EPSG','10963','Scoresbysund 1952 to GR96 (2)','Determined through the GR96(1996) coordinates of 628 points, standard deviation 5.1m. Replaces Scoresbysund 1952 to GR96 (1) (code 15899).','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4195','EPSG','4747',10.0,218.233,270.6151,253.1391,'EPSG','9001',0.26337,-0.15733,-1.19862,'EPSG','9104',-59.923872,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'KDS-Grl 2021',0);
INSERT INTO "usage" VALUES('EPSG','23619','helmert_transformation','EPSG','10963','EPSG','2570','EPSG','1026');
INSERT INTO "helmert_transformation" VALUES('EPSG','10965','QazTRF-23 to WGS 84 (1)','Approximation at the +/- 1m level assuming that QazTRF-23 is equivalent to WGS 84 within the accuracy of the transformation.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','10941','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Kaz',0);
INSERT INTO "usage" VALUES('EPSG','23452','helmert_transformation','EPSG','10965','EPSG','1131','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','10969','NATRF2022 to WGS 84 (1)','For many purposes NATRF2022 can be considered to be coincident with WGS 84. This approximation ignores the low accuracy of the WGS 84 ensemble and the application of tectonic plate motion to WGS 84 data.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','10968','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-N Am',0);
INSERT INTO "usage" VALUES('EPSG','23509','helmert_transformation','EPSG','10969','EPSG','4803','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','10988','ETRF89 to ETRF2000 (1)','Derived from concatenation of ETRF89 to ITRF89 (CT code 7932, with direction reversed) reduced to epoch 2000.00 and ITRF89 to ETRF2000 (CT code 7942) reduced to epoch 2000.00. Scale difference in ppb and scale difference rate in ppb/yr where 1/billion = 1E-9 or nm/m.','EPSG','1053','Time-dependent Position Vector tfm (geocentric)','EPSG','7914','EPSG','7930',0.01,24.3,10.7,42.7,'EPSG','1025',-0.319,-0.88,-0.962,'EPSG','1031',-5.97,'EPSG','1028',0.0,0.6,1.4,'EPSG','1027',-0.029,-0.08,-0.102,'EPSG','1032',-0.01,'EPSG','1030',2000.0,'EPSG','1029',NULL,NULL,NULL,NULL,NULL,'IOGP-Eur 2000',0);
INSERT INTO "usage" VALUES('EPSG','24505','helmert_transformation','EPSG','10988','EPSG','1298','EPSG','1296');
INSERT INTO "helmert_transformation" VALUES('EPSG','11028','ETRS89-NOR [EUREF89] to WGS 84 (1)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','10875','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Nor',0);
INSERT INTO "usage" VALUES('EPSG','24398','helmert_transformation','EPSG','11028','EPSG','1182','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','11044','CR-SIRGAS epoch 2019.24 to CR-SIRGAS epoch 2014.59 (1)','Derived at 45 stations. Note that rotation convention (position vector) differs from that used in the transformation from CR05 to CR-SIRGAS epoch 2014.59 (code 9751).','EPSG','1061','Molodensky-Badekas (PV geocentric domain)','EPSG','11041','EPSG','8905',0.04,-0.0533,0.0136,-0.0707,'EPSG','9001',0.0122,-0.0284,-0.0037,'EPSG','9104',0.0209,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,629134.9009,-6249130.4704,1101655.6817,'EPSG','9001','IGN-Cri 2020',0);
INSERT INTO "usage" VALUES('EPSG','24186','helmert_transformation','EPSG','11044','EPSG','1074','EPSG','1035');
INSERT INTO "helmert_transformation" VALUES('EPSG','15483','Tokyo to JGD2000 (1)','Derived at Tokyo datum origin. Accuracy on main islands 9m. Also used on remote islands with significantly less accuracy: Io-To 793m, Kitadaito and Minamidaito Jima 642m, Tarama and Minna Shima 560m, Ishigaki and Taketomi Jima 251m, Yonaguni Jima 248m.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4301','EPSG','4612',9.0,-146.414,507.337,680.507,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'GSI-Jpn',0);
INSERT INTO "usage" VALUES('EPSG','11494','helmert_transformation','EPSG','15483','EPSG','3957','EPSG','1142');
INSERT INTO "helmert_transformation" VALUES('EPSG','15484','Tokyo to WGS 84 (108)','Parameter values from Tokyo to JGD2000 (1) (code 15483). Assumes JGD2000 and WGS 84 can be considered the same to within the accuracy of the transformation.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4301','EPSG','4326',9.0,-146.414,507.337,680.507,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-Jpn',0);
INSERT INTO "usage" VALUES('EPSG','11495','helmert_transformation','EPSG','15484','EPSG','3957','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','15485','SAD69 to SIRGAS 2000 (1)','Accuracy generally better than 1m except in Amazon basin where it degenerates to 5m. May be used as CT between SAD69(96) and SIRGAS 2000 and between SAD69 and WGS 84 - see tfm codes 5881 and 5882. Used by Petrobras and ANP throughout Brazil from 1994.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4618','EPSG','4674',5.0,-67.35,3.88,-38.22,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IBGE-Bra 5m 1994',0);
INSERT INTO "usage" VALUES('EPSG','11496','helmert_transformation','EPSG','15485','EPSG','1053','EPSG','1257');
INSERT INTO "helmert_transformation" VALUES('EPSG','15493','Minna to WGS 84 (15)','Adopted by MPN for all joint venture operations from 1/1/1996.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4263','EPSG','4326',5.0,-94.031,-83.317,116.708,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'MPN-Nga',0);
INSERT INTO "usage" VALUES('EPSG','11504','helmert_transformation','EPSG','15493','EPSG','3590','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','15494','Kalianpur 1962 to WGS 84 (6)','Derived by Fugro-Geodetic in 2004 at 6 closely-spaced stations. Used by OMV in all blocks in Pakistan where operator.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4145','EPSG','4326',3.0,274.164,677.282,226.704,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'omv-Pak Gambat',0);
INSERT INTO "usage" VALUES('EPSG','11505','helmert_transformation','EPSG','15494','EPSG','3589','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','15495','Accra to WGS 84 (3)','Derived via WGS 72BE. Found in use within oil industry erroneously concatenated via WGS 72. See tfm code 8571.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4168','EPSG','4326',25.0,-171.16,17.29,325.21,'EPSG','9001',0.0,0.0,0.814,'EPSG','9104',-0.38,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Gha 1',0);
INSERT INTO "usage" VALUES('EPSG','11506','helmert_transformation','EPSG','15495','EPSG','1505','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','15496','Pulkovo 1942(58) to WGS 84 (18)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4179','EPSG','4326',10.0,44.107,-116.147,-54.648,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'Shell-Rom',0);
INSERT INTO "usage" VALUES('EPSG','11507','helmert_transformation','EPSG','15496','EPSG','1197','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','15497','Pulkovo 1942(58) to WGS 84 (9)','Derived at 4 stations.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4179','EPSG','4326',7.0,28.0,-121.0,-77.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NIMA-Rom',0);
INSERT INTO "usage" VALUES('EPSG','11508','helmert_transformation','EPSG','15497','EPSG','1197','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','15698','ITRF2000 to ITRF2005 (1)','At epoch 2000.0. Rates dX=0.0002 m/yr, dy=-0.0001 m/yr, dZ=0.0018 m/yr, rX=rY=rZ=0.0"/yr, dS=-0.00008 ppm/yr.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4919','EPSG','4896',0.0,-0.0001,0.0008,0.0058,'EPSG','9001',0.0,0.0,0.0,'EPSG','9104',-0.0004,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IERS',1);
INSERT INTO "usage" VALUES('EPSG','11709','helmert_transformation','EPSG','15698','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','15699','NAD27 to WGS 84 (87)','Developed by John E Chance and Associates at 19°44''N, 92°21''W. Geoid height used =-13.34m.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4267','EPSG','4326',5.0,-2.0,124.7,196.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'JECA-Mex GoM CamS',0);
INSERT INTO "usage" VALUES('EPSG','11710','helmert_transformation','EPSG','15699','EPSG','3462','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','15700','Gulshan 303 to WGS 84 (1)','Derived at origin station in Dhaka.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4682','EPSG','4326',1.0,283.8,735.9,261.1,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'SB-BGD',1);
INSERT INTO "usage" VALUES('EPSG','11711','helmert_transformation','EPSG','15700','EPSG','1041','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','15701','Kalianpur 1962 to WGS 84 (2)','Derived at Geodetic Survey office in Karachi in 1997.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4145','EPSG','4326',1.0,275.57,676.78,229.6,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'TFE-Pak Indus',0);
INSERT INTO "usage" VALUES('EPSG','11712','helmert_transformation','EPSG','15701','EPSG','2985','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','15702','Kalianpur 1962 to WGS 84 (3)','Derived at station S0001, an approximate offset to Survey of India primary station Kat Baman, in 1992 from 180 single point Transit passes observed in 1991 by Fugro-Geodetic for UTP.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4145','EPSG','4326',3.0,278.9,684.39,226.05,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'utp-Pak Badin',0);
INSERT INTO "usage" VALUES('EPSG','11713','helmert_transformation','EPSG','15702','EPSG','2984','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','15703','Kalianpur 1962 to WGS 84 (4)','Derived at Chitrawala triangulation station by Fugro-Geodetic for UTP.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4145','EPSG','4326',3.0,271.905,669.593,231.495,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'utp-Pak Karachi',0);
INSERT INTO "usage" VALUES('EPSG','11714','helmert_transformation','EPSG','15703','EPSG','2982','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','15704','Kalianpur 1962 to WGS 84 (5)','Derived by Western Geophysical for UTP 1996 East Sind 2D survey.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4145','EPSG','4326',3.0,230.25,632.76,161.03,'EPSG','9001',-1.114,1.115,1.212,'EPSG','9104',12.584,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'utp-Pak E Sind',1);
INSERT INTO "usage" VALUES('EPSG','11715','helmert_transformation','EPSG','15704','EPSG','2983','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','15705','Minna to WGS 84 (12)','Derived via WGS 72(BE). Minna to WGS 72(BE) transformation derived in 1981 for Mobil E&P Nigeria (MEPCON) by Geodetic Survey through Transit translocation at six stations in southern Nigeria. Used by MEPCON in blocks OPL 215 and 221.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4263','EPSG','4326',8.0,-83.13,-104.95,114.63,'EPSG','9001',0.0,0.0,0.554,'EPSG','9104',0.0,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'WGC-Nga 211',0);
INSERT INTO "usage" VALUES('EPSG','11716','helmert_transformation','EPSG','15705','EPSG','3819','EPSG','1216');
INSERT INTO "helmert_transformation" VALUES('EPSG','15706','Minna to WGS 84 (13)','Used by Elf in Blocks OPL 222 and OPL 223 and by Mobil in 1994.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4263','EPSG','4326',7.0,-93.6,-83.7,113.8,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'Elf-Nga',0);
INSERT INTO "usage" VALUES('EPSG','11717','helmert_transformation','EPSG','15706','EPSG','1717','EPSG','1216');
INSERT INTO "helmert_transformation" VALUES('EPSG','15707','ELD79 to WGS 84 (6)','Used by Petrocanada and previous licence holders in Amal field, concession 12.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4159','EPSG','4326',10.0,-118.996,-111.177,-198.687,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'PCan-Lby Amal',0);
INSERT INTO "usage" VALUES('EPSG','11718','helmert_transformation','EPSG','15707','EPSG','2987','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','15708','PRS92 to WGS 84 (1)','Derived during GPS campaign which established PRS92 coordinates at 330 first order stations.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4683','EPSG','4326',0.05,-127.62,-67.24,-47.04,'EPSG','9001',3.068,-4.903,-1.578,'EPSG','9104',-1.06,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'CGS-Phl',0);
INSERT INTO "usage" VALUES('EPSG','11719','helmert_transformation','EPSG','15708','EPSG','1190','EPSG','1153');
INSERT INTO "helmert_transformation" VALUES('EPSG','15709','Nouakchott 1965 to WGS 84 (1)','Derived by IGN in 1992 at 7 stations within Nouakchott city.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4680','EPSG','4326',5.0,124.5,-63.5,-281.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Mau',0);
INSERT INTO "usage" VALUES('EPSG','11720','helmert_transformation','EPSG','15709','EPSG','2972','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','15710','Aratu to WGS 84 (10)','Replaced by Aratu to WGS 84 (14) (tfm code 5053) which Petrobras now recommends for the area.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4208','EPSG','4326',5.0,-160.0,315.0,-142.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'PB-Bra Campos',0);
INSERT INTO "usage" VALUES('EPSG','11721','helmert_transformation','EPSG','15710','EPSG','2963','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','15711','Aratu to WGS 84 (11)','Replaced by Aratu to WGS 84 (13) (tfm code 5051) which Petrobras now recommends for the area.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4208','EPSG','4326',5.0,-158.0,309.0,-147.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'PB-Bra Santos',0);
INSERT INTO "usage" VALUES('EPSG','11722','helmert_transformation','EPSG','15711','EPSG','2962','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','15712','Aratu to WGS 84 (12)','Replaced by Aratu to WGS 84 (15) (tfm code 5055) which Petrobras now recommends for the area.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4208','EPSG','4326',5.0,-161.0,310.0,-145.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'PB-Bra EspS',0);
INSERT INTO "usage" VALUES('EPSG','11723','helmert_transformation','EPSG','15712','EPSG','2964','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','15713','Gan 1970 to WGS 84 (1)','Derived at 1 station. Accuracy 25m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4684','EPSG','4326',44.0,-133.0,-321.0,50.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NIMA-Mdv',0);
INSERT INTO "usage" VALUES('EPSG','11724','helmert_transformation','EPSG','15713','EPSG','3274','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','15714','Bogota 1975 to MAGNA-SIRGAS (1)','May be taken as transformation to WGS 84 - see tfm code 15715. See Bogota 1975 to MAGNA-SIRGAS (9), tfm code 15730, for an equivalent transformation using the Molodenski-Badekas 10-parameter method. OGP recommends this alternative.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4218','EPSG','4686',1.0,-806.413,-263.5,-622.671,'EPSG','9001',6.018583e-05,-1.450001e-05,-0.0001892455,'EPSG','9101',-20.81616,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGAC-Col CF reg 1',0);
INSERT INTO "usage" VALUES('EPSG','11725','helmert_transformation','EPSG','15714','EPSG','3082','EPSG','1256');
INSERT INTO "helmert_transformation" VALUES('EPSG','15715','Bogota 1975 to WGS 84 (3)','Parameter values taken from Bogota 1975 to MAGNA-SIRGAS (1) (tfm code 15714). Approximation assumes that MAGNA-SIRGAS is equivalent to WGS 84. Accuracy about 1 part in 10^5 of distance between points, depending on relative tectonic motion.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4218','EPSG','4326',1.0,-806.413,-263.5,-622.671,'EPSG','9001',6.018583e-05,-1.450001e-05,-0.0001892455,'EPSG','9101',-20.81616,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Col reg 1',0);
INSERT INTO "usage" VALUES('EPSG','11726','helmert_transformation','EPSG','15715','EPSG','3082','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','15716','Bogota 1975 to MAGNA-SIRGAS (2)','May be taken as transformation to WGS 84 - see tfm code 15717. See Bogota 1975 to MAGNA-SIRGAS (10), tfm code 15731, for an equivalent transformation using the Molodenski-Badekas 10-parameter method. OGP recommends this alternative.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4218','EPSG','4686',1.0,100.783,187.382,-47.0,'EPSG','9001',-4.471839e-05,1.175093e-05,-4.027967e-05,'EPSG','9101',-13.56561,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGAC-Col CF reg 2',0);
INSERT INTO "usage" VALUES('EPSG','11727','helmert_transformation','EPSG','15716','EPSG','3083','EPSG','1256');
INSERT INTO "helmert_transformation" VALUES('EPSG','15717','Bogota 1975 to WGS 84 (4)','Parameter values taken from Bogota 1975 to MAGNA-SIRGAS (2) (tfm code 15716). Approximation assumes that MAGNA-SIRGAS is equivalent to WGS 84. Accuracy about 1 part in 10^5 of distance between points, depending on relative tectonic motion.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4218','EPSG','4326',1.0,100.783,187.382,-47.0,'EPSG','9001',-4.471839e-05,1.175093e-05,-4.027967e-05,'EPSG','9101',-13.56561,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Col reg 2',0);
INSERT INTO "usage" VALUES('EPSG','11728','helmert_transformation','EPSG','15717','EPSG','3083','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','15718','Bogota 1975 to MAGNA-SIRGAS (3)','May be taken as transformation to WGS 84 - see tfm code 15719. See Bogota 1975 to MAGNA-SIRGAS (10), tfm code 15732, for an equivalent transformation using the Molodenski-Badekas 10-parameter method. OGP recommends this alternative.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4218','EPSG','4686',1.0,336.026,348.565,252.978,'EPSG','9001',-8.358813e-05,-3.057474e-05,7.573031e-06,'EPSG','9101',-5.771909,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGAC-Col CF reg 3',0);
INSERT INTO "usage" VALUES('EPSG','11729','helmert_transformation','EPSG','15718','EPSG','3084','EPSG','1256');
INSERT INTO "helmert_transformation" VALUES('EPSG','15719','Bogota 1975 to WGS 84 (5)','Parameter values taken from Bogota 1975 to MAGNA-SIRGAS (3) (tfm code 15718). Approximation assumes that MAGNA-SIRGAS is equivalent to WGS 84. Accuracy about 1 part in 10^5 of distance between points, depending on relative tectonic motion.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4218','EPSG','4326',1.0,336.026,348.565,252.978,'EPSG','9001',-8.358813e-05,-3.057474e-05,7.573031e-06,'EPSG','9101',-5.771909,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Col reg 3',0);
INSERT INTO "usage" VALUES('EPSG','11730','helmert_transformation','EPSG','15719','EPSG','3084','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','15720','Bogota 1975 to MAGNA-SIRGAS (4)','May be taken as transformation to WGS 84 - see tfm code 15721. See Bogota 1975 to MAGNA-SIRGAS (10), tfm code 15733, for an equivalent transformation using the Molodenski-Badekas 10-parameter method. OGP recommends this alternative.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4218','EPSG','4686',1.0,963.273,486.386,190.997,'EPSG','9001',-7.992171e-05,-8.090696e-06,0.0001051699,'EPSG','9101',-13.89914,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGAC-Col CF reg 4',0);
INSERT INTO "usage" VALUES('EPSG','11731','helmert_transformation','EPSG','15720','EPSG','3085','EPSG','1256');
INSERT INTO "helmert_transformation" VALUES('EPSG','15721','Bogota 1975 to WGS 84 (6)','Parameter values taken from Bogota 1975 to MAGNA-SIRGAS (4) (tfm code 15720). Approximation assumes that MAGNA-SIRGAS is equivalent to WGS 84. Accuracy about 1 part in 10^5 of distance between points, depending on relative tectonic motion.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4218','EPSG','4326',1.0,963.273,486.386,190.997,'EPSG','9001',-7.992171e-05,-8.090696e-06,0.0001051699,'EPSG','9101',-13.89914,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Col reg 4',0);
INSERT INTO "usage" VALUES('EPSG','11732','helmert_transformation','EPSG','15721','EPSG','3085','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','15722','Bogota 1975 to MAGNA-SIRGAS (5)','May be taken as transformation to WGS 84 - see tfm code 15723. See Bogota 1975 to MAGNA-SIRGAS (10), tfm code 15734, for an equivalent transformation using the Molodenski-Badekas 10-parameter method. OGP recommends this alternative.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4218','EPSG','4686',1.0,-90.29,247.559,-21.989,'EPSG','9001',-4.216369e-05,-2.030416e-05,-6.209623e-05,'EPSG','9101',2.181658,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGAC-Col CF reg 5',0);
INSERT INTO "usage" VALUES('EPSG','11733','helmert_transformation','EPSG','15722','EPSG','3086','EPSG','1256');
INSERT INTO "helmert_transformation" VALUES('EPSG','15723','Bogota 1975 to WGS 84 (7)','Parameter values taken from Bogota 1975 to MAGNA-SIRGAS (5) (tfm code 15722). Approximation assumes that MAGNA-SIRGAS is equivalent to WGS 84. Accuracy about 1 part in 10^5 of distance between points, depending on relative tectonic motion.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4218','EPSG','4326',1.0,-90.29,247.559,-21.989,'EPSG','9001',-4.216369e-05,-2.030416e-05,-6.209623e-05,'EPSG','9101',2.181658,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Col reg 5',0);
INSERT INTO "usage" VALUES('EPSG','11734','helmert_transformation','EPSG','15723','EPSG','3086','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','15724','Bogota 1975 to MAGNA-SIRGAS (6)','May be taken as transformation to WGS 84 - see tfm code 15725. See Bogota 1975 to MAGNA-SIRGAS (10), tfm code 15735, for an equivalent transformation using the Molodenski-Badekas 10-parameter method. OGP recommends this alternative.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4218','EPSG','4686',1.0,-0.562,244.299,-456.938,'EPSG','9001',3.329153e-05,-4.001009e-05,-4.507206e-05,'EPSG','9101',3.74656,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGAC-Col CF reg 6',0);
INSERT INTO "usage" VALUES('EPSG','11735','helmert_transformation','EPSG','15724','EPSG','3087','EPSG','1256');
INSERT INTO "helmert_transformation" VALUES('EPSG','15725','Bogota 1975 to WGS 84 (8)','Parameter values taken from Bogota 1975 to MAGNA-SIRGAS (6) (tfm code 15724). Approximation assumes that MAGNA-SIRGAS is equivalent to WGS 84. Accuracy about 1 part in 10^5 of distance between points, depending on relative tectonic motion.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4218','EPSG','4326',1.0,-0.562,244.299,-456.938,'EPSG','9001',3.329153e-05,-4.001009e-05,-4.507206e-05,'EPSG','9101',3.74656,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Col reg 6',0);
INSERT INTO "usage" VALUES('EPSG','11736','helmert_transformation','EPSG','15725','EPSG','3087','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','15726','Bogota 1975 to MAGNA-SIRGAS (7)','May be taken as transformation to WGS 84 - see tfm code 15727. See Bogota 1975 to MAGNA-SIRGAS (10), tfm code 15736, for an equivalent transformation using the Molodenski-Badekas 10-parameter method. OGP recommends this alternative.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4218','EPSG','4686',1.0,-305.356,222.004,-30.023,'EPSG','9001',-4.698084e-05,5.003123e-06,-9.578655e-05,'EPSG','9101',6.325747,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGAC-Col CF reg 7',0);
INSERT INTO "usage" VALUES('EPSG','11737','helmert_transformation','EPSG','15726','EPSG','3088','EPSG','1256');
INSERT INTO "helmert_transformation" VALUES('EPSG','15727','Bogota 1975 to WGS 84 (9)','Parameter values taken from Bogota 1975 to MAGNA-SIRGAS (7) (tfm code 15726). Approximation assumes that MAGNA-SIRGAS is equivalent to WGS 84. Accuracy about 1 part in 10^5 of distance between points, depending on relative tectonic motion.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4218','EPSG','4326',1.0,-305.356,222.004,-30.023,'EPSG','9001',-4.698084e-05,5.003123e-06,-9.578655e-05,'EPSG','9101',6.325747,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Col reg 7',0);
INSERT INTO "usage" VALUES('EPSG','11738','helmert_transformation','EPSG','15727','EPSG','3088','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','15728','Bogota 1975 to MAGNA-SIRGAS (8)','May be taken as transformation to WGS 84 - see tfm code 15729. See Bogota 1975 to MAGNA-SIRGAS (10), tfm code 15737, for an equivalent transformation using the Molodenski-Badekas 10-parameter method. OGP recommends this alternative.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4218','EPSG','4686',1.0,221.899,274.136,-397.554,'EPSG','9001',1.361573e-05,-2.174431e-06,-1.36241e-05,'EPSG','9101',-2.199943,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGAC-Col CF reg 8',0);
INSERT INTO "usage" VALUES('EPSG','11739','helmert_transformation','EPSG','15728','EPSG','3089','EPSG','1256');
INSERT INTO "helmert_transformation" VALUES('EPSG','15729','Bogota 1975 to WGS 84 (10)','Parameter values taken from Bogota 1975 to MAGNA-SIRGAS (8) (tfm code 15728). Approximation assumes that MAGNA-SIRGAS is equivalent to WGS 84. Accuracy about 1 part in 10^5 of distance between points, depending on relative tectonic motion.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4218','EPSG','4326',1.0,221.899,274.136,-397.554,'EPSG','9001',1.361573e-05,-2.174431e-06,-1.36241e-05,'EPSG','9101',-2.199943,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Col reg 8',0);
INSERT INTO "usage" VALUES('EPSG','11740','helmert_transformation','EPSG','15729','EPSG','3089','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','15730','Bogota 1975 to MAGNA-SIRGAS (9)','Source also quotes an equivalent transformation using the Coordinate Frame 7-parameter method - see Bogota 1975 to MAGNA-SIRGAS (1), tfm code 15714.','EPSG','9636','Molodensky-Badekas (CF geog2D domain)','EPSG','4218','EPSG','4686',1.0,300.449,293.757,-317.306,'EPSG','9001',6.018581e-05,-1.450002e-05,-0.0001892455,'EPSG','9101',-20.81615,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1891881.173,-5961263.267,1248403.057,'EPSG','9001','IGAC-Col MB reg 1',0);
INSERT INTO "usage" VALUES('EPSG','11741','helmert_transformation','EPSG','15730','EPSG','3082','EPSG','1256');
INSERT INTO "helmert_transformation" VALUES('EPSG','15731','Bogota 1975 to MAGNA-SIRGAS (10)','Source also quotes an equivalent transformation using the Coordinate Frame 7-parameter method - see Bogota 1975 to MAGNA-SIRGAS (2), tfm code 15716.','EPSG','9636','Molodensky-Badekas (CF geog2D domain)','EPSG','4218','EPSG','4686',1.0,308.833,282.519,-314.571,'EPSG','9001',-4.471845e-05,1.175087e-05,-4.027981e-05,'EPSG','9101',-13.56561,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1625036.59,-6054644.061,1172969.151,'EPSG','9001','IGAC-Col MB reg 2',0);
INSERT INTO "usage" VALUES('EPSG','11742','helmert_transformation','EPSG','15731','EPSG','3083','EPSG','1256');
INSERT INTO "helmert_transformation" VALUES('EPSG','15732','Bogota 1975 to MAGNA-SIRGAS (11)','Source also quotes an equivalent transformation using the Coordinate Frame 7-parameter method - see Bogota 1975 to MAGNA-SIRGAS (3), tfm code 15718.','EPSG','9636','Molodensky-Badekas (CF geog2D domain)','EPSG','4218','EPSG','4686',1.0,311.118,289.167,-310.641,'EPSG','9001',-8.358815e-05,-3.057474e-05,7.573043e-06,'EPSG','9101',-5.771882,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1555622.801,-6105353.313,991255.656,'EPSG','9001','IGAC-Col MB reg 3',0);
INSERT INTO "usage" VALUES('EPSG','11743','helmert_transformation','EPSG','15732','EPSG','3084','EPSG','1256');
INSERT INTO "helmert_transformation" VALUES('EPSG','15733','Bogota 1975 to MAGNA-SIRGAS (12)','Source also quotes an equivalent transformation using the Coordinate Frame 7-parameter method - see Bogota 1975 to MAGNA-SIRGAS (4), tfm code 15720.','EPSG','9636','Molodensky-Badekas (CF geog2D domain)','EPSG','4218','EPSG','4686',1.0,306.666,315.063,-318.837,'EPSG','9001',-7.992173e-05,-8.090698e-06,0.0001051699,'EPSG','9101',-13.89912,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1845222.398,-6058604.495,769132.398,'EPSG','9001','IGAC-Col MB reg 4',0);
INSERT INTO "usage" VALUES('EPSG','11744','helmert_transformation','EPSG','15733','EPSG','3085','EPSG','1256');
INSERT INTO "helmert_transformation" VALUES('EPSG','15734','Bogota 1975 to MAGNA-SIRGAS (13)','Source also quotes an equivalent transformation using the Coordinate Frame 7-parameter method - see Bogota 1975 to MAGNA-SIRGAS (5), see tfm code 15722.','EPSG','9636','Molodensky-Badekas (CF geog2D domain)','EPSG','4218','EPSG','4686',1.0,307.871,305.803,-311.992,'EPSG','9001',-4.216368e-05,-2.030416e-05,-6.209624e-05,'EPSG','9101',2.181655,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1594396.206,-6143812.398,648855.829,'EPSG','9001','IGAC-Col MB reg 5',0);
INSERT INTO "usage" VALUES('EPSG','11745','helmert_transformation','EPSG','15734','EPSG','3086','EPSG','1256');
INSERT INTO "helmert_transformation" VALUES('EPSG','15735','Bogota 1975 to MAGNA-SIRGAS (14)','Source also quotes an equivalent transformation using the Coordinate Frame 7-parameter method - see Bogota 1975 to MAGNA-SIRGAS (6), tfm code 15724.','EPSG','9636','Molodensky-Badekas (CF geog2D domain)','EPSG','4218','EPSG','4686',1.0,302.934,307.805,-312.121,'EPSG','9001',3.329153e-05,-4.001009e-05,-4.507205e-05,'EPSG','9101',3.746562,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1558280.49,-6167355.092,491954.219,'EPSG','9001','IGAC-Col MB reg 6',0);
INSERT INTO "usage" VALUES('EPSG','11746','helmert_transformation','EPSG','15735','EPSG','3087','EPSG','1256');
INSERT INTO "helmert_transformation" VALUES('EPSG','15736','Bogota 1975 to MAGNA-SIRGAS (15)','Source also quotes an equivalent transformation using the Coordinate Frame 7-parameter method - see Bogota 1975 to MAGNA-SIRGAS (7), tfm code 15726.','EPSG','9636','Molodensky-Badekas (CF geog2D domain)','EPSG','4218','EPSG','4686',1.0,295.282,321.293,-311.001,'EPSG','9001',-4.698084e-05,5.003127e-06,-9.578653e-05,'EPSG','9101',6.325744,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1564000.62,-6180004.879,243257.955,'EPSG','9001','IGAC-Col MB reg 7',0);
INSERT INTO "usage" VALUES('EPSG','11747','helmert_transformation','EPSG','15736','EPSG','3088','EPSG','1256');
INSERT INTO "helmert_transformation" VALUES('EPSG','15737','Bogota 1975 to MAGNA-SIRGAS (16)','Source also quotes an equivalent transformation using the Coordinate Frame 7-parameter method - see Bogota 1975 to MAGNA-SIRGAS (8), tfm code 15728.','EPSG','9636','Molodensky-Badekas (CF geog2D domain)','EPSG','4218','EPSG','4686',1.0,302.529,317.979,-319.08,'EPSG','9001',1.361566e-05,-2.174456e-06,-1.362418e-05,'EPSG','9101',-2.199976,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1738580.767,-6120500.388,491473.306,'EPSG','9001','IGAC-Col MB reg 8',0);
INSERT INTO "usage" VALUES('EPSG','11748','helmert_transformation','EPSG','15737','EPSG','3089','EPSG','1256');
INSERT INTO "helmert_transformation" VALUES('EPSG','15738','MAGNA-SIRGAS to WGS 84 (1)','Approximation at the +/- 1m level assuming that MAGNA-SIRGAS is equivalent to WGS 84.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4686','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG',0);
INSERT INTO "usage" VALUES('EPSG','11749','helmert_transformation','EPSG','15738','EPSG','1070','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','15739','Amersfoort to ETRS89 (3)','Replaces Amersfoort to ETRS89 (1) (tfm code 1751). Replaced by Amersfoort to ETRS89 (5) (tfm code 4830). Dutch sources also quote an equivalent transformation using the Molodenski-Badekas 10-parameter method (M-B) - see tfm code 15740.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4289','EPSG','4258',0.5,565.2369,50.0087,465.658,'EPSG','9001',1.9725,-1.7004,9.0677,'EPSG','9109',4.0812,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NCG-Nld 2004 CF',0);
INSERT INTO "usage" VALUES('EPSG','11750','helmert_transformation','EPSG','15739','EPSG','1275','EPSG','1035');
INSERT INTO "helmert_transformation" VALUES('EPSG','15740','Amersfoort to ETRS89 (4)','Replaces Amersfoort to ETRS89 (2) (tfm code 1066). Replaced by Amersfoort to ETRS89 (6) (tfm code 4831). Dutch sources also quote an equivalent transformation using the Coordinate Frame 7-parameter method - see tfm code 15739.','EPSG','9636','Molodensky-Badekas (CF geog2D domain)','EPSG','4289','EPSG','4258',0.5,593.0297,26.0038,478.7534,'EPSG','9001',1.9725,-1.7004,9.0677,'EPSG','9109',4.0812,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,3903453.1482,368135.3134,5012970.3051,'EPSG','9001','NCG-Nld 2004 MB',0);
INSERT INTO "usage" VALUES('EPSG','11751','helmert_transformation','EPSG','15740','EPSG','1275','EPSG','1035');
INSERT INTO "helmert_transformation" VALUES('EPSG','15741','Deir ez Zor to WGS 84 (2)','Derived by Elf in 1991 from tfm code 1584 concatenated with a tfm from WGS72BE to WGS84.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4227','EPSG','4326',5.0,-187.5,14.1,237.6,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'Elf-Syr Deir 1991',0);
INSERT INTO "usage" VALUES('EPSG','11752','helmert_transformation','EPSG','15741','EPSG','2329','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','15742','Deir ez Zor to WGS 84 (5)','Derived for 1998 Omar seismic survey and used in 2000 for El Isba seismic survey.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4227','EPSG','4326',5.0,-190.421,8.532,238.69,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'CGG-Syr Isba',0);
INSERT INTO "usage" VALUES('EPSG','11753','helmert_transformation','EPSG','15742','EPSG','3314','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','15743','Deir ez Zor to WGS 84 (6)','Derived 2005 at 5 triangulation stations and using (EGM96 geoid model +1.15m). Used by Total/DEZPC for Jafra and Mazraa seismic surveys. Can be approximated using geocentric translations of dX=-190.6m, dY=+8.8m, dZ=+239.6m.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4227','EPSG','4326',0.5,-83.58,-397.54,458.78,'EPSG','9001',-17.595,-2.847,4.256,'EPSG','9104',3.225,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'Tot-Syr Deir 2005',0);
INSERT INTO "usage" VALUES('EPSG','11754','helmert_transformation','EPSG','15743','EPSG','2329','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','15745','ED50(ED77) to WGS 84 (6)','Derived in Tombak district in March 2005. Used for South Pars phase 11.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4154','EPSG','4326',0.2,-123.02,-158.95,-168.47,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'Tot-Irn Spars',0);
INSERT INTO "usage" VALUES('EPSG','11756','helmert_transformation','EPSG','15745','EPSG','3140','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','15746','Nakhl-e Ghanem to WGS 84 (6)','Derived in Tombak district in March 2005. Used for South Pars phase 11 and Pars LNG plants.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4693','EPSG','4326',0.2,0.0,-0.15,0.68,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'TFE-Irn Tombak',0);
INSERT INTO "usage" VALUES('EPSG','11757','helmert_transformation','EPSG','15746','EPSG','3141','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','15748','BD72 to ETRS89 (2)','May be taken as approximate transformation BD72 to WGS 84 - see code 15749. Scale difference is given by information source as 1.0000012747. Given in this record in ppm to assist application usage.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4313','EPSG','4258',0.2,-106.8686,52.2978,-103.7239,'EPSG','9001',-0.3366,0.457,-1.8422,'EPSG','9104',1.2747,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Bel 0.2m',1);
INSERT INTO "usage" VALUES('EPSG','11759','helmert_transformation','EPSG','15748','EPSG','1044','EPSG','1032');
INSERT INTO "helmert_transformation" VALUES('EPSG','15749','BD72 to WGS 84 (3)','Parameter values from BD72 to ETRS89 (2) (code 15748). Scale difference is given by information source as 1.0000012747. Given in this record in ppm to assist application usage.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4313','EPSG','4326',0.2,-106.8686,52.2978,-103.7239,'EPSG','9001',-0.3366,0.457,-1.8422,'EPSG','9104',1.2747,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Bel 0.2m',1);
INSERT INTO "usage" VALUES('EPSG','11760','helmert_transformation','EPSG','15749','EPSG','1044','EPSG','1035');
INSERT INTO "helmert_transformation" VALUES('EPSG','15750','St. Kitts 1955 to WGS 84 (2)','Derived at 2 stations. Accuracy 25m in each of X, Y and Z axes.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4605','EPSG','4326',44.0,-7.0,215.0,225.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NIMA-Kna',0);
INSERT INTO "usage" VALUES('EPSG','11761','helmert_transformation','EPSG','15750','EPSG','3297','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','15751','Reunion 1947 to WGS 84 (2)','Derived at 1 station. Accuracy 25m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4626','EPSG','4326',44.0,94.0,-948.0,-1262.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Reu 30m',0);
INSERT INTO "usage" VALUES('EPSG','11762','helmert_transformation','EPSG','15751','EPSG','3337','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','15752','ED79 to WGS 84 (1)','Derived at 22 stations. Accuracy 3m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4668','EPSG','4326',6.0,-86.0,-98.0,-119.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NIMA-Eur',0);
INSERT INTO "usage" VALUES('EPSG','11763','helmert_transformation','EPSG','15752','EPSG','1297','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','15754','Aratu to WGS 84 (1)','Mean for 3 basins. See Aratu to WGS 84 (10) through (12) (codes 15710-12) for transformations for individual basins. Replaced by Aratu to WGS 84 (13) through (15) (tfm codes 5051, 5053 and 5055) which Petrobras now recommends for the areas.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4208','EPSG','4326',10.0,-158.0,315.0,-148.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'PB-Bra BC BS ES',0);
INSERT INTO "usage" VALUES('EPSG','11765','helmert_transformation','EPSG','15754','EPSG','2307','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','15755','Minna to WGS 84 (14)','Derived in 1995 at unspecified DMA ADOS stations and Racal stations M101 and ZVS3003. Used by Elf in onshore Block OML 58.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4263','EPSG','4326',7.0,-90.2,-87.32,114.17,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'Elf-Nga-OML58',0);
INSERT INTO "usage" VALUES('EPSG','11766','helmert_transformation','EPSG','15755','EPSG','3113','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','15756','Tahiti 79 to RGPF (1)','May be taken as approximate transformation Tahiti 79 to WGS 84 - see code 4835.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4690','EPSG','4687',0.5,221.525,152.948,176.768,'EPSG','9001',2.3847,1.3896,0.877,'EPSG','9104',11.4741,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Pyf',1);
INSERT INTO "usage" VALUES('EPSG','11767','helmert_transformation','EPSG','15756','EPSG','3124','EPSG','1035');
INSERT INTO "helmert_transformation" VALUES('EPSG','15757','Moorea 87 to RGPF (1)','May be taken as approximate transformation Moorea 87 to WGS 84 - see code 15769.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4691','EPSG','4687',0.5,215.525,149.593,176.229,'EPSG','9001',3.2624,1.692,1.1571,'EPSG','9104',10.4773,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Pyf',1);
INSERT INTO "usage" VALUES('EPSG','11768','helmert_transformation','EPSG','15757','EPSG','3125','EPSG','1035');
INSERT INTO "helmert_transformation" VALUES('EPSG','15758','Tahaa 54 to RGPF (1)','May be taken as approximate transformation Tahaa 54 to WGS 84 - see code 15770.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4629','EPSG','4687',0.5,72.438,345.918,79.486,'EPSG','9001',-1.6045,-0.8823,-0.5565,'EPSG','9104',1.3746,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Pyf',1);
INSERT INTO "usage" VALUES('EPSG','11769','helmert_transformation','EPSG','15758','EPSG','2812','EPSG','1035');
INSERT INTO "helmert_transformation" VALUES('EPSG','15759','Maupiti 83 to RGPF (1)','May be taken as approximate transformation Maupiti 83 to WGS 84 - see code 15771.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4692','EPSG','4687',0.5,217.037,86.959,23.956,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Pyf',0);
INSERT INTO "usage" VALUES('EPSG','11770','helmert_transformation','EPSG','15759','EPSG','3126','EPSG','1035');
INSERT INTO "helmert_transformation" VALUES('EPSG','15760','Fatu Iva 72 to RGPF (1)','May be taken as approximate transformation Fatu Iva 72 to WGS 84 - see code 15772.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4688','EPSG','4687',2.0,347.103,1078.125,2623.922,'EPSG','9001',33.8875,-70.6773,9.3943,'EPSG','9104',186.074,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Pyf',1);
INSERT INTO "usage" VALUES('EPSG','11771','helmert_transformation','EPSG','15760','EPSG','3133','EPSG','1151');
INSERT INTO "helmert_transformation" VALUES('EPSG','15761','IGN63 Hiva Oa to RGPF (1)','May be taken as approximate transformation IGN63 Hiva Oa to WGS 84 - see code 15773.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4689','EPSG','4687',0.5,410.721,55.049,80.746,'EPSG','9001',-2.5779,-2.3514,-0.6664,'EPSG','9104',17.3311,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Pyf HivaOa',1);
INSERT INTO "usage" VALUES('EPSG','11772','helmert_transformation','EPSG','15761','EPSG','3131','EPSG','1035');
INSERT INTO "helmert_transformation" VALUES('EPSG','15762','IGN63 Hiva Oa to RGPF (2)','May be taken as approximate transformation IGN63 Hiva Oa to WGS 84 - see code 15774.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4689','EPSG','4687',2.0,374.715,-58.407,-0.957,'EPSG','9001',-16.2111,-11.4626,-5.5357,'EPSG','9104',-0.5409,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Pyf Tahuata',1);
INSERT INTO "usage" VALUES('EPSG','11773','helmert_transformation','EPSG','15762','EPSG','3132','EPSG','1151');
INSERT INTO "helmert_transformation" VALUES('EPSG','15763','IGN72 Nuku Hiva to RGPF (1)','May be taken as approximate transformation IGN72 Nuku Hiva to WGS 84 - see code 15775.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4630','EPSG','4687',0.5,165.732,216.72,180.505,'EPSG','9001',-0.6434,-0.4512,-0.0791,'EPSG','9104',7.4204,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Pyf NukuHiva',1);
INSERT INTO "usage" VALUES('EPSG','11774','helmert_transformation','EPSG','15763','EPSG','2810','EPSG','1035');
INSERT INTO "helmert_transformation" VALUES('EPSG','15764','IGN72 Nuku Hiva to RGPF (2)','May be taken as approximate transformation IGN72 Nuku Hiva to WGS 84 - see code 15776.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4630','EPSG','4687',2.0,1363.785,1362.687,398.811,'EPSG','9001',-4.5322,-6.7579,-1.0574,'EPSG','9104',268.361,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Pyf UaHuka',1);
INSERT INTO "usage" VALUES('EPSG','11775','helmert_transformation','EPSG','15764','EPSG','3127','EPSG','1151');
INSERT INTO "helmert_transformation" VALUES('EPSG','15765','IGN72 Nuku Hiva to RGPF (3)','May be taken as approximate transformation IGN72 Nuku Hiva to WGS 84 - see code 15777.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4630','EPSG','4687',0.5,259.551,297.612,197.833,'EPSG','9001',1.4866,2.1224,0.4612,'EPSG','9104',27.0249,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Pyf UaPou',1);
INSERT INTO "usage" VALUES('EPSG','11776','helmert_transformation','EPSG','15765','EPSG','3128','EPSG','1035');
INSERT INTO "helmert_transformation" VALUES('EPSG','15766','RGPF to WGS 84 (1)','Transformation is to original definition of WGS 84. It is consistent with later WGS 84 realisations G730, G873 and G1150 to no better than 1m.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4999','EPSG','4979',0.5,0.072,-0.507,-0.245,'EPSG','9001',0.0183,-0.0003,0.007,'EPSG','9104',-0.0093,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Pyf',1);
INSERT INTO "usage" VALUES('EPSG','11777','helmert_transformation','EPSG','15766','EPSG','1098','EPSG','1035');
INSERT INTO "helmert_transformation" VALUES('EPSG','15767','RGPF to WGS 84 (2)','Approximation at the +/- 1m level assuming that RGPF is equivalent to WGS 84.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4999','EPSG','4979',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Pyf',1);
INSERT INTO "usage" VALUES('EPSG','11778','helmert_transformation','EPSG','15767','EPSG','1098','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','15768','Tahiti 79 to WGS 84 (1)','Approximation at the +/- 1m level assuming that RGPF is equivalent to WGS 84. Parameter values taken from Tahiti 79 to RGPF (1) (tfm code 15756).','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4690','EPSG','4687',1.0,221.525,152.948,176.768,'EPSG','9001',2.3847,1.3896,0.877,'EPSG','9104',11.4741,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Pyf',1);
INSERT INTO "usage" VALUES('EPSG','11779','helmert_transformation','EPSG','15768','EPSG','3124','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','15769','Moorea 87 to WGS 84 (1)','Approximation at the +/- 1m level assuming that RGPF is equivalent to WGS 84. Parameter values taken from Moorea 87 to RGPF (1) (tfm code 15757).','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4691','EPSG','4326',1.0,215.525,149.593,176.229,'EPSG','9001',3.2624,1.692,1.1571,'EPSG','9104',10.4773,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Pyf',1);
INSERT INTO "usage" VALUES('EPSG','11780','helmert_transformation','EPSG','15769','EPSG','3125','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','15770','Tahaa 54 to WGS 84 (2)','Approximation at the +/- 1m level assuming that RGPF is equivalent to WGS 84. Parameter values taken from Tahaa 54 to RGPF (1) (tfm code 15758).','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4629','EPSG','4326',1.0,72.438,345.918,79.486,'EPSG','9001',-1.6045,-0.8823,-0.5565,'EPSG','9104',1.3746,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Pyf',1);
INSERT INTO "usage" VALUES('EPSG','11781','helmert_transformation','EPSG','15770','EPSG','2812','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','15771','Maupiti 83 to WGS 84 (1)','Approximation at the +/- 1m level assuming that RGPF is equivalent to WGS 84. Parameter values taken from Maupiti 83 to RGPF (1) (tfm code 15759).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4692','EPSG','4326',1.0,217.037,86.959,23.956,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Pyf',1);
INSERT INTO "usage" VALUES('EPSG','11782','helmert_transformation','EPSG','15771','EPSG','3126','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','15772','Fatu Iva 72 to WGS 84 (1)','Approximation at the +/- 1m level assuming that RGPF is equivalent to WGS 84. Parameter values taken from Fatu Iva 72 to RGPF (1) (tfm code 15760).','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4688','EPSG','4326',2.0,347.103,1078.125,2623.922,'EPSG','9001',33.8875,-70.6773,9.3943,'EPSG','9104',186.074,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Pyf',1);
INSERT INTO "usage" VALUES('EPSG','11783','helmert_transformation','EPSG','15772','EPSG','3133','EPSG','1042');
INSERT INTO "helmert_transformation" VALUES('EPSG','15773','IGN63 Hiva Oa to WGS 84 (1)','Approximation at the +/- 1m level assuming that RGPF is equivalent to WGS 84. Parameter values taken from IGN63 Hiva Oa to RGPF (1) (tfm code 15761).','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4689','EPSG','4326',2.0,410.721,55.049,80.746,'EPSG','9001',-2.5779,-2.3514,-0.6664,'EPSG','9104',17.3311,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Pyf HivaOa',1);
INSERT INTO "usage" VALUES('EPSG','11784','helmert_transformation','EPSG','15773','EPSG','3131','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','15774','IGN63 Hiva Oa to WGS 84 (2)','Approximation at the +/- 1m level assuming that RGPF is equivalent to WGS 84. Parameter values taken from IGN63 Hiva Oa to RGPF (2) (tfm code 15762).','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4689','EPSG','4326',2.0,374.716,-58.407,-0.957,'EPSG','9001',-16.2111,-11.4626,-5.5357,'EPSG','9104',-0.5409,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Pyf Tahuata',1);
INSERT INTO "usage" VALUES('EPSG','11785','helmert_transformation','EPSG','15774','EPSG','3132','EPSG','1042');
INSERT INTO "helmert_transformation" VALUES('EPSG','15775','IGN72 Nuku Hiva to WGS 84 (2)','Approximation at the +/- 1m level assuming that RGPF is equivalent to WGS 84. Parameter values taken from IGN72 Nuku Hiva to RGPF (1) (tfm code 15763).','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4630','EPSG','4326',1.0,165.732,216.72,180.505,'EPSG','9001',-0.6434,-0.4512,-0.0791,'EPSG','9104',7.4204,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Pyf NukuHiva',1);
INSERT INTO "usage" VALUES('EPSG','11786','helmert_transformation','EPSG','15775','EPSG','2810','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','15776','IGN72 Nuku Hiva to WGS 84 (3)','Approximation at the +/- 1m level assuming that RGPF is equivalent to WGS 84. Parameter values taken from IGN72 Nuku Hiva to RGPF (2) (tfm code 15764).','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4630','EPSG','4326',2.0,1363.785,1362.687,398.811,'EPSG','9001',-4.5322,-6.7579,-1.0574,'EPSG','9104',268.361,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Pyf UaHuka',1);
INSERT INTO "usage" VALUES('EPSG','11787','helmert_transformation','EPSG','15776','EPSG','3127','EPSG','1042');
INSERT INTO "helmert_transformation" VALUES('EPSG','15777','IGN72 Nuku Hiva to WGS 84 (4)','Approximation at the +/- 1m level assuming that RGPF is equivalent to WGS 84. Parameter values taken from IGN72 Nuku Hiva to RGPF (2) (tfm code 15765).','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4630','EPSG','4326',1.0,259.551,297.612,197.833,'EPSG','9001',1.4866,2.1224,0.4612,'EPSG','9104',27.0249,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Pyf UaPou',1);
INSERT INTO "usage" VALUES('EPSG','11788','helmert_transformation','EPSG','15777','EPSG','3128','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','15778','ELD79 to WGS 84 (7)','Derived by Total at stations SDL 130-03, 04 and 05 in May 2005.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4159','EPSG','4326',0.5,-114.7,-98.5,-150.7,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'Tot-Lby NC192',0);
INSERT INTO "usage" VALUES('EPSG','11789','helmert_transformation','EPSG','15778','EPSG','3142','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','15779','Gulshan 303 to WGS 84 (1)','Derived at origin station in Dhaka. Source information given to 3 decimal places but rounded by OGP to be commensurate with stated accuracy. For consistency with Survey of Bangladesh, Gulshan 303 to WGS 84 (2) [code 9679] should be preferred.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4682','EPSG','4326',1.0,283.7,735.9,261.1,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-Bgd rounded',0);
INSERT INTO "usage" VALUES('EPSG','11790','helmert_transformation','EPSG','15779','EPSG','1041','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','15780','POSGAR 94 to WGS 84 (1)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4190','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Arg',1);
INSERT INTO "usage" VALUES('EPSG','11791','helmert_transformation','EPSG','15780','EPSG','1033','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','15782','Campo Inchauspe to POSGAR 94 (1)','Adopted from U.S. Defense Mapping Agency values for Campo Inchauspe to WGS 84 (tfm code 1127) assuming that POSGAR 94 is equivalent to WGS 84.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4221','EPSG','4694',5.0,-148.0,136.0,90.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGM-Arg',0);
INSERT INTO "usage" VALUES('EPSG','11793','helmert_transformation','EPSG','15782','EPSG','3215','EPSG','1045');
INSERT INTO "helmert_transformation" VALUES('EPSG','15783','IGN53 Mare to WGS 84 (2)','Withdrawn by information source and replaced by improved information from local authority - see tfm code 15901.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4641','EPSG','4326',5.0,287.0,178.0,-136.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Ncl Mare',0);
INSERT INTO "usage" VALUES('EPSG','11794','helmert_transformation','EPSG','15783','EPSG','2819','EPSG','1045');
INSERT INTO "helmert_transformation" VALUES('EPSG','15784','Le Pouce 1934 to WGS 84 (1)','Derived at 17 stations in 1994 by University of East London. Residuals less than 2m.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4699','EPSG','4326',2.0,-770.1,158.4,-498.2,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'UEL-Mus',0);
INSERT INTO "usage" VALUES('EPSG','11795','helmert_transformation','EPSG','15784','EPSG','3209','EPSG','1042');
INSERT INTO "helmert_transformation" VALUES('EPSG','15787','IGCB 1955 to WGS 84 (1)','Derived by Topnav in 1991 at station TSH 85.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4701','EPSG','4326',5.0,-79.9,-158.0,-168.9,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'Tot-Cod',0);
INSERT INTO "usage" VALUES('EPSG','11798','helmert_transformation','EPSG','15787','EPSG','3171','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','15788','AGD66 to WGS 84 (16)','Parameter values from AGD66 to GDA94 (1) (code 1278). Approximation assuming WGS 84 is equivalent to GDA94; ignores the low accuracy of the WGS 84 ensemble and the inconsistent application of tectonic plate motion to WGS 84 data.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4202','EPSG','4326',5.0,-127.8,-52.3,152.9,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-Aus 5m',0);
INSERT INTO "usage" VALUES('EPSG','11799','helmert_transformation','EPSG','15788','EPSG','2575','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','15789','AGD84 to WGS 84 (8)','Parameter values from AGD84 to GDA94 (1) (code 1279). Approximation assuming WGS 84 is equivalent to GDA94; ignores the low accuracy of the WGS 84 ensemble and the inconsistent application of tectonic plate motion to WGS 84 data.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4203','EPSG','4326',5.0,-128.5,-53.0,153.4,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-Aus 5m',0);
INSERT INTO "usage" VALUES('EPSG','11800','helmert_transformation','EPSG','15789','EPSG','2576','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','15790','Mhast (offshore) to WGS 72BE (1)','Derived by Oceaneering for CABGOC in 1979. Mean of parameters derived by single point Transit translocation at 2 stations (Mongo Tando and N''To). Applied to single point Transit translocations at other stations to define Mhast (offshore) coordinates.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4705','EPSG','4324',10.0,-255.0,-29.0,-105.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'CHV-Ago Cab',0);
INSERT INTO "usage" VALUES('EPSG','11801','helmert_transformation','EPSG','15790','EPSG','3180','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','15791','Malongo 1987 to WGS 84 (3)','Derived via WGS 72BE by Geodetic for Chevron in 1987 by single point Transit translocation at 1 station (Malongo Y). Replaced in 1989 by Malongo 1987 to WGS 84 (1) (code 1330).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4259','EPSG','4326',10.0,-259.99,-5.28,-97.09,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'CHV-Ago Cab87',0);
INSERT INTO "usage" VALUES('EPSG','11802','helmert_transformation','EPSG','15791','EPSG','3180','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','15792','Egypt Gulf of Suez S-650 TL to WGS 72BE (1)','Derived by Egypt Surveys Limited through single point Transit translocation at 1 station (S-650).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4706','EPSG','4324',5.0,-123.0,98.0,2.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'ESL-Egy GoS',0);
INSERT INTO "usage" VALUES('EPSG','11803','helmert_transformation','EPSG','15792','EPSG','2341','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','15793','Barbados 1938 to WGS 84 (1)','Derived at 2 stations (S40 and M1, St Annes Tower) in 2004. Accuracy 2.5m.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4212','EPSG','4326',3.0,31.95,300.99,419.19,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'UKHO-Brb',0);
INSERT INTO "usage" VALUES('EPSG','11804','helmert_transformation','EPSG','15793','EPSG','3218','EPSG','1043');
INSERT INTO "helmert_transformation" VALUES('EPSG','15794','Cocos Islands 1965 to WGS 84 (1)','Derived at 1 satellite station. Accuracy 25m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4708','EPSG','4326',44.0,-491.0,-22.0,435.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Cck',0);
INSERT INTO "usage" VALUES('EPSG','11805','helmert_transformation','EPSG','15794','EPSG','1069','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','15795','Tern Island 1961 to WGS 84 (1)','Derived at 1 satellite station. Same transformation parameter values related to same datum area given in original 1987 DMA TR8350.2 edition for Sorol Atoll.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4707','EPSG','4326',44.0,114.0,-116.0,-333.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Usa HI Tern',0);
INSERT INTO "usage" VALUES('EPSG','11806','helmert_transformation','EPSG','15795','EPSG','3181','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','15796','Iwo Jima 1945 to WGS 84 (1)','Derived at 1 satellite station. Accuracy 25m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4709','EPSG','4326',44.0,145.0,75.0,-272.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Jpn IwoJ',0);
INSERT INTO "usage" VALUES('EPSG','11807','helmert_transformation','EPSG','15796','EPSG','3200','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','15797','Ascension Island 1958 to WGS 84 (1)','Derived at 2 satellite stations. Accuracy 25m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4712','EPSG','4326',44.0,-205.0,107.0,53.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Shn Asc',0);
INSERT INTO "usage" VALUES('EPSG','11808','helmert_transformation','EPSG','15797','EPSG','3182','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','15798','Astro DOS 71 to WGS 84 (1)','Derived at 1 satellite station. Accuracy 25m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4710','EPSG','4326',44.0,-320.0,550.0,-494.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Shn Hel',0);
INSERT INTO "usage" VALUES('EPSG','11809','helmert_transformation','EPSG','15798','EPSG','3183','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','15799','Marcus Island 1952 to WGS 84 (1)','Derived at 1 satellite station. Accuracy 25m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4711','EPSG','4326',44.0,124.0,-234.0,-25.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Jpn Marcus',0);
INSERT INTO "usage" VALUES('EPSG','11810','helmert_transformation','EPSG','15799','EPSG','1872','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','15800','Ayabelle Lighthouse to WGS 84 (1)','Derived at 1 satellite station. Accuracy 25m in each axis. Replaced by Ayabelle Lighthouse to WGS 84 (2) (code 6907).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4713','EPSG','4326',44.0,-79.0,-129.0,145.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Dji',0);
INSERT INTO "usage" VALUES('EPSG','11811','helmert_transformation','EPSG','15800','EPSG','1081','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','15801','Bellevue to WGS 84 (1)','Derived at 3 satellite stations. Accuracy +/- 20 m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4714','EPSG','4326',35.0,-127.0,-769.0,472.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Vut',0);
INSERT INTO "usage" VALUES('EPSG','11812','helmert_transformation','EPSG','15801','EPSG','3193','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','15802','Camp Area Astro to WGS 84 (1)','No accuracy estimate available.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4715','EPSG','4326',999.0,-104.0,-129.0,239.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Ata McMurdo',0);
INSERT INTO "usage" VALUES('EPSG','11813','helmert_transformation','EPSG','15802','EPSG','3205','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','15803','Phoenix Islands 1966 to WGS 84 (1)','Derived at 4 satellite stations. Accuracy +/- 15 m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4716','EPSG','4326',26.0,298.0,-304.0,-375.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Kir Phoenix',0);
INSERT INTO "usage" VALUES('EPSG','11814','helmert_transformation','EPSG','15803','EPSG','3196','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','15804','Cape Canaveral to WGS 84 (1)','Derived at 19 satellite stations. Accuracy +/- 3 m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4717','EPSG','4326',6.0,-2.0,151.0,181.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Bha Usa-FL',0);
INSERT INTO "usage" VALUES('EPSG','11815','helmert_transformation','EPSG','15804','EPSG','3206','EPSG','1233');
INSERT INTO "helmert_transformation" VALUES('EPSG','15805','Solomon 1968 to WGS 84 (1)','Derived at 1 satellite station. Accuracy +/- 25m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4718','EPSG','4326',44.0,230.0,-199.0,-752.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Slb Gizo',0);
INSERT INTO "usage" VALUES('EPSG','11816','helmert_transformation','EPSG','15805','EPSG','3198','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','15806','Easter Island 1967 to WGS 84 (1)','Derived at 1 satellite station. Accuracy +/- 25m in each axis','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4719','EPSG','4326',44.0,211.0,147.0,111.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Chl Easter',0);
INSERT INTO "usage" VALUES('EPSG','11817','helmert_transformation','EPSG','15806','EPSG','3188','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','15807','Solomon 1968 to WGS 84 (2)','Derived at 1 satellite station. Accuracy +/- 25m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4718','EPSG','4326',44.0,252.0,-209.0,-751.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Slb Guad',0);
INSERT INTO "usage" VALUES('EPSG','11818','helmert_transformation','EPSG','15807','EPSG','3197','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','15808','Diego Garcia 1969 to WGS 84 (1)','Derived at 2 satellite stations. Accuracy +/- 25m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4724','EPSG','4326',44.0,208.0,-435.0,-229.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Iot Garcia',0);
INSERT INTO "usage" VALUES('EPSG','11819','helmert_transformation','EPSG','15808','EPSG','3189','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','15809','Johnston Island 1961 to WGS 84 (1)','Derived at 2 satellite stations. Accuracy +/- 25m in each axis. Note: NGA online html files carry a different dZ value - OGP believe this is an erroneous transcription from the TR8350.2 line above.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4725','EPSG','4326',44.0,189.0,-79.0,-202.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Umi Johnston',0);
INSERT INTO "usage" VALUES('EPSG','11820','helmert_transformation','EPSG','15809','EPSG','3201','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','15810','Kusaie 1951 to WGS 84 (1)','Derived at 1 satellite station. Accuracy +/- 25m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4735','EPSG','4326',44.0,647.0,1777.0,-1124.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Fsm Carol',0);
INSERT INTO "usage" VALUES('EPSG','11821','helmert_transformation','EPSG','15810','EPSG','3192','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','15811','Antigua 1943 to WGS 84 (2)','Determined from 1 satellite station. Accuracy +/- 25m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4601','EPSG','4326',44.0,-270.0,13.0,62.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Atg Ant',0);
INSERT INTO "usage" VALUES('EPSG','11822','helmert_transformation','EPSG','15811','EPSG','1273','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','15812','Deception Island to WGS 84 (1)','Accuracy +/- 20m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4736','EPSG','4326',35.0,260.0,12.0,-147.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Ata Dec',0);
INSERT INTO "usage" VALUES('EPSG','11823','helmert_transformation','EPSG','15812','EPSG','3204','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','15813','South Georgia 1968 to WGS 84 (1)','Determined from 1 satellite station. Accuracy +/- 25m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4722','EPSG','4326',44.0,-794.0,119.0,-298.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Sgs Sgeorg',0);
INSERT INTO "usage" VALUES('EPSG','11824','helmert_transformation','EPSG','15813','EPSG','3529','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','15814','SIGD61 to WGS 84 (1)','Determined from 1 satellite station. Accuracy +/- 25m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4726','EPSG','4326',44.0,42.0,124.0,147.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Cym Little Brac',0);
INSERT INTO "usage" VALUES('EPSG','11825','helmert_transformation','EPSG','15814','EPSG','3186','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','15815','PN84 to WGS 84 (1)','Determined at 1 satellite station. Accuracy +/- 25m in each axis. Unclear from information source whether the source CRS is PN68 or PN84 - see also CT code 9743. Given the accuracy, can be considered to apply to either.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4728','EPSG','4326',44.0,-307.0,-92.0,127.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Esp Canary',0);
INSERT INTO "usage" VALUES('EPSG','11826','helmert_transformation','EPSG','15815','EPSG','4598','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','15816','Tristan 1968 to WGS 84 (1)','Determined at 1 satellite station. Accuracy +/- 25m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4734','EPSG','4326',44.0,-632.0,438.0,-609.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Shn Tris',0);
INSERT INTO "usage" VALUES('EPSG','11827','helmert_transformation','EPSG','15816','EPSG','3184','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','15817','Midway 1961 to WGS 84 (1)','Derived at 1 satellite station. Accuracy +/- 25m in each axis. Information source states "provided for historical purposes only. These parameter [values] should not be used". Replaced by Midway 1961 to WGS 84 (2) (tfm code 15818).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4727','EPSG','4326',44.0,912.0,-58.0,1227.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Umi Midway 1987',0);
INSERT INTO "usage" VALUES('EPSG','11828','helmert_transformation','EPSG','15817','EPSG','3202','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','15818','Midway 1961 to WGS 84 (2)','Derived at 1 satellite station. Accuracy +/- 25m in each axis. Replaces Midway 1961 to WGS 84 (1) (tfm code 15817).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4727','EPSG','4326',44.0,403.0,-81.0,277.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Umi Midway 2003',0);
INSERT INTO "usage" VALUES('EPSG','11829','helmert_transformation','EPSG','15818','EPSG','3202','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','15819','Pitcairn 1967 to WGS 84 (1)','Derived at 1 satellite station. Accuracy +/- 25 m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4729','EPSG','4326',44.0,185.0,165.0,42.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Pcn Pitcairn Isl',0);
INSERT INTO "usage" VALUES('EPSG','11830','helmert_transformation','EPSG','15819','EPSG','3208','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','15820','Santo 1965 to WGS 84 (1)','Derived at 1 satellite station. Accuracy 25m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4730','EPSG','4326',44.0,170.0,42.0,84.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Vut',0);
INSERT INTO "usage" VALUES('EPSG','11831','helmert_transformation','EPSG','15820','EPSG','3194','EPSG','1153');
INSERT INTO "helmert_transformation" VALUES('EPSG','15821','Viti Levu 1916 to WGS 84 (1)','Derived at 1 satellite station. Accuracy +/-25m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4731','EPSG','4326',44.0,51.0,391.0,-36.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Fji',1);
INSERT INTO "usage" VALUES('EPSG','11832','helmert_transformation','EPSG','15821','EPSG','3195','EPSG','1153');
INSERT INTO "helmert_transformation" VALUES('EPSG','15822','Marshall Islands 1960 to WGS 84 (1)','Derived at 10 satellite stations. Accuracy +/-3 m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4732','EPSG','4326',6.0,102.0,52.0,-38.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Mhl 1960',0);
INSERT INTO "usage" VALUES('EPSG','11833','helmert_transformation','EPSG','15822','EPSG','3191','EPSG','1153');
INSERT INTO "helmert_transformation" VALUES('EPSG','15823','Wake Island 1952 to WGS 84 (1)','Derived at 2 satellite stations. Accuracy +/-25m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4733','EPSG','4326',44.0,276.0,-57.0,149.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Mhl Wake',0);
INSERT INTO "usage" VALUES('EPSG','11834','helmert_transformation','EPSG','15823','EPSG','3190','EPSG','1153');
INSERT INTO "helmert_transformation" VALUES('EPSG','15824','Old Hawaiian to WGS 84 (3)','Derived at 15 satellite stations. Accuracy +/- 25m in X axis, +/- 20m in Y and Z axes.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4135','EPSG','4326',38.0,61.0,-285.0,-181.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Usa HI 1987',0);
INSERT INTO "usage" VALUES('EPSG','11835','helmert_transformation','EPSG','15824','EPSG','1334','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','15825','Old Hawaiian to WGS 84 (4)','Derived at 2 satellite stations. Accuracy +/- 25m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4135','EPSG','4326',44.0,89.0,-279.0,-183.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Usa HI Haw 1991',0);
INSERT INTO "usage" VALUES('EPSG','11836','helmert_transformation','EPSG','15825','EPSG','1546','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','15826','Old Hawaiian to WGS 84 (5)','Derived at 3 satellite stations. Accuracy +/- 20m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4135','EPSG','4326',35.0,45.0,-290.0,-172.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Usa HI Kauai 1991',0);
INSERT INTO "usage" VALUES('EPSG','11837','helmert_transformation','EPSG','15826','EPSG','1549','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','15827','Old Hawaiian to WGS 84 (6)','Derived at 2 satellite stations. Accuracy +/- 25m in each axis.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4135','EPSG','4326',44.0,65.0,-290.0,-190.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Usa HI Maui 1991',0);
INSERT INTO "usage" VALUES('EPSG','11838','helmert_transformation','EPSG','15827','EPSG','1547','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','15828','Old Hawaiian to WGS 84 (7)','Derived at 8 satellite stations. Accuracy +/- 10m in X axis, +/- 6m in Y and Z axes.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4135','EPSG','4326',14.0,58.0,-283.0,-182.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Usa HI Oahu 1991',0);
INSERT INTO "usage" VALUES('EPSG','11839','helmert_transformation','EPSG','15828','EPSG','1548','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','15829','SIGD61 to WGS 84 (2)','Determined from 2 satellite stations. Accuracy +/- 1m.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4726','EPSG','4326',1.0,44.4,109.0,151.7,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'UNO-Cym Little Brac',0);
INSERT INTO "usage" VALUES('EPSG','11840','helmert_transformation','EPSG','15829','EPSG','3186','EPSG','1153');
INSERT INTO "helmert_transformation" VALUES('EPSG','15830','GCGD59 to WGS 84 (1)','Determined from 6 satellite stations. Accuracy +/- 1m.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4723','EPSG','4326',1.0,67.8,106.1,138.8,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'UNO-Cym Grand',0);
INSERT INTO "usage" VALUES('EPSG','11841','helmert_transformation','EPSG','15830','EPSG','3185','EPSG','1153');
INSERT INTO "helmert_transformation" VALUES('EPSG','15831','KGD2002 to WGS 84 (1)','Approximation at the +/- 1m level assuming that ITRF2000 is equivalent to WGS 84.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4737','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-Kor',0);
INSERT INTO "usage" VALUES('EPSG','11842','helmert_transformation','EPSG','15831','EPSG','1135','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','15832','RGPF to WGS 84 (1)','Transformation is to original definition of WGS 84. It is consistent with later WGS 84 realisations G730, G873 and G1150 to no better than 1m.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4687','EPSG','4326',0.5,0.072,-0.507,-0.245,'EPSG','9001',0.0183,-0.0003,0.007,'EPSG','9104',-0.0093,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Pyf',1);
INSERT INTO "usage" VALUES('EPSG','11843','helmert_transformation','EPSG','15832','EPSG','1098','EPSG','1035');
INSERT INTO "helmert_transformation" VALUES('EPSG','15833','RGPF to WGS 84 (2)','Approximation at the +/- 1m level assuming that RGPF is equivalent to WGS 84.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4687','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Pyf',0);
INSERT INTO "usage" VALUES('EPSG','11844','helmert_transformation','EPSG','15833','EPSG','1098','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','15842','Hong Kong 1963(67) to WGS 84 (1)','Derived at 2 satellite stations. Accuracy +/- 1m. Care: does not use Hong Kong 1963 (code 4838) as the source CRS.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4739','EPSG','4326',1.0,-156.0,-271.0,-189.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'UKHO-Hkg',0);
INSERT INTO "usage" VALUES('EPSG','11853','helmert_transformation','EPSG','15842','EPSG','1118','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','15843','PZ-90 to WGS 84 (1)','Derived through Glonass and GPS at 30 stations throughout USSR - Former Soviet Union (FSU). Accuracy better than 1.5 metres.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4740','EPSG','4326',1.5,0.0,0.0,1.5,'EPSG','9001',0.0,0.0,-0.076,'EPSG','9104',0.0,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'GiK-World',0);
INSERT INTO "usage" VALUES('EPSG','11854','helmert_transformation','EPSG','15843','EPSG','1262','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','15844','Pulkovo 1942 to PZ-90 (1)','Derived through Glonass at 30 stations throughout USSR - Former Soviet Union (FSU). Mandated for use in Russia by GosStandard of Russia Decree #327 of August 9, 2001.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4284','EPSG','4740',4.0,25.0,-141.0,-80.0,'EPSG','9001',0.0,-0.35,-0.66,'EPSG','9104',0.0,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'GiK-Rus',0);
INSERT INTO "usage" VALUES('EPSG','11855','helmert_transformation','EPSG','15844','EPSG','2423','EPSG','1153');
INSERT INTO "helmert_transformation" VALUES('EPSG','15845','Pampa del Castillo to WGS 84 (1)','Transformation parameter precision given to millimetres in information source but due to accuracy rounded to nearest decimetre for EPSG database.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4161','EPSG','4326',25.0,27.5,14.0,186.4,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'UNO-Arg ComRiv',1);
INSERT INTO "usage" VALUES('EPSG','11856','helmert_transformation','EPSG','15845','EPSG','1265','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','15846','Egypt Gulf of Suez S-650 TL to WGS 84 (2)','Sometime referred to as "Egypt 1907 to WGS 84". However, application to WGS 84 coordinates of the reverse of this tfm results in Gulf of Suez S-650 TL, not Egypt 1907, position. Gulf of Suez S-650 TL and Egypt 1907 CRSs differ by some 20 metres.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4706','EPSG','4326',5.0,-146.21,112.63,4.05,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'Racal-Egy GoS',0);
INSERT INTO "usage" VALUES('EPSG','11857','helmert_transformation','EPSG','15846','EPSG','2341','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','15847','MOP78 to WGS 84 (2)','Replaces information from 2001 (tfm code 1925).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4639','EPSG','4326',10.0,253.0,-132.0,-127.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Wlf Wallis',0);
INSERT INTO "usage" VALUES('EPSG','11858','helmert_transformation','EPSG','15847','EPSG','2815','EPSG','1050');
INSERT INTO "helmert_transformation" VALUES('EPSG','15848','ST84 Ile des Pins to WGS 84 (2)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4642','EPSG','4326',10.0,-13.0,-348.0,292.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Ncl Pins',0);
INSERT INTO "usage" VALUES('EPSG','11859','helmert_transformation','EPSG','15848','EPSG','2820','EPSG','1050');
INSERT INTO "helmert_transformation" VALUES('EPSG','15849','Beduaram to WGS 84 (2)','Used by Elf / CGG between December 1991 and March 1992. Probably derived from results of concatenated tfm Beduaram to WGS 84 (1) (code 8634).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4213','EPSG','4326',15.0,-106.0,-87.0,188.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'ELF-Ner SE 91',0);
INSERT INTO "usage" VALUES('EPSG','11860','helmert_transformation','EPSG','15849','EPSG','2771','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','15850','IGN 1962 Kerguelen to WGS 84 (1)','Also published in US NIMA/NGA TR8350.2 which gives accuracy of +/-25m in each axis and states that derived at one station.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4698','EPSG','4326',10.0,145.0,-187.0,103.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Atf Kerg',0);
INSERT INTO "usage" VALUES('EPSG','11861','helmert_transformation','EPSG','15850','EPSG','2816','EPSG','1050');
INSERT INTO "helmert_transformation" VALUES('EPSG','15852','NAD27 to WGS 84 (80)','Developed by John E Chance and Associates. Replaced by NAD27 to WGS 84 (79) (tfm code 15851).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4267','EPSG','4326',5.0,-3.0,154.0,177.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'JECA-Usa GoM E',0);
INSERT INTO "usage" VALUES('EPSG','11863','helmert_transformation','EPSG','15852','EPSG','3358','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','15853','NAD27 to WGS 84 (81)','Developed by John E Chance and Associates. Replaced by NAD27 to WGS 84 (79) (tfm code 15851).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4267','EPSG','4326',5.0,-7.0,151.0,175.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'JECA-Usa GoM C',0);
INSERT INTO "usage" VALUES('EPSG','11864','helmert_transformation','EPSG','15853','EPSG','3359','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','15854','NAD27 to WGS 84 (82)','Developed by John E Chance and Associates. Replaced by NAD27 to WGS 84 (79) (tfm code 15851).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4267','EPSG','4326',5.0,-7.0,151.0,178.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'JECA-Usa GoM W',0);
INSERT INTO "usage" VALUES('EPSG','11865','helmert_transformation','EPSG','15854','EPSG','3360','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','15855','NAD27 to WGS 84 (83)','Developed by John E Chance and Associates at 21°55''N, 97°20''W. Geoid height used =-17m.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4267','EPSG','4326',5.0,-8.0,125.0,190.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'JECA-Mex GoM Tam',0);
INSERT INTO "usage" VALUES('EPSG','11866','helmert_transformation','EPSG','15855','EPSG','3361','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','15856','NAD27 to WGS 84 (84)','Developed by EnSoCo Inc. Replaced by NAD27 to WGS 84 (79) (tfm code 15851).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4267','EPSG','4326',8.0,-7.0,158.0,172.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'ESC-Usa GoM',0);
INSERT INTO "usage" VALUES('EPSG','11867','helmert_transformation','EPSG','15856','EPSG','3357','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','15860','Mauritania 1999 to WGS 84 (1)','Mauritania 1999 can be considered to be the same as WGS 84 within the accuracy of this transformation.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4702','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-Mau',0);
INSERT INTO "usage" VALUES('EPSG','11871','helmert_transformation','EPSG','15860','EPSG','1157','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','15865','Pulkovo 1942 to WGS 84 (16)','Derived via PZ-90 at 30 stations throughout USSR (Former Soviet Union, FSU) through concatenation of Pulkovo 1942 to PZ-90 (1) (tfm code 15844) and PZ-90 to WGS 84 (1) (tfm code 15843).','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4284','EPSG','4326',4.5,25.0,-141.0,-78.5,'EPSG','9001',0.0,-0.35,-0.736,'EPSG','9104',0.0,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-Rus',0);
INSERT INTO "usage" VALUES('EPSG','11876','helmert_transformation','EPSG','15865','EPSG','2423','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','15866','FD54 to ED50 (1)','Derived at 3 points in 1976. This transformation then used to define ED50 on the Faroe Islands.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4741','EPSG','4230',0.0,-153.33,-169.41,86.39,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'KMS-Fro',0);
INSERT INTO "usage" VALUES('EPSG','11877','helmert_transformation','EPSG','15866','EPSG','3248','EPSG','1116');
INSERT INTO "helmert_transformation" VALUES('EPSG','15867','PD/83 to ETRS89 (1)','Derived at 10 points of the German GPS Network DREF.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4746','EPSG','4258',1.0,599.4,72.4,419.2,'EPSG','9001',-0.062,-0.022,-2.723,'EPSG','9104',6.46,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'BKG-Deu Thur',0);
INSERT INTO "usage" VALUES('EPSG','11878','helmert_transformation','EPSG','15867','EPSG','2544','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','15868','RD/83 to ETRS89 (1)','Derived in 2001 at 31 points of the German GPS Network DREF in former East Germany. Although for high accuracy limited to Saxony, may be taken as approximate transformation between DHDN and WGS 84 for all former East German states - see code 15869.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4745','EPSG','4258',1.0,612.4,77.0,440.2,'EPSG','9001',-0.054,0.057,-2.797,'EPSG','9104',2.55,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'BKG-Deu Sach',0);
INSERT INTO "usage" VALUES('EPSG','11879','helmert_transformation','EPSG','15868','EPSG','2545','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','15869','DHDN to WGS 84 (3)','Parameter values taken from RD/83 to ETRS89 (1) (tfm code 15868) assuming that within the accuracy of the transformation ETRS89 is equivalent to WGS 84 and RD/83 is equivalent to DHDN.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4314','EPSG','4326',2.0,612.4,77.0,440.2,'EPSG','9001',-0.054,0.057,-2.797,'EPSG','9104',2.55,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-Deu E',0);
INSERT INTO "usage" VALUES('EPSG','11880','helmert_transformation','EPSG','15869','EPSG','1343','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','15870','Jouik 1961 to WGS 84 (1)','Derived at 5 points in 2002.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4679','EPSG','4326',1.0,-80.01,253.26,291.19,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'Wood-Mrt',0);
INSERT INTO "usage" VALUES('EPSG','11881','helmert_transformation','EPSG','15870','EPSG','2967','EPSG','1198');
INSERT INTO "helmert_transformation" VALUES('EPSG','15871','Nahrwan 1967 to WGS 84 (6)','Derived by concatenation of parameter values published by IGN Paris from Nahrwan 1967 to WGS 72 at the Nahrwan SE Base trig station near Baghdad with DMA WGS 72 to WGS 84 parameter values.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4270','EPSG','4326',5.0,-242.2,-144.9,370.3,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Irq',1);
INSERT INTO "usage" VALUES('EPSG','11882','helmert_transformation','EPSG','15871','EPSG','3625','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','15872','Karbala 1979 to WGS 84 (1)','Derived from shifts in UTM rectangular coordinates for one point in Basra area provided by Iraq National Oil Exploration Company. Replaced by Karbala 1979 to WGS 84 (2) (tfm code 5078).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4743','EPSG','4326',5.0,84.1,-320.1,218.7,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OEC-Irq Bas',0);
INSERT INTO "usage" VALUES('EPSG','11883','helmert_transformation','EPSG','15872','EPSG','3397','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','15873','Douala 1948 to WGS 84 (1)','Derived at Manoca tower assuming the pyramid on the tower and the centre of the tower reservoir are co-located. This assumption carries a few metres uncertainty.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4192','EPSG','4326',10.0,-206.1,-174.7,-87.7,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'Tot-Cmr',0);
INSERT INTO "usage" VALUES('EPSG','11884','helmert_transformation','EPSG','15873','EPSG','2555','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','15874','Nord Sahara 1959 to WGS 84 (7)','Derived at 11 stations throughout blocks 317b, 319b, 321b and 322b. Network based on station P4 (horizontal) and benchmark RN51 (vertical) using EGM96 geoid height. Used by Equinor in Hassi Mouina. May be taken as a transformation to RGSH2020 (CT 10342).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4307','EPSG','4326',5.0,-169.559,-72.34,303.102,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'ENG-Dza Mou',0);
INSERT INTO "usage" VALUES('EPSG','11885','helmert_transformation','EPSG','15874','EPSG','3402','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','15875','Fiji 1956 to WGS 84 (1)','Derived at 20 stations. Also published by NGA in GeoTrans v3.4 software with parameter values rounded to integer.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4721','EPSG','4326',7.0,265.025,384.929,-194.046,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DGC-Fji',0);
INSERT INTO "usage" VALUES('EPSG','11886','helmert_transformation','EPSG','15875','EPSG','3398','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','15876','Fiji 1986 to WGS 84 (1)','Approximation at the +/- 2m level assuming that Fiji 1986 is equivalent to WGS 72. Parameter values taken from WGS 72 to WGS 84 (1) (tfm code 1237).','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4720','EPSG','4326',2.0,0.0,0.0,4.5,'EPSG','9001',0.0,0.0,0.554,'EPSG','9104',0.2263,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-Fji',0);
INSERT INTO "usage" VALUES('EPSG','11887','helmert_transformation','EPSG','15876','EPSG','1094','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','15877','Fiji 1986 to WGS 84 (2)','Horizontal accuracy 2m, vertical accuracy approximately 40 metres. Suitable for GIS mapping purposes but not rigorous surveying. Very similar results may be obtained through Fiji 1986 to WGS 84 (1) (tfm code 15876).','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4720','EPSG','4326',40.0,-35.173,136.571,-36.964,'EPSG','9001',1.37,-0.842,-4.718,'EPSG','9104',-1.537,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'FD-Fji',0);
INSERT INTO "usage" VALUES('EPSG','11888','helmert_transformation','EPSG','15877','EPSG','3398','EPSG','1157');
INSERT INTO "helmert_transformation" VALUES('EPSG','15878','Vanua Levu 1915 to WGS 84 (1)','Parameter values taken from Viti Levu 1912 to WGS 84 (1) (tfm code 15897). Approximation at the +/- 50m level assuming that CRS 4748 is equivalent to CRS 4752 within the transformation accuracy. Source CRSs 4748 and 4752 are independent but connected.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4748','EPSG','4326',50.0,51.0,391.0,-36.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-Fji',0);
INSERT INTO "usage" VALUES('EPSG','11889','helmert_transformation','EPSG','15878','EPSG','3401','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','15879','GR96 to WGS 84 (1)','Approximation at the +/- 1m level assuming that GR96 is equivalent to WGS 84 within the accuracy of the transformation.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4747','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-Grl',0);
INSERT INTO "usage" VALUES('EPSG','11890','helmert_transformation','EPSG','15879','EPSG','1107','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','15880','RGNC91-93 to WGS 84 (1)','','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4749','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Ncl',0);
INSERT INTO "usage" VALUES('EPSG','11891','helmert_transformation','EPSG','15880','EPSG','1174','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','15881','ST87 Ouvea to WGS 84 (2)','Parameter values taken from ST87 Ouvea to RGNC91-93 (1) (code 15885) assuming that RGNC91-93 is equivalent to WGS 84 to within the accuracy of the transformation.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4750','EPSG','4326',1.0,-56.263,16.136,-22.856,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'BGN-Ncl',0);
INSERT INTO "usage" VALUES('EPSG','11892','helmert_transformation','EPSG','15881','EPSG','2813','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','15882','IGN72 Grande Terre to RGNC91-93 (1)','Determined in May 2001. May be taken as approximate transformation to WGS 84 - see IGN72 Grande Terre to WGS 84 (3) (code 15903).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4662','EPSG','4749',2.0,-11.64,-348.6,291.98,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'BGN-Ncl 2m',0);
INSERT INTO "usage" VALUES('EPSG','11893','helmert_transformation','EPSG','15882','EPSG','2822','EPSG','1042');
INSERT INTO "helmert_transformation" VALUES('EPSG','15883','IGN56 Lifou to RGNC91-93 (1)','Determined in April 1993. May be taken as approximate transformation to WGS 84 - see IGN56 Lifou to WGS 84 (3) (code 15902).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4633','EPSG','4749',1.0,335.47,222.58,-230.94,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'BGN-Ncl 1m',0);
INSERT INTO "usage" VALUES('EPSG','11894','helmert_transformation','EPSG','15883','EPSG','2814','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','15884','IGN53 Mare to RGNC91-93 (1)','Determined in April 1993, modified in December 1999. May be taken as approximate transformation to WGS 84: see IGN53 Mare to WGS 84 (3) (code 15901).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4641','EPSG','4749',2.0,287.58,177.78,-135.41,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'BGN-Ncl 2m',0);
INSERT INTO "usage" VALUES('EPSG','11895','helmert_transformation','EPSG','15884','EPSG','2819','EPSG','1042');
INSERT INTO "helmert_transformation" VALUES('EPSG','15885','ST87 Ouvea to RGNC91-93 (1)','Determined in December 1999. May be used as approximate transformation to WGS 84 - see ST87 Ouvea to WGS 84 (2) (code 15881).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4750','EPSG','4749',0.5,-56.263,16.136,-22.856,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'BGN-Ncl 1m',0);
INSERT INTO "usage" VALUES('EPSG','11896','helmert_transformation','EPSG','15885','EPSG','2813','EPSG','1035');
INSERT INTO "helmert_transformation" VALUES('EPSG','15886','NEA74 Noumea to RGNC91-93 (1)','Determined in July 2000. May be taken as approximate transformation to WGS 84 - see NEA74 Noumea to WGS 84 (3) (code 15904).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4644','EPSG','4749',1.0,-10.18,-350.43,291.37,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'BGN-Ncl 1m',0);
INSERT INTO "usage" VALUES('EPSG','11897','helmert_transformation','EPSG','15886','EPSG','2823','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','15887','IGN72 Grande Terre to RGNC91-93 (2)','Determined in April 1993.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4662','EPSG','4749',0.3,97.297,-263.243,310.879,'EPSG','9001',1.5999,-0.8387,-3.1409,'EPSG','9104',13.326,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'BGN-Ncl 1m',0);
INSERT INTO "usage" VALUES('EPSG','11898','helmert_transformation','EPSG','15887','EPSG','2822','EPSG','1033');
INSERT INTO "helmert_transformation" VALUES('EPSG','15888','IGN72 Grande Terre to RGNC91-93 (3)','Determined in July 2000','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4662','EPSG','4749',0.1,48.812,-205.932,343.993,'EPSG','9001',3.4427,0.4999,-4.0878,'EPSG','9104',6.5215,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'BGN-Ncl Noum 0.1m',0);
INSERT INTO "usage" VALUES('EPSG','11899','helmert_transformation','EPSG','15888','EPSG','2823','EPSG','1031');
INSERT INTO "helmert_transformation" VALUES('EPSG','15889','NEA74 Noumea to RGNC91-93 (2)','Determined in May 2001','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4644','EPSG','4749',0.1,-166.0684,-154.7826,254.8282,'EPSG','9001',37.546,-7.7018,10.2029,'EPSG','9104',-30.84,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'BGN-Ncl 0.1m',0);
INSERT INTO "usage" VALUES('EPSG','11900','helmert_transformation','EPSG','15889','EPSG','2823','EPSG','1031');
INSERT INTO "helmert_transformation" VALUES('EPSG','15890','IGN56 Lifou to RGNC91-93 (2)','Determined in April 1993.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4633','EPSG','4749',0.1,137.092,131.675,91.478,'EPSG','9001',1.9435,11.5995,4.3316,'EPSG','9104',-7.4801,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'BGN-Ncl 0.1m',0);
INSERT INTO "usage" VALUES('EPSG','11901','helmert_transformation','EPSG','15890','EPSG','2814','EPSG','1031');
INSERT INTO "helmert_transformation" VALUES('EPSG','15891','IGN53 Mare to RGNC91-93 (2)','Determined in April 1993, modified in December 1999.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4641','EPSG','4749',0.1,-408.809,366.857,-412.987,'EPSG','9001',-1.8843,0.5308,-2.1657,'EPSG','9104',-121.0994,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'BGN-Ncl 0.1m',0);
INSERT INTO "usage" VALUES('EPSG','11902','helmert_transformation','EPSG','15891','EPSG','2819','EPSG','1031');
INSERT INTO "helmert_transformation" VALUES('EPSG','15892','ST87 Ouvea to RGNC91-93 (2)','Determined in December 1999.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4750','EPSG','4749',0.1,-122.386,-188.707,103.334,'EPSG','9001',-3.511,4.9665,5.7048,'EPSG','9104',4.4799,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'BGN-Ncl 0.1m',0);
INSERT INTO "usage" VALUES('EPSG','11903','helmert_transformation','EPSG','15892','EPSG','2813','EPSG','1031');
INSERT INTO "helmert_transformation" VALUES('EPSG','15893','ST84 Ile des Pins to RGNC91-93 (1)','Determined in December 1999.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4642','EPSG','4749',0.1,244.42,85.352,168.129,'EPSG','9001',8.936,-7.752,-12.5952,'EPSG','9104',14.2723,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'BGN-Ncl 0.1m',0);
INSERT INTO "usage" VALUES('EPSG','11904','helmert_transformation','EPSG','15893','EPSG','2820','EPSG','1031');
INSERT INTO "helmert_transformation" VALUES('EPSG','15894','SIRGAS 2000 to WGS 84 (1)','Approximation at the +/- 1m level assuming that SIRGAS 2000 is equivalent to WGS 84.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4674','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-C&S America',0);
INSERT INTO "usage" VALUES('EPSG','11905','helmert_transformation','EPSG','15894','EPSG','3418','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','15896','Kertau (RSO) to Kertau 1968 (1)','To transform Kertau (RSO) to WGS 84, see concatenated transformation code 8659.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4751','EPSG','4245',0.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-Mys',0);
INSERT INTO "usage" VALUES('EPSG','11907','helmert_transformation','EPSG','15896','EPSG','1309','EPSG','1164');
INSERT INTO "helmert_transformation" VALUES('EPSG','15897','Viti Levu 1912 to WGS 84 (1)','Derived at 1 satellite station. Accuracy +/-25m in each axis. Replaced by Viti Levu 1912 to WGS 84 (2) (code 6895).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4752','EPSG','4326',44.0,51.0,391.0,-36.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DMA-Fji',0);
INSERT INTO "usage" VALUES('EPSG','11908','helmert_transformation','EPSG','15897','EPSG','3195','EPSG','1153');
INSERT INTO "helmert_transformation" VALUES('EPSG','15898','Qornoq to GR96 (1)','Derived via NWL 9D.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4747','EPSG','4747',1.0,163.511,127.533,-159.789,'EPSG','9001',0.0,0.0,0.814,'EPSG','9104',-0.6,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'KMS-Grl',1);
INSERT INTO "usage" VALUES('EPSG','11909','helmert_transformation','EPSG','15898','EPSG','1107','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','15899','Scoresbysund 1952 to GR96 (1)','Derived via NWL 9D.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4195','EPSG','4747',12.0,105.0,326.0,-102.5,'EPSG','9001',0.0,0.0,0.814,'EPSG','9104',-0.6,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'KMS-Grl',0);
INSERT INTO "usage" VALUES('EPSG','11910','helmert_transformation','EPSG','15899','EPSG','2570','EPSG','1026');
INSERT INTO "helmert_transformation" VALUES('EPSG','15900','Ammassalik 1958 to GR96 (1)','Derived via NWL 9D.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4196','EPSG','4747',1.0,-45.0,417.0,-3.5,'EPSG','9001',0.0,0.0,0.814,'EPSG','9104',-0.6,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'KMS-Grl',0);
INSERT INTO "usage" VALUES('EPSG','11911','helmert_transformation','EPSG','15900','EPSG','2571','EPSG','1026');
INSERT INTO "helmert_transformation" VALUES('EPSG','15901','IGN53 Mare to WGS 84 (3)','Parameter values taken from IGN53 Mare to RGNC91-93 (1) (code 15884) assuming that RGNC91-93 is equivalent to WGS 84 to within the accuracy of the transformation.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4641','EPSG','4326',2.0,287.58,177.78,-135.41,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'BGN-Ncl v3',0);
INSERT INTO "usage" VALUES('EPSG','11912','helmert_transformation','EPSG','15901','EPSG','2819','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','15902','IGN56 Lifou to WGS 84 (3)','Parameter values taken from IGN56 Lifou to RGNC91-93 (1) (code 15883) assuming that RGNC91-93 is equivalent to WGS 84 to within the accuracy of the transformation.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4633','EPSG','4326',1.0,335.47,222.58,-230.94,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'BGN-Ncl v3',0);
INSERT INTO "usage" VALUES('EPSG','11913','helmert_transformation','EPSG','15902','EPSG','2814','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','15903','IGN72 Grande Terre to WGS 84 (3)','Parameter values taken from IGN72 Grande Terre to RGNC91-93 (1) (code 15882) assuming that RGNC91-93 is equivalent to WGS 84 to within the accuracy of the transformation.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4662','EPSG','4326',2.0,-11.64,-348.6,291.98,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'BGN-Ncl v3',0);
INSERT INTO "usage" VALUES('EPSG','11914','helmert_transformation','EPSG','15903','EPSG','2822','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','15904','NEA74 Noumea to WGS 84 (2)','Parameter values taken from NEA74 Noumea to RGNC91-93 (1) (code 15886) assuming that RGNC91-93 is equivalent to WGS 84 to within the accuracy of the transformation.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4644','EPSG','4326',1.0,-10.18,-350.43,291.37,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'BGN-Ncl',0);
INSERT INTO "usage" VALUES('EPSG','11915','helmert_transformation','EPSG','15904','EPSG','2823','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','15908','LGD2006 to WGS 84 (1)','Derived at 5 stations throughout Libya used to define LGD2006 in May 2006.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4754','EPSG','4326',0.1,-208.4058,-109.8777,-2.5764,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'SDL-Lby',0);
INSERT INTO "usage" VALUES('EPSG','11919','helmert_transformation','EPSG','15908','EPSG','1143','EPSG','1031');
INSERT INTO "helmert_transformation" VALUES('EPSG','15909','ELD79 to WGS 84 (8)','Derived at 29 stations throughout Libya in May 2006.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4159','EPSG','4326',5.0,-115.8543,-99.0583,-152.4616,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'SDL-Lby',0);
INSERT INTO "usage" VALUES('EPSG','11920','helmert_transformation','EPSG','15909','EPSG','3271','EPSG','1045');
INSERT INTO "helmert_transformation" VALUES('EPSG','15910','ELD79 to LGD2006 (1)','Derived at 29 stations throughout Libya in May 2006.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4159','EPSG','4754',5.0,-92.5515,-10.8194,149.8852,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'SDL-Lby',1);
INSERT INTO "usage" VALUES('EPSG','11921','helmert_transformation','EPSG','15910','EPSG','3271','EPSG','1042');
INSERT INTO "helmert_transformation" VALUES('EPSG','15911','ID74 to DGN95 (1)','Derived at 38 stations. Standard deviations of translations are 1.3, 1.1 and 3.6m, of rotations 0.11, 0.06 and 0.04 sec and ppm 0.18. May be taken as a tfm ID74 to WGS 84 - see tfm 1833.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4238','EPSG','4755',3.0,-1.977,-13.06,-9.993,'EPSG','9001',-0.364,-0.254,-0.689,'EPSG','9104',-1.037,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'Bak-Idn',0);
INSERT INTO "usage" VALUES('EPSG','11922','helmert_transformation','EPSG','15911','EPSG','4020','EPSG','1043');
INSERT INTO "helmert_transformation" VALUES('EPSG','15912','DGN95 to WGS 84 (1)','Approximation at the +/- 1m level assuming that DGN95 is equivalent to WGS 84 within the accuracy of the transformation.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4755','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-Idn',0);
INSERT INTO "usage" VALUES('EPSG','11923','helmert_transformation','EPSG','15912','EPSG','1122','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','15913','NAD27 to WGS 84 (86)','Developed by John E Chance and Associates at 21°33''N, 92°33''W. Geoid height used =-16.7m.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4267','EPSG','4326',5.0,0.0,125.0,196.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'JECA-Mex GoM CamN',0);
INSERT INTO "usage" VALUES('EPSG','11924','helmert_transformation','EPSG','15913','EPSG','3461','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','15918','Beijing 1954 to WGS 84 (1)','Provided by BGP to TOTAL in June 2006.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4214','EPSG','4326',1.0,12.646,-155.176,-80.863,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'BGP-Chn Ord',0);
INSERT INTO "usage" VALUES('EPSG','11929','helmert_transformation','EPSG','15918','EPSG','3466','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','15919','Beijing 1954 to WGS 84 (2)','Derived via WGS 72BE. Original transformation derived in 1979 at 4 stations on Yellow Sea coast.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4214','EPSG','4326',15.0,15.53,-113.82,-41.38,'EPSG','9001',0.0,0.0,0.814,'EPSG','9104',-0.38,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'BP-Chn YS',0);
INSERT INTO "usage" VALUES('EPSG','11930','helmert_transformation','EPSG','15919','EPSG','3469','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','15920','Beijing 1954 to WGS 84 (3)','Derived via WGS 72BE. Original transformation derived by GSI in 1980-81. The GSI memo incorrectly gave the parameters as from WGS 72 to Beijing 1954, but it has been determined by the OGP that the memo should have stated from Beijing 1954 to WGS 72BE.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4214','EPSG','4326',15.0,31.4,-144.3,-74.8,'EPSG','9001',0.0,0.0,0.814,'EPSG','9104',-0.38,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'GSI-Chn SCS',0);
INSERT INTO "usage" VALUES('EPSG','11931','helmert_transformation','EPSG','15920','EPSG','3470','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','15921','Beijing 1954 to WGS 84 (4)','Provided by BGP to ELF in 1994.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4214','EPSG','4326',1.0,15.8,-154.4,-82.3,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'BGP-Chn Tarim',0);
INSERT INTO "usage" VALUES('EPSG','11932','helmert_transformation','EPSG','15921','EPSG','3507','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','15923','ELD79 to WGS 84 (9)','Derived by SDL for Total in Cyrenaica blocks 2 & 4.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4159','EPSG','4326',2.0,-117.7,-100.3,-152.4,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'TOT-Lby Cyr',0);
INSERT INTO "usage" VALUES('EPSG','11934','helmert_transformation','EPSG','15923','EPSG','3477','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','15924','ELD79 to LGD2006 (1)','Derived at 29 stations throughout Libya in May 2006.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4159','EPSG','4754',5.0,92.5515,10.8194,-149.8852,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'SDL-Lby',0);
INSERT INTO "usage" VALUES('EPSG','11935','helmert_transformation','EPSG','15924','EPSG','3271','EPSG','1045');
INSERT INTO "helmert_transformation" VALUES('EPSG','15925','JAD2001 to WGS 84 (1)','For many practical purposes JAD2001 can be considered to be coincident with WGS 84.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4758','EPSG','4326',0.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NLA-Jam',0);
INSERT INTO "usage" VALUES('EPSG','11936','helmert_transformation','EPSG','15925','EPSG','1128','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','15926','JAD69 to JAD2001 (1)','Accuracy 0.3 to 0.5 metres. May be used as tfm to WGS 84 - see JAD69 to WGS 84 (3) (tfm code 15927).','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4242','EPSG','4758',0.5,-33.722,153.789,94.959,'EPSG','9001',8.581,4.478,-4.54,'EPSG','9104',8.95,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NLA-Jam',0);
INSERT INTO "usage" VALUES('EPSG','11937','helmert_transformation','EPSG','15926','EPSG','3342','EPSG','1034');
INSERT INTO "helmert_transformation" VALUES('EPSG','15927','JAD69 to WGS 84 (3)','Derived at 4 stations, tested at a further 9. Also used as tfm to JAD69 to JAD2001 (see code 15926). Note: Info source paper contains an error in sign of dS, subsequently confirmed by primary author and NLA of Jamaica, and corrected in this record.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4242','EPSG','4326',1.0,-33.722,153.789,94.959,'EPSG','9001',8.581,4.478,-4.54,'EPSG','9104',8.95,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'UT-Jam 1m',0);
INSERT INTO "usage" VALUES('EPSG','11938','helmert_transformation','EPSG','15927','EPSG','3342','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','15928','BD72 to ETRS89 (2)','May be taken as approximate transformation BD72 to WGS 84 - see code 15929. Scale difference is given by information source as -1.0000012747. Given in this record in ppm to assist application usage.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4313','EPSG','4258',0.2,-106.8686,52.2978,-103.7239,'EPSG','9001',-0.3366,0.457,-1.8422,'EPSG','9104',-1.2747,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Bel 0.2m',0);
INSERT INTO "usage" VALUES('EPSG','11939','helmert_transformation','EPSG','15928','EPSG','1347','EPSG','1032');
INSERT INTO "helmert_transformation" VALUES('EPSG','15929','BD72 to WGS 84 (3)','Parameter values from BD72 to ETRS89 (2) (code 15928) assuming that ETRS89 is equivalent to WGS 84 within the accuracy of the tfm. Scale difference is given by information source as -1.0000012747; given in this record in ppm to assist application usage.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4313','EPSG','4326',1.0,-106.8686,52.2978,-103.7239,'EPSG','9001',-0.3366,0.457,-1.8422,'EPSG','9104',-1.2747,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'IGN-Bel 1m',0);
INSERT INTO "usage" VALUES('EPSG','11940','helmert_transformation','EPSG','15929','EPSG','1347','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','15930','NAD83(HARN) to NAD83(NSRS2007) (1)','Accuracy 0.1 to 0.2m in California, 0.05-0.11 in Oregon, elsewhere better than 0.05m.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4152','EPSG','4326',0.1,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-USA conus',1);
INSERT INTO "usage" VALUES('EPSG','11941','helmert_transformation','EPSG','15930','EPSG','1323','EPSG','1032');
INSERT INTO "helmert_transformation" VALUES('EPSG','15931','NAD83(NSRS2007) to WGS 84 (1)','Approximation assuming that NAD83(NSRS2007) is equivalent to WGS 84 within the accuracy of the transformation.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4759','EPSG','4326',2.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-USA conus AK',0);
INSERT INTO "usage" VALUES('EPSG','11942','helmert_transformation','EPSG','15931','EPSG','1511','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','15934','Amersfoort to WGS 84 (3)','Parameter values from Amersfoort to ETRS89 (3) (tfm code 15739) assuming that ETRS89 is equivalent to WGS 84 within the accuracy of the transformation. Replaces Amersfoort to WGS 84 (2) (code 1672). Replaced by Amersfoort to WGS 84 (4) (tfm code 4833).','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4289','EPSG','4326',1.0,565.2369,50.0087,465.658,'EPSG','9001',1.9725,-1.7004,9.0677,'EPSG','9109',4.0812,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-Nld 2004',0);
INSERT INTO "usage" VALUES('EPSG','11945','helmert_transformation','EPSG','15934','EPSG','1275','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','15935','Beijing 1954 to WGS 84 (5)','Concatenated via WGS 72BE. Recomputation by Shelltech in 1981 of SSB 1980 observation.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4214','EPSG','4326',10.0,18.0,-136.8,-73.7,'EPSG','9001',0.0,0.0,0.814,'EPSG','9104',-0.38,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'Shlt-Chn BeiBu',0);
INSERT INTO "usage" VALUES('EPSG','11946','helmert_transformation','EPSG','15935','EPSG','3561','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','15936','Beijing 1954 to WGS 84 (6)','Provided by Sinopec to TOTAL in January 2007.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4214','EPSG','4326',1.0,11.911,-154.833,-80.079,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'Sino-Chn Ord',0);
INSERT INTO "usage" VALUES('EPSG','11947','helmert_transformation','EPSG','15936','EPSG','3466','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','15937','Nahrwan 1967 to WGS 84 (7)','Parameter values adopted by Total are mean of those derived by Oceonics and Geoid through ties at station TC58 to 4 IGS stations in March 1995.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4270','EPSG','4326',2.0,-245.8,-152.2,382.9,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'TOT-UAE Abd',0);
INSERT INTO "usage" VALUES('EPSG','11948','helmert_transformation','EPSG','15937','EPSG','3509','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','15938','Nahrwan 1967 to WGS 84 (8)','Derived via WGS 72BE from Transit observations at station TC58 in 1976 by BP for ADMA.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4270','EPSG','4326',5.0,-225.4,-158.7,380.8,'EPSG','9001',0.0,0.0,0.814,'EPSG','9104',-0.38,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'ADMA-UAE Abd',0);
INSERT INTO "usage" VALUES('EPSG','11949','helmert_transformation','EPSG','15938','EPSG','3509','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','15952','Nahrwan 1967 to WGS 84 (9)','Used by DPC for Al Fateh field. Applying this transformation gives same result as Nahrwan 1967 to WGS 84 (8) (code 15938).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4270','EPSG','4326',5.0,-244.2,-149.8,379.3,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DPC-UAE Fat',0);
INSERT INTO "usage" VALUES('EPSG','11963','helmert_transformation','EPSG','15952','EPSG','3530','EPSG','1136');
INSERT INTO "helmert_transformation" VALUES('EPSG','15953','Nahrwan 1967 to WGS 84 (10)','Used by Dubai Municipality before 1994.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4270','EPSG','4326',5.0,-250.7,-157.9,380.4,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'Dub-UAE Dub',0);
INSERT INTO "usage" VALUES('EPSG','11964','helmert_transformation','EPSG','15953','EPSG','3531','EPSG','1248');
INSERT INTO "helmert_transformation" VALUES('EPSG','15957','Qoornoq 1927 to GR96 (1)','Derived via NWL 9D.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4194','EPSG','4747',10.0,163.511,127.533,-159.789,'EPSG','9001',0.0,0.0,0.814,'EPSG','9104',-0.6,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'KMS-Grl',0);
INSERT INTO "usage" VALUES('EPSG','11968','helmert_transformation','EPSG','15957','EPSG','3362','EPSG','1026');
INSERT INTO "helmert_transformation" VALUES('EPSG','15963','Yacare to SIRGAS (1)','Derived at 11 stations during 1998 densification of Uruguay control based on SIRAGAS 1995.','EPSG','9606','Position Vector transformation (geog2D domain)','EPSG','4309','EPSG','4170',1.5,-124.45,183.74,44.64,'EPSG','9001',-0.4384,0.5446,-0.9706,'EPSG','9104',-2.1365,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'SGM-Ury',1);
INSERT INTO "usage" VALUES('EPSG','11974','helmert_transformation','EPSG','15963','EPSG','1247','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','15964','ED50 to WGS 84 (42)','Developed by the Portuguese Hydrographic Institute and used by the Directorate of Energy and Geology.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4230','EPSG','4326',5.0,-86.277,-108.879,-120.181,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'DGEG-Por off',0);
INSERT INTO "usage" VALUES('EPSG','11975','helmert_transformation','EPSG','15964','EPSG','3537','EPSG','1187');
INSERT INTO "helmert_transformation" VALUES('EPSG','15965','S-JTSK to WGS 84 (3)','Derived at 6 stations.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4156','EPSG','4326',6.0,589.0,76.0,480.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NIMA-Cze',0);
INSERT INTO "usage" VALUES('EPSG','11976','helmert_transformation','EPSG','15965','EPSG','1306','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','15966','HTRS96 to ETRS89 (1)','May be taken as approximate transformation HTRS96 to WGS 84 - see code 15967.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4761','EPSG','4258',0.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-Hrv',0);
INSERT INTO "usage" VALUES('EPSG','11977','helmert_transformation','EPSG','15966','EPSG','1076','EPSG','1161');
INSERT INTO "helmert_transformation" VALUES('EPSG','15967','HTRS96 to WGS 84 (1)','Approximation at the +/- 1m level assuming that ETRS89 is equivalent to WGS 84. HTRS96 is a regional realization of ETRS89.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4761','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-Hrv',0);
INSERT INTO "usage" VALUES('EPSG','11978','helmert_transformation','EPSG','15967','EPSG','1076','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','15969','Bermuda 1957 to BDA2000 (1)','Derived in 1998 at 12 stations. Used for transformation of 1:2500 mapping. May be taken as approximate transformation Bermuda 1957 to WGS 84 - see code 15970.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4216','EPSG','4762',1.0,-292.295,248.758,429.447,'EPSG','9001',-4.9971,-2.99,-6.6906,'EPSG','9104',1.0289,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'LBS-Bmu',0);
INSERT INTO "usage" VALUES('EPSG','11979','helmert_transformation','EPSG','15969','EPSG','3221','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','15970','Bermuda 1957 to WGS 84 (2)','Parameter values from Bermuda 1957 to BDA2000 (1) (code 15969). Assumes BDA2000 and WGS 84 can be considered the same to within the accuracy of the transformation.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4216','EPSG','4326',1.0,-292.295,248.758,429.447,'EPSG','9001',-4.9971,-2.99,-6.6906,'EPSG','9104',1.0289,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-Bmu',0);
INSERT INTO "usage" VALUES('EPSG','11980','helmert_transformation','EPSG','15970','EPSG','3221','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','15971','BDA2000 to WGS 84 (1)','Approximation at the +/- 1m level assuming that BDA2000 is equivalent to WGS 84.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4762','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-Bmu',0);
INSERT INTO "usage" VALUES('EPSG','11981','helmert_transformation','EPSG','15971','EPSG','1047','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','15972','Pitcairn 2006 to WGS 84 (1)','Approximation at the +/- 1m level assuming that Pitcairn 2006 is equivalent to WGS 84.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4763','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-Pcn',0);
INSERT INTO "usage" VALUES('EPSG','11982','helmert_transformation','EPSG','15972','EPSG','3208','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','15973','Popular Visualisation CRS to WGS 84 (1)','Executes change of sphere/ellipsoid','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4055','EPSG','4326',800.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-web',1);
INSERT INTO "usage" VALUES('EPSG','11983','helmert_transformation','EPSG','15973','EPSG','1262','EPSG','1098');
INSERT INTO "helmert_transformation" VALUES('EPSG','15974','RSRGD2000 to WGS 84 (1)','Approximation at the +/- 1m level assuming that RSRGD2000 is equivalent to WGS 84.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4764','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-Ata',0);
INSERT INTO "usage" VALUES('EPSG','11984','helmert_transformation','EPSG','15974','EPSG','3558','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','15975','NZGD49 to WGS 84 (4)','These parameter values are taken from NZGD49 to NZGD2000 (1) (code 1566) and assume that NZGD2000 and WGS 84 are coincident to within the accuracy of the tfm. For better accuracy use NZGD49 to WGS 84 (2) (code 1564) or NZGD49 to WGS 84 (3) (code 1670).','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4272','EPSG','4326',5.0,54.4,-20.1,183.1,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OSG-Nzl 5m',0);
INSERT INTO "usage" VALUES('EPSG','11985','helmert_transformation','EPSG','15975','EPSG','3285','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','15976','Slovenia 1996 to WGS 84 (1)','Approximation at the +/- 1m level assuming that ETRS89 is equivalent to WGS 84. Slovenia 1996 is a national realization of ETRS89.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4765','EPSG','4326',1.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-Svn',0);
INSERT INTO "usage" VALUES('EPSG','11986','helmert_transformation','EPSG','15976','EPSG','1212','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','15977','Slovenia 1996 to ETRS89 (1)','Slovenia 1996 is a local densification of ETRS89.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4765','EPSG','4258',0.0,0.0,0.0,0.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-Svn',0);
INSERT INTO "usage" VALUES('EPSG','11987','helmert_transformation','EPSG','15977','EPSG','1212','EPSG','1161');
INSERT INTO "helmert_transformation" VALUES('EPSG','15978','NAD27 to WGS 84 (88)','','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4267','EPSG','4326',1.0,2.478,149.752,197.726,'EPSG','9001',-0.526,-0.498,0.501,'EPSG','9104',0.685,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'ONHG-Cub',0);
INSERT INTO "usage" VALUES('EPSG','11988','helmert_transformation','EPSG','15978','EPSG','1077','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','15979','AGD66 to GDA94 (12)','Use only offshore: onshore, transformations 1458 (ACT), 1594 (Tas), 1460 (NSW and Vic) and 1595 (NT) are more accurate.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4202','EPSG','4283',3.0,-117.808,-51.536,137.784,'EPSG','9001',-0.303,-0.446,-0.234,'EPSG','9104',-0.29,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'ICSM-Aus off',0);
INSERT INTO "usage" VALUES('EPSG','14200','helmert_transformation','EPSG','15979','EPSG','3559','EPSG','1043');
INSERT INTO "helmert_transformation" VALUES('EPSG','15980','AGD66 to WGS 84 (18)','Parameter values from AGD66 to GDA94 (12) (code 15979). Assumes WGS 84 is equivalent to GDA94; ignores low accuracy of ensemble and tectonic plate motion. Use only offshore: onshore CTs 1665-68 for ACT, NSW/Vic, Tas and NT respectively are more accurate.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4202','EPSG','4326',3.0,-117.808,-51.536,137.784,'EPSG','9001',-0.303,-0.446,-0.234,'EPSG','9104',-0.29,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'EPSG-Aus off',0);
INSERT INTO "usage" VALUES('EPSG','11990','helmert_transformation','EPSG','15980','EPSG','3559','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','15981','MGI to Slovenia 1996 (1)','Info source also gives a separate reverse tfm with slightly different parameter values. Given the tfm accuracy these differences are not significant and this tfm can be considered reversible. May be taken as approximate tfm MGI to WGS 84 (see code 15982)','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4312','EPSG','4765',1.0,409.545,72.164,486.872,'EPSG','9001',-3.085957,-5.46911,11.020289,'EPSG','9104',17.919665,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'GuRS-Svn',1);
INSERT INTO "usage" VALUES('EPSG','11991','helmert_transformation','EPSG','15981','EPSG','1212','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','15982','MGI to WGS 84 (9)','Parameter values from MGI to Slovenia 1996 (1) (code 15981). Assumes Slovenia 1996 and WGS 84 can be considered the same to within the accuracy of the transformation.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4312','EPSG','4326',1.0,409.545,72.164,486.872,'EPSG','9001',-3.085957,-5.46911,11.020289,'EPSG','9104',17.919665,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-Svn 96',1);
INSERT INTO "usage" VALUES('EPSG','11992','helmert_transformation','EPSG','15982','EPSG','1212','EPSG','1041');
INSERT INTO "helmert_transformation" VALUES('EPSG','15983','MGI to Slovenia 1996 (2)','Info source also gives a separate reverse tfm with slightly different parameter values. Given the tfm accuracy these differences are not significant and this tfm can be considered reversible.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4312','EPSG','4765',0.5,315.393,186.223,499.609,'EPSG','9001',-6.445954,-8.131631,13.208641,'EPSG','9104',23.449046,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'GuRS-Svn W',1);
INSERT INTO "usage" VALUES('EPSG','11993','helmert_transformation','EPSG','15983','EPSG','3564','EPSG','1035');
INSERT INTO "helmert_transformation" VALUES('EPSG','15984','MGI to Slovenia 1996 (3)','Info source also gives a separate reverse tfm with slightly different parameter values. Given the tfm accuracy these differences are not significant and this tfm can be considered reversible.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4312','EPSG','4765',0.5,464.939,-21.478,504.497,'EPSG','9001',0.403,-4.228747,9.954942,'EPSG','9104',12.795378,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'GuRS-Svn NE',1);
INSERT INTO "usage" VALUES('EPSG','11994','helmert_transformation','EPSG','15984','EPSG','3565','EPSG','1035');
INSERT INTO "helmert_transformation" VALUES('EPSG','15985','MGI to Slovenia 1996 (4)','Info source also gives a separate reverse tfm with slightly different parameter values. Given the tfm accuracy these differences are not significant and this tfm can be considered reversible.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4312','EPSG','4765',0.5,459.968,82.193,458.756,'EPSG','9001',-3.565234,-3.700593,10.860523,'EPSG','9104',15.507563,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'GuRS-Svn SE 0.5m',1);
INSERT INTO "usage" VALUES('EPSG','11995','helmert_transformation','EPSG','15985','EPSG','3566','EPSG','1035');
INSERT INTO "helmert_transformation" VALUES('EPSG','15986','MGI to Slovenia 1996 (5)','Info source also gives a separate reverse tfm with slightly different parameter values. Given the tfm accuracy these differences are not significant and this tfm can be considered reversible.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4312','EPSG','4765',0.3,427.914,105.528,510.908,'EPSG','9001',-4.992523,-5.898813,10.306673,'EPSG','9104',12.431493,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'GuRS-Svn SE 0.3m',1);
INSERT INTO "usage" VALUES('EPSG','11996','helmert_transformation','EPSG','15986','EPSG','3567','EPSG','1033');
INSERT INTO "helmert_transformation" VALUES('EPSG','15987','MGI to Slovenia 1996 (6)','Info source also gives a separate reverse tfm with slightly different parameter values. Given the tfm accuracy these differences are not significant and this tfm can be considered reversible.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4312','EPSG','4765',0.3,468.63,81.389,445.221,'EPSG','9001',-3.839242,-3.262525,10.566866,'EPSG','9104',16.132726,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'GuRS-Svn Dol',1);
INSERT INTO "usage" VALUES('EPSG','11997','helmert_transformation','EPSG','15987','EPSG','3568','EPSG','1033');
INSERT INTO "helmert_transformation" VALUES('EPSG','15988','MGI to Slovenia 1996 (7)','Info source also gives a separate reverse tfm with slightly different parameter values. Given the tfm accuracy these differences are not significant and this tfm can be considered reversible.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4312','EPSG','4765',0.3,439.5,-11.77,494.976,'EPSG','9001',-0.026585,-4.65641,10.155824,'EPSG','9104',16.270002,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'GuRS-Svn Staj',1);
INSERT INTO "usage" VALUES('EPSG','11998','helmert_transformation','EPSG','15988','EPSG','3569','EPSG','1033');
INSERT INTO "helmert_transformation" VALUES('EPSG','15989','MGI to Slovenia 1996 (8)','Info source also gives a separate reverse tfm with slightly different parameter values. Given the tfm accuracy these differences are not significant and this tfm can be considered reversible.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4312','EPSG','4765',0.3,524.442,3.275,519.002,'EPSG','9001',0.013287,-3.119714,10.232693,'EPSG','9104',4.184981,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'GuRS-Svn Pom',1);
INSERT INTO "usage" VALUES('EPSG','11999','helmert_transformation','EPSG','15989','EPSG','3570','EPSG','1033');
INSERT INTO "helmert_transformation" VALUES('EPSG','15990','MGI to Slovenia 1996 (9)','Info source also gives a separate reverse tfm with slightly different parameter values. Given the tfm accuracy these differences are not significant and this tfm can be considered reversible.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4312','EPSG','4765',0.3,281.529,45.963,537.515,'EPSG','9001',-2.570437,-9.648271,10.759507,'EPSG','9104',26.465548,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'GuRS-Svn Gor',1);
INSERT INTO "usage" VALUES('EPSG','12000','helmert_transformation','EPSG','15990','EPSG','3571','EPSG','1033');
INSERT INTO "helmert_transformation" VALUES('EPSG','15991','MGI to Slovenia 1996 (10)','Info source also gives a separate reverse tfm with slightly different parameter values. Given the tfm accuracy these differences are not significant and this tfm can be considered reversible.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4312','EPSG','4765',0.3,355.845,274.282,462.979,'EPSG','9001',-9.086933,-6.491055,14.502181,'EPSG','9104',20.888647,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'GuRS-Svn Prim',1);
INSERT INTO "usage" VALUES('EPSG','12001','helmert_transformation','EPSG','15991','EPSG','3572','EPSG','1033');
INSERT INTO "helmert_transformation" VALUES('EPSG','15992','MGI to Slovenia 1996 (11)','Info source also gives a separate reverse tfm with slightly different parameter values. Given the tfm accuracy these differences are not significant and this tfm can be considered reversible.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4312','EPSG','4765',0.3,400.629,90.651,472.249,'EPSG','9001',-3.261138,-5.263404,11.83739,'EPSG','9104',20.022676,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'GuRS-Svn cen',1);
INSERT INTO "usage" VALUES('EPSG','12002','helmert_transformation','EPSG','15992','EPSG','3573','EPSG','1033');
INSERT INTO "helmert_transformation" VALUES('EPSG','15993','Pulkovo 1942(58) to ETRS89 (3)','Withdrawn and replaced by S-42 to ETRS89 (4) (tfm code 15994). Consistent with Transdat v2.0 to better than 10m.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4179','EPSG','4258',10.0,68.1564,32.7756,80.2249,'EPSG','9001',2.20333014,2.19256447,-2.54166911,'EPSG','9104',-0.14155333,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'ANCPI-Rom 2007',0);
INSERT INTO "usage" VALUES('EPSG','12003','helmert_transformation','EPSG','15993','EPSG','1197','EPSG','1027');
INSERT INTO "helmert_transformation" VALUES('EPSG','15994','Pulkovo 1942(58) to ETRS89 (4)','Replaces S-42 to ETRS89 (3) (tfm code 15993). Consistent with Transdat v3.0 to better than 0.5m. May be taken as approximate transformation Pulkovo 1942(58) to WGS 84 - see code 15995.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4179','EPSG','4258',3.0,2.3287,-147.0425,-92.0802,'EPSG','9001',0.3092483,-0.32482185,-0.49729934,'EPSG','9104',5.68906266,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'ANCPI-Rom 2008',0);
INSERT INTO "usage" VALUES('EPSG','12004','helmert_transformation','EPSG','15994','EPSG','1197','EPSG','1043');
INSERT INTO "helmert_transformation" VALUES('EPSG','15995','Pulkovo 1942(58) to WGS 84 (19)','Parameter values taken from Pulkovo 1942(58) to ETRS89 (4) (code 15994) assuming that ETRS89 is equivalent to WGS 84 within the accuracy of the transformation.','EPSG','9607','Coordinate Frame rotation (geog2D domain)','EPSG','4179','EPSG','4326',3.0,2.329,-147.042,-92.08,'EPSG','9001',0.309,-0.325,-0.497,'EPSG','9104',5.69,'EPSG','9202',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'OGP-Rom',0);
INSERT INTO "usage" VALUES('EPSG','12005','helmert_transformation','EPSG','15995','EPSG','1197','EPSG','1252');
INSERT INTO "helmert_transformation" VALUES('EPSG','15996','Pulkovo 1942(83) to WGS 84 (3)','Derived at 5 stations.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4178','EPSG','4326',4.0,28.0,-121.0,-77.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NIMA-Hun',0);
INSERT INTO "usage" VALUES('EPSG','12006','helmert_transformation','EPSG','15996','EPSG','1119','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','15997','Pulkovo 1942(58) to WGS 84 (4)','Derived at 11 stations.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4179','EPSG','4326',6.0,23.0,-124.0,-82.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NIMA-Pol',0);
INSERT INTO "usage" VALUES('EPSG','12007','helmert_transformation','EPSG','15997','EPSG','3293','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','15998','Pulkovo 1942(83) to WGS 84 (5)','Derived at 6 stations.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4178','EPSG','4326',5.0,26.0,-121.0,-78.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NIMA-Cze',0);
INSERT INTO "usage" VALUES('EPSG','12008','helmert_transformation','EPSG','15998','EPSG','1306','EPSG','1160');
INSERT INTO "helmert_transformation" VALUES('EPSG','15999','Pulkovo 1942(58) to WGS 84 (8)','Derived at 7 stations.','EPSG','9603','Geocentric translations (geog2D domain)','EPSG','4179','EPSG','4326',6.0,24.0,-130.0,-92.0,'EPSG','9001',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,'NIMA-Alb',0);
INSERT INTO "usage" VALUES('EPSG','12009','helmert_transformation','EPSG','15999','EPSG','3212','EPSG','1160');
