--- This file has been generated by scripts/build_db.py. DO NOT EDIT !

INSERT INTO "geodetic_crs" VALUES('EPSG','3819','HD1909','Replaced earlier HD1863 adjustment also on Bessel ellipsoid. Both HD1863 and HD1909 were originally on Ferro Prime Meridian but subsequently converted to Greenwich. Replaced by HD72 (CRS code 4237).','geographic 2D','EPSG','6422','EPSG','1024',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2825','geodetic_crs','EPSG','3819','EPSG','1119','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','3821','TWD67','Shares the same origin point with the earlier Hu Tzu Shan system (CRS code 4236) but away from this point coordinates differ. Do not confuse!','geographic 2D','EPSG','6422','EPSG','1025',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2826','geodetic_crs','EPSG','3821','EPSG','3315','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','3822','TWD97','','geocentric','EPSG','6500','EPSG','1026',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2827','geodetic_crs','EPSG','3822','EPSG','1228','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','3823','TWD97','','geographic 3D','EPSG','6423','EPSG','1026',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2828','geodetic_crs','EPSG','3823','EPSG','1228','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','3824','TWD97','','geographic 2D','EPSG','6422','EPSG','1026',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2829','geodetic_crs','EPSG','3824','EPSG','1228','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','3887','IGRS','','geocentric','EPSG','6500','EPSG','1029',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2873','geodetic_crs','EPSG','3887','EPSG','1124','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','3888','IGRS','','geographic 3D','EPSG','6423','EPSG','1029',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2874','geodetic_crs','EPSG','3888','EPSG','1124','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','3889','IGRS','','geographic 2D','EPSG','6422','EPSG','1029',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2875','geodetic_crs','EPSG','3889','EPSG','1124','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','3906','MGI 1901','Adopted in 1924 replacing MGI (Ferro) (CRS code 4805). Densified in 1948. In Slovenia replaced by D96 (CRS code 4765). In Croatia replaced by HTRS96 (CRS code 4761). In Serbia replaced by SREF98 and then by SRB_ETRS89 (STRS00) (CRS codes 4075 and 8691).','geographic 2D','EPSG','6422','EPSG','1031',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2884','geodetic_crs','EPSG','3906','EPSG','2370','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4000','MOLDREF99','','geocentric','EPSG','6500','EPSG','1032',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2922','geodetic_crs','EPSG','4000','EPSG','1162','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4001','Unknown datum based upon the Airy 1830 ellipsoid','Use only in cases where geodetic datum is unknown.','geographic 2D','EPSG','6422','EPSG','6001',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2923','geodetic_crs','EPSG','4001','EPSG','1263','EPSG','1214');
INSERT INTO "geodetic_crs" VALUES('EPSG','4002','Unknown datum based upon the Airy Modified 1849 ellipsoid','Use only in cases where geodetic datum is unknown.','geographic 2D','EPSG','6422','EPSG','6002',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2924','geodetic_crs','EPSG','4002','EPSG','1263','EPSG','1214');
INSERT INTO "geodetic_crs" VALUES('EPSG','4003','Unknown datum based upon the Australian National Spheroid','Use only in cases where geodetic datum is unknown.','geographic 2D','EPSG','6422','EPSG','6003',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2925','geodetic_crs','EPSG','4003','EPSG','1263','EPSG','1214');
INSERT INTO "geodetic_crs" VALUES('EPSG','4004','Unknown datum based upon the Bessel 1841 ellipsoid','Use only in cases where geodetic datum is unknown.','geographic 2D','EPSG','6422','EPSG','6004',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2926','geodetic_crs','EPSG','4004','EPSG','1263','EPSG','1214');
INSERT INTO "geodetic_crs" VALUES('EPSG','4005','Unknown datum based upon the Bessel Modified ellipsoid','Use only in cases where geodetic datum is unknown.','geographic 2D','EPSG','6422','EPSG','6005',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2927','geodetic_crs','EPSG','4005','EPSG','1263','EPSG','1214');
INSERT INTO "geodetic_crs" VALUES('EPSG','4006','Unknown datum based upon the Bessel Namibia ellipsoid','Use only in cases where geodetic datum is unknown.','geographic 2D','EPSG','6422','EPSG','6006',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2928','geodetic_crs','EPSG','4006','EPSG','1263','EPSG','1214');
INSERT INTO "geodetic_crs" VALUES('EPSG','4007','Unknown datum based upon the Clarke 1858 ellipsoid','Use only in cases where geodetic datum is unknown.','geographic 2D','EPSG','6422','EPSG','6007',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2929','geodetic_crs','EPSG','4007','EPSG','1263','EPSG','1214');
INSERT INTO "geodetic_crs" VALUES('EPSG','4008','Unknown datum based upon the Clarke 1866 ellipsoid','Use only in cases where geodetic datum is unknown.','geographic 2D','EPSG','6422','EPSG','6008',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2930','geodetic_crs','EPSG','4008','EPSG','1263','EPSG','1214');
INSERT INTO "geodetic_crs" VALUES('EPSG','4009','Unknown datum based upon the Clarke 1866 Michigan ellipsoid','Use only in cases where geodetic datum is unknown.','geographic 2D','EPSG','6422','EPSG','6009',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2931','geodetic_crs','EPSG','4009','EPSG','1263','EPSG','1214');
INSERT INTO "geodetic_crs" VALUES('EPSG','4010','Unknown datum based upon the Clarke 1880 (Benoit) ellipsoid','Use only in cases where geodetic datum is unknown.','geographic 2D','EPSG','6422','EPSG','6010',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2932','geodetic_crs','EPSG','4010','EPSG','1263','EPSG','1214');
INSERT INTO "geodetic_crs" VALUES('EPSG','4011','Unknown datum based upon the Clarke 1880 (IGN) ellipsoid','Use only in cases where geodetic datum is unknown.','geographic 2D','EPSG','6422','EPSG','6011',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2933','geodetic_crs','EPSG','4011','EPSG','1263','EPSG','1214');
INSERT INTO "geodetic_crs" VALUES('EPSG','4012','Unknown datum based upon the Clarke 1880 (RGS) ellipsoid','Use only in cases where geodetic datum is unknown.','geographic 2D','EPSG','6422','EPSG','6012',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2934','geodetic_crs','EPSG','4012','EPSG','1263','EPSG','1214');
INSERT INTO "geodetic_crs" VALUES('EPSG','4013','Unknown datum based upon the Clarke 1880 (Arc) ellipsoid','Use only in cases where geodetic datum is unknown.','geographic 2D','EPSG','6422','EPSG','6013',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2935','geodetic_crs','EPSG','4013','EPSG','1263','EPSG','1214');
INSERT INTO "geodetic_crs" VALUES('EPSG','4014','Unknown datum based upon the Clarke 1880 (SGA 1922) ellipsoid','Use only in cases where geodetic datum is unknown.','geographic 2D','EPSG','6422','EPSG','6014',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2936','geodetic_crs','EPSG','4014','EPSG','1263','EPSG','1214');
INSERT INTO "geodetic_crs" VALUES('EPSG','4015','Unknown datum based upon the Everest 1830 (1937 Adjustment) ellipsoid','Use only in cases where geodetic datum is unknown.','geographic 2D','EPSG','6422','EPSG','6015',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2937','geodetic_crs','EPSG','4015','EPSG','1263','EPSG','1214');
INSERT INTO "geodetic_crs" VALUES('EPSG','4016','Unknown datum based upon the Everest 1830 (1967 Definition) ellipsoid','Use only in cases where geodetic datum is unknown.','geographic 2D','EPSG','6422','EPSG','6016',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2938','geodetic_crs','EPSG','4016','EPSG','1263','EPSG','1214');
INSERT INTO "geodetic_crs" VALUES('EPSG','4017','MOLDREF99','','geographic 3D','EPSG','6423','EPSG','1032',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2939','geodetic_crs','EPSG','4017','EPSG','1162','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4018','Unknown datum based upon the Everest 1830 Modified ellipsoid','Use only in cases where geodetic datum is unknown.','geographic 2D','EPSG','6422','EPSG','6018',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2940','geodetic_crs','EPSG','4018','EPSG','1263','EPSG','1214');
INSERT INTO "geodetic_crs" VALUES('EPSG','4019','Unknown datum based upon the GRS 1980 ellipsoid','Use only in cases where geodetic datum is unknown.','geographic 2D','EPSG','6422','EPSG','6019',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2941','geodetic_crs','EPSG','4019','EPSG','1263','EPSG','1214');
INSERT INTO "geodetic_crs" VALUES('EPSG','4020','Unknown datum based upon the Helmert 1906 ellipsoid','Use only in cases where geodetic datum is unknown.','geographic 2D','EPSG','6422','EPSG','6020',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2942','geodetic_crs','EPSG','4020','EPSG','1263','EPSG','1214');
INSERT INTO "geodetic_crs" VALUES('EPSG','4021','Unknown datum based upon the Indonesian National Spheroid','Use only in cases where geodetic datum is unknown.','geographic 2D','EPSG','6422','EPSG','6021',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2943','geodetic_crs','EPSG','4021','EPSG','1263','EPSG','1214');
INSERT INTO "geodetic_crs" VALUES('EPSG','4022','Unknown datum based upon the International 1924 ellipsoid','Use only in cases where geodetic datum is unknown.','geographic 2D','EPSG','6422','EPSG','6022',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2944','geodetic_crs','EPSG','4022','EPSG','1263','EPSG','1214');
INSERT INTO "geodetic_crs" VALUES('EPSG','4023','MOLDREF99','','geographic 2D','EPSG','6422','EPSG','1032',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2945','geodetic_crs','EPSG','4023','EPSG','1162','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','4024','Unknown datum based upon the Krassowsky 1940 ellipsoid','Use only in cases where geodetic datum is unknown.','geographic 2D','EPSG','6422','EPSG','6024',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2946','geodetic_crs','EPSG','4024','EPSG','1263','EPSG','1214');
INSERT INTO "geodetic_crs" VALUES('EPSG','4025','Unknown datum based upon the NWL 9D ellipsoid','Use only in cases where geodetic datum is unknown.','geographic 2D','EPSG','6422','EPSG','6025',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2947','geodetic_crs','EPSG','4025','EPSG','1263','EPSG','1214');
INSERT INTO "geodetic_crs" VALUES('EPSG','4027','Unknown datum based upon the Plessis 1817 ellipsoid','Use only in cases where geodetic datum is unknown.','geographic 2D','EPSG','6422','EPSG','6027',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2949','geodetic_crs','EPSG','4027','EPSG','1263','EPSG','1214');
INSERT INTO "geodetic_crs" VALUES('EPSG','4028','Unknown datum based upon the Struve 1860 ellipsoid','Use only in cases where geodetic datum is unknown.','geographic 2D','EPSG','6422','EPSG','6028',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2950','geodetic_crs','EPSG','4028','EPSG','1263','EPSG','1214');
INSERT INTO "geodetic_crs" VALUES('EPSG','4029','Unknown datum based upon the War Office ellipsoid','Use only in cases where geodetic datum is unknown.','geographic 2D','EPSG','6422','EPSG','6029',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2951','geodetic_crs','EPSG','4029','EPSG','1263','EPSG','1214');
INSERT INTO "geodetic_crs" VALUES('EPSG','4030','Unknown datum based upon the WGS 84 ellipsoid','Use only in cases where geodetic datum is unknown.','geographic 2D','EPSG','6422','EPSG','6030',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2952','geodetic_crs','EPSG','4030','EPSG','1263','EPSG','1214');
INSERT INTO "geodetic_crs" VALUES('EPSG','4031','Unknown datum based upon the GEM 10C ellipsoid','Use only in cases where geodetic datum is unknown.','geographic 2D','EPSG','6422','EPSG','6031',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2953','geodetic_crs','EPSG','4031','EPSG','1263','EPSG','1214');
INSERT INTO "geodetic_crs" VALUES('EPSG','4032','Unknown datum based upon the OSU86F ellipsoid','Use only in cases where geodetic datum is unknown.','geographic 2D','EPSG','6422','EPSG','6032',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2954','geodetic_crs','EPSG','4032','EPSG','1263','EPSG','1214');
INSERT INTO "geodetic_crs" VALUES('EPSG','4033','Unknown datum based upon the OSU91A ellipsoid','Use only in cases where geodetic datum is unknown.','geographic 2D','EPSG','6422','EPSG','6033',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2955','geodetic_crs','EPSG','4033','EPSG','1263','EPSG','1214');
INSERT INTO "geodetic_crs" VALUES('EPSG','4034','Unknown datum based upon the Clarke 1880 ellipsoid','Use only in cases where geodetic datum is unknown.','geographic 2D','EPSG','6422','EPSG','6034',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2956','geodetic_crs','EPSG','4034','EPSG','1263','EPSG','1214');
INSERT INTO "geodetic_crs" VALUES('EPSG','4035','Unknown datum based upon the Authalic Sphere','Deprecated. Use code 4047.','geographic 2D','EPSG','6402','EPSG','6035',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2957','geodetic_crs','EPSG','4035','EPSG','1263','EPSG','1214');
INSERT INTO "geodetic_crs" VALUES('EPSG','4036','Unknown datum based upon the GRS 1967 ellipsoid','Use only in cases where geodetic datum is unknown.','geographic 2D','EPSG','6422','EPSG','6036',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2958','geodetic_crs','EPSG','4036','EPSG','1263','EPSG','1214');
INSERT INTO "geodetic_crs" VALUES('EPSG','4039','RGRDC 2005','','geocentric','EPSG','6500','EPSG','1033',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2961','geodetic_crs','EPSG','4039','EPSG','3613','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4040','RGRDC 2005','','geographic 3D','EPSG','6423','EPSG','1033',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2962','geodetic_crs','EPSG','4040','EPSG','3613','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4041','Unknown datum based upon the Average Terrestrial System 1977 ellipsoid','Use only in cases where geodetic datum is unknown.','geographic 2D','EPSG','6422','EPSG','6041',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2963','geodetic_crs','EPSG','4041','EPSG','1263','EPSG','1214');
INSERT INTO "geodetic_crs" VALUES('EPSG','4042','Unknown datum based upon the Everest (1830 Definition) ellipsoid','Use only in cases where geodetic datum is unknown.','geographic 2D','EPSG','6422','EPSG','6042',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2964','geodetic_crs','EPSG','4042','EPSG','1263','EPSG','1214');
INSERT INTO "geodetic_crs" VALUES('EPSG','4043','Unknown datum based upon the WGS 72 ellipsoid','Use only in cases where geodetic datum is unknown.','geographic 2D','EPSG','6422','EPSG','6043',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2965','geodetic_crs','EPSG','4043','EPSG','1263','EPSG','1214');
INSERT INTO "geodetic_crs" VALUES('EPSG','4044','Unknown datum based upon the Everest 1830 (1962 Definition) ellipsoid','Use only in cases where geodetic datum is unknown.','geographic 2D','EPSG','6422','EPSG','6044',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2966','geodetic_crs','EPSG','4044','EPSG','1263','EPSG','1214');
INSERT INTO "geodetic_crs" VALUES('EPSG','4045','Unknown datum based upon the Everest 1830 (1975 Definition) ellipsoid','Use only in cases where geodetic datum is unknown.','geographic 2D','EPSG','6422','EPSG','6045',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2967','geodetic_crs','EPSG','4045','EPSG','1263','EPSG','1214');
INSERT INTO "geodetic_crs" VALUES('EPSG','4046','RGRDC 2005','','geographic 2D','EPSG','6422','EPSG','1033',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2968','geodetic_crs','EPSG','4046','EPSG','3613','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','4047','Unspecified datum based upon the GRS 1980 Authalic Sphere','Use only in cases where geodetic datum is unknown.','geographic 2D','EPSG','6422','EPSG','6047',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2969','geodetic_crs','EPSG','4047','EPSG','1263','EPSG','1162');
INSERT INTO "geodetic_crs" VALUES('EPSG','4052','Unspecified datum based upon the Clarke 1866 Authalic Sphere','Use only in cases where geodetic datum is unknown.','geographic 2D','EPSG','6422','EPSG','6052',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2974','geodetic_crs','EPSG','4052','EPSG','1263','EPSG','1162');
INSERT INTO "geodetic_crs" VALUES('EPSG','4053','Unspecified datum based upon the International 1924 Authalic Sphere','Use only in cases where geodetic datum is unknown.','geographic 2D','EPSG','6422','EPSG','6053',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2975','geodetic_crs','EPSG','4053','EPSG','1263','EPSG','1162');
INSERT INTO "geodetic_crs" VALUES('EPSG','4054','Unspecified datum based upon the Hughes 1980 ellipsoid','DMSP SSM/I data sets provided by NSIDC for polar research.','geographic 2D','EPSG','6422','EPSG','6054',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2976','geodetic_crs','EPSG','4054','EPSG','1263','EPSG','1110');
INSERT INTO "geodetic_crs" VALUES('EPSG','4055','Popular Visualisation CRS','Some applications erroneously call this WGS 84. It uses a sphere with a radius having the same value as the semi-major axis of the WGS 84 ellipsoid. There is no geodetic recognition of this system.','geographic 2D','EPSG','6422','EPSG','6055',NULL,1);
INSERT INTO "usage" VALUES('EPSG','2977','geodetic_crs','EPSG','4055','EPSG','1262','EPSG','1098');
INSERT INTO "geodetic_crs" VALUES('EPSG','4073','SREF98','Replaced by SRB_ETRS89 (STRS00) (CRS code 8683).','geocentric','EPSG','6500','EPSG','1034',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2987','geodetic_crs','EPSG','4073','EPSG','4543','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4074','SREF98','Replaced by SRB_ETRS89 (STRS00) (CRS code 8684).','geographic 3D','EPSG','6423','EPSG','1034',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2988','geodetic_crs','EPSG','4074','EPSG','4543','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4075','SREF98','Replaces MGI 1901 (CRS code 3906) in Serbia. Replaced by SRB_ETRS89 (STRS00) (CRS code 8685).','geographic 2D','EPSG','6422','EPSG','1034',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2989','geodetic_crs','EPSG','4075','EPSG','4543','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','4079','REGCAN95','','geocentric','EPSG','6500','EPSG','1035',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2990','geodetic_crs','EPSG','4079','EPSG','3199','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4080','REGCAN95','','geographic 3D','EPSG','6423','EPSG','1035',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2991','geodetic_crs','EPSG','4080','EPSG','3199','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4081','REGCAN95','Replaces Pico de las Nieves 1984 (PN84).','geographic 2D','EPSG','6422','EPSG','1035',NULL,0);
INSERT INTO "usage" VALUES('EPSG','2992','geodetic_crs','EPSG','4081','EPSG','3199','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','4120','Greek','','geographic 2D','EPSG','6422','EPSG','6120',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3005','geodetic_crs','EPSG','4120','EPSG','3254','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4121','GGRS87','','geographic 2D','EPSG','6422','EPSG','6121',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3006','geodetic_crs','EPSG','4121','EPSG','3254','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4122','ATS77','In use from 1979. To be phased out in late 1990''s.','geographic 2D','EPSG','6422','EPSG','6122',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3007','geodetic_crs','EPSG','4122','EPSG','1283','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4123','KKJ','','geographic 2D','EPSG','6422','EPSG','6123',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3008','geodetic_crs','EPSG','4123','EPSG','3333','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4124','RT90','','geographic 2D','EPSG','6422','EPSG','6124',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3009','geodetic_crs','EPSG','4124','EPSG','1225','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4125','Samboja','','geographic 2D','EPSG','6402','EPSG','6125',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3010','geodetic_crs','EPSG','4125','EPSG','1328','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4126','LKS94 (ETRS89)','','geographic 2D','EPSG','6402','EPSG','6126',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3011','geodetic_crs','EPSG','4126','EPSG','1145','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4127','Tete','','geographic 2D','EPSG','6422','EPSG','6127',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3012','geodetic_crs','EPSG','4127','EPSG','3281','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4128','Madzansua','Replaced by values transformed to Tete GeogCRS (code 4127).','geographic 2D','EPSG','6422','EPSG','6128',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3013','geodetic_crs','EPSG','4128','EPSG','1315','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4129','Observatario','Replaced by values transformed to Tete geogCRS (code 4127).','geographic 2D','EPSG','6422','EPSG','6129',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3014','geodetic_crs','EPSG','4129','EPSG','1329','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4130','Moznet','','geographic 2D','EPSG','6422','EPSG','6130',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3015','geodetic_crs','EPSG','4130','EPSG','1167','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','4131','Indian 1960','','geographic 2D','EPSG','6422','EPSG','6131',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3016','geodetic_crs','EPSG','4131','EPSG','4007','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4132','FD58','','geographic 2D','EPSG','6422','EPSG','6132',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3017','geodetic_crs','EPSG','4132','EPSG','1300','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4133','EST92','This name is also used for a projected CRS (see projCRS code 3300). Replaced by EST97 (code 4180).','geographic 2D','EPSG','6422','EPSG','6133',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3018','geodetic_crs','EPSG','4133','EPSG','3246','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4134','PSD93','Replaced Fahud geogCRS (code 4232) in 1993. Maximum differences to Fahud adjustment are 20 metres.','geographic 2D','EPSG','6422','EPSG','6134',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3019','geodetic_crs','EPSG','4134','EPSG','3288','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4135','Old Hawaiian','Note: this CRS includes longitudes which are POSITIVE EAST.','geographic 2D','EPSG','6422','EPSG','6135',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3020','geodetic_crs','EPSG','4135','EPSG','1334','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4136','St. Lawrence Island','Note: this CRS includes longitudes which are POSITIVE EAST.','geographic 2D','EPSG','6422','EPSG','6136',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3021','geodetic_crs','EPSG','4136','EPSG','1332','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4137','St. Paul Island','Note: this CRS includes longitudes which are POSITIVE EAST.','geographic 2D','EPSG','6422','EPSG','6137',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3022','geodetic_crs','EPSG','4137','EPSG','1333','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4138','St. George Island','Note: this CRS includes longitudes which are POSITIVE EAST.','geographic 2D','EPSG','6422','EPSG','6138',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3023','geodetic_crs','EPSG','4138','EPSG','1331','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4139','Puerto Rico','NAD27 (CRS code 4267) used for military purposes. Note: this CRS includes longitudes which are POSITIVE EAST.','geographic 2D','EPSG','6422','EPSG','6139',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3024','geodetic_crs','EPSG','4139','EPSG','1335','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4140','NAD83(CSRS98)','In New Brunswick superseded ATS77 from 1999.','geographic 2D','EPSG','6402','EPSG','6140',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3025','geodetic_crs','EPSG','4140','EPSG','1336','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4141','Israel 1993','Replaces Palestine 1923 (CRS code 4281) from June 1998. Replaced by IGD05 (CRS code 6980) from January 2005.','geographic 2D','EPSG','6422','EPSG','6141',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3026','geodetic_crs','EPSG','4141','EPSG','2603','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4142','Locodjo 1965','Replaced by Abidjan 1987 (EPSG code 4143).','geographic 2D','EPSG','6422','EPSG','6142',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3027','geodetic_crs','EPSG','4142','EPSG','1075','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4143','Abidjan 1987','Replaces Locodjo 1965 (EPSG code 4142).','geographic 2D','EPSG','6422','EPSG','6143',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3028','geodetic_crs','EPSG','4143','EPSG','1075','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4144','Kalianpur 1937','Adopts 1937 metric conversion of 0.30479841 metres per Indian foot.','geographic 2D','EPSG','6422','EPSG','6144',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3029','geodetic_crs','EPSG','4144','EPSG','1308','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4145','Kalianpur 1962','Adopts 1962 metric conversion of 0.3047996 metres per Indian foot.','geographic 2D','EPSG','6422','EPSG','6145',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3030','geodetic_crs','EPSG','4145','EPSG','1184','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4146','Kalianpur 1975','Adopts 1975 metric conversion of 0.3047995 metres per Indian foot.','geographic 2D','EPSG','6422','EPSG','6146',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3031','geodetic_crs','EPSG','4146','EPSG','3341','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4147','Hanoi 1972','Replaces use of Indian 1960. Replaced by VN-2000 (CRS code 4756).','geographic 2D','EPSG','6422','EPSG','6147',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3032','geodetic_crs','EPSG','4147','EPSG','3328','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4148','Hartebeesthoek94','Replaces Cape (code 4222) from 1999.','geographic 2D','EPSG','6422','EPSG','6148',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3033','geodetic_crs','EPSG','4148','EPSG','4540','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','4149','CH1903','Replaced by CH1903+.','geographic 2D','EPSG','6422','EPSG','6149',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3034','geodetic_crs','EPSG','4149','EPSG','1286','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4150','CH1903+','Replaces CH1903.','geographic 2D','EPSG','6422','EPSG','6150',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3035','geodetic_crs','EPSG','4150','EPSG','1286','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4151','CHTRS95','','geographic 2D','EPSG','6422','EPSG','6151',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3036','geodetic_crs','EPSG','4151','EPSG','1286','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','4152','NAD83(HARN)','In CONUS, AK, HI and PRVI replaces NAD83 for applications with an accuracy of better than 1m. Replaced by NAD83(FBN) in CONUS, American Samoa and Guam / NMI, by NAD83(NSRS2007) in Alaska, by NAD83(PA11) in Hawaii and by NAD83(HARN Corrected) in PRVI.','geographic 2D','EPSG','6422','EPSG','6152',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3037','geodetic_crs','EPSG','4152','EPSG','1337','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','4153','Rassadiran','','geographic 2D','EPSG','6422','EPSG','6153',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3038','geodetic_crs','EPSG','4153','EPSG','1338','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4154','ED50(ED77)','','geographic 2D','EPSG','6422','EPSG','6154',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3039','geodetic_crs','EPSG','4154','EPSG','1123','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4155','Dabola 1981','Replaces Conakry 1905 (EPSG code 4315).','geographic 2D','EPSG','6422','EPSG','6155',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3040','geodetic_crs','EPSG','4155','EPSG','3257','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4156','S-JTSK','Greenwich-referenced equivalent to S-JTSK (CRS code 4818). Technically improved and replaced through S-JTSK [JTSK03] in Slovakia, CRS code 8351. Remains the legal system in Czechia but see scientific working system CRSs 5228 and 5229.','geographic 2D','EPSG','6422','EPSG','6156',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3041','geodetic_crs','EPSG','4156','EPSG','1306','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4157','Mount Dillon','','geographic 2D','EPSG','6422','EPSG','6157',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3042','geodetic_crs','EPSG','4157','EPSG','1322','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4158','Naparima 1955','Extended to Tobago as Naparima 1972. (Note: Naparima 1972 is not used in Trinidad).','geographic 2D','EPSG','6422','EPSG','6158',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3043','geodetic_crs','EPSG','4158','EPSG','3143','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4159','ELD79','','geographic 2D','EPSG','6422','EPSG','6159',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3044','geodetic_crs','EPSG','4159','EPSG','1143','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4160','Chos Malal 1914','Replaced by Campo Inchauspe (geogCRS code 4221) for topographic mapping, use for oil exploration and production continues.','geographic 2D','EPSG','6422','EPSG','6160',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3045','geodetic_crs','EPSG','4160','EPSG','4562','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4161','Pampa del Castillo','Replaced by Campo Inchauspe (geogCRS code 4221) for topographic mapping, use for oil exploration and production in Golfo San Jorge basin (44°S to 47.5°S) continues.','geographic 2D','EPSG','6422','EPSG','6161',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3046','geodetic_crs','EPSG','4161','EPSG','4563','EPSG','1136');
INSERT INTO "geodetic_crs" VALUES('EPSG','4162','Korean 1985','Replaces use of Tokyo datum. Replaced by KGD2002.','geographic 2D','EPSG','6422','EPSG','6162',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3047','geodetic_crs','EPSG','4162','EPSG','3266','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4163','Yemen NGN96','','geographic 2D','EPSG','6422','EPSG','6163',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3048','geodetic_crs','EPSG','4163','EPSG','1257','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','4164','South Yemen','','geographic 2D','EPSG','6422','EPSG','6164',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3049','geodetic_crs','EPSG','4164','EPSG','1340','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4165','Bissau','','geographic 2D','EPSG','6422','EPSG','6165',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3050','geodetic_crs','EPSG','4165','EPSG','3258','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4166','Korean 1995','','geographic 2D','EPSG','6422','EPSG','6166',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3051','geodetic_crs','EPSG','4166','EPSG','3266','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4167','NZGD2000','Replaces NZGD49 (code 4272) and CI79 (code 4673) from March 2000.','geographic 2D','EPSG','6422','EPSG','6167',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3052','geodetic_crs','EPSG','4167','EPSG','1175','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','4168','Accra','Ellipsoid semi-major axis (a)=20926201 exactly Gold Coast feet. Replaced by Leigon (code 4250) in 1978.','geographic 2D','EPSG','6422','EPSG','6168',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3053','geodetic_crs','EPSG','4168','EPSG','1104','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4169','American Samoa 1962','','geographic 2D','EPSG','6422','EPSG','6169',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3054','geodetic_crs','EPSG','4169','EPSG','3109','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4170','SIRGAS 1995','Replaced by SIRGAS 2000 (CRS code 4674).','geographic 2D','EPSG','6422','EPSG','6170',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3055','geodetic_crs','EPSG','4170','EPSG','3448','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','4171','RGF93 v1','See CRS code 7084 for alternate system with axes reversed used by IGN for GIS purposes. Replaced by RGF93 v2 (CRS code 9777) from 2010-06-18.','geographic 2D','EPSG','6422','EPSG','6171',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3056','geodetic_crs','EPSG','4171','EPSG','1096','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','4172','POSGAR','A geodetic network of 127 high accuracy surved points that define the National Geodetic System (Sistema Geodésico Nacional), adopted by IGM in May 1997','geographic 2D','EPSG','6402','EPSG','6172',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3057','geodetic_crs','EPSG','4172','EPSG','1033','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4173','IRENET95','','geographic 2D','EPSG','6422','EPSG','6173',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3058','geodetic_crs','EPSG','4173','EPSG','1305','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','4174','Sierra Leone 1924','Ellipsoid semi-major axis (a)=20926201 exactly Gold Coast feet; 1 Gold Coast foot = 0.3047997101815 m.','geographic 2D','EPSG','6422','EPSG','6174',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3059','geodetic_crs','EPSG','4174','EPSG','1342','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4175','Sierra Leone 1968','Replaces Sierra Leone 1960. The 1968 readjustment coordinates are within 3m of the 1960 provisional adjustment.','geographic 2D','EPSG','6422','EPSG','6175',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3060','geodetic_crs','EPSG','4175','EPSG','3306','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4176','Australian Antarctic','','geographic 2D','EPSG','6422','EPSG','6176',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3061','geodetic_crs','EPSG','4176','EPSG','1278','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','4178','Pulkovo 1942(83)','Replaces 1956 adjustment (CRS code 4179). In Brandenburg replaced by ETRS89. In Sachsen and Thuringen replaced by RD83 and PD/83 which for practical purposes may be considered to be the same as DHDN.','geographic 2D','EPSG','6422','EPSG','6178',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3062','geodetic_crs','EPSG','4178','EPSG','3900','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4179','Pulkovo 1942(58)','Shares same origin definition as Pulkovo 1942 (CRS code 4284) and for low accuracy purposes these systems can be considered consistent with each other. Locally densified during 1957 and 1958. Replaced by 1983 adjustment (CRS code 4178).','geographic 2D','EPSG','6422','EPSG','6179',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3063','geodetic_crs','EPSG','4179','EPSG','3574','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4180','EST97','This name is also used for a projected CRS (see projCRS code 3301). Replaces EST92 (code 4133).','geographic 2D','EPSG','6422','EPSG','6180',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3064','geodetic_crs','EPSG','4180','EPSG','1090','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','4181','LUREF','','geographic 2D','EPSG','6422','EPSG','6181',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3065','geodetic_crs','EPSG','4181','EPSG','1146','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4182','Azores Occidental 1939','Replaced by PTRA08 (CRS code 5013).','geographic 2D','EPSG','6422','EPSG','6182',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3066','geodetic_crs','EPSG','4182','EPSG','1344','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4183','Azores Central 1948','Replaced by 1995 system (CRS code 4665).','geographic 2D','EPSG','6422','EPSG','6183',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3067','geodetic_crs','EPSG','4183','EPSG','1301','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4184','Azores Oriental 1940','Replaced by 1995 system (CRS code 4664).','geographic 2D','EPSG','6422','EPSG','6184',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3068','geodetic_crs','EPSG','4184','EPSG','1345','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4185','Madeira 1936','','geographic 2D','EPSG','6402','EPSG','6185',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3069','geodetic_crs','EPSG','4185','EPSG','1314','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4188','OSNI 1952','Replaced by 1975 Mapping Adjustment alias TM75. See CRS code 4300.','geographic 2D','EPSG','6422','EPSG','6188',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3070','geodetic_crs','EPSG','4188','EPSG','2530','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4189','REGVEN','Densification in Venezuela of SIRGAS.','geographic 2D','EPSG','6422','EPSG','6189',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3071','geodetic_crs','EPSG','4189','EPSG','1251','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','4190','POSGAR 98','Densification in Argentina of SIRGAS 1995. Until May 2009 replaced POSGAR 94 for many practical purposes (but not as the legal system). POSGAR 94 was officially replaced by POSGAR 2007 in May 2009.','geographic 2D','EPSG','6422','EPSG','6190',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3072','geodetic_crs','EPSG','4190','EPSG','1033','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','4191','Albanian 1987','Replaced by KRGJSH-2010.','geographic 2D','EPSG','6422','EPSG','6191',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3073','geodetic_crs','EPSG','4191','EPSG','3212','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4192','Douala 1948','Replaced by Manoca 1962 (code 4193).','geographic 2D','EPSG','6422','EPSG','6192',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3074','geodetic_crs','EPSG','4192','EPSG','2555','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4193','Manoca 1962','Replaces Doula 1948 (code 4192). The intent of the Bukavu 1953 conference was to adopt the Clarke 1880 (RGS) ellipsoid (code 7012) but in practice this CRS has used the IGN version.','geographic 2D','EPSG','6422','EPSG','6193',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3075','geodetic_crs','EPSG','4193','EPSG','2555','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4194','Qoornoq 1927','','geographic 2D','EPSG','6422','EPSG','6194',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3076','geodetic_crs','EPSG','4194','EPSG','3362','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4195','Scoresbysund 1952','','geographic 2D','EPSG','6422','EPSG','6195',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3077','geodetic_crs','EPSG','4195','EPSG','2570','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4196','Ammassalik 1958','','geographic 2D','EPSG','6422','EPSG','6196',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3078','geodetic_crs','EPSG','4196','EPSG','2571','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4197','Garoua','','geographic 2D','EPSG','6422','EPSG','6197',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3079','geodetic_crs','EPSG','4197','EPSG','2590','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4198','Kousseri','','geographic 2D','EPSG','6422','EPSG','6198',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3080','geodetic_crs','EPSG','4198','EPSG','2591','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4199','Egypt 1930','Note that Egypt 1930 uses the International 1924 ellipsoid, unlike the Egypt 1907 CRS (code 4229) which uses the Helmert ellipsoid. Oil industry references to the Egypt 1930 name and the Helmert ellipsoid probably mean Egypt 1907.','geographic 2D','EPSG','6422','EPSG','6199',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3081','geodetic_crs','EPSG','4199','EPSG','3242','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4200','Pulkovo 1995','Decree #1463 of 2012-12-28 announced that S-95 to be phased out and replaced by GSK-11 (CRS code 7683) by 2017.','geographic 2D','EPSG','6422','EPSG','6200',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3082','geodetic_crs','EPSG','4200','EPSG','1198','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4201','Adindan','The 12th parallel traverse of 1966-70 (geogCRS Point 58, code 4620) is connected to the Blue Nile 1958 network in western Sudan. This has given rise to misconceptions that the Blue Nile 1958 network is used in west Africa.','geographic 2D','EPSG','6422','EPSG','6201',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3083','geodetic_crs','EPSG','4201','EPSG','1271','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4202','AGD66','','geographic 2D','EPSG','6422','EPSG','6202',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3084','geodetic_crs','EPSG','4202','EPSG','1279','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4203','AGD84','National system replacing AGD66 but officially adopted only in Queensland, South Australia and Western Australia. Replaced by GDA94.','geographic 2D','EPSG','6422','EPSG','6203',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3085','geodetic_crs','EPSG','4203','EPSG','2576','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4204','Ain el Abd','','geographic 2D','EPSG','6422','EPSG','6204',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3086','geodetic_crs','EPSG','4204','EPSG','1272','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4205','Afgooye','','geographic 2D','EPSG','6422','EPSG','6205',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3087','geodetic_crs','EPSG','4205','EPSG','3308','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4206','Agadez','','geographic 2D','EPSG','6422','EPSG','6206',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3088','geodetic_crs','EPSG','4206','EPSG','1177','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4207','Lisbon','Replaces Lisbon 1890 system which used Bessel 1841 ellipsoid (code 4666). Replaced by Datum 73 (code 4274).','geographic 2D','EPSG','6422','EPSG','6207',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3089','geodetic_crs','EPSG','4207','EPSG','1294','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4208','Aratu','','geographic 2D','EPSG','6422','EPSG','6208',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3090','geodetic_crs','EPSG','4208','EPSG','1274','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4209','Arc 1950','','geographic 2D','EPSG','6422','EPSG','6209',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3091','geodetic_crs','EPSG','4209','EPSG','1276','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4210','Arc 1960','','geographic 2D','EPSG','6422','EPSG','6210',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3092','geodetic_crs','EPSG','4210','EPSG','1277','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4211','Batavia','','geographic 2D','EPSG','6422','EPSG','6211',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3093','geodetic_crs','EPSG','4211','EPSG','3666','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4212','Barbados 1938','','geographic 2D','EPSG','6422','EPSG','6212',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3094','geodetic_crs','EPSG','4212','EPSG','3218','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4213','Beduaram','','geographic 2D','EPSG','6422','EPSG','6213',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3095','geodetic_crs','EPSG','4213','EPSG','2771','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4214','Beijing 1954','In 1982 replaced by Xian 1980 (CRS code 4610) and New Beijing (CRS code 4555).','geographic 2D','EPSG','6422','EPSG','6214',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3096','geodetic_crs','EPSG','4214','EPSG','1067','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4215','BD50','','geographic 2D','EPSG','6422','EPSG','6215',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3097','geodetic_crs','EPSG','4215','EPSG','1347','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4216','Bermuda 1957','Replaced by BDA2000 (CRS code 4762).','geographic 2D','EPSG','6422','EPSG','6216',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3098','geodetic_crs','EPSG','4216','EPSG','3221','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4218','Bogota 1975','Replaces earlier 3 adjustments of 1951, 1944 and 1941. Replaced by MAGNA-SIRGAS (CRS code 4685).','geographic 2D','EPSG','6422','EPSG','6218',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3100','geodetic_crs','EPSG','4218','EPSG','3686','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4219','Bukit Rimpah','','geographic 2D','EPSG','6422','EPSG','6219',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3101','geodetic_crs','EPSG','4219','EPSG','1287','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4220','Camacupa 1948','Provisional adjustment. Coastal stations used for offshore radio-navigation positioning and determination of transformations to WGS. Differs from second adjustment (Camacupa 2015, CRS code 8694), which is not used for offshore E&P, by up to 25m.','geographic 2D','EPSG','6422','EPSG','6220',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3102','geodetic_crs','EPSG','4220','EPSG','1288','EPSG','1136');
INSERT INTO "geodetic_crs" VALUES('EPSG','4221','Campo Inchauspe','','geographic 2D','EPSG','6422','EPSG','6221',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3103','geodetic_crs','EPSG','4221','EPSG','3843','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4222','Cape','Replaced by Hartbeesthoek94 from 1999.','geographic 2D','EPSG','6422','EPSG','6222',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3104','geodetic_crs','EPSG','4222','EPSG','1290','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4223','Carthage','','geographic 2D','EPSG','6422','EPSG','6223',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3105','geodetic_crs','EPSG','4223','EPSG','1236','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4224','Chua','The Chua origin and associated network is in Brazil with a connecting traverse through northern Paraguay. In Brazil used only as input into the Corrego Allegre adjustment (CRS code 4225), except for government work including SICAD in Distrito Federal.','geographic 2D','EPSG','6422','EPSG','6224',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3106','geodetic_crs','EPSG','4224','EPSG','3356','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4225','Corrego Alegre 1970-72','Replaces 1961 adjustment (CRS code 5524). Replaced by SAD69 (CRS code 4291).','geographic 2D','EPSG','6422','EPSG','6225',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3107','geodetic_crs','EPSG','4225','EPSG','1293','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4226','Cote d''Ivoire','','geographic 2D','EPSG','6402','EPSG','6226',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3108','geodetic_crs','EPSG','4226','EPSG','1075','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4227','Deir ez Zor','','geographic 2D','EPSG','6422','EPSG','6227',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3109','geodetic_crs','EPSG','4227','EPSG','1623','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4228','Douala','','geographic 2D','EPSG','6402','EPSG','6228',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3110','geodetic_crs','EPSG','4228','EPSG','1060','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4229','Egypt 1907','','geographic 2D','EPSG','6422','EPSG','6229',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3111','geodetic_crs','EPSG','4229','EPSG','1086','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4230','ED50','','geographic 2D','EPSG','6422','EPSG','6230',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3112','geodetic_crs','EPSG','4230','EPSG','1296','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4231','ED87','','geographic 2D','EPSG','6422','EPSG','6231',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3113','geodetic_crs','EPSG','4231','EPSG','1297','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4232','Fahud','Since 1993 replaced by PSD93 geogCRS (code 4134). Maximum differences to Fahud adjustment are 20 metres.','geographic 2D','EPSG','6422','EPSG','6232',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3114','geodetic_crs','EPSG','4232','EPSG','4009','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4233','Gandajika 1970','','geographic 2D','EPSG','6422','EPSG','6233',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3115','geodetic_crs','EPSG','4233','EPSG','1152','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4234','Garoua','','geographic 2D','EPSG','6402','EPSG','6234',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3116','geodetic_crs','EPSG','4234','EPSG','1060','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4235','Guyane Francaise','','geographic 2D','EPSG','6402','EPSG','6235',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3117','geodetic_crs','EPSG','4235','EPSG','1097','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4236','Hu Tzu Shan 1950','','geographic 2D','EPSG','6422','EPSG','6236',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3118','geodetic_crs','EPSG','4236','EPSG','3315','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4237','HD72','Replaced HD1909 (EPSG CRS code 3819).','geographic 2D','EPSG','6422','EPSG','6237',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3119','geodetic_crs','EPSG','4237','EPSG','1119','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4238','ID74','Replaced by DGN95.','geographic 2D','EPSG','6422','EPSG','6238',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3120','geodetic_crs','EPSG','4238','EPSG','4020','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4239','Indian 1954','','geographic 2D','EPSG','6422','EPSG','6239',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3121','geodetic_crs','EPSG','4239','EPSG','1304','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4240','Indian 1975','','geographic 2D','EPSG','6422','EPSG','6240',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3122','geodetic_crs','EPSG','4240','EPSG','3741','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4241','Jamaica 1875','','geographic 2D','EPSG','6422','EPSG','6241',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3123','geodetic_crs','EPSG','4241','EPSG','3342','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4242','JAD69','Replaced by JAD2001 (CRS code 4758).','geographic 2D','EPSG','6422','EPSG','6242',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3124','geodetic_crs','EPSG','4242','EPSG','3342','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4243','Kalianpur 1880','','geographic 2D','EPSG','6422','EPSG','6243',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3125','geodetic_crs','EPSG','4243','EPSG','1307','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4244','Kandawala','','geographic 2D','EPSG','6422','EPSG','6244',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3126','geodetic_crs','EPSG','4244','EPSG','3310','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4245','Kertau 1968','Not used for metrication of RSO grid - see Kertau (RSO) (CRS code 4751). Replaced by GDM2000 (CRS code 4742).','geographic 2D','EPSG','6422','EPSG','6245',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3127','geodetic_crs','EPSG','4245','EPSG','4223','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4246','KOC','','geographic 2D','EPSG','6422','EPSG','6246',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3128','geodetic_crs','EPSG','4246','EPSG','3267','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4247','La Canoa','This CRS is incorporated within PSAD56. See CRS code 4248.','geographic 2D','EPSG','6422','EPSG','6247',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3129','geodetic_crs','EPSG','4247','EPSG','3327','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4248','PSAD56','Incorporates La Canoa (CRS code 4247) and within Venezuela (but not beyond) the names La Canoa and PSAD56 are synonymous.','geographic 2D','EPSG','6422','EPSG','6248',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3130','geodetic_crs','EPSG','4248','EPSG','1348','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4249','Lake','','geographic 2D','EPSG','6422','EPSG','6249',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3131','geodetic_crs','EPSG','4249','EPSG','1312','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4250','Leigon','Replaced Accra (code 4168) from 1978.','geographic 2D','EPSG','6422','EPSG','6250',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3132','geodetic_crs','EPSG','4250','EPSG','1104','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4251','Liberia 1964','','geographic 2D','EPSG','6422','EPSG','6251',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3133','geodetic_crs','EPSG','4251','EPSG','3270','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4252','Lome','','geographic 2D','EPSG','6422','EPSG','6252',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3134','geodetic_crs','EPSG','4252','EPSG','1232','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4253','Luzon 1911','Replaced by PRS92 (CRS code 4683).','geographic 2D','EPSG','6422','EPSG','6253',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3135','geodetic_crs','EPSG','4253','EPSG','3969','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4254','Hito XVIII 1963','','geographic 2D','EPSG','6422','EPSG','6254',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3136','geodetic_crs','EPSG','4254','EPSG','1303','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4255','Herat North','','geographic 2D','EPSG','6422','EPSG','6255',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3137','geodetic_crs','EPSG','4255','EPSG','1024','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4256','Mahe 1971','This CRS has no known local application. South East Island 1943 (CRS codes 6892 and 6915) is used for topographic mapping, cadastral and hydrographic survey.','geographic 2D','EPSG','6422','EPSG','6256',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3138','geodetic_crs','EPSG','4256','EPSG','2369','EPSG','1160');
INSERT INTO "geodetic_crs" VALUES('EPSG','4257','Makassar','','geographic 2D','EPSG','6422','EPSG','6257',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3139','geodetic_crs','EPSG','4257','EPSG','1316','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4258','ETRS89','Has been realized through ETRF89, ETRF90, ETRF91, ETRF92, ETRF93, ETRF94, ETRF96, ETRF97, ETRF2000, ETRF2005 and ETRF2014. This ''ensemble'' covers any or all of these realizations without distinction.','geographic 2D','EPSG','6422','EPSG','6258',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3140','geodetic_crs','EPSG','4258','EPSG','4755','EPSG','1026');
INSERT INTO "geodetic_crs" VALUES('EPSG','4259','Malongo 1987','Replaced Mhast (offshore) (CRS code 4705) in 1987. References to "Mhast" since 1987 often should have stated "Malongo 1987".','geographic 2D','EPSG','6422','EPSG','6259',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3141','geodetic_crs','EPSG','4259','EPSG','3180','EPSG','1136');
INSERT INTO "geodetic_crs" VALUES('EPSG','4260','Manoca','','geographic 2D','EPSG','6402','EPSG','6260',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3142','geodetic_crs','EPSG','4260','EPSG','1060','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4261','Merchich','','geographic 2D','EPSG','6422','EPSG','6261',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3143','geodetic_crs','EPSG','4261','EPSG','4581','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4262','Massawa','','geographic 2D','EPSG','6422','EPSG','6262',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3144','geodetic_crs','EPSG','4262','EPSG','1089','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4263','Minna','','geographic 2D','EPSG','6422','EPSG','6263',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3145','geodetic_crs','EPSG','4263','EPSG','1178','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4264','Mhast','','geographic 2D','EPSG','6422','EPSG','6264',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3146','geodetic_crs','EPSG','4264','EPSG','1318','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4265','Monte Mario','','geographic 2D','EPSG','6422','EPSG','6265',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3147','geodetic_crs','EPSG','4265','EPSG','3343','EPSG','1187');
INSERT INTO "geodetic_crs" VALUES('EPSG','4266','M''poraloko','','geographic 2D','EPSG','6422','EPSG','6266',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3148','geodetic_crs','EPSG','4266','EPSG','1100','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4267','NAD27','Note: this CRS includes longitudes which are POSITIVE EAST. Replaced by NAD27(76) (code 4608) in Ontario, CGQ77 (code 4609) in Quebec, Mexican Datum of 1993 (code 4483) in Mexico, NAD83 (code 4269) in Canada (excl. Ontario & Quebec) & USA.','geographic 2D','EPSG','6422','EPSG','6267',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3149','geodetic_crs','EPSG','4267','EPSG','1349','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4268','NAD27 Michigan','Ellipsoid taken to be 800ft above the NAD27 reference ellipsoid. Note: this coordinate reference system includes longitudes which are POSITIVE EAST.','geographic 2D','EPSG','6422','EPSG','6268',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3150','geodetic_crs','EPSG','4268','EPSG','1391','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4269','NAD83','Longitude is POSITIVE EAST. The adjustment included connections to Greenland and Mexico but the system was not adopted there. For applications with an accuracy of better than 1m replaced by NAD83(HARN) in the US and PRVI and by NAD83(CSRS) in Canada.','geographic 2D','EPSG','6422','EPSG','6269',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3151','geodetic_crs','EPSG','4269','EPSG','1350','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4270','Nahrwan 1967','','geographic 2D','EPSG','6422','EPSG','6270',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3152','geodetic_crs','EPSG','4270','EPSG','1351','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4271','Naparima 1972','Naparima 1972 is an extension to Tobago of the Naparima 1955 network of Trinidad.','geographic 2D','EPSG','6422','EPSG','6271',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3153','geodetic_crs','EPSG','4271','EPSG','1322','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4272','NZGD49','Replaced by NZGD2000 (CRS code 4167) in March 2000.','geographic 2D','EPSG','6422','EPSG','6272',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3154','geodetic_crs','EPSG','4272','EPSG','3285','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4273','NGO 1948','','geographic 2D','EPSG','6422','EPSG','6273',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3155','geodetic_crs','EPSG','4273','EPSG','1352','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4274','Datum 73','','geographic 2D','EPSG','6422','EPSG','6274',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3156','geodetic_crs','EPSG','4274','EPSG','1294','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4275','NTF','','geographic 2D','EPSG','6422','EPSG','6275',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3157','geodetic_crs','EPSG','4275','EPSG','3694','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4276','NSWC 9Z-2','','geographic 2D','EPSG','6422','EPSG','6276',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3158','geodetic_crs','EPSG','4276','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4277','OSGB36','','geographic 2D','EPSG','6422','EPSG','6277',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3159','geodetic_crs','EPSG','4277','EPSG','4390','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4278','OSGB70','','geographic 2D','EPSG','6422','EPSG','6278',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3160','geodetic_crs','EPSG','4278','EPSG','1264','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4279','OS(SN)80','','geographic 2D','EPSG','6422','EPSG','6279',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3161','geodetic_crs','EPSG','4279','EPSG','1354','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4280','Padang','','geographic 2D','EPSG','6422','EPSG','6280',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3162','geodetic_crs','EPSG','4280','EPSG','1355','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4281','Palestine 1923','','geographic 2D','EPSG','6422','EPSG','6281',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3163','geodetic_crs','EPSG','4281','EPSG','1356','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4282','Pointe Noire','','geographic 2D','EPSG','6422','EPSG','6282',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3164','geodetic_crs','EPSG','4282','EPSG','1072','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4283','GDA94','','geographic 2D','EPSG','6422','EPSG','6283',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3165','geodetic_crs','EPSG','4283','EPSG','4177','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','4284','Pulkovo 1942','Extended to Eastern Europe through Uniform Astro-Geodetic Network (UAGN) of 1956 - see CRS code 4179.','geographic 2D','EPSG','6422','EPSG','6284',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3166','geodetic_crs','EPSG','4284','EPSG','2423','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4285','Qatar 1974','','geographic 2D','EPSG','6422','EPSG','6285',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3167','geodetic_crs','EPSG','4285','EPSG','1195','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4286','Qatar 1948','','geographic 2D','EPSG','6422','EPSG','6286',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3168','geodetic_crs','EPSG','4286','EPSG','1346','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4287','Qornoq','','geographic 2D','EPSG','6402','EPSG','6287',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3169','geodetic_crs','EPSG','4287','EPSG','1107','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4288','Loma Quintana','Replaced by La Canoa (code 4247).','geographic 2D','EPSG','6422','EPSG','6288',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3170','geodetic_crs','EPSG','4288','EPSG','1313','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4289','Amersfoort','Use of geographic2D CRS Amersfoort (code 4289) for spatial referencing is discouraged. Use projected CRS Amersfoort / RD New (code 28992).','geographic 2D','EPSG','6422','EPSG','6289',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3171','geodetic_crs','EPSG','4289','EPSG','1275','EPSG','1269');
INSERT INTO "geodetic_crs" VALUES('EPSG','4291','SAD69','Uses GRS67 ellipsoid with 1/f to exactly 2 decimal places. Precision of ellipsoid entry increased from 2 to 5 dp with change id 97.251. Error introduced if not using the truncated precision is 0 to 31mm.','geographic 2D','EPSG','6402','EPSG','6291',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3172','geodetic_crs','EPSG','4291','EPSG','1358','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4292','Sapper Hill 1943','','geographic 2D','EPSG','6422','EPSG','6292',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3173','geodetic_crs','EPSG','4292','EPSG','3247','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4293','Schwarzeck','','geographic 2D','EPSG','6422','EPSG','6293',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3174','geodetic_crs','EPSG','4293','EPSG','1169','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4294','Segora','','geographic 2D','EPSG','6402','EPSG','6294',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3175','geodetic_crs','EPSG','4294','EPSG','1359','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4295','Serindung','','geographic 2D','EPSG','6422','EPSG','6295',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3176','geodetic_crs','EPSG','4295','EPSG','4005','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4296','Sudan','','geographic 2D','EPSG','6402','EPSG','6296',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3177','geodetic_crs','EPSG','4296','EPSG','1361','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4297','Tananarive','','geographic 2D','EPSG','6422','EPSG','6297',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3178','geodetic_crs','EPSG','4297','EPSG','1149','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4298','Timbalai 1948','Adopts metric conversion of 39.370147 inches per metre. Replaced by GDM2000 (CRS code 4742) in East Malaysia and by GDBD2009 (CRS code 5247) in Brunei.','geographic 2D','EPSG','6422','EPSG','6298',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3179','geodetic_crs','EPSG','4298','EPSG','1362','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4299','TM65','Replaced by 1975 Mapping Adjustment alias TM75 (CRS code 4300). Not to be confused with the Geodetic Datum of 1965 (datum code 6300) which is used by TM75.','geographic 2D','EPSG','6422','EPSG','6299',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3180','geodetic_crs','EPSG','4299','EPSG','1305','EPSG','1089');
INSERT INTO "geodetic_crs" VALUES('EPSG','4300','TM75','Uses Geodetic Datum of 1965 which should not be confused with the 1965 adjustment (TM65, datum code 6299 and CRS code 4299). Replaces OSNI52 (CRS code 4188) and TM65 (CRS code 4299). Replaced by IRENET95 (CRS code 4173).','geographic 2D','EPSG','6422','EPSG','6300',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3181','geodetic_crs','EPSG','4300','EPSG','1305','EPSG','1090');
INSERT INTO "geodetic_crs" VALUES('EPSG','4301','Tokyo','In Japan, replaces Tokyo 1892 (CRS code 5132) and replaced by JGD2000 (code 4612) from April 2002. In Korea used only for geodetic applications before being replaced by Korean 1985 (code 4162).','geographic 2D','EPSG','6422','EPSG','6301',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3182','geodetic_crs','EPSG','4301','EPSG','1364','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4302','Trinidad 1903','','geographic 2D','EPSG','6422','EPSG','6302',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3183','geodetic_crs','EPSG','4302','EPSG','1339','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4303','TC(1948)','','geographic 2D','EPSG','6422','EPSG','6303',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3184','geodetic_crs','EPSG','4303','EPSG','1363','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4304','Voirol 1875','The appropriate usage of CRSs using the Voirol 1875 and 1879 datums is lost in antiquity. They differ by about 9 metres. Oil industry references to one could in reality be to either. All replaced by Nord Sahara 1959 (CRS code 4307).','geographic 2D','EPSG','6422','EPSG','6304',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3185','geodetic_crs','EPSG','4304','EPSG','1365','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4306','Bern 1938','Used for the geographic coordinates overprinted on topographic maps constructed on the CH1903 / LV03 projected CS (code 21781).','geographic 2D','EPSG','6422','EPSG','6306',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3186','geodetic_crs','EPSG','4306','EPSG','1286','EPSG','1153');
INSERT INTO "geodetic_crs" VALUES('EPSG','4307','Nord Sahara 1959','Sometimes incorrectly referred to as Voirol Unifie 1960: this is NOT a GeogCRS but two projected CRSs based on Nord Sahara 1959 (codes 30791-92). Strictly applicable only to north of 32°N but extended southwards non-homogoneously by oil industry.','geographic 2D','EPSG','6422','EPSG','6307',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3187','geodetic_crs','EPSG','4307','EPSG','1026','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4308','RT38','','geographic 2D','EPSG','6422','EPSG','6308',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3188','geodetic_crs','EPSG','4308','EPSG','3313','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4309','Yacare','Replaced by SIRGAS-ROU98 (CRS code 5381).','geographic 2D','EPSG','6422','EPSG','6309',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3189','geodetic_crs','EPSG','4309','EPSG','3326','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4310','Yoff','','geographic 2D','EPSG','6422','EPSG','6310',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3190','geodetic_crs','EPSG','4310','EPSG','1207','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4311','Zanderij','Introduced in 1975.','geographic 2D','EPSG','6422','EPSG','6311',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3191','geodetic_crs','EPSG','4311','EPSG','1222','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4312','MGI','Retrospectively defined as derived after the introduction of geographic 3D CRS (code 9267).','geographic 2D','EPSG','6422','EPSG','6312',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3192','geodetic_crs','EPSG','4312','EPSG','1037','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4313','BD72','','geographic 2D','EPSG','6422','EPSG','6313',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3193','geodetic_crs','EPSG','4313','EPSG','1347','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4314','DHDN','See also RD/83 for Saxony and PD/83 for Thuringen. For national digital cartographic purposes used across all German states.','geographic 2D','EPSG','6422','EPSG','6314',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3194','geodetic_crs','EPSG','4314','EPSG','2326','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4315','Conakry 1905','Replaced by Dabola 1981 (EPSG code 4155).','geographic 2D','EPSG','6422','EPSG','6315',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3195','geodetic_crs','EPSG','4315','EPSG','3257','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4316','Dealul Piscului 1930','Replaced by Pulkovo 1942(58) (geogCRS code 4179).','geographic 2D','EPSG','6422','EPSG','6316',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3196','geodetic_crs','EPSG','4316','EPSG','3295','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4317','Dealul Piscului 1970','Replaces 1933 system (geogCRS code 4316).','geographic 2D','EPSG','6422','EPSG','6317',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3197','geodetic_crs','EPSG','4317','EPSG','1197','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4318','NGN','','geographic 2D','EPSG','6422','EPSG','6318',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3198','geodetic_crs','EPSG','4318','EPSG','3267','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4319','KUDAMS','','geographic 2D','EPSG','6422','EPSG','6319',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3199','geodetic_crs','EPSG','4319','EPSG','1310','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4322','WGS 72','Replaced by WGS 84.','geographic 2D','EPSG','6422','EPSG','6322',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3200','geodetic_crs','EPSG','4322','EPSG','1262','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','4324','WGS 72BE','Broadcast ephemeris. Replaced by WGS 84.','geographic 2D','EPSG','6422','EPSG','6324',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3201','geodetic_crs','EPSG','4324','EPSG','1262','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','4326','WGS 84','','geographic 2D','EPSG','6422','EPSG','6326',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3202','geodetic_crs','EPSG','4326','EPSG','2830','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','4327','WGS 84 (geographic 3D)','','geographic 3D','EPSG','6401','EPSG','6326',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3203','geodetic_crs','EPSG','4327','EPSG','1262','EPSG','1176');
INSERT INTO "geodetic_crs" VALUES('EPSG','4328','WGS 84 (geocentric)','','geocentric','EPSG','6500','EPSG','6326',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3204','geodetic_crs','EPSG','4328','EPSG','1262','EPSG','1176');
INSERT INTO "geodetic_crs" VALUES('EPSG','4329','WGS 84 (3D)','','geographic 3D','EPSG','6401','EPSG','6326',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3205','geodetic_crs','EPSG','4329','EPSG','2830','EPSG','1176');
INSERT INTO "geodetic_crs" VALUES('EPSG','4330','ITRF88 (geocentric)','Superseded by ITRF89 (code 4331).','geocentric','EPSG','6500','EPSG','6647',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3206','geodetic_crs','EPSG','4330','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4331','ITRF89 (geocentric)','Supersedes ITRF88 (code 4330). Superseded by ITRF91 (code 4332).','geocentric','EPSG','6500','EPSG','6648',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3207','geodetic_crs','EPSG','4331','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4332','ITRF90 (geocentric)','Supersedes ITRF89 (code 4331). Superseded by ITRF91 (code 4333).','geocentric','EPSG','6500','EPSG','6649',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3208','geodetic_crs','EPSG','4332','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4333','ITRF91 (geocentric)','Supersedes ITRF90 (code 4332). Superseded by ITRF92 (code 4334).','geocentric','EPSG','6500','EPSG','6650',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3209','geodetic_crs','EPSG','4333','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4334','ITRF92 (geocentric)','Supersedes ITRF91 (code 4333). Superseded by ITRF93 (code 4335).','geocentric','EPSG','6500','EPSG','6651',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3210','geodetic_crs','EPSG','4334','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4335','ITRF93 (geocentric)','Supersedes ITRF92 (code 4334). Superseded by ITRF94 (code 4336).','geocentric','EPSG','6500','EPSG','6652',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3211','geodetic_crs','EPSG','4335','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4336','ITRF94 (geocentric)','Supersedes ITRF93 (code 4335). Superseded by ITRF96 (code 4337).','geocentric','EPSG','6500','EPSG','6653',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3212','geodetic_crs','EPSG','4336','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4337','ITRF96 (geocentric)','Supersedes ITRF94 (code 4336). Superseded by ITRF97 (code 4338).','geocentric','EPSG','6500','EPSG','6654',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3213','geodetic_crs','EPSG','4337','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4338','ITRF97 (geocentric)','Supersedes ITRF96 (code 4337). Superseded by ITRF2000 (code 4385).','geocentric','EPSG','6500','EPSG','6655',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3214','geodetic_crs','EPSG','4338','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4339','Australian Antarctic (3D)','','geographic 3D','EPSG','6401','EPSG','6176',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3215','geodetic_crs','EPSG','4339','EPSG','1278','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4340','Australian Antarctic (geocentric)','','geocentric','EPSG','6500','EPSG','6176',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3216','geodetic_crs','EPSG','4340','EPSG','1278','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4341','EST97 (3D)','','geographic 3D','EPSG','6401','EPSG','6180',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3217','geodetic_crs','EPSG','4341','EPSG','1090','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4342','EST97 (geocentric)','','geocentric','EPSG','6500','EPSG','6180',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3218','geodetic_crs','EPSG','4342','EPSG','1090','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4343','CHTRF95 (3D)','','geographic 3D','EPSG','6401','EPSG','6151',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3219','geodetic_crs','EPSG','4343','EPSG','1286','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4344','CHTRF95 (geocentric)','','geocentric','EPSG','6500','EPSG','6151',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3220','geodetic_crs','EPSG','4344','EPSG','1286','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4345','ETRS89 (3D)','The distinction in usage between ETRF89 and ETRS89 is confused: although in principle conceptually different in practice both are used as synonyms.','geographic 3D','EPSG','6401','EPSG','6258',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3221','geodetic_crs','EPSG','4345','EPSG','1298','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4346','ETRS89 (geocentric)','The distinction in usage between ETRF89 and ETRS89 is confused: although in principle conceptually different in practice both are used as synonyms.','geocentric','EPSG','6500','EPSG','6258',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3222','geodetic_crs','EPSG','4346','EPSG','1298','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4347','GDA94 (3D)','Horizontal coordinates referenced to this CRS are in degrees. Any degree representation (e.g. DMSH, decimal, etc.) may be used but that used must be declared for the user.','geographic 3D','EPSG','6423','EPSG','6283',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3223','geodetic_crs','EPSG','4347','EPSG','1036','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4348','GDA94 (geocentric)','','geocentric','EPSG','6500','EPSG','6283',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3224','geodetic_crs','EPSG','4348','EPSG','1036','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4349','Hartebeesthoek94 (3D)','','geographic 3D','EPSG','6401','EPSG','6148',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3225','geodetic_crs','EPSG','4349','EPSG','1215','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4350','Hartebeesthoek94 (geocentric)','','geocentric','EPSG','6500','EPSG','6148',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3226','geodetic_crs','EPSG','4350','EPSG','1215','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4351','IRENET95 (3D)','','geographic 3D','EPSG','6401','EPSG','6173',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3227','geodetic_crs','EPSG','4351','EPSG','1305','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4352','IRENET95 (geocentric)','','geocentric','EPSG','6500','EPSG','6173',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3228','geodetic_crs','EPSG','4352','EPSG','1305','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4353','JGD2000 (3D)','','geographic 3D','EPSG','6401','EPSG','6612',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3229','geodetic_crs','EPSG','4353','EPSG','1129','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4354','JGD2000 (geocentric)','','geocentric','EPSG','6500','EPSG','6612',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3230','geodetic_crs','EPSG','4354','EPSG','1129','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4355','LKS94 (ETRS89) (3D)','','geographic 3D','EPSG','6401','EPSG','6126',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3231','geodetic_crs','EPSG','4355','EPSG','1145','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4356','LKS94 (ETRS89) (geocentric)','','geocentric','EPSG','6500','EPSG','6126',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3232','geodetic_crs','EPSG','4356','EPSG','1145','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4357','Moznet (3D)','','geographic 3D','EPSG','6401','EPSG','6130',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3233','geodetic_crs','EPSG','4357','EPSG','1167','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4358','Moznet (geocentric)','','geocentric','EPSG','6500','EPSG','6130',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3234','geodetic_crs','EPSG','4358','EPSG','1167','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4359','NAD83(CSRS) (3D)','','geographic 3D','EPSG','6401','EPSG','6140',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3235','geodetic_crs','EPSG','4359','EPSG','2784','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4360','NAD83(CSRS) (geocentric)','','geocentric','EPSG','6500','EPSG','6140',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3236','geodetic_crs','EPSG','4360','EPSG','2784','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4361','NAD83(HARN) (3D)','','geographic 3D','EPSG','6401','EPSG','6152',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3237','geodetic_crs','EPSG','4361','EPSG','1337','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4362','NAD83(HARN) (geocentric)','','geocentric','EPSG','6500','EPSG','6152',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3238','geodetic_crs','EPSG','4362','EPSG','1337','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4363','NZGD2000 (3D)','','geographic 3D','EPSG','6401','EPSG','6167',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3239','geodetic_crs','EPSG','4363','EPSG','1175','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4364','NZGD2000 (geocentric)','','geocentric','EPSG','6500','EPSG','6167',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3240','geodetic_crs','EPSG','4364','EPSG','1175','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4365','POSGAR 98 (3D)','Densification in Argentina of SIRGAS.','geographic 3D','EPSG','6401','EPSG','6190',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3241','geodetic_crs','EPSG','4365','EPSG','1033','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4366','POSGAR 98 (geocentric)','A geodetic network of 127 high accuracy surved points that define the National Geodetic System (Sistema Geodésico Nacional). Densification of SIRGAS 1995.','geocentric','EPSG','6500','EPSG','6190',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3242','geodetic_crs','EPSG','4366','EPSG','1033','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4367','REGVEN (3D)','Densification in Venezuela of SIRGAS.','geographic 3D','EPSG','6401','EPSG','6189',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3243','geodetic_crs','EPSG','4367','EPSG','1251','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4368','REGVEN (geocentric)','Densification in Venezuela of SIRGAS.','geocentric','EPSG','6500','EPSG','6189',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3244','geodetic_crs','EPSG','4368','EPSG','1251','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4369','RGF93 (3D)','','geographic 3D','EPSG','6401','EPSG','6171',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3245','geodetic_crs','EPSG','4369','EPSG','1096','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4370','RGF93 (geocentric)','','geocentric','EPSG','6500','EPSG','6171',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3246','geodetic_crs','EPSG','4370','EPSG','1096','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4371','RGFG95 (3D)','','geographic 3D','EPSG','6401','EPSG','6624',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3247','geodetic_crs','EPSG','4371','EPSG','1097','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4372','RGFG95 (geocentric)','','geocentric','EPSG','6500','EPSG','6624',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3248','geodetic_crs','EPSG','4372','EPSG','1097','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4373','RGR92 (3D)','','geographic 3D','EPSG','6401','EPSG','6627',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3249','geodetic_crs','EPSG','4373','EPSG','1196','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4374','RGR92 (geocentric)','','geocentric','EPSG','6500','EPSG','6627',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3250','geodetic_crs','EPSG','4374','EPSG','1196','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4375','SIRGAS (3D)','','geographic 3D','EPSG','6401','EPSG','6170',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3251','geodetic_crs','EPSG','4375','EPSG','1341','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4376','SIRGAS (geocentric)','','geocentric','EPSG','6500','EPSG','6170',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3252','geodetic_crs','EPSG','4376','EPSG','1341','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4377','SWEREF99 (3D)','','geographic 3D','EPSG','6401','EPSG','6619',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3253','geodetic_crs','EPSG','4377','EPSG','1225','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4378','SWEREF99 (geocentric)','','geocentric','EPSG','6500','EPSG','6619',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3254','geodetic_crs','EPSG','4378','EPSG','1225','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4379','Yemen NGN96 (3D)','','geographic 3D','EPSG','6401','EPSG','6163',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3255','geodetic_crs','EPSG','4379','EPSG','1257','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4380','Yemen NGN96 (geocentric)','','geocentric','EPSG','6500','EPSG','6163',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3256','geodetic_crs','EPSG','4380','EPSG','1257','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4381','RGNC 1991 (3D)','Supersedes older local 2D systems IGN56 Lifou, IGN72 Grande Terre, ST87 Ouvea, IGN53 Mare, ST84 Ile des Pins, ST71 Belep and NEA74 Noumea (CRS codes 4633, 4635, 4650-53 and 4662).','geographic 3D','EPSG','6401','EPSG','6645',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3257','geodetic_crs','EPSG','4381','EPSG','1174','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4382','RGNC 1991 (geocentric)','Supersedes older local geographic 2D systems IGN56 Lifou, IGN72 Grande Terre, ST87 Ouvea, IGN53 Mare, ST84 Ile des Pins, ST71 Belep and NEA74 Noumea (CRS codes 463, 4635, 4650-53 and 4662).','geocentric','EPSG','6500','EPSG','6645',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3258','geodetic_crs','EPSG','4382','EPSG','1174','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4383','RRAF 1991 (3D)','Supersedes older local 2D systems Fort Marigot and Sainte Anne CRS (codes 4621-22) in Guadeloupe and Fort Desaix (CRS code 4625) in Martinique.','geographic 3D','EPSG','6401','EPSG','6640',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3259','geodetic_crs','EPSG','4383','EPSG','2824','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4384','RRAF 1991 (geocentric)','Supersedes older local geographic 2D systems Fort Marigot and Sainte Anne CRS (codes 4621-22) in Guadeloupe and Fort Desaix (CRS code 4625) in Martinique.','geocentric','EPSG','6500','EPSG','6640',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3260','geodetic_crs','EPSG','4384','EPSG','2824','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4385','ITRF2000 (geocentric)','Supersedes ITRF97 (code 4336).','geocentric','EPSG','6500','EPSG','6656',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3261','geodetic_crs','EPSG','4385','EPSG','2830','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4386','ISN93 (3D)','','geographic 3D','EPSG','6401','EPSG','6659',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3262','geodetic_crs','EPSG','4386','EPSG','1120','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4387','ISN93 (geocentric)','','geocentric','EPSG','6500','EPSG','6659',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3263','geodetic_crs','EPSG','4387','EPSG','1120','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4388','LKS92 (3D)','','geographic 3D','EPSG','6401','EPSG','6661',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3264','geodetic_crs','EPSG','4388','EPSG','1139','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4389','LKS92 (geocentric)','','geocentric','EPSG','6500','EPSG','6661',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3265','geodetic_crs','EPSG','4389','EPSG','1139','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4463','RGSPM06','Replaces Saint Pierre et Miquelon 1950 (CRS code 4638). See CRS code 7035 for alternate system with axes reversed used by IGN for GIS purposes.','geographic 2D','EPSG','6422','EPSG','1038',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3319','geodetic_crs','EPSG','4463','EPSG','1220','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','4465','RGSPM06','','geocentric','EPSG','6500','EPSG','1038',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3320','geodetic_crs','EPSG','4465','EPSG','1220','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4466','RGSPM06','See CRS code 7034 for alternate system with horizontal axes reversed used by IGN for GIS purposes.','geographic 3D','EPSG','6423','EPSG','1038',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3321','geodetic_crs','EPSG','4466','EPSG','1220','EPSG','1182');
INSERT INTO "geodetic_crs" VALUES('EPSG','4468','RGM04','Replaced by RGM23 (CRS code 10669) with effect from 2023-01-01.','geocentric','EPSG','6500','EPSG','1036',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3323','geodetic_crs','EPSG','4468','EPSG','1159','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4469','RGM04','Replaced by RGM23 (CRS code 10670) from 2023-01-01. See CRS code 7038 for alternate system with horizontal axes reversed used by IGN for GIS purposes.','geographic 3D','EPSG','6423','EPSG','1036',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3324','geodetic_crs','EPSG','4469','EPSG','1159','EPSG','1182');
INSERT INTO "geodetic_crs" VALUES('EPSG','4470','RGM04','Replaces Combani 1950 (CRS code 4632) except for cadastral purposes which uses Cadastre 1997 (CRS code 4475). Replaced by RGM23 (CRS code 10671) from 2023-01-01. See CRS code 7039 for alternate system with axes reversed used by IGN for GIS purposes.','geographic 2D','EPSG','6422','EPSG','1036',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3325','geodetic_crs','EPSG','4470','EPSG','1159','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','4472','Cadastre 1997','','geographic 3D','EPSG','6423','EPSG','1037',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3327','geodetic_crs','EPSG','4472','EPSG','3340','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4473','Cadastre 1997','','geocentric','EPSG','6500','EPSG','1037',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3328','geodetic_crs','EPSG','4473','EPSG','3340','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4475','Cadastre 1997','Replaces Combani 1950 (CRS code 4632) for cadastral purposes only. For other purposes see RGM04 (CRS code 4470).','geographic 2D','EPSG','6422','EPSG','1037',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3330','geodetic_crs','EPSG','4475','EPSG','3340','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','4479','China Geodetic Coordinate System 2000','','geocentric','EPSG','6500','EPSG','1043',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3331','geodetic_crs','EPSG','4479','EPSG','1067','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4480','China Geodetic Coordinate System 2000','','geographic 3D','EPSG','6423','EPSG','1043',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3332','geodetic_crs','EPSG','4480','EPSG','1067','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4481','Mexico ITRF92','Replaced by Mexico ITRF2008 (CRS code 6363) from December 2010.','geocentric','EPSG','6500','EPSG','1042',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3333','geodetic_crs','EPSG','4481','EPSG','1160','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4482','Mexico ITRF92','Replaced by Mexico ITRF2008 (CRS code 6364) from December 2010.','geographic 3D','EPSG','6423','EPSG','1042',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3334','geodetic_crs','EPSG','4482','EPSG','1160','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4483','Mexico ITRF92','Replaces NAD27 (CRS code 4267). Replaced by Mexico ITRF2008 (CRS code 6365) from December 2010.','geographic 2D','EPSG','6422','EPSG','1042',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3335','geodetic_crs','EPSG','4483','EPSG','1160','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','4490','China Geodetic Coordinate System 2000','Adopted July 2008. Replaces Xian 1980 (CRS code 4610).','geographic 2D','EPSG','6422','EPSG','1043',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3342','geodetic_crs','EPSG','4490','EPSG','1067','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','4555','New Beijing','Replaces Beijing 1954 (CRS code 4214). Replaced by CGCS2000 (code 4490).','geographic 2D','EPSG','6422','EPSG','1045',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3407','geodetic_crs','EPSG','4555','EPSG','3228','EPSG','1153');
INSERT INTO "geodetic_crs" VALUES('EPSG','4556','RRAF 1991','Replaces older local geographic 2D systems Fort Marigot and Sainte Anne CRS (codes 4621-22) in Guadeloupe and Fort Desaix (CRS code 4625) in Martinique. Replaced by RGAF09 (CRS code 5487).','geocentric','EPSG','6500','EPSG','1047',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3408','geodetic_crs','EPSG','4556','EPSG','2824','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4557','RRAF 1991','Replaces older local 2D systems Fort Marigot and Sainte Anne CRS (codes 4621-22) in Guadeloupe and Fort Desaix (CRS code 4625) in Martinique. Replaced by RGAF09 (CRS code 5488).','geographic 3D','EPSG','6423','EPSG','1047',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3409','geodetic_crs','EPSG','4557','EPSG','2824','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4558','RRAF 1991','Replaces older local systems Fort Marigot and Sainte Anne CRS (codes 4621-22) in Guadeloupe and Fort Desaix (CRS code 4625) in Martinique. Replaced by RGAF09 (CRS code 5489).','geographic 2D','EPSG','6422','EPSG','1047',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3410','geodetic_crs','EPSG','4558','EPSG','2824','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','4600','Anguilla 1957','','geographic 2D','EPSG','6422','EPSG','6600',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3434','geodetic_crs','EPSG','4600','EPSG','3214','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4601','Antigua 1943','','geographic 2D','EPSG','6422','EPSG','6601',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3435','geodetic_crs','EPSG','4601','EPSG','1273','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4602','Dominica 1945','','geographic 2D','EPSG','6422','EPSG','6602',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3436','geodetic_crs','EPSG','4602','EPSG','3239','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4603','Grenada 1953','','geographic 2D','EPSG','6422','EPSG','6603',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3437','geodetic_crs','EPSG','4603','EPSG','1551','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4604','Montserrat 1958','','geographic 2D','EPSG','6422','EPSG','6604',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3438','geodetic_crs','EPSG','4604','EPSG','3279','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4605','St. Kitts 1955','','geographic 2D','EPSG','6422','EPSG','6605',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3439','geodetic_crs','EPSG','4605','EPSG','3297','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4606','St. Lucia 1955','','geographic 2D','EPSG','6422','EPSG','6606',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3440','geodetic_crs','EPSG','4606','EPSG','3298','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4607','St. Vincent 1945','','geographic 2D','EPSG','6422','EPSG','6607',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3441','geodetic_crs','EPSG','4607','EPSG','3300','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4608','NAD27(76)','Note: this CRS includes longitudes which are POSITIVE EAST.','geographic 2D','EPSG','6422','EPSG','6608',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3442','geodetic_crs','EPSG','4608','EPSG','1367','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4609','NAD27(CGQ77)','Note: this CRS includes longitudes which are POSITIVE EAST.','geographic 2D','EPSG','6422','EPSG','6609',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3443','geodetic_crs','EPSG','4609','EPSG','1368','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4610','Xian 1980','Replaces Beijing 1954 (CRS code 4214). Replaced by CGCS2000(CRS code 4490).','geographic 2D','EPSG','6422','EPSG','6610',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3444','geodetic_crs','EPSG','4610','EPSG','3228','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4611','Hong Kong 1980','Replaces Hong Kong 1963 and Hong Kong 1963(67).','geographic 2D','EPSG','6422','EPSG','6611',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3445','geodetic_crs','EPSG','4611','EPSG','1118','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4612','JGD2000','Replaces Tokyo (CRS code 4301) from April 2002. From 21st October 2011 replaced by JGD2011 (CRS code 6668).','geographic 2D','EPSG','6422','EPSG','6612',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3446','geodetic_crs','EPSG','4612','EPSG','1129','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','4613','Segara','','geographic 2D','EPSG','6422','EPSG','6613',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3447','geodetic_crs','EPSG','4613','EPSG','1360','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4614','QND95','','geographic 2D','EPSG','6422','EPSG','6614',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3448','geodetic_crs','EPSG','4614','EPSG','1346','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4615','Porto Santo','Replaced by 1995 system (CRS code 4663).','geographic 2D','EPSG','6422','EPSG','6615',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3449','geodetic_crs','EPSG','4615','EPSG','1314','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4616','Selvagem Grande','','geographic 2D','EPSG','6422','EPSG','6616',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3450','geodetic_crs','EPSG','4616','EPSG','2779','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4617','NAD83(CSRS)','Includes all versions of NAD83(CSRS) from v2 [CSRS98] onwards without specific identification. As such it has an accuracy of approximately 1m. Note: this CRS includes longitudes which are POSITIVE EAST.','geographic 2D','EPSG','6422','EPSG','6140',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3451','geodetic_crs','EPSG','4617','EPSG','1061','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','4618','SAD69','Uses GRS 1967 ellipsoid with 1/f to exactly 2 decimal places. In Brazil only, replaced by SAD69(96) (CRS code 5527).','geographic 2D','EPSG','6422','EPSG','6618',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3452','geodetic_crs','EPSG','4618','EPSG','1358','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4619','SWEREF99','','geographic 2D','EPSG','6422','EPSG','6619',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3453','geodetic_crs','EPSG','4619','EPSG','1225','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','4620','Point 58','The 12th parallel traverse of 1966-70 is connected to the Blue Nile 1958 (Adindan) network in western Sudan (geogCRS code 4201).','geographic 2D','EPSG','6422','EPSG','6620',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3454','geodetic_crs','EPSG','4620','EPSG','2790','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4621','Fort Marigot','Replaced by RRAF 1991 (CRS code 4558).','geographic 2D','EPSG','6422','EPSG','6621',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3455','geodetic_crs','EPSG','4621','EPSG','2828','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4622','Guadeloupe 1948','Replaced by RRAF 1991 (CRS code 4558).','geographic 2D','EPSG','6422','EPSG','6622',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3456','geodetic_crs','EPSG','4622','EPSG','2829','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4623','CSG67','','geographic 2D','EPSG','6422','EPSG','6623',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3457','geodetic_crs','EPSG','4623','EPSG','3105','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4624','RGFG95','See CRS code 7041 for alternate system with axes reversed used by IGN for GIS purposes.','geographic 2D','EPSG','6422','EPSG','6624',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3458','geodetic_crs','EPSG','4624','EPSG','1097','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','4625','Martinique 1938','Replaced by RRAF 1991 (CRS code 4558).','geographic 2D','EPSG','6422','EPSG','6625',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3459','geodetic_crs','EPSG','4625','EPSG','3276','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4626','Reunion 1947','Replaced by RGR92 (code 4627).','geographic 2D','EPSG','6422','EPSG','6626',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3460','geodetic_crs','EPSG','4626','EPSG','3337','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4627','RGR92','Replaces Piton des Neiges (code 4626). See CRS code 7037 for alternate system with axes reversed used by IGN for GIS purposes.','geographic 2D','EPSG','6422','EPSG','6627',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3461','geodetic_crs','EPSG','4627','EPSG','3902','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','4628','Tahiti 52','Replaced by Tahiti 79 (CRS code 4690) in Tahiti and Moorea 87 (CRS code 4691) in Moorea.','geographic 2D','EPSG','6422','EPSG','6628',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3462','geodetic_crs','EPSG','4628','EPSG','2811','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4629','Tahaa 54','Replaced by RGPF, CRS code 4687.','geographic 2D','EPSG','6422','EPSG','6629',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3463','geodetic_crs','EPSG','4629','EPSG','2812','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4630','IGN72 Nuku Hiva','Replaced by RGPF, CRS code 4687.','geographic 2D','EPSG','6422','EPSG','6630',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3464','geodetic_crs','EPSG','4630','EPSG','3129','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4631','K0 1949','','geographic 2D','EPSG','6422','EPSG','6631',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3465','geodetic_crs','EPSG','4631','EPSG','2816','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4632','Combani 1950','Replaced by Cadastre 1997 (CRS code 4475) for cadastral purposes only and by RGM04 (CRS code 4470) for all other purposes.','geographic 2D','EPSG','6422','EPSG','6632',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3466','geodetic_crs','EPSG','4632','EPSG','3340','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4633','IGN56 Lifou','Replaced by RGNC91-93 (CRS code 4749).','geographic 2D','EPSG','6422','EPSG','6633',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3467','geodetic_crs','EPSG','4633','EPSG','2814','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4634','IGN72 Grand Terre','Superseded by RGNC 1991 (CRS code 4645).','geographic 2D','EPSG','6402','EPSG','6634',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3468','geodetic_crs','EPSG','4634','EPSG','2822','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4635','ST87 Ouvea','Superseded by RGNC 1991 (CRS code 4645).','geographic 2D','EPSG','6422','EPSG','6635',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3469','geodetic_crs','EPSG','4635','EPSG','2813','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4636','Petrels 1972','Replaced by RGTAAF07 (CRS code 7073).','geographic 2D','EPSG','6422','EPSG','6636',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3470','geodetic_crs','EPSG','4636','EPSG','2817','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4637','Perroud 1950','Replaced by RGTAAF07 (CRS code 7073).','geographic 2D','EPSG','6422','EPSG','6637',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3471','geodetic_crs','EPSG','4637','EPSG','2818','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4638','Saint Pierre et Miquelon 1950','Replaced by RGSPM06 (CRS code 4463).','geographic 2D','EPSG','6422','EPSG','6638',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3472','geodetic_crs','EPSG','4638','EPSG','3299','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4639','MOP78','Replaced by RGWF96 (CRS code 8900) for geodetic survey and RGWF96 (lon-lat) (CRS code 8902) for GIS.','geographic 2D','EPSG','6422','EPSG','6639',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3473','geodetic_crs','EPSG','4639','EPSG','2815','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4640','RRAF 1991','Replaces older local systems Fort Marigot and Sainte Anne CRS (codes 4621-22) in Guadeloupe and Fort Desaix (CRS code 4625) in Martinique.','geographic 2D','EPSG','6422','EPSG','6640',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3474','geodetic_crs','EPSG','4640','EPSG','2824','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','4641','IGN53 Mare','Replaced by RGNC91-93 (CRS code 4749).','geographic 2D','EPSG','6422','EPSG','6641',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3475','geodetic_crs','EPSG','4641','EPSG','2819','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4642','ST84 Ile des Pins','Replaced by RGNC91-93 (CRS code 4749).','geographic 2D','EPSG','6422','EPSG','6642',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3476','geodetic_crs','EPSG','4642','EPSG','2820','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4643','ST71 Belep','Replaced by RGNC91-93 (CRS code 4749).','geographic 2D','EPSG','6422','EPSG','6643',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3477','geodetic_crs','EPSG','4643','EPSG','2821','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4644','NEA74 Noumea','Replaced by RGNC91-93 (CRS code 4749).','geographic 2D','EPSG','6422','EPSG','6644',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3478','geodetic_crs','EPSG','4644','EPSG','2823','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4645','RGNC 1991','Supersedes older local systems IGN56 Lifou, IGN72 Grande Terre, ST87 Ouvea, IGN53 Mare, ST84 Ile des Pins, ST71 Belep and NEA74 Noumea (CRS codes 4633, 4635, 4641-44 and 4662).','geographic 2D','EPSG','6422','EPSG','6645',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3479','geodetic_crs','EPSG','4645','EPSG','1174','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','4646','Grand Comoros','','geographic 2D','EPSG','6422','EPSG','6646',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3480','geodetic_crs','EPSG','4646','EPSG','2807','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4657','Reykjavik 1900','See ellipsoid remarks.','geographic 2D','EPSG','6422','EPSG','6657',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3487','geodetic_crs','EPSG','4657','EPSG','3262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4658','Hjorsey 1955','','geographic 2D','EPSG','6422','EPSG','6658',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3488','geodetic_crs','EPSG','4658','EPSG','3262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4659','ISN93','Replaced by ISN2004 (CRS code 5324).','geographic 2D','EPSG','6422','EPSG','6659',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3489','geodetic_crs','EPSG','4659','EPSG','1120','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','4660','Helle 1954','','geographic 2D','EPSG','6422','EPSG','6660',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3490','geodetic_crs','EPSG','4660','EPSG','2869','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4661','LKS-92','Replaced by LKS-2020 (CRS code 10305) from 2026-10-01.','geographic 2D','EPSG','6422','EPSG','6661',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3491','geodetic_crs','EPSG','4661','EPSG','1139','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','4662','IGN72 Grande Terre','Replaced by RGNC91-93 (CRS code 4749).','geographic 2D','EPSG','6422','EPSG','6634',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3492','geodetic_crs','EPSG','4662','EPSG','2822','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4663','Porto Santo 1995','Replaces 1936 system (CRS code 4615). Replaced by PTRA08 (CRS code 5013).','geographic 2D','EPSG','6422','EPSG','6663',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3493','geodetic_crs','EPSG','4663','EPSG','1314','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4664','Azores Oriental 1995','Replaces 1948 system (CRS code 4184). Replaced by PTRA08 (CRS code 5013).','geographic 2D','EPSG','6422','EPSG','6664',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3494','geodetic_crs','EPSG','4664','EPSG','1345','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4665','Azores Central 1995','Replaces 1948 system (CRS code 4183). Replaced by PTRA08 (CRS code 5013).','geographic 2D','EPSG','6422','EPSG','6665',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3495','geodetic_crs','EPSG','4665','EPSG','1301','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4666','Lisbon 1890','Replaced by Lisbon 1937 system which uses International 1924 ellipsoid (code 4207).','geographic 2D','EPSG','6422','EPSG','6666',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3496','geodetic_crs','EPSG','4666','EPSG','1294','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4667','IKBD-92','','geographic 2D','EPSG','6422','EPSG','6667',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3497','geodetic_crs','EPSG','4667','EPSG','2876','EPSG','1053');
INSERT INTO "geodetic_crs" VALUES('EPSG','4668','ED79','','geographic 2D','EPSG','6422','EPSG','6668',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3498','geodetic_crs','EPSG','4668','EPSG','1297','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4669','LKS94','','geographic 2D','EPSG','6422','EPSG','6126',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3499','geodetic_crs','EPSG','4669','EPSG','1145','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','4670','IGM95','Replaced by RDN2008 (CRS code 6706) from 2011-11-10.','geographic 2D','EPSG','6422','EPSG','6670',NULL,0);
INSERT INTO "usage" VALUES('EPSG','14404','geodetic_crs','EPSG','4670','EPSG','3343','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','4671','Voirol 1879','The appropriate usage of CRSs using the Voirol 1875 and 1879 datums is lost in antiquity. They differ by about 9 metres. Oil industry references to one could in reality be to either. All replaced by Nord Sahara 1959 (CRS code 4307).','geographic 2D','EPSG','6422','EPSG','6671',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3501','geodetic_crs','EPSG','4671','EPSG','1365','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4672','Chatham Islands 1971','Replaced by CI1979.','geographic 2D','EPSG','6422','EPSG','6672',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3502','geodetic_crs','EPSG','4672','EPSG','2889','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4673','Chatham Islands 1979','Replaces CI1971. Replaced by NZGD2000 from March 2000.','geographic 2D','EPSG','6422','EPSG','6673',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3503','geodetic_crs','EPSG','4673','EPSG','2889','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4674','SIRGAS 2000','Replaces SIRGAS 1995 system (CRS code 4179) for South America; expands SIRGAS to Central America.','geographic 2D','EPSG','6422','EPSG','6674',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3504','geodetic_crs','EPSG','4674','EPSG','3418','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','4675','Guam 1963','Replaced by NAD83(HARN) alias Guam Geodetic Network 1993 (CRS code 4152) from 1995.','geographic 2D','EPSG','6422','EPSG','6675',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3505','geodetic_crs','EPSG','4675','EPSG','4525','EPSG','1056');
INSERT INTO "geodetic_crs" VALUES('EPSG','4676','Vientiane 1982','Replaced by Lao 1993 and then by Lao 1997. Vientiane 1982 coordinate values are within 3m of Lao 1997 values.','geographic 2D','EPSG','6422','EPSG','6676',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3506','geodetic_crs','EPSG','4676','EPSG','1138','EPSG','1210');
INSERT INTO "geodetic_crs" VALUES('EPSG','4677','Lao 1993','Replaces Vientiane 1982. Replaced by Lao 1997. Lao 1993 coordinate values are within 1m of Lao 1997 values.','geographic 2D','EPSG','6422','EPSG','6677',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3507','geodetic_crs','EPSG','4677','EPSG','1138','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','4678','Lao 1997','Replaces Lao 1993 which in turn replaced Vientiane 1982. Lao 1993 coordinate values are within 1m of Lao 1997 values. Vientiane 1982 coordinate values are within 3m of Lao 1997 values.','geographic 2D','EPSG','6422','EPSG','6678',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3508','geodetic_crs','EPSG','4678','EPSG','1138','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','4679','Jouik 1961','Replaced by Mauritania 1999 (CRS code 4702).','geographic 2D','EPSG','6422','EPSG','6679',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3509','geodetic_crs','EPSG','4679','EPSG','2967','EPSG','1198');
INSERT INTO "geodetic_crs" VALUES('EPSG','4680','Nouakchott 1965','Replaced by Mauritania 1999 (CRS code 4702).','geographic 2D','EPSG','6422','EPSG','6680',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3510','geodetic_crs','EPSG','4680','EPSG','2968','EPSG','1198');
INSERT INTO "geodetic_crs" VALUES('EPSG','4681','Mauritania 1999','Mining title descriptions referring to "Clarke 1880 ellipsoid" should be assumed to be referenced to this CRS. Supersedes all earlier local systems.','geographic 2D','EPSG','6422','EPSG','6681',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3511','geodetic_crs','EPSG','4681','EPSG','1157','EPSG','1249');
INSERT INTO "geodetic_crs" VALUES('EPSG','4682','Gulshan 303','','geographic 2D','EPSG','6422','EPSG','6682',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3512','geodetic_crs','EPSG','4682','EPSG','1041','EPSG','1181');
INSERT INTO "geodetic_crs" VALUES('EPSG','4683','PRS92','Replaces Luzon 19111 (CRS code 4253).','geographic 2D','EPSG','6422','EPSG','6683',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3513','geodetic_crs','EPSG','4683','EPSG','1190','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','4684','Gan 1970','In some references incorrectly named "Gandajika 1970". See CRS "Gandajika", code 4685, from the Democratic Republic of the Congo (Zaire).','geographic 2D','EPSG','6422','EPSG','6684',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3514','geodetic_crs','EPSG','4684','EPSG','3274','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4685','Gandajika','In some references incorrectly attributed to the Maldives. See CRS "Gan 1970", code 4684.','geographic 2D','EPSG','6422','EPSG','6685',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3515','geodetic_crs','EPSG','4685','EPSG','1259','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4686','MAGNA-SIRGAS','Replaces Bogota 1975 (CRS code 4218). For high accuracy purposes replaced by MAGNA-SIRGAS 2018 (code 20046).','geographic 2D','EPSG','6422','EPSG','6686',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3516','geodetic_crs','EPSG','4686','EPSG','1070','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','4687','RGPF','Replaces Tahaa 54 (CRS code 4629), IGN 63 Hiva Oa (4689), IGN 72 Nuku Hiva (4630), Maupiti 83 (4692), MHEFO 55 (4688), Moorea 87 (4691) and Tahiti 79 (4690).','geographic 2D','EPSG','6422','EPSG','6687',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3517','geodetic_crs','EPSG','4687','EPSG','1098','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','4688','Fatu Iva 72','Recomputed by IGN in 1972 using origin and observations of 1953-1955 Mission Hydrographique des Etablissement Francais d''Oceanie (MHEFO 55). Replaced by RGPF, CRS code 4687.','geographic 2D','EPSG','6422','EPSG','6688',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3518','geodetic_crs','EPSG','4688','EPSG','3133','EPSG','1201');
INSERT INTO "geodetic_crs" VALUES('EPSG','4689','IGN63 Hiva Oa','Replaced by RGPF, CRS code 4687.','geographic 2D','EPSG','6422','EPSG','6689',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3519','geodetic_crs','EPSG','4689','EPSG','3130','EPSG','1201');
INSERT INTO "geodetic_crs" VALUES('EPSG','4690','Tahiti 79','Replaces Tahiti 52 (CRS code 4628) in Tahiti. Replaced by RGPF (CRS code 4687).','geographic 2D','EPSG','6422','EPSG','6690',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3520','geodetic_crs','EPSG','4690','EPSG','3124','EPSG','1201');
INSERT INTO "geodetic_crs" VALUES('EPSG','4691','Moorea 87','Replaces Tahiti 52 (CRS code 4628) in Moorea. Replaced by RGPF (CRS code 4687).','geographic 2D','EPSG','6422','EPSG','6691',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3521','geodetic_crs','EPSG','4691','EPSG','3125','EPSG','1201');
INSERT INTO "geodetic_crs" VALUES('EPSG','4692','Maupiti 83','Replaced by RGPF, CRS code 4687.','geographic 2D','EPSG','6422','EPSG','6692',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3522','geodetic_crs','EPSG','4692','EPSG','3126','EPSG','1201');
INSERT INTO "geodetic_crs" VALUES('EPSG','4693','Nakhl-e Ghanem','','geographic 2D','EPSG','6422','EPSG','6693',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3523','geodetic_crs','EPSG','4693','EPSG','2362','EPSG','1140');
INSERT INTO "geodetic_crs" VALUES('EPSG','4694','POSGAR 94','Legally adopted in May 1997. Replaced by POSGAR 98 for scientific and many practical purposes until May 2009. Officially replaced by POSGAR 2007 in May 2009.','geographic 2D','EPSG','6422','EPSG','6694',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3524','geodetic_crs','EPSG','4694','EPSG','1033','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','4695','Katanga 1955','','geographic 2D','EPSG','6422','EPSG','6695',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3525','geodetic_crs','EPSG','4695','EPSG','3147','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4696','Kasai 1953','','geographic 2D','EPSG','6422','EPSG','6696',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3526','geodetic_crs','EPSG','4696','EPSG','3148','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4697','IGC 1962 6th Parallel South','','geographic 2D','EPSG','6422','EPSG','6697',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3527','geodetic_crs','EPSG','4697','EPSG','3149','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4698','IGN 1962 Kerguelen','Replaced by RGTAAF07 (CRS code 7073).','geographic 2D','EPSG','6422','EPSG','6698',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3528','geodetic_crs','EPSG','4698','EPSG','2816','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4699','Le Pouce 1934','Densified with a GPS-derived coordinate set for 80 stations in 1994. This 1994 coordinate set is sometimes referred to as "Mauritius 1994".','geographic 2D','EPSG','6422','EPSG','6699',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3529','geodetic_crs','EPSG','4699','EPSG','3209','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4700','IGN Astro 1960','Mining title descriptions referring only to "Clarke 1880 ellipsoid" should be assumed to be referenced to this CRS. Oil industry considers Mining Cadastre 1999 to be exactly defined through tfm codes 15857-9. Replaced by Mauritania 1999 (code 4702).','geographic 2D','EPSG','6422','EPSG','6700',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3530','geodetic_crs','EPSG','4700','EPSG','3277','EPSG','1249');
INSERT INTO "geodetic_crs" VALUES('EPSG','4701','IGCB 1955','Replaced by IGC 1962 Arc of the 6th Parallel South, except for oil industry activities.','geographic 2D','EPSG','6422','EPSG','6701',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3531','geodetic_crs','EPSG','4701','EPSG','3171','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4702','Mauritania 1999','Replaces all earlier CRSs.','geographic 2D','EPSG','6422','EPSG','6702',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3532','geodetic_crs','EPSG','4702','EPSG','1157','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','4703','Mhast 1951','A variation of this system has been adopted by the oil industry but using the International 1924 ellipsoid - see Mhast (onshore) and Mhast (offshore) (codes 4704 and 4705).','geographic 2D','EPSG','6422','EPSG','6703',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3533','geodetic_crs','EPSG','4703','EPSG','1318','EPSG','1105');
INSERT INTO "geodetic_crs" VALUES('EPSG','4704','Mhast (onshore)','Adopted by CABGOC with intention of being Mhast 1951 (CRS code 4703) but because it uses a different ellipsoid it is a different system. From 1979, for offshore use replaced by Mhast (offshore) (CRS code 4705) from which this CRS differs by approximately 10m.','geographic 2D','EPSG','6422','EPSG','6704',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3534','geodetic_crs','EPSG','4704','EPSG','3179','EPSG','1136');
INSERT INTO "geodetic_crs" VALUES('EPSG','4705','Mhast (offshore)','Used by CABGOC. Differs from Mhast (onshore) by approximately 10m. Replaced by Malongo 1987 (CRS code 4259) in 1987.','geographic 2D','EPSG','6422','EPSG','6705',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3535','geodetic_crs','EPSG','4705','EPSG','3180','EPSG','1136');
INSERT INTO "geodetic_crs" VALUES('EPSG','4706','Egypt Gulf of Suez S-650 TL','Differs from Egypt 1907 (CRS code 4229) by approximately 20m.','geographic 2D','EPSG','6422','EPSG','6706',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3536','geodetic_crs','EPSG','4706','EPSG','2341','EPSG','1136');
INSERT INTO "geodetic_crs" VALUES('EPSG','4707','Tern Island 1961','','geographic 2D','EPSG','6422','EPSG','6707',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3537','geodetic_crs','EPSG','4707','EPSG','3181','EPSG','1160');
INSERT INTO "geodetic_crs" VALUES('EPSG','4708','Cocos Islands 1965','','geographic 2D','EPSG','6422','EPSG','6708',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3538','geodetic_crs','EPSG','4708','EPSG','1069','EPSG','1160');
INSERT INTO "geodetic_crs" VALUES('EPSG','4709','Iwo Jima 1945','','geographic 2D','EPSG','6422','EPSG','6709',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3539','geodetic_crs','EPSG','4709','EPSG','3200','EPSG','1160');
INSERT INTO "geodetic_crs" VALUES('EPSG','4710','Astro DOS 71','Used between 1972 and 2015. Replaced by SHGD2015 (CRS code 7886) from 2015.','geographic 2D','EPSG','6422','EPSG','6710',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3540','geodetic_crs','EPSG','4710','EPSG','3183','EPSG','1180');
INSERT INTO "geodetic_crs" VALUES('EPSG','4711','Marcus Island 1952','','geographic 2D','EPSG','6422','EPSG','6711',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3541','geodetic_crs','EPSG','4711','EPSG','1872','EPSG','1160');
INSERT INTO "geodetic_crs" VALUES('EPSG','4712','Ascension Island 1958','','geographic 2D','EPSG','6422','EPSG','6712',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3542','geodetic_crs','EPSG','4712','EPSG','3182','EPSG','1160');
INSERT INTO "geodetic_crs" VALUES('EPSG','4713','Ayabelle Lighthouse','','geographic 2D','EPSG','6422','EPSG','6713',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3543','geodetic_crs','EPSG','4713','EPSG','1081','EPSG','1160');
INSERT INTO "geodetic_crs" VALUES('EPSG','4714','Bellevue','','geographic 2D','EPSG','6422','EPSG','6714',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3544','geodetic_crs','EPSG','4714','EPSG','3193','EPSG','1160');
INSERT INTO "geodetic_crs" VALUES('EPSG','4715','Camp Area Astro','Replaced by RSRGD2000 (CRS code 4764). The relationship to this is variable. See Land Information New Zealand LINZS25001.','geographic 2D','EPSG','6422','EPSG','6715',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3545','geodetic_crs','EPSG','4715','EPSG','3205','EPSG','1160');
INSERT INTO "geodetic_crs" VALUES('EPSG','4716','Phoenix Islands 1966','','geographic 2D','EPSG','6422','EPSG','6716',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3546','geodetic_crs','EPSG','4716','EPSG','3196','EPSG','1160');
INSERT INTO "geodetic_crs" VALUES('EPSG','4717','Cape Canaveral','','geographic 2D','EPSG','6422','EPSG','6717',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3547','geodetic_crs','EPSG','4717','EPSG','3206','EPSG','1233');
INSERT INTO "geodetic_crs" VALUES('EPSG','4718','Solomon 1968','','geographic 2D','EPSG','6422','EPSG','6718',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3548','geodetic_crs','EPSG','4718','EPSG','1213','EPSG','1160');
INSERT INTO "geodetic_crs" VALUES('EPSG','4719','Easter Island 1967','','geographic 2D','EPSG','6422','EPSG','6719',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3549','geodetic_crs','EPSG','4719','EPSG','3188','EPSG','1160');
INSERT INTO "geodetic_crs" VALUES('EPSG','4720','Fiji 1986','Replaces Viti Levu 1912 (CRS code 4752), Vanua Levu 1915 (CRS code 4748) and Fiji 1956 (CRS code 4721).','geographic 2D','EPSG','6422','EPSG','6720',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3550','geodetic_crs','EPSG','4720','EPSG','1094','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4721','Fiji 1956','For topographic mapping replaces Viti Levu 1912 (CRS code 4752) and Vanua Levu 1915 (CRS code 4748). Replaced by Fiji 1986 (CRS code 4720).','geographic 2D','EPSG','6422','EPSG','6721',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3551','geodetic_crs','EPSG','4721','EPSG','3398','EPSG','1160');
INSERT INTO "geodetic_crs" VALUES('EPSG','4722','South Georgia 1968','','geographic 2D','EPSG','6422','EPSG','6722',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3552','geodetic_crs','EPSG','4722','EPSG','3529','EPSG','1160');
INSERT INTO "geodetic_crs" VALUES('EPSG','4723','GCGD59','Superseded by CIGD11 (CRS code 6135).','geographic 2D','EPSG','6422','EPSG','6723',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3553','geodetic_crs','EPSG','4723','EPSG','3185','EPSG','1056');
INSERT INTO "geodetic_crs" VALUES('EPSG','4724','Diego Garcia 1969','','geographic 2D','EPSG','6422','EPSG','6724',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3554','geodetic_crs','EPSG','4724','EPSG','3189','EPSG','1160');
INSERT INTO "geodetic_crs" VALUES('EPSG','4725','Johnston Island 1961','','geographic 2D','EPSG','6422','EPSG','6725',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3555','geodetic_crs','EPSG','4725','EPSG','3201','EPSG','1160');
INSERT INTO "geodetic_crs" VALUES('EPSG','4726','SIGD61','Superseded by CIGD11 (CRS code 6135).','geographic 2D','EPSG','6422','EPSG','6726',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3556','geodetic_crs','EPSG','4726','EPSG','3186','EPSG','1056');
INSERT INTO "geodetic_crs" VALUES('EPSG','4727','Midway 1961','','geographic 2D','EPSG','6422','EPSG','6727',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3557','geodetic_crs','EPSG','4727','EPSG','3202','EPSG','1160');
INSERT INTO "geodetic_crs" VALUES('EPSG','4728','PN84','Replaces PN68 (CRS code 9403) only on western islands (El Hierro, La Gomera, La Palma and Tenerife). Replaced by REGCAN95 (CRS code 4081).','geographic 2D','EPSG','6422','EPSG','6728',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3558','geodetic_crs','EPSG','4728','EPSG','4598','EPSG','1160');
INSERT INTO "geodetic_crs" VALUES('EPSG','4729','Pitcairn 1967','Replced by Pitcairn 2006 (CRS code 4763).','geographic 2D','EPSG','6422','EPSG','6729',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3559','geodetic_crs','EPSG','4729','EPSG','3208','EPSG','1160');
INSERT INTO "geodetic_crs" VALUES('EPSG','4730','Santo 1965','','geographic 2D','EPSG','6422','EPSG','6730',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3560','geodetic_crs','EPSG','4730','EPSG','3194','EPSG','1160');
INSERT INTO "geodetic_crs" VALUES('EPSG','4731','Viti Levu 1916','','geographic 2D','EPSG','6422','EPSG','6731',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3561','geodetic_crs','EPSG','4731','EPSG','3195','EPSG','1160');
INSERT INTO "geodetic_crs" VALUES('EPSG','4732','Marshall Islands 1960','','geographic 2D','EPSG','6422','EPSG','6732',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3562','geodetic_crs','EPSG','4732','EPSG','3191','EPSG','1160');
INSERT INTO "geodetic_crs" VALUES('EPSG','4733','Wake Island 1952','','geographic 2D','EPSG','6422','EPSG','6733',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3563','geodetic_crs','EPSG','4733','EPSG','3190','EPSG','1160');
INSERT INTO "geodetic_crs" VALUES('EPSG','4734','Tristan 1968','','geographic 2D','EPSG','6422','EPSG','6734',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3564','geodetic_crs','EPSG','4734','EPSG','3184','EPSG','1160');
INSERT INTO "geodetic_crs" VALUES('EPSG','4735','Kusaie 1951','','geographic 2D','EPSG','6422','EPSG','6735',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3565','geodetic_crs','EPSG','4735','EPSG','3192','EPSG','1160');
INSERT INTO "geodetic_crs" VALUES('EPSG','4736','Deception Island','','geographic 2D','EPSG','6422','EPSG','6736',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3566','geodetic_crs','EPSG','4736','EPSG','3204','EPSG','1160');
INSERT INTO "geodetic_crs" VALUES('EPSG','4737','KGD2002','','geographic 2D','EPSG','6422','EPSG','6737',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3567','geodetic_crs','EPSG','4737','EPSG','1135','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','4738','Hong Kong 1963','Replaced by Hong Kong 1963(67) (CRS code 4839) for military purposes only. For all purposes, replaced by Hong Kong 1980 (CRS code 4611).','geographic 2D','EPSG','6422','EPSG','6738',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3568','geodetic_crs','EPSG','4738','EPSG','1118','EPSG','1153');
INSERT INTO "geodetic_crs" VALUES('EPSG','4739','Hong Kong 1963(67)','For military purposes only, replaces Hong Kong 1963. Replaced by Hong Kong 1980 (CRS code 4611).','geographic 2D','EPSG','6422','EPSG','6739',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3569','geodetic_crs','EPSG','4739','EPSG','1118','EPSG','1160');
INSERT INTO "geodetic_crs" VALUES('EPSG','4740','PZ-90','Used by the Glonass satellite navigation system prior to 2007-09-20.','geographic 2D','EPSG','6422','EPSG','6740',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3570','geodetic_crs','EPSG','4740','EPSG','1262','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','4741','FD54','Except for cadastral survey, replaced by ED50 in the late 1970''s. For cadastral survey, replaced by fk89 (CRS code 4753).','geographic 2D','EPSG','6422','EPSG','6741',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3571','geodetic_crs','EPSG','4741','EPSG','3248','EPSG','1181');
INSERT INTO "geodetic_crs" VALUES('EPSG','4742','GDM2000','Replaces all earlier Malaysian geographic CRSs.','geographic 2D','EPSG','6422','EPSG','6742',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3572','geodetic_crs','EPSG','4742','EPSG','1151','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','4743','Karbala 1979','Geodetic network established by Polservice consortium. Replaces Nahrwan 1934 (CRS code 4744). Replaced by IGRS (CRS code 3889). At time of record population, information regarding usage within oil sector is not available.','geographic 2D','EPSG','6422','EPSG','6743',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3573','geodetic_crs','EPSG','4743','EPSG','3625','EPSG','1178');
INSERT INTO "geodetic_crs" VALUES('EPSG','4744','Nahrwan 1934','In Iran, replaced by FD58. In Iraq, replaced by Karbala 1979.','geographic 2D','EPSG','6422','EPSG','6744',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3574','geodetic_crs','EPSG','4744','EPSG','4238','EPSG','1136');
INSERT INTO "geodetic_crs" VALUES('EPSG','4745','RD/83','Consistent with DHDN (CRS code 4314) at the 1-metre level. For low accuracy applications RD/83 can be considered the same as DHDN.','geographic 2D','EPSG','6422','EPSG','6745',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3575','geodetic_crs','EPSG','4745','EPSG','2545','EPSG','1091');
INSERT INTO "geodetic_crs" VALUES('EPSG','4746','PD/83','Consistent with DHDN (CRS code 4314) at the 1-metre level. For low accuracy applications PD/83 can be considered the same as DHDN.','geographic 2D','EPSG','6422','EPSG','6746',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3576','geodetic_crs','EPSG','4746','EPSG','2544','EPSG','1091');
INSERT INTO "geodetic_crs" VALUES('EPSG','4747','GR96','Replaces all earlier Greenland geographic CRSs (Ammassalik 1958, Qoornoq 1927 and Scoresbysund 1952).','geographic 2D','EPSG','6422','EPSG','1421',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3577','geodetic_crs','EPSG','4747','EPSG','1107','EPSG','1056');
INSERT INTO "geodetic_crs" VALUES('EPSG','4748','Vanua Levu 1915','For topographic mapping, replaced by Fiji 1956 (CRS code 4721). For other purposes, replaced by Fiji 1986 (CRS code 4720).','geographic 2D','EPSG','6422','EPSG','6748',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3578','geodetic_crs','EPSG','4748','EPSG','3401','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4749','RGNC91-93','Replaces older systems IGN56 Lifou, IGN72 Grande Terre, ST87 Ouvea, IGN53 Mare, ST84 Ile des Pins, ST71 Belep and NEA74 Noumea. Replaced by RGNC15 (CRS 10310). See CRS code 10307 for alternate system with axes reversed used by DITTT for GIS purposes.','geographic 2D','EPSG','6422','EPSG','6749',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3579','geodetic_crs','EPSG','4749','EPSG','1174','EPSG','1182');
INSERT INTO "geodetic_crs" VALUES('EPSG','4750','ST87 Ouvea','Replaced by RGNC91-93 (CRS code 4749).','geographic 2D','EPSG','6422','EPSG','6750',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3580','geodetic_crs','EPSG','4750','EPSG','2813','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4751','Kertau (RSO)','Used only for metrication of RSO grid. See Kertau 1968 (CRS code 4245) for other purposes. Replaced by GDM2000 (CRS code 4742).','geographic 2D','EPSG','6422','EPSG','6751',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3581','geodetic_crs','EPSG','4751','EPSG','1309','EPSG','1250');
INSERT INTO "geodetic_crs" VALUES('EPSG','4752','Viti Levu 1912','For topographic mapping, replaced by Fiji 1956 (CRS code 4721). For other purposes, replaced by Fiji 1986 (CRS code 4720).','geographic 2D','EPSG','6422','EPSG','6752',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3582','geodetic_crs','EPSG','4752','EPSG','3195','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4753','fk89','Replaces FD54 (CRS code 4741). Coordinate differences are less than 0.05 seconds (2m). The name of this system is also used for the dependent projected CRS - see CRS code 3173.','geographic 2D','EPSG','6422','EPSG','6753',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3583','geodetic_crs','EPSG','4753','EPSG','3248','EPSG','1028');
INSERT INTO "geodetic_crs" VALUES('EPSG','4754','LGD2006','Replaces ELD79.','geographic 2D','EPSG','6422','EPSG','6754',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3584','geodetic_crs','EPSG','4754','EPSG','1143','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','4755','DGN95','Replaces ID74.','geographic 2D','EPSG','6422','EPSG','6755',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3585','geodetic_crs','EPSG','4755','EPSG','1122','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','4756','VN-2000','Replaces Hanoi 1972 (CRS code 4147).','geographic 2D','EPSG','6422','EPSG','6756',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3586','geodetic_crs','EPSG','4756','EPSG','3328','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4757','SVY21','','geographic 2D','EPSG','6422','EPSG','6757',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3587','geodetic_crs','EPSG','4757','EPSG','1210','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4758','JAD2001','Replaces JAD69 (CRS code 4242).','geographic 2D','EPSG','6422','EPSG','6758',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3588','geodetic_crs','EPSG','4758','EPSG','1128','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','4759','NAD83(NSRS2007)','Note: this CRS includes POSITIVE EAST longitudes. Replaces NAD83(HARN) and NAD83(FBN). Replaced by NAD83(2011).','geographic 2D','EPSG','6422','EPSG','6759',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3589','geodetic_crs','EPSG','4759','EPSG','1511','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','4760','WGS 66','Replaced by WGS 72.','geographic 2D','EPSG','6422','EPSG','6760',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3590','geodetic_crs','EPSG','4760','EPSG','1262','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','4761','HTRS96','','geographic 2D','EPSG','6422','EPSG','6761',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3591','geodetic_crs','EPSG','4761','EPSG','1076','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','4762','BDA2000','Replaces Bermuda 1957 (CRS code 4216).','geographic 2D','EPSG','6422','EPSG','6762',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3592','geodetic_crs','EPSG','4762','EPSG','1047','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','4763','Pitcairn 2006','Replaces Pitcairn 1967 (CRS code 4729). For practical purposes may be considered to be WGS 84.','geographic 2D','EPSG','6422','EPSG','6763',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3593','geodetic_crs','EPSG','4763','EPSG','3208','EPSG','1091');
INSERT INTO "geodetic_crs" VALUES('EPSG','4764','RSRGD2000','Replaces Camp Area Astro (CRS code 4715). The relationship to this is variable. See Land Information New Zealand LINZS25001.','geographic 2D','EPSG','6422','EPSG','6764',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3594','geodetic_crs','EPSG','4764','EPSG','3558','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','4765','Slovenia 1996','Replaces MG! alias D48 (CRS code 4312).','geographic 2D','EPSG','6422','EPSG','6765',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3595','geodetic_crs','EPSG','4765','EPSG','1212','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','4801','CH1903 (Bern)','Replaced by CH1903 (CRS code 4149).','geographic 2D','EPSG','6422','EPSG','6801',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3631','geodetic_crs','EPSG','4801','EPSG','1286','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4802','Bogota 1975 (Bogota)','Replaces earlier 3 adjustments of 1951, 1944 and 1941.','geographic 2D','EPSG','6422','EPSG','6802',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3632','geodetic_crs','EPSG','4802','EPSG','3229','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4803','Lisbon (Lisbon)','Replaces Lisbon 1890 (Lisbon) system which used Bessel 1841 ellipsoid (code 4904). Replaced by Datum 73 (code 4274).','geographic 2D','EPSG','6422','EPSG','6803',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3633','geodetic_crs','EPSG','4803','EPSG','1294','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4804','Makassar (Jakarta)','','geographic 2D','EPSG','6422','EPSG','6804',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3634','geodetic_crs','EPSG','4804','EPSG','1316','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4805','MGI (Ferro)','Replaced by MGI (CRS code 4312) in Austria and MGI 1901 (CRS code 3906) in former Yugoslavia.','geographic 2D','EPSG','6422','EPSG','6805',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3635','geodetic_crs','EPSG','4805','EPSG','1321','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4806','Monte Mario (Rome)','','geographic 2D','EPSG','6422','EPSG','6806',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3636','geodetic_crs','EPSG','4806','EPSG','3343','EPSG','1188');
INSERT INTO "geodetic_crs" VALUES('EPSG','4807','NTF (Paris)','','geographic 2D','EPSG','6403','EPSG','6807',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3637','geodetic_crs','EPSG','4807','EPSG','3694','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4808','Padang (Jakarta)','','geographic 2D','EPSG','6422','EPSG','6808',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3638','geodetic_crs','EPSG','4808','EPSG','1355','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4809','BD50 (Brussels)','','geographic 2D','EPSG','6422','EPSG','6809',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3639','geodetic_crs','EPSG','4809','EPSG','1347','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4810','Tananarive (Paris)','','geographic 2D','EPSG','6403','EPSG','6810',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3640','geodetic_crs','EPSG','4810','EPSG','3273','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4811','Voirol 1875 (Paris)','The appropriate usage of CRSs using the Voirol 1875 and 1879 datums is lost in antiquity. They differ by about 9 metres. Oil industry references to one could in reality be to either. All replaced by Nord Sahara 1959 (CRS code 4307).','geographic 2D','EPSG','6403','EPSG','6811',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3641','geodetic_crs','EPSG','4811','EPSG','1365','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4813','Batavia (Jakarta)','','geographic 2D','EPSG','6422','EPSG','6813',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3643','geodetic_crs','EPSG','4813','EPSG','1285','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4814','RT38 (Stockholm)','','geographic 2D','EPSG','6422','EPSG','6814',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3644','geodetic_crs','EPSG','4814','EPSG','3313','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4815','Greek (Athens)','','geographic 2D','EPSG','6422','EPSG','6815',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3645','geodetic_crs','EPSG','4815','EPSG','3254','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4816','Carthage (Paris)','Replaced by Greenwich-based Carthage geogCRS.','geographic 2D','EPSG','6403','EPSG','6816',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3646','geodetic_crs','EPSG','4816','EPSG','1618','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4817','NGO 1948 (Oslo)','','geographic 2D','EPSG','6422','EPSG','6817',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3647','geodetic_crs','EPSG','4817','EPSG','1352','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4818','S-JTSK (Ferro)','Initial realization, observed and calculated in projected CRS domain (CRS code 2065). Later densification introduced distortion with inaccuracy of several decimetres. In Slovakia has been deprecated and replaced by Greenwich equivalent, CRS code 4156.','geographic 2D','EPSG','6422','EPSG','6818',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3648','geodetic_crs','EPSG','4818','EPSG','1306','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4819','Nord Sahara 1959 (Paris)','','geographic 2D','EPSG','6403','EPSG','6819',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3649','geodetic_crs','EPSG','4819','EPSG','1366','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4820','Segara (Jakarta)','','geographic 2D','EPSG','6422','EPSG','6820',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3650','geodetic_crs','EPSG','4820','EPSG','1360','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4821','Voirol 1879 (Paris)','The appropriate usage of CRSs using the Voirol 1875 and 1879 datums is lost in antiquity. They differ by about 9 metres. Oil industry references to one could in reality be to either. All replaced by Nord Sahara 1959 (CRS code 4307).','geographic 2D','EPSG','6403','EPSG','6821',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3651','geodetic_crs','EPSG','4821','EPSG','1365','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4823','Sao Tome','','geographic 2D','EPSG','6422','EPSG','1044',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3653','geodetic_crs','EPSG','4823','EPSG','3645','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4824','Principe','','geographic 2D','EPSG','6422','EPSG','1046',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3654','geodetic_crs','EPSG','4824','EPSG','3646','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4882','Slovenia 1996','','geocentric','EPSG','6500','EPSG','6765',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3683','geodetic_crs','EPSG','4882','EPSG','1212','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4883','Slovenia 1996','','geographic 3D','EPSG','6423','EPSG','6765',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3684','geodetic_crs','EPSG','4883','EPSG','1212','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4884','RSRGD2000','','geocentric','EPSG','6500','EPSG','6764',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3685','geodetic_crs','EPSG','4884','EPSG','3558','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4885','RSRGD2000','','geographic 3D','EPSG','6423','EPSG','6764',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3686','geodetic_crs','EPSG','4885','EPSG','3558','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4886','BDA2000','','geocentric','EPSG','6500','EPSG','6762',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3687','geodetic_crs','EPSG','4886','EPSG','1047','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4887','BDA2000','','geographic 3D','EPSG','6423','EPSG','6762',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3688','geodetic_crs','EPSG','4887','EPSG','1047','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4888','HTRS96','','geocentric','EPSG','6500','EPSG','6761',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3689','geodetic_crs','EPSG','4888','EPSG','1076','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4889','HTRS96','','geographic 3D','EPSG','6423','EPSG','6761',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3690','geodetic_crs','EPSG','4889','EPSG','1076','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4890','WGS 66','Replaced by WGS 72.','geocentric','EPSG','6500','EPSG','6760',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3691','geodetic_crs','EPSG','4890','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4891','WGS 66','Replaced by WGS 72.','geographic 3D','EPSG','6423','EPSG','6760',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3692','geodetic_crs','EPSG','4891','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4892','NAD83(NSRS2007)','Replaces NAD83(HARN) and NAD83(FBN). Replaced by NAD83(2011).','geocentric','EPSG','6500','EPSG','6759',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3693','geodetic_crs','EPSG','4892','EPSG','1511','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4893','NAD83(NSRS2007)','Note: this CRS includes longitudes which are POSITIVE EAST. Replaces NAD83(HARN) and NAD83(FBN). Replaced by NAD83(2011).','geographic 3D','EPSG','6423','EPSG','6759',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3694','geodetic_crs','EPSG','4893','EPSG','1511','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4894','JAD2001','','geocentric','EPSG','6500','EPSG','6758',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3695','geodetic_crs','EPSG','4894','EPSG','1128','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4895','JAD2001','','geographic 3D','EPSG','6423','EPSG','6758',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3696','geodetic_crs','EPSG','4895','EPSG','1128','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4896','ITRF2005','Replaces ITRF2000 (CRS code 4919). Replaced by ITRF2008 (CRS code 5332).','geocentric','EPSG','6500','EPSG','6896',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3697','geodetic_crs','EPSG','4896','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4897','DGN95','','geocentric','EPSG','6500','EPSG','6755',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3698','geodetic_crs','EPSG','4897','EPSG','1122','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4898','DGN95','','geographic 3D','EPSG','6423','EPSG','6755',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3699','geodetic_crs','EPSG','4898','EPSG','1122','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4899','LGD2006','','geocentric','EPSG','6500','EPSG','6754',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3700','geodetic_crs','EPSG','4899','EPSG','1143','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4900','LGD2006','','geographic 3D','EPSG','6423','EPSG','6754',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3701','geodetic_crs','EPSG','4900','EPSG','1143','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4901','ATF (Paris)','ProjCRS covering all mainland France based on this datum used Bonne projection. In Alsace, suspected to be an extension of core network based on transformation of old German system.','geographic 2D','EPSG','6403','EPSG','6901',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3702','geodetic_crs','EPSG','4901','EPSG','1326','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4902','NDG (Paris)','','geographic 2D','EPSG','6403','EPSG','6902',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3703','geodetic_crs','EPSG','4902','EPSG','1369','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4903','Madrid 1870 (Madrid)','Replaced by ED50 in 1970.','geographic 2D','EPSG','6422','EPSG','6903',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3704','geodetic_crs','EPSG','4903','EPSG','2366','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4904','Lisbon 1890 (Lisbon)','Replaced by Lisbon 1937 system which uses International 1924 ellipsoid (code 4803).','geographic 2D','EPSG','6422','EPSG','6904',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3705','geodetic_crs','EPSG','4904','EPSG','1294','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4906','RGNC91-93','Replaces older local geographic 2D systems IGN56 Lifou, IGN72 Grande Terre, ST87 Ouvea, IGN53 Mare, ST84 Ile des Pins, ST71 Belep and NEA74 Noumea (CRS codes 4633, 4641-44, 4662 and 4750). Replaced by RGNC15 (CRS code 10308).','geocentric','EPSG','6500','EPSG','6749',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3706','geodetic_crs','EPSG','4906','EPSG','1174','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4907','RGNC91-93','Replaces older systems IGN56 Lifou, IGN72 Grande Terre, ST87 Ouvea, IGN53 Mare, ST84 Ile des Pins, ST71 Belep and NEA74 Noumea. Replaced by RGNC15 (CRS 10309). See CRS code 10300 for alternate system with axes reversed used by DITTT for GIS purposes.','geographic 3D','EPSG','6423','EPSG','6749',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3707','geodetic_crs','EPSG','4907','EPSG','1174','EPSG','1182');
INSERT INTO "geodetic_crs" VALUES('EPSG','4908','GR96','','geocentric','EPSG','6500','EPSG','1421',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3708','geodetic_crs','EPSG','4908','EPSG','1107','EPSG','1056');
INSERT INTO "geodetic_crs" VALUES('EPSG','4909','GR96','','geographic 3D','EPSG','6423','EPSG','1421',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3709','geodetic_crs','EPSG','4909','EPSG','1107','EPSG','1056');
INSERT INTO "geodetic_crs" VALUES('EPSG','4910','ITRF88','Replaced by ITRF89 (code 4911).','geocentric','EPSG','6500','EPSG','6647',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3710','geodetic_crs','EPSG','4910','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4911','ITRF89','Replaces ITRF88 (code 4910). Replaced by ITRF90 (code 4912).','geocentric','EPSG','6500','EPSG','6648',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3711','geodetic_crs','EPSG','4911','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4912','ITRF90','Replaces ITRF89 (code 4911). Replaced by ITRF91 (code 4913).','geocentric','EPSG','6500','EPSG','6649',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3712','geodetic_crs','EPSG','4912','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4913','ITRF91','Replaces ITRF90 (code 4912). Replaced by ITRF92 (code 4914).','geocentric','EPSG','6500','EPSG','6650',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3713','geodetic_crs','EPSG','4913','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4914','ITRF92','Replaces ITRF91 (code 4913). Replaced by ITRF93 (code 4915).','geocentric','EPSG','6500','EPSG','6651',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3714','geodetic_crs','EPSG','4914','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4915','ITRF93','Replaces ITRF92 (code 4914). Replaced by ITRF94 (code 4916).','geocentric','EPSG','6500','EPSG','6652',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3715','geodetic_crs','EPSG','4915','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4916','ITRF94','Replaces ITRF93 (code 4915). Replaced by ITRF96 (code 4917).','geocentric','EPSG','6500','EPSG','6653',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3716','geodetic_crs','EPSG','4916','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4917','ITRF96','Replaces ITRF94 (code 4916). Replaced by ITRF97 (code 4918).','geocentric','EPSG','6500','EPSG','6654',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3717','geodetic_crs','EPSG','4917','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4918','ITRF97','Replaces ITRF96 (code 4917). Replaced by ITRF2000 (code 4919).','geocentric','EPSG','6500','EPSG','6655',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3718','geodetic_crs','EPSG','4918','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4919','ITRF2000','Replaces ITRF97 (code 4918). Replaced by ITRF2005 (code 4896).','geocentric','EPSG','6500','EPSG','6656',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3719','geodetic_crs','EPSG','4919','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4920','GDM2000','','geocentric','EPSG','6500','EPSG','6742',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3720','geodetic_crs','EPSG','4920','EPSG','1151','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4921','GDM2000','','geographic 3D','EPSG','6423','EPSG','6742',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3721','geodetic_crs','EPSG','4921','EPSG','1151','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4922','PZ-90','Replaced by PZ-90.02 from 2007-09-20.','geocentric','EPSG','6500','EPSG','6740',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3722','geodetic_crs','EPSG','4922','EPSG','1262','EPSG','1177');
INSERT INTO "geodetic_crs" VALUES('EPSG','4923','PZ-90','Replaced by PZ-90.02 from 2007-09-20.','geographic 3D','EPSG','6423','EPSG','6740',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3723','geodetic_crs','EPSG','4923','EPSG','1262','EPSG','1177');
INSERT INTO "geodetic_crs" VALUES('EPSG','4924','Mauritania 1999','','geocentric','EPSG','6500','EPSG','6702',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3724','geodetic_crs','EPSG','4924','EPSG','1157','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4925','Mauritania 1999','','geographic 3D','EPSG','6423','EPSG','6702',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3725','geodetic_crs','EPSG','4925','EPSG','1157','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4926','KGD2002','','geocentric','EPSG','6500','EPSG','6737',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3726','geodetic_crs','EPSG','4926','EPSG','1135','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4927','KGD2002','','geographic 3D','EPSG','6423','EPSG','6737',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3727','geodetic_crs','EPSG','4927','EPSG','1135','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4928','POSGAR 94','Legally adopted in May 1997. Replaced by POSGAR 98 for scientific and many practical purposes until May 2009. Officially replaced by POSGAR 2007 in May 2009.','geocentric','EPSG','6500','EPSG','6694',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3728','geodetic_crs','EPSG','4928','EPSG','1033','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4929','POSGAR 94','Legally adopted in May 1997. Replaced by POSGAR 98 for scientific and many practical purposes until May 2009. Officially replaced by POSGAR 2007 in May 2009.','geographic 3D','EPSG','6423','EPSG','6694',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3729','geodetic_crs','EPSG','4929','EPSG','1033','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4930','Australian Antarctic','','geocentric','EPSG','6500','EPSG','6176',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3730','geodetic_crs','EPSG','4930','EPSG','1278','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4931','Australian Antarctic','','geographic 3D','EPSG','6423','EPSG','6176',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3731','geodetic_crs','EPSG','4931','EPSG','1278','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4932','CHTRS95','Referenced to ETRS89 at epoch 1993.0. For CRS used for topographic and cadastral purposes see CH1903+ (CRS code 4150).','geocentric','EPSG','6500','EPSG','6151',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3732','geodetic_crs','EPSG','4932','EPSG','1286','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4933','CHTRS95','Referenced to ETRS89 at epoch 1993.0. For CRS used for topographic and cadastral purposes see CH1903+ (CRS code 4150).','geographic 3D','EPSG','6423','EPSG','6151',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3733','geodetic_crs','EPSG','4933','EPSG','1286','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4934','EST97','','geocentric','EPSG','6500','EPSG','6180',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3734','geodetic_crs','EPSG','4934','EPSG','1090','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4935','EST97','','geographic 3D','EPSG','6423','EPSG','6180',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3735','geodetic_crs','EPSG','4935','EPSG','1090','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4936','ETRS89','Has been realized through ETRF89, ETRF90, ETRF91, ETRF92, ETRF93, ETRF94, ETRF96, ETRF97, ETRF2000, ETRF2005 and ETRF2014. This ''ensemble'' covers any or all of these realizations without distinction.','geocentric','EPSG','6500','EPSG','6258',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3736','geodetic_crs','EPSG','4936','EPSG','4755','EPSG','1026');
INSERT INTO "geodetic_crs" VALUES('EPSG','4937','ETRS89','Has been realized through ETRF89, ETRF90, ETRF91, ETRF92, ETRF93, ETRF94, ETRF96, ETRF97, ETRF2000, ETRF2005 and ETRF2014. This ''ensemble'' covers any or all of these realizations without distinction.','geographic 3D','EPSG','6423','EPSG','6258',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3737','geodetic_crs','EPSG','4937','EPSG','4755','EPSG','1026');
INSERT INTO "geodetic_crs" VALUES('EPSG','4938','GDA94','','geocentric','EPSG','6500','EPSG','6283',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3738','geodetic_crs','EPSG','4938','EPSG','4177','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4939','GDA94','','geographic 3D','EPSG','6423','EPSG','6283',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3739','geodetic_crs','EPSG','4939','EPSG','4177','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4940','Hartebeesthoek94','','geocentric','EPSG','6500','EPSG','6148',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3740','geodetic_crs','EPSG','4940','EPSG','4540','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4941','Hartebeesthoek94','','geographic 3D','EPSG','6423','EPSG','6148',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3741','geodetic_crs','EPSG','4941','EPSG','4540','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4942','IRENET95','','geocentric','EPSG','6500','EPSG','6173',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3742','geodetic_crs','EPSG','4942','EPSG','1305','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4943','IRENET95','','geographic 3D','EPSG','6423','EPSG','6173',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3743','geodetic_crs','EPSG','4943','EPSG','1305','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4944','ISN93','Replaced by ISN2004 (CRS code 5322).','geocentric','EPSG','6500','EPSG','6659',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3744','geodetic_crs','EPSG','4944','EPSG','1120','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4945','ISN93','Replaced by ISN2004 (CRS code 5323).','geographic 3D','EPSG','6423','EPSG','6659',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3745','geodetic_crs','EPSG','4945','EPSG','1120','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4946','JGD2000','From 21st October 2011 replaced by JGD2011 (CRS code 6666).','geocentric','EPSG','6500','EPSG','6612',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3746','geodetic_crs','EPSG','4946','EPSG','1129','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4947','JGD2000','From 21st October 2011 replaced by JGD2011 (CRS code 6667).','geographic 3D','EPSG','6423','EPSG','6612',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3747','geodetic_crs','EPSG','4947','EPSG','1129','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4948','LKS-92','Replaced by LKS-2020 (CRS code 10303) from 2026-10-01.','geocentric','EPSG','6500','EPSG','6661',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3748','geodetic_crs','EPSG','4948','EPSG','1139','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4949','LKS-92','Replaced by LKS-2020 (CRS code 10304) from 2026-10-01.','geographic 3D','EPSG','6423','EPSG','6661',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3749','geodetic_crs','EPSG','4949','EPSG','1139','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4950','LKS94','','geocentric','EPSG','6500','EPSG','6126',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3750','geodetic_crs','EPSG','4950','EPSG','1145','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4951','LKS94','','geographic 3D','EPSG','6423','EPSG','6126',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3751','geodetic_crs','EPSG','4951','EPSG','1145','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4952','Moznet','','geocentric','EPSG','6500','EPSG','6130',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3752','geodetic_crs','EPSG','4952','EPSG','1167','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4953','Moznet','','geographic 3D','EPSG','6423','EPSG','6130',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3753','geodetic_crs','EPSG','4953','EPSG','1167','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4954','NAD83(CSRS)','Includes all versions of NAD83(CSRS) from v2 [CSRS98] onwards without specific identification. As such it has an accuracy of approximately 1m.','geocentric','EPSG','6500','EPSG','6140',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3754','geodetic_crs','EPSG','4954','EPSG','1061','EPSG','1189');
INSERT INTO "geodetic_crs" VALUES('EPSG','4955','NAD83(CSRS)','Includes all versions of NAD83(CSRS) from v2 [CSRS98] onwards without specific identification. As such it has an accuracy of approximately 1m. Note: this CRS includes longitudes which are POSITIVE EAST.','geographic 3D','EPSG','6423','EPSG','6140',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3755','geodetic_crs','EPSG','4955','EPSG','1061','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4956','NAD83(HARN)','In CONUS and Hawaii replaces NAD83 for applications with an accuracy of better than 1m. Replaced by NAD83(FBN) in CONUS, American Samoa and Guam/NMI, by NAD83(NSRS2007) in Alaska, by NAD83(PA11) in Hawaii and by NAD83(HARN Corrected) in PRVI.','geocentric','EPSG','6500','EPSG','6152',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3756','geodetic_crs','EPSG','4956','EPSG','1337','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4957','NAD83(HARN)','In CONUS and Hawaii replaces NAD83 for applications with an accuracy of better than 1m. Replaced by NAD83(FBN) in CONUS, American Samoa and Guam / NMI, by NAD83(NSRS2007) in Alaska, by NAD83(PA11) in Hawaii and by NAD83(HARN Corrected) in PRVI.','geographic 3D','EPSG','6423','EPSG','6152',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3757','geodetic_crs','EPSG','4957','EPSG','1337','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4958','NZGD2000','','geocentric','EPSG','6500','EPSG','6167',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3758','geodetic_crs','EPSG','4958','EPSG','1175','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4959','NZGD2000','','geographic 3D','EPSG','6423','EPSG','6167',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3759','geodetic_crs','EPSG','4959','EPSG','1175','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4960','POSGAR 98','Densification in Argentina of SIRGAS 1995. Until May 2009 replaced POSGAR 94 for many practical purposes (but not as the legal system). POSGAR 94 was officially replaced by POSGAR 2007 in May 2009.','geocentric','EPSG','6500','EPSG','6190',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3760','geodetic_crs','EPSG','4960','EPSG','1033','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4961','POSGAR 98','Densification in Argentina of SIRGAS 1995. Until May 2009 replaced POSGAR 94 for many practical purposes (but not as the legal system). POSGAR 94 was officially replaced by POSGAR 2007 in May 2009.','geographic 3D','EPSG','6423','EPSG','6190',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3761','geodetic_crs','EPSG','4961','EPSG','1033','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4962','REGVEN','Densification in Venezuela of SIRGAS.','geocentric','EPSG','6500','EPSG','6189',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3762','geodetic_crs','EPSG','4962','EPSG','1251','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4963','REGVEN','Densification in Venezuela of SIRGAS.','geographic 3D','EPSG','6423','EPSG','6189',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3763','geodetic_crs','EPSG','4963','EPSG','1251','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4964','RGF93 v1','Replaced by RGF93 v2 (CRS code 9775) from 2010-06-18.','geocentric','EPSG','6500','EPSG','6171',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3764','geodetic_crs','EPSG','4964','EPSG','1096','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4965','RGF93 v1','See CRS code 7042 for alternate system with horizontal axes reversed used by IGN for GIS purposes. Replaced by RGF93 v2 (CRS code 9776) from 2010-06-18.','geographic 3D','EPSG','6423','EPSG','6171',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3765','geodetic_crs','EPSG','4965','EPSG','1096','EPSG','1182');
INSERT INTO "geodetic_crs" VALUES('EPSG','4966','RGFG95','','geocentric','EPSG','6500','EPSG','6624',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3766','geodetic_crs','EPSG','4966','EPSG','1097','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4967','RGFG95','See CRS code 7040 for alternate system with horizontal axes reversed used by IGN for GIS purposes.','geographic 3D','EPSG','6423','EPSG','6624',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3767','geodetic_crs','EPSG','4967','EPSG','1097','EPSG','1182');
INSERT INTO "geodetic_crs" VALUES('EPSG','4968','RGNC 1991','Supersedes older local geographic 2D systems IGN56 Lifou, IGN72 Grande Terre, ST87 Ouvea, IGN53 Mare, ST84 Ile des Pins, ST71 Belep and NEA74 Noumea (CRS codes 463, 4635, 4650-53 and 4662).','geocentric','EPSG','6500','EPSG','6645',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3768','geodetic_crs','EPSG','4968','EPSG','1174','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4969','RGNC 1991','Supersedes older local 2D systems IGN56 Lifou, IGN72 Grande Terre, ST87 Ouvea, IGN53 Mare, ST84 Ile des Pins, ST71 Belep and NEA74 Noumea (CRS codes 4633, 4635, 4650-53 and 4662).','geographic 3D','EPSG','6423','EPSG','6645',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3769','geodetic_crs','EPSG','4969','EPSG','1174','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4970','RGR92','','geocentric','EPSG','6500','EPSG','6627',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3770','geodetic_crs','EPSG','4970','EPSG','3902','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4971','RGR92','See CRS code 7036 for alternate system with horizontal axes reversed used by IGN for GIS purposes.','geographic 3D','EPSG','6423','EPSG','6627',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3771','geodetic_crs','EPSG','4971','EPSG','3902','EPSG','1182');
INSERT INTO "geodetic_crs" VALUES('EPSG','4972','RRAF 1991','Replaces older local geographic 2D systems Fort Marigot and Sainte Anne CRS (codes 4621-22) in Guadeloupe and Fort Desaix (CRS code 4625) in Martinique.','geocentric','EPSG','6500','EPSG','6640',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3772','geodetic_crs','EPSG','4972','EPSG','2824','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4973','RRAF 1991','Replaces older local 2D systems Fort Marigot and Sainte Anne CRS (codes 4621-22) in Guadeloupe and Fort Desaix (CRS code 4625) in Martinique.','geographic 3D','EPSG','6423','EPSG','6640',NULL,1);
INSERT INTO "usage" VALUES('EPSG','3773','geodetic_crs','EPSG','4973','EPSG','2824','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4974','SIRGAS 1995','Replaced by SIRGAS 2000 (CRS code 4988).','geocentric','EPSG','6500','EPSG','6170',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3774','geodetic_crs','EPSG','4974','EPSG','3448','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4975','SIRGAS 1995','Replaced by SIRGAS 2000 (CRS code 4989).','geographic 3D','EPSG','6423','EPSG','6170',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3775','geodetic_crs','EPSG','4975','EPSG','3448','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4976','SWEREF99','','geocentric','EPSG','6500','EPSG','6619',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3776','geodetic_crs','EPSG','4976','EPSG','1225','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4977','SWEREF99','','geographic 3D','EPSG','6423','EPSG','6619',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3777','geodetic_crs','EPSG','4977','EPSG','1225','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4978','WGS 84','','geocentric','EPSG','6500','EPSG','6326',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3778','geodetic_crs','EPSG','4978','EPSG','1262','EPSG','1176');
INSERT INTO "geodetic_crs" VALUES('EPSG','4979','WGS 84','','geographic 3D','EPSG','6423','EPSG','6326',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3779','geodetic_crs','EPSG','4979','EPSG','1262','EPSG','1176');
INSERT INTO "geodetic_crs" VALUES('EPSG','4980','Yemen NGN96','','geocentric','EPSG','6500','EPSG','6163',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3780','geodetic_crs','EPSG','4980','EPSG','1257','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4981','Yemen NGN96','','geographic 3D','EPSG','6423','EPSG','6163',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3781','geodetic_crs','EPSG','4981','EPSG','1257','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4982','IGM95','Replaced by RDN2008 (CRS code 6704) from 2011-11-10.','geocentric','EPSG','6500','EPSG','6670',NULL,0);
INSERT INTO "usage" VALUES('EPSG','14402','geodetic_crs','EPSG','4982','EPSG','3343','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4983','IGM95','Replaced by RDN2008 (CRS code 6705) from 2011-11-10.','geographic 3D','EPSG','6423','EPSG','6670',NULL,0);
INSERT INTO "usage" VALUES('EPSG','14403','geodetic_crs','EPSG','4983','EPSG','3343','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4984','WGS 72','Replaced by WGS 84.','geocentric','EPSG','6500','EPSG','6322',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3784','geodetic_crs','EPSG','4984','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4985','WGS 72','Replaced by WGS 84.','geographic 3D','EPSG','6423','EPSG','6322',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3785','geodetic_crs','EPSG','4985','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4986','WGS 72BE','Broadcast ephemeris. Replaced by WGS 84.','geocentric','EPSG','6500','EPSG','6324',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3786','geodetic_crs','EPSG','4986','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4987','WGS 72BE','Broadcast ephemeris. Replaced by WGS 84.','geographic 3D','EPSG','6423','EPSG','6324',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3787','geodetic_crs','EPSG','4987','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4988','SIRGAS 2000','Replaces SIRGAS 1995 system (CRS code 4974) for South America; expands SIRGAS to Central America.','geocentric','EPSG','6500','EPSG','6674',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3788','geodetic_crs','EPSG','4988','EPSG','3418','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4989','SIRGAS 2000','Replaces SIRGAS 1995 system (CRS code 4975) for South America; expands SIRGAS to Central America.','geographic 3D','EPSG','6423','EPSG','6674',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3789','geodetic_crs','EPSG','4989','EPSG','3418','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4990','Lao 1993','Replaced by Lao 1997. Lao 1993 coordinate values are within 1m of Lao 1997 values.','geocentric','EPSG','6500','EPSG','6677',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3790','geodetic_crs','EPSG','4990','EPSG','1138','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4991','Lao 1993','Replaced by Lao 1997. Lao 1993 coordinate values are within 1m of Lao 1997 values.','geographic 3D','EPSG','6423','EPSG','6677',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3791','geodetic_crs','EPSG','4991','EPSG','1138','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4992','Lao 1997','Replaces Lao 1993. Lao 1993 coordinate values are within 1m of Lao 1997 values.','geocentric','EPSG','6500','EPSG','6678',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3792','geodetic_crs','EPSG','4992','EPSG','1138','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4993','Lao 1997','Replaces Lao 1993. Lao 1993 coordinate values are within 1m of Lao 1997 values.','geographic 3D','EPSG','6423','EPSG','6678',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3793','geodetic_crs','EPSG','4993','EPSG','1138','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4994','PRS92','','geocentric','EPSG','6500','EPSG','6683',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3794','geodetic_crs','EPSG','4994','EPSG','1190','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4995','PRS92','','geographic 3D','EPSG','6423','EPSG','6683',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3795','geodetic_crs','EPSG','4995','EPSG','1190','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4996','MAGNA-SIRGAS','For high accuracy purposes replaced by MAGNA-SIRGAS 2018 (code 20044).','geocentric','EPSG','6500','EPSG','6686',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3796','geodetic_crs','EPSG','4996','EPSG','1070','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4997','MAGNA-SIRGAS','For high accuracy purposes replaced by MAGNA-SIRGAS 2018 (code 20045).','geographic 3D','EPSG','6423','EPSG','6686',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3797','geodetic_crs','EPSG','4997','EPSG','1070','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4998','RGPF','','geocentric','EPSG','6500','EPSG','6687',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3798','geodetic_crs','EPSG','4998','EPSG','1098','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','4999','RGPF','','geographic 3D','EPSG','6423','EPSG','6687',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3799','geodetic_crs','EPSG','4999','EPSG','1098','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','5011','PTRA08','','geocentric','EPSG','6500','EPSG','1041',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3800','geodetic_crs','EPSG','5011','EPSG','3670','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','5012','PTRA08','','geographic 3D','EPSG','6423','EPSG','1041',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3801','geodetic_crs','EPSG','5012','EPSG','3670','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','5013','PTRA08','Replaces Azores Occidental 1939, Azores Central 1995, Azores Oriental 1995 and Porto Santo 1995 (CRS codes 4182 and 4663-65).','geographic 2D','EPSG','6422','EPSG','1041',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3802','geodetic_crs','EPSG','5013','EPSG','3670','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','5132','Tokyo 1892','Extended from Japan to Korea in 1898. In Japan, replaced by Tokyo 1918 (CRS code 4301). In South Korea replaced by Tokyo 1918 only for geodetic applications; for all other purposes replaced by Korean 1985 (code 4162).','geographic 2D','EPSG','6422','EPSG','1048',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3841','geodetic_crs','EPSG','5132','EPSG','1364','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','5228','S-JTSK/05','Derived through projCRS 5515 to improve the scale and homogeneity of CRS 4156 within Czechia as a scientific working system, but CRS 4156 remains the legal system. See CRS code 5229 for Ferro-referenced alternative.','geographic 2D','EPSG','6422','EPSG','1052',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3871','geodetic_crs','EPSG','5228','EPSG','1079','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','5229','S-JTSK/05 (Ferro)','Derived through projCRS code 5224 to improve the scale and homogeneity of CRS 4818 within Czechia as a scientific working system, but CRS 4818 remains the legal system. See CRS code 5228 for Greenwich-referenced alternative.','geographic 2D','EPSG','6422','EPSG','1055',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3872','geodetic_crs','EPSG','5229','EPSG','1079','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','5233','SLD99','','geographic 2D','EPSG','6422','EPSG','1053',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3873','geodetic_crs','EPSG','5233','EPSG','3310','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','5244','GDBD2009','','geocentric','EPSG','6500','EPSG','1056',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3878','geodetic_crs','EPSG','5244','EPSG','1055','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','5245','GDBD2009','','geographic 3D','EPSG','6423','EPSG','1056',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3879','geodetic_crs','EPSG','5245','EPSG','1055','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','5246','GDBD2009','Introduced from July 2009 to replace Timbalai 1948 (CRS code 4298) for government purposes.','geographic 2D','EPSG','6422','EPSG','1056',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3880','geodetic_crs','EPSG','5246','EPSG','1055','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','5250','TUREF','','geocentric','EPSG','6500','EPSG','1057',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3882','geodetic_crs','EPSG','5250','EPSG','1237','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','5251','TUREF','','geographic 3D','EPSG','6423','EPSG','1057',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3883','geodetic_crs','EPSG','5251','EPSG','1237','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','5252','TUREF','','geographic 2D','EPSG','6422','EPSG','1057',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3884','geodetic_crs','EPSG','5252','EPSG','1237','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','5262','DRUKREF 03','','geocentric','EPSG','6500','EPSG','1058',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3892','geodetic_crs','EPSG','5262','EPSG','1048','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','5263','DRUKREF 03','','geographic 3D','EPSG','6423','EPSG','1058',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3893','geodetic_crs','EPSG','5263','EPSG','1048','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','5264','DRUKREF 03','','geographic 2D','EPSG','6422','EPSG','1058',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3894','geodetic_crs','EPSG','5264','EPSG','1048','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','5322','ISN2004','Replaces ISN93 (CRS code 4944). Replaced by ISN2016 (CRS code 8084).','geocentric','EPSG','6500','EPSG','1060',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3928','geodetic_crs','EPSG','5322','EPSG','1120','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','5323','ISN2004','Replaces ISN93 (CRS code 4945). Replaced by ISN2016 (CRS code 8085).','geographic 3D','EPSG','6423','EPSG','1060',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3929','geodetic_crs','EPSG','5323','EPSG','1120','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','5324','ISN2004','Replaces ISN93 (CRS code 4659). Replaced by ISN2016 (CRS code 8086).','geographic 2D','EPSG','6422','EPSG','1060',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3930','geodetic_crs','EPSG','5324','EPSG','1120','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','5332','ITRF2008','Replaces ITRF2005 (CRS code 4896). Replaced by ITRF2014 (CRS code 7789).','geocentric','EPSG','6500','EPSG','1061',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3935','geodetic_crs','EPSG','5332','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','5340','POSGAR 2007','Adopted as official replacement of POSGAR 94 in May 2009. Also replaces de facto use of POSGAR 98 as of same date.','geographic 2D','EPSG','6422','EPSG','1062',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3938','geodetic_crs','EPSG','5340','EPSG','1033','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','5341','POSGAR 2007','Adopted as official replacement of POSGAR 94 in May 2009. Also replaces de facto use of POSGAR 98 as of same date.','geocentric','EPSG','6500','EPSG','1062',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3939','geodetic_crs','EPSG','5341','EPSG','1033','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','5342','POSGAR 2007','Adopted as official replacement of POSGAR 94 in May 2009. Also replaces de facto use of POSGAR 98 as of same date.','geographic 3D','EPSG','6423','EPSG','1062',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3940','geodetic_crs','EPSG','5342','EPSG','1033','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','5352','MARGEN','','geocentric','EPSG','6500','EPSG','1063',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3948','geodetic_crs','EPSG','5352','EPSG','1049','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','5353','MARGEN','','geographic 3D','EPSG','6423','EPSG','1063',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3949','geodetic_crs','EPSG','5353','EPSG','1049','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','5354','MARGEN','Replaces PSAD56 (CRS code 4248) in Bolivia','geographic 2D','EPSG','6422','EPSG','1063',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3950','geodetic_crs','EPSG','5354','EPSG','1049','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','5358','SIRGAS-Chile 2002','Densification of SIRGAS 2000 within Chile. Replaced by SIRGAS-Chile 2010 (CRS code 8947).','geocentric','EPSG','6500','EPSG','1064',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3954','geodetic_crs','EPSG','5358','EPSG','1066','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','5359','SIRGAS-Chile 2002','Densification of SIRGAS 2000 within Chile. Replaced by SIRGAS-Chile 2010 (CRS code 8948).','geographic 3D','EPSG','6423','EPSG','1064',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3955','geodetic_crs','EPSG','5359','EPSG','1066','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','5360','SIRGAS-Chile 2002','Densification of SIRGAS 2000 within Chile. Replaces PSAD56 (CRS code 6248) in Chile, HITO XVIII (CRS code 6254) in Chilean Tierra del Fuego and Easter Island 1967 (CRS code 6719) in Easter Island. Replaced by SIRGAS-Chile 2010 (CRS code 8949).','geographic 2D','EPSG','6422','EPSG','1064',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3956','geodetic_crs','EPSG','5360','EPSG','1066','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','5363','CR05','Replaced by CR-SIRGAS epoch 2014.59 (CRS code 8905) from April 2018.','geocentric','EPSG','6500','EPSG','1065',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3959','geodetic_crs','EPSG','5363','EPSG','1074','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','5364','CR05','Replaced by CR-SIRGAS epoch 2014.59 (CRS code 8906) from April 2018.','geographic 3D','EPSG','6423','EPSG','1065',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3960','geodetic_crs','EPSG','5364','EPSG','1074','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','5365','CR05','Replaces Ocotepeque (CRS code 5451) in Costa Rica from March 2007. Replaced by CR-SIRGAS epoch 2014.59 (CRS code 8907) from April 2018.','geographic 2D','EPSG','6422','EPSG','1065',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3961','geodetic_crs','EPSG','5365','EPSG','1074','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','5368','MACARIO SOLIS','Densification of SIRGAS 2000 within Panama.','geocentric','EPSG','6500','EPSG','1066',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3963','geodetic_crs','EPSG','5368','EPSG','1186','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','5369','Peru96','Densification of SIRGAS95 in Peru.','geocentric','EPSG','6500','EPSG','1067',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3964','geodetic_crs','EPSG','5369','EPSG','1189','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','5370','MACARIO SOLIS','Densification of SIRGAS 2000 within Panama.','geographic 3D','EPSG','6423','EPSG','1066',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3965','geodetic_crs','EPSG','5370','EPSG','1186','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','5371','MACARIO SOLIS','','geographic 2D','EPSG','6422','EPSG','1066',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3966','geodetic_crs','EPSG','5371','EPSG','1186','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','5372','Peru96','','geographic 3D','EPSG','6423','EPSG','1067',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3967','geodetic_crs','EPSG','5372','EPSG','1189','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','5373','Peru96','Replaces PSAD56 (CRS code 4248) in Peru.','geographic 2D','EPSG','6422','EPSG','1067',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3968','geodetic_crs','EPSG','5373','EPSG','1189','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','5379','SIRGAS-ROU98','','geocentric','EPSG','6500','EPSG','1068',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3969','geodetic_crs','EPSG','5379','EPSG','1247','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','5380','SIRGAS-ROU98','Densification of SIRGAS 1995 in Uruguay.','geographic 3D','EPSG','6423','EPSG','1068',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3970','geodetic_crs','EPSG','5380','EPSG','1247','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','5381','SIRGAS-ROU98','Replaces Yacare (CRS code 4309) in Uruguay.','geographic 2D','EPSG','6422','EPSG','1068',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3971','geodetic_crs','EPSG','5381','EPSG','1247','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','5391','SIRGAS_ES2007.8','Densification of SIRGAS 2000 within El Salvador.','geocentric','EPSG','6500','EPSG','1069',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3977','geodetic_crs','EPSG','5391','EPSG','1087','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','5392','SIRGAS_ES2007.8','Densification of SIRGAS 2000 within El Salvador.','geographic 3D','EPSG','6423','EPSG','1069',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3978','geodetic_crs','EPSG','5392','EPSG','1087','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','5393','SIRGAS_ES2007.8','','geographic 2D','EPSG','6422','EPSG','1069',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3979','geodetic_crs','EPSG','5393','EPSG','1087','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','5451','Ocotepeque 1935','Replaced in Costa Rica by Costa Rica 2005 (CR05) from March 2007 and replaced in El Salvador by SIRGAS_ES2007 from August 2007.','geographic 2D','EPSG','6422','EPSG','1070',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3981','geodetic_crs','EPSG','5451','EPSG','3876','EPSG','1180');
INSERT INTO "geodetic_crs" VALUES('EPSG','5464','Sibun Gorge 1922','','geographic 2D','EPSG','6422','EPSG','1071',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3990','geodetic_crs','EPSG','5464','EPSG','3219','EPSG','1180');
INSERT INTO "geodetic_crs" VALUES('EPSG','5467','Panama-Colon 1911','','geographic 2D','EPSG','6422','EPSG','1072',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3992','geodetic_crs','EPSG','5467','EPSG','3290','EPSG','1180');
INSERT INTO "geodetic_crs" VALUES('EPSG','5487','RGAF09','Replaces RRAF 1991 (CRS code 4556).','geocentric','EPSG','6500','EPSG','1073',NULL,0);
INSERT INTO "usage" VALUES('EPSG','3999','geodetic_crs','EPSG','5487','EPSG','2824','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','5488','RGAF09','Replaces RRAF 1991 (CRS code 4557). See CRS code 7085 for alternate system with horizontal axes reversed used by IGN for GIS purposes.','geographic 3D','EPSG','6423','EPSG','1073',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4000','geodetic_crs','EPSG','5488','EPSG','2824','EPSG','1182');
INSERT INTO "geodetic_crs" VALUES('EPSG','5489','RGAF09','Replaces RRAF 1991. See CRS code 7086 for alternate system with axes reversed used by IGN for GIS purposes.','geographic 2D','EPSG','6422','EPSG','1073',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4001','geodetic_crs','EPSG','5489','EPSG','2824','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','5524','Corrego Alegre 1961','Replaced by Corrego Alegre 1970-72 (CRS code 4225).','geographic 2D','EPSG','6422','EPSG','1074',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4014','geodetic_crs','EPSG','5524','EPSG','3874','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','5527','SAD69(96)','Uses GRS 1967 ellipsoid with 1/f to exactly 2 decimal places. Replaces SAD69 original adjustment (CRS code 4618) only in Brazil.','geographic 2D','EPSG','6422','EPSG','1075',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4015','geodetic_crs','EPSG','5527','EPSG','1053','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','5544','PNG94','','geocentric','EPSG','6500','EPSG','1076',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4026','geodetic_crs','EPSG','5544','EPSG','1187','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','5545','PNG94','','geographic 3D','EPSG','6423','EPSG','1076',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4027','geodetic_crs','EPSG','5545','EPSG','1187','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','5546','PNG94','Adopted 1996, replacing AGD66.','geographic 2D','EPSG','6422','EPSG','1076',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4028','geodetic_crs','EPSG','5546','EPSG','1187','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','5558','UCS-2000','Adopted 1st January 2007. Defined through transformation code 7817 at epoch 2005.0.','geocentric','EPSG','6500','EPSG','1077',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4035','geodetic_crs','EPSG','5558','EPSG','1242','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','5560','UCS-2000','Adopted 1st January 2007.','geographic 3D','EPSG','6423','EPSG','1077',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4037','geodetic_crs','EPSG','5560','EPSG','1242','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','5561','UCS-2000','Adopted 1st January 2007, replacing Pulkovo 1942 (CRS 4284).','geographic 2D','EPSG','6422','EPSG','1077',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4038','geodetic_crs','EPSG','5561','EPSG','1242','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','5591','FEH2010','Trans-national system created due to small (but unacceptable for engineering tolerance) differences between the German and Danish realisations of ETRS89.','geocentric','EPSG','6500','EPSG','1078',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4063','geodetic_crs','EPSG','5591','EPSG','3889','EPSG','1139');
INSERT INTO "geodetic_crs" VALUES('EPSG','5592','FEH2010','','geographic 3D','EPSG','6423','EPSG','1078',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4064','geodetic_crs','EPSG','5592','EPSG','3889','EPSG','1139');
INSERT INTO "geodetic_crs" VALUES('EPSG','5593','FEH2010','Created for engineering survey and construction of Fehmarnbelt tunnel.','geographic 2D','EPSG','6422','EPSG','1078',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4065','geodetic_crs','EPSG','5593','EPSG','3889','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','5681','DB_REF','Geometric component of both DB_REF2003 and DB_REF2016 systems. Differs from DHDN by 0.5-1m in former West Germany and by a maximum of 3m in former East Germany.','geographic 2D','EPSG','6422','EPSG','1081',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4136','geodetic_crs','EPSG','5681','EPSG','3339','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','5828','DB_REF','Geometric component of both DB_REF2003 and DB_REF2016 systems. Derived from ETRS89 through transformation code 5826.','geocentric','EPSG','6500','EPSG','1081',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4266','geodetic_crs','EPSG','5828','EPSG','3339','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','5830','DB_REF','Geometric component of both DB_REF2003 and DB_REF2016 systems.','geographic 3D','EPSG','6423','EPSG','1081',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4268','geodetic_crs','EPSG','5830','EPSG','3339','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','5884','TGD2005','','geocentric','EPSG','6500','EPSG','1095',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4314','geodetic_crs','EPSG','5884','EPSG','1234','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','5885','TGD2005','','geographic 3D','EPSG','6423','EPSG','1095',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4315','geodetic_crs','EPSG','5885','EPSG','1234','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','5886','TGD2005','','geographic 2D','EPSG','6422','EPSG','1095',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4316','geodetic_crs','EPSG','5886','EPSG','1234','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','6133','CIGD11','','geocentric','EPSG','6500','EPSG','1100',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4460','geodetic_crs','EPSG','6133','EPSG','1063','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','6134','CIGD11','','geographic 3D','EPSG','6423','EPSG','1100',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4461','geodetic_crs','EPSG','6134','EPSG','1063','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','6135','CIGD11','','geographic 2D','EPSG','6422','EPSG','1100',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4462','geodetic_crs','EPSG','6135','EPSG','1063','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','6207','Nepal 1981','Adopts 1937 metric conversion of 0.30479841 metres per Indian foot.','geographic 2D','EPSG','6422','EPSG','1111',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4512','geodetic_crs','EPSG','6207','EPSG','1171','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','6309','CGRS93','','geocentric','EPSG','6500','EPSG','1112',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4548','geodetic_crs','EPSG','6309','EPSG','3236','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','6310','CGRS93','','geographic 3D','EPSG','6423','EPSG','1112',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4549','geodetic_crs','EPSG','6310','EPSG','3236','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','6311','CGRS93','Adopted by DLS in 1993 for new survey plans and maps.','geographic 2D','EPSG','6422','EPSG','1112',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4550','geodetic_crs','EPSG','6311','EPSG','3236','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','6317','NAD83(2011)','Replaces NAD83(CORS96) and NAD83(NSRS2007) (CRS codes 6781 and 4892).','geocentric','EPSG','6500','EPSG','1116',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4553','geodetic_crs','EPSG','6317','EPSG','1511','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','6318','NAD83(2011)','Note: this CRS includes longitudes which are POSITIVE EAST. Replaces NAD83(CORS96) and NAD83(NSRS2007) (CRS codes 6783 and 4759).','geographic 2D','EPSG','6422','EPSG','1116',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4554','geodetic_crs','EPSG','6318','EPSG','1511','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','6319','NAD83(2011)','Note: this CRS includes longitudes which are POSITIVE EAST. Replaces NAD83(CORS96) and NAD83(NSRS2007) (CRS codes 6782 and 4893).','geographic 3D','EPSG','6423','EPSG','1116',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4555','geodetic_crs','EPSG','6319','EPSG','1511','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','6320','NAD83(PA11)','Replaces NAD83(HARN) and NAD83(FBN) in Hawaii and American Samoa.','geocentric','EPSG','6500','EPSG','1117',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4556','geodetic_crs','EPSG','6320','EPSG','4162','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','6321','NAD83(PA11)','Note: this CRS includes longitudes which are POSITIVE EAST. Replaces NAD83(HARN) and NAD83(FBN) in Hawaii and American Samoa.','geographic 3D','EPSG','6423','EPSG','1117',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4557','geodetic_crs','EPSG','6321','EPSG','4162','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','6322','NAD83(PA11)','Note: this CRS includes longitudes which are POSITIVE EAST. Replaces NAD83(HARN) and NAD83(FBN) in Hawaii and American Samoa.','geographic 2D','EPSG','6422','EPSG','1117',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4558','geodetic_crs','EPSG','6322','EPSG','4162','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','6323','NAD83(MA11)','Replaces NAD83(HARN) (GGN93) and NAD83(FBN) in Guam.','geocentric','EPSG','6500','EPSG','1118',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4559','geodetic_crs','EPSG','6323','EPSG','4167','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','6324','NAD83(MA11)','Note: this CRS includes longitudes which are POSITIVE EAST. Replaces NAD83(HARN) (GGN93) and NAD83(FBN) in Guam.','geographic 3D','EPSG','6423','EPSG','1118',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4560','geodetic_crs','EPSG','6324','EPSG','4167','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','6325','NAD83(MA11)','Note: this CRS includes longitudes which are POSITIVE EAST. Replaces NAD83(HARN) (GGN93) and NAD83(FBN) in Guam.','geographic 2D','EPSG','6422','EPSG','1118',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4561','geodetic_crs','EPSG','6325','EPSG','4167','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','6363','Mexico ITRF2008','Replaces Mexico ITRF92 (CRS code 4481) from December 2010.','geocentric','EPSG','6500','EPSG','1120',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4596','geodetic_crs','EPSG','6363','EPSG','1160','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','6364','Mexico ITRF2008','Replaces Mexico ITRF92 (CRS code 4482) from December 2010.','geographic 3D','EPSG','6423','EPSG','1120',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4597','geodetic_crs','EPSG','6364','EPSG','1160','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','6365','Mexico ITRF2008','Replaces Mexico ITRF92 (CRS code 4483) from December 2010.','geographic 2D','EPSG','6422','EPSG','1120',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4598','geodetic_crs','EPSG','6365','EPSG','1160','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','6666','JGD2011','Replaces JGD2000 (CRS code 4946) with effect from 21st October 2011.','geocentric','EPSG','6500','EPSG','1128',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4885','geodetic_crs','EPSG','6666','EPSG','1129','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','6667','JGD2011','Replaces JGD2000 (CRS code 4947) with effect from 21st October 2011.','geographic 3D','EPSG','6423','EPSG','1128',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4886','geodetic_crs','EPSG','6667','EPSG','1129','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','6668','JGD2011','Replaces JGD2000 (CRS code 4612) with effect from 21st October 2011.','geographic 2D','EPSG','6422','EPSG','1128',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4887','geodetic_crs','EPSG','6668','EPSG','1129','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','6704','RDN2008','Replaces IGM95 (CRS code 4982) from 2011-11-10.','geocentric','EPSG','6500','EPSG','1132',NULL,0);
INSERT INTO "usage" VALUES('EPSG','14411','geodetic_crs','EPSG','6704','EPSG','3343','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','6705','RDN2008','Replaces IGM95 (CRS code 4983) from 2011-11-10.','geographic 3D','EPSG','6423','EPSG','1132',NULL,0);
INSERT INTO "usage" VALUES('EPSG','14410','geodetic_crs','EPSG','6705','EPSG','3343','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','6706','RDN2008','Replaces IGM95 (CRS code 4670) from 2011-11-10.','geographic 2D','EPSG','6422','EPSG','1132',NULL,0);
INSERT INTO "usage" VALUES('EPSG','14412','geodetic_crs','EPSG','6706','EPSG','3343','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','6781','NAD83(CORS96)','Replaced by NAD83(2011) (CRS code 6317) from 2011-09-06.','geocentric','EPSG','6500','EPSG','1133',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4937','geodetic_crs','EPSG','6781','EPSG','1511','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','6782','NAD83(CORS96)','Note: this CRS includes POSITIVE EAST longitudes. Replaced by NAD83(2011) (CRS code 6319) from 2011-09-06.','geographic 3D','EPSG','6423','EPSG','1133',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4938','geodetic_crs','EPSG','6782','EPSG','1511','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','6783','NAD83(CORS96)','Note: this CRS includes POSITIVE EAST longitudes. Replaced by NAD83(2011) (CRS code 6318) from 2011-09-06.','geographic 2D','EPSG','6422','EPSG','1133',NULL,0);
INSERT INTO "usage" VALUES('EPSG','4939','geodetic_crs','EPSG','6783','EPSG','1511','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','6881','Aden 1925','','geographic 2D','EPSG','6422','EPSG','1135',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5028','geodetic_crs','EPSG','6881','EPSG','1340','EPSG','1138');
INSERT INTO "geodetic_crs" VALUES('EPSG','6882','Bekaa Valley 1920','','geographic 2D','EPSG','6422','EPSG','1137',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5029','geodetic_crs','EPSG','6882','EPSG','3269','EPSG','1153');
INSERT INTO "geodetic_crs" VALUES('EPSG','6883','Bioko','','geographic 2D','EPSG','6422','EPSG','1136',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5030','geodetic_crs','EPSG','6883','EPSG','4220','EPSG','1153');
INSERT INTO "geodetic_crs" VALUES('EPSG','6892','South East Island 1943','','geographic 2D','EPSG','6422','EPSG','1138',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5035','geodetic_crs','EPSG','6892','EPSG','4183','EPSG','1237');
INSERT INTO "geodetic_crs" VALUES('EPSG','6894','Gambia','','geographic 2D','EPSG','6422','EPSG','1139',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5037','geodetic_crs','EPSG','6894','EPSG','3250','EPSG','1153');
INSERT INTO "geodetic_crs" VALUES('EPSG','6934','IGS08','Used for products from International GNSS Service (IGS) analysis centres from 2011-04-17 through 2012-10-06. Replaces IGS05 (code 9010). Replaced by IGb08 (code 9015). For most practical purposes IGS08 is equivalent to ITRF2008.','geocentric','EPSG','6500','EPSG','1141',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5049','geodetic_crs','EPSG','6934','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','6978','IGD05','Replaced by IGD05/12 (CRS code 6985).','geocentric','EPSG','6500','EPSG','1143',NULL,1);
INSERT INTO "usage" VALUES('EPSG','5056','geodetic_crs','EPSG','6978','EPSG','1126','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','6979','IGD05','Replaced by IGD05/12 (CRS code 6986).','geographic 3D','EPSG','6423','EPSG','1143',NULL,1);
INSERT INTO "usage" VALUES('EPSG','5057','geodetic_crs','EPSG','6979','EPSG','1126','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','6980','IGD05','Replaces Israel 1993 (CRS code 4141) from January 2005. Replaced by IGD05/12 (CRS code 6987) from March 2012.','geographic 2D','EPSG','6422','EPSG','1143',NULL,1);
INSERT INTO "usage" VALUES('EPSG','5058','geodetic_crs','EPSG','6980','EPSG','1126','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','6981','IG05 Intermediate CRS','Intermediate system not used for spatial referencing - use IGD05 (CRS code 6978).','geocentric','EPSG','6500','EPSG','1142',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5059','geodetic_crs','EPSG','6981','EPSG','2603','EPSG','1203');
INSERT INTO "geodetic_crs" VALUES('EPSG','6982','IG05 Intermediate CRS','Intermediate system not used for spatial referencing - use IGD05 (CRS code 6979).','geographic 3D','EPSG','6423','EPSG','1142',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5060','geodetic_crs','EPSG','6982','EPSG','2603','EPSG','1203');
INSERT INTO "geodetic_crs" VALUES('EPSG','6983','IG05 Intermediate CRS','Intermediate system not used for spatial referencing - use IGD05 (CRS code 6980). Referred to in Israeli documentation as "in GRS80".','geographic 2D','EPSG','6422','EPSG','1142',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5061','geodetic_crs','EPSG','6983','EPSG','2603','EPSG','1203');
INSERT INTO "geodetic_crs" VALUES('EPSG','6985','IGD05/12','Replaces IGD05 (CRS code 6978).','geocentric','EPSG','6500','EPSG','1145',NULL,1);
INSERT INTO "usage" VALUES('EPSG','5063','geodetic_crs','EPSG','6985','EPSG','1126','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','6986','IGD05/12','Replaces IGD05 (CRS code 6979).','geographic 3D','EPSG','6423','EPSG','1145',NULL,1);
INSERT INTO "usage" VALUES('EPSG','5064','geodetic_crs','EPSG','6986','EPSG','1126','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','6987','IGD05/12','Replaces IGD05 (CRS code 6980) from March 2012.','geographic 2D','EPSG','6422','EPSG','1145',NULL,1);
INSERT INTO "usage" VALUES('EPSG','5065','geodetic_crs','EPSG','6987','EPSG','1126','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','6988','IG05/12 Intermediate CRS','Intermediate system not used for spatial referencing - use IGD05/12 (CRS code 6985).','geocentric','EPSG','6500','EPSG','1144',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5066','geodetic_crs','EPSG','6988','EPSG','2603','EPSG','1203');
INSERT INTO "geodetic_crs" VALUES('EPSG','6989','IG05/12 Intermediate CRS','Intermediate system not used for spatial referencing - use IGD05/12 (CRS code 6986).','geographic 3D','EPSG','6423','EPSG','1144',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5067','geodetic_crs','EPSG','6989','EPSG','2603','EPSG','1203');
INSERT INTO "geodetic_crs" VALUES('EPSG','6990','IG05/12 Intermediate CRS','Intermediate system not used for spatial referencing - use IGD05/12 (CRS code 6987). Referred to in Israeli documentation as "in GRS80".','geographic 2D','EPSG','6422','EPSG','1144',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5068','geodetic_crs','EPSG','6990','EPSG','2603','EPSG','1203');
INSERT INTO "geodetic_crs" VALUES('EPSG','7034','RGSPM06 (lon-lat)','See CRS code 4466 for system with horizontal axes in sequence lat-lon to be used for air, land and sea navigation and safety of life purposes.','geographic 3D','EPSG','6426','EPSG','1038',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5075','geodetic_crs','EPSG','7034','EPSG','1220','EPSG','1189');
INSERT INTO "geodetic_crs" VALUES('EPSG','7035','RGSPM06 (lon-lat)','Replaces Saint Pierre et Miquelon 1950 (CRS code 4638). See CRS code 4463 for system with axes in sequence lat-lon to be used for air, land and sea navigation and safety of life purposes.','geographic 2D','EPSG','6424','EPSG','1038',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5076','geodetic_crs','EPSG','7035','EPSG','1220','EPSG','1189');
INSERT INTO "geodetic_crs" VALUES('EPSG','7036','RGR92 (lon-lat)','See CRS code 4971 for system with horizontal axes in sequence lat-lon to be used for air, land and sea navigation and safety of life purposes.','geographic 3D','EPSG','6426','EPSG','6627',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5077','geodetic_crs','EPSG','7036','EPSG','3902','EPSG','1189');
INSERT INTO "geodetic_crs" VALUES('EPSG','7037','RGR92 (lon-lat)','Replaces Piton des Neiges (code 4626). See CRS code 4627 for system with axes in sequence lat-lon to be used for air, land and sea navigation and safety of life purposes.','geographic 2D','EPSG','6424','EPSG','6627',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5078','geodetic_crs','EPSG','7037','EPSG','3902','EPSG','1189');
INSERT INTO "geodetic_crs" VALUES('EPSG','7038','RGM04 (lon-lat)','Replaced by RGM23 (lon-lat) (CRS code 10672) with effect from 2023-01-01. See CRS code 4469 for system with horizontal axes in sequence lat-lon to be used for air, land and sea navigation and safety of life purposes.','geographic 3D','EPSG','6426','EPSG','1036',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5079','geodetic_crs','EPSG','7038','EPSG','1159','EPSG','1189');
INSERT INTO "geodetic_crs" VALUES('EPSG','7039','RGM04 (lon-lat)','Replaces Combani 1950. Replaced by RGM23 (lon-lat) from 2023-01-01 except for cadastral purposes which use Cadastre 1997. See CRS code 4470 for system with axes in sequence lat-lon to be used for air, land and sea navigation and safety of life purposes.','geographic 2D','EPSG','6424','EPSG','1036',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5080','geodetic_crs','EPSG','7039','EPSG','1159','EPSG','1189');
INSERT INTO "geodetic_crs" VALUES('EPSG','7040','RGFG95 (lon-lat)','See CRS code 4967 for system with horizontal axes in sequence lat-lon to be used for air, land and sea navigation and safety of life purposes.','geographic 3D','EPSG','6426','EPSG','6624',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5081','geodetic_crs','EPSG','7040','EPSG','1097','EPSG','1189');
INSERT INTO "geodetic_crs" VALUES('EPSG','7041','RGFG95 (lon-lat)','See CRS code 4624 for system with axes in sequence lat-lon to be used for air, land and sea navigation and safety of life purposes.','geographic 2D','EPSG','6424','EPSG','6624',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5082','geodetic_crs','EPSG','7041','EPSG','1097','EPSG','1189');
INSERT INTO "geodetic_crs" VALUES('EPSG','7042','RGF93 v1 (lon-lat)','See CRS code 4965 for system with horizontal axes in sequence lat-lon to be used for air, land and sea navigation and safety of life purposes. Replaced by RGF93 v2 (lon-lat) (CRS code 9778) from 2010-06-18.','geographic 3D','EPSG','6426','EPSG','6171',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5083','geodetic_crs','EPSG','7042','EPSG','1096','EPSG','1189');
INSERT INTO "geodetic_crs" VALUES('EPSG','7071','RGTAAF07','','geocentric','EPSG','6500','EPSG','1113',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5098','geodetic_crs','EPSG','7071','EPSG','4246','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','7072','RGTAAF07','See CRS code 7087 for alternate system with horizontal axes reversed used by IGN for GIS purposes.','geographic 3D','EPSG','6423','EPSG','1113',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5099','geodetic_crs','EPSG','7072','EPSG','4246','EPSG','1182');
INSERT INTO "geodetic_crs" VALUES('EPSG','7073','RGTAAF07','Replaces various local systems on several French overseas territories. See CRS code 7133 for alternate system with axes reversed used by IGN for GIS purposes.','geographic 2D','EPSG','6422','EPSG','1113',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5100','geodetic_crs','EPSG','7073','EPSG','4246','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','7084','RGF93 v1 (lon-lat)','See CRS code 4171 for system with axes in sequence lat-lon to be used for air, land and sea navigation and safety of life purposes. Replaced by RGF93 v2 (lon-lat) (CRS code 9779) from 2010-06-18.','geographic 2D','EPSG','6424','EPSG','6171',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5110','geodetic_crs','EPSG','7084','EPSG','1096','EPSG','1189');
INSERT INTO "geodetic_crs" VALUES('EPSG','7085','RGAF09 (lon-lat)','Replaces RRAF 1991 (CRS code 4557). See CRS code 5488 for system with horizontal axes in sequence lat-lon to be used for air, land and sea navigation and safety of life purposes.','geographic 3D','EPSG','6426','EPSG','1073',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5111','geodetic_crs','EPSG','7085','EPSG','2824','EPSG','1189');
INSERT INTO "geodetic_crs" VALUES('EPSG','7086','RGAF09 (lon-lat)','Replaces RRAF 1991. See CRS code 5489 for system with axes in sequence lat-lon to be used for air, land and sea navigation and safety of life purposes.','geographic 2D','EPSG','6424','EPSG','1073',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5112','geodetic_crs','EPSG','7086','EPSG','2824','EPSG','1189');
INSERT INTO "geodetic_crs" VALUES('EPSG','7087','RGTAAF07 (lon-lat)','See CRS code 7072 for alternate system with horizontal axes in sequence lat-lon to be used for air, land and sea navigation purposes.','geographic 3D','EPSG','6426','EPSG','1113',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5113','geodetic_crs','EPSG','7087','EPSG','4246','EPSG','1189');
INSERT INTO "geodetic_crs" VALUES('EPSG','7088','RGTAAF07 (lon-lat)','Replaces various local systems on several French overseas territories. See CRS code 7073 for alternate system with axes in sequence lat-lon to be used for air, land and sea navigation purposes.','geographic 2D','EPSG','6424','EPSG','1113',NULL,1);
INSERT INTO "usage" VALUES('EPSG','5114','geodetic_crs','EPSG','7088','EPSG','4246','EPSG','1189');
INSERT INTO "geodetic_crs" VALUES('EPSG','7133','RGTAAF07 (lon-lat)','Replaces various local systems on several French overseas territories. See CRS code 7073 for alternate system with axes in sequence lat-lon to be used for air, land and sea navigation purposes.','geographic 2D','EPSG','6424','EPSG','1113',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5137','geodetic_crs','EPSG','7133','EPSG','4246','EPSG','1189');
INSERT INTO "geodetic_crs" VALUES('EPSG','7134','IGD05','Replaced by IGD05/12 (CRS code 7137).','geocentric','EPSG','6500','EPSG','1114',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5138','geodetic_crs','EPSG','7134','EPSG','1126','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','7135','IGD05','Replaced by IGD05/12 (CRS code 7138).','geographic 3D','EPSG','6423','EPSG','1114',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5139','geodetic_crs','EPSG','7135','EPSG','1126','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','7136','IGD05','Replaces Israel 1993 (CRS code 4141) from January 2005. Replaced by IGD05/12 (CRS code 7139) from March 2012.','geographic 2D','EPSG','6422','EPSG','1114',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5140','geodetic_crs','EPSG','7136','EPSG','1126','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','7137','IGD05/12','Replaces IGD05 (CRS code 7134).','geocentric','EPSG','6500','EPSG','1115',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5141','geodetic_crs','EPSG','7137','EPSG','1126','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','7138','IGD05/12','Replaces IGD05 (CRS code 7135).','geographic 3D','EPSG','6423','EPSG','1115',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5142','geodetic_crs','EPSG','7138','EPSG','1126','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','7139','IGD05/12','Replaces IGD05 (CRS code 7136) from March 2012.','geographic 2D','EPSG','6422','EPSG','1115',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5143','geodetic_crs','EPSG','7139','EPSG','1126','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','7371','ONGD14','In Oman replaces usage of WGS 84 (G873) from 2014. Replaced by ONGD17 (CRS code 9292) from March 2019.','geocentric','EPSG','6500','EPSG','1147',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5259','geodetic_crs','EPSG','7371','EPSG','1183','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','7372','ONGD14','In Oman replaces usage of WGS 84 (G873) from 2014. Replaced by ONGD17 (CRS code 9293) from March 2019.','geographic 3D','EPSG','6423','EPSG','1147',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5260','geodetic_crs','EPSG','7372','EPSG','1183','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','7373','ONGD14','In Oman replaces usage of WGS 84 (G873) from 2014. Replaced by ONGD17 (CRS code 9294) from March 2019.','geographic 2D','EPSG','6422','EPSG','1147',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5261','geodetic_crs','EPSG','7373','EPSG','1183','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','7656','WGS 84 (G730)','Replaces WGS 84 (Transit) (CRS code 7815) from 1994-06-29. Replaced by WGS84 (G873) (CRS code 7658) from 1997-01-29.','geocentric','EPSG','6500','EPSG','1152',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5411','geodetic_crs','EPSG','7656','EPSG','1262','EPSG','1176');
INSERT INTO "geodetic_crs" VALUES('EPSG','7657','WGS 84 (G730)','Replaces WGS 84 (Transit) (CRS code 7816) from 1994-06-29. Replaced by WGS84 (G873) (CRS code 7659) from 1997-01-29.','geographic 3D','EPSG','6423','EPSG','1152',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5412','geodetic_crs','EPSG','7657','EPSG','1262','EPSG','1176');
INSERT INTO "geodetic_crs" VALUES('EPSG','7658','WGS 84 (G873)','Replaces WGS 84 (G730) (CRS code 7656) from 1997-01-29. Replaced by WGS 84 (G1150) (CRS code 7660) from 2002-01-20.','geocentric','EPSG','6500','EPSG','1153',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5413','geodetic_crs','EPSG','7658','EPSG','1262','EPSG','1176');
INSERT INTO "geodetic_crs" VALUES('EPSG','7659','WGS 84 (G873)','Replaces WGS 84 (G730) (CRS code 7657) from 1997-01-29. Replaced by WGS 84 (G1150) (CRS code 7661) from 2002-01-20.','geographic 3D','EPSG','6423','EPSG','1153',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5414','geodetic_crs','EPSG','7659','EPSG','1262','EPSG','1176');
INSERT INTO "geodetic_crs" VALUES('EPSG','7660','WGS 84 (G1150)','Replaces WGS 84 (G873) (CRS code 7658) from 2002-01-20. Replaced by WGS 84 (G1674) (CRS code 7662) from 2012-02-08.','geocentric','EPSG','6500','EPSG','1154',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5415','geodetic_crs','EPSG','7660','EPSG','1262','EPSG','1176');
INSERT INTO "geodetic_crs" VALUES('EPSG','7661','WGS 84 (G1150)','Replaces WGS 84 (G873) (CRS code 7659) from 2002-01-20. Replaced by WGS 84 (G1674) (CRS code 7663) from 2012-02-08.','geographic 3D','EPSG','6423','EPSG','1154',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5416','geodetic_crs','EPSG','7661','EPSG','1262','EPSG','1176');
INSERT INTO "geodetic_crs" VALUES('EPSG','7662','WGS 84 (G1674)','Replaces WGS 84 (G1150) (CRS code 7660) from 2012-02-08. Replaced by WGS 84 (G1762) (CRS code 7664) from 2013-10-16.','geocentric','EPSG','6500','EPSG','1155',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5417','geodetic_crs','EPSG','7662','EPSG','1262','EPSG','1176');
INSERT INTO "geodetic_crs" VALUES('EPSG','7663','WGS 84 (G1674)','Replaces WGS 84 (G1150) (CRS code 7661) from 2012-02-08. Replaced by WGS 84 (G1762) (CRS code 7665) from 2013-10-16.','geographic 3D','EPSG','6423','EPSG','1155',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5418','geodetic_crs','EPSG','7663','EPSG','1262','EPSG','1176');
INSERT INTO "geodetic_crs" VALUES('EPSG','7664','WGS 84 (G1762)','Replaces WGS 84 (G1674) (CRS code 7662) from 2013-10-16. Redesignated WGS 84 (G1762'') in 2015 after changes to 7 NGA tracking station locations and antennas. Replaced by WGS 84 (G2139) (CRS code 9753) from 2021-01-03.','geocentric','EPSG','6500','EPSG','1156',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5419','geodetic_crs','EPSG','7664','EPSG','1262','EPSG','1176');
INSERT INTO "geodetic_crs" VALUES('EPSG','7665','WGS 84 (G1762)','Replaces WGS 84 (G1674) (CRS code 7663) from 2013-10-16. Redesignated WGS 84 (G1762'') in 2015 after changes to 7 NGA tracking station locations and antennas. Replaced by WGS 84 (G2139) (CRS code 9754) from 2021-01-03.','geographic 3D','EPSG','6423','EPSG','1156',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5420','geodetic_crs','EPSG','7665','EPSG','1262','EPSG','1176');
INSERT INTO "geodetic_crs" VALUES('EPSG','7677','PZ-90.02','Replaces PZ-90 (CRS code 4922) from 2007-09-20. Replaced by PZ-90.11 (CRS code 7679) from 2014-01-15.','geocentric','EPSG','6500','EPSG','1157',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5421','geodetic_crs','EPSG','7677','EPSG','1262','EPSG','1177');
INSERT INTO "geodetic_crs" VALUES('EPSG','7678','PZ-90.02','Replaces PZ-90 (CRS code 4923) from 2007-09-20. Replaced by PZ-90.11 (CRS code 7680) from 2014-01-15.','geographic 3D','EPSG','6423','EPSG','1157',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5422','geodetic_crs','EPSG','7678','EPSG','1262','EPSG','1177');
INSERT INTO "geodetic_crs" VALUES('EPSG','7679','PZ-90.11','Replaces PZ-90.02 (CRS code 7677) from 2014-01-15.','geocentric','EPSG','6500','EPSG','1158',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5423','geodetic_crs','EPSG','7679','EPSG','1262','EPSG','1177');
INSERT INTO "geodetic_crs" VALUES('EPSG','7680','PZ-90.11','Replaces PZ-90.02 (CRS code 7678) from 2014-01-15.','geographic 3D','EPSG','6423','EPSG','1158',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5424','geodetic_crs','EPSG','7680','EPSG','1262','EPSG','1177');
INSERT INTO "geodetic_crs" VALUES('EPSG','7681','GSK-2011','','geocentric','EPSG','6500','EPSG','1159',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5425','geodetic_crs','EPSG','7681','EPSG','1198','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','7682','GSK-2011','','geographic 3D','EPSG','6423','EPSG','1159',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5426','geodetic_crs','EPSG','7682','EPSG','1198','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','7683','GSK-2011','Replaces Pulkovo 1995 (CRS code 4200) with effect from 21st October 2011.','geographic 2D','EPSG','6422','EPSG','1159',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5427','geodetic_crs','EPSG','7683','EPSG','1198','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','7684','Kyrg-06','','geocentric','EPSG','6500','EPSG','1160',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5428','geodetic_crs','EPSG','7684','EPSG','1137','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','7685','Kyrg-06','','geographic 3D','EPSG','6423','EPSG','1160',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5429','geodetic_crs','EPSG','7685','EPSG','1137','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','7686','Kyrg-06','Replaces usage of Pulkovo 1942 in Kyrgyzstan from 7th October 2010.','geographic 2D','EPSG','6422','EPSG','1160',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5430','geodetic_crs','EPSG','7686','EPSG','1137','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','7789','ITRF2014','Replaces ITRF2008 (CRS code 5332). Replaced by ITRF2020 (CRS code 9988).','geocentric','EPSG','6500','EPSG','1165',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5473','geodetic_crs','EPSG','7789','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','7796','BGS2005','Adopted 2010-07-29.','geocentric','EPSG','6500','EPSG','1167',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5479','geodetic_crs','EPSG','7796','EPSG','1056','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','7797','BGS2005','Adopted 2010-07-29.','geographic 3D','EPSG','6423','EPSG','1167',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5480','geodetic_crs','EPSG','7797','EPSG','1056','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','7798','BGS2005','Adopted 2010-07-29. Replaces earlier systems.','geographic 2D','EPSG','6422','EPSG','1167',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5481','geodetic_crs','EPSG','7798','EPSG','1056','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','7815','WGS 84 (Transit)','Replaced by WGS84 (G730) (CRS code 7656) from 1994-06-29.','geocentric','EPSG','6500','EPSG','1166',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5488','geodetic_crs','EPSG','7815','EPSG','1262','EPSG','1176');
INSERT INTO "geodetic_crs" VALUES('EPSG','7816','WGS 84 (Transit)','Replaced by WGS84 (G730) (CRS code 7657) from 1994-06-29.','geographic 3D','EPSG','6423','EPSG','1166',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5489','geodetic_crs','EPSG','7816','EPSG','1262','EPSG','1176');
INSERT INTO "geodetic_crs" VALUES('EPSG','7842','GDA2020','','geocentric','EPSG','6500','EPSG','1168',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5501','geodetic_crs','EPSG','7842','EPSG','4177','EPSG','1181');
INSERT INTO "geodetic_crs" VALUES('EPSG','7843','GDA2020','','geographic 3D','EPSG','6423','EPSG','1168',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5502','geodetic_crs','EPSG','7843','EPSG','4177','EPSG','1181');
INSERT INTO "geodetic_crs" VALUES('EPSG','7844','GDA2020','','geographic 2D','EPSG','6422','EPSG','1168',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5503','geodetic_crs','EPSG','7844','EPSG','4177','EPSG','1181');
INSERT INTO "geodetic_crs" VALUES('EPSG','7879','St. Helena Tritan','Replaced by SHGD2015 (CRS code 7884) from 2015.','geocentric','EPSG','6500','EPSG','1173',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5521','geodetic_crs','EPSG','7879','EPSG','3183','EPSG','1146');
INSERT INTO "geodetic_crs" VALUES('EPSG','7880','St. Helena Tritan','Closely aligned to SHGD2015 (CRS code 7885) with difference attributable to different reference epoch and 10 cm difference in ellipsoid height. Replaced by SHGD2015 from 2015.','geographic 3D','EPSG','6423','EPSG','1173',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5522','geodetic_crs','EPSG','7880','EPSG','3183','EPSG','1146');
INSERT INTO "geodetic_crs" VALUES('EPSG','7881','St. Helena Tritan','Replaced by SHGD2015 (CRS code 7886) from 2015.','geographic 2D','EPSG','6422','EPSG','1173',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5523','geodetic_crs','EPSG','7881','EPSG','3183','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','7884','SHGD2015','','geocentric','EPSG','6500','EPSG','1174',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5526','geodetic_crs','EPSG','7884','EPSG','3183','EPSG','1146');
INSERT INTO "geodetic_crs" VALUES('EPSG','7885','SHGD2015','Closely aligned to SHGD2015 (CRS code xxxx) with difference attributable to different reference epoch and 10 cm difference in ellipsoid height.','geographic 3D','EPSG','6423','EPSG','1174',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5527','geodetic_crs','EPSG','7885','EPSG','3183','EPSG','1146');
INSERT INTO "geodetic_crs" VALUES('EPSG','7886','SHGD2015','','geographic 2D','EPSG','6422','EPSG','1174',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5528','geodetic_crs','EPSG','7886','EPSG','3183','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','7900','ITRF88','Replaced by ITRF89 (code 7901).','geographic 3D','EPSG','6423','EPSG','6647',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5534','geodetic_crs','EPSG','7900','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','7901','ITRF89','Replaces ITRF88 (code 7900). Replaced by ITRF90 (code 7902).','geographic 3D','EPSG','6423','EPSG','6648',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5535','geodetic_crs','EPSG','7901','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','7902','ITRF90','Replaces ITRF89 (code 7901). Replaced by ITRF91 (code 7903).','geographic 3D','EPSG','6423','EPSG','6649',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5536','geodetic_crs','EPSG','7902','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','7903','ITRF91','Replaces ITRF90 (code 7902). Replaced by ITRF92 (code 7904).','geographic 3D','EPSG','6423','EPSG','6650',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5537','geodetic_crs','EPSG','7903','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','7904','ITRF92','Replaces ITRF91 (code 7903). Replaced by ITRF93 (code 7905).','geographic 3D','EPSG','6423','EPSG','6651',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5538','geodetic_crs','EPSG','7904','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','7905','ITRF93','Replaces ITRF92 (code 7904). Replaced by ITRF94 (code 7906).','geographic 3D','EPSG','6423','EPSG','6652',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5539','geodetic_crs','EPSG','7905','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','7906','ITRF94','Replaces ITRF93 (code 7905). Replaced by ITRF96 (code 7907).','geographic 3D','EPSG','6423','EPSG','6653',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5540','geodetic_crs','EPSG','7906','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','7907','ITRF96','Replaces ITRF94 (code 7906). Replaced by ITRF97 (code 7908).','geographic 3D','EPSG','6423','EPSG','6654',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5541','geodetic_crs','EPSG','7907','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','7908','ITRF97','Replaces ITRF96 (code 7907). Replaced by ITRF2000 (code 7909).','geographic 3D','EPSG','6423','EPSG','6655',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5542','geodetic_crs','EPSG','7908','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','7909','ITRF2000','Replaces ITRF97 (code 7908). Replaced by ITRF2005 (code 7910).','geographic 3D','EPSG','6423','EPSG','6656',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5543','geodetic_crs','EPSG','7909','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','7910','ITRF2005','Replaces ITRF2000 (code 7909). Replaced by ITRF2008 (code 7911).','geographic 3D','EPSG','6423','EPSG','6896',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5544','geodetic_crs','EPSG','7910','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','7911','ITRF2008','Replaces ITRF2005 (code 7910). Replaced by ITRF2014 (code 7912).','geographic 3D','EPSG','6423','EPSG','1061',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5545','geodetic_crs','EPSG','7911','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','7912','ITRF2014','Replaces ITRF2008 (code 7911). Replaced by ITRF2020 (CRS code 9989).','geographic 3D','EPSG','6423','EPSG','1165',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5546','geodetic_crs','EPSG','7912','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','7914','ETRF89','Replaced by ETRF90 (code 7916).','geocentric','EPSG','6500','EPSG','1178',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5547','geodetic_crs','EPSG','7914','EPSG','1298','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','7915','ETRF89','Replaced by ETRF90 (code 7917).','geographic 3D','EPSG','6423','EPSG','1178',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5548','geodetic_crs','EPSG','7915','EPSG','1298','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','7916','ETRF90','Replaces ETRF89 (code 7914). Replaced by ETRF91 (code 7918).','geocentric','EPSG','6500','EPSG','1179',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5549','geodetic_crs','EPSG','7916','EPSG','1298','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','7917','ETRF90','Replaces ETRF89 (code 7915). Replaced by ETRF91 (code 7919).','geographic 3D','EPSG','6423','EPSG','1179',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5550','geodetic_crs','EPSG','7917','EPSG','1298','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','7918','ETRF91','Replaces ETRF90 (code 7916). Replaced by ETRF92 (code 7920).','geocentric','EPSG','6500','EPSG','1180',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5551','geodetic_crs','EPSG','7918','EPSG','1298','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','7919','ETRF91','Replaces ETRF90 (code 7917). Replaced by ETRF92 (code 7921).','geographic 3D','EPSG','6423','EPSG','1180',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5552','geodetic_crs','EPSG','7919','EPSG','1298','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','7920','ETRF92','Replaces ETRF91 (code 7918). Replaced by ETRF93 (code 7922).','geocentric','EPSG','6500','EPSG','1181',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5553','geodetic_crs','EPSG','7920','EPSG','1298','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','7921','ETRF92','Replaces ETRF91 (code 7919). Replaced by ETRF93 (code 7923).','geographic 3D','EPSG','6423','EPSG','1181',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5554','geodetic_crs','EPSG','7921','EPSG','1298','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','7922','ETRF93','Replaces ETRF92 (code 7920). Replaced by ETRF94 (code 7924).','geocentric','EPSG','6500','EPSG','1182',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5555','geodetic_crs','EPSG','7922','EPSG','1298','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','7923','ETRF93','Replaces ETRF92 (code 7921). Replaced by ETRF94 (code 7925).','geographic 3D','EPSG','6423','EPSG','1182',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5556','geodetic_crs','EPSG','7923','EPSG','1298','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','7924','ETRF94','Replaces ETRF93 (code 7922). Replaced by ETRF96 (code 7926).','geocentric','EPSG','6500','EPSG','1183',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5557','geodetic_crs','EPSG','7924','EPSG','1298','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','7925','ETRF94','Replaces ETRF93 (code 7923). Replaced by ETRF96 (code 7927).','geographic 3D','EPSG','6423','EPSG','1183',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5558','geodetic_crs','EPSG','7925','EPSG','1298','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','7926','ETRF96','Replaces ETRF94 (code 7924). Replaced by ETRF97 (code 7928).','geocentric','EPSG','6500','EPSG','1184',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5559','geodetic_crs','EPSG','7926','EPSG','1298','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','7927','ETRF96','Replaces ETRF94 (code 7925). Replaced by ETRF97 (code 7929).','geographic 3D','EPSG','6423','EPSG','1184',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5560','geodetic_crs','EPSG','7927','EPSG','1298','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','7928','ETRF97','Replaces ETRF96 (code 7926). Replaced by ETRF2000 (code 7930).','geocentric','EPSG','6500','EPSG','1185',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5561','geodetic_crs','EPSG','7928','EPSG','1298','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','7929','ETRF97','Replaces ETRF96 (code 7927). Replaced by ETRF2000 (code 7931).','geographic 3D','EPSG','6423','EPSG','1185',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5562','geodetic_crs','EPSG','7929','EPSG','1298','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','7930','ETRF2000','Replaces ETRF97 (code 7928). On the publication of ETRF2005 the EUREF Technical Working Group recommended ETRF2000 be the realization of ETRS89. ETRF2014 and ETRF2020 (codes 8401 and 10569) are technically superior to all earlier realizations of ETRS89.','geocentric','EPSG','6500','EPSG','1186',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5563','geodetic_crs','EPSG','7930','EPSG','4755','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','7931','ETRF2000','Replaces ETRF97 (code 7929). On the publication of ETRF2005 the EUREF Technical Working Group recommended ETRF2000 be the realization of ETRS89. ETRF2014 and ETRF2020 (codes 8403 and 10570) are technically superior to all earlier realizations of ETRS89.','geographic 3D','EPSG','6423','EPSG','1186',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5564','geodetic_crs','EPSG','7931','EPSG','1298','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8042','Gusterberg (Ferro)','','geographic 2D','EPSG','6422','EPSG','1188',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5596','geodetic_crs','EPSG','8042','EPSG','4455','EPSG','1028');
INSERT INTO "geodetic_crs" VALUES('EPSG','8043','St. Stephen (Ferro)','','geographic 2D','EPSG','6422','EPSG','1189',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5597','geodetic_crs','EPSG','8043','EPSG','4456','EPSG','1028');
INSERT INTO "geodetic_crs" VALUES('EPSG','8084','ISN2016','Replaces ISN2004 (CRS code 5322) from September 2017.','geocentric','EPSG','6500','EPSG','1187',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5612','geodetic_crs','EPSG','8084','EPSG','1120','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8085','ISN2016','Replaces ISN2004 (CRS code 5323) from September 2017.','geographic 3D','EPSG','6423','EPSG','1187',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5613','geodetic_crs','EPSG','8085','EPSG','1120','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8086','ISN2016','Replaces ISN2004 (CRS code 5324) from September 2017.','geographic 2D','EPSG','6422','EPSG','1187',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5614','geodetic_crs','EPSG','8086','EPSG','1120','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','8227','IGS14','Used for products from the International GNSS Service (IGS) from 2017-01-29 to 2020-05-16. Replaces IGb08 (code 9015), replaced by IGb14 (code 9378). For most practical purposes IGS14 is equivalent to ITRF2014.','geocentric','EPSG','6500','EPSG','1191',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5735','geodetic_crs','EPSG','8227','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8230','NAD83(CSRS96)','Adopted by the Canadian federal government from 1996-01-01. Replaced by NAD83(CSRS)v2 (CRS code 8233).','geocentric','EPSG','6500','EPSG','1192',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5737','geodetic_crs','EPSG','8230','EPSG','1061','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8231','NAD83(CSRS96)','Adopted by the Canadian federal government from 1996-01-01. Replaced by NAD83(CSRS)v2 (CRS code 8235). Note: this CRS includes longitudes which are POSITIVE EAST.','geographic 3D','EPSG','6423','EPSG','1192',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5738','geodetic_crs','EPSG','8231','EPSG','1061','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8232','NAD83(CSRS96)','Adopted by the Canadian federal government from 1996-01-01. Replaced by NAD83(CSRS)v2 (CRS code 8237). Note: this CRS includes longitudes which are POSITIVE EAST.','geographic 2D','EPSG','6422','EPSG','1192',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5739','geodetic_crs','EPSG','8232','EPSG','1061','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','8233','NAD83(CSRS)v2','Adopted by the Canadian federal government from 1998-01-01 and by the provincial governments of British Columbia, New Brunswick, Prince Edward Island and Quebec. Replaces NAD83(CSRS96). Replaced by NAD83(CSRS)v3 (CRS code 8238).','geocentric','EPSG','6500','EPSG','1193',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5740','geodetic_crs','EPSG','8233','EPSG','1061','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8235','NAD83(CSRS)v2','Adopted by the Canadian federal government from 1998-01-01 and by the provincial governments of British Columbia, New Brunswick, Prince Edward Island and Quebec. Replaces NAD83(CSRS96). Replaced by NAD83(CSRS)v3 (code 8239). Longitudes are POSITIVE EAST.','geographic 3D','EPSG','6423','EPSG','1193',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5741','geodetic_crs','EPSG','8235','EPSG','1061','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8237','NAD83(CSRS)v2','Adopted by the Canadian federal government from 1998-01-01 and by the provincial governments of British Columbia, New Brunswick, Prince Edward Island and Quebec. Replaces NAD83(CSRS96). Replaced by NAD83(CSRS)v3 (code 8240). Longitudes are POSITIVE EAST.','geographic 2D','EPSG','6422','EPSG','1193',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5742','geodetic_crs','EPSG','8237','EPSG','1061','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','8238','NAD83(CSRS)v3','Adopted by the Canadian federal government from 1999-01-01 and by the provincial governments of Alberta, British Columbia, Manitoba, Newfoundland and Labrador, Nova Scotia, Ontario and Saskatchewan. Replaces NAD83(CSRS)v2. Replaced by NAD83(CSRS)v4.','geocentric','EPSG','6500','EPSG','1194',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5743','geodetic_crs','EPSG','8238','EPSG','1061','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8239','NAD83(CSRS)v3','Adopted by the Canadian federal government from 1999-01-01 and by the provincial governments of Alberta, British Columbia, Manitoba, Newfoundland and Labrador, Nova Scotia, Ontario and Saskatchewan. Replaces NAD83(CSRS)v2. Replaced by NAD83(CSRS)v4.','geographic 3D','EPSG','6423','EPSG','1194',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5744','geodetic_crs','EPSG','8239','EPSG','1061','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8240','NAD83(CSRS)v3','Adopted by the Canadian federal government from 1999-01-01 and by the provincial governments of Alberta, British Columbia, Manitoba, Newfoundland and Labrador, Nova Scotia, Ontario and Saskatchewan. Replaces NAD83(CSRS)v2. Replaced by NAD83(CSRS)v4.','geographic 2D','EPSG','6422','EPSG','1194',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5745','geodetic_crs','EPSG','8240','EPSG','1061','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','8242','NAD83(CSRS)v4','Adopted by the Canadian federal government from 2002-01-01 and by the provincial governments of Alberta and British Columbia. Replaces NAD83(CSRS)v3. Replaced by NAD83(CSRS)v5 (CRS code 8247).','geocentric','EPSG','6500','EPSG','1195',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5746','geodetic_crs','EPSG','8242','EPSG','1061','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8244','NAD83(CSRS)v4','Adopted by the Canadian federal government from 2002-01-01 and by the provincial governments of Alberta and British Columbia. Replaces NAD83(CSRS)v3. Replaced by NAD83(CSRS)v5 (CRS code 8248). Longitudes are POSITIVE EAST.','geographic 3D','EPSG','6423','EPSG','1195',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5747','geodetic_crs','EPSG','8244','EPSG','1061','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8246','NAD83(CSRS)v4','Adopted by the Canadian federal government from 2002-01-01 and by the provincial governments of Alberta and British Columbia. Replaces NAD83(CSRS)v3. Replaced by NAD83(CSRS)v5 (CRS code 8249). Longitudes are POSITIVE EAST.','geographic 2D','EPSG','6422','EPSG','1195',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5748','geodetic_crs','EPSG','8246','EPSG','1061','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','8247','NAD83(CSRS)v5','Adopted by the Canadian federal government from 2006-01-01. Replaces NAD83(CSRS)v4. Replaced by NAD83(CSRS)v6 (CRS code 8250).','geocentric','EPSG','6500','EPSG','1196',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5749','geodetic_crs','EPSG','8247','EPSG','1061','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8248','NAD83(CSRS)v5','Adopted by the Canadian federal government from 2006-01-01. Replaces NAD83(CSRS)v4. Replaced by NAD83(CSRS)v6. Longitudes are POSITIVE EAST.','geographic 3D','EPSG','6423','EPSG','1196',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5750','geodetic_crs','EPSG','8248','EPSG','1061','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8249','NAD83(CSRS)v5','Adopted by the Canadian federal government from 2006-01-01. Replaces NAD83(CSRS)v4. Replaced by NAD83(CSRS)v6 (CRS code 8252). Longitudes are POSITIVE EAST.','geographic 2D','EPSG','6422','EPSG','1196',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5751','geodetic_crs','EPSG','8249','EPSG','1061','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','8250','NAD83(CSRS)v6','Adopted by the Canadian federal government from 2010-01-01 and the provincial governments of Alberta, British Columbia, Manitoba, Newfoundland and Labrador, Nova Scotia, Ontario and Prince Edward Island. Replaces NAD83(CSRS)v5. Replaced by NAD83(CSRS)v7.','geocentric','EPSG','6500','EPSG','1197',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5752','geodetic_crs','EPSG','8250','EPSG','1061','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8251','NAD83(CSRS)v6','Adopted by the Canadian federal government from 2010-01-01 and the provincial governments of Alberta, British Columbia, Manitoba, Newfoundland and Labrador, Nova Scotia, Ontario and Prince Edward Island. Replaces NAD83(CSRS)v5. Replaced by NAD83(CSRS)v7.','geographic 3D','EPSG','6423','EPSG','1197',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5753','geodetic_crs','EPSG','8251','EPSG','1061','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8252','NAD83(CSRS)v6','Adopted by the Canadian federal government from 2010-01-01 and the provincial governments of Alberta, British Columbia, Manitoba, Newfoundland and Labrador, Nova Scotia, Ontario and Prince Edward Island. Replaces NAD83(CSRS)v5. Replaced by NAD83(CSRS)v7.','geographic 2D','EPSG','6422','EPSG','1197',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5754','geodetic_crs','EPSG','8252','EPSG','1061','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','8253','NAD83(CSRS)v7','Adopted by the Canadian federal government from 2017-05-01 and the provincial government of Alberta. Replaces NAD83(CSRS)v6. Replaced by NAD83(CSRS)v8.','geocentric','EPSG','6500','EPSG','1198',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5755','geodetic_crs','EPSG','8253','EPSG','1061','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8254','NAD83(CSRS)v7','Adopted by the Canadian federal government from 2017-05-01 and the provincial government of Alberta. Replaces NAD83(CSRS)v6. Replaced by NAD83(CSRS)v8. Longitudes are POSITIVE EAST.','geographic 3D','EPSG','6423','EPSG','1198',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5756','geodetic_crs','EPSG','8254','EPSG','1061','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8255','NAD83(CSRS)v7','Adopted by the Canadian federal government from 2017-05-01 and the provincial government of Alberta. Replaces NAD83(CSRS)v6. Replaced by NAD83(CSRS)v8. Longitudes are POSITIVE EAST.','geographic 2D','EPSG','6422','EPSG','1198',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5757','geodetic_crs','EPSG','8255','EPSG','1061','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','8351','S-JTSK [JTSK03]','Defined by transformation from ETRS89 (ETRF2000 realization) (transformation code 8365) to improve the scale and homogeneity of S-JTSK (CRS 4156) within Slovakia.','geographic 2D','EPSG','6422','EPSG','1201',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5800','geodetic_crs','EPSG','8351','EPSG','1211','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8397','ETRF2005','On publication in 2007 of this CRS, the EUREF Technical Working Group recommended that ETRF2000 (EPSG code 7930) remained as the preferred realization of ETRS89. Replaced by ETRF2014 (code 8401).','geocentric','EPSG','6500','EPSG','1204',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5819','geodetic_crs','EPSG','8397','EPSG','1298','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8399','ETRF2005','On publication in 2007 of this CRS, the EUREF Technical Working Group recommended that ETRF2000 (EPSG code 7931) remained as the preferred realization of ETRS89. Replaced by ETRF2014 (code 8403).','geographic 3D','EPSG','6423','EPSG','1204',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5820','geodetic_crs','EPSG','8399','EPSG','1298','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8401','ETRF2014','Replaces ETRF2005 (code 8397). ETRF2014 is technically superior to ETRF2000 but ETRF2000 and other previous realizations may be preferred for backward compatibility reasons. Differences between ETRF2014 and ETRF2000 can reach 7cm.','geocentric','EPSG','6500','EPSG','1206',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5821','geodetic_crs','EPSG','8401','EPSG','1298','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8403','ETRF2014','Replaces ETRF2005 (code 8399). ETRF2014 is technically superior to ETRF2000 but ETRF2000 and other previous realizations may be preferred for backward compatibility reasons. Differences between ETRF2014 and ETRF2000 can reach 7cm.','geographic 3D','EPSG','6423','EPSG','1206',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5822','geodetic_crs','EPSG','8403','EPSG','1298','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8425','Hong Kong Geodetic CS','Locally sometimes referred to as ITRF96 or WGS 84, these are not strictly correct.','geocentric','EPSG','6500','EPSG','1209',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5823','geodetic_crs','EPSG','8425','EPSG','1118','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8426','Hong Kong Geodetic CS','Locally sometimes referred to as ITRF96 or WGS 84, these are not strictly correct.','geographic 3D','EPSG','6423','EPSG','1209',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5824','geodetic_crs','EPSG','8426','EPSG','1118','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8427','Hong Kong Geodetic CS','Locally sometimes referred to as ITRF96 or WGS 84, these are not strictly correct.','geographic 2D','EPSG','6422','EPSG','1209',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5825','geodetic_crs','EPSG','8427','EPSG','1118','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','8428','Macao 1920','','geographic 2D','EPSG','6422','EPSG','1207',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5826','geodetic_crs','EPSG','8428','EPSG','1147','EPSG','1181');
INSERT INTO "geodetic_crs" VALUES('EPSG','8429','Macao 2008','Locally sometimes referred to as ITRF2005, this is not strictly correct.','geocentric','EPSG','6500','EPSG','1208',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5827','geodetic_crs','EPSG','8429','EPSG','1147','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8430','Macao 2008','Locally sometimes referred to as ITRF2005, this is not strictly correct.','geographic 3D','EPSG','6423','EPSG','1208',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5828','geodetic_crs','EPSG','8430','EPSG','1147','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8431','Macao 2008','Locally sometimes referred to as ITRF2005, this is not strictly correct.','geographic 2D','EPSG','6422','EPSG','1208',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5829','geodetic_crs','EPSG','8431','EPSG','1147','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','8449','NAD83(FBN)','In Continental US (excludes Alaska) and Hawaii, replaces NAD83 for applications with an accuracy of better than 1m. In Continental US, Puerto Rico and US Virgin Islands replaced by NAD83(NSRS2007). In American Samoa and Hawaii replaced by NAD83(PA11).','geographic 2D','EPSG','6423','EPSG','6152',NULL,1);
INSERT INTO "usage" VALUES('EPSG','5833','geodetic_crs','EPSG','8449','EPSG','4515','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8541','NAD83(FBN)','In Continental US, American Samoa, Guam/NMI and PRVI, replaces NAD83(HARN). In Continental US, Puerto Rico and US Virgin Islands replaced by NAD83(NSRS2007). In American Samoa and Hawaii replaced by NAD83(PA11). In Guam/NMI replaced by NAD83(MA11).','geocentric','EPSG','6500','EPSG','1211',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5856','geodetic_crs','EPSG','8541','EPSG','4515','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8542','NAD83(FBN)','Continental US, American Samoa, Guam/NMI and PRVI, replaces NAD83(HARN). In Continental US, Puerto Rico and US Virgin Islands replaced by NAD83(NSRS2007). In American Samoa and Hawaii replaced by NAD83(PA11). In Guam/NMI replaced by NAD83(MA11).','geographic 3D','EPSG','6423','EPSG','1211',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5857','geodetic_crs','EPSG','8542','EPSG','4515','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8543','NAD83(HARN Corrected)','In PRVI replaces NAD83(HARN) = NAD83(1993 PRVI) to correct errors. Replaced by NAD83(FBN) = NAD83(2002 PRVI).','geocentric','EPSG','6500','EPSG','1212',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5858','geodetic_crs','EPSG','8543','EPSG','3634','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8544','NAD83(HARN Corrected)','Note: this CRS includes POSITIVE EAST longitudes. In PRVI replaces NAD83(HARN) = NAD83(1993 PRVI) to correct errors. Replaced by NAD83(FBN) = NAD83(2002 PRVI).','geographic 3D','EPSG','6423','EPSG','1212',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5859','geodetic_crs','EPSG','8544','EPSG','3634','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8545','NAD83(HARN Corrected)','Note: this CRS includes POSITIVE EAST longitudes. In PRVI replaces NAD83(HARN) = NAD83(1993 PRVI) to correct errors. Replaced by NAD83(FBN) = NAD83(2002 PRVI).','geographic 2D','EPSG','6422','EPSG','1212',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5860','geodetic_crs','EPSG','8545','EPSG','3634','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','8683','SRB_ETRS89','Replaces SREF98 (CRS code 4073).','geocentric','EPSG','6500','EPSG','1214',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5866','geodetic_crs','EPSG','8683','EPSG','4543','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8684','SRB_ETRS89','Replaces SREF98 (CRS code 4074).','geographic 3D','EPSG','6423','EPSG','1214',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5867','geodetic_crs','EPSG','8684','EPSG','4543','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8685','SRB_ETRS89','In Serbia replaces MGI 1901 and SREF98 (CRS codes 3906 and 4075).','geographic 2D','EPSG','6422','EPSG','1214',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5868','geodetic_crs','EPSG','8685','EPSG','4543','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','8694','Camacupa 2015','Camacupa 1948 (CRS code 4220) is used for offshore oil and gas exploration and production. Camacupa 2015 differs from Camacupa 1948 by up to 25m.','geographic 2D','EPSG','6422','EPSG','1217',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5875','geodetic_crs','EPSG','8694','EPSG','1029','EPSG','1130');
INSERT INTO "geodetic_crs" VALUES('EPSG','8697','RSAO13','','geocentric','EPSG','6500','EPSG','1220',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5876','geodetic_crs','EPSG','8697','EPSG','1029','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8698','RSAO13','','geographic 3D','EPSG','6423','EPSG','1220',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5877','geodetic_crs','EPSG','8698','EPSG','1029','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8699','RSAO13','','geographic 2D','EPSG','6422','EPSG','1220',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5878','geodetic_crs','EPSG','8699','EPSG','1029','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','8816','MTRF-2000','','geocentric','EPSG','6500','EPSG','1218',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5995','geodetic_crs','EPSG','8816','EPSG','1206','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8817','MTRF-2000','','geographic 3D','EPSG','6423','EPSG','1218',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5996','geodetic_crs','EPSG','8817','EPSG','1206','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8818','MTRF-2000','Replaces Ain el Abd (CRS 4204) in Saudi Arabia.','geographic 2D','EPSG','6422','EPSG','1218',NULL,0);
INSERT INTO "usage" VALUES('EPSG','5997','geodetic_crs','EPSG','8818','EPSG','1206','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','8860','NAD83(FBN)','In Continental US, American Samoa, Guam/NMI and PRVI, replaces NAD83(HARN). In Continental US, Puerto Rico and US Virgin Islands replaced by NAD83(NSRS2007). In American Samoa and Hawaii replaced by NAD83(PA11). In Guam/NMI replaced by NAD83(MA11).','geographic 2D','EPSG','6422','EPSG','1211',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6010','geodetic_crs','EPSG','8860','EPSG','4515','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8888','WGS 84 (Transit)','Replaced by WGS84 (G730) (CRS code 9053) from 1994-06-29.','geographic 2D','EPSG','6422','EPSG','1166',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6011','geodetic_crs','EPSG','8888','EPSG','1262','EPSG','1176');
INSERT INTO "geodetic_crs" VALUES('EPSG','8898','RGWF96','','geocentric','EPSG','6500','EPSG','1223',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6014','geodetic_crs','EPSG','8898','EPSG','1255','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8899','RGWF96','See CRS code 8901 for alternate system with horizontal axes reversed used by IGN for GIS purposes.','geographic 3D','EPSG','6423','EPSG','1223',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6015','geodetic_crs','EPSG','8899','EPSG','1255','EPSG','1182');
INSERT INTO "geodetic_crs" VALUES('EPSG','8900','RGWF96','See CRS code 8902 for alternate system with axes reversed used by IGN for GIS purposes. On Wallis island, replaces MOP78 (CRS code 4639) for geodetic purposes.','geographic 2D','EPSG','6422','EPSG','1223',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6016','geodetic_crs','EPSG','8900','EPSG','1255','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','8901','RGWF96 (lon-lat)','See CRS code 8899 for system with horizontal axes in sequence lat-lon to be used for air, land and sea navigation and safety of life purposes.','geographic 3D','EPSG','6426','EPSG','1223',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6017','geodetic_crs','EPSG','8901','EPSG','1255','EPSG','1189');
INSERT INTO "geodetic_crs" VALUES('EPSG','8902','RGWF96 (lon-lat)','See CRS code 8900 for system with axes in sequence lat-lon to be used for air, land and sea navigation and safety of life purposes. On Wallis island, replaces MOP78 (CRS code 4639) for GIS purposes.','geographic 2D','EPSG','6424','EPSG','1223',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6018','geodetic_crs','EPSG','8902','EPSG','1255','EPSG','1189');
INSERT INTO "geodetic_crs" VALUES('EPSG','8905','CR-SIRGAS epoch 2014.59','Replaces CR05 (CRS code 5363) from April 2018.','geocentric','EPSG','6500','EPSG','1225',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6021','geodetic_crs','EPSG','8905','EPSG','1074','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8906','CR-SIRGAS epoch 2014.59','Replaces CR05 (CRS code 5364) from April 2018.','geographic 3D','EPSG','6423','EPSG','1225',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6022','geodetic_crs','EPSG','8906','EPSG','1074','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8907','CR-SIRGAS epoch 2014.59','Replaces CR05 (CRS code 5365) from April 2018.','geographic 2D','EPSG','6422','EPSG','1225',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6023','geodetic_crs','EPSG','8907','EPSG','1074','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','8915','SIRGAS-CON DGF00P01','Replaced by SIRGAS-CON DGF01P01 (CRS code 8917).','geocentric','EPSG','6500','EPSG','1227',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6029','geodetic_crs','EPSG','8915','EPSG','4530','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8916','SIRGAS-CON DGF00P01','Replaced by SIRGAS-CON DGF01P01 (CRS code 8918).','geographic 3D','EPSG','6423','EPSG','1227',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6030','geodetic_crs','EPSG','8916','EPSG','4530','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8917','SIRGAS-CON DGF01P01','Replaces SIRGAS-CON DGF00P01 (CRS code 8915). Replaced by SIRGAS-CON DGF01P02 (CRS code 8919).','geocentric','EPSG','6500','EPSG','1228',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6031','geodetic_crs','EPSG','8917','EPSG','4530','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8918','SIRGAS-CON DGF01P01','Replaces SIRGAS-CON DGF00P01 (CRS code 8916). Replaced by SIRGAS-CON DGF01P02 (CRS code 8920).','geographic 3D','EPSG','6423','EPSG','1228',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6032','geodetic_crs','EPSG','8918','EPSG','4530','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8919','SIRGAS-CON DGF01P02','Replaces SIRGAS-CON DGF01P01 (CRS code 8917). Replaced by SIRGAS-CON DGF02P01 (CRS code 8921).','geocentric','EPSG','6500','EPSG','1229',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6033','geodetic_crs','EPSG','8919','EPSG','4530','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8920','SIRGAS-CON DGF01P02','Replaces SIRGAS-CON DGF01P01 (CRS code 8918). Replaced by SIRGAS-CON DGF02P01 (CRS code 8922).','geographic 3D','EPSG','6423','EPSG','1229',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6034','geodetic_crs','EPSG','8920','EPSG','4530','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8921','SIRGAS-CON DGF02P01','Replaces SIRGAS-CON DGF01P02 (CRS code 8919). Replaced by SIRGAS-CON DGF04P01 (CRS code 8923).','geocentric','EPSG','6500','EPSG','1230',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6035','geodetic_crs','EPSG','8921','EPSG','4530','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8922','SIRGAS-CON DGF02P01','Replaces SIRGAS-CON DGF01P02 (CRS code 8920). Replaced by SIRGAS-CON DGF04P01 (CRS code 8924).','geographic 3D','EPSG','6423','EPSG','1230',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6036','geodetic_crs','EPSG','8922','EPSG','4530','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8923','SIRGAS-CON DGF04P01','Replaces SIRGAS-CON DGF02P01 (CRS code 8921). Replaced by SIRGAS-CON DGF05P01 (CRS code 8925).','geocentric','EPSG','6500','EPSG','1231',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6037','geodetic_crs','EPSG','8923','EPSG','4530','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8924','SIRGAS-CON DGF04P01','Replaces SIRGAS-CON DGF02P01 (CRS code 8922). Replaced by SIRGAS-CON DGF05P01 (CRS code 8926).','geographic 3D','EPSG','6423','EPSG','1231',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6038','geodetic_crs','EPSG','8924','EPSG','4530','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8925','SIRGAS-CON DGF05P01','Replaces SIRGAS-CON DGF04P01 (CRS code 8923). Replaced by SIRGAS-CON DGF06P01 (CRS code 8927).','geocentric','EPSG','6500','EPSG','1232',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6039','geodetic_crs','EPSG','8925','EPSG','4530','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8926','SIRGAS-CON DGF05P01','Replaces SIRGAS-CON DGF04P01 (CRS code 8924). Replaced by SIRGAS-CON DGF06P01 (CRS code 8928).','geographic 3D','EPSG','6423','EPSG','1232',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6040','geodetic_crs','EPSG','8926','EPSG','4530','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8927','SIRGAS-CON DGF06P01','Replaces SIRGAS-CON DGF05P01 (CRS code 8925). Replaced by SIRGAS-CON DGF07P01 (CRS code 8929).','geocentric','EPSG','6500','EPSG','1233',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6041','geodetic_crs','EPSG','8927','EPSG','4530','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8928','SIRGAS-CON DGF06P01','Replaces SIRGAS-CON DGF05P01 (CRS code 8926). Replaced by SIRGAS-CON DGF07P01 (CRS code 8930).','geographic 3D','EPSG','6423','EPSG','1233',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6042','geodetic_crs','EPSG','8928','EPSG','4530','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8929','SIRGAS-CON DGF07P01','Replaces SIRGAS-CON DGF06P01 (CRS code 8927). Replaced by SIRGAS-CON DGF08P01 (CRS code 8931).','geocentric','EPSG','6500','EPSG','1234',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6043','geodetic_crs','EPSG','8929','EPSG','4530','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8930','SIRGAS-CON DGF07P01','Replaces SIRGAS-CON DGF06P01 (CRS code 8928). Replaced by SIRGAS-CON DGF08P01 (CRS code 8932).','geographic 3D','EPSG','6423','EPSG','1234',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6044','geodetic_crs','EPSG','8930','EPSG','4530','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8931','SIRGAS-CON DGF08P01','Replaces SIRGAS-CON DGF07P01 (CRS code 8929). Replaced by SIRGAS-CON SIR09P01 (CRS code 8933).','geocentric','EPSG','6500','EPSG','1235',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6045','geodetic_crs','EPSG','8931','EPSG','4530','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8932','SIRGAS-CON DGF08P01','Replaces SIRGAS-CON DGF07P01 (CRS code 8930). Replaced by SIRGAS-CON SIR09P01 (CRS code 8934).','geographic 3D','EPSG','6423','EPSG','1235',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6046','geodetic_crs','EPSG','8932','EPSG','4530','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8933','SIRGAS-CON SIR09P01','Replaces SIRGAS-CON DGF08P01 (CRS code 8931). Replaced by SIRGAS-CON SIR10P01 (CRS code 8935).','geocentric','EPSG','6500','EPSG','1236',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6047','geodetic_crs','EPSG','8933','EPSG','4530','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8934','SIRGAS-CON SIR09P01','Replaces SIRGAS-CON DGF08P01 (CRS code 8932). Replaced by SIRGAS-CON SIR10P01 (CRS code 8936).','geographic 3D','EPSG','6423','EPSG','1236',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6048','geodetic_crs','EPSG','8934','EPSG','4530','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8935','SIRGAS-CON SIR10P01','Replaces SIRGAS-CON SIR09P01 (CRS code 8933). Replaced by SIRGAS-CON SIR11P01 (CRS code 8937).','geocentric','EPSG','6500','EPSG','1237',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6049','geodetic_crs','EPSG','8935','EPSG','4530','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8936','SIRGAS-CON SIR10P01','Replaces SIRGAS-CON SIR09P01 (CRS code 8934). Replaced by SIRGAS-CON SIR11P01 (CRS code 8938).','geographic 3D','EPSG','6423','EPSG','1237',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6050','geodetic_crs','EPSG','8936','EPSG','4530','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8937','SIRGAS-CON SIR11P01','Replaces SIRGAS-CON SIR10P01 (CRS code 8935). Replaced by SIRGAS-CON SIR13P01 (CRS code 8939).','geocentric','EPSG','6500','EPSG','1238',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6051','geodetic_crs','EPSG','8937','EPSG','4530','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8938','SIRGAS-CON SIR11P01','Replaces SIRGAS-CON SIR10P01 (CRS code 8936). Replaced by SIRGAS-CON SIR13P01 (CRS code 8940).','geographic 3D','EPSG','6423','EPSG','1238',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6052','geodetic_crs','EPSG','8938','EPSG','4530','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8939','SIRGAS-CON SIR13P01','Replaces SIRGAS-CON SIR11P01 (CRS code 8937). Replaced by SIRGAS-CON SIR14P01 (CRS code 8941).','geocentric','EPSG','6500','EPSG','1239',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6053','geodetic_crs','EPSG','8939','EPSG','4530','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8940','SIRGAS-CON SIR13P01','Replaces SIRGAS-CON SIR11P01 (CRS code 8938). Replaced by SIRGAS-CON SIR14P01 (CRS code 8942).','geographic 3D','EPSG','6423','EPSG','1239',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6054','geodetic_crs','EPSG','8940','EPSG','4530','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8941','SIRGAS-CON SIR14P01','Replaces SIRGAS-CON SIR13P01 (CRS code 8939). Replaced by SIRGAS-CON SIR15P01 (CRS code 8943).','geocentric','EPSG','6500','EPSG','1240',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6055','geodetic_crs','EPSG','8941','EPSG','4530','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8942','SIRGAS-CON SIR14P01','Replaces SIRGAS-CON SIR13P01 (CRS code 8940). Replaced by SIRGAS-CON SIR15P01 (CRS code 8944).','geographic 3D','EPSG','6423','EPSG','1240',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6056','geodetic_crs','EPSG','8942','EPSG','4530','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8943','SIRGAS-CON SIR15P01','Replaces SIRGAS-CON SIR14P01 (CRS code 8941). Replaced by SIRGAS-CON SIR17P01 (CRS code 8945).','geocentric','EPSG','6500','EPSG','1241',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6057','geodetic_crs','EPSG','8943','EPSG','4530','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8944','SIRGAS-CON SIR15P01','Replaces SIRGAS-CON SIR14P01 (CRS code 8942). Replaced by SIRGAS-CON SIR17P01 (CRS code 8946).','geographic 3D','EPSG','6423','EPSG','1241',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6058','geodetic_crs','EPSG','8944','EPSG','4530','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8945','SIRGAS-CON SIR17P01','Replaces SIRGAS-CON SIR15P01 (CRS code 8943).','geocentric','EPSG','6500','EPSG','1242',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6059','geodetic_crs','EPSG','8945','EPSG','4530','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8946','SIRGAS-CON SIR17P01','Replaces SIRGAS-CON SIR15P01 (CRS code 8944).','geographic 3D','EPSG','6423','EPSG','1242',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6060','geodetic_crs','EPSG','8946','EPSG','4530','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8947','SIRGAS-Chile 2010','Densification of SIRGAS-CON within Chile at epoch 2010.00. Replaces SIRGAS-Chile 2002 (CRS code 5358), replaced by SIRGAS-Chile 2013 (CRS code 9146) due to significant tectonic deformation.','geocentric','EPSG','6500','EPSG','1243',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6061','geodetic_crs','EPSG','8947','EPSG','1066','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8948','SIRGAS-Chile 2010','Densification of SIRGAS-CON within Chile at epoch 2010.00. Replaces SIRGAS-Chile 2002 (CRS code 5359), replaced by SIRGAS-Chile 2013 (CRS code 9147) due to significant tectonic deformation.','geographic 3D','EPSG','6423','EPSG','1243',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6062','geodetic_crs','EPSG','8948','EPSG','1066','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8949','SIRGAS-Chile 2010','Densification within Chile of SIRGAS-CON at epoch 2010.00. Replaces SIRGAS-Chile 2002 (CRS code 5360) due to significant tectonic deformation. Replaced by SIRGAS-Chile 2013 (CRS code 9148)','geographic 2D','EPSG','6422','EPSG','1243',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6063','geodetic_crs','EPSG','8949','EPSG','1066','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','8972','SIRGAS-CON DGF00P01','Replaced by SIRGAS-CON DGF01P01 (CRS code 8973).','geographic 2D','EPSG','6422','EPSG','1227',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6066','geodetic_crs','EPSG','8972','EPSG','4530','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8973','SIRGAS-CON DGF01P01','Replaces SIRGAS-CON DGF00P01 (CRS code 8972). Replaced by SIRGAS-CON DGF01P02 (CRS code 8974).','geographic 2D','EPSG','6422','EPSG','1228',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6067','geodetic_crs','EPSG','8973','EPSG','4530','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8974','SIRGAS-CON DGF01P02','Replaces SIRGAS-CON DGF01P01 (CRS code 8973). Replaced by SIRGAS-CON DGF02P01 (CRS code 8975).','geographic 2D','EPSG','6422','EPSG','1229',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6068','geodetic_crs','EPSG','8974','EPSG','4530','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8975','SIRGAS-CON DGF02P01','Replaces SIRGAS-CON DGF01P02 (CRS code 8974). Replaced by SIRGAS-CON DGF04P01 (CRS code 8976).','geographic 2D','EPSG','6422','EPSG','1230',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6069','geodetic_crs','EPSG','8975','EPSG','4530','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8976','SIRGAS-CON DGF04P01','Replaces SIRGAS-CON DGF02P01 (CRS code 8975). Replaced by SIRGAS-CON DGF05P01 (CRS code 8977).','geographic 2D','EPSG','6422','EPSG','1231',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6070','geodetic_crs','EPSG','8976','EPSG','4530','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8977','SIRGAS-CON DGF05P01','Replaces SIRGAS-CON DGF04P01 (CRS code 8976). Replaced by SIRGAS-CON DGF06P01 (CRS code 8978).','geographic 2D','EPSG','6422','EPSG','1232',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6071','geodetic_crs','EPSG','8977','EPSG','4530','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8978','SIRGAS-CON DGF06P01','Replaces SIRGAS-CON DGF05P01 (CRS code 8977). Replaced by SIRGAS-CON DGF07P01 (CRS code 8979).','geographic 2D','EPSG','6422','EPSG','1233',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6072','geodetic_crs','EPSG','8978','EPSG','4530','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8979','SIRGAS-CON DGF07P01','Replaces SIRGAS-CON DGF06P01 (CRS code 8978). Replaced by SIRGAS-CON DGF08P01 (CRS code 8980).','geographic 2D','EPSG','6422','EPSG','1234',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6073','geodetic_crs','EPSG','8979','EPSG','4530','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8980','SIRGAS-CON DGF08P01','Replaces SIRGAS-CON DGF07P01 (CRS code 8979). Replaced by SIRGAS-CON SIR09P01 (CRS code 8981).','geographic 2D','EPSG','6422','EPSG','1235',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6074','geodetic_crs','EPSG','8980','EPSG','4530','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8981','SIRGAS-CON SIR09P01','Replaces SIRGAS-CON DGF08P01 (CRS code 8980). Replaced by SIRGAS-CON SIR10P01 (CRS code 8982).','geographic 2D','EPSG','6422','EPSG','1236',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6075','geodetic_crs','EPSG','8981','EPSG','4530','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8982','SIRGAS-CON SIR10P01','Replaces SIRGAS-CON SIR09P01 (CRS code 8981). Replaced by SIRGAS-CON SIR11P01 (CRS code 8983).','geographic 2D','EPSG','6422','EPSG','1237',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6076','geodetic_crs','EPSG','8982','EPSG','4530','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8983','SIRGAS-CON SIR11P01','Replaces SIRGAS-CON SIR10P01 (CRS code 8982). Replaced by SIRGAS-CON SIR13P01 (CRS code 8984).','geographic 2D','EPSG','6422','EPSG','1238',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6077','geodetic_crs','EPSG','8983','EPSG','4530','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8984','SIRGAS-CON SIR13P01','Replaces SIRGAS-CON SIR11P01 (CRS code 8983). Replaced by SIRGAS-CON SIR14P01 (CRS code 8985).','geographic 2D','EPSG','6422','EPSG','1239',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6078','geodetic_crs','EPSG','8984','EPSG','4530','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8985','SIRGAS-CON SIR14P01','Replaces SIRGAS-CON SIR13P01 (CRS code 8984). Replaced by SIRGAS-CON SIR15P01 (CRS code 8986).','geographic 2D','EPSG','6422','EPSG','1240',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6079','geodetic_crs','EPSG','8985','EPSG','4530','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8986','SIRGAS-CON SIR15P01','Replaces SIRGAS-CON SIR14P01 (CRS code 8985). Replaced by SIRGAS-CON SIR17P01 (CRS code 8987).','geographic 2D','EPSG','6422','EPSG','1241',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6080','geodetic_crs','EPSG','8986','EPSG','4530','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8987','SIRGAS-CON SIR17P01','Replaces SIRGAS-CON SIR15P01 (CRS code 8986).','geographic 2D','EPSG','6422','EPSG','1242',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6081','geodetic_crs','EPSG','8987','EPSG','4530','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8988','ITRF88','Replaced by ITRF89 (code 8989).','geographic 2D','EPSG','6422','EPSG','6647',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6082','geodetic_crs','EPSG','8988','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8989','ITRF89','Replaces ITRF88 (code 8988). Replaced by ITRF90 (code 8990).','geographic 2D','EPSG','6422','EPSG','6648',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6083','geodetic_crs','EPSG','8989','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8990','ITRF90','Replaces ITRF89 (code 8989). Replaced by ITRF91 (code 8991).','geographic 2D','EPSG','6422','EPSG','6649',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6084','geodetic_crs','EPSG','8990','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8991','ITRF91','Replaces ITRF90 (code 8990). Replaced by ITRF92 (code 8992).','geographic 2D','EPSG','6422','EPSG','6650',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6085','geodetic_crs','EPSG','8991','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8992','ITRF92','Replaces ITRF91 (code 8991). Replaced by ITRF93 (code 8993).','geographic 2D','EPSG','6422','EPSG','6651',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6086','geodetic_crs','EPSG','8992','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8993','ITRF93','Replaces ITRF92 (code 8992). Replaced by ITRF94 (code 8994).','geographic 2D','EPSG','6422','EPSG','6652',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6087','geodetic_crs','EPSG','8993','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8994','ITRF94','Replaces ITRF93 (code 8993). Replaced by ITRF96 (code 8995).','geographic 2D','EPSG','6422','EPSG','6653',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6088','geodetic_crs','EPSG','8994','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8995','ITRF96','Replaces ITRF94 (code 8994). Replaced by ITRF97 (code 8996).','geographic 2D','EPSG','6422','EPSG','6654',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6089','geodetic_crs','EPSG','8995','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8996','ITRF97','Replaces ITRF96 (code 8995). Replaced by ITRF2000 (code 8997).','geographic 2D','EPSG','6422','EPSG','6655',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6090','geodetic_crs','EPSG','8996','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8997','ITRF2000','Replaces ITRF97 (code 8996). Replaced by ITRF2005 (code 8998).','geographic 2D','EPSG','6422','EPSG','6656',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6091','geodetic_crs','EPSG','8997','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8998','ITRF2005','Replaces ITRF2000 (code 8997). Replaced by ITRF2008 (code 8999).','geographic 2D','EPSG','6422','EPSG','6896',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6092','geodetic_crs','EPSG','8998','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','8999','ITRF2008','Replaces ITRF2005 (code 8998). Replaced by ITRF2014 (code 9000).','geographic 2D','EPSG','6422','EPSG','1061',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6093','geodetic_crs','EPSG','8999','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','9000','ITRF2014','Replaces ITRF2008 (code 8999). Replaced by ITRF2020 (CRS code 9990).','geographic 2D','EPSG','6422','EPSG','1165',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6094','geodetic_crs','EPSG','9000','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','9001','IGS97','Adopted by the International GNSS Service (IGS) from 2000-06-04 through 2001-12-01. Replaced by IGS00 (CRS code 9004). For all practical purposes IGS97 is equivalent to ITRF97.','geocentric','EPSG','6500','EPSG','1244',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6095','geodetic_crs','EPSG','9001','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','9002','IGS97','Adopted by the International GNSS Service (IGS) from 2000-06-04 through 2001-12-01. Replaced by IGS00 (CRS code 9005). For all practical purposes IGS97 is equivalent to ITRF97.','geographic 3D','EPSG','6423','EPSG','1244',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6096','geodetic_crs','EPSG','9002','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','9003','IGS97','Adopted by the International GNSS Service (IGS) from 2000-06-04 through 2001-12-01. Replaced by IGS00 (CRS code 9006). For all practical purposes IGS97 is equivalent to ITRF97.','geographic 2D','EPSG','6422','EPSG','1244',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6097','geodetic_crs','EPSG','9003','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','9004','IGS00','Adopted by the International GNSS Service (IGS) from 2001-12-02 through 2004-01-10. Replaces IGS97, replaced by IGb00 (CRS codes 9001 and 9007). For all practical purposes IGS00 is equivalent to ITRF2000.','geocentric','EPSG','6500','EPSG','1245',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6098','geodetic_crs','EPSG','9004','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','9005','IGS00','Adopted by the International GNSS Service (IGS) from 2001-12-02 through 2004-01-10. Replaces IGS97, replaced by IGb00 (CRS codes 9002 and 9008). For all practical purposes IGS00 is equivalent to ITRF2000.','geographic 3D','EPSG','6423','EPSG','1245',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6099','geodetic_crs','EPSG','9005','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','9006','IGS00','Adopted by the International GNSS Service (IGS) from 2001-12-02 through 2004-01-10. Replaces IGS97, replaced by IGb00 (CRS codes 9003 and 9009). For all practical purposes IGS00 is equivalent to ITRF2000.','geographic 2D','EPSG','6422','EPSG','1245',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6100','geodetic_crs','EPSG','9006','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','9007','IGb00','Adopted by the International GNSS Service (IGS) from 2004-01-11 through 2006-11-04. Replaces IGS00, replaced by IGS05 (CRS codes 9004 and 9010). For all practical purposes IGb00 is equivalent to ITRF2000.','geocentric','EPSG','6500','EPSG','1246',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6101','geodetic_crs','EPSG','9007','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','9008','IGb00','Adopted by the International GNSS Service (IGS) from 2004-01-11 through 2006-11-04. Replaces IGS00, replaced by IGS05 (CRS codes 9005 and 9011). For all practical purposes IGb00 is equivalent to ITRF2000.','geographic 3D','EPSG','6423','EPSG','1246',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6102','geodetic_crs','EPSG','9008','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','9009','IGb00','Adopted by the International GNSS Service (IGS) from 2004-01-11 through 2006-11-04. Replaces IGS00, replaced by IGS05 (CRS codes 9006 and 9012). For all practical purposes IGb00 is equivalent to ITRF2000.','geographic 2D','EPSG','6422','EPSG','1246',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6103','geodetic_crs','EPSG','9009','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','9010','IGS05','Adopted by the International GNSS Service (IGS) from 2006-11-05 through 2011-04-16. Replaces IGb00, replaced by IGS08 (CRS codes 9007 and 6834). For all practical purposes IGS05 is equivalent to ITRF2005.','geocentric','EPSG','6500','EPSG','1247',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6104','geodetic_crs','EPSG','9010','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','9011','IGS05','Adopted by the International GNSS Service (IGS) from 2006-11-05 through 2011-04-16. Replaces IGb00, replaced by IGS08 (CRS codes 9008 and 9013). For all practical purposes IGS05 is equivalent to ITRF2005.','geographic 3D','EPSG','6423','EPSG','1247',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6105','geodetic_crs','EPSG','9011','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','9012','IGS05','Adopted by the International GNSS Service (IGS) from 2006-11-05 through 2011-04-16. Replaces IGb00, replaced by IGS08 (CRS codes 9009 and 9014). For all practical purposes IGS05 is equivalent to ITRF2005.','geographic 2D','EPSG','6422','EPSG','1247',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6106','geodetic_crs','EPSG','9012','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','9013','IGS08','Used for products from International GNSS Service (IGS) analysis centres from 2011-04-17 through 2012-10-06. Replaces IGS05 (code 9011). Replaced by IGb08 (code 9016). For most practical purposes IGS08 is equivalent to ITRF2008.','geographic 3D','EPSG','6423','EPSG','1141',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6107','geodetic_crs','EPSG','9013','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','9014','IGS08','Used for products from International GNSS Service (IGS) analysis centres from 2011-04-17 through 2012-10-06. Replaces IGS05 (code 9012). Replaced by IGb08 (code 9017). For most practical purposes IGS08 is equivalent to ITRF2008.','geographic 2D','EPSG','6422','EPSG','1141',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6108','geodetic_crs','EPSG','9014','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','9015','IGb08','Adopted by the International GNSS Service (IGS) from 2012-10-07 through 2017-01-28. Replaces IGS08, replaced by IGS14 (CRS codes 6934 and 8227). For all practical purposes IGb08 is equivalent to ITRF2008.','geocentric','EPSG','6500','EPSG','1248',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6109','geodetic_crs','EPSG','9015','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','9016','IGb08','Adopted by the International GNSS Service (IGS) from 2012-10-07 through 2017-01-28. Replaces IGS08, replaced by IGS14 (CRS codes 9013 and 9018). For all practical purposes IGb08 is equivalent to ITRF2008.','geographic 3D','EPSG','6423','EPSG','1248',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6110','geodetic_crs','EPSG','9016','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','9017','IGb08','Adopted by the International GNSS Service (IGS) from 2012-10-07 through 2017-01-28. Replaces IGS08, replaced by IGS14 (CRS codes 9014 and 9019). For all practical purposes IGb08 is equivalent to ITRF2008.','geographic 2D','EPSG','6422','EPSG','1248',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6111','geodetic_crs','EPSG','9017','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','9018','IGS14','Used for products from the International GNSS Service (IGS) from 2017-01-29 to 2020-05-16. Replaces IGb08 (code 9016), replaced by IGb14 (code 9379). For most practical purposes IGS14 is equivalent to ITRF2014.','geographic 3D','EPSG','6423','EPSG','1191',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6112','geodetic_crs','EPSG','9018','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','9019','IGS14','Used for products from the International GNSS Service (IGS) from 2017-01-29 to 2020-05-16. Replaces IGb08 (code 9017), replaced by IGb14 (code 9380). For most practical purposes IGS14 is equivalent to ITRF2014.','geographic 2D','EPSG','6422','EPSG','1191',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6113','geodetic_crs','EPSG','9019','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','9053','WGS 84 (G730)','Replaces WGS 84 (Transit) (CRS code 8888) from 1994-06-29. Replaced by WGS84 (G873) (CRS code 9054) from 1997-01-29.','geographic 2D','EPSG','6422','EPSG','1152',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6116','geodetic_crs','EPSG','9053','EPSG','1262','EPSG','1176');
INSERT INTO "geodetic_crs" VALUES('EPSG','9054','WGS 84 (G873)','Replaces WGS 84 (G730) (CRS code 9053) from 1997-01-29. Replaced by WGS 84 (G1150) (CRS code 9055) from 2002-01-20.','geographic 2D','EPSG','6422','EPSG','1153',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6117','geodetic_crs','EPSG','9054','EPSG','1262','EPSG','1176');
INSERT INTO "geodetic_crs" VALUES('EPSG','9055','WGS 84 (G1150)','Replaces WGS 84 (G873) (CRS code 9054) from 2002-01-20. Replaced by WGS 84 (G1674) (CRS code 9056) from 2012-02-08.','geographic 2D','EPSG','6422','EPSG','1154',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6118','geodetic_crs','EPSG','9055','EPSG','1262','EPSG','1176');
INSERT INTO "geodetic_crs" VALUES('EPSG','9056','WGS 84 (G1674)','Replaces WGS 84 (G1150) (CRS code 9055) from 2012-02-08. Replaced by WGS 84 (G1762) (CRS code 9057) from 2013-10-16.','geographic 2D','EPSG','6422','EPSG','1155',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6119','geodetic_crs','EPSG','9056','EPSG','1262','EPSG','1176');
INSERT INTO "geodetic_crs" VALUES('EPSG','9057','WGS 84 (G1762)','Replaces WGS 84 (G1674) (CRS code 9056) from 2013-10-16. Redesignated WGS 84 (G1762'') in 2015 after changes to 7 NGA tracking station locations and antennas. Replaced by WGS 84 (G2139) (CRS code 9755) from 2021-01-03.','geographic 2D','EPSG','6422','EPSG','1156',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6120','geodetic_crs','EPSG','9057','EPSG','1262','EPSG','1176');
INSERT INTO "geodetic_crs" VALUES('EPSG','9059','ETRF89','Replaced by ETRF90 (code 9060).','geographic 2D','EPSG','6422','EPSG','1178',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6121','geodetic_crs','EPSG','9059','EPSG','1298','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','9060','ETRF90','Replaces ETRF89 (code 9059). Replaced by ETRF91 (code 9061).','geographic 2D','EPSG','6422','EPSG','1179',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6122','geodetic_crs','EPSG','9060','EPSG','1298','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','9061','ETRF91','Replaces ETRF90 (code 9060). Replaced by ETRF92 (code 9062).','geographic 2D','EPSG','6422','EPSG','1180',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6123','geodetic_crs','EPSG','9061','EPSG','1298','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','9062','ETRF92','Replaces ETRF91 (code 9061). Replaced by ETRF93 (code 9063).','geographic 2D','EPSG','6422','EPSG','1181',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6124','geodetic_crs','EPSG','9062','EPSG','1298','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','9063','ETRF93','Replaces ETRF92 (code 9062). Replaced by ETRF94 (code 9064).','geographic 2D','EPSG','6422','EPSG','1182',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6125','geodetic_crs','EPSG','9063','EPSG','1298','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','9064','ETRF94','Replaces ETRF93 (code 9063). Replaced by ETRF96 (code 9065).','geographic 2D','EPSG','6422','EPSG','1183',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6126','geodetic_crs','EPSG','9064','EPSG','1298','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','9065','ETRF96','Replaces ETRF94 (code 9064). Replaced by ETRF97 (code 9066).','geographic 2D','EPSG','6422','EPSG','1184',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6127','geodetic_crs','EPSG','9065','EPSG','1298','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','9066','ETRF97','Replaces ETRF96 (code 9065). Replaced by ETRF2000 (code 9067).','geographic 2D','EPSG','6422','EPSG','1185',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6128','geodetic_crs','EPSG','9066','EPSG','1298','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','9067','ETRF2000','Replaces ETRF97 (code 9066). On the publication of ETRF2005 the EUREF Technical Working Group recommended ETRF2000 be the realization of ETRS89. ETRF2014 and ETRF2020 (codes 9069 and 10571) are technically superior to all earlier realizations of ETRS89.','geographic 2D','EPSG','6422','EPSG','1186',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6129','geodetic_crs','EPSG','9067','EPSG','1298','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','9068','ETRF2005','On publication in 2007 of this CRS, the EUREF Technical Working Group recommended that ETRF2000 (EPSG code 9067) remained as the preferred realization of ETRS89. Replaced by ETRF2014 (code 9069).','geographic 2D','EPSG','6422','EPSG','1204',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6130','geodetic_crs','EPSG','9068','EPSG','1298','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','9069','ETRF2014','Replaces ETRF2005 (code 9068). ETRF2014 is technically superior to ETRF2000 but ETRF2000 and other previous realizations may be preferred for backward compatibility reasons. Differences between ETRF2014 and ETRF2000 can reach 7cm.','geographic 2D','EPSG','6422','EPSG','1206',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6131','geodetic_crs','EPSG','9069','EPSG','1298','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','9070','NAD83(MARP00)','Replaces NAD83(HARN) (GGN93) and NAD83(FBN) in Guam. Replaced by NAD83(MA11).','geocentric','EPSG','6500','EPSG','1221',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6132','geodetic_crs','EPSG','9070','EPSG','4167','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','9071','NAD83(MARP00)','Replaces NAD83(HARN) (GGN93) and NAD83(FBN) in Guam. Replaced by NAD83(MA11).','geographic 3D','EPSG','6423','EPSG','1221',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6133','geodetic_crs','EPSG','9071','EPSG','4167','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','9072','NAD83(MARP00)','Replaces NAD83(HARN) (GGN93) and NAD83(FBN) in Guam. Replaced by NAD83(MA11).','geographic 2D','EPSG','6422','EPSG','1221',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6134','geodetic_crs','EPSG','9072','EPSG','4167','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','9073','NAD83(PACP00)','Replaces NAD83(HARN) and NAD83(FBN) in Hawaii and American Samoa. Replaced by NAD83(PA11).','geocentric','EPSG','6500','EPSG','1249',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6135','geodetic_crs','EPSG','9073','EPSG','4162','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','9074','NAD83(PACP00)','Replaces NAD83(HARN) and NAD83(FBN) in Hawaii and American Samoa. Replaced by NAD83(PA11).','geographic 3D','EPSG','6423','EPSG','1249',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6136','geodetic_crs','EPSG','9074','EPSG','4162','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','9075','NAD83(PACP00)','Note: this CRS includes longitudes which are POSITIVE EAST. Replaces NAD83(HARN) and NAD83(FBN) in Hawaii and American Samoa. Replaced by NAD83(PA11).','geographic 2D','EPSG','6422','EPSG','1249',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6137','geodetic_crs','EPSG','9075','EPSG','4162','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','9138','KOSOVAREF01','','geocentric','EPSG','6500','EPSG','1251',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6139','geodetic_crs','EPSG','9138','EPSG','4542','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','9139','KOSOVAREF01','','geographic 3D','EPSG','6423','EPSG','1251',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6140','geodetic_crs','EPSG','9139','EPSG','4542','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','9140','KOSOVAREF01','In Kosovo replaces MGI 1901 (CRS code 3906).','geographic 2D','EPSG','6422','EPSG','1251',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6141','geodetic_crs','EPSG','9140','EPSG','4542','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','9146','SIRGAS-Chile 2013','Densification of SIRGAS-CON within Chile at epoch 2013.00. Replaces SIRGAS-Chile 2010 (CRS code 8947), replaced by SIRGAS-Chile 2016 (CRS code 9151) due to significant tectonic deformation.','geocentric','EPSG','6500','EPSG','1252',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6143','geodetic_crs','EPSG','9146','EPSG','1066','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','9147','SIRGAS-Chile 2013','Densification of SIRGAS-CON within Chile at epoch 2013.00. Replaces SIRGAS-Chile 2010 (CRS code 8948), replaced by SIRGAS-Chile 2016 (CRS code 9152) due to significant tectonic deformation.','geographic 3D','EPSG','6423','EPSG','1252',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6144','geodetic_crs','EPSG','9147','EPSG','1066','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','9148','SIRGAS-Chile 2013','Densification within Chile of SIRGAS-CON at epoch 2013.00. Replaces SIRGAS-Chile 2010 (CRS code 8949) due to significant tectonic deformation. Replaced by SIRGAS-Chile 2016 (CRS code 9153).','geographic 2D','EPSG','6422','EPSG','1252',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6145','geodetic_crs','EPSG','9148','EPSG','1066','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','9151','SIRGAS-Chile 2016','Densification of SIRGAS-CON within Chile at epoch 2016.00. Replaces SIRGAS-Chile 2013 (CRS code 9146), replaced by SIRGAS-Chile 2021 (CRS code 20039) due to significant tectonic deformation.','geocentric','EPSG','6500','EPSG','1253',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6148','geodetic_crs','EPSG','9151','EPSG','1066','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','9152','SIRGAS-Chile 2016','Densification of SIRGAS-CON within Chile at epoch 2016.00. Replaces SIRGAS-Chile 2013 (CRS code 9147), replaced by SIRGAS-Chile 2021 (CRS code 20040) due to significant tectonic deformation.','geographic 3D','EPSG','6423','EPSG','1253',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6149','geodetic_crs','EPSG','9152','EPSG','1066','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','9153','SIRGAS-Chile 2016','Densification within Chile of SIRGAS-CON at epoch 2016.00. Replaces SIRGAS-Chile 2013 (CRS code 9148), replaced by SIRGAS-Chile 2021 (CRS code 20041) due to significant tectonic deformation.','geographic 2D','EPSG','6422','EPSG','1253',NULL,0);
INSERT INTO "usage" VALUES('EPSG','6150','geodetic_crs','EPSG','9153','EPSG','1066','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','9182','SIRGAS-Chile','The datum ensemble underpinning this CRS describes all or any of the realizations without distinction. For a specific realization see CRSs 5358, 8947, 9146 or 9151.','geocentric','EPSG','6500','EPSG','1254',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6157','geodetic_crs','EPSG','9182','EPSG','1066','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','9183','SIRGAS-Chile','The datum ensemble underpinning this CRS describes all or any of the realizations without distinction. For a specific realization see CRSs 5359, 8948, 9147 or 9152.','geographic 3D','EPSG','6423','EPSG','1254',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6158','geodetic_crs','EPSG','9183','EPSG','1066','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','9184','SIRGAS-Chile','The datum ensemble underpinning this CRS describes all or any of the realizations without distinction. For a specific realization see CRSs 5360, 8949, 9148 or 9153.','geographic 2D','EPSG','6422','EPSG','1254',NULL,1);
INSERT INTO "usage" VALUES('EPSG','6159','geodetic_crs','EPSG','9184','EPSG','1066','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','9248','Tapi Aike','Replaced by Campo Inchauspe (geogCRS code 4221) for topographic mapping, use for oil exploration and production continues.','geographic 2D','EPSG','6422','EPSG','1257',NULL,0);
INSERT INTO "usage" VALUES('EPSG','13901','geodetic_crs','EPSG','9248','EPSG','4569','EPSG','1136');
INSERT INTO "geodetic_crs" VALUES('EPSG','9251','MMN','','geographic 2D','EPSG','6422','EPSG','1258',NULL,0);
INSERT INTO "usage" VALUES('EPSG','13904','geodetic_crs','EPSG','9251','EPSG','2357','EPSG','1136');
INSERT INTO "geodetic_crs" VALUES('EPSG','9253','MMS','','geographic 2D','EPSG','6422','EPSG','1259',NULL,0);
INSERT INTO "usage" VALUES('EPSG','13906','geodetic_crs','EPSG','9253','EPSG','2357','EPSG','1136');
INSERT INTO "geodetic_crs" VALUES('EPSG','9266','MGI','','geocentric','EPSG','6500','EPSG','6312',NULL,0);
INSERT INTO "usage" VALUES('EPSG','13910','geodetic_crs','EPSG','9266','EPSG','1037','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','9267','MGI','Created retrospectively to support geoid model based on Bessel ellipsoid.','geographic 3D','EPSG','6423','EPSG','6312',NULL,0);
INSERT INTO "usage" VALUES('EPSG','13911','geodetic_crs','EPSG','9267','EPSG','1037','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','9292','ONGD17','Replaces ONGD14 (CRS code 7371) from March 2019.','geocentric','EPSG','6500','EPSG','1263',NULL,0);
INSERT INTO "usage" VALUES('EPSG','13963','geodetic_crs','EPSG','9292','EPSG','1183','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','9293','ONGD17','Replaces ONGD14 (CRS code 7372) from March 2019.','geographic 3D','EPSG','6423','EPSG','1263',NULL,0);
INSERT INTO "usage" VALUES('EPSG','13964','geodetic_crs','EPSG','9293','EPSG','1183','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','9294','ONGD17','Replaces ONGD14 (CRS code 7373) from March 2019.','geographic 2D','EPSG','6422','EPSG','1263',NULL,0);
INSERT INTO "usage" VALUES('EPSG','13965','geodetic_crs','EPSG','9294','EPSG','1183','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','9299','HS2-IRF','Intermediate CRS created to assist the emulation of the ETRS89 / HS2P1+14 SnakeGrid projected CRS through transformation HS2-IRF to ETRS89 (1) (code 9302) used in conjunction with the HS2-TM map projection (code 9301).','geographic 2D','EPSG','6422','EPSG','1264',NULL,0);
INSERT INTO "usage" VALUES('EPSG','14047','geodetic_crs','EPSG','9299','EPSG','4582','EPSG','1260');
INSERT INTO "geodetic_crs" VALUES('EPSG','9307','ATRF2014','','geocentric','EPSG','6500','EPSG','1291',NULL,0);
INSERT INTO "usage" VALUES('EPSG','14135','geodetic_crs','EPSG','9307','EPSG','4177','EPSG','1267');
INSERT INTO "geodetic_crs" VALUES('EPSG','9308','ATRF2014','','geographic 3D','EPSG','6423','EPSG','1291',NULL,0);
INSERT INTO "usage" VALUES('EPSG','14136','geodetic_crs','EPSG','9308','EPSG','4177','EPSG','1267');
INSERT INTO "geodetic_crs" VALUES('EPSG','9309','ATRF2014','','geographic 2D','EPSG','6422','EPSG','1291',NULL,0);
INSERT INTO "usage" VALUES('EPSG','14137','geodetic_crs','EPSG','9309','EPSG','4177','EPSG','1267');
INSERT INTO "geodetic_crs" VALUES('EPSG','9331','KSA-GRF17','','geocentric','EPSG','6500','EPSG','1268',NULL,0);
INSERT INTO "usage" VALUES('EPSG','13919','geodetic_crs','EPSG','9331','EPSG','1206','EPSG','1181');
INSERT INTO "geodetic_crs" VALUES('EPSG','9332','KSA-GRF17','','geographic 3D','EPSG','6423','EPSG','1268',NULL,0);
INSERT INTO "usage" VALUES('EPSG','13920','geodetic_crs','EPSG','9332','EPSG','1206','EPSG','1181');
INSERT INTO "geodetic_crs" VALUES('EPSG','9333','KSA-GRF17','','geographic 2D','EPSG','6422','EPSG','1268',NULL,0);
INSERT INTO "usage" VALUES('EPSG','13921','geodetic_crs','EPSG','9333','EPSG','1206','EPSG','1181');
INSERT INTO "geodetic_crs" VALUES('EPSG','9364','TPEN11-IRF','Intermediate CRS created in 2020 to assist the emulation of the ETRS89 / TPEN11 SnakeGrid projected CRS through transformation ETRS89 to TPEN11-IRF (1) (code 9365) used in conjunction with the TPEN11-TM map projection (code 9366).','geographic 2D','EPSG','6422','EPSG','1266',NULL,0);
INSERT INTO "usage" VALUES('EPSG','13974','geodetic_crs','EPSG','9364','EPSG','4583','EPSG','1141');
INSERT INTO "geodetic_crs" VALUES('EPSG','9372','MML07-IRF','Intermediate CRS created in 2020 to assist the emulation of the ETRS89 / MML07 SnakeGrid projected CRS (code 9373) through transformation ETRS89 to MML07-IRF (1) (code 9369) used in conjunction with the MML07-TM map projection (code 9370).','geographic 2D','EPSG','6422','EPSG','1271',NULL,0);
INSERT INTO "usage" VALUES('EPSG','13995','geodetic_crs','EPSG','9372','EPSG','4588','EPSG','1141');
INSERT INTO "geodetic_crs" VALUES('EPSG','9378','IGb14','Used for products from the International GNSS Service (IGS) from 2020-05-17. Replaces IGS14 (code 8227), replaced by IGS20 (code 10176). For most practical purposes IGb14 is equivalent to ITRF2014.','geocentric','EPSG','6500','EPSG','1272',NULL,0);
INSERT INTO "usage" VALUES('EPSG','13999','geodetic_crs','EPSG','9378','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','9379','IGb14','Used for products from the International GNSS Service (IGS) from 2020-05-17. Replaces IGS14 (code 9018), replaced by IGS20 (code 10177). For most practical purposes IGb14 is equivalent to ITRF2014.','geographic 3D','EPSG','6423','EPSG','1272',NULL,0);
INSERT INTO "usage" VALUES('EPSG','14000','geodetic_crs','EPSG','9379','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','9380','IGb14','Used for products from the International GNSS Service (IGS) from 2020-05-17. Replaces IGS14 (code 9019), replaced by IGS20 (code 10178). For most practical purposes IGb14 is equivalent to ITRF2014.','geographic 2D','EPSG','6422','EPSG','1272',NULL,0);
INSERT INTO "usage" VALUES('EPSG','14213','geodetic_crs','EPSG','9380','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','9384','AbInvA96_2020-IRF','Intermediate CRS created in 2020 to assist the emulation of the ETRS89 / AbInvA96_2020 SnakeGrid projected CRS through transformation ETRS89 to AbInvA96_2020-IRF (1) (code 9386) used in conjunction with the AbInvA96_2020-TM map projection (code 9385).','geographic 2D','EPSG','6422','EPSG','1273',NULL,0);
INSERT INTO "usage" VALUES('EPSG','14028','geodetic_crs','EPSG','9384','EPSG','4589','EPSG','1196');
INSERT INTO "geodetic_crs" VALUES('EPSG','9403','PN68','On western islands (El Hierro, La Gomera, La Palma and Tenerife) replaced by PN84 (CRS code 4728) and later by REGCAN95 (CRS code 4081). On eastern islands (Fuerteventura, Gran Canaria and Lanzarote) replaced by REGCAN95 (CRS code 4081).','geographic 2D','EPSG','6422','EPSG','1286',NULL,0);
INSERT INTO "usage" VALUES('EPSG','14042','geodetic_crs','EPSG','9403','EPSG','3873','EPSG','1178');
INSERT INTO "geodetic_crs" VALUES('EPSG','9453','GBK19-IRF','Intermediate CRS created in 2020 to assist the emulation of the ETRS89 / GBK19 SnakeGrid projected CRS through transformation ETRS89 to GBK19-IRF (1) (code 9454) used in conjunction with the GBK19-TM map projection (code 9455).','geographic 2D','EPSG','6422','EPSG','1289',NULL,0);
INSERT INTO "usage" VALUES('EPSG','14128','geodetic_crs','EPSG','9453','EPSG','4607','EPSG','1141');
INSERT INTO "geodetic_crs" VALUES('EPSG','9468','SRGI2013','Supports horizontal component of national horizontal control network (JKHN). Adopted 2013-10-11. Replaces DGN95 and all older systems.','geocentric','EPSG','6500','EPSG','1293',NULL,0);
INSERT INTO "usage" VALUES('EPSG','14150','geodetic_crs','EPSG','9468','EPSG','1122','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','9469','SRGI2013','Supports horizontal component of national horizontal control network (JKHN). Adopted 2013-10-11. Replaces DGN95 and all older systems.','geographic 3D','EPSG','6423','EPSG','1293',NULL,0);
INSERT INTO "usage" VALUES('EPSG','14151','geodetic_crs','EPSG','9469','EPSG','1122','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','9470','SRGI2013','Supports horizontal component of national horizontal control network (JKHN). Adopted 2013-10-11. Replaces DGN95 and all older systems.','geographic 2D','EPSG','6422','EPSG','1293',NULL,0);
INSERT INTO "usage" VALUES('EPSG','14152','geodetic_crs','EPSG','9470','EPSG','1122','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','9474','PZ-90.02','Replaces PZ-90 (CRS code 4740) from 2007-09-20. Replaced by PZ-90.11 (CRS code 9475) from 2014-01-15.','geographic 2D','EPSG','6422','EPSG','1157',NULL,0);
INSERT INTO "usage" VALUES('EPSG','14195','geodetic_crs','EPSG','9474','EPSG','1262','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','9475','PZ-90.11','Replaces PZ-90.02 (CRS code 9474) from 2014-01-15.','geographic 2D','EPSG','6422','EPSG','1158',NULL,0);
INSERT INTO "usage" VALUES('EPSG','14194','geodetic_crs','EPSG','9475','EPSG','1262','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','9545','LTF2004(G)','','geocentric','EPSG','6500','EPSG','1295',NULL,0);
INSERT INTO "usage" VALUES('EPSG','14547','geodetic_crs','EPSG','9545','EPSG','4613','EPSG','1271');
INSERT INTO "geodetic_crs" VALUES('EPSG','9546','LTF2004(G)','','geographic 3D','EPSG','6423','EPSG','1295',NULL,0);
INSERT INTO "usage" VALUES('EPSG','14539','geodetic_crs','EPSG','9546','EPSG','4613','EPSG','1271');
INSERT INTO "geodetic_crs" VALUES('EPSG','9547','LTF2004(G)','','geographic 2D','EPSG','6422','EPSG','1295',NULL,0);
INSERT INTO "usage" VALUES('EPSG','14546','geodetic_crs','EPSG','9547','EPSG','4613','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','9694','REDGEOMIN','','geocentric','EPSG','6500','EPSG','1304',NULL,0);
INSERT INTO "usage" VALUES('EPSG','14939','geodetic_crs','EPSG','9694','EPSG','1066','EPSG','1181');
INSERT INTO "geodetic_crs" VALUES('EPSG','9695','REDGEOMIN','','geographic 3D','EPSG','6423','EPSG','1304',NULL,0);
INSERT INTO "usage" VALUES('EPSG','14947','geodetic_crs','EPSG','9695','EPSG','1066','EPSG','1181');
INSERT INTO "geodetic_crs" VALUES('EPSG','9696','REDGEOMIN','','geographic 2D','EPSG','6422','EPSG','1304',NULL,0);
INSERT INTO "usage" VALUES('EPSG','14949','geodetic_crs','EPSG','9696','EPSG','1066','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','9700','ETRF2000-PL','Adopted from 2012-12-01.','geocentric','EPSG','6500','EPSG','1305',NULL,0);
INSERT INTO "usage" VALUES('EPSG','15062','geodetic_crs','EPSG','9700','EPSG','1192','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','9701','ETRF2000-PL','Adopted from 2012-12-01.','geographic 3D','EPSG','6423','EPSG','1305',NULL,0);
INSERT INTO "usage" VALUES('EPSG','15063','geodetic_crs','EPSG','9701','EPSG','1192','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','9702','ETRF2000-PL','','geographic 2D','EPSG','6422','EPSG','1305',NULL,0);
INSERT INTO "usage" VALUES('EPSG','15131','geodetic_crs','EPSG','9702','EPSG','1192','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','9739','EOS21-IRF','Intermediate CRS created in 2021 to assist the emulation of the ETRS89 / EOS21 SnakeGrid projected CRS through transformation ETRS89 to EOS21-IRF (1) (code 9740) used in conjunction with the EOS21-TM map projection (code 9738).','geographic 2D','EPSG','6422','EPSG','1308',NULL,0);
INSERT INTO "usage" VALUES('EPSG','15341','geodetic_crs','EPSG','9739','EPSG','4620','EPSG','1141');
INSERT INTO "geodetic_crs" VALUES('EPSG','9753','WGS 84 (G2139)','Replaces WGS 84 (G1762) (CRS code 7664) from 2021-01-03. Replaced by WGS 84 (G2296) (CRS code 10604) from 2024-01-07.','geocentric','EPSG','6500','EPSG','1309',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16657','geodetic_crs','EPSG','9753','EPSG','1262','EPSG','1176');
INSERT INTO "geodetic_crs" VALUES('EPSG','9754','WGS 84 (G2139)','Replaces WGS 84 (G1762) (CRS code 7665) from 2021-01-03. Replaced by WGS 84 (G2296) (CRS code 10605) from 2024-01-07.','geographic 3D','EPSG','6423','EPSG','1309',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16853','geodetic_crs','EPSG','9754','EPSG','1262','EPSG','1176');
INSERT INTO "geodetic_crs" VALUES('EPSG','9755','WGS 84 (G2139)','Replaces WGS 84 (G1762) (CRS code 9057) from 2021-01-03. Replaced by WGS 84 (G2296) (CRS code 10606) from 2024-01-07.','geographic 2D','EPSG','6422','EPSG','1309',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16852','geodetic_crs','EPSG','9755','EPSG','1262','EPSG','1176');
INSERT INTO "geodetic_crs" VALUES('EPSG','9758','ECML14_NB-IRF','Intermediate CRS created in 2021 to assist the emulation of the ETRS89 / ECML14_NB SnakeGrid projected CRS through transformation ETRS89 to ECML14_NB-IRF (1) (code 9759) used in conjunction with the ECML14_NB-TM map projection (code 9760).','geographic 2D','EPSG','6422','EPSG','1310',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16495','geodetic_crs','EPSG','9758','EPSG','4621','EPSG','1141');
INSERT INTO "geodetic_crs" VALUES('EPSG','9763','EWR2-IRF','Intermediate CRS created in 2021 to assist the emulation of the ETRS89 / EWR2 SnakeGrid projected CRS through transformation ETRS89 to EWR2-IRF (1) (code 9764) used in conjunction with the EWR2-TM map projection (code 9765). In 2024 the EWR2 Grid (ETRS89 / EWR2 SnakeGrid) was extended eastwards as the EWR3 Grid (ETRS89 / EWR3 SnakeGrid). To the west of Bedford the EWR2 Grid is an exact subset of the EWR3 Grid; over this extent the EWR3 Grid replaces the EWR2 Grid.','geographic 2D','EPSG','6422','EPSG','1311',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16507','geodetic_crs','EPSG','9763','EPSG','4622','EPSG','1141');
INSERT INTO "geodetic_crs" VALUES('EPSG','9775','RGF93 v2','Replaces RGF93 v1 (CRS code 4964) from 2010-06-18. Replaced by RGF93 v2b (CRS code 9780) from 2021-01-05.','geocentric','EPSG','6500','EPSG','1312',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16642','geodetic_crs','EPSG','9775','EPSG','1096','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','9776','RGF93 v2','Replaces RGF93 v1 CRS code 4965) from 2010-06-18 . Replaced by RGF93 v2b (CRS code 9781) from 2021-01-05. See CRS code 9778 for alternate system with horizontal axes reversed used by IGN for GIS purposes.','geographic 3D','EPSG','6423','EPSG','1312',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16641','geodetic_crs','EPSG','9776','EPSG','1096','EPSG','1182');
INSERT INTO "geodetic_crs" VALUES('EPSG','9777','RGF93 v2','Replaces RGF93 v1 (CRS code 4171) from 2010-06-18. Replaced by RGF93 v2b (CRS code 9782) from 2021-01-05. See CRS code 9779 for alternate system with axes reversed used by IGN for GIS purposes.','geographic 2D','EPSG','6422','EPSG','1312',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16643','geodetic_crs','EPSG','9777','EPSG','1096','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','9778','RGF93 v2 (lon-lat)','Replaces RGF93 v1 (lon-lat) from 2010-06-18. Replaced by RGF93 v2b (lon-lat) (CRS code 9783) from 2021-01-05. See CRS code 9776 for system with horizontal axes in sequence lat-lon to be used for air, land and sea navigation and safety of life purposes.','geographic 3D','EPSG','6426','EPSG','1312',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16647','geodetic_crs','EPSG','9778','EPSG','1096','EPSG','1189');
INSERT INTO "geodetic_crs" VALUES('EPSG','9779','RGF93 v2 (lon-lat)','Replaces RGF93 v1 (lon-lat) (code 7084) from 2010-06-18. Replaced by RGF93 v2b (lon-lat) (CRS code 9784) from 2021-01-05. See CRS code 9777 for system with axes in sequence lat-lon to be used for air, land and sea navigation and safety of life purposes.','geographic 2D','EPSG','6424','EPSG','1312',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16646','geodetic_crs','EPSG','9779','EPSG','1096','EPSG','1189');
INSERT INTO "geodetic_crs" VALUES('EPSG','9780','RGF93 v2b','Replaces RGF93 v2 (CRS code 9775) from 2021-01-05.','geocentric','EPSG','6500','EPSG','1313',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16649','geodetic_crs','EPSG','9780','EPSG','1096','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','9781','RGF93 v2b','Replaces RGF93 v2 (CRS code 9776) from 2021-01-05. See CRS code 9783 for alternate system with horizontal axes reversed used by IGN for GIS purposes.','geographic 3D','EPSG','6423','EPSG','1313',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16650','geodetic_crs','EPSG','9781','EPSG','1096','EPSG','1182');
INSERT INTO "geodetic_crs" VALUES('EPSG','9782','RGF93 v2b','Replaces RGF93 v2 (CRS code 9777) from 2021-01-05. See CRS code 9784 for alternate system with axes reversed used by IGN for GIS purposes.','geographic 2D','EPSG','6422','EPSG','1313',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16651','geodetic_crs','EPSG','9782','EPSG','1096','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','9783','RGF93 v2b (lon-lat)','Replaces RGF93 v2 (lon-lat) (CRS code 9778) from 2021-01-05. See CRS code 9781 for system with horizontal axes in sequence lat-lon to be used for air, land and sea navigation and safety of life purposes.','geographic 3D','EPSG','6426','EPSG','1313',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16652','geodetic_crs','EPSG','9783','EPSG','1096','EPSG','1189');
INSERT INTO "geodetic_crs" VALUES('EPSG','9784','RGF93 v2b (lon-lat)','Replaces RGF93 v2 (lon-lat) (CRS code 9779) from 2021-01-05. See CRS code 9782 for system with axes in sequence lat-lon to be used for air, land and sea navigation and safety of life purposes.','geographic 2D','EPSG','6424','EPSG','1313',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16653','geodetic_crs','EPSG','9784','EPSG','1096','EPSG','1189');
INSERT INTO "geodetic_crs" VALUES('EPSG','9866','MRH21-IRF','Intermediate CRS created in 2021 to assist the emulation of the ETRS89 / MRH21 SnakeGrid projected CRS through transformation ETRS89 to MRH21-IRF (1) (code 9867) used in conjunction with the MRH21-TM map projection (code 9868).','geographic 2D','EPSG','6422','EPSG','1314',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16934','geodetic_crs','EPSG','9866','EPSG','4652','EPSG','1141');
INSERT INTO "geodetic_crs" VALUES('EPSG','9871','MOLDOR11-IRF','Intermediate CRS created in 2021 to assist the emulation of the ETRS89 / MOLDOR11 SnakeGrid projected CRS through transformation ETRS89 to MOLDOR11-IRF (1) (code 9878) used in conjunction with the MOLDOR11-TM map projection (code 9879).','geographic 2D','EPSG','6422','EPSG','1315',NULL,0);
INSERT INTO "usage" VALUES('EPSG','16960','geodetic_crs','EPSG','9871','EPSG','4655','EPSG','1141');
INSERT INTO "geodetic_crs" VALUES('EPSG','9892','LUREF','','geocentric','EPSG','6500','EPSG','6181',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17243','geodetic_crs','EPSG','9892','EPSG','1146','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','9893','LUREF','Ellipsoidal height approximates to NG95 gravity-related heights to within 5-15cm. For accurate heighting use compound CRS LUREF / Luxembourg TM + NG95 height (code 9897).','geographic 3D','EPSG','6423','EPSG','6181',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17309','geodetic_crs','EPSG','9893','EPSG','1146','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','9939','EBBWV14-IRF','Intermediate CRS created in 2022 to assist the emulation of the ETRS89 / EBBWV14 SnakeGrid projected CRS through transformation ETRS89 to EBBWV14-IRF (1) (code 9941) used in conjunction with the EBBWV14-TM map projection (code 9942).','geographic 2D','EPSG','6422','EPSG','1319',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17353','geodetic_crs','EPSG','9939','EPSG','4661','EPSG','1141');
INSERT INTO "geodetic_crs" VALUES('EPSG','9964','HULLEE13-IRF','Intermediate CRS created in 2022 to assist the emulation of the ETRS89 / HULLEE13 SnakeGrid projected CRS through transformation ETRS89 to HULLEE13-IRF (1) (code 9965) used in conjunction with the HULLEE13-TM map projection (code 9966).','geographic 2D','EPSG','6422','EPSG','1317',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17424','geodetic_crs','EPSG','9964','EPSG','4663','EPSG','1141');
INSERT INTO "geodetic_crs" VALUES('EPSG','9969','SCM22-IRF','Intermediate CRS created in 2022 to assist the emulation of the ETRS89 / SCM22 SnakeGrid projected CRS through transformation ETRS89 to SCM22-IRF (1) (code 9969) used in conjunction with the SCM22-TM map projection (code 9971).','geographic 2D','EPSG','6422','EPSG','1320',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17438','geodetic_crs','EPSG','9969','EPSG','4665','EPSG','1141');
INSERT INTO "geodetic_crs" VALUES('EPSG','9974','FNL22-IRF','Intermediate CRS created in 2022 to assist the emulation of the ETRS89 / FNL22 SnakeGrid projected CRS through transformation ETRS89 to FNL22-IRF (1) (code 9975) used in conjunction with the FNL22-TM map projection (code 9976).','geographic 2D','EPSG','6422','EPSG','1321',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17451','geodetic_crs','EPSG','9974','EPSG','4664','EPSG','1141');
INSERT INTO "geodetic_crs" VALUES('EPSG','9988','ITRF2020','Replaces ITRF2014 (CRS code 7789). Replaced by ITRF2020-u2023 (CRS code 10779).','geocentric','EPSG','6500','EPSG','1322',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17904','geodetic_crs','EPSG','9988','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','9989','ITRF2020','Replaces ITRF2014 (code 7912).  Replaced by ITRF2020-u2023 (CRS code 10780).','geographic 3D','EPSG','6423','EPSG','1322',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17902','geodetic_crs','EPSG','9989','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','9990','ITRF2020','Replaces ITRF2014 (code 9000).  Replaced by ITRF2020-u2023 (CRS code 10781).','geographic 2D','EPSG','6422','EPSG','1322',NULL,0);
INSERT INTO "usage" VALUES('EPSG','17901','geodetic_crs','EPSG','9990','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','10158','S34J-IRF','Artificial CRS created in 2022 to assist the transformation of coordinates between the historic S34J CRS and ETRS89 through transformation ETRS89 to S34J-IRF (1) (code 10161) used in conjunction with the S34-reconstruction map projection (code 10159).','geographic 2D','EPSG','6422','EPSG','1332',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19659','geodetic_crs','EPSG','10158','EPSG','2531','EPSG','1203');
INSERT INTO "geodetic_crs" VALUES('EPSG','10175','DoPw22-IRF','Intermediate CRS created in 2022 to assist the emulation of the ETRS89 / DoPw22 SnakeGrid projected CRS through transformation ETRS89 to DoPw22-IRF (1) (code 10181) used in conjunction with the DoPw22-TM map projection (code 10182).','geographic 2D','EPSG','6422','EPSG','1334',NULL,0);
INSERT INTO "usage" VALUES('EPSG','18657','geodetic_crs','EPSG','10175','EPSG','4686','EPSG','1141');
INSERT INTO "geodetic_crs" VALUES('EPSG','10176','IGS20','Used for products from the International GNSS Service (IGS) from 2022-11-27 to 2025-02-01. Replaces IGb14 (code 9378). Replaced by IGb20 (code 10783). For most practical purposes IGS20 is equivalent to ITRF2020.','geocentric','EPSG','6500','EPSG','1333',NULL,0);
INSERT INTO "usage" VALUES('EPSG','18627','geodetic_crs','EPSG','10176','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','10177','IGS20','Used for products from the International GNSS Service (IGS) from 2022-11-27 to 2025-02-01. Replaces IGb14 (code 9379). Replaced by IGb20 (code 10784). For most practical purposes IGS20 is equivalent to ITRF2020.','geographic 3D','EPSG','6423','EPSG','1333',NULL,0);
INSERT INTO "usage" VALUES('EPSG','18628','geodetic_crs','EPSG','10177','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','10178','IGS20','Used for products from the International GNSS Service (IGS) from 2022-11-27 to 2025-02-01. Replaces IGb14 (code 9380). Replaced by IGb20 (code 10785). For most practical purposes IGS20 is equivalent to ITRF2020.','geographic 2D','EPSG','6422','EPSG','1333',NULL,0);
INSERT INTO "usage" VALUES('EPSG','18629','geodetic_crs','EPSG','10178','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','10185','ShAb07-IRF','Intermediate CRS created in 2022 to assist the emulation of the ETRS89 / ShAb07 SnakeGrid projected CRS through transformation ETRS89 to ShAb07-IRF (1) (code 10186) used in conjunction with the ShAb07-TM map projection (code 10187).','geographic 2D','EPSG','6422','EPSG','1335',NULL,0);
INSERT INTO "usage" VALUES('EPSG','18854','geodetic_crs','EPSG','10185','EPSG','4687','EPSG','1141');
INSERT INTO "geodetic_crs" VALUES('EPSG','10191','CNH22-IRF','Intermediate CRS created in 2022 to assist the emulation of the ETRS89 / CNH22 SnakeGrid projected CRS through transformation ETRS89 to CNH22-IRF (1) (code 10192) used in conjunction with the CNH22-LCC map projection (code 10193).','geographic 2D','EPSG','6422','EPSG','1336',NULL,0);
INSERT INTO "usage" VALUES('EPSG','18717','geodetic_crs','EPSG','10191','EPSG','4677','EPSG','1141');
INSERT INTO "geodetic_crs" VALUES('EPSG','10196','CWS13-IRF','Intermediate CRS created in 2022 to assist the emulation of the ETRS89 / CWS13 SnakeGrid projected CRS through transformation ETRS89 to CWS13-IRF (1) (code 10197) used in conjunction with the CWS13-TM map projection (code 10198).','geographic 2D','EPSG','6422','EPSG','1338',NULL,0);
INSERT INTO "usage" VALUES('EPSG','18705','geodetic_crs','EPSG','10196','EPSG','4678','EPSG','1141');
INSERT INTO "geodetic_crs" VALUES('EPSG','10204','DIBA15-IRF','Intermediate CRS created in 2022 to assist the emulation of the ETRS89 / DIBA15 SnakeGrid projected CRS through transformation ETRS89 to DIBA15-IRF (1) (code 10205) used in conjunction with the DIBA15-TM map projection (code 10206).','geographic 2D','EPSG','6422','EPSG','1339',NULL,0);
INSERT INTO "usage" VALUES('EPSG','18714','geodetic_crs','EPSG','10204','EPSG','4681','EPSG','1141');
INSERT INTO "geodetic_crs" VALUES('EPSG','10209','GWPBS22-IRF','Intermediate CRS created in 2022 to assist the emulation of the ETRS89 / GWPBS22 SnakeGrid projected CRS through transformation ETRS89 to GWPBS22-IRF (1) (code 10210) used in conjunction with the GW22-LCC map projection (code 10211).','geographic 2D','EPSG','6422','EPSG','1340',NULL,0);
INSERT INTO "usage" VALUES('EPSG','18796','geodetic_crs','EPSG','10209','EPSG','4685','EPSG','1141');
INSERT INTO "geodetic_crs" VALUES('EPSG','10214','GWWAB22-IRF','Intermediate CRS created in 2022 to assist the emulation of the ETRS89 / GWWAB22 SnakeGrid projected CRS through transformation ETRS89 to GWWAB22-IRF (1) (code 10215) used in conjunction with the GW22-LCC map projection (code 10211).','geographic 2D','EPSG','6422','EPSG','1341',NULL,0);
INSERT INTO "usage" VALUES('EPSG','18798','geodetic_crs','EPSG','10214','EPSG','4684','EPSG','1141');
INSERT INTO "geodetic_crs" VALUES('EPSG','10219','GWWWA22-IRF','Intermediate CRS created in 2022 to assist the emulation of the ETRS89 / GWWWA22 SnakeGrid projected CRS through transformation ETRS89 to GWWWA22-IRF (1) (code 10220) used in conjunction with the GW22-LCC map projection (code 10211).','geographic 2D','EPSG','6422','EPSG','1342',NULL,0);
INSERT INTO "usage" VALUES('EPSG','18800','geodetic_crs','EPSG','10219','EPSG','4683','EPSG','1141');
INSERT INTO "geodetic_crs" VALUES('EPSG','10224','MALS09-IRF','Intermediate CRS created in 2022 to assist the emulation of the ETRS89 / MALS09 SnakeGrid projected CRS through transformation ETRS89 to MALS09-IRF (1) (code 10225) used in conjunction with the MALS09-TM map projection (code 10226).','geographic 2D','EPSG','6422','EPSG','1343',NULL,0);
INSERT INTO "usage" VALUES('EPSG','18749','geodetic_crs','EPSG','10224','EPSG','4682','EPSG','1141');
INSERT INTO "geodetic_crs" VALUES('EPSG','10229','OxWo08-IRF','Intermediate CRS created in 2022 to assist the emulation of the ETRS89 / OxWo08 SnakeGrid projected CRS through transformation ETRS89 to OxWo08-IRF (1) (code 10230) used in conjunction with the OxWo08-TM map projection (code 10234).','geographic 2D','EPSG','6422','EPSG','1344',NULL,0);
INSERT INTO "usage" VALUES('EPSG','18784','geodetic_crs','EPSG','10229','EPSG','4680','EPSG','1141');
INSERT INTO "geodetic_crs" VALUES('EPSG','10237','SYC20-IRF','Intermediate CRS created in 2022 to assist the emulation of the ETRS89 / SYC20 SnakeGrid projected CRS through transformation ETRS89 to SYC20-IRF (1) (code 10238) used in conjunction with the SYC20-TM map projection (code 10239).','geographic 2D','EPSG','6422','EPSG','1345',NULL,0);
INSERT INTO "usage" VALUES('EPSG','18789','geodetic_crs','EPSG','10237','EPSG','4679','EPSG','1141');
INSERT INTO "geodetic_crs" VALUES('EPSG','10249','S34S-IRF','Artificial CRS created in 2022 to assist the transformation of coordinates between the historic S34S CRS and ETRS89 through CT ETRS89 to S34S-IRF (1) (code 10251) used in conjunction with the S34-reconstruction map projection (code 10159).','geographic 2D','EPSG','6422','EPSG','1337',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19667','geodetic_crs','EPSG','10249','EPSG','2532','EPSG','1028');
INSERT INTO "geodetic_crs" VALUES('EPSG','10252','S45B-IRF','Artificial CRS created in 2022 to assist the transformation of coordinates between the historic S45 CRS and ETRS89 through transformation ETRS89 to S45B-IRF (1) (code 10255) used in conjunction with the S45B-reconstruction map projection (code 10253).','geographic 2D','EPSG','6422','EPSG','1346',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19671','geodetic_crs','EPSG','10252','EPSG','2533','EPSG','1203');
INSERT INTO "geodetic_crs" VALUES('EPSG','10256','GS-IRF','Artificial CRS created in 2022 to assist the transformation of coordinates between the historic Generalstabens System and ETRS89 through CT ETRS89 to GS-IRF (1) (code 10259) used in conjunction with the GS LCC map projection (code 10257).','geographic 2D','EPSG','6422','EPSG','1347',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19707','geodetic_crs','EPSG','10256','EPSG','4575','EPSG','1203');
INSERT INTO "geodetic_crs" VALUES('EPSG','10260','GSB-IRF','Artificial CRS created in 2022 to assist the transformation of coordinates between the historic Generalstabens System and ETRS89 through CT ETRS89 to GSB-IRF (1) (code 10263) used in conjunction with the GSB-reconstruction map projection (code 10261).','geographic 2D','EPSG','6422','EPSG','1348',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19682','geodetic_crs','EPSG','10260','EPSG','2533','EPSG','1203');
INSERT INTO "geodetic_crs" VALUES('EPSG','10265','KK-IRF','Artificial CRS created in 2022 to assist the transformation of coordinates between the historic Københavns Kommunes CRS and ETRS89 through CT ETRS89 to KK-IRF (1) (code 10267) used in conjunction with the GS LCC map projection (code 10257).','geographic 2D','EPSG','6422','EPSG','1349',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19712','geodetic_crs','EPSG','10265','EPSG','4693','EPSG','1203');
INSERT INTO "geodetic_crs" VALUES('EPSG','10268','Ostenfeld-IRF','Artificial CRS created in 2022 to assist the transformation of coordinates between the historic Ostenfeld CRS and ETRS89 through transformation ETRS89 to Ostenfeld-IRF (1) (code 10271) used with the Ostenfeld-reconstruction map projection (code 10269).','geographic 2D','EPSG','6422','EPSG','1350',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19696','geodetic_crs','EPSG','10268','EPSG','4694','EPSG','1203');
INSERT INTO "geodetic_crs" VALUES('EPSG','10272','SMITB20-IRF','Intermediate CRS created in 2022 to assist the emulation of the ETRS89 / SMITB20 SnakeGrid projected CRS through transformation ETRS89 to SMITB20-IRF (1) (code 10273) used in conjunction with the SMITB20-TM map projection (code 10274).','geographic 2D','EPSG','6422','EPSG','1351',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19175','geodetic_crs','EPSG','10272','EPSG','4688','EPSG','1141');
INSERT INTO "geodetic_crs" VALUES('EPSG','10277','RBEPP12-IRF','Intermediate CRS created in 2022 to assist the emulation of the ETRS89 / RBEPP12 SnakeGrid projected CRS through transformation ETRS89 to RBEPP12-IRF (1) (code 10278) used in conjunction with the RBEPP12-LCC map projection (code 10279).','geographic 2D','EPSG','6422','EPSG','1352',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19181','geodetic_crs','EPSG','10277','EPSG','4689','EPSG','1141');
INSERT INTO "geodetic_crs" VALUES('EPSG','10282','ETRS89/DREF91/2016','German national realization of ETRS89. Replaces ETRS89/DREF91 Realization 2002 from 2016-12-01.','geocentric','EPSG','6500','EPSG','1353',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19243','geodetic_crs','EPSG','10282','EPSG','1103','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','10283','ETRS89/DREF91/2016','German national realization of ETRS89. Replaces ETRS89/DREF91 Realization 2002 from 2016-12-01.','geographic 3D','EPSG','6423','EPSG','1353',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19203','geodetic_crs','EPSG','10283','EPSG','1103','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','10284','ETRS89/DREF91/2016','German national realization of ETRS89. Replaces ETRS89/DREF91 Realization 2002 from 2016-12-01.','geographic 2D','EPSG','6422','EPSG','1353',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19244','geodetic_crs','EPSG','10284','EPSG','1103','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','10297','RGSH2020','','geocentric','EPSG','6500','EPSG','1355',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19531','geodetic_crs','EPSG','10297','EPSG','1026','EPSG','1136');
INSERT INTO "geodetic_crs" VALUES('EPSG','10298','RGSH2020','','geographic 3D','EPSG','6423','EPSG','1355',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19533','geodetic_crs','EPSG','10298','EPSG','1026','EPSG','1136');
INSERT INTO "geodetic_crs" VALUES('EPSG','10299','RGSH2020','','geographic 2D','EPSG','6422','EPSG','1355',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19534','geodetic_crs','EPSG','10299','EPSG','1026','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','10300','RGNC91-93 (lon-lat)','See CRS code 4907 for system with horizontal axes in sequence lat-lon to be used for air, land and sea navigation and safety of life purposes. Replaced by RGNC15 (lon-lat) (CRS code 10311).','geographic 3D','EPSG','6426','EPSG','6749',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19598','geodetic_crs','EPSG','10300','EPSG','1174','EPSG','1189');
INSERT INTO "geodetic_crs" VALUES('EPSG','10303','LKS-2020','Replaces LKS-92 (CRS code 4948) from 2026-10-01.','geocentric','EPSG','6500','EPSG','1356',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19562','geodetic_crs','EPSG','10303','EPSG','1139','EPSG','1026');
INSERT INTO "geodetic_crs" VALUES('EPSG','10304','LKS-2020','Replaces LKS-92 (CRS code 4949) from 2026-10-01.','geographic 3D','EPSG','6423','EPSG','1356',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19554','geodetic_crs','EPSG','10304','EPSG','1139','EPSG','1026');
INSERT INTO "geodetic_crs" VALUES('EPSG','10305','LKS-2020','Replaces LKS-92 (CRS code 4661) from 2026-10-01.','geographic 2D','EPSG','6422','EPSG','1356',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19552','geodetic_crs','EPSG','10305','EPSG','1139','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','10307','RGNC91-93 (lon-lat)','See CRS code 4749 for system with axes in sequence lat-lon to be used for air, land and sea navigation and safety of life purposes. Replaced by RGNC15 (lon-lat) (CRS code 10312).','geographic 2D','EPSG','6424','EPSG','6749',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19599','geodetic_crs','EPSG','10307','EPSG','1174','EPSG','1189');
INSERT INTO "geodetic_crs" VALUES('EPSG','10308','RGNC15','Replaces RGNC91-93 (CRS code 4906).','geocentric','EPSG','6500','EPSG','1357',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19579','geodetic_crs','EPSG','10308','EPSG','1174','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','10309','RGNC15','Replaces RGNC91-93 (CRS code 4907). See CRS code 10311 for alternate system with horizontal axes reversed used by DITTT for GIS purposes.','geographic 3D','EPSG','6423','EPSG','1357',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19592','geodetic_crs','EPSG','10309','EPSG','1174','EPSG','1182');
INSERT INTO "geodetic_crs" VALUES('EPSG','10310','RGNC15','Replaces RGNC91-93 (CRS code 4749). See CRS code 10312 for alternate system with axes reversed used by DITTT for GIS purposes.','geographic 2D','EPSG','6422','EPSG','1357',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19593','geodetic_crs','EPSG','10310','EPSG','1174','EPSG','1182');
INSERT INTO "geodetic_crs" VALUES('EPSG','10311','RGNC15 (lon-lat)','Replaces RGNC91-93 (lon-lat) (CRS code 10300). See CRS code 10309 for system with horizontal axes in sequence lat-lon to be used for air, land and sea navigation and safety of life purposes.','geographic 3D','EPSG','6426','EPSG','1357',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19601','geodetic_crs','EPSG','10311','EPSG','1174','EPSG','1189');
INSERT INTO "geodetic_crs" VALUES('EPSG','10312','RGNC15 (lon-lat)','Replaces RGNC91-93 (lon-lat) (CRS code 10307). See CRS code 10310 for system with axes in sequence lat-lon to be used for air, land and sea navigation and safety of life purposes.','geographic 2D','EPSG','6424','EPSG','1357',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19595','geodetic_crs','EPSG','10312','EPSG','1174','EPSG','1189');
INSERT INTO "geodetic_crs" VALUES('EPSG','10326','BH_ETRS89','','geocentric','EPSG','6500','EPSG','1358',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19732','geodetic_crs','EPSG','10326','EPSG','1050','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','10327','BH_ETRS89','','geographic 3D','EPSG','6423','EPSG','1358',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19742','geodetic_crs','EPSG','10327','EPSG','1050','EPSG','1026');
INSERT INTO "geodetic_crs" VALUES('EPSG','10328','BH_ETRS89','In Bosnia and Herzegovina replaces MGI 1901 (CRS code 3906).','geographic 2D','EPSG','6422','EPSG','1358',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19734','geodetic_crs','EPSG','10328','EPSG','1050','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','10345','Hughes 1980','Used as basis for DMSP SSM/I data sets provided by NSIDC for polar research.','geographic 2D','EPSG','6422','EPSG','1359',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19882','geodetic_crs','EPSG','10345','EPSG','1262','EPSG','1110');
INSERT INTO "geodetic_crs" VALUES('EPSG','10346','NSIDC Authalic Sphere','Adopted by NSIDC for use with EASE-Grid v1. For EASE-Grid v2, WGS 84 is used.','geographic 2D','EPSG','6422','EPSG','1360',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19878','geodetic_crs','EPSG','10346','EPSG','1262','EPSG','1195');
INSERT INTO "geodetic_crs" VALUES('EPSG','10412','NAD83(CSRS)v8','Adopted by the Canadian federal government from 2022-11-27. Replaces NAD83(CSRS)v7.','geocentric','EPSG','6500','EPSG','1365',NULL,0);
INSERT INTO "usage" VALUES('EPSG','20139','geodetic_crs','EPSG','10412','EPSG','1061','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','10413','NAD83(CSRS)v8','Adopted by the Canadian federal government from 2022-11-27. Replaces NAD83(CSRS)v7. Longitudes are POSITIVE EAST.','geographic 3D','EPSG','6423','EPSG','1365',NULL,0);
INSERT INTO "usage" VALUES('EPSG','20141','geodetic_crs','EPSG','10413','EPSG','1061','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','10414','NAD83(CSRS)v8','Adopted by the Canadian federal government from 2022-11-27. Replaces NAD83(CSRS)v7. Longitudes are POSITIVE EAST.','geographic 2D','EPSG','6422','EPSG','1365',NULL,0);
INSERT INTO "usage" VALUES('EPSG','20156','geodetic_crs','EPSG','10414','EPSG','1061','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','10468','COV23-IRF','Intermediate CRS created in 2023 to assist the emulation of the ETRS89 / COV23 SnakeGrid projected CRS through transformation ETRS89 to COV23 (1) (code 10469) used in conjunction with the COV23-TM map projection (code 10470).','geographic 2D','EPSG','6422','EPSG','1366',NULL,0);
INSERT INTO "usage" VALUES('EPSG','20311','geodetic_crs','EPSG','10468','EPSG','4743','EPSG','1141');
INSERT INTO "geodetic_crs" VALUES('EPSG','10473','BBT2000','','geocentric','EPSG','6500','EPSG','1367',NULL,0);
INSERT INTO "usage" VALUES('EPSG','20338','geodetic_crs','EPSG','10473','EPSG','4744','EPSG','1285');
INSERT INTO "geodetic_crs" VALUES('EPSG','10474','BBT2000','','geographic 3D','EPSG','6423','EPSG','1367',NULL,0);
INSERT INTO "usage" VALUES('EPSG','20339','geodetic_crs','EPSG','10474','EPSG','4744','EPSG','1285');
INSERT INTO "geodetic_crs" VALUES('EPSG','10475','BBT2000','','geographic 2D','EPSG','6422','EPSG','1367',NULL,0);
INSERT INTO "usage" VALUES('EPSG','20331','geodetic_crs','EPSG','10475','EPSG','4744','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','10569','ETRF2020','Replaces ETRF2014 (code 8401). ETRF2020 is technically superior to ETRF2000 but ETRF2000 and other previous realizations may be preferred for backward compatibility reasons. Differences between ETRF2020 and ETRF2000 can reach 7cm.','geocentric','EPSG','6500','EPSG','1382',NULL,0);
INSERT INTO "usage" VALUES('EPSG','21226','geodetic_crs','EPSG','10569','EPSG','4755','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','10570','ETRF2020','Replaces ETRF2014 (code 8403). ETRF2020 is technically superior to ETRF2000 but ETRF2000 and other previous realizations may be preferred for backward compatibility reasons. Differences between ETRF2020 and ETRF2000 can reach 7cm.','geographic 3D','EPSG','6423','EPSG','1382',NULL,0);
INSERT INTO "usage" VALUES('EPSG','21227','geodetic_crs','EPSG','10570','EPSG','1298','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','10571','ETRF2020','Replaces ETRF2014 (code 9069). ETRF2020 is technically superior to ETRF2000 but ETRF2000 and other previous realizations may be preferred for backward compatibility reasons. Differences between ETRF2020 and ETRF2000 can reach 7cm.','geographic 2D','EPSG','6422','EPSG','1382',NULL,0);
INSERT INTO "usage" VALUES('EPSG','21228','geodetic_crs','EPSG','10571','EPSG','1298','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','10604','WGS 84 (G2296)','Replaces WGS 84 (G2139) (CRS code 9753) from 2024-01-07.','geocentric','EPSG','6500','EPSG','1383',NULL,0);
INSERT INTO "usage" VALUES('EPSG','21200','geodetic_crs','EPSG','10604','EPSG','1262','EPSG','1176');
INSERT INTO "geodetic_crs" VALUES('EPSG','10605','WGS 84 (G2296)','Replaces WGS 84 (G2139) (CRS code 9754) from 2024-01-07.','geographic 3D','EPSG','6423','EPSG','1383',NULL,0);
INSERT INTO "usage" VALUES('EPSG','21195','geodetic_crs','EPSG','10605','EPSG','2830','EPSG','1176');
INSERT INTO "geodetic_crs" VALUES('EPSG','10606','WGS 84 (G2296)','Replaces WGS 84 (G2139) (CRS code 9755) from 2024-01-07.','geographic 2D','EPSG','6422','EPSG','1383',NULL,0);
INSERT INTO "usage" VALUES('EPSG','21196','geodetic_crs','EPSG','10606','EPSG','1262','EPSG','1176');
INSERT INTO "geodetic_crs" VALUES('EPSG','10623','ECML14-IRF','Intermediate CRS created in 2024 to assist the emulation of the ETRS89 / ECML14 SnakeGrid projected CRS through transformation ETRS89 to ECML14-IRF (1) (code 10624) used in conjunction with the ECML14-TM map projection (code 10625).','geographic 2D','EPSG','6422','EPSG','1385',NULL,0);
INSERT INTO "usage" VALUES('EPSG','21387','geodetic_crs','EPSG','10623','EPSG','4774','EPSG','1141');
INSERT INTO "geodetic_crs" VALUES('EPSG','10628','WC05-IRF','Intermediate CRS created in 2024 to assist the emulation of the ETRS89 / WC05 SnakeGrid projected CRS through transformation ETRS89 to WC05-IRF (1) (code 10629) used in conjunction with the WC05-TM map projection (code 10631).','geographic 2D','EPSG','6422','EPSG','1386',NULL,0);
INSERT INTO "usage" VALUES('EPSG','21381','geodetic_crs','EPSG','10628','EPSG','4775','EPSG','1141');
INSERT INTO "geodetic_crs" VALUES('EPSG','10634','Saba','','geocentric','EPSG','6500','EPSG','1379',NULL,0);
INSERT INTO "usage" VALUES('EPSG','21863','geodetic_crs','EPSG','10634','EPSG','4757','EPSG','1269');
INSERT INTO "geodetic_crs" VALUES('EPSG','10635','Saba','','geographic 3D','EPSG','6423','EPSG','1379',NULL,0);
INSERT INTO "usage" VALUES('EPSG','21864','geodetic_crs','EPSG','10635','EPSG','4757','EPSG','1269');
INSERT INTO "geodetic_crs" VALUES('EPSG','10636','Saba','','geographic 2D','EPSG','6422','EPSG','1379',NULL,0);
INSERT INTO "usage" VALUES('EPSG','21865','geodetic_crs','EPSG','10636','EPSG','4757','EPSG','1269');
INSERT INTO "geodetic_crs" VALUES('EPSG','10637','BES2020 Saba','','geocentric','EPSG','6500','EPSG','1380',NULL,0);
INSERT INTO "usage" VALUES('EPSG','21866','geodetic_crs','EPSG','10637','EPSG','4757','EPSG','1179');
INSERT INTO "geodetic_crs" VALUES('EPSG','10638','BES2020 Saba','','geographic 3D','EPSG','6423','EPSG','1380',NULL,0);
INSERT INTO "usage" VALUES('EPSG','21867','geodetic_crs','EPSG','10638','EPSG','4757','EPSG','1179');
INSERT INTO "geodetic_crs" VALUES('EPSG','10639','BES2020 Saba','','geographic 2D','EPSG','6422','EPSG','1380',NULL,0);
INSERT INTO "usage" VALUES('EPSG','21868','geodetic_crs','EPSG','10639','EPSG','4757','EPSG','1178');
INSERT INTO "geodetic_crs" VALUES('EPSG','10669','RGM23','Replaces RGM04 (CRS code 4468) with effect from 2023-01-01.','geocentric','EPSG','6500','EPSG','1389',NULL,0);
INSERT INTO "usage" VALUES('EPSG','21846','geodetic_crs','EPSG','10669','EPSG','1159','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','10670','RGM23','Replaces RGM04 (CRS code 4469) with effect from 2023-01-01. See CRS code 10672 for alternate system with horizontal axes reversed used by IGN for GIS purposes.','geographic 3D','EPSG','6423','EPSG','1389',NULL,0);
INSERT INTO "usage" VALUES('EPSG','21845','geodetic_crs','EPSG','10670','EPSG','1159','EPSG','1182');
INSERT INTO "geodetic_crs" VALUES('EPSG','10671','RGM23','Replaces RGM04 (CRS code 4470) with effect from 2023-01-01. See CRS code 10673 for alternate system with axes reversed used by IGN for GIS purposes.','geographic 2D','EPSG','6422','EPSG','1389',NULL,0);
INSERT INTO "usage" VALUES('EPSG','21844','geodetic_crs','EPSG','10671','EPSG','1159','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','10672','RGM23 (lon-lat)','Replaces RGM04 (lon-lat) (CRS code 7038) with effect from 2023-01-01. See CRS code 10670 for system with horizontal axes in sequence lat-lon to be used for air, land and sea navigation and safety of life purposes.','geographic 3D','EPSG','6426','EPSG','1389',NULL,0);
INSERT INTO "usage" VALUES('EPSG','21686','geodetic_crs','EPSG','10672','EPSG','1159','EPSG','1189');
INSERT INTO "geodetic_crs" VALUES('EPSG','10673','RGM23 (lon-lat)','Replaces RGM04 (lon-lat) (code 7039) from 2023-01-01 except for cadastral purposes which use Cadastre 1997 (code 4475). See CRS code 10671 for system with axes in sequence lat-lon to be used for air, land and sea navigation and safety of life purposes.','geographic 2D','EPSG','6424','EPSG','1389',NULL,0);
INSERT INTO "usage" VALUES('EPSG','21687','geodetic_crs','EPSG','10673','EPSG','1159','EPSG','1189');
INSERT INTO "geodetic_crs" VALUES('EPSG','10688','EUREF-FIN','EUREF-FIN is the national realization of ETRS89 in Finland.','geocentric','EPSG','6500','EPSG','1391',NULL,0);
INSERT INTO "usage" VALUES('EPSG','21977','geodetic_crs','EPSG','10688','EPSG','1095','EPSG','1181');
INSERT INTO "geodetic_crs" VALUES('EPSG','10689','EUREF-FIN','EUREF-FIN is the national realization of ETRS89 in Finland.','geographic 3D','EPSG','6423','EPSG','1391',NULL,0);
INSERT INTO "usage" VALUES('EPSG','21978','geodetic_crs','EPSG','10689','EPSG','1095','EPSG','1181');
INSERT INTO "geodetic_crs" VALUES('EPSG','10690','EUREF-FIN','EUREF-FIN is the national realization of ETRS89 in Finland.','geographic 2D','EPSG','6422','EPSG','1391',NULL,0);
INSERT INTO "usage" VALUES('EPSG','22200','geodetic_crs','EPSG','10690','EPSG','1095','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','10723','UZGD2024','','geocentric','EPSG','6500','EPSG','1392',NULL,0);
INSERT INTO "usage" VALUES('EPSG','22087','geodetic_crs','EPSG','10723','EPSG','1248','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','10724','UZGD2024','','geographic 3D','EPSG','6423','EPSG','1392',NULL,0);
INSERT INTO "usage" VALUES('EPSG','22088','geodetic_crs','EPSG','10724','EPSG','1248','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','10725','UZGD2024','Replaces usage of Pulkovo 1942 in Uzbekistan from 4th July 2024.','geographic 2D','EPSG','6422','EPSG','1392',NULL,0);
INSERT INTO "usage" VALUES('EPSG','22089','geodetic_crs','EPSG','10725','EPSG','1248','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','10734','Sint Eustatius','','geocentric','EPSG','6500','EPSG','1393',NULL,0);
INSERT INTO "usage" VALUES('EPSG','22282','geodetic_crs','EPSG','10734','EPSG','4788','EPSG','1269');
INSERT INTO "geodetic_crs" VALUES('EPSG','10735','Sint Eustatius','','geographic 3D','EPSG','6423','EPSG','1393',NULL,0);
INSERT INTO "usage" VALUES('EPSG','22283','geodetic_crs','EPSG','10735','EPSG','4788','EPSG','1269');
INSERT INTO "geodetic_crs" VALUES('EPSG','10736','Sint Eustatius','','geographic 2D','EPSG','6422','EPSG','1393',NULL,0);
INSERT INTO "usage" VALUES('EPSG','22284','geodetic_crs','EPSG','10736','EPSG','4788','EPSG','1269');
INSERT INTO "geodetic_crs" VALUES('EPSG','10737','BES2020 Sint Eustatius','','geocentric','EPSG','6500','EPSG','1394',NULL,0);
INSERT INTO "usage" VALUES('EPSG','22285','geodetic_crs','EPSG','10737','EPSG','4788','EPSG','1179');
INSERT INTO "geodetic_crs" VALUES('EPSG','10738','BES2020 Sint Eustatius','','geographic 3D','EPSG','6423','EPSG','1394',NULL,0);
INSERT INTO "usage" VALUES('EPSG','22286','geodetic_crs','EPSG','10738','EPSG','4788','EPSG','1179');
INSERT INTO "geodetic_crs" VALUES('EPSG','10739','BES2020 Sint Eustatius','','geographic 2D','EPSG','6422','EPSG','1394',NULL,0);
INSERT INTO "usage" VALUES('EPSG','22287','geodetic_crs','EPSG','10739','EPSG','4788','EPSG','1178');
INSERT INTO "geodetic_crs" VALUES('EPSG','10758','Bonaire','','geographic 2D','EPSG','6422','EPSG','1396',NULL,0);
INSERT INTO "usage" VALUES('EPSG','22168','geodetic_crs','EPSG','10758','EPSG','3822','EPSG','1269');
INSERT INTO "geodetic_crs" VALUES('EPSG','10760','Bonaire 2004','','geocentric','EPSG','6500','EPSG','1397',NULL,0);
INSERT INTO "usage" VALUES('EPSG','22269','geodetic_crs','EPSG','10760','EPSG','3822','EPSG','1179');
INSERT INTO "geodetic_crs" VALUES('EPSG','10761','Bonaire 2004','','geographic 3D','EPSG','6423','EPSG','1397',NULL,0);
INSERT INTO "usage" VALUES('EPSG','22172','geodetic_crs','EPSG','10761','EPSG','3822','EPSG','1179');
INSERT INTO "geodetic_crs" VALUES('EPSG','10762','Bonaire 2004','','geographic 2D','EPSG','6422','EPSG','1397',NULL,0);
INSERT INTO "usage" VALUES('EPSG','22173','geodetic_crs','EPSG','10762','EPSG','3822','EPSG','1178');
INSERT INTO "geodetic_crs" VALUES('EPSG','10779','ITRF2020-u2023','Replaces ITRF2020 (CRS code 9988).','geocentric','EPSG','6500','EPSG','1399',NULL,0);
INSERT INTO "usage" VALUES('EPSG','22372','geodetic_crs','EPSG','10779','EPSG','2830','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','10780','ITRF2020-u2023','Replaces ITRF2020 (code 9989).','geographic 3D','EPSG','6423','EPSG','1399',NULL,0);
INSERT INTO "usage" VALUES('EPSG','22335','geodetic_crs','EPSG','10780','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','10781','ITRF2020-u2023','Replaces ITRF2020 (code 9990).','geographic 2D','EPSG','6422','EPSG','1399',NULL,0);
INSERT INTO "usage" VALUES('EPSG','22336','geodetic_crs','EPSG','10781','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','10783','IGb20','Used for products from the International GNSS Service (IGS) from 2025-02-02. Replaces IGS20 (code 10176). For most practical purposes IGb20 is equivalent to ITRF2020-u2023.','geocentric','EPSG','6500','EPSG','1400',NULL,0);
INSERT INTO "usage" VALUES('EPSG','22363','geodetic_crs','EPSG','10783','EPSG','2830','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','10784','IGb20','Used for products from the International GNSS Service (IGS) from 2025-02-02. Replaces IGS20 (code 10177). For most practical purposes IGb20 is equivalent to ITRF2020-u2023.','geographic 3D','EPSG','6423','EPSG','1400',NULL,0);
INSERT INTO "usage" VALUES('EPSG','22364','geodetic_crs','EPSG','10784','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','10785','IGb20','Used for products from the International GNSS Service (IGS) from 2025-02-02. Replaces IGS20 (code 10178). For most practical purposes IGb20 is equivalent to ITRF2020-u2023.','geographic 2D','EPSG','6422','EPSG','1400',NULL,0);
INSERT INTO "usage" VALUES('EPSG','22365','geodetic_crs','EPSG','10785','EPSG','1262','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','10789','UGRF','','geocentric','EPSG','6500','EPSG','1401',NULL,0);
INSERT INTO "usage" VALUES('EPSG','22457','geodetic_crs','EPSG','10789','EPSG','1241','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','10790','UGRF','','geographic 3D','EPSG','6423','EPSG','1401',NULL,0);
INSERT INTO "usage" VALUES('EPSG','22458','geodetic_crs','EPSG','10790','EPSG','1241','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','10791','UGRF','Replaces Arc 1960 (code 4210) in Uganda.','geographic 2D','EPSG','6422','EPSG','1401',NULL,0);
INSERT INTO "usage" VALUES('EPSG','22459','geodetic_crs','EPSG','10791','EPSG','1241','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','10798','LibRef21','','geocentric','EPSG','6500','EPSG','1402',NULL,0);
INSERT INTO "usage" VALUES('EPSG','22444','geodetic_crs','EPSG','10798','EPSG','1142','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','10799','LibRef21','','geographic 3D','EPSG','6423','EPSG','1402',NULL,0);
INSERT INTO "usage" VALUES('EPSG','22445','geodetic_crs','EPSG','10799','EPSG','1142','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','10800','LibRef21','Replaces Liberia 1964 (code 4251).','geographic 2D','EPSG','6422','EPSG','1402',NULL,0);
INSERT INTO "usage" VALUES('EPSG','22446','geodetic_crs','EPSG','10800','EPSG','1142','EPSG','1181');
INSERT INTO "geodetic_crs" VALUES('EPSG','10805','NKG_ETRF14','Used as hub for NKG2020 transformations. Replaces NKG_ETRF00 (ETRF2000 at epoch 2000.0).','geocentric','EPSG','6500','EPSG','1403',NULL,0);
INSERT INTO "usage" VALUES('EPSG','23988','geodetic_crs','EPSG','10805','EPSG','4795','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','10806','NKG_ETRF14','Replaces NKG_ETRF00 (ETRF2000 at epoch 2000.0).','geographic 3D','EPSG','6423','EPSG','1403',NULL,0);
INSERT INTO "usage" VALUES('EPSG','23989','geodetic_crs','EPSG','10806','EPSG','4795','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','10807','NKG_ETRF14','Replaces NKG_ETRF00 (ETRF2000 at epoch 2000.0).','geographic 2D','EPSG','6422','EPSG','1403',NULL,0);
INSERT INTO "usage" VALUES('EPSG','23990','geodetic_crs','EPSG','10807','EPSG','4795','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','10829','Georgia Geodetic Datum','','geocentric','EPSG','6500','EPSG','1404',NULL,0);
INSERT INTO "usage" VALUES('EPSG','22788','geodetic_crs','EPSG','10829','EPSG','1102','EPSG','1181');
INSERT INTO "geodetic_crs" VALUES('EPSG','10830','Georgia Geodetic Datum','','geographic 3D','EPSG','6423','EPSG','1404',NULL,0);
INSERT INTO "usage" VALUES('EPSG','22789','geodetic_crs','EPSG','10830','EPSG','1102','EPSG','1181');
INSERT INTO "geodetic_crs" VALUES('EPSG','10831','Georgia Geodetic Datum','','geographic 2D','EPSG','6422','EPSG','1404',NULL,0);
INSERT INTO "usage" VALUES('EPSG','22790','geodetic_crs','EPSG','10831','EPSG','1102','EPSG','1181');
INSERT INTO "geodetic_crs" VALUES('EPSG','10849','EWR3-IRF','Intermediate CRS created in 2024 to assist the emulation of the ETRS89 / EWR3 SnakeGrid projected CRS through transformation ETRS89 to EWR3-IRF (1) (code 10850) used in conjunction with the EWR3-TM map projection (code 9765). The EWR3 SnakeGrid is an extension of the EWR2 SnakeGrid, and between Oxford and Bedford the two grids are identical. ','geographic 2D','EPSG','6422','EPSG','1405',NULL,0);
INSERT INTO "usage" VALUES('EPSG','22763','geodetic_crs','EPSG','10849','EPSG','4799','EPSG','1141');
INSERT INTO "geodetic_crs" VALUES('EPSG','10860','WSPG-IRF','Intermediate CRS created in 2025 to assist the emulation of the ETRS89 / WSPG SnakeGrid projected CRS through transformation ETRS89 to WSPG-IRF (1) (code 10861) used in conjunction with the WSPG-TM map projection (code 10862).','geographic 2D','EPSG','6422','EPSG','1406',NULL,0);
INSERT INTO "usage" VALUES('EPSG','22830','geodetic_crs','EPSG','10860','EPSG','4801','EPSG','1293');
INSERT INTO "geodetic_crs" VALUES('EPSG','10873','ETRS89-NOR [EUREF89]','EUREF89 is the national realization of ETRS89 in Norway.','geocentric','EPSG','6500','EPSG','1407',NULL,0);
INSERT INTO "usage" VALUES('EPSG','23763','geodetic_crs','EPSG','10873','EPSG','1182','EPSG','1181');
INSERT INTO "geodetic_crs" VALUES('EPSG','10874','ETRS89-NOR [EUREF89]','EUREF89 is the national realization of ETRS89 in Norway.','geographic 3D','EPSG','6423','EPSG','1407',NULL,0);
INSERT INTO "usage" VALUES('EPSG','23764','geodetic_crs','EPSG','10874','EPSG','1182','EPSG','1181');
INSERT INTO "geodetic_crs" VALUES('EPSG','10875','ETRS89-NOR [EUREF89]','Horizontal component of the national realization of ETRS89 in Norway.','geographic 2D','EPSG','6422','EPSG','1407',NULL,0);
INSERT INTO "usage" VALUES('EPSG','23927','geodetic_crs','EPSG','10875','EPSG','1182','EPSG','1181');
INSERT INTO "geodetic_crs" VALUES('EPSG','10890','ETRS89-DNK','ETRS89-DNK is the national realization of ETRS89 in Denmark.','geocentric','EPSG','6500','EPSG','1412',NULL,0);
INSERT INTO "usage" VALUES('EPSG','23441','geodetic_crs','EPSG','10890','EPSG','1080','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','10891','ETRS89-DNK','ETRS89-DNK is the national realization of ETRS89 in Denmark.','geographic 3D','EPSG','6423','EPSG','1412',NULL,0);
INSERT INTO "usage" VALUES('EPSG','23442','geodetic_crs','EPSG','10891','EPSG','1080','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','10892','ETRS89-DNK','Horizontal component of the national realization of ETRS89 in Denmark.','geographic 2D','EPSG','6422','EPSG','1412',NULL,0);
INSERT INTO "usage" VALUES('EPSG','23928','geodetic_crs','EPSG','10892','EPSG','1080','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','10898','Asse 2025','Defined through 3-dimensional transformation from ETRS89/DREF91 Realization 2016 (see CT 10905). Coordinate values are close to but not identical to DHDN.','geographic 2D','EPSG','6422','EPSG','1413',NULL,0);
INSERT INTO "usage" VALUES('EPSG','23353','geodetic_crs','EPSG','10898','EPSG','4805','EPSG','1144');
INSERT INTO "geodetic_crs" VALUES('EPSG','10908','CSRN epoch 2025.0 (NAD83 2011)','Replaces NAD83(2011) in California. Supersedes  CSRS epoch 2017.5 (NAD83 2011) from 2025-06-15.','geocentric','EPSG','6500','EPSG','1414',NULL,0);
INSERT INTO "usage" VALUES('EPSG','23586','geodetic_crs','EPSG','10908','EPSG','1375','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','10909','CSRN epoch 2025.0 (NAD83 2011)','Replaces NAD83(2011) in California. Supersedes CSRS epoch 2017.5 (NAD83 2011) from 2025-06-15.','geographic 3D','EPSG','6423','EPSG','1414',NULL,0);
INSERT INTO "usage" VALUES('EPSG','23585','geodetic_crs','EPSG','10909','EPSG','1375','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','10910','CSRN epoch 2025.0 (NAD83 2011)','Replaces NAD83(2011) in California. Supersedes CSRS epoch 2017.5 (NAD83 2011) from 2025-06-15.','geographic 2D','EPSG','6422','EPSG','1414',NULL,0);
INSERT INTO "usage" VALUES('EPSG','23584','geodetic_crs','EPSG','10910','EPSG','1375','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','10939','QazTRF-23','','geocentric','EPSG','6500','EPSG','1417',NULL,0);
INSERT INTO "usage" VALUES('EPSG','23415','geodetic_crs','EPSG','10939','EPSG','1131','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','10940','QazTRF-23','','geographic 3D','EPSG','6423','EPSG','1417',NULL,0);
INSERT INTO "usage" VALUES('EPSG','23279','geodetic_crs','EPSG','10940','EPSG','1131','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','10941','QazTRF-23','Replaces usage of Pulkovo 1942 (CRS code 4284) in Kazakhstan from 2025-01-01.','geographic 2D','EPSG','6422','EPSG','1417',NULL,0);
INSERT INTO "usage" VALUES('EPSG','23416','geodetic_crs','EPSG','10941','EPSG','1131','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','10950','CSRN epoch 2025.0 (ITRF2020)','Californian ''snapshot'' of ITRF2020 taking into account local tectonic plate movement and deformation. Supersedes CSRS epoch 2017.5 (ITRF2014) from 2025-06-15.','geocentric','EPSG','6500','EPSG','1418',NULL,0);
INSERT INTO "usage" VALUES('EPSG','23570','geodetic_crs','EPSG','10950','EPSG','1375','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','10951','CSRN epoch 2025.0 (ITRF2020)','Californian ''snapshot'' of ITRF2020 taking into account local tectonic plate movement and deformation. Supersedes CSRS epoch 2017.5 (ITRF2014) from 2025-06-15.','geographic 3D','EPSG','6423','EPSG','1418',NULL,0);
INSERT INTO "usage" VALUES('EPSG','23569','geodetic_crs','EPSG','10951','EPSG','1375','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','10952','CSRN epoch 2025.0 (ITRF2020)','Californian ''snapshot'' of ITRF2020 taking into account local tectonic plate movement and deformation. Supersedes CSRS epoch 2017.5 (ITRF2014) from 2025-06-15.','geographic 2D','EPSG','6422','EPSG','1418',NULL,0);
INSERT INTO "usage" VALUES('EPSG','23568','geodetic_crs','EPSG','10952','EPSG','1375','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','10954','GR96(1996)','','geocentric','EPSG','6500','EPSG','1420',NULL,0);
INSERT INTO "usage" VALUES('EPSG','23378','geodetic_crs','EPSG','10954','EPSG','1107','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','10955','GR96(1996)','','geographic 3D','EPSG','6423','EPSG','1420',NULL,0);
INSERT INTO "usage" VALUES('EPSG','23379','geodetic_crs','EPSG','10955','EPSG','1107','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','10956','GR96(1996)','Replaces all earlier Greenland geographic CRSs.','geographic 2D','EPSG','6422','EPSG','1420',NULL,0);
INSERT INTO "usage" VALUES('EPSG','23380','geodetic_crs','EPSG','10956','EPSG','1107','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','10957','GR96(2021)','','geocentric','EPSG','6500','EPSG','1419',NULL,0);
INSERT INTO "usage" VALUES('EPSG','23381','geodetic_crs','EPSG','10957','EPSG','1107','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','10958','GR96(2021)','','geographic 3D','EPSG','6423','EPSG','1419',NULL,0);
INSERT INTO "usage" VALUES('EPSG','23382','geodetic_crs','EPSG','10958','EPSG','1107','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','10959','GR96(2021)','Replaces all earlier Greenland geographic CRSs.','geographic 2D','EPSG','6422','EPSG','1419',NULL,0);
INSERT INTO "usage" VALUES('EPSG','23383','geodetic_crs','EPSG','10959','EPSG','1107','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','10966','NATRF2022','CAUTION: Preliminary data from NGS beta website (https://beta.ngs.noaa.gov/). This data has been added to the EPSG Dataset prior to official adoption to facilitate software testing and evaluation. In the unlikely event that definitions change, the record will be deprecated and replaced with a new EPSG code.','geocentric','EPSG','6500','EPSG','1422',NULL,0);
INSERT INTO "usage" VALUES('EPSG','23623','geodetic_crs','EPSG','10966','EPSG','4803','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','10967','NATRF2022','CAUTION: Preliminary data from NGS beta website (https://beta.ngs.noaa.gov/). This data has been added to the EPSG Dataset prior to official adoption to facilitate software testing and evaluation. In the unlikely event that definitions change, the record will be deprecated and replaced with a new EPSG code.','geographic 3D','EPSG','6423','EPSG','1422',NULL,0);
INSERT INTO "usage" VALUES('EPSG','23624','geodetic_crs','EPSG','10967','EPSG','4803','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','10968','NATRF2022','CAUTION: Preliminary data from NGS beta website (https://beta.ngs.noaa.gov/). This data has been added to the EPSG Dataset prior to official adoption to facilitate software testing and evaluation. In the unlikely event that definitions change, the record will be deprecated and replaced with a new EPSG code.','geographic 2D','EPSG','6422','EPSG','1422',NULL,0);
INSERT INTO "usage" VALUES('EPSG','23625','geodetic_crs','EPSG','10968','EPSG','4803','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','11041','CR-SIRGAS epoch 2019.24','','geocentric','EPSG','6500','EPSG','1428',NULL,0);
INSERT INTO "usage" VALUES('EPSG','24176','geodetic_crs','EPSG','11041','EPSG','1074','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','11042','CR-SIRGAS epoch 2019.24','','geographic 3D','EPSG','6423','EPSG','1428',NULL,0);
INSERT INTO "usage" VALUES('EPSG','24178','geodetic_crs','EPSG','11042','EPSG','1074','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','11043','CR-SIRGAS epoch 2019.24','','geographic 2D','EPSG','6422','EPSG','1428',NULL,0);
INSERT INTO "usage" VALUES('EPSG','24179','geodetic_crs','EPSG','11043','EPSG','1074','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','20033','MWC18-IRF','Intermediate CRS created in 2022 to assist the emulation of the ETRS89 / MWC18 SnakeGrid projected CRS through transformation ETRS89 to MWC18 (1) (code 10108) used in conjunction with the MWC18-TM map projection (code 10127).','geographic 2D','EPSG','6422','EPSG','1324',NULL,0);
INSERT INTO "usage" VALUES('EPSG','18335','geodetic_crs','EPSG','20033','EPSG','4666','EPSG','1141');
INSERT INTO "geodetic_crs" VALUES('EPSG','20039','SIRGAS-Chile 2021','Densification of SIRGAS-CON within Chile at epoch 2021.00. Replaces SIRGAS-Chile 2016 (CRS code 9151) due to significant tectonic deformation.','geocentric','EPSG','6500','EPSG','1327',NULL,0);
INSERT INTO "usage" VALUES('EPSG','18454','geodetic_crs','EPSG','20039','EPSG','1066','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','20040','SIRGAS-Chile 2021','Densification of SIRGAS-CON within Chile at epoch 2021.00. Replaces SIRGAS-Chile 2016 (CRS code 9152) due to significant tectonic deformation.','geographic 3D','EPSG','6423','EPSG','1327',NULL,0);
INSERT INTO "usage" VALUES('EPSG','18455','geodetic_crs','EPSG','20040','EPSG','1066','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','20041','SIRGAS-Chile 2021','Densification within Chile of SIRGAS-CON at epoch 2021.00. Replaces SIRGAS-Chile 2016 (CRS code 9153) due to significant tectonic deformation.','geographic 2D','EPSG','6422','EPSG','1327',NULL,0);
INSERT INTO "usage" VALUES('EPSG','18456','geodetic_crs','EPSG','20041','EPSG','1066','EPSG','1183');
INSERT INTO "geodetic_crs" VALUES('EPSG','20044','MAGNA-SIRGAS 2018','','geocentric','EPSG','6500','EPSG','1329',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19038','geodetic_crs','EPSG','20044','EPSG','1070','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','20045','MAGNA-SIRGAS 2018','','geographic 3D','EPSG','6423','EPSG','1329',NULL,0);
INSERT INTO "usage" VALUES('EPSG','19039','geodetic_crs','EPSG','20045','EPSG','1070','EPSG','1027');
INSERT INTO "geodetic_crs" VALUES('EPSG','20046','MAGNA-SIRGAS 2018','Replaces MAGNA-SIRGAS (CRS code 4686) for high accuracy purposes. Change is approximately 0.31m in latitude, 0.02m in longitude. For mapping and cadastral purposes considered equivalent.','geographic 2D','EPSG','6422','EPSG','1329',NULL,0);
INSERT INTO "usage" VALUES('EPSG','18485','geodetic_crs','EPSG','20046','EPSG','1070','EPSG','1183');
